#Hey emacs, this is basically -*- perl -*-
# $Id: $

## @summary     NFS Stress Scalability 7Mode Cluster Topology Library
##
## @author      dascenj@netapp.com
##
## @pod here

=head1 NAME

Topology::NAS::NFS::7M_Cluster - NFS Stress Scalability test Topology Library for 7Mode Cluster ONTAP

=head1 SYNOPSIS

    use qw(Topology);
    my $topology = new Topology(topology=>"Topology::NAS::NFS::7M_Cluster");

=head1 DESCRIPTION

This topology is very much like the 7M topology, but this will 
utilize both nodes in an SFO pair.

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/TestManagerLite

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/Topology

=head1 TODO

=head1 SEE ALSO

Options TestManagerLite Topology

=head1 BUGS

Some probably exist but have not been listed yet.

=head1 STATUS

  Shared

=head1 AUTHOR

  Jd (dascenj@netapp.com)
  $Id: $
  Last revised $Date: 2008/05/23 $
    
=head1 COPYRIGHT

Copyright 2008, Network Appliance., All Rights Reserved

=cut

package Topology::NAS::NFS::7M_Cluster;
use base qw(Topology::NAS::NFS);

use strict;
use warnings;

use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use SayWrap qw(sayEOF);
use Tharn qw($Log config_result fail_result script_result snooze);
use Data::Dumper qw(Dumper);
use Options;
use RemoteExecution;
use NACL::APISet;
use NACL::C::Node;
use NACL::C::Volume;
use NACL::C::Node;
use NATE::BaseException qw(:try);
use Topology::Discover;
use Topology::Lookup;

my ($Options, @vlan_addrs, $last_vlan_id, $Options_hash, %apiset, $Response, %conn);
my $Remote = new RemoteExecution;

###################################################################

$Options = new Options;
$Options->parameters(
    definitions => [
        {name        => "TESTUSER",
         spec        => "s@",
         description => "List of test users to use for the testing",
         default     => undef,},
        {name        => "TESTUSER_FILE",
         spec        => "s",
         description => "file containing the list of testusers who test run as",
         file_tests  => "fr",
         subroutine  => [\&_read_testuser_file],},
        {name        => "SIS_ENABLE",
         spec        => "s",
         description => "Param that is used to enable the sis volumes",
         default     => "0",},
        {name => "QUOTA_VOLUME_ENABLE",
         spec => "boolean",
         description =>
           "Enable a volume quota for each user that will utilize said volume",
         default => 0,},
        {name => "VOLUMES_PER_INSTANCE",
         spec => "s",
         description =>
           "How many different mount points should be available to the instance",
         default => 1,},
        {name => "QUOTAS",
         spec => "s",
         description =>
           "Param that contains target:type:disk:files:thold:sdisk:sfile",},
        {name        => "MAX_VLAN_ENABLE",
         spec        => "boolean",
         description => "Enable max vlan for each controller",
         default     => 0,},
        {name => "VLAN_ADDRS",
         spec => "s",
         description =>
           "Starting IP and number of IPs to be assigned to vlan separated by '/'",
         default => undef,},
        {name        => "VLAN_IDS",
         spec        => "s",
         description => "Starting Vlan ID",
         default     => undef,},
        {name        => "VLAN_NETMASK",
         spec        => "s",
         description => "Netmask that uses for vlan setup",
         default     => "255.255.192.0",},
        {name        => "FILER_DISCOVER",
         spec        => "s",
         description => "Disable or enable discovery of existing filer configuration",
         default     => 1,},
        {name        => "VLAN_INTERFACE",
         spec        => "s",
         description => "The interface that is used for creating vlans",
         default     => undef,},
        {name => "VOLPATHS_FILE",
         spec => "s",
         description =>
           "file containing the list of volume paths for clients to mount to",
         file_tests => "fr",
         subroutine => [\&_read_volpaths_file],},]);

sub new {
    my $invocant = shift;
    my $class    = ref($invocant) || $invocant;
    my $self     = {@_,};
    $Log->enter();
    bless($self, $class);

    $Log->exit();
    return $self;
} ## end sub new

our @filers;
my $discovered_topology = {};

sub define_topology {
    my $self = shift;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::define_topology(@_);

    $Options_hash = $Options->get_hash();

    @filers = NACL::C::Node->find();

    $Log->comment("using filers:" . Dumper(\@filers));
     
    if (scalar(@filers) > 2) {
        die config_result(
             "This Topology supports only one 7Mode SFO pair. Given: @filers");
    }

    # Make 7Mode and ZAPI APISet connections for all nodes and store
    # for reuse
    foreach my $node (@filers) {
        my $nodeName = $node->hostrec()->id();
        push (@{$self->{filerNames}}, $nodeName);

#        $Log->comment("Creating CLI connection object for $nodeName");
#        $node->{cli} =
#          NACL::APISet->new(hostobj   => $node->hostrec(),
#                            category  => "Node",
#                            interface => "CLI",
#                            set       => "7Mode",
#                            cdef_path => $Options_hash->{'CDEF_PATH'});

        $Log->comment("Creating ZAPI connection object for $nodeName");
        $node->{zapi} =
          NACL::APISet->new(hostobj   => $node->hostrec(),
                            category  => "Node",
                            interface => "ZAPI",
                            set       => "7Mode",
                            cdef_path => $Options_hash->{'CDEF_PATH'});

         # Discover the existing configuration of this node
         $discovered_topology->{$nodeName} = Topology::Discover::run(node => $node);
    } 

    ##################
    # Figure out the interfaces by saving the ones that have addresses on to the filer_setup 
    # hash for later use
    ##################
    @{$self->{filer_setup}{_data_interface_names}} = @{$Options->get(option => "DATA_INTERFACES")};
    my @dataInterfaces = @{$self->{filer_setup}{_data_interface_names}};
    $Log->warn("Because there were no interfaces names in DATA_INTERFACES, the topology will use" . 
                  " any interface with an address.") if (not scalar(@dataInterfaces));
    foreach my $node (@filers) { 
        my $interfaceFound = 0;
        my $nodeName = $node->hostrec()->id();
        foreach my $int (@{Topology::Lookup::Interface(topo => $discovered_topology->{$nodeName})}) {
            if (defined(@{$int->{ipv4}})) { 
                next if (not grep(/$int->{name}/i, @dataInterfaces));
 
                $int->{node} = $nodeName;
                $interfaceFound = 1;
                push(@{$self->{filer_setup}->{interfaces}}, $int);
            }
        }
        if (not $interfaceFound) { 
            die script_result("Could not find an interface to use on $nodeName. Check for a configured " . 
                                " interface, as well as the value of DATA_INTERFACES filter.");
        }
    }

    ##################
    # Figure out the aggregate mess. I miss CMode :(
    ##################

    my @givenAggrNames = @{$Options->get(option => "aggregate")};
    my @discoveredAggrs;
    # Check all the filers for avaialable aggregates
    foreach my $node (@filers) { 
        my $aggrOnThisFiler = 0;
        my $nodeName = $node->hostrec()->id();
        my @aggrs = @{Topology::Lookup::aggregate(topo => $discovered_topology->{$nodeName})};

        die config_result("The discovered topology does not contain any aggregates for this node $node") 
           if (not @aggrs);

        foreach my $aggr (@aggrs) { 
            # skip aggregates with the root vol
            $Log->debug("Examine aggregate $aggr->{aggregate}...");
            $Log->debug("Aggregate $aggr->{aggregate} root setting is $aggr->{root}...");

            next if ((defined($aggr->{root})) and ($aggr->{root} =~ /true/i)); 

            my $aggrData = {name => $aggr->{aggregate},
                           filer => $nodeName};

            push (@discoveredAggrs, $aggrData); 
            $aggrOnThisFiler = 1;
        }
   
        if (not $aggrOnThisFiler) { 
            die config_result("The filer $nodeName was given to this topology, " . 
               "but we were unable to find a suitable aggregate on it for testing. " . 
                  "aggrs which contain the root volume are skipped");
        }
    }

    $Log->debug("Discovered aggregates" . Dumper(\@discoveredAggrs));

    # If we were given a list of aggregates to use, then fix our discovered list to match the given
    # otherwise use what we discovered
    if (@givenAggrNames) { 
        # pull out the ones we want to use
        foreach my $aggr (@discoveredAggrs) { 
           if (grep (/$aggr->{name}/i, @givenAggrNames)) { 
               push (@{$self->{filer_setup}->{aggregates}}, $aggr);
           }
        }
    } else { 
        foreach my $aggr (@discoveredAggrs) { 
            @{$self->{filer_setup}->{aggregates}} = @discoveredAggrs;
        }
    }

    # Make sure that every node in FILER_LIST has at least one aggregate
    # specified
    foreach my $node (@filers) { 
        my $filerName = $node->hostrec()->id();
        my $filerFound = 0;
        foreach my $aggr (@{$self->{filer_setup}->{aggregates}}) { 
           if ($aggr->{filer} =~ /$filerName/i) { 
               $filerFound = 1;
               last;
           }
        }
        if (not $filerFound) { 
            die script_result("The filer $filerName was given to this topology, " .
               "but we were unable to find a suitable aggregate to use on that filer. " .
                  "aggrs which contain the root volume are skipped. Aggrs:" . Dumper($self->{filer_setup}->{aggregates}));
        }
    }
    $Log->comment("The test will use these aggregates:" . Dumper($self->{filer_setup}->{aggregates}));
        
    # Get specified vlan addrs
    @vlan_addrs = $self->_create_range($Options->get(option => "VLAN_ADDRS"));

    # Options used here
    my $num_exports          = $Options_hash->{NUM_EXPORTS};
    my $dir_per_instance     = $Options_hash->{DIR_PER_INSTANCE};
    my @volumes = $self->SUPER::volumes();
    my $export_options =
      ($Options_hash->{EXPORT_OPTIONS})
      ? $Options_hash->{EXPORT_OPTIONS}
      : "rw,anon=0";
    my $max_nfs_clients = $Options_hash->{MAX_NFS_CLIENTS};
    $Log->comment("Options: \n" . Dumper($Options_hash));

    my @clients = $self->clients();

    # figure out the size business. VOL_SIZE is size per instance. 
    my $size = $Options->get(option => "VOL_SIZE");
    # Set 30MB per instance, in case if none is set.
    $size = ($size) ? $size : 30;
    if ($size !~ /[m|g|t]/i) { 
        $size = $size . 'm';
    }
    $size = $self->size_in_mb(size => $size);

    # Use the nodeIndex value to round-robin the given volumes amongst the 
    # available nodes. Also alternate the available IP addresses
    my $nodeIndex = 0;
    foreach my $vol (@volumes) {
        my $nodeName = $filers[$nodeIndex]->hostrec()->id();
        my $volData = {name => $vol, 
                       filer => $nodeName,
                       assignedInstances => 0,
                       sizePerInstance => $size,
                       size => $size,
                       aggregate => '' ,
                       filer_addr => '',};

        $self->{filer_setup}->{volumes}->{$volData->{name}} = $volData;

        # This is cheap, I know...but for now...It is all I have time for
        # right now...sorry. 
        my $aggrs = $self->_get_aggr_names_by_node(node => $nodeName);
        $volData->{aggregate} = $aggrs->[0];

        $nodeIndex = 0 if (++$nodeIndex > (scalar(@filers) - 1));
    }

    # sometimes a filer might have more than 1 data address, 
    # make sure we round robin to use all the supplied IP addresses
    foreach my $node (@filers) { 
        my $addrIndex = 0;
        my $nodeName = $node->hostrec()->id();
        my @addrs = @{$self->_get_int_addr_by_node(node => $nodeName)};

        foreach my $vol (@{$self->_get_volumes_by_node(node => $nodeName)}) { 
            $vol->{filer_addr} = $addrs[$addrIndex];
        }

        $addrIndex = 0 if (++$addrIndex > (scalar(@addrs) - 1));
    }
    $self->_configure_instances();

    $self->_assign_path_to_instance();

    #
    # Export Layout for all test instances
    #
    my $totalInstancesNum = $self->{_num_instances};
    if ($num_exports == $totalInstancesNum) {
        # Number of test instances == number of exports. This means each instance will run
        # against exclusive directory/volume with exclusive export
        $Log->comment("An export per test instance detected");

        foreach my $client (@clients) {
            foreach my $inst (keys %{$self->{test_options}{$client}}) {
                my $volpath = $self->{test_options}{$client}{$inst}{VOLPATH};
                $self->{filer_setup}{exports}{$volpath}{options} =
                  $export_options;
            }
        }

    } elsif ($num_exports < $totalInstancesNum) {
        if ($num_exports == scalar(@volumes)) {

            # Export each volume separately
            $Log->comment(
                "An export per volume is detected with export_options $export_options"
                );
            foreach my $volume (@volumes) {
                my $export = "/vol/" . $volume;
                $self->{filer_setup}{exports}{$export}{options} =
                  $export_options;
                $self->{filer_setup}{exports}{$export}{volume} = 
                  $self->{filer_setup}->{volumes}->{$volume};
            }
        } elsif ($num_exports > scalar(@volumes)) {

            # ??? - GV
            # Not sure if there are any other better ways to handle this. For now, export all volumes first
            # to make sure all are usable, then export sub directories until reach $num_exports

            my $exports = $num_exports;
            $Log->comment("exports $num_exports > volumes "
                . scalar(@volumes)
                . " detected. Exporting volumes first and then export directories"
                );
            foreach my $volume (@volumes) {
                my $export = "/vol/" . $volume;
                $self->{filer_setup}{exports}{$export}{options} =
                  $export_options;
                $self->{filer_setup}{exports}{$export}{volume} = 
                  $self->{filer_setup}->{volumes}->{$volume};
              
                $exports--;
            }
            foreach my $client (@clients) {
                foreach my $inst (keys %{$self->{test_options}{$client}}) {
                    my $volpath =
                      $self->{test_options}{$client}{$inst}{VOLPATH};
                    $self->{filer_setup}{exports}{$volpath}{options} =
                      $export_options;
                    $exports--;
                    last if (!$exports);
                }
                last if (!$exports);
            } ## end foreach my $client (@clients)
        } else {
            die config_result(  "This doesn't make sense. NUM_VOLS("
                              . scalar(@volumes)
                              . ") && NUM_EXPORTS ($num_exports)");
        }
    } else {
        die config_result("This doesn't make sense. NUM_VOLS("
             . scalar(@volumes)
             . ") && NUM_EXPORTS ($num_exports) && INSTANCES ($totalInstancesNum)"
             );
    }

    # Spread test users across test instances if given
    my $testusers = $Options_hash->{TESTUSER};
    my $index     = 0;
    if (my $num_users = scalar(@$testusers)) {
        foreach my $client (@clients) {
            foreach my $inst (keys %{$self->{test_options}{$client}}) {
                $self->{test_options}{$client}{$inst}{TESTUSER} =
                  $testusers->[$index];
                $index = 0 if (++$index >= $num_users);
            }
        }
    }

    $self->{_num_instances} = $totalInstancesNum;

    $Log->debug(Dumper($self));
    $Log->exit();
} 

sub filer_setup {
    my ($self, %options) = @_;

    # Execute the one from the parent
    $self->SUPER::filer_setup(%options);

    my @clients = $self->clients();

    foreach my $filer (@filers) { 
        my $zapi = $filer->{zapi};
        my $filerName = $filer->hostrec()->id();
        $Log->comment("Begin setup phase for filer " . $filerName);
   
        # Get a list of current exports already there. This should someday check 
        # to make sure that the rules are correct...for now, we just look to see if it 
        # is already there.
        my $existingExports = {};
        try {
            my $response = $zapi->nfs_exportfs_list_rules_2()->get_parsed_output();
            foreach my $export (@{$response->[0]->{rules}->[0]->{'exports-rule-info-2'}}) { 
                $existingExports->{$export->{pathname}} = $export->{'security-rules'};
            }
    
        } catch NACL::APISet::Exceptions::ResponseException with {
            $self->_exception_handler(exception => shift);
        };
    
        foreach my $vol (@{$self->_get_volumes_by_node(node => $filerName)}) { 
            my $zapi = $filer->{zapi};
       
            try {  
                $zapi->volume_create('volume' => $vol->{name},
                       'containing-aggr-name' => $vol->{aggregate},
                                       'size' => $vol->{size} . 'm');
            } catch NATE::BaseException with {
                 $self->_exception_handler(exception => shift);
            };
        }    
        # Create the volume quotas
        if ($Options_hash->{QUOTA_VOLUME_ENABLE} != 0) {
            $self->configure_quotas('node' => $filer);
        }
    
        # Create required directories
        if (keys %{$self->{filer_setup}{dirs}}) {
            $Log->comment("Directories need to be created on volumes");
             
            my $Remote = new RemoteExecution;

            # pick an ip address to use
            my @filerAddrs = @{$self->_get_int_addr_by_node(node => $filerName)};
            my $filerip = $filerAddrs[0];

            my $client_idx = 0;
            my ($client, $mntpnt);
            my $num_subs = 0;
            foreach my $volumeObj (@{$self->_get_volumes_by_node(node => $filerName)}) {
                my $volume = $volumeObj->{name};
                my @directories = @{$self->{filer_setup}{dirs}{$volume}};
    
                # If volume already exists and exported with restricted permission
                # create_topology() didn't seem to change anything, hence we need
                # override here so dir creation can succeed. This shouldn't be a
                # problem, since we will reexport them anyways.
                my $exportRuleInfo = { pathname => "/vol/$volume",
                                           anon => 0,};
                my @rules;
                push(@rules, $exportRuleInfo);

                try {
                    my $response = $zapi->nfs_exportfs_append_rules(rules => \@rules);
    
                } catch NACL::APISet::Exceptions::ResponseException with {
                    $self->_exception_handler(exception => shift);
                };
    
                # Pick a number 500 directories per client as optimal value
                while (my @dirs = splice(@directories, 0, 500)) {
                    if ($client_idx >= scalar(@clients)) {
                        $client_idx = 0;
                    }
                    $client = $clients[$client_idx];
    
                    $mntpnt =
                      "/t/${filerip}_${volume}_${client}_${client_idx}_filer_setup";
    
                    # create directories on the client
                    my $params = {
                        DIRS => join(',', @dirs),
                        MOUNT_LIST => "${filerip};/vol/${volume};${mntpnt};rw;777",
                        };
                    $Log->comment(
                        "Run tools/TestManager/filer_directory_setup.thpl on $volume from $client to create dirs: "
                          . join(",", @dirs));
                    $Remote->execute(
                           host        => $client,
                           nate_params => $params,
                           cmd   => "tools/TestManager/filer_directory_setup.thpl",
                           runid => sprintf("%03d_%s_%s",
                                            $num_subs, "filer_setup_dirs", $client),
                           handler => sub { });
                    $client_idx++;
                    $num_subs++;
                }
            } 
            my $fail_result = $Remote->wait(timeout => 300);
            die config_result("dirs creation failed: $fail_result")
              if ($fail_result);
        } else {
            $Log->comment(
                    "Directories weren't required to be created for this Topology");
        }
    
        # Setup exports appropriately
        # If one wants to export max exports allowed (10240), then we have to unexport
        # everything for max exports to succeed.
        if ((keys %{$self->{filer_setup}{exports}}) >= 10240) {
            unexport_all($filer->hostrec());
        }
    
        foreach my $export (keys %{$self->{filer_setup}{exports}}) {
            my $options = $self->{filer_setup}{exports}{$export}{options};

            # Zapi wants all this information in a particular format
            # http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/mainN/web/nfs/index.html#nfs-exportfs-append-rules
            my $exportRuleInfo = {pathname => $export,};

            # figure out what export options should be included with this rule
            # this section needs to be improved to support the rest of the valid
            # export options found in the zapi doc above.
            if ($options =~ /anon=(\d+)/i) { 
                $exportRuleInfo->{anon} = $1;
            }
            if ($options =~ /nosuid/i) {
               $exportRuleInfo->{nosuid} = 1;
            }

            my @rules;
            push(@rules, $exportRuleInfo);

            my %exportOptions = (rules => \@rules);
            if ($Options_hash->{PERSISTENT_EXPORT}) {
                $exportOptions{persistent} = 1;
            }

            # bail out if the export is alrady there.     
            if ($existingExports->{$export}) {
                $Log->comment("Found an existing export with the name $export. Reusing this export...");
                next;
            } 

            $Log->trace("Adding export $export to /etc/exports");
            $Log->debug("with options:" . Dumper(%exportOptions));

            try {
                my $response = $zapi->nfs_exportfs_append_rules(%exportOptions);
    
            } catch NACL::APISet::Exceptions::ResponseException with {
                $self->_exception_handler(exception => shift);
            };
        
        }
        
        # Set this option, so clients using non-previlaged ports also will have access
        try {
            my $optionString = "nfs.mount_rootonly";
            my $optionValue = "off";
            my $response = $zapi->options_set(name => $optionString,
                                             value => $optionValue);

        } catch NACL::APISet::Exceptions::ResponseException with {
            $self->_exception_handler(exception => shift);
        };
         
    }
    $Log->exit();
}

sub filer_cleanup {
    my ($self, %options) = @_;

    # Remove Vlans
    my $filer = ($self->filers())[0];
    Tharn::host("$filer");
    my $fc      = Tharn::connect("#telnet");
    my $vlan_id = $Options_hash->{VLAN_IDS};
    foreach my $num ($vlan_id .. ($last_vlan_id - 1)) {
        vlan_delete(fc      => $fc,
                    if_name => $Options_hash->{VLAN_INTERFACE},
                    vlanid  => '$num');
    }

    #Clean up quota
    if ($Options_hash->{QUOTA_VOLUME_ENABLE} != 0) {

        my $Zapi    = $apiset{'zapi'}{$filer};
        my @volumes = $self->volumes();
        my @clients = $self->clients();
        my $quota   = $Options_hash->{QUOTAS};
        foreach my $client (@clients) {
            foreach my $inst (keys %{$self->{test_options}{$client}}) {
                my $user   = $self->{test_options}{$client}{$inst}{TESTUSER};
                my $volume = $self->{test_options}{$client}{$inst}{VOLUME};
                $Log->comment("Checking quota status on $volume");
                my $Status;
                try {
                    $Response = $Zapi->quota_status('volume' => $volume);
                    $Status = $Response->get_parsed_output();
                }
                catch NACL::APISet::Exceptions::ResponseException with {
                    $self->_exception_handler(exception => shift);
                };

                if (grep { $_ eq 'on' } @{$Status->[0]->{'status'}}) {

                    $Log->comment("Stopping quotas on $volume");

                    try {

                        $Response = $Zapi->quota_off('volume' => $volume);
                    }
                    catch NACL::APISet::Exceptions::ResponseException with {
                        $self->_exception_handler(exception => shift);
                    };
                } ## end if (grep { $_ eq 'on' ...
                my @fields = split(/:/, $quota);
                $fields[0] = $user;
                $fields[1] =~ s/\@volume//g;
                try {
                    $Response =
                      $Zapi->quota_delete_entry('quota-target' => $fields[0],
                                                'quota-type'   => $fields[1],
                                                'volume'       => $volume,
                                                'qtree'        => '');
                }
                catch NACL::APISet::Exceptions::ResponseException with {
                    $self->_exception_handler(exception => shift);
                };
            } ## end foreach my $inst (keys %{$self...
        } ## end foreach my $client (@clients)
    } ## end if ($Options_hash->{QUOTA_VOLUME_ENABLE...
    my $test_topology = $self->{test_topology};

    # create a list of all node resources in the test_topology
    my @resources;
    foreach my $resource_ref (values %{$test_topology->{cluster}}) {
        push(@resources, @{$resource_ref})
          if (ref($resource_ref) eq "ARRAY");
    }
    foreach my $node (keys %{$test_topology->{node}}) {
        foreach my $res_ref (values %{$test_topology->{node}{$node}}) {
            push(@resources, @{$res_ref}) if (ref($res_ref) eq "ARRAY");
        }
    }

    # check all of the resources for a comment that indicates that the resource
    # was previously created by TestManager. Mark this resource for cleanup
    foreach my $resource (@resources) {
        next if (ref($resource) ne "HASH");
        if ($resource->{comment} =~ "created by TestManagerLite") {
            $resource->{Delete_Resource} = 1;
        } elsif ($resource->{rulesetname}) {
            $resource->{Delete_Resource} = 1;
        }
    }

    # Execute the one from the parent
    $self->SUPER::filer_cleanup(%options);
} ## end sub filer_cleanup

sub client_setup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_setup(%options);

    $Log->exit();
}

sub DESTROY {
    my $self = shift;
    local ($@);

    $self->client_cleanup();
}

sub client_cleanup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_cleanup(%options);

    $Log->exit();
}

sub _update_test_topology {
    my $self = shift;

    $Log->exit();
} ## end sub _update_test_topology

######################### Internal Routines ##################

sub _create_range {
    my ($self, $ip) = @_;
    my ($begin, $range);
    if ($ip =~ /(.+)\/(\d+)/) {
        ($begin, $range) = ($1, $2);
    } elsif (!defined($ip)) {
        return ();
    } else {
        $begin = $ip;
        $range = 1;
    }
    $range--;

    my @range;
    ## This works for both IPv4 and IPv6
    my $ipobj = new Net::IP("$begin + $range")
      or die "Failed to create ip object: $@";

    do {
        push(@range, $ipobj->ip());
    } while (++$ipobj);

    return (wantarray) ? @range : $range[$#range];
} ## end sub _create_range

sub unexport_all {
    my ($conn) = @_;
    my $output = sayEOF($conn, "exportfs -ua", timeout => 180);

    $Log->comment("unexport all: $output");
    return $output;
}

sub export_all {
    my ($conn) = @_;
    my $output = sayEOF($conn, "exportfs -a", timeout => 180);

    $Log->comment("export all: $output");
    return $output;
}

my $File_read = undef;

sub configure_quotas {

    $Log->enter();

    my ($self, %options) = @_;

    my $node             = $options{'node'};
    my $Zapi_Api_Set_Obj = $apiset{'zapi'}{$node};
    $Log->comment("Configuring quotas for Node: $node");

    my $path = '/vol/vol0/etc/quotas';
    my $file_info;
    eval {
        $Response = $Zapi_Api_Set_Obj->file_get_file_info('path' => $path);
        $file_info = $Response->get_parsed_output();
    };

    # if the file does not exist create it
    if (!$file_info) {
        $self->write_file('ascii' => "\n",
                          'node'  => $node,
                          'path'  => $path);
    }

    #target:type:disk:files:thold:sdisk:sfile
    #NFS_USER:user@volume:-:-:700M:700M:5K
    my @volumes = $self->volumes();
    my @clients = $self->clients();
    my $quota   = $Options_hash->{QUOTAS};
    my $vol;

    foreach my $client (@clients) {
        foreach my $inst (keys %{$self->{test_options}{$client}}) {

            if ($quota !~ /\@volume/) { next; }
            my $user = $self->{test_options}{$client}{$inst}{TESTUSER};
            $vol = $self->{test_options}{$client}{$inst}{VOLUME};
            my @fields = split(/:/, $quota);
            $fields[0] = $user;
            $fields[1] =~ s/\@volume//g;
            my $Entry;
            eval {
                $Response =
                  $Zapi_Api_Set_Obj->quota_get_entry(
                                                   'quota-target' => $fields[0],
                                                   'quota-type'   => $fields[1],
                                                   'volume'       => $vol,
                                                   'qtree'        => '');
                $Entry = $Response->get_parsed_output();

            };
            if (!$Entry) {
                $Log->comment("Adding quota to $vol\n");
                try {

                    $Response =
                      $Zapi_Api_Set_Obj->quota_add_entry(
                                                'quota-target'    => $fields[0],
                                                'quota-type'      => $fields[1],
                                                'disk-limit'      => $fields[2],
                                                'file-limit'      => $fields[3],
                                                'threshold '      => $fields[4],
                                                'soft-disk-limit' => $fields[5],
                                                'soft-file-limit' => $fields[6],
                                                'volume'          => $vol,
                                                'qtree'           => '');
                } ## end try
                catch NACL::APISet::Exceptions::ResponseException with {
                    $self->_exception_handler(exception => shift);
                    $self->_exception_handler(@_);
                };
            }    # foreach my $inst (keys %

            push @{$self->{filer_setup}{volume}{$vol}{quota}}, $quota;

            # Turn on the quota for vols
            try {

                $Response = $Zapi_Api_Set_Obj->quota_on('volume' => $vol,);
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                $self->_exception_handler(exception => shift);
            };
            snooze 1;

        }    # foreach my $inst
    }    # foreach my $client
} ## end sub configure_quotas

sub write_file {

    $Log->enter();

    my ($self, %options) = @_;

    my $ascii     = $options{'ascii'};
    my $node      = $options{'node'};
    my $path      = $options{'path'};
    my $offset    = $options{'offset'} || 0;
    my $overwrite = $options{'overwrite'} || 'true';

    my $Zapi = $apiset{'zapi'}{$node};
    my $Cli  = $apiset{'cli'}{$node};

    # Set the vfiler context for the next ZAPI calls
    try {
        $Response = $Zapi->set_vfiler('vfiler-name' => 'vfiler0');
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        $self->_exception_handler(exception => shift);
    };

    $Log->comment("Writing to file $path");
    try {

        $Response =
          $Zapi->file_write_file('data'      => unpack("H*", $ascii),
                                 'offset'    => $offset,
                                 'overwrite' => $overwrite,
                                 'path'      => $path);

    }
    catch NACL::APISet::Exceptions::ResponseException with {
        eval {
            $self->truncate_file('node'   => $node,
                                 'vfiler' => 'vfiler0',
                                 'path'   => $path);

            snooze 1;

            foreach my $line (split(/\n/, $ascii)) {
                $Cli->wrfile(append    => 1,
                             'content' => "$line",
                             'file'    => $path,);
            }
        };
        if ($@) { die config_result($@); }

    };

    $Log->exit();
} ## end sub write_file

sub _read_testuser_file {
    my $tfile = shift;
    my $file =
      NATE::Inc::find_file($tfile) ? NATE::Inc::find_file($tfile) : $tfile;
    $Log->enter("file=>$file");
    if ($File_read) {
        $Log->trace("file already processed");
        goto done;
    }
    my $testuser = $Options->get(option => "TESTUSER");

    if (open(IF, $file)) {
        while (<IF>) {
            next if (/^\s*\#/);
            chomp;
            push(@{$testuser}, $_);
        }
        close(IF);
    } else {
        die "Could not open TESTUSER_FILE: $tfile ($file) : $! / $@";
    }

    $File_read = 1;

  done:
    $Log->exit();
} ## end sub _read_testuser_file

sub get_address { 
    my ($self, $node) = @_;

    return  my $hostrec = $node->hostrec()->default_ip();
    #my $addr = $hostrec->{default_ip};

}

sub _get_volumes_by_node {
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {node => {type => SCALAR, optional => 1},});

    my @filerNames;
    # if the node isn't specified, get them all
    if ($params{node})  { 
        push(@filerNames, $params{node});
    } else { 
        @filerNames = @{$self->{filerNames}};
    }

    my @volumes;
    foreach my $filerName (@filerNames) { 
        foreach my $key (keys %{$self->{filer_setup}->{volumes}}) {
            push (@volumes, $self->{filer_setup}->{volumes}->{$key}) if 
                $self->{filer_setup}->{volumes}->{$key}->{filer} =~ /^$filerName$/i;
        }
    } 
    return \@volumes;
}

sub _get_volumes_names_by_node { 
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {node => {type => SCALAR,},});

    my $filerName = $params{node};
    my @return;

    my @objs = @{$self->_get_volumes_by_node(node => $filerName)};
    foreach my $obj (@objs) { 
        push(@return, $obj->{name});
    }

    return \@return;
}

sub _get_aggr_names_by_node { 
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {node => {type => SCALAR,},});
    my $filerName = $params{node};
    my @return;

    foreach my $aggr (@{$self->{filer_setup}->{aggregates}}) { 
        if ($aggr->{filer} =~ /^$filerName$/i) { 
           push (@return, $aggr->{name});
        }
    }

    return \@return;
}

sub _get_int_by_node { 
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {node => {type => SCALAR,},});
    my $filerName = $params{node};
    my @return;

    foreach my $int (@{$self->{filer_setup}->{interfaces}}) { 
        if ($int->{node} =~ /^$filerName$/i) { 
           push (@return, $int);
        }
    }

    return \@return;
}

sub _get_int_addr_by_node { 
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {node => {type => SCALAR,},
                      interface_name => {type => SCALAR,optional => 1},});
    my $filerName = $params{node};
    my $intName = $params{interface_name};
    my @return;

    foreach my $int (@{$self->_get_int_by_node(node => $filerName)}) { 
       foreach my $addr (@{$int->{ipv4}}) { 
           if (($intName) and ($intName = $int->{name})) { 
               push (@return, $addr);
           } elsif (not $intName) { 
               push (@return, $addr);
           }
       }
    }

    return \@return;
}

sub _get_node_by_addr {
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {addr => {type => SCALAR,},});
    my $addr = $params{addr};
    my $return = 0;


    foreach my $filerName (@{$self->{filerNames}}) {
        foreach my $int (@{$self->_get_int_by_node(node => $filerName)}) {
           foreach my $localAddr (@{$int->{ipv4}}) {
               # The address can only be assigned to one node at a time
               if ($addr eq $localAddr) {
                   $return = $int->{node};
                   last;
               }
           }
        }
    }
    die script_result("Could not find the node with assigned address $addr") if (not $return);
    return $return;
}

sub _get_instances_by_node { 
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {node => {type => SCALAR,},});
    my $nodeName = $params{node};

    my %hash;
    foreach my $key (keys %{$self->{instances}}) {
       my $inst = $self->{instances}->{$key};
       foreach my $volpath (@{$inst->{DEST_PATHS}}) { 
           my ($addr, $path) = split(/:\//, $volpath);
           die script_result("Could not split the volpath into the required address and path. Data: $volpath")
               if ((not $addr) or (not $path));

            my $filerName = $self->_get_node_by_addr(addr => $addr);
            # Keep these in a hash so we don't duplicate the instances in an array   
            $hash{$inst->{name}} = $inst if ($filerName =~ /$nodeName/i);
       }
    }

    my @return;
    foreach my $key (keys(%hash)) { 
        push (@return, $hash{$key});
    }
    return \@return;
}

sub _configure_instances {
    $Log->enter();
    my ($self) = @_;

    # Create the client_setup hash its just a very small static hash of a few options that the supervisor
    # will use. client_config contains the relational setup of the clients <-> instances and everything else.
    my @clients            = $self->clients();
    my $instancesPerClient = $Options->get(option => "INSTANCES_PER_CLIENT");
    my $max_nfs_clients   = $Options->get(option => "MAX_NFS_CLIENTS");
    my $testUsers = $Options->get(option => "TESTUSER");
    my $numberOfInstances = 0;

    # If MAX_NFS_CLIENTS, then use that number as the number clients to create.
    if ($max_nfs_clients) {
        $Log->comment("Received a MAX_NFS_CLIENTS value of $max_nfs_clients. Using this " .
                        "value as the number of NFS clients to create.");
        $numberOfInstances = $max_nfs_clients;
    }

    # Only look at the IP addresses if this is being called as part
    # of the virtual topology. Use the number of supplied addresses
    # as the default number of synthetic clients to create.
    my $numAddrs;
    my ($package, $filename, $line) = caller;
    if ($package =~ /virtual/i) {
        my @srcNet = $self->{ipManager}->source_networks();
        $numAddrs = $srcNet[0]->unassigned_addresses();

        if ($numAddrs < $numberOfInstances) {
            # Freak out if there are not enough IP addresses to reach MAX_NFS_CLIENTS
            die config_result("There were $numAddrs supplied addresss. The value of MAX_NFS_CLIENTS " .
                                "indicates the need for $numberOfInstances clients. You should rethink this.", stacktrace => 0);

            # If not MAX_NFS_CLIENTS use the number of addresses supplied.
        } elsif (not $numberOfInstances) {
            $Log->comment("The number of instances will be equal to the supplied number of IP addresses ($numAddrs).");
            $numberOfInstances = $numAddrs;
        }
    }

    # If there is no value of MAX_NFS_CLIENTS, then just do one instance per client.
    # this means we are not creating synthetic clients.
    if (not $numberOfInstances) {
            $Log->comment("Setting the number of NFS instances to one per client (" . scalar(@clients) . ").");

            $numberOfInstances = scalar(@clients);
    }

    # If INSTANCES_PER_CLIENT is specified, then just make sure we didn't exceed it.
    if (($instancesPerClient) and ($instancesPerClient < ($numberOfInstances / scalar(@clients)))) {
        die config_result("The requested number of instances, ($numberOfInstances total, or " .
                            ($numberOfInstances / scalar(@clients)) . " per client) will exceed the " .
                              "maximum number of instances per client (There are " . scalar(@clients) . " clients), " .
                                "which is defined as (" .  $instancesPerClient . ").\nThis can be overridden by " .
                                 "changing the value of INSTANCES_PER_CLIENT on the command line. Requested " .
                                   $numberOfInstances / scalar(@clients) . " instances for each one of the " .
                                    scalar(@clients) . " clients.");
    }
    if ($numberOfInstances == 0) {
        die config_result("I was asked to create zero instances per client. Review configuration");
    }
    my $inst = int($numberOfInstances/scalar(@clients));
    if (not $inst) {
        $inst = 1;
    }

    $self->{_num_instances} = $numberOfInstances;
    $Log->comment("Creating $numberOfInstances total instances, resulting in " .
        "roughly $inst instances per client.(+/-1)");

    # Spread all the instances among the clients.
    # Spread test users across test instances if given
    my $clientIndex = 0;
    my $userIndex = 0;
    foreach my $instanceNumber (1 .. $numberOfInstances) {
        $Log->debug("Configure instance number: $instanceNumber on client $clients[$clientIndex]");
        my $instanceHash = {
                     client      => $clients[$clientIndex],
                     isSynthetic => 0,
                     name        => $instanceNumber};

        # Add the user ID's if we are going to run processes as different users
        # Cycle through the available user IDs
        if ($testUsers->[0]) {
            $instanceHash->{userName} = $testUsers->[$userIndex] if ($testUsers->[$userIndex]);
            $userIndex = 0 if (++$userIndex > scalar(@{$testUsers}));
        }

        # Keep all instances in a single place
        $self->{instances}->{$instanceNumber} = $instanceHash;

        # Link each instance to its physical client
        $self->{client_config}->{$clients[$clientIndex]}->{instances}
          ->{$instanceNumber} = $self->{instances}->{$instanceNumber};

        # Loop through the clients
        $clientIndex = 0 if (++$clientIndex > (scalar(@clients) - 1));

    }

    $Log->exit();
    return 1;
}

sub _assign_path_to_instance {
    my ($self) = @_;

    my $allclis2vols = $Options->get(option => "ALLCLIENTS_2_ALLVOLS");
    my $volsPerInstance = $Options->get(option => "VOLUMES_PER_INSTANCE");
 
    # get all the usable volumes
    my @topoVolumes = @{$self->_get_volumes_by_node};

    my $instancesPerVolume = int($self->{_num_instances} / scalar(@topoVolumes));
    $Log->comment("Planning for about $instancesPerVolume instances per volume.");
   
    my $string = "Assignment Mode: ";
    $string .= "Allclients2AllVols" if ($allclis2vols);
    $string .= "VolsPerInstance: $volsPerInstance" if ($volsPerInstance);
    $Log->enter($string);
    if (not $volsPerInstance) {
        die config_result("The value of VOLUMES_PER_INSTANCE is zero. What did you think would happen?");
    }
    # Decide which instances will access which volumes
    my $volumeIndex = 0;
    foreach my $key (keys %{$self->{instances}}) {
        my $instance = $self->{instances}->{$key};
        my $clientName = $instance->{client};
        my $volObj = $topoVolumes[$volumeIndex];

        # All clients hit all volumes
        if ($allclis2vols) {
            foreach my $volObj (@topoVolumes) {
                $volObj->{assignedInstances}++;

                push(@{$instance->{DEST_PATHS}}, $volObj->{filer_addr} . ':/vol/' . $volObj->{name} . '/' . $clientName .'_' . $instance->{name});
            }

        } else {
            # Each client will get a predetermined number of volumes
            foreach my $num (1 .. $volsPerInstance) {
                $volObj->{assignedInstances}++;

                push(@{$instance->{DEST_PATHS}}, $volObj->{filer_addr} . ':/vol/' . $volObj->{name} . '/' . $clientName .'_' . $instance->{name});
                
                # Reset the volumes array after all the volumes are used
                $volumeIndex = 0 if (++$volumeIndex > (scalar(@topoVolumes) - 1));
            }
        }
        # Let us also build the test_options hash. The clients are the keys and
        # and the instances are the values. The instance is also a key to a lower hash
        # that which contains DEST_PATHS. The SRCIP is only valid when synthetic
        # clients are used...
        $self->{test_options}->{$clientName}->{$instance->{name}}
               ->{DEST_PATHS} = $instance->{DEST_PATHS};

        $self->{test_options}->{$clientName}->{$instance->{name}}->{TESTUSER} = $instance->{userName};

        $Log->debug("Configured client $clientName with instance $instance->{name}, and  DEST_PATHS: " . Dumper(\@{$instance->{DEST_PATHS}}));

        $volumeIndex = 0 if ($volumeIndex > (scalar(@topoVolumes) - 1));

        # VOL_SIZE is considered per instance
       
    }

    # Now make sure the size reflects the instances
    foreach my $volData (@topoVolumes) { 
        $volData->{size} = $volData->{assignedInstances} * $volData->{sizePerInstance};
    }
    $Log->exit();
}

# handle some well know exceptions that are not fatal
sub _exception_handler {
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {exception => {type => OBJECT},});

    my $exc = $params{exception};


    if ($exc->text() =~ /already\sexists/i) { 
        return;    
    }

    die fail_result($exc->text());

}

sub _set_option { 
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {option_string => {type => SCALAR,},
                      option_value  => {type => SCALAR,},});

   my $optionString = $params{option_string};
   my $optionValue = $params{option_value};
    
   foreach my $filer (@filers) {
        my $zapi = $filer->{zapi};

        try {
            my $response = $zapi->options_set(name => $optionString,
                                             value => $optionValue);

        } catch NACL::APISet::Exceptions::ResponseException with {
            $self->_exception_handler(exception => shift);
        };
    }
}

sub _get_nacl_filers { 
    my ($self) = @_;

    return \@filers;
}


#sub instances { 
#    return $self->{instances};
#}
1;
