#Hey emacs, this is basically -*- perl -*-
# $Id: $

## @summary     NFS Stress Scalability 7Mode NFS Security enabled Topology Library
##
## @author      gvg@netapp.com
##
## @pod here

=head1 NAME

Topology::NAS::NFS::7M_Krb - NFS Stress Scalability NFS Security test Topology Library for 7Mode ONTAP

=head1 SYNOPSIS

    use qw(Topology);
    my $topology = new Topology(topology=>"Topology::NAS::NFS::7M_Sec");

=head1 DESCRIPTION

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/TestManagerLite

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/Topology

=head1 TODO

=head1 SEE ALSO

Options TestManagerLite Topology

=head1 BUGS

Some probably exist but have not been listed yet.

=head1 STATUS

  Shared

=head1 AUTHOR

  GV Govindasamy (gvg@netapp.com)
  $Id: $
  Last revised $Date: 2008/07/04 $
    
=head1 COPYRIGHT

Copyright 2008, Network Appliance., All Rights Reserved

=cut

package Topology::NAS::NFS::7M_Sec;
use base qw(Topology::NAS::NFS::7M);

use strict;
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use Utils qw(make_conn);
use Krb::Admin;
use Tharn qw($Log config_result);

use Data::Dumper qw(Dumper);

use Options;
my $Options;
my $Options_hash;

use RemoteExecution;
my $Remote = new RemoteExecution;

###################################################################

$Options = new Options;
$Options->parameters(definitions=>
   [
    {
        name         => "NFS_SECURITY",
        spec         => "s",
        description  => "The security type used by the clients to mount NFS filesystem",
        default      => "krb5",
        valid_values => ["krb5","krb5i","krb5p","sys"],
    },
    {
        name         => "KRB_DOMAIN",
        spec         => "s",
        description  => "Kerberos Domain Name. Required when NFS_SECURITY=*krb*",
        default      => undef,
    },
    {
        name         => "KRB_REALM",
        spec         => "s",
        description  => "Kerberos Realm Name. Required when NFS_SECURITY=*krb*",
        default      => undef,
    },
    {
        name         => "KDC",
        spec         => "s",
        description  => "Kerberos KDC Server. Required when NFS_SECURITY=*krb*",
        default      => undef,
    },
    {
        name         => "TESTUSER",
        spec         => "s@",
        description  => "Kerberos needs at least one user name. Take the current user as the default",
        default      => undef,
    },
    ]);

sub new {
    my $invocant = shift;
    my $class = ref($invocant) || $invocant;
    my $self = {
                @_,
            };
    $Log->enter();
    bless($self, $class);

    $Log->exit();
    return $self;
}

# The users list that will be used by this test run. 
# If more users are given than the number of test instances (each instance can use unique user
# at the maximum) given, then configure the only users that will be used during filer_setup.
my $used_users;

sub define_topology {
    my $self = shift;
    $Log->enter();

    $Options_hash = $Options->get_hash();
    if(! @{$Options_hash->{TESTUSER}}) {
	$Log->debug("This topology requires TESTUSER to be set. Default to the current user, as no TESTUSER is given.");
	$Options->set(TESTUSER => ["$ENV{LOGNAME}"]);
    }
    $Options_hash = $Options->get_hash();

    # Execute the one from the parent
    $self->SUPER::define_topology(@_);

    # Make sure Krb information is sufficient if NFS_SECURITY is set to use Krb
    if($Options_hash->{NFS_SECURITY} =~ /krb/) {
        die config_result("Please specify KDC") if(! $Options_hash->{KDC});
        die config_result("Please specify KRB_DOMAIN") if(! $Options_hash->{KRB_DOMAIN});
        die config_result("Please specify KRB_REALM") if(! $Options_hash->{KRB_REALM});
    }

    #
    # Set MOUNT_OPTIONS and export options for each instance appropriately.
    #
    my $krb_options = "sec=".$Options_hash->{NFS_SECURITY};
    my $export_options = ($Options_hash->{EXPORT_OPTIONS}) ? $Options_hash->{EXPORT_OPTIONS} : "rw,anon=0";
    $export_options .= ",".$krb_options;

    #
    # Set export options for all exports
    # 
    foreach my $volpath (keys %{$self->{filer_setup}{exports}}) {
	$Log->debug("Setting export options $export_options for $volpath");	
	$self->{filer_setup}{exports}{$volpath}{options} = $export_options;
    }

    #
    # Set MOUNT_OPTIONS for all client instances
    #
    foreach my $client ($self->clients()) {
        my $testusers;
        foreach my $inst (keys %{$self->{test_options}{$client}}) {
            $self->{test_options}{$client}{$inst}{MOUNT_OPTIONS} = $krb_options;
            if($self->{test_options}{$client}{$inst}{TESTUSER}) {
                push @$testusers, $self->{test_options}{$client}{$inst}{TESTUSER};
            }
        }
        $self->{client_setup}{$client}{KDC} = $Options_hash->{KDC};
        $self->{client_setup}{$client}{KRB_REALM} = $Options_hash->{KRB_REALM};
        $self->{client_setup}{$client}{KRB_DOMAIN} = $Options_hash->{KRB_DOMAIN};
        $self->{client_setup}{$client}{CLIENTNAME} = $client;
        $self->{client_setup}{$client}{TESTUSER} = $testusers if(@$testusers);
	push @$used_users, @$testusers if(@$testusers);
    }
    # Keep only the unique users in this list.
    @$used_users = keys %{{ map {$_ => 1} @$used_users }};

    $Log->exit(Dumper($self));
}

sub filer_setup {
    my ($self, %options) = @_;
    $Log->enter();

    my $filer = ($self->filers())[0];
    # If NFS security is set to Kerberos, then setup Kerberos
    if($Options_hash->{NFS_SECURITY} =~ /krb/) {
        my $kadmin = new Krb::Admin(kdc    => $Options_hash->{KDC},
                                    domain => $Options_hash->{KRB_DOMAIN},
                                    realm  => $Options_hash->{KRB_REALM});

        # $kadmin->setup_kdc(); This is not supported for now, hence setup manually
        
        $kadmin->setup_filer( filer => $filer );
        $kadmin->add_users(users => $used_users);
    }

    # Execute the one from the parent
    $self->SUPER::filer_setup(%options);

    $Log->exit();
}

sub client_setup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_setup(%options);

    my $kadmin = new Krb::Admin(kdc    => $options{KDC},
                                domain => $options{KRB_DOMAIN},
                                realm  => $options{KRB_REALM});

    $kadmin->setup_host(host => $options{CLIENTNAME},
                        user => $options{TESTUSER});

    $Log->exit();
}

sub DESTROY {
    my $self = shift;
    local($@);

    $self->client_cleanup();
}


sub client_cleanup {
    my ($self, %options) = @_;
    $Log->enter();

    # ??? - GV Kerberos Cleanup should be added here. For now, it's harmless leaving those clients
    # configured for Kerberos, since kerberos will kick in only if kerberos option is used to mount
    # the clients.

    # Execute the one from the parent
    $self->SUPER::client_cleanup(%options);
    
    $Log->exit();
}

######################### Internal Routines ##################

1;
