# Hey emacs, this is basically -*- perl -*-
# $Id: $

## @summary     NFS Stress Scalability 10Mode Topology Library
##
## @author      gvg@netapp.com
##
## @pod here

=head1 NAME

Topology::NAS::NFS::10M - NFS Stress Scalability test Topology Library for 10Mode ONTAP

=head1 SYNOPSIS

    See Topology.pm

=head1 DESCRIPTION

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/TestManagerLite

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/Topology

=head1 TODO

=head1 SEE ALSO

Options TestManagerLite Topology

=head1 BUGS

Gv writes bug free code. He reviewed it. 

=head1 STATUS

  Awesome

=head1 AUTHOR

  Jd (dascenj@netapp.com)
  $Id: $
  Last revised $Date: 2011 Mar 26 $
    
=head1 COPYRIGHT

Copyright 2011, Network Appliance., All Rights Reserved


=cut

package Topology::NAS::NFS::CMode;
use base qw(Topology::NAS::NFS);

use strict;
use warnings;

use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use Limits;
use SayWrap qw(sayEOF);
use Tharn qw($Log $Staf subtest config_result script_result fail_result);
use NACL::C::Node;
use NACL::C::SystemServicesNtpServer;
use NACL::C::SystemServicesNtpConfig;
use NACL::C::Vserver;
use NACL::C::ExportPolicy;
use NACL::C::ExportPolicyRule;
use NACL::C::VolumeEfficiencyPolicy;
use NACL::C::VolumeEfficiency;
use NACL::C::Interface;
use NACL::C::Route;
use NACL::C::RoutingGroups;
use NACL::C::SystemServicesDns;
use NACL::C::Volume;
use NACL::C::VserverNFS;
use NACL::C::VolumeFlexcache;
use NACL::C::VserverServicesNisDomain;
use NACL::C::QosWorkload;
use NACL::C::UnixUser;
use NACL::C::UnixGroup;
use NACL::C::Quota;
use NACL::C::QuotaPolicy;
use NACL::C::QuotaPolicyRule;
use NACL::C::VserverServicesLdapClient;

use Topology::Discover;
use Topology::Lookup;

use SSQA::CMode::ExportPolicyRule;
use SSQA::CMode::ExportPolicy;
use SSQA::License;

use NATE::BaseException qw(:try);

use Data::Dumper qw(Dumper);

use Utils::IPManager;

use Options;
my $Options;
my $Options_hash;
my @nodes;

my $discovered_topology;

my $Server;

###################################################################

$Options = new Options;
$Options->parameters(
    definitions => [
        {name         => "NSSWITCH",
         spec         => "s@",
         delimeters   => ":",
         valid_values => ["file", "ldap", "nis"],
         default      => 'file:nis',
         description  => "Set the ns-switch to the appropriate name service",},
        {name       => "LDAP_SERVERS",
         spec       => "s@",
         delimeters => ":",
         default    => '10.229.155.141',
         description =>
           "LDAP servers(colon seperated)to use for client authentication",},
        {name    => "LDAP_BASE_DN",
         spec    => "s",
         default => "dc=ssqa,dc=gdl,dc=englab,dc=netapp,dc=com",
         description =>
           "LDAP Base Dn to search on ldap server for client authentication",},
        {name         => "VOL_TYPE",
         spec         => "s",
         description  => "The type of volume to be created for testing",
         valid_values => ["flex", "striped", "mixed"],
         default      => undef,},
        {name        => "VOLUME_NAME",
         spec        => "s",
         description => "a string to use in naming the volume",
         default     => undef,},
        {name        => "VOLUME_EFFICIENCY",
         spec        => "s",
         description => "controls the enabling of volume efficiency features",
         default     => 1},
        {name => "VOLUMES_PER_INSTANCE",
         spec => "s",
         description =>
           "The number of volumes that each instance should perform IO on",
         default => 1,},
        {name => "VOLUME_IO_ALL_REMOTE",
         spec => "s",
         description =>
           "If true, then all IO to any volume will always cross the cluster network",
         default => 0,},
        {name => "VOLUME_IO_ALL_LIFS",
         spec => "s",
         description =>
           "If true, then all IO to any volume will fill DEST_PATH with an access path through every lif on the given vserver",
         default => 0,},
        {name        => "FILER_DISCOVER",
         spec        => "s",
         description => "Discover the existing filer configuration",
         default     => 1,},
        {name => "USE_EXISTING_FILER_SETUP",
         spec => "s",
         description =>
           "Use the current filer configuration as is. Client setup is executed and directory structures are still created",
         default => 0,},
        {name         => "VOL_STRIPE_FORMAT",
         spec         => "s",
         description  => "The format type used for striped volumes",
         valid_values => ["full", "data"],
         default      => undef,},
        {name => "QUOTA_VOLUME_ENABLE",
         spec => "boolean",
         description =>
           "Enable a volume quota for each user that will utilize said volume",
         default => 0,},
        {name    => "ALLCLIENTS_2_ALLVOLS",
         spec    => "boolean",
         default => 0,
         description =>
           "Determines whether or not a client should access all available mount points on the given server.",
        },
        {name       => "DNS_SERVERS",
         spec       => "s@",
         delimiters => ":",
         default    => '10.229.152.23:10.229.152.22',
         description =>
           "The name servers (colon seperated) to use for name resolution",},
        {name => "DNS_DOMAINS",
         spec => "s@",
         default =>
           "ssqa.gdl.englab.netapp.com,gdl.englab.netapp.com,rtp.netapp.com,eng.netapp.com,hq.netapp.com,netapp.com",
         description => "The domains to search for name resolution",},
        {name        => "NIS_DOMAIN",
         spec        => "s",
         default     => "ssqa.gdl.englab.netapp.com",
         description => "The NIS domain name",},
        {name        => "NIS_SERVERS",
         spec        => "s@",
         default     => "10.229.152.151,10.229.152.152",
         description => "The NIS servers hosting the NIS_DOMAIN",},
        {name        => "TIME_ZONE",
         spec        => "s",
         description => "The time zone to configure on the filer",
         default     => undef,},
        {name        => "TIME_SERVERS",
         spec        => "s@",
         delimiter   => ":",
         description => "The time host servers the filer should sync with",
         default     => undef,},
        {name => "FILER_LICENSE",
         spec => "boolean",
         description =>
           "Control whether or not the proper licensing should be guaranteed",
         default => 1,},
        {name        => "TESTUSER",
         spec        => "s@",
         description => "List of test users to use for the testing",
         default     => undef,},
        {name        => "VSERVER_NAME",
         spec        => "s",
         description => "The base text to use for naming vservers",
         default     => undef,},
        {name        => "TESTUSER_FILE",
         spec        => "s",
         description => "file containing the list of testusers who test run as",
         file_tests  => "fr",
         subroutine  => [\&_read_testuser_file],},
        {name        => "MAX_ACES",
         spec        => "s",
         default     => "0",
         description => "Controls the enabling of max aces on NFS vservers",},
        {name => "NUM_EXPORT_RULES",
         spec => "s",
         description =>
           "The number of export rules per export policy to be configured",
         default => 1,},
        {name    => "FLEXCACHE_PERCENT",
         default => 0,
         spec    => "s",
         description =>
           "The percentage of the total volumes that will be designated as flex cache origin volumes",
        },
        {name    => "FLEXCACHE_AGGRS",
         default => undef,
         spec    => "s@",
         description =>
           "The names of the aggregates where the flexcache volumes are allowed to live",
        },
        {name    => "NUM_VSERVERS",
         spec    => "s",
         default => 1,
         description =>
           "The number of virtual servers to create in the cluster.",},
        {name        => "NUM_LIFS",
         spec        => "s",
         default     => 0.99999,
         description => "The number of LIFS to create in the cluster.",},
        {name        => "DATA_LIFS",
         spec        => "s@",
         description => "The name of data interfaces that can be used",},
        {name => "ACCESS_ALLOW",
         spec => "s@",
         description =>
           "The netgroup or hostname that is to be granted access."},
        {name => "SKTRACE_OPTIONS",
         spec => "s@",
         description =>
           "A list of values to be modified with the sysctl command. Example: SKTRACE_OPTIONS=sysvar.sktrace.clear=1,sysvar.sktrace.Nfs4Lock_enable=63"
        },
        {name        => "VSERVER",
         spec        => "s",
         description => "VSERVER to use",
         required => "Enter the VSERVER test should run against on the cluster",
         default  => "vs0",},
        {name        => "MNTPNT_MODE",
         description => "The mode to set the client mountpoints to.",
         default     => "777",},
        {name => "DEST_PATHS_FILE",
         spec => "s",
         description =>
           "file containing the list of volume paths for clients to mount to",
         file_tests => "fr",
         subroutine => [\&_read_volpaths_file],},]);

sub new {
    my $invocant = shift;
    my $class    = ref($invocant) || $invocant;
    my $self     = {@_,};
    $Log->enter();
    bless($self, $class);

    $Log->exit();
    return $self;
} ## end sub new

sub define_topology {
    my ($self, %options) = @_;

    #my $self = shift;
    $Log->enter();

    $Options_hash = $Options->get_hash();

    # Options used here
    my $ip_manager_start_delay = $Options_hash->{IP_MANAGER_DELAY};
    my $num_exports            = $Options_hash->{NUM_EXPORTS};
    my $dup_ips                = $Options_hash->{DUP_IPS};

    $Log->trace("Options: \n" . Dumper($Options_hash));

    # Execute the one from the parent
    $self->SUPER::define_topology(%options);

    # Startup the IP Manager
    $self->{ipManager} = $self->_start_ip_manager();

    # Obtain the node object
    @nodes = NACL::C::Node->find();
    my $node = $nodes[0];

    # Create a limits object
    my $limits = Limits->new(host => $node->hostrec()->id());
    my $filer = @{$Options->get(option => "FILER")}[0];

    my $limits = Limits->new(host => $filer);

    # discover the existing layout of the filer
    $discovered_topology = Topology::Discover::run(node => $node)
      if ($Options_hash->{FILER_DISCOVER});

    my $ipManager = $self->{ipManager};
    my @destNets  = $ipManager->destination_networks();

    # Keep the names of the filers around. We need to know how
    # many there are when it comes to figuring flexcache stuff
    foreach my $n (@nodes) {
        push(@{$self->{nodes}}, $n->hostrec->id());
    }
    $Log->comment("The topology will utilize these filers: "
                  . join(',', @{$self->{nodes}}));

    my $apiSet = $node->apiset(category  => "Node",
                               interface => "CLI",
                               set       => "CMode");

    # Get all the data ports in the cluster. Use this to determine the LIFS to create
    my $dataPortResponse;
    try {
        $dataPortResponse =
          $apiSet->network_port_show(role => "data",
                                     link => "up");
    }
    catch NATE::BaseException with {
        my $ex = shift;
        die config_result("EXCEPTION: " . $ex->text());
    };

    # Create the client_setup hash
    $self->_create_client_setup();

    $self->_configure_volumes();

    ##########################################
    # Add all the vservers to the Topology hash
    ##########################################

    # vservers are just round-robin(ed) across the given aggrs
    # assign volumes to vservers by dividing requested number of vserver by number of volumes.
    my $numVservers    = $Options_hash->{NUM_VSERVERS};
    my @usableVolumes  = $self->get_volumes_usable();
    my $volsPerVserver = int(scalar(@usableVolumes) / $numVservers);
    if ($volsPerVserver < 1) {
        die config_result(  "The requested number of volumes ("
                          . scalar(@usableVolumes)
                          . "), is not sufficient to allow at least"
                          . " one volume for each ("
                          . $numVservers
                          . ") vserver.");
    }

    my @allDataPorts = @{$dataPortResponse->get_parsed_output()};
    my @dataPorts;
    my @dataInterfaces = @{$Options->get(option => "DATA_INTERFACES")};

    # Get an array of all the data ports on the filers specified
    foreach my $dataPort (@allDataPorts) {
        next
          if (    (@dataInterfaces)
              and (not grep(/$dataPort->{port}/i, @dataInterfaces)));

        # Only use data ports on the filers specified by FILER_LIST
        foreach my $nodeObj (@nodes) {
            my $nodeName = $nodeObj->hostrec()->id();
            if ($nodeName =~ /$dataPort->{node}/i) {
                push(@dataPorts, $dataPort);
                last;
            }
        }
    } ## end foreach my $dataPort (@allDataPorts)

    # Verify we found at least one data port
    if (not scalar(@dataPorts)) {
        die config_result(
             "Could not find even a single data port on the specified filers.");
    }

    my $numLifs = $Options->get(option => "NUM_LIFS");
    if ($numLifs == 0.99999) {
        $numLifs = $numVservers * scalar(@dataPorts);
        $Log->comment(
            "the number of lifs is set to $numLifs. This is one lif for each of the "
              . scalar(@dataPorts)
              . " data ports found in each of the $numVservers vservers.");
    } else {
        $Log->comment(
            "the number of lifs is set to $numLifs as supplied by the user parameter NUM_LIFS."
            );
    }

    die config_result(
        "Calculated $numLifs LIFs and $numVservers Vservers. The cluster requires at least "
          . "one lif per vserver. Check the value of NUM_LIFS.")
      if ($numLifs < $numVservers);

    my $numLifsPerVserver = $numLifs / $numVservers;
    die config_result(
        "The number of lifs specified by NUM_LIFS ($numLifs) does not result in a round number of "
          . "LIFs for each vserver in NUM_VSERVERS ($numVservers)")
      if (   ($numLifsPerVserver != int($numLifsPerVserver))
          or (not $numLifsPerVserver >= 1));

    $Log->comment(
        "Will create $numLifsPerVserver lifs for each of $numVservers vservers."
        );

    my $maxAllowedLifsPerNode = $limits->getvalue(limit => "max_ip_lifs");
    die config_result(
        "This configuration exceeds the maximum allowed number of lifs per node of $maxAllowedLifsPerNode."
      )
      if (($numLifs / scalar(@{$self->{nodes}})) > $maxAllowedLifsPerNode);

    my $aggrIndex     = 0;
    my $volIndex      = 0;
    my $dataPortIndex = 0;
    foreach my $number (1 .. $numVservers) {

        # Make up a name
        my $vserverBaseName = $Options->get(option => "VSERVER_NAME");
        $vserverBaseName = $self->testcase_string() if (not $vserverBaseName);
        my $vserverName          = $vserverBaseName . '_vs_' . $number;
        my $vserverRootVolName   = $vserverName . '_root_vol';
        my $numLifsOnThisVserver = $numLifsPerVserver;

        # If we reach the end of the aggr array, reset to zero
        # this causes the vserver root vols to be spread around the given aggrs.
        if (not $aggrIndex < scalar(@{$Options_hash->{aggregate}})) {
            $aggrIndex = 0;
        }

        my $aggrName = $Options_hash->{aggregate}[$aggrIndex];

        # Build a vserver hash ref and hang on to it
        my $vserverHash = {rootvol_security_style => "unix",
                           name                   => $vserverName,
                           nis_servers => \@{$Options_hash->{NIS_SERVERS}},
                           nis_domain  => $Options_hash->{NIS_DOMAIN},
                           aggregate   => $aggrName,};

        $self->{filer_setup}->{vservers}->{$vserverName} = $vserverHash;

        my $vserver = $self->{filer_setup}->{vservers}->{$vserverName};

        # Make a root volume for this vserver, add it to the volumes hash
        my $vserverRootHash = {name         => $vserverRootVolName,
                               vserver      => $vserver,
                               aggregate    => $aggrName,
                               isRootVolume => 1,};

        # Put the vserver root vol with the rest of the volumes
        $self->{filer_setup}->{volumes}->{$vserverRootHash->{name}} =
          $vserverRootHash;

        # give the vserver access to the root vol object stored with the rest of the volumes
        $vserver->{rootvolume} =
          $self->{filer_setup}->{volumes}->{$vserverRootHash->{name}};

        #####################################
        # Setup rule sets and rules
        #####################################

        # Set up the rule set container hash for this vserver
        foreach my $ruleSetNum (1 .. $num_exports) {
            my $ruleSetName = $vserverName . "_ruleSet_" . $ruleSetNum;

            # Build a rule set
            my $ruleSetHash = {name    => $ruleSetName,
                               vserver => $vserver,};

            $vserver->{exportRuleSet}->{$ruleSetName} = $ruleSetHash;

            my $ruleSet = $vserver->{exportRuleSet}->{$ruleSetName};

            # Keep all rule references in another hash at $self->{ruleSets} also
            $self->{filer_setup}->{ruleSets}->{$ruleSetName} = $ruleSet;

            # Build the requested number of rules inside the rule set, store on the set as an array
            my $numRules        = $Options_hash->{NUM_EXPORT_RULES};
            my @clientMatchList = @{$Options_hash->{ACCESS_ALLOW}};
            my $numAccessAllow  = scalar(@clientMatchList);

            # Need to figure out if we will exceed the NUM_EXPORT_RULES value
            # If the NUM_EXPORT_RULES value is > the default (1), then adjust it
            # to match the rules needed for ACCESS_AllOW.
            if (($numRules == 1) and ($numRules < $numAccessAllow)) {
                $Log->comment(
                    "The NUM_EXPORT_RULES value will be modified to match the number of values sepcified "
                      . "in ACCESS_ALLOW ($numAccessAllow). This occurs because the NUM_EXPORT_RULES value has not been modified "
                      . "from the default value of 1, and ACCESS_ALLOW entries exceed this value."
                      );
                $numRules = $numAccessAllow;

            } elsif ($numRules < $numAccessAllow) {
                die config_result(
                    "You have specified $numAccessAllow clientmatch values. Each of these leads to the "
                      . "creation of an independent rule to grant access. When NUM_EXPORT_RULES is set "
                      . "we cannot exceed it's value. Please review and adjust the values of NUM_EXPORT_RULES "
                      . "and ACCESS_ALLOW.");
            }

            $Log->debug("Defining $numRules rules for this export.");
            my $clientMatchIndex = 0;
            foreach my $ruleNum (1 .. $numRules) {

                # Create a basic, random rule
                my ($rule, undef) =
                  SSQA::CMode::ExportPolicyRule::generate_random();

                # Fill in the specific stuff
                $rule->{ruleindex}     = $ruleNum;
                $rule->{vserver}       = $vserver;
                $rule->{exportRuleSet} = $ruleSet;

                # Do some magic to make sure all dummy rules appear before
                # legitimate rules.
                my $clientMatch = undef;
                if (($numRules - $ruleNum) <= $numAccessAllow) {
                    $clientMatch = $clientMatchList[$clientMatchIndex];

                    # Each clientmatch value makes a seperate rule. We plan to cycle through the
                    # clientmatch values, and once we pass the end of that array, the code starts to
                    # substitute random rules as needed to meet the NUM_RULES value.
                    $clientMatchIndex++;
                }

                # Make the last rule an allow all rule, this should make the filer check each existing "dummy" rule
                # before a match is found at the end of the list
                if ($clientMatch) {
                    $rule->{clientmatch} = $clientMatch;
                } elsif ($numRules == $ruleNum) {
                    $rule->{clientmatch} = "0.0.0.0/0";
                }

                # Add the hashref for this rule to the ruleset in the vserver
                push(@{$ruleSet->{rules}}, $rule);

                # Also create array of $self->{rules}
                push(@{$self->{filer_setup}->{rules}}, $rule);
            } ## end foreach my $ruleNum (1 .. $numRules)
        } ## end foreach my $ruleSetNum (1 .....)

        # Figure out all the fun to go with each lif on this vserver
        # Use the response object to gather the parsed information on data ports
        my @destNetworks = $ipManager->destination_networks();
        if (not @destNetworks) {
            die config_result(
                   "Failed to obtain a destination network from the IPManager");
        }
        my $netObj = $destNetworks[0];

        foreach my $num (1 .. $numLifsOnThisVserver) {
            my $dataPort = $dataPorts[$dataPortIndex];
            my $ipObj    = $netObj->assign_next_address();
            if (not $ipObj->address()) {
                die config_result(
                    "Unable to obtain an unassigned address from the IPManager for "
                      . " the "
                      . $netObj->networkID()
                      . " network. Check the value of DESTIP?");
            }

            # Create a name for the lif and add the details to the topology hash
            my $lifName =
                $vserverName
              . "_lif${num}_"
              . $dataPort->{node} . "_"
              . $dataPort->{port};
            my $lifHash = {name        => $lifName,
                           role        => "data",
                           "home-node" => $dataPort->{node},
                           "home-port" => $dataPort->{port},
                           address     => $ipObj->address(),
                           netmask     => $ipObj->netmask(),
                           server      => $vserver,};

            # Keep the lif in its own bucket.
            $self->{filer_setup}->{lifs}->{$lifName} = $lifHash;

            # Tie the lif to the vserver
            $vserver->{lifs}->{$lifName} =
              $self->{filer_setup}->{lifs}->{$lifName};

            # Round robin LIFs on available ports
            $dataPortIndex = 0 if (++$dataPortIndex > (scalar(@dataPorts) - 1));
        } ## end foreach my $num (1 .. $numLifsOnThisVserver)

        # Bump the index of the aggr array, so that the next vserver goes to the next aggregate.
        $aggrIndex++;
    } ## end foreach my $number (1 .. $numVservers)

    # assign the data volumes to the vservers round robin like
    my $vserverIndex = 0;
    my @vserverKeys  = keys(%{$self->{filer_setup}->{vservers}});

    foreach my $volume ($self->get_volumes_usable()) {
        my $vserver =
          $self->{filer_setup}->{vservers}->{$vserverKeys[$vserverIndex]};

        # Don't mess with vserver root volumes
        next if ($volume->{isRootVolume});

        # add a link in the vserver hash to point to the volume objects.
        $volume->{vserver} = $vserver;

        # the volumes should show up under the vservers
        $vserver->{volumes}->{$volume->{name}} = $volume;

        # add the cache volumes that use this volume is their origin also
        # to the same vserver
        my @cacheVols = $self->get_volumes_cache(origin => $volume->{name});
        foreach my $c (@cacheVols) {
            $c->{vserver} = $vserver;
            $vserver->{volumes}->{$c->{name}} = $c;
        }

        # round robin the vservers
        $vserverIndex = 0 if (++$vserverIndex > (scalar(@vserverKeys) - 1));
    } ## end foreach my $volume ($self->get_volumes_usable...)

    # Configure the sktracing if it is applicable. Normally something like this
    # should be placed in filer setup. However, I have decided to stick it here in
    # define_topology because I want to be sure that sktracing is set correctly even
    # when the topology is run with SKIPSTAGE=filer_setup to use an existing setup
    $self->sktrace_setup(1);

    # Let's see if we were called from the CMode_Virtual class,
    # if so , we need to head back there now.
    my ($package, $filename, $line) = caller;
    if ($package =~ /virtual/i) {
        $Log->trace(
                "Returning to the virtual topology for synthetic client setup");
        $Log->exit();
        return;
    }

    # Setup the instances
    $self->_configure_instances();

    # Configure each instance with a DEST_PATHS
    $self->_assign_path_to_instance();

    # Set the size of each volume based on the number of instances.
    $self->_set_usable_volume_size();

    # Determine the users who will need quotas configured
    if ($Options->get(option => "QUOTA_VOLUME_ENABLE")) {
        $self->_define_volume_quotas();
    }

    # Get creative about logging all our information
    my $top_logdir = Tharn::param('TOP_LOGDIR');
    my $runid      = Tharn::param('RUNID');
    my $lonely_log = $top_logdir . '/' . $runid . '_config_info.log';
    if ($Log->may_debug()) {
        $Log->comment("define_topology Topology: \n" . Dumper($self),
                      path => $lonely_log);

    } else {
        $Log->comment("filer_setup:" . Dumper($self->{filer_setup}),
                      path => $lonely_log);
        $Log->comment("client_config:" . Dumper($self->{client_config}),
                      path => $lonely_log);
        $Log->comment("test_options:" . Dumper($self->{test_options}),
                      path => $lonely_log);
    }

} ## end sub define_topology

sub filer_setup {
    my ($self, %options) = @_;
    $Options_hash = $Options->get_hash();

    # I know, the goto is a drag. This is here because we need to have the
    # directory creation right where it is due to the ACCESS_ALLOW rules and
    # making sure we have the right setup for it.
    goto CREATE_DIRS if ($Options_hash->{USE_EXISTING_FILER_SETUP});

    # Obtain the node object
    my $node = $nodes[0];
    my %vserverNfsOptions;

    my $gracePeriodSeconds = 45;

    # Execute the one from the parent
    $self->SUPER::filer_setup(%options);

    if ($Options_hash->{FILER_LICENSE}) {
        my $localResult =
          SSQA::License::add(command_interface => $node, service => "NFS");
        if (not $localResult) {
            die config_result("Unable to license NFS");
        }
    }

    # Create vservers
    foreach my $key (keys(%{$self->{filer_setup}->{vservers}})) {
        my $vserver = $self->{filer_setup}->{vservers}->{$key};

        $Log->trace("Creating vserver $vserver->{name}");
        $Log->debug("vserver data:" . Dumper($vserver));
        my @nsswitch = ("file");

        try {
            NACL::C::Vserver->create(
                vserver                     => $vserver->{name},
                command_interface           => $node,
                "ns-switch"                 => \@nsswitch,
                "rootvolume-security-style" => "unix",
##### BURT: 541946
                "method-timeout" => "300",
                rootvolume       => $vserver->{rootvolume}->{name},
                aggregate        => $vserver->{aggregate});

        } ## end try
        catch NATE::BaseException with {
            my $ex = shift;
            die config_result("EXCEPTION: " . $ex->text());
        };

        # The vservers have to have NFS on for any of the modify commands to work
        try {
            NACL::C::VserverNFS->on(command_interface => $node,
                                    vserver           => $vserver->{name});
        }
        catch NATE::BaseException with {
            my $ex = shift;
            die config_result("EXCEPTION: " . $ex->text());
        };

        # Build an ldap client config for possible use depending on the name service selection.
        my $ldapclient = ("ldapclient");

        # base-dn only supports scalar at this point
        my $ldapbasedn = $Options_hash->{LDAP_BASE_DN};
        try {
            NACL::C::VserverServicesLdapClient->create(
                                   command_interface => $node,
                                   "client-config"   => $ldapclient,
                                   servers => \@{$Options_hash->{LDAP_SERVERS}},
                                   vserver => $vserver->{name},
                                   "base-dn" => $ldapbasedn,);
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->warn("EXCEPTION: " . $ex->text());
        };

        # create a volume efficiency policy for this vserver
        my $efficiency_policy_name = "nfs_topo_policy";
        if ($Options_hash->{VOLUME_EFFICIENCY}) {
            try {
                NACL::C::VolumeEfficiencyPolicy->create(
                                              policy => $efficiency_policy_name,
                                              schedule          => "hourly",
                                              command_interface => $node,
                                              vserver => $vserver->{name});
            }
            catch NATE::BaseException with {
                my $ex = shift;
                die config_result("EXCEPTION: " . $ex->text());
            };
        } ## end if ($Options_hash->{VOLUME_EFFICIENCY...})

        # Setup QOS workload if specified.
        if ($vserver->{qos_workload}) {
            try {
                NACL::C::QosWorkload->create(
                                           command_interface => $node,
                                           workload => $vserver->{qos_workload},
                                           vserver  => $vserver->{name});
            }
            catch NATE::BaseException with {
                my $ex = shift;
                die config_result("EXCEPTION: " . $ex->text());
            };
        } ## end if ($vserver->{qos_workload...})
############
        # Configure name service based on the values in NSSWITCH param

        # Build the array to search through for name service info.
        my @nsswitchuse = @{$Options->get(option => "NSSWITCH")};

        # Print a warning about using ldap and nis together but allow it to happen
        if ((grep /ldap/, @nsswitchuse) && (grep /nis/, @nsswitchuse)) {
            $Log->warn(
                "You have specified both ldap and nis as the NS for your vservers."
                );
        }

        # Test to see if we should configure ldap.
        if (grep(/ldap/, @nsswitchuse)) {

            # Going to need to run raw command.
            my $APISet = $node->apiset(category  => "Node",
                                       interface => "CLI",
                                       set       => "CMode");
            my $vserverldapsrvc = $APISet->execute_raw_command(

                #Create the service per vserver
                # I am using the raw command here because NACL does not have "client-enabled" listed as a valid option. The
                # error is  "EXCEPTION: The following parameter was passed in the call to NACL::C::Component::_common_validate_with
                # but was not listed in the validation options: client-enabled"

                command =>
                  "vserver services ldap create $vserver->{name} -client-config $ldapclient -client-enabled true"
                  );

            # Modify the ns-switch to use values provided with NSSWITCH param.
            try {
                NACL::C::Vserver->modify(
                         "ns-switch" => \@{$Options->get(option => "NSSWITCH")},
                         command_interface => $node,
                         vserver           => $vserver->{name});

            }
            catch NATE::BaseException with {
                my $ex = shift;
                die config_result("EXCEPTION: " . $ex->text());
            };
        } ## end if (grep(/ldap/, @nsswitchuse...))

        # Test to see if we should configure nis ( file:nis ) is the default.
        if (grep(/nis/, @nsswitchuse)) {
            try {
                NACL::C::VserverServicesNisDomain->create(
                                          vserver           => $vserver->{name},
                                          active            => "true",
                                          command_interface => $node,
                                          domain  => $vserver->{nis_domain},
                                          servers => \@{$vserver->{nis_servers}}
                                          );

            } ## end try
            catch NATE::BaseException with {
                my $ex = shift;
                die config_result("EXCEPTION: " . $ex->text());
            };

            # Enable the nis server for nsswitch option on this vserver
            #my @nsswitch = ("file","nis");
            my $nsswitch = "file,nis";
            try {
                # NACL::C::Vserver->modify("ns-switch"       => $nsswitch,
                NACL::C::Vserver->modify(
                         "ns-switch" => \@{$Options->get(option => "NSSWITCH")},
                         command_interface => $node,
                         vserver           => $vserver->{name});

            }
            catch NATE::BaseException with {
                my $ex = shift;
                die config_result("EXCEPTION: " . $ex->text());
            };
        } ## end if (grep(/nis/, @nsswitchuse...))

        # Adjust the size of the vserver root volume if needed. The default root volume size
        # is too small to accomodate large numbers of volumes. 20MB per 400 volumes seems sufficient.
        my %root_vol_options = (vserver           => $vserver->{name},
                                command_interface => $node,
                                volume => $vserver->{rootvolume}->{name},
                                "unix-permissions" => '---rwxrwxrwx',);
        my $vols_accomodated = 400;
        my $growth_increment = 20;
        my @volumes_on_vserver =
          $self->get_volumes_usable(vserver => $vserver->{name});
        my $num_of_vols = scalar(@volumes_on_vserver);
        if ($num_of_vols > $vols_accomodated) {
            $root_vol_options{size} =
              (($num_of_vols / $vols_accomodated) * 20) . "MB";
        }

        NACL::C::Volume->modify(%root_vol_options);

        # Create volumes
        # volumes are created with the default policy applied for now to allow setup to complete.
        # as setup is finishing, the volume will receive the intended rule set.
        foreach my $volume (@volumes_on_vserver) {

            # If there is no indication of value(k|m|g), assume mb
            $volume->{size} =
              ($volume->{size} =~ m/\d+[k|m|g|t]/i)
              ? $volume->{size}
              : $volume->{size} . "m";

            my %vol_create_options = (
                volume            => $volume->{name},
                vserver           => $vserver->{name},
                aggregate         => $volume->{aggregate},
                size              => $volume->{size},
                "junction-path"   => $volume->{"junction-path"},
                command_interface => $node,
##### BURT: 541946
                "method-timeout" => "300",);

            # BR and lower versions of ontap may not support volume efficiency.
            if ($Options_hash->{VOLUME_EFFICIENCY}) {
                $vol_create_options{"space-guarantee"}    = "none";
                $vol_create_options{"max-autosize"}       = "1TB";
                $vol_create_options{"autosize-increment"} = "100mb";
                $vol_create_options{"autosize"}           = "true";
                $vol_create_options{policy}               = "default";
            }

            my $volObj;
            try {
                $volObj = NACL::C::Volume->create(%vol_create_options);

            }
            catch NATE::BaseException with {
                my $ex = shift;
                die config_result("EXCEPTION: " . $ex->text());
            };

            # Wait a while for the volume to come online
            my $retries            = 0;
            my $volCreateSleepTime = 5;
            my $maxOnlineRetries   = 10;
            while ($volObj->state()->state() !~ /online/) {
                sleep $volCreateSleepTime;
                if (++$retries > $maxOnlineRetries) {
                    die config_result(  "Volume has not come online in "
                                      . $volCreateSleepTime * $retries
                                      . " seconds.");
                }
            }

            if ($volume->{qos_workload}) {
                try {
                    NACL::C::QosWorkload->create(
                                            command_interface => $node,
                                            workload => $volume->{qos_workload},
                                            volume   => $volume->{name},
                                            vserver  => $vserver->{name});
                }
                catch NATE::BaseException with {
                    my $ex = shift;
                    die config_result("EXCEPTION: " . $ex->text());
                };
            } ## end if ($volume->{qos_workload...})

            # Create the flexcache if needed
            if ($volume->{flexcache_origin}) {
                my @cacheVols =
                  $self->get_volumes_cache(origin => $volume->{name});

                foreach my $cacheVol (@cacheVols) {
                    try {
                        NACL::C::Volume->create(
                                   "flexcache-origin-volume" => $volume->{name},
                                   type              => $cacheVol->{type},
                                   aggregate         => $cacheVol->{aggregate},
                                   volume            => $cacheVol->{name},
                                   vserver           => $vserver->{name},
                                   command_interface => $node);
                    }
                    catch NATE::BaseException with {
                        my $ex = shift;

                        # Ignore burt.460797 http://web.netapp.com/~burt/views/459/459948.html
                        #die config_result("EXCEPTION: " . $ex->text()) unless ($ex->text() =~ /"are\s+not\s+valid\s+for\s+volumes\s+of\s+type\s+\'DC\'"/);
                        die config_result("EXCEPTION: " . $ex->text());
                    };
                } ## end foreach my $cacheVol (@cacheVols)
            } ## end if ($volume->{flexcache_origin...})

            # configure volume quotas
            if (defined(@{$volume->{quota_users}})) {

                $Options_hash = $Options->get_hash();

                # use the default policy for now. I cannot turn on volume quotas
                # when there is nothing in the default policy.
                my $policyName = "default";

                # create a rule for each user on this volume
                foreach my $userName (@{$volume->{quota_users}}) {
                    my $quotaSize = $volume->{size_per_instance};
                    $Log->comment(  "Setting disk quota to 90% of VOL_SIZE at "
                                  . $quotaSize * 0.9
                                  . "mb");
                    try {
                        NACL::C::QuotaPolicyRule->create(
                                  command_interface => $node,
                                  vserver           => $vserver->{name},
                                  volume            => $volume->{name},
                                  "disk-limit"      => $quotaSize . "m",
                                  "soft-disk-limit" => ($quotaSize * 0.5) . "m",
                                  "policy-name"     => $policyName,
                                  qtree             => "",
                                  type              => "user",
                                  target            => $userName);
                    } ## end try
                    catch NATE::BaseException with {
                        my $ex = shift;
                        die config_result("EXCEPTION: " . $ex->text());
                    };
                } ## end foreach my $userName (@{$volume...})

                # turn the quota on
                try {
                    NACL::C::Quota->on(command_interface => $node,
                                       vserver           => $vserver->{name},
                                       volume            => $volume->{name},);
                }
                catch NATE::BaseException with {
                    my $ex = shift;
                    die config_result("EXCEPTION: " . $ex->text());
                };
            } ## end if (defined(@{$volume->...}))
        } ## end foreach my $volume (@volumes_on_vserver)

        if ($Options_hash->{VOLUME_EFFICIENCY}) {

            # turn on compression, and assign the volume to the vserver efficiency policy
            try {
                NACL::C::VolumeEfficiency->on(command_interface => $node,
                                              vserver => $vserver->{name},
                                              volume  => "*",);
            }
            catch NATE::BaseException with {
                my $ex = shift;
                die config_result("EXCEPTION: " . $ex->text());
            };

            try {
                NACL::C::VolumeEfficiency->modify(
                                              command_interface => $node,
                                              vserver => $vserver->{name},
                                              policy => $efficiency_policy_name,
                                              compression => "true",
                                              volume      => "*",);
            }
            catch NATE::BaseException with {
                my $ex = shift;
                die config_result("EXCEPTION: " . $ex->text());
            };
        } ## end if ($Options_hash->{VOLUME_EFFICIENCY...})

        # This is just here to show its nfs only.
        my $PROTOCOL = "nfs";
        if ($PROTOCOL =~ /nfs/) {

            $Log->trace("Configuring vservers for NFS...");

            # Ask the vserver which options are approriate. Turn them on if they exist.
            # we only need to populate the vserverNfsOptions hash once, the values are good
            # between vservers
            # Create a limits object
            my $limits = Limits->new(host => $node->hostrec()->id());
            my $filer = @{$Options->get(option => "FILER")}[0];
            my $limits = Limits->new(host => $filer);
            my $maxNfsv4Acl =
              ($Options_hash->{MAX_ACES})
              ? $limits->getvalue(limit => "max_nfsv4_acl_entries")
              : 400;

            try {
                NACL::C::VserverNFS->modify(
                                  vserver                 => $vserver->{name},
                                  access                  => "true",
                                  'mount-rootonly'        => "disabled",
                                  "udp"                   => "enabled",
                                  "tcp"                   => "enabled",
                                  "v3"                    => "enabled",
                                  "v4.0"                  => "enabled",
                                  "v4.0-acl"              => "enabled",
                                  "v4.0-read-delegation"  => "enabled",
                                  "v4.0-write-delegation" => "enabled",
                                  "v4.1"                  => "enabled",
                                  "v4.1-acl"              => "enabled",
                                  "v4.1-pnfs"             => "enabled",
                                  "v4.1-read-delegation"  => "enabled",
                                  "v4.1-write-delegation" => "enabled",
                                  "vstorage"              => "enabled",
                                  'v4-acl-max-aces'       => "$maxNfsv4Acl",
                                  'v4-id-domain' => $Options_hash->{NIS_DOMAIN},
                                  command_interface => $node,);

            } ## end try
            catch NATE::BaseException with {
                my $ex = shift;
                die config_result("EXCEPTION: " . $ex->text());
            };

            # Create all the rule sets for this vserver
            foreach my $ruleSetKey (keys(%{$vserver->{exportRuleSet}})) {
                my $ruleSet = $vserver->{exportRuleSet}->{$ruleSetKey};

                try {
                    NACL::C::ExportPolicy->create(vserver => $vserver->{name},
                                                  command_interface => $node,
                                                  policyname => $ruleSet->{name}
                                                  );

                }
                catch NATE::BaseException with {
                    my $ex = shift;
                    die config_result("EXCEPTION: " . $ex->text());
                };

                # Create all the rules to go with this set
                foreach my $rule (@{$ruleSet->{rules}}) {
                    try {
                        NACL::C::ExportPolicyRule->create(
                                            vserver => $vserver->{name},
                                            command_interface => $node,
                                            policyname  => $ruleSet->{name},
                                            protocol    => $rule->{protocol},
                                            clientmatch => $rule->{clientmatch},
                                            rorule      => $rule->{rorule},
                                            rwrule      => $rule->{rwrule},
                                            superuser   => $rule->{superuser},);

                    } ## end try
                    catch NATE::BaseException with {
                        my $ex = shift;
                        die config_result("EXCEPTION: " . $ex->text());
                    };

                } ## end foreach my $rule (@{$ruleSet...})
            } ## end foreach my $ruleSetKey (keys...)

            # add a rule to the default Export Policy to allow all. This is only meant to allow all the setup
            # to complete normally and will be undone as the setup is completed.
            SSQA::CMode::ExportPolicy::default_export_allow_all(
                                                    enable  => 1,
                                                    vserver => $vserver->{name},
                                                    command_interface => $node,
                                                    );

        } ## end if ($PROTOCOL =~ /nfs/)
    } ## end foreach my $key (keys(%{$self...}))

    # Create LIFs
    foreach my $key (keys(%{$self->{filer_setup}->{lifs}})) {
        my $lif = $self->{filer_setup}->{lifs}->{$key};

        try {
            NACL::C::Interface->create(lif               => $lif->{name},
                                       command_interface => $node,
                                       server      => $lif->{server}->{name},
                                       role        => $lif->{role},
                                       "home-node" => $lif->{"home-node"},
                                       address     => $lif->{address},
                                       netmask     => $lif->{netmask},
                                       "home-port" => $lif->{"home-port"},);

        } ## end try
        catch NATE::BaseException with {
            my $ex = shift;
            die config_result("EXCEPTION: " . $ex->text());
        };
    } ## end foreach my $key (keys(%{$self...}))

    # NFS4 grace period is 45 seconds. Make sure we have exhausted them before
    # we allow clients to try to connect.
    my $lifCreateTime = time();

    my $ipManager = $self->{ipManager};
    foreach my $netObj ($ipManager->destination_networks()) {
        foreach my $key (keys(%{$self->{filer_setup}->{vservers}})) {
            my $vserver = $self->{filer_setup}->{vservers}->{$key};

            # Create a default route for each interface
            try {
                NACL::C::Route->create(command_interface => $node,
                                       server            => $vserver->{name},
                                       "routing-group"   => 'd'
                                         . $netObj->networkID() . '/'
                                         . $netObj->cidr(),
                                       destination => '0.0.0.0/0',
                                       gateway     => $netObj->gateway(),);

            } ## end try
            catch NATE::BaseException with {
                my $ex = shift;
                die config_result("EXCEPTION: " . $ex->text());
            };
        } ## end foreach my $key (keys(%{$self...}))
    } ## end foreach my $netObj ($ipManager...)

    # Update the DNS configuration
    $self->_configure_filer_dns(node => $node);

    # Set the filer timezone and ntp
    foreach my $node (@nodes) {
        $self->_configure_filer_date_time(node => $node);
    }

  CREATE_DIRS:

    # Each volume will require a directory for every instance that will touch that volume
    $self->_create_instance_dirs();

    # done if we are using existing config. All we want to do is create directories.
    return if ($Options_hash->{USE_EXISTING_FILER_SETUP});

    # Remove the rule we created on the default policy. This is only done for nfs so
    # that the directory creation can succeed. Needs to be done on every vserver.
    # We will also apply all the policies that we created here.
    foreach my $vserverKey (keys(%{$self->{filer_setup}->{vservers}})) {
        my $vserver = $self->{filer_setup}->{vservers}->{$vserverKey};

        SSQA::CMode::ExportPolicy::default_export_allow_all(
                                                    enable  => 0,
                                                    vserver => $vserver->{name},
                                                    command_interface => $node,
                                                    );

        # Apply the rules specified by define_topology to each volume in the vserver.
        foreach
          my $vol ($self->get_volumes_usable(vserver => $vserver->{name})) {
            foreach my $ruleSet (
                     $self->get_export_rule_sets(vserver => $vserver->{name})) {

                try {
                    NACL::C::Volume->modify(vserver => $vserver->{name},
                                            command_interface => $node,
                                            volume            => $vol->{name},
                                            policy => $ruleSet->{name});

                }
                catch NATE::BaseException with {
                    my $ex = shift;
                    die config_result("EXCEPTION: " . $ex->text());
                };
            } ## end foreach my $ruleSet ($self->get_export_rule_sets...)
        } ## end foreach my $vol ($self->get_volumes_usable...)

        # Set the export policy up on the root volume of the given vserver
        # For now, we just set the same rules as the rest of the volumes on this vserver
        foreach my $vol ($self->get_volumes_root(vserver => $vserver->{name})) {
            foreach my $ruleSet (
                     $self->get_export_rule_sets(vserver => $vserver->{name})) {
                try {
                    NACL::C::Volume->modify(vserver => $vserver->{name},
                                            command_interface => $node,
                                            volume            => $vol->{name},
                                            policy => $ruleSet->{name});

                }
                catch NATE::BaseException with {
                    my $ex = shift;
                    die config_result("EXCEPTION: " . $ex->text());
                };
            } ## end foreach my $ruleSet ($self->get_export_rule_sets...)
        } ## end foreach my $vol ($self->get_volumes_root...)
    } ## end foreach my $vserverKey (keys...)

    # Make sure we allow 45 seconds to get past the grace period
    while ((time() - $lifCreateTime) < $gracePeriodSeconds) {
        NATE::Events::eventloop(seconds => 1);
    }

    $Log->comment(  "finishing filer setup at "
                  . time()
                  . ". The lif creation finished at $lifCreateTime");
    $Log->exit();
} ## end sub filer_setup

sub filer_cleanup {
    my ($self, %options) = @_;

    # Obtain the node object
    my $node = $nodes[0];

    # Stop the sktracing if it was enabled.
    if ($Options->get(option => "SKTRACE_OPTIONS")) {
        $self->sktrace_setup(0);
    }

    # Delete LIFs
    foreach my $key (keys(%{$self->{filer_setup}->{lifs}})) {
        my $lif = $self->{filer_setup}->{lifs}->{$key};

        my ($lifObj, $result) = try {
            NACL::C::Interface->delete(lif               => $lif->{name},
                                       command_interface => $node,
                                       server => $lif->{server}->{name},);

        }
        catch NATE::BaseException with {
            my $ex = shift;
            die config_result("EXCEPTION: " . $ex->text());
        };
    } ## end foreach my $key (keys(%{$self...}))

    # Delete the ldap client if it was created.
    if ($Options_hash->{NSSWITCH} =~ m/ldap/) {
        my $ldapclient = ("ldapclient");
        try {
            NACL::C::VserverServicesLdapClient->delete(
                                                 command_interface => $node,
                                                 "client-config" => $ldapclient,
                                                 );
        }
        catch NATE::BaseExcepton with {
            my $ex = shift;
            die config_result("EXCEPTION: ", $ex->text());
        };
    } ## end if ($Options_hash->{NSSWITCH...})

    my @cacheVols  = $self->get_volumes_cache();
    my @usableVols = $self->get_volumes_usable();
    my @rootVols   = $self->get_volumes_root();
    my @delOrder   = ();
    push(@delOrder, @cacheVols);
    push(@delOrder, @usableVols);
    push(@delOrder, @rootVols);

    foreach my $volume (@delOrder) {

        # delete the QOS workload first, if it exists. Don't bother with status checks the delete
        # will fail if this doesn't work.
        if ($volume->{qos_workload}) {
            NACL::C::QosWorkload->delete(command_interface => $node,
                                         volume            => $volume->{name},
                                         vserver  => $volume->{vserver}->{name},
                                         workload => $volume->{qos_workload},);
        }

        # unmount the volume. In RR.0+ you cannot unmount the root volume,
        # just continue if you get the error.
        try {
            NACL::C::Volume->unmount(volume  => $volume->{name},
                                     vserver => $volume->{vserver}->{name},
                                     command_interface => $node,);
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->comment("EXCEPTION: " . $ex->text());
        };

        # offline, don't fail, see if we can delete the volume first.
        try {
            NACL::C::Volume->offline(volume  => $volume->{name},
                                     vserver => $volume->{vserver}->{name},
                                     command_interface => $node,);
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->comment("EXCEPTION: " . $ex->text());
        };

        try {
            NACL::C::Volume->delete(volume  => $volume->{name},
                                    vserver => $volume->{vserver}->{name},
                                    command_interface => $node,);

        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->comment("EXCEPTION: " . $ex->text());
        };
    } ## end foreach my $volume (@delOrder)

    # Delete vservers
    foreach my $key (keys(%{$self->{filer_setup}->{vservers}})) {
        my $vserver = $self->{filer_setup}->{vservers}->{$key};
        try {
            NACL::C::Route->delete(server            => $vserver->{name},
                                   "routing-group"   => '*',
                                   command_interface => $node,
                                   destination       => '*',);

        }
        catch NATE::BaseException with {
            my $ex = shift;
            die config_result("EXCEPTION: " . $ex->text());
        };

        # The routing groups must be eliminated first. For now, we just blow away all
        # routing groups on this vserver.
        try {
            NACL::C::RoutingGroups->delete(server          => $vserver->{name},
                                           "routing-group" => '*',
                                           command_interface => $node);

        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->comment("EXCEPTION: " . $ex->text());

            # Don't return fail for certain exceptions
            if (grep(/There are no entries matching your query/i, $ex->text()))
            {
                $Log->comment(
                    "The preceeding exception was determined to be non-fatal.");
            } else {
                die config_result("EXCEPTION: " . $ex->text());
            }
        };

        # delete the QOS workload if it exists.
        if ($vserver->{qos_workload}) {
            try {
                NACL::C::QosWorkload->delete(
                                           command_interface => $node,
                                           vserver  => $vserver->{name},
                                           workload => $vserver->{qos_workload},
                                           );
            }
            catch NATE::BaseException with {
                my $ex = shift;
                die config_result("EXCEPTION: " . $ex->text());
            };
        } ## end if ($vserver->{qos_workload...})

        try {
            NACL::C::Vserver->delete(vserver           => $vserver->{name},
                                     command_interface => $node,);

        }
        catch NATE::BaseException with {
            my $ex = shift;
            die config_result("EXCEPTION: " . $ex->text());
        };

    } ## end foreach my $key (keys(%{$self...}))

    # Execute the one from the parent
    $self->SUPER::filer_cleanup(%options);
} ## end sub filer_cleanup

sub client_setup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_setup(%options);

    $Log->exit();
}

sub DESTROY {
    my $self = shift;
    local ($@);

    $self->client_cleanup();
}

sub client_cleanup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_cleanup(%options);

    $Log->exit();
}

sub _update_test_topology {
    my $self = shift;
    $Log->enter();

    $Log->exit();
}

######################### Internal Routines ##################

my $File_read = undef;

sub _read_testuser_file {
    my $tfile = shift;
    my $file =
      NATE::Inc::find_file($tfile) ? NATE::Inc::find_file($tfile) : $tfile;
    $Log->enter("file=>$file");
    if ($File_read) {
        $Log->trace("file already processed");
        goto done;
    }
    my $testuser = $Options->get(option => "TESTUSER");

    if (open(IF, $file)) {
        while (<IF>) {
            next if (/^\s*\#/);
            chomp;
            push(@{$testuser}, $_);
        }
        close(IF);
    } else {
        die "Could not open TESTUSER_FILE: $tfile ($file) : $! / $@";
    }

    $File_read = 1;

  done:
    $Log->exit();
} ## end sub _read_testuser_file

sub _create_instance_dirs {
    my ($self) = @_;

    my $remote           = new RemoteExecution;
    my $dirSetupScript   = "tools/TestManager/filer_directory_setup.thpl";
    my $numSubs          = 0;
    my @clients          = $self->clients();
    my $createDirTimeout = 600;

    # First thing to do is arrange all the data from the topology hash
    # into a smaller subset of what we need. Build a hash where the volumes/exports
    # are the keys and an array of the all the dirs to create on said volume. This is
    # determined by the the DEST_PATHS value on each instance
    my $exportHashRef = {};
    foreach my $instanceKey (keys(%{$self->{instances}})) {
        my $instance = $self->{instances}->{$instanceKey};

        foreach my $volPath (@{$instance->{DEST_PATHS}}) {

            # 172.17.92.203:/p_002_jd_test_max_export_policies/ssqa-2224-e0824
            if ($volPath =~ m!^(.+):/(.+)/(.+)$!) {
                my $ipAddress = $1;
                my $export    = $2;
                my $dir       = $3;

                $exportHashRef->{$export}->{filerip} = $ipAddress;

                # An example of the topology volume hash with the data embedded that is needed
                # to influence directory creator.thpl above to create the files we need
                # 'p_001_vstorage_intra_volume_copy' => {
                #      isRootVolume' => 0,
                #      'vserver' => $VAR1->{'filer_setup'}{'vservers'}{'vstorage_intra_volume_copy_vs_1'},
                #      'FILE_CREATE' => {
                #           'FILE_NAME' => 'vstorage_file_7629.1',
                #           'FILE_CREATE_IN_EXPORT' => 1,
                #           'FILE_FILL' => 1,
                #           'FILE_NUM' => 16,
                #           'FILE_SIZE' => 524288000
                #        },
                #       'name' => 'p_001_vstorage_intra_volume_copy',

                # If the directory should have some files pre-created,
                # make note of those here also. We keep all the parameters destined
                # for the dir creation thpl in a hash ref on the volume object. We need
                # to transfer those into the exportHashRef that will ultimately get sent
                # to each client
                #          my $volObj = $self->{filer_setup}->{volumes}->{$export};
                #          if ($volObj->{FILE_CREATE}) {
                #              $exportHashRef->{$export}->{FILE_CREATE} = $volObj->{FILE_CREATE};

                # if we are going to be creating files, we need to bump the timeout
                #              if (defined($exportHashRef->{$export}->{FILE_CREATE}->{FILE_SIZE})) {
                #                  $createDirTimeout = .5 * $exportHashRef->{$export}->{FILE_CREATE}->{FILE_SIZE};

                #              } else {
                # just guess
                #                 $createDirTimeout = 3600;
                #              }
                #          }

                # Keep the directories in an array.
                push(@{$exportHashRef->{$export}->{dirs}}, $dir);

            } else {
                die config_result(
                    "The volpath for the instance called $instance->{name} was not formatted as expected."
                      . " volpath: $volPath");
            }
        } ## end foreach my $volPath (@{$instance...})
    } ## end foreach my $instanceKey (keys...)

    # Not being very smart about this right now. One client gets one export
    # and creates the required directories...we'll see...
    my $clientIndex   = 0;
    my $numSubsActive = 0;
    my @exportKeys    = keys(%{$exportHashRef});
    foreach my $exportKey (@exportKeys) {
        my $export = $exportHashRef->{$exportKey};
        my $client = $clients[$clientIndex];
        my $mntpt =
          '/t/' . $export->{filerip} . '_' . $exportKey . '_filer_setup';
        my @dirs = join(",", @{$export->{dirs}});

        my $params = {
               NFS_VERSION => 3,
               MOUNT_LIST => "$export->{filerip};/${exportKey};${mntpt};rw;777",
               DIRS       => @dirs,};

        # if there is file information here, add it to the params
        if ($export->{FILE_CREATE}) {
            @{$params}{keys %{$export->{FILE_CREATE}}} =
              values %{$export->{FILE_CREATE}};
        }

        $Log->comment(
            "Run tools/TestManager/filer_directory_setup.thpl (timeout: $createDirTimeout secs) on "
              . "$exportKey from $client to create dirs:"
              . join(",", @dirs));

        my $subTest = $remote->execute(
                 host        => $client,
                 nate_params => $params,
                 cmd         => $dirSetupScript,
                 runid =>
                   sprintf("%03d_%s_%s", $numSubs, "filer_setup_dirs", $client),
                 handler => sub { });

        if (not $subTest) {
            die config_result("Remote Execute on $client failed: $@");
        }
        $clientIndex++;
        $numSubs++;
        $numSubsActive++;

        # Keep the concurrent number of subtests to 100 so as not to
        # overload the filers
        if (   ($numSubsActive >= 100)
            or ($exportHashRef->{$exportKeys[-1]} eq $export)) {
            my $fail_result = $remote->wait(timeout => $createDirTimeout);
            die config_result("Directory creation failed: $fail_result")
              if ($fail_result);
            $numSubsActive = 0;
        }

        # Don't run out of clients. i.e. if there are more volumes
        # than clients account for that. For now, just loop through
        # the clients, and continue to subtest out to them. This could
        # create problems if, for instance, one client is specified and
        # 500 exports with one directory are also specified.
        $clientIndex = 0 if ($clientIndex > (scalar(@clients) - 1));
    } ## end foreach my $exportKey (@exportKeys)
    return 1;
} ## end sub _create_instance_dirs

sub _assign_path_to_instance {
    my ($self) = @_;

    my $allclis2vols    = $Options->get(option => "ALLCLIENTS_2_ALLVOLS");
    my $volsPerInstance = $Options->get(option => "VOLUMES_PER_INSTANCE");
    my @usableVolumes = $self->get_volumes_usable();
    my $string        = "Assignment Mode: ";
    $string .= "Allclients2AllVols" if ($allclis2vols);
    $string .= "VolsPerInstance: $volsPerInstance" if ($volsPerInstance);
    $Log->enter($string);
    if (not $volsPerInstance) {
        die config_result(
            "The value of VOLUMES_PER_INSTANCE is zero. What did you think would happen?"
            );
    }

    # Now we need to go through all the instances, and figure out which
    # volumes the instances will use
    my $volIndex = 0;
    foreach my $instanceKey (keys(%{$self->{instances}})) {
        my $instance = $self->{instances}->{$instanceKey};

        # All clients hit all volumes
        if ($allclis2vols) {
            foreach my $volObj (@usableVolumes) {
                push(@{$instance->{target_volumes}}, $volObj->{name});
            }

        } else {

            # Each client will get a predetermined number of volumes
            foreach my $num (1 .. $volsPerInstance) {
                push(@{$instance->{target_volumes}},
                     $usableVolumes[$volIndex]->{name});

                # Reset the volumes array after all the volumes are used
                $volIndex = 0 if (++$volIndex > (scalar(@usableVolumes) - 1));
            }
        } ## end else [ if ($allclis2vols) ]
    } ## end foreach my $instanceKey (keys...)

    # Go through all the usable volumes get the lifs, and the instances that will use this volume
    foreach my $usableVol (@usableVolumes) {
        my @lifs = keys(%{$usableVol->{vserver}->{lifs}});

        # Check to see if we should allow IO to the lifs on the same node
        # where the volume resides.
        if ($Options->get(option => "VOLUME_IO_ALL_REMOTE")) {
            my @lifsMod;
            my $aggrs =
              Topology::Lookup::aggregate(
                                         topo         => $discovered_topology,
                                         filter_key   => "aggregate",
                                         filter_value => $usableVol->{aggregate}
                                         );

            my $aggr = $aggrs->[0];
            die config_result(
                "unable to find an existing aggregate name $usableVol->{aggregate} in the given cluster"
              )
              if (not $aggr);

            # fix the lifs array to our liking
            foreach my $lifKey (@lifs) {
                my $lif = $self->{filer_setup}->{lifs}->{$lifKey};
                next if (grep(/^$lif->{"home-node"}$/i, @{$aggr->{nodes}}));

                # we can use this
                push(@lifsMod, $lifKey);
            }
            die config_result(
                "Unable to find a lif that offers cross-cluster access to the volume $usableVol->{name}"
              )
              if (not @lifsMod);

            @lifs = @lifsMod;
        } ## end if ($Options->get(option...))

        # Go through the instances that are using the current volume and assign target IP address round robin
        my $lifIndex = 0;
        foreach my $instance (
                $self->_get_instances_by_volume(volume => $usableVol->{name})) {
            my $client   = $instance->{client};
            my $exitLoop = 1;                    # default to one lif per volume
            $exitLoop = 0 if ($Options->get(option => "VOLUME_IO_ALL_LIFS"));

            while ($lifIndex <= (scalar(@lifs) - 1)) {
                my $volPath =
                    $self->{filer_setup}->{lifs}->{$lifs[$lifIndex]}->{address}
                  . ':/'
                  . $usableVol->{name} . '/'
                  . $client->{name} . '_'
                  . $instance->{name};

                push(@{$instance->{DEST_PATHS}}, $volPath);

                # Let us also build the test_options hash. The clients are the keys and
                # and the instances are the values. The instance is also a key to a lower hash
                # that which contains DEST_PATHS. The SRCIP is only valid when synthetic
                # clients are used...
                @{$self->{test_options}->{$client->{name}}->{$instance->{name}}
                      ->{DEST_PATHS}} = @{$instance->{DEST_PATHS}};

                # Duck out early if there should only be one lif for each dest volume
                last if ($exitLoop);
                $lifIndex++;
            } ## end while ($lifIndex <= (scalar...))

            # Round robin the lifs on the vserver where the target volume lives
            $lifIndex = 0 if (++$lifIndex > (scalar(@lifs) - 1));

            $self->{test_options}->{$client->{name}}->{$instance->{name}}
              ->{TESTUSER} = $instance->{userName};

            $Log->debug(
                "Configured client $client->{name} with instance $instance->{name}, and  DEST_PATHS: "
                  . Dumper(@{$instance->{DEST_PATHS}}));
        } ## end foreach my $instance ($self...)
    } ## end foreach my $usableVol (@usableVolumes)

    $Log->exit();
} ## end sub _assign_path_to_instance

sub _configure_instances {
    $Log->enter();
    my ($self) = @_;

    # Create the client_setup hash its just a very small static hash of a few options that the supervisor
    # will use. client_config contains the relational setup of the clients <-> instances and everything else.
    # I wanted to use the client_setup area for this, but the way that the topology.pm interacts with that hash
    # makes that too risky.
    my @clients            = $self->clients();
    my $instancesPerClient = $Options->get(option => "INSTANCES_PER_CLIENT");
    my $max_nfs_clients    = $Options->get(option => "MAX_NFS_CLIENTS");
    my $testUsers          = $Options->get(option => "TESTUSER");
    my $numberOfInstances  = 0;

    # If MAX_NFS_CLIENTS, then use that number as the number clients to create.
    if ($max_nfs_clients) {
        $Log->comment(
             "Received a MAX_NFS_CLIENTS value of $max_nfs_clients. Using this "
               . "value as the number of NFS clients to create.");
        $numberOfInstances = $max_nfs_clients;
    }

    # Only look at the IP addresses if this is being called as part
    # of the virtual topology. Use the number of supplied addresses
    # as the default number of synthetic clients to create.
    my $numAddrs;
    my ($package, $filename, $line) = caller;
    if ($package =~ /virtual/i) {
        my @srcNet = $self->{ipManager}->source_networks();

        # We may not have SRCIP addresses. In such a case we will be using the local IP address
        # of a given physical client
        if (@srcNet) {
            $numAddrs = $srcNet[0]->unassigned_addresses();

            if ($numAddrs < $numberOfInstances) {

                # Freak out if there are not enough IP addresses to reach MAX_NFS_CLIENTS
                die config_result(
                    "There were $numAddrs supplied addresss. The value of MAX_NFS_CLIENTS "
                      . "indicates the need for $numberOfInstances clients. You should rethink this.",
                    stacktrace => 0);

                # If not MAX_NFS_CLIENTS use the number of addresses supplied.
            } elsif (not $numberOfInstances) {
                $Log->comment(
                    "The number of instances will be equal to the supplied number of IP addresses ($numAddrs)."
                    );
                $numberOfInstances = $numAddrs;
            }
        } ## end if (@srcNet)
    } ## end if ($package =~ /virtual/i)

    # If there is no value of MAX_NFS_CLIENTS, then just do one instance per client.
    # this means we are not creating synthetic clients.
    if (not $numberOfInstances) {
        $Log->comment("Setting the number of NFS instances to one per client ("
                      . scalar(@clients)
                      . ").");

        $numberOfInstances = scalar(@clients);
    }

    # If INSTANCES_PER_CLIENT is specified, then just make sure we didn't exceed it.
    if (    ($instancesPerClient)
        and ($instancesPerClient < ($numberOfInstances / scalar(@clients)))) {
        die config_result(
            "The requested number of instances, ($numberOfInstances total, or "
              . ($numberOfInstances / scalar(@clients))
              . " per client) will exceed the "
              . "maximum number of instances per client (There are "
              . scalar(@clients)
              . " clients), "
              . "which is defined as ("
              . $instancesPerClient
              . ").\nThis can be overridden by "
              . "changing the value of INSTANCES_PER_CLIENT on the command line. Requested "
              . $numberOfInstances / scalar(@clients)
              . " instances for each one of the "
              . scalar(@clients)
              . " clients.");
    } ## end if (($instancesPerClient...))
    if ($numberOfInstances == 0) {
        die config_result(
            "I was asked to create zero instances per client. Review configuration"
            );
    }
    my $inst = int($numberOfInstances / scalar(@clients));
    if (not $inst) {
        $inst = 1;
    }

    $self->{_num_instances} = $numberOfInstances;
    $Log->comment(  "Creating $numberOfInstances total instances, resulting in "
                  . "roughly $inst instances per client.(+/-1)");

    # Spread all the instances among the clients.
    # Spread test users across test instances if given
    my $clientIndex = 0;
    my $userIndex   = 0;
    foreach my $instanceNumber (1 .. $numberOfInstances) {
        my $instanceHash = {
                     client => $self->{client_config}->{$clients[$clientIndex]},
                     isSynthetic => 0,
                     name        => $instanceNumber};

        # Add the user ID's if we are going to run processes as different users
        # Cycle through the available user IDs
        if ($testUsers->[0]) {
            $instanceHash->{userName} = $testUsers->[$userIndex]
              if ($testUsers->[$userIndex]);
            $userIndex = 0 if (++$userIndex > scalar(@{$testUsers}));
        }

        # Keep all instances in a single place
        $self->{instances}->{$instanceNumber} = $instanceHash;

        # Link each instance to its physical client
        $self->{client_config}->{$clients[$clientIndex]}->{instances}
          ->{$instanceNumber} = $self->{instances}->{$instanceNumber};

        # Loop through the clients
        $clientIndex = 0 if (++$clientIndex > (scalar(@clients) - 1));

    } ## end foreach my $instanceNumber ...

    $Log->exit();
    return 1;
} ## end sub _configure_instances

#
# Modify the topology to include a list of users that will require
# quotas configured for their usage.
#
sub _define_volume_quotas {
    $Log->enter();
    my ($self) = @_;

    # make a list of all unique users accessing the volume
    foreach my $vol ($self->get_volumes_usable()) {
        my %hash;
        foreach
          my $user ($self->_get_testusers_by_volume(volume => $vol->{name})) {
            $hash{$user} = 1;
        }
        foreach my $key (keys %hash) {
            push(@{$vol->{quota_users}}, $key);
        }
    } ## end foreach my $vol ($self->get_volumes_usable...)

    $Log->exit();
} ## end sub _define_volume_quotas

sub _get_instances_by_volume {
    my ($self, %params) =
      validate_mixed(@_, [{type => OBJECT,}], {volume => {type => SCALAR,}});

    my @instancesUsingThisVolume = ();

    # Go through every instance, and find the ones using the current usable volume
    foreach my $instanceName (keys(%{$self->{instances}})) {
        my $instanceHash = $self->{instances}->{$instanceName};

        foreach my $volName (@{$instanceHash->{target_volumes}}) {
            if ($volName =~ /^$params{volume}$/i) {
                push(@instancesUsingThisVolume, $instanceHash);
            }
        }
    }
    return @instancesUsingThisVolume;
} ## end sub _get_instances_by_volume

sub _get_testusers_by_volume {
    $Log->enter();
    my ($self, %params) =
      validate_mixed(@_, [{type => OBJECT,}], {volume => {type => SCALAR,}});

    my @return;

    # Go through every instance, and find the ones using the current usable volume
    foreach my $instanceHash (
                   $self->_get_instances_by_volume(volume => $params{volume})) {
        if ($instanceHash->{userName}) {
            push(@return, $instanceHash->{userName});
        }
    }

    $Log->exit();
    return @return;
} ## end sub _get_testusers_by_volume

sub _set_usable_volume_size {
    $Log->enter();
    my ($self) = @_;

    $Options_hash = $Options->get_hash();

    foreach my $usableVol ($self->get_volumes_usable()) {
        my @instancesUsingThisVolume =
          $self->_get_instances_by_volume(volume => $usableVol->{name});

        # Set the size of the volume because we know now how many instances will access it. VOL_SIZE is considered per instance.
        my $size = $Options_hash->{VOL_SIZE};
        my $sizeInMb = $self->size_in_mb(size => $size);
        if (not $size) {
            die config_result(
                "The VOL_SIZE parameter was not defined. This is not acceptable."
                );
        } else {
            $Log->comment(
                "Received a volume size of VOL_SIZE = $size. This will be multiplied by the number of instances accessing each volume."
                );
            $Log->comment("Size in mb is $sizeInMb");
        }

        $usableVol->{size}              = $sizeInMb;
        $usableVol->{size_per_instance} = $usableVol->{size};
        if (scalar(@instancesUsingThisVolume) > 0) {
            $usableVol->{size} =
              (scalar(@instancesUsingThisVolume) * $usableVol->{size});
        }

        $Log->trace(
            "Setting volume size to $usableVol->{size} which is the number of instances ("
              . scalar(@instancesUsingThisVolume)
              . ") multiplied by the given VOL_SIZE ($size).");
    } ## end foreach my $usableVol ($self...)
    $Log->exit();
    return 1;
} ## end sub _set_usable_volume_size

sub _configure_filer_dns {
    $Log->enter();
    my ($self, %params) =
      validate_mixed(@_, [{type => HASHREF},], {node => {type => OBJECT},},);
    my $node = $params{node};

    # So, at the time of this writing, there is no CDEF/Component to handle VserverSystemServicesDNS
    # por lo tanto when you setup DNS on a RR filer, you have to use APISets.

    my @dnsServers = @{$Options->get(option => "DNS_SERVERS")};
    if (not scalar(@dnsServers) > 0) {
        die config_result(
                         "You are required to specify at least one DNS server");
    }

    $Log->trace("Using DNS Servers: " . Dumper(@dnsServers));

    # Arrange the DNS servers into a hash so that we can send the right
    # number of arguments to the NACL Sub.
    my $addrIndex    = 1;
    my $maxAddrIndex = 3;
    my %dns_servers;
    foreach my $addr (@dnsServers) {

        # NACL allows 3 servers to be specified.
        last if ($addrIndex == $maxAddrIndex);

        my $key = 'ns' . $addrIndex;
        $dns_servers{$key} = $addr;

        $addrIndex++;
    } ## end foreach my $addr (@dnsServers)

    $Log->trace("DNS Args: " . Dumper(%dns_servers));

    my $apiSet = $node->apiset(category  => "Node",
                               interface => "CLI",
                               set       => "CMode");

    my $version_mgr = $apiSet->get_version_manager();

    # Attribute can be any of branch; release; build_options; build_path;
    # change_number; wafl_version; node_version
    my $string = $version_mgr->get_version_attribute(attribute => "release");
    my $mappedRelease =
      $version_mgr->map_release_to_codename(release => $string);

    $Log->comment("Release version determined to be $mappedRelease");

    # There is a NACL component for BR releases
    if ($mappedRelease =~ /boiler/i) {

        require NACL::C::SystemServicesDns;

        try {
            NACL::C::SystemServicesDns->modify(
                                    %dns_servers,
                                    state   => 'enabled',
                                    domains => \@{$Options_hash->{DNS_DOMAINS}},
                                    node    => $node->hostrec()->id(),
                                    command_interface => $node);
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->warn("EXCEPTION: " . $ex->text());
        };

    } else {

        # This would be RR.x(or better). Its per vserver
        foreach my $key (keys(%{$self->{filer_setup}->{vservers}})) {
            my $vserver = $self->{filer_setup}->{vservers}->{$key};

            # HACK ALERT!
            # The APISets and Components are not ready for this yet.
            # for now, lets use say. I know...so 90's. sigh.
            my $conn_str = $node->hostrec()->id() . "#c-login";
            my $fc       = Tharn::connect($conn_str);

            my $command =
                "vserver services dns create -vserver "
              . $vserver->{name}
              . " -domains "
              . join(',', @{$Options_hash->{DNS_DOMAINS}})
              . " -state enabled -name-servers ";

            # add the DNS servers
            while (my (undef, $value) = each(%dns_servers)) {
                $command .= $value . ",";
            }

            $Log->trace("RR DNS configuration command: $command");

            Tharn::say($fc, $command);
        } ## end foreach my $key (keys(%{$self...}))

    } ## end else [ if ($mappedRelease =~ ...)]
} ## end sub _configure_filer_dns

sub _create_client_setup {
    my ($self) = @_;

    foreach my $client (@{$self->{clients}}) {
        $self->{client_setup}->{$client}->{ADDR_TYPE} =
          $Options_hash->{ADDR_TYPE};
        $self->{client_setup}->{$client}->{DEVICE} = $Options_hash->{DEVICE};

        # All the relational stuff is in client_config
        $self->{client_config}->{$client}->{name} = $client;
    }

    return 1;
} ## end sub _create_client_setup

#
# Just let the topology decide when this is a
# a good idea.
#
sub _start_ip_manager {
    $Log->enter();
    my ($self) = @_;

    my $ipManager;
    my %argsForNew;

    # Because CMode_Virtual is derived from CMode,
    # The potential to start the IPManager twice exists.
    # Don't do that.
    if ($self->{ipManager}) {
        $ipManager = $self->{ipManager};

    } else {
        my $mode = $Options->get(option => "P_EXE_MODE");
        $argsForNew{start_messenger} = 0;

        $Log->trace(
                "Creating a new IPManager. The TestManagerLite mode is $mode.");

        # The IPManager should only be communicating with
        # the IPManager server when it is in manager mode.
        # supervisors get their information via Staph.
        if ($mode =~ /mgr/i) {
            $argsForNew{start_messenger} = 1;
        }

        # Start the client IP manager software
        $ipManager = Utils::IPManager::start_ip_manager(%argsForNew);
    } ## end else [ if ($self->{ipManager})]

    $Log->exit();
    return $ipManager;
} ## end sub _start_ip_manager

sub _configure_filer_date_time {
    $Log->enter();
    my ($self, %params) =
      validate_mixed(@_, [{type => HASHREF},], {node => {type => OBJECT},},);

    my $node = $params{node};
    my $apiSet = $node->apiset(category  => "Node",
                               interface => "CLI",
                               set       => "CMode");

    my $version_mgr = $apiSet->get_version_manager();

    # Attribute can be any of branch; release; build_options; build_path;
    # change_number; wafl_version; node_version
    my $string = $version_mgr->get_version_attribute(attribute => "release");
    my $mappedRelease =
      $version_mgr->map_release_to_codename(release => $string);

    $Log->comment("Release version determined to be $mappedRelease");

    # This isn't working for BR yet. It doesn't like the timezone command
    return if ($mappedRelease =~ /boiler/i);

    my $nodeName = $node->hostrec()->id();

    my @timeServers = @{$Options->get(option => "TIME_SERVERS")};
    if (not @timeServers) {
        $Log->trace(
            "NTP will not be configured because there is no value specified for TIME_SERVERS"
            );
        return;
    }

    # system node date modify -node NODENAME -timezone TZONE
    my $timeZone = $Options->get(option => "TIME_ZONE");

    #    my $output_tz = $apiSet->timezone(timezone => $timeZone);

    # Let us see what is there already
    my $output = $apiSet->system_services_ntp_server_show();

    foreach my $tServer (@timeServers) {
        my $alreadyConfigured = 0;

        foreach my $line (@{$output->{parsed_output}}) {
            if (    ($line->{server} =~ /$tServer/i)
                and ($line->{node} =~ /$nodeName/i)) {
                $alreadyConfigured = 1;
                last;
            }
        }

        if (not $alreadyConfigured) {
            try {
                NACL::C::SystemServicesNtpServer->create(
                                                     command_interface => $node,
                                                     server => $tServer,
                                                     node   => $nodeName);

            }
            catch NATE::BaseException with {
                my $ex = shift;
                $Log->warn("EXCEPTION: " . $ex->text());
            };
            try {
                NACL::C::SystemServicesNtpConfig->modify(
                                                     command_interface => $node,
                                                     enabled => "true",);

            }
            catch NATE::BaseException with {
                my $ex = shift;
                $Log->warn("EXCEPTION: " . $ex->text());
            };
        } else {
            $Log->comment(
                "Node $nodeName is already configured to use time server $tServer."
                );
        }
    } ## end foreach my $tServer (@timeServers)

    return;
} ## end sub _configure_filer_date_time

sub get_export_rule_sets {
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {vserver => {type => UNDEF | SCALAR, optional => 1},});

    my $vserver = $params{vserver};

    my @exportsToReturn;
    foreach my $key (keys(%{$self->{filer_setup}->{ruleSets}})) {
        my $export = $self->{filer_setup}->{ruleSets}->{$key};

        if (defined($vserver) and ($export->{vserver}->{name} !~ /^$vserver$/))
        {
            next;

        } else {
            push(@exportsToReturn, $export);
        }
    } ## end foreach my $key (keys(%{$self...}))

    return @exportsToReturn;
} ## end sub get_export_rule_sets

sub volumes {
    my ($self) = @_;

    my @volumes;
    foreach my $key (keys %{$self->{filer_setup}->{volumes}}) {
        push(@volumes, $self->{filer_setup}->{volumes}->{$key});
    }

    return @volumes;
} ## end sub volumes

sub get_volumes_flexcache_origin {
    my ($self) = @_;

    my @return;
    foreach my $vol ($self->get_volumes_usable()) {
        if ($vol->{flexcache_origin}) {
            push(@return, $vol);
        }
    }

    return @return;
} ## end sub get_volumes_flexcache_origin

sub get_volumes_usable_by_aggr {
    my ($self, %params) =
      validate_mixed(@_, [{type => OBJECT}], {aggr => {type => SCALAR},});

    my @return;
    my $aggr = $params{aggr};
    foreach my $vol ($self->get_volumes_usable()) {
        if ($vol->{aggregate} =~ /^$aggr$/i) {
            push(@return, $vol);
        }
    }

    return @return;
} ## end sub get_volumes_usable_by_aggr

sub get_volumes_by_aggr {
    my ($self, %params) =
      validate_mixed(@_, [{type => OBJECT}], {aggr => {type => SCALAR},});
    my @return;
    my $aggr = $params{aggr};
    foreach my $vol ($self->volumes()) {
        if ($vol->{aggregate} =~ /^$aggr$/i) {
            push(@return, $vol);
        }
    }

    return @return;
} ## end sub get_volumes_by_aggr

sub get_volumes_usable {
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {vserver => {type => UNDEF | SCALAR, optional => 1},});

    my $vserver = $params{vserver};

    my @volumesUsable;
    my @volObjs = $self->volumes();
    foreach my $volObj (@volObjs) {
        my $vsObj = $volObj->{vserver};

        # The root volumes are not usable
        if ($volObj->{isRootVolume}) {
            next;

            # The cache volumes are not usable. Ontap will decide
            # when to serve data from a cache volume
        } elsif ($volObj->{"flexcache-origin-volume"}) {
            next;

            # If a vserver was specified, only return the volumes that belong to it.
        } elsif (defined($vserver) and ($vsObj->{name} !~ /^$vserver$/)) {
            next;

        } else {
            push(@volumesUsable, $volObj);
        }

    } ## end foreach my $volObj (@volObjs)

    return @volumesUsable;
} ## end sub get_volumes_usable

sub get_volumes_cache {
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {origin => {type => UNDEF | SCALAR, optional => 1},});

    my $originVol = $params{origin};

    my @volumesCache;
    foreach my $key (keys(%{$self->{filer_setup}->{volumes}})) {
        my $volObj = $self->{filer_setup}->{volumes}->{$key};

        # Return only cache volumes
        if (defined($volObj->{"flexcache-origin-volume"})) {

            # If a origin was specified, only return the volumes that belong to it.
            if (defined($originVol)
                and ($originVol !~ /^$volObj->{"flexcache-origin-volume"}$/)) {
                next;

            } else {
                push(@volumesCache, $volObj);
            }
        } ## end if (defined($volObj->{...}))
    } ## end foreach my $key (keys(%{$self...}))
    return @volumesCache;
} ## end sub get_volumes_cache

sub get_volumes_cache_by_aggr {
    my ($self, %params) =
      validate_mixed(@_, [{type => OBJECT}], {aggr => {type => SCALAR},});
    my @return;
    my $aggr = $params{aggr};
    foreach my $vol ($self->get_volumes_cache()) {
        if ($vol->{aggregate} =~ /^$aggr$/i) {
            push(@return, $vol);
        }
    }

    return @return;
} ## end sub get_volumes_cache_by_aggr

sub get_volumes_root {
    my ($self, %params) =
      validate_mixed(@_,
                     [{type => OBJECT}],
                     {vserver => {type => UNDEF | SCALAR, optional => 1},});

    my $vserver = $params{vserver};

    my @volumesRoot;
    foreach my $key (keys(%{$self->{filer_setup}->{volumes}})) {
        my $volObj = $self->{filer_setup}->{volumes}->{$key};
        my $vsObj  = $volObj->{vserver};

        # Return only the root volumes
        if ($volObj->{isRootVolume}) {

            # If a vserver was specified, only return the volumes that belong to it.
            if (defined($vserver) and ($vsObj->{name} !~ /^$vserver$/)) {
                next;

            } else {
                push(@volumesRoot, $volObj);
            }
        } ## end if ($volObj->{isRootVolume...})
    } ## end foreach my $key (keys(%{$self...}))
    return @volumesRoot;
} ## end sub get_volumes_root

sub _configure_volumes {
    my ($self) = @_;
    $Log->enter();

    my $node         = $nodes[0];
    my $local_limits = Limits->new(host => $node->hostrec()->id());
    my $filer        = @{$Options->get(option => "FILER")}[0];
    my $local_limits = Limits->new(host => $filer);
    my @volumes      = @{$Options_hash->{VOLUME}};
    my $numVservers  = $Options_hash->{NUM_VSERVERS};
    my @aggrs        = @{$Options->get(option => "aggregate")};
    die script_result("The value of aggregates is not defined")
      if (not(@aggrs));

    # we can use the option FLEXCACHE_AGGRS to decide which
    # aggregates will host flexcache volumes...this way we can direct
    # all regular volumes to one filer, and flexcache volumes to another
    # filer
    my @cache_aggrs;
    if (@{$Options_hash->{FLEXCACHE_AGGRS}}) {
        @cache_aggrs = @{$Options_hash->{FLEXCACHE_AGGRS}};
        $Log->comment(
            "FLEXCACHE_AGGRS is defined. Flexcache volumes will only be created on "
              . "aggregates as specified."
              . Dumper(\@cache_aggrs));
    } else {
        @cache_aggrs = @aggrs;
    }
    die script_result(
         "The aggregates which are to be used for cache volumes were undefined")
      if not(@cache_aggrs);

    my $numNodes       = scalar(@{$self->{nodes}});
    my $maxVolsPerNode = 500;
    my $maxVols        = $maxVolsPerNode * $numNodes;
    my $numVols;

    # What's in a name?
    my $volstr = $Options_hash->{VOLUME_NAME};
    $volstr = $self->testcase_string() if (not $volstr);

    # How many volumes?
    if (@volumes) {
        $numVols = scalar(@volumes);

    } else {
        $Log->debug("Volume was not given in cmdline");
        $numVols = $Options_hash->{NUM_VOLS};
    }

    # Firstly, Lets assume that the NUM_VOLS supplied by the user is the TOTAL volume count to exist on a node
    # This would include vserver root volumes. However, that may not be what the user was thinking.
    # It is possible that the user expected to create NUM_VOLS, in addition to the vserver root vols required.
    # Por lo tanto if the number of usable volumes, that is the volume count left after one volume has
    # been subtracted for each root volume, increase the usable volumes value to match that of the vserver.
    # If this would result in more than the maximum volume count, throw an error.

    $Log->comment("The user requested $numVols volume(s) be created.");

    # account for vserver root volumes
    my $numUsableVols = $numVols - $numVservers;

    $Log->comment(
        "Of these, $numVservers will be created as vserver root volume(s). This leaves $numUsableVols usable volume(s)."
        );

    # After root vols are created, will there be enough volumes for each vserver to get at least one.
    if (($numUsableVols < $numVservers) and ($numVols < $maxVols)) {
        $Log->warn(
            "The NUM_VOLS parameter is used to determine the total number of volumes that \n"
              . "will exist on the given filer(s), including vserver root volumes and data (usable) volumes. \n"
              . "The configuration requested ($numVols) total volumes, and ($numVservers) "
              . "total vservers.\nWith this parameter setup, there are not enough volumes for each vserver to \n"
              . "get at least one root volume and one data volume. Therefore the config will be modified to \n"
              . "create ("
              . ($numVservers * 2)
              . ") total volumes.");

        # Adjust the usable volumes so that one there is enough to assign one root and one data volume to each vserver
        $numVols       = ($numVservers * 2);
        $numUsableVols = $numVols - $numVservers;

    } ## end if (($numUsableVols < ...))

    # Handle flexcache origin volumes
    my $maxNumCacheVols = 0;
    my $numOriginVols   = 0;
    if (my $flexCachePercent = $Options->get(option => "FLEXCACHE_PERCENT")) {
        die config_result(
                   "The value of FLEXCACHE_PERCENT should be between 1 and 100")
          if ((not $flexCachePercent > 0) or (not $flexCachePercent < 100));

        $Log->comment(
            "Creating $flexCachePercent percent of the usable volumes ($numUsableVols) as flexcache volumes"
            );
        $Log->trace(  "Creating "
                    . scalar(@cache_aggrs)
                    . " cache vols for each cache origin volume.");

        # find the filer associated with each aggregate. We want to know how many cache vols will
        # end up on each filer.
        my %aggrHash;
        foreach my $aggrName (@cache_aggrs) {
            my $aggr =
              Topology::Lookup::aggregate(topo         => $discovered_topology,
                                          filter_key   => "aggregate",
                                          filter_value => $aggrName);
            die config_result(
                "I was unable to find the aggregate $aggrName on the specified filers."
              )
              if (not $aggr);

            foreach my $nodeName (@{$aggr->[0]->{nodes}}) {
                $aggrHash{$nodeName} = 1;
            }
        } ## end foreach my $aggrName (@cache_aggrs)

        my $numFilersGettingFlexCacheVols = scalar(keys(%aggrHash));
        $maxNumCacheVols =
          $local_limits->getvalue(limit => "max_flexcachevols");

        # find the flexcache target. That is, for example, user asked for 100 volumes, and 50 percent flexcache,
        # we will look to create as close to 50 flexcache volumes (not origin volumes) as possible.
        my $numCaches         = 0;
        my $totalUsableVolNum = $numUsableVols;
        while ( ($numUsableVols)
            and ($flexCachePercent > (($numCaches / $totalUsableVolNum) * 100)))
        {
            # We can only have 100 cache vols per node. Warn if we try to exceed it
            if (($numFilersGettingFlexCacheVols * $maxNumCacheVols) <
                $numCaches) {
                $Log->warn("Trying to achieve "
                    . $flexCachePercent
                    . "% of $numUsableVols usable volumes. "
                    . "Currently at "
                    . (($numCaches / $totalUsableVolNum) * 100) . "% of "
                    . "usable volumes, however we have reached the limit of allowed "
                    . "flexcache volumes. The test will continue with the max number of "
                    . "flexcache volumes $maxNumCacheVols.");
                last;
            } ## end if (($numFilersGettingFlexCacheVols...))

            $numOriginVols++;
            $numCaches = ($numOriginVols * scalar(@cache_aggrs));
        } ## end while (($numUsableVols) and...)

        # We lose a "usable" volume for every cache volume we create
        $numUsableVols = $numUsableVols - $numCaches;

        # Make sure we were able to meet the target with enough left over
        $Log->warn(
            "There are not enough vols ($numUsableVols) to give every aggregate ("
              . scalar(@aggrs)
              . ") at least one data (origin) volume "
              . "after dividing the the usable volumes ($numUsableVols) into $numOriginVols origin vols, and $numCaches cache volumes. "
              . "It is likely that setting the value of FLEXCACHE_PERCENT lower, or the NUM_VOLS higher will alleviate this condition."
          )
          if ($numUsableVols < scalar(@aggrs));

        $Log->comment(
            "Arrived at $numOriginVols flexcache origin volumes, and $numCaches total cache volumes, "
              . "leaving $numUsableVols usable data volumes.");
    } ## end if (my $flexCachePercent...)

    # If volumes were not supplied on the command line then make some up.
    if (not @volumes) {
        foreach my $volNum (1 .. $numUsableVols) {
            push @volumes, sprintf("p_%03d_%s", $volNum, $volstr);
        }
    }

    ####################################
    # Configure the data (usable) volumes in the topology hash
    ####################################
    my $aggrIndex = 0;
    foreach my $volName (@volumes) {

        # Make a hash of relevant volume information
        my $volHash = {
            name            => $volName,
            aggregate       => $aggrs[$aggrIndex],
            vserver         => "",
            isRootVolume    => 0,
            "junction-path" => "/" . $volName,

            # Size is increased as we determine how many instances will require access to this volume
            # The vserver will be setup later when the volumes are all assigned to vservers
            size => "0"};

        # If the flex cache option was supplied, tag the volume so that
        # flexcache can be enabled in filer_setup
        if ($numOriginVols) {
            $volHash->{flexcache_origin} = 1;
            $numOriginVols--;
            my $cacheNum = 0;

            # Create and add the hash values for the flexcache volumes
            foreach my $cacheAggr (@cache_aggrs) {
                my $cacheVolHash = {"flexcache-origin-volume" => $volName,
                                    type                      => "DC",
                                    aggregate                 => $cacheAggr,
                                    vserver => $volHash->{vserver},
                                    name    => $volName . "_cache_" . $cacheNum
                                    };

                $cacheNum++;
                $self->{filer_setup}->{volumes}->{$cacheVolHash->{name}} =
                  $cacheVolHash;
            } ## end foreach my $cacheAggr (@cache_aggrs)
        } ## end if ($numOriginVols)
        if (not $volHash->{aggregate}) {
            die config_result(
                       "Unable to determine the aggregate for volume $volName");
        }

        # Save the volume hash in its own location
        $self->{filer_setup}->{volumes}->{$volHash->{name}} = $volHash;

        $aggrIndex = 0 if (++$aggrIndex > (scalar(@aggrs) - 1));
    } ## end foreach my $volName (@volumes)

    # check some limits
    foreach my $node (@nodes) {
        my $nodeName = $node->hostrec()->id();
        my @aggrNames;

        $Log->comment("Check volume limits for node $nodeName");

        my $disc_aggrs =
          Topology::Lookup::aggregate(topo => $discovered_topology);
        foreach my $aggr (@{$disc_aggrs}) {
            if (grep(/$nodeName/, @{$aggr->{nodes}})) {
                push(@aggrNames, $aggr->{aggregate});
            }
        }
        if (@aggrNames) {
            $Log->comment(  "Found "
                          . scalar(@aggrNames)
                          . " aggregates belonging to $nodeName");
            $Log->debug("Aggregates: " . Dumper(\@aggrNames));
        } else {

            # If there are no aggrs on this filer, there are no volumes. And we are done.
            $Log->comment(
                 "Found no aggregates on this filer, moving to the next filer");
            next;
        }

        # find all the volumes on all the aggregates on this filer
        my $numVols      = 0;
        my $numCacheVols = 0;
        foreach my $aggr (@aggrNames) {
            $numVols += scalar($self->get_volumes_by_aggr(aggr => $aggr));
            $numCacheVols +=
              scalar($self->get_volumes_cache_by_aggr(aggr => $aggr));
        }

        # my $maxNumFlexVols = $local_limits->getvalue(limit => "max_flexvols");
        my $maxNumFlexVols = 1000;
        die config_result(
                    "The number of requested volumes for this node is $numVols."
                      . "This exceeds the limit of $maxNumFlexVols volumes "
                      . "for this node.")
          if ($numVols > $maxNumFlexVols);

        die config_result(
                  "The number of requested flexcache volumes of $numCacheVols, "
                    . "exceeds the limit of allowed flexcache volumes for this "
                    . "node $maxNumCacheVols.")
          if ($numCacheVols > $maxNumCacheVols);

        $Log->comment(
                   "This filer configuration has a total of $numVols volumes.");
        $Log->comment("Of these, $numCacheVols are flexcache volumes.")
          if ($numCacheVols);
    } ## end foreach my $node (@nodes)

    $Log->debug("Usable volumes: " . Dumper($self->get_volumes_usable()));
    $Log->debug("All volumes: " . Dumper($self->{filer_setup}->{volumes}));
    $Log->exit();
    return;
} ## end sub _configure_volumes

sub sktrace_setup {
    my ($self, $sktrace_toggle) = @_;
    my @ops = @{$Options->get(option => "SKTRACE_OPTIONS")};
    my $enable = (scalar(@ops)) ? 1 : 0;

    $Log->comment("SKTRACE_OPTIONS: " . Dumper(\@ops));
    $Log->comment("sktrace toggle: $sktrace_toggle");

    # set this on all nodes in use
    foreach my $node (@nodes) {
        my $bsd = NACL::APISet->new(category  => 'Node',
                                    interface => 'CLI',
                                    set       => 'Systemshell',
                                    hostobj   => $node->hostrec());

        # the CDEFS for sysctl are less than awesome. Use the
        # execute_raw_command instead. We just always run cleanup to make sure
        # we are only getting the options that we want, just in case cleanup fails or
        # is skipped
        if ($sktrace_toggle == 1) {
            $Log->comment(
                "Cleaning up the sktrace options on " . $node->hostrec()->id());

            # This is cleanup, start with a dump to finish everything off

            # This little gem resets all your sktrace counters to zero. Don't reset the Nfs4Info counters. They are on by default,
            # leave them that way.
            my $coolCommand =
              'sudo sysctl sysvar.sktrace | grep _enable: | grep -vi Nfs4Info | grep -v \': 0\' | sed \'s/:.*//\' | xargs -I SYSCTL sudo sysctl SYSCTL=0';
            $bsd->execute_raw_command(command => $coolCommand);
        } ## end if ($sktrace_toggle ==...)

        if ($enable and ($sktrace_toggle == 1)) {
            $Log->comment(
                 "Setting up the sktrace options on " . $node->hostrec()->id());

            # We are enabling. Start with a clear
            $bsd->execute_raw_command(
                               command => "sudo sysctl sysvar.sktrace.clear=1");
            $bsd->execute_raw_command(
                                command => "sudo sysctl sysvar.sktrace.mlog=1");

            foreach my $op (@ops) {
                $bsd->execute_raw_command(command => "sudo sysctl " . $op);
            }
        } ## end if ($enable and ($sktrace_toggle...))

        # Stop all sktraces if enabled
        if ($enable and ($sktrace_toggle == 0)) {
            $bsd->execute_raw_command(
                                command => "sudo sysctl sysvar.sktrace.mlog=0");
        }
    } ## end foreach my $node (@nodes)
} ## end sub sktrace_setup

sub get_nacl_nodes {
    my ($self) = @_;

    return \@nodes;
}

1;
