#Hey emacs, this is basically -*- perl -*-
# $Id: $

## @summary     NFS Stress Scalability CMode NFS Security enabled Topology Library
##
## @author      dascenj@netapp.com
##
## @pod here

=head1 NAME

Topology::NAS::NFS::CMode_Sec - NFS Stress Scalability NFS Security test Topology Library for CMode ONTAP

=head1 SYNOPSIS

    use qw(Topology);
    my $topology = new Topology(topology=>"Topology::NAS::NFS::CMode_Sec");

=head1 DESCRIPTION

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/TestManagerLite

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/Topology

=head1 TODO

=head1 SEE ALSO

Options TestManagerLite Topology

=head1 BUGS

Some probably exist but have not been listed yet.

=head1 STATUS

  Shared

=head1 AUTHOR

    Jd dascenj@netapp.com
    
=head1 COPYRIGHT

Copyright 201, Network Appliance., All Rights Reserved

=cut

package Topology::NAS::NFS::CMode_Sec;
use base qw(Topology::NAS::NFS::CMode);

use strict;
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use Utils qw(make_conn);
use Krb::Admin;
use Tharn qw($Log config_result);

use Data::Dumper qw(Dumper);

use Options;
my $Options;
my $Options_hash;

use RemoteExecution;
my $Remote = new RemoteExecution;

###################################################################

$Options = new Options;
$Options->parameters(
    definitions => [
        {name => "NFS_SECURITY",
         spec => "s",
         description =>
           "The security type used by the clients to mount NFS filesystem",
         default      => "krb5",
         valid_values => ["krb5", "krb5i", "krb5p", "sys"],},
        {name => "KRB_DOMAIN",
         spec => "s",
         description =>
           "Kerberos Domain Name. Read from tharnhost file if not supplied.",
         default => undef,},
        {name => "KRB_REALM",
         spec => "s",
         description =>
           "Kerberos Realm Name. Read from tharnhost file if not supplied.",
         default => undef,},
        {name        => "KDC",
         spec        => "s",
         description => "Kerberos KDC Server. Required when NFS_SECURITY=*krb*",
         default     => undef,},
        {name => "TESTUSER",
         spec => "s@",
         description =>
           "Kerberos needs at least one user name. Take the current user as the default",
         default => undef,},]);

sub new {
    my $invocant = shift;
    my $class    = ref($invocant) || $invocant;
    my $self     = {@_,};
    $Log->enter();
    bless($self, $class);

    $Log->exit();
    return $self;
}

# The users list that will be used by this test run.
# If more users are given than the number of test instances (each instance can use unique user
# at the maximum) given, then configure the only users that will be used during filer_setup.
my $used_users;

sub define_topology {
    my $self = shift;
    $Log->enter();

    $Options_hash = $Options->get_hash();
    if (!@{$Options_hash->{TESTUSER}}) {
        $Log->debug(
            "This topology requires TESTUSER to be set. Default to the current user, as no TESTUSER is given."
            );
        $Options->set(TESTUSER => ["$ENV{LOGNAME}"]);
    }
    $Options_hash = $Options->get_hash();

    # Execute the one from the parent
    $self->SUPER::define_topology(@_);

    # Make sure Krb information is sufficient if NFS_SECURITY is set to use Krb
    if ($Options_hash->{NFS_SECURITY} =~ /krb/) {
        die config_result("Please specify KDC") if (!$Options_hash->{KDC});
    }

    my %args;
    $args{domain} = $Options_hash->{KRB_DOMAIN}
      if ($Options_hash->{KRB_DOMAIN});
    $args{realm} = $Options_hash->{KRB_REALM} if ($Options_hash->{KRB_REALM});
    my $kadmin = new Krb::Admin(kdc => $Options_hash->{KDC}, %args);

    #
    # Set MOUNT_OPTIONS for all client instances
    #

    foreach my $client ($self->clients()) {
        my $testusers = [];
        foreach my $inst (keys %{$self->{test_options}{$client}}) {
            $self->{test_options}{$client}{$inst}{MOUNT_OPTIONS} =
              "sec=" . $Options_hash->{NFS_SECURITY};
            if ($self->{test_options}{$client}{$inst}{TESTUSER}) {
                push @$testusers,
                  $self->{test_options}{$client}{$inst}{TESTUSER};
            }
        }
        $self->{client_setup}{$client}{KDC}        = $kadmin->{kdc};
        $self->{client_setup}{$client}{KRB_REALM}  = $kadmin->{realm};
        $self->{client_setup}{$client}{KRB_DOMAIN} = $kadmin->{domain};
        $self->{client_setup}{$client}{CLIENTNAME} = $client;
        $self->{client_setup}{$client}{TESTUSER} = $testusers if (@$testusers);

        push @$used_users, @$testusers if (@$testusers);
    }

    # Keep only the unique users in this list.
    @$used_users = keys %{{map { $_ => 1 } @$used_users}};

    $Log->exit(Dumper($self));
}

sub filer_setup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::filer_setup(%options);

    my $filer = ($self->filers())[0];

    # If NFS security is set to Kerberos, then setup Kerberos
    if ($Options_hash->{NFS_SECURITY} =~ /krb/) {
        my %args;
        $args{domain} = $Options_hash->{KRB_DOMAIN}
          if ($Options_hash->{KRB_DOMAIN});
        $args{realm} = $Options_hash->{KRB_REALM}
          if ($Options_hash->{KRB_REALM});
        my $kadmin = new Krb::Admin(kdc => $Options_hash->{KDC}, %args);

        # $kadmin->setup_kdc(); This is not supported for now, hence setup manually

        foreach my $key (keys(%{$self->{filer_setup}->{vservers}})) {
            my $vserver = $self->{filer_setup}->{vservers}->{$key};
            $kadmin->setup_filer(filer => $filer, vserver => $vserver->{name});
            $kadmin->add_users(users => $used_users);
        }
    }

    $Log->exit();
}

sub client_setup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_setup(%options);

    my %args;
    $args{domain} = $options{KRB_DOMAIN} if ($options{KRB_DOMAIN});
    $args{realm}  = $options{KRB_REALM}  if ($options{KRB_REALM});
    my $kadmin = new Krb::Admin(kdc => $options{KDC}, %args);

    $kadmin->setup_host(host => $options{CLIENTNAME},
                        user => $options{TESTUSER});

    $Log->exit();
}

sub DESTROY {
    my $self = shift;
    local ($@);

    $self->client_cleanup();
}

sub client_cleanup {
    my ($self, %options) = @_;
    $Log->enter();

    # ??? - GV Kerberos Cleanup should be added here. For now, it's harmless leaving those clients
    # configured for Kerberos, since kerberos will kick in only if kerberos option is used to mount
    # the clients.

    # Execute the one from the parent
    $self->SUPER::client_cleanup(%options);

    $Log->exit();
}

######################### Internal Routines ##################

1;
