# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Unicode.pm#1 $
#
# $Change: 1917575 $
#
# $DateTime: 2013/01/22 11:22:46 $
#
# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
#
#
# NAME
#      Unicode.pm - I2P library to test Unicode characters 
#
# DESCRIPTION
#      This perl module provide interfaces to deal with different
#      translation tables used for nfs to cifs character translation  
#      and vice versa for different supported volume languages by filer.
#
# AUTHOR
#       tarung@netapp.com
#
#
## @pod here
##
##

=pod

=head1 NAME

    Unicode.pm - I2P library to test Unicode characters

=head1 SYNOPSIS

    use Unicode qw(:unicode);

    # Get the supported vol lang name list
    my $vol_list =get_supported_vol_list();
    my @vol_list =get_supported_vol_list();

    # Get the vol language code  for different vol language name
    foreach my $v (@vol_list) {
        my $lang_code1 = get_vol_lang_code(vol_lang => $v, vol_type => 'utf8');
        my $lang_code2 = get_vol_lang_code(vol_lang => $v, vol_type => 'nonutf8');
        my $lang_code3 = get_vol_lang_code(vol_lang => $v, vol_type => 'all');
        my @lc_list    = get_vol_lang_code(vol_lang => $v, vol_type => 'all');
    }

    # Get the SGML code for different volume language name and code range
    foreach my $v ( @vol_list )
    {
        foreach my $code ( 128 .. 255 )
        {
            my $sgml_code = get_sgml_code(
                            code     => $code, 
                            vol_type => 'utf8',
                            vol_lang => $v,
                            data_set => 'nfs') or die("error");
            my $sgml_code = get_sgml_code(
                            code     => $code,
                            vol_type => 'utf8',
                            vol_lang => $v,
                            data_set => 'cifs') or die("error");
        }
    }

    # Get conversion table ids
    my $nfs_tbl_ids    = get_conv_table_id_list(tbl_type => 'nfs');

    my @nfs_tblid_list = get_conv_table_id_list(tbl_type => 'nfs');

    my $oem_tbl_ids    = get_conv_table_id_list(tbl_type => 'oem');

    my @oem_tblid_list = get_conv_table_id_list(tbl_type => 'oem');


    # Get the convertible code list from nfs_char_set table
    my @cc_list = get_nfs_tbl_conv_code_list(
                    'vol_lang' => 'Czech',
                    'conv_type' => 'nfs_to_cifs') or die("error");

    my @cc_list = get_nfs_tbl_conv_code_list(
                    'vol_lang' => 'Czech',
                    'conv_type' => 'cifs_to_nfs') or die("error");


    # Get the conversion/translation table id
    my $table_id = get_conv_table_id(
                    'vol_lang' => 'Czech',
                    'tbl_type' => 'nfs'
                    ) or die ("error");

    my @cc_list = get_nfs_tbl_conv_code_list(
                    'vol_lang' => 'Czech',
                    'table_id' => $table_id,
                    'conv_type' => 'cifs_to_nfs') or die("error");

    # Get the convertible code list from oem translation table
    my @cc_list = get_oem_tbl_conv_code_list(
                    'vol_lang' => 'Czech',
                    'table_id' => $table_id,
                    'conv_type' => 'owc_to_nwc') or die("error");

    # Check, if used code is convertible in nfs table
    if ( is_nfs_tbl_code_convertible(
                    'code    ' => '165',
                    'vol_lang' => 'Czech',
                    'table_id' => $table_id,
                    'conv_type' => 'nfs_to_cifs') ) {
        print "code is convertible";
    }
    
    OR

    if ( is_nfs_tbl_code_convertible(
                    'code    ' => '165',
                    'vol_lang' => 'Czech',
                    'conv_type' => 'cifs_to_nfs') ) {
        print "code is convertible";
    }

    # check, if code is convertible in oem table
    if ( is_oem_tbl_code_convertible(
                    'code    ' => '165',
                    'vol_lang' => 'Czech',
                    'conv_type' => 'owc_to_nwc') ) {
        print "code is convertible";
    }


    # Get the convertible and non-convertible code list
    # from the given code range
    my $op_hash = get_nfs_tbl_conv_and_nonconv_code_list(
                        'vol_lang'  => 'Czech',
                        'code_list' => [128..255],
                        'conv_type' => 'cifs_to_nfs',
                    ) or die("error");

    my @conv_code    = @{$$op_hash{'conv'}};
    my @nonconv_code = @{$$op_hash{'non_conv'}};

    my $op_hash = get_oem_tbl_conv_and_nonconv_code_list(
                        'vol_lang'  => 'Czech',
                        'code_list' => [128..255],
                        'conv_type' => 'nwc_to_owc',
                    ) or die("error");

    my @conv_code    = @{$$op_hash{'conv'}};
    my @nonconv_code = @{$$op_hash{'non_conv'}};


    # Check, if table is valid in nfs_char_set or oem_char_set
    if ( is_valid_conv_table_id(
            'tbl_type' => 'nfs',
            'table_id' => $table_id) ) {
        print "table id $table_id is valid";
    }

    OR

    if ( is_valid_conv_table_id(
            'tbl_type' => 'oem',
            'table_id' => $table_id) ) {
        print "table id $table_id is valid";
    }


    # Get reference to complete translation table
    my $op_ref_hash = get_complete_conv_table(
                        tbl_type   => 'oem', 
                        'vol_lang' => 'POSIX', 
                        table_id   => 'cp1250') or die("error");
    OR
    my $op_ref_hash = get_complete_conv_table(
                        tbl_type   => 'nfs', 
                        'vol_lang' => 'Arabic', 
                        table_id   => 'cp1250') or die("error");

    my @codes = keys %{$op_ref_hash};

    # Get the cifs translated code for a given code
    my $trns_code = get_cifs_translated_code('code' => 128);

    # Get the supported vol list for a given nfs translation table id
    my @vol_list = get_vol_list_by_nfs_conv_table_id('table_id' => 'iso_8859_1');

    # Get the supported vol list for all nfs translation table ids
    my %vol = %{get_vol_list_for_all_nfs_conv_table_ids()};

    # Get file name for non-utf8 volume
    my $filename = create_non_utf8_file_name('nfs_code' => 0x12121);

=head1 DESCRIPTION

    This perl module provide interfaces to deal with different
    translation tables used for nfs to cifs character translation
    and viceversa for diffrent supported volume languages by filer.

    This module make use of translation tables provided in following
    modules,
    use OEM_Conv_Tables qw(:oem_conv_tables);
    use NFS_Conv_Tables qw(:nfs_conv_tables);

    Refer to the <FUNCTIONS> section below for an explanation on using 
    the predefined methods.

=head1 FUNCTIONS

I2P Unicode Functions

=cut


package Unicode;

#------------------------------------------------------
#   Imports
#------------------------------------------------------
use Tharn;
use Data::Dumper;
use strict qw(vars subs refs);
use base   qw(Exporter);
use OEM_Conv_Tables      qw(:oem_conv_tables);
use NFS_Conv_Tables      qw(:nfs_conv_tables);
use Params::Validate     qw(validate validate_pos SCALAR ARRAYREF);
use SPARSE::Util::Logger qw(logWarn logError);

#------------------------------------------------------
#   Module Vars
#------------------------------------------------------
our ($Validation, $VERSION);

#
# Define constants
#
use constant UC_ERROR => undef;
use constant UC_PASS  => 1;
use constant UC_FAIL  => 2;
use constant UC_TRUE  => 1;
use constant UC_FALSE => 0;


BEGIN {

    
    #--------------------------------------------
    #   Exports
    #--------------------------------------------

    $Validation = 1; # unused as of now
    $VERSION    = '0.01';

    my @all_sub = qw(

                &get_supported_vol_list
                &get_vol_lang_code
                &get_cifs_translated_code
                &get_sgml_code
                &get_conv_table_id
                &get_complete_conv_table
                &get_nfs_tbl_conv_code_list
                &is_nfs_tbl_code_convertible
                &get_nfs_tbl_conv_and_nonconv_code_list
                &get_oem_tbl_conv_code_list
                &is_oem_tbl_code_convertible
                &get_oem_tbl_conv_and_nonconv_code_list
                &get_conv_table_id_list
                &is_valid_conv_table_id
                &get_vol_list_by_nfs_conv_table_id
                &get_vol_list_for_all_nfs_conv_table_ids
                &create_non_utf8_file_name
            );

    my @constants = qw( UC_ERROR UC_PASS UC_FAIL UC_TRUE UC_FAIL);

    our %EXPORT_TAGS = (
        'unicode' => [ @all_sub, @constants]
    );

    $EXPORT_TAGS{all} = [ map{ @{$_} } values %EXPORT_TAGS ];

    Exporter::export_ok_tags('all');
}

#------------------------------------------------------
#     Prototypes
#------------------------------------------------------
sub get_supported_vol_list ;
sub get_vol_lang_code ;
sub get_cifs_translated_code ;
sub get_sgml_code ;
sub get_conv_table_id ;
sub get_complete_conv_table ;
sub get_nfs_tbl_conv_code_list ;
sub is_nfs_tbl_code_convertible ;
sub get_nfs_tbl_conv_and_nonconv_code_list ;
sub get_oem_tbl_conv_code_list ;
sub is_oem_tbl_code_convertible ;
sub get_oem_tbl_conv_and_nonconv_code_list ;
sub get_conv_table_id_list ;
sub is_valid_conv_table_id ;
sub get_vol_list_by_nfs_conv_table_id ;
sub get_vol_list_for_all_nfs_conv_table_ids ;
sub create_non_utf8_file_name ;


our %std_code_table = (
                        '160' => 'nbsp',   '161' => 'iexcl',  '162' => 'cent',
                        '163' => 'pound',  '164' => 'curren', '165' => 'yen',
                        '166' => 'brvbar', '167' => 'sect',   '168' => 'uml', 
                        '169' => 'copy',   '170' => 'ordf',   '171' => 'laquo',
                        '172' => 'not',    '173' => 'shy',    '174' => 'reg',
                        '175' => 'macr',   '176' => 'deg',    '177' => 'plusmn',
                        '178' => 'sup2',   '179' => 'sup3',   '180' => 'acute',
                        '181' => 'micro',  '182' => 'para',   '183' => 'middot',
                        '184' => 'cedil',  '185' => 'sup1',   '186' => 'ordm',
                        '187' => 'raquo',  '188' => 'frac14', '189' => 'frac12',
                        '190' => 'frac34', '191' => 'iquest', '192' => 'Agrave',
                        '193' => 'Aacute', '194' => 'Acirc',  '195' => 'Atilde',
                        '196' => 'Auml',   '197' => 'Aring',  '198' => 'AElig',
                        '199' => 'cedil',  '200' => 'Egrave', '201' => 'Eacute',
                        '202' => 'Ecirc',  '203' => 'Euml',   '204' => 'Igrave',
                        '205' => 'Iacute', '206' => 'Icirc',  '207' => 'Iuml',
                        '208' => 'ETH',    '209' => 'Ntilde', '210' => 'Ograve',
                        '211' => 'Oacute', '212' => 'Ocirc',  '213' => 'Otilde',
                        '214' => 'Ouml',   '215' => 'times',  '216' => 'Oslash',
                        '217' => 'Ugrave', '218' => 'Uacute', '219' => 'circ',
                        '220' => 'Uuml',   '221' => 'Yacute', '222' => 'THORN',
                        '223' => 'szlig',  '224' => 'agrave', '225' => 'aacute',
                        '226' => 'acirc',  '227' => 'atilde', '228' => 'auml',
                        '229' => 'aring',  '230' => 'aelig',  '231' => 'ccedil',
                        '232' => 'egrave', '233' => 'eacute', '234' => 'ecirc',
                        '235' => 'euml',   '236' => 'igrave', '237' => 'iacute',
                        '238' => 'icirc',  '239' => 'iuml',   '240' => 'eth',
                        '241' => 'ntilde', '242' => 'ograve', '243' => 'oacute',
                        '244' => 'ocirc',  '245' => 'otilde', '246' => 'ouml', 
                        '247' => 'divide', '248' => 'oslash', '249' => 'ugrave',
                        '250' => 'uacute', '251' => 'ucirc',  '252' => 'uuml',
                        '253' => 'yacute', '254' => 'thorn',  '255' => 'yuml'
                        );

our %cifs_ext_ascii_conv_table = (
                                    '128' => '8364',
                                    '129' => '129',
                                    '130' => '8218',
                                    '131' => '402',
                                    '132' => '8222',
                                    '133' => '8230',
                                    '134' => '8224',
                                    '135' => '8225',
                                    '136' => '710',
                                    '137' => '8240',
                                    '138' => '352',
                                    '139' => '8249',
                                    '140' => '338',
                                    '141' => '141',
                                    '142' => '381',
                                    '143' => '143',
                                    '144' => '144',
                                    '145' => '8216',
                                    '146' => '8217',
                                    '147' => '8220',
                                    '148' => '8221',
                                    '149' => '8226',
                                    '150' => '8211',
                                    '151' => '8212',
                                    '152' => '732',
                                    '153' => '8482',
                                    '154' => '353',
                                    '155' => '8250',
                                    '156' => '339',
                                    '157' => '157',
                                    '158' => '382',
                                    '159' => '376',
                                    '160' => '160',
                                 );

our %supported_vol_type_list = (
                                'POSIX'           => 'C',
                                'Arabic'          => 'ar',
                                'Czech'           => 'cs',
                                'Croatian'        => 'hr',
                                'Hungarian'       => 'hu',
                                'Polish'          => 'pl',
                                'Romanian'        => 'ro',
                                'Slovak'          => 'sk',
                                'EnglishUS'       => 'en_US',
                                'English'         => 'en',
                                'Danish'          => 'da',
                                'German'          => 'de',
                                'Spanish'         => 'es',
                                'Finnish'         => 'fi',
                                'French'          => 'fr',
                                'Italian'         => 'it',
                                'Norwegian'       => 'no',
                                'Dutch'           => 'nl',
                                'Swedish'         => 'sv',
                                'Hebrew'          => 'he',
                                'Portuguese'      => 'pt',
                                'Russian'         => 'ru',
                                'Slovenian'       => 'sl',
                                'Turkish'         => 'tr',
                                'Korean'          => 'ko',
                                'Japanese'        => 'ja',
                                'JapaneseV1'      => 'ja_v1',
                                'JapanesePCK'     => 'ja_JP.PCK',
                                'Japanese932'     => 'ja_JP.932',
                                'JapanesePCKV2'   => 'ja_JP.PCK_v2',
                                'Chinese'         => 'zh',
                                'ChineseGBK'      => 'zh.GBK',
                                'TradChineseTW'   => 'zh_TW',
                                'TradChineseBIG5' => 'zh_TW.BIG5',
                               );

my %lang_conv_table_id = (
                                'POSIX' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp1'    
                                    },    
                                'Arabic' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_6',
                                        'oem_char_set' => 'cp1256'    
                                    },    
                                'Czech' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_2',
                                        'oem_char_set' => 'cp1250'    
                                    },    
                                'Croatian' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_2',
                                        'oem_char_set' => 'cp1250'    
                                    },    
                                'Hungarian' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_2',
                                        'oem_char_set' => 'cp1250'    
                                    },    
                                'Polish' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_2',
                                        'oem_char_set' => 'cp1250'    
                                    },    
                                'Romanian' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_2',
                                        'oem_char_set' => 'cp1250'    
                                    },    
                                'Slovak' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_15',
                                        'oem_char_set' => 'cp1250'    
                                    },    
                                'EnglishUS' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp437'    
                                    },    
                                'English' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp850'    
                                    },    
                                'Danish' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp850'    
                                    },    
                                'German' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp850'    
                                    },    
                                'Spanish' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp850'    
                                    },    
                                'Finnish' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp850'    
                                    },    
                                'French' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp850'    
                                    },    
                                'Italian' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp850'    
                                    },    
                                'Norwegian' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp850'    
                                    },    
                                'Dutch' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp850'    
                                    },    
                                'Swedish' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp850'    
                                    },    
                                'Hebrew' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_8',
                                        'oem_char_set' => 'cp862'    
                                    },    
                                'Portuguese' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_1',
                                        'oem_char_set' => 'cp860'    
                                    },    
                                'Russian' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_5',
                                        'oem_char_set' => 'cp866'    
                                    },    
                                'Slovenian' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_2',
                                        'oem_char_set' => 'cp852'    
                                    },    
                                'Turkish' =>
                                    {
                                        'nfs_char_set' => 'iso_8859_9',
                                        'oem_char_set' => 'cp857'    
                                    },    
                                'Korean' =>
                                    {
                                        'nfs_char_set' => 'euckr',
                                        'oem_char_set' => 'cp949'    
                                    },    
                                'Japanese' =>
                                    {
                                        'nfs_char_set' => 'eucj',
                                        'oem_char_set' => 'cp932'    
                                    },    
                                'JapaneseV1' =>
                                    {
                                        'nfs_char_set' => 'eucj_v1',
                                        'oem_char_set' => 'cp932_v1'    
                                    },    
                                'JapanesePCK' =>
                                    {
                                        'nfs_char_set' => 'sjis',
                                        'oem_char_set' => 'cp932_v1'    
                                    },    
                                'Japanese932' =>
                                    {
                                        'nfs_char_set' => 'sjis932',
                                        'oem_char_set' => 'cp932'    
                                    },    
                                'JapanesePCKV2' =>
                                    {
                                        'nfs_char_set' => 'sjis_v2',
                                        'oem_char_set' => 'cp932_v1'    
                                    },    
                                'Chinese' =>
                                    {
                                        'nfs_char_set' => 'euccn',
                                        'oem_char_set' => 'cp936'    
                                    },    
                                'ChineseGBK' =>
                                    {
                                        'nfs_char_set' => 'cp936',
                                        'oem_char_set' => 'cp936'    
                                    },    
                                'TradChineseTW' =>
                                    {
                                        'nfs_char_set' => 'euctw',
                                        'oem_char_set' => 'cp950'    
                                    },    
                                'TradChineseBIG5' =>
                                    {
                                        'nfs_char_set' => 'big_five',
                                        'oem_char_set' => 'cp950'    
                                    },    
                            );


=head2 get_supported_vol_list()

Return the array or string of filer supported volumes language name list

=over 4

=item Arguments:

None

=item Returns:

Returns @vol_list, if called in array context 

OR

Returns $vol_list, if called in scalar context

=back

=cut

sub get_supported_vol_list {

    my @vol_list = keys %supported_vol_type_list;
    return wantarray ? @vol_list : "@vol_list";
}

=head2 get_vol_lang_code()

Returns the language code for utf8 or nonutf8 or both flexvol type

=over 4

=item Arguments:

=over 4

=item vol_lang

volume language name

=item vol_type

volume type [ utf8 or nonutf8 or both ]

=back

=item Returns:

Returns an array of volume language code, if called in array context

OR

Returns a space separated language code string, if called in scalar context

=back

=cut

sub get_vol_lang_code {

    my (%options) = validate(@_,
                        {
                            'vol_lang' =>
                                {
                                    type => SCALAR
                                },
                            'vol_type' =>
                                {
                                    type => SCALAR,
                                    regex => qr/^(utf8|nonutf8|all)$/
                                },
                        } );


    my ($vol_lang); # input vol lang name
    my ($vol_type); # type of vol: utf8 | nonutf8 | both
    my ($sup_vol);  # string of all supported vol lang name
    my (@vol_list); # returned vol list
   

    $vol_lang = $options{'vol_lang'};
    $vol_type = $options{'vol_type'};

    # validate vol_lang value
    if ( ! exists($supported_vol_type_list{$vol_lang}) ) {
        $sup_vol = get_supported_vol_list();
        logWarn("Invalid vol_lang: $vol_lang,"
                    . " correct types:\[ $sup_vol \]\n");
        return UC_ERROR;
    }

    if ( $vol_type eq "utf8" ) {
        push( @vol_list, "$supported_vol_type_list{$vol_lang}\.UTF-8" );
    } elsif ( $vol_type eq "nonutf8" ) {
        push( @vol_list, "$supported_vol_type_list{$vol_lang}" );
    } else {
        # vol_type eq all
        push( @vol_list, "$supported_vol_type_list{$vol_lang}\.UTF-8" );
        push( @vol_list, "$supported_vol_type_list{$vol_lang}" );
    }

    return wantarray ? @vol_list : "@vol_list";
}

=head2 get_cifs_translated_code()

Returns the cifs translated code for a given

code (which was used to generate char in file name)

=over 4

=item Arguments:

=over 4

=item code

an integer number from 128 to 65536 range

=back

=item Returns:

returns translated code

=back

=cut

sub get_cifs_translated_code {

    my (%options) = validate(@_,
                        {
                            'code' => {type => SCALAR, regex => qr/^\d+$/ }
                        } );

    my ($code); # input code, which has to be translated

    $code = $options{'code'};

    return ( exists($cifs_ext_ascii_conv_table{$code})
                ? $cifs_ext_ascii_conv_table{$code}
                : $code );
}


=head2 get_sgml_code()

Returns the sgml code for a given code, volume language,

volume type and data set.

=over 4

=item Arguments:

=over 4

=item code

An integer number from 128 to 65536 range.

This code is used to generate a character in file name.

=item vol_lang

Name of volume language like English, Arabic etc.

=item vol_type

Volume type [ utf8 or nonutf8  ]

=item data_set

Type of data set [ cifs or nfs ]

=over 4

cifs : when data set is created on cifs client

nfs  : when data set is created on nfs  client

nfs_to_cifs  : when data set is created on nfs and 
               copied on cifs client

=back

=back

=item Returns:

Returns sgml code, if successful

OR

Returns UNDEFINED as a string, if sgml is not found

OR

Return undef, in case of error

=back

=cut

sub get_sgml_code {

    my (%options) = validate(@_,
                        {
                            'code' =>
                                {
                                    type => SCALAR,
                                    regex => qr/^\d+$/
                                },
                            'vol_lang' =>
                                {
                                    type => SCALAR
                                },
                            'vol_type' =>
                                {
                                    type => SCALAR,
                                    regex => qr/^(utf8|nonutf8)$/
                                },
                            'data_set' =>
                                {
                                    type => SCALAR,
                                    regex => qr/^(nfs|cifs|nfs_to_cifs)$/
                                },
                        } );

    my ($code);       # input char code, whose sgml has to be found
    my ($vol_lang);   # input vol lang name
    my ($vol_type);   # type of input vol, utf8 or nonutf8
    my ($data_set);   # type of data set
    my ($sup_vol);    # string with supported vol lang name
    my ($sgml_code);  # sgml code to be returned
    my ($sgml_error); # sgml error string
    my ($table_id);   # id of conversion table used
    my ($translated_code); # translated code of input code

    $code     = $options{'code'};
    $vol_type = $options{'vol_type'};
    $vol_lang = $options{'vol_lang'};
    $data_set = $options{'data_set'};

    $sgml_error = 'UNDEFINED';

    #
    # validate the vol_lang value
    #
    if ( ! $supported_vol_type_list{$vol_lang} ) {
        $sup_vol = get_supported_vol_list();
        logWarn("Invalid vol_lang: $vol_lang,"
                    . " correct langs:\[ $sup_vol \]");
        return UC_ERROR;
    }

    if ( $data_set eq 'nfs' ) {

        if ( $vol_type eq 'nonutf8' ) {
            $table_id = $lang_conv_table_id{$vol_lang}{'nfs_char_set'};
            if ( exists($g_NFS_Conv_Tables{$table_id}{$code}) ) {
                $translated_code = $g_NFS_Conv_Tables{$table_id}{$code};
                if ( $translated_code <= 127 ) {
                    # if translated_code is <= 127,
                    # then return the character with 
                    # respect to code.
                    $sgml_code = chr($code);
                } else {
                    $sgml_code = exists($std_code_table{$translated_code})
                                ? "&$std_code_table{$translated_code};"
                                : "&#$translated_code;";
                }
            } else {
                $sgml_code = $sgml_error;
            }
        } elsif ( $vol_type eq 'utf8' ) {
            $sgml_code = exists($std_code_table{$code})
                            ? "&$std_code_table{$code};"
                            : "&#$code;";
        }
    } elsif ( $data_set eq "cifs" ) {
        # in this case files are generated on cifs.
        # in case of cifs, vol_type does not matter.
        # cifs speaks ucs-2, which is  directly converted
        # into sgml code. For file name having character
        # from code range [128 to 159], cifs uses translated
        # code as per cifs_ext_ascii_conv_table table
        $translated_code = exists($cifs_ext_ascii_conv_table{$code})
                        ? "$cifs_ext_ascii_conv_table{$code}"
                        : "$code";

        if ( $translated_code <= 127 ) {
            $sgml_code = chr($code);
        } else {
            $sgml_code = exists($std_code_table{$translated_code})
                        ? "&$std_code_table{$translated_code};"
                        : "&#$translated_code;";
        }
    } elsif ( $data_set eq "nfs_to_cifs" ) {
        if ( $code <= 127 ) {
            $sgml_code = chr($code);
        } else {
            $sgml_code = exists($std_code_table{$code})
                        ? "&$std_code_table{$code};"
                        : "&#$code;";
        }
    }
    return $sgml_code;
}

=head2 get_conv_table_id()

Returns the translation or conversion table id for a given

volume language and table type.

=over 4

=item Arguments:

=over 4

=item vol_lang

Name of volume language like English, Arabic etc.

=item tbl_type

Type of conversion table [ nfs or oem ]

=back

=item Returns:

Returns table id, if successful

OR

Returns undef, in case of error

=back

=cut

sub get_conv_table_id {

    my (%options) = validate(@_,
                        {
                            'tbl_type' =>
                                {
                                    type => SCALAR,
                                    regex => qr/^(nfs|oem)$/
                                },
                            'vol_lang' =>
                                {
                                    type => SCALAR
                                },
                        } );

    my ($tbl_type); # input character conversion table type
    my ($vol_lang); # input vol lang name
    my ($sup_vol);  # string with supported vol lang name

    $tbl_type = $options{'tbl_type'};
    $vol_lang = $options{'vol_lang'};

    #
    # validate the vol_lang value
    #
    if ( ! $supported_vol_type_list{$vol_lang} ) {
        $sup_vol = get_supported_vol_list();
        logWarn("Invalid vol_lang: $vol_lang,"
                    . " correct langs:\[ $sup_vol \]");
        return UC_ERROR;
    }

    if ( $tbl_type eq "nfs" ) {
        return $lang_conv_table_id{$vol_lang}{'nfs_char_set'};
    } else {
        # tbl_type is oem
        return $lang_conv_table_id{$vol_lang}{'oem_char_set'};
    }
}

=head2 get_complete_conv_table()

Returns reference to the translation or conversion table for a given

volume language and table type.

=over 4

=item Arguments:

=over 4

=item vol_lang

Name of volume language like English, Arabic etc.

=item tbl_type

Type of conversion table [ nfs or oem ]

=item table_id [optional param]

Table id of conversion table. if not given, then the same

will be found out using vol_lang and tbl_type field.

=back

=item Returns:

Returns reference to conversion table, if successful

OR

Returns undef, in case of error

=back

=cut

sub get_complete_conv_table {

    my (%options) = validate(@_,
                        {
                            'vol_lang' =>
                                {
                                    type => SCALAR
                                },
                            'table_id' =>
                                {
                                    type     => SCALAR,
                                    optional => 1
                                },
                            'tbl_type' =>
                                {
                                    type => SCALAR,
                                    regex => qr/^(nfs|oem)$/
                                },
                        } );

    my ($vol_lang); # vol lang name
    my ($table_id); # input table id
    my ($tbl_type); # input conversion table type
    my ($sup_vol);  # string with supported vol lang name
    my (%conv_table);  # conversion table to be returned
    my ($vld_tbl_ids); # string with valid table id
    my ($ref_conv_table); # reference to correct conv table

    $vol_lang = $options{'vol_lang'};
    $table_id = $options{'table_id'};
    $tbl_type = $options{'tbl_type'};


    #
    # get the table_id from vol_lang, if not given
    #
    if ( ! defined($table_id) ) {
        if ( ! $supported_vol_type_list{$vol_lang} ) {
            $sup_vol = get_supported_vol_list();
            logWarn("Invalid vol_lang: $vol_lang,"
                . " correct langs:\[ $sup_vol \]");
            return UC_ERROR;
        }
        if ( $tbl_type eq 'nfs' ) {
            $table_id = $lang_conv_table_id{$vol_lang}{'nfs_char_set'};
        } else {
            $table_id = $lang_conv_table_id{$vol_lang}{'oem_char_set'};
        }
    }

    if ( $tbl_type eq 'nfs' ) {
        $vld_tbl_ids = get_conv_table_id_list('tbl_type'=>'nfs');
        $ref_conv_table = \%g_NFS_Conv_Tables;
    } else {
        # tbl_type is oem
        $vld_tbl_ids = get_conv_table_id_list('tbl_type'=>'oem');
        $ref_conv_table = \%g_OEM_Conv_Tables;
    }

    #
    # validate the table_id
    #
    if ( ! exists($$ref_conv_table{$table_id}) ) {
        logWarn("Invalid $tbl_type conv_table_id:$table_id,"
            . " correct ids:\[ $vld_tbl_ids \]");
        return UC_ERROR;
    }

    # get a copy of conversion table
    %conv_table = %{$$ref_conv_table{$table_id}};

    return (\%conv_table); 
}

=head2 get_nfs_tbl_conv_code_list()

Returns the list of convertible code for a given

volume language and list type, from nfs translation table set.

=over 4

=item Arguments:

=over 4

=item vol_lang

Name of volume language like English, Arabic etc.

=item conv_type

Type of conversion [ nfs_to_cifs or cifs_to_nfs ]

=over 4

nfs_to_cifs  : if data set will be created on Unix client

cifs_to_nfs  : if data set will be created on Windows client

=back

=item table_id [optional param]

Table id of conversion table. if not given, then the same

will be found out using vol_lang and tbl_type field.

=back

=item Returns:

Returns an array of convertible code, if successful

OR

Returns undef, in case of error

=back

=cut

sub get_nfs_tbl_conv_code_list {

    my (%options) = validate(@_,
                        {
                            'vol_lang' =>
                                {
                                    type => SCALAR
                                },
                            'table_id' =>
                                {
                                    type     => SCALAR,
                                    optional => 1
                                },
                            'conv_type' =>
                                {
                                    type  => SCALAR,
                                    regex => qr/^(nfs_to_cifs|cifs_to_nfs)$/
                                },
                        } );


    my ($vol_lang);  # input vol lang name
    my ($table_id);  # input table id
    my ($conv_type); # input conversion type 
    my ($sup_vol);   # string with supported vol lang name
    my ($vld_tbl_ids);    # string with valid table id
    my (%nfs_conv_table); # table to be returned
    my (@conv_code_list); # returned list of convertible code

    $vol_lang  = $options{'vol_lang'};
    $table_id  = $options{'table_id'};
    $conv_type = $options{'conv_type'};

    #
    # get the table_id from vol_lang, if not given
    #
    if ( ! defined($table_id) ) {
        if ( ! $supported_vol_type_list{$vol_lang} ) {
            $sup_vol  = get_supported_vol_list();
            logWarn("Invalid vol_lang: $vol_lang,"
                . " correct langs:\[ $sup_vol \]");
            return UC_ERROR;
        }
        $table_id = $lang_conv_table_id{$vol_lang}{'nfs_char_set'};
    }

    #
    # validate the table_id key exists
    #
    if ( ! exists($g_NFS_Conv_Tables{$table_id}) ) {
        $vld_tbl_ids = get_conv_table_id_list('tbl_type' => 'nfs');
        logWarn("Invalid NFS conv_table_id:$table_id,"
            . " correct ids:\[ $vld_tbl_ids \]\n");
        return UC_ERROR;
    }

    #
    # let's point to the correct translation table
    #
    %nfs_conv_table = %{$g_NFS_Conv_Tables{$table_id}};

    if ( $conv_type eq "nfs_to_cifs" ) {
        @conv_code_list = keys %nfs_conv_table;
    } else {
        # conv_type is cifs_to_nfs
        @conv_code_list = values %nfs_conv_table;
    }

    return @conv_code_list; 
}


=head2 is_nfs_tbl_code_convertible()

Returns true, if input code is convertible(code has an entry 

in nfs_char_set translation table), Otherwise false.

=over 4

=item Arguments:

=over 4

=item code

An integer number from 128 to 65536 range

=item vol_lang

Name of volume language like English, Arabic etc.

=item conv_type

Type of conversion [ nfs_to_cifs or cifs_to_nfs ]

=over 4

nfs_to_cifs : if data set is created on Unix client

cifs_to_nfs : if data set is created on Windows client

=back

=item table_id [ optional param ]

Translation table id, if not given, then same will

be found out using vol_lang.

=back

=item Returns:

Returns 1, if successful

OR

Returns 0, if failed

OR

Returns undef, in case of error

=back

=cut

sub is_nfs_tbl_code_convertible {

    my (%options) = validate(@_,
                        {
                            'code' =>
                                {
                                    type => SCALAR,
                                    regex => qr/^\d+$/
                                },
                            'vol_lang' =>
                                {
                                    type => SCALAR
                                },
                            'table_id' =>
                                {
                                    type => SCALAR,
                                    optional => 1
                                },
                            'conv_type' =>
                                {
                                    type  => SCALAR,
                                    regex => qr/^(nfs_to_cifs|cifs_to_nfs)$/
                                },
                        } );

    my ($code);       # input code which has to be checked
    my ($vol_lang);   # input vol lang name
    my ($table_id);   # input table id
    my ($conv_type);  # input conversion type
    my (%conv_table); # valid conv table hash 
    my ($vld_tbl_ids);# string with valid table ids
    my ($sup_vol);    # string with supported vol lang name
    my (@conv_code_list); # list of convertible code

    $code      = $options{'code'};
    $vol_lang  = $options{'vol_lang'};
    $table_id  = $options{'table_id'};
    $conv_type = $options{'conv_type'};

    #
    # get the table_id from vol_lang, if not given
    #
    if ( ! defined($table_id) ) {
        if ( ! $supported_vol_type_list{$vol_lang} ) {
            $sup_vol  = get_supported_vol_list();
            logWarn("Invalid vol_lang: $vol_lang,"
                        . " correct langs:\[ $sup_vol \]");
            return UC_ERROR;
        }
        $table_id = $lang_conv_table_id{$vol_lang}{'nfs_char_set'};
    }

    #
    # validate the table_id
    #
    if ( ! exists($g_NFS_Conv_Tables{$table_id}) ) {
        $vld_tbl_ids = get_conv_table_id_list('tbl_type' => 'nfs');
        logWarn("Invalid NFS conv_table_id: $table_id,"
                    . " correct ids:\[ $vld_tbl_ids \]\n");
        return UC_ERROR;
    }

    #
    # get the correct translation table
    #
    if ( $conv_type eq "nfs_to_cifs" ) {
        %conv_table = %{$g_NFS_Conv_Tables{$table_id}};
    } else {
        # conv_type is cifs_to_nfs
        %conv_table = reverse %{$g_NFS_Conv_Tables{$table_id}};
        $code = get_cifs_translated_code($code);
    }

    return exists($conv_table{$code}) ? UC_TRUE : UC_FALSE;
}


=head2 get_nfs_tbl_conv_and_nonconv_code_list()

This interface separate out the convertible and non_convertible codes

from the given code ranges.

=over 4

=item Arguments:

=over 4

=item code_list

Reference to input code range array

=item vol_lang

Name of volume language like English, Arabic etc.

=item conv_type

Type of conversion table [ nfs_to_cifs or cifs_to_nfs ]

=over 4

nfs_to_cifs : if data set will be created on Unix client

cifs_to_nfs : if data set will be created on Windows client

=back

=item table_id [optional param]

Translation table id, if not given, then same will

be found out using vol_lang.

=back

=item Returns:

Returns a reference to hash(say $rh) with key conv and non_conv.

=over 4

$rh{conv} : a reference to convertible code array

$rh{non_conv} : a reference to convertible code array

=back

OR

Returns undef, in case of error

=back

=cut

sub get_nfs_tbl_conv_and_nonconv_code_list {

    my (%options) = validate(@_,
                        {
                            'code_list' =>
                                {
                                    type => ARRAYREF,
                                },
                            'vol_lang' =>
                                {
                                    type => SCALAR
                                },
                            'table_id' =>
                                {
                                    type => SCALAR,
                                    optional => 1
                                },
                            'conv_type' =>
                                {
                                    type  => SCALAR,
                                    regex => qr/^(nfs_to_cifs|cifs_to_nfs)$/
                                },
                        } );


    my (@codeList); # input code which has to be checked
    my ($vol_lang); # input vol lang name
    my ($table_id); # input conv table id
    my ($conv_type);   # input conv table type
    my (%conv_table);  # valid conv table hash 
    my ($vld_tbl_ids); # string with valid table ids
    my ($sup_vol); # string with supported vols lang name
    my (@conv_code_list); # list of convertible code
    my (@nonconv_code_list); # list of non convertible code
    my (%op_hash); # returned hash of conv and nonconv list

    @codeList  = @{$options{'code_list'}};
    $vol_lang  = $options{'vol_lang'};
    $table_id  = $options{'table_id'};
    $conv_type = $options{'conv_type'};

    # add returned list to output hash
    $op_hash{'conv'} = \@conv_code_list;
    $op_hash{'non_conv'} = \@nonconv_code_list;

    #
    # findout the table_id from vol_lang,
    # if it is not defined
    #
    if ( ! defined($table_id) ) {
        if ( ! exists($supported_vol_type_list{$vol_lang}) ) {
            $sup_vol  = get_supported_vol_list();
            logWarn("Invalid vol_lang: $vol_lang,"
                        . " correct types:\[ $sup_vol \]");
            return UC_ERROR;
        }
        $table_id = $lang_conv_table_id{$vol_lang}{'nfs_char_set'};
    }

    #
    # validate the table_id
    #
    if ( ! exists($g_NFS_Conv_Tables{$table_id}) ) {
        $vld_tbl_ids = get_conv_table_id_list('tbl_type' => 'nfs');
        logWarn("Invalid NFS conv_table_id:$table_id,"
                    . " correct ids:\[ $vld_tbl_ids \]");
        return UC_ERROR;
    }

    #
    # get the convertible and non-convertible code
    #
    if ( $conv_type eq "nfs_to_cifs" ) {
        %conv_table = %{$g_NFS_Conv_Tables{$table_id}};
        foreach my $code ( sort @codeList ) {
            if ( exists($conv_table{$code}) ) {
                push(@conv_code_list, $code);
            } else {
                push(@nonconv_code_list, $code);
            }
        }
    } else {
        # conv_type is cifs_to_nfs
        %conv_table = reverse %{$g_NFS_Conv_Tables{$table_id}};

        foreach my $code ( sort @codeList ) {
            my $trns_code = get_cifs_translated_code('code' => $code);
            if ( exists($conv_table{$trns_code}) ) {
                push(@conv_code_list, $code);
            } else {
                push(@nonconv_code_list, $code);
            }
        }
    }

    return ( \%op_hash );
}


=head2 get_oem_tbl_conv_code_list()

Returns the list of convertible code for a given

volume language and list type, from OEM translation table set.

=over 4

=item Arguments:

=over 4

=item vol_lang

Name of volume language like English, Arabic etc.

=item conv_type

Type of conversion [ owc_to_nwc or nwc_to_owc ]

=over 4

owc_to_nwc : if data set will be created on Old Window client(like Windows95)

nwc_to_owc : if data set will be created on New Windows client(like Windows XP)

=back

=item table_id [optional param]

Table id of conversion table. if not given, then the same

will be found out using vol_lang and tbl_type field.

=back

=item Returns:

Returns an array of convertible code, if successful

OR

Returns undef, in case of error

=back

=cut

sub get_oem_tbl_conv_code_list {

    my (%options) = validate(@_,
                        {
                            'vol_lang' =>
                                {
                                    type => SCALAR
                                },
                            'table_id' =>
                                {
                                    type     => SCALAR,
                                    optional => 1
                                },
                            'conv_type' =>
                                {
                                    type  => SCALAR,
                                    regex => qr/^(owc_to_nwc|nwc_to_owc)$/
                                },
                        } );


    my ($vol_lang);  # input vol lang name
    my ($table_id);  # input table id
    my ($conv_type); # input conversion type
    my ($sup_vol);   # string with supported vol lang name
    my ($vld_tbl_ids);    # string with valid table id
    my (%oem_conv_table); # table to be returned
    my (@conv_code_list); # returned list of convertible code

    $vol_lang  = $options{'vol_lang'};
    $table_id  = $options{'table_id'};
    $conv_type = $options{'conv_type'};

    #
    # get the table_id from vol_lang, if not given
    #
    if ( ! defined($table_id) ) {
        if ( ! $supported_vol_type_list{$vol_lang} ) {
            $sup_vol  = get_supported_vol_list();
            logWarn("Invalid vol_lang: $vol_lang,"
                . " correct langs:\[ $sup_vol \]");
            return UC_ERROR;
        }
        $table_id = $lang_conv_table_id{$vol_lang}{'oem_char_set'};
    }

    #
    # validate the table_id key exists
    #
    if ( ! exists($g_OEM_Conv_Tables{$table_id}) ) {
        $vld_tbl_ids = get_conv_table_id_list('tbl_type' => 'oem');
        logWarn("Invalid OEM conv_table_id:$table_id,"
            . " correct ids:\[ $vld_tbl_ids \]\n");
        return UC_ERROR;
    }

    #
    # let's point to the correct translation table
    #
    %oem_conv_table = %{$g_OEM_Conv_Tables{$table_id}};

    if ( $conv_type eq "owc_to_nwc" ) {
        @conv_code_list = keys %oem_conv_table;
    } else {
        # conv_type is nwc_to_owc
        @conv_code_list = values %oem_conv_table;
    }

    return @conv_code_list;
}


=head2 is_oem_tbl_code_convertible()

Returns true, if input code is convertible(code has an entry

in OEM translation table), Otherwise false.

=over 4

=item Arguments:

=over 4

=item code

An integer number from 128 to 65536 range

=item vol_lang

Name of volume language like English, Arabic etc.

=item conv_type

Type of conversion [ owc_to_nwc or nwc_to_owc ]

=over 4

owc_to_nwc : if data set will be created on Old Window client(like Windows95)

nwc_to_owc : if data set will be created on New Windows client(like Windows XP)

=back

=item table_id [ optional param ]

Translation table id, if not given, then same will

be found out using vol_lang.

=back

=item Returns:

Returns 1, if successful

OR

Returns 0, if failed

OR

Returns undef, in case of error

=back

=cut

sub is_oem_tbl_code_convertible {

    my (%options) = validate(@_,
                        {
                            'code' =>
                                {
                                    type => SCALAR,
                                    regex => qr/^\d+$/
                                },
                            'vol_lang' =>
                                {
                                    type => SCALAR
                                },
                            'table_id' =>
                                {
                                    type => SCALAR,
                                    optional => 1
                                },
                            'conv_type' =>
                                {
                                    type  => SCALAR,
                                    regex => qr/^(owc_to_nwc|nwc_to_owc)$/
                                },
                        } );

    my ($code);       # input code which has to be checked
    my ($vol_lang);   # input vol lang name
    my ($table_id);   # input table id
    my ($conv_type);  # input conversion type
    my (%conv_table); # valid conv table hash
    my ($vld_tbl_ids);# string with valid table ids
    my ($sup_vol);    # string with supported vol lang name
    my (@conv_code_list); # list of convertible code

    $code      = $options{'code'};
    $vol_lang  = $options{'vol_lang'};
    $table_id  = $options{'table_id'};
    $conv_type = $options{'conv_type'};

    #
    # get the table_id from vol_lang, if not given
    #
    if ( ! defined($table_id) ) {
        if ( ! $supported_vol_type_list{$vol_lang} ) {
            $sup_vol  = get_supported_vol_list();
            logWarn("Invalid vol_lang: $vol_lang,"
                        . " correct langs:\[ $sup_vol \]");
            return UC_ERROR;
        }
        $table_id = $lang_conv_table_id{$vol_lang}{'oem_char_set'};
    }

    #
    # validate the table_id
    #
    if ( ! exists($g_OEM_Conv_Tables{$table_id}) ) {
        $vld_tbl_ids = get_conv_table_id_list('tbl_type' => 'oem');
        logWarn("Invalid OEM conv_table_id: $table_id,"
                    . " correct ids:\[ $vld_tbl_ids \]\n");
        return UC_ERROR;
    }

    #
    # get the correct translation table
    #
    if ( $conv_type eq "owc_to_nwc" ) {
        %conv_table = %{$g_OEM_Conv_Tables{$table_id}};
    } else {
        # conv_type is nwc_to_owc
        %conv_table = reverse %{$g_OEM_Conv_Tables{$table_id}};
        $code = get_cifs_translated_code($code);
    }

    return exists($conv_table{$code}) ? UC_TRUE : UC_FALSE;
}


=head2 get_oem_tbl_conv_and_nonconv_code_list()

This interface separate out the convertible and non_convertible codes

from the given code ranges.

=over 4

=item Arguments:

=over 4

=item code_list

Reference to input code range array

=item vol_lang

Name of volume language like English, Arabic etc.

=item conv_type

Type of conversion table [ owc_to_nwc or nwc_to_owc ]

=over 4

owc_to_nwc : if data set will be created on Old Window client(like Windows95)

nwc_to_owc : if data set will be created on New Windows client(like Windows XP)

=back

=item table_id [optional param]

Translation table id, if not given, then same will

be found out using vol_lang.

=back

=item Returns:

Returns a reference to hash(say $rh) with key conv and non_conv.

=over 4

$rh{conv} : a reference to convertible code array

$rh{non_conv} : a reference to convertible code array

=back

OR

Returns undef, in case of error

=back

=cut

sub get_oem_tbl_conv_and_nonconv_code_list {

    my (%options) = validate(@_,
                        {
                            'code_list' =>
                                {
                                    type => ARRAYREF,
                                },
                            'vol_lang' =>
                                {
                                    type => SCALAR
                                },
                            'table_id' =>
                                {
                                    type => SCALAR,
                                    optional => 1
                                },
                            'conv_type' =>
                                {
                                    type  => SCALAR,
                                    regex => qr/^(owc_to_nwc|nwc_to_owc)$/
                                },
                        } );


    my (@codeList); # input code which has to be checked
    my ($vol_lang); # input vol lang name
    my ($table_id); # input conv table id
    my ($conv_type);   # input conv table type
    my (%conv_table);  # valid conv table hash
    my ($vld_tbl_ids); # string with valid table ids
    my ($sup_vol); # string with supported vols lang name
    my (@conv_code_list); # list of convertible code
    my (@nonconv_code_list); # list of non convertible code
    my (%op_hash); # returned hash of conv and nonconv list

    @codeList  = @{$options{'code_list'}};
    $vol_lang  = $options{'vol_lang'};
    $table_id  = $options{'table_id'};
    $conv_type = $options{'conv_type'};

    # add returned list to output hash
    $op_hash{'conv'} = \@conv_code_list;
    $op_hash{'non_conv'} = \@nonconv_code_list;

    #
    # findout the table_id from vol_lang,
    # if it is not defined
    #
    if ( ! defined($table_id) ) {
        if ( ! exists($supported_vol_type_list{$vol_lang}) ) {
            $sup_vol  = get_supported_vol_list();
            logWarn("Invalid vol_lang: $vol_lang,"
                        . " correct types:\[ $sup_vol \]");
            return UC_ERROR;
        }
        $table_id = $lang_conv_table_id{$vol_lang}{'oem_char_set'};
    }

    #
    # validate the table_id
    #
    if ( ! exists($g_OEM_Conv_Tables{$table_id}) ) {
        $vld_tbl_ids = get_conv_table_id_list('tbl_type' => 'oem');
        logWarn("Invalid OEM conv_table_id:$table_id,"
                    . " correct ids:\[ $vld_tbl_ids \]");
        return UC_ERROR;
    }

    #
    # get the convertible and non-convertible code
    #
    if ( $conv_type eq "owc_to_nwc" ) {
        %conv_table = %{$g_OEM_Conv_Tables{$table_id}};
        foreach my $code ( sort @codeList ) {
            if ( exists($conv_table{$code}) ) {
                push(@conv_code_list, $code);
            } else {
                push(@nonconv_code_list, $code);
            }
        }
    } else {
        # conv_type is nwc_to_owc
        %conv_table = reverse %{$g_OEM_Conv_Tables{$table_id}};

        foreach my $code ( sort @codeList ) {
            my $trns_code = get_cifs_translated_code('code' => $code);
            if ( exists($conv_table{$trns_code}) ) {
                push(@conv_code_list, $code);
            } else {
                push(@nonconv_code_list, $code);
            }
        }
    }

    return ( \%op_hash );
}


=head2 get_conv_table_id_list()

Returns the list of valid translation table ids for a given

table type.

=over 4

=item Arguments:

=over 4

=item tbl_type

Type of conversion table [ nfs or oem ]

=back

=item Returns:

Returns an array of table ids, if called in array context

OR

Returns a string of space separated table ids, if called in scalar context

OR

Returns undef, in case of error

=back

=cut


sub get_conv_table_id_list {

    my (%options) = validate(@_,
                        {
                            'tbl_type' => 
                                {
                                    type  => SCALAR,
                                    regex => qr/^(nfs|oem)$/,
                                },
                        } );

    my ($tbl_type);    # type of conv table, nfs|oem
    my (@tableIdList); # list of id to be returned
    
    $tbl_type = $options{'tbl_type'};

    if ( $tbl_type eq "nfs" ) {
        @tableIdList = keys %g_NFS_Conv_Tables;
    } elsif ($tbl_type eq "oem" ) {
        @tableIdList = keys %g_OEM_Conv_Tables;
    } else {
        logWarn("Invalid tbl_type: $tbl_type, correct types: cifs|oem");
        return UC_ERROR;
    }

    return wantarray ? @tableIdList : "@tableIdList";
}

=head2 is_valid_conv_table_id()

Returns true, if input translation table id is valid for given

table type, Otherwise false.

=over 4

=item Arguments:

=over 4

=item tbl_type

Type of conversion table [ nfs or oem ]

=item table_id

Translation table id

=back

=item Returns:

Returns 1, if successful

OR

Returns 0, if failed

=back

=cut

sub is_valid_conv_table_id {

    my (%options) = validate(@_,
                        {
                            'tbl_type' =>
                                {
                                    type  => SCALAR,
                                    regex => qr/^(nfs|oem)$/
                                },
                            'table_id' =>
                                {
                                    type => SCALAR
                                },
                        } );


    my ($tbl_type); # type of conv table set, nfs|oem
    my ($table_id); # input table id

    $tbl_type = $options{'tbl_type'};
    $table_id = $options{'table_id'};

    if ( $tbl_type eq "nfs" ) {
        return exists($g_NFS_Conv_Tables{$table_id}) ? UC_TRUE : UC_FALSE;
    } elsif ($tbl_type eq "oem" ) {
        return exists($g_OEM_Conv_Tables{$table_id}) ? UC_TRUE : UC_FALSE;
    }
}


=head2 get_vol_list_by_nfs_conv_table_id()

Returns the list of supported volumes for a given translation

table id.

=over 4

=item Arguments:

=over 4

=item table_id

Translation table id

=back

=item Returns:

Returns an array of supported volumes

OR

Returns undef, in case of error

=back

=cut

sub get_vol_list_by_nfs_conv_table_id {

    my (%options) = validate(@_,
                        {
                            'table_id' => {type => SCALAR}
                        } );

    my ($table_id); # input table id
    my (@vol_list); # returned vol list

    $table_id = $options{'table_id'};

    #
    # validate the table_id
    #
    if ( ! $g_NFS_Conv_Tables{$table_id} ) {
        my $vld_tbl_ids = get_conv_table_id_list('tbl_type' => 'nfs');
        logWarn("Invalid NFS conv_table_id:$table_id,"
                    . " correct ids:\[ $vld_tbl_ids \]");
        return UC_ERROR;
    }

    #
    # Retrieve the vol from lang_conv_table_id
    #
    foreach my $vol ( keys %lang_conv_table_id ) {
        if ( $lang_conv_table_id{$vol}->{'nfs_char_set'} eq $table_id ) {
            push(@vol_list, $vol);
        }
    }

    return @vol_list;
}


=head2 get_vol_list_for_all_nfs_conv_table_ids()

Returns supported volumes lists for all nfs translation table ids

=over 4

=item Arguments:

None

=item Returns:

Returns a referece hash which has table ids as a key,

and a reference array of supported volumes

=back

=cut

sub get_vol_list_for_all_nfs_conv_table_ids {

    my %op_vol_hash; # returned output vol hash

    #
    # Get all table ids
    #
    my @tableIds = get_conv_table_id_list('tbl_type' => 'nfs');

    #
    # Retrieve the volumes supported by each table id
    #
    foreach my $id (@tableIds) {
        my @vol_list = get_vol_list_by_nfs_conv_table_id('table_id' => $id);
        $op_vol_hash{$id} = \@vol_list;
    }

    return (\%op_vol_hash);
}


=head2 create_non_utf8_file_name()

Returns the name of file name for the given nfs code.
This function is useful to test volume language supporting
multibyte encoding (for example: Japanese, Korean etc)

=over 4

=item Arguments:

=over 4

=item nfs_code

Value of nfs code

=back

=item Returns:

Returns a string having proper file name

=back

=cut

sub create_non_utf8_file_name {

    my (%options) = validate(@_,
                        {
                            'nfs_code' => {type => SCALAR}
                        } );

    my ($nfsCode) = $options{'nfs_code'};

    my $hex = sprintf("%lx",$nfsCode);

    my @tmp = split('', $hex);

    my $elmCnt = scalar(@tmp);

    if ( $elmCnt%2 != 0 ) {
        unshift @tmp, "0";
    }

    my (@hex);

    for ( my $i = 0; $i <= $#tmp; $i=$i+2 ) {
        push(@hex, "0x$tmp[$i]$tmp[$i+1]");
    }

    my $chr_string = join('', map { chr( eval($_) ) } @hex );

    my $fileName = "$nfsCode-$chr_string-$nfsCode.txt";

    return $fileName;
}


1;

=head1 SEE ALSO

    OEM_Conv_Tables.pm

    NFS_Conv_Tables.pm

    http://wikid.netapp.com/w/Unicode#What_is_the_language_setting_used_for

=head1 STATUS

    Shared

=head1 AUTHOR

    Tarun Garg (tarung@netapp.com)
    $Id: //depot/prod/test/Rsierranevada/lib/Unicode.pm#1 $
    Last $Change: 1917575 $
    Last Revised $DateTime: 2013/01/22 11:22:46 $

=head1 HISTORY

    2009-08-19 author: tarung      original lib created

=head1 COPYRIGHT

Copyright 2009, Network Appliance., All Rights Reserved

=cut

