# Hey, emacs, this is basically -*-perl-*-
#$Id: //depot/prod/test/Rsierranevada/lib/Upgrade/Upgrade.pm#1 $
#
#Copyright (c) 2004 Network Appliance, Inc.
#All rights reserved.
#
## @summary     Upgrade library (OBSOLETE)
##
## @description
##      This library provides access to all upgrade objects , methods,
##      and internal values associated with these objects.
##
## @synopsis
##      use Upgrade::Upgrade;
##      my $r = new Upgrade( $c, "6.5" , "RUNNING");
##      where $c is a valid NATE connection object and $r is a
##      reference to a Upgrade object (described below.) "RUNNING" is the
##	version we wish to upgrade from
##
## @example
##      Before using this library, please run Example/test_upgrade.thpl.
##      It prints out all attributes that objects know about.
##
## @usage
##
##
## <hr><center><b>Upgrade Object</b></center>
##
## <p>Creating a new object of type Upgrade queries
## the filer about its current version and config information and creates all
## of the objects necessary to do the upgrade.</p>
## <pre>
##    $r = new Upgrade( $c, "6.5" , "RUNNING");
## </pre>
## where $c is a valid NATE connection object. 6.5 is the release we want to
## upgrade to
## <pre>
##    $r->do_upgrade();
## </pre>
##
## @change      10/05/2004 conor: First pass
##
## @status obsolete -- use Filer::Upgrade
##
## @author shenry@netapp.com (original conor@netapp.com)

package Upgrade;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use Sysconfig;
use Tharn;
use SPARSE::Util::Logger;
use FS::Util::Setup qw(mount_root_vol create_vol);
use SayWrap qw(sayEOF);
use Carp;
use filer qw(filer_version_get);
use Volume qw(vol_root vol_status);
use Data::Dumper;
use mount qw(mount umount);
#use mount qw(mount umount);
use FS::Util::Utilities qw(uSystem);
use boot;



@ISA    = qw();
@EXPORT = qw(new test_hardware_allows test_config_allows test_running_status
		test_version_allows do_upgrade v_compare max_snaps
		my_create_vol_and_mount do_reboot);

$VERSION = '0.1';

# ---------------------
# sub new()
# Constructor requires a connection record as the first argument
# All optional parameters are supplied in a named fashion. If not
# supplied, the "to_version" argument will default to undef
# ---------------------
sub new {
    my $class = shift;
    my $conn = shift;
    my $to_version = shift;
    my $from_version = shift;
    my %parms = @_;

    my $from_version_long = $from_version;
    # first check if we need to find out the version we are upgrading from
    if ($from_version eq "RUNNING") { # normally FROM_VERSION is set to 'RUNNING'
  	$from_version = _current_version($class, $conn); #get the version from filer
	logDbg("Upgrade::new -- from_version = $from_version");
  	$from_version_long = $from_version;
	logDbg("RUNNING version is $from_version");
  	if ($from_version =~ /\//) {
	    # note that some results from _current_version will not enable us to
	    # know the version in x.x form. e.g /u/conor/mytestbuild/040649N
	    # impossible to tell if is 6.3, 6.5 or 7.1 etc
	    # we could try to get clever by mapping the beer in the pathname
	    # to the version but then sombody would have a path 'as' for
	    # anchorsteam. So simply remove everything up to the last '/'
	    # parse for x.x and if we cannot succeed the user must
	    # explicitly pass in the FROM_VERSION (instead of using 'RUNNING')
	    #UPDATE Sam Rafter reckons we can tell by issuing a revert_to
	    # XXX TBD CD
	    $from_version =~ s/.*\/(.*)/$1/;
	    $from_version_long = $from_version;
  	}

	# strip off any extra chars beyond x.x
      	logDbg("RUNNING version is $from_version");
	if (!($from_version =~ s/.*?(\d\.\d).*/$1/)) {
	    logWarn("the running version is $from_version, it is not "
		    ." possible to tell if this is 6.3, 6.4 or 7.1 etc."
		    ." Please supply param FROM_VERSION=x.x e.g x.x=6.6");
	    return 0;
	}
    } else {
	$from_version =~ s/.*?(\d\.\d).*/$1/; #reduce to x.x; used for most purposes
    }
    # else the version has been supplied as a param
    return 0 unless $to_version;
    return 0 unless $from_version;
    my $to_version_long=$to_version;
    $to_version=substr($to_version_long,0,3);
    # at this stage from_version should be something like x.x,
    # from_version_long can be formatted in a number (e.g 6.3.2RN) of ways
    # but must have an x.x substr. to_version must be x.x.x or x.x.
    # To specify a release which cannot be deduced from the the TO_VERSION
    # set the env var RELEASEVERSION (still supply TO_VERSION)
    logDbg("from_version is $from_version to_version is $to_version.");

    bless {
	   conn          => $conn,
	   to_version    => $to_version,
	   to_version_long    => $to_version_long,
	   from_version  => $from_version,
	   from_version_long  => $from_version_long,
	   FILER         => $parms{FILER} || $main::FILER,
	   rootvol_mount => $parms{rootvol_mount} || undef,
	   mount         => undef
	  }, $class;
}

# --------------------
# sub allowed
# The upgrade is only allowed if four specific "tests" pass:
# (1) version
# (2) hardware
# (3) config
# (4) run status
# --------------------
sub allowed {
    my $self = shift;

    if ($self->test_version_allows()   &&
	$self->test_hardware_allows()  &&
	$self->test_config_allows()    &&
	$self->test_running_status_allows() ) {
	return 1
    } else {
	return 0
    }
}

# --------------------
# sub test_version_allows
# member test_version_allows checks whether the "to_version" that's been
# requested is compatible with the acceptable previous versions
# returns 1 if allowed 0 if not
# --------------------
sub test_version_allows {
    my $self = shift;

    # first test that that we are actually upgrading, that the to_version is a
    # bigger number than the from_version
    if (v_compare($self->{to_version_long}, $self->{from_version_long}) != 1) {
	logWarn("from_version ($self->{from_version_long}) is >= to_version ($self->{to_version_long}). Not an upgrade!");
	return 0;
    }

    # to quote Mary Vetter
    # systems with PC cards had to go through another release, before they
    # could be upgraded to Anchorsteam.  The release that the systems have
    # to go through is 6.4.2.  The firmware was upgraded that release to FAT
    # format and the system has to be running firmware 4.2.2 or later to be
    # able to upgrade beyond that point.

    my $to_version5 = _str_5_version($self->{to_version_long});
    my $from_version5 = _str_5_version($self->{from_version_long});
    logDbg("str_5_versions: from $from_version5 to $to_version5");

    if ( (v_compare("6.42", $from_version5)==1) &&
	 (v_compare($to_version5, "6.42")==1) ) {
	my ($match, undef, $after) = say($self->{conn},
					 "sysconfig -v",
					 "Firmware rev:"
					 ,'-eval_error');
	$after =~ m/.*?\s+(.?\d+)\.(.?\d+)\.(.?\d+).*/;
        logDbg("1 is $1 2 is $2 3 is $3");
	if ($1 < 4) {
	    logWarn("For this upgrade you must upgrade to release 6.4.2 first");
	    return 0;
	}
	if ( ($1 == 4) && ($2 < 2) ) {
	    logWarn("For this upgrade you must upgrade to release 6.4.2 first");
	    return 0;
	}
	if ( ($1 == 4) && ($2 == 2) && ($3 < 2) ) {
	    logWarn("For this upgrade you must upgrade to release 6.4.2 first");
	    return 0;
	}
    }
    # otherwise all is well for upgrade
    return 1;
}

#get the version to x.x.x level of precision - needed to check for 6.4.2
sub _str_5_version {
    my $self = shift if ref $_[0];
    my $version = shift;
    my $versiontmp="";

    if ($version =~ /.*(\d\.\d\.\d).*/) {
	$versiontmp = $1;
        $version = substr($versiontmp,0,3);
        $version .= substr($versiontmp,4,1);
    } elsif ($version =~ /.*(\d\.\d).*/) {
	$versiontmp = $1;
        $version = substr($versiontmp,0,3);
        $version .= "0";
    }
    return $version;
}


# --------------------
# sub test_hardware_allows
# --------------------
sub test_hardware_allows {
    my $self = shift;

    # It would certainly be more logical, for the moment, if we let a
    # Sysconfig object be its own object, not roll it into the upgrade object,
    # so why don't the following two lines of code work?
    # (Note: whether or not Sysconfig is in @ISA doesn't affect the error)
    #
    #  my $cfg = Sysconfig->new($self->{conn});
    #  print Dumper($cfg);
    #|| croak $@;
    # $cfg->_init;

    # Sysconfig expects that the connection record will be found under
    # hash key "filer" not hash key "conn", so make it happy
    #  $self->{filer} = $self->conn && $self->_init;

    return 1;
}

# --------------------
# sub test_config_allows
# --------------------
sub test_config_allows {
    my $self = shift;
    return 1;
}

# --------------------
# sub test_running_status_allows
# --------------------
sub test_running_status_allows {
    my $self = shift;
    return 1;
}

# --------------------
# sub v_compare
# v_compare is a class method that takes two netapp version strings
# and converts them to a representative integer. If the first version is
# higher than the second version, v_compare returns 1. If they are equal, it
# returns 0. If the first version is lower than the second version, it returns
# -1
# See the documentation for _integer_version for more information about
# acceptable string formats
# Currently, it croaks if the format is wrong. This needs to be fixed.
# --------------------
sub v_compare {
    my $self = shift if ref $_[0];
    my ($v1, $v2) = @_;
    my $result = _integer_version($v1) - _integer_version($v2);
    if ($result > 0 ) {
	return 1
    } elsif ($result < 0 ) {
	return -1
    } elsif ($result == 0) {
	return 0
    } else {
	return undef
    }
}
#find out the max number of snapshots allowed for this filer version
sub max_snaps {
    my $self = shift if ref $_[0];
    if ($self->v_compare($self->{from_version}, "6.4") == 1) {
	return ("255");
    }
    if ($self->v_compare($self->{from_version}, "6.4") == 0) {
	return ("255");
    }
    if ($self->v_compare($self->{from_version}, "6.3") == 1) {
	return ("32");
    }
    return 0;
}
# --------------------
# sub _integer_version
# Versioning follows very strict rules here
# (1) Versions are split on a single period
# (2) No letters or other characters are allowed
# (3) Only the major and minor digits are significant
# (4) A partial version of .11 is greater than a partial version of .2
# but a partial version of .20 is greater than a partial version of .11
# (i.e. right-padded zeroes are significant)
# --------------------
sub _integer_version {
    my $self = shift if ref $_[0];
    my $version = shift;

    my @partials = (split /\./, $version)[0..2];
    unless ($partials[0] >= '7' && $partials[1] =~/^\d+RC\d+$|^\d+X\d+$/) {
	croak "Version '$version' contains an illegal character"
	  if (grep /\D/, @partials);
    }
    my $intvers = join "", map { sprintf("%06d", $_) } @partials;
    logDbg("_integer_version($version) => $intvers (partials=@partials)");
    return ( $intvers ) ||
      croak "Could not split version '$version' into partials";
}

# --------------------
# sub _current_version
# If the method already knows what version it is
# This retrieves and returns what the filer thinks its current version is
# --------------------
sub _current_version {
    my $self = shift;
    my $conn = shift;

    return filer_version_get($conn);
}

# this sub is almost identical to Setup.pm's create_vol_and_mount. The reason I
# have made a copy is so that I can experiment with how best to incorporate
# testing upgrade with simulators which requires a boot to get the console
# eventually any diff will be merged into create_vol_and_mount and this will go
sub my_create_vol_and_mount {
    my (%opts) = @_;

    my $FILER           = $opts{filer}         || $main::FILER;
    my $CLIENT          = $opts{client}        || $main::CLIENT;
    my $FILER_CONN      = $opts{filer_conn}    || $main::FILER_CONN;
    my $FILER_VOL       = $opts{filer_vol}     || $main::FILER_VOL;
    my $FILER_IP        = $opts{filer_ip}      || $main::FILER_IP;
    my $FILER_VOL_MNT   = $opts{filer_vol_mnt} || $main::FILER_VOL_MNT;
    my $MOUNTPOINT      = $opts{mountpoint}    || $main::MOUNTPOINT;
    my $VOL_TYPE        = $opts{vol_type}      || $main::VOL_TYPE;
    my $VOL_SIZE        = $opts{vol_size}      || $main::VOL_SIZE;
    my $VOL_DISKS       = $opts{vol_disks}     || $main::VOL_DISKS;
    my $FILER_AGGR      = $opts{filer_aggr}    || $main::FILER_AGGR;
    my $AGGR_SIZE       = $opts{aggr_size}     || $main::AGGR_SIZE;
    my $TRAD_SIZE       = $opts{trad_size}     || $main::TRAD_SIZE;
    my $EXTERN_SETUP    = $opts{extern_setup}  || $main::EXTERN_SETUP;
    my $TIMEOUT         = $opts{timeout}       || $main::TIMEOUT;
    my $CMDTIMEOUT      = $opts{cmdtimeout}    || $main::CMDTIMEOUT;
    return eval {

        my $vol = $FILER_VOL;
        my $vol_path = $FILER_VOL_MNT || (length($FILER_VOL) ?
                                          "/vol/$FILER_VOL" : "/");
        # filer connection
        my $fh = host($FILER);	#thats how we access
        my $fc="";
        if ($fh->hosttype() eq 'filer-simulator') {
	    $fc = connect('#console',-timeout, $TIMEOUT);
	    $fc->timeout($CMDTIMEOUT);
	    # boot so that we have a console!
	    boot($fc, timeout=>$TIMEOUT);
	    $@="";		#or die $@;
        } else {
	    $fc = connect($FILER_CONN, -timeout, $TIMEOUT);
        }
        $fc->timeout($CMDTIMEOUT);
        my $hostname = $FILER_IP || $fh->default_ip();
        my $mntpt = $MOUNTPOINT || "/t/$hostname/$vol";
        my $hostrec = host($FILER);
        my $pfiler = $hostrec->{PFILER};

        # client connection
        my $ch = host($CLIENT);
        # XXX/tapl Arvind advocate to always use #rsh for clients
        my $cc = connect('#rsh', -timeout, $TIMEOUT);
        $cc->timeout($CMDTIMEOUT);
        if (!$pfiler) {
            my ($fc, $vol) = create_vol(%opts)
	      or die $@;
            logDbg("create_vol_and_mount() - create_vol() " .
                   "returned: fc:($fc), vol:($vol)");
        } else {
            logTrc("vfiler $FILER running on $pfiler , NO volume setup done");
        }

        logTrc("attempt to mount volume $vol");

        $ENV{OLD_MOUNT_LIB} ?
	  mount::umount($mntpt) :
	      mount::umount($mntpt,client=>$CLIENT);

        my $vol_mount_dir =
	  $ENV{OLD_MOUNT_LIB} ?
            mount::mount($hostname, $vol_path,
                             mountpoint=>$mntpt,
                             mountopts=>"actimeo=0,noac,hard,intr,noquota")
		:
		  mount::mount($hostname, $vol_path, client=>$CLIENT,
			       mountpoint=>$mntpt,
			       mountopts=>"actimeo=0,noac,hard,intr,noquota")
		      or die($@);

        logTrc("Mounted volume $vol_path on $mntpt");

        logTrc("chmod perms on test dir $vol_mount_dir");
        system("sudo chmod 0777 $vol_mount_dir") == 0
	  or die("chmod $vol_mount_dir failed");

        logDbg("create_vol_and_mount returning $fc, $vol, $vol_mount_dir");
        return ($fc, $vol, $vol_mount_dir);
    };
}

# --------------------
# sub determine_archive_path
# Based on the hardware config and to_version, determine which
# archive to untar
# --------------------
sub determine_archive_path {
    my $self = shift;

    my $versionforpath = $self->{to_version_long};
    #if ($self->{to_version_long} =~ /^7/) {
	#CDCORRECT$versionforpath .= "X1";
	#$versionforpath .= "X10";
    #}
    # in case the user wants a specific release with an odd name
    $versionforpath = $ENV{RELEASEVERSION} if (defined($ENV{RELEASEVERSION}));

    my $archive = "/rlse/$versionforpath".
      "/releases/".
	"pc/dist/tar_floppy_pc";
    return $archive;
}

# --------------------
# sub install_release
# --------------------
sub install_release {
    my $self = shift;
    my %parms = @_;

    my $rootvol_mount ="";
    if (defined($self->{rootvol_mount})) {
  	$rootvol_mount= $self->{rootvol_mount};
    } else {
	$rootvol_mount = mount_root_vol($self->{FILER});
  	$self->{rootvol_mount} = $rootvol_mount;
    }
    my $mount = $rootvol_mount;
    $self->{mount} = $rootvol_mount;

    my $archive = $self->determine_archive_path;
    my $output;

    logDbg "pre-clean for old tar files";
    $output = uSystem("cd $mount && rm -f tar_image* install_netapp");
    logDbg "PROCEEDING TO UNTAR $archive: ";
    $output = uSystem("cd $mount && sudo tar -xvf $archive");
    logDbg "tar $output \n";

    logDbg "PROCEEDING TO INSTALL in $mount: $@";
    $output = uSystem("cd $mount && sudo ./install_netapp $mount");
    logDbg "FINISHED INSTALL output $output: $@\n";

    logDbg "PROCEEDING TO DOWNLOAD: $@";
    my ($match, undef, $after) = say($self->{conn}, "download",
				"boot blocks successfully copied to disk"
				,'-eval_error');
    logDbg("after is $after");

    #some tests need to do the reboot separately from the install and download
    return 1 if ($parms{REBOOT} eq 'NO');

    logDbg "PROCEEDING TO REBOOT: $@";
    my $filer = $self->{FILER};
    my $fhost = host($filer);
    reboot_to($fhost, UP, timeout=>'1200') or die $@;
    logStep("brought the system back up");

    return 1;
}

sub do_reboot {
    my $self = shift;
    return 1 if ($ENV{DONOTREALLYUPGRADE} == 1);
    logDbg "PROCEEDING TO REBOOT: $@";
    my $filer = $self->{FILER};
    my $fhost = host($filer);
    reboot_to($fhost, UP, timeout=>'1200') or die $@;
    logStep("brought the system back up");
    return 1;
}

# --------------------
# sub do_upgrade
# --------------------
sub do_upgrade {
    my $self = shift;
    my %parms = @_;

    $self->install_release(@_) if ($ENV{DONOTREALLYUPGRADE} != 1);
    return 1;
}

1;
