# $Id: //depot/prod/test/Rsierranevada/lib/Utils.pm#1 $

## @summary misc routines for nate script
##
## @description
##      Utils.pm functions
##
## @status public
##
## @author dl-test-infra@netapp.com
##
## @keywords Utils

package Utils;
use strict;
use base qw(Exporter);
use Cwd qw(getcwd);
use Carp qw(croak);
use Tharn;
use Params::Validate qw(validate validate_pos SCALAR OBJECT);
BEGIN
{
  our @EXPORT = qw(
    filertime
    time2sec
    time2min
    time2hour
    make_conn
    make_conn_and_complain
    make_host
    make_host_and_complain
    to_sec
    absolute_testname
    relative_testname
    subtest_local
    subtest_remote
  );
}


sub filertime($;%) {
	my ($fc, %args) = @_;

	my (undef, undef, $after) = say($fc, "date");

	if (!defined($args{format})) {
		return ($after);
	}

	if ($args{format} =~ /second/i) {
		return time2sec($after);
	} elsif ($args{format} =~ /minute/i) {
		return time2min($after);
	} elsif ($args{format} =~ /hour/i) {
		return time2hour($after);
	}

	return ($after);
}

sub time2sec($) {
	my ($strtime) = @_;
	my ($fdate, $fhour, $fmin, $fsec) =
			($strtime =~ /(\d+)\s+(\d+):(\d+):(\d+)/);
	return (($fdate * 24 + $fhour) * 60 + $fmin) * 60 + $fsec;
}

sub time2min($) {
	my ($strtime) = @_;
	my ($fdate, $fhour, $fmin) = ($strtime =~ /(\d+)\s+(\d+):(\d+)/);
	return ($fdate * 24 + $fhour) * 60 + $fmin;
}

sub time2hour($) {
	my ($strtime) = @_;
	my ($fdate, $fhour, $fmin) = ($strtime =~ /(\d+)\s+(\d+):(\d+)/);
	return $fdate * 24 + $fhour;
}

sub make_host
{
  my($spec) = @_;
  if (UNIVERSAL::isa($spec, 'Connectrec')) {
    return $spec->hostp();
  } elsif (UNIVERSAL::isa($spec, 'Hostrec')) {
    return $spec;
  } else {
    return host($spec);
  }
}

# XXX - make_host() with warnings. used in places where by popular
# demand make_host()-ish behavior (which, IMHO is a bad thing) is needed.
# complain loudly if anything other than a Connectrec is used
sub make_host_and_complain
{
  my($spec, $conn) = validate_pos(@_,
    { type => SCALAR|OBJECT,
      callbacks => {
        'spec or Hostrec or Connectrec' => sub {
           !ref($_[0]) ||
           UNIVERSAL::isa($_[0], 'Hostrec') ||
           UNIVERSAL::isa($_[0], 'Connectrec')
        }
      }
    }
  );
  # complain loudly if $spec is not a Connectrec
  if( !UNIVERSAL::isa($spec, 'Hostrec') ) {
    my($filename, $line) = (caller(2))[1,2];
    Tharn::logresult(
      type => 'WARN',
      msg  => "$filename line $line: Use of hostspec or Connectrec object instead of Hostrec is not recommended. Use host() before calling this function!"
    );
  }
  make_host($spec, $conn);
}

## @name ::make_conn()

## @synopsis make_conn ($spec [, $conn]);

## @arg $spec Required: Name of host, Hostspec, Connspec
## @arg $conn Optional: Name of connection
##            to choose if not specified
##            Default: "#rsh"

## @description make a connection

## @returns a connection to the host

## @status public

sub make_conn ( $;$ ) {

  my ($spec, $conn) = @_;

  return eval {
    $conn = "#rsh" if (!$conn);
    if (ref($spec) =~ /Connectrec/) {
      return $spec;
    } elsif (ref($spec) =~ /Hostrec/) {
      return connect($spec->{id} . $conn);
    } else {
      my $hspec=host($spec);
      return connect($hspec->{id} . $conn);
    }
  };
}

# XXX - make_conn() with warnings. used in places where by popular
# demand make_conn()-ish behavior (which, IMHO is a bad thing) is needed.
# complain loudly if anything other than a Connectrec is used
sub make_conn_and_complain
{
  my($spec, $conn) = validate_pos(@_,
    { type => SCALAR|OBJECT,
      callbacks => {
        'spec or Hostrec or Connectrec' => sub {
           !ref($_[0]) ||
           UNIVERSAL::isa($_[0], 'Hostrec') ||
           UNIVERSAL::isa($_[0], 'Connectrec')
        }
      }
    },
    { type => SCALAR, optional => 1, default => '#rsh' }
  );
  # complain loudly if $spec is not a Connectrec
  if( !UNIVERSAL::isa($spec, 'Connectrec') ) {
    my($filename, $line) = (caller(2))[1,2];
    Tharn::logresult(
      type => 'WARN',
      msg  => "$filename line $line: Use of hostspec or Hostrec object instead of Connectrec is not recommended. Use host() and connect() before calling this function!"
    );
  }
  make_conn($spec, $conn);
}

## @name ::to_sec

## @description convert to seconds
## format has to be like
## 20, 20s, 20m, 20h, 20d

## @synopsis to_sec($num)

## @notes does not take care of decimal

## @returns time in sec (in case of -1 will return as it is)

sub to_sec($) {

    my ($num) = validate_pos( @_,
            { type=>SCALAR, optional=>0, regex=>qr/^-?\d+[dhms]?$/i }
            );

    return($num) if ($num eq -1);
    $num = lc($num);
    my $sec;
    $num =~ /^(\d+)([dhms]?)$/;
    if ($2 eq 'd') {
        $sec = $1*60*60*24;
    } elsif ($2 eq 'h') {
        $sec = $1*60*60;
    } elsif ($2 eq 'm') {
        $sec = $1*60;
    } elsif (($2 eq 's') || ($2 eq '')) {
        $sec = $1;
    }
    return($sec);
}

## @name ::absolute_testname
## @description find the current testname
## @synopsis testname()
## @notes might not work under Windows
## @returns the absolute name and path of the current test script
sub absolute_testname {
    my $testname = param("TESTNAME", -mesg, "Nate did not set the TESTNAME parameter");
    if (substr($testname, 0, 1) ne "/") {
	# testname should always return the full path, but there are some cases
	# when it doesn't. For these cases, work around the bug.

	# Remove the leading dot slash (./) from the relative path
	$testname =~ s!^./!!;

	# build the full path
	my $cwd = getcwd;
	$testname = "$cwd/$testname";
    }
    return $testname;
}

## @name ::relative_testname
## @description find the relative path to the current testname (from test)
## @synopsis relative_testname()
## @notes might not work under Windows
## @returns the relative name and path of the current test script (from test)
sub relative_testname {
    my $testname = absolute_testname();

    my $relative_testname = $testname;
    $relative_testname =~ s!.+/test/!!;
    return ($relative_testname) ? $relative_testname : $testname;
}

## @name ::subtest_local
## @synopsis subtest_local(test => 'mytest.thpl')
## @description Subtests the specified test on localhost using
##  subtest_remote. The main motivation for using this routine
##  instead of calling subtest directly is to ensure that the
##  test results are logged by the parent of the test.
## @arg
##
##  test	Required: The path to the test to be executed locally. Can
##		be relative to the location of the parent test.
##
##  username    Optional: The username under which the test will be run.
##
## @returns Nothing
sub subtest_local ( % ) {
    my %opts = validate(
        @_,
        {
            test     => { type => SCALAR },
            username => { type => SCALAR, optional => 1 },
        }
    );
    my (@user_arg);

    if (defined($opts{username})) {
        @user_arg = ( 'username', $opts{username} );
    }
    subtest_remote( test => $opts{test}, client => 'localhost', @user_arg );

    return;
}

## @name ::subtest_remote
## @synopsis subtest_remote(test => 'mytest.thpl', client => $client)
## @description Subtests the specified test on the client indicated. The
##  routine croaks if there is an error in the subtest call, otherwise the
##  results of the subtest are evaluated and logged by the parent test.
##  It is important to note the the results of the subtest are logged
##  by the caller of this test, typically the parent test, instead of
##  the child.
## @arg
##
##  test	Required: The path to the test to be executed remotely. Can
##		be relative to the location of the parent test.
##
##  client	Required: Client where the remote test will be executed.
##
##  username    Optional: The username under which the remote test will be run.
##
## @returns Nothing
sub subtest_remote ( % ) {
    my %opts = validate(
        @_,
        {
            test     => { type => SCALAR },
            client   => { type => SCALAR },
            username => { type => SCALAR, optional => 1 },
        }
    );
    my ($sobj, @results, @user_arg);

    if (defined($opts{username})) {
        @user_arg = ( '-username', $opts{username} );
    }

    $sobj = eval {
        subtest( $opts{test}, -client, $opts{client},
                 'CLIENT=localhost', @user_arg, 
                 -onexit, sub {} );
        };
    # Get out if the subtest call itself had a problem.
    croak $@ if $@;

    @results = $sobj->results();
    foreach my $rslt ( @results ) {
        next if ( $rslt->type() eq 'INFO' );
        logresult($rslt->type(), $rslt->messages());
    }
    return;
}

1;
