# $Id$
## @summary Common library for Dump functions
## @author bellizzi@netapp.com
## @pod here

=head1 NAME

Utils::Dump.pm - Library for Dump functions

=head1 DESCRIPTION

This package defines routines which can be used for using Data::Dumper

=head1 SYNOPSIS

    use Utils::Dump qw(write_dump);

=head1 ROUTINES

=cut

package Utils::Dump;
use strict;
use Tharn;
$Log->comment("Starting " . __FILE__);
use Params::Validate qw(validate SCALAR OBJECT HASHREF ARRAYREF);
use Module::Common qw(validate_mixed);
use Data::Dumper;
use POSIX qw(strftime);

our $VERSION = 1.00;
our @ISA     = qw (Exporter);
our @EXPORT  = qw (write_dump);

my $Options = new Options;

=head2 write_dump

 use Utils::Dump qw(write_dump);
 write_dump(file => $filepath, creator => "cifs_combo", var_name => $name, var_ref => \%hash);

C<write_dump> Write the contents of a perl data structure to a file using Dumper

=over

=item file

(Required) The path the to file

=item creator

(Required) A string denoting who created the file

=item var_name`

(Required) Variable name

=item var_ref

(Required) Variable reference 

=back

=cut

sub write_dump {

    $Log->enter();

    my (%opts) =
      validate(@_,
               {creator  => {type => SCALAR,  optional => 0},
                var_name => {type => SCALAR,  optional => 0},
                var_ref  => {type => HASHREF, optional => 0},
                file     => {
                         type     => SCALAR,
                         optional => 0,
                         default  => 'data.dat'
                  }
               }
               );

    my $date = strftime("%m%d%y%H%M%S", localtime());

    open(DMP, ">$opts{'file'}") || die "unable to open $opts{'file'}: $!";
    print DMP "# Created by $opts{'creator'} on $date\n";
    print DMP Data::Dumper->Dump([$opts{'var_ref'}], [$opts{'var_name'}]);
    close DMP;
    snooze 1;

    $Log->exit();

}

1;
