# $Id: //depot/prod/test/main/lib/Utils/File/Transfer.pm#1 $
# 
# Copyright (c) 2007 Network Appliance, Inc., All Rights Reserved
# Permission is granted to use this script on a Network Appliance Product.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from Network Appliance, Inc.
#
## @summary Provides ability to transfer files to and from GX Nodes
##
## @author medmonds@netapp.com
## @status public
## @pod here
##
## KNOWN ISSUES: this does NOT appear to WORK when the Compute server is Not Linux
##

package Utils::File::Transfer;
use strict;

use Params::Validate qw(:all);
use TracePkg;
use Server;

my $Trace = TracePkg->new();

sub new
{
    $Trace->entry();
    my $class = shift;
    my $hash = {error  => {optional => 1, type => SCALAR, default=> "die",
                           callbacks =>
                           {test => sub{(($_[0] eq qr/^die$/i)
                                                ||
                                         ($_[0] eq qr/^return$/i))}}
                           },
                proto  => {optional => 1, type => SCALAR, default=> "rcp",
                           callbacks =>
                           {test => sub{($_[0] eq "rsh")}}
                           },
                enable => {optional => 1, type => SCALAR, default=> 0,
                           callbacks =>
                           {test => sub{(int($_[0]) > -1) && (int($_[0]) < 2)}}
                           }
                };

    my %params = validate(@_, $hash);

    my $return_type;
    if($params{error} =~ qr/^die$/i)
    {
        $return_type = 1;
    }
    else
    {
        $return_type = 0;
    }

    my $self = bless {}, $class;

    # Fill in the object hash
    $self->{protocol}     = $params{proto};
    $self->{return_type}  = $return_type;
    $self->{enable}       = $params{enable}; # enable RCP on the nodes?
    $self->{xfers}        = undef;

    $self->_result();
    $Trace->exit();
    return $self;
}
# end new()

sub transfer
{
    $Trace->entry();
    my $self = shift;
    my $hash = {node        => {optional => 0, type => SCALAR},
                host        => {optional => 0, type => SCALAR},
                source_path => {optional => 0, type => SCALAR},
                dest_path   => {optional => 0, type => SCALAR},
                is_tonode   => {optional => 1, type => SCALAR, default => 1},
                is_bin      => {optional => 1, type => SCALAR, default => 1},
                };
    my %params = validate(@_, $hash);

    my ($src, $dst, $cmd, $transfer);

    # Are we transfering a file TO or FROM the node?
    if($params{is_tonode})
    {
        $src = $params{host} . ":" . $params{source_path};
        $dst = $params{dest_path};
        $transfer->{dst} = $params{node};
    }
    else
    {
        #$src = "localhost:" . $params{source_path};
        $src = $params{source_path};
        $dst = $params{host} . ":" . $params{dest_path};
        $transfer->{dst} = $params{host};
    }

    $transfer->{on_node} = $params{is_tonode};
    $transfer->{file}    = $params{dest_path};

    # What transfer mechanims are we going to use?
    if($self->{protocol} =~ qr/^rcp$/i)
    {
        $cmd = "rcp " . $src . " " . $dst;

	if ($self->_need_sudo(node => $params{node})) {
	    $cmd = "sudo " . $cmd;
	}
    }
    else
    {
        $self->_result(-1, $self->{protocol} .
                     " is not a supported transfer protocol yet.");
        $Trace->exit();
        return;
    }

    # Commense Transfer
    $self->_send($params{node}, $cmd);
    if($self->error())
    {
        $Trace->exit();
        return;
    }
    $Trace->debug(msg=>$self->{result}->{msg});
    # Keep an audit trail of what is being transfered
    push(@{$self->{xfers}}, $transfer);

    # If it's a Binary, lets be nice and set it to executable for them
    if ($params{is_bin} && $params{is_tonode})
    {
	$cmd = 'chmod +x ' . $params{dest_path};
	
	# See if we need sudo (diag user).
	if ($self->_need_sudo(node => $params{node})) {
	    $cmd = 'sudo ' . $cmd;
	}

        $self->_send($params{node}, $cmd);
        if ($self->error())
        {
            $Trace->exit();
            return;
        }
    }

    $self->_result();
    $Trace->exit();
    return;
}
# end transfer()

sub clean_node
{
    $Trace->entry();
    my $self = shift;
    my $hash = {node  => {optional => 1, type => SCALAR},
                path  => {optional => 1, type => SCALAR}};
    my %params = validate(@_, $hash);

    # If something specific wasn't passed in, delete everything
    #   that we copied to all of the nodes
    if((!($params{node})) || (!($params{path})))
    {
        foreach my $xfer (@{$self->{xfers}})
        {
            if($xfer->{on_node})
            {
                $self->_send($xfer->{dst}, "rm -f " . $xfer->{file});
                if($self->error())
                {
                    $Trace->exit();
                    return;
                }
            }
        }
    }
    else # Delete $path from $node if they were passed in
    {
        $self->_send($params{node}, "rm -f " . $params{path});
        if($self->error())
        {
            $Trace->exit();
            return;
        }
    }

    $self->_result();
    $Trace->exit();
    return;
}
# end clean_node()

sub _start_proto
{
    $Trace->entry();
    my $self = shift;
    my $node = shift;
    push(@{$self->{nodes}}, $node);
    if($self->{proto} eq "rcp")
    {
        eval
        {
            my $server_obj = $self->get_server_obj(node => $node);

            $server_obj->set_execution_server("-Server" => $node);
            $server_obj->shell(cmd => "cd /mroot");
            $server_obj->shell(cmd => "sed 's/\^#shell\/shell/' /etc/inetd.conf | tee /etc/inetd.conf");
            $server_obj->shell(cmd => "sed 's/\^#login\/login/' /etc/inetd.conf | tee /etc/inetd.conf");
            $server_obj->shell(cmd => "sed '/inetd=/ d' /etc/rc.conf | tee /etc/rc.conf");
            $server_obj->shell(cmd => "echo inetd='YES' | tee -a /etc/rc.conf");
            $server_obj->shell(cmd => "pkill -HUP inetd");
            sleep(2);
            $server_obj->shell(cmd => "/usr/sbin/inetd");
            sleep(2);
            $server_obj->shell(cmd => "/etc/rc.d/inetd  restart");
        };
        if($@)
        {
            $self->_result(-1, $@);
            return;
        }
    }
    $self->_result();
    $Trace->exit();
}
# end _start_proto()

sub disable_proto
{
    $Trace->entry();
    my $self = shift;
    my $hash = {nodes  => {optional => 1, type => SCALAR | ARRAYREF}};

    my %params = validate(@_, $hash);
    if(!defined($params{nodes}))
    {
        $params{nodes} = $self->{nodes};
    }
    elsif(ref($params{nodes}) ne "ARRAY")
    {
        $params{nodes} = [$params{nodes}];
    }
    if(!($self->{enable}))
    {
        $self->_result();
        $Trace->exit();
        return;
    }
    if($self->{proto} eq "rcp")
    {
        foreach my $node (@{$params{nodes}})
        {
            eval
            {
                my $server_obj = $self->get_server_obj(node => $node);
                $server_obj->shell(-Server => $node, cmd => "cd /mroot");
                $server_obj->shell(-Server => $node, cmd => "sed 's/\^shell\/#shell/' /etc/inetd.conf | tee /etc/inetd.conf");
                $server_obj->shell(-Server => $node, cmd => "sed 's/\^login\/#login/' /etc/inetd.conf | tee /etc/inetd.conf");
                $server_obj->shell(-Server => $node, cmd => "sed '/inetd=/ d' /etc/rc.conf | tee /etc/rc.conf");
                $server_obj->shell(-Server => $node, cmd => "echo inetd='YES' | tee -a /etc/rc.conf");
                $server_obj->shell(-Server => $node, cmd => "pkill -HUP inetd");
                sleep(2);
                $server_obj->shell(-Server => $node, cmd => "/usr/sbin/inetd");
                sleep(2);
                $server_obj->shell(-Server => $node, cmd => "/etc/rc.d/inetd  restart");
            };
            if($@)
            {
                $self->_result(-1, $@);
                $Trace->exit();
                return;
            }
        }
    }
    $self->_result();
    $Trace->exit();
}
# end disable_proto()

sub get_server_obj
{
    my $self = shift;
    my $hash = {node  => {optional => 0, type => SCALAR}};
    my %params = validate(@_, $hash);
    return $self->{$params{node}}->{conn_obj};
}

sub _send
{
    $Trace->entry();
    my ($self, $node, $cmd) = @_;
    my ($output, $result);
    if(!($self->{$node}->{is_connected}))
    {
        $self->_get_connected($node);
        if($self->error())
        {
            $Trace->exit();
            return;
        }
    }
    eval
    {
        $self->{$node}->{conn_obj}->set_execution_server("-Server" => $node);
        $output = $self->{$node}->{conn_obj}->shell(cmd => $cmd);
        $result = $self->{$node}->{conn_obj}->shell(cmd => "echo \$?");
    };
    if($@)
    {
        $self->_result(-1, $@);
        $Trace->exit();
        return;
    }
    $self->_result($result, $output);
    $Trace->exit();
    return;
}
# end _send()

sub _get_connected
{
    $Trace->entry();
    my $self = shift;
    my $node = shift;

    eval{$self->{$node}->{conn_obj} = Server->new(server=>$node);};
    if($@)
    {
        $self->_result(-1, $@);
        $Trace->exit();
        return;
    }
    if($self->{enable})
    {
        $self->_start_proto($node);
        if($self->error())
        {
            $Trace->exit();
            return;
        }
    }
    $self->{$node}->{is_connected} = 1;
    $self->_result();
    $Trace->exit();
    return;
}
# end _get_connected()

sub _need_sudo
{
    $Trace->entry();
    my $self = shift;
    my $hash = {node  => {optional => 0, type => SCALAR}};
    my %params = validate(@_, $hash);

    my $need_sudo = 0;

    # In BR.0, the root user has gone away and we now must use the
    # diag user.  Since the diag user is unprivileged, certain commands
    # require sudo - rcp is one of those commands.

    # We need to be connected (Server object in scope) before we can do
    # the next bit.
    unless ($self->{$params{node}}{is_connected}) {
	$self->_get_connected($params{node});
	if ($self->error()) {
	    $Trace->exit();
	    return;
	}
    }

    my $server_obj = $self->get_server_obj(node => $params{node});
    if ($server_obj->has_change(change => "node-scope")) {
	$need_sudo = 1;
    }

    return $need_sudo;
}


sub _result
{
    $Trace->entry();
    my $self = shift;

    # If there is something that was passed in (besides self), fill the result
    if(@_)
    {
        $self->{result}->{rc} = shift;
        $self->{result}->{msg} = shift;
        if($self->{result}->{rc} != 0)
        {
            $self->{is_err} = 1;
            if($self->{return_type})
            {
                $Trace->fatal(msg    => $self->{result}->{msg},
                              result => $self->{result}->{rc});
            }
        }
    }
    else
    {
        # set the existing Error to NO ERROR by default
        $self->{result}->{rc} = 0;
        $self->{result}->{msg} = undef;
        $self->{is_err} = 0;
    }
    $Trace->exit();
    return;
}
# end _result()

sub error
{
    $Trace->entry();
    my $self = shift;
    $Trace->exit();
    return $self->{is_err};
}
# end error()

1;


=pod

=head1 Package Name

Utils::File::Transfer

=head1 Author

Michael Edmonds

=head1 Description

This is intended to provide a mechanism to easily transfer
    files to or from Nodes. 


=head1 Synopsis

Here are some examples. 

=over 4

use Utils::File::Transfer;

my $source_path = $ENV{HOME} . "/someBinaryFile";
my $dest_path = "/mroot/someBinaryFile";
my $is_tonode = 0;
my $is_bin = 0;

my $xfer_obj = Utils::File::Transfer->new(error=>"return", proto=>"rcp");
if($xfer_obj->error())
{
    print "Error code = " . $xfer_obj->{result}->{rc} . "\n";
    print "Error message = " . $xfer_obj->{result}->{msg} . "\n";
}

$xfer_obj->transfer(node => $node, host => `hostname`, source_path => $source_path,
                    dest_path => $dest_path, is_tonode => $is_tonode, is_bin => $is_bin);
if($xfer_obj->error())
{
    print "Error code = " . $xfer_obj->{result}->{rc} . "\n";
    print "Error message = " . $xfer_obj->{result}->{msg} . "\n";
}

$xfer_obj->clean_node();
if($xfer_obj->error())
{
    print "Error code = " . $xfer_obj->{result}->{rc} . "\n";
    print "Error message = " . $xfer_obj->{result}->{msg} . "\n";
}

------------------------------------------------------------------------------
use Utils::File::Transfer;

my $source_path = $ENV{HOME} . "/someBinaryFile";
my $dest_path = "/mroot/someBinaryFile";

my $xfer_obj = Utils::File::Transfer->new();

$xfer_obj->transfer(node => $node, host => `hostname`, source_path => $source_path,
                    dest_path => $dest_path);

$xfer_obj->clean_node();


=head1 Public Methods

=head2 new

=over 4

C<new(%params)>

=item Description

Creates a new transfer object.  

=item Input

=item * error => <"die" | "return">

(Optional)
Default = "die"
this tells Process::Load what type of io to subclass

=item * proto => <"rpc" | ...>

(Optional)
Default = "rpc"
It's the Only Protocol this tool supports right now.

=item * enable => <0 | 1>

(Optional)
Default = 0
Setting this to 1 will cause this tool to try to enable RSH on BSD

=item Output

Class Reference

=item Example

use Utils::File::Transfer;

my $xfer_obj = Utils::File::Transfer->new(error=>"return", proto=>"rcp");
if($xfer_obj->error())
{
    print "Error code = " . $xfer_obj->{result}->{rc} . "\n";
    print "Error message = " . $xfer_obj->{result}->{msg} . "\n";
}
------------------------------------------------------------------------------

use Utils::File::Transfer;

my $xfer_obj = Utils::File::Transfer->new();

=back

=head2 transfer

=over 4

C<transfer(%params)>

=item Description

Transfers a file from node to host or vice versa.
This method copies the source_path to the dest_path between the host and node.
It will also "chmod +x <file>" on the Node if is_bin is 1.

=item Input

=item * node => <$node>

(Required)
this is the Node that the file is going to be transfered to/from

=item * host => <IP Addr or FQDN>

(Required)
This is either the IP Addr or FQDN of the Compute server that is going to takepart in the transfer

=item * source_path => <full path + file>

(Required)
Full Path (Including the name of the file to transfer)

=item * dest_path => <full path + file>

(Required)
Full Path (Including the New Name of the File that will be transfered)

=item * is_tonode => <0 | 1>

(Optional)

Default = 1

are we transfering to To the node?
0 indicates that we want to transfer from the Node to the Compute Server.
1 indicates that we want to transfer from the Compute Server to the Node.

=item * is_bin => <0 | 1>

(Optional)

Default = 1

having this set to 1 indicates that the object is to do a chmod +x ... after the transfer

=item Output

None

=item Example

use Utils::File::Transfer;

my $xfer_obj = Utils::File::Transfer->new();

my $source_path = "/mroot/cores/someCoreFile";
my $dest_path = "$ENV{HOME} . "/myCores/" . $node . time();

$xfer_obj->transfer(node => "modeux-19", host => `hostname`,
                    source_path => $source_path, dest_path => $dest_path,
                    is_tonode => 0, is_bin => 0);

=back

=head2 clean_node

=over 4

C<clean_node(%params)>

=item Description

if $node and/or $path are NULL, then:
This will Delete the files that were copied ONTO the node(s)
else:
This will delete the specified file in $path from $node

=item Input

=item * node => <$node>

(Optional)
Name of the Node/Filer to delete the file (specified in $path) from

=item * path => <full path + file>

(Optional)
Full Path (Including the name of the file) to delete

=item Outputs

None

=item Examples

$xfer_obj->clean_node(node => "modeux-19", path => "/mroot/cores/*");

------------------------------------------------------------------------------

$xfer_obj->clean_node();
if($xfer_obj->error())
{
    print "Error code = " . $xfer_obj->{result}->{rc} . "\n";
    print "Error message = " . $xfer_obj->{result}->{msg} . "\n";
}

=back

=head1 TODO

=over 4

=item * Add support for Other transfer protocols -> FTP, ... ?

=back

=cut
