#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/main/lib/Utils/Highcharts.pm#1 $

## @summary     Common graphing library to support bar graphs, pie charts and line graphs
##
## @author
##
## @pod here

=head1 NAME

Highcharts - Common graphing library to support bar graphs, pie charts and line graphs

=head1 DESCRIPTION

C<Highcharts.pm> can be called as an OBJECT method.

The Highcharts library contains functions to suporrts Bar graphs, Pie charts and Line graphs

This library may take in the following inputs:
1. The container ( or <div> ) name which will contain the javascript code returned by the Highcharts Library to generate the graph
2. Title for the Graph
3. The graph data
4. The legend for the graph
5. The width of the container ( or <div> ) containing the graph
6. The height of the container ( or <div> ) containing the graph
7. The graph type (namely, bar graph, pie chart or spline graph)

=head1 SYNOPSIS

Example #1 - Create Highcharts object and use it to create piechart

    package Highcharts;
	my $graphdata = {
        'container' => 'graph',
        'data'      => [
                   [['base_for_disk_busy', '137945'],],
                   [['cp_read_chain',      '115.05'],],
                   [['cp_read_latency',    '2492049.94'],],
                   [['cp_reads',           '263931'],],
                   [['disk_busy',          '1.35'],],
                   [['disk_capacity',      '100792'],],],
        'height' => '480',
        'legend' => ['disk:0b.0:base_for_disk_busy:', 'disk:0b.0:cp_read_chain:',
                     'disk:0b.0:cp_read_latency:',    'disk:0b.0:cp_reads:',
                     'disk:0b.0:disk_busy:',          'disk:0b.0:disk_capacity:'
          ],
        'title' => 'disk (Last  Records)',
        'type'  => 'pie',
        'width' => '510',
        'logo'  => true,};

	my $obj = Utils::Highcharts->new(container => $graphdata->{'container'},
                                  title     => $graphdata->{'title'},
                                  data      => $graphdata->{'data'},
                                  legend    => $graphdata->{'legend'},
                                  width     => $graphdata->{'width'},
                                  height    => $graphdata->{'height'},
                                  type      => $graphdata->{'type'});

	#Returns javascript code to generate piechart
	1. my $output = $obj->piechart();
	2. <div id=$graphdata->{'container'} style=\"width:$graphdata->{'width'}; height:$graphdata->{'height'}; margin: 0 auto\"></div>; #Include this div tag in your html code to place the graph 
     (OR)
	#Returns the html page containing the piechart
	1. my $output = $obj->piechart(logo => $graphdata->{'logo'});

Example #2 - Create Highcharts object and use it to create bargraph

    package Highcharts;
	my $graphdata = {'container' => 'graph',
                  'data'      => [
                             [['5', '3'],],
                             [['2', '2'],],
                             [['2', '2'],],
                             [['2', '2'],],
                             [['2', '2'],],
                             [['2', '2'],],
                             [['2', '2'],],
                             [['2', '2'],],
                             [['2', '2'],],
                             [['2', '2'],],
                             [['3', '4'],],],
                  'height' => '480',
                  'legend' => ['WAFL', 'CSI', 'SAN', 'DPG', 'RAID', 'Platform',
                               'SCON', 'kernel', 'CIFS', 'MIG', 'PERF'],
                  'title'      => 'ONTAP Memory Comparison',
                  'type'       => 'column',
                  'width'      => '510',
                  'categories' => ['RR.x', 'SN.x'],
                  'logo'       => true};

	 my $obj = Utils::Highcharts->new(container => $graphdata->{'container'},
                                  title     => $graphdata->{'title'},
                                  data      => $graphdata->{'data'},
                                  legend    => $graphdata->{'legend'},
                                  width     => $graphdata->{'width'},
                                  height    => $graphdata->{'height'},
                                  type      => $graphdata->{'type'});

	#Returns javascript code to generate bargraph
	1. my $output = $obj->bargraph();
	2. <div id=$graphdata->{'container'} style=\"width:$graphdata->{'width'}; height:$graphdata->{'height'}; margin: 0 auto\"></div>; #Include this div tag in your html code to place the graph
     (OR)
	#Returns the html page containing the bargraph
	1. my $output = $obj->bargraph(logo => $graphdata->{'logo'});

Example #3 - Create Highcharts object and use it to create spline graph

    package Highcharts;
	my $graphdata = {
       'container' => 'graph',
       'data'      => [
                  [['1348348606000', '1758942457'], ['1348348685000', '72189'],],
                  [['1348348606000', '115.05'],     ['1348348685000', '100.00']],
                  [['1348348606000', '2492049.94'],
                  ['1348348685000', '1892307.69'],],
                  [['1348348606000', '263931'],],
                  [['1348348606000', '1.35'],],
                  [['1348348606000', '100792'],],],
       'height' => '480',
       'legend' => ['disk:0b.0:base_for_disk_busy:', 'disk:0b.0:cp_read_chain:',
                    'disk:0b.0:cp_read_latency:',    'disk:0b.0:cp_reads:',
                    'disk:0b.0:disk_busy:',          'disk:0b.0:disk_capacity:'],
       'title'  => 'disk (Last  Records)',
       'type'   => 'spline',
       'width'  => '510',
       'ylabel' => 'ylabel test',
       'logo'   => true};

	my $obj = Utils::Highcharts->new(container => $graphdata->{'container'},
                                  title     => $graphdata->{'title'},
                                  data      => $graphdata->{'data'},
                                  legend    => $graphdata->{'legend'},
                                  width     => $graphdata->{'width'},
                                  height    => $graphdata->{'height'},
                                  type      => $graphdata->{'type'});

	 #Returns javascript code to generate spline graph
     1. my $output = $obj->splinegraph();
	 2. <div id=$graphdata->{'container'} style=\"width:$graphdata->{'width'}; height:$graphdata->{'height'}; margin: 0 auto\"></div>; #Include this div tag in your html code to place the graph
      (OR)
	 #Returns the html page containing the spline graph
	 1. my $output = $obj->splinegraph(logo => $graphdata->{'logo'});

=cut

package Utils::Highcharts;

use strict;
use warnings;
use Params::Validate
  qw(validate validate_with SCALAR HASHREF BOOLEAN ARRAYREF OBJECT UNDEF CODEREF);
use Module::Common qw(validate_mixed);
use Data::Dumper;
use Sort::Key::Natural qw(natsort);
use NATE::Inc qw(find_file);
use Exporter;
our @ISA       = qw(Exporter);
our @EXPORT    = qw(new);
our @EXPORT_OK = qw(piechart bargraph splinegraph);

my $container;
my $legend;
my $data;
my $title;
my $type;
my $width;
my $height;
my $series;
my $logo;
####################### Advertised Routines ################################

=head1 METHODS

=head2 new

	 my $obj = Utils::Highcharts->new(container => 'graph',
                                 title     => 'disk (Last records)',
                                 data      => [
												[['base_for_disk_busy', '137945'],],
												[['cp_read_chain',      '115.05'],],
												[['cp_read_latency',    '2492049.94'],],
												[['cp_reads',           '263931'],],
												[['disk_busy',          '1.35'],],
												[['disk_capacity',      '100792'],],],,
                                 legend    => ['disk:0b.0:base_for_disk_busy:', 'disk:0b.0:cp_read_chain:',
											   'disk:0b.0:cp_read_latency:',    'disk:0b.0:cp_reads:',
											   'disk:0b.0:disk_busy:',          'disk:0b.0:disk_capacity:'],
                                 width     => '510',
                                 height    => '480',
                                 type      => 'pie';	
     
This function returns Highcharts Object on success. 

=over

=item Options

=over

=item C<< data => $graph_data >>

(Required). The graph data that needs to be plotted.

=item C<< legend => $graph_legend >>

(Required). The graph legend.

=item C<< container => $container_name >>

(Optional). The container ( or <div> ) name which will contain the javascript code returned by the Highcharts Library to generate the graph. Default: 'default'

=item C<< title => $graph_title >>

(Optional). The graph title.

=item C<< width => $graph_width >>

(Optional). The  width of the container ( or <div> ) containing the graph.

=item C<< height => $graph_height >>

(Optional). The height of the container ( or <div> ) containing the graph.

=item C<< type => $graph_type >>

(Optional). The graph type (namely, 'bar', 'pie' or 'spline'). Default: spline

=back

=back

=cut

sub new {
    my ($self, %opts) = validate_mixed(
        @_,
        [{type => SCALAR},],
        {   container => {type => SCALAR | UNDEF, default  => 'default'},
            title     => {type => SCALAR | UNDEF, optional => 1},
            data   => {type => ARRAYREF | HASHREF | UNDEF},
            legend => {type => ARRAYREF | HASHREF | UNDEF},
            width  => {type => SCALAR | UNDEF, optional => 1},
            height => {type => SCALAR | UNDEF, optional => 1},
            ylabel => {type => SCALAR | UNDEF, optional => 1},
            type   => {type => SCALAR | UNDEF, default => 'spline'},
        },
    );

    my $chart = {
        'container' => $opts{'container'},
        'title'     => $opts{'title'} || '',
        'data'      => $opts{'data'},
        'legend'    => $opts{'legend'},
        'ylabel'    => $opts{'width'} || '',
        'width'     => ($opts{'width'} eq '') ? 1200 : $opts{'width'},
        'height'    => ($opts{'height'} eq '') ? 500 : $opts{'height'},
        'type'      => $opts{'type'},
    };
    bless($chart, $self);
    return $chart;
} ## end sub new

=head2 piechart

	(1) my $output = $obj->piechart();
		<div id=$graphdata->{'container'} style=\"width:$graphdata->{'width'}; height:$graphdata->{'height'}; margin: 0 auto\"></div>; #Include this div tag in your html code to place the graph		  (OR)
	(1) my $output = $obj->piechart(logo => true);

This function is called using Highcharts Object as shown.

Usage (1) returns javascript code to generate piechart and Usage (2) returns the entire html page containing the piechart

=over

=item Options

=over

=item C<< logo => $boolean_value >>

(Optional). logo => true returns the entire html page containing the piechart and logo => false returns javascript code to generate piechart.

=back

=back

=cut

sub piechart {
    my $self = shift;
    my (%opts) = validate(@_, {logo => {type => BOOLEAN, optional => 1},},);

    my $container = $self->{container};
    my $type      = $self->{type};
    my $title     = $self->{title};
    my $data      = $self->{data};
    my $legend    = $self->{legend};
    my $width     = $self->{width} . 'px';
    my $height    = $self->{height} . 'px';
    my $logo      = $opts{'logo'};
    my $str       = <<END;
    <script type='text/javascript'>
    var chart;
    jQuery(document).ready(function() {
        chart = new Highcharts.Chart({
            chart: {
                renderTo: "graph",
                type: "$type",
                zoomType: 'x'
            },
            title: {
                text: "$title",
                x: -20 //center
            },
            tooltip: { formatter: function() { return '<b>'+ this.point.name +'</b><br/>' + this.percentage + '%'; } },
            plotOptions: {
             pie: {
                    allowPointSelect: true,
                    cursor: 'pointer',
                    dataLabels: {
                        enabled: true,
                        color: '#000000',
                        connectorColor: '#000000',
                        formatter: function() {
                            return '<b>'+ this.point.name +'</b>: '+ this.percentage +' %';
                        }
                    }
                }
             },
            series: [
END

    my $series = 0;
    $str .= tabs(4) . "{type: \'$type\',\n";
    $str .= tabs(4) . " data: [\n";

    foreach my $name (@{$legend}) {
        my $xvalue = 0;
        foreach my $d (@{$data->[$series]}) {
            my $coordinates = array_to_list_pie($d);
            $str .= " $coordinates";

            if ($xvalue < $#{$data->[$series]}) {
                $str .= ",";
            }
            $str .= "\n";
            $xvalue++;
        }
        if ($series < $#{$legend}) {
            $str .= ",";
        }
        $str .= "\n";
        $series++;
    }
    $str .= tabs(4) . " ]\n";
    $str .= tabs(4) . " }\n";

    $str .= <<END;
             ]
          });
         });
     </script>
END

    my $ret = $str;

    if ($logo) {
        my $header = create_header('./js');
        my $footer = create_footer();
		$str .= "<div id=\"$container\" style=\"width:$width; height:$height; margin: 0 auto\"></div>\n";
        $ret = $header . $str . $footer;
    }
    return $ret;

} ## end sub piechart

=head2 bargraph

	(1) my $output = $obj->bargraph();
		<div id=$graphdata->{'container'} style=\"width:$graphdata->{'width'}; height:$graphdata->{'height'}; margin: 0 auto\"></div>; #Include this div tag in your html code to place the graph
	  (OR)
	(2) my $output = $obj->bargraph(logo => true);

This function is called using Highcharts Object as shown.

Usage (1) returns javascript code to generate bargraph and Usage (2) returns the entire html page containing the bargraph

=over

=item Options

=over

=item C<< logo => $boolean_value >>

(Optional). logo => true returns the entire html page containing the bargraph and logo => false returns javascript code to generate bargraph.

=back

=back

=cut

sub bargraph {
    my $self = shift;
    my (%opts) = validate(@_, {logo => {type => BOOLEAN, optional => 1},},);

    my $container  = $self->{container};
    my $type       = $self->{type};
    my $title      = $self->{title};
    my $data       = $self->{data};
    my $legend     = $self->{legend};
    my $width      = $self->{width} . 'px';
    my $height     = $self->{height} . 'px';
    my $categories = $self->{categories};
    my $logo       = $opts{'logo'};

    my $str = <<END;
    <script type='text/javascript'>
    var chart;
    jQuery(document).ready(function() {
        chart = new Highcharts.Chart({
            chart: {
                renderTo: "graph",
                type: "$type",
            },
            title: {
                text: "$title",
            },
	    xAxis: {
                 categories: [
END
    my $count = 0;
    foreach (@{$categories}) {
        $str .= "\'" . $_ . "\'";
        my $array_size = @{$categories};
        if ($count < $array_size) { $str .= ","; }
        $count++;
    }
    $str .= <<END;
            ]},
            yAxis: {
                min: 0,
                title: {
                    text: 'Total ONTAP memory'
                }
            },
            tooltip: { formatter: function() { return ''+ this.series.name +': '+ this.y +' ('+ Math.round(this.percentage) +'%)'; }},
            plotOptions: {
             column: {
                    stacking: 'percent',
                    dataLabels: {
                        enabled: true,
                        color: '#000000',
                    }
                }
             },
            series: [
END

    my $series = 0;

    foreach my $name (@{$legend}) {
        $str .= tabs(4) . "{name: \'$name\',\n";
        $str .= tabs(4) . " data: ";
        my $xvalue = 0;
        foreach my $d (@{$data->[$series]}) {
            my $coordinates = array_to_list($d);
            $str .= " $coordinates";

            if ($xvalue < $#{$data->[$series]}) {
                $str .= ",";
            }
            $str .= "\n";
            $xvalue++;
        }
        $str .= tabs(4) . "}";

        if ($series < $#{$legend}) {
            $str .= ",";
        }
        $str .= "\n";
        $series++;
    }

    $str .= <<END;
             ]
          });
         });
     </script>
END

    my $ret = $str;
    if ($logo) {
        my $header = create_header('./js');
        my $footer = create_footer();
		$str .="<div id=\"$container\" style=\"width:$width; height:$height; margin: 0 auto\"></div>\n";
        $ret = $header . $str . $footer;
    }
    return $ret;
} ## end sub bargraph

=head2 splinegraph

	 (1) my $output = $obj->splinegraph();
		 div id=$graphdata->{'container'} style=\"width:$graphdata->{'width'}; height:$graphdata->{'height'}; margin: 0 auto\"></div>; #Include this div tag in your html code to place the graph
	  (OR)
	 (2) my $output = $obj->splinegraph(logo => true);

This function is called using Highcharts Object as shown. 

Usage (1) returns javascript code to generate spline graph and Usage (2) returns the entire html page containing the spline graph

=over

=item Options

=over

=item C<< logo => $boolean_value >>

(Optional). logo => true returns the entire html page containing the spline graph and logo => false returns javascript code to generate spline graph.

=back

=back

=cut

sub splinegraph {
    my $self = shift;
    my (%opts) = validate(@_, {logo => {type => BOOLEAN, optional => 1},},);

    my $container = $self->{container};
    my $type      = $self->{type};
    my $title     = $self->{title};
    my $ylabel    = $self->{ylabel};
    my $data      = $self->{data};
    my $legend    = $self->{legend};
    my $width     = $self->{width} . 'px';
    my $height    = $self->{height} . 'px';
    my $series    = 0;
    my $logo      = $opts{'logo'};
    my @dates;
    foreach my $name (@{$legend}) {
        foreach my $d (@{$data->[$series]}) {
            my ($date, $value) = @{$d};
            if (!grep { $_ == $date } @dates) {
                push(@dates, $date);
            }
        }
    }

   my $str = <<END;
	<script type='text/javascript'>
    var chart;
    jQuery(document).ready(function() {
        chart = new Highcharts.Chart({
            chart: {
                renderTo: "$container",
                type: "$type",
                zoomType: 'x'
            },
            title: {
                text: "$title",
                x: -20 //center
            },
             xAxis: {
               type: 'datetime',
               labels: {
                   rotation: 90,
                   formatter: function() {
                       return Highcharts.dateFormat('\%Y\%m\%d_\%H\%M', this.value);
                   }
               },
               style: {
                    fontWeight: 'bold'
               }
            },
            yAxis: {
                title: {
                    text: "$ylabel"
                },
                /*plotLines: [ {value: 0,width: 1, color: '#808080'} ]*/ //Line present in the LMR source code, not being used in this library
            },
            plotOptions: {
            column: {
                    dataLabels: {
                        enabled: true,rotation: 90,y: -20
                    }
                }
            },
            tooltip: { formatter: function() { return '<b>'+ this.series.name +'</b><br/>' +Highcharts.dateFormat('\%Y\%m\%d_\%H\%M\%S', this.x) + ': ' + this.y; } },
            series: [
END
    $series = 0;
    foreach my $name (@{$legend}) {
        $str .= tabs(4) . "{name: \'$name\',\n";
        $str .= tabs(4) . " data: [\n";

        my $xvalue = 0;
        foreach my $d (@{$data->[$series]}) {
            my $coordinates = array_to_list($d);
            $str .= tabs(5) . " $coordinates";

            if ($xvalue < $#{$data->[$series]}) {
                $str .= ",";
            }
            $str .= "\n";
            $xvalue++;
        }
        $str .= tabs(4) . " ]\n";
        $str .= tabs(4) . "}";
        if ($series < $#{$legend}) {
            $str .= ",";
        }
        $str .= "\n";
        $series++;
    }

    $str .= <<END;
            ]
        });
    });
</script>

END

    my $ret = $str ;#.=
     # "<div id=\"$container\" style=\"width:$width; height:$height; margin: 0 auto\"></div>\n";
    if ($logo) {
        my $header = create_header('./js');
        my $footer = create_footer();
		$str .= "<div id=\"$container\" style=\"width:$width; height:$height; margin: 0 auto\"></div>\n";
        $ret = $header . $str . $footer;
    }
    return $ret;
} ## end sub splinegraph

########################## Internal Routines ##################################

# Adds TAB spacing
sub tabs {
    my $tabs = shift;
    my $string;
    for (my $t = 1 ; $t <= $tabs ; $t++) {
        for (my $s = 1 ; $s <= $tabs ; $s++) {
            $string .= ' ';
        }
    }
    return $string;
} ## end sub tabs

# Converts an array to a javascript list for piecharts
sub array_to_list_pie {
    my ($arr) = @_;
    if (!@{$arr}) { @{$arr} = (); }
    my $str;
    $str = '[';
    $str .= '\'' . @{$arr}[0] . '\'';
    $str .= ',' . @{$arr}[1];
    $str .= ']';
    return $str;
} ## end sub array_to_list_pie

# Converts an array to a javascript list for a bargraph and spline graphs
sub array_to_list {
    my ($arr) = @_;
    if (!@{$arr}) { @{$arr} = (); }
    my $str;
    $str = '[';
    $str .= join ",", @{$arr};
    $str .= ']';
    return $str;
} ## end sub array_to_list

sub create_header {

    # Generate the HTML header
    my $header = <<END;

<html>
<head>
 <meta http-equiv="pragma" content="no-cache">
 <meta http-equiv="cache-control" content="no-cache">
 <meta http-equiv="Content-Type" content="text/html;charset=utf-8">

 <style type="text/css">

 body {
    font-family: Arial, Helvetica, sans-serif;
     color:#454545;
 }

 div.tab_area {
     font-size: 75%;
     font-weight: bold;
     color:#b2b4b3;
     padding: 0px 5px 5px 0px;
 }

 a.tab {
     background-color: #b2b4b3;
     border: 2px solid #000000;
     border-bottom-width: 1px;
     border-color: #f0d0ff #b090e0 #b090e0 #f0d0ff;
     -moz-border-radius: .75em .75em 0em 0em;
     padding: 1px 1em 1px 1em;
     position: relative;
     text-decoration: none;
     top: 3px;
     z-index: 1;
     color: #8060b0;
 }

 a.tab:hover {
     background-color: #0067c5;
     border-color: #c0a0f0 #8060b0 #8060b0 #c0a0f0;
     color: #ffe0ff;
 }

 a.active_tab {
     background-color: #0067c5;
     color: #ffe0ff;
     border: 2px solid #000000;
     border-bottom-width: 1px;
     border-color: #f0d0ff #b090e0 #b090e0 #f0d0ff;
    -moz-border-radius: .75em .75em 0em 0em;
     padding: 1px 1em 1px 1em;
     position: relative;
     text-decoration: none;
     top: 3px;
     z-index: 10;
 }

 div.body {
     height:84%;
     overflow: auto;
     overflow-y: auto;
     position:static;
 }

 div.banner {
     clear: both;
     color:#ffffff;
     background-color: #0067c5;
     padding:10px 20px 0px 20px;
 }

 div.title {
     text-align:left;
     color:#ffffff;
     line-height:32px;
     font:20px Arial, Helvetica, sans-serif;
 }

 div.node_options {
     font-size: 75%;
     color:#ffffff;
     text-align:left;
 }

 .hidden{
     display:none;
 }

 .unhidden{
     display:block;
 }

  </style>

 <a name="top">
 <div class="banner" style="clear:both;">
 <table>
 <tr>
 <td><img src="http://eng-web.rtp.netapp.com/highcharts/logo-netapp.gif"></td></tr>
 </table>
 </div>

 <script type="text/javascript" src="http://eng-web.rtp.netapp.com/highcharts/js/jquery.min.js"></script>
 <script type="text/javascript" src="http://eng-web.rtp.netapp.com/highcharts/js/highcharts.js"></script>
 <script type="text/javascript" src="http://eng-web.rtp.netapp.com/highcharts/js/modules/exporting.js"></script>

END
    return $header;
} ## end sub create_header

sub create_footer {

    # Generate the HTML footer
    my $footer;
    $footer .= "</body>";
    $footer .= "</html>";

    return $footer;
} ## end sub create_footer
1;
