package Utils::IPManager::Address;

use strict;
use warnings;

use Params::Validate qw(validate SCALAR OBJECT HASHREF BOOLEAN ARRAYREF UNDEF);
use Module::Common qw(validate_mixed);
use Tharn qw($Log config_result script_result);
use Net::IP;
use Data::Dumper;

# This library consists of all private routines, and are not meant to
# be called directly. Any of these routines should be called through
# the top level IPManager interface. These subs act mainly as a redicter
# to provide support for addresses of v4 and v6 types.

#
# Create a new address object. Load the correct libraries
#
sub _new {
    $Log->enter();
    my ($caller, %opts) =
      validate_mixed(@_,
                     [{type => SCALAR},],
                     {version => {type => SCALAR, optional => 0},},);

    my $addrObj = {};

    # handle IP version 4
    if ($opts{version} == 4) {
        require Utils::IPManager::Address::IP4;
        $addrObj = Utils::IPManager::Address::IP4->_new();
    }

    # handle IP version 6
    # TODO: Add support for IP6.
    if ($opts{version} == 6) {
        require Utils::IPManager::Address::IP6;
        $addrObj = Utils::IPManager::Address::IP6->_new();
    }

    return $addrObj;
}
1;
