# $Id $
## @summary Common library for SSQA to keep util functions
## @author gvg@netapp.com
## @pod here

=head1 NAME

Utils::TharnHost.pm - Common library for Tharn host helper functions

=head1 DESCRIPTION

This package defines routines which can be used by tests that require a way to 
get information out of tharnhost files 

=head1 SYNOPSIS

use Utils::TharnHost qw(parse_nate_host_file);

=head1 ROUTINES

=cut

package Utils::TharnHost;
use strict;
use Tharn;
use Params::Validate qw(validate UNDEF SCALAR OBJECT HASHREF ARRAYREF);
use Data::Dumper;

our $VERSION = 1.00;
our @ISA     = qw (Exporter);
our @EXPORT  = qw (
  parse_nate_host_file
  );

=head2 parse_nate_host_file

 use Utils::TharnHost qw(parse_nate_host_file);
 parse_nate_host_file(host => $host);

C<parse_nate_host_file> Parse the NATE host record

=over

=item host

(Required) The the host object

=back

=cut

sub parse_nate_host_file {

    $Log->enter();

    my (%opts) = validate(@_, {host => {type => OBJECT, optional => 0},});

    # get all the host file param
    my %hostFileParams = $opts{'host'}->param_fields();

    # Walk through all params and store the intersting ones in a hash
    my %interfaces;
    my %ifgrps;
    foreach my $key (sort(keys(%hostFileParams))) {
        if ($key =~ /(^IFGRP_NAME$|^IFGRP_INTERFACES$|^IFGRP_MODE$)/) {
            $ifgrps{0}{$1} = $hostFileParams{$key};
            if ($key =~ /^IFGRP_NAME$/) {
                $ifgrps{'ids'}{$hostFileParams{$key}} = 0;
            }
        } elsif ($key =~
                 /(^IFGRP_NAME_\d+$|^IFGRP_INTERFACES_\d+$|^IFGRP_MODE_\d+$)/) {
            $key =~ /^IFGRP_(.+)_(\d+)$/;
            $ifgrps{$2}{$1} = $hostFileParams{$key};
            if ($key =~ /^IFGRP_NAME_(\d+)$/) {
                $ifgrps{'ids'}{$hostFileParams{$key}} = $1;
            }
        } elsif ($key =~ /(^IFC$|^IP$|^MASK$|^MEDIATYPE$|^GATEWAY$)/) {
            $interfaces{0}{$1} = $hostFileParams{$key};
            if ($key =~ /^IFC$/) {
                $interfaces{'ids'}{$hostFileParams{$key}} = 0;
            }
        } elsif ($key =~
              /(^IFC_\d+$|^IP_\d+$|^MASK_\d+$|^MEDIATYPE_\d+$|^GATEWAY_\d+$)/) {
            if ($key =~ /^IFC_(\d+)$/) {
                $interfaces{'ids'}{$hostFileParams{$key}} = $1;
            }
            $key =~ /(.+)_(\d+)$/;
            $interfaces{$2}{$1} = $hostFileParams{$key};
        }
    } ## end foreach my $key (sort(keys(...

    # Get the partner IP
    if ($opts{'host'}->part_host()) {
        my $partnerHostObj        = host($opts{'host'}->part_host());
        my %partnerHostFileParams = $partnerHostObj->param_fields();
        foreach my $key (sort(keys(%partnerHostFileParams))) {
            if ($key =~ /^IP$/) {
                $interfaces{0}{'PARTNER'} = $partnerHostFileParams{$key};
            } elsif ($key =~ /IP_(\d+)$/) {
                $interfaces{$1}{'PARTNER'} = $partnerHostFileParams{$key};
            }
        }
    } ## end if ($opts{'host'}->part_host...

    # Get the IPv6 info
    my @networks = $opts{'host'}->networks();
    foreach my $net (@networks) {
        if ($net->{'family'} eq 'inet6') {
            my $id = $interfaces{'ids'}{$net->{'interface'}};
            $interfaces{$id}{'IPV6'} = $net->{'address'};
        }
    }

    $Log->exit();
    return (\%ifgrps, \%interfaces, \%hostFileParams);

} ## end sub parse_nate_host_file

1;
