# $Id: //depot/prod/test/Rsierranevada/lib/VFILER/ClientUsers.pm#1 $

## @summary misc routines for nate script
##
## @description
##      ClientUsers.pm functions
##
## @status in-progress
##
## @author  Smit (Multistore QA)
##
## @keywords Vfiler User Group Account

package VFILER::ClientUsers;
use strict;
use Carp qw(croak);
use base qw(Exporter);
use Tharn;
use SPARSE::Util::Logger;
use CLI::FileIO qw(rdfile wrfile);
use Utils qw(make_conn);
use Filer::Version qw(get_wafl_version WV_SCRIMSHAW_MAX);
use SayWrap qw(sayEOF);
use Data::Dumper;

BEGIN
{
  our @EXPORT = qw(
	addGroupListForClient
	addPasswdListForClient
	);
}




# addGroupListForClient
#
#       Adds entries to /etc/group for the groups in grouphash
#
# Args
#
#       Clinetconn:              Required.
#
#       grouphash:              Required. HASHREF for list of groups
#                               to be added
#
sub addGroupListForClient($%)
{
  my ($clientspec,%grouphash) = @_;

  return eval {
    my @lines;
    my @keys = keys %grouphash;
    my $ghash = \%grouphash;
    foreach my $g (@keys) {
            my $info = $ghash->{$g};
            my $group = $info->{group};
            my $passwd = $info->{passwd} || "x";
            my $gid = $info->{gid};
            my $members = $info->{members} || "";
            push(@lines, "${group}:${passwd}:${gid}:${members}");
    }
    my $oldtext = join("\n", @lines);
    $oldtext .= "\n" unless ($oldtext =~ /\n$/);
    writefileOnClient($clientspec, "/etc/group", $oldtext)
                or croak("can't write to /etc/group : $@");
  }
}

#addPasswdListForClient
#
#       Adds entries to /etc/passwd for the users in userhash
#
# Args
#
#       filerconn:              Required.
#
#       userhash:               Required. HASHREF for list of users
#                               to be added
#
# Optional Args:
#
#       cifs_user:              Optional. Default is 0. If '1', then
#                               use the 'winlogin' and 'winpasswd' instead of
#                               'login', and 'passwd' fields for the users.
#

sub addPasswdListForClient($%;%)
{
  my ($clientspec,%userhash, %opt) = @_;
  return eval {
    my @lines;
    my @keys = keys %userhash;
	
    my $uhash = \%userhash;
    foreach my $u (@keys) {
            my $info = $uhash->{$u};
            my ($login, $passwd);
            if ($opt{cifs_user}) {
                $login = $info->{winlogin};
                $passwd = $info->{winpasswd} || $main::CIFS_PASSWD || "cifspass123";
            } else {
                $login = $info->{login};
                $passwd = $info->{passwd};
            }
            my $uid = $info->{uid};
            my $gid = $info->{gid};
            my $name = $info->{name} || "";
            my $home = $info->{home} || "";
            my $shell = $info->{shell} || "";

            my $newline = join(":", $login, $passwd, $uid, $gid, $shell, $name, $home);
            push(@lines, $newline);
    }

    my $oldtext = join("\n", @lines);
    $oldtext .= "\n" unless ($oldtext =~ /\n$/);
    writefileOnClient($clientspec, "/etc/passwd", $oldtext)
        or croak("can't write to /etc/passwd : $@");
    sleep(4);
  }
}


sub readfileFromClient($$)
{
    return eval {
        my ($conn, $filename) = @_;
        my (undef, undef, $text) = say($conn, "cat $filename",'-eval_error');
        die $@ if $@;
        $text =~ s/\r+\n/\n/g;
        my $filename_re = quotemeta($filename);
        if ($conn->is_persay()) {
            $text =~ s/^readfile $filename_re\n//;
        }
        if ($text =~ /^$filename_re\: /) {
            die "error from readfile reading $filename.  rdfile says:\n" .
                "$text\n";
        }
        return $text;
     }
}



sub writefileOnClient ( $$$ ) {
    my ($conn,$filename,$data) = @_;
    return eval {
        # invoke writefile

	my (@lines) = split(/\n/,$data);
	foreach my $line (@lines) {
	        my $linere = "(?:" . Tharn::string2regex($line) . ")";
                $linere =~ s/\t/\\s+/g;
                my $endtime = Tharn::timeout2time($conn->timeout());
		sayEOF($conn, "echo $line >> $filename");
        	die $@ if $@;
	}

        return 1;
   }	
}

1;	
