## Package to help VVOL opertaions
## This package offers the following
## 1. add vvol to ConfigBroker
## 2. delete vvol from ConfigBroker
## 3. get vvols from ConfigBroker
## 4. print vvols from ConfigBroker
## 5. modify vvol from ConfigBroker
## @author
## dl-nst-virtual@netapp.com

package VVOL_Helper;

use strict;
use warnings;

use POSIX qw (ceil floor);
use Data::Dumper;
use ConfigBroker;
use HyperV;

sub new {
    my $class = shift;
    my $cb = shift;

    my $self = {};

    die "'Config Broker' is a required parameter for ConfigBroker Helper" if(!defined($cb));
    $self->{resource_group} = $cb->get_resource_group(name => "vvol");;

    bless( $self, $class );
    return $self;
}

sub add_vvol {
    my $self = shift;
    my $params = shift;

    my $vvol_rg = $self->{resource_group};

    my $row = {
		uuid => $params->{uuid},
		state => $params->{state},
		path => $params->{path},
		protocol => $params->{protocol},
		vserver => $params->{vserver},
    };

    return $vvol_rg->add_resources(
	resource_type    => "vvol",
	resourcearrayref => [$row],
    );
}

sub get_vvols {
    my $self = shift;
    my (%opts) = @_;

    my $percent = $opts{percent} || 100;
    my $name = $opts{name};
    my $host = $opts{host};
    my $state = $opts{state};

    my $vvol_rg = $self->{resource_group};

    my $vvols;
    if(defined $name && defined $host) {
	$name = "_" . $name . "_";
        $vvols = $vvol_rg->get_resources(type => "vvol", regex_filters => { uuid => $name, host => $host });
    }
    elsif (defined $name) {
	$name = "_" . $name . "_";
        $vvols = $vvol_rg->get_resources(type => "vvol", regex_filters => { uuid => $name });
    }
    elsif (defined $host) {
        $vvols = $vvol_rg->get_resources(type => "vvol", regex_filters => { host => $host });
    }
    else {
	$vvols = $vvol_rg->get_resources(type => "vvol");
    }

    my $total = scalar(@$vvols);
    my $count = floor($percent / 100 * $total);
    print("Getting $count vvols out of $total \n");

    my @ret_vvols;
    for(my $i = 0; $i < $count; $i++) {
	push( @ret_vvols, $vvols->[$i] );
    }

    print("vvol list: ". Dumper(\@ret_vvols). "\n");
    return \@ret_vvols;
}

sub print_vvols {
    my $self = shift;
    my $name = shift;

    my $vvol_rg = $self->{resource_group};

    my $title = "vvol table";
    my $vvols;
    if (defined $name) {
	$title .= " on $name";
	$name = "_" . $name . "_";
        $vvols = $vvol_rg->get_resources(type => "vvol", regex_filters => { uuid => $name });
    }
    else {
	$vvols = $vvol_rg->get_resources(type => "vvol");
    }

    my $i = 1;
    my $ret = sprintf("**************************************** $title ****************************************\n");
    foreach my $vvol (@$vvols) {
	$ret .= sprintf ("%4d %30s %30s %20s %20s\n", $i++, $vvol->{vvol_name}, $vvol->{datastore}, $vvol->{host}, $vvol->{state});
    }
    return $ret;
}

sub delete_vvol {
    my $self = shift;
    my $params = shift;

    my $vvol_rg = $self->{resource_group};
    my $vvol_name = $params->{uuid};

    return $vvol_rg->delete_resources(
	type    => "vvol",
	regex_filters => { uuid => $vvol_name },
    );
}

sub modify_vvol_state {
    my $self = shift;
    my $params = shift;

    my $vvol_rg = $self->{resource_group};
    my $vvol_name = $params->{uuid};
    my $state = $params->{state};

    return $vvol_rg->modify_resources(
	type    => "vvol",
	regex_filters => { uuid => $vvol_name },
	modify_info => { state => $state },
    );
}

sub modify_vvol_path {
    my $self = shift;
    my $params = shift;

    my $vvol_rg = $self->{resource_group};
    my $vvol_name = $params->{uuid};
    my $path = $params->{path};
	
    return $vvol_rg->modify_resources(
	type    => "vvol",
	regex_filters => { uuid => $vvol_name },
	modify_info => { path => $path },
    );	
}

1;
