#$Id: //depot/prod/test/Rsierranevada/lib/VMwareClient.pm#1 $:
#Copyright (c) 2006 Network Appliance, Inc.
#All rights reserved.
#
## @summary Creates a VMwareClient
##
## @description
##   Creates a VMwareClient before diving into VMwareHost that contains the rest of the 
##   the methods to manipulate the host
## @usage
##   use VMwareClient.pm;   
##
## @keywords SAN FCP iSCSI Blocks VMware
## @dependencies
##   VMwareClient.pm package;
##
## @author holly harb (harb@netapp.com)

use strict;

# Package Name
##############
package VMwareClient;


sub new;



# import common libs
use Error;
use POSIX;
use Burt;
use Config;
use English;
use Net::Ping;
use Tie::IxHash;
use File::Basename;

use VMwareClient::Guests;
use VMwareClient::ClientUtil;

use vars qw( @EXPORT @ISA );
@ISA = qw( Error );


## @name ::new
## @description Create a new VMware Host Object.
## This routine creates a new host object for VMWARE.
## 
## $VMware = new VMwareClient(-connection => $client_connection, -host => $p_client, -timeout =>30,-protocol => $PROTOCOL); 
## @arg $connection Required: The host connection to manipulate SCSI disks on
## @arg $p_client Required: Then name of the vmware host
## @arg $timeout Required: Amount of time before giving up on the connection
## @arg $protocol Required: FCP or iSCSI
 
sub new (\%)
{
        my ($class, %args) = @_;
    
        my $this = {};
    
        my ($msg, $connection, $host, $timeout, $protocol);
    
        # bless and initialize object
        bless $this, $class;
    
        # clear any errors to begin
        $this->_clear_err();
    
        # general host properties
        $this->{connection} = undef;
        $this->{hosttype}   = "linux-vmware";
        ;
        $this->{hostname}   = undef;
        $this->{timeout}    = undef;
        $this->{protocol}   = undef;
        $this->{useSD}      = undef;
    
        # check the validity of params
        foreach (keys %args)
        {
            if (/^-?connection/i)
            {
                if (!length($connection = $args{$_}))
                {
                    $msg = "VMwareClient::new invalid value for -connection: $args{$_}";
                    &main::logcomment($msg);
                    $this->_set_err($msg);
                    return $this;
                }
                next;
            }
    
            if (/^-?timeout/i)
            {
                if (!length($timeout = $args{$_}))
                {
                    $msg = "VMwareClient::new invalid value for -timeout: $args{$_}";
                    &main::logcomment($msg);
                    $this->_set_err($msg);
                    return $this;
                }
                else
                {
                    $this->{timeout} = $timeout;
                }
                next;
            }
            if (/^-?host/i)
	                {
	                    if (!length($host = $args{$_}))
	                    {
	                        $msg = "VMwareClient::new invalid value for -host: $args{$_}";
	                        &main::logcomment($msg);
	                        $this->_set_err($msg);
	                        return $this;
	                    }
	                    else
	                    {
	                        $this->{host} = $host;
	                    }
	                    next;
            }
    
            if (/^-?protocol/i)
            {
                if (!length($protocol = $args{$_}) || $protocol !~ /^(?:fcp|iscsi)$/i)
                {
                    $msg = "VMwareClient::new invalid value for -protocol: $args{$_}";
                    &main::logcomment($msg);
                    $this->_set_err($msg);
                    return $this;
                }
                else
                {
                    $this->{protocol} = $protocol;
                }
                next;
            }
    
        }
    
        # we must have a connection, else this isn't going to be nice
        if (!defined($connection))
        {
            $this->_set_err("VMwareClient::new -connection not defined");
        }
        else
        {
            $this->{connection} = $connection;
            $this->{hosttype} = $connection->{hostp}->{hosttype};
            $this->{hostname} = $connection->{hostp}->{hostname};
        }
    
        # this library only supports vmware
        if ($this->{hosttype} !~ /linux-vmware/i)
        {
            $this->_set_err("VMwareClient::new {hosttype} is not VMware. Please supply a VMware host or check your hosttype in tharnhost");
        }
    
        # if user didn't specify the timeout, default to 300 sec
        $this->{timeout} = 300 if (!defined($timeout));
    
        # user must specify the protocol used
        # TODO: support multiple protocols
        $this->_set_err("VMwareClient::new -protocol not defined") if (!defined($protocol));
 
        
        
        $this = VMwareClient::Guests->new(-self => $this, -connection => $connection, -hosttype => $this->{hosttype});
    
         return $this;
  
}

1;
