#$Id$: 
#Copyright (c) 2006 Network Appliance, Inc.
#All rights reserved.
#
## @summary An interface to create each specific guest os object based on platform
##
## @description
##          When called the interface determines which guest os platform to choose
##          by passing in the connection type, the host name, a timeout, the protocol, and the host type.  
##
## @usage
##   use VMwareClient.pm;   
##
## @example   
## $dmObj = VMwareClient::Guests->new(-connection => $conn, -host => $guest, -timeout =>30,
##                                  -protocol => $protocol, -hosttype => $hosttype);  
## @arg $conn Required: The host connection to manipulate SCSI disks on
## @arg $guest Required: Guest name of found in tharnhost
## @arg $timeout Required: Amount of time given before giving up on the connection
## @arg $protocol Required: FCP or iSCSI
## @arg $hosttype Required: Host type name so the right guest platform can be selected
##
## @keywords SAN FCP iSCSI Blocks
## @dependencies
##   VMwareClient.pm package;
##
## @author holly harb (harb@netapp.com)



package VMwareClient::Guests;

use VMwareClient;
use strict;
use Error;



use vars qw( @ISA @EXPORT );
@ISA = qw(Error);


## @name ::new
## @description Create a new guest Object of the right platform.
## This routine creates a new guest object and choose which platform to use
## @synopsis 
##  $dmObj = VMwareClient::Guests->new(-connection => $conn, -host => $guest, -timeout =>30,
##                                  -protocol => $protocol, -hosttype => $hosttype);  
## @arg $conn Required: The host connection to manipulate SCSI disks on
## @arg $guest Required: Guest name found in tharnhost
## @arg $timeout Required: Amount of time given before giving up on the connection
## @arg $protocol Required: FCP or iSCSI
## @arg $hosttype Required: Host type name so the right guest platform can be selected
## @returns The guest object pointing to the correct platform

sub new
{
  
    my ($class, %args) = @_;
    my ($self, $hosttype);

    $self={};
  
   
    
    # bless the object upfront in case an error occurs.
    bless $self,$class;

    
    $hosttype=$args{-hosttype}; 
   
   
    
    # load the appropriate Guest for the hosttype
    if ($hosttype =~ /unix-solaris/i) { 
        push (@ISA,"VMwareClient::Guests::Solaris::SolarisGuests");
        require VMwareClient::Guests::Solaris::SolarisGuest;        
        $self = VMwareClient::Guests::Solaris::SolarisGuest->_new(%args);
          return $self;  
    }
    elsif ($hosttype =~ /unix-linux/i) {
        push (@ISA,"VMwareClient::Guests::Linux::LinuxGuest");
        require VMwareClient::Guests::Linux::LinuxGuest;  
        $self = VMwareClient::Guests::Linux::LinuxGuest->_new(%args);
          return $self;  
    }
    #This is the VMware object, which isn't a guest, but it still needs to be configured as if it was a guest
    elsif ($hosttype =~ /linux-vmware/i) {
        push (@ISA,"VMwareClient::Guests::VMware::VMwareHost");
        require VMwareClient::Guests::VMware::VMwareHost;  
        $self = VMwareClient::Guests::VMware::VMwareHost->_new(%args);
          return $self;  
    }  
   elsif ($hosttype =~ /windows/i) {
        push (@ISA,"VMwareClient::Guests::Windows::WindowsGuests");
        require VMwareClient::Guests::Windows::WindowsGuest;  
        $self = VMwareClient::Guests::Windows::WindowsGuest->_new(%args);
          return $self;  
    }  
    else {
        $self->_set_err("Error: Guests::_new() Guest type '".$hosttype."' not currently supported with this library");
          return $self;                           
    }
  
    bless $self,$class;

   
    return($self);
}



1;


