# $Id$
#Copyright (c) 2006 Network Appliance, Inc.
#All rights reserved.
#
## @summary Configuring and setup methods for luns on VMware
##
## @description
##   The functions described in this package allow for setting up 
##   the LUNs in either a VMFS3 or RDM (raw device mapping) format for VMware. 
##   After the LUNs are setup, they are allocated to the different VMs. 
##
## @usage
##   use VMwareClient.pm;   
##
## @keywords SAN FCP iSCSI Blocks
## @dependencies
##   VMwareClient.pm package;
##
## @author holly harb (harb@netapp.com)

use strict;

package VMwareClient::Guests::VMware::VMwareHost;

# define the subroutines used in this package
sub new;                    # creates new host object 
sub create_guest;           # creates a guest object


use Error;
use VMwareClient::Guests;
use VMwareClient::ClientUtil;

use Data::Dumper;

use vars qw( @ISA @EXPORT );


@ISA = qw(Error);



## @name ::new
## @description Create a new Host Object.
## This routine creates a new host object for VMware
## It initializes the host data structure,
## gets VMware specific information (populates some global variables),
## bless(es) and returns the object.
## @synopsis 
##  $VMware = new VMwareClient(-connection => $client_connection, -host => $p_client, -timeout =>30,-protocol => $PROTOCOL);
## @arg $connection Required: The host connection to manipulate SCSI disks on
## @arg $p_client Required: Host name of vmware found in tharnhost
## @arg $timeout Required: Amount of time given before giving up on the connection
## @arg $protocol Required: FCP or iSCSI

sub _new (\%)
{
   my ($class, %args) = @_;
   my ($this, $msg, $connection,$protocol);
  
    $protocol = $args{-protocol};
    
 
   if (defined $args{-self})
   {
        $this = $args{-self};
   }else
   {
        bless $this, $class;
        $msg = "Host::new not called with VMwareClient object.";
        &main::logcomment($msg);
        $this->_set_err($msg);
        goto new_EXIT;
    }

    # need to bless this object
    bless $this, $class;

    # clear the errors first
    $this->_clear_err();

    # point back to ourselves for compat and calling host methods from
    # various objects
    
    
    
    $connection = $args{-connection};
    
    
     
    $this->{host} = $this;
     
    $this->{connection} = $connection;
    #$this->{protocol} = $protocol;
    
   
    # get information about the remote host
    $this->_get_host_info();
  
   
    
    return $this;
}    
    

## @name ::get_guests
## @description Gets the number and location of the guests in vmware
## @synopsis
## @guests = $VMware->get_guests()
## @args $connection : The connection into the vmware host
## @returns The guests in an array

sub get_guests()
{
  
   my ($this, %args) = @_;
   my ($line, $connection);
   my (@guests, $line, $status, @output, $cmd);
    
   
   $connection = $this->{connection};
     
   $cmd = "vmware-cmd -l";
 ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd, 120);
       
   #($status, @output) = &ClientUtil::execute($this->{connection}, $cmd, 120);
      
      
      
   foreach $line (@output) {
       push(@guests,$line);
   }
     
   return (@guests);
  
}


## @name ::allocate_luns
## @description Allocates VMFS3 disks to the guest oses
## @synopsis
## $placeholder = $VMware->allocate_luns(\@guests,\@vmdk_files,\@locations,$RDM,\@list,\@guest_types,$RDM_VOL);
## @args @guests Required: The @guests from the $GUESTS input parameter
## @args @vmdk_files Required: All the vmdk vmfs3 disk files that were created on the system
## @args @locations Required: The number and locations returned from get_guests()
## @args $RDM : 1 it is a raw device mapping, 0 it is not
## @args @list Required for RDM only: The sizes of each lun type
## @args @guest_types Required only for RDM: If RDM we must keep track of the LUN and only allocate window luns to windows, for example
## @args $RDM_VOL Required only for RDM: Where the RDMs are stored on VMware
## @returns The number of disks that were allocated per VM
## @note: Currently this method works for VMFS3 only. RDM functionality will be added later. 

sub allocate_luns
{

   
   my ($num, @location, $machine, $x, $tmp);
   #$num will the number of disks that were allocated to each VM
   #$x starts at 1 and increases through the foreach loop until it reaches the number of VMs
   #$tmp is the config file that will have additional entries appended
   
   my ($status, @output, $cmd, @guest);
   
   my ($g_length,$v_length,$disk) = 0;
   #initalizing all these 
   #$disk will be used to keep track of what vmdk file we are on
   
   my ($g,$host,$guest_type,$rdm,$guest);	
   #$g will be a place holder for the guest with rdm, not yet available
   
   my ($self,$guests,$vmdk_files,$locations,$RDM,$machines,$types,$dont_allocate_to_guests) = @_;

   print "This is my dont allocate variable $dont_allocate_to_guests\n";   
   
	  
   #the number of guests	  
   my @guests_array = @$guests; 
   
   #the number of vmdk files
   my @vmdk_array = @$vmdk_files;
   
   #the locations of the guest's config files
   my @location_array = @$locations;
   
   #to be used with rdm, the os type of the vmdk files
   my @names = @$types;
   
   
   #to be used with rdm, the vm names
   #my @machines = @$machines;

   #dont allocate to these guests
   
   my @dont_allocate = @$dont_allocate_to_guests;
   
	
	
   foreach (@guests_array) {
       $g_length++;
   }
	
   foreach (@vmdk_array) {
       $v_length++;
   }
   
   foreach (@location_array){
   }

   
   print "My number of vmdk disks is $v_length";
   print "My number of guests is $g_length";
   
    
   
   
   #Divide the number of vmdk files we have, by the number of guests we have
   #From this we get the number of luns to allocate per VM
   #There may be vmdk files left over if they do not divide into a whole number
   $v_length = $v_length;
   $num = int($v_length/$g_length);

   #If no RDM then add VMFS3 vmdk files
   
   if($RDM == 0){
      #Start with each Guest's location. 
      #From the location we can get into the vmx config file that let's us add disks
      
      $disk = 0;
      
      foreach $machine(@location_array){
        #foreach my $guest(@dont_allocate){
           if($machine =~ m/$guest/){
           
           }else{
           
        
        
         	$x = 1;
          
        	 while($x <= $num){
        	 
        	    $tmp = $machine;
        	    $tmp =~s/ /\\ /g;
            
        	    #Add the disks to the vmx file
        	    $cmd = "cat $tmp";
        	    ($status, @output) = VMwareClient::ClientUtil::execute($self->{connection}, $cmd, 120);
		                    
        	    $cmd = "echo scsi0:$x.present = \'\"true\"\' >> $tmp";
        	    ($status, @output) = VMwareClient::ClientUtil::execute($self->{connection}, $cmd, 120);
		                    
		    $cmd = "echo  >> $tmp";
		    ($status, @output) = VMwareClient::ClientUtil::execute($self->{connection}, $cmd, 120);
		                    
		                    
		    $cmd = "echo scsi0:$x.deviceType = \'\"scsi-hardDisk\"\' >> $tmp";
		    ($status, @output) = VMwareClient::ClientUtil::execute($self->{connection}, $cmd, 120);
		           
		    $cmd = "echo scsi0:$x.filename = \'\"/vmfs/volumes/mydisk$disk/mydisk$disk.vmdk\"\' >> $tmp";
		    ($status, @output) = VMwareClient::ClientUtil::execute($self->{connection}, $cmd, 120);
		                  
		     
		    $cmd = "echo scsi0:$x.redo = \'\"\"\' >> $tmp";
		    ($status, @output) = VMwareClient::ClientUtil::execute($self->{connection}, $cmd, 120);
		                                      
        	    #Continue to add disks to this VM's vmx file until we hit our limit
        	    #Then continue to the next VM
        	    $x++;
        	    $disk++;
        	       
        	 } #end while
        	} #end else
              # } #end inside foreach
             }  #end outside foreach
  	   } #end outside if     
 

   
  return $num;

}




## @name ::turn_VMs_off
## @description Turns the VMs off. This is necessary for VMs to rescan the scsi bus for additional disks
## @synopsis
##  $VMware->turn_VMs_off(-vm => $vm, -hobj => $hostobj, -version => $esx_version);
##              
## @args $vm Required: The name of the VM that is going to get turned off
## @args $hostobj Required: The VMware host object
## @args $esx_version Required: The version of VMware because in time we will need 2.5.x automation. Turning VMs off in 2.5.x 
##                              is different from 3.0

sub turn_VMs_off()
{
  
   my ($this, %args) = @_;
   my ($status, @output, $cmd, $vm, $t, @output2, $cmd2);
 
   $vm = $args{'-vm'};
   my $hobj = $args{'-hobj'};
   my $version = $args{'-version'};

   $vm =~s/ /\\ /g;
			 
   if($version =~ m/VMware ESX Server 3(\S+)/){ 

       $cmd = "ps -e | grep vmkload_app";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd, 120);

   foreach $t(@output){
    
      if($t =~ m/(\d+)\s/){
         $cmd2 = "kill $1";
         ($status, @output2) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
      }
   }

   sleep(5);

   }else{

         $cmd = "vmware-cmd $vm stop hard";
         ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd, 120);
   }


}


## @name ::turn_VMs_on
## @description Turns the VMs on. 
## @synopsis
##  $hostobj->turn_VMs_on(-vm => $vm);
##              
## @args $vm Required: The name of the VM that is going to get turned on

sub turn_VMs_on()
{

   my $vm;
   my ($this, %args) = @_;
       
   $vm = $args{'-vm'};
       
   my ($status, @output, $cmd);
  
   $vm =~s/ /\\ /g;
   			 
   $cmd = "vmware-cmd $vm start now";
   ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd, 120);
   
   #We sleep for 50 seconds to make sure things are booted before starting the config 
   #on the guest OS
   sleep(50);
	
}
      




## @name: _get_host_info
## @description:    This routine will determine characteristics about the current
##                  test system and create global variables that allow users
##                  to determine the operating system type and tools for that
##                  operating system.
## @returns the version of vmware

sub _get_host_info ()
{
   my ($this) = @_;

   my ($cmd, $status, @output,$cmd2,$protocol);
   

   if ($this->{hosttype} =~ /vmware/i) {

        $Host::HOST_TYPE     = $this->{hosttype};
        if ($this->{hosttype} =~ /linux-vmware/i) {
            $Host::TEST_BIN      = "/x/eng/localtest/arch/i386-redhat-linux8/bin";
        } elsif ($this->{hosttype} =~ /fedora/i) {
            $Host::TEST_BIN      = "/x/eng/localtest/arch/i386-redhat-linux8/bin";
        #} elsif ($this->{hosttype} =~ /sles/i) {
        } else {
            $Host::TEST_BIN      = "/x/eng/localtest/arch/i386-redhat-linux8/bin";
        }

        $Host::PLATFORM      = "vmware";
        $Host::NATE_MNT_DIR  = "/mnt/lun"; 	
        $Host::NATE_MNT_CNT  = 0;
        $Host::PATH_SEP      = "/";   
        $Host::RSH_CMD       = "/usr/bin/rsh";
        $Host::RCP_CMD       = "/usr/bin/rcp";
        $Host::PS_CMD        = "/bin/ps -eaf";
        $Host::DF_CMD        = "/bin/df";
        $Host::DU_CMD        = "/usr/bin/du";
        $Host::FORMAT_CMD    = "/sbin/fdisk";
        $Host::MOUNT_CMD     = "mount";
        $Host::UNMOUNT_CMD   = "umount";
        $Host::EXPORTFS_CMD  = "/usr/sbin/exportfs";
        $Host::FSCK_CMD      = "/sbin/fsck";
        $Host::LS_CMD        = "ls";
        $Host::ECHOFILE_CMD  = "cat";
        $Host::RMDIR_CMD     = "rm -rf";  # This even removes non-empty directories
        $Host::RM_CMD        = "rm -f";
        $Host::DIFF_CMD      = "diff";
        $Host::CMP_CMD       = "cmp";
        $Host::DIRCMP_CMD    = "dircmp -ds";  # Not found in path!
        $Host::GZIP_CMD      = "/usr/bin/gzip";
        $Host::GUNZIP_CMD    = "/usr/bin/gunzip";

        $Host::MKFS_CMD      = "/sbin/mkfs";
        $Host::LABEL_CMD     = "/sbin/e2label";

        $Host::LSATTR        = "/usr/bin/lsattr";
        $Host::SYNC          = "/bin/sync";

        $Host::REBOOT        = "/sbin/shutdown -r now";
        $Host::SHUTDOWN      = "/sbin/shutdown now";
        #$Host::BOSBOOT       = "/usr/sbin/bosboot";

        $Host::TEMP          = "/usr/tmp";

        # LVM defintions
        $Host::LVM_VG_CREATE = "/usr/sbin/mkvg";
        $Host::LVM_VG_CHANGE = "/usr/sbin/chvg";
        $Host::LVM_VG_SYNC   = "/usr/sbin/syncvg";
        $Host::LVM_VG_MIRROR = "/usr/sbin/mirrorvg";
        $Host::LVM_VG_LS     = "/usr/sbin/lsvg";
        $Host::LVM_VG_EXTEND = "/usr/sbin/extendvg";
        $Host::LVM_VG_REDUCE = "/usr/sbin/reducevg";
        $Host::LVM_VG_REMOVE = "/usr/sbin/reducevg";
        $Host::LV_CREATE     = "/usr/sbin/mklv";
        $Host::LV_COPY       = "/usr/sbin/mklvcopy";
        $Host::LV_REMOVE     = "/usr/sbin/rmlv";
        $Host::LV_LS         = "/usr/sbin/lslv";
        $Host::LVM_VG_ON     = "/usr/sbin/varyonvg";
        $Host::LVM_VG_OFF    = "/usr/sbin/varyoffvg";

        # get the os revision
        if ($this->{hosttype} =~ /redhat|fedora/i) {
            $cmd = "cat /etc/redhat-release";    
        } elsif ($this->{hosttype} =~ /sles/i) {
            $cmd = "cat /etc/issue.net";    
        } else {
            $cmd = "cat /etc/issue.net";    
        }
       
       
       
       
       
      #($status, @output) = &ClientUtil::execute($this->{connection}, $cmd, 120);
        #Welcome to UnitedLinux 1.0 (i586) - Kernel %r (%t).
      ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd, 120);
       
       
       &main::logcomment("$output[0]\n     $output[1]\n"); 
	


     
	
	
	if ($status == 0) {
	$this->{os_revision} = $output[0];
        } else {
            $this->{os_revision} =  $output[0];
        }
        
        
       $cmd2 = "esxcfg-advcfg -s 1 /Scsi/LogAborts";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
	       
       $cmd2 = "esxcfg-advcfg -s 1 /Scsi/LogMultiPath";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
	       
       $cmd2 = "esxcfg-advcfg -s 1 /Scsi/PrintCmdErrors";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
	     	
       $cmd2 = 	"echo \'\"scsi log error 5\"\' > /proc/scsi/scsi";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
	     	
       $cmd2 = "echo \'\"scsi log timeout 5\"\' > /proc/scsi/scsi";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
	     	
	     	
	
       if($this->{protocol} eq "iscsi"){
       
       $cmd2 = "echo \'\"log sense always\"\' > /proc/scsi/vmkiscsi/*";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
	
       $cmd2 = "echo \'\"log login on\"\' > /proc/scsi/vmkiscsi/*";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
	
       $cmd2 = "echo \'\"log init on\"\' > /proc/scsi/vmkiscsi/*";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
	
       $cmd2 = "echo \'\"log queue on\"\' > /proc/scsi/vmkiscsi/*";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
	
       $cmd2 = "echo \'\"log retry on\"\' > /proc/scsi/vmkiscsi/*";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
	
       $cmd2 = "echo \'\"log eh on\"\' > /proc/scsi/vmkiscsi/*";
       ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd2, 120);
       
		    
       }
	     
    }

	my $version = "$output[0]" . "$output[1]";

       

    return $version;


}


## @name ::discover
## @description Method that discovers the adapter information
## and LUN information from the sanlun tool.
## It stores the information in the host object for later retreval. 
##
## @synopsis
## $esx_version = $hostobj->discover(-protocol => $PROTOCOL);
## @args $protocol Required: Either fcp or iscsi    
## @returns the discovered devices

sub discover ()
{
   my ($this, %args) = @_;
   my ($ver, $status, @output, $rescan, @scanner, $cmd, @output, $wwpn, @scan, @scanned, $hba, @scans);
   my $protocol = $args{-protocol};
   my $version = "vmware -v";


  ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $version, 120);
		 
  #If we are using iscsi on 3.0, then scan only the vmhba40 adapter, cause this is iscsi only		 
  if($output[0] =~ m/VMware ESX Server 3(\S+)/){   
		   
     if($protocol eq "iscsi"){
        $rescan = "esxcfg-rescan vmhba40";
	($status, @scanner) = VMwareClient::ClientUtil::execute($this->{connection}, $rescan); 
     #otherwise, get the number of adapters here 	
     }else{
	$rescan = "esxcfg-vmhbadevs"; 
	($status, @scanner) = VMwareClient::ClientUtil::execute($this->{connection}, $rescan); 
	
     #This parse all the vmhba devices for vmhbaX only
     #Where X is a number between 0 and up
     #We only need to scan vmhba0 (for example) once
     
         foreach $hba(@scanner){
      
            if($hba =~ m/vmhba+(\d+):(\d+):(\d+)\s+\/dev\/(\S+)/){
               $hba = "vmhba$1";
               #see if the hba has already been scanned
               #1 it has been scanned, so skip it
               #0 it has not been scanned, so scan it
    
               if(&scan($hba,@scanned) eq "1"){
               }
               else{
                  push(@scanned,$hba);
                  $rescan = "esxcfg-rescan $hba";
                  ($status, @scans) = VMwareClient::ClientUtil::execute($this->{connection}, $rescan); 
               }
            
            }else{
         }
     }
		      
		    
		          #determine what hbas need to be scanned
		          #rescan all the hbas
		          #after scanning, choose non-proxy paths 
		     
		          #choose_paths($ver);
		                            
  }
    }elsif(@output =~ m/VMware ESX Server 2.5/ && $protocol !="iscsi"){
		         
		         #must be fcp, no iscsi in 2.5.x
		         #Determine HBAs and scan
		     
		         $cmd = "wwpn.pl";
		         ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd);
		     
		                           foreach $wwpn (@output){
		                             if($wwpn =~ m/^vmhba+(\d+):/){
		                             
		                                     $cmd = "cos-rescan.sh vmhba$1";
		                                   ($status, @scan) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd); 
		                             }
		                           }
		              
		                         
		                          #after scanning, call choose_paths() method in VmwareHost.pm 
		                          #to choose non-proxy paths
		     
		 }else{
		         #ABORT
		         #You cannot use iscsi with ESX 2.5
		 }
		     

   return($output[0]);    
      
}



## @name scan
## @summary A helper method for discover()
## @args $hba Required; The hba. Looks like vmhba0
## @args @scanned: A list of hbas that have been scanned
## @returns True of false if the hba has been scanned or not 

sub scan ()
{

   my $hba = $_[0];
   my @scan = $_[1];
   my $t;

    foreach $t(@scan){
       if($hba eq $t){
          return "1";
       }else{
          return "0";
       }    
    }

}




## @name ::create_guest
## @description Creates the guest object
## @synopsis
## $theGuest = $VMware->create_guest(-connection => $client_connection, 
##                          -host => $client, -timeout =>30,-protocol => $PROTOCOL, -type => $type);
## @arg $connection Required: The host connection to manipulate SCSI disks on
## @arg $p_client Required: Host name of vmware found in tharnhost
## @arg $timeout Required: Amount of time given before giving up on the connection
## @arg $protocol Required: FCP or iSCSI
## @returns the guest object
sub create_guest{
  
   my($self,%args) = @_;
   my ($dev,$dmObj,%hArgs,);
   my ($conn, $guest, $timeout, $protocol, $hosttype);
    
   $conn = $args{-connection};
   $guest = $args{-host};
   $timeout = $args{-timeout};
   $protocol = $args{-protocol};
   $hosttype = $args{-type};
    
    
   $dmObj = VMwareClient::Guests->new(-connection => $conn, -host => $guest, -timeout =>30,-protocol => $protocol, -hosttype => $hosttype);
  
    
   return $dmObj;
}

## @name ::make
## @description Creates a filesystem (with the makefs or newfs command) on
## the current device and partition. User can pass the file system type
## and file system specific options.
## @synopsis 
## $VMware->make(-device => $device);
## @arg $device Required: The device to be fdisked
## @arg $type Optional: The file system type to use ('ext2', 'ext3'. 'reiserfs')
## @arg $bs Optional: The block size to use on the mkfs call (-b flag)
## @arg $fs Optional: The fragment size to use on the mkfs call (-f flag)
## @arg $size Optional: The size of the fileystem to create, defaults to
## the full size of specified device/parition.
## @arg $pt Optional: Any pass through option to mkfs or newfs you need to pass


sub make (\%)
{

    my ($this, %args) = @_;
    my $device = $args{'-device'};


    my ($extra_args, $block_size, $type, $frag_size);
    my ($n_cmd, $blocks, $size, $dobj);
    my ($ret, @stdin, $loglv);
    my ($no_format, $pass_thru);
    my (@output, $status, $revision);

    # Other file systems are not supported at this time
    my @ok_types = ('ext2','ext3','reiserfs');

    $extra_args = "";
    $blocks = '';

    

    # get the disk object
    $dobj = $this->{dobj};

    # Establish default filesystem type if needed.
    if (not defined($type) or ($type eq "")) {
        $type = 'ext2';
    } 
    $this->{volume_type} = $type;

    # check to make sure the filesystem type is valid
    if (0 == grep(/^$type$/i, @ok_types)) {
        my $msg = "Supported Linux filesystem types include: ".join(',',@ok_types)." not: $type";
        &ExitScriptError($msg);
    }

  
    # create format partition command
    $n_cmd = "echo -e 'd\n1\nn\np\n1\n\n\nt\nfb\nw\n' | fdisk /dev/$device";

    # execute
    ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $n_cmd);

    # determine whether or not format command worked
   
    # check the size and convert
    if (defined($size) and $size ne "") {
        $size   = &ClientUtil::get_byte_size($size);
        $blocks = ($size / 1024);
    } else {
        $size = "";
    }

    # Support filesystem specific parameters
    if ($type =~ /^ext2$/i) {
        # See mke2fs man page for these parameters
        # We should determine if specified values are valid.
        if (defined($block_size)) { 
            $extra_args .= "-b $block_size ";
            $blocks = ($size / $block_size);
        }
        if (defined($frag_size)) { 
            $extra_args .= "-f $frag_size ";
        }
    }

    # construct mkfs command
    if ($type =~ /^reiserfs$/i) {
    $n_cmd = "echo -e 'y\n' | $Host::MKFS_CMD -V -t $type $this->{parteddisk} $extra_args";
    } else {
    $n_cmd = "/sbin/mkfs -V -t $type /dev/$device" . "1";
    }

    # Log the command
    &main::logcomment("mkfs of type ($type) being run on $this->{parteddisk}",
                      "command: $n_cmd");

    ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $n_cmd);

    # determine if mkfs command worked
    if ($status > 0) {
        &ExitError("mkfs returned a bad status ($status))", @output);
    } else {
        &main::logcomment("mkfs was successful",@output);
    }
    
    # label the filesystem, if it is ext2 or ext3
    # (e2label command doesn't work with reiser)
    if (($type =~ /^ext2$/i) || ($type =~ /^ext3$/i)) {
    	$n_cmd = "/sbin/e2label /dev/$device" . "1" . "  $device" . "_label";
    }

    ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $n_cmd);

    # determine if filesystem labeling command worked
    if ($status > 0) {
        &ExitError("label command returned a bad status ($status))", @output);
    } else {
        &main::logcomment("label command was successful",@output);
    }

    return $ClientUtil::TRUE;

}

## @name ::make_vmfs
## @description Creates vmfs3 volumes and vmdk files on the volumes 
##
## @synopsis 
##      $hostobj->make_vmfs(-lun => $lun,-num => $size, -lun_size => $lun_size, -hobj => $hostobj, 
##                          -vmfs => $VMFS, -rdm => $RDM, -version => $esx_version,
##                          -controller => $controller, -rdm_vol => $rdm_vol);
## @args $lun Required: The lun that we are going to make the vmfs3 volumes out of 
## @args $num Required: The number of devices
## @args $lun_size Required: The size of the lun
## @args $hostobj Requred: The VMware host object
## @args $VMFS Required: VMFS3 or VMFS2
## @args $RDM Required: 1 use RDM, 0 do not
## @args $esx_version Required; The version of vmware
## @args $controller Required: Either lsilogic or buslogic
## @args $rdm_vol Required only for RDM: Where the RDM volumes will be stored

sub make_vmfs(\%)
{

   my ($this, %args, $vmfs, $rdm) = @_;
   my ($lun, $num, $size, $length_size,$numbers);
   my ($status, @output, $n_cmd, $letter, $t, @numbers);
   my ($version, $hostobj, $size_letter);

   $lun = $args{'-lun'};
   $num = $args{'-num'};
   $size = $args{'-lun_size'};
   $hostobj = $args{'-hobj'};
   
   my $version = $args{'-version'};
   my $controller = $args{'-controller'};
   my $rdm_vol = $args{'-rdm_vol'};
   
   
   
   my $vmfs = $args{'-vmfs'};
   my $rdm = $args{'-rdm'};
              
   $this->{parted_lun}  = $lun;

   my $mydisk = "mydisk$num";




   #Get the version
   $version = $hostobj->_get_host_info();
          
   #Make the vmfs partition in either vmfs2 or vmfs3
   #VMware 2.5.x can only do vmfs2
   #Once the partition is created, make a symbolic link to it with the -S option
   #Vmware 2.5.X cannot make a symbolic link and create the partition at the same time
    
    
    
    &main::logcomment("These are my arguments: RDM $rdm and VMFS $vmfs and this is my rdm vol $rdm_vol");
    
    
	if($version =~ m/VMware ESX Server 3/){ 
		 if($rdm == 1){
		 }else{
		     $n_cmd = "vmkfstools -C $vmfs -b 8m -S $mydisk /vmfs/devices/disks/$lun:1";

		     #Log the command
		    &main::logcomment("Making filesystem of type vmfs being run on ","command: $n_cmd");

		    ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $n_cmd);
                  }
	}else{

             $n_cmd = "echo -e 'y' | vmkfstools -C vmfs2 $lun:1";

             #Log the command
             &main::logcomment("Making filesystem of type vmfs being run on ","command: $n_cmd");
             ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $n_cmd);
         
             $n_cmd = "vmkfstools -S $mydisk $lun:1";
    
             #Log the command
             &main::logcomment("Making filesystem of type vmfs being run on ","command: $n_cmd");
    
             ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $n_cmd);
         }    






     #After the VMFS volume is created, create a vmdk disk on the volume
     #To do this you need to know the size of the volume and how much 
     #space you have to create the disk


        if($version =~ m/VMware ESX Server 3/){ 	
        
              #Create a disk in the VMFS dir
              #The SCSI controller can be either lsilogic or buslogic, I will create
              #an option parameter for this  type later
          
          
	   if($rdm == 1){
          
        	$n_cmd = "vmkfstools -z /vmfs/devices/disks/$lun:0 /vmfs/volumes/$rdm_vol/myrdm$mydisk.vmdk -a $controller";
          
        	#Log the command
        	&main::logcomment("Making filesystem of type vmfs being run on ","command: $n_cmd");
        	  
        	($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $n_cmd);
        
    	   }else{
          		  
  	      $n_cmd = "cd /vmfs/volumes; vmkfstools -P -h $mydisk";
  	      &main::logcomment("Making filesystem of type vmfs being run on ","command: $n_cmd");
	     ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $n_cmd);
      
   	              foreach $t(@output){
	             
		                if($t =~ m/Capacity\s+([\d+|\.]+)(\S),\s([\d|\.]+)(\S+)+(\D+)(\S)/){
		                                 
	                                  $letter = "$4";
	                                  $numbers = "$3";
	                                  $letter =~ tr/A-Z/a-z/;
	                             
	                                 if($numbers =~ /[.]/){
	                                      @numbers = split(/\./,$numbers);
	                                  
	                                  	#Convert gigs to megabytes because
	                                  	#the vmkfstools command cannot handle gigs
	                                  	if($letter =~ m/t/){
					       
	  					     $numbers[0] =~s/ /./g;
						     $numbers[0] = "$numbers[0]000";
						     $size_letter = "$numbers[0]g";
	                                  	}elsif($letter =~ m/g/){
	                                  	     $size_letter = "$numbers[0]$numbers[1]00m";
	                                  	}else{
	                                  	}
	                                  }
	                                  else{
	                                  $size_letter = "$numbers$4";
	                                  }

	                         }
	               }

       
             
   	     $n_cmd = "cd /vmfs/volumes; vmkfstools -c $size_letter -a $controller /vmfs/volumes/$mydisk/$mydisk.vmdk";
             &main::logcomment("Making filesystem of type vmfs being run on $n_cmd");
            ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $n_cmd);
         
             }
        }else{
         }
   
   
    return($ClientUtil::TRUE);

}




## @name ::get_vmdk
## @description Gets the number of vmdk files created 
##
## @synopsis 
##      @vmdk_files = $hostobj->get_vmdk(-hobj => $hostobj, -rdm => $RDM, -rdm_vol => $RDM_VOL);
## @args $hobj Required: The Vmware host object
## @args $RDM Required: 1 use RDM, 0 do not
## @args $rdm_vol Required only for RDM: Where the RDM mappings will be stored
## @returns The vmdk files in a array

sub get_vmdk(){

   my ($this, %args) = @_;

   my $hobj = $args{'-hobj'};
   my $rdm = $args{'-rdm'};
   my $rdm_vol = $args{'-rdm_vol'};



   my ($lun,$connection);
   my (@mp_devices, @devices, $device, @path_devices, $attr_hash);
   my ($disk_rep, $disk_lun, $line, $status, @output, $cmd);
   my (@lun_sizes, $lun_size,$dev, @vmdk_files, $vol, $vmdkf, @vmdk, $go);


   $connection = $this->{connection};
   my $version = $hobj->_get_host_info();
             
  #First determine where we are looking for vmkd files or rdm vmdk files
  #Get the vmdk files an add them to the stack
  
   if($version =~ m/VMware ESX Server 3/){   
      if($rdm == 1){
     	  $cmd = "cd /vmfs/volumes/$rdm_vol; ls *-rdmp.vmdk";    
   	  ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd, 120);
          @lun_sizes  = split(/\n/,@output);
             foreach $vol(@output){
                @lun_sizes =  split(/	/,$vol);
          	   foreach $go(@lun_sizes){   
         	      if($go =~ m/myrdmmydisk(\d+)(\S+)/){
         	         push(@vmdk_files,$go);
              	      }else{
           	      }
              	   }  
              }
     }
     else{ 
        $cmd = "ls /vmfs/volumes";    
 	($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd, 120); 
 	
 	 if($rdm == 1){
	 }else{
	    @lun_sizes  = split(/\n/,@output);
	       foreach $vol(@output){
	          @lun_sizes =  split(/ /,$vol);
	             #My automation creates VMFS3 vmdk files in this format
	             # mydiskX.vmdk starting with X equal to 0 
	             foreach $go(@lun_sizes){             
	                if($go =~ /mydisk(\d+)/){
	                    $vmdkf = "mydisk$1";  
	                    push(@vmdk_files,$vmdkf);
	                }
	             }  
	       }
	
	 }  
	 
	 return(@vmdk_files);

 	
      }	 
}


   $cmd = "ls /vmfs";    
   ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd, 120);
   
 ##########################################  
   
   
   if($rdm == 1){
   }else{
 
   @lun_sizes  = split(/\n/,@output);
 
   foreach $vol(@output){
      @lun_sizes =  split(/ /,$vol);
         foreach $go(@lun_sizes){             
            if($go =~ /mydisk(\d+)/){
                $vmdkf = "mydisk$1";
                push(@vmdk_files,$vmdkf);
            }
         }  
   }

 }  
 
   return(@vmdk_files);

}


## @name ::get_info
## @description Gets the devices, the luns, and their sizes on vmware
##
## @synopsis 
##         $info = $hostobj->get_info(-version => $esx_version);
## @args $esx_version Required: the version of vmware
## @returns A hash of the devices, the luns they are associated too, and the sizes of those luns


sub get_info() 
{
   my ($this, %args) = @_;
   my %hashy;
   my ($cmd, $dev, $temp, $status, @output, $line);
   my (@devices, @vmware_lun_sizes, @vmware_luns, $lun_size);



   my $ver = $args{-version};



   if($ver =~ m/VMware ESX Server 3(\S+)/){
           
           $cmd = "esxcfg-mpath -l";
           ($status, @output) = VMwareClient::ClientUtil::execute($this->{connection}, $cmd);
 
 #parse it
 #get lun names
 #devices
 #sizes    

    foreach $line (@output) {
        if($line =~ m/Disk\s+vmhba+(\d+):(\d+):(\d+)\s+\/dev\/(\S+)(\D)+(\d+)(\D{2})/){
                  $temp = "vmhba$1:$2:$3";
                  $lun_size = "$6.$7";
                  $dev = "$4";
                  
                  push(@devices,$dev);
                  push(@vmware_lun_sizes,$lun_size);
                  push(@vmware_luns,$temp);
         }
        if($line =~ m/Local/){
                pop(@devices);
                pop(@vmware_luns);
                pop(@vmware_lun_sizes);
         }    
     }

}
else{
 #much more diffcult then one command in 2.5
}


   @{$hashy{devices}}=@devices;
   @{$hashy{luns}}=@vmware_luns;
   @{$hashy{sizes}}=@vmware_lun_sizes;

	

   return \%hashy;
}




## @name ::get_wwpns
## @description
## @synopsis 
## $obj = $hostobj->get_wwpns()
## @returns a list of world wide port names for each of the adapters in the client.

sub get_wwpns
{
    my ($this, $lun) = @_;

    my ($connection, $status, @output, $line, $cmd);
    my ($adapter, $num_adapters, $port_line, $wwpn);
    my (@return, @hbas);
    @return = ();

    $connection = $this->{connection};

    #VMware 2.5.x only 
    
    $cmd = "vmware -v";
    ($status, @output) = VMwareClient::ClientUtil::execute($connection, $cmd, 120);


    if($output[0] =~ m/VMware ESX Server 3(\S+)/){
        $cmd = "esxcfg-mpath -a";
        ($status, @output) = VMwareClient::ClientUtil::execute($connection, $cmd, 120);
         
          foreach $wwpn (@output){
	  
	     if ($wwpn =~ m/vmhba(\d)\s(\S+)/){
	       push(@return,$2);
	     }
	  
	  }

    }
    else{
    
    
        $cmd = "wwpn.pl";
    
        # execute the command
        ($status, @output) = VMwareClient::ClientUtil::execute($connection, $cmd, 120);

    foreach $wwpn (@output){
                     	if($wwpn =~ m/^vmhba+(\d+):(\d+)/){
   			   push(@return, $2);
   			}
   	
   }
  }      

    return(@return);

}


## @name :: get_macs
## get mac addresses of iscsi initiators



sub get_macs{ 

    my ($this, $lun) = @_;

    my ($connection, $status, @output, $line, $cmd);
    my ($adapter, $num_adapters, $port_line, $mac);
    my (@return, @hbas, @array);
    @return = ();




    
    $cmd =  "ifconfig vswif1 | grep HWaddr";
    ($status, @output) = VMwareClient::ClientUtil::execute($connection, $cmd, 120);

    
    @array = split(/HWaddr/,$output[0]);
    
    my @one_element = $array[1];    

    return(@one_element);

}
1;
