#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/VSA.pm#1 $
## @summary VSA configuration
## @author dl-vsa-qa
## @status private
package VSA;

use strict;

use CIFS::Setup qw(is_cifs_setup cifs_setup);
use Data::Dumper qw(Dumper);
use Error qw(_clear_err _set_err);
use File::Basename qw(basename);
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR SCALARREF BOOLEAN OBJECT ARRAYREF UNDEF);
use POSIX qw(strftime);
use Server;
use Options;
use TestUtils qw(get_site_info);
use Tharn qw($Log);
use Utils qw(make_host make_conn);
use VSA::Vmhost;
use VSA::UICommon qw(_caller _reformat_err _log _refresh_cache
                     _size_to_mbytes _trim);
use install;
use boot;
use base qw(VSA::UICommon Error);

my %License_Of =(
    'DOvM10'   => 'BFIE-0ZLG-IMM8-46N9-F7FE',
    'DOvM100'  => 'CHXE-7Z7B-XMRQ-4DDA-J73L',
    'FCvM100'  => '6FIE-0ZLV-IMM8-469B-D7FE',
);

################################################################################
# Subroutine Name: new()
# Objective:       Create a new ESXHost object
# Details:         See POD documentation at the end of this file
################################################################################
sub new {
    my ($class, %args) = @_;
    my (
        $errmsg,
        $options,
        $self,
        %site_of,
        );
 
    ### bless instance into class
    $self= bless{}, $class;
    
    ### Clear error buffer
    $self->_clear_err();

    
    ### Determine err handle upfront in case validation method
    ### fails before error handler is selected
    $self->{on_err}=(defined $args{on_err} and $args{on_err}=~/die|rtn/)
        ? $args{on_err}
        : "die";
    
    ### Validate input arguments
    eval {
        ($class,%args) = validate_mixed (
					 @_,
					 [ { type => SCALAR} ],
					 {
					     license => {type => SCALAR|UNDEF, optional => 1},  
					     loghdl =>  {type => SCALARREF, optional=>1 },
					     memory =>  {type => SCALAR, optional=>1, default=>4096,
							 regex => qr/^\d+$/},
					     name   =>  {type => SCALAR},
					     datastore => { type => SCALAR|UNDEF, optional=>1 },
					     network_name  => { type => SCALAR },
					     vmhost        => { type => SCALAR },
					     loader => { type => SCALAR, optional => 1, default => "VLOADER" },
					     on_err =>  {type => SCALAR, optional=>1, default=>"die",
							 callbacks=>{
							     'legal values'=>sub{shift()=~/die|rtn/i
										 }}},
								 client =>  { type => SCALAR },
								 show_cmd =>{type=>SCALAR, optional=>1, default=>0},
							     }
					 );
        
        $self->{license}       = $args{license};
        $self->{loghdl}        = $args{loghdl};
        $self->{memory}        = $args{memory};
        $self->{network_name}  = $args{network_name};
        $self->{ovf_client}    = $args{client};
        $self->{datastore}     = $args{datastore};
        $self->{name}          = $args{name};
        $self->{on_err}        = $args{on_err};
        $self->{vmhost}        = $args{vmhost};
        $self->{loader}        = $args{loader};
        
        #### Retrieve tharnhost file for the machine
        $self->_log(msg=>"retrieving tharnhost info", indent=>3);
        $self->{thost_of}     = Utils::make_host($self->{name});
		$self->{default_password} = $self->{thost_of}->default_password();
        $self->{root_pw} = (length $self->{thost_of}->default_password())
             ? $self->{thost_of}->default_password()
            : "netapp1!";
        
		
		    
        $self->{model} = (defined $args{license})
            ? "TBD from platform license key"
            : $self->{thost_of}->{MODEL};
        
        ### Create a console connection to the vsa 
        #$self->_log(msg=>"creating a vsa console connection", indent=>3);
        #$self->{console_conn} = Tharn::connect($self->{name}."#console");
        
        ### Group all the standard mode prompts together
        $self->{mode_prompts}=[
            @{$self->{thost_of}->prompt_fw()},
            @{$self->{thost_of}->prompt_norm()},
            @{$self->{thost_of}->prompt_take()},
            "dblade\\*?>" ,
            "sysconfig\\*?>",        
            "\\*?>",                               
        ];
                          
        ### BUG: get_site_info() doesn't know where site file lives unless an
        ### Option package is created.  The Option instance doesn't see site
        ### variable unless a Server object is created (I'll get site info from
        ### the env myself).
        $self->_log(msg=>"creating server object", indent=>3);
        $self->{serverObj} = new Server(no_cli=>1,nodes=>$args{name});
        $options = new Options();
        $options->process();
        $self->_log(msg=>"retrieving site info", indent=>3);
        if ($ENV{SITE}) {
            $self->_log(
                msg=>"loading site specific params for $ENV{SITE}",
                indent=>5);
            my %site_of         = TestUtils::get_site_info();
            %{$self->{site_of}} = %{$site_of{$ENV{SITE}}};
        }
        else {
            die "No SITE variable found. Define a SITE parameter\n".
                "in your testbed '.ntest' file";
        }
        
        ### Create Vmhost Object           
        $self->{vmhostObj}=new VSA::Vmhost(
			loghdl=>$args{loghdl},
            hostname=>$self->get_vmhost(),
            loghdl=>$self->{loghdl},
            show_cmd=>$args{show_cmd},
	);
      
        if (not defined $self->{datastore}) {
            $self->{datastore}=$self->{vmhostObj}->get_local_datastore();
        }
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    return $self;
}

sub model {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $model,
        @outputs,
    );
    
    $model="TBD";
    
    eval {
        $self->{console_conn} = Tharn::connect(
            $self->{name}."#console",   
            -eval=>sub{
                if (@_) {
                    $errmsg = (@_)[0]->{messages};
                    die @$errmsg;
                    return $self;   
                }
            }
        );
        
        my (undef,undef,@outputs,undef) =
            &Tharn::say(
                $self->{console_conn},
                "sysconfig -a",
                );
        
        ($model)="@outputs"=~/Model Name:\s+(\S+)/i;
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    return $model;
}

###############################################################################
# Method:    create_vm
# Objective: Create the vm
# Details:   See POD documentation at the end of this file.
###############################################################################
sub create_vm {
    my ($self,%args)=@_;
    my (
        $cmd,
        $conn,
        $errmsg,
        $hostname,
        $host_of,
        $line,
        $ova_file_path,
        $vmx_file,
        @localhosts,
        @outputs,
       );
    
    ### Initialize error handler
    $self->_clear_err();
    
    eval {
        ### Verify input parameters
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                image    => { type => SCALAR },
				raid_type=>  {type => SCALAR, 
                        callbacks=>{
                           'legal values'=>sub{shift()=~/RAID0|RAID4|RAID_DP/i
                        }}},
            }
        );
         
        ### Validate that vm_name is not in use
        $self->_log(msg=>"verify vm name is unique on ESX host",indent=>3);
        if ($self->{vmhostObj}->is_exist(vm_name=>$self->get_name())){
            die "VM name '".$self->get_name()."' currently in use.  Choose a ".
                "different name or destroy the existing VM using that name";
        }
    
        ### Parameter assignment 
        # $ova_file_path = $args{image_root}."/".
        #"final/bedrock/export/x86_64.debug.sim.ovf/ovf/DataONTAP.ova";
        $cmd="/usr/software/test/bin/ovftool ".
             "--acceptAllEulas ".
             "--disableVerification ".
             "--name='".      $self->get_name().   "' ".
             "--network='".   $self->get_network_name().       "' ".
             "--datastore='". $self->get_datastore() .     "' ".
             $args{image}." ".
             'vi://root@'.$self->get_vmhost();
         
        ### Create persistent connection to a remote client to execute
        ### ovftool as the root user.  This is because test "cycle
        ### servers" don't allow persistent connections as the root
        ### user for executing local host commands. 
        $self->_log(
            msg=>"creating ssh connection to '".
            $self->{ovf_client}."' to execute ovftool as root",
            indent=>3);
    
        $host_of=Utils::make_host($self->{ovf_client});

        $conn=Tharn::connect(
            $self->{ovf_client}."#ssh;".
            "conntype=ssh-persist;".
            "username=".$host_of->{conn}{ssh}{username}.";".
            "password=".$host_of->{conn}{ssh}{password}.";"
        );
        
        ### Execute ovftool binary 
        $self->_log(
            msg=>"executing ovftool to create virtual machine ".
                 $self->get_name()."\n",
            indent=>3
        );
        
        (undef,undef,@outputs,undef) =
            Tharn::say($conn,$cmd,-prompt=>"Password:",-timeout => 180);
            
        if ("@outputs"=~/command not found/){
            die "@outputs"; 
        }
        else {
            $line="@outputs";
            $line=~s/ Password:/Password: ********/;
            $self->_log(msg=>$line);
        }
	   
        ### Respond to ovftool password prompt
        (undef,undef,@outputs,undef) =
            Tharn::say(
                $conn,$self->{vmhostObj}->get_password() ,-blind,-swallow);
        
        ### Display progress of command while its running
        while ($line !~/Completed successfully/i){
		
            $line=&Tharn::say($conn,"",".*\r",-noprompt,-nocr,-timeout=>300);
			
            if ($line=~m/\*\*/) {
                next;
            }
            elsif ($line=~/Disk progress/){
                my ($nbr)=$line=~/Disk progress: (\d+)/;
                $self->_log(msg=>"Disk progress ...",nocr=>1) if ($nbr==1);
                $self->_log(msg=>$nbr."%",nocr=>1) if ($nbr%10 ==0);
                next; 
            }
            else {
                $self->_log(msg=>$line);
            }
        };

         $self->_vmx_file_additions(raid_type=>$args{raid_type});
            
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    ### return object handle
    return $self;   
}

###############################################################################
# Method:    _vmx_file_additions
# Objective: updates to vmx file definition
###############################################################################
sub _vmx_file_additions {
    my ($self,%args) = @_;
    my (
        $disk,
        $dns_domain,
        $errmsg,
        $lvmObj,
        $mode,
        $nis_domain,
        $nis_servers,
        $root_pw,
        $site,
        $timezone,
        $vmx_file,
        @datastores,
        @dns_servers,
        @lines,
        @outputs,
    );
    
    ### Initialize variables
    $vmx_file="/vmfs/volumes/".
        $self->{vmhostObj}->_get_uuid($self->get_datastore()).
        "/".$self->get_name()."/".$self->get_name().".vmx";
        
    ### UnRegister the new vm   
    $self->_log(
        msg=>"unregistering vm to make vmx file changes",indent=>3);
    $self->{vmhostObj}->unregister(vmx_file=>$vmx_file);
  
    $disk=$self->get_name()."_2.vmdk";
    
    ### Retrieve handle to LVM object
    $lvmObj=$self->{vmhostObj}->create_lvm_obj(lvm=>"vmfs");  
        
    ### Retrieve list of datastore for vm
    @datastores=$lvmObj->get_datastores(name=>$self->get_name());

    ### Set ide disk mode
    $mode="independent-persistent";
    $self->_log(msg=>"set ide disk mode ($mode)",indent=>5);
    $self->{vmhostObj}->config_append (
        datastore => $self->get_datastore(),
        name          => $self->get_name(),
        lines=>[$self->_set_ide_disk_mode(mode=>$mode)],
    );


	if (defined $self->{thost_of}->{NETWORK_4}) {
		$self->_log(msg=>"modifying network name for ".
						$self->{thost_of}->{IP_4},
						indent=>5);
		$self->{vmhostObj}->config_modify (
			datastore => $self->get_datastore(),
			name          => $self->get_name(),
			from => "ethernet3.networkName = \"".$self->get_network_name()."\"",
			to => "ethernet3.networkName = \"".$self->{thost_of}->{NETWORK_4}."\"",
		);	
	}

    ### Move disk to another datastore
    $self->_log(
        msg=>"moving disk $disk to datastore '$datastores[0]'",
        indent=>5);
    
    $self->{vmhostObj}->move_vmdk(
        vm_name=>$self->get_name(),
        vm_path=>"/vmfs/volumes/".$self->get_datastore()."/".$self->get_name(),
        disk=>$disk,
        source=>"/vmfs/volumes/".$self->get_datastore(),
        target=>"/vmfs/volumes/".$datastores[0],
    );


    ### Adding Data Disks
    $self->_log(msg=>"adding data disks",indent=>5);
    $self->{vmhostObj}->config_append (
        name          => $self->get_name(),
        datastore => $self->get_datastore(),
        lines=>[$self->_get_data_disks_config()]
    );

    
    ### Adding Serial Ports
    $self->_log(msg=>"adding serial port info",indent=>5);
    $self->{vmhostObj}->config_append (
        datastore => $self->get_datastore(),
        name          => $self->get_name(),
        lines=>[$self->_get_serial_port_config()]
    );

    ### Adding Setup Info 
    $self->_log(msg=>"adding ontap setup info",indent=>5);
    $self->{vmhostObj}->config_append (
        name          => $self->get_name(),
        datastore => $self->get_datastore(),
        lines=>[$self->_get_ontap_setup_config(raid_type=>$args{raid_type})]
    );
 
    ### Memory
    if ($self->get_memory() !~/4096/){
        $self->_log(msg=>"modifying vm memory to ".
                    $self->get_memory(),indent=>5);
        $self->{vmhostObj}->update_vm_memory(
            name          => $self->get_name(),
            datastore => $self->get_datastore(),
            size=>$self->get_memory()
        );
    }
   
    ### Register the vm 
    $self->_log(msg=>"registering vm",indent=>3);
    $self->{vmhostObj}->register(vmx_file=>$vmx_file);
    
}

###############################################################################
# Method:    _get_ontap_setup_config
# Objective: retrieve params from environment for setting up ONTAP
###############################################################################
sub _get_ontap_setup_config {
    my ($self,%args)=@_;
    my (
        $dns_domain,
        $dns_servers,
        $errmsg,
        $nis_domain,
        $nis_servers,
        $root_pw,
        $site,
        $timezone,
        $vmx_file,
        @dns_servers,
        @lines,
        @outputs,
    );
    
    ### Retrieve information to use for ontap setup
    $dns_domain = (defined $ENV{DNS_DOMAIN})
        ? $ENV{DNS_DOMAIN}
        : $self->{site_of}->{classic_dns_domain_name};
        
    push @dns_servers,
        $self->{site_of}->{mgmt_dns1},
        $self->{site_of}->{mgmt_dns2};
    
    $dns_servers = (defined $ENV{DNS_SERVERS})
        ? $ENV{DNS_SERVERS}
        : join(",",@dns_servers);
    
    $nis_domain = (defined $ENV{NIS_DOMAIN})
        ? $ENV{NIS_DOMAIN}
        : $self->{site_of}->{nis_domain};
    
    $nis_servers = (defined $ENV{NIS_SERVERS})
        ? $ENV{NIS_SERVERS}
        : $self->{site_of}->{nis_servers};
    
    $timezone = (defined $ENV{TIMEZONE})
        ? $ENV{TIMEZONE}
        : $self->{thost_of}->{TIMEZONE};
    
    ### Set up the hash structure containing the machine info to
    ### pass into the install::basic_setup routine.
    @outputs=(
        '',
        'guestinfo.vsa.license="'.$self->get_vsa_license().'"',
        'guestinfo.vsa.hostName="'.$self->get_name().'"',
        'guestinfo.vsa.ipaddr="'.$self->{thost_of}->{MGMT_IP}.'"',
        'guestinfo.vsa.gateway="'.$self->{thost_of}->{MGMT_GATEWAY}.'"',
        'guestinfo.vsa.netmask="'.$self->{thost_of}->{MGMT_MASK}.'"',
        'guestinfo.vsa.raidtype="'.uc($args{raid_type}).'"',
        'guestinfo.vsa.password="'.$self->get_password().'"',
        'guestinfo.vsa.vsphere_hostname="'.$self->{vmhostObj}->get_ip().'"',
        'guestinfo.vsa.vsphere_username="'.
            $self->{vmhostObj}->get_username().'"',
        'guestinfo.vsa.vsphere_password="'.
            $self->{vmhostObj}->get_password().'"',
        '# the variables below are optional',
        'guestinfo.vsa.useweb="NO"',
        'guestinfo.vsa.tmz="'.$timezone.'"',
        'guestinfo.vsa.dns_domainname="'.$dns_domain.'"',
        'guestinfo.vsa.dns_ipaddr="'.$dns_servers.'"',
        'guestinfo.vsa.nis_domainname="'.$nis_domain.'"',
        'guestinfo.vsa.nis_ipaddr="'.$nis_servers.'"',
        '',
    );
    
   return (wantarray()) ? @outputs : [@outputs]; 
}

###############################################################################
# Method:    _display_setup_info
# Objective: display parameters to use for setting up ONTAP
###############################################################################
sub _display_setup_info( ) {
    my ($self,%args) = @_;
    my (
        @bufs,
        @outputs,
    );
    
    ### Format the values to display to STDOUT
    @bufs=$self->{vmhostObj}->config_get(
        name          => $self->get_name(),
        datastore => $self->get_datastore(),
        filter=>"guestinfo"
    );
    if (@bufs){
        $self->_log(msg=>"input responses to use for ONTAP setup", indent=>3);
        push @outputs, sprintf(
            "   %-20.20s %-15.15s\n".
            "   %-20.20s %-15.15s",
            "Parameter",
            "Value",
            "-"x20,
            "-"x15
        );
   
        foreach my $line  (sort @bufs) {
            next if (not $line);
            my ($param,$value)=$line=~/.*\.(\S+)=\"(\S+)\"/i;
            push @outputs,
                sprintf ("   %-20.20s %-s",$param,$value);   
        }
        
        $self->_log(msg=>[@outputs]);
    }
    
    $self->_log(msg=>"");
    
    return $self;
}

################################################################################
# Subroutine Name: _get_data_disks_config
# Objective: Retrieve data disk configuration
# Details: See POD documentation at the end of this file
################################################################################
sub _get_data_disks_config {
    my ($self,%args) = @_;
    my (
        $ctrl_id,
        $target_id,
        $vmdk_cnt,
        @disk_headers,
        @outputs,
    );
   
    ### Initialize variables
    $target_id=0;
    $vmdk_cnt=0;
    
    foreach my $vmdk (
        $self->{vmhostObj}->get_vmdks(vm_name=>$self->get_name())){
        
		### Skip over then nvram_coredump_disk
        next if ($vmdk=~/vsa-\d+\_2\.vmdk$/i);
		
        ### Determine the scsi ctrl to use
        $ctrl_id=$vmdk_cnt++%4;
        
        ### Skip over SCSIn:7 which is reserved for the iscsi
        ### adapter.
        $target_id++ if ($target_id==7);
        
        ### Generate config entries for each device
        my @disk_headers=(
            '',
            'scsi'.$ctrl_id.':'.$target_id.'.present = "TRUE"',
            'scsi'.$ctrl_id.':'.$target_id.'.fileName = "'.$vmdk.'"',
            'scsi'.$ctrl_id.':'.$target_id.'.mode = "independent-persistent"',
            'scsi'.$ctrl_id.':'.$target_id.'.deviceType = "scsi-hardDisk"',
            'scsi'.$ctrl_id.':'.$target_id.'.redo = ""',
            'sched.scsi'.$ctrl_id.':'.$target_id.'.shares = "normal"',
            '',
        );
        
        push @outputs,@disk_headers;

        ### Increment the target id on last ctrl id
        $target_id++ if ($ctrl_id==3);
        
    }
    
    return (wantarray()) ? @outputs : [@outputs];    
}

###############################################################################
# Method:    _set_ide_disk_mode
# Objective: set mode of ide disk 
###############################################################################
sub _set_ide_disk_mode {
    my ($self,%args)=@_;
    my (
        @outputs,
        );
    
    @outputs=(
        '',
        'ide0:0.mode = "'.$args{mode}.'"',
        'ide0:1.mode = "'.$args{mode}.'"',
        'ide1:0.mode = "'.$args{mode}.'"',
        '' , 
    );
    
    return (wantarray()) ? @outputs : [@outputs]; 
}

###############################################################################
# Method:    _get_serial_port_config
# Objective: Retrieve serial port configuration
###############################################################################
sub _get_serial_port_config {
    my ($self, %args)=@_;
    my (
        @outputs,
        );
    
    ### Append entries to config file
    @outputs=(
        '',
        'serial0.present = "TRUE"',
        'serial0.yieldOnMsrRead = "TRUE"',
        'serial0.fileType = "pipe"',
        'serial0.fileName = "/tmp/'.$self->get_name().'_serial_0"',
        'serial0.tryNoRxLoss = "TRUE"',
        '',
        'serial1.present = "TRUE"',
        'serial1.yieldOnMsrRead = "TRUE"',
        'serial1.fileType = "pipe"',
        'serial1.fileName = "/tmp/'.$self->get_name().'_serial_1"',
        'serial1.tryNoRxLoss = "TRUE"',
        '',
    );
    
    return (wantarray()) ? @outputs : [@outputs];  
}

###############################################################################
# Method:    _get_nic_config
# Objective: retrieve nic configuration
###############################################################################
sub _get_nic_config {
    my ($self, %args)=@_;
    my (
        $network,
        @outputs,
        );
	# sed -i.orig 's/ethernet1.networkName = "Management Network"/ethernet1.networkName = "Data Network"/g' csiqa-vsa-2.vmx

   if (defined $self->{thost_of}->{IP_4}) {
		$network=(defined $self->{thost_of}->{NETWORK_4})
			? $self->{thost_of}->{NETWORK_4}
			: $self->get_network_name();
		
		### Append entries to config file
		@outputs=(
			'',
		   'ethernet3.present = "TRUE"',
		   'ethernet3.virtualDev = "e1000"',
		   'ethernet3.networkName = "'.$network.'"',
		   'ethernet3.addressType = "generated"',
			''
		);
    }
   
    return (wantarray()) ? @outputs : [@outputs];  
}


###############################################################################
# Method:    get_vsa_license()
# Objective: Get vsa license()
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_vsa_license {
    my ($self)=@_;
    my $license;
    
    $license = (defined $self->{license})
        ? $self->{license}
        : $License_Of{$self->{model}};
    
    return $license;
}

###############################################################################
# Method:    get_memory()
# Objective: Get vsa memory()
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_memory { my ($self)=@_; return $self->{memory}; }

###############################################################################
# Method:    get_model()
# Objective: Get vsa model()
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_model { my ($self,%args)=@_; return $self->{model}; }


###############################################################################
# Method:    get_name()
# Objective: Get vsim name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_name { my ($self)=@_; return $self->{name};}

###############################################################################
# Method:    get_network_name()
# Objective: Get network name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_network_name { my ($self)=@_; return $self->{network_name};}

###############################################################################
# Method:    get_password()
# Objective: Get password
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_password { my ($self)=@_; return $self->{root_pw};}


###############################################################################
# Method:    get_datastore()
# Objective: Get datastore name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_datastore { my ($self)=@_; return $self->{datastore};}

###############################################################################
# Method:    get_state
# Objective: Return machine state 
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_state {
    my ($self)=@_;
    my (
        $errmsg,
        $state,
    );
    
    ### Initialize error buffer
    $self->_clear_err();
    
    eval {
        ### Retrieve machine state
        $state = boot_state($self->{console_conn});
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    ### return state
    return $state;
    
}

###############################################################################
# Method:    get_vmhost()
# Objective: Get vmhost name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_vmhost { my ($self)=@_; return $self->{vmhost};}

###############################################################################
# Method:    get_vmhost_obj()
# Objective: Get vmhost object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_vmhost_obj { my ($self)=@_; return $self->{vmhostObj};}

###############################################################################
# Method:    setup_vm()
# Objective: setup vsa software
# Details:   See POD documentation at the end of this file.
###############################################################################
sub setup_vm {
    my ($self, %args)=@_;
    my (
	  $ads_domain,
	  $ads_server,
      $buf,
      $errmsg,
      $line,
	  $option,
      $redo,
      @bufs,
      @outputs,
      @progress_markers,
      %input_of,
      %setup_of,
    );
    
    eval {
        ### Verify input parameters
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                feature_licenses   => { type => SCALAR, optional=>1,
                                        default=>1 },
                fake_init => { type => SCALAR, optional=>1,
                                        default=>1 },
            }
        );
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    ### List of message markers to report on when matched       
    @progress_markers=(
		"Automatic setup in progress",
        "starting boot processor",
        "nblade kmod loading",
        "sldiag kmod",
        "clearing fm mailbox",
        "registering local blade",
        "bsd initialization",
        "verifying disk signature",
        "verifying data blocks",
        "erasing disk signature",
        "writing zeroed label",
        "initializing of disks",
        "system rebooting",
        "final disk device",
        "checking whether all disks",
        "all disks are already zeroed",
        "zeroing disks",
        "zeroing nvram",
        "aggr0 has been",
        "created root volume",
        "new flexible volume",
        "configuring onboard ethernet",
        "stored checksum file",
        "completed extracting",
		"Wipe filer procedure requested",
		"bootarg.bootmenu.selection",
		"filer.serialnum",
		"spare disks",
		"create new volumes"
    );
    
    eval {

        ### Create a console connection to vsa
        $self->_log(msg=>"creating a vsa console connection", indent=>3);
        $self->{console_conn} = Tharn::connect(
            $self->{name}."#console",   
            -eval=>sub{
                if (@_) {
                    $errmsg = (@_)[0]->{messages};
                    die @$errmsg;
                    return $self;   
                }
            }
        );
       
        ### Power vm on
        $self->_log(msg=>"powering on vm '".$self->get_name()."'",indent=>3);
        $self->{vmhostObj}->_power_on_vm(vm=>$self->get_name());
        
        ### Break into the loader prompt
        while (1) {
            eval {
                &Tharn::say(
                    $self->{console_conn},
                    "s",  
                    -prompt=>"^$self->{loader}",
                    -timeout => 1, 
                );
            };
            if ($@){
                redo if ($redo++<10);
                $self->_log(msg=>"missed loader prompt, retrying",indent=>3);
            }
            last;
        };
       
        ### Display cfe parameter settings 
        $self->_display_cfe_info(fake_init=>$args{fake_init});
        
        ### Display ontap setup values to be used
        $self->_display_setup_info();
        
        ### Boot ontap
        $self->_log(
            msg=>"booting ontap with auto configure enabled ...",
            indent=>3
        );
        &Tharn::say(
          $self->{console_conn},
          "boot_ontap",  
          -noprompt,
          -timeout => 30,
        );

		$self->_log(msg=>"display progress messages along the way",indent=>3);
        ### Read and display selective console messages
        ### while booting to show progress.  Break out
        ### once password prompt is issued
        while ($line !~/Password:/i){
            (undef,$line,undef,undef)=
                Tharn::say($self->{console_conn},
                             "",
                             -wait=>"\r\n",
                             -noprompt,
                             -nocr,
                             -timeout=>300);
       
            if ( grep($line=~/$_/si,@progress_markers)) {
               $self->_log(msg=>$line,indent=>5);
            }

            if ($line =~ /Hit [Enter] to boot/si){
                $self->_log(msg=>"rebooting vsa ...",indent=>5);
                Tharn::say($self->{console_conn},
                           "\n",
                           -noprompt,
                           -nocr,
                           -timeout=>5);
            }
            elsif ($line=~ /skip dhcp search/si){
                $self->_log(msg=>"skip dhcp search ...",indent=>5);
                Tharn::say($self->{console_conn},
                           "\cc",
                           -noprompt,
                           -nocr,
                           -timeout=>5);
            }
            else {
                next;
            }
        };
        
        ### Provide password
        &Tharn::say(
          $self->{console_conn},
          "netapp1!",  
          -prompt=>[$self->_get_final_prompts()],
          -timeout => 5,
        );            
            
        ### Add licenses
        $self->_license_add() if ($args{feature_licenses});
        
        ### Setting up ntp
        $self->_ntp_setup();
        
        ### Enable protocols
        $self->_protocol_enable();
         
        ### Remove password
		
		eval {
			$option='options security.passwd.rules.enable off';
			
			$self->{serverObj}->console(
				cmd=>$option,
				noprompt=>1,
				additional_prompts_timeout => "10",
				additional_prompts => {
					"$option" => "FINAL_PROMPT",
					"options failed" => "FATAL_PROMPT"
				});
		};
			if ($@){
				$self->_log(msg=>"failed adding option".$option, indent=>5);
		}
		
        $self->_password_remove()
			if (not $self->{thost_of}->default_password());
         
        ### BUG:  Underlying library used to setup CIFS 
        ### isn't aware of special promoted licenses for
        ### VSA.
        if ((not $self->{promoted}) and $args{feature_licenses}) {
            $self->_log(msg=>"");
            
			$ads_server=($ENV{AD_SERVER_FULLDOMAIN})
                    ? $ENV{AD_SERVER_FULLDOMAIN}
                    : 0;
			      
			($ads_domain)=$ads_server=~/\.(.*)/;	  
				  
            %input_of = (
                ads_domain=>$ads_domain,
                #time_server => $self->{site_of}->{mgmt_dns1},
                user=>"adadmin",
                password=>"adadmin",
            );
            
            unless ($input_of{ads_domain}) {
               $self->_log(
                    msg=>"Missing AD_SERVER_FULLDOMAIN in .ntest file. ".
                         "Needed to setup cifs.",
                    indent=>3);
                return $self;
            }
            
            ### Check if an wins address has been defined and
            ### exit if it hasn't
            unless ($ENV{WINS_SERVER_IP}) {
                $self->_log(
                    msg=> "Missing WINS_SERVER_IP in .ntest file.",
                    indent=>3
                );  
                return $self;
            }
       
            %setup_of = ( 
                wins_option        => 1,
                wins_server_list   => $ENV{WINS_SERVER_IP},
                passwd_group       => "yes",
                'security-style'   => "multiprotocol",
                'server-name'      => uc($self->get_name()),
                'auth-type'        => "ad",
                domain             => $input_of{ads_domain},
                domain_user        => "adadmin",
                domain_pass        => "adadmin",
                'ou-name'          => "CN=computers",
                create_admin       => "yes",
                local_admins       => "administrator",
                cifs_pass          => "",
            );
      
            ### Format the values to display to STDOUT
            $self->_log(msg=>"input responses to use for 'cifs setup'", indent=>3);
            push @outputs, sprintf(
                "    %-20.20s %-15.15s\n".
                "    %-20.20s %-15.15s",
                "Parameter",
                "Value",
                "-"x20,
                "-"x15
            );
                 
            foreach my $param (sort keys %setup_of){
                push @outputs,
                    sprintf ("    %-20.20s %-s",$param,$setup_of{$param});            
            }
       
            $self->_log(msg=>[@outputs,"\n"]);
     
			### Sleep 5s to make sure change took affect
			sleep(5);
	 
            $self->_log(msg=>"executing cifs setup ...",indent=>3);
            CIFS::Setup::cifs_setup(
                host               => $self->get_name(),
                %setup_of,
            );
            $self->_log(msg=>"cifs setup completed",indent=>5); 
        }
       
	    #$self->_remove_password();
    };
    if ($@){
		
        $self->_log(msg=>"DBG: $@");
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return undef; 
    }   
    
    ### return object handle
    return $self;
}

###############################################################################
# Method:    _display_cfe_info
# Objective: display cfe variables
###############################################################################
sub _display_cfe_info {
    my ($self,%args)=@_;
      my (
      $buf,
      $errmsg,
      $fake_init,
      $redo,
      @bufs,
      @cmds,
      @outputs,
    );
    
    $fake_init=($args{fake_init}) ? "true" : "false";  
      
    @cmds=(
           "",
           "setenv LINES 250",
	   # KL - 10/01/10
	   # We don't believe we need qatest.fakeinit any more, it's being
	   # commented out for now.
           #"setenv bootarg.qatest.fakeinit ".$fake_init,
           "setenv bootarg.vm.no_poweroff_on_halt true",
           "setenv bootarg.setup.minroot true",
           "setenv bootarg.setup.nic_".$self->{thost_of}->{IFC_1}." ".
                 $self->{thost_of}->{MGMT_IP}.      ";".
                 $self->{thost_of}->{MASK_1}.       ";".
                 $self->{thost_of}->{MEDIATYPE_1}.  ";".
                 $self->{thost_of}->{FLOWCONTROL_1}.";".
                 $self->{thost_of}->{JUMBOFRAMES_1},  
           "setenv bootarg.setup.nic_".$self->{thost_of}->{IFC_2}." ".
                 $self->{thost_of}->{DATA_IP1}.      ";".
                 $self->{thost_of}->{MASK_2}.       ";".
                 $self->{thost_of}->{MEDIATYPE_2}.  ";".
                 $self->{thost_of}->{FLOWCONTROL_2}.";".
                 $self->{thost_of}->{JUMBOFRAMES_2},
           "setenv bootarg.setup.nic_".$self->{thost_of}->{IFC_3}." ".
                 $self->{thost_of}->{DATA_IP2}.      ";".
                 $self->{thost_of}->{MASK_3}.       ";".
                 $self->{thost_of}->{MEDIATYPE_3}.  ";".
                 $self->{thost_of}->{FLOWCONTROL_3}.";".
                 $self->{thost_of}->{JUMBOFRAMES_3},
           );
    
    if (defined $self->{thost_of}->{IP_4}) {
        push @cmds,
            "setenv bootarg.setup.nic_".$self->{thost_of}->{IFC_4}." ".
               $self->{thost_of}->{IP_4}.      ";".
               $self->{thost_of}->{MASK_4}.       ";".
               $self->{thost_of}->{MEDIATYPE_4}.  ";".
               $self->{thost_of}->{FLOWCONTROL_4}.";".
               $self->{thost_of}->{JUMBOFRAMES_4};
    }
    
    push @cmds,"printenv";
    
    foreach my $cmd (@cmds) {  
        (undef,undef,$buf,undef)=&Tharn::say(
            $self->{console_conn},
            $cmd,  
            -prompt=>"^$self->{loader}",
            -timeout => 5,
            );  
    }
     
    @bufs=split("\r\n",$buf); 
     
    if (@bufs){
        $self->_log(msg=>"cfe parameter settings on bootup", indent=>3); 
        push @outputs, sprintf(
            "   %-40.40s %-30.30s\n".
            "   %-40.40s %-30.30s",
            "Parameter",
            "Value",
            "-"x40,
            "-"x30
        );
      
        foreach my $line  (sort @bufs) {
            my ($param,$value)=$line=~/(\S+)=(\S+)/i;
            next if (not $value);
            push @outputs,
                sprintf ("   %-40.30s %-s",$param,$value);   
        }
        push @outputs,"\n";
        $self->_log(msg=>[@outputs]);
    }   
        
    return $self;
}

###############################################################################
# Method:    init_setup
# Objective: Initialize disks and setup ONTAP software
# Details:   See POD documentation at the end of this file.
###############################################################################
sub OLD_init_setupA {
    my ($self,%args) = @_;
    
=comment       
    my (
        $connObj,
        $dns_domain,
        $errmsg,
        $nis_domain,
        $nis_servers,
        $redo,
        $root_pw,
        $site,
        $timezone,
        @dns_servers,
        @outputs,
        %setup_of,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        
        ### Transit to Boot Menu
        unless ($self->get_state() =~ /BOOT_MENU/) { 
            $self->_transit_to(mode=>"BOOT_MENU");
        }
        
        ### Initialize disks 
        $self->_log(msg=>"initing disks (24/7)", indent=>3);
        $self->{serverObj}->console(
            cmd=>"24/7",
            additional_prompts_timeout => "180",
            noprompt => 1,
            additional_prompts => {
              "Install new file system" => "y", 
              "REALLY install file system" => "y",                   
              "How many disks" =>"\n",
              "How many volumes"=>"\n",
              "RAID groups per volume"=>"\n",
              "System ID:"     => "FINAL_PROMPT",
              "Failed to load D-blade" => "FATAL_PROMPT",
            });
         
        ### Sit in loop until hostname prompt is responsive to input.
        ### This is because console input will be blocked while nblade
        ### is coming online. Die if still unresponsive after 60 seconds
        $self->_log(msg=>"waiting for nblade to be ready",indent=>3);
        while (1) { 
            eval {
                $self->{serverObj}->console(
                    cmd=>"\n",
                    additional_prompts_timeout => "1",
                    noprompt => 1,
                    additional_prompts => {
                        "new hostname" => "FINAL_PROMPT",
                });
            };
            if ($@){
                redo if ($redo++<60);
                die "Unable to initialize machine";
            }
            last;
        }
          
        ### Retrieve information to use for setup prompts.
        $dns_domain = $self->{site_of}->{classic_dns_domain_name};
        push @dns_servers,
                      $self->{site_of}->{mgmt_dns1},
                      $self->{site_of}->{mgmt_dns2};
        $nis_domain = $self->{site_of}->{nis_domain};
        #$nis_server = $self->{site_of}->{nis_servers};
        $nis_servers = $ENV{NIS_SERVERS};
        $root_pw    = (length($self->{thost_of}->default_password()))
                        ? $self->{thost_of}->default_password()
                        : "netapp1!";
        $timezone="EST";
        
        ### Set up the hash structure containing the machine info to
        ### pass into the install::basic_setup routine.
        @configs=(
            '',
            'guestinfo.vsa.license="BFIE-2ZLA-PMMR-47NA-F7FE"',
            'guestinfo.vsa.hostName="'.$self->get_name().'"',
            'guestinfo.vsa.ipaddr= "'.$self->{thost_of}->{IFC_1}.'"',
            'guestinfo.vsa.gateway="'.$self->{thost_of}->{GATEWAY}.'"',
            'guestinfo.vsa.netmask="'.$self->{thost_of}->{MASK_1}.'"',
            'guestinfo.vsa.raidtype=RAID0',
            'guestinfo.vsa.password="'.$root_pw.'"',
#            'guestinfo.vsa.vsphere_hostname="10.97.128.56"',
            'guestinfo.vsa.vsphere_username="root"',
            'guestinfo.vsa.vsphere_password="netapp123"',
            '# the variables below are optional',
            'guestinfo.vsa.useweb=NO',
            'guestinfo.vsa.tmz="'.$timezone."'",
            'guestinfo.vsa.dns_domainname="'.$dns_domain.'"',
            'guestinfo.vsa.dns_ipaddr="'.join(",",@dns_servers).'"',
            'guestinfo.vsa.nis_domainname="'.$nis_domain.'"',
            'guestinfo.vsa.nis_ipaddr="'.$nis_servers.'"',

        );
        
      
        
        ### Format the values to display to STDOUT
        $self->_log(msg=>"input responses to use for ONTAP setup", indent=>3);
        push @outputs, sprintf(
            "   %-30.30s %-15.15s\n".
            "   %-30.30s %-15.15s",
            "Parameter",
            "Value",
            "-"x30,
            "-"x15
        );
             
        foreach my $param (sort keys %setup_of){
            push @outputs,
                sprintf ("   %-30.30s %-s",$param,$setup_of{$param});            
        }
        
        $self->_log(msg=>[@outputs]);
        
        ### Log the setup hash to show what values were defined
        $Log->comment("Setup Parameters:\n".Dumper(%setup_of)); 

        ### Setup the machine
        $self->_log(
            msg=>"setting up machine info and extracting images",
            indent=>3);
        
        install::basic_setup( 
            $self->{console_conn},
            issue_setup_cmd=>0,
            setup=>\%setup_of,
            timeout=>120
        );      
           
        ### Wait for rootpassword prompt
        eval {
            $self->{serverObj}->console(
                cmd=>" ",
                additional_prompts_timeout => "120",
                noprompt => 1,
                additional_prompts => {
                    "^Password:"     => "FINAL_PROMPT",
                    }
            );
        };
        
        ### Enter root password
        $self->{serverObj}->console(
            cmd=>$root_pw,
            additional_prompts_timeout => "30",
            noprompt => 1,
            additional_prompts => {
                 $self->_get_final_prompts(),
              
        });
       
        ### Add licenses
        $self->_license_add();
        
        ### Enable protocols
        $self->_protocol_enable();
        
        ### Setting up ntp
        $self->_ntp_setup();
         
        ### Remove password
        $self->_password_remove();
        
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
=cut     
    return $self;  
}

###############################################################################
# Method:    additions
# Objective: updates to vmx file definition
###############################################################################
sub OLD_additions {
    my ($self,%args) = @_;
    my (
        $errmsg,
    );
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        
       ## Create a console connection to the vsa 
        $self->_log(msg=>"creating a vsa console connection", indent=>3);
        $self->{console_conn} = Tharn::connect($self->{name}."#console");
        
   
        ### Add licenses
        $self->_license_add();
        
        ### Enable protocols
        $self->_protocol_enable();
        
        ### Setting up ntp
        $self->_ntp_setup();
         
        ### Remove password
        $self->_password_remove();
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }    
        
    return $self;    
}


sub _OLD_get_nics {
    my ($self,%args)=@_;
    
    #bootarg.setup.nic.e0d="10.97.129.72;255.255.248.0;auto;full;n"
    $self->_log ( msg=>
    "setenv bootarg.setup.nic.".$self->{thost_of}->{IFC_1}." ".
         $self->{thost_of}->{MGMT_IP}.      ";".
         $self->{thost_of}->{MASK_1}.       ";".
         $self->{thost_of}->{MEDIATYPE_1}.  ";".
         $self->{thost_of}->{FLOWCONTROL_1}.";".
         $self->{thost_of}->{JUMBOFRAMES_1}
    );
    
    $self->_log ( msg=>
    "setenv bootarg.setup.nic.".$self->{thost_of}->{IFC_2}." ".
         $self->{thost_of}->{DATA_IP1}.      ";".
         $self->{thost_of}->{MASK_2}.       ";".
         $self->{thost_of}->{MEDIATYPE_2}.  ";".
         $self->{thost_of}->{FLOWCONTROL_2}.";".
         $self->{thost_of}->{JUMBOFRAMES_2}
         );
    
    $self->_log ( msg=>     
    "setenv bootarg.setup.nic.".$self->{thost_of}->{IFC_3}." ".
         $self->{thost_of}->{IP_e0d}.      ";".
         $self->{thost_of}->{MASK_3}.       ";".
         $self->{thost_of}->{MEDIATYPE_3}.  ";".
         $self->{thost_of}->{FLOWCONTROL_3}.";".
         $self->{thost_of}->{JUMBOFRAMES_3}
    );
    
  #  if (defined $self->{thost_of}->{IP_4}) {
        $self->_log ( msg=>     
        "setenv bootarg.setup.nic.".$self->{thost_of}->{IFC_4}." ".
             $self->{thost_of}->{IP_4}.      ";".
             $self->{thost_of}->{MASK_4}.       ";".
             $self->{thost_of}->{MEDIATYPE_4}.  ";".
             $self->{thost_of}->{FLOWCONTROL_4}.";".
             $self->{thost_of}->{JUMBOFRAMES_4}
        );
   # }
   
}

###############################################################################
# Method:    init_setup
# Objective: Initialize disks and setup ONTAP software
# Details:   See POD documentation at the end of this file.
###############################################################################
sub OLD_init_setupB {
    my ($self,%args) = @_;
    my (
        $connObj,
        $dns_domain,
        $errmsg,
        $nis_domain,
        $nis_servers,
        $redo,
        $root_pw,
        $site,
        @dns_servers,
        @outputs,
        %setup_of,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        
        ### Transit to Boot Menu
        unless ($self->get_state() =~ /BOOT_MENU/) { 
            $self->_transit_to(mode=>"BOOT_MENU");
        }
        
        ### Initialize disks 
        $self->_log(msg=>"initing disks (24/7)", indent=>3);
        $self->{serverObj}->console(
            cmd=>"24/7",
            additional_prompts_timeout => "180",
            noprompt => 1,
            additional_prompts => {
              "Install new file system" => "y", 
              "REALLY install file system" => "y",                   
              "How many disks" =>"\n",
              "How many volumes"=>"\n",
              "RAID groups per volume"=>"\n",
              "System ID:"     => "FINAL_PROMPT",
              "Failed to load D-blade" => "FATAL_PROMPT",
            });
         
        ### Sit in loop until hostname prompt is responsive to input.
        ### This is because console input will be blocked while nblade
        ### is coming online. Die if still unresponsive after 60 seconds
        $self->_log(msg=>"waiting for nblade to be ready",indent=>3);
        while (1) { 
            eval {
                $self->{serverObj}->console(
                    cmd=>"\n",
                    additional_prompts_timeout => "1",
                    noprompt => 1,
                    additional_prompts => {
                        "new hostname" => "FINAL_PROMPT",
                });
            };
            if ($@){
                redo if ($redo++<60);
                die "Unable to initialize machine";
            }
            last;
        }
          
        ### Retrieve information to use for setup prompts.
        $dns_domain = $self->{site_of}->{classic_dns_domain_name};
        push @dns_servers,
                      $self->{site_of}->{mgmt_dns1},
                      $self->{site_of}->{mgmt_dns2};
        $nis_domain = $self->{site_of}->{nis_domain};
        #$nis_server = $self->{site_of}->{nis_servers};
        $nis_servers = $ENV{NIS_SERVERS};
        $root_pw    = (length($self->{thost_of}->default_password()))
                        ? $self->{thost_of}->default_password()
                        : "netapp1!";
        
        ### Set up the hash structure containing the machine info to
        ### pass into the install::basic_setup routine.
        %setup_of = (
            HOSTNAME              => $self->{name}, 
            IFC_1                 => $self->{thost_of}->{IFC_1},
            IP_1                  => $self->{thost_of}->{MGMT_IP},
            MASK_1                => $self->{thost_of}->{MASK_1},
            MEDIATYPE_1           => $self->{thost_of}->{MEDIATYPE_1},
            FLOWCONTROL_1         => $self->{thost_of}->{FLOWCONTROL_1},
            JUMBOFRAMES_1         => $self->{thost_of}->{JUMBOFRAMES_1},
            
            IFC_2                 => $self->{thost_of}->{IFC_2},
            IP_2                  => $self->{thost_of}->{DATA_IP1},
            MASK_2                => $self->{thost_of}->{MASK_2},
            MEDIATYPE_2           => $self->{thost_of}->{MEDIATYPE_2},
            FLOWCONTROL_2         => $self->{thost_of}->{FLOWCONTROL_2},
            JUMBOFRAMES_2         => $self->{thost_of}->{JUMBOFRAMES_2},
            
            IFC_3                 => $self->{thost_of}->{IFC_3},
            IP_3                  => $self->{thost_of}->{IP_e0d},
            MASK_3                => $self->{thost_of}->{MASK_3},
            MEDIATYPE_3           => $self->{thost_of}->{MEDIATYPE_3},
            FLOWCONTROL_3         => $self->{thost_of}->{FLOWCONTROL_3},
            JUMBOFRAMES_3         => $self->{thost_of}->{JUMBOFRAMES_3},
            
            TIMEZONE              => "America/New_York", 
            GATEWAY               => $self->{thost_of}->{GATEWAY},
            LOCATION              => $self->{thost_of}->{LOCATION},
            DNS                   => "y",
            DNS_DOMAIN            => $dns_domain,
            DNS_SERVER_LIST       => join(",",@dns_servers),
            NIS                   => "y",
            NIS_DOMAIN            => $nis_domain,
            NIS_SERVER_LIST       => $nis_servers,
            ROOT_PASS             => $root_pw,
            PRIMARY_CONTACT_NAME  => $ENV{USER},
            PRIMARY_CONTACT_EMAIL => "$ENV{USER}\@netapp.com",
        );
        
        ### Format the values to display to STDOUT
        $self->_log(msg=>"input responses to use for ONTAP setup", indent=>3);
        push @outputs, sprintf(
            "   %-30.30s %-15.15s\n".
            "   %-30.30s %-15.15s",
            "Parameter",
            "Value",
            "-"x30,
            "-"x15
        );
             
        foreach my $param (sort keys %setup_of){
            push @outputs,
                sprintf ("   %-30.30s %-s",$param,$setup_of{$param});            
        }
        
        $self->_log(msg=>[@outputs]);
        
        ### Log the setup hash to show what values were defined
        $Log->comment("Setup Parameters:\n".Dumper(%setup_of)); 

        ### Setup the machine
        $self->_log(
            msg=>"setting up machine info and extracting images",
            indent=>3);
        
        install::basic_setup( 
            $self->{console_conn},
            issue_setup_cmd=>0,
            setup=>\%setup_of,
            timeout=>120
        );      
           
        ### Wait for rootpassword prompt
        eval {
            $self->{serverObj}->console(
                cmd=>" ",
                additional_prompts_timeout => "120",
                noprompt => 1,
                additional_prompts => {
                    "^Password:"     => "FINAL_PROMPT",
                    }
            );
        };
        
        ### Enter root password
        $self->{serverObj}->console(
            cmd=>$root_pw,
            additional_prompts_timeout => "30",
            noprompt => 1,
            additional_prompts => {
                 $self->_get_final_prompts(),
              
        });
       
        ### Add licenses
        $self->_license_add();
        
        ### Enable protocols
        $self->_protocol_enable();
        
        ### Setting up ntp
        $self->_ntp_setup();
         
        ### Remove password
        $self->_password_remove();
        
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
     
    return $self;  
}

###############################################################################
# Method:    on_err
# Objective: Return errhdlr_type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub on_err {
    my ($self,$value)=@_; 
    my ($errmsg);
    
    if (defined $value) {
        
        unless ($value=~/die|rtn/){
            $errmsg="Invalid value '".$value."'";
            die _caller($errmsg);
        }
        
        $self->{on_err}=$value;
    }
    
    return $self->{on_err}; 
}

################################################################################
###  P  R  I  V  A  T  E
################################################################################
# Subroutine Name: _get_final_prompts
# Objective: Return hash of final mode wait prompts
################################################################################
sub _get_final_prompts {
    my ($self,%args) =@_;
    my (%hsh);
    
    foreach my $mode_prompt (@{$self->{mode_prompts}}){
		if ($mode_prompt =~ /^\^/) {
			$hsh{$mode_prompt}="FINAL_PROMPT";
		} else {
			$hsh{"^".$mode_prompt}="FINAL_PROMPT";
		}
    }
    
    return %hsh;
}

################################################################################
# Subroutine Name: _add_license
# Objective: Add standard licenses 
################################################################################
sub _license_add {
    my ($self,%args)=@_;
    my (
        $build_type,
        $license_of,
        $model,
        @outputs,
    );
    
    ### Turn dbg mode to off
    $self->{serverObj}->console(
        cmd=>"priv set -q test; dbg to off",
        noprompt=>1
    );
    
    ### List of default license codes
    $license_of = {
        'DOvM100' => {
            nonpromoted=>{
                cifs        => "GTLYRVH", 
                flex_clone  => "QNFYRVH",
                http        => "AODUWOJ",
                iscsi       => "IKVAREM" ,
                nfs         => "UYTCNCG",
                snapmirror  => "UUNLDQC",
                snaprestore => "EOYPBIL",
            },
            promoted=>{
                cifs        => "XTVSPYC",     
                flex_clone  => "UHDXNQL",   
                http        => "HVQHACC",   
                iscsi       => "VMDKZKG",    
                nfs         => "VKGXNQL",    
                snapmirror  => "XZMQTAJ",    
                snaprestore => "XHVUOHH",        
            },
        },
        'FCvM100' => {
            nonpromoted=>{},
            promoted=>{},
        },
        'DOvM10' => {
            nonpromoted=>{
                cifs        => "GTLYRVH", 
                flex_clone  => "QNFYRVH",
                http        => "AODUWOJ",
                iscsi       => "IKVAREM" ,
                nfs         => "UYTCNCG",
                snapmirror  => "UUNLDQC",
                snaprestore => "EOYPBIL",
            },
            promoted=>{
                cifs        => "PPBSJZC", 	 	 
                flex_clone  => "MDJWHRL", 	 	 
                http 	    => "ZQWGUCC", 	 
                iscsi 	    => "NIJJTLG", 	  
                nfs 	    => "NGMWHRL", 	 
                snapmirror  => "PVSPNBJ", 	 	 
                snaprestore => "PDBUIIH" 
            }
        }
    };
               
    ### Add license codes
    $self->_log(msg=>"");   
    $self->_log(msg=>"adding licenses", indent=>3);
    
    ### NetApp Release RboilermakerN_100223_0300 7-Mode: Tue Feb 23 03:36:02 PST 2010   <1d>
    ### NetApp Release 8.0.1X5 7-Mode: Sun Feb 28 03:23:26 PST 2010
    $self->_log(
        msg=>"get model and build to determine correct license code set",
        indent=>3
    );
    
    (undef,undef,@outputs,undef) =
         &Tharn::say(
             $self->{console_conn},
             "sysconfig -a",
             );
     
     ($model)="@outputs"=~/Model Name:\s+(\S+)/i;
     $self->{model}=$model;
        
    $model=$self->get_model();
    $self->_log(msg=>"model name: ".$model,indent=>5);
    
    (undef,undef,@outputs,undef)=&Tharn::say(
        $self->{console_conn},
        "version",
        -timeout=>5,
    );
    
    $build_type=("@outputs"=~/\</)
        ? "nonpromoted"
        : "promoted";
    
    ### Flag to indicate whether to run CIFS SETUP.  CIFS library
    ### doesn't yet know about special licenses for promoted VSA
    ### builds
    $self->{promoted} = ($build_type=~/^promoted$/)
        ? 1
        : 0;
    
    $self->_log(msg=>"build type: ".$build_type,indent=>5);
   
    foreach my $license (sort keys %{$license_of->{$model}{$build_type}}) {
        eval {
            $self->_log(msg=>sprintf(
                "%-23.23s %-s",
                $license,
                $license_of->{$model}{$build_type}{$license}),
                indent=>5);
            
            ### BUG (Server.pm): Individual server calls taking
            ### close to 2s.  Use say() instead
            (undef,undef,@outputs,undef) = &Tharn::say(
                $self->{console_conn},
                "license add ".$license_of->{$model}{$build_type}{$license},
                -prompt=>"has been installed",
                -timeout => 30, 
            );
            
            ### Throw a warning if license is invalid
            $self->_log(msg=>"WARNING: @outputs",indent=>5)
                if ("@outputs" =~ /invalid/i);
            
        };
        if ($@){
            $self->_log(msg=>"failed adding license ".$license, indent=>5);
        }
    }
 
    ### return object handle
    return $self;   
}



################################################################################
# Subroutine Name: _password_remove
# Objective: Clear root password
################################################################################
sub _password_remove {    
    my ($self,%args)=@_;
    my (
        $option,
    );
    
    ### Turn off password rules so a blank password can
    ### can be used
    $self->_log(msg=>"");   
    $self->_log(msg=>"removing root password", indent=>3);
    
	### HACK: Sleep 5s to assure console messages from previous
	### command don't interfere with prompt matching
	sleep(5);
	
    ### Enter blank password
    eval {
        $self->{serverObj}->console(
            cmd=>"passwd",
            noprompt=>1,
            additional_prompts_timeout => "10",
            additional_prompts => {
                "New password:" => "\n",
                "Retype new password:" => "\n",
                 $self->_get_final_prompts(),
        });     
        
    };
    if ($@){
        $self->_log(msg=>"failed removing root password", indent=>5);
    }
    
    return $self;
}

################################################################################
# Subroutine Name: _protocol_enable
# Objective: set list of standard options
################################################################################
sub _protocol_enable {
    my ($self,%args)=@_;
    my (
        %options,
    );
    
    ### List of standard options
    %options = (
       'httpd.admin'  => 'httpd.admin.enable',
        http   => 'httpd.enable',
        rsh    => 'rsh.enable', 
        ssh    => 'ssh.enable', 
        telnet => 'telnet.enable',
    );
    
    $self->_log(msg=>"");   
    $self->_log(msg=>"enabling protocols", indent=>3);
    foreach my $option (sort keys %options) {
        eval {
            $self->_log(msg=>$option, indent=>5);   
                        
            ### BUG (Server.pm): Individual server calls taking close to 2s.
            ### Using say() instead
            &Tharn::say(
                $self->{console_conn},
                "options ".$options{$option}." on",
                -prompt=>"$option",
                -timeout => 30, 
            ); 

       
        };
        if ($@){
            $self->_log(msg=>"failed adding option".$option, indent=>5);
        }
    }
    
}

################################################################################
# Subroutine Name: _ntp_setup
# Objective: set up ntp
################################################################################
sub _ntp_setup {
    my ($self,%args)=@_;
    my (
        $cmd,
        @cmds,
    );
    
    ### Check if an ntp address has been defined and
    ### exit if it hasn't
    unless ($ENV{NTP_SERVER_IP}) {
        $self->_log(
            msg=> "Missing NTP_SERVER_IP in .ntest file.",
            indent=>3
        );
        return $self;
    }
    
    ### Commands to initialize data and set up ntp server 
    push @cmds,
        'options timed.enable off',
        'options timed.servers '.$ENV{NTP_SERVER_IP},
        #'date '.strftime ("%Y%m%d%H%M.%S", localtime),
        'rdate '.$ENV{NTP_SERVER_IP},
        'options timed.enable on';
    
    $self->_log(msg=>"");      
    $self->_log(msg=>"setting up ntp",indent=>3);        
    eval {
        
        ### Execute commands
        foreach $cmd (@cmds) {
            $self->_log(msg=>$cmd,indent=>5);
            &Tharn::say(
                $self->{console_conn},
                $cmd,
                -prompt => "$cmd",
                -timeout => 30, 
            );
        }  
    };
    if ($@){
        $self->_log(msg=>"failed  running ".$cmd."\n$@", indent=>5);
    }
    
    return $self;
}

###############################################################################
# Method:    assign_disks
# Objective: Assign disks to system
# Details:   See POD documentation at the end of this file.
###############################################################################
sub OLD_assign_disks {
    my ($self,%args)=@_;
    my (
       $errmsg,
       $output,
       $state,
       $success,
       $sysid,
       @channels,
    );
    
    ### Initialize error buffer
    $self->_clear_err();
    
    eval {
        
        ### Transit to MAINT 
        unless ($self->get_state() =~ /MAINT/) {
            $self->_transit_to(mode=>"MAINT");
        }
        
        ### Retrieve System id
        $output =
            $self->{serverObj}->console(
                cmd=>"disk show -v",
                additional_prompts_timeout => "30",
                noprompt => 1,
                additional_prompts => {
                    "\\*>"              => "FINAL_PROMPT",
                });
        
        ($sysid)=$output =~ /System ID: (\d+)$/m;
        
        ### Assign all disks to system id
        $self->_log(msg=>"assigning disks to system id ".$sysid, indent=>3);
        $self->{serverObj}->console(
            cmd => "disk assign all -s ".$sysid,
            additional_prompts_timeout => 30,
            noprompt => 1,
            additional_prompts => {
                "partner is NOT"     => "y",
                "continue"           => "y",
                "No disks found"     => "FATAL_PROMPT",
                $self->_get_final_prompts(),
            });
        
        ### Retrieve disk output
        $output=$self->{serverObj}->console(
            cmd=>"disk show -v",
            additional_prompts_timeout => "60",
            noprompt => 1,
            additional_prompts => {
                 $self->_get_final_prompts(),
              
            });
        
        $self->_log(msg=>$output."\n", indent=>3);
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
     
    return $self;
}

###############################################################################
# Method:    cfe
# Objective: Set up firmware environment
# Details:   See POD documentation at the end of this file.
###############################################################################
sub OLD_cfe {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        @outputs,
    );
    
    ### Initialize error buffer
    $self->_clear_err();
    
    ### Transit to firmware
    unless ($self->get_state() =~ /FIRMWARE/) {
        $self->_transit_to(mode=>"FIRMWARE");
    }
        
    eval{
        ($self,%args) = validate_mixed(
            @_,
            [ { type => OBJECT} ],
            {
            'AUTOBOOT' => {type=>SCALAR, optional=>1, default=>"false"},
            'ONTAP_NG' => {type=>SCALAR, optional=>1, default=>"true"},
            'USE_SECONDARY' => {type=>SCALAR, optional=>1, default=>''},
            'bootarg.activatebsdport' =>
                        {type=>SCALAR, optional=>1, default=>"false"},
            'bootarg.autosupport.tag' =>
                        {type=>SCALAR, optional=>1, default=>''},             
            'bootarg.bootmenu.allowall' =>
                        {type=>SCALAR, optional=>1, default=>"true"},
            'bootarg.bsdportname' =>
                        {type=>SCALAR, optional=>1,
                         default=>$self->{thost_of}{MGMT_PORT}},
            'bootarg.dblade.aggr_default_64bit' =>
                        {type=>SCALAR, optional=>1, default=>"false"},
            'bootarg.init.boot_clustered' =>
                        {type=>SCALAR, optional=>1, default=>"false"},    
            'bootarg.init.cf_new' =>
                        {type=>SCALAR, optional=>1, default=>"0"},
            'bootarg.init.clearvarfsnvram' => 
                        {type=>SCALAR, optional=>1, default=>"true"},
            'bootarg.init.console_muted' =>
                        {type=>SCALAR, optional=>1, default=>"true"},
            'bootarg.init.dhcp.disable' =>
                        {type=>SCALAR, optional=>1, default=>"true"},
            'bootarg.init.nokmod' =>
                        {type=>SCALAR, optional=>1, default=>''},
            'bootarg.init.trace_on_panic' =>
                        {type=>SCALAR, optional=>1, default=>"1"},
            'bootarg.init.use_RR_behavior' =>
                        {type=>SCALAR, optional=>1, default=>"false"},
            'bootarg.init.usebootp' =>
                        {type=>SCALAR, optional=>1, default=>"false"},
            'bootarg.mgmt.autoconf.cluster' =>
                        {type=>SCALAR, optional=>1, default=>''}, 
            'bootarg.mgmt.autoconf.nodename' =>
                        {type=>SCALAR, optional=>1, default=>''},           
            'bootarg.mgwd.autoconf.disable' =>
                        {type=>SCALAR, optional=>1, default=>"true"},
            'bootarg.mgwd.autoconf.fakeinit' =>
                        {type=>SCALAR, optional=>1, default=>''},   
            'bootarg.mgwd.autoconf.mgmtport' =>
                        {type=>SCALAR, optional=>1,
                         default=>$self->{thost_of}{MGMT_PORT}},
            'bootarg.mgwd.maxlogfiles' =>
                        {type=>SCALAR, optional=>1, default=>"15"},
            'bootarg.rlm.gdb' =>
                        {type=>SCALAR, optional=>1, default=>"1"},
            'bootarg.vsa.default.raidtype' =>
                        {type=>SCALAR, optional=>1, default=>"0"},
            'console' =>
                        {type=>SCALAR, optional=>1, default=>"comconsole"},
            'SYS_MODEL' => {type=>SCALAR, optional=>1, default=>"DOvM100"},
            },
        );
 

        push @outputs,
             sprintf("   %-30.30s %-15.15s\n".
                     "   %-30.30s %-15.15s",
                     "Parameter",
                     "Value",
                     "-"x30,
                     "-"x15
                    );
        
        foreach my $cfe_param (sort keys %args) {

            $cmd = (length $args{$cfe_param})
                ? "setenv ".$cfe_param." ".$args{$cfe_param}
                : "unsetenv ".$cfe_param;
            
            ### BUG: The call $Server->setenv() takes close to 2s
            ### to issue each command request.  Use say() instead
            &Tharn::say(
                $self->{console_conn},
                $cmd,
                -prompt=>"^SIMLOADER",
                -timeout => 5, 
            );
            
            if (length $args{$cfe_param}) {
                push @outputs,  
                    sprintf ("   %-30.30s %-s",$cfe_param,$args{$cfe_param});
            }
        }
    
        $self->_log(msg=>[@outputs]);
    
    }; 
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
     
    return $self;
}

###############################################################################
# Subroutine Name: cifs_setup
# Objective: add cifs setup 
################################################################################
sub OLD_cifs_setup {
    my ($self,%args)=@_;
    my (
        $errmsg,
        %input_of,
    );
    
    #$self->_log(msg=>"IMPLEMENTATION PROBLEMATIC!!! SORRY ",indent=>3);
    #return $self;
    
    ### Initialize error buffer
    $self->_clear_err();
    
    eval {
        
        ### Transit to CLI
        $self->_log(msg=>"verifying state is UP",indent=>3);
        unless ($self->get_state() =~ /UP/) { 
            $self->_transit_to(mode=>"UP");
        }
        
        %input_of = (
            ads_domain=>($ENV{AD_SERVER_FULLDOMAIN})
                ? ($ENV{AD_SERVER_FULLDOMAIN}=~/\.(\S+)/)
                : 0,
           # time_server => $self->{site_of}->{mgmt_dns1},
            user=>"adadmin",
            password=>"adadmin",
        );
        
        unless ($input_of{ads_domain}) {
           $self->_log(
                msg=>"Missing AD_SERVER_FULLDOMAIN in .ntest file. ".
                     "Needed to setup cifs.",
                indent=>3);
            return $self;
        }
        
        ### Check if an wins address has been defined and
        ### exit if it hasn't
        unless ($ENV{WINS_SERVER_IP}) {
            $self->_log(
                msg=> "Missing WINS_SERVER_IP in .ntest file.",
                indent=>3
            );  
            return $self;
        }
   
        
        foreach my $key (sort keys %input_of) {
            $self->_log(msg=>$key.": ".$input_of{$key}, indent=>3);
        }
            
        ### Initialize disks 
        $self->{serverObj}->console(
            cmd=>"cifs setup",
            additional_prompts_timeout => "60",
            swallow=>1,
            noprompt => 1,
            additional_prompts => {
                "change the current filer account" => "y",
                "be an NTFS-only filer\\?"         => "n",
                "visible via WINS\\?"              => "y",
                #'enter up to 4"                   => "",
                "IPv4 address\\(es\\)"             => $ENV{WINS_SERVER_IP},
                "additional WINS name"             => "n",
                "Selection \\(1-2\\)\\?"           => "1",
                "Should CIFS create default"       => "y",
                "password for the root user"       => "\n",
                "Retype the password:"             => "\n",
                "to enable NIS group caching\\?"   => "y",
                "update the group cache"           => "24",
                "specify additional hours"         => "n",
                "change this name\\?"              => "\n",
                "Active Directory domain authent"  => "1",
                "Selection \\(1-4\\)\\?"           => "1",   
                "name of the Active Directory"     => $input_of{ads_domain},
                "configure time services\\?"       => "n",
               #"time server host(s)"              => $input_of{time_server},
                "additional time servers"          => "\n",
                "name of the Windows user"         => $input_of{user},
                "Password for ".$input_of{user}.":"=> $input_of{password},
                "re-use this machine"              => "y",
                "administrator account"            => "y",
                "group that can administer CIFS"   => "y",
                "administer CIFS on the filer"     => "administrator",
                "Selection \\(1-3\\)\\?"           => "\n",
                "Do you want to create"            => "y",
                "Enter the new password"           => "\n",
                "CIFS local server is running."    => "FINAL_PROMPT",
                "while CIFS is active"             => "FINAL_PROMPT",
            });
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
 
    ### return object handle
    return $self;   
}
################################################################################
# Subroutine Name: _reboot_to
# Objective: reboot machine to requested mode
################################################################################
sub _OLD_reboot_to {
    my ($self,%args)=@_;
    
    ### Validate input parameters
    ($self,%args) = validate_mixed (
        @_,
        [ { type => OBJECT} ],
        {
         mode    => {type => SCALAR, 
                       callbacks=>{
                          'legal values'=>sub{shift()=~
                            /FIRMWARE|BOOT_MENU|MAINT|UP/i
                       }}},
         timeout => {type => SCALAR, optional=>1, default=>60,
                              regex => qr/^\d+$/ },
        }
    );
    
    ### Transit to the requested state
    $self->_log(msg=>"rebooting to ".uc($args{mode})." mode", indent=>3);            
    
    boot::halt(
        $self->{console_conn},
        $args{mode},
        timeout=>$args{timeout});
    
    ### If mode is MAINT turn dbg off 
    if ($args{mode}=~/MAINT/i){
        $self->_log(msg=>"turning dbg to off ",indent=>3);
        $self->{serverObj}->console(cmd=>"dbg to off", noprompt=>1);
    }
   
    return $self;
}

################################################################################
# Subroutine Name: _transit_to
# Objective: transition machine to requested state
################################################################################
sub _OLD_transit_to {
    my ($self,%args)=@_;
    
    ### Validate input parameters
    ($self,%args) = validate_mixed (
        @_,
        [ { type => OBJECT} ],
        {
         mode    => {type => SCALAR, 
                       callbacks=>{
                          'legal values'=>sub{shift()=~
                            /FIRMWARE|BOOT_MENU|MAINT|UP/i
                       }}},
         timeout => {type => SCALAR, optional=>1, default=>60,
                              regex => qr/^\d+$/ },
        }
    );
    
    ### Transit to the requested state
    $self->_log(msg=>"transitioning to ".uc($args{mode})." mode", indent=>3);            
    
    boot::transit_to(
        $self->{console_conn},
        $args{mode},
        timeout=>$args{timeout});
    
    ### If mode is MAINT turn dbg off 
    if ($args{mode}=~/MAINT/i){
        $self->_log(msg=>"turning dbg to off ",indent=>3);
        $self->{serverObj}->console(cmd=>"dbg to off", noprompt=>1);
    }
   
    return $self;
}


1;
