# Hey emacs, this is basically -*- perl -*-
# $Id$
#####################################################################
## Testcase:    config.pm
## Description: VSA setup and configuring perl module
## Author:      Gary Knopp  (Gary.knopp@netapp.com)
## Modified by: Ashrut Vora (ashrut.vora@netapp.com)
## Modified Release:     May 2013
## Team:        VSA QA (dl-vsa-qa@netapp.com)
#####################################################################

package VSA::Config;

use strict;
use warnings;
#use CIFS::Setup qw(is_cifs_setup cifs_setup);
use Data::Dumper qw(Dumper);
use Error qw(_clear_err _set_err);
use File::Basename qw(basename);
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR SCALARREF BOOLEAN OBJECT ARRAYREF UNDEF);
use POSIX qw(strftime);


#use TestUtils qw(get_site_info);
use Tharn qw($Log host);

use VSA::Vmhost;
use VSA::UICommon qw(_caller _reformat_err _log _refresh_cache
                     _size_to_mbytes _trim);
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::STask::Node;
use NACL::C::CommandInterface;
use NACL::C::ExportPolicyRule;
use NACL::C::Failover;
use NACL::C::ClusterHa;
use NACL::C::Aggregate;
use NACL::C::Vserver;
use NACL::C::RoutingGroups;
use NACL::STask::SystemLicense;
use NACL::Transit;
use NACL::MTask::CIFS;
use NACL::STask::NetworkInterface;
use NACL::STask::NetworkRoutingGroupsRoute;
use NACL::C::SystemNodeDate;
use NACL::C::SystemServicesNtpServer;
use NACL::C::VserverNfs;
use NACL::C::VserverServicesUnixUser;
use NACL::C::VserverServicesUnixGroup;
use NACL::C::VserverServicesNisDomain;
use NACL::C::VserverServicesDns;
#use NATE::Exceptions qw(:try);

#use install;
#use boot;
my %Model_Of=(
  DOVM100=>"DOvM100",
  DOVM200=>"DOvM200",
  CBVM100=>"CBvM100",
  FDVM100=>"FDvM100",
  FDVM200=>"FDvM200",
  CBVM200=>"CBvM200",
);

my %Raid_Of=(
  RAID0   => "RAID0",
  RAID4   => "RAID4",
  RAID_DP => "RAID-DP"
);

my %Hosttype_Of=(
  C => "filer-ng-vsa",
  7 => "filer-vsa",
  SNC => "filer-ng-vsa",
);

my $Options;
use base qw(VSA::UICommon Error);

################################################################################
# Subroutine Name: new()
# Objective:       Create a new ESXHost object
# Details:         See POD documentation at the end of this file
################################################################################
sub new {
    my ($class, %args) = @_;
    my (
        $errmsg,
        $options,
        $self,
        @ic_macs,
        @licenses,
        @nodes,
        %site_of,
        );

    ### bless instance into class
    $self= bless{}, $class;

    ### Clear error buffer
    $self->_clear_err();

    ### Determine err handle upfront in case validation method
    ### fails before error handler is selected
    $self->{on_err}=(defined $args{on_err} and $args{on_err}=~/die|rtn/)
        ? $args{on_err}
        : "die";
    $self->_log(msg=>"hello");
    ### Validate input arguments
    eval {
        ($class,%args) = validate_mixed (
        @_,
        [ { type => SCALAR} ],
        {
        cifs_setup => { type => SCALAR,
            callbacks=>{
                'legal values'=>sub{uc(shift())=~/1|0/i
                      }}},
        iscsi_setup => { type => SCALAR,
            callbacks=>{
                'legal values'=>sub{uc(shift())=~/1|0/i
                      }}},
        lif_setup => { type => SCALAR,
            callbacks=>{
                'legal values'=>sub{uc(shift())=~/1|0/i
                      }}},
        loghdl    => {type => SCALARREF, optional=>1 },
        nodes     => {type => SCALAR},
        datastore => {type => SCALAR},
        ic_macs   => {type => SCALAR|UNDEF, optional=>1 },
        image     => {type => SCALAR},
        layout    => {type => SCALAR},
        licenses  => {type => SCALAR},
        model    =>  {type => SCALAR,
        #modified to add FDvM200,CBvM200 for SN.1 singlenode c-mode by Ashrut
        callbacks=>{
            'legal values'=>sub{uc(shift())=~/DOVM100|DOVM200|CBVM100|FDVM100|CBVM200|FDVM200/i
                                }}},
        mode =>  {type => SCALAR,  
        callbacks=>{
            'legal values'=>sub{uc(shift())=~/7|C|SNC/i
                                }}},
        network_name  => { type => SCALAR },
        vmhost => { type => SCALAR },
        vsphere => { type => SCALAR|UNDEF, optional=>1 },
        on_err =>  {type => SCALAR, optional=>1, default=>"die",
        callbacks =>{
            'legal values'=>sub{shift()=~/die|rtn/i
                                }}},
        mgmtvm =>  { type => SCALAR },
        raid_type => { type => SCALAR,
                callbacks=>{
                   'legal values'=>sub{shift()=~/RAID0|RAID4|RAID_DP/i
                }}},
        show_cmd =>{type=>SCALAR, optional=>1, default=>0},
            }
        );

        @licenses=split(",",$args{licenses});
        @nodes=split(",",$args{nodes});
        @ic_macs=split(",",$args{ic_macs});
        $self->{loghdl}        = $args{loghdl};
        $self->{image}         = $args{image};
        $self->{licenses}      = [@licenses];
        $self->{model}         = uc($args{model});
        ###why to check with if else loop??? remove the check and just keep $args{mode}
        #$self->{mode}          = ($args{mode}=~/7/) ? "7" : "C";
        $self->{mode}          = $args{mode};
        $self->{network_name}  = $args{network_name};
        $self->{mgmtvm}        = $args{mgmtvm};
        $self->{datastore}     = $args{datastore};
        $self->{ic_macs}       = [@ic_macs];
        $self->{nodes}         = [@nodes];
        $self->{on_err}        = $args{on_err};
        $self->{vmhost}        = $args{vmhost};
        $self->{vsphere}       = $args{vsphere};
        $self->{loader}        = $args{loader};
        $self->{layout}        = $args{layout};
        $self->{raid_type}     = $args{raid_type};
        $self->{cifs_setup}    = $args{cifs_setup};
        $self->{iscsi_setup}   = $args{iscsi_setup};
        $self->{lif_setup}     = $args{lif_setup};
        $self->{server}        = $self->{vsphere} || $self->{vmhost};

        my @elements=split(/:/,$args{layout});
        if (scalar(@elements)==2) {
            my ($vmdisk_cnt,$vmdisk_size,$unit)=$args{layout}=~/(\d+):(\d+)(\S)?/;
            die "Invalid LAYOUT format vmdisk_cnt:vmdisk_size[m,g]" if ($unit!~/m|g/i);
        }
        else {
            die "Invalid LAYOUT format vmdisk_cnt:vmdisk_size[m,g]";
        }

        ### Retrieve each VSA's corresponding tharnhost file.
        $self->_log(msg=>"verifying existance of tharnhost files",indent=>3);
        foreach my $node (@{$self->{nodes}}){

            #### Retrieve tharnhost file for each VSA
            $self->_log(msg=>"retrieving tharnhost info for '".$node."'", indent=>6);
            $self->{thost_of}{$node} = Utils::make_host($node);
            my $hosttype=$self->{thost_of}{$node}{hosttype};
            my $value=$Hosttype_Of{$self->{mode}};
            unless ($hosttype=~/^$value$/){
                die "tharnhost parameter 'hosttype' is incorrect.  It should be hosttype=$value";
            }
            $self->{thost_of}{$node}{hosttype}=$value;
            
            $hosttype=$self->{thost_of}{$node}{hosttype};
          
            $self->{root_pw}{$node}  = (length($self->{thost_of}{$node}->default_password()))
                ? $self->{thost_of}{$node}->default_password()
                : "netapp1!";
            $self->{timezone}=$self->{thost_of}{$node}{TIMEZONE};
        }

        ### Retrieve each VSA static mac address for interconnect
        if (scalar @{$self->{nodes}} %2 == 0) {
            unless (scalar(@ic_macs) == scalar(@{$self->{nodes}})) {
                die "A static mac address is needed for each node";
            }

            unless (scalar(@licenses) == scalar(@{$self->{nodes}})) {
                die "A unique model license is needed for each node";
            }

            my @ids;
            my $cnt=0;

            foreach my $node (@{$self->{nodes}}){
                my ($id)=$node=~/\-(\d+)$/;
                push @ids,$id;
                $self->{thost_of}{$node}{ic_mac}=$ic_macs[$cnt];
                $self->{thost_of}{$node}{license}=$licenses[$cnt];
                $cnt++;
            }

            $self->{ha_name}="ha".$ids[0]."_".$ids[1];

        } else {
            $self->{thost_of}{$self->{nodes}->[0]}{license}=$self->{licenses}->[0];
        }

        ### Generate the cluster name using the id from the first node
        $self->{cl_name}= $ENV{CLUSTER_NAME} || sprintf("cl%d",$self->{nodes}->[0]=~/\-(\d+)$/);

        ### Retrieve VM host tharnhost file
        $self->_log(msg=>"retrieving tharnhost info for '".$self->{vmhost}."'", indent=>6);
        $self->{thost_of}{$self->{vmhost}} = Utils::make_host($self->{vmhost});

        ### Retrieve Server tharnhost file
        $self->_log(msg=>"retrieving tharnhost info for '".$self->{server}."'", indent=>6)
            if ($self->{vmhost} ne $self->{server});
        $self->{thost_of}{$self->{server}} = Utils::make_host($self->{server});
    
        ### Retreive server password
        $self->{root_pw}{$self->{server}} = (length($self->{thost_of}{$self->{server}}->default_password()))
            ? $self->{thost_of}{$self->{server}}->default_password()
            : (defined $self->{thost_of}{$self->{server}}{conn}{ssh}{password})
            ? $self->{thost_of}{$self->{server}}{conn}{ssh}{password}
            :"netapp1!";

        ### Retrieve server domain
        $self->{domain}{$self->{server}} = $self->{thost_of}{$self->{server}}{DOMAIN};
    
        ### Retrieve vmhost domain
        $self->{domain}{$self->{vmhost}} = $self->{thost_of}{$self->{vmhost}}{DOMAIN};

        ### Validate image path exists
        $self->_log(msg=>"verifying VSA image location", indent=>3);
        if (-e $self->{image}){
            $self->_log(msg=>"image exists", indent=>6);
        } else {
            die "Invalid image location: ".$self->{image};
        }

        ### Connect to mgmtvm for dvadmin use
        $self->_log(msg=>"connecting to vsa mgmt host", indent=>3);
        $self->{apiClientObj} = NACL::APISet->new(
            hostobj   => Tharn::host($self->{mgmtvm}),
            category  => "Host",
            interface => "CLI",
            set       => "Linux",
            connid    => "ssh",
            );

        ### Validate VSA model vs ONTAP release support
        $self->_log(msg=>"verifying ontap release and mode for VSA model",
                indent=>3);
        my $cmd = "grep -r0 -m1 -a FullVersion ".($self->{image});
        my $buf=$self->{apiClientObj}->execute_raw_command(command=>$cmd);
        ($self->{release})=$buf=~/\<FullVersion\>(\S+)\<\/FullVersion\>/i;
        unless ($self->{release}=~/rollingrock|sierra|fullsteam/i) {
            die "Invalid Release: This tool only supports releases RR0 forward";
        }

        ($self->{release_id})=$self->{release}=~/__(\S+)/;

        if ($self->{model} =~ /CBVM100/i) {
            unless ($self->{mode}=~/7/i) {
                die "Invalid Mode: Cloudbond VSA's are only supported on 7mode";
            }
            unless ($self->_get_release_nbr($self->{release_id}) >=
                    $self->_get_release_nbr("8.1.1")) {
                die "Invalid Release: Cloudbond VSA's are only supported on ".
                    "releases starting with RR1 (>= 8.1.1) ";
            }
        } elsif ($self->{model} =~ /DOVM100|FDVM100/i) {
            if ($self->{mode}=~/c/i) {
                die "Invalid Mode: Only 7m is supported on DOvM100";
            }
        } elsif ($self->{model} =~ /DOVM200/i) {
            unless ($self->{release}=~/sierra|fullsteam/i) {
                die "Invalid Release: HA and Cmode are first supported on VSA ".
                    "in sierranevada";
            }
        }

        ### Connect to the dvadmin utility
        $self->_log(msg=>"connecting to dvadmin utility",indent=>3);
        
        my $vsamgmt_dir = (defined $ENV{VSAMGMT_DIR})
            ? $ENV{VSAMGMT_DIR}
            : "~";
       
        my $out=$self->{apiClientObj}->execute_raw_command(command=>"cd ".$vsamgmt_dir);
        $cmd = "dvadmin -s ".$self->{server};
        if(defined $self->{domain}{$self->{server}}) {
            $cmd .= ".".$self->{domain}{$self->{server}};
        }
        if(defined $self->{vsphere}) {
            if (defined $self->{domain}{$self->{vmhost}}) {
             $cmd .= " -h ".$self->{vmhost}.".".$self->{domain}{$self->{vmhost}};
            }
            else{
            $cmd .= " -h ".$self->{vmhost};#.".".$self->{domain}{$self->{vmhost}};
            }
        }
        $cmd .= " -u ".$self->{thost_of}{$self->{server}}{conn}{ssh}{username}.
        " -p ".$self->{root_pw}{$self->{server}}.
        #" --debug 10".
        #commenting -D code till DEV fixes BURT# 703550,703551 in rsn1 build
        " -D "."~/.vsa/".$self->{nodes}->[0];
        $out=$self->{apiClientObj}->execute_raw_command(
            command=>$cmd,
            prompts=>['dvadmin'],
            'connectrec-timeout' => '60'
            );
            die "Unable to launch vsa mgmt tool" if ($out=~/usage\:|invalid/i);

            ### Create Vmhost Object
            $self->_log(msg=>"creating esx host object", indent=>3);
            $self->{vmhostObj}=new VSA::Vmhost(
                hostname=>$self->{vmhost},
                loghdl=>$self->{loghdl},
                show_cmd=>$args{show_cmd},
            );

    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return "";
    }

    return $self;
}

###############################################################################
# Method:    create
# Objective: create a new VSA
###############################################################################
sub create {
    my ($self)=@_;
    my (
    $license,
        $node_cnt,
        @cmds,
        @nodes,
        );

    $node_cnt=scalar(@{$self->{nodes}});
    if ($node_cnt==2){
        $self->_log(msg=>"generating and executing command set for HA configuration",indent=>3);
     @cmds=$self->_get_standalone_config();
     #$self->_log(msg=>"".Dumper(@cmds));
    } elsif($node_cnt==1) {
        $self->_log(msg=>"generating and executing command set for standalone configuration",indent=>3);
        @cmds=$self->_get_standalone_config();
    }

    my $retry_cnt;
    foreach my $cmd (@cmds) {
        $self->_log(msg=>$cmd,indent=>6);
        #$cmd=~s/\r|\n//g;
        $retry_cnt=0;
        while(1) {
            my $output=$self->{apiClientObj}->execute_raw_command(
                command=>$cmd,
                prompts=>['dvadmin'],
                'connectrec-timeout' => '600');
            if ($output=~/another task is in progress/i) {
                sleep(5);
                redo if ($retry_cnt < 12);
            } elsif (
                $output =~ /error/i and
                $output !~ /settings on the host is restricted/i
                ) {
                die $output;
            }
            last;
        }
    }

    ## Create the vmsocats process for console access
    $self->_log(msg=>"creating communication sockets on ESX host",indent=>5);
    foreach my $node (@{$self->{nodes}}) {
        my ($id)=$node=~/(\d+)$/;
        $self->{vmhostObj}->create_socats(
            datastore => $self->{datastore},
            name         => $node,
            console_port => 7000 + $id,
            gdb_port     => 8000 + $id
        );
    }

    return 1;
}

###############################################################################
# Method:    destroy
# Objective: destroy an existing VSA
###############################################################################
sub destroy {
    my ($self,%args)=@_;
    my (
    $output,
    @cmds,
       );

    ### power off
    foreach my $node (@{$self->{nodes}}) {
        $self->_log(msg=>"powering off ".$node,indent=>5);
        $self->_power_off(node=>$node);

        ### rm lock files
        my $output=$self->{vmhostObj}->execute(
            cmd  => "rm /vmfs/volumes/".$self->{datastore}."/".$node."/.lck*",
            hide =>1
        );
    }

    ### destroy cluster (if cmode)
    if ( $self->{last_config_mode}=~/^c/i) {
        foreach my $node (@{$self->{nodes}}) {
            $self->_log(msg=>"removing $node from cluster",indent=>5);
            $output=$self->{apiClientObj}->execute_raw_command(
                command=>"cluster remove --force ".$self->{cl_name}." ".$node,
                prompts=>['dvadmin'],
                'connectrec-timeout' => '600'
                
            );
        }

        $self->_log(msg=>"destroying cluster ".$self->{cl_name},indent=>5);
        $output=$self->{apiClientObj}->execute_raw_command(
            command=>"cluster destroy ".$self->{cl_name},
            prompts=>['dvadmin'],
            'connectrec-timeout' => '1800'
            );
    }

    ### destroy ha
    if (scalar @{$self->{nodes}} == 2) {
        $self->_log(msg=>"destroying ha ".$self->{ha_name},indent=>5);
        $output=$self->{apiClientObj}->execute_raw_command(
            command=>"ha destroy ".$self->{ha_name},
            prompts=>['dvadmin'],
            'connectrec-timeout' => '600'
        );
    }

    ### destroy vms and vmsocat processes
    foreach my $node (@{$self->{nodes}}) {
        $self->_log(msg=>"destroying ".$node,indent=>5);
        $output=$self->{apiClientObj}->execute_raw_command(
            command=>"vm destroy --force ".$node,
            prompts=>['dvadmin'],
            'connectrec-timeout' => '1800'
        );

        if ($output=~/Error: \[12\]/i) {
            die "Unable to destroy the existing VSA config";
        }

        $self->_log(msg=>"destroying vmsocat processes on ESX host",indent=>5);
        $self->{vmhostObj}->delete_socats(name=>$node);
    }

    return 1;

}
###############################################################################
# Method:    display
# Objective: display VSA configuration
###############################################################################
sub display {
    my ($self,%args) = @_;

    $self->_log(msg=>"display vsa configuration", indent=>5);
    foreach my $node (@{$self->{nodes}}) {
    my $output=$self->{apiClientObj}->execute_raw_command(
        command=>"vm show ".$node,
        prompts=>['dvadmin'],
        'connectrec-timeout' => '300');

    my @lines=split(/\n/,$output);
    ($lines[0])=$lines[0]=~/(VM Name.*)/;
    $self->_log(msg=>"-"x125,indent=>7);
    foreach my $line (@lines) {
        $self->_log(msg=>$line,indent=>7);
    }
    $self->_log(msg=>"-"x125,indent=>7);
    }
    return 1;
}


###############################################################################
# Method:    exists
# Objective: check if a VSA exists
###############################################################################
sub exists {
    my ($self,%args)=@_;
    my $exist_cnt;

    foreach my $node (@{$self->{nodes}}){
    my $out=$self->{apiClientObj}->execute_raw_command(
        command=>"vm show ".$node,
        prompts=>['dvadmin'],
        'connectrec-timeout' => '10'
    );

    ### Check for VM existance.
    $exist_cnt++ if ($out !~ /no VM/i);
    }

    ### Check for VM existance.
    return ($exist_cnt) ? 1 : 0;
}

###############################################################################
# Method:    on_err
# Objective: Return errhdlr_type
###############################################################################
sub on_err {
    my ($self,$value)=@_;
    my ($errmsg);

    if (defined $value) {

        unless ($value=~/die|rtn/){
            $errmsg="Invalid value '".$value."'";
            die _caller($errmsg);
        }

        $self->{on_err}=$value;
    }

    return $self->{on_err};
}

###############################################################################
# Method:    ontap
# Objective: configure ontap with standard setup expected by tests
###############################################################################
sub ontap {
    my ($self,%args)=@_;
    if ($self->{mode}=~/7/) {
    $self->_ontap_7mode();
    }elsif($self->{mode}=~/^c/i){
    $self->_ontap_cmode();
    }elsif($self->{mode}=~/snc/i){
    $self->_ontap_sncmode();
    }
    1;
}

###############################################################################
# Method:    start
# Objective: start up VSA instance
###############################################################################
sub start {
    my ($self,%args)=@_;

    my @nodes=@{$self->{nodes}};

    if ($self->{mode}=~/7/) {
        ### Start up the nodes
        $self->_start(nodes=>[@nodes]);
    } elsif ($self->{mode}=~/^c/i) {
        ### Retreive the first node off the stack and start up by
        ### itself, so that there's not multiple nodes trying to
        ### initially create the cluster
        my $node=shift @nodes;
        $self->_log(msg=>"first node in C cluster is booted alone", indent=>3);
        $self->_log(msg=>"c-mode nodes get rebooted 3 times during autosetup",indent=>3);
        $self->_start(nodes=>[$node]);

        ### If this is not a single node cluster, start the remaining
        ### nodes in parallel
        if (scalar(@nodes) > 0) {
            $self->_log(msg=>"remaining node(s) in C cluster can be booted in parallel", indent=>3);
            $self->_start(nodes=>[@nodes]);
        }
    } elsif($self->{mode}=~/snc/i){
        #Starting single node c-mode
        $self->_log(msg=>"Starting single node in C-mode VSA", indent=>3);
        $self->_log(msg=>"C-mode node get rebooted 3 times during autosetup",indent=>3);
        $self->_start(nodes=>[@nodes]);
    }

    return 1;
}

###############################################################################
# Method:_feature_licenses
# Objective: add ontap feature licenses
###############################################################################
sub _feature_licenses {
    my ($self,%args)=@_;
    my (
        $node_obj,
        $done,
        $retry_cnt,
        $retry_limit,
        $sleep,
	);
    
    $self->_log(msg=>"adding feature licenses",indent=>6);
    $node_obj=$args{node_obj};   
    my @features=qw(CIFS NFS ISCSI FLEX_CLONE SNAPMIRROR SNAPRESTORE);
    push @features,"HTTP" if ($self->{mode}=~/7/);
    push @features, "SV_ONTAP_PRI" if ($self->{model}=~/cbvm100/i);
    $self->_log(msg=>join(", ",@features),indent=>6);
    foreach my $feature (@features) {
        $done        = 0;
        $retry_cnt   = 0;
        $retry_limit = 5;
        $sleep       = 11;
        while (1) {
            try {
                NACL::STask::SystemLicense->add(
                command_interface => $node_obj,
                feature           => "$feature",
                );
                $done=1;
            }  
            catch NACL::APISet::Exceptions::InvalidParamValueException with {
                my $exception_object = shift;
            }
            catch NATE::BaseException with {
                my $exception_object = shift;
                $self->_log(msg=>"Exception is of type: " . ref ($exception_object));
                $self->_log(msg=>"Error text is: " . $exception_object->text());
                Tharn::logresult("FAIL", msg => $node_obj->name().
                                 " failed during licensing of ".$feature);
            };

            ### Retry adding licensing again 
            if (not $done) {
                if (++$retry_cnt < $retry_limit) {
                    $self->_log(msg=>"retry $retry_cnt in $sleep seconds to add licenses",
                                indent=>9);
                    sleep($sleep);
                    redo;
                } else {
                    Tharn::logresult("FAIL", msg => $node_obj->name().
                                     " failed during licensing of ".$feature);
                }
            }
	    
	        ### Exit while loop
	        last;
	    }    
    }


}

###############################################################################
# Method:    _ontap_cmode
# Objective: configure ontap with standard setup expected by tests
###############################################################################
sub _ontap_cmode{
    my ($self,%args)=@_;
    my (
        $ads_server,
        $ads_domain,
        $aggr_name,
        $bHa,
        $first,
        $node_name,
        $vs_name,
        $aggr_obj,
        $apiset_obj,
        $export_obj,
        $node_obj,
        $vs_obj,
        $rgr_obj,
       );

    $node_name=$self->{nodes}->[0];
    $bHa=(scalar(@{$self->{nodes}}) > 1) ? 1 : 0;
    $first=0;

    try {

        $node_obj=NACL::STask::Node->new(name=>$node_name);
        $apiset_obj=$node_obj->apiset(
                category => "Node",
                interface => "CLI",
                set => "CMode",
        );
           
        foreach my $node (@{$self->{nodes}}) {
            #my $transit_obj = NACL::Transit->new(name =>$node);
            #$transit_obj->change_state(to=>"CLI");
                 
            ### Enable failover if more than 1 node
            if ($bHa) {

                if ($first) {
                    $self->_log(msg=>"enabling cluster failover",indent=>6);
                    NACL::C::ClusterHa->modify(
                        command_interface => $node_obj,
                        configured        => "true",
                    );
                }

                $self->_log(msg=>"enabling storage failover on ".$node,indent=>6);
                NACL::C::Failover->modify(
                    command_interface => $node_obj,
                    node              => $node,
                    'enabled'         => 'true',
                    'auto-giveback'   => 'false'
                );
            }

            my $retry_cnt=0;
            $self->_log(msg=>"setting timezone on ".$node,indent=>6);
            while (1) {
                try {
                        NACL::C::SystemNodeDate->modify(
                        command_interface => $node_obj,
                        node              => $node,
                        timezone          => "America/New_York"
                    );
                    last;
                } catch NACL::APISet::Exceptions::ConnectionFailedException with {
                    my $exception_object = shift;
                    if ($retry_cnt++ < 10) {
                         sleep(3);
                         $self->_log(msg=>"Retrying to connect via ssh: ".$retry_cnt,
                                     indent=>9
                                     );
                    } else {
                        $exception_object ->throw();
                    }
                };
                redo;
            }

            try {
                $self->_log(msg=>"setting ntp server on ".$node,indent=>6);
                NACL::C::SystemServicesNtpServer->create(
                    command_interface => $node_obj,
                    server            => $ENV{NTP_SERVER_IP},
                    node              => $node
                );
            } catch NACL::APISet::Exceptions::CommandFailedException with {
                my $exception_object = shift;
                unless ($exception_object->text()=~/duplicate/i){
                    $exception_object ->throw();
                }
            };
        }


        try {

            my $tmp=$node_name;
            $tmp=~s/-//g;
            $aggr_name= $ENV{AGGR} || $tmp."_aggr1";
            $self->_log(msg=>"creating aggregate ".$aggr_name." ".
                     "on node ".$node_name,indent=>6);

            $aggr_obj = NACL::C::Aggregate->create(
            command_interface => $node_obj,
            aggregate         => $aggr_name,
            diskcount         => 3,
            nodes             => [$node_name],
            );

        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/aggregate already uses/i){
                $exception_object ->throw();
            }
        };
 

        try {

            $vs_name= $ENV{VSERVER} || "vs0";
            $self->_log(msg=>"creating vserver ".$vs_name." ".
                     "on node ".$node_name,indent=>6);

            $vs_obj = NACL::C::Vserver->create(
              command_interface => $node_obj,
              aggregate         => $aggr_name,
              vserver           => $vs_name,
              "ns-switch"       => [qw(file nis)],
              rootvolume        => "root_vs0",
              "rootvolume-security-style" => "mixed",
            );
	    ### License additions
            unless($self->{model}=~/CBVM200|FDVM200/i)
	    {$self->_feature_licenses(node_obj=>$node_obj);
            }

            $vs_obj->modify('allowed-protocols' => 
                            [ qw( nfs cifs iscsi )]);

        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/is already used/i){
                $exception_object ->throw();
            }
        };

    
        ### Check if release SN or later since license rqmts are different
        if ($self->_get_release_nbr($self->{release_id}) >= 
            $self->_get_release_nbr("8.2.0")) {
            foreach my $cmd (
               'iscsi create -target-alias vs0 -status-admin up -vserver vs0',
               'nfs on',
            ){
               $apiset_obj->execute_raw_command(command=>$cmd);
	    }           
        }


        try {

            $self->_log(msg=>"creating export rule for vserver ".$vs_name. " ".
                     "on node ".$node_name,indent=>6);

            $export_obj = NACL::C::ExportPolicyRule->create(
              command_interface => $node_obj,
              clientmatch       => "0.0.0.0/0",
              policyname        => "default",
              vserver           => $vs_name,
              rwrule            => ["any"],
              rorule            => ["any"],
              "allow-dev"       => "true",
              "allow-suid"      => "true",
              "superuser"       => ["any"],
            );

        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/is already used/i){
                $exception_object ->throw();
            }
        };

        $self->_log(msg=>"creating vservers users",indent=>6);
        $self->_vs_unix_user(
            command_interface => $node_obj,
            vserver           => $vs_name,
            user              => "root",
            id                => 0,
            "primary-gid"     => 0,
            "full-name"       => "LocalRoot",
        );

        $self->_vs_unix_user(
            command_interface => $node_obj,
            vserver           => $vs_name,
            user              => "nobody",
            id                => 65534,
            "primary-gid"     => 65534,
            "full-name"       => "nobody",
        );

        $self->_log(msg=>"creating vservers groups",indent=>6);
        $self->_vs_unix_group(
            command_interface => $node_obj,
            vserver           => $vs_name,
            name              => "root",
            id                => 0,
        );

        $self->_vs_unix_group(
            command_interface => $node_obj,
            vserver           => $vs_name,
            name              => "nobody",
            id                => 65534,
        );

        if ($self->{lif_setup}) {

            foreach my $node (@{$self->{nodes}}) {    
                $self->_log(msg=>"creating data interfaces for $node",indent=>6);
                NACL::STask::NetworkInterface->create(
                command_interface => $node_obj,
                lif               => $node."_lif_data1",
                vserver           => $vs_name,
                role              => "data",
                "data-protocol"   => [qw(nfs cifs fcache)],
                'home-node'       => $node,
                'home-port'       => "e0d",
                address           => $self->{thost_of}{$node}{DATA_IP1},
                netmask           => $self->{thost_of}{$node}{DATA_MASK1},
                "auto-revert"     => "true",
                nacltask_if_exists => "reuse",
                );

                if ($self->{iscsi_setup}) {
                    NACL::STask::NetworkInterface->create(
                    command_interface => $node_obj,
                    lif               => $node."_lif_data2",
                    vserver           => $vs_name,
                    role              => "data",
                    "data-protocol"   => [qw(iscsi)],
                    'home-node'       => $node,
                    'home-port'       => "e0e",
                    address           => $self->{thost_of}{$node}{DATA_IP2},
                    netmask           => $self->{thost_of}{$node}{DATA_MASK2},
                    #"auto-revert"     => "true",
                    nacltask_if_exists => "reuse",
                    );
                } else {
                    NACL::STask::NetworkInterface->create(
                    command_interface => $node_obj,
                    lif               => $node."_lif_data2",
                    vserver           => $vs_name,
                    role              => "data",
                    "data-protocol"   => [qw(nfs cifs fcache)],
                    'home-node'       => $node,
                    'home-port'       => "e0e",
                    address           => $self->{thost_of}{$node}{DATA_IP2},
                    netmask           => $self->{thost_of}{$node}{DATA_MASK2},
                    "auto-revert"     => "true",
                    nacltask_if_exists => "reuse",
                    );
                }
            }

            try {

                my $out = NACL::C::RoutingGroups->find(command_interface => $node_obj,
                               filter            => {vserver => $vs_name});
                my $routing_group = $out->routing_group();

                $self->_log(msg=>"creating routing-group route for routing group ".
                        $routing_group." and vserver ".$vs_name." ".
                        "on node ".$node_name,indent=>6);

                $rgr_obj = NACL::STask::NetworkRoutingGroupsRoute->create(
                    command_interface   => $node_obj,
                    vserver             => $vs_name,
                    'routing-group'     => $routing_group,
                    destination         => "0.0.0.0/0",
                    gateway             => $self->{thost_of}{$node_name}{DATA_GATEWAY1},
                    nacltask_if_exists  => "reuse",
                );

            } catch NACL::APISet::Exceptions::CommandFailedException with {
                my $exception_object = shift;
                unless ($exception_object->text()=~/is already used/i){
                    $exception_object ->throw();
                }
            };
            
        }


        try {
            $self->_log(msg=>"creating vserver service nis-domain",indent=>6);
            NACL::C::VserverServicesNisDomain->create(
            command_interface => $node_obj,
            domain            => $ENV{NIS_DOMAIN},
            servers           => [$ENV{NIS_SERVERS}],
            vserver           => $vs_name,
            active            => 'true',
            );
         } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/duplicate/i){
                $exception_object ->throw();
            }
        };


        try {
            $self->_log(msg=>"creating vserver dns service",indent=>6);
            NACL::C::VserverServicesDns->create(
            command_interface => $node_obj,
            domains           => [($ENV{FILER_DNS_DOMAINNAME})],
            'name-servers'    => [($ENV{FILER_DNS_SERVER_LIST})],
            vserver           => $vs_name,
            );
         } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/duplicate/i){
                $exception_object ->throw();
            }
        };


        try {
            $self->_log(msg=>"enabling nfs[3,4] on vserver ".$vs_name,indent=>6);
            NACL::C::VserverNfs->create(
            command_interface => $node_obj,
            vserver           => $vs_name,
            access            => "true",
            v3                => "enabled",
            v4                => "enabled",
            );
        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/duplicate/i){
                $exception_object ->throw();
            }
        };

        if ($self->{cifs_setup}) {
            $self->_log(msg=>"setting up CIFS",indent=>6);

            if ($ENV{CIFS_DOMAIN_LONG}){
                $ads_domain=($ENV{CIFS_DOMAIN_LONG});	
            }
            else {
                $ads_server=($ENV{AD_SERVER_FULLDOMAIN})
                ? $ENV{AD_SERVER_FULLDOMAIN}
                : 0;
                ($ads_domain)=$ads_server=~/\.(.*)/;
            }

            my $cifs_server=$node_name;
            $cifs_server=~s/-//g;

            my %setup_of = (
                'vserver'          => $vs_name,
                'cifs-server'      => uc($cifs_server),
                'cifs_domain'      => $ads_domain,
                'dns_domain'       => $ENV{FILER_DNS_DOMAINNAME},
                'username'         => $ENV{AD_SERVER_USER}, #"adadmin",
                'password'         => $ENV{AD_SERVER_PASSWORD}, #"adadmin",
                'ou'                => "CN=computers",
                'wins-option'       => 1,
                'wins-servers'      => $ENV{WINS_SERVER_IP},
                #'name-servers'      => \@name_servers,
            );

            push my @name_maps, $ENV{NAME_MAPPINGS};
            my $position;
            my @maps;
            foreach my $name_map ( @name_maps) {
                my ( $type, $windows, $unix ) = split /\|/ , $name_map;
                $position++;

                $windows =~ s/\\\\/\\/g;

                push @maps,
                {
                direction   => "win-unix",
                position    => $position,
                pattern     => $windows,
                replacement => $unix,
                },
                {
                direction   => "unix-win",
                position    => $position,
                pattern     => $unix,
                replacement => $windows,
                };
            }

            ### Format the values to display to STDOUT
            $self->_log(msg=>"input responses to use for 'cifs setup'", indent=>8);
            push my @outputs, sprintf(
                "         %-20.20s %-15.15s\n".
                "         %-20.20s %-15.15s",
                "Parameter",
                "Value",
                "-"x20,
                "-"x15
            );

            foreach my $param (sort keys %setup_of){
                push @outputs,
                sprintf ("         %-20.20s %-s",$param,$setup_of{$param});
            }

            $self->_log(msg=>[@outputs,"\n"]);

            $setup_of{vserver_name_mapping} = [@maps];


            my $cifs = NACL::MTask::CIFS->setup(
                command_interface => $node_obj,
                if_exists         => "purge",
                %setup_of,

            );
        }
    } catch NATE::BaseException with {
        my $exception_object = shift;
        $self->_log(msg=>"Exception is of type: " . ref ($exception_object));
        $self->_log(msg=>"Error text is: " . $exception_object->text());

        my @lines=split(/\n/,$exception_object->text());
        die ($lines[0])
            unless ($exception_object->text()=~/server is running/i);
    };

    return 1;
}

###############################################################################
# Method:    _ontap_cmode
# Objective: configure ontap with standard setup expected by tests
###############################################################################
sub _ontap_sncmode{
    my ($self,%args)=@_;
    my (
        $ads_server,
        $ads_domain,
        $aggr_name,
        $bHa,
        $first,
        $node_name,
        $vs_name,
        $aggr_obj,
        $apiset_obj,
        $export_obj,
        $node_obj,
        $vs_obj,
        $rgr_obj,
       );

    $node_name=$self->{nodes}->[0];
    $bHa=(scalar(@{$self->{nodes}}) > 1) ? 1 : 0;
    $first=0;

    try {

        $node_obj=NACL::STask::Node->new(name=>$node_name);
        $apiset_obj=$node_obj->apiset(
                category => "Node",
                interface => "CLI",
                set => "CMode",
        );
       
        foreach my $node (@{$self->{nodes}}) {
            #my $transit_obj = NACL::Transit->new(name =>$node);
            #$transit_obj->change_state(to=>"CLI");
         
            ### Enable failover if more than 1 node
            if ($bHa) {

                if ($first) {
                    $self->_log(msg=>"enabling cluster failover",indent=>6);
                    NACL::C::ClusterHa->modify(
                        command_interface => $node_obj,
                        configured        => "true",
                    );
                }

                $self->_log(msg=>"enabling storage failover on ".$node,indent=>6);
                NACL::C::Failover->modify(
                    command_interface => $node_obj,
                    node              => $node,
                    'enabled'         => 'true',
                    'auto-giveback'   => 'false'
                );
            }

            my $retry_cnt=0;
            $self->_log(msg=>"setting timezone on ".$node,indent=>6);
            while (1) {
                try {
                        NACL::C::SystemNodeDate->modify(
                        command_interface => $node_obj,
                        node              => $node,
                        timezone          => "America/New_York"
                    );
                    last;
                } catch NACL::APISet::Exceptions::ConnectionFailedException with {
                    my $exception_object = shift;
                    if ($retry_cnt++ < 10) {
                         sleep(3);
                         $self->_log(msg=>"Retrying to connect via ssh: ".$retry_cnt,
                                     indent=>9
                                     );
                    } else {
                        $exception_object ->throw();
                    }
                };
                redo;
            }

            try {
                $self->_log(msg=>"setting ntp server on ".$node,indent=>6);
                NACL::C::SystemServicesNtpServer->create(
                    command_interface => $node_obj,
                    server            => $ENV{NTP_SERVER_IP},
                    node              => $node
                );
            } catch NACL::APISet::Exceptions::CommandFailedException with {
                my $exception_object = shift;
                unless ($exception_object->text()=~/duplicate/i){
                    $exception_object ->throw();
                }
            };
        }


        try {

            my $tmp=$node_name;
            $tmp=~s/-//g;
            $aggr_name= $ENV{AGGR} || $tmp."_aggr1";
            $self->_log(msg=>"creating aggregate ".$aggr_name." ".
                     "on node ".$node_name,indent=>6);

            $aggr_obj = NACL::C::Aggregate->create(
            command_interface => $node_obj,
            aggregate         => $aggr_name,
            diskcount         => 3,
            nodes             => [$node_name],
            );

        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/aggregate already uses/i){
                $exception_object ->throw();
            }
        };
 

        try {

            $vs_name= $ENV{VSERVER} || "vs0";
            $self->_log(msg=>"creating vserver ".$vs_name." ".
                     "on node ".$node_name,indent=>6);

            $vs_obj = NACL::C::Vserver->create(
              command_interface => $node_obj,
              aggregate         => $aggr_name,
              vserver           => $vs_name,
              "ns-switch"       => [qw(file nis)],
              rootvolume        => "root_vs0",
              "rootvolume-security-style" => "mixed",
            );
	    ### License additions
            ### Since licenses are added from dvadmin we donot require to call this method in SNC
            unless($self->{model}=~/CBVM200|FDVM200/i)
	    {$self->_feature_licenses(node_obj=>$node_obj);
            }

            $vs_obj->modify('allowed-protocols' => 
                            [ qw( nfs cifs iscsi )]);

        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/is already used/i){
                $exception_object ->throw();
            }
        };

    
        ### Check if release SN or later since license rqmts are different
        if ($self->_get_release_nbr($self->{release_id}) >= 
            $self->_get_release_nbr("8.2.0")) {
            foreach my $cmd (
               'iscsi create -target-alias vs0 -status-admin up -vserver vs0',
               'nfs on',
            ){
               $apiset_obj->execute_raw_command(command=>$cmd);
	    }           
        }


        try {

            $self->_log(msg=>"creating export rule for vserver ".$vs_name. " ".
                     "on node ".$node_name,indent=>6);

            $export_obj = NACL::C::ExportPolicyRule->create(
              command_interface => $node_obj,
              clientmatch       => "0.0.0.0/0",
              policyname        => "default",
              vserver           => $vs_name,
              rwrule            => ["any"],
              rorule            => ["any"],
              "allow-dev"       => "true",
              "allow-suid"      => "true",
              "superuser"       => ["any"],
            );

        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/is already used/i){
                $exception_object ->throw();
            }
        };

        $self->_log(msg=>"creating vservers users",indent=>6);
        $self->_vs_unix_user(
            command_interface => $node_obj,
            vserver           => $vs_name,
            user              => "root",
            id                => 0,
            "primary-gid"     => 0,
            "full-name"       => "LocalRoot",
        );

        $self->_vs_unix_user(
            command_interface => $node_obj,
            vserver           => $vs_name,
            user              => "nobody",
            id                => 65534,
            "primary-gid"     => 65534,
            "full-name"       => "nobody",
        );

        $self->_log(msg=>"creating vservers groups",indent=>6);
        $self->_vs_unix_group(
            command_interface => $node_obj,
            vserver           => $vs_name,
            name              => "root",
            id                => 0,
        );

        $self->_vs_unix_group(
            command_interface => $node_obj,
            vserver           => $vs_name,
            name              => "nobody",
            id                => 65534,
        );

        if ($self->{lif_setup}) {

            foreach my $node (@{$self->{nodes}}) {    
                $self->_log(msg=>"creating data interfaces for $node",indent=>6);
                NACL::STask::NetworkInterface->create(
                command_interface => $node_obj,
                lif               => $node."_lif_data1",
                vserver           => $vs_name,
                role              => "data",
                "data-protocol"   => [qw(nfs cifs fcache)],
                'home-node'       => $node,
                'home-port'       => "e0d",
                address           => $self->{thost_of}{$node}{DATA_IP1},
                netmask           => $self->{thost_of}{$node}{DATA_MASK1},
                "auto-revert"     => "true",
                nacltask_if_exists => "reuse",
                );

                if ($self->{iscsi_setup}) {
                    NACL::STask::NetworkInterface->create(
                    command_interface => $node_obj,
                    lif               => $node."_lif_data2",
                    vserver           => $vs_name,
                    role              => "data",
                    "data-protocol"   => [qw(iscsi)],
                    'home-node'       => $node,
                    'home-port'       => "e0e",
                    address           => $self->{thost_of}{$node}{DATA_IP2},
                    netmask           => $self->{thost_of}{$node}{DATA_MASK2},
                    #"auto-revert"     => "true",
                    nacltask_if_exists => "reuse",
                    );
                } else {
                    NACL::STask::NetworkInterface->create(
                    command_interface => $node_obj,
                    lif               => $node."_lif_data2",
                    vserver           => $vs_name,
                    role              => "data",
                    "data-protocol"   => [qw(nfs cifs fcache)],
                    'home-node'       => $node,
                    'home-port'       => "e0e",
                    address           => $self->{thost_of}{$node}{DATA_IP2},
                    netmask           => $self->{thost_of}{$node}{DATA_MASK2},
                    "auto-revert"     => "true",
                    nacltask_if_exists => "reuse",
                    );
                }
            }

            try {

                my $out = NACL::C::RoutingGroups->find(command_interface => $node_obj,
                               filter            => {vserver => $vs_name});
                my $routing_group = $out->routing_group();

                $self->_log(msg=>"creating routing-group route for routing group ".
                        $routing_group." and vserver ".$vs_name." ".
                        "on node ".$node_name,indent=>6);

                $rgr_obj = NACL::STask::NetworkRoutingGroupsRoute->create(
                    command_interface   => $node_obj,
                    vserver             => $vs_name,
                    'routing-group'     => $routing_group,
                    destination         => "0.0.0.0/0",
                    gateway             => $self->{thost_of}{$node_name}{DATA_GATEWAY1},
                    nacltask_if_exists  => "reuse",
                );

            } catch NACL::APISet::Exceptions::CommandFailedException with {
                my $exception_object = shift;
                unless ($exception_object->text()=~/is already used/i){
                    $exception_object ->throw();
                }
            };
            
        }


        try {
            $self->_log(msg=>"creating vserver service nis-domain",indent=>6);
            NACL::C::VserverServicesNisDomain->create(
            command_interface => $node_obj,
            domain            => $ENV{NIS_DOMAIN},
            servers           => [$ENV{NIS_SERVERS}],
            vserver           => $vs_name,
            active            => 'true',
            );
         } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/duplicate/i){
                $exception_object ->throw();
            }
        };


        try {
            $self->_log(msg=>"creating vserver dns service",indent=>6);
            NACL::C::VserverServicesDns->create(
            command_interface => $node_obj,
            domains           => [($ENV{FILER_DNS_DOMAINNAME})],
            'name-servers'    => [($ENV{FILER_DNS_SERVER_LIST})],
            vserver           => $vs_name,
            );
         } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/duplicate/i){
                $exception_object ->throw();
            }
        };


        try {
            $self->_log(msg=>"enabling nfs[3,4] on vserver ".$vs_name,indent=>6);
            NACL::C::VserverNfs->create(
            command_interface => $node_obj,
            vserver           => $vs_name,
            access            => "true",
            v3                => "enabled",
            v4                => "enabled",
            );
        } catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception_object = shift;
            unless ($exception_object->text()=~/duplicate/i){
                $exception_object ->throw();
            }
        };

        if ($self->{cifs_setup}) {
            $self->_log(msg=>"setting up CIFS",indent=>6);

            if ($ENV{CIFS_DOMAIN_LONG}){
                $ads_domain=($ENV{CIFS_DOMAIN_LONG});	
            }
            else {
                $ads_server=($ENV{AD_SERVER_FULLDOMAIN})
                ? $ENV{AD_SERVER_FULLDOMAIN}
                : 0;
                ($ads_domain)=$ads_server=~/\.(.*)/;
            }

            my $cifs_server=$node_name;
            $cifs_server=~s/-//g;

            my %setup_of = (
                'vserver'          => $vs_name,
                'cifs-server'      => uc($cifs_server),
                'cifs_domain'      => $ads_domain,
                'dns_domain'       => $ENV{FILER_DNS_DOMAINNAME},
                'username'         => $ENV{AD_SERVER_USER}, #"adadmin",
                'password'         => $ENV{AD_SERVER_PASSWORD}, #"adadmin",
                'ou'                => "CN=computers",
                'wins-option'       => 1,
                'wins-servers'      => $ENV{WINS_SERVER_IP},
                #'name-servers'      => \@name_servers,
            );

            push my @name_maps, $ENV{NAME_MAPPINGS};
            my $position;
            my @maps;
            foreach my $name_map ( @name_maps) {
                my ( $type, $windows, $unix ) = split /\|/ , $name_map;
                $position++;

                $windows =~ s/\\\\/\\/g;

                push @maps,
                {
                direction   => "win-unix",
                position    => $position,
                pattern     => $windows,
                replacement => $unix,
                },
                {
                direction   => "unix-win",
                position    => $position,
                pattern     => $unix,
                replacement => $windows,
                };
            }

            ### Format the values to display to STDOUT
            $self->_log(msg=>"input responses to use for 'cifs setup'", indent=>8);
            push my @outputs, sprintf(
                "         %-20.20s %-15.15s\n".
                "         %-20.20s %-15.15s",
                "Parameter",
                "Value",
                "-"x20,
                "-"x15
            );

            foreach my $param (sort keys %setup_of){
                push @outputs,
                sprintf ("         %-20.20s %-s",$param,$setup_of{$param});
            }

            $self->_log(msg=>[@outputs,"\n"]);

            $setup_of{vserver_name_mapping} = [@maps];


            my $cifs = NACL::MTask::CIFS->setup(
                command_interface => $node_obj,
                if_exists         => "purge",
                %setup_of,

            );
        }
    } catch NATE::BaseException with {
        my $exception_object = shift;
        $self->_log(msg=>"Exception is of type: " . ref ($exception_object));
        $self->_log(msg=>"Error text is: " . $exception_object->text());

        my @lines=split(/\n/,$exception_object->text());
        die ($lines[0])
            unless ($exception_object->text()=~/server is running/i);
    };

    return 1;
}

sub _vs_unix_user {
    my ($self, %args) = @_;

    my $command_interface = $args{command_interface};
    my $vserver           = $args{vserver};
    my $user              = $args{user};
    my $id                = $args{id};
    my $primary_gid       = $args{"primary-gid"};
    my $full_name         = $args{"full-name"};

    try {

    NACL::C::VserverServicesUnixUser->create(
        command_interface => $command_interface,
        vserver           => $vserver,
        user              => $user,
        id                => $id,
        "primary-gid"     => $primary_gid,
        "full-name"       => $full_name,
    );

    } catch NACL::APISet::Exceptions::CommandFailedException with {
        my $exception_object = shift;
        unless ($exception_object->text()=~/duplicate/i){
        $exception_object ->throw();
        }
    };

}


sub _vs_unix_group {
    my ($self, %args) = @_;

    my $command_interface = $args{command_interface};
    my $vserver           = $args{vserver};
    my $name              = $args{name};
    my $id                = $args{id};

    try {

    NACL::C::VserverServicesUnixGroup->create(
        command_interface => $command_interface,
        vserver           => $vserver,
        name              => $name,
        id                => $id,
    );

    } catch NACL::APISet::Exceptions::CommandFailedException with {
        my $exception_object = shift;
        unless ($exception_object->text()=~/duplicate/i){
        $exception_object ->throw();
        }
    };

}



###############################################################################
# Method:    _ontap_7mode
# Objective: configure ontap with standard setup expected by tests
###############################################################################
sub _ontap_7mode {
    my ($self,%args)=@_;
    my (
    $ads_server,
    $ads_domain,
    $node_obj,
    $apiset_obj,
    );

    try {
    
        foreach my $node (@{$self->{nodes}}) {
    
            $self->_log(msg=>"setting up ".$node,indent=>3);

            $node_obj=NACL::STask::Node->new(name=>$node);
            $apiset_obj=$node_obj->apiset(
                category => "Node",
                interface => "CLI",
                set => "7Mode",
            );
           
            $self->_log(msg=>"secureadmin ssh setup",indent=>6);
                $apiset_obj->execute_raw_command(
                command=>"secureadmin setup -q ssh 768 512 768"
            );

            $self->_log(msg=>"enabling protocol options",indent=>6);
            $apiset_obj->options(
                'httpd.admin.enable' => 'on',
                'httpd.enable'       => 'on',
                'rsh.enable'         => 'on',
                'rsh.access'         => 'all',
                'ssh.enable'         => 'on',
                'telnet.enable'      => 'on',
            );
   
            ### Add feature licenses
            $self->_feature_licenses(node_obj=>$node_obj);
			
            ### Check if release SN or later since license rqmts are different
            if ($self->_get_release_nbr($self->{release_id}) >= $self->_get_release_nbr("8.2.0")) {
                foreach my $cmd (
                   'options licensed_feature.iscsi.enable on',
                   'nfs on'
                ){
                  $apiset_obj->execute_raw_command(command=>$cmd);
	        }
            }

            ### Start up iSCSI service
            $apiset_obj->execute_raw_command(command=>"iscsi start");

            $self->_log(msg=>"enabling cluster failover",indent=>6);
            $apiset_obj->cf_enable();
			
            if ($self->{cifs_setup}) {
                $self->_log(msg=>"setting up CIFS",indent=>6);

                if ($ENV{CIFS_DOMAIN_LONG}){
                    $ads_domain=($ENV{CIFS_DOMAIN_LONG});	
                }
                else {
                    $ads_server=($ENV{AD_SERVER_FULLDOMAIN})
                    ? $ENV{AD_SERVER_FULLDOMAIN}
                    : 0;
                    ($ads_domain)=$ads_server=~/\.(.*)/;
                }
				
                my %setup_of = (
                    'security-style'   => "multiprotocol",
                    'auth-type'        => "ad",
                    'domain'           => $ads_domain,
                   # 'domain-user'       => "adadmin",
                   # 'domain-pass'       => "adadmin",
                    'domain-user'         => $ENV{AD_SERVER_USER}, #"adadmin",
                    'domain-pass'         => $ENV{AD_SERVER_PASSWORD}, #"adadmin",

                    'ou'                => "CN=computers",
                    'password'          => "",
                    'passwd-group'      => "y",
                    'wins-option'       => 1,
                    'wins-servers'      => $ENV{WINS_SERVER_IP}
                );

                if ($ENV{NTP_SERVER_IP}) {
                    $setup_of{'timed-enable'}     ="y";
                    $setup_of{'timed-server-list'} = $ENV{NTP_SERVER_IP};
                }


                ### Format the values to display to STDOUT
                $self->_log(msg=>"input responses to use for 'cifs setup'", indent=>8);
                push my @outputs, sprintf(
                "         %-20.20s %-15.15s\n".
                "         %-20.20s %-15.15s",
                "Parameter",
                "Value",
                "-"x20,
                "-"x15
                );

                foreach my $param (sort keys %setup_of){
                    push @outputs,
                        sprintf ("         %-20.20s %-s",$param,$setup_of{$param});
                }

                $self->_log(msg=>[@outputs,"\n"]);

                try {
                    my $cifs = NACL::MTask::CIFS->setup_7Mode(
                        'command_interface' => $node_obj,
                        %setup_of,
                        if_exists => "purge");
                } catch NACL::APISet::Exceptions::InvalidParamValueException with {
                  ### Temporary fix while determining cause of ERROR in
                  ### MTask::CIFS.pm that results in this error with
                  ### 7m HA pairs.
                  my $exception_object = shift;
                };
            }		
            else {
                 $self->_log(msg=>"enabling NTP",indent=>6);
                 foreach my $cmd (
                      'options timed.enable off',
                      'options timed.servers '.$ENV{NTP_SERVER_IP},
                      'rdate '.$ENV{NTP_SERVER_IP},
                      'options timed.enable on') {
                         $apiset_obj->execute_raw_command(command=>$cmd);
                       }
            }
        }
    } catch NATE::BaseException with {
        my $exception_object = shift;
        $self->_log(msg=>"Exception is of type: " . ref ($exception_object));
        $self->_log(msg=>"Error text is: " . $exception_object->text());
        my @lines=split(/\n/,$exception_object->text());
        die ($lines[0])
        unless ("@lines"=~/server is running/i);
    };

    return 1;
}

######################################################################
# P  R  I  V  A  T  E     M  E  T  H  O  D  S
######################################################################
# Method:    _get_mode
# Objective: return existing VSA ONTAP mode
###############################################################################
sub _get_mode {
    my ($self,%args)=@_;
    my $mode="";

    my $output=$self->{apiClientObj}->execute_raw_command(
        command=>"vm show ".$args{node},
        prompts=>['dvadmin'],
        'connectrec-timeout' => '10'
        );
    my @lines=split(/\n/,$output);
    ($mode)=$lines[4]=~(/\S+\s+\S+\s+(\S+)/);

    return $mode;
}
###############################################################################
# Method:    _get_release_nbr
# Objective: get ascii release id returned in normalized nbr
################################################################################
sub _get_release_nbr { 
    my ($self,$value)=@_;
    my (
        $nbr,
        @elements,
    );

    @elements=split(/\./,$value);
    $elements[1]=0 if(not defined $elements[1]);
    $elements[2]=0 if(not defined $elements[2]);

    $nbr=sprintf("%s%s%s%s%s%s",
        "0"x(3-length($elements[0])),$elements[0],
        "0"x(3-length($elements[1])),$elements[1],
        "0"x(3-length($elements[2])),$elements[2],
    );

    return $nbr;

}
###############################################################################
# Method:    _get_standalone_config
# Objective: get dvadmin commands for creating requested VSA type
################################################################################
sub _get_standalone_config {
    my ($self) =@_;
    my @cmds;
    my $host_obj=$self->{thost_of}{$self->{vmhost}};
    my ($bHa) = (scalar @{$self->{nodes}}%2==0) ? 1 : 0;

    foreach my $node (@{$self->{nodes}}) {
        # moved the following code into if else for each MODE since its different command for singlenode c-mode
        #push @cmds,
        #    "vm create ".$node." ".
        #    "\"".$self->{datastore}."\" ".
        #    "\"".$self->{datastore}."\" ".
        #    $self->{network_name}." ".
        #    $self->{image}." ".
        #    "mode=".$self->{mode}." ".
        #    "license=".$self->{thost_of}{$node}{license}." ".
        #    "hostname=".$node." ".
        #    "ipaddr=".$self->{thost_of}{$node}{MGMT_IP}. " ".
        #    "netmask=".$self->{thost_of}{$node}{MGMT_MASK}. " ".
        #    "gateway=".$self->{thost_of}{$node}{MGMT_GATEWAY}. " ".
        #    "password=netapp1!"." ".
        #    "vsphere_username=".$host_obj->{conn}{ssh}{username}." ".
        #    "vsphere_password=".$host_obj->{conn}{ssh}{password};

        if ($self->{mode}=~/7/) {
            push @cmds,
            "vm create ".$node." ".
            "\"".$self->{datastore}."\" ".
            "\"".$self->{datastore}."\" ".
            $self->{network_name}." ".
            $self->{image}." ".
            "mode=".$self->{mode}." ".
            "license=".$self->{thost_of}{$node}{license}." ".
            "hostname=".$node." ".
            "ipaddr=".$self->{thost_of}{$node}{MGMT_IP}. " ".
            "netmask=".$self->{thost_of}{$node}{MGMT_MASK}. " ".
            "gateway=".$self->{thost_of}{$node}{MGMT_GATEWAY}. " ".
            "password=netapp1!"." ".
            "vsphere_username=".$host_obj->{conn}{ssh}{username}." ".
            "vsphere_password=".$host_obj->{conn}{ssh}{password},
            "vm network connect ".$node." \"Network adapter 2\" VSA-data",
            "vm network connect ".$node." \"Network adapter 3\" VSA-data";
        } elsif ($self->{mode}=~/^C/){
             push @cmds,
            "vm create ".$node." ".
            "\"".$self->{datastore}."\" ".
            "\"".$self->{datastore}."\" ".
            $self->{network_name}." ".
            $self->{image}." ".
            "mode=".$self->{mode}." ".
            "license=".$self->{thost_of}{$node}{license}." ".
            "hostname=".$node." ".
            "ipaddr=".$self->{thost_of}{$node}{MGMT_IP}. " ".
            "netmask=".$self->{thost_of}{$node}{MGMT_MASK}. " ".
            "gateway=".$self->{thost_of}{$node}{MGMT_GATEWAY}. " ".
            "password=netapp1!"." ".
            "vsphere_username=".$host_obj->{conn}{ssh}{username}." ".
            "vsphere_password=".$host_obj->{conn}{ssh}{password}, 
            "vm network connect ".$node." \"Network adapter 1\" VSA-cluster",
            "vm network connect ".$node." \"Network adapter 2\" VSA-cluster",
            "vm network connect ".$node." \"Network adapter 3\" VSA-mgmt",
            "vm network connect ".$node." \"Network adapter 4\" VSA-data",
            "vm network connect ".$node." \"Network adapter 5\" VSA-data";
        } elsif ($self->{mode} eq "SNC"){
             push @cmds,
            "vm create ".$node." ".
            "\"".$self->{datastore}."\" ".
            "\"".$self->{datastore}."\" ".
            "\"".$self->{datastore}."\" ".
            $self->{network_name}." ".
            $self->{image}." ".
            #Starting SN.1 mode is detected based on license and not passed explicitly "mode=".$self->{mode}." ".
            "license=".$self->{thost_of}{$node}{license}." ".
            "clustername=".$node."-cluster". " ".
            "nodename=".$node. " ".
            "nodemgmt_ipaddr=".$self->{thost_of}{$node}{MGMT_IP}. " ".
            "clustermgmt_ipaddr=".$self->{thost_of}{$node}{CLUSTER_MGMT_IP}. " ".
            "clustermgmt_netmask=".$self->{thost_of}{$node}{CLUSTER_MGMT_MASK}. " ".
            "clustermgmt_gateway=".$self->{thost_of}{$node}{CLUSTER_MGMT_GATEWAY}. " ".
            "password=netapp1!"." ".
            "vsphere_username=".$host_obj->{conn}{ssh}{username}." ".
            "vsphere_password=".$host_obj->{conn}{ssh}{password},
            "vm network connect ".$node." \"Network adapter 1\" VSA-mgmt",
            "vm network connect ".$node." \"Network adapter 2\" VSA-data",
            "vm network connect ".$node." \"Network adapter 3\" VSA-data",
            "vm network connect ".$node." \"Network adapter 4\" VSA-data",
            "vm network connect ".$node." \"Network adapter 5\" VSA-data",
            "vm network connect ".$node." \"Network adapter 6\" VSA-data",
            "vm network connect ".$node." \"Network adapter 7\" VSA-data",
            "vm network connect ".$node." \"Network adapter 8\" VSA-data",
            ####ADDING DEMO LICENES TILL BURT# 652894 GETS FIXED
            "vm prop set ".$node." feature_licenses=IFSOONFLWWPCPFAAAAAAAAAAAAAA,UZPDPNFLWWPCPFAAAAAAAAAAAAAA,OSCPSNFLWWPCPFAAAAAAAAAAAAAA,WKUZNNFLWWPCPFAAAAAAAAAAAAAA,QDHLRNFLWWPCPFAAAAAAAAAAAAAA";
            if($self->{model}=~/FDVM200/i){
            #Uncomment this line after BURT# 652894 is fixed.
            #push @cmds, "vm prop set ".$node." feature_licenses=GIHGEKLHSOVOGDAAAAAAAAAAAAAA";
            }
        }

        my ($disk_cnt, $disk_size)=$self->{layout}=~/(\d+)\:(\S+)/;
        foreach my $cnt (1..$disk_cnt){
            push @cmds,
            "vm disk create ".
            $node." ".
            "\"".$self->{datastore}."\" ".
            $disk_size;
        }

        if ($bHa) {
            if ($self->{mode}=~/7/i ){
            push @cmds,
                "vm prop set ".$node." reboot_after_setup=YES";
            }

        }

        push @cmds,
            "vm prop set ".$node." raidtype=".$Raid_Of{$self->{raid_type}}." ",
            "vm prop set ".$node." tmz=".$self->{timezone}." ",
            "vm prop set ".$node." dns_ipaddr=".$ENV{FILER_DNS_SERVER_LIST}." ",
            "vm prop set ".$node." dns_domainname=".$ENV{FILER_DNS_DOMAINNAME};

        if ($self->{mode}=~/7/i) {
            push @cmds,
            "vm prop set ".$node." iface2_ipaddr=". $self->{thost_of}{$node}{DATA_IP1},
            "vm prop set ".$node." iface2_netmask=". $self->{thost_of}{$node}{DATA_MASK1},
            "vm prop set ".$node." iface3_ipaddr=". $self->{thost_of}{$node}{DATA_IP2},
            "vm prop set ".$node." iface3_netmask=". $self->{thost_of}{$node}{DATA_MASK2},
            "vm prop set ".$node." nis_domainname=netapp.com",
            "vm prop set ".$node." nis_ipaddr=".$ENV{NIS_SERVERS},
        } elsif($self->{mode}=~/^C/i){
            push @cmds,
            "vm prop set ".$node." clusternetwork_ipaddrs=".
                $self->{thost_of}{$node}{CLUSTER_IP1}.",".
                $self->{thost_of}{$node}{CLUSTER_IP2},
            "vm prop set ".$node." clusternetwork_netmask=".
                $self->{thost_of}{$node}{CLUSTER_MASK1};
        }
        
    }

    if ($self->{mode}=~/^c/i) {
        push @cmds,
            "cluster create ".$self->{cl_name}." ".
            "clusterlicense=ETYYFLXUQUMADFAAAAAAAAAAAAAA ".
           #"clusterlicense=JWFJEXMWZWYQSD ".
            "clustermgmt_ipaddr=".$self->{thost_of}{$self->{nodes}->[0]}->{CLUSTER_MGMT_IP}." ".
            "clustermgmt_netmask=".$self->{thost_of}{$self->{nodes}->[0]}->{CLUSTER_MGMT_MASK}." ".
            "clustermgmt_gateway=".$self->{thost_of}{$self->{nodes}->[0]}->{CLUSTER_MGMT_GATEWAY};

        foreach my $node (@{$self->{nodes}}) {
            push @cmds,
                "cluster add ".$self->{cl_name}." ".$node;
        }
    }

    if ($bHa){
        push @cmds,
            "ha create ".$self->{ha_name}." @{$self->{nodes}}";

        foreach my $node (@{$self->{nodes}}) {
            push @cmds,
                "vm network assign ".$node." \"Network adapter 6\" \"".$self->{thost_of}{$node}{ic_mac}."\"",
                "vm network designate ".$node." \"Network adapter 6\" \"ha\"";
        }

    }

    return @cmds;
}


###############################################################################
# Method:    _power_off
# Objective: power down the VSA
###############################################################################
sub _power_off {
    my ($self,%args)=@_;
    my $power="";
    my $mode="";
    my $output=$self->{apiClientObj}->execute_raw_command(
    command=>"vm stop --force ".$args{node},
    prompts=>['dvadmin'],
    'connectrec-timeout' => '300');

    if ($output=~/already powered off|no VM named/i) {
    $self->{last_config_mode} = $self->_get_mode(%args);
    return 1;
    }

    while ($power !~ /off/i) {
    $output=$self->{apiClientObj}->execute_raw_command(
        command=>"vm show ".$args{node},
        prompts=>['dvadmin'],
        'connectrec-timeout' => '10'
        );
    my @lines=split(/\n/,$output);
    ($power)=$lines[1]=~(/\S+\s+(\S+)/);
    ($mode)=$lines[4]=~(/\S+\s+\S+\s+(\S+)/);
    sleep(1);
    }
    $self->{last_config_mode}=$mode;
    return 1;
}

###############################################################################
# Method:    _start
# Objective: start up Cm VSA instance
###############################################################################
sub _start {
    my ($self,%args)=@_;
    my (
        $cnt,
        $output,
        $retry_cnt,
        $transit_obj,
        @nodes,
        @subtest_objs,
    );

    ### Retrieve array of nodes to start
    @nodes=@{$args{nodes}};

    try {

        foreach my $node (@nodes){

            ### Create transit object for managing node state
            $transit_obj = NACL::Transit->new(name =>$node);

            ### Start up the node
            $self->_log(msg=>"starting up VSA ".$node, indent=>3);
            while (1) {

                eval {
                    $output=$self->{apiClientObj}->execute_raw_command(
                        command=>"vm start ".$node,
                        prompts=>['dvadmin'],
                        'connectrec-timeout' => '20'
                    );
                };

                if ($output=~/can't join cluster/i || $output=~/timeout/i ||
                    $output=~/another task is in progress/i) {
                    ### Sometimes there is a time lag between dvadmin recognizing
                    ### when the cluster is up.  This can result in the 2nd node
                    ### in the cluster not starting up.  To circumvent this
                    ### condition, retry starting the 2nd VM several times over
                    ### the next 60s.
                    sleep(5);
                    redo if (++$retry_cnt < 12);
                }
                else {
                    die "OUTPUT:".$output."\nERROR:".$@ if ($output=~/err/i);
                }

                last;

            }

            ### Tranist to firmware prompt
            $self->_log(msg=>"transiting to firmwmare prompt", indent=>6);
            $transit_obj->change_state(to=>"FIRMWARE");

            ### Create a connection to the node's console
            my $conn_obj = Tharn::connect($node.'#console');
            my $apiset_obj = NACL::APISet->new( connection => $conn_obj,
                          category => "Node",
                          interface => "CLI",
                          set       => "Firmware",
                        );

            ### Update specific firmware parameters
            $self->_log(msg=>"setting loader bootarg variables",indent=>6);
            $apiset_obj->setenv(varname => "bootarg.setup.minroot", value =>"true")
                if ($self->{mode}=~/7/);
            $apiset_obj->setenv(varname => "bootarg.vm.no_poweroff_on_halt", value =>"true");

            ### Transition Node to CLI state using a background subtest.
            ### This will allow other nodes (where supported) to boot
            ### in parallel
            $self->_log(msg=>"boot up VSA to CLI prompt (be patient)",indent=>6);
            my $subtest_obj=Tharn::subtest(
                    \&_transit_cli,
                    -runid=>$node."_bootup",
                    -bg,
                    -onexit =>  sub {
                    my ($subtest_obj) = @_;
                    $subtest_obj->rslts();
                                },
                    "--", $self,$node);
            push @subtest_objs, $subtest_obj;
        }

        ### boot up node in a subtest, so if there are two nodes it
        ### can be done in parallel (7m HA only)
        $self->_log(msg=>"booting in progress ...",nocr=>1,indent=>9);
        foreach my $subtest_obj (@subtest_objs){
            while ($subtest_obj->is_alive()){
                $self->_log(msg=>($cnt-1)."s\n       ",nocr=>1) if ($cnt++%20==0);
                $self->_log(msg=>".",nocr=>1);
                sleep (1);
            }
        }
        $self->_log(msg=>($cnt-1)."s") if ($cnt++%20!=0);

        foreach my $subtest_obj (@subtest_objs){
            my @results = $subtest_obj->rslts();
            foreach my $result (@results) {
                next if $result->type()=~/WARN/;
                if ($result->type() =~ /FAIL/){
                    die $result;
                }elsif($result->type() =~ /PASS/) {
                    $self->_log(msg=>$result,indent=>6);
                }
            }
        }

    } catch NATE::BaseException with {
        my $exception_object = shift;
        die ($exception_object->text());
    };

    return 1;
}

###############################################################################
# Method:    _transit_cli
# Objective: transition to cli
###############################################################################
sub _transit_cli {
    my ($self,$node) = @_;
    my $done=1;
    my $retry_cnt;
    my $transit_obj;

    ### Create a transit object for managing node state transitions
    $transit_obj = NACL::Transit->new(name =>$node);

    while (1) {
        try {

            ### Transition VSA to cli prompt
            $transit_obj->change_state(to=>"CLI", timeout=>1200);

            ### If 7m enable rsh
            if ($self->{mode}=~/7/) {
                my $conn_obj = Tharn::connect($node.'#console');
                my $apiset_obj = NACL::APISet->new( connection => $conn_obj,
                                  category => "Node",
                                  interface => "CLI",
                                  set       => "Firmware",
                                );
                $apiset_obj->execute_raw_command(command=>"options rsh.enable on");
            }

            Tharn::logresult("PASS", msg => "$node has booted to the CLI");
            $done=1;
        }
        catch NACL::Transit::Exceptions::Internal with {
            my $exception_object = shift;
            $done=0;
        }
        catch NACL::Transit::Exceptions::TransitException with {
            my $exception_object = shift;
            $done=0;
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $self->_log(msg=>"IN BASE EXCEPTION");
            $self->_log(msg=>"Exception is of type: " . ref ($exception_object));
            $self->_log(msg=>"Error text is: " . $exception_object->text());
            Tharn::logresult("FAIL", msg => "$node failed during bootup to CLI");
            $done=1;
        };

        ### Continue reboot transition if not done yet
        redo if (not $done and ++$retry_cnt < 3);

        ### exit while loop
        last;
    }

    1;

}

1;
