#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/VSA/Storage.pm#1 $

package VSA::Storage;

use strict;

use Data::Dumper qw(Dumper);
use Error qw(_clear_err _set_err);
use IO::File;
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR SCALARREF BOOLEAN OBJECT ARRAYREF UNDEF);
use Time::HiRes qw(gettimeofday);
use Tharn qw($Log say);
use Utils qw(make_host);
use VSA::UICommon qw(_caller _reformat_err _log _refresh_cache
                     _size_to_mbytes _trim);
                     
use base qw(VSA::UICommon Error);

###############################################################################
# Method:    new()
# Objective: Create new Storage instance
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my ($class,%args) = @_;
    my (
      $errmsg,
      $hosttype,
      $self,
    );
    
    ### bless instance into class
    $self= bless{}, $class;
    
    ### Clear error buffer
    $self->_clear_err();
    
    ### Determine err handle upfront in case validation method
    ### fails before error handler is selected
    $self->{on_err}=(defined $args{on_err} and $args{on_err}=~/die|rtn/)
        ? $args{on_err}
        : "die";
    
    eval {
        
         ($class,%args) = validate_mixed (
            @_,
            [ { type => SCALAR} ],
            {
                hostname  => {type => SCALAR},
                conn_type => {type => SCALAR, optional=>1,
                              default=>"rsh-socket",
                              callbacks=>{
                              'legal values'=>
                                sub{shift()=~/rsh-socket|rsh-persay|ssh|telnet/i
                             }}},
                conn_name => {type => SCALAR, optional=>1},
                loghdl    => {type => SCALARREF, optional=>1 },
                on_err    => {type => SCALAR, optional=>1, default=>"die",
                              callbacks=>{
                                'legal values'=>sub{shift()=~/die|rtn/i
                             }}},
                password  => {type => SCALAR, optional=>1,},
                show_cmd  => {type => SCALAR, optional=>1, default=>0},
                timeout   => {type => SCALAR, optional=>1, default=>10,
                              regex => qr/^\d+$/ },
                username  => {type => SCALAR, optional=>1,},
               
            }
        );
        
        ### Create a host object
        $self->{loghdl}   = $args{loghdl};
        $self->{thost_of} = Utils::make_host($args{hostname});
        $self->{timeout}  = $args{timeout};
        $hosttype         = $self->{thost_of}->hosttype() ;
    
        ### Figure out what host implementation to use
        ### Make sure any metadata prior to this call is
        ### resaved since the object reference is being
        ### blessed into another class.
        $self->_log(msg=>"hosttype: $hosttype");
        
        if ($hosttype =~ /^filer-cfe$/i) {
            $self->_log(msg=>"creating filer");
            require VSA::Storage::Filer;
            $self = VSA::Storage::Filer->_new(%args);            
        }
        else {
            die "Host platform '".$hosttype."' is not ",
                "currently supported by VSA::Storage";   
        }
          
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }

    return $self;
}


###############################################################################
# Method:    on_err
# Objective: Return errhdlr_type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub on_err {
    my ($self,$value)=@_; 
    my ($errmsg);
    
    if (defined $value) {
        
        unless ($value=~/die|rtn/){
            $errmsg="Invalid value '".$value."'";
            die _caller($errmsg);
        }
        
        $self->{on_err}=$value;
    }
    
    return $self->{on_err}; 
}

1;
