#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/VSA/Storage/Filer/Igroup.pm#1 $
## @summary Igroup interface
## @author dl-vsa-qa
## @status private

package VSA::Storage::Filer::Igroup;

use strict;
use Data::Dumper qw(Dumper);
use Error qw(_clear_err _set_err);
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR SCALARREF BOOLEAN OBJECT ARRAYREF UNDEF);
use Tharn qw($Log);
use VSA::UICommon qw(_caller _reformat_err _log _refresh_cache
                     _size_to_mbytes _trim);

use base qw(VSA::UICommon Error);

###############################################################################
# Method:    destroy
# Objective: destroy igroup
# Details:   See POD documentation at the end of this file.
###############################################################################
sub destroy {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        @outputs
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        $cmd="igroup destroy -f ".$self->get_name();
        
        @outputs=$self->{filer}->_execute(cmd=>$cmd,hide=>1);
        if (@outputs){
            die $outputs[0];
        }
        
        $self->{state}="DESTROYED";
        
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->{filer}->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return 0; 
    }
     
    1;
}


###############################################################################
# Method:    get_*()
# Objective: Get object metadata
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_hostname { my ($self) = @_; return $self->{filer}->get_hostname();}
sub get_iqns     {
    my ($self)=@_;
    
    return (ref($self->{iqns}) eq "ARRAY")
        ? (wantarray())
            ? @{$self->{iqns}}
            : $self->{iqns}
        : $self->{iqns};
}
sub get_name     { my ($self)=@_; $self->{name}; }
sub get_protocol { my ($self)=@_;  $self->{protocol}; }
sub get_state    { my ($self)=@_; ; $self->{state}; }
sub get_type     { my ($self)=@_;  $self->{type}; }

###############################################################################
# Method:    refresh
# Objective: Initialize metadata for object instance
# Details:   See POD documentation at the end of this file
###############################################################################
sub refresh {
    my ($self)=@_;
    my ($errmsg);
    
    ### Initialize error buffer
    $self->_clear_err();
    
    eval {
        $self->_init();
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->{filer}->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    return $self;
}

###############################################################################
#  P  R  I  V  A  T  E
###############################################################################
###############################################################################
# Method:    _create
# Objective: Create new igroup
###############################################################################
sub _create {
    my ($self,%args)=@_;
    my (
        $cmd,
        $protocol,
        @iqns,
        @outputs
    );
    
    $protocol=($self->get_protocol=~/iscsi/i) ? "-i" : "-f";
    @iqns=$self->get_iqns();
    
    $cmd="igroup create ".
         $protocol . " ".
         "-t ".$self->get_type()." ".
         $self->get_name()." ".
         "@iqns";
    
    @outputs=$self->{filer}->_execute(cmd=>$cmd,hide=>1);
    if (@outputs){
        die $outputs[0];
    }
    
    1;
}

###############################################################################
# Method:    _init
# Objective: Initialize metadata for object instance
###############################################################################
sub _init {
    my ($self)=@_;
    my (
        $cmd,
        $errmsg,
        $iqn,
        $login_cnt,
        $protocol,
        $state,
        $type,
        @iqns,
        @outputs,
    );

    ### Execute command
    $cmd="igroup show -v ".$self->get_name();
    @outputs=$self->{filer}->_execute(cmd=>$cmd,hide=>1);
    
    $login_cnt=0;
    foreach my $line (@outputs[0..scalar(@outputs)-1]) {
        if ($line=~/\(iSCSI|FCP\)/) {
            ($protocol)=$line=~/\((\S+)\)/;
        }
        elsif ($line=~/OS Type/) {
            ($type)=$line=~/OS Type: (\S+)/;
        }
        elsif ($line=~/Member/){
            ($iqn,$state)=$line=~/Member: (\S+)\s\((\S+ \S+)/;
            push @iqns,$iqn;
            $login_cnt++ if ($state=~/logged in/);
        }
    }

    $self->{protocol}=$protocol;
    $self->{type}=$type;
    $self->{iqns}=\@iqns;
    $self->{state}=$login_cnt."/".@iqns."-logged in";
        
    return 1;
}

###############################################################################
# Method:    _new
# Objective: Create new igroup object
############################################################################### 
sub _new {
    my ($class,%args) = @_;
    my (
      $errmsg,
      $self,
    );
    
    ### bless instance into base class
    $self= bless {}, $class;
        
    ### Validate input arguments
    ($class,%args) = validate_mixed (
        @_,
        [ { type => SCALAR} ],
        {
            name      => {type => SCALAR},
            type      => {type => SCALAR|UNDEF, optional=>1},
            protocol  => {type => SCALAR|UNDEF, optional=>1},
            iqns      => {type => SCALAR|UNDEF, optional=>1},
            timeout   =>  {type => SCALAR, optional=>1, default=>30,
                          regex => qr/^\d+$/ },
            _create    => {type => SCALAR, optional=>1, default=>0 },
            _filer     => {type => OBJECT},
            _loghdl    => {type => SCALARREF|UNDEF, optional=>1},
            _parent    => {type => OBJECT},
        }
    );
    
    ###  Initialize object metadata
    $self->{name}    = $args{name};
    $self->{type}    = $args{type};
    $self->{protocol}= $args{protocol};
    $self->{iqns}    = $args{iqns}; 
    $self->{timeout} = $args{timeout};
    $self->{filer}   = $args{_filer};
    $self->{parent}  = $args{_parent};
    $self->{create}  = $args{_create};
    $self->{loghdl}  = $args{_loghdl};

    $self->_create() if ($self->{create});

    $self->refresh();
    
    return $self;
}

1;
