#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/VSA/Storage/Filer/Lun.pm#1 $
## @summary Lun interface 
## @author dl-vsa-qa
## @status private

package VSA::Storage::Filer::Lun;

use strict;
use Data::Dumper qw(Dumper);
use Error qw(_clear_err _set_err);
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR SCALARREF BOOLEAN OBJECT ARRAYREF UNDEF);
use Tharn qw($Log);
use VSA::UICommon qw(_caller _reformat_err _log _refresh_cache
                     _size_to_mbytes _trim);

use base qw(VSA::UICommon Error);

###############################################################################
# Method:    destroy
# Objective: destroy lun
# Details:   See POD documentation at the end of this file
###############################################################################################################################################################
sub destroy {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        @outputs
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        $cmd="lun destroy -f ".$self->get_path();
        
        @outputs=$self->{filer}->_execute(cmd=>$cmd,hide=>1);
        if (@outputs){
            die $outputs[0];
        }
      
        $self->{state}="DESTROYED";
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->{filer}->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
     
    return $self;
}

###############################################################################
# Method:    get_*()
# Objective: retrieve object metadata
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_access   { my ($self)=@_; $self->_clear_err(); $self->{access}; }
sub get_hostname { my ($self) = @_; return $self->{filer}->get_hostname();}
sub get_id       { my ($self)=@_; $self->_clear_err(); $self->{id};     }
sub get_igrp     { my ($self)=@_; $self->_clear_err(); $self->{igrp};   }
sub get_mapped   { my ($self)=@_; $self->_clear_err(); $self->{mapped}; }
sub get_name     { my ($self)=@_; $self->_clear_err(); $self->{name};   }
sub get_path     { my ($self)=@_; $self->_clear_err(); $self->{path};   }
sub get_size     { my ($self)=@_; $self->_clear_err(); $self->{size};   }
sub get_state    { my ($self)=@_; $self->_clear_err(); $self->{state};  }
sub get_type     { my ($self)=@_; $self->_clear_err(); $self->{type};   }
sub get_vol      { my ($self)=@_; $self->_clear_err(); $self->{vol};    }

################################################################################
# Method:    info
# Objective: return general lun info 
# Details:   See POD documentation at the end of this file
################################################################################
sub info {
    my ($self,%args)=@_;
    my (
        $header,
        @bufs,
    );
    
    $header=(defined $args{header}) ? $args{header} : 1;
    
    push @bufs, $self->info_header() if ($header);
           
    push @bufs,
        sprintf("\t%-30.30s %-3.3s %-7.7s %-15.15s %-15.15s",   
        $self->get_path(),
        $self->get_id(),
        $self->get_type(),
        $self->get_state(),
        $self->get_size()
        );

    return (wantarray()) ? @bufs : [@bufs];
    
}

################################################################################
# Method:    info_header
# Objective: return info header
# Details:   See POD documentation at the end of this file
################################################################################
sub info_header {
    my ($self)=@_;
    
    return
        sprintf("\t%-30.30s %3.3s %-7.7s %-15.15s %-15.15s\n".
                "\t%-30.30s %3.3s %-7.7s %-15.15s %-15.15s",
                "Lun Path","Id","Type", "State","Size", 
                "="x30,"="x3,"="x7, "="x15,"="x15);
}

################################################################################
# Method:    map
# Objective: map lun
# Details:   See POD documentation at the end of this file
################################################################################
sub map {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        $id,
        @outputs
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    ### Validate input arguments
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                igrp      => {type => SCALAR},
            }
        );
    
        $cmd="lun map ".$self->get_path()." ".$args{igrp};
        @outputs=$self->{filer}->_execute(cmd=>$cmd,hide=>1);
        if (@outputs){
            die $outputs[0];
        }
      
        $self->{igrp}=$args{igrp};
        
        $cmd="lun show -m ".$self->get_path();
        @outputs=$self->{filer}->_execute(cmd=>$cmd,hide=>1);
        ($id)=$outputs[0]=~/\S+\s+\S+\s+(\d+)/;
        if (length($id)==0){
            die $outputs[0];
        }
        
        $self->{id}=$id;
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->{filer}->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
     
    return $self;
    
}


###############################################################################
# Method:    offline
# Objective: Offline lun
# Details:   See POD documentation at the end of this file
###############################################################################
sub offline {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        @outputs
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    $cmd="lun offline ".$self->get_path();
    eval {     
        @outputs=$self->{filer}->_execute(cmd=>$cmd,hide=>1);
        if (@outputs){
            die $outputs[0];
        }
    
        $self->{state}="offline";
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->{filer}->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $self;
}

###############################################################################
# Method:    online
# Objective: Online lun
# Details:   See POD documentation at the end of this file
###############################################################################
sub online {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        @outputs
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    $cmd="lun online ".$self->get_path();
    eval {     
        @outputs=$self->{filer}->_execute(cmd=>$cmd,hide=>1);
        if (@outputs){
            die $outputs[0];
        }
    
        $self->{state}="online";
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->{filer}->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $self;
}

###############################################################################
# Method:    refresh
# Objective: Initialize metadata for object instance
# Details:   See POD documentation at the end of this file
###############################################################################
sub refresh {
    my ($self)=@_;
    my ($errmsg);
    
    ### Initialize error buffer
    $self->_clear_err();
    
    eval {
        $self->_init();
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->{filer}->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    return $self;
}

###############################################################################
# Method:    unmap
# Objective: unmap lun
# Details:   See POD documentation at the end of this file
###############################################################################
sub unmap {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        $id,
        @outputs
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    ### Validate input arguments
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                igrp => {type => SCALAR, optional=>1,
                         default=>$self->get_igrp()},
            }
        );
    
        $cmd="lun unmap ".$self->get_path()." ".$args{igrp};
        @outputs=$self->{filer}->_execute(cmd=>$cmd,hide=>1);
        if (@outputs){
            die $outputs[0];
        }
      
        $self->{igrp}="";
        $self->{id}=$id;
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->{filer}->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
     
    return $self;
    
}

###############################################################################
#  P  R  I  V  A  T  E
###############################################################################
# Method:    _create
# Objective: Create new volume
###############################################################################
sub _create {
    my ($self,%args)=@_;
    my (
        $cmd,
        @outputs
    );
    
    $cmd="lun create ".
         "-s ".$self->get_size()." ".
         "-t ".$self->get_type()." ".
         $self->get_path();
    
    @outputs=$self->{filer}->_execute(cmd=>$cmd,hide=>1);
    if (@outputs and "@outputs" !~ /created a LUN/i){
        die $outputs[0];
    }
    
    1;
}

###############################################################################
# Method:    _init()
# Objective: Initialize metadata for object instance
###############################################################################
sub _init{
    my ($self)=@_;
    my (
        $access,
        $cmd,
        $errmsg,
        $id,
        $igrp,
        $maps,
        $path,
        $reserve,
        $size,
        $state,
        $type,
        @outputs,
    );
    
    ### Execute command
    $cmd="lun show -v ".$self->get_path();
    @outputs=$self->{filer}->_execute(cmd=>$cmd,hide=>1);
    
    if ("@outputs"=~/mapped/){
       ($path,$size,$access,$state,$reserve,$type,$maps)="@outputs" =~ 
       /(\S+)\s+(\S+)\s+\S+\s+\((\S+), (\S+),.*\).*Reservation: (\S+).*Type: (\S+) Maps: (\S+)/;
       
       ($igrp,$id)=$maps=~/(\S+)=(\d+)/;
       $self->{igrp}=$igrp;
       $self->{id}=$id;
     
    }
    else {
        ($path,$size,$access,$state,$reserve,$type)="@outputs" =~ 
        /(\S+)\s+(\S+)\s+\S+\s+\((\S+), (\S+)\).*Reservation: (\S+).*Type: (\S+)/;
    }
    
    $self->{path}=$path;
    $self->{size}=$size;
    $self->{access}=$access;
    $self->{state}=$state;
    $self->{reserve}=$reserve;
    $self->{type}=$type;
  
    return 1;
}

###############################################################################
# Method:    _new
# Objective: Create new lun object
###############################################################################
sub _new {
    my ($class,%args) = @_;
    my (
      $errmsg,
      $self,
    );
    
    ### bless instance into base class
    $self= bless {}, $class;
        
    ### Validate input arguments
    ($class,%args) = validate_mixed (
        @_,
        [ { type => SCALAR} ],
        {
            name      => {type => SCALAR},
            size      => {type => SCALAR},
            access    => {type => SCALAR, optional=>1},
            state     => {type => SCALAR, optional=>1},            
            mapped    => {type => SCALAR, optional=>1},
            path      => {type => SCALAR, optional=>1},
            timeout   => {type => SCALAR, optional=>1, default=>30,
                          regex => qr/^\d+$/ },
            type      => {type => SCALAR, optional=>1},
            _create   => {type => SCALAR, optional=>1, default=>0 },
            _filer    => {type => OBJECT},
            _loghdl   => {type => SCALARREF|UNDEF, optional=>1},
            _parent   => {type => OBJECT},
        }
    );
    
    ###  Initialize object metadata
    $self->{name}    = $args{name};
    $self->{size}    = $args{size};
    $self->{access}  = $args{access};
    $self->{state}   = $args{state};    
    $self->{mapped}  = $args{mapped};  
    $self->{timeout} = $args{timeout};
    $self->{type}    = $args{type};
    $self->{filer}   = $args{_filer};
    $self->{parent}  = $args{_parent};
    $self->{create}  = $args{_create};
    $self->{loghdl}  = $args{_loghdl};
    $self->{vol}     = $args{_parent}->get_name();
    $self->{path}    = (defined $args{path})
        ? $args{path}
        : "/vol/".$self->get_vol()."/".$self->get_name();

    $self->_create() if ($self->{create});
    
    $self->refresh();
    
    return $self;
}


1;
