#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/VSA/UICommon.pm#1 $
## @summary Common UI functions
## @author dl-vsa-qa
## @status private

package VSA::UICommon;
use strict;

#use VSA::ConnMgr;
use Data::Dumper;
use Error;
use Module::Common qw(validate_mixed);
use Tharn;
use Time::HiRes qw(gettimeofday);
use Params::Validate qw(SCALAR SCALARREF HASHREF ARRAYREF OBJECT UNDEF);

use base qw(Error);

############################################################################
# P  R  I  V  A  T  E
##############################################################################
# Method:    _caller
# Objective: Issue arbitrary command to host and get results..
###############################################################################
sub _caller() {
    my ($self,$errmsg)=@_;
    my $parent=(caller(1))[3];
    my $line_nbr=(caller(0))[2];
    return "ERROR: " . $parent . "():" . $line_nbr . " - ".$errmsg;
}

###############################################################################
# Method:    _reformat_err()
# Objective: Parse and format error message
###############################################################################
sub _reformat_err {
    my ($self,$err)=@_;
    my (
        $callback,
        $errmsg,
        $param,
        $value,
        );
    
    $Log->comment($err);
    
    ($err)=(ref($err) and $err->isa("NATE::Result")) ? $err->messages() : $err;
    $err=~s/\n//g;

    ### Error may have been thrown.  Remove ERROR: if it exists.
    ($err)=$err=~/(.*)at \// if ($err=~/ at \/.* line /);
    $err=~s/ERROR:.*-\s//;
    

    if ($err=~/Mandatory parameter/i){
        ($param)=$err=~/Mandatory parameter (.*) /;
        $errmsg="Mandatory input parameter ".$param."'";
    }
    elsif ($err=~/was not listed/i){
        ($param)=$err=~/options: (\S+)/;
        $errmsg="Invalid input parameter '".$param."'";
    }
    elsif ($err=~/callback/i){
        ($param,$value,$callback)=
            $err=~/'(\S+)' parameter \(\"(\S+)\"\).*'(.*)' callback/i;
        $errmsg="Failed callback '".$callback. "' for ". $param."=>".$value;
    }
    elsif ($err=~/did not pass/i){
        ($param,$value)=$err=~/The '(\S+)' parameter \((\S+)\)/;
        $errmsg="Invalid input parameter value ".$param."=>'".$value."'";
    }
    else {
        ### Error was not due to function input
        #$self->_log(msg=>"MESSAGE NOT FOUND");
        return $err;
    }
    
    return $errmsg;
}

###############################################################################
# Method:    _get_loghdl
# Objective: get/set log handle
###############################################################################
sub _get_loghdl {
    my ($self)=@_; 
    
    return $self->{loghdl}; 
}

###############################################################################
# Method: _log
# Objective: display library messages to stdout and natelog
################################################################################
sub _log{
    my ($self,%args) = @_;
    my (
        $bFirst,
        $dash,
        @lines,
        $indent,
        $nocr,
    );

    $indent=0;

    @lines=(ref $args{msg} eq "ARRAY") ? @{$args{msg}} : ($args{msg});
    $indent=(defined $args{indent}) ? $args{indent} : 0;
    $nocr=(defined $args{nocr}) ? $args{nocr} : 0;
    $bFirst=1;
    
    if (defined $self->_get_loghdl()){
        my $loghdl=$self->_get_loghdl();
        if (not $indent){
            foreach my $line (@lines){
                print {$$loghdl} $line.
                (($nocr==1) ? " " : "\n");
            }
        }
        else {
            foreach my $line (@lines){
                $dash = ($bFirst == 1)
                    ? " "x($indent-1)."- "
                    : " "x$indent;
                $bFirst=0;
                print {$$loghdl} $dash.$line.
                (($nocr==1) ? " " : "\n");
            }
        }
        $Log->comment(join("\n",@lines));
      
    }

}

###############################################################################
# Method:    _refresh_cache
# Objective: Return errhdlr type
###############################################################################
sub _refresh_cache(){
    my ($self,%args)=@_;
 
    my $tmpObjs=[];
  
    foreach my $tmpObj (@{$self->{$args{name}}}){
        push @{$tmpObjs}, $tmpObj
            if ($tmpObj->get_state() !~ /DESTROYED/);
    }
    
    $self->{$args{name}}=$tmpObjs;
    
}

################################################################
# Subroutine Name: _size_to_kbytes()
# Objective:       convert size strings to kbytes.
# Details:         See POD documentation at the end of this file
################################################################
sub _size_to_kbytes {
    my ($self,$size)=@_;
    my (
        $kbytes,
        $nbr,
        $unit
    );
    
    ### If no units then byte 
    $size .= "k" if ($size=~/^\d+$/);
   
    ### Separate number and units
    ($nbr,$unit)=$size=~/^(\d+\.?\d*)\s{0,}(\w+)/;
    return $nbr if ($unit=~/k/i);

    if ($unit =~/^b/i) {
     $kbytes = $nbr/1024;
    }
    elsif ($unit =~/^m/i) {
     $kbytes = $nbr * 1024;
    }
    elsif ($unit=~/^g/i){
     $kbytes = $nbr * 1024**2;
    }
    elsif ($unit=~/^t/i){
     $kbytes = $nbr * 1024**3;
    }
    return int($kbytes);
}

################################################################
# Subroutine Name: _size_to_mbytes()
# Objective:       convert size strings to mbytes.
# Details:         See POD documentation at the end of this file
################################################################
sub _size_to_mbytes {
    my ($self,$size)=@_;
    my (
        $mbytes,
        $nbr,
        $unit
    );
    
   ### If no units then byte 
   $size .= "b" if ($size=~/^\d+$/);
   
   ### Separate number and units
   ($nbr,$unit)=$size=~/^(\d+\.?\d*)\s{0,}(\w+)/;
   return $nbr if ($unit=~/m/i);

   if ($unit =~/^b/i) {
    $mbytes = $nbr/1024**2;
   }
   elsif ($unit =~/^k/i) {
    $mbytes = $nbr/1024;
   }
   elsif ($unit=~/^g/i){
    $mbytes = $nbr * 1024;
   }
   elsif ($unit=~/^t/i){
    $mbytes = $nbr * 1024**2;
   }
   return int($mbytes);
}

###############################################################################
# Method:    _trim
# Objective: remove leading/triling whitespace
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _trim
{
    my @out = @_;
    for (@out) {
        s/^\s+//;
        s/\s+$//;
    }
    return wantarray ? @out : $out[0];
}

1;
