#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/VSA/Vmhost.pm#1 $
## @summary Vm host base class
## @author dl-vsa-qa
## @status private

package VSA::Vmhost;

use strict;

use Data::Dumper qw(Dumper);
use Error qw(_clear_err _set_err);
use IO::File;
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR SCALARREF BOOLEAN OBJECT ARRAYREF UNDEF);
use Time::HiRes qw(gettimeofday);
use Tharn qw($Log say);
use Utils qw(make_host);
use VSA::UICommon qw(_caller _reformat_err _log _refresh_cache
                     _size_to_mbytes _trim);
                     
use VSA::Vmhost::Sanlun;
use VSA::Vmhost::DevMgr;
use VSA::Vmhost::Lvm;
use NACL::APISet;
use base qw(VSA::UICommon Error);


###############################################################################
# Method:    new()
# Objective: Create new instance of Filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my ($class,%args) = @_;
    my (
      $cmd,
      $connObj,
      $errmsg,
      $host_of,
      $os_ver,
      $self,
      @outputs,
    );
    
    ### bless instance into class
    $self= bless{}, $class;
    
    ### Clear error buffer
    $self->_clear_err();
    #$self->{call_stack_on_error}=1;
    
    ### Determine err handle upfront in case validation method
    ### fails before error handler is selected
    $self->{on_err}=(defined $args{on_err} and $args{on_err}=~/die|rtn/)
        ? $args{on_err}
        : "die";
    
    ### Create a host object
    $host_of=Utils::make_host($args{hostname});
    $self->{host_of}=$host_of;
    $self->{hosttype}=$host_of->{hosttype}; 
    $self->{loghdl}   = $args{loghdl};
    $os_ver=$host_of->{OS_VER};
    
    eval {
=comment        
        ### Validate input arguments
        ($class,%args) = validate_mixed (
            @_,
            [ { type => SCALAR} ],
            {
                hostname  => {type => SCALAR},
                conn_type => {type => SCALAR, optional=>1, default=>"ssh",
                              callbacks=>{
                                'legal values'=>
                                    sub{shift()=~/ssh/i
                             }}},
                conn_name => {type => SCALAR, optional=>1},
                loghdl    => {type => SCALARREF, optional=>1 },
                on_err    => {type => SCALAR, optional=>1, default=>"die",
                              callbacks=>{
                                'legal values'=>sub{shift()=~/die|rtn/i
                             }}},
                password  => {type => SCALAR, optional=>1,},
                protocol  => {type => SCALAR, optional=>1, default=>"iscsi",
                              callbacks=>{
                                'legal values'=>sub{shift()=~/iscsi/i
                             }}},
                show_cmd  => {type => SCALAR, optional=>1, default=>0},
                timeout   => {type => SCALAR, optional=>1, default=>30,
                              regex => qr/^\d+$/ },
                username  => {type => SCALAR, optional=>1,},
               
            }
        );
        
        $self->{loghdl}=$args{loghdl};
        
        ### Create connection to the host
        $self->_log( 
            msg=>"creating $args{conn_type} connection to ".$args{hostname},
            indent=>3);
        
        $connObj=$self->_connect(
            name=>(defined $args{conn_name})
                ? $args{conn_name}
                : $args{conn_type},
            type=>$args{conn_type},
            hostname=>$args{hostname},
            username=>$args{username},
            password=>$args{password},
            timeout=>$args{timeout}
        );
        
        $self->_log(msg=>"DBG2> PASSWORD IS ".$self->get_root_password());
=cut        
        ### Figure out what host implementation to use
        ### Make sure any metadata prior to this call is
        ### resaved since the object reference is being
        ### blessed into another class.
        if ($self->get_hosttype() =~ /linux-vmware/i) {
            require VSA::Vmhost::ESXhost;
            #$self = VSA::Vmhost::ESXhost->_new(connection=>$connObj);
            $self = VSA::Vmhost::ESXhost->_new(
                %args,
                shell_type=>($host_of->{OS_VER}=~/^5/) ? "sh" : "bash"
                );
            $self->{os_ver}=$os_ver;
        }
        else {
            die ("Host platform '".$self->get_hosttype()."' not",
                 "currently supported by VSA::Vmhost object");   
        }
        
      ##  $self->_log(msg=>"DBG3> PASSWORD IS ".$self->get_password()); 
       # $self->_log(msg=>"DBG3> USERNAME IS ".$self->get_username());
=comment        
        ###  Set object metadata
        #$self->{password}   = $args{password};
        #$self->{connection} = $connObj;
        $self->{name}     = $args{hostname};
        $self->{on_err}   = $args{on_err};
        $self->{show_cmd} = $args{show_cmd};
        $self->{timeout}  = $args{timeout};
       # $self->{loghdl}   = $args{loghdl};
        $self->{protocol} = $args{protocol};
        $host_of=Utils::make_host($args{hostname});
        $self->{hosttype}=$host_of->{hosttype};
        $self->{ip}=$host_of->{default_ip};
=cut        
        ### Retrieve os version
        @outputs=$self->execute(cmd=>"uname -r",hide=>1);      
        $self->{os_version}=$outputs[0];
        $self->{mfg} = "VMware" if ($self->{os_version}=~/esx/i); 
         
        ### Retrieve architecture
        @outputs=$self->execute(cmd=>"uname -p",hide=>1);      
        $self->{arch}=$outputs[0];
        
        ### Create a Sanlun object
        #$self->_log(msg=>"creating sanlun object",indent=>3); 
       # $self->{sanlunObj} = VSA::Vmhost::Sanlun->_new(_parent=>$self);
          
    };
    if ($@){
        #$self->_log("msg=>DEBUG: $@");
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }

    return $self;
}

###############################################################################
# Method:    create_devmgr()
# Objective: Create and pass back a devmgr handle
# Details:   See POD documentation at the end of this file.
###############################################################################
sub create_devmgr_obj {
    my ($self, %args) = @_;
    my (
        $dmObj,
        $errmsg,
       );
    
    ### Initialize error buffer
    $self->_clear_err();
    
    return $self->{dmObj} if (defined $self->{dmObj});
    
    eval {
        $dmObj = VSA::Vmhost::DevMgr->_new(
			%args,
            _parent=>$self,
            #_loghdl=>$self->_get_loghdl()
        );
        
        $self->{dmObj} = $dmObj; 
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $self->{dmObj};
}

###############################################################################
# Method:    create_devmgr()
# Objective: Create and pass back a devmgr handle
# Details:   See POD documentation at the end of this file.
###############################################################################
sub create_lvm_obj {
    my ($self, %args) = @_;
    my (
        $lvmObj,
        $errmsg,
       );
    
    ### Initialize error buffer
    $self->_clear_err();
    
    #return $self->{dmObj} if (defined $self->{dmObj});
    
    eval {
        $lvmObj = VSA::Vmhost::Lvm->_new(
            %args,
            _parent=>$self,
            #_loghdl=>$self->_get_loghdl()
        );
        
        $self->{lvmObj} = $lvmObj; 
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $self->{lvmObj};
}

##############################################################################
# Method:    execute
# Objective: Issue arbitrary command to host and get results..
###############################################################################
sub execute
{
    my($self, %args) = @_;
    my(
       $buffer,
       $connection,
       $cmd_fmt,
       $errmsg,
       $t0,
       $t1,
       @arr,
       @rows,
       );

    ### Clear errors.
    $self->_clear_err;

    ### Validate input arguments
    ($self,%args) = validate_mixed (
        @_,
        [ { type => OBJECT} ],
        {   cmd    => {type => SCALAR}, 
            hide    => {type => SCALAR, optional=>1, default=>0},  
            timeout => {type => SCALAR, optional=>1,
                        default=>$self->get_timeout(),
                        regex => qr/^\d+$/},
        }
    );
   
     
    ### Print the command unless the user has specified otherwise.  
    if (!$args{hide} or $self->show_cmd()){
       $cmd_fmt=$self->get_name()."(HOST)> ".$args{cmd}; 
       $self->_log(msg=>$cmd_fmt, indent=>5);
    }
    
    # Remove the carriage returns from the cli command
    $args{cmd} =~ s/[\n]/ /g;

    # Init execution timer
    $t0 = gettimeofday;
  
    $connection=$self->get_connection();
    $buffer=$connection->execute_raw_command(command=>$args{cmd},timeout=>$args{timeout});
  
    $t1=gettimeofday;
    $self->{command_time}=$t1-$t0;

    # Split the buffer into array
    
    @arr=split(/\n/,$buffer);
    
    
    # Remove command and trailing blank line results array
    @rows=(@arr[0..scalar(@arr)-1]);

    return (wantarray()) ? @rows : \@rows; 

} # end sub cmd


##############################################################################
# Method:    execute
# Objective: Issue arbitrary command to host and get results..
###############################################################################
sub OLD_execute
{
    my($self, %args) = @_;
    my(
       $buffer,
       $connection,
       $cmd_fmt,
       $errmsg,
       $t0,
       $t1,
       @arr,
       @rows,
       );

    ### Clear errors.
    $self->_clear_err;

    ### Validate input arguments
    ($self,%args) = validate_mixed (
        @_,
        [ { type => OBJECT} ],
        {   cmd    => {type => SCALAR}, 
            hide    => {type => SCALAR, optional=>1, default=>0},  
            timeout => {type => SCALAR, optional=>1,
                        default=>$self->get_timeout(),
                        regex => qr/^\d+$/},
        }
    );
   
     
    ### Print the command unless the user has specified otherwise.  
    if (!$args{hide} or $self->show_cmd()){
       $cmd_fmt=$self->get_name()."(HOST)> ".$args{cmd}; 
       $self->_log(msg=>$cmd_fmt, indent=>5);
    }
    
    # Remove the carriage returns from the cli command
    $args{cmd} =~ s/[\n]/ /g;

    # Init execution timer
    $t0 = gettimeofday;
  
    $connection=$self->get_connection();

    # Issue command and get results.
    (undef,undef,$buffer,undef)   
        = Tharn::say(
            $connection,
            $args{cmd},
            -timeout=>$args{timeout},
            -eval=>sub{
                if (@_) {
                    $errmsg = join("",@_);
                    $errmsg =~ s/\n//g;
                    die ($self->_caller($errmsg),"\n")
                }
            });
    
    $t1=gettimeofday;
    $self->{command_time}=$t1-$t0;

    # Split the buffer into array
    $buffer=~s/\cI//g;
    @arr=split(/\n/,$buffer);
    
    
    
    # Remove command and trailing blank line results array
    @rows=(@arr[0..scalar(@arr)-1]);

    return (wantarray()) ? @rows : \@rows; 

} # end sub cmd


###############################################################################
# Method:    get_*()
# Objective: Get client metadata
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_connection { my ($self) = @_; $self->{connection};    }
sub get_hosttype   { my ($self) = @_; $self->{hosttype};      }
sub get_ip         { my ($self) = @_; $self->{ip}             }
sub get_mfg        { my ($self) = @_; $self->{mfg};           }
sub get_name       { my ($self) = @_; $self->{name};          }
sub get_os_version { my ($self) = @_; $self->{os_ver};        }
sub get_protocol   { my ($self) = @_; $self->{protocol};      }
sub get_password   { my ($self) = @_; $self->{password};      }
   
#sub get_sanlun_obj { my ($self) = @_; $self->{sanlunObj};     }
sub get_timeout    { my ($self) = @_; $self->{timeout};       }
sub get_username   { my ($self) = @_; $self->{username};      }
sub get_os_version { my ($self) = @_; $self->{os_version};    }

###############################################################################
# Method:    on_err
# Objective: Return errhdlr_type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub on_err {
    my ($self,$value)=@_; 
    my ($errmsg);
    
    if (defined $value) {
        
        unless ($value=~/die|rtn/){
            $errmsg="Invalid value '".$value."'";
            die _caller($errmsg);
        }
        
        $self->{on_err}=$value;
    }
    
    return $self->{on_err}; 
}


###############################################################################
# Method:    show_cmd
# Objective: display_cmd
# Details:   See POD documentation at the end of this file.
###############################################################################
sub show_cmd {
    my ($self,$value)=@_; 
    my ($errmsg);
    
    ### Initialize error buffer
    $self->_clear_err();
    
    if (defined $value) {
        
        unless ($value=~/1|0/){
            $errmsg="Invalid value '".$value."'";
            ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        }
        
        $self->{show_cmd}=$value;
    }
    
    return $self->{show_cmd};  
}


###############################################################################
#  P  R  I  V  A  T  E
###############################################################################
# Method:    _connect
# Objective: Create ssh connection to host machine
###############################################################################
sub _connect {
    my ($self,%args)=@_;
    my (
        $connObj,
        $conn_str,
        $errmsg,
        $host_of,
        $username,
        $password,
        $timeout,
        $type,
       );
    
    ### Validate input
    ($self, %args) = validate_mixed( @_,
        [ { type => OBJECT} ],
        {
            name      =>  { type => SCALAR, }, 
            hostname  =>  { type => SCALAR, }, 
            type      =>  { type => SCALAR, },
            password  =>  { type => SCALAR|UNDEF, optional => 1},
            username  =>  { type => SCALAR|UNDEF, optional => 1},
            timeout   =>  { type => SCALAR, optional=>1, default=>10,
                            regex => qr/^\d+$/ },
        }                
    );
    
 
    
    $type=$args{type};
    
    ### Create a host object
    $host_of=Utils::make_host($args{hostname});
    $self->{hosttype}=$host_of->{hosttype};
    
    #$self->_log(msg=>Dumper($host_of));
    
    $username=(defined $args{username})
        ? $args{username}
        : $host_of->{conn}{$type}{username};
        
    $password=(defined $args{password})
        ? $args{password}
        : $host_of->{conn}{$type}{password};
        
    $timeout=(defined $args{timeout})
        ? $args{timeout}
        : $host_of->{conn}{$type}{timeout};
        
    $type = $host_of->{conn}{$type}{conntype};
        
    $self->{default_ip}=$host_of->{default_ip};
        
    $self->_log(msg=>"DBG> PASSWORD IS ".$password); 
        
    ### Construct the connect string
    $conn_str=$args{hostname}."#".$type.";inherit=0;".
        "conntype=".$type.";".
        "username=".$username.";".
        "password=".$password.";".
        "timeout=".$timeout.";".
        "shell_type=".($host_of->{OS_VER}=~/^5/) ? "sh;" : "bash;";
        
        
    #$self->_log(msg=>"conn_str: ".$conn_str,indent=>3);
     
    ### Create connection object
    $connObj = Tharn::connect(
        $conn_str,   
        -eval=>sub{
            if (@_) {
                $errmsg = (@_)[0]->{messages};
                die @$errmsg;
                return $self;   
            }
        }
    );

    if (not defined $connObj){
        die $self->_caller(
            "Unable to connect to host ".
            $args{hostname}.
            "(".$args{username}."/".$args{password}.")"
            );
    }
   
    $self->{username}=$username;
    $self->{password}=$password;
   $self->{connection}=$connObj;
    
    return $connObj;

} # end sub connect;

###############################################################################
# Method:    _connect_cred
# Objective: connection credentials
###############################################################################
sub _connect_cred {
    my ($self,%args)=@_;
    my (
        $conn_str,
        $errmsg,
        $host_of,
        $username,
        $password,
        $timeout,
        $type,
       );
    
    ### Validate input
    ($self, %args) = validate_mixed( @_,
        [ { type => OBJECT} ],
        {
           # name      =>  { type => SCALAR, }, 
            hostname  =>  { type => SCALAR, }, 
            type      =>  { type => SCALAR, },
            password  =>  { type => SCALAR|UNDEF, optional => 1},
            username  =>  { type => SCALAR|UNDEF, optional => 1},
            timeout   =>  { type => SCALAR, optional=>1, default=>10,
                            regex => qr/^\d+$/ },
        }                
    );
    
    $type=$args{type};
    
    ### Create a host object
    $host_of=Utils::make_host($args{hostname});
    $self->{hosttype}=$host_of->{hosttype};
    
    #$self->_log(msg=>Dumper($host_of));
    
    $username=(defined $args{username})
        ? $args{username}
        : $host_of->{conn}{$type}{username};
        
    $password=(defined $args{password})
        ? $args{password}
        : $host_of->{conn}{$type}{password};
        
    $timeout=(defined $args{timeout})
        ? $args{timeout}
        : $host_of->{conn}{$type}{timeout};
        
    $type = $host_of->{conn}{$type}{conntype};
        
    $self->{default_ip}=$host_of->{default_ip};
 
    $self->{username}=$username;
    $self->{password}=$password;
    

} # end sub connect;


sub _OLD_connect {
    my ($self,%args)=@_;
    my (
        $connObj,
        $conn_str,
        $errmsg,
        $host_of,
        $username,
        $password,
        $timeout,
        $type,
       );
    
    ### Validate input
    ($self, %args) = validate_mixed( @_,
        [ { type => OBJECT} ],
        {
            name      =>  { type => SCALAR, }, 
            hostname  =>  { type => SCALAR, }, 
            type      =>  { type => SCALAR, },
            password  =>  { type => SCALAR|UNDEF, optional => 1},
            username  =>  { type => SCALAR|UNDEF, optional => 1},
            timeout   =>  { type => SCALAR, optional=>1, default=>10,
                            regex => qr/^\d+$/ },
        }                
    );
    
    $type=$args{type};
    
    ### Create a host object
    $host_of=Utils::make_host($args{hostname});
    $self->{hosttype}=$host_of->{hosttype};
    
    #$self->_log(msg=>Dumper($host_of));
    
    $username=(defined $args{username})
        ? $args{username}
        : $host_of->{conn}{$type}{username};
        
    $password=(defined $args{password})
        ? $args{password}
        : $host_of->{conn}{$type}{password};
        
    $timeout=(defined $args{timeout})
        ? $args{timeout}
        : $host_of->{conn}{$type}{timeout};
        
    $type = $host_of->{conn}{$type}{conntype};
        
    $self->{default_ip}=$host_of->{default_ip};
   
     
    #$self->_log(msg=>"DBG> PASSWORD IS ".$password); 
        
    ### Construct the connect string
    $conn_str=$args{hostname}."#".$type.";inherit=0;".
        "conntype=".$type.";".
        "username=".$username.";".
        "password=".$password.";".
        "timeout=".$timeout.";".
	"shell_type=sh;";
        
        
    #$self->_log(msg=>"conn_str: ".$conn_str,indent=>3);
     
    ### Create connection object
    $connObj = Tharn::connect(
        $conn_str,   
        -eval=>sub{
            if (@_) {
                $errmsg = (@_)[0]->{messages};
                die @$errmsg;
                return $self;   
            }
        }
    );

    if (not defined $connObj){
        die $self->_caller(
            "Unable to connect to host ".
            $args{hostname}.
            "(".$args{username}."/".$args{password}.")"
            );
    }
    $self->{username}=$username;
    $self->{password}=$password;
    $self->{connection}=$connObj;
    
    return $connObj;

} # end sub connect;
##############################################################################
# Method:    _execute
# Objective: Execute command and return results
###############################################################################
sub _execute {
    my ($self,%args)=@_;
    my ($cmd, $cmd_fmt,$connection, $timeout, $stderr);
    my ($output, @outputs, $status);
    my ($match, $before, $after, $sentinel);
    
    ### Validate input arguments
    ($self,%args) = validate_mixed (
        @_,
        [ { type => OBJECT} ],
        {   cmd     => {type => SCALAR}, 
            hide    => {type => SCALAR, optional=>1, default=>0},
            stderr  => {type => SCALAR, optional=>1, default=>1},
            timeout => {type => SCALAR, optional=>1,
                        default=>$self->get_timeout(),
                        regex => qr/^\d+$/},
        }
    );
   
    $cmd = $args{cmd};
    $connection = $self->get_connection();
    $sentinel = 'DoNe';
    $stderr  = (defined $args{stderr}) ? $args{stderr} : 1; 
    $timeout = (defined $args{timeout}) ? $args{timeout} : 30; 
    @outputs  = ();
    
    # Print the command unless the user has specified otherwise.  
    if (!$args{hide} or $self->show_cmd()){
       $cmd_fmt=$self->get_name()."(VMHOST)> ".$args{cmd}; 
       $self->_log(msg=>$cmd_fmt, indent=>3);
    } 
  
    # add to the command the echo of status to get that information from
    # the unix shell. Also add the sentinel string DoNe to command so that
    # we can match on that string.
    $cmd = $cmd . ";echo \"rstatus=\$\?\"";
    $output=$connection->execute_raw_command(command=>$cmd,timeout=>$timeout);
    ($status)=$output=~/rstatus=(\d+)/;
    
    #$self->_log(msg=>"STATUS=".$status);
    $status=(defined $status) ? $status : 1;
    $output=~s/rstatus.*//g;
    @outputs=split(/\n/,$output); 
    
    # execute the passed command. Use just timeout since we check earlier
    # now if it was not passed and set the default if that is the case.
 
    
    return($status, @outputs);

}
##############################################################################
# Method:    _execute
# Objective: Execute command and return results
###############################################################################
sub _OLD_execute {
    my ($self,%args)=@_;
    my ($cmd, $cmd_fmt,$connection, $timeout, $stderr);
    my ($output, @outputs, $status);
    my ($match, $before, $after, $sentinel);
    
    ### Validate input arguments
    ($self,%args) = validate_mixed (
        @_,
        [ { type => OBJECT} ],
        {   cmd     => {type => SCALAR}, 
            hide    => {type => SCALAR, optional=>1, default=>0},
            stderr  => {type => SCALAR, optional=>1, default=>1},
            timeout => {type => SCALAR, optional=>1,
                        default=>$self->get_timeout(),
                        regex => qr/^\d+$/},
        }
    );
   
    $cmd = $args{cmd};
    $connection = $self->get_connection();
    $sentinel = 'DoNe';
    $stderr  = (defined $args{stderr}) ? $args{stderr} : 1; 
    $timeout = (defined $args{timeout}) ? $args{timeout} : 30; 
    @outputs  = ();
    
    # Print the command unless the user has specified otherwise.  
    if (!$args{hide} or $self->show_cmd()){
       $cmd_fmt=$self->get_name()."(VMHOST)> ".$args{cmd}; 
       $self->_log(msg=>$cmd_fmt, indent=>3);
    } 
    
    # For unix determine whether to include or discard stderr with stdout
    if ($self->get_hosttype() =~ /filer|linux|unix|vmware/) {
        $cmd.=  ($stderr) ? " 2>&1" : " 2>/dev/null";
    }

    # add to the command the echo of status to get that information from
    # the unix shell. Also add the sentinel string DoNe to command so that
    # we can match on that string.
    $cmd = $cmd . ";echo \"rstatus=\$\?\" ;echo $sentinel";

    # libssh/libpersay do not print out normal headers letting you know
    # command being executed. Add these.
    #my $hostname = $connection->{hostp}->hostname();
    #my $conntype = $connection->conntype();
    my $try;
    # execute the passed command. Use just timeout since we check earlier
    # now if it was not passed and set the default if that is the case.
    ($try,$match, $output, $after) =  &Tharn::say($connection, $cmd, $sentinel,
                                             '-timeout', $timeout, 
                                             '-ignore', "echo $sentinel",
                                             '-comment', "Attempted '$cmd'");
    
    $self->_log(msg=>"TRY|$try|\nMATCH|$match|\nOUTPUT|$output|\nAFTER|$after|\n");
    
    
    $output=$match if (length($output));
    # strip output and remove CR/LF
    $self->_log(msg=>"OUTPUT1:  |$output|");
    chomp($output);
    $output =~ s/\r//g;

    # remove first line 
    $cmd =~ s/"rstatus=\$\?" ;echo $sentinel//g;
    $output =~ s/"rstatus=\$\?" ;//g;

    # strip sentinel and then status from output
    $output =~ s/\n$sentinel.*//;

    # strip any telnet ctrl characters that get
    # added to the output (this problem seems 
    # to only occur with certain platforms).
    $output =~ s/ \cH//g;

    # locate status.  Remember that commands
    # echo themselves so make sure you don't 
    # get the command line
    $self->_log(msg=>"OUTPUT2: |$output|");    
    ($status) = $output =~ /rstatus=(\d+)/;
$self->_log(msg=>"OUTPUT3:  |$output|");    
    # handle case where rstatus is not in the output
    # stream. One case is when NATE times out the command
    # In case of filer, do not do this since rstatus only
    # exists for clients.
    
    if (not defined $status and 
        $self->get_hosttype() =~ /unix|vmware/) {
        $status = 1;
    }
    #$self->_log(msg=>"STATUS> |".$status."|");
    $output =~ s/rstatus=\d+//;
   
    # create an array from the output
    @outputs = split(/\n/, $output);

    ## We need to determine if command is multiline 
    ## and exists in the output.  IMPORTANT: Not all
    ## commands echo back their input so don't forget to
    ## check before modifying the output.
    my @cmds = split(/\n/, $cmd);

    ## If output contains second line of cmd 
    ## then cmd was echoed to output and is 
    ## multiline
    if (defined $cmds[1] and length $cmds[1]) {
        ## Remove continuation character and parens (from group commands
        ## added recently)
        $cmds[1] =~ s/\\//g;
        $cmds[1] =~ s/\)//g;
        $cmds[1] =~ s/\(//g;
        if ($outputs[0] =~ /$cmds[1]/) {
          ## determine size of command and strip
          ## from there.
          @outputs = @outputs[@cmds-1..@outputs-1]; 

        }
    }
 
    ## Go through the output and remove any line that has the command in it
    ## We do this, because none of the function that call execute expect
    ## to have to filter out their cmd from the returned output.
    my @output_sorted;
    foreach my $line (@outputs) {
        my $match = 0;
        foreach my $cmd (@cmds) {
            if ($cmd eq $line) {
                $match = 1;
                last;
            }
        }

        if ($match == 0) {
            push (@output_sorted, $line);
        }
    }

    @outputs = @output_sorted;

    # case with dt that run w/aios flag - error status from dt is not reliable
    # add this case to catch/compensate
   
    $status = 1 if ($output =~ /Error number \d+ occurred on/i);
    
    return($status, @outputs);

}

1;
