#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/VSA/Vmhost/ESXDevMgr.pm#1 $
## @summary DevMgr ESX interface
## @author dl-vsa-qa
## @status private

# Package Name
##############
package VSA::Vmhost::ESXDevMgr;

use strict;

use Data::Dumper qw(Dumper);
use Error qw(_clear_err _set_err);
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR SCALARREF BOOLEAN OBJECT ARRAYREF HASHREF UNDEF);
use Tie::IxHash;
use VSA::UICommon qw(_caller _reformat_err _log _refresh_cache
                     _size_to_mbytes _trim);
                     
use base qw(VSA::Vmhost::DevMgr VSA::UICommon Error);

use VSA::Vmhost::ESXDev;


################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
# Subroutine Name: get_device_objs
# Objective:       return all currently configured device objects
# Details:         See POD documentation at the end of this file
################################################################################
sub get_device_objs {
    my ($self,%args) = @_;
    my (
        $cnt,
        $errmsg,
        $match,
        $value,
        @devObjs,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    #@devObjs=undef;
    
    eval {     
        ## Validate input arguments
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                cnt       => {type => SCALAR, optional=>1, default=>9999},
              
            }
        );
         # lun_path_filter  => {type => SCALAR},
        $cnt=$args{cnt};
        push @devObjs,@{$self->{devObjs}};
=comment        
        ### Apply filter if provided              
        if (keys %args) {
            foreach my $devObj (@{$self->{devObjs}}){
                
                $match=0;
                
                foreach my $key (keys %args){
                    ### Assume exact parameter match if
                    ### no regex chars are included
                    $value=($args{$key} !~/\^|\$|\.|\*/)
                        ? '^'.$args{$key}.'$' 
                        : $args{$key};
                        
                    if ($key=~/lun_path_filter/){
                        $match=1 if ($devObj->get_lun()=~/$value/); 
                    }
                
                    last if (not $match);
                }
                
                push @devObjs,$devObj if ($match);
            }
        }
        else {
            push @devObjs,@{$self->{devObjs}}
        }
=cut

        unless (@devObjs) {
            die "No Devices Found";
        }
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->{parent}->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    $cnt=($cnt>(scalar(@devObjs)-1)) ? scalar(@devObjs) : $cnt;
   
    return $devObjs[0] if ($cnt==1);
    return (wantarray()) ? @devObjs[0..$cnt-1] : [@devObjs[0..$cnt-1]];
    
} 

################################################################################
# Subroutine Name: refresh
# Objective:       refresh device list
# Details:         See POD documentation at the end of this file
################################################################################
sub refresh {
    my ($self, %args)=@_;
    my (
        $errmsg,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {     
        $self->_init(%args);
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->{parent}->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    return $self;
}

################################################################################
# Subroutine Name: _init
# Objective:       create a device object
# Details:         See POD documentation at the end of this file
################################################################################
sub _init {
    my ($self,%args) = @_;
    my (
        $cmd,
        $cnt,
        $device_of,
        $devObj,
        $filer,
        $iqn,
        $lun_id,
        $lun_path, 
        $device_name, 
        $adapter, 
        $protocol, 
        $lun_size, 
        $lun_state,
        $retry_cnt,
        $runtime_name,
        $vmkdisk_name,
        @ids,
        @mpaths,
        @outputs,
        
       );
    
    ### Initialize variables  
    $cnt=(defined $args{cnt}) ? $args{cnt} : 0;  
    @ids=(sort keys %{$self->{target_lun_hsh}});
    $iqn=$self->{target_lun_hsh}{$ids[0]}{iqn};
    
    
    while (1) {
        
        $device_of={};
        
        ### query available paths
        $cmd="esxcfg-mpath -b |grep $iqn |sort";
        @mpaths = $self->{parent}->execute(cmd=>$cmd,hide=>1);
        
        $self->{devObjs}=[];
        
        ### create a device object for each lun.
        foreach my $line (@mpaths) {
          
            ### filter out
            ($lun_id) = $line =~ /(LUN\:\d+)/;
            next if (not grep $lun_id=~/LUN\:$_/,@ids);
          
            ($runtime_name,$adapter,$lun_id,$lun_state,$protocol)=$line
                =~ /((\w+):\S+) LUN:(\d+) state:(\S+) (\S+)/;
            
            $cmd="esxcli nmp path list -p ".$runtime_name."|grep Device:";
            @outputs = $self->{parent}->execute(cmd=>$cmd,hide=>1);           
            ($vmkdisk_name)=$outputs[0]=~/Device: (\S+)/;
            
            $cmd="esxcfg-scsidevs -c -d ".$vmkdisk_name."|grep naa";
            @outputs = $self->{parent}->execute(cmd=>$cmd,hide=>1);           
            ($device_name,$lun_size)=$outputs[0]=~/\S+\s+\S+\s+(\S+)\s+(\S+)/;
           
            
            ### create the ESXDev object
            my $devObj = VSA::Vmhost::ESXDev->_new(
                vmkdisk_name => $vmkdisk_name,
                filer        => $self->{target_lun_hsh}{$lun_id}{hostname},
                lun_path     => $self->{target_lun_hsh}{$lun_id}{path},
                device_name  => $device_name,
                adapter      => $adapter,
                protocol     => $protocol,
                lun_size     => $lun_size,
                lun_state    => $lun_state,

            );
          
            $device_of->{$lun_id}=$devObj;  
        }
        
        ### Store the device objects in devmgr object in
        ### lun id order
        foreach my $id (sort keys %{$device_of}){
            push @{$self->{devObjs}}, $device_of->{$id};
        }
       
        ### Check that all device were found.  If not
        ### try again until retry cnt limit exhausted
        if ($cnt and scalar(@{$self->{devObjs}}) < $cnt) {
            if ($retry_cnt++<5){
                $self->_log(
                    msg=>"retrying sanlun to discover more devices (only found ".
                          scalar(@{$self->{devObjs}})." of $cnt)",
                    indent=>3);
                sleep(1);
                redo;
            }
            else {
                die "Not enough devices discovered (only found scalar(@{$self->{devObjs}})";
            }
        }
        
        $self->_log(msg=>scalar(@{$self->{devObjs}})." devices found",
                    indent=>3);
       
        last;
    }
    
    return $self;
                             
} 



################################################################################
# P R I V A T E   M E T H O D S
################################################################################

################################################################################
# Subroutine Name: _new()
# Objective:       Create a new devices object
# Details:         See POD documentation at the end of this file
################################################################################
sub _new {
    my ($class, %args) = @_;
    my (
        $hosttype,
        $parent,
        $self,
        @outputs,
        );
 
    ### bless instance into class
    $self = bless{}, $class;
    
    ### Validate input arguments
    ($class,%args) = validate_mixed (
        @_,
        [ { type => SCALAR} ],
        {
            #target_iqn => {type => SCALAR, optional=>1},
            target_lun_hsh  => {type => HASHREF},
            _parent     => {type => OBJECT},
            _loghdl     => {type => SCALARREF|UNDEF, optional=>1}
        }
    );
    
    ### initilize the object members
    $self->{parent}            = $args{_parent};
    $self->{loghdl}            = $args{_loghdl};
    $self->{target_lun_hsh}    = $args{target_lun_hsh},
    #$self->{target_ids}        = $args{target_ids},
    $self->{vmfs_devices}      = [];
    $self->{vmfs_device_count} = 0;
   
   
    #$self->_init();

    return $self;
}

1;
