#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/VSA/Vmhost/ESXhost.pm#1 $
## @summary ESX host 
## @author dl-vsa-qa
## @status private

# Package Name
##############
package VSA::Vmhost::ESXhost;

# Package Requirements
######################
use strict;
use Data::Dumper qw(Dumper);
use Error qw(_clear_err _set_err);
use File::Basename qw(basename);
use Module::Common qw(validate_mixed);
use NACL::APISet;
use Params::Validate qw(SCALAR SCALARREF BOOLEAN OBJECT ARRAYREF UNDEF);
use Tharn qw($Log);
use VSA::Vmhost;
use VSA::UICommon qw(_caller _reformat_err _log _refresh_cache
                     _size_to_mbytes _trim);
                     
use base qw(VSA::Vmhost VSA::UICommon Error );

my %Cache_of;
my $PsimCmd = "/usr/software/test/bin/psim";
################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
# Subroutine Name: _new()
# Objective:       Create a new ESXHost object
# Details:         See POD documentation at the end of this file
################################################################################
sub _new {
    my ($class, %args) = @_;
    my (
        $connObj,
        $self,
        );
 
    ### bless instance into class
    $self= bless{}, $class;
    
    ### Validate input arguments
    ($class,%args) = validate_mixed (
        @_,
        [ { type => SCALAR} ],
        {
            hostname  => {type => SCALAR},
            conn_type => {type => SCALAR, optional=>1, default=>"ssh",
                          callbacks=>{
                            'legal values'=>
                                sub{shift()=~/ssh/i
                         }}},
            conn_name => {type => SCALAR, optional=>1},
            loghdl    => {type => SCALARREF, optional=>1 },
            on_err    => {type => SCALAR, optional=>1, default=>"die",
                          callbacks=>{
                            'legal values'=>sub{shift()=~/die|rtn/i
                         }}},
            password  => {type => SCALAR, optional=>1,},
            protocol  => {type => SCALAR, optional=>1, default=>"iscsi",
                          callbacks=>{
                            'legal values'=>sub{shift()=~/iscsi/i
                         }}},
            shell_type => {type => SCALAR, optional=>1, default=>"sh",
                          callbacks=>{
                            'legal values'=>
                                sub{shift()=~/sh|bash/i
                         }}},
            show_cmd  => {type => SCALAR, optional=>1, default=>0},
            timeout   => {type => SCALAR, optional=>1, default=>60,
                          regex => qr/^\d+$/ },
            username  => {type => SCALAR, optional=>1,},
           
        }
    );
        
    $self->{loghdl}=$args{loghdl};
    
    ### Create connection to the host
    $self->_log( 
        msg=>"creating $args{conn_type} connection to ".$args{hostname},
        indent=>3);
       my  $type=$args{type};
    
    $self->_connect_cred(
        type=>$args{conn_type},
        hostname=>$args{hostname},
        username=>$args{username},
        password=>$args{password},
        timeout=>$args{timeout}
        );

  
	$connObj=Connectrec->new(
	    host => Tharn::host($args{hostname}),
	    template => "ssh",
	    connect => 1,
	    shell_type => $args{shell_type}
	);
    
	my $apiset = NACL::APISet->new(
        connection   => $connObj,
        category     => "Host",
        interface    => "CLI",
        set          => "Linux"
    );

    $self->{name}     = $args{hostname};
    $self->{on_err}   = $args{on_err};
    $self->{show_cmd} = $args{show_cmd};
    $self->{timeout}  = $args{timeout};
   # $self->{loghdl}   = $args{loghdl};
    $self->{protocol} = $args{protocol};
    my $host_of=Utils::make_host($args{hostname});
    $self->{hosttype}=$host_of->{hosttype};
    $self->{ip}=$host_of->{default_ip};
 
 
 
    $self->{connection}=$apiset; 
 
    $self->{vmware_version} = $self->_vmware_version();
    $self->{swiscsi_vmhbas} = [];   
    
    $self->_update_vmhbas();    
        
    return $self;
}

################################################################################
# Subroutine Name: add_iscsi_targets
# Objective:       add iscsi targets
# Details:         See POD documentation at the end of this file
################################################################################
sub add_iscsi_targets {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        $status,
        @outputs,
    );
    
    ### Clear error buffer
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                hbas => { type => ARRAYREF },
                ip   => { type => SCALAR },
                iqn  => { type => SCALAR },
            }
        );
        
        foreach my $hba ( @{$args{hbas}}) {
            
            $self->_log(msg=>"adding target $args{iqn} to $hba",indent=>3);
            
            $cmd = "/usr/sbin/vmkiscsi-tool $hba ".
                   "-S -a \"$args{ip}:3260 $args{iqn}\"";
            ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
            if ($status != 0) {  
                die "iscsi target addition failed on $hba (".
                     $args{ip}.":3260 ".$args{iqn}.")";            
            }
        }   
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    return $self;
    
}

################################################################################
# Subroutine Name: configure
# Objective:       configure host machine for fcp/iscsi/mp
# Details:         See POD documentation at the end of this file
################################################################################
sub configure {
    my ($self, %args) = @_;
    my (
        $errmsg,
    );
    
    ### initialize error buffer
    $self->_clear_err();
   
    eval {
        ### Configure software iSCSI 
        if ($self->get_protocol() =~ /iscsi/i) {
            $self->_swiscsi_enable();
            $self->_swiscsi_service_enable();
            $self->_nfsclient_service_enable();
        }
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $self;
}

################################################################
# Subroutine Name: OLD_create_vsim()
# Objective:       create vsim config for boot and CF vmdks
# Details:         See POD documentation at the end of this file
################################################################
sub OLD_create_vsim {
    my ($self,%args)=@_;;
    my (
        $cmd,
        $conn,
        $errmsg,
        @buffers,
        @outputs,
        );
   
    ### Initialize error buffer
    $self->_clear_err();
    
    ### parse arguments
    eval{
        ($self, %args) = validate_mixed(
            @_,
            [ { type => OBJECT } ],
            {
                name         =>  {type=>SCALAR},
                user         =>  {type=>SCALAR},
                quickload    =>  {type=>SCALAR,optional=>1,
                                  callbacks=>{ 'Legal Values'=>                     
                                    sub{shift()=~/^quick$|^quickl$|
                                                  ^quicks$|^quickc$/i}}},
                image        =>  {type => SCALAR},
                platform     =>  {type => SCALAR, optional => 1,
                                  default=>"x86_64",
                                  callbacks=>{ 'Legal Values'=>
                                    sub{shift()=~/^x86$|^x86_64$/i }}},
                memory       =>  {type => SCALAR, default=>2940,
                                  callbacks=>{'Is An Integer>0'=>
                                    sub{shift()>0}}},
                cpus         =>  {type => SCALAR, optional => 1,
                                  default=>2,
                                  callbacks=>{'Legal Values'=>
                                    sub{shift()=~/1|2/i }}},
                pcons        =>  {type => SCALAR, optional => 1,
                                  callbacks=>{'Legal Values'=>
                                    sub{shift()=~/0|1/i }}},
                debug        =>  {type => SCALAR, optional => 1,
                                  callbacks=>{'Legal Values'=>
                                    sub{shift()=~/0|1/i }}},
                force        =>  {type => SCALAR, optional => 1,
                                  callbacks=>{'Legal Values'=>
                                    sub{shift()=~/0|1/i }}},
                mode         =>  {type => SCALAR, 
                                  callbacks=>{'Legal Values'=>
                                    sub{shift()=~/7m|10m/i }}},
            }
        );
        
        $cmd =
            sprintf("%s make %s%s%s%s%s%s%s%s%s%s",
                $PsimCmd,
                ($args{user})        ? " -u " .$args{user}       : "", 
                ($args{quickload})   ? " -"   .$args{quickload}  : "",
                ($args{image})       ? " -ir ".$args{image}      : "",
                ($args{pcons})       ? " -"   ."pcons"           : "",
                ($args{platform})    ? " -p " .$args{platform}   : "",  
                ($args{memory})      ? " -m " .$args{memory}     : "", 
                ($args{debug})       ? " -d"  .""                : "",
                ($args{force})       ? " -f"  .""                : "",
                ($args{mode})        ? " -"   .$args{mode}       : "",
                ($args{name})        ? " "    .$args{name}       : "",                               
            );
         
        $self->_log(msg=>"creating localhost connection", indent=>3);
        $conn=Tharn::connect("localhost#staf-persay");
         
        $self->_log(
            msg=>"creating vsim ".$args{user}."_vsim_".$args{name},
            indent=>3
        );
        
        (undef,undef,@outputs,undef) =
        Tharn::say(
            $conn,
            $cmd,
            -prompt=>"simulator created",
            -timeout => 120, 
        ); 
      
        $self->_log(msg=>"@outputs");
      
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
        
    return $self;
    #return (wantarray()) ? @outputs : [@outputs];
     
}

################################################################################
# Subroutine Name: create_socats
# Objective: Create vm sockets for console and gdb access
# Details: See POD documentation at the end of this file
################################################################################
sub create_socats{
    my ($self,%args) = @_;
    my (
        $cmd,
        $datastore,
        $errmsg,
        $name,
        $redo_cnt,
        $status,
        @outputs,
        %port_of
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        
        ### parse input
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                
                name       => { type => SCALAR },
                datastore  => { type => SCALAR, optional=>1},
                console_port  => { type => SCALAR,
                                   callbacks=>{'Is An Number'=>
                                      sub{shift()=~/\d+/}}},
                gdb_port      => { type => SCALAR,
                                   callbacks=>{'Is An Number'=>
                                      sub{shift()=~/\d+/}}},                 
            }
        );
        
        ### assign input to variables
        $name          = $args{name};
        $datastore     = $args{datastore};
        

        $port_of{serial_0} = $args{console_port};
        $port_of{serial_1}  = $args{gdb_port};
 
        
        my $path=($self->get_os_version=~/^5/)
            ? "/vmfs/volumes/".$datastore
            : "/usr/bin";
		        
        ### Create vmsocat and open firewall for port access
        foreach my $type (sort keys %port_of) {
            
            $self->_log(
                msg=>"configuring ".$type." socket on port ".$port_of{$type},
                indent=>3);
            
         #   if ($self->_vmware_version()=~/ESXi/i){ 
                $cmd = "'".$path."/vmsocat'". " /tmp/".$name."_".$type." ".
                       $port_of{$type}." -telnet";
          #  } else {
           #     $cmd = "'".$path."/vmsocat'". " /tmp/".$name."_vsa_".$type." ".
            #           $port_of{$type}." -telnet";
            #}
            
            ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1); 
            if ($status){# or "@outputs" =~ /bind error/i) { 
                die "Unable to create vmsocat for $type port";             
            }
=comment            
            ### This command randomly fails due to file lock.  Retry
            ### on failure
            $self->_log(msg=>"opening ".$type." firewall",indent=>3);
            while (1) {
                
                $cmd = "/usr/sbin/esxcfg-firewall -o ".$port_of{$type}.
                       ",tcp,in,vmsocat";
                ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);
                if ($status or @outputs and "@outputs"!~/previously opened/i) {
                   if($redo_cnt++<10){
                      $self->_log(
                            msg=>"unable to open firewall on $type port - ".
                                 "retrying again (".$redo_cnt.")",
                            indent=>3);
                      sleep(1);
                      redo
                   }
                   die "Unable to open firewall on $type port";             
                }
                
                last;
            }
            
            $redo_cnt=0;
=cut
        }  

    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
        
    return $self;
    
}

################################################################################
# Subroutine Name: add_serial_port
# Objective: add_serial_ports
# Details: See POD documentation at the end of this file
################################################################################
sub OLD_add_serial_ports{
    my ($self,%args) = @_;
    my (
        $cmd,
        $ctrl_id,
        $dir,
        $dstore,
        $errmsg,
        $pipe_name,
        $redo,
        $status,
        $vmx_file,
        $vsim_disk_path,
        $name,
        @lines,
        @outputs,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                datastore  => { type => SCALAR },
                network_name   => { type => SCALAR },
                name        => { type => SCALAR },
            }
        );
         
        $name        = $args{name};
        $dstore      = $args{datastore};
        $dir         = "/vmfs/volumes/".$dstore."/".$name;
          
        ### Append entries to config file
        @lines=(
            '',
            'serial0.present = "TRUE"',
            'serial0.yieldOnMsrRead = "TRUE"',
            'serial0.fileType = "pipe"',
            'serial0.fileName = "/tmp/'.$name.'_vsa_cons"',
            'serial0.tryNoRxLoss = "TRUE"',
            '',
            'serial1.present = "TRUE"',
            'serial1.yieldOnMsrRead = "TRUE"',
            'serial1.fileType = "pipe"',
            'serial1.fileName = "/tmp/'.$name.'_vsa_gdb"',
            'serial1.tryNoRxLoss = "TRUE"',
            '',
        );
    
        foreach my $line (@lines) {
            $cmd = "echo '$line' >> $dir/$name.vmx";
            ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
            if ($status > 0) {
                die "Unable to update VSA config file ".$name.".vmx";         
            } 
        }
     
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $self;
    
    
}

################################################################################
# Subroutine Name: config_append
# Objective: Add setup info
# Details: See POD documentation at the end of this file
################################################################################
sub config_append{
    my ($self,%args) = @_;
    my (
        $cmd,
        $dir,
        $dstore,
        $errmsg,
        $lines,
        $name,
        $status,
        $vmx_file,
        @outputs,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                name          => { type => SCALAR },
                datastore => { type => SCALAR },
                lines         => { type => ARRAYREF},
            }
        );
         
        $lines          = $args{lines}; 
        $name           = $args{name};
        $dstore         = $args{datastore};
        $dir            = quotemeta("/vmfs/volumes/".$dstore."/".$name);
        
        foreach my $line (@$lines) {
            $cmd = "echo '$line' >> $dir/$name.vmx";
            ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
            if ($status > 0) {
                die "Unable to create VSA config file ".$name.".vmx";         
            } 
        }
        
=comment        
        ### UnRegister the new vm 
        #$self->_log( msg=>"registering vm '".$name."'", indent=>3);
        #$self->register(vmx_file=>$vmx_file);
        
        
        ### Power it up.
        #$self->_log( msg=>"powering on vm '".$name."'", indent=>3);
        #$self->_power_on_vm(vm=>$name);
        
        
        ### TEMP HACK - To get to SIMLOADER prompt
        $self->{console_conn} = Tharn::connect(
            $name."#console",   
            -eval=>sub{
                if (@_) {
                    $errmsg = (@_)[0]->{messages};
                    die @$errmsg;
                    return $self;   
                }
            }
        );
        
        while (1) {
            eval {
                &Tharn::say(
                        $self->{console_conn},
                        "s",
                        -prompt=>"^VSALOADER",
                        -timeout => 1, 
                );
            };
            if ($@){
                $self->_log(msg=>"missed loader prompt, retrying",indent=>3);
                redo if ($redo++<10);
            }
            last;
        };
=cut    
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $self;
    
    
}

################################################################################
# Subroutine Name: config_get
# Objective: Retrieve config file
# Details: See POD documentation at the end of this file
################################################################################
sub config_get{
    my ($self,%args) = @_;
    my (
        $cmd,
        $dir,
        $dstore,
        $errmsg,
        $filter,
        $name,
        $status,
        $vmx_file,
        @outputs,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                name          => { type => SCALAR },
                datastore => { type => SCALAR },
                filter        => { type => SCALAR, optional=>1},
            }
        );
         
        $filter         = defined ($args{filter}) ? $args{filter} : ''; 
        $name           = $args{name};
        $dstore         = $args{datastore};
        $dir            = quotemeta("/vmfs/volumes/".$dstore."/".$name);
        
 
        $cmd = "grep $filter $dir/$name.vmx";
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        if ($status > 0) {
            die "Unable to create VSA config file ".$name.".vmx";         
        } 
     
         
    };
    if ($@){
        
       # $self->_log(msg=>"DEBUG: $@");
        
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return (wantarray()) ? @outputs : [@outputs];
    
    
}
################################################################################
# Subroutine Name: config_modify
# Objective: Add setup info
# Details: See POD documentation at the end of this file
################################################################################
sub config_modify{
    my ($self,%args) = @_;
    my (
        $cmd,
        $dir,
        $dstore,
        $errmsg,
		$from,
        $name,
        $status,
		$to,
        $vmx_file,
        @outputs,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                name          => { type => SCALAR },
                datastore => { type => SCALAR },
                from        => { type => SCALAR},
				to          => { type => SCALAR},
            }
        );
     
        $from           = ($args{from});
		$to             = quotemeta($args{to});
        $name           = $args{name};
        $dstore         = $args{datastore};
        $dir            = ("/vmfs/volumes/".$dstore."/".$name);
 
	$cmd="sed -i 's/".$from."/".$to."/g' $dir/$name.vmx";
	$self->_log(msg=>"from: $from", indent=>7);
	$self->_log(msg=>"to:   $to", indent=>7);
	($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
	if ($status > 0) {
		die "Unable to modify VSA config file ".$name.".vmx";         
	} 
   
        
=comment        
        ### UnRegister the new vm 
        #$self->_log( msg=>"registering vm '".$name."'", indent=>3);
        #$self->register(vmx_file=>$vmx_file);
        
        
        ### Power it up.
        #$self->_log( msg=>"powering on vm '".$name."'", indent=>3);
        #$self->_power_on_vm(vm=>$name);
        
        
        ### TEMP HACK - To get to SIMLOADER prompt
        $self->{console_conn} = Tharn::connect(
            $name."#console",   
            -eval=>sub{
                if (@_) {
                    $errmsg = (@_)[0]->{messages};
                    die @$errmsg;
                    return $self;   
                }
            }
        );
        
        while (1) {
            eval {
                &Tharn::say(
                        $self->{console_conn},
                        "s",
                        -prompt=>"^VSALOADER",
                        -timeout => 1, 
                );
            };
            if ($@){
                $self->_log(msg=>"missed loader prompt, retrying",indent=>3);
                redo if ($redo++<10);
            }
            last;
        };
=cut    
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $self;
    
    
}

################################################################################
# Subroutine Name: is_exist
# Objective: check if vm exists
# Details: See POD documentation at the end of this file
################################################################################
sub is_exist {
    my ($self,%args)=@_;
    my (
      $cmd,
      $errmsg,
      $found,
      $status,
      @outputs,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    $found=1;
     
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                vm_name   => { type => SCALAR },
            }
        );
    
        $cmd = "ls -l /vmfs/volumes/*/".$args{vm_name}; 
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        $found=0 if ("@outputs" =~ /no such file/i);
       
    };
    if ($@) {
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $found;   
}

################################################################################
# Subroutine Name: move_vmdk
# Objective: Move vmdk file
# Details: See POD documentation at the end of this file
################################################################################
sub move_vmdk {
    my ($self,%args)=@_;
    my (
      $cmd,
      $dir,
      $disk,
      $dstore,
      $errmsg,
      $name,
      $source,
      $status,
      $target,
      $uuid,
      $vm_name,
      $vm_path,
      @outputs,
    );
     
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                #datastore => { type => SCALAR },
                disk      => { type => SCALAR },       
                source    => { type => SCALAR },
                target    => { type => SCALAR },
                vm_name   => { type => SCALAR },
                vm_path  => { type => SCALAR },
            }
        );
        
        $vm_name        = $args{vm_name};
        $vm_path        = $args{vm_path};
        #$dstore         = $args{datastore};
        #$uuid           = $self->_get_uuid($dstore);
        #$dir            = "/vmfs/volumes/".$uuid."/".$name;
        $disk           = $args{disk};
        $source         = $args{source};
        $target         = $args{target};
        
        ### Remove target disk if it already exists.
        ### This would occur if the vsa was destroyed without removing
        ### the disk from the target datastore.
        $cmd = "vmkfstools -U \"$target/$disk\""; 
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        if ($status > 0) {
            die "Unable to remove existing target \"$target/$disk\""
                if ("@outputs" !~ /cannot find/i);
        }
        
        ### Clone source disk onto target datastore
        $self->_log(msg=>"cloning disk to target datastore", indent=>7);
        $cmd="vmkfstools -i \"$source/$vm_name/$disk\" \"$target/$disk\"";  
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1,timeout=>300);  
        if ($status > 0) {
            die "Unable to move \"$source/$vm_name/$disk\" to \"$target/$disk\"";         
        }
        
        ### Remove source disk
        $self->_log(msg=>"removing disk from source datastore",indent=>7);
        $cmd = "vmkfstools -U \"$source/$vm_name/$disk\"";  
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        if ($status > 0) {
            die "Unable to remove disk from source datastore";         
        }
           
        ### Update vmx file with new target location
        $self->_log(msg=>"updating vmx file with disk target location",indent=>7);
        $cmd = "sed -i 's/$disk/".quotemeta("$target/$disk")."/' ".
                quotemeta("$vm_path/$vm_name".".vmx");       
                
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        if ($status > 0) {
            die "Unable to update vmx file with new disk location";         
        }      
 
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $self;
}

################################################################################
# Subroutine Name: update_vm_memory
# Objective: Create vm config file
# Details: See POD documentation at the end of this file
################################################################################
sub update_vm_memory{
    my ($self,%args) = @_;
    my (
        $cmd,
        $dir,
        $dstore,
        $errmsg,
        $name,
        $status,
        $uuid,
        @outputs,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                name          => { type => SCALAR },
                datastore => { type => SCALAR },
                size          => { type => SCALAR },
            }
        );
         
        $name           = $args{name};
        $dstore         = $args{datastore};
        $uuid           = $self->_get_uuid($dstore);
        $dir            = "/vmfs/volumes/".$uuid."/".$name;
            
        
        $cmd = "sed -i 's/memsize = .*/memsize = ".$args{size}.
               "/' $dir/$name.vmx";
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        if ($status > 0) {
            die "Unable to update vm memory in ".$name.".vmx";         
        }
        
            
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $self;
      
}

################################################################################
# Subroutine Name: create_vm
# Objective: Create vm config file
# Details: See POD documentation at the end of this file
################################################################################
sub OLD_create_vm{
    my ($self,%args) = @_;
    my (
        $cmd,
        $ctrl_id,
        $dir,
        $dstore,
        $errmsg,
        $memory,
        $name,
        $network_name,
        $redo,
        $status,
        $target_id,
        $uuid,
        $vmdk_cnt,
        $vmx_file,
        $vsim_disk_path,
        @general_cfgs,
        @outputs,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                memory        => { type => SCALAR },
                name          => { type => SCALAR },
                datastore => { type => SCALAR },
                network_name  => { type => SCALAR },
            }
        );
         
        ($memory)       = $args{memory} =~ /(\d+)/;
        $name           = $args{name};
        $dstore         = $args{datastore};
        $dir            = "/vmfs/volumes/".$dstore."/".$name;
        $network_name   = $args{network_name};
        $uuid           = $self->_get_uuid($dstore);
        
        $self->_log(
            msg=>"retrieving path to vsim vmdks (boot/CFcard)",
            indent=>3);
        $vsim_disk_path = $self->_get_vsim_disk_path($dstore,$name);
        
        ### Append entries to config file
        @general_cfgs=(
            '#!/usr/bin/vmware',
            '.encoding = "UTF-8"',
            'config.version = "8"',
            'virtualHW.version = "7"',
            '',
            'cleanShutdown = "FALSE"',
            'disk.enableUUID="TRUE"',
            'evcCompatibilityMode = "FALSE"',
            'powerType.powerOff = "soft"',
            'powerType.powerOn = "default"',
            'powerType.reset = "soft"',
            'powerType.suspend = "hard"',
            'replay.supported = "FALSE"',
            'tools.remindInstall = "TRUE"',
            'tools.upgrade.policy = "useGlobal"',
            'unity.customColor = "|23C0C0C0"',
            'virtualHW.productCompatibility = "hosted"',
            'vmci0.present = "TRUE"',
            '',
            'numvcpus = "2"',
            'memsize = "'.$memory.'"',
            'deploymentPlatorm = "windows"',
            'guestOSAltName = "FreeBSD (64-bit)"',
            'guestOS = "freebsd-64"',
            'displayName = "'.$name.'"',
            'extendedConfigFile = "'.$name.'.vmxf"',
            'nvram = "'.$name.'.nvram"',
            '',
            'pciBridge0.present = "TRUE"',
            'pciBridge4.present = "TRUE"',
            'pciBridge4.virtualDev = "pcieRootPort"',
            'pciBridge4.functions = "8"',
            'pciBridge5.present = "TRUE"',
            'pciBridge5.virtualDev = "pcieRootPort"',
            'pciBridge5.functions = "8"',
            'pciBridge6.present = "TRUE"',
            'pciBridge6.virtualDev = "pcieRootPort"',
            'pciBridge6.functions = "8"',
            'pciBridge7.present = "TRUE"',
            'pciBridge7.virtualDev = "pcieRootPort"',
            'pciBridge7.functions = "8"',
            '',
            'ethernet0.present = "TRUE"',
            'ethernet0.virtualDev = "e1000"',
            'ethernet0.networkName = "'.$network_name.'"',
            'ethernet0.addressType = "generated"',
            '',
            'ethernet1.present = "TRUE"',
            'ethernet1.virtualDev = "e1000"',
            'ethernet1.networkName = "'.$network_name.'"',
            'ethernet1.addressType = "generated"',
            '',
            'ethernet2.present = "TRUE"',
            'ethernet2.virtualDev = "e1000"',
            'ethernet2.networkName = "'.$network_name.'"',
            'ethernet2.addressType = "generated"',
            '',
            'ethernet3.present = "TRUE"',
            'ethernet3.virtualDev = "e1000"',
            'ethernet3.networkName = "'.$network_name.'"',
            'ethernet3.addressType = "generated"',
            '',
            'serial0.present = "TRUE"',
            'serial0.yieldOnMsrRead = "TRUE"',
            'serial0.fileType = "pipe"',
            'serial0.fileName = "/tmp/'.$name.'_serial_0"',
            'serial0.tryNoRxLoss = "TRUE"',
            '',
            'serial1.present = "TRUE"',
            'serial1.yieldOnMsrRead = "TRUE"',
            'serial1.fileType = "pipe"',
            'serial1.fileName = "/tmp/'.$name.'_serial_1"',
            'serial1.tryNoRxLoss = "TRUE"',
            '',
            'floppy0.present = "TRUE"',
            'floppy0.startConnected = "FALSE"',
            'floppy0.clientDevice = "TRUE"',
            'floppy0.fileName = "/dev/fd0"',
            '',
            'ide0:0.present = "TRUE"',
            'ide0:0.fileName = "'.$vsim_disk_path.'/DataONTAP.vmdk"',
            'ide0:0.mode = "independent-persistent"',
            'ide0:0.redo = ""',
            '',
            'ide0:1.present = "TRUE"',
            'ide0:1.fileName = "'.$vsim_disk_path.'/DataONTAP-cf.vmdk"',
            'ide0:1.mode = "independent-persistent"',
            'ide0:1.redo = ""',
            '',
            'ide1:0.present = "TRUE"',
            'ide1:0.clientDevice = "TRUE"',
            'ide1:0.deviceType = "cdrom-raw"',
            'ide1:0.startConnected = "FALSE"',
            '',     
            'scsi0.present = "TRUE"',
            'scsi0.sharedBus = "none"',
            'scsi0.virtualDev = "lsisas1068"',
            'scsi0.pciSlotNumber = "160"',
            '',     
            'scsi1.present = "TRUE"',
            'scsi1.sharedBus = "none"',
            'scsi1.virtualDev = "lsisas1068"',
            'scsi1.pciSlotNumber = "192"',
            '',     
            'scsi2.present = "TRUE"',
            'scsi2.sharedBus = "none"',
            'scsi2.virtualDev = "lsisas1068"',
            'scsi2.pciSlotNumber = "224"',
            '',     
            'scsi3.present = "TRUE"',
            'scsi3.sharedBus = "none"',
            'scsi3.virtualDev = "lsisas1068"',
            'scsi3.pciSlotNumber = "256"',
        );
   
      
        $self->_log(
            msg=>"creating VSA config file ".$name.".vmx",
            indent=>3);
        
        $vmx_file=$self->_create_vmx_file($dstore,$dir,$name);
                 
    
        foreach my $line (@general_cfgs) {
            $cmd = "echo '$line' >> $dir/$name.vmx";
            ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
            if ($status > 0) {
                die "Unable to create VSA config file ".$name.".vmx";         
            } 
        }
        
        $target_id=0;
        $vmdk_cnt=0;
        foreach my $vmdk ($self->get_vmdks(vm_name=>$name)){
            
            ### Determine the scsi ctrl to use
            $ctrl_id=$vmdk_cnt++%4;
            
            ### Skip over SCSIn:7 which is reserved for the iscsi
            ### adapter.
            $target_id++ if ($target_id==7);
            
            ### Generate config entries for each device
            my @disk_headers=(
                'scsi'.$ctrl_id.':'.$target_id.'.present = "TRUE"',
                'scsi'.$ctrl_id.':'.$target_id.'.fileName = "'.$vmdk.'"',
                'scsi'.$ctrl_id.':'.$target_id.'.mode = "independent-persistent"',
                'scsi'.$ctrl_id.':'.$target_id.'.deviceType = "scsi-hardDisk"',
                'scsi'.$ctrl_id.':'.$target_id.'.redo = ""',
                'sched.scsi'.$ctrl_id.':'.$target_id.'.shares = "normal"',
                '',
            );
            
            ### Append entries to config file
            foreach my $line (@disk_headers){
                $cmd = "echo '$line' >> $dir/$name.vmx";
                ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
                if ($status > 0) {
                    die "Unable to create VSA config file ".$name.".vmx";         
                }
            }

            ### Increment the target id on last ctrl id
            $target_id++ if ($ctrl_id==3);
            
        }  

        ### Register the new vm 
        $self->_log( msg=>"registering vm '".$name."'", indent=>3);
        $self->register(vmx_file=>$vmx_file);
        
        
        ### Power it up.
        $self->_log( msg=>"powering on vm '".$name."'", indent=>3);
        $self->_power_on_vm(vm=>$name);
        
        
        ### TEMP HACK - To get to SIMLOADER prompt
        $self->{console_conn} = Tharn::connect(
            $name."#console",   
            -eval=>sub{
                if (@_) {
                    $errmsg = (@_)[0]->{messages};
                    die @$errmsg;
                    return $self;   
                }
            }
        );
        
        while (1) {
            eval {
                &Tharn::say(
                        $self->{console_conn},
                        "s",
                        -prompt=>"^SIMLOADER",
                        -timeout => 1, 
                );
            };
            if ($@){
                $self->_log(msg=>"missed loader prompt, retrying",indent=>3);
                redo if ($redo++<10);
            }
            last;
        };
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
    
    return $self;
    
    
}
################################################################################
# Subroutine Name: delete_socats
# Objective: Create vm sockets for console and gdb access
# Details: See POD documentation at the end of this file
################################################################################
sub delete_socats{
    my ($self,%args) = @_;
    my (
        $cmd,
        $errmsg,
        $name,
        $status,
        @outputs,
	@outputs2,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        
        ### parse input
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {    
                name       => { type => SCALAR },              
            }
        );
        
        ### assign input to variables
        $name          = $args{name};
        
        ### Retrieve socat pids for vm
	
	if ($self->_vmware_version()=~/ESXi/i){
	    $cmd = "ps -c | grep \"/tmp/".$name."_\" |grep -v grep| ".
	           "awk '{print \$2 \$5}'";
	} else {
	    $cmd = "ps -elf | grep -v grep | grep \"/tmp/".$name."_\" | ".
	           "awk '{print \$4\$16}'";
	}
	
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        unless (@outputs) { 
            $self->_log(msg=>"There are no sockets to delete", indent=>3);
            return $self;
        }
     
	
        ### Delete socat pid
        foreach my $line (@outputs) {
	   
            my ($pid,$type)=$line=~/^(\d+).*\/(\S+)/i;
            next if (not defined $pid);
            
            $cmd = "kill -9 ".$pid;
            ($status, @outputs2) = $self->_execute(cmd=>$cmd,hide=>1);
            if ($status) { 
                die "@outputs2";             
            }   
        }  

    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
        
    return $self;
    
}

################################################################################
# Subroutine Name: delete_vm
# Objective: Delete vm config file
# Details: See POD documentation at the end of this file
################################################################################
sub delete_vm {
    my ($self,%args) = @_;
    my (
        $cmd,
        $dir,
        $errmsg,
        $idx,
        $name,
        $network_name,
        $dstore,
        $status,
        $uuid,
        $vmx_file,
        $vsim_disk_path,
        @general_cfgs,
        @outputs,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                name          => { type => SCALAR },
                datastore     => { type => SCALAR|UNDEF, optional=>1},
            }
        );
        
        $name           = $args{name};
        $dstore         = (defined $args{datastore})
                            ? $args{datastore}
                            : $self->get_local_datastore();
        $uuid           = $self->_get_uuid($dstore);
        $dir            = "/vmfs/volumes/".$uuid."/".$name;
        
        #$vsim_disk_path = $self->_get_vsim_disk_path($dstore,$name);
        
        $vmx_file=$self->_get_vmconfig_file(vm_name=>$name);
        
        $self->_log(msg=>"powering off vm '".$name."'", indent=>3); 
        $self->_power_off_vm(vm=>$name);
    
        $self->_log(msg=>"unregistering vm '".$name."'", indent=>3);
        $self->unregister(vmx_file=>$vmx_file) if (defined $vmx_file);
    
        $self->_log(msg=>"deleting vm".$name ,indent=>3);
        ($status, @outputs) = $self->_execute(cmd=>"rm -rf $dir",hide=>1);  
        if ($status > 0) {
            die "@outputs";   
        }      
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    return $self;
    
    
}

###############################################################################
# Method:    discover()
# Objective: Discover new devices on host
# Details:   See POD documentation at the end of this file.
###############################################################################
sub discover {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        $status,
        @outputs,
    );
    
    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                hbas       => { type => ARRAYREF },                                     
            }
        );
        
        foreach my $hba ( @{$args{hbas}}) {
            $self->_log(msg=>"rescan $hba for new devices",indent=>3);
            $cmd = "/usr/sbin/esxcfg-rescan " . $hba;
            ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
            if ($status > 0) {
                die "rescan of hbas failed ($cmd)";            
            } 
        } 
        
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    return $self;
    
}

###############################################################################
# Method:    get_iqn()
# Objective: Retrieve iscsi qualified name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_iqn {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        $iqn,
        $status,
        @hbas,
        @outputs,
       );

    ### Clear error buffer
    $self->_clear_err();
    
    eval {
        
        ### Retrieve swiscsi hba
        @hbas=$self->get_swiscsi_vmhbas();
        
        ### Query for iqn
        $cmd = "vmkiscsi-tool -I " . $hbas[0];
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        if ($status > 0) {
            die "@outputs ($cmd)";            
        } 
        
        ($iqn)=$outputs[0]=~/Name: (\S+)/;
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    return $iqn;
    
}
###############################################################################
# Method:    get_local_datastore()
# Objective: Retrieve local datastore name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub  get_local_datastore {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        $name,
        $status,
        @outputs,
    );
    
    eval {
        
        ### Temp sol'n to get the full name of the local
        ### datastore full name that include (n) identifier
        ### to make it unique
        $cmd="esxcfg-scsidevs -m|grep Storage1|".
             "awk '{printf(\"%s %s\\n\",\$5,\$6);}'";
    
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        if ($status > 0) {
            die "@outputs ($cmd)";            
        } 
        
        ### Trim trailing spaces
        $name=$outputs[0];
        $name=~s/\s+$//i;
        
        #$self->_log(msg=>"DBG: |".$name."|");
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return ""; 
    }
    
    return $name;
}

###############################################################################
# Method:    get_swiscsi_vmhbas 
# Objective: Retrieve sw iscsi vmhbas
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_swiscsi_vmhbas   {
    my ($self)=@_;
    return (wantarray()) ? @{$self->{swiscsi_vmhbas}} : $self->{swiscsi_vmhbas};
}

###############################################################################
# Method:    get_vmdks
# Objective: Retrieve list of vmdk files
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_vmdks {
    my ($self, %args) = @_;
    my (
        $cmd,
        $errmsg,
        $lvm_type,
        $status,
        $vm_cfg_file,
        $vm_name,
        @outputs,
        @vmdk_files,
    );
   
    ### Initialize error buffer
    $self->_clear_err();

    eval {
        ## Validate input arguments
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                lvm_type => {type => SCALAR, optional=>1, default=>"vmfs",
                              callbacks=>{
                                'legal values'=>sub{shift()=~/vmfs/i
                             }}},
                vm_name  => {type => SCALAR},
            }
        );
        
        $lvm_type = $args{lvm_type};
        $vm_name  = $args{vm_name};
    
        ### Retrieve vm config file 
    #    $vm_cfg_file= $self->_get_vmconfig_file(vm_name => $vm_name); 
        $vm_cfg_file="";
        ### If a config file exists use that to determine the list of vmfdks,
        ### otherwise query the ESX host operating system for the names.
        if ($vm_cfg_file) {
            
            ### Retrieve vm file content
            $cmd = "cat ".$vm_cfg_file;
            ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
            if ($status>0) {  
                die "Unable to read contents of vm config file: $vm_cfg_file\n";            
            }
        
            ### Retrieve all the disk files
            @vmdk_files = grep { /vmfs\/volumes/i and 
                                /.*?fileName\s*\=\s*\"(.+?)\"/i } @outputs;
            
        
            ### Remove vmx syntax around the file name on each vmdk file
            @vmdk_files = map { 
                if ($_ =~ s/.*?fileName\s*\=\s*\"(.+?)\"/$1/i) {
                    $_;
                } 
            } @vmdk_files;
        }
        else {
            my $vm_name_ds=$vm_name;
            $vm_name_ds=~ s/-|_//g;
            
            
            ### Retrieve vm file content
            $cmd = "ls -1R /vmfs/volumes/".$vm_name_ds."_*/*.vmdk|".
                   "grep -v \"flat\"";
            ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
            if ($status>0) {  
                return (wantarray()) ? () : [] if ("@outputs"=~/no such file/i);     
                die "Unable to query host for vmdk files\n";            
            }
            else {
                push @vmdk_files, @outputs;
            }
        }
    
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return (wantarray()) ? () : []; 
    }

    
    
    # return vmdk file list
    return (wantarray()) ? @vmdk_files : \@vmdk_files;
}

###############################################################################
# Method:    get_vmware_version
# Objective: Retrieve vmware version
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_vmware_version   { my ($self)=@_;  $self->{vmware_version};}


################################################################################
# Subroutine Name: nfs_mount
# Objective:       NFS mount volume
# Details:         See POD documentation at the end of this file
################################################################################
sub nfs_mount {
    my ($self, %args) = @_;
    my (
        $cmd,
        $errmsg,
        $host,
        $name,
        $path,
        $status,
        @outputs,
        );

    ### Initialize error buffer
    $self->_clear_err();

    eval {
        ### Validate input arguments
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                host  => {type => SCALAR},
                path  => {type => SCALAR},
                name  => {type => SCALAR},
            }
        );
        
        $host = $args{host};
        $name = $args{name};
        $path = $args{path};
        
        ### Create nfs mount
        $cmd = "/usr/sbin/esxcfg-nas -a -o $host -s $path $name";
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);
        if ($status > 0 and "@outputs" !~ /already exported/) {
            die "Could not mount NFS host $host\:$path on $name";
        }
        
        ### Display message
        $self->_log(
            msg =>"mounted NFS volume $host\:$path on '$name'",
            indent=>3
        );
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }

    return $self;
}

###############################################################################
# Method:    register
# Objective: register vm instance with host
# Details:   See POD documentation at the end of this file.
###############################################################################
sub register  {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        $status,
        $vmx_file,
        @outputs,
    );
    
    $self->_clear_err();
    
    eval {
       ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                vmx_file   => { type => SCALAR },
            }
        );
        if ($self->get_os_version()=~/^5/) {
            $cmd="vim-cmd solo/registervm ".$args{vmx_file};
        } else {
            $cmd = "vmware-cmd -s register ".$args{vmx_file};
        }
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        if ($status > 0) {
           die "@outputs";   
        }
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
     
    return $self;
}

###############################################################################
# Method:    unregister
# Objective: unregister vm instance with host
# Details:   See POD documentation at the end of this file.
###############################################################################
sub unregister  {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        $status,
        $vmx_file,
        @outputs,
    );
    
    $self->_clear_err();
    
    eval {
       ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                vmx_file   => { type => SCALAR },
            }
        );
        
        if ($self->get_os_version()=~/^5/) {
            $cmd="vim-cmd vmsvc/unregister ".$args{vmx_file};
        } else {
            $cmd = "vmware-cmd -s unregister ".$args{vmx_file};
        }
        
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        if ($status > 0) {
           die "@outputs";   
        }
    };
    if ($@){
        $errmsg=$self->_reformat_err($@);
        ($self->on_err()=~/die/)
            ? die ($self->_caller($errmsg),"\n")
            : $self->_set_err($errmsg);
        return $self; 
    }
     
    return $self;

}

################################################################################
# P R I V A T E   M E T H O D S
################################################################################

################################################################################
# Subroutine Name: _create_vmx_file
# Objective: Create vmx file in datastore
################################################################################
sub _OLD_create_vmx_file {
    my ($self,$dstore,$dir,$name)=@_;
    my (
        $cmd,
        $file_full_name,
        $status,
        @outputs,
    );
    
    $file_full_name=$dir."/".$name.".vmx";
    
    $cmd = "mkdir -m 0777 ".$dir;
    ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
    if ($status > 0) {
        unless ("@outputs"=~/file exists/i) {
            die "Unable to create VSA $name in datastore $dstore $!";
        }
    }    
        
    
    $cmd = "touch ".$file_full_name;
    ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
    if ($status > 0) {
        die "@outputs";   
    }
    
    return $file_full_name;

}

################################################################################
# Subroutine Name: _get_uuid()
# Objective:       Given the datastore name, convert to the UUID label
################################################################################
sub _get_uuid {
    my ($self, $datastore) = @_;
    my (
        $cmd,
        $datastore_orig,
        $retry_cnt,
        $status,
        $uuid,
        @datastores,
        @outputs,
    );
    
    ### Initialize variables
    $datastore_orig=$datastore;
    $retry_cnt=1;
    
    while (1) {
        $cmd = "/usr/sbin/vmkfstools -P -h ".
               "\"/vmfs/volumes/$datastore\"|grep UUID";
        ($status, @outputs) = $self->_execute(cmd=>$cmd, hide=>1);
        if ($status or "@outputs" =~ /Could not open/ and $retry_cnt) {
            $cmd = "ls /vmfs/volumes | grep \"".$datastore_orig."\"";
            ($status, @datastores) = $self->_execute(cmd=>$cmd, hide=>1);
            if ($status or not @datastores) {
                die "Could not obtain the UUID for datastore $datastore_orig";
            }
            
            $datastore=$datastores[0];
            $datastore=~s#(\(|\)|\s)#\\$1#g;
            
            $retry_cnt --;
            redo;
        }
        elsif ($status==0){
            ($uuid)=$outputs[0]=~/UUID: (\S+)/;
            $uuid=~s/-0000-000000000000//g;
            last;
        }
        else {
             die "Could not obtain the UUID for datastore $datastore_orig";
        }
    }
    
    
    return $uuid;
}
#
################################################################################
# Subroutine Name: get_vmconfig_file()
# Objective:       Returns the filepath to the vm's config file
# Details:         See POD documentation at the end of this file
################################################################################
sub _get_vmconfig_file {
    my ($self, %args) = @_;
    my (
        $vm_name,
        $vm_cfg_file,
        @vm_cfg_files,
    );
    
    ## Validate input arguments
    ($self,%args) = validate_mixed (
        @_,
        [ { type => OBJECT} ],
        {
            vm_name  => {type => SCALAR},
        }
    );
  
    $vm_name=$args{vm_name};
    @vm_cfg_files = sort($self->_get_vmconfig_files());
    ($vm_cfg_file) = grep (/\/$vm_name\.vmx/, @vm_cfg_files);
    #unless ($vm_cfg_file){
        #die "A vm config file for '$vm_name' does not exist.\n"                          
    #}
 
    return $vm_cfg_file;
}

################################################################################
# Subroutine Name: _get_vmconfig_files()
# Objective:       Returns an array of Guest OS .vmx configuration files
# Details:         See POD documentation at the end of this file
################################################################################
sub _get_vmconfig_files {
    my ($self) = @_;
    my (
        $cmd,
        $retry_cnt,
        $retry_max,
        $status,
        @outputs,
        );

    $retry_max=3;

    ### Retrieve list all the vm's on this esx host.
    while (1) {
        $cmd = "/usr/bin/vmware-cmd -l";
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);  
        if ("@outputs"=~/Could not connect to server/i) {
            if ($retry_cnt++ < $retry_max){
                sleep(20);
                next;
            }
            
            die "Could not retrieve vm config files ($cmd) @outputs";
        }
        last;
    }

    ### Remove any lines containing interspersed msg corruption
    ### NOTE: Sometimes the output of vmware-cmd -l gets corrupted
    ### by this string: "pid file /var/run/vmkiscsid.pid already locked"
    foreach my $line (@outputs) {
        $line =~ s/\/var\/run\/vmkiscsid.pid already locked//g;
    }        

    return (wantarray()) ? @outputs : \@outputs;    
}

################################################################################
# Subroutine Name: _get_vsim_disk_path()
# Objective:       Retrieve vsim disk path to DataONTAP and disks.
################################################################################
sub _get_vsim_disk_path {
    my ($self, $datastore, $name) = @_;
    my (
        $cmd,
        $disk_path,
        $status,
        @outputs,
    );
    
    $cmd = "ls -d /vmfs/volumes/$datastore/*_".$name;
    ($status, @outputs) = $self->_execute(cmd=>$cmd, hide=>1);
    if ($status > 0) {
        die "@outputs"; 
    } 
    $disk_path=$outputs[0];
    
    return $disk_path;
}

################################################################################
# Subroutine Name: _nfsclient_service_enable()
# Objective:       Enable nfsClient service
################################################################################
sub _nfsclient_service_enable {
    my ($self, %args) = @_;
    my (
        $cmd,
        $status,
        @outputs, 
    );

    ### Check if the nfsClient service is enabled.
    $self->_log(msg=>"verifing nfsClient service is enabled",indent=>3);
    $cmd="/usr/sbin/esxcfg-firewall --enableService nfsClient";
    ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);
    if ($status > 0) {  
        die "Could not verify nfsClient service ($cmd)";       
    }

    return $self;
}

################################################################################
# Subroutine Name: _power_on_vms()
# Objective:       Will turn on specified vms on this ESX 
#                  host.
# Details:         See POD documentation at the end of this file
################################################################################
sub _power_on_vm {
    my ($self, %args) = @_;
    my (
        $cmd,
        $status,
        $vm,
        $vmx_file,
        $redo_cnt,
        @outputs,
        @vms,    
    );

  
    ($self,%args) = validate_mixed (
        @_,
        [ { type => OBJECT} ],
        {
            vm      =>  { type => SCALAR },
        }
    );
    
    $vm   = $args{vm};
         
    unless ($vmx_file = $self->_get_vmconfig_file(vm_name => $vm)){
        $self->_log(msg => "unable to find vm $vm to power on", indent=>3);
        return 0;
    }
    
=comment      
    # get the vm's power status
    $cmd = "/usr/bin/vmware-cmd $vmx_file getstate";
    ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);
    if ($status > 0) {
        die "@outputs";    
    }
    
    # if the vm is already on, skip
    if ("@outputs" =~ /on/i) {
        $self->_log(msg => "$vm is already powered on", indent=>3);
        next;
    }
=cut

    # otherwise start it up
    while (1) {
        $cmd = "/usr/bin/vmware-cmd $vmx_file start";
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);
        if ($status > 0 or "@outputs" =~ /fault/i and "@outputs" !~/powered on/i) {
            redo if ($redo_cnt++<5);
            die "@outputs";    
        }
        last;
    }
    $self->_log(msg => "powered on", indent=>5);
    
   return 1;

}

################################################################################
# Subroutine Name: _power_off_vm()
# Objective:       Will turn off specified vms on this ESX 
#                  host.
# Details:         See POD documentation at the end of this file
################################################################################
sub _power_off_vm {
    my ($self, %args) = @_;
    my (
        $cmd,
        $status,
        $vm,
        $vmx_file,
        $wait,
        @outputs,
        @vms,    
    );
  
    ($self,%args) = validate_mixed (
        @_,
        [ { type => OBJECT} ],
        {
            vm      => { type => SCALAR},
        }
    );
    
    $vm   = $args{vm};
    $wait = $args{wait};
         
    unless ($vmx_file = $self->_get_vmconfig_file(vm_name => $vm)){
        $self->_log(msg => "unable to find vm $vm to power off", indent=>3);
        return 0;
    }
    
=comment  
    # get the vm's power status
    $cmd = "/usr/bin/vmware-cmd $vmx_file getstate";
    ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);
    if ($status > 0) {
        die "@outputs";    
    }
    
    # if the vm is already on, skip
    if ("@outputs" =~ /off/i) {
        $self->_log(msg => "$vm is already powered off", indent=>3);
    }
    else {
        # otherwise start it up
=cut        
        $cmd = "/usr/bin/vmware-cmd $vmx_file stop hard";
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);
        if ($status > 0 or "@outputs" =~ /fault/i and
            "@outputs" !~/powered off/i) {
            die "@outputs";    
        }
        
        $self->_log(msg => "powered off", indent=>5);
 #   }
        
   return 1;
}

################################################################################
# Subroutine Name: power_recycle()
# Objective:       Will turn on specified vms on this ESX 
#                  host.
# Details:         See POD documentation at the end of this file
################################################################################
sub power_recycle {
    my ($self, %args) = @_;
    my (
        $cmd,
        $status,
        $vm,
        $vmx_file,
        @outputs,
        @vms,    
    );

  
    ($self,%args) = validate_mixed (
        @_,
        [ { type => OBJECT} ],
        {
            vm      =>  { type => SCALAR },
        }
    );
    
    $vm   = $args{vm};
         
    unless ($vmx_file = $self->_get_vmconfig_file(vm_name => $vm)){
        $self->_log(msg => "unable to find vm $vm to power on", indent=>3);
        return 0;
    }
    

    # otherwise start it up
    $cmd = "/usr/bin/vmware-cmd $vmx_file reset hard";
    ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);
    if ($status > 0 or "@outputs" =~ /fault/i) {
        die "@outputs";    
    }
    
    $self->_log(msg => "power reset $vm", indent=>3);
    
   return 1;

}


################################################################################
# Subroutine Name: _swiscsi_enable()
# Objective:       Enable software iscsi
################################################################################
sub _swiscsi_enable {
    my ($self, %args) = @_;
    my (
        $cmd,
        $status,
        @outputs, 
        @targets
    );

    ### Check if the sw iscsi initiator is enabled
    $self->_log(msg=>"verifing swiscsi is enabled",indent=>3);
    $cmd = "/usr/sbin/esxcfg-swiscsi --query";
    ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);
    if ($status > 0) {  
        die "Could not query software iscsi ($cmd)";       
    }           
     
    ### Enable sw iscsi if its disabled     
    if ( "@outputs"=~/not enabled/i) {
        
        $self->_log(msg=>"enabling swiscsi",indent=>3);
        $cmd = "/usr/sbin/esxcfg-swiscsi --enable";
        ($status, @outputs)= $self->_execute(cmd=>$cmd,hide=>1);
        if ($status > 0) {  
            die "Could not enable software iscsi ($cmd)";       
        }     
        
        # we need to reboot the esx machine after
        # enabling the iscsi initiator
       
        #$self->reboot();
    }
}

################################################################################
# Subroutine Name: _swiscsi_service_enable()
# Objective:       Enable software iscsi service
################################################################################
sub _swiscsi_service_enable {
    my ($self, %args) = @_;
    my (
        $cmd,
        $status,
        @outputs, 
        @targets
    );

    ### Check if the swISCSIClient service is enabled.
    $self->_log(msg=>"verifing swISCSIClient service is enabled",indent=>3);
    $cmd = "/usr/sbin/esxcfg-firewall --query swISCSIClient";
    ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);
    if ($status > 0) {  
        die "Could not verify swISCSIClient service ($cmd)";       
    }
        
    ### Enable swISCSIClient if the service is blocked
    if ("@outputs" =~ /Service swISCSIClient is blocked/i ) {
        $cmd = "/usr/sbin/esxcfg-firewall --enableService swISCSIClient";
        ($status, @outputs) = $self->_execute(cmd=>$cmd,hide=>1);
        if ($status > 0) {  
            die "Could not enable the swISCSIClient service";           
        }
    }

    return $self;
}


################################################################################
# Subroutine Name: _update_vmhbas
# Objective:       Update object metadata with adapter info
################################################################################
sub _update_vmhbas {
    my ($self) = @_;
    my (
        $cmd,
        $status,
        @swiscsi_vmhbas,
        @outputs,
        );
    
    # search for the fcp and iscsi adapters
    $cmd = "/usr/sbin/esxcfg-scsidevs -a";
    ($status, @outputs) = $self->_execute(cmd=>$cmd, hide=>1);
    if ($status > 0) {
        die "Command ($cmd) failed. Status = ($status)";
    }

    foreach my $line (@outputs) {
        if ($line =~ /Software iSCSI|iscsi_vmk/i and $line =~ /(vmhba\d+)\s*/) {
            push @{$self->{swiscsi_vmhbas}}, $1;
        }
    }
    
    return $self;
}


###############################################################################
# Method:    _vmware_version
# Objective: Queries the Sanlun version and build number on this client.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _vmware_version {
    my ($self) = @_;
    my (
        $cmd,
        $status,
        $version,
        @outputs,
    );
    
    ### Initialize error buffer    
    $self->_clear_err();

    $cmd = "vmware -v";   
    eval {
        ($status, @outputs) = $self->_execute(cmd=>$cmd, hide=>1);
        if ($status > 0) {
            die $outputs[0];
        }
    };
 
    return $outputs[0];

}

1;