# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/VSA/dvadmin.pm#6 $

## @summary     dvadmin support library
## @author      lippe@netapp.com
## @status      TBD
## @pod here

package VSA::dvadmin;

use strict;
use Error;
use Params::Validate qw(:all);
use Tharn qw($Log);
use Error qw(_clear_err _set_err);
use Data::Dumper;

use Connectrec qw();

use vars qw(@ISA);

@ISA = qw(Error);


###############################################################################
# Method:    new
# Objective: Constructor
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my $class = shift;
    my (%args) = @_;
    my (@buf,
	$ch,  
	$self);
    
    $self = bless {}, $class;
    
    $self->_clear_err();

    # Validate input arguments
    (%args) = validate (
			@_,
			{
			    connection => {type => SCALAR, default => "cmd"},
			    mgmt_vm  => {type => SCALAR},
			    esx_server => {type => SCALAR},
			    esx_host => {},
			    esx_user => {type => SCALAR},
			    esx_pass => {type => SCALAR},
			    username => {type => SCALAR},
			    password => {type => SCALAR},
			    prepackaged => {type => BOOLEAN, default => undef},
			    save => {type => BOOLEAN, default => undef}
			}
			
			);

    # Set object properties
    $self->{mgmt_vm} = $args{mgmt_vm};
    $self->{esx_server} = $args{esx_server};
    $self->{esx_host} = $args{esx_host};
    $self->{esx_user} = $args{esx_user};
    $self->{esx_pass} = $args{esx_pass};
    $self->{username} = $args{username};
    $self->{password} = $args{password};
    $self->{mode} = $args{connection};
    $self->{save} = $args{save};

    # Pre-packaged VMs behave differently from other management VMs
    # The user is "locked" in a dvshell.  This requires some differences
    # when connecting to the VM
    $self->{prepackaged} = $args{prepackaged};

    # Clear error buffer
    $self->_clear_err();

    # Create an SSH connection
    if ($self->{prepackaged}) {

	# The pre-packaged VM shell is wrapped in a "dvshell" that restricts shell access.
	# As a result, the hosttype value in the nate_host file for the management VM must
	# be changed to something NATE doesn't recognise, otherwise it will try to run bash
	# commands and do some other useless stuff.
	eval {
	    $self->{conn_dvadmin} = Connectrec->new(
						    host => $self->{mgmt_vm},
						    template => "libssh-persist",
						    username => $self->{username},
						    password => $self->{password},
						    shell_type => "none",
						    connect => 1,
						    prompts => ["Do you want to use saved login", 
								"Is the ESX host managed"],
						    timeout => 60
						    );
	};	    
	if ($@) {
	    $self->_set_err($@);
	    return $self;
	}

	#Edited by Ashrut(ashrut@netapp.com) to check for hostname prompt which is dataontapv-installer by default.
	eval {
	    $self->{conn} = Connectrec->new(
					    host => $self->{mgmt_vm},
					    template => "libssh-persist",
					    username => $self->{username},
                                            password => $self->{password},
					    shell_type => "none",
					    connect => 1,
					    prompts => [".+\@$self->{mgmt_vm}",
							".+\@dataontapv-installer",
							"Do you want to use saved login", 
							"Is the ESX host managed"],
					    timeout => 60
					    );
	};
	if ($@) {
	    $self->_set_err($@);
	    return $self;
	}

	# The pre-packaged management VM requires that the user hit control-C
	# to escape "dvshell" and enter the user shell.
	eval {
 		$self->{conn}->send_intr();
        	@buf = $self->{conn}->execute_expect();
	};
	if ($@) {
	    $self->_set_err($@);
	    return $self;
	}
	
    } else {
	eval {
	    $self->{conn} = Connectrec->new(
					    host => $self->{mgmt_vm},
					    template => "libssh-persist",
					    username => $self->{username},
                                            password => $self->{password},
					    connect => 1
					    );
	};	    
	if ($@){
	    $self->_set_err($@);
	    return $self;
	}
	
	# Create an SSH connection for the dvadmin shell
	eval {	
	    $self->{conn_dvadmin} = Connectrec->new(
						    host => $self->{mgmt_vm},
						    template => "libssh-persist",
						    username => $self->{username},
                                                    password => $self->{password},
						    connect => 1
						    );
	};
	if ($@){
	    $self->_set_err($@);
	    return $self;
	}
    }

    # Start dvadmin
    $self->_start_dvadmin;

    return $self;
} 

###############################################################################
# Method:    destroy
# Objective: Cleans up the open connection handles
# Details:   See POD documentation at the end of this file.
###############################################################################
sub destroy {
    my $self = shift;

    $self->{conn}->destroy();
    $self->{conn_dvadmin}->destroy();

    return $self;
}

###############################################################################
# Method:    execute
# Objective: Executes an aribrary command and checks for a return code
# Details:   See POD documentation at the end of this file.
###############################################################################
sub execute {
    my $self = shift;
    my (%args) = @_;
    my (@buf,
	$host,
	$line,
	$prompts,
	@tmpbuf,
	@retbuf,
	$cmd_time,
	$start_time,
	$end_time);
    
    # Clear error buffer
    $self->_clear_err();

    # Reset command execution time
    $self->{cmdtime} = 0;

    # Validate input arguments
    (%args) = validate (
			@_,
			{
			    cmd => {type => SCALAR},
			    prompts => {type => ARRAYREF, default => ()},
			    timeout => {type => SCALAR, default => 60},
			}
			);

    # Expected prompts change depending on execution mode
    if ($self->{mode} =~ /shell/) {
	$start_time = time;

        $host = $self->{esx_host} ? $self->{esx_host} : $self->{esx_server};

	$prompts = ['dvadmin>', "$host>"];
	push @{$prompts}, @{$args{prompts}} if defined($args{prompts});

	eval {		
	    @buf = $self->{conn_dvadmin}->execute(command => $args{cmd},
						  additional_prompts => $prompts,
						  timeout => $args{timeout});

	};	
	if ($@) {
	    $self->_set_err($@);
	    return $self;
	}
	    
	$end_time = time;
	$cmd_time = $end_time - $start_time;

	$self->{cmdtime} = $cmd_time;

    } elsif ($self->{mode} =~ /cmd/) {
	$start_time = time;

	eval {	
	    @buf = $self->{conn_dvadmin}->execute(command => "$self->{dvcmd} $args{cmd}",
						  timeout => $args{timeout});
	};	
	if ($@) {
	    $self->_set_err($@);
	    return $self;
	}
	    
	$end_time = time;
	$cmd_time = $end_time - $start_time;

	$self->{cmdtime} = $cmd_time;

    } else {
	$self->_set_err("Unknown connection mode!");
	return $self;
    }


    # This hokey code splits up @buf by newlines
    my @retbuf = ();
    foreach (@buf) {
	my @tmpbuf = split /\n/, $_;
	push @retbuf, @tmpbuf;
    }

    $self->{output} = \@retbuf;

    # Dvadmin errors are of the form: Error: [13]: InvalidArg...
    foreach $line (@retbuf) {
	if ($line =~ /(Error:\s.+)/) {
	    $self->_set_err($1);
	}
    }

    # $Log->comment("DVADMIN.PM: " . Dumper(@retbuf));

    return $self;
}

###############################################################################
# Method:    execute_mgmtvm
# Objective: Execute an OS command on the management VM, outside of dvadmin
# Details:   See POD documentation at the end of this file.
###############################################################################
sub execute_mgmtvm {
    my $self = shift;
    my (%args, 
	@buf,
	$cmd,
	$params,
	$prompts);

    # Clear error buffer
    $self->_clear_err();
    $self->{output} = undef;

    # Validate input arguments
    (%args) = validate (
			@_,
			{
			    cmd => {type => SCALAR, required => 1},
			    prompts => {type => ARRAYREF, default => ()},
			    timeout => {type => SCALAR, default => 30},
			}
			);

    # Execute the linux command
    eval {	
	$params->{command} = $args{cmd};
	$params->{additional_prompts} = $args{prompts} if defined($args{prompts});
	$params->{timeout} = $args{timeout};

	@buf = $self->{conn}->execute($params);
    }; 
    if ($@) {
	$self->_set_err($@);
    }

    # This hokey code splits up @buf by newlines
    my @retbuf = ();
    
    foreach (@buf) {
	my @tmpbuf = split /\n/, $_;
	push @retbuf, @tmpbuf;
    }

    $self->{output} = \@retbuf;

    return $self;
}

###############################################################################
# Method:    set_fault
# Objective: Set a dvadmin fault injection point
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_fault {
    my $self = shift;
    my (%args, $cmd);

    # Clear error buffer
    $self->_clear_err();

    # Validate input arguments
    (%args) = validate (
			@_,
			{
			    fault => {type => SCALAR, required => 1},
			    value => {type => SCALAR, required => 1},
			    timeout => {type => SCALAR, default => 30},
			}
			
			);

    # This is a no-op for prepackaged management VMs
    if ($self->{prepackaged}) {
	$self->_set_err("Pre-packaged VM: unsupported");
	return $self;
    }

    eval {
	$self->{conn_dvadmin}->disconnect();
	$self->{conn_dvadmin}->connect();
    };
    if ($@) {
	$self->_set_err($@);
    }
    
    # Set the fault injection point as user vsa
    $cmd = "export DVADMIN_INJECT_" . $args{fault} . "=" . $args{value};
    eval {
	$self->{conn_dvadmin}->execute(command => $cmd,
				       timeout => $args{timeout});
    }; 
    if ($@) {
	$Log->comment("$@");
	$self->_set_err($@);
    }

    # Start dvadmin
    $self->_start_dvadmin();
    
    return $self;
}

###############################################################################
# Method:    clear_faults
# Objective: Clear all dvadmin fault injection points
# Details:   See POD documentation at the end of this file.
###############################################################################
sub clear_faults {
    my $self = shift;

    # Clear error buffer
    $self->_clear_err();

    # This is a no-op for prepackaged management VMs
    if ($self->{prepackaged}) {
	$self->_set_err("Pre-packaged VM: unsupported");
	return $self;
    }

    eval {
	$self->{conn_dvadmin}->disconnect();
	$self->{conn_dvadmin}->connect();
    };
    if ($@) {
	$self->_set_err($@);
    }
    
    # Start dvadmin
    $self->_start_dvadmin();
    
    return $self;
}

###############################################################################
# Method:    get_time
# Objective: Returns the execution time from the last command executed
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_time {
    my $self = shift;
    
    return $self->{cmdtime};
}

###############################################################################
# Method:    get_result
# Objective: Returns the output from the last command executed
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_result {
    my $self = shift;
    
    return @{$self->{output}};
}

###############################################################################
# Method:    get_vm
# Objective: Returns a hash containing the properties listed in 'vm show'
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_vm {
    my $self = shift;
    my (%args) = @_;
    my (@buf,
	$i,
	$vm);

    # Validate input arguments
    (%args) = validate (
			@_,
			{
			    vm => {type => SCALAR, required => 1},
			    timeout => {type => SCALAR, default => 30},
			}
			
			);


    # The "vm show" command shows VM state and info
    $self->execute(cmd => "vm show $args{vm}");
    if ($self->error) {
	return undef;
    }

    @buf = $self->get_result;

    for ($i=0; $i<@buf; $i++) {
	if ($buf[$i] =~ /VM Name/) {
	    ($vm->{name}, $vm->{power}, $vm->{state}, $vm->{heartbeat}, $vm->{hostname}, $vm->{ip}) = split(/\s+/, $buf[++$i], '\n');
	} elsif ($buf[$i] =~ /^VM Version/) {
	    ($vm->{vm_ver}, $vm->{ontap_ver}, $vm->{mode}, $vm->{hostname}, $vm->{serial}, $vm->{sysid}) = split(/\s+/, $buf[++$i], '\n');
	} elsif ($buf[$i] =~ /^Network adapter/) {
	    my $nic;
	    ($nic->{name}, $nic->{mac}, $nic->{network}, $nic->{designation}) = split(/\s\s+/, $buf[$i], '\n');
	    push @{$vm->{adapters}}, $nic;
	}
    }
    
    unless (defined $vm->{name}) {
	$self->_set_err("Unable to find $args{vm}");
	return undef;
    }
    
    return $vm;
}

###############################################################################
# Method:    list_pools
# Objective: Returns a list of storage pools available on a given ESX host
# Details:   See POD documentation at the end of this file.
###############################################################################
sub list_pools {
    my $self = shift;
    my (%args) = @_;
    my (@buf,
	$i,
	@output,
	$pool);

    # Validate input arguments
    (%args) = validate (
			@_,
			{
			    timeout => {type => SCALAR, default => 30},
			}
			
			);


    # The "vm show" command shows VM state and info
    $self->execute(cmd => "pool show",
		   timeout => $args{timeout});
    if ($self->error) {
	return undef;
    }

    @buf = $self->get_result;

    for ($i=0; $i<@buf; $i++) {
	# this will skip over the header and prompt info
	if ($buf[$i] =~ /\d+\s+\d+/) {
	    # split on multiple spaces as some pool names have spaces
	    my @output = split /\s\s+/, $buf[$i];

	    $pool->{$output[0]}->{type} = $output[1];
	    $pool->{$output[0]}->{access} = $output[2];
	    $pool->{$output[0]}->{capacity} = $output[3];
	    $pool->{$output[0]}->{available} = $output[4];
	    $pool->{$output[0]}->{disk} = $output[5];
	}
    }
    
    return $pool;
}

###############################################################################
# Method:    list_networks
# Objective: Returns a list of networks available on a given ESX host
# Details:   See POD documentation at the end of this file.
###############################################################################
sub list_networks {
    my $self = shift;
    my (%args) = @_;
    my (@buf,
	$i,
	@output,
	$network);

    # Validate input arguments
    (%args) = validate (
			@_,
			{
			    timeout => {type => SCALAR, default => 30},
			}
			
			);


    # The "vm show" command shows VM state and info
    $self->execute(cmd => "network show",
		   timeout => $args{timeout});
    if ($self->error) {
	return undef;
    }

    @buf = $self->get_result;

    for ($i=0; $i<@buf; $i++) {
	# this will skip over the header and prompt info
	if ($buf[$i] =~ /vSwitch/) {
	    # split on multiple spaces as some vSwitch names have spaces
	    my @output = split /\s\s+/, $buf[$i];

	    $network->{$output[0]}->{vswitch} = $output[1];
	}
    }
    
    return $network;
}

###############################################################################
# Method:    get_disk
# Objective: Returns a disk object containing detsils of the physical disk
#            when the user passes in a datastore name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_disk {
    my $self = shift;
    my (%args) = @_;
    my (@buf,
        $disk);
    
    # Validate input arguments
    (%args) = validate (
                        @_,
                        {
			    datastore => {type => SCALAR, required => 1},
                            timeout => {type => SCALAR, default => 30},
                        }
			
			);
    
    $self->execute(cmd => "pdisk show",
		   timeout => $args{timeout});
    if ($self->error) {
        return undef;
    }

    @buf = $self->get_result;

    foreach (@buf) {
	if ($_ =~ /(.+)\s+(\w.+)\s+(\d+)\s+($args{datastore})/) {
	    $disk->{name} = $1;
	    $disk->{cname} = $2;
	    $disk->{size} = $3;
	    $disk->{datastore} = $4;
	}
    }

    unless (defined($disk)) {
	$self->_set_err("Unable to find physical disk info");
	return undef; 
    }

    return $disk;
}

###############################################################################
# Method:    destroy_disk
# Objective: Destroys a user specified ONTAP data disk or all data disks
#            available.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub destroy_disk {
    my $self = shift;
    my (%args) = @_;
    my (@buf);
    
    # Validate input arguments
    (%args) = validate (
                        @_,
                        {
			    vsa => {type => SCALAR, required => 1},
			    disk => {type => SCALAR, default => 'all'},
                            timeout => {type => SCALAR, default => 30},
                        }
			);
    
    if ($args{disk} eq 'all') {
	# Destroy all visible data disks
	$self->execute(cmd => "vm disk show $args{vsa}");
	if ($self->error) {
	    return undef;
	}
	
	@buf = $self->get_result();

	foreach (@buf) {
	    if ($_ =~ /(scsi\d+:\d+)/) {   
		$self->execute(cmd => "vm disk destroy $args{vsa} '$1'",
			       timeout => 120);
		if ($self->error) {
		    return undef;
		}
	    }
	}
    } else {
	# Destroy user specified data disk
	$self->execute(cmd => "vm disk destroy $args{vsa} $args{disk}",
		       timeout => $args{timeout});
	if ($self->error) {
	    return undef;
	}
    }

    return undef;
}

###############################################################################
# Method:    create_vm
# Objective: Create a VSA using the supplied properties
# Details:   See POD documentation at the end of this file.
###############################################################################
sub create_vm {
    my $self = shift;
    my (%args) = @_;
    my ($cmd,
	$opt,
	$prop,
	@req_prop);

    # Validate input arguments
    (%args) = validate (
			@_,
			{
			    props => {type => HASHREF, required => 1},
			    timeout => {type => SCALAR, default => 600},
			}
			);

    if ($args{props}->{mode}) {
	&_create_cmode_vm($self, props => $args{props}, timeout => $args{timeout});
    } else {
	&_create_7mode_vm($self, props => $args{props}, timeout => $args{timeout});
    }

    return undef;
}

###############################################################################
# Method:    get_vm_state
# Objective: Returns a hash containing the properties listed in 'vm state show'
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_vm_state {
    my $self = shift;
    my (%args) = @_;
    my (@buf,
	$line,
	$vm);

    # Validate input arguments
    (%args) = validate (
			@_,
			{
			    vm => {type => SCALAR, required => 1},
			    timeout => {type => SCALAR, default => 30},
			}
			
			);


    # The "vm show" command shows VM state and info
    $self->execute(cmd => "vm state show $args{vm}");
    $self->_clear_err();

    @buf = $self->get_result;

    foreach $line (@buf) {
	if ($line =~ /State: (\w+)/) {
	    $vm->{state} = $1;
	}
	elsif ($line =~ /Error: (.+)$/) {
	    $vm->{error} = $1;
	}
    }

    unless (defined $vm->{state}) {
	$self->_set_err("Unable to determine VM state");
	return undef;
    }
    
    return $vm;
}

###############################################################################
# Method:    get_version
# Objective: Return the version of dvadmin's status command
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_version {
    my $self = shift;
    my (@buf,
        $line,
	@find,
        $version);

    $self->execute(cmd => 'status', timeout => 30);

    @buf = $self->get_result;

    $version = 'unknown';

    foreach $line (@buf) {
        if ($line =~ /Version: (.+)/) {
		if ($line =~ /^(\s?Version:) (.+)/) {
			@find = ($line =~ /(.*)(\d\.\d\.\d+)/);
			$version = $2;
		}
	}
    }

    return $version;
}

###############################################################################
# P  R  I  V  A  T  E    M  E  T  H  O  D  S
###############################################################################
# Method:    _start_dvadmin
# Objective: Starts dvadmin on a given connection handle 
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _start_dvadmin {
    my $self = shift;
    my (@buf,
	$dvadmin,
	$host,
	$prompt,
	$vcs);

    $host = defined($self->{esx_host}) ? $self->{esx_host} : $self->{esx_server};

    # Change to the appropriate mode for dvadmin (e.g. shell or cmd)
    if ($self->{prepackaged}) {
	$vcs = $self->{esx_host} ? "yes" : "no";

	if ($self->{mode} =~ /shell/) {
            $Log->comment("dvadmin.pm using SHELL mode");

	    eval {
		$self->{conn_dvadmin}->execute(command => 'exit',
					       match_table => [
							       "Do you want to use saved login" => "no",
							       "Is the ESX host managed" => $vcs,
							       "Enter ESX host name or IP address" => $host,
							       "Enter vCenter server name or IP" => $self->{esx_server},
							       "Enter vCenter server username" => $self->{esx_user},
							       "Enter vCenter server password" => $self->{esx_pass},
							       "Enter ESX host username" => $self->{esx_user},
							       "Enter ESX host password" => $self->{esx_pass},
							       "Do you want to save" => "yes"
							       ],
					       prompts => ['dvadmin>', "$host>"],
					       timeout => 60);
	    };
	    if ($@) {
		$self->_set_err($@);
		return $self;
	    }
        } elsif ($self->{mode} =~ /cmd/) {
            $Log->comment("dvadmin.pm using CMD mode");

	    eval {
                $self->{conn_dvadmin}->send_intr();
                @buf = $self->{conn_dvadmin}->execute_expect();
	    };
	    if ($@) {
		$self->_set_err($@);
		return $self;
	    }

            $self->{dvcmd} = "dvadmin -s $self->{esx_server} ";
            $self->{dvcmd} .= "-h $self->{esx_host} " if ($self->{esx_host});
            $self->{dvcmd} .= "-u $self->{esx_user} ";
            $self->{dvcmd} .= "-p $self->{esx_pass} ";
            $self->{dvcmd} .= "-save " if ($self->{save});
        } else {
            $self->_set_err("Unknown connection mode!");
            return $self;
        }
    } else {
	if ($self->{mode} =~ /shell/) {
	    $Log->comment("dvadmin.pm using SHELL mode");
	    
	    $dvadmin = "dvadmin -s $self->{esx_server} ";
	    $dvadmin .= "-h $self->{esx_host} " if ($self->{esx_host});
	    $dvadmin .= "-u $self->{esx_user} ";
	    $dvadmin .= "-p $self->{esx_pass} ";
	    $dvadmin .= "-save " if ($self->{save});
	    
	    # Start dvadmin in shell mode
	    eval {
		$self->{conn_dvadmin}->execute(command => $dvadmin,
					       prompts => ['dvadmin>', "$host>"],
					       timeout => 60);
	    }; 
	    if ($@) {
		$self->_set_err($@);
		return $self;
	    }
	} elsif ($self->{mode} =~ /cmd/) {
	    $Log->comment("dvadmin.pm using CMD mode");
	    
	    $self->{dvcmd} = "dvadmin -s $self->{esx_server} ";
	    $self->{dvcmd} .= "-h $self->{esx_host} " if ($self->{esx_host});
	    $self->{dvcmd} .= "-u $self->{esx_user} ";
	    $self->{dvcmd} .= "-p $self->{esx_pass} ";
	    $self->{dvcmd} .= "-save " if ($self->{save});
	} else {
	    $self->_set_err("Unknown connection mode!");
	    return $self;
	}
    }
    return $self;
}

###############################################################################
# Method:    _create_cmode_vm
# Objective: Create a Cmode VSA
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _create_cmode_vm {
    my $self = shift;
    my (%args) = @_;
    my ($cmd,
	$opt,
	$prop,
	@req_prop);


    # Base properties required to build the vm create command
    @req_prop = ('vm_name', 
		'pool_name', 
		'nv_pool_name',
		'root_agg_pool_name',
		'mgmtnet_name',
		'license',
		'clustermgmt_ipaddr',
		'clustermgmt_netmask',
		'clustermgmt_gateway',
		'nodemgmt_ipaddr',	
		'password',
		'vsphere_username',
		'vsphere_password',
		'mode'
		);

    $cmd = "vm create $args{props}->{vm_name} " .
	'"' . "$args{props}->{pool_name}" . '" ' .
	'"' . "$args{props}->{nv_pool_name}" . '" ' .
	'"' . "$args{props}->{root_agg_pool_name}" . '" ' .
	"$args{props}->{mgmtnet_name} " .
	"$args{props}->{ovf_pkg} " .
	"license=$args{props}->{license} " .
	"clustermgmt_ipaddr=$args{props}->{clustermgmt_ipaddr} " .
	"clustermgmt_netmask=$args{props}->{clustermgmt_netmask} " .
	"clustermgmt_gateway=$args{props}->{clustermgmt_gateway} " .
	"nodemgmt_ipaddr=$args{props}->{nodemgmt_ipaddr} " .
	"password=$args{props}->{password} " .
	"vsphere_username=$args{props}->{vsphere_username} " .
	"vsphere_password=$args{props}->{vsphere_password} " .
	"mode=C";

    foreach $prop (keys %{$args{props}}) {
	unless (grep(/$prop/, @req_prop)) {
	    $opt .= $prop . '=' . $args{prop}->{$prop} . ' ';
	}
    }

    $cmd .= $opt;

    $self->execute(cmd => $cmd, timeout => $args{timeout});

    return undef;
}

###############################################################################
# Method:    _create_7mode_vm
# Objective: Create a 7mode VSA
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _create_7mode_vm {
    my $self = shift;
    my (%args) = @_;
    my ($cmd,
	$opt,
	$prop,
	@req_prop);


    # Base properties required to build the vm create command
    @req_prop = ('vm_name', 
		 'pool_name', 
		 'nv_pool_name', 
		 'network_name', 
		 'ovf_pkg', 
		 'license', 
		 'hostname', 
		 'ipaddr', 
		 'netmask', 
		 'gateway', 
		 'password', 
		 'vsphere_username', 
		 'vsphere_password');

    $cmd = "vm create $args{props}->{vm_name} " .
	'"' . "$args{props}->{pool_name}" . '" ' .
	'"' . "$args{props}->{nv_pool_name}" . '" ' .
	"$args{props}->{network_name} ";
    
    unless ($self->{prepackaged}) {
	$cmd .= "$args{props}->{ovf_pkg} ";
    }

    $cmd .= "license=$args{props}->{license} " .
	"hostname=$args{props}->{hostname} " .
	"ipaddr=$args{props}->{ipaddr} " .
	"netmask=$args{props}->{netmask} " .
	"gateway=$args{props}->{gateway} " .
	"password=$args{props}->{password} " .
	"vsphere_username=$args{props}->{vsphere_username} " .
	"vsphere_password=$args{props}->{vsphere_password} ";

    foreach $prop (keys %{$args{props}}) {
	unless (grep(/$prop/, @req_prop)) {
	    $opt .= $prop . '=' . $args{prop}->{$prop} . ' ';
	}
    }

    $cmd .= $opt;

    $self->execute(cmd => $cmd, timeout => $args{timeout});

    return undef;
}

1;

############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

dvadmin.pm

=head1 Author

lippe@netapp.com

=head1 Description

Dvadmin is the managment tool customers use to deploy, monitor, and manage
VSA instances.  

This package makes no attempt at encapsulating all of the functionality of
the dvadmin tool through individual objects.  Rather, it offers users a few
basic methods which they may use to issue commands while capturing any errors 
they encounter.

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Utils::Error>

=head1 Public Methods

=head2 B<new( B<mgmt_vm>, B<esx_server>, B<esx_user>, B<esx_pass>, I<connection>, I<esx_host> )>

Creates a new VSA::dvadmin object for sending dvadmin commands to a given 
management VM.

=over 4

=item Inputs

=over 4

=item * B<mgmt_vm> 

Hostname of the managemnt VM on which the dvadmin RPM is installed.

=item * B<esx_server> 

Hostname of the ESX server on which the VSA will be deployed.

=item * B<esx_user> 

The username used to authenticate to the ESX server when the dvadmin
tool is started.

=item * B<esx_pass> 

The password used to authenticate to the ESX server when the dvadmin
tool is started.

=item * I<connection> (default: "cmd") 

Connection type.  Dvadmin supports two primary methods of interaction.
The first allows for commands to be sent individially as parameters
passed in during the tool's invocation.  The second offers users an
interactive shell, much like ONTAP, where command sets are lexically scoped
and the user has the ability to change in and out these different scopes.

=begin html

<ul>
<li><b>cmd</b>: The dvadmin tool will be instantiated each time a command is 
executed.  As a result, commands will take longer to run due to the overhead of
starting the tool.
<li><b>shell</b>: The dvadmin tool will be started once.  All commands sent to
the tool will be done from within the interactive shell.
</ul>

=end html

=item * I<esx_host> 

When ESX_SERVER is defined as a VMware virtual center server, an ESX_HOST must
be specified.  This will be the name of the ESX host that is managed by the 
virtual center server defined in ESX_SERVER.

=back 

=item Outputs

Returns a handle to the VSA::dvadmin object.  

=item Examples 

    # Create a dvadmin object
    $Dvadmin = VSA::dvadmin->new(
                                 connection => "shell",
                                 mgmt_vm  => 'nqa-vsa-mgmt1',
                                 esx_server => 'vsaesxnane06.sim',
                                 esx_user => 'root',
                                 esx_pass => 'netapp123',
    );

=back

=head2 B<execute( B<cmd>, I<timeout> )>

Execute an arbitrary dvadmin command. 

=over 4

=item Inputs

=over 4

=item * B<cmd> 

Command dvadmin will execute.

=item * I<timeout> (default: 60)

Timeout used by Tharn::say for the given dvadmin command.  If the timeout
is reached, the command will fail with a TIMEOUT error.

=back

=item Outputs

Returns a handle to the VSA::dvadmin object.  

=item Examples

    # List VSA data disks
    $Dvadmin->execute(cmd => "vm disk show $Params->{VSA}");
    if ($Dvadmin->error) {
        $Test->log($Dvadmin->errmsg);
        return $TCD::FAIL;
    }

=back

=head2 B<execute_mgmtvm( B<cmd>, I<timeout> )>

Execute an arbitrary command on the Management VM, outside of the dvadmin session.
To do this, the object must first disconnect from the current session.  Once the
old session is gone, it reconnects (to get out from the dvadmin shell) and executes
the command specified.  On completion, the object then restarts dvadmin.

=over 4

=item Inputs

=over 4

=item * B<cmd> 

Command dvadmin will execute.

=item * I<timeout> (default: 60)

Timeout used by Tharn::say for the given dvadmin command.  If the timeout
is reached, the command will fail with a TIMEOUT error.

=back

=item Outputs

Returns a handle to the VSA::dvadmin object.  

=item Examples

    # List VSA data disks
    $Dvadmin->execute_mgmtvm(cmd => "ls -al /tmp");
    if ($Dvadmin->error) {
        $Test->log($Dvadmin->errmsg);
        return $TCD::FAIL;
    }

=back

=head2 B<get_time()>

Return the time, in seconds, of the last command executed using the execute
method.

=over 4

=item Outputs

The time, in seconds, of the last command executed using the execute method.

=item Examples

    # Time the dvadmin "vm disk create" command 
    $Dvadmin->execute(cmd => "vm disk create $Params->{VSA} $Params->{VSA_DATA_POOL} 5g");
    if ($Dvadmin->error) {
	$Test->log($Dvadmin->errmsg);
	return $TCD::FAIL;
    }
    $Test->log("   - Command execution time (s): " . $Dvadmin->get_time);

=back

=head2 B<get_result()>

Return the output of the last command executed using either execute() or 
execute_mgmtvm().

=over 4

=item Outputs

The output of the last command executed using either execute() or 
execute_mgmtvm().

=item Examples

    $Test->log(" - Retrieving new disk configuration for $Params->{VSA}");
    $Dvadmin->execute(cmd => "vm disk show $Params->{VSA}");
    if ($Dvadmin->error) {
	$Test->log($Dvadmin->errmsg);
	return $TCD::FAIL;
    }
    @buf = $Dvadmin->get_result();

=back

=head2 B<set_fault( B<fault>, B<value> )>

Set a dvadmin fault injection point.

=over 4

=item Inputs

=over 4

=item * B<fault> 

The dvadmin fault to set.  The library will prepend the 'DVADMIN_INJECT_' portion
of the fault so the user is only required to specify the remaining piece.

=item * B<value>

Some dvadmin faults support multiple injection points.  This required value
is an integer indicating which fault injection point to trip.

=back

=item Outputs

Returns a handle to the VSA::dvadmin object.  

=item Examples

    $Test->log(" - Setting fault injection point for $Params->{VSA}");
    $Dvadmin->set_fault(fault => "vm_prop_show_cmdfail", value => 1);
    if ($Dvadmin->error) {
	$Test->log($Dvadmin->errmsg);
	return $TCD::FAIL;
    }

=back

=head2 B<clear_faults()>

Clear all dvadmin fault injection point.

=over 4

=item Outputs

Returns a handle to the VSA::dvadmin object.  

=item Examples

    $Test->log(" - Removing fault injection point for $Params->{VSA}");
    $Dvadmin->clear_faults();
    if ($Dvadmin->error) {
	$Test->log($Dvadmin->errmsg);
	return $TCD::FAIL;
    }

=back

=head2 B<get_vm( B<vm> )>

Returns a hash containing the VM properties listed in 'vm show'

=over 4

=item Inputs

=over 4

=item * B<vm> 

The VSA virtual machine object to retrieve.

=back

=item Outputs

A hash containing the properties listed in 'vm show'

=item Examples

    $vm = $Dvadmin->get_vm(vm => $Params->{VSA});
    if ($Dvadmin->error) {
        $Test->log($Dvadmin->errmsg);
        return $TCD::FAIL;
    }

    unless ($vm->{state} =~ /ready/) {
	$Test->log("     - VM not started!");
	return $TCD::FAIL;
    }

=back

=cut
