# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/VSeries/filer_zapi.pm#2 $
#
## @summary filer setup/cleanup library functions to be used by VSeries tests
##
## @description
##   These test functions are needed for the VSeries test suites
## @usage
##   use filer;
##   use filer qw(...);
##
## @keywords  filer test library
##
## @dependencies
## use config_base
## use Tharn
##
## @author Matt Cattel(cattel@netapp.com), dl-vseries-automation
## @status testme
## @change 2006-11-26 clairek: created, based on style of VSeries.pm as well
## as the OO style of the host side libraries
##

package VSeries::filer_zapi;

use strict;
use ZapiTest;
use Time::localtime;
use Data::Dumper;

use vars ('@ISA','@EXPORT');
require Exporter;
our @ISA = ('Exporter');


# This is where we create the inheritance from filer
use VSeries::filer;
our @ISA = qw(VSeries::filer);
use VSeries::Clustered::nacl_convenience;

my $ERR_STATUS = 1;
my $GOOD_STATUS = 0;



## @name zapi_disk_list_info()
##
## @description
## This method calls the disk-list-info zapi api.  It turns the
## NaElement returned by the call into a string and then parses the string
## into a workable form and returns a hash of hashes keyed at the top level
## by each disk's serial number and keyed at the secondary level by the
## informational xml tags that are provided by the disks.  There's too many
## to list here so it is recommended that in order to learn everything
## that's returned make a call here and then list the keys of the secondary
## level hashes.
##
## @synopsis
## $return_hash_ref = $filer->zapi_disk_list_info();
##
##
## Here's some sample XML output:
## <results status="passed">
##    <disk-details>
##            <disk-detail-info>
##                    <name>0b.25</name>
##                    <disk-uid>20000000:875AF6F1:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000</disk-uid>
##                    <bay>9</bay>
##                    <bytes-per-sector>520</bytes-per-sector>
##                    <checksum-compatibility>zoned/block</checksum-compatibility>
##                    <disk-type>FCAL</disk-type>
##                    <effective-disk-type>XSAS</effective-disk-type>
##                    <firmware-revision>NA03</firmware-revision>
##                    <host-adapter>0b</host-adapter>
##                    <id>1376259</id>
##                    <is-zeroed>true</is-zeroed>
##                    <physical-blocks>281442144</physical-blocks>
##                    <physical-space>144098377728</physical-space>
##                    <pool>Pool0</pool>
##                    <port>B</port>
##                    <port-name>FC:B </port-name>
##                    <raid-state>spare</raid-state>
##                    <raid-type>pending</raid-type>
##                    <rpm>10000</rpm>
##                    <secondary-name>0d.25</secondary-name>
##                    <secondary-port>A</secondary-port>
##                    <serial-number>V5YGV65A</serial-number>
##                    <shelf>1</shelf>
##                    <used-blocks>278528000</used-blocks>
##                    <used-space>142606336000</used-space>
##                    <vendor-id>NETAPP  </vendor-id>
##                    <blocks-read>0</blocks-read>
##                    <blocks-written>0</blocks-written>
##                    <shm-time-interval>0</shm-time-interval>
##                    <grown-defect-list-count>0</grown-defect-list-count>
##                    <scrub-last-done>1012</scrub-last-done>
##                    <scrub-count>2</scrub-count>
##                    <is-dynamically-qualified>0</is-dynamically-qualified>
##                    <raw-disk-sectors>281442151</raw-disk-sectors>
##                    <disk-model>X274_HPYTA146F10</disk-model>
##            </disk-detail-info>
##            <disk-detail-info>
##                    <name>vns-m4400-2:0.127L3</name>
##                    <disk-uid>60060480:00018746:14325359:4D313639:00000000:00000000:00000000:00000000:00000000:00000000</disk-uid>
##                    <bay>?</bay>
##                    <bytes-per-sector>520</bytes-per-sector>
##                    <checksum-compatibility>zoned/block</checksum-compatibility>
##                    <disk-type>LUN</disk-type>
##                    <effective-disk-type>LUN</effective-disk-type>
##                    <firmware-revision>0000</firmware-revision>
##                    <host-adapter>0a</host-adapter>
##                    <id>524291</id>
##                    <is-zeroed>false</is-zeroed>
##                    <physical-blocks>95397120</physical-blocks>
##                    <physical-space>48843325440</physical-space>
##                    <pool>Pool0</pool>
##                    <port> </port>
##                    <port-name>     </port-name>
##                    <raid-state>spare</raid-state>
##                    <raid-type>pending</raid-type>
##                    <secondary-name>vss_b3800_7:3.126L3</secondary-name>
##                    <secondary-port> </secondary-port>
##                    <serial-number>461432169000</serial-number>
##                    <shelf>?</shelf>
##                    <used-blocks>94441472</used-blocks>
##                    <used-space>48354033664</used-space>
##                    <vendor-id>EMC     </vendor-id>
##                    <blocks-read>0</blocks-read>
##                    <blocks-written>0</blocks-written>
##                    <shm-time-interval>0</shm-time-interval>
##                    <grown-defect-list-count>0</grown-defect-list-count>
##                    <scrub-last-done>0</scrub-last-done>
##                    <scrub-count>0</scrub-count>
##                    <is-dynamically-qualified>0</is-dynamically-qualified>
##                    <raw-disk-sectors>109025280</raw-disk-sectors>
##                    <disk-model>SYMMETRIX       </disk-model>
##                    <v-series-detail-info>
##                            <product-id>SYMMETRIX       </product-id>
##                            <primary-switch-name>vns-m4400-2</primary-switch-name>
##                            <primary-switch-port>0</primary-switch-port>
##                            <primary-lun-number>3</primary-lun-number>
##                            <primary-vseries-port-wwn>500a098200014388</primary-vseries-port-wwn>
##                            <primary-array-port-wwn>50060482cb1bce3c</primary-array-port-wwn>
##                            <primary-array-switch-port-wwn>2000080088e36174</primary-array-switch-port-wwn>
##                            <alternate-switch-name>vns-m4400-2</alternate-switch-name>
##                            <alternate-switch-port>0</alternate-switch-port>
##                            <alternate-lun-number>3</alternate-lun-number>
##                            <alternate-vseries-port-wwn>500a098000014388</alternate-vseries-port-wwn>
##                            <alternate-array-port-wwn>50060482cb1bce2c</alternate-array-port-wwn>
##                            <alternate-array-switch-port-wwn>200300606951f1e4</alternate-array-switch-port-wwn>
##                    </v-series-detail-info>
##            </disk-detail-info>
##    </disk-details>
## </results>


sub zapi_disk_list_info(){
    my $self = shift;

    # Invoke the server to run zapi commands at this filer.
    my $server = new ZapiTest($self->get_name());

    # This will be the hash of hashes returned to the caller
    my %return_hash = ();

    # Make the call to the zapi api and check that the result is ok.  If there
    # was an error return it to the caller via the proper mechanism.
    my $result = $server->invoke("disk-list-info");
    if (! $result->results_ok() ) {
        $self->{error} = $ERR_STATUS;
        $self->{errmsg} = $result->results_reason();
    }

    # The NaElement data object is a bit clunky so let's turn it to a string
    # and parse the xml.
    my $string = $result->toString();

    # In the event of an error this produces a nice tabbed human readable
    # form of the xml for printing in logs or to the console.
    my $print_string = $result->sprint_pretty();

    # Here we strip out some of the tags that we already know to be good
    # or will mess up the parsing of the tags in a nice repeatable way.
    $string =~ s/<results status=\"passed\">//;
    $string =~ s/<\/results>//;
    $string =~ s/<\/?disk-details>//g;


    # This XML tag is key, it's the demarkation of one disk's set of information
    # from the next.  Splitting on this will give us all the XML for a specific
    # disk in it's own array element for parsing.
    my @split_array = split(/\<\/disk-detail-info\>/, $string);

    # Now we'll loop over the array walking the xml string and pulling
    # out the disk's information and putting it into a hash.  When we
    # hit the end we put that hash into the hash of hashes returned
    # to the caller.
    foreach my $element (@split_array){

        # Hash containing the tokenized information about a given disk
        my %hash_of_attributes = ();

        # This is the up front check to ensure that no garbage tags
        # start getting parsed since the regexes here are pretty
        # tight and expect the xml in a certain form.
        if($element =~ m/\<disk-detail-info\>/){

            # Clear out any information from the last loop.
            %hash_of_attributes = ();

            # Strip out that leading tag since it's just a marker and
            # not attribute information.
            $element =~ s/\<disk-detail-info\>//;

            # This will match on any of the xml tags and remember what
            # the name of the tag was.  This will become a key in the
            # hash of attributes.  The attribute will be the value
            # associated with this tag/key.
            while($element =~ m/\<\/?([^>]*)\>/){

                my $match = $1;
                my $attribute = "";

                # Strip out the leading tag so that when we split
                # on the closing match of that tag we get the information
                # associated with that tag into attribute.
                $element =~ s/\<\/?$match\>//;

                # The format of this xml output contains a nested tag for
                # specific v-series information.  While it itself is just
                # a demarkating tag there are test cases calling for the
                # checking of this flag being there for LUNS and not for
                # disks so we'll make it a key in the attribute hash that has
                # an empty string paired with it.
                if($match =~ m/v-series-detail-info/){
                    $element =~ s/\<\/?$match\>//;
                }
                else{

                    # For all other tags we'll split on the closing tag so that
                    # attribute contains the information associated with this
                    # tag.
                    $attribute = (split(/\<\/?$match\>/, $element))[0];
                    $element = (split(/\<\/?$match\>/, $element))[1];
                }

                # Put the xml tag as a key and the information associated with
                # it into the hash of attributes.
                $hash_of_attributes{$match} = $attribute;

            }
        }

        # If there are no more attributes for this disk, enter it into the hash
        # to be returned to the caller keyed at the top level by the serial
        # number.
        $return_hash{$hash_of_attributes{'serial-number'}} = \%hash_of_attributes;

    }

    # Return the hash of hashes and the formatted xml to the caller.
    return (\%return_hash, $print_string);



}



## @name zapi_disk_sanown_list_info()
##
## @description
## This method calls the disk-sanown-list-info zapi api.  It turns the
## NaElement returned by the call into a string and then parses the string
## into a workable form and returns a hash of hashes keyed at the top level
## by each disk's serial number and keyed at the secondary level by the
## informational xml tags that are provided by the disks.  There's to many
## to list here so it is recommended that in order to learn everything
## that's returned make a call here and then list the keys of the secondary
## level hashes.
##
## @synopsis
## $return_hash_ref = $filer->zapi_disk_sanown_list_info();
##
##
## Here's sample XML:
## <results status="passed">
##    <disk-sanown-details>
##            <disk-sanown-detail-info>
##                    <name>vns-m4400-2:0.127L4</name>
##                    <is-failed>false</is-failed>
##                    <serial-no>4614325AC000</serial-no>
##                    <reserved-by>0</reserved-by>
##                    <checksum>zoned</checksum>
##            </disk-sanown-detail-info>
##            <disk-sanown-detail-info>
##                    <name>0b.19</name>
##                    <is-failed>false</is-failed>
##                    <serial-no>V5YHHG0A</serial-no>
##                    <reserved-by>101190053</reserved-by>
##                    <owner>vsa-v3050auto-1a</owner>
##                    <owner-id>101190053</owner-id>
##                    <pool>0</pool>
##                    <home>vsa-v3050auto-1a</home>
##                    <home-id>101190053</home-id>
##            </disk-sanown-detail-info>
##    </disk-sanown-details>
## </results>
##
sub zapi_disk_sanown_list_info(){
    my $self = shift;

    # Invoke the server to run zapi commands at this filer.
    my $server = new ZapiTest($self->get_name());

    # This will be the hash of hashes returned to the caller
    my %return_hash = ();

    # Make the call to the zapi api and check that the result is ok.  If there
    # was an error return it to the caller via the proper mechanism.
    my $result = $server->invoke("disk-sanown-list-info");
    if (! $result->results_ok() ) {
        $self->{error} = $ERR_STATUS;
        $self->{errmsg} = $result->results_reason();
    }

    # The NaElement data object is a bit clunky so let's turn it to a string
    # and parse the xml.
    my $string = $result->toString();

    # In the event of an error this produces a nice tabbed human readable
    # form of the xml for printing in logs or to the console.
    my $print_string = $result->sprint_pretty();

    # Here we strip out some of the tags that we already know to be good
    # or will mess up the parsing of the tags in a nice repeatable way.
    $string =~ s/<results status=\"passed\">//;
    $string =~ s/<\/results>//;
    $string =~ s/<\/?disk-sanown-details>//g;

    # This XML tag is key, it's the demarkation of one disks set of information
    # from the next.  Splitting on this will give us all the XML for a specific
    # disk in it's own array element for parsing.
    my @split_array = split(/\<\/disk-sanown-detail-info\>/, $string);

    # Now we'll loop over the array walking the xml string and pulling
    # out the disk's information and putting it into a hash.  When we
    # hit the end we put that hash into the hash of hashes returned
    # to the caller.
    foreach my $element (@split_array){

        # Hash containing the tokenized information about a given disk
        my %hash_of_attributes = ();

        # This is the up front check to ensure that no garbage tags
        # start getting parsed since the regexes here are pretty
        # tight and expect the xml in a certain form.
        if($element =~ m/\<disk-sanown-detail-info\>/){

            # Clear out any information from the last loop.
            %hash_of_attributes = ();

            # Strip out that leading tag since it's just a marker and
            # not attribute information.
            $element =~ s/\<disk-sanown-detail-info\>//;

            # This will match on any of the xml tags and remember what
            # the name of the tag was.  This will become a key in the
            # hash of attributes.  The attribute will be the value
            # associated with this tag/key.
            while($element =~ m/\<\/?([^>]*)\>/){

                my $match = $1;
                my $attribute = "";

                # Strip out the leading tag so that when we split
                # on the closing match of that tag we get the information
                # associated with that tag into attribute.
                $element =~ s/\<\/?$match\>//;

                # Split on the closing tag so that attribute contains the
                # information associated with this tag.  Props to Mark Bozen
                # for this baller way to split.
                $attribute = (split(/\<\/?$match\>/, $element))[0];

                # Now for the sake of making progress strip off all that
                # information from the front of element so we can move
                # onto the next attribute tag.
                $element = (split(/\<\/?$match\>/, $element))[1];

                # Put the xml tag as a key and the information associated with
                # it into the hash of attributes.
                $hash_of_attributes{$match} = $attribute;

            }

        }

        # If there are no more attributes for this disk, enter it into the hash
        # to be returned to the caller keyed at the top level by the serial
        # number.
        $return_hash{$hash_of_attributes{'serial-no'}} = \%hash_of_attributes;
    }

    # Return the hash of hashes and the formatted xml to the caller.
    return (\%return_hash, $print_string);



}


## @name zapi_disk_sanown_assign($)
##
## @description
## A method that takes in the serial number of a disk to assign.  It then
## searches the internal disks hash of the object for the name of the disk
## that matches the serial number.  It is expected that the method
## discover_disks() has already been called to populate the hash.  It will
## then make the appropriate call through the zapi "disk-sanown-assign" api
## to assign the specified disk.  This method takes in an optional hash that
## is expected to be valid options that can be specified through the zapi
## "disk-sanown-assign" api.  Currently the only supported option is the
## word checksum which can be either block or zoned though no validation is
## done in the method, the interface handles the checking.
##
## @arg $disk_serial_param Required: serial number of the disk to be assigned
##
## @arg %options_hash Optional: hash of options supported by the
##                              disk-sanown-assign zapi api.
##
## @synopsis
## $return_hash_ref = $filer->zapi_disk_sanown_assign($disk_serial_param);
##
##
sub zapi_disk_sanown_assign($;%){
    my $self = shift;
    my ($disk_serial_param, $options_hash_ref) = @_;

    # This variable will be populated and used for later verification if
    # the checksum option is passed in.
    my $verify_checksum = "";

    # In cases where forcing the assign is desired we'll populate this
    # variable.
    my $force_assign = "";

    my %options_hash = ();

    # Search the options hash if one was passed in and pull out the checksum
    if(defined($options_hash_ref)){
        %options_hash = %$options_hash_ref;
        if(defined($options_hash{'checksum'})){
            $verify_checksum = $options_hash{'checksum'};
        }
        if(defined($options_hash{'force'})){
            $force_assign = $options_hash{'force'};
        }
    }

    # This will be the name of the disk that matches the serial number passed
    # in.
    my $disk_name = "";

    # Search the internal disk hash that should have been populated already
    # once we match the serial number passed in set disk name accordingly.
    foreach my $disk_serial (keys(%{$self->{disks}})){
        if($disk_serial eq $disk_serial_param){
            $disk_name = $self->{disks}->{$disk_serial}->{primary};
        }
    }

    # If the name is blank then we didn't find the serial number we were
    # looking for and will return an error.
    if($disk_name eq ""){
        my $status = "Could not find disk with Serial Number " . $disk_serial_param . " on filer " . $self->{name};
        $self->{error} = $ERR_STATUS;
        $self->{errmsg} = $status;
        return $ERR_STATUS;
    }

    # Invoke the server to run zapi commands at this filer.
    my $server = new ZapiTest($self->get_name());

    # This will contain the result of the zapi call, it's of type NaElement
    # we'll check that it was successful and validate the assign through
    # calling the zapi interface storage_disk_get_iter to check that this
    # filer shows up as the owner of the disk.
    my $result = "";

    # There's no real easy/dynamic way to call invoke with potentially optional
    # parameters so we have to separate out the case where checksum is
    # explicitly called for and when it is not.
    if($verify_checksum ne ""){
        if($force_assign ne ""){
            $result = $server->invoke("disk-sanown-assign", "disk", $disk_name, "checksum", $options_hash{'checksum'}, "force", $options_hash{'force'});
        }
        else{
            $result = $server->invoke("disk-sanown-assign", "disk", $disk_name, "checksum", $options_hash{'checksum'});
        }

        # Check the result of the call with a call to a method that can be run
        # on any NaElement.  It effectively returns a boolean if the command
        # was successful or not.
        if (! $result->results_ok() ) {
            $self->{error} = $ERR_STATUS;

            # Pull out the failure reason from the NaElement and return it to
            # the caller via the proper mechanism.
            $self->{errmsg} = $result->results_reason();

            print "HERE'S THE XML\n" . $result->sprint_pretty() . "\n";
            return $ERR_STATUS;
        }
    }
    else{
        # Same behavior as above only no checksum being passed to the zapi api.
        $result = $server->invoke("disk-sanown-assign", "disk", $disk_name);
        if (! $result->results_ok() ) {
            $self->{error} = $ERR_STATUS;
            $self->{errmsg} = $result->results_reason();
            return $ERR_STATUS;
        }
    }
    
    # Fix for BURT 550817: For RR1 and beyond, disks assigned with a zoned checksum 
    # show 'advanced_zoned' as their checksum status after getting assigned. So change 
    # $verify_checksum to 'advanced_zoned' for these disks before validation.
    if (!(defined $options_hash{'pre_rr1'}) ) {
      $verify_checksum = 'advanced_zoned' if ($verify_checksum eq 'zoned');
    }
    

    # If we got here then the disk assign should have worked.  We'll call the
    # storage_disk_get_iter api because that contains the information about
    # what filer owns what disks.  

    
    # Creating a NACL object to execute storage_disk_get_iter as it will be easy to obtain the output
    # in hash format.
    my $status = VSeries::Clustered::nacl_convenience::setup_api_set_obj([$self->get_name()], '7Mode', 'ZAPI');
    if ($status->[0] == $ERR_STATUS) {
        $self->{error} = $ERR_STATUS;
        $self->{errmsg} = "Setting up of ZAPI API Set object failed\n" . $status->[1];
        return $ERR_STATUS;
    }
    my $Zapi_7_mode_Set_Obj = $status->[0];

    # This option is required because by default storage_disk_get_iter shows information on only 200 disks/luns.
    my %options = ('max-records' => 2000);

    my $zapi_response = undef;
    my $output_ref = VSeries::Clustered::nacl_convenience::execute_cmd($Zapi_7_mode_Set_Obj, 'storage_disk_get_iter',\%options);
    if ($output_ref == $ERR_STATUS) {
        $self->{error} = $ERR_STATUS;
        $self->{errmsg} = "ERROR executing ZAPI command storage_disk_get_iter";
        return $ERR_STATUS;
    }

    # Fetching the information of all the disks.
    my @parsed_zapi_output = @{$output_ref};
    my %attributes_list = %{$parsed_zapi_output[0]};
    my %storage_disk_info = %{$attributes_list{'attributes-list'}->[0]};
    my @disk_info = @{$storage_disk_info{'storage-disk-info'}};

    # Loop through the disk info array looking for the serial number
    # that matches the serial number we were passed in.  Once we have that
    # pull out that disk hash and validate some info.
    # Output has of one disk is given below
    #  $VAR1 = {
    #            'disk-paths' => [
    #                            {
    #                              'disk-path-info' => [
    #                                                  {
    #                                                    'path-io-kbps' => '27744',
    #                                                    'disk-name' => 'vgci9148s160:1-15.126L1',
    #                                                    'target-port-access-state' => 'AO',
    #                                                    'target-io-kbps' => '27744',
    #                                                    'node' => 'vgv8040f132a',
    #                                                    'target-iops' => '309',
    #                                                    'path-lun-in-use-count' => '37',
    #                                                    'target-side-switch-port' => 'vgci9148s160:1-15',
    #                                                    'array-name' => 'HP_OPEN_1',
    #                                                    'lun-number' => '1',
    #                                                    'lun-path-use-state' => 'INU',
    #                                                    'disk-port-name' => 'FC:A',
    #                                                    'initiator-side-switch-port' => 'vgci9148s160:1-11',
    #                                                    'initiator-port-speed' => '8',
    #                                                    'path-iops' => '309',
    #                                                    'initiator-lun-in-use-count' => '37',
    #                                                    'path-link-errors' => '0',
    #                                                    'path-quality' => '0',
    #                                                    'initiator-iops' => '309',
    #                                                    'initiator-port' => '0h',
    #                                                    'target-wwpn' => '50060e8006ff1d20',
    #                                                    'tpgn' => '3',
    #                                                    'lun-io-kbps' => '7',
    #                                                    'target-lun-in-use-count' => '37',
    #                                                    'initiator-io-kbps' => '27744',
    #                                                    'preferred-target-port' => 'false',
    #                                                    'disk-port' => 'A',
    #                                                    'disk-uid' => '46463144:30303031:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
    #                                                    'lun-iops' => '2'
    #                                                  },
    #                                                  {
    #                                                    'path-io-kbps' => '31222',
    #                                                    'disk-name' => 'vgbr300s152:15.126L1',
    #                                                    'target-port-access-state' => 'AO',
    #                                                    'target-io-kbps' => '31222',
    #                                                    'node' => 'vgv8040f132a',
    #                                                    'target-iops' => '374',
    #                                                    'path-lun-in-use-count' => '14',
    #                                                    'target-side-switch-port' => 'vgbr300s152:15',
    #                                                    'array-name' => 'HP_OPEN_1',
    #                                                    'lun-number' => '1',
    #                                                    'lun-path-use-state' => 'RDY',
    #                                                    'disk-port-name' => 'FC:A',
    #                                                    'initiator-side-switch-port' => 'vgbr300s152:11',
    #                                                    'initiator-port-speed' => '8',
    #                                                    'path-iops' => '374',
    #                                                    'initiator-lun-in-use-count' => '14',
    #                                                    'path-link-errors' => '0',
    #                                                    'path-quality' => '0',
    #                                                    'initiator-iops' => '374',
    #                                                    'initiator-port' => '0g',
    #                                                    'target-wwpn' => '50060e8006ff1d00',
    #                                                    'tpgn' => '1',
    #                                                    'lun-io-kbps' => '0',
    #                                                    'target-lun-in-use-count' => '14',
    #                                                    'initiator-io-kbps' => '31222',
    #                                                    'preferred-target-port' => 'false',
    #                                                    'disk-port' => 'A',
    #                                                    'disk-uid' => '46463144:30303031:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
    #                                                    'lun-iops' => '0'
    #                                                  }
    #                                                ]
    #                            }
    #                          ],
    #            'disk-inventory-info' => [
    #                                       {
    #                                         'disk-type' => 'LUN',
    #                                         'is-foreign' => 'false',
    #                                         'is-dynamically-qualified' => 'false',
    #                                         'model' => 'OPEN-V          ',
    #                                         'rpm' => '1',
    #                                         'capacity-sectors' => '1258291200',
    #                                         'checksum-compatibility' => 'block',
    #                                         'firmware-revision' => '7006',
    #                                         'import-in-progress' => 'false',
    #                                         'bytes-per-sector' => '512',
    #                                         'health-monitor-time-interval' => '0',
    #                                         'serial-number' => '50 0FF1D0001',
    #                                         'grown-defect-list-count' => '0',
    #                                         'right-size-sectors' => '1089994752',
    #                                         'vendor' => 'HP      ',
    #                                         'disk-uid' => '46463144:30303031:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
    #                                         'is-multidisk-carrier' => 'false'
    #                                       }
    #                                     ],
    #            'disk-ownership-info' => [
    #                                     {
    #                                       'is-failed' => 'false',
    #                                       'dr-home-node-id' => '4294967295',
    #                                       'reserved-by-node-id' => '536871919',
    #                                       'home-node-name' => 'vgv8040f132b',
    #                                       'owner-node-name' => 'vgv8040f132b',
    #                                       'home-node-id' => '536871919',
    #                                       'pool' => '0',
    #                                       'owner-node-id' => '536871919',
    #                                       'dr-home-node-name' => [
    #                                                              {}
    #                                                            ],
    #                                       'disk-uid' => '46463144:30303031:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000'
    #                                     }
    #                                   ],
    #            'node-name' => 'vgv8040f132a',
    #            'disk-raid-info' => [
    #                                {
    #                                  'position' => 'present',
    #                                  'container-type' => 'unknown',
    #                                  'disk-uid' => '46463144:30303031:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000'
    #                                }
    #                              ],
    #            'disk-name' => 'vgci9148s160:1-15.126L1',
    #            'disk-uid' => '46463144:30303031:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
    #            'disk-stats-info' => [
    #                                 {
    #                                   'sectors-written' => '0',
    #                                   'average-latency' => '0',
    #                                   'sectors-read' => '0',
    #                                   'bytes-per-sector' => '512',
    #                                   'disk-io-kbps' => '7',
    #                                   'path-error-count' => '0',
    #                                   'disk-iops' => '2',
    #                                   'power-on-time-interval' => '0',
    #                                   'disk-uid' => '46463144:30303031:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000'
    #                                 }
    #                               ]
    #          };
    
    my $disk_info_hash = undef;
    foreach my $temp_hash (@disk_info) {
        if ($temp_hash->{'disk-inventory-info'}->[0]->{'serial-number'} eq $disk_serial_param) {
            $disk_info_hash = $temp_hash;
            last;
        }
    }

    if (!defined $disk_info_hash) {
        $self->{error} = $ERR_STATUS;
        $self->{errmsg} = "Information of disk with serial number $disk_serial_param was not found in storage_disk_get_iter " .
                          "Here's the output hash :\n" . Dumper(@disk_info);
        return $ERR_STATUS;
    }


    # Fetching the onwer-id and checksum-compatibility beforehand to be validated later
    my $owner_id = $disk_info_hash->{'disk-ownership-info'}->[0]->{'owner-node-id'};
    my $checksum_result = $disk_info_hash->{'disk-inventory-info'}->[0]->{'checksum-compatibility'};

    if ($checksum_result eq 'zoned/advanced_zoned') {
        $checksum_result = 'advanced_zoned';
    };

    # Check that the owner-id tag was in the xml; if it wasn't then fail
    if (!defined($owner_id)) { 
        $self->{error} = $ERR_STATUS;
        $self->{errmsg} = "Disk with serial number $disk_serial_param did not have an owner-id tag in " . 
        "storage_disk_get_iter after calling disk-sanown-assign, unable to verify disk assign\n " . 
        " Here's the output :\n" . Dumper($disk_info_hash);
        return $ERR_STATUS;
    }

    # Check that the system id of this filer matches what was returned
    # by the XML.  The dynamic leading zeros is a problem that exists
    # in the output of sysconfig and most disk commands so we work
    # around that here with matching on any number of leading zeros.
    elsif ($self->{sys_id} !~ m/0*$owner_id/) {  
        $self->{error} = $ERR_STATUS;
        $self->{errmsg} = "Disk with serial number $disk_serial_param was assigned but the owner-id $owner_id " . 
        "doesn't appear to match this filer ($self->{sys_id}).";
        return $ERR_STATUS;
    }

    # If checksum was passed into this method we'll know here.  In the
    # case that it was then verify_checksum will be something besides
    # the empty string.  Then check that the returned xml did in fact
    # have the checksum tag and that it was set to what we expected.
    elsif ($verify_checksum ne "" && (!defined($checksum_result) || $verify_checksum ne $checksum_result)) {
        $self->{error} = $ERR_STATUS;
        $self->{errmsg} = "Disk with serial number $disk_serial_param was assigned but the specified checksum " . 
                          "($verify_checksum) doesn't appear to match the assigned checksum ($checksum_result).\n" .
                          " Here's the output :\n" . Dumper($disk_info_hash);
        return $ERR_STATUS;
    }

    # If we get here then the assign was successful and return good
    # status.
    else {
        return $GOOD_STATUS;
    }

}



## @name zapi_send_cmd($;%)
##
## @description
## A method that takes in the serial number of a disk to assign.  It then
## searches the internal disks hash of the object for the name of the disk
## that matches the serial number.  It is expected that the method
## discover_disks() has already been called to populate the hash.  It will
## then make the appropriate call through the zapi "disk-sanown-assign" api
## to assign the specified disk.  This method takes in an optional hash that
## is expected to be valid options that can be specified through the zapi
## "disk-sanown-assign" api.  Currently the only supported option is the
## word checksum which can be either block or zoned though no validation is
## done in the method, the interface handles the checking.
##
## @arg $disk_serial_param Required: serial number of the disk to be assigned
##
## @arg %options_hash Optional: hash of options supported by the
##                              disk-sanown-assign zapi api.
##
## @synopsis
## $return_hash_ref = $filer->zapi_disk_sanown_assign($disk_serial_param);
##
##
sub zapi_send_cmd($;%){
    my $self = shift;
    my ($zapi_api, $options_ref) = @_;
    my $result = "";
    my $return_string = "";
    my $print_string = "";

    # Invoke the server to run zapi commands at this filer.
    my $server = new ZapiTest($self->get_name());


    if(! defined($options_ref)){
        $result = $server->invoke($zapi_api);
        $return_string = $result->toString();
        $print_string = $result->sprint_pretty();

    }
    else{
        my %options = %$options_ref;

        my @option_keys = keys(%options);

        if(scalar(@option_keys) == 1){
            $result = $server->invoke($zapi_api, $option_keys[0], $options{$option_keys[0]});
        }
        elsif(scalar(@option_keys) == 2){
            $result = $server->invoke($zapi_api, $option_keys[0], $options{$option_keys[0]}, $option_keys[1], $options{$option_keys[1]});
        }
        elsif(scalar(@option_keys) == 3){
            $result = $server->invoke($zapi_api, $option_keys[0], $options{$option_keys[0]}, $option_keys[1], $options{$option_keys[1]}, $option_keys[2], $options{$option_keys[2]});
        }
        elsif(scalar(@option_keys) == 4){
            $result = $server->invoke($zapi_api, $option_keys[0], $options{$option_keys[0]}, $option_keys[1], $options{$option_keys[1]}, $option_keys[2], $options{$option_keys[2]}, $option_keys[3], $options{$option_keys[3]});
        }
        elsif(scalar(@option_keys) == 5){
            $result = $server->invoke($zapi_api, $option_keys[0], $options{$option_keys[0]}, $option_keys[1], $options{$option_keys[1]}, $option_keys[2], $options{$option_keys[2]}, $option_keys[3], $options{$option_keys[3]}, $option_keys[4], $options{$option_keys[4]});
        }
        elsif(scalar(@option_keys) == 6){
            $result = $server->invoke($zapi_api, $option_keys[0], $options{$option_keys[0]}, $option_keys[1], $options{$option_keys[1]}, $option_keys[2], $options{$option_keys[2]}, $option_keys[3], $options{$option_keys[3]}, $option_keys[4], $options{$option_keys[4]}, $option_keys[5], $options{$option_keys[5]});
        }
        elsif(scalar(@option_keys) == 7){
            $result = $server->invoke($zapi_api, $option_keys[0], $options{$option_keys[0]}, $option_keys[1], $options{$option_keys[1]}, $option_keys[2], $options{$option_keys[2]}, $option_keys[3], $options{$option_keys[3]}, $option_keys[4], $options{$option_keys[4]}, $option_keys[5], $options{$option_keys[5]}, $option_keys[6], $options{$option_keys[6]});
        }
        elsif(scalar(@option_keys) == 8){
            $result = $server->invoke($zapi_api, $option_keys[0], $options{$option_keys[0]}, $option_keys[1], $options{$option_keys[1]}, $option_keys[2], $options{$option_keys[2]}, $option_keys[3], $options{$option_keys[3]}, $option_keys[4], $options{$option_keys[4]}, $option_keys[5], $options{$option_keys[5]}, $option_keys[6], $options{$option_keys[6]}, $option_keys[7], $options{$option_keys[7]});
        }
        elsif(scalar(@option_keys) == 9){
            $result = $server->invoke($zapi_api, $option_keys[0], $options{$option_keys[0]}, $option_keys[1], $options{$option_keys[1]}, $option_keys[2], $options{$option_keys[2]}, $option_keys[3], $options{$option_keys[3]}, $option_keys[4], $options{$option_keys[4]}, $option_keys[5], $options{$option_keys[5]}, $option_keys[6], $options{$option_keys[6]}, $option_keys[7], $options{$option_keys[7]}, $option_keys[8], $options{$option_keys[8]});
        }
        elsif(scalar(@option_keys) >= 10){
            $result = $server->invoke($zapi_api, $option_keys[0], $options{$option_keys[0]}, $option_keys[1], $options{$option_keys[1]}, $option_keys[2], $options{$option_keys[2]}, $option_keys[3], $options{$option_keys[3]}, $option_keys[4], $options{$option_keys[4]}, $option_keys[5], $options{$option_keys[5]}, $option_keys[6], $options{$option_keys[6]}, $option_keys[7], $options{$option_keys[7]}, $option_keys[8], $options{$option_keys[8]}, $option_keys[9], $options{$option_keys[9]});
        }

        $return_string = $result->toString();
        $print_string = $result->sprint_pretty();


    }

    return($return_string, $print_string);
}






1; # END
