#$Id: //depot/prod/test/Rsierranevada/lib/VSeries/switch.pm#1 $
#$DateTime: 2013/01/22 11:22:46 $
#$Author: build $
#$Change: 1917575 $
#$Revision: #1 $
#
# Archana was the author before me...
#
#Copyright (c) 2002 Network Appliance, Inc.
#All rights reserved.
#
## @summary switch library functions to be used by NATE tests
##
## @description
##   These test functions implement features to support various types
##   of LAN and SAN (Fibre Channel (Brocade/McDATA), iSCSI, and LAN) switches
##
## @usage
##   use VSeries::switch;
##   use VSeries::switch qw(switch_port_map_get switch_port_enable);
##
## @keywords SAN FCP iSCSI Blocks switch
##
## @dependencies
## use config_base;
## use Tharn;
##
## @bug filer_port_map_get has not been implemented yet.
## @author ferrara@netapp.com, sajan@netapp.com (IAQ, Storage & RAID QA)
## @status released
## @change 2002-02-19 johnm: created
## @change 2002-02-27 johnm: changed function names and released
## @change 2002-03-06 johnm: removed -blind w/say function in switch_port_enable
## @change 2002-03-08 johnm: removed all -blind switches
## @change 2002-11-11 thomasm: allow for WWPNs without ':'
##                             allow port_num_get code to work on Brocade 3900
## @change 2003-04-14 thomasm: add support for McDATA
## @change 2003-06-19 johnm:   add Brocade support for Public-Loop/Multi-ID
## @change 2003-06-20 thomasm: add switchShow for debug reasons
## @change 2004-12-23 sajan:   add new functions
## @change 2005-06-01 archana: Updated author tag
## @change 2007-12-07 silva: Added cisco support for switch_name_get

package VSeries::switch;

use strict;
use vars ('@ISA','@EXPORT');

require Exporter;
@ISA = ('Exporter');
@EXPORT = qw(
         switch_zone_enable
         switch_zone_disable
         switch_zone_enabled_get
         switch_zone_all_get
         switch_port_map_get
         switch_port_num_get
         switch_port_enable
         switch_port_disable
         switch_port_logo
         switch_disable
         switch_enable
         switch_state_get
         switch_logout
         switch_name_set
         switch_name_get
         switch_reboot
         switch_fastboot
         switch_eport_get
         switch_eport_num_get
         switch_hw_sw_info_get
         );

use config_base;
use Tharn;

## @name ::switch_port_num_get
## @description
## Returns the port number for the provided WWPN, else `undef' if not found
##
## @arg $conn Required: The switch connection to send this operation over
## @arg $wwpn Required: The WWPN to search for
##
## @synopsis
## switch_port_num_get {$conn, $wwpn}
##
## @example
## CLIENT_INIT2_PORT1_WWN=00:01:02:03:04:05:06:07
## SWITCH=brcd2
## param("CLIENT_INIT2_PORT1_WWN", -mesg, "WWPN of Client HBA 2, Port 1");
## param(SWITCH, -mesg, "hostspec of SWITCH");
## $sa = host($SWITCH);
## $sc = connect("#telnet");
## my($port) = switch_port_num_get($sc, $CLIENT_INIT2_PORT1_WWN);
## if (!defined($port)){ exit 1; } # Port not there;
##
sub switch_port_num_get($$) {
   my ($conn, $wwpn) = @_;

   my ($port, $version, @info, $director);
   my ($b_port, $b_port_nc, $line, @nodes, $n_wwn, $n_wwn_nc);
   my ($match, $before, $after, $host_type, $node);

   if (!defined($conn) || !defined($wwpn)) {
       return 0;
   }

   &tstlog('-comment', "Looking for port $wwpn");

   chomp($wwpn);
   my $wwpnre = quotemeta($wwpn);
      $wwpnre = lc($wwpnre);


   # determine switch type and method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       # check to see if this is a director switch and hence and
       # operate in loop mode.
       ($match,$before,$after) = say($conn, "show system", '-timeout', 30);
       if ($after =~ /director/i) {
           $director = 1;
       } else {
           $director = 0;
       }

       # get port status from the switch
       ($match,$before,$after) = say($conn, "show port status", '-timeout', 30);

       my (@info) = split(/\r\n/, $after);

       foreach $line (@info) {
          if ($line =~ /Online/i) {
              # check to see if line contains MULTIPLE. If it is
              # the port is in loop mode and we need to get the port in another way
              if ($line =~ /MULTIPLE/i and $director == 0) {
                  $port = (split(/\s+/, $line))[0];
                  ($match,$before,$after) = say($conn, "show port nodes $port",
                                                '-timeout', 30);
                  @nodes = split(/\r\n/, $after);
                  # check the WWN returned in show port nodes looking for
                  # the WWPN from the client.
                  foreach $node (@nodes) {
                      if ($node =~ /:/) {
                          $n_wwn = (split(/\s+/, $node))[1];
                          $n_wwn = lc($n_wwn);
                          $n_wwn_nc = $n_wwn;
                          $n_wwn_nc =~ s/://g;
                          if ($n_wwn =~ /$wwpnre/ or
                              $n_wwn_nc =~ /$wwpnre/) {
                              return $port;
                          }
                      }
                  }
              } else {
                  $b_port = (split(/\s+/, $line))[3];
                  $b_port = lc($b_port);
                  $b_port_nc = lc($b_port);
                  $b_port_nc =~ s/://g;

                  # check for syntax with or without ':'
                  if ($b_port =~ /$wwpnre/ or
                      $b_port_nc =~ /$wwpnre/) {
                      $port = (split(/\s+/, $line))[0];
                      return $port;
                  }
              }
          }
       }

       foreach my $line (@info) {
          if ($line =~ /^(\d+)\s+Online.*$wwpnre.*/) {
               ($port,undef) = split(/\s+/, $line);
               return $port;
           }
       }
   } elsif ($host_type =~ /switch-cisco/i) {
        # set terminal length to 0 which disables the PAGER
    ($match,$before,$after) = say($conn, "terminal length 0",
                                      '-timeout', 30, -swallow);
    ($match,$before,$after) = say($conn, "show flogi database" ,
                                      '-timeout', 30, -swallow);
    my (@info) = split(/\r\n/, $after);

    foreach $line (@info)  {
            $b_port = (split(/\s+/, $line))[3];
            $b_port = lc($b_port);
            $b_port_nc = lc($b_port);
            $b_port_nc =~ s/://g;

            #check for wwpn with or without ':'
            if ($b_port =~ /$wwpnre/ or
                $b_port_nc =~ /$wwpnre/) {
                $port = (split(/\s+/, $line))[0];
                # remove the fc portion of the fc<1>/port_num
                $port =~ s/fc//;
                return $port;
            }
    }
    } else {
       # Brocade case
       ($match, $before, $after) = say($conn, "version", '-timeout', 30);

       # check for v3 or v4 switch
       if ($after =~ /Fabric OS:\s*v3/) {
           $version = "v3";
       } else {
           $version = "v4";
       }

       # run switchShow to get the output in the log to see the state of
       # the switch before the block. It helps with debug
       ($match, $before, $after) = say($conn, "switchShow", '-timeout', 30);

       # Use nsShow to support Multi-ID/Public loop, switchShow doesn't work.
       ($match,$before,$after) = say($conn, "nsshow", '-timeout', 30);

       # need to handle a V3 OS differently can V4
       if ($version eq "v3") {
          @info = split(/\n\r/, $after);
       } else {
          @info = split(/\r\n/, $after);
       }

       foreach $line (@info) {
       next unless ($line =~ /^\sN\s+|^\sNL\s+/);

       my ($node, $pname) = (split(/;/, $line))[0,2];

       $b_port = lc($pname);
       $b_port_nc = lc($b_port);
       $b_port_nc =~ s/://g;

       # check for syntax with or without ':'
       next unless ($b_port =~ /$wwpnre/ or
            $b_port_nc =~ /$wwpnre/);

       ($port) = (split(/\s+/, $node))[2];

       if ($version eq "v3") {
           ($port) = $port =~ /...(.)../;
       } else {
           ($port) = $port =~ /..(..)../;
       }

       $port = hex($port);

       return ($port);
       }
   }

   # no matches
   return undef;

}

## @name ::switch_port_enable
## @description Enable port on switch
##
## @synopsis
## switch_port_enable{$conn, $port}
##
## @arg $conn Required: The switch connection to send this operation over
## @arg $port Required: The switch port number to enable
##
## @example
## if (!switch_port_enable($sc, $port=1)) {exit 1; } # Port not enabled or error
##
sub switch_port_enable($$) {
   my($conn, $port) = @_;

   my ($host_type);

   if ((not defined($port)) or (not defined($conn))) {
       return 0;
   }

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       say($conn, "config port blocked $port false");
   } else {
       if ($host_type eq "switch-dell") {
           say($conn, "configure");
           say($conn, "interface ethernet 1/$port");
           say($conn, "no shutdown");
           say($conn, "exit");
           say($conn, "exit");
       } else {
       if ($host_type =~ /switch-cisco/i) {
        my ($cisco_prompt);
        say($conn, "conf t", -noprompt,  "-wait", "#");
        say($conn, "int fc $port", -noprompt, "-wait", "#");
        say($conn, "no shutdown", -noprompt, "-wait", "#");
        say($conn, "end", -noprompt, "-wait", "#");
           }
       }
       if ($host_type =~ /switch-brocade/i) {
           # brocade support
           say($conn, "portenable $port");
       }
   }

   return 1;

}

## @name ::switch_port_disable
## @description Disable port on switch
## @synopsis
## switch_port_disable{$conn, $port}
##
## @arg $conn Required: The switch connection to send this operation over
## @arg $port Required: The switch port number to disable
##
## @example
## if (!switch_port_disable($sc, $port=1)) {exit 1; } # Port not disabled or error
sub switch_port_disable($$) {
   my($conn, $port) = @_;

   my ($host_type);

   if ((not defined($port)) or (not defined($conn))) {
       return 0;
   }

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       say($conn, "config port blocked $port true");
   } else {
       if ($host_type eq "switch-dell") {
           say($conn, "configure");
           say($conn, "interface ethernet 1/$port");
           say($conn, "shutdown");
           say($conn, "exit");
           say($conn, "exit");
       } else {
           if ($host_type =~ /switch-cisco/i) {
                my ($cisco_prompt);
                say($conn, "conf t", -noprompt,  "-wait", "#");
                say($conn, "int fc $port", -noprompt, "-wait", "#");
                say($conn, "shutdown", -noprompt, "-wait", "#");
                say($conn, "end", -noprompt, "-wait", "#");
           }
       }
       if ($host_type =~ /switch-brocade/i) {
           # brocade support
           say($conn, "portdisable $port");
       }
   }

   return 1;

}

## @name ::switch_port_logo
## @description Disable port number on switch
## @synopsis
## switch_port_logo{$sc, $port}
##
## @arg $conn Required: The switch connection to send this operation over
## @arg $port Required: The switch port number to logout
##
## @example
## if (!switch_port_logo($sc, $port=1)) {exit 1; } # Error
sub switch_port_logo($$) {
   my($conn, $port) = @_;

   my ($host_type);

   if ((not defined($port)) or (not defined($conn))) {
       return 0;
   }

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       say($conn, "portdisable $port", '-timeout', 5);
       say($conn, "portenable $port", '-timeout', 5);
   } else {
       if ($host_type eq "switch-dell") {
           say($conn, "configure");
           say($conn, "interface ethernet 1/$port");
           say($conn, "shutdown");
           say($conn, "no shutdown");
           say($conn, "exit");
           say($conn, "exit");
       } else {
           # Brocade case
           say($conn, "portdisable $port", '-timeout', 5);
           say($conn, "portenable $port", '-timeout', 5);
       }
   }

   return 1;
}

## @name ::switch_disable
## @description Disable whole switch (soft reset/powerdown)
## @synopsis
## switch_disable{$conn}
## @arg $conn Required: The switch connection to send this operation over
## @example
## if (!switch_disable($sc)) {exit 1; } # Error
sub switch_disable($) {
   my($conn) = @_;

   my ($host_type);

   if (not (defined($conn))) {
       return 0;
   }
   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       say($conn, "maint system setOnlineState 0");
   } else {
       # Brocade case
       say($conn, "switchdisable");
   }

   sleep(1);

   return 1;

}

## @name ::switch_enable
## @description Enable switch (warm reboot)
## @synopsis
##  switch_enable {$conn}
##
## @arg $conn Required: The switch connection to send this operation over
## @example
## if (!switch_enable($sc)) { exit 1; } # Error
sub switch_enable($) {
   my($conn) = @_;

   my ($host_type);

   if (not (defined($conn))) {
       return 0;
   }

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       say($conn, "maint system setOnlineState 1");
   } else {
       # Brocade case
       say($conn, "switchenable");
   }

   sleep(10);

   return 1;

}

## @name ::switch_zone_enable
## @description Enable zone {alias} on switch {name}
## @synopsis
## switch_zone_enable {$conn, $zone}
## @arg $conn Required: The switch connection to send this operation over
## @arg $zone Required: The zone configuration name to enable
## @example
## if (!switch_zone_enable($sc, $zone=config1)) { exit 1; } # Error
sub switch_zone_enable ($$) {
   my ($conn, $zone) = @_;

   my ($host_type);

   if ((not (defined($conn))) or (not (defined($zone)))) {
       return 0;
   }

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       say($conn, "config zoning activateZoneSet \"$zone\"", '-timeout', 10);
   } else {
       # Brocade case
       say($conn, "cfgenable \"$zone\"", '-timeout', 10);
   }

   sleep(2);

   return 1;

}

## @name ::switch_zone_disable
## @description Disable zone alias on switch
## @synopsis
## switch_zone_disable {$conn, $zone}
## @arg $conn Required: The switch connection to send this operation over
## @arg $zone Required: The zone configuration name to disable, if NULL then all zone configurations are disabled
## @example
## if (!switch_zone_disable($sc, $zone=config1)) { exit 1; } # Error
## switch_zone_disable($sc, 0); # Disable all zone configurations.
sub switch_zone_disable($$) {
   my ($conn, $zone) = @_;

   my ($host_type);

   if ((not (defined($conn))) or (not (defined($zone)))) {
       return 0;
   }

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       if (not $zone) {
           if (defined($zone = switch_zone_enabled_get($conn))) {
               say($conn, "config zoning deactivateZoneSet \"$zone\" ",
                   '-timeout', 10);
           }
       } else {
           say($conn, "config zoning deactivateZoneSet \"$zone\" ",
               '-timeout', 10);
       }
       sleep(2);

       if (defined($zone = switch_zone_enabled_get($conn))) {
           return 0;
       }
   } else {
       # Brocade case
       if (not $zone) {
           if (defined($zone = switch_zone_enabled_get($conn))) {
               say($conn, "cfgdisable \"$zone\" ", '-timeout', 10);
           }
       } else {
           say($conn, "cfgdisable \"$zone\" ", '-timeout', 10);
       }
       sleep(2);

       if (defined($zone = switch_zone_enabled_get($conn))) {
           return 0;
       }
   }

   return 1;
}

## @name ::switch_zone_enabled_get
## @description Returns the name of the zone configuration currently enabled on the switch, else `undef' if none.
## @synopsis
## switch_zone_enabled_get {$conn}
## @arg $conn Required: The switch connection to send this operation over
## @example
## my ($zone) = switch_zone_enabled_get($sc);
## if (!defined($zone)) { print "No zone configuration enabled!\n";

sub switch_zone_enabled_get($) {
   my ($conn) = @_;

   my ($index, $i);
   my ($line, $cfg);
   my ($match, $before, $after);
   my ($host_type);

   if (not (defined($conn))) {
       return undef;
   }

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       ($match, $before, $after) = say($conn, "show zoning", '-timeout', 10);
       my(@info) = split(/\n\r/, $after);

       foreach $line (@info) {
           if ($line =~ /Zone:/) {
               $cfg = (split(/\s+/, $line))[2];
           }
       }
   } else {
       # Brocade case
       ($match,$before,$after) = say($conn, "cfgshow", '-timeout', 10);
       my(@info) = split(/\n\r/, $after);

       for ($index=0; $index<@info; $index++) {
           $line = $info[$index];
           if ($line =~ /^Effective configuration/) {
             for ($i=$index; $i<@info; $i++) {
               $line = $info[$i];
               if ($line =~ /^\s*cfg:\s+(\S+)\s/) {
               $cfg = $1;
               }
             }
           }
       }
   }

   return ($cfg);
}

## @name ::switch_zone_all_get
## @description Returns an array of the valid zone configuration aliases defined on this switch, else `undef' if no zones are configured
## @synopsis
## switch_zone_all_get {$conn}
## if (!defined(@zones)) { print "No zone configurations found!\n";
## @arg $conn Required: The switch connection to send this operation over
## @example
## my (@zones) = switch_zone_all_get($sc);

sub switch_zone_all_get($) {
   my ($conn) = @_;

   my ($index) = 0;
   my (@cfgs, $cfg);
   my ($match, $before, $after);
   my ($host_type, $line);

   if (not (defined($conn))) {
       return undef;
   }

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       ($match, $before, $after) = say($conn, "show zoning", '-timeout', 10);
       my(@info) = split(/\n\r/, $after);

       foreach $line (@info) {
           if ($line =~ /Zone:/) {
               $cfg = (split(/\s+/, $line))[2];
               push(@cfgs, $cfg);
           }
       }
   } else {
       # Brocade case
       ($match, $before, $after) = say($conn, "cfgshow", '-timeout', 5);

       my(@info) = split(/\n\r/, $after);

       foreach my $line (@info) {
           if ($line =~ /^\s*cfg:\s+(\S+)\s/) {
           $cfgs[$index++] = $1;
           } elsif ($line =~ /^Effective configuration/) {
               last;
           }
       }
   }

   return (@cfgs);
}

## @name ::switch_port_map_get
## @description Returns the current portmap from the switch
## @arg $conn Required: The switch connection to send this operation over
##
sub switch_port_map_get($) {
   my($conn) = shift;

   exit &EXIT_NOT_APPLICABLE;

}

## @name ::switch_state_get
## @description Get the state of the Switch
## @synopsis
## $state = switch_state_get{$conn}
## @arg $conn Required: The switch connection to send this operation over
## @example
## $state = switch_state_get($sc);
sub switch_state_get($) {
   my($conn) = @_;

   my ($match, $before, $after);
   my (@info, $line, $version, $state);
   my ($host_type);

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       # run switchShow to get the state of the switch
       ($match, $before, $after) = say($conn, "show switch", '-timeout', 30);
       @info = split(/\n\r/, $after);
       $state = undef;
       foreach $line (@info) {
           if ($line =~ /State:/) {
               $state = (split(/\s+/, $line))[1];
               return($state);
           }
       }
   } else {
       # Brocade case
       ($match, $before, $after) = say($conn, "version", '-timeout', 30);

       # check for v3 or v4 switch
       if ($after =~ /Fabric OS:\s*v3/) {
           $version = "v3";
       } else {
           $version = "v4";
       }

       # run switchShow to get the state of the switch
       ($match, $before, $after) = say($conn, "switchshow", '-timeout', 30);

       # need to handle a V3 OS differently can V4
       if ($version eq "v3") {
          @info = split(/\n\r/, $after);
       } else {
          @info = split(/\r\n/, $after);
       }

       $state = undef;
       foreach $line (@info) {
          if ($line =~ /switchState:/i) {
              $state = (split(/\s+/, $line))[1];
              return($state);
          }
       }
   }

   return ($state);

}

## @name ::switch_logout
## @description Logout of the Switch
## @synopsis
## $state = switch_logout{$conn}
## @arg $conn Required: The switch connection to send this operation over
## @example
## $state = switch_logout($sc);
sub switch_logout($) {
   my($conn) = @_;

   my ($match, $before, $after);
   my ($host_type);

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       # logout of the CLI
       ($match, $before, $after) = say($conn, "logout", '-blind',
                                       '-timeout', 30);
   } else {
       # logout of the CLI
       ($match, $before, $after) = say($conn, "exit", '-blind', '-timeout', 30);
   }

}

# @name ::switch_name_set
## @description Set the name the Switch
## @synopsis
## switch_name_set{$conn, $name}
## @arg $conn Required: The switch connection to send this operation over
## @arg $name Required: New name of the switch
## @example
## switch_name_set($sc, $name);
sub switch_name_set($$) {
   my($conn, $name) = @_;
   my ($temp, $user_id);
   my ($host_type);
   my @brocade_v5_prompts = ();

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       Tharn::say($conn, "config system name $name", '-timeout' , 30);
   } elsif ($host_type =~ /switch-cisco/i) {
       Tharn::say($conn, "config t", -noprompt, '-timeout', 30);
       Tharn::say($conn, "switchname $name", -noprompt, '-timeout', 30);
       Tharn::say($conn, "end", -noprompt, '-timeout', 30);
   } elsif ($host_type =~ /switch-brocade/i) {
       ($temp, $user_id) = split(/:/, $conn->{hostp}->{prompt_norm}[0]);

       # need to get some switch info (model, vendor, firmware)
       # incase we need to do different stuff for different levels
       # of firmware or brocade switch models yada yada yada...
       my %switch_info = switch_hw_sw_info_get($conn);

       # if its firmware v5 (like b200e's and 3850's) we can't be
       # sensitive to the prompt to wait for because name changes
       # don't get reflected instantly they require a logout first.
       # see burt 242549
       if (($switch_info{"firmware"} =~ /v5/i) || ($switch_info{"firmware"} =~ /v6/i) || ($switch_info{"firmware"} =~ /v7/i)) {
           @brocade_v5_prompts = $conn->prompts();
           push(@brocade_v5_prompts, "${name}:${user_id}");

           Tharn::say($conn, "switchname \"$name\"", '-prompt', @brocade_v5_prompts, '-timeout', 30);

       } else {
           Tharn::say($conn, "switchname \"$name\"", -noprompt, -prompt,
           "${name}:${user_id}", '-timeout', 30);
       }
   }

}

## @name ::switch_name_get
## @description Get the name the Switch
## @synopsis
## $name = switch_name_get{$conn}
## @arg $conn Required: The switch connection to send this operation over
## @example
## $name = switch_name_get($sc);
sub switch_name_get($) {
   my($conn) = @_;

   my ($match, $before, $after);
   my (@info, $line, $version, $name);
   my ($host_type);

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       (undef, undef, $after) = say($conn, "config system show",
                                    '-timeout' , 30);
       @info = split(/\n\r/, $after);
       $name = undef;
       foreach $line (@info) {
           if ($line =~ /Name:/) {
               $name = (split(/\s+/, $line))[1];
               tstlog ( -comment, "The switch name is: $name\n");
               return($name);
           }
       }
   } elsif ($host_type =~ /switch-cisco/i) {
        # - - - - - - -  - - - - - - - - - - - - - -
        #>>vns-c9124-3#telnet|show switchname\r
        #<<vns-c9124-3#telnet|show switchname\r\r\n
        #<<vns-c9124-3#telnet|vns-c9124-3\r\n
        #<<vns-c9124-3#telnet|\rvns-c9124-3# 
        # - - - - - - -  - - - - - - - - - - - - - -
        
        (undef, undef, $after) = say($conn, "show switchname", '-timeout', 30);
        @info = split(/\r\n/,$after);
        $name = $info[1];
        
        return($name);

   } elsif ($host_type =~ /switch-brocade/i) {
       # check for v3 or v4 switch
       (undef, undef, $after) = say($conn, "version", '-timeout', 30);
       if ($after =~ /Fabric OS:\s*v3/) {
           $version = "v3";
       } else {
           $version = "v4";
       }
       (undef, undef, $after) = say($conn, "switchname", '-timeout', 30);
       # need to handle a V3 OS differently can V4
       if ($version eq "v3") {
          @info = split(/\n\r/, $after);
       } else {
          @info = split(/\r\n/, $after);
       }
       $name = undef;
       chomp($info[1]);
       $name = $info[1];
       return($name);
   }

}

## @name ::switch_reboot
## @description Reboot the switch, works only for cisco and brocade
## @synopsis
## switch_reboot{$conn}
## @arg $conn Required: The switch connection to send this operation over
## @example
## switch_reboot($sc);
sub switch_reboot($) {
   my($conn) = @_;

   my ($host_type);

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};

   if ($host_type =~ /switch-mcdata/i) {
       #say($conn, "maint system setOnlineState 0");
       #say($conn, "maint system setOnlineState 1");
   } elsif ($host_type =~ /switch-cisco/i) {
       my $CONF_PROMPT = "This command will reboot the system.";
       say($conn, "reload", '-prompt', $CONF_PROMPT, '-timeout', 30);
       say($conn, "y", '-noprompt', '-prompt', "y", '-timeout', 30);
   } elsif ($host_type =~ /switch-brocade/i) {
       my $conf_prompt = "Are you sure you want to reboot the switch";
       Tharn::say($conn, "reboot", '-prompt', $conf_prompt, '-timeout', 30);
       # Send "y" blindly in case the switch FW prompts for confirmation.
       # If not, no harm done.
       Tharn::say($conn, "y", '-blind');
   }
}

## @name ::switch_fastboot
## @description fastboot the switch, works only for brocade
## @synopsis
## switch_fastboot{$conn}
## @arg $conn Required: The switch connection to send this operation over
## @example
## switch_fastboot($sc);
sub switch_fastboot($) {
   my($conn) = @_;

   my ($host_type);

   # get host type and determine proper method
   $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-mcdata/i) {
       #say($conn, "maint system setOnlineState 0");
       #say($conn, "maint system setOnlineState 1");
   } elsif ($host_type =~ /switch-cisco/i) {
       ######
   } elsif ($host_type =~ /switch-brocade/i) {
       Tharn::say($conn, "fastboot", '-noprompt', '-prompt', "fastboot", '-timeout', 30);
       # Send "y" blindly in case the switch FW prompts for confirmation.
       # If not, no harm done.
       Tharn::say($conn, "y", '-blind');
   }
}

## @name ::switch_eport_get
## @description Returns array of hash having port number,  its WWN and
## peer's WWN
##
## @synopsis
## switch_eport_get$conn}
##
## @arg $conn Required: The switch connection to send this operation over
##
sub switch_eport_get ($) {
    my ($conn) = @_;

    my ($after, @info, $line, $length_port_name);
    my @port = ();
    my (@port_WWN, @peer_port_WWN);
    my $version;
    my @eportlist = ();
    my $hosttype = $conn->{hostp}->{hosttype};

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # Cisco switch
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if ($hosttype =~ /switch-cisco/i) {
        say($conn, "terminal length 0", -swallow, '-timeout', 60);
        (undef, undef, $after) = say($conn, "show interface brief",
                                                  -swallow, '-timeout', 60);
        my (@info) = split(/\r\n/, $after);
        my $i = 0;
        foreach $line (@info)  {
            my $port_type = (split (/\s+/, $line))[6];
            next unless (($port_type eq "E") || ($port_type eq "TE"));
            ($port[$i]) = (split (/\s+/, $line))[0];
            $length_port_name = length($port[$i]);
            $port[$i] = substr($port[$i], 2, $length_port_name);
            ++$i;
        }

        $i = 0;
        foreach my $port (@port) {
            say($conn, "terminal length 0", -swallow, '-timeout', 60);
            (undef, undef, $after) = say($conn, "show interface fc $port",
                                                  -swallow, '-timeout', 60);
            my (@info) = split(/\r\n/, $after);
            foreach $line (@info)  {
                next unless ($line =~ "Port WWN is");
                $line =~ s/^\s+//g;
                ($port_WWN[$i]) = (split (/\s+/, $line))[3];
            }

            say($conn, "terminal length 0", -swallow, '-timeout', 60);
            (undef, undef, $after) = say($conn, "show interface fc $port",
                                                  -swallow, '-timeout', 60);
            my (@info) = split(/\r\n/, $after);
            foreach $line (@info)  {
                next unless ($line =~ "Peer port WWN is");
                $line =~ s/^\s+//g;
                ($peer_port_WWN[$i]) = (split (/\s+/, $line))[4];
            }
            my %eport_info = ();
            $eport_info{"port"} = $port[$i];
            $eport_info{"port_WWN"} = $port_WWN[$i];
            $eport_info{"peer_port_WWN"} = $peer_port_WWN[$i];
            ++$i;
            push (@eportlist, \%eport_info);
         }
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # Brocade switch
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    } elsif ($hosttype =~ /switch-brocade/i) {
        (undef, undef, $after) = say($conn, "version", -swallow,
                                                       '-timeout', 60);
        # check for v2 & v3 or v4 switch
        if ($after =~ /Fabric OS:\s*v4/) {
            $version = "v4";
        } else {
            $version = "v3";
        }

        (undef, undef, $after) = say($conn, "switchshow", -swallow,
                                                        '-timeout', 60);
        # need to handle a V3 OS differently can V4
        if ($version eq "v3") {
           @info = split(/\n\r/, $after);
        } else {
           @info = split(/\r\n/, $after);
        }

        my $i = 0;
        foreach $line (@info) {
           next unless ($line =~ "E-Port");
           $line =~ s/^\s+//g;
           if ($version eq "v3") {
               ($port[$i]) = (split (/\s+/, $line))[1];
               chop($port[$i]);
           } else {
               ($port[$i]) = (split (/\s+/, $line))[0];
           }
           ++$i;
        }

        my $j = 0;
        foreach my $port (@port) {
            (undef, undef, $after) = say($conn, "portshow $port",
                                            -swallow, '-timeout', 60);
            # need to handle a V3 OS differently can V4
            if ($version eq "v3") {
               @info = split(/\n\r/, $after);
            } else {
               @info = split(/\r\n/, $after);
            }
            foreach $line (@info) {
                next unless ($line =~ /portWwn:/i);
                $line =~ s/^\s+//g;
                ($port_WWN[$j]) = (split (/\s+/, $line))[1];
            }

            for (my $i=0; $i<@info; ++$i) {
                $line = undef;
                $line = $info[$i];
                next unless ($line =~ /connected:/i);
                if ($version eq "v4") {
                    $line = undef;
                    $line = $info[$i + 1];
                    $line =~ s/^\s+//g;
                    ($peer_port_WWN[$j]) = (split (/\s+/, $line))[0];
                } else {
                    $line =~ s/^\s+//g;
                    ($peer_port_WWN[$j]) = (split (/\s+/, $line))[4];
                }
            }
            my %eport_info = ();
            $eport_info{"port"} = $port[$j];
            $eport_info{"port_WWN"} = $port_WWN[$j];
            $eport_info{"peer_port_WWN"} = $peer_port_WWN[$j];
            ++$j;
            push (@eportlist, \%eport_info);
        }
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # Mcdata switch
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    } elsif ($hosttype =~ /switch-mcdata/i) {
        my $director;
        my $peer_switch_wwn;
        (undef, undef, $after) = say($conn, "show system", -swallow,
                                                      '-timeout', 60);
        if ($after =~ /director/i) {
            $director = 1;
        } else {
            $director = 0;
        }

        (undef, undef, $after) = say($conn, "show port status", -swallow,
                                                      '-timeout', 60);
        @info = split(/\r\n/, $after);
        my $i = 0;
        foreach $line (@info) {
            next unless (($line =~ /Online/) && ($line =~ /ePort/));
            $port[$i] = (split(/\s+/, $line))[0];
            ++$i;
        }

        $i = 0;
        foreach my $port (@port) {
            (undef, undef, $after) = say($conn, "show port info", -swallow,
                                                     '-timeout', 60);
            @info = split(/\r\n/, $after);
            $line = $info[$port + 2];
            $port_WWN[$i] = (split (/\s+/, $line))[1];
            (undef, undef, $after) = say($conn, "show switch", -swallow,
                                                     '-timeout', 60);
            @info = split(/\r\n/, $after);
            foreach $line (@info) {
                next unless ($line =~ /World Wide Name:/);
                $peer_port_WWN[$i] = (split(/\s+/, $line))[3]; #my switch WWN
            }
            my %eport_info = ();
            $eport_info{"port"} = $port[$i];
            $eport_info{"port_WWN"} = $port_WWN[$i];
            $eport_info{"peer_port_WWN"} = $peer_port_WWN[$i];
            ++$i;
            push (@eportlist, \%eport_info);
        }
    }

    return (@eportlist);
}

## @name ::switch_eport_num_get
## @description Returns E-port number for a given peer's WWN, undef otherwise
##
## @synopsis
## switch_eport_get$conn, $wwpn}
##
## @arg $conn Required: The switch connection to send this operation over
## @arg $wwpn Required: WWPN of the other side
## @arg $entry_number Required: ISL link number, to take care of multple
##      ISL links (needed onlt for mcdata switch, for any other switches
##      this parameter is ignored)
##
sub switch_eport_num_get ($$$) {
    my ($conn, $wwpn, $entry_number) = @_;

    my ($after, @info, $line, $length_port_name);
    my @port = ();
    my @port_WWN = ();
    my $port = undef;
    my $version;
    my ($port_WWN, $peer_port_WWN);
    my $hosttype = $conn->{hostp}->{hosttype};

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # Cisco switch
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if ($hosttype =~ /switch-cisco/i) {
        say($conn, "terminal length 0", -swallow, '-timeout', 60);
        (undef, undef, $after) = say($conn, "show interface brief",
                                          -swallow, '-timeout', 60);
        my (@info) = split(/\r\n/, $after);
        my $i = 0;
        foreach $line (@info)  {
            my $port_type = (split (/\s+/, $line))[6];
            next unless (($port_type eq "E") || ($port_type eq "TE"));
            ($port[$i]) = (split (/\s+/, $line))[0];
            $length_port_name = length($port[$i]);
            $port[$i] = substr($port[$i], 2, $length_port_name);
            ++$i;
        }

        foreach my $port_temp (@port) {
            say($conn, "terminal length 0", -swallow, '-timeout', 60);
            (undef, undef, $after) = say($conn, "show interface fc $port_temp",
                                          -swallow, '-timeout', 60);
            my (@info) = split(/\r\n/, $after);
            foreach $line (@info)  {
                next unless ($line =~ "Port WWN is");
                $line =~ s/^\s+//g;
                ($port_WWN) = (split (/\s+/, $line))[3];
            }

            say($conn, "terminal length 0", -swallow, '-timeout', 60);
            (undef, undef, $after) = say($conn, "show interface fc $port_temp",
                                          -swallow, '-timeout', 60);
            my (@info) = split(/\r\n/, $after);
            foreach $line (@info)  {
                next unless ($line =~ "Peer port WWN is");
                $line =~ s/^\s+//g;
                ($peer_port_WWN) = (split (/\s+/, $line))[4];
            }
            if ($port_WWN eq $wwpn) {
                return ($port_temp);
            }
        }
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # Brocade switch
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    } elsif ($hosttype =~ /switch-brocade/i) {
        (undef, undef, $after) = say($conn, "version", -swallow,
                                                    '-timeout', 60);
        # check for v2 & v3 or v4 switch
        if ($after =~ /Fabric OS:\s*v4/) {
            $version = "v4";
        } else {
            $version = "v3";
        }

        (undef, undef, $after) = say($conn, "switchshow", -swallow,
                                                    '-timeout', 60);
        # need to handle a V3 OS differently can V4
        if ($version eq "v3") {
           @info = split(/\n\r/, $after);
        } else {
           @info = split(/\r\n/, $after);
        }

        my $i = 0;
        foreach $line (@info) {
           next unless ($line =~ "E-Port");
           $line =~ s/^\s+//g;
           if ($version eq "v3") {
               ($port[$i]) = (split (/\s+/, $line))[1];
               chop($port[$i]);
           } else {
               ($port[$i]) = (split (/\s+/, $line))[0];
           }
           ++$i;
        }
        foreach my $port_temp (@port) {
            (undef, undef, $after) = say($conn, "portshow $port_temp",
                                         -swallow, '-timeout', 60);
            # need to handle a V3 OS differently can V4
            if ($version eq "v3") {
               @info = split(/\n\r/, $after);
            } else {
               @info = split(/\r\n/, $after);
            }
            foreach $line (@info) {
                next unless ($line =~ /portWwn:/i);
                $line =~ s/^\s+//g;
                ($port_WWN) = (split (/\s+/, $line))[1];
            }

            if ($port_WWN eq $wwpn) {
                return ($port_temp);
            }
        }
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # Mcdata switch
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    } elsif ($hosttype =~ /switch-mcdata/i) {
        my $director;
        my $peer_switch_wwn;
        (undef, undef, $after) = say($conn, "show system", -swallow,
                                                  '-timeout', 60);
        if ($after =~ /director/i) {
            $director = 1;
        } else {
            $director = 0;
        }

        (undef, undef, $after) = say($conn, "show port status",
                                       -swallow, '-timeout', 60);
        @info = split(/\r\n/, $after);
        my $i = 0;
        foreach $line (@info) {
            next unless (($line =~ /Online/) && ($line =~ /ePort/));
            $port[$i] = (split(/\s+/, $line))[0];
            $port_WWN[$i] = (split(/\s+/, $line))[3];
            ++$i;
        }

        for (my $i=0; $i < @port; ++$i) {
            if (($port_WWN[$i] eq $wwpn) && ($entry_number == $i + 1)) {
                return ($port[$i]);
            }
        }
    }
    return (undef);
}

## @name ::switch_hw_sw_info_get
## @description Returns hash having following info
##      "vendor" => <name>
##      "model" => <value>
##      "firmware" => <value>
## @synopsis
## switch_hw_sw_info_get($conn}
##
## @arg $conn Required: The switch connection to send this operation over
##
sub switch_hw_sw_info_get ($) {
    my ($conn) = @_;

    my ($after, @info, $line, $length_port_name);
    my $vendor = undef;
    my $model = undef;
    my $firmware = undef;
    my (%switch_hw_sw_info);

    my $hosttype = $conn->{hostp}->{hosttype};
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # Cisco switch
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if ($hosttype =~ /switch-cisco/i) {
        $vendor = 'Cisco';
        chomp($vendor);
        say($conn, "terminal length 0", -swallow, '-timeout', 60);
        (undef, undef, $after) = say($conn, "show hardware",
                                          -swallow, '-timeout', 60);
        (@info) = split(/\r\n/, $after);
        foreach $line (@info)  {
            if ($line =~ /System version:/i) {
                $line =~ s/^\s+//g;
                ($firmware) = (split (/\s+/, $line))[2];
            }
            if ($line =~ /Model number is/i) {
                $line =~ s/^\s+//g;
                ($model) = (split (/\s+/, $line))[3];
            }
            if ((defined($model)) && (defined($firmware))) {
                last; #get out of the foreach loop, got the needed info
            }
        }
    } #cisco

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # Mcdata switch
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if ($hosttype =~ /switch-mcdata/i) {
        $vendor = 'Mcdata';
        chomp($vendor);
        my $director;
        (undef, undef, $after) = say($conn, "show system", -swallow,
                                                  '-timeout', 60);
        if ($after =~ /director/i) {
            $director = 1;
        } else {
            $director = 0;
        }

        (undef, undef, $after) = say($conn, "show system",
                                       -swallow, '-timeout', 60);
        (@info) = split(/\r\n/, $after);
        foreach $line (@info)  {
            if ($line =~ /Firmware Version:/i) {
                $line =~ s/^\s+//g;
                ($firmware) = (split (/\s+/, $line))[2];
            }
            if ($line =~ /Model Name:/i) {
                $line =~ s/^\s+//g;
                ($model) = (split (/\s+/, $line))[3];
            }
        }
    } #mcdata

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # Brocade switch
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if ($hosttype =~ /switch-brocade/i) {
        $vendor = 'Brocade';
        chomp($vendor);
        my $version = undef;
        my @brocade_prompts = ('^\s*newname\d+:.*>');

        (undef, undef, $after) = Tharn::say($conn, "version", -swallow, '-prompt', @brocade_prompts, '-timeout', 60);

        # check for the Fabric OS version of the switch
        if ($after =~ /Fabric OS:\s*v7/i) {
            $version = "v7";
        } elsif ($after =~ /Fabric OS:\s*v6/i) {
            $version = "v6";
        } elsif ($after =~ /Fabric OS:\s*v5/i) {
            $version = "v5";
        } elsif ($after =~ /Fabric OS:\s*v4/i) {
            $version = "v4";
        } elsif ($after =~ /Fabric OS:\s*v3/i) {
            $version = "v3";
        } elsif ($after =~ /Fabric OS:\s*v2/i) {
            $version = "v2";
        } else { $version = "unsupported version"; }

        # need to handle a V3 OS differently than other versions...
        if ($version eq "v3") {
           @info = split(/\n\r/, $after);
        } elsif ($version eq "v5"||"v4"||"v2") {
           @info = split(/\r\n/, $after);
        }

        foreach $line (@info)  {
            if ($version eq "unsupported version") { last; }
            if ($line =~ /Fabric OS:/i) {
                $line =~ s/^\s+//g;
                ($firmware) = (split (/\s+/, $line))[2];
            }
        }

        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        # need to get the model name only possible in version 5 (not 4,3,2)
        #
        #vss-b200e-1:admin> chassisshow
        #
        #POWER SUPPLY  Unit: 1
        #Time Awake:             44 days
        #
        #FAN  Unit: 1
        #Time Awake:             44 days
        #
        #FAN  Unit: 2
        #Time Awake:             44 days
        #
        #FAN  Unit: 3
        #Time Awake:             44 days
        #
        #CHASSIS/WWN  Unit: 1
        #Header Version:         2
        #Factory Part Num:       40-0500027-06
        #Factory Serial Num:     RD060033215
        #Manufacture:            Day: 23  Month:  4  Year: 2006
        #Update:                 Day: 12  Month:  4  Year: 2007
        #Time Alive:             298 days
        #Time Awake:             44 days
        #ID:                     BRD0000CA
        #Part Num:               SLKWRM0000200
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        if ($version eq "v5") {
            (undef, undef, $after) = Tharn::say($conn, "chassisshow", -swallow, '-prompt', @brocade_prompts, '-timeout', 60);
            @info = ();
            $line = '';
            @info = split(/\n/, $after);

            foreach $line (@info) {
                if ($line =~ /^\s*Part Num:/i) {
                    $line =~ s/^s+//g;
                    ($model) = (split(/\s+/, $line))[2];
                }
            }
        } # if ($version eq "v5")
    } #Brocade

    $switch_hw_sw_info{"vendor"}   = $vendor;
    $switch_hw_sw_info{"model"}    = $model;
    $switch_hw_sw_info{"firmware"} = $firmware;

    return (%switch_hw_sw_info);

}

1;

