# -*- perl -*-
# $Id:$
#
use strict;
use VSeries::switch;
use cluster;
use VSeries::VSeries;

#- - - - - - - - - - - - - - - - - - - - - - - - - -
# Subroutine init
# Intializes the data structures, called by all the VSeries::VSeries scripts
# Parameters to the init_global_var function are passed by reference
#- - - - - - - - - - - - - - - - - - - - - - - - - -

sub init_global_var {

    my ($tot_filers, $tot_switches, $filer_list, $switch_list, 
        $filer_conn_list, $switch_conn_list, $fh_rec, $sh_rec, 
        $fc_rec, $sc_rec, $filer_info, $hba_conn_list, 
        $storage_conn_list, $sleep_time, $CLUSTERED, $cluster_type, 
        $heartbeat_conn_list, $isl_conn_list, $switch_info, $storage_info) = @_;

    my $FAST_ACTION_SLEEPTIME = 6;
    my $SLOW_ACTION_SLEEPTIME = 70;

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Populate @switch_list, $tot_switches, @filer_list, $tot_filers, 
    # @switch_conn_list, @filer_conn_list
    # - - - - - - - - - - - - - - - - - - - - - - - - -  

    if (defined ($SWITCH0)) {
        $switch_list->[${$tot_switches}] = $SWITCH0;
        $switch_conn_list->[${$tot_switches}] = $SWITCH0_CONN;
        ++${$tot_switches};
    }

    if (defined($SWITCH1)) {
        $switch_list->[${$tot_switches}] = $SWITCH1;
        $switch_conn_list->[${$tot_switches}] = $SWITCH1_CONN;
        ++${$tot_switches};
    }

    if (defined($SWITCH2)) {
        $switch_list->[${$tot_switches}] = $SWITCH2;
        $switch_conn_list->[${$tot_switches}] = $SWITCH2_CONN;
        ++${$tot_switches};
    }

    if (defined($SWITCH3)) {
        $switch_list->[${$tot_switches}] = $SWITCH3;
        $switch_conn_list->[${$tot_switches}] = $SWITCH3_CONN;
        ++${$tot_switches};
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Total no of switches should not be more than 4
    # - - - - - - - - - - - - - - - - - - - - - - - - -  

    if (${$tot_switches} > 4) {
        tstlog(-comment, "Max number of switches can be 4");
        tstlog(-comment, "There are more than 4 switches defined");
        tstlog(-fatal, "Test FAILED");
    } 

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Check if a switch is specified more than once 
    # - - - - - - - - - - - - - - - - - - - - - - - - -  

    {
        my %seen = ();
        foreach my $switch (@{$switch_list}) {
            if (exists $seen{$switch}) {
                tstlog(-comment, "Switch $switch is mentioned more than once");
                tstlog(-fatal, "Test FAILED");
            } else {
                $seen{$switch} = 1;
            }
        }
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Work on FILER* parameters
    # - - - - - - - - - - - - - - - - - - - - - - - - -  
    
    if (defined($FILERA)) {
        $filer_list->[${$tot_filers}] = $FILERA;
        $filer_conn_list->[${$tot_filers}] = $FILERA_CONN;
        ++${$tot_filers};
    }
    if (defined($FILERB)) {
        $filer_list->[${$tot_filers}] = $FILERB;
        $filer_conn_list->[${$tot_filers}] = $FILERB_CONN;
        ++${$tot_filers};
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # At least 1 filer should be mentioned and 
    # Total no of filers should not be more than 2
    # - - - - - - - - - - - - - - - - - - - - - - - - -  

    if ((${$tot_filers} == 0) || (${$tot_filers} > 2)) {
        tstlog(-comment, "There should at least be 1 filer defined OR");
        tstlog(-comment, "There are more than 2 filers defined");
        tstlog(-fatal, "Test FAILED");
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Check if a filer is specified more than once
    # - - - - - - - - - - - - - - - - - - - - - - - - -

    {
        my %seen = ();
        foreach my $filer (@{$filer_list}) {
            if (exists $seen{$filer}) {
                tstlog(-comment, "Filer $filer is mentioned more than once");
                tstlog(-fatal, "Test FAILED");
            } else {
                $seen{$filer} = 1;
            }
        }
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Establish the connections to filers and switches
    # Populate %fh_rec, %fc_rec, %sh_rec, %sc_rec
    # - - - - - - - - - - - - - - - - - - - - - - - - -  

    for (my $i=0; $i<${$tot_filers}; ++$i) {
        $fh_rec->{$filer_list->[$i]} = host($filer_list->[$i]);
        $fc_rec->{$filer_list->[$i]} = connect($filer_conn_list->[$i]);
    }

    for (my $i=0; $i<${$tot_switches}; ++$i) {
        $sh_rec->{$switch_list->[$i]} = host($switch_list->[$i]);
        $sc_rec->{$switch_list->[$i]} = connect($switch_conn_list->[$i]);
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Initialize sleeptime between each actions
    # - - - - - - - - - - - - - - - - - - - - - - - - -  
    ${$sleep_time} = ($ACTION_TYPE =~ "FAST") ? $FAST_ACTION_SLEEPTIME :
                     $SLOW_ACTION_SLEEPTIME;

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # If only one filer is given in the command but it is  clustered 
    # then then partner's name should also be given in the command 
    # line. The main reason for this is; a test may require verification 
    # in the partner filer, so its better to capture the partner's 
    # console messages while running the test.
    # - - - - - - - - - - - - - - - - - - - - - - - - -  
    if (${$tot_filers} == 1) {
        my $partner_name = undef;
        $partner_name = VSeries::VSeries::vseries_partner_name_get($fc_rec->{$filer_list->[0]});
        if (defined($partner_name)) {
             tstlog (-comment, "$filer_list->[0] is partnered 2 $partner_name");
             tstlog (-comment, "Give partner filer in command line as well");
             tstlog (-fatal, "Test FAILED"); 
        }
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # The filers should be clustered if they are 2
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    my %cluster_partner = ();

    if (${$tot_filers} == 2) {
        # Both should be partnered
        if (($filer_list->[1] ne 
                       VSeries::VSeries::vseries_partner_name_get($fc_rec->{$filer_list->[0]})) ||
            ($filer_list->[0] ne 
                       VSeries::VSeries::vseries_partner_name_get($fc_rec->{$filer_list->[1]}))) {
                tstlog (-comment, "Filers $filer_list->[0] and $filer_list->[1] 
                                   are not connected. Exiting");
                tstlog (-fatal, "Test FAILED"); 
        } 

        ${$CLUSTERED} = 1;
        $cluster_partner{$filer_list->[0]} = $filer_list->[1];
        $cluster_partner{$filer_list->[1]} = $filer_list->[0];
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Intitialize switch information like, switchname, firmware, vendor etc..
    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    {
        #foreach my $switch (keys %{$sc_rec}) {
        foreach my $switch (@{$switch_list}) {
            my $conn = $sc_rec->{$switch};
            my %info = VSeries::switch::switch_hw_sw_info_get($conn); 
            my $rec = {};
            #my $model = $conn->{hostp}->{model_type};
            $switch_info->{$switch} = $rec;
            $switch_info->{$switch}{"vendor"} = $info{"vendor"};
            $switch_info->{$switch}{"model"} = $info{"model"};
            $switch_info->{$switch}{"firmware"} = $info{"firmware"};
            @{$switch_info->{$switch}{"hba_portlist"}} = ();
            @{$switch_info->{$switch}{"storage_portlist"}} = ();
            @{$switch_info->{$switch}{"cluster_portlist"}} = ();
            @{$switch_info->{$switch}{"isl_portlist"}} = ();
        }
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Intitialize storage array(s) information
    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    {
        my @conn = ();
        for (my $i=0; $i<${$tot_filers}; ++$i) {
            $conn[$i] = $fc_rec->{$filer_list->[$i]} ;
        }
        %{$storage_info} = VSeries::VSeries::vseries_storage_info_get(\@conn);
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Intitialize filer(s) information
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    foreach my $filer (keys %{$fc_rec}) {
        my $conn = $fc_rec->{$filer};
        my $rec = {};
        $filer_info->{$filer} = $rec;
        my %temp1 = VSeries::VSeries::vseries_hw_sw_info_get($conn);
        my %temp = VSeries::VSeries::vseries_info_get($conn);
        $rec->{"model name"} = $temp1{"model name"};
        $rec->{"firmware release"} = $temp1{"firmware release"};
        $rec->{"software release"} = $temp1{"software release"};

        if (${$CLUSTERED}) {
            $rec->{"clustered"} = "yes";
            $rec->{"partner"} = $cluster_partner{$filer};
            my @cluster_slotlist =  VSeries::VSeries::vseries_cluster_slot_info_get($conn);

            foreach my $entry (@cluster_slotlist) {
                $rec->{"cluster_type"} = $entry->{"cluster_type"};
                push (@{$rec->{"cluster_slotlist"}}, $entry->{"slot"});
                ${$cluster_type} = $entry->{"cluster_type"};
            }
        } else {
            $rec->{"clustered"} = "no";
            $rec->{"partner"} = undef;
            $rec->{"cluster_type"} = undef; 
            @{$rec->{"cluster_slotlist"}} = (); 
            ${$cluster_type} = undef;
        }

        @{$rec->{"localswitches"}} = @{$temp{"localswitches"}};
        @{$rec->{"remoteswitches"}} = @{$temp{"remoteswitches"}};

        # -  - - - - - - - - - - - - - - - - -  - - - - - -
        # Check if storage arrays are directly connected
        # -  - - - - - - - - - - - - - - - - -  - - - - - -
        if ((@{$temp{"localswitches"}} == 0) && 
            (@{$temp{"remoteswitches"}} == 0)) {
            $rec->{"storagearrays_are_directly_connected"} = "yes";
        } else {
            $rec->{"storagearrays_are_directly_connected"} = "no";
        }

        $rec->{"no_prim_paths"} =  $temp{"no_prim_paths"};
        $rec->{"no_sec_paths"} = $temp{"no_sec_paths"};

        if ($temp{"primary"}) {
            @{$rec->{"prim_luns"}} = @{$temp{"primary"}};
        } else {
            @{$rec->{"prim_luns"}} = ();
        }

        if ($temp{"secondary"}) {
            @{$rec->{"sec_luns"}} = @{$temp{"secondary"}};
        } else {
            @{$rec->{"sec_luns"}} = ();
        }

        @{$rec->{"storage_slotlist"}} = @{$temp{"slotlist"}};
    } #filer

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    #  All switches given in the command line should be connected to each and 
    #  every filer(s)
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    my %switch_connected = ();
    foreach my $switch (@{$switch_list}) {
        foreach my $filer (keys %{$filer_info}) {
            my @local_switchlist = @{$filer_info->{$filer}{"localswitches"}};
            my @remote_switchlist = @{$filer_info->{$filer}{"remoteswitches"}};
            my @tot_switchlist;
            push (@tot_switchlist, @local_switchlist);
            push (@tot_switchlist, @remote_switchlist);
            foreach my $switch1 (@tot_switchlist) {
                if ($switch eq $switch1) {
                    $switch_connected{$switch} = 1;
                }
            }
        }
    }

    foreach my $switch (@{$switch_list}) {
        if (!exists ($switch_connected{$switch})) {
            tstlog(-comment, 
                   "Switch $switch is not connected to any of the given filer");
            tstlog(-fatal, "Test FAILED");
        }
    }

    
    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Number of primary and secondary paths should be same before 
    # starting any switch related test.
    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    foreach my $filer (keys %{$fc_rec}) {
        if (($filer_info->{$filer}{"no_prim_paths"}) !=
             ($filer_info->{$filer}{"no_sec_paths"})) {
            tstlog (-comment,
                 "$filer prim luns=$filer_info->{$filer}{\"no_prim_paths\"}");
            tstlog (-comment,
                 "$filer sec luns=$filer_info->{$filer}{\"no_sec_paths\"}");
            tstlog (-comment,
                  "Number of prim and sec paths are NOT equal in $filer");
            tstlog (-fatal, "Test FAILED");
        }
    }


    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Perform some error checking for cluster configuration
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    if (${$CLUSTERED}) {
        foreach my $filer (keys %{$fc_rec}) {
            # -  - - - - - - - - - - - - - - - - -  - - - - - -
            # Cluster should be UP and running
            # -  - - - - - - - - - - - - - - - - -  - - - - - -
            if ("UP" ne get_cf_state($fh_rec->{$filer})) {
                tstlog (-comment, "Cluster is Not UP");
                tstlog (-fatal, "Test FAILED");
            }

            # -  - - - - - - - - - - - - - - - - -  - - - - - -
            # Perform some error checking for metro cluster
            # -  - - - - - - - - - - - - - - - - -  - - - - - -
            if (${$cluster_type} =~ /metro/) {
                 # -  - - - - - - - - - - - - - - - - -  - - - - - -
                 # Both cluster links should be UP for metro cluster
                 # -  - - - - - - - - - - - - - - - - -  - - - - - -
                if ( (VSeries::VSeries::vseries_heartbeat_link_status_find(
                                       $fc_rec->{$filer}, 0) ne "up" ) &&
                     (VSeries::VSeries::vseries_heartbeat_link_status_find(
                                       $fc_rec->{$filer}, 1) ne "up" )) {
                tstlog (-comment, "Both cluster links are Not UP on $filer");
                tstlog (-fatal, "Test FAILED");
                }
                # -  - - - - - - - - - - - - - - - - -  - - - - - -
                # Metro cluster should have atleast 2 switches on the local site
                # and 2 switches on the remote site
                # -  - - - - - - - - - - - - - - - - -  - - - - - -
                if ($filer_info->{$filer}{"remoteswitches"} < 2) {
                    tstlog (-comment, "$filer remoteswitches are less than 2");
                    tstlog (-comment,
                    "remoteswitches=$filer_info->{$filer}{\"remoteswitches\"}");
                    tstlog (-fatal, "Test FAILED");
       
                }

                if ($filer_info->{$filer}{"localswitches"} < 2) {
                    tstlog (-comment, "$filer localswitches are less than 2");
                    tstlog (-comment,
                    "localswitches = $filer_info->{$filer}{\"localswitches\"}");
                    tstlog (-fatal, "Test FAILED");
                }
            } #metro
        } #filer
    } #CLUSTERED

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Intitialize switch ports on the host (filer) side
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    foreach my $filer (keys %{$fc_rec}) {
        my $conn = $fc_rec->{$filer};
        my @temp = VSeries::VSeries::vseries_host_portlist_get($conn);
        foreach my $entry (@temp) {
            my %rec = ();
            $rec{"filer"} = $filer;
            $rec{"filer_slot"} = $entry->{"slot"};
            $rec{"switch"} = $entry->{"switch"};
            my ($sw_conn, $sw_host);
            my ($hosttype);
            #- - - - - - - - - - - - - - - - - - - - - -  - - - - - 
            #Check if the this switch is mentioned in command line
            #if not then make a temporary connection to get the info
            #- - - - - - - - - - - - - - - - - - - - - -  - - - - - 
            if (!exists $sc_rec->{$entry->{"switch"}}) {   
                $sw_host = host($entry->{"switch"});
                $sw_conn = connect("#telnet");
                $hosttype = $sw_conn->{hostp}->{hosttype};
                disconnect($sw_conn);
            } else {
                $sw_conn = $sc_rec->{$rec{"switch"}};
                $hosttype = $sw_conn->{hostp}->{hosttype};
            }
            if ($hosttype =~ /switch-cisco/i) {
                my $temp_switchport = $entry->{"switchport"};
                my ($port_num, $blade_num);
                ($blade_num, $port_num) = split (/-/, $temp_switchport);
                ++$blade_num;
                $temp_switchport = "$blade_num" . "\/" . "$port_num";
                $rec{"switch_port"} = $temp_switchport;
            } else {
                $rec{"switch_port"} = $entry->{"switchport"};
            }
            push (@{$hba_conn_list}, \%rec);
        }
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Intialize the data structure used for executing the test
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    foreach my $temp1 (@{$hba_conn_list}) {
        push (@{$switch_info->{$temp1->{"switch"}}{"hba_portlist"}}, 
                                              $temp1->{"switch_port"});
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Intialize switch ports on the storage side
    # Storage ports of all switches can be found out by using only 1 filer
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    {
        my $filer = $filer_list->[0]; 
        my $conn = $fc_rec->{$filer};
        my  @temp = VSeries::VSeries::vseries_storage_portlist_get($conn);
        foreach my $entry (@temp) {
            my %rec = ();
            $rec{"switch"} = $entry->{"switch"};
            my ($sw_conn, $sw_host);
            my ($hosttype);
            #- - - - - - - - - - - - - - - - - - - - - -  - - - - - 
            #Check if the this switch is mentioned in command line
            #if not then make a temporary connection to get the info
            #- - - - - - - - - - - - - - - - - - - - - -  - - - - - 
            if (!exists $sc_rec->{$entry->{"switch"}}) {   
                $sw_host = host($entry->{"switch"});
                $sw_conn = connect("#telnet");
                $hosttype = $sw_conn->{hostp}->{hosttype};
                disconnect($sw_conn);
            } else {
                $sw_conn = $sc_rec->{$rec{"switch"}};
                $hosttype = $sw_conn->{hostp}->{hosttype};
            }
            if ($hosttype =~ /switch-cisco/i) {
                my $temp_switchport = $entry->{"port_num"};
                my ($port_num, $blade_num);
                ($blade_num, $port_num) = split (/-/, $temp_switchport);
                ++$blade_num;
                $temp_switchport = "$blade_num" . "\/" . "$port_num";
                $rec{"port_num"} = $temp_switchport;
            } else {
                $rec{"port_num"} = $entry->{"port_num"};
            }
            push (@{$storage_conn_list}, \%rec);
        }
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Intialize the data structure used for executing the test
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    foreach my $temp1 (@{$storage_conn_list}) {
        push (@{$switch_info->{$temp1->{"switch"}}{"storage_portlist"}}, 
                                              $temp1->{"port_num"});
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    #Initialize heartbeat ports for switches for Metrocluster
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    if (${$CLUSTERED}) {
        foreach my $filer (keys %{$fc_rec}) {
            next unless ($filer_info->{$filer}{"storagearrays_are_directly_connected"} =~ /no/i);
            my $conn = $fc_rec->{$filer};
            my @cluster_slotlist =  VSeries::VSeries::vseries_cluster_slot_info_get($conn);
            foreach my $entry (@cluster_slotlist) {
                my %rec = ();
                $rec{"filer"} = $filer;
                $rec{"filer_slot"} = $entry->{"slot"};
                $rec{"link"} = $entry->{"link"};
                $rec{"cluster_type"} = $entry->{"cluster_type"};
                if ($entry->{"cluster_type"} =~ /metro/) {
                    $rec{"filer_slot_WWN"} = $entry->{"WWN"};
                    # - - - - - - -  - - -  - - -  - - -  - - -  - -
                    # Find the cluster port on a switch, it should be connected 
                    # to one of the local switches
                    # - - - - - - -  - - -  - - -  - - -  - - -  - -
                    foreach my $switch 
                        (@{$filer_info->{$filer} {"localswitches"}}) {
                        my $switch_port = undef;
                        #- - - - - - - - - - - - - - - - - - - - - -  - - - - - 
                        #Check if the this switch is mentioned in command line
                        #if not then make a temporary connection to get the info
                        #- - - - - - - - - - - - - - - - - - - - - -  - - - - - 
                        if (!exists $sc_rec->{$switch}) {   
                            my $temp_switch_host = host($switch);
                            my $temp_switch_conn = connect("#telnet");
                            $switch_port =  VSeries::switch::switch_port_num_get(
                                                            $temp_switch_conn, 
                                                            $entry->{"WWN"}
                                                            );
                            disconnect($temp_switch_conn);
                        } else { 
                            $switch_port =  VSeries::switch::switch_port_num_get(
                                                            $sc_rec->{$switch}, 
                                                            $entry->{"WWN"}
                                                            );
                        }
                        if (defined($switch_port)) {
                             $rec{"switch"} = $switch;
                             $rec{"switch_port_num"} = $switch_port;
                        }
                    }
                    # - - - - - - -  - - -  - - -  - - -  - - -  - -
                    # If the cluster port is not connected to any of the 
                    # the local switches, the switch should be mentioned in 
                    # the command line as well, then fail the test. 
                    # The heartbeat links need to be brought up before 
                    # starting the test
                    # - - - - - - -  - - -  - - -  - - -  - - -  - -
                    if (!exists $rec{"switch"}) {
                        tstlog(-comment, 
                                 "$filer: The Heartbeat links are not UP");
                        tstlog(-fatal, "Test FAILED");
                    }
                } else { #metro
                    $rec{"switch"} = undef;
                    $rec{"switch_port_num"} = undef;
                }
                push (@{$heartbeat_conn_list}, \%rec);
            } #entry
        } #filer
    } #CLUSTER

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Intialize the data structure used for executing the test
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    foreach my $temp1 (@{$heartbeat_conn_list}) {
        push (@{$switch_info->{$temp1->{"switch"}}{"cluster_portlist"}}, 
                                              $temp1->{"switch_port_num"});
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Initialize ISL ports for switches for Metrocluster
    # This information can be built by using only 1 filer
    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    {
    my $filer = $filer_list->[0]; 
    if ((${$cluster_type} =~ /metro/) || (${$cluster_type} =~ /Spectre_Stretched/i)) {
        foreach my $switch (@{$filer_info->{$filer} {"localswitches"}}) {
            my @eport_info = ();
            #- - - - - - - - - - - - - - - - - - - - - -  - - - - - 
            #Check if the this switch is mentioned in command line
            #if not then make a temporary connection to get the info
            #- - - - - - - - - - - - - - - - - - - - - -  - - - - - 
            if (!exists $sc_rec->{$switch}) {   
                my $temp_switch_host = host($switch);
                my $temp_switch_conn = connect("#telnet");
                @eport_info =  VSeries::switch::switch_eport_get($temp_switch_conn);
                disconnect($temp_switch_conn);
            } else {
                @eport_info =  VSeries::switch::switch_eport_get($sc_rec->{$switch});
            }

            if (@eport_info == 0) {
                tstlog(-comment, 
                   "ISL Config problem: No E port is configured for $switch");
                tstlog(-fatal, "Test FAILED");
            } else {
                my $entry_number = 1;
                foreach my $eport_info (@eport_info) {
                    my %rec = ();
                    my %rec1 = ();
                    $rec{"switch"} = $switch;
                    $rec{"port"} = $eport_info->{"port"};
                    $rec{"port_WWN"} = $eport_info->{"port_WWN"};
                    $rec{"peer_port_WWN"} = $eport_info->{"peer_port_WWN"};
                    $rec{"peer_switch"} = undef;   
                    $rec{"peer_port"} = undef;

                    # Find the remote switch connected to this port
                    foreach my $remote_switch 
                            (@{$filer_info->{$filer} {"remoteswitches"}}) {
                        #- - - - - - - - - - - - - - - - - - - - - -  - - - - - 
                        #Check if the this switch is mentioned in command line
                        #if not then make a temporary connection to get the info
                        #- - - - - - - - - - - - - - - - - - - - - -  - - - - - 
                        my $peer_port = undef;
                        if (!exists $sc_rec->{$remote_switch}) {   
                            my $temp_switch_host = host($remote_switch);
                            my $temp_switch_conn = connect("#telnet");
                            $peer_port = VSeries::switch::switch_eport_num_get(
                                        $temp_switch_conn, 
                                        $eport_info->{"peer_port_WWN"},
                                        $entry_number);
                            disconnect($temp_switch_conn);
                        } else {
                            $peer_port = VSeries::switch::switch_eport_num_get(
                                        $sc_rec->{$remote_switch}, 
                                        $eport_info->{"peer_port_WWN"},
                                        $entry_number);
                        }
                        if (defined($peer_port)) {
                            $rec{"peer_switch"} = $remote_switch;
                            $rec{"peer_port"} = $peer_port;
                        }
                    }
                    push (@{$isl_conn_list}, \%rec);

                    # -  - - - - - - - - - - - - - - - - -  - - - - - -
                    # Now reverse the switch and peer switch info in the hash
                    # so that lateron the ISL port disable enable can be 
                    # performed  by looking at the "switch" and "port" 
                    # combination in the array  of hash. Equivalent of putting
                    # entry for the remote switch.  
                    # -  - - - - - - - - - - - - - - - - -  - - - - - -
                    $rec1{"switch"} = $rec{"peer_switch"};
                    $rec1{"port"} = $rec{"peer_port"};
                    $rec1{"port_WWN"} = $rec{"peer_port_WWN"};
                    $rec1{"peer_port_WWN"} = $rec{"port_WWN"};
                    $rec1{"peer_switch"} = $rec{"switch"};
                    $rec1{"peer_port"} = $rec{"port"};
                    push (@{$isl_conn_list}, \%rec1);

                    ++$entry_number;
                } #eport_info
            }
        } #switch local 

    } #metro || Spectre_Stretched
    } #ISL

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Intialize the data structure used for executing the test
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    foreach my $temp1 (@{$isl_conn_list}) {
        push (@{$switch_info->{$temp1->{"switch"}}{"isl_portlist"}}, 
                                              $temp1->{"port"});
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Disconnect all switch connections
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    for (my $i=0; $i<${$tot_switches}; ++$i) {
        VSeries::switch::switch_logout($sc_rec->{$switch_list->[$i]});
        disconnect($sc_rec->{$switch_list->[$i]});
    }

    sleep 10; #just to make sure that switch  conns are cleaned up properly
      
}   # end of init_global_var function

 
# -  - - - - - - - - - - - - - - - - -  - - - - - -
# Subroutine to verify the current and original number of primary paths
# -  - - - - - - - - - - - - - - - - -  - - - - - -

sub verify_prim_no_paths($$$) {
    my ($filer, $conn, %filer_info) = @_;


    my $currrent_no_prim_paths = VSeries::VSeries::vseries_no_primary_path_get($conn);
    if ($currrent_no_prim_paths != $filer_info{$filer}{"no_prim_paths"}) {
        tstlog (-comment, "Primary LUN count mismatch");
        tstlog (-comment, "The affected filer is: $filer");
        tstlog (-comment, "Current LUN count = $currrent_no_prim_paths");
        tstlog (-comment, 
                "Original LUN couunt = $filer_info{$filer}{\"no_prim_paths\"}");
        return (1);
    }
}

# -  - - - - - - - - - - - - - - - - -  - - - - - -
# Subroutine to verify the current and original number of secondary paths
# -  - - - - - - - - - - - - - - - - -  - - - - - -

sub verify_sec_no_paths($$$) {
    my ($filer, $conn, %filer_info) = @_;

    my $currrent_no_sec_paths = VSeries::VSeries::vseries_no_redundant_path_get($conn);
    if ($currrent_no_sec_paths != $filer_info{$filer}{"no_sec_paths"}) {
        tstlog (-comment, "Secondary LUN count mismatch");
        tstlog (-comment, "The affected filer is: $filer");
        tstlog (-comment, "Current LUN count = $currrent_no_sec_paths");
        tstlog (-comment,
                "Original LUN coount = $filer_info{$filer}{\"no_sec_paths\"}");
        return (1);
    }
}

# -  - - - - - - - - - - - - - - - - -  - - - - - -
# Subroutine to get local filer of a switch, needed only for Metrocluster
# -  - - - - - - - - - - - - - - - - -  - - - - - -

sub local_filer_of_switch_get($$) {
    my ($switch, %filer_info) = @_;

    foreach my $filer (keys %filer_info) {
        my @local_switchlist = @{$filer_info{$filer}{"localswitches"}};
        foreach my $local_switch (@local_switchlist) {
            if ($switch eq $local_switch) {
                return ($filer);
            }
        }
    }
    return (undef);
}

# -  - - - - - - - - - - - - - - - - -  - - - - - -
# Subroutine to find list of storage slots connecting given switch
# and filer.
# -  - - - - - - - - - - - - - - - - -  - - - - - -

sub storage_slotlist_connecting_switch_and_filer($$$) {
    my ($filer, $switch, @hba_conn_list) = @_;

    my (@slotlist) = ();
    my $i = 0;
    foreach my $conn (@hba_conn_list) {
        if (($conn->{"filer"} eq $filer) && ($conn->{"switch"} eq $switch)) {
            $slotlist[$i] = $conn->{"filer_slot"};
            ++$i;
        }
    } #foreach
    return @slotlist;
}

# -  - - - - - - - - - - - - - - - - -  - - - - - -
# Subroutine to print the topology (testbed info)
# -  - - - - - - - - - - - - - - - - -  - - - - - -

sub print_test_setup() {

    my ($fc_rec, $sc_rec, $filer_info, $switch_info, $storage_info, 
        $hba_conn_list, $storage_conn_list, $heartbeat_conn_list, 
        $isl_conn_list) = @_;    

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Filers(s) info: The following prints are just for debugging purpose
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    print "- - - - - - - - - - - - - - - - - - -  - - - \n";
    print "\n";
    print "Filer(s) Information:\n";
    foreach my $filer (keys %{$fc_rec}) {
        print "- - - - - - - - - - - - - - - - - - -  - - - \n";
        print "$filer:model name: $filer_info->{$filer}{\"model name\"}\n";
        print "$filer:firmware release: $filer_info->{$filer}{\"firmware release\"}\n";
        print "$filer:software release: $filer_info->{$filer}{\"software release\"}\n";
        print "$filer:clustered: $filer_info->{$filer}{\"clustered\"}\n";
        print "$filer:partner: $filer_info->{$filer}{\"partner\"}\n";
        print "$filer:cluster_type: $filer_info->{$filer}{\"cluster_type\"}\n";
        print "$filer:cluster slot list: @{$filer_info->{$filer}{\"cluster_slotlist\"}}\n";
        print "$filer:localswitches: @{$filer_info->{$filer} {\"localswitches\"}}\n";
        print "$filer:remoteswitches: @{$filer_info->{$filer} {\"remoteswitches\"}}\n";
        print "$filer:no_prim_paths: $filer_info->{$filer}{\"no_prim_paths\"}\n";
        print "$filer:no_sec_paths: $filer_info->{$filer}{\"no_sec_paths\"}\n";
        print "$filer:storage slot list: @{$filer_info->{$filer} {\"storage_slotlist\"}}\n";
        print "$filer:storage directly connected: $filer_info->{$filer}{\"storagearrays_are_directly_connected\"}\n";
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Switch(s) info: The following prints are just for debugging purpose
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    print "\n";
    print "Switch(s) Information:\n";
    foreach my $switch (keys %{$sc_rec}) {
        print "- - - - - - - - - - - - - - - - - - -  - - - \n";
        print "$switch:vendor: $switch_info->{$switch}{\"vendor\"}\n";
        print "$switch:model: $switch_info->{$switch}{\"model\"}\n";
        print "$switch:firmware: $switch_info->{$switch}{\"firmware\"}\n";
        print "$switch:hba port list: @{$switch_info->{$switch}{\"hba_portlist\"}}\n";
        print "$switch:storage port list: @{$switch_info->{$switch}{\"storage_portlist\"}}\n";
        print "$switch:cluster port list: @{$switch_info->{$switch}{\"cluster_portlist\"}}\n";
        print "$switch:isl port list: @{$switch_info->{$switch}{\"isl_portlist\"}}\n";
    }

    print "\n";
    print "Storage Array(s) Information:\n";
    print "- - - - - - - - - - - - - - - - - - -  - - - \n";
    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Storage(s) info: The following prints are just for debugging purpose
    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    foreach my $vendor (keys %{$storage_info}) {
        print "Vendor and Models $vendor: @{$storage_info->{$vendor}}\n"; 
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # HBA ports: The following prints are just for debugging purpose
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    print "\n";
    print "Switch(s) HBA ports Information:\n";
    foreach my $temp1 (@{$hba_conn_list}) {
        print "- - - - - - - - - - - - - - - - - - -  - - - \n";
        print "HBA Filer is: $temp1->{\"filer\"}\n";
        print "HBA Filer Slot is: $temp1->{\"filer_slot\"}\n";
        print "Switch is: $temp1->{\"switch\"}\n";
        print "Switch port is : $temp1->{\"switch_port\"}\n";
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Storage ports: The following prints are just for debugging purpose
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    print "\n";
    print "Switch(s) Storage ports Information:\n";
    foreach my $temp1 (@{$storage_conn_list}) {
        print "- - - - - - - - - - - - - - - - - - -  - - - \n";
        print "Storage Switch: $temp1->{\"switch\"}\n";
        print "Storage Switch Port Num : $temp1->{\"port_num\"}\n";
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # Heartebat (Cluster) ports: Following prints are just for debugging purpose
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    print "\n";
    print "Switch(s) Cluster ports Information:\n";
    foreach my $temp1 (@{$heartbeat_conn_list}) {
        print "- - - - - - - - - - - - - - - - - - -  - - - \n";
        print "Cluster Filer: $temp1->{\"filer\"}\n";
        print "Cluster Filer Slot: $temp1->{\"filer_slot\"}\n";
        print "Cluster Type: $temp1->{\"cluster_type\"}\n";
        print "Cluster Link: $temp1->{\"link\"}\n";
        print "Switch: $temp1->{\"switch\"}\n";
        print "Switch Port: $temp1->{\"switch_port_num\"}\n";
    }

    # -  - - - - - - - - - - - - - - - - -  - - - - - -
    # ISL ports: The following prints are just for debugging purpose
    # -  - - - - - - - - - - - - - - - - -  - - - - - -

    print "\n";
    print "Switch(s) ISL ports Information:\n";
    foreach my $temp1 (@{$isl_conn_list}) {
        print "- - - - - - - - - - - - - - - - - - -  - - - \n";
        print "ISL Switch: $temp1->{\"switch\"}\n";
        print "ISL Switch Port Num : $temp1->{\"port\"}\n";
        print "ISL Switch Port WWN: $temp1->{\"port_WWN\"}\n";
        print "ISL Peer Switch : $temp1->{\"peer_switch\"}\n";
        print "ISL Peer Switch Port : $temp1->{\"peer_port\"}\n";
        print "ISL Peer Switch Port WWN: $temp1->{\"peer_port_WWN\"}\n";
    }
    print "- - - - - - - - - - - - - - - - - - -  - - - \n";
}

return 1;
