
#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/main/lib/VSim/VSim.pm#9 $
#!/usr/local/bin/perl
## @summary The heper methods vsim command testing and vsim creation
##
## @description
##   All the vsim command and flag operations will be automated in this module.
##   Each command will be available as a perl API call.
##   These are the list of commands supported
##                            make,
##                            reserve,
##                            binfo,
##                            prompt,
##                            unbind,
##                            rehost,
##                            reset,
##                            suspend,
##                            resume,
##                            states,
##                            state,
##                            stop,
##                            start,
##                            update,
##                            snaplist,
##                            rm,
##                            rescheck,
##                            printenv,
##                            setenv,
##                            unsetenv,
##                            cmd,
##                            cmds,
##                            bind,
##                            rebind,
##                            snap,
##                            snapback,
##                            snapdelete,
##                            mkfod,
##                            rmfod,
##                            hajoin,
##                            help,
##                            hasplit,
##                            showres,
##                            mkrc,
##                            list,
##                            version,
##
## @status public
## @author prasadt@netapp.com
## History none


package VSim;

use strict;
use warnings;

use vars qw(@ISA @EXPORT $AUTOLOAD);
use Tharn;
use Tharn qw($Log);
require Exporter;
use Data::Dumper;
use Params::Validate qw(:all);

use VSim::Exceptions::VSimCMDException qw(:try);

use NACL::C::Client;
use NACL::APISet;

# This value will be returned from each method on success, unless there is
# defined purpose.
my $PASS = 0;

## @name VSim new method
## @description Creates a new VSim object
##
##
## @arg ident Mandatory:      The VSA/VSim identifier name
## @arg type Optional:        The type of the Simulator (VSA/VSim)
##                            Default is VSim
## @arg domain Optional:      The domain of the VSA/VSim
## @arg  vsimpath Optional:   The path to the vsim binary.
##                            Default is "/usr/software/test/bin/vsim"
## @synopsis
## my $vsim_obj = VSim->new( ident    => $name,
##                           type => "vsim" );
##
sub new {
    my ($class, @params)   = @_;
    my $self = {};

    my %args = validate (@params, { ident => {type => SCALAR, optional =>1},
                                    domain => {type => SCALAR, optional =>1},
                                    type => {type => SCALAR,
                                             regex => qr/^(vsa|vsim)$/,
                                             default => "vsim"},
                                    vsimpath => {type => SCALAR,
                                                 default => "/usr/software/".
                                                            "test/bin/vsim"},
                                  });

    $self = {
        ident    => $args{ident},
        type     => $args{type},
        vsimpath => $args{vsimpath},
        domain   => $args{domain},
        ctrl_mode  => "normal",
    };

   # vsim commands and usage
   # "opts-id" mean's the command flow is "vsim command [Options] <id>"
   # "opts-id-inst" mean's the command flow is "vsim command [Options]
   #                                                  <id> <servinst> "
   # "id-snapname" means the command flow is vsim command <id> <snap_name>
   # "value" means the command flow is  "vsim command value"
   # "" means the command flow is "vsim command"
   # "opts-id-value" means the command flow is  "vsim command [options] <id> <value>"
   $self -> {vsim_cmd}={
                            make       => "opts-id",
                            reserve    => "opts-id",
                            binfo      => "opts-id",
                            prompt     => "opts-id",
                            unbind     => "opts-id",
                            rehost     => "opts-id",
                            poweroff   => "opts-id",
                            reset      => "opts-id",
                            suspend    => "opts-id",
                            resume     => "opts-id",
                            states     => "opts-id",
                            state      => "opts-id",
                            stop       => "opts-id",
                            start      => "opts-id",
                            update     => "opts-id",
                            snaplist   => "opts-id",
                            rm         => "opts-id",
                            rescheck   => "opts-id",
                            printenv   => "opts-id",
                            setenv     => "id-env-value",
                            unsetenv   => "id-env-value",
                            cmd        => "opts-id-value",
                            cmds       => "opts-id-value",
                            bind       => "opts-id-inst",
                            rebind     => "opts-id-inst",
                            snap       => "id-snapname",
                            snapback   => "id-snapname",
                            snapdelete => "id-snapname",
                            mkfod      => "value",
                            rmfod      => "value",
                            hajoin     => "value",
                            help       => "value",
                            hasplit    => "value",
                            showdef    => "",
                            showres    => "",
                            mkrc       => "",
                            list       => "",
                            version   => "",
                        };
    # Get the client object on which the test has to be executed.
    # The client should be passed as a parameter to the test.
    my $client = NACL::C::Client->find();
    $self->{clientobj} =  $client -> apiset(category  => "Host",interface => "CLI",
                                       set => "Linux");

    # The hash contails all the nodes that the VSim object is creating or managing.
    $self -> {nodes} = {type => $args{type},
                        ident => $args{ident},
                        hajoin_partner_ready => 0,
                        ha_pair => 0,
                        cluster_created => 0};
    bless($self, $class);

    # TODO::Do the ESX test setup check for VSA
    if ( $self -> {type} eq "vsa" ) {
        _ESX_checkconfig();
        # TODO::Create a ssh session for ESX
        # Create the VSA licenses and add it as a constructor.
    }
    return $self;
}

## @name VSim::_ESX_checkconfig
##
## @description
## This is a private method. The method is designed to verify the test setup
## at the ESX.
##
## If the vmsocat process is not configured, that will be handled here
## If any datastore [NAS, iscsi or local] needs to be done at the setup,
## that can be handled here.
##
## @synopsis
## _ESX_checkconfig();
##
sub _ESX_checkconfig {
     # TODO::Create a method to verify the console connection and
     # create vmsocat connection if necessary.
     # TODO::Create a method to verify and create nas datastore if neccessary
    return $PASS;
}

## @name VSim::_ESX_execute
##
## @description
## This is a private method. The method is designed to execute the commands
## at the ESX. The method uses the ssh session created at the constructor, and
## executes the commands on ESX.
##
## @synopsis
## _ESX_execute();
##
sub _ESX_execute {
    # TODO:: need to handle the esx command execution, similar to _execute
    return $PASS;
}

## @name VSim::execute
##
## @description
## The method is designed to execute the vsim commands at the linux client.
## The method uses the _execute method, and executes the commands on the
## client. A basic validation of the result is handled as per the need.
## @arg cmd Required: The complete command to be executed
## @arg verify Optional: 1 if the method has to handle the basic error check,
##                       else 0. Default is 1.
## @arg timeout Optional: Time out value for the command execution.
##                        Default value is 60 seconds.
## @note The method can be used by the user's if a perticular VSim command
## is not automated.
##
## @synopsis
## execute(cmd => "xyz");
##
sub execute {
    my ( $self, @params) = @_;
    my %args = validate (@params, { cmd => {type => SCALAR},
                                    verify => {type => BOOLEAN,  default => 1},
                                    timeout => {type => SCALAR,default => 60}});
    my $timeout = $args{timeout};
    my $cmd =  $args{cmd};
    my ($rc,$output) = $self -> _execute (cmd => $cmd, timeout => $timeout);

     if (! $args{verify} ) {
        return $rc, $output;
    }

    if (($output =~ /Fatal|ERROR|denied|Warning/ig) or ($rc != 0)) {
        VSim::Exceptions::VSimCMDException->throw(
            "The vsim command exwecution failed, 'vsim' ".
            "output contains one of the Fatal|ERROR|denied|Warning. ".
            "output is\n $output");
    }
    return $rc, $output;
}

## @name VSim::_execute
##
## @description
## This is a private method.The method is designed to execute the vsim
## commands at the linux client.The method makes use of the client ssh session
## created at the constructor and executes the commands on it.
## @arg cmd Required: The command to be executed
## @arg prompt Optional: For the prompt command, the answer need to be passed
##                       as per the users need.
## @arg timeout Optional: Time out value for the command execution at the
##                        client.Default value is 60 seconds.
##
## @synopsis
## _execute(cmd => "xyz");
##

sub _execute {
    my ( $self, @params) = @_;
    my %args = validate (@params, { cmd => {type => SCALAR},
                                    prompt => {type => SCALAR|UNDEF, optional => 1},
                                    timeout => {type => SCALAR,default => 60}});
    my $timeout = $args{timeout};
    my $cmd =  $args{cmd};
    my $status = undef;

    # set the timeout value for the command.
    $self->{clientobj}->set_timeout("connectrec-timeout" => $timeout);

    my $output = $self->{clientobj}->execute_raw_command(
                     command=> $cmd,
                     match_table => [("(one of the answers below|Please enter your choice)" =>
                                      $args{prompt}),],
                     get_status => \$status);
    return $status,$output;
}

## @name VSim::AUTOLOAD
##
## @description
## Any VSim commad call will be entering this method. The method accepts the vsim
## commands applicable for both VSim/VSA.
# handled for both VSim/VSA. The method is designed to execute the vsim
## commands at the linux client.
## @arg opts Optional: The options to be used for the vsim command execution.
## @arg timeout Optional: Time out value for the command execution.
##                        Default value is 600 seconds.
## @arg verify Optional: 1 if the command output needs to get verified, else 0.
##                       Default is 1.
## @arg ident Optional: The VSim identifier name. Default value is the
##                      identifier name used for the object creation.
## @arg snap Optional:  The snapshot name. Required for snapshot related
##                      operations.
## @arg instance Optional: The ESX instance name for the reservation.
##                         Required for the commands that requires the
##                         instance as option, for the command execution.
## @arg env Optional: Required if the command is setenv. This is
##                    the enviormental variable name.
## @args value  Optional: Required if the command execution requires a value.
##                      Here is the list of commands for which this parameter
##                      is mandatory;
##                      1. mkfod
##                      2. rmfod
##                      3. hajoin
##                      4. help
##                      5. hasplit
##                      6. setenv
##
## @synopsis
## make(opts => "-7m -p x86_64");
##
sub AUTOLOAD {
    my ( $self, @params) = @_;
    my %args = validate (@params, { opts => {type => SCALAR, default => ""},
                                    verify => {type => BOOLEAN,  default => 1},
                                    ident  => {type => SCALAR|ARRAYREF,
                                               default => $self->{ident}},
                                    snap   => {type => SCALAR, optional => 1},
                                    env    => {type => SCALAR, optional => 1},
                                    value   => {type => SCALAR, optional => 1},
                                    instance   => {type => SCALAR, optional => 1},
                                    timeout => {type => SCALAR,default => 600}
                                  }
                        );

    # Get the vsim command name(method call and the command name are the same).
    my $method = $AUTOLOAD;
    $method =~  s/.*::(.*)/$1/g;

    # Form the vsim command. vsim command path is defined at the constructor
    my $cmd = $self -> {vsimpath};

    # Form the VSim command based on the type.
    if ( grep { $_ eq $method } keys %{$self -> {vsim_cmd}} ){
        my $cmdtype = $self -> {vsim_cmd} -> {$method};
        if ($cmdtype =~ /^opts-id$/) {
            $cmd = $cmd." $method ".$args{opts}." ".$args{ident};
        } elsif ($cmdtype =~ /^value$/) {
            if (not defined $args{value}){
                VSim::Exceptions::VSimCMDException->throw("Cannot execute the vsim".
                                                      " command $method. please".
                                                      " send the value for 'value' ".
                                                      "parameter");
            }
            $cmd = $cmd." $method \"$args{value}\"";
        } elsif ($cmdtype =~ /^opts-id-value$/) {
            if (not defined $args{value}){
                VSim::Exceptions::VSimCMDException->throw("Cannot execute the vsim".
                                                      " command $method. please".
                                                      " send the value for 'value' ".
                                                      "parameter");
            }
            $cmd = $cmd." $method $args{opts} $args{ident} \"$args{value}\"";
        } elsif ($cmdtype =~ /^id-snapname$/) {
            if (not $args{snap}){
                VSim::Exceptions::VSimCMDException->throw(
                    "Cannot execute the snapshot command as snap name is not".
                    " provided. need to send the value for 'snap' to the method");
            }
            $cmd = "$cmd $method $args{ident} $args{snap}";
        } elsif ($cmdtype =~ /^$/) {
            $cmd = $cmd." $method ";
        } elsif ($cmdtype =~ /^opts-id-inst$/) {
            my $instance = $args{instance};
            if (not $instance){
                if ($method =~ /rebind/i) {
                    $instance = "";
                } else {
                    VSim::Exceptions::VSimCMDException->throw(
                        "Cannot execute the command as the ESX instance name is not".
                        " provided. need to send the value for 'instance' to the method");
                }
            }
            my ($rc,$reser) = $self -> _execute (cmd => "/usr/software/utils/bin/elms ".
                                 " reserve $instance -l 1w");
            $cmd = $cmd." $method ".$args{opts}." ".$args{ident}. " $instance" ;
        } elsif ($cmdtype =~ /^id-env-value/) {
            if (not $args{env}){
                VSim::Exceptions::VSimCMDException->throw(
                    "Cannot execute the setenv command as enviornmental".
                    " variable name is not provided. need to send the ".
                    "value for 'env' to the method");
            }
            if ((defined $args{value}) and $args{value}){
                $cmd = $cmd." $method  $args{ident} $args{env} \"$args{value}\"";
            } else {
                $cmd = $cmd." $method  $args{ident} $args{env}";
            }
        }
    } else {
       $Log->comment("The command $method is not supported with ".
                         "the VSim test library");
    }

    # Execute the vsim command
    my ($rc,$output) = $self -> _execute (cmd => $cmd, timeout => $args{timeout},
                                          prompt => $args{prompt});

    # If the user doesn't need to verify the result, he can bypass teh verification
    if (! $args{verify} ) {
        return $rc, $output;
    }

    my $status = $self -> _verifyexec(command => $method, rc => $rc,
                                      output => $output,ident => $args{ident},
                                     );
    return $rc, $output;
}

## @name VSim::_verifyexec
##
## @description
## This is a private method. The method is called from the AUTOLOAD to
## verify the result of a vsim command execution.The verification is
## common for both VSim/VSA.
## @arg rc Required: The return code for the command executed
## @arg output Required: The output of the command executed
## @arg command Required: The VSim command that has got executed
## @arg ident Required: The VSim identifier
##
## @synopsis
## _verifyexec(rc => $rc,output => $output,
##             command => $method, ident => $ident);
##
sub _verifyexec {
    my ( $self, @params) = @_;
    my %args = validate (@params, { ident  => {type => SCALAR},
                                    command => {type => SCALAR},
                                    rc      => {type => SCALAR},
                                    output  => {type => SCALAR},});
    my $rc = $args{rc};
    my $output = $args{output};
    my $command = $args{command};
    my $ident = $args{ident};
    if ($output =~ /Warning\W/){
        $Log->comment("WARNING: A warning was thrown in the ".
                     "output for the command $command");
    }
    if ( $rc != 0 ){
        VSim::Exceptions::VSimCMDException->throw(
            "vsim returned $rc after the $command command execution.".
            "\nThe command output is : \n $output");
    } elsif ($output =~ /Fatal|Error|denied/ig){
        VSim::Exceptions::VSimCMDException->throw(
            "The vsim command $command execution failed.command output contains".
            " one among the Fatal|ERROR|denied|Warning. ".
            "output is\n $output");
    } elsif ($command =~ /^make|unbind|unreserve$/) {
        $self -> _verify_unboundVSim(ident => $ident,
                                     command => $command);
    } elsif ($command =~ /^reserve|bind|rebind|rehost$/) {
        $self -> _verify_boundVSim(ident => $ident,
                                   command => $command);
    } elsif ($command =~ /^start|stop|suspend|poweroff|resume|reset$/) {

        $Log->comment("Let's wait for 3 sec, to make sure power state".
                      " is reflected.");
        sleep 3;
        $self -> _verify_powerstate(ident => $ident, command => $command,);
    } elsif ($command =~ /^rm$/) {
        my $vsims = $self -> list();
        if ($args{ident} =~ $vsims){
            VSim::Exceptions::VSimCMDException->throw("VSim is not removed".
                                                      " after the rm command");
        }
    }
    return $PASS;
}

## @name VSim::_verify_unboundVSim
##
## @description
## This is a private method. The method verifies the unbound VSim.
## The verification is common for both VSim/VSA.
## @arg command Required: The VSim command that has got executed
## @arg ident Required: The VSim identifier
##
## @synopsis
## _verify_unboundVSim(command => $method, ident => $ident);
##
sub _verify_unboundVSim {
    my ( $self, @params) = @_;
    my %args = validate (@params, { ident  => {type => SCALAR},
                                    command => {type => SCALAR}});
    my $command = $args{command};
    my $ident = $args{ident};
    # Verify the presence of the vmx file
    my $lsout = $self -> _execute (cmd => "ls ~/vsims/*_vsim_".
                                          $ident."/");
    if ($lsout !~ /DataONTAP.vmx/g) {
        VSim::Exceptions::VSimCMDException->throw(
            "vsim $command command execution failed. The VMX file is missing".
            " at the vsim directory");
    }
    # Verify the bind status
    my $binfo_out = $self -> binfo(ident => $ident);
    if ($binfo_out =~ /Reservation:/){
        VSim::Exceptions::VSimCMDException->throw(
            "vsim $command command execution failed. The VSim should not be".
            " reserved after the command execution");
    }
    return $PASS;
}

## @name VSim::_verify_boundVSim
##
## @description
## This is a private method. The method verifies the bound VSim.
## The verification is common for both VSim/VSA.
## @arg command Required: The VSim command that has got executed
## @arg ident Required: The VSim identifier
##
## @synopsis
## _verify_boundVSim(command => $method, ident => $ident);
##
sub _verify_boundVSim {
    my ( $self, @params) = @_;
    my %args = validate (@params, { ident  => {type => SCALAR},
                                    command => {type => SCALAR}});
    my $command = $args{command};
    my $ident = $args{ident};

    # Verify the bind status
    my $binfo_out = $self -> binfo(ident => $ident);
    if ($binfo_out !~ /Server name:/){
        VSim::Exceptions::VSimCMDException->throw(
            "vsim $command command execution failed. The VSim should be".
            " bound/registered after the command execution");
    }
    # Verify the power status
    $self -> _verify_powerstate (ident  => $ident,
                                 command => $command,
                                 state   => "off",
                                );
}

## @name VSim::_verify_powerstate
##
## @description
## This is a private method. The method verifies the power state of the  VSim.
## The verification is common for both VSim/VSA.
## @arg command Required: The VSim command that has got executed
## @arg ident Required: The VSim identifier
## @arg state Optional: The Expected state of the VSA/VSim
##
## @synopsis
## _verify_powerstate(command => $method, ident => $ident
##                    state   => $state);
##
sub _verify_powerstate {
    my ( $self, @params) = @_;
    my %args = validate (@params, { ident  => {type => SCALAR},
                                    command => {type => SCALAR},
                                    state   => {type => SCALAR, optional => 1},
                                  });
    my $command = $args{command};
    my $ident = $args{ident};
    my $expected_state = undef;
    my %power_state = (
                          start    => "on",
                          stop     => "off",
                          suspend  => "suspended",
                          poweroff => "off",
                          resume   => "on",
                          reset    => "on"
                      );
    if (not defined $args{state}) {
        if (not defined $power_state{$command}){
            VSim::Exceptions::VSimCMDException->throw("_verify_powerstate".
                                                      " cannot verify ".
                                                      "$args{command}");
        }
        $expected_state = $power_state{$command};
    } else {
        $expected_state = $args{state};
    }
    # Verify the power status
    my $state_result = $self -> state (ident => $ident);
    if ($state_result !~ /$expected_state/i) {
        VSim::Exceptions::VSimCMDException->throw(
            "verify_powerstate:: vsim $command command failed.".
            "expected power state was $expected_state, state is $state_result");
    }
    return $PASS;
}

## @name VSim::create
##
## @description
## The method can be used to create a VSA or VSim of any configuration.
##
## @arg  nodes Required : Either a reference to the array of the
##                         nodes or the node name.
## @arg  domain Optional: The domain on which VSim needs to reserved
##                        Required if domain is not defined at the object
## @arg  mode  Optional:   The mode of the VSim/VSA. default is c-mode.
## @arg  if_exists  Optional: What needs to be done if the VSim is aleady
##                            existing.default is "overwrite".
## @arg  platform Optional: The platform of the VSim/VSA.
##                          default is "x86_64".
## @arg  disk_model Optional: The disk model of the VSim/VSA.
##                            Default is "quickl".
## @arg  bsd_ip   Optional:  The bsdip values passed to vsim script for HA, in
##                           a arrayref.Required if the VSim is HA.
## @arg  ic_mac   Optional:  The icmac values passed to vsim script for HA, in
##                           a arrayref.Required if the VSim is HA.
## @arg make_option Optional: The additional custom make options.
## @arg  HA Optional: 1 if the VSim is HA. Default is 0.
## @arg  cons Optional: The type of the console [Eg:vidcons]
##                      Default is the vsim default.
## @arg  fod   Optional: The fod name. Required for HA VSim.
## @arg init_option Optional: The boot menu init option.Default is 4.
##                            User can send 44/4a for fake-init.
## @arg  build_path Optional: Build root. default is "/x/eng/rlse/DOT/mainN".
## @arg  nameserver_ip Optional: The name server ip for th ontap.
##                               There is no default value. But if it is not
##                               passed as a parameter, this entry should be
##                               present in the natehost file.
## @arg  cluster_label Optional: Cluster Label name for the VSim cluster.
##                               There is no default value. But if it is not
##                               passed as a parameter, this entry should be
##                               present in the natehost file for c-mode VSim.
##
## @arg  domain Optional: The domain name of the VSim.
##                        There is no default value. But if it is not
##                        passed as a parameter, this entry should be
##                        present in the natehost file.
## @arg   timeout Optional: Timeout value for the vsim command executions. default is 60
##
## @synopsis
##
##  c-mode HA:
##  $vsim =  $Client_Object  -> create (
##                                            nodes      => ["prasadt-vsim3","prasadt-vsim4"],
##                                            domain     => "sim.eng.btc.netapp.in",
##                                            disk_model => "quickl",
##                                            build_path => "/x/eng/rlse/DOT/RrollingrockN",
##                                            HA         => "1",
##                                            bsd_ip     => [("prasadt-vsim3:10.72.44.185:255.255.224.0:10.72.32.1"),
##                                                           ("prasadt-vsim4:10.72.44.226:255.255.224.0:10.72.32.1")],
##                                            ic_mac      => [("00:50:56:10:07:b8,00:50:56:10:07:b9"),("00:50:56:10:07:b9,00:50:56:10:07:b8")],
##                                            fod        => "testpsim",
##                                          );
##   c-mode stand alone:
##   my $vsim =  $Client_Object  -> create (
##                                            nodes      => "prasadt-vsim3",
##                                            domain     => "sim.eng.btc.netapp.in",
##                                            disk_model => "quickl"
##                                         );
##    7-mode standalone
##    my $vsim =  $Client_Object  -> create (
##                                            nodes      => "prasadt-vsim3",#["prasadt-vsim3","prasadt-vsim4"],
##                                            domain     => "sim.eng.btc.netapp.in",
##                                            disk_model => "quickl",
##                                            build_path => "/x/eng/rlse/DOT/RrollingrockN",
##                                            mode       => "7m"
##                                          );
##    c-mode cluster
##    $vsim =  $Client_Object  -> create (
##                                           nodes      => ["prasadt-vsim3","prasadt-vsim4"],
##                                           domain     => "sim.eng.btc.netapp.in",
##                                           disk_model => "quickl",
##                                           make_option => "-d",
##                                           build_path => "/x/eng/rlse/DOT/mainN",
##                                          );
##
sub create {
    my ( $self, @params) = @_;
    my %args = validate (@params, { nodes      => {type => SCALAR|ARRAYREF},
                                    domain     => {type => SCALAR, default => $self -> {domain}},
                                    mode       => {type => SCALAR,default => "cmode"},
                                    if_exists  => {type => SCALAR,default => "overwrite"},
                                    platform   => {type => SCALAR, default => "x86_64"},
                                    disk_model => {type => SCALAR, default => "quickl"},
                                    make_option => {type => SCALAR, default => ""},
                                    init_option => {type => SCALAR, default => "4"},
                                    bsd_ip     => {type => ARRAYREF, optional => 1},
                                    ic_mac      => {type => ARRAYREF, optional => 1},
                                    HA         => {type => BOOLEAN, default => 0},
                                    cons       => {type => SCALAR, optional => 1},
                                    fod        => {type => SCALAR, optional => 1},
                                    build_path => {type => SCALAR,
                                                   default => "/x/eng/rlse/DOT/mainN"},
                                    nameserver_ip => {type => SCALAR, optional => 1},
                                    cluster_label => {type => SCALAR, optional => 1},
                                    domain        => {type => SCALAR, optional => 1},
                                    timeout       => {type => SCALAR,default => 60}});

    my $custom_make = $args{make_option};
    my @ha_common_make = ();
    my $nonha_make_opts = undef;
    my $nodes; # Array ref
    my $sims = []; # Array ref to populate VSim/VSA Identifiers

    if ( ref($args{nodes} ) ne "ARRAY" ) {
        $nodes = [$args{nodes},];
    } else {
        $nodes = $args{nodes};
    }

    foreach my $node (@{$nodes}){
        if ($node =~ /([^_\-]+)[_\-]vsim[_\-]*(\S+)$/) {
            # As of now, the code is not depend on the user,
            # we may need it in future.
            my $user = $1;
            my $id = $2;
            my $skip_setup = 0;
            $skip_setup = 1 if ($custom_make =~ /(node|cfg)/i);
            push ( @{$sims},$id);
            # Populate/Add the entries to $self->{nodes}
            $self -> {nodes}->{$node} = { mode => $args{mode},
                                          platform   => $args{platform},
                                          disk_model => $args{disk_model},
                                          cons       => $args{cons},
                                          fod        => $args{fod},
                                          id         => $id,
                                          build_path => $args{build_path},
                                          skip_setup => $skip_setup,
                                        };
            $self -> {nodes}->{ha_pair} = $args{HA};
            if (exists $args{nameserver_ip}) {
                $self -> {nodes}->{$node}->{nameserver_ip} = $args{nameserver_ip};
            }
            if (exists $args{cluster_label}){
                $self -> {nodes}->{$node}->{cluster_label} = $args{cluster_label};
            }
            if (exists $args{domain}){
                $self -> {nodes}->{$node}->{domain} = $args{domain};
            }
        } else {
            VSim::Exceptions::VSimCMDException->throw("vsim name should be in the".
                                                      " form ([^_\\-]+[_\\-]".
                                                      "vsim[_\\-]*\\S+)$/)");
        }
    }
    if ($args{HA} == 1) {
        # verify we have two entries for nodes.
        if (@{$nodes} != 2){
            VSim::Exceptions::VSimCMDException->throw("Two and only Two members".
                                                      " should be present in the".
                                                      " nodes, for HA");
        }
        # This logic will change, once the HA.2 is released.
        if (( not defined $args{bsd_ip} ) || ( not defined $args{ic_mac} ) ||
		     ( not defined $args{fod}) || ( ref($args{nodes} ) ne "ARRAY" )){
            VSim::Exceptions::VSimCMDException->throw(
                "For a HA VSim we need to have following ".
                "parameters: bsd_ip, ic_mac, fod and nodes".
                "as a ARRAYREF");
        }

        $ha_common_make[0] = "-ir $args{build_path} -$args{disk_model} ".
                         "-p $args{platform} -fa $args{fod} -$args{mode}".
                         " -bsdmgmt -bsdip $args{bsd_ip}->[0] ".
                         "-icmac \"$args{ic_mac}->[0]\" ";
        $ha_common_make[0] = $ha_common_make[0]." -$args{cons}" if defined $args{cons};
        if (defined $custom_make){
            $ha_common_make[0] = $ha_common_make[0]." ".$custom_make;
        }
        $ha_common_make[1] = "-ir $args{build_path} -$args{disk_model} ".
                         "-p $args{platform} -fb $args{fod} -$args{mode}".
                         " -bsdmgmt -bsdip $args{bsd_ip}->[1] ".
                         "-icmac \"$args{ic_mac}->[1]\" "; # verify the arrays bsdip and ic_mac
        $ha_common_make[1] = $ha_common_make[1].
                             " -$args{cons}" if defined $args{cons};
        if (defined $custom_make){
            $ha_common_make[1] = $ha_common_make[1]." ".$custom_make;
        }
    # create the make command
    } elsif ($self -> {type} =~ /vsim|vsa/i) {
        $nonha_make_opts = "-ir $args{build_path} -p $args{platform}".
                           " -$args{mode} -$args{disk_model}";
        if (defined $args{cons}) {
            $nonha_make_opts = $nonha_make_opts." -$args{cons}";
        }
        if (defined $custom_make){
            $nonha_make_opts = $nonha_make_opts." ".$custom_make;
        }
        if (($self -> {type} =~ /vsa/i) && ($custom_make !~ /vsalic/)
            && (defined $self ->{license})){
            $nonha_make_opts = $nonha_make_opts." -vsalic $self ->{license}";
        }
        $nonha_make_opts = $nonha_make_opts . " -vsa" if ($self->{type} =~ /vsa/);
    }

    # Handle the case, where the VSim already exist.
    # TODO: If we are reusing the VSim, we can skip lot of these codepath.
    # Add an option to re-use
    foreach my $node ( @{$sims}) {
        my $status;
        try {
            $status = ($self -> binfo(ident => $node))[0];
        } catch VSim::Exceptions::VSimCMDException with {
            $Log->comment("There is no VSim with the name $node");
        };
        if ((defined $status) and ($status == 0)) {
            $self -> rm(ident => $node, opts => "-f");
        }
    }

    # create the fod for HA. This is a temporary code.This code
    #will change with new HA implementation.
    if ($args{HA} == 1) {
        $self->rmfod(value => $args{fod}, verify => 0);
        $self->mkfod(value => $args{fod});
        my $node1 = $1 if ($args{nodes}[0] =~ /[^_\-]+[_\-]vsim[_\-]*(\S+)$/);
        my $node2 = $1 if ($args{nodes}[1] =~ /[^_\-]+[_\-]vsim[_\-]*(\S+)$/);
        $self -> make(opts => $ha_common_make[0], ident => $node1);
        $self -> make(opts => $ha_common_make[1], ident => $node2);
    } else {
        foreach my $node ( @{$sims}) {
            $self -> make(opts => $nonha_make_opts, ident => $node);
        }
    }
    # Do the VSim/VSA setup
    $self -> _sim_setup(nodes => $nodes,init_option => $args{init_option});
    $self -> _post_setup(nodes => $nodes);
    return $PASS;
}

## @name VSim::_wait_for_login
##
## @description
## This is a private method.The method make sure the VSA/VSim land up on the
## login prompt.
## @arg node Required: The VSim/VSA node name [same as nate host file name]
##
## @synopsis
## _wait_for_login(nodes => "prasadt-vsim1");
##
sub _wait_for_login {
    my ( $self, @params) = @_;
    my %args = validate (@params, { node      => {type => SCALAR}});
    my $node = $args{node};
    my $node_con = $self -> {nodes} -> {$node}->{conn_obj};
    # Wait for the login prompt.
    $Log->comment("Wait for the booting to finish, and ".
                      "expect the 'Password:' prompt");
    sleep 20;
    eval {
        $node_con -> execute(command => "version",timeout => 3600,
                             prompts => ['[Pp]assword\:',])
    };
    if ($@) {
        VSim::Exceptions::VSimCMDException->throw("Hit $@");
    }
}

## @name VSim::_sim_setup
##
## @description
## This is a private method.The method handles the setup of the VSim/VSA from
## bind/register to the login prompt. For an HA Vsim, HA will not be enabled, but HA
## Setup will be done.
## @arg nodes Required: The VSim/VSA node names [same as nate host file name]
##
## @synopsis
## _sim_setup(nodes => ["prasadt-vsim1",]);
##
sub _sim_setup {
    my ( $self, @params) = @_;
    my %args = validate (@params, { nodes      => {type => ARRAYREF,
                                                   default => $self->{ident}},
                                    init_option => {type => SCALAR},
                                    skip_start  => {type => BOOLEAN,
                                                    default => 0},
                                  }
                        );
    my $host_obj;
    my $skip_start = $args{skip_start};
    my $conn = []; # Array Ref for the connection objects.

    foreach my $node (@{$args{nodes}}){
        # Get the VSim/VSA identifier
        my $vsim_ident = $1 if ($node =~ /[^_\-]+[_\-]vsim[_\-]*(\S+)$/);

        # Reserve the VSim/VSA.
        $self -> reserve(ident => $vsim_ident) if (!$skip_start);
        my $num_attempt = 0;
        my $result =1;

        # Attempt 10 times to start the machine, rule out the bad ESX host.
        while (($num_attempt < 10) and (!$skip_start)){
            $num_attempt++;
            eval {
                # Rehost the VSim/VSA if its on bad host.
                $self -> rehost(ident => $vsim_ident) if ($num_attempt > 1);
                # Now the console connection is ready.Populate the
                # connection objects.
                $Log->comment("Creating the connectrec object for $node");
                $host_obj = Hostrec->new(id => "$node");
                my $conn_obj = Connectrec->new(host => $host_obj,
                                               template => "console",
                                               conntype => "pm_cons",
                                               prompts => ['',]);

#Connectrec->new(host => $host_obj, id => $node,
#                                               conntype => "pm_cons", prompts => ['',]);
                $self->{nodes}->{$node} -> {conn_obj} = $conn_obj;

                # Start the VSim/VSA
                sleep 20;
                my $op;
                ($result, $op) = $self -> start(ident => $vsim_ident,
                                                timeout => 900);
                if ($result == 0){
                    last;
                }
            };
            if ($@) {

                my $text = $@;
                if ($text =~ /vsim prompt /ig){
                    _execute(cmd => $self -> {vsimpath}." prompt $vsim_ident",
                             prompt => "1",);
                    _execute(cmd => $self -> {vsimpath}." prompt $vsim_ident",
                             prompt => "1",);
                }
                $Log->comment("Hit $text.Retrying: to start the VSim.");
            }
        }
        if (($num_attempt == 3) and ( $result != 0)){
            VSim::Exceptions::VSimCMDException->throw( "Start operation ".
                                                       "failed for $node");
        }

        if (($self->{nodes}->{$node}->{skip_setup} == 1) or
           ($self->{type} =~ /vsa/i)) {
            $self -> _wait_for_login(node => $node);
        } elsif ($self->{type} =~ /vsim/i){
            # Get all the VSims to setup.
            # TODO : Skip it, if bootarg.setup.auto is set.
            # TODO : Use subtest to launch _sim_gettosetup for
            # all the VSims in parallel.
            $Log->comment("Stop the node at Simloader");
            sleep 3;
            my $node_con = $self->{nodes}->{$node} -> {conn_obj};
            $node_con -> execute( command => "");

            $Log->comment("Get to setup for the $node");
            my $boot_type = $self->_sim_gettosetup(node => $node,
                                                   init_option => $args{init_option});
            my $mode = $self->{nodes}->{$node}->{mode};
            if ($mode =~ /cmode|10m/i) {
                $self -> configure_cmode(node => $node);
            } elsif ($mode =~ /7m/ig) {
                $self -> configure_7mode(node => $node);
            } else {
                VSim::Exceptions::VSimCMDException->throw("Invalid mode, the".
                                                          " VSim/VSA is not not".
                                                          " of type cmode/7mode");
            }
        } else {
            VSim::Exceptions::VSimCMDException->throw("VSA should be ".
                                                      "skipping the setup");
        }
    }
    if ($self -> {nodes}->{ha_pair} == 1){
         # HA won't be enabled
         $self -> configure_HA(nodes => $args{nodes});
    }
}

## @name VSim::_sim_gettosetup
##
## @description
## The method handles the setup of the VSim/VSA from booting to
## the setup prompt.
##
## @arg node Required: The VSim node name
##
## @synopsis
## _sim_gettosetup(nodes => "prasadt-vsim1");
##
sub _sim_gettosetup {
    my ( $self, @params) = @_;
    my %args = validate (@params, { node => {type => SCALAR},
                                    init_option => {type => SCALAR}
                                  },);
    my $init_option =  $args{init_option};
    # Get the simulator name
    my $node = $args{node};
    my $node_con = $self -> {nodes} -> {$node}->{conn_obj};

    use NACL::Transit;
    my $t = NACL::Transit->new( name => $node_con );
    # Get to boot menu
    my $status;
    my $num_attempt = 0;
    while ($num_attempt < 15){
        $num_attempt++;
        eval {
            $status = $t->change_state( to => "BOOT_MENU" );
            if (defined $status){
                if ($status == "BOOT_MENU"){
                    last;
                } elsif ($status == "FIRMWARE"){
                    $node_con -> execute(command => "boot");
                }
            } else {
                my $state = $t->get_state();
                $node_con -> execute(command => "boot") if ($state == "FIRMWARE");
            }
        };
        if ($@) {
            $Log->comment("Hit $@.Retrying: to get to boot menu.");
        }
    }
    #$t->change_state( to => "ZERO_DISKS" ); #Blocker : burt 549109
    if ($init_option eq "4"){
        my $output = $node_con -> execute(command => "4",
                                          prompts => ["install a new file system",],
                                          match_table => [("Please make a selection"
                                                           => "init"),
                                             ],
                                          timeout => 1800);
        if ($output =~ /\(init\)/) {
            $self -> {ctrl_mode}  = "init";
        }
        $output = $node_con -> execute(command => "y",
                                       prompts => ["are you sure","login"],
                                       timeout => 1800);
        $init_option = "y";
    }
    $node_con -> execute(command => $init_option,timeout => 3600,
                    match_table => [
                        ("$node\:\:\>" => "cluster setup"),
                        ("login"       => "admin"),
                        ("Password"    => ""),
                        ("[Ii]nstall a new file system" => "y"),
                        ("file system without zeroing disks" => "y"),
                    ],
                         prompts => ["create a new cluster or join an ".
                                     "existing cluster","Please enter the (new|node) hostname",
                                    ]);
}

## @name VSim::_get_natehost_data
##
## @description
## The method grabs the vsim details from the nate host file.
##
## @arg node Required: The VSim node name
##
## @synopsis
## _get_natehost_data(nodes => "prasadt-vsim1");
##
sub _get_natehost_data {
    my ( $self, @params) = @_;
    my %args = validate (@params, { sim_name => {type => SCALAR}});
    my %hash;
    my $sim_name = $args{sim_name};

    # Collecting the data from the nate host file.
    my $vsim_data = host($sim_name);

    # Validating the nate host file.
    my @host_File = ('CLUSTER_IP1','default_ip','MGMT_MASK','CLUSTER_IP2','GATEWAY',
                     'CLUSTER_MASK1','CLUSTER_MASK2','default_password','default_ip_7g',
                     'SITE', 'CLUSTER_MGMT_PORT','CLUSTER_MGMT_IP',
		     'CLUSTER_MGMT_GATEWAY','TIMEZONE','hostname');

    foreach my $tharnReqEntry (@host_File){
       if ( not defined $vsim_data->{$tharnReqEntry} ){
          VSim::Exceptions::VSimCMDException->throw(
              "Error : The entry $tharnReqEntry is missing in the ".
              "Nate Host File for $sim_name");
       }
    }

    # individual conditional checks.
    if ((not defined $vsim_data->{DOMAIN}) and
       (not exists $self -> {nodes}->{$sim_name}->{domain})) {
        VSim::Exceptions::VSimCMDException->throw("The domain name has to be ".
            "present in the tharnhost file[DOMAIN], or it should be passed as a parameter");
    }
    if ((not defined $vsim_data->{CLUSTER_LABEL}) and
       (not exists $self -> {nodes}->{$sim_name}->{cluster_label})) {
        VSim::Exceptions::VSimCMDException->throw("The cluster_label name has to be ".
            "present in the tharnhost file[CLUSTER_LABEL], or it should ".
            "be passed as a parameter");
    }
    if ((not defined $vsim_data->{NAME_SERVER_IP1}) and
       (not exists $self -> {nodes}->{$sim_name}->{nameserver_ip})) {
        VSim::Exceptions::VSimCMDException->throw("The nameserver_ip name has to be ".
            "present in the tharnhost file[NAMESERVER_IP1],".
            " or it should be passed as a parameter");
    }
    if ((not defined $vsim_data->{CLUSTER_MASK1}) and
       (not defined $vsim_data->{CLUSTER_MGMT_NETMASK})) {
       VSim::Exceptions::VSimCMDException->throw("one among CLUSTER_MASK1 or".
                                                 " CLUSTER_MGMT_NETMASK should".
                                                 " be present in the nate host file");
    }

    $hash{host}                 = $sim_name;
    $hash{hostname}             = $vsim_data->{hostname};
    $hash{ip}                   = $vsim_data->{default_ip_gx};
    $hash{cluster_ip}           = $vsim_data->{IP_e0d};
    $hash{mask}                 = $vsim_data->{MGMT_MASK};
    $hash{gway}                 = $vsim_data->{GATEWAY};
    $hash{cluster_ip1}          = $vsim_data->{CLUSTER_IP1};
    $hash{cluster_netmask1}     = $vsim_data->{CLUSTER_MASK1} ||
                                  $vsim_data->{CLUSTER_MGMT_NETMASK};
    $hash{cluster_ip2}          = $vsim_data->{CLUSTER_IP2};
    $hash{cluster_netmask2}     = $vsim_data->{CLUSTER_MASK2};
    $hash{password}             = $vsim_data->{default_password};
    $hash{site}                 = $vsim_data->{SITE};
    $hash{time_zone}            = $vsim_data->{TIMEZONE};
    $hash{domain_name}          = $self -> {nodes}->{$sim_name}->{domain} ||
                                  $vsim_data->{DOMAIN};
    $hash{name_server_ip1}      = $self -> {nodes}->{$sim_name}->{nameserver_ip} ||
                                  $vsim_data->{NAME_SERVER_IP1};
    $hash{cluster_name}         = $self -> {nodes}->{$sim_name}->{cluster_label} ||
                                  $vsim_data->{CLUSTER_LABEL};
    $hash{cluster_mgmt_port}    = $vsim_data->{CLUSTER_MGMT_PORT};
    $hash{cluster_mgmt_ip}      = $vsim_data->{CLUSTER_MGMT_IP};
    $hash{cluster_mgmt_netmask} = $vsim_data->{CLUSTER_MGMT_NETMASK};
    $hash{cluster_mgmt_gateway} = $vsim_data->{CLUSTER_MGMT_GATEWAY};
    return ( \%hash );
}

## @name VSim::configure_cmode
##
## @description
## The method handles the setup for the c-mode VSim/VSA.
##
## @arg node Required: The VSim/VSA node name
##
## @synopsis
## configure_cmode(nodes => "prasadt-vsim1");
##
sub configure_cmode {
    my ( $self, @params) = @_;
    my %args = validate (@params, { node => {type => SCALAR}},);
    # Get the simulator name
    my $node = $args{node};
    my $node_con = $self -> {nodes} -> {$node}->{conn_obj};
    my $build_path = $self -> {nodes} -> {$node}->{build_path};

    # Collect the host details from the Tharn host file.
    my $host_data = $self -> _get_natehost_data (sim_name => $node);

    $Log->comment("Doing the setup for $node");
    my $init_cmd;
    if ($self -> {ctrl_mode} eq "init") {
        $init_cmd = $host_data->{hostname};
    } elsif (!$self -> {nodes} -> {cluster_created}){
        $init_cmd = "create";
    } else {
        $init_cmd = "join"
    }

    # Defining cluster license.
    my $license = undef;
    if ($build_path !~ /main|qa/gi) {
        $license = "JWFJEXMWZWYQSD";
    } else {
        $license = "ETYYFLXUQUMADFAAAAAAAAAAAAAA";
    }

    # TODO : accept custom messages and answers
    $node_con -> execute(command => $init_cmd,
                         timeout => 3600,
                         prompts => ["login:","$host_data->{cluster_name}\:\:\>"],
                         match_table => [("Do you want to use these defaults" => "no"),
                                         ("Do you want to configure a ".
                                          "management interface" => ""),
                                         ("Which port do you want to use" => "e0c"),
                                         ("List the private cluster network ports" => "e0a"),
                                         ("Enter the cluster ports.*MTU size" => "1500"),
                                         ("Enter the cluster network netmask" => $host_data->{cluster_netmask1}),
                                         ("Enter the cluster interface IP address".
                                          " for port e0a" => $host_data->{cluster_ip1}),
                                         ("Enter the cluster name"  => $host_data->{cluster_name}),
                                         ("Enter the name of the cluster you would like to join" => $host_data->{cluster_name}),
                                         ("Enter the cluster base license key" => $license), # TODO Handle licence through nate
                                         #("Enter the cluster base license key" => "ETYYFLXUQUMADFAAAAAAAAAAAAAA"),
                                         ("Enter an additional license key" => ""),
                                         ("Enter the cluster administrator.*? password" => $host_data->{password}),
                                         ("Retype the password" => $host_data->{password}),
                                         ("Enter the cluster management interface port" => "e0d"),
                                         ("Enter the cluster management interface IP address" => $host_data->{cluster_ip}),
                                         ("Enter the cluster management interface netmask" => $host_data->{mask}),
                                         ("Enter the cluster management interface default gateway" => $host_data->{cluster_mgmt_gateway}),
                                         ("IP [Aa]ddress of the default gateway" => $host_data->{cluster_mgmt_gateway}),
                                         # ("Please enter the IP Address of the default gateway"
                                         ("Enter the DNS domain names" => ""),
                                         ("Enter the name server IP addresses"  => $host_data->{name_server_ip1}),
                                         ("Where is the (controller|node) located" => ""),
                                         ("Enter the node management interface port" => "e0c"),
                                         ("Enter the node management interface IP address" => $host_data->{ip}),
                                         ("Please enter the IP Address for this interface" => $host_data->{ip}),
                                         ("Enter the node management interface netmask" => $host_data->{mask}),
                                         ("Please enter the Netmask for this interface" => $host_data->{mask}),
                                         ("Do you want to enable remote management"  => ""),
                                         ("Please specify the remote management username" => "admin"),
                                         ("Which access protocol" => ""),
                                         ("enter a password for user" => $host_data->{password}),
                                         ("Please enter it again" =>  $host_data->{password}),
                                         ("Do you want to run DNS resolver" => ""),
                                         ("[pP]ress (enter|the return key) to continue" => ""),
                                         ("[dD]o you want to setup contact information now" => "n"),
                                         ("Name of primary contact" => "VSim"),
                                         ("Phone number of primary contact" => "99999"),
                                         ("Alternate phone number of primary contact" => ""),
                                         ("Primary contact e.mail address" => ""),
                                         ("Name of secondary contact" => ""),
                                         ("Phone number of secondary contact" => ""),
                                         ("Alternate phone number of secondary contact" => ""),
                                         ("Secondary contact e.mail address" => ""),
                                         ("Business name" => "QA"),
                                         ("Business address" => "Netapp"),
                                         ("City where the business resides" => "IN"),
                                         ("State where the business resides" => ""),
                                         ("character country code" => "IN"),
                                         ("Postal code where the business resides" => ""),
                                         ("Enter the node management interface default gateway" => $host_data->{gway}),
                                         ("Do you intend for this node to be used as a single node"=>"no"),
                                         ("Will the cluster network be configured to use network switches" => ""),
                                        ]
                        );

    if ($self -> {ctrl_mode} eq "init") {
        my $cluster_cmd = undef;
        if (!$self -> {nodes} -> {cluster_created}){
            $cluster_cmd = "cluster create -clustername $host_data->{cluster_name}".
                           " -license $license -ipaddr1 $host_data->{cluster_ip1} ".
                           "-netmask $host_data->{mask} -mgmt-ip $host_data->{cluster_ip} ".
                           "-mgmt-netmask $host_data->{mask} -mgmt-gateway $host_data->{gway}";

            # TODO : define the object attribute
            $self -> {clusteripaddr} =  $host_data->{cluster_ip1};
        } else {
            $cluster_cmd = "cluster join -clusteripaddr $self->{clusteripaddr} ".
                           "-ipaddr1 $host_data->{cluster_ip1}  -netmask  $host_data->{mask}";
        }

        $node_con -> execute( command => "admin",
                              timeout => 3600,
                              match_table => [("Password" => "$host_data->{password}"),],
                              prompts => ["\:\:\>",],
                             );
        my $output = $node_con -> execute( command => $cluster_cmd,
                                          timeout => 3600,
                                          prompts => ["\:\:\>",],
                                        );
        if ($output =~ /ERROR/ig){
            VSim::Exceptions::VSimCMDException->throw("Cluster create/join ".
                "command has failed");
        }
    }
    $Log->comment("Waiting for 5 minutes after the setup");
    sleep 300;
    $Log->comment("The VSim $node is configured and is at the prompt");
    $self -> {nodes} -> {cluster_created} = 1;
    return $PASS;
}

## @name VSim::configure_7mode
##
## @description
## The method handles the setup for the 7-mode VSim/VSA.
##
## @arg node Required: The VSim/VSA node name
##
## @synopsis
## configure_7mode(nodes => "prasadt-vsim1");
##
sub configure_7mode {

    my ( $self, @params) = @_;
    my %args = validate (@params, { node => {type => SCALAR},});
    # Get the simulator name
    my $node = $args{node};
    my $node_con = $self -> {nodes} -> {$node}->{conn_obj};

    # Collect the host details from the Tharn host file.
    my $host_data = $self -> _get_natehost_data (sim_name => $node);

    $Log->comment("Doing the setup for $node");
    my $cmd = $host_data->{hostname};
    # TODO :: Accept custom changes in the messages.
    $node_con -> execute(command => "$cmd",prompts => ["Retype new password",],
                         timeout => 3600,  match_table => [("enable" => "n"),
                                                         ("Do you want to configure interface groups" => "n"),
                                                         ("Please enter the IP address for Network Interface e0a" => "" ),
                                                         ("Please enter the IP address for Network Interface e0b" => ""),
                                                         ("Please enter the IP address for Network Interface e0d" => $host_data->{ip},),
                                                         ("Please enter the netmask for Network Interface e0d" => $host_data->{mask}),
                                                         ("Please enter media type for" => ""),
                                                         ("Please enter flow control for" => ""),
                                                         ("to support jumbo frames" => "n"),
                                                         ("Please enter the IP address for Network Interface e0c" => ""),
                                                         ("Would you like to continue setup through the web interface" => ""),
                                                         ("Please enter the name or IP ".
                                                          "address of the.*?default ".
                                                          "gateway" => $host_data->{gway}),
                                                         ("Please enter the name or IP ".
                                                          "address of the administration host" => ""),
                                                         ("Please enter timezone" => $host_data->{time_zone}),
                                                         ("Where is the filer located" => ""),
                                                         ("Enter the root directory" => ""),
                                                         ("Do you want to run DNS resolver" => ""),
                                                         ("Do you want to run NIS client" => ""),
                                                         ("Do you want to configure the Shelf Alternate" => ""),
                                                         ("Enter the root directory for HTTP files" => ""),
                                                         ("New password" => $host_data->{password}),
                                                         ("take over a partner IP address during failover" => ""),
                                                         ("[nN]ame of primary contact" =>  "netapp"),
                                                         ("[pP]hone number of primary contact" => "99999999"),
                                                         ("Business name" => "ibm"),
                                                         ("Business address" => "bangalore"),
                                                         ("City where business resides" => "bangalore"),
                                                         ("mail address or IBM WebID" => "dl-vsim-qa\@netapp.com"),
                                                         ("[nN]ame of secondary contact" => ""),
                                                         ("[pP]hone number of secondary contact" => ""),
                                                         ("[sS]tate where business resides" => ""),
                                                         ("Postal code where business resides" => ""),
                                                         ("character country code" => "IN"),
                                                        ]
                        );
    $node_con -> execute(command => $host_data->{password},timeout => 60,);
    $Log->comment("Waiting for 5 minutes after the setup");
    sleep 300;
    $node_con -> execute(command => $host_data->{password},timeout => 60,);
    $Log->comment("The VSim $node is configured and is at the prompt");
    return $PASS;
}

## @name VSim::configure_HA
##
## @description
## The method handles the HA setup for the VSim.
##
## @arg node Required: The VSim/VSA node name
##
## @synopsis
## configure_HA(nodes => "prasadt-vsim1");
##
sub configure_HA{
    my ( $self, @params) = @_;
    my %args = validate (@params, {
                                    nodes   => {type => ARRAYREF},
                                  }
                        );
    my $nodes = $args{nodes};
    my $transit = undef;
    my $fod =$self -> {nodes} -> {$nodes->[0]}->{fod};
    foreach my $node (@{$nodes}){
        my $node_con = $self -> {nodes} -> {$node}->{conn_obj};
        my $mode = $self -> {nodes} -> {$node}->{mode};
        # Naming as per NACL.
        $mode =  ($mode =~ /7/)? "7Mode":"CMode";

        # Do Disk assign all on the VSim
        if ($mode eq "7Mode"){
            $node_con -> execute(command => "disk assign all",
                         timeout =>60);
        } else {
            try {
                # Do the same for 7Mode, once the burt xxx is fixed
                $Log->comment("Doing the disk assign all");
                my $node_C = NACL::C::Node->new(name => $node);
                NACL::C::Disk->assign("command_interface" => $node_C,"all" => "true",
                                  "force" => "true", node => $node);
            } catch NACL::APISet::Exceptions::ResponseException with {
                my $exception_object = shift;
                $Log->comment("Error is: " . $exception_object->text());
                VSim::Exceptions::VSimCMDException->throw(
                    "Failed to do the disk assign (all) ");

            };
        }

        # Halt the VSims
        $Log->comment("Halting the VSims and stopping at SIMLOADER");
        try {
            my $haltcmd;
            if ($mode =~ /cmode/i){
                $haltcmd = "run local halt -t 1";
            } else {
                $haltcmd = "halt";
            }
            $transit = NACL::Transit->new( name => $node );
            $transit->change_state( to => "FIRMWARE");
        } catch NACL::Transit::Exceptions::TransitException with {
             my $exception_object = shift;
             $Log->comment("Error is: " . $exception_object->text() );
             VSim::Exceptions::VSimCMDException->throw(
                 "Failed to trasit towards the SIMLOADER");
        };
    }
    $Log->comment ("Doing the HA Join operation");
    # Do HA Join
    $self -> hajoin(value => $fod);
    foreach my $node (@{$nodes}){
        try {
            $transit = NACL::Transit->new( name => $node );
            $transit->change_state( to => "CLI" );
        } catch NACL::Transit::Exceptions::TransitException with {
             my $exception_object = shift;
             $Log->comment("Error is: " . $exception_object->text() );
             VSim::Exceptions::VSimCMDException->throw(
                 "Failed to trasit towards the login prompt");
        };
    }
    return $PASS;
}

## @name VSim::elms_reserve
##
## @description
## For testing the methods like bind/rebind, the user needs to have the
## elms reservation for a instance.
##
## @arg instance Optional: The ESX instance node name. By default the method
##                         will look for a free instance using the elms command.
## @arg domain Required: The domain on which, reservation needs to be done.
## @arg duration Optional: The span of the reservation. Default is 1d.
##                         The value should be given in the form
##   Duration OPTIONS
##      Time can be an integer of seconds or a simple string with
##      no spaces containig an integer and modifier.
##      Example:
##          -l 1w (for a length of one week)
##      The options available are what you would expect, with the exception
##      of month and minute since they both begin with 'm'...
##        s - second
##        i - minute
##        h - hour
##        d - day
##        w - week
##
## @synopsis
## elms_reserve();
##
sub elms_reserve {
    my ( $self, @params) = @_;
    my %args = validate (@params, {
                                    instance   => {type => SCALAR,
                                                   optional => 1},
                                    domain     =>  {type => SCALAR},
                                    duration   => {type => SCALAR,
                                                   regex => qr/^(\d+[sihdw])$/,
                                                   default => "1d"},
                                  }
                        );
    my $instance = undef;
    my $rc       = undef;
    my $output   = undef;
    my $domain   = $args{domain};
    my $duration = $args{duration};
    my $elms_search_avail = "/usr/software/utils/bin/elms find -g vsimesx ".
                            "-d $domain --available -i | tail -n 1 ";
    if (defined $args{instance}) {
        $instance = $args{instance};
    } else {
        ($rc, $instance) = $self -> _execute (cmd => $elms_search_avail);
        if ($rc != 0){
            VSim::Exceptions::VSimCMDException->throw("elms command ".
                "$elms_search_avail failed.Output: $instance");
        }
    }
    $instance =~ s/\n//;
    my $elms_reserve = "/usr/software/utils/bin/elms reserve".
                       " $instance -l $duration";

    ($rc,$output) = $self -> _execute (cmd => $elms_reserve,);
    if ($rc != 0){
        VSim::Exceptions::VSimCMDException->throw("elms command ".
            "$elms_reserve failed.Output: $output");
    }
    return $instance;
}


## @name VSim::elms_unreserve
##
## @description
## For testing the methods like bind/rebind, the user needs to have the
## elms reservation for a instance. This method can be used to clean up the
## reservation if required.
##
## @arg instance Required: The ESX instance name.
##
## @synopsis
## elms_unreserve();
##
sub elms_unreserve {
    my ( $self, @params) = @_;
    my %args = validate (@params, {
                                    instance   => {type => SCALAR,},
                                  }
                        );
    my $instance = $args{instance};
    my $elms_reserve_cancel = "/usr/software/utils/bin/elms cancel $instance";
    my ($rc,$output) = $self -> _execute (cmd => $elms_reserve_cancel,);
    if ($rc != 0){
        VSim::Exceptions::VSimCMDException->throw("elms command ".
            "$elms_reserve_cancel failed.Output: $output");
    }
    return $PASS;
}

## @name VSim::create_connectrec
##
## @description
## This method is created for the laptop automation, were we need to
## Do the VSim creation and setup on an existing VSim.The method
## can be used to create  the connect rec object of a node.
##
## @arg node Required: The VSim node name.
##
## @synopsis
## create_connectrec(node => $node);
##
sub create_connectrec {
    my ( $self, @params) = @_;
    my %args = validate (@params, {
                                    node   => {type => SCALAR,},
                                  }
                        );
    my $node = $args{node};

    $Log->comment("Creating the connectrec object for $node");
    sleep 2;
    my $conn_obj;
    try {
        my $host_obj = Hostrec->new(id => "$node");
        $conn_obj = Connectrec->new(host => $host_obj, id => $node,
                                    conntype => "pm_cons", prompts => ['',]);
        } catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $Log->comment("Error is: " . $exception_object->text());
            my $resp = $exception_object->get_response_object();
            $Log->comment("Raw output was:\n" . $resp->get_raw_output());
            VSim::Exceptions::VSimCMDException->throw("Creation of the connectrec failed.");
        };
    $self->{nodes}->{$node} -> {conn_obj} = $conn_obj;
    return $PASS;
}

## @name VSim::_post_setup
##
## @description
## The post setup method is used to do any kind of ONTAP setup at the VSim.
## The method will be called from the create method, after creating and
## finishing the ONTAP Setup.
##
## @arg nodes Required: The VSim nodes names.
##
## @synopsis
## _post_setup(nodes => [$node1,$node2]);
##
sub _post_setup {
    my ( $self, @params) = @_;
    my %args = validate (@params, {
                                     nodes      => {type => ARRAYREF,
                                                    default => $self->{ident}},
                                  }
                     );
    foreach my $node (@{$args{nodes}}){
        my $mode = $self -> {nodes} -> {$node}->{mode};
        if ($mode =~ /7/){
            my $node_con = $self->{nodes}->{$node} -> {conn_obj};
            eval {
                $node_con -> execute(command => "options rsh.enable on",
                                     prompts => ["The full command line is \'options rsh.enable on\'",]);
            };
            if ($@) {
                $Log->comment("Retrying to enable the rsh, after 5 minutes");
                sleep 300;
                $node_con -> execute(command => "options rsh.enable on",
                                     prompts => ["The full command line is \'options rsh.enable on\'",]);
            }
        }
    }
    return $PASS;
}
1;
