#
# $Id: //depot/prod/test/Rsierranevada/lib/VolOps/BaseException.pm#1 $
#
# Copyright (c) 2010 NetApp Inc.
# All rights reserved.
#
## @summary Provide a VolOps::BaseException class that is a subclass of
##          NACL::Error and other standard exceptions that are subclasses
##          of VolOps::BaseException.
##
## @author dl-volops-qa@netapp.com
## @status private
## @pod here

=pod

=head1 Package Name

VolOps::BaseException.pm

=head1 Author

dl-volops-qa

=head1 Description

VolOps::BaseException.pm provides a collection of basic exceptions
associated with volume transition

=head1 See Also

Package inherits from the following modules. See these modules for further
interface detail.

B<NACL/Error.pm>

=head1 VolOps::BaseException

=head2 Expected use

Mostly this should be the base class for more specific exceptions.
All exceptions that are subclasses of VolOps::BaseException will be
caught if the catch clause checks VolOps::BaseException

=head2 Base Class

NACL::Error::Simple

=cut

###############################################################################
###  VolOps::BaseException
###############################################################################
package VolOps::BaseException;
use base qw(NATE::BaseException);
1;


=pod

=head1 VolOps::JobFailed

=head2 Expected use

Exception that is used when one or more jobs failed.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::JobFailed
###############################################################################
package VolOps::JobFailed;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::JobStatusFailure

=head2 Expected use

Exception that is used when there is a failure obtaining the status of a job.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::JobStatusFailure
###############################################################################
package VolOps::JobStatusFailure;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::ConfigError

=head2 Expected use

Exception that is used when there is a configuration error, such as an 
invalid block or engine type.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::ConfigError
###############################################################################
package VolOps::ConfigError;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::AttributeComparison

=head2 Expected use

Exception that is used when there is a miscomparison when performing attribute
comparison.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::AttributeComparison
###############################################################################
package VolOps::AttributeComparison;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::MissingValidationRule

=head2 Expected use

Exception that is used when performing volume attribute comparison, the default
action is 'die' and no validation rule exists for one or more attributes.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::MissingValidationRule
###############################################################################
package VolOps::MissingValidationRule;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::RetriesExhausted

=head2 Expected use

Exception that is used when a number of retries have been specified and the
retries have been exhausted

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::RetriesExhausted
###############################################################################
package VolOps::RetriesExhausted;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::SrcVolumeFailure

=head2 Expected use

Exception that is used when there is a failure accessing the source volume. 

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::SrcVolumeFailure
###############################################################################
package VolOps::SrcVolumeFailure;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::SrcAggregateFailure

=head2 Expected use

Exception that is used when there is a failure accessing the aggregate on which
the source volume resides.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::SrcAggregateFailure
###############################################################################
package VolOps::SrcAggregateFailure;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::AggregateSelectionFailure

=head2 Expected use

Exception that is used when a destination aggregate meeting the specification
cannot be found. 

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::AggregateSelectionFailure
###############################################################################
package VolOps::AggregateSelectionFailure;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::DestAggregateFailure

=head2 Expected use

Exception that is used when the destination aggregate cannot be accessed. 

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::DestAggregateFailure
###############################################################################
package VolOps::DestAggregateFailure;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::PropsNotAvailable

=head2 Expected use

Exception that is used when volume properties cannot be obtained for a given
volume.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::PropsNotAvailable
###############################################################################
package VolOps::PropsNotAvailable;
use base qw(VolOps::BaseException);
1;

=pod

=head1 VolOps::StripingNotSupported

=head2 Expected use

Exception that is used when attempting to move/copy a striped volume.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::StripingNotSupported
###############################################################################
package VolOps::StripingNotSupported;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::ConnectionFailure

=head2 Expected use

Exception that is used when there is a failure to connect to a node in the 
cluster.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::ConnectionFailure
###############################################################################
package VolOps::ConnectionFailure;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::DataStructureStoreFailure

=head2 Expected use

Exception that is used when save_status is set and there is a problem saving 
the status data.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::DataStructureStoreFailure
###############################################################################
package VolOps::DataStructureStoreFailure;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::InternalError

=head2 Expected use

Exception that is used when there is a problem with expected data stored
within the move/copy object not being available. 

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::InternalError
###############################################################################
package VolOps::InternalError;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::Move::MoveFailed

=head2 Expected use

Exception that is used when executing volume move and the move command failed. 

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::Move::MoveFailed
###############################################################################
package VolOps::Move::MoveFailed;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::Move::JobNotDeferred

=head2 Expected use

Exception that is used when executing volume move and the move job should have
been deferred but wasn't.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::Move::JobNotDeferred
###############################################################################
package VolOps::Move::JobNotDeferred;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::Move::TriggerFailed

=head2 Expected use

Exception that is used when executing volume move trigger-cutover and the command failed.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::Move::TriggerFailed
###############################################################################
package VolOps::Move::TriggerFailed;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::Move::NotDeferred

=head2 Expected use

Exception that is used when executing volume move trigger-cutover and the move is not deferred. 

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::Move::NotDeferred
###############################################################################
package VolOps::Move::NotDeferred;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::Move::NoMoveJob

=head2 Expected use

Exception that is used when executing volume move trigger-cutover and there is no move job associated with the volume/vserver.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::Move::NoMoveJob
###############################################################################
package VolOps::Move::NoMoveJob;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::Move::MoveDeferred

=head2 Expected use

Exception that is used when executing volume move and the move job is deferred. 

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::Move::MoveDeferred
###############################################################################
package VolOps::Move::MoveDeferred;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::Copy::CopyFailed

=head2 Expected use

Exception that is used when executing volume copy and the copy command failed. 

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::Copy::CopyFailed
###############################################################################
package VolOps::Copy::CopyFailed;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::Copy::DestVolumeExists

=head2 Expected use

Exception that is used when executing volume copy and the destination volume already exists.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::Copy::DestVolumeExists
###############################################################################
package VolOps::Copy::DestVolumeExists;
use base qw(VolOps::BaseException);
1;


=pod

=head1 VolOps::Copy::DestVolumeFailure

=head2 Expected use

Exception that is used when executing volume copy and there is a problem creating an object for the destination volume.

=head2 Base Class

VolOps::BaseException

=cut

###############################################################################
###  VolOps::Copy::DestVolumeFailure
###############################################################################
package VolOps::Copy::DestVolumeFailure;
use base qw(VolOps::BaseException);
1;


