#
# Copyright (c) 2009-2012 NetApp, Inc.
# All rights reserved.
#
## @summary Provide a Volume::Transition::BaseException class that is a subclass of
##          NATE::Error and other standard exceptions that are subclasses
##          of Volume::Transition::BaseException.
##
## @author astraub@netapp.com
## @status private
## @pod here

=pod

=head1 Package Name

Volume::Transition::BaseException.pm

=head1 Author

Albert Straub

=head1 Description

Volume::Transition::BaseException.pm provides a collection of basic exceptions
associated with volume transition

=head1 See Also

Package inherits from the following modules. See NATE::Error module for further
interface detail.

=head1 Volume::Transition::BaseException

=head2 Expected use

Mostly this should be the base class for more specific exceptions.
All exceptions that are subclasses of Volume::Transition::BaseException will be
caught if the catch clause checks Volume::Transition::BaseException

=head2 Base Class

NATE::Error::Simple

=cut

###############################################################################
###  Volume::Transition::BaseException
###############################################################################
package Volume::Transition::BaseException;
use base qw(NATE::BaseException);
1;

=pod

=head1 Volume::Transition::ZapiException

=head2 Expected use

Standard exception that can be used when the transition class encounters a
zapi failure while attempting transition

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::ZapiException
###############################################################################
package Volume::Transition::ZapiException;
use base qw(Volume::Transition::BaseException);
1;


=pod

=head1 Volume::Transition::CliException

=head2 Expected use

Standard exception that can be used when the transition class encounters a
cli failure while attempting transition

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::CliException
###############################################################################
package Volume::Transition::CliException;
use base qw(Volume::Transition::BaseException);
1;


=pod

=head1 Volume::Transition::InvalidOperationType

=head2 Expected use

Exception that is used when a transition operation returns an invalid
transition type error

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::InvalidOperationType
###############################################################################
package Volume::Transition::InvalidOperationType;
use base qw(Volume::Transition::BaseException);
1;


=pod

=head1 Volume::Transition::NoVolumes

=head2 Expected use

Exception that is used when a transition operation returns an error indicating
no volumes were defined for a transition operation

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::NoVolumes
###############################################################################
package Volume::Transition::NoVolumes;
use base qw(Volume::Transition::BaseException);
1;


=pod

=head1 Volume::Transition::JunctionParamInvalid

=head2 Expected use

Exception that is used when a transition operation returns an error indicating
the junction path parameter is not allowed

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::JunctionParamInvalid
###############################################################################
package Volume::Transition::JunctionParamInvalid;
use base qw(Volume::Transition::BaseException);
1;



=pod

=head1 Volume::Transition::InvalidState

=head2 Expected use

Exception that is used when a transition operation fails because the transition
job is not in the proper state for the request

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::InvalidState
###############################################################################
package Volume::Transition::InvalidState;
use base qw(Volume::Transition::BaseException);
1;


=pod

=head1 Volume::Transition::JobNotFound

=head2 Expected use

Exception that is used when a transition operation can not find the
specified job id

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::JobNotFound
###############################################################################
package Volume::Transition::JobNotFound;
use base qw(Volume::Transition::BaseException);
1;

=pod

=head1 Volume::Transition::TooManyVols

=head2 Expected use

Exception that is used when a transition operation is attempted with too
many volumes in the volinfo list.

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::TooManyVols
###############################################################################
package Volume::Transition::TooManyVols;
use base qw(Volume::Transition::BaseException);
1;

=pod

=head1 Volume::Transition::JobConflict

=head2 Expected use

Exception that is used when a failure occurs attempting to queue a transition
job due to a conflict.

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::JobConflict
###############################################################################
package Volume::Transition::JobConflict;
use base qw(Volume::Transition::BaseException);
1;

=pod

=head1 Volume::Transition::JobTerminated

=head2 Expected use

Exception that is used when a failure occurs that causes the job to terminate.

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::JobTerminated
###############################################################################
package Volume::Transition::JobTerminated;
use base qw(Volume::Transition::BaseException);
1;


=pod

=head1 Volume::Transition::Counter

=head2 Expected use

Exception that is used when a counter does not increment.

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::Counter
###############################################################################
package Volume::Transition::Counter;
use base qw(Volume::Transition::BaseException);
1;


=pod

=head1 Volume::Transition::CounterWarn

=head2 Expected use

Exception that is used when a counter increments unexpectedly.

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::CounterWarn
###############################################################################
package Volume::Transition::CounterWarn;
use base qw(Volume::Transition::BaseException);
1;

=pod

=head1 Volume::Transition::Timeout

=head2 Expected use

Exception that is used when a timeout occurs.

=head2 Base Class

Volume::Transition::BaseException

=cut

###############################################################################
###  Volume::Transition::Timeout
###############################################################################
package Volume::Transition::Timeout;
use base qw(Volume::Transition::BaseException);
1;
