package WaflFit::CModeOps;

use strict;
use vars qw(
	%snap_props
	@nodes
	%wfit
	@volumes
	$vol_name
	$aggr_name
	@results
	$response_object
	$node
	$Node
	$output
	$header
	$footer
	);

BEGIN {
  use Module::Common;
  Module::Common::setup_export_tags(
        SnapShot =>  [ qw(sfsr) ],
        SnapMirror =>  [ qw(ls_snap) ],        
  );
}


use Tharn;
use TCD;
use Data::Dumper;
use NACL::APISet;
use NACL::C::Node;
use Data::Dumper;
use WaflFit::Space qw(Get_All_Volumes);
use NACL::APISet::Exceptions::ResponseException qw(:try);    
    

$header="================================================================================";
$footer="================================================================================";



##########################################################################
# Function     : sfsr()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub sfsr() {
	my ($href,$nodes_ref,$duration,$aggr_name,$vol_name)=@_;
	@nodes=@{$nodes_ref};
	%wfit=%{$href};
	my $ndx=0;
	my $SFSRObj;	
	
	@volumes=&Get_All_Volumes(%wfit);
	my $end_time=time()+$duration;
	while(time() <$end_time) {

		$Log->log("sfsr(): Create an APISet Obj");				
		eval {
			$SFSRObj = NACL::APISet->new(
				name      => $nodes[$ndx++ % @nodes],
				category  => "Node",
				interface => "CLI",
				set       => "CMode"
			);
		};
		while($@) {
			eval {
				$SFSRObj = NACL::APISet->new(
					name      => $nodes[$ndx++ % @nodes],
					category  => "Node",
					interface => "CLI",
					set       => "CMode"
				);
			};	
		}
	    	$SFSRObj->set_timeout( 'connectrec-timeout' => $duration );	

		#Choose a file randomly,if none exists create one	
		my $ridx=int(rand(time()));
		$ridx=int(rand($ridx))%@volumes;
		$vol_name=$volumes[$ridx];
		
		$Log->log("sfsr(): Chosen volume is <$vol_name>");
		

		eval {
			$response_object=$SFSRObj->volume_show(volume=>"$vol_name","connectrec-timeout" => 1200);
			@results=@{$response_object->get_parsed_output()};
		};
		eval {
			$response_object=$SFSRObj->storage_aggregate_show(aggregate=>$results[0]->{'aggregate'},"connectrec-timeout" => 1200);
			@results=@{$response_object->get_parsed_output()};		
		};	
		$node=$results[0]->{'nodes'};
		$Node = NACL::C::Node->new( node => $node, hostrec => Tharn::host($node));	    	
		
        	$output = $Node->run( command =>
        	        "\"priv set -q test; ls /vol/$vol_name/\"" );	
        	my @files=split(/\s+/,$output);
        	my @t;
		foreach my $file(@files) {
			push @t,$file if $file !~ (/(\.|vm)/);
			
		}
		@files=@t;		
		
		#Create a snapshot
		#%snap_props;
		my $snap_name="snap_${vol_name}_$ridx";
		$snap_props{'volume'}="$vol_name";
		$snap_props{'snapshot'}="snap_${vol_name}_$ridx";
		$snap_props{'vserver'}="vs0";
		eval {
			$response_object=$SFSRObj->volume_snapshot_show(%snap_props,"connectrec-timeout" => 1200);    	
			@results=@{$response_object->get_parsed_output()};
		};
		if(@results+0 != 0) {
			$Log->log("sfsr(): Snapshot <$snap_name> already exists...deleting it now");
			eval {
				$response_object=$SFSRObj->volume_snapshot_delete(%snap_props,"connectrec-timeout" => 1200);    	
				@results=@{$response_object->get_parsed_output()};
			};		
		}
		
		eval {
			$Log->log("Creating a snapshot <$snap_name>");
			$response_object=$SFSRObj->volume_snapshot_create(%snap_props,"connectrec-timeout" => 1200);    	
		};
		#Modify all the files
		#1. Hole punching (e.g file punchhole 0 409600)
		#2. Hammer run    (e.g. hammer -f 1 /vol/wfit_node147_dedupe_vol1/Space_TC3_1 10)
		#3. Chown/mode    (e.g testsetattr /vol/wfit_node147_dedupe_vol1/Space_TC3_1 -g x -p 777)
		
		my $fcnt=1;
		$Log->log("sfsr(): Modify $fcnt files from volume <$vol_name>");
		
		$fcnt=100;
		if($#files < 100) {
			$fcnt=$#files;
		}
		while($fcnt >0) {
			@files=grep /\S/,@files;																							
			my $ridx=int(rand(time()));
			$ridx=int(rand($ridx))%@files;
			
			my $file=$files[$ridx];
			my $cmd;
			$cmd="file punchhole /vol/$vol_name/$file 0 409600;";	
			$cmd.="hammer -f 1 /vol/$vol_name/$file 10240;";
			$cmd.="testsetattr /vol/$vol_name/$file -g nobody -p 777;";
			
			eval {
			        $Node->run( command =>"\"priv set -q test;$cmd;sync\"" );	
			};
			
			sleep(10);
			
			eval {
			        $Node->run( command =>"\"priv set -q test;rm /vol/$vol_name/$file;sync\"" );	
			};	
			delete $files[$ridx];
			$fcnt--;
			
		}	
		
		#Restore all the files
		$Log->log("sfsr(): Restore all the files in volume <$vol_name> from snapshot <$snap_name>");
		foreach my $file(@files) {
			eval {
				$Log->log("sfsr(): Restoring file </vol/$vol_name/$file>");
				$SFSRObj->volume_snapshot_restore_file(%snap_props,"path"=>"/vol/$vol_name/$file","connectrec-timeout" => 1200);    	
			};	
		}		
		eval {
			$response_object=$SFSRObj->volume_snapshot_delete(%snap_props,"connectrec-timeout" => 1200);    	
			@results=@{$response_object->get_parsed_output()};
		};			
	}
	return $TCD::PASS;	
}

##########################################################################
# Function     : ls_snap()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub ls_snap() {
	my ($href,$nodes_ref,$duration)=@_;
	@nodes=@{$nodes_ref};
	%wfit=%{$href};
	my $ndx=0;
	my $LSSnapObj;	
	my $dest_vol;
	my @space_gur=qw(volume file none);
	my ($src_vol_size,$src_aggr,%init_props);
	
	#@volumes=&Get_All_Volumes(%wfit);
	$Log->log("ls_snap(): Create an APISet Obj");				
	eval {
		$LSSnapObj = NACL::APISet->new(
			name      => $nodes[$ndx++ % @nodes],
			category  => "Node",
			interface => "CLI",
			set       => "CMode"
		);
	};
	while($@) {
		eval {
			$LSSnapObj = NACL::APISet->new(
				name      => $nodes[$ndx++ % @nodes],
				category  => "Node",
				interface => "CLI",
				set       => "CMode"
			);
		};	
	}	
	
	eval {
		$response_object=$LSSnapObj->volume_show("vserver"=>"vs0","connectrec-timeout" => 1200);
		@results=@{$response_object->get_parsed_output()};
	};		
	@volumes=();
	foreach my $vol(@results) {
		push @volumes,$vol->{'volume'};
	}
	$Log->log("List of volumes filtered: <@volumes>");
	#eval {
	#	$response_object=$LSSnapObj->snapmirror_show("connectrec-timeout" => 1200);
	#	@results=$response_object->search("destination-volume"=>"_ls_cmode_ops");
	#};
	#my %ls_props;
	#my @sms=@results;
	#foreach my $sm(@sms) {
	#	$ls_props{'source-cluster'}=$sm->{'source-cluster'}
	#	$ls_props{'destination-cluster'}=$sm->{'destination-cluster'};
	#	$ls_props{'source-vserver'}=$sm->{'source-vserver'};
	#	$ls_props{'destination-vserver'}=$sm->{'destination-vserver'};
	#	$ls_props{'source-volume'}=$sm->{'source-volume'};
	#	$ls_props{'destination-volume'}=$sm->{'destination-volume'};
	#	eval {
	#		$response_object=$LSSnapObj->volume_show(volume=>$sm->{'destination-volume'},vserver=>$sm->{'destination-vserver'},"connectrec-timeout" => 1200);
	#		@results=@{$response_object->get_parsed_output()};
	#	};
	#
	#	while($#results+1 != 0) {
	#		$Log->log("Destroying volume <$sm->{'destination-volume'}>");
	#		eval {
	#			$LSSnapObj->snapmirror_abort(%ls_props,"connectrec-timeout" => 1200);
	#		};
	#		eval {
	#			$LSSnapObj->snapmirror_delete(%ls_props,"connectrec-timeout" => 1200);
	#		};								
	#		eval {
	#			$LSSnapObj->volume_unmount(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
	#		};
	#		eval {
	#			$LSSnapObj->volume_offline(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
	#		};
	#		eval {
	#			$LSSnapObj->volume_delete(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
	#		};	
	#		sleep(60);
	#		eval {
	#			$response_object=$LSSnapObj->volume_show(volume=>$sm->{'destination-volume'},vserver=>$sm->{'destination-vserver'},"connectrec-timeout" => 1200);
	#			@results=@{$response_object->get_parsed_output()};
	#		};			
	#	}
	#	
	#}
	#$Log->log("Remove all the snapmirrors created already");
	
	my $end_time=time()+$duration;
	while(time() <$end_time) {


	    	$LSSnapObj->set_timeout( 'connectrec-timeout' => $duration );	

		#Choose a volume randomly,if none exists create one	
		my $ridx=int(rand(time()));
		$ridx=int(rand($ridx))%@volumes;
		$vol_name=$volumes[$ridx];
		$dest_vol="$vol_name"."_ls_cmode_ops";
		
		$Log->log("$header");		
		$Log->log("ls_snap(): Chosen volume is <$vol_name>");
		$Log->log("ls_snap(): Destination volume is <$dest_vol>");
		$Log->log("$footer");		
		
		#$Log->log("$header");
		#$Log->log("ls_snap(): Cleanup: Delete volume <$dest_vol> if it exists already");
		#$Log->log("$footer");		
		#
		#eval {
		#	$LSSnapObj->volume_unmount(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
		#};
		#eval {
		#	$LSSnapObj->volume_offline(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
		#};
		#eval {
		#	$LSSnapObj->volume_delete(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
		#};
		
		eval {
			$LSSnapObj->volume_show(volume=>"$vol_name","vserver"=>"vs0","connectrec-timeout" => 1200);
			@results=@{$response_object->get_parsed_output()};
		};		
		
		$Log->log("$header");
		$Log->log("ls_snap(): Get some volume attributes from source volume <$vol_name>");
		$Log->log("$footer");
		
		$src_vol_size=$results[0]->{'total'};
		$src_aggr=$results[0]->{'aggregate'};
		
		my @aggrs;
		eval {
			$response_object=$LSSnapObj->storage_aggregate_show("ha-policy"=>"sfo","connectrec-timeout" => 1200);
			@aggrs=@{$response_object->get_parsed_output()};		
		};

		$Log->log("$header");
		$Log->log("ls_snap(): Choose an aggregate to create destination volume <$dest_vol>");
		$Log->log("$footer");
		
		my $ops_done=0;
		while( ($#aggrs+1 >0) && ($ops_done==0)) {
		
			my $ridx=int(rand(time()));
			$ridx=int(rand($ridx))%@aggrs;
			my $aggr=$aggrs[$ridx];			
			
			if($aggr->{'availsize'} > $src_vol_size) {
				my @vrecords;
				eval {
					$response_object=$LSSnapObj->volume_show(volume=>$dest_vol,"connectrec-timeout" => 1200);
					@vrecords=@{$response_object->get_parsed_output()};		
				};				
				if($vrecords[0]->{'volume'} ne $dest_vol) {
					$Log->log("$header");
					$Log->log("ls_snap(): Got an aggr <$aggr->{'aggregate'}>..proceeding to create destination volume <$dest_vol>");
					$Log->log("$footer");		
			
					my %vol_props;
					$vol_props{'volume'}=$dest_vol;
					$vol_props{'vserver'}="vs0";
					$vol_props{'aggregate'}=$aggr->{'aggregate'};
					$vol_props{'size'}=$src_vol_size;
					$vol_props{'space-guarantee'}=$space_gur[$ridx%@space_gur];
					$vol_props{'type'}="DP";			
					eval {
						$Log->log("$header");
						$Log->log("ls_snap(): Creating destination volume <$dest_vol>");
						$Log->log("$footer");				
						
						$LSSnapObj->volume_create(%vol_props,"connectrec-timeout" => 1200);
					};			
				}
				if(!$@) {
					my %ls_props;	
					my @vsrecords;
					eval {
						$Log->log("$header");
						$Log->log("ls_snap(): Enable dedupe on source and destination volumes");
						$Log->log("$footer");					
						
						eval {
							$LSSnapObj->volume_efficiency_on(volume=>$vol_name,"connectrec-timeout" => 1200);
						};
						eval {
							$LSSnapObj->volume_efficiency_on(volume=>$dest_vol,"connectrec-timeout" => 1200);
						};
						eval {
							$LSSnapObj->volume_efficiency_start(volume=>$vol_name,"connectrec-timeout" => 1200);
						};
						eval {
							$LSSnapObj->volume_efficiency_on(volume=>$dest_vol,"connectrec-timeout" => 1200);
						};

					};					
					eval {
						$Log->log("$header");
						$Log->log("ls_snap(): Dest vol <$dest_vol> is available now...now collecting cluster info to create snapmirror");
						$Log->log("$footer");					
						
						$response_object=$LSSnapObj->vserver_show(type=>"admin","connectrec-timeout" => 1200);
						@vsrecords=@{$response_object->get_parsed_output()};
					};
					if(!$@) {
						$ls_props{'source-cluster'}=$ls_props{'destination-cluster'}=$vsrecords[0]->{'vserver'};
						$ls_props{'source-vserver'}=$ls_props{'destination-vserver'}="vs0";
						$ls_props{'source-volume'}=$vol_name;
						$ls_props{'destination-volume'}=$dest_vol;
						
						my @smrecords;
						eval {
							$response_object=$LSSnapObj->snapmirror_show(%ls_props,"connectrec-timeout" => 1200);
							@smrecords=@{$response_object->get_parsed_output()};		
						};				
						if($smrecords[0]->{'destination-volume'} ne $dest_vol) {
							#Create a snapmirror relationship now
							$Log->log("$header");
							$Log->log("ls_snap(): Creating snapmirror relatinship with source volume <$vol_name> and destination volume <$dest_vol>");
							$Log->log("$footer");						
							eval {
								$LSSnapObj->snapmirror_create(%ls_props,type=>"LS",schedule=>"5min","connectrec-timeout" => 1200);
								#Initialize the relationship
							};
						}
						$Log->log("$header");
						$Log->log("ls_snap(): Initialize snapmirror relatinship with source volume <$vol_name> and destination volume <$dest_vol>");
						$Log->log("$footer");						
						eval {
							%init_props=undef;
							$init_props{'source-cluster'}=$vsrecords[0]->{'vserver'};
							$init_props{'source-vserver'}="vs0";
							$init_props{'source-volume'}=$vol_name;
							
							$LSSnapObj->snapmirror_initialize_ls_set(%init_props,"connectrec-timeout" => 1200);							
							#sleep(120);
						};		
						
						#if(!$@) {
							#vol resize delta
							my $delta=1024*1024*1024;
							
							#Do all of the following operations randomly
							my @sm_ops;
							@sm_ops=qw(snapmirror_show snapmirror_check snapmirror_abort snapmirror_update_ls_set snapmirror_quiesce snapmirror_resume volume_modify storage_aggregate_wafliron_start);
							while($#sm_ops+1 >0) {
								@sm_ops=grep /\S/, @sm_ops; 							
								my $ridx=int(rand(time()));
								$ridx=int(rand($ridx))%@sm_ops;
								my $sm_op=$sm_ops[$ridx];

								
								#$Log->log("@@@@@@@@@@ Remaining snapmirror ops <@sm_ops> @@@@@@@@");

								$Log->log("$header");
								$Log->log("Going to do <$sm_op>");
								$Log->log("$header");
								eval {
									if( $sm_op =~ /snapmirror/) {
										if($sm_op =~ /ls_set/) {
											$Log->log("$header");
											$Log->log("ls_snap(): Running <$sm_op> with source volume <$vol_name> and destination volume <$dest_vol>");
											$Log->log("$footer");									
											$LSSnapObj->$sm_op(%init_props,"connectrec-timeout" => 1200);    								
										} else {
											$Log->log("$header");
											$Log->log("ls_snap(): Running <$sm_op> with source volume <$vol_name> and destination volume <$dest_vol>");
											$Log->log("$footer");									
											$LSSnapObj->$sm_op(%ls_props,"connectrec-timeout" => 1200);    								
										} 
										
									} elsif ($sm_op =~ /volume_modify/) {
										my %vol_attrs;
										$vol_attrs{$vol_name}=int($src_vol_size+$delta);
										$vol_attrs{$dest_vol}=int($src_vol_size-$delta);
										
										my @vols;
										push @vols,keys %vol_attrs;
										while($#vols+1 >0) {
											@vols=grep /\S/,@vols;																					
											#$Log->log("+++++ Remaining vols <@vols> +++++");										
											my $ridx=int(rand(time()));
											$ridx=int(rand($ridx))%@vols;
											my $vol=$vols[$ridx];
											
											$Log->log("$header");
											$Log->log("ls_snap(): Resizing volume <$vol>");
											$Log->log("$footer");	
											$LSSnapObj->$sm_op(volume=>$vol,vserver=>"vs0",size=>$vol_attrs{$vol},"connectrec-timeout" => 1200);																			
											delete $vols[$ridx];											
										}
									} elsif ($sm_op =~ /wafliron/) {
										my @agrs=($src_aggr,$aggr->{'aggregate'});
										
										while($#agrs+1 >0) {
											@agrs=grep /\S/,@agrs;										
											#$Log->log("+++++ Remaining aggrs <@agrs> +++++");										
											my $ridx=int(rand(time()));
											$ridx=int(rand($ridx))%@agrs;
											my $agr=$agrs[$ridx];
										
											$Log->log("$header");
											$Log->log("ls_snap(): Running wafliron on aggregate <$agr>");
											$Log->log("$footer");							
								
											$LSSnapObj->$sm_op(aggregate=>$agr,"connectrec-timeout" => 1200);									
											delete $agrs[$ridx];
										}
									}
								};								
								delete $sm_ops[$ridx];
							}
							
							sleep(120);
							#Stop wafliron
							eval {
								$Log->log("$header");
								$Log->log("ls_snap(): Stop wafliron on source aggregate <$src_aggr>");
								$Log->log("$footer");							
								
								$LSSnapObj->storage_aggregate_wafliron_stop(aggregate=>$src_aggr,"connectrec-timeout" => 1200);
							};
							eval {
								$Log->log("$header");
								$Log->log("ls_snap(): Stop wafliron on destination aggregate <$aggr->{'aggregate'}>");
								$Log->log("$footer");							
								
								$LSSnapObj->storage_aggregate_wafliron_stop(aggregate=>$aggr->{'aggregate'},"connectrec-timeout" => 1200);
							};								
							sleep(60);
							
							#Abort the relationship
							eval {
								$Log->log("$header");
								$Log->log("ls_snap(): Abort snapmirror relationship with source volume <$vol_name> and destination volume <$dest_vol>");
								$Log->log("$footer");							

								$LSSnapObj->snapmirror_abort(%ls_props,"connectrec-timeout" => 1200);    								
							};			
							
							eval {
								$LSSnapObj->snapmirror_delete(%ls_props,"connectrec-timeout" => 1200);
							};	
							
							sleep(60);
							
							#Delete the destination volume
							$Log->log("$header");
							$Log->log("ls_snap(): Destroying the destination volume <$dest_vol>");
							$Log->log("$footer");							

							$Log->log("$header");
							$Log->log("ls_snap(): Cleanup");
							$Log->log("$footer");						
							eval {
								$LSSnapObj->snapmirror_abort(%ls_props,"connectrec-timeout" => 1200);
							};
							eval {
								$LSSnapObj->snapmirror_delete(%ls_props,"connectrec-timeout" => 1200);
							};										
							
							eval {
								$LSSnapObj->volume_unmount(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
							};
							eval {
								$LSSnapObj->volume_offline(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
							};
							eval {
								$LSSnapObj->volume_delete(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
							};							
						#} else {
						#	$Log->log("$header");
						#	$Log->log("ls_snap(): LS Snapmirror creation failed...cleaning up the snapmirror stuff done so far");
						#	$Log->log("$footer");						
						#	eval {
						#		$LSSnapObj->snapmirror_abort(%ls_props,"connectrec-timeout" => 1200);
						#	};
						#	eval {
						#		$LSSnapObj->snapmirror_delete(%ls_props,"connectrec-timeout" => 1200);
						#	};								
						#	eval {
						#		$LSSnapObj->volume_unmount(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
						#	};
						#	eval {
						#		$LSSnapObj->volume_offline(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
						#	};
						#	eval {
						#		$LSSnapObj->volume_delete(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
						#	};								
						#}						
					} 
				} else {
					$Log->log("$header");
					$Log->log("ls_snap(): Failed to create destination volume <$dest_vol>... proceeding with another aggregate");
					$Log->log("$footer");				
					eval {
						$LSSnapObj->volume_unmount(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
					};
					eval {
						$LSSnapObj->volume_offline(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
					};
					eval {
						$LSSnapObj->volume_delete(volume=>"$dest_vol","vserver"=>"vs0","connectrec-timeout" => 1200);
					};					
				}
				$ops_done=1;
			} else {
				$Log->log("@@@@ Aggr <$aggr->{'aggregate'}> does not have enough space: Available <$aggr->{'availsize'}>, Required <$src_vol_size>@@@@");			
				@aggrs=grep /\S/,@aggrs;
				delete $aggrs[$ridx];				
			}

		}
	}
	return $TCD::PASS;	
}

1;
