package WaflFit::DataManip;

use strict;
use vars qw(
	$header
	$footer
	$node
	$vol_name
	$aggr_name
	$seed
	$duration
	$end_time
	$response
	$num_lifs
	$num_clients
	$FileOpsObj
	$avail_size
	$total_size
	$aggr_avail_size
	$fname
	$fcall
	$rsize
	$data_size
	$data_size_in_bytes
	$incr_size
	$vol_new_size
	$type
	$href
	$cli_int
	$client
	$lif_int
	$selected_ip
	$mnt_pt
	$fcnt
	$avail_filecnt
	$fileops
	$fsize
	
	@clients
	@lifs
	@vol_parsed_output
	@results
	@data_grow_refs
	@data_grow_methods
	@data_shrink_refs
	@data_shrink_methods
	@fsizes
	@data_manips
	@data_manips_refs
	
	
	%temp_list
	%dist_list	
	
	);

BEGIN {
  use Module::Common;
  Module::Common::setup_export_tags(
        FileOps =>  [ qw(FileOps Set_fileops) ],
  );
}


use Tharn;
use TCD;
use Data::Dumper;
use NACL::APISet;
use NACL::C::Node;
use Data::Dumper;
use DataManip;
use Populate qw(create_set delete_set);
use GenericMount qw(mount umount);
use NACL::APISet::Exceptions::ResponseException qw(:try);    
    

$header="================================================================================";
$footer="================================================================================";
%dist_list = (
                                "1b"   => 1.03,
				"2b"   => 0.01,
                                "4b"   => 0.03,
                                "8b"   => 0.05,
                                "16b"  => 0.39,
                                "32b"  => 0.91,
                                "128b" => 0.48,
                                "256b" => 1.02,
                                "512b" => 1.89,
                                "1k"   => 7.18,
                                "2k"   => 18.88,
                                "4k"   => 21.85,
                                "8k"   => 17.27,
                                "16k"  => 14.63,
                                "32k"  => 7.98,
                                "64k"  => 2.84,
                                "128k" => 1.10,
                                "256k" => 0.87,
                                "512k" => 0.63,
                                "1m"   => 0.51,
                                "2m"   => 0.03,
                                "128m" => 0.20,
                                "512m" => 0.10,
                                "1g"   => 0.05,
                                "2g"   => 0.02,
                                "4g"   => 0.05
                        );
    
sub convert_to_bytes( ) {
	my $size=shift;
	my $size_in_bytes;
	if (!$size) { 
		return -1;
	}
	
	if($size =~ /^(\d+)[kK]$/ ) {
		$size_in_bytes=$1 * 1024;
	} elsif($size =~ /^(\d+)[Mm]$/ ) {
		$size_in_bytes=$1 * 1024 * 1024;
	} elsif($size =~ /^(\d+)[Gg]$/ ) {
		$size_in_bytes=$1 * 1024 * 1024 * 1024;
	} elsif($size =~ /^(\d+)[Tt]$/ ) {
		$size_in_bytes=$1 * 1024 * 1024 * 1024 * 1024;
	} elsif($size =~ /^(\d+)[bB]$/ ) {
		$size_in_bytes=$1;
	} elsif($size =~ /^\d+$/) {
		$size_in_bytes=$size;
	}
	$Log->log("@@@@ covert_to_bytes receives <$size> and returns <$size_in_bytes> @@@@");
	return $size_in_bytes;
}

##########################################################################
# Function     : Set_fileops()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub Set_fileops() {
	my ($node,$fops,@vols)=@_;
        my $low_cnt=-1;
        my $fileops;
        
	#Create APISet object
	my $FileOpsObj = NACL::APISet->new(
		name      => $node,
		category  => "Node",
		interface => "CLI",
		set       => "CMode"
	);

        $FileOpsObj->set_timeout( 'connectrec-timeout' => 1200 );	        
        for(my $k=0;$k<@vols;$k++) {
                $vol_name=$vols[$k];
                $vol_name=~ s/-/_/g;
		eval {
        		$response = $FileOpsObj->volume_show(volume=>$vol_name,"connectrec-timeout" => 1200);
			@results=@{$response->get_parsed_output()};
		};
		if($@) {
                	$Log->log("volume_show() failed, error: $@");
                	return $TCD::FAIL;
                }

                if($low_cnt == -1 || ($low_cnt > $results[0]->{'files'})) {
                	$low_cnt = $results[0]->{'files'};
                }
         }


        #Process the file iops speed level
	return (int($fops*$low_cnt/100));
}


sub fileops_setup() {
        ($vol_name)=@_;
        # Randomly select a client
        $cli_int = int(rand($seed))%$num_clients;
        $client = $clients[$cli_int];

        # randomly select a data lif
        $lif_int = undef;
        $selected_ip = undef;
        $lif_int = int(rand($seed*2))%$num_lifs;
        # make sure the data lif's vserver matches the volume's vserver
        $selected_ip = $lifs[$lif_int]->{address};

        #mount 'root' volume
        #mount("ip-address" => $selected_ip, "client" => $client, "mount-point" => $mnt_pt, "options" => $mount_ops);
        $mnt_pt = mount(
                          $selected_ip,
                          '/',
                          client => $client,
                          chmod  => '0777',
                          mntopts => "hard,intr,noquota,vers=3",
                     );
	my $path="$mnt_pt/" . "$vol_name/";
        my @subtests= subtest( sub {
		        my $ch = host $client;
		        my $cc=  connect("#rsh",'-timeout',1200);        
		        my $t=time();
			say($cc,"mv $path/regular_files $path/regular_files_old$t;mv $path/sparse_files $path/sparse_files_old$t;mv $path/dup_files $path/dup_files_old$t;mv $path/misc $path/misc_old$t",'-timeout',7200);        
			
			$ch = host $client;
			$cc=  connect("#rsh",'-timeout',1200);        
                	say($cc,"mkdir $path/regular_files;mkdir $path/sparse_files;mkdir $path/dup_files;mkdir $path/misc",'-timeout',7200);
                	
                	$ch = host $client;
                	$cc=  connect("#rsh",'-timeout',1200);        
			say($cc,"chmod 777 $path/regular_files;chmod 777 $path/sparse_files;chmod 777 $path/dup_files;chmod 777 $path/misc",'-timeout',7200);                	
        	},'-iffatal','-ok','-ifscript','-ok');
        Subtest::wait_finish(subtests=>\@subtests);
        return $TCD::PASS;
}


sub fileops_cleanup() {
        ($vol_name)=@_;
        # Randomly select a client
        $cli_int = int(rand($seed))%$num_clients;
        $client = $clients[$cli_int];

        # randomly select a data lif
        $lif_int = undef;
        $selected_ip = undef;
        $lif_int = int(rand($seed*2))%$num_lifs;
        # make sure the data lif's vserver matches the volume's vserver
        $selected_ip = $lifs[$lif_int]->{address};

        #mount 'root' volume
        #mount("ip-address" => $selected_ip, "client" => $client, "mount-point" => $mnt_pt, "options" => $mount_ops);
        $mnt_pt = mount(
                          $selected_ip,
                          '/',
                          client => $client,
                          chmod  => '0777',
                          mntopts => "hard,intr,noquota,vers=3",
                     );
	my $path="$mnt_pt/" . "$vol_name/";
        my @subtests= subtest( sub {
		        my $ch = host $client;
		        my $cc=  connect("#rsh",'-timeout',1200);        
                	say($cc,"rm -rf $path/regular_files_old*",'-timeout',7200);
                	
                	$ch = host $client;
		        $cc=  connect("#rsh",'-timeout',1200);        
                	say($cc,"rm -rf $path/sparse_files_old*",'-timeout',7200);
                	
                	$ch = host $client;
                	$cc=  connect("#rsh",'-timeout',1200);        
			say($cc,"rm -rf $path/dup_files_old*",'-timeout',7200);                	
			
			
                	$cc=  connect("#rsh",'-timeout',1200);        
			say($cc,"rm -rf $path/misc_old*",'-timeout',7200);                				
        	},'-bg','-iffatal','-ok','-ifscript','-ok');
        Subtest::wait_finish(subtests=>\@subtests);
        return $TCD::PASS;
}

sub FileOps() {
	#Get nodename, volname,seed,duration);
	($node,$vol_name,$seed,$duration,$fileops,@clients)=@_;

	#Create APISet object
	$FileOpsObj = NACL::APISet->new(
		name      => $node,
		category  => "Node",
		interface => "CLI",
		set       => "CMode"
	);

        $FileOpsObj->set_timeout( 'connectrec-timeout' => $duration );	
        
	#Other varaibles
        $response = $FileOpsObj->network_interface_show(vserver=>"vs0","status-admin"=>"up","connectrec-timeout" => 1200);
	@lifs=@{$response->get_parsed_output()};	
	
        $num_lifs=$#lifs+1;		
	$num_clients=$#clients+1;
	$vol_name=~ s/-/_/g;	
	
        #Grow methods
        $data_grow_methods[0]="grow_new_regular_file";
	$data_grow_refs[0]=\&grow_new_regular_file;	

        $data_grow_methods[1]="grow_new_sparse_file";
	$data_grow_refs[1]=\&grow_new_sparse_file;

        $data_grow_methods[2]="grow_dup_regular_file";
	$data_grow_refs[2]=\&grow_dup_regular_file;

        $data_grow_methods[3]="grow_dup_sparse_file";
	$data_grow_refs[3]=\&grow_dup_sparse_file;

        $data_grow_methods[4]="grow_regular_extend";
	$data_grow_refs[4]=\&grow_regular_extend;

        $data_grow_methods[5]="grow_holey_extend";
	$data_grow_refs[5]=\&grow_holey_extend;
        
        #Shrink methods
        $data_shrink_methods[0]="shrink_rm_regular_file";
	$data_shrink_refs[0]=\&shrink_rm_regular_file;

        $data_shrink_methods[1]="shrink_rm_sparse_file";
	$data_shrink_refs[1]=\&shrink_rm_sparse_file;

        $data_shrink_methods[2]="shrink_regular_truncate";
	$data_shrink_refs[2]=\&shrink_regular_truncate;

        $data_shrink_methods[3]="shrink_holey_truncate";
	$data_shrink_refs[3]=\&shrink_holey_truncate;
        
        #Overall methods
        @data_manips=(@data_grow_methods,@data_shrink_methods);
	@data_manips_refs=(@data_grow_refs,@data_shrink_refs);	
	
        ### Created directories
        &fileops_setup($vol_name);

	#Run the entire stuff for the specified duraiton
	$duration=$duration*60*60;
	$end_time=scalar(time())+$duration;

	$Log->log("@@@@ Volume name <$vol_name> @@@@");
	$Log->log("@@@@ Node <$node> @@@@");
	$Log->log("@@@@ Seed <$seed> @@@@");
	$Log->log("@@@@ This subroutine will run for <$duration> seconds @@@@");		

	$Log->log("@@@@ Data manipulation starts at<" . int(scalar(time())) . "> @@@@@");
	$Log->log("@@@@ Data manipulation expected to end at <$end_time> @@@@@");
	

        while( int(scalar(time())) < $end_time) {
		subtest( sub {
			&fileops_cleanup($vol_name);
			},'-bg','-iffatal','-ok','-ifscript','-ok');
		$response = $FileOpsObj->volume_show(volume=>$vol_name,"connectrec-timeout" => 1200);
	        @vol_parsed_output=@{$response->get_parsed_output()};
	        
                #Step4: Get the volume's available size & used size
                $avail_size=$vol_parsed_output[0]->{'available'};
                $total_size=$vol_parsed_output[0]->{'total'};
                $aggr_name=$vol_parsed_output[0]->{'aggregate'};
                
                $Log->log("#### Volume <$vol_name> available size <$avail_size> at time " . int(scalar(time())) . " #####");
              
		eval {
                       	#Data operation is 'grow'. Pick any grow method
                       	my $i=int(rand($seed))%($#data_grow_methods+1);
                       	$fname=$data_grow_refs[$i];
                       	$fcall=$data_grow_methods[$i];
                       	$rsize= int(rand($avail_size))%$avail_size;
                       	$data_size_in_bytes= ($rsize==0)?int(0.1*$avail_size):$rsize;
                       	$Log->log("#### Function called <$fcall> with data size <$data_size_in_bytes> ####");
			&$fname($vol_name,$data_size_in_bytes);
			
		};
		$Log->log("+++++++ We hit <No space error> +++++. More details: " . Dumper($@));
		if( $@ =~ /No space/i) {
			$Log->log("#### We hit <No space> error on volume <$vol_name> after calling the function <$fcall> with data size <$data_size_in_bytes> ####");
			$Log->log("#### We are proceeding with increasing the volume size or delete some data sets in the volume ####");
			#Increase the volume size or reduce the dataset as well
			#Get the aggregate available size
			eval {
				$response = $FileOpsObj->storage_aggregate_show(aggregate=>$aggr_name,"connectrec-timeout" => 1200);
			        @results=@{$response->get_parsed_output()};		
			        $aggr_avail_size=$results[0]->{'availsize'};
			        $Log->log("#### Aggregate containing volume is <$aggr_name> having available size <$aggr_avail_size> ####");
			        
			        #Calculate volume resize value and resize it
			        if($total_size <$aggr_avail_size) {
			        	$incr_size=$total_size;
			        } elsif(2*$data_size_in_bytes <$aggr_avail_size) {
			        	$incr_size=int(1.5*$data_size_in_bytes);
			        } elsif($data_size_in_bytes <$aggr_avail_size) {
			        	$incr_size=$data_size_in_bytes;
			        } else {
			        	$incr_size=$aggr_avail_size;
			        }
			        $vol_new_size=$total_size+$data_size_in_bytes;
			        $Log->log("#### Attempting to increase the volume size <$vol_name>: Old size <$total_size>, New size: <$vol_new_size> ####");
			        $response = $FileOpsObj->volume_modify(volume=>$vol_name,size=>"$vol_new_size","connectrec-timeout" => 1200);	
			};
			## Handle aggregate out of space error
			if($@ =~ /not enough space in the aggregate/) {
				$Log->log("#### Aggregate <$aggr_name> is running out of space, going to delete some data set ####");
			        #Reduce the data set size until we get available size is min 30% of total size
				$response = $FileOpsObj->volume_show(volume=>$vol_name,"connectrec-timeout" => 1200);
			        @results=@{$response->get_parsed_output()};		
			        my $asize=$results[0]->{'available'};
			        
			        $Log->log("#### Volume available size before start reducing it is <$asize> ####");
			        while($asize < int(0.3*$total_size)) {
                       			my $i=int(rand($seed*3))%($#data_shrink_methods+1);
                       			$fname=$data_shrink_refs[$i];
                       			$fcall=$data_shrink_methods[$i];
                       			$data_size_in_bytes=int(0.1*$total_size);
                       			&$fname($vol_name,$data_size_in_bytes);
                       			
					$response = $FileOpsObj->volume_show(volume=>$vol_name,"connectrec-timeout" => 1200);
				        @results=@{$response->get_parsed_output()};		
			        	$asize=$results[0]->{'available'};                       			
					$Log->log("#### Volume <$vol_name> available size after calling the function <$fcall> is <$asize> ####");			        	
                       		}
                       		$Log->log("#### Volume available size at the end of dataset reduction is <$asize> ####");
                       	}
		}
		$Log->log("@@@@@ Current time is " . int(scalar(time())) . " ,end time <$end_time> @@@@");
	}
	return $TCD::PASS;
}

##########################################################################
# Function     : files_distribution()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
#File distribution api
#     Size  Percentage
#         1  1.03%
#         2  0.00%
#         4  0.03%
#         8  0.05%
#        16  0.40%
#        32  0.91%
#        64  0.45%
#       128  0.48%
#       256  1.02%
#       512  1.89%
#      1024  7.18%
#      2048 18.88%
#      4096 21.85%
#      8192 17.27%
#     16384 14.63%
#     32768  7.98%
#     65536  2.84%
#    131072  1.10%
#    262144  0.87%
#    524288  0.63%
#   1048576  0.51%
sub files_distribution {
        my ($total_data_size,$avail_filecnt)=@_;
        my $size;
	%temp_list=();

        if(! $total_data_size) {
                $Log->error("Data size is null");
                return $TCD::FAIL;
        }
        #Calculate the file count for each size
	#Get all the file sizes and process them randomly
	@fsizes=keys %dist_list;
        while($#fsizes >=0) {
		my $i=int(rand($seed/2))%($#fsizes+1);
		$size=$fsizes[$i];
		my $val=$dist_list{$size};
                $data_size=int($val*$total_data_size);
                $size=~ /(.*)([bkmg])/;
                my $div=$1;
                my $fact=$2;
                if($fact eq "k") {
                        $div*=1024;
                } elsif ($fact eq "m") {
                        $div*=1024*1024;
                } elsif ($fact eq "g") {
                        $div*=1024*1024*1024;
                }

		#Number of files should not exceed the max limit supported by
		#the volume
		my $calc_fcnt=int($data_size/$div);
		if( ($avail_filecnt==0) || ($calc_fcnt <= $avail_filecnt) ) {
                	$temp_list{"$size"}=$calc_fcnt;
		} elsif ( $calc_fcnt > $avail_filecnt) {
			$temp_list{"$size"}=$avail_filecnt;
			last;
		}
		splice @fsizes,$i,1;
        }
}

##########################################################################
# Function     : DistributeFiles()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub DistributeFiles () {

        ($vol_name,$type,$href)=@_;
        # Randomly select a client
        $cli_int = int(rand($seed+1))%$num_clients;
        $client = $clients[$cli_int];

        # randomly select a data lif
        $lif_int = undef;
        $selected_ip = undef;
        $lif_int = int(rand($seed+8))%$num_lifs;
        # make sure the data lif's vserver matches the volume's vserver
        $selected_ip = $lifs[$lif_int]->{address};

        #mount 'root' volume
        #mount("ip-address" => $selected_ip, "client" => $client, "mount-point" => $mnt_pt, "options" => $mount_ops);
        $mnt_pt = mount(
                          $selected_ip,
                          '/',
                          client => $client,
                          chmod  => '0777',
                          mntopts => "hard,intr,noquota,vers=3",
                     );


	#Collect all the keys in the dist list
	$Log->log("***** File ops are=<$fileops> ****");
	@fsizes=keys %{$href};
	if( $#fsizes < 0) {
		$Log->log("Distribution list is empty");
		return $TCD::PASS;
	}
	for(my $i=0;$i<@fsizes;$i++) {
        	## Write data to volume using populate
        	$fsize=&convert_to_bytes($fsizes[$i]);
        	$fcnt=($href->{$fsizes[$i]}%$fileops)+1;
		$Log->log("**** Type=<$type>, File size=<$fsizes[$i]>, File size in bytes =<$fsize>,File count=<$fcnt> ****");
		if($type eq "regular") {
			my $path="$mnt_pt/" . "$vol_name/" . "regular_files/" . "$fsizes[$i]";        		
                        my $ch = host $client;
                        my $cc=  connect("#rsh",'-timeout',1200);
                        eval {
                        	say($cc,"mkdir -p $path ;chmod 777 $path",'-timeout',$duration);        		
                        };
                        
                        $Log->log("@@@@@ populate_tree function is called @@@@");
        		my ($seed, $subpaths, $subseeds)= populate_tree($client,
                                                        "$path",
                                                        SEED => '1929978431',
                                                        FILE_AVG => $fcnt,
                                                        BREADTH_AVG => $fsize,
                                                        SCRAMBLE => 1,
                                                        DIR_AVG => 0,
                                                        PRECREATE => 0,
                                                        DEPTH_AVG => 0);
        		if (!defined($seed)) {
              			$Log->log("Error in populate: $@");
              			return $TCD::FAIL;
        		}

		} elsif ($type eq "sparse") {
			my $path="$mnt_pt/" . "$vol_name/" . "sparse_files/" . "$fsizes[$i]";
                        my $ch = host $client;
                        my $cc=  connect("#rsh",'-timeout',1200);
                        eval {
                        	say($cc,"mkdir -p $path;chmod 777 $path",'-timeout',$duration);
                        };
                        $Log->log("@@@@@ create_set function is called @@@@");
			Populate::create_set(
             				clients => [$client],
             				testdir => $path,
             				base_name => "sparse",
             				type => "sparse",
             				count => $fcnt,
             				size => $fsize,
             				max_size => $fsize,
					skip => 4096,
					gap => 1024,
					timeout => ($fcnt>6000)?$fcnt:6000
				  );		
		} elsif ($type eq "dup_reg") {
			#We are going to duplicate $fcnt/2 amount of files
			my $srcpath="$mnt_pt/" . "$vol_name/" . "dup_files/" . "$fsizes[$i]" . "/${type}_orig";
                	my $ch = host $client;
                	my $cc=  connect("#rsh",'-timeout',1200);
                	eval {
                		say($cc,"mkdir -p $srcpath;chmod 777 $srcpath",'-timeout',$duration);
                	};
                        my ($seed, $subpaths, $subseeds)= populate_tree($client,
                                                        "$srcpath",
                                                        SEED => '1929978431',
                                                        FILE_AVG => int($fcnt/2)>0?int($fcnt/2):1,
                                                        BREADTH_AVG => $fsize,
                                                        SCRAMBLE => 1,
                                                        DIR_AVG => 0,
                                                        PRECREATE => 1,
                                                        DEPTH_AVG => 0);
                        if (!defined($seed)) {
                                $Log->log("Error in populate: $@");
                                return $TCD::FAIL;
                        }

			#Copy the files and make duplicates
			my $duppath="$mnt_pt/" . "$vol_name/" . "dup_files/" . "$fsizes[$i]" . "/${type}_dup";
                        $ch = host $client;
                        $cc=  connect("#rsh",'-timeout',1200);
                        say($cc,"sudo chmod 777 $srcpath",'-timeout',$duration);                                                
                        
                        $cc=  connect("#rsh",'-timeout',1200);
                        say($cc,"mkdir -p $duppath;chmod 777 $duppath; cp -r $srcpath $duppath",'-timeout',$duration);



		} elsif ($type eq "dup_sparse") {
                       #We are going to duplicate $fcnt/2 amount of files
                        my $srcpath="$mnt_pt/" . "$vol_name/" . "dup_files/" . "$fsizes[$i]" . "/${type}_orig";
                        my $ch = host $client;
                        my $cc=  connect("#rsh",'-timeout',1200);
                        eval {
                        	say($cc,"mkdir -p $srcpath",'-timeout',$duration);
                        };

                        Populate::create_set(
                                        clients => [$client],
                                        testdir => $srcpath,
                                        base_name => "sparse",
                                        type => "sparse",
                                        count => int($fcnt/2)>0?int($fcnt/2):1,
                                        size => $fsize,
                                        max_size => $fsize,
                                        skip => 10240,
                                        gap => 512,
					timeout => ($fcnt>6000)?$fcnt:6000,
                                  );
                        #Copy the files and make duplicates
                        my $duppath="$mnt_pt/" . "$vol_name/" . "dup_files/" . "$fsizes[$i]" . "/${type}_dup";
                        $ch = host $client;
                        $cc=  connect("#rsh",'-timeout',1200);
			say($cc,"sudo chmod 777 $srcpath",'-timeout',$duration);                                                
                        
                        $cc=  connect("#rsh",'-timeout',1200);                        
                        say($cc,"mkdir -p $duppath;chmod 777 $duppath; cp -r $srcpath $duppath",'-timeout',$duration);
                }
	}
}

##########################################################################
# Function     : DeleteFiles()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub DeleteFiles () {

        my($vol_name,$type,$href)=@_;
        # Randomly select a client
        $cli_int = int(rand($seed+199))%$num_clients;
        $client = $clients[$cli_int];

        # randomly select a data lif
        $lif_int = undef;
        $selected_ip = undef;
        $lif_int = int(rand($seed+8000))%$num_lifs;
        # make sure the data lif's vserver matches the volume's vserver
        $selected_ip = $lifs[$lif_int]->{address};

        #mount 'root' volume
        #mount("ip-address" => $selected_ip, "client" => $client, "mount-point" => $mnt_pt, "options" => $mount_ops);
        $mnt_pt = mount(
                          $selected_ip,
                          '/',
                          client => $client,
                          chmod  => '0777',
                          mntopts => "hard,intr,noquota,vers=3",
                     );

        #Collect all the keys in the dist list
        @fsizes=keys %{$href};
        if( $#fsizes < 0) {
                $Log->log("Distribution list is empty");
                return $TCD::PASS;
        }
        for(my $i=0;$i<@fsizes;$i++) {
		my $path="$mnt_pt/" . "$vol_name/" . "${type}_files/" . "$fsizes[$i]";
                my $ch = host $client;
                my $cc=  connect("#rsh",'-timeout',1200);
                say($cc,"chmod 777 $path ;(cd $path && rm -rf $path/*) || rm -rf $path",'-timeout',1200);
        }
}

##########################################################################
# Function     : Available_File_Count()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub Available_File_Count() {
	my $vol_name=shift;
	$Log->log("@@@@ Available_File_Count is called @@@");
	$response = $FileOpsObj->volume_show(volume=>$vol_name,"connectrec-timeout" => 1200);
	$Log->log("(((((((((( volume show is executed )))))))");
       	@vol_parsed_output=@{$response->get_parsed_output()};
	if($@) {
		$Log->log("volume_show() failed, error: $@");
		return $TCD::FAIL;
	}
	
	my $total_files=$vol_parsed_output[0]->{'files'};
	my $used_files=$vol_parsed_output[0]->{'files-used'};
	my $avail_files=$total_files-$used_files;
	$Log->log("@@@@@ Avail files returned =<$avail_files> @@@@");
	return $avail_files;	
}

##########################################################################
# Function     : Used_File_Count()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub Used_File_Count() {
        my $vol_name=shift;
        $Log->log("@@@@ Used_File_Count is called @@@");
	$response = $FileOpsObj->volume_show(volume=>$vol_name,"connectrec-timeout" => 1200);
	$Log->log("(((((((((( volume show is executed )))))))");
       	@vol_parsed_output=@{$response->get_parsed_output()};
	if($@) {
		$Log->log("volume_show() failed, error: $@");
		return $TCD::FAIL;
	}

	$Log->log("@@@@@ Used files returned =<$vol_parsed_output[0]->{'files-used'}> @@@@");
        return $vol_parsed_output[0]->{'files-used'};
}

##########################################################################
# Function     : grow_new_regular_file()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub grow_new_regular_file() {
	($vol_name,$data_size_in_bytes)=@_;
	$Log->log("Function name=<grow_new_regular_file>,Volume name=<$vol_name>, Data size=<$data_size_in_bytes>");
	my $avail_filecnt=&Available_File_Count($vol_name);
	if($avail_filecnt>0) {
		&files_distribution($data_size_in_bytes,$avail_filecnt);	
	}
	$Log->log("Files list" . Dumper(%temp_list));
	&DistributeFiles($vol_name,"regular",\%temp_list);
	return $TCD::PASS;
}

##########################################################################
# Function     : grow_new_sparse_file()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub grow_new_sparse_file() {
	($vol_name,$data_size_in_bytes)=@_;
	$Log->log("Function name=<grow_new_sparse_file>,Volume name=<$vol_name>, Data size=<$data_size_in_bytes>");
        my $avail_filecnt=&Available_File_Count($vol_name);
        if($avail_filecnt>0) {
                &files_distribution($data_size_in_bytes,$avail_filecnt);
        }
        $Log->log("Files list" . Dumper(%temp_list));
	&DistributeFiles($vol_name,"sparse",\%temp_list);
	return $TCD::PASS;
}

##########################################################################
# Function     : grow_dup_regular_file()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub grow_dup_regular_file() {
	($vol_name,$data_size_in_bytes)=@_;
	$Log->log("Function name=<grow_dup_regular_file>,Volume name=<$vol_name>, Data size=<$data_size_in_bytes>");
        my $avail_filecnt=&Available_File_Count($vol_name);
        if($avail_filecnt>0) {
                &files_distribution($data_size_in_bytes,$avail_filecnt);
        }
        $Log->log("Files list" . Dumper(%temp_list));
	&DistributeFiles($vol_name,"dup_reg",\%temp_list);
	return $TCD::PASS;
}

##########################################################################
# Function     : grow_dup_sparse_file()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub grow_dup_sparse_file() {
	($vol_name,$data_size_in_bytes)=@_;
	$Log->log("Function name=<grow_dup_sparse_file>,Volume name=<$vol_name>, Data size=<$data_size_in_bytes>");
        my $avail_filecnt=&Available_File_Count($vol_name);
        if($avail_filecnt>0) {
                &files_distribution($data_size_in_bytes,$avail_filecnt);
        }
        $Log->log("Files list" . Dumper(%temp_list));
	&DistributeFiles($vol_name,"dup_sparse",\%temp_list);
	return $TCD::PASS;
}

##########################################################################
# Function     : grow_regular_extend()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub grow_regular_extend() {
	($vol_name,$data_size_in_bytes)=@_;
	$Log->log("Function name=<grow_regular_extend>,Volume name=<$vol_name>, Data size=<$data_size_in_bytes>");
	return $TCD::PASS;
}

##########################################################################
# Function     : grow_holey_extend()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub grow_holey_extend() {
	($vol_name,$data_size_in_bytes)=@_;
	$Log->log("Function name=<grow_holey_extend>,Volume name=<$vol_name>, Data size=<$data_size_in_bytes>");
	return $TCD::PASS;
}

##########################################################################
# Function     : shrink_rm_regular_file()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub shrink_rm_regular_file() {
	($vol_name,$data_size_in_bytes)=@_;
	$Log->log("Function name=<shrink_rm_regular_file>,Volume name=<$vol_name>, Data size=<$data_size_in_bytes>");
        my $used_fcnt=&Used_File_Count($vol_name);
        if($used_fcnt>0) {
                &files_distribution($data_size_in_bytes);
        }
        $Log->log("Files list" . Dumper(%temp_list));
	&DeleteFiles($vol_name,"regular",\%temp_list);

	return $TCD::PASS;
}

##########################################################################
# Function     : shrink_rm_sparse_file()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub shrink_rm_sparse_file() {
        ($vol_name,$data_size_in_bytes)=@_;
        $Log->log("Function name=<shrink_rm_sparse_file>,Volume name=<$vol_name>, Data size=<$data_size_in_bytes>");
        my $avail_filecnt=&Available_File_Count($vol_name);
        my $used_fcnt=&Used_File_Count($vol_name);
        if($used_fcnt>0) {
                &files_distribution($data_size_in_bytes);
        }
        $Log->log("Files list" . Dumper(%temp_list));
	&DeleteFiles($vol_name,"sparse",\%temp_list);

        return $TCD::PASS;
}

##########################################################################
# Function     : shrink_regular_truncate()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub shrink_regular_truncate() {
	($vol_name,$data_size_in_bytes)=@_;
	$Log->log("Function name=<shrink_regular_truncate>,Volume name=<$vol_name>, Data size=<$data_size_in_bytes>");
	return $TCD::PASS;
}

##########################################################################
# Function     : shrink_holey_truncate()
# Input        : None
# Return value :
# Summary      :
#
##########################################################################
sub shrink_holey_truncate() {
	($vol_name,$data_size_in_bytes)=@_;
	$Log->log("Function name=<shrink_holey_truncate>,Volume name=<$vol_name>, Data size=<$data_size_in_bytes>");
	return $TCD::PASS;
}

1;