package WaflFit::Dedupe;

use strict;
use vars qw(
	$header
	$footer
	$Trace
	);

BEGIN {
  use Module::Common;
  Module::Common::setup_export_tags(
        convert =>  [ qw(convert_to_bytes) ],
        create =>   [ qw(create_files_in_volume) ],
        client =>   [ qw(healthy_clients collect_clients_from_file)],
        volume =>   [ qw(Get_All_Volumes) ],
  );
}


use Tharn;
use TCD;
use Data::Dumper;
use TracePkg;    
    

$header="================================================================================";
$footer="================================================================================";
$Trace = new TracePkg();
    
sub convert_to_bytes( ) {
	my $size=shift;
	my $size_in_bytes;
	if (!$size) { 
		return -1;
	}
	
	if($size=~ m/^(\d+)[kK][bB]$/ ) {
		$size_in_bytes=$1 * 1024;
	} elsif($size=~ m/^(\d+)[Mm][bB]$/ ) {
		$size_in_bytes=$1 * 1024 * 1024;
	} elsif($size=~ m/^(\d+)[Gg][bB]$/ ) {
		$size_in_bytes=$1 * 1024 * 1024 * 1024;
	} elsif($size=~ m/^(\d+)[Tt][bB]$/ ) {
		$size_in_bytes=$1 * 1024 * 1024 * 1024 * 1024;
	} elsif($size=~ m/^(\d+)$/ ) {
		$size_in_bytes=$1;
	}
	return $size_in_bytes;
}

sub create_files_in_volume() {

}


### Get the list of clients from various sources
sub collect_clients_from_file() {
	my $filename=shift;
	if(-e $filename) {
        	open(FILE,$filename);
        	my @clients=<FILE>;
        	chomp @clients;
        	return @clients;
	} else {
		return "";
	}
}


###################################################################################
# Run client check on all the clients passed and pick only the ones that passed to
# run Kitchen sink
###################################################################################
sub healthy_clients() {
	my ($u_or_w,@clients);
	$u_or_w=shift;
	@clients=@_;
	### For the time being we consider only Linux clients
	my @healthy_clients=undef;
	$Log->log("****List of clients passed are <@clients> ****");
	
	### Step 1. Filter out non-Linux clients
	### Step 2. Make sure the clients are alive
	### Step 3. Run staf on all of them
	### Step 4. Return the list
	
        unless (@clients) {
        	$Trace->fatal("No clients passed");
    	}

        my $orig_env_client = $ENV{'CLIENT'};
        my $orig_env_client_check_CLIENT = $ENV{'client_check_CLIENT'};
        $ENV{'client_check_SKIP_EXECUTION'} = 0;
        my @subtests;
        foreach my $client (@clients) {
        	$Trace->comment(msg=>"client : $client\n");
        	my $subtest;
        	$ENV{'CLIENT'} = $client;
        	$ENV{'client_check_CLIENT'} = $client;
        	eval {
            		$subtest = subtest(
                               "tools/client_check.thpl",
                               "CLIENT_CONN=#ssh-persist",
                               "CHECKS=ping,staf,rshd,space,mem,subtest",
                               "CLIENTFILE="."",
                               "CLIENT=".$client,
                               -runid => "client_check",
                               -onexit => sub {},
                               '-bg',
                               );
        	};
        	if ($@) {
            		$Trace->fatal(msg=>"Encountered unexpected error while trying ".
               	           "to subtest client_check.thpl");
        	}
        	push(@subtests,$subtest);
    	}
    Subtest::wait_finish(subtests => \@subtests);
    $ENV{'CLIENT'} = $orig_env_client;
    $ENV{'client_check_CLIENT'} = $orig_env_client_check_CLIENT;
    $ENV{'client_check_SKIP_EXECUTION'} = 1;
    my (%failclients,%Passclients);
    for (my $ii=0; $ii<scalar(@subtests); $ii++) {
        my $st = $subtests[$ii];
        my @result = $st->results();
        if ($result[0]->type() =~ /PASS/i) {
            $Passclients{$clients[$ii]} = $result[0]->{'messages'}[0];
        } else {
            $failclients{$clients[$ii]} = $result[0]->{'messages'}[0];
        }
    }
    $Trace->comment(msg=>"Failed clients #: ".Dumper(\%failclients));
    $Trace->comment(msg=>"Passed clients #: ".Dumper(\%Passclients));
    $Trace->comment(msg=>scalar(keys %Passclients)." of ".scalar(@clients).
                    " Passed client check",result=>"INFO");
                    
                    
    my (@winclients,@unixclients);
    foreach my $client (keys %Passclients) {
        my $h = host($client);
        my $hosttype = $h->hosttype();
        if ($hosttype =~ /unix/i) {
            push(@unixclients,$client);
        } elsif ($hosttype =~ /windows/i) {
            push(@winclients,$client);
        }
    }                    
    if($u_or_w =~ /unix/i) {
    	return (sort @unixclients);                    
    } else {
    	return (sort @winclients);
    }
}

sub Get_All_Volumes() {
	my %wfit=@_;
	my (@total_vols,@aggrs,@volids,@vols,$aggr_name);
	
	if(%wfit) {
		@aggrs=keys %wfit;
		for(my $i=0;$i<@aggrs;$i++) {
			$aggr_name=$aggrs[$i];
			@vols=keys %{$wfit{$aggr_name}};
			push @total_vols,@vols;
		}
	}
	return @total_vols;
}


1;
