#
## @summary Utilility class for Log
## @status   Public
## @pod      here
## @author   dl-dpgqa-automation@netapp.com

=head1 NAME

WaflMCC::MCC::Log::Util

=head1 DESCRIPTION

This is the WaflMCC::MCC::Log::Util class.

It contains various helper methods, many of which are for log analysis.

=head1 METHODS

=cut

package WaflMCC::MCC::Log::Util;

use Time::HiRes qw(gettimeofday tv_interval);

use strict;
use warnings;
no warnings qw/ redefine /;

#TODO
#be able to generate reports like which thpls use a method
###############################################################################
#
#   Sub Name:      new
#
#   Description:   creates new WaflMCC::MCC::Log::Util object
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $class    IN      scalar    this class name
#                   %arg      IN      hash      hash containing initial values,
#                                               namely logger (WaflMCC::MCC::Log ref)
#   Globals:
#       $WaflMCC::MCC::Log::TRACE     IN      scalar    trace log type
#
#   Environment:
#
#   Returns:        Success:  object of this class
#                   Failure:
#
###############################################################################

sub new {
    my ($class, %arg) = @_;

    #my $start = [gettimeofday()];
    generate_filter($arg{filter});

    my $self = {
        #%arg, #for now, assume logger is set, or we'll fail later...
        #start    => $start,
        #incall   => $incall,     # get sub we are in
        #calledby => $calledby ,  # who called that sub?
        central => $arg{central} || $WaflMCC::MCC::Log::CENTRAL_LOG_DIR,
        filter  => $arg{filter},
        axis    => $arg{axis},
        count   => {},             # used to process/sum data
        report  => $arg{report},   #arrayref of field names
    };
    bless $self,$class;

    return $self;

}

###############################################################################
#
#   Sub Name:      process_data
#
#   Description:   generates a report based on data in a repository
#
#                  It supports N dimensions of data that are passed as
#                  part of the query (the field names).  It also supports
#                  specifying that a particular fields should consist of
#                  various manipulations of that data:
#
#                  unique - group all unique values together
#                  sum - add up values for result
#           ** this should be in result  unique count - number of unique values
#
#                  filters:
#                  a particular field (whether it is one of the axes or not
#                  can have a filter (regex) to determine if that particular
#                  line should be included in the report or not.  This
#                  is most often used to ignore methods we are not
#                  interested in but can also be used for anything else.
#
#
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     IN      ref       Timer Object
#                   $line     IN      scalar    raw log line
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success:  hashref or array of log line
#                   Failure:
#
###############################################################################
sub process_data {
    my $self = shift;
    my $line = shift;
    my $axis = shift || $self->{axis};
    my $count = shift || $self->{count};

    return unless defined $line;
    #use Data::Dumper; print 'AXIS:' . Dumper $axis;
    print ".";

    foreach my $name (keys %$axis) {


        my $field = $axis->{$name}{field} || $name; #usually no field defined
        my $process = $axis->{$name}{process} || 'unique'; #unique most common
        my $value = $line->{$field};
        my $return;
        #print ">>>>>>>>\n";
        #print "N:$name F:$field P:$process V: $value\n";


        my $subcount;

        if ($process eq 'unique') {
            #print "unique:$value\n";
            #$subcount = {$count->{$name}{$value}};
            #if (!defined $count->{$name}) {
                #$count->{$name} = {};
                #}
            if (!defined $count->{$name}{$value}) {
                $count->{$name}{$value} = {};
            }

            $subcount = $count->{$name}{$value} ;
            #print "writes to count?\n";
            #print Dumper $subcount;
            #print Dumper $count;
        } elsif ($process eq 'sum') {
            #$count->{$name}{_sum} += $value;
            #$subcount = {$count->{$name}};
            if (!defined $count->{$name}) {
                $count->{$name} = {};
            }
            $subcount = $count->{$name};
            $count->{$name}{_sum} += $value;
            #print "sum:$count->{$name}{_sum} [$value]\n";
        } else { #what?
            print "Unknown process $process\n";

        }

        #print "*COUNT:" . Dumper $count;
        #print "***FULL COUNT:" . Dumper $self->{count};

        if (defined $axis->{$name}{child}) {
            $self->process_data($line,$axis->{$name}{child},$subcount)
        } else {
        }


        #print "<<<<<<<<\n";

    }


}

###############################################################################
#
#   Sub Name:      generate_filter
#
#   Description:   generates regex filter func
#
#
#                  NOT method!
#
#
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success:  nothing
#                   Failure:
#
###############################################################################
sub generate_filter {
    my $filter = shift;

use Data::Dumper;
print Dumper $filter;

    foreach my $key (keys %{$filter}) {
        if (ref $filter->{$key}) {
        }
        else { #passed info the lazy way...
            my $regex = $filter->{$key};
            delete $filter->{$key};

            if ($regex =~ s/^!//) { #remove leading !
                #assume ^! in string ==  !~ instead of =~
                #a "not" regex

                $filter->{$key}{notregex} = $regex;
            }
            else { #normal regex
                $filter->{$key}->{regex} = $regex;

            }


        }

    }
use Data::Dumper;
print Dumper $filter;



}

###############################################################################
#
#   Sub Name:      process_day
#
#   Description:   process day
#
#
#
#
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     IN      ref       Timer Object
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success:  nothing
#                   Failure:
#
###############################################################################
sub process_day {
    my ($self,%args) = @_;

    my $y = $args{y};
    my $m = $args{m};
    my $d = $args{d};

    return unless ($y && $m && $d);

    my $dir      = "$self->{central}/$y/$m/$d";
    my $logpaths = "$dir/logpaths.log";

    print "opening $logpaths...\n";

    open LOGPATHS, q{<}, $logpaths || warn "cannot open $logpaths";

    foreach my $line (<LOGPATHS>) {
        my ($date,$type,$logdir,$csvlog) = parse_central_log_line($line);
        print "open $dir/$csvlog.csv\n";
        $self->process_file("$dir/$csvlog.csv");
    }

}

###############################################################################
#
#   Sub Name:      process_file
#
#   Description:   process file
#
#
#
#
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     IN      ref       Timer Object
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success:  nothing
#                   Failure:
#
###############################################################################
sub dump_count {
    my $self = shift;
    use Data::Dumper;
    print Dumper $self->{count};
}

###############################################################################
#
#   Sub Name:      process_file
#
#   Description:   process file
#
#
#
#
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     IN      ref       Timer Object
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success:  nothing
#                   Failure:
#
###############################################################################
sub process_file {
    my $self = shift;
    my $file = shift;
    #print "process file:$file\n";
    open CSV, q{<}, $file || warn "can't open csv file $file";

    foreach my $csvline (<CSV>) {
        next if ($csvline =~ /^log name /); #only for 1st, move out?
        $self->process_data($self->filter($csvline));
    }

    close CSV;

}

###############################################################################
#
#   Sub Name:      report
#
#   Description:   generate report
#
#
#
#
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     IN      ref       Timer Object
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success:  nothing
#                   Failure:
#
###############################################################################
sub report {
    my ($self, $args) = @_;
print "REPORT\n";
print Dumper $self->{report};
    foreach my $key (@{$self->{report}}) {
        print "K:$key\n";
        my $path = $self->find_report_key(key => $key);

        print "path:" ,Dumper $path if defined $path;

        my $hash = '$count->';
        my $x = 0;
        foreach my $segment (@$path) {
            $hash .= "{$segment}{\$_[" . $x++ . "]}";
        }
        print "H:$hash\n";
        #TODO construct (closure?? or loop) for each of these to iteratively
        #grab the first element and walk the tree.  However, we also
        #need to keep track of the values so far used in this loop and pass
        #along to future calls (in this line).  ie, if we are on a particular
        #value for 'method', we should stay with that same value the next
        #value down...
        #
        foreach my $segment (@$path) {


        }



        #TODO construct an eval (or something) to reference the full
        #path from the links in the path.  : [a,b,c] --> count{a}->{b}{c}

        #    foreach my $segment (@$path) {
            #print Dumper $self->{count}{$segment};

            #}
    }

    print "############################\n";
        $self->expand_report_node();

    foreach my $item (keys %{$self->{count}}) {

        print "C:$item\n";
        #$self->expand_report_node($item);

    }
    print "\n";

}

###############################################################################
#
#   Sub Name:      expand_report_node
#
#   Description:
#
#
#
#
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     IN      ref       Timer Object
#                  $args{key} IN      ref       required key to search for
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success:  arrayref of successive hashref keys (path)
#                   Failure:  undef
#
###############################################################################
sub expand_report_node {
    my $self = shift;
    my $node = shift || $self->{count};
    my $vals = shift || {};

use Data::Dumper;
    foreach my $key (sort keys %{$node}) {
        #print "K:$key" , Dumper  $node->{$key};
        print "K:$key\n";
        #next if $key eq '_sum'; # this is a val, not hashref
        if (ref $node->{$key} eq 'HASH') { #hash contains children
            #print Dumper [sort keys %{$node->{$key}}];
            foreach my $child (sort keys %{$node->{$key}}) {
                print "Child: $child [$key]\n";
                if (ref $node->{$key}->{$child} eq 'HASH') {
                    #push @{$vals}, "$key:$child";
                    $vals->{$key} = $child;
                    $self->expand_report_node($node->{$key}->{$child},$vals);
                } else {
                    print "V: $key:$child ($node->{$key}->{$child})\n";
                    #push @{$vals}, "$key:$node->{$key}->{$child}";
                    $vals->{$key} = $node->{$key}->{$child};
            print Dumper $vals;

                }
            }

        } else {
            print "V:...  $key: $node->{$key}\n";
            print Dumper $vals;
        }
        print "<=";
        #pop @{$vals};

        #TODO
        #get expander to work.  Maybe a 2 step reporter, 'rotate' data,
        #then dump as csv.  or maybe we can do that on the fly if we
        #search first to grab all the headers in order to order things
        #correctly for each line.  We also need to support empty columns
        #for a report template of some type
    }








}


###############################################################################
#
#   Sub Name:      find_report_key
#
#   Description:   recursively searches $self->{axis} for $args{key}
#
#
#
#
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     IN      ref       Timer Object
#                  $args{key} IN      ref       required key to search for
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success:  arrayref of successive hashref keys (path)
#                   Failure:  undef
#
###############################################################################
sub find_report_key {
    my ($self, %args) = @_;
    my $axis = $args{axis} || $self->{axis};
    my $path = $args{path} || [];
    my $found = $args{found};
    #my $count = shift || $self->{count};
    #
    my $key = $args{key};
    my $subkey;

    #also search for subkey if passed ( {key => subkey} )
    if (ref $key eq 'HASH') {
        $subkey = (values %$key)[0]; #only 1
        $key = (keys %$key)[0];
        #print "hashref ($key,$subkey)\n";
    }

    #print "PATH:" . Dumper $path;
   foreach my $name (keys %$axis) {

       if (($name eq $key) && ($subkey ? $axis->{$name}{child}{$subkey} : 1)) {
           #if ($name eq $key) {
           print "found $key ($subkey)\n";
           push  @{$path}, $name;


           $found = $path;
           #print "FOUND:$found\n";
       }
       else {
           push  @{$path}, $name;


            if (!$found && defined $axis->{$name}{child}) {
                $found = $self->find_report_key(
                       key=>$args{key},
                       axis=>$axis->{$name}{child},
                       path=>$path,
                       found=>$found,
                   );

            } else { # no children
            }
        }
        if ($found) {
            #print "FOUND:$found ($name)#\n";
            #print Dumper $found;
            last;
        }
    }
    if (!$found) { #pop off nodes as we backtrack
        #print "backtracking....\n";
        my $pop = pop @{$path};
        #print ("pop:$pop\n");
    } else {

    }

    return $found;


}


###############################################################################
#
#   Sub Name:      filter
#
#   Description:   filters line values
#
#
#
#
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     IN      ref       Timer Object
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success:  nothing
#                   Failure:
#
###############################################################################
sub filter {
    my $self = shift;
    my $text = shift;

    my $line = parse_csv_log_line($text);
    my $filter = $self->{filter};
    #use Data::Dumper;
    #print Dumper $filter;
    #print Dumper $line;
    foreach my $item (keys %$filter) {
        #print "check:$item\n";
        if (defined $filter->{$item}{'regex'} &&
            !($line->{$item} =~ /$filter->{$item}{regex}/)) {
                print "$line->{$item} does not match $filter->{$item}{regex}\n";
                return undef;
        }

        if (defined $filter->{$item}{notregex} &&
            !($line->{$item} !~ /$filter->{$item}{notregex}/)) {
                print "$line->{$item} does not match NOT $filter->{$item}{notregex}\n";
                return undef;
        }
    }

    return $line;

}

###############################################################################
#
#   Sub Name:      parse_central_log_line
#
#   Description:   parse a line from central log file (logs listing logdir
#                  from test run and csv log name)
#
#
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success:  nothing
#                   Failure:
#
###############################################################################

sub parse_central_log_line {
    #my $self = shift;
    my $line = shift;

    my ($date,$type,$logdir,$csvlog) =
       ($line =~ /^\[([^\]]+)\].*\s(\w*)\s\[logdir=([^\]]*)\] \[csvlog=([^\]]*)/);


    if (wantarray) {
        return ($date,$type,$logdir,$csvlog);
    }
    else {
        return ( {
            date => $date,
            type => $type,
            logdir => $logdir,
            csvlog => $csvlog
            }
        );
    }

}
###############################################################################
#
#   Sub Name:      parse_csv_log_line
#
#   Description:   parses internal csv log format and returns named hashref/array
#
#
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     IN      ref       Timer Object
#                   $line     IN      scalar    raw log line
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success:  hashref or array of log line
#                   Failure:
#
###############################################################################

sub parse_csv_log_line {
    my $line = shift;

    my $guess_type = 1;

    #depending on the file, $thpl could also just be the grep value
    my ($thpl,$package,$method,$count,$time,$logdate,$logtime,$release,$user,
        $type,$comment,$uuid,@lost) = split ',',$line;

    my ($basethpl) = $thpl =~ /([^\/]+)$/;
    #type is one of (smokes, nightly, regression, script_pool, sme, san,
    #                other, unknown)
    #print "T:$type\n" if $type;
    if ($guess_type && !$type) {

        #if ($user eq 'smoke') {
        if ($thpl =~ /u\/smoke\//) {
            $type = 'smokes';
        }

        else {
            $type = 'unknown';

        }
    }


    if (wantarray) {
        return ($thpl,$package,$method,$count,$time,$logdate,$logtime,
        $release,$user, $type,$comment,$uuid); #@lost);
    }
    else {
        return ( {
            thpl      => $thpl,
            basethpl  => $basethpl,
            'package' => $package,
            method    => $method,
            count     => $count,
            'time'    => $time,
            logdate   => $logdate,
            logtime   => $logtime,
            release   => $release,
            user      => $user,
            type      => $type,
            comment   => $comment,
            uuid      => $uuid,
            _lost     => \@lost,
            }
        );
    }

}

###############################################################################
#
#   Sub Name:      DESTROY
#
#   Description:   We use the destructor as a way to end the timer.  This
#                  depends on the timer being assigned to the appropriate
#                  variable type (ie 'my' usually)
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   $self     IN      ref       Timer Object
#
#   Globals:
#       $WaflMCC::MCC::Log::TRACE     IN      scalar    trace log type
#       $WaflMCC::MCC::Log::TIMER     IN      scalar    timer log type
#
#   Environment:   valid && log_init'd WaflMCC::MCC::Log object
#
#   Returns:        Success:  nothing
#                   Failure:
#
###############################################################################

sub DESTROY {
    my $self = shift;
    #my $time = tv_interval ($self->{start});
    #
    ## log TIMER line, improve later to show sys, user time
    #$self->{logger}->log(sprintf('[call=%s] [seconds=%f]',
    #$self->{incall}[3],$time),
    #$WaflMCC::MCC::Log::TIMER);
    #$self->{logger}->increment_call_timer($self->{incall}[3],
    #$self->{incall}[0],$time);
    #
    ## log exit line (trace)
    #$self->{logger}->log("[exiting=$self->{incall}[3]]" .
    #(defined $self->{message} ? " [message=$self->{message}]" : q{}),
    #$WaflMCC::MCC::Log::TRACE
    #);

    return;
}

1;
