# $Id: //depot/prod/test/main/lib/NACL/MTask/MCC/Network/Armadillo.pm#1 $
# -*-perl-*-

##no critic (RequireArgUnpacking)
##no critic (ControlStructures::ProhibitUnlessBlocks)
##no critic (ValuesAndExpressions::ProhibitInterpolationOfLiterals)

package WaflMCC::MCC::Network::Armadillo;

use strict;
use warnings;
use Carp;
use base qw (NACL::MTask::MTask);
use Data::Dumper;
use NACL::APISet;
use NACL::APISet::Switch;
use Tharn;
use WaflMCC::MCC::Log;;

use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR);

use List::MoreUtils qw{ any };

=head1 NAME

WaflMCC::MCC::Network::Armadillo;


=head1 DESCRIPTION

This Class is to be used for Armadillo testing

    # Exmaple:
  use WaflMCC::MCC::Network::Armadillo;

=cut

=head1 METHODS

=head2 new(...)

  my $arm = WaflMCC::MCC::Network::Armadillo->new(armadillo_name=>$ATTO);

Creates a new WaflMCC::MCC::Network::Armadillo object.
With this object you can call other method of this library.

=over

=item Argument

 Required: <armadillo_name> string

=item Return Value

WaflMCC::MCC::Network::Armadillo object

=back

 use WaflMCC::MCC::Network::Armadillo;
 my $arm = WaflMCC::MCC::Network::Armadillo->new(armadillo_name=>$ATTO);
 my $armadillo_info = $arm->get_info();

=cut


use Class::MethodMaker [
    new    => [ '-init', 'new' ],
    scalar => [
                  { -type => 'NACL::APISet::Switch'},
                  '_armadillo_apiset',
              ],
    scalar => [
	qw/  _armadillo_name
             /,
              ],
		       ];


sub init {
  my $trace = WaflMCC::MCC::Log::TRACELOG->start_timer(); #metrics

    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            armadillo_name            => { regex => qr/^[\w\-]+$/ },
                  },
                            );

    my $armadillo_name = delete($opts{armadillo_name});

    my $Host = host($self->_armadillo_name($armadillo_name));
    $self->_armadillo_apiset(NACL::APISet->new(
            hostobj   => $Host,
            category  => 'Switch',
            interface => 'CLI',
            set       => 'Armadillo',
            cdef_path => '/usr/software/test/agnostic/lib/' .
                         'NACL/APISet/Switch/CLI/Armadillo/default'));

    return;
}

=head2 get_armadillo_APISet

  This method return the NACL apiset for armadillo
  This can be used to call API set method directly

=over

=item Arguments

 none

=item Return Value

Apiset obj

=back

    my $atto_apiset_obj = $arm->get_armadillo_APISet;

=cut

sub get_armadillo_APISet {
  my $self = shift;
  return $self->_armadillo_apiset;
}

=head2 get_armadillo_name

  This method returns the name of the armadillo

=over

=item Arguments

 none

=item Return Value

string

=back

    my $armadillo_name = $arm->get_armadillo_name;

=cut

sub get_armadillo_name {
  my $self = shift;
  return $self->_armadillo_name;
}

=head2 get_info

  This method runs the get info command on the armadillo and returns
  the information inside a hash.

=over

=item Arguments

 none

=item Return Value

hash ref

=back

    my $armadillo_info = $arm->get_info;

=cut

sub get_info {
  my $self = shift;
  return ($self->_armadillo_apiset->info)->get_parsed_output();
}

1;
