# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/ontap/main/test/wafl/wafl_mcc_utils.pm
# 
# Copyright (c) 2003 Network Appliance, Inc.
# All rights reserved.
#

package WaflMCC::wafl_mcc_utils;

use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK);
use Tharn;

### NACL Component libs ##
use NACL::C::Node;
use NACL::C::StorageDisk;
use NACL::C::SystemNode;
use NACL::CS::StorageAggregate;

### NACL Task layer libs ##
use NACL::STask::Aggregate;
use NACL::STask::Vserver;
use NACL::STask::Volume;
use NACL::STask::StorageFailover;
use WaflMCC::MCC::Metrocluster;
use NACL::MTask::EventLogDetector;
use NACL::MTask::MCC::Utils;

### NACL Other libs ##
use NACL::Exceptions::EventCheckFailure;
use NACL::GeneralUtils qw(random_name_generator);

use TCD;
use Carp;
use Tharn;
use Data::Compare;
use HAutils;
use HAiosetup;

require Exporter;
@ISA       = qw(Exporter);
@EXPORT    = ();
@EXPORT_OK = qw(do_wafl_validation_get_vol_states
                do_wafl_validation_get_aggr_states
                run_traffic
                stop_traffic
                panic_node
                powercycle_node
                bringup_nodes
                check_vserver);
              
################# do_wafl_validation_get_vol_states ################################
sub do_wafl_validation_get_vol_states {
    my %opts     = @_;
    my $node     = $opts{node};
    my @vol_list = @{$opts{vols}};
    my $vol_status;

    # check vol states of all aggrs in param.
    foreach my $vol (@vol_list) { 
        my $vol_obj = NACL::C::Volume->find(
            command_interface => $node,
            filter            => {'volume' => $vol},
        );

        my @requested_fields = (qw(state type));
        my $cs = $vol_obj->state(requested_fields => \@requested_fields);
        foreach my $field (@requested_fields) {
            my $val = $cs->$field();
            if (defined $val) {
                #$Test->nlog("Value of field '$field' is " . NACL::ComponentUtils::Dumper($val));
                $vol_status->{$vol}->{$field} = $val;
            } else {
                NATE::BaseException->throw("Value of field '$field' was not "
                . 'defined in the CS object');
            }
        }
    }

    return $vol_status;
}

################# do_wafl_validation_get_aggr_states  ################################
sub do_wafl_validation_get_aggr_states {
    my %opts      = @_;
    my $node      = $opts{node};
    my @aggr_list = @{$opts{aggrs}};
    my $aggr_status;
    
    # check aggr states of all aggrs in param.
    foreach my $aggr (@aggr_list) { 
        my $aggr_obj = NACL::C::Aggregate->find(
            command_interface => $node,
            filter            => {'aggregate' => $aggr},
        );

        my @requested_fields = (qw(state));
        my $cs = $aggr_obj->state(requested_fields => \@requested_fields);
        foreach my $field (@requested_fields) {
            my $val = $cs->$field();
            if (defined $val) {
                #$Test->nlog("Value of field '$field' is " . NACL::ComponentUtils::Dumper($val));
                $aggr_status->{$aggr}->{$field} = $val;
            } else {
                NATE::BaseException->throw("Value of field '$field' was not "
                . 'defined in the CS object');
            }
        }
    }

    return $aggr_status;
}


################# do_wafl_validation_check_ems_status ################################
sub do_wafl_validation_check_ems_status {
}

################# do_wafl_validation_verify_mailbox_status ################################
sub do_wafl_validation_verify_mailbox_status {
}

################# do_wafl_validation_voldb_verify_status ################################
sub do_wafl_validation_verify_voldb_status {
}

################# do_wafl_validation_drsom_verify_status ################################
sub do_wafl_validation_verify_drsom_status {
}

################# run_traffic  ################################
sub run_traffic {
    my %opts = @_;

    my $node_ci = $opts{node_ci};
    my $client_ci = $opts{client_ci};
    my $volname = $opts{volname};
    my $vs = $opts{vserver};
    my $lifdetails = $opts{lifdetails};    
    my $runtime = $opts{runtime};
    
    $Log->enter();

    my $traffic = NACL::MTask::NASTraffic->start(
        'node_info' => { 
            command_interface => $node_ci,
            volume            => $volname,
            server            => $lifdetails->{ADDRESS}, 
            vserver           => $vs,
            'home-port'       => $lifdetails->{PORT}, 
            'home-node'       => $node_ci->node(),
            lif               => $lifdetails->{LIFNAME}, 
            'junction-path'   => "/$volname",
            netmask           => $lifdetails->{NETMASK}, 
            gateway           => $lifdetails->{GATEWAY}, 
        },
        'host_info' => [
            {
                command_interface => $client_ci,
                tool              => 'hammer',
                tool_options    => {
                    'nofilercore' => 1,
                }
            }
        ],
        nacltask_traffic_type => 'nfs',
        nacltask_traffic_direction => 'read',
    );
 
    NACL::C::Statistics->show_periodic(
        command_interface => $node_ci,
        iterations => 5,
        interval => 1,
        'summary' => 'true',
    );
    
    $Log->exit();    
    return $traffic;
}    


################# run_traffic  ################################
sub stop_traffic {
    my %opts = @_;

    my $traffic = $opts{traffic};
    my $volname = $opts{volname};
    
    $Log->enter();
    $traffic->stop();
    $Log->exit();    
}    

################# powercycle_node  ################################
sub powercycle_node {
    my %opts = @_;
    my $node = $opts{node};

    $Log->comment("Powercycle node : $node");        
    my $mccHost = host($node);
    my $mcc_Api_Set_Obj = NACL::APISet->new(
                hostobj   => $mccHost,
                category  => "Node",
                interface => "CLI",
                connid    => "sp_login",
                set       => "SP",
            );
    try {
        my @prompts_answers =
            ( '.*This will cause a dirty shutdown of your appliance.*' => 'y', );
        my $response_create = $mcc_Api_Set_Obj->system_power( cycle => 1,
            'connectrec-match_table' => \@prompts_answers, );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $Log->comment( "Error is: " . $exception_object->text() );
        my $resp = $exception_object->get_response_object();
        $Log->comment( "Raw output was:\n" . $resp->get_raw_output() );
    };

    # we need to create a transit object here and pass it back to Cluster.pm 
    my $trans_obj = NACL::Transit->new( name => $node );
    $trans_obj->change_state(to => 'FIRMWARE', maxcycle => "20",timeout =>2000);
    return $trans_obj;

}

################# panic_node  ################################
sub panic_node {
    my %opts = @_;
    my @peer_cluster_mcc_nodes = @{$opts{nodes}};

    $Log->comment('Panic the nodes at disaster site simultaneously...');

    my @subtest_handles = ();
    my @halted_node_objects;
    my $trans_obj;
    
    foreach my $peer_mcc_node (@peer_cluster_mcc_nodes) {
        my $node_name = $peer_mcc_node;
        my $halt_cmd  = 'set -confirmations off;'.
                            'system node halt -dump true -skip-lif-migration true -reason '.
                            'prepare_for_switchover_execution -node '.
                            $node_name;
        $trans_obj = NACL::Transit->new( name => $node_name );
        push @subtest_handles, subtest(sub {
                $trans_obj->change_state( to      => 'FIRMWARE',
                                          haltcmd => $halt_cmd,
                                          timeout => 2000 );
                },
                '-bg',
                '-runid',
                $node_name . '_panic_this_node');
        push(@halted_node_objects, $trans_obj);
    }
    Subtest::wait_finish(subtests => \@subtest_handles);

    # we need to create and return a transit object here and pass it back to Cluster.pm 
    return \@halted_node_objects;
}

################# bringup_nodes  ################################
sub bringup_nodes {
    my %opts = @_;
    my @peer_cluster_mcc_nodes = @{$opts{nodes}};
    my $endstate = $opts{endstate};
    #TAKEN_OVER_WAITING_ONTAP
    $Log->comment('Boot the nodes at disaster site simultaneously...');

    my @subtest_handles = ();
    my @halted_node_objects;
    my $trans_obj;
    
    foreach my $peer_mcc_node (@peer_cluster_mcc_nodes) {
        my $node_name = $peer_mcc_node;
        $trans_obj = NACL::Transit->new( name => $node_name );
        push @subtest_handles, subtest(sub {
                $trans_obj->change_state( to      => $endstate,
                                          timeout => 2000 );
                },
                '-bg',
                '-runid',
                $node_name . '_panic_this_node');
        push(@halted_node_objects, $trans_obj);
    }
    Subtest::wait_finish(subtests => \@subtest_handles);

    # we need to create and return a transit object here and pass it back to Cluster.pm 
    return \@halted_node_objects;
}

################# Find existing vserver ################################
sub check_vserver {
    my %opts = @_;
    my $vserver_obj = NACL::STask::Vserver->find(
                              command_interface => $opts{node},
                                      filter    => {
                                                      type  => 'cluster',
                                                      vserver =>  $opts{vserver},
                                                    },
                                    allow_empty => 1, );
    if (!$vserver_obj) {
            croak("Vserver $opts{vserver} not found on $opts{node}->name " .
                  'please check test-bed config');
    }
    return $vserver_obj;
}



   
1;
