#Hey emacs, this is basically -*- perl -*-
# $Id: $

## @summary     WAFL Common library to control test execution
##              through TestManagerLite.
##
## @author      gvg@netapp.com
##
## @pod here

=head1 NAME

WaflTestManager - Test execution controller for WAFL tests
(subclass of TestManagerLite)

=head1 SYNOPSIS

    package Test;
    use base qw(WaflTestManager);

    sub test_main {
        my $self = shift;
        my ($failure) = $self->get(parameter=>"INDUCE_FAILURE");
        # some test code here to implement the test case
        die "This test failed\n" if ($failure);
    }

    sub test_setup {
	# Do the test specific setup here
    }

    sub test_cleanup {
	# Do test specific cleanup here
    }

    my $test = Test->new();
   
    $test->parameters(definitions=>
        [
         {
             name => "INDUCE_FAILURE",
             description => "force a failure in test_main",
             default => undef,
         }
        ]);
    $test->run_test();

=head1 DESCRIPTION

C<WaflTestManager> is a base class of tests and sub class of
C<TestManagerLite>. C<WaflTestManager> is a place holder
to control only WAFL tests without affecting other group'
tests. This layer will help for any WAFL specific
changes later.

=head1 SEE ALSO

TestManagerLite Options TracePkg Topology Topology::WaflDefault

Please see C<TestManagerLite> for specific details on
test execution control.

Please see C<Topology> for specific details on environment
setup / cleanup / topology

=head1 BUGS

Some probably exist but have not been listed yet.

=head1 STATUS

  Shared

=head1 AUTHOR

  GV Govindasamy (gvg@netapp.com)
  $Id: $
  Created $Date: 2006/04/12 $

=head1 COPYRIGHT

Copyright 2006, Network Appliance., All Rights Reserved

=cut

package WaflTestManager;

use base qw(TestManagerLite);

use Tharn;
use strict;

use Params::WAFL;
use Data::Dumper qw(Dumper);

use TracePkg;
my $Trace = new TracePkg;

use Options;
my $Options = new Options;
my $Options_hash;

sub new {
    my ($self) = @_;
    $self->SUPER::new;   
}

sub filer_setup {
    my ($self) = @_;
    $self->SUPER::filer_setup;
}
    
sub client_setup {
    my ($self) = @_;
    $self->SUPER::client_setup;
}

sub client_cleanup {
    my ($self) = @_;
    $self->SUPER::client_cleanup;
}

sub filer_cleanup {
    my ($self) = @_;
    $self->SUPER::filer_cleanup;
}

# The following three test functions are supposed to be 
# overridden by test scripts. If they don't these
# functions do nothing.

sub test_setup {
    $Trace->entry();
    $Trace->debug(msg=>"Processing test_setup in WaflTestManager");
    $Trace->exit();
}

sub test_main {
    $Trace->entry();
    $Trace->debug(msg=>"Processing test_main in WaflTestManager");
    $Trace->exit();
}

sub test_cleanup {
    $Trace->entry();
    $Trace->debug(msg=>"Processing test_cleanup in WaflTestManager");
    $Trace->exit();
}

sub run_test {
    my ($self) = @_;
    $self->SUPER::run_test();
}

1;
