#
# $Id: //depot/prod/test/Rsierranevada/lib/WaflironLibrary/Utils.pm#2 $
#
# Copyright (c) 2009-2010 NetApp Inc.
# All rights reserved.
#
## @summary Provide an helper class to the WaflironLibrary. This basically
## extends the WaflironLibrary Class with only helper functions consolidated
## in this class.
## None of the methods in this class are exposed to the users. All the methods
## in this class are consumed by the class WaflironLibrary.
##
## @author chethanm@netapp.com
## @status Public
## @pod Since it is a private library, POD is not added.

package WaflironLibrary::Utils;

package WaflironLibrary;

use WaflironLibrary::Exceptions::WaflironLibException;
use Admin::License::Default;
use NATE::BaseException qw(:try);
use NACL::Exceptions::NoElementsFound qw(:try);

use constant INOFILE_PROMPT => qr/Which inofile should be used for (\w+)\?/i;
use constant INOFILE_PROMPT_RA =>
    qr/Which inofile should be used for aggregate (\w+)\?/i;
use constant VOL_PROMPT =>
    qr/Which flexvols in aggregate \w+ should perform prev_cp/i;
use constant VOL_INOFILE_PROMPT1 =>
    qr/Which inofile should be used for (?:\w+:)*(\w+)\?/i;
use constant VOL_INOFILE_PROMPT2 =>
    qr/Which\sinofile\sshould\sbe\sused\sfor\s(\w+)\@vserver:/i;
use constant VOL_INOFILE_PROMPT3 =>
    qr/Which\sinofile\sshould\sbe\sused\sfor\svolume\s(\w+)\@vserver:/i;
use constant VOL_INOFILE_PROMPT4 =>
    qr/Which\sinofile\sshould\sbe\sused\sfor\svolume\s(\w+)\?/i;
use constant VOLINFO_VBN_PROMPT => qr/prev_volinfo_vbn\s*:\s*(\d+)/i;
use constant INVALID_SEL_PROMPT =>
    qr/invalid inofile selection|bad volinfo_vbn/i;
use constant VOL_PROMPT_RR =>
    qr/List\sof\svolumes\son\swhich\sWafliron\sin\sPrevious-Checkpoint\smode\scan\srun\sis:/i;
use constant RE_COMMITTED_FINISH =>
    'Committed change log files for (aggregate|volume)';
use constant RE_COMMIT_CHANGES =>
    qr/"Commit\schanges\sfor\s(aggregate|volume)\s.*\sto\sdisk\?"/i;
use constant RE_SNAPMIRRORED_VOLS =>
    qr/Committing these changes will break the synchronization between the volumes.*\. Do you still want to continue?/i;
use constant LOCAL_WAFLIRON_TIMEOUT => 2400;
use constant RE_UNMOUNT_AGGR_WITH_VOLS =>
    qr/unmount all of the volumes hosted by the aggregate\?/s;
use constant RE_TEST_MODE_CONFIRMATION =>
    qr/test commands are not supported.*/s;

##############################################################################
# prev_cp non-interactive helper function
##############################################################################

sub _do_prev_cp_non_interactive {
    my ( $pkg, %opts ) = @_;

    $pkg->_logmsg( "Processing the prev_cp non-interactive request!",
        'comment' );

    # get the context to execute
    my $context = $opts{'context'};
    my $timeout = $opts{'timeout'} || LOCAL_WAFLIRON_TIMEOUT;
    my $console_output;

    # Get the relevent apiset obj based on context
    my $api_set_obj = $pkg->_get_context_based_api($context);

    # phrase the command based on context
    my $cmd
        = $context =~ /7Mode|Nodescope/i
        ? "aggr_wafliron_start"
        : "storage_aggregate_wafliron_start";

    if (   ( $context !~ /CMode|7Mode|Nodescope/i )
        && ( $::IRON_ON_ROOT !~ /(core|multi|spare)/i ) )
    {
        NATE::Exceptions::Argument->throw(
                  "WaflironLibrary::wafliron_start - "
                . "Invalid value for context: $opts{'context'}!" );
    }

    my $args = $pkg->_get_wafliron_default_args(%opts);
    $Log->comment( "_get_wafliron_default_args returned : " . Dumper($args) );

    $args->{'connectrec-match_table'} = [
        eval INOFILE_PROMPT =>
            sub { return $opts{'prev_cp'}->{'aggr_resp'}->{$1} },
        eval INOFILE_PROMPT_RA =>
            sub { return $opts{'prev_cp'}->{'aggr_resp'}->{$1} }
        ]
        if defined( $opts{'prev_cp'}->{'aggr_resp'} );
    $Log->comment(
        "aggr response is : " . Dumper( $args->{'connectrec-match_table'} ) );
    $Log->comment( "Timeout is: " . $timeout );

    if ( $::IRON_ON_ROOT && defined $opts{'wafl_bm_command'} ) {
        my $cr                 = $pkg->conn_console();
        my $wafl_bm_command    = $opts{'wafl_bm_command'};
        my $additional_prompts = $opts{'additional_prompts'}
            || [ 'Login:', 'Password', 'Enabling rsh service' ];
        $wafl_bm_command .= " " . $args->{'prev_cp'};
        my %cmd_args
            = ( 'command' => $wafl_bm_command, 'timeout' => $timeout );
        $cmd_args{'match_table'} = $args->{'connectrec-match_table'}
            if ( defined $args->{'connectrec-match_table'} );
        $cmd_args{'additional_prompts'} = $additional_prompts
            if ( defined $additional_prompts );
        $console_output = $cr->execute(%cmd_args);
        return ($console_output);
    }
    else {
        $api_set_obj->$cmd(
            %{$args},
            'connectrec-timeout' => $timeout,
            'privilege-level'    => 'test'
        );
    }
}
##############################################################################
# prev_cp interactive for CMode
##############################################################################

sub _do_prev_cp_interactive {
    my ( $pkg, %opts ) = @_;

    $pkg->_logmsg( "Processing the prev_cp interactive request!", 'comment' );

    my %cp_hash;
    my $vol_list_answered = 0;
    my $console_output;
    my $timeout = $opts{'timeout'} || LOCAL_WAFLIRON_TIMEOUT;

    # Get the relevent apiset obj based on context
    my $context    = $opts{'context'};
    my $apiset_obj = $pkg->_get_context_based_api($context);

    # phrase the command based on context
    my $cmd
        = $context =~ /7Mode|Nodescope/i
        ? "aggr_wafliron_start"
        : "storage_aggregate_wafliron_start";

    # Validate the parameters.

    # For prev_cp in interactive mode, both aggr_resp and vol_resp must be
    # provided for RR. But for BR, only vol_resp is mandatory
    my $release = $pkg->ontap_release();

    if (!(     defined $opts{'prev_cp'}->{'aggr_resp'}
            || defined $opts{'prev_cp'}->{'vol_resp'}
        )
        && $release eq "RollingRock"
        )
    {
        NATE::Exceptions::Argument->throw(
                  "WaflironLibrary::wafliron_start - "
                . "For wafliron to start in prev_cp interactive mode, "
                . "parameters: aggr_resp and vol_resp must be provided for "
                . "RollingRock and above" );
    }
    else {
        if ( !defined $opts{'prev_cp'}->{'vol_resp'}
            && ( $release =~ /Boilermaker|IronCity/ ) )
        {
            NATE::Exceptions::Argument->throw(
                      "WaflironLibrary::wafliron_start - "
                    . "For wafliron to start in prev_cp interactive mode, "
                    . "parameter: vol_resp is mandatory!" );
        }
    }

    # Build the wafliron params
    my $args = $pkg->_get_wafliron_default_args(%opts);

    my $prev_volinfo_vbn_dep = 0;

    # Get the prev_cp hash into %Input
    %Input = %{ $opts{'prev_cp'} };

    # list of vvols for interative mode
    my $list_of_vols = join( ',', keys %{ $Input{'vol_resp'} } );
    $list_of_vols =~ s/,\s+/,/g;

    my $answer_vol_prompt = sub {
        my $resp
            = $Input{'vol_with_snap_id_list'}
            ? $list_of_vols . ',' . $Input{'vol_with_snap_id_list'}
            : $list_of_vols;
        return $resp;
    };

    # list of vvol and inofile for interative mode
    my $vol_ni_resp_list
        = join( ',', map { $_ . ':' . $hash{$_}; } keys %hash );

    my $validate_and_answer = sub {

      # if the previous prompt was answered '#', this prompt is invalid. Hence
      # log an error and exit
        WaflironLibrary::Exceptions::WaflironLibException->throw(
                  "WaflironLibrary::wafliron_cluster_start - "
                . "Invalid vvol matching done!" )
            if $prev_volinfo_vbn_dep;

        # if prompted for aggregate inofile, answer the aggr inofile
        if ( defined $Input{'aggr_resp'}->{$1} ) {
            $cp_hash{$1} = $Input{'aggr_resp'}->{$1};
            return $Input{'aggr_resp'}->{$1};

        }

  # when #0 is the vvol option, it should be invalidated and better option
  # to use is 'r' instead
  # when '#' is the vvol option send '#' as answer and handle prev_volinfo_vbn
  # in the prev_volinfo_vbn prompt with #prev_volinfo_vbn

        if ( $Input{'vol_resp'}->{$1} eq "#0" ) {
            $Log->comment( "WaflironLibrary::wafliron_cluster_start - "
                    . "Invalid prev_vol_vbn no; using r instead" );
            $cp_hash{$1} = 'r';
            return 'r';
        }
        elsif ( $Input{'vol_resp'}->{$1} eq "#" ) {
            $prev_volinfo_vbn_dep = 1;
            $cp_hash{$1} = '#';
            return '#';
        }
        else {
            $cp_hash{$1} = $Input{'vol_resp'}->{$1};
            return $Input{'vol_resp'}->{$1};
        }
    };

    my $answer_prev_volinfo_vbn = sub {
        if ($prev_volinfo_vbn_dep) {
            $prev_volinfo_vbn_dep = 0;
            $cp_hash{$1} = "#" . $1;
            return "#" . $1;
        }
        else {
            WaflironLibrary::Exceptions::WaflironLibException->throw(
                "WaflironLibrary::wafliron_start - Invalid prompt found");
        }
    };
    my $answer_invalid_sel_prompt = sub {
        return 'r';
    };

    my $match_table = [
        eval INOFILE_PROMPT      => $validate_and_answer,
        eval VOL_PROMPT          => $answer_vol_prompt,
        eval VOL_PROMPT_RR       => $answer_vol_prompt,
        eval VOL_INOFILE_PROMPT1 => $validate_and_answer,
        eval VOL_INOFILE_PROMPT2 => $validate_and_answer,
        eval VOL_INOFILE_PROMPT3 => $validate_and_answer,
        eval VOL_INOFILE_PROMPT4 => $validate_and_answer,
        eval VOLINFO_VBN_PROMPT  => $answer_prev_volinfo_vbn,
        eval INVALID_SEL_PROMPT  => $answer_invalid_sel_prompt,
        eval INOFILE_PROMPT_RA   => $validate_and_answer
    ];

    if ( $::IRON_ON_ROOT && defined $opts{'wafl_bm_command'} ) {
        my $cr              = $pkg->conn_console();
        my $wafl_bm_command = $opts{'wafl_bm_command'};
        my $a_prompts       = $opts{'additional_prompts'}
            || [ 'Login:', 'Password', 'Enabling rsh service' ];
        $console_output = $cr->execute(
            'command'            => $wafl_bm_command,
            'timeout'            => $timeout,
            'match_table'        => $match_table,
            'additional_prompts' => $a_prompts,
            'max_idle'           => 20,
            'on_idle'            => sub { $cr->send("\r"); undef; },
        );
        $console_output .= $pkg->_reboot();
        return ( \%cp_hash, $console_output );
    }
    else {
        $apiset_obj->$cmd(
            %{$args},
            'connectrec-match_table' => $match_table,
            'connectrec-timeout'     => $timeout,
            'privilege-level'        => 'test'
        );
        return \%cp_hash;
    }
}

##############################################################################
# prev_cp semi-interactive for CMode
##############################################################################

sub _do_prev_cp_semi_interactive {
    my ( $pkg, %opts ) = @_;
    my %cp_hash;

    $pkg->_logmsg( "Processing the prev_cp semi-interactive request!",
        'comment' );

    # Get the relevent apiset obj based on context
    my $context    = $opts{'context'};
    my $apiset_obj = $pkg->_get_context_based_api($context);
    my $console_output;

    # phrase the command based on context
    my $cmd
        = $context =~ /7Mode|Nodescope/i
        ? "aggr_wafliron_start"
        : "storage_aggregate_wafliron_start";

    # Validate the parameters.

    # For prev_cp in semi-interactive mode, aggr_resp,vol_resp and
    # ic_vol_resp must be provided
    if (!(     defined $opts{'prev_cp'}->{'aggr_resp'}
            || defined $opts{'prev_cp'}->{'vol_resp'}
            || defined $opts{'prev_cp'}->{'ic_vol_resp'}
        )
        )
    {
        NATE::Exceptions::Argument->throw(
                  "WaflironLibrary::wafliron_start - "
                . "For wafliron to start in prev_cp interactive mode, "
                . "parameters: aggr_resp,vol_resp and  ic_vol_respmust be "
                . "provided" );
    }

    # Get the prev_cp hash into %Input
    %Input = %{ $opts{'prev_cp'} };

    my $no_of_vols = 0;
    my @incomplete_list = split( /\s*,\s*/, $Input{'ic_vol_resp'} );
    $no_of_vols = scalar @incomplete_list;

    # Populate the cp_hash with vol and resp provided in ic_vol_resp param.
    map {
        if (/\:/)
        {
            my ( $vol, $resp ) = split /\:/,;
            $cp_hash{$vol} = $resp;
        }
    } @incomplete_list;

    # update the volume count to include volumes prvided in vol_rep
    foreach my $resp ( keys %{ $Input{'vol_resp'} } ) {
        ++$no_of_vols;
    }
    $pkg->_logmsg(
        "Total no of vols to be ironed in semi interactive mode: $no_of_vols",
        'comment'
    );

    # Build the wafliron params
    my $args = $pkg->_get_wafliron_default_args(%opts);
    my $timeout = $opts{'timeout'} || LOCAL_WAFLIRON_TIMEOUT;

    # Get the prev_cp hash into %Input
    %Input = %{ $opts{'prev_cp'} };

    # list of vvols for interative mode
    my $list_of_vols = join( ',', keys %{ $Input{'vol_resp'} } );
    my $answer_vol_prompt = sub {
        my $resp
            = $list_of_vols
            ? $list_of_vols . ',' . $Input{'vol_with_snap_id_list'}
            : $Input{'vol_with_snap_id_list'};
        return $resp;
    };

    my $validate_and_answer = sub {

        # if prompted for aggregate inofile, answer the aggr inofile
        if ( defined $Input{'aggr_resp'}->{$1} ) {
            $cp_hash{$1} = $Input{'aggr_resp'}->{$1};
            return $Input{'aggr_resp'}->{$1};
        }

        # if there is response provided, return the same
        elsif ( defined $Input{'vol_resp'}->{$1} ) {
            $cp_hash{$1} = $Input{'vol_resp'}->{$1};
            return $Input{'vol_resp'}->{$1};
        }

        # if there is no response provided, 'r' is used as default
        else {
            $cp_hash{$1} = 'r';
            return 'r';
        }
    };

    my $answer_invalid_sel_prompt = sub {
        return 'r';
    };

    my $login = \{ $pkg->_wi_login() };

    my $match_table = [
        eval INOFILE_PROMPT      => $validate_and_answer,
        eval VOL_PROMPT          => $answer_vol_prompt,
        eval VOL_INOFILE_PROMPT1 => $validate_and_answer,
        eval VOL_PROMPT_RR       => $answer_vol_prompt,
        eval INVALID_SEL_PROMPT  => $answer_invalid_sel_prompt,
        eval VOL_INOFILE_PROMPT2 => $validate_and_answer,
    ];

    if ( $::IRON_ON_ROOT && defined $opts{'wafl_bm_command'} ) {
        my $cr              = $pkg->conn_console();
        my $wafl_bm_command = $opts{'wafl_bm_command'};
        my $a_prompts       = $opts{'additional_prompts'}
            || \[ 'Login:', 'Password', 'Enabling rsh service' ];
        $console_output = $cr->execute(
            command              => $wafl_bm_command,
            'timeout'            => $timeout,
            'match_table'        => $match_table,
            'additional_prompts' => $a_prompts
        );
        return ( \%cp_hash, $console_output );
    }
    else {
        $apiset_obj->$cmd(
            %{$args},
            'connectrec-match_table' => $match_table,
            'connectrec-timeout'     => $timeout,
            'privilege-level'        => 'test'
        );
        return \%cp_hash;
    }
}

sub _start_wafliron {
    my ( $pkg, %opts ) = @_;

    $pkg->_logmsg( "Running the command - wafliron start!", 'comment' );
    my $context    = $opts{'context'};
    my $apiset_obj = $pkg->_get_context_based_api($context);
    my $timeout    = $opts{'timeout'} || LOCAL_WAFLIRON_TIMEOUT;

    # phrase the command based on context
    my $cmd
        = $context =~ /7Mode|Nodescope/i
        ? "aggr_wafliron_start"
        : "storage_aggregate_wafliron_start";

    # get the wafliron args
    my $args = $pkg->_get_wafliron_default_args(%opts);

    $apiset_obj->$cmd(
        %{$args},
        'connectrec-timeout' => $timeout,
        'privilege-level'    => 'test'
    );
}

##############################################################################
# Purge the given aggregate using the proper mode of execution
##############################################################################

sub _aggr_purge {
    my ( $pkg, $aggr ) = @_;
    my $mode        = $pkg->mode();
    my $api_set_obj = $pkg->_get_context_based_api($mode);
    my ( $aggr_purge_cmd, $aggr_purge_opts );

    if ( $mode =~ /7Mode/ ) {
        $aggr_purge_cmd  = "aggr_destroy";
        %aggr_purge_opts = (
            'aggregate'       => $aggr,
            'force'           => 1,
            'privilege-level' => 'test'
        );
    }
    elsif ( $mode =~ /CMode/ ) {
        $aggr_purge_cmd  = "storage_aggregate_delete";
        %aggr_purge_opts = (
            'aggregate'       => $aggr,
            'force'           => "true",
            'privilege-level' => 'test'
        );
    }
    else {
        NATE::Exceptions::Argument->throw(
            "Invalid value for \'Mode\': " . $mode );
    }
    $pkg->_aggr_offline( $aggr, $mode );
    $api_set_obj->$aggr_purge_cmd(%aggr_purge_opts);
    $pkg->_logmsg( "$aggr purged succesfully!", 'comment' );

    return 1;
}

##############################################################################
# Change the aggregate state to offline
##############################################################################

sub _aggr_offline {
    my ( $pkg, $aggregate, $context ) = @_;
    my $mode = $pkg->mode();
    if ( $pkg->_get_container_type($context) ne "aggregate" ) {
        NATE::Exceptions::Argument->throw("$aggregate is not an aggregate!)");
    }
    my $timeout       = $pkg->cmd_timeout();
    my $state         = $pkg->_get_aggr_status($context);
    my $rlse_codename = $pkg->ontap_release_codename();
    my ( $cmd, %opts );

    # Check if the aggregate is offline, return 1 if already in offline state
    if ( $state eq 'offline' ) {
        $pkg->_logmsg( "Aggregate: $aggregate is already in offline state!",
            'comment' );
        return 1;
    }

    # If aggregate is not 'online', throw an exception
    else {
        if ( $state !~ /online/ && $state !~ /restricted/i ) {
            my $exception_msg
                = "Can't modify the aggregate state to offline as "
                . "aggregate $aggregate is in state: $state";
            WaflironLibrary::Exceptions::WaflironLibException->throw(
                $exception_msg);
        }
    }

    my $api_set_obj = $pkg->_get_context_based_api($context);

    if ( $context =~ /7Mode|Nodescope/i ) {
        $cmd  = "aggr_offline";
        %opts = (
            'aggregate'       => $aggregate,
            'aggr-with-vols'  => 1,
            'force'           => 1,
            'privilege-level' => 'test'
        );
    }
    elsif ( $context =~ /CMode/i ) {

        $cmd  = "storage_aggregate_modify";
        %opts = (
            'aggregate'              => $aggregate,
            'state'                  => 'offline',
            'privilege-level'        => 'test',
            'connectrec-match_table' => [
                eval RE_UNMOUNT_AGGR_WITH_VOLS => 'y',
                eval RE_TEST_MODE_CONFIRMATION => 'y'
            ],
        );
    }
    else {
        NATE::Exceptions::Argument->throw( "Invalid context: " . $context );
    }

    # handle prompt for aggr modify state to offline for CMode with vols

    if ( $cmd eq "storage_aggregate_modify" ) {
        $pkg->_logmsg(
            "Modifying the session attribute 'confirmation' to 'on' temporarily",
            'debug'
        );
        $api_set_obj->set( "confirmations" => 'on' );
    }

    $api_set_obj->$cmd(%opts);

    if ( $cmd eq "storage_aggregate_modify" ) {
        $pkg->_logmsg(
            "Modifying the session attribute 'confirmation' back to 'off' ",
            'debug' );
        $api_set_obj->set(
            "confirmations" => 'off',
            'connectrec-match_table' =>
                [ eval RE_TEST_MODE_CONFIRMATION => 'y' ],
        );
    }

    $timeout = $timeout + time();

    my $s_get_aggr_state = sub {
        my $cmd
            = ( $context =~ /7Mode|Nodescope/i )
            ? "aggr_status"
            : "storage_aggregate_show";
        return $api_set_obj->$cmd( aggregate => $aggregate )
            ->get_parsed_output()->[0]->{'state'};
    };
    $state = &$s_get_aggr_state();

    # Wait for aggregate to be offlined
    while ( $state ne "offline" || time() > $timeout ) {
        $state = &$s_get_aggr_state();
    }

    # Even after timeout value, if the aggregate doesn't gets offline, throw
    # exception message
    if ( $state ne "offline" ) {
        WaflironLibrary::Exceptions::WaflironLibException->throw(
            "Could not offline the aggregate: $aggregate!");
    }
    return 1;
}

##############################################################################
# Change the aggregate state to offline
##############################################################################

sub _aggr_create {
    my ( $pkg, %opts ) = @_;
    my $aggregate   = $opts{'aggregate'};
    my $aggr_size   = $opts{'aggr_size'};
    my $mode        = $pkg->mode();
    my $api_set_obj = $pkg->_get_context_based_api($mode);

    my $s_get_aggr_state = sub {
        my $cmd
            = ( $mode =~ /7Mode/i )
            ? "aggr_status"
            : "storage_aggregate_show";
        return $api_set_obj->$cmd( aggregate => $aggregate )
            ->get_parsed_output()->[0]->{'state'};
    };

    if ( $mode =~ /7Mode/ ) {
        $cmd  = "aggr_create";
        %opts = (
            'aggregate'       => $aggregate,
            'disk-count'      => $aggr_size,
            'privilege-level' => 'diag'
        );
    }
    elsif ( $mode =~ /CMode/ ) {

        $cmd  = "storage_aggregate_create";
        %opts = (
            'aggregate'       => $aggregate,
            'diskcount'       => $aggr_size,
            'privilege-level' => 'diag',
        );
    }
    else {
        NATE::Exceptions::Argument->throw(
            "Invalid value for \'Mode\': " . $mode );
    }
    $api_set_obj->$cmd(%opts);

    my $timeout = time() + LOCAL_WAFLIRON_TIMEOUT;
    my $state   = &$s_get_aggr_state();
    while ( $state ne "online" && time() <= $timeout ) {
        sleep 30;
        $state = &$s_get_aggr_state();
    }
    if ( $state ne "online" ) {
        WaflironLibrary::Exceptions::WaflironLibException->throw(
            "Could not get the $aggregate online in $timeout sec!");
    }
    else {
        $pkg->_logmsg( "$aggregate created succesfully!", 'comment' );
    }

    return 1;
}

##############################################################################
# Resizes the given volume
##############################################################################

sub _vol_resize {
    my ( $pkg, %opts ) = @_;
    $Log->enter();
    $pkg->_logmsg( "Arguements are : \n" . Dumper( \%opts ), 'debug' );

    my $aggr    = $opts{'aggr'} ? $opts{'aggr'} : "";
    my $vol     = $opts{'vol'};
    my $size    = $opts{'size'};
    my $vserver = $opts{'vserver'} ? $opts{'vserver'} : $::VSERVER;
    my $cmd;
    my $mode        = $pkg->mode();
    my $api_set_obj = $pkg->_get_context_based_api($mode);

    if ( $mode =~ /7Mode/ ) {
        $vol_params{'volume'} = $vol;
        $vol_params{'size'}   = $size;
        $cmd                  = "vol_size";
    }
    elsif ( $mode =~ /CMode/ ) {
        $vol_params{'vserver'}   = $vserver;
        $vol_params{'volume'}    = $vol;
        $vol_params{'aggregate'} = $aggr;
        $vol_params{'size'}      = $size;
        $cmd                     = "volume_modify";
    }
    else {
        NATE::Exceptions::Argument->throw(
            "Invalid value for \'Mode\': " . $mode );
    }
    $api_set_obj->$cmd(%vol_params);
    $Log->exit();
    return 1;
}

##############################################################################
# Create the volume using given parameter and wait till it is online
##############################################################################

sub _vol_create {
    my ( $pkg, %opts ) = @_;
    $Log->enter();
    $pkg->_logmsg( "Arguements are : \n" . Dumper( \%opts ), 'debug' );
    my $aggr        = $opts{'aggr'}       ? delete $opts{'aggr'} : "";
    my $vol         = $opts{'vol'}        ? delete $opts{'vol'}  : "";
    my $is_tvol     = $opts{'disk-count'} ? 1                    : 0;
    my $size_attrib = $is_tvol            ? 'disk-count'         : 'size';
    my $size
        = $opts{$size_attrib} ? delete $opts{$size_attrib} : $::FLEX_SIZE;
    my $vserver = $opts{'vserver'} ? delete $opts{'vserver'} : 'vs0';
    my $is_root = $opts{'root'}    ? delete $opts{'root'}    : 0;
    my $cmd;

    $root_vol = $pkg->_get_root_obj('vol');
    my $mode        = $pkg->mode();
    my $api_set_obj = $pkg->_get_context_based_api($mode);
    my %vol_params;
    my $s_get_vol_state = sub {
        my $cmd
            = ( $mode =~ /7Mode/i )
            ? "vol_status"
            : "volume_show";
        return $api_set_obj->$cmd( 'volume' => $opts{'volume'} )
            ->get_parsed_output()->[0]->{'state'};
    };

    if ( $mode =~ /7Mode/ ) {
        $vol_params{'volume'}               = $vol;
        $vol_params{'containing-aggr-name'} = $aggr;
        $vol_params{$size_attrib}           = $size;
        $cmd                                = "vol_create";
    }
    elsif ( $mode =~ /CMode/ ) {
        $vol_params{'vserver'}   = $vserver;
        $vol_params{'volume'}    = $vol;
        $vol_params{'aggregate'} = $aggr;
        $vol_params{'size'}      = $size;
        $cmd                     = "volume_create";
    }
    else {
        NATE::Exceptions::Argument->throw(
            "Invalid value for \'Mode\': " . $mode );
    }

    if ($is_root) {
        my $proper_vol_size = $pkg->_verify_vol_size( 'size' => $size );
        $vol_params{'size'} = $proper_vol_size;
    }

    $api_set_obj->$cmd(%vol_params);

    my $timeout = time() + LOCAL_WAFLIRON_TIMEOUT;
    my $state   = &$s_get_vol_state();
    while ( $state ne "online" && time() <= $timeout ) {
        sleep 30;
        $state = &$s_get_vol_state();
    }
    if ( $state ne "online" ) {
        WaflironLibrary::Exceptions::WaflironLibException->throw(
            "Could not get the $opts{'volume'} online in $timeout sec!");
    }
    else {
        $pkg->_logmsg( "$opts{'volume'} created succesfully!", 'comment' );
    }
    if ($is_root) {
        $pkg->_logmsg( "Copying the system files", 'comment' );
        $pkg->_copy_system_files(
            'svol'    => $root_vol,
            'dvol'    => $vol,
            'client'  => $pkg->client(),
            'timeout' => 1200
        );
    }

    $Log->exit();
    return 1;
}

##############################################################################
# Verify and fix the necessary vol size for new temp_root
##############################################################################

sub _verify_vol_size {
    my ( $pkg, %opts ) = @_;
    $Log->enter();
    $pkg->_logmsg( "Arguements are : \n" . Dumper( \%opts ), 'debug' );

    my $mode = $pkg->mode();
    if ( $mode eq "CMode" ) {
        WaflironLibrary::Exceptions::WaflironLibException->throw(
            "root vol can't be created in $mode");
    }

    my $api_set_obj = $pkg->_get_context_based_api($mode);
    $root_vol = $pkg->_get_root_obj('vol');
    my $resp_obj = $api_set_obj->df( 'volume' => '/vol/' . $root_vol );
    my $root_vol_objs = $resp_obj->get_parsed_output();

    my %r_vol_size_attribs;
    foreach my $element ( @{$root_vol_objs} ) {
        foreach my $attrib ( keys %{$element} ) {
            if ( $element->{$attrib} =~ /(\d+)MB/ ) {
                $r_vol_size_attribs{$attrib} += $1;
            }
        }
    }
    my $required_vol_size = $opts{'size'};
    if ( $opts{'size'} < $r_vol_size_attribs{'total'} ) {
        $required_vol_size = int( $r_vol_size_attribs->{used} * 2 );
    }
    $Log->exit();
    return $required_vol_size;
}

##############################################################################
# Rename the volume using given parameter
##############################################################################

sub _vol_rename {
    my ( $pkg, %opts ) = @_;
    $Log->enter();
    $pkg->_logmsg( "Arguements are : \n" . Dumper( \%opts ), 'debug' );

    my $mode        = $pkg->mode();
    my $api_set_obj = $pkg->_get_context_based_api($mode);
    my $newname     = $opts{'newname'} || $opts{'new-volume-name'};

    if ( $mode =~ /7Mode/ ) {
        $cmd = "vol_rename";
        $opts{'privilege-level'} = 'diag';
    }
    elsif ( $mode =~ /CMode/ ) {

        $cmd = "volume_rename";
        $opts{'privilege-level'} = 'diag';
    }
    else {
        NATE::Exceptions::Argument->throw(
            "Invalid value for \'Mode\': " . $mode );
    }
    $api_set_obj->$cmd(%opts);
    $pkg->_logmsg( "$opts{'volume'} renamed to $newname succesfully!",
        'comment' );

    $Log->exit();
    return 1;
}

##############################################################################
# Find out if the container is an aggregate or volume
##############################################################################

sub _get_container_type {
    my ( $pkg, $context ) = @_;
    my $ci          = $pkg->command_interface();
    my $container   = $pkg->container();
    my $api_set_obj = $pkg->_get_context_based_api($context);
    my $aggr_cmd
        = $context =~ /7Mode|Nodescope/i
        ? "aggr_status"
        : "storage_aggregate_show";
    my $vol_cmd
        = $context =~ /7Mode|Nodescope/i
        ? "vol_status"
        : "volume_show";

    try {
        $api_set_obj->$aggr_cmd( 'aggregate' => $container );
        return "aggregate";
    }
    catch NATE::BaseException with {
        my $eo   = shift;
        my $text = $eo->text();
        if ( $text
            =~ /No matching aggregates found|No aggregate named|There are no entries matching your query|\'$container\' is a volume; use 'vol status/
            )
        {
            try {
                $api_set_obj->$vol_cmd( 'volume' => $container );
                return "volume";
            }
            catch NATE::BaseException with {
                my $eo   = shift;
                my $text = $eo->text();
                if ( $text
                    =~ /No matching volumes found|There are no entries matching your query/
                    )
                {
                    NATE::Exceptions::Argument->throw(
                        "$container does not exist (neither volume nor aggr)"
                    );
                }
            };
        }
    };
}

##############################################################################
# Build the default arguments for wafliron_start based on options
##############################################################################

#sub _get_wafliron_default_args {
#    my ( $pkg, %args ) = @_;
#    $Log->enter();
#    $pkg->_logmsg( "Arguements are : \n" . Dumper( \%opts ), 'debug' );
#
#    my %params;
#    my $context = $args{'context'};
#
#    # Build default arguments for CMode
#    if ( $context eq "CMode" ) {
#        $params{'aggregate'} = $pkg->container();
#        $params{'include-mirrors'}
#            = $args{'include_mirrors'} ? "true" : "false";
#
#        $params{'optional-commit'}
#            = ( $args{'optional_commit'} == 1 )
#            ? "true"
#            : "false";
#
#        if ( ref $args{'prev_cp'} eq "HASH" ) {
#            $params{'optional-commit'} = "true";
#            if ( $args{'prev_cp'}->{'interactive'} == 0 ) {
#                my %vol_resp_hash = %{ $args{'prev_cp'}->{'vol_resp'} };
#                $params{'previous-cp'} = join( ',',
#                    map { $_ . ':' . $vol_resp_hash{$_}; }
#                        keys %vol_resp_hash );
#            }
#            elsif ( $args{'prev_cp'}->{'interactive'} == 1 ) {
#                $params{'previous-cp'} = "";
#            }
#            elsif ( $args{'prev_cp'}->{'interactive'} == 2 ) {
#                my $resp = $args{'prev_cp'}->{'ic_vol_resp'};
#                $resp =~ s/\s+//;
#                $params{'previous-cp'} = $resp;
#            }
#            else {
#                if ( $args{'prev_cp'}->{'interactive'} !~ /[0-2]/ ) {
#                    NATE::Exceptions::Argument->throw(
#                              "Invalid arguement value for 'ic_vol_resp' : "
#                            . "$args{'prev_cp'}->{'ic_vol_resp'}" );
#                }
#            }
#        }
#    }
#
#    # Build default arguments for 7Mode/Nodescope or IRON_ON_ROOT
#    elsif ( $context eq "7Mode" || $context eq "Nodescope" || $::IRON_ON_ROOT =~ /(core|multi|spare)/) {
#
#        $params{'aggregate'} = $pkg->container();
#        $params{'force'}     = 1 if ( $args{'force'} );
#        $params{'triage'}    = 1 if ( defined $args{'triage'} );
#        $params{'check-compressed-data-blocks'} = 1
#            if ( defined $args{'compression'} );
#        $params{'inject-errors'} = $args{'inject_file'}
#            if ( defined $args{'inject_file'} );
#        $params{'volume-to-iron-first'} = $args{'volume_order'}
#            if ( defined $args{'volume_order'} );
#        $params{'optional-commit'}
#            = defined $args{'optional_commit'} ? $args{'optional_commit'} : 0;
#
#        if ( ref $args{'prev_cp'} eq "HASH" ) {
#            $params{'optional-commit'} = 1;
#            if ( $args{'prev_cp'}->{'interactive'} == 0 ) {
#                my %vol_resp_hash = %{ $args{'prev_cp'}->{'vol_resp'} };
#                $params{'prev_cp'} = join( ',',
#                    map { $_ . ':' . $vol_resp_hash{$_}; }
#                        keys %vol_resp_hash );
#            }
#            elsif ( $args{'prev_cp'}->{'interactive'} == 1 ) {
#                $params{'prev_cp'} = " ";
#            }
#            elsif ( $args{'prev_cp'}->{'interactive'} == 2 ) {
#                my $resp = $args{'prev_cp'}->{'ic_vol_resp'};
#                $resp =~ s/\s+//;
#                $params{'prev_cp'} = $resp;
#            }
#            else {
#                if ( $args{'prev_cp'}->{'interactive'} !~ /[0-2]/ ) {
#                    NATE::Exceptions::Argument->throw(
#                              "Invalid arguement value for 'ic_vol_resp' : "
#                            . "$args{'prev_cp'}->{'ic_vol_resp'}" );
#                }
#            }
#        }
#
#    }
#    else {
#        my $error = "Context should be either of CMode, Nodescope or 7Mode";
#        if(defined $::IRON_ON_ROOT){
#            $error .= " or the IRON_ON_ROOT should be either of core, spare or multi.";
#        }
#        $error .= "\nProvided inputs are,\n";
#        $error .= "Context is : " . $context;
#        $error .= "\$IRON_ON_ROOT is: " . $::IRON_ON_ROOT if(defined $::IRON_ON_ROOT);
#
#        NATE::Exceptions::Argument->throw($error);
#    }
#    $Log->exit();
#    return \%params;
#
#}

sub _get_wafliron_default_args {
    my ( $pkg, %opts ) = @_;
    $Log->enter();
    $pkg->_logmsg( "Arguements are : \n" . Dumper( \%opts ), 'comment' );

    my $context = $opts{'context'};
    $pkg->_logmsg( "\$::IRON_ON_ROOT is : " . $::IRON_ON_ROOT, 'comment' );

    if ( defined $::IRON_ON_ROOT && $::IRON_ON_ROOT =~ /(core|multi|spare)/ )
    {
        return $pkg->_get_7Mode_or_ioc_on_ra_wafliron_default_args(%opts);
    }
    elsif ( $context eq "CMode" ) {
        return $pkg->_get_CMode_wafliron_default_args(%opts);
    }
    elsif ( $context eq "7Mode" || $context eq "Nodescope" ) {
        return $pkg->_get_7Mode_or_ioc_on_ra_wafliron_default_args(%opts);
    }
    else {
        my $error = "Context should be either of CMode, Nodescope or 7Mode";
        if ( defined $::IRON_ON_ROOT ) {
            $error
                .= " or the IRON_ON_ROOT should be either of core, spare or multi.";
        }
        $error .= "\nProvided inputs are,\n";
        $error .= "Context is : " . $context;
        $error .= "\$IRON_ON_ROOT is: " . $::IRON_ON_ROOT
            if ( defined $::IRON_ON_ROOT );
        $Log->exit();
        NATE::Exceptions::Argument->throw($error);
    }
    $Log->exit();
}

sub _get_7Mode_or_ioc_on_ra_wafliron_default_args {
    my ( $pkg, %opts ) = @_;
    $Log->enter();
    my %params;
    $params{'aggregate'} = $pkg->container();
    $params{'force'}     = 1 if ( $opts{'force'} );
    $params{'triage'}    = 1 if ( defined $opts{'triage'} );
    $params{'check-compressed-data-blocks'} = 1
        if ( defined $opts{'compression'} );
    $params{'inject-errors'} = $opts{'inject_file'}
        if ( defined $opts{'inject_file'} );
    $params{'volume-to-iron-first'} = $opts{'volume_order'}
        if ( defined $opts{'volume_order'} );
    $params{'optional-commit'}
        = defined $opts{'optional_commit'} ? $opts{'optional_commit'} : 0;

    if ( ref $opts{'prev_cp'} eq "HASH" ) {
        $params{'optional-commit'} = 1;
        if ( $opts{'prev_cp'}->{'interactive'} == 0 ) {
            my %vol_resp_hash = %{ $opts{'prev_cp'}->{'vol_resp'} };
            $params{'prev_cp'} = join( ',',
                map { $_ . ':' . $vol_resp_hash{$_}; }
                    keys %vol_resp_hash );
        }
        elsif ( $opts{'prev_cp'}->{'interactive'} == 1 ) {
            $params{'prev_cp'} = " ";
        }
        elsif ( $opts{'prev_cp'}->{'interactive'} == 2 ) {
            my $resp = $opts{'prev_cp'}->{'ic_vol_resp'};
            $resp =~ s/\s+//;
            $params{'prev_cp'} = $resp;
        }
        else {
            if ( $opts{'prev_cp'}->{'interactive'} !~ /[0-2]/ ) {
                NATE::Exceptions::Argument->throw(
                          "Invalid arguement value for 'ic_vol_resp' : "
                        . "$opts{'prev_cp'}->{'ic_vol_resp'}" );
            }
        }
    }
    $Log->exit();
    return \%params;
}

sub _get_CMode_wafliron_default_args {
    my ( $pkg, %opts ) = @_;
    $Log->enter();
    my %params;

    $params{'aggregate'} = $pkg->container();
    $params{'include-mirrors'} = $opts{'include_mirrors'} ? "true" : "false";

    $params{'optional-commit'}
        = ( $opts{'optional_commit'} == 1 )
        ? "true"
        : "false";

    if ( ref $opts{'prev_cp'} eq "HASH" ) {
        $params{'optional-commit'} = "true";
        if ( $opts{'prev_cp'}->{'interactive'} == 0 ) {
            my %vol_resp_hash = %{ $opts{'prev_cp'}->{'vol_resp'} };
            $params{'previous-cp'} = join( ',',
                map { $_ . ':' . $vol_resp_hash{$_}; }
                    keys %vol_resp_hash );
        }
        elsif ( $opts{'prev_cp'}->{'interactive'} == 1 ) {
            $params{'previous-cp'} = "";
        }
        elsif ( $opts{'prev_cp'}->{'interactive'} == 2 ) {
            my $resp = $opts{'prev_cp'}->{'ic_vol_resp'};
            $resp =~ s/\s+//;
            $params{'previous-cp'} = $resp;
        }
        else {
            if ( $opts{'prev_cp'}->{'interactive'} !~ /[0-2]/ ) {
                NATE::Exceptions::Argument->throw(
                          "Invalid arguement value for 'ic_vol_resp' : "
                        . "$opts{'prev_cp'}->{'ic_vol_resp'}" );
            }
        }
    }
    $Log->exit();
    return \%params;
}

sub _get_wafliron_status {
    my ( $pkg, $context ) = @_;
    my $state;

    if ( $context eq 'CMode' || $context eq 'Nodescope' ) {
        try {
            $state = $pkg->wafl_comp()->state()->iron_status();
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            goto not_running if ($exception_object->text()
                            =~ /No matching wafliron found/);
        };
        catch NACL::Exceptions::NoElementsFound with {
            my $exception_object = shift;
            if (ref($exception_object)
                =~ /NACL::Exceptions::NoElementsFound/ )
            {
                goto not_running;
            }
        };

        if (   ( $state eq "starting" )
            || ( $state eq "scanning" )
            || ( $state eq "checking_lost_blocks" )
            || ( $state eq "checking_lost_inodes" )
            || ( $state eq "finishing" )
            || ( $state eq "aborting" ) )
        {

            # wafliron is running
            $pkg->_logmsg( "wafliron is running, returning 1", 'comment' );
            return ( wantarray ? ( 1, '' ) : 1 );
        }
        elsif ( ( $state eq "not_running" )
             || ( $state eq "waiting_for_commit" ) ) {
            my $mount_state = $pkg->wafl_comp()->state->state();
            if ( $mount_state eq 'iron_restricted' ) {

                # looks like IOC is waiting for commit/reject
                $pkg->_logmsg(
                    "IOC is waiting for commit/reject, returning 2 (1)",
                    'comment' );
                return ( wantarray ? ( 2, '' ) : 2 );
            }
            else {

             # wafliron is not running (but has been run sometime in the past)
            not_running: $pkg->_logmsg(
                    "wafliron is NOT running (but did run sometime earlier), returning 0",
                    'comment'
                );
                return ( wantarray ? ( 0, '' ) : 0 );
            }
        }
        else {
            WaflironLibrary::Exceptions::WaflironLibException->throw(
                "unexpected wafliron state result: '$state'");
        }
    }
    else {

        my $try_again   = 1;
        my $api_set_obj = $pkg->_get_context_based_api($context);
        my $aggregate   = $pkg->container();
    again:

        my $resp_obj = $api_set_obj->aggr_wafliron_status(
            'aggregate'          => $aggregate,
            'privilege-level'    => "advanced",
            'connectrec-timeout' => 300
        );
        my $output = $resp_obj->get_raw_output();

        if (   $output =~ /is\s+active/
            || $output =~ /is\s+in\s+the\s+process\s+of\s+starting/ )
        {
            $pkg->_logmsg( "wafliron is running, returning 1", 'trace' );
            return ( wantarray ? ( 1, $output ) : 1 );
        }
        elsif ($output =~ /is\s+not\s+currently\s+active/
            || $output =~ /no\s+waflirons\s+in\s+progress/ )
        {
            $pkg->_logmsg( "wafliron is NOT running now, returning 0",
                'trace' );
            return ( wantarray ? ( 0, $output ) : 0 );
        }
        elsif ( $output =~ /either\s+commit\s+or\s+reject/ ) {
            $pkg->_logmsg(
                "IOC is waiting for commit/reject, returning 2 (2)",
                'trace' );
            return ( wantarray ? ( 2, $output ) : 2 );
        }
        else {

      # after some operations, e.g. wafliron commit, it can take a bit for the
      # new status to be reflected in the wafliron status output. so, if we
      # end up here, we try just one more time before failing. see burt338391.
            if ($try_again) {
                $try_again = 0;
                sleep 20;
                goto again;
            }
            else {
                WaflironLibrary::Exceptions::WaflironLibException->throw(
                    "unexpected result: '$output'");
            }
        }
    }
}

##############################################################################
# Function: _get_ontap_version
# Purpose:  Determine the ontap version
# Notes:    This method updates the ontap version only on to WaflironLibrary
#           instance. This method can't be used to return the ontap version to
#            the caller.
##############################################################################
sub _get_ontap_version {
    my $pkg = shift;
    $pkg->_logmsg( "Getting the ONTAP Version info", "debug" );
    my $vm_obj = NACL::APISet::Version->new(
        hostobj    => host( $pkg->hostname() ),
        category   => "Node",
        shell_type => $pkg->mode()
    );
    my $release = $vm_obj->get_version_attribute( attribute => "release" );
    my $mapped_release
        = $vm_obj->map_release_to_codename( release => $release );
    $pkg->ontap_release_codename($mapped_release);

    if ( $mapped_release =~ /rollingrock/ ) {
        $pkg->ontap_release("RollingRock");
    }
    elsif ( $mapped_release =~ /boilermaker/ ) {
        $pkg->ontap_release("Boilermaker");
    }
    elsif ( $mapped_release =~ /ironcity/ ) {
        $pkg->ontap_release("IronCity");
    }
    elsif ( $mapped_release =~ /sierranevada/ ) {
        $pkg->ontap_release("SierraNevada");
    }
    elsif ( $mapped_release =~ /fullsteam/ ) {
        $pkg->ontap_release("Fullsteam");
    }
    else {
        WaflironLibrary::Exceptions::WaflironLibException->throw(
            "Unknown release: $mapped_release");
    }
}

sub _wait_wafliron_status {
    my ( $pkg, $expected_state ) = @_;

    my $timeout = $pkg->cmd_timeout();
    $timeout = $timeout + time();

    while (( $pkg->_get_wafliron_status() != $expected_state )
        || ( time() > $timeout ) )
    {
        Tharn::sleep 10;
    }
    if ( $pkg->_get_wafliron_status() != $expected_state ) {
        WaflironLibrary::Exceptions::WaflironLibException->throw(
                  "Wafliron could not reach the state: $expected_state in "
                . $pkg->cmd_timeout()
                . " sec" );
    }
}

sub _set_flags {
    my ( $pkg, $flags ) = @_;
    my $priv_level = 'diag';
    my $mode       = $pkg->mode();
    my $apiset_obj
        = $mode eq "CMode"
        ? $pkg->filer_api_nodescope()
        : $pkg->filer_api();
    my %available_flags
        = %{ $apiset_obj->printflag()->get_parsed_output()->[0] };
    foreach my $flag ( keys %{$flags} ) {
        if ( !exists $available_flags{$flag} ) {
            $pkg->_logmsg(
                "Can't set the $flag, as this flag is unavailable on "
                    . $pkg->ontap_release_codename(),
                'warn'
            );
            next;
        }
        my $value = $flags->{$flag};
        my $resp  = $apiset_obj->setflag(
            'flag'            => $flag,
            'value'           => $value,
            'privilege-level' => $priv_level
        ) if ( $value ne "" );
    }
    return 1;
}

sub _set_scan_speed {
    my ( $pkg, $speed ) = @_;

    my $priv_level = 'diag';
    my $mode       = $pkg->mode();
    my $api_set_obj
        = $mode eq "CMode"
        ? $pkg->filer_api_nodescope()
        : $pkg->filer_api();

    $api_set_obj->wafl_scan_speed(
        'speed'           => $speed,
        'privilege-level' => $priv_level
    );

    my $parsed_output
        = $api_set_obj->wafl_scan_speed( 'privilege-level' => $priv_level )
        ->get_parsed_output();
    my $value = $parsed_output->[0]->{'wafl_scan_speed'};
    if ( $value != $speed ) {
        WaflironLibrary::Exceptions::WaflironLibException->throw(
            "Could not set the wafl scan speed to $speed");
    }
    return 1;
}

sub _wi_login {
    my $pkg = shift;
    my $c_out;
    my $transit_obj = $pkg->transit_obj();
    $transit_obj->change_state( to => 'CLI', 'append_output_to' => \$c_out );
    $pkg->_check_connection_state();
    return $c_out;
}

sub _reboot {
    my $pkg = shift;
    my $c_out;
    my $transit_obj = $pkg->transit_obj();
    $transit_obj->reboot( 'append_output_to' => \$c_out );
    $pkg->_check_connection_state();
    return $c_out;
}

sub _logmsg {
    my ( $pkg, $msg, $level ) = @_;
    my $caller = ( caller(1) )[3];
    $Log->$level( $caller . ": " . $msg );
}

##############################################################################
# Function: _misc_printflag
# Purpose:  return the printflag value
# Notes:    Execute 'printflag <flag>' on a 7Mode console or CMode Nodescope
#           prompt and return single flag the value if single flag requested
#            or hashref if multiple flags are requested.
##############################################################################

sub _misc_printflag {
    my ( $pkg, %opts ) = @_;

    my $apiset_obj
        = $pkg->mode() eq "CMode"
        ? $pkg->filer_api()
        : $pkg->filer_api_nodescope();

    my $flag          = $opts{flag};
    my $parsed_output = $apiset_obj->printflag(
        'flag'            => $flag,
        'privilege-level' => "diag"
    )->get_parsed_output();
    my $num_of_flags = scalar( keys %{ $parsed_output->[0] } );

    if ( $num_of_flags == 1 ) {

        # single flag, just return value
        return $parsed_output->[0]->{$flag};
    }
    else {

        # multiple flags, return data structure ref.
        return \%{ $parsed_output->[0] };
    }
}

##############################################################################
# Function: _get_root_obj
# Purpose:  return the root object name based on object type (volume/aggregate)
# Notes:    Execute aggr status of vol status command, get the root vol/aggr'
#           and return the same.
##############################################################################

sub _get_root_obj {
    my ( $pkg, $obj_type ) = @_;

    my $apiset_obj
        = $pkg->mode() eq "7Mode"
        ? $pkg->filer_api()
        : $pkg->filer_api_nodescope();

    my ( $cmd_prefix, $type );
    if ( $obj_type =~ /aggr/i ) {
        $cmd_prefix = "aggr";
        $type       = "aggregate";
    }
    elsif ( $obj_type =~ /vol/i ) {
        $cmd_prefix = "vol";
        $type       = "volume";
    }
    else {
        NATE::Exceptions::Argument->throw("Invalid object_type passed!");
    }
    my $command       = $cmd_prefix . "_status";
    my $parsed_output = $apiset_obj->$command()->get_parsed_output();

    foreach my $href ( @{$parsed_output} ) {
        if ( $href->{'options'}->[0]->{'root'} ) {
            return $href->{$type};
        }
    }
    WaflironLibrary::Exceptions::WaflironLibException->throw(
        "Error finding the root $obj_type");
}

##############################################################################
# Function: _get_default_context
# Purpose:  return the default based on 'mode' of the filer
##############################################################################

sub _get_default_context {
    my $pkg = shift;
    return $pkg->mode() eq "CMode" ? "CMode" : "7Mode";
}

##############################################################################
# Function: _get_context_based_api
# Purpose:  return the appropriate apiset obj based on the context
##############################################################################

sub _get_context_based_api {
    my ( $pkg, $context ) = @_;

    my $apiset_obj;
    my $mode = $pkg->mode();

    if ( $mode eq "CMode" ) {
        if (   $::IRON_ON_ROOT =~ /(core|multi|spare)/i
            && $context !~ /(Nodescope|CMode)/i )
        {
            return $pkg->filer_api_console();
        }
        elsif ( $context =~ /(Nodescope|CMode)/ ) {
            return $context eq "CMode"
                ? $pkg->filer_api()
                : $pkg->filer_api_nodescope();
        }
        else {
            NATE::Exceptions::Argument->throw(
                "Invalid context: " . $context . " for CMode" );
        }
    }
    elsif ( $mode eq "7Mode" ) {
        return $::IRON_ON_ROOT =~ /(core|multi|spare)/
            ? $pkg->filer_api_console()
            : $pkg->filer_api();
    }
    else {
        NATE::Exceptions::Argument->throw(
            "Invalid value for filer mode : " . $mode );
    }
}
###############################################################################
# Function: _create_apiset_objs
# Purpose: Creates APISet Objects on demand and push them to the instance
#
# Note   : This method can only be used in the WaflironLibrary::init()
#
###############################################################################

sub _create_apiset_objs {
    my $pkg             = shift;
    my $mode            = $pkg->mode();
    my $conn_type       = $mode eq "CMode" ? "ssh" : "rsh";
    my $conn_persistent = $pkg->conn_console();
    my $conn_persay     = $pkg->conn();

    my %conn_obj_args = (
        'filer_api'         => $conn_persay,
        'filer_api_console' => $conn_persistent
    );
    $conn_obj_args{'filer_bsd_ssh'} = $conn_persay
        if ( $pkg->ontap_release() ne "IronCity" );
    if ( $mode eq "CMode" ) {
        $conn_obj_args{'filer_api_nodescope'}
            = defined $::IRON_ON_ROOT ? $conn_persistent : $conn_persay;
    }

    @conn_obj_args{ 'filer_api_firmware', 'filer_api_maint' }
        = ( $conn_persistent, $conn_persistent )
        if ($::IRON_ON_ROOT);
    my %conn_set_args = (
        'filer_api'           => $mode,
        'filer_api_console'   => $mode,
        'filer_bsd_ssh'       => 'Systemshell',
        'filer_api_maint'     => 'Maintenance',
        'filer_api_firmware'  => 'Firmware',
        'filer_api_nodescope' => 'Nodescope'
    );

    foreach my $conn_name ( keys %conn_obj_args ) {
        my $connection = $conn_obj_args{$conn_name};
        my $set        = $conn_set_args{$conn_name};
        my $hostname   = $pkg->hostname();
        my $apiset_obj;

        $pkg->_logmsg( "Creating an APISet object with set: " . $set,
            'comment' );
        if ( $set eq "Systemshell" ) {
            $apiset_obj = NACL::APISet->new(
                name      => $hostname,
                category  => "Node",
                interface => "CLI",
                set       => $set,
            );
        }
        else {
            $apiset_obj = NACL::APISet->new(
                connection => $connection,
                category   => "Node",
                interface  => "CLI",
                set        => $set,
            );
        }
        $pkg->$conn_name($apiset_obj);
    }
}

###############################################################################
# Function: _wafliron_pick_new_root
# Purpose: Halt the system and pick a new root volume.
#          Bring system back to up state unless there is a panic. In the case
#          of a panic, control will be returned to the caller at that point.
###############################################################################

#sub _wafliron_pick_new_root {
#    my ( $pkg, $opts ) = @_;
#
#    $Log->enter();
#    $pkg->_logmsg( "Arguements are : \n" . Dumper( \%opts ), 'debug' );
#
#    # declare variables
#    my ( $text, $cmd, $boot_state, $root_aggr, $apiset_obj_ns );
#
#    # get the apiset obj
#    my $context = $pkg->mode() eq 'CMode' ? "CMode" : "7Mode";
#    my $apiset_obj = $pkg->_get_context_based_api( 'context' => $context );
#
#    if ( $pkg->mode() eq "CMode" ) {
#        $apiset_obj_ns
#            = $pkg->_get_context_based_api( 'context' => "Nodescope" );
#    }
#
#    # reboot option
#    my $reboot = defined( $opts{reboot} ) ? $opts{reboot} : 0;
#
#    # wafl scan speed
#    my $scan_speed = defined( $opts{scan_speed} ) ? $opts{scan_speed} : 10000;
#
#    ## TODO : Delete this param as not used
#    # tmp_root indicates a new root volume has already been created and we
#    # should not online the aggr before iron
#    my $tmp_root_done = defined( $opts{tmp_root} ) ? "$opts{tmp_root}" : 0;
#
#    # Eventhough after setting the flags If there is any panic when onlining
#    # the aggregate when filer is up, then powercycle can be set so that the
#    # aggregate will be onlined from maintainance mode
#    my $powercycle = defined( $opts{powercycle} ) ? "$opts{powercycle}" : 0;
#
#    my $already_halted
#        = defined( $opts{already_halted} ) ? "$opts{already_halted}" : 0;
#    my $type = defined( $opts{type} ) ? "$opts{type}" : 'aggr';
#
#    # This option can be used when we pick corrupted vol as root in which case
#    # the volume wil be restricted
#    my $corrupted_vol
#        = defined( $opts{corrupted_vol} ) ? "$opts{corrupted_vol}" : 0;
#
#    # If the delete_all_snaps option is set then we must get into boot menu
#    #  and delete all snaps before the system will come up
#    my $delete_all_snaps
#        = defined( $opts{delete_all_snaps} ) ? "$opts{delete_all_snaps}" : 0;
#    my $aggr = $opts{'aggr'};
#
## for CMode, we can't create temp root hence, forcing the root volume to be AUTOROOT
#    my $vol = ( $pkg->mode() eq "CMode" ) ? "AUTOROOT" : $opts{'vol'};
#
#    # Get the transit_to object
#    my $transit_obj = $pkg->transit_obj();
#    my $api_maint   = $pkg->filer_api_maint();
#
#    # For AUTOROOT as the new root vol, we will need to reboot the filer
#    # to come up with AUTOROOT as the new root vol. This overwrites
#    # $reboot setting if it was set to 0.
#    if ( $vol eq "AUTOROOT" ) {
#        $reboot = 1;
#    }
#
#    if ( !$already_halted ) {
#
#        # make sure the filer is in the CLI prompt
#        $pkg->_wi_login();
#
#        $boot_state = $transit_obj->get_state( 'timeout' => 300 );
#        if ( $boot_state eq 'CLI' ) {
#            $root_aggr = $pkg->_get_root_obj('aggregate');
#
#    # Corrupted volume will be restricted/not visible(when aggr is restricted)
#    # Hence this check not required(volume will not be visible).
#    # This check will be avoided only in case corrupted_vol options is set
#
#            if ( ( !$corrupted_vol ) && ( $pkg->mode() ne "CMode" ) ) {
#                $pkg->_logmsg( "Checking corrupted volume", 'comment' );
#                $@ = undef;
#                eval {
#                    my $apiset
#                        = $pkg->mode() eq "CMode"
#                        ? $apiset_obj_ns
#                        : $apiset_obj;
#                    $apiset->vol_options( 'volume' => $vol );
#                };
#                if ( $@ =~ /No volume named .* exists/is ) {
#                    $pkg->_logmsg( "Intended root volume $vol does not exist",
#                        'debug' );
#                    $vol = ( $aggr eq 'aggr0' ) ? 'vol0' : 'vol0_temp';
#                    $pkg->_logmsg( "Using $vol as root volume!", 'debug' );
#                }
#            }
#        }
#        $pkg->_logmsg( "Checking root aggr", 'debug' );
#        if ( $root_aggr ne $aggr ) {
#
#            # aggr is not already the root volume so we have to
#            # get into maint mode and make it root
#            $transit_obj->change_state(
#                'to'         => 'MAINT',
#                'powercycle' => $powercycle
#            );
#            my $aggr_state = $api_maint->aggr_status( aggregate => $aggr )
#                ->get_parsed_output()->[0]->{'state'};
#            if ( $aggr_state ne 'online' ) {
#                $text = $api_maint->aggr_online(
#                    'aggregate'          => $aggr,
#                    'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
#                )->get_raw_output();
#
#                if ( $text =~ /Panic/i ) {
#                    $pkg->_logmsg(
#                        "Filer panicked - returning before setting aggr as new root",
#                        'debug'
#                    );
#                    return ($text);
#                }
#            }
#
#            $text = $api_maint->aggr_options(
#                'aggregate'          => $aggr,
#                'option-name'        => 'root',
#                'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
#            )->get_raw_output();
#
#            if ( $text =~ /Panic/i ) {
#                $pkg->_logmsg(
#                    "Filer panicked - returning before vol_pick_root",
#                    'debug' );
#                return 0;
#            }
#        }
#
#        # Set the required bootargs and reboot to the boot menu
#        $pkg->_logmsg(
#            "Calling transit_to BOOT_MENU -- intended root vol is: $vol",
#            'debug' );
#        $transit_obj->change_state( 'to' => 'FIRMWARE' );
#    }
#
#  # set scanner speed, of course, all we have is slow or not slow. If the user
#  # wants to set the scan speed to less then 1000 then we will make it slow,
#  # otherwise we will leave it at not slow
#    my $api_firmware = $pkg->filer_api_firmware();
#    if ( $scan_speed == -1 || $scan_speed < 2000 ) {
#        $api_firmware->setenv(
#            'varname'            => 'setenv wafl-scanners-slow?',
#            'value'              => 1,
#            'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
#        );
#    }
#    else {
#        $api_firmware->setenv(
#            'varname'            => 'setenv wafl-scanners-slow?',
#            'value'              => 0,
#            'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
#        );
#    }
#
#    # Must set do not panic flag to successfully boot up
#    $api_firmware->setenv(
#        'varname'            => 'wafl-iron-panic-on-corruption?',
#        'value'              => 0,
#        'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
#    );
#    $transit_obj->change_state( 'to' => 'BOOT_MENU' );
#
#    # No need to do a vol_pick_root if the root is a traditional volume
#    my $api_console = $pkg->filer_api_console();
#
#    if ( $vol eq 'AUTOROOT' ) {
#
#        my $dhcp_str       = 'Ctrl-C to skip DHCP search';
#        my $dhcp_str_found = 0;
#
#     # Send '1' for normal boot, without running vol_pick_root to use AUTOROOT
#        $cmd = '1';
#        $api_console->execute_raw_command(
#            'command' => $cmd,
#            'connectrec-match_table' =>
#                [ eval $dhcp_str => sub { $dhcp_str_found = 1; } ],
#            'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
#        );
#
#        if ($dhcp_str_found) {
#            $pkg->_logmsg( "Starting Setup...", 'comment' );
#
#            # Do a warm start on the filer
#            $pkg->cold_start_filer( newfs => 0 );
#        }
#
#    }
#
#    if ( $type ne 'trad' && $vol ne "AUTOROOT" ) {
#        $cmd = "vol_pick_root $vol $aggr";
#        $@   = undef;
#        eval {
#            $api_console->execute_raw_command(
#                'command'            => $cmd,
#                'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
#            );
#        };
#
#        # If we had a timeout error try the command again
#        if ( $@ && $@ =~ /IDLE|TIMEOUT/ ) {
#            $api_console->execute_raw_command(
#                'command'            => $cmd,
#                'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
#            );
#        }
#    }
#
#    if ($delete_all_snaps) {
#        $pkg->_logmsg( "Must delete snapshots", 'debug' );
#        $pkg->_delete_all_snaps();
#    }
#    if ($reboot) {
#        $transit_obj->change_state( 'to' => 'CLI' );
#        $pkg->_check_connection_state();
#    }
#    $Log->exit();
#    return 1;
#}

sub _wafliron_pick_new_root {
    my ( $pkg, %opts ) = @_;
    $Log->enter();
    $pkg->_logmsg( "Arguements are : \n" . Dumper( \%opts ), 'debug' );

    my %validated_args;

    # reboot option
    $validated_args{'reboot'} = defined( $opts{reboot} ) ? $opts{reboot} : 0;

    # wafl scan speed
    $validated_args{'scan_speed'}
        = defined( $opts{scan_speed} ) ? $opts{scan_speed} : 10000;

    # Eventhough after setting the flags If there is any panic when onlining
    # the aggregate when filer is up, then powercycle can be set so that the
    # aggregate will be onlined from maintainance mode
    $validated_args{'powercycle'}
        = defined( $opts{powercycle} ) ? "$opts{powercycle}" : 0;

    $validated_args{'already_halted'}
        = defined( $opts{already_halted} ) ? "$opts{already_halted}" : 0;
    $validated_args{'type'} = defined( $opts{type} ) ? "$opts{type}" : 'aggr';

    # This option can be used when we pick corrupted vol as root in which case
    # the volume wil be restricted
    $validated_args{'corrupted_vol'}
        = defined( $opts{corrupted_vol} ) ? "$opts{corrupted_vol}" : 0;

    # If the delete_all_snaps option is set then we must get into boot menu
    #  and delete all snaps before the system will come up
    $validated_args{'delete_all_snaps'}
        = defined( $opts{delete_all_snaps} ) ? "$opts{delete_all_snaps}" : 0;
    $validated_args{'aggr'} = $opts{'aggr'};

    # for CMode, we can't create temp root hence, forcing the root volume to
    # be AUTOROOT
    $validated_args{'vol'}
        = $pkg->mode() eq "CMode" ? "AUTOROOT" : $opts{'vol'};

    # For AUTOROOT as the new root vol, we will need to reboot the filer
    # to come up with AUTOROOT as the new root vol. This overwrites
    # $reboot setting if it was set to 0.
    if ( $vol eq "AUTOROOT" ) {
        $validated_args{'reboot'} = 1;
    }

    if ( $pkg->mode() eq "7Mode" ) {
        return $pkg->_wafliron_pick_new_root_7m(%validated_args);
    }
    elsif ( $pkg->mode() eq "CMode" ) {
        return $pkg->_wafliron_pick_new_root_cm(%validated_args);
        return 0;
    }
    else {
        WaflironLibrary::Exceptions::WaflironLibException->throw(
            "Invalid mode: " . $pkg->mode() );
    }
    $Log->exit();
    return 1;
}

sub _wafliron_on_root {
    my ( $pkg, %opts ) = @_;
    $pkg->_logmsg( Dumper( \%opts ), 'comment' );
    my ( %cp_hash, $prompt, $cmd, $force, $wait, $waitstring );

    # tmp_root indicates a new root volume has already been created and we
    # should not online the aggr before iron
    my $tmp_root_done = defined( $opts{tmp_root} ) ? "$opts{tmp_root}" : 0;

    my $msg
        = "Provided options are : \n"
        . "oc=$opts{'oc'}, root=$opts{'root_vol'}, wait=$opts{'wait'}"
        . "force=$opts{'force'}, type=$opts{'type'}";
    $pkg->_logmsg( $msg, 'debug' );
    $wait = $opts{'wait'};
    $force = $opts{force} || "";
    my $object = $opts{'object'};
    my $cr     = $pkg->conn_console();
    my $console_output;

    # If AUTOROOT was selected as the root vol, we need to transit
    # to boot menu
    $pkg->transit_obj->change_state( 'to' => 'BOOT_MENU' );

    # # At this point we should be at the boot_menu prompt
    ## @step Start wafliron from boot menu

    $pkg->_logmsg( "Start wafliron from boot menu", 'debug' );
    $cmd   = "wafliron ";
    $found = 0;

    if ( $opts{'type'} eq 'trad' ) {
        $waitstring = "Volume $object, wafliron completed|system will reboot";
    }
    else {
        $waitstring
            = "Aggregate $object, wafliron completed|system will reboot";
    }

    if ( $opts{'oc'} ) {
        $cmd .= "-o";
        $cmd .= " -t" if ( $opts{'triage'} );
        $cmd .= " $force" if ($force);
        $cmd .= " $force" if ( $opts{'status_interval'} );

        if ( $opts{'oc'} =~ /prev_cp/ ) {

            $cmd .= " -prev_cp ";
            my $interactive = $opts{'prev_cp'}->{'interactive'};
            my $cp_hash;

            $opts{'wafl_bm_command'} = $cmd;

            # start wafliron with prev_cp in semi-interactive mode
            ( $cp_hash, $console_output )
                = $pkg->_do_prev_cp_semi_interactive(%opts)
                if ( $interactive == 2 );

            # start wafliron with prev_cp in interactive mode
            ( $cp_hash, $console_output )
                = $pkg->_do_prev_cp_interactive(%opts)
                if ( $interactive == 1 );

            # start wafliron with prev_cp in non-interactive mode
            $console_output = $pkg->_do_prev_cp_non_interactive(%opts)
                if ( !$interactive );

            # throw exception if invalid value for 'interactive' parameter
            if ( $interactive !~ /[0-2]/ ) {
                NATE::Exceptions::Argument->throw(
                    "Invalid value -  $opts{'interactive'} for option 'interactive'"
                );
            }
            if ($wait) {

                # See if we have already hit the waitstring
                if ( $console_output !~ /$waitstring/ ) {
                    eval {
                        $console_output .= $cr->execute(
                            'command'            => "",
                            'additional_prompts' => [
                                'Login:', 'Password',
                                'Enabling rsh service'
                            ],
                            'timeout' => LOCAL_WAFLIRON_TIMEOUT
                        );
                    };
                    if ( $@ =~ /timeout/ ) {

                  # If no corruption was found, then no need to commit changes
                        if ( $console_output
                            !~ /Wafliron did not find any corruption/i )
                        {
                            logWarn(
                                "Wafliron time out - did not see the waitstring: $waitstring"
                            );
                        }
                    }
                }
            }

            # Handle any password prompt
            $console_output .= $pkg->_wi_login();
        }
        else {
            $pkg->_logmsg( "I am here = 1: $cmd ", 'comment' );
            if ($wait) {

                # issue wafliron command and wait for it to complete
                $console_output .= $cr->execute(
                    'command' => $cmd,
                    'additional_prompts' =>
                        [ 'Login', 'Password', 'Enabling rsh service' ],
                    'timeout' => LOCAL_WAFLIRON_TIMEOUT
                );
            }
            else {
                $pkg->_logmsg( "I am here = 2: $cmd ", 'comment' );

       # issue wafliron command wait for rsh service to begin before returning
                $console_output .= $cr->execute(
                    'command' => $cmd,
                    'additional_prompts' =>
                        [ 'Enabling rsh service', 'Enabling ssh service' ],
                    'timeout' => LOCAL_WAFLIRON_TIMEOUT
                );
            }
            if ( $console_output =~ /system will reboot/ ) {

                # wait for system to become stable before trying to logon
                sleep(60);
            }

            # Handle any password prompt
            $console_output .= $pkg->_wi_login();
        }
    }
    else {
        $pkg->_logmsg( "I am here = 3: $cmd ", 'comment' );

        # Issue wafliron without any opt_commit parameter
        $cmd .= "-t"                        if ( $opts{'triage'} );
        $cmd .= " $force"                   if ($force);
        $cmd .= " $opts{'status_interval'}" if ( $opts{'status_interval'} );
        if ($wait) {

            # issue wafliron command and wait for it to complete

            $console_output .= $cr->execute(
                'command'            => $cmd,
                'additional_prompts' => ['[Ll]ogin:\s+$'],
                'match_table'        => [ 'rpc failed' => "\n" ],
                'timeout'            => LOCAL_WAFLIRON_TIMEOUT
            );
        }
        if ( $console_output =~ /system will reboot/ ) {

            # wait for system to become stable before trying to logon
            sleep(60);
        }
        $console_output .= $pkg->_wi_login();
    }
    if (   $console_output =~ /wafl.check.info:error|wafl.check.info:ALERT/
        || $console_output =~ /(wafl.iron.*:CRITICAL)/ )
    {
        $found = 1;
    }
    if ( $console_output
        =~ /vol_pick_root: .* in aggregate .* is now the root volume/ )
    {
        $pkg->_logmsg( "Successfully chose new root volume!", 'debug' );
    }

    ## @step Wait for wafliron to finish?
    if ($wait) {
        return ( $found, $console_output, $cp_hash );
    }
    else {

        # Update the log lines
        $pkg->{'log_lines'} .= $console_output;
    }

}

sub _delete_all_snaps {
    my ( $pkg, $opts ) = @_;

    my $transit_obj = $pkg->transit_obj();
    my $api_console = $pkg->filer_api_console();
    my $cmd_output;

    #    $transit_obj->change_state(
    #        'to'      => 'BOOT_MENU',
    #        'haltcmd' => 'halt',
    #        'timeout' => 3600
    #    );

    my $cmd = 'boot_snap_delete';
    eval {
        $cmd_output = $api_console->execute_raw_command(
            'command'                => $cmd,
            'connectrec-timeout'     => 660,
            'connectrec-max_idle'    => 120,
            'connectrec-match_table' => [
                'Delete?'                          => 'y',
                'Do you want to stop the mount of' => 'n'
            ],
            'connectrec-prompts' => [ 'Password', 'Login' ]
        );
    };
}

sub _check_connection_state {
    my $pkg = shift;
    my ( $host_state, $transit_obj );
    $pkg->_logmsg( "Finding the current state of the filer host ... ",
        'comment' );

    my $apiset_obj = $pkg->filer_api_console();
    $apiset_obj->api_reconnect_session()
               unless $apiset_obj->api_is_session_alive();

    $transit_obj = $pkg->transit_obj();
    my $probe_timeout = $pkg->probe_timeout();
    try {
        $host_state = $transit_obj->get_state(
            'timeout'       => 128,
            'probe_timeout' => $probe_timeout
        );
    }
    catch NATE::BaseException with {
        my $ex = shift;
        die("Unknown Exception occurred: $ex->text()");
    };

    $pkg->_logmsg( "Verifying the  connections ...", 'comment' );

    if ( $host_state eq 'CLI' ) {
        foreach my $apiset ( grep ( /_api_|bsd/, keys %{$pkg} ) ) {
            next if ( $apiset =~ /firmware|maint/ );
            $pkg->$apiset->api_reconnect_session()
                unless $pkg->$apiset->api_is_session_alive();
        }
        my $ci = $pkg->command_interface();
        foreach my $key ( keys %{ $ci->_apisets() } ) {
            if ( $key =~ /CLI/ ) {
                $apiset = $ci->{_apisets}{$key};
                $apiset->api_reconnect_session()
                    unless $apiset->api_is_session_alive();
                $apiset->set_privilege( 'privilege-level' => 'diag' );
                last;
            }
        }
    }
    else {
        $pkg->_logmsg(
            "Filer host is in $host_state state hence can't call reconnect_session ...",
            'comment'
        );
    }
}

sub _get_aggr_status {
    my ( $pkg, $context ) = @_;
    my $aggregate   = $pkg->container();
    my $api_set_obj = $pkg->_get_context_based_api($context);
    my $cmd
        = $context =~ /7Mode|Nodescope/i
        ? "aggr_status"
        : "storage_aggregate_show";
    my $state
        = $api_set_obj->$cmd( 'aggregate' => $aggregate )->get_parsed_output()
        ->[0]->{'state'};

    return $state;
}

sub _wafliron_create_temp_root {
    my ( $pkg, %opts ) = @_;
    my ($cmd,       $err_msg1,  $err_msg2,  $err_msg3,    $err_msg4,
        $err_msg5,  $err_msg6,  $err_msg7,  $err_msg8,    $err_msg9,
        $err_msg10, $err_msg11, $err_msg12, $err_msg13,   $pass_msg,
        $fail_msg,  $query1,    $query2,    $start_setup, $dhcp_str,
        $match,     $before,    $after,     $text,        $err_regex
    );
    $err_msg1  = 'Invalid Volume name for command create_temp_root';
    $err_msg2  = 'No (disk|disks) specified for command create_temp_root';
    $err_msg3  = 'Invalid syntax for command create_temp_root';
    $err_msg4  = 'Invalid option .* for command create_temp_root';
    $err_msg5  = 'Invalid raid type.*for command create_temp_root';
    $err_msg6  = 'No disk or invalid disk name specified';
    $err_msg7  = 'Disk \w+\.\w+ is listed twice';
    $err_msg8  = 'Disk \w+\.\w+ does not exist\.';
    $err_msg9  = 'Disk \w+ does not exist';
    $err_msg10 = 'Disk \w+\.\w+ is not a spare disk';
    $err_msg11 = 'Disk \w+\.\w+ is not owned by local host';
    $err_msg12 = 'Assimilation failed';
    $err_msg13 = 'No raid-type specified for command create_temp_root';
    $pass_msg  = 'wafl\.vol\.add:notice';
    $fail_msg  = 'raid\.label\.io\.writeError';
    $query1    = 'Install a new file system\?';
    $query2    = 'This will erase all the data on the disks, are you sure\?';
    $start_setup = 'new hostname';
    $dhcp_str    = 'Ctrl-C to skip DHCP search';
    $err_regex
        = qr/($err_msg1|$err_msg2|$err_msg3|$err_msg4|$err_msg5|$err_msg6|$err_msg7|$err_msg8|$err_msg9|$err_msg10|$err_msg11|$err_msg12|$err_msg13)/i;

    my $aggr = defined( $opts{aggr} ) ? $opts{aggr} : 'temp_aggr';
    my $vol  = defined( $opts{vol} )  ? $opts{vol}  : 'temp_root';
    my $vserver = defined( $opts{'vserver'} ) ? $opts{'vserver'} : $::VSERVER;
    my $aggr_size = defined( $opts{disks} ) ? $opts{disks} : $::AGGR_SIZE;
    my $vol_size
        = defined( $opts{vol_size} ) ? $opts{vol_size} : $::FLEX_SIZE;
    my $root_vol     = $pkg->_get_root_obj("volume");
    my $root_aggr    = $pkg->_get_root_obj("aggregate");
    my $context      = $opts{'context'} || $pkg->_get_default_context();
    my $conn_persist = $pkg->conn_console();
    my $mode         = $pkg->mode();
    my $apiset_obj   = $pkg->filer_api_console();
    if ( defined $::IRON_ON_ROOT && $mode eq 'CMode' ) {
        $vserver = 'vs0';
        $vol     = 'AUTOROOT';
    }
    if ( $::IRON_ON_ROOT =~ /multi|core/ ) {

        # If the aggr does not already exist - create it
        if ( !$pkg->_aggr_exists( $context, $aggr ) ) {

            # If the volume and aggr name are the same create a trad volume
            if ( $aggr eq $vol && $mode eq "7Mode" ) {
                eval {
                    $pkg->_vol_create(
                        'volume'     => $aggr,
                        'disk-count' => $aggr_size,
                        'root'       => 1
                    );
                };
                if ( $@ && $@ =~ /Couldn't create .* disks needed/ ) {
                    $pkg->_vol_rename(
                        'volume'          => $::FILER_VOL_TRAD,
                        'new-volume-name' => $aggr
                    );
                }
            }
            else {
                $pkg->_aggr_create(
                    'aggregate' => $aggr,
                    'aggr_size' => $aggr_size,
                );
            }
        }

        # If the volume does not already exist - create it
        # force guarantee=none since it will become a root volume
        # This should get all the system files copied over as well
        # In CMode, AUTOROOT will be used.
        if ( $vol ne 'AUTOROOT' ) {
            if ( !$pkg->_vol_exists( $mode, $vol ) ) {
                $pkg->_vol_create(
                    'vol'  => $vol,
                    'aggr' => $aggr,
                    'size' => $vol_size,
                    'root' => 1
                );
            }
            else {
                if ( !$vol_size == 0 ) {
                    ## @step Get space requirements for new root
                    my $req_rvol_size
                        = $pkg->_verify_vol_size( 'size' => $vol_size );
                    if ( $vol_size != $req_rvol_size ) {
                        $pkg->_vol_resize(
                            'vol'     => $vol,
                            'aggr'    => $aggr,
                            'vserver' => $vserver,
                            'size'    => $req_rvol_size
                        );
                    }
                }
            }
        }
        $pkg->_wafliron_pick_new_root(
            'aggr'   => $aggr,
            'vol'    => $vol,
            'reboot' => 1
        );

    }
    elsif ( $::IRON_ON_ROOT =~ /spare/ ) {

        # If the aggr already exist and it is the root volume already
        # just return, else destroy it before creating a new one
        my $aggr_options = $pkg->_aggr_exists( $context, $aggr );
        if ($aggr_options) {
            my $is_root_aggr = $aggr_options->[0]->{'options'}->[0]->{'root'};
            if ($is_root_aggr) {
                return $aggr;
            }
            $pkg->_aggr_purge($aggr);
        }

        # save filer license information, need to restore it afterwards

        my @i_licenses = ( keys %{ $pkg->_get_installed_licenses() } );

        my $disklist = $pkg->_get_disk_list();
        my @short_list = @{$disklist}[ 1, 2, 3 ];

        ##Going to boot menu for creation of temp root volume
        my $transit_to_obj = $pkg->transit_obj();

        $transit_to_obj->change_state( 'to' => 'MAINT' );

        # Offline the previous root aggr so we don't end up with two roots
        my $maint_api = $pkg->filer_api_maint();
        $maint_api->aggr_offline(
            'aggregate'              => $root_aggr,
            'connectrec-match_table' => [ "Are you sure want" => "yes" ]
        );
        $transit_to_obj->change_state( 'to' => 'BOOT_MENU' );
        $cmd = "create_temp_root $aggr -t raid0 @short_list";
        my $cold_start_filer = sub {
            $pkg->_logmsg(
                "Temporary trad vol creation successfull. Starting Setup",
                'comment' );

            # Do a warm start on the filer
            $pkg->cold_start_filer( newfs => 0 );
            $pkg->_set_flags(
                $flags => {
                    'wafl_inconsistent_panic_testing' => 1,
                    'wafl_inconsistent_panic'         => 0,
                    'wafl_iron_panic_on_corruption'   => 0
                }
            );

            # Re-License the filer
            $pkg->_license( \@i_licenses );
            $transit_to_obj->reboot()
                if ( grep( /syncmirror/, @i_licenses ) );

            # Root vol is now a traditional volume
            $vol = $aggr;
        };

        $report_err = sub {
            WaflironLibrary::Exceptions::WaflironLibException->throw(
                "\"create_temp_root\" failed!\nError: $1");
        };

        $pkg->_logmsg( "Creating temp root volume", "comment" );
        my $username = $apiset_obj->get_connection()->username();
        my $password = $apiset_obj->get_connection()->password();
        $pkg->_logmsg( "user name is : " . $username, 'comment' );
        $pkg->_logmsg( "Password is : " . $password,  'comment' );

        # Since 'additional_prompts' doesn't works, passing username and
        # password in match table to get the prompt
        $apiset_obj->execute_raw_command(
            'command'     => $cmd,
            'match_table' => [

                eval $err_regex => $report_err,
                eval $query1    => "yes",
                eval $query2    => "yes",
                eval $dhcp_str  => $cold_start_filer,
                "login:"             => sub { goto LABEL; },
                "Password:|password" => sub { goto LABEL; },
            ],

            #            'timeout' => LOCAL_WAFLIRON_TIMEOUT,
            'timeout' => 700,
        );
    }
    else {
        WaflironLibrary::Exceptions::WaflironLibException->throw(
            "Invalid IRON_ON_ROOT value. $::IRON_ON_ROOT");
    }
LABEL:
    $pkg->_logmsg( "out of the command!", 'comment' );
    $transit_to_obj->change_state( 'to' => 'CLI' );
    $pkg->recover_root_vol() if ( $mode eq 'CMode' );

}

sub _aggr_exists {
    my ( $pkg, $context, $aggregate ) = @_;
    my $api_set_obj = $pkg->_get_context_based_api($context);
    my $aggr_cmd
        = $context =~ /7Mode|Nodescope/i
        ? "aggr_status"
        : "storage_aggregate_show";
    my $exists;

    try {
        my $parsed_output
            = $api_set_obj->$aggr_cmd( 'aggregate' => $aggregate )
            ->get_parsed_output();
        $exists = $parsed_output;
    }
    catch NATE::BaseException with {
        my $eo   = shift;
        my $text = $eo->text();
        if ( $text
            =~ /No matching aggregates found|No aggregate named|There are no entries matching your query|\'$aggregate\' is a volume; use 'vol status/
            )
        {
            $exists = 0;
        }
    };
    return $exists;
}

sub _vol_exists {
    my ( $pkg, $context, $volume, $vserver ) = @_;
    my $api_set_obj = $pkg->_get_context_based_api($context);
    my $timeout = $pkg->cmd_timeout() || 90;
    my ( $vol_cmd, %vol_options, $exists );
    if ( $context =~ /7Mode|Nodescope/i ) {
        if ( !$volume ) {
            NATE::BaseException->throw(
                "volume is mandatory for volume search!");
        }
        $vol_cmd = "vol_status";
        %vol_options
            = ( 'volume' => $volume, 'connectrec-timeout' => $timeout );
    }
    elsif ( $context =~ /CMode/i ) {

        if ( !( $volume && $vserver ) ) {
            NATE::BaseException->throw(
                "vserver and volume are mandatory for volume search!");
        }
        $vol_cmd     = "volume_show";
        %vol_options = (
            'vserver'            => $vserver,
            'volume'             => $volume,
            'connectrec-timeout' => $timeout
        );
    }
    else {
        NATE::BaseException->throw("Invalid Context: $context");
    }

    try {
        $api_set_obj->$vol_cmd(%vol_options);
        $exists = 1;
    }
    catch NATE::BaseException with {
        my $eo   = shift;
        my $text = $eo->text();
        if ( $text
            =~ /No volume named|Invalid|not found|There are no entries matching your query/i
            )
        {
            $exists = 0;
        }
    };
    return $exists;
}

sub _wafliron_wait_wo_log_check {
    my ( $pkg, %opts ) = @_;
    my $timeout = $opts{'timeout'} || LOCAL_WAFLIRON_TIMEOUT;
    my $context = $opts{'context'} || $pkg->_get_default_context();
    my $endtime = time() + $timeout;
    while ( time() < $endtime ) {
        my $wi_status = $pkg->_get_wafliron_status($context);
        if ( $wi_status == 0 || $wi_status == 2 ) {
            return 1;
        }
        elsif ( $wi_status == 1 ) {
            sleep 45;
        }
    }
    WaflironLibrary::Exceptions::WaflironLibException->throw(
        "wafliron timed out!");
}

sub _basic_setup {
    my ( $pkg, %opts ) = @_;
    my $setup          = $opts{setup} || {};
    my $host_obj       = $opts{'host_obj'};
    my $timeout        = $opts{timeout} || 60;
    my $apiset_obj     = $pkg->filer_api_console();
    my %Cache_licenses = (
        netcache  => "GXVPYWA",
        nntp      => "WJSRXFF",
        mms       => "EKMAREM",
        real      => "YMWRXFF",
        quicktime => "MOWPYWA",
        icap      => "GFCJHSI",
        grm       => "WRQLGBN",
        ncagent   => "GJOLDQC",
        https     => "IQWRXFF",
        pnfs      => "SZNNCZG",
    );

    # Pull the parameters from various sources into a single hash:

    my %defaults = (
        GATEWAY                     => "",
        LOCATION                    => "",
        DNS                         => "",
        DNS_SERVER_LIST             => "",
        DNS_DOMAIN                  => "",
        DNS_DOMAINNAME              => "",
        NIS                         => "",
        NIS_SERVER_LIST             => "",
        NIS_DOMAIN                  => "",
        NIS_DOMAINNAME              => "",
        AUTOSUPPORT                 => "n",
        TIMEZONE                    => "",
        ADMIN_HOST                  => "",
        PRIMARY_CONTACT_NAME        => "",
        PRIMARY_CONTACT_PHONE       => "",
        PRIMARY_CONTACT_ALT_PHONE   => "",
        PRIMARY_CONTACT_EMAIL       => "",
        SECONDARY_CONTACT_NAME      => "",
        SECONDARY_CONTACT_PHONE     => "",
        SECONDARY_CONTACT_ALT_PHONE => "",
        SECONDARY_CONTACT_EMAIL     => "",
        BUSINESS_NAME               => "",
        BUSINESS_ADDR_STREET        => "",
        BUSINESS_ADDR_CITY          => "",
        BUSINESS_ADDR_STATE         => "",
        BUSINESS_ADDR_COUNTRY       => "",
        BUSINESS_ADDR_POSTALCODE    => "",
    );
    my %params = param_merge(
        hash         => \%defaults,
        hostrec      => $host_obj,
        param_prefix => $opts{prefix},
        hash         => $setup,
    );
    $pkg->_logmsg( "This is the param_merge output : \n" . Dumper( \%params ),
        "debug" );

    # Shuffle deprecated parameters into non-deprecated ones
    foreach my $key ( keys %params ) {
        my ($suffix) = ( $key =~ /^NETMASK(.*)/ );
        next if !defined($suffix);
        $params{"MASK$suffix"} ||= $params{"NETMASK$suffix"};
    }
    $params{DNS_DOMAIN} ||= $params{DNS_DOMAINNAME};
    $params{NIS_DOMAIN} ||= $params{NIS_DOMAINNAME};

    # Construct an array of DNS servers from DNS_SERVER_LIST

    my @dns_server_list = split( /,/, $params{DNS_SERVER_LIST} );
    my $dns = $params{DNS}
        || ( ( $params{DNS_DOMAIN} && @dns_server_list ) ? "y" : "n" );
    my $root_pass = $host_obj->default_password();
    if ( defined( $params{ROOT_PASS} ) ) {
        $root_pass = $params{ROOT_PASS};
    }

    # Construct comma separated list of NIS servers in NIS_SERVER_LIST
    # (remove extra spaces, as the filer does not allow them)

    my $nis_server_list = $params{NIS_SERVER_LIST};
    $nis_server_list =~ s/\s+//g;
    my $nis = $params{NIS}
        || ( ( $params{NIS_DOMAIN} && $nis_server_list ) ? "y" : "n" );

    my $cmd = ( $opts{issue_setup_cmd} ) ? "setup" : "";

    # Let's define a pattern to match the interface types that setup
    # might query us for.
    my $failure;
    my @prompts_setup = (
        "Are you sure you want to continue"         => "y",
        "Do you want to configure interface groups" => "n",
        "new hostname"                              => $params{HOSTNAME},
        "(e|E)nable IPv6\\?"                        => "n",
        "virtual network interfaces"                => "n",
        "IP address for (?:FC Network Interface|VI Network Interface|Network Interface|SPANS interface) (\\w+)\\W"
            => sub { $pkg->_get_ifc_info( \%params, $1, "IP" ) || ""; },
        "netmask for (VI )?Network Interface (\\w+)\\W" => sub {
            $pkg->_get_ifc_info( \%params, $2, "MASK" )
                || "255.255.255.0";
        },
        "Should interface (\\w+) take over " => sub {
            $pkg->_get_ifc_info( \%params, $1, "PARTNER" ) ? "y" : "n";
        },
        "Should adapter (\\w+) take over ELANs" => sub {
            $pkg->_get_ifc_info( \%params, $1, "PARTNER" ) ? "y" : "n";
        },
        "name to be taken over by (\\w+)\\W" =>
            sub { $pkg->_get_ifc_info( \%params, $1, "PARTNER" ); },
        "Please enter the ATM adapter to take over" =>
            sub { $pkg->_get_ifc_info( \%params, $1, "PARTNER" ); },
        "media type for (\\w+)\\W" => sub {
            $pkg->_get_ifc_info( \%params, $1, "MEDIATYPE" ) || "";
        },
        "flow control for (\\w+).*: " => sub {
            $pkg->_get_ifc_info( \%params, $1, "FLOWCONTROL" ) || "";
        },
        "want (\\w+) to support jumbo frames\\?" => sub {
            $pkg->_get_ifc_info( \%params, $1, "JUMBOFRAMES" ) || "";
        },
        "enter name of ELAN to add to ATM adapter" => "",
        "setup through the web interface"          => "n",
        "default gateway"                          => $params{GATEWAY},
        "IP address of the administration host"    => $params{ADMIN_HOST},
        "filer located"                            => $params{LOCATION},
        "enter timezone"                           => $params{TIMEZONE},
        "root directory for HTTP files"            => "",
        "DNS resolver"                             => $dns,
        "DNS domain name"                          => $params{DNS_DOMAIN},
        "IP address for [a-z]+ nameserver" =>
            sub { shift(@dns_server_list) || ""; },
        "another nameserver" => sub { @dns_server_list ? "y" : "n"; },
        "NIS client" => $nis,
        "NIS domain name"  => $params{NIS_DOMAIN},
        "NIS servers"      => $nis_server_list,
        "run autosupport"  => "n",
        "(N|n)ew password" => $root_pass
        ,    #this covers both 'New password' and 'Retype new password'
        "Password:" => $root_pass
        ,    #if running via console you get prompted to login with new pass
        "What language .* for multi-protocol files" => "",
        "Number of disks to add to root volume" => "0",   # (pre-redhook only)
        "Would you like to configure the RMC LAN interface" => "n",
        "Would you like to configure the RLM LAN interface" => "n",
        "Would you like to configure the BMC"               => "n",
        "Do you want to configure the Shelf Alternate Control Path Management interface for SAS shelves"
            => "n",

        #hitting ctrl-c here because basic setup should not configure CIFS
        #there are other libraries that exists to run the CIFS setp wizard
        #this prompt does not always appear, except in special cases
        "Ctrl-C to exit without committing changes" => "\cc",

# more prompts to add later
#"Would you like to enable DHCP on the RMC LAN interface [y]:"
#"Please enter media type for the RMC {100tx-fd, tp-fd, 100tx, tp, auto} [auto]:"
        "Errors during setup, no changes made" => sub {

            # don't die yet; there's a filer prompt that our caller
            # still has to consume.
            $failure
                = "setup aborted with \"Errors during setup, no changes made\"";
            return undef;
        },

        #console collisions make looking for "Press retrun key to continue"
        #impossible thus looking for smaller subset of the text
        "key to continue" => "",

        # (After change 200344.  It is letting us choose between
        # "netcache" and "pnfs"; we always choose netcache for now)
        "Please enter your license: " => $Cache_licenses{netcache},

        # (Before change 200344)
        "enter your NetCache license" => $Cache_licenses{netcache},

        # (end change 200344)
        "enter your NNTP license" => "",

        # (After change 284296)
        "WM \\(.*\\) protocol license" => "",

        # (Before change 284296)
        "MMS \\(.*\\) protocol license" => "",

        # (end change 284296)
        "MMS Pro license"          => "",
        "RTSP \\(\\S+\\) license"  => "",
        "ICAP license"             => "",
        "GRM Server license"       => "",
        "GRM Agent license"        => "",
        "HTTPS proxy license"      => "",
        "Telnet proxy license"     => "",
        "PNFS license"             => "",
        "SOCKS license"            => "",
        "HTTP Compression license" => "",

        # prompts required for IBM branded machines
        "Name of primary contact"         => $params{PRIMARY_CONTACT_NAME},
        "Phone number of primary contact" => $params{PRIMARY_CONTACT_PHONE},
        "Alternate phone number of primary contact" =>
            $params{PRIMARY_CONTACT_ALT_PHONE},
        "Primary Contact e-mail address or IBM WebID" =>
            $params{PRIMARY_CONTACT_EMAIL},
        "Name of secondary contact" => $params{SECONDARY_CONTACT_NAME},
        "Phone number of secondary contact" =>
            $params{SECONDARY_CONTACT_PHONE},
        "Alternate phone number of secondary contact" =>
            $params{SECONDARY_CONTACT_ALT_PHONE},
        "Secondary Contact e-mail address or IBM WebID" =>
            $params{SECONDARY_CONTACT_EMAIL},
        "Business name"               => $params{BUSINESS_NAME},
        "Business address"            => $params{BUSINESS_ADDR_STREET},
        "City where business resides" => $params{BUSINESS_ADDR_CITY},
        "State where business resides" =>
            $params{BUSINESS_ADDR_STATE},    # optional
        "2-character country code" => $params{BUSINESS_ADDR_COUNTRY},
        "Postal code where business resides" =>
            $params{BUSINESS_ADDR_POSTALCODE},    # optional
    );

    return eval {
        $apiset_obj->get_connection()->execute_expect(
            'match_table' => \@prompts_setup,
            'timeout'     => $timeout
        );
        if ($failure) {
            die "$failure\n";
        }
        $apiset_obj->api_reconnect_session();
        return 1;
    };

}

sub _set_security_defaults {
    my ( $pkg, $opts ) = @_;
    my $apiset_obj = $pkg->filer_api_console();
    $opts{"rsh.enable"}
        = defined $opts{"rsh.enable"} ? $opts{"rsh.enable"} : "on";
    $opts{"ssh.enable"}
        = defined $opts{"ssh.enable"} ? $opts{"ssh.enable"} : "on";
    $opts{"httpd.enable"}
        = defined $opts{"httpd.enable"} ? $opts{"httpd.enable"} : "on";
    $opts{"rsh.enable"}
        = defined $opts{"rsh.enable"} ? $opts{"rsh.enable"} : "on";
    $opts{"telnet.enable"}
        = defined $opts{"telnet.enable"} ? $opts{"telnet.enable"} : "on";

    #    $opts{"http.admin.enable"}
    #        = defined $opts{"http.admin.enable"}
    #        ? $opts{"http.admin.enable"}
    #        : "on";
    $opts{"security.passwd.rules.everyone"}
        = defined $opts{"security.passwd.rules.everyone"}
        ? $opts{"security.passwd.rules.everyone"}
        : "off";

    foreach my $option ( keys %opts ) {
        my $value = $opts{$option};
        $apiset_obj->options( $option => $value );
        my $parsed_output
            = $apiset_obj->options($option)->get_parsed_output();
        if ( $parsed_output->[0]->{$option} ne $value ) {
            $pkg->_logmsg( "Failed to set $option to $value", 'comment' );
        }
    }
}

sub _license {
    my ( $pkg, $service ) = @_;
    if ( $pkg->mode() eq "CMode" ) {
        $pkg->_logmsg( "_license() is not supported for CMode", "comment" );
        return 0;
    }
    my $apiset_obj = $pkg->filer_api_console();
    my %i_licenses;
    my $parsed_output = $apiset_obj->license()->get_parsed_output();
    foreach my $lcnse ( @{$parsed_output} ) {
        $i_licenses{ $lcnse->{'service'} } = $lcnse->{'is_licensed'};
    }
    my @services = ref($service) ? @$service : ($service);
    foreach my $s (@services) {
        my $licenses = $Admin::License::Default::DEFAULT_LICENSES{"filer"};
        if ( !$licenses->{$s} ) {
            $pkg->_logmsg( "License unavailable for this service : $s",
                "comment" );
            return undef;
        }
        next if ( $i_licenses{$s} eq "true" );
        $apiset_obj->license_add( 'code' => $licenses->{$s} );
    }
}

sub _get_ifc_info {
    my ( $pkg, $hash, $ifname, $field ) = @_;
    my $result = undef;
    $result = $hash->{ $field . '_' . $ifname };
    return $result if defined $result;
    if ( $field eq 'IP' || $field eq 'MASK' ) {
        while ( my ( $key, $value ) = each %$hash ) {
            if ( $key =~ /(CLUSTER|DATA|MGMT)_PORT(.*)/ && $ifname eq $value )
            {
                $result = $hash->{ $1 . '_' . $field . $2 };
                return $result;
            }
        }
    }
    return "";
}

sub _get_disk_list {
    my $pkg        = shift;
    my $mode       = $pkg->mode();
    my $context    = ( $mode eq "CMode" ) ? "Nodescope" : "7Mode";
    my $apiset_obj = $pkg->_get_context_based_api($context);
    my @ret_array;
    my $parsed_output = $apiset_obj->raid_config(
        'info'            => "listdisk",
        'privilege-level' => 'test'
    )->get_parsed_output();

    foreach my $href ( @{$parsed_output} ) {
        if ( $href->{'fsm_state'} =~ /ZEROED|ZEROING|NOTZEROED/ ) {
            push @ret_array, $href->{'name'};
        }
    }
    @ret_array = sort { $a <=> $b } @ret_array;
    return \@ret_array;
}
##############################################################################
# Exports the given path for NFS clients to use. (Works only on 7Mode)
##############################################################################

sub _export_path {
    my ( $pkg, %opts ) = @_;
    $pkg->_logmsg( "Arguemnts provided are : \n" . Dumper( \%opts ),
        'comment' );
    if ( $opts{license_nfs} ) {
        $pkg->_license('nfs');
    }
    my $context     = $pkg->mode() eq "CMode" ? "Nodescope" : "7Mode";
    my $api_set_obj = $pkg->_get_context_based_api($context);
    my $path        = $opts{'path'};
    my $options     = $param{'options'};

    my $optstring
        = ( $options ne "" ) ? $options : "sec=sys,rw,anon=0,nosuid";
    my $text = $api_set_obj->exportfs(
        'ignore'          => 1,
        'options'         => $optstring,
        'path'            => $path,
        'privilege-level' => 'test'
    )->get_raw_output();
    unless ( $text eq "" ) {
        $text =~ s/^exportfs:\s*(\S+)/$1/;
        die "$text\n";
    }

    if ( $opts{'WRITE_ETC'} ) {
        my @input = split( /\n/,
            $api_set_obj->rdfile( 'file' => '/etc/exports', )
                ->get_parsed_output()->[0]->{'contents'} );

        # Skip our path if it is in the file,
        # since we will overwrite it or remove it.
        my $pathexp = quotemeta($path);
        my @output  = grep( !/^$pathexp(\s|\z)/, @input );
        my $data    = join( "\n", @output ) . "\n";

        if ( defined $options ) {

            # Add in our path (possibly with no options).
            $optstring = ( $options ? "-" . $options : "" );
            $data .= sprintf( "%s\t%s\n", $path, $optstring );
        }

        # else, we just removed the path.
        # Write out the file.
        my $conn = $api_set_obj->get_connection();
        $api_set_obj->execute_raw_command(
            'command'                       => "wrfile /etc/exports",
            'connectrec-additional_prompts' => [""]
        );

        # send file contents as input to wrfile

        if ( $conn->conntype =~ /rsh-socket/ ) {
            $conn->send($data);
        }
        else {

            # For non-rsh-socket connections, one cannot send input
            # too quickly.  Also, non-rsh-socket connections echo back
            # the input, which we should gather anyway, so we can
            # match on the echo back of each line before we send the
            # next line.
            my (@lines) = split( /\n/, $data );
            foreach my $line (@lines) {
                my $linere = "(?:" . Tharn::string2regex($line) . ")";
                $linere =~ s/\t/\\s+/g;
                my $endtime = Tharn::timeout2time( $conn->timeout() );
                $conn->send( $line . "\n" );
                if (!defined(
                        $conn->expect(
                            'match' => [$linere],
                            timeout => $endtime
                        )
                    )
                    )
                {
                    die(      "Error waiting for echo back of line \n"
                            . "  \"$line\"\n"
                            . "(regular expression:\n"
                            . "  \"$linere\"\n"
                            . "): $@" );
                }
            }
        }

        # Finish wrfile command and wait for following prompt

        if ( $conn->conntype =~ /rsh/ ) {

            # over rsh, wrfile input is finished in a normal way by
            # closing input, and we know the command is finished if we
            # see output close (EOF).
            $conn->send_EOF();
            $conn->expect();
            if ( $@ !~ /EOF/ ) {
                die
                    "Error waiting for end of file after end of input to wrfile: $@";
            }
        }
        else {

            # over telnet or console there is no way to send end of
            # input, so wrfile must be finished by killing the running
            # wrfile command, which should result in a normal filer
            # prompt (which we are reponsible for consuming)
            $conn->send_intr();
            $conn->expect( match => [ $conn->prompts() ] )
                or die
                "Error waiting for prompt after end of input to wrfile: $@";
        }

    }

    return 1;
}

##############################################################################
# Copy system files from one volume to another
##############################################################################

sub _copy_system_files {
    my ( $pkg, %opts ) = @_;
    $pkg->_logmsg( "Arguemnts provided are : \n" . Dumper( \%opts ),
        'comment' );

    # timeout to complete entire copy for big root volumes
    my $timeout = $opts{timeout} ? $opts{timeout} : 900;
    my $client  = $opts{client}  ? $opts{client}  : $pkg->client();
    my $filer_ip
        = $opts{filer_ip}
        ? $opts{filer_ip}
        : $pkg->conn->{hostp}->{default_ip};
    my $filer = $pkg->hostname();
    my $svol  = $opts{'svol'};
    my $dvol  = $opts{'dvol'};

    if ( $client eq "" || $client eq 'localhost' ) {
        NATE::Exceptions::Argument->throw(
            "Client: localhost is not supported as it requires sudo password on some machines"
        );
    }

    $client_apiset_obj = NACL::APISet->new(
        name      => $client,
        category  => "Host",
        interface => "CLI",
        set       => 'Linux',
    );
    $pkg->_export_path(
        'path'      => "/vol/" . $svol,
        'options'   => "anon=0",
        'WRITE_ETC' => 1
    );
    $pkg->_export_path(
        'path'      => "/vol/" . $dvol,
        'options'   => "anon=0",
        'WRITE_ETC' => 1
    );

    $pkg->_logmsg( "Mounting source volume $svol of $filer", 'debug' );
    my $src_vol_path = $pkg->_mount_vol(
        vol_path   => '/vol/' . $svol,
        filer      => $filer,
        filer_ip   => $filer_ip,
        client     => $client,
        mountpoint => "/t/$filer/$svol"
    );

    $pkg->_logmsg( "Mounting source volume $svol of $filer", 'debug' );
    my $dst_vol_path = $pkg->_mount_vol(
        vol_path   => '/vol/' . $dvol,
        filer      => $filer,
        filer_ip   => $filer_ip,
        client     => $client,
        mountpoint => "/t/$filer/$dvol"
    );
    $pkg->_logmsg( "Copying system files from $src_vol_path to $dst_vol_path",
        'debug' );

    $client_apiset_obj->mkdir(
        'paths'           => "/t/$filer/$dvol" . '/etc',
        'make-parents'    => 1,
        'privilege-level' => 'root'
    );
    $client_apiset_obj->chmod(
        'paths'           => "/t/$filer/$dvol" . '/etc',
        'mode'            => '0777',
        'privilege-level' => 'root'
    );
    my $txt = $client_apiset_obj->cp(

        #        'recursive'       => 1,
        #        'force'           => 1,
        'source'             => "/t/$filer/$svol" . '/etc/*',
        'dest'               => "/t/$filer/$dvol" . '/etc',
        'privilege-level'    => 'root',
        'connectrec-timeout' => $timeout
    )->get_raw_output();

    if ( $out !~ /^\d+ blocks/ ) {
        $pkg->_umount_vol(
            'vol'        => $svol,
            'mount_path' => "/t/$filer/$svol"
        );
        $pkg->_umount_vol(
            'vol'        => $svol,
            'mount_path' => "/t/$filer/$dvol"
        );
    }

}

##############################################################################
# Mounts the volume on given client
##############################################################################

sub _mount_vol() {
    my ( $pkg, %opts ) = @_;
    my $vol_path   = $opts{vol_path};
    my $mountpoint = $opts{mountpoint};
    my $client     = $opts{client} || $main::CLIENT || 'localhost';
    my $mountopts  = $opts{mountopts}

        #      || $main::MOUNTOPTS
        || 'actimeo=0,noac,hard,intr';
    my $timeout = $opts{timeout} || $main::TIMEOUT || 120;
    my $chmod = defined( $opts{chmod} ) ? $opts{CHMOD} : '0777';
    my $filer_ip = $opts{'filer_ip'}
        || $pkg->conn->hostp->{'net'}->{'address'};
    my $source_path = $filer_ip . ':' . $vol_path;

  # if the client is a localhost, in some systems it asks for sudo password to
  #  run mount command hence localhost can't be used as client

    if ( $client eq "" || $client eq 'localhost' ) {
        NATE::Exceptions::Argument->throw(
            "Client: localhost is not supported as it requires sudo password on some machines"
        );
    }
    my $client_apiset_obj = NACL::APISet->new(
        'name'    => $client,
        category  => "Host",
        interface => "CLI",
        set       => "Linux"
    );

    # create directory
    $client_apiset_obj->mkdir(
        'paths'              => $mountpoint,
        'make-parents'       => 1,
        'connectrec-timeout' => $timeout,
        'privilege-level'    => 'root'
    );

    $client_apiset_obj->chmod(
        'paths'           => $mountpoint,
        'mode'            => '0777',
        'privilege-level' => 'root'
    );

    $client_apiset_obj->mount(
        'mount-point'        => $mountpoint,
        'source-path'        => $source_path,
        'options'            => $mountopts,
        'privilege-level'    => 'root',
        'connectrec-timeout' => $timeout
    );
    return $vol_path;
}

##############################################################################
# Unmounts the volume on given client
##############################################################################

sub _umount_vol() {
    my ( $pkg, %opts ) = @_;
    my $client     = $opts{client} || $main::CLIENT || 'localhost';
    my $vol        = $opts{'vol'};
    my $mount_path = $opts{'mount_path'};
    my $timeout    = $opts{timeout} || $main::TIMEOUT || 120;

  # if the client is a localhost, in some systems it asks for sudo password to
  #  run mount command hence localhost can't be used as client

    if ( $client eq "" || $client eq 'localhost' ) {
        NATE::Exceptions::Argument->throw(
            "Client: localhost is not supported as it requires sudo password on some machines"
        );
    }
    if ( !( $vol && $mount_path ) ) {
        NATE::Exceptions::Argument->throw(
            "Both volume and mount path must be provided for umount");
    }

    # Create a client APISet object
    my $client_apiset_obj = NACL::APISet->new(
        'name'    => $client,
        category  => "Host",
        interface => "CLI",
        set       => "Linux"
    );

    # If the volume is not mounted, return 1
    my $mounted_vols
        = $client_apiset_obj->mount( 'list' => 1, )->get_parsed_output();
    foreach my $mount_info ( @{$mounted_vols} ) {
        if ( !$mount_info->{'source_path'} =~ /$vol/s ) {
            return 1;
        }
    }

    # umount the mounted volume
    $client_apiset_obj->umount(
        'paths'              => $mount_path,
        'force'              => 1,
        'lazy'               => 1,
        'connectrec-timeout' => $timeout,
        'privilege-level'    => 'root',
        'connectrec-timeout' => $timeout
    );
}

##############################################################################
# Returns the default scan speed on the release running on filer
##############################################################################

sub _get_default_scan_speed {
    my $pkg = shift;
    $Log->enter();
    my $priv_level = 'test';
    my $context    = $pkg->mode() eq "CMode" ? "Nodescope" : "7Mode";
    my $apiset_obj = $pkg->_get_context_based_api($context);

    # Store the original wafl scan speed of the filer
    my $resp_obj
        = $apiset_obj->wafl_scan_speed( 'privilege-level' => $priv_level );
    my $original_scanspeed
        = $resp_obj->get_parsed_output()->[0]->{'wafl_scan_speed'};

# Set the wafl scan speed to zero and get the scan speed which is default scan speed
    $apiset_obj->wafl_scan_speed(
        'speed'           => 0,
        'privilege-level' => $priv_level
    );
    $resp_obj
        = $apiset_obj->wafl_scan_speed( 'privilege-level' => $priv_level );
    $default_scanspeed
        = $resp_obj->get_parsed_output()->[0]->{'wafl_scan_speed'};

    # Set the original scan speed back
    $apiset_obj->wafl_scan_speed(
        'speed'           => $original_scanspeed,
        'privilege-level' => $priv_level
    );

    $Log->exit();
    return $default_scanspeed;
}

##############################################################################
# picks a new root aggr and volume on 7Mode
##############################################################################
sub _wafliron_pick_new_root_7m {
    my ( $pkg, %opts ) = @_;

    $Log->enter();
    $pkg->_logmsg( "Arguements are : \n" . Dumper( \%opts ), 'debug' );

    # declare variables
    my $reboot           = $opts{'reboot'};
    my $scan_speed       = $opts{'scan_speed'};
    my $powercycle       = $opts{'powercycle'};
    my $already_halted   = $opts{'already_halted'};
    my $type             = $opts{'type'};
    my $corrupted_vol    = $opts{'corrupted_vol'};
    my $delete_all_snaps = $opts{'delete_all_snaps'};
    my $aggr             = $opts{'aggr'};
    my $vol              = $opts{'vol'};
    my ( $root_aggr, $panic_string );
    my $d_scanspeed = $pkg->default_scan_speed();

    # Get the transit_to object
    my $transit_obj = $pkg->transit_obj();
    my $api_maint   = $pkg->filer_api_maint();
    my $apiset_obj  = $pkg->_get_context_based_api('7Mode');

    if ( !$already_halted ) {

        # make sure the filer is in the CLI prompt
        $pkg->_wi_login();

        $boot_state = $transit_obj->get_state( 'timeout' => 300 );
        if ( $boot_state eq 'CLI' ) {
            $root_aggr = $pkg->_get_root_obj('aggregate');

    # Corrupted volume will be restricted/not visible(when aggr is restricted)
    # Hence this check not required(volume will not be visible).
    # This check will be avoided only in case corrupted_vol options is set

            if ( ( !$corrupted_vol && $vol ne 'AUTOROOT' ) ) {
                $pkg->_logmsg( "Checking corrupted volume", 'comment' );
                $@ = undef;
                eval { $apiset_obj->vol_options( 'volume' => $vol ); };
                if ( $@ =~ /No volume named .* exists/is ) {
                    $pkg->_logmsg( "Intended root volume $vol does not exist",
                        'debug' );
                    $vol = ( $aggr eq 'aggr0' ) ? 'vol0' : 'vol0_temp';
                    $pkg->_logmsg( "Using $vol as root volume!", 'debug' );
                }
            }
        }
        $pkg->_logmsg( "Checking root aggr", 'debug' );
        if ( $root_aggr ne $aggr ) {

            # aggr is not already the root volume so we have to
            # get into maint mode and make it root
            $transit_obj->change_state(
                'to'         => 'MAINT',
                'powercycle' => $powercycle
            );
            my $aggr_state = $api_maint->aggr_status( aggregate => $aggr )
                ->get_parsed_output()->[0]->{'state'};
            if ( $aggr_state ne 'online' ) {
                $text = $api_maint->aggr_online(
                    'aggregate'          => $aggr,
                    'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
                )->get_raw_output();
                $panic_string
                    = "Filer panicked - returning before setting aggr as new root";
                if ( $text =~ /Panic/i ) {
                    $pkg->_logmsg( $panic_string, 'debug' );
                    return ($text);
                }
            }

            $text = $api_maint->aggr_options(
                'aggregate'          => $aggr,
                'option-name'        => 'root',
                'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
            )->get_raw_output();

            if ( $text =~ /Panic/i ) {
                $panic_string
                    = "Filer panicked - returning before vol_pick_root";
                $pkg->_logmsg( $panic_string, 'debug' );
                return $text;
            }
        }

        # Set the required bootargs and reboot to the boot menu
        $pkg->_logmsg(
            "Calling transit_to BOOT_MENU -- intended root vol is: $vol",
            'debug' );
        $transit_obj->change_state( 'to' => 'FIRMWARE' );
    }

  # set scanner speed, of course, all we have is slow or not slow. If the user
  # wants to set the scan speed to less then 1000 then we will make it slow,
  # otherwise we will leave it at not slow

    my $api_firmware = $pkg->filer_api_firmware();
    if ( $scan_speed == -1 || $scan_speed < $d_scanspeed ) {
        $api_firmware->setenv(
            'varname'            => 'wafl-scanners-slow?',
            'value'              => 1,
            'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
        );
    }
    else {
        $api_firmware->setenv(
            'varname'            => 'wafl-scanners-slow?',
            'value'              => 0,
            'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
        );
    }

    # Must set do not panic flag to successfully boot up
    $api_firmware->setenv(
        'varname'            => 'wafl-iron-panic-on-corruption?',
        'value'              => 0,
        'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
    );
    $transit_obj->change_state( 'to' => 'BOOT_MENU' );

    # No need to do a vol_pick_root if the root is a traditional volume
    my $api_console = $pkg->filer_api_console();
    my $dhcp_str    = 'Ctrl-C to skip DHCP search';

    if ( $vol eq 'AUTOROOT' ) {
        $pkg->_logmsg( "In the autoroot if construct...", 'comment' );

        my $dhcp_str_found = 0;

     # Send '1' for normal boot, without running vol_pick_root to use AUTOROOT
        $cmd = '1';
        $api_console->execute_raw_command(
            'command'                       => $cmd,
            'connectrec-timeout'            => LOCAL_WAFLIRON_TIMEOUT,
            'connectrec-additional_prompts' => [$dhcp_str]
        );

        $dhcp_str_found = 1;
        $pkg->_logmsg( "\$dhcp_str_found: $dhcp_str_found...", 'comment' );

        if ($dhcp_str_found) {
            $pkg->_logmsg( "Starting Setup...", 'comment' );

            # Do a warm start on the filer
            $pkg->cold_start_filer( newfs => 0 );
        }
    }

    if ( $type ne 'trad' && $vol ne "AUTOROOT" ) {
        $cmd = "vol_pick_root $vol $aggr";
        $@   = undef;
        eval {
            $api_console->execute_raw_command(
                'command' => $cmd,
                'connectrec-additional_prompts' =>
                    ['Please choose one of the following'],
                'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
            );
        };

        # If we had a timeout error try the command again
        if ( $@ && $@ =~ /IDLE|TIMEOUT/ ) {
            $api_console->execute_raw_command(
                'command' => $cmd,
                'connectrec-additional_prompts' =>
                    ['Please choose one of the following'],
                'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
            );
        }
        my $cr = $pkg->conn_console();
        $cr->execute_start( 'command' => '1' );
        $pkg->_logmsg( "Starting Setup...", 'comment' );
        $pkg->cold_start_filer( newfs => 0 );

    }
    if ($delete_all_snaps) {
        $pkg->_logmsg( "Must delete snapshots", 'debug' );
        $pkg->_delete_all_snaps();
    }
    $transit_obj->change_state( 'to' => 'CLI' );
    if ($reboot) {
        $transit_obj->reboot();
        $pkg->_check_connection_state();
    }
    $Log->exit();
    return 1;
}

sub _wafliron_pick_new_root_cm {
    my ( $pkg, %opts ) = @_;

    $Log->enter();
    $pkg->_logmsg( "Arguements are : \n" . Dumper( \%opts ), 'debug' );

    # declare variables
    my $reboot         = $opts{'reboot'};
    my $scan_speed     = $opts{'scan_speed'};
    my $powercycle     = $opts{'powercycle'};
    my $already_halted = $opts{'already_halted'};
    my $type           = $opts{'type'};
    my $aggr           = $opts{'aggr'};
    my ( $root_aggr, $panic_string, $ha_policy );
    my $d_scanspeed = $pkg->default_scan_speed();

    # Get the transit_to object
    my $transit_obj = $pkg->transit_obj();
    my $api_maint   = $pkg->filer_api_maint();
    my $apiset_obj  = $pkg->_get_context_based_api('CMode');

    if ( !$already_halted ) {

        # make sure the filer is in the CLI prompt
        $pkg->_wi_login();

        $boot_state = $transit_obj->get_state( 'timeout' => 300 );
        if ( $boot_state ne 'CLI' ) {
            $transit_obj->change_state(
                'to'         => 'CLI',
                'powercycle' => $powercycle
            );
        }
        $root_aggr = $pkg->_get_root_obj('aggregate');
        $ha_policy
            = $apiset_obj->storage_aggregate_show( 'aggregate' => $aggr )
            ->get_parsed_output->[0]->{'ha-policy'};
        if ( $ha_policy ne 'cfo' ) {
            $apiset_obj->storage_aggregate_modify(
                'aggregate' => $aggr,
                'ha-policy' => 'cfo'
            );
        }

        $pkg->_logmsg( "Checking root aggr", 'debug' );
        if ( $root_aggr ne $aggr ) {

            # aggr is not already the root volume so we have to
            # get into maint mode and make it root
            $transit_obj->change_state(
                'to'         => 'MAINT',
                'powercycle' => $powercycle
            );
            my $aggr_state = $api_maint->aggr_status( aggregate => $aggr )
                ->get_parsed_output()->[0]->{'state'};
            if ( $aggr_state ne 'online' ) {
                $text = $api_maint->aggr_online(
                    'aggregate'          => $aggr,
                    'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
                )->get_raw_output();
                $panic_string
                    = "Filer panicked - returning before setting aggr as new root";
                if ( $text =~ /Panic/i ) {
                    $pkg->_logmsg( $panic_string, 'debug' );
                    return ($text);
                }
            }

            $text = $api_maint->aggr_options(
                'aggregate'   => $aggr,
                'option-name' => 'root',
            )->get_raw_output();

            if ( $text =~ /Panic/i ) {
                $panic_string
                    = "Filer panicked - returning before vol_pick_root";
                $pkg->_logmsg( $panic_string, 'debug' );
                return $text;
            }
        }

        # Set the required bootargs and reboot to the boot menu
        $pkg->_logmsg(
            "Calling transit_to BOOT_MENU -- intended root vol is: $vol",
            'debug' );
        $transit_obj->change_state( 'to' => 'FIRMWARE' );
    }

  # set scanner speed, of course, all we have is slow or not slow. If the user
  # wants to set the scan speed to less then 1000 then we will make it slow,
  # otherwise we will leave it at not slow

    my $api_firmware = $pkg->filer_api_firmware();
    if ( $scan_speed == -1 || $scan_speed < $d_scanspeed ) {
        $api_firmware->setenv(
            'varname'            => 'wafl-scanners-slow?',
            'value'              => 1,
            'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
        );
    }
    else {
        $api_firmware->setenv(
            'varname'            => 'wafl-scanners-slow?',
            'value'              => 0,
            'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
        );
    }

    # Must set do not panic flag to successfully boot up
    $api_firmware->setenv(
        'varname'            => 'wafl-iron-panic-on-corruption?',
        'value'              => 0,
        'connectrec-timeout' => LOCAL_WAFLIRON_TIMEOUT
    );

    #    $pkg->recover_root_vol();

    $transit_obj->change_state( 'to' => 'CLI' );
    $pkg->_check_connection_state();
    $Log->exit();
}

###############################################################################
# Function: _vol_type
# Purpose: Returns a the type of a given volume
#
###############################################################################

sub _vol_type {
    my ( $pkg, $vol_to_check ) = @_;
    my $apiset = $pkg->command_interface()
        ->get_7m_or_nodescope_apiset( interface => 'CLI' );
    my $vols = $apiset->vol_status()->get_parsed_output();
    foreach my $vol ( @{$vols} ) {
        if ( $vol->{'volume'} eq $vol_to_check ) {
            my $status = $vol->{'status'};
            return ( split /\,\s*/, $status )[1];
        }
    }
}
###############################################################################
# Function: _get_installed_licenses
# Purpose: Returns a reference to a hash containing feature as key and license
#          code as value.
#          It works on both CMode and 7Mode
###############################################################################
sub _get_installed_licenses {
    my ( $pkg, $service ) = @_;
    my $mode       = $pkg->mode();
    my $apiset_obj = $pkg->_get_context_based_api($mode);

    my %i_licenses;
    if ( $mode eq "CMode" ) {
        my $parsed_output
            = $apiset_obj->system_license_show()->get_parsed_output();
        foreach my $license ( @{$parsed_output} ) {
            $i_licenses{ $license->{'feature'} }
                = $license->{'serial-number'};
        }
    }
    elsif ( $mode eq '7Mode' ) {
        my $parsed_output = $apiset_obj->license()->get_parsed_output();
        foreach my $license ( @{$parsed_output} ) {
            next if ( $license->{'is_licensed'} eq 'false' );
            $i_licenses{ $license->{'service'} } = $license->{'license_code'};
        }
    }
    else {
        WaflironLibrary::Exceptions::WaflironLibException->throw(
            "Invalid context: " . $mode );
    }

    return \%i_licenses;
}
1;
