# $Id: //depot/prod/test/main/lib/WinClient.pm#7 $
#
# This library is used to create the Windows Host objects
#
# This is THARN Doc
#
## @summary
##  Wrapper for WinClient host functionality.
##
## @author  kenny speer
## @status  private
##
#
# TODO: Complete documentation pending.
#

use strict;

package WinClient;

# define the subroutines in this package
sub new;

# import object libs
use WinClient::Host;

# import common libs
use TestEnv::MinWin;
use Error;

use vars qw( @ISA @EXPORT );
@ISA = qw( Error );

#
#
#
sub new
{
    my ($class, %args) = @_;

    my $this = {};

    my ($msg, $connection, $timeout, $protocol, $multipath, $pnate_environment, $obj_template);

    # bless and initialize object
    bless $this, $class;

    # clear any errors to begin
    $this->_clear_err();

    # general host properties
    $this->{connection}        = undef;
    $this->{hosttype}          = undef;
    $this->{hostname}          = undef;
    $this->{timeout}           = undef;
    $this->{protocol}          = undef;
    $this->{useSD}             = undef;
    $this->{multipath}         = undef;
    $this->{pnate_environment} = 0;
    $this->{hypervisor_type}   = undef;

    # check the validity of params
    foreach (keys %args)
    {
        if (/^-?connection/i)
        {
            if (!length($connection = $args{$_}))
            {
                $msg = "WinClient::new invalid value for -connection: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                return $this;
            }
            next;
        }

        if (/^-?timeout/i)
        {
            if (!length($timeout = $args{$_}))
            {
                $msg = "WinClient::new invalid value for -timeout: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                return $this;
            }
            else
            {
                $this->{timeout} = $timeout;
            }
            next;
        }

        if (/^-?protocol/i)
        {
	    &main::logcomment($protocol);
            if (!length($protocol = $args{$_}) || $protocol !~ /fcp|iscsi|nfs|cifs/i)
            {
                $msg = "WinClient::new invalid value for -protocol: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                return $this;
            }
            else
            {
                $this->{protocol} = $protocol;
            }
            next;
        }

        if (/^-?hypervisor_type/i)
        {
            $this->{hypervisor_type} = $args{$_};
            next;
        }

        if (/^-?multipath/i)
        {
            if (!length($multipath = $args{$_}))
            {
                $msg = "WinClient::new invalid value for -multipath: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                return $this;
            }
            else
            {
                $this->{multipath} = $multipath;
            }
            next;
        }

        if (/^-?pnate_environment/i)
        {
            if (!length($pnate_environment = $args{$_}))
            {
                $msg = "WinClient::new invalid value for -pnate_environment: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                return $this;
            }
            else
            {
                $this->{pnate_environment} = $pnate_environment;
            }
            next;
        }

        if (/^-?obj_template/i)
        {
            if (!length($obj_template = $args{$_}))
            {
                $msg = "WinClient::new invalid value for -obj_template: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                return $this;
            }
            next;
        }
    }

    # if an object templace was specified, use that
    if (defined($obj_template)) {
        return WinClient::Host->new(-self => $this,
                                    -obj_template => $obj_template);
    }

    # we must have a connection, else this isn't going to be nice
    if (!defined($connection))
    {
        $this->_set_err("WinClient::new -connection not defined");
    }
    else
    {
        $this->{connection} = $connection;
        $this->{hosttype} = $connection->{hostp}->{hosttype};
        $this->{hostname} = $connection->{hostp}->{hostname};
    }

    # this library only supports windows
    if ($this->{hosttype} !~ /windows/i)
    {
        $this->_set_err("WinClient::new {hosttype} is not Windows");
    }

    # if user didn't specify the timeout, default to 300 sec
    $this->{timeout} = 300 if (!defined($timeout));

    # user must specify the protocol used
    # TODO: support multiple protocols
    $this->_set_err("WinClient::new -protocol not defined") if (!defined($protocol));

    # it's safest to assume we have mpio installed
    $this->{multipath} = 1 if (!defined($multipath));

    # define this trigger to enable sdcli
    $this->{useSD} = $args{"-useSD"};

    $this = WinClient::Host->new(-self => $this);

    return $this;

} # END new

1;
