# $Id: //depot/prod/test/Rsierranevada/lib/WinClient/DT.pm#1 $
#
# Copyright (c) 2006 Network Appliance, Inc.
# All rights reserved.
#
# This library generates IO for a specific disk.  These methods are
# designed to be called via a disk object method and not directly from a
# thpl.
#
# TODO: Complete documentation pending.
#
## @summary
##  Wrapper for WinClient Disk IO functionality.
##
## @author  kenny speer
## @status  private
##

use strict;

package WinClient::DT;

# define the subroutines used in this package
sub _startDT;

use lib();

#
#
#
sub _startDT
{
    my ($this, %args) = @_;

    my ($status, @output);
    my ($time, @randBlockSizes, @randFileSizes, $randCount, $dtCmd);
    my ($location, $ioType, $msg, $runLength, $blockSize, $fileSize, $threads);

    foreach (keys %args)
    {
        if (/^-location$/i)
        {
            if (!length($location = $args{$_}))
            {
                $msg = "DT::startDT - invalid value for -location: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startDT_EXIT;
            }
            next;
        }
        if (/^-iotype$/i)
        {
            if (!length($ioType = $args{$_}))
            {
                $msg = "DT::startDT - invalid value for -iotype: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startDT_EXIT;
            }
            next;
        }
        if (/^-runlength$/i)
        {
            if (!length($runLength = $args{$_}))
            {
                $msg = "DT::startDT - invalid value for -runlength: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startDT_EXIT;
            }
            next;
        }
        if (/^-blocksize$/i)
        {
            if (!length($blockSize = $args{$_}))
            {
                $msg = "DT::startDT - invalid value for -blocksize: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startDT_EXIT;
            }
            next;
        }
        if (/^-filesize$/i)
        {
            if (!length($fileSize = $args{$_}))
            {
                $msg = "DT::startDT - invalid value for -filesize: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startDT_EXIT;
            }
            next;
        }
        if (/^-threads$/i)
        {
            if (!length($threads = $args{$_}))
            {
                $msg = "DT::startDT - invalid value for -threads: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startDT_EXIT;
            }
            next;
        }
    }

    # adjust the time for seconds so we know how long to wait
    $time = $runLength;
    if ($runLength =~ /(s|h|m|d)/i)
    {
        $time =~ s/(s|h|m|d)//ig;
        $time = $time * 60 if ($runLength =~ /m/i);
        $time = $time * 60 * 60 if ($runLength =~ /h/i);
        $time = $time * 60 * 60 * 24 if ($runLength =~ /d/i);
    }

    # fix runlength before munging time
    $runLength = $time;

    # add 10 min to time for say() timeout
    $time += 900;
    
    # define random block and file sizes
    @randBlockSizes = ("1k", "2k", "4k", "8k", "16k", "29k",
                       "32k", "64k", "128k", "201k", "256k");
    @randFileSizes = ("100m", "250m", "500m", "750m", "1g",
                      "1250m", "1500m", "1750m", "2g");

    # initialize random number generator
    srand(time ^ $$ * $randCount++);

    # setup the block and file sizes
    if ($blockSize =~ /^random$/i)
    {
        $blockSize = $randBlockSizes[int rand scalar(@randBlockSizes)];
        &main::logcomment("DT::startDT - Using a random ($blockSize) block size.");
    }

    if ($fileSize =~ /^random$/i)
    {
        $fileSize = $randFileSizes[int rand scalar(@randBlockSizes)];
        &main::logcomment("DT::startDT - Using a random ($fileSize) file size.");
    }

    # build the command
    $dtCmd = "$this->{host}->{cmd_dt} of=$location limit=$fileSize bs=$blockSize " .
             "procs=$threads oncerr=abort disable=pstats errors=1 " .
             "enable=compare runtime=$runLength ";

    if ($ioType =~ /^raw$/i)
    {
        $dtCmd = $dtCmd . "rlimit=$fileSize iotype=random ";
    }

    &main::logcomment("DT::startDT - Executing command: $dtCmd");

    # execute the command
    ($status, @output) = $this->execute($dtCmd, $time);

    if ($status != $Host::TRUE)
    {
        &main::logcomment("DT::startDT - Command failed: $dtCmd");
        $this->_set_err(@output);
        goto startDT_EXIT;
    }

    # look for some DT specific errors
    if (grep(/Total errors detected:/i, @output))
    {
        if (!grep(/Total errors detected: 0/i, @output))
        {
            $msg = "DT::startDT - IO ERRORS were detected during IO run.";
            &main::logcomment($msg);
            $this->_set_err($msg);
            goto startDT_EXIT;
        }
    }
    elsif (grep(/Total Statistics/i, @output))
    {
        &main::logcomment("DT::startDT - Completed successfully.");
    }
    else
    {
        $msg = "DT::startDT - Command did not complete: $dtCmd";
        &main::logcomment($msg);
        $this->_set_err($msg);
        goto startDT_EXIT;
    }
        
startDT_EXIT:

    return;

} # END _startDT

1;
