# $Id: //depot/prod/test/Rsierranevada/lib/WinClient/Install.pm#1 $
#
# Copyright (c) 2006 Network Appliance, Inc.
# All rights reserved.
#
# TODO: Complete documentation pending.
#
## @summary
##  Wrapper for WinClient host functionality.
##
## @author  kenny speer
## @status  private
##

use strict;

package WinClient::Install;

# define the subroutines used in this package
sub _installMPIO;
sub _installSD;

use lib();

#
#
#
sub _installMPIO
{
    my ($this, %args) = @_;

    my ($installType, $buildType, $build, $vendor, $installEXE);
    my ($arch, $dstDir, $srcDir, $msg, $retryCount, $chkCmd);
    my ($status, $line, @output, $cmd, $srcFile, $dstFile);

    # initially clear errors
    $this->_clear_err();

    foreach (keys %args)
    {
        if (/^-installType$/i)
        {
            if (!length($installType = $args{$_}) || $installType !~ /(?:dev|full|oinst)/i)
            {
                $msg = "Install::_installMPIO - invalid value for -installType: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto _installMPIO_EXIT;
            }
            next;
        }
        if (/^-buildType$/i)
        {
            if (!length($buildType = $args{$_}) || $buildType !~ /(?:chk|fre)/)
            {
                $msg = "Install::_installMPIO - invalid value for -buildType: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto _installMPIO_EXIT;
            }
            next;
        }
        if (/^-build$/i)
        {
            if (!length($build = $args{$_}))
            {
                $msg = "Install::_installMPIO - invalid value for -build: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto _installMPIO_EXIT;
            }
            next;
        }
        if (/^-vendor$/i)
        {
            if (!length($vendor = $args{$_}) || $vendor !~ /(?:IBM|NetApp)/)
            {
                $msg = "Install::_installMPIO - invalid value for -vendor: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto _installMPIO_EXIT;
            }
            next;
        }
    }

    # build date is required, build
    if (!defined($build))
    {
        $msg = "Install::_installMPIO - invalid value for -build: $build" .
            "\n-build is required and must be a valid build id";
        &main::logcomment($msg);
        $this->_set_err($msg);
        goto _installMPIO_EXIT;
    }

    # set the build to nightly checked dev install if not defined
    $installType = "full" if (!defined($installType));
    $vendor = "NetApp" if (!defined($vendor));
    $arch = $this->{arch};
    
    # set buildType, special case for windows 2000
    $buildType = "fre" if (!defined($buildType));
    $buildType .= "W2K" if ($this->{hostver} =~ /windows 2000/);

    # set the destination directory
    $dstDir = "/x/eng/rtpwaktmp/installs/$this->{hostname}/";

    # set the source directory
    $srcDir = "/x/eng/rlse/$build/bin/$vendor/$buildType/$arch/*";

    # now rm anything in the temp dir and copy (rsync) the files over
    system("r rm -rf $dstDir");

    # retry three times to rsync the build over
    $retryCount = 3;
    while($retryCount)
    {
        &main::logcomment("syncing\nsrcDir: $srcDir\ndstDir: $dstDir");
        system("rsync -av $srcDir $dstDir");
        last if ($? == 0);
        $retryCount--;
    }
    
    if ($? != $Host::TRUE)
    {
        $msg = "Install::_installMPIO - Command failed: $!";
        &main::logcomment($msg);
        $this->_set_err($msg);
        goto _installMPIO_EXIT;
    }

    # this needs to be done for all installs
    # let's clear the mount point, if it's assigned to a drive, we need
    # to fail, otherwise umount the share
    $this->execute("net use z: /d", 60);

    # mount \\lethe\cores\wak_install_temp
    $cmd = "net use z: \\\\agate.rtp.netapp.com\\rtpwaktmp\\installs\\" .
            $this->{hostname}  . " /u:agate\\minwin qa4testing /persistent:NO";

    ($status, @output) = $this->execute($cmd, $this->{timeout});

    if (!grep(/The command completed successfully./, @output) || $status != $Host::TRUE)
    {
        &main::logcomment("Host::_installMPIO - Command failed: $cmd");
        $this->_set_err(@output);
        goto _installMPIO_EXIT;
    }

    # assuming we made it here, we need to do the install after an
    # uninstall, so do that first, reboot, then remount the share and
    # proceed with the install
    if ($installType =~ /oinst/i)
    {
        $cmd = "z: & cd raw & oinstall.exe -u ";
        
        ($status, @output) = $this->execute($cmd, 3600);

        if (!grep(/(?:finished|pending reboot|should reboot)/, @output) ||
            $status != $Host::TRUE)
        {
            &main::logcomment("Install::_installMPIO - Command failed: $cmd");
            $this->_set_err(@output);
            goto _installMPIO_EXIT;
        }

        $this->reboot();
        goto _installMPIO_EXIT if ($this->error());

        # mount \\lethe\cores\wak_install_temp
        $cmd = "net use z: \\\\agate.rtp.netapp.com\\rtpwaktmp\\installs\\" .
                $this->{hostname}  . " /u:agate\\minwin qa4testing /persistent:NO";

        ($status, @output) = $this->execute($cmd, $this->{timeout});

        if (!grep(/The command completed successfully./, @output) ||
            $status != $Host::TRUE)
        {
            &main::logcomment("Host::_installMPIO - Command failed: $cmd");
            $this->_set_err(@output);
            goto _installMPIO_EXIT;
        }

        $cmd = "z: & cd raw & oinstall.exe -i";

        ($status, @output) = $this->execute($cmd, 3600);

        if (!grep(/(?:finished|should reboot|pending reboot)/, @output) || 
            $status != $Host::TRUE)
        {
            &main::logcomment("Install::_installMPIO - Command failed: $cmd");
            $this->_set_err(@output);
            goto _installMPIO_EXIT;
        }
    }
    elsif ($installType =~ /dev/i)
    {
        $srcFile = "z:\\raw\\ontapdsm.sys";
        $dstFile = "$Host::SYSROOT\\system32\\drivers\\ontapdsm.sys";

        # first we need to rename the orig file
        $this->deleteRemoteFile($dstFile . "orig");
        $cmd = "rename $dstFile ontapdsm.sys.orig";
        $this->execute($cmd);

        # then we copy the new file
        ($status, @output) = $this->copyRemoteFile($srcFile, $dstFile);

        if ($status != $Host::TRUE)
        {
            &main::logcomment("Install::_installMPIO - Copy of $srcFile failed: \n@output");
            $this->_set_err(@output);
            goto _installMPIO_EXIT;
        }
    }
    elsif ($installType =~ /full/i)
    {
        # TODO: add build id verification after installation
        $srcDir = "/x/eng/rtpwaktmp/installs/" . $this->{hostname} . "/package/";
        @output = `ls $srcDir`;

        # find the filename of the executable
        foreach $line (@output)
        {
            $installEXE = $1 if ($line =~ /(san_host_win_mpio.*)\s+/i);
        }

        $srcFile = "z:\\package\\" . $installEXE;
        $this->deleteRemoteFile("$Host::TEMP\\*");

        # copy the installation file to a local folder
        ($status, @output) = $this->copyRemoteFile($srcFile, "$Host::TEMP\\");

        if ($status != $Host::TRUE)
        {
            &main::logcomment("Install::_installMPIO - Copy of $srcFile failed: \n@output");
            $this->_set_err(@output);
            goto _installMPIO_EXIT;
        }
        # build the command to uninstall first
        $cmd = "$Host::TEMP\\$installEXE" . ' /s /x /v\"/qn ENABLE_LOGGING=1 /l*v' .
            ' %SystemDrive%\mpio_cleanup.log\"';

        # set retry count for uninstallation
        for (my $retryCount = 5; $retryCount != 0; $retryCount--)
        {
            # verify we have X mounted and accessible
            $this->_isMinWin();
            goto _installMPIO_EXIT if ($this->error());

            # send the command via root, this is a hack due to installshield
            &main::logcomment("Removing new build via 'sudo rsh' hack");
            `r rsh $this->{hostname} start /b /wait "$cmd"`;

            $this->reboot();
            goto _installMPIO_EXIT if ($this->error());

            # check to see if the service still exists to verify whether
            # uninstallation was successfull or not
            $chkCmd = "$this->{cmd_sc} query MPIOSvc";
            ($status, @output) = $this->execute($chkCmd);

            if ($status != $Host::TRUE)
            {
                &main::logcomment("Install::_installMPIO - CMD failed -  $chkCmd: \n@output");
                $this->_set_err(@output);
                goto _installMPIO_EXIT;
            }
            
            last if (grep(/service does not exist/, @output));
        }

        # build the command to perform the installation
        $cmd = "$Host::TEMP\\$installEXE" . ' /s /v\"/qn ENABLE_LOGGING=1 /l*v' .
            ' %SystemDrive%\mpio_setup.log MPIO_LICENSECODE=JQQMTZXQLGIDMA\"';

        # set retry count for installation and begin installation
        for (my $retryCount = 5; $retryCount != 0; $retryCount--)
        {
            # verify we have X mounted and accessible
            $this->_isMinWin();
            goto _installMPIO_EXIT if ($this->error());

            # second part of hack above
            &main::logcomment("Installing new build via 'sudo rsh' hack");
            `r rsh $this->{hostname} start /b /wait "$cmd"`;

            $this->reboot();
            goto _installMPIO_EXIT if ($this->error());

            # check to see if the service still exists to verify whether
            # uninstallation was successfull or not
            $chkCmd = "$this->{cmd_sc} query MPIOSvc";
            ($status, @output) = $this->execute($chkCmd);

            if ($status != $Host::TRUE)
            {
                &main::logcomment("Install::_installMPIO - CMD failed -  $chkCmd: \n@output");
                $this->_set_err(@output);
                goto _installMPIO_EXIT;
            }
            
            last if (grep(/SERVICE_NAME: MPIOSvc/, @output));
            next if (grep(/service does not exist/i, @output));

            # if we get here, then we failed 5 retries
            &main::logcomment("Install::_installMPIO - Installation FAILED.");
            $this->_set_err(@output);
            goto _installMPIO_EXIT;
        }

    }
    else
    {
        $msg = "Install::_installMPIO - CRITICAL: invalid installType: $installType";
        &main::logcomment($msg);
        $this->_set_err($msg);
        goto _installMPIO_EXIT;
    }

    &main::logcomment("Successfully installed new ontapdsm build.");

_installMPIO_EXIT:
    
    return;

} # END _installMPIO

1;
