# $Id: //depot/prod/test/Rsierranevada/lib/WinClient/SDCLI.pm#1 $
#
# Copyright (c) 2006 Network Appliance, Inc.
# All rights reserved.
#
# TODO: Complete documentation pending.
#
## @summary
##  Library for implementing SnapDrive SDCLI for Windows.  This library
##  is used to execute commands and parse results.  Some functionality
##  here replaces that in the standard host libs.
##
## @author  kenny speer
## @status  private
##

use strict;

package WinClient::SDCLI;

# define the subroutines used in this package
sub sdSysconfig;
sub sdDisk;
sub sdSnap;
sub sdIscsiInitiator;
sub sdIscsiTarget;
sub sdLicense;
sub sdPreferredIP;
sub sdSpacemon;
sub sdDiskList;
sub sdSnapList;
sub _setMountPoint;

use lib();

#
#
#
sub sdSysconfig
{
    my ($this) = @_;

    my ($status, @output, $cmd, $line, @fcpAdapters, @iscsiAdapters);

    # build the sdcli command
    $cmd = "$this->{cmd_sdcli} sysconfig list";

    # execute the command
    ($status, @output) = $this->execute($cmd);

    if (!grep(/The operation completed successfully/i, @output) || 
        $status != $Host::TRUE)
    {
        &main::logcomment("SDCLI::sdSysconfig - Command failed: $cmd");
        $this->_set_err(@output);
        goto sdSysconfig_EXIT;
    }

    &main::logcomment("SDCLI::sdSysconfig - Output: ", @output);

    # add properties to host for anything displayed here
    foreach $line (@output)
    {
        if ($line =~ /SnapDrive\s(.*),\s+Build:\s(.*)$/i)
        {
            $this->{sdVersion} = $1;
            $this->{sdBuild}   = $2;
        }
        elsif ($line =~ /Port\s+WWN:\s+(.*)/i)
        {
            push(@fcpAdapters, $1);
        }
        # The o/p of sdcli sysconfig list command seems to be inconsistent
        # across the various build or version hence changing the regex. Ex:
       	# iSCSI initiator : iqn.1991-05.com.microsoft:w2k3e212-103.sts.lab.eng.btc.netapp.in
        # iSCSI initiator:	iqn.1991-05.com.microsoft:kc50b8-e1
        #elsif ($line =~ /iSCSI\s+initiator\s+:\s+(.*)/i)
        elsif ($line =~ /iSCSI\s+initiator\s*:\s+(.*)/i)
        {
            push(@iscsiAdapters, $1);
        }
    }

    # populate the protocol adapter properties if they exist
    $this->{fcpAdapters} = \@fcpAdapters if (@fcpAdapters);
    $this->{iscsiAdapters} = \@iscsiAdapters if (@iscsiAdapters);

sdSysconfig_EXIT:

    return;

} # END sdSysconfig

#
#
#
sub sdDisk
{
    my ($this, %args) = @_;

    my ($action, $mountPoint, $uncPath, $reserve, $driveSize, $dtype);
    my ($rsrcGrpName, $rsrcGrpDesc, $clusterName, $force, $initPortName);
    my ($volumeName, $volMntPoint, $createFolder, $cmd, $msg);
    my ($status, @output, $line, $letter, $mountType, @adapters);

    foreach (keys %args)
    {
        if (/^-action$/i)
        {
            if (!length($action = $args{$_}) || 
                $action !~ /(?:create|delete|connect|disconnect|expand|list)/i &&
                $action !~ /(?:add_initiator|remove_initiator|add_mount|remove_mount)/i)
            {
                $msg = "SDCLI::sdDisk - invalid value for -action: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto sdDisk_EXIT;
            }
            next;
        }
        if (/^-mountPoint$/i)
        {
            if (!length($mountPoint = $args{$_}))
            {
                $msg = "SDCLI::sdDisk - invalid value for -mountPoint: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto sdDisk_EXIT;
            }
            next;
        }
        if (/^-mountType$/i)
        {
            if (!length($mountType = $args{$_}))
            {
                $msg = "SDCLI::sdDisk - invalid value for -mountType: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto sdDisk_EXIT;
            }
            next;
        }
    }

    # set @adapters to the current protocols adapters
    if ($this->{protocol} =~ /iscsi/i)
    {
        @adapters = @{$this->{iscsiAdapters}};
    }
    elsif ($this->{protocol} =~ /fcp/i)
    {
        @adapters = @{$this->{fcpAdapters}};
    }
    else
    {
        $msg = "ERROR: Unsupported protocol: $this->{protocol}";
        &main::logcomment($msg);
        $this->_set_err($msg);
        goto sdDisk_EXIT;
    }
    
    # for disk create, define the required elements and build the
    # command line
    if ($action =~ /^(?:create|connect)$/i)
    {
        # validate required args
        if (!length($uncPath    = $args{"-uncPath"})   ||
            (($action =~ /create/) && !length($driveSize  = $args{"-driveSize"})) ||
            !length($dtype      = $args{"-dtype"}))
        {
            $msg = "SDCLI::sdDisk - Incomplete arguments: " . %args;
            &main::logcomment($msg);
            $this->_set_err($msg);
            goto sdDisk_EXIT;
        }
        
        # default to drive letter if type not defined
        $mountType = "letter" if (!defined($mountType));

        #if the mountpoint is not defined then create one
        # automatically, else validate then use the mounpoint supplied

        # clear the errors first
        $this->_clear_err();
        _setMountPoint(\$mountType, \$mountPoint, $uncPath, $this);

        goto sdDisk_EXIT
            if ($this->error);

        # build the command
        $cmd = "$this->{cmd_sdcli} disk $action -dtype $dtype -p $uncPath -d $mountPoint";

        # add drive size if this is create
        $cmd .= " -z $driveSize" if ($action =~ /create/i);

        # set flags 'snapshot reserve' and cluster
        $cmd .= " -rs y" if ($reserve && $action =~ /create/i);

        # TODO: add cluster support here
        #$cmd .= " -c $clusterName" if (defined($clusterName));

        # set the adapter type based on protocol and if multipath is not
        # enabled, then only add the first initiator
        $cmd .= " -I";

        if ($this->{multipath})
        {
            foreach $initPortName (@adapters)
            {
                $cmd .= " $this->{hostname} $initPortName";
            }
        }
        else
        {
            $cmd .= " $this->{hostname} $adapters[0]";
        }

        # for shared disks only, add cluster resource information
        if ($dtype =~ /shared/i)
        {
            $cmd .= " -e $rsrcGrpName" if defined($rsrcGrpName);
            $cmd .= " -n $rsrcGrpName $rsrcGrpDesc" if defined($rsrcGrpDesc);
        }
        
    } #end create|connect
    elsif ($action =~ /^(?:delete|disconnect)$/i)
    {
        if (!length($mountPoint = $args{"-mountPoint"}) &&
            !length($uncPath    = $args{"-uncPath"}))
        {
            $msg = "SDCLI::sdDisk - invalid arguments for action '$action'";
            &main::logcomment($msg);
            $this->_set_err($msg);
            goto sdDisk_EXIT;
        }

        # build the command
        $cmd = "$this->{cmd_sdcli} disk $action ";

        # add either mountpoint or uncpath, but not both
        $cmd .= " -d $mountPoint" if (defined($mountPoint));
        $cmd .= " -p $uncPath" if (defined($uncPath));

        # append -f if -force is true; only valid for disconnect
        $cmd .= " -f" if ($args{"-force"} && $action =~ /disconnect/i);

    } #end delete|disconnect
    elsif ($action =~ /^(?:add|remove)_initiator$/i)
    {
        if (!length($mountPoint = $args{"-mountPoint"}) &&
            !length($uncPath    = $args{"-uncPath"}))
        {
            $msg = "SDCLI::sdDisk - invalid arguments for action '$action'";
            &main::logcomment($msg);
            $this->_set_err($msg);
            goto sdDisk_EXIT;
        }

        # build the command
        $cmd = "$this->{cmd_sdcli} disk $action ";

        # add either mountpoint or uncpath, but not both
        $cmd .= " -d $mountPoint" if (defined($mountPoint));
        $cmd .= " -p $uncPath" if (defined($uncPath));

        # add the initiaor
        $cmd .= " -i $adapters[1]";

        # now we adjust the multipath property based on the action
        $this->{multipath} = 1 if ($action =~ /add_initiator/i);
        $this->{multipath} = 0 if ($action =~ /remove_initiator/i);

    } #end add|remove_initiator

    # execute the command and parse the output
    ($status, @output) = $this->execute($cmd, $this->{timeout});
    
    # if succes or if we reran setup on a configured system, just keep
    # going, the lun exists that we wanted
    if ((!grep(/The operation completed successfully/i, @output) &&
        !grep(/A LUN with the specified name already exist/i, @output)) ||
        $status != $Host::TRUE)
    {
        &main::logcomment("SDCLI::sdSysconfig - Command failed: $cmd");
        $this->_set_err(@output);
        goto sdDisk_EXIT;
    }

sdDisk_EXIT:

    return;

} # END sdDisk

#
#
#
sub sdSnap
{
    my ($this, %args) = @_;
    my ($action, $snapshot, $newsnapshot, $cmd, $msg, $path);
    my ($mnt_type, $mnt_ref, $mnt_pt);
    my ($snap_mnt_type, $snap_mnt_pt);

    # check if the required args : action, mountpoint
    # have been passed 
    if (!length($action = $args{'-action'}) || 
        $action !~ /(?:create|delete|list|restore|rename|mount|unmount
                       |update_mirror|mirror_list)/i)
    {
        $msg = "SDCLI::sdSnap - invalid value for -action: $action";
        &main::logcomment($msg);
        $this->_set_err($msg);
        goto sdSnap_EXIT;
    }
    
    #mountPoint is mandatory argument for all commands except "list"
    if ( ($action !~ /list/) && (!defined($args{'-mountPoint'})) )
    { 
        $msg = "SDCLI::sdSnap - Incomplete arguments: Mountpoint not specified" . %args;
        &main::logcomment($msg);
        $this->_set_err($msg);
        goto sdSnap_EXIT;
    }

    # for snap create, define the required elements and build the
    # command line
    if ($action =~ /^(?:create)$/i)
    {
        #Get the reference to the mountpoint
        $mnt_ref = \$args{'-mountPoint'}; 

        $mnt_pt =  $$mnt_ref
            if (ref($mnt_ref) eq "SCALAR");
        $mnt_pt =  join(" ", @$mnt_ref)
            if (ref($mnt_ref) eq "ARRAY");

        # build the command
        $cmd = "$this->{cmd_sdcli} snap $action -D $mnt_pt";

        #get the snapshot name from the input hash args if it exists  
        #or set a default name for snapshot using timestamp
        if (!length($snapshot = $args{'-snapshot'}) )
        {
            my ($sec,$min,$hr,$day,$month,$yr) = localtime(time);
            $yr = $yr + 1900;
            $month++;
            $snapshot = "snp_" . $hr.$min.$sec. "_" . $month.$day.$yr ; 
        }
        $cmd .= " -s $snapshot";

        #add the option -x and -u if specified
        $cmd .= " -x" if ( defined($args{'-specific_virtual_disks'}) && 
                        $args{'-specific_virtual_disks'} ) ; 
        $cmd .= " -u $args{'-snapmirror_update'}" 
            if (defined($args{'-snapmirror_update'}) );  

    }

    if ($action =~ /^(?:delete|restore)$/i)
    {
        #Check if the input, snapshot name has been specified 
        if ( !length($snapshot = $args{'-snapshot'}) ) 
        {
            $msg = "SDCLI::sdSnap delete - Incomplete arguments: snapshot name not specified" . %args;
            &main::logcomment($msg);
            $this->_set_err($msg);
            goto sdSnap_EXIT;
        }
        $mnt_pt =  $args{'-mountPoint'};

        # build the command
        $cmd = "$this->{cmd_sdcli} snap $action -s $snapshot -d $mnt_pt";
    }

    if ($action =~ /^(?:rename)$/i)
    {
        #Check if the input, snapshot names have been specified 
        if (!(length($snapshot = $args{'-snapshot'}) &&
            length($newsnapshot = $args{'-newsnapshot'}) )) 
        {
            $msg = "SDCLI::sdSnap rename - Incomplete arguments: snapshot names not specified" . %args;
            &main::logcomment($msg);
            $this->_set_err($msg);
            goto sdSnap_EXIT;
        }
        $mnt_pt =  $args{'-mountPoint'};

        # build the command
        $cmd = "$this->{cmd_sdcli} snap $action -o $snapshot -n $newsnapshot -d $mnt_pt";
    }

    if ($action =~ /^(?:list)$/i)
    {
        #sdcli snap list can use mountpoint or filer,volume pair as argument
        if (defined($args{'-mountPoint'}) )
        {
            $mnt_pt =  $args{'-mountPoint'};
            $cmd = "$this->{cmd_sdcli} snap $action -d $mnt_pt";
        }
        elsif ( (defined($args{'-sdf'})) && (defined($args{'-sdv'})) )
        {
            $cmd = "$this->{cmd_sdcli} snap $action -sdf $args{'-sdf'} -sdv $args{'-sdv'}";
        }
        else
        {
            $msg = "SDCLI::sdSnap list - Incomplete arguments" . %args;
            &main::logcomment($msg);
            $this->_set_err($msg);
            goto sdSnap_EXIT;
        }
    }

    if ($action =~ /^(?:unmount|update_mirror|mirror_list)$/i)
    {
        $mnt_pt =  $args{'-mountPoint'};
        # build the command
        $cmd = "$this->{cmd_sdcli} snap $action -d $mnt_pt";
    }

    if ($action =~ /^(?:mount)$/i)
    {
        #Check if the input, snapshot name has been specified 
        if ( !length($snapshot = $args{'-snapshot'}) ) 
        {
            $msg = "SDCLI::sdSnap mount - Incomplete arguments: snapshot name not specified" . %args;
            &main::logcomment($msg);
            $this->_set_err($msg);
            goto sdSnap_EXIT;
        }
        $mnt_pt =  $args{'-mountPoint'};

        # default to drive letter if mount type not defined
        if (!defined($args{'-snapmountType'})) {
            $snap_mnt_type = "letter"; 
        }
        else
        {
            $snap_mnt_type = $args{'-snapmountType'}; 
        }

        if ($snap_mnt_type =~ /letter/)
        {
            $snap_mnt_pt = $args{'-snapmountPoint'}
                if (defined($args{'-snapmountPoint'})); 
        }
        elsif ($snap_mnt_type =~ /mount/)
        {
            if (defined($args{'-snapmountPoint'}))
            {
                $snap_mnt_pt = $args{'-snapmountPoint'};
                $path = $Host::MOUNT . "\\" .$snap_mnt_pt ;
                $this->execute("mkdir $path", $this->{timeout});
                $snap_mnt_pt = $path;
            }
            else
            {
                my ($sec,$min,$hr,$day,$month,$yr) = localtime(time);
                $yr = $yr + 1900;
                $month++;
                $path = "\\\\mnt_" . $hr.$min.$sec. "_" . $month.$day.$yr ; 
            }
        }

        if (!(($snap_mnt_type =~ /letter/) && ($args{'-trydrive'} == 1)) )
        {
            # clear the errors first
            $this->_clear_err();
            _setMountPoint(\$snap_mnt_type, \$snap_mnt_pt, $path, $this);
            goto sdDisk_EXIT
                if ($this->error);
        }

        # build the command
        $cmd = "$this->{cmd_sdcli} snap $action -k $mnt_pt -s $snapshot -d $snap_mnt_pt";

        #Include -smdest -trydrive option if set to true 
        $cmd .= " -smdest"
            if ($args{'-smdest'} == 1);
        $cmd .= " -trydrive"
            if ($args{'-trydrive'} == 1);

        #Include -sdf -sdv option if they are defined 
        $cmd .= " -sdf $args{'-sdf'}"
            if (defined($args{'-sdf'}));
        $cmd .= " -sdv $args{'-sdv'}"
            if (defined($args{'-sdv'}));
    }

    # execute the command and parse the output
    my ($status, @output) = $this->execute($cmd, $this->{timeout});
    
    # if succes or if we reran setup on a configured system, just keep
    # going, the lun exists that we wanted
    if ((!grep(/The operation completed successfully/i, @output)) ||
        $status != $Host::TRUE)
    {
        &main::logcomment("SDCLI::sdSnap - Command failed: $cmd");
        $this->_set_err(@output);
    }

sdSnap_EXIT:
    return;
} # END sdSnap

#
#
#
sub sdIscsiInitiator
{
} # END sdIscsiInitiator

#
#
#
sub sdIscsiTarget
{
} # END sdIscsiTarget

#
#
#
sub sdLicense
{
} # END sdLicense

#
#
#
sub sdPreferredIP
{
} # END sdPreferredIOP

#
#
#
sub sdSpacemon
{
} # END sdSpacemon

#if the mountpoint is not passed thru cmd line, then set it or 
#else validate the mountpoint passed thru cmd line
sub _setMountPoint
{
    my ($mntTypeRef, $mntPtRef, $mntPath, $this) = @_;
    my $msg;

    # if the mountpoint is not defined then create one
    # automatically, else validate then use the mounpoint supplied
    if (!defined($$mntPtRef))
    {
        if ($$mntTypeRef =~ /letter/)
        {
            # avoid a race condition where we've lost track of the
            # used and available drive letters
            if (!defined($this->{availLetters}) ||
                !defined($this->{usedLetters}))
            {
                $msg = "ERROR:" .
                "No available or used drive letters defined.\n" .
                "Please execute getDevices()";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto _setMountPoint_EXIT;
            }

            # SnapDrive won't allow A|B as they are reserved for
            # removable media (not sure of side effects of using)
            while (!defined($$mntPtRef) || $$mntPtRef =~ /^[A-C]$/)
            {
                $$mntPtRef = shift(@{$this->{availLetters}});
                push(@{$this->{usedLetters}}, $$mntPtRef);
            }
        }
        elsif ($$mntTypeRef =~ /mount/)
        {
            $mntPath =~ /\\\\(.*)$/;
            $$mntPtRef = $Host::MOUNT . "\\" . $1;
            $this->execute("mkdir $$mntPtRef", $this->{timeout});
        }
    }
    else 
    {
        if ($$mntTypeRef =~ /letter/)
        {
            # drive letter assignments don't have :
            $$mntPtRef =~ s/://;
            if (grep(/$$mntPtRef/i, @{$this->{usedLetters}}))
            {
                $msg = "SDCLI::sdDisk - Requested drive letter '$$mntPtRef' already assigned to another device";
                &main::logcomment($msg);
                $this->_set_err($msg);
            }
        }
    }

_setMountPoint_EXIT:
    return ;
}

sub sdDiskList
{
    my $this = shift;
    my ($line, $uncPath, $LUNPath);
    my ($key1, $key2, $hkey, $hval);
    my %HoH;

    #Get the SDW version info
    my $version = $this->{sdVersion};
    $version =~ /(\d+)(.*)/;
    $version = $1;

    # build the command
    my $cmd = "$this->{cmd_sdcli} disk list";

    # execute the command and parse the output
    my ($status, @output) = $this->execute($cmd, $this->{timeout});
    
    # if succes or if we reran setup on a configured system, just keep
    # going, the lun exists that we wanted
    if ((!grep(/The operation completed successfully/i, @output)) ||
        $status != $Host::TRUE)
    {
        &main::logcomment("SDCLI::sdDiskList - Command failed: $cmd");
        $this->_set_err(@output);
        goto sdDiskList_EXIT;
    }

    my $out = join("\n", @output);
    my @lines = split('UNC', $out);
    foreach (@lines)
    {
        my @drive = split('\n', $_)
            if (length($_));
        foreach $line (@drive)
        {
            if ($line =~ /\s*(.*?):\s*(.*)/)
            {
                $hkey = $1;
                $hval = $2;
                if ($hkey =~ /^Path/i)
                {
                    $uncPath = $hval;
                }elsif ($hkey =~ /LUN Path/i)
                {
                    $LUNPath = $hval;
                }else
                {
                    if ($version < 6) {
                        $HoH{$uncPath}{$hkey} = $hval;
                    }else {
                        $HoH{$LUNPath}{$hkey} = $hval;
                        $HoH{$LUNPath}{'UNC Path'} = $uncPath ;
                    }
                }
            }
        }#for every property of a particular disk (a uncPath) 
    } #for every disk (all uncPaths) 

sdDiskList_EXIT:
    return %HoH;
}


sub sdSnapList
{
    my ($this, $mnt_pt) = @_;
    my %snapshots;

    # build the command
    my $cmd = "$this->{cmd_sdcli} snap list -d $mnt_pt";

    # execute the command and parse the output
    my ($status, @output) = $this->execute($cmd, $this->{timeout});

    # if succes or if we reran setup on a configured system, just keep
    # going, the lun exists that we wanted
    if ((!grep(/The operation completed successfully/i, @output)) ||
        $status != $Host::TRUE)
    {
        &main::logcomment("SDCLI::sdSnapList - Command failed: $cmd");
        $this->_set_err(@output);
        goto sdSnapList_EXIT;
    }
    shift(@output);
    shift(@output);

    foreach my $line (@output) {
        if ($line =~ /snapshot/) {
            my @fields = split('\t', $line); 
            my $snap = shift @fields;                        
            my $snp_status  =  pop @fields;                        
            $snapshots{$snap} = $snp_status;
        }
    }
sdSnapList_EXIT:
    return %snapshots;
}
1;
