# $Id: //depot/prod/test/Rsierranevada/lib/WinClient/SIO.pm#1 $
#
# Copyright (c) 2006 Network Appliance, Inc.
# All rights reserved.
#
# This library generates IO for a specific disk.  These methods are
# designed to be called via a disk object method and not directly from a
# thpl.
#
# TODO: Complete documentation pending.
#
## @summary
##  Wrapper for WinClient host functionality.
##
## @author  kenny speer
## @status  private
##

use strict;

package WinClient::SIO;

# define the subroutines used in this package
sub _startSIO;

use lib();

#
#
#
sub _startSIO
{
    my ($this, %args) = @_;

    my ($status, @output);
    my ($time, @randBlockSizes, @randFileSizes, $randCount, $dtCmd);
    my ($location, $ioType, $msg, $runLength, $blockSize, $fileSize, $threads);
    my ($fillCmd, $testCmd, $cmd);

    foreach (keys %args)
    {
        if (/^-location$/i)
        {
            if (!length($location = $args{$_}))
            {
                $msg = "SIO::startSIO - invalid value for -location: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startSIO_EXIT;
            }
            next;
        }
        if (/^-iotype$/i)
        {
            if (!length($ioType = $args{$_}))
            {
                $msg = "SIO::startSIO - invalid value for -iotype: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startSIO_EXIT;
            }
            next;
        }
        if (/^-?runlength$/i)
        {
            if (!length($runLength = $args{$_}))
            {
                $msg = "SIO::startSIO - invalid value for -runlength: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startSIO_EXIT;
            }
            next;
        }
        if (/^-?blocksize$/i)
        {
            if (!length($blockSize = $args{$_}))
            {
                $msg = "SIO::startSIO - invalid value for -blocksize: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startSIO_EXIT;
            }
            next;
        }
        if (/^-?filesize$/i)
        {
            if (!length($fileSize = $args{$_}))
            {
                $msg = "SIO::startSIO - invalid value for -filesize: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startSIO_EXIT;
            }
            next;
        }
        if (/^-?threads$/i)
        {
            if (!length($threads = $args{$_}))
            {
                $msg = "SIO::startSIO - invalid value for -threads: $args{$_}";
                &main::logcomment($msg);
                $this->_set_err($msg);
                goto startSIO_EXIT;
            }
            next;
        }
    }

    # adjust the time for seconds so we know how long to wait
    $time = $runLength;
    if ($runLength =~ /(s|h|m|d)/i)
    {
        $time =~ s/(s|h|m|d)//ig;
        $time = $time * 60 if ($runLength =~ /m/i);
        $time = $time * 60 * 60 if ($runLength =~ /h/i);
        $time = $time * 60 * 60 * 24 if ($runLength =~ /d/i);
    }

    # fix runlength before munging time
    $runLength = $time;

    # add 10 min to time for say() timeout
    $time += 900;
    
    # define random block and file sizes
    @randBlockSizes = ("1k", "2k", "4k", "8k", "16k", "29k",
                       "32k", "64k", "128k", "201k", "256k");
    @randFileSizes = ("100m", "250m", "500m", "750m", "1g",
                      "1250m", "1500m", "1750m", "2g");

    # initialize random number generator
    srand(time ^ $$ * $randCount++);

    # setup the block and file sizes
    if ($blockSize =~ /^random$/i)
    {
        $blockSize = $randBlockSizes[int rand scalar(@randBlockSizes)];
        &main::logcomment("SIO::startSIO - Using a random ($blockSize) block size.");
    }

    if ($fileSize =~ /^random$/i)
    {
        $fileSize = $randFileSizes[int rand scalar(@randBlockSizes)];
        &main::logcomment("SIO::startSIO - Using a random ($fileSize) file size.");
    }

    # create the destination location
    $this->{host}->createRemoteFile($location);

    # create the fill command used for (fillonce)
    $fillCmd = "$this->{host}->{cmd_sio} 0 0 $blockSize 1 $fileSize $runLength 1 $location -fillonce -nosysinfo";
    
    # create the test command
    $testCmd = "$this->{host}->{cmd_sio} 50 50 $blockSize 1 $fileSize $runLength $threads $location -verify -nosysinfo";

    # execute the command
    &main::logcomment("SIO::startSIO - Executing command: $fillCmd");

    ($status, @output) = $this->execute($fillCmd, $time);

    if ($status != $Host::TRUE)
    {
        &main::logcomment("SIO::startSIO - Command failed: $fillCmd");
        $this->_set_err(@output);
        goto startSIO_EXIT;
    }

    # look for some SIO specific errors
    if (!grep(/I\/O\s+completed successfully/i, @output))
    {
        &main::logcomment("SIO::startSIO - Errors were detected during IO run.");
        $this->_set_err(@output);
        goto startSIO_EXIT;
    }

    # execute the command
    &main::logcomment("SIO::startSIO - Executing command: $testCmd");

    ($status, @output) = $this->execute($testCmd, $time);

    if ($status != $Host::TRUE)
    {
        &main::logcomment("SIO::startSIO - Command failed: $testCmd");
        $this->_set_err(@output);
        goto startSIO_EXIT;
    }

    if (grep(/Maximum\s+latency\s+observed:/i, @output))
    {
        &main::logcomment("SIO::startSIO - Completed successfully.");
    }
    else
    {
        $msg = "SIO::startSIO - Command did not complete: $testCmd";
        &main::logcomment($msg);
        $this->_set_err($msg);
        goto startSIO_EXIT;
    }
        
startSIO_EXIT:

    return;

} # END _startSIO

1;
