# $Id: //depot/prod/test/Rsierranevada/lib/XMLDotPlan.pm#1 $
# $Author: build $
## @author: nsesha@netapp.com [ dl-raid-qa ]

# TODO:
#   change all XML::XPath->new() type of code to XML::XPath::Node::Element->new()
#   revisit debug_levels param added to api's as some of it may not have any importance
#   revisit all $self->log() and add more of these as required
# NOTE:
#   If filter is id, then all TCs are listed and there is no restriction on filter name
# 

package XMLDotPlan;

use strict;
#==========================================
#       Library Includes
#==========================================
use XML::LibXML;
use XML::XPath;
use XML::Twig;
use XML::Parser;
use Data::Dumper;
use Params::Validate qw(:all);

our @ISA = qw(Utility);

my $Package = "XMLDotPlan";

use vars qw($VERSION %xml_tags %error_flag);
$VERSION  = "1.0";

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my %opts = validate(
        @_, {
            xmlfile             => { type => SCALAR, optional => 0},
            filter              => { type => SCALAR | UNDEF, optional => 1},
            default_value_only  => { type => SCALAR, optional => 1, default => 0},
            debug_level         => { type => SCALAR, optional => 1, default => 1},
        }
    );
    my $self = $class->SUPER::new(debug_level=>$opts{debug_level});
    %opts = (%{$self}, %opts);
    $self = \%opts;
    bless $self, $class;

    $self->{xml_tags}->{global_dependency} = "/xmlplan/global_dependency";
    $self->{xml_tags}->{test_script} = "/xmlplan/test_script";
    
    # Few initialization methods
    $self->load_xmlfile();
    $self->validate_xmlfile();
    $self->generateDS();
    return $self;
} 

sub load_xmlfile{
    my $self = shift;
    $self->log(1, "$Package: Loading XML File... " . $self->get_xmlfile());
    $self->{root_element} = XML::XPath->new( filename => $self->get_xmlfile() );
    $self->log(1, "$Package: Loading XML File... done");
} 

sub validate_xmlfile {
    my $self = shift;
    $self->log(1, "$Package: Started XML validation ... " );
    # Validate presence of 1..n <test_script>
    if(!$self->{root_element}->findnodes( $self->{xml_tags}->{test_script} )->size){
        die("$Package: Error: XML file does not have even a single <test_script> tag");
    }
    
    # # NOTE: May be validation of dependency is not required for creating expanded version of XML
    # # Validate presence of 0..1 <global_dependency>
    # if($self->{root_element}->findnodes( $self->{xml_tags}->{global_dependency} )->size){
        # if($self->{root_element}->findnodes( $self->{xml_tags}->{global_dependency} )->size > 1){
            # die("$Package: Error: global_dependency is found more than one instance in this xmlfile: \n".
                # $self->get_xmlfile()."\n"
            # );
        # }
        # # Validate <global_dependency/spare_disks>
        # if($self->{root_element}->findnodes( $self->{xml_tags}->{global_dependency}."/spare_disks" )->size != 1){
            # die("$Package: Error: <global_dependency/spare_disks> is either not found or ".
                # "more than one instance is found in this xmlfile: \n".
                # $self->get_xmlfile()."\n"
            # );
        # }
        # my $diskTag = $self->{xml_tags}->{global_dependency}."/spare_disks/disk";
        # if(!$self->{root_element}->findnodes( $diskTag )->size){
            # die("$Package: Error: <global_dependency/spare_disks/disk> is not found in this xmlfile: \n".
                # $self->get_xmlfile()."\n"
            # );
        # }
        # foreach my $disk ($self->{root_element}->findnodes($diskTag)->get_nodelist()){
            # my %diskAttrs = $self->getAttributesFromElement(element=>$disk);
            # if(scalar(grep /^\s*(pool|count|storage_type|checksum_type)\s*$/i, keys(%diskAttrs)) !=4){
                # die("$Package: Error: One of the required attribute is missed in <global_dependency/spare_disks/disk>: \n".
                    # Dumper(\%diskAttrs)."\n".
                    # "Required attributes are: pool, count, storage_type, checksum_type"."\n"
                # );
            # }
            # if(exists $diskAttrs{pool} and $diskAttrs{pool} !~ /0|1/){
                # die("$Package: Error: pool attribute has invalid value in <global_dependency/spare_disks/disk>: \n".
                    # Dumper(\%diskAttrs)."\n"."Correct value: 0,1"."\n"
                # );
            # }
            # if(exists $diskAttrs{count} and $diskAttrs{count} < 2){
                # die("$Package: Error: count attribute has invalid value in <global_dependency/spare_disks/disk>: \n".
                    # Dumper(\%diskAttrs)."\n"."Correct value: >= 2"."\n"
                # );
            # }
        # }
    # }
    
    $self->log(1, "$Package: XML Validation passed" );
    return 1;
} 

sub has_testcase{
    my $self = shift;
    my $expanded_xmlplan = $self->get_xmlplan();
    return $expanded_xmlplan->findnodes('/test_script')->size;
}

sub get_xmlfile{
    my $self = shift;
    if(!defined($self->{xmlfile})){
        die "$Package: Error: error in xmlfile";
    }
    if(! -e $self->{xmlfile}){
        die "$Package: Error: XML File not found - $self->{xmlfile}";
    }    
    return $self->{xmlfile};
}

# TODO: Remove this function
sub get_xsdfile{
    my $self = shift;
    if(!defined($self->{xsdfile})){
        die "$Package: Error: error in xsdfile";
    }
    if(! -e $self->{xsdfile}){
        die "$Package: Error: XSD File not found - $self->{xssdfile}";
    }    
    return $self->{xsdfile};
}

# Returns xml string form for root_element
sub toString{
    my $self = shift;
    $self->log(1, "$Package: Get string format of xml file");
    my @nodeSet=$self->{root_element}->findnodes('/');
    my $xml=XML::XPath::XMLParser::as_string($nodeSet[0]);
    return $xml;    
}

sub get_filter{
    my $self = shift;
    return $self->{filter};
}

# By default returns "expanded_xmlplan". if original=>1, then returns original 
sub get_xmlplan{
    my $self = shift;
    my %opts = validate(@_, {
        original => { type=>SCALAR, optional=>1, default=>0},
    });
    
    if ($opts{original}){
        my @nodeSet=$self->{root_element}->findnodes('/');
        return $nodeSet[0];
    }
    return $self->{expanded_xmlplan};
}

sub generateDS{
    my $self = shift;
    
    my $xpGlobalDependency = $self->{root_element}->find( $self->{xml_tags}->{global_dependency} );
    $self->set_global_dependency(global_dependency=>${$xpGlobalDependency}[0]) if $self->has_global_dependency();
    
    my $xpTestScript = $self->{root_element}->find( $self->{xml_tags}->{test_script} );
    my $ts_count = 0;
    
    my $xmlroot = "<xmlplan/>";
    my $xp= XML::XPath->new($xmlroot);
    my @nodes = $xp->findnodes('*')->get_nodelist();
    my $parent = $nodes[0];
    $parent->appendChild($self->get_global_dependency()) if $self->has_global_dependency();
    foreach my $testscript_element ( $xpTestScript->get_nodelist() ) {
        my $testscript = XMLDotPlan::TestScript->new(
            testscript_element      => $testscript_element,
            # root_element          => $self->{root_element},
            testscript_position     => $ts_count,
            # parent_obj            => $self,
            filter                  => $self->get_filter(),
            default_value_only      => $self->get_default_value_only_flag(),
            has_global_dependency   => $self->has_global_dependency(),
            debug_level             => $self->{debug_level},
        );
        $ts_count++;
        # $testscript->validate_testscript();
        $parent->appendChild( $testscript->get_expanded_testscript() ) if $testscript->is_ts_expanded();
    } 
    # my @nodeSet=$parent->findnodes('/');
    # $self->printXML(nodexml=>$nodeSet[0]);
    $self->log(2, "$Package: Generated Data structure" );
    $self->set_expanded_xmlplan(expanded_xmlplan => $parent);
}

sub has_global_dependency{
    my $self = shift;
    return $self->{root_element}->findnodes( $self->{xml_tags}->{global_dependency} )->size;
}

sub set_global_dependency{
    my $self = shift;
    my %opts = validate(@_, {
        global_dependency => { type=>OBJECT, isa=>"XML::XPath::Node::Element", optional=>0},
    });
    $self->{global_dependency} = $opts{global_dependency};
}

sub get_global_dependency{
    my $self = shift;
    return $self->{global_dependency};
}

sub get_default_value_only_flag{
    my $self = shift;
    return $self->{default_value_only};
}

sub set_expanded_xmlplan{
    my $self = shift;
    my %opts = validate(@_, {
        expanded_xmlplan => { type=>OBJECT, isa=>"XML::XPath::Node::Element", optional=>0},
    });
    # Fix : Memory leak
    undef $self->{expanded_xmlplan} if defined( $self->{expanded_xmlplan} );
    $self->{expanded_xmlplan} = $opts{expanded_xmlplan};
}

1;

#===================================================================================================
package XMLDotPlan::TestScript;
use XML::XPath;
use Data::Dumper;
use Params::Validate qw(:all);

our @ISA = qw(Utility);

my $Package = "XMLDotPlan::TestScript";

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    
    my %opts = validate(
        @_, {
            testscript_element      => { type => OBJECT, isa=>"XML::XPath::Node::Element", optional => 0},
            # root_element          => { type => OBJECT, isa=>"XML::XPath", optional => 0},
            testscript_position     => { type => SCALAR, optional => 1, default => 0},
            # parent_obj            => { type => OBJECT, isa=>"XMLDotPlan", optional=>0},
            filter                  => { type => SCALAR | UNDEF, optional=>1 },
            default_value_only      => { type => SCALAR, optional => 1, default => 0},
            has_global_dependency   => { type => SCALAR, optional=>1, default=>0 },
            debug_level             => { type => SCALAR, optional => 1, default => 1},
        }
    );
    my $self = $class->SUPER::new(debug_level=>$opts{debug_level});
    %opts = (%{$self}, %opts);
    $self = \%opts;
    bless $self, $class;

    $self->{xml_tags}->{path} = "./path";
    $self->{xml_tags}->{dependency} = "./dependency";
    $self->{xml_tags}->{script_params} = "./script_params";
    $self->{xml_tags}->{testcases} = "./testcase";
    
    # Validate testscript
    $self->validate_testscript(has_global_dependency=>$opts{has_global_dependency});

    # Initialize few fields. If "dependency" and "script_params" are not found, ignore it.
    $self->set_path(path=>${$self->{testscript_element}->find( $self->{xml_tags}->{path} )}[0] );
    if($self->has_tag(tag_name=>"dependency")){
        $self->set_dependency(dependency=>${$self->{testscript_element}->find( $self->{xml_tags}->{dependency} )}[0] );
    }
    if($self->has_tag(tag_name=>"script_params")){
        $self->set_script_params(script_params=>${$self->{testscript_element}->find( $self->{xml_tags}->{script_params} )}[0] );
    }
    $self->set_testcases(testcases=>$self->{testscript_element}->find( $self->{xml_tags}->{testcases} ) );
    
    # Expand testscript
    $self->expand_testscript();

    # $self->print_testscript();
    return $self;
} 

sub validate_testscript{
    my $self = shift;
    my %opts = validate(
        @_, {
            has_global_dependency => { type => SCALAR, optional=>1, default=>0}
        }
    );

    # Validate tag - path
    if($self->{testscript_element}->findnodes( $self->{xml_tags}->{path} )->size != 1){
        die("$Package: Error: testscript path is either not found or found more than one instance for script: \n".
             $self->{testscript_element}->toString()
            );
    }

    # Validate tag - dependency
    if($self->{testscript_element}->findnodes( $self->{xml_tags}->{dependency} )->size < 1 and 
        $opts{has_global_dependency} == 0){
        die("$Package: Error: Neither <global_dependency> nor <test_script/dependency> is found. ".
            "Atleast one of them should exist: \n".
             $self->{testscript_element}->toString()
            );
    }
    if($self->{testscript_element}->findnodes( $self->{xml_tags}->{dependency} )->size > 1){
        die("$Package: Error: testscript dependency is found more than one instance for script: \n".
             $self->{testscript_element}->toString()
            );
    }
    if($self->{testscript_element}->findnodes( $self->{xml_tags}->{dependency})->size and
        !$self->{testscript_element}->findnodes( $self->{xml_tags}->{dependency}."/*" )->size){
        die("$Package: Error: testscript has empty dependency tag: \n".
             $self->{testscript_element}->toString()
            );
    }

    # Validate tag - script_params
    if($self->{testscript_element}->findnodes( $self->{xml_tags}->{script_params} )->size > 1){
        die("$Package: Error: testscript script_params is found more than one instance for script: \n".
             $self->{testscript_element}->toString()
            );
    }
    if($self->{testscript_element}->findnodes( $self->{xml_tags}->{script_params})->size and
        !$self->{testscript_element}->findnodes( $self->{xml_tags}->{script_params}."/*" )->size){
        die("$Package: Error: testscript has empty script_params tag: \n".
             $self->{testscript_element}->toString()
            );
    }

    # Validate tag - testcases
    if(!$self->{testscript_element}->findnodes( $self->{xml_tags}->{testcases} )->size){
        die("$Package: Error: testscript has no testcase: \n".
             $self->{testscript_element}->toString()
            );
    }
}

sub has_tag{
    my $self = shift;
    my %opts = validate(
        @_, {
            tag_name => { type => SCALAR },
        });
        
    my $tag_name = $opts{tag_name};
    return $self->{testscript_element}->findnodes( $self->{xml_tags}->{$tag_name})->size;
}

sub expand_testscript{
    my $self = shift;
    
    my $xmlroot = "<test_script/>";
    my $xp= XML::XPath->new($xmlroot);
    my @nodes = $xp->findnodes('*')->get_nodelist();
    my $parent = $nodes[0];
    $parent->appendChild($self->get_path());
    $parent->appendChild($self->get_dependency()) if $self->has_tag(tag_name=>"dependency");
    # $parent->appendChild($self->get_script_params()) if $self->has_tag(tag_name=>"script_params");

    my $tc_count = 0;
    foreach my $testcase_element ( $self->get_testcases()->get_nodelist() ) {
        my $filter = $self->get_filter();
        my $foundAttr='';
        eval{
            my $found = 0;
            foreach my $attrFilter (split(/,/,$filter)){
                my @attrs = $testcase_element->findnodes('@'.$attrFilter)->get_nodelist();
                if(scalar(@attrs) and $attrs[0]->getValue() =~ /yes/i){
                    $found = 1;
                    $foundAttr=$attrFilter;
                    last;
                }
            }
            if(defined($filter) and !$found){
                my @arr = $testcase_element->findnodes('@id')->get_nodelist();
                $self->log(2, "$Package: filter \"".$self->get_filter()."\" not found in testcase id: ".$arr[0]->getValue());
                next;
            }
        };
        if($@){
            print $@,"\n";
            die("$Package: Error: Invalid filter provided - ".$self->get_filter()."\n".
                "For e.g. \nFilter name should not start with a number or \n".
                "Filter name should not be empty string");
        }
        my %params = (
            testcase_element    => $testcase_element,
            script_path         => $self->get_path(),
            testcase_position   => $tc_count,           # Use this to print position on debub messages
            filter              => $foundAttr,
            default_value_only  => $self->get_default_value_only_flag(),
            debug_level         => $self->{debug_level},
        );
        $params{script_params} = $self->get_script_params() if $self->has_tag(tag_name=>"script_params");
        my $testscript = XMLDotPlan::TestCase->new(%params);

        $tc_count++;
        $parent->appendChild( $testscript->get_expanded_testcase() );
    }


    # my @nodeSet=$parent->findnodes('/');
    # $self->printXML(nodexml=>$nodeSet[0]);
    # $self->log(2, "$Package: Generated Data structure" );
    $self->set_tc_count(tc_count=>$tc_count);
    $self->set_expanded_testscript(testscript_element=>$parent) if $tc_count; 
}

sub is_ts_expanded{
    my $self = shift;
    return $self->{tc_count};
}

sub set_tc_count{
    my $self = shift;
    my %opts = validate(@_, {
        tc_count => { type=>SCALAR, default=>0},
    });
    $self->{tc_count} = $opts{tc_count};
}

sub set_path{
    my $self = shift;
    my %opts = validate(@_, {
        path  => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    $self->{path} = $opts{path}
}

sub set_dependency{
    my $self = shift;
    my %opts = validate(@_, {
        dependency => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    $self->{dependency} = $opts{dependency};
}

sub set_script_params{
    my $self = shift;
    my %opts = validate(@_, {
        script_params => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    # Fix : Memory leak
    undef $self->{script_params} if defined( $self->{script_params} );
    $self->{script_params} = $opts{script_params};
}

sub set_testcases{
    my $self = shift;
    my %opts = validate(@_, {
        testcases => { type=>OBJECT, isa=>"XML::XPath::NodeSet"},
    });
    # Fix : Memory leak
    undef $self->{testcases} if defined( $self->{testcases} );    
    $self->{testcases} = $opts{testcases};
}

sub set_expanded_testscript{
    my $self = shift;
    my %opts = validate(@_, {
        testscript_element => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    # Fix : Memory leak
    undef $self->{testscript_element} if defined( $self->{testscript_element} );    
    $self->{testscript_element} = $opts{testscript_element};
}

sub get_filter{
    my $self = shift;
    return $self->{filter};
}

sub get_path{
    my $self = shift;
    return $self->{path};
}

sub get_dependency{
    my $self = shift;
    return $self->{dependency};
}

sub get_script_params{
    my $self = shift;
    return $self->{script_params};
}

sub get_testcases{
    my $self = shift;
    return $self->{testcases};
}

sub get_expanded_testscript{
    my $self = shift;
    return $self->{testscript_element};
}

sub get_default_value_only_flag{
    my $self = shift;
    return $self->{default_value_only};
}

sub print_testscript{
    my $self = shift;

    $self->log(1, $self->{testscript_position}." "."$Package: Get string format of testscript file");
    print $self->{testscript_element}->toString(), "\n";
    print "\n\n";
}

1;

#===================================================================================================
package XMLDotPlan::TestCase;
use XML::XPath;
use Clone qw(clone);
use Data::Dumper;
use Params::Validate qw(:all);

our @ISA = qw(Utility);

my $Package = "XMLDotPlan::TestCase";

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    
    my %opts = validate(
        @_, {
            testcase_element    => { type => OBJECT, isa=>"XML::XPath::Node::Element", optional => 0},
            script_path         => { type => OBJECT, isa=>"XML::XPath::Node::Element", optional => 0},
            script_params       => { type => OBJECT, isa=>"XML::XPath::Node::Element", optional => 1},
            testcase_position   => { type => SCALAR, optional => 1, default => 0},
            filter              => { type => SCALAR | UNDEF, optional => 1 },
            default_value_only  => { type => SCALAR, optional => 1, default => 0},
            debug_level         => { type => SCALAR, optional => 1, default => 1},
        }
    );
    my $self = $class->SUPER::new(debug_level=>$opts{debug_level});
    %opts = (%{$self}, %opts);
    $self = \%opts;
    bless $self, $class;

    $self->{xml_tags}->{description} = "./description";
    $self->{xml_tags}->{applicable} = "./applicable";
    $self->{xml_tags}->{testcase_params} = "./testcase_params";
    
    # Validate testcase
    $self->validate_testcase();

    # Initialize few fields
    my %attr = $self->getAttributesFromElement(element=>$self->{testcase_element});
    $self->set_testcase_attributes(attributes=>\%attr);
    
    $self->set_description(description=>${$self->{testcase_element}->find( $self->{xml_tags}->{description} )}[0] );
    
    if($self->has_tag(tag_name=>"applicable")){
        $self->set_applicable(applicable=>${$self->{testcase_element}->find( $self->{xml_tags}->{applicable} )}[0] );
    }
    if($self->has_tag(tag_name=>"testcase_params")){
        $self->set_testcase_params(testcase_params=>${$self->{testcase_element}->find( $self->{xml_tags}->{testcase_params} )}[0] );
    }
    
    # Pick super set of script_params and testcase_params
    $self->prepare_testcase_params();

    # validate testcase_params
    $self->validate_testcase_params();
    
    # Expand testcase
    $self->expand_testcase();

    # $self->print_testscript();
    return $self;
} 

sub validate_testcase{
    my $self = shift;
    # validate presence of tags - description, applicable, testcase_params, 
    
    # testcase should have attribute - id
    my %tcAttr = $self->getAttributesFromElement(element=>$self->{testcase_element});
    if(!exists $tcAttr{id}){
        die("$Package: Error: testcase has no attribute - 'id': \n".
             $self->{testcase_element}->toString()
            );
    }   
    
    # Validate tag - description
    if($self->{testcase_element}->findnodes( $self->{xml_tags}->{description} )->size != 1){
        die("$Package: Error: testcase description is either not found or found more than one instance for testcase: \n".
             $self->{testcase_element}->toString()
            );
    }

    # Validate tag - applicable
    if($self->{testcase_element}->findnodes( $self->{xml_tags}->{applicable} )->size > 1){
        die("$Package: Error: testcase applicable is found more than one instance for testcase: \n".
             $self->{testcase_element}->toString()
            );
    }
    if($self->{testcase_element}->findnodes( $self->{xml_tags}->{applicable})->size and
        $self->{testcase_element}->findnodes( $self->{xml_tags}->{applicable}."/*" )->size){
        die("$Package: Error: testcase applicable tag should not have child tags: \n".
             $self->{testcase_element}->toString()
            );
    }
}

sub validate_testcase_params{
    my $self = shift;
    # testcase_params should have attribute - default
    #   If other filter type default is not available, then "default" will be used.
    #   for e.g. if basic_sanity_default is not available then it would use "default's value.
    # For params tag, check for type to exist compulsorily and it has only allowed types
    #   If type exists, then check for its pairing attributes to exist
    # Check that if "<applicable>" tag exists, 
    #   all attributes listed in "<applicable>" is of type=menu and has some defaults.
    #   all attributes listed in  "<applicable>" has its corresponding "<testcase_params>" or "<script_params>"
    # testcase_params should have only one instance of params as its child
        
    # Validate tag - testcase_params
    if($self->{testcase_element}->findnodes( $self->{xml_tags}->{testcase_params} )->size > 1){
        die("$Package: Error: testcase_params is found more than one instance for testcase: \n".
             $self->{testcase_element}->toString()
            );
    }
    if($self->{testcase_element}->findnodes( $self->{xml_tags}->{testcase_params})->size){
        # testcase_params should have child nodes
        if(!$self->{testcase_element}->findnodes( $self->{xml_tags}->{testcase_params}."/*" )->size){
            die("$Package: Error: testcase has empty testcase_params tag: \n".
                 $self->{testcase_element}->toString()
                );
        }

        my $applicable      = $self->get_applicable() if $self->is_applicable();
        my %applicableAttrs = ();
        %applicableAttrs = $self->getAttributesFromElement(element=>$applicable) if $self->is_applicable();
        
        my @elemArr = ();
        
        my $testcase_params = $self->get_testcase_params();
        foreach my $elem ($testcase_params->findnodes('*')->get_nodelist()){
            my $elemName = $elem->getName();
            push(@elemArr, $elemName);

            # every testcase_param entry should have attribute - default
            my %elemAttrs = $self->getAttributesFromElement(element=>$elem);
            if(!exists $elemAttrs{default}){
                die("$Package: Error: testcase_param has no attribute - 'default': \n".
                     $elem->toString()
                    );
            }
            
            # every testcase_param entry should have attribute - type and with correct value assigned to it
            if(!exists $elemAttrs{type}){
                die("$Package: Error: testcase_param has no attribute - 'type': \n".
                     $elem->toString()
                    );
            }
            if($elemAttrs{type} !~ /^\s*(range|checkbox|text|menu)\s*$/ig){
                die("$Package: Error: testcase_param has 'type' attribute with invalid value - '".$elemAttrs{type}."': \n".
                     $elem->toString()
                    );
            }
            
            # every testcase_param should have corresponding attr for attribute - "type"
            # for e.g if type==text, then attribute - "length" should exist
            # for type==range|menu|checkbox, then same named attribute should exist
            if($elemAttrs{type} =~ /text/i){
                if(!exists $elemAttrs{length} ){
                    die("$Package: Error: testcase_param has 'type' attribute as 'text', but 'length' attribute is missing \n".
                         $elem->toString()."\n"
                        );
                }
            } else {
                if(!exists $elemAttrs{$elemAttrs{type}}){
                    die("$Package: Error: testcase_param has 'type' attribute as '".$elemAttrs{type}.
                        "', but '".$elemAttrs{type}."' attribute is missing \n".
                         $elem->toString()."\n"
                        );
                }
            }
    
            # every attribute of applicable tag should exist as testcase_param of type - "menu"
            if( $self->is_applicable() and exists $applicableAttrs{$elemName}){
                if($elemAttrs{type} !~ /menu/i){
                    die("$Package: Error: applicable tag has attribute '".$elemName."' but this is not of type 'menu' \n".
                         $elem->toString()."\n"
                        );
                } else {                    
                    # this attribute of applicable tag is mapped to testcase_params, can be deleted.
                    delete $applicableAttrs{$elemName};
                }
            }
        }
        
        if(scalar(keys %applicableAttrs)){
            die("$Package: Error: following attributes of applicable tag has no mapping testcase_params or script_param \n".
                Dumper(\%applicableAttrs)."\n".$applicable->toString()."\n".
                "Testcase params:  ".$testcase_params->toString()."\n"
                );
        }
        
        # only one instance of testcase_params should exist
        if(scalar(@elemArr) != scalar($self->uniq(@elemArr))){
            die("$Package: Error: duplicate testcase_params are found in this testcase\n".
                "Testcase params:  ".$self->{testcase_element}->toString()."\n"
                );   
        }
    }
}

sub expand_testcase{
    my $self = shift;
    
    my $xmlroot = "<testcase/>";
    my $xp= XML::XPath->new($xmlroot);
    my @nodes = $xp->findnodes('*')->get_nodelist();
    my $parent = $nodes[0];
    $parent->appendChild($self->get_description());
    # TODO: Removed below line to save space in output xml
    # $parent->appendChild($self->get_applicable()) if $self->is_applicable();  
    
    # TODO: Removed below line to save space in output xml
    # $parent->appendChild($self->get_testcase_params());
    $parent=$self->setAttributesToElement(element=>$parent, attributes=>$self->get_testcase_attributes());

    $parent->appendChild($self->prepare_expanded_testcases());
    # my @nodeSet=$parent->findnodes('/');
    # $self->printXML(nodexml=>$nodeSet[0]);
    # $self->log(2, "$Package: Inf: Generated Data structure" );
    
    $self->set_expanded_testcase(testcase_element=>$parent); 
}

sub has_tag{
    my $self = shift;
    my %opts = validate(
        @_, {
            tag_name => { type => SCALAR },
        });
        
    my $tag_name = $opts{tag_name};
    return $self->{testcase_element}->findnodes( $self->{xml_tags}->{$tag_name})->size;
}

sub is_applicable{
    my $self = shift;
    return defined($self->{applicable});
}

sub set_description{
    my $self = shift;
    my %opts = validate(@_, {
        description  => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    $self->{description} = $opts{description}
}

sub set_applicable{
    my $self = shift;
    my %opts = validate(@_, {
        applicable => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    $self->{applicable} = $opts{applicable};
}

sub set_testcase_params{
    my $self = shift;
    my %opts = validate(@_, {
        testcase_params => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    # Fix : Memory leak
    undef $self->{testcase_params} if defined( $self->{testcase_params} );        
    $self->{testcase_params} = $opts{testcase_params};
}

sub set_testcase_attributes{
    my $self = shift;
    my %opts = validate(@_, {
        attributes => { type=>HASHREF},
    });
    $self->{testcase_attributes} = $opts{attributes};
}

sub set_expanded_testcase{
    my $self = shift;
    my %opts = validate(@_, {
        testcase_element => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    # Fix : Memory leak
    undef $self->{testcase_element} if defined( $self->{testcase_element} );
    $self->{testcase_element} = $opts{testcase_element};
}

sub get_filter{
    my $self = shift;
    return $self->{filter};
}

sub get_each_filter{
    my $self = shift;
    my @filterArr = ();
    my $filter = $self->get_filter();
    
    if( defined($filter) ){
        @filterArr = map{$_."_default"} split(/,/,$filter);
    } else {
        @filterArr = ( 'default' );
    }
    # Return unique entries from this array
    return $self->uniq(@filterArr);
}

sub get_description{
    my $self = shift;
    return $self->{description};
}

sub get_applicable{
    my $self = shift;
    return $self->{applicable};
}

sub get_testcase_params{
    my $self = shift;
    return $self->{testcase_params};
}

sub get_testcase_attributes{
    my $self = shift;
    return $self->{testcase_attributes};
}

sub get_script_params{
    my $self = shift;
    return $self->{script_params};
}

sub get_script_path{
    my $self = shift;
    return $self->{script_path};
}

sub get_expanded_testcase{
    my $self = shift;
    return $self->{testcase_element};
}

sub get_default_value_only_flag{
    my $self = shift;
    return $self->{default_value_only};
}

# Return testcase_params for given filter as hash
# Return format:
#   ( param1 => [value1],
#     param2 => [value2, value3],
#   )
sub get_params_hash{
    my $self = shift;
    my %opts = validate(@_, {
        filter  => { type=>SCALAR},
    });

    my $filter = $opts{filter};
    my $testcase_params = $self->get_testcase_params();

    my $is_default_only = $self->get_default_value_only_flag();

    my %outputHash=();
    foreach my $elem ($testcase_params->findnodes('*')->get_nodelist()){
        # Iterate every param element
        # if type="menu", then retain all menu items as separate array element
        # else, if textValue of element is not empty, then retain its value as key's value
        # else, if value for filter entry is available, then retain its value
        # else, retain value of attr "default"
        my %attrHash = $self->getAttributesFromElement(element=>$elem);
        my $textValue = $self->getTextNodeValue(element=>$elem);

        if($attrHash{type} =~ /menu/i and !$is_default_only){
            my @tmpArr = split(/,/, $attrHash{menu});
            $outputHash{$elem->getName()} = \@tmpArr;
        } else {
            my $value = exists $attrHash{$filter} ? $attrHash{$filter} : $attrHash{'default'};
            $value = $textValue !~ /^\s*$/mig ? $textValue : $value;
            $outputHash{$elem->getName()} = [$value];
        }
    }
    return %outputHash;
}

# Add script_params to testcase_params. Skip if already it exists in testcase_params
# update menu item defaults with value mentioned in <applicable> tag
sub prepare_testcase_params{
    my $self = shift;
    my $testcase_params = $self->get_testcase_params();
    my $script_params   = $self->get_script_params();

    # If testcase_params is not defined, consider only values from script_params    
    if(!defined $testcase_params){
        $testcase_params = XML::XPath::Node::Element->new("testcase_params");
    }
    
    # If script_params is not defined, consider only testcase_params
    if(defined $script_params){
        $self->log(3, "$Package: preparing_testcase_params: overriding script_params with testcase_params and pick superset");
        foreach my $elem ($script_params->findnodes('*')->get_nodelist()){
            # if(scalar($testcase_params->findnodes($elem->getName())->get_nodelist())){
            if($testcase_params->findnodes($elem->getName())->size){
                $self->log(3, "$Package: found element in testcase_params and will be retained without replacing with script_params: ".$elem->getName());
                next;
            }
            
            # # Fix: Memory Leak
            my $newElem = XML::XPath::Node::Element->new($elem->getName());
            my %attrs = $self->getAttributesFromElement(element=>$elem);
            $newElem = $self->setAttributesToElement(element=>$newElem, attributes=>\%attrs);
            
            $testcase_params->appendChild($newElem);
            undef $elem; # Fix : Memory leak            
            undef $newElem; # Fix : Memory leak
        }
    } else {
        $self->log(3, "$Package: prepare_testcase_params: script_params not found and is not added to testscript_params");
    }

    my $applicable      = $self->get_applicable() if $self->is_applicable();
    my %applicableAttrs = ();
    %applicableAttrs = $self->getAttributesFromElement(element=>$applicable) if $self->is_applicable();

    foreach my $elem ($testcase_params->findnodes('*')->get_nodelist()){
        $testcase_params->removeChild($elem);

        # TODO: Fix for <applicable> tag.
        # if exists $applicableAttrs{$elem->getName()}, i.e. paramName is part of %applicableAttrs, then
        #   if $elem is of type==menu and items in %applicableAttrs is part of menu="item1,..." , then
        #     # may be check for %applicableAttrs's being value present in menu-list. can happen in validation logic itself
        #     replace all *default of this $elem with the value of $applicableAttrs{$elem->getName()}
        my $elemName = $elem->getName();
        my %elemAttrs = $self->getAttributesFromElement(element=>$elem);
        my %newElemAttrs=();
        if( $self->is_applicable() and exists $applicableAttrs{$elemName} and $elemAttrs{type} =~ /menu/i  ){
            my @appAttrVal = split(/,/, $applicableAttrs{$elemName});
            my @elemAttrVal = split(/,/, $elemAttrs{menu});
            
            # all values mentioned in applicable attribute's value should be 
            #   present as testcase_param's menu's value
            foreach my $item (@elemAttrVal){
                @appAttrVal = @{$self->remove_array_item(array=>\@appAttrVal, item=>$item)};
            }
            if(scalar(@appAttrVal)){
                die("$Package: Error: following values of applicable tag is not part ".
                    "of testcase_params or script_params values \n\n".
                    "Attribute: $elemName\n"."Attribute value: @appAttrVal"."\n\n".
                    "Testcase params:  ".$testcase_params->toString()."\n\n".
                    "elem: ".$elem->toString()."\n\n".
                    "Testcase tag: ".$self->{testcase_element}->toString()."\n\n"
                    );
            }
            
            my %newElemAttrs = $self->updateValueForHashKeys(
                originalHash=>\%elemAttrs, 
                keyRegex=>"default|menu", 
                newValue=>$applicableAttrs{$elemName}
                );
            $elem = $self->setAttributesToElement(element=>$elem, attributes=>\%newElemAttrs);
        }
        $self->log(3, "$Package: New testcase_param tag after expanding it with <applicable> value: ".$elem->toString());
        $testcase_params->appendChild($elem);
        undef $elem; # Fix : Memory Leak
    }
    
    $self->set_testcase_params(testcase_params=>$testcase_params);
}

sub prepare_expanded_testcases{
    my $self = shift;
    
    my $expandedTCs = XML::XPath::Node::Element->new("expanded_testcases");
    my $testcase_params = $self->get_testcase_params();
    
    # Algorithm...
    # For each filter:
    #   gen key_value_pairs(TCs, Filter)
    #       { P1 => [ A ],
    #         P2 => [ B, C ],
    #         P3 => [ D, E, F ]
    #       }
    #   For each $TC in expandedTCList(element){
    #       populate expanded_testcases->appendChild($TC) 
    foreach my $filter ($self->get_each_filter()){
        # print "###=>",$filter,"\n";
        my %hash = $self->get_params_hash(filter=>$filter);
        # print Dumper(\%hash);
        
        my @dsArr=$self->get_key_value_array(key_value_hash=>\%hash);
        # print "key_value_pairs==>  ",Dumper(\@dsArr);

        my @outputArr = ();
        foreach my $arr (@dsArr){
            @outputArr = $self->multiple_two_arrays(array1=>$arr, array2=>\@outputArr);
        }
        
        my @key_value_arr = $self->get_combined_key_value_array(key_value_array=>\@outputArr);
        # print "KV_HASH====> ",Dumper(\@key_value_arr);
        
        foreach my $tcElem ($self->get_expanded_tc_elem(key_value_array=>\@key_value_arr)){
            $expandedTCs->appendChild($tcElem);
        }
    }
    return $expandedTCs;    
}

sub get_expanded_tc_elem{
    my $self = shift;
    my %opts = validate(@_, {
            key_value_array => {type=>ARRAYREF},
        });
        
    my @kvArr = @{$opts{key_value_array}};
    my @tcElemArr = ();
    my $script_path = $self->getTextNodeValue(element=>$self->get_script_path());
    foreach my $hashRef ( @kvArr){
        my $tcElem = XML::XPath::Node::Element->new("tc");
        my $tcStr = "";
        foreach my $key ( keys(%{$hashRef})){
            $tcStr = $tcStr." $key=".$hashRef->{$key}."  ";
        }
        $tcStr = $tcStr." ".$script_path;
        
        my $textNode = XML::XPath::Node::Text->new($tcStr);
        $tcElem->appendChild($textNode);
        $tcElem = $self->setAttributesToElement(element=>$tcElem, attributes=>$hashRef);
        push(@tcElemArr, clone($tcElem));
        undef $tcElem; # Fix : Memory leak
    }
    
    # If there are no testcase_params or script_params, then it is just the script path as TC
    if(!scalar(@tcElemArr)){
        my $tcElem = XML::XPath::Node::Element->new("tc");
        my $textNode = XML::XPath::Node::Text->new($script_path);
        $tcElem->appendChild($textNode);
    
        push(@tcElemArr, clone($tcElem));
        undef $tcElem; # Fix : Memory leak
    }
    
    return @tcElemArr;
}

sub get_key_value_array{
    my $self = shift;
    my %opts = validate(@_, {
            key_value_hash => {type=>HASHREF},
        });

    # This translates from source format:
    #       { P1 => [ A ],
    #         P2 => [ B, C ],
    #         P3 => [ D, E, F ]
    #       }
    # To destination format:
    #       [ 
    #           [ {P1=>A} ],
    #           [ {P2=>B}, {P2=>C}],
    #           [ {P3=>D}, {P3=>E}, {P3=>F}]
    #       ]
    my %key_value_hash = %{$opts{key_value_hash}};
    
    my @dsArr = ();
    foreach my $key ( keys %key_value_hash){
        my @tmpArr = ();
        foreach my $val ( @{$key_value_hash{$key}} ){
            push(@tmpArr, {$key=>$val});
        }
        push(@dsArr, \@tmpArr);    
    }
    return @dsArr;
}

sub get_combined_key_value_array{
    my $self = shift;
    my %opts = validate(@_, {
            key_value_array => {type=>ARRAYREF},
        });
    # This translates from source format:
    #       [ 
    #           [ {P1=>A}, {P2=>B} ],
    #           [ {P1=>A}, {P2=>C} ],
    #       ]
    # To destination format:
    #       [ 
    #           {P1=>A, P2=>B},
    #           {P1=>A, P2=>C},
    #       ]
    
    my @kvArr = @{$opts{key_value_array}};
    my @dsArr = ();
    foreach my $arrRef ( @kvArr){
        my %tmpHash = ();
        foreach my $hashRef ( @{$arrRef} ){
            foreach my $key (keys %{$hashRef}){
                $tmpHash{$key} = $hashRef->{$key};
            }
        }
        push(@dsArr, clone(\%tmpHash));
        undef %tmpHash; # Fix : Memory leak
    }
    return @dsArr;
}

sub multiple_two_arrays{
    my $self = shift;
    my %opts = validate(@_, {
            array1 => {type=>ARRAYREF},
            array2 => {type=>ARRAYREF}
        });

    # Start with key_value_pair:Arr ...
    # [ [A], [B, Q], [D, E, F] ]
        
    # Format of array1:
    #   [ D, E, F] 
    # Format of array2:
    #   [  [A, B], [A, Q] ]
    # This results in 
    #  [ [A, B, D], [A, Q, D],
    #    [A, B, E], [A, Q, E],
    #    [A, B, F], [A, Q, F],
    #  ]
    
    my @arr1 = @{$opts{array1}};
    my @arr2 = @{$opts{array2}};

    if(scalar(@arr1)==0 and scalar(@arr2)==0){
        return ();
    } elsif(scalar(@arr1)==0){
        return map{[$_]} @arr2;
    } elsif(scalar(@arr2)==0){
        return map{[$_]} @arr1;
    } 
    
    my @outputArr;
    for(my $i=0;$i<scalar(@arr1);$i++){
        for(my $j=0;$j<scalar(@arr2);$j++){
            my $val1 = clone($arr2[$j]);
            push(@outputArr, [@$val1, clone($arr1[$i])]);
        }
    }
    return @outputArr;
}

1;

package Testplan;
use XML::XPath;
use Clone qw(clone);
use Data::Dumper;
use Params::Validate qw(:all);
use POSIX qw(strftime);

our @ISA = qw(Utility);

my $Package = "Testplan";

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    
    my %opts = validate(
        @_, {
            debug_level         => { type => SCALAR, optional => 1, default => 1},
        }
    );
    my $self = $class->SUPER::new(debug_level=>$opts{debug_level});    
    %opts = (%{$self}, %opts);
    $self = \%opts;
    bless $self, $class;
    
    return $self;
} 

sub buildDotPlan{
    my $self = shift;
    my %opts = validate(
        @_, {
            nodexml             => { type=>OBJECT|UNDEF, optional=>1},
            filename            => { type => SCALAR | UNDEF, optional => 1 },
            xmlfile             => { type => SCALAR | UNDEF, optional => 1 },
            params_filter       => { type => SCALAR | UNDEF, optional => 1 },
            filter              => { type => SCALAR | UNDEF, optional => 1 },
            default_value_only  => { type => SCALAR, optional => 1, default => 0},
            debug_level         => { type => SCALAR, optional => 1, default => 1},
        }
    );
    my $nodexml = $opts{nodexml};
    my %filterHash = ();
    if(defined $opts{params_filter}){
        %filterHash = $self->convert_str_to_hash(string=>$opts{params_filter}, delimiter=>";");
    }
    
    # init testplan string
    # init tc_counter
    # loop through all <test_script>
    #   loop through all <testcase>
    #       loop through all <expanded_testcases/tc>
    #           if <tc> has required "params_filter"
    #               increment tc_counter
    #               concat commented "description"
    #               generate TC string ( suffix qc_name if exists, else suffix id)
    #               concat TC string 
    # Attach header to testplan string
    # save to file

    my %xml_tags=();
    $xml_tags{test_script} = "./test_script";
    $xml_tags{testcase} = "./testcase";
    $xml_tags{description} = "./description";
    $xml_tags{tc} = "./expanded_testcases/tc";
    
    my $dotplan = "";
    my $tc_count = 0;
    
    foreach my $testscript_element ( $nodexml->findnodes($xml_tags{test_script})->get_nodelist() ) {
        foreach my $testcase ( $testscript_element->findnodes($xml_tags{testcase})->get_nodelist() ) {
            my @arr = $testcase->findnodes($xml_tags{description})->get_nodelist();
            my $description = join("\n## ",split(/[\n\r]/, $self->getTextNodeValue(element=>$arr[0])));
            foreach my $tc ( $testcase->findnodes($xml_tags{tc})->get_nodelist() ) {
                my %testcaseAttrs = $self->getAttributesFromElement(element=>$testcase);
                my $tcStr = $self->getTextNodeValue(element=>$tc);
                my $id = exists $testcaseAttrs{qc_name} ? $testcaseAttrs{qc_name} : $testcaseAttrs{id};
                $tcStr = $tcStr." -testcase ".$id;
                $tcStr = $tcStr." TESTCASES=".$testcaseAttrs{id};

                my %tcAttrs = $self->getAttributesFromElement(element=>$tc);
                if(defined $opts{params_filter}){
                    # TODO: Currently this is in "OR" case, even if one of the params_filter exists, it is added to dotplan
                    # This can be made as "AND" Case by keeping found_counter to see all keys of filterHash match tcAttrs
                    my $found_counter=0;
                    foreach my $key (keys %filterHash){
                        my $filters = join('$|^', split(/,/, $filterHash{$key}));
                        if(exists $tcAttrs{$key} and $tcAttrs{$key} =~ /^$filters$/){
                            $found_counter++;
                        }
                    }
                    if($found_counter == scalar(keys %filterHash)){
                        $dotplan = $dotplan."## ".$description."\n".$tcStr."\n\n"; 
                        $tc_count++;
                    }
                } else {
                    $dotplan = $dotplan."## ".$description."\n".$tcStr."\n\n";
                    $tc_count++;
                }
            }
        }
    }

    my $filename = defined $opts{filename} ? $opts{filename} : "User created dotplan";
    my $xmlfile = defined $opts{xmlfile} ? $opts{xmlfile} : "No name mentioned";
    my $params_filter = defined $opts{params_filter} ? $opts{params_filter} : "No config filter mentioned";
    my $filter = defined $opts{filter} ? $opts{filter} : "No filter mentioned";
    my $default_value_only = $opts{default_value_only} ? "Yes" : "No";
    my $createDate = strftime "%Y-%m-%d %H:%M:%S %Z", localtime;
    my $topLine = "#"x50;
    my $header = $topLine."\n".
        "##  Name          : ".$filename."\n".
        "##  TC Count      : ".$tc_count."\n".
        "##  Filter        : ".$filter."\n".
        "##  Config Filter : ".$params_filter."\n".
        "##  Defaults only : ".$default_value_only."\n".
        "##  Create On     : ".$createDate."\n".
        "##  XMLplan File  : ".$xmlfile."\n".
        $topLine."\n\n";

    $dotplan = $header.$dotplan."\n";
    return $dotplan;
}

sub generateDotPlan{
    my $self = shift;
    my %opts = validate(
        @_, {
            nodexml             => { type=>OBJECT|UNDEF, optional=>1},
            filename            => { type => SCALAR | UNDEF, optional => 1},
            xmlfile             => { type => SCALAR, optional => 1},
            overwrite           => { type => SCALAR, optional => 1},
            params_filter       => { type => SCALAR | UNDEF, optional => 1 },
            filter              => { type => SCALAR | UNDEF, optional => 1 },
            default_value_only  => { type => SCALAR, optional => 1, default => 0},
            debug_level         => { type => SCALAR, optional => 1, default => 1},
        }
    );
    my $dotplan = $self->buildDotPlan(
        nodexml             => $opts{nodexml},
        filename            => $opts{filename},
        xmlfile             => $opts{xmlfile},
        params_filter       => $opts{params_filter},
        filter              => $opts{filter},
        default_value_only  => $opts{default_value_only},
        debug_level         => $opts{debug_level}
    );
    if(defined $opts{filename}){
        $self->writeToFile(
            filename    =>$opts{filename},
            text        =>$dotplan,
            overwrite   =>$opts{overwrite},
            debug_level =>$opts{debug_level}
        );
    }
    return $dotplan;
}

sub writeToFile{
    my $self = shift;
    my %opts = (
        @_, {
            filename        => { type => SCALAR, optional => 0},
            text            => { type => SCALAR, optional => 0 },            
            overwrite       => { type => SCALAR, optional => 1, default => 0},            
            debug_level     => { type => SCALAR, optional => 1, default => 1},
        }
    );
    
    my $mode = $opts{overwrite} ? ">" : ">>";
    open( DOTPLAN, $mode, $opts{filename} ) or die("$Package: Error: writing to dotplan file failed\n");
    print DOTPLAN $opts{text};
    close DOTPLAN;
}

sub generate_filtered_XML{
    my $self = shift;
    my %opts = validate(
        @_, {
            nodexml         => { type=>OBJECT|UNDEF, optional=>1},
            params_filter   => { type => SCALAR | UNDEF, optional => 1 },            
            debug_level     => { type => SCALAR, optional => 1, default => 1},
        }
    );

    my $nodexml = $opts{nodexml};
    my %filterHash = ();
    if(defined $opts{params_filter}){
        %filterHash = $self->convert_str_to_hash(string=>$opts{params_filter}, delimiter=>";");
    }
    
    my %xml_tags=();
    $xml_tags{global_dependency} = "./global_dependency";
    $xml_tags{script_dependency} = "./dependency";
    $xml_tags{test_script} = "./test_script";
    $xml_tags{tc} = "./testcase/expanded_testcases/tc";
    
    my $xmlroot = XML::XPath::Node::Element->new("xmlplan");
    if($nodexml->findnodes( $xml_tags{global_dependency} )->size){
        if($nodexml->findnodes( $xml_tags{global_dependency} )->size > 1){
            die("$Package: Error: global_dependency is found more than one instance in this xmlfile\n");
        }
    
        my @globalDeps = $nodexml->findnodes( $xml_tags{global_dependency} )->get_nodelist();
        $self->validate_dependency(element=>$globalDeps[0], type=>"global_dependency");
        $xmlroot->appendChild($globalDeps[0]);
    }
    
    my $ts_count = 0;
    foreach my $testscript_element ( $nodexml->findnodes($xml_tags{test_script})->get_nodelist() ) {
        my $tc_count = 0;
        my $test_script = XML::XPath::Node::Element->new("test_script");
        if($testscript_element->findnodes( $xml_tags{script_dependency} )->size){
            my @scriptDeps = $testscript_element->findnodes( $xml_tags{script_dependency} )->get_nodelist();
            $self->validate_dependency(element=>$scriptDeps[0], type=>"test_script/dependency");
            $test_script->appendChild($scriptDeps[0]);
        }
        foreach my $tc ( $testscript_element->findnodes($xml_tags{tc})->get_nodelist() ) {
            my %tcAttrs = $self->getAttributesFromElement(element=>$tc);
            if(defined $opts{params_filter}){
                my $found_counter=0;
                foreach my $key (keys %filterHash){
                    my $filters = join('$|^', split(/,/, $filterHash{$key}));
                    if(exists $tcAttrs{$key} and $tcAttrs{$key} =~ /^$filters$/){
                        $found_counter++;
                    }
                }
                if($found_counter == scalar(keys %filterHash)){
                    $test_script->appendChild(clone($tc));
                    undef $tc; # Fix : Memory leak
                    $tc_count++;
                }
            } else {
                $test_script->appendChild(clone($tc));
                undef $tc; # Fix : Memory leak
                $tc_count++;
            }            
        }
        
        if($tc_count){
            $xmlroot->appendChild(clone($test_script));
            undef $test_script; # Fix : Memory leak
            $ts_count++;
        }
    } 
    if(!$ts_count){
        die("$Package: Error: Some error with 'params_filter'. Not even a single TC exists with this filter. \n".
            "Filter: ".$opts{params_filter}."\n");
    }
    return $xmlroot;
}

sub validate_dependency{
    my $self = shift;
    my %opts = validate(
        @_, {
            element => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
            type => { type => SCALAR, default=>"global_dependency"},   # possible values; global, test_script
        }
    );
    my $element = $opts{element};
    my $type = $opts{type};
    my %xml_tags=();

    # Validate <global_dependency/spare_disks>
    if($element->findnodes( "./spare_disks" )->size != 1){
        die("$Package: Error: <$type/spare_disks> is either not found or ".
            "more than one instance is found in this xmlfile: \n"
        );
    }
    my $diskTag = "./spare_disks/disk";
    if(!$element->findnodes( $diskTag )->size){
        die("$Package: Error: <$type/spare_disks/disk> is not found in this xmlfile: \n".
            $element->toString()."\n"
        );
    }

    foreach my $disk ($element->findnodes($diskTag)->get_nodelist()){
        my %diskAttrs = $self->getAttributesFromElement(element=>$disk);
        # TODO: If pool requirement has to be removed then, modify below line. Remove pool & compare keys with 3.
        if(scalar(grep /^\s*(pool|count|storage_type|checksum_type)\s*$/i, keys(%diskAttrs)) !=4){
            die("$Package: Error: One of the required attribute is missed in <$type/spare_disks/disk>: \n".
                Dumper(\%diskAttrs)."\n".
                "Required attributes are: pool, count, storage_type, checksum_type"."\n"
            );
        }
        if(exists $diskAttrs{pool} and $diskAttrs{pool} !~ /0|1/){
            die("$Package: Error: 'pool' attribute has invalid value in <$type/spare_disks/disk>: \n".
                Dumper(\%diskAttrs)."\n"."Correct value: 0,1"."\n"
            );
        }
        if(exists $diskAttrs{count} and int($diskAttrs{count}) < 2){
            die("$Package: Error: 'count' attribute has invalid value in <$type/spare_disks/disk>: \n".
                Dumper(\%diskAttrs)."\n"."Correct value: >= 2"."\n"
            );
        }
        # TODO: May be similarly add validation for checksum_type and storage_type. Can this be skipped ?
    }
}

1;

package Utility;
use Data::Dumper;
use Params::Validate qw(:all);

sub new {
    my $class = shift;
    my $self = validate(
        @_, {
            debug_level => { type => SCALAR, optional => 1, default => 0},
        }
    );
    bless $self, $class;
    return $self;
} 

# Print the info
sub print {
    my $self = shift;
    my (@output) = @_;
    print @output;
}

# Print the log information  based on the debug level
sub log {
    my $self = shift;
    my ( $level, @output ) = @_;
    if ( $level <= $self->{debug_level} ) {
        print "    " x ( $level - 1 );
        print @output;
        print "\n";
    }
} 

# Return unique entries from input array
sub uniq{
    my $self = shift;
    my %hash;
    return grep { !$hash{$_}++ } @_;
}

sub printXML {
    my $self = shift;
    my %opts = validate(@_, {
        xmlString  => { type=>SCALAR|UNDEF, optional=>1},
        nodexml => { type=>OBJECT|UNDEF, optional=>1},
    });
    
    eval {
        if(defined $opts{xmlString}){
            my $xml = XML::Twig->new(pretty_print=>"indented");
            $xml->parse($opts{xmlString});
            $xml->set_indent("   ");
            $xml->print();
        }elsif(defined $opts{nodexml}){
            my $xml=XML::XPath::XMLParser::as_string($opts{nodexml});
            $self->printXML(xmlString=>$xml);
        }
    };
    if($@){
        $self->log(2, "$Package: Error; $@");
    }
    return 1;
}

sub getAttributesFromElement{
    my $self = shift;
    my %opts = validate(@_, {
        element => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    my @attrs = $opts{element}->getAttributes();
    my %attrHash = ();
    foreach my $attr (@attrs){
        $attrHash{$attr->getName()} = $attr->getValue();
    }
    return %attrHash;
}

sub setAttributesToElement{
    my $self = shift;
    my %opts = validate(@_, {
        element => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
        attributes => { type=>HASHREF},
    });
    my $elem = $opts{element};
    my %attrHash = %{$opts{attributes}};
    foreach my $key (keys %attrHash){
        my %existingAttr = $self->getAttributesFromElement(element=>$elem);
        if(exists $existingAttr{$key}){
            $elem->removeAttribute($key)
        }
        my $childattr=XML::XPath::Node::Attribute->new($key, $attrHash{$key});
        $elem->appendAttribute($childattr);        
    }
    return $elem;
}

sub getTextNodeValue{
    my $self = shift;
    my %opts = validate(@_, {
        element => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    
    my $element = $opts{element};
    my $textValue;
    if($element->findnodes('*')->size){
        my @tmpNodes = $element->findnodes('*');
        $textValue = $tmpNodes[0]->toString();
    }else{
        $textValue = $element->string_value();
    }        
    return $textValue;
}
# Replace value for all keys matching keyRegex
# %originalHash = (val1=>"1", val2=>"2", default=>"3", key1=>"4", key2=>"5")
# $keyRegex = "default|val"
# $newValue = "test"
# Return value:
# %newHash = (val1=>"test", val2=>"test", default=>"test", key1=>"4", key2=>"5")
sub updateValueForHashKeys{
    my $self = shift;
    my %opts = validate(@_, {
        originalHash => { type=>HASHREF },
        keyRegex     => { type=>SCALAR },
        newValue     => { type=>SCALAR }
    });
    
    my %originalHash = %{$opts{originalHash}};
    my $keyRegex     = $opts{keyRegex};
    my $newValue= $opts{newValue};
    
    # Get all attrs that has suffix as "default"
    my @defaultAttrs = grep (/$keyRegex/i, keys %originalHash );
    
    # Update new value to attribute's with suffix as "default"
    my %newHash = map{ $_=>$newValue  } @defaultAttrs;
    
    # Merge these attrs have suffix "default" with remaining atts to create super set of attrs
    %newHash = ( %originalHash, %newHash );
    return %newHash;
}

# Trims preceeding and ending white spaces in the string
sub trim {
    my $self = shift;
    print "I am in utility";
    my $str  = shift;
    $str =~ s/^\s*//;
    $str =~ s/\s*$//;
    return $str;
} 

# To check if the current node is an XML Node Element
sub is_node_element {
    my $self = shift;
    my ($node) = @_;
    if ( $node == undef ) {
        return 0;
    }
    return $node->isa("XML::XPath::Node::Element");
} 

sub remove_array_item{
    my $self = shift;    
    my (%opts) = validate(@_, {
                            array => {type=>ARRAYREF},
                            item  => {type=>SCALAR},
                            });  
    my $arrayref = $opts{array};
    my $item = $opts{item};

    my $index = 0;          
    while ($index <= $#{$arrayref} ) {
        my $value = ${$arrayref}[$index];
        if ( $value eq $item ) {
            splice @{$arrayref}, $index, 1;
        } else {
            $index++;
        }    
    }    
    return $arrayref;
}

# Convert a=b,c=d or a=b;c=d to {a=>b, c=>d}
sub convert_str_to_hash{
    my $self = shift;
    my (%opts) = validate(
        @_, {
            string      => {type=>SCALAR},
            delimiter   => {type=>SCALAR, optiona=>1, default=>","},
        }
    );  

    my $delimiter = $opts{delimiter};
    my %filterHash = ();
    foreach my $item (split(/$delimiter/, $opts{string})){
        my @arr = split(/=/, $item);
        $filterHash{$arr[0]} = $arr[1];
    }
    return %filterHash;
}

sub read_file{
    my $self = shift;
    my %opts = validate(
        @_, {
            filename => { type => SCALAR }
        }
    );
    my $filename = $opts{filename};
    open FILE, $filename or die $!;
    my $output = join("", <FILE>);
    return $output;    
}

1;
