# this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Xen/XenParams.pl#1 $
#
# Copyright (c) 2004 Network Appliance, Inc.
# All rights reserved.
#
## @summary
##    Provides a set of common parameters, with their defaults, that
##    thpls may choose to instantiate.
##
## @description
##    This file provides a list of commonly used parameters. .thpls that
##    want to make use of these parameters need only include a 
##    "require XEN/XenParams.pl" line in their .thpl after defining 
##    params used locally in the thpl.
##
##    To ease in other test developers using this file, please add new params
##    in alphabetical order both in the Tharndoc section, and in the vars array.
##
## @param BUILDROOT	    Required. The location of your build
##
## @param FILER		    Required. The filer name or hostspec
##
## @param KERNELGDB         Optional. Determines whether the .thpl should
##			    attempt to get a GDB stacktrace when badness 
##			    occurs.
##			    Options: <0|1>
##			    Default: 1, meaning obtain a stacktrace
##
## @example
##     This is a code snippet from a .thpl that wants to utilize the params 
##     defined in this XenParams.pl file. If the default value of a 
##     param defined in XenParams.pl needs to be modified for use in a
##     specific .thpl, then the .thpl must provide the new default *before*
##     importing the Xen/XenParams.pl file.
##
##     The BEGIN block is necessary.
##
##     BEGIN {
##        use vars qw($KERNELGDB);
##        param("KERNELGDB", "-default", "0");
##
##        # import in the rest of the common parameters and
##        # their default values
##        require "Xen/XenParams.pl";
##     };
##
## @keywords  tricky XEN common parameters
##
## @status    design
##
## @author    ericb@netapp.com
##


use strict;
use Tharn;
use vars qw(
	    $BUILDROOT
	    $FILER
	    $KERNELGDB
	    );

param('BUILDROOT', '-mesg', 'The location of your build is required.');
param('FILER','-mesg', 'A Filer hostspec is required.');
param("KERNELGDB", "-default", 1);


##############################################
# Do not delete anything below this line
##############################################
1;

