# Hey emacs, this is basically -*- perl -*-
# $Id$
#
#Copyright (c) 2002 Network Appliance, Inc.
#All rights reserved.
#===============================================================================
#  //depot/prod/test/main/lib/ZapiCli.pm
##
## @summary Framework for ZAPI/CLI Validation using XML based inputs
## @usage
##     use ZapiCli.pm;
##
## @keywords Zapi CLI
## @author Seshadri Nagaraja Seshadri.Nagaraja@netapp.com
##
## @burt 354821 'ZapiCli framework'
##
## @status new
##
## @change 2009/04/16 nsesha : 'Initial Version of ZapiCli.pm.'
## @change 2009/04/17 nsesha : 'updated validateXML( ) to support multiple values in attributes - @array and @key'
## @change 2009/04/17 nsesha :  Updates & Defect fixes:
##           1.  Fixed logic error in <expectedResults> and <notExpectedResults>
##               introduced due to string updations and few conditional statements update in
##               previous version.
##           2. Fixes to comparatorCLI( ):  Changed the position of quotemeta to fix few
##              issues with comparatorXML( ) found during attributes comparison.
##           3. Support for Zapi:
##              Added executeZAPI
##              updated validateXML( ) to support zapi tags
##              Added required logic in validateZAPI - both zapi command based execution
##              and <inputxml> based execution.
## @change 2009/04/17 nsehsa : correction in validateXML().
##           Regex quote meta for @array attributes in <values> and <required> having indexed hashOfArrays.
## @change 2009/04/17 nsesha : Defect fix in executeZAPI.
##           Errors with $username and $password.
##           Fix helps to run invoke_elem and invoke_api without errors.
## @change 2009/04/20 prashanc: Added random_name_generate() function.
## @change 2009/04/21 nsesha  : comparatorXML.
##                   Multi-node comparison : attributes - type="zapi", attributes="true", text="true
##                   <prompt> feature attributes:   choice="text" <command interactive="true">
##                   updated validateXML( )
## @change 2009/04/21 nsesha  : minor changes to fix merge issues.
## @change 2009/04/24 reddyk  : Fixed defect in pathsPopulate: to remove unwanted paths,
##                              and other minor changes regarding the same.
## @change 2009/04/27 reddyk  : Fixed bug in <NotExpected><Regex> validation.
## @change 2009/04/28 sdivya  : Fixed bug in pathsPopulate() subroutine.
## @change 2009/04/29 sdivya  : checked in multinode support for regex tags.
##              included optional parameter regex=true in comparatorCLI & comparatorXML.
##              Extreme caution was taken to prevent change in previous functionality of comparatorCLI.
##              Also removed some unwanted logcomments.
## @change 2009/04/30 sujathat: prompt feature has been made generic.
## @change 2009/05/04 nraju   : Fixed bug in setting ENV variable in executeZAPI().
## @change 2009/05/04 nsesha  : few indentation changes.
## @change 2009/06/17 nraju   : Enhancement provided as follows:
##                Included a new argument to invoke_zapi_cli( ) =>  "filerconn"
##                which will receive the connection object from the .thpl script.
##                The framework now executes cli commands in different formats based on varying "filermode"
##              parameter to invoke_zapi_cli( ).
##                 a.       NGSH (Cluster environment in 10 mode) : filermode    => "10modeNGSH "
##                 b.       NODESCOPE (Non cluster environment in 10 mode): filermode    => .10modeNODESCOPE.
##                 c.       7 Mode Maintenance  :     filermode    => .7modeMAINT.
##                Fix provided for prompt feature of 10Mode NGSH and NODESCOPE .
##                Included support for switching between NGSH and NODESCOPE for execution of NODESCOPE commands and
##                returning back to NGSH .
## @change 2009/06/17 nraju   : 7 mode Maintenance feature related issue was fixed.
## @change 2009/06/30 nraju   : BR/IC independent XML selection feature for thpl files(7-mode).
## @change 2009/07/03 nraju   : Added documentation to every sub-routine.
## @change 2009/07/13 nraju   : BR/IC independent XML selection feature added for 7-mode maintenance
## @change 2009/08/10 nraju   : omiting GMT message removal.
##                              Changing regex for DBG messages removal.
##                              Fix for handling NULL parameter in zapi TC's
## @change 2009/08/14 nraju   : added new parameter "debug" to invoke_zapi_cli()
##                              added new subroutine logcommentXML() to log xmlstrings using Data::Dumper.
## @change 2010/11/30 nagajyot  : RR/BR independent XML selection feature for thpl files(7-mode).
## @change 2010/12/06 nagajyot  : Changed maintenance mode prompt comparision for local function _get_ontap_version().
## @change 2011/03/14 nraju     : added "use re 'eval';" to override eval errors during regex match.
## @change 2013/04/06 ganji     : added get version for NORMAL and MAINT modes in _get_ontap_version().
## @change 2013/04/20 ganji     : Added _get_default_username() in invoke_zapi_cli() to get default username for 7mode & Cmode.

use re 'eval';
use strict;
use XML::XPath;
use XML::Simple;
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF GLOBREF);
use Data::Dumper;
use File::Temp;
use ZapiTest;
use Filer::Version qw (get_ontap_version codename_to_version);
use dblade::wafl::tools::panamax::Version qw(build_version);
my $DEBUG=0;

#-----------------------------------------------------------------------------------------------------------------
## @name ::    invoke_zapi_cli()
## @summary executes a given command on the cli interface or using Zapi call and returns matches the output,
##              with the set of values provided in the predefined tags provided in the XML file.
## @arg $cliSignature Required:  path to the xml file containing the tags necessary for validating the command output.
## @arg $host Required: The host object to the filer if #rsh connection is used( for 7mode execution ).
##                       becomes a mandatory parameter if (filermode eq "normal").
## @arg $testStepId Required: The teststep id of the TC that has to betested.
## @arg \%hashOfVars Optional: reference to hash of scalar and/or arrays items which will get replaced in the
##                              <command> tag of the teststep under execution.
## @arg \%hashOfRefs Optional: reference to hash of References to functions that returns an array of scalar items
## @arg \%hashOfArrays Optional: reference to hash of arrays having scalar items
## @arg $filermode optional:  normal 7G and 7mode normal(UP state) execution
##                            7modeMAINT  for executing command in 7-mode maintenance prompt.
##                            10modeNGSH   executing command in 10-mode(cluster mode) NG shell.
##                            10modeNODESCOPE   for executing command in 10-mode(cluster mode) Nodescope.
##                            10modeMAINT  for executing command in 10-mode(cluster mode) maintenance prompt.
##                      Default is normal
## @arg $conntype  optional:  The connection type required for executing the command.
##                      Default  #rsh for (filermode eq "normal").
##                      Default  #console for all other cases.
## @arg $filerconn optional:  The filer connection object.(must be a console connection object)
##                      A mandatory parameter if (filermode ne "normal").
## @arg $username optional: username to send for Zapi Call.
##                      -Default is "root".
## @arg $password optional: Password to send to Zapi Call.
##                      -Default is "".
## @arg $debug    optional: set to 1 to enable extra logcomments for debugging purposes.
##                      -Default 0
## @Returns    The exact CLI output with out formatting or extracting values.
##             For #console connections syslog messages appear inline along with the actual output. Therefore
##             Those GMT,DBG messages are removed explicitly. But this fix is not complete when running on
##             maintenance mode prompt.
#-----------------------------------------------------------------------------------------------------------------
sub invoke_zapi_cli{
    my %DEFAULTHASH;
    my %opts = validate(@_, {
        cliSignature => { type=>SCALAR, default=>"input.xml"},
        teststepId   => { type=>SCALAR, default=>undef},
        hashOfVars   => { type=>HASHREF, default=>\%DEFAULTHASH, optional=>1},
        hashOfArrays => { type=>HASHREF, default=>\%DEFAULTHASH, optional=>1},
        hashOfRefs   => { type=>HASHREF, default=>\%DEFAULTHASH, optional=>1},
        host         => { type=>OBJECT, isa=>'Hostrec' },
        filermode    => { type=>SCALAR, default=>"normal", optional=>1 },
        conntype     => { type=>SCALAR, default=>"#rsh",optional=>1},
        filerconn    => { type=>HASHREF, default=>\%DEFAULTHASH, optional=>1},
        debug        => { type=>SCALAR, default=>0,optional=>1},
        # Following params are required for "zapis"
        username     => { type=>SCALAR, default=>"root", optional=>1 },
        password     => { type=>SCALAR, default=>"",optional=>1},
        });

    # Set DEBUG=1 to enable debug messages
    $DEBUG = $opts{debug};

    # Validate whether the TestStep ID exists in XML
    my $xpathVar = XML::XPath->new(filename => $opts{cliSignature});
    my $tstNodes = $xpathVar->find('//@'.$opts{teststepId});
    if($tstNodes->size == 0){
        logcomment 'Test Step ID:'.$opts{teststepId}.' not found',
             'Modifying Test Step ID for build specific testcase';

        my $vers = _get_ontap_version($opts{filermode});
        $Log->comment("The ontap version on this build is : $vers");
        if (int($vers) == 7) {
            $opts{teststepId}="$opts{teststepId}"."_IC";
        }elsif ($vers == 8.0) {
            $opts{teststepId}="$opts{teststepId}"."_BR";
        }elsif ($vers == 8.1) {
            $opts{teststepId}="$opts{teststepId}"."_RR";
        }elsif ($vers == 8.2) {
            $opts{teststepId}="$opts{teststepId}"."_SN";
        } elsif ($vers == 8.3) {
            $opts{teststepId}="$opts{teststepId}"."_FS";
        }else {
            logcomment("Unknown filer version :$vers \n EXITING....");
            return 0;
        }
        logcomment 'Modified Test Step ID=> '.$opts{teststepId};
    }

    # Validate CliSignature file
    logcomment "Validating XML...\n";
    if(! &validateXML(cliSignature=>$opts{cliSignature}, teststepId=>$opts{teststepId})){
        logcomment 'Validation of XML Failed.  Stopping TC "'.$opts{teststepId}.'".',"\n";
        return 0;#exit;
    }else{
        logcomment "Validation of XML Passed\n";
    }

    # Get test step node
    my $testStepNode = &getTestStepNode(cliSignature=>$opts{cliSignature}, teststepId=>$opts{teststepId});
    logcomment "DEBUG:: ", $testStepNode->toString(), "\n" if $DEBUG;

    # Print title of test step
    logcomment "Executing Step: ",$testStepNode->find('logcomment')->string_value,"\n";

    # Call combinator for ZAPI and CLI
    my $type = getTestStepType($testStepNode);
    if($type eq "commands"){
        my $cliCommandStr = $testStepNode->find('command')->string_value;
        my $cmdsNode = combinatorCLI(command=>$cliCommandStr, hashOfVars=>$opts{hashOfVars});
        $testStepNode->appendChild($cmdsNode);
    }
    elsif($type eq "zapis"){
        my $zapiCommandStr = $testStepNode->find('zapi')->string_value;
        my $cmdsNode = combinatorCLI(command=>$zapiCommandStr, hashOfVars=>$opts{hashOfVars});
        $testStepNode->appendChild($cmdsNode);
        #attach <inputxml> to $testStepNode with values child nodes in it

        #Getting default username for 7mode/Cmode
        logcomment("Getting default username of any 7mode/Cmode setup");
        $opts{username} = _get_default_username();
        logcomment("Default username is $opts{username}");
    }

    # Execute and validate ZAPI/CLI
    logcomment "Test Step Node:\n",$testStepNode->toString(), "\n";
    my $result = ($type eq 'commands')? validateCLI(testStepNode=>$testStepNode, hashOfVars=>$opts{hashOfVars},
                                                    hashOfArrays=>$opts{hashOfArrays}, hashOfRefs=>$opts{hashOfRefs},
                                                    host=>$opts{host}, filermode=>$opts{filermode},
                                                    conntype=>$opts{conntype}, filerconn => $opts{filerconn}
                                                    ):
                                        validateZAPI(testStepNode=>$testStepNode, hashOfVars=>$opts{hashOfVars},
                                                    hashOfArrays=>$opts{hashOfArrays}, hashOfRefs=>$opts{hashOfRefs},
                                                    host=>$opts{host}, filermode=>$opts{filermode},
                                                    conntype=>$opts{conntype}, username=>$opts{username},
                                                    password=>$opts{password}
                                                    );

    # return the validated result to test script files
    return $result;

    # Sample code on how to convert string to XML::Simple object and convert it back to string
    #my $xml = new XML::Simple;
    #my $data = $xml->XMLin($testStepNode->toString(), ForceArray => 1,KeyAttr =>$opts{teststepId});
    #logcomment Dumper(XMLout($data, rootname => $type));
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::     validateCLI()
## @summary Validates the correctness of tags in given XML test step node.
## @arg $testStepNode Required: xmlNode for a required test step.
## @Returns    If validation passes returns 1, else return 0.
#-----------------------------------------------------------------------------------------------------------------
sub validateCLI{
    my %opts = @_;

    my $testStepNode = $opts{testStepNode};
    my $result = 0; # By default - Result is False

    my $cmdString = $testStepNode->find('command')->string_value;
    my @cmdNodes = $testStepNode->findnodes('command');
    my %cmdAttrs = getAttributes($cmdNodes[0]);

    my $choice = "";
    my $prompt = "";
    if(exists $cmdAttrs{interactive} and $cmdAttrs{interactive} eq "true"){
        my @promptArr = $testStepNode->findnodes('prompt');
        $prompt = $promptArr[0]->string_value;
        my %pHash = getAttributes($promptArr[0]);
        $choice = $pHash{choice};
    }

    foreach my $cmdVal ($testStepNode->findnodes('//cmds/value')){
        logcomment 'EXECUTE = ',$cmdVal->string_value(),"\n";
        my %valueAttrs = getAttributes($cmdVal);

        # Execute each command from <cmds>
        my $output = executeCLI(command=>$cmdVal->string_value, attributes=>\%valueAttrs,
                                prompt=>$prompt, choice=>$choice,
                                host=>$opts{host}, filermode=>$opts{filermode},
                                conntype=>$opts{conntype},filerconn => $opts{filerconn}
                                );

        # Sleep based on sleepPostExec, timeout or pollingfunc
        sleepOnCondition(cmd=>$cmdNodes[0], hashOfRefs=>$opts{hashOfRefs});

        # Expand XML after every execution of command line.
        my $newTestStepNode = expandXML(testStepNode=>$testStepNode, hashOfArrays=>$opts{hashOfArrays}, hashOfRefs=>$opts{hashOfRefs});
        if(! $newTestStepNode) {
            logcomment "Expand XML Failed\n" ;
            return 0;
        }
        logcomment "Expanded Test Step Node:\n",$newTestStepNode->toString(), "\n";

        # logcomment "Command output is\n",$output,"\n";
        $result = validateNotExpected(output=>$output, type=>'commands', attributes=>\%valueAttrs, hashOfVars=>$opts{hashOfVars}, testStepNode=>$newTestStepNode);
        if($result){
            $result = validateExpected(output=>$output, type=>'commands', attributes=>\%valueAttrs, hashOfVars=>$opts{hashOfVars}, testStepNode=>$newTestStepNode)
        }
    }
    return $result;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::     validateZAPI()
## @summary Validates the correctness of tags related to ZAPI in given XML test step node.
## @arg $testStepNode Required: xmlNode for a required test step.
## @Returns    If validation passes returns 1, else return 0.
#-----------------------------------------------------------------------------------------------------------------
sub validateZAPI{
    my %opts = @_;
    my $testStepNode = $opts{testStepNode};
    my $result = 0; # By default - Result is False

    my @zapiNodes = $testStepNode->findnodes('zapi');
    my ($output,$errNo,$errReason);

    # If <zapi> tag has text, then run it as command string
    if(length($testStepNode->find('zapi')->string_value) ){
        foreach my $cmdVal ($testStepNode->findnodes('//cmds/value')){
            logcomment 'EXECUTE = ',$cmdVal->string_value(),"\n";
            my %valueAttrs = getAttributes($cmdVal);

            eval{
                ($output,$errNo,$errReason) = executeZAPI(command=>$cmdVal->string_value, attributes=>\%valueAttrs,
                                                                  host=>$opts{host},
                                                                  username=>$opts{username}, password=>$opts{password}
                                                                 );
            };
            if($@) {
                logcomment __PACKAGE__." : validateZAPI : Failed at executeZAPI. Reason: $@\n";
                return 0;
            }
            # NOTE:  For now we are ignoring $errNo and $errReason validation, which is handled by validateFields section.
            logcomment __PACKAGE__." : validateZAPI : errReason : $errReason. \n";
            logcomment __PACKAGE__." : validateZAPI : errNo : $errNo. \n";
            logcomment __PACKAGE__." : validateZAPI : xmlOutput :";
            logcommentXML(logxml => $output);

            # Sleep based on sleepPostExec, timeout or pollingfunc
            sleepOnCondition(cmd=>$zapiNodes[0], hashOfRefs=>$opts{hashOfRefs});

            # Expand XML after every execution of command line.
            my $newTestStepNode = expandXML(testStepNode=>$testStepNode, hashOfArrays=>$opts{hashOfArrays}, hashOfRefs=>$opts{hashOfRefs});
            if(! $newTestStepNode) {
                logcomment "Expand XML Failed\n" ;
                return 0;
            }
            logcomment "Expanded Test Step Node:\n",$newTestStepNode->toString(), "\n";

            # logcomment "Command output is\n",$output,"\n";
            $result = validateNotExpected(output=>$output, type=>'zapis', attributes=>\%valueAttrs, hashOfVars=>$opts{hashOfVars}, testStepNode=>$newTestStepNode);
            if($result){
                $result = validateExpected(output=>$output, type=>'zapis', attributes=>\%valueAttrs, hashOfVars=>$opts{hashOfVars}, testStepNode=>$newTestStepNode)
            }
        }
    }else{
        # If <zapi> tag is empty, then run it as <inputxml>
        my %valueAttrs;
        logcomment "EXECUTE = Zapi Invoke using <inputxml>\n";

        my @inputxmlNodes = $testStepNode->findnodes('inputxml');
        my $inputXMLNode = expandXML(testStepNode=>$inputxmlNodes[0], hashOfArrays=>$opts{hashOfArrays}, hashOfRefs=>$opts{hashOfRefs});

        logcomment "DEBUG:: ", $inputXMLNode->toString(),"\n\n" if $DEBUG;

        @inputxmlNodes = $inputXMLNode->findnodes('*');
        $inputXMLNode = $inputxmlNodes[0];
        foreach my $childNode ($inputXMLNode->findnodes('*')->get_nodelist){
            # Execute each <inputxml> 's childNode
            logcomment 'INPUTXML = ',$childNode->toString(),"\n\n";
            eval{
                ($output,$errNo,$errReason) = executeZAPI(command=>$childNode->toString(), inputXML=>"true", attributes=>\%valueAttrs,
                                                                  host=>$opts{host},
                                                                  username=>$opts{username},    password=>$opts{password}
                                                                 );
            };
            if($@) {
                logcomment __PACKAGE__." : validateZAPI : Failed at executeZAPI. Reason: $@\n";
                return 0;
            }

            # NOTE:  For now we are ignoring $errNo and $errReason validation, which is handled by validateFields section.
            logcomment __PACKAGE__." : validateZAPI : errReason : $errReason. \n";
            logcomment __PACKAGE__." : validateZAPI : errNo : $errNo. \n";
            logcomment __PACKAGE__." : validateZAPI : xmlOutput :";
            logcommentXML(logxml => $output);

            # Sleep based on sleepPostExec, timeout or pollingfunc
            sleepOnCondition(cmd=>$zapiNodes[0], hashOfRefs=>$opts{hashOfRefs});

            # Expand XML after every execution of command line.
            my $newTestStepNode = expandXML(testStepNode=>$testStepNode, hashOfArrays=>$opts{hashOfArrays}, hashOfRefs=>$opts{hashOfRefs});
            if(! $newTestStepNode) {
                logcomment "Expand XML Failed\n" ;
                return 0;
            }
            logcomment "Expanded Test Step Node:\n",$newTestStepNode->toString(), "\n";

            # logcomment "Command output is\n",$output,"\n";
            $result = validateNotExpected(output=>$output, type=>'zapis', attributes=>\%valueAttrs, hashOfVars=>$opts{hashOfVars}, testStepNode=>$newTestStepNode);
            if($result){
                $result = validateExpected(output=>$output, type=>'zapis', attributes=>\%valueAttrs, hashOfVars=>$opts{hashOfVars}, testStepNode=>$newTestStepNode)
            }
        }
    }
    return $result;
}


#-----------------------------------------------------------------------------------------------------------------
## @name :: logcommentXML()
## @summary pretty prints a xml string.
## @arg $logxml Required: xml as a string which needs to be pretty printed.
## @Returns   1 on succesfull print.
#-----------------------------------------------------------------------------------------------------------------
sub logcommentXML {
    my %opts = validate(@_, {
        logxml => { type=>SCALAR, default=>""},
    });
    my $xml= new XML::Simple;
    my $printXML;
    eval {
        $printXML = $xml->XMLin($opts{logxml}, keeproot => 1);
        logcomment Dumper(XMLout($printXML, keeproot => 1));
    };
    logcomment "check the value passed to logcommentXML subroutine" if($@);
    return 1;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::    executeCLI()
## @summary executes a given command on the cli interface and returns the output.
##             called internally by validateCLI();
## @arg $command Required:  '-cli cmd' "The command to execute on the filer CLI"
## @arg $host Required: The host object to the filer if #rsh connection is used( for 7mode execution ).
##                       becomes a mandatory parameter if (filermode eq "normal").
## @arg $testStepId Required: The teststep id of the TC that has to betested.
## @arg \%hashOfRefs Optional: reference to hash of References to functions that returns an array of scalar items
## @arg \%hashOfArrays Optional: reference to hash of arrays having scalar items
## @arg $filermode optional:  normal 7G and 7mode normal(UP state) execution
##                            7modeMAINT  for executing command in 7-mode maintenance prompt.
##                            10modeNGSH   executing command in 10-mode(cluster mode) NG shell.
##                            10modeNODESCOPE   for executing command in 10-mode(cluster mode) Nodescope.
##                            10modeMAINT  for executing command in 10-mode(cluster mode) maintenance prompt.
##                      Default is normal
## @arg $conntype  optional:  The connection type required for executing the command.
##                      Default  #rsh for (filermode eq "normal").
##                      Default  #console for all other cases.
## @arg $filerconn optional:  The filer connection object.(must be a console connection object)
## @arg $prompt optional:   The prompt string or dialogue to wait for if the command prompts for confirmation.
##                          ex: please enter a selection {y|n}?,please enter choice {1|2}:
## @arg $choice optional:   The choice to send if 'prompt' is found. ex: "y", "1"
## @arg $attributes optional:  attributes found at the <command> tag.
##
## @Returns    The exact CLI output with out formatting or extracting values.
##            For #console connections syslog messages appear inline along with the actual output. Therefore
##            Those GMT,DBG messages are removed explicitly. But this fix is not complete when running on
##            maintenance mode prompt.
#-----------------------------------------------------------------------------------------------------------------
## TODO :     Fix for 10-mode maintenance prompt.
##            If somehow syslog/EMS messages can be suppressed while in 7-mode maintenance prompt then,
##            the line marked "#remove after TODO" should be removed.
##
#-----------------------------------------------------------------------------------------------------------------
sub executeCLI{
    my %DEFAULTHASH;
    my %opts = validate(@_, {
        command => { type=>SCALAR, default=>""},
        prompt => { type=>SCALAR, default=>"", optional=>1},
        choice  => { type=>SCALAR, default=>"", optional=>1},
        attributes => { type=>HASHREF, default=>undef, optional=>1},
        host => { type=>OBJECT, isa=>'Hostrec', optional=>1 },
        filermode => { type=>SCALAR, default=>"normal", optional=>1 },
        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
        filerconn => { type=>HASHREF, default=>\%DEFAULTHASH, optional=>1}
        });

    # Init variables
    my $command = $opts{command};
    my $conn;
    my ($match, $before, $output)=("", "", "");
    my %attrs = %{$opts{attributes}};

    my $host = $opts{host};
    my $filermode = $opts{filermode};
    my $conntype = $opts{conntype};

    #Check for filer mode
    if($filermode ne "normal") {
        $conntype = "#console";
    }

    if ($conntype eq '#console') {
        if($opts{filerconn} !~ /Connectrec/) {
            logcomment("Connection object not found. if conection type is \#console\n",
                       "Then connection object must be passed .");
            return 0;
        }
        $conn=$opts{filerconn};
    } else {
        $conn = connect($conntype);
    }

    #  If "#console" connection is used
    if ($conntype eq '#console') {

        # If prompt tag is provided
        if($opts{choice} ne "" and $opts{prompt} ne ""){
            if($filermode =~ /7modeMAINT|10modeMAINT/) {                     # Added '10modeMAINT' for burt600565 fix
                eval {
                    ($match,$before,$output)=say($conn,$command,'-noprompt','-wait', $opts{prompt}, -timeout,60);
                    ($match,$before,$output)=say($conn, $opts{choice},".",'-prompt','.*\*>',-swallow);
                };
                if ($@) {
                    $output=$before.$match.$output;
                }
                #$output =~ s/(Mon|Tue|Wed|Thu|Fri|Sat|Sun)[^\n\r]*(\n|\r)//ig; #remove after TODO
            }elsif ($filermode =~ /10modeNGSH/){
                ($match,$before,$output)=say($conn,$command,'-noprompt','-wait', $opts{prompt}, -timeout,60);
                ($match,$before,$output)=say($conn,$opts{choice});
            }elsif ($filermode =~ /10modeNODESCOPE/) {
                eval {
                    say($conn, "run local", '-prompt', ".*\>");
                    ($match,$before,$output)=say($conn,$command,'-noprompt','-wait', $opts{prompt}, -timeout,60);
                    ($match,$before,$output)=say($conn,$opts{choice},'.','-prompt',$conn->{connhost}.'[#*]*>');
                    say($conn,"exit");
                    sleep 1;
                };
            }

        # If prompt tag is not provided
        }else{
            if($filermode =~ /7modeMAINT|10modeMAINT/) {                          # Added '10modeMAINT' for burt600565 fix
                eval {
                    ($match,$before,$output)=say($conn,$command,".",'-prompt','.*\*>',-swallow);
                };
            }elsif ($filermode =~ /10modeNGSH/){
                eval {
                    ($match,$before,$output)=say($conn,$command,".",'-prompt',$conn->{connhost}.'::[*]*>',-swallow);
                };
            }elsif ($filermode =~ /10modeNODESCOPE/) {
                eval {
                    say($conn, "run local",'-prompt', ".*\>");
                    ($match,$before,$output)=say($conn,$command,".",'-prompt',$conn->{connhost}.'[#*]*>');
                    say($conn,"exit");
                    sleep 2;
                };
            }
        }

    # If "#rsh" connection is used
    }else{
        if($opts{choice} ne "" and $opts{prompt} ne ""){
            ($match,$before,$output)=say($conn,$command, '-noprompt', '-wait', $opts{prompt}, -timeout, 300);
            $conn->send(trim($opts{choice})."\r");
            $output=$before.$match.$output;
        }else{
            ($match,$before,$output) = say($conn, $command);
        }
    }

    #removing syslog messages if present
    $output =~ s/result not found[^\n]*//;
    $output =~ s/DBG:[^\n\r]*(\n|\r)//ig;
    $output =~ s/echo not found[^\n]*//;
    logcomment("CLI::OUTPUT :\n $output");
    return $output;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::     sleepOnCondition()
## @summary Sleeps for a given post execution time if not sleeps for a given polling function to satisfy.
## @arg $cmdNode Required: command node from the test step node that has all required attributes for this function.
## @arg \%hashOfRefs Optional: hashOfRefs has function reference that will be used as polling function.
##                   This param is mandatory if attribute "pollingFunction" is mentioned in the cmdNode.
##                   If both attributes - "sleepPostExec" and "pollingFunction" exists in cmdNode,
##                   then "sleepPostExec" takes precedence over the other.
## @Returns    After sleeping for a given time or till the polling function is satisfied, returns back to
##                   calling function.
#-----------------------------------------------------------------------------------------------------------------
sub sleepOnCondition{
    my %opts = @_;
    my $cmdNode = $opts{cmd};
    my %hashOfRefs = %{$opts{hashOfRefs}};

    my %attrs = getAttributes($cmdNode);

    logcomment Dumper(%attrs),"\n\n\n" if $DEBUG;
    if(exists $attrs{sleepPostExec}){
        my $sleepTime = 30;
        if($attrs{sleepPostExec} =~ /^\d+$/){
            $sleepTime = 0+$attrs{sleepPostExec};
        }
        logcomment "Sleeping for \"$sleepTime\" seconds\n";
        sleep($sleepTime);
    }elsif(exists $attrs{pollingFunction}){
        if(exists $hashOfRefs{ $attrs{pollingFunction}  }){
            my $delay = 5;      # Default value is 5 seconds
            if(exists $attrs{delay}){
                $delay = $attrs{delay};
            }

            my $timeout = 300;  # Default value is 5 minutes(i.e 300 seconds)
            if(exists $attrs{delay}){
                $timeout = $attrs{timeout};
            }

            my $startTime = time();
            while(1){
                my $returnCondition = 0;
                eval{
                    my $polling = $hashOfRefs{ $attrs{pollingFunction} };
                    my $pollingFunc = ${$polling}[0];
                    my $parameters = ${$polling}[1];

                    # Calling polling function
                    logcomment __PACKAGE__." : sleepOnCondition: Running \"pollingFunc\".  \$hashOfRefs{".$attrs{pollingFunction}."}.\n";
                    $returnCondition = &{$pollingFunc}(%{$parameters});
                };
                if($@){
                    logcomment __PACKAGE__." : sleepOnCondition: pollingFunction failed with message: $@\n";
                    logcomment __PACKAGE__." : sleepOnCondition: Possible error in function \$hashOfRefs{".$attrs{pollingFunction}."}.\n";
                    return 0;
                }
                if( $returnCondition){
                    logcomment __PACKAGE__." : sleepOnCondition: Stopping \"pollingFunc\" as polling condition satisfied.\n";
                    return 1;
                }
                my $endTime = time();
                logcomment " ", $startTime,"  ",$endTime,"\n" if $DEBUG;
                if(($endTime-$startTime) > $timeout){
                    logcomment __PACKAGE__." : sleepOnCondition: Stopping \"pollingFunc\" due to timeout(".$timeout.").\n";
                    return 1;
                }
                sleep($delay);
            }
        }else{
            logcomment __PACKAGE__." : sleepOnCondition: Skipping \"pollingFunc\".  \$hashOfRefs{".$attrs{pollingFunction}."} not found.\n";
        }
    }
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::     validateNotExpected()
## @summary Validates the absence of not expected values in output.
## @arg $testStepNode Required: xmlNode for a required test step.
## @Returns    If validation passes returns 1, else return 0.
#-----------------------------------------------------------------------------------------------------------------
sub validateNotExpected{
    my %opts = @_;
    $opts{'validateSection'}='notExpectedResults';
    my $tsNode = $opts{testStepNode};

    if($tsNode->findnodes('//notExpectedResults/fields')->size > 0){
        my $result = validateFields(%opts);
        if(${$result}[0]){
            logcomment "Validation Failed <notExpectedResults/fields>: Not expected fields are found in <".${$result}[1]."> output","\n";
            return 0;
        }
    }else{
        logcomment "Skipping validation of <notExpectedResults/fields>: Tags not found.","\n";
    }

    if($tsNode->findnodes('//notExpectedResults/regex')->size > 0){
        my $result = validateRegex(%opts);
        if(${$result}[0]){
            logcomment "Validation Failed <notExpectedResults/regex>: Not expected regex is found in <".${$result}[1]."> output","\n";
            return 0;
        }
    }else{
        logcomment "Skipping validation of <notExpectedResults/regex>: Tags not found.","\n";
    }
    logcomment "Validation Passed: <notExpectedResults>","\n";
    return 1;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::     validateExpected()
## @summary Validates the presence of expected values in output.
## @arg $testStepNode Required: xmlNode for a required test step.
## @Returns    If validation passes returns 1, else return 0.
#-----------------------------------------------------------------------------------------------------------------
sub validateExpected {
    my %opts = @_;
    $opts{'validateSection'}='expectedResults';
    my $tsNode = $opts{testStepNode};

    if($tsNode->findnodes('//expectedResults/fields')->size > 0){
        my $result = validateFields(%opts);
        if(! ${$result}[0]){
            if(${$result}[1] eq "both"){
                logcomment "Validation Failed <expectedResults/fields>: Expected fields are not found in <required> and <values> output","\n";
            }
            else{
                logcomment "Validation Failed <expectedResults/fields>: Expected fields are not found in <".${$result}[1]."> output","\n";
            }
            return 0;
        }
    }else{
        logcomment "Skipping validation of <expectedResults/fields>: Tags not found.","\n";
    }

    if($tsNode->findnodes('//expectedResults/regex')->size > 0){
        my $result = validateRegex(%opts);
        if(! ${$result}[0]){
            if(${$result}[1] eq "both"){
                logcomment "Validation Failed <expectedResults/regex>: Expected regex is not found in <required> and <values> output","\n";
            }
            else{
                logcomment "Validation Failed <expectedResults/regex>: Expected regex is not found in <".${$result}[1]."> output","\n";
            }
            return 0;
        }
    }else{
        logcomment "Skipping validation of <expectedResults/regex>: Tags not found.","\n";
    }

    logcomment "Validation Passed: <expectedResults>","\n";
    return 1;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::     validateFields()
## @summary Validates the presence of expected field values in output.
## @arg $testStepNode Required: xmlNode for a required test step.
## @Returns    If validation passes returns 1, else return 0.
#-----------------------------------------------------------------------------------------------------------------
sub validateFields{
    my %opts = @_;
    my $tsNode = $opts{testStepNode};
    my $result = 0;
    my $output=[$result,""];

    # Validate <required> tags
    foreach my $required ($tsNode->findnodes('//'.$opts{validateSection}.'/fields/required')){
        my %reqAttrs = getAttributes($required);
        my $order = (exists $reqAttrs{order})? $reqAttrs{order}: "false";
        my $exact = (exists $reqAttrs{exact})? $reqAttrs{exact}: "false";
        my $type = (exists $reqAttrs{type})? $reqAttrs{type}: "cli";
        my $xmlAttrs = (exists $reqAttrs{attributes})? $reqAttrs{attributes}: "false";
        my $xmlText = (exists $reqAttrs{text})? $reqAttrs{text}: "false";

        my $sourceStr = "";
        if ($required->findnodes('*')->size){
            my @tmpNodes = $required->findnodes('*');
            $sourceStr = $tmpNodes[0]->toString();
        }else{
            $sourceStr = $required->string_value();
        }

        my @expected = split(/\n/, rephraseExpectedStrings(source=>$sourceStr, attributes=>$opts{attributes}) );
        my @console = split(/\n/, $opts{output});

        logcomment __PACKAGE__." : validateFields : <".$opts{validateSection}.
                                "/fields/required> : Comparing output with : ".join("\n", @expected)."\n";

        if($type eq "cli"){
            $output = &comparatorCLI(console=>\@console, expected=>\@expected, order=>$order, exact=>$exact);
        }elsif($type eq "zapi"){
            $output = &comparatorXML(console=>$opts{output}, expected=>join("\n", @expected),attributes=>$xmlAttrs, text=>$xmlText );
        }

        if($opts{validateSection} eq "expectedResults" && ! ${$output}[0]){
            logcomment __PACKAGE__." : validateFields : Validation Failed <".$opts{validateSection}.
                                    "/fields/required>.  Expected string not found: ",${$output}[1],".\n";
            return [0, "required"];
        }elsif($opts{validateSection} eq "notExpectedResults" && ! ${$output}[0]){
            logcomment __PACKAGE__." : validateFields : Validation Passed <".$opts{validateSection}.
                                    "/fields/required>.  Not expected string not found: ",${$output}[1],".\n";
            return [0, "required"];
        }
        $result = 1;
    }

    # Validate <values> tag
    if($opts{validateSection} eq "expectedResults" && ! $tsNode->findnodes('//'.$opts{validateSection}.'/fields/values')->size){
        logcomment __PACKAGE__." : validateFields : Validation Passed <".$opts{validateSection}."/fields/required>.\n";
        return [1, "required"];
    }elsif($opts{validateSection} eq "notExpectedResults" && ! $tsNode->findnodes('//'.$opts{validateSection}.'/fields/values')->size){
        logcomment __PACKAGE__." : validateFields : Validation Failed <".$opts{validateSection}."/fields/required>.\n";
        return [1, "required"];
    }
    logcomment __PACKAGE__." : validateFields : Validation Passed <".$opts{validateSection}."/fields/required>.\n";

    foreach my $values ($tsNode->findnodes('//'.$opts{validateSection}.'/fields/values')){
        my %reqAttrs = getAttributes($values);
        my $order = (exists $reqAttrs{order})? $reqAttrs{order}: "false";
        my $exact = (exists $reqAttrs{exact})? $reqAttrs{exact}: "false";
        my $type = (exists $reqAttrs{type})? $reqAttrs{type}: "cli";
        my $xmlAttrs = (exists $reqAttrs{attributes})? $reqAttrs{attributes}: "false";
        my $xmlText = (exists $reqAttrs{text})? $reqAttrs{text}: "false";

        my $sourceStr = "";
        if($values->findnodes('*')->size){
            my @tmpNodes = $values->findnodes('*');
            $sourceStr = $tmpNodes[0]->toString();
        }else{
            $sourceStr = $values->string_value();
        }
        my @expected = split(/\n/, rephraseExpectedStrings(source=>$sourceStr, attributes=>$opts{attributes}));
        my @console = split(/\n/, $opts{output});

        logcomment __PACKAGE__." : validateFields : <".$opts{validateSection}."/fields/values> : Comparing output with : ".join("\n", @expected)."\n";

        if($type eq "cli"){
            $output = &comparatorCLI(console=>\@console, expected=>\@expected, order=>$order, exact=>$exact);
        }elsif($type eq "zapi"){
            $output = &comparatorXML(console=>$opts{output}, expected=>join("\n", @expected),attributes=>$xmlAttrs, text=>$xmlText );
        }

        if($opts{validateSection} eq "expectedResults"){
            if(! ${$output}[0]){
                logcomment __PACKAGE__." : validateFields : Validating <".$opts{validateSection}.
                                    "/fields/values>.  Expected string not found: ",${$output}[1],".\n";
            }elsif(${$output}[0]){
                logcomment __PACKAGE__." : validateFields : Validation Passed <".$opts{validateSection}.
                                    "/fields/values>. Expected string found: ",${$output}[1],".\n";
                return [1, "values"];
            }
        }elsif($opts{validateSection} eq "notExpectedResults"){
            if(!${ $output}[0]){
                logcomment __PACKAGE__." : validateFields : Validating <".$opts{validateSection}.
                                    "/fields/values>.  Not expected string not found: ",${$output}[1],".\n";
            }elsif(${$output}[0]){
                logcomment __PACKAGE__." : validateFields : Validation Failed <".$opts{validateSection}.
                                    "/fields/values>.  Not expected string found: ",${$output}[1],".\n";
                return [1, "values"];
            }
        }
    }
    if($opts{validateSection} eq "expectedResults" && $result){
        logcomment __PACKAGE__." : validateFields : Validation Failed <".$opts{validateSection}."/fields/values>.  Expected strings not found\n";
        return [0, "values"];
    }
    return [0, "both"];
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::     validateRegex()
## @summary Validates the presence of expected regular expressions in output.
## @arg $testStepNode Required: xmlNode for a required test step.
## @Returns    If validation passes returns 1, else return 0.
#-----------------------------------------------------------------------------------------------------------------
sub validateRegex {
    my %opts = @_;
    my $tsNode = $opts{testStepNode};
    my ($result,$output,$regex) = ("","",0);

    # Validate <required> tags
    foreach my $required ($tsNode->findnodes('//'.$opts{validateSection}.'/regex/required')){
        my %reqAttrs = getAttributes($required);
        my $order = (exists $reqAttrs{order})? $reqAttrs{order}: "false";
        my $exact = (exists $reqAttrs{exact})? $reqAttrs{exact}: "false";
        my $type = (exists $reqAttrs{type})? $reqAttrs{type}: "cli";
        my $xmlAttrs = (exists $reqAttrs{attributes})? $reqAttrs{attributes}: "false";
        my $xmlText = (exists $reqAttrs{text})? $reqAttrs{text}: "false";
        my $sourceStr = "";

        if($required->findnodes('*')->size){
            my @tmpNodes = $required->findnodes('*');
            $sourceStr = $tmpNodes[0]->toString();
        }else{
            $sourceStr = $required->string_value();
        }

        my @expected = split(/\n/, rephraseExpectedStrings(source=>$sourceStr, attributes=>$opts{attributes}));
        my @console = split(/\n/, $opts{output});

        $regex = rephraseExpectedStrings(source=>$required->string_value(), attributes=>$opts{attributes});

        if($type eq "cli"){
                logcomment __PACKAGE__." : validateRegex : <".$opts{validateSection}.
                                "/regex/required> : Comparing output with : ".trim($regex)."\n";

            $output = &comparatorRegex(console=>$opts{output}, regex=>$regex);
        }elsif($type eq "zapi"){
            $output = &comparatorXML(console=>$opts{output}, expected=>join("\n", @expected),
                                        attributes=>$xmlAttrs, text=>$xmlText,regex=>"true" );
        }

        if($opts{validateSection} eq "expectedResults" && ! ${$output}[0]) {
            logcomment __PACKAGE__." : validateRegex : Validation Failed <".$opts{validateSection}.
                                    "/regex/required>.  Expected regex not found: ",${$output}[1],".\n";
            return [0, "required"];
        }elsif($opts{validateSection} eq "notExpectedResults" && ! ${$output}[0]){
            logcomment __PACKAGE__." : validateRegex : Validation Passed <".$opts{validateSection}.
                                    "/regex/required>.  Not expected regex not found: ",${$output}[1],".\n";
            return [0, "required"];
        }
        $result = 1;
    }

    # Validate <values> tags
    if($opts{validateSection} eq "expectedResults" && ! $tsNode->findnodes('//'.$opts{validateSection}.'/regex/values')->size){
        logcomment __PACKAGE__." : validateRegex : Validation Passed <".$opts{validateSection}."/regex/required>.\n";
        return [1, "required"];
    }elsif($opts{validateSection} eq "notExpectedResults" && ! $tsNode->findnodes('//'.$opts{validateSection}.'/regex/values')->size){
        logcomment __PACKAGE__." : validateRegex : Validation Failed <".$opts{validateSection}."/regex/required>.\n";
        return [1, "required"];
    }
    logcomment __PACKAGE__." : validateRegex : Validation Passed <".$opts{validateSection}."/regex/required>.\n";

    foreach my $values ($tsNode->findnodes('//'.$opts{validateSection}.'/regex/values')){
        my %reqAttrs = getAttributes($values);
        my $order = (exists $reqAttrs{order})? $reqAttrs{order}: "false";
        my $exact = (exists $reqAttrs{exact})? $reqAttrs{exact}: "false";
        my $type = (exists $reqAttrs{type})? $reqAttrs{type}: "cli";
        my $xmlAttrs = (exists $reqAttrs{attributes})? $reqAttrs{attributes}: "false";
        my $xmlText = (exists $reqAttrs{text})? $reqAttrs{text}: "false";
        my $sourceStr = "";

        if($values->findnodes('*')->size){
            my @tmpNodes = $values->findnodes('*');
            $sourceStr = $tmpNodes[0]->toString();
        }else{
            $sourceStr = $values->string_value();
        }

        my @expected = split(/\n/, rephraseExpectedStrings(source=>$sourceStr, attributes=>$opts{attributes}));
        my @console = split(/\n/, $opts{output});

        $regex = rephraseExpectedStrings(source=>$values->string_value(), attributes=>$opts{attributes});

        if($type eq "cli"){
                logcomment __PACKAGE__." : validateRegex : <".$opts{validateSection}.
                                "/regex/values> : Comparing output with : ".$regex."\n";
            $output = &comparatorRegex(console=>$opts{output}, regex=>$regex);
        }elsif($type eq "zapi"){
            $output = &comparatorXML(console=>$opts{output}, expected=>join("\n", @expected),
                                attributes=>$xmlAttrs, text=>$xmlText, regex=>"true");
        }

        if($opts{validateSection} eq "expectedResults"){
            if(! ${$output}[0]){
                logcomment __PACKAGE__." : validateRegex : Validating <".$opts{validateSection}.
                                    "/regex/values>. Expected regex not found: ",${$output}[1],".\n";
            }elsif(${$output}[0]){
                logcomment __PACKAGE__." : validateRegex : Validation Passed <".$opts{validateSection}.
                                    "/regex/values>. Expected string found: ",${$output}[1],".\n";
                return [1, "values"];
            }
        }elsif($opts{validateSection} eq "notExpectedResults"){
            if(!${$output}[0]){
                logcomment __PACKAGE__." : validateRegex : Validating <".$opts{validateSection}.
                                    "/regex/values>.  Not expected regex not found: ",${$output}[1],".\n";
            } elsif(${$output}[0]) {
                logcomment __PACKAGE__." : validateRegex : Validation Failed <".$opts{validateSection}.
                                    "/regex/values>.  Not expected regex found: ",${$output}[1],".\n";
                return [1, "values"];
            }
        }
    }

    if($opts{validateSection} eq "expectedResults" and $result){
        logcomment __PACKAGE__." : validateRegex : Validation Failed <".
                                $opts{validateSection}."/regex/values>.  Expected regex not found\n";
        return [0, "values"];
    }

    return [0, "both"];
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::     rephraseExpectedStrings()
## @summary Rephrase the expected/notExpected node text values for $var$ and @var@ that were used in cmdString
## @arg $source Required: source string to be rephrased
## @arg %attributes Required: hash of variables that has the required destination value to be substituted
##              during rephrase of strings.
## @Returns    Returns rephrased string
#-----------------------------------------------------------------------------------------------------------------
sub rephraseExpectedStrings{
    my %opts = validate(@_, {
        source => { type=>SCALAR, default=>""},
        attributes => { type=>HASHREF, default=>undef},
        });

    my $newString = $opts{source};
    my %attributes = %{$opts{attributes}};

    logcomment "DEBUG:: \n",Dumper(%attributes),"\n\n" if $DEBUG;
    foreach my $var (keys %attributes){
        $newString =~ s/\$$var\$/$attributes{$var}/g;
        $newString =~ s/\@$var\@/$attributes{$var}/g;
    }

    logcomment "DEBUG:: ", $newString,"\n\n" if $DEBUG;
    return $newString;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::     validateXML()
## @summary Validates the correctness of all tags in given XML for a particular test step id.
## @arg $cliSignature Required: Absolute file name of XML having test steps.
## @arg $teststepId Required: Test step id to be validated in the xml file.
## @Returns    If validation passes returns 1, else return 0.
#-----------------------------------------------------------------------------------------------------------------
sub validateXML{
    my %opts = validate(@_, {
        cliSignature => { type=>SCALAR, default=>"input.xml"},
        teststepId => { type=>SCALAR, default=>undef}
        });

    my $cliSignature = $opts{"cliSignature"};
    my $tsid = $opts{"teststepId"};

    if(! -e $cliSignature ){
        logcomment "File not found: ",$cliSignature,"\n";
        logcomment "Stopping further validation\n";
        return 0;#exit;
    }
    my $xpath = XML::XPath->new(filename => $cliSignature);

    # Validate the presence of required TestStep node
    my $tsNodes = $xpath->find('//@'.$tsid);
    if(!isNode($tsNodes)){
        return 0;
    }
    if($tsNodes->size == 0){
        logcomment 'Test Step ID:'.$tsid.' not found';
        return 0;
    }
    elsif($tsNodes->size > 1){
        logcomment 'Duplicates nodes with same test step id found: '.$tsid;
        return 0;
    }
    else{
        # find type - commands/zapis
        my $type = 'commands';
        if($xpath->exists('Database/zapis[@'.$tsid.']')){
            $type = 'zapis';
        }
        logcomment 'Found node of type: '.$type."\n";
        my $xpathStr = ($type eq "commands")? 'Database/commands[@'.$tsid.']' : 'Database/zapis[@'.$tsid.']';
        my @nodeset = $xpath->findnodes($xpathStr);

        foreach my $node (@nodeset){
            logcomment "DEBUG:: getTagName :", $node->getName,"\n" if $DEBUG;
            logcomment "DEBUG:: getTagName :", $node->getTagName,"\n" if $DEBUG;

            # Find the presence of ( <command> or <zapi> ) and <logcomment>
            my $cmd = ($type eq "commands") ? "command": "zapi";
            foreach my $toValidate ($cmd, 'logcomment'){
                if( ! $xpath->exists($toValidate, $node)){
                    logcomment 'Tag missing: <'.$toValidate.'>',"\n";
                    return 0;
                }
            }

            # If interactive="true in <command> then <prompt> tag should be present.
            if($type eq "commands" && length($node->find('command')->string_value) ){
                my @cmdArr = $node->findnodes('command');
                my %cHash = getAttributes($cmdArr[0]);

                if(exists $cHash{interactive} && $cHash{interactive} eq "true" ){
                    if(!$xpath->exists('prompt', $node)){
                        logcomment __PACKAGE__." : If interactive=\"true\", then <prompt> tag should be present.\n";
                        return 0;
                    }
                    my @promptArr = $node->findnodes('prompt');
                    my %pHash = getAttributes($promptArr[0]);
                    if(exists $pHash{choice} && !length(trim($pHash{choice}))){
                        logcomment __PACKAGE__." : \"choice\" attribute of <prompt> tag is empty.\n";
                        return 0;
                    }
                    if(!length(trim($promptArr[0]->string_value))){
                        logcomment __PACKAGE__." : <prompt> tag should not be empty.\n";
                        return 0;
                    }
                }
            }

            # Validating if <zapi> has no value and presence of <inputxml>
            # Feature plugin:  [ Make changes here if you want... ]
            # Multiple childnodes in <inputxml>.  Also change validateZAPI for looping logic for each entry in <inputxml>
            # Remove check for no attributes in childnodes of <inputxml>
            if($type eq "zapis" && ! length($node->find('zapi')->string_value) ){
                # Validating presence of <inputxml>
                if( ! $xpath->exists("inputxml", $node)){
                    logcomment __PACKAGE__." : If <zapi> tag is empty, <inputxml> tag should be present.\n";
                    return 0;
                }

                # Validating presence of single/multiple childnodes to <inputxml>
                my @inputXmlNodes = $node->findnodes('inputxml');
                if(! $inputXmlNodes[0]->findnodes('*')->size){
                    logcomment __PACKAGE__." : <inputxml> tag has no child nodes.\n";
                    return 0;
                }

                if($inputXmlNodes[0]->findnodes('*')->size > 1){
                    logcomment __PACKAGE__." : <inputxml> tag has more than one child nodes.\n";
                    return 0;
                }

                # First child of <inputxml> should not have attributes if single <inputxml> algo is assumed.
                # As it can expand to multiple childnodes of <inputxml>
                my @inputXMLChild = $inputXmlNodes[0]->findnodes('*')->get_nodelist;
                my %hash = getAttributes($inputXMLChild[0]);
                foreach (keys %hash){
                    logcomment __PACKAGE__." : <inputxml/".$inputXMLChild[0]->getName()."> tag should not have attributes.\n";
                    return 0;
                }
            }

            # Find the presence of <expectedResults>, <notExpectedResults>
            if( ! $xpath->exists('expectedResults', $node) && ! $xpath->exists('notExpectedResults', $node)){
                logcomment 'Both Tags missing: <expectedResults> and <notExpectedResults>',"\n";
                return 0;
            }

            # Find presence of duplicate <expectedResults> or <notExpectedResults>
            # Find the presence of <fields>, <regex> within <expectedResults>, <notExpectedResults>
            foreach my $toValidate ('expectedResults', 'notExpectedResults') {
                if($node->findnodes($toValidate)->size > 1){
                    logcomment 'Duplicate tags found: <'.$toValidate.'>',"\n";
                    return 0;
                }
                if( $xpath->exists($toValidate, $node) && ! $xpath->exists($toValidate.'/fields', $node) && ! $xpath->exists($toValidate.'/regex', $node)){
                    logcomment 'Both Tags missing in <'.$toValidate.'>: <fields> and <regex>',"\n";
                    return 0;
                }
            }

            # Find presence of duplicate <fields> or <regex>
            foreach my $toValidate ('expectedResults/fields', 'expectedResults/regex'){
                if($xpath->exists('expectedResults', $node) && $node->findnodes($toValidate)->size > 1){
                    logcomment 'Duplicate tags found: <'.$toValidate.'>',"\n";
                    return 0;
                }
            }

            foreach my $toValidate ('notExpectedResults/fields', 'notExpectedResults/regex'){
                if($xpath->exists('notExpectedResults', $node) && $node->findnodes($toValidate)->size > 1){
                    logcomment 'Duplicate tags found: <'.$toValidate.'>',"\n";
                    return 0;
                }
            }

            # Find the presence of <required>, <values> within <fields>, <regex> of <expectedResults>, <notExpectedResults>
            foreach my $toValidate ('expectedResults/fields', 'expectedResults/regex', 'notExpectedResults/fields', 'notExpectedResults/regex'){
                if( $xpath->exists($toValidate, $node) && ! $xpath->exists($toValidate.'/required', $node) &&
                   ! $xpath->exists($toValidate.'/values', $node)){
                    logcomment 'Both Tags missing in <'.$toValidate.'>: <required> and <values>',"\n";
                    return 0;
                }
            }

            # Validate all sub tags within <expectedResults> and <notExpectedResults>
            my @toValidate = (
                              'notExpectedResults/fields/required',
                              'notExpectedResults/fields/values',
                              'notExpectedResults/regex/required',
                              'notExpectedResults/regex/values',
                              'expectedResults/fields/required',
                              'expectedResults/fields/values',
                              'expectedResults/regex/required',
                              'expectedResults/regex/values',
                              );

            foreach my $toVal (@toValidate){
                if(!$xpath->exists($toVal, $node)){
                    logcomment "Skipping Node as it is not found : ",$toVal,"\n";
                    next;
                }
                foreach my $tmp ($node->findnodes($toVal)){
                    #logcomment "getName : ",$tmp->getName,"\n";
                    #logcomment "toString : ",$tmp->toString, "\n";
                    #logcomment "string_value : ",$tmp->string_value, "\n";
                    #logcomment "attribute type : ", $tmp->find('@type')->string_value,"\n";
                    #logcomment "exists : ", $xpath->exists('@array', $tmp), "\n";

                    # If text node is empty, then fail the validation
                    if(! length($tmp->string_value)){
                        logcomment 'Validation Failed: Empty text nodes are not allowed at <'.$toVal.'>.', "\n";
                        return 0;
                    }

                    # If type attribute doesn't exist, it defaults to "cli" type
                    if($xpath->exists('@type', $tmp) && $tmp->find('@type')->string_value eq "zapi"){
                        #logcomment "its zapi type\n\n",$tmp->getName(),"\n";
                        #logcomment "Size=",$tmp->findnodes('*')->size,"\n";
                        if($tmp->findnodes('*')->size > 1){
                            logcomment __PACKAGE__." : validateXML : Validation Failed for zapi text within <$toVal>.\n";
                            logcomment __PACKAGE__." : validateXML : <".$tmp->getName()."> has more than one child node for type=\"zapi\".\n";
                            return 0;

                        }
                        foreach my $i ($tmp->findnodes('*')){
                            logcomment $i->toString,"\n";

                            # validate the sub-nodes within <required> or <values> if its of type="zapi"
                            if(! &validate_xml_in_node($i->toString)){
                                logcomment 'Validation of zapi text within <'.$toVal.'> failed',"\n";
                                return 0;
                            }
                        }
                    }
                    else{  # TODO:  To get this validated for zapi, just make condition here as if(1).
                        # Validate CLI type tags
                        if($xpath->exists('@array', $tmp)) {
                            if (length($tmp->find('@array')->string_value) < 3){
                                logcomment 'Attribute "array" should be of format @var@ with atleast one character between two "@" symbols ',"\n";
                                return 0;
                            }
                            #logcomment $tmp->find('@array')->string_value,"\n";

                            # Validate each attribute split in @array.  eg. array="@var1@ @var2@"
                            my $arrayAttr = $tmp->find('@array')->string_value;
                            foreach my $eachAttr (split(/ /, $arrayAttr)){
                                $eachAttr =~ s/\s*(\S*)\s*/$1/;
                                my $matchStr = '\[\s*'.$eachAttr.'\s*\]';
                                if($eachAttr =~ /\[/ && $eachAttr =~ /\]/){
                                    $matchStr = '\[\s*'.quotemeta($eachAttr).'\s*\]';
                                }
                                if($tmp->string_value !~ /$matchStr/){
                                    logcomment 'Value of "array" attribute defined but not used in child node of <'.$toVal.'> : ', $eachAttr, "\n";
                                    return 0;
                                }
                            }
                        }
                        if($xpath->exists('@hash', $tmp) && !$xpath->exists('@key', $tmp)){
                            logcomment 'Attribute "key" not found at <'.$toVal.'>',"\n";
                            return 0;
                        }
                        elsif($xpath->exists('@key', $tmp) && !$xpath->exists('@hash', $tmp)){
                            logcomment 'Attribute "hash" not found at <'.$toVal.'>',"\n";
                            return 0;
                        }
                        elsif($xpath->exists('@hash',$tmp) && $xpath->exists('@key',$tmp)){
                            if (length($tmp->find('@key')->string_value) < 3){
                                logcomment 'Attribute "key" should be of format @var@ with atleast one character between two "@" symbols ',"\n";
                                return 0;
                            }

                            # Validate each attribute split in @key.  eg. key="@var1@ @var2@"
                            my $keyAttr = $tmp->find('@key')->string_value;
                            foreach my $eachAttr (split(/ /, $keyAttr)){
                                $eachAttr =~ s/\s*(\S*)\s*/$1/;
                                my $matchStr = '\{\s*'.$eachAttr.'\s*\}';
                                if($eachAttr =~ /\[/ && $eachAttr =~ /\]/){
                                    $matchStr = '\{\s*'.quotemeta($eachAttr).'\s*\}';
                                }
                                if($tmp->string_value !~ /$matchStr/){
                                    logcomment 'Value of "array" attribute defined but not used in child node of <'.$toVal.'> : ', $eachAttr, "\n";
                                    return 0;
                                }
                                if($tmp->string_value !~ /$matchStr/){
                                    logcomment 'Value of "key" attribute defined but not used in child node of <'.$toVal.'> : ', $eachAttr, "\n";
                                    return 0;
                                }
                            }
                        }
                    }
                }
                logcomment "Validation passed for node : ",$toVal,"\n";
            }
        }
    }
=cut
    ## find type - commands/zapis
    $nodes = $xpath->find('Database/zapis[@'.$tsid.']');
    my $type = 'commands';
    if($nodes->size == 1){
        $type = 'zapis'
    }
    logcomment 'one found of type: '.$type."\n";

    ## Check for presence of <command> or <zapi> based on type
    my $xpathStr = ($type eq "commands")? 'Database/commands[@'.$tsid.']/command' : 'Database/zapis[@'.$tsid.']/zapi';
    $nodes = $xpath->find($xpathStr);
    if($nodes->size != 1){
        my $str = ($type eq "commands")? "<command>": "<zapi>";
        logcomment "Tag not found: $str\n";
        return 0;
    }

    ## Check for presence of atleast one occurence of <expectedResults> or <notExpectedResults>
    $xpathStr = ($type eq "commands")? 'Database/commands[@'.$tsid.']/expectedResults | Database/commands[@'.$tsid.']/notExpectedResults' :
            'Database/zapis[@'.$tsid.']/expectedResults | Database/zapis[@'.$tsid.']/notExpectedResults';
    $nodes = $xpath->find($xpathStr);
    if($nodes->size >= 1 ){
        logcomment STDERR "Found ", $nodes->size, " nodes:\n";
        foreach my $node ($nodes->get_nodelist) {
            logcomment $node->find('expectedResults')->string_value;
            logcomment STDERR "-- NODE --\n";
            #logcomment $node->toString;
        }
    }
=cut
    return 1;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::     validate_xml_in_node
## @summary  validate a xml for a given node
## @arg $string string be validated in the xml file.
## @Returns the test step node of CliSignature for given filename and testStepId
#-----------------------------------------------------------------------------------------------------------------
sub validate_xml_in_node{
    # This is incomplete function yet.  have to validate for array, hash, key in all node recursively
    my $string = $_[0];
    return 1;
}

#-----------------------------------------------------------------------------------------------------------------
## @name :: isNode
## @summary  find out wheather a given xpath object refers to a node or not.
## @arg $nodes the xpath::Node object reference.
## @Returns  If validation passes returns 1, else return 0.
#-----------------------------------------------------------------------------------------------------------------
sub isNode{
    my $nodes = $_[0];
    unless ($nodes->isa('XML::XPath::NodeSet')) {
        logcomment STDERR "Query didn't return a nodeset. Value: ";
        logcomment $nodes->value, "\n";
        return 0;
    }
    return 1;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::      getTestStepNode
## @summary find the XML node for a given test, step Id
## @arg $cliSignature Required: Absolute file name of XML having test steps.
## @arg $teststepId Required: Test step id to be validated in the xml file.
## @Returns the test step node of CliSignature for given filename and testStepId
#-----------------------------------------------------------------------------------------------------------------
sub getTestStepNode{
    my %opts = validate(@_, {
        cliSignature => { type=>SCALAR, default=>"input.xml"},
        teststepId => { type=>SCALAR, default=>undef},
        });

    my $tsid = $opts{"teststepId"};
    my $xpath = XML::XPath->new(filename => $opts{cliSignature});

    my $type = 'commands';
    if($xpath->exists('Database/zapis[@'.$tsid.']')){
        $type = 'zapis';
    }

    my $xpathStr = ($type eq "commands")? 'Database/commands[@'.$tsid.']' : 'Database/zapis[@'.$tsid.']';
    my @nodeset = $xpath->findnodes($xpathStr);
    return $nodeset[0];
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::      getAttributes()
## @summary find the XML node for a given test, step Id
## @arg $node the xpath::Node object reference.
## @Returns Hash of list of attributes for a given leaf node
#-----------------------------------------------------------------------------------------------------------------
sub getAttributes{
    my $node = $_[0];
    my %hash = ();

    foreach my $val ($node->findnodes('@*')){
        my @arr = split(/=/,trim($val->toString()));
        my $key = shift(@arr);
        @arr = split(//, join('=',@arr));
        my $value = join('', @arr[1..$#arr-1]);
        $hash{$key} = $value;
    }
    return %hash;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::      getTestStepType()
## @summary find the type of a given testStepNode
## @arg $node Required: the xpath::Node object reference "the reference to testStepNode.
## @Returns "zapis" if the node is of type Zapi else "commands" if the node is of type CLI
#-----------------------------------------------------------------------------------------------------------------
sub getTestStepType{
    my $node = $_[0];
    if($node->getName() eq "commands"){
        return "commands";
    }
    return 'zapis';
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::      combinatorCLI()
## @summary Creates CLI combinations for given command and hashOfVars.
## @arg $command optional: the CLI command to be executed on the filer.
## @arg $hashOfVars optional: the hash reference of the variables which needs to be subtituted inline in the command string.
## @Returns string of xml node with root node as <cmds>
#-----------------------------------------------------------------------------------------------------------------
sub combinatorCLI{
    my %opts = validate(@_, {
        command => { type=>SCALAR, default=>""},
        hashOfVars => { type=>HASHREF, default=>undef, optional=>1}
        });

    my %hashOfVars = %{$opts{hashOfVars}};

    my ($refList,$refHash)=&generateCommands(command=>$opts{command}, hashOfVars=>\%hashOfVars);
    logcomment Dumper($refHash) if $DEBUG;

    my %cmd_hash;
    $cmd_hash{value}=[@{$refList}];

    my $cmds=XMLout(\%cmd_hash, rootname => "cmds");
    foreach my $val (@{$refHash}) {
        $cmds =~ s/<value>/<value $val>/;
    }
    my $xp = XML::XPath->new($cmds);
    my @nodes = $xp->findnodes('cmds');
    my $cmdsNode = $nodes[0];
    logcomment $cmdsNode->toString(), "\n" if $DEBUG;

    return $cmdsNode;
}

#-------------------------------------------------------------------------------------------
# @name ::    generateCommands()
# @summary expands a comand with cartesian product of variables involved.
# @arg :  recieves a cmdstring,and hash containing the vars to be expanded.
# @Returns     the string of commands in XML fromat on success.
#-------------------------------------------------------------------------------------------
sub generateCommands{
    my %opts = validate(@_, {
        command => { type=>SCALAR, default=>""},
        hashOfVars => { type=>HASHREF, default=>undef, optional=>1},
        expandXML => { type=>SCALAR, default=>0, optional=>1}
        });
    my %hashOfVars = %{$opts{hashOfVars}};

    # $expandXML is set only if this subroutine was called by expandXML().
    my $expandXML = $opts{expandXML};

    my @cmds = ($opts{command});
    my @tmpArr = ();

    my @attr = ("");
    my @tmpAttr = ();


    # Validate for presence of keys in %hashOfVars
    if (!$expandXML)
    {
        foreach my $tmp (split(/\s/, $opts{command}) ){
            if($tmp =~ /(\@|\$).*(\@|\$)/) {
                if(! exists $hashOfVars{$tmp}){
                    logcomment __PACKAGE__." : combinatorCLI : Variable not found in hashOfVars.  Ignoring \"$tmp\" for command expansion.\n";
                }
            }
        }
    }

    # Iterate for every key in hashOfVars
    foreach my $hKey (keys %hashOfVars){
        # Iterate for every value of hashOfRefs
        foreach my $hVal ($hashOfVars{$hKey}){
            # Iterate for all commands set
            for(my $i=0;$i<scalar(@cmds);$i++){
                # If key is of type $var$, replace it and add it to @tmpArr
                if(ref($hVal) ne "ARRAY"){
                    if($cmds[$i] =~ /\Q$hKey\E/){
                        $cmds[$i] =~ s/\Q$hKey\E/$hVal/g;
                        push(@tmpArr, $cmds[$i]);
                        push(@tmpAttr, $attr[$i]."$hKey=\"$hVal\" ");
                    }
                }else{
                # If key is of type @var@ replace for all values of @var@ and add to @tmpArr
                    my $oldCmd = $cmds[$i];  # Used to preserve the old value of $cmd
                    foreach my $aVal (@{$hVal}){
                        if ($expandXML eq "true" && $cmds[$i] =~/(\[|\{)\s*$hKey\s*(\]|\})/) {
                            $cmds[$i] =~ s/(\[|\{)\s*$hKey\s*(\]|\})/$aVal/g;
                            push(@tmpArr, $cmds[$i]);
                            push(@tmpAttr, $attr[$i]."$hKey=\"$aVal\" ");
                            $cmds[$i] = $oldCmd;
                        } elsif($cmds[$i] =~ /\Q$hKey/){
                            $cmds[$i] =~ s/\Q$hKey\E/$aVal/;
                            push(@tmpArr, $cmds[$i]);
                            push(@tmpAttr, $attr[$i]."$hKey=\"$aVal\" ");
                            $cmds[$i] = $oldCmd;
                        }
                    }
                }
            }

            # If @tmpArr has some command values then iterate again over those values as @cmds
            if(scalar(@tmpArr)){
                @cmds = @tmpArr;
                @tmpArr = ();
                @attr = @tmpAttr;
                @tmpAttr = ();
            }
        }
    }

    # Replace all attrs from $var$, @arr@ to var, arr format.
    foreach (@attr){
        $_ =~ s/(\$|\@)//g;
    }
    return (\@cmds, \@attr);
}

=cut
---------------------------------------------------------snip-------------------
    # REMOVE:  This is old code of combinatorCLI() to be removed later
sub generate_commands {
    my ($cmdstr,%hash)=@_;

    my (@var_key_list,@var_val_list ,@final_cmd_list);
    my $length=1;
    my $count=0;
    my (@ret_list,$repeats,$i,$j,$counter_4);
    my @ret_hash_list;
    my $hash_count=0;

    my @cmd_split=split /\s/,$cmdstr;
    my @cmd_split1=@cmd_split;


    # Validate keys in %hash
    foreach my $tmp (@cmd_split) {
        if($tmp =~ /\@.*\@/) {
            if(! exists $hash{$tmp}){
                logcomment __PACKAGE__." : combinatorCLI : variable \"$tmp\"".
                " for command expansion not found in hashOfVars\n";
                exit;#(\@final_cmd_list,\@ret_hash_list);
            }
        }
        if($tmp =~ /\$.*\$/) {
            if(! exists $hash{$tmp}){
                logcomment __PACKAGE__." : combinatorCLI : variable \"$tmp\".
                " for command expansion not found in hashOfVars\n";
                exit;#(\@final_cmd_list,\@ret_hash_list);
            }
        }
    }

    #substitute the scalar variables in their rightfull places..
    # Extracts the $var$ format and replace based on hash key entry
    foreach my $tmp (@cmd_split) {
        if( $tmp =~ /(\$.*\$)/) {
            if( !exists $hash{$1}) {
                #logcomment "Error.. variable $1 not found in hash";
                #return 0;
            }else{
                $tmp =~ s/(\$.*\$)/$hash{$1}/ ;
            }
        }
    }

    # Extracts the $var$ format and create the attributes for the xml format
    foreach my $tmp (@cmd_split1) {
        if( $tmp =~ /(\$.*\$)/) {
            if(exists $hash{$1}){
                $tmp =~ s/(\$(.*)\$)/$2=\'$hash{$1}\'/;
            }
        } else {
            if ($tmp !~ /\@.*\@/) {
                $tmp =~ s/.*//;
            }
        }
    }

    # Extracts the @var@ format and create the list of @key@
    foreach my $tmp (@cmd_split) {
        if($tmp =~ /\@.*\@/) {
            $var_key_list[$count++]=$tmp;
        }
    }

    #substitute the array variables in their rightfull places..
    $count=0;
    foreach (@var_key_list) {
        if( !exists $hash{$_}) {
            #logcomment "Error.. variable $1 not found in hash";
            #return 0;
        }else{
            $var_val_list [$count++]=$hash{$_};
        }
    }

    foreach (@var_val_list ) {
        $length *= @$_
    }          #$length=total no of cmds..

    # Generate combination of @key@ entries.
    for ($count = 0; $count < @var_val_list ; $count++) {
        for($repeats = 1, $i = $count+1; $i < @var_val_list ; $i++) {
            $repeats *= @{$var_val_list [$i]};
        }
        for($j = 0; $j < $length;) {
            foreach ( @{ $var_val_list [$count] }) {
                for($counter_4=0;$counter_4<$repeats;$counter_4++) {
                    $ret_list[$j++][$count]=$_;
                }
            }
        }
    }

    # Generate the attribute values for XML
    for($i = 0; $i < $length;$i++) {
        $count=0;
        my @cmd_list2=@cmd_split;
        foreach (@cmd_list2) {
            s/(\@.*\@)/$ret_list[$i][$count++]/;
        }
        $final_cmd_list[$i] =join " ",@cmd_list2;

        @cmd_list2=@cmd_split1;
        $count=0;
        foreach (@cmd_list2) {
            s/(\@(.*)\@)/$2=\'$ret_list[$i][$count++]\'/;
        }
        $ret_hash_list[$i] =join " ",@cmd_list2;
    }

    return (\@final_cmd_list,\@ret_hash_list);
}
-------------------------------------snip---------------------------------------
=cut

#-----------------------------------------------------------------------------------------------------------------
## @name :: comparatorCLI()
## @summary compares two strings based on set optional Parameters.
## @arg $console Required: console string/array of strings obtained from (console output)/(actual string) .
## @arg $expected Required: expected string to match(reference string).
## @arg $order Optional: to compare the set of strings b/w actual and expected in same order, set this as "true". Default is "false"
## @arg $text Optional: to compare the set of strings b/w actual and expected for an exact match, set this as "true". Default is "false"
## @Returns     1 if expected string is found in console string.
##              on failure returns 0 and error strings.
#-----------------------------------------------------------------------------------------------------------------
sub comparatorCLI($$;%){
    my %opts = validate(@_, {
        console => { type=>ARRAYREF, default=>undef},
        expected => { type=>ARRAYREF, default=>undef},
        order => { type=>SCALAR, default=>"false"},
        exact => { type=>SCALAR, default=>"false"},
        regex => { type=>SCALAR, default=>"false"}
        } );

    my @console = grep{/\w/} @{$opts{console}};    # Get non-blank lines
    my @expected = grep{/\w/} @{$opts{expected}};
    my $consoleStr = join("", @console);
    my $cmpTxt;
    my $var ="";

    if(exists $opts{order} && $opts{order} eq "true"){
        my @arr = [];
        foreach $var (@expected){
            if (exists $opts{regex} && $opts{regex} eq "true") {
                $cmpTxt = trim($var);
                $cmpTxt = (exists $opts{exact} && $opts{exact} eq "true")? "\^$cmpTxt\$": trim($cmpTxt);
            }else{
                $cmpTxt = quotemeta(trim($var));
                $cmpTxt = (exists $opts{exact} && $opts{exact} eq "true")? "\^$cmpTxt\$": trim($cmpTxt);
                logcomment "DEBUG::===consoleStr===>",$consoleStr, "\n\n" if $DEBUG;
                logcomment "DEBUG::===var===>",$var, "\n\n\n" if $DEBUG;
            }

            if($consoleStr =~ /$cmpTxt/){
                my @arr = split(/$cmpTxt/, $consoleStr);
                $consoleStr = join("", @arr[1..$#arr]);
            }
            else{
                return [0, trim($var)];
            }
        }
    }

    foreach $var (@expected){
        if (exists $opts{regex} && $opts{regex} eq "true") {
            $cmpTxt = trim($var);
            $cmpTxt = (exists $opts{exact} && $opts{exact} eq "true")? "\^$cmpTxt\$": trim($cmpTxt);
        } else {
            $cmpTxt = quotemeta(trim($var));
            $cmpTxt = (exists $opts{exact} && $opts{exact} eq "true")? "\^$cmpTxt\$": trim($cmpTxt);
            logcomment "DEBUG::===consoleStr===>",$consoleStr, "\n\n" if $DEBUG;
            logcomment "DEBUG::===var===>",$var, "\n\n\n" if $DEBUG;
        }

        if(!grep{/$cmpTxt/}@console){
            return [0, trim($var)];
        }
    }
    return [1, join("\n", @expected)];
}

#-----------------------------------------------------------------------------------------------------------------
## @name :: comparatorRegex()
## @summary compares a string with a regular expression.
## @arg $console Required: console string/array of strings obtained from (console output)/(actual string) .
## @arg $regex Required: expected string/Regexp to match.
## @Returns     1 and attempted regexp if expected string is found in console string.
##              on failure returns 0 and attempted regexp.
#-----------------------------------------------------------------------------------------------------------------
sub comparatorRegex($$;%){
    my %opts = validate(@_, {
        console => { type=>SCALAR, default=>""},
        regex => { type=>SCALAR, default=>""},
        });

    my $regex = trim($opts{regex});
    my $res = ($opts{console} =~ m/($regex)/g)? 1:0;
    return [$res, $regex];
}

#-----------------------------------------------------------------------------------------------------------------
## @name :: trim()
## @summary  removes preceding andtrailing blank spaces from a string.
## @arg $string Required: string to trim.
## @Returns     trimed string..
#-----------------------------------------------------------------------------------------------------------------
sub trim($){
    my $string = shift;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::    expandxml()
## @summary expands a given testnode with variables expanded in multilines.
## @arg $testStepNode Required:  Xmlpath testnode, a hash containing the vars to be expanded.
## @arg \%hashOfRefs Optional: reference to hash of References to functions that returns an array of scalar items
## @arg \%hashOfArrays Optional: reference to hash of arrays having scalar items
## @Returns     Expanded testnode on success.  Returns 0 on failure.
## @Usage
##              my %opts=("arg1key"=>"arg1value"); arguments to be passed to the referenced function
##              my %opts1=("arg1key"=>"arg1value",...."argnkey"=>"argnvalue");
##
##              my %hashOfRefs=('@key1@'=>[\&subroutine,\%opts],
##                              '@key2@'=>[\&subroutine,\%opts1] );
##              my %hashOfArrays=('@sparedisks@',['disk1','disk2','disk3','disk4'],
##                                '@aggr@',['aggr1'],
##                                '@disks@',[@diskarray] );
##              $outputNode=expandXML(testnode => $testStepNode,
##                                    hashOfRefs=>\%hashOfRefs,
##                                    hashOfArrays =>\%hashOfArrays);
#-----------------------------------------------------------------------------------------------------------------
sub expandXML {
    my %opts = @_;
    my $node = $opts{testStepNode};
    my $input=$node->toString();

    my $exp_inp;
    my @nodelist;

    # Validates the hashOfArrays and hashOfRefs for its inputs.
    # invoke expandNode() if the validation passes.
    if ($opts{hashOfArrays}) {
        my %hashOfArrays = %{$opts{hashOfArrays}};
        if (validateHashOfArrays(%hashOfArrays)) {
            logcomment __PACKAGE__." : expandXML : validation of hashOfArrays passed\n";
        } else {
            logcomment __PACKAGE__." : expandXML : validation of hashOfArrays failed\n";
            return 0;
        }

        if ($opts{hashOfRefs}) {
            $exp_inp = expandNode(xmlString=>$input,hashOfArrays=>\%hashOfArrays,hashOfRefs=>$opts{hashOfRefs});
        } else {
            $exp_inp = expandNode(xmlString=>$input,hashOfArrays=>\%hashOfArrays);
        }
        if(!$exp_inp) {
            return 0;
        }
        $node= XML::XPath->new($exp_inp);
    } elsif ($opts{hashOfRefs}) {
        my %hashOfArrays;
        $exp_inp = expandNode(xmlString=>$input,hashOfArrays=>\%hashOfArrays,hashOfRefs=>$opts{hashOfRefs});
        if(!$exp_inp) {
            return 0;
        }
        $node= XML::XPath->new($exp_inp);
    }
    @nodelist=$node->findnodes('/');
    return $nodelist[0];
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::    validateHashofArrays()
## @summary expands a given testnode with variables expanded in multilines.
## @arg %hashOfArrays Optional: hash of arrays having scalar items.
## @Returns    If validation passes returns 1, else return 0.
#-----------------------------------------------------------------------------------------------------------------
sub validateHashOfArrays {
    my %hashOfArrays = @_;
    my @key = keys(%hashOfArrays);
    foreach (@key) {
        if (!ref $hashOfArrays{$_}) {
            logcomment "$hashOfArrays{$_} is not a reference variable\n";
            return 0;
        }
    }
    return 1;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::    expandNode()
## @summary internaly called by expandXML.
##              expands a given xml string with variables expanded in multilines.
## @arg $xmlString Required: Xml as a string
## @arg \%hashOfRefs Optional: reference to hash of References to functions that returns an array of scalar items
## @arg \%hashOfArrays Optional: reference to hash of arrays having scalar items
## @Returns     Expanded Xml in String format on success. Return 0 on failure.
#-----------------------------------------------------------------------------------------------------------------
sub expandNode {
    my %opts = @_;
    my $xml = $opts{xmlString};

    # If hashOfRefs is defined, then call expandRefs, else call expandArrays
    if ($opts{hashOfRefs}) {
        $xml=expandRefs ( xmlString=>$opts{xmlString}, hashOfArrays=>$opts{hashOfArrays},hashOfRefs=>$opts{hashOfRefs});
    } elsif ($opts{hashOfArrays}) {
        $xml=expandArrays ( xmlString=>$xml, hashOfArrays=>$opts{hashOfArrays});
    }
    return $xml;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::    expandRefs()
## @summary internaly called by expandNode.
##              generates array from hashOfRefs and adds it to hashOfArrays.
##              appends the attribute 'key's value to the attribute 'array'
##              internally calls expandArrays with new values.
## @arg $xmlString Required: Xml as a string
## @arg \%hashOfRefs Optional: reference to hash of References to functions that returns an array of scalar items
## @arg \%hashOfArrays Optional: reference to hash of arrays having scalar items
## @Returns     Expanded xmlstring on success, on failure returns 0
#-----------------------------------------------------------------------------------------------------------------
sub expandRefs {
    my %opts = @_;
    my $xml = $opts{xmlString};
    my %hashOfArrays = %{$opts{hashOfArrays}};
    my %hashOfRefs = %{$opts{hashOfRefs}};
    my %tempHashOfArrays = %hashOfArrays;

    # Creating a XML::Xpath element with the given xml string
    my $xp= XML::XPath->new($xml);

    # Find all attribute nodes named "hash"
    my @nodeSet=$xp->findnodes('//@hash');
    @nodeSet=reverse @nodeSet;

    # Iterate for each attribute node named "hash"
    foreach my $node (@nodeSet) {
        my $parent=$node->getParentNode();
        my $grandParent=$parent->getParentNode();
        my @parentAttrArr=$parent->getAttributes();
        my ($key,$hashAttr,$array_attr)=("","","");
        my (@hashArray,@attrKeys,@attrHash);

        # Iterate for all attribute nodes present in $parent
        foreach my $tmpAttr (@parentAttrArr) {
            my $tmpAttrName=$tmpAttr->getName();
            if($tmpAttrName eq "hash") {
                $hashAttr=$tmpAttr->getValue;
                @attrHash=split " ",$hashAttr;
                $parent->removeAttribute($tmpAttr);
            } elsif($tmpAttrName eq "key") {
                $key=$tmpAttr->getValue;
                @attrKeys=split " ",$key;
                $parent->removeAttribute($tmpAttr);
            } elsif($tmpAttrName eq "array") {
                $array_attr=$tmpAttr->getValue;
                $parent->removeAttribute($tmpAttr);
            }
        }
        my $childattr=XML::XPath::Node::Attribute->new("array",$array_attr." $key");

        # Updating the attribute field of the parent node.
        # Appending the attribute "key"'s value to attribute "array"
        $parent->appendAttribute($childattr);

        foreach (@attrKeys) {
            my $tKey=$_;
            my ($index) = $tKey =~ /\[(\d+)\]/;

            # If $tKey is of the format : @variable[index]@
            if(defined $index) {

                # Changing $tKey's the format to : @variable@
                $tKey =~ s/(\@.*)\[\d+\]/$1/;
                if(!exists $hashOfRefs{$tKey}) {
                    logcomment __PACKAGE__." : expandArrays : Data not found.  Skipping <".$grandParent->getName()."/".$parent->getName().">/\$hashOfRefs{$tKey}\n";
                    shift @attrHash if(@attrHash > 1);
                    next;
                }
            }else{
                if (!exists $hashOfRefs{$tKey}) {
                    logcomment __PACKAGE__." : expandArrays : Data not found.  Skipping <".$grandParent->getName()."/".$parent->getName().">/\$hashOfRefs{$tKey}\n";
                    shift @attrHash if(@attrHash > 1);
                    next;
                }
            }

            # @hashArray is the array that is used to expand the "key" variable in xml.
            if(@attrHash > 1) {
                @hashArray=executeHashOfRefs('functionRef'=>$hashOfRefs{$tKey},'hash'=>shift @attrHash);
            } else {
                @hashArray=executeHashOfRefs('functionRef'=>$hashOfRefs{$tKey},'hash'=>$attrHash[0]);
            }

            # Adding the array got from executing hashOfRefs into hashOfarrays
            $tempHashOfArrays{$tKey}=[@hashArray];
        }

    }

    my @tmpXp=$xp->findnodes('/');
    $xml=XML::XPath::XMLParser::as_string($tmpXp[0]);
    $xml=expandArrays ( hashOfArrays=>\%tempHashOfArrays,xmlString=>$xml );
    return $xml;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::    executeHashOfRefs()
## @summary internaly called by expandRefs.
##              Generates an array from hashOfRefs.
## @arg $xmlString Required: Xml as a string
## @arg \%hashOfRefs Optional: reference to hash of References to functions that returns an array of scalar items
## @Returns     Generates an array on success, on failure returns 0.
#-----------------------------------------------------------------------------------------------------------------
sub executeHashOfRefs {
    my %opts=@_;
    my $function=$opts{functionRef};
    my $hash=$opts{hash};

    #Not using shift below as it affects original %hashOfRefs
    my $funcName=$$function[0];
    my $funcArgs=$$function[1];

    my @return_array;
    return 0 if (!$funcName);
    if($funcName !~ /CODE/) {
            logcomment __PACKAGE__." : executeHashOfRefs : Not a function reference : \$hashOfRefs{$hash}.\n";
            return 0;
        }
    $$funcArgs{hash}=$hash;
    eval {
            @return_array=&$funcName(%{$funcArgs});
    };
    #return o if function fails to return any array.
    if ($@ or !@return_array) {
        logcomment __PACKAGE__." : executeHashOfRefs : Subroutine call for \$hashOfRefs{$hash} failed : $@";
        return 0;
    }
    return @return_array;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::    expandArrays()
## @summary Expands a given xml string with hashOfArrays variables expanded in multilines
##      internaly called by expandNode or expandRefs..
## @arg $xmlString Required: Xml as a string
## @arg \%hashOfArrays Optional: reference to hash of arrays having scalar items
## @Returns     Expanded xmlstring on success, on failure returns 0
#-----------------------------------------------------------------------------------------------------------------
sub expandArrays {
    my %opts = @_;
    my $xml = $opts{xmlString};
    my %hashOfArrays = %{$opts{hashOfArrays}};

    my %hashOfVars=();
    # Creating a XML::Xpath element with the given xml string
    my $xp= XML::XPath->new($xml);

    # Find all attribute nodes named "hash"
    my @nodeSet = $xp->findnodes('//@array');
    @nodeSet=reverse @nodeSet;

    # Iterate for each attribute node named "array"
    foreach my $node (@nodeSet) {
        my $parent=$node->getParentNode();
        my $grandParent=$parent->getParentNode();
        my @tmpPAttrArr=$parent->getAttributes();
        my $array_attr;
        my @attrArr;
        my %hashOfVars=();

        # Iterate for all attribute nodes present in $parent
        foreach (@tmpPAttrArr) {
            my $atr_name=$_->getName();
            if($atr_name eq "array") {
                $array_attr=$_->getValue();
                $parent->removeAttribute($_);
                @attrArr=split " ",$array_attr;
            }
        }
        my $parent_str=$parent->toString();

        # Iterate for all values present in attribte "array"
        foreach my $tAttr (@attrArr)  {
            $array_attr=$tAttr;
            my ($index) = $array_attr =~ /\[(\d+)\]/;

            # If $tKey is of the format : @variable[index]@
            if(defined $index) {
                # Changing $tKey's the format to : @variable@
                $array_attr =~ s/(\@.*)\[\d+\]/$1/;
                if (exists $hashOfArrays{$array_attr}) {
                    if ($index <= @{$hashOfArrays{$array_attr}}) {
                        my $tVal=$hashOfArrays{$array_attr}[$index-1];
                        $tAttr=quotemeta($tAttr);
                        if( $parent_str =~ /(\[|\{)\s*$tAttr\s*(\]|\})/) {
                            $parent_str =~ s/(\[|\{)\s*$tAttr\s*(\]|\})/$tVal/g;
                        }
                    }
                } else {
                    logcomment __PACKAGE__." : expandArrays : Data not found.  Skipping <".$grandParent->getName()."/".$parent->getName().">/\$hashOfArrays{$array_attr}\n";
                }
            } else {
                if (exists $hashOfArrays{$array_attr}) {
                    $hashOfVars{$array_attr}=$hashOfArrays{$array_attr};
                } else {
                    logcomment __PACKAGE__." : expandArrays : Data not found.  Skipping <".
                        $grandParent->getName()."/".$parent->getName().">/\$hashOfArrays{$array_attr}\n";
                }
            }
        }

        my ($refList,undef)=&generateCommands(command    => $parent_str,
                                              hashOfVars => \%hashOfVars,
                                              expandXML  => "true" );
        my $nochangeFlag=0;
        foreach my $tParentStr (@$refList) {
            if ($tParentStr eq $parent_str) {
                $nochangeFlag=1;
                next;
            }

            my $tmpXP = XML::XPath->new($tParentStr);
            foreach my $tNode ($tmpXP->findnodes('*')->get_nodelist()){
                $grandParent->insertBefore($tNode,$parent);
            }
        }

        if($nochangeFlag) {
            my $tmpXP = XML::XPath->new($parent_str);
            foreach my $tNode ($tmpXP->findnodes('*')->get_nodelist()){
                $grandParent->insertBefore($tNode,$parent);
            }
        }
        $grandParent->removeChild($parent);
    }
    @nodeSet=$xp->findnodes('/');
    $xml=XML::XPath::XMLParser::as_string($nodeSet[0]);
    return $xml;
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::    executeZAPI()
## @summary executes a given Zapi command on the filer and returns the output.
##             called internally by validateZAPI();
## @arg $command optional:  "The command to execute using ZAPI"
## @arg $host Required: The host object to the filer if #rsh connection is used( for 7mode execution ).
##                       becomes a mandatory parameter if (filermode eq "normal").
## @arg $testStepId Required: The teststep id of the TC that has to betested.
## @arg \%hashOfRefs Optional: reference to hash of References to functions that returns an array of scalar items
## @arg \%hashOfArrays Optional: reference to hash of arrays having scalar items
## @arg $inputXML optional: The xml to use as input for the zapitest call.
##                          Is Required if command => ""
## @arg $conntype  optional:  The connection type required for executing the command.
##                      Default  #rsh for (filermode eq "normal").
##                      Default  #console for all other cases.
## @arg $username optional:  The username to use to invoke zapitest.
##                           Default "root"
## @arg $password optional:  The password to use to invoke zapitest.
##                           Default undef
## @arg $attributes optional:  attributes found at the <command> tag.
##
## @Returns   The exact Zapi output in xml format, the error no, and the reason for the error.
#-----------------------------------------------------------------------------------------------------------------
sub executeZAPI {
    my %opts = validate(@_, {
        command => { type=>SCALAR, default=>""},
        attributes => { type=>HASHREF, default=>undef, optional=>1},
        inputXML => { type=>SCALAR, default=>"false", optional=>1},
        host => { type=>OBJECT, isa=>'Hostrec', optional=>1 },
        username => { type=>SCALAR, default=>"root", optional=>1 },
        password => {type=>SCALAR,default=>undef,optional=>1},
        });

    my $inputXML=$opts{inputXML};
    my $commandInput=$opts{command};
    my $username = $opts{username};
    my $Host= $opts{host};
    if ($Host->hosttype() =~ /filer|vsim-ng/) {
        #if 7G or 10mode do nothing.
    } elsif ($Host->hosttype() =~ /vsim/) {
        $ENV{ZAPI_USES_HTTPS}="yes";
    }

    my ($errNo,$reason,$outputString,$zapiObj,$parseData,$status,$returnObj);

    eval {
        $username = $opts{username};
        my $password;
        if($opts{password}) {
            $password=$opts{password};
        } else {
        $password=$Host->default_password();
        }
        my $Filer_ip  = $Host->default_ip();

        if ($inputXML eq "true" ){
            $zapiObj = new Zapi($Filer_ip, $username, $password);
            $parseData=$zapiObj->parse_raw_xml($commandInput);
            $returnObj= $zapiObj->invoke_elem($parseData,"any");

            if ( ! $returnObj->results_ok() ) {
                logcomment("Failure in invoking zapi: ".$returnObj->toString());
            }else{
                logcomment("success in invoking zapi: ".$returnObj->toString());
            }

            $outputString = $returnObj->toString();
            $errNo = $returnObj->results_errno();
            $reason = $returnObj->results_reason();
        } else {
            my $cmd_build=$commandInput;
            my @splitCmd = split (/\s+/,$cmd_build);
            my $even=-1;
            foreach (@splitCmd) {
                $even++;
                next if ($even == 0 );
                $_="" if ($_ eq "NULL" and ($even%2)==0);
            }
            my $server = new ZapiTest($Filer_ip, $username, $password);
            # New addition to fix 400 HTTP Bad format error
            $server->Zapi::set_legacy_http_conn(0);
            $returnObj= $server->Zapi::invoke_api(@splitCmd);
            if ( ! $returnObj->results_ok() ) {
                logcomment("Failure in invoking zapi: ".$returnObj->toString());
            } else {
                logcomment("success in invoking zapi: ".$returnObj->toString());
            }
            $outputString=$returnObj->toString();
            $errNo = $returnObj->results_errno();
            $reason = $returnObj->results_reason();
            logcomment("output=$outputString","result=$errNo");
        }
    }; #end of eval
    if ($@ =~ m/\d+/) {
        $errNo = $&;
        logcomment("$@ \n");
    }

    return ($outputString,$errNo,$reason);

}   ## end sub executeZAPI

#-----------------------------------------------------------------------------------------------------------------
## @name :: comparatorXML()
## @summary compares two XML strings based on set optional Parameters.
## @arg $console Required: console XML as string format.
## @arg $expected Required: expected XML as string format.
## @arg $attributes Optional: to compare the XML for attributes, set this as "true". Default is "false"
## @arg $text Optional: to compare the XML for text, set this as "true". Default is "false"
## @Returns     1 if expected XMLstring is found in console XMLstring.
##              on failure returns 0 and error strings.
#-----------------------------------------------------------------------------------------------------------------
sub comparatorXML {
    my %opts = validate(@_, {
                            console => { type=>SCALAR, default=>""},
                            expected => { type=>SCALAR, default=>""},
                            attributes=>{ type=>SCALAR, default=>"false", optional=>1},
                            text=> { type=>SCALAR, default=>"false", optional=>1},
                            regex=> { type=>SCALAR, default=>"false", optional=>1}
                            });

    my $xPathExp=XML::XPath->new($opts{expected});
    my $xPathCon=XML::XPath->new(trim($opts{console}));
    my ($root,%paths) = ("","");
    ($root,%paths) = &pathsPopulate($xPathExp);

    my ($tagResult,@errStrings)=&compareTAG(console=>$xPathCon, expected=>$xPathExp,expectedRoot=>$root,paths=>\%paths);
    my($fieldResult,@errFields);

    if($opts{attributes} eq "true" or $opts{text} eq "true") {
        ($fieldResult,@errFields)=&compareFields(console=>$xPathCon, expected=>$xPathExp,
                           expectedRoot=>$root, paths=>\%paths,
                           attributes=>$opts{attributes}, text=>$opts{text}, regex=>$opts{regex});
        if(!$tagResult or !$fieldResult) {
            push @errStrings,@errFields;
            return [0,join("",@errStrings)];
        }
    }

    return [$tagResult,join("",@errStrings)];
}

#-----------------------------------------------------------------------------------------------------------------
## @name :: pathsPopulate();
## @summary called by comparatorXML. Takes a XML string as a input and returns a hash
##            of all possible unique XPaths present in the xml and root node of the same.
## @arg $xmlString Required: XML as a string
## @Returns     returns root element name and hash of unique XML::XPaths, where key is the
##             Xpath and value is no.of times the xpath occurs in the XML string.
#-----------------------------------------------------------------------------------------------------------------
sub pathsPopulate {
    my ($PathExp)=@_;
    my %paths;
    my @inXML=$PathExp->findnodes('*');
    my $root=$inXML[0]->getName();
    $paths{'//'.$root}=1;
    if (!$PathExp->exists('//'.$root.'/*')) {
        return ($root,%paths);
    }
    my @tempPaths=&genXPaths($PathExp,'//'.$root);
    my $tmpPath;
    my $nodes;

    while (@tempPaths) {
        $tmpPath=shift @tempPaths;
        if (grep(/$tmpPath/,keys(%paths))) {
            next;
        } else {
            $nodes=$PathExp->findnodes($tmpPath);
            if($nodes->size()) {
                $paths{$tmpPath}=$nodes->size();
            } else {
                delete($paths{$tmpPath});
            }
        }
    }
    logcomment "Tags to be matched: ",Dumper(\%paths);
    return ($root,%paths);
}

#-----------------------------------------------------------------------------------------------------------------
## @name :: genXPaths();
## @summary called by pathPopulate(). Takes xpath object and ref path from where to check for xpaths.
## @arg $xpExp Required: xpath object
## @arg $path Required: reference path to start generating xpaths
## @Returns     returns an array of all XPaths present in the xml.
#-----------------------------------------------------------------------------------------------------------------
sub genXPaths {
    my ($xpExp,$path)=@_;
    my $t='/';
    our @paths;
    my @nodes=$xpExp->findnodes($path.'/*');
    foreach my $node (@nodes)
    {
        my $nodename=$node->getName();
        $path=$path.'/'.$nodename;
        if($xpExp->exists($path)) {
            push @paths,$path;
            genXPaths($xpExp,$path); # Recursively call to find all xpath's
        } else {
            $path=~s/(.*)$t\S*$t$nodename.*/$1/;
            redo;  # This is to solve the multi-sibling issue
        }
    }
    return @paths;
}

#-----------------------------------------------------------------------------------------------------------------
## @name :: compareTag();
## @summary   Does Tag level comparision between two XML strings.
##             Called by comparatorXML.
## @arg $console Required: console XML as string format.
## @arg $expected Required: expected XML as string format.
## @arg %paths Required: Hash of expected XPaths.
## @arg $expectedRoot Required: Tagname of root of the expected XML.
## @Returns     1 if expected XPath tags are found in console XMLstring.
##            On failure returns 0 and array of error strings;
#-----------------------------------------------------------------------------------------------------------------
sub compareTAG {
    my %opts = @_;
    my $actual=$opts{console};
    my $expected=$opts{expected};
    my $root=$opts{expectedRoot};
    my %paths=%{$opts{paths}};
    my @missingPaths;
    my ($flag,$pathFlag)=(1,0);

    my @actualContext=();
    eval{
        @actualContext=$actual->findnodes('//'.$root);
    };
    if($@){
        # $root itself missing
        logcomment $@,"\n";
        return (0, $root);
    }

    # Iterate through blocks(context) of console XML,
    # having node name same as root name of expectedXML.
    foreach my $context (@actualContext) {
        $flag=1;
        @missingPaths="";

        # Iterate through each expected Xpaths
        foreach my $path (keys(%paths)) {
            $path =~ s/\/\///g;
            my $tmpPath=$path;
            my $name=$context->getName();
            $tmpPath =~ s/$name\/(.*)/$1/; #removing name of context from the xpath

            # if context name is not removed : $path has only one element,
            # the root name.hence context can not be mentioned in subsequent call.
            if($tmpPath eq $path) {
                next;
            } else {
                if(!$actual->exists($tmpPath,$context)) {
                    push @missingPaths,"xmlpath $path is not found in Console output\n";
                    $flag=0;
                }
            }
        }

        # if all expected XPaths present in Console XML
        if($flag) {
            $pathFlag=1;
            # Iterate through each expected Xpaths
            foreach (keys(%paths)) {
                my $nodes=$actual->findnodes($_,$context);
                # if no of nodes in consoleXML having same $path
                        # no of nodes in expectedXML having same $path.
                if($nodes->size() < $paths{$_}) {
                    $pathFlag=0;
                } elsif ($paths{$_} > 1) {
                    my @actualParents;
                    my @expectedParents;

                    # Iterate through all nodes in consoleXML which match the XPath
                            foreach ($nodes->get_nodelist()) {
                        push @actualParents,$_->getParentNode->get_global_pos();
                    }

                    $nodes=$expected->findnodes($_);
                            # Iterate through all nodes in expectedXML which match the XPath
                    foreach ($nodes->get_nodelist()) {
                        push @expectedParents,$_->getParentNode->get_global_pos();
                    }

                    my $count;
                    my @tmpArray;
                    my @expcount;
                    my @actcount;
                    my ($i,$j)=(0,0);

                            # Iterate through all expected parent's global positions and
                            # get array having count of unique parent positions.
                    while(@expectedParents) {
                        $tmpArray[$i]=shift(@expectedParents);
                        $count=grep(/$tmpArray[$i]/,@tmpArray);
                        if($count==1)
                        {
                            $count=grep(/$tmpArray[$i]/,@expectedParents);
                            $count ++;
                            $expcount[$j]=$count;
                            $j++;
                        }
                        $i++;
                    }
                    ($i,$j)=(0,0);

                    @tmpArray="";
                            # Iterate through all console parent's global positions and
                            # get array having count of unique parent positions.
                    while(@actualParents) {
                        $tmpArray[$i]=shift(@actualParents);
                        $count=grep(/$tmpArray[$i]/,@tmpArray);
                        if($count==1)
                        {
                            $count=grep(/$tmpArray[$i]/,@actualParents);
                            $count ++;
                            $actcount[$j]=$count;
                            $j++;
                        }
                        $i++;
                    }

                    @expcount=sort(@expcount);
                    @actcount=sort(@actcount);
                    while(@expcount and @actcount) {
                        $pathFlag=0 if(pop(@expcount) > pop(@actcount));
                    }
                    $pathFlag=0 if(@expcount);
                }
            }
        }
    }

    if ($pathFlag) {
        return 1;
    } else {
        return (0,@missingPaths);
    }
}

#-----------------------------------------------------------------------------------------------------------------
## @name :: compareFields();
## @summary called by comparatorXML. Does Attribute and Text level comparision between two XML strings.
## @arg $console Required: console XML as string format.
## @arg $expected Required: expected XML as string format.
## @arg %paths Required: Hash of Expected XPaths.
## @arg $expectedRoot Required: Tagname of root of the expected XML.
## @arg $attributes Required: set to "true" if attribute level comparison is required.
##                             Default is "false".
## @arg $text Required: set to "true" if text level comparision is required.
##                             Default is "false".
## @Returns     1 if expected Attributes and/or Texts are found in console XMLstring.
##            On failure returns 0 and array of error strings;
#-----------------------------------------------------------------------------------------------------------------
sub compareFields {
    my %opts=@_;
    my $actual=$opts{console};
    my $expected=$opts{expected};
    my $root=$opts{expectedRoot};
    my %paths=%{$opts{paths}};

    my (@expArrayAttrs,@actArrayAttrs,@expArrayTexts,@actArrayTexts,@missingFields);
    my $flag;

    my @actualContext=();
    eval{
        @actualContext=$actual->findnodes('//'.$root);
    };
    if($@){
        # $root itself missing
        return (0, $root);
    }

    # Iterate through each expected Xpaths
    foreach my $path (keys(%paths)) {
        $path =~ s/\/\///g;
        my @expNodes = $expected->findnodes('/'.$path);

        # Iterate through all nodes in expected which match the XPath
        foreach my $expNode (@expNodes) {
            # if attributes option is set
                if($opts{attributes} eq "true") {
                my @expAttrs=$expNode->getAttributes();
                # Iterate through all attributes of the current node.
                foreach my $expAttr (@expAttrs) {
                    push @expArrayAttrs,$path.'::'.$expAttr->getName().'='.$expAttr->getValue();
                }
            }

            # if text option is set
            if($opts{text} eq "true") {
                # Iterate through all childnodes of current node
                foreach ($expNode->getChildNodes()) {
                    my $expValue=$_->getValue();

                    # If text field of childnode is not empty
                    if(($expValue and $expValue !~ /^\s*$/) or $expValue =~ /^\s*0\s*$/) {
                        push @expArrayTexts,$path.'::'.trim($expValue);
                    }
                }
            }
        }
    }

    # Iterate through blocks(context) of console XML,
    # having node name same as root name of expectedXML.
    foreach my $context (@actualContext) {
        @actArrayTexts="";
        @actArrayAttrs="";

        # Iterate through each expected Xpaths
        foreach my $path (keys(%paths)) {
            $path =~ s/\/\///g;
            my $tmpPath=$path;
            my $name=$context->getName();
            $tmpPath =~ s/$name\/(.*)/$1/; #removing name of context from the xpath
            my @actNodes;

            # if context name is not removed : $path has only one element,
            # the root name.hence context can not be mentioned in subsequent call.
            if($tmpPath eq $path) {
                @actNodes = $actual->findnodes('//'.$tmpPath);
            } else {
                @actNodes = $actual->findnodes($tmpPath,$context);
            }

            # Iterate through all nodes in consoleXML which match the XPath
            foreach my $actNode (@actNodes) {
                        # if attributes option is set
                if($opts{attributes} eq "true") {
                    my @actAttrs=$actNode->getAttributes();
                    foreach my $actAttr (@actAttrs) {
                        push @actArrayAttrs,
                            $path.'::'.$actAttr->getName().'='.$actAttr->getValue();
                    }
                }

                # if text option is set
                if($opts{text} eq "true") {
                    foreach ($actNode->getChildNodes()) {
                        my $actValue=$_->getValue();
                        if(($actValue and $actValue !~ /^\s*$/) or ($actValue  =~ /^\s*0\s*$/)) {
                            push @actArrayTexts,$path.'::'.trim($actValue);
                        }
                    }
                }
            }
        }

        $flag=1;
        if($opts{attributes} eq "true") {
            logcomment "\nDEBUG :: Expected : " if $DEBUG;
            if ($DEBUG) {
                 logcomment "\"$_\"" foreach (@expArrayAttrs);
            }
            logcomment "\n \t \tActual :  " if $DEBUG;
            if ($DEBUG) {
                logcomment "\"$_\"" foreach (@actArrayAttrs);
            }

            my $output=comparatorCLI(console=>\@actArrayAttrs,expected=>\@expArrayAttrs,exact=>"true");
            if(!${$output}[0]) {
                $flag=0;
                push @missingFields,"Missing attribute ${$output}[1]\n";
            }
        }

        if($opts{text} eq "true") {
            my $output=comparatorCLI(console=>\@actArrayTexts,expected=>\@expArrayTexts, regex=>$opts{regex});
            if(!${$output}[0]) {
                $flag=0;
                push @missingFields,"Mismatch in Text ${$output}[1]\n";
            }
        }

        if($flag) {
            return 1;
        }
    }
    return (0,@missingFields);
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::    random_name_generate()
## @summary This subroutine returns random name prefixed with the argument.
## @Usage       random_name_generate(type=>"aggr")
## @arg $type Required:  Prefix string that needs to be appended to the randomly generated string.
## @Returns     random string. For eg: "<prefix_string>_<random_string>"
#-----------------------------------------------------------------------------------------------------------------
sub random_name_generate{
    my ($name, %opts, $dir_name, $file_name, $file);
    (%opts) = validate(@_, {type   => {type => SCALAR}});
    $name=$opts{type};

    #tmpnam returns the complete path of the file which is going to be created
    $file = tmpnam();

    ($dir_name, $file_name) = $file =~ m/(.*\/)(.*)$/;

    return "$name" .'_'. "$file_name";
} ## end sub random_name_generate

#-----------------------------------------------------------------------------------------------------------------
## @name ::    _get_ontap_version()
## @summary Gets the ontap version of filer/vsim for MAINT mode.
## @arg $filerconn Required:  connection object(filerconn).
## @Retuns ontap version of filer/vsim in numerical format.
##    Internally called by invoke_zapi_cli().
#-----------------------------------------------------------------------------------------------------------------
sub _get_ontap_version {
    my ($conn)=@_;
    my $version;

   logcomment("Getting the version in $conn filermode");
    if($conn =~ /normal|10modeNODESCOPE|10modeNGSH/) {
        my $Node = NACL::STask::Node->find();
        my $version = build_version($Node);
        logcomment('ONTAP Version is :  ' . $version);
        my $vers = codename_to_version($version);
        return $vers;
    } else {
        $conn = "#console";
        # getting the version string for 7modeMAINT and 10modeMAINT
        my ($match, $before, $text) = say($conn, "version", -prompt, "\\*>", -timeout, 120);

        if ($text and $text =~ /(?:NetApp|Data ONTAP) Release (\d+\.\d+\S*)\s*:/) {
            $version = $1;
        } elsif ($text and $text =~ /(?:NetApp|Data ONTAP) Release (\w+.*?:)/) {
            $version = $1;
        }
        #converting the version string to numerical values
        if ($version =~ /^(\d+\.\d+)/) {
            # return numeric versions immediately
            return $1;
        } elsif ($version =~ /^R?\w+N?_\d+/) {
            my $vers = codename_to_version($version);
            $vers =~ /^(\d+\.\d+).*/;
            return $1;
        } else {
            die "can't determine numeric release: filer down/innacessible?";
        }
    }
}

#-----------------------------------------------------------------------------------------------------------------
## @name ::    _get_default_username()
## @summary :: Gets default username of filer/vsim.
## @Returns :: Default username for 7mode/Cmode.
##    Internally called by invoke_zapi_cli().
#-----------------------------------------------------------------------------------------------------------------
sub _get_default_username {
    #Getting default username for 7mode/Cmode
    my @node_list = NACL::STask::Node->find();
    my $Node = $node_list[0];
    my $hostrec = host ($Node->name());
    my $username = $Node->hostrec->default_username();
    return $username;
}

1;

##################################### EOF #################################################


=cut
TODO:
validateXML:
    validate each xml tags for array, hash and key values

1-April:
    Change all logcomment statements to logcomment statements
    update zapis part in invoke_zapi_cli() where <inputxml> type tags are not required
    validate regex for zapis as well
    write comparatorZAPIs and integrate for <expected> <notExpected> with <fields>
    have expandXML similar to rephraseExpectedStrings for <inputxml>, <required> and <values>
        based on these validate multiple XML strings.

    Minor fixes:
        put second level logcomment statements in comparatorCLI and comparatorRegex
        Update all logcomment messages to denote the package and api name shown.

        Run executeZAPI and release atleast initial version of framework for ZAPIs
        Take care of DEFAULTHASH

Completed:
    put a check if any of the tags has empty text node and fail for such nodes.
        e.g. <node/>
    Install XML::XPath on NATE   - DONE
    in executeCLI change the temp cmd statement to execute statement using say or cli_say.
        return output as single string - DONE
    <required> tags logic changes in <notExpectedResults> case, have to update this for fields and regex - DONE
    update new combinatorCLI - DONE



Porting guidelines from pl file to pm:
1. change all logcomment statements to logcomments
2. uncomment all lines that has comment as "Uncomment this".
3. comment all lines that has comment as "Comment this".

#-----PARSER-----
# Parse XML file
# extract the node that refers to particular step
# If cli or zapi:
#   run commands/Zapis combinator
#   attach the expanded cmds to extracted node
#----EXECUTION----
# If zapi:
#   if <inputxml> exists,
#       expand the $var$ within xmls
#       If array attribute exists:
#           expand the child node and replace all occurances of [array] with value of array
#       If hash, key attribute exists:
#           expand the child node and replace all occurances of {hash} with values from hash(based on its scalar or array}
#       after expansion use different method to invoke zapi with xml input
#   else:
#       invoke zapi in normal mode
#   logcomment the executed zapi with xml that is fed as input
# If cli:
#   Execute each command
#   log comment the command executed
#----COMPARATOR----
#   ---Fields comparator ---
# If <fields> is present, Expand the output of <required> and <valued> individually
#   use the $var$ and @var@ from <cmds> to expand the text nodes
#   If execution is cli
#       If hash and key exists,
#           replace the {hash} with scalar value from array
#       If array exists
#           replace each line having [array] with value from array
#   if execution is zapi
#       If array exists
#           and duplicate the node with all its child nodes and replace [array] construct with value of @array@
#       If hash and key exists,
#           If value is scalar, replace the {hash} construct with the value
#           If value is array, duplicate the node with all its child nodes and replace the {hash} construct with each value of array item
#       wherever repeat="count" attribute is provided, then count for number of occurences of that node in the zapi output under that parent node.
#   Now compare the expected result from xml with that of console output
#   Based on required/values and its attributes like order="true" or exact="true" return results and logcomment the results
#
#   ---Regex comparator---
#   If <regex> is present then run regular expression comparsion for each <required> and <values> nodes
#   log comment each step of comparison
#
#----FINAL RESULT----
# Based on comparator for all tags of <fields> and <regex> from <expectedResults> and <notExpectedResults> send out the message as passed or failed.



#&main_Stub();

sub main_Stub{
    my $cliSignature= 'input.xml';
    #my $teststepId = 'cli_1b';
    my $teststepId = 'combinator_2a';
    #my $teststepId = 'combinator_zapi';

    my %hashOfVars=('@w@',['w1','w2','w3','w4'],
                  '$E$',42,'@r@',['r1','r2'],
                  '@y@',['y1','y2','y3'],
                  '$LL$','45',
                  '$var1$','test',
                  '@var2@', ['a', 'b'],
                  '@aggr@', ['za1']);
    my %hashOfArrays =('@w@',['w1','w2','w3','w4'],
                  '@y@',['y1','y2','y3'],
                  '@disks@', ['0c.26', '0c.17', '0c.28', '0c.29', '0c.27'],
                  '@aggr@', ['za1']);
    my %opts;
    my %hashOfRefs = (pollingFunction=>[\&stub_polling_func, %opts],
                      checkAggr=>[\&checkAggr, %opts],
                      );

    logcomment &invoke_zapi_cli(cliSignature=>$cliSignature,
                           teststepId=>$teststepId,
                           hashOfVars=>\%hashOfVars,
                           hashOfArrays=>\%hashOfArrays,
                           hashOfRefs=>\%hashOfRefs
                           );
    #logcomment &invoke_zapi_cli(cliSignature=>$cliSignature, teststepId=>$teststepId, hashOfRefs=>\%hashOfRefs);
}

sub stub_polling_func{
    my %opts = @_;
    logcomment "Polling:  Do some task...\n";
    return 0;
}

sub checkAggr{
    my %opts = @_;
    # logcomment "Polling:  Do some task...\n";
    return 1;
}
=cut
