# $Id: //depot/prod/ontap/main/test/lib/CIFS/Cifs_cmd.pm
# #
# ## @summary Libraries using NACL API Framework, used to test ZAPI and CLI commands
# ##
# ## @description
# ##              These routines  are used primarily by all ZAPI and CLI in C Mode
# ##              
# ##
# ##
# ## use Tharn;
# ##
# ## @author  rohith@netapp.com
# ## @author   susmitha@netapp.com
# ##                

package Zapi_cmode;

require Exporter;
use strict;
use Tharn;
use SayWrap qw(sayEOF);

use Spreadsheet::ParseExcel;
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use Data::Dumper;

use HTML::HashTable;


use vars qw (@ISA @EXPORT_OK);

@ISA = qw(Exporter);
@EXPORT_OK = qw (
	    new
		get_total_comb
        get_input_params 
        get_exp_output_result
		execute_zapi
		execute_cli
		verify_output
		verify_output_cli
		logValues
		dumpzapilogs
		);

################ Global Variables that should not be changed ###########
my $zapi_name_col   = '1';
my $numcomb_col     = '2';
my $numinparam_col  = '3';
my $startinrow_col  = '4';
my $numoutparam_col = '5';
my $startoutrow_col = '6';
my $outputcmd_col   = '7';
my $test_desc_col   = '1';

 


######################################################################################################################################

sub new {
    my($class) = @_;        # Class name is in the first parameter
    my $self = {};  # Anonymous hash reference holds instance attributes

    bless($self, $class);          # Say: $self is a $class
    return $self;
}


##########################################################################################################################################
# Name 		: find_zapi_row
# Desc 		: This function takes the name of ZAPI and worksheet, calculates which row has the corresponding values
# 			  and accordingly returns all the column values in that row.
# Inputs	: ZAPI Name ; 1st Worksheet of the excel workbook which has all the information
# Outputs	: numcomb      = Number of Combinations specified in the excel sheet for the zapi/cli
# 			  numinparam   = No. of Input parameters for the specified ZAPI/CLI
# 			  actinrow     = Actual row in the excel sheet 2 of specified work-book, where the parameter details for ZAPI/CLI exist.
# 			  numoutparam  = No. of Output parameters for the specified ZAPI/CLI
# 			  actoutrow    = Actual row in the excel sheet 3 of specified work-book, where the expected output details for ZAPI/CLI exist.
##########################################################################################################################################

sub find_zapi_row {
    my $zapi_name    = shift;
    my $worksheet    = shift;

    my ( $row_min, $row_max ) = $worksheet->row_range();

    for my $row ( $row_min .. $row_max ) {

        my $cell = $worksheet->get_cell( $row, $zapi_name_col );
        next unless $cell;
        if ($cell->value() eq $zapi_name) {

            my $numcomb  = $worksheet->get_cell($row, $numcomb_col)->value();
            my $numinparam = $worksheet->get_cell($row, $numinparam_col)->value();
            my $actinrow = $worksheet->get_cell($row, $startinrow_col)->value();
            my $numoutparam = $worksheet->get_cell($row, $numoutparam_col)->value();
            my $actoutrow = $worksheet->get_cell($row, $startoutrow_col)->value();

            return ($numcomb,$numinparam,($actinrow -1), $numoutparam, ($actoutrow -1));
        }
    }
    return -1;
}




##########################################################################################################################################
#Name 		: get_hashmap_one_record
#Desc		: This function is used to return all the parameters and corresponding values for the combination of inputs provided
#Input		: actrow	= Actual Row of the ZAPI where details regarding parameter names/values exist
#			  combo_num = Which particular specified combination of parameters have to be executed
#			  numparam  = Number of Input/output parameters which is essential to collect all information
#			  worksheet = Which worksheet has to be used (worksheet1 for Input combinations, worksheet 2 for exp-output combinations
#Output		: param_values = Key-value combinations indicating parameter-value pairs.			  
##########################################################################################################################################			  
sub get_hashmap_one_record {
    my $self	  = shift;
	my $actrow    = shift;
    my $combo_num = shift;
    my $numparam  = shift;
    my $worksheet = shift;

    my %param_values = ();

    for my $col (0..$numparam-1) {
	

        my $cell_param = $worksheet->get_cell($actrow, $col+3); #Parameters in Sheet 2 start with 4th column (0-3)
        my $cell_value = $worksheet->get_cell($actrow+$combo_num, $col+3); #Parameters in Sheet 2 start with 4th column (0-3)


			my $key   = $cell_param->value(); # actual parameter name
		


            if ($cell_value ne '')  
			{
			  my $value = $cell_value->value(); #Value of the parameter
              if ($value ne '')  
			  {
			    if(substr($key,-2) eq "[]") # The parameter will have an array of values
				{ 
					push @{ $param_values{substr($key,0,-2)} }, split(/,/, $value);
				}

                elsif(substr($key,-1) eq '%')  # The parameter is a hash
				{
					chomp;
    			    my @pairs = split /,/, $value;   # spaces not taken into consideration - work on it later
					my $hash;   
				    for (@pairs) 
					{ 
				       my ($k, $v) = split /=/; # spaces not taken into consideration - work on it later
					   if(substr($k,-2) eq "[]") # This indicates, one parameter value in turn  has array of values.
					   {
							my @list_of_values = split(/:/, $v);
							foreach my $val (@list_of_values)
						    {
						    	push @{$hash->{substr($k,0,-2)}} , $val ;
							}
					   }	
					   else
					   {	
        		         $hash->{$k} = $v; 
					   }					
    				} 
				    $param_values{substr($key,0,-1)} = $hash;
					   	 
                }
                else
			    {
                	$param_values{$key} = $value;
			    }
	
			  }
            }
           # else {
           #     $param_values{$cell_param->value()} = '';
           # }

        }

    return %param_values;

}


##########################################################################################################################################
#Name		: get_exp_output_result
#Desc		: This function checks for the expected status for a particular combination of ZAPI, which is filled up in the excel-sheet.
#			  If the expected output is passed, it simply returns all the exp-output parameter-value pairs. 
#			  If the expected output is failed, it returns the expected errorcode and result
#Input		: zapi_name    = name of the zapi
#			  zapi_doc     = Excel sheet to be used
#			  combi_num    = Combination number of the zapi for which, we need the expected output
#Output		: output_param_values = All output paramater-value pairs if the expected output is passed
#			  return_output		  = Error code and reason, if the expected output is failed.
#########################################################################################################################################

sub get_exp_output_result {
	my $zapi_name   = shift;
	my $zapi_doc    = shift;
	my $combi_num   = shift;
    

    	my $parser   = Spreadsheet::ParseExcel->new();
         my $workbook = $parser->parse($zapi_doc);
    
         if ( !defined $workbook ) {
             die $parser->error(), ".\n";
         }
	
    my $worksheet1 = $workbook->worksheet('zapis');
    my $worksheet3 = $workbook->worksheet('zapi_exp_outputs');

    ( my $numcomb, my $numinparam, my $actinrow, my $numoutparam, my $actoutrow) = find_zapi_row($zapi_name, $worksheet1);


    my %output_param_values = get_hashmap_one_record("aa",$actoutrow, $combi_num, $numoutparam, $worksheet3);

    if (($output_param_values{'status'}) eq "failed") {
	
		my %return_output;

	    $return_output{'status'} =  $output_param_values{'status'};
		
		if($output_param_values{'errorcode'} ne '') {
			$return_output{'errorcode'} = $output_param_values{'errorcode'};
		}

		if($output_param_values{'reason'} ne  '') {
			$return_output{'reason'} = $output_param_values{'reason'};
		}

		return %return_output;
    }
	return %output_param_values;
}



##########################################################################################################################################
#Name		: get_input_params
#Desc		: This function returns all the input parameters for a given zapi name and combination as a hash
#Input		: zapi_name    = name of the zapi. If it is setup or cleanup, returns all setup cleanup values for the setup/cleanup instance
#			  zapi_doc     = Excel sheet to be used
#			  combi_num    = Combination number of the zapi for which, we need the expected output
#Output		: param_values = All input paramater-value pairs if the expected output is passed
#########################################################################################################################################
sub get_input_params {
        my $zapi_name = shift;
        my $zapi_doc  = shift;
        my $combi_num = shift;


	my $parser   = Spreadsheet::ParseExcel->new();
    my $workbook = $parser->parse($zapi_doc);

	if ( !defined $workbook ) {
        die $parser->error(), ".\n";
    }

    my $worksheet1 = $workbook->worksheet('zapis');
    my $worksheet2 = $workbook->worksheet('zapi_inputs');

    (my $numcomb, my $numinparam, my $actinrow, my $numoutparam, my $actoutrow) = find_zapi_row($zapi_name, $worksheet1);

	if($zapi_name eq 'setup' || $zapi_name eq 'cleanup')
	{
		$actinrow = $actinrow+(($combi_num)*2);
		$numinparam = $worksheet2->get_cell($actinrow-1, $numinparam_col-1)->value();
	    my %param_values = get_hashmap_one_record("bb", $actinrow-1, 1, $numinparam, $worksheet2);
		return %param_values;

	}
	else
	{
    	my %param_values = get_hashmap_one_record("bb", $actinrow, $combi_num, $numinparam, $worksheet2);
		my $test_desc = $worksheet2->get_cell($actinrow+$combi_num,$test_desc_col);
		if($test_desc ne '') { $param_values{'test-description'} = $test_desc->value(); }
		return %param_values;
	}
    

}


##########################################################################################################################################
#Name		: get_total_comb
#Desc		: This function returns the total number of combinations to be tested, as listed in the excel sheet for a particular zapi
#Input		: zapi_name    = name of the zapi
#			  zapi_doc     = Excel sheet to be used
#Output		: return_values = No. of Combinations if zapi is listed in excelsheet, -1 otherwise
#########################################################################################################################################


sub get_total_comb {
	my $self      = shift;
	my $zapi_name = shift;
    my $zapi_doc  = shift;

    my $parser   = Spreadsheet::ParseExcel->new();
    my $workbook = $parser->parse($zapi_doc);

    if ( !defined $workbook ) {
          die $parser->error(), ".\n";
    }

    my $worksheet = $workbook->worksheet('zapis');

    my ( $row_min, $row_max ) = $worksheet->row_range();

    for my $row ( $row_min .. $row_max ) {

        my $cell = $worksheet->get_cell( $row, $zapi_name_col );
        next unless $cell;
        if ($cell->value() eq $zapi_name) {

            my $numcomb  = $worksheet->get_cell($row, $numcomb_col)->value();
            return ($numcomb);
        }
    }
    return -1;
 

}

##########################################################################################################################################
#Name		: get_outputtest_command
#Desc		: This function returns the output test command that needs to be executed for the given zapi, as specified in excel sheet.
#Input		: zapi_name    = name of the zapi
#			  zapi_doc     = Excel sheet to be used
#Output		: return_values = Returns the corresponding output test zapi to be executed, -1 if nothing is specified.
#########################################################################################################################################
sub get_outputtest_command {
    my $zapi_name = shift;
    my $zapi_doc  = shift;

    my $parser   = Spreadsheet::ParseExcel->new();
    my $workbook = $parser->parse($zapi_doc);

    if ( !defined $workbook ) {
          die $parser->error(), ".\n";
    }

    my $worksheet = $workbook->worksheet('zapis');

    my ( $row_min, $row_max ) = $worksheet->row_range();

    for my $row ( $row_min .. $row_max ) {

        my $cell = $worksheet->get_cell( $row, $zapi_name_col );
        next unless $cell;
        if ($cell->value() eq $zapi_name) {

            my $outputtest_cmd  = $worksheet->get_cell($row, $outputcmd_col)->value();
            return ($outputtest_cmd);
        }
    }
    return -1;


}

##########################################################################################################################################
#Name		: run_setup_teardown
#Desc		: This function executes all the zapi commands as specified in excel sheet for setup and teardown modules
#Input		: zapi_name    = name of the zapi
#			  excel_workbook     = Excel sheet to be used
#			  Api_Set_Obj		= Handler for NACL APISet to invoke ZAPIs
#			  command			= Command to be executed (valid values are "setup" and "cleanup")
#Output		: return_values =  "Return nothing as of now!!"
#########################################################################################################################################

sub run_setup_teardown {
	my $self	       = shift;
	my $excel_workbook = shift;
	my $Api_Set_Obj    = shift;
	my $command		   = shift;

    
	logcomment("in run_setup_teardown module");

	my $numcomb  = get_total_comb('bb',$command, $excel_workbook);


    for(1..$numcomb){
		my %setup_input_params = get_input_params($command,$excel_workbook, $_);

		my $zapi_command_to_execute = $setup_input_params{'command'};

		$zapi_command_to_execute =~ tr/-/_/;
		
		delete($setup_input_params{'command'});
		try {	
			if (exists $setup_input_params{'primary_key'}) {
					
				my $primary_key = $setup_input_params{'primary_key'};
				my $value       = $setup_input_params {$primary_key};
				my $count = $setup_input_params{'count'};
	
				delete ($setup_input_params{'count'});                 ## These are for our purpose and should not be passed to library.
				delete ($setup_input_params{'primary_key'});
				delete ($setup_input_params{$primary_key});

				if (exists $setup_input_params {'vserver-name-context'} && $primary_key ne 'vserver-name-context') {
					my $vserver_val = $setup_input_params{'vserver-name-context'};
					$Api_Set_Obj->clear_vserver();
					$Api_Set_Obj->set_vserver('vserver-name' => $vserver_val);
					logcomment($vserver_val);
					delete ($setup_input_params{'vserver-name-context'});
				}


			
				for(1..$count) {
					
					try
					{	
						if ($primary_key eq 'vserver-name-context') {
							logcomment($primary_key);
							logcomment($value.$_);
							$Api_Set_Obj->clear_vserver();
							$Api_Set_Obj->set_vserver('vserver-name' => ($value.$_));
							logcomment($Api_Set_Obj->get_vserver());
							my $print_cmd = "xlate_".$zapi_command_to_execute;
							logcomment($Api_Set_Obj->$print_cmd(%setup_input_params));
							logcomment(Dumper(%setup_input_params));
							my $result = $Api_Set_Obj->$zapi_command_to_execute(%setup_input_params);
						}
						else {
							my $print_cmd = "xlate_".$zapi_command_to_execute;
							logcomment($Api_Set_Obj->$print_cmd(%setup_input_params, $primary_key => ($value.$_)));
							logcomment($Api_Set_Obj->get_vserver());
							logcomment($primary_key.$value.$_);
							my $result = $Api_Set_Obj->$zapi_command_to_execute(%setup_input_params,$primary_key => ($value.$_));
						
						}
					}catch NACL::APISet::Exceptions::ResponseException with
					{
						logcomment("encountered exception for combination : " . $_);
            			my $exception_object = shift;
						logcomment("Error is: " . $exception_object->text());
       	    			my $response = $exception_object->get_response_object();
						logcomment(Dumper($exception_object->text()));
					};
		
				}
			}
			else {
				my $result = $Api_Set_Obj->$zapi_command_to_execute(%setup_input_params);
			}	
		}catch NACL::APISet::Exceptions::ResponseException with
		{
			logcomment("encountered exception for combination : " . $_);
            my $exception_object = shift;
			logcomment("Error is: " . $exception_object->text());
       	    my $response = $exception_object->get_response_object();
			logcomment(Dumper($exception_object->text()));
		};
	}
	return 0;
}


##########################################################################################################################################
#Name		: execute_cli
#Desc		: This function is called by actual thpls to execute CLI commands
#Input		: cli_name    = name of the cli
#			  excel_workbook     = Excel sheet to be used
#			  row_num			= Which Combination has to be executed for the particular cli
#			  Api_Set_Obj		= Handler for NACL APISet to invoke CLIs
#			  Vserver			= If the corresponding CLI uses a vserver, takes the value 1; 0 otherwise
#Output		: return_values =  "Return nothing as of now!!"
#########################################################################################################################################
sub execute_cli {
	my $self           = shift;
	my $cli_name      = shift;
	my $excel_workbook = shift;
	my $row_num        = shift;
	my $Api_Set_Obj    = shift;
	my $vserver		   = shift;

	my $test_cmd = $cli_name;
	$test_cmd =~ tr/-/_/ ;
	
	### This is for vserversied CLIs
	if($vserver == 1) {
		$test_cmd = "vserver_".$test_cmd;
	}
	
    my %org_input_param = get_input_params($cli_name, $excel_workbook, $row_num);
	my %input_param = %org_input_param;
	delete($input_param{'test-description'});
	
	try 
	{
      my $response = $Api_Set_Obj->$test_cmd(%input_param);   #Invoke the ZAPI for the specifying the input params
	  logcomment(" Output of the executed CLI is  ");

	  logcomment(Dumper($response->get_raw_output()));
   	  my %output_result = get_exp_output_result($cli_name, $excel_workbook, $row_num);
						
	  return (0, \%org_input_param, \%output_result);

	}
 	catch NACL::APISet::Exceptions::ResponseException with
    {
            my $exception_object = shift;
			logcomment("Error is: " . $exception_object->text());
       	    my $response = $exception_object->get_response_object();
			my %output_result = get_exp_output_result($cli_name, $excel_workbook, $row_num);
			$output_result{'Error'} =  $exception_object->text();

            if ($response->get_execution_status() ne $output_result{'status'} ) ## Expeted result differs from result obtained.
            {
				return (-1, \%input_param, \%output_result);
            }
			return (0, \%input_param, \%output_result);
    };
}
#########################################################################################################################################
sub execute_cli_inter {
        my $self           = shift;
        my $cli_name      = shift;
        my $vserver_name = shift;
        my $cifs_server1    = shift;
	my $domain1 = shift;
	my $password = shift;
        my $Api_Set_Obj    = shift;
        my $vsrvr    = shift;
        my $test_cmd = $cli_name;
        $test_cmd =~ tr/-/_/ ;

        ### This is for vserversied CLIs
        if($vsrvr   == 1) {
                $test_cmd = "vserver_".$test_cmd;
        }

    #my %org_input_param = get_input_params($cli_name, $excel_workbook, $row_num);
        #my %input_param = %org_input_param;
        #delete($input_param{'test-description'});

        my $prompts_answers = ['.*Enter the user name:*'=> 'administrator','.*Enter the password:*'=> "$password"];
	my $response = $Api_Set_Obj->$test_cmd('vserver'=>"$vserver_name",'cifs-server'=>"$cifs_server1",'domain'=>"$domain1",
                        'connectrec-match_table' => $prompts_answers);
	
          logcomment(" Output of the executed CLI is  ");

          logcomment(Dumper($response->get_raw_output()));
	my @parsed_output = @{$response->get_parsed_output()};
	my $vnam = $parsed_output[0]->{'vserver'};
		if ($vnam eq $vserver_name) {

          return (0);                                                     
        } else {
		return (1);

}}
##########################################################################################################################################
#Name		: execute_zapi
#Desc		: This function is called by actual thpls to execute ZAPIs
#Input		: zapi_name    = name of the zapi
#			  excel_workbook     = Excel sheet to be used
#			  row_num			= Which Combination has to be executed for the particular zapi
#			  Api_Set_Obj		= Handler for NACL APISet to invoke ZAPIs
#			  Vserver			= If the corresponding ZAPI uses a vserver, takes the value 1; 0 otherwise
#Output		: return_values =  Input params used, Expected output as per excel sheet and Observed Output on ZAPI execution
#							   Also returns -1 if execution status different from expected status ; 0 otherwise.
#########################################################################################################################################

sub execute_zapi {
	my $self           = shift;
	my $zapi_name      = shift;
	my $excel_workbook = shift;
	my $row_num        = shift;
	my $Api_Set_Obj    = shift;
	my $vserver		   = shift;

	my $test_cmd = $zapi_name;
	$test_cmd =~ tr/-/_/ ;

    my %org_input_param = get_input_params($zapi_name, $excel_workbook, $row_num);
	logcomment(Dumper(%org_input_param));
	my %input_param = %org_input_param;
	delete($input_param{'test-description'});
	
    if ($vserver)
	{
		if($input_param{"vserver"} ne '')
		{
    		$Api_Set_Obj->set_vserver("vserver-name" => $input_param{"vserver"});
			logcomment($input_param{"vserver"});
	    	delete($input_param{"vserver"});  # Once it is set, need not be passed to Zephyr Server as seperate xml tag
		}
		else
		{
			logcomment("VSERVER NOT SPECIFIED IN THIS CASE ... \n"); # Do nothing as Vserver input is not expected here
		}
 	}
	try 
	{
      my $response = $Api_Set_Obj->$test_cmd(%input_param);   #Invoke the ZAPI for the specifying the input params
	  logcomment(" Output of the executed ZAPI .............");
	  logcomment("Execution Time : ".$response->get_execution_time());	
 	  if($vserver) { $Api_Set_Obj->clear_vserver() };

	  logcomment(Dumper($response->get_raw_output()));
   	  
	  my %exp_output_result = get_exp_output_result($zapi_name, $excel_workbook, $row_num);
		
	  if ($response->get_execution_status() ne $exp_output_result{'status'} ) ## Expected result differs from the result obtained
	  {
		  return (-1, \%org_input_param, \%exp_output_result, ($response->get_parsed_output()));
	  }
	  return (0, \%org_input_param, \%exp_output_result, ($response->get_parsed_output()));
	}
 	
	catch NACL::APISet::Exceptions::ResponseException with
    {
 	  		if($vserver) { $Api_Set_Obj->clear_vserver() };
            my $exception_object = shift;
			logcomment("Error is: " . $exception_object->text());
       	    my $response = $exception_object->get_response_object();
			my %exp_output_result = get_exp_output_result($zapi_name, $excel_workbook, $row_num);
			$exp_output_result{'Error'} =  $exception_object->text();

            if ($response->get_execution_status() ne $exp_output_result{'status'} ) ## Expeted result differs from result obtained.
            {
				return (-1, \%org_input_param, \%exp_output_result, ($response->get_raw_output()));
            }
			return (0, \%org_input_param, \%exp_output_result, ($response->get_raw_output()));
    };
}

##########################################################################################################################################
#Name		: verify_output
#Desc		: This function is called by actual thpls to execute verification commands for the corresponding zapis
#Input		: zapi_name    = name of the zapi
#			  excel_workbook     = Excel sheet to be used
#			  row_num			= Which Combination has to be executed for the particular zapi
#			  Api_Set_Obj		= Handler for NACL APISet to invoke ZAPIs
#			  Vserver			= If the corresponding ZAPI uses a vserver, takes the value 1; 0 otherwise
#			  verify_param		= Contains the query parameter which indicates for what set of values, the verification has to be done	
#Output		: return_values =  Input params used, Expected output as per excel sheet and Observed Output on ZAPI execution
#							   Also returns -1 if execution status different from expected status ; 0 otherwise.
##########################################################################################################################################
sub verify_output {

	my $self                   = shift;
	my $zapi_name              = shift;
	my $excel_workbook         = shift;
	my $row_num                = shift;
	my $Api_Set_Obj    		   = shift;
	my $vserver				   = shift;
	my %verify_param		   = @_;
     
    
	my $verify_cmd = get_outputtest_command($zapi_name, $excel_workbook, $row_num);
    $verify_cmd =~ tr/-/_/ ;

   if ($vserver)
    {
		my %input_param = get_input_params($zapi_name, $excel_workbook, $row_num);
		logcomment("vserver is " . $input_param{"vserver"});
        $Api_Set_Obj->set_vserver("vserver-name" => $input_param{"vserver"});
	    delete($input_param{"vserver"});  # Once it is set, need not be passed to Zephyr Server as seperate xml tag
    }
    
	try
    {
                my $response = $Api_Set_Obj->$verify_cmd(%verify_param);

 	  			if($vserver) { $Api_Set_Obj->clear_vserver() };
                my $actual_result = $response->get_execution_status();          #Find out if the test case result passed/failed

                my %exp_output_result = get_exp_output_result($zapi_name, $excel_workbook, $row_num);
				delete($exp_output_result{'status'});  # Do not worry about the status here.
					
				foreach my $key (keys %exp_output_result)
				{
					my @search_results = $response->search($key => quotemeta($exp_output_result{$key}));
					if ((@search_results)==0) ## The search did not return the result for given key value pair, or key is complex.
					{
						logcomment("Entering into deep search criteria...");
						my  @searchresultsagain = '';

						my @valueinarray = @{$exp_output_result{$key}};

						my $arraycount = $#valueinarray;

						logcomment($arraycount);

						logcomment(Dumper(@valueinarray));

						if ($arraycount == 0) # array contains only one element!! junk way as workaround. need to improvise
						{
							@searchresultsagain = $response->search($key => $exp_output_result{$key}->[0],
                                                                   '__deep-search-items__' => [$key] );
						}
						else
						{
						
							@searchresultsagain = $response->search($key => $exp_output_result{$key},
																   '__deep-search-items__' => [$key]);
						}
					    if((@searchresultsagain)==0)
						{
							logcomment("Search failed for key : ". $key . " and value : " . Dumper($exp_output_result{$key}));
							logcomment(Dumper($response->get_parsed_output()));
							return (-1,  $response->get_parsed_output());
						}
					}
                }
				logcomment($response->get_parsed_output());
				return (0,  $response->get_parsed_output());
     }

     catch NACL::APISet::Exceptions::ResponseException with
     {
 	  		if($vserver) { $Api_Set_Obj->clear_vserver() };
            my $exception_object = shift;
			logcomment("Error is: " . $exception_object->text());
       	    my $response = $exception_object->get_response_object();
			my %output_result = get_exp_output_result($zapi_name, $excel_workbook, $row_num);

            if ($response->get_execution_status() ne $output_result{'status'} ) ## Expeted result differs from result obtained.
            {
				return (-1,  $response->get_raw_output());
	
            }
			return (0,  $response->get_raw_output());


    };
}
                                                                      

##########################################################################################################################################
#Name		: verify_output_cli
#Desc		: This function is called by actual thpls to execute the verification command for the executed CLI command
#Input		: cli_name    = name of the cli
#			  excel_workbook     = Excel sheet to be used
#			  row_num			= Which Combination has to be executed for the particular cli
#			  Api_Set_Obj		= Handler for NACL APISet to invoke CLIs
#			  Vserver			= If the corresponding CLI uses a vserver, takes the value 1; 0 otherwise
#			  verify_param		= Contains the query parameter which indicates for what set of values, the verification has to be done	
#Output		: return_values =  "Return nothing as of now!!"
#########################################################################################################################################

sub verify_output_cli {
	my $self                   = shift;
	my $cli_name              = shift;
	my $excel_workbook         = shift;
	my $row_num                = shift;
	my $Api_Set_Obj    		   = shift;
	my $vserver				   = shift;
	my $verify_cmd   	   = shift;
     
    
	#my $verify_cmd = get_outputtest_command($cli_name, $excel_workbook, $row_num);
    $verify_cmd =~ tr/-/_/ ;
	
	if($vserver == 1) {
		$verify_cmd = "vserver_".$verify_cmd;
	}

    
	my $verify_cmd1 = "xlate_".$verify_cmd;
	try
    {

                my $response = $Api_Set_Obj->$verify_cmd("vserver" => "vs1");

                                #if($vserver) { $Api_Set_Obj->clear_vserver() };

                my $actual_result = $response->get_execution_status();          #Find out if the test case result passed/failed

				logcomment(Dumper($response->get_parsed_output()));
				logcomment($actual_result);

                my %exp_output_result = get_exp_output_result($cli_name, $excel_workbook, $row_num);
				delete($exp_output_result{'status'});  # Do not worry about the status here.

					
				foreach my $key (keys %exp_output_result)
				{
					
					my @search_results = $response->search($key => quotemeta($exp_output_result{$key}));


					if ((@search_results)==0) ## The search did not return the result for given key value pair, or key is complex.
					{

						logcomment("Entering into deep search criteria...");
						my @searchresultsagain = $response->search($key => $exp_output_result{$key},
																   '__deep-search-items__' => [$key]);

						
					    if((@searchresultsagain)==0)
						{
							logcomment("Search failed for key : ". $key . " and value : " . Dumper($exp_output_result{$key}));
							logcomment(Dumper($response->get_parsed_output()));
							return (-1,  \$response->get_parsed_output());
						}
					}
                }
				logcomment($response->get_parsed_output());
				return (0,  \$response->get_parsed_output());

     }
     catch NACL::APISet::Exceptions::ResponseException with
     {
            my $exception_object = shift;
			logcomment("Error is: " . $exception_object->text());
       	    my $response = $exception_object->get_response_object();
			my %output_result = get_exp_output_result($cli_name, $excel_workbook, $row_num);

            if ($response->get_execution_status() ne $output_result{'status'} ) ## Expeted result differs from result obtained.
            {
				return (-1,  \$response->get_parsed_output());
	
            }
			return (0,  $response->get_parsed_output());


    };
}



##########################################################################################################################################
#Name		: dumpzapilogs
#Desc		: This function is called by actual thpls dump complete log for all combinations of a given ZAPI in HTML table format
#Input		: log_file_name     = HTML file where logging has to be done
#			  numcomb		    = Total number of combinations
#			  complete_logs	    = Complete log information vector (contains input,exp_output,obs_output and other important info)
#Output		: return_values =  "Return nothing as of now!!"
#########################################################################################################################################

sub dumpzapilogs {

	my $self = shift;
	my $log_file_name = shift;
	my $numcomb = shift;
	my @complete_logs = @_;

    foreach (1..$numcomb) {

        my $tabular_format =  tablify({
        BORDER      => 1,
        DATA        => $complete_logs[$_],
        SORTBY      => 'key',
        ORDER       => 'asc'}
        );

		logcomment("file to be logged : ".$log_file_name);
        open FILE, ">>$log_file_name" or die $!;
        print FILE $tabular_format;
        close FILE;

    }

}



##########################################################################################################################################
#Name		: logValues
#Desc		: This function is called by actual thpls to collect test related information intermittently into one vector/hash
#Input		: zapi_name    = name of the zapi
#			  status       = whether the zapi test instance passed or failed
#			  summary      = at what level the test instance passed/failed and extra information related to the test
#			  input_vector = Inputs of the test instance
#			  obs_output_vector = Observed of the test instance
#			  exp_output_vector = Actual/Expected of the test instance
#			  combi_num			=  Which test instance (or the row number for the given ZAPI/CLI)
#Output		: return_values =  Returns the single vector/hash containing all the provided information in an orderly format
#########################################################################################################################################
sub logValues {
    my $self = shift;
        my $zapi_name = shift;
    my $status = shift;
    my $summary = shift;
    my $input_vector = shift;
    my $exp_output_vector = shift;
    my $obs_output_vector = shift;
    my $combi_num = shift;
        my %loginstance = '';

    $loginstance{'01_summary'} = $zapi_name." - ".$combi_num. " : ".$input_vector->{'test-description'};
        delete($input_vector->{'test-description'});
    if ($status == -1)
    {
            $loginstance{'02_result'} = 'failed : '.$summary;
    }
    else
    {
            $loginstance{'02_result'} = 'passed : '.$summary;
    }
    $loginstance{'03_input_vector'} = $input_vector;
    $loginstance{'04_observed_output'} = $obs_output_vector;
    $loginstance{'05_expected_output'} = $exp_output_vector;

        return (\%loginstance);
}
1;
