## -*- perl -*-
##
## Copyright (c) 2010 NetApp, Inc.
## All rights reserved.
##
## @summary     Basic interfaces into the ZAPI NFS hooks.
##
## @description This will help people write their own NFS tests.
##
## @author dl-nb-nfs-qa (NFS QA Bangalore)
##

package  Znfs;

use strict;
use ZapiTest;
use misc;
use NaElement;
use Tharn;

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA  = qw(Exporter);
@EXPORT  = qw(
    nfs_disable
    nfs_enable
    nfs_append_rules
    nfs_append_rules2
    nfs_exportfs_check_permission
    nfs_delete_rules
    nfs_exportfs_fence_disable
    nfs_exportfs_fence_enable
    nfs_flush_cache
    nfs_list_rules
    nfs_list_rules2
    nfs_load_exports
    nfs_modify_rule
    nfs_modify_rule2
    nfs_exportfs_storage_path
    nfs_get_supported_sec_flavors
    nfs_monitor_add
    nfs_monitor_list
    nfs_monitor_reclaim
    nfs_monitor_remove    
    nfs_monitor_remove_locks
    nfs_stats_get_client_stats
    nfs_stats_top_clients_list_iter_end
    nfs_stats_top_clients_list_iter_next
    nfs_stats_top_clients_list_iter_start
    nfs_stats_zero_stats
    nfs_status

    nfs_create_rule
    nfs_create_rule2
    nfs_create_pathname_info
    nfs_create_hostname_info
    nfs_create_sec_info
    nfs_create_owner_info
    nfs_compare_rules
    nfs_compare_rules2
    nfs_create_sec_rule
    create_nfs_rule_for_export_entry
);

# -----------------------------------------------------------------
# @function     : nfs_disable()
# @arg1         : server required.
# @return       : status from server.
# @summary      : disable nfs, if it is on.
# @status       : public
# -----------------------------------------------------------------
sub nfs_disable {
    my $server = shift;

    logcomment("nfs-disable zapi begin");
    my ($element) = new NaElement("nfs-disable");
    logcomment("nfs-disable input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-disable output:\n" . $out->sprint_pretty());
    logcomment("nfs-disable zapi end");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_enable()
# @arg1         : server required.
# @return       : status from server.
# @summary      : Enable nfs, if it is licensed.
# @status       : public
# -----------------------------------------------------------------
sub nfs_enable {
    my $server = shift;

    logcomment("nfs-enable zapi begin");
    my ($element) = new NaElement("nfs-enable");
    logcomment("nfs-enable input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-enable output:\n" . $out->sprint_pretty());
    logcomment("nfs-enable zapi end");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_append_rules()
# @arg1         : server required.
# @arg2         : persistent required if set append to the contents
#                 of /etc/exports else append to the hash tables.
# @arg3         : verbose required whether to be chatty or not
# @arg4         : rawrules required the array of rules to append.
#                 these rules are created using nfs_create_rule.
# @return       : status from server.
# @summary      : append the set of rawrules to the exports.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_append_rules {
    my ($server, $persistent, $verbose, @rawrules) = @_;

    logcomment("nfs-exportfs-append-rules zapi begin");
    my ($element) = new NaElement("nfs-exportfs-append-rules");


    if ($persistent) {
        $element->child_add(new NaElement("persistent", $persistent));
    }
    if ($verbose) {
        $element->child_add(new NaElement("verbose", $verbose));
    }
    if (@rawrules) {
        my ($rule);
        my ($rules) = new NaElement("rules");

        foreach $rule (@rawrules) {
            $rules->child_add($rule); #Created with nfs_create_rule
        }

        $element->child_add($rules);
    }

    logcomment("nfs-exportfs-append-rules input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element);
    logcomment("nfs-exportfs-append-rules output:\n" . $out->sprint_pretty());

    logcomment("nfs-exportfs-append-rules zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_append_rules2()
# @arg1         : server required.
# @arg2         : persistent required if set append to the contents
#                 of /etc/exports else append to the hash tables.
# @arg3         : verbose required whether to be chatty or not
# @arg4         : rawrules required the array of rules to append.
#                 these rules are created using nfs_create_rule.
# @return       : status from server.
# @summary      : append the set of rawrules to the exports.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_append_rules2 {
    my ($server, $persistent, $verbose, @rawrules) = @_;

    logcomment("nfs-exportfs-append-rules-2 zapi begin");
    my ($element) = new NaElement("nfs-exportfs-append-rules-2");
    if ($persistent) {
        $element->child_add(new NaElement("persistent", $persistent));
    }
    if (@rawrules) {
        my ($rule);
        my ($rules) = new NaElement("rules");

        foreach $rule (@rawrules) {
            $rules->child_add($rule); #Created with nfs_create_rule2
        }

        $element->child_add($rules);
    }
    if ($verbose) {
        $element->child_add(new NaElement("verbose", $verbose));
    }

    logcomment("nfs-exportfs-append-rules-2 input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element);
    logcomment("nfs-exportfs-append-rules-2 output:\n" . $out->sprint_pretty());

    logcomment("nfs-exportfs-append-rules-2 zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_exportfs_check_permission()
# @arg1         : server required.
# @arg2         : persistent required if set append to the contents
#                 of /etc/exports else append to the hash tables.
# @arg3         : verbose required whether to be chatty or not
# @arg4         : rawrules required the array of rules to append.
#                 these rules are created using nfs_create_rule.
# @return       : status from server.
# @summary      : append the set of rawrules to the exports.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_exportfs_check_permission {
    my $server      = shift;
    my $client_ip   = shift;
    my $path        = shift;
    my $permission  = shift;

    logcomment("nfs-exportfs-check-permission zapi begin.");
    my ($element) = new NaElement("nfs-exportfs-check-permission");
    $element->child_add(new NaElement("host", $client_ip));
    $element->child_add(new NaElement("pathname", $path));
    $element->child_add(new NaElement("permission", $permission));

    logcomment("nfs-exportfs-check-permission input:\n" . $element->sprint_pretty());
    my $out = $server->invoke_elem($element, "passed");
    logcomment("nfs-exportfs-check-permission output:\n" . $out->sprint_pretty());
    logcomment("nfs-exportfs-check-permission zapi end.");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_delete_rules()
# @arg1         : server required.
# @arg2         : persistent required if set delete the contents
#                 of /etc/exports else delete in the hash tables.
# @arg3         : allpathnames required if set delete all paths, 
#                 else delete the set in names.
# @arg4         : names required, the set of pathnames to delete
# @return       : status from server.
# @summary      : Delete the set of paths from the exports.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_delete_rules {
    my ($server, $persistent, $allpathnames, @names) = @_;

    logcomment("nfs-exportfs-delete-rules zapi begin");
    my ($element) = new NaElement("nfs-exportfs-delete-rules");

    if ($allpathnames) {
        $element->child_add(new NaElement("all-pathnames", $allpathnames));
    }

    if (@names && $names[0] ne "") {
        my ($name);
        my ($pathnames) = new NaElement("pathnames");

        foreach $name (@names) {
            my ($pathname_info) = new NaElement("pathname-info");
            $pathname_info->child_add(new NaElement("name", $name));
            $pathnames->child_add($pathname_info);
        }
        $element->child_add($pathnames);
    }

    if ($persistent) {
        $element->child_add(new NaElement("persistent", $persistent));
    }
    $element->child_add(new NaElement("verbose", "true"));

    logcomment("nfs-exportfs-delete-rules input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element);
    logcomment("nfs-exportfs-delete-rules output:\n" . $out->sprint_pretty());
    logcomment("nfs-exportfs-delete-rules zapi end");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_exportfs_fence_disable()
#  @arg1        : server required.
#  @arg2        : persistent required whether to save to
#                 /etc/exports or just online.
#  @arg3        : hosts required the set of hosts to fence.
#  @arg4        : allpathnames required whether to apply to all 
#                 paths or not.
#  @arg5        : paths required the set of paths to fence if not
#                 allpathnames.
# @return       : status from server
# @summary      : unfence the hosts from the paths.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_exportfs_fence_disable {
    my ($server, $persistent, $hosts, $allpathnames, $paths) = @_;

    logcomment("nfs-exportfs-fence-disable zapi begin");
    my ($out) = nfs_exportfs_fence_dowork($server, "nfs-exportfs-fence-disable",
            $persistent, $hosts, $allpathnames, $paths);
    logcomment("nfs-exportfs-fence-disable zapi end");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_exportfs_fence_enable()
#  @arg1        : server required.
#  @arg2        : persistent required whether to save to
#                 /etc/exports or just online.
#  @arg3        : hosts required the set of hosts to fence.
#  @arg4        : allpathnames required whether to apply to all 
#                 paths or not.
#  @arg5        : paths required the set of paths to fence if not
#                 allpathnames.
# @return       : status from server
# @summary      : fence the hosts from the paths.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_exportfs_fence_enable {
    my ($server, $persistent, $hosts, $allpathnames, $paths) = @_;

    logcomment("nfs-exportfs-fence-enable zapi begin");
    my ($o) = nfs_exportfs_fence_dowork($server, "nfs-exportfs-fence-enable",
            $persistent, $hosts, $allpathnames, $paths);
    logcomment("nfs-exportfs-fence-enable zapi end");

    return($o);
}

# -----------------------------------------------------------------
# @function     : nfs_flush_cache()
# @arg1         : server required.
# @arg2         : pathname required if set, then flush only that 
#                 entry in the cache, else flush all entries.
# @return       : return zapi output element.
# @summary      : Flush the access cache.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_flush_cache {
    my ($server, $pathname) = @_;

    logcomment("nfs-exportfs-flush-cache zapi begin");
    my ($element) = new NaElement("nfs-exportfs-flush-cache");
    if ($pathname) {
        $element->child_add(new NaElement("pathname", $pathname));
    }

    logcomment("nfs-exportfs-flush-cache input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-exportfs-flush-cache output:\n" . $out->sprint_pretty());
    logcomment("nfs-exportfs-flush-cache zapi end");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_list_rules()
# @arg1         : server required.
# @arg2         : persistent required if set list the contents of 
#                 /etc/exports else list the hash tables.
# @return       : status from server.
# @summary      : List all exports.
# @status       : public
# -----------------------------------------------------------------
sub nfs_list_rules {
    my ($server, $persistent) = @_;

    logcomment("nfs-exportfs-list-rules zapi begin");
    my ($element) = new NaElement("nfs-exportfs-list-rules");
    if ($persistent) {
        $element->child_add(new NaElement("persistent", $persistent));
    } else {
        $persistent = "";
    }

    logcomment("nfs-exportfs-list-rules input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-exportfs-list-rules output:\n" . $out->sprint_pretty());
    logcomment("nfs-exportfs-list-rules zapi end");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_list_rules2()
# @arg1         : server required.
# @arg2         : persistent required if set list the contents of 
#                 /etc/exports else list the hash tables.
# @arg3         : pathname to test with.
# @arg4         : expected result:'passed' or 'failed'
# @return       : status from server.
# @summary      : List all exports. version #2.
# @status       : public
# -----------------------------------------------------------------
sub nfs_list_rules2 {
    my ($server, $persistent, $pathname, $expected) = @_;

    logcomment("nfs-exportfs-list-rules-2 zapi begin");
    my ($element) = new NaElement("nfs-exportfs-list-rules-2");
    if ($persistent) {
        $element->child_add(new NaElement("persistent", $persistent));
    }
    if ($pathname) {
        $element->child_add(new NaElement("pathname", $pathname));
    }

    if (!$expected) { $expected = "passed"; }

    logcomment("nfs-exportfs-list-rules-2 input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, $expected);
    logcomment("nfs-exportfs-list-rules-2 output:\n" . $out->sprint_pretty());
    logcomment("nfs-exportfs-list-rules-2 zapi end");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_load_exports()
# @arg1         : server required.
# @arg2         : persistent required if set then do exportfs -r,
#                 else do exportfs -a.
# @return       : status from server.
# @summary      : Load exports. The difference between -r and -a is
#                 that -r gets rid of all online exports which are
#                 not in /etc/exports.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_load_exports {
    my ($server, $persistent) = @_;

    logcomment("nfs-exportfs-load-exports zapi begin");

    my ($element) = new NaElement("nfs-exportfs-load-exports");
    if (defined($persistent)) {
        $element->child_add(new NaElement("persistent-only", $persistent));
    }

    logcomment("nfs-exportfs-load-exports input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-exportfs-load-exports output:\n" . $out->sprint_pretty());    
    logcomment("nfs-exportfs-load-exports zapi end");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_modify_rule()
# @arg1         : server required.
# @arg2         : Persistent required. true or false
# @arg3         : rule required, created using nfs_create_rule.
# @return       : return zapi output element.
# @summary      : Modify the rule in the exports.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_modify_rule {
    my ($server, $persistent, $rule) = @_;

    logcomment("nfs-exportfs-modify-rule zapi begin");
    my ($element) = new NaElement("nfs-exportfs-modify-rule");
    $element->child_add(new NaElement("persistent", $persistent));

    my ($newrule) = new NaElement("rule");
    $newrule->child_add($rule);
    $element->child_add($newrule);

    logcomment("nfs-exportfs-modify-rule input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-exportfs-modify-rule output:\n" . $out->sprint_pretty());

    logcomment("nfs-exportfs-modify-rule zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_modify_rule2()
# @arg1         : server required.
# @arg2         : Persistent required. true or false
# @arg3         : rule required, created using nfs_create_rule2.
# @return       : return zapi output element.
# @summary      : Modify the rule in the exports.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_modify_rule2 {
    my ($server, $persistent, $rule) = @_;

    logcomment("nfs-exportfs-modify-rule-2 zapi begin");
    my ($element) = new NaElement("nfs-exportfs-modify-rule-2");
    $element->child_add(new NaElement("persistent", $persistent));

    my ($newrule) = new NaElement("rule");
    $newrule->child_add($rule);
    $element->child_add($newrule);

    logcomment("nfs-exportfs-modify-rules-2 input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-exportfs-append-rules-2 output:\n" . $out->sprint_pretty());

    logcomment("nfs-exportfs-modify-rule-2 zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_exportfs_storage_path()
# @arg1         : server required.
# @arg2         : path required. for which actual path to be 
#                 determined.
# @return       : return zapi output element.
# @summary      : Obtains the actual path for given path.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_exportfs_storage_path {
    my $server = shift;
    my $path   = shift;

    logcomment("nfs-exportfs-storage-path zapi begin.");
    my $element = new NaElement("nfs-exportfs-storage-path");
    $element->child_add(new NaElement("pathname", $path));

    logcomment("nfs-exportfs-storage-path input:\n" . $element->sprint_pretty());
    my $out = $server->invoke_elem($element, "passed");
    logcomment("nfs-exportfs-storage-path output:\n" . $out->sprint_pretty());
    logcomment("nfs-exportfs-storage-exports zapi end.");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_get_supported_sec_flavors()
# @arg1         : server required.
# @return       : status from server.
# @summary      : Get the set of supported security flavors on the
#                 filer.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_get_supported_sec_flavors {
    my ($server) = @_;

    logcomment("nfs-get-supported-sec-flavors zapi begin");
    my ($element) = new NaElement("nfs-get-supported-sec-flavors");

    logcomment("nfs-get-supported-sec-flavors input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-get-supported-sec-flavors output:\n" . $out->sprint_pretty());    
    logcomment("nfs-get-supported-sec-flavors zapi end");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_monitor_add()
# @arg1         : server required.
# @arg2         : hosts required. list of hosts.
# @return       : return zapi output element.
# @summary      : Adds nfs monitor for specified hosts.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_monitor_add {
    my $server = shift;
    my @hosts = @_;
    my $host;

    logcomment("nfs-monitor-add zapi begin");
    my ($element) = new NaElement("nfs-monitor-add");
    my $host_elem = new NaElement("hosts");

    foreach $host (@hosts) {
        $host_elem->child_add(new NaElement("hostaddr", $host));
    }
    $element->child_add($host_elem);

    logcomment("nfs-monitor-add input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-monitor-add output:\n" . $out->sprint_pretty());

    logcomment("nfs-monitor-add zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_monitor_list()
# @arg1         : server required.
# @return       : return zapi output element.
# @summary      : Removes nfs monitor for specified hosts.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_monitor_list {
    my $server = shift;

    logcomment("nfs-monitor-list zapi begin");
    my ($element) = new NaElement("nfs-monitor-list");

    logcomment("nfs-monitor-list input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-monitor-list output:\n" . $out->sprint_pretty());

    logcomment("nfs-monitor-list zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_monitor_reclaim()
# @arg1         : server required.
# @arg2         : hosts required. list of hosts.
# @return       : return zapi output element.
# @summary      : reclaims the locks for the specified hosts.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_monitor_reclaim {
    my $server = shift;
    my @hosts = @_;
    my $host;

    logcomment("nfs-monitor-reclaim zapi begin");
    my ($element) = new NaElement("nfs-monitor-reclaim");
    my $host_elem = new NaElement("hosts");

    foreach $host (@hosts) {
        $host_elem->child_add(new NaElement("hostaddr", $host));
    }
    $element->child_add($host_elem);

    logcomment("nfs-monitor-reclaim input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-monitor-reclaim output:\n" . $out->sprint_pretty());

    logcomment("nfs-monitor-reclaim zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_monitor_remove()
# @arg1         : server required.
# @arg2         : hosts required. list of hosts.
# @return       : return zapi output element.
# @summary      : Removes nfs monitor for specified hosts.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_monitor_remove {
    my $server = shift;
    my @hosts = @_;
    my $host;

    logcomment("nfs-monitor-remove zapi begin");
    my ($element) = new NaElement("nfs-monitor-remove");
    my $host_elem = new NaElement("hosts");

    foreach $host (@hosts) {
        $host_elem->child_add(new NaElement("hostaddr", $host));
    }
    $element->child_add($host_elem);

    logcomment("nfs-monitor-remove input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-monitor-remove output:\n" . $out->sprint_pretty());

    logcomment("nfs-monitor-remove zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_monitor_remove_locks()
# @arg1         : server required.
# @arg2         : owner_info required. list of owner_ifo elements
#                 created using nfs_create_owner_info.
# @return       : return zapi output element.
# @summary      : removes locks for the specified hosts and pid.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_monitor_remove_locks {
    my $server          = shift;
    my @owner_info = @_;
    my $owner;

    logcomment("nfs-monitor-remove-locks zapi begin");
    my ($element) = new NaElement("nfs-monitor-remove-locks");
    my $owner_elem = new NaElement("owners");

    foreach $owner (@owner_info) {
        $owner_elem->child_add($owner);
    }
    $element->child_add($owner_elem);

    logcomment("nfs-monitor-remove-locks input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-monitor-remove-locks output:\n" . $out->sprint_pretty());

    logcomment("nfs-monitor-remove-locks zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_stats_get_client_stats()
# @arg1         : server required.
# @arg2         : host required. Hostname or IP address.
# @return       : return zapi output element.
# @summary      : Obtains nfs stat for specified client.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_stats_get_client_stats {
    my $server = shift;
    my $host = shift;

    logcomment("nfs-stats-get-client-stats zapi begin");
    my ($element) = new NaElement("nfs-stats-get-client-stats");
    $element->child_add(new NaElement("host", $host));

    logcomment("nfs-stats-get-client-stats input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-stats-get-client-stats output:\n" . $out->sprint_pretty());

    logcomment("nfs-stats-get-client-stats zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_stats_top_clients_list_iter_end()
# @arg1         : server required.
# @arg2         : tag required. tag information from the nfs-stats
#                 -top-clients-list-iter-next output.
# @return       : return zapi output element.
# @summary      : executes nfs-stats-top-clients-list-iter-end 
#                 zapi and cleans up saved info.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_stats_top_clients_list_iter_end {
    my $server = shift;
    my $tag = shift;

    logcomment("nfs-stats-top-clients-list-iter-end zapi begin");
    my ($element) = new NaElement("nfs-stats-top-clients-list-iter-end");
    $element->child_add(new NaElement("tag", $tag));

    logcomment("nfs-stats-top-clients-list-iter-end input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-stats-top-clients-list-iter-end output:\n" . $out->sprint_pretty());

    logcomment("nfs-stats-top-clients-list-iter-end zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_stats_top_clients_list_iter_next()
# @arg1         : server required.
# @arg2         : maximum required. maximum number of records to
#                 be retrieved.
# @arg3         : tag required. tag information from the nfs-stats
#                 -top-clients-list-iter-next output.
# @return       : return zapi output element.
# @summary      : executes nfs-stats-top-clients-list-iter-next 
#                 zapi and returns records and nfs-top-info.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_stats_top_clients_list_iter_next {
    my $server = shift;
    my $maximum = shift;
    my $tag = shift;

    logcomment("nfs-stats-top-clients-list-iter-next zapi begin");
    my ($element) = new NaElement("nfs-stats-top-clients-list-iter-next");
    $element->child_add(new NaElement("maximum", $maximum));
    $element->child_add(new NaElement("tag", $tag));

    logcomment("nfs-stats-top-clients-list-iter-next input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-stats-top-clients-list-iter-next output:\n" . $out->sprint_pretty());

    logcomment("nfs-stats-top-clients-list-iter-next zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_stats_top_clients_list_iter_start()
# @arg1         : server required.
# @arg2         : maxclients required. maximum number of clients
#                 for which the stat info to be listed.
# @return       : return zapi output element.
# @summary      : executes nfs-stats-top-clients-list-iter-start 
#                 zapi and returns records and tag information.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_stats_top_clients_list_iter_start {
    my $server = shift;
    my $maxclients = shift;

    logcomment("nfs-stats-top-clients-list-iter-start zapi begin");
    my ($element) = new NaElement("nfs-stats-top-clients-list-iter-start");
    $element->child_add(new NaElement("maxclients", $maxclients));

    logcomment("nfs-stats-top-clients-list-iter-start input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-stats-top-clients-list-iter-start output:\n" . $out->sprint_pretty());

    logcomment("nfs-stats-top-clients-list-iter-start zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_stats_zero_stats()
# @arg1         : server required.
# @return       : return zapi output element.
# @summary      : zeros all the nfs statistics.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_stats_zero_stats {
    my $server = shift;

    logcomment("nfs-stats-zero-stats zapi begin");
    my ($element) = new NaElement("nfs-stats-zero-stats");

    logcomment("nfs-stats-get-client-stats input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-stats-get-client-stats output:\n" . $out->sprint_pretty());

    logcomment("nfs-stats-zero-stats zapi end");
    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_status()
# @arg1         : server required.
# @return       : status from server.
# @summary      : Determine the status of nfs.
# @status       : public
# -----------------------------------------------------------------
sub nfs_status {
    my $server = shift;

    logcomment("nfs-status zapi begin");
    my ($element) = new NaElement("nfs-status");
    logcomment("nfs-status input:\n" . $element->sprint_pretty());
    my ($out) = $server->invoke_elem($element, "passed");
    logcomment("nfs-status output:\n" . $out->sprint_pretty());
    logcomment("nfs-status zapi end");

    return($out);
}

# -----------------------------------------------------------------
# @function     : nfs_create_rule()
# @arg1         : pathname required exported pathname.
# @arg2         : ro required list of hosts allowed ro access.
#                 created with nfs_create_hostname_info.
# @arg3         : rw required list of hosts allowed rw access.
#                 created with nfs_create_hostname_info.
# @arg4         : root required list of hosts allowed root access.
#                 created with nfs_create_hostname_info.
# @arg5         : actual required actual storage path.
# @arg6         : anon required UID for anon user.
# @arg7         : sec required list of security flavors.
#                 created with nfs_create_sec_info.
# @arg8         : nosuid required is nosuid set or not?
# @return       : element created.
# @summary      : create an export rule for the input.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_create_rule {
    my ($pathname, $ro, $rw, $root, $actual, $anon, $sec, $nosuid) = @_;

    logcomment("Creating an export rule");
    my ($element) = new NaElement("exports-rule-info");
    if ($pathname ne "") {
        $element->child_add(new NaElement("pathname", $pathname));
    }
    if ($ro ne "") {
        $element->child_add($ro);    #created with nfs_create_hostname_info
    }
    if ($rw ne "") {
        $element->child_add($rw);    #created with nfs_create_hostname_info
    }
    if ($root ne "") {
        $element->child_add($root);    #created with nfs_create_hostname_info
    }
    if ($actual ne "") {
        $element->child_add(new NaElement("actual-pathname", $actual));
    }
    if ($anon ne "") {
        $element->child_add(new NaElement("anon", $anon));
    }

    # Create the deault sec if none passed in
    my ($defaultSec);

    if ($sec eq "") {
        $defaultSec = nfs_create_sec_info(("sys"));
    } else {
        $defaultSec = $sec;
    }
    $element->child_add($defaultSec);    #created with nfs_create_sec_info

    if ($nosuid ne "") {
        $element->child_add(new NaElement("nosuid", $nosuid));
    }

    return($element);
}

# -----------------------------------------------------------------
# @function     : nfs_create_rule2()
# @arg1         : pathname required exported pathname.
# @arg2         : actual required actual storage path.
# @arg3         : sec_rule required list of security flavors.
#                 created with nfs_create_sec_rule.
# @return       : element created.
# @summary      : create an export rule for the input.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_create_rule2 {
    my ($pathname, $actual, @sec_rule) = @_;

    logcomment("Creating an export rule");
    my ($element) = new NaElement("exports-rule-info-2");
    if ($actual ne "") {
        $element->child_add(new NaElement("actual-pathname", $actual));
    }
    if ($pathname ne "") {
        $element->child_add(new NaElement("pathname", $pathname));
    }
    if (@sec_rule ne "") {
        my ($rule);
        my ($rules) = new NaElement("security-rules");

        foreach $rule (@sec_rule) {
            $rules->child_add($rule); #Created with nfs_create_sec_rule
        }
        $element->child_add($rules);
    }

    return($element);
}

# -----------------------------------------------------------------
# @function     : nfs_create_pathname_info()
# @arg1         : element required label for the array
# @arg2         : paths required the list of path names
# @return       : element created.
# @summary      : Create an array of path names.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_create_pathname_info {
    my ($name, @paths) = @_;
    my ($path);

    logcomment("Building hostname for $name\n");
    my ($element) = new NaElement($name);
    foreach $path (@paths){
        my ($pi) = new NaElement("pathname-info");
        $pi->child_add(new NaElement("name", $path));
        $element->child_add($pi);
    }

    return($element);
}

# -----------------------------------------------------------------
# @function     : nfs_create_hostname_info()
# @arg1         : label for the array. Required.
# @arg2         : the list of host names. Required. For all-hosts
#                 specify empty names list.
# @return       : element created.
# @summary      : create an array of host names.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_create_hostname_info {
    my ($perm_name, @names) = @_;
    my ($name, $flag);

    logcomment("Building hostnames for $perm_name");
    my ($element) = new NaElement($perm_name);

    my $tot_clients = @names;
    if ($tot_clients == 0) {
        my ($host_info) = new NaElement("exports-hostname-info");
    }

    $flag = 0;
    foreach $name (@names){
        my ($host_info) = new NaElement("exports-hostname-info");
        if ($name =~ /^-/) {
            $host_info->child_add(new NaElement("negate", "true"));
            $host_info->child_add(new NaElement("name", substr($name, 1)));
        } else {
            $host_info->child_add(new NaElement("name", $name));
        }
        $element->child_add($host_info);
        $flag = 1;
    }

    #For specifying all hosts.
    if ($flag == 0) {
        my ($host_info) = new NaElement("exports-hostname-info");
        $host_info->child_add(new NaElement("all-hosts", "true"));
        $element->child_add($host_info);
    }

    return($element);
}

# -----------------------------------------------------------------
# @function     : nfs_create_sec_info()
# @arg1         : flavors required, the set of flavors to create.
# @return       : element created.
# @summary      : create the set of security flavors from flavors.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_create_sec_info {
    my (@flavors) = @_;
    my ($element, $flavor);

    logcomment("Creating Security Flavors info element");
    $element = new NaElement("sec-flavor");
    foreach $flavor (@flavors) {
        my ($flavour_elem) = new NaElement("sec-flavor-info");
        $flavour_elem->child_add(new NaElement("flavor", $flavor));
        $element->child_add($flavour_elem);
    }

    return($element);
}

# -----------------------------------------------------------------
# @function     : nfs_create_owner_info()
# @arg1         : client_host required. client for which the locks
#                 has to be removed.
# @arg2         : client_host_pid required. process owner on the
#                 above client whose locks has to be removed.
# @return       : element created.
# @summary      : create the set of security flavors from flavors.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_create_owner_info {
    my $client_host     = shift;
    my $client_host_pid = shift;

    logcomment("Creating owner info element");
    my $element = new NaElement("owner-info");
    $element->child_add(new NaElement("client-host", $client_host));
    $element->child_add(new NaElement("client-host-pid", $client_host_pid));

    return($element);
}

# -----------------------------------------------------------------
# @function     : nfs_compare_rules()
# @arg1         : rule1 required, created using nfs_create_rule.
# @arg2         : rule2 required, expected rule.
# @return       : return 1 on pass else 0.
# @summary      : Compare two export rules.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_compare_rules {
    my ($rule1, $rule2) = @_;
    my ($rule1_item, $rule2_item);

    logcomment("nfs_compare_rules begin.");

    #Verifying the pathname
    $rule1_item = $rule1->child_get("pathname");
    $rule2_item = $rule2->child_get("pathname");
    if ($rule1_item->content() ne $rule2_item->content()) {
        logcomment("nfs_compare_rules Error: Mismatch in pathname");
        logcomment("rule1 pathname: " .  $rule1_item->content());
        logcomment("rule2 pathname: " .  $rule2_item->content());
        return 0;
    }

    #Verifying the actual-pathname
    $rule1_item = $rule1->child_get("actual-pathname");
    $rule2_item = $rule2->child_get("actual-pathname");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("actual-pathname match failed for path " . 
                        $rule1->child_get("pathname")->content());
            return 0;
        }
    } else {
        if ($rule1_item->content() ne $rule2_item->content()) {
            logcomment("nfs_compare_rules Error: Mismatch in actual-pathname");
            logcomment("rule1 actual-pathname: " . $rule1_item->content());
            logcomment("rule2 actual-pathname: " . $rule2_item->content());
            return 0;
        }
    }

    #Verifying the anon
    $rule1_item = $rule1->child_get("anon");
    $rule2_item = $rule2->child_get("anon");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("anon match failed for path " . 
                        $rule1->child_get("pathname")->content());
            return 0;
        }
    } else {
        if ($rule1_item->content() != $rule2_item->content()) {
            logcomment("nfs_compare_rules Error: Mismatch in anon");
            logcomment("rule1 anon: " . $rule1_item->content());
            logcomment("rule2 anon: " . $rule2_item->content());
            return 0;
        }
    }

    #Verifying nosuid
    $rule1_item = $rule1->child_get("nosuid");
    $rule2_item = $rule2->child_get("nosuid");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("nosuid match failed for path " . 
                        $rule1->child_get("pathname")->content());
            return 0;
        }
    } else {
        if ($rule1_item->content() != $rule2_item->content()) {
            logcomment("nfs_compare_rules Error: Mismatch in nosuid");
            logcomment("rule1 nosuid: " . $rule1_item->content());
            logcomment("rule2 nosuid: " . $rule2_item->content());
            return 0;
        }
    }

    #Verifying ro option
    $rule1_item = $rule1->child_get("read-only");
    $rule2_item = $rule2->child_get("read-only");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("read-only match failed for path " . 
                        $rule1->child_get("pathname")->content());
            return 0;
        }
    } else {
        if ( nfs_compare_hostname_info($rule1_item, $rule2_item) != 1 )    { 
            logcomment("nfs_compare_rules Error: Mismatch in read-only " .
                       "hosts for path " . $rule1->child_get("pathname")->content());
            return 0; 
        }
    }

    #Verifying rw option
    $rule1_item = $rule1->child_get("read-write");
    $rule2_item = $rule2->child_get("read-write");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("read-write match failed for path " . 
                        $rule1->child_get("pathname"));
            return 0;
        }
    } else {
        if ( nfs_compare_hostname_info($rule1_item, $rule2_item) != 1 )    { 
            logcomment("nfs_compare_rules Error: Mismatch in read-write " .
                       "hosts for path " . $rule1->child_get("pathname")->content());
            return 0; 
        }
    }

    #Verifying root option
    $rule1_item = $rule1->child_get("root");
    $rule2_item = $rule2->child_get("root");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("root match failed for path " . 
                        $rule1->child_get("pathname"));
            return 0;
        }
    } else {
        if ( nfs_compare_hostname_info($rule1_item, $rule2_item) != 1 )    { 
            logcomment("nfs_compare_rules Error: Mismatch in root " .
                       "hosts for path " . $rule1->child_get("pathname")->content());
            return 0; 
        }
    }

    #Verifying sec option
    $rule1_item = $rule1->child_get("sec-flavor");
    $rule2_item = $rule2->child_get("sec-flavor");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("sec-flavor match failed for path " . 
                        $rule1->child_get("pathname"));
            return 0;
        }
    } else {
        if ( nfs_compare_sec_info($rule1_item, $rule2_item) != 1 )    { 
            logcomment("nfs_compare_rules Error: Mismatch in sec-flavor for path " 
                        . $rule1->child_get("pathname")->content());
            return 0; 
        }
    }

    logcomment("Rules check successful.");
    logcomment("nfs_compare_rules end.");

    return 1;
}

# -----------------------------------------------------------------
# @function     : nfs_compare_rules2()
# @arg1         : rule1 required, created using nfs_create_rule2.
# @arg2         : rule2 required, expected rule.
# @return       : returns 1 on pass else 0.
# @summary      : Compare two export rules.
# @status       : public.
# -----------------------------------------------------------------
sub nfs_compare_rules2 {
    my ($rule1, $rule2) = @_;
    my ($rule1_item, $rule2_item);

    logcomment("nfs_compare_rules2 begin.");

    #Verifying the pathname
    $rule1_item = $rule1->child_get("pathname");
    $rule2_item = $rule2->child_get("pathname");
    if ($rule1_item->content() ne $rule2_item->content()) {
        logcomment("nfs_compare_rules2 Error: Mismatch in pathname");
        logcomment("rule1 pathname: " .  $rule1_item->content());
        logcomment("rule2 pathname: " .  $rule2_item->content());
        return 0;
    }

    #Verifying the actual-pathname
    $rule1_item = $rule1->child_get("actual-pathname");
    $rule2_item = $rule2->child_get("actual-pathname");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("actual-pathname match failed for path" . 
                        $rule1->child_get("pathname")->content());
            return 0;
        }
    } else {
        if ($rule1_item->content() ne $rule2_item->content()) {
            logcomment("nfs_compare_rules2 Error: Mismatch in actual-pathname");
            logcomment("rule1 actual-pathname: " . $rule1_item->content());
            logcomment("rule2 actual-pathname: " . $rule2_item->content());
            return 0;
        }
    }

    #Verifying the security-rules
    $rule1_item = $rule1->child_get("security-rules");
    $rule2_item = $rule2->child_get("security-rules");
    my $pathname = $rule1->child_get("pathname")->content();
    if (nfs_compare_security_rule($rule1_item, $rule2_item, $pathname) != 1) {
        logcomment("nfs_compare_rules2 Error: Mismatch in security-rules");
        return 0;
    }

    logcomment("Rules check successful.");
    logcomment("nfs_compare_rules2 end.");
    return 1;
}

# -----------------------------------------------------------------
# @function     : create_nfs_rule_for_export_entry()
# @arg1         : export entry.
# @arg2         : if 1 creates nfs_create_rule element else
#                 creates nfs_create_rule2 element.
# @return       : return the nfs_create_rule element or
#                  nfs_create_rule2 element.
# @summary      : Creates the nfs export element for the given 
#                 export entry.
# @status       : public.
# -----------------------------------------------------------------
sub create_nfs_rule_for_export_entry($$) {
    my $path_access_list = shift;
    my $type_flag = shift;
    my ($rule, $sec_rule);
    my ($element, $temp);
    my ($anon, $nosuid, $rw, $ro, $root, $actual, $sec) = "";

    #Extract path
    my($path, $access) = split(" -", $path_access_list);
    my @access_list = split(",", $access);

    foreach $element (@access_list) {

        if ($element =~ /^anon/) {
            ($temp, $anon) = split('=', $element);
        }
        if ($element =~ /^nosuid/) {
            $nosuid = "true";
        }
        if ($element =~ /^actual/) {
            ($temp, $actual) = split("=", $element);
        }
        if ($element =~ /^ro$|^ro=/) {
            my $cli_list;
            ($temp, $cli_list) = split("=", $element);
            my @clnts = get_clients($cli_list);
            $ro = nfs_create_hostname_info('read-only', @clnts);
        }
        if ($element =~ /^rw/) {
            my $cli_list;
            ($temp, $cli_list) = split("=", $element);
            my @clnts = get_clients($cli_list);
            $rw = nfs_create_hostname_info('read-write', @clnts);
        }
        if ($element =~ /^root/) {
            my $cli_list;
            ($temp, $cli_list) = split("=", $element);
            my @clnts = get_clients($cli_list);
            $root = nfs_create_hostname_info('root', @clnts);
        }
        if ($element =~ /^sec=/) {
            ($temp, $sec) = split("=", $element);
            my @sec = split(":", $sec);

            $sec = nfs_create_sec_info(@sec);
        }
    }

    if ($type_flag eq 1) {
        $rule = nfs_create_rule($path, $ro, $rw, $root, $actual, $anon, $sec, $nosuid);
    } else {
        $sec_rule = nfs_create_sec_rule($ro, $rw, $root, $anon, $sec, $nosuid);
        $rule = nfs_create_rule2($path, $actual, $sec_rule);
    }
    return $rule;
}

# -----------------------------------------------------------------
# @function     : get_clients()
# @arg1         : client list separated by :.
# @return       : return the list of clients.
# @summary      : Return list of clients.
# @status       : private.
# -----------------------------------------------------------------
sub get_clients($) {
    my $cli_list = shift;
    my (@temp_clnts, @clnts);

    @temp_clnts = split(":", $cli_list);
    my $length = @temp_clnts;

    my $j = 0;
    for (my $i = 0; $i < $length; $i++) {
        #For capturing ipv6 address.
        if (@temp_clnts[$i] =~ /\[/) {
            while ( (@temp_clnts[$i] !~ /\]/) && ($i < $length) ) {
                if(@clnts[$j] eq "") {
                    @clnts[$j] = @temp_clnts[$i++];
                } else {
                    @clnts[$j] = @clnts[$j] . ":" . @temp_clnts[$i++];
                }
            }
            @clnts[$j] = @clnts[$j] . ":" . @temp_clnts[$i++];
            $j++;
        } else {
            @clnts[$j++] = @temp_clnts[$i];
        }
    }

    return(@clnts);
}

# -----------------------------------------------------------------
# @function     : nfs_compare_hostname_info()
# @arg1         : rule1 required, created using 
#                 nfs_create_hostname_info.
# @arg2         : rule2 required, expected rule.
# @return       : return 1 on pass else 0.
# @summary      : Compare two hostname info.
# @status       : private.
# -----------------------------------------------------------------
sub nfs_compare_hostname_info {
    my ($rule1, $rule2) = @_;
    my ($rule1_item, $rule2_item);
    my ($max_child1, $max_child2);
    my $childrens;

    logcomment("nfs_compare_hostname_info begin for " . 
               $rule1->name());

    #Verifying for total number of elements to be same
    $max_child1 = $rule1->child_max_number();
    $max_child2 = $rule2->child_max_number();
    if ($max_child1 != $max_child2) {
        logcomment("nfs_compare_hostname_info Error: Mismatch in number of clients");
        logcomment("rule1 max_child: $max_child1");
        logcomment("rule2 max_child: $max_child2");
        return 0;
    }

    my $tmp_rule1 = $rule1->child_get("exports-hostname-info");
    my $tmp_rule2 = $rule2->child_get("exports-hostname-info");

    #Verifying the all-hosts option
    if ( defined($tmp_rule1->child_get("all-hosts")) &&
         defined($tmp_rule2->child_get("all-hosts")) ) {
        my $val1 = $tmp_rule1->child_get("all-hosts");
        my $val2 = $tmp_rule2->child_get("all-hosts");

        if ($val1->content() != $val2->content()) {
            logcomment("nfs_compare_hostname_info Error: Mismatch in all-hosts");
            logcomment("rule1 all-hosts: " . $val1->content());
            logcomment("rule2 all-hosts: " . $val2->content());
            return 0;
        }
        return 1;
    }

    #Verifying the clients
    for(my $i = 0; $i < $max_child1; $i++) {
        $childrens = $rule1->{children};
        $rule1_item = $childrens->[$i];
        my $leaf1_name = $rule1_item->child_get("name");
        my $leaf1_neg  = $rule1_item->child_get("negate");

        # Checking for any match, since the hostnames are not in any particulat order
        my $match_flag = 0;
        for(my $j = 0; $j < $max_child2; $j++) {
            my $childrens = $rule2->{children};
            $rule2_item = $childrens->[$j];
            my $leaf2_name = $rule2_item->child_get("name");
            my $leaf2_neg = $rule2_item->child_get("negate");

            if ( defined($leaf1_name) && defined($leaf2_name) ) {
                if ($leaf1_name->content() eq $leaf2_name->content()) {
                    if ( defined($leaf1_neg) && defined($leaf2_neg) ) {
                        if ($leaf1_neg->content() eq $leaf2_neg->content()) {
                            $match_flag = 1;
                        }
                    } else {
                        $match_flag = 1;
                    }
                }
            }
        }
        if ($match_flag != 1) {
            logcomment("nfs_compare_hostname_info Error: Mismatch in hostnames");
            logcomment("Rule1 hostname: " . $leaf1_name->content());
            return 0
        }
    }

    logcomment("nfs_compare_hostname_info end for " . 
               $rule1->name());
    return 1;
}

# -----------------------------------------------------------------
# @function     : nfs_compare_sec_info()
# @arg1         : rule1 required, created using 
#                 nfs_create_sec_info.
# @arg2         : rule2 required, expected rule.
# @return       : return 1 on pass else 0.
# @summary      : Compare two sec info.
# @status       : private.
# -----------------------------------------------------------------
sub nfs_compare_sec_info {
    my ($rule1, $rule2) = @_;
    my ($rule1_item, $rule2_item);
    my ($max_child1, $max_child2);
    my $childrens;

    logcomment("nfs_compare_sec_info begin");

    #Verifying for total number of elements to be same
    $max_child1 = $rule1->child_max_number();
    $max_child2 = $rule2->child_max_number();
    if ($max_child1 != $max_child2) {
        logcomment("nfs_compare_sec_info Error: Mismatch in number of sec-flavor");
        logcomment("rule1 max_child: $max_child1");
        logcomment("rule2 max_child: $max_child2");
        return 0;
    }

    for (my $i = 0; $i < $max_child1; $i++) {
        $childrens = $rule1->{children};
        $rule1_item = $childrens->[$i];
        my $leaf1_sec = $rule1_item->child_get("flavor");

        # Checking for any match, since the hostnames are not in any particulat order
        my $match_flag = 0;
        for (my $j = 0; $j < $max_child2; $j++) {
            $childrens = $rule2->{children};
            $rule2_item = $childrens->[$j];
            my $leaf2_sec = $rule2_item->child_get("flavor");

            if ( defined($leaf1_sec) && defined($leaf2_sec) ) {
                if ($leaf1_sec->content() eq $leaf2_sec->content()) {
                    $match_flag = 1;
                }
            }        
        }
        if ($match_flag != 1) {
            logcomment("nfs_compare_sec_info Error: Mismatch in sec-flavor");
            return 0
        }
    }

    logcomment("nfs_compare_sec_info end");
    return 1;
}

# -----------------------------------------------------------------
# @function     : nfs_compare_security_rule()
# @arg1         : rule1 required, created using nfs_create_sec_rule.
# @arg2         : rule2 required, expected rule.
# @arg3         : pathname required, usen when comparision fails.
# @return       : returns 1 on pass else 0.
# @summary      : Compare two export rules.
# @status       : private.
# -----------------------------------------------------------------
sub nfs_compare_security_rule {
    my ($rule1, $rule2, $pathname) = @_;
    my ($rule1_item, $rule2_item);

    #Verifying the anon
    $rule1_item = $rule1->child_get("anon");
    $rule2_item = $rule2->child_get("anon");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("anon match failed for path $pathname.");
            return 0;
        }
    } else {
        if ($rule1_item->content() != $rule2_item->content()) {
            logcomment("nfs_compare_rules Error: Mismatch in anon");
            logcomment("rule1 anon: " . $rule1_item->content());
            logcomment("rule2 anon: " . $rule2_item->content());
            return 0;
        }
    }

    #Verifying nosuid
    $rule1_item = $rule1->child_get("nosuid");
    $rule2_item = $rule2->child_get("nosuid");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("nosuid match failed for path $pathname.");
            return 0;
        }
    } else {
        if ($rule1_item->content() != $rule2_item->content()) {
            logcomment("nfs_compare_rules Error: Mismatch in nosuid");
            logcomment("rule1 nosuid: " . $rule1_item->content());
            logcomment("rule2 nosuid: " . $rule2_item->content());
            return 0;
        }
    }

    #Verifying ro option
    $rule1_item = $rule1->child_get("read-only");
    $rule2_item = $rule2->child_get("read-only");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("read-only match failed for path $pathname.");
            return 0;
        }
    } else {
        if ( nfs_compare_hostname_info($rule1_item, $rule2_item) != 1 )    { 
            logcomment("nfs_compare_rules Error: Mismatch in read-only $pathname.");
            return 0; 
        }
    }

    #Verifying rw option
    $rule1_item = $rule1->child_get("read-write");
    $rule2_item = $rule2->child_get("read-write");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("read-write match failed for path $pathname.");
            return 0;
        }
    } else {
        if ( nfs_compare_hostname_info($rule1_item, $rule2_item) != 1 )    { 
            logcomment("nfs_compare_rules Error: Mismatch in read-write $pathname.");
            return 0; 
        }
    }

    #Verifying root option
    $rule1_item = $rule1->child_get("root");
    $rule2_item = $rule2->child_get("root");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("root match failed for path $pathname.");
            return 0;
        }
    } else {
        if ( nfs_compare_hostname_info($rule1_item, $rule2_item) != 1 )    { 
            logcomment("nfs_compare_rules Error: Mismatch in root $pathname.");
            return 0; 
        }
    }

    #Verifying sec option
    $rule1_item = $rule1->child_get("sec-flavor");
    $rule2_item = $rule2->child_get("sec-flavor");
    if ($rule1_item == 'undef' || $rule2_item == 'undef') {
        if ($rule1_item != $rule2_item) {
            logcomment("sec-flavor match failed for path $pathname.");
            return 0;
        }
    } else {
        if ( nfs_compare_sec_info($rule1_item, $rule2_item) != 1 )    { 
            logcomment("nfs_compare_rules Error: Mismatch in sec-flavor for path $pathname.");
            return 0; 
        }
    }

    return 1;
}

# -----------------------------------------------------------------
# @function     : nfs_create_sec_rule()
# @arg1         : ro required list of hosts allowed ro access.
#                 created with nfs_create_hostname_info.
# @arg2         : rw required list of hosts allowed rw access.
#                 created with nfs_create_hostname_info.
# @arg3         : root required list of hosts allowed root access.
#                 created with nfs_create_hostname_info.
# @arg4         : anon required UID for anon user.
# @arg5         : sec required list of security flavors.
#                 created with nfs_create_sec_info.
# @arg6         : nosuid required is nosuid set or not?
# @return       : element created.
# @summary      : create an export rule for the input.
# @status       : private.
# -----------------------------------------------------------------
sub nfs_create_sec_rule {
    my ($ro, $rw, $root, $anon, $sec, $nosuid) = @_;

    logcomment("Creating an security rule");
    my ($element) = new NaElement("security-rule-info");
    if ($ro ne "") {
        $element->child_add($ro);    #created with nfs_create_hostname_info
    }
    if ($rw ne "") {
        $element->child_add($rw);    #created with nfs_create_hostname_info
    }
    if ($root ne "") {
        $element->child_add($root);    #created with nfs_create_hostname_info
    }
    if ($anon ne "") {
        $element->child_add(new NaElement("anon", $anon));
    }

    # Create the deault sec if none passed in
    my ($defaultSec);

    if ($sec eq "") {
        $defaultSec = nfs_create_sec_info(("sys"));
    } else {
        $defaultSec = $sec;
    }
    $element->child_add($defaultSec);    #created with nfs_create_sec_info

    if ($nosuid ne "") {
        $element->child_add(new NaElement("nosuid", $nosuid));
    }

    return($element);
}

# -----------------------------------------------------------------
# @function     : nfs_exportfs_fence_dowork()
#  @arg1        : server required.
## @arg2        : element required whether fencing or unfencing
#  @arg3        : persistent required whether to save to
#                 /etc/exports or just online.
#  @arg4        : hosts required the set of hosts to fence.
#  @arg5        : allpathnames required whether to apply to all 
#                 paths or not.
#  @arg6        : paths required the set of paths to fence if not
#                 allpathnames.
# @return       : status from server
# @summary      : Main body of work for fencing operations.
# @status       : private.
# -----------------------------------------------------------------
sub nfs_exportfs_fence_dowork {
    my ($server, $element, $persistent, $hosts, $allpathnames, $paths) = @_;

    my ($e) = new NaElement($element);
    if ($persistent ne "") {
        $e->child_add(new NaElement("persistent", $persistent));
    }

    my ($fh) = new NaElement("fenced-hosts");    
    if ($hosts eq "") {
        Tharn::tstlog("-fail", "No hosts defined for $element\n");
    } elsif ($hosts eq "all-hosts") {
        my ($ah) = new NaElement("exports-hostname-info");
        $ah->child_add(new NaElement("all-hosts", "true"));
        $fh->child_add($ah);
    } else {
        $e->child_add($hosts);    #created with nfs_create_hostname_info
    }

    if ($allpathnames) {
        $e->child_add(new NaElement("all-pathnames", $allpathnames));
    } elsif ($paths eq "") {
        Tharn::tstlog("-fail", "No paths defined for $element\n");
    } else {
        $e->child_add($paths);    #created with nfs_create_pathname_info
    }

    logcomment("$element input:\n" . $e->sprint_pretty());
    my ($out) = $server->invoke_elem($e, "passed");
    logcomment("$element output:\n" . $out->sprint_pretty());    

    return($out);
}


sub _gx_vserver_check($) {
    my ($elem) = shift;
    if ($main::VSERVER) {
        $elem->child_add(new NaElement("vserver", $main::VSERVER));
    }
}
1;
