# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/apcon.pm#1 $
#
## @summary apcon library functions 
##
## @description
##   These test functions are needed for apcon switch with blades
## @usage
##   use apcon;
##   use apcon qw(apcon_connectports apcon_disconnectport);
##
## @keywords  apcon test library
##
## @dependencies ApconCmdX binary, and this binary runs only on Solaris 9,
## so the script invoking these funtions should be executed on solaris 
## otherwise the command will fail.
## 
##
## @author dl-sar-qa@netapp.com, dl-step@netapp.com
## @status testme
## @change 2004-12-17 sajan: created
##

package apcon;

use strict;
use IO::File;
use Fcntl qw(O_EXCL O_CREAT O_RDWR);
use POSIX qw(tmpnam);
use vars ('@ISA','@EXPORT');

require Exporter;
@ISA = ('Exporter');
@EXPORT = qw(
             apcon_create_tempfile
             apcon_portsettings_find
             apcon_connectports
             apcon_disconnectport
             apcon_port_otherend_find
             );

use config_base;
use Tharn;

## @name ::apcon_create_tempfile
## @description Returns the file having apcon commands based on the 
## parameters passed to it. 
##
## @synopsis
## apcon_create_tempfile{$apcon_switch, $apcon_switch_number,
##                       $port_src, $port_dst, $conn_mode, 
##                       $USERNAME, $PASSWORD};
##
## @arg All are needed, the possible values are:
## $apcon_switch = <apcon_switch-ip-address> | <apcon_intellazone-ip-address>
## $apcon_switch_number = <the switch_number> | undef (will assume 1)
## $port_src = <apcon_port> | undef
## $port_dst = <apcon_port> | undef
## $conn_mode = advanced | undef (undef meanns port to port)
## $USERNAME = <user_name> | undef (needed only for intellazone server)
## $PASSWORD = <password> | undef (needed only for intellazone server)
##
sub apcon_create_tempfile($$$$$$$) {
    my ($apcon_switch, $apcon_switch_number, $port_src, $port_dst, $conn_mode,
        $USERNAME, $PASSWORD) = @_;

    if (!defined($apcon_switch_number)) {
         if (($apcon_switch_number == undef)) {
            $apcon_switch_number = 1; 
         }
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Create a temporary file (usually in dir /var/tmp) to keep
    # apcon connection info. This file is then used to execute the
    # apcon command "ApconCmdX -r <filename>". Using this method
    # makes sure that a uniq file name is created and then this
    # file gets deleted once the progrma exits..
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    my ($name) = undef;
    my ($tmpnam) = undef;
    my $fh;

    do { $name = tmpnam() }
        until $fh = IO::File->new($name, O_RDWR|O_CREAT|O_EXCL);
    END { unlink($name) }

    #open(TMP_FH, "> $name") or die "Can't open file $name error: $!";
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Enable autoflush, so that the line can be written (flush out)
    # to the file immediately
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    my $old_fh = select(TMP_FH); # get the old file handler
    $| = 1;
    select($old_fh); # set the old file handler as STDOUT

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Set the string for apcon command "open"
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    my $open_cmd_string = undef;
    if (defined($USERNAME) && defined($PASSWORD)) {
        if (($USERNAME ne undef) && ($PASSWORD ne undef)) {
            $open_cmd_string = "$open_cmd_string" . "-z $USERNAME|$PASSWORD";
        }
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Set the string for apcon command "set port settings <string>"
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    my $set_cmd_string = undef;
    if (($port_dst ne undef) && ($port_src ne undef)) {
        if (defined($conn_mode)) {
            if (($conn_mode ne undef)) {
                $set_cmd_string = "$set_cmd_string" . "$conn_mode" .  " ";
            }
        }
        $set_cmd_string = "$set_cmd_string" . "$port_src" .  "$port_dst";
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Start writing to the temp file
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    print $fh 
            "open $apcon_switch -n $apcon_switch_number $open_cmd_string\n";
    print $fh "show port settings\n";
    if (($port_dst ne undef) && ($port_src ne undef)) {
        print $fh "set port settings $set_cmd_string\n";
        print $fh "show port settings\n";
    }
    close($fh) or die "Couldn't close Temp file, error : $!";

    return $name;
}

## @name ::apcon_portsettings_find
## @description Subroutine to find the current port settings. If the command 
## is successful then it returns an array holding all the output lines of 
## command "show port settings", else undef for failure.
##
## @synopsis
## apcon_portsettings_find{$apcon_switch, $apcon_switch_number, 
##                       $USERNAME, $PASSWORD};
##
## @arg All are needed, the possible values are:
## $apcon_switch = <apcon_switch-ip-address> | <apcon_intellazone-ip-address>
## $apcon_switch_number = <the switch_number> | undef (will assume 1)
## $USERNAME = <user_name> | undef (needed only for intellazone server)
## $PASSWORD = <password> | undef (needed only for intellazone server)
##
sub apcon_portsettings_find ($$$$) {
    my ($apcon_switch, $apcon_switch_number, $USERNAME, $PASSWORD) = @_;

    my ($name) = undef;
    $name = apcon_create_tempfile($apcon_switch, $apcon_switch_number, undef, 
                                  undef, undef, $USERNAME, $PASSWORD);
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Issue the commands to apcon switch
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    my $status = system("cat $name");
    my @current_port_settings = `/u/sajan/apcon/ApconCmdX -r $name`;
    if ($?) {
        return undef;
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Print and parse the output of apcon commands
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    foreach my $line (@current_port_settings) {
        chomp($line);
        print "$line\n";
        if ($line =~ /ERROR/i) {
            return undef;
        }
    }
    return (@current_port_settings);
}   

## @name ::apcon_connectports
## @description Subroutine to connect two apcon ports. It returns 1 if 
## the command is completed successfully, otherwise undef. 
##
## @synopsis
## apcon_connectports{$apcon_switch, $apcon_switch_number,
##                    $port_src, $port_dst, $conn_mode, 
##                    $USERNAME, $PASSWORD};
##
## @arg All are needed, the possible values are:
## $apcon_switch = <apcon_switch-ip-address> | <apcon_intellazone-ip-address>
## $apcon_switch_number = <the switch_number> | undef (will assume 1)
## $port_src = <apcon_port> 
## $port_dst = <apcon_port> 
## $conn_mode = advanced | undef (undef meanns port to port)
## $USERNAME = <user_name> | undef (needed only for intellazone server)
## $PASSWORD = <password> | undef (needed only for intellazone server)
##
sub apcon_connectports ($$$$$$$) {
    my ($apcon_switch, $apcon_switch_number, $port_src, $port_dst, $conn_mode,
        $USERNAME, $PASSWORD) = @_;

    my ($name) = undef;

    if (defined($conn_mode)) {
         if (!($conn_mode =~ /advanced/i) && !($conn_mode =~ /full/i) &&
            (!($conn_mode eq undef))) {
            return undef;
         }
    }
    $name = apcon_create_tempfile($apcon_switch, $apcon_switch_number, 
                                  $port_src, $port_dst, $conn_mode, 
                                  $USERNAME, $PASSWORD);
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Issue the commands to apcon switch
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    my $status = system("cat $name");
    my @current_port_settings = `/u/sajan/apcon/ApconCmdX -r $name`;
    if ($?) {
        return undef;
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Print and parse the output of apcon commands
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    foreach my $line (@current_port_settings) {
        chomp($line);
        print "$line\n";
        if ($line =~ /ERROR/i) {
            return undef;
        }
    }

    return (1);
}   

## @name ::apcon_disconnectport
## @description Subroutine to disconnect an apcon port. It returns 1 if the
## command is completed successfully, otherwise undef.
##
## @synopsis
## apcon_disconnectport{$apcon_switch, $apcon_switch_number,
##                      $port, $conn_mode, $USERNAME, $PASSWORD};
##
## @arg All are needed, the possible values are:
## $apcon_switch = <apcon_switch-ip-address> | <apcon_intellazone-ip-address>
## $apcon_switch_number = <the switch_number> | undef (will assume 1)
## $port = <apcon_port> 
## $conn_mode = advanced | undef (undef meanns port to port)
## $USERNAME = <user_name> | undef (needed only for intellazone server)
## $PASSWORD = <password> | undef (needed only for intellazone server)
##
sub apcon_disconnectport ($$$$$$) {
    my ($apcon_switch, $apcon_switch_number, $port, $conn_mode,
        $USERNAME, $PASSWORD) = @_;

    my ($name) = undef;

    if (defined($conn_mode)) {
         if (!($conn_mode =~ /advanced/i) && !($conn_mode =~ /full/i) &&
            (!($conn_mode eq undef))) {
            return undef;
         }
    }

    $name = apcon_create_tempfile($apcon_switch, $apcon_switch_number, $port, 
                            "A00", $conn_mode, $USERNAME, $PASSWORD);
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Issue the commands to apcon switch
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    my $status = system("cat $name");
    my @current_port_settings = `/u/sajan/apcon/ApconCmdX -r $name`;
    if ($?) {
        return undef;
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Print and parse the output of apcon commands
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    foreach my $line (@current_port_settings) {
        chomp($line);
        print "$line\n";
        if ($line =~ /ERROR/i) {
            return undef;
        }
    }
    return 1;
}   

## @name ::apcon_port_otherend_find
## @description Subroutine to find the other end of an apcon port. It returns 
## an hash which has following three fields:
## "port" =>  <port-num> | undef
## "mode" =>  advanced | p2p
## "transmit_port" =>  <port-num>   #needed only in advanced mode
## If the port is not connected to anyone then all the keys would have
## undefined value. If the execution of command fails then it would 
## return undef.  
##
## @synopsis
## apcon_port_otherend_find{$apcon_switch, $apcon_switch_number,
##                      $port, $USERNAME, $PASSWORD};
##
## @arg All are needed, the possible values are:
## $apcon_switch = <apcon_switch-ip-address> | <apcon_intellazone-ip-address>
## $apcon_switch_number = <the switch_number> | undef (will assume 1)
## $port = <apcon_port> 
## $USERNAME = <user_name> | undef (needed only for intellazone server)
## $PASSWORD = <password> | undef (needed only for intellazone server)
##
sub apcon_port_otherend_find ($$$$$) {
    my ($apcon_switch, $apcon_switch_number, $port, $USERNAME, $PASSWORD) = @_;

    my (%partner_info) = ();
    $partner_info{"port"} = undef;
    $partner_info{"mode"} = undef;
    $partner_info{"transmit_port"} = undef;
    my ($name) = undef;


    $name = apcon_create_tempfile($apcon_switch, $apcon_switch_number, $port, 
                            undef, undef, $USERNAME, $PASSWORD);

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Issue the commands to apcon switch
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    my $status = system("cat $name");
    my @current_port_settings = `/u/sajan/apcon/ApconCmdX -r $name`;
    if ($?) {
        return undef;
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - -
    # Print and parse the output of apcon commands
    # - - - - - - - - - - - - - - - - - - - - - - - - -
    foreach my $line (@current_port_settings) {
        chomp($line);
        print "$line\n";
        if ($line =~ /ERROR/i) {
            return undef;
        }
        if ($line =~ /$port/i) { 
            if ($line =~ /<</) {
                my ($enda, $endb) = split(/<</, $line);
                if ($enda =~ $port) {
                    $partner_info{"port"} = $endb;
                } else {
                    $partner_info{"port"} = $enda;
                }
                $partner_info{"mode"} = "advanced";
                $partner_info{"transmit_port"} = $endb;
            } else {
                my ($enda, $endb) = split(/<>/, $line);
                if ($enda =~ $port) {
                    $partner_info{"port"} = $endb;
                } else {
                    $partner_info{"port"} = $enda;
                }
                $partner_info{"mode"} = "p2p";
                $partner_info{"transmit_port"} = undef;
            }
            last;   # terminate the for loop because we found the port
        }
    }
    return (%partner_info);
}   

return 1;
