#Hey emacs, this is basically -*- perl -*-
#
#Copyright (c) 2002 Network Appliance, Inc.
#All rights reserved.
#
## @summary Misc error insertion library functions for Sledgehammer.
##
## @description
##
## $Id: //depot/prod/test/Rsierranevada/lib/blocks_misc.pm#1 $<br>
##
##   These test functions implement features to support various types 
##   of error insertion in a Fibrechannel and iSCSI environment.
##
## @usage
##   use blocks_misc ;
##
## @keywords SAN FCP iSCSI Blocks
## @dependencies
##
## @author pressley@netapp.com (Blocks QA)
## @status public
## @change 2002-03-13 johnm: created
## @change 2002-03-27 johnm: modified blocks_misc_pwrcmd to use $host->{powercmd};
## @change 2002-03-27 johnm: add $number parameter to blocks_misc_pwrcmd
## @change 2002-04-10 rfair: add blocks_misc_truncate
## @change 2002-05-14 johnm: modified blocks_misc_pwrcmd so that full
##                           path specification can be used from host record.
## @change 2002-06-15 sli: modified blocks_misc_pwrcmd to handle 
##     1) the provided power_cmd in tharnhost file may have post space or not.
##          like power_cmd="power_cycle " or "power_cycle path" or 
##          "rpb" or "rpb path"
##     2) the power_cmd provided maybe as full path, or just rpb
##     3) the $host->{id}1 should as {id} 1, there is a space between, 
##         same as {id}2 and {id}both 
##     4) use rtp's power_cycle for power 1 or power 2, it will not return text,##         CA's power_cycle will return text
## @change 2002-07-1 sli: modified blocks_misc_pwrcmd, because RTP's power_cycle
##      use $host->{id}$n but CA use $host->{id} $n 
## @change 2002-08-02 johnm: added new subroutines:
##                    wwn_clear
##                    string_search
##                    blocks_config_file
##
## @change 2002-08-15 johnm: added new subroutines:
##                    wwn_set
## @change 2002-09-25 johnm: modified parse_lun_list.
## @change 2004-06-24 olinger: Removed "device" from error message text matched
##              against the output of "power_cycle".
## @change 2005-01-24 hedstrom: use Burt.pm to support error classification and die standard on fatal errors
## @change 2005-06-01 archana: Updated author tag

package blocks_misc;

use strict;
use vars ('@ISA','@EXPORT');

require Exporter;
@ISA = ('Exporter');
@EXPORT = qw(
         wwn_set
         wwn_clear
         buff_to_array
         check_param
         blocks_misc_pwrcmd
         blocks_misc_truncate
         blocks_config_file
         string_search
         parse_lun_list
         );

use config_base;
use Tharn;
use Burt;

## @name ::blocks_misc_pwrcmd
## @description 
## Issues a power command to turn on or off equipment.  Note that
## the unix power command used is specified in the host record "powercmd".
## E.g. The `power_cycle' utility (an expect script) can be used.  What ever
## utilitiy is specified in the host "powercmd" record, it must reside in the
## search path of the host machine running this perl sub-routine, and it must
## conform to the power_cycle API. 
## when running machine's 'power_cycle' is different from the host's 'powercmd',## caller need to provide this parameter.
##
## Returns the output from the system command on error, else null for
## success.  More at: power_cycle help
##
## @synopsis
## blocks_misc_pwrcmd {$host, $cmd, $number}
##
## @arg $host Required: The host record of the equipment to power on or off
## @arg $cmd Required: "on" or "off" or "reboot"<br>
## <b> Note: </b> when your powercmd is `rpb', only the reboot command can be passed.
## @arg $number optional: "1", "2", "both", or `undef' - the power supply number.<br>
## Undef implies that only one power supply is present (i.e. `$host->{id}` is passed to powercmd). When 1 is passed $host->{id} 1 is used. When 2 is passed $host->{id} 2 is used. When "both" is passed both $host->{id} 1 and $host->{id} 2 are passed.

## @example
## my $fa = host($FILER);
## my $error = blocks_misc_pwrcmd($fa, "off"); #power off filer w/one supply
## if ($error) {
##   print "$error"; exit 1
## } 
##
## $error = blocks_misc_pwrcmd($fa, "off", 1);      # power off supply 1
## $error = blocks_misc_pwrcmd($fa, "on", 2);       # power on supply 2
## $error = blocks_misc_pwrcmd($fa, "off", "both"); # power off both supplys
##
sub blocks_misc_pwrcmd($$;$) {
   my ($h, $cmd, $n) = @_;
   my $name="blocks_misc_pwrcmd";
   my $power_cmd;
   my $error;

   if (!defined($h)) { return "ERROR: undefined host record passed"; }

   if (!($cmd = check_param($name, $cmd, "on|off|reboot"))) {
       return "ERROR: invalid param \$cmd passed";
   }

   if (defined($n) && !($n = check_param($name, $n, "1|2|both")) ) {
       return "ERROR: invalid param \$number passed";
   }


      # handle the space or no space after the powercmd
      if ($h->{powercmd}=~ m:([\/A-Za-z0-9_]+)\s*:) {
         $power_cmd = $1;
      } else {
         return "Error: illegal power command \"$h->{powercmd} $cmd\"";
      }

   # This script works with two basic power management utilities:
   #
   # 1. power_cycle - used in sunnyvale - provides null output on success 
   # 2. rpb - used in rtp - provides a text message on success
   #

   # handle when rpb provided is a full path
   if ($power_cmd eq "rpb" || $power_cmd =~ m:\/rpb:) {
       if (!($cmd eq "reboot")) {
       $error = "Error: illegal request \"$power_cmd $cmd\" ";          
       } else {
       $error = `$power_cmd $h->{id}`;
       if ($error =~ /Shutting down power supply/) {
           $error = 0;
       }
       }
   } else {
       if (defined($n)) {
       if ($n eq "both") {
           if ($cmd eq "reboot") {
           $error = `$power_cmd $h->{id} 1 off`;
                   if ($error =~ qr/device $h->{id} not found/) { #try rtp way
                  $error = `$power_cmd $h->{id}1 off`;
                   }
                   if (($error eq "")|| ($error =~ 
                   /This will take about 10 seconds to return/ )){
               $error = `$power_cmd $h->{id} 2 off`;
                       if ($error =~ qr/device $h->{id} not found/) { #try rtp way
                     $error = `$power_cmd $h->{id}2 off`;
                       }
                       if(($error ne "" ) && ($error !~ 
                       /This will take about 10 seconds to return/)){
                            return $error;
                       }
               $error = `$power_cmd $h->{id} 1 on`;
                       if ($error =~ qr/device $h->{id} not found/) { #try rtp way
                     $error = `$power_cmd $h->{id}1 on`;
                       }
                       if(($error ne "" ) && ($error !~ 
                       /This will take about 10 seconds to return/)){
                            return $error;
                       }
               $error = `$power_cmd $h->{id} 2 on`;
                       if ($error =~ qr/device $h->{id} not found/) { #try rtp way
                     $error = `$power_cmd $h->{id}2 on`;
                       }
                       if(($error ne "" ) && ($error !~ 
                       /This will take about 10 seconds to return/)){
                            return $error;
                       }
           } else {
               $error = `$power_cmd $h->{id} off`;
                       if(($error ne "" ) && ($error !~ 
                        /Processing - please wait .../ ) ) {
                            return $error;
                       }
               $error = `$power_cmd $h->{id} on`;
                       if(($error ne "" ) && ($error !~ 
                       /Processing - please wait .../ )){
                            return $error;
                       }
           }
           } else {
           $error = `$power_cmd $h->{id} 1 $cmd`;
                   if ($error =~ qr/device $h->{id} not found/) { #try rtp way
                  $error = `$power_cmd $h->{id}1 $cmd`;
                   }
                   if(($error eq "" )||($error =~ 
                   /This will take about 10 seconds to return/)){
               $error = `$power_cmd $h->{id} 2 $cmd`;
                       if ($error =~ qr/device $h->{id} not found/) { #try rtp way
                      $error = `$power_cmd $h->{id}2 $cmd`;
                       }
                       if(($error ne "" ) && ($error !~ 
                       /This will take about 10 seconds to return/)){
                            return $error;
                       }
           } else {
               $error = `$power_cmd $h->{id} $cmd`;
                       if(($error ne "" ) && ($error !~ 
                       /Processing - please wait .../ )){
                            return $error;
                       }
           }
           }
       } else {
           $error = `$power_cmd $h->{id} $n $cmd`;
               if ($error =~ qr/device $h->{id} not found/) { #try rtp way
              $error = `$power_cmd $h->{id}$n $cmd`;
               }
       }
       } else {
       $error = `$power_cmd $h->{id} $cmd`;
           if ($error =~ qr/device $h->{id} not found/) { 
              #maybe user runs test in RTP, machine has 2 power supplies 
          $error = `$power_cmd $h->{id}1 $cmd`;
          $error = `$power_cmd $h->{id}2 $cmd`;
           }
       }
   }

   if ($error =~ /This will take about 10 seconds to return/ ||
         $error =~ /Processing - please wait .../ ){
      $error =0;
   }
   return $error;
}
#
# This runs the truncate command on the client, its assumed to be 
# in the client path
#
sub blocks_misc_truncate($$$) {
    my($clnt, $file, $size) = @_;
    my $error;
    my $truncate_cmd="trunc -w";
    $error=say($clnt,"$truncate_cmd $file $size");
    return $error
}


## @name ::check_param
## @description 
##
## Checks if contents of a given scalar ($param) is a part of the
## regular expression ($mask).  Used for sytax and input validation.
##
## Returns the input on success, else 0 on error.  Prints error message 
## on error.
##
## @synopsis
## check_param {callers_name, param_input, "regular_expression"}
##
## @arg $name Required: Name of calling routine
## @arg $param Required: Input parameter to parse
## @arg $mask Required: Regular expression to check against.
## @example
## my $name="blocks_misc_pwrcmd";
## my $cmd="shutdown";  # this would be an error.
## if (!($cmd = check_param($name, $cmd, "on|off|reboot"))) {
##       return "ERROR: invalid param \$cmd passed";
## }
##
sub check_param($$$) {
    my($name, $param, $mask) = @_;
    if (!defined($param)) {
        print "$name: undefined parameter!\n";
        return 0;
    }
    if (!($param =~ /$mask/)) {
        print "$name: invalid request: \"$param\". Valid input is \"$mask\"\n";
        return 0;
    }
    return($param);
}

## @name ::wwn_set
## @description 
## Inserts colons in WWN values.  Returns 8 pairs of colon delimited values:
## (0a00200035a3091f == 0a:00:20:00:35:a3:09:1f)  Will also parse colons:
## (0a:00:20:00:35:a3:09:1f == 0a:00:20:00:35:a3:09:1f)
##
## @synopsis
## wwn_set {wwn}
##
## @arg $wwn Required: the wwn value to set
##
## @example
## my $wwn = "02002100a46701f1";
## $wwn = wwn_set($wwn);
##
sub wwn_set($) {
    my ($wwn) = @_;
    wwn_clear($wwn);
    $wwn =~ s/(..)(..)(..)(..)(..)(..)(..)(..)/$1:$2:$3:$4:$5:$6:$7:$8/;
    return ($wwn);
}

## @name ::wwn_clear
## @description 
## Cleans colons out of WWN values.  
## E.g.:  0a:00:20:00:35:a3:09:1f == 0a00200035a3091f
##
## @synopsis
## wwn_clear {wwn}
##
## @arg $wwn Required: the wwn value to clear
##
## @example
## my $wwn = "02:00:21:00:a4:67:01:f1";
## $wwn = wwn_clear($wwn);
##
sub wwn_clear($) {
    my ($wwn) = @_;
    $wwn = join(' ', split(' ', $wwn));
    $wwn = join('', split(/:/, $wwn));
    return ($wwn);
}

## @name ::buff_to_array
## @description 
## Converts the provided buffer from a scaler into an array with 
##
## @synopsis 
##  buff_to_array {buff}
##
## @arg $buff Required: the scalar string buffer to convert
##
## @example
## my $buff = "This is a test\nof the buff_to_array\r\nfunction\n";
## my @array = buff_to_array($buff);
##
sub buff_to_array {
   my ($buff) = @_;
   chomp($buff);
   $buff =~ s/\r//g;
   return(split(/\n/, $buff));
}

## @name :: string_search
## @description Searches input buffer for string or value.
## Returns first line found when "find" is used
## Returns total number of occurrences when "count" is used
## 
## @synopsis 
## string_search {$input, $action, $timeout} 
##
## @arg $input Required: buffer to search
## @arg $action Required: "find" | "count"
## @arg $param Required: string or value to find or count
##
## @example 
## my $fa = host($CLIENT);
## my $fc = connect ($CLIENT_CONN);
## my $volname="vol1"
## $line = string_search(ha_filer_cmd($fc, "vol status"), "find", $volname);
##
sub string_search($$$) {
    my ($input, $action, $param) = @_;
    my $name="string_search";
    my $count=0;

    return 0 unless defined($param);

    my $metaparam = quotemeta($param);

    #print "$name: $action ($param)\n";

    ExitScriptError("$name: Invalid action parameter passed")
        if (!($action = check_param($name, $action, "find|count")));
    
    foreach my $line (split(/\n/, $input)) {
        # skip blank lines
        next if ($line =~ /^$/);
        if ($line =~ /$metaparam/) {
            return ($line) if ($action eq "find");
            $count += 1;
        }
    }

    print "$name: $action ($param) returns $count\n";

    return $count;
}

## @name :: blocks_config_file
## @description Creates and manages configuration files.
## 
## A "configuration file" is an ascii flat file which contains any ascii
## data.  When blocks_config_file is called it checks-for/creates a
## subdirectory called "cfg" in the present working directory of the
## host machine (not the target machine).  All files are read from /
## written to this subdirectory by appending "cfg/" to the filename.
## Filenames and data are all free form.  The caller can read/write any
## data to any file wanted.
##
## @synopsis 
## string_search {$action, $filename, $data}
##
## @arg $action Required: "create" | "destroy" | "write" | "read" | "replace"
## @arg $filename Required: name of file 
## @arg $data Optional: data to write (required with create|write|replace)
##
## @example 
## my $fa = host($FILER);
## my $fc = connect ($FILER);
## blocks_config_file("replace", $FILER.".adapters",
##        ha_filer_cmd($fc, "fcp show adapters -v"));
##
sub blocks_config_file($$;$) {
    my ($action, $filename, $data) = @_;    
    my ($err);
    my $name = "blocks_config_file";

    ExitScriptError("$name: Invalid parameter passed")
        if (!($action = check_param($name, $action,
                    "create|destroy|write|read|replace")));

    $data = 0 unless ($data);

    ExitError("$name: Cannot create \"cfg\" directory\n $err")
        if ($err = `mkdir -p cfg`);

    $filename="cfg/".$filename;
    
  again:

    if ($action =~ /destroy|replace/) {
    logcomment("Destroying \"$filename\"");
    ExitError("$name: Cannot destroy \"$filename\"\n $data")
        if ($data = `rm -f $filename`);
    if ($action =~ /replace/) {
        $action = "create";
        goto again;
    }
    } elsif ($action =~ /create/) {
    ExitError("$name: Cannot open \"$filename\" for write")
        unless (open(FH, ">$filename"));
    logcomment("Creating \"$filename\"");
    ExitError("$name: Cannot write to \"$filename\"")
        unless (print FH $data);
    close (FH);
    } elsif ($action =~ /write/) {
    # Don't update the file if it already exists
    return if (-e $filename);
    ExitError("$name: Cannot open \"$filename\" for write")
        unless (open(FH, ">$filename"));
    logcomment("Writing \"$filename\"");
    ExitError("Cannot write to \"$filename\"")
        unless (print FH $data);
    close (FH);
    } elsif ($action =~ /read/) {
    ExitError("$name: \"$filename\" doesn't exit")
        unless (-e $filename);
    ExitError("$name: Cannot open \"$filename\" for read")
        unless (open(FH, "<$filename"));
    my @data_array=<FH>;
    my $size = @data_array;
    $data = join('', @data_array);
    logcomment("Read $size lines from \"$filename\"");
    close (FH);
    return $data;
    }

    return;
}

## not used
sub config_file_cmd($$$;$;$) {
    my ($action, $conn, $filename, $cmd, $timeout) = @_;    
    my $name = "config_file";
    my ($data, $size, $metacmd);

    ExitScriptError("$name: Invalid parameter passed")
        if (!($action = check_param($name, $action,
                    "create|destroy|write|read")));

    $cmd = 0 unless ($cmd);
    $timeout = 60 unless ($timeout);

    if ($action =~ /destroy/) {
        logcomment("Destroying \"$filename\"");
        ExitError("$name: Cannot destroy \"$filename\"\n $data")
            if ($data = `rm $filename`);
    } elsif ($action =~ /create/) {
        ExitScriptError("$name: Invalid parameter passed: \$cmd is null")
            unless ($cmd);
        $metacmd = quotemeta($cmd);
        (undef, undef, $data) = 
            say($conn, "$cmd", '-timeout', $timeout, '-ignore', "$metacmd\r\n",
            '-fatal', "$name: \"$cmd\" failed on $conn->{hostp}->{id}");
        ExitError("$name: Cannot open \"$filename\" for write")
            unless (open(FH, ">$filename"));
        logcomment("Creating \"$filename\"");
        ExitError("$name: Cannot write to \"$filename\"")
            unless (print FH $data);
        close (FH);
    } elsif ($action =~ /write/) {
        # Don't update the file if it already exists
        return if (-e $filename);
        ExitScriptError("$name: Invalid parameter passed: \$cmd is null")
            unless ($cmd);
        $metacmd = quotemeta($cmd);
        (undef, undef, $data) = 
            say($conn, "$cmd", '-timeout', $timeout, '-ignore', "$metacmd\r\n",
            '-fatal', "$name: \"$cmd\" failed on $conn->{hostp}->{id}");
        ExitError("$name: Cannot open \"$filename\" for write")
            unless (open(FH, ">$filename"));
        logcomment("Writing \"$filename\"");
        ExitError("Cannot write to \"$filename\"")
            unless (print FH $data);
        close (FH);
    } elsif ($action =~ /read/) {
        ExitScriptError("$name: Invalid parameter passed: \$cmd is null")
            unless ($cmd);
        ExitError("$name: \"$filename\" doesn't exit")
            unless (-e $filename);
        ExitError("$name: Cannot open \"$filename\" for read")
            unless (open(FH, "<$filename"));
        my @data_array=<FH>;
        $size = @data_array;
        logcomment("Read $size lines from \"$filename\"");
        close (FH);
        return @data_array;
    }

    return;
}

## @name :: parse_lun_list
## @description Convert LUN_LIST stanza string into lun_list array.
## Exit on error.
## Returns a hash with two arrays (@lun_list and @lun_ids) on success.
##
## Lun information is passed via a scalar string.  Each scaler string
## contains a colon delimited list of comma separated values: 
## 
##  (lun_id),(size) 
##
## Note that a range of lun-ids can be sepecified in the lun_id
## feild with a `-'.
##
## More at example.
##
## @synopsis 
## %list = parse_lun_list {$LUN_LIST} 
##
## @arg $LUN_LIST Required: lun_list scalar variable
##
## @example 
## my $LUN_LIST="0,2g:1-10,100m:12,250m:15,30g";
## my $lun_list = parse_lun_list($LUN_LIST);
## my @lun_list = @{$lun_list->{lun_list}};
## my @lun_ids = @{$lun_list->{lun_ids}};
##
sub parse_lun_list($) {
    my ($lun_list) = @_;
    my $name="parse_lun_list";
    my ($luns, @luns, @list);

    my @lun_list = split(/:/, $lun_list);

    foreach my $param (@lun_list) {
    my ($id, $size) = split(/,/,$param);
    if ($id =~ /\-/) {
        my ($begin, $end) = split(/\-/,$id);
        for (my $i=$begin; $i<=$end; $i++) {
        push (@list, join(',', $i, $size));
        push (@luns, $i);
        print "lun-id = $i, size = $size\n";
        }
    } else {
        push (@list, join(',', $id,$size));
        push (@luns, $id);
        print "lun-id = $id, size = $size\n";
    }
    }

    $luns = {};
    $luns->{lun_list} = [ @list ];
    $luns->{lun_ids} = [ @luns ];
    $luns->{disk_ids} = join(':',@luns);

    return ($luns);
}

1;




