# $Id$

## @summary Library to run initialization routines for Classic
##          systems. A similar routine for NG systems is also
##          available.
## @author mueller@netapp.com
## @author boyapati@netapp.com changed the script to be site
## independent.

package cifs_sanity_classic;
use strict;

use Tharn;
use filer;
use Utils;
use ExportFS qw(:cifs);
use CIFS::Setup qw(is_cifs_setup cifs_setup);

BEGIN {
  use Module::Common;
  Module::Common::setup_export_tags(
    ops => [ qw(init init_lite) ]
    );
}


sub init
{
	my $FILER = shift;
	my $hash = shift;

        my $cifs_server = checkLength($FILER);

	if (!cifs_setup(host => $FILER,
			'auth-type' => $$hash{'auth_type'},
		        domain => $$hash{'domain'},
		        domain_user => $$hash{'domain_user'},
		        domain_pass => $$hash{'domain_pass'},
		        dns_domainname => $$hash{'dns_domain'},
		        dns_server_list => $$hash{'dns_servers'},
		        dns_enable => $$hash{'dns_enable'},
                        'server-name' => $cifs_server,
			'ou-name' => $$hash{'ou_name'})) {
		return 0;
	}
	logresult("info", "\"cifs setup\" joined $$hash{'domain'}  successfully.");
	return init_lite($FILER);
}

sub init_lite
{
	my $FILER = shift;
	my $conn = make_conn($FILER);

	my %shares = ("sanity-ntfs" => "ntfs",
	              "sanity-mixed" => "mixed",
		      "sanity-unix" => "unix");

	foreach my $share (keys (%shares)) {
		if (!share_exists($conn, $share)) {
			filer_qtree_create($conn, $share);
			if (!filer_qtree_exists($conn, $share, "vol0")) {
				logresult("fail", "Unable to create $share qtree.");
				disconnect($conn);
				return 0;
			}
			if (!filer_qtree_security_set($conn, "/vol/vol0/$share", $shares{$share})) {
				logresult("fail", "Unable to set the $share qtree security style.");
				disconnect($conn);
				return 0;
			}
			if (!share_add($conn, $share, "/vol/vol0/$share")) {
				logresult("fail", "Unable to add the $share share.");
				disconnect($conn);
				return 0;
			}
		}
	}

	logresult("pass", "created \"ntfs\", \"mixed\" and \"unix\" shares successfully.");
	disconnect($conn);
	return 1;
}
sub checkLength {
    # Define a variable
    my $server_name = shift;
       $server_name = $1 if($server_name =~ m/(.*);/);
    # If the value is longer then 15 charcters
    if( length($server_name) > 15 ) {
        #Report server name.
        logresult("info","CIFS SERVER: $server_name is longer then 15 characters.");
        #Loop to remove the dashes and vowles
        foreach( qw(dashes vowels) ) {
            $server_name =~ s/[a,e,i,o,u]//g if($_ =~ m/vowels/);
            $server_name =~ s/[-,_]//g if($_ =~ m/dashes/);
            last if( length($server_name) <= 15);
        }
        #Still too long
        if( length($server_name) > 15 ) {
            $server_name = substr($server_name,0,15);
        }
        #Report new value
       logresult("info","CIFS SERVER shortened to: $server_name");
    }
    return("$server_name");
}

1;
