# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/config_client.pm#1 $
# 
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.
#
# lib/config_client.pm
## @summary Establishes NATE params for a single client configuration
## @description
##
## Obtains values for the standard 'client' configuration parameters.
## Such a configuration defines a single client perspective of a test bed.
##
## Builds arrays of FCP_CLIENTS ISCSI_CLIENTS PTI_CLIENTS  
## AIX_CLIENTS HPUX_CLIENTS LINUX_CLIENTS SOLARIS_CLIENTS UNIX_CLIENTS WIN_CLIENTS
## containing the hostnames of each such client.
##
## Builds CLIENT_FCP_INIT_NAMES array of FCP WWPN values.
## Builds CLIENT_ISCSI_INIT_NAMES array of iSCSI node names.
##
## @synopsis
##   require config_filer;
##   require config_client;
##
## @param CLIENT                    
##  DNS name of the client.
## @param CLIENTS
## A colon separated list of clients to be included in this test
## These values are used in creating the lun names.
## The default is to include all clients on the test bed.
## Instead of a client name,
## this list can also include entries enclosed in parentheses that
## describe the desired client in terms of a comma separated list of configuration types.
## See lib/ConfigTypes.pm for a list of valid values.
## The first entry in this list may be a numeric value that defines how many clients are 
## requested with these configuration types.
## @param CLIENT_CONN          
##  Client connection mode, default is $CLIENT.#rsh
##
## @param CLIENT_FCP_INSTALLED            
##  Set to 'yes' if blocks FCP hardware is installed on the client.
## @param CLIENT_FCP_WWNN             
##  FCP World Wide Node Name for this client.
## @param CLIENT_FCP_IO_DEV           
##  The FCP I/O device for the test case to use.
## @param CLIENT_ISCSI_INSTALLED              
##  Set to 'yes' if blocks iSCSI hardware is installed on the client.
## @param CLIENT_ISCSI_WWNN           
##  iSCSI World Wide Node Name for this client.
## @param CLIENT_ISCSI_IO_DEV             
##  The iSCSI I/O device for the test case to use.

## @param CLIENT_INITn_PROTOCOL     
##  'fcp' or 'iscsi' protocol of the n'th initiator
## @param CLIENT_INITn_WWNN     
##  World Wide Node Name of the n'th initiator
## @param CLIENT_INITn_WWPN     
##  World Wide Port Name of the n'th initiator
## @param CLIENT_INITn_LUN1     
##  First client LUN  on initiator n
## @param CLIENT_INITn_LUN2     
##  Second client LUN  on initiator n
## @param CLIENT_INITn_LUN3     
##  Third client LUN  on initiator n
## @param CLIENT_INITn_LUN4     
##  Fourth client LUN  on initiator n

## @param CLIENT_IO_DEV           
##  The I/O device for the test case to use.
##  Defaults to the value of $CLIENT_FCP_IO_DEV, $CLIENT_ISCSI_IO_DEV or $CLIENT_IO_DEV_1
## @param CLIENT_IO_DEV_n             
##  Each I/O device that the CLIENT is expected to have is defined with one of these parameters.
##  At some point, this package should test for the existence of these devices and create them
##  if they do not exist.
## @param CLIENT_IO_DEV_n_OFFSET_BEGIN
##  Byte offset into the n'th device where I/O operation is to begin.
## @param CLIENT_IO_DEV_n_OFFSET_END
##  Byte offset into the n'th device where I/O operation is to end.
## @param CLIENT_IO_DEV_n_SIZE
##  Size of the n'th device in bytes.
## @param CLIENT_IO_DEV_NUM
##  The number of IO devices that are being used by the current test.
##  This is for tests that use more than one device on a client.
##  The test script is to limit its testing to devices from 1 to this specified value.
##  Default: the number of defined CLIENT_IO_DEV_n devices

## @param CLIENT_IP           
##  IP address of interface to client, defaults to hosttype record DEFAULT_IP value
## @param CLIENT_IP_n             
##  Additional IP address of interface to client. 
##  Up to 4 additional addresses are recognized as separate parameters

## @param CLIENT_MODE          
##  'c2' for 2-way clustered client, or 'c4', 'c8'. Default is 'mono'
## @param CLIENT_MODEL        
##  Client model number
## @param CLIENT_OS       
##  Client operating system
## @param CLIENT_OS_VER       
##  Client operating system version
##
## @param CLIENT_PTI_INIT1_ID     
##  ID of PTI client first initiator port 
## @param CLIENT_PTI_INIT2_ID     
##  ID of PTI client second initiator port 
## @param CLIENT_PTI_INIT3_ID     
##  ID of PTI client third initiator port 
## @param CLIENT_PTI_INIT4_ID     
##  ID of PTI client fourth initiator port 
## @param CLIENT_PTI_INIT_ID     
##  ID of PTI client initiator port to test, defaults to $CLIENT_PTI_INIT1_ID 

## @param CLIENT_PTI_TARGET1_ID     
##  ID of PTI client first target port 
## @param CLIENT_PTI_TARGET2_ID     
##  ID of PTI client second target port 
## @param CLIENT_PTI_TARGET3_ID     
##  ID of PTI client third target port 
## @param CLIENT_PTI_TARGET4_ID     
##  ID of PTI client fourth target port 
## @param CLIENT_PTI_TARGET_ID     
##  ID of PTI client target port to test, defaults to $CLIENT_PTI_TARGET1_ID 

## @param CLIENT_VI_ADDRn The client VI device address corresponding to the CLIENT_VI_DEVn value. 
## These values use the IP address decimal octet format.
## @param CLIENT_VI_DEVn    The client VI device name as defined with ifconfig.
## @param CLIENT_VI_NAMEn The client VI device hostname as found in the host file.  
## @param CLIENT_IB_ADDRn The client IB device address corresponding to the CLIENT_IB_DEVn value. 
## These values use the IP address decimal octet format.
## @param CLIENT_IB_DEVn    The client IB device name as defined with ifconfig.
## @param CLIENT_IB_NAMEn The client IB device hostname as found in the host file.  
## @param CLIENT_DAFS_ADDRn The client DAFS device address corresponding to the CLIENT_DAFS_DEVn value. 
## These values use the IP address decimal octet format.
## @param CLIENT_DAFS_DEVn  The client DAFS device name as defined with ifconfig.
## @param CLIENT_DAFS_NAMEn The client DAFS device hostname as found in the host file.  

## @param LUN_LIST
## A colon separated list of lun id,size 
## NO LUN SETUP will be done unless this list is specified

## @author Charlie Hedstrom (hedstrom@netapp.com)
## @status public
## @see-also lib/config_base.pm
## @see-also lib/config_clientn.pm
## @change 2002-03-14 hedstrom: created
## @change 2002-03-19 hedstrom: document params
## @change 2002-03-25 hedstrom: update PTI params
## @change 2002-04-03 hedstrom: add PTI params to use vars
## @change 2002-04-12 hedstrom: add more LUN params, drop redundant $ENV defaults
## @change 2002-07-09 pressley: add CLIENT_IO_DEV_10
## @change 2002-07-24 hedstrom: add CLIENT_FCP_IO_DEV CLIENT_ISCSI_IO_DEV
## @change 2002-08-09 jhayden: change default of CLIENT_VI_DEV_? to /dev/orvi0
## @change 2002-10-02 hedstrom: rewrote for multiple client support
## @change 2002-10-12 taojin: use CLIENT_ISCSI_INIT_NAMES CLIENT_FCP_INIT_NAMES
## @change 2002-11-06 hedstrom: fix CLIENT_FCP CLIENT_ISCSI CLIENT_PTI
## @change 2002-12-03 hedstrom: save FCP WWPN values in FCP_INIT_NAMES array
## @change 2003-01-07 hedstrom: support config_clientc.pm, capture client lists
## @change 2003-01-10 hedstrom: override empty string param values when default exists
## @change 2003-01-15 hedstrom: support up to 5 clients
## @change 2003-01-16 hedstrom: add linux support
## @change 2003-02-12 hedstrom: add CLIENTS array and client_param_get
## @change 2003-02-27 hedstrom: rename CLIENTS array to ALL_CLIENTS, add IO_DEV parameters
## @change 2003-03-12 hedstrom: fix DAFS VI and IB params, add IO_DEV params
## @change 2003-03-13 sli: add CLIENT_INIT_LUN, CLIENT_INIT1_LUN, CLIENT_INIT2_LUN, CLIENT_INIT3_LUN, and CLIENT_INIT4_LUN
## @change 2003-04-02 hedstrom: properly initialize CLIENT_FCP_IP CLIENT_IB CLIENT_VI CLIENT_ISCSI
## @change 2003-05-19 hedstrom: add default for CLIENT_IO_DEV_NUM
## @change 2003-06-04 hedstrom: remove limit on 'n' for CLIENTn, fix Perl warnings
## @change 2003-06-09 hedstrom: fix client_param_get to search ALL_CLIENTS
## @change 2003-06-09 hedstrom: set CLIENTn_IO_DEV_NUM to 0 if no devices definitions were found
## @change 2003-06-17 hedstrom: Fix problem in counting IO_DEV definitions
## @change 2003-09-24 hedstrom: use client name, not connection as parameter to client_get_param
## @change 2003-10-13 hedstrom: move client_get_param to config_base.pm
## @change 2003-10-22 pressley: fixed default for CLIENT_IO_PASS_THRU for burt 101193
## @change 2004-03-09 hedstrom: use CLIENT_ params as default for any CLIENTn_ param 
## @change 2004-06-29 hedstrom: preserve command line CLIENT_ params, ensure they are read in too
## @change 2004-06-30 hedstrom: propagate CLIENT1 values to CLIENT when there is no reassignment
## @change 2005-01-12 hedstrom: limit client_device_setup to current LUNLIST request
## @change 2005-01-27 hedstrom: burt151200  skip Windows partitioning if make_fs=0
## @change 2005-03-04 hedstrom: make SKIP_CLEANUP = ON_FAIL the default
## @change 2005-03-31 hedstrom: always SKIP_CLEANUP for subtests

print("Starting ".__FILE__."\n");

use strict;
use English;
use Carp;
use Tharn;
use config_base;
# Client package
use SanClient;
use TestEnv::MinWin;

use vars qw(@ISA @EXPORT
    @CLIENT_FCP_INIT_NAMES
    @CLIENT_ISCSI_INIT_NAMES
    
    @CLIENTS_FCP_INIT_NAMES
    @CLIENTS_ISCSI_INIT_NAMES
    
    @ALL_CLIENTS
    @FCP_CLIENTS @ISCSI_CLIENTS 
    @DAFS_CLIENTS @VI_CLIENTS @IB_CLIENTS
    @PTI_CLIENTS
    @AIX_CLIENTS @HPUX_CLIENTS @LINUX_CLIENTS @SOLARIS_CLIENTS @UNIX_CLIENTS @WIN_CLIENTS 
    
    $CLIENT $CLIENT_CONN $CLIENT_FCP
    $CLIENT_FCP_IP $CLIENT_FCP_IO_DEV
    
    $CLIENT_IP $CLIENT_IP_1 $CLIENT_IP_2 $CLIENT_IP_3 $CLIENT_IP_4 
    
    $CLIENT_ISCSI
    $CLIENT_ISCSI_IP $CLIENT_ISCSI_IO_DEV
    
    $CLIENT_IO_DEVS
    $CLIENT_IO_DEV_NUM    @SETUP_CLIENT_NAMES
    
    $CLIENT_IO_FS_TYPE $USE_FS $CLIENT_VM_VOL_SIZE $CLIENT_VM_LAYOUT
    $CLIENT_VM_STRIPE_SIZE $CLIENT_VM_NUM_LVS
    
    $CLIENTS $FILERS $MULTIPATH $LUN_LIST 
    
    $LOGDIR
    $PROJECT
    
    @FSOBJ_LIST
    
    $SKIP_CLEANUP $SKIP_SETUP
    );

require Exporter;
@ISA = ('Exporter');

@EXPORT = qw(
    client_cleanup
    client_device_setup
    client_params_get
    );
#
# Define global parameters within this package and initialize them
#
# Determine command line invocation parameters
# to preserve user's command line values
# $ARGV does not work, since NATE ate the original arguments
my %nate_args_client;  # key=param name, value is original param value
my %nate_args;         # key=param name, value=1 if seen on ntest invocation
my @loglines;
my $starting_log = "$ENV{LOGDIR}/00_ntest.log";
if (open (LOG,$starting_log) ) {
    @loglines = (<LOG>);
    close(LOG);
    my $invocation_line = '';
    if (defined($loglines[0])) {
        $invocation_line = $loglines[0];
        if ($invocation_line =~ m/ntest:/) {
            print("$invocation_line\n");
            # Now parse the input to determine which arguments were passed
            my @input_args = split(" ",$invocation_line);
            foreach my $input_arg (@input_args) {
                if ($input_arg =~ m/^(\S+)=/) {
                    $nate_args{$1} = 1;
                }
            }
        } else {
            logresult('WARN', "Original NATE invocation could not be determined from: $starting_log");
        }
    } else {
        logresult('WARN', "Empty NATE log file: $starting_log");
    }
} else {
    logresult('WARN', "Could not open NATE log file: $starting_log");
}

# Subroutines
#

################################################################################
# Subroutine Name: client_cleanup
# Objective:       This routine destroys the devices defined by the  
#                  FILERS, CLIENTS and LUN_LIST parameters.
#    SKIP_CLEANUP = ON_FAIL means do not cleanup if there was a FAIL or FATAL error
#                    - 0 means do cleanup, 1 means SKIP_CLEANUP (default)
#                    - This value controls cleanup regardless of setup
#                    - If LUN_LIST = '' then we skip LUN cleanup.
################################################################################
my $StartingPID = $PID;  # This is set only during the initial loading of this module
sub client_cleanup(;$) {
    my $result_now = shift;

    my @callstack;
    Burt::CallStack(\@callstack,0);
    if ($PID ne $StartingPID and $callstack[1] =~ /\/config_client.pm\s*\(/) {
        # Do not allow subtests to invoke any cleanup until the initial PID
        print("Ignoring config_client_cleanup call from subtest PID $PID");
        return;
    }
    &logcomment("config_client_cleanup called in PID $PID ",@callstack);
    
     param('SKIP_CLEANUP','-default',"1");
     if ($main::SKIP_CLEANUP =~ m/ON_FAIL/i) {
         # See if we have any FATAL or FAIL results objects
         my @results = (prev_results(),own_results());
         if (defined($result_now)) {
             push(@results,$result_now);
         }
         foreach my $result_obj (prev_results(),own_results()) {
             my $result_type = $result_obj->type();
             if ($result_type eq "FAIL" or $result_type eq "FATAL") {
                 &logcomment("FAIL found, skipping Cleanup by user request (SKIP_CLEANUP = $main::SKIP_CLEANUP)");
                 param('SKIP_CLEANUP','-set',"1");
                 last;
             }
         }
     }
     if ($main::SKIP_CLEANUP eq '1') {
         return;
     } else {
         &logcomment("Cleanup by user request (SKIP_CLEANUP = $main::SKIP_CLEANUP)");
     }

     my ($filesys, $disk_obj);
     my $client_connection;

     my @disk_objects; # A list of all 'disk' objects to be processed
     

     # If this call is made within the same invocation that created 
     # the disk and file system objects, it works OK without
     # further setup
     # However, if it is called separately from the script that did the setup,
     # these objects need to be re-created to match what exists

     if (0 == @FSOBJ_LIST) {
         # Discover disks associated with the LUN_LIST
         param('FILERS','-default','');
         my @filers = split(/:/, $main::FILERS);
         if (0 == @filers) {
             logcomment("FILERS list is empty, no clean up performed");
             return;
         }

         param('CLIENTS','-default','');
         my @clients = split(':',$main::CLIENTS);
         if (0 == @clients) {
             logcomment("CLIENTS list is empty, no clean up performed");
             return;
         }

         param('CLEANUP_LIST','-default','all');
         if ($main::CLEANUP_LIST eq '') {
             $main::CLEANUP_LIST = "all";
         }
         if ($main::CLEANUP_LIST !~ m/all|lun/i) {
             logcomment("CLEANUP_LIST does not request LUN clean up to be performed: $main::CLEANUP_LIST");
             return;
         }

         param('MULTIPATH','-default',''); 

         param('FILER_EXCLUDE_LUNS','-default','');
         my $keep_luns = ' '.$main::FILER_EXCLUDE_LUNS.' ';

         my @filer_wwnns;
         foreach my $filer (@filers) {
             my $fh = host($filer);
             my $fc = connect ("$filer#rsh-socket");
             print("Checking WWNN for filer: $filer\n"); #DEBUG
             my $next_wwnn = filer_fcp_nodename_get($fc);
             if (defined($next_wwnn) and $next_wwnn ne '') {
                 $next_wwnn =~ s/://g;  #drop optional colon separators 
                 push(@filer_wwnns,$next_wwnn)
             }
             $next_wwnn = filer_iscsi_nodename_get($fc);
             if (defined($next_wwnn) and $next_wwnn ne '') {
                 push(@filer_wwnns,$next_wwnn)
             }
         }
         print("Got WWNNs: ".join(' ',@filer_wwnns)."\n"); #DEBUG

         param('LUN_LIST','-default','');
         if ($main::LUN_LIST eq '') {
              logcomment("LUN_LIST is empty, no LUN clean up performed");
              return;
         }
         my $client_io_devs = $main::LUN_LIST;
         # Drop any extra arguments that this function does not support
         # (Some input may have a LUN size and LUN ostype here)
         $client_io_devs =~ s/,.*?:/:/g;  # ? keeps it from being greedy
         $client_io_devs =~ s/,.*//;
         if ($client_io_devs =~ m/-/) {
             my @device_numbers;
             my @device_list = split(/:/,$client_io_devs);
             foreach my $device_list_entry (@device_list) {
                 if ($device_list_entry =~ m/^(\d+)-(\d+)$/) {
                     my ($start,$end) = ($1,$2);
                     for (my $dev_num = $start; $dev_num<=$end; $dev_num++) {
                         push(@device_numbers,$dev_num);
                     }
                 } else {
                     if ($device_list_entry !~ m/$(\d+)$/) {
                         ExitScriptError("Device list must have numeric entries, not: $device_list_entry");
                     }
                     push(@device_numbers,$device_list_entry);
                 }
             }
             # Replace the old device list containing ranges with the new colon separated list
             $client_io_devs = join(':',@device_numbers);
          }
          if ($client_io_devs !~ m/^\s*$/) {
              foreach my $client (@clients) {
                  my @devices = ();
                  my $hostobj = host($client,'-ok');
                  my $client_conn;
                  if ($hostobj->{hosttype} =~ m/windows/) {
                      # First ensure client is alive
                      my $client_ip = $hostobj->default_ip();
                      if (host_wait($client_ip)) {
                          # Do not use rsh-socket from UNIX (root is required, NATE prevents its use)
                          $client_conn =  "#rsh-persay";    
                          if (!TestEnv::MinWin::minwin_compatible($client)) {
                              # Give it a second chance
                              print("MinWin compatibility failed for $client at $client_ip, trying again\n"); #DEBUG
                              sleep(30);
                              if (!TestEnv::MinWin::minwin_compatible($client)) {
                                  # each client must be MinWin Compatible, we cannot continue
                                  ExitConfigError("All Windows clients must be MinWin compatible. $client is not");
                              }
                          }
                      } else {
                          ExitConfigError("Client $client at $client_ip does not appear to be alive");
                      }
                  } else {
                      $client_conn =  "#telnet;inherit=0";  # rsh-persay may hang Solaris
                  }
                  my $client_connection = connect($client.$client_conn,-ok);
                  if (defined($client_connection)) {
                      # Create a new Host object, passing the client connection
                      # and list of devices LUNs you want to use (i.e 0:1:2)
                      my $hostobj = SanClient->new(-connection => $client_connection,
                                                   -devices    => $client_io_devs);

                      # Some have reported that the rescan operation (on Windows) 
                      # may not be reliable. The following loop supports retry
                      my $retry = 1;
                      my $max_retry = 3;
                      while ($retry) {
                          $retry = 0;

                          # Discover resources on the host. This includes configuration
                          # information (configuration files) and sanlun information.
                          # It will also run devfsadm to discover LUNs
                          $hostobj->discover();

                          # Get the list of device pathnames to use
                          # on disk and filesystem operations. This is 
                          # extracted from the object which got this information
                          # from sanlun during the discover method.
                          @devices = $hostobj->get_devices(-multipath => $main::MULTIPATH,
                                                           -filer_wwnns => join(' ',@filer_wwnns),
                                                            );
                          if (@devices) {
                              &logcomment("List of all requested devices on $client includes:",
                                          sort(@devices));
                          } else {
                              if ($hostobj->{hosttype} =~ m/windows/) {
                                  if ($max_retry > 0) {
                                      $max_retry--;
                                      $retry = 1;
                                      # When a client had a LUN at this ID,
                                      # it takes an extra rescan pass to drop the old one
                                      # before finding the new one
                                      &logcomment("Retrying rescan, none of the requested devices ($client_io_devs) are available to use on client $client");
                                      sleep(20);  # Perhaps giving it some time will help before rescaning?
                                  } else {
                                      &logcomment("None of the requested devices ($client_io_devs) are available to use on client $client");
                                  }
                              } else {
                                  &logcomment("None of the requested devices ($client_io_devs) are available to use on client $client");
                              }
                          }
                      }

                      if (@devices) {
                          # create the objects representing the passed devices
                          &logcomment("Starting host cleanup on $client");

                          # These scripts do not support logical volumes with MPIO on Windows
                          if ($main::MULTIPATH and $hostobj->{hosttype} !~ m/windows/) {
                              my $diskobj = Disk->new(-hobj   => $hostobj,
                                                      -multipath => $main::MULTIPATH,
                                                      -devices   => \@devices,
                                                      -filer_wwnns => join(' ',@filer_wwnns),
                                                      -filers => join(':',@filers));

                              push (@disk_objects, $diskobj);

                              if (defined($diskobj->{vm_device_list})) {
                                  my $filesys = 0;;
                                  foreach my $device (@{$diskobj->{vm_device_list}}) {
                                      $filesys++;
                                      # create a filesystem from the passed disk object
                                      my $fsobj = FS->new(-cleanup => 1,
                                           -hobj => $hostobj,
                                           -dobj => $diskobj);
                                      push (@FSOBJ_LIST, $fsobj);
                                  }
                              }

                          } else {
                              # no multipathing or Windows host
                              # Each device is to be a separate volume - not part of a multiple device volume group
                              my $dev_list = join(', ',sort(@devices));
                              print("client_cleanup: devices without logical volumes: $dev_list\n");
                              foreach my $device (@devices) {

                                  # create a disk object representing the passed device
                                  my $diskobj = Disk->new(-hobj => $hostobj,
                                                          -multipath => $main::MULTIPATH,
                                                          -devices   => [($device)],
                                                          -filer_wwnns => join(' ',@filer_wwnns),
                                                          -filers => join(':',@filers));
                                  push (@disk_objects, $diskobj);

                                  # create a filesystem object from the passed disk object
                                  my $fsobj = FS->new(-cleanup => 1,
                                       -hobj => $hostobj,
                                       -dobj => $diskobj);
                                  push (@FSOBJ_LIST, $fsobj);
                              }
                          }

                          # Determine the filesystem
                    }
                } else {
                  ExitSoftError("A connection to client $client$client_conn could not established");
                }
            }
        }
    }

    if (@FSOBJ_LIST) {
        # cleanup the client by first umounting the filesystems
        foreach $filesys (@FSOBJ_LIST) {
            # umount the filesystem
            $filesys->umount();
        }

        # destroy the filesystems objects
        &logcomment("Destroying Filesystem objects");
        foreach $filesys (@FSOBJ_LIST) {
            $filesys->destroy();
        }           

    } else {
        &logcomment("No Filesystem objects found to clean up");
    }

    if (@disk_objects) {
        # destroy the disk  objects
        &logcomment("Destroying Disk objects");
        foreach my $disk_obj (@disk_objects) {
            # cleanup the disk object
            if (defined($disk_obj)) {
                $disk_obj->destroy();
            }
        }           
    } else {
        &logcomment("No Disk objects found to clean up");
    }
    
    # next, cleanup the filer and the LUNs/igroups you used
    &config_filer_cleanup();

}

## @name ::client_param_copy
## @summary Copies a param value to a second param name
## @description
## This function performs a parameter copy operation.
## If the target parameter name is defined and not empty
## and is different from the source parameter name,
## the target parameter value is set.
## The value is taken from the source parameter value, if it is defined.
## Else, the corresponding CLIENT_ parameter is used if it exists.
## If source prefix begins with NEW,
## the source parameter is erased after the copy
## Any new CLIENT1_ value is also copied to CLIENT_ 
## unless the CLIENT_ value was an invocation request
## @arg $param_suffix Required: The param name without a prefix (include the underscore)
## @arg $param_source_prefix Required: The param name prefix that may define the param value
## @arg $param_target_prefix Optional: The param name prefix that is to get a copy of the param value
sub client_param_copy($$;$) {
    my $param_suffix = shift;
    my $param_source_prefix = shift;
    my $param_target_prefix = shift;
    
    if (!defined($param_suffix) or $param_suffix eq '') {
        ExitScriptError("A client suffix must be specified");
    }
    if (!defined($param_source_prefix) or $param_source_prefix eq '') {
        ExitScriptError("A source client prefix must be specified");
    }
    
    no strict 'refs';
    # Make sure source parameters have been processed
    param($param_source_prefix.$param_suffix,-default,undef);
    my $param_value  = ${'main::'.$param_source_prefix.$param_suffix};
    my $client_value = undef;
    if (!defined($param_value)) {
        # Skip temporary copying to 'NEW" clients
        if (defined($param_target_prefix) and $param_target_prefix !~ m/^NEW/) {
            # Only read the CLIENT_ value if we need to see it
            # Skipping this when NEW parameters are being set
            # because it will be done later when NEW to CLIENTn is performed 
            if (defined($nate_args{'CLIENT'.$param_suffix})) {
                # Only read it if it was specified on the command line
                param('CLIENT'.$param_suffix,-default,undef);
                $client_value  = ${'main::CLIENT'.$param_suffix};
            }
        }
    } else {
        # Remember original CLIENT and CLIENT1 parameters
        if ($param_source_prefix eq 'CLIENT' or $param_source_prefix eq 'CLIENT1') {
            if (!defined($nate_args_client{$param_source_prefix.$param_suffix})) {
                $nate_args_client{$param_source_prefix.$param_suffix} = $param_value;
            }
        }
    }
    
    if (defined($param_target_prefix) and $param_target_prefix ne '') {
        # This request is part of a client reordering
        if (defined($param_value) and $param_value ne '') {
            if ($param_target_prefix ne $param_source_prefix) {
                # The source and target clients are different
                # But still ensure that command line CLIENT values are not overridden
                if ($param_target_prefix eq 'CLIENT' and defined($nate_args{'CLIENT'.$param_suffix})) {
                    param('CLIENT'.$param_suffix,'-default',undef);
                    print("Preserving CLIENT"."$param_suffix=${'main::CLIENT'.$param_suffix} from command line\n");
                } else {
                    print("Setting $param_target_prefix"."$param_suffix=$param_value from $param_source_prefix\n");
                    param($param_target_prefix.$param_suffix,'-set',$param_value);
                }
            } else {
                # Source and Target parameters are the same
                # and a non-empty value was seen
                # It should be OK to do nothing
            }
        } elsif (defined($client_value) and $client_value ne '') {
            # There was no value from the expected CLIENTn
            # See if there is a generic value from the CLIENT parameter
            if ($param_target_prefix ne 'CLIENT') {
                # OK to store when source is not the target
                print("Setting $param_target_prefix"."$param_suffix=$param_value from CLIENT value on command line\n");
                param($param_target_prefix.$param_suffix,'-set',$client_value);
            }
        } else {
            # There was no value to assign, so undef any existing value
            # Skip temporary copying to 'NEW" clients (it is already undefined)
            if ($param_target_prefix !~ m/^NEW/) {
                # If target and source are the same, the value is already cleared
                if ($param_target_prefix ne $param_source_prefix) {
                    print("Clearing any $param_target_prefix"."$param_suffix value (not defined by $param_source_prefix or CLIENT)\n");
                    param($param_target_prefix.$param_suffix,'-set',undef);
                }
            }
        }
    }

    if (defined($param_value)) {
        # If target prefix is CLIENT1,
        # we still need to copy the new value to CLIENT
        if ($param_target_prefix eq 'CLIENT1') {
            # Can we tell if this CLIENT parameter was specified
            # on the command line (thus, should not be overridden)?
            if (defined($nate_args{'CLIENT'.$param_suffix})) {
                param('CLIENT'.$param_suffix,'-default',undef);
                print("Preserving CLIENT"."$param_suffix=${'main::CLIENT'.$param_suffix} from command line\n");
            } else {
                print("Setting CLIENT"."$param_suffix=$param_value from CLIENT1\n");
                param('CLIENT'.$param_suffix,'-set',$param_value);
            }
        }
        if ($param_source_prefix =~ m/^NEW/) {
            # Erase the temporary parameter after it has been reassigned
            param($param_source_prefix.$param_suffix,-set,undef);
        }
    }
    
    use strict 'refs';
}

## @name ::client_params_get
## @summary Establishes all the client param values associated with the requested 'CLIENTn'
## @description
## @arg $client_param Required: The client param name that acts as a prefix to all param values
sub client_params_get($;$) {
   my $client = shift;
   my $client2 = shift;

   # PLEASE NOTE: client_param_copy must be called only after all references to
   # the source parameter value have been executed.
   # since it erases the source value when the source parameter name starts with 'NEW_'

   if (!defined($client) or $client eq '') {
      ExitScriptError("A client parameter prefix value must be specified, 'CLIENT1' or example.");
   }
   if ($client eq 'CLIENT') {
      use POSIX ('uname');
      my (undef, $local_hostname) = POSIX::uname();
      # Keep just the node name if the full nodename.hq.netapp.com format was given
      $local_hostname =~ s/\..*//;
      param($client,          '-default', $local_hostname);
   } else {
      # Values like 'CLIENT1' 'CLIENT2' are expected
      param ($client,  '-mesg', "Name of $client" );
   }

   no strict 'refs';
   my $ch = host(${'main::'.$client});
   param('CLIENT_MODE',     '-default',"mono");

   # Determine this client's configuration types, surround with commas to aid searching
   param($client.'_CONFIGS','-default',"");
   my $client_configs = ','.${'main::'.$client.'_CONFIGS'}.',';
   client_param_copy('_CONFIGS',$client,$client2);

   param ($client.'_IP',    '-default', ( $ch->{default_ip} || '' ));
   print("config_client saw $client"."_IP=${'main::'.$client.'_IP'}\n");
   client_param_copy('_IP',$client,$client2);
   param ($client.'_IFC',        '-default', undef);
   
   if ( $ENV{CONNECTTYPE} ) {
      param ($client.'_CONN',    '-default', ( ${'main::'.$client}.  $ENV{CONNECTTYPE} ));
   } else {
       # If this connection is really not defined, pick a default based on hosttype
       my $hosttype = $ch->hosttype();
       my $default_conn;
       if ($hosttype =~ m/windows/) {
          $default_conn = "#rsh";
       } else {
          $default_conn = "#telnet";
       }
      param ($client.'_CONN',    '-default', ( ${'main::'.$client} . $default_conn));
   }
   client_param_copy('_CONN',$client,$client2);
   
   param($client.'_OS','-default',"");
   # Remember OS type for each client except 'CLIENT'
   if ($client ne 'CLIENT') {
      if (${'main::'.$client.'_OS'} =~ m/aix|hp-ux|linux|sunos/i) {
         push(@UNIX_CLIENTS,$client);
         if (${'main::'.$client.'_OS'} =~ m/sunos/i) {
            push(@SOLARIS_CLIENTS,$client);
         } elsif (${'main::'.$client.'_OS'} =~ m/aix/i) {
            push(@AIX_CLIENTS,$client);
         } elsif (${'main::'.$client.'_OS'} =~ m/hp-ux/i) {
            push(@HPUX_CLIENTS,$client);
         } elsif (${'main::'.$client.'_OS'} =~ m/linux/i) {
            push(@LINUX_CLIENTS,$client);
         }
      } elsif (${'main::'.$client.'_OS'} =~ m/windows/i) {
         push(@WIN_CLIENTS,$client);
      }

   }
   param($client.'_OS_VER','-default',undef);
   client_param_copy('_OS_VER',$client,$client2);
   
   param($client.'_MODEL',      '-default', undef);
   if (defined(${'main::'.$client.'_MODEL'}) and ${'main::'.$client.'_MODEL'} ne '') {
      $ch->{'model'} = ${'main::'.$client.'_MODEL'};
      #$CLIENT_MODELS{${'main::'.$client}} => ${'main::'.$client.'_MODEL'};
      #print("CLIENT_MODELS{${'main::'.$client}} = $ch->{'model'}\n");  #DEBUG
   }
   client_param_copy('_MODEL',$client,$client2);

   param($client.'_PASSWORD',   '-default', $ch->{default_password});
   client_param_copy('_PASSWORD',$client,$client2);
   
   param($client.'_FCP_WWNN','-default',undef);
   client_param_copy('_FCP_WWNN',$client,$client2);
   param($client.'_ISCSI_WWNN','-default',undef);
   client_param_copy('_ISCSI_WWNN',$client,$client2);
   #print("${client}_ISCSI_WWNN=${'main::'.$client.'_ISCSI_WWNN'}\n"); #DEBUG
   if (defined(${'main::'.$client.'_ISCSI_WWNN'})) {
       push(@{$client.'_ISCSI_INIT_NAMES'},${'main::'.$client.'_ISCSI_WWNN'});
   }
   
   # Build arrays of client parameters
   
   @{$client.'_FCP_INIT_NAMES'} = ();
   @{$client.'_ISCSI_INIT_NAMES'} = ();
   my $fcp_installed_default = "";
   my $iscsi_installed_default = "";

   my $adapter = 1;

   while (1) {
      # Pick up all Initiator HBA parameters
      # If PROTOCOL was defined, get the other values
      param($client.'_INIT'.$adapter.'_PROTOCOL','-default',undef);
      if (!defined(${'main::'.$client.'_INIT'.$adapter.'_PROTOCOL'}) or ${'main::'.$client.'_INIT'.$adapter.'_PROTOCOL'} eq '') {
         last;
      }
      #print("${client} adapter=$adapter \n"); #DEBUG

      if (${'main::'.$client.'_INIT'.$adapter.'_PROTOCOL'} =~ /^fcp$/i) {
         #print("CLIENT $client supports FCP\n"); #DEBUG
         $fcp_installed_default = "yes";
         param($client.'_INIT'.$adapter.'_WWNN','-default',undef);
         if (!defined(${'main::'.$client.'_INIT'.$adapter.'_WWNN'}) or ${'main::'.$client.'_INIT'.$adapter.'_WWNN'} eq '') {
            if (defined(${'main::'.$client.'_FCP_WWNN'}) and ${'main::'.$client.'_FCP_WWNN'} ne '') {
               param($client.'_INIT'.$adapter.'_WWNN','-set',${'main::'.$client.'_FCP_WWNN'});
            }
         }
         client_param_copy('_INIT'.$adapter.'_WWNN',$client,$client2);

         param($client.'_INIT'.$adapter.'_WWPN','-default','');
         if (${'main::'.$client.'_INIT'.$adapter.'_WWPN'} ne '') {
            push(@{$client.'_FCP_INIT_NAMES'},${'main::'.$client.'_INIT'.$adapter.'_WWPN'});
            push(@CLIENTS_FCP_INIT_NAMES,${'main::'.$client.'_INIT'.$adapter.'_WWPN'});
            #print("CLIENTS_FCP_INIT_NAMES=@CLIENTS_FCP_INIT_NAMES\n");#DEBUG
         } else {
            logresult('WARN',,"No FCP node name for $client ${'main::'.$client} adapter $adapter \n");
         }
         client_param_copy('_INIT'.$adapter.'_WWPN',$client,$client2);
      } 
      if (${'main::'.$client.'_INIT'.$adapter.'_PROTOCOL'} =~ /^iscsi$/i) {
         #print("Client $client supports iSCSI\n"); #DEBUG
         $iscsi_installed_default = "yes";
         param($client.'_INIT'.$adapter.'_WWNN','-default',undef);
         if (!defined(${'main::'.$client.'_INIT'.$adapter.'_WWNN'}) or ${'main::'.$client.'_INIT'.$adapter.'_WWNN'} eq '') {
            if (defined(${'main::'.$client.'_ISCSI_WWNN'}) and ${'main::'.$client.'_ISCSI_WWNN'} ne '') {
               param($client.'_INIT'.$adapter.'_WWNN','-set',${'main::'.$client.'_ISCSI_WWNN'});
            }
         } else {
            if (!defined(${'main::'.$client.'_ISCSI_WWNN'}) or ${'main::'.$client.'_ISCSI_WWNN'} eq '') {
               param($client.'_ISCSI_WWNN','-set',${'main::'.$client.'_INIT'.$adapter.'_WWNN'});
            }
         }

         if (${'main::'.$client.'_INIT'.$adapter.'_WWNN'} ne '') {
            push(@{$client.'_ISCSI_INIT_NAMES'},${'main::'.$client.'_INIT'.$adapter.'_WWNN'});
            push(@CLIENTS_ISCSI_INIT_NAMES,${'main::'.$client.'_INIT'.$adapter.'_WWNN'});
            #print("CLIENTS_ISCSI_INIT_NAMES=@CLIENTS_ISCSI_INIT_NAMES\n");#DEBUG
         } else {
            logresult('WARN',,"No ISCSI node name for $client ${'main::'.$client} adapter $adapter \n");
         }
         client_param_copy('_INIT'.$adapter.'_WWNN',$client,$client2);
      } 
      if (${'main::'.$client.'_INIT'.$adapter.'_PROTOCOL'} !~ /^fcp$|^iscsi$/i) {
         ExitConfigError("Unrecognized protocol for ".$client."_INIT".$adapter."_PROTOCOL: '".${'main::'.$client.'_INIT'.$adapter.'_PROTOCOL'}."'");
      }
      # Copy after we have used this variable 
      client_param_copy('_INIT'.$adapter.'_PROTOCOL',$client,$client2);

      my $lun = 1;
      while (1) {
         param($client.'_INIT'.$adapter.'_LUN'.$lun,'-default',"");
         if (${'main::'.$client.'_INIT'.$adapter.'_LUN'.$lun} eq '') {
            last;
         }
         client_param_copy('_INIT'.$adapter.'_LUN'.$lun,$client,$client2);
         $lun++;
      }

      $adapter++;
   }
   print("${client}_FCP_INIT_NAMES=@{$client.'_FCP_INIT_NAMES'}\n");
   print("${client}_ISCSI_INIT_NAMES=@{$client.'_ISCSI_INIT_NAMES'}\n");
   
   # client interfaces
   param($client.'_IFS',        '-default', undef);
   client_param_copy('_IFS',$client,$client2);

   param($client.'_IPS',        '-default', "");
   ${'main::'.$client.'_IPS'} = ${'main::'.$client.'_IP'};
   my $ip = 1;
   while ($ip > 0) {
      param($client.'_IP_'.$ip,'-default',"");
      if (${'main::'.$client.'_IP_'.$ip} eq '') {
         $ip = 0;
         last;
      }
      if ( !${'main::'.$client.'_IPS'} ) {
         ${'main::'.$client.'_IPS'} = ${'main::'.$client.'_IPS'}.",".${'main::'.$client.'_IP_'.$ip};
      } else {
         ${'main::'.$client.'_IPS'} = ${'main::'.$client.'_IP_'.$ip};
      }
      $ip++;
   }
   client_param_copy('_IPS',$client,$client2);

   
   # VI Interfaces to the Filer
   my $vi_num = 1;
   while ($vi_num <= 4) {
      param($client.'_VI_ADDR'.$vi_num,'-default',"");
      if (${'main::'.$client.'_VI_ADDR'.$vi_num} ne '') {
         my $vi_dev = $vi_num - 1;
         param($client.'_VI_DEV'.$vi_num,'-default',"");
         if (${'main::'.$client.'_VI_DEV'.$vi_num} eq '') {
            param($client.'_VI_DEV'.$vi_num,'-set',"/dev/orvi$vi_dev");
         }
         client_param_copy('_VI_DEV'.$vi_num,$client,$client2);
         param($client.'_VI_NAME'.$vi_num,'-default',"");
         client_param_copy('_VI_NAME'.$vi_num,$client,$client2);
      } else {
         last;
      }
      $vi_num++;
   }
   # Define default VI values
   param($client.'_VI_ADDR','-default',(${'main::'.$client.'_VI_ADDR1'} || ""));
   client_param_copy('_VI_ADDR',$client,$client2);
   param($client.'_VI_DEV','-default',(${'main::'.$client.'_VI_DEV1'} || ""));
   client_param_copy('_VI_DEV',$client,$client2);
   param($client.'_VI_NAME','-default',(${'main::'.$client.'_VI_NAME1'} || ""));
   client_param_copy('_VI_NAME',$client,$client2);
   
   # IB Interfaces to the Filer
   my $ib_num = 1;
   while ($ib_num <= 4) {
      param($client.'_IB_ADDR'.$ib_num,'-default',"");
      if (defined(${'main::'.$client.'_IB_ADDR'.$ib_num}) and ${'main::'.$client.'_IB_ADDR'.$ib_num} ne '') {
          my $ib_dev = $ib_num - 1;
          param($client.'_IB_DEV'.$ib_num,'-default',"");
          if (${'main::'.$client.'_IB_DEV'.$ib_num} eq '') {
              param($client.'_IB_DEV'.$ib_num,'-set',"ib$ib_dev");
          }
          client_param_copy('_IB_DEV'.$ib_num,$client,$client2);
          param($client.'_IB_NAME'.$ib_num,'-default',"");
          client_param_copy('_IB_NAME'.$ib_num,$client,$client2);

          client_param_copy('_IB_ADDR'.$ib_num,$client,$client2);
      } else {
          client_param_copy('_IB_ADDR'.$ib_num,$client,$client2);
          last;
      }
      $ib_num++;
   }
   # Define default IB values
   param($client.'_IB_ADDR','-default',(${'main::'.$client.'_IB_ADDR1'} || ""));
   client_param_copy('_IB_ADDR',$client,$client2);
   param($client.'_IB_DEV','-default',(${'main::'.$client.'_IB_DEV1'} || ""));
   client_param_copy('_IB_DEV',$client,$client2);
   param($client.'_IB_NAME','-default',(${'main::'.$client.'_IB_NAME1'} || ""));
   client_param_copy('_IB_NAME',$client,$client2);
   
   # DAFS Interfaces to the Filer
   my $dafs_num = 1;
   while ($dafs_num <= 4) {
      param($client.'_DAFS_ADDR'.$dafs_num,
         '-default',(${'main::'.$client.'_VI_ADDR'.$dafs_num} || ${'main::'.$client.'_IB_ADDR'.$dafs_num} || ""));
      client_param_copy('_VI_ADDR'.$dafs_num,$client,$client2);
      my $dafs_dev = $dafs_num - 1;
      param($client.'_DAFS_DEV'.$dafs_num,
         '-default',(${'main::'.$client.'_VI_DEV'.$dafs_num} || ${'main::'.$client.'_IB_DEV'.$dafs_num} || ""));
      client_param_copy('_DAFS_DEV'.$dafs_num,$client,$client2);
      param($client.'_DAFS_NAME'.$dafs_num,
         '-default',(${'main::'.$client.'_VI_NAME'.$dafs_num} || ${'main::'.$client.'_IB_NAME'.$dafs_num} || ""));
      client_param_copy('_DAFS_NAME'.$dafs_num,$client,$client2);
      $dafs_num++;
   }
   # Define default DAFS values
   param($client.'_DAFS_DEV','-default',(
      ${'main::'.$client.'_DAFS_DEV1'} || ${'main::'.$client.'_VI_DEV'} || ${'main::'.$client.'_IB_DEV'} || ""));
   client_param_copy('_DAFS_DEV',$client,$client2);

   param($client.'_DAFS_ADDR', '-default',(
      ${'main::'.$client.'_DAFS_ADDR1'} || ${'main::'.$client.'_VI_ADDR'} || ${'main::'.$client.'_IB_ADDR'} ||""));
   client_param_copy('_DAFS_ADDR',$client,$client2);

   param($client.'_DAFS_NAME', '-default',(
      ${'main::'.$client.'_DAFS_NAME1'} || ${'main::'.$client.'_VI_NAME'} || ${'main::'.$client.'_IB_NAME'} || ""));
   client_param_copy('_DAFS_NAME',$client,$client2);
   
   # Get generic client default values if they have been defined
   param('CLIENT_IO_DEV_OFFSET_BEGIN','-default',undef);
   param('CLIENT_IO_DEV_OFFSET_END','-default',undef);
   param('CLIENT_IO_DEV_SIZE','-default',undef);
   param('CLIENT_IO_PASS_THRU','-default',undef);

   param('CLIENT_IO_READ_PCT','-default',undef);
   param('CLIENT_IO_RANDOM_PCT','-default',undef);
   param('CLIENT_IO_NUM_THREADS','-default',undef);
   param('CLIENT_PARENT_LUN_PATHS','-default',undef);
   param('CLIENT_PARENT_LUN_IDS','-default',undef);

   param('CLIENT_IO_DEV_BLOCK_SIZE','-default',undef);
   param('CLIENT_IO_BLOCK_SIZE','-default',$main::CLIENT_IO_DEV_BLOCK_SIZE);
   my $client_io_block_size = $main::CLIENT_IO_BLOCK_SIZE;

   # I/O Device Interfaces associated with this Client
   param($client.'_IO_DEV_NUM','-default',undef);
   my $io_num = 1;
   while (1) {
      param($client.'_IO_DEV_'.$io_num,'-default',undef);
      if (!defined(${'main::'.$client.'_IO_DEV_'.$io_num})) {
         $io_num--;  # Go back to the correct io device count
         last;
      }
      client_param_copy('_IO_DEV_'.$io_num,$client,$client2);

      # Get default values if they have been defined
      param($client.'_IO_DEV_OFFSET_BEGIN','-default',undef);
      param($client.'_IO_DEV_OFFSET_END','-default',undef);
      param($client.'_IO_DEV_SIZE','-default',undef);
      param($client.'_IO_PASS_THRU','-default',undef);

      param($client.'_IO_NUM_THREADS','-default',undef);
      param($client.'_PARENT_LUN_PATHS','-default',undef);
      param($client.'_PARENT_LUN_IDS','-default',undef);
      param($client.'_IO_READ_PCT','-default',undef);
      param($client.'_IO_RANDOM_PCT','-default',undef);

      param($client.'_IO_DEV_BLOCK_SIZE','-default',$client_io_block_size);
      param($client.'_IO_BLOCK_SIZE','-default',${'main::'.$client.'_IO_DEV_BLOCK_SIZE'});
      
      # Now handle related I/O device parameters
      # Take value from CLIENTn first (This supports values in the tharnhosts filer.), 
      # if not defined, look for CLIENT_ parameter (This supports generic overrides)
      param($client.'_IO_DEV_'.$io_num.'_OFFSET_BEGIN','-default',
         (${'main::'.$client.'_IO_DEV_OFFSET_BEGIN'}||${'main::CLIENT_IO_DEV_OFFSET_BEGIN'}||"0"));
      client_param_copy('_IO_DEV_'.$io_num.'_OFFSET_BEGIN',$client,$client2);

      param($client.'_IO_DEV_'.$io_num.'_OFFSET_END','-default',
         (${'main::'.$client.'_IO_DEV_OFFSET_END'}||${'main::CLIENT_IO_DEV_OFFSET_END'}||"4m"));
      client_param_copy('_IO_DEV_'.$io_num.'_OFFSET_END',$client,$client2);

      param($client.'_IO_DEV_'.$io_num.'_SIZE','-default',
         (${'main::'.$client.'_IO_DEV_SIZE'}||${'main::CLIENT_IO_DEV_SIZE'}||""));
      client_param_copy('_IO_DEV_'.$io_num.'_SIZE',$client,$client2);

      param($client.'_IO_DEV_'.$io_num.'_BLOCK_SIZE','-default',
         (${'main::'.$client.'_IO_BLOCK_SIZE'}||${'main::CLIENT_IO_DEV_BLOCK_SIZE'}||${'main::CLIENT_IO_BLOCK_SIZE'}||"512"));
      client_param_copy('_IO_DEV_'.$io_num.'_BLOCK_SIZE',$client,$client2);

      if (defined(${'main::'.$client.'_IO_DEV_NUM'}) and ${'main::'.$client.'_IO_DEV_NUM'} ne '') {
         if (${'main::'.$client.'_IO_DEV_NUM'} <= $io_num ) {
            # User wants to limit the number of defined devices that are used
            last;
         }
      }
      $io_num++;
   }
   # If user did not limit the number of IO devices, set the limit to the number that are defined
   if (!defined(${'main::'.$client.'_IO_DEV_NUM'}) or ${'main::'.$client.'_IO_DEV_NUM'} eq '') {
      param($client.'_IO_DEV_NUM','-set',"$io_num");
      print("Setting $client"."_IO_DEV_NUM=$io_num\n");
   }
   client_param_copy('_IO_DEV_NUM',$client,$client2);

   #  Establish the non-device numbered values here for this client
   #  using the values from the first device if it was specified
   if (defined( ${'main::'.$client.'_IO_DEV_1'}) and ${'main::'.$client.'_IO_DEV_1'} ne '') {
      my %io_dev_defaults = (
         '_IO_DEV' => '_IO_DEV_1',
         '_IO_BLOCK_SIZE' => '_IO_DEV_1_BLOCK_SIZE',
         '_IO_DEV_OFFSET_BEGIN' => '_IO_DEV_1_OFFSET_BEGIN',
         '_IO_DEV_OFFSET_END' => '_IO_DEV_1_OFFSET_END',
         );

      foreach my $key (keys(%io_dev_defaults)) {
         param($client.$key,'-default',(""));
         if (${'main::'.$client.$key} eq '') {
            if (defined(${'main::'.$client.$io_dev_defaults{$key}}) and ${'main::'.$client.$io_dev_defaults{$key}} ne '') {
               param($client.$key,'-set',${'main::'.$client.$io_dev_defaults{$key}});
               client_param_copy($key,$client,$client2);
            }
         }
      }
   }

   param($client.'_FCP_IO_DEV','-default',undef);
   param($client.'_ISCSI_IO_DEV','-default',undef);
   # Define default client I/O device
   param($client.'_IO_DEV', '-default',undef);

   # Parameters for a client with FCP installed
   param($client.'_FCP_WWNN','-default',undef);
   if ((defined(${'main::'.$client.'_FCP_WWNN'}) and ${'main::'.$client.'_FCP_WWNN'} ne '') 
   ||(defined(${'main::'.$client.'_FCP_IO_DEV'}) and ${'main::'.$client.'_FCP_IO_DEV'} ne '')) {
      $fcp_installed_default = "yes";
   }
   client_param_copy('_FCP_WWNN',$client,$client2);
   client_param_copy('_FCP_IO_DEV',$client,$client2);

   param($client.'_FCP_INSTALLED','-default',$fcp_installed_default);
   if (defined(${'main::'.$client.'_FCP_INSTALLED'}) and ${'main::'.$client.'_FCP_INSTALLED'} =~ m/yes/i) {
      # Update list of all FCP installed clients
      if ($client !~ m/^NEW/ and 0 == grep (/^$client$/, @FCP_CLIENTS) ) {
         push (@FCP_CLIENTS,$client);
      }
      # Now pick the best FCP CLIENT - one with at least one device

      # Assume the current FCP client if a better one does not appear
      param('CLIENT_FCP','-default',${'main::'.$client});
      param('CLIENT_FCP_IO_DEV','-default','');   # The user may specify this, or it will be set later

      # If the latest client has an FCP device, ensure we pick it
      if (defined(${'main::'.$client.'_IO_DEV'}) and ${'main::'.$client.'_IO_DEV'} ne '' and $client !~ m/^NEW/) {
         # See if we already have an FCP device
         # if not, pass the client's IO_DEV definition as the FCP_IO_DEV value
         if ($CLIENT_FCP_IO_DEV eq '') {
            param('CLIENT_FCP_IO_DEV','-set',${'main::'.$client.'_IO_DEV'});
            # Replace the first FCP client pick. This one is better
            param('CLIENT_FCP','-set',${'main::'.$client});
         }
      }
      # Make sure we keep the right IP address to go with the picked client
      if ($CLIENT_FCP eq ${'main::'.$client}) {
         param('CLIENT_FCP_IP','-set',${'main::'.$client.'_IP'});
      }
   }
   client_param_copy('_FCP_INSTALLED',$client,$client2);
   
   # Parameters for a client with iSCSI installed
   param($client.'_ISCSI_WWNN','-default',undef);
   if ((defined(${'main::'.$client.'_ISCSI_WWNN'}) and ${'main::'.$client.'_ISCSI_WWNN'} ne '') 
   || (defined(${'main::'.$client.'_ISCSI_IO_DEV'}) and ${'main::'.$client.'_ISCSI_IO_DEV'} ne '')) {
      $iscsi_installed_default = "yes";
   }
   client_param_copy('_ISCSI_WWNN',$client,$client2);
   client_param_copy('_ISCSI_IO_DEV',$client,$client2);

   print("$client"."_ISCSI_INSTALLED default=$iscsi_installed_default\n");
   param($client.'_ISCSI_INSTALLED','-default',$iscsi_installed_default);
   if (defined(${'main::'.$client.'_ISCSI_INSTALLED'}) and ${'main::'.$client.'_ISCSI_INSTALLED'} =~ m/yes/i) {
      if ($client !~ m/^NEW/ and 0 == grep (/^$client$/, @ISCSI_CLIENTS) ) {
         push (@ISCSI_CLIENTS,$client);
      }
      # Now pick the best iSCSI CLIENT - one with at least one device

      # Assume the current iSCSI client if a better one does not appear
      param('CLIENT_ISCSI','-default',${'main::'.$client});

      # If the latest client has an iSCSI device, ensure we pick it
      if (defined(${'main::'.$client.'_IO_DEV'}) and ${'main::'.$client.'_IO_DEV'} ne '') {
         # See if we already have an iSCSI device
         # Pass the client's IO_DEV definition as the ISCSI_IO_DEV value
         param('CLIENT_ISCSI_IO_DEV','-default','');
         if ($client !~ m/^NEW/ and $main::CLIENT_ISCSI_IO_DEV eq '' and (defined(${'main::'.$client.'_IP'}) and ${'main::'.$client.'_IP'} ne '')) {
            param('CLIENT_ISCSI_IO_DEV','-set',${'main::'.$client.'_IO_DEV'});
            # Replace the first iSCSI client pick. This one is better
            param('CLIENT_ISCSI','-set',${'main::'.$client});
         }
      }
      # Make sure we keep the right IP address to go with the picked client
      if ($main::CLIENT_ISCSI eq ${'main::'.$client} and (defined(${'main::'.$client.'_IP'}) and ${'main::'.$client.'_IP'} ne '')) {
          print("Setting CLIENT_ISCSI_IP=${'main::'.$client.'_IP'} from $client\n");
          param('CLIENT_ISCSI_IP','-set',(${'main::'.$client.'_IP'} ));
      }
   }
   client_param_copy('_ISCSI_INSTALLED',$client,$client2);
   
   # Parameters for a client with either FCP or iSCSI installed
   client_param_copy('_IO_DEV',$client,$client2);
   if ((defined(${'main::'.$client.'_FCP_INSTALLED'}) and ${'main::'.$client.'_FCP_INSTALLED'} =~ /yes/i )
   || (defined(${'main::'.$client.'_ISCSI_INSTALLED'}) and ${'main::'.$client.'_ISCSI_INSTALLED'} =~ /yes/i)) {
      # Define default Initiator HBA values
      my $lun = 1;
      while (1) {
         param($client.'_INIT_LUN'.$lun,'-default',(${'main::'.$client.'_INIT1_LUN'.$lun} || ""));
         if (${'main::'.$client.'_INIT_LUN'.$lun} eq '') {
            last;
         }
         client_param_copy('_INIT_LUN'.$lun,$client,$client2);
         $lun++;
      }

      param($client.'_INIT_LUN','-default',(${'main::'.$client.'_INIT_LUN1'} || ""));
      client_param_copy('_INIT_LUN',$client,$client2);
      param($client.'_INIT_PROTOCOL','-default',(""));
      client_param_copy('_INIT_PROTOCOL',$client,$client2);
      param($client.'_INIT_WWNN','-default',(""));
      client_param_copy('_INIT_WWNN',$client,$client2);
      param($client.'_INIT_WWPN','-default',(""));
      client_param_copy('_INIT_WWPN',$client,$client2);

      # Define default PTI Initiator and Target HBA values

      param($client.'_PTI_TARGET1_ID','-default',(undef));
      client_param_copy('_PTI_TARGET1_ID',$client,$client2);
      if (defined(${'main::'.$client.'_PTI_TARGET1_ID'}) and ${'main::'.$client.'_PTI_TARGET1_ID'} ne '') {
         param($client.'_PTI_TARGET2_ID','-default',(""));
         client_param_copy('_PTI_TARGET2_ID',$client,$client2);
         param($client.'_PTI_TARGET3_ID','-default',(""));
         client_param_copy('_PTI_TARGET3_ID',$client,$client2);
         param($client.'_PTI_TARGET4_ID','-default',(""));
         client_param_copy('_PTI_TARGET4_ID',$client,$client2);
      }
      param($client.'_PTI_TARGET_ID','-default',(${'main::'.$client.'_PTI_TARGET1_ID'}));
      client_param_copy('_PTI_TARGET_ID',$client,$client2);

      if ((defined(${'main::'.$client.'_PTI_TARGET_ID'}) and ${'main::'.$client.'_PTI_TARGET_ID'} ne '') 
      or $client_configs =~ m/,pti,/) {
          # CLIENT and CLIENT1 are the same,
          # but CLIENT1 is not always defined
          # Remember each PTI client just once
          if ($client ne 'CLIENT1') {
              push (@PTI_CLIENTS,$client);
          }
          # Look for other PTI param values only if we got a TARGET_ID
          param($client.'_PTI_INIT1_ID',  '-default',(""));
          client_param_copy('_PTI_INIT1_ID',$client,$client2);
          if (defined(${'main::'.$client.'_PTI_INIT1_ID'}) and ${'main::'.$client.'_PTI_INIT1_ID'} ne '') {
              param($client.'_PTI_INIT2_ID',  '-default',(""));
              client_param_copy('_PTI_INIT2_ID',$client,$client2);
              param($client.'_PTI_INIT3_ID',  '-default',(""));
              client_param_copy('_PTI_INIT3_ID',$client,$client2);
              param($client.'_PTI_INIT4_ID',  '-default',(""));
              client_param_copy('_PTI_INIT4_ID',$client,$client2);
          }
          param($client.'_PTI_INIT_ID',  '-default',(${'main::'.$client.'_PTI_INIT1_ID'}));
          client_param_copy('_PTI_INIT_ID',$client,$client2);
      }
   }

   # Parameters for a client with VI (for DAFS) installed
   my $vi_installed_default = "";
   if (defined(${'main::'.$client.'_VI_ADDR1'}) and ${'main::'.$client.'_VI_ADDR1'} ne '' ) {
      $vi_installed_default = "yes";
   }
   param($client.'_VI_INSTALLED','-default',$vi_installed_default);
   if (defined(${'main::'.$client.'_VI_INSTALLED'}) and ${'main::'.$client.'_VI_INSTALLED'} =~ /yes/i) {
      if ($client ne 'CLIENT') {
         push (@VI_CLIENTS,$client);
      }
      param('CLIENT_VI','-default',${'main::'.$client});
      param('CLIENT_VI_IP','-default',${'main::'.$client.'_IP'});
   }
   client_param_copy('_VI_INSTALLED',$client,$client2);

   # Parameters for a client with IB (for DAFS) installed
   my $ib_installed_default = "";
   if (defined(${'main::'.$client.'_IB_ADDR1'}) and ${'main::'.$client.'_IB_ADDR1'} ne '' ) {
      $ib_installed_default = "yes";
   }
   param($client.'_IB_INSTALLED','-default',$ib_installed_default);
   if (defined(${'main::'.$client.'_IB_INSTALLED'}) and ${'main::'.$client.'_IB_INSTALLED'} =~ /yes/i) {
      if ($client ne 'CLIENT') {
         push (@IB_CLIENTS,$client);
      }
      param('CLIENT_IB','-default',${'main::'.$client});
      param('CLIENT_IB_IP','-default',${'main::'.$client.'_IP'});
   }
   client_param_copy('_IB_INSTALLED',$client,$client2);
   push (@ALL_CLIENTS,$client);

    # Be sure to set the new 'CLIENT' and 'CLIENTn' parameters, too
   if (defined($client2) and $client2 =~ m/^CLIENT/) {
       print("Setting $client2=${'main::'.$client}\n"); #TRACE
       param($client2,'-set', "${'main::'.$client}");
   }
 
    use strict 'refs';

}

################################################################################
# Subroutine Name: _configure_client
# Objective:       This routine is responsible for starting a 
#                  connection to the specified host, create a host object
#                  representing that host, create filessystems that represent
#                  information passed in CLIENT_IO_DEVS
################################################################################
sub _configure_client() {
    my ($client, $multipath, $use_fs, $fs_type) = @_;

    &client_device_setup( client => $client,
        skip_setup => $main::SKIP_SETUP,
        devices => $CLIENT_IO_DEVS,
        fs_type => $fs_type,
        multipath => $multipath,
        next_io_dev => 1,
        use_fs => $use_fs,
        make_fs => 1,
        filers => $FILERS);
    
}

## @name ::client_device_setup
## @summary Establishes all the client param values associated with the requested 'CLIENTn'
## @description
## @arg %args Required: hash of parameters:
## conn Required: a client connection object
## devices Required colon separated list of LUN numbers
## filers  Required colon separated list of filers that each have the specified LUN numbers on the client
## multipath (defaults to 0) =1 if multiple paths are to be used to the client
## next_io_dev (defaults to 0) defines the starting CLIENT_IO_DEV number to be used
## (a value less than 1 causes no parameter setting to occur)
## skip_setup (defaults to 1) =0 if devices are to be set up on the client
## fs_type (defaults to '') the filesystem expected or to be installed during setup
## make_fs (defaults to 0) non zero values request the filesystem to be installed during setup
## 0 skips partitioning on Windows and skips filesystem installation
## use_fs (defaults to 0) non zero values request the filesystem to be mounted during setup
## an empty value requests the platform default filesystem
sub client_device_setup($%) {
    my %args = @_;

    my (@output, $status, $cmd, $diskobj, $fsobj);
    my ($filesys, $device, @devices);
    my ($hostobj, $layout);
    my $tracemsg = '';
    my @this_request_fsobj_list = (); # holds all file system objects created by this request

    my @required_parms = qw (devices filers);
    my @valid_parms = (qw (client conn fs_type make_fs multipath next_io_dev skip_setup use_fs) , @required_parms); 

    # Verify input parameters
    foreach my $parm (@required_parms) {
       if (!defined($args{$parm})) {
           ExitScriptError("A '$parm' parameter must be specified");
       }
       if (not length($args{$parm})) {
           ExitScriptError("Invalid value for $parm: '$args{$_}'");
       }
    }
    # Verify there are no invalid input parameters 
    foreach my $parm (keys %args) {
        if (0 == grep(/^$parm$/,@valid_parms)) {
            ExitScriptError("The '$parm' parameter is not recognized");
        }
    }
    # Establish default parameter values
    my $multipath = 0;
    if (defined($args{'multipath'})) {
        $multipath = $args{'multipath'};
        $tracemsg .= " multipath=$multipath";
    }
    my $skip_setup = 1;
    if (defined($args{'skip_setup'})) {
        $skip_setup = $args{'skip_setup'};
        $tracemsg .= " skip_setup=$skip_setup";
    }
    my $next_io_dev = 0;
    if (defined($args{'next_io_dev'})) {
        $next_io_dev = $args{'next_io_dev'};
        $tracemsg .= " next_io_dev=$next_io_dev";
    }
    my $make_fs = 1;
    if (defined($args{'make_fs'})) {
        $make_fs = $args{'make_fs'};
        $tracemsg .= " make_fs=$make_fs";
    }
    my $use_fs = 0;
    if (defined($args{'use_fs'})) {
        $use_fs = $args{'use_fs'};
        $tracemsg .= " use_fs=$use_fs";
    }
    my $fs_type = '';
    if (defined($args{'fs_type'})) {
        $fs_type = $args{'fs_type'};
        $tracemsg .= " fs_type=$fs_type";
    }

    if ($skip_setup ne '0' and $skip_setup !~ m/^n/i) {
        print("Skipping host setup\n");
    } else {
        my $client;
        my $client_ip;
        my $client_connection;
        if (defined($args{'conn'})) {
            $client_connection = $args{'conn'};
            $tracemsg .= " conn=$client_connection";
            $client = $args{'conn'}->hostp()->hostname();
            $client_ip = $args{'conn'}->hostp()->default_ip();
        } else {
            if (defined($args{'client'})) {
                $client = $args{'client'};
                $tracemsg .= " client=$client";
                my $hostobj = host($client,'-ok');
                if (defined($hostobj)) {
                    my $client_conn;
                    if ($hostobj->{hosttype} =~ m/windows/) {
                        # First ensure client is alive
                        $client_ip = $hostobj->default_ip();
                        if (host_wait($client_ip)) {
                            # Do not use rsh-socket from UNIX (root is required, NATE prevents its use)
                            $client_conn =  "#rsh-persay";    
                            if (!TestEnv::MinWin::minwin_compatible($client)) {
                                # Give it a second chance
                                print("MinWin compatibility failed for $client at $client_ip, trying again\n"); #DEBUG
                                sleep(30);
                                if (!TestEnv::MinWin::minwin_compatible($client)) {
                                    # each client must be MinWin Compatible, we cannot continue
                                    ExitConfigError("All Windows clients must be MinWin compatible. $client is not");
                                }
                            }
                        } else {
                            ExitConfigError("Client $client at $client_ip does not appear to be alive");
                        }
                    } else {
                        $client_conn =  "#telnet;inherit=0";  # rsh-persay may hang Solaris
                    }
                    $client_connection = connect($client.$client_conn);
                } else {
                    ExitScriptError("CLIENT '$client' does not exist or does not have a hosttype");
                }
            } else {
                ExitScriptError("Either a 'client' or 'conn' parameter is required");
            }
        }

        my $client_prefix = &client_param_get($client,0); # let CLIENT take precedence over any CLIENTn
        my $client_prefix2 = &client_param_get($client,1); # get CLIENTn also
        print("Beginning to configure client $client_prefix=$client\n"); #DEBUG

        my $devices = $args{'devices'};
        $tracemsg .= " devices=$devices";
        # Drop any extra arguments that this function does not support
        # (Some input may have a LUN size and LUN ostype here)
        $devices =~ s/,.*?:/:/g;  # ? keeps it from being greedy and taking out the middle data

        if ($devices =~ m/-/) {
            my @device_numbers;
            my @device_list = split(/:/,$devices);
            foreach my $device_list_entry (@device_list) {
                if ($device_list_entry =~ m/^(\d+)-(\d+)$/) {
                    my ($start,$end) = ($1,$2);
                    for (my $dev_num = $start; $dev_num<=$end; $dev_num++) {
                        push(@device_numbers,$dev_num);
                    }
                } else {
                    if ($device_list_entry !~ m/$(\d+)$/) {
                        ExitScriptError("Device list must have numeric entries, not: $device_list_entry");
                    }
                    push(@device_numbers,$device_list_entry);
                }
            }
            # Replace the old device list containing ranges with the new colon separated list
            $devices = join(':',@device_numbers);
         }
         if ($devices =~ m/^\s*$/) {
             print("devices='$devices' so no volume manager or lun discovery work is needed\n");
         } else {
             $tracemsg .= " filers=$args{'filers'}";
             my @filers = split(':',$args{'filers'});
             my @filer_wwnns;
             foreach my $filer (@filers) {
                 my $fh = host($filer);
                 my $fc = connect ("$filer#rsh-socket");
                 print("Checking WWNN for filer: $filer\n"); #DEBUG
                 my $next_wwnn = filer_fcp_nodename_get($fc);
                 if (defined($next_wwnn) and $next_wwnn ne '') {
                     $next_wwnn =~ s/://g;  #drop optional colon separators 
                     push(@filer_wwnns,$next_wwnn)
                 }
                 $next_wwnn = filer_iscsi_nodename_get($fc);
                 if (defined($next_wwnn) and $next_wwnn ne '') {
                     push(@filer_wwnns,$next_wwnn)
                 }
             }
             print("Got WWNNs: ".join(' ',@filer_wwnns)."\n"); #DEBUG
             
             print("client_device_setup($tracemsg)\n");
             # Create a new Host object, passing the client connection
             # and list of devices LUNs you want to use (i.e 0:1:2)
             $hostobj = SanClient->new(-connection => $client_connection,
                                       -devices    => $devices);

             # Some have reported that the rescan operation (on Windows) 
             # may not be reliable. The following loop supports retry
             my $retry = 1;
             my $max_retry = 3;
             while ($retry) {
                 $retry = 0;

                 # Discover resources on the host. This includes configuration
                 # information (configuration files) and sanlun information.
                 # It will also run devfsadm to discover LUNs
                 $hostobj->discover();

                 # Get the list of device pathnames to use
                 # on disk and filesystem operations. This is 
                 # extracted from the object which got this information
                 # from sanlun during the discover method.
                 @devices = $hostobj->get_devices(-multipath => $multipath,
                                                  -filer_wwnns => join(' ',@filer_wwnns),
                                                    );
                 if (@devices) {
                     &logcomment("List of all requested devices on $client includes:",
                                 @devices);
                 } else {
                     if ($hostobj->{hosttype} =~ m/windows/) {
                         if ($max_retry > 0) {
                             $max_retry--;
                             $retry = 1;
                             # When a client had a LUN at this ID,
                             # it takes an extra rescan pass to drop the old one
                             # before finding the new one
                             &logcomment("Retrying rescan, none of the requested devices ($devices) are available to use on client $client");
                             sleep(20);  # Perhaps giving it some time will help before rescaning?
                         } else {
                             &logcomment("Please verify that $args{'filers'} are listed as available targets on client: $client");
                             &ExitError("None of the requested devices ($args{'devices'}) are available to use on Windows client $client");
                         }
                     } else {
                         &ExitError("None of the requested devices ($args{'devices'}) are available to use on client $client");
                     }
                 }
             }


             # create the objects representing the passed devices

             &logcomment("Starting host setup on $client");

             # These scripts do not support logical volumes with MPIO on Windows
             if ($multipath and $hostobj->{hosttype} !~ m/windows/) {
                 $diskobj = Disk->new(-hobj   => $hostobj,
                                      -filers => $args{'filers'});
                 $diskobj->format_vm(-devices   => \@devices,
                                     -multipath => $multipath);

                 sleep(60);

                 my $client_vm_vol_size = $main::CLIENT_VM_VOL_SIZE;
                 $client_vm_vol_size =~ s/['"]//g; # quotes have caused problems here
                 # We should be able to calculate a default value based on the average LUN size
                 # minus a margin of error
                 my $total_bytes = 0;
                 if ($client_vm_vol_size eq '') {
                     foreach my $device (@devices) {
                         # Get the LUN size from the filer or the client

                     }
                     $client_vm_vol_size = $total_bytes/$main::CLIENT_VM_NUM_LVS;
                     # Veritas minimum is 8K, although 2M overhead is assumed
                     # But filer minimum is 4M
                     my $min_vol_size = 4*1024*1024;
                     if ($client_vm_vol_size < $min_vol_size) {
                         print("Setting minimum CLIENT_VM_VOL_SIZE: $min_vol_size\n");
                         $client_vm_vol_size = $min_vol_size;
                     }
                 }

                 # create N logical volumes from the created Disk Group.
                 # N is specified in the variable CLIENT_VM_NUM_LVS
                 &logcomment("Creating $main::CLIENT_VM_NUM_LVS logical volumes");
                 foreach $filesys (1 .. $main::CLIENT_VM_NUM_LVS) {
                     # create a filesystem from the passed disk object
                     $fsobj = FS->new(-hobj   => $hostobj,
                                      -layout => $CLIENT_VM_LAYOUT,
                                      -dobj   => $diskobj);

                     # AIX limits volume names to 15 characters
                     # Names must be unique, even across all volume groups
                     # Let create_lv pick a volume_name
                     &logcomment("Creating Logical $filesys with parameters:",
                                 "LAYOUT: $CLIENT_VM_LAYOUT",
                                 "VOLUME_SIZE: $client_vm_vol_size",
                                 "STRIPE_SIZE: $CLIENT_VM_STRIPE_SIZE");

                     # create the logical volume. It can be:
                     #  stripe | mirror | concat | raid5 (for Veritas)
                     #  stripe | concat (for HP-UX or AIX)
                     $fsobj->create_lv(-layout       => $main::CLIENT_VM_LAYOUT,
                                       -stripe_width => $main::CLIENT_VM_STRIPE_SIZE,
                                       -volume_size  => $client_vm_vol_size);

                     push (@FSOBJ_LIST, $fsobj);
                     push (@this_request_fsobj_list, $fsobj);
                }

             } else {
                 # no multipathing or Windows host
                 # Each device is to be a separate volume - not part of a multiple device volume group
                 my $dev_list = join(', ',@devices);
                 print("client_device_setup: devices without logical volumes: $dev_list\n");
                 foreach $device (@devices) {
                     print("client_device_setup: device: '$device'\n"); #DEBUG

                     if ($device !~ m/^\s*$/) {
                         # create a disk object representing the passed device
                         $diskobj = Disk->new(-hobj => $hostobj,
                                          -filers => $args{'filers'});

                         # Format and label the disk
                         # Pass a partition_map or disk_size parameter if auto-format is not wanted
                         if ($fs_type =~ m/raw/i or !$make_fs ) {
                             $diskobj-> format(-device => $device,
                                -no_format => 1);   # Skip format or disk partition creation
                         } else {
                             $diskobj-> format(-device => $device);
                         }

                         # Create a filesystem from the passed disk object
                         $fsobj = FS->new(-hobj => $hostobj,
                                          -dobj => $diskobj);
                         push (@FSOBJ_LIST, $fsobj);
                         push (@this_request_fsobj_list, $fsobj);
                     }
                 }
             }

             if ($fs_type !~ m/^raw$/i) {
                 # create and mount the filesystem (if specified)
                 if ($use_fs or $make_fs) {
                     foreach $fsobj (@this_request_fsobj_list) {
                         if ($fs_type ne '') {
                             $fsobj->make(-type => $fs_type);
                         } else {
                             $fsobj->make();
                         }
                         # mount the filesystem on the host
                         $fsobj->mount();
                     }
                 }
             }

             if ($next_io_dev > 0) {
                 # Pass new and modified parameters and their values to scripts that follow
                 my $io_dev_num = $next_io_dev -1;
                 foreach $fsobj (@this_request_fsobj_list) {
                     $io_dev_num++;
                     # Create parameters to describe the devices (rawdisk does not work with sio on HPUX?)
                     no strict 'refs';
                     if (defined($fsobj->{drive_letter})) {
                         # Handle Windows drives
                         print("client_device_setup: Setting ${client_prefix}_IO_DEV_${io_dev_num}=$fsobj->{drive_letter}\n");
                         param($client_prefix."_IO_DEV_".$io_dev_num, -set, $fsobj->{drive_letter});
                         if ($client_prefix ne $client_prefix2) {
                             param($client_prefix2."_IO_DEV_".$io_dev_num, -set, $fsobj->{drive_letter});
                         }
                     } elsif (defined($fsobj->{rawdisk})) {
                         print("client_device_setup: Setting ${client_prefix}_IO_DEV_${io_dev_num}=$fsobj->{rawdisk}\n");
                         param($client_prefix."_IO_DEV_".$io_dev_num, -set, $fsobj->{rawdisk});
                         if ($client_prefix ne $client_prefix2) {
                             param($client_prefix2."_IO_DEV_".$io_dev_num, -set, $fsobj->{rawdisk});
                         }
                     } else {
                         print("client_device_setup: Neither a rawdisk nor a drive letter were defined?\n");
                     }
                     if (defined($fsobj->{size}) ) {
                         print("Setting ${client_prefix}_IO_DEV_${io_dev_num}_SIZE=$fsobj->{size}\n");
                         param($client_prefix."_IO_DEV_".$io_dev_num."_SIZE", -set, $fsobj->{size});
                         if ($client_prefix ne $client_prefix2) {
                             param($client_prefix2."_IO_DEV_".$io_dev_num."_SIZE", -set, $fsobj->{size});
                         }
                     } else {
                         print("No definition of the file system size was made?\n"); #DEBUG
                     }
                     use strict 'refs';
                 }
                 if (0 == @this_request_fsobj_list) {
                     print("No  file system objects were defined?\n"); #DEBUG
                 }
                 # Override the CLIENTn_IO_DEV_NUM value that may have been defined in this client's host file
                 # We just completed setup for a different value
                 if ($client_prefix ne 'CLIENT' or !defined($nate_args{'CLIENT_IO_DEV_NUM'})) {
                     print("client_device_setup: Setting ${client_prefix}_IO_DEV_NUM=$io_dev_num based on the device count\n"); #DEBUG
                     param($client_prefix."_IO_DEV_NUM", -set, $io_dev_num);
                 } else {
                     if (defined($nate_args{'CLIENT_IO_DEV_NUM'})) {
                         print("client_device_setup: Not changing ${client_prefix}_IO_DEV_NUM to $io_dev_num since CLIENT_IO_DEV_NUM was specified\n"); #DEBUG
                     } else {
                         print("client_device_setup: Not changing ${client_prefix}_IO_DEV_NUM to $io_dev_num (we only override CLIENT values)\n"); #DEBUG
                     }
                 }
                 if ($client_prefix ne $client_prefix2) {
                     # But only if this value will not change what was on the command line
                     if ($client_prefix2 ne 'CLIENT' or !defined($nate_args{'CLIENT_IO_DEV_NUM'})) {
                         print("client_device_setup: Setting ${client_prefix2}_IO_DEV_NUM=$io_dev_num based on the device count\n"); #DEBUG
                         param($client_prefix2."_IO_DEV_NUM", -set, $io_dev_num);
                     }
                 }
            } else {
                print("client_device_setup: next_io_dev=$next_io_dev so no additional client parameters were established\n");
            }
        }
    }
}

# Global variables defined here
@CLIENTS_FCP_INIT_NAMES = ();
@CLIENTS_ISCSI_INIT_NAMES = ();

@ALL_CLIENTS = ();
@FCP_CLIENTS = ();
@ISCSI_CLIENTS = ();
@DAFS_CLIENTS = ();
@VI_CLIENTS = ();
@IB_CLIENTS = ();
@PTI_CLIENTS = ();
@AIX_CLIENTS = ();
@HPUX_CLIENTS = ();
@LINUX_CLIENTS = ();
@SOLARIS_CLIENTS = ();
@UNIX_CLIENTS = ();
@WIN_CLIENTS = ();
@SETUP_CLIENT_NAMES = ();  # All hostnames from the CLIENTS parameter
@FSOBJ_LIST = ();

param('PROJECT','-default', "");
my $project = $main::PROJECT;
if (not defined($project) or $project eq '') {
    $project = 'test';
}
param('CLIENT','-default', "");
param('CLIENTS','-default', "");
param('LUN_LIST','-default', "");
# Skip reassignment if we can
if ($LUN_LIST ne '' or ($CLIENTS ne '' and $CLIENTS ne $CLIENT) ) {
    param('FILER','-mesg', "Name of the filer");
    param('FILERS','-default', "$main::FILER");
    param('MULTIPATH','-default', "0");
    param('USE_FS','-default', "0");
    param('SKIP_SETUP','-default', "0");
    param('CLIENT_IO_FS_TYPE','-default', "");
    param('CLIENT_VM_VOL_SIZE','-default', "");
    param('CLIENT_VM_LAYOUT','-default', "nostripe");
    param('CLIENT_VM_STRIPE_SIZE','-default', "64k");
    param('CLIENT_VM_NUM_LVS','-default', "1");
    if ($CLIENT_VM_NUM_LVS !~ m/^[0-9]+$/) {
        ExitConfigError("CLIENT_VM_NUM_LVS must be numeric");
    }
    if ($CLIENT_VM_NUM_LVS <= 0) {
        ExitConfigError("CLIENT_VM_NUM_LVS must be a positive integer");
    }
    print("LUN_LIST=$main::LUN_LIST CLIENTS=$main::CLIENTS FILERS=$main::FILERS\n");

    # Default CLIENT_IO_DEVS to all LUNS in the LUN_LIST
    my $client_io_devs = $LUN_LIST;
    $client_io_devs =~ s/,.*?:/:/g;  # ? keeps it from being greedy
    $client_io_devs =~ s/,.*//;
    param('CLIENT_IO_DEVS','-default',$client_io_devs);
    if ($CLIENT_IO_DEVS eq '') {
        print("No client I/O devices will be created. CLIENT_IO_DEVS is empty.\n");
    }

    # Client list should have been parsed by config_filer.pm by now
    # if we have more than one client to talk to, split
    # into array of clients.
    my @clients = split(/:/, $CLIENTS);

    # Do all client param processing here (config_clientn.pm has been modifed to skip processing if CLIENTS is defined)
    # Only read the CLIENTn params that match the host names defined by CLIENTS
    # Does this imply that we must renumber CLIENTn?
    # If so, we should reorder the CLIENTS to the same order defined by CLIENT1 to CLIENTn
    # This would allow us to reassign CLIENTx to CLIENTy as x would always be less than or equal to y

    my @REASSIGNED_CLIENTS;
    my $client_num = 0;
    foreach my $client (@clients) {
        my $client_prefix = &client_param_get($client,1);
        if (!defined($client_prefix)) {
            ExitConfigError("Client '$client' is not defined for this test bed");
        }
        $client_num++;

        push(@SETUP_CLIENT_NAMES, $client);

        if ($CLIENT_IO_DEVS ne '') {
            print("The following client I/O devices will be used: $CLIENT_IO_DEVS\n");
            &_configure_client($client, $MULTIPATH, $USE_FS, $CLIENT_IO_FS_TYPE); 
        }

        # Second parameter here causes all matching CLIENT param names to get the $client_param's value
        my $client_param2;
        if (@clients >1) {
            # See if there is a reassignment
            if ('CLIENT'.$client_num eq $client_prefix) {
                # No reassignment
                $client_param2 = $client_prefix;   
            } else {
                $client_param2 = 'NEW_CLIENT'.$client_num;   
                # Be sure to do CLIENT1 last to prevent
                # early CLIENT1_ to CLIENT_ parameter copying
                # to bleed into other CLIENT_ to CLIENTn_ copying
                unshift(@REASSIGNED_CLIENTS, 'CLIENT'.$client_num);
                no strict 'refs';
                param($client_param2,-set,${$client_prefix});
                use strict 'refs';
            }
        } else {
            # CLIENTS only has a single host name
            # This client must become the CLIENT
            # (config_filer.pm may have picked a new CLIENT instead of the testbed.info value)
            $client_param2 = 'CLIENT';
        }
        client_params_get($client_prefix,$client_param2);
    }

    if (@REASSIGNED_CLIENTS) {
        # Complete the reassignments
        foreach my $client_prefix (@REASSIGNED_CLIENTS) {
            client_params_get('NEW_'.$client_prefix,$client_prefix);
        }
    }

    if ($CLIENT ne $SETUP_CLIENT_NAMES[0]) {
        print("Setting CLIENT=$SETUP_CLIENT_NAMES[0]\n"); #TRACE
        param('CLIENT','-set', "$SETUP_CLIENT_NAMES[0]");
    }

    # Redefine the number of clients
    param('CLIENT_NUM',-set,scalar(@SETUP_CLIENT_NAMES));
    print("Finished config of CLIENTS=$main::CLIENTS\n"); #DEBUG


} else {
    # No setup or client reassignment has been requested
    # Just look for CLIENT parameters
    client_params_get('CLIENT');
}


1;

#print "\n ".__FILE__." LOADED \n";

END {
    # Handle SKIP_CLEANUP=0 requests after the script has otherwise finished
    my $save_return_code = $?;
    print("Starting END routine for ".__FILE__." with exit code $save_return_code\n");
    eval {
        client_cleanup($save_return_code);
    };
    print("Finished END routine for ".__FILE__." with exit code $save_return_code\n");
    $? = $save_return_code;
}
