# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/config_clientn.pm#1 $
# 
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.
#
# lib/config_clientn.pm
## @summary Establishes NATE params for a multiple client configuration
## @description
##
## Obtains values for the standard 'client' configuration parameters.
## Such a configuration defines a single client perspective of a test bed.
##
## Builds arrays of FCP_CLIENTS ISCSI_CLIENTS PTI_CLIENTS  
## SOL_CLIENTS UNIX_CLIENTS WIN_CLIENTS
## containing the hostnames of each such client.
##
## Builds CLIENT_FCP_INIT_NAMES array of FCP WWPN values.
## Builds CLIENT_ISCSI_INIT_NAMES array of iSCSI node names.
##
## @synopsis
##   require config_clientn;
##
## @param CLIENT			        
##  DNS name of the client.
## @param CLIENT_CONN	       
##  Client connection mode, default is $CLIENT.#rsh
##
## @param CLIENT_FCP_INSTALLED		      
##  Set to 'yes' if blocks FCP hardware is installed on the client.
## @param CLIENT_FCP_WWNN		      
##  FCP World Wide Node Name for this client.
## @param CLIENT_FCP_IO_DEV		      
##  The FCP I/O device for the test case to use.
## @param CLIENT_ISCSI_INSTALLED		      
##  Set to 'yes' if blocks iSCSI hardware is installed on the client.
## @param CLIENT_ISCSI_WWNN		      
##  iSCSI World Wide Node Name for this client.
## @param CLIENT_ISCSI_IO_DEV		      
##  The iSCSI I/O device for the test case to use.

## @param CLIENT_INITn_PROTOCOL     
##  'fcp' or 'iscsi' protocol of the n'th initiator
## @param CLIENT_INITn_WWNN     
##  World Wide Node Name of the n'th initiator
## @param CLIENT_INITn_WWPN     
##  World Wide Port Name of the n'th initiator
## @param CLIENT_INITn_LUN1     
##  First client LUN  on initiator n
## @param CLIENT_INITn_LUN2     
##  Second client LUN  on initiator n
## @param CLIENT_INITn_LUN3     
##  Third client LUN  on initiator n
## @param CLIENT_INITn_LUN4     
##  Fourth client LUN  on initiator n

## @param CLIENT_IO_DEV		      
##  The I/O device for the test case to use.
##  Defaults to the value of $CLIENT_FCP_IO_DEV, $CLIENT_ISCSI_IO_DEV or $CLIENT_IO_DEV_1
## @param CLIENT_IO_DEV_n		      
##  Each I/O device that the CLIENT is expected to have is defined with one of these parameters.
##  At some point, this package should test for the existence of these devices and create them
##  if they do not exist.
## @param CLIENT_IO_DEV_NUM
##  The number of IO devices that are being used by the current test.
##  This is for tests that use more than one device on a client.
##  The test script is to limit its testing to devices from 1 to this specified value.

## @param CLIENT_IP		      
##  IP address of interface to client, defaults to hosttype record DEFAULT_IP value
## @param CLIENT_IP_n		      
##  Additional IP address of interface to client. 
##  Up to 4 additional addresses are recognized as separate parameters

## @param CLIENT_MODE          
##  'c2' for 2-way clustered client, or 'c4', 'c8'. Default is 'mono'
## @param CLIENT_MODEL	      
##  Client model number
## @param CLIENT_OS	      
##  Client operating system
## @param CLIENT_OS_VER	      
##  Client operating system version
##
## @param CLIENT_PTI_INIT1_ID     
##  ID of PTI client first initiator port 
## @param CLIENT_PTI_INIT2_ID     
##  ID of PTI client second initiator port 
## @param CLIENT_PTI_INIT3_ID     
##  ID of PTI client third initiator port 
## @param CLIENT_PTI_INIT4_ID     
##  ID of PTI client fourth initiator port 
## @param CLIENT_PTI_INIT_ID     
##  ID of PTI client initiator port to test, defaults to $CLIENT_PTI_INIT1_ID 

## @param CLIENT_PTI_TARGET1_ID     
##  ID of PTI client first target port 
## @param CLIENT_PTI_TARGET2_ID     
##  ID of PTI client second target port 
## @param CLIENT_PTI_TARGET3_ID     
##  ID of PTI client third target port 
## @param CLIENT_PTI_TARGET4_ID     
##  ID of PTI client fourth target port 
## @param CLIENT_PTI_TARGET_ID     
##  ID of PTI client target port to test, defaults to $CLIENT_PTI_TARGET1_ID 

## @param CLIENT_VI_ADDR The client VI device address corresponding to the CLIENT_VI_DEV_n value. 
## These values use the IP address decimal octet format.
## @param CLIENT_VI_DEV	The client VI device name as defined with ifconfig.
## @param CLIENT_VI_NAME The client VI device hostname as found in the host file.  
## @param CLIENT_IB_ADDR The client IB device address corresponding to the CLIENT_IB_DEVn value. 
## These values use the IP address decimal octet format.
## @param CLIENT_IB_DEV	The client IB device name as defined with ifconfig.
## @param CLIENT_IB_NAME The client IB device hostname as found in the host file.  
## @param CLIENT_DAFS_ADDR The client DAFS device address corresponding to the CLIENT_DAFS_DEVn value. 
## These values use the IP address decimal octet format.
## @param CLIENT_DAFS_DEV	The client DAFS device name as defined with ifconfig.
## @param CLIENT_DAFS_NAME The client DAFS device hostname as found in the host file.  

## @author Charlie Hedstrom (hedstrom@netapp.com)
## @status public
## @see-also lib/config_client.pm
## @change 2003-01-13 hedstrom: created 
## @change 2003-01-15 hedstrom: remove variables not used 
## @change 2003-03-11 jhayden: Added IB device and DAFS device info
## @change 2003-04-14 hedstrom: use list of variables to report

use strict;
use English;
use Carp;
use Tharn;
use config_client;

# Look for all defined clients
#
param('CLIENTS','-default',undef);
if (!defined($main::CLIENTS) or $main::CLIENTS eq '') {
   my $clientnum = 1;
   while ($clientnum > 0) {
   
      no strict 'refs';
      param('CLIENT'.$clientnum,'-default',undef);
      if (!defined(${'CLIENT'.$clientnum}) or ${'CLIENT'.$clientnum} eq '') {
         $clientnum = 0;
         last;
      }
      client_params_get('CLIENT'.$clientnum);
      use strict 'refs';
   
      $clientnum++;
   }
}

# Include trace information about global parameters that were deduced
my @globals = qw( 
   DAFS_CLIENTS IB_CLIENTS VI_CLIENTS 
   FCP_CLIENTS CLIENTS_FCP_INIT_NAMES
   ISCSI_CLIENTS 
   );
foreach my $global (@globals) {
   no strict 'refs';
   my $variable = "main::$global";
   if (scalar(@{$variable}) > 0) {
      print("$global=@{$variable}\n");
   }
   use strict 'refs';
}


1;

print "\n ".__FILE__." LOADED \n";
