# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/config_filer.pm#1 $
# 
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.
#
# lib/config_filer.pm
## @summary Establishes NATE params for a single filer configuration
## @description
##
## Obtains values for the standard 'filer' configuration parameters.
## Such a configuration defines a single filer perspective of a test bed.
##
## @synopsis
##   require config_filer;
##
# Setup variables
## @param LUN_LIST
## A colon separated list of lun id,size 
## No LUN SETUP will be done unless this list is specified.
## LUNs to be created will have the following lun-path:
## <LUN_PREFIX>nate_<hostname>_lun<lunid>
## @param CLIENTS
## A colon separated list of clients to be included in this test
## These values are used in creating the lun names.
## The default is to include all clients on the test bed.
## Instead of a client name,
## this list can also include entries enclosed in parentheses that
## describe the desired client in terms of a comma separated list of configuration types.
## See lib/ConfigTypes.pm for a list of valid values.
## The first entry in this list may be a numeric value that defines how many clients are 
## requested with these configuration types.
## @param FILERS                    
##  A colon separated list of filers to be included in this test
## @param LUN_PREFIX
## defaults to the value found on the first existing lun id in the LUN_LIST
## If no such LUNs exist, defaults to /vol/$FILER_VOL/
## /vol/volume/ to define where luns are created,
## /vol/volume/qtree/ to define where luns are created in a qtree
## /vol/volume/qtree/prefix to define where luns in a qtree with a file name prefix
## @param MULTIPATH
## 1 Defines multipath as being present, default is 0
## If this is 1, one igroup is created for all host HBAs (on each client)

## @param FILER                 
##  host name of the Filer.
## @param FILER_BOOTIMAGE      
##  Filer Bootimage (or alias name) to boot. If omitted, then FILER_BOOTIMAGE_TMPL must be provided.
## @param FILER_BOOTIMAGE_TMPL  
##  If FILER_BOOTIMAGE is blank, then this string is evaluated to create the bootimage string. 
##  Example: "/ethernet@2:172.25.1.10,$FILER_BOOTIMAGE_NAME, 172.25.1.129"
## @param FILER_BOOTIMAGE_NAME  
##  Filer boot image name that is substituted in the FILER_BOOTIMAGE_TMPL
## @param FILER_CONN           
##  Filer connection mode, default is $FILER.#rsh
## @param FILER_EXCLUDE_LUNS
## a list of blank delimited lun_paths of LUNs that are not be be deleted during cleanup
## @param FILER_IP            
##  IP address of interface to Filer
## @param FILER_MASK          
##  Net mask of interface corresponding to FILER_IP
## @param FILER_MEDIATYPE     
##  Mediatype for the first interface, 100tx-fd is default
## @param FILER_MODE         
##  'cf' for clustered filer (this is established with the NATE -cf option), Default is 'mono' 
## @param FILER_MODEL         
##  Filer model number
## @param FILER_TEST_UI         
##  Optional: default is CLI, other value recognized is 'ZAPI'
## @param FILER_VOL
##  Filer volume to be used by the current test.
##  A list of attributes may be enclosed in parentheses to select an existing volume with those attributes.
## The one with the most free space will be picked if more than one exists.
##  Defaults first to any volume in the LUN_PREFIX value
##  If that is not defined, defaults to the $PROJECT value
##  If that is not defined, defaults to 'test'

## @param HOSTNAME            
##  hostname of the Filer
##
## @param FILER_DAFS_VOL1_NAME   Name of DAFS volume 1 (define as many as you need)
## @param FILER_DAFS_VOL1_PATH   Path for DAFS volume 1, defaults to /vol/$FILER_DAFS_VOL1_NAME
## @param FILER_DAFS_VOL1_ENABLE Enable of DAFS volume 1, defaults to 'true'

## @param FILER_FCP_INSTALLED   "Yes" if FCP is installed on the filer
## @param FILER_FCP_WWNN    FCP node name of the filer
## @param FILER_ISCSI_INSTALLED "Yes" if iSCSI is installed on the filer
## @param FILER_ISCSI_WWNN  iSCSI node name of the filer
## 
## @param FILER_IFS     list of interfaces on the filer,separated by comma
## @param FILER_IPS     list of ip addresses on the filer,separated by comma
## @param FILER_IPSPS   list of ipspaces on the filer,separated by comma
## @param FILER_VFILER_IPS  list of blank delimited vfiler definitions each consisting of:
## interface_name:IP_address[:volume_or_qtree] for use by that vfiler
## @param FILER_VI_ADDR The filer VI device address corresponding to the FILER_VI_DEVn value. 
## These values use the IP address decimal octet format.
## @param FILER_VI_DEV  The filer VI device name as defined with ifconfig.
## @param FILER_VI_NAME The filer VI device hostname as found in the host file. 
## @param FILER_IB_ADDR The filer IB device address corresponding to the FILER_IB_DEVn value. 
## These values use the IP address decimal octet format.
## @param FILER_IB_DEV  The filer IB device name as defined with ifconfig.
## @param FILER_IB_NAME The filer IB device hostname as found in the host file. 
## @param FILER_DAFS_ADDR The filer DAFS device address corresponding to the FILER_DAFS_DEVn value. 
## These values use the IP address decimal octet format.
## @param FILER_DAFS_DEV    The filer DAFS device name as defined with ifconfig.
## @param FILER_DAFS_NAME The filer DAFS device hostname as found in the host file. 

## @param NONNETHOST            Name of a host that is not on the FILER's directly attached network
## @param NONNETADDR            IP address of NONNETHOST
## @param MISSINGHOST           Name of a non-existent host, default is "missing"
## @param MISSINGADDR           IP address of a non-existent host, default is "127.0.0.2"
## @param PRESENTHOST           Name of a host that is on the FILER's directly attached network
## @param PRESENTADDR           IP address of PRESENTHOST
## @dependencies Hardware: 
##   FilerConfig=filer
## @author Charlie Hedstrom (hedstrom@netapp.com)
## @status public
## @see-also lib/config_base.pm
## @change 2002-03-13 hedstrom: WWNN parameter name changes, add parameters
## @change 2002-03-14 hedstrom: fix vars
## @change 2002-04-11 hedstrom: defaults
## @change 2002-04-17 hedstrom: add @FILER_TARGET_NAMES
## @change 2002-04-18 hedstrom: add FILER_DISK_CONFIG
## @change 2002-05-15 hedstrom: add DAFS_VOL params
## @change 2002-05-22 hedstrom: pick up default bootimage from tharnhosts
## @change 2002-07-23 hedstrom: add FILER_VOL_DISK_COUNT with max option
## @change 2002-08-09 jhayden: Change default value FILER_VI_NAME
## @change 2002-08-16 jhayden: Change default value for an 'option dafs'
## @change 2002-12-04 hedstrom: use PROJECT as default FILER_VOL
## @change 2003-03-11 jhayden: Add IB and DAFS device info
## @change 2003-07-28 hedstrom: add NANE's LUN setup params, default to all CLIENTn as defined on the test bed
## @change 2003-08-22 hedstrom: look for all test bed clients to fix ability to verify specific client requests in CLIENTS param
## @change 2004-02-17 hedstrom: drop FILER_DISK_CONFIG, improve default for CLIENT_BIND_METHOD
## @change 2004-03-03 hedstrom: set CLIENT_NUM when CLIENTS is defined
## @change 2004-05-17 sli: place "$searching_igroups = 1;   # Keep looking" outside of else { ..}, because we need to search all
##            all igroup client_name, client_name-0 and client_name-1 
## @change 2004-05-26 hedstrom: make sure FCP and ISCSI services are initialized before calling configure.thpl
## @change 2004-09-21 hedstrom: exit as not applicable if LUN setup was requested without a SAN client
## @change 2004-09-23 hedstrom: skip vfiler setup if vfiler is not listed as a required config type
## @change 2004-12-02 hedstrom: add FILER_TEST_UI support
## @change 2004-12-14 hedstrom: Use -i suffix for iSCSI igroup names
## @change 2004-12-14 hedstrom: Make sure specified volume is ready
## @change 2004-12-14 hedstrom: Determine FCP and iSCSI client lists even when no config types selected them
## @change 2005-02-23 hedstrom: allow FILER_VOL='([!]<vol attributes>)' to select an existing volume
## @change 2005-02-24 hedstrom: only verify volume is online if SKIP_SETUP=0
## @change 2005-03-04 hedstrom: make SKIP_CLEANUP = ON_FAIL the default
## @change 2005-03-14 hedstrom: SKIP_CLEANUP = 1 the default, watch PID to only cleanup once
## @change 2005-03-31 hedstrom: always SKIP_CLEANUP for subtests

print("Starting ".__FILE__."\n");

use strict;
use English;
use Carp;
use Tharn;
use Burt;
use SharedTestBed::ConfigTypes;
use config_base;
use filer;
use ModelInfo;
use blocks_filer;
use TestEnv::MinWin;

use vars qw(@ISA @EXPORT
            $CLIENT_BIND_METHOD
            $CLIENT_IO_DEVS
            @FILER_DAFS_VOLS
            @FILER_FCP_TARGET_NAMES 
            @FILER_FCP_TARGET_PARTNERS 
            @FILER_ISCSI_TARGET_NAMES 
            @FILER_TARGET_NAMES

            $FILER $FILER_CONN 
            $FILER_BOOTIMAGE 
            $FILER_BOOTIMAGE_TMPL
            $FILER_FCP_WWNN
            $FILER_IP 
            $FILER_IP_1 
            $FILER_IP_2 
            $FILER_IP_3 
            $FILER_IP_4 
            $FILER_IPS 
            $FILER_IFC 
            $FILER_IFC_1 
            $FILER_IFC_2 
            $FILER_IFC_3 
            $FILER_IFC_4 
            $FILER_IFCS 
            $FILER_ISCSI_INSTALLED 
            $FILER_ISCSI_WWNN
            $FILER_MASK
            $FILER_MAX_MTUSIZE
            $FILER_MODE 
            $FILER_MODEL
            $FILER_VOL
            $FILER_VOL_DISK_COUNT
            $FILER_TARGET1_MEDIATYPE 
            $FILER_TARGET1_NAME 
            $FILER_TARGET1_PARTNER 
            $FILER_TARGET1_PROTOCOL 
            $FILER_TARGET1_WWNN 
            $FILER_TARGET1_WWPN
            $FILER_TARGET_MEDIATYPE 
            $FILER_TARGET_NAME 
            $FILER_TARGET_PROTOCOL
            $FILER_TARGET_WWNN 
            $FILER_TARGET_WWPN
            $FILER_VDISK1_COMMENT 
            $FILER_VDISK1_DIR 
            $FILER_VDISK1_NAME 
            $FILER_VDISK1_SIZE
            $FILER_VDISK_COMMENT 
            $FILER_VDISK_DIR 
            $FILER_VDISK_NAME 
            $FILER_VDISK_SIZE
            $FILER_VI_ADDR1 
            $FILER_VI_DEV1 
            $FILER_VI_NAME1
            $FILER_VI_ADDR 
            $FILER_VI_DEV 
            $FILER_VI_NAME
            $FILER_IB_ADDR1 
            $FILER_IB_DEV1 
            $FILER_IB_NAME1
            $FILER_IB_ADDR 
            $FILER_IB_DEV 
            $FILER_IB_NAME
            $FILER_DAFS_ADDR1 
            $FILER_DAFS_DEV1 
            $FILER_DAFS_NAME1
            $FILER_DAFS_ADDR 
            $FILER_DAFS_DEV 
            $FILER_DAFS_NAME

            $LUN_LIST

            $PROJECT
            );

require Exporter;
@ISA = ('Exporter');

@EXPORT = qw(
    config_filer_cleanup
          );

my @required_config;    # Holds a list of all configuration types pertinent to this test execution

# Subroutines

sub config_filer_cleanup(;$);

## @name :: local_filer_parse_lun_list
## @description Convert LUN_LIST stanza string into lun_list array.
## Exit on error.  Returns @lun_list or @lun arrays on success.
##
## Lun information is passed via a scalar string.  Each scalar string
## contains a space delimited group of comma separated values:
##
##  <lun-id>, <size> and <type>
##
## more at example.
##
## @synopsis
## local_filer_parse_lun_list {$action, $LUN_LIST}
##
## @arg $action Required: "luns" | "list"
## @arg $lun_list Required: lun_list scalar variable
##
## @example
## my $LUN_LIST="0,2g,solaris:1,1g,default:2,30g,solaris:10,250m,windows";
## my @luns = local_filer_parse_lun_list("luns", $LUN_LIST);
##
sub local_filer_parse_lun_list($$) {
    my ($action, $lun_list) = @_;
    my $name="local_filer_parse_lun_list";
    my @luns;

    &ExitScriptError("$name: Invalid parameter passed ($action)")
        if ($action !~ m/^luns$|^list$/);

    # drop quotes if present
    $lun_list =~ s/["']//g;

    my @lun_list = split(/:/, $lun_list);

    print "$name:\n";
    foreach my $param (@lun_list) {
        my ($n, $size, $type) = split(/,/,$param);
        print "lun-id = $n";
        if (defined($size)) {
           print ", size = $size";
           if (defined($type)) {
              print ", type = $type";
           }
        }
        print "\n";
        push (@luns, $n);
    }

    return (@lun_list) if ($action =~ /list/);
    return (@luns);
}

#
# config_filer_cleanup()
#
# Description: This is a general lib call using the setup variables
#              to cleanup up what was created.
#
# Parameters:
#    filer_conn - filer connection variable, to do cleanup
#
# Variables used to do cleanup:
#    LUN_PREFIX   = "/vol/sledgehammer/"
#    FILERS       = "rtpqa-fas270-1:rtpqa-fas270-2"
#    CLIENTS      = "rtpqa-linux-0:rtpqa-linux-1"
#    LUN_LIST     = "0,2g,linux:1,2g,linux"
#                    - Lun ID 0 of size 2g of type linux,
#                      Lun ID 1 of size 2g of type linux.
#    MULTIPATH    = 0
#                    - 0 means no multipathing, 1 means use multipathing
#    SKIP_CLEANUP = ON_FAIL means do not cleanup if there was a FAIL or FATAL error
#                    - 0 means do cleanup, 1 means SKIP_CLEANUP (default)
#                    - This value controls cleanup regardless of setup
#                    - If LUN_LIST = '' then we skip LUN cleanup.
#    CLEANUP_LIST = "all" or "luns:igroups"
#                    - Cleanup will clean everything currently supported.
#                    - Listing the object types will cause
#                    - only those object types to be cleaned up.
################################################################################
my $StartingPID = $PID;  # This is set only during the initial loading of this module
sub config_filer_cleanup(;$) {
    my $result_now = shift;

    my @callstack;
    Burt::CallStack(\@callstack,0);
    if ($PID ne $StartingPID and $callstack[1] =~ /\/config_filer.pm\s*\(/) {
        # Do not allow subtests to invoke any cleanup until the initial PID
        print("Ignoring config_filer_cleanup call from subtest PID $PID");
        return;
    }
    &logcomment("config_filer_cleanup called in PID $PID ",@callstack);
    
    param('SKIP_CLEANUP','-default',"1");
    if ($main::SKIP_CLEANUP =~ m/ON_FAIL/i) {
        # See if we have any FATAL or FAIL results objects
        my @results = (prev_results(),own_results());
        if (defined($result_now)) {
            push(@results,$result_now);
        }
        foreach my $result_obj (@results) {
            my $result_type = $result_obj->type();
            if ($result_type eq "FAIL" or $result_type eq "FATAL") {
                &logcomment("FAIL found, skipping Cleanup by user request (SKIP_CLEANUP = $main::SKIP_CLEANUP)");
                param('SKIP_CLEANUP','-set',"1");
                last;
            }
        }
    }
    if ($main::SKIP_CLEANUP eq '1') {
        return;
    } else {
        &logcomment("Cleanup by user request (SKIP_CLEANUP = $main::SKIP_CLEANUP)");
    }

    param('FILERS','-default','');
    my @filers = split(/:/, $main::FILERS);
    if (0 == @filers) {
        logcomment("FILERS list is empty, no clean up performed");
        return;
    }

    param('CLIENTS','-default','');
    my @client_list = split(':',$main::CLIENTS);
    if (0 == @client_list) {
         logcomment("CLIENTS list is empty, no clean up performed");
         return;
    }

    param('LUN_LIST','-default','');
    if ($main::LUN_LIST eq '') {
         logcomment("LUN_LIST is empty, no LUN clean up performed");
         return;
    }
    my @list_of_luns = local_filer_parse_lun_list("luns", $main::LUN_LIST);
    print("LUN_LIST parsed to:".join(" ",@list_of_luns)."\n");

    param('LUN_PREFIX','-default',''); # empty default supports any value for existing LUNs
    
    param('FILER_EXCLUDE_LUNS','-default','');
    my $keep_luns = ' '.$main::FILER_EXCLUDE_LUNS.' ';

    param('CLEANUP_LIST','-default','all');
    if ($main::CLEANUP_LIST eq '') {
        $main::CLEANUP_LIST = "all";
    }


    # It is assumed that client cleanup was completed
    # as config_client.pm exited.

    my $lun;

    my $filer_conn;
    my ($this_filer);

    foreach $this_filer (@filers) {

        $filer_conn = &main::connect($this_filer."#rsh-socket".";inherit=0");

        if ($main::CLEANUP_LIST eq 'all' or $main::CLEANUP_LIST =~ m/lun/i) {
            if (0 == @list_of_luns) {
                logcomment("LUN_LIST is empty, no LUN clean up performed");
            } else {

                foreach my $single_client (@client_list) {
                    # Get the client's nodename
                    my $client_param = client_param_get($single_client);
                    if (defined($client_param)) {
                    } else {
                        print "No such client on this test bed:$single_client\n"; #DEBUG
                        next; # This client is not defined on this test bed
                    }
                    my @client_nodenames;
                    foreach my $param_suffix ('_FCP_WWNN','_ISCSI_WWNN','_INIT1_WWPN','_INIT2_WWPN','_INIT3_WWPN','_INIT4_WWPN') {
                        my $client_nodename = param($client_param.$param_suffix,'-default',undef);
                        if (defined($client_nodename)) {
                            push(@client_nodenames,$client_nodename);
                        }
                    }
                    if (0 == @client_nodenames) {
                        print "No nodenames found for client:$single_client\n"; #DEBUG
                        next; # This client's nodenames are not defined on this test bed
                    }
                    #
                    # Remove the specified luns
                    #
                    foreach my $lun_range (@list_of_luns) {

                        # Drop any lun attributes (size or ostype)
                        $lun_range =~ s/,.*$//g;

                        my ($start,$end) = ($lun_range,$lun_range);
                        if ($lun_range =~ m/^(\d+)-(\d+)$/) {
                            ($start,$end) = ($1,$2);
                        }
                        if ($start !~ m/^\d+$/) {
                            ExitScriptError("LUN_LIST entries must be numeric. This value is invalid: $start");
                        }
                        if ($end !~ m/^\d+$/) {
                            ExitScriptError("LUN_LIST entries must be numeric. This value is invalid: $end");
                        }
                        logcomment("Beginning LUN clean up on filer: $this_filer for client: $single_client of LUNs: $lun_range");
                        for (my $single_lun = $start; $single_lun<=$end; $single_lun++) {
                            foreach my $a_nodename (@client_nodenames) {
                                filer_nodename_lunid_vaporize($filer_conn,$a_nodename,$single_lun);
                            }
                        }
                    }
                    foreach my $igroup ("${single_client}-0","${single_client}-1","${single_client}","${single_client}-i") {
                        #
                        # Remove empty, existing igroups
                        #
                        if (filer_igroup_is_defined($filer_conn, $igroup)) {
                            # if this igroup no longer has LUNs, destroy it
                            if (scalar(filer_igroup_luns_get($filer_conn, $igroup)) == 0) {
                                filer_igroup_vaporize($filer_conn, $igroup);
                            } else{
                                print "igroup $igroup cannot be deleted as it still has mappings on filer: $this_filer\n"; #DEBUG
                            }
                        } else {
                            print "igroup $igroup does not exist on filer: $this_filer\n"; #DEBUG
                        }
                    }
                }
            }
        }
    }
}



sub filer_params_get($) {
    my $filer = shift;

    my $filer_model_info;

    param ($filer,  '-default', "" );
    no strict 'refs';
    my $filer_name = ${$filer};
    if ($filer_name eq '') {
        param($filer,'-set',undef);
        return;  # There is no such filer
    }

    my $fh = host(${$filer});
    param('FILER_MODE',     '-default',"mono"); # Must appear before BOOTIMAGE params
    param ($filer.'_IP',    '-default', ( $fh->{default_ip} || undef ));
    print("config_filer saw $filer"."_IP=${$filer.'_IP'}\n");
    param ($filer.'_IFC',        '-default', (''));

    if ( $ENV{CONNECTTYPE} ) {
        param ($filer.'_CONN',    '-default', ( ${$filer}.  $ENV{CONNECTTYPE} ));
    }
    else {
        param ($filer.'_CONN',    '-default', ( ${$filer} . "#rsh-socket" || undef));
    }
    my $fc = connect (${$filer.'_CONN'}, -timeout, 10);
    # Make sure filer was not left in a vfiler context
    if (${$filer.'_CONN'} !~ m/#rsh/) {
        filer_vfiler_context_off($fc);
    }
    
    if ($main::FILER_TEST_UI =~ m/zapi/i) {
        # Make sure ZAPI is enabled on this filer
        filer_options_set($fc,'httpd.enable','on');
        filer_options_set($fc,'httpd.admin.enable','on');
    }

    param($filer.'_BOOTIMAGE',     '-default',($fh->{bootimage} || '') );
    param($filer.'_BOOTIMAGE_TMPL','-default',"");
    param($filer.'_BOOTIMAGE_NAME','-default',"");
    eval "\${$filer.'_BOOTIMAGE'} = \"${$filer.'_BOOTIMAGE_TMPL'}\"" if (${$filer.'_BOOTIMAGE'} eq '');
    eval "\${$filer.'_BOOTIMAGE'} = \"${$filer.'_BOOTIMAGE_TMPL'}\"" if (${$filer.'_BOOTIMAGE'} eq '');
    # XXX duplicate eval because of weird perl 5.005_03 bug (?)

    param($filer.'_DISK_CONFIG','-default',"");

    param($filer.'_DIR',        '-default', ( '' ));
    param($filer.'_MASK',       '-default', ( '0xffffff00' ));
    param($filer.'_MAX_MTUSIZE','-default', ( '' ));

    param($filer.'_MODEL',      '-default', undef);
    if (defined(${$filer.'_MODEL'}) and ${$filer.'_MODEL'} ne '') {
        $fh->{'model'} = ${$filer.'_MODEL'};
        #$FILER_MODELS{${$filer}} => ${$filer.'_MODEL'};
        print("FILER_MODELS{${$filer}} = $fh->{'model'}\n");  #DEBUG
        $filer_model_info = new ModelInfo(${$filer.'_MODEL'});
        print("FILER_ARCH = $filer_model_info->{'arch'}\n");  #DEBUG
    }
    param($filer.'_ARCH',      '-default', ( $filer_model_info->{'arch'} || undef ));

    param($filer.'_MEDIATYPE',  '-default', ( "100tx-fd" ));
    param($filer.'_PASSWORD',   '-default', $fh->{default_password});

    param('DEFAULT_IF',       '-default', ("e0"));
    param('TELNET_IF',        '-default', ("e0"));
    param('HOSTNAME',         '-default', (''));
    param('PROTOCOL',         '-default', (''));  # If a protocol is specified, do not process other protocols

    param($filer.'_FCP_INSTALLED','-default',"");
    param($filer.'_FCP_WWNN','-default',"");
    param($filer.'_ISCSI_INSTALLED','-default',"");
    param($filer.'_ISCSI_WWNN','-default',"");

    # Pick up all Target HBA parameters
    # Build arrays of filer parameters

    @{'main::'.$filer.'_TARGET_NAMES'} = ();
    @{'main::'.$filer.'_FCP_TARGET_PARTNERS'} = ();
    @{'main::'.$filer.'_FCP_TARGET_NAMES'} = ();
    @{'main::'.$filer.'_ISCSI_TARGET_NAMES'} = ();

    my $adapter = 1;

    while ($adapter > 0) {
        param($filer.'_TARGET'.$adapter.'_NAME','-default',"");
        if (${$filer.'_TARGET'.$adapter.'_NAME'} eq '') {
            $adapter = 0;
            last;
        }
        # If NAME was defined, get the other values
        param($filer.'_TARGET'.$adapter.'_MEDIATYPE','-default',"");
        param($filer.'_TARGET'.$adapter.'_PARTNER','-default',"");
        param($filer.'_TARGET'.$adapter.'_PROTOCOL','-default',"");
        param($filer.'_TARGET'.$adapter.'_WWPN','-default',"");
        push(@{'main::'.$filer.'_TARGET_NAMES'},${$filer.'_TARGET'.$adapter.'_NAME'});
        if (${$filer.'_TARGET'.$adapter.'_PROTOCOL'} =~ /^fcp$/i) {
            if ($main::PROTOCOL eq '' or $main::PROTOCOL =~ m/fcp/i) {
                param($filer.'_TARGET'.$adapter.'_WWNN','-default',${$filer.'_FCP_WWNN'});
                push(@{'main::'.$filer.'_FCP_TARGET_NAMES'},${$filer.'_TARGET'.$adapter.'_NAME'});
                if (${$filer.'_TARGET'.$adapter.'_PARTNER'} ne '' and ${$filer.'_TARGET'.$adapter.'_PARTNER'} ne 'None') {
                    push(@{'main::'.$filer.'_FCP_TARGET_PARTNERS'},${$filer.'_TARGET'.$adapter.'_PARTNER'});
                } else {
                    push(@{'main::'.$filer.'_FCP_TARGET_PARTNERS'},'');
                }
            }
        } elsif (${$filer.'_TARGET'.$adapter.'_PROTOCOL'} =~ /^iscsi$/i) {
            if ($main::PROTOCOL eq '' or $main::PROTOCOL =~ m/iscsi/i) {
                param($filer.'_TARGET'.$adapter.'_WWNN','-default',${$filer.'_ISCSI_WWNN'});
                push(@{'main::'.$filer.'_ISCSI_TARGET_NAMES'},${$filer.'_TARGET'.$adapter.'_NAME'});
            }
        } else {
            ExitConfigError("Unrecognized protocol for ".$filer."_TARGET".$adapter."_PROTOCOL '".${$filer.'_TARGET'.$adapter.'_PROTOCOL'});
        }
        $adapter++;
    }
    print("${filer}_FCP_TARGET_NAMES=@{'main::'.$filer.'_FCP_TARGET_NAMES'}\n");
    print("${filer}_ISCSI_TARGET_NAMES=@{'main::'.$filer.'_ISCSI_TARGET_NAMES'}\n");

    # Define default Target HBA values
    param($filer.'_TARGET_MEDIATYPE','-default',(${$filer.'_TARGET1_MEDIATYPE'} || ""));
    param($filer.'_TARGET_NAME','-default',(${$filer.'_TARGET1_NAME'} || ""));
    param($filer.'_TARGET_PROTOCOL','-default',(${$filer.'_TARGET1_PROTOCOL'} || ""));
    param($filer.'_TARGET_WWNN','-default',(${$filer.'_TARGET1_WWNN'} || ""));
    param($filer.'_TARGET_WWPN','-default',(${$filer.'_TARGET1_WWPN'} || ""));


    # VI Interfaces to the Filer
    while ($adapter < 5) {
        param($filer.'_VI_NAME'.$adapter.'','-default',undef);
        param($filer.'_VI_ADDR'.$adapter.'','-default',undef);
        param($filer.'_VI_DEV'.$adapter.'','-default',undef);
        $adapter++;
    }
    # Define default VI values
    param($filer.'_VI_ADDR','-default',(${$filer.'_VI_ADDR1'} || undef));
    param($filer.'_VI_DEV','-default',(${$filer.'_VI_DEV1'} || undef));
    param($filer.'_VI_NAME','-default',(${$filer.'_VI_NAME1'} || undef));

    # IB Interfaces to the Filer
    $adapter = 1;
    while ($adapter < 5) {
        param($filer.'_IB_NAME'.$adapter,'-default',undef);
        param($filer.'_IB_ADDR'.$adapter.'','-default',undef);
        param($filer.'_IB_DEV'.$adapter.'','-default',undef);
        $adapter++;
    }
    # Define default IB values
    param($filer.'_IB_ADDR','-default',(${$filer.'_IB_ADDR1'} || undef));
    param($filer.'_IB_DEV','-default',(${$filer.'_IB_DEV1'} || undef));
    param($filer.'_IB_NAME','-default',(${$filer.'_IB_NAME1'} || undef));

    # DAFS Interfaces to the Filer
    $adapter = 1;
    while ($adapter < 5) {
        param($filer.'_DAFS_NAME'.$adapter, 
              '-default',(${$filer.'_VI_NAME'.$adapter} || ${$filer.'_IB_NAME'.$adapter} || undef));
        # If NAME was defined, get the other values
        param($filer.'_DAFS_ADDR'.$adapter, 
              '-default',(${$filer.'_VI_ADDR'.$adapter} || ${$filer.'_IB_ADDR'.$adapter} || undef));
        param($filer.'_DAFS_DEV'.$adapter, 
              '-default',(${$filer.'_VI_DEV'.$adapter} || ${$filer.'_IB_DEV'.$adapter} || undef));
        $adapter++;
    }
    # Define default DAFS values
    param($filer.'_DAFS_ADDR', '-default',(${$filer.'_DAFS_ADDR1'} || undef));
    param($filer.'_DAFS_DEV', '-default',(${$filer.'_DAFS_DEV1'} || undef));
    param($filer.'_DAFS_NAME', '-default',(${$filer.'_DAFS_NAME1'} || undef));

    # VDISK values
    my $vdisk = 1;
    while ($vdisk > 0) {
        param($filer.'_VDISK'.$vdisk.'_NAME','-default',undef);
        if (!defined(${$filer.'_VDISK'.$vdisk.'_NAME'}) or ${$filer.'_VDISK'.$vdisk.'_NAME'} eq '') {
            $vdisk = 0;
            last;
        }
        # If NAME was defined, get the other values
        param($filer.'_VDISK'.$vdisk.'_COMMENT','-default',undef);
        param($filer.'_VDISK'.$vdisk.'_DIR','-default',undef);
        param($filer.'_VDISK'.$vdisk.'_SIZE','-default',undef);
        $adapter++;
    }
    $adapter = 1;
    # filer interfaces
    param($filer.'_IFS',        '-default', "");
    param($filer.'_IPS',        '-default', "");
    if (defined(${$filer.'_IP'})) {
        ${$filer.'_IPS'} = ${$filer.'_IP'};
    }
    my $ip = 1;
    while ($ip > 0) {
        param($filer.'_IP_'.$ip,'-default',undef);
        if (!defined( ${$filer.'_IP_'.$ip}) or ${$filer.'_IP_'.$ip} eq '') {
            $ip = 0;
            last;
        }
        if ( !${$filer.'_IPS'} ) {
            ${$filer.'_IPS'} = ${$filer.'_IPS'}.",".${$filer.'_IP_'.$ip};
        } else {
            ${$filer.'_IPS'} = ${$filer.'_IP_'.$ip};
        }
        $ip++;
    }

    # Define default vdisk values
    param($filer.'_VDISK_COMMENT','-default',(${$filer.'_VDISK1_COMMENT'} || ""));
    param($filer.'_VDISK_DIR','-default',(${$filer.'_VDISK1_DIR'} || ""));
    param($filer.'_VDISK_NAME','-default',(${$filer.'_VDISK1_NAME'} || ""));
    param($filer.'_VDISK_SIZE','-default',(${$filer.'_VDISK1_SIZE'} || ""));

    param('LUN_PREFIX','-default', "");
    param($filer.'_VOL','-default', "");
    if (${$filer.'_VOL'} eq '') {
        if (${main::LUN_PREFIX} ne '' and ${main::LUN_PREFIX} ne '*') {
            my $vol = ${main::LUN_PREFIX};
            $vol =~ s#^/vol/##;
            $vol =~ s#/.*##g;
            param($filer.'_VOL','-set', "$vol");
        }
        if (${$filer.'_VOL'} eq '') {
            my $project = $ENV{PROJECT};
            if (not defined($project) or $project eq '') {
                $project = 'test';
            }
            param($filer.'_VOL','-set', "$project");
        }
    }
    param($filer.'_VOL_MNT','-default', (length(${$filer.'_VOL'}) ? "/vol/${$filer.'_VOL'}" : "/"));
    my $filer_vol_mnt = ${"main::$filer".'_VOL_MNT'};
    # Determine value for number of disks to include in FILER_VOL if it is to be created
    # Default to the minimun volume disk count
    param($filer.'_VOL_DISK_COUNT','-default', "3");
    my $vol_count = ${$filer.'_VOL_DISK_COUNT'};
    if ($vol_count =~ /max/i) {
        # The most it needs is enough to define a maximum sized volume
        #The XFiler H = 8TB 
        #The F880, F840 and XFiler M = 4TB
        #The F820, F825 (RH.1), F760 = 2TB
        #All others 1.5TB or lower depending on the max capacity supported.
        my $max_capacity;
        if ($filer.'_MODEL' =~ /FAS960|FAS980/) {
            $max_capacity = 8*1024*1024*1024*1024;
        } elsif ($filer.'_MODEL' =~ /F840|F880|FAS940/ ) {
            $max_capacity = 4*1024*1024*1024*1024;
        } elsif ($filer.'_MODEL' =~ /F820|F825|F760/ ) {
            $max_capacity = 2*1024*1024*1024*1024;
        } else {
            $max_capacity = 3*512*1024*1024*1024;
        }

        # The most it can have is all of the spares
        my $spare_disk_count = filer_spares_get($fc);
        my $disk_count = $spare_disk_count;
        if ($spare_disk_count > 0) {
            my $volume_size = filer_disk_capacity_get($fc);
            print("max capacity=$max_capacity\n");
            print("volume size=$volume_size\n");
            print("spare disk count=$spare_disk_count\n");

            if ($volume_size > 0) {
                $disk_count = 1+int($max_capacity/$volume_size);
                if ($disk_count > $spare_disk_count) {
                    $disk_count = $spare_disk_count;
                }
            }
        }
        # Now substitute the 'max' word with the disk count
        $vol_count =~ s/max/$disk_count/;
        eval "$disk_count = $vol_count";
        # Pass value back
        ${$filer.'_VOL_DISK_COUNT'} = $disk_count;
        print("$filer"."_VOL_DISK_COUNT=$disk_count\n"); #TRACE
        
    }

    # Define Filer 'options' from param values
    param($filer.'_OPTIONS_DAFS_AUTH_NONE_ENABLE','-default',"on");
    param($filer.'_OPTIONS_DAFS_CHECKSUMS_ENABLE','-default',"off");
    param($filer.'_OPTIONS_DAFS_DEFAULT_GID','-default',"100");
    param($filer.'_OPTIONS_DAFS_DEFAULT_UID','-default',"100");
    param($filer.'_OPTIONS_DAFS_ENABLE','-default',"on");
    param($filer.'_OPTIONS_DAFS_IDLE_TIMEOUT','-default',"7200");
    param($filer.'_OPTIONS_DAFS_MAX_DISCONNECTED_SESSIONS','-default',"32");
    param($filer.'_OPTIONS_DAFS_MAX_PENDING_REQUESTS_SERVER','-default',"512");
    param($filer.'_OPTIONS_DAFS_MAX_REQUEST_SIZE','-default',"69632");
    param($filer.'_OPTIONS_DAFS_MAX_REQUESTS','-default',"50");
    param($filer.'_OPTIONS_DAFS_MAX_REQUESTS_SERVER','-default',"1024");
    param($filer.'_OPTIONS_DAFS_MAX_RESPONSE_SIZE','-default',"69632");
    param($filer.'_OPTIONS_DAFS_NUM_REQUESTD','-default',"2");
    param($filer.'_OPTIONS_DAFS_RECEIVE_MULTI_SEGMENT_ENABLE','-default',"off");

    # filer DAFS volumes
    @{'main::'.$filer.'_DAFS_VOLS'} = ();

    my $volnum = 1;
    while ($volnum > 0) {
        param($filer.'_DAFS_VOL'.$volnum.'_NAME','-default',undef);
        if (!defined(${$filer.'_DAFS_VOL'.$volnum.'_NAME'}) or ${$filer.'_DAFS_VOL'.$volnum.'_NAME'} eq '') {
            $volnum = 0;
            last;
        }
        push (@{'main::'.$filer.'_DAFS_VOLS'},${$filer.'_DAFS_VOL'.$volnum.'_NAME'});

        param($filer.'_DAFS_VOL'.$volnum.'_PATH','-default',"/vol/${$filer.'_DAFS_VOL'.$volnum.'_NAME'}");
        param($filer.'_DAFS_VOL'.$volnum.'_ENABLED','-default',"true");
        $volnum++;
    }
   
    if ($main::LISTTESTCASES !~ m/y/i and ($main::SKIP_SETUP eq '0' or $main::SKIP_SETUP =~ m/^n/i)) {
        if (0 < grep(/^vfiler$/,@required_config) ) {
            param($filer.'_VFILER_IPS','-default',undef);
            if (defined(${$filer.'_VFILER_IPS'}) and ${$filer.'_VFILER_IPS'} ne '') {
                # Verify vfiler configuration uses these IP addresses
                # or establish them
                print("$filer"."_VFILER_IPS=${$filer.'_VFILER_IPS'}\n");#DEBUG

                # License the vfiler service if it is not licensed
                if (!filer_service_is_licensed($fc,'multistore')) {
                    filer_service_license($fc,'multistore')
                }

                # Find existing vfilers and vfiler limit settings
                # These values include vfiler0 as one of the counts
                my @existing_vfilers = filer_vfilers_get($fc);
                my ($vfiler_limit,$platform_limit,$vfiler_current_count) = filer_vfiler_limit_get($fc);

                # Define a hash of vfilers containing their [interface,IP address,vol or qtree]
                my %Vfiler;
                my $unique = 1;  # Start the count with vfiler0
                foreach my $entry (split(/\s+/,${$filer.'_VFILER_IPS'})) {
                    print("vfiler entry='$entry'\n"); #DEBUG
                    my $vfiler = "vf" . $unique;
                    $unique++;
                    if ($unique > $platform_limit) {
                        tstlog('-warn',["Adjusting the number of vfilers down to the platform limit of $platform_limit",
                        "Please update the $filer_name VFILER_IPS variable for this limit"]);
                        last;
                    }
                     my ($ifc,$ip,$path) = split(/:/,$entry);
                     # Default a qtree on the current FILER_VOL for this vfiler's path
                     if (!defined($path)) {
                         $path = "$filer_vol_mnt/$vfiler $filer_vol_mnt/${vfiler}_root";
                     } else {
                         $path =~ s/,/ /g;  # Replace comma vol/qtree separators with a space
                     }
                     $Vfiler{$vfiler} = [$ifc, $ip, $path];
                }

                # Make sure enough vfilers can be created
                my $vfiler_need = keys %Vfiler;
                my $real_vfiler_need = $vfiler_need +1;  # Count vfiler0 also
                if ($vfiler_limit < ($real_vfiler_need)) {
                    if ($real_vfiler_need <= $platform_limit) {
                        if ($main::LISTTESTCASES !~ m/y/i and ($main::SKIP_SETUP eq '0' or $main::SKIP_SETUP =~ m/^n/i)) {
                            filer_vfiler_limit_set($fc,$real_vfiler_need);
                            # A reboot is needed to get this change!
                            filer_reboot($fc);
                        } else {
                            ExitConfigError("SKIP_SETUP has been specified but filer $filer_name is not configured for $real_vfiler_need vfilers");
                        }
                    } else {
                        tstlog('-warn',"Vfiler physical limit of $platform_limit is insufficient for the VFILER_IPS value with $vfiler_need entries\n");
                    }
                } else {
                    print("Vfiler limit of $vfiler_limit is sufficient for the VFILER_IPS value with $vfiler_need entries\n");
                }
                # Identify any requested vfilers that are missing
                my @missing_vfilers = ();
                foreach my $requested_vf (keys %Vfiler) {
                    my $meta = quotemeta($requested_vf);
                    if (0 == grep(/^$meta$/,@existing_vfilers) ) {
                        push(@missing_vfilers,$requested_vf);
                    }
                }


                if (@missing_vfilers and $fc->conntype() !~ m/pm_cons|serial/) {
                   # Console connection needs to be established, since rsh will fail after interface is taken down next
                   disconnect($fc);
                   $fc = connect("$filer_name#console");

                }
                # Define each missing vfiler
                foreach my $requested_vf (@missing_vfilers) {
                    my ($ifc,$ip,$vfpaths) = @{$Vfiler{$requested_vf}};

                    # Make sure this interface is down, so we can assign it to a vfiler
                    filer_if_down($fc,$ifc);

                    # Define storage for this vfiler if it does not exist
                    my @paths = split(' ',$vfpaths);
                    foreach my $path (@paths) {
                        if ($path =~ m#/.*/.*/#) {
                            # This is a qtree, not just a volume
                            if (!filer_qtree_exists($fc,$path)) {
                                filer_qtree_create($fc,$path);
                                if ($main::SKIP_SETUP eq '0' or $main::SKIP_SETUP =~ m/^n/i) {
                                } else {
                                    ExitConfigError("SKIP_SETUP has been specified but filer $filer_name needs qtree $path to be established");
                                }
                            }
                        }
                    }

                    # Determine IP addresses associated with this interface
                    my @ips = filer_if_ips_get($fc,$ifc);
                    if (@ips) {
                        my $meta = quotemeta($ip);
                        if (0 == grep(/^$meta$/, @ips)) {
                            # Alias the undefined IP address to the specified interface
                            filer_if_alias($fc,$ifc,$ip);
                        }
                    } else {
                        # No IPs defined yet, add the first one
                        filer_if_ip($fc,$ifc,$ip);

                        #ExitConfigError("VFILER_IPS lists an interface, $ifc, that has no IP addresses defined on filer $filer_name.");
                    }
                    if ($main::SKIP_SETUP eq '0' or $main::SKIP_SETUP =~ m/^n/i) {
                        # filer_vfiler_create($fc,"vfiler_name","IPlist","Paths");
                        filer_vfiler_create($fc,$requested_vf,$ip,$vfpaths);
                    } else {
                        ExitConfigError("SKIP_SETUP has been specified but filer $filer_name does not have vfiler $requested_vf");
                    }

                    # Now make sure interface is up
                    filer_if_up($fc,$ifc);
                }
            }
        }
        use strict 'refs';
    }

}


my $ran_once;
if (defined($ran_once)) {
    print("Skipping config_filer.pm execution, it already ran\n");
} else {
    $ran_once = 1;
    # All set up operations are to be skipped if this is non-zero
    # This must be determined before setting up the first filer
    param('SKIP_SETUP','-default', "0");
    param('LISTTESTCASES','-default', "0");
    # Configuration errors can be issued if set up verification fails for configuration types included in the TESTPLAN_CONFIGS value
    
    param('FILER_TEST_UI','-default','CLI');

    param('PROJECT','-default', undef);
    my $project = $main::PROJECT;
    if (not defined($project) or $project eq '') {
        $project = 'test';
    }


    ## Verify test case is applicable to the test bed.
    ## Determine which configuration types are required for this test
    ## This can influence whether or not related services or setup operations are to be performed
    @required_config = applies_to_testbed();

    # Keep track of parameters whose values we change
    my @params = ();
    # filer
    filer_params_get('FILER');

    param('FILERS','-default', "");
    # Remove quotes from FILERS param
    if ($main::FILERS =~ m/^['"].*['"]$/) {
        $main::FILERS =~ s/^['"]//;
        $main::FILERS =~ s/['"]$//;
    }
    if ($main::FILERS ne '') {
        # Perform late variable substitution to get current values (not initial values)
        while ($main::FILERS =~ m/\$(\w+)/) { # Substitute parameters
            my $newparm = $1;
            param($newparm,'-default','');
            $main::FILERS =~ s/\$$newparm/${'main::'.$newparm}/;
        };  
    }
    if ($main::FILERS eq '') {
        param('FILERA','-default',undef);
        param('FILERB','-default',undef);
        if (defined($main::FILERA) and $main::FILERA ne '' and defined($main::FILERB) and $main::FILERB ne '') {
            param('FILERS','-set', "$main::FILERA:$main::FILERB");
            push(@params,"FILERS=$main::FILERS");
        }
    } 
    my @filers = split(':',$main::FILERS);

    param('CLIENTS','-default', "");
    # Remove quotes from CLIENTS param
    if ($main::CLIENTS =~ m/^['"].*['"]$/) {
        $main::CLIENTS =~ s/^['"]//;
        $main::CLIENTS =~ s/['"]$//;
    }
    if ($main::CLIENTS ne '') {
        # Perform late variable substitution to get current values (not initial values)
        while ($main::CLIENTS =~ m/\$(\w+)/) { # Substitute parameters
            my $newparm = $1;
            param($newparm,'-default','');
            $main::CLIENTS =~ s/\$$newparm/${'main::'.$newparm}/;
        };  
    }

    param('FILER_VOL','-default', undef);
    my $filer_vol = $main::FILER_VOL;
    if (defined($main::FILER_VOL) and $main::FILER_VOL ne '') {
        $filer_vol = $main::FILER_VOL;
        $filer_vol =~ s#^/vol/##;
        $filer_vol =~ s#/$##g;
        # See if a list of volume attributes were given
        if ($main::FILER_VOL =~ m/\((\S*)\)/) {
            my $vol_attributes = $1;
            logcomment("Looking for volumes on all filers with these attributes: '$vol_attributes'");
            my @possible_vols;
            # Now find an existing volume with these attributes
            foreach my $filer (@filers) {
                my $fh = host($filer);
                my $fc = connect("$filer#rsh-socket");
                my @filer_vols = filer_vols_get($fc,$vol_attributes);
                if (@filer_vols) {
                    if (@possible_vols) {
                        # Keep all volumes that are commmon to the previously found names
                        my @still_possible_volumes = ();
                        foreach my $old_vol (@possible_vols) {
                            if (grep(/^$old_vol$/,@filer_vols)) {
                                push(@still_possible_volumes,$old_vol);
                            }
                        }
                        if (@still_possible_volumes) {
                            @possible_vols = @still_possible_volumes;
                        } else {
                            logcomment("No same named volumes exist on all filers with these attributes: $vol_attributes");
                            @possible_vols = ();
                            last;
                        }
                    } else {
                        @possible_vols = @filer_vols;
                    }
                } else {
                    logcomment("No volumes exist on $filer with these attributes: $vol_attributes");
                    @possible_vols = ();
                    last;  # Skip any other filers, no common volume on all filers
                }
            }
            # Now look through all possible volume names
            if (@possible_vols) {
                my $fh = host($filers[0]);
                my $fc = connect("$filers[0]#rsh-socket");
                # Pick the one with the most space
                my $max_space = 0;
                my $max_space_vol = undef;
                foreach my $possible_vol (@possible_vols) {
                    my %space_info = filer_vol_spaceinfo_get($fc,$possible_vol);
                    if (defined($space_info{FREE}) and $space_info{FREE} > $max_space) {
                        $max_space = $space_info{FREE};
                        $max_space_vol = $possible_vol;
                    }
                }
                if (defined($max_space_vol)) {
                    param('FILER_VOL','-set', "$max_space_vol");
                    $filer_vol = $max_space_vol;
                    logcomment("Setting FILER_VOL=$filer_vol (an existing volume with these attributes: '$vol_attributes' and free space=$max_space)");
                } else {
                    # Could not determine space on any volume, set default to try to allocate a volume
                    logcomment("Could not determine space on existing volumes".join(' ',@possible_vols));
                    param('FILER_VOL','-set', "");
                    $filer_vol = '';
                }
            } else {
                # No existing volumes were found with the specified attributes
                logcomment("Could not find existing volumes with these attributes: '$vol_attributes'");
                # Can we create such volumes?
                
                param('FILER_VOL','-set', "");
                $filer_vol = '';
            }
        }
    }
    
    param('LUN_LIST','-default', "");
    if ($main::LUN_LIST ne '' or $main::CLIENTS ne '') {
        no strict 'refs';

        param('LUN_PREFIX','-default', "");
        my $existing_vol = '';
        if (($main::LUN_PREFIX eq '' or $main::LUN_PREFIX eq '*') and $main::LUN_LIST ne '') {
            if (!defined($filer_vol) or $filer_vol eq '') {
                # Find an existing volume from either the $project name or 'test'
                foreach my $test_vol ($project,'test') {
                    $existing_vol = $test_vol;
                    foreach my $filer (@filers) {
                        my $fh = host($filer);
                        my $fc = connect("$filer#rsh-socket");
                        if (!filer_vol_exists($fc,"/vol/$test_vol")) {
                            $existing_vol = '';
                            last;  # Skip any other filers, try next volume
                        }
                    }
                    if ($existing_vol ne '') {
                        last;   # We found one, stop looking
                    }
                }
                if ($existing_vol ne '') {
                    $filer_vol = $existing_vol;
                } else {
                    ExitConfigError("Any LUN_LIST value requires a LUN_PREFIX to be specified if neither /vol/\$PROJECT/ nor /vol/test/ exists.");
                }
            }
        }
            
        if ($filer_vol eq '') {
            # No volume name was given, default to project name
            if ($existing_vol ne '') {
                $filer_vol = $existing_vol;
                print("Setting FILER_VOL to an existing volume: $filer_vol\n");
            } else {
                if (defined($project) and $project ne '')  {
                    $filer_vol = $project;
                    print("Defaulting FILER_VOL to the project name: $filer_vol\n");
                } else {
                    $filer_vol = 'test';
                    print("Defaulting FILER_VOL to '$filer_vol'\n");
                }
            }
            param('FILER_VOL','-set', "$filer_vol");
        } else {
            while ($filer_vol =~ m/\$\{?(\w+)\}?/) { 
                my $param_name = $1;
                my $param_value = param($param_name,'-default',undef);
                if (defined($param_value)) {
                    print("Substituting $param_name=$param_value in FILER_VOL\n");
                    $filer_vol = s/\$$param_name/$param_value/g;
                } else {
                    print("Dropping $param_name from FILER_VOL\n");
                    $filer_vol = s/\$$param_name//g;
                }
            };  # Substitute parameters
        }
        # Create volume if SKIP_SETUP=0
        if ($main::LISTTESTCASES !~ m/y/i and ($main::SKIP_SETUP eq '0' or $main::SKIP_SETUP =~ m/^n/i)) {
            foreach my $filer (@filers) {
                my $fh = host($filer);
                my $fc = connect("$filer#rsh-socket");
                # Make sure volume exists
                if (!filer_vol_exists($fc,"/vol/$filer_vol")) {
                    # Determine how many disks the test volume must have
                    my $input_disk_count;
                    if (defined($main::FILER_DISK_COUNT)) {
                        $input_disk_count = $main::FILER_DISK_COUNT;
                    } else {
                        # We may want to consider requested LUN size and count
                        $input_disk_count = 3;
                    }
                    # Make sure requested spares are available
                    my @spare_disks = filer_spares_get ($fc);
                    if (@spare_disks >= $input_disk_count) {
                        filer_vol_create($fc,$filer_vol,"$input_disk_count");
                    } else {
                        ExitConfigError("A total of $input_disk_count spare disks is needed on filer: $filer");
                    }
                }
            }
            
            foreach my $filer (@filers) {
                my $fh = host($filer);
                my $fc = connect("$filer#rsh-socket");
                # If we have defined the volume by now, make sure it is ready to be used
                if (filer_vol_exists($fc,"/vol/$filer_vol")) {
                    if (!filer_vol_wait_until_ready($fc,$filer_vol)) {
                        ExitConfigError("Filer volume, $filer_vol is not ready for use on filer: $filer");
                    }
                }
            }
        }
        

        my @clients = ();
        my @fcp_clients = ();
        my @fcp_multipath_clients = ();
        my @iscsi_clients = ();
        # Determine all clients to validate values specified, or to default the CLIENTS parameter
        my @all_clients = ();
        my $client_num = 1;
        while (1) {
            my $client_param = 'CLIENT'.$client_num;
            param($client_param,'-default',undef);
            if (!defined(${"main::$client_param"}) or ${"main::$client_param"} eq '') {
                use strict 'refs';
                last;
            }
            my $client_name = ${"main::$client_param"};
            # If CLIENTn was defined, add it to the list of clients
            push(@all_clients,$client_name);
            # Determine client configuration types if CLIENTS parameter has parentheses
            if ($main::CLIENTS =~ m/\(.*\)/ ) {
                # Get the configuration types for this client
                param('CLIENT'.$client_num."_CONFIGS",'-default',"");
                # Convert specifed client configuration types to include implied configuration types
                my $raw_configs = ${'main::CLIENT'.$client_num."_CONFIGS"};
                $raw_configs =~ s/['"]//g; # remove possible enclosing quotes
                my @configs = split(',',$raw_configs);
                @configs = testbed_config_types(@configs);
                my $client_configs = join(',',@configs);
                param('CLIENT'.$client_num."_CONFIGS",'-set',$client_configs);
                logcomment("CLIENT$client_num=$client_name has CONFIGS=$client_configs"); 
            } else{
                logcomment("CLIENT$client_num=$client_name"); 
            }
            $client_num++;
        }
        if ($main::CLIENTS eq '') {
            # If no CLIENTS value, default to all CLIENTn values
            if (@all_clients == 0) {
                param('CLIENT','-default', "");
                if ($main::CLIENT eq '') {
                    ExitConfigError("Either CLIENTS, CLIENT or CLIENT1 must be defined if LUN_LIST is to be initialized");
                } else {
                    param('CLIENTS','-set', "$main::CLIENT");
                    print("Setting CLIENTS=$main::CLIENT\n"); #TRACE
                }
            } else {
                my $clients_list = join(':',@all_clients);
                @clients = @all_clients;
                param('CLIENTS','-set', "$clients_list");
                print("Setting CLIENTS=$clients_list \nfrom found CLIENTn parameters\n"); #TRACE
            }
            push(@params,"CLIENTS=$main::CLIENTS");
            # CLIENTS list should now have specific host names of each client
         } else {
            # Parse CLIENTS definition to resolve to clients in this test bed
            # Limit client selection to only those in this test bed
            my @available_clients = @all_clients;
            my @allocated_clients = ();
            # Parse each colon separated value
            my @client_requests = split(':',$main::CLIENTS);
            # expand requests for multiple clients of like configuration types
            my @new_client_requests = ();
            foreach my $client_request (@client_requests) {
                # If value is enclosed in quotes
                if ($client_request =~ m/^['"].*['"]$/) {
                    $client_request =~ s/^['"]//;
                    $client_request =~ s/['"]$//;
                }
                # If value is enclosed in parentheses
                if ($client_request =~ m/^\(.*\)$/) {
                    # Parse the ([number,][configuration_type,...]) list
                    $client_request =~ s/^\(//;
                    $client_request =~ s/\)$//;
                    my @client_configs = split(',',$client_request);
                    if ($client_configs[0] =~ m/^\d+$/) {
                        my $repeat_count = shift(@client_configs);
                        $client_request = join(',',@client_configs);
                        while ($repeat_count>0) {
                            # Replace it with [number] of client requests that each match all specified configuration_type values
                            $repeat_count--;
                            push(@new_client_requests,"($client_request)");
                        }
                    } else {
                        if ($client_request =~ m/^\?$|^\?,/) {
                            # Question mark is a request for one such client if it exists,
                            # so put the request back (picks ONE such client)
                            push(@new_client_requests,"($client_request)");
                        } else {
                            # No [number], and no question mark for optional request,
                            # so request is for all such clients
                            # Put one entry on the stack for each remaining client
                            my $max_optional_clients = scalar(@all_clients) - scalar(@allocated_clients);
                            for (my $i = 1; $i <= $max_optional_clients; $i++) {
                                push(@new_client_requests,"(?,$client_request)");
                            }
                        }
                    }
                } else {
                    # Add the next client name to the list
                    # If CLIENTn was defined, add it to the list of clients
                    my $meta = quotemeta($client_request);
                    if (0 == grep(/^$meta$/,@all_clients)) {
                        my $clients_list = join(',',@all_clients);
                        logcomment("Test bed clients include: $clients_list");
                        ExitConfigError("Client $client_request, specified in the CLIENTS parameter, is not part of this test bed");
                    } else {
                        if (0 == grep(/^$meta$/,@allocated_clients)) {
                            push(@new_client_requests,$client_request);
                            push(@allocated_clients,$client_request);
                        } else {
                            ExitConfigError("Client $client_request, is specified more than once in the CLIENTS parameter");
                        }
                    }
                }
            }
            if (scalar(@new_client_requests) > scalar(@available_clients)) {
                ExitNA("CLIENTS parameter asks for more clients than this test bed has");
            }
            foreach my $client_request (@new_client_requests) {
                # If value is enclosed in parentheses
                if ($client_request =~ m/^\(.*\)$/) {
                    # Parse the ([configuration_type,...]) list
                    $client_request =~ s/^\(//;
                    $client_request =~ s/\)$//;
                    my $is_optional_request = 0;
                    if ($client_request =~ m/^\?$|^\?,/) {
                        $is_optional_request = 1;
                        $client_request =~ s/^\?,*//;
                    }
                    my @client_configs = split(',',$client_request);
                    # Find a client from the available, but not allocated clients
                    my @unallocated_clients = ();
                    foreach my $available_client (@available_clients) {
                       my $meta = quotemeta($available_client);
                       if (0 == grep(/^$meta$/,@allocated_clients)) {
                          push(@unallocated_clients,$available_client);
                       }
                    }
                    # See if we can find a client to match all requested configuration types
                    my $got_client = 0;
                    foreach my $unallocated_client (@unallocated_clients) {
                        # Find out which CLIENTn this is
                        my $this_clientn = client_param_get($unallocated_client,1);
                        # Determine its configuration types
                        my $raw_configs = ${"main::".$this_clientn."_CONFIGS"};
                        $raw_configs =~ s/['"]//g; # remove possible enclosing quotes
                        my @this_clientn_configs = split(',',$raw_configs);
                        # See if all requested types are present
                        $got_client = 1;  # Assume all are there, look for first missing one
                        foreach my $wanted_config_type (@client_configs) {
                            my $meta = quotemeta($wanted_config_type);
                            if (0 == grep(/^$meta$/,@this_clientn_configs)) {
                                $got_client = 0;
                                #print("Client $this_clientn does not have configuration type: $wanted_config_type\n");#DEBUG
                                last;
                            }
                        }
                        if ($got_client) {
                            push(@clients,$unallocated_client);
                            push(@allocated_clients,$unallocated_client);
                            last;
                        }
                    }
                    if (!$got_client and !$is_optional_request) {
                        ExitNA("No client available for CLIENTS ($client_request) configuration types on this test bed");
                    }
                } else {
                    # Add the next client name to the list
                    # If CLIENTn was defined, add it to the list of clients
                    push(@clients,$client_request);
                }
            }

            my $clients_list = join(':',@clients);
            param('CLIENTS','-set', "$clients_list");
            print("Setting CLIENTS=$clients_list \nfor clients selected by configuration types\n"); #TRACE

            # Establish the number of clients to be used
            my $client_num = scalar(@clients);

            if ($client_num == 0) {
                ExitNA("No clients selected for this test");
            }
            param('CLIENT_NUM','-set',$client_num);
            print("Setting CLIENT_NUM=$client_num\n"); #TRACE

         }
         if ($main::LUN_LIST ne '') {
             param('PROTOCOL',         '-default', (''));  # If a protocol is specified, do not process other protocols
             foreach my $client (@clients) {
                 # Determine if this is an iSCSI or FCP client
                 # Find out which CLIENTn this is
                 my $this_clientn = client_param_get($client,1);
                 # Determine its configuration types
                 param($this_clientn."_CONFIGS",'-default',undef);
                 param($this_clientn."_ISCSI_WWNN",'-default',undef);
                 param($this_clientn."_INIT1_WWPN",'-default',undef);
                 my $iscsi_wwnn;
                 if (defined(${"main::".$this_clientn."_ISCSI_WWNN"})) {
                     $iscsi_wwnn = ${"main::".$this_clientn."_ISCSI_WWNN"};
                 } else {
                     $iscsi_wwnn = '';
                 }
                 my $fcp_wwpn;
                 if (defined(${"main::".$this_clientn."_INIT1_WWPN"})) {
                     $fcp_wwpn = ${"main::".$this_clientn."_INIT1_WWPN"};
                 } else {
                     $fcp_wwpn = '';
                 }
                 my $raw_configs = '';
                 if (defined(${"main::".$this_clientn."_CONFIGS"})) {
                     $raw_configs = ${"main::".$this_clientn."_CONFIGS"};
                     $raw_configs =~ s/['"]//g; # remove possible enclosing quotes
                 }

                 my @this_clientn_configs = split(',',$raw_configs);
                 if (($main::PROTOCOL eq '' or $main::PROTOCOL =~ m/fcp/i) 
                     and (grep(/^fcp-|^init-fcp/,@this_clientn_configs) or $fcp_wwpn ne '')) {


                     push(@fcp_clients,$client);
                     print("Including CLIENT=$client in the FCP list\n"); #TRACE
                     # Determine if this is a multipath client
                     param($this_clientn."_INIT2_WWPN",'-default',undef);
                     if (defined(${"main::".$this_clientn."_INIT2_WWPN"})) {
                         push(@fcp_multipath_clients,$client);
                     }
                 } elsif (($main::PROTOCOL eq '' or $main::PROTOCOL =~ m/iscsi/i) 
                     and (grep(/^iscsi-|^init-iscsi/,@this_clientn_configs) or $iscsi_wwnn =~ m/^iqn\.|^eui\./)) {
                     push(@iscsi_clients,$client);
                     print("Including CLIENT=$client in the iSCSI list\n"); #TRACE
                 } else {
                     if ($main::PROTOCOL eq '') {
                         print("CLIENT=$client does not support FCP or iSCSI\n"); #TRACE
                     } else {
                         print("CLIENT=$client does not support PROTOCOL=$main::PROTOCOL\n"); #TRACE
                     }
                 }
             }
             if (0 == @fcp_clients and 0 == @iscsi_clients) {
                ExitNA("No SAN clients selected for LUN setup");
             }
             if ($main::LISTTESTCASES !~ m/y/i and ($main::SKIP_SETUP eq '0' or $main::SKIP_SETUP =~ m/^n/i)) {
                 # Keep track of whether or not a new LUN is being created.
                 my $new_lun_to_be_created = 0;
                 my $fcp_is_ok = 1;
                 my $iscsi_is_ok = 1;
                 # Determine default parameters based on filer configuration
                 # if filer is clustered and Veritas is present, default to MULTIPATH=1

                 my @filers = split(':',$main::FILERS);
                 foreach my $filer (@filers) {
                     my $fh = host($filer);
                     my $fc = connect("$filer#rsh-socket");

                     # Initialize FCP and iSCSI services if they are supported on this filer
                     if ($fcp_is_ok and ($main::PROTOCOL eq '' or $main::PROTOCOL =~ m/fcp/i) ) {
                         # start up FCP service if it is supported on this filer
                         $fcp_is_ok = filer_fcp_initialize($fc);
                     }
                     if ($iscsi_is_ok and ($main::PROTOCOL eq '' or $main::PROTOCOL =~ m/iscsi/i)) {
                         # start up ISCSI service if it is supported on this filer
                         $iscsi_is_ok = filer_iscsi_initialize($fc);
                     }

                     my $default_multipath = 0;
                     if (1< @filers) {
                         # More than one filer is requested
                         if (filer_cluster_is_enabled($fc)) {
                             # And this filer is clustered
                             if ($fcp_is_ok) {
                                 # And this filer has FCP
                                 if (0 < @fcp_multipath_clients) {
                                     # And there is a client with a second FCP path
                                     # Then default to use a single LUN for all paths
                                     $default_multipath = 1;
                                     # (Or, How do we tell that Veritas is installed?)
                                 }
                             }

                         }
                     }
                     param('MULTIPATH','-default', $default_multipath);
                     
                     if ($main::PROTOCOL =~ m/iscsi/i and $main::MULTIPATH eq '1') {
                         ExitScriptError(__FILE__." does not support Multipathing over iSCSI");
                     }

                     # Make sure each new lun does not already exist in the wrong place
                     my $lunids = $main::LUN_LIST;
                     # Remove parameters after any lun ids
                     $lunids =~ s/,.*:/:/g;
                     $lunids =~ s/,.*$//;
                     my @lunids = split(':',$lunids);
                     foreach my $client (@clients) {
                         my $is_iscsi_client = grep(/^$client$/,@iscsi_clients);
                         foreach my $lunid (@lunids) {
                             my $max_lunid = $lunid;
                             my $now_lunid = $lunid;
                             $max_lunid =~ s/.*-//;  
                             $now_lunid =~ s/-.*//; 
                             print("Checking LUNS $now_lunid to $max_lunid\n"); #TRACE
                             while ($now_lunid <= $max_lunid) {
                                 my $current_lun_path = undef;
                                 my $good_lun_path = undef;
                                 # The only igroup names we look for have the form:
                                 # 1) <client_name>
                                 # 2) <client_name>-<multipath_number>
                                 # 3) <client_name>-i  (for iSCSI igroups)
                                 my $path_num = undef;
                                 while (1) {
                                     # Cycle through all expected igroup names managed by these scripts
                                     # We need to look for conflicts in igroup definitions
                                     if (defined($path_num)) {
                                         if ($path_num eq '') {
                                             $path_num = 'i';
                                         } elsif ($path_num eq 'i') {
                                             $path_num = 0;
                                         } else {
                                             $path_num++;
                                             if ($path_num > 1) {
                                                 last;
                                             }
                                         }
                                     } else {
                                         $path_num = '';
                                     }
                                     my $igroup_name = $client;
                                     if ($path_num ne '') {
                                         $igroup_name = $client."-$path_num";
                                     }
                                     if (filer_igroup_is_defined($fc,$igroup_name)) {
                                         my $current_protocol = filer_igroup_is_fcp($fc,$igroup_name)? "FCP" : "iSCSI";
                                         if ($main::PROTOCOL ne '') {
                                             # A specific protocol was requested
                                             # Only consider existing initiator groups that match that request
                                             if ($main::PROTOCOL =~ m/fcp/i) {
                                                 if ($current_protocol =~ m/iSCSI/i) {
                                                     if ($path_num ne 'i') {
                                                         # The name we would use is already taken for the wrong protocol
                                                         ExitConfigError("Initiator group $igroup_name is not configured as an FCP igroup on $filer");
                                                     }
                                                     next; # This igroup has a valid name but is inappropriate for this request
                                                 }
                                             } elsif ($main::PROTOCOL =~ m/iscsi/i) {
                                                 if ($current_protocol =~ m/FCP/i) {
                                                     if ($path_num eq 'i') {
                                                         # The name we would use is already taken for the wrong protocol
                                                         ExitConfigError("Initiator group $igroup_name is not configured as an iSCSI igroup on $filer");
                                                     }
                                                     next; # This igroup has a valid name but is inappropriate for this request
                                                 }
                                             } else {
                                                 ExitScriptError("Unrecognized PROTOCOL value: $main::PROTOCOL");
                                             }
                                         }

                                         # If no specific PROTOCOL was requested,
                                         # we should use whatever exists for clients that support both
                                         $current_lun_path = filer_igroup_lunid_lun_get($fc,$igroup_name,$now_lunid);
                                         if (defined($current_lun_path)) {
                                             # Verify that the existing lunid is a mapping of the expected lun_path
                                             # even considering MULTIPATH igroup conventions used in this script
                                             # if this is an iSCSI client,
                                             #    there is to be one igroup for the iSCSI nodename named $client-i
                                             # if MULTIPATH=1 and this is an FCP client,
                                             #    there is to be one igroup for all FCP HBAs named $client
                                             # if MULTIPATH=0, there is to be one LUN for every path,
                                             #    and one igroup for every path named $client-$pathnum
                                             if ($path_num eq '' and ($main::MULTIPATH eq '0' and $is_iscsi_client == 0)) {
                                                 ExitConfigError("LUN $now_lunid on $filer at $current_lun_path, already belongs to the MULTIPATH igroup: $igroup_name");
                                             } elsif ($path_num eq 'i' and $is_iscsi_client) {
                                                 # This is OK: an iSCSI igroup for an iSCSI client
                                             } elsif ($path_num ne '' and ($main::MULTIPATH ne '0' or $is_iscsi_client == 1) ) {
                                                 # This is NOT OK: an non-iSCSI igroup for an iSCSI client
                                                 # or a separate igroup for a LUN dedicated to a separate PATH when only one LUN should be there
                                                 ExitConfigError("LUN $now_lunid on $filer at $current_lun_path, already belongs to a non-MULTIPATH igroup: $igroup_name");
                                             }

                                             # Make sure this igroup name follows the protocol conventions of this script
                                             # This is needed in order to allow both FCP and iSCSI igroups to a single client

                                             # Make sure this igroup is for a protocol supported on this client
                                             if (0 < grep(/^$client$/,@fcp_clients) and 0 < grep(/^$client$/,@iscsi_clients)) {
                                                 # This client supports both FCP and iSCSI
                                                 # Just pick the first protocol that has an initiator group.
                                                 # Remove the client from the other protocol list
                                                 if ($current_protocol =~ m/FCP/i) {
                                                     # Remove this client from the iSCSI list
                                                     my @keep;
                                                     foreach my $keep (@iscsi_clients) {
                                                         if ($keep ne $client) {
                                                             push(@keep,$keep);
                                                         }
                                                     }
                                                     @iscsi_clients = @keep;
                                                 } else {
                                                     # The igroup must be for iSCSI
                                                     # Remove this client from the FCP list
                                                     my @keep;
                                                     foreach my $keep (@fcp_clients) {
                                                         if ($keep ne $client) {
                                                             push(@keep,$keep);
                                                         }
                                                     }
                                                     @fcp_clients = @keep;
                                                 }
                                             } elsif (0 < grep(/^$client$/,@fcp_clients)) {
                                                 # This appears to be a client that only supports FCP
                                                 # Make sure this igroup is for the protocol supported on this client
                                                 if ($current_protocol !~ m/FCP/i) {
                                                     # The igroup must be for iSCSI
                                                     # But this is an FCP only client
                                                     if ($path_num ne 'i' and $path_num ne '') {
                                                         # And this was an iteration to check on the FCP igroup name
                                                         ExitConfigError("Initiator group $igroup_name is not configured as an FCP igroup on $filer");
                                                     }
                                                     next; # This igroup has a valid name but is inappropriate for this request
                                                 }
                                             } elsif (0 < grep(/^$client$/,@iscsi_clients)) {
                                                 # This appears to be a client that only supports iSCSI
                                                 # Make sure this igroup is for the protocol supported on this client
                                                 if ($current_protocol !~ m/iSCSI/i) {
                                                     # The igroup must be for FCP
                                                     # But this is an iSCSI only client
                                                     if ($path_num eq 'i') {
                                                         # And this was the iteration to check on the iSCSI igroup name
                                                         ExitConfigError("Initiator group $igroup_name is not configured as an iSCSI igroup on $filer");
                                                     }
                                                     next; # This igroup has a valid name but is inappropriate for this request
                                                 }
                                             }
                                             $good_lun_path = $current_lun_path;
                                             # continue to look for conflicts with existing initiator groups
                                         } else {
                                             print("No existing lun_path for igroup: $igroup_name using lun_id: $now_lunid\n");   #DEBUG
                                         }
                                     } else {
                                         print("No existing igroup: $igroup_name\n");  #DEBUG
                                     }
                                 }
                                 if (defined($good_lun_path)) {
                                     logcomment("LUN=$now_lunid for client $client from filer $filer is to be reused via lun_path: $good_lun_path");
                                     # Establish default LUN_PREFIX from first found existing lun_path
                                     if ($main::LUN_PREFIX eq '' or $main::LUN_PREFIX eq '*') {
                                         my $lun_vol = $good_lun_path;
                                         $lun_vol =~ s/^\/vol\///;
                                         $lun_vol =~ s/\/.*//;
                                         logcomment("Setting LUN_PREFIX=/vol/$lun_vol/ from an existing lun_path: $good_lun_path)");
                                         param('LUN_PREFIX','-set', "/vol/$lun_vol/");
                                         push(@params,"LUN_PREFIX=$main::LUN_PREFIX");
                                     } else {
                                         my $meta_prefix = quotemeta($main::LUN_PREFIX);
                                         if ($good_lun_path !~ m/$meta_prefix/) {
                                             logcomment("Waring: LUN_PREFIX=$main::LUN_PREFIX is not consistent with existing lun_path: $good_lun_path)");
                                         }
                                     }
                                 } else {
                                     logcomment("LUN=$now_lunid for client $client from filer $filer is to be created");
                                     $new_lun_to_be_created = 1;
                                 }
                                 $now_lunid++;
                             }
                         }
                     }
                 }

                 if ($new_lun_to_be_created) {
                     # Establish default LUN_PREFIX from FILER_VOL
                     if ($main::LUN_PREFIX eq '' or $main::LUN_PREFIX eq '*') {
                         logcomment("Setting LUN_PREFIX=/vol/$filer_vol/ from FILER_VOL: $filer_vol)");
                         param('LUN_PREFIX','-set', "/vol/$filer_vol/");
                         push(@params,"LUN_PREFIX=$main::LUN_PREFIX");
                     }
                     
                     # Make sure volume exists
                     my $vol = $main::LUN_PREFIX;
                     $vol =~ s#^(/vol/.*?)/.*#$1#;
                     foreach my $filer (@filers) {
                         my $fh = host($filer);
                         my $fc = connect("$filer#rsh-socket");
                         if (!filer_vol_exists($fc,$vol)) {
                             ExitConfigError("Volume $vol, needed for LUN_PREFIX=$main::LUN_PREFIX does not exist on $filer.");
                         }
                     }

                     # Make sure all Windows clients are MinWin compatible
                     foreach my $client (@clients) {
                         my $client_obj = host($client);
                         if ($client_obj->{hosttype} =~ m/windows/) {
                             if (!TestEnv::MinWin::minwin_compatible($client)) {
                                 # Give it a second chance
                                 print("MinWin compatibility failed for client: $client, trying again\n"); #DEBUG
                                 sleep(30);
                                 if (!TestEnv::MinWin::minwin_compatible($client)) {
                                     # each client must be MinWin Compatible, we cannot continue
                                     ExitConfigError("All Windows clients must be MinWin compatible. $client is not");
                                 }
                             }
                         }
                     }

                     my $id = "create_config";
                     my $subtest;
                     # Create requested LUNs on the filer(s) for each client
                     if (@fcp_clients) {
                         param('CLIENT_BIND_METHOD','-default',  'wwpn');
                         my $protocol_clients = join(':',@fcp_clients);

                         logcomment("Starting subtest to create FCP LUNS on $main::FILERS for clients: $protocol_clients with MULTIPATH=$main::MULTIPATH");
                         param('SD_CONF_MAX_LUNS','-default',  undef);
                         if (defined($main::SD_CONF_MAX_LUNS)) {
                            $subtest = subtest("host/configure.thpl", -runid, $id."_fcp",
                                               "CLIENTS=$protocol_clients",
                                               "FILERS=$main::FILERS",
                                               "MULTIPATH=$main::MULTIPATH",
                                               "LUN_PREFIX=$main::LUN_PREFIX",
                                               "LUN_LIST=$main::LUN_LIST",
                                               "CLIENT_BIND_METHOD=$main::CLIENT_BIND_METHOD",
                                               "PROTOCOL=fcp",
                                               "SD_CONF_MAX_LUNS=$main::SD_CONF_MAX_LUNS",
                                               "ACTION=create");
                         } else {
                            $subtest = subtest("host/configure.thpl", -runid, $id."_fcp",
                                               "CLIENTS=$protocol_clients",
                                               "FILERS=$main::FILERS",
                                               "MULTIPATH=$main::MULTIPATH",
                                               "LUN_PREFIX=$main::LUN_PREFIX",
                                               "LUN_LIST=$main::LUN_LIST",
                                               "CLIENT_BIND_METHOD=$main::CLIENT_BIND_METHOD",
                                               "PROTOCOL=fcp",
                                               "ACTION=create");
                         }
                     } 
                     if (@iscsi_clients) {
                        my $protocol_clients = join(':',@iscsi_clients);
                        logcomment("Starting subtest to create iSCSI LUNS on $main::FILERS for clients: $protocol_clients");
                        $subtest = subtest("host/configure.thpl", -runid, $id."_iscsi",
                                           "CLIENTS=$protocol_clients",
                                           "FILERS=$main::FILERS",
                                           "MULTIPATH=1",    # MULTIPATH=0 only applies to FCP
                                           "LUN_PREFIX=$main::LUN_PREFIX",
                                           "LUN_LIST=$main::LUN_LIST",
                                           "PROTOCOL=iscsi",
                                           "ACTION=create");
                     }
                     # Reboot the client(s) if it is required for the client to see the new lun(s)
                     $id = "client_reboot";
                     logcomment("Starting subtest to reboot clients: $main::CLIENTS if necessary to see new LUNs");
                     $subtest = subtest("host/configure.thpl", -runid, $id,
                                        "CLIENTS=$main::CLIENTS",
                                        "FILERS=$main::FILERS",
                                        "MULTIPATH=$main::MULTIPATH",
                                        "LUN_PREFIX=$main::LUN_PREFIX",
                                        "LUN_LIST=$main::LUN_LIST",
                                        "ACTION=reboot");
                 }
            }
        }
    }
    # Redefine any input parameters that have changed
    if (0 < scalar(@params)) {
       my $result_obj = NATE::Result::Param -> new([@params]);
       $result_obj -> store();
    }
}


print "\n ".__FILE__." LOADED \n";


END {
    # Handle SKIP_CLEANUP=0 requests after the script has otherwise finished
    my $save_return_code = $?;
    print("Starting END routine for ".__FILE__." with exit code $save_return_code\n");
    eval {
        config_filer_cleanup($save_return_code);
    };
    print("Finished END routine for ".__FILE__." with exit code $save_return_code\n");
    $? = $save_return_code;
}

#######################################################################################################
# Fix for burt 388966
# File not returning a true value, return value 1 should be added after END block and not before that.
# Date: 11/Dec/2010 See Burt : 388966
# Modified by : Alphine Valentine
#######################################################################################################
1;
