# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/config_filerc.pm#1 $
# 
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.
#
# lib/config_filerc.pm
## @summary Establishes NATE params for a clustered filer configuration
## @description
##
## Obtains values for the standard FILERA and FILERB configuration parameters.
## Such parameters define a clustered filer perspective of a test bed.
##
## This module also establishes the arrays:
##  <pre>
##  &nbsp; @FILERA_DAFS_VOLS
##  &nbsp; @FILERA_FCP_TARGET_NAMES 
##  &nbsp; @FILERA_FCP_TARGET_PARTNERS 
##  &nbsp; @FILERA_ISCSI_TARGET_NAMES 
##  &nbsp; @FILERA_TARGET_NAMES
##  </pre>
## from the corresponding enumerated param values.
## Similar arrays are built for FILERB's values.
##
## @synopsis
##   use config_filerc;
##
## @param FILERA			        
##  DNS name of the first Filer.
## @param FILERB			        
##  DNS name of the second Filer. All other parameters with FILERA are also defined for FILERB.
## FILER_ prefixed parameters must be the same value for both FILERA and FILERB, 
## so they are not looked for with the FILERA or FILERB prefix.
## @param FILER_BOOTIMAGE	   
##  Filer Bootimage (or alias name) to boot. If omitted, then FILER_BOOTIMAGE_TMPL must be provided.
## @param FILER_BOOTIMAGE_TMPL  
##  If FILER_BOOTIMAGE is blank, then this string is evaluated to create the bootimage string. 
##  Example: "/ethernet@2:172.25.1.10,$FILER_BOOTIMAGE_NAME, 172.25.1.129"
## @param FILER_BOOTIMAGE_NAME  
##  Filer boot image name that is substituted in the FILER_BOOTIMAGE_TMPL
## @param FILER_MODE          
##  'cf' for clustered filer, Default is 'mono'
## @param FILER_MODEL	      
##  Filer model number
## @param HOSTNAME		      
##  hostname of the Filer
##
## @param FILERA_CONN	       
##  Filer connection mode, default is $FILER.#rsh
## @param FILERA_IP and FILERA_1, _2, _3, _4 		      
##  IP address of an IP interface to Filer
## @param FILERA_MASK	      
##  Net mask of interface corresponding to FILER_IP
## @param FILERA_MEDIATYPE     
##  Mediatype for the first interface, 100tx-fd is default
##
## @param FILERA_FCP_INSTALLED   "Yes" if FCP is installed on the filer
## @param FILERA_FCP_WWNN    FCP node name of the filer
## @param FILERA_ISCSI_INSTALLED "Yes" if iSCSI is installed on the filer
## @param FILERA_ISCSI_WWNN  iSCSI node name of the filer
## 
## @param FILERA_IFS		list of interfaces on the filer,separated by comma
## @param FILERA_IPS		list of ip addresses on the filer,separated by comma
## @param FILERA_IPSPS	list of ipspaces on the filer,separated by comma
## @param FILERA_VI_ADDR The filer VI device address corresponding to the FILER_VI_DEVn value. 
## These values use the IP address decimal octet format.
## @param FILERA_VI_DEV	The filer VI device name as defined with ifconfig.
## @param FILERA_VI_NAME The filer VI device hostname as found in the host file.	
## @param FILERA_IB_ADDR The filer IB device address corresponding to the FILER_IB_DEVn value. 
## These values use the IP address decimal octet format.
## @param FILERA_IB_DEV	The filer IB device name as defined with ifconfig.
## @param FILERA_IB_NAME The filer IB device hostname as found in the host file.	
## @param FILERA_DAFS_ADDR The filer DAFS device address corresponding to the FILER_DAFS_DEVn value. 
## These values use the IP address decimal octet format.
## @param FILERA_DAFS_DEV	The filer DAFS device name as defined with ifconfig.
## @param FILERA_DAFS_NAME The filer DAFS device hostname as found in the host file.	

## @param NONNETHOST            Name of a host that is not on the FILER's directly attached network
## @param NONNETADDR            IP address of NONNETHOST
## @param MISSINGHOST           Name of a non-existent host, default is "missing"
## @param MISSINGADDR           IP address of a non-existent host, default is "127.0.0.2"
## @param PRESENTHOST           Name of a host that is on the FILER's directly attached network
## @param PRESENTADDR           IP address of PRESENTHOST
## @dependencies Hardware: 
##   FilerConfig=filer
## @author Charlie Hedstrom (hedstrom@netapp.com)
## @status public
## @see-also lib/config_base.pm
## @see-also lib/config_filer.pm
## @change 2002-04-09 hedstrom: create
## @change 2002-04-11 hedstrom: defaults
## @change 2003-07-07 hedstrom: drop extra 'use vars' statement

use strict;
use English;
use Carp;
use Tharn;
require config_filer;

# filer
filer_params_get('FILERA');
filer_params_get('FILERB');

1;

print "\n ".__FILE__." LOADED \n";
