## @summary Command Functional specification test library to get SAN ConfigAdvisor Fixed command set.
##
## $Id: //depot/prod/test/Rsierranevada/lib/configadvisor/CmdFs.pm#1 $<br>
## @description More details available in pod.
##
## These test functions are used to read command funtional specification and returun spec. of a commond.
##
## @usage
##   use CmdFs;
##
## @keywords Tools
## @dependencies N/A
##
## @author Rajiv Gupta (rajivg@netapp.com)
## @status public
##

#Package Name
##################
package CmdFs;

#Package Requirements
######################
use strict;
use Tharn;

use vars qw (@ISA);
use XML::Simple;

use San::SanLog;
use Error;

use Data::Dumper;

my $Full_lib_path;
BEGIN { use Cwd ();
        $Full_lib_path = Cwd::abs_path(__FILE__);
        $Full_lib_path =~ s/(.*)\/.*\.pm$/$1/; 
        $ENV{NATE_SCA_LIB_PATH} = $Full_lib_path;
      }

#my $USER_CMD = $ENV{'NATE_TESTDIR'} . "/lib/configadvisor/user_commands.xml";    #Default user command. 
my $USER_CMD = $ENV{NATE_SCA_LIB_PATH} . "/user_commands.xml";    #Default user command.


#
# Inherit from the Error class
#
@ISA = qw(Error SanLog Simple);

#######################################################
#####################Constructor#######################
#######################################################
## @name ::new
## @description Create a new CmdFs Objet. This will initializes the data structure for CmdFs object
## get
## @synopsis
## $obj = CmdFs(%args);
##
## @example
## $dc_obj = CmdFs->new(-file => $filename);
##
##
###########################################################
sub new () {
    my ($class, %args) = @_;

    # Declare and bless empty hash for CmdFs
    my $self = {};

    bless($self, $class);

    $self->_clear_err();

    my $filename = $USER_CMD;

    foreach (keys %args) {
        if (/-?filename$/i) {
            if (not length($filename = $args{$_})) {
                $self->_set_err("Error:CmdFs.pm:new():Command File name is not provided.");
                return $self;
            }
        }
    }
    my $xml = new XML::Simple(forcearray => 1);
    $self = $xml->XMLin($filename, KeyAttr => []);

    bless($self, $class);
    return $self;
}

# Get Command Array based on the comonenet type and command type
sub get_cmd_array {
    my ($self, $component_type, $command_type) = @_;

    my @cmd_array;

    if ((not defined $component_type) or (not defined $command_type)) {
        $self->_set_err(
                     "Error:CmdFs:get_cmd_array:One or more Paramters $component_type or $command_type is not defined");
        return;
    }

    #Sample user_command.xml

    # my $VAR1 = {
    #          'user_specified_commands' => [
    #                                       {
    #                                         'component_info' => [
    #                                                             {
    #                                                               'input_command_list' => [
    #                                                                                       {
    #                                                                                         'command' => [
    #                                                                                                      ' bootinfo -K ',
    #                                                                                                      ' bootinfo -y ',
    #                                                                                                      ' bootinfo -r ',
    #                                                                                                      ' lsattr -El sys0 -a realmem ',
    #                                                                                                      ' lsattr -l rmt0 -E'
    #                                                                                                      ]
    #                                                                                       }                                                                                     ],
    #                                                               'component_type' => 'aix',
    #                                                               'command_type' => 'cli'
    #                                                             },
    #															  {
    #                                                               'input_command_list' => [
    #                                                                                       {
    #                                                                                         'command' => [
    #                                                                                                      ' bootinfo -K ',
    #                                                                                                      ' bootinfo -y ',
    #                                                                                                      ' bootinfo -r ',
    #                                                                                                      ' lsattr -El sys0 -a realmem ',
    #                                                                                                      ' lsattr -l rmt0 -E'
    #                                                                                                      ]
    #                                                                                       }                                                                                     ],
    #                                                               'component_type' => 'aix',
    #                                                               'command_type' => 'cli'
    #                                                             },
    #                                                          ]
    #                                       }
    #                                     ],
    #          'run_fixed_command_set' => 'no'
    # }

    foreach my $cmd_fs (@{$self->{'user_specified_commands'}}) {
        foreach my $comp_info (@{$cmd_fs->{'component_info'}}) {
            if (    ($comp_info->{'component_type'} =~ /$component_type/i)
                and ($comp_info->{'command_type'} =~ /$command_type/i)) {
                @cmd_array = @{$comp_info->{'input_command_list'}}[0]->{'command'};
            }
        }
    }

    if (not @cmd_array) {
        $self->_set_err(
            "Error:CmdFs:get_cmd_array:Component type $component_type and command type $command_type not found in user command xml"
        );
        return;
    }

    if (defined $cmd_array[0]) {
        my @ext_cmd_array = @{$cmd_array[0]};
        return \@ext_cmd_array;
    } else {
        $self->_set_err("Error:CmdFs:get_cmd_array:No commands found for the specified component type $component_type");
        return;
    }
}
