## @summary DataCollector Clients test library inheritate from SanClient
##
## @description
##
## $Id: //depot/prod/test/Rsierranevada/lib/configadvisor/DCClient.pm#1 $<br>
## $DateTime: 2013/01/22 11:22:46 $<br>
## $Author: build $<br>
##
## This perl module has additional functions and properties that are required for Data collector
## For SCA client only few additional attribute are added.
##
##
## @usage
##   use DCClient;
##
## @keywords Tools
## @dependencies none
##
## @author Rajiv Gupta (rajivg@netapp.com)
## @status public
## @change 2006-09-11 rajivg: created
##

#Package Name
##################
package DCClient;

#Package Requirements
######################
use strict;
use Data::Dumper;

use Tharn;
use Data::Dumper;

#Load San Packages
############################
use San::SanClient;
use San::SanClient::ClientUtil;

use configadvisor::SCAUtil;

use vars ('@ISA', '@EXPORT');

#
# Inherit from the SanClient, Error, SanLog and ClientUtil
#
@ISA = qw(SanClient ClientUtil Error SanLog);
@PARENT::ISA = @ISA;

#######################################################
#####################Constructor#######################
#######################################################
## @name ::new
## @description Create a new Data Collector client Object. This will initializes the data structure for DataCollector client object
## get DataCollector specific information (populate data collector specific variables).
## @synopsis
## $sanclient = DCClient->new();
##
## @example
## $sanclient = DCClient->new();
##
## Description : See pod at the bottom for more details.

sub new {
    my ($class, %args) = @_;

    #Data Collector Client object

    my $self = {};
    bless $self, $class;
    $self->_clear_err();

    my ($host, $client_local);

    my $DC = 0;    #Variable to differentiate between SAN component and Data Collector.

    foreach (keys %args) {
        if (/-?DC$/) {
            if (not length($DC = $args{$_}) or $DC !~ /no|yes|1|0/i) {
                $self->sanlog('-msg', 'Assuming client is not the Data collector host');
                $DC = 0;
            } else {
                if ($DC =~ /yes|1/i) {
                    $DC = 1;
                } else {
                    $DC = 0;
                }
            }
            delete $args{'-DC'};    # To pass valid arguments to SanClient::new()
        }
        if (/-?host$/) {
            length($host = $args{$_})
              or do {
                $self->_set_err("Error: DCClient->new invalid value for \"host\":", "$args{$_}");
                return $self;
              };
        }
    }

    unless (defined $host and length $host) {
        $self->_set_err("Error: DCClient->new mandatory parameter \"-host\"", "not specified");
        return $self;
    }

    if ($DC) {
        $client_local = $host;    ##This is the data collector.
    } else {
        $self = SanClient->new(%args);
        if ($self->error()) {
            my $errmsg = join "\n", $self->errmsg();
            $self->_set_err("Error: DCClient->new", "Instantiating new client object", "$errmsg");
            return $self;
        }
        $client_local = $self->hostname();
    }

    my $c1 = &Tharn::host($client_local);
    my $c2 = $c1->{'connection_fields'};
    my @orig_conn = keys(%$c2);
    
    $self->sanlog(-msg => "Original client connection is = > $orig_conn[0]\n");

    my $access_type;
    my $dc_access_type;
    my $username;
    my $password;
    my $client_ip;
    my $client_type;

    if ($DC) {
        foreach my $access (@orig_conn) {
                if ($access =~ m/rsh/i) {
                    $access_type = $access;
                    last;
                }
            }
        $username    = $c1->{'connection_fields'}->{$access_type}->{'username'};
        $password    = $c1->{'connection_fields'}->{$access_type}->{'password'};
        $client_type = $c1->{hosttype};
        $client_ip   = $c1->{default_ip};
        $self->{connection_retries} = 5;  ##Retry connection for 5 times. Required parameter for SanClient::_connect method.
        $self->{connection} = $self->_connect($host, $username, $password, "", "THARNHOST");
        if ($self->error())
        {
        	my $errmsg = join ("\n", $self->errmsg());
        	$self->_set_err("Error:DCClient.pm:new():", "failed to connect DC client $client_ip", "$errmsg");
        }
    } else {
        if ($c1->{hosttype} =~ m/(windows)/i) {
            foreach my $access (@orig_conn) {

                if ($access =~ m/rsh/i) {
                    $access_type = $access;
                    last;
                }
            }

            if (not length $access_type) {
                my $errmsg =
                    "There are no supported access methods defined in the host file \"$client_local\" ."
                  . "access type in the host file\n. ";

                $self->_set_err("Error: DCCLIENT->new()", "Initiating client object for $client_local.", "$errmsg");
                return $self;
            }

            if ($c1->{'connection_fields'}->{$access_type}->{'username'} eq "") {
                $self->_clear_err();
                my $errmsg =
                    "Windows username and password is blank, usually it happens when rsh connection method is used."
                  . "Config Advisor required Administrator username and Password. "
                  . "With the rsh connection method in the host file provide username and password field also with rsh connection method.\n";

                $self->_set_err("Error: DCCLIENT->new()", "Create node for $client_local", "$errmsg");
                return $self;
            }    # End if ($c1->{'connection_fields'}->{$access_type}->{'username'} eq "" )

            $username    = $c1->{'connection_fields'}->{$access_type}->{'username'};
            $password    = $c1->{'connection_fields'}->{$access_type}->{'password'};
            $client_ip   = $c1->{'default_ip'};
            $access_type = "wmi";
        } else {

            foreach my $access (@orig_conn) {

                if ($access =~ m/telnet|ssh/i) {
                    $access_type = $access;
                    last;
                }
            }

            if (not length $access_type) {
                $self->_clear_err();
                my $errmsg =
                    "There are no supported access methods defined in the host file \"$client_local\" ."
                  . "Please either change it to ssh or telnet access type in the host file or add new telnet or ssh"
                  . "access type in the host file\n. ";

                $self->_set_err("Error: DCCLIENT->new()", "Initiating client object for $client_local.", "$errmsg");
                return $self;
            }

            $username  = $c1->{'connection_fields'}->{$access_type}->{'username'};
            $password  = $c1->{'connection_fields'}->{$access_type}->{'password'};
            $client_ip = $c1->{'default_ip'};

        }    # end of else not windows.

        $client_type = "solaris" if ($c1->{hosttype} =~ m/(solaris)/i);
        $client_type = "hpux"    if ($c1->{hosttype} =~ m/(hpux)/i);
        $client_type = "aix"     if ($c1->{hosttype} =~ m/(aix)/i);
        $client_type = "linux"   if ($c1->{hosttype} =~ m/(linux)/i);
        $client_type = "vmware"  if ($c1->{hosttype} =~ m/(vmware)/i);
        $client_type = "windows" if ($c1->{hosttype} =~ m/(windows)/i);

        if ($access_type =~ m/(ssh)/i) {
            $dc_access_type = "ssh";
        } elsif ($access_type =~ m/(telnet)/i) {
            $dc_access_type = "telnet";
        } elsif ($access_type =~ m/(wmi)/i) {
            $dc_access_type = "wmi";
        } else {
            $dc_access_type = "";
        }
    }

##Populate DCClient hash with DC specific values.

    $self->{dc_username}    = $username;
    $self->{dc_conn}        = $c1;
    $self->{dc_password}    = $password;
    $self->{dc_client_ip}   = $client_ip;
    $self->{dc_client_name} = $client_local;
    $self->{dc_client_type} = $client_type;
    $self->{dc_node_access} = $dc_access_type;
    $self->{DC}             = $DC;

    if ($DC) {
        $self->{conn} = $c1;    ## If this only the DC client then set the connection to execute commands.
    }

    bless($self, $class);
    return $self;
}

###############################################################################
# Subroutine Name: dir_test
# Objective:       Test if directory exist
###############################################################################
sub dir_test() {
    my ($self, $dirname) = @_;
    return $self->file_test($dirname);
}

###############################################################################
# Subroutine Name: file_test
# Objective:       Test for file
###############################################################################
sub file_test() {
    my ($self, $filename) = @_;
    $self->_clear_err();

    if ((not defined $filename) or ($filename eq "")) {
        my $errmsg = "File name is blank or not provided in DCCLIENT->file_test()";
        $self->_set_err("Error: DCCLIENT->file_test", "No File provided", "$errmsg");
        return $SCAUtil::FALSE;
    }

    if ($self->{dc_client_type} =~ m/(windows)/i) {

        return $self->_win_file_test($filename);
    } else {
        return $self->_unix_file_test($filename);
    }
}

###############################################################################
# Subroutine Name: _unix_file_test
# Objective:       Test if unix file exists
###############################################################################

sub _unix_file_test() {

    my ($self, $filename) = @_;
    $self->_clear_err();
    my $name = "_unix_file_test";

    my ($buff) = $self->execute("ls $filename");

    if ($self->error) {
        return $SCAUtil::FALSE;
    }
    return $SCAUtil::TRUE;    # File found
}

###############################################################################
# Subroutine Name: _win_file_test
# Objective:       Test if windows file exists
###############################################################################

sub _win_file_test() {

    my ($self, $filename) = @_;
    my $name = "_win_file_test";
	$self->_clear_err();
    my ($buff) = $self->execute("dir $filename");

    if ($self->error) {
        return $SCAUtil::FALSE;
    }

    foreach my $line (@$buff) {
        return $SCAUtil::FALSE if ($line =~ m/Invalid switch/i);
        return $SCAUtil::FALSE if ($line =~ m/File Not Found/i);
        return $SCAUtil::FALSE if ($line =~ m/The system cannot find/i);
        return $SCAUtil::FALSE if ($line =~ m/Access is denied/i);
    }
    return $SCAUtil::TRUE;    # File found
}

###############################################################################
# Subroutine Name: execute
# Objective:       Execute command on remote host.
###############################################################################

sub execute() {
    my ($self, $cmd, $timeout) = @_;
    my ($status, @result);
    $self->_clear_err();
    my $conn = undef;
	if ($self->{DC}) {
         $conn = $self->{connection};
	}else{
		 $conn = $self->get_connection();	
	} 
        ($status, @result) = &ClientUtil::execute($conn, $cmd, $timeout);
        
    if ($status) {
        $self->_set_err("Error: ",\@result);
        return \@result;
    }
   
   return (\@result);

#    } else {
#        ($status, @result) = $self->SanClient::execute($cmd);
#        print ntest::NTEST_STDOUT ("Inside DC client\n @result\n");
#        return ($status, @result);
#    }    
}

###############################################################################
# Subroutine Name: create_directory
# Objective:       Create remote directory.
###############################################################################

sub create_directory() {

    my ($self, $dir) = @_;
    $self->_clear_err();
    unless (defined $dir and length $dir) {
        $self->_set_err("Error: DCClient->create_directory - directory is mandatory parameter", "not specified");
        return $SCAUtil::FALSE;
    }

    my $dir_cmd;

    # Check Windows format directory and update \\

    $dir =~ s/(\\\\+)/\\/g;    ### Remove multiple '\' if exists, for windows

    #First test if directory is already exists and writable.
    if ($self->dir_test($dir)) {
        $self->sanlog(-msg => "Directory $dir already exists, checking if writable or not");
        if ($self->is_dir_writable($dir)) {
            $self->sanlog(-msg => "Directory $dir already exists, and writable");
            return $SCAUtil::TRUE;
        } else {
            $self->_set_err("Error: DCClient->create_directory", "Directory $dir is not writable");
            return $SCAUtil::FALSE;
        }

    } else {

        return $SCAUtil::FALSE if ($dir =~ /\w:$/);    #return if reached to based directory like c: or x:
              #Check for parent directory exist or not if not then try to create it
        my $dir_priv_level = $dir;
        $dir_priv_level =~ s/[\\\/]+$//;                       #Remove \ from the end of the string.
        $dir_priv_level =~ s/(\w:)(.*)([\\\/]+)(.*)$/$1$2/;    #get last but one directory.

        if (!$self->create_directory($dir_priv_level)) {
            $self->_set_err("Error: DCClient->create_directory", "Cannot create directory $dir_priv_level");
            return $SCAUtil::FALSE;
        }
    }

    $dir_cmd = "mkdir " . $dir;

    my ($results) = $self->execute($dir_cmd);

    if ($self->error) {
        my $err_msg = "Directory $dir not create the error message is \n @$results";
        $self->_set_err("Error: DCClient->create_directory", $err_msg);
        return $SCAUtil::FALSE;
    } else {
        $self->sanlog(-msg => "Directory created");
        return $SCAUtil::TRUE;
    }
}

###############################################################################
# Subroutine Name: is_dir_writable
# Objective:       Check if directory is writable by writing a file in it.
###############################################################################

sub is_dir_writable() {
    my ($self, $dir) = @_;
    $self->_clear_err();
    my $conn = $self->{'dc_conn'};
    my $file;

    my $tmp_file_name = &SCAUtil::get_timestamp();

    if ($self->{dc_client_type} =~ m/(windows)/i) {
        $file = $dir;
        $file =~ s/(.*)$/$1\\$tmp_file_name/;
    } else {
        $file = $dir;
        $file =~ s/(.*)$/$1\/$tmp_file_name/;
    }

    if ($self->create_remote_file($conn, $file)) {
        $self->sanlog(-msg => "Directory $dir is writable");
        $self->remove_remote_file($conn, $file);
        return $SCAUtil::TRUE;
    } else {
        $self->sanlog(-msg => "Directory $dir is not writable");
        return $SCAUtil::FALSE;
    }
}

###############################################################################
# Subroutine Name: remove_remote_file
# Objective:       delete file on remote machine
###############################################################################
sub remove_remote_file {

    my ($self, $connection, $rem_file) = @_;
    $self->_clear_err();
    
    my ($output, $status);

    if ($connection->{hosttype} =~ /windows/i) {
        $output = $self->execute("del $rem_file");
    } else {
        $output = $self->execute("rm -f $rem_file");
    }

    if ($self->error) {
        $self->sanlog(-msg => "Cannot remove remote file: $rem_file");
    }
}

###############################################################################
# Subroutine Name: create_remote_file
# Objective:       create file on remote machine
###############################################################################
sub create_remote_file {
    my ($self, $connection, $rem_file, @filebuf) = @_;
    my ($output, $status, $line);
    $self->_clear_err();
    if (not defined($rem_file)) {
        $self->sanlog(-msg, "Remote file name not specified.");
        return $SCAUtil::FALSE;
    }

    if (!(@filebuf)) {
        @filebuf = ("text");
    }

    foreach $line (@filebuf) {
        $line =~ s/\$/\\\$/g;
        $line =~ s/\"/\\\"/g;

        # need to handle Windows differently, it echo's the double
        # quotes on the string and UNIX does not.
        if ($connection->{hostp}->{hosttype} =~ /windows/i) {
            ($output) = $self->execute("echo $line >> $rem_file");
            foreach my $out (@$output) {
                return $status = -1 if ($out =~ m/Invalid switch/i);
                return $status = -1 if ($out =~ m/File Not Found/i);
                return $status = -1 if ($out =~ m/The system cannot find/i);
                return $status = -1 if ($out =~ m/Access is denied/i);
            }

        } else {
            ($output) = $self->execute("echo \"$line\" >> $rem_file");
        }

        if ($self->error) {
            my $msg_out = join('\n', @$output);
            my $err_msg = "Fail to create file on the host got message \n$msg_out\n";
            $self->_set_err("Error: DCClient->create_remote_file", "Fail to create file at remote host", $err_msg);
            return $SCAUtil::FALSE;
        }
    }
    return $SCAUtil::TRUE;
}


###############################################################################
# Subroutine Name: read_remote_file
# Objective:       Read the remote file using type (windows) or cat (unix)
###############################################################################

sub read_remote_file(){
	my ($self, $filename) = @_;
	
	my ($cmd, $out);

	if(!$self->file_test($filename)){
		$self->_set_err("Remote file $filename not exists");
		return $SCAUtil::FALSE;
	}
	
	if ($self->{dc_client_type} =~ m/(windows)/i){
		$cmd = "type $filename";
	}else{
		$cmd = "cat $filename";
	}
	$out = $self->execute($cmd);
	
	if ($self->error) {
		my $msg_out = join('\n', @$out);
        my $err_msg = "Fail to read file on the host got message \n$msg_out\n";
        $self->_set_err("Error: DCClient->read_remote_file", "Fail to read file at remote host", $err_msg);
        return $SCAUtil::FALSE;
	}
	
	return $out;
}


###############################################################################
# Subroutine Name: copy_directory
# Objective:       Copy source directory to destination
###############################################################################
sub copy_directory {
    my ($self, $src_dir, $des_dir) = @_;
    $self->_clear_err();
    my $connection = $self->{dc_conn};

    my ($output, $status, $line);

    if (not defined($src_dir) or not defined($des_dir)) {
        $self->_set_err("Error: DCClient->copy_directory", "Source or destination directory is not provided");
        return $SCAUtil::FALSE;
    }

    if (not $self->dir_test($src_dir)) {
        $self->_set_err("Error: DCClient->copy_directory", "Source directory does not exists");
        return $SCAUtil::FALSE;
    }

    if (not $self->create_directory($des_dir)) {
        $self->_set_err("Error: DCClient->copy_directory", "Cannot create destination directory");
        return $SCAUtil::FALSE;
    }

    # need to handle Windows differently,

    if ($connection->{hosttype} =~ /windows/i) {
        ($output) = $self->execute("copy \"$src_dir\"  \"$des_dir\"");
        foreach my $out (@$output) {
            return $status = -1 if ($out =~ m/Invalid switch/i);
            return $status = -1 if ($out =~ m/File Not Found/i);
            return $status = -1 if ($out =~ m/The system cannot find/i);
            return $status = -1 if ($out =~ m/Access is denied/i);
        }

    } else {
        ($output) = $self->execute("cp -r \"$src_dir\"  \"$des_dir\"");
    }

    if ($self->error) {
        my $msg_out = join('\n', @$output);
        my $err_msg = "Fail to copy the source directory $src_dir to destination $des_dir \n$msg_out\n";
        $self->_set_err("Error: DCClient->copy_directory", "Fail to copy data collector output directory", $err_msg);
        return $SCAUtil::FALSE;
    }

    return $SCAUtil::TRUE;
}


1;

###############################################################################
###################### POD Documents###########################################
###############################################################################

=pod

=head1 Package Name

DCClient.pm

=head1 Author

Rajiv Gupta

=head1 Description

DataCollector Clients test library inheritate from SanClient

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.


B<San::SanClient.pm>

B<San::ClientUtil.pm>

B<San::SanLog.pm>

B<Error.pm>

=head1 Attributes

=head2 B<dc_username>

String = User name to access san client by data collector.

=head2 B<dc_conn>

Hash = Connection.

=head2 B<dc_password>

String = Password to access the san client by data collector.

=head2 B<dc_client_ip>

string = Client IP address.

=head2 B<dc_client_type>

String = Client type "example Windows, solaris, aix etc"

=head2 B<dc_node_access>

string = Method to access the client (ssh or telnet or wmi)

=head2 B<DC>

Number = Is this client San Config advisor data collector or the san client.

=head2 B<new(%args)>

Creates a new DCClient object.  Uses the Testbed information and creates Datacollector Client Object. As per arguments decide if the object is the SCA host or the SAN host then Initialized SCA related parameters in addition to the SanClient parameters.

=over 4

=item Inputs

=item * -DC

Provide this parameter if the client is Data Collector, valid input values are 1|0|y|n

=item * -host

Mandatory parameter for the host name. Host name should be name of the host file.

=item Outputs

Returns a DCClient object with all information required to execute data collector. 

=item Examples

 $san_client = new DCClient(
        -host => "w2k3-198-69"
        -DC   => 0    
       );

=back

=head2 b<dir_test($dir_name)>

Test if given directory exist in the client.

=over 4 

=item Inputs

=item * Directory Name

Provide fully qualified path for the directory.

=item Outputs

Returns 1 if directory exist else 0 if not.

=item Examples

if($san_client->dir_test(c:\test\cds))
{

}

=back

=head2 b<file_test($dir_name)>

Test if given file exist in the client.

=over 4 

=item Inputs

=item * File Name

Provide fully qualified name for the file.

=item Outputs

Returns 1 if file exist else 0 if not.

=item Examples

if($san_client->file_test(c:\test\cds\test.txt))
{

}

=back

=head2 b<is_dir_writable($dirname)>

Test if given directory is writable or not.

=over 4

=item Inputs

=item * Directory Name

Provide fully qualified directory name.

=item Outputs

Returns 1 if directory writable else 0 if not.

=item Examples

if($san_client->is_dir_writable($dirname)>{

}

=back

=head2 b<execute($command, $timeout)>

Execute CLI command on the host.

=item Inputs

=item * CLI Command

Command to execute. Mandatory parameter.

=item * Command timeout

Command timeout, default is 1500 (DC sometime takes long time to execute the command)

=item Outputs

Returns Command execution status and cli output.

=item Examples

my ($status, $output) = $san_client->execute('uname -a', 20);

=back 

=head2 b<create_directory($directory_name)>

Create directory on the client. Creates the directory recursive. Not uses "dir -m" options, that not work
with some hosts.

=item Inputs

=item * Directory Name

Name of the directory to be created.

=item Outputs

Return 1 if directory created else 0 if not created.

=item Examples

if($san_client->create_directory("c:\test\see\list", "this is a test" , "text entry1")) {

}

=back

=head2 b<create_remote_file($filename, @buffer)>

Create file on the client remotely. Does not create directory if directory not present. Assumes that directory already exist and create file and write strings from the buffer supplied as parameter.

=item Inputs

=item * File Name

Fully qualified file name to be created.

=item * Input buffer

Array, Array of input stream to be written in the file.

=item Outputs

Return 1 if file created else 0 if not created.

=item Examples

if($san_client->create_remote_file("c:\test\see\list")) {

}

=back

=head2 b<remove_remote_file($filename)>

Delete file on the client remotely.

=item Inputs

=item * File Name

Fully qualified file name to be deleted.

=item Outputs

Return 1 if file deleted else 0 if not created.

=item Examples

if($san_client->delete_remote_file("c:\test\see\list")) {

}

=back

=head2 b<copy_directory($source_dir $destination_dir)>

Copy Directory to destination. First create directory stucture at the destination and then copy the content from source directory.

=item Inputs

=item * Source directory to copy

Fully qualified path, check for if exisit or not. If not exisit return 0.

=item * Destination directory

Fully qualified path, create directory structure if not present. If not writable then return 0.

=item Outputs

Return 1 if directory is copied  else 0 if not copied.

=item Examples

if($san_client->copy_directory("c:\test\see\list", "s:\automation\logs")) {

}

=back

=cut

