## @summary DataCollector Filer test library inheritate from SanFiler
##
## @description
##
## $Id: //depot/prod/test/Rsierranevada/lib/configadvisor/DCFiler.pm#1 $<br>
## $DateTime: 2013/01/22 11:22:46 $<br>
## $Author: build $<br>
##
## This perl module has additional Data Collector specific functions/attributes which are not supported by SanFiler.pm
##
##
## @usage
##   use DCFiler;
##
## @keywords Tools
## @dependencies SanFiler Error SanLog
##
## @author Rajiv Gupta (rajivg@netapp.com)
## @status public
## @change 2008-09-11 rajivg: created
##

#Package Name
##################
package DCFiler;

#Package Requirements
######################
use strict;

use Tharn;

#Load San Packages
############################
use San::SanFiler;

use vars ('@ISA', '@EXPORT');

#
# Inherit from the SanFiler, SanLog and Error
#
#@ISA = qw(SanFiler Error SanLog);
@ISA = qw(San::SanFiler Error SanLog );

#######################################################
#####################Constructor#######################
#######################################################
## @name ::new
## @description Create a new Data Collector filer Object. This will initializes the data structure for DataCollector filer object
## get DataCollector specific information (populate data collector specific variables).
## @synopsis
## $sanfiler = DCFiler->new();
##
## @example
## $sanfiler = DCFiler->new();
##
## Description : See pod at the bottom for more details.

sub new {

    my ($class, %args) = @_;
    
    my $self = {};
    
   bless($self, $class);

    $self = SanFiler->new(%args);
    
    if ($self->error) {
            my $errmsg = join "\n", $self->errmsg();
            $self->sanlog(-msg => $errmsg);
            $self->_set_err("Error: DCFiler::new", "Instantiating filer object", "$errmsg");
            return $self;
        }

    my $FILER = $self->{name};

    ## If user name and password is not defined assuming defaults.
    my $ROOT = "root";
    my $PASS = "";

    my $c1 = &Tharn::host($FILER);
    my $c2 = $c1->{'connection_fields'};
    my $username;
    my $password;

    my @orig_conn = keys(%$c2);

    foreach my $access (@orig_conn) {
        if (    (defined $c1->{'connection_fields'}->{$access}->{'username'})
            and (defined $c1->{'connection_fields'}->{$access}->{'password'})) {
            $username = $c1->{'connection_fields'}->{$access}->{'username'};
            $password = $c1->{'connection_fields'}->{$access}->{'password'};
            last;
        }
    }

    if (not length($username)) {
        $self->sanlog(-msg => "User name is blank assuming \"root\"");
        $username = $ROOT;
    }

    if (not length($password)) {
        $self->sanlog(-msg => "Filer password is not defined assuming blank $PASS");
        $password = $PASS;
    }

    $self->{dc_username}    = $username;
    $self->{dc_password}    = $password;
    $self->{dc_conn}        = $c1;
    $self->{dc_filer_ip}    = $c1->{'default_ip'};
    $self->{dc_filer_type}  = "netapp-controller";
    $self->{dc_node_access} = "ontapi";


    return $self;
}

###############################################################################
###################### POD Documents###########################################
###############################################################################

=pod

=head1 Package Name

DCFiler.pm

=head1 Author

Rajiv Gupta

=head1 Description

DataCollector Filer test library inheritate from SanFiler

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.


B<San::SanFiler.pm>

B<San::SanLog.pm>

B<Error.pm>

=head1 Attributes

=head2 B<dc_username>

String = User name to access san Filer by data collector.

=head2 B<dc_conn>

Hash = Connection.

=head2 B<dc_password>

String = Password to access the san Filer by data collector.

=head2 B<dc_filer_ip>

string = Filer IP address.

=head2 B<dc_filer_type>

String = Filer type "example netapp-controller"

=head2 B<dc_node_access>

string = Method to access the Filer (ontapi)

=head2 B<new(%args)>

Creates a new DCFiler object.  Uses the Testbed information and creates Datacollector Filer Object. Other than SanFiler attribute add Data Collector related attribute.

=over 4

=item Outputs

Returns a DCFiler object with all information required to execute data collector. 

=item Examples

 $san_filer = new DCFiler();

=back

=cut

1;

