## @summary Datacollector switch lib test library inheritate from SanSwitch
##
## @description
##
## $Id: //depot/prod/test/Rsierranevada/lib/configadvisor/DCSwitch.pm#1 $<br>
## $DateTime: 2013/01/22 11:22:46 $<br>
## $Author: build $<br>
##
## This perl module has additional Data Collector specific functions/attributes which are not supported by SanSwitch.pm
##
##
## @usage
##   use DCSwitch;
##
## @keywords Tools
## @dependencies none
##
## @author Rajiv Gupta (rajivg@netapp.com)
## @status public
## @change 2006-09-11 rajivg: created
##

#Package Name
##################
package DCSwitch;

use strict;
#Load San Packages
############################
use San::SanSwitch;
use Tharn;



use vars ('@ISA', '@EXPORT');

#
# Inherit from the SanSwitch, SanLog and Error
#
@ISA = qw(SanSwitch Error SanLog);

#######################################################
#####################Constructor#######################
#######################################################
## @name ::new
## @description Create a new Data Collector switch Object. This will initializes the data structure for DataCollector switch object
## get DataCollector specific information (populate data collector specific variables).
## @synopsis
## $sanSwitch = DCSwitch->new();
##
## @example
## $sanSwitch = DCSwitch->new();
##
## Description : See pod at the bottom for more details.

sub new {

    my ($class, %args) = @_;
    
    my $self = {};
    bless ($self, $class);
    $self = SanSwitch->new(%args);

    bless($self, $class);

    my $switch_local = $self->hostname();

    my $c1 = &Tharn::host($switch_local);

    my $c2        = $c1->{'connection_fields'};
    my @orig_conn = keys(%$c2);

    my $access_type;
    my $username;
    my $password;

    foreach my $access (@orig_conn) {

        if ($access =~ m/telnet|ssh/i) {
            $access_type = $access;
            last;
        }
    }

    if (not length $access_type) {
        $self->_clear_err();
        my $errmsg = "There are no supported access methods defined in the host file \"$switch_local\" ."
          . "Please either change it to ssh or telnet access type in the host file or add new telnet or ssh access type in the host file\n. ";
        $self->_set_err("Error: DCSwitch->new()", "Initiating switch object for $switch_local.", "$errmsg");
        return $self;
    }

    $username = $c1->{'connection_fields'}->{$access_type}->{'username'};
    $password = $c1->{'connection_fields'}->{$access_type}->{'password'};

###SCA1.1 Data Collector only support SSH with Mcdata Switch. Below code will be removed if ssh will also be supported.
    if (($c1->{hosttype} =~ m/(mcdata)/i) and ($access_type != m/ssh/i)) {

        my $errmsg =
          "WARNING :: SCA 1.1.2 or below versions are not supporting Mcdata Switch with given access type : $access_type. It only supports SSH.\n
   		              Using credentials from the given access method and running the data collector on McData using ssh. If Data collector will fail with the given\n
   		              credentials then add SSH access method in the host file and run your test again\n";
        $self->sanlog(-msg => $errmsg);
    }
### Remove above block of code when SSH is supported with McData.

    my $switch_ip = $c1->{'default_ip'};

    my $dc_access_type;    ##To Get Access type exactly as required by Data Collector

    if ($access_type =~ m/(ssh)/i) {
        $dc_access_type = "ssh";
    } elsif ($access_type =~ m/(telnet)/i) {
        $dc_access_type = "telnet";
    } else {
        $dc_access_type = "";
    }

    my $switch_type = undef;

    if ($c1->{hosttype} =~ m/(brocade|brcd)/i) {
        $switch_type = "brocade";
    } elsif ($c1->{hosttype} =~ m/(mcdata)/i) {
        $switch_type = "mcdata";
        $access_type = "ssh";      #Current DC only support ssh for mcdata
    } elsif ($c1->{hosttype} =~ m/(cisco)/i) {
        $switch_type = "cisco";
    } elsif ($c1->{hosttype} =~ m/(qlogic)/i) {
        $switch_type = "qlogic";
    } else {
        $switch_type = "";
    }

    $self->{dc_conn}        = $c1;
    $self->{dc_username}    = $username;
    $self->{dc_password}    = $password;
    $self->{dc_switch_ip}   = $switch_ip;
    $self->{dc_switch_type} = $switch_type;
    $self->{dc_node_access} = $dc_access_type;

    bless($self, $class);

    return $self;
}

################################################################################
# Subroutine Name: disconnect
# Objective:       disconnect a switch object
# Details:         See POD documentation at the end of this file
################################################################################
sub disconnect (@) {
    my ($self) = @_;

    if (defined $self->{conn}) {
        # exit from the switch (gracefull) if available
        if ($self->{conn}->{hostp}{hosttype} =~ /cisco/i or
            $self->{conn}->{hostp}{hosttype} =~ /brocade/i) {
            &Tharn::say($self->{conn}, "exit", '-blind');

            # additional logout needed for admin domains
            if ($self->{admin_domain} ne "" and
                $self->{conn}->{hostp}{hosttype} =~ /brocade/i) {
                &Tharn::say($self->{conn}, "exit", '-blind');
            }
        }
        $self->sanlog(-msg => "Disconnecting Switch object '" . 
                              $self->hostname() . "'");
        &Tharn::disconnect($self->{conn}, '-ok');
        $self->{connected} = 0;
    }
}


###############################################################################
###################### POD Documents###########################################
###############################################################################

=pod

=head1 Package Name

DCSwitch.pm

=head1 Author

Rajiv Gupta

=head1 Description

DataCollector Switch test library inheritate from SanSwitch

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.


B<San::SanSwitch.pm>

B<San::SanLog.pm>

B<Error.pm>

=head1 Attributes

=head2 B<dc_username>

String = User name to access san switch by data collector.

=head2 B<dc_conn>

Hash = Connection.

=head2 B<dc_password>

String = Password to access the san Switch by data collector.

=head2 B<dc_Switch_ip>

string = Switch IP address.

=head2 B<dc_Switch_type>

String = Switch type "example brocade, Mcdata or Qlocic"

=head2 B<dc_node_access>

string = Method to access the Switch (ssh or telnet)

=head2 B<new(%args)>

Creates a new DCSwitch object.  Uses the Testbed information and creates Datacollector Switch Object. Other than SanSwitch attributes add Data Collector related attributes.

=over 4

=item Outputs

Returns a DCSwitch object with all information required to execute data collector. 

=item Examples

 $san_Switch = new DCSwitch();

=back

=cut

1;

