## Java Loader is a generic module to load the java object
## $Id: //depot/prod/test/Rsierranevada/lib/configadvisor/Java_Loader.pm#1 $<br>
##
##

package Java_Loader;

use strict;
use Tharn;
use San::SanLog;
use Error;
use Data::Dumper;

use vars qw (@ISA);

my $Full_lib_path;
my $Classpath;

BEGIN {
   if (!defined $ENV{'NATE_SCA_LIB_PATH'}){
        use Cwd ();
        $Full_lib_path = Cwd::abs_path(__FILE__);
        $Full_lib_path =~ s/(.*)\/.*\.pm$/$1/;
        $ENV{NATE_SCA_LIB_PATH} = $Full_lib_path;
    }
    $Classpath = ":.:".
		     "$ENV{'NATE_SCA_LIB_PATH'}/:".
		     "$ENV{'NATE_SCA_LIB_PATH'}/GUI/:".
             "$ENV{'NATE_SCA_LIB_PATH'}/GUI/scada/lib/plugins/:".
		     "$ENV{'NATE_SCA_LIB_PATH'}/GUI/scada/package/Cdf.jar:".
		     "$ENV{'NATE_SCA_LIB_PATH'}/GUI/scada/package/activation.jar:".
		     "$ENV{'NATE_SCA_LIB_PATH'}/GUI/scada/package/jaxb-xjc.jar:".
		     "$ENV{'NATE_SCA_LIB_PATH'}/GUI/scada/package/jaxb-impl.jar:".
		     "$ENV{'NATE_SCA_LIB_PATH'}/GUI/scada/package/jaxb1-impl.jar:".
		     "$ENV{'NATE_SCA_LIB_PATH'}/GUI/scada/package/jaxb-api.jar:".
		     "$ENV{'NATE_SCA_LIB_PATH'}/GUI/scada/package/commons-codec-1.3.jar:".
		     "$ENV{'NATE_SCA_LIB_PATH'}/GUI/scada/package/jsr173_api.jar";
    $ENV{CLASSPATH} = $Classpath;
}
## Set class path here for now move it to configuration file later.

use Inline (Java             => 'STUDY',
            AUTOSTUDY        => 1,
            CLEAN_BUILD_AREA => 1,
            PRINT_INFO       => 1,
            DEBUG            => 3,
            SHARED_JVM       => 1,
            START_JVM        => 1,
            PRIVATE          => 1,
            CLASSPATH        => $Classpath,
            STARTUP_DELAY    => 20,);

use Inline::Java qw(study_classes caught);
#
# Inherit from the Error class
#
@ISA = qw(Error SanLog);


##Java_Loader Constructor.
## Take input as class name to load and create Java Object
## Return prel reference java object.

sub new {

    my ($class, %args) = @_;
    my $java_class;
    my $self = {};
    bless($self, $class);

    foreach (keys %args) {
        if (/-?java_class$/i) {
             if (not length($java_class = $args{$_})) {
            $self->_set_err("Error:Java_Loader:new():Value for \"java_class\": is not supplied");
            return $self;
            }
        }
    }
 
    if (not length($java_class)){
       $self->_set_err("ERROR:Java_Loader:new:java_class parameter is mandatory");
       return $self;
    }
    $self->{"JAVA_CLASS"} = $java_class;
    $self->{"JAVA_OBJ"}   = $self->load($java_class);
    return $self;
}

## Subroutine to load the class.
##

sub load($) {

    my ($self, $_class_to_load) = @_;

    if ($_class_to_load eq "") {
        $self->_set_err("ERROR:Java_Loader:load:Class name supplied was blank\n");
        return 0;
    }

    my $this_class = ref($self);

    eval {
           study_classes([$_class_to_load], undef);      
     };
    if($@){
         if (caught("java.lang.Exception")){
         my $msg = $@->getMessage() ;
          $self->_set_err("ERROR::JavaLoader::Study Class error $msg");
      }
      else{
         # It wasn't a Java exception after all...
          $self->_set_err("ERROR::JavaLoader::Study Class error $@");
      }
    }

    my @class_split = split(/\./, $_class_to_load);    #parse package names
 
    unshift(@class_split, $this_class);                #Add base class name to crate loading package

    my $_inline_pkg_string = join('::', @class_split);

    &main::tstlog(-comment => "Inline package string to load : $_inline_pkg_string");

    #$_inline_pkg_string = "Java_Loader::scada::lib::plugins::Analyzer";#KEEP this comment for future refernce.
    #my $ana = new Java_Loader::scada::lib::plugins::Analyzer();

    my $ana = $_inline_pkg_string->new();
    return $ana;
}

## Subroutine call method with name and perform functions.
##
sub get_method {
    my ($self, $method, @args) = @_;
    my $output;
    
    eval {$output = $self->{'JAVA_OBJ'}->$method([@args])};
    if($@){
         if (caught("java.lang.Exception")){
         my $msg = $@->getMessage() ;
         $self->_set_err("ERROR:Java_Loader:method_call execptoin:$method",$msg); 
      }
      else{
         # It wasn't a Java exception after all...
         $self->_set_err("ERROR:Java_Loader:method_call:$method",$@); 
      }
    }
return $output;
}
1;
