## @summary SCAUtil library to have test utilities..
##
## @description
##
## $Id: //depot/prod/test/Rsierranevada/lib/configadvisor/SCAUtil.pm#1 $<br>
## $DateTime: 2013/01/22 11:22:46 $<br>
## $Author: build $<br>
##
## These functions use utility tasks for SCA.
##
## @usage
##   use SCAUtil;
##
## @keywords Tools
## @dependencies San, SanLog, Error
##
## @author Rajiv Gupta (rajivg@netapp.com)
## @status public
## @change 2006-09-11 rajivg: created
##

#Package Name
##################
package SCAUtil;

#Package Requirements
######################
use strict;
use Tharn;
use Data::Dumper;
use MIME::Base64;
use Time::Local;
use San::SanClient::ClientUtil;

$SCAUtil::TRUE  = 1;
$SCAUtil::FALSE = 0;

use vars ('@ISA', '@EXPORT');

sub mail_results() {

    my ($email_address, $subject, @mail_contents) = @_;

    if (scalar @mail_contents == 0) {
        return "There is nothing to send.";
    }

    if (not length($email_address)) {
        return "No email address provided.";
    }

    my $tmp_file_name = $main::LOGDIR . "/sca_log_" . SCAUtil->get_timestamp();

    open(FD, ">$tmp_file_name");
    print FD @mail_contents;
    close FD;

    $subject = "\"" . $subject . "\"";

    # Email results
    my $out2    = `mailx -s $subject $email_address < $tmp_file_name`;
    my $success = `echo $?`;

    if ($success != 0) {
        return $SCAUtil::FALSE;
    } else {
        return $SCAUtil::TRUE;
    }
}

# To update the remark file in given log directory and log file name
sub update_remarks_file {
	my ($logdir, $pid_name, @remarks)= @_;
	return return $SCAUtil::FALSE if (not length $logdir || not length "$pid_name");
	my $file_name = $logdir . "/" . $pid_name . ".remarks";
	
	eval{
			open(FD, ">$file_name");
			print FD @remarks;
			close FD;
	};
	if ($@){
         my $msg = $@->getMessage();
         return $msg;
	}
	return return $SCAUtil::TRUE;
}


# Perl trim function to remove whitespace from the start and end of the string
sub trim($) {
    my ($string) = @_;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
}

# Left trim function to remove leading whitespace
sub ltrim($) {
    my ($string) = @_;
    $string =~ s/^\s+//;
    return $string;
}

# Right trim function to remove trailing whitespace
sub rtrim($) {
    my ($string) = @_;
    $string =~ s/\s+$//;
    return $string;
}

#Function to return the Time stamp
sub get_timestamp() {
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
    $year = 1900 + $year;

    $mon  = $mon + 1 ;  # Somehow it is giving month one less.
    $mon  = "0" . $mon  if ($mon  =~ /^[0-9]$/);
    $mday = "0" . $mday if ($mday =~ /^[0-9]$/);
    $hour = "0" . $hour if ($hour =~ /^[0-9]$/);
    $sec  = "0" . $sec  if ($sec  =~ /^[0-9]$/);
    $min  = "0" . $min  if ($min  =~ /^[0-9]$/);

    my $timestamp = $year . $mon . $mday . "_" . $hour . $min . $sec;
    return $timestamp;
}

#Function to give difference between two timestamp

sub get_diff_time(){

my ($start, $end) = @_;

if (not defined $start || $start <= 0){
	return 0;
}
if (not defined $end || $end <= 0){
	return 0;
}
$start =~ s/"//g;
$start =~ s/\s*//g;
$end =~ s/"//g;
$end =~ s/\s*//g;

#$start = "20081211_022000";
#$end   = "20081211_053030";            
my ($year, $mon, $day, $hr, $min, $sec) = $start =~ /^(.{4})(.{2})(.{2})_(.{2})(.{2})(.{2})$/;

$mon = $mon - 1; # month range is 0...11
$day = $day - 1; # Day range is 0...31
my $start_secs += timegm($sec, $min, $hr, $day, $mon, $year);


my ($year_e, $mon_e, $day_e, $hr_e, $min_e, $sec_e) = $end =~ /^(.{4})(.{2})(.{2})_(.{2})(.{2})(.{2})$/;
$mon_e = $mon_e - 1;
$day_e = $day_e - 1;
my $end_secs += timegm ($sec_e, $min_e, $hr_e, $day_e, $mon_e, $year_e);

my $diff = $end_secs - $start_secs;
$diff=sprintf("%.2f", $diff);

my $ret = &SCAUtil::get_hrs_from_sec($diff);

return $ret;
}

#function to get hours from the second.
sub get_hrs_from_sec(){
	my ($sec) = @_;		
	if ($sec <= 0){
		return 0;
	}
	my ($hr, $min);
	if (($sec/3600) >= 1){
		my $rem = $sec % 3600;
		$hr = (($sec-$rem)/3600);
		my $sec_rem = $rem % 60;
		$min = (($rem-$sec_rem)/60);
		my $str_ret = "$hr Hours, $min Minutes, $sec_rem Seconds";
		return $str_ret;
	}else{		
		my $sec_rem = $sec % 60;
		$min = (($sec- $sec_rem)/60);
		my $str_ret = "$min Minutes, $sec_rem Seconds";
		return $str_ret;
	}
}

#Function to retrive the test controller system on which directory is mounted. Valid for only unix.

sub get_fs_unix(){
	my ($dir) = @_;
	my $df_out = `df -k $dir`;
	my @file_array = split (/\s+/,$df_out);
	return $file_array[7];
}

#Function to check existance of file on the test controller.
sub file_test(){
	my ($file) = @_;
	return $SCAUtil::TRUE if (-f $file);
	return $SCAUtil::FALSE;
}

#Function to create link on the test controller.
sub create_link(){
	my ($src,$dest) = @_;
	return `ln -s $dest $src; echo $?`;
}

#Function to compare two arrays.
sub compare_arrays {
	my ($first, $second) = @_;
	
	my $first_size = scalar @$first;
	my $second_size = scalar @$second;
	my @ret_buff; 
	
	if($first_size != $second_size){
		my @diff_array = [];
		my @diff_array_xml = [];
		#for (my $i = 0; $i < $first_size; $i++) { # Limiting the out array size to 10 elements compare to avoid any overflow.
		push @diff_array,"First 10 elements:\n";
		for (my $i = 0; $i < 10; $i++) {			
			push @diff_array, "$first->[$i]\n";
			push @diff_array_xml, "$second->[$i]\n";
		}
		    push @diff_array ,"\n";
		    push @diff_array, @diff_array_xml;
		    my $elements_10 = join "\n", @diff_array;	    
		return "Ouput Array size is not equal.The actual array/first size is $first_size and XML data/second array size is $second_size.
		        First 10 elements are:\n $elements_10";	
	}
	for (my $i = 0; $i < @$first; $i++) {
	    push @ret_buff, "Data Value is different at location $i\n\<$first->[$i] \n\>$second->[$i]" if $first->[$i] ne $second->[$i];
	}
	if (@ret_buff){
		my $ret_str = join "\n", @ret_buff;
		return $ret_str;
	}
	return 1;
}

#Function to run Zapi Command and get output.
sub filer_zapi_cmd {
	
	my ($filer, $CMD) = @_;
	
	my $server = new ZapiTest($filer->connection()->{hostp}->{default_ip}, "root", $filer->connection()->{hostp}->{password});
		
	unless (defined $server ) {
		logresult('-FATAL',": unable to connect to filer through zapi $filer->{hostp}->{hostname}");
	}
	
	# Declare variable for NaElement;
	my $e = undef;
	
	# Command has arguments
	if ($CMD =~ /^\w+\s+(.)?/){
		$e = new NaElement("system-cli");
        my $args = new NaElement("args");
        my ($cmd, @cmdargs) = split ('\s+', $CMD);
        $args->child_add(new NaElement("arg", $cmd));
        foreach (@cmdargs) {
        	$args->child_add(new NaElement("arg", "$_"));
            }
            $e->child_add($args);
	}else{
		my $comment = "Zapi: host :" . $filer->{hostp}->{hostname} . " ZAPI: " . $CMD;
		$e = new NaElement($CMD);		
	}
	
	my ($key, $value);
	my $o = $server->invoke_elem($e,"any");
	my $errno = $o->results_errno();
	my $reason = $o->results_reason();
	my $cmd_out = $o->sprint_pretty();
	
	return ($errno,$reason,$cmd_out);
}  

#Function to get the equivalant hosttype used in DC for given nate hosttype.
sub get_dc_nodetype{
	my ($hosttype) = @_;
	    my $client_type;
        $client_type = "solaris" if ($hosttype =~ m/(solaris)/i);
        $client_type = "hpux"    if ($hosttype =~ m/(hpux)/i);
        $client_type = "aix"     if ($hosttype =~ m/(aix)/i);
        $client_type = "linux"   if ($hosttype =~ m/(linux)/i);
        $client_type = "vmware"  if ($hosttype =~ m/(vmware)/i);
        $client_type = "windows" if ($hosttype =~ m/(windows)/i);
        $client_type = "brocade" if ($hosttype =~ m/(brocade|brcd)/i);
        $client_type = "mcdata" if ($hosttype =~ m/(mcdata)/i);
        $client_type = "qlogic" if ($hosttype =~ m/(qlogic)/i);
        $client_type = "netapp-controller" if ($hosttype =~ m/(filer|filer-cfe)/i);
        return $client_type;
}

#Function to convert Windows path to unix path
sub get_unix_path (){
	my ($win_path, $unix_prefix) = @_;
	
	if (not defined $unix_prefix){
		$unix_prefix = "/";
	}
	
	if ($win_path =~ m/^[A-za-z]:\\(.*)/){
		my $var = $1;
		
		#Add / after unix prifix
		if ($unix_prefix !~ /\/$/){
			$unix_prefix = $unix_prefix . "/";
		}
		#Add / before unix prifix
		if ($unix_prefix !~ /^\//){
			$unix_prefix = "/" . $unix_prefix;
		}
		
		$var =~ s/\\/\//g;
		my $unix_path = $unix_prefix . $var;
		return $unix_path;
	}else{
		return "SCAUTIL::ERROR::Supplied path $win_path is not the windows path.";
	}
	
}

1;

###############################################################################
###################### POD Documents###########################################
###############################################################################

=pod

=head1 Package Name

SCAUtil.pm

=head1 Author

Rajiv Gupta

=head1 Description

San Config Advisor utility library to perform various utility functions. Lib should be called statically, no reference required.
There is no constructor for this library.
 
=head1 Attributes

=head1 Public Methods

=head2 B<mail_results($email_address, $subject, @mail_contents)>

Email results to given email address with the supplied content.

=over 4

=item Inputs

=item * Email Address - String 

=item * Email Content - Array - Array of content to send in the email body.

=item Output

=item * Send string "Mail sent to $email_address" if email send out.

=item * Send string ""There is nothing to send." if Array is blank or not provided.

=item * Send string "No email address provided." if email address is not provided.

=back

=head2 B<trim($string)>

Perl trim function to remove whitespace from the start and end of the string

=over 4

=item Inputs

=item * String to trim

=item Output

=item *String(Trimed) 

=back

=head2 B<ltrim($string)>

Perl trim function to remove whitespace from the start of the string

=over 4

=item Inputs

=item * String to trim

=item Output

=item *String(Trimed) 

=back

=head2 B<rtrim($string)>

Perl trim function to remove whitespace from the end of the string

=over 4

=item Inputs

=item * String to trim

=item Output

=item *String(Trimed) 

=back

=head2 B<get_timestamp())>

Perl function to return time stamp string in folloing format : YYYYmmDD_HHMMSS

=over 4

=item Inputs



=item Output

=item * String 

=back

=cut

