# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/dp_limits.pm#1 $
# 
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.
#

## @summary Library of functions for limits testing
## @description
##   These functions are used by the DP limits scripts.
## @usage use dp_limits ;
## @keywords DP
## @author Jon Hayden (jhayden@netapp.com)
## @status public
## @change 2008-04-01 jhayden: Original library added

package dp_limits;

use strict;
use Tharn;
use Time::HiRes('gettimeofday');
use filer qw(filer_sysstat_get filer_service_unlicense filer_service_is_licensed filer_service_license);
use Filer::Version qw(get_ontap_version version_compare);
use NACL::C::SystemLicense;
use NACL::CS::SystemLicense;
use NATE::BaseException qw(:try);

use vars qw(
   @ISA @EXPORT
   );

require Exporter;
@ISA = ('Exporter');
@EXPORT = qw( 
	convert_time
	filer_vol_compress_start
	get_max_vsm_limit_src
	get_max_vsm_limit_dst
	get_max_legacy_vsm_limit_src
	get_max_legacy_vsm_limit_dst
	get_max_qsm_limit_src
	get_max_qsm_limit_dst
	get_max_legacy_qsm_limit_src
	get_max_legacy_qsm_limit_dst
	get_max_ssm_limit_src
	get_max_ssm_limit_dst
	get_max_vol_copy_limit_src
	get_max_vol_copy_limit_dst
	get_max_sv_limit_src
	get_max_sv_limit_dst
	get_max_legacy_sv_limit_src
	get_max_legacy_sv_limit_dst
	get_max_luns_per_node
	get_max_ndmp_sessions
	get_max_ndmp_sessions_cmode
	get_max_dump_sessions
	get_max_sm_endpoints_per_node
	get_max_sm_relationships
	get_max_sm_fanout_dp
	get_max_sm_fanout_ls
	get_max_active_endpoints
	get_max_active_endpoints_per_node
	get_max_active_flexvols_per_node
	get_max_sm_checker_sessions
	get_max_snapshots_per_volume_cmode
	get_max_vol_move_per_cluster

	get_curr_vsm_xfers_src
	get_curr_vsm_xfers_dst
	get_curr_legacy_vsm_xfers_src
	get_curr_legacy_vsm_xfers_dst
	get_curr_qsm_xfers_src
	get_curr_qsm_xfers_dst
	get_curr_legacy_qsm_xfers_src
	get_curr_legacy_qsm_xfers_dst
	get_curr_ssm_xfers_src
	get_curr_ssm_xfers_dst
	get_curr_vol_copy_xfers_src
	get_curr_vol_copy_xfers_dst
	get_curr_sv_xfers_src
	get_curr_sv_xfers_dst
	get_curr_legacy_sv_xfers_src
	get_curr_legacy_sv_xfers_dst
	get_curr_dump_sessions
	get_curr_ndmp_sessions
	get_curr_ndmp_sessions_cmode
	get_curr_sm_relationships
	get_curr_sm_xfers
	get_curr_sm_active_endpoints
	get_curr_sm_checker_sessions
	get_curr_snapshots_per_volume_cmode
	get_curr_vol_move_copy_per_cluster

	get_vsm_cost_src
	get_vsm_cost_dst
	get_legacy_vsm_cost_src
	get_legacy_vsm_cost_dst
	get_qsm_cost_src
	get_qsm_cost_dst
	get_legacy_qsm_cost_src
	get_legacy_qsm_cost_dst
	get_ssm_cost_src
	get_ssm_cost_dst
	get_vol_copy_cost_src
	get_vol_copy_cost_dst
	get_sv_cost_src
	get_sv_cost_dst
	get_legacy_sv_cost_src
	get_legacy_sv_cost_dst

	get_avail_vsm_tokens_src
	get_avail_vsm_tokens_dst
	get_avail_legacy_vsm_tokens_src
	get_avail_legacy_vsm_tokens_dst
	get_avail_qsm_tokens_src
	get_avail_qsm_tokens_dst
	get_avail_legacy_qsm_tokens_src
	get_avail_legacy_qsm_tokens_dst
	get_avail_ssm_tokens_src
	get_avail_ssm_tokens_dst
	get_avail_vol_copy_tokens_src
	get_avail_vol_copy_tokens_dst
	get_avail_sv_tokens_src
	get_avail_sv_tokens_dst
	get_avail_legacy_sv_tokens_src
	get_avail_legacy_sv_tokens_dst

	get_poss_vsm_xfers_src
	get_poss_vsm_xfers_dst
	get_poss_legacy_vsm_xfers_src
	get_poss_legacy_vsm_xfers_dst
	get_poss_qsm_xfers_src
	get_poss_qsm_xfers_dst
	get_poss_legacy_qsm_xfers_src
	get_poss_legacy_qsm_xfers_dst
	get_poss_ssm_xfers_src
	get_poss_ssm_xfers_dst
	get_poss_vol_copy_xfers_src
	get_poss_vol_copy_xfers_dst
	get_poss_sv_xfers_src
	get_poss_sv_xfers_dst
	get_poss_legacy_sv_xfers_src
	get_poss_legacy_sv_xfers_dst

	get_limits_dump
	wait_for_CPU
);

#===================================================================
# Global variables
#-------------------------------------------------------------------

#===================================================================
# filer_vol_compress_start ($$)
# Issue a 'vol compress start -a' to the requested filer and vol 
# In:
#	fc	        Filer connection
#	vol	        Volume name
#-------------------------------------------------------------------
sub filer_vol_compress_start ($$) {
    my $fc = shift;
    my $vol = shift;

    # Strip off leading '/vol/' if present
    $vol =~ s/\/vol\///;

    my $cmd = "vol compress start -a $vol";
    my $waitstring = 'Are you sure you want to proceed';
    my @results = say($fc, $cmd, $waitstring, -noprompt, -timeout => 120);
    $Log->comment("Vol compress start returned match: " . $results[0] . ", after: " . $results[1]);
    @results = say($fc, 'y', -blind);
    $Log->comment("Vol compress start returned match: " . $results[0] . ", after: " . $results[2]);
    return 1;
}

#===================================================================
# wait_for_CPU ($$$)
# Keep source and dst filers CPU below the input percentage
# In:
#	sourcefc	Source filer connection
#	dstfc		Destination filer connection
#	cpu_percentage	Percentage to keep filer below
#-------------------------------------------------------------------
sub wait_for_CPU ($$$) {
    my $sourcefc = shift;
    my $destfc = shift;
    my $cpu_percentage = shift;

    # KeepCPU below $cpu_percentage before exiting function
    my $cpu_util_src=100;
    my $cpu_util_dst=100;
    while ($cpu_util_src >80 || $cpu_util_dst > 80) {
        my %sysstat_output_src;
        my %sysstat_output_dst;
        eval {
            %sysstat_output_src = filer_sysstat_get($sourcefc,
                {
                    'FILER_SYSSTAT_INTERVAL'=> 5,
                });
            %sysstat_output_dst = filer_sysstat_get($destfc,
                {
                    'FILER_SYSSTAT_INTERVAL'=> 5,
                });
        };
        if (! $@) {
            $cpu_util_src = $sysstat_output_src{'CPU'}{'Avg'};
            $cpu_util_dst = $sysstat_output_dst{'CPU'}{'Avg'};
        } else {
            logresult("WARN", "$@");
        }
	sleep 5;
    }
}

#===================================================================
# convert_time($)
# Converts a time containing d (days), h (hours), m (minutes), and 
# s (seconds) to a time in seconds.
# Example: Converts 1h1m to 3660
#-------------------------------------------------------------------
sub convert_time ($) {
    my $duration  = shift;
    my $runlength = 0;

    if ( ( $duration =~ /s/ ) || ( $duration !~ /[a-z]/ ) ) {
        my $sectime = $duration;
        $sectime =~ s/^(\d+)s.*/$1/;
        $sectime =~ s/.*[a-z](\d+)s.*/$1/;
        $runlength += $sectime;
    }

    if ( $duration =~ /m/ ) {
        my $mintime = $duration;
        $mintime =~ s/^(\d+)m.*/$1/;
        $mintime =~ s/.*[a-z](\d+)m.*/$1/;
        $runlength += ( 60 * $mintime );
    }

    if ( $duration =~ /h/ ) {
        my $hourtime = $duration;
        $hourtime =~ s/^(\d+)h.*/$1/;
        $hourtime =~ s/.*[a-z](\d+)h.*/$1/;
        $runlength += ( 3600 * $hourtime );
    }

    if ( $duration =~ /d/ ) {
        my $daytime = $duration;
        $daytime =~ s/^(\d+)d.*/$1/;
        $daytime =~ s/.*[a-z](\d+)d.*/$1/;
        $runlength += ( 86400 * $daytime );
    }

    return ($runlength);
}

#===================================================================
# get_max_ndmp_sessions ($)
#
# The platform specific NDMP limit is psm_ndmpd_scale() + 40
# The psm_ndmpd_scale value is the largest of the 'Possible' values (as 
# shown in the 'Possible' column in the 'rsm show_limits' output) when 
# the filer has a nearstore_option license installed.
#
# In:
#   filer connection object
#
# Out:
#   the max NDMP sessions
#-------------------------------------------------------------------
sub get_max_ndmp_sessions ($) {
    my $fc = shift;

    my $fh = $fc->hostp();
    my $fn = NACL::C::SystemNode->new('hostrec' => $fh);
    my $ontap_version = get_ontap_version($fh);
    
    # First license Nearstore on filer if it is not already licensed
    my $ns_licensed = 0;
    if(version_compare('8.0', $ontap_version) < 1){
	# Check value of 'licensed_feature.nearstore.enable' option
        my $tmp_opt = NACL::CS::Options->fetch('command_interface' => $fn,
						'filter' => {'name' => 'licensed_feature.nearstore_option.enable'});
        if ($tmp_opt->{'value'} ne 'on'){
	    NACL::C::Options->option('command_interface' => $fn,
					'option-name' => 'licensed_feature.nearstore_option.enable',
					'option-value' => 'on');
	} else {
	    $ns_licensed = 1;
        }
    } else { 
	my $f_license = NACL::CS::SystemLicense->fetch(
					'command_interface' => $fn,
					'filter' => {'feature' => 'nearstore_option'},
					'allow_empty' => 'true');
	if(defined($f_license) and $f_license->{'is_licensed'}) {
	    $ns_licensed = 1;
	} else {
	    NACL::C::SystemLicense->add('command_interface' => $fn,
					'license-code' => 'SAXCNCG',
					'feature' => 'nearstore_option');
	}
    }

    my $max_sessions = 0;
    my (undef, undef, $show_limit) = say($fc,"priv set diag;rsm show_limits", -timeout, 600);
    foreach my $line (split(/\n/,$show_limit)) {
	if ($line !~ /SRC|DST/) {next;}
	my $possible_value = $line;
	$possible_value =~ s/.*( \d+)/$1/; # Get Possible value from show_limit output
	if ($possible_value > $max_sessions) {
	    $max_sessions = $possible_value;
	}
    }

    # If Nearstore wasn't licensed to begin with, put it back to that state
    if (! $ns_licensed) {
        if(version_compare('8.0', $ontap_version) < 1){
	    NACL::C::Options->option('command_interface' => $fn,
					'option-name' => 'licensed_feature.nearstore_option.enable',
					'option-value' => 'off');
	} else {
	    NACL::C::SystemLicense->delete('command_interface' => $fn,
					'feature' => 'nearstore_option');
	}
    }


    # Return highest 'Possible' value plus 40 for Max NDMP sessions
    return $max_sessions+40;

}

#===================================================================
# get_max_vsm_limit_src ($)
#
# 
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for VSM
#-------------------------------------------------------------------
sub get_max_vsm_limit_src ($) {
    my $fc = shift;
    my %max_vsm_src = get_limits($fc, "MP VSM", "SRC");
    if(%max_vsm_src and exists($max_vsm_src{'Sys_Max'})){
        return $max_vsm_src{'Sys_Max'};
    } else {
        return 0;
    }
}

#===================================================================
# get_max_vsm_limit_dst ()
#
# 
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for VSM
#-------------------------------------------------------------------
sub get_max_vsm_limit_dst ($) {
    my $fc = shift;
    my %max_vsm_dst = get_limits($fc, "MP VSM", "DST");
    if(%max_vsm_dst and exists($max_vsm_dst{'Sys_Max'})){
        return $max_vsm_dst{'Sys_Max'};
    } else {
        return 0;
    }
}

#===================================================================
# get_max_legacy_vsm_limit_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for Legacy VSM
#-------------------------------------------------------------------
sub get_max_legacy_vsm_limit_src ($) {
    my $fc = shift;
    my %max_vsm_src = get_limits($fc, "Legacy VSM", "SRC");
    if(%max_vsm_src and exists($max_vsm_src{'Sys_Max'})){
        return $max_vsm_src{'Sys_Max'};
    } else {
        return 0;
    }
}

#===================================================================
# get_max_legacy_vsm_limit_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for Legacy VSM
#-------------------------------------------------------------------
sub get_max_legacy_vsm_limit_dst ($) {
    my $fc = shift;
    my %max_vsm_dst = get_limits($fc, "Legacy VSM", "DST");
    if(%max_vsm_dst and exists($max_vsm_dst{'Sys_Max'})){
        return $max_vsm_dst{'Sys_Max'};
    } else {
        return 0;
    }
}

#===================================================================
# get_max_qsm_limit_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for QSM
#-------------------------------------------------------------------
sub get_max_qsm_limit_src ($) {
    my $fc = shift;
    my %max_qsm_src = get_limits($fc, "QSM", "SRC");
    if(%max_qsm_src and exists($max_qsm_src{'Sys_Max'})){
        return $max_qsm_src{'Sys_Max'};
    } else {
        return 0;
    }
}

#===================================================================
# get_max_qsm_limit_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for QSM
#-------------------------------------------------------------------
sub get_max_qsm_limit_dst ($) {
    my $fc = shift;
    my %max_qsm_dst = get_limits($fc, "QSM", "DST");
    if(%max_qsm_dst and exists($max_qsm_dst{'Sys_Max'})){
        return $max_qsm_dst{'Sys_Max'};
    } else {
        return 0;
    }
}

#===================================================================
# get_max_legacy_qsm_limit_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for Legacy QSM
#-------------------------------------------------------------------
sub get_max_legacy_qsm_limit_src ($) {
    my $fc = shift;
    my %max_qsm_src = get_limits($fc, "Legacy QSM", "SRC");
    if(%max_qsm_src and exists($max_qsm_src{'Sys_Max'})){
        return $max_qsm_src{'Sys_Max'};
    } else {
        return 0;
    }
}

#===================================================================
# get_max_legacy_qsm_limit_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for Legacy QSM
#-------------------------------------------------------------------
sub get_max_legacy_qsm_limit_dst ($) {
    my $fc = shift;
    my %max_qsm_dst = get_limits($fc, "Legacy QSM", "DST");
    if(%max_qsm_dst and exists($max_qsm_dst{'Sys_Max'})){
        return $max_qsm_dst{'Sys_Max'};
    } else {
        return 0;
    }
}

#===================================================================
# get_max_ssm_limit_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for SSM
#-------------------------------------------------------------------
sub get_max_ssm_limit_src ($) {
    my $fc = shift;
    my %max_ssm_src = get_limits($fc, "Sync SM", "SRC");
    if(%max_ssm_src and exists($max_ssm_src{'Sys_Max'})){
        return $max_ssm_src{'Sys_Max'};
    } else {
        return 0;
    }
}

#===================================================================
# get_max_ssm_limit_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for SSM
#-------------------------------------------------------------------
sub get_max_ssm_limit_dst ($) {
    my $fc = shift;
    my %max_ssm_dst = get_limits($fc, "Sync SM", "DST");
    if(%max_ssm_dst and exists($max_ssm_dst{'Sys_Max'})){
        return $max_ssm_dst{'Sys_Max'};
    } else {
        return 0;
    }
}

#===================================================================
# get_max_vol_copy_limit_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for VolCopy
#-------------------------------------------------------------------
sub get_max_vol_copy_limit_src ($) {
    my $fc = shift;
    my %max_vol_copy_src = get_limits($fc, "VolCopy", "SRC");

    return $max_vol_copy_src{'Sys_Max'};
}

#===================================================================
# get_max_vol_copy_limit_dst ()
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for VolCopy
#-------------------------------------------------------------------
sub get_max_vol_copy_limit_dst ($) {
    my $fc = shift;
    my %max_vol_copy_dst = get_limits($fc, "VolCopy", "DST");

    return $max_vol_copy_dst{'Sys_Max'};
}

#===================================================================
# get_max_sv_limit_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for SV
#-------------------------------------------------------------------
sub get_max_sv_limit_src ($) {
    my $fc = shift;
    my %max_qsm_src = get_limits($fc, "SV", "SRC");

    return $max_qsm_src{'Sys_Max'};
}

#===================================================================
# get_max_sv_limit_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for SV
#-------------------------------------------------------------------
sub get_max_sv_limit_dst ($) {
    my $fc = shift;
    my %max_qsm_src = get_limits($fc, "SV", "DST");

    return $max_qsm_src{'Sys_Max'};
}

#===================================================================
# get_max_legacy_sv_limit_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for Legacy SV
#-------------------------------------------------------------------
sub get_max_legacy_sv_limit_src ($) {
    my $fc = shift;
    my %max_qsm_src = get_limits($fc, "Legacy SV", "SRC");

    return $max_qsm_src{'Sys_Max'};
}

#===================================================================
# get_max_legacy_sv_limit_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for Legacy SV
#-------------------------------------------------------------------
sub get_max_legacy_sv_limit_dst ($) {
    my $fc = shift;
    my %max_qsm_src = get_limits($fc, "Legacy SV", "DST");

    return $max_qsm_src{'Sys_Max'};
}


#===================================================================
# get_max_dump_sessions ($)
#
# In:
#   filer connection object
#
# Out:
#   the max transfer limit for QSM
#-------------------------------------------------------------------
sub get_max_dump_sessions ($) {
    my $fc = shift;

    # Limit for dump sessions is hard coded at 32 regardless
    # of platforms
    return 32;
}

#===================================================================
# get_max_sm_relationships ($)
#
# There is no hard coded limit for the max SnapMirror Relationships
# per cluster.  Only documented limits exist here:
# http://wikid.netapp.com/w/Paloma_Limits
#
# In:
#   filer connection object
#
# Out:
#   the max SnapMirror Relationships limit for C-Mode
#-------------------------------------------------------------------
sub get_max_sm_relationships () {

    # For BR, the Limit is 500 
    # Max number of volumes a system can have is 500
    # so this needs to be 499 due to the root volume on each node.
    return 499;
}

#===================================================================
# get_max_sm_endpoints_per_node ()
#
# There is no hard coded limit for the max SnapMirror Relationships
# per cluster.  Only documented limits exist here:
# http://wikid.netapp.com/w/Paloma/RR.0/Limits_Requirements
#
# In:
#   filer connection object
#
# Out:
#   the max SnapMirror Relationships limit for C-Mode
#-------------------------------------------------------------------
sub get_max_sm_endpoints_per_node () {

    # For RR, the Limit is 12000 per cluster.  This translates to
    # 2x24-node clusters with 500 volumes per node:
    #    500 vols * 24 nodes is 12000, which is then mirrored to
    #    another 24 nodes with 500 vols.
    # So, the max number of volumes a node can have is 500
    # and this needs to be 498 due to the additional root and vserver
    # volumes.
    return 499;
}

#===================================================================
# get_max_luns_per_node ()
#
# There is no hard coded limit for the max luns per node.
# Only documented limits exist here:
# http://wikid.netapp.com/w/Paloma/RR.0/Limits_Requirements
#
# In:
#   filer connection object
#
# Out:
#   the max SnapMirror Relationships limit for C-Mode
#-------------------------------------------------------------------
sub get_max_luns_per_node () {

    # For RR, the Limit is 2048 luns per node.
    return 2048;
}

#===================================================================
# get_max_ndmp_sessions_cmode ($)
#
# There is no hard coded limit for the max concurrent NDMP sessions.
#
# In:
#   filer connection object
#
# Out:
#   the max Concurrent NDMP sessions limit for C-Mode
#-------------------------------------------------------------------
sub get_max_ndmp_sessions_cmode ($) {

    # Max is 40, but is achieved by doing 20 sessions on the same
    # filer.  20 DUMP plus 20 RESTORE will equal 40 NDMP. 
    # Any script calling this function needs to be aware that the
    # max DUMP sessions is 32 so you will not be able to run 40
    # DUMPS to another filer.
    #
    # Burt 430446 has further reduced the limit to 32
    return 32;
}

#===================================================================
# get_curr_ndmp_sessions_cmode ($)
#
# Get the current number of active NDMP sessions
#
# In:
#   node
#
# Out:
#   the current number of active NDMP sessions
#-------------------------------------------------------------------
sub get_curr_ndmp_sessions_cmode ($) {
    my $node = shift;

    my $curr_ndmp_sessions = 0;
    my @backup_status = get_backup_status_cmode($node);

    for(my $x=0; $x<@backup_status; $x++) {
    	if ($backup_status[$x]{'Type'} =~ /NDMP/ && $backup_status[$x]{'State'} =~ /ACTIVE/) {
	    $curr_ndmp_sessions++;
	}
    }

    return $curr_ndmp_sessions;
}

#===================================================================
# get_max_active_flexvols_per_node ()
#
# There is no hard coded limit for max active endpoints
# per cluster.  Only documented limits exist here:
# http://wikid.netapp.com/w/Paloma_Limits
#
# In:
#   filer connection object
#
# Out:
#   the max active SnapMirror endpoint limit 
#-------------------------------------------------------------------
sub get_max_active_flexvols_per_node () {

    # Suggested limit is 50 per node

    return 50;
}

#===================================================================
# get_max_active_endpoints ()
#
# There is no hard coded limit for max active endpoints
# per cluster.  Only documented limits exist here:
# http://wikid.netapp.com/w/Paloma_Limits
#
# In:
#   filer connection object
#
# Out:
#   the max active SnapMirror endpoint limit 
#-------------------------------------------------------------------
sub get_max_active_endpoints () {

    # Suggested limit is 100 per cluster

    return 100;
}

#===================================================================
# get_max_active_endpoints_per_node ()
#
# There is no hard coded limit for max active endpoints
# per cluster.  Documented rollingrock limits here:
# http://wikid.netapp.com/w/Paloma/RR.0/Limits_Requirements
#
# 	Suggested limit is 100 per cluster
# 	100 (for FAS6070 and above)
# 	75 (for FAS31xx, FAS6040, FAS 6030, FAS31xx)
# 	40 (lower end platforms).
# In:
#   filer connection object
#
# Out:
#   the max active SnapMirror endpoint limit 
#-------------------------------------------------------------------
sub get_max_active_endpoints_per_node (;$) {

    my $platform_level = shift;

    if ($platform_level =~ /high/i) {
	return 100;
    } elsif ($platform_level =~ /mid/i) {
	return 75;
    } elsif ($platform_level =~ /low/i) {
	return 40;
    } else {
	return 100;
    }
}

#===================================================================
# get_max_sm_checker_sessions ()
#
# There is no hard coded limit for max active endpoints
# per cluster.  Only documented limits exist here:
# http://wikid.netapp.com/w/Paloma_Limits
#
# In:
#   filer connection object
#
# Out:
#   the max active SnapMirror endpoint limit 
#-------------------------------------------------------------------
sub get_max_sm_checker_sessions () {

    # Suggested limit is 25 per node

    return 25;
}

#===================================================================
# get_max_snapshots_per_volume_cmode ()
#
# There is no hard coded limit for max active endpoints
# per cluster.  Only documented limits exist here:
# http://wikid.netapp.com/w/Paloma_Limits (boilermaker)
# http://wikid.netapp.com/w/Paloma/RR.0/Limits_Requirements (rollingrock)
#
# In:
#
# Out:
#   the max active SnapMirror endpoint limit 
#-------------------------------------------------------------------
sub get_max_snapshots_per_volume_cmode (;$) {

    my $release = shift;
    # Limit is 128 per volume in BR and 254 in RR

    if($release =~ /rr|rollingrock/i) {
	return 254;
    } else {
	return 128;
    }

}

#===================================================================
# get_max_vol_move_per_cluster ()
#
# There is no hard coded limit for max vol move.
# Only documented limits exist here:
# http://wikid.netapp.com/w/Paloma_Limits
#
# In:
#
# Out:
#   the max active SnapMirror endpoint limit 
#-------------------------------------------------------------------
sub get_max_vol_move_per_cluster (;$) {
    my $release = shift;
    # Limit is 16 per volume in BR and 25 in RR

    if($release =~ /rr|rollingrock/i) {
	return 25;
    } else {
	return 16;
    }

}

#===================================================================
# get_max_sm_fanout_ls ()
#
# There is no hard coded limit for the max fanout Relationships
# per cluster.  Only documented limits exist here:
# http://wikid.netapp.com/w/Paloma_Limits
#
# In:
#   filer connection object
#
# Out:
#   the max SnapMirror fanout limit for LS Mirrors
#-------------------------------------------------------------------
sub get_max_sm_fanout_ls () {

    # Suggested limit is 1 per node.
    #For internal testing we support a fanout of 24 on smaller clusters.

    return 24;
}

#===================================================================
# get_max_sm_fanout_dp ($)
#
# There is no hard coded limit for the max fanout Relationships
# per cluster.  Only documented limits exist here:
# http://wikid.netapp.com/w/Paloma_Limits
#
# In:
#   filer connection object
#
# Out:
#   the max SnapMirror fanout limit for DP Mirrors
#-------------------------------------------------------------------
sub get_max_sm_fanout_dp () {

    # Suggested limit is 4 per single source.

    return 4;
}

#===================================================================
# get_curr_dump_sessions ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for the 'dump' command
#-------------------------------------------------------------------
sub get_curr_dump_sessions ($) {
    my $fc = shift;
    my @curr_dump_sessions = get_limits_dump($fc);
   
    logcomment('get_curr_dump_sessions: ' . Data::Dumper->Dump(\@curr_dump_sessions));

    my $sessions=0;
    for(my $x=0;$x<@curr_dump_sessions;$x++) {
	if ($curr_dump_sessions[$x]{'state'} eq "ACTIVE" && $curr_dump_sessions[$x]{'type'} eq "dump") {
	    $sessions++;
	}
    }

    return $sessions;
}

#===================================================================
# get_curr_ndmp_sessions ($)
#
# In:
#   filer connection object
#
# Out:
#   the current sessions for the 'ndmpcopy' command
#-------------------------------------------------------------------
sub get_curr_ndmp_sessions ($) {
    my $fc = shift;
    my @curr_ndmp_sessions = get_limits_ndmpd($fc);
    my $sessions = @curr_ndmp_sessions;

    return $sessions;
}

#===================================================================
# get_curr_sm_relationships ($$)
#
# In:
#   node 
#   smtype
#
# Out:
#   the current transfers for LSM
#-------------------------------------------------------------------
sub get_curr_sm_relationships ($$) {
    my $node = shift;
    my $smtype = shift;

    my $conn = connect("$node#ssh", -timeout, 300);
    
    my $rel_count = 0;
    my (undef, undef, $relationships) = say ($conn, "rows 0; snapmirror show -fields status");
    foreach my $line (split(/\n/,$relationships)) {
	if ( $line =~ /:/ ) {
	    $rel_count++;
	}
    }

    disconnect($conn);
    return $rel_count;

}

#===================================================================
# get_curr_sm_xfers ($$)
#
# In:
#   node
#   smtype
#
# Out:
#   the current transfers for LSM
#-------------------------------------------------------------------
sub get_curr_sm_xfers ($$) {
    my $node = shift;
    my $smtype = shift;

    my $conn = connect("$node#ssh", -timeout, 300);
    
    my $xfer_count = 0;
    my (undef, undef, $active) = say ($conn, "rows 0; set diag -confirmations off; snapmirror show -status Transferring", -timeout, 600);
    foreach my $line (split(/\n/,$active)) {
	if ( $line =~ /Transferring/) {
	    $xfer_count++;
	}
    }

    disconnect($conn);
    return $xfer_count;

}

#===================================================================
# get_curr_sm_checker_sessions ($)
#
# In:
#   node
#
# Out:
#   the current SM checker sessions
#-------------------------------------------------------------------
sub get_curr_sm_checker_sessions ($) {
    my $node = shift;

    my $conn = SSQA::DPHelper::connect_ssh_persist("300001", $node);
    
    my $checker_count = 0;

    my (undef, undef, $active) = say ($conn, "rows 0; set diag -confirmations off; snapmirror show -status Checking", -timeout, 3600);
    foreach my $line (split(/\n/,$active)) {
	#if ( $line =~ /entries were displayed/) {
	if ( $line =~ /Snapmirrored/i && $line =~ /checking/i) {
	#    $checker_count = $line;
	#    $checker_count =~ s/(\d+)\s+entries were displayed.*/$1/;
	    $checker_count++;
	}
    }

    $conn->destroy();
    return $checker_count;

}

#===================================================================
# get_curr_vol_move_copy_per_cluster ($)
#
# In:
#   node
#
# Out:
#   the current SM checker sessions
#-------------------------------------------------------------------
sub get_curr_vol_move_copy_per_cluster ($) {
    my $node_conn = shift;

    my $vol_move_total = 0;
    my %job_info = get_job_show_output($node_conn);
    foreach my $job (keys %job_info) {
	#logcomment("found key of $job with value of ". $job_info{$job});
        if (($job_info{$job}{'Name'} =~ /Volume Move|Volume Copy/) && ($job_info{$job}{'State'} eq "Running")) {
	    $vol_move_total++;
	}
    }
    
    return $vol_move_total;

}

#===================================================================
# get_curr_snapshots_per_volume_cmode ($$)
#
# In:
#   node connection
#   volume
#
# Out:
#   the current snapshots per volume
#-------------------------------------------------------------------
sub get_curr_snapshots_per_volume_cmode ($$) {
    my $conn = shift;
    my $volume = shift;

    
    my $snapshot_count = 0;
    my @snapshot_info=();
    my $current_snapshot=0;
    my $snapshot_show="";
    eval {
	(undef, undef, $snapshot_show) = say ($conn, "rows 0; set diag -confirmations off; snapshot show -volume $volume -instance");
	};

    if ($@) {
        return 0;
    } else {
	foreach my $line (split(/\n/,$snapshot_show)) {
	    $line =~ s/
//g;
	    if ( $line =~ /Vserver:/) {
		$current_snapshot++;
		my $field = $line;
		$field =~ s/.*Vserver: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Vserver'} = $field;
	    } elsif ($line =~ /Volume:/) {
		my $field = $line;
		$field =~ s/.*Volume: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Volume'} = $field;
	    } elsif ($line =~ /Snapshot:/ && $line !~ /7-Mode Snapshot|Constituent Snapshot/) {
		my $field = $line;
		$field =~ s/.*Snapshot: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Snapshot'} = $field;
	    } elsif ($line =~ /Snapshot Data Set ID:/) {
		my $field = $line;
		$field =~ s/.*Snapshot Data Set ID: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Snapshot Data Set ID'} = $field;
	    } elsif ($line =~ /Snapshot Master Data Set ID:/) {
		my $field = $line;
		$field =~ s/.*Snapshot Master Data Set ID: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Snapshot Master Data Set ID'} = $field;
	    } elsif ($line =~ /Creation Time:/) {
		my $field = $line;
		$field =~ s/.*Creation Time: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Creation Time'} = $field;
	    } elsif ($line =~ /Snapshot Busy:/) {
		my $field = $line;
		$field =~ s/.*Snapshot Busy: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Snapshot Busy'} = $field;
	    } elsif ($line =~ /List of Owners:/) {
		my $field = $line;
		$field =~ s/.*List of Owners: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'List of Owners'} = $field;
	    } elsif ($line =~ /Snapshot Size:/) {
		my $field = $line;
		$field =~ s/.*Snapshot Size: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Snapshot Size'} = $field;
	    } elsif ($line =~ /Percentage of Total Blocks:/) {
		my $field = $line;
		$field =~ s/.*Percentage of Total Blocks: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Percentage of Total Blocks'} = $field;
	    } elsif ($line =~ /Percentage of Used Blocks:/) {
		my $field = $line;
		$field =~ s/.*Percentage of Used Blocks: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Percentage of Used Blocks'} = $field;
	    } elsif ($line =~ /Consistency Point Count:/) {
		my $field = $line;
		$field =~ s/.*Consistency Point Count: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Consistency Point Count'} = $field;
	    } elsif ($line =~ /Internal Status:/) {
		my $field = $line;
		$field =~ s/.*Internal Status: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Internal Status'} = $field;
	    } elsif ($line =~ /Comment:/) {
		my $field = $line;
		$field =~ s/.*Comment: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Comment'} = $field;
	    } elsif ($line =~ /File System Version:/) {
		my $field = $line;
		$field =~ s/.*File System Version: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'File System Version'} = $field;
	    } elsif ($line =~ /Physical Snap ID:/) {
		my $field = $line;
		$field =~ s/.*Physical Snap ID: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Physical Snap ID'} = $field;
	    } elsif ($line =~ /Logical Snap ID:/) {
		my $field = $line;
		$field =~ s/.*Logical Snap ID: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Logical Snap ID'} = $field;
	    } elsif ($line =~ /Owner of Snapshot as Specified in Database Record:/) {
		my $field = $line;
		$field =~ s/.*Owner of Snapshot as Specified in Database Record: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Owner of Snapshot as Specified in Database Record'} = $field;
	    } elsif ($line =~ /Snapshot Tags:/) {
		my $field = $line;
		$field =~ s/.*Snapshot Tags: (.*)/$1/;
		$snapshot_info[$current_snapshot]{'Snapshot Tags'} = $field;
	    } elsif ( $line =~ /entries were displayed/) {
		$snapshot_count = $line;
		$snapshot_count =~ s/(\d+)\s+entries were displayed.*/$1/;
	    }
	}
    }

    if (wantarray) {
    	return @snapshot_info;
    } else {
	return $snapshot_count;
    }
}


#===================================================================
# get_curr_sm_active_endpoints ($$)
#
# In:
#   node
#   smtype
#
# Out:
#   the current active endpoints for LSM
#-------------------------------------------------------------------
sub get_curr_sm_active_endpoints ($$) {
    my $node = shift;
    my $smtype = shift;

    use Connectrec;
    my $conn = Connectrec->new(template=>"ssh-persist",host=>host($node),id=>"300000",timeout=>3600);
    
    my $xfer_count = 0;
    my (undef, undef, $active) = say ($conn, "set diag -confirmations off; rows 0; snapmirror show -status Transferring,Restoring,Checking", -timeout, 3600);
    foreach my $line (split(/\n/,$active)) {
	if ( $line =~ /Transferring|Restoring|Checking/) {
	    $xfer_count++;
	}
    }

    $conn->destroy();

    if ($xfer_count > 0) {
	$xfer_count--; # Remove 1 since the count includes the command line
    }
    logcomment("Found $xfer_count transfers in progress");
    return $xfer_count;

}

#===================================================================
# get_curr_vsm_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for VSM
#-------------------------------------------------------------------
sub get_curr_vsm_xfers_src ($) {
    my $fc = shift;
    my %curr_vsm_src = get_limits($fc, "MP VSM", "SRC");

    return $curr_vsm_src{'Running'};
}

#===================================================================
# get_curr_vsm_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for VSM
#-------------------------------------------------------------------
sub get_curr_vsm_xfers_dst ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "MP VSM", "DST");

    return $curr_vsm_dst{'Running'};
}

#===================================================================
# get_curr_legacy_vsm_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for Legacy VSM
#-------------------------------------------------------------------
sub get_curr_legacy_vsm_xfers_src ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "Legacy VSM", "SRC");

    return $curr_vsm_dst{'Running'};
}

#===================================================================
# get_curr_legacy_vsm_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for Legacy VSM
#-------------------------------------------------------------------
sub get_curr_legacy_vsm_xfers_dst ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "Legacy VSM", "DST");

    return $curr_vsm_dst{'Running'};
}

#===================================================================
# get_curr_qsm_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for QSM
#-------------------------------------------------------------------
sub get_curr_qsm_xfers_src ($) {
    my $fc = shift;
    my %curr_qsm_src = get_limits($fc, "QSM", "SRC");

    return $curr_qsm_src{'Running'};
}

#===================================================================
# get_curr_qsm_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for QSM
#-------------------------------------------------------------------
sub get_curr_qsm_xfers_dst ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "QSM", "DST");

    return $curr_qsm_dst{'Running'};
}

#===================================================================
# get_curr_legacy_qsm_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for Legacy QSM
#-------------------------------------------------------------------
sub get_curr_legacy_qsm_xfers_src ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "Legacy QSM", "SRC");

    return $curr_qsm_dst{'Running'};
}

#===================================================================
# get_curr_legacy_qsm_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for Legacy QSM
#-------------------------------------------------------------------
sub get_curr_legacy_qsm_xfers_dst ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "Legacy QSM", "DST");

    return $curr_qsm_dst{'Running'};
}

#===================================================================
# get_curr_ssm_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for SSM on source
#-------------------------------------------------------------------
sub get_curr_ssm_xfers_src ($) {
    my $fc = shift;
    my %curr_ssm_src = get_limits($fc, "Sync SM", "SRC");

    return $curr_ssm_src{'Running'};
}

#===================================================================
# get_curr_ssm_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for SSM on destination
#-------------------------------------------------------------------
sub get_curr_ssm_xfers_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Sync SM", "DST");

    return $curr_ssm_dst{'Running'};
}

#===================================================================
# get_curr_vol_copy_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for VolCopy
#-------------------------------------------------------------------
sub get_curr_vol_copy_xfers_src ($) {
    my $fc = shift;
    my %curr_vol_copy_src = get_limits($fc, "VolCopy", "SRC");

    return $curr_vol_copy_src{'Running'};
}

#===================================================================
# get_curr_vol_copy_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for VolCopy
#-------------------------------------------------------------------
sub get_curr_vol_copy_xfers_dst ($) {
    my $fc = shift;
    my %curr_vol_copy_dst = get_limits($fc, "VolCopy", "DST");

    return $curr_vol_copy_dst{'Running'};
}

#===================================================================
# get_curr_sv_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for SV
#-------------------------------------------------------------------
sub get_curr_sv_xfers_src ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "SV", "SRC");

    return $curr_ssm_dst{'Running'};
}

#===================================================================
# get_curr_sv_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for SV
#-------------------------------------------------------------------
sub get_curr_sv_xfers_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "SV", "DST");

    return $curr_ssm_dst{'Running'};
}

#===================================================================
# get_curr_legacy_sv_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for Legacy SV
#-------------------------------------------------------------------
sub get_curr_legacy_sv_xfers_src ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Legacy SV", "SRC");

    return $curr_ssm_dst{'Running'};
}

#===================================================================
# get_curr_legacy_sv_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the current transfers for Legacy SV
#-------------------------------------------------------------------
sub get_curr_legacy_sv_xfers_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Legacy SV", "DST");

    return $curr_ssm_dst{'Running'};
}


#===================================================================
# get_vsm_cost_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for VSM
#-------------------------------------------------------------------
sub get_vsm_cost_src ($) {
    my $fc = shift;
    my %curr_vsm_src = get_limits($fc, "MP VSM", "SRC");

    return $curr_vsm_src{'Cost'};
}

#===================================================================
# get_vsm_cost_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for VSM
#-------------------------------------------------------------------
sub get_vsm_cost_dst ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "MP VSM", "DST");

    return $curr_vsm_dst{'Cost'};
}

#===================================================================
# get_legacy_vsm_cost_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for Legacy VSM
#-------------------------------------------------------------------
sub get_legacy_vsm_cost_src ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "Legacy VSM", "SRC");

    return $curr_vsm_dst{'Cost'};
}

#===================================================================
# get_legacy_vsm_cost_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for Legacy VSM
#-------------------------------------------------------------------
sub get_legacy_vsm_cost_dst ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "Legacy VSM", "DST");

    return $curr_vsm_dst{'Cost'};
}

#===================================================================
# get_qsm_cost_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for QSM
#-------------------------------------------------------------------
sub get_qsm_cost_src ($) {
    my $fc = shift;
    my %curr_qsm_src = get_limits($fc, "QSM", "SRC");

    return $curr_qsm_src{'Cost'};
}

#===================================================================
# get_qsm_cost_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for QSM
#-------------------------------------------------------------------
sub get_qsm_cost_dst ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "QSM", "DST");

    return $curr_qsm_dst{'Cost'};
}

#===================================================================
# get_legacy_qsm_cost_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for Legacy QSM
#-------------------------------------------------------------------
sub get_legacy_qsm_cost_src ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "Legacy QSM", "SRC");

    return $curr_qsm_dst{'Cost'};
}

#===================================================================
# get_legacy_qsm_cost_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for Legacy QSM
#-------------------------------------------------------------------
sub get_legacy_qsm_cost_dst ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "Legacy QSM", "DST");

    return $curr_qsm_dst{'Cost'};
}

#===================================================================
# get_ssm_cost_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for SSM on source
#-------------------------------------------------------------------
sub get_ssm_cost_src ($) {
    my $fc = shift;
    my %curr_ssm_src = get_limits($fc, "Sync SM", "SRC");

    return $curr_ssm_src{'Cost'};
}

#===================================================================
# get_ssm_cost_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for SSM on destination
#-------------------------------------------------------------------
sub get_ssm_cost_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Sync SM", "DST");

    return $curr_ssm_dst{'Cost'};
}

#===================================================================
# get_vol_copy_cost_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for VolCopy
#-------------------------------------------------------------------
sub get_vol_copy_cost_src ($) {
    my $fc = shift;
    my %curr_vol_copy_src = get_limits($fc, "VolCopy", "SRC");

    return $curr_vol_copy_src{'Cost'};
}

#===================================================================
# get_vol_copy_cost_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for VolCopy
#-------------------------------------------------------------------
sub get_vol_copy_cost_dst ($) {
    my $fc = shift;
    my %curr_vol_copy_dst = get_limits($fc, "VolCopy", "DST");

    return $curr_vol_copy_dst{'Cost'};
}

#===================================================================
# get_sv_cost_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for SV
#-------------------------------------------------------------------
sub get_sv_cost_src ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "SV", "SRC");

    return $curr_ssm_dst{'Cost'};
}

#===================================================================
# get_sv_cost_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for SV
#-------------------------------------------------------------------
sub get_sv_cost_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "SV", "DST");

    return $curr_ssm_dst{'Cost'};
}

#===================================================================
# get_legacy_sv_cost_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for Legacy SV
#-------------------------------------------------------------------
sub get_legacy_sv_cost_src ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Legacy SV", "SRC");

    return $curr_ssm_dst{'Cost'};
}

#===================================================================
# get_legacy_sv_cost_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the cost for Legacy SV
#-------------------------------------------------------------------
sub get_legacy_sv_cost_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Legacy SV", "DST");

    return $curr_ssm_dst{'Cost'};
}


#===================================================================
# get_avail_vsm_tokens_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for VSM
#-------------------------------------------------------------------
sub get_avail_vsm_tokens_src ($) {
    my $fc = shift;
    my %curr_vsm_src = get_limits($fc, "MP VSM", "SRC");

    return $curr_vsm_src{'Avail_Tokens'};
}

#===================================================================
# get_avail_vsm_tokens_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for VSM
#-------------------------------------------------------------------
sub get_avail_vsm_tokens_dst ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "MP VSM", "DST");

    return $curr_vsm_dst{'Avail_Tokens'};
}

#===================================================================
# get_avail_legacy_vsm_tokens_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for Legacy VSM
#-------------------------------------------------------------------
sub get_avail_legacy_vsm_tokens_src ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "Legacy VSM", "SRC");

    return $curr_vsm_dst{'Avail_Tokens'};
}

#===================================================================
# get_avail_legacy_vsm_tokens_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for Legacy VSM
#-------------------------------------------------------------------
sub get_avail_legacy_vsm_tokens_dst ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "Legacy VSM", "DST");

    return $curr_vsm_dst{'Avail_Tokens'};
}

#===================================================================
# get_avail_qsm_tokens_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for QSM
#-------------------------------------------------------------------
sub get_avail_qsm_tokens_src ($) {
    my $fc = shift;
    my %curr_qsm_src = get_limits($fc, "QSM", "SRC");

    return $curr_qsm_src{'Avail_Tokens'};
}

#===================================================================
# get_avail_qsm_tokens_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for QSM
#-------------------------------------------------------------------
sub get_avail_qsm_tokens_dst ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "QSM", "DST");

    return $curr_qsm_dst{'Avail_Tokens'};
}

#===================================================================
# get_avail_legacy_qsm_tokens_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for Legacy QSM
#-------------------------------------------------------------------
sub get_avail_legacy_qsm_tokens_src ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "Legacy QSM", "SRC");

    return $curr_qsm_dst{'Avail_Tokens'};
}

#===================================================================
# get_avail_legacy_qsm_tokens_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for Legacy QSM
#-------------------------------------------------------------------
sub get_avail_legacy_qsm_tokens_dst ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "Legacy QSM", "DST");

    return $curr_qsm_dst{'Avail_Tokens'};
}

#===================================================================
# get_avail_ssm_tokens_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for SSM on source
#-------------------------------------------------------------------
sub get_avail_ssm_tokens_src ($) {
    my $fc = shift;
    my %curr_ssm_src = get_limits($fc, "Sync SM", "SRC");

    return $curr_ssm_src{'Avail_Tokens'};
}

#===================================================================
# get_avail_ssm_tokens_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for SSM on destination
#-------------------------------------------------------------------
sub get_avail_ssm_tokens_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Sync SM", "DST");

    return $curr_ssm_dst{'Avail_Tokens'};
}

#===================================================================
# get_avail_vol_copy_tokens_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for VolCopy
#-------------------------------------------------------------------
sub get_avail_vol_copy_tokens_src ($) {
    my $fc = shift;
    my %curr_vol_copy_src = get_limits($fc, "VolCopy", "SRC");

    return $curr_vol_copy_src{'Avail_Tokens'};
}

#===================================================================
# get_avail_vol_copy_tokens_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for VolCopy
#-------------------------------------------------------------------
sub get_avail_vol_copy_tokens_dst ($) {
    my $fc = shift;
    my %curr_vol_copy_dst = get_limits($fc, "VolCopy", "DST");

    return $curr_vol_copy_dst{'Avail_Tokens'};
}

#===================================================================
# get_avail_sv_tokens_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for SV
#-------------------------------------------------------------------
sub get_avail_sv_tokens_src ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "SV", "SRC");

    return $curr_ssm_dst{'Avail_Tokens'};
}

#===================================================================
# get_avail_sv_tokens_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for SV
#-------------------------------------------------------------------
sub get_avail_sv_tokens_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "SV", "DST");

    return $curr_ssm_dst{'Avail_Tokens'};
}

#===================================================================
# get_avail_legacy_sv_tokens_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for Legacy SV
#-------------------------------------------------------------------
sub get_avail_legacy_sv_tokens_src ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Legacy SV", "SRC");

    return $curr_ssm_dst{'Avail_Tokens'};
}

#===================================================================
# get_avail_legacy_sv_tokens_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the available tokens for Legacy SV
#-------------------------------------------------------------------
sub get_avail_legacy_sv_tokens_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Legacy SV", "DST");

    return $curr_ssm_dst{'Avail_Tokens'};
}


#===================================================================
# get_poss_vsm_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for VSM
#-------------------------------------------------------------------
sub get_poss_vsm_xfers_src ($) {
    my $fc = shift;
    my %curr_vsm_src = get_limits($fc, "MP VSM", "SRC");

    return $curr_vsm_src{'Possible'};
}

#===================================================================
# get_poss_vsm_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for VSM
#-------------------------------------------------------------------
sub get_poss_vsm_xfers_dst ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "MP VSM", "DST");

    return $curr_vsm_dst{'Possible'};
}

#===================================================================
# get_poss_legacy_vsm_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for Legacy VSM
#-------------------------------------------------------------------
sub get_poss_legacy_vsm_xfers_src ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "Legacy VSM", "SRC");

    return $curr_vsm_dst{'Possible'};
}

#===================================================================
# get_poss_legacy_vsm_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for Legacy VSM
#-------------------------------------------------------------------
sub get_poss_legacy_vsm_xfers_dst ($) {
    my $fc = shift;
    my %curr_vsm_dst = get_limits($fc, "Legacy VSM", "DST");

    return $curr_vsm_dst{'Possible'};
}

#===================================================================
# get_poss_qsm_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for QSM
#-------------------------------------------------------------------
sub get_poss_qsm_xfers_src ($) {
    my $fc = shift;
    my %curr_qsm_src = get_limits($fc, "QSM", "SRC");

    return $curr_qsm_src{'Possible'};
}

#===================================================================
# get_poss_qsm_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for QSM
#-------------------------------------------------------------------
sub get_poss_qsm_xfers_dst ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "QSM", "DST");

    return $curr_qsm_dst{'Possible'};
}

#===================================================================
# get_poss_legacy_qsm_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for Legacy QSM
#-------------------------------------------------------------------
sub get_poss_legacy_qsm_xfers_src ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "Legacy QSM", "SRC");

    return $curr_qsm_dst{'Possible'};
}

#===================================================================
# get_poss_legacy_qsm_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for Legacy QSM
#-------------------------------------------------------------------
sub get_poss_legacy_qsm_xfers_dst ($) {
    my $fc = shift;
    my %curr_qsm_dst = get_limits($fc, "Legacy QSM", "DST");

    return $curr_qsm_dst{'Possible'};
}

#===================================================================
# get_poss_ssm_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for SSM on source
#-------------------------------------------------------------------
sub get_poss_ssm_xfers_src ($) {
    my $fc = shift;
    my %curr_ssm_src = get_limits($fc, "Sync SM", "SRC");

    return $curr_ssm_src{'Possible'};
}

#===================================================================
# get_poss_ssm_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for SSM on destination
#-------------------------------------------------------------------
sub get_poss_ssm_xfers_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Sync SM", "DST");

    return $curr_ssm_dst{'Possible'};
}

#===================================================================
# get_poss_vol_copy_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for VolCopy
#-------------------------------------------------------------------
sub get_poss_vol_copy_xfers_src ($) {
    my $fc = shift;
    my %curr_vol_copy_src = get_limits($fc, "VolCopy", "SRC");

    return $curr_vol_copy_src{'Possible'};
}

#===================================================================
# get_poss_vol_copy_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for VolCopy
#-------------------------------------------------------------------
sub get_poss_vol_copy_xfers_dst ($) {
    my $fc = shift;
    my %curr_vol_copy_dst = get_limits($fc, "VolCopy", "DST");

    return $curr_vol_copy_dst{'Possible'};
}

#===================================================================
# get_poss_sv_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for SV
#-------------------------------------------------------------------
sub get_poss_sv_xfers_src ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "SV", "SRC");

    return $curr_ssm_dst{'Possible'};
}

#===================================================================
# get_poss_sv_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for SV
#-------------------------------------------------------------------
sub get_poss_sv_xfers_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "SV", "DST");

    return $curr_ssm_dst{'Possible'};
}

#===================================================================
# get_poss_legacy_sv_xfers_src ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for Legacy SV
#-------------------------------------------------------------------
sub get_poss_legacy_sv_xfers_src ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Legacy SV", "SRC");

    return $curr_ssm_dst{'Possible'};
}

#===================================================================
# get_poss_legacy_sv_xfers_dst ($)
#
# In:
#   filer connection object
#
# Out:
#   the possible transfers for Legacy SV
#-------------------------------------------------------------------
sub get_poss_legacy_sv_xfers_dst ($) {
    my $fc = shift;
    my %curr_ssm_dst = get_limits($fc, "Legacy SV", "DST");

    return $curr_ssm_dst{'Possible'};
}


#===================================================================
# get_limits_dump ($)
#
# Gets dump status 
#
# In:
#   fc		filer connection object
#
# Out:
#   return	Returns array of dump status info
#-------------------------------------------------------------------
sub get_limits_dump ($) {
    my ($fc) = @_;

    my (undef, undef, $dump_status) = say($fc,"backup status", -timeout, 3600);
    logcomment("get_limits_dump: match is $dump_status");

    my @dump_info=();
    my $found_result=0;
    my $id="";
    my $state="";
    my $type="";
    my $device="";
    my $start_date="";
    my $level = "";
    my $path = "";
    my $found_active_session=0;
    foreach my $line (split(/\n/,$dump_status)) {
	next if($line =~/ID|-----|backup status|^$/);
	# If line is ACTIVE, there are 7 fields
	# If line is RESERVED, there are 2 fields
	if($line =~ /RESERVED/) {
	    $line =~ s/\s+/ /g;
	    ($id, $state, $type) = split(/ /, $line);
	    push(@dump_info, {id=>$id, state=>$state, type=>$type});
	} elsif ($line =~ /ACTIVE/) {
	    # Sometimes incomplete lines are returned - skip them
	    $found_active_session++;
	    $id = $line;
	    $id =~ s/\s?(\d+)\s+\w+\s+\w+\s+.*/$1/;
	    $state = $line;
	    $state =~ s/\s?\d+\s+(\w+)\s+\w+\s+.*/$1/;
	    $type = $line;
	    $type =~ s/\s?\d+\s+\w+\s+(\w+)\s+.*/$1/;
            if($line =~ /\s?\d+\s+\w+\s+\w+\s+\w+\s+\w+\s+\d+\s+\d\d:\d\d\s+\d\s+.*/){
	        $device = $line;
	        $device =~ s/\s?\d+\s+\w+\s+\w+\s+(\w+)\s+\w+\s+\d+\s+\d\d:\d\d\s+\d\s+.*/$1/;
	        $start_date = $line;
	        $start_date =~ s/\s?\d+\s+\w+\s+\w+\s+\w+\s+(\w+\s+\d+\s+\d\d:\d\d)\s+\d\s+.*/$1/;
	        $level = $line;
	        $level =~ s/\s?\d+\s+\w+\s+\w+\s+\w+\s+\w+\s+\d+\s+\d\d:\d\d\s+(\d)\s+.*/$1/;
	        $path = $line;
	        $path =~ s/\s?\d+\s+\w+\s+\w+\s+\w+\s+\w+\s+\d+\s+\d\d:\d\d\s+\d\s+(.*)/$1/;
	    }
	    push(@dump_info, {id=>$id, state=>$state, type=>$type, device=>$device,
	    			start_date=>$start_date, level=>$level, path=>$path});
	} elsif ($line =~ /No backup activity/) {
	} else {
	}
    }
    return @dump_info;
}

#===================================================================
# get_limits_ndmpd ($)
#
# Gets max limit for requested SM type
#
# In:
#   fc		filer connection object
#
# Out:
#   return	Returns array of ndmpd status info
#-------------------------------------------------------------------
sub get_limits_ndmpd ($) {
    my ($fc) = @_;

    my (undef, undef, $ndmp_status) = say($fc,"ndmpd status", -timeout, 600);

    my @ndmp_info=();
    my $found_result=0;
    my $session_num=0;
    my $current_session="";
    my $session_state="";
    my $ndmp_version="";
    my $tape_device = "";
    my $mover_state = "";
    my $data_state = "";
    my $data_operation = "";
    my $found_session=0;
    foreach my $line (split(/\n/,$ndmp_status)) {
    	if ($line =~ /Session/) {
	    if($found_session) {
		$session_num++;
	    }
	    $current_session = $line;
	    $current_session =~ s/Session:\s+(\d+).*/$1/;
	    $ndmp_info[$session_num]{'Current Session'} = $current_session;
	    $found_session=1;
	    next;
	} elsif ($line =~ /Active/) {
	    $session_state = "Active";
	    $ndmp_info[$session_num]{'Session State'} = $session_state;
	} elsif ($line =~ /version/) {
	    $ndmp_version = $line;
	    $ndmp_version =~ s/version:\s+(\d).*/$1/;
	    $ndmp_info[$session_num]{'version'} = $ndmp_version;
	} elsif ($line =~ /tape device/) {
	    $tape_device = $line;
	    $tape_device =~ s/tape device:\s+(\w.*)/$1/;
	    $ndmp_info[$session_num]{'tape device'} = $tape_device;
	} elsif ($line =~ /mover state/) {
	    $mover_state = $line;
	    $mover_state =~ s/mover state:\s+(\w.*)/$1/;
	    $ndmp_info[$session_num]{'mover state'} = $mover_state;
	} elsif ($line =~ /data state/) {
	    $data_state = $line;
	    $data_state =~ s/data state:\s+(\w.*)/$1/;
	    $ndmp_info[$session_num]{'data state'} = $data_state;
	} elsif ($line =~ /data operation/) {
	    $data_operation = $line;
	    $data_operation =~ s/data operation:\s+(\w.*)/$1/;
	    $ndmp_info[$session_num]{'data operation'} = $data_operation;
	}
    }

    return @ndmp_info;
}

#===================================================================
# get_backup_status_cmode ($)
#
# Gets max limit for requested SM type
#
# In:
#   node	node
#
# Out:
#   return	Returns array of backup status info
#-------------------------------------------------------------------
sub get_backup_status_cmode ($) {
    my ($node) = shift;

    my $conn = Connectrec->new(template=>"ssh-persist",host=>host($node),id=>"300001",timeout=>3600);


    my (undef, undef, $backup_status) = say($conn,"run -node * backup status", -timeout, 600);
    my @backup_info;

    my ($id, $state, $type, $device, $start_date, $level, $path);
    foreach my $line (split(/\n/,$backup_status)) {
	chomp $line;
	next if $line =~ /------|Start Date/;
	next if $line !~ /\w/;
        $line =~ s/
//g;
        $line =~ s/^\s+//;

	if ($line =~ /^\d/) {
	    my $id = $line;
	    $id =~ s/(\d+)\s+.*/$1/;
	    my $state = $line;
	    $state =~ s/\d+\s+(\w+)\s+.*/$1/;
	    my $type = $line;
	    $type =~ s/\d+\s+\w+\s+(\w+)\s+.*/$1/;
	    my $device = $line;
	    $device =~ s/\d+\s+\w+\s+\w+\s+(\w+)\s+.*/$1/;
	    my $start_date = $line;
	    $start_date =~ s/\d+\s+\w+\s+\w+\s+\w+\s+(\w+\s+\d+\s+\d+:\d+)\s+.*/$1/;
	    my $level = $line;
	    $level =~ s/\d+\s+\w+\s+\w+\s+\w+\s+\w+\s+\d+\s+\d+:\d+\s+(\d+)\s+.*/$1/;
	    my $path = $line;
	    $path =~ s/\d+\s+\w+\s+\w+\s+\w+\s+\w+\s+\d+\s+\d+:\d+\s+\d+\s+(.*)/$1/;
	    push(@backup_info, {
	    		'ID' => $id,
	    		'State' => $state,
	    		'Type' => $type,
	    		'Device' => $device,
	    		'Start Date' => $start_date,
	    		'Level' => $level,
	    		'Path' => $path,
			});
	}

    }

    return @backup_info;
}

#===================================================================
# get_limits ($$$)
#
# Gets max limit for requested SM type
#
# In:
#   fc		filer connection object
#   sm_type	SM type (vsm, rsm, qsm, sv, volcopy)
#   sm_origin	Do you want max for source or destination (src, dst)
#
# Out:
#   return	Returns hash of max limits and current xfers used
#-------------------------------------------------------------------
sub get_limits ($$$) {
    my ($fc, $sm_type, $sm_origin) = @_;
    my %max_limits;

    my (undef, undef, $show_limit) = say($fc,"priv set diag;rsm show_limits", -timeout, 600);
    my $found_result=0;
    foreach my $line (split(/\n/,$show_limit)) {
    	if ($line =~ /$sm_type/i && $line =~ /$sm_origin/i) {
	    logcomment("Matched line is: $line");
	    $found_result=1;
	    # Get rid of beginning characters befor SM type
	    $line =~ s/^.*($sm_type.*)/$1/i;
	    my @sm_type_words = split(/ /, $sm_type);
	    if ($#sm_type_words > 0) {
		# Get rid of space if sm_type has 2 words
		$line =~ s/$sm_type/$sm_type_words[0]$sm_type_words[1]/;
	    }
	    my @sm_type_values = split(/\s+/, $line);
	    $max_limits{'Cost'} = $sm_type_values[2];
	    $max_limits{'Sys_Max'} = $sm_type_values[3];
	    $max_limits{'Running'} = $sm_type_values[4];
	    $max_limits{'Avail_Tokens'} = $sm_type_values[5];
	    $max_limits{'Possible'} = $sm_type_values[6];

	    logcomment(Data::Dumper->Dump([ \%max_limits ]));
	}
    }

    return %max_limits;
}

#===================================================================
# get_job_show_output ($%)
#
# Gets job show output.
#
# In:
#   node	node connection
#
# Out:
#   return	Returns hash of hashes of job info
#-------------------------------------------------------------------
sub get_job_show_output ($) {
    my $node_conn = shift;

    my $current_job="";
    my %job_info=();
    my $job_show;
    eval {
	(undef, undef, $job_show) = say ($node_conn, "rows 0; set diag -confirmations off; job show -instance");
	};

    if ($@) {
        return 0;
    } else {
	foreach my $line (split(/\n/,$job_show)) {
	    $line =~ s/
//g;
	    if ( $line =~ /Job ID:/) {
		my $field = $line;
		$field =~ s/.*Job ID: (.*)/$1/;
		$current_job = $field;
		$job_info{$current_job}{'Job ID'} = $field;
	    } elsif ($line =~ /Name:/ && $line !~ /User/) {
		my $field = $line;
		$field =~ s/.*Name: (.*)/$1/;
		$job_info{$current_job}{'Name'} = $field;
	    } elsif ($line =~ /Description:/) {
		my $field = $line;
		$field =~ s/.*Description: (.*)/$1/;
		$job_info{$current_job}{'Description'} = $field;
	    } elsif ($line =~ /Priority:/) {
		my $field = $line;
		$field =~ s/.*Priority: (.*)/$1/;
		$job_info{$current_job}{'Priority'} = $field;
	    } elsif ($line =~ /Node:/) {
		my $field = $line;
		$field =~ s/.*Node: (.*)/$1/;
		$job_info{$current_job}{'Node'} = $field;
	    } elsif ($line =~ /Affinity:/) {
		my $field = $line;
		$field =~ s/.*Affinity: (.*)/$1/;
		$job_info{$current_job}{'Affinity'} = $field;
	    } elsif ($line =~ /Schedule:/) {
		my $field = $line;
		$field =~ s/.*Schedule: (.*)/$1/;
		$job_info{$current_job}{'Schedule'} = $field;
	    } elsif ($line =~ /Queue Time:/) {
		my $field = $line;
		$field =~ s/.*Queue Time: (.*)/$1/;
		$job_info{$current_job}{'Queue Time'} = $field;
	    } elsif ($line =~ /Start Time:/) {
		my $field = $line;
		$field =~ s/.*Start Time: (.*)/$1/;
		$job_info{$current_job}{'Start Time'} = $field;
	    } elsif ($line =~ /End Time:/) {
		my $field = $line;
		$field =~ s/.*End Time: (.*)/$1/;
		$job_info{$current_job}{'End Time'} = $field;
	    } elsif ($line =~ /Drop-dead Time:/) {
		my $field = $line;
		$field =~ s/.*Drop-dead Time: (.*)/$1/;
		$job_info{$current_job}{'Drop-dead Time'} = $field;
	    } elsif ($line =~ /Restarted?:/) {
		my $field = $line;
		$field =~ s/.*Restarted?: (.*)/$1/;
		$job_info{$current_job}{'Restarted?'} = $field;
	    } elsif ($line =~ /State:/) {
		my $field = $line;
		$field =~ s/.*State: (.*)/$1/;
		$job_info{$current_job}{'State'} = $field;
	    } elsif ($line =~ /Status Code:/) {
		my $field = $line;
		$field =~ s/.*Status Code: (.*)/$1/;
		$job_info{$current_job}{'Status Code'} = $field;
	    } elsif ($line =~ /Completion String:/) {
		my $field = $line;
		$field =~ s/.*Completion String: (.*)/$1/;
		$job_info{$current_job}{'Completion String'} = $field;
	    } elsif ($line =~ /Job Type:/) {
		my $field = $line;
		$field =~ s/.*Job Type: (.*)/$1/;
		$job_info{$current_job}{'Job Type'} = $field;
	    } elsif ($line =~ /Job Category:/) {
		my $field = $line;
		$field =~ s/.*Job Category: (.*)/$1/;
		$job_info{$current_job}{'Job Category'} = $field;
	    } elsif ($line =~ /UUID:/) {
		my $field = $line;
		$field =~ s/.*UUID: (.*)/$1/;
		$job_info{$current_job}{'UUID'} = $field;
	    } elsif ($line =~ /Execution Progress:/) {
		my $field = $line;
		$field =~ s/.*Execution Progress: (.*)/$1/;
		$job_info{$current_job}{'Execution Progress'} = $field;
	    } elsif ($line =~ /Virtual Server:/) {
		my $field = $line;
		$field =~ s/.*Virtual Server: (.*)/$1/;
		$job_info{$current_job}{'Virtual Server'} = $field;
	    } elsif ($line =~ /User Name:/) {
		my $field = $line;
		$field =~ s/.*User Name: (.*)/$1/;
		$job_info{$current_job}{'User Name'} = $field;
	    }
        }
    }

    return %job_info;
}

#===================================================================
# get_hires_timestamp ()
#
# Gets a high resolution time of day.
#
# Out:
#   return	Returns a high resolution timestamp
#-------------------------------------------------------------------
sub get_hires_timestamp {
    my $hi_res_time = gettimeofday();
    my $seconds     = $hi_res_time;
    $seconds =~ s/\..*//;
    my $fraction = $hi_res_time;
    $fraction =~ s/.*\.//;
    my ( $ss, $mm, $hh, $dd, $mon, $yy ) =
      ( localtime($seconds) )[ 0, 1, 2, 3, 4, 5 ];
    $yy  += 1900;
    $mon += 1;

    my $return =
      sprintf( "%4d%02d%02d%02d%02d%02d.%d", $yy, $mon, $dd, $hh, $mm, $ss,
      $fraction );
    return $return;
}

1;
