# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/gfiler.pm#1 $
#
## @summary gfiler library functions to be used by gfiler tests
##
## @description
##   These test functions are needed for the gfiler test suites
## @usage
##   use gfiler;
##   use gfiler qw(gfiler_disklist_get siwtchlist_get);
##
## @keywords  gFiler test library
##
## @dependencies
## use config_base
## use Tharn
## 
## @author dl-sar-qa@netapp.com, dl-step@netapp.com
## @status testme
## @change 2004-09-13 sajan: created
## 

package gfiler;

use strict;
use vars ('@ISA','@EXPORT');

require Exporter;
@ISA = ('Exporter');
@EXPORT = qw(
             gfiler_info_get
             gfiler_hw_sw_info_get
	     gfiler_disklist_get
	     gfiler_no_redundant_path_get
             gfiler_no_primary_path_get
             gfiler_slot_info_get
             gfiler_local_switchlist_get
             gfiler_remote_switchlist_get
             gfiler_storage_portlist_get
             gfiler_storage_port_find
             gfiler_host_portlist_get
             gfiler_fclink_state_get
             gfiler_slotlist_get
             gfiler_partner_name_get
             gfiler_cluster_slot_info_get
             gfiler_heartbeat_slot_status_find
             gfiler_heartbeat_link_status_find
             gfiler_luns_thru_switch_check
             gfiler_switch_info_get
             gfiler_storage_info_get
             gfiler_raidv2_vol_info_get
             gfiler_raidv2_plex_info_get
             gfiler_cisco_switch_enable
             gfiler_cisco_switch_disable
	     );

use config_base;
use Tharn;

## @name ::gfiler_info_get
## @description Returns hash containing gfiler information
##
## @synopsis 
## gfiler_info_get {$conn}
##
## @arg $conn Required: The gFiler connection to send this operation over
##
##
sub  gfiler_info_get ($) {
    my ($conn) = @_;

    my (%filer_info) = ();
    my (%disklist);
    my (@local_switchlist, @remote_switchlist, @slotlist);

    %disklist = gfiler_disklist_get($conn);
    %filer_info = %disklist;
    if ($disklist{"primary"}) {
        $filer_info{"no_prim_paths"} = @{$disklist{"primary"}};
    } else {
        $filer_info{"no_prim_paths"} = 0;
    }

    if ($disklist{"secondary"}) {
        $filer_info{"no_sec_paths"} = @{$disklist{"secondary"}};
    } else {
        $filer_info{"no_sec_paths"} = 0;
    }

    @local_switchlist = gfiler_local_switchlist_get($conn);
    @{$filer_info{"localswitches"}} = @local_switchlist;
    @remote_switchlist = gfiler_remote_switchlist_get($conn);
    @{$filer_info{"remoteswitches"}} = @remote_switchlist;
    @slotlist = gfiler_slotlist_get($conn, "UP");
    @{$filer_info{"slotlist"}} = @slotlist;
    return (%filer_info);
}

## @name ::gfiler_disklist_get
## @description Returns hash having list of primary and secondary 
## LUNS connected to the gFiler
## @synopsis
## gfiler_disklist_get {$conn}
##
## @arg $conn Required: The gFiler connection to send this operation over
##
##
sub  gfiler_disklist_get ($) {
    my ($conn) = @_;

    my (%disklist) = ("primary" => undef, "secondary" => undef);
    my ($after, @temp, $line);
    my ($item, $item1, $item2, $disktype);

    (undef, undef, $after) = say($conn, "storage show disk -p", -swallow, '-timeout', 60);
    @temp = split(/\n/, $after);
    @temp = @temp[2 .. $#temp];
    foreach $item (@temp) {
        ($item1, undef, $item2, undef, undef, undef) = split(/\s+/, $item);
        push(@{$disklist{"primary"}}, $item1);
        if (!($item2 =~ /^-$/)) {
            push(@{$disklist{"secondary"}}, $item2);
        }
    }
    if ($disklist{"primary"}) {
        @{$disklist{"primary"}} = sort (@{$disklist{"primary"}});
    }

    if ($disklist{"secondary"}) {
        @{$disklist{"secondary"}} = sort (@{$disklist{"secondary"}});
    } 
    return (%disklist);
}


## @name ::gfiler_no_redundant_path_get
## @description Returns number of redundant LUNS connected to gfiler
##
## @synopsis 
## gfiler_no_redundant_path_get {$conn}
##
## @arg $conn Required: The gFiler connection to send this operation over
##
##
sub gfiler_no_redundant_path_get($) {
    my ($conn) = @_;

    my ($no_secondary_paths) = undef;
    my (%disklist);

    %disklist = gfiler_disklist_get($conn);
    $no_secondary_paths = @{$disklist{"secondary"}};
    return ($no_secondary_paths);
}

## @name ::gfiler_no_primary_path_get
## @description Returns number of primary LUNS connected to gfiler
##
## @synopsis 
## gfiler_no_primary_path_get {$conn}
##
## @arg $conn Required: The gFiler connection to send this operation over
##
##
sub gfiler_no_primary_path_get($) {
    my ($conn) = @_;

    my ($no_primary_paths) = undef;
    my (%disklist);

    %disklist = gfiler_disklist_get($conn);
    $no_primary_paths = @{$disklist{"primary"}};
    return ($no_primary_paths);
}

## @name ::gfiler_slot_info_get
## @description Returns hash having slot information
##
## @synopsis
## gfiler_slot_info_get {$conn, $slot}
##
## @arg $conn Required: The gFiler connection to send this operation over
## @arg $slot Required: The slot for which information is needed
##
##
sub  gfiler_slot_info_get ($$) {
    my ($conn, $slot_num) = @_;

    my (%slot_info) = ();
    my ($after, $line, @temp, @temp1, $field1, $field2);

    (undef, undef, $after) = say($conn, "storage show adapter $slot_num", 
                                 -swallow, '-timeout' , 60);
    @temp = split(/\n/, $after);
    foreach $line (@temp) {
        @temp1 = split(/\s+/, $line);
        if ("FC Node Name:" eq ($temp1[0] . " $temp1[1]" . " $temp1[2]")) {
             $slot_info{"FC Node Name"} = $temp1[3];
        } else {
            ($field1, $field2) = split(/:/, $line);
             $slot_info{$field1} = $field2;
        }
    }
    return (%slot_info);
}

## @name ::gfiler_local_switchlist_get
## @description Returns an array of switches locally connected to gfiler
##
## @synopsis
## gfiler_local_switchlist_get {$conn}
##
## @arg $conn Required: The gfiler connection to send this operation over
##
##
sub gfiler_local_switchlist_get ($) {
    my ($conn) = @_;

    my (%local_switchlist) = ();
    my ($after, @temp, $line, $switch, $switch_name, $length_switch_name);
    my ($state, @switchlist);

    (undef, undef, $after) = say($conn, "priv set -q advanced; 
                                 fcadmin link_state", -swallow, '-timeout', 60);
    @temp = split(/\n/, $after);
    foreach $line (@temp) {
        (undef, undef, undef, undef, undef, undef, $state, undef, 
           undef, undef, undef, undef, undef, $switch) = split(/\s+/, $line);
        if ($state =~ "UP") {
            if ($switch =~ "switch") {
                (undef, undef, undef, undef, undef, undef, undef, undef, 
                   undef, undef, undef, undef, undef, undef, 
                     $switch_name ) = split(/\s+/, $line);
                chop($switch_name);
                $length_switch_name = length($switch_name);
                $switch_name = substr($switch_name, 1, $length_switch_name);
                $local_switchlist{$switch_name} = 1;
                @switchlist = keys (%local_switchlist);
            }
        }
    }
    return (keys(%local_switchlist));
}

## @name ::gfiler_remote_switchlist_get
## @description Returns an array of remote-switches connected to gfiler
##
## @synopsis
## gfiler_remote_switchlist_get {$conn}
##
## @arg $conn Required: The gfiler connection to send this operation over
##
##
sub gfiler_remote_switchlist_get ($) {
    my ($conn) = @_;

    my (%remote_switchlist);
    my (%disklist, @local_switchlist);
    my ($path, $lun, $switch, $local_switch);

    %disklist = gfiler_disklist_get($conn);
    @local_switchlist = gfiler_local_switchlist_get($conn);
    foreach $path (keys %disklist) {
        foreach $lun (@{$disklist{$path}}) {
             ($switch, undef) = split(/:/, $lun);
             $remote_switchlist{$switch} = 1;
        }
    }
    foreach $local_switch (@local_switchlist) {
        delete ($remote_switchlist{$local_switch});
    }
    return (keys (%remote_switchlist));
}

## @name ::gfiler_storage_portlist_get
## @description For a given gfiler it returns an array of hash 
## (key = "switchname" value = port_num) having info about switch 
## and the its port connected to storage array. 
##
## @synopsis
## gfiler_storage_portlist_get {$conn}
##
## @arg $conn Required: The gfiler connection to send this operation over
##
##
sub gfiler_storage_portlist_get ($) {
    my ($conn) = @_;

    my (@storage_portlist) = ();
    my (%disklist, $path, $lun, $switch_and_port_num, $switch); 
    my ($local_switch, $port_num, $key);
    my (%temp) = ();

    %disklist = gfiler_disklist_get($conn);
    foreach $path (keys %disklist) {     #primary and secondary
        foreach $lun (@{$disklist{$path}}) {
             ($switch_and_port_num, undef) = split(/\./, $lun);
             $temp{"$switch_and_port_num"} = 1;
        }
    }
    foreach $key (keys %temp) {
        my %storage_portlist = ();
        next unless ($key =~ /:/);   #if storage is direcltly connected 2 filer
        ($switch, $port_num) = split(/:/, $key);
        $storage_portlist{"switch"} = $switch;
        $storage_portlist{"port_num"} = $port_num;
        push (@storage_portlist, \%storage_portlist);
    }
    return (@storage_portlist);
}


## @name ::gfiler_storage_port_find
## @description Returns 1 if the given storage port (switchname and 
## port_num) can be seen from gfiler
##
## @synopsis
## gfiler_storage_port_find {$conn, $switch, $port}
##
## @arg $conn Required: The gfiler connection to send this operation over
## @arg $switch Required: the switchname
## @arg $conn Required: the port_num on the given switch
##
##
sub gfiler_storage_port_find ($$$) {
    my ($conn, $switch, $port) = @_;

    my (%disklist, $path, $lun, $switch_and_port_num);

    %disklist = gfiler_disklist_get($conn);
    foreach $path (keys %disklist) {  #primary and secondary
        foreach $lun (@{$disklist{$path}}) {
             ($switch_and_port_num, undef) = split(/\./, $lun);
             if ($port =~ /\//) {     #it is a Cisco port
                 my ($blade_num, $temp_port) = split (/\//, $port);
                 --$blade_num; 
                 $port = "$blade_num" . "-" . "$temp_port";
             }
             if ($switch_and_port_num eq ("$switch" . ":" . "$port")) {
                 return 1; #found the storage port
             }
        }
    }
    return (undef);
}

## @name ::gfiler_host_portlist_get
## @description Returns an array of hash (key = "switchname" 
## value = port_num) having info about the switch port connected to 
## the storage adapters of gfiler.
##
## @synopsis
## gfiler_host_portlist_get {$conn}
##
## @arg $conn Required: The gFiler connection to send this operation over
##
##
sub gfiler_host_portlist_get ($) {
    my ($conn) = @_;

    my (@channel_info);
    my ($slot, $state, $port_num, );
    my ($after, @temp, $line, $switch, $switch_name, $length_switch_name);
    my ($port_type);

    (undef, undef, $after) = say($conn, "priv set -q advanced; 
                                fcadmin link_state", -swallow, '-timeout', 60);
    @temp = split(/\n/, $after);
    foreach $line (@temp) {
        (undef, undef, undef, undef, undef, $slot, $state, 
          undef, $port_type, undef, undef, $port_num, undef, $switch, 
          $switch_name) = split(/\s+/, $line);
        if (($state =~ "UP") && ($port_type =~ "N-port")) {
            my (%channel_info) = ();
            chop($slot);
            $channel_info{"slot"} = $slot;
            if ($switch =~ "switch") {
                chop($switch_name);
                $length_switch_name = length($switch_name);
                $switch_name = substr($switch_name, 1, $length_switch_name);
                $channel_info{"switch"} = $switch_name;
                $channel_info{"switchport"} = $port_num;
            } else {
                $channel_info{"switch"} = undef;
                $channel_info{"switchport"} = undef;
            }
            push(@channel_info, \%channel_info);
        }
    }
    return (@channel_info);
}

## @name ::gfiler_fclink_state_get
## @description Returns "OFFLINE" or "DOWN" or "UP" for a given filer and its 
## storage slot connected to the switch
##
## @synopsis
## gfiler_fclink_state_get {$conn, $slot}
##
## @arg $conn Required: The gfiler connection to send this operation over
## @arg $slot Required: Link state of this slot is needed
##
##
sub gfiler_fclink_state_get ($$) {
    my ($conn, $slot) = @_;

    my ($state);
    my ($after);

    (undef, undef, $after) = say($conn, "priv set -q advanced; 
                         fcadmin link_state $slot", -swallow, '-timeout', 60);
    (undef, undef, undef, undef, undef, $slot, $state) = split(/\s+/, $after);
    return ($state);

}

## @name ::gfiler_slotlist_get
## @description Returns array of storage slots having given state on a gfiler 
##
## @synopsis
## gfiler_fclink_state_get {$conn, $state}
##
## @arg $conn Required: The gfiler connection to send this operation over
## @arg $state Required: The state a slot should be into
##
##
sub gfiler_slotlist_get ($$) {
    my ($conn, $state) = @_;

    my (@slotlist);
    my ($after, @temp, $line, $slot, $current_state);

    (undef, undef, $after) = say($conn, "priv set -q advanced; 
                             fcadmin link_state", -swallow, '-timeout', 60);
    @temp = split(/\n/, $after);
    foreach $line (@temp) {
        (undef, undef, undef, undef, undef, $slot, 
                          $current_state) = split(/\s+/, $line);
        if ($state eq $current_state) {
            chop($slot);
            push(@slotlist, $slot);
        }
    }
    return (@slotlist);

}
## @name ::gfiler_partner_name_get
## @description Returns name of the partner filer
##
## @synopsis
## gfiler_fclink_state_get {$conn}
##
## @arg $conn Required: The gfiler connection to send this operation over
##
sub gfiler_partner_name_get ($) {
    my ($conn) = @_;

    my ($partner_name) = undef;
    my ($after, @temp, $line, $name_length);

    (undef, undef, $after) = say($conn, "sysconfig", -swallow, '-timeout', 60);
    @temp = split(/\n/, $after);
    foreach $line (@temp) {
         next unless ($line =~ "partner ID");
         $line =~ s/^\s+//g;
         ($partner_name) = (split (/\s+/, $line))[7]; 
         chop($partner_name);
         $name_length = length($partner_name);
         $partner_name = substr($partner_name, 1, $name_length);
    }
    return ($partner_name);

}

## @name ::gfiler_cluster_slot_info_get
## @description Returns array of hash, each hash has information about 
## cluster slot number, its WWN (if it is metro cluster), cluster 
## link number and cluster_type (metro | regular-troika | regular-mellanox)
## for a given filer
##
## @synopsis
## gfiler_cluster_slot_info_get{$conn}
##
## @arg $conn Required: The gFiler connection to send this operation over
##
sub gfiler_cluster_slot_info_get($) {
    my ($conn) = @_;

    my ($after, @temp, $line, @cluster_slotlist);
    my ($WWN) = undef;
    my ($cluster_type) = undef;
    my ($maybe_cluster_slot) = undef;
    my (@heartbeat_slot_num, @link);

    (undef, undef, $after) = 
                       say($conn, "sysconfig -v", -swallow, '-timeout', 240);
    @temp = split(/\n/, $after);
    my $i=0;

    # - - - - - - - -  - - - - - - - - - - - - - - - - - -  -
    # FCIV = Qlogic Metrocluster card, 
    # Troika VI = Troika regular cluster card
    # Mellanox = Mellanox another regular cluster card
    # NVRAMV  
    # - - - - - - - -  - - - - - - - - - - - - - - - - - -  -
    # Tested on GF980 - metro cluster with NVRAM 5
    # Tested on GF960 - metro cluster without NVRAM 5
    # Tested on GF825 - metro cluster without NVRAM 5
    # Tested on GF825 - Regular Troika cluster without NVRAM 5
    # Tested on GF825 - Regular IB cluster without NVRAM 5
    # Tested on GF960 - IB Mellanox cluster without NVRAM 5
    # Tested on GF960 - NVRAMV cluster obviously with NVRAM 5
    # - - - - - - - -  - - - - - - - - - - - - - - - - - -  -
    foreach $line (@temp) {
        next unless (($line =~ "FCVI Host Adapter ") || 
                     ($line =~ "Troika FCVI") || 
                     ($line =~ "NVRAM V") ||
                     ($line =~ "Cluster Interconnect Port") ||
                     ($line =~ "Mellanox IB"));
        $line =~ s/^\s+//g;
        if ($line =~ "FCVI Host Adapter") {
            $cluster_type = "metro";
            ($heartbeat_slot_num[$i]) = (split (/\s+/, $line))[5];
            ++$i;
        } elsif ($line =~ "Troika FCVI") {
            $cluster_type = "regular-troika";
            ($heartbeat_slot_num[$i]) = (split (/\s+/, $line))[1];
            chop($heartbeat_slot_num[$i]);
            # Troika has only 1 link, so don't need to get value for other link
        } elsif ($line =~ "Mellanox IB") {
            $cluster_type = "regular-mellanox";
            ($heartbeat_slot_num[$i]) = (split (/\s+/, $line))[1];
            chop($heartbeat_slot_num[$i]);
            # The sysconfig -v has only slot number output for IB cluster,
            # (no a nd b), so covering up that
            $heartbeat_slot_num[$i + 1] = $heartbeat_slot_num[$i]
        } elsif ($line =~ "NVRAM V") {
            $maybe_cluster_slot = (split (/\s+/, $line))[1];
            chop($maybe_cluster_slot);
	    $cluster_type = "NVRAMV";
        } elsif ($line =~ "Cluster Interconnect Port") {
            if ($line =~ /copper/i) {
                $cluster_type = "NVRAMV";
                ($heartbeat_slot_num[$i]) = $maybe_cluster_slot; 
                ++$i;
            }
        }
    }

    if ($cluster_type =~ "metro") {
        (undef, undef, $after) = say($conn, "sysconfig -M", -swallow, 
                                                         '-timeout', 240);
        @temp = split(/\n/, $after);
        my $i=0;
        foreach $line (@temp) {
            next unless (($line =~ "FCVI") || ($line =~ "Troika VI") || 
                         ($line =~ "Mellanox IB"));
            if ($line =~ "FCVI") {
                ($WWN) = (split (/!/, $line))[2];
            }

            my %heartbeat_info;
            $heartbeat_info{"cluster_type"} = $cluster_type;
            $heartbeat_info{"slot"} = $heartbeat_slot_num[$i];
            $heartbeat_info{"WWN"} = $WWN;
            $heartbeat_info{"link"} = $i;
            push (@cluster_slotlist, \%heartbeat_info); 
            ++$i;
        }
    } elsif ($cluster_type =~ "regular-mellanox") {
        foreach my $i (0,1) {   # IB has 2 cluster links
            my %heartbeat_info;
            $heartbeat_info{"cluster_type"} = $cluster_type;
            $heartbeat_info{"slot"} = $heartbeat_slot_num[$i];
            $heartbeat_info{"WWN"} = undef;
            $heartbeat_info{"link"} = $i;
            push (@cluster_slotlist, \%heartbeat_info); 
        }
    } elsif ($cluster_type =~ "regular-troika") { 
        # Troika has only 1 cluster link
        my $i=0;
        my %heartbeat_info;
        $heartbeat_info{"cluster_type"} = $cluster_type;
        $heartbeat_info{"slot"} = $heartbeat_slot_num[$i];
        $heartbeat_info{"WWN"} = undef;
        $heartbeat_info{"link"} = $i;
        push (@cluster_slotlist, \%heartbeat_info); 
    } elsif ($cluster_type =~ "NVRAMV" ){
        foreach my $i (0,1) {   # NVRAM V has 2 cluster links
            my %heartbeat_info;
            $heartbeat_info{"cluster_type"} = $cluster_type;
            $heartbeat_info{"slot"} = $heartbeat_slot_num[$i];
            $heartbeat_info{"WWN"} = undef;
            $heartbeat_info{"link"} = $i;
            push (@cluster_slotlist, \%heartbeat_info); 
        }
    }
    #print "Cluster type: $cluster_type\n";
    #foreach my $temp_slot (@heartbeat_slot_num) {
    #    print "Cluster slot: $temp_slot\n"
    #}
    return @cluster_slotlist;

}

## @name ::gfiler_heartbeat_slot_status_find
## @description Returns status of the cluster slot card, works only 
## for metro cluster card (return value = UP/DOWN/undef)
##
## @synopsis
## gfiler_heartbeat_slot_status_find{$conn, $slot}
##
## @arg $conn Required: The gFiler connection to send this operation over
## @arg $slot Required: cluster slot number  
##
sub gfiler_heartbeat_slot_status_find($$) {
    my ($conn, $slot) = @_;

    my ($after, @temp, $line);
    my ($slot_num);
    my ($slot_status) = undef;

    $slot_num = $slot;
    chop ($slot_num);
    (undef, undef, $after) = say($conn, "sysconfig -v $slot_num", -swallow, 
                                                    '-timeout', 60);
    @temp = split(/\n/, $after);
    foreach $line (@temp) {
        next unless (($line =~ "FCVI") && ($line =~ $slot));
        $line =~ s/^\s+//g;
        ($slot_status) = (split (/\</, $line))[1];
        chop($slot_status);
        chop($slot_status);
    }
    return ($slot_status);

}

## @name ::gfiler_heartbeat_link_status_find
## @description Returns status of the cluster link (up/down/undef)
##
## @synopsis
## gfiler_heartbeat_link_status_find{$conn, $link_num}
##
## @arg $conn Required: The gFiler connection to send this operation over
## @arg $link_num Required: cluster link number  
##
sub gfiler_heartbeat_link_status_find($$) {
    my ($conn, $link) = @_;

    my ($after, @temp, $line);
    my ($link_status) = undef;

    #(undef, undef, $after) = say($conn, "priv set -q advanced; ic status", 
    #                             -swallow, '-timeout', 60);
    #@temp = split(/\n/, $after);
    #foreach $line (@temp) {
    #    next unless ($line =~ /Link $link/i);
    #    $line =~ s/^\s+//g;
    #    ($link_status) = (split (/:/, $line))[1];
    #    $link_status =~ s/^\s+//g;
    #}
    #(undef, undef, $after)= say($conn, "cf monitor", -swallow, '-timeout', 60);
    #@temp = split(/\n/, $after);
    #foreach $line (@temp) {
    #    next unless ($line =~ /VIA Interconnect/i);
    #    $line =~ s/^\s+//g;
    #    if ( 0 == $link) {
    #        ($link_status) = (split (/\s+/, $line))[6];
    #    } elsif ( 1 == $link) {
    #        ($link_status) = (split (/\s+/, $line))[9];
    #    }
    #    $link_status =~ s/^\s+//g;
    #    if (($link_status =~ /,/) || ($link_status =~ /\)/)) {
    #         chop ($link_status);
    #    }
    #    if (($link_status =~ /,/) || ($link_status =~ /\)/)) {
    #         chop ($link_status);
    #    }
    #    print "LInks staus for link $link is: $link_status\n";
    #    return ($link_status);
    #}

    (undef, undef, $after)=say($conn, "priv set -q advanced; ic stats error -v",
                                 -swallow, '-timeout', 300);
    @temp = split(/\n/, $after);
    my $link_counter = 0;   #used for metro cluster
    foreach $line (@temp) {
        
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        # Troika (tested on GF825, raid-u11 and raid-u12)
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        if ($line =~ /Link alive/i) {  
            $line =~ s/^\s+//g;
            ($link_status) = (split (/:/, $line))[1];
            $link_status =~ s/^\s+//g;
            if ($link_status == 1) { 
                $link_status = 'up';
            } else {
                $link_status = 'down';
            }
            chomp($link_status);
            return ($link_status);
        }

        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        # Metro Cluster, (tested on GF825, raid-u32 and raid-u33 without NVRAM5)
        # Tested on GF960 (raid-u30 and raid-u31 without NVRAM5)
        # Tested on GF980 (gf980-svl1 and gf980-svl2 with NVRAM5)
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        if ($line =~ /Nic state:/i) {  
            $line =~ s/^\s+//g;
            ($link_status) = (split (/:/, $line))[1];
            $link_status =~ s/^\s+//g;
            chomp($link_status);
            if ($link == $link_counter) { 
                return ($link_status);
            } else { 
                ++$link_counter;
            }
        }

        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        # Need to insert code for IB cluster type
        # Tested on GF825 without NVRAM V (gf825-svl1 and svl2)
        # Tested on GF960 without NVRAM V (gf960-svl1 and svl2)
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        if ($line =~ /Port $link state/i) {  
            $line =~ s/^\s+//g;
            ($link_status) = (split (/:/, $line))[1];
            $link_status =~ s/^\s+//g;
            if ($link_status =~ /ACTIVE/i) { 
                $link_status = 'up';
            } else {
                $link_status = 'down';
            }
            chomp($link_status);
            return ($link_status);
        }

        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        # NVRAM5 (tested on GF960, snake and tooz)
        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        my $temp1 = $link + 1;
        if ($line =~ /Link $temp1 state:/i) {  
            $line =~ s/^\s+//g;
            ($link_status) = (split (/:/, $line))[1];
            $link_status =~ s/^\s+//g;
            chomp($link_status);
            $link_status = lc($link_status);
            return ($link_status);
        }

    }
    return (undef);
}

## @name ::gfiler_luns_thru_switch_check
## @description Returns 1 if gfiler can see any lun from given switch, else
## undef
##
## @synopsis
## gfiler_luns_thru_switch_check {$conn, $switch}
##
## @arg $conn Required: The gFiler connection to send this operation over
## @arg $switch Required: the switch to check for
##
##
sub gfiler_luns_thru_switch_check ($$) {
    my ($conn, $switch) = @_;

    my (%disklist, $path, $lun, $switch_and_port_num, $current_switch);

    %disklist = gfiler_disklist_get($conn);
    foreach $path (keys %disklist) {  #primary and secondary
        foreach $lun (@{$disklist{$path}}) {
             ($switch_and_port_num, undef) = split(/\./, $lun);
             ($current_switch) = split (/:/, $switch_and_port_num);
             if ($current_switch eq $switch) {
                 return 1; #found the switch
             }
        }
    }
    return (undef);
}

## @name ::gfiler_hw_sw_info_get
## @description Returns hash having following keys and their value
## "model name" => <value>
## "firmware release" => <value>
## "software release" => <value>
##
## @synopsis
## gfiler_hw_sw_info_get{$conn}
##
## @arg $conn Required: The gfiler connection to send this operation over
##
##
sub gfiler_hw_sw_info_get ($) {
    my ($conn) = @_;

    my ($after, @temp, $line);
    my ($model_name, $firmware_release, $software_release);
    my (%filer_hw_sw_info) = ();

    (undef, undef, $after) = say($conn, "sysconfig -v 0", -swallow, 
                                                    '-timeout', 60);

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # get model name
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    @temp = split(/\n/, $after);
    foreach $line (@temp) {
        next unless ($line =~ /Model Name/);
        $line =~ s/^\s+//g;
        ($model_name) = (split (/\s+/, $line))[2];
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # get firmware release
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    foreach $line (@temp) {
        next unless ($line =~ /Firmware release/);
        $line =~ s/^\s+//g;
        ($firmware_release) = (split (/\s+/, $line))[2];
    }

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # get software release
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    (undef, undef, $after) = say($conn, "version", -swallow, '-timeout', 60);
    @temp = split(/\n/, $after);
    foreach $line (@temp) {
        next unless ($line =~ /Release/);
        $line =~ s/^\s+//g;
        ($software_release) = (split (/\s+/, $line))[2];
        chop($software_release);
    }

    $filer_hw_sw_info{"model name"} = $model_name;
    $filer_hw_sw_info{"firmware release"} = $firmware_release;
    $filer_hw_sw_info{"software release"} = $software_release;
     
    return (%filer_hw_sw_info);

}

## @name ::gfiler_switch_info_get
## @description Extract information about switches connected to gfiler. 
## Returns array of hash having following keys and their value
## "switch" => <switchname>
## "firmware" => <value>
## "vendor" => <value>
##
## @synopsis
## gfiler_switch_info_get{$conn}
##
## @arg $conn Required: The gfiler connection to send this operation over
##
##
sub gfiler_switch_info_get($) {
    my ($conn) = @_;

    my ($after, @temp, $line);
    my ($switch_name, $firmware_release, $vendor);
    my (@switch_info) = ();
    my (%filer_hw_sw_info) = ();

    (undef, undef, $after) = say($conn, "storage show switch", -swallow,
                                                    '-timeout', 1800);

    @temp = split(/\n/, $after);
    my $i=0;
    foreach $line (@temp) {
        next unless (($line =~ /Name:/) || ($line =~ /Version:/) ||
                     ($line =~ /Vendor:/));
        if ($line =~ /Name:/) {
            my %rec = ();
            $line =~ s/^\s+//g;
            $rec{"switch"} = (split (/\s+/, $line))[1];
            push (@switch_info, \%rec);
        }      

        if ($line =~ /Version:/) {
            $line =~ s/^\s+//g;
            $switch_info[$i]->{"version"} = (split (/\s+/, $line))[1];
        }

        if ($line =~ /Vendor:/) {
            $line =~ s/^\s+//g;
            $switch_info[$i]->{"vendor"} = (split (/\s+/, $line))[1];
            ++$i;
        }
        
    }
    return (@switch_info);
}

## @name ::gfiler_storage_info_get
## @description Extract information about storage arrays connected to gfiler. 
## Returns an hash having following keys and their value
## <vendor_iname> => @ of model 
##
## @synopsis
## gfiler_switch_info_get{@conn}
##
## @arg $conn Required: Array of gfiler connections
##
sub gfiler_storage_info_get($) {
    my ($conn_param) = @_;

    my ($after, @temp, $line);
    my ($vendor, $model);
    my (%storage_info) = ();
    my (%seen) = ();

    foreach my $conn (@{$conn_param}) {
        (undef, undef, $after) = say($conn, "storage show disk", -swallow,
                                                    '-timeout', 60);

        @temp = split(/\n/, $after);
        @temp = @temp[2 .. $#temp];
        foreach $line (@temp) {
            $vendor = (split (/\s+/, $line))[4];
            $model = (split (/\s+/, $line))[5];
            if (!exists($storage_info{$vendor})) {
                push (@{$storage_info{$vendor}}, $model);
                $seen{$model} = 1;
            }

            if (!exists($seen{$model})) {
                push (@{$storage_info{$vendor}}, $model);
            }
        }
    }
    return (%storage_info);
}

my $RAIDV2_TIMEOUT = 240;

## @name ::gfiler_raidv2_vol_info_get
## @description To get the trad-vol/aggregate/flex-vol info for the given
## gfiler.
##
## @synopsis
## gfiler_raidv2_vol_info_get {$conn, ....options.....)
##
## @arg $conn Required: The gFiler connection to send this operation over
##
## @arg vol_type=>0|1 Optional - default is 0 to get info for aggr, 1 for vols 
##
## @arg partner=>0|1 Optional - default is 0 to get info for the node 
## corrosponding to $conn. 1 to get the information for the takenover node, so
## this param is needed only if the gfiler is takeove mode.
##
## @arg vol_name=><vol-name> Optional - default is "undef" means get info for 
## the the aggr/vol otherwise for the defined aggr/vol only. If you want to 
## get info for a particular flex-vol then vol_type should also be set to 1.
##
## @arg info_type=>0|1|2 Optional - default is 0, means return the % containing
## the info by_name, 1 to return % containing info by_id, 2 to return @ 
## containing name of the aggr/flex-vol 
##
##
sub gfiler_raidv2_vol_info_get($@) {
    my ($conn, %opts) = @_;

    my $vol_type = defined($opts{vol_type}) ? $opts{vol_type} : 0;
    my $partner = defined($opts{partner}) ? $opts{partner} : 0;
    my $vol_name = defined($opts{vol_name}) ? $opts{vol_name} : undef;
    my $info_type = defined($opts{info_type}) ? $opts{info_type} : 0;

    my %raidv2_info_vol_by_name = ();
    my %raidv2_info_vol_by_id = ();
    my @raidv2_info_vol_list;
    
    my $priv = "priv set -q diag";

    my(@info);
    my($line);
    my($output);
    
    if ($partner == 0) {	
        say($conn, "$priv; ps -h", '-timeout', $RAIDV2_TIMEOUT);
    } else {
        say($conn, "partner $priv; partner ps -h", '-timeout', $RAIDV2_TIMEOUT);
    }

    #--------------------------------------------------
    # "raid_config info listaggr" is not supported in aprtner's mode
    # in fact if this command is issued in NON-partner mode then the
    # info with host=1 is fpr partner node. Command "raidconfig info listvol"
    # works for both local and partner node.
    #--------------------------------------------------

    my ($raid_info_cmd);
    if ($vol_type == 0) {
	$raid_info_cmd = "$priv; raid_config info listaggr $vol_name";
    } elsif ($partner == 0) {
	$raid_info_cmd = "$priv; raid_config info listvol $vol_name";
    } else {
	$raid_info_cmd = "partner $priv; partner raid_config info listvol $vol_name";
    }

    my ($aggr_cmd) = "aggr status -r";
    if ($partner !=0) {
        $aggr_cmd = "partner vol status -r";
    }

    (undef, undef, $output) = say($conn, "$raid_info_cmd", '-timeout', $RAIDV2_TIMEOUT);

    # burt146242 - work around, call cmd again  ---Viv
    # if no success after 4 retries, panic the filer
    my ($flag) = 0;
    for (my $i=0; $i<4; $i++) {
        if ($output eq "") {
            say($conn, "$priv; $aggr_cmd", '-timeout', $RAIDV2_TIMEOUT);
            tstlog -comment, "$raid_info_cmd provides no output. Try again\n";
            sleep(1);
            (undef, undef, $output) = say($conn, "$priv; $raid_info_cmd",'-timeout', $RAIDV2_TIMEOUT);
        } else {
            $flag = 1;
            last;
        }
    }
    if ($flag != 1) {
       tstlog (-comment, "panicking because $raid_info_cmd produce no output after retries.\n");
       say($conn, "sysconfig -r");
       say($conn, "$priv; panic", '-timeout', $RAIDV2_TIMEOUT);
    }

    @info = split(/\r?\n/, $output);
    my(%count) = ();
    my(%entrydb) = ();
    foreach $line (@info) {
	my(@vol_info) = split(",", $line);
	my($entry);
	#
	# Load the entry
	#
	foreach $entry (@vol_info) {
 	    if ($entry =~ /(.*)=(.*)/) {
                $entrydb{$1} = $2;
	    }
	}

	#
	# Return info either for the local node or for partner node
	#
        if (($partner == 0) && ($entrydb{"host"} == 1)) {
            next;
        } 

        if (($partner == 1) && ($entrydb{"host"} == 0)) {
            next;
        } 

	#
	# Now, go back and get the vol name
	#
	my($name) = $entrydb{"name"};
	push (@raidv2_info_vol_list, $name) ;
	if (!$name) {
	    next;
	}

	my($id) = $entrydb{"id"};
	my($e);

	foreach $e (keys %entrydb) {
            $raidv2_info_vol_by_name{$name}{$e} = $entrydb{$e};
	    $raidv2_info_vol_by_id{$id}{$e} = $entrydb{$e};
	}

    } #foreach line

    if ($info_type == 0) {
        return %raidv2_info_vol_by_name;
    } elsif ($info_type == 1) {
        return %raidv2_info_vol_by_id;
    } else {
        return @raidv2_info_vol_list;
    }
}

## @name ::gfiler_raidv2_plex_info_get
## @description To get the trad-vol/aggregate/flex-vol info for the given
## gfiler.
##
## @synopsis
## gfiler_raidv2_plex_info_get {$conn, ....options.....)
##
## @arg $conn Required: The gFiler connection to send this operation over
##
## @arg vol_name=><vol-name> Optional - default is "undef" means get info for 
## the the aggr/vol otherwise for the defined aggr/vol only. If you want to 
## get info for a particular flex-vol then vol_type should also be set to 1.
##
## @arg info_type=>0|1|2|3|4|5 Optional - default is 0, means return the % 
## containing the info by_name, 1 to return % containing info by_id, 2 to 
## return return @ containing name of the plexes, 3 to return info by 
## parent_id, 4 to return % where each key is tree name and element are array
## of plexes for that tree, and 5 to return @ containing id of plexes 
##
##
sub gfiler_raidv2_plex_info_get($@) {
    my ($conn, %opts) = @_;

    my $plex_name = defined($opts{plex_name}) ? $opts{plex_name} : undef;
    my $info_type = defined($opts{info_type}) ? $opts{info_type} : 0;

    my %raidv2_info_plex_by_name = ();
    my %raidv2_info_plex_by_id = ();
    my %raidv2_info_plex_by_parent_id = ();
    my %raidv2_info_plex_by_tree = ();
    my @raidv2_info_plex_list;
    my @raidv2_info_plex_id_list;
    
    my $priv = "priv set -q diag";

    my(@info);
    my($line);
    my($output);
    
    say($conn, "$priv; ps -h", '-timeout', $RAIDV2_TIMEOUT);

    my ($raid_info_cmd);
    $raid_info_cmd = "$priv; raid_config info listplex $plex_name";

    (undef, undef, $output) = say($conn, "$raid_info_cmd", '-timeout', $RAIDV2_TIMEOUT);

    @info = split(/\r?\n/, $output);
    my(%count) = ();
    my(%entrydb) = ();
    foreach $line (@info) {
	my(@plex_info) = split(",", $line);
	my($entry);
	#
	# Load the entry
	#
	foreach $entry (@plex_info) {
 	    if ($entry =~ /(.*)=(.*)/) {
                $entrydb{$1} = $2;
	    }
	}

	#
	# Now, go back and get the vol name
	#
	my($name) = $entrydb{"raid_name"};
	push (@raidv2_info_plex_list, $name) ;
	if (!$name) {
	    next;
	}

	my($tree) = $entrydb{"tree"};
	push (@{$raidv2_info_plex_by_tree{$tree}}, $name) ;
	if (!$tree) {
	    next;
	}

	my($id) = $entrydb{"id"};
	push (@raidv2_info_plex_id_list, $id) ;

	my($parent_id) = $entrydb{"parent_id"};
	my($e);

	foreach $e (keys %entrydb) {
            $raidv2_info_plex_by_name{$name}{$e} = $entrydb{$e};
	    $raidv2_info_plex_by_id{$id}{$e} = $entrydb{$e};
	    $raidv2_info_plex_by_id{$parent_id}{$e} = $entrydb{$e};
	}

    } #foreach line

    if ($info_type == 0) {
        return %raidv2_info_plex_by_name;
    } elsif ($info_type == 1) {
        return %raidv2_info_plex_by_id;
    } elsif ($info_type == 2) {
        print "The array is: @raidv2_info_plex_list\n";
        return @raidv2_info_plex_list;
    } elsif ($info_type == 3) {
        return %raidv2_info_plex_by_parent_id;
    } elsif ($info_type == 4) {
        return %raidv2_info_plex_by_tree;
    } else {
        return @raidv2_info_plex_id_list;
    }
}

## @name ::gfiler_cisco_switch_enable
## @description Enable all fc ports on a switch
##
## @synopsis
## gfiler_cisco_switch_enable{$conn}
##
## @arg $conn Required: The switch connection to send this operation over
##
## @example
## if (!gfiler_cisco_switch_enable($sc) {exit 1; } # Port not enabled or error
##
sub gfiler_cisco_switch_enable($) {
   my($conn) = @_;
  
   my ($after);
   my %blade_list = ();

   if ((not defined($conn))) {
       return 0;
   }

   # get host type and determine proper method
   my $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-cisco/i) {
        say($conn, "terminal length 0", -swallow, '-timeout', 60);
        (undef, undef, $after) = say($conn, "show module", -swallow, '-timeout', 60);
        my (@info) = split(/\r\n/, $after);
        foreach my $line (@info)  {
            next unless (($line =~ /FC Module/i) || ($line =~ /FC\/Supervisor/));
            my $blade_num = (split (/\s+/, $line))[0];
            my $total_ports = (split (/\s+/, $line))[1];
            $blade_list{$blade_num} = $total_ports;
        }
 
        foreach my $blade (keys %blade_list) {
            #my $interface = "fc" . "$blade" . "\/" . "1-$blade_list{$blade}";
            my $interface = "fc" . "$blade" . "\/" . "1-10";
            say($conn, "conf t", -noprompt,  "-wait", "#");
            say($conn, "int $interface", -noprompt, "-wait", "#");
            say($conn, "no shutdown", -noprompt, "-wait", "#");
            say($conn, "end", -noprompt, "-wait", "#");
        }
   }
   return 1;
}

## @name ::gfiler_cisco_switch_disable
## @description Disable all fc ports on a switch
##
## @synopsis
## gfiler_cisco_switch_disable{$conn}
##
## @arg $conn Required: The switch connection to send this operation over
##
## @example
## if (!gfiler_cisco_switch_disable($sc) {exit 1; } # Port not enabled or error
##
sub gfiler_cisco_switch_disable($) {
   my($conn) = @_;
  
   my ($after);
   my %blade_list = ();

   if ((not defined($conn))) {
       return 0;
   }

   # get host type and determine proper method
   my $host_type = $conn->{hostp}->{hosttype};
   if ($host_type =~ /switch-cisco/i) {
        say($conn, "terminal length 0", -swallow, '-timeout', 60);
        (undef, undef, $after) = say($conn, "show module", -swallow, '-timeout', 60);
        my (@info) = split(/\r\n/, $after);
        foreach my $line (@info)  {
            next unless (($line =~ /FC Module/i) || ($line =~ /FC\/Supervisor/));
            my $blade_num = (split (/\s+/, $line))[0];
            my $total_ports = (split (/\s+/, $line))[1];
            $blade_list{$blade_num} = $total_ports;
        }
 
        foreach my $blade (keys %blade_list) {
            #my $interface = "fc" . "$blade" . "\/" . "1-$blade_list{$blade}";
            my $interface = "fc" . "$blade" . "\/" . "1-10";
            say($conn, "conf t", -noprompt,  "-wait", "#");
            say($conn, "int $interface", -noprompt, "-wait", "#");
            say($conn, "shutdown", -noprompt, "-wait", "#");
            say($conn, "end", -noprompt, "-wait", "#");
        }
   }
   return 1;
}
1;
