# -*- perl -*-
# $Id: //depot/prod/test/main/lib/gx_licenses.pm#1 $
#
## @summary gx_utils library functions pertaining to ONTAP v1 and v2
## licenses
##
## @description
## This library contains routines shared by gx_boot.thpl and deux_init.thpl
## for v1 and v2 license lookups
## 
## @usage
##      use gx_licenses;
##      use gx_licenses qw(get_licenses);
##
## @keywords  gx_utils test library
## @dependencies
## none
## @author dl-central-libs@netapp.com
## status open
## author 2012-01-13 venkatg created
##

package gx_licenses;

use strict;
use vars ('@ISA','@EXPORT');
require Exporter;

@ISA = ('Exporter');

@EXPORT = qw(
    get_licenses
    get_base_license
    get_license_attributes_from_key
    get_default_serial_number
    get_packages
    map_v1_to_v2
);

# C mode licenses
my $licenses = {
    "1-80-123456" => {
        "BASE" =>           "NWUZZJPJYBFDAA",
        "MIRROR" =>         "LLQDBKPJYBFDAA",
        "FLEXVOL_HPO" =>    "XFOSBKPJYBFDAA",
        "CIFS" =>           "JAMHCKPJYBFDAA",
        "SNAPRESTORE" =>    "VUJWCKPJYBFDAA",
        "NFS" =>            "TJFAEKPJYBFDAA",
        "SNAPMIRROR_DP" =>  "FEDPEKPJYBFDAA",
        "STRIPED_VOLUME" => "DTYSFKPJYBFDAA",
        "ISCSI" =>          "RYAEFKPJYBFDAA",
        "FCP" =>            "PNWHGKPJYBFDAA",
        "FLEXCLONE" =>      "BYKBBPAMUBDAAA",
    },

    "1-80-000011" => {
        "BASE" =>           "VMZIVZGKWVMADF",
        "MIRROR" =>         "TBVMWZGKWVMADF",
        "FLEXVOL_HPO" =>    "FWSBXZGKWVMADF",
        "CIFS" =>           "RQQQXZGKWVMADF",
        "SNAPRESTORE" =>    "DLOFYZGKWVMADF",
        "NFS" =>            "BAKJZZGKWVMADF",
        "SNAPMIRROR_DP" =>  "NUHYZZGKWVMADF",
        "STRIPED_VOLUME" => "LJDCBAHKWVMADF",
        "ISCSI" =>          "ZOFNAAHKWVMADF",
        "FCP" =>            "XDBRBAHKWVMADF",
        "FLEXCLONE" =>      "PFMUYZGKWVMADF",
    },
    "1-80-000013" => {
        "BASE" =>           "XBOLYLDPMZSEBG",
        "MIRROR" =>         "VQJPZLDPMZSEBG",
        "FLEXVOL_HPO" =>    "HLHEAMDPMZSEBG",
        "CIFS" =>           "TFFTAMDPMZSEBG",
        "SNAPRESTORE" =>    "FADIBMDPMZSEBG",
        "NFS" =>            "DPYLCMDPMZSEBG",
        "SNAPMIRROR_DP" =>  "PJWADMDPMZSEBG",
        "STRIPED_VOLUME" => "NYREEMDPMZSEBG",
        "ISCSI" =>          "BEUPDMDPMZSEBG",
        "FCP" =>            "ZSPTEMDPMZSEBG",
        "FLEXCLONE" =>      "RUAXBMDPMZSEBG",
    },
    "1-80-000017" => {
        "BASE" =>           "ZYSOQSYLTDIFMA",
        "MIRROR" =>         "XNOSRSYLTDIFMA",
        "FLEXVOL_HPO" =>    "JIMHSSYLTDIFMA",
        "CIFS" =>           "VCKWSSYLTDIFMA",
        "SNAPRESTORE" =>    "HXHLTSYLTDIFMA",
        "NFS" =>            "FMDPUSYLTDIFMA",
        "SNAPMIRROR_DP" =>  "RGBEVSYLTDIFMA",
        "STRIPED_VOLUME" => "PVWHWSYLTDIFMA",
        "ISCSI" =>          "DBZSVSYLTDIFMA",
        "FCP" =>            "BQUWWSYLTDIFMA",
        "FLEXCLONE" =>      "TRFAUSYLTDIFMA",
    },
    default =>              "1-80-000011",
    default_base =>         "1-80-000011",
};

# SN v2.0 licenses - both 7 mode and C mode
# In v2.0 the base license and the feature licenses can be
# from different serial numbers
my $v2_licenses = {
    "1-80-000013" => {
        "BASE" =>               "GINBJXTZGYSEBGAAAAAAAAAAAAAA",
    },
    "1-80-000011" => {
        "BASE" =>               "ETYYFLXUQUMADFAAAAAAAAAAAAAA",
    },
    "1-80-000017" => {
        "BASE" =>               "IFSEBEPWNCIFMAAAAAAAAAAAAAAA",
    },
    "1-81-000000" => {
        "CDMI" =>               "WSQFRWDUSAFDAAAAAAAAAAAAAAAA",
        "CIFS" =>               "MJXMPWDUSAFDAAAAAAAAAAAAAAAA",
        "FCP" =>                "KYSQQWDUSAFDAAAAAAAAAAAAAAAA",
        "FLEXCLONE" =>          "GCKYSWDUSAFDAAAAAAAAAAAAAAAA",
        "ISCSI" =>              "YDVBQWDUSAFDAAAAAAAAAAAAAAAA",
        "NFS" =>                "APZXOWDUSAFDAAAAAAAAAAAAAAAA",
        "SNAPLOCK" =>           "ERFCUWDUSAFDAAAAAAAAAAAAAAAA",
        "SNAPMANAGER_SUITE" =>  "QLDRUWDUSAFDAAAAAAAAAAAAAAAA",
        "SNAPMIRROR" =>         "UHMJSWDUSAFDAAAAAAAAAAAAAAAA",
        "SNAPPROTECT_APP" =>    "CGBGVWDUSAFDAAAAAAAAAAAAAAAA",
        "SNAPRESTORE" =>        "INOURWDUSAFDAAAAAAAAAAAAAAAA",
        "SNAPVAULT" =>          "SWHNTWDUSAFDAAAAAAAAAAAAAAAA",
        "SNAPLOCK_ENTERPRISE" => "MRDIXDBBVAAAAAAAAAAAAAAAAAAA",
        "VSTORAGE_ATTACH" =>    "AXFTWDBBVAAAAAAAAAAAAAAAAAAA",
        "INSIGHT_BALANCE" =>    "YLBXXDBBVAAAAAAAAAAAAAAAAAAA",
    },
    default =>                  "1-81-000000",
    default_base =>             "1-80-000011",
};

# 7 Mode only licenses (works for ONTAP prior to SN.0)
# No serial numbers
my $licenses7m = {
    "no_serial_number" => {
        "NFS" =>                "UYTCNCG",
        "CIFS" =>               "GTLYRVH",
        "SNAPMIRROR" =>         "UUNLDQC",
        "FCP" =>                "ICRNCZG",
        "ISCSI" =>              "IKVAREM",
        "SYNCMIRROR_LOCAL" =>   "UELJHSI",
        "CF" =>                 "KDQLGBN",
        "SNAPRESTORE" =>        "EOYPBIL",
        "FLEXCLONE" =>          "QNFYRVH",
    }
};

# Add a v1 -> v2 mapping table
my $v1_to_v2_mapping = {
    "NFS" => "NFS",
    "CIFS" => "CIFS",
    "FCP" => "FCP",
    "ISCSI" => "ISCSI",
    "SNAPMANAGER_HYPERV" => "SNAPMANAGER_SUITE",
    "SNAPMANAGER_HYPERV" => "SNAPMANAGER_SUITE",
    "SNAPMANAGER_ORACLE" => "SNAPMANAGER_SUITE",
    "SNAPMANAGER_SAP" => "SNAPMANAGER_SUITE",
    "SNAPMANAGER_VI" => "SNAPMANAGER_SUITE",
    "SNAPDRIVE_UNIX" => "SNAPMANAGER_SUITE",
    "FLEXCLONE" => "FLEXCLONE",
    "FLEX_CLONE" => "FLEXCLONE",
    "SNAPLOCK" => "SNAPLOCK",
    "SV_ONTAP_PRI" => "SNAPVAULT",
    "SV_ONTAP_SEC" => "SNAPVAULT",
    "SNAPRESTORE" => "SNAPRESTORE",
    "SNAPMIRROR" => "SNAPMIRROR",
};

## @name get_packages
## @description Returns array that contains all the package names
#
## @synopsis
## get_packages( )
##
sub get_packages {
    my @packages = keys ( %{$v2_licenses->{1-81-000000}} );

    return ( wantarray ? @packages : "@packages" );
}

## @name map_v1_to_v2
## @description Takes in a v1 feature string and returns a package name
## if it can be mapped otherwise returns undef. The input string is
## case insensitive
##
## @synopsis
## map_v1_to_v2( 'snaprestore' )

sub map_v1_to_v2 {
    my $v1_feature = uc( shift @_ );

    return $v1_to_v2_mapping->{$v1_feature};
}

## @name get_licenses
## @description Returns array that contains all the feature license keys
##
## @synopsis
## get_licenses( version => 2, serial_number => "1-81-000000" );
##
## @version            Optional: This is to flag a SN ONTAP, defaults to 0
## @serial_number      Optional: serial number that the licenses belong to
##                               Has defaults
## @features           Optional: list of features for which the licenses are
##                               to be looked up. (hashref to array)
## @cmode              Optional: 0 by default.
##
## Notes
## serial_number is ignored for 7 mode
##
sub get_licenses {
    my %params = (
        version => 1,
        serial_number => undef,
        features => undef,
        cmode => 0,
        @_,
    );
    my @lic;
    my $hash;
    my @flist;

    if ( $params{version} == 2 || $params{cmode} ) {
        my @temp = _hash_serial_number( $params{serial_number},$params{version} );
        $hash = $temp[1];
        if ( !$hash ) {
            return undef;
        }
    } else {
        $hash = $licenses7m->{no_serial_number};
    }

    if ( ref($params{features}) eq 'ARRAY' && @{$params{features}} ) {
        @flist = @{$params{features}};
    } else {
        @flist = keys %{$hash};
    }

    foreach my $l (@flist) {
        if ( $l ne 'BASE' && exists $hash->{$l} ) {
            push( @lic,$hash->{$l} );
        }
    }

    return ( wantarray ? @lic : "@lic" );
}

## @name get_base_license
## @description Returns the base license key
##
## @synopsis
## get_base_license( version => 2, serial_number => "1-81-000000" );
##
## @version            Optional: This is to flag a SN ONTAP, defaults to 0
## @serial_number      Optional: serial number that the licenses belong to
##
sub get_base_license {
    my %params = (
        serial_number => undef,
        version => 1,
        @_,
    );

    my @hash = _hash_serial_number( $params{serial_number},$params{version} );

    return $hash[0]->{BASE};
}

## Private method to get serial number hash that has the license keys
## Called for C mode processing only
##
sub _hash_serial_number {
    my ($serial_num, $version) = @_;

    my $hash;
    my ($cluster_base,$feature);

    if ( $version == 2 ) {
        $hash = $v2_licenses;
    } elsif ( $version == 1 ) {
        $hash = $licenses;
    } else {
        return( undef,undef );
    }

    if ( !$serial_num ) {
        $cluster_base = $hash->{$hash->{default_base}};
        $feature = $hash->{$hash->{default}};
    } else {
        $cluster_base = $hash->{$serial_num};
        $feature = $cluster_base;
    }

    return ( $cluster_base, $feature );
}

## @name get_license_attributes_from_key
## @description Returns the serial number given a license key
##
## @synopsis
## get_license_attributes_from_key( key => "HXHLTSYLTDIFMA" );
##
## @key         Required: License key
##
sub get_license_attributes_from_key {
    my %params = (
        key => undef,
        @_
    );

    my @search = ( $licenses, $v2_licenses, $licenses7m );
    my $version = 1;

    foreach my $hash ( @search ) {
        foreach my $s ( keys %{$hash} ) {
            if ( $s eq 'default' || $s eq 'default_base' ) {
                next;
            }
            foreach my $f ( keys %{$hash->{$s}} ) {
                if ( $params{key} eq $hash->{$s}->{$f} ) {
                    if ( $hash == $v2_licenses ) {
                        $version = 2;
                    }
                    return (
                        {
                            serial_number => $s,
                            feature => $f,
                            version => $version,
                            key => $params{key},
                        }
                    );
                }
            }
        }
    }

    return ( {key => $params{key}} );
}

## @name get_default_serial_number
## @description Returns the default serial number given version
##
## @synopsis
## get_default_serial_number( version => 1 );
## @description Returns the default serial number given version
##
## @synopsis
## get_default_serial_number( version => 1 );
##
## @version         Optional: Version 1|2
sub get_default_serial_number {
    my %params = (
        version => 1,
        @_,
    );

    if ( $params{version} == 2 ) {
        return $v2_licenses->{default_base};
    } elsif ( $params{version} == 1 ) {
        return $licenses->{default_base};
    }
}

