# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/label_assim.pm#1 $ 
## @summary 
## Lib containing function used in label and assimilation tests suite
## @description
## This lib contains helper function for label and assimilation test suites.
##
## @author ratnesh@netapp.com, dl-sar-qa@netapp.com, dl-step@netapp.com
##

use Cwd;
use boot;
use raidv2_lib;

use vars('$FILER', '$FILER_CONN');
param("FILER","-mesg", "hostspec of FILER");
param ("FILER_CONN","-default", "#telnet");

my @sim_disks_files;
my $TIMEOUT="30";

#
# Prepare for the test. Check if specified volume exists. If not, create it.
# It may fail the test if the volume needs to be created and there is less 
# than specified no. of spares disks available. Or, if the volume exists and 
# it couldn't be brought online
# Argument: vol_name
# Argument: disk count
# Return: none
#

sub lat_do_test_setup($$) {
	my ($vol_name) = $_[0];
	my $disks_needed = $_[1];

	if ($vol_name eq "") {
		# shouldn't be here, still fail the test if vol_name is missing
		tstlog -fatal, "no volname is specified in test setup \n";
	}

	if (&lat_vol_exists($vol_name)) {
		tstlog("-comment", "volume $vol_name already exists \n");
		return 0;
	}

	# 
	# volume doesn't exists. Try to create it 
	# 
	if ($disks_needed <= 1) { 
		tstlog('-fatal', "invalid no of disks to create a vol \n"); 
	}

	raidv2_info_disk_load();
	my @spare_disks = split(",", $raidv2_info_tree_disk_list{"POOL:Pool0"});
	tstlog("-comment", "no. of disks needed:  $disks_needed\n");
	tstlog("-comment", "Pool0 spare disks are: @spare_disks\n");


	#index of last element: $#
	my $avail_disks = $#spare_disks + 1;
	tstlog("-comment", "count of spare disk available is $avail_disks\n");

	if ($avail_disks < $disks_needed) {
		#
		# can halt the simulator, create more disks and boot. For now, 
		# fail the test
		#
		tstlog("-fatal", "$disks_needed spare disk is needed. but " .
			"only $avail_disks disks available\n");
	} elsif ($avail_disks > $disks_needed) {
		$avail_disks = $avail_disks - 1;
		if ($avail_disks > 10) {
			$avail_disks = 10;
		}
	}

	# create the volume with all but one available disks
	&lat_create_volume($vol_name, $avail_disks);
	return 1;
}


#
# check if specified volume exists and it is in normal, degraded or recons state
# Argument: volume name
# Return 0 or 1
#
sub lat_vol_exists($) {
	# make sure that vol name passed is unique in the output of 'vol status'
	my $vol_name=$_[0];
	raidv2_info_vol_load();

	if (!$raidv2_info_vol_by_name{$vol_name}) {
		tstlog -comment, "lat_vol_exists: volume $vol_name doesn't ".
				 " exist";
		return 0;
	}

	my $vol_state = $raidv2_info_vol_by_name{$vol_name}{"state_name"}; 
	tstlog -comment, "vol $vol_name state $vol_state";
	
	#vol exists but not online
	if ($vol_state !~ /online/i) {
                tstlog -comment,
               		"volume $vol_name is not online. State is $vol_state".
			" trying to bring the volume online";
		$my_status = raidv2_vol_online($vol_name);
		if (!$my_status) {
			tstlog -fatal, "couldn't online volume: $vol_name";
		}
		return 1;
        } elsif ($vol_state eq "online") {
		return 1;
	}

	tstlog -comment, "lat_vol_exists: vol $vol_name doesn't exist";
	return 0;
}

#
# Create a volume as specified by argments
# Argument: 
# Volume name
# no of disks
#
sub lat_create_volume($) {
	my ($vol_name) = $_[0];
	my ($no_disk) = $_[1];
	if (!lat_vol_exists($vol_name)) {
		my $vol_create_args = "$vol_name -f $no_disk";
		tstlog -comment, "creating vol $vol_create_args\n";

		#
		#raidv2_vol_create return 1 on success, 
		#0 on failure to create the volume
		#
		if (!raidv2_vol_create($vol_create_args)) {
			tstlog -fatal, 
				"Could not create volume ($vol_create_args) \n";
		}

		#
		# verify that vol exists
		#
		if (!lat_vol_exists($vol_name)) {
			tstlog -fatal, 
				"didn't find volume '$vol_name' \n";
		}
	}
}

#
# return a randomly selected filesystem disk of the specified volume. 
#
sub lat_get_a_filesystem_disk($) {
	my ($vol_name) = $_[0];
	#----------------------------------
	# get all disks from specified vol_name
	#----------------------------------
	raidv2_info_disk_load();
	my @all_disks = split(",", $raidv2_info_tree_disk_list{$vol_name});

	if ($#all_disks < 0) { 
		tstlog -fatal, "Could not find any disk in $vol_name \n"; 
	}

	my $which_disk = int(rand ($#all_disks + 1));
	return $all_disks[$which_disk];
}


#
# Argument: disk name
# returns the secondary name of the disk
# e.g. v0.1 secondary name is v4.1 if the disk is dual path
# It is needed due to the fact that simpulled disk name may be the primary name
# of the disk.
#
sub lat_get_disk_sec_name($) {
	my $disk_name = $_[0];
	my $ret = saycmd_real($FILER, 
		"priv set -q diag; storage show disk $disk_name", 
		$TIMEOUT);
	my @list = split(/\r?\n/, $ret);
	my $entry;
	foreach $entry (@list) {
		if ( $entry =~ /Sec Name:/) {
			my @tokens = split(/Sec Name:/, $entry);
			if ($#tokens != 1) {
				#no disk name is specified in the output
				return "";
			}
			return &lat_trim($tokens[1]);
		}
	}
	return "";
}

#
# returns the disk file name of the simulator disk which was simpulled earlier.
# This file name can be used to do simpush for the specified
# disks
# Argument: disk name e.g. v0.4, disk secondary name e.g. v4.4
# Returns: disk filename which is needed in order to do disk simpush e.g.
# v0.4:NETAPP__:VD-16MB_________:26420606:87168
# or "" if no matching disks found.
#
sub lat_get_simpush_disk_file_name($$) {
	my ($disk_name, $disk_sec_name) = @_;

	my $ret = saycmd_real($FILER, 
		"priv set -q test;disk simpush -l", $TIMEOUT);
	tstlog -comment, "simpush ret $ret \n";
	my @list = split(/\r?\n/, $ret);
	my $e;
	$disk_name .= ":";
	$disk_sec_name .= ":";
	
	foreach $e (@list) {
		#try disk name or disk secondary name to match file name
		if (($e =~ /\b$disk_name\b/) || ($e =~ /\b$disk_sec_name\b/)) {
			return &lat_trim($e);
		} 
	}
	return "";
}

#
# remove leading and trailing spaces, newline
#
sub lat_trim($) {
	local ($_) = @_; 
	s/^\s+//; 
	s/\s+$//; 
	return $_;
}

sub lat_num_pool0_spare_disk() { 
	raidv2_info_disk_load(); 
	my @spare_disks = split(",", $raidv2_info_tree_disk_list{"POOL:Pool0"});	
        #index of last element: $#
        return ($#spare_disks + 1);
}

#rename a file
#Arg:
#from: source path 
#to : destinaion path
#
sub my_rename($$) {
	#since rename is failing due to permission denied, exec shell command 
	#using sudo
	my ($from, $to ) = @_;
	my $ccdir = getcwd();
	tstlog -comment, "\nmoving $from to $to\n";
	`sudo mv -f $from $to`;
	return 1;
}

#
# Build sim_disks_files to be used by other methods
# Return count of simulator disks 
#
sub lat_get_simulator_disk_count($) {
	#sim disk is like v0.8:NETAPP__:VD-35MB-FZ-520__:98284907:124544
	my $sim_disk_file_name_pattern = "*:NETAPP*:*:*";

	my $cur_dir = getcwd();
	my $sim_dir = @_[0];

	#if not in ,disks directory, cd now to ,disks directory
	if ($cur_dir ne "$sim_dir/,disks") {
		if ( -e "$sim_dir/,disks" ) {
			chdir "$sim_dir/,disks";
		} else {
			tstlog -fatal, 
				"directory $sim_dir/,disks doesn't exist";
		}
	}

	@sim_disks_files = glob $sim_disk_file_name_pattern ;

	#change back to original dir
	if ($cur_dir ne "$sim_dir/,disks") {
		chdir $cur_dir;
	}
	return $#sim_disks_files + 1;
}

#
# check to make sure at least as many sim disks as specified by arg are 
# available.  Also create the ,pulled directory and remove ,reservation file
#Argument: no of disks that should be in ,disks directory
#return NONE
#

sub lat_verify_test_prerequisites($$) {
	my ($disks_needed, $sim_dir) = @_;
	#sim disk is like v0.8:NETAPP__:VD-35MB-FZ-520__:98284907:124544
	my $sim_disk_file_name_pattern = "*:NETAPP*:*:*";

	#need to have at least five disks
	my $cur_dir = getcwd();

	#if not in ,disks directory, cd now to ,disks directory
	if ($cur_dir ne "$sim_dir/,disks") {
		if ( -e "$sim_dir/,disks" ) {
			chdir "$sim_dir/,disks";
		} else {
			tstlog -fatal, 
				"directory $sim_dir/,disks doesn't exist";
		}
	}

	my $avail_sim_disks = lat_get_simulator_disk_count($sim_dir);
	tstlog -comment, "There are $avail_sim_disks sim disk available\n";
	if ($avail_sim_disks < $disks_needed ) {
	     tstlog -fatal, 
		"The test needs atleast $disks_needed sim disks. ".
		"Found $avail_sim_disks \n";
	}

	#create or empty ,pulled directory
	if (-e ",pulled") {
		#remove all simulator disk files from ,pulled directory
		#unlink glob ",pulled/$sim_disk_file_name_pattern";
	} else {
		mkdir ",pulled", oct("0755");
	}

	#remove ,reservation file
	if ( -e ",reservations" ) {
		unlink ",reservations" 
			or tstlog -fatal, 
			"failed to unlink ,reservation file. error $!";
	} 

	#change back to original dir
	if ($cur_dir ne "$sim_dir/,disks") {
		chdir $cur_dir;
	}
}

# Return disk file name for the disk specified by disk name
# It uses sim_disks_files which was build by lat_verify_test_prerequisites()
#
#Arg:
#disk name e.g. v0.0
#disk sec name e.g. v4.0
#return
#disk file name e.g. v0.0:NETAPP...
#
sub lat_get_sim_disk_file_name($$$) {
	my ($diskname, $disk_secname, $sim_dir) = @_;
	my $e;

	if ($diskname eq "") {
		$diskname = "dummy";
	}
	if ($disk_secname eq "") {
		$disk_secname = "dummy";
	}
	$diskname .= ":";
	$disk_secname .= ":";

	if ($#sim_disks_files == -1) {
		#sim_disks_files is not initialized. Initialize it now.
		lat_get_simulator_disk_count($sim_dir);
	}
	
	foreach $e (@sim_disks_files) {
		#try disk name or disk secondary name to match file name
		if (($e =~ /\b$diskname\b/) || ($e =~ /\b$disk_secname\b/)) {
			return &lat_trim($e);
		} 
	}
	return "";
}

# Return first ip address of the host specified by hostname
#
#Arg:
#hostname e.g. "host" or "host1.domain" or "a.b.c.d"
#return
#host address as "a.b.c.d"
#
sub lat_get_host_ip_addr($) {
	my $hostname = @_[0];
	if (!defined($hostname) || $hostname eq "") {
		tstlog -fatal, 
		    "Can't get host address as specified hostname is invalid.";
	}
	my $addr = gethostbyname($hostname);
	my $first, $sec, $third, $fourth;
	($first, $sec, $third, $fourth) = unpack('C4', $addr);

	return "$first"."."."$sec"."."."$third"."."."$fourth";
}

#
#make sure that script is not invoked from same host as simhost defined in 
#simulator ntest configuration file
#
sub lat_check_sim_host($) {
	my $sim_host = @_[0];
	my $curhost = `/bin/uname -n`; 
	if (defined ($curhost) && defined($sim_host)) { 
		chomp($curhost); 
		if ($curhost eq $sim_host) { 
			tstlog -fatal, "Script is invoked on same host as " .
				  "simhost. Rsh will fail in this condition. ".
				  "Please run it from any other host\n"; 
		} 
	}
}

#
# Make sure all parameter needed by setup and newfs are specified.
# If necessary, modify them using the defaults. 
#
sub lat_ensure_setup_parameters_values($) {
	my ($host) = @_;

	my $host_name = $host->{hostname};
	my $host_ipaddress = $host->{default_ip};

	#if nothing is defined, bail out
	if (!defined($host_name) || $host_name eq "") {
		tstlog -fatal, "No hostname defined\n";
	}

	$host_name = &lat_trim($host_name);
	if (!defined($host_ipaddress) || $host_ipaddress eq "") {
		$host_ipaddress = $host_name;
	}

	#specified ip address may be name, convert it to  a.b.c.d
	$host_ipaddress = lat_get_host_ip_addr($host_ipaddress);
	if ($host_ipaddress eq "...") {
		tstlog -fatal, "couldn't resolve $host_name.\n";
	}

	#discard domain name from hostname if there
	$host_name = (split(/\./, $host_name))[0];
	
	#set FILER_IP_ETH
	if (!defined($FILER_IP_ETH) || $FILER_IP_ETH eq "") {
		$FILER_IP_ETH = $host_ipaddress;
	}

	if (!defined($FILER_HOSTNAME) || $FILER_HOSTNAME eq "") {
		$FILER_HOSTNAME = $host_name;
	}
	
	if ($host->hosttype =~ /simulator/) {
		&setup_simulator_params();
	} else {
		&setup_filer_params($host_ipaddress);
	}

	tstlog -comment, "Host name $FILER_HOSTNAME";
	tstlog -comment, "Host ipadress $FILER_IP_ETH";
	tstlog -comment, "ethernet interface name $FILER_IFC_ETH";
	tstlog -comment, "ethernet interface netmask $FILER_NETMASK_ETH";
	tstlog -comment, "gateway address $FILER_GATEWAY";
	tstlog -comment, "FILER_IP ip addr $FILER_IP \n";
	tstlog -comment, "UPGRADE_IMAGE $UPGRADE_IMAGE";
	tstlog -comment, "CUR_IMAGE $CUR_IMAGE";

}

sub setup_filer_params($) {
	my ($host_ipaddress) = @_;
	#specify netmask
	if (!defined($FILER_NETMASK_ETH) || $FILER_NETMASK_ETH eq "") {
		$FILER_NETMASK_ETH = "255.255.255.0";
	}

	#specify GATEWAY
	if (!defined($FILER_GATEWAY) || $FILER_GATEWAY eq "") {
		my ($first, $sec, $third, $fourth) = split(/\./, 
							$host_ipaddress);
		$FILER_GATEWAY = "$first.$sec.$third.1";
	}

	#define interface name
	
	if (!defined($FILER_IFC_ETH) || $FILER_IFC_ETH eq "") {
		$FILER_IFC_ETH = "e0";
	}

	#modify CUR_IMAGE AND UPGRADE_IMAGE
	if (defined($UPGRADE_IMAGE) && !($UPGRADE_IMAGE =~ /$host_ipaddress/)) {
		$UPGRADE_IMAGE = $UPGRADE_IMAGE . ",$host_ipaddress";
	}
	if (defined($CUR_IMAGE) && !($CUR_IMAGE =~ /$host_ipaddress/)) {
		$CUR_IMAGE = $CUR_IMAGE . ",$host_ipaddress";
	}

	$SIMARGS = "";
}

sub setup_simulator_params() {
	if (!defined($FILER_NETMASK_ETH) || $FILER_NETMASK_ETH eq "") {
		$FILER_NETMASK_ETH = "255.255.240.0";
	}
	
	#specify GATEWAY
	if (!defined($FILER_GATEWAY) || $FILER_GATEWAY eq "") {
		$FILER_GATEWAY = "10.56.16.1";	
	}
	
	#define interface name
	if (!defined($FILER_IFC_ETH) || $FILER_IFC_ETH eq "") {
		$FILER_IFC_ETH = "ns0";
	}

	if (!defined($SIMARGS) || $SIMARGS eq "") {
		$SIMARGS = "-z";
	}
}

#
# reinit all the disks, create a new filesystem
# use the boot option specified by input parameter
# Args:
#host param: 
#host connection
#boot option
#
sub lat_reinitialize_system($$$) {
	my ($fh, $fc, $command) = @_; 
	newfs($fh,prefix=>"FILER",command=>"$command") or tstlog('-fatal',$@); 
	tstlog -comment, "newfs done"; 
	basic_setup($fc, prefix=>"FILER") or tstlog('-fatal',$@); 
	tstlog -comment, "basic setup done"; 
	wait_for_rsh($fh) or tstlog('-fatal',$@);

	tstlog("-comment", "Boot done"); 
	my $res = saycmd_real($FILER, "version", $TIMEOUT); 
	tstlog("-comment", "version is $res");
}

#do floppy boot 4
sub lat_floppy_boot_4($$) {
	my ($fh, $fc) = @_;
	lat_reinitialize_system($fh, $fc, "4");
}

#do boot 24/7
sub lat_24_7_boot($$) {
	my ($fh, $fc) = @_;
	lat_reinitialize_system($fh, $fc, "24/7");
}

