# Copyright (c) 2011 Network Appliance, Inc.
# All rights reserved.
#
## @summary Automated testcases created for the Roster Distribution
## infrastructure of the Leagues Infrastructure project (chunk 1)
##
## @description
##  This file contains common functions to be re-used for automation within
##  .thpl files created for the leagues infrastructure.
##
## @keywords leagues,fs.0,chunk1
##
## @burt 529872 Tracks creation and modification of initial script.
##
## @author mshishir@netapp.com

# TODO
# - No hardcoded values
package league_common;
# Compiler directives.
use strict;

# Module imports
use TCD;
use Tharn qw(sleep);
use Options;

use Data::UUID;

# NACL APISet
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);

# NACL Components
use NACL::C::Node ;
use NACL::C::Cluster ;
use NACL::C::NetworkInterface ;

# NACL Component State
use NACL::CS::ClusterIdentity ;

use Intercluster ;


use vars ('@ISA','@EXPORT');

require Exporter;

@ISA = ('Exporter');

@EXPORT = ('_create_league', 
           '_modify_league', 
           '_modify_member', 
           '_contained_in_league_test_internal', 
           '_contained_in_league_replicated_table', 
           '_contained_in_roster_replicated_table', 
           '_contained_in_roster_replicated_table_extra', 
           '_delete_league',
           '_delete_member', 
           '_distribute_roster', 
           '_add_fake_member', 
           '_add_real_member', 
           '_generate_uuid', 
           '_add_active_session', 
           '_check_distributed_league_table', 
           '_check_dummy_league_table', 
           '_dummy_league_distributed_view',
           '_delete_dummy_league_table',
           '_create_real_league',
           '_apply_for_league_membership',
           '_accept_application_for_league_membership',
           '_get_league_uuid',
           '_get_member_uuid',
           '_basic_roster_cleanup');

# Global Variables.
use vars qw($Test @Testcases %Apisets %Clusters %Nodes $Intercluster_obj);

########## PRIVATE #################################################
# PRIVATE METHODS

# Given a list of limited league creation parameters, run the
# 'league create' command to actually create a league and use
# all the intercluster lifs configured with the cluster. This will
# also create an entry for the master in the league roster.
sub _create_real_league {
    my $league_name = shift;
    my $admin_contact = shift;
    my $comment = shift;
    my $keyphrase = shift;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    $Test->step('Create a real league: ' . $league_name . 
                'with keyphrase: '. $keyphrase) ;
    my $cmd = 'league create -league-name ' . $league_name . ' -admin-contact ' .
              $admin_contact . ' -comment ' . $comment . ' -keyphrase ' .
              $keyphrase;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return ;
    } else {
        my $league_uuid1 = _get_league_uuid($league_name,
                                            $Test,
                                            \$apiset);
        return $league_uuid1 ;
    }
}
# Apply for membership to a league, specifically sending the request to the 
# league master
sub _apply_for_league_membership {
    my $league_name = shift;
    my $cluster_ref = shift;
    my $cluster = $$cluster_ref;
    my $keyphrase = shift;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    my @net_ints = NACL::C::NetworkInterface->find(
                      command_interface => $cluster,
                      filter => { role => 'intercluster' }) ;

    my @ips ;
    foreach my $int (@net_ints) {
        push (@ips, $int->get_one_state_attribute('address')) ;
    }

    $Test->step('Create an application to league: ' . $league_name .
                'on ip address: '. $ips[0]) ;

    my $cmd = 'league apply -league-name ' . $league_name . ' -master-address ' .
              $ips[0] . ' -keyphrase ' . $keyphrase;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return 0;
    } else {
        return 1 ;
    }
}
# Accept an application for membership to a league on the league master.
sub _accept_application_for_league_membership {
    my $league_name = shift;
    my $league_uuid = shift;
    my $keyphrase = shift;
    my $remote_cluster_name = shift;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    $Test->step('Accept an application to league: ' . $league_name .
                'from cluster: '. $remote_cluster_name) ;
    my $cmd = 'league application accept -league-name ' . $league_name . 
              ' -application-number * -remote-cluster-name ' .
              $remote_cluster_name . ' -keyphrase ' . $keyphrase;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return ;
    } else {
        my $league1_member2_uuid = _get_member_uuid($league_uuid,
                                                    $remote_cluster_name,
                                                    $Test,
                                                    \$apiset);
        return $league1_member2_uuid;
    }
}
# Get the league UUID fom the debug smdb command. This will remain hackinsh till
# we get NACL support for debug smdb.
sub _get_league_uuid {
    my $league_name = shift;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    $Test->step('Retrieving UUID for league: ' . $league_name) ;
    my $cmd = 'debug smdb table league_replicated show -league-name '. $league_name .
              ' -fields league-uuid';
  
    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return ;
    } else {
        my @values = split('X;X',$raw);
        my $league_uuid = trim($values[($#values)-1]);
        $Test->step("League UUID is $league_uuid") ;
        return $league_uuid ;
    }
}
# Get the member UUID from the debug smdb command. This will remain hackish till
# we get NACL support for debug smdb.
sub _get_member_uuid {
    my $league_uuid = shift;
    my $member_name = shift;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    $Test->step('Retrieving UUID for cluster: ' . $member_name . ' belonging to' .
                ' league ' . $league_uuid);
    my $cmd = 'debug smdb table league_roster_replicated show -league-uuid ' .
              $league_uuid . ' -cluster-name ' . $member_name . 
              ' -fields cluster-uuid';

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return ;
    } else {
        my @values = split('X;X',$raw);
        my $member_uuid = $values[($#values)-1];
        $Test->step("Cluster UUID is $member_uuid");
        return $member_uuid;
    }
}
#Delete leaugues, members and active sessions from both specified clusters.
sub _basic_roster_cleanup {
    my $league_name = shift ;
    my $arrayref=shift;
    my @cluster_names=@$arrayref;
    my $Test = shift;
    my $apiset_ref1 = shift;
    my $apiset_ref2 = shift;
    # Delete the leagues created for this test. This causes members to be
    # deleted automatically.
    foreach (@cluster_names) {
        if (!_delete_member($league_name,
                            $_,
                            $Test,
                            $apiset_ref1)) {
             return 0;
        }
    }
    if (!_delete_league($league_name,
                        $Test,
                        $apiset_ref1)) {
        _delete_league($league_name,
                       $Test,
                       $apiset_ref2);
        return 0;
    }

    # Delete the leagues created for this test. This causes members to be
    # deleted automatically.
    if (!_delete_league($league_name,
                        $Test,
                        $apiset_ref2)) {
        return 0;
    }

    return 1;
}
# Given a list of league creation parameters,run the
# 'league test league create' test command which returns the UUID of the
# new league, or undef if the league creation resulted in an errror.
sub _create_league {
    my $league_name = shift;
    my $passphrase = shift;
    my $keyphrase = shift;
    my $ipspace = shift;
    my $contact = shift;
    my $comment = shift;
    my $lg_status = shift;
    my $updates_enabled = shift;
    my $roster_version =  shift;
    my $Test = shift;    
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    my $uuid = _generate_uuid() ;

    if (!defined $uuid) {
        return ;
    }

    $Test->step('Create league: ' . $league_name . 'with UUID: '. $uuid) ;

    my $cmd = 'league test league create -league-name ' . $league_name . ' ' .
        '-league-uuid ' . $uuid . ' -passphrase ' . $passphrase .
        ' -keyphrase ' . $keyphrase . ' -league-ipspace ' . $ipspace .
        ' -league-job-uuid ' . $uuid . ' -admin-contact ' . $contact .
        ' -comment ' . $comment . ' -self-league-status ' . $lg_status .
        ' -updates-enabled ' . $updates_enabled . ' -roster-version ' . $roster_version;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return ;
    } else {
        return $uuid ;
    }
}
# Check if a cluster is registered in the distributed league table after an
# active session for the same has been created. Return a 0 if no cluster is
# found or return 1 if a cluster is successfully found.
sub _check_distributed_league_table {
    my $cluster_uuid = shift ;
    my $cluster_name = shift;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    $Test->step("Checking if cluster $cluster_uuid is found in the league distributed table in cluster $cluster_name") ;
    my $cmd = 'debug smdb table league show ' ;
       

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    my $result = ($raw =~ /$cluster_uuid/i) ;

    $Test->step("Checking for $cluster_uuid in distributed league table of $cluster_name" . ($result ? '(Yes)' : '(No)')) ;

    return $result ;
}
# Perform a bunch of operations on the league dummy view. Depending on the
# context of the operation passed in, command arguments are parsed accordingly
# and then the command gets executed. Any failure in execution is reported back
# with a 0 and all successes are reported back with a 1.
sub _dummy_league_distributed_view {
    my $arrayref=shift;
    my @args=@$arrayref;
    my $operation = shift;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    if ($operation eq "show") {
        $Test->step("Displaying entries in the view") ;
        my $cmd = 'league test application_registration show' ;

        my $raw = $apiset->execute_raw_command(command => $cmd) ;

        my $result = ($raw =~ /$args[0]/i) ;

        $Test->step("Checking for league uuid $args[0] in view." . ($result ? '(Yes)' : '(No)')) ;

        if ($result == 0) {
            return $result;
        }

        my $result = ($raw =~ /$args[1]/i) ;

        $Test->step("Checking for cluster uuid $args[1] in view." . ($result ? '(Yes)' : '(No)')) ;

        if ($result == 0) {
            return $result;
        }

        my $result = ($raw =~ /$args[2]/i) ;

        $Test->step("Checking for application_id $args[2] in view." . ($result ? '(Yes)' : '(No)')) ;

        if ($result == 0) {
            return $result;
        }

        my $result = ($raw =~ /$args[3]/i) ;

        $Test->step("Checking for application_name $args[3] in view." . ($result ? '(Yes)' : '(No)')) ;

        return $result ;

    } elsif ($operation eq "create") {
        $Test->step("Creating entries in the view") ;
        my $cmd = "league test application_registration create $args[0] $args[1] $args[2] $args[3] $args[4]" ;

        my $raw = $apiset->execute_raw_command(command => $cmd) ;

        if ($raw =~ /Error/) {
            return 0 ;
        } else {
            return 1;
        }
    } elsif ($operation eq "modify") {
        $Test->step("Modifying entries in the view") ;
        my $cmd = "league test application_registration modify -league $args[0] -peer $args[1] -application-id $args[2] -application-name $args[3] -application-time $args[4]" ;

        my $raw = $apiset->execute_raw_command(command => $cmd) ;

        if ($raw =~ /Error/) {
            return 0;
        } else {
            return 1;
        }
    } elsif ($operation eq "delete") {
        $Test->step("Deleting entries in the view") ;
        my $cmd = "league test application_registration delete -league $args[0] -peer $args[1] -application-id $args[2]" ;

        my $raw = $apiset->execute_raw_command(command => $cmd) ;

        if ($raw =~ /Error/) {
            return 0;
        } else {
            return 1;
        }
    } else {
        $Test->step("Invalid operation requested on application_registration table: $operation");
    }
    return 0;
}
# Check the dummy league table to see if the value we were trying to create
# exists. If found, return 1 for success else return 0 for failure.
sub _check_dummy_league_table {
    my $application_id = shift;
    my $application_name = shift;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    $Test->step("Checking if cluster $application_id & $application_name is found in the dummy league table") ;
    my $cmd = 'debug smdb table test_xc_dsmf_table show ' ;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    my $result = ($raw =~ /$application_id/i) ;

    $Test->step("Checking for $application_id in dummy league table" . ($result ? '(Yes)' : '(No)')) ;
 
    if ($result == 0) {
        return $result;
    }

    my $result = ($raw =~ /$application_name/i) ;

    $Test->step("Checking for $application_name in dummy league table" . ($result ? '(Yes)' : '(No)')) ;

    return $result ;
}
# Delete all entries in the dummy league table
sub _delete_dummy_league_table {
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    $Test->step("Deleting all entires in the dummy league table") ;
    my $cmd = 'debug smdb table test_xc_dsmf_table delete * ' ;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return 0;
    } else {
        return 1 ;
    }
}
# Add an active session to a specific member (cluster) of the specified league
# Return a 0 if there is an error in creating the active session else return 1.
sub _add_active_session {
    my $league_name = shift ;
    my $cluster_name = shift;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;    

    $Test->step("Adding active session to $cluster_name belonging to league: $league_name") ;
    my $cmd = 'league test active-peer create ' .
        '-league-name ' . $league_name .' ' .
        '-cluster-name ' . $cluster_name ;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return 0;
    } else {
        return 1 ;
    }
}
#
# Given a list of league modification parameters, run the league test league
# modify command.
#
sub _modify_league {
    my $league_uuid = shift;
    my $league_name = shift;
    my $passphrase = shift;
    my $keyphrase = shift;
    my $ipspace = shift;
    my $contact = shift;
    my $comment = shift;
    my $lg_status = shift;
    my $updates_enabled = shift;
    my $roster_version =  shift;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    my $uuid = _generate_uuid() ;

    if (!defined $uuid) {
        return ;
    }

    $Test->step('Create league: ' . $league_name . 'with UUID: '. $uuid) ;

    my $cmd = 'league test league update -league-name ' . $league_name . ' ' .
        '-league-uuid ' . $league_uuid . ' -passphrase ' . $passphrase .
        ' -keyphrase ' . $keyphrase . ' -league-ipspace ' . $ipspace .
        ' -league-job-uuid ' . $uuid . ' -admin-contact ' . $contact .
        ' -comment ' . $comment . ' -self-league-status ' . $lg_status .
        ' -updates-enabled ' . $updates_enabled . ' -roster-version ' . $roster_version;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return 0;
    } else {
        return 1;
    }
}
#
# Given a list of member modification parameters, run the league test member
# modify.
sub _modify_member {
    my $league_uuid = shift;
    my $cluster_uuid = shift;
    my $cluster_name = shift;
    my $shared_secret = shift;
    my $roster_dist_version = shift;
    my $roster_dist_timestamp = shift;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    $Test->step('Modify member: ' . $cluster_name . 'with UUID: '. $league_uuid) ;

    my $cmd = 'league test member update -cluster-name ' . $cluster_name . ' ' .
        '-league-uuid ' . $league_uuid . ' -cluster-uuid ' . $cluster_uuid .
        ' -shared-secret ' . $shared_secret . ' -roster-dist-version ' . $roster_dist_version .
        ' -roster-dist-timestamp ' . $roster_dist_timestamp;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return 0;
    } else {
        return 1;
    }
}
# Given a league UUID, and a node string, returns 0 if the UUID is not
# shown by the 'league test internal show' command on that node
sub _contained_in_league_test_internal {
    my $league_uuid = shift ;
    my $league_name =  shift ;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    my $step = "Query league test internal for '$league_uuid' " .
        "on " . $apiset->get_name  . " " ;

    my $cmd = 'league test internal show -league-uuid ' . $league_uuid;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    my $result = ($raw =~ /$league_uuid/i) ;

    $Test->step($step . ($result ? '(Yes)' : '(No)')) ;

    if ($result == 0) {
        return $result;
    }

    my $result = ($raw =~ /$league_name/i) ;

    $Test->step($step . ($result ? '(Yes)' : '(No)')) ;

    return $result ;

}
# Given a league UUID, and a node string, returns 0 if the UUID is not
# shown by the 'debug smdb table league_replicated show' command on
# that node
sub _contained_in_league_replicated_table {
    my $uuid = shift ;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    my $step = "Query league_replicated_table for '$uuid' " .
        "on " . $apiset->get_name  . " " ;

    my $cmd = 'debug smdb table league_replicated show -league-uuid ' . $uuid;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;


    my $result = ($raw =~ /$uuid/i) ;

    $Test->step($step . ($result ? '(Yes)' : '(No)')) ;

    return $result ;
}
# Given a league UUID, and a node string, returns 0 if the UUID is not
# shown by the 'debug smdb table league_roster_replicated show' command on
# that node
sub _contained_in_roster_replicated_table {
    my $uuid = shift ;
    my $cluster_uuid = shift ;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    my $step = "Query roster_replicated_table for '$uuid' " .
        "on " . $apiset->get_name  . " " ;

    my $cmd = 'debug smdb table league_roster_replicated show ' .
        '-league-uuid ' . $uuid;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    my $result = ($raw =~ /$cluster_uuid/i) ;

    $Test->step($step . ($result ? '(Yes)' : '(No)')) ;

    return $result ;
}
# Given a league UUID, and a node string, returns 0 if the UUID is not
# shown by the 'debug smdb table league_roster_replicated show' command on
# that node
sub _contained_in_roster_replicated_table_extra {
    my $league_uuid = shift ;
    my $cluster_uuid = shift ;
    my $node_name = shift ;
    my $shared_key = shift ;
    my $roster_dist_version = shift ;
    my $roster_dist_timestamp = shift ;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    my $step = "Query roster_replicated_table for '$league_uuid' " .
        "on " . $apiset->get_name  . " " ;

    my $cmd = 'debug smdb table league_roster_replicated show ' .
        '-league-uuid ' . $league_uuid . ' -cluster-uuid ' . $cluster_uuid;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    my $result = ($raw =~ /$cluster_uuid/i) ;

    $Test->step($step . ($result ? '(Yes)' : '(No)')) ;

    if ($result == 0) {
        return $result;
    }

    my $result = ($raw =~ /$shared_key/i) ;

    $Test->step($step . ($result ? '(Yes)' : '(No)')) ;

    if ($result == 0) {
        return $result;
    }

    my $result = ($raw =~ /$roster_dist_version/i) ;

    $Test->step($step . ($result ? '(Yes)' : '(No)')) ;

    return $result ;
}
# Delete the member specified by the provided name on the specified node
sub _delete_member {
    my $league_name = shift ;
    my $cluster_name = shift ;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    $Test->step('Delete member: ' . $cluster_name . ' in league: ' . $league_name .
                ' on ' . $apiset->get_name) ;

    my $cmd = 'league member delete -league-name ' . $league_name . ' -cluster-name ' . $cluster_name;
    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return 0 ;
    } else {
        return 1 ;
    }
}

# Delete the league specified by the provided name on the specified node
sub _delete_league {
    my $league_name = shift ;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    $Test->step('Delete league: ' . $league_name .
                ' on ' . $apiset->get_name) ;

    my $cmd = 'league delete -league-name ' . $league_name ;
    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return 0 ;
    } else {
        return 1 ;
    }
}
# Distribute the roster to the node specified.
sub _distribute_roster {
    my $league_uuid = shift ;
    my $cluster_uuid = shift ;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;
    
    $Test->step('Distributing roster to league: ' . $league_uuid . ' cluster: ' . $cluster_uuid .
                ' on ' . $apiset->get_name) ;

    my $cmd = 'league test roster distribute -league-uuid ' . $league_uuid . ' -cluster-uuid ' . $cluster_uuid;
    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return 0 ;
    } else {
        return 1 ;
    }
}
# Add a fake member (cluster) to the specified league
sub _add_fake_member {
    my $league_uuid = shift ;
    my $shared_secret = shift;
    my $roster_dist_version = shift;
    my $time_stamp = shift;
    my $is_master = shift ;
    my $cluster_name = shift;
    my $cluster_ref = shift;
    my $cluster = $$cluster_ref;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    my $cluster_uuid = _generate_uuid() ;

    if (!defined $cluster_uuid) {
        return ;
    }

    my @net_ints = NACL::C::NetworkInterface->find(
                      command_interface => $cluster,
                      filter => { role => 'intercluster' }) ;

    my @ips ;
    foreach my $int (@net_ints) {
        push (@ips, $int->get_one_state_attribute('address')) ;
    }
    my $ips_str = join (',', @ips) ;

    $Test->step("Adding $cluster_name to league: $league_uuid") ;
    my $cmd = 'league test member create ' .
        '-league-uuid ' . $league_uuid .' ' .
            '-cluster-uuid ' . $cluster_uuid . ' ' .
                '-member-cluster-ips ' . $ips_str . ' ' .
                    '-shared-secret ' . $shared_secret . ' ' .
                        '-roster-dist-version ' . $roster_dist_version . ' ' .
                            '-roster-dist-timestamp ' . $time_stamp . ' ' .
                                '-is-master ' . $is_master . ' ' .
                                    '-cluster-name ' . $cluster_name;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return ;
    } else {
        return $cluster_uuid ;
    }
}
# Add a specified member (cluster) to the specified league
sub _add_real_member {
    my $uuid = shift ;
    my $cluster_ref = shift;
    my $cluster = $$cluster_ref;
    my $is_master = shift ;
    my $Test = shift;
    my $apiset_ref = shift;
    my $apiset = $$apiset_ref;

    my $cluster_uuid = NACL::CS::ClusterIdentity->fetch(
                          command_interface => $cluster)->uuid ;
    my $cluster_name = $cluster->name ;

    my @net_ints = NACL::C::NetworkInterface->find(
                      command_interface => $cluster,
                      filter => { role => 'intercluster' }) ;

    my @ips ;
    foreach my $int (@net_ints) {
        push (@ips, $int->get_one_state_attribute('address')) ;
    }
    my $ips_str = join (',', @ips) ;

    my $shared_secret = 'd00dad' ;

    $Test->step("Adding $cluster_name to league: $uuid") ;
    my $cmd = 'league test member create ' .
        '-league-uuid ' . $uuid .' ' .
            '-cluster-uuid ' . $cluster_uuid . ' ' .
                '-member-cluster-ips ' . $ips_str . ' ' .
                    '-shared-secret ' . $shared_secret . ' ' .
                        '-roster-dist-version 2 ' .
                            '-roster-dist-timestamp 123456 ' .
                                '-is-master ' . $is_master . ' ' .
                                    '-cluster-name ' . $cluster_name;

    my $raw = $apiset->execute_raw_command(command => $cmd) ;

    if ($raw =~ /Error/) {
        return 0 ;
    } else {
        return $cluster_uuid ;
    }
}
# Returns a new standard UUID, using the Data::UUID package.
sub _generate_uuid {
    my $ug = new Data::UUID;
    return $ug->create_str() ;
}

# Perl trim function to remove whitespace from the start and end of the string
sub trim($)
{
    my $string = shift;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
}
