#  $Id: //depot/prod/test/Rsierranevada/lib/mount_old.pm#1 $

## @summary Invoke mount(8) and umount(8) idempotently
## @author lincoln@netapp.com
## @status deprecated
## @pod here

=head1 NAME

mount_old - invoke "mount" and "umount" idempotently

=head1 DESCRIPTION

This is an older version of the library "lib/mount.pm".  The newer
version uses NATE rsh connections to run various commands even when
the mount is local, and that does not work very well in tests which
are run as a different user (e.g. root).

When a way exists in NATE to use connections to run processes locally,
the tests which need this library can be made to use the new library,
and this library will be removed.

=cut

package mount_old;
use strict;
use Tharn qw(logcomment logresult);
use base qw(Exporter);
use SPARSE::Util::Logger;
use POSIX ('uname');

# Export list;
our @EXPORT = qw(mount umount);

# Prototypes.
sub mount ( $$% );
sub umount ( $% );
sub _real_nfs_mount_opts ( $ );
sub _do_stat ( $ );
sub _get_dev_stat ( $ );

my ($sysname, $hostname) = POSIX::uname();
my $uid = $>;

# Change following to "/usr/software/bin/sudo" when appropriate
my $sudo = $::SUDO_PATH || "sudo";

sub mount ( $$% ) {
    my ($server, $path, %opts) = @_;
    # A commline telling which server and path we are mounting
    # don't hurt.
    logcomment("Enter mount_old::mount\n\tserver: \"$server\"\n\tpath: \"$path\"\n".
	     "\tuid: \"$uid\"\n\thost: \"$hostname\"");
    my $path_env = `echo \$PATH`;
    chomp($path_env);
    $path_env =~ s/:/\n/g;
    logDbg("mount_old::mount  PATH:\"$path_env\"");
    Tharn::tstlog('-commline',"Mounting ${server}:$path");
    return eval {
	my $type = $opts{type} || "nfs";
	if ($type =~ /^nfs$/i) {
	    $type="nfs";
	} elsif ($type =~ /^device$/i) {
	    $type="device";
	} else {
	    die("mount_old::mount: unknown type \"$type\"");
	}
	my $mountopts = $opts{mountopts};
	my $proto = $opts{proto};
	my $vers = $opts{vers};
	if ($type eq "nfs") {
	    if (!$mountopts) {
		$mountopts = "hard,intr,noquota";
	    }
	    $mountopts .= 
		($proto ? ",proto=$proto" : "") .
		    ($vers ? ",vers=$vers" : "");
	}
	my $mountpoint = $opts{mountpoint};
	if (!$mountpoint) {
	    my $extra = $path;
	    $extra =~ s@^/@@;
	    if (!length($extra)) {
		$extra = "root";
	    }
	    $extra .= "-$proto" if (defined($proto));
	    $extra .= "-$vers" if (defined($vers));
	    $extra =~ tr@ /@_@;
	    $mountpoint = "/t/$server/$extra";
	}

	my $client = $opts{client};
	if ($client) {
	    logDbg("mount_old::mount attempting remote mount on client: \"$client\"");
	    my $sobj = Tharn::subtest("{ use mount; \&mount(\@_) ".
				      "or tstlog('-fatal', \$@) }",
				      '-client',$client,
				      '-onexit',sub {},'--',
				      $server,$path,
				      %opts,
				      mountpoint=>$mountpoint, client=>"");
	    my $result = Rslt::worst($sobj->rslts());
	    if ($result && $result->severity() >
                           NATE::Result::Warn->severity()) {
		die(join("\n", $result->msgs()));
	    }
	    return $mountpoint;
	} else {
	    logDbg("mount_old::mount attempting local mount");
	    my $abovepoint = $mountpoint;
	    $abovepoint =~ s@/[^/]+$@@;
	    if ($type eq "nfs") {
		$mountopts = _real_nfs_mount_opts($mountopts);
	    }

	    logDbg("mount_old  Line: ".__LINE__."   Calling _do_stat \"$abovepoint\"");
	    my $abovedev = _do_stat($abovepoint);
	    logDbg("mount_old  Line: ".__LINE__."   Returned from _do_stat \"$abovepoint\"\n\"$abovedev\"");
	    logDbg("mount_old  Line: ".__LINE__."   Calling _do_stat \"$mountpoint\"");
	    my $mountdev = _do_stat($mountpoint);
	    logDbg("mount_old  Line: ".__LINE__."   Returned from _do_stat \"$mountpoint\"\n\"$mountdev\"");

	    if (defined($abovedev) and defined($mountdev) and $abovedev != $mountdev) {
		# There has already been a mount of some sort on $mountpoint.
		# We'll assume that we or some other equivalent entity doing the
		# proper mount was responsible.
		return $mountpoint;
	    }
	    my $res;
	    if (!defined($mountdev)) {
		if ($! =~ /stale/i) {
		    # there is a mount point, but it seems to be stale.  Try to
		    # umount before mounting it.
		    logDbg("mount_old::mount attempting to unmount before remounting \"$mountpoint\"");
		    umount($mountpoint) 
		        or die("mount point $mountpoint\n" .
			        "  is stale, but umount failed:\n$@");
		} else {
		    logDbg("mount_old::mount attempting to mkdir mountpoint \"$mountpoint\"");
		    my $mkdir_cmd = "$sudo mkdir -p $mountpoint";
	    	    logDbg("mount_old::mount  Line: ".__LINE__."   mkdir_cmd: \n\n$mkdir_cmd");
		    $res = `$mkdir_cmd 2>&1`;
		    if ($res ne '') {
			chomp $res;
			die("mkdir cmd \"$mkdir_cmd\"\n" .
			     "  failed on host \"$hostname\":\n$res");
		    } elsif (! -d $mountpoint) {
			die("mkdir cmd \"$mkdir_cmd\"\n" .
			     "  failed on host \"$hostname\"");
		    }
		}
	    }
	    my $chown_cmd = "$sudo chown $uid $mountpoint";
	    $res = `$chown_cmd 2>&1`;
	    logDbg("mount_old::mount  Line: ".__LINE__."   chown_cmd: \n\n$chown_cmd");
	    if ($res) {
		logresult('WARN', "Error trying to 'chown' $mountpoint to uid $uid: $res");
	    }
	    logDbg("mount_old::mount Line: ".__LINE__."\n\n".`ls -ld $mountpoint`);
	    my @cmd = ($sudo, 'mount');
	    if ($mountopts) {
		push @cmd, ('-o',$mountopts);
	    }
	    push @cmd, ($type eq "nfs") ? "$server:$path" : $path;
	    push @cmd, $mountpoint;

	    my $cmd = join(" ",@cmd);
	    logcomment("mount_old::mount Will attempt local mount with cmd:\n\t$cmd");
        # We may hang while mounting if the mount target is down.
        eval {
            local $SIG{ALRM} = sub { die "Alarm doing mount\n"};
            alarm (60);
	        $res = `$cmd 2>&1`;
        };
        alarm(0);
        if($@ =~ /Alarm/) {
		die ("Timeout while attempting to mount \"$mountpoint\"\n" .
			"  using \"$cmd\"\n  on host \"$hostname\" uid: \"$uid\" ");
        }
	    chomp $res;
	  
	    # we don't use mount's output to check whether the mount is
	    # successful, since we want to return success when there is
	    # already something mounted at this mountpoint (under the
	    # assumption that some other mount command already came in and
	    # mounted this directory).
        
	logDbg("mount_old  Line: ".__LINE__."   Calling _get_dev_stat \"$abovepoint\"");
        ($abovedev) = _get_dev_stat($abovepoint);
	logDbg("mount_old  Line: ".__LINE__."   Returned from _get_dev_stat \"$abovepoint\"\n\"$abovedev\"");
	logDbg("mount_old  Line: ".__LINE__."   Calling _get_dev_stat \"$mountpoint\"");
        ($mountdev) = _get_dev_stat($mountpoint);
	logDbg("mount_old  Line: ".__LINE__."   Returned from _get_dev_stat \"$mountpoint\"\n\"$mountdev\"");
	    if ($abovedev == $mountdev) {
		die ("Failed to mount \"$mountpoint\"\n".
			"  using \"$cmd\"\n  on host \"$hostname\" with uid: \"$uid\":\n" . 
		      ((length($res)>0) ? "mount printed: $res\n" :
					"mount printed no output.\n"));
	    }

	    if ($opts{chmod}) {
		my $perms = $opts{chmod};
	        my $chmod_cmd = "$sudo chmod $perms $mountpoint";
		logDbg("mount_old::mount  Line: ".__LINE__."   chmod_cmd:\n\n$chmod_cmd");
	        $res = `$chmod_cmd 2>&1`;
	        if ($res) {
		    logresult('WARN', "Error trying to 'chmod' $mountpoint to perms $perms: $res");
		}
	    }
	    logcomment("mount_old::mount succeeded for mountpoint: $mountpoint\n\n".`ls -ld $mountpoint`);
	    return $mountpoint;
	}
    };
}


sub umount ( $% ) {
    my ($mountpoint, %opts) = @_;
    logDbg("Enter mount_old::umount\n\tmountpoint: \"$mountpoint\"\n".
	     "\tuid: \"$uid\"\n\thost: \"$hostname\"");
    return eval {
	my $client = $opts{client};
	my $force = $opts{force};
	my $flags = $force?"-f":"";
	if ($client) {
	    logDbg("mount_old::umount attempting remote umount on client: \"$client\"");
	    my $sobj = Tharn::subtest("{ use mount; \&umount(\@_) ".
				      "or tstlog('-fatal', \$@) }",
				      '-client',$client,
				      '-onexit',sub {},'--',
				      $mountpoint,
				       %opts,
				      client=>"");
	    my $result = Rslt::worst($sobj->rslts());
	    if ($result && $result->severity() >
                         NATE::Result::Warn->severity()) {
		die join("\n", $result->msgs());
	    }
	    return 1;
	} else {
	    logDbg("mount_old::umount attempting local umount");
	    my $cmd = "$sudo umount $flags $mountpoint";
	    my $text = `$cmd 2>&1`;
	    $text =~ s/^\s+//;
	    $text =~ s/\s+$//;
	    if ($?) {
	         die ("Failed to unmount \"$mountpoint\"\n" .
			"  using \"$cmd\"\n  on host \"$hostname\" with uid: \"$uid\":\n" . 
			"$text");
	    }
	    logDbg("mount_old::umount succeeded for mountpoint: $mountpoint");
	    return 1;
	}
    };
}


# Given Solaris-style mount options, return what the local host should
# be using instead.

# XXX This also may include a trailing "-t xxx" option for Linux nfsv4

sub _real_nfs_mount_opts ( $ ) {
    my ($old) = @_;
    my ($sysname) = POSIX::uname();
    if ($sysname eq 'SunOS') {
	# (we defined the mount options so that there would be no fixup
	# needed for Solaris)
	return $old;
    } elsif ($sysname eq 'Linux') {
	my $append = "";
	my @new;
	for (split(/,/,$old)) {
	    if (/^noquota$/) {
		# (no corresponding option)
	    } elsif (/^proto=(\w+)/) {
		if ($old =~ /vers=4/) {
		    push @new, "proto=$1";
		} else {
		    push @new, $1;
		}
	    } elsif (/^vers=4/) {
	        # nfsV4 on linux
		$append = " -t nfs4";
	    } elsif (/^vers=(\w+)/) {
		push @new, "nfsvers=$1";
	    } else {
		# (otherwise, assume it's the same option on Linux as Solaris)
		push @new, $_;
	    }
	}
	return join(',',@new) . $append;
    } elsif ($sysname eq 'FreeBSD') {
	my @new;
	for (split(/,/,$old)) {
	    if (/^noquota$/) {
		# (no corresponding option)
	    } elsif (/^hard$/) {
		# (no corresponding option; hard is assumed)
	    } elsif (/^proto=udp/i) {
		push @new, 'mntudp';
	    } elsif (/^proto=tcp/i) {
		push @new, 'tcp';
	    } elsif (/^vers=(\w+)/) {
		push @new, "nfsv$1";
	    } else {
		# (otherwise, assume it's the same option on FreeBSD as Solaris)
		push @new, $_;
	    }
	}
	return join(',',@new);
    } else {
	return $old;
    }
}

sub _get_dev_stat ($) {
    my ($path) = @_;
    my ($stat_output, $stat_error) = (undef) x 2;
    eval {
        local $SIG{ALRM} = sub {
            die "Alarm doing sysstat\n";
        };
        alarm(30);
        ($stat_output) = stat($path);
        $stat_error = $! if($!);
    };
    alarm(0);
    if($@ =~ /Alarm/) {
        die("Timed out trying to stat \"$path\" on host \"$hostname\"");
    }
    if(!defined($stat_output)) {
        die("Unable to stat \"$path\" on host \"$hostname\": $stat_error");
    }
    return $stat_output;
}

sub _do_stat ($) {
    my ($path) = @_;
    my $stat_output = undef;
    $stat_output = eval {_get_dev_stat($path)};
    if($@ =~ /Timed out/) {
        umount($path) or die("Error umounting \"$path\": $@");
        $stat_output = _get_dev_stat($path);
    }
    # We are ok if the mount point is not present.
    # We would create it in the next step.
    return $stat_output;
};

1;
