# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDENFSERRORS}) {
    my $newModule = $ENV{OVERRIDENFSERRORS};
    delete $ENV{OVERRIDENFSERRORS};
    print "The main nfsErrors module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

package nfsErrors;
use RandomData;
use Socket;
use nblade::TCP;
use FindBin;
use TracePkg;
use Params::Common;
use Params::NFS;
use Params::Volume;
use Options;
use Tharn;

my $Options;

$Options = new Options;
$Options->parameters(definitions=>
		     [
		      {
		          name=>"SRCIP",
			  description=>"the source IP address to use for the synthetic client",
		      },
		      {
			  name=>"TIMEOUT",
			  description=>"the default timeout for each NFS packet",
		      },
		      {
			  name=>"SOURCE",
			  default=>"/x/eng/localtest/noarch/agnostic/nblade",
		      },
		      {
			  name => "VERBOSITY",
			  description => "the verbiosity level for output",
			  default => 2,
		      },
		      {
			  name => "DEVICE",
			  description => "the default device to send packets out",
		      },
		      {
			  name => "ETHER",
			  description => "the specific source ethernet address to use",
		      },
		      {
			  name => "NAME",
			  description => "the default name of the user",
			  default => "nobody",
		      },
		      {
			  name => "GROUP",
			  description => "the default name of the group",
			  default => "nobody",
		      },
		      {
			  name => "CLONE_TEST",
			  description => "should the clone tests be run?",
		      },
		      {
			  name => "QUOTA_TEST",
			  description => "should the quota tests be run?",	
		      },
		      {
			  name => "SPACE_TEST",
			  description => "should the space tests be run?",
		      },
		      {
			  name => "TIME_DELTA",
			  description => "the maximum number of seconds that the time attributes can differ from the local clock",
			  default => 300,
		      },
		      {
			  name => "UDP_SPORT",
			  description => "the default source UDP port",
			  default => 300,
		      },
		      {
			  name => "FILENAME_LENGTH",
			  description => "the specific length of files to be created for readdir* calls",
		      },
		      {
			  name => "NUMBER_FILES",
			  description => "the specific number of files to be created for readdir* calls",
		      },
		      {
			  name => "FILE_SIZE",
			  description => "the specific size of the files to be created for readdir* calls",
		      },
		      {
			  name => "PLATFORM",
			  description => "the OS platform that we are testing",
			  default => "ontap",
		      },
		      {
			  name => "JUNCTION",
			  description => "the name of the junction to use on a volume",
		      },
		      {
			  name => "TEST",
			  spec => "s@",
			  description => "a specific list of test cases to execute",
		      },
		      {
			  name => "NOTEST",
			  spec => "s@",
			  description => "a specific list of test cases to NOT execute",
			  default => [28, 30, 69],
		      },
		      {
			  name => "XID",
			  description => "the specific XID to use on an RPC call",
		      },
		      {
			  name => "MOUNTFH",
			  description => "the specific mount filehandle to use. Default is to discover it.",
		      },
		      {
			  name => "NLM_PORT",
			  description => "the specific NLM port to use on the filer. Default is to discover it.",
		      },
		      {
			  name => "SM_PORT",
			  description => "the specific SM port to use on the filer. Default is to discover it.",
		      },
		      {
			  name => "NFS_PORT",
			  description => "the specific NFS port to use on the filer. Default is to discover it.",
		      },
		      {
			  name => "MOUNT_PORT",
			  description => "the specific MOUNT port to use on the filer. Default is to discover it.",
		      },
		      {
			  name => "PROC", 
			  description => "the specific RPC thpl to execute",
		      },
		      {
			  name => "RESULTS", 
                          spec => "boolean",
                          default => undef,
			  description => "should NATE results be used",
		      },
		      {
			  name => "ITERATE", 
			  description => "how many iterations should be run",
		      },
		      {
			  name => "IS_7MODE",
			  description => "If running BR (or later), is test being run against 7-mode volume? (yes or no)",
		      },
		      {
			  name => "TC_PREPEND",
			  description => "Prepend the script name (excluding .thpl) followed by a space to the test case name when it is registered",
			  spec => "boolean",
			  default => "false",
		      },
		      ]);
$Options->compatability(definitions=>
			{
			    verbosity => "VERBOSITY",
			    timedelta => "TIME_DELTA",
			    udpsport => "UDP_SPORT",
			    platform => "PLATFORM",
			    device => "DEVICE",
			    testResults => "RESULTS",
			    });



my $oldfh = select STDOUT; 
$| =1;
select $oldfh;

use Term::ANSIColor;
use Cwd 'realpath';
use Data::Dumper qw(Dumper);
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR HASHREF ARRAYREF UNDEF);
use Params::Common; # pick up FILER which forces Options to define default_data_interface

# set auto reset to true
$Term::ANSIColor::AUTORESET = 1;
my $Trace = new TracePkg;
$Trace->set_options(separateNewline=>1);
my $attrStatus;
my $filehandleStatus;
my @ExecutedTests;
my %colors = (
	      good => "green",
	      fail => "red",
	      bad => "yellow", 
	      normal => "clear",
	      info => "cyan",
   );

my %Procedures = (
		  nfs => {
		      2 => {
			  Null => 0,
			  Getattr => 1,
			  Setattr => 2,
			  Root => 3,
			  Lookup => 4,
			  Readlink => 5,
			  Read => 6,
			  Writecache => 7,
			  Write => 8,
			  Create => 9,
			  Remove => 10,
			  Rename => 11,
			  Link => 12,
			  Symlink => 13,
			  Mkdir => 14,
			  Rmdir => 15,
			  Readdir => 16,
			  Statfs => 17,
		      },
		      3 => {
			  Null => 0,
			  Getattr => 1,
			  Setattr => 2,
			  Lookup => 3,
			  Access => 4,
			  Readlink => 5,
			  Read => 6,
			  Write => 7,
			  Create => 8,
			  Mkdir => 9,
			  Symlink => 10,
			  Mknod => 11,
			  Remove => 12,
			  Rmdir => 13,
			  Rename => 14,
			  Link => 15,
			  Readdir => 16,
			  Readdirplus => 17,
			  Fsstat => 18,
			  Fsinfo => 19,
			  Pathconf => 20,
			  Commit => 21,
		      },
		  },
		  nlm => {
		      Null => 0,
		      Test => 1,
		      Lock => 2,
		      Cancel => 3,
		      Unlock => 4,
		      Granted => 5,
		  },
		  sm => {
		      Null => 0,
		      Stat => 1,
		      Mon => 2,
		      Unmon => 3,
		      UnmonAll => 4,
		      SimuCrash => 5,
		      Notify => 6,
		  },
		  mount => {
		      Null => 0,
		      Mnt => 1,
		      Dump => 2,
		      Umnt => 3,
		      Umntall => 4,
		      Export => 5,
		      Exportall => 6,
		      Pathconf => 7,
		  },
		  );

my %Masks = (
		  2 => {
		      Null => 0x0,
		      Getattr => 0x1,
		      Setattr => 0x2,
		      Root => 0x4,
		      Lookup => 0x10,
		      Readlink => 0x20,
		      Read => 0x40,
		      Writecache => 0x80,
		      Write => 0x100,
		      Create => 0x200,
		      Remove => 0x400,
		      Rename => 0x800,
		      Link => 0x1000,
		      Symlink => 0x2000,
		      Mkdir => 0x4000,
		      Rmdir => 0x8000,
		      Readdir => 0x10000,
		      Statfs => 0x20000,
		  },
		  3 => {
		      Null => 0x0,
		      Getattr => 0x1,
		      Setattr => 0x2,
		      Lookup => 0x4,
		      Access => 0x8,
		      Readlink => 0x10,
		      Read => 0x20,
		      Write => 0x40,
		      Create => 0x80,
		      Mkdir => 0x100,
		      Symlink => 0x200,
		      Mknod => 0x400,
		      Remove => 0x800,
		      Rmdir => 0x1000,
		      Rename => 0x2000,
		      Link => 0x4000,
		      Readdir => 0x8000,
		      Readdirplus => 0x1000,
		      Fsstat => 0x20000,
		      Fsinfo => 0x40000,
		      Pathconf => 0x80000,
		      Commit => 0x100000,
		  },
		  );

my %Statfs = (
	      solaris => {
			  transferSize => 8192,
			  blockSize => 1024,
			  },
	      ontap => {
			  transferSize => 8192,
			  blockSize => 4096,
			  },
	      spinfs => {
			  transferSize => 8192,
			  blockSize => 4096,
			  },
	      );

my %Fsstat = (
#	      availableSlots => 256,
#	      availableUserSlots => 256,
#	      invariantTime => 1,
	      );
my %Fsinfo = (
	      solaris => {
	      		  readMax => 32768,
	      		  readPreferred => 32768,
	      		  readMultiple => 512,
	      		  writeMax => 32768,
	      		  writePreferred => 32768,
	      		  writeMultiple => 512,
	      		  directoryPreferred => 8192,
	      		  maxFileSize => "1099511627775",
	      		  minTime => "",
	      		  hardLinks => "supported",
	      		  symbolicLinks => "supported",
	      		  homogeneous => true,
	      		  cansettime => true,
	      		  },
	      ontap => {
	      		  readMax => 32768,
	      		  readPreferred => 32768,
	      		  readMultiple => 512,
	      		  writeMax => 32768,
	      		  writePreferred => 32768,
	      		  writeMultiple => 512,
	      		  directoryPreferred => 32768,
	      		  maxFileSize => "17557623775232",
	      		  minTime => "",
	      		  hardLinks => "supported",
	      		  symbolicLinks => "supported",
	      		  homogeneous => true,
	      		  cansettime => true,
	      		  },
	      );
my %Pathconf = (
		solaris => {
			  linkMax => 32767,
			  nameMax => 255,
			  noTrunc => true,
			  chownRestricted => true,
			  caseInsensitive => false,
			  casePreserving => true,
			  },
		ontap => {
			  linkMax => 100000,
			  nameMax => 255,
			  noTrunc => true,
			  chownRestricted => true,
			  caseInsensitive => false,
			  casePreserving => true,
			  },
		);

my $xid;
my $release = "alpha";
my $arpCmd = "arp/linux/bin/arpValidation";
my $mountCmd = "Mnt";
my %fids;
my $Pass_Cases;
my $Fail_Cases;
my $Last_Test;
my $Errors;
my $Failed_command;
my $Failed_output;
my $Source;   # HACK - because this isn't completely an object

use vars qw(@Tests);

sub clone
{
    my $this = shift;
    my $class = ref($this) || $this;
    my $self = {$this};
    bless($self, $class);
    $Trace->entry();
    foreach my $key (keys %$this) {
	$self->{$key} = $this->{$key};
    }
    %$self = (%$self, @_);
    if ($self->{Program} eq "sm") {
	$self->{version} = 1;
	$self->{testdir} = $self->{TestDir}."/$self->{Program}/linux/bin";
    } else {
	$self->{testdir} = $self->{TestDir}."/$self->{Program}$self->{version}/linux/bin";
    }
    $self->{testcase} = sprintf("nblade_%s%d_%s",$self->{Program},$self->{version},$self->{program});
    $self->{dport} = $self->{"$self->{Program}Port"};
    delete $self->{object}; # don't save the object attributes hashes for nfsv2
    $self->_verify_ports();
    return $self;
}

sub DESTROY
{
}

sub new
{
    my $this = shift;
    my $class = ref($this) || $this;
    my $name = our $AUTOLOAD;
    my $self = {
	name => undef,
	verbosity => undef,
	Tests => [],
	share => "/",
	exitCode => 0,
	errorCode => 0,
	version => 2,
	Program => nfs,
	srcip => undef,
	saveAttributes => 0,
	objectName => undef,
	AdditionalOpts => [],
	MinimumRelease => 1.0,
	chmod => "0777",
	mount => 1,
	retries => 5,
	hard_mount => undef,
	timeo => 1,
	device => undef,
	server => undef,
	synthetic => undef,
	@_
    };
    bless($self, $class);
    $Trace->entry();
    my $os = `uname`;
    chomp $os;
    $Options->process(hostrec=>1);
    $options = $Options->get_hash();

    $self->{name} = $options->{NAME} || $self->{name};
    if (!$options->{LISTTESTCASES} && $os !~ /Linux/i) {
	$Trace->fatal(msg=>"$0 only runs under Linux, not $os\n");
    }
    $Fail_Cases = 0;
    $Pass_Cases = 0;
    $self->{volume} = $options->{volume}[0];
    $self->{source} = $options->{SOURCE} if ($options->{SOURCE});
    if (! -d $self->{source}) {
	$Trace->fatal(msg=>"testsdir $self->{source} does not exist!\n", result=>"CONFIG");
    }
    # no we are sure we have reasonable sources - change our LD_LIBRARY_PATH
    $ENV{LD_LIBRARY_PATH} = "$self->{source}/lib/$^O:/usr/software/lib";
    $Source = $self->{source}; # HACK to have a global contain the path to the source
    $self->{listcases} = $options->{LISTTESTCASES};
    $self->{delta} = $options->{TIME_DELTA};
    $self->{sport} = $options->{UDP_SPORT};
    $SIG{INT} = \&InterruptHandler; # override
    $self->{source} =~ s/nfs\d?\/?//;
    $self->{Options} = $options;
    $self->{junction} = $options->{JUNCTION};
    $self->{Release} = $options->{release};
    $self->{Platform} = lc $options->{PLATFORM};
    $self->{maxFileName} = ($self->{Platform} =~ /spinfs/i) ? 512 : ($self->{Platform} =~ /solaris/i) ? 256 : 255;
    $self->{blockSize} = ($self->{Platform} =~ /solaris/i) ? 512 : 4096;
    $self->{maxLinkLength} = 1024;
    if ($options->{IS_7MODE}) {
	if ($options->{IS_7MODE} eq "yes") {
	    $self->{is_7mode} = 1;
	} elsif ($options->{IS_7MODE} eq "no") {
	    $self->{is_7mode} = 0;
	} else {
	    $Trace->comment(msg=>"Invalid value for IS_7MODE; assuming 'no'\n");
	    $self->{is_7mode} = 0;
	}
    } else {
	$self->{is_7mode} = 0;
    }

    $self->{volume_style} = $options->{VOLUME_STYLE};
    $self->{stripe_format} = $options->{VOL_STRIPE_FORMAT};
    if ($self->{volume_style} =~ /striped/i) {
	if ($self->{stripe_format} =~ /full/i) {
	    $self->{striped_dirsize} = 12288;
	}
    }	
    if ($self->{volume_style} =~ /striped/i && $self->{stripe_format} =~ /full/i) {
	$Pathconf{ontap}{linkMax} = 4294967295;
    }

    $self->{Release} = "tricky" if (!$self->{Release});

    $self->{arpCmd} = "$self->{source}/$arpCmd";

    if (!defined($self->{srcip}) && defined($options->{SRCIP})) {
	$self->{srcip} = $options->{SRCIP};
    }
    $self->{srcIPcmd} = " --srcip $self->{srcip}" if (defined($self->{srcip}));

    $self->set_server($self->{server} || @{$options->{DATA_INTERFACES}}[0] || $options->{default_data_interface});

    $self->{loop} = $options->{ITERATE} || 1;
    if (defined($options->{XID})) {
	$xid = $options->{XID};
    } else {
	my $seed = (defined($options->{SEED})) ? $options->{SEED} : time() ^ ($$ + ($$ <<15));
	$Trace->comment(msg=>"SEED=$seed");
	$xid = int(rand 0xffffffff);
    }

    if (exists($options->{TEST})) {
	my $ref = $self->{Tests};
	my $element;
	foreach my $test (@{$options->{TEST}}) {
	    foreach $element (split(/,/, $test)) {
		if ($element =~ /^(\d+)$/) {
		    push(@{$self->{Tests}}, "^$1\\D?\$");
		    push(@{$self->{Checks}}, "^$1\\D+\$", "^$1\$");
		    next;
		}
		if ($element =~ /^(\D+)$/) {
		    push(@{$self->{Tests}}, "^\\d+$1\$");
		    push(@{$self->{Checks}}, "^\d+\$");
		    next;
		}
		if ($element =~ /^(\d+)\D+$/) {
		    push(@{$self->{Tests}}, "^$element\$");
		    push(@{$self->{Checks}}, "^$element?\$");
		    next;
		}
		$Trace->fatal(msg=>"$element is not supported as a test\n", result=>"CONFIG");
	    }
	}
    }
    if (exists($options->{NOTEST})) {
	my $element;
	foreach my $notest (@{$options->{NOTEST}}) {
	    foreach $element (split(/,/, $notest)) {
		if ($element =~ /^(\d+)$/) {
		    push(@{$self->{noTests}}, "^$1\\D?\$");
		    next;
		}
		if ($element =~ /^(\D+)$/) {
		    push(@{$self->{noTests}}, "^\\d+$1\$");
		    next;
		}
		if ($element =~ /^(\d+)\D+$/) {
		    push(@{$self->{noTests}}, "^$element\$");
		    next;
		}
		$Trace->fatal(msg=>"$element is not supported as a test\n", result=>"CONFIG");
	    }
	}
    }
    $self->{program} = $0;
    $self->{program} =~ s!.+/!!;
    $self->{program} =~ s/\.thpl\s*$//;
    $self->{testcase} = sprintf("nblade_%s%d_%s",$self->{Program},$self->{version},$self->{program});
    if ($self->{listcases}) {
	return $self;
    }
    
    $self->{share} = $options->{share} if ($options->{share});
    $self->{group} = $options->{GROUP};
    if (!defined($self->{uid}) && !defined($self->{uid} = scalar(getpwnam($self->{name})))) {
	$Trace->fatal(msg=>"Could not determine UID of name $self->{name}\n");
    }
    if (!defined($self->{gid}) && !defined($self->{gid} = scalar(getgrnam($self->{group})))) {
	$Trace->fatal(msg=>"Could not determine GID of group $self->{group}\n");
    }
    $Trace->trace(msg=>"uid=$self->{uid} for name '$self->{name}'\n");
    $Trace->trace(msg=>"gid=$self->{gid} for group '$self->{group}'\n");

    $self->{attributesExpected} = 1;
    $self->{TestDir} = $self->{source};
    $self->{testdir} = $self->{source};
    $self->{cmdOptions} = "--destip $self->{server} --device $self->{device}";
    $self->{verbosity} = $options->{VERBOSITY} if (!defined($self->{verbosity}));
    $self->{cmdOptions} .= " --verbosity 4" if ($self->{verbosity} > 3);
    $self->{cmdOptions} .= " --timeout $options->{TIMEOUT}" if (defined($options->{TIMEOUT}));
    $self->{cmdOptions} .= " $self->{ether}" if (defined($self->{ether}));
    $Trace->comment(msg=>"verbosity set to $self->{verbosity}");
    $self->{numFiles} = $options->{NUMBER_FILES};
    $self->{fileLength} = $options->{FILENAME_LENGTH};
    $self->{datasize} = $options->{FILE_SIZE};
    $self->{NoSpaceFH} = $options->{SPACE_TESTS};
    $self->{results} = $options->{RESULTS};
    $mountCmd = "$self->{source}/mount$self->{version}/linux/bin/$mountCmd --udpdport $self->{mountPort} self->{ether}";
    if ($self->{mount} && $self->{Program} eq "nfs") {
	# get the mount handle
	$self->{handle} = $options->{MOUNTFH} || $self->mount(share=>$self->{share});
    }
    if ($self->{Program} eq "sm") {
	$self->{version} = 1;
	$self->{testdir} .= "/$self->{Program}/linux/bin";
    } else {
	$self->{testdir} .= "/$self->{Program}$self->{version}/linux/bin";
    }
    if ($self->{results}) {
        use Burt qw(testcase_start);
	$Trace->fatal(msg=>$@) if ($@);
    }
    if (!$self->{listcases} && !$self->{Release} && $self->ConnectToServer()) {
	my $release;
	if (!($release = $self->{serverobject}->get_release())) {
	    $Trace->warn(msg=>"Encountered an error trying to determine the Spin Release number\n");
	} else {
	    $self->{Release} = $release;
	}
    }
    $self->{chmod} = undef if (!$self->{mount});
    if (defined($self->{chmod})) {
	$self->getattr(handle=>$self->{handle}, acceptable_codes=>[0]);
	my %attributes = $self->getAttributes();
	if ($attributes{mode} != 0777) {
	    $self->setattr(handle=>$self->{handle}, mode=>$self->{chmod}, authuid=>$attributes{uid}, acceptable_codes=>[0, 1]);
	}

    }
    if (wantarray) {
        $Trace->exit(msg=>"return (self, $self->{handle}, $self->{uid}, $self->{gid})");
        return ($self, $self->{handle}, $self->{uid}, $self->{gid});
    } else {
        $Trace->exit(msg=>"return self");
        return $self;
    }
}

sub set_server {
    my ($self, $server) = @_;
    $Trace->entry(msg=>"server=$server");
    return if ($self->{listcases});
    if (defined($server)) {
	$self->{server} = Socket::inet_ntoa(scalar(gethostbyname($server))); 
    } elsif (!$self->{listcases}) {
	$Trace->fatal(msg=>"Please specify a server\n", result=>"SCRIPT");
    }
    $self->{server} = $server;

    if (!defined($self->{device})) {
	if (defined($options->{DEVICE})) {
	    $self->{device} = $options->{DEVICE};
	} elsif (!$self->{listcases}) {
	    if ($self->{srcip}) {
		$Trace->warn(msg=>"You are using a srcip of $self->{srcip} and did not specify a DEVICE. I can only guess at the correct device and destaddress to use.");
	    }
	    my $ether;
	    ($self->{device}, $ether) = TCP::determineDevice($self->{server},$self->{device});
	    if (($ether ne "") && !defined($options->{ETHER})) {
		$Trace->debug(msg=>"determineDevice set ETHER to $ether");
		$options->{ETHER} = $ether;
	    }
	}
    }
    # get the destination ethernet address
    $self->{ether} = $options->{ETHER} || $self->arp();
    $self->{ether} = "--destaddress $self->{ether}";

    # send a gratuitous ARP so we don't have to answer any ARP requests later
    $self->gratuitous_arp();
    $self->_verify_ports();
    $Trace->trace(msg=>"server=$self->{server}");
    $Trace->exit();
}

sub set_srcip {
    my ($self, $srcip) = @_;
    $Trace->entry(msg=>"src=$srcip");
    
    $self->{srcip} = $srcip;
    $self->{srcIPcmd} = " --srcip $self->{srcip}";
    $self->gratuitous_arp();
    $Trace->exit();
}

sub _verify_ports {
    my $self = shift;

    return if ($self->{listcases});
    if ($self->{Program} eq "nfs" || $self->{Program} eq "mount") {
	# verify the mount port
	if ($options->{MOUNT_PORT}) {
	    $self->{mountPort} = $options->{MOUNT_PORT};
	} else {
	    $self->{mountPort} = $self->_get_port("mount", 100005);
	}
    }
    
    if ($self->{Program} eq "nfs") {
	# verify the NFS port
	if ($options->{NFS_PORT}) {
	    $self->{nfsPort} = $options->{NFS_PORT};
	} else {
	    $self->{nfsPort} = $self->_get_port("nfs", 100003);
	}
    }
    
    if ($self->{Program} eq "nlm") {
	# verify the NLM port
	if ($options->{NLM_PORT}) {
	    $self->{nlmPort} = $options->{NLM_PORT};
	} else {
	    $self->{nlmPort} = $self->_get_port("nlm", 100021);
	}
    }
    
    if ($self->{Program} eq "sm") {
	# verify the SM port
	if ($options->{SM_PORT}) {
	    $self->{smPort} = $options->{SM_PORT};
	} else {
	    $self->{smPort} = $self->_get_port("sm", 100024);
	}
    }
    $self->{dport} = $self->{"$self->{Program}Port"};
}

sub _get_port {
    my ($self, $prog, $prog_num) = @_;
    $Trace->entry();
    my $port = undef;
    my $GetPort = wrap_cmd("$self->{source}/portmap3/linux/bin/GetportValidation --destip $self->{server} --udpsport $self->{sport} --device $self->{device} --portmapprog $prog_num --novalidation --portmapvers $self->{version} $self->{srcIPcmd} $self->{ether}");
    $Trace->debug(msg=>"$GetPort\n");
    
    my $retries = $self->{retries};
  getPort: {
      open(CMD, "$GetPort 2>&1 |") || $Trace->fatal(msg=>"Cannot run $GetPort\n");
      my $GetPortOut = "";
      while (<CMD>) {
	  last if /Timeout/;
	  $GetPortOut .= $_;
	  $Trace->debug(msg=>$_);
	  if (/Port=(\d+)/) {
	      $port = $1;
	  }
      }
      close CMD;
      
      if (!$port) {
	  if ($self->{hard_mount} || ($retries-- > 0)) {
	      sleep($self->{timeo});
	      redo getPort;
	  }
	  $Trace->comment(msg=>"$GetPort\n");
	  $Trace->comment(msg=>$GetPortOut);
	  $Trace->fatal(msg=>"The $prog port came back as 0 from the RPC server. Aborting\n");
      }
  }
    return $port;
}

sub gratuitous_arp 
{
    my $self = shift;
    $Trace->entry();
    my $srcIP = $self->{srcip};
    if ($srcIP && $srcIP ne "") {
	my $cmd = wrap_cmd("$self->{arpCmd} --destip $srcIP --device $self->{device} --novalidation $self->{srcIPcmd} --count 3 --timeout 0");
	$Trace->trace(msg=>$cmd);
	`$cmd`;
    }
    $Trace->exit();
}

sub arp
{
    my $self = shift;
    my %params = (
		  ip => $self->{server},
		  src_ip => $self->{srcip},
		  timeout => undef,
		  retries => $self->{retries},
		  @_,
		  );
    $Trace->entry();
    my $ether = undef;
    my $attempts = 1;
    my $timeout = (defined($params{timeout})) ? "--timeout $params{timeout}" : "";

    if (-x $self->{arpCmd}) {
      arp: {
	  my $cmd = "$self->{arpCmd} --destip $params{ip} --device $self->{device} --novalidation $timeout";
	  $cmd .= " --srcip $params{src_ip}" if ($params{src_ip});
	  $Trace->debug(msg=>"$cmd\n");
	  $cmd = wrap_cmd($cmd);
	  open(CMD, "$cmd 2>&1 |");
	  while (<CMD>) {
	      $Trace->debug(msg=>$_);
	      if (/Address is (.+)/) {
		  $ether = $1;
	      }
	  }
	  close CMD;
	  if (!$ether || $ether eq "00:00:00:00:00:00") {
              # failed to get the MAC address using our arp utility, try arping
              my $arping = "/sbin/arping";
              if ( -e $arping ) {
                  $cmd = "$arping -f -c 5 $params{ip} -I $self->{device}";
                  $cmd .= " -s $params{src_ip} " if ( $params{src_ip} );
                  $cmd .= " -w $params{timeout} " if ( $params{timeout} );
                  $cmd = wrap_cmd($cmd); 
                  $Trace->debug(msg => "$cmd");
                  open(CMD, "$cmd 2>%1 |");
                  while ( <CMD> ) {
                      $Trace->debug(msg => $_);
                      if ( /reply from .+ \[(.+)\]/ ) {
                          $ether = $1;
                      }
                  }
                  close CMD;
                  # break out if found an ether address, otherwise us let us go through the retry loop
                  if ( $ether ) {
                      last arp;
                  }
              } else {
                  $Trace->warn(msg => "Needed to fall back to '$arping' to find the MAC address for the server, and that command doesn't seem to exist.");
              }
	      $Trace->debug(msg=>"Could not get MAC address for $self->{server} on attempt # $attempts\n");
	      if (!$self->{hard_mount} && ($attempts >= $params{retries})) {
		  $Trace->fatal(msg=>"Could not get MAC address for $self->{server} after $attempts attempts\n");
	      }
	      $attempts++;
	      sleep($self->{timeo});
	      redo arp;
	  }
      }
    } else {
	$Trace->fatal(msg=>"Could not find command for arp: $self->{arpCmd}\n");
    }
    return $ether;
}

sub getMountFH
{
    my ($self, $share) = @_;
    $Trace->entry();
    my $handle = "";
    my $attempts = 1;
  getMount: {
      my $cmd = wrap_cmd("$mountCmd $self->{cmdOptions} --novalidation --mountpoint $share");
      $Trace->debug(msg=>"$cmd\n");
      open(CMD, "$cmd 2>&1 |");
      my $output;
      while (<CMD>) {
	  $output .= $_;
	  $Trace->debug(msg=>$_);
	  if (/Filehandle\s*=\s*(.+)/) {
	      $handle = $1;
	  }
      }
      close CMD;
      if ($handle eq "") {
	  $Trace->warn(msg=>"Could not get mountpoint's filehandle for $share on attempt # $attempts\n");
	  if (!$self->{hard_mount} && ($attempts >= $self->{retries})) {
	      $Trace->fatal(msg=>"Could not get mountpoint's filehandle for $share after $attempts attempts\n");
	  }
	  $attempts++;
	  sleep($self->{timeo});
	  redo getMount;
      }
  }
    return $handle;
}

sub createClone()
{
    my $self = shift;
    my $clone = undef;
    $Trace->entry();
    return 1 if ($self->{listcases});
    return $self->{CloneFH} if ($self->{CloneFH});
    if ($self->{Options}{CLONE_TESTS}) {
	$self->{CloneFH} = $self->getMountFH($self->{Options}{CLONE_TESTS});
	return $self->{CloneFH};
    }
    my $cloneShare = $self->{share}."clone".int(rand 32768);;
    return undef if (!$self->ConnectToServer());
    if ($self->{Clone} = $self->{serverobject}->CloneVFS(VFS=>$self->{volume}, Mount => $cloneShare)) {
	return $self->getMountFH($cloneShare);
    }
    return undef;
}

sub deleteClone
{
    my $self = shift;
    $Trace->entry();
    return 1 if ($self->{listcases});
    return 1 if ($self->{CloneFH});
    return undef if ($self->{Clone} && !$self->ConnectToServer());
    my $output;
    my $clone = $self->{Clone};
    return $self->{serverobject}->DeleteVFS(Name=>$self->{Clone});
}    

sub ConnectToServer {
    my $self = shift;
    $Trace->entry();
    return undef if ($self->{listcases});
    return $self->{serverobject} if ($self->{serverobject});
    eval "use Server";
    if ($@){
	$Trace->fatal(msg=>"An error occurred while loading Server: $@\n");
    }
    $self->{serverobject} = main::Server->new();
    # commented out because we are no longer manipulating volumes
    #if (!$self->{volume}) {
#	my $output = $self->{serverobject}->volume_show("junction-path"=>$self->{share});
#	$self->{volume} = $$output[0]{volume};
    #}
    #$Trace->fatal(msg=>"Count not determine volume for $self->{share}\n") if (!$self->{volume});
    $Trace->trace(msg=>"volume set to $self->{volume}\n");
    return $self->{serverobject};
}

sub saveName 
{
    my $self = shift;
    $Trace->entry();
    return if (!$self->{saveAttributes});
    $self->{objectName} = shift;
    $Trace->debug(msg=>"saveName $self->{objectName}\n");
}

sub clearNames
{
    my $self = shift;
    $Trace->entry();
    $self->{object}{saveAttributes} = undef;
    $self->{object}{readdir} = undef;
}

sub MinimumRelease
{
    my ($self, $release) = @_;
    $Trace->entry();
    $self->{MinimumRelease} = $release if ($self->{Platform} =~ /spinfs/i);
}

sub checkRelease
{
    my ($self, $minRelease) = @_;
    $Trace->entry();
    return 1;
    my @serverRelease = split(/\./, $self->{Release});
    $minRelease = $self->{MinimumRelease} if (!$minRelease);
    my @minRelease = split(/\./, $minRelease);
    while ($#serverRelease > $#minRelease) {
	push(@minRelease, 0);
    }
    while ($#minRelease > $#serverRelease) {
	push(@serverRelease, 0);
    }

    for (my $index=0; $index <= $#serverRelease; $index++) {
	if ($serverRelease[$index] > $minRelease[$index]) {
	    return 1;
	} elsif ($serverRelease[$index] < $minRelease[$index]) {
	    return undef;
	}
    }
    return 1;
}

sub AddTestcase {
    my ($self, $test, $testcase) = @_;
    $Trace->entry();
    $self->{TestCases}{$test} = $testcase;
    my $ref = $self->{Tests};
    my $noRef = $self->{noTests};
    return "" if ($#$ref != -1 && !grep($test =~ /$_/, @$ref));
    return "" if (grep($test =~ /$_/, @$noRef));
    my $description = $self->{TestCases}{$test};
    if ($self->{Options}{TC_PREPEND}) {
        #-----------------------------------------------------------------
        # Prepend the name of the script (excluding the extension) to the
        # test case name
        #-----------------------------------------------------------------
        my $script_name = $Tharn::Scriptspec;
        $script_name    =~ s/.+\///;     # Just get the name of the test
        $script_name    =~ s/\.thpl/ /;  # Strip off .thpl and add a space
        $test           = $script_name . $test;
        $Trace->comment(msg=>"Test case name: $test");
    }
    if ($self->{listcases} =~ /full/i) {
	print "testcase : $self->{testcase}_$test : $description\n";
    } elsif ($self->{listcases}) {
	print "testcase : $test : $description\n";
    } elsif ($self->{results}) {
	Burt::testcase_start($test, $testcase);
    }
    $self->{returnStatus} = undef;
}

sub TestAsync {
    my ($self, $number, $cmd, $options) = @_;

    my $ref = $self->{Tests};
    my $noRef = $self->{noTests};

    if ($number =~ /^INTERNAL$/i) {
    } elsif ($number =~ /^ALWAYS$/i) {
    } elsif ($number =~ /^QUIET$/i) {
    } else {
	if ($number =~ /ALWAYS|QUIET/i) {
	    $number =~ s/QUIET//i;
	}
	return "" if ($#$ref != -1 && !grep($number =~ /$_/, @$ref));
	return "" if (grep($number =~ /$_/, @$noRef));
    }
    if ($self->{listcases}) {
	return "fakefilehandle"; # make sure to fake out the caller since it might be looking for a filehandle
    }

    $cmd = $self->{testdir}."/$cmd ".$self->{cmdOptions}." $options";
    $cmd .= " --udpdport $self->{dport}";
    $cmd .= " --udpsport $self->{sport}" if ($self->{sport});
    $cmd .= " --rpcxid ".$xid++ if ($cmd !~ /--rpcxid/);
    $cmd = wrap_cmd($cmd);

    $Last_Test = $number;

    local *CMD;
    $self->{CMD} = *CMD;

    open($self->{CMD}, "$cmd 2>&1 |") || $Trace->fatal(msg=>"Cannot open $cmd: $!\n");

    $self->{cmd} = $cmd;
}

sub TestJoin {
    my $self = shift;
    my $number = shift;

    my $ref = $self->{Tests};
    my $noRef = $self->{noTests};

    if ($number =~ /^INTERNAL$/i) {
    } elsif ($number =~ /^ALWAYS$/i) {
    } elsif ($number =~ /^QUIET$/i) {
    } else {
	if ($number =~ /ALWAYS|QUIET/i) {
	    $number =~ s/QUIET//i;
	}
	return "" if ($#$ref != -1 && !grep($number =~ /$_/, @$ref));
	return "" if (grep($number =~ /$_/, @$noRef));
    }

    my (@outputFromCmd) = ();
    my $CMD = $self->{CMD};

    while (<$CMD>) {
	  $Trace->trace(msg=> $_);
	  push(@outputFromCmd, $_);
    }
    $self->{outputFromCmd} = join '', @outputFromCmd;

    $Trace->trace(msg=>"output from '$self->{cmd}' was \n@outputFromCmd\n");

    close $self->{CMD};
    $self->{CMD} = undef;
}

sub Test 
{
    my ($self, $number, $command, $options, @statusExpected) = @_;
    $Trace->entry();
    my $status = "";
    my $attempts = 1;
    my $finaltest = 1;
    my $internal = 0;
    my $ref = $self->{Tests};
    my $noRef = $self->{noTests};
    my $filehandle = "";
    my @outputFromCmd;
    my $error;
#    return if (!$self->checkRelease());
    if ($number =~ /^INTERNAL$/i) {
	$internal = 1;
	$finaltest = 0;
    } elsif ($number =~ /^ALWAYS$/i) {
	$finaltest = 0;
    } elsif ($number =~ /^QUIET$/i) {
	$finaltest = 0;
    } else {
	if ($number =~ /ALWAYS|QUIET/i) {
	    $number =~ s/QUIET//i;
	    $finaltest = 0;
	}
	return "" if ($#$ref != -1 && !grep($number =~ /$_/, @$ref));
	return "" if (grep($number =~ /$_/, @$noRef));
    }
    if ($self->{listcases}) {
	return "fakefilehandle"; # make sure to fake out the caller since it might be looking for a filehandle
    }
    $self->{errorCode} = 0;
    push(@ExecutedTests, $number) if ($finaltest);
    my $proc = $command;
    $command = $self->{testdir}."/$command ".$self->{cmdOptions}." $options";
    $command .= " --authuid $self->{uid}" if ($command !~ /--authuid/);
    $command .= " --authgid $self->{gid}" if ($command !~ /--authgid/);
    $command .= " --udpdport $self->{dport}";
    $command .= " --udpsport $self->{sport}" if ($self->{sport});
    $command .= $self->{srcIPcmd} if ($self->{srcIPcmd});

    $Last_Test = $number;
    $self->{NonIdempotentHack} = undef if (!$internal);
    if ($command =~ /--timeout (\d+)/) {
	$timeout = $1;
    }
  sendAndReceivePacket: {
      my $attrType = "attributes";
      my $thingie = "object";
      my $WCCData = 0;
      my $postop = 0;
      my $result = 0;
      my $invalid = undef;
      my $fsinfoType = "";
      my $saveAttributes = 0;
      my $objectName = $self->{objectName};
      my $readdirplusname = "";
      my $readdirplusentry = 0;
      $error = undef;
      @outputFromCmd = ();
      delete $self->{EOF};
      delete $self->{Proc};
      delete $self->{object}{attributes}{Path};
      delete $self->{object}{postop}{Path};
      $self->{proc_failure} = 0;

      my $timeout = undef;
      
      # Make a local copy of the command with an updated xid since we may be in a retry and need
      #  a new xid
      my $cmd = $command;
      $cmd .= " --rpcxid ".$xid++ if ($cmd !~ /--rpcxid/);
      $cmd = wrap_cmd($cmd);
      $Trace->trace(msg=>"$cmd\n");
      $self->{last_cmd} = $cmd;
      open(CMD, "$cmd 2>&1 |") || $Trace->fatal(msg=>"Cannot open $cmd: $!\n");
      my $retval = $?;
      if ($retval) {
	  if (($retval/256) == 139) {
	      $Trace->comment(msg=>"$cmd dumped core\n");
	      sleep($self->{timeo});
	      $Trace->debug(msg=>"resend packet\n");
	      redo sendAndReceivePacket;
	  }
	  if ((($retval & 0x7f) == 11) && ($retval & 0x80)) {
	      $Trace->comment(msg=>"$cmd dumped core*\n");
	      sleep($self->{timeo});
	      $Trace->debug(msg=>"resend packet\n");
	      redo sendAndReceivePacket;
	  }
	  if ((($retval & 0x7f) == 11) && (!($retval & 0x80))) {
	      $Trace->comment(msg=>"$cmd had a seg fault\n");
	      sleep($self->{timeo});
	      $Trace->debug(msg=>"resend packet\n");
	      redo sendAndReceivePacket;
	  }
      }
      $self->{last_cmd} = $cmd;
      @outputFromCmd = ();
      while (<CMD>) {
	  $Trace->trace(msg=> $_);
	  push(@outputFromCmd, $_);
	  next if (/^\s*$/);
	  if ($self->{retries} && ((!defined($timeout) || $timeout) && /bind call failed/ ||
				   /something isn.t right in/i || /mount failed/i || /Alarm clock/i)) {
	      if ($self->{verbosity} >= 2) {
		  $Trace->comment(msg=>"No reply was received from the command on attempt # $attempts: $_");
	      }
	      if (!$self->{hard_mount} && $attempts >= $self->{retries}) {
                  $Failed_command = $self->{last_cmd};
                  $Failed_output = join '', @outputFromCmd;
		  last sendAndReceivePacket;
	      }
	      $attempts++;
	      close CMD;
	      sleep($self->{timeo});
	      $Trace->debug(msg=>"resend packet\n");
	      redo sendAndReceivePacket;
	  }
	  if (/Status\s*=\s*(\d+)/) {
	      $status = $1;
	      $self->{attributesExpected} = ($_ =~ /STALE|BADHANDLE|NOT_SYNC|Garbage/) ? 0 : 1;
	      next;
	  }
	  if (/Successful/) {
	      $status = 0;
	      next;
	  }
	  if (/Filehandle\s*=\s*(.+)/) {
	      $filehandle = $1;
	      $self->{checkAttributesObjectHandle} = $filehandle;
	      $self->{object}{readdir}{$readdirplusname}{filehandle} = $1 if ($self->{Proc} eq "READDIRPLUS");
	      if ($self->{saveAttributes} && $objectName ne "") {
		  if (exists($self->{object}{saveAttributes}{$objectName})) {
		      $Trace->comment(msg=>"Error: already have an entry for $objectName\n");
		  }
		  $saveAttributes = 1;
		  $self->{object}{saveAttributes}{$objectName}{attributes} = "";
		  $self->{object}{saveAttributes}{$objectName}{filehandle} = $1;
	      }
	      if ($self->{checkHandle}) {
		  if ($filehandle ne $self->{checkHandle}) {
                      my $msg = "Error: filehandle mismatch: $filehandle expected to be $self->{checkHandle}\n";
                      $Errors .= $msg;
		      $Trace->comment(msg=>$msg);
		      $error = 1;
		  }
		  $self->{checkHandle} = undef;
	      }
	      next;
	  }
	  if (/Result\s*=\s*(.+)/) {
	      if ($1 eq "Failure") {
		  $status = 1;
	      } elsif ($1 eq "Success") {
		  $status = 0;
	      } else {
		  $Trace->warn(msg=>"Unknown status: $1\n");
	      }
	      next;
	  } elsif (/Successful/i) {
	      $status = 0;
	      next;
	  }
	  if (/DEBUG MESSAGE: (.+)/i) {
	      $Trace->debug(msg=>"DEBUG: $1\n");
	      next;
	  }	    
	  if (/WCC_Data\s*(.*)/) {
	      $WCCData = 1;
	      $thingie = "dir" if (grep(/$self->{Proc}/, (CREATE, MKDIR, RMDIR, SYMLINK, MKNOD, REMOVE, RMDIR, LINK)));
	      next;
	  }
	  if (/Pre-operation attributes/) {
	      $attrType = "preop";
	      if (/\((.+)\)/) {
		  $thingie = $1;
	      }
	      $self->{$thingie}{$attrType}{NONE} = 0;
	      next;
	  }
	  if (!$readdirplusentry && /Post-operation attributes/) {
	      $postop = 1;
	      $attrType = "postop";
	      if (/\((.+)\)/) {
		  $thingie = $1;
	      }
	      $thingie = "dir" if ($self->{Proc} eq "LOOKUP" && /directory/);
	      $thingie = "linkdir" if ($self->{Proc} eq "LINK" && /linkdir/i);
	      $thingie = "fromdir" if ($self->{Proc} eq "LINK" && /fromdir/i);
	      $self->{$thingie}{$attrType}{NONE} = 0 if (!$self->{noAttrs});
	      next;
	  }
	  if (/\s+None/ && $postop && !$readdirplusentry) {
	      next if !$self->{attributesExpected};
	      $self->{$thingie}{$attrType}{NONE} = 1;
	      next;
	  }
	  if (/^Proc = \d+ \(NFSPROC\d*_(.+)\)/) {
	      my $ref = ($self->{version} == 2) ? $self->{object}{attributes}{Path} : $self->{object}{postop}{Path};
	      $self->{Proc} = $1;
	      $$ref = "" if ($1 eq "READLINK");
	      $self->{object}{postop}{Access} = "" if ($1 eq "ACCESS");
	      if ($self->{saveAttributes} && $self->{Proc} eq "GETATTR") {
		  $saveAttributes = 1;
		  $self->{object}{saveAttributes}{$objectName}{attributes} = "";
	      }
	      next;
	  }
	  if (/^Proc = \d+ \((\w+)\)/) {
	      $self->{Proc} = $1;
	      next;
	  }
	  if (/File Type = .+/) {
	      if (/Should not appear/i) {
#		  $self->{exitCode} |= 1;
#		  $Trace->warn(msg=>"Unknown file type\n");
		  
	      }
	      if (!$readdirplusentry && /File Type = (.+) \(/) {
		  $self->{$thingie}{$attrType}{type} = $1;
		  next if ($self->{Proc} =~ /READDIR/);
	      }
	  }
	  next if ($self->{noAttrs});
	  if ($saveAttributes && $objectName ne "" && !$WCCData) {
	      $self->{object}{saveAttributes}{$objectName}{attributes} .= $_;
	  }
	  if (/RPC Message:\s*(.+)/) {
	      $status = $1;
	      $self->{Proc} = "RPC";
	      last;
	  }
	  if ($self->{Proc} eq "READLINK" && /Path = (.+)\s+$/) {
	      my $ref = ($self->{version} == 2) ? \$self->{object}{attributes}{Path} : \$self->{object}{postop}{Path};
	      $$ref = $1;
	      next;
	  }
	  if (/(\d+) bytes read/) {
	      my $ref = ($self->{version} == 2) ? \$self->{object}{attributes}{bytesRead} : \$self->{object}{postop}{bytesRead};
	      my $dataRef = ($self->{version} == 2) ? \$self->{object}{attributes}{readData} : \$self->{object}{postop}{readData};
	      $$ref = $1;
	      $$dataRef = "";
	      next;
	  }
	  if (/Data = (.+)$/) {
	      my $ref = ($self->{version} == 2) ? \$self->{object}{attributes}{readData} : \$self->{object}{postop}{readData};
	      $$ref .= $1;
	  }	    
	  if (/Verifier = (.+)\s+$/ && grep/$self->{Proc}/, ("COMMIT", "WRITE")) {
	      if (defined($self->{internal}{Verifier})) {
		  $self->{internal}{prevVerifier} = $self->{internal}{Verifier};
		  if ($self->{internal}{Verifier} ne $1) {
		      $Trace->warn(msg=>"Verifier in $self->{Proc} did not match previous value: $1");
		  }
	      }
	      $self->{internal}{Verifier} = $1;
	      next;
	  }
	  if ($self->{Proc} eq "ACCESS" && /Access = (.+)$/) {
	      $self->{object}{postop}{Access} = $1;
	      $self->{object}{postop}{Access} =~ s/\s+$//;
	      next;
	  }
	  if (/Cookie Verifier = ([\d|a-hA-H]+)/) {
	      $Trace->trace(msg => "Found Cookie Verifier  $1");
	      $self->{cookieVerifier} = $1;
	      next;
	  }
	  if (/EOF = (\d+)/) {
	      $self->{EOF} = $1;
	      next;
	  }
	  if ($self->{Proc} eq "READDIRPLUS") {
	      next if (/\d+ entries/);
	      next if (/Post-operation/);
	      next if (/^$/);
	      $readdirplusentry = 1, next if (/------------------/);
	      if ($readdirplusentry) {
		  if (/Name = (.+)/) {
		      $readdirplusname = $1;
		      $self->{object}{readdir}{$readdirplusname}{attributes} = "";
		      next;
		  }
		  if (/File ID = 0x([a-fA-F0-9]+)/) {
		      $self->{object}{readdir}{$readdirplusname}{fileid} = $1;
		      next;
		  }
		  if (/Cookie = 0x([a-fA-F0-9]+)/) {
		      $self->{object}{readdir}{$readdirplusname}{cookie} = $1;
		      $self->{cookie} = $1;
		      next;
		  }
		  $self->{object}{readdir}{$readdirplusname}{attributes} .= $_;
		  next;
	      }
	  }
	  if (/Mode = (.+)/) {
	      $self->{$thingie}{$attrType}{mode} = oct $1;
	      next;
	  }
	  if (/Link count = (\d+), UID = (\d+), GID = (\d+)/) {
	      $self->{$thingie}{$attrType}{count} = $1;
	      $self->{$thingie}{$attrType}{uid} = $2;
	      $self->{$thingie}{$attrType}{gid} = $3;
	      next;
	  }
	  if (/File size = (.+), Block size = (.+), No. of blocks = (.+)/) {
	      $self->{$thingie}{$attrType}{size} = $1;
#	      $self->{$thingie}{$attrType}{blocksize} = $2;
#	      $self->{$thingie}{$attrType}{blocks} = $3;
	      next;
	  }
	  if (/File size = (\d+), Used = (\d+)/) {
	      $self->{$thingie}{$attrType}{size} = $1;
	      $self->{$thingie}{$attrType}{used} = $2;
	      next;
	  }
	  if (/Special: Major = (\d+), Minor = (\d+)/) {
	      $self->{$thingie}{$attrType}{major} = $1;
	      $self->{$thingie}{$attrType}{minor} = $2;
	      next;
	  }
	  if (/Device number = (\d+)/) {
	      $self->{$thingie}{$attrType}{device} = $1;
	      next;
	  }
	  if (/File system id = 0x([a-fA-F0-9]+), File id = 0x([a-fA-F0-9]+)/) {
	      $self->{$thingie}{$attrType}{fsid} = $1;
	      $self->{$thingie}{$attrType}{fid} = $2;
	      next if (!$self->{saveAttributes} || $objectName eq "");
	      if ($thingie eq "object" && $self->{Proc} ne "READDIRPLUS" &&
		  $self->{Proc} ne "READDIR") {
		  $self->{object}{saveAttributes}{$objectName}{fileid} = $2;
	      }
	      next;
	  }	    
	  if (/\s*(.+) time\s+=\s+\((.+)\)/) {
	      my $time = $2;
	      my $type = ($1 =~ /Inode|Attribute/) ? "ctime" : lc(substr($1, 0, 1))."time";
	      my $prev = "prev$type";
	      if (defined($self->{$thingie}{$attrType}{$type})) {
		  $self->{$thingie}{$attrType}{$prev} = $self->{$thingie}{$attrType}{$type};
		  $Trace->debug(msg=>"set $self $thingie $attrType $prev to $self->{$thingie}{$attrType}{$type}\n");
	      }
	      $self->{$thingie}{$attrType}{$type} = $time;
	      $Trace->debug(msg=>"set $self $thingie $attrType $type to $time\n");
	      next;
	  }
	  if ($self->{Proc} eq "STATFS") {
	      if (/Transfer size = (\d+)/) {
		  $self->{$thingie}{$attrType}{transferSize} = $1;
		  next;
	      }
	      if (/Block size = (\d+)/) {
		  $self->{$thingie}{$attrType}{blockSize} = $1;
		  next;
	      }
	  }
	  if ($self->{Proc} eq "FSSTAT") {
#	    if (/Total space = (\d+)/) {
#		$self->{$thingie}{$attrType}{totalSpace} = $1;
#		next;
#	    }	    
#	    if (/Available space = (\d+)/) {
#		$self->{$thingie}{$attrType}{availableSpace} = $1;
#		next;
#	    }	    
#	    if (/Available space - this user = (\d+)/) {
#		$self->{$thingie}{$attrType}{availableUserSpace} = $1;
#		next;
#	    }	    
	      if (/Available file slots = (\d+)/) {
		  $self->{$thingie}{$attrType}{availableSlots} = $1;
		  next;
	      }	    
	      if (/Available file slots - this user = (\d+)/) {
		  $self->{$thingie}{$attrType}{availableUserSlots} = $1;
		  next;
	      }	    
	      if (/Invariant time = (\d+)/) {
		  $self->{$thingie}{$attrType}{invariantTime} = $1;
		  next;
	      }
	  }
	  if ($self->{Proc} eq "FSINFO") {
	      $fsinfoType = "read",next if (/Read transfer sizes/);
	      $fsinfoType = "write",next if (/Write transfer sizes/);
	      $fsinfoType = "directory",next if (/Directory read size/);
	      if (/Maximum = (\d+) bytes/) {
		  $self->{$thingie}{$attrType}{$fsinfoType."Max"} = $1;
		  next;
	      }
	      if (/Preferred = (\d+) bytes/) {
		  $self->{$thingie}{$attrType}{$fsinfoType."Preferred"} = $1;
		  next;
	      }
	      if (/Suggested multiple = (\d+) bytes/) {
		  $self->{$thingie}{$attrType}{$fsinfoType."Multiple"} = $1;
		  next;
	      }
	      if (/Max file size = (.+) bytes/) {
		  $self->{$thingie}{$attrType}{maxFileSize} = $1;
		  next;
	      }
	      if (/Server minimum time discrimination = (.+) sec/) {
		  $self->{$thingie}{$attrType}{minTime} = $1;
		  next;
	      }
	      if (/Hard links supported/) {
		  $self->{$thingie}{$attrType}{hardLinks} = "supported";
		  next;
	      }
	      if (/Symbolic links supported/) {
		  $self->{$thingie}{$attrType}{symbolicLinks} = "supported";
		  next;
	      }
	      if (/Pathconf cannot vary per file/) {
		  $self->{$thingie}{$attrType}{homogeneous} = "true";
		  next;
	      }
	      if (/Server can always set file times/) {
		  $self->{$thingie}{$attrType}{cansettime} = "true";
		  next;
	      }
	  }
	  if ($self->{Proc} eq "PATHCONF") {
	      if (/Link max = (\d+)/) {
		  $self->{$thingie}{$attrType}{linkMax} = $1;
		  next;
	      }
	      if (/Name max = (\d+)/) {
		  $self->{$thingie}{$attrType}{nameMax} = $1;
		  next;
	      }
	      if (/No trunc\s+= (\w+)/) {
		  $self->{$thingie}{$attrType}{noTrunc} = $1;
		  next;
	      }
	      if (/Chown restricted\s+= (\w+)/) {
		  $self->{$thingie}{$attrType}{chownRestricted} = $1;
		  next;
	      }
	      if (/Case insensitive\s+= (\w+)/) {
		  $self->{$thingie}{$attrType}{caseInsensitive} = $1;
		  next;
	      }
	      if (/Case preserving\s+= (\w+)/) {
		  $self->{$thingie}{$attrType}{casePreserving} = $1;
		  next;
	      }
	  }
	  if ($self->{Proc} eq "READDIR") {
	      next if (/File id/);
	      next if (/\d+ entries/);
	      if (/([a-fA-F0-9]+)\s+([a-fA-F0-9]+) (.+)/) {
		  $self->{object}{readdir}{$3}{fileid} = $1;
		  $self->{object}{readdir}{$3}{cookie} = $2;
		  $self->{cookie} = $2;
		  next;
	      }
	      $Trace->warn(msg=>"Should not reach this point in a Readdir: $_");
	  }
	  if (($self->{Proc} eq "READDIR" || $self->{Proc} eq "READDIRPLUS") && 
	      !exists($self->{EOF}) && $status == 0) {
	      if ($self->{verbosity} > 0 && $finaltest) {
		  $self->{errorCode} = 1; 
		  $self->{exitCode} |= 1;
		  $Trace->comment(msg=>"Readddir did not return EOF\n");
	      }
	  }
	  $invalid = 1 if (/Invalid/);
      }
      close CMD;
      $self->{outputFromCmd} = join '', @outputFromCmd;
      if ($error){
          $Failed_command = $self->{last_cmd};
          $Failed_output = $self->{outputFromCmd};
      }
      $self->{Proc} = $proc if (!$self->{Proc});
      # this is a hack to handle the case where we ran a nonidempotent request and the original reply was lost.
      # If the server overruns its replay cache before we retransmit, then we are toast
      
      if ((!defined($timeout) || !$timeout) && $status && ($attempts > 1) && grep($_ eq "0", @statusExpected)) {
	  # different approach. Delete the entry and retry the op
	  if (grep(/$self->{Proc}/i, qw(SYMLINK MKNOD RENAME LINK MKDIR CREATE))) {
	      push(@statusExpected, 17);
	      # normally the call would have returned the filehandle and attributes but they won't in this case 
	      #  so we need to perform a lookup to get them
	      my $objectname;
	      my $dirfh;
	      my $auth = "";
	      if ($options =~ /--authuid (\S+)/) {
		  $auth = "--authuid $1";
	      }
	      if ($options =~ /--authgid (\S+)/) {
		  $auth .= " --authgid $1";
	      }
	      print "####  Remove and try again!  ############\n";
	      my $tmp = $self->clone();
	      if ($self->{Proc} eq "MKDIR") {
		  my $dirfh = $1 if ($options =~ /--mkdirFH (\S+)/);
		  my $objectname = $1 if ($options =~ /--dirname (\S+)/);
		  $filehandle = $tmp->Test("ALWAYS", "Rmdir", "--rmdirFH $dirfh --rmdirfilename $objectname $auth", 0, 2);
	      } elsif ($self->{Proc} eq "RENAME") {
		  my $fromdir = $1 if ($options =~ /--fromdirhandle (\S+)/);
		  my $todir = $1 if ($options =~ /--todirhandle (\S+)/);
		  my $fromfile = $1 if ($options =~ /--fromfilename (\S+)/);
		  my $tofile = $1 if ($options =~ /--tofilename (\S+)/);
		  $filehandle = $tmp->Test("ALWAYS", "Rename", "--fromdirhandle $todir --todirhandle $fromdir --fromfilename $tofile --tofilename $fromfile $auth", 0, 2);
	      } else {
		  if ($options =~ /--(fromdirFH||symdir||mknodFH||todirhandle||linkfromdirhandle||createFH) (\S+)/) {
		      $dirfh = $2;
		  }
		  if ($options =~ /--(fromname||symname||mknodname||tofilename||linkfromfilename||createfilename) (\S+)/) {
		      $objectname = $2;
		  }
		  $filehandle = $tmp->Test("ALWAYS", "Remove", "--removeFH $dirfh --removefilename $objectname $auth", 0, 2);
	      }
	      if ($self->{verbosity} >= 2) {
		  $Trace->comment(msg=>"### Retry $self->{Proc} $options\n");
	      }
	      if (!$self->{hard_mount} && $attempts++ >= $self->{retries}) {
                  $Failed_command = $self->{last_cmd};
                  $Failed_output = join '', @outputFromCmd;
		  last sendAndReceivePacket;
	      }
	      sleep($self->{timeo});
	      $Trace->debug(msg=>"resend packet\n");
	      redo sendAndReceivePacket;
	  }
	  if (grep(/$self->{Proc}/i, qw(RMDIR REMOVE RENAME))) {
	      push(@statusExpected, 2);
	      $self->{NonIdempotentHack} = 1;
	  }
      }
  }
    $Trace->comment(msg=>"Invalid packet length for the $self->{Proc} reply!!\n") if ($invalid);
#    return $filehandle if ($internal);

    $self->{returnStatus} |= $status;
    $self->{printStatus} = $status;
    $self->{attempts} = $attempts;
    # successful if (no errors, status is undefined, and no timeout specified) or
    #  the status is in the expected list. The first case is true when a timeout of 0
    #  is specified during a shoot-and-forget test
    if ((!$error && !defined($status) && (!defined($timeout) || !$timeout)) || 
	grep($status eq $_, @statusExpected)) {
	my $result = 1;
	my $status = $self->{printStatus};
	my $proc = $self->{Proc};
	$self->printResult(0, $result, $number, $attempts);
	if ($self->{checkAttributesSubroutine} && $finaltest) {
	    my $sub = $self->{checkAttributesSubroutine};
	    delete $self->{checkAttributesSubroutine};
	    $attrStatus = 1;
	    &$sub();
	    $result &= $attrStatus if (defined($attrStatus));
	}
	if ($status ne "Garbage Arguments") {
	    my $attrs;
	    # $result will be 1 on success and 0 on failure
	    if ($self->{objectAttributes}) {
		$attrs = $self->{objectAttributes};
		my $update = ($self->{updateAttributes}) ? $attrs : undef;
		delete $self->{objectAttributes};
		$result &= $self->checkAttributes(%{$attrs}, %{$self->{NewObjectAttributes}}, thingie=>"object", attrType=>"postop", Update=>$update);
	    }
	    if ($self->{dirAttributes}) {
		$attrs = $self->{dirAttributes};
		my $update = ($self->{updateAttributes}) ? $attrs : undef;
		delete $self->{dirAttributes};
		$result &= $self->checkAttributes(%{$attrs}, %{$self->{NewDirAttributes}}, thingie=>"dir", attrType=>"postop", Update=>$update);
	    }
	    if ($self->{checkAttributesHandle}) {
		my $handle = $self->{checkAttributesHandle};
		delete $self->{checkAttributesHandle};
		$self->Test("QUIET", "Getattr", "--getattrFH $handle", 0);
		$result = 0 if ($self->{printStatus});
	    }
	    if ($self->{checkAttributes}) {
		$attrs = $self->{checkAttributes};
		my $update = ($self->{updateAttributes}) ? $attrs : undef;
		delete $self->{checkAttributes};
		$result &= $self->checkAttributes(%{$attrs}, %{$self->{NewAttributes}}, Update=>$update);
	    }
	} else {
	    delete $self->{objectAttributes};
	    delete $self->{dirAttributes};
	    delete $self->{checkAttributes};
	}
	delete $self->{updateAttributes};
	delete $self->{NewDirAttributes};
	delete $self->{NewObjectAttributes};
	delete $self->{NewAttributes};
	$Last_Test = $number;
	$self->{Proc} = $proc;
	$self->{error} = 0;
	$result &= $filehandleStatus if (defined($filehandleStatus));
	$self->{proc_failure} = !$result;
	$self->{printStatus} = $status;
	$self->printResult($finaltest, $result, $number, $attempts) if ($finaltest);
    } else {
	if ($finaltest && !$error) {
	    my $msg = "$self->{Proc} returned error code '$self->{printStatus}' rather than one of the allowable ones (";
	    $msg .= join(",", @statusExpected);
            $msg .= ")\n";
	    $Trace->comment(msg=>$msg);
            $Errors .= $msg;
            $Failed_command = $self->{last_cmd};
            $Failed_output = $self->{outputFromCmd};
	}
	$self->printResult(0, 0, $number, $attempts);
	$Trace->trace(msg=>"output from '$cmd' was \n@outputFromCmd\n");
	$self->printResult($finaltest, 0, $number, $attempts) if ($finaltest);
	$self->{exitCode} |= 1 if (!$internal);
	$self->{error} = 1;
	$self->{proc_failure} = 1;
    }
    if ($finaltest) {
	delete $self->{checkAttributesObjectHandle};
	delete $self->{objectAttributes};
	delete $self->{dirAttributes};
	delete $self->{checkAttributes};
	delete $self->{updateAttributes};
	delete $self->{NewDirAttributes};
	delete $self->{NewObjectAttributes};
	delete $self->{NewAttributes};
    }
    return $filehandle;
}

sub printResult
{
    my ($self, $finaltest, $result, $number, $attempts, $expected) = @_;
    $Trace->entry(msg=>"$number, $result, $finaltest");
              
    my $msg = "";
    my $output;
    my @log_result;

    if ($Last_Test ne $number) {
	$Trace->exit(msg=>"$Last_Test ne $number");
	return;
    }

    $msg = " ($self->{printStatus})" if ($self->{verbosity} > 1);
    $msg .= " after $attempts attempts" if ($attempts > 1);
    if ($attempts > 1 && $self->{verbosity} > 3) {
	$trace->comment(msg=>"retries=$self->retries, hard_mount=$self->{hard_mount}");
    }
    if ($finaltest) {
	if (defined($self->{ops}) && defined($self->{results})) {
	    $self->{ops} = $self->{serverobject}->GetOps();
	    $self->{results}->update(Ops => $self->{ops}, );
	}
	if ($result) {
	    $Pass_Cases++;
	    $output = "Test $number: ".colored("pass$msg", $colors{good});
	    $self->{errorCode} = 0;
	    @log_result = ('PASS', "returned '$self->{printStatus}'");
	} else {
	    $Fail_Cases++;
	    $output = "Test $number: ".colored("fail$msg", $colors{fail});
	    $self->{errorCode} = 1; 
            if (!$Errors) {
                $Errors = "returned '$self->{printStatus}'";
            }
	    @log_result = ('FAIL', $Errors);
	}
	if (exists($self->{TestCases}{$number}) && $self->{verbosity} > 1) {
	    $output .= " (".$self->{TestCases}{$number}.")";
	}
	$Trace->comment(msg=>"$output\n");

	main::logresult(@log_result) if ($self->{results});
    } elsif ($self->{verbosity} >= 2) {
	if ($result) {
	    $Trace->comment(msg=>"Proc $self->{Proc}: ".colored("pass$msg", $colors{info})."\n");
	} else {
	    $Trace->comment(msg=>"Proc $self->{Proc}: ".colored("fail$msg", $colors{bad})."\n");
	}
    }
    
    # print out the last command and output under certain conditions
    # 1. we are a synthetic client and there was a failure
    # 2. we are not a synthetic client, there was a failure and this is the result being reported
    if (($self->{synthetic} && !$result ) ||
        (!$self->{synthetic} && !$result && $finaltest)) {
	$Trace->comment(msg=>"ERROR from last command:\n$Failed_command\n");
	$Trace->comment(msg=>"OUTPUT from last command:\n$Failed_output\n");
    }
    if ($finaltest) {
        $Errors = undef;
    }
    $Trace->exit();
}

sub returnStatus
{
    my $self = shift @_;
    $Trace->entry();
    my $returnStatus = $self->{returnStatus};
    $self->{returnStatus} = undef;
    return $returnStatus;
}

sub printStatus
{
    my $self = shift @_;
    $Trace->entry();
    my $printStatus = $self->{printStatus};
    $self->{printStatus} = undef;
    return $printStatus;
}

sub checkEOF
{
    my $self = shift @_;
    $Trace->entry();
    my $eof = shift;
    if ($self->{EOF} != $eof) {
	$self->{exitCode} |= 1;
	$self->{errorCode} = 1;
	$Trace->warn(msg=>"EOF does not match $eof\n");
    }	
    return $self->{EOF};
}

sub checkSavedAttributes
{
    my $self = shift @_;
    $Trace->entry();
    my $status = 0;
#    return if ($self->{Release} < $self->{MinimumRelease});
    if ($self->{saveAttributes}) {
	my $name;
	my $attrs = $self->{object}{saveAttributes};
	my $readdir = $self->{object}{readdir};
	my $count = keys(%$readdir) - 1; # remove the entriesCount
	    
	foreach $name (keys %$attrs) {
	    $Trace->debug(msg=>"Readdir check: $name\n");
	    next if ($name =~ /^\s*$/);
	    if (!exists($$readdir{$name})) {
	        $status = 1;
		$Trace->comment(msg=>"Readdir check: $name did not appear\n");
	    }
	    if ($$attrs{$name}{fileid} != $$readdir{$name}{fileid}) {
	        $status = 1;
		$Trace->comment(msg=>"Readdir check: file ID for $name does not match $$attrs{$name}{fileid}:$$readdir{$name}{fileid}\n");
	    }
	    if ($readdir->{$name}{attributes} =~ /None/) {
		$Trace->comment(msg=>"No attributes returned from $self->{Proc} for entry $name\n");
	    } elsif ($self->{Proc} eq "READDIRPLUS") {
		# skip the atime since it will change
		my @savedAttrs = split(/\n/, $$attrs{$name}{attributes});
		my @attrs = split(/\n/, $$readdir{$name}{attributes});
		if ($name eq ".") {
		    @savedAttrs = grep(!/Access time/, @savedAttrs);
		    @attrs = grep(!/Access time/, @attrs);
		}
		if ($self->{stripe_format} =~ /full/i && $self->{volume_style} =~ /striped/i) {
                    if ($name eq "..") {
			$Trace->debug(msg => "Skipping major/minor device checks for directory $name BURT# 328587");
                        @savedAttrs = grep(!/Special/, @savedAttrs);
                        @attrs = grep(!/Special/, @attrs);
                    }
		    if (grep(/File Type.+?\(Directory\)/,@attrs)) { 
			$Trace->debug(msg => "Skipping size checks for directory $name\nDirectory sizes are not reliable on fully striped volumes");
			@savedAttrs = grep(!/File size/, @savedAttrs);
			@attrs = grep(!/File size/, @attrs);
			$Trace->debug(msg => "Saves Attributes: @savedAttrs \n Readdir Attrs: @attrs");
		    }
		}
		if (join(":", @savedAttrs) ne join(":", @attrs)) {
		    $status = 1;
		    $Trace->comment(msg=>"Readdir check: attributes for $name does not match\n");
		    $Trace->debug(msg=>"attrs=>\n$$attrs{$name}{attributes}\n");
		    $Trace->debug(msg=>"readdir=>\n$$readdir{$name}{attributes}\n");
		}
	    }
	    delete $$readdir{$name};
	}
	foreach $name (keys %$readdir) {
	    $status = 1;
	    $Trace->comment(msg=>"Readdir check: $name should not exist\n");
	}
    }
    return ($status != 1);
}

sub clearAttributes 
{
    my $self = shift;

    $Trace->entry();
    delete $self->{object};
    delete $self->{dir};
    delete $self->{linkdir};
    delete $self->{fromdir};
    delete $self->{saveAttributes};
}    

sub checkAttributes
{
    my $self = shift @_;
    $Trace->entry();
#    return if ($self->{Release} < $self->{MinimumRelease});
    my $status = 0;
    return if (@_ == ()); # part of the NonIdempotentHack
    my %options = (uid => undef,
		   gid => undef,
		   size => undef,
                   used => undef,
		   type => undef,
		   mode => undef,
		   blocksize => undef,
		   blocks => undef,
		   device => undef,
		   fsid => undef,
		   fid => undef,
		   atime => undef,
		   mtime => undef,
		   ctime => undef,
		   anow => 0,
		   mnow => 0,
		   cnow => 0,
		   attrType => "attributes",
		   thingie => "object",
		   quiet => 0,
		   Update => 1,
		   @_);
    if ($options{Attributes} || $options{Subroutine} || $options{dirAttributes} || $options{objectAttributes}) {
	my $ref = $self->{Tests};
	my $noRef = $self->{noTests};
	return if (!$options{Testcase});
	return if ($#$ref != -1 && !grep($options{Testcase} =~ /$_/, @$ref));
	return if (grep($options{Testcase} =~ /$_/, @$noRef));
	$self->{updateAttributes} = $options{Update};
	$self->{checkAttributes} = $options{Attributes};
	$self->{NewAttributes} = $options{NewAttributes};
	$self->{objectAttributes} = $options{objectAttributes};
	$self->{NewObjectAttributes} = $options{NewObjectAttributes};
	$self->{dirAttributes} = $options{dirAttributes};
	$self->{NewDirAttributes} = $options{NewDirAttributes};
	$self->{checkAttributesHandle} = $options{Getattr};
	$self->{checkAttributesSubroutine} = $options{Subroutine};
	return;
    }
    if ($options{Proc}) {
	$self->{Proc} = $options{Proc};
	delete $options{Proc};
    }
    return if ($self->{NonIdempotentHack});
    %options = (%options, %{$Statfs{$self->{Platform}}}) if ($self->{Proc} eq "STATFS" && $options{thingie} eq "object");
    %options = (%options, %Fsstat) if ($self->{Proc} eq "FSSTAT" && $options{thingie} eq "object");
    %options = (%options, %{$Fsinfo{$self->{Platform}}}) if ($self->{Proc} eq "FSINFO" && $options{thingie} eq "object");
    %options = (%options, %{$Pathconf{$self->{Platform}}}) if ($self->{Proc} eq "PATHCONF" && $options{thingie} eq "object");

    my $attrType = $options{attrType};
    my $thingie = $options{thingie};
    my $quiet = $options{quiet};
    my $now = "";
    my $Update = undef;
    if (ref($options{Update}) eq "HASH") {
	$Update = $options{Update};
    }
    delete $options{Update};
    $now .= " atime" if $options{anow};
    $now .= " mtime" if $options{mnow};
    $now .= " ctime" if $options{cnow};
    delete $options{attrType};
    delete $options{thingie};
    delete $options{quiet};
    delete $options{anow};
    delete $options{mnow};
    delete $options{cnow};
    if ($self->{$thingie}{$attrType}{NONE}) {
	$Trace->comment(msg=>"No attributes returned from $self->{Proc}\n");
	return 1;
    }
    delete $options{NONE};
    foreach my $key (keys %options) {
	my $skip = 0;
	my $prev = "prev$key";
        if (!defined($options{$key})) {
            next;
        }
	if ($Update) {
	    $$Update{$key} = $self->{$thingie}{$attrType}{$key};
	}
	if ($key =~ /time/) {
	    if ($options{$key} eq "same") {
		if ($self->{version} != 2) {
		    $Trace->warn(msg=>"same is only supported in times for version 2\n");
		    next;
		}
		$skip = 1;
		if (defined($options{$key})) {
		    if (exists($self->{$thingie}{$attrType}{$prev})) {
			if ($self->{$thingie}{$attrType}{$key} ne $self->{$thingie}{$attrType}{$prev}) {
			    $status = 1;
                            my $msg = "ERROR with $thingie: $key not same: $self->{$thingie}{$attrType}{$prev} to $self->{$thingie}{$attrType}{$key}\n";
                            $Errors .= $msg;
			    if (!$quiet) {
				$Trace->comment(msg=>$msg);
			    }
			}
		    } 
		}
	    } elsif ($self->{version} == 2 && $self->{$thingie}{$attrType}{$key} eq $self->{$thingie}{$attrType}{$prev}) {
		$status = 1;
		$skip = 1;
                my $msg = "ERROR with $thingie: $key not different: $self->{$thingie}{$attrType}{$key}\n";
                $Errors .= $msg;
		if (!$quiet) {
		    $Trace->comment(msg=>$msg);
		}
	    } elsif ($options{$key} eq $self->{$thingie}{$attrType}{$key}) {
		if ($now =~ /$key/) {
		    $status = 1;
                    my $msg = "ERROR with $thingie: $key not different: $self->{$thingie}{$attrType}{$key}\n";
                    $Errors .= $msg;
		    $Trace->comment(msg=>$msg);
		}
		$skip = 1;
	    } elsif (($now =~ /$key/) || $options{$key} eq "now") {
		my $now = time();
		$skip = 1;
		if (($self->{$thingie}{$attrType}{$key} > ($now + $self->{delta})) || $self->{$thingie}{$attrType}{$key} < ($now - $self->{delta})) {
		    $status = 1;
                    my $msg = "ERROR with $thingie: $key=> $self->{$thingie}{$attrType}{$key} not within $self->{delta} seconds of $now\n";
                    $Errors .= $msg;
		    if (!$quiet) {
			$Trace->comment(msg=>$msg);
		    }
		}
	    }
	}
        if (!$skip) {
            my $value1 = $self->{$thingie}{$attrType}{$key};
            my $value2 = $options{$key};
	    my $msg;

            if ($key =~ /time/i && $self->{volume_style} =~ /striped/i && $self->{stripe_format} =~ /full/i && $self->{concurrency} !~ /low/) {
                if ($value1 < $value2) {
                    $status = 1;
                    $msg = "ERROR with $thingie: $key: $value1 is less than $value2\n";
		    $Errors .= $msg;
		}
	    } elsif ($options{$key} ne $self->{$thingie}{$attrType}{$key}) { 
                $value1 = sprintf("0%o", $value1) if ($key eq "mode");
                $value2 = sprintf("0%o", $value2) if ($key eq "mode");
    	        $status = 1;
                $msg = "ERROR with $thingie: $key: expected $value2 but got $value1\n";
                $Errors .= $msg;
            }
    	    if (!$quiet && $msg) {
    	        $Trace->comment(msg=>$msg); 
    	    }
    	    
	}    
    }
    $self->{exitCode} |= $status if (!$quiet);
    # return 0 on failure and 1 on success
    if ($status) {
        $Failed_command = $self->{last_cmd};
        $Failed_output = $self->{outputFromCmd};
	$attrStatus &= 0;
	return 0;
    } else {
	$attrStatus &= 1;
	return 1;
    }
} # end checkAttributes

sub LogFailure {
    my $self = shift;
    $Trace->entry();
    $Failed_command = $self->{last_cmd};
    $Failed_output = $self->{outputFromCmd};
    $attrStatus = 0;
    $Trace->exit();
}

sub checkHandle
{
    my ($self, $number, $handle) = @_;
    $Trace->entry();
    my $ref = $self->{Tests};
    my $noRef = $self->{noTests};
    return if ($#$ref != -1 && !grep($number =~ /$_/, @$ref));
    return if (grep($number =~ /$_/, @$noRef));
    $self->{checkHandle} = $handle;
}

sub getAttributes
{
    my $self = shift @_;
    $Trace->entry();
    return () if ($self->{NonIdempotentHack});
    my $thingie = (@_ != ()) ? shift @_ : "object";
    my $attrType = (@_ != ()) ? shift @_ : "attributes";
    my %attributes = ();
    foreach my $key (keys %{$self->{$thingie}{$attrType}}) {
	next if $key =~ /prev|read/i;
	$attributes{$key} = ${$self->{$thingie}{$attrType}}{$key};
    }
    return %attributes;
}

sub Masks
{
    my ($self, $procedure) = @_;
    $Trace->entry();
    return $Masks{$self->{version}}{$procedure};
}


sub errorCode
{
    my ($self) = @_;
    $Trace->entry();
    return $self->{errorCode};
}

sub exitCode
{
    my ($self) = @_;
    $Trace->entry();
    if ($self->{memory}) {
	$self->{serverobject}->GetMemoryUsage(AdminUser => $self->{user},
					      AdminPassword => $self->{password},
					      Memory => $self->{memory},
					      );
	$self->{memory} = undef;
    }
    if ($self->{serverobject}) {
	$self->{serverobject}->DESTROY();
	$self->{serverobject} = undef;
    }
    if ( exists($main::{'Tharn::'}) ) { # NATE
	if (!$self->{results}) {
	    my $result = $Fail_Cases ? "FAIL" : "PASS";
	    $Trace->fatal(msg=>"$Pass_Cases PASS, $Fail_Cases FAIL", result=>$result);
	} 
    } else {
	return $self->{exitCode};
    }
}

sub mountpoint
{
    my ($self) = @_;
    $Trace->entry();
    return $self->{mount};
}


sub server
{
    my ($self) = @_;
    $Trace->entry();
    return $self->{server};
}

sub lastTest
{
    my ($self, $test) = @_;
    $Trace->entry();
    my $code = ($Last_Test =~ /$test/) ? 1 : 0;
    return 1 if ($self->{listcases});
    return ($code && !$self->{errorCode});
}
    
sub checkForTestFailure
{
    my ($self, $msg) = @_;
    $Trace->entry();
    if ($self->{error}) {
	$msg = "\n" if (!$msg || $msg eq "");
	$Trace->comment(msg=>$msg);
	$self->{exitCode} |= 1;
    }
}

sub TestFailed
{
    my ($self, $test) = @_; 
    $Trace->entry();
    return 0 if ($Last_Test !~ /$test/);
    return $self->{errorCode};
}

sub TestPassed
{
    my ($self, $test) = @_;
    $Trace->entry();
    return 0 if ($Last_Test !~ /$test/);
    return !$self->{errorCode};
}

sub TestExecuted
{
    my ($self, @test) = @_;
    $Trace->entry();
    return undef if ($#ExecutedTests == -1);
    my $rc = undef;
    foreach my $test (@test) {
	$rc |= grep($test eq $_, @ExecutedTests);
    }
    return $rc;
}

sub runTest
{
    my ($self, $test) = @_;
    $Trace->entry();
    my $ref = $self->{Tests};
    return ($#$ref == -1 || grep($test =~ /$_/, @$ref));
}

sub checkForTest
{
    my ($self, @tests) = @_;
    $Trace->entry();
    my $ref = $self->{Checks};
    my $status;
    if ($#$ref == -1) {
	my $ref = $self->{noTests};
	if ($#$ref == -1) {
	    return 1;
	} else {
	    foreach my $test (@tests) {
		return 0 if grep($test =~ /$_/, @$ref);
	    }
	    return 1;
	}
    }
    $status = 0;
    foreach my $test (@tests) {
	$status |= grep($test =~ /$_/, @$ref);
    }
    return $status;
}

sub verifyFileGone
{
    my ($self, $file) = @_;
    $Trace->entry();
#    return if ($self->{Release} < $self->{MinimumRelease});
    if ($self->Lookup($file)) {
	$Trace->comment(msg=>"Error-> $file should not exist\n");
	$attrStatus &= 0;
	$self->{exitCode} |= 1;
    }
}

sub verifyFileExistence
{
    my ($self, $file, $type);
    $self = shift;
    $Trace->entry();
#    return if ($self->{Release} < $self->{MinimumRelease});
    $file = shift;
    $type = ($#_ == -1) ? "REG" : shift;
    my $fh;
    if (!($fh=$self->Lookup($file))) {
	if ($self->{error} == 2) {
	    $Trace->comment(msg=>"Error=> $file does not exist.\n");
	    $attrStatus &= 0;
	} else {
	    $Trace->comment(msg=>"Error=> lookup of $file returned $self->{printStatus}.\n");
	    $attrStatus &= 0;
	}
    } elsif ($self->{version} == 3 && $self->{object}{postop}{type} !~ /$type/) {
	$Trace->comment(msg=>"Error=> $file exists but is not a $type:$self->{object}{postop}{type}\n");
	$attrStatus &= 0;
    }
    if (!grep($type eq $_, ("REG", "LNK", "DIR", "CHR", "BLK", "FIFO", "SOCK"))) {
	$Trace->comment(msg=>"Error=> $type is not a known type for verifyFileExistence\n");
	$attrStatus &= 0;
    }
    return $fh;
}

sub verifyDirGone
{
    my ($self, $file) = @_;
    $Trace->entry();
#    return if ($self->{Release} < $self->{MinimumRelease});
    if ($self->Lookup($file)) {
	$Trace->comment(msg=>"Error=> $file was not removed correctly\n");

    }
}

sub setMinQuotas {
    my $self = shift;
    $Trace->entry();
    return undef if ($self->{listcases});
    return $self->{QuotaFH} if ($self->{QuotaFH});
    if ($self->{Options}{QUOTA_TESTS}) {
	$self->{quotaFH} = $self->getMountFH($self->{Options}{QUOTA_TESTS});
	return $self->{quotaFH};
    }
    return undef if (!$self->ConnectToServer() || !$self->{volume});
    my $output;
    my $vfs = $self->{volume};
    if ($self->{serverobject}->ExecuteCLI(Command=>"vfs show $vfs", Output=>\$output)) {
	foreach $_ (split(/\n/, $output)) {
	    $self->{minQuota} = $1 if (/^\s+MinQ\s+(\S+)/);
	    $self->{maxQuota} = $1 if (/^\s+MaxQ\s+(\S+)/);
	}
	if (exists($self->{minQuota}) && exists($self->{maxQuota})) {
	    $self->{minQuota} =~ s/,//g;
	    $self->{maxQuota} =~ s/,//g;
	    $Trace->debug(msg=> "MinQ saved as $self->{minQuota}, MaxQ saved as $self->{maxQuota}\n");
	    if ($self->{serverobject}->ExecuteCLI(Command=>"vfs modify -vfsname $vfs -qmin 0 -qmax 0", Output=>\$output) &&
		$output !~ /^\s*$/) {
		$Trace->warn(msg=>"Could not set min quotas: $output\n");
		delete $self->{minQuota};
		delete $self->{maxQuota};
		return undef;
	    }
	    return $self->{handle};
	}
    }
    return undef;
}    

sub setMaxQuota {
    my $self = shift;
    $Trace->entry();
    return undef if ($self->{listcases});
    return undef if (!$self->ConnectToServer() || !$self->{volume});
    my $output;
    my $quota = undef;
    my $vfs = $self->{volume};
    if ($self->{serverobject}->ExecuteCLI(Command=>"vfs show $vfs", Output=>\$output)) {
	foreach $_ (split(/\n/, $output)) {
	    $self->{minQuota} = $1 if (/^\s+MinQ\s+(\S+)/);
	    $self->{maxQuota} = $1 if (/^\s+MaxQ\s+(\S+)/);
	    $quota = $1 if (/^\s+SP Avail\s+(\S+)/);
	}
	if (exists($self->{minQuota}) && exists($self->{maxQuota}) && defined($quota)) {
	    $self->{minQuota} =~ s/,//g;
	    $self->{maxQuota} =~ s/,//g;
	    $quota =~ s/,//g;
	    $Trace->debug(msg=>"MinQ saved as $self->{minQuota}, MaxQ saved as $self->{maxQuota}\n");
	    if ($self->{serverobject}->ExecuteCLI(Command=>"vfs modify -vfsname $vfs -qmin $quota -qmax $quota", Output=>\$output) &&
		$output !~ /^\s*$/) {
		$Trace->warn(msg=>"Could not set max quotas: $output\n");
		delete $self->{minQuota};
		delete $self->{maxQuota};
		return undef;
	    }
	    return 1;
	}
    }
    return undef;
}    

sub restoreQuotas {
    my $self = shift;
    $Trace->entry();
    return undef if ($self->{listcases});
    return undef if (!$self->ConnectToServer() || !exists($self->{minQuota}));
    my $vfs = $self->{volume};
    my $output;
    if ($self->{serverobject}->ExecuteCLI(Command=>"vfs modify -vfsname $vfs -qmin $self->{minQuota} -qmax $self->{maxQuota}", 
					  Output=>\$output) && $output !~ /^\s*$/) {
	$Trace->warn(msg=>"Could not restore quotas: $output\n");
	return undef;
    }
    delete $self->{minQuota};
    delete $self->{maxQuota};
    return 1;
}    

sub VPrint {
    my ($self, $verbosity, $msg, $status) = @_;
    $Trace->entry();
    $status = "normal" if ($status eq "");
    if ($verbosity >= $self->{verbosity}) {
	$Trace->comment(msg=>$msg);
    }
}

my %common_params = (
		     acceptable_codes => { type => ARRAYREF, optional => 1, default => [0] },
		     testcase => { type => SCALAR , optional => 1, default => "QUIET" },
		     authuid => { type => SCALAR | UNDEF , optional => 1},
		     authgid => { type => SCALAR | UNDEF, optional => 1},
		     udp_sport => { type => SCALAR | UNDEF, optional => 1},
		     server => { type => SCALAR | UNDEF, optional => 1},
		     src_ip => { type => SCALAR | UNDEF, optional => 1},
		     device => { type => SCALAR | UNDEF, optional => 1},
		     ether => { type => SCALAR | UNDEF, optional => 1},
		     timeout => { type => SCALAR | UNDEF , optional => 1},
		     fatal => {type => SCALAR | UNDEF, optional => 1, default => 1},
		     );

my %attribute_params = (
			mode => { type => SCALAR | UNDEF, optional => 1},
			uid => { type => SCALAR | UNDEF, optional => 1},
			gid => { type => SCALAR | UNDEF , optional => 1},
			size => { type => SCALAR | UNDEF, optional => 1},
			);

sub _process_common_params {
    my ($self, %params) = @_;
    my $options = "";
    $options .= " --authuid $params{authuid}" if (defined($params{authuid}));
    $options .= " --authgid $params{authgid}" if (defined($params{authgid}));
    $options .= " --srcip $params{src_ip}" if (defined($params{src_ip}));
    $options .= " --destaddress $params{ether}" if (defined($params{ether}));
    $options .= " --device $params{device}" if (defined($params{device}));
    $options .= " --destip $params{server}" if (defined($params{server}));
    $options .= " --timeout $params{timeout}" if (defined($params{timeout}));
    $options .= " --udpsport $params{udpsport}" if (defined($params{udp_sport}));
    return $options;
}

sub _process_attribute_params {
    my ($self, %params) = @_;
    my $flag = ($self->{version} == 3) ? "object" : "file";
    my $options = "";
    $options .= " --${flag}mode $params{mode}" if (defined($params{mode}));
    $options .= " --${flag}uid $params{uid}" if (defined($params{uid}));
    $options .= " --${flag}gid $params{gid}" if (defined($params{gid}));
    $options .= " --${flag}size $params{size}" if (defined($params{size}));
    return $options;
}

sub get_port {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     port => { type => SCALAR },
					 }
					 );
    $options = $self->_process_common_params(%params);
    $Trace->entry();
    my $port;
    if ($params{port} =~ /nfs/) {
	$port = $self->_get_port("nfs", 100003);
	$self->{nfsPort} = $port;
	$self->{dport} = $port;
    } elsif ($params{port} =~ /mount/) {
	$port = $self->_get_port("mount", 100005);
	$self->{mountPort} = $port;
    } else {
	$Trace->fatal(msg=>"$params{port} is not nfs or mount\n");
    }
    return $port;
}
    
sub mount_null {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					 }
					 );
    $Trace->entry();
    my $mount;
    if (!($mount = $self->{Mount})) {
	$mount = $self->clone(Program=>mount, mount=>1);
	$self->{Mount} = $mount;
	$mount->{dport} = $mount->{mountPort};
    }
    $params{authuid} = 0 if (!defined(authuid));
    $params{authgid} = 0 if (!defined(authgid));
    $options = $self->_process_common_params(%params);
    $mount->Test($params{testcase}, "Null", $option, @{$params{acceptable_codes}});
    $self->{printStatus} = $mount->{printStatus};
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($mount->{proc_failure});
    return 1;
}

sub mount {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     share => { type => SCALAR },
					 }
					 );
    $Trace->entry(msg=>"$params{share}");
    my $mount;
    if (!($mount = $self->{Mount})) {
	$mount = $self->clone(Program=>mount, mount=>1);
	$self->{Mount} = $mount;
	$mount->{dport} = $mount->{mountPort};
    }
    $params{authuid} = 0 if (!defined($params{authuid}));
    $params{authgid} = 0 if (!defined($params{authgid}));
    $options = $self->_process_common_params(%params);
    $options .= " --mountpoint $params{share}";
    my $new_handle = $mount->Test($params{testcase}, "Mnt", $options, @{$params{acceptable_codes}});
    $self->{printStatus} = $mount->{printStatus};
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($mount->{proc_failure});
    $Trace->exit(msg=>"return $new_handle");
    return $new_handle;
}

sub umount {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     share => { type => SCALAR },
					 }
					 );
    $Trace->entry(msg=>"$params{share}");
    my $mount;
    if (!($mount = $self->{Mount})) {
	$mount = $self->clone(Program=>mount, mount=>1);
	$self->{Mount} = $mount;
	$mount->{dport} = $mount->{mountPort};
    }
    $params{authuid} = 0 if (!defined($params{authuid}));
    $params{authgid} = 0 if (!defined($params{authgid}));
    $options = $self->_process_common_params(%params);
    $options .= " --umntpath $params{share}";
    $mount->Test($params{testcase}, "Umnt", $options, @{$params{acceptable_codes}});
    $self->{printStatus} = $mount->{printStatus};
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($mount->{proc_failure});
    $Trace->exit(msg=>"return $new_handle");
    return $new_handle;
}

sub nfs_null {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					 }
					 );
    $Trace->entry();
    $options = $self->_process_common_params(%params);
    $self->Test($params{testcase}, "Null", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit();
    return 1;
}

sub getattr {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					 }
					 );
    $Trace->entry(msg=>"$params{handle}");
    my $options = "--getattrFH $params{handle}";
    $options .= $self->_process_common_params(%params);
    $self->Test($params{testcase}, "Getattr", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    my %attributes = ($self->{version} == 2) ? $self->getAttributes() : $self->getAttributes("object", "postop");
    $Trace->exit();
    return %attributes;
}

sub setattr {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     %attribute_params,
					     handle => { type => SCALAR },
					 }
					 );
    $Trace->entry(msg=>"$params{handle}");
    my $options = "--setattrFH $params{handle}";
    $options .= $self->_process_common_params(%params);
    $options .= $self->_process_attribute_params(%params);
    $self->Test($params{testcase}, "Setattr", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit();
}

sub lookup {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					     name => { type => SCALAR },
					     name_length => { type => SCALAR, optional => 1 },
					 }
					 );
    
    $Trace->entry(msg=>"$params{handle} $params{name}");
    my $flag = ($self->{version} == 3) ? "object" : "file";
    my $options = "--dirhandle $params{handle} --${flag}name $params{name}";
    $options .= $self->_process_common_params(%params);
    $options .= " --namelength $params{name_length}" if (defined($params{name_length}));
    my $handle = $self->Test($params{testcase}, "Lookup", $options, @{$params{acceptable_codes}});
    if ($params{fatal}) {
	$Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    }
    $Trace->exit(msg=>"return $handle");
    return $handle;
}

sub access {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					     access => { type => SCALAR},
					 }
					 );
    $Trace->entry(msg=>"$params{handle} $params{access}");

    if ($self->{synthetic} && $self->{version} == 2) {
	# maybe this should be pushed up a layer...
	$Trace->exit(msg=>"Access not supported in NFSv2");
	return "read lookup modify extend delete execute";
    }
    my $options = "--accessFH $params{handle}";
    $options .= $self->_process_common_params(%params);
    $options .= " --accessbits $params{access}";
    $self->Test($params{testcase}, "Access", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    my %attributes = ($self->{version} == 2) ? $self->getAttributes() : $self->getAttributes("object", "postop");
    $Trace->exit(msg=>"return $attributes{Access}");
    return $attributes{Access};
}

sub readlink {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					 }
					 );
    $Trace->entry();
    my $options = "--readlinkFH $params{handle}";
    $options .= $self->_process_common_params(%params);
    $self->Test($params{testcase}, "Readlink", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit();
}

sub read {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					     offset => { type => SCALAR | UNDEF, optional => 1, default => 0 },
					     count => { type => SCALAR },
					 }
					 );
    $Trace->entry();
    my $options = "--readFH $params{handle} --readoffset $params{offset} --readcount $params{count}";
    $options .= $self->_process_common_params(%params);
    $self->Test($params{testcase}, "Read", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit();
    return $self->{object}{postop}{readData};
}

sub write {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					     offset => { type => SCALAR | UNDEF, optional => 1, default => 0 },
					     count => { type => SCALAR | UNDEF, optional => 1 },
					     stable_how => { type => SCALAR | UNDEF, optional => 1, default => 1 },
					     data => {type => SCALAR },
					     write_length => { type => SCALAR | UNDEF, optional => 1 },
					 }
					 );
    $Trace->entry();
    if (!defined($params{count})) {
	$params{count} = length($params{data});
    }
    my $options = "--writeFH $params{handle} --writeoffset $params{offset}";
    if ($self->{version} == 3) {
	$options .= " --writecount $params{count}";
	$options .= " --writestablehow $params{stable_how}";
    } else {
	$options .= " --writetotalcount $params{count}";
    }
    $options .= " --writedata \"$params{data}\"";
    $options .= " --writelength $params{write_length}" if (defined($params{write_length}));
    $options .= $self->_process_common_params(%params);
    $self->Test($params{testcase}, "Write", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit();
}

sub create {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     %attribute_params,
					     handle => { type => SCALAR },
					     file => { type => SCALAR },
					     name_length => { type => SCALAR, optional => 1 },
					 }
					 );
    $Trace->entry();
    my $options = "--createFH $params{handle} --createfilename $params{file}";
    $options .= $self->_process_common_params(%params);
    $options .= $self->_process_attribute_params(%params);
    $options .= " --createnamelength $params{name_length}" if (defined($params{name_length}));
    my $new_handle = $self->Test($params{testcase}, "Create", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit(msg=>"return $new_handle");
    return $new_handle;
}

sub mkdir {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     %attribute_params,
					     handle => { type => SCALAR },
					     directory => { type => SCALAR },
					     dir_length => { type => SCALAR , optional => 1},
					 }
					 );
    $Trace->entry();
    my $options = "--mkdirFH $params{handle} --dirname $params{directory}";
    $options .= $self->_process_common_params(%params);
    $options .= $self->_process_attribute_params(%params);
    $options .= " --dirlengthname $params{dir_length}" if (defined($params{dir_length}));
    my $new_handle = $self->Test($params{testcase}, "Mkdir", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit(msg=>"return $new_handle");
    return $new_handle;
}

sub symlink {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     %attribute_params,
					     handle => { type => SCALAR },
					     target => { type => SCALAR },
					     name => { type => SCALAR },
					     name_length => { type => SCALAR , optional => 1},
					     target_length => { type => SCALAR , optional => 1},
					 }
					 );
    $Trace->entry();
    my $options;
    if ($self->{version} == 3) {
	$options = "--symdir $params{handle} --symname $params{name} --symdata $params{target}";
	$options .= " --symlengthname $params{name_length}" if (defined($params{name_length}));
	$options .= " --symlengthdata $params{target_length}" if (defined($params{target_length}));
    } else {
	$options = "--fromdirFH $params{handle} --fromname $params{name} --linktext $params{target}";
	$options .= " --symlengthname $params{from_length}" if (defined($params{name_length}));
	$options .= " --symlengthdata $params{link_length}" if (defined($params{target_length}));
    }
    $options .= $self->_process_common_params(%params);
    $options .= $self->_process_attribute_params(%params);
    my $new_handle = $self->Test($params{testcase}, "Symlink", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit(msg=>"return $new_handle");
    return $new_handle;
}

sub mknod {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     %attribute_params,
					     handle => { type => SCALAR },
					     name => { type => SCALAR },
					     name_length => { type => SCALAR , optional => 1},
					     type => { type => SCALAR , optional => 1},
					     spec => { type => SCALAR , optional => 1},
					 }
					 );
    $Trace->entry();
    my $options = "--mknodFH $params{handle} --mknodname $params{name}";
    $options .= $self->_process_common_params(%params);
    $options .= $self->_process_attribute_params(%params);
    $options .= " --mknodenamelength $params{name_length}" if (defined($params{name_length}));
    $options .= " --mknodtype $params{type}" if (defined($params{type}));
    $options .= " --mknodspec $params{spec}" if (defined($params{spec}));
    my $new_handle = $self->Test($params{testcase}, "Mknod", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit(msg=>"return $new_handle");
    return $new_handle;
}

sub remove {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					     file => { type => SCALAR },
					     name_length => { type => SCALAR, optional => 1 },
					 }
					 );
    $Trace->entry();
    my $options = "--removeFH $params{handle} --removefilename $params{file}";
    $options .= $self->_process_common_params(%params);
    $options .= "--removenamelength $params{name_length}" if (defined($params{name_length}));
    $self->Test($params{testcase}, "Remove", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit();
}

sub rmdir {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					     directory => { type => SCALAR },
					     name_length => { type => SCALAR, optional => 1 },
					 }
					 );
    $Trace->entry();
    my $options = "--rmdirFH $params{handle} --rmdirfilename $params{directory}";
    $options .= $self->_process_common_params(%params);
    $options .= "--rmdirnamelength $params{name_length}" if (defined($params{name_length}));
    $self->Test($params{testcase}, "Rmdir", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit();
}

sub rename {
}

sub link {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     %attribute_params,
					     linkhandle => { type => SCALAR },
					     dirhandle => { type => SCALAR },
					     name => { type => SCALAR },
					     name_length => { type => SCALAR , optional => 1},
					 }
					 );
    $Trace->entry();
    my $options;
    if ($self->{version} == 3) {
	$options = "--linkfilehandle $params{linkhandle} --linkfromdirhandle $params{dirhandle} --linkfromfilename $params{name}";
	$options .= " --linkfromnamelength $params{name_length}" if (defined($params{name_length}));
    } else {
	$options = "--fromhandle $params{handle} --todirhandle $params{name} --tofilename $params{target}";
	$options .= " --tonamelength $params{from_length}" if (defined($params{name_length}));
    }
    $options .= $self->_process_common_params(%params);
    $options .= $self->_process_attribute_params(%params);
    my $new_handle = $self->Test($params{testcase}, "Link", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit(msg=>"return $new_handle");
    return $new_handle;
}

sub readdir {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					     cookie => { type => SCALAR, optional => 1 },
					     count => { type => SCALAR, optional => 1 },
					 }
					 );
    $Trace->entry();
    delete $self->{object}{readdir};
    my $options = "--readdirFH $params{handle}";
    $options .= $self->_process_common_params(%params);
    $options .= " --cookie $params{cookie}" if (defined($params{cookie}));
    $options .= " --readdircount params{count}" if (defined($params{count}));
    $self->Test($params{testcase}, "Readdir", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus})") if ($self->{proc_failure});
    my @entries;
    foreach my $file (keys %{$self->{object}{readdir}}) {
	my %entry = %{$self->{object}{readdir}{$file}};
	$entry{name} = $file;
	push(@entries, \%entry);
    }
    $Trace->exit();
    return @entries;
}

sub readdirplus {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					     cookie => { type => SCALAR, optional => 1 },
					     verf => { type => SCALAR, optional => 1 }, 
					     dircount => { type => SCALAR, optional => 1 },
					     maxcount => { type => SCALAR, optional => 1 },
					 }
					 );
    $Trace->entry(msg=>"$params{handle}");
    delete $self->{object}{readdir};
    my $options = "--readdirplusFH $params{handle}";
    $options .= $self->_process_common_params(%params);
    $options .= " --cookie $params{cookie}" if (defined($params{cookie}));
    $options .= " --cookieverf $params{verf}" if (defined($params{verf}));
    $options .= " --dircount params{dircount}" if (defined($params{dircount}));
    $options .= " --maxcount params{maxcount}" if (defined($params{maxcount}));
    $self->Test($params{testcase}, "Readdirplus", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    my @entries;
    foreach my $file (keys %{$self->{object}{readdir}}) {
	my %entry = %{$self->{object}{readdir}{$file}};
	$entry{name} = $file;
	push(@entries, \%entry);
    }
    $Trace->exit();
    return @entries;
}

sub fsstat {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					 }
					 );
    $Trace->entry();
    my $options = "--fsstatFH $params{handle}";
    $options .= $self->_process_common_params(%params);
    $self->Test($params{testcase}, "Fsstat", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
}

sub fsinfo {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					 }
					 );
    $Trace->entry();
    my $options = "--fsinfoFH $params{handle}";
    $options .= $self->_process_common_params(%params);
    $self->Test($params{testcase}, "Fsinfo", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
}

sub pathconf {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
					     %common_params,
					     handle => { type => SCALAR },
					 }
					 );
    $Trace->entry();
    my $options = "--pathconfFH $params{handle}";
    $options .= $self->_process_common_params(%params);
    $self->Test($params{testcase}, "Pathconf", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
}

sub commit {
}

sub lock {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
                                          %common_params,
                                          handle => { type => SCALAR },
                                          offset => { type => SCALAR | UNDEF},
                                          length => { type => SCALAR },
                                          svid => { type => SCALAR },
                                          exclusive => { type => SCALAR },
                                          block => { type => SCALAR },
                                          owner => { type => SCALAR, optional => 1 },
					 }
					 );
    $Trace->entry(msg=>"$params{handle}");
    my $options = "--filehandle $params{handle}";
    $options .= $self->_process_common_params(%params);
    $options .= " --exclusive $params{exclusive}";
    $options .= " --block $params{block}";
    $options .= " --locksvid $params{svid}";
    $options .= " --lockowner $params{owner}" if (defined($params{owner}));
    $options .= " --lockoffset $params{offset}";
    $options .= " --locklength $params{length}";

    $self->Test($params{testcase}, "Lock", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit();
}

sub unlock {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => HASHREF } ],
					 {
                                          %common_params,
                                          handle => { type => SCALAR },
                                          offset => { type => SCALAR },
                                          length => { type => SCALAR },
                                          svid => { type => SCALAR },
                                          owner => { type => SCALAR, optional => 1 },
					 }
					 );
    $Trace->entry(msg=>"$params{handle}");
    my $options = "--filehandle $params{handle}";
    $options .= $self->_process_common_params(%params);
    $options .= " --locksvid $params{svid}";
    $options .= " --lockowner $params{owner}" if (defined($params{owner}));
    $options .= " --lockoffset $params{offset}";
    $options .= " --locklength $params{length}";

    $self->Test($params{testcase}, "Unlock", $options, @{$params{acceptable_codes}});
    $Trace->fatal(msg=>"failed with exit code $self->{printStatus}") if ($self->{proc_failure});
    $Trace->exit();
}

sub Lookup
{
    my ($self, $file) = @_;
    $Trace->entry();
    my $dirfh = $self->{handle};
    my $fh = "";
    my @path = split(/\//, $file);
    $index = scalar @path;
    foreach my $object (@path) {
	$self->{noAttrs} = 1 if ($index != 1); # not the last entry
	if ($self->{version} == 3) {
	    $fh = $self->Test("INTERNAL", "Lookup", "--dirhandle $dirfh --objectname $object", 0);
	} else {
	    $fh = $self->Test("INTERNAL", "Lookup", "--dirhandle $dirfh --filename $object", 0);
	}
	$self->{noAttrs} = 0 if ($index != 1); # not the last entry
	return undef if ($self->{error});
	$dirfh = $fh;
	$index--;
    }
    return $fh;
}

sub Help {
    print <<"xxEOFxx";
  Usage:
    $0 -server SpinServer [-mount mountpoint] [-device device] 
    [-testdir path_to_tests] [-share sharepath]
    [-verbosity 012..] [-help] [ -ether Address] 
    [-name uid] [-group gid] 
    [-srcip IP address] 
    
    SpinServer is the IP address or name of the server to test against
    Address is the ethernet address of the SpinServer
    device is the name of the ethernet device (eth1 is default)
    path_to_tests is the full pathname to the TESTs testdir 
    mountpoint is defaulted to /mnt/nfs
    sharepath is defaulted to / (see previous line)
    -srcip will specify an IP address to use as the sender
    -user will specify the user for VFS commands [root]
    -password is the password for the user account
    
    -verbosity will run the tests in verbose mode
    -help prints out this message
    
    Note that all command-line options and modes can be specified as the shortest
    string that uniquely identifies the option or mode.
xxEOFxx
    
    return;
}

sub GetRequirements {
    my $ref = shift; # get the pointer to the options hash
    my %options = %$ref; # dereference to get the hash itself

    # Using the options that were passed in, calculate and return the
    # requirements.

    my %Requirements = (clients => [1, 1],
                        servers => [1, 1],
                        );
    return %Requirements;
} # end GetRequirements

sub InterruptHandler {
    exit(255);
}

sub wrap_cmd {
    my $cmd = shift;

    return ($<) ?
        "/usr/software/bin/sudo sh -c \'export LD_LIBRARY_PATH=$Source/lib/$^O:/usr/software/lib; $cmd\'" : $cmd;
}    
1;



