# File Name     :       UnitTestCLIGen.pm
#
# Summary       :       Contains methods to generate DFM CLI unit tests using NACL APISet method
#
# Creation Date :       6th August 2010
#
# Author        :       tanzim , dl-sm-nacl-dev@netapp.com
#
# Version History
# ---------------
#
#  Version 1
#       Summary: Added method for "command expected output"
#       Date   : 31st August 2010
#       Author : Deepthi
#
#  Version 2
#       Summary: Added method for "inserting test case list"
#       Date   : 1st September 2010
#       Author : Deepthi
#
#  Version 3
#       Summary: Modifying patterns for expected output
#       Date   : 2nd September 2010
#       Author : Tanzim
#
# Copyright (c) 2010 NetApp, Inc.
# All rights reserved.
#

#
# Package Declaration
#

package CodeModules::CLI::UnitTestCLIGen;

#
# Compiler Directives
#

use strict;
use warnings;

#
# Class Inheritance
#

use base qw(CodeModules::TestTemplateGen);

#
# Perl Modules
#

use Data::Dumper;
use Tie::IxHash;
use Tie::File;	
use IO::File;

#
# User Specific Modules
#
use ParseEngines::CommonParseEngine;

#
# Class Objectives
#

my $parse_obj =  ParseEngines::CommonParseEngine->new();

# Constructor Definition

sub new {
    my $class = shift;
    my $self  = {};
    bless $self, $class;
    return $self;
}


################################################################################
# Subroutine  : gen_unit_tests                                                 #
#                                                                              #
# Description : This method generates DFM CLI unit tests using NACL APISet     #
#		methods							       #
#                                                                              #
# Returns     : Nothing                                                        #
#               Generates the unit test files                                  #
################################################################################


sub gen_unit_tests {
    my $self = shift;
    my $conf_hash = shift;

     # Variable initialization	
     my $code_type = $conf_hash->{'code_type'};	
     my $code_path = $conf_hash->{'code_path'}; 	
     my $file_path = $conf_hash->{'code'}->{$code_type}->{'file_path'};
     my $commands  = $conf_hash->{'code'}->{$code_type}->{'command'};
     my $interface = $conf_hash->{'code'}->{$code_type}->{'interface'}; 

     tie my %test_case_num, "Tie::IxHash"; 	


	# Parsing is invoked first for the case where only "dfm" or "dfpm" is passed as the command.
	# In this case , the CPE provides information of all the current CDEFs under the corresponding command set.
	# The command list is then generated from the CPE provided information.

        #my $cdef_hash = $parse_obj->parse_file("file_path" => "/u/tanzim/CDEFNorm/","CommandName" => "dfm aggr","interface" => "CLI");
	my $cdef_hash = $parse_obj->parse_file("file_path" => $file_path , "CommandName" => $commands , "interface" => $interface);
	#print Dumper($cdef_hash);getc();

	my @command_list= $self->_get_command_list($commands,$cdef_hash);	
	# map s/\s+/_/ , @command_list;	
	#print "\nCOMMAND  LIST : @command_list\n";

        foreach my $cmd (@command_list){

		my ($success,$ut_fh,$file_name) = $self->create_test_file($cmd,$code_path,$interface);

		if ($success){

			my $command_hash = $cdef_hash->{$cmd};	

                	$self->gen_test_template("file_handle" => $ut_fh ,"interface" => $interface , "file_name" => $file_name );

			my ($pos_tc_num,$neg_tc_num) = $self->_gen_cli_test_cases("file_handle" => $ut_fh , "command_hash" => $command_hash , "interface" => $interface , "command" => $cmd, "file_name" => $file_name, "code_path" => $code_path);
			$test_case_num{$cmd}{'pos_num'} = $pos_tc_num;
			$test_case_num{$cmd}{'neg_num'} = $neg_tc_num; 		

			#$self->write_get_failure_type("file_handle" => $ut_fh);

			#$self->close_test_file($ut_fh);

		}
        }

	print "\nUnit Test Generation Summary\n";
	print "=============================\n";
	
	foreach my $hash_cmd(keys %test_case_num){
		print "\nCommand : $hash_cmd \n";
		print "------------------- \n";
		print "Positive Test Cases : $test_case_num{$hash_cmd}{'pos_num'}\n";
		print "Negative Test Cases : $test_case_num{$hash_cmd}{'neg_num'}\n";
		print "\nTotal Number of Test Cases : ".($test_case_num{$hash_cmd}{'pos_num'} + $test_case_num{$hash_cmd}{'neg_num'});
		print "\n";
	}
	
}


################################################################################
# Subroutine  : _gen_cli_test_cases                                            #
#                                                                              #
# Description : This method generates various categories of DFM CLI 	       #	 	
#		unit test cases 					       #	
#		Categories :						       #	
#			Positive					       #	
#			Negative					       #	   
#                                                                              #
# Returns     : Nothing                                                        #
#               Identifies the unit tests in each category to be generated     #
################################################################################


sub _gen_cli_test_cases {

	my ( $self, %args ) = @_;
	
	my $ut_fh = $args{"file_handle"};
        my $file_name = $args{"file_name"};
        my $code_path = $args{"code_path"};

	my ($global_variables_list);
	tie my %tc_list, "Tie::IxHash";
	my ($positive_tc_num,$negative_tc_num) = ("01","01");
	#my ($positive_tc_aref , $negative_tc_aref);

	my $options_map_href = $args{'command_hash'}->{'options_map'};

	# Generate Positive test cases
	foreach my $sub_command (keys %{$args{'command_hash'}}){

		next if ($sub_command =~ /Param Type|Primary Keys|counter_commands|options_map/i);

		#print "\nSub Command : $sub_command\n";
		#print "\nSub Command Hash\n".Dumper ($args{'command_hash'}->{$sub_command}); 
		my ($positive_tc_aref,$global_variables_positive) = $self->_positive_cli_test_cases($sub_command,$args{'command_hash'}->{$sub_command},$options_map_href);

		#print "\nSub Command Positive Test param set\n" .Dumper ($positive_tc_aref);
		#print "\nSub Command Positive Global Variable List\n" .Dumper ($global_variables_positive);

		push (@$global_variables_list,@$global_variables_positive) if (defined $global_variables_positive);

		#print "\nGlobal variable set(increment)\n" . Dumper ($global_variables_list);

		
		foreach my $param_set (@$positive_tc_aref){

			
			my $test_case_type_positive = delete $param_set->{'test_type'};
			my $test_case_description_positive = delete $param_set->{'test_description'};
			my $test_case_config_positive = delete $param_set->{'test_config'};
			my $test_case_name_positive = $sub_command."_positive_".$test_case_type_positive."_".$positive_tc_num;

			my $expected_output_positive = $self->_get_expected_output('command' => "$sub_command",'type' => "positive_".$test_case_type_positive);

			#push (@$tc_list,$test_case_name_positive);
			$tc_list{$test_case_name_positive} = $test_case_description_positive;	

			$self->gen_test_case_template('command' => "$sub_command", 'interface' => "CLI", 'testcase_name' => $test_case_name_positive,'testcase_description' => $test_case_description_positive,'parameter_set' => $param_set,'expected_output' => $expected_output_positive,'testcase_type' => "positive_".$test_case_type_positive,'file_handle' => $ut_fh,'config_status' => $test_case_config_positive);	
			
			$positive_tc_num++;	
		}
	}
	
	################################
        #1:Invalid command - raw command
        ################################
	
	# Generate negative test case for the base command
	unless($args{'command'} =~ /dfm\s+(version|help|about|detail|diag)/i){
		my $incomplete_base_cmd = $args{'command'};
		$args{'command'} =~ s/\s+/_/g;
		#$incomplete_base_cmd .= $self->_get_switch('letter');
		my $incomplete_base_cmd_tc_name = $args{'command'}."_negative_invalid_command_".$negative_tc_num;
		$self->gen_negative_test_raw_template('command' => $args{'command'},'testcase_name' =>$incomplete_base_cmd_tc_name ,'testcase_description' => "Incomplete base command\n# Command      : $incomplete_base_cmd",'raw_command' => $incomplete_base_cmd,'expected_output' => $self->_get_expected_output('command' => $args{'command'},'type' => "invalid_command"),'testcase_type' => "invalid_command",'file_handle' =>  $ut_fh,'config_status' => " Setup/Cleanup code is not required for this test case");
		$negative_tc_num++;
		#push (@$tc_list,$incomplete_base_cmd_tc_name);	
		$tc_list{$incomplete_base_cmd_tc_name} = "Incomplete base command";
	}

	# Generate negative test cases
	foreach my $sub_command (keys %{$args{'command_hash'}}){

                next if ($sub_command =~ /Param Type|Primary Keys|counter_commands|options_map/i);

		my ($negative_tc_aref,$global_variables_negative) = $self->_negative_cli_test_cases($sub_command,$args{'command_hash'}->{$sub_command},$options_map_href);
			
		#print "\nSub Command Negative Test param set\n" .Dumper ($negative_tc_aref);
                #print "\nSub Command Negative Global Variable List\n" .Dumper ($global_variables_negative);

                push (@$global_variables_list,@$global_variables_negative) if (defined $global_variables_negative);

                #print "\nGlobal variable set(increment)\n" . Dumper ($global_variables_list);

		
		foreach my $param_set (@$negative_tc_aref){

			my $test_case_type = delete $param_set->{'test_type'};
			my $test_case_description_negative = delete $param_set->{'test_description'};
			my $test_case_config_negative = delete $param_set->{'test_config'};
                        my $test_case_name_negative = $sub_command."_negative_".$test_case_type."_".$negative_tc_num;
			
			#print Dumper ($param_set);getc();
                        my $expected_output_negative = $self->_get_expected_output('command' => "$sub_command",'type' => "$test_case_type");

                        #push (@$tc_list,$test_case_name_negative);
			$tc_list{$test_case_name_negative} = $test_case_description_negative;				

			unless(exists $param_set->{'raw_cmd'}){

                        	$self->gen_negative_test_api_template('command' => "$sub_command",'testcase_name' => $test_case_name_negative,'testcase_description' => $test_case_description_negative,'parameter_set' => $param_set,'expected_output' => $expected_output_negative,'testcase_type' => "negative_".$test_case_type,'file_handle' => $ut_fh,'config_status' => $test_case_config_negative);
			}else{
				my $raw_cmd = delete $param_set->{'raw_cmd'};
				$self->gen_negative_test_raw_template('command' => "$sub_command",'testcase_name' => $test_case_name_negative,'testcase_description' => $test_case_description_negative,'parameter_set' => $param_set,'raw_command' => $raw_cmd,'expected_output' => $expected_output_negative,'testcase_type' => "negative_".$test_case_type,'file_handle' => $ut_fh,'config_status' => $test_case_config_negative);
				
			}

                        $negative_tc_num++;
                }
	}

	$self->_remove_duplicate_variables($global_variables_list);

	#print "\nTEST CASE LIST\n";
        #print Dumper (\%tc_list);
        #print "\nGLOBAL VARIABLE LIST\n";
        #print Dumper ($global_variables_list);

	my $new_ut_fh = $self->_insert_testcase_list(\%tc_list,$ut_fh,$code_path,$file_name);
	$self->write_get_failure_type("file_handle" => $new_ut_fh);
	close($new_ut_fh);

	return(--$positive_tc_num,--$negative_tc_num);
}

################################################################################
# Subroutine  : _positive_cli_test_cases                                       #
#                                                                              #
# Description : This method generates parameter combinations for  positive     #
#               unit test cases                                                #
#                                                                              #
# Returns     :  An array of hash references                                   #
#                Each array reference holds a combination of parameters        #
#		  and their corresponding variable values                      #
################################################################################

sub _positive_cli_test_cases {
	my $self = shift;
	my $command = shift;
	my $option_hash = shift;
	my $option_tx_hash = shift;

	my (%params_hash_all,%params_hash_true,%params_hash_false,%params_hash_none,%params_hash_help,%params_hash_sub_help,%params_hash_set);
	tie %params_hash_all, "Tie::IxHash";
	tie %params_hash_true, "Tie::IxHash";
	tie %params_hash_false, "Tie::IxHash";

	my $command_sep = undef;
	my ($variable_list,$options_aref); 

	%params_hash_none = ();	
	$command_sep = $option_hash->{'command_sep'} if (exists $option_hash->{'command_sep'});
	
	#my $options_aref;
	
	# IMP : need to return 
		# array of hash references
		# name of testcases generated
		# variable names

	# create option arrays based on command type ,parameter type  
	# "or" param types not handled,manual intervention to correct the test case 

	# create testcases for all options , none option , boolean true,boolean false
	#print Dumper ($option_hash);getc();

	foreach my $options(@{$option_hash->{'option_order'}}){
                        $params_hash_all{$options} = "\$$options";
			$params_hash_all{$options} = "\"1\"" if (grep {$_ eq $options} @{$option_hash->{'boolean_options'}});
        }
	
	%params_hash_true = %params_hash_all;
	%params_hash_false = %params_hash_all;

	if (scalar @{$option_hash->{'option_order'}} != 0){


	if (scalar @{$option_hash->{'boolean_options'}} > 0){
	
		foreach my $boolean_option( @{$option_hash->{'boolean_options'}}){
	
			if (exists $params_hash_true{$boolean_option}){
				$params_hash_true{$boolean_option} = "\"1\"";
				$params_hash_true{'test_description'} = "Testing command with single switch \'$option_tx_hash->{$boolean_option}\'\n";
			}	
		}
		my $cmd_str_true = $self->_command_translate($command,\%params_hash_true,$option_tx_hash,$command_sep);
		$params_hash_true{'test_type'} = "with_single_switch";
		$params_hash_true{'test_description'}.= "# Command      : $cmd_str_true" ;
		$params_hash_true{'test_config'}.= " Setup/Cleanup code to be added for this test case" ;
		
		push (@$options_aref,\%params_hash_true);
		
		foreach my $boolean_option( @{$option_hash->{'boolean_options'}}){

                        if (exists $params_hash_false{$boolean_option}){
                                $params_hash_false{$boolean_option} = "\"0\"";
				$params_hash_false{'test_description'} = "Testing command without single switch \'$option_tx_hash->{$boolean_option}\'\n";
                        }
                }
		my $cmd_str_false = $self->_command_translate($command,\%params_hash_false,$option_tx_hash,$command_sep);
		$params_hash_false{'test_type'} = "without_single_switch";
                $params_hash_false{'test_description'}.= "# Command      : $cmd_str_false" ;
		$params_hash_false{'test_config'}.= " Setup/Cleanup code to be added for this test case" ;
		push (@$options_aref,\%params_hash_false);

	}elsif(defined $command_sep){

        	tie my %required_params, "Tie::IxHash";
                tie my %param_hash_all_dup, "Tie::IxHash";
                %param_hash_all_dup = %params_hash_all;

                #Assuming that required options always come before the set options
                foreach my $req_param(@{$option_hash->{'required_options'}}){
                       $required_params{$req_param} = delete $param_hash_all_dup{$req_param};
                }
		foreach my $option_param(keys %param_hash_all_dup){
          	                tie my %params_hash_option, "Tie::IxHash";
                                %params_hash_option = %required_params;
                                $params_hash_option{$option_param} = delete $param_hash_all_dup{$option_param} if ($option_param eq $option_tx_hash->{$option_param});
                                my $cmd_str_opt = $self->_command_translate($command,\%params_hash_option,$option_tx_hash,$command_sep);
                                $params_hash_option{'test_type'} = "set_option";
                                $params_hash_option{'test_description'} = "Testing individual set option \'$option_param\'\n# Command      : $cmd_str_opt";
                                $params_hash_option{'test_config'}.= " Setup/Cleanup code to be added for this test case" ;

                                push (@$options_aref,\%params_hash_option);
                }

	}else{
		my $cmd_str_all = $self->_command_translate($command,\%params_hash_all,$option_tx_hash,$command_sep);
		$params_hash_all{'test_type'} = "all_options";
		$params_hash_all{'test_description'} = "Testing all options\n";
                $params_hash_all{'test_description'}.= "# Command      : $cmd_str_all" ;
		$params_hash_all{'test_config'}.= " Setup/Cleanup code to be added for this test case" ;
		push (@$options_aref,\%params_hash_all);
	}	

	}
	
	# if reqd options is not specified
		# method call with all options
		# method call without any option

	#if ((scalar @{$option_hash->{'required_options'}} == 0) && (scalar @{$option_hash->{'option_order'}} != 0)){
	if ((scalar @{$option_hash->{'required_options'}} == 0)){
		$command =~ s/_/ /g;
		$params_hash_none{'test_type'} = "base_command";
		$params_hash_none{'test_description'} = "Testing the base command only\n";
		$params_hash_none{'test_description'}.= "# Command      : $command " ;
		$params_hash_none{'test_config'}.= " Setup/Cleanup code is not required for this test case" ;
		push (@$options_aref,\%params_hash_none);

	}

	# Base command execution for "set" type commands which have required parameters
	if (defined $command_sep){
                $command =~ s/_/ /g;
                $params_hash_set{'test_type'} = "base_command";
                $params_hash_set{'test_description'} = "Testing the base command only\n";
                $params_hash_set{'test_description'}.= "# Command      : $command " ;
                $params_hash_set{'test_config'}.= " Setup/Cleanup code is not required for this test case" ;
                push (@$options_aref,\%params_hash_set);

        }

	# Base command help
	if($command =~ /help/i){

		$command =~ s/_/ /g;
		$params_hash_help{'test_type'} = "help_command";
                $params_hash_help{'test_description'} = "Testing the command help\n";
                $params_hash_help{'test_description'}.= "# Command      : $command" ;
                $params_hash_help{'test_config'}.= " Setup/Cleanup code is not required for this test case" ;
                push (@$options_aref,\%params_hash_help);

	}else{
	#Sub Command help
		$command =~ s/_/ /g;
		$command.= " help"; 
                $params_hash_sub_help{'test_type'} = "help_command";
                $params_hash_sub_help{'test_description'} = "Testing the sub-command help\n";
                $params_hash_sub_help{'test_description'}.= "# Command      : $command " ;
                $params_hash_sub_help{'test_config'}.= " Setup/Cleanup code is not required for this test case" ;
		$params_hash_sub_help{'help'} = "\"1\"";
                push (@$options_aref,\%params_hash_sub_help);		
	}
	
	foreach my $param_hash (@$options_aref){

                while (my ($param_key,$param_value)= (each %$param_hash)){

                        push (@$variable_list,$param_value) if (($param_key !~ /test_description|test_config|test_type/) && ($param_value !~ /true|false|help|1|0/i));
                }
        }

	
	$self->_remove_duplicate_variables($variable_list);

	return ($options_aref,$variable_list);

}

################################################################################
# Subroutine  : _negative_cli_test_cases                                       #
#                                                                              #
# Description : This method generates parameter combinations for negative      #
#               unit test cases                                                #
#                                                                              #
# Returns     :  An array of hash references                                   #
#                Each hash reference holds a combination of parameters         #
#                 and their corresponding variable values                      #
################################################################################

sub _negative_cli_test_cases {
        my $self = shift;
        my $command = shift;
        my $option_hash = shift;
        my $option_tx_hash = shift;

        #my (%params_hash_all,%params_hash_true,%params_hash_false,%params_hash_none);
        tie my %params_incorrect_cmd_str, "Tie::IxHash";
	#tie my %params_incorrect_cmd_spec, "Tie::IxHash";#All printable characters are considered valid for host names within DFM
	tie my %params_incorrect_switch, "Tie::IxHash";
	tie my %params_incorrect_order, "Tie::IxHash";
        tie my %params_invalid_name_str, "Tie::IxHash";
	#tie my %params_invalid_name_spec, "Tie::IxHash";#All printable characters are considered valid for host names within DFM
        tie my %params_missing, "Tie::IxHash";
	tie my %params_no_reqd_param, "Tie::IxHash";
	tie my %params_reqd_param_invalid, "Tie::IxHash";
	tie my %params_reqd_param_exist, "Tie::IxHash";
	tie my %params_reqd_param_nonexist, "Tie::IxHash";
	tie my %params_incorrect_command_sep, "Tie::IxHash";
	tie my %params_single_switch_val, "Tie::IxHash";

	tie my %params_hash_all, "Tie::IxHash";

	my ($variable_list,$options_aref);


	foreach my $options(@{$option_hash->{'option_order'}}){

		$params_hash_all{$options} = "\$$options";
		$params_hash_all{$options} = "\"1\"" if (grep {$_ eq $options} @{$option_hash->{'boolean_options'}});

	}

	my $command_sep = undef;
	$command_sep = $option_hash->{'command_sep'} if (exists $option_hash->{'command_sep'});	

	$command =~ s/_/ /g;

	################################
        #1:Invalid command - raw command
        ################################
        tie my %invalid_cmd_str, "Tie::IxHash";

        my $invalid_cmd = $command;
	trim($invalid_cmd);
        $invalid_cmd =~ s/\s+\w+$//;

        $invalid_cmd .= $self->_get_switch('letter');
        $invalid_cmd_str{'test_type'} = "invalid_command";
        $invalid_cmd_str{'raw_cmd'} = $invalid_cmd;
        $invalid_cmd_str{'test_description'} = "Incorrect sub command\n# Command      : $invalid_cmd";
        $invalid_cmd_str{'test_config'}.= " Setup/Cleanup code is not required for this test case" ;

        push (@$options_aref,\%invalid_cmd_str);


	###############################
	#2:Invalid switch - raw command
	###############################	
	
	# Note : Single switch or switch with value does not make a difference
	# Generating single switch commands
	
	my $cmd_invalid_switch = $command." -".$self->_get_switch('letter');
	$params_incorrect_switch{'test_type'} = "invalid_switch";
	$params_incorrect_switch{'raw_cmd'} = $cmd_invalid_switch;
	$params_incorrect_switch{'test_description'} = "Command with invalid switch\n# Command      : $cmd_invalid_switch";
	$params_incorrect_switch{'test_config'}.= " Setup/Cleanup code is not required for this test case" ;

        push (@$options_aref,\%params_incorrect_switch);

	if (scalar @{$option_hash->{'option_order'}} > 0){ 
		#######################################
		#3:Incorrect Option Order - raw command	
		#######################################

		 # With a single option , option order does not make a difference
	         # In most cases , option order makes a difference only when a required option is specified
		if ((scalar @{$option_hash->{'option_order'}} > 1) && ((scalar @{$option_hash->{'required_options'}} > 0)) && (!$self->_verify_params($option_hash->{'option_order'},$option_tx_hash))){ 
			
			# Reodering of only options without switches does not make a difference # TANZIM
			#next if ($self->_verify_params($option_hash->{'option_order'},$option_tx_hash));

	
			tie my %params_incorrect_order, "Tie::IxHash";
			my @params_array = @{$option_hash->{'option_order'}};
			#$self->_shuffle_array(\@params_array);	
			@params_array = reverse @params_array ;
			
			foreach my $param (@params_array){
				
                        	$params_incorrect_order{$param} = "\$$param";
				if (scalar @{$option_hash->{'boolean_options'}} > 0){

			                foreach my $boolean_option( @{$option_hash->{'boolean_options'}}){
						$params_incorrect_order{$boolean_option} = "\"1\"";
					}
				}

			}

			my $cmd_incorrect_order = $self->_command_translate($command,\%params_incorrect_order,$option_tx_hash,$command_sep);

	        	$params_incorrect_order{'test_type'} = "incorrect_order";
        		$params_incorrect_order{'raw_cmd'} = $cmd_incorrect_order;
        		$params_incorrect_order{'test_description'} = "Command with incorrect order\n# Command      : $cmd_incorrect_order";	
			$params_incorrect_order{'test_config'}.= " Setup/Cleanup code is not required for this test case" ;

			push (@$options_aref,\%params_incorrect_order);
		}

		 ################################################
                 #4:Invalid parameter value - APISet method call
                 #5:Missing switch parameter - APISet method call
                 ################################################
		foreach my $option(@{$option_hash->{'option_order'}}){
			if(($option_tx_hash->{$option} =~ /\-[a-zA-Z]/) && (!grep {$_ eq $option} @{$option_hash->{'boolean_options'}})){
				# For the correct order
				tie my %param_invalid_value, "Tie::IxHash" ; 
				tie my %param_missing_value, "Tie::IxHash" ;
				while ( my ($key,$val) = each (%params_hash_all)){

					#$param_invalid_value{$key} = $val if (($key eq $option) || (grep {$_ eq $key} @{$option_hash->{'required_options'}}));
					if($key eq $option){
						$param_invalid_value{$key} = $self->_get_random_name;
						$param_invalid_value{$key} = "\"$param_invalid_value{$key}\"";
						$param_missing_value{$key} = "\"\"";
					}elsif(grep {$_ eq $key} @{$option_hash->{'required_options'}}){
						$param_invalid_value{$key} = $val;	
						$param_missing_value{$key} = $val;
					}else{
					}
				}
				 my $cmd_invalid_value = $self->_command_translate($command,\%param_invalid_value,$option_tx_hash,$command_sep);
				 my $cmd_missing_value = $self->_command_translate($command,\%param_missing_value,$option_tx_hash,$command_sep); 

				$param_invalid_value{'test_type'}= "invalid_switch_value";
	        	        $param_invalid_value{'test_description'}= "Command with invalid parameter value for a switch \'$option_tx_hash->{$option}\'\n# Command      : $cmd_invalid_value" ;
				$param_invalid_value{'test_config'}.= " Setup/Cleanup code maybe required for this test case.This may require modification of default values assigned to the scalar variables" ;

				$param_missing_value{'test_type'}= "invalid_missing_switch_value";
				$param_missing_value{'test_description'}= "Command with missing parameter value for a switch \'$option_tx_hash->{$option}\'\n# Command      : $cmd_missing_value" ;
				$param_missing_value{'test_config'}.= " Setup/Cleanup code is not required for this test case" ;
				#print Dumper (\%param_invalid_value);getc();
        	        	push (@$options_aref,\%param_invalid_value);
				push (@$options_aref,\%param_missing_value);

			}	
		}

		# Negative test cases for required parameters
		if(scalar @{$option_hash->{'required_options'}} > 0){
			
			tie my %param_reqd_param_invalid, "Tie::IxHash"; # Object does not exist
                        tie my %param_reqd_param_exist, "Tie::IxHash";   # Object exists
	
			%param_reqd_param_invalid = %param_reqd_param_exist = %params_hash_all;

			my %params_hash_none = ();

			##################################################
                        #6:Missing Required Parameter - APISet method call
                        ##################################################
		
			unless(defined $command_sep){
				$params_hash_none{'test_type'} = "missing_required_parameter";
		                $params_hash_none{'test_description'} = "Command with missing required parameters \n# Command      : $command ";
        			$params_hash_none{'test_config'}.= " Setup/Cleanup code is not required for this test case" ;
	        	       	push (@$options_aref,\%params_hash_none);
			}

			foreach my $req_param(@{$option_hash->{'required_options'}}){
			
				tie my %param_no_reqd_param, "Tie::IxHash";
			
				%param_no_reqd_param = %params_hash_all; 
				delete $param_no_reqd_param{$req_param};
			
				# Test case will not be negative in case of "create" command as an object of any name will successfully be created
				unless($command =~ /create/i){
					$param_reqd_param_invalid{$req_param} = $self->_get_random_name() ;	
					$param_reqd_param_invalid{$req_param} = "\"$param_reqd_param_invalid{$req_param}\"";
				}
				##################################################
 	                        #6:Missing Required Parameter - APISet method call
        	                ##################################################
				unless((defined $command_sep) && ((scalar keys %param_no_reqd_param) == 0)){

					my $cmd_no_reqd_param = $self->_command_translate($command,\%param_no_reqd_param,$option_tx_hash,$command_sep);
					$param_no_reqd_param{'test_type'} = "missing_required_parameter";
					$param_no_reqd_param{'test_description'} = "Command with missing required parameter \'$req_param\'\n# Command      : $cmd_no_reqd_param ";
					$param_no_reqd_param{'test_config'}.= " Setup/Cleanup code maybe required for this test case.This may require modification of default values assigned to the scalar variables" ;
					push (@$options_aref,\%param_no_reqd_param);
				}

			}

			 ##################################################
                         #7:Invalid Required Parameter - APISet method call
                         ##################################################
	
			 # Test case will not be negative in case of "create" command as an object of any name will successfully be created	
			 unless ($command =~ /create/i){
				 my $cmd_invalid_reqd_param = $self->_command_translate($command,\%param_reqd_param_invalid,$option_tx_hash,$command_sep);
        	                 $param_reqd_param_invalid{'test_type'} = "invalid_required_parameter";
				$param_reqd_param_invalid{'test_description'} = "Command with invalid value for required parameter \n# Command      : $cmd_invalid_reqd_param ";	
				 $param_reqd_param_invalid{'test_config'}.= " Setup/Cleanup code maybe required for this test case.This may require modification of default values assigned to the scalar variables" ;
				 push (@$options_aref,\%param_reqd_param_invalid);
			 }
			
	                 ###################################################
                         #8:Existing Required Parameter - APISet method call
                         ###################################################

			 # The negative scenario where the object already exists is applicable to commands like create,add,enable		
			 if($command =~ /create|add|enable/i){
		                 my $cmd_object_exists = $self->_command_translate($command,\%param_reqd_param_exist,$option_tx_hash,$command_sep);
        	                 $param_reqd_param_exist{'test_type'} = "required_parameter_object_exists";
                	         $param_reqd_param_exist{'test_description'} = "Command with required parameter already existing\n# Command      : $cmd_object_exists ";
 				 $param_reqd_param_exist{'test_config'}.= " Setup/Cleanup code to be added for this test case" ;
				 push (@$options_aref,\%param_reqd_param_exist);
			}
		}

		if(defined $command_sep ){
			#############################################
			#9:Incorrect option format - raw command
			#10:Invalid option value - APISet method call
			#############################################
		
			tie my %required_params, "Tie::IxHash";
			tie my %param_hash_all_copy, "Tie::IxHash";
			tie my %param_hash_all_dup, "Tie::IxHash";
			%param_hash_all_copy = %params_hash_all;
			%param_hash_all_dup = %params_hash_all;
			my $new_command_sep = "-";		

			#Assuming that required options always come before the set options
			foreach my $req_param(@{$option_hash->{'required_options'}}){

				$required_params{$req_param} = delete $param_hash_all_copy{$req_param};
			}

			tie my %param_incorrect_command_sep, "Tie::IxHash";
                        %param_incorrect_command_sep = %required_params;	

			foreach my $option_param(keys %param_hash_all_copy){
				tie my %param_options_invalid, "Tie::IxHash";
				%param_options_invalid = %required_params;

				$param_incorrect_command_sep{$option_param} = delete $param_hash_all_copy{$option_param} if ($option_param eq $option_tx_hash->{$option_param});

				$param_options_invalid{$option_param} = delete $param_hash_all_dup{$option_param} if ($option_param eq $option_tx_hash->{$option_param});
				$param_options_invalid{$option_param} = $self->_get_random_name() ;
				$param_options_invalid{$option_param} = "\"$param_options_invalid{$option_param}\"";

				my $cmd_invalid_opt = $self->_command_translate($command,\%param_options_invalid,$option_tx_hash,$command_sep);
	                        $param_options_invalid{'test_type'} = "invalid_optional_parameter";
                	        $param_options_invalid{'test_description'} = "Command with invalid value for option \'$option_param\'\n# Command      : $cmd_invalid_opt";
                        	$param_options_invalid{'test_config'}.= " Setup/Cleanup code to be added for this test case.Dummy values for following variables to be replaced with real existing entity names : " ;
				foreach my $init_optional_param(values %required_params){
                                	$param_options_invalid{'test_config'}.= "\n\t#\t\t$init_optional_param ";
                        	}
			
				push (@$options_aref,\%param_options_invalid);
							
		        } 		
			my $cmd_incorrect_sep = $self->_command_translate($command,\%param_incorrect_command_sep,$option_tx_hash,$new_command_sep);
                        $param_incorrect_command_sep{'test_type'} = "incorrect_option_sep";
       	                $param_incorrect_command_sep{'raw_cmd'} = $cmd_incorrect_sep;
               	        $param_incorrect_command_sep{'test_description'} = "Command with incorrect option separator\n# Command      : $cmd_incorrect_sep";
			$param_incorrect_command_sep{'test_config'}.= " Setup/Cleanup code to be added for this test case.Dummy values for following variables to be replaced with real existing entity names : " ;
			
			foreach my $init_required_param(values %required_params){
				$param_incorrect_command_sep{'test_config'}.= "\n\t#\t\t$init_required_param ";

			}

                       	push (@$options_aref,\%param_incorrect_command_sep);	


		}

	
		if (scalar @{$option_hash->{'boolean_options'}} > 0){
			#################################################
                        #11:Assign a value to single switch - raw command
                        #################################################
			
			foreach my $option(@{$option_hash->{'option_order'}}){	
				if(($option_tx_hash->{$option} =~ /\-[a-zA-Z]/) && (grep {$_ eq $option} @{$option_hash->{'boolean_options'}})){

                	                # For the correct order
                        	        tie my %param_single_switch_value, "Tie::IxHash" ;
                                	while ( my ($key,$val) = each (%params_hash_all)){

        	                                if($key eq $option){
                	                                $param_single_switch_value{$key} = $self->_get_random_name();
                                	        #}elsif(grep {$_ eq $key} @{$option_hash->{'required_options'}}){
                                        	#        $param_single_switch_value{$key} = $val;
	                                        }else{
							$param_single_switch_value{$key} = $val;
        	                                }
                	                }
                        	         my $cmd_single_switch_value = $self->_command_translate($command,\%param_single_switch_value,$option_tx_hash,$command_sep);

	                                $param_single_switch_value{'test_type'}= "invalid_single_switch_value";
					$param_single_switch_value{'raw_cmd'} = $cmd_single_switch_value;
        	                        $param_single_switch_value{'test_description'}= "Command with invalid parameter value for a single switch \'$option_tx_hash->{$option}\'\n# Command      : $cmd_single_switch_value" ;
					$param_single_switch_value{'test_config'}.= " Setup/Cleanup code is not required for this test case" ;
                                	push (@$options_aref,\%param_single_switch_value);
				}
			}
		}
	}

	foreach my $param_hash (@$options_aref){

                while (my ($param_key,$param_value)= (each %$param_hash)){

                        push (@$variable_list,$param_value) if (($param_key !~ /test_description|test_config|test_type|raw_cmd/) && ($param_value =~ /^\$.*/i));
                }
        }
	$self->_remove_duplicate_variables($variable_list);

	#print "\nNegative hash for command $command\n". Dumper($options_aref);
	#print Dumper($options_aref);print Dumper ($variable_list);

	return ($options_aref,$variable_list);

}


################################################################################
# Subroutine  : _get_expected_output                                           #
#                                                                              #
# Description : This method executes a command and returns the command output  #
#                                                                              #
#                                                                              #
# Returns     :  Scalar (command output)                                       #
#                This may be either the entire raw output or a regexp          #
#                 					                       #
################################################################################

sub _get_expected_output{

	my ($self,%args) = @_;
	#my $command_output = "\n\t#>>>>>>>>>> USER NOTE: EXPECTED OUTPUT TBD \n";

    # Extract the command and the type
    my $command = $args{"command"};
    my $type    = $args{"type"};

    # The regex to be returened
    my $output_pattern;

    if ( $type =~ /positive/ ) {

	if($type =~ /help/){
		
             # Help command output
             $output_pattern = '(NAME.*SYNOPSIS.*DESCRIPTION)';	
		
	}else{	
	        if ( $command =~ /list|view/i ) {
        	    # Words, followed by lines with hyphens, followed by any content
	            #$output_pattern = '((\w\s)+(-+\s)+.*)|(There are no)';
		     $output_pattern = '((\w+\s+)+(-+\s+)+.*)|(There\s+(are|is)\s+no)';	

	        } elsif ( $command =~ /detail|get|summary/i ) {

        	    # Key value pairs seperated by colon or = sign
	            $output_pattern = '(.+\s*[=|:]\s*.*\s*)+';

	        } elsif ( $command
        	    =~ /(add|delete|destroy|discover|create|modify|pull|import)/i )
	        {

        	    # For 'modify',discover change $operation
	            my $operation = $1;
	            $operation = "modif"      if ( $operation =~ /modify/ );
	            $operation = "refreshing" if ( $operation =~ /discover/ );
	            $output_pattern = ".*$operation.*";

        	} elsif ( $command =~ /set/i ) {

	            # List of options and description, when no option is provide to set
        	    # Staement saying options changed/unchanged depending on the value
	            $output_pattern = '((\w+\s+.*)*)|(changed.*)';
		}else {

	            # Generic pattern
        	    $output_pattern = '.*';
        	}
	}

    } elsif ( $type =~ /^invalid_command$/ ) {

        # Specify an invalid string with the base command
        $output_pattern = '(Error: Unknown command: )|(Error: The .* command requires a sub-command)';

    } elsif ( $type =~ /^invalid_switch$/ ) {

        # An incorrect switch to the command
        $output_pattern = 'Error: Illegal option';

    } elsif ( $type =~ /^incorrect_order$/ ) {

        # Reverse option order provided
        $output_pattern
            = '(Error: There is no.*named .*)|(The.*?command\stakes.*?argument(|s);\syou\sgave.*)';

    } elsif ( $type =~ /^invalid_switch_value$|^invalid_missing_switch_value$/ ) {

# Provide an invalid parameter value or no parameter value to a valid switch (non-single switch)
        $output_pattern
            = '(The.*?command\stakes.*?argument(|s);\syou\sgave.*)|(Option.*?requires\sa\svalue.*)|(Error: .*option requires an argument)|(Error:.*is not a valid.*)|(Error:.*is not a number.*)';

    }

    elsif ( $type =~ /^missing_required_parameter$/ ) {

        # Do not provide a required parameter to the command
        $output_pattern
            = '(Error: Required .* option is missing.)|(command\stakes.*?argument(|s);\syou\sgave.*)|(Error: There is no.*named.*)|(Valid options are.*)';

    } elsif ( $type =~ /^invalid_required_parameter$/ ) {

        # Provide a random (invalid) string for a required parameter
        $output_pattern
            = '(Error:.*name not allowed)|(.*already\sexists.*)|(Error: There is no.*named.*)|(command takes .*argument(|s); you gave)|(Error: .*option requires an argument)|(Error: Could not resolve host.*)';

    } elsif ( $type =~ /^required_parameter_object_exists$/ ) {

        # Provide a parameter name that already exists
        $output_pattern = '.*already\sexists.*';

    } elsif ( $type =~ /^incorrect_option_sep$/ ) {

        # Provide an incorrect option separator for set commands
        $output_pattern = 'Error: Option.*requires a value.*';

    } elsif ( $type =~ /^invalid_optional_parameter$/) {

	# Provide an incorrect value for command sep options
	$output_pattern = 'Error:.*is not a [number|valid interval].*';

    } elsif ( $type =~ /^invalid_single_switch_value$/ ) {

        # Provide a value for single switches
        # Works when tehre's no option after the boolean option or when the option that follows it has a switch
        $output_pattern = '(command takes (no|one) argument(|s); you gave)|(Error: There is no object named.*)|(Error: Could not resolve host.*)';

    }
    return $output_pattern;

}

################################################################################
# Subroutine  : _get_command_list                                              #
#                                                                              #
# Description : This method generates a user specified command list            #
#                                                                              #
#                                                                              #
# Returns     :  Array (command list)                                          #
#                                                                              #
################################################################################

sub _get_command_list{

	my $self = shift;
	my $commands = shift;
	my $cdef_hash = shift;

 	# Populate user specified config list
        #my @command_list = split (/,/,$conf_hash->{'code'}->{$code_type}->{'command'});
        #my @command_list = split (/,/,$commands);
	
	# Reading the command list from the CPE data structure and not the user specified command list as the CPE data structure
	# returns all the sub commands under base commands also. Hence , splitting CDEF files on the basis of components will be
	# transparent to the end user as unit test cases for all the commands under the base command will be generated.	
	my @command_list = keys %$cdef_hash;

	#print "\n@command_list\n";getc();

        # Trim white spaces
        @command_list = map  trim() ,@command_list;
	
        # Populating command list array with available commands from CDEF files in case only "dfm" or "dfpm" is specified

        if ((scalar @command_list == 1) && ($command_list[0] =~ /^(dfm|dfpm)$/i)){

                foreach my $out_key (keys %$cdef_hash){

                        #foreach my $in_key (keys %{$cdef_hash->{$out_key}}){
                                push ( @command_list , $out_key ) ;
                        #}
                }
                # Ejecting the single command "dfm" or "dfpm"
                 shift @command_list;
        }

	return @command_list;
}


################################################################################
# Subroutine  : _command_translate                                             #
#                                                                              #
# Description : This method constructs a systematic command from programmatic  #
#               parameters                                                     #
#									       #					
# Returns     : The command string                                             #
#                                                                              #
################################################################################

sub _command_translate{

	my $self = shift;
	my $cmd = shift;
	my $param_href = shift;
	my $param_tx_href = shift;
	my $command_sep = shift;

	$cmd =~ s/_/ /g;
	#print "\nParam HREF\n".Dumper ($param_href);
	#print "\nParam TX HREF\n".Dumper ($param_tx_href);

	my $command_string = $cmd; 
	#print "\n$command_string\n";

	 # if value exists
	 # if value is true or false

	while(my ($program_param,$sys_param) = each (%$param_href)){
		
		next if ($program_param eq "test_description");
	#	if($sys_param){
			$sys_param =~ s/-/_/g;
		
			if((defined $command_sep) && ($program_param eq $param_tx_href->{$program_param})){
				$command_string .= " $param_tx_href->{$program_param} $command_sep $sys_param";
				next;
			}

			if ($sys_param =~ /true|1/i){
				#print "\nIN TRUE LOOP\n";
				$command_string .= " $param_tx_href->{$program_param} " ;		

			}else{
				#print "\nIN NON_TRUE LOOP\n";
				#print "\n$param_tx_href->{$program_param} :$program_param:$sys_param\n";
				$command_string .= " $param_tx_href->{$program_param} $sys_param"  unless ($sys_param =~ /false|0/);
			}

	#	}else{
	#		print "\nIF ELSE LOOP\n";
	#		$command_string .= "  $program_param";  
	#	}

	}

	#print "\nSTRING : $command_string\n";
	return $command_string;
}


################################################################################
# Subroutine  : _verify_params                                                 #
#                                                                              #
# Description : This method verifies the parameter type of all elements in the #
#               array							       #	
#                                                                              #
# Returns     : True , if all elements are single non-switch parameters        #
#               False, if otherwise                                            #
#                                                                              #
################################################################################


sub _verify_params{

	my $self = shift;
	my $param_aref = shift;
	my $param_tx_href = shift;

	my $status = 1;

	foreach my $param (@$param_aref){
		$status = 0 unless ($param_tx_href->{$param} eq "");

	} 

	return $status;
}


################################################################################
# Subroutine  : _insert_testcase_list                                          #
#                                                                              #
# Description : This method  inserts the test case list into the unit test file#
#                                                                              #
#                                                                              #
# Returns     : Nothing							       #
#                                                                              #
################################################################################


sub _insert_testcase_list {

	my $self = shift;
	my $tc_list = shift;
	my $ut_fh = shift;
	my $code_path = shift;
	my $file_name = shift;
	
	my $new_ut_fh;
        # Insert the test case list into thpl file

        # Close and reopen in read mode
        close( $ut_fh );

        open( FH, "< $code_path/$file_name");
        my @file_contents = <FH>;
         close(FH);

        open($new_ut_fh, "> $code_path/$file_name");
	#my $new_fh_ut = IO::File->new("+>> $code_path/$file_name");

        my $len1= @file_contents - 1;
        #my $len2 = @$tc_list;
        my $len2 = scalar(keys %$tc_list);

        foreach( my $i =0; $i< @file_contents; $i++) {

                if( $file_contents[$i] =~ /_NG_TESTCASE_INFO/) {

                        for( my $j= $len1 ; $j >= $i ; $j-- ){

                                $file_contents[$j+$len2-1] = $file_contents[$j];
                        }
                        $len1 += $len2;
                        while(my ($name,$description) = each(%$tc_list)){
				
				$description =~ s/#.*//;
				$description =~ s/"//g;
		                chomp($description);
                		$tc_list->{$name} = $description;	
                                $file_contents[$i] = "$name =>\"$description \", ";
                                $i++;
                        }
                }
        }
        print $new_ut_fh @file_contents;
	#print $new_fh_ut @file_contents;
        return $new_ut_fh;
	#$self->close_test_file($new_fh_ut);

}


################################################################################
# Subroutine  : _get_random_name                                               #
#                                                                              #
# Description : This method returns a random string                            #
#                                                                              #
# Returns     : returns a random string comprising of alphabets , numbers and  # 
#		underscores					               #
#                                                                              #
################################################################################


sub _get_random_name{

        my $self = shift;

	my @alphabet = ('a'..'z', 'A'..'Z');
	my @number   = (0..999);

	my $random_string_1 = join '',map $alphabet[rand @alphabet], 0..7; 
	my $random_string_2 = join '',map $alphabet[rand @alphabet], 0..7;
	my $random_number   = join '',map $number[rand @number], 0..3;

	
	return $random_string_1."_".$random_string_2."_".$random_number;

}

################################################################################
# Subroutine  : _get_switch                                                    #
#                                                                              #
# Description : This method returns either a string of letters or numbers      #
#                                                                              #
# Returns     : returns a string(size=2) of letters or numbers                 #
#                                                                              #
################################################################################


sub _get_switch{

	my $self = shift;
	my $type = shift;#type = letter , number

	#@alphabet contains double letter strings so that they do not co-incide with valid single letter switches of a command
	my @alphabet = ('aa'.."zz");
	my @number   = (1..99);

    my $randomswitch = $alphabet[int(rand(@alphabet))];

	return  ($type eq "letter") ? $alphabet[int(rand(@alphabet))] : $alphabet[int(rand(@number))];

}


################################################################################
# Subroutine  : _shuffle_array                                                 #
#                                                                              #
# Description : This method returns the randomized source array                #
#                                                                              #
# Returns     : Randomized source array                                        #
#                                                                              #
################################################################################

sub _shuffle_array{

    my $self = shift;	
    my $param_array = shift;
    my $i;
    for ($i = @$param_array; --$i; ) {
        my $j = int rand ($i+1);
        next if $i == $j;
        @$param_array[$i,$j] = @$param_array[$j,$i];
    }
	
   #return @$param_array
}

################################################################################
# Subroutine  : _cmp_array                                                     #
#                                                                              #
# Description : This method compares two arrays                                #
#                                                                              #
# Returns     : True, if the arrays are equal                                  #
#               False,if arrays are not equal                                  #
################################################################################

sub _cmp_arrays{

	my $self = shift;
	my $array1 = shift;
	my $array2 = shift;

	my $status = 1;

	if (scalar @$array1 == scalar @$array2){
	
	foreach my $element (@$array1){	
		
		unless (grep {$_ eq $element} @$array2){

			$status =0;
			last;
		}
	}

	}else{

		$status =0;
	}

	return $status;

}

sub _remove_duplicate_variables{

	my $self =shift;
	my $variable_list = shift;
	my %temp_sort_hash   = map { $_, 1 } @$variable_list;
        @$variable_list = keys %temp_sort_hash;

}

################################################################################
# Subroutine  : trim                                                           #
#                                                                              #
# Description : This method trims a string of spaces                           #
#                                                                              #
# Returns     : Nothing                                                        #
#                				                               #
################################################################################


sub trim {
   local $_ = @_ ? $_[0] : $_;
   s/^\s+//g;
   s/\s+$//g;
   $_
}




1;

