# File Name     :       CDefGen.pm
#
# Summary       :       The package generates .cdefs file for the specified command(s) or HelpFile .
#
# Creation Date :       20th October 2010
#
# Author        :       ramyar@netapp.com, dl-sm-nacl-dev@netapp.com
#
# Version History
# ---------------
#
#  Version 1
#       Summary : 
#       Date    : 
#       Author  : 
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#

#
# Package Declaration
#
package CodeModules::CdefGen;


#
# Compiler Directives
#
use strict;
use warnings;

#
# Perl Modules
#
use Data::Dumper;
#

# User Specific Modules
#
use CommonLibs::DFMCLIparser;
use CommonLibs::ConfigParse;
use CommonLibs::Utils;

# Constructor Definition
#
sub new {
    my $class = shift;
    my $self  = {};
    bless $self, $class;
    return $self;
}

################################################################################
# Subroutine  : cdefgenerator                                                  #
#                                                                              #
# Description : This method generates CDEF files                               #
#                                                                              #
# Input       : Takes the input from the config file where the name of the     #
#				commands should be specified in a comma separated list or the  #
#				path of the helpfile should be provided                        #
#                                                                              #
# Returns     : Nothing                                                        #
#                                                                              #
################################################################################

sub cdefgenerator {

    my $self      = shift;
    my $conf_hash = shift;
    my $code_path = $conf_hash->{'code_path'};
    my $code_type = $conf_hash->{'code_type'};
    my $interface = $conf_hash->{'code'}->{$code_type}->{'interface'};
    my $dfmParser = CommonLibs::DFMCLIparser->new();
    if ( $interface eq "CLI" ) {
        my $filename_cmd = $conf_hash->{'code'}->{'cdef'}->{'fileName_cmd'};

        my ($hashreference) = $dfmParser->parseDfmCLI(
            "commandname-or-path" => $filename_cmd,
            "interface"           => $interface
        );
        #print "HASHHH" . Dumper($hashreference);
        my %hash = %$hashreference;
		my %singswitchhash;
        while ( my ( $hashkey, $hashref ) = each %hash ) {
            my $file = $hashkey;
            $file =~ s/ /_/g;
			my $command = $file."_help";
            $file = $file . ".cdefs";
            open my $input_fh, "> $code_path/$file"
                or die "COULD not open '$file'-$!";
            print $input_fh <<"FOO" or warn "\n\tWARNING : $!\n";
# \$Id 
#
# -*- perl -*-
#
# Filename: $file
#
## \@cdefmap
## $command::Get
FOO

            #sub command level
            my %hash = ();
            my $cmd_sep;
            while ( my ( $k, $v ) = each(%$hashref) ) {
                my @option_array = ();
                my @optionnames  = ();
                while ( my ( $key, $value ) = each(%$v) ) {
                    if ( $key eq "option_order" ) {
                        push( @option_array, @$value );
                    }
                    if ( $key eq "option_names" ) {
                        push( @optionnames, @$value );
                    }
                }
                $k =~ s/^\s+//smg;
                $k =~ s/\s+$//smg;
				my $cmdname = $k;
				$cmdname =~ s/ /_/g;
				my $options = join (',', @option_array,@optionnames);
                print $input_fh <<"FOO" or warn "\n\tWARNING : $!\n";
## $cmdname:$options
FOO
            }
            print $input_fh <<"FOO" or warn "\n\tWARNING : $!\n";
#
## \@parametermap
FOO
            while ( my ( $k, $v ) = each(%$hashref) ) {
                my @option_array = ();
                if ( ref($v) eq 'HASH' ) {
                    while ( my ( $key, $value ) = each(%$v) ) {
                        unless (
                            ref($value) eq 'ARRAY' | ref($value) eq 'HASH' )
                        {
						if ($key ne "cmd_sep") {
                            print $input_fh
                                <<"FOO" or warn "\n\tWARNING : $!\n";
## $value:$key:-
FOO
        }                } elsif ( ref($value) eq 'HASH' ) {
                            while ( my ( $key1, $value1 ) = each(%$value) ) {
                                print $input_fh
                                    <<"FOO" or warn "\n\tWARNING : $!\n";
## $value1:$key1:Boolean
FOO
                            }

                        }
						elsif ((ref($value) eq 'ARRAY') && ($key =~ /(-[a-zA-Z])/)) {
						$singswitchhash{$key."switch"} = $key;
						}
								 
						
                    }
                }
            }
			  while ( my ( $key1, $value1 ) = each(%singswitchhash) ) {
                                print $input_fh
                                    <<"FOO" or warn "\n\tWARNING : $!\n";
## $value1:$key1:Boolean
FOO
}
            print $input_fh <<"FOO" or warn "\n\tWARNING : $!\n";
## help:help:Boolean
FOO
            print $input_fh <<"FOO" or warn "\n\tWARNING : $!\n";
#
## \@type
## library(NACL)
#
# Copyright (c) 2010 NetApp as an unpublished work.
#
# All rights reserved.
# \@author sm-nacl-dev (dl-sm-nacl-dev\@netapp.com)
#
FOO
            print $input_fh <<"FOO" or warn "\n\tWARNING : $!\n";
my %Keys = ();
my %Options = (
FOO
            my %optionmap;
			my ( @array, @switcharray, @optionnamesarray );
            while ( my ( $k, $v ) = each(%$hashref) ) {    #foreach subcommand

                #type of options
                while ( my ( $k1, $v1 ) = each(%$v) ) {
                    if ( ref($v1) eq 'ARRAY' ) {
                         
						if ( $k1 eq "option_names" ) {

                            @optionnamesarray = @$v1;

                        } elsif ( $k1 eq "mandatory" ) {
                            push( @array, @$v1 );
                        }

                        elsif ( $k1 eq "alternate_param_options" ) {
                            foreach (@$v1) {
                                if ( ref($_) eq 'HASH' ) {
                                    my %altparhash = %$_;
                                    while ( my ( $key, $value )
                                        = each(%altparhash) )
                                    {
                                        if ( ref($value) eq 'ARRAY' ) {
										 $optionmap {join "-", @$value} = $key ;
                                            # $optionmap{$key} = join "-", @$value;
                                        } else {
                                            $optionmap{$key} = $value;
                                        }
                                    }

                                } else {
                                    push( @array, @$v1 );
                                }
                            }
                        } elsif ( $k1 eq "option_order" ) {
                            next;
                        } elsif ( $k1 eq "alternate_switch_options" ) {
                            next;
                        } elsif ( $k1 eq "all_options" ) {
                            push ( @array, @$v1 );
                        } else {
						my $sinswitch = $k1."switch";
							$optionmap{$sinswitch} = $k1;
                            push( @switcharray, $sinswitch );
                        }
                    }
						elsif (ref($v1) eq 'HASH') {
						 if ( $k1 eq "single_switches" ) {
                            my %switchhash = %$v1;
                            while ( my ( $k2, $v2 ) = each(%switchhash) ) {
                                $optionmap{$v2} = $k2;
                                push( @switcharray, $v2 );
                            }

                        } 
						}
					
					unless (
                            ref($v1) eq 'ARRAY' | ref($v1) eq 'HASH') 
                         {
                       if ($k1 ne "cmd_sep") {
                        $optionmap{$v1} = $k1;
						}
                    }

                }
            }
            my %optionnames = map { $_ => "" } @optionnamesarray;

            push( @switcharray, "help" );
            my %singleswitchhash = map { $_ => "" } @switcharray;
            my @unique           = keys %singleswitchhash;
			push (@unique, keys %singswitchhash);
            my %switchhash       = map { $_ => "" } @array;
            foreach my $arrkey ( keys %optionnames ) {
                next if defined( $optionmap{$arrkey} );
                $optionmap{$arrkey} = "$arrkey";
            }
            foreach my $arrkey ( keys %switchhash ) {
                next if defined( $optionmap{$arrkey} );
                $optionmap{$arrkey} = "";
            }
            $optionmap{"help"} = "help";
            foreach my $arrkey ( keys %optionmap ) {
                print $input_fh <<"FOO" or warn "\n\tWARNING : $!\n";				
	"$arrkey" => "$optionmap{$arrkey}",
FOO
            }
            print $input_fh <<"FOO" or warn "\n\tWARNING : $!\n";
);

\@Boolean_Options = (qw( @unique ) );

%CommandDefinitions = (
	$command => {
		command          => "$hashkey help",
		options          => \\\%Options,
        keys             => \\\%Keys,
        required_options => [qw( )],
        option_order     => [qw( )],
    },
	
FOO
            while ( my ( $k, $v ) = each(%$hashref) ) {
                my @reqdoptionarray  = ();
                my @optionorder      = ();
                my @optionnamesarray = ();
				my $command_sep;
				my $flag = 0 ;
				$k =~ s/^\s+//smg;
                $k =~ s/\s+$//smg;
                my $newk             = $k;
                $newk =~ s/^\s+//smg;
                $newk =~ s/\s+$//smg;
                $newk =~ s/ /_/g;
                if ( ref($v) eq 'HASH' ) {

                    while ( my ( $key, $value ) = each(%$v) ) {
                        if ( $key eq "mandatory" ) {
                            @reqdoptionarray = @$value;
                        }
                        if ( $key eq "option_order" ) {
                            push( @optionorder, @$value );
                        }
                        if ( $key eq "option_names" ) {
                            push( @optionnamesarray, @$value );
                        }
						if ( $key eq "cmd_sep") {
						$command_sep = $value;
						$flag = 1;
						}
                    }
		if ($flag == 1) {
		print $input_fh <<"FOO" or warn "\n\tWARNING : $!\n";
		##Command Separator needs to be added
		## command_sep      => "="
FOO
		}
                    print $input_fh <<"FOO" or warn "\n\tWARNING : $!\n";
	$newk  => {
		command          => "$k",
		options          => \\\%Options,
        keys             => \\\%Keys,
        required_options => [qw( @reqdoptionarray )],
        option_order     => [qw( help @optionorder @optionnamesarray)],
    },
FOO

                }

            }
            print $input_fh <<"FOO" or warn "\n\tWARNING : $!\n";
);
FOO
            print "CONF HASH CODE PATH: $conf_hash->{'code_path'}";
            my $perl_tidy = CommonLibs::Utils->new();
            $perl_tidy->run_perltidy_cdefs( $conf_hash->{'code_path'},
                $file );

        }
    } else {
        $dfmParser->parseDfmZAPI();
    }
}
1;
