# File Name     :       UnitTestGen.pm
#
# Summary       :       Generates DFM unit test codes using NACL APISet calls .
#		:	The generated unit test code resides in a .thpl file .
#
# Creation Date :       4th August 2010
#
# Author        :       tanzim , dl-sm-nacl-dev@netapp.com
#
# Version History
# ---------------
#
#  Version 1
#       Summary: 
#       Date   : 
#       Author :  
#
# Copyright (c) 2010 NetApp, Inc.
# All rights reserved.
#

#
# Package Declaration
#

package CodeModules::UnitTestGen;

#
# Compiler Directives
#

use strict;
use warnings;

#
# Class Inheritance
#

#use base qw(CodeModules::TestTemplateGen);


#
# Perl Modules
#

use Data::Dumper;
use Cwd;
use IO::File;

#
# User Specific Modules
#
use CodeModules::CLI::UnitTestCLIGen;
use CodeModules::ZAPI::UnitTestZAPIGen;
use ParseEngines::CommonParseEngine;
use CommonLibs::Utils;

# Class Objects

#my $unit_test_cli_obj = CodeModules::CLI::UnitTestCLIGen->new();
#my $unit_test_zapi_obj = CodeModules::ZAPI::UnitTestZAPIGen->new();

# Constructor Definition

sub new {
    my $class = shift;
    my $self  = {};
    bless $self, $class;
    return $self;
}

################################################################################
# Subroutine  : gen_unit_test                                                  #
#                                                                              #
# Description : This method generates the unit test code for DFM using 	       #
#		NACL APISet methods. 					       #
#		The resuling code resides in thpl files.                       #
#		The module handles the dynamic and static part of unit test code
#                                                                              #
# Returns     : Generates the unit test code 		                       #
#				or					       #
#		Displays an error 			                       #
################################################################################

sub gen_unit_test {
    my $self      = shift;
    my $conf_hash = shift;

    my $code_type = $conf_hash->{'code_type'};
    my $interface = $conf_hash->{'code'}->{$code_type}->{'interface'};
    my $unit_test_obj;

    print "\nGenerating Unit test Code for $interface\n";

    # No Input handling as it is already done at the Config Parse level


    $unit_test_obj = CodeModules::CLI::UnitTestCLIGen->new() if ( $interface =~ /cli/i );	
    $unit_test_obj = CodeModules::ZAPI::UnitTestZAPIGen->new() if ( $interface =~ /zapi/i );

    $unit_test_obj->gen_unit_tests($conf_hash);
     #   if ( $interface =~ /cli/i );

#$unit_test_zapi_obj->gen_zapi_unit_tests($conf_hash) if ($interface =~ /zapi/i);

    # Run Perltidy on the generated unit test files	
	

    my $perl_tidy = CommonLibs::Utils->new();
    $perl_tidy->run_perltidy($conf_hash->{'code_path'});	

}



1;
