# File Name     :       DataValidation.pm
#
# Summary       :       The package parses zapidoc and returns a list ZAPI corresponding to the given ZAPI
#
# Creation Date :       18th February 2011
#
# Author        :       Ramya.R@netapp.com, dl-sm-nacl-dev@netapp.com
#
# Version History
# ---------------
#
#  Version 1
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#

#
# Package Declaration
#

package CodeModules::ZAPI::DataValidation;

#
# Compiler Directives
#

use strict;
use warnings;

#
# Perl Modules
#

use Data::Dumper;
use XML::Parser;
use XML::Simple;

#
# User Specific Modules
#

use ParseEngines::CommonParseEngine;

#
# Class Objectives
#
my $zapi_parser = ParseEngines::CommonParseEngine->new;

# Constructor Definition

sub new {
    my $class = shift;
    my $self  = {
        'filenames' => undef,
        'logger'    => undef,
    };
    bless $self, $class;
    return $self;
} ## end sub new

##########################################################################################
# Subroutine : datavalidation                                                            #
#                                                                                        #
# Description : The function returns a list ZAPI corresponding to the given ZAPI which   #
#               is used for Validating XML data                                          #
#                                                                                        #
# Return     : Nested Hash                                                               #
##########################################################################################

sub data_validate {
    my ( $self, %input_args ) = @_;
    my $category = $input_args{"category"};
    my $hashref  = $input_args{"hashref"};
    my $zapi     = $input_args{"zapi"};
    my @RequiredZapiArray;
    my @list;
    my %zapi_category_hash = %{ $hashref->{'zapi_category'} };
    if ( exists( $zapi_category_hash{$category} ) ) {
        @RequiredZapiArray = @{ $zapi_category_hash{$category} };
    } else {
        return undef;
    }
    my @zapi_list;

## Find if there is a list command corresponding to the given ZAPI
    if ( $zapi =~ /[a-zA-Z-_]*/ ) {
        @list = split( /-|_/, $zapi );
    }

    my $flag = 0;
    my @final_list;
    my $count   = 0;
    my $element = $list[0];
    my @substring_array;
    push( @substring_array, $element );
    foreach (@list) {
        if ( $count > 0 ) {
            $element = $element . "-" . $_;
            push( @substring_array, $element );
        }
        $count++;
    }

    # foreach (@substring_array) {
    # print "$_\n";
    # }
    foreach (@RequiredZapiArray) {
        my $currentzapi = $_;
        my $count       = 0;
        foreach (@substring_array) {
            if ( $currentzapi =~ /($_.*list.*)/ ) {

                push( @final_list, $1 );

                # print "found:$currentzapi:list\n";
                $flag = 1;
            }

        }
    }

    # foreach (@final_list) {
    # print "------------FINAL LIST-----------\n";
    # print "$_\n";
    # }

    if ( $flag == 0 ) {
        return undef;
    }

    my %hash = map { $_ => 1 } @final_list;
    my @unique = keys %hash;
    my $Zapi_to_Return;

    my $number = scalar(@substring_array);
    foreach (@unique) {
        if ( $_ =~ /$zapi/ ) {
            $Zapi_to_Return = $_;
            return $Zapi_to_Return;

        } else {
            my $count1 = 0;
            my $current;
            while ( $count1 < $number ) {
                $current = $substring_array[$count1];
                # unless ( $_ =~ /iter/ ) {
                    # if ( $_ =~ /$current-list/ ) {
                        # $Zapi_to_Return = $_;
                        # return $Zapi_to_Return;
                    # }
                # } 
				if ( $_ =~ /$current-list(.*)iter(.*)next/ ) {
                    $Zapi_to_Return = $_;
                    return $Zapi_to_Return;
                }
				else {
				return undef;
				}
                $count1++;
            }
        }
    }
}
1;

