# File Name 	:	CLIHandle.pm 
#
# Summary	:	Contains methods which process command line parameters provided by the user
#
# Creation Date :	4th August 2010
#
# Author	:	tanzim ;dl-sm-nacl-dev@netapp.com
#
# Version History
# ---------------
#
#  Version 1
#       Summary: Modified CLI handling to consolidate CDEF/Component/Task check into libcheck	
#       Date  : 11th August 2010
#       Author: Tanzim
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#

#
# Package Declaration
#

package CommonLibs::CLIHandle;

#
# Compiler Directives
#

use strict;
use warnings;

#
# Perl Modules
#

use Data::Dumper;


# Constructor Definition

sub new {
    my $class = shift;
    my $self  = {};
    bless $self, $class;
    return $self;
}


################################################################################
# Subroutine  : check_cli                                                      #
#                                                                              #
# Description : This method displays the tool options incase the invalid       #
# arguments are provided or tool help/usage is required                        #
#                                                                              #
# Returns : Code Type  		                                               #
#		or							       #	
#	    Displays CLI Help and exits the tool                               #  
################################################################################


sub check_cli {

    my ( $self, %args ) = @_;

    if ( !keys %args ) {

        # If no parameter is specified

        print "\n\t\tError : No switch selected\n";
        $self->_cli_help();
    }

    else {

        # return the code type

        return $args{'code_type'};
    }

}

sub _cli_help {

    my $self = shift;
    my $here_doc;

    $here_doc = <<HELP;

                NCAGE Tool CLI Usage Help
                -------------------------

                Invocation : /usr/software/bin/perl ncage.pl --code_type <code_type> --interface <interface>
                            --file_path <file_path> --type <type> --command <command> --version <version> --test_type <test case type>

                Code Type  :
                                * cdef
                                  Generates DFM NACL CDEF files

                                * component
                                  Generates DFM NACL Components

                                * task
                                  Generates DFM NACL Tasks

                                * unittest
                                  Generates DFM unit tests with NACL APISet functions

                                * functest
                                  Generates DFM functional tests

                                * libcheck
                                  Verifies if the specified NACL Library(CDEF , Component , Task) is present

                Interface  :
                                * CLI
                                * ZAPI

                File_path  :
                                * Location of the files required as input

                Type       :
                                * This is required only when the interface is ZAPI. It specifies the ZAPIdoc path

                Command    :
                                *  Valid DFM CLI or ZAPI command

                Version    :
                                *  Version of the ZAPIdoc or CDEF


HELP
    print $here_doc;
    exit 1;

}


1;
