################################################################################
#                                                                              #
# Created by  : Ramya.M.R                                           	       #
# File Name   : DFMCLIParser.pm                                                #
#                                                                              #
# Methods     : new         						       #
#	          : getCmdParams					       #
#             : getOptList						       #
#	          : getParameterList				               #
#	          : getCmdList		 				       #
#             : parseDfmCLI                                                    #
# Copyright MEI QA , NetApp                                                    #
#                                                                              #
################################################################################

package CommonLibs::DFMCLIparser;
use strict;
use warnings;
use Data::Dumper;
use CommonLibs::OptionOrder;

my $file;

sub new {
    my $class = shift;
    my $self  = {};
    bless $self, $class;
    return $self;
}

################################################################################
# Subroutine  : getParameterList	                                       #
#                                                                              #
# Description : Parses through the syntax tree                                 #
#                                                                              #
#                                                                              #
# Returns :   Array of parameters                                              #
#                                                                              #
################################################################################
sub getParameterList {

    my $_cli_ref = shift;
    my @_param_list;
    my ( $_key, $_value, $_element );

    if ( ( ref $_cli_ref ) =~ /.*?HASH.*/ ) {

        while ( ( $_key, $_value ) = each %$_cli_ref ) {

            if ( $_value =~ /.*?ARRAY.*/i ) {

                push( @_param_list, getParameterList($_value) );

            } elsif ( $_value =~ /.*?HASH.*/i ) {

                push( @_param_list, getParameterList($_value) );

            } else {

                push( @_param_list, $_value )
                  ;    #if ($_value !~ /^\-[a-zA-Z]{1}/);

            }
        }
    }

    if ( ( ref $_cli_ref ) =~ /.*?ARRAY.*/ ) {

        foreach $_element (@$_cli_ref) {

            if ( $_element =~ /.*?ARRAY.*/i ) {

                push( @_param_list, getParameterList($_element) );

            } elsif ( $_element =~ /.*?HASH.*/i ) {

                push( @_param_list, getParameterList($_element) );

            } else {

                push( @_param_list, $_element )
                  ;    #if ($_element !~ /^\-[a-zA-Z]{1}/);

            }
        }
    }

    return @_param_list;
}

################################################################################
# Subroutine  : getCmdList	                                               #
#                                                                              #
# Description : This module parses the command synopsis and obtains the names  #
#               of sub commands                                                #
# Returns     : Array containing the names of sub commands                     #
#                                                                              #
################################################################################

sub getCmdList {
    my $cmdinfo = shift;
    my @command_array;
    my @cmdinfo = @$cmdinfo;
    foreach (@cmdinfo) {
        next if ( $_ !~ /^df[a-zA-Z]?m/ );
        my ( $command_name, $blah ) = split( /[\<\[\{-]/, $_, 2 );
        push( @command_array, $command_name );

    }
    return @command_array;
}

################################################################################
# Subroutine  : process_help_output	                                       #
#                                                                              #
# Description : This module parses the command help output                     #
#                	                                                       #
# Returns      : Hash  of the processed output                                 #
#                                                                              #
################################################################################

sub process_help_output {

    my $help_out = shift;
    my %help_hash;
    my $key;
    $help_out =~ s/^\s+//smg;
    $help_out =~ s/\s+$//smg;
    my @help_arr = split( /([A-Z]{4,})/, $help_out );
    shift @help_arr;
    %help_hash = @help_arr;
    return %help_hash;
}

################################################################################
# Subroutine  : getCmdParams	                                               #
#                                                                              #
# Description : This module parses the command line synopsis syntax 	       #
# Returns : Hash of the syntax                                                 #
#                                                                              #
################################################################################

sub getCmdParams {

    my $cmd_synopsis = shift;
    my $cmd          = shift;
    my %alloptions;
    my @temp = split( /\s+/, $cmd_synopsis );
    my ( @param, @switch, @arg, @arg1, @alt_par, @alt_par_dup,
        @alt_par_an_dup );
    my $line;
    my $alt_cnt         = 0;
    my $alt_cnt_dup     = 0;
    my $alt_cnt_an_dup  = 0;
    my @mandate         = ();
    my $alt_line        = "";
    my $alt_line_dup    = "";
    my $alt_line_an_dup = "";
    my %switch_param;
    my @option_names;
    my @switch_single;
    my @mandatoryop;
    my @alloptions;
    my $flag   = 0;
    my @params = ();

    foreach $line (@temp) {

#################################################################################################################
# handle alternate options  egs : { all | <objects> ... | aggregate | lun | volume | -v <volume> | -s <striped> }
#################################################################################################################
        if ( ( $line =~ /^\{/ ) and ( ( scalar @switch ) == 0 ) ) {

            $alt_cnt = 1;
            next;
        }

        $alt_cnt = 0 if ( $line =~ /\}$/ );

        if ( $alt_cnt == 1 ) {

            $alt_line .= $line;

            next;
        }

        my %switch_optional = ();
        my @optional        = ();

        if ( $alt_line ne "" ) {

            @alt_par = split( /\|/, $alt_line );

            foreach (@alt_par) {

                #Assigning alternate options array dsc to the final dsc

                if (/.*?(\-[a-zA-Z]){1}.*?\<(.*)\>/i) {

                    my $switch           = $1;
                    my $opt_param        = $2;
                    my @alt_switch_param = split( /\>\</, $opt_param );

                    $switch_optional{"$switch"} = \@alt_switch_param;
                    push @optional, \%switch_optional;

                } elsif (/.*?([a-zA-Z:]+).*/i) {

                    my $arg = $1;
                    $arg =~ s/://;

                    push @optional, $arg;

                } else {

                }

            }
            $switch_param{"alternate_param_options"} = \@optional;

            $alt_line = "";
            next;

        }

###################################################################################################
# Handle switch options with the following arguments : -t <temp> , -t {hello|world} , [-a|-e|-o|-u]
###################################################################################################

        push @switch, $line if ( $line =~ /^\-[a-zA-Z]{1}/ );

        push @switch_single, shift @switch
          if ( ( ( scalar @switch ) > 1 ) |
            ( ( $line =~ /^\]/ ) & ( ( scalar @switch ) > 0 ) ) );

##########################################################################
      # handles switches of the type : [ -F { text | html | perl | paragraph } ]

##########################################################################

        if ( ( $line =~ /^\{/ ) and ( ( scalar @switch ) != 0 ) ) {

            $alt_cnt_dup = 1;
            next;
        }

        $alt_cnt_dup = 0 if ( $line =~ /\}$/ );

        if ( $alt_cnt_dup == 1 ) {

            $alt_line_dup .= $line;
            next;
        }

        my %switch_optional_dup = ();
        my @optional_dup        = ();

        if ( $alt_line_dup ne "" ) {

            @alt_par_dup = split( /\|/, $alt_line_dup );

            foreach (@alt_par_dup) {

                if (/.*?(\-[a-zA-Z]){1}.*?\<(.*)\>/i) {

                    my $switch           = $1;
                    my $opt_param        = $2;
                    my @alt_switch_param = split( /\>\</, $opt_param );

                    $switch_optional_dup{"$switch"} = \@alt_switch_param;
                    push @optional_dup, \%switch_optional_dup;

                } elsif (/.*?([a-zA-Z]+).*/i) {

                    push @optional_dup, $1;

                } else {

                }

            }

            #Assigning alternate options array dsc to the final dsc
            $switch_param{ pop @switch } = \@optional_dup;

            $alt_line_dup = "";
            next;
        }

##########################################################################
        # handles alternate switch options egs :  [ -a | -m | -o | -r | -i ]
##########################################################################

        if ( ( $line =~ /(\|)/ ) and ( $alt_cnt_dup == 0 ) ) {

            $flag = 1;
            if ( ( scalar @switch ) > 0 ) {
                my $syn = pop @switch;

                push( @params, $syn ) if ( $syn =~ /\-[a-zA-Z]{1}/ );

            }
            next;
        }

        push( @params, pop @switch )
          if ( ( $flag == 1 ) && ( $line =~ /\-[a-zA-Z]{1}/ ) );

        $flag = 0;

#############################################################################################################################################################################
        # handles parameters of the type : <option-name>=<option-value>
#############################################################################################################################################################################

        if ( $line =~ /\<(.*?)\>=\<(.*?)\>/g ) {

            my $option_list = getOptList($cmd);

            #Assigning option names array to the final dsc
            $switch_param{'cmd_sep'}      = "=";
            $switch_param{'option_names'} = $option_list
              if ( scalar(@$option_list) > 0 );

            next;

        }

#############################################################################################################################################################################
        # handles parameters of the type : option-name=<option-value>
#############################################################################################################################################################################

        if ( $line =~ /(.*?)=\<(.*?)\>/g ) {

            push @option_names, $1;

            next;

        }

#############################################################################################################################################################################
# handles assigning switches to their respective parameters ,handles single switches also, handles mandatory parameteres without switches
# egs : -n <full-name> ,-n  name, -n , <name> ,
#############################################################################################################################################################################

        #if($line =~  /\<(.*?)\>/g){

        if ( ( $line =~ /\<(.*?)\>/g ) |
            ( ( $line =~ /^([a-zA-Z]*)$/i ) & ( ( scalar @switch ) > 0 ) ) )
        {

            if ( ( scalar @arg1 ) > 0 ) {

                push( @arg, $1 ) if ( $1 ne $arg1[-1] );
                push( @arg1, $1 );

            } else {

                push( @arg,  $1 );
                push( @arg1, $1 );
            }

            #Assigning switch-parameter pair to final dsc

            $switch_param{ pop @switch } = pop @arg
              unless ( ( scalar @switch ) == 0 );
        }

        if ( $line =~ /<(.*?)>/ ) {
            push( @alloptions, $1 );
        }
    }    #end of foreach loop

    #Assigning alternate switches array to the final dsc
    if ( ( scalar @params ) > 0 ) {
        my %hash = map { $_ => 1 } @params;
        @params = keys %hash;
    }
    $switch_param{"alternate_switch_options"} = \@params
      if ( ( scalar @params ) > 0 );

    my $synopsisparam = $cmd_synopsis;
    $synopsisparam =~ s/(\[.*?\])|(\{.*?\})//g;

    while ( $synopsisparam =~ m/<(.*?)>/mg ) {
        next if ( ( $1 eq "option-name" ) | ( $1 eq "option-value" ) );
        push( @mandatoryop, $1 );

    }

    # naming single_switches

    my $newcmd = $cmd;
    $newcmd =~ s/ /-/g;
    my %switchhash = map { $_ => 1 } @switch_single;
    foreach my $key ( keys %switchhash ) {
        $switchhash{$key} = $key . $newcmd . "switch";
    }

    #Assigning single switches array to the final dsc

    $switch_param{'single_switches'} = \%switchhash
      if ( ( scalar @switch_single ) > 0 );

    #Assigning option names array to the final dsc

    $switch_param{'option_names'} = \@option_names
      if ( scalar(@option_names) > 0 );

    #Assigning mandatory parameters array to the final dsc

    $switch_param{"mandatory"} = \@mandatoryop if ( ( scalar @arg ) > 0 );
    $switch_param{"all_options"} = \@alloptions
      if ( ( scalar @alloptions ) > 0 );
    $alloptions{$cmd} = \%switch_param;

    return \%switch_param;
}
################################################################################
# Subroutine  : getOptList	                                               #
#                                                                              #
# Description : This module looks for all options       	               #
# Returns : Array of all possible options                                      #
#                                                                              #
################################################################################
sub getOptList {

    my $command = shift;

    my @out  = `$command`;
    my $size = scalar @out;

    my @opt_list;
    my $cnt = 0;

    foreach (@out) {

        $cnt++;
        next if ( $cnt == 1 );

        my @opt_defn = split( /\s+/, $_ );

        push( @opt_list, $opt_defn[1] );

    }

    return \@opt_list;
}

################################################################################
# Subroutine  : parseCmdHelp	                                               #
#                                                                              #
# Description : This module parses the sub command help output	               #
# Returns     : Sub command synopsis and description                           #
#                                                                              #
################################################################################

sub parseCmdHelp {
    my $subSubCmd   = shift;
    my $cmd         = "$subSubCmd help";
    my $result      = `$cmd`;
    my $fullCmd     = "";
    my $outLine     = "";
    my $isName      = 0;
    my $isSynopsis  = 0;
    my $isDesc      = 0;
    my $cmdName     = "";
    my $cmdSynopsis = "";
    my $cmdDesc     = "";

    my @temp = split( /\n/, $result );
    foreach $outLine (@temp) {

        if ( $outLine =~ /NAME/ ) {
            $isName     = 1;
            $isSynopsis = 0;
            $isDesc     = 0;
            next;
        } elsif ( $outLine =~ /SYNOPSIS|COMMANDS/ ) {
            $isName     = 0;
            $isSynopsis = 1;
            $isDesc     = 0;
            next;
        } elsif ( $outLine =~ /DESCRIPTION/ ) {
            $isName     = 0;
            $isSynopsis = 0;
            $isDesc     = 1;
            next;
        }

        if ($isName) {
            $cmdName = $cmdName . $outLine;

        } elsif ($isSynopsis) {
            $cmdSynopsis = $cmdSynopsis . $outLine;

        } elsif ($isDesc) {
            $cmdDesc = $cmdDesc . $outLine;

        }
    }

    $cmdName = trimWhiteSpace($cmdName);
    my @cmdBrk = split( /--/, $cmdName );
    $cmdName = $cmdBrk[1];

    return ( $cmdName, $cmdSynopsis, $cmdDesc );
}
################################################################################
# Subroutine  : trimWhiteSpace	                                               #
#                                                                              #
# Description : This module eliminates leading and trailing white spaces       #
# Returns     : Line free of leading and trailing white spaces                 #
#                                                                              #
################################################################################

sub trimWhiteSpace {

    my $dsLine = shift;
    $dsLine =~ s/^\s*//;
    $dsLine =~ s/\s*$//;
    $dsLine =~ s/\s+/ /g;

    return $dsLine;

}
################################################################################
# Subroutine  : parseDfmCLI                                                    #
#                                                                              #
# Description : This module checks if the command name or help file is         #
#               provided and processes accordingly	                       #
# Returns     : Hash Containing all sub commands and their corresponding hashes#  
#                                                                              #
################################################################################

sub parseDfmCLI {
    my ( $cmd, $cdefpath, $result, @cmdarray, $flag );
    my ( $self, %args ) = @_;
    my $optionOrder = CommonLibs::OptionOrder->new();

    if ( defined $args{"commandname-or-path"} ) {

        #if command name is provided
        if ( $args{"commandname-or-path"} =~ /\s+/ )

        {

            @cmdarray = split( /,/, $args{"commandname-or-path"} );

        }

        #if helpfile path is provided
        else {
            if ( -e $args{"commandname-or-path"} ) {
                if ( $args{"commandname-or-path"} =~
                    /(df[a-zA-Z]?m[a-zA-Z_]*)\.*/ )
                {
                    my $execmd = $args{"commandname-or-path"};
                    $result = `cat $args{"commandname-or-path"}`;
                    $file   = $1 . ".cdefs";
                    $cmd    = $1;
                    $flag   = 1;
                } else {
                    print "Please provide a proper command name\n";
                    exit;
                }
            } else {
                print "File does not exist.";
                exit;
            }
        }
    }

    if ( scalar(@cmdarray) > 0 ) {
        my ( %allcommands, @filenamearray );
        foreach (@cmdarray) {
            my %alloptions = ();
            $cmd = $_;
            my $commandname = $cmd;
            $commandname =~ s/_/ /g;

            $file = $cmd . ".cdefs";
            $cmd =~ s/_/ /g;
            $cmd = $cmd . " " . "help";
            next if ( system("$cmd") != 0 );
            $result = `$cmd`;

            my %hash          = process_help_output($result);
            my $list          = $hash{'COMMANDS'};
            my @synopsis      = ();
            my @list_cmds     = split /\n/, $list;
            my @cmd_list      = getCmdList( \@list_cmds );
            my @param_all_cmd = ();
            foreach (@cmd_list) {
                @param_all_cmd = ();
                my ( $cmd_name, $cmd_synopsis, $cmd_desc ) = parseCmdHelp($_);
                my $cmd_syntax_hash = getCmdParams( $cmd_synopsis, $_ );

                push( @param_all_cmd, getParameterList($cmd_syntax_hash) );
                my $optionorderref =
                  $optionOrder->getOptionorderArray( $_, $cmd_synopsis,
                    $cmd_syntax_hash );

                $cmd_syntax_hash->{'option_order'} = $optionorderref;
                $alloptions{$_} = $cmd_syntax_hash;

            }

            $allcommands{$commandname} = \%alloptions;
            push( @filenamearray, $file );

        }    #foreach ends
        return ( \%allcommands );
    }    #if ends

    elsif ( $flag == 1 ) {    #file-name provided
        my %allcommands;
        my %hash = process_help_output($result);

        my $list = $hash{'COMMANDS'};

        my ( @synopsis, $synopsis );
        my %alloptions;

        my @list_cmds = split /\n/, $list;

        my @cmd_list = getCmdList( \@list_cmds );
        my @param_all_cmd;
        my $count   = 0;
        my $synflag = 0;
        my %index;
        @index{@list_cmds} = ( 0 .. $#list_cmds );

        foreach (@list_cmds) {
            my $index = $index{$_};

            if ( ( $_ !~ /^df[a-zA-Z]?m/ ) && ( $index != $#list_cmds ) ) {
                $synopsis = $synopsis . $_;
                next;
            } elsif ( $_ =~ /^df[a-zA-Z]?m/ ) {
                if ( $synflag == 1 ) {

                    my ( $command_name, $blah ) =
                      split( /[\<\[\{-]/, $synopsis, 2 );

                    @param_all_cmd = ();
                    my $cmd_syntax_hash =
                      getCmdParams( $synopsis, $command_name );

                    my $optionorderref =
                      $optionOrder->getOptionorderArray( $command_name,
                        $synopsis, $cmd_syntax_hash );

                    $cmd_syntax_hash->{'option_order'} = $optionorderref;
                    $alloptions{$command_name}         = $cmd_syntax_hash;
                    $synopsis                          = ();
                    $synflag                           = 0;
                    redo;
                } else {
                    $synflag  = 1;
                    $synopsis = $_;
                    next;
                }

                $allcommands{$cmd} = \%alloptions;
                return ( \%allcommands );
            } elsif ( $index == $#list_cmds ) {
                $synopsis = $synopsis . $_;
                my ( $command_name, $blah ) =
                  split( /[\<\[\{-]/, $synopsis, 2 );

                my $cmd_syntax_hash = getCmdParams( $synopsis, $command_name );

                my $optionorderref =
                  $optionOrder->getOptionorderArray( $command_name, $synopsis,
                    $cmd_syntax_hash );

                $cmd_syntax_hash->{'option_order'} = $optionorderref;
                $alloptions{$command_name} = $cmd_syntax_hash;
            }
        }

        $allcommands{$cmd} = \%alloptions;
        return ( \%allcommands );

    }
}

1;
