# File Name     :       LogInit.pm
#
# Summary       :       The package initializes the log mechanism for the tool .
#
# Creation Date :       4th August 2010
#
# Author        :       tanzim,dl-sm-nacl-dev@netapp.com
#
# Version History
# ---------------
#
#  Version 1
#       Summary:
#       Date  :
#       Author:
#
# Copyright (c) 2003 Network Appliance, Inc.
# All rights reserved.
#

#
# Package Declaration
#

package CommonLibs::LogInit;

#
# Compiler Directives
#

use strict;
use warnings;

#
# Perl Modules
#

use Log::Log4perl qw(get_logger :levels);
use Data::Dumper;

#
# User Specific Modules
#


# Constructor Definition

sub new {
    my $class = shift;
    my $self  = {};
    bless $self, $class;
    $self->{'logger'} = undef;  	
    return $self;
}

################################################################################
# Subroutine  : create_log_object                                              #
#                                                                              #
# Description : This method creates a log object for use by different modules  #
#                                                                              #
# Returns     : Config hash with the log object added                          #
#                                                                              #
################################################################################

sub create_log_object {
    my $self        = shift;
    my $config_hash = shift;
    my $code_type   = shift;	

    # need to decide where to read the code_type from
	# Code type to be read from Config hash
		# Code type is passed instead of config hash as logging is moved above config  parsing and code type is pushed into the config hash at config parsing level

    # log level needs to be determined from hash structure
    my $level
        = ( defined $config_hash->{common}->{log_level} )
        ? $config_hash->{common}->{log_level}
        : "INFO";

    my $loglev = $self->_set_log_level($level);

    my $logdir = "Logs";

    # Linux specific , needs to be tested - done
    unless ( -d "../$logdir" ) {
        mkdir $logdir or die("ERROR : Cannot create log directory:$!");
    }

    #reads code type from config hash 
	# code type passed from driver file
    my $logfile = $code_type. "_"
        . join( '_', split( /\s+/, localtime() ) ) . "_log";

    #needs to be tested - done
    $logfile = "$logdir/$logfile";

    #print "\nLOGFILE $logfile\n";getc();

    $self->{'logger'} = get_logger();	
    $self->{'logger'}->level($$loglev);    # DEBUG,INFO,WARN,ERROR,FATAL

    my $appender = Log::Log4perl::Appender->new(
        "Log::Dispatch::File",
        filename => "$logfile",
        mode     => "append",
    );

    $self->{'logger'}->add_appender($appender);

    my $layout = Log::Log4perl::Layout::PatternLayout->new(
        "%d %p> %F{1}:%L %M - %m%n");
    $appender->layout($layout);

    $config_hash->{'logger'}  = $self->{'logger'};
    $config_hash->{'logfile'} = $logfile;

    return $config_hash;
}

################################################################################
# Subroutine  : _set_log_level                                                 #
#                                                                              #
# Description : This method  sets the user specified log level                 #
#                                                                              #
# Returns     : Log  Level                                                     #
#                                                                              #
################################################################################

sub _set_log_level {

    my $self = shift;
    my $lev  = shift;

    my $loglev;

    if ( $lev =~ /.*DEBUG.*/i ) {
        $loglev = \$DEBUG;
    } elsif ( $lev =~ /.*INFO.*/i ) {
        $loglev = \$INFO;
    } elsif ( $lev =~ /.*WARN.*/i ) {
        $loglev = \$WARN;
    } elsif ( $lev =~ /.*ERROR.*/i ) {
        $loglev = \$ERROR;
    } elsif ( $lev =~ /.*FATAL.*/i ) {
        $loglev = \$FATAL;
    } else {
        die
            "\n\t\tERROR:Please provide the correct log level\nThe levels are : DEBUG , INFO , WARN, ERROR , FATAL";
    }

    return $loglev;

}

################################################################################
# Subroutine  : log_message()                                                  #
#                                                                              #
# Description : This method  logs messages to a log file                       #
#                                                                              #
# Returns     : Nothing                                                        #
#                                                                              #
################################################################################

sub log_message {

    my $self = shift;
    my $level  = shift;
    my $message = shift;

    $self->{'logger'}->$level("\nTesting log functionality\n");
    	

}

1;

