# File Name 	:	Utils.pm 
#
# Summary	:	Contains commonly used utility methods
#
# Creation Date :	31st August 2010
#
# Author	:	dl-sm-nacl-dev@netapp.com
#
# Version History
# ---------------
#
#  Version 1
#       Summary: Added method to perltidy files 	
#       Date   : 31st August 2010 
#       Author : Jagadeesh
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#

#
# Package Declaration
#

package CommonLibs::Utils;

#
# Compiler Directives
#

use strict;
use warnings;

#
# Perl Modules
#

use Data::Dumper;
use File::Basename;
use Cwd;


# Constructor Definition

sub new {
    my $class = shift;
    my $self  = {};
    bless $self, $class;
    return $self;
}


################################################################################
# Subroutine  : run_perltidy                                                   #
#                                                                              #
# Description : This method runs perltidy on  files in the specified location  #
#                                                                              #
# Returns : Nothing   		                                               #
#									       #	
################################################################################


sub run_perltidy {
	my $self = shift;
        my $SCRIPT_PATH = shift;
        my $PERL_TIDY_CMD  = "/usr/software/bin/perltidy -pbp -ce -nst";

        # Take back up
        _cp_files("thpl","thpl","cp",$SCRIPT_PATH) ;

        # run perl tidy
        system ("$PERL_TIDY_CMD $SCRIPT_PATH/*.thpl");

        _cp_files("thpl.tdy","thpl","mv",$SCRIPT_PATH);
}

sub _cp_files () {
        my $source_ext   = shift;
        my $dest_ext     = shift;
        my $cmd          = shift;
        my $SCRIPT_PATH  = shift;

        unless (opendir(DIR, $SCRIPT_PATH)) {
                warn "Can't open $SCRIPT_PATH $!\n";
                closedir(DIR);
                return;
        }

        my $bkp_dir = $SCRIPT_PATH.'/Backup/' ;
        if ($cmd =~ /cp/) {
                system ("mkdir $bkp_dir");
        }

        foreach my $file (readdir(DIR)) {
                next unless $file =~ /${source_ext}$/ ;
                my $path = "$SCRIPT_PATH/$file";
                if( $file =~ /((\w+)\.${source_ext})$/ ) {
                        my $fileName = $2;
                        if ($cmd =~ /cp/) {
				#print "\n$cmd  $path $bkp_dir$fileName.$dest_ext\n";
                                system("$cmd  $path $bkp_dir$fileName.$dest_ext ");
                        }else {
				#print "\n$cmd  $path $SCRIPT_PATH/$fileName.thpl\n";
                                system("$cmd  $path $SCRIPT_PATH/$fileName.thpl");
                              }
                }
         }
}

sub run_perltidy_cdefs {
		my $self = shift;
        my $CDEF_PATH = shift;
		my $file = shift;
        my $PERL_TIDY_CMD  = "/usr/software/bin/perltidy -pbp -ce -nst";
		# Take back up
        _cp_files_cdefs("cdefs","cdefs","cp",$CDEF_PATH,$file) ;

        # run perl tidy
        system ("$PERL_TIDY_CMD $CDEF_PATH/*.cdefs");

        _cp_files_cdefs("cdefs.tdy","cdefs","mv",$CDEF_PATH,$file);
}
sub _cp_files_cdefs () {
        my $source_ext   = shift;
        my $dest_ext     = shift;
        my $cmd          = shift;
        my $CDEF_PATH  = shift;
		my $file         = shift;

        unless (opendir(DIR, $CDEF_PATH)) {
                warn "Can't open $CDEF_PATH $!\n";
                closedir(DIR);
                return;
        }

        my $bkp_dir = $CDEF_PATH.'/Backup/' ;
        if ($cmd =~ /cp/) {
                system ("mkdir $bkp_dir");
        }

			  foreach my $file (readdir(DIR)) {
                next unless $file =~ /${source_ext}$/ ;
                my $path = "$CDEF_PATH/$file";
                if( $file =~ /((\w+)\.${source_ext})$/ ) {
                        my $fileName = $2;
                        if ($cmd =~ /cp/) {
				#print "\n$cmd  $path $bkp_dir$fileName.$dest_ext\n";
                                system("$cmd  $path $bkp_dir$fileName.$dest_ext ");
                        }else {
				#print "\n$cmd  $path $SCRIPT_PATH/$fileName.thpl\n";
                                system("$cmd  $path $CDEF_PATH/$fileName.cdefs");
                              }
                }
         }
}






1;
