#!/usr/software/bin/perl 

# Name		:	ncage.pl (NACL Code Automation Generation Engine)	
# 
# Summary	:	The tool generates automated NACL-specific DFM code for 
#				- CDEFs
#				- Components
#				- Tasks
#				- Unit Tests
#				- Functional tests
#
# Author	:	tanzim ;dl-sm-nacl-dev@netapp.com
#
# Version History 
# ---------------
#
# Version 1 	
#	Summary: Developed driver file 
#	Date  :  11th August , 2010
#       Author:  Tanzim
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.

#
# Compiler Directives
#

use strict;
use warnings;

#
# Perl Modules
#

use Data::Dumper;
use Expect;
use Net::Ping;
use Log::Log4perl qw(get_logger :levels);
use Getopt::Long;

BEGIN { push @INC,'.'; }

#
# Tool Specific Modules
#

use CodeModules::UnitTestGen; 
use CommonLibs::ConfigParse;
use CommonLibs::CLIHandle;
use CommonLibs::LogInit; 
use CommonLibs::SetupInit;
use CodeModules::CdefGen;  

#
# Variable Declaration
#

# Objects
my ($unit_test_obj, $config_parse_obj, $cli_handle_obj, $log_obj, $setup_obj, $cdefgen_obj);
# Rest
my ($code_type,$config_hash, $interface, $file_path, $type, $command, $version, $test_type);
my $help = 0; 

#
# Class Objects
#

$unit_test_obj    = CodeModules::UnitTestGen->new();
$config_parse_obj = CommonLibs::ConfigParse->new();
$cli_handle_obj   = CommonLibs::CLIHandle->new();
$log_obj          = CommonLibs::LogInit->new(); 
$setup_obj        = CommonLibs::SetupInit->new();
$cdefgen_obj = CodeModules::CdefGen->new();  

#
#Handling the option values from CLI overriding the values in the config file
#
if (!@ARGV) {
 $cli_handle_obj->_cli_help();
}

my $result = 0;
$result = GetOptions(
    "code_type=s" => \$code_type,
    "interface=s" => \$interface,
    "file_path=s" => \$file_path,
    "type=s"      => \$type,
    "command=s"   => \$command,
    "version=s"   => \$version,
    "test_type=s" => \$test_type,  
    "help"        => \$help,
);

if ( $result != 1 || $help == 1 ) {
    $cli_handle_obj->_cli_help();
}
my %optionshash = (
    "code_type" => $code_type,
    "interface" => $interface,
    "file_path" => $file_path,
    "type"      => $type,
    "command"   => $command,
    "version"   => $version,
    "test_type" => $test_type, 
);

#
# CLI Handling
#

$code_type = $cli_handle_obj->check_cli( %optionshash );
my $cli_options_hash = \%optionshash;

#
# Logging
#

 #$config_hash = $log_obj->create_log_object($config_hash,$code_type);
 #print Dumper ($config_hash);

 #$config_hash->{'logger'}->info("\nTesting log functionality\n");

 #$log_obj->log_message("info","\nTesting log functionality\n");


#
# Parse User Config File
#

 # Code type is being passed so that the config parser can look for required params specific to the code type and generate an error.
 # The code type is also used as input in the config file.Need to decide where to read the code type from. 
 # Suggestion  : Read the code type from the CLI and insert it into the config hash when parse_config is called 

 $config_hash = $config_parse_obj->parse_config( $config_hash, %optionshash ); 


#
# Pre-Configuration Setup / Check
#	
	
$config_hash = $setup_obj->setup_init($config_hash);	

#print Dumper ($config_hash);

print "\n Nacl Code Automation Generation Engine\n";
print " ========================================\n";

#
# Direct Execution Flow
#
	if ($code_type =~ /^cdef$/i){

                $cdefgen_obj->cdefgenerator($config_hash);

	}elsif($code_type =~ /^component$/i){
		
		print "\nUnder Construction : Targeted for Milestone 2 : October 2010 \n";

	}elsif($code_type =~ /^task$/i){

		print "\nUnder Construction : Targeted for Milestone 4 : November 2010 \n";

	}elsif($code_type =~ /^unittest$/i){

		$unit_test_obj->gen_unit_test($config_hash);

	}elsif($code_type eq /^functest$/i){

		print "\nUnder Construction : Targeted for Milestone 5 : January 2011 \n";

	}elsif($code_type eq /^libcheck$/i){

		print "\nUnder Construction : Targeted for Milestone 1 : September 2010 \n";

	}else{


	}
 

#
# Post-Configuration Cleanup 
#

$config_hash = $setup_obj->setup_uninit($config_hash);

print "\nCompleted Execution\n";
print "===================\n";
