# this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/nfs/ACL/AclNFSV4.pm#1 $
#
# Copyright (c) 2005 Network Appliance, Inc.
# All rights reserved.
#

package nfs::ACL::AclNFSV4;

## @summary  
##   Provide NFS V4 ACL protocol constants
##
## @description  
##   Provide NFS V4 ACL protocol constants. The constants are
##   grouped under the following tags:
##
##	ACL4_SUPP_ATTR
##
##	ACE4_TYPES
##
##	ACE4_FLAGS
##
##	ACE4_MASK
##   
## @usage
##   use nfs::ACL::AclNFSV4(:ACL4_SUPP_ATTR);
##
## @status	review
##
## @author	dl-maryv (FSS Protocols QA)
## @notes	Created by EP Network Storage Performance Lab, Inc.

use strict;
use vars('@ISA','@EXPORT_OK','%EXPORT_TAGS');
require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(
    ACL4_SUPPORT_ALLOW_ACL
    ACL4_SUPPORT_DENY_ACL
    ACL4_SUPPORT_AUDIT_ACL
    ACL4_SUPPORT_ALARM_ACL
    ACE4_ACCESS_ALLOWED_ACE_TYPE
    ACE4_ACCESS_DENIED_ACE_TYPE
    ACE4_SYSTEM_AUDIT_ACE_TYPE
    ACE4_SYSTEM_ALARM_ACE_TYPE
    ACE4_FILE_INHERIT_ACE
    ACE4_DIRECTORY_INHERIT_ACE
    ACE4_NO_PROPAGATE_INHERIT_ACE
    ACE4_INHERIT_ONLY_ACE
    ACE4_SUCCESSFUL_ACCESS_ACE_FLAG
    ACE4_FAILED_ACCESS_ACE_FLAG
    ACE4_IDENTIFIER_GROUP
    ACE4_READ_DATA
    ACE4_LIST_DIRECTORY
    ACE4_WRITE_DATA
    ACE4_ADD_FILE
    ACE4_APPEND_DATA
    ACE4_ADD_SUBDIRECTORY
    ACE4_READ_NAMED_ATTRS
    ACE4_WRITE_NAMED_ATTRS
    ACE4_EXECUTE
    ACE4_DELETE_CHILD
    ACE4_READ_ATTRIBUTES
    ACE4_WRITE_ATTRIBUTES
    ACE4_DELETE
    ACE4_READ_ACL
    ACE4_WRITE_ACL
    ACE4_WRITE_OWNER
    ACE4_SYNCHRONIZE
);

# Create export tags to group the constants together.
%EXPORT_TAGS = (
    ACL4_SUPP_ATTR => [qw( ACL4_SUPPORT_ALLOW_ACL
                           ACL4_SUPPORT_DENY_ACL
                           ACL4_SUPPORT_AUDIT_ACL
                           ACL4_SUPPORT_ALARM_ACL)],

    ACE4_TYPES     => [qw( ACE4_ACCESS_ALLOWED_ACE_TYPE
                           ACE4_ACCESS_DENIED_ACE_TYPE
                           ACE4_SYSTEM_AUDIT_ACE_TYPE
                           ACE4_SYSTEM_ALARM_ACE_TYPE)],

    ACE4_FLAGS     => [qw( ACE4_FILE_INHERIT_ACE
                           ACE4_DIRECTORY_INHERIT_ACE
                           ACE4_NO_PROPAGATE_INHERIT_ACE
                           ACE4_INHERIT_ONLY_ACE
                           ACE4_SUCCESSFUL_ACCESS_ACE_FLAG
                           ACE4_FAILED_ACCESS_ACE_FLAG	
                           ACE4_IDENTIFIER_GROUP)],

    ACE4_MASK      => [qw( ACE4_READ_DATA
                           ACE4_LIST_DIRECTORY
                           ACE4_WRITE_DATA
                           ACE4_ADD_FILE
                           ACE4_APPEND_DATA
                           ACE4_ADD_SUBDIRECTORY
                           ACE4_READ_NAMED_ATTRS
                           ACE4_WRITE_NAMED_ATTRS
                           ACE4_EXECUTE
                           ACE4_DELETE_CHILD
                           ACE4_READ_ATTRIBUTES
                           ACE4_WRITE_ATTRIBUTES
                           ACE4_DELETE	
                           ACE4_READ_ACL
                           ACE4_WRITE_ACL
                           ACE4_WRITE_OWNER
                           ACE4_SYNCHRONIZE)]
);

# ACL support attribute (bitmask)
use constant ACL4_SUPPORT_ALLOW_ACL		=> 0x00000001;
use constant ACL4_SUPPORT_DENY_ACL		=> 0x00000002;
use constant ACL4_SUPPORT_AUDIT_ACL		=> 0x00000004;
use constant ACL4_SUPPORT_ALARM_ACL		=> 0x00000008;

# ACE Types
use constant ACE4_ACCESS_ALLOWED_ACE_TYPE	=> 0x00000000;
use constant ACE4_ACCESS_DENIED_ACE_TYPE	=> 0x00000001;
use constant ACE4_SYSTEM_AUDIT_ACE_TYPE		=> 0x00000002;
use constant ACE4_SYSTEM_ALARM_ACE_TYPE		=> 0x00000003;

# ACE Flags (bitmask)
use constant ACE4_FILE_INHERIT_ACE	        => 0x00000001;
use constant ACE4_DIRECTORY_INHERIT_ACE		=> 0x00000002;
use constant ACE4_NO_PROPAGATE_INHERIT_ACE	=> 0x00000004;
use constant ACE4_INHERIT_ONLY_ACE		=> 0x00000008;
use constant ACE4_SUCCESSFUL_ACCESS_ACE_FLAG	=> 0x00000010;
use constant ACE4_FAILED_ACCESS_ACE_FLAG	=> 0x00000020;
use constant ACE4_IDENTIFIER_GROUP		=> 0x00000040;

# ACE Mask (bitmask)
use constant ACE4_READ_DATA			=> 0x00000001;
use constant ACE4_LIST_DIRECTORY		=> 0x00000001;
use constant ACE4_WRITE_DATA			=> 0x00000002;
use constant ACE4_ADD_FILE			=> 0x00000002;
use constant ACE4_APPEND_DATA			=> 0x00000004;
use constant ACE4_ADD_SUBDIRECTORY		=> 0x00000004;
use constant ACE4_READ_NAMED_ATTRS		=> 0x00000008;
use constant ACE4_WRITE_NAMED_ATTRS		=> 0x00000010;
use constant ACE4_EXECUTE			=> 0x00000020;
use constant ACE4_DELETE_CHILD			=> 0x00000040;
use constant ACE4_READ_ATTRIBUTES		=> 0x00000080;
use constant ACE4_WRITE_ATTRIBUTES		=> 0x00000100;
use constant ACE4_DELETE			=> 0x00010000;
use constant ACE4_READ_ACL			=> 0x00020000;
use constant ACE4_WRITE_ACL			=> 0x00040000;
use constant ACE4_WRITE_OWNER			=> 0x00080000;
use constant ACE4_SYNCHRONIZE			=> 0x00100000;

1;
