package nfs::ACL::Audit_ace_events;

use strict;
use nfs::ACL::AclNFSV4 qw(:ACE4_TYPES :ACE4_FLAGS :ACE4_MASK);

use vars qw(
  @ISA
  @EXPORT_OK
);
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw( @dacl_allow_deny_file
                     @dacl_allow_deny_dir
					 @dacl_allow_file
					 @dacl_allow_dir
					 
					 @sacl_all_file
					 @sacl_all_dir
					 
					 @dacl_allow_deny_sacl_all_file
				     @dacl_allow_deny_sacl_all_dir
					 @dacl_allow_sacl_all_file
				     @dacl_allow_sacl_all_dir
					 					 
				     @audit_events_nfsv3
					 @audit_events_nfsv4
					 @audit_events_nfsv41
                );

# These list ACEs are for Audit test cases
our @dacl_allow_deny_file = (
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_READ_DATA | ACE4_WRITE_DATA | ACE4_APPEND_DATA | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS | 
				 ACE4_READ_ATTRIBUTES| ACE4_WRITE_ATTRIBUTES | ACE4_READ_ACL | ACE4_DELETE | ACE4_EXECUTE |
				 ACE4_WRITE_ACL,
        who   => 'OWNER@'
    },
	 {
        type  => ACE4_ACCESS_DENIED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_WRITE_OWNER| ACE4_SYNCHRONIZE,
        who   => 'OWNER@'
    },
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => ACE4_IDENTIFIER_GROUP,
        mask  => ACE4_READ_DATA | ACE4_WRITE_DATA | ACE4_APPEND_DATA | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS | 
				 ACE4_READ_ATTRIBUTES| ACE4_WRITE_ATTRIBUTES | ACE4_READ_ACL | ACE4_DELETE | ACE4_EXECUTE,
         who   => 'GROUP@'
    },
    {
        type  => ACE4_ACCESS_DENIED_ACE_TYPE,
	    flags => ACE4_IDENTIFIER_GROUP,
        mask  => ACE4_WRITE_OWNER| ACE4_WRITE_ACL | ACE4_SYNCHRONIZE,
        who   => 'GROUP@'
    },
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_READ_DATA | ACE4_WRITE_DATA | ACE4_APPEND_DATA | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS | 
				 ACE4_READ_ATTRIBUTES| ACE4_WRITE_ATTRIBUTES | ACE4_READ_ACL | ACE4_DELETE | ACE4_EXECUTE,
        who   => 'EVERYONE@'
    },
    {
        type  => ACE4_ACCESS_DENIED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_WRITE_OWNER| ACE4_WRITE_ACL | ACE4_SYNCHRONIZE,
        who   => 'EVERYONE@'
    }
);

our @dacl_allow_deny_dir = (
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_LIST_DIRECTORY | ACE4_ADD_FILE | ACE4_ADD_SUBDIRECTORY | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS |
                 ACE4_EXECUTE | ACE4_DELETE_CHILD | ACE4_READ_ATTRIBUTES | ACE4_WRITE_ATTRIBUTES | ACE4_DELETE | ACE4_READ_ACL |
                 ACE4_WRITE_ACL,
        who   => 'OWNER@'
    },
	{
        type  => ACE4_ACCESS_DENIED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_WRITE_OWNER | ACE4_SYNCHRONIZE,
        who   => 'OWNER@'
    },
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => ACE4_IDENTIFIER_GROUP,
        mask  => ACE4_LIST_DIRECTORY | ACE4_ADD_FILE | ACE4_ADD_SUBDIRECTORY | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS |
                 ACE4_EXECUTE | ACE4_DELETE_CHILD | ACE4_READ_ATTRIBUTES | ACE4_WRITE_ATTRIBUTES | ACE4_DELETE | ACE4_READ_ACL,
        who   => 'GROUP@'
    },
    {
        type  => ACE4_ACCESS_DENIED_ACE_TYPE,
	    flags => ACE4_IDENTIFIER_GROUP,
        mask  => ACE4_WRITE_ACL | ACE4_WRITE_OWNER | ACE4_SYNCHRONIZE,
        who   => 'GROUP@'
    },
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_LIST_DIRECTORY | ACE4_ADD_FILE | ACE4_ADD_SUBDIRECTORY | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS |
                 ACE4_EXECUTE | ACE4_DELETE_CHILD | ACE4_READ_ATTRIBUTES | ACE4_WRITE_ATTRIBUTES | ACE4_DELETE | ACE4_READ_ACL,
        who   => 'EVERYONE@'
    },
    {
        type  => ACE4_ACCESS_DENIED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_WRITE_ACL | ACE4_WRITE_OWNER | ACE4_SYNCHRONIZE,
        who   => 'EVERYONE@'
    }
); 
our @dacl_allow_file = (
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_READ_DATA | ACE4_WRITE_DATA | ACE4_APPEND_DATA | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS | 
				 ACE4_READ_ATTRIBUTES| ACE4_WRITE_ATTRIBUTES | ACE4_READ_ACL | ACE4_DELETE | ACE4_EXECUTE |
				 ACE4_WRITE_OWNER| ACE4_WRITE_ACL | ACE4_SYNCHRONIZE,
        who   => 'OWNER@'
    },
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => ACE4_IDENTIFIER_GROUP,
        mask  => ACE4_READ_DATA | ACE4_WRITE_DATA | ACE4_APPEND_DATA | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS | 
				 ACE4_READ_ATTRIBUTES| ACE4_WRITE_ATTRIBUTES | ACE4_READ_ACL | ACE4_DELETE | ACE4_EXECUTE |
				 ACE4_WRITE_OWNER| ACE4_WRITE_ACL | ACE4_SYNCHRONIZE,
         who   => 'GROUP@'
    },
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_READ_DATA | ACE4_WRITE_DATA | ACE4_APPEND_DATA | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS | 
				 ACE4_READ_ATTRIBUTES| ACE4_WRITE_ATTRIBUTES | ACE4_READ_ACL | ACE4_DELETE | ACE4_EXECUTE |
				 ACE4_WRITE_OWNER| ACE4_WRITE_ACL | ACE4_SYNCHRONIZE,
        who   => 'EVERYONE@'
    },
);

our @dacl_allow_dir = (
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_LIST_DIRECTORY | ACE4_ADD_FILE | ACE4_ADD_SUBDIRECTORY | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS |
                 ACE4_EXECUTE | ACE4_DELETE_CHILD | ACE4_READ_ATTRIBUTES | ACE4_WRITE_ATTRIBUTES | ACE4_DELETE | ACE4_READ_ACL |
                 ACE4_WRITE_ACL | ACE4_WRITE_OWNER | ACE4_SYNCHRONIZE,
        who   => 'OWNER@'
    },
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => ACE4_IDENTIFIER_GROUP,
        mask  => ACE4_LIST_DIRECTORY | ACE4_ADD_FILE | ACE4_ADD_SUBDIRECTORY | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS |
                 ACE4_EXECUTE | ACE4_DELETE_CHILD | ACE4_READ_ATTRIBUTES | ACE4_WRITE_ATTRIBUTES | ACE4_DELETE | ACE4_READ_ACL |
				 ACE4_WRITE_ACL | ACE4_WRITE_OWNER | ACE4_SYNCHRONIZE,
        who   => 'GROUP@'
    },
    {
        type  => ACE4_ACCESS_ALLOWED_ACE_TYPE,
	    flags => 0,
        mask  => ACE4_LIST_DIRECTORY | ACE4_ADD_FILE | ACE4_ADD_SUBDIRECTORY | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS |
                 ACE4_EXECUTE | ACE4_DELETE_CHILD | ACE4_READ_ATTRIBUTES | ACE4_WRITE_ATTRIBUTES | ACE4_DELETE | ACE4_READ_ACL |
				 ACE4_WRITE_ACL | ACE4_WRITE_OWNER | ACE4_SYNCHRONIZE,
        who   => 'EVERYONE@'
    },
); 
our @sacl_all_file = (
    {
        type  => ACE4_SYSTEM_AUDIT_ACE_TYPE,
	    flags => ACE4_SUCCESSFUL_ACCESS_ACE_FLAG | ACE4_FAILED_ACCESS_ACE_FLAG,
        mask  => ACE4_READ_DATA | ACE4_WRITE_DATA | ACE4_APPEND_DATA | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS | 
				 ACE4_READ_ATTRIBUTES| ACE4_WRITE_ATTRIBUTES | ACE4_READ_ACL | ACE4_DELETE | ACE4_EXECUTE |
				 ACE4_WRITE_OWNER| ACE4_WRITE_ACL | ACE4_SYNCHRONIZE,
        who   => 'OWNER@'
    },
    {
        type  => ACE4_SYSTEM_AUDIT_ACE_TYPE,
	    flags => ACE4_SUCCESSFUL_ACCESS_ACE_FLAG | ACE4_FAILED_ACCESS_ACE_FLAG | ACE4_IDENTIFIER_GROUP,
        mask  => ACE4_READ_DATA | ACE4_WRITE_DATA | ACE4_APPEND_DATA | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS | 
				 ACE4_READ_ATTRIBUTES| ACE4_WRITE_ATTRIBUTES | ACE4_READ_ACL | ACE4_DELETE | ACE4_EXECUTE |
				 ACE4_WRITE_OWNER| ACE4_WRITE_ACL | ACE4_SYNCHRONIZE,
        who   => 'GROUP@'
    },
    {
        type  => ACE4_SYSTEM_AUDIT_ACE_TYPE,
	    flags => ACE4_SUCCESSFUL_ACCESS_ACE_FLAG | ACE4_FAILED_ACCESS_ACE_FLAG,
        mask  => ACE4_READ_DATA | ACE4_WRITE_DATA | ACE4_APPEND_DATA | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS | 
				 ACE4_READ_ATTRIBUTES| ACE4_WRITE_ATTRIBUTES | ACE4_READ_ACL | ACE4_DELETE | ACE4_EXECUTE |
				 ACE4_WRITE_OWNER| ACE4_WRITE_ACL | ACE4_SYNCHRONIZE,
        who   => 'EVERYONE@'
    }

);
our @sacl_all_dir = (
    {
        type  => ACE4_SYSTEM_AUDIT_ACE_TYPE,
        flags => ACE4_SUCCESSFUL_ACCESS_ACE_FLAG | ACE4_FAILED_ACCESS_ACE_FLAG,
        mask  => ACE4_LIST_DIRECTORY | ACE4_ADD_FILE | ACE4_ADD_SUBDIRECTORY | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS |
                 ACE4_EXECUTE | ACE4_DELETE_CHILD | ACE4_READ_ATTRIBUTES | ACE4_WRITE_ATTRIBUTES | ACE4_DELETE | ACE4_READ_ACL |
                 ACE4_WRITE_ACL | ACE4_WRITE_OWNER | ACE4_SYNCHRONIZE,
        who   => 'OWNER@'
    },
    {
        type  => ACE4_SYSTEM_AUDIT_ACE_TYPE,
	    flags => ACE4_SUCCESSFUL_ACCESS_ACE_FLAG | ACE4_FAILED_ACCESS_ACE_FLAG | ACE4_IDENTIFIER_GROUP,
        mask  => ACE4_LIST_DIRECTORY | ACE4_ADD_FILE | ACE4_ADD_SUBDIRECTORY | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS |
                 ACE4_EXECUTE | ACE4_DELETE_CHILD | ACE4_READ_ATTRIBUTES | ACE4_WRITE_ATTRIBUTES | ACE4_DELETE | ACE4_READ_ACL |
                 ACE4_WRITE_ACL | ACE4_WRITE_OWNER | ACE4_SYNCHRONIZE,
        who   => 'GROUP@'
    },
    {
        type  => ACE4_SYSTEM_AUDIT_ACE_TYPE,
	    flags => ACE4_SUCCESSFUL_ACCESS_ACE_FLAG | ACE4_FAILED_ACCESS_ACE_FLAG,
        mask  => ACE4_LIST_DIRECTORY | ACE4_ADD_FILE | ACE4_ADD_SUBDIRECTORY | ACE4_READ_NAMED_ATTRS | ACE4_WRITE_NAMED_ATTRS |
                 ACE4_EXECUTE | ACE4_DELETE_CHILD | ACE4_READ_ATTRIBUTES | ACE4_WRITE_ATTRIBUTES | ACE4_DELETE | ACE4_READ_ACL |
                 ACE4_WRITE_ACL | ACE4_WRITE_OWNER | ACE4_SYNCHRONIZE,
        who   => 'EVERYONE@'
    }
);

our @dacl_allow_deny_sacl_all_file;
push (@dacl_allow_deny_sacl_all_file, @dacl_allow_deny_file);
push (@dacl_allow_deny_sacl_all_file, @sacl_all_file);

our @dacl_allow_deny_sacl_all_dir;
push (@dacl_allow_deny_sacl_all_dir, @dacl_allow_deny_dir);
push (@dacl_allow_deny_sacl_all_dir, @sacl_all_dir);
					 
our @dacl_allow_sacl_all_file;
push (@dacl_allow_sacl_all_file, @dacl_allow_file);
push (@dacl_allow_sacl_all_file, @sacl_all_file);

our @dacl_allow_sacl_all_dir;
push (@dacl_allow_sacl_all_dir,  @dacl_allow_dir );
push (@dacl_allow_sacl_all_dir,  @sacl_all_dir );

############################
#Audit events list
############################
our @audit_events_nfsv3 = (
	{
		'ObjectType' => 'File',
		'ObjectName' => '/logging_nfsv3_file',
		'EventName' => 'Get Object Attributes',
		'EventID' => '4663',
	},
	{
		'ObjectType' => 'File',
		'ObjectName' => '/logging_nfsv3_file',
		'EventName' => 'Set Object Attributes',
		'EventID' => '4663',
	},
	{
		'ObjectType' => 'File',
		'ObjectName' => '/logging_nfsv3_file',
		'EventName' => 'Write Object',
		'EventID' => '4663',
	},
	{
		'ObjectType' => 'File',
		'ObjectName' => '/logging_nfsv3_file',
		'EventName' => 'Read Object',
		'EventID' => '4663',
	},
	{
		'ObjectType' => 'Directory',
		'ObjectName' => '/logging_nfsv3_dir',
		'EventName' => 'Get Object Attributes',
		'EventID' => '4663',
	},
	{
		'ObjectType' => 'Directory',
		'ObjectName' => '/logging_nfsv3_dir',
		'EventName' => 'Set Object Attributes',
		'EventID' => '4663',
	},
	{
		'ObjectType' => 'File',
		'Attributes' => 'Set Attributes; ',
		'ObjectName' => '/logging_nfsv3_dir/logging_nfsv3_dir_created_file',
		'EventName' => 'Create Object',
		'EventID' => '4656',
	},
	{
		'ObjectName' => '/logging_nfsv3_dir',
		'EventName' => 'Read Directory',
		'EventID' => '4663',
	},
	{
		'FileName' => '/logging_nfsv3_dir/logging_nfsv3_dir_created_file',
		'EventName' => 'Unlink Object',
		'EventID' => '9998',
	},
	{
		'ObjectType' => 'File',
		'FileName' => '/logging_nfsv3_file',
		'EventName' => 'Create Hard Link',
		'EventID' => '4664',
	},
	{
		'Attributes' => 'Replace Existing; ',
		'OldPath' => '/logging_nfsv3_file',
		'NewPath' => '/logging_nfsv3_dir/logging_nfsv3_file_renamed',
		'EventName' => 'Rename Object',
		'EventID' => '9999',
	}
);

our @audit_events_nfsv4 = (
	{
    'ObjectType' => 'File',
    'ObjectName' => '/logging_nfsv4_file',
	'EventName' => 'Get Object Attributes',
	'EventID' => '4663',
    },
	{
    'ObjectType' => 'File',
    'ObjectName' => '/logging_nfsv4_file',
	'EventName' => 'Set Object Attributes',
    'EventID' => '4663',
    },
	{
    'ObjectType' => 'File',
    'ObjectName' => '/logging_nfsv4_file',
	'EventName' => 'Write Object',
    'EventID' => '4663',
	},
  {
    'ObjectType' => 'File',
    'ObjectName' => '/logging_nfsv4_file',
	'EventName' => 'Read Object',
	'EventID' => '4663',
      },
  {
    'ObjectType' => 'Directory',
    'ObjectName' => '/logging_nfsv4_dir',
	'EventName' => 'Get Object Attributes',
	'EventID' => '4663',
      },
  {
    'ObjectType' => 'Directory',
    'ObjectName' => '/logging_nfsv4_dir',
	'EventName' => 'Set Object Attributes',
	'EventID' => '4663',
      },
  {
    'ObjectType' => 'Symbolic Link',
    'Attributes' => 'Set Attributes; ',
    'ObjectName' => '/logging_nfsv4_dir/logging_nfsv4_dir_created_file',
	'EventName' => 'Create Object',
	'EventID' => '4656',
      },
  {
    'ObjectName' => '/logging_nfsv4_dir',
	'EventName' => 'Read Directory',
	'EventID' => '4663',
      },
  {
	'FileName' => '/logging_nfsv4_dir/logging_nfsv4_dir_created_file',
    'EventName' => 'Unlink Object',
	'EventID' => '9998',
      },
  {
    'ObjectType' => 'File',
	'FileName' => '/logging_nfsv4_file',
    'EventName' => 'Create Hard Link',
	'EventID' => '4664',
      },
  {
    'ObjectType' => 'File',
    'Attributes' => 'Set Attributes; Create; Open a non-directory; ',
    'ObjectName' => '/logging_nfsv4_dir_opened_file',
	'EventName' => 'Open Object',
	'DesiredAccess' => 'Read Data; List Directory; Write Data; Add File; Append Data; Add Subdirectory; ',
    'EventID' => '4656',
      },
 {
    'ObjectType' => 'File',
    'Attributes' => 'Open a non-directory; ',
    'ObjectName' => '/logging_nfsv4_file',
	'EventName' => 'Open Object',
	'DesiredAccess' => 'Read Data; List Directory; ',
    'EventID' => '4656',
      },
  {
    'ObjectType' => 'Directory',
    'ObjectName' => '/logging_nfsv4_dir',
	'EventName' => 'Get Object Attributes',
	'EventID' => '4663',
      },
  {
    'ObjectType' => 'Directory',
    'ObjectName' => '/logging_nfsv4_dir',
	'EventName' => 'Get Object Attributes',
	'EventID' => '4663',
      },
  {
    'ObjectType' => 'File',
    'ObjectName' => '/logging_nfsv4_file',
	'EventName' => 'Get Object Attributes',
	'EventID' => '4663',
      },
  {
    'ObjectType' => 'File',
    'ObjectName' => '/logging_nfsv4_file',
	'EventName' => 'Get Object Attributes',
	'EventID' => '4663',
      },
  {
	'Attributes' => 'Replace Existing; ',
	'OldPath' => '/logging_nfsv4_file',
    'NewPath' => '/logging_nfsv4_dir/logging_nfsv4_file_renamed',
	'EventName' => 'Rename Object',
	'EventID' => '9999',
	  }
);