# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/nfs/CommonParams.pm#5 $

package nfs::CommonParams;

## @summary
##  Common parameters used in NFS tests.
##
## @description
##  This sets up correct environment variables. Note that the
##  variables are coming out of main's name space.
##
## @usage
##  use nfs::CommonParams;
##
## @status      review
##
## @author      dl-nfsqa-only (NFS QA Sunnyvale)
## @notes       Created by EP Network Storage Performance Lab, Inc.
## @notes
##  Put the 'use nfs::CommonParams;' right after the BEGIN
##  block with the thpl script param statements to make sure
##  the the variables below are defined before other library
##  modules are used.

use strict;

#
# The following package declaration is required for these variables
# to be declared global in the main name space. It was done this
# way because 'use vars' doesn't support package names in the
# variable name, ex: $main::variable. Neither does the 'our'
# statement.
package main;

# Please keep the list in alphabetic order
use vars qw(
  $ADDR_TYPE
  $AGGR_GX_TYPE
  $AGGR_SIZE
  $CIFS_DOMAIN
  $CIFS_SHORT_DOMAIN
  $CIFS_DOMAIN_ADMIN
  $CIFS_DOMAIN_ADMIN_PASSWD
  $CIFS_DRIVE
  $CIFS_PASSWD
  $CIFS_SHARE
  $CIFS_USER
  $CLEANUP
  $CLIENT
  $CMDTIMEOUT
  $CONTAINER
  $EXPORT_ANON
  $EXPORT_GX_POL
  $EXPORT_POLICY_INDEX
  $EXTERN_SETUP
  $FILER_MODE
  $FILER
  $FILERA
  $FILERB
  $FILER_AGGR
  $FILER_CONN
  $FILER_GX_AGGR
  $FILER_GX_VOL
  $FILER_IP
  $FILER_QTREE
  $FILER_VOL
  $FILER_VOL_MNT
  $FLEX_SIZE
  $JUNCTION_PATH
  $MOUNT_VERS
  $MOUNTPOINT
  $NBLADE_DIR
  $NBLADE_HOST
  $NBLADE_START
  $NFS_V4_DELEGATIONS
  $NFS_V4_ID_DOMAIN
  $NIS_DOMAIN
  $NIS_DOMAIN_SERVER
  $NOBODY_GROUP
  $NOBODY_GROUP_ID
  $NOBODY_USER
  $NOBODY_USER_ID
  $ONTAPDIR
  $PFILER
  $RAID_TYPE
  $SETUP_DONE
  $SKIP_SNAPLOCK_RETENTION_SETUP
  $SLCC_STRICT
  $SUDO_PATH
  $TIMEOUT
  $TRAD_SIZE
  $VOL_SIZE
  $VOL_TYPE
  $WORM_VOL
  $WORM_AGGR
  $WORM_RETENTION
  $WAFLIRON_TIMEOUT
  $VOL_GX_SIZE
  $VSERVER_GX_NAME
);

# Wait ! Please maintain the alphabetical order, it is easy to add more in the future.

# Change the name space back
package nfs::CommonParams;

use Tharn;

# param sets up the variables in main's name space
param( 'AUTH',               '-default', 'sys' );
param( 'AGGR_SIZE',         '-default', 5 );
param( 'CIFS_DOMAIN',       '-default', 'cifsqa.lab.netapp.com' );
param( 'CIFS_SHORT_DOMAIN', '-default', 'cifsqa' );

#param('CIFS_DOMAIN_ADMIN', '-default', "$::CIFS_DOMAIN\\administrator");

param( 'CIFS_DOMAIN_ADMIN', '-default', "$::CIFS_SHORT_DOMAIN\\administrator" );
param( 'CIFS_DOMAIN_ADMIN_PASSWD', '-default', 'Hello123' );
param( 'CIFS_DRIVE',               '-default', '*' );
param( 'CIFS_PASSWD',              '-default', 'cifspass' );
param( 'CIFS_SHARE',               '-default', 'testshare' );
param( 'CIFS_USER',                '-default', 'administrator' );
param( 'CLEANUP',                  '-default', 'test' );
param( 'CLIENT',                   '-default', 'localhost' );
param( 'CMDTIMEOUT',               '-default', 120 );               # 2 mins
param( 'SETUP_DONE',               '-default', 'reuse' )
  ;    #this will replace EXTERN_SETUP in the future
param( 'EXPORT_ANON',        '-default', '65534' );
param( 'EXTERN_SETUP',       '-default', $::SETUP_DONE );
param( 'FILER_AGGR',         '-default', 'testaggr' );
param( 'FILER_CONN',         '-default', '#rsh-socket' );
param( 'FILER_IP',           '-default', undef );
param( 'FILER_QTREE',        '-default', 'filerqtree' );
param( 'FILER_VOL_MNT',      '-default', undef );
param( 'MOUNTPOINT',         '-default', undef );
param( 'NBLADE_START',       '-default', undef() );
param( 'NBLADE_HOST',        '-default', undef() );
param( 'NBLADE_DIR',         '-default', undef() );
param( 'NFS_V4_DELEGATIONS', '-default', 'on' );
param( 'NFS4_MINOR_VERSION', '-default', 0 );
param( 'NFS_V4_ID_DOMAIN',   '-default', 'netapp.com' );
param( 'NIS_DOMAIN',         '-default', 'netapp.com' );
param( 'NIS_DOMAIN_SERVER',  '-default', '172.19.3.11' );
param( 'NOBODY_GROUP',       '-default', 'nogroup' );
param( 'NOBODY_GROUP_ID',    '-default', '65535' );
param( 'NOBODY_USER',        '-default', 'nogroup' );
param( 'NOBODY_USER_ID',     '-default', $::EXPORT_ANON );
param( 'ONTAPDIR',           '-default', '/rlse/mainN' );
param( 'PFILER',             '-default', '' );
param( 'SUDO_PATH',          '-default', '/usr/software/bin/sudo');
param( 'TIMEOUT',            '-default', 600 );                # 10 mins
param( 'MOUNT_VERS',         '-default', undef );              #could be 4
param( 'FLEX_SIZE',          '-default', '30m' );
param( 'TRAD_SIZE',          '-default', 3 );
param( 'WORM_RETENTION',     '-default', '30s' );
param( 'WORM_AGGR',          '-default', 'wormaggr' );
param( 'WAFLIRON_TIMEOUT',   '-default', 2400 );
param( 'SKIP_SNAPLOCK_RETENTION_SETUP', '-default', 0 );
param( 'SLCC_STRICT',                   '-default', 1 );
param( 'FILER_MODE',                    '-default', 'mono' );
param( 'RAID_TYPE',                     '-default', undef );
param( 'ADDR_TYPE',                     '-default', "inet4" );

use config_base;
use Filer::Version qw(get_wafl_version WV_SCRIMSHAW_MAX);
my $wv;

if ( $::FILER_MODE eq 'mono' ) {
    param( 'FILER', '-mesg', '  hostspec of filer' );
} else {
    param( 'FILERA', '-mesg', '  FILERA for cluster' );
    param( 'FILERB', '-mesg', '  FILERB for cluster' );
    param( 'FILER',  '-set',  $::FILERA );
}

my $fh = host($::FILER);
if ( $fh->hosttype() !~ /filer-ng/ ) {
    eval { $wv = get_wafl_version($fh); };
    if ( $@ and !defined( $ENV{"VOL_TYPE"} ) and !defined($::VOL_TYPE) ) {
        logresult( 'CONFIG',
            "filer/sim is down or unreachable -- define \$VOL_TYPE to continue"
        );
    } elsif ( $wv > WV_SCRIMSHAW_MAX ) {
        param( 'VOL_TYPE', '-default', 'flex' );
    } else {
        param( 'VOL_TYPE',                      '-default', 'trad' );
        param( 'SKIP_SNAPLOCK_RETENTION_SETUP', '-default', 1 );
    }

    if ( $::VOL_TYPE eq 'flex' ) {
        param( 'VOL_SIZE',  '-default', $::FLEX_SIZE );
        param( 'FILER_VOL', '-default', 'testvol_flex' );
        param( 'CONTAINER', '-default', $::FILER_AGGR );
        param( 'WORM_VOL',  '-default', 'testvol_worm_flex' );
    } else {
        param( 'VOL_SIZE',  '-default', $::TRAD_SIZE );
        param( 'FILER_VOL', '-default', 'testvol_trad' );
        param( 'CONTAINER', '-default', $::FILER_VOL );
        param( 'WORM_VOL',  '-default', 'testvol_worm_trad' );
    }
}
if ( $fh->hosttype() =~ /filer-ng/ ) {

    param( 'FILER_GX_VOL',    '-default', 'testvol_flex' );
    param( 'JUNCTION_PATH',   '-default', '/' . $::FILER_GX_VOL );
    param( 'FILER_GX_AGGR',   '-default', 'testaggr' );
    param( 'AGGR_GX_TYPE',    '-default', 'flex' );
    param( 'VSERVER_GX_NAME', '-default', 'testvserver' );
    param( 'EXPORT_GX_POL',   '-default', 'test_pol' );
    param("EXPORT_POLICY_INDEX", "-default", time);
    param( 'VOL_GX_SIZE',     '-default', '200m' );

}

1;

