# this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/nfs/Export/ExportClass.pm#1 $
#
# Copyright (c) 2005 Network Appliance, Inc.
# All rights reserved.
#

package nfs::Export::ExportClass;

## @summary
##   Provide an OS independent export class
##
## @description
##   Provides an interface for OS dependent exports.
##
## @usage
##   use nfs::Export::ExportClass;
##
## @status  public
##
## @author  dl-maryv (FSS Protocols QA)
## @notes   Created by EP Network Storage Performance Lab, Inc.

use strict;
use Carp qw(croak);
use nfs::Export::Solaris;
use nfs::Export::Linux;
use nfs::Export::Filer;
use nfs::Export::Hpux;
use nfs::Export::Aix;
use vars qw( @ISA );

#---------------------------------------------------------------------
## @name ::new
##
## @description
##  Creates a new Export Object. This routine will create a new
##  object by calling the platform specific 'new' constructor
##  This way the user uses a single API to do exports and unexports
##  and the code underneath creates the appropriate object based on
##  the platform type.
##
## @synopsis
##    $obj = nfs::Export::ExportClass::->new($conn);
##
## @arg $conn
##  Required: The Connectrec object for the server.
##
## @return
##  Platform specific export object.
#---------------------------------------------------------------------
sub new($$;) {
    my ($class, $conn) = @_;

    # Check for correct connection object
    croak "Not a 'Connectrec' object"
      if ! UNIVERSAL::isa($conn, "Connectrec");

    # Determine the right platform
    # and inherit correct OS dependent class.
    my $type = $conn->hostp()->hosttype();
    if ( $type =~ /unix-solaris/i ) {
        @ISA = qw(nfs::Export::Solaris);
    } elsif ( $type =~ /unix-hpux/i ) {
        @ISA = qw(nfs::Export::Hpux);
    } elsif ( $type =~ /unix-aix/i ) {
        @ISA = qw(nfs::Export::Aix);
    } elsif ( $type =~ /unix-linux/i ) {
        @ISA = qw(nfs::Export::Linux);
    } elsif ( $type =~ /^filer/i ) {
        @ISA = qw(nfs::Export::Filer);
    } else {
        croak "ExportClass::new does not support hosttype: $type";
    }

    # Call the constructor for the OS dependent class
    SUPER::new $class($conn);
}

1;

