# this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/nfs/Export/Solaris.pm#1 $
#
# Copyright (c) 2005 Network Appliance, Inc.
# All rights reserved.
#

package nfs::Export::Solaris;

## @summary
##  Class to handle OS dependent exports on Solaris.
##
## @description
##  This class handles the OS specific portion of the NFS export
##  functions. It overrides the default behavior of the base
##  export class.
##
## @usage
##  use nfs::Export::Solaris;
##
## @status  public
##
## @author  dl-maryv (FSS Protocols QA)
## @notes   Created by EP Network Storage Performance Lab, Inc.

use strict;
use nfs::Export::ExportBase;
use vars qw( @ISA );

#---------------------------------------------------------------------
# Inherit all functionality from ExportBase class.
#---------------------------------------------------------------------
@ISA = qw(nfs::Export::ExportBase);

#---------------------------------------------------------------------
## @name ::new
##
## @description
##  Solaris specific class constructor. It overrides default
##  behavior of the base exports class.
##
## @return
##  Solaris specific export object.
#---------------------------------------------------------------------
sub new($$;) {
    my ($class, $conn) = @_;

    # Calls constructor of parent class with
    # the following overrides:
    SUPER::new $class ( $conn,
        export_cmd     => '/usr/sbin/share',
        unexport_cmd   => '/usr/sbin/unshare',
        export_r_cmd   => '/usr/sbin/shareall',
        unexport_r_cmd => '/usr/sbin/unshareall',
        export_flags   => '-F nfs -o',
        unexport_flags => '-F nfs',
        export_r_flags => '',
        export_file    => '/etc/dfs/dfstab',
        export_format  => '"share -F nfs -o $access $path"',
        export_list_s  => '-',
        path_origin    => nfs::Export::ExportBase::P_END,
    );
}

1;

