# $Id: //depot/prod/test/Rsierranevada/lib/nfs/Kerberos.pm#1 $
# -*- perl -*-

# ---------------------------------------------------
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Permission is granted to use this script on a NetApp Product.
# Any other use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
# ---------------------------------------------------
#{[(
package nfs::Kerberos;
##
## @summary AD and MIT KDC Library
##
## @usage use Kerberos;
##
## @status new
##
## @see-also lib/TCD.pm {name of related test case}
##
## @author Sanjay Shanthakumar (sanjay09@netapp.com)
##
## @pod here

=head1 NAME

Kerberos

=head1 DESCRIPTION

This library contains functions for AD KDC setup

=cut

#-------------------------------------------------------------------------------
# Compiler directives.
#-------------------------------------------------------------------------------
use strict;
use base qw(Exporter);
use warnings;

#-------------------------------------------------------------------------------
# Modules Import
#-------------------------------------------------------------------------------

use Tharn;
use Params::Validate qw(validate SCALAR ARRAYREF HASHREF UNDEF OBJECT);

use NACL::CS::Volume;
use NACL::C::Client;
use NACL::C::Client::Mount;
use Data::Dumper;
use Carp qw(croak);
use NATE::BaseException qw(:try);
use Net::LDAP;
use nfs::CommonParams;
use NACL::C::VserverCifs;
use NACL::CS::VserverCifs;
use NACL::STask::Volume;
use NACL::STask::VserverNFS;
use NACL::C::ExportPolicy;
use NACL::C::ExportPolicyRule;
use NACL::STask::VserverServicesLdap;
use NACL::STask::VserverServicesLdapClient;
use NACL::STask::VserverServicesUnixUser;
use NACL::STask::VserverServicesUnixGroup;
use NACL::C::VserverNfsKerberosConfig;
use NACL::C::VserverServicesKerberosRealm;
use NACL::C::NetworkInterface;
use NACL::C::VserverServicesDns;
use NACL::C::SystemNodeDate;
use NACL::Exceptions::NoElementsFound qw{:try};
use nfs::CommonUtilities
    qw{client_execute $Node_cmd $Client_cmd client_exit_sudo client_enter_sudo set_client_object reset_client_object check_client_and_sudo return_client check_client_object};
use vars qw{$KDC_Connection_Obj};

BEGIN {
    use vars qw(
        $LOGDIR
        $BACKUP_DIRECTORY
        $RCP_PATH
        $KDC_SERVICE_PORT
        $KADMIN_SERVICE_PORT
        $REALM_CONFIG_NAME
        $MIT_CONFIG_NAME
        $MIT_HOST_PASSWD
        $MIT_KDC_ROOT
        $MIT_KDC_ADMIN
        $MIT_KDC_HOST
        $PASSWD_SERVER_PORT
        $CLOCK_SKEW
        $LDAP_PORT
        $SUDO_PATH
        $SPN_PASSWORD
        $LDAP_SCHEMA
        $LDAP_USER
        %default_values
    );

    param('LOGDIR',              '-mesg',    'No LOGDIR, Something is SERIOUSLY WRONG');
    param('BACKUP_DIRECTORY',    '-default', "$::LOGDIR/backup_setup_files");
    param('RCP_PATH',            '-default', '/usr/bin/rcp');
    param('WIN_RCP_PATH',        '-default', 'C:\WINDOWS\system32\rcp.exe');
    param('LDAP_SCHEMA',         '-default', 'AD-IDMU');
    param('KDC_SERVICE_PORT',    '-default', '88');
    param('KADMIN_SERVICE_PORT', '-default', '749');
    param('REALM_CONFIG_NAME',   '-default', 'AD_WIN2K8');
    param('MIT_CONFIG_NAME',     '-default', 'MIT_Kerberos');
    param('MIT_HOST_PASSWD',     '-default', 'sundance');
    param('MIT_KDC_ADMIN',       '-default', 'admin');
    param('MIT_KDC_ROOT',        '-default', 'root');
    param('MIT_KDC_HOST',        '-default', 'host');
    param('PASSWD_SERVER_PORT',  '-default', '464');
    param('CLOCK_SKEW',          '-default', '5');
    param('LDAP_PORT',           '-default', '389');
    param('LDAP_USER',           '-default', 'admin');
    param('LDAP_PASSWD',         '-default', 'netapp1!');
    param('SPN_PASSWORD',        '-default', 'sundance');
    param('KERB_CLEANUP',        '-default', 'ALL');

    # Initiate some default values for this PM.
    $default_values{sudo_path} = param('SUDO_PATH', '-default', '/usr/software/bin/sudo');
    $Log->comment(qq{sudo path is being: $default_values{sudo_path}});
}    # end BEGIN

# Export stuff, so that others can use these.
our @EXPORT = qw(
    filer_kerberos_enable
    filer_kerberos_disable
    KDC_Server_Setup
    client_rpcgssd_restart
    filer_ntp_config
    client_krb5_config_create
    client_etc_hosts_modify
    client_etc_group_passwd_modify
    client_etc_sysconfig_nfs_modify
    client_etc_idmapd_modify
    KRB_Keytab_Setup
    KRB_Client_Setup
    KRB_Filer_Setup
    KRB_Cleanup
    KRB_Filer_Kerberos_Config_Create
    MIT_client_krb5_config_create
    MIT_verify_server_setup
    verify_client_installation
    MIT_verify_kinit
    MIT_add_client_principal_in_KDC
    MIT_add_host_principal_in_client
    MIT_Server_Setup
    MIT_delete_principal_from_client
    MIT_delete_client_principal_from_KDC
    MIT_KDC_add_current_user_spn
    MIT_KDC_add_principal
    MIT_KDC_delete_principal
    MIT_KDC_verify_principal
    Client_kinit
    client_rpcidmapd_restart
    get_fqn
    get_hostname
    establish_kdc_connection
    kerberos_create_dc
    KRB_Filer_LDAP_create
);
our @EXPORT_OK = @EXPORT;

=head1 METHODS

=head2 filer_kerberos_enable

This function enables kerberos on the specified interface

filer_kerberos_enable( kdc => \%<KDC-Details> , filer_details => \%<Filer-hash-details> );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item filer_details

(Required) : Reference to the Filer hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item MIT_Setup

(Optional) : If the subroutine is used during an MIT setup, then set the value to 1, Default value is 0

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub filer_kerberos_enable( % ) {
    my %opts = validate(
        @_,
        {   kdc           => {type => HASHREF},
            filer_details => {type => HASHREF},
            MIT_Setup     => {
                type     => SCALAR,
                optional => 1,
                default  => 0,
                regex    => qr/[01]/
            },
        }
    );

    $Log->enter();

    #validate the kdc and filer hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_filer_hash($opts{filer_details});

    my $intf;
    my $Api_Set_Obj = $Node_cmd->apiset();
    my $user;

    # If its AD setup, then administrator or any such user of the AD will
    # be in $user, otherwise it will be root/admin or any such users for the
    # MIT setup
    if (!$opts{MIT_Setup}) {    #Steps to be performed for MIT setup only
        $user = $opts{kdc}->{kdc_user};
    } else {
        $user = $opts{kdc}->{kdc_user} . '/' . $::MIT_KDC_ADMIN;
    }

    $intf = NACL::C::NetworkInterface->find(
        command_interface => $Node_cmd,
        filter            => {address => $opts{filer_details}->{filer_mount_ip}}
    );

    my $realm = uc("$opts{ kdc }->{ kdc_realm }");
    $Api_Set_Obj->execute_raw_command(command => 'set admin -confirmations on');

    my $cmd_output = get_fqn("junk", $opts{filer_details}->{filer_mount_ip}, 0);
    my $lif_name;
    if ($cmd_output =~ /(.*).$opts{ filer_details }->{ filer_dns }/) {    # obtain the interface name of the mount ip. for ex . <filername>-d1 or -d2
        $lif_name = $1;
    }

    # If the lif name was not returned, possible cause of failure could be that
    # the dns provided was wrong. So, obtain the dns and lif name.
    if ($lif_name eq '') {
        $lif_name = KRB_obtain_filer_lif_name(
            filer_details => $opts{filer_details},
            cmd_output    => $cmd_output
        );
    }

    #construct the nfs spn to be added into the filer
    my $spn = 'nfs/' . $lif_name . '.' . $opts{filer_details}->{filer_dns} . '@' . "$realm";

    try {

        # kerberos config
        NACL::C::VserverNfsKerberosConfig->modify(
            command_interface => $Node_cmd,
            vserver           => $opts{filer_details}->{filer_vserver},
            'lif'             => "$intf->{lif}",
            'kerberos'        => 'enable',
            'spn'             => $spn,
            'admin-username'  => $user,
            'admin-password'  => $opts{kdc}->{kdc_passwd},
        );
    }
    catch NATE::BaseException with {
        my $exception = shift;
        if ($exception =~ /already enabled/i) {
            $Log->comment('Kerberos already enabled on the lif ' . $exception->text());
        } else {
            croak("Failure during Kerberos config enable: $exception->text()");
        }
    };

    $Api_Set_Obj->execute_raw_command(command => 'set admin -confirmations off');

    $Log->exit();
    return 1;
} ## end sub filer_kerberos_enable( % )

sub KRB_obtain_filer_lif_name( % ) {
    my %opts = validate(
        @_,
        {   cmd_output    => {type => SCALAR},
            filer_details => {type => HASHREF},
        }
    );
    $Log->enter();
    my $lif_name = "";

    if ($opts{cmd_output} =~ /($opts{filer_details}->{ filer_name }.*)/) {

        #splits the host name. ex: nfsqa21.rtp.netapp.com would be split into 4 values
        my @host_ip_output = split('\.', $1);
        if ($host_ip_output[0] =~ /$opts{filer_details}->{ filer_name }-?(.*)/) {
            $lif_name = $1;
        }
        my ($length, $i, $dns_string);
        $length     = $#host_ip_output;
        $dns_string = '';
        for ($i = 1; $i <= $length; $i++) {
            $dns_string .= $host_ip_output[$i] . '.';
        }
        chop($dns_string);
        if ($opts{filer_details}->{filer_dns} ne $dns_string) {
            $opts{filer_details}->{filer_dns} = $dns_string;
        }
    }

    $Log->exit();
    return $lif_name;
} ## end sub KRB_obtain_filer_lif_name( % )

=head2 filer_kerberos_disable

This function disables kerberos on the specified interface

filer_kerberos_disable( kdc => \%<KDC-Details> , filer_details => \%<Filer-hash-details> );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item filer_details

(Required) : Reference to the Filer hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item MIT_Setup

(Optional) : If the subroutine is used during an MIT setup, then set the value to 1, Default value is 0

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub filer_kerberos_disable( % ) {
    my %opts = validate(
        @_,
        {   kdc           => {type => HASHREF},
            filer_details => {type => HASHREF},
            MIT_Setup     => {
                type     => SCALAR,
                optional => 1,
                default  => 0,
                regex    => qr/[01]/
            },
        }
    );
    $Log->enter();

    #validate the kdc and filer hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_filer_hash($opts{filer_details});

    my $intf;
    my $user;

    # If its AD setup, then administrator or any such user of the AD will
    # be in $user, otherwise it will be root/admin or any such users for the
    # MIT setup
    if (!$opts{MIT_Setup}) {
        $user = $opts{kdc}->{kdc_user};
    } else {
        $user = $opts{kdc}->{kdc_user} . "/" . $::MIT_KDC_ADMIN;
    }

    $intf = NACL::C::NetworkInterface->find(
        command_interface => $Node_cmd,
        filter            => {address => $opts{filer_details}->{filer_mount_ip}}
    );

    try {

        # kerberos config
        NACL::C::VserverNfsKerberosConfig->modify(
            command_interface => $Node_cmd,
            vserver           => $opts{filer_details}->{filer_vserver},
            'lif'             => "$intf->{lif}",
            'kerberos'        => 'disable',
            'admin-username'  => $user,
            'admin-password'  => $opts{kdc}->{kdc_passwd},
        );
    }
    catch NATE::BaseException with {
        my $exception = shift;
        my $text      = $exception->text();
        if ($text =~ /already disabled/i) {
            $Log->comment("Kerberos already disabled on the lif, error:$text");
        } else {
            croak("Failure during Kerberos config disable: $text");
        }
    };

    $Log->exit();
    return 1;
} ## end sub filer_kerberos_disable( % )

=head2 KDC_Server_Setup

This subroutine performs the initial AD setup, which includes AD user creation and verifies whether the user was successfully created

KDC_Server_Setup( kdc => \%<Kerberos-hash-details> , client => $<Client-name> );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client

(Required) : Name of the client which would be used to create the user on AD

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub KDC_Server_Setup( % ) {
    my %opts = validate(
        @_,
        {   kdc    => {type => HASHREF},
            client => {type => OBJECT},
        }
    );
    $Log->enter();

    #validate the kdc hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});

    # Establish the NACL::C::Client object to the KDC server
    establish_kdc_connection($opts{kdc}->{kdc_server});
    check_client_and_sudo($opts{client});

    # Making sure the server handle retrieved is a windows client.
    if (!$KDC_Connection_Obj->is_windows()) {
        croak("KDC_Server_Setup : Incorrect or invalid client is detected for the KDC server:\n$opts{ kdc }->{ kdc_server }");
    }

    my @client_name = split('\.', client_execute("hostname"));
    $opts{short_clientname} = $client_name[0];

    my $DC = kerberos_create_dc(kdc_realm => $opts{kdc}->{kdc_realm});

    # Add client name in AD computers account & Create Service Principal and verify that is registered
    my $AD_user_create = $KDC_Connection_Obj->execute(command => "dsget user \"cn=$opts{short_clientname},$opts{ kdc }->{ kdc_ou },$DC\"");

    #verify  if the dsget user succeeded, if so delete the previous entry
    if ($AD_user_create =~ /dsget succeeded/) {
        $KDC_Connection_Obj->execute(command => "dsrm -noprompt \"cn=$opts{short_clientname},$opts{ kdc }->{ kdc_ou },$DC\"");
    }
    my $user_create_output = $KDC_Connection_Obj->execute(command => "dsadd user \"cn=$opts{short_clientname},$opts{ kdc }->{ kdc_ou },$DC\"");

    #verify  if the dsget user failed then abort the script exection
    $AD_user_create = $KDC_Connection_Obj->execute(command => "dsget user \"cn=$opts{short_clientname},$opts{ kdc }->{ kdc_ou },$DC\"");
    if ($AD_user_create !~ /dsget succeeded/) {
        croak("AD user creation failed : $user_create_output");
    }

    my $AD_domain;
    my @arr = split('\.', $opts{kdc}->{kdc_realm});
    $AD_domain = $arr[0];

    #if the realm is nfsqa-cifs.rtp.netapp.com then obtain nfsqa-cifs
    if ($AD_domain eq '') {
        croak('KDC REALM was not passed into the subroutine');
    }

    #create user with the name nfs$client_name.
    my $status = $KDC_Connection_Obj->execute(command => "net user $opts{short_clientname} /DELETE");

    #The command completed successfully.
    my $net_user_create
        = qq{net user $opts{short_clientname} $opts{ kdc }->{ kdc_passwd } /ADD /FULLNAME:"$AD_domain\\$opts{ short_clientname }" /DOMAIN};
    $status = $KDC_Connection_Obj->execute(command => "$net_user_create");

    if ($status !~ /The command completed successfully/) {
        croak("creating user failed. :$@ and status :$status");
    }

    $Log->exit();
    return 1;
} ## end sub KDC_Server_Setup( % )

=head2 client_krb5_config_create

The krb5.conf file is created during run time and copied into /$KRB5_PATH/krb5.conf.

client_krb5_config_create( kdc => \%<Kerberos-hash-details> , client_object => $<APISet-client-object> , dns_domain => $<Filer-DNS_DOMAIN> , [sudo_path => $<sudo-with-fully-qualify-path>] ); 

Optional: sudo_path

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item dns_domain

(Required) : Dns domain, to which the filer has been configured

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub client_krb5_config_create( % ) {
    my %opts = validate(
        @_,
        {   kdc           => {type => HASHREF},
            client_object => {type => OBJECT},
            dns_domain    => {type => SCALAR},
            sudo_path     => {type => SCALAR, default => $default_values{sudo_path}},
        }
    );
    $Log->enter();

    #validate the kdc hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    check_client_and_sudo($opts{client_object});

    check_backup_directory(object => $opts{client_object});

    my $KRB5_PATH = verify_client_hosttype($opts{client_object});

    #Create backup files
    backup_files("/$KRB5_PATH/krb5.conf");

    #obtain the host information either by executing it in the krb client or in the client running the script
    my $tempSpn = client_execute("host $opts{ client_object }->{ hostrec }->{default_ip} ");

    #the output should contain the client name, otherwise execute it on the client
    #running the script
    if ($tempSpn !~ /.* $opts{ client_object }->{ client }.(.*)/) {
        $tempSpn = `host $opts{ client_object }->{ hostrec }->{default_ip}`;
    }

    #the output should contain the client name
    my $CLI_DOMAIN = "";
    if ($tempSpn =~ /($opts{client_object}->{client}.*)/) {
        my $cli_fqn = $1;
        if ($cli_fqn =~ /[^\.]\.(.*)/) {
            my $temp = $1;
            chop($temp);
            $CLI_DOMAIN = $temp;
        }
    }

    # If $CLI_DOMAIN is not filled by now, croak.
    if ($CLI_DOMAIN =~ /^[ \t]*$/) {
        croak(    "Serious issue with the clients. Can run host command or invalid ip,"
                . " host command output didnot have the client name:"
                . $opts{client_object}->{client});
    }

    #Generate the krb5.conf file dynamically instead of taking it as an input
    my $kdc_realm       = $opts{kdc}->{kdc_realm};
    my $kdc_upper_realm = uc($opts{kdc}->{kdc_realm});
    my $kdc_fqn         = lc($opts{kdc}->{kdc_fqn});
    my $file_string     = '';
    my $enc_type        = $opts{kdc}->{kdc_enc_type};
    my $lc_dns_domain   = lc($opts{dns_domain});

    #Check if the filer and client are in different DNS
    my $insert_string = "";
    if ($CLI_DOMAIN ne $opts{dns_domain}) {
        my $lc_cli = lc($CLI_DOMAIN);

        $insert_string = <<"EOT";
    $lc_cli  = $kdc_upper_realm
    .$lc_cli = $kdc_upper_realm
EOT

    }

    $Log->comment(qq{CLI_DOMAIN=$CLI_DOMAIN, opts{dns_domain}=$opts{dns_domain}});
    $Log->comment(qq{Insert string = $insert_string});

    # This is the content of the krb5.conf file for each client
    $file_string = << "EOT";

[logging]
default = FILE:/var/log/krb5libs.log
kdc = FILE:/var/log/krb5kdc.log
admin_server = FILE:/var/log/kadmind.log

[libdefaults]
default_realm = $kdc_upper_realm
default_tgs_enctypes = $enc_type
default_tkt_enctypes = $enc_type
dns_lookup_realm = false
dns_lookup_kdc = false
allow_weak_crypto = yes

[realms]
$kdc_upper_realm = {
    kdc = $kdc_fqn:$::KDC_SERVICE_PORT
    admin_server = $kdc_fqn:$::KADMIN_SERVICE_PORT
    default_domain =  $kdc_upper_realm
}

[domain_realm]
    $kdc_realm = $kdc_upper_realm
   .$kdc_realm = $kdc_upper_realm
$insert_string    $opts{dns_domain} = $kdc_upper_realm
   .$opts{dns_domain} = $kdc_upper_realm

EOT

    #remove the previous version of the file since the backup has been created and add the new file
    client_execute("\\rm -rf /$KRB5_PATH/krb5.conf;echo \"$file_string\" > /$KRB5_PATH/krb5.conf", $opts{client_execute});
    my $krb_config_output = client_execute("ls -l /$KRB5_PATH/krb5.conf");
    my $config_file_name  = "krb5.conf";
    if ($krb_config_output !~ /$config_file_name/) {
        croak("Failed to create krb5.conf file in /$KRB5_PATH location");
    }
    client_execute("cat /$KRB5_PATH/krb5.conf");

    load_kerberos_module($opts{client_object});

    return_client();
    $Log->exit();
    return 1;
} ## end sub client_krb5_config_create( % )

=head2 MIT_client_krb5_config_create

The krb5.conf file is created during run time and copied into /$KRB5_PATH/krb5.conf for the MIT Setup.

MIT_client_krb5_config_create( kdc => \%<Kerberos-hash-details> , client_object => $<APISet-client-object> , dns_domain => $<Filer-DNS_DOMAIN> , [sudo_path => $<sudo-with-fully-qualify-path>] );

Optional: sudo_path

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item dns_domain

(Required) : Dns domain, to which the filer has been configured

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub MIT_client_krb5_config_create( % ) {
    my %opts = validate(
        @_,
        {   kdc           => {type => HASHREF},
            client_object => {type => OBJECT},
            dns_domain    => {type => SCALAR},
            sudo_path     => {type => SCALAR, default => $default_values{sudo_path}},
        }
    );
    $Log->enter();

    #validate the kdc hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    check_client_and_sudo($opts{client_object});
    check_backup_directory(object => $opts{client_object});

    if ($opts{dns_domain} eq "") {
        croak("DNS domain of the filer is needed for the subroutine");
    }
    my ($CLI_DOMAIN, $KRB5_PATH);
    $KRB5_PATH = verify_client_hosttype($opts{client_object});

    #Create backup files
    backup_files("/$KRB5_PATH/krb5.conf");

    #obtain the host information either by executing it in the krb client or in the client running the script
    my $tempSpn = client_execute("host $opts{ client_object }->{ hostrec }->{default_ip} ");

    #the output should contain the client name, otherwise execute it on the client
    #running the script
    if ($tempSpn !~ /.* $opts{ client_object }->{ client }.(.*)/) {
        $tempSpn = `host $opts{ client_object }->{ hostrec }->{default_ip}`;
    }

    #the output should contain the client name
    if ($tempSpn =~ /.* $opts{ client_object }->{ client }.(.*)/) {
        my $temp = $1;
        chop($temp);
        $CLI_DOMAIN = $temp;
    } else {
        croak("Serious issue with the clients. Can't run host command or ip is not resolvable.");
    }

    #Generate the krb5.conf file dynamically instead of taking it as an input
    my $file_string = "";
    $file_string .= "[logging]\n";
    $file_string .= "default = FILE:/var/log/krb5libs.log \n";
    $file_string .= "kdc = FILE:/var/log/krb5kdc.log \n";
    $file_string .= "admin_server = FILE:/var/log/kadmind.log \n\n";

    $file_string .= "[libdefaults]\n";
    $file_string .= "default_realm = " . uc($opts{kdc}->{kdc_realm}) . "\n";
    $file_string .= "dns_lookup_realm = false \n";
    $file_string .= "dns_lookup_kdc = false \n";
    $file_string .= "ticket_lifetime = 24h \n";
    $file_string .= "forwardable = yes \n\n";

    $file_string .= "[realms]\n";
    $file_string .= uc($opts{kdc}->{kdc_realm}) . " = { \n";
    $file_string .= "  kdc = " . lc($opts{kdc}->{kdc_fqn}) . ":" . $::KDC_SERVICE_PORT . " \n";
    $file_string .= "  admin_server = " . lc($opts{kdc}->{kdc_fqn}) . ":" . $::KADMIN_SERVICE_PORT . " \n";
    $file_string .= "  default_domain = " . uc($opts{kdc}->{kdc_domain}) . " \n} \n\n";

    $file_string .= "[domain_realm]\n";
    $file_string .= lc($opts{kdc}->{kdc_realm}) . " = " . uc($opts{kdc}->{kdc_realm}) . " \n";
    $file_string .= "." . lc($opts{kdc}->{kdc_realm}) . " = " . uc($opts{kdc}->{kdc_realm}) . " \n";

    #Check if the filer and client are in different DNS
    if ($CLI_DOMAIN ne $opts{dns_domain}) {
        $file_string .= lc($CLI_DOMAIN) . " = " . uc($opts{kdc}->{kdc_realm}) . " \n";
        $file_string .= "." . lc($CLI_DOMAIN) . " = " . uc($opts{kdc}->{kdc_realm}) . " \n";
    }

    $file_string .= lc($opts{dns_domain}) . " = " . uc($opts{kdc}->{kdc_realm}) . " \n";
    $file_string .= "." . lc($opts{dns_domain}) . " = " . uc($opts{kdc}->{kdc_realm}) . " \n\n";

    $file_string .= "[appdefaults]\n";
    $file_string .= "pam = {\n";
    $file_string .= "   debug = false\n";
    $file_string .= "   ticket_lifetime = 36000\n";
    $file_string .= "   renew_lifetime = 36000\n";
    $file_string .= "   forwardable = true\n";
    $file_string .= "   krb4_convert = false\n";
    $file_string .= "}\n\n";

    #remove the previous version of the file since the backup has been created and add the new file
    client_execute("\\rm -rf /$KRB5_PATH/krb5.conf;echo \"$file_string\" > /$KRB5_PATH/krb5.conf");
    my $krb_config_output = client_execute("ls -l /$KRB5_PATH/krb5.conf");
    my $config_file_name  = "krb5.conf";
    if ($krb_config_output !~ /$config_file_name/) {
        croak("Failed to create krb5.conf file in /$KRB5_PATH location");
    }
    client_execute("cat /$KRB5_PATH/krb5.conf");

    return_client();
    $Log->exit();
    return 1;
} ## end sub MIT_client_krb5_config_create( % )

=head2 MIT_verify_server_setup

This subroutine verifies whether the required files for Kerberos, are installed in the MIT server

MIT_verify_server_setup( kdc => \%<Kerberos-hash-details> , [sudo_path => $<sudo-with-fully-qualify-path>] );

Optional: sudo_path

=over

=item Arguments

=over

=item kdc_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub MIT_verify_server_setup( % ) {
    my %opts = validate(
        @_,
        {   kdc_object => {type => OBJECT},
            sudo_path  => {type => SCALAR, default => $default_values{sudo_path}},
        }
    );
    $Log->enter();

    check_client_and_sudo($opts{kdc_object});

    #The MIT server should have all the 3 files given below
    my $output = $opts{kdc_object}->execute(command => "yum list installed '*krb*'");
    if (!($output =~ /krb5-libs/ && $output =~ /krb5-workstation/)) {
        croak("Server doesn't contain the required installation files, expecting: krb5-libs, krb5-server, and krb5-workstation");
    }

    return_client();

    $Log->exit();
    return 1;
} ## end sub MIT_verify_server_setup( % )

=head2 verify_client_installation

This subroutine verifies whether the required files for Kerberos, are installed in the MIT client

verify_client_installation( client_object => \%<Client-hash-details> , [sudo_path => $<sudo-with-fully-qualify-path>] );

Optional: sudo_path

=over

=item Arguments

=over

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub verify_client_installation( % ) {
    my %opts = validate(
        @_,
        {   client_object => {type => OBJECT},
            sudo_path     => {type => SCALAR, default => $default_values{sudo_path}},
        }
    );
    $Log->enter();

    check_client_and_sudo($opts{client_object});

    #The MIT server should have all the 2 files given below
    my $output = $opts{client_object}->execute(
        command => "yum list installed '*krb*'",
        timeout => 300
    );
    if (!($output =~ /krb5-libs/ && $output =~ /krb5-workstation/)) {
        croak("Client doesn't contain the required installation files, we need to install krb5-libs and krb5-workstation packages");
    }

    return_client();

    $Log->exit();
    return 1;
} ## end sub verify_client_installation( % )

=head2 MIT_verify_kinit

This subroutine validates the MIT setup on both the Kerberos client and server by performing kinit

MIT_verify_kinit( kdc => \%<Kerberos-hash-details> , object => <NACL::C::Client> object ,client_match => 0 or 1 , [sudo_path => $<sudo-with-fully-qualify-path>] );

Optional: sudo_path

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item object

(Required) : NACL::C::Client object has to be passed into the subroutine. If executed from server then server object
             or if executed from client, then client object

=item client_match

(Optional) : This indicates whether the subroutine is executed on MIT KDC client or server. default : 1 (client)

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub MIT_verify_kinit( % ) {
    my %opts = validate(
        @_,
        {   kdc          => {type => HASHREF},
            object       => {type => OBJECT},
            client_match => {
                type     => SCALAR,
                default  => 1,
                optional => 1,
                regex    => qr/[01]/
            },
            sudo_path => {type => SCALAR, default => $default_values{sudo_path}},
        }
    );
    $Log->enter();

    #validate the kdc hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    check_client_and_sudo($opts{object});

    $opts{object}->execute(command => "kdestroy");

    #kinit when issued, provides a password prompt which can contain either admin or root
    my $root_match_string  = "Password for " . kdc_root_principal($opts{kdc}->{kdc_realm});
    my $admin_match_string = "Password for " . kdc_admin_principal($opts{kdc}->{kdc_realm});

    $opts{object}->execute(
        command     => "kinit",
        match_table => [
            $root_match_string  => $::MIT_HOST_PASSWD,
            $admin_match_string => $opts{kdc}->{kdc_passwd},
        ]
    );

    #verify that the ticket was issued. This would indicate that the client/ server setup is successful
    my $klist_output = $opts{object}->execute(command => "klist");
    my $klist = "krbtgt/" . uc($opts{kdc}->{kdc_realm}) . "@" . uc($opts{kdc}->{kdc_realm});
    if ($klist_output !~ /$klist/) {
        croak("Unable to obtain ticket from the KDC. May be a config issue");
    }

    return_client();

    $Log->exit();
    return 1;
} ## end sub MIT_verify_kinit( % )

=head2 MIT_add_client_principal_in_KDC

This function adds the client principal into the MIT KDC server

MIT_add_client_principal_in_KDC( kdc_object => $<AD-Server-object> , kdc => \%<Kerberos-hash-details> , client => \%<Client-hash-details> , [sudo_path => $<sudo-with-fully-qualify-path>] );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client

(Required) : Reference to the Client hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item kdc_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub MIT_add_client_principal_in_KDC( % ) {
    my %opts = validate(
        @_,
        {   kdc        => {type => HASHREF},
            client     => {type => HASHREF},
            kdc_object => {type => OBJECT},
            sudo_path  => {type => SCALAR, default => $default_values{sudo_path}},
        }
    );
    $Log->enter();

    #validate the kdc and client hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_client_hash($opts{client});

    check_client_and_sudo($opts{kdc_object});

    $opts{kdc_object}->execute(command => "service kadmin start");
    $opts{kdc_object}->execute(command => "service krb5kdc start");

    MIT_KDC_verify_principal("$::MIT_KDC_ROOT/$::MIT_KDC_ADMIN", $opts{kdc_object}, $::REALM);

    # Add the root and host principal of the client machines in MIT server
    my $host_principal = get_host_principal($opts{client}->{client_fqn});
    MIT_KDC_add_principal($host_principal, $opts{kdc_object}, $::MIT_HOST_PASSWD, $::REALM);

    my $root_principal = get_root_principal($opts{client}->{client_fqn});
    MIT_KDC_add_principal($root_principal, $opts{kdc_object}, $::MIT_HOST_PASSWD, $::REALM);

    if ($opts{kdc}->{kdc_user} ne $::MIT_KDC_ROOT) {
        my $user_principal = $opts{kdc}->{kdc_user};
        MIT_KDC_add_principal($user_principal, $opts{kdc_object}, $::MIT_HOST_PASSWD, $::REALM);
    }

    _exit_kadmin_prompt($opts{kdc_object});
    return_client();

    $Log->exit();
    return 1;
} ## end sub MIT_add_client_principal_in_KDC( % )

=head2 MIT_delete_client_principal_from_KDC

This function deletes the client principal from the MIT KDC server

MIT_delete_client_principal_from_KDC( kdc_object => $<AD-Server-object> , kdc => \%<Kerberos-hash-details> , client => \%<Client-hash-details> , [sudo_path => $<sudo-with-fully-qualify-path>] );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client

(Required) : Reference to the Client hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item kdc_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub MIT_delete_client_principal_from_KDC {
    my %opts = validate(
        @_,
        {   kdc        => {type => HASHREF},
            client     => {type => HASHREF},
            kdc_object => {type => OBJECT},
            sudo_path  => {type => SCALAR, default => $default_values{sudo_path}},
        }
    );
    $Log->enter();

    #validate the kdc and client hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_client_hash($opts{client});

    check_client_and_sudo($opts{kdc_object});

    # Delete host and root principal in the Krb client
    my $host_principal = get_host_principal($opts{client}->{client_fqn});
    my $root_principal = get_root_principal($opts{client}->{client_fqn});
    my $host_flag      = 0;

    MIT_KDC_delete_principal($host_principal, $opts{kdc_object}, $opts{kdc}->{kdc_realm});
    MIT_KDC_delete_principal($root_principal, $opts{kdc_object}, $opts{kdc}->{kdc_realm});
    if ($opts{kdc}->{kdc_user} ne $::MIT_KDC_ROOT) {
        my $user_principal = $opts{kdc}->{kdc_user};
        MIT_KDC_delete_principal($user_principal, $opts{kdc_object}, $opts{kdc}->{kdc_realm});
    }

    return_client();
    $Log->exit();
    return 1;
} ## end sub MIT_delete_client_principal_from_KDC

=head2 MIT_add_host_principal_in_client

This function adds the host principal into the MIT KDC client

MIT_add_host_principal_in_client( client_object => $<AD-Server-object> , kdc => \%<Kerberos-hash-details> , client => \%<Client-hash-details> , [sudo_path => $<sudo-with-fully-qualify-path>] );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client

(Required) : Reference to the Client hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub MIT_add_host_principal_in_client( % ) {
    my %opts = validate(
        @_,
        {   kdc           => {type => HASHREF},
            client        => {type => HASHREF},
            client_object => {type => OBJECT},
            sudo_path     => {type => SCALAR, default => "$default_values{sudo_path}"},
        }
    );
    $Log->enter();

    #validate the kdc and client hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_client_hash($opts{client});

    check_backup_directory(object => $opts{client_object});
    check_client_and_sudo($opts{client_object});

    my $root_match_string       = "Password for root@" . uc($opts{kdc}->{kdc_realm});
    my $root_admin_match_string = "Password for " . $opts{kdc}->{kdc_user} . "/" . $::MIT_KDC_ADMIN . "@" . uc($opts{kdc}->{kdc_realm});
    my $KRB5_PATH               = verify_client_hosttype($opts{client_object});
    backup_files("/$KRB5_PATH/krb5.keytab");
    client_execute("\\rm -f /$KRB5_PATH/krb5.keytab");

    #remove the previous keytabs and add entries into the new keytab
    $opts{client_object}->execute(
        command            => "kadmin -p $::MIT_KDC_ROOT/$::MIT_KDC_ADMIN",
        additional_prompts => ['kadmin'],
        match_table        => [
            $root_match_string       => $::MIT_HOST_PASSWD,
            $root_admin_match_string => $::MIT_HOST_PASSWD,
            "Password"               => $::MIT_HOST_PASSWD,
            ]    ## end

    );

    # Add the host and root entries into the keytab files
    my $host_principal = $::MIT_KDC_HOST . "/" . $opts{client}->{client_fqn};
    $opts{client_object}->execute(
        command            => "ktadd -k /$KRB5_PATH/krb5.keytab $host_principal",
        additional_prompts => ['kadmin']
    );

    my $host_output = $opts{client_object}->execute(
        command            => "getprinc $host_principal",
        additional_prompts => ['kadmin']
    );

    if (   $host_output =~ /Principal does not exist/
        || $host_output !~ /$host_principal/) {
        croak("Host principal could not be added into the MIT KDC Client");
    }

    my $root_principal = get_root_principal($opts{client}->{client_fqn});
    $opts{client_object}->execute(
        command            => "ktadd -k /$KRB5_PATH/krb5.keytab $root_principal",
        additional_prompts => ['kadmin']
    );

    my $root_output = $opts{client_object}->execute(
        command            => "getprinc $root_principal",
        additional_prompts => ['kadmin']
    );

    if (   $root_output =~ /Principal does not exist/
        || $root_output !~ /$root_principal/) {
        croak("Root principal could not be added into the MIT KDC Client");
    }

    _exit_kadmin_prompt($opts{client_object});
    return_client();

    $Log->exit();
    return 1;
} ## end sub MIT_add_host_principal_in_client( % )

=head2 MIT_delete_principal_from_client

This function deletes the host principal from the MIT KDC client

MIT_delete_principal_from_client( client_object => $<AD-Server-object> , kdc => \%<Kerberos-hash-details> , client => \%<Client-hash-details> , [sudo_path => $<sudo-with-fully-qualify-path>] );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client

(Required) : Reference to the Client hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub MIT_delete_principal_from_client {
    my %opts = validate(
        @_,
        {   kdc           => {type => HASHREF},
            client        => {type => HASHREF},
            client_object => {type => OBJECT},
            sudo_path     => {type => SCALAR, default => "$default_values{sudo_path}"},
        }
    );
    $Log->enter();

    #validate the kdc and client hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_client_hash($opts{client});
    check_client_and_sudo($opts{client_object});

    # After issuing kadmin.local a password prompt would be given to the user
    my $root_match_string       = "Password for root@" . uc($opts{kdc}->{kdc_realm});
    my $root_admin_match_string = "Password for " . $opts{kdc}->{kdc_user} . "/" . $::MIT_KDC_ADMIN . "@" . uc($opts{kdc}->{kdc_realm});
    my $KRB5_PATH               = verify_client_hosttype($opts{client_object});

    $opts{client_object}->execute(
        command            => "kadmin",
        additional_prompts => ['kadmin'],
        match_table        => [
            $root_match_string       => $::MIT_HOST_PASSWD,
            $root_admin_match_string => $::MIT_HOST_PASSWD,
            "Password"               => $::MIT_HOST_PASSWD,
        ]
    );

    #delete the host and root principal's in the client
    my $host_principal = get_host_principal($opts{client}->{client_fqn});
    my $root_principal = get_root_principal($opts{client}->{client_fqn});
    my $host_flag      = 0;

    my $host_output = $opts{client_object}->execute(
        command            => "getprinc $host_principal",
        additional_prompts => ['kadmin']
    );

    # If the script execution fails before adding the host or root principal,
    # that needs to be handled by the library
    if (   $host_output =~ /Principal does not exist/
        || $host_output !~ /$host_principal/) {
        $Log->comment("Host principal could not be found in the MIT KDC Client");
        $host_flag = 1;
    }

    if (!$host_flag) {
        my $delete_princ     = "Are you sure you want to delete the principal ";
        my $principal_output = $opts{client_object}->execute(
            command            => "delprinc $host_principal",
            additional_prompts => ['kadmin'],
            match_table        => [$delete_princ => "yes",]
        );

        my $princ_msg = "Principal \"" . $host_principal . "\@" . uc($opts{kdc}->{kdc_realm}) . "\" deleted";
        if ($principal_output !~ /$princ_msg/) {
            $Log->comment("Unable to delete principle from the client : $principal_output ");
        }
    }

    my $root_flag = 0;

    my $root_output = $opts{client_object}->execute(
        command            => "getprinc $root_principal",
        additional_prompts => ['kadmin']
    );

    if (   $root_output =~ /Principal does not exist/
        || $root_output !~ /$root_principal/) {
        $Log->comment("Host principal could not be found in the MIT KDC Client");
        $root_flag = 1;
    }

    if (!$root_flag) {
        my $delete_princ     = "Are you sure you want to delete the principal ";
        my $principal_output = $opts{client_object}->execute(
            command            => "delprinc $root_principal",
            additional_prompts => ['kadmin'],
            match_table        => [$delete_princ => "yes",]
        );

        my $princ_msg = "Principal \"" . $root_principal . "\@" . uc($opts{kdc}->{kdc_realm}) . "\" deleted";
        if ($principal_output !~ /$princ_msg/) {
            $Log->comment("Unable to delete principle from the client : $principal_output ");
        }
    }

    _exit_kadmin_prompt($opts{client_object});
    return_client();

    $Log->exit();
    return 1;
} ## end sub MIT_delete_principal_from_client

=head2 MIT_Server_Setup

This function performs complete verification, setup and validation of the MIT KDC server

MIT_Server_Setup( kdc_object => $<AD-Server-object> , kdc => \%<Kerberos-hash-details> , client => \%<Client-hash-details> );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client

(Required) : Reference to the Client hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item kdc_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub MIT_Server_Setup( % ) {
    my %opts = validate(
        @_,
        {   kdc        => {type => HASHREF},
            client     => {type => HASHREF},
            kdc_object => {type => OBJECT},
        }
    );
    $Log->enter();

    #validate the kdc and client hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_client_hash($opts{client});

    my $date = $opts{kdc_object}->execute(command => "date");

    $Log->comment("=====================================================");
    $Log->comment("date on $opts{ kdc }->{ kdc_server } : $date");
    $Log->comment("=====================================================");

    # Verify if the server has the required files, add the required principal's
    # then issue kinit to verify the setup
    MIT_verify_server_setup(
        kdc_object => $opts{kdc_object},
        sudo_path  => $opts{client}->{sudo_path}
    );

    MIT_add_client_principal_in_KDC(
        kdc        => $opts{kdc},
        client     => $opts{client},
        kdc_object => $opts{kdc_object},
        sudo_path  => $opts{client}->{sudo_path}
    );

    MIT_verify_kinit(
        kdc          => $opts{kdc},
        object       => $opts{kdc_object},
        client_match => 0,
        sudo_path    => $opts{client}->{sudo_path}
    );

    $Log->exit();
    return 1;
} ## end sub MIT_Server_Setup( % )

=head2 verify_client_hosttype

This subroutine verifies the hosttype of the client_object

KDC_Server_Setup( kdc => \%<Kerberos-hash-details> , client => $<Client-name> );

=over

=item Arguments

=over

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=back

=item Return

KRB5_PATH on success or croaks on failure

=back

=cut

sub verify_client_hosttype( $ ) {
    my $client_object = shift;
    $Log->enter();

    check_client_object($client_object);

    # The hosttype of the client has to be either unix or solaris
    if ($client_object->type() =~ /unix-linux/) {
        return "etc";
    } elsif ($client_object->type() =~ /unix-solaris/) {
        return "etc/krb5";
    } else {
        croak("Invalid hosttype detect: $client_object->type()");
    }

    $Log->exit();
} ## end sub verify_client_hosttype( $ )

=head2 kerberos_create_dc

This subroutine creates the directory connection (DC) using the kdc realm

KDC_Server_Setup( kdc_realm => $<REALM-name> );

=over

=item Arguments

=over

=item kdc_realm

(Required) : REALM to which KDC belongs to

=back

=item Return

DC corresponding to the realm or croaks on failure

=back

=cut

sub kerberos_create_dc( % ) {
    my %opts = validate(@_, {kdc_realm => {type => SCALAR},});
    $Log->enter();

    if ($opts{kdc_realm} eq "") {
        croak("KDC REALM parameter was not passed into the subroutine");
    }

    # Change the realm into DC. ex: nfsqa-cifs.rtp.netapp.com becomes
    # dc=nfsqa-cifs,dc=rtp,dc=netapp,dc=com
    my $realm = uc($opts{kdc_realm});
    my @dc = split('\.', $realm);
    my @dc_list;
    push(@dc_list, "dc=$_,") foreach (@dc);
    my $DC = join('', @dc_list);
    chop($DC);

    $Log->exit();
    return $DC;
} ## end sub kerberos_create_dc( % )

=head2 client_rpcgssd_restart

This function restarts the rpcgssd daemon on the Kerberos client and verifies whether the rpcgssd was restarted successfully.

client_rpcgssd_restart( client_object => $<client-object>, [sudo_path => $<sudo-with-fully-qualify-path>] );

Optional: sudo_path

=over

=item Arguments

=over

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on Success or croaks on failure

=back

=cut

sub client_rpcgssd_restart( % ) {
    my %opts = validate(
        @_,
        {   client_object => {type => OBJECT},
            sudo_path     => {type => SCALAR, default => "$default_values{sudo_path}"},
        }
    );
    $Log->enter();

    check_client_and_sudo($opts{client_object});

    if ($opts{client_object}->type() =~ /unix-linux/) {
        eval { client_execute("/bin/systemctl restart nfs-secure.service"); };
        if ($@) {
            if ($@ =~ /No such file or directory/) {
                _restart_service("rpcgssd");
            } else {
                croak(qq{Unexpected error in restarting nfs service: $@});
            }
        }
    } elsif ($opts{client_object}->type() =~ /unix-solaris/) {
        $Log->comment("Restart the services /etc/init.d/nfs.server");

        # Procedure: Stop the rpcgssd daemon, verify it and then issue start
        client_execute("/etc/init.d/nfs.server stop");
        client_execute("/etc/init.d/nfs.server start");
        $Log->comment("Restart the services svc:/network/rpc/gss:default");

        # Procedure: Stop the rpcgssd daemon, verify it and then issue start
        client_execute("svcadm -v disable svc:/network/rpc/gss:default");

        #The solaris setup code was taken from the Interop script.
        #And basically the sleep would be to allow some time between disable and enable.
        sleep(4);
        my $gssd_chk = client_execute("svcadm -v enable svc:/network/rpc/gss:default");

        if ($gssd_chk !~ /svc:\/network\/rpc\/gss:default enabled/) {
            croak("Failed to start rpcgssd services");
        }

    } else {    # when all else fails, script fails
        croak("Invalid hosttype detected in client_rpc:0: $opts{ client_object }->type()");
    }

    client_exit_sudo();
    reset_client_object();
    $Log->exit();
    return 1;
} ## end sub client_rpcgssd_restart( % )

=head2 client_rpcidmapd_restart

This function restarts the rpcidmap daemon on the Kerberos client and verifies whether the rpcidmapd was restarted successfully.

client_rpcidmapd_restart( client_object => $<client-object>, [sudo_path => $<sudo-with-fully-qualify-path>] );

Optional: sudo_path

=over

=item Arguments

=over

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on Success or croaks on failure

=back

=cut

sub client_rpcidmapd_restart( % ) {
    my %opts = validate(
        @_,
        {   client_object => {type => OBJECT},
            sudo_path     => {type => SCALAR, default => "$default_values{sudo_path}"},
        }
    );
    $Log->enter();

    check_client_and_sudo($opts{client_object});

    if ($opts{client_object}->type() =~ /unix-linux/) {
        eval { client_execute("/bin/systemctl restart nfs-idmap.service"); };
        if ($@) {
            if ($@ =~ /No such file or directory/) {
                _restart_service("rpcidmapd");
            } else {
                croak(qq{Unexpected error in restarting idmapd service: $@});
            }
        }
    } elsif ($opts{client_object}->type() =~ /unix-solaris/) {

        # Procedure: Stop the rpcidmapd daemon, verify it and then issue start
        client_rpcgssd_restart(client_object => $opts{client_object});
    } else {    # when all else fails, failure is bound to occur
        croak("Invalid hosttype detected in client_rpc:0: $opts{ client_object }->type()");
    }

    return_client();
    $Log->exit();
    return 1;
} ## end sub client_rpcidmapd_restart( % )

sub check_backup_directory( % ) {
    my %opts = validate(
        @_,
        {   object    => {type => OBJECT},
            sudo_path => {type => SCALAR, default => "$default_values{sudo_path}"},
        }
    );
    $Log->enter();

    check_client_and_sudo($opts{object});

    #verify if the backup directory is present, if not create it.
    eval { my $directory_output = client_execute("\\ls -l $::BACKUP_DIRECTORY"); };

    if ($@ =~ /No such file or directory/) {
        client_execute("\\mkdir $::BACKUP_DIRECTORY");
        client_execute("\\ls -l $::BACKUP_DIRECTORY");
    }

    return_client();
    $Log->exit();
    return;
} ## end sub check_backup_directory( % )

=head2 client_etc_hosts_modify

The /etc/hosts file has to be modified to contain the Server name ( filer ) and the KDC AD name, if not present.

client_etc_hosts_modify( kdc => \%<KDC-Details> , filer_details => \%<Filer-Details> , client_object => $<client-object>, [sudo_path => $<sudo-with-fully-qualify-path>] );

Optional: sudo_path

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item filer_details

(Required) : Reference to the filer hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub client_etc_hosts_modify( % ) {
    my %opts = validate(
        @_,
        {   kdc           => {type => HASHREF},
            filer_details => {type => HASHREF},
            client_object => {type => OBJECT},
            sudo_path     => {type => SCALAR, default => "$default_values{sudo_path}"},
        }
    );
    $Log->enter();

    check_client_and_sudo($opts{client_object});

    #validate the kdc and filer hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_filer_hash($opts{filer_details});

    my $KRB5_PATH = "etc";

    $Log->comment("Creating backup file for /etc/hosts");
    check_backup_directory(object => $opts{client_object});

    my $client_etc_hosts = client_execute("cat /$KRB5_PATH/hosts");

    # create the backup files for hosts
    backup_files("/$KRB5_PATH/hosts");

    $Log->comment("Update the /etc/hosts file of the Filer, KDC and Client");

    #execute host command either on the krb client or on the machine where
    #the script is running
    #    my $cmd_output = get_fqn("junk", $opts{ filer_details}->{ filer_mount_ip }, 0);
    #    $opts{filer_details}->{filer_fqn} = $cmd_output;

    # I have commented this section, since we are not going to use the same hostname for
    # Kerberos-config modify command. We are generating our own hostname on the fly and
    # use that - to limit ourselves to 15 character limits.
    #    my $lif_name;
    #    if ($cmd_output =~ /(.*).$opts{ filer_details }->{ filer_dns }/) {
    #        $lif_name = $1;
    #    }
    #
    #   # If the lif name was not returned, possible cause of failure could be that
    #   # the dns provided was wrong. So, obtain the dns and lif name.
##    if ($lif_name eq "") {
##        $lif_name = KRB_obtain_filer_lif_name(
##            filer_details => $opts{filer_details},
##            cmd_output    => $cmd_output
##        );
##    }

    # Get a hostname for the data lif ip.
    my $filer_fqn_short_name = _obtain_ip_host_name($opts{filer_details}->{filer_mount_ip});

    #my $filer_fqn_short_name = (split(/\./, $opts{filer_details}->{filer_fqn}))[0];
    # Adding the filer and KDC entry in client /etc/hosts file
    if ($client_etc_hosts =~ /$opts{filer_details}->{filer_mount_ip}/) {
        eval {
            client_execute(qq{\\grep -v '$opts{filer_details}->{filer_mount_ip}\\|^\$' /etc/hosts > /etc/hosts_});
            client_execute(qq{\\mv /etc/hosts_ /etc/hosts});
        };
        if ($@) {
            $Log->warn(qq{Ignoring a possible error from grep. This can be ignored, if there is no matching line present in the /etc/hosts});
        }
    }
    my $command = "echo '$opts{ filer_details }->{ filer_mount_ip } $filer_fqn_short_name' >> /etc/hosts ";
    client_execute($command);

    if ($client_etc_hosts =~ /$opts{kdc}->{kdc_ip}/) {
        eval {
            client_execute(qq{\\grep -v '$opts{kdc}->{kdc_ip}\\|^\$' /etc/hosts > /etc/hosts_});
            client_execute(qq{\\mv /etc/hosts_ /etc/hosts});
        };
        if ($@) {
            $Log->warn(qq{Ignoring a possible error from grep. This can be ignored, if there is no matching line present in the /etc/hosts});
        }
    }
    $command = "echo '$opts{ kdc }->{ kdc_ip } $opts{kdc}->{kdc_fqn} $opts{kdc}->{kdc_server}' >> /etc/hosts";
    client_execute($command);

    client_execute("cat /etc/hosts");
    return_client();

    $Log->exit();
    return 1;
} ## end sub client_etc_hosts_modify( % )

=head2 validate_kdc_hash

This subroutine verifies whether the kdc hash passed into the function contains NULL value for any of the keys
and whether it conatins KDC_HASH_COUNT entries

validate_kdc_hash( kdc => \%<KDC-Details> );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub validate_kdc_hash( $ ) {
    my ($kdc, %kdc_hash);
    $kdc = shift;
    $Log->enter();

    if ($kdc eq '') {
        croak("KDC hash was expected, but no parameter was obtained by the script");
    }

    %kdc_hash = %{$kdc};
    my @list_of_kdc_keys = keys(%kdc_hash);

    my @list_of_keys = qw{kdc_server kdc_user kdc_passwd kdc_domain kdc_realm kdc_enc_type kdc_ip kdc_fqn kdc_ou};
    if ($#list_of_kdc_keys < $#list_of_keys) {
        croak("The subroutine expects " . ($#list_of_keys + 1) . " <key,value> pairs in the KDC hash and was sent:\n" . Dumper(\%kdc_hash));
    }
    foreach my $key (@list_of_keys) {
        if ($kdc_hash{$key} eq "") {
            croak("NULL value was passed in for the kdc hash. key:$key value:$kdc_hash{$key}");
        }
    }

    $Log->exit();
    return;
} ## end sub validate_kdc_hash( $ )

=head2 KRB_LDAP_uid_modify

The UID of the non root user has to be modified in the AD LDAP so that when
a non root user, wants to use the Kerberos Library and perform operations
user credentials will be obtained from the AD LDAP.

KRB_LDAP_uid_modify( user => $<Username> , kdc => \%<KDC-details> );

=over

=item Arguments

=over

=item kdc_AD

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item user

(Required) : Username of the user whose entries are to be added into the files so that mount would succeed

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub KRB_LDAP_uid_modify( % ) {
    my %opts = validate(
        @_,
        {   kdc_AD => {type => HASHREF},
            user   => {type => SCALAR},
        }
    );
    $Log->enter();

    if ($opts{user} eq '') {
        croak(" The subroutine expects a valid username");
    }

    # Establish the NACL::C::Client object to the KDC server
    establish_kdc_connection($opts{kdc_AD}->{kdc_server});

    # Making sure the server handle retrieved is a windows client.
    if (!$KDC_Connection_Obj->is_windows()) {
        croak("KDC_Server_Setup : Incorrect or invalid client is detected for the KDC server:\n$opts{ kdc }->{ kdc_server }");
    }

    my $DC = kerberos_create_dc(kdc_realm => $opts{kdc_AD}->{kdc_realm});

    # Add client name in AD computers account & Create Service Principal and verify that is registered
    my $base_dn        = "$opts{ kdc_AD }->{ kdc_ou },$DC";
    my $user_principal = "cn=$opts{ user },$base_dn";
    my $AD_user_create = $KDC_Connection_Obj->execute(command => "dsget user \"$user_principal\"");

    #verify if the dsget user succeeded, if so delete the previous entry
    if ($AD_user_create =~ /dsget succeeded/) {
        $KDC_Connection_Obj->execute(command => "dsrm -noprompt \"$user_principal\"");
    }
    my $user_create_output = $KDC_Connection_Obj->execute(command => "dsadd user \"$user_principal\"");

    #verify  if the dsget user failed then abort the script exection
    $AD_user_create = $KDC_Connection_Obj->execute(command => "dsget user \"$user_principal\"");
    if ($AD_user_create !~ /dsget succeeded/) {
        croak("AD user creation failed : $user_create_output");
    }

    #make an LDAP connection to the windows AD
    my $ldap = Net::LDAP->new($opts{kdc_AD}->{kdc_fqn}, port => $::LDAP_PORT)
        or die "$@";
    my $dn = $opts{kdc_AD}->{kdc_user} . ',' . $base_dn;
    my $message = $ldap->bind(dn => $dn, password => $opts{kdc_AD}->{kdc_passwd});
    if ($message->code) {    # if bind fails then fail the script
        croak("Failure during LDAP bind: $message->error");
    }

    #search if the kdc_admin_user is present in LDAP, if so
    #obtain its group ID
    my $groupID;
    my $result = $ldap->search(
        base   => $base_dn,
        scope  => 'sub',
        filter => "(& (ObjectClass=user) ($opts{kdc_AD}->{kdc_user}) )",
        attrs  => ['cn', 'primaryGroupID', 'uid'],
    );

    if ($result->entries) {
        foreach my $entry ($result->entries) {
            $groupID = ${$entry->get('primaryGroupID')}[0];
        }
    } else {
        croak("The requested user : $opts{kdc_AD}->{kdc_user} was not found\n");
    }

    #search for the user that was added before and modify its UID
    $result = $ldap->search(
        base   => $base_dn,
        scope  => 'sub',
        filter => "(| (objectclass => user ) (cn=$opts{user}) )",
        attrs  => ['cn', 'primaryGroupID', 'uid'],
    );

    my $id_message = `id $opts{user}`;
    my $user_id;

    # Check if the id <user> output contains the UID of the user
    if ($id_message =~ /uid=(\d+)\($opts{user}\)/) {
        $user_id = $1;
    } else {    # 2 issues. 1. unable to run the command or 2. invalid username provided
        croak("The client could not even run the id command for user $opts{user}");
    }

    if ($result->entries) {
        foreach my $entry ($result->entries) {
            $entry->add('uid' => "$user_id");
            $entry->update($ldap);
        }
    } else {
        croak("The requested user : $opts{user} was not found\n");
    }

    $ldap->unbind;

    $Log->exit();
    return;
} ## end sub KRB_LDAP_uid_modify( % )

=head2 validate_client_hash

This subroutine verifies whether the client hash passed into the function contains NULL value for any of the keys
and whether it conatins CLIENT_HASH_COUNT entries

validate_client_hash( client => \%<Client-hash-details> );

=over

=item Arguments

=over

=item client

(Required) : Reference to the Client hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub validate_client_hash( $ ) {
    my ($client, %client_hash);
    $client = shift;
    $Log->enter();

    if ($client eq '') {
        croak("Client hash was expected, but no parameter was obtained by the script");
    }

    %client_hash = %{$client};
    my @client_keys = qw{client_name client_fqn};

    foreach my $key (@client_keys) {
        if ($client_hash{$key} eq "") {
            croak("NULL value was passed in for the kdc hash. key: $key value: $client_hash{ $key }." . "We need all these keys:@client_keys");
        }
    }

    $Log->exit();
    return;
} ## end sub validate_client_hash( $ )

=head2 validate_filer_hash

This subroutine verifies whether the filer hash passed into the function contains NULL value for any of the keys
and whether it conatins FILER_HASH_COUNT entries

validate_filer_hash( filer => \%<Filer-Details> );

=over

=item Arguments

=over

=item filer

(Required) : Reference to the Filer hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub validate_filer_hash( $ ) {
    my ($filer, %filer_hash);
    $filer = shift;
    $Log->enter();

    if ($filer eq '') {
        croak("Filer hash was expected, but no parameter was obtained by the script");
    }

    %filer_hash = %{$filer};
    my @filer_keys = qw{filer_name filer_vserver filer_dns filer_ldap_schema filer_mount_ip};

    foreach my $key (@filer_keys) {
        if ($filer_hash{$key} eq "") {
            croak("NULL value was passed in for the filer hash. key: $key value: $filer_hash{ $key }." . "All these keys are expected: @filer_keys");
        }
    }

    $Log->exit();
    return;
} ## end sub validate_filer_hash( $ )

=head2 client_etc_group_passwd_modify

The /etc/group and /etc/passwd file has to be modified to add the entries for the root , nfs and other users that are created in the script.

client_etc_group_passwd_modify( client_object => $<client-object> , user => $<Username>, [sudo_path => $<sudo-with-fully-qualify-path>] );

Optional: sudo_path

=over

=item Arguments

=over

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item user

(Required) : Username of the user whose entries are to be added into the files so that mount would succeed

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on Success or croaks on failure

=back

=cut

sub client_etc_group_passwd_modify( % ) {
    my %opts = validate(
        @_,
        {   client_object => {type => OBJECT},
            user          => {type => SCALAR, default => "root"},
            sudo_path     => {type => SCALAR, default => "$default_values{sudo_path}"},
        }
    );
    $Log->enter();

    check_client_and_sudo($opts{client_object});

    check_backup_directory(object => $opts{client_object});

    #Create backup files of group and passwd files in the same location as
    #well as the backup directory
    backup_files("/etc/group");
    backup_files("/etc/passwd");

    my $client_etc_group  = client_execute("cat /etc/group");
    my $client_etc_passwd = client_execute("cat /etc/passwd");

    chomp($client_etc_group);
    chomp($client_etc_passwd);
    my ($user_passwd_entry, $user_group_entry);

    $Log->comment("Add the  $opts{ user } entires in the /etc/passwd and /etc/group files");

    # Add entries for root and non root users in the group and passwd files.
    if ($opts{user} ne "root") {
        if ($client_etc_passwd =~ /^$opts{ user }:.*/m) {
            $user_passwd_entry = $&;
        } else {
            $user_passwd_entry = $opts{user} . '::0:0::/:/';

            #add the nfs entry in the /etc/passwd of the client
            client_execute("echo  '$user_passwd_entry' >> /etc/passwd");
        }
    }

    if ($client_etc_passwd =~ /^root:.*/m) {
        $user_passwd_entry = $&;
    } else {
        $user_passwd_entry = 'root::0:0::/:/';

        #add the nfs entry in the /etc/passwd of the client
        $opts{client_object}->execute(command => "echo  '$user_passwd_entry' >> /etc/passwd");
    }

    if ($client_etc_group =~ /root:.*/) {
        $user_group_entry = $&;
    } else {
        $user_group_entry = 'root::0:';

        #add the nfs entry in the /etc/group of the client
        $opts{client_object}->execute(command => "echo  '$user_group_entry' >> /etc/group");
    }

    client_exit_sudo($opts{client_object});

    $Log->exit();
    return 1;
} ## end sub client_etc_group_passwd_modify( % )

=head2 client_etc_sysconfig_nfs_modify

The /etc/sysconfig/nfs file has to be modified to enable secure NFS and
Verbose mode in the kerberos for debugging purpose and restart the rpcgssd server

client_etc_sysconfig_nfs_modify( client_object => $<client-object> , [sudo_path => $<sudo-with-fully-qualify-path>] );

Optional: sudo_path

=over

=item Arguments

=over

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item MIT_Setup

(Optional) : If the subroutine is used during an MIT setup, then set the value to 1, Default value is 0

=back

=item Return

1 on Success or croaks on failure

=back

=cut

sub client_etc_sysconfig_nfs_modify( % ) {
    my %opts = validate(
        @_,
        {   client_object => {type => OBJECT},
            sudo_path     => {type => SCALAR, default => "$default_values{sudo_path}"},
            MIT_Setup => {
                type     => SCALAR,
                optional => 1,
                default  => 0,
                regex    => qr/[01]/
            },
        }
    );
    $Log->enter();

    check_client_and_sudo($opts{client_object});

    check_backup_directory(object => $opts{client_object});

    if ($opts{client_object}->type() =~ /unix-linux/) {
        $Log->comment("Client : Linux");

        #Create backup file of sysconfig/nfs in the same location as
        #well as the backup directory
        backup_files("/etc/sysconfig/nfs");

        $Log->comment("Uncomment SECURE_NFS=yes entires in the file /etc/sysconfig/nfs");

        # Create/edit the file /etc/sysconfig/nfs and add the following line
        if (client_execute("cat /etc/sysconfig/nfs") =~ /SECURE_NFS/) {
            client_execute("sed -e 's/.*SECURE_NFS.*//g' /etc/sysconfig/nfs > /etc/sysconfig/nfs_");
            client_execute("sed '/^\$/d'  /etc/sysconfig/nfs_ > /etc/sysconfig/nfs");
        }
        client_execute("echo 'SECURE_NFS=\"yes\"' >>  /etc/sysconfig/nfs");

        if (client_execute("cat /etc/sysconfig/nfs") =~ /RPCGSSDARGS/) {
            client_execute("sed -e 's/.*RPCGSSDARGS.*//g' /etc/sysconfig/nfs > /etc/sysconfig/nfs_");
            client_execute("sed '/^\$/d'  /etc/sysconfig/nfs_ > /etc/sysconfig/nfs");
        }
        if ($opts{MIT_Setup}) {
            client_execute("echo 'RPCGSSDARGS=\"-vvvrrr -n\"' >>  /etc/sysconfig/nfs");
        } else {
            client_execute("echo 'RPCGSSDARGS=\"-vvvrrr\"' >>  /etc/sysconfig/nfs");
        }

        if (client_execute("cat /etc/sysconfig/nfs") =~ /NFS_SECURITY_GSS/) {
            client_execute("sed -e 's/.*NFS_SECURITY_GSS.*//g' /etc/sysconfig/nfs > /etc/sysconfig/nfs_");
            client_execute("sed '/^\$/d'  /etc/sysconfig/nfs_ > /etc/sysconfig/nfs");
        }
        client_execute("echo 'NFS_SECURITY_GSS=\"yes\"' >>  /etc/sysconfig/nfs");
    } elsif ($opts{client_object}->type() =~ /unix-solaris/) {
        $Log->comment("Solaris Client");

        #Create backup file of sysconfig/nfs file in the same location as
        #well as the backup directory
        backup_files("/etc/nfssec.conf");

        #Un-comment all the krb lines in /etc/nfssec.conf

        client_execute("sed -e 's/^#krb5/krb5/g' /etc/nfssec.conf > /etc/nfssec.conf_");
        $opts{client_object}->execute(command => "\\mv /etc/nfssec.conf_  /etc/nfssec.conf");

    } else {
        croak("The object's hosttype passed into the client_etc_sysconfig_nfs_modify subroutine is incorrect: $opts{ client_object }->type() ");
    }

    client_exit_sudo();
    $Log->exit();
    return 1;
} ## end sub client_etc_sysconfig_nfs_modify( % )

=head2 client_etc_idmapd_modify

The /etc/idmapd.conf file has to be modified to add the KDC domain name into the file.

client_etc_idmapd_modify( client_object => $<client-object> , kdc_domain_name => $<KDC-NIS-Domain-name> , [sudo_path => $<sudo-with-fully-qualify-path>] );

Optional: sudo_path

=over

=item Arguments

=over

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item kdc_domain_name

(Required) : name of the domain to which KDC belongs to

=item sudo_path

(Optional) : Path where the sudo file is located in the client. default: /usr/software/bin/sudo

=back

=item Return

1 on Success or croaks on failure

=back

=cut

sub client_etc_idmapd_modify( % ) {
    my %opts = validate(
        @_,
        {   client_object   => {type => OBJECT},
            kdc_domain_name => {type => SCALAR},
            sudo_path       => {type => SCALAR, default => $default_values{sudo_path}},
        }
    );
    $Log->enter();

    check_client_and_sudo($opts{client_object});

    if ($opts{kdc_domain_name} eq "") {
        croak("Null value was sent for kdc_domain_name in client_etc_idpmapd_modify subroutine");
    }

    check_backup_directory(object => $opts{client_object});

    if ($opts{client_object}->type() =~ /unix-linux/) {

        #adding the nfs v4 domain in idmapd.conf
        my $idmapd_output = $opts{client_object}->execute(command => "ls -l /etc/idmapd.conf");
        my $file_match = "/etc/idmapd.conf";

        #Create backup files of idmapd.conf file in the same location as
        #well as the backup directory
        if ($idmapd_output =~ /$file_match/) {
            backup_files("/etc/idmapd.conf");
        } else {
            $opts{client_object}->execute(command => "touch /etc/idmapd.conf");
        }

        #Create backup files of nfs-common file in the same location as
        #well as the backup directory
        backup_files("/etc/default/nfs-common");

        $Log->comment("Add nis domain info in the file /etc/idmapd.conf ");

        client_execute('\\grep -vw Domain /etc/idmapd.conf > /etc/idmapd.conf_');
        client_execute('sed "/^\$/d" /etc/idmapd.conf_ > /etc/idmapd.conf');
        client_execute('rm -f /etc/idmapd.conf_');

        my $idmap_chk = client_execute("cat /etc/idmapd.conf");
        $idmap_chk =~ s/\[General\]/\[General\]\nDomain=$opts{ kdc_domain_name }/i;
        client_execute("echo \"$idmap_chk\" > /etc/idmapd.conf");

        my @idmapd_output = $opts{client_object}->execute(command => "cat /etc/default/nfs-common");
        foreach (@idmapd_output) {
            if ($_ =~ /NEED_IDMAPD/) {
                client_execute("sed -e 's/.*NEED_IDMAPD.*//g' /etc/default/nfs-common > /etc/default/nfs-common_");
                client_execute("sed '/^\$/d' /etc/default/nfs-common_ > /etc/default/nfs-common");
            }
        }

        client_execute("echo 'NEED_IDMAPD=yes' >>  /etc/default/nfs-common");

        client_rpcidmapd_restart(
            client_object => $opts{client_object},
            sudo_path     => $opts{sudo_path}
        );

    } elsif ($opts{client_object}->type() =~ /unix-solaris/) {
        $Log->comment("Solaris Client");

        #Create backup files of idmapd.conf file in the same location as
        #well as the backup directory
        backup_files("/etc/default/nfs");
        $Log->comment("Add the nis domain info in the file /etc/default/nfs");

        #adding the nfs v4 domain in idmapd.conf
        my @client_default_nfs = $opts{client_object}->execute(command => "cat /etc/default/nfs");

        foreach (@client_default_nfs) {
            if ($_ =~ /^NFSMAPID_DOMAIN/) {
                client_execute("sed -e 's/.*NFSMAPID_DOMAIN.*//g' /etc/default/nfs > /etc/default/nfs_");
                client_execute("sed '/^\$/d' /etc/default/nfs_ > /etc/default/nfs");
            }
        }

        client_execute("echo 'NFSMAPID_DOMAIN=$opts{ kdc_domain_name }' >> /etc/default/nfs");

        client_rpcidmapd_restart(
            client_object => $opts{client_object},
            sudo_path     => $opts{sudo_path}
        );
    } else {
        croak("The object's hosttype passed into the client_etc_idmapd_modify subroutine is incorrect: $opts{ client_object }->type() ");
    }

    client_exit_sudo();

    $Log->exit();
    return 1;
} ## end sub client_etc_idmapd_modify( % )

=head2 KRB_Keytab_Setup

This function creates the keytab on the AD and then transfers the keytab to the Kerberos Client

KRB_Keytab_Setup( kdc_object => $<AD-Server-object> , client_object => $<Client-object> , user => $<Username> , kdc => \%<Kerberos-hash-details> , client => \%<Client-hash-details> );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client

(Required) : Reference to the Client hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item user

(Required) : Username of the user whose entries are to be added into the files so that mount would succeed

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item kdc_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub KRB_Keytab_Setup( % ) {
    my %opts = validate(
        @_,
        {   kdc           => {type => HASHREF},
            client        => {type => HASHREF},
            kdc_object    => {type => OBJECT},
            client_object => {type => OBJECT},
            user          => {type => SCALAR, default => "root"},
        }
    );
    $Log->enter();

    #validate the kdc and client hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_client_hash($opts{client});

    check_client_and_sudo($opts{client_object});

    check_backup_directory(object => $opts{client_object});

    # Cleaning up the kinit users on the client machine.
    client_execute("kdestroy");

    my $Client_name = $opts{client_object}->{client};

    # create root and keytab principals for the client, mapping to the above created user.
    $opts{kdc_object}->execute(command => "del C:\\$Client_name.$opts{ user }.keytab");

    $Log->comment("Generate the Keytab files on the KDC server");
    my $enc        = $opts{kdc}->{kdc_enc_type};
    my $user_princ = $opts{user} . "/" . $opts{client}->{client_fqn} . "\@" . uc("$opts{ kdc }->{ kdc_realm }");

    my $AD_domain;
    my @arr = split('\.', $opts{kdc}->{kdc_realm});
    $AD_domain = $arr[0];
    @arr = split(/\./, $Client_name);
    my $short_client_name = $arr[0];

    # Check for SPN existence before proceed further.
    my $dup_spn_check = $opts{kdc_object}->execute(command => "setspn -Q $opts{user}/$Client_name");
    if ($dup_spn_check =~ /Existing SPN found/) {
        $Log->warn("ERROR: Failed SPN duplication check:\n $dup_spn_check");
    }

    my $keytab_file = "$Client_name.$opts{ user }.keytab";

    #Create the keytab entries for client machine, in the KDC AD server
    my $status
        = $opts{kdc_object}->execute(command =>
            "ktpass -princ $user_princ -pass $::SPN_PASSWORD -mapuser $AD_domain\\$short_client_name -crypto $enc -ptype KRB5_NT_PRINCIPAL -out C:\\$keytab_file"
        );

    $status = $opts{kdc_object}->execute(command => "dir C:\\$keytab_file");
    if ($status !~ /$keytab_file/) {
        croak("creating keytab failed. :$@");
    }

    my $KRB5_PATH = verify_client_hosttype($opts{client_object});

    # keytabs created . now rcp them onto the client . Before that, delete the existing files on the client.
    client_execute("\\rm -rf  /$KRB5_PATH/$Client_name.$opts{ user }.keytab");
    client_execute("\\rm -rf  /*.keytab");

    $Log->comment("Copy the keytab files on to the client");

    # Encountering issue similar to http://support.microsoft.com/kb/323719
    # So copying from the KDC to the client with KDC CLI ;)
    #my $content = client_execute("type C:\\$keytab_file", "", $opts{kdc_object});

    eval {
        my $cmd = qq{$::RCP_PATH $opts{kdc}->{kdc_user}\@$opts{kdc}->{kdc_ip}:"C:\\$Client_name.$opts{user}.keytab" /$KRB5_PATH/};
        client_execute($cmd);
    };

    if ($@) {
        client_execute("cd C:\\", "", $opts{kdc_object});
        $Log->comment(qq{RCP from client itself failed, trying from the KDC});
        my $cmd = "$::WIN_RCP_PATH $keytab_file $opts{client}->{client_fqn}.$opts{user}:/$KRB5_PATH/";
        client_execute($cmd, "", $opts{kdc_object});
    }

    client_execute("ls -l /$KRB5_PATH/$keytab_file");

    # Then, connect to the client, and using rkt,wkt write the keytab files.

    # change to the ktutil shell on the client
    #Create backup files of keytab file in the same location as
    #well as the backup directory
    backup_files("/$KRB5_PATH/krb5.keytab");
    client_execute("\\rm -rf  /$KRB5_PATH/krb5.keytab");
    $opts{client_object}->execute(
        command            => "ktutil",
        additional_prompts => ['ktutil']
    );
    $opts{client_object}->execute(
        command            => "rkt /$KRB5_PATH/$keytab_file",
        additional_prompts => ['ktutil']
    );
    $opts{client_object}->execute(
        command            => "wkt /$KRB5_PATH/krb5.keytab",
        additional_prompts => ['ktutil']
    );

    # Exiting the ktutil prompt.
    $opts{client_object}->execute(command => "exit");

    #done with the ketyab writing.

    return_client();

    $Log->exit();
    return 1;
} ## end sub KRB_Keytab_Setup( % )

=head2 KRB_Client_Setup

This function handles the modification of entries /$KRB5_PATH/group , /$KRB5_PATH/passwd , /$KRB5_PATH/sysconfig/nfs ,
/$KRB5_PATH/krb5.conf , /$KRB5_PATH/hosts in the client

KRB_Client_Setup( kdc_object => $<AD-Server-object> , client_object => $<Client-object> , user => <Username> ,
               kdc => $<Kerberos-hash-details> , client => $<Client-hash-details> , filer_details => %<Filer-Details> );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client

(Required) : Reference to the Client hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item filer_details

(Required) : Reference to the Filer hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item user

(Required) : Username of the user whose entries are to be added into the files so that mount would succeed

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item kdc_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item MIT_Setup

(Optional) : If the subroutine is used during an MIT setup, then set the value to 1, Default value is 0

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub KRB_Client_Setup( % ) {
    my %opts = validate(
        @_,
        {   kdc           => {type => HASHREF},
            client        => {type => HASHREF},
            filer_details => {type => HASHREF},
            client_object => {type => OBJECT},
            kdc_object    => {type => OBJECT},
            user          => {type => SCALAR},
            MIT_Setup     => {
                type     => SCALAR,
                optional => 1,
                default  => 0,
                regex    => qr/[01]/
            },
        }
    );
    $Log->enter();

    #validate the kdc ,filer and client hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_filer_hash($opts{filer_details});
    validate_client_hash($opts{client});

    check_client_object($opts{client_object});
    check_client_object($opts{kdc_object});

    if ($opts{user} eq "") {
        croak("NULL value has been passed in for the user, into KRB_Client_Setup subroutine");
    }

    $Log->comment("Perform time NTP sync on $opts{ client }->{ client_name }");

    #    my $ntpd_running;
    #    eval {
    #        $ntpd_running = client_execute("ps af | grep ntpd | grep -v grep");
    #    };
    #    my $ntpd_command;
    #    if (!$@) {
    #        my @ntpd_running = split(/  */, $ntpd_running);
    #        my $ntpd_pid = $ntpd_running[0];
    #        $ntpd_command = @ntpd_running[7 .. $#ntpd_running];
    #        client_execute("kill -2 $ntpd_pid", "sudo");
    #    }
    #    client_execute("ntpdate $opts{ filer_details }->{ filer_ntp_server }", "sudo");
    #    client_execute("$ntpd_command", "sudo");

    my $date = client_execute("date");

    $Log->comment("=====================================================");
    $Log->comment("date on $opts{ client }->{ client_name } : $date");
    $Log->comment("=====================================================");

    verify_client_installation(
        client_object => $opts{client_object},
        sudo_path     => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
    );
    client_etc_hosts_modify(
        kdc           => $opts{kdc},
        filer_details => $opts{filer_details},
        client_object => $opts{client_object},
        sudo_path     => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
    );
    client_etc_idmapd_modify(
        client_object   => $opts{client_object},
        kdc_domain_name => $opts{kdc}->{kdc_domain},
        sudo_path       => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
    );
    client_etc_group_passwd_modify(
        client_object => $opts{client_object},
        user          => $opts{user},
        sudo_path     => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
    );
    client_etc_sysconfig_nfs_modify(
        client_object => $opts{client_object},
        sudo_path     => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
        MIT_Setup     => $opts{MIT_Setup}
    );
    client_rpcgssd_restart(
        client_object => $opts{client_object},
        sudo_path     => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
    );

    if (!$opts{MIT_Setup}) {    # for AD Kerberos client setup
        client_krb5_config_create(
            kdc           => $opts{kdc},
            client_object => $opts{client_object},
            dns_domain    => $opts{filer_details}->{filer_dns}
        );

        KRB_Keytab_Setup(
            kdc_object    => $opts{kdc_object},
            client_object => $opts{client_object},
            user          => $opts{user},
            kdc           => $opts{kdc},
            client        => $opts{client}
        );
    } else {    # for MIT kerberos client setup
        MIT_client_krb5_config_create(
            kdc           => $opts{kdc},
            client_object => $opts{client_object},
            dns_domain    => $opts{filer_details}->{filer_dns},
            sudo_path     => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
        );

        MIT_add_host_principal_in_client(
            kdc           => $opts{kdc},
            client        => $opts{client},
            client_object => $opts{client_object},
            sudo_path     => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
        );

        MIT_verify_kinit(
            kdc    => $opts{kdc},
            object => $opts{client_object}
        );
    }
    client_rpcgssd_restart(
        client_object => $opts{client_object},
        sudo_path     => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
    );

    $Log->exit();
    return 1;
} ## end sub KRB_Client_Setup( % )

=head2 KRB_Filer_Setup

This function handles the server setup configuration, which involves creation of UnixUser, UnixGroup,
Volumes, Kerberos-Config, Kerberos-Realm, LDAP client creation on the server

KRB_Filer_Setup( filer_details => \%<Filer-details> , user => $<Username> , kdc => \%<KDC-Details> );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item filer_details

(Required) : Reference to the Filer hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item user

(Required) : Username of the user whose entries are to be added into the files so that mount would succeed

=item MIT_Setup

(Optional) : If the subroutine is used during an MIT setup, then set the value to 1, Default value is 0

=back

=item Return

Returns the hash which contains information of which entities were created or reused.
This information would be made use of during cleanup or croaks on failure

=back

=cut

sub KRB_Filer_Setup( % ) {
    my %opts = validate(
        @_,
        {   kdc           => {type => HASHREF},
            filer_details => {type => HASHREF},
            user          => {type => SCALAR},
            MIT_Setup     => {
                type     => SCALAR,
                optional => 1,
                default  => 0,
                regex    => qr/[01]/
            },
        }
    );
    $Log->enter();

    #validate the kdc and filer hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_filer_hash($opts{filer_details});

    if ($opts{user} eq "") {
        croak("NULL value has been passed in for the user, into KRB_Filer_Setup subroutine");
    }

    my $result;

    # Create the root and the nfs user
    $Log->comment("Create the root and nfs users on the vserver");

    $result->{unixuser_created} = 0;

    #Create the entry for non root user in the filer, if it doesnt exist
    if ($opts{user} ne "root") {
        try {
            $result->{unixuser} = NACL::C::VserverServicesUnixUser->find(
                command_interface => $Node_cmd,
                filter            => {
                    vserver => $opts{filer_details}->{filer_vserver},
                    user    => $opts{user},
                },
            );
        }
        catch NACL::Exceptions::NoElementsFound with {
            $result->{unixuser} = NACL::STask::VserverServicesUnixUser->create(
                command_interface => $Node_cmd,
                vserver           => $opts{filer_details}->{filer_vserver},
                user              => $opts{user},
                id                => "50000",
                'primary-gid'     => "0",
            );
            $result->{unixuser_created} = 1;
        };

        if ($result->{unixuser_created} == 0) {
            $result->{unixuser} = NACL::C::VserverServicesUnixUser->modify(
                command_interface => $Node_cmd,
                vserver           => $opts{filer_details}->{filer_vserver},
                user              => $opts{user},
                'id'              => "50000",
                'primary-gid'     => 0,
            );
        }
    }

    #Create the entry for nfs user in the filer, if it doesn't exist
    try {
        $result->{unixuser_nfs} = NACL::C::VserverServicesUnixUser->find(
            command_interface => $Node_cmd,
            filter            => {
                vserver => $opts{filer_details}->{filer_vserver},
                user    => "nfs",
            },
        );
        $result->{unixuser_nfs_created} = 0;
    }
    catch NACL::Exceptions::NoElementsFound with {
        $result->{unixuser_nfs} = NACL::STask::VserverServicesUnixUser->create(
            command_interface => $Node_cmd,
            vserver           => $opts{filer_details}->{filer_vserver},
            user              => "nfs",
            'id'              => "500",
            'primary-gid'     => 0,
        );
        $result->{unixuser_nfs_created} = 1;
    };

    if ($result->{unixuser_nfs_created} == 0) {
        $result->{unixuser_nfs} = NACL::C::VserverServicesUnixUser->modify(
            command_interface => $Node_cmd,
            vserver           => $opts{filer_details}->{filer_vserver},
            user              => "nfs",
            'id'              => "500",
            'primary-gid'     => 0,
        );
    }

    #Create the entry for root user in the filer, if it doesn't exist
    try {
        $result->{unixuser_root} = NACL::C::VserverServicesUnixUser->find(
            command_interface => $Node_cmd,
            filter            => {
                vserver => $opts{filer_details}->{filer_vserver},
                user    => "root",
            },
        );
        $result->{unixuser_root_created} = 0;
    }
    catch NACL::Exceptions::NoElementsFound with {
        $result->{unixuser_root} = NACL::STask::VserverServicesUnixUser->create(
            command_interface => $Node_cmd,
            vserver           => $opts{filer_details}->{filer_vserver},
            user              => "root",
            'id'              => 0,
            'primary-gid'     => 0,
        );
        $result->{unixuser_root_created} = 1;
    };

    if ($result->{unixuser_root_created} == 0) {
        $result->{unixuser_root} = NACL::C::VserverServicesUnixUser->modify(
            command_interface => $Node_cmd,
            vserver           => $opts{filer_details}->{filer_vserver},
            user              => "root",
            'id'              => 0,
            'primary-gid'     => 0,
        );
    }

    $Log->comment("Create the root group and add the " . $opts{user} . " users");
    try {
        $result->{unix_group} = NACL::C::VserverServicesUnixGroup->find(
            command_interface => $Node_cmd,
            filter            => {
                vserver => $opts{filer_details}->{filer_vserver},
                name    => "root",
                id      => "0"
            },
        );
        $result->{unixgroup_created} = 0;
    }
    catch NACL::Exceptions::NoElementsFound with {
        $result->{unix_group} = NACL::STask::VserverServicesUnixGroup->create(
            command_interface => $Node_cmd,
            vserver           => $opts{filer_details}->{filer_vserver},
            name              => "root",
            id                => "0",
        );
        $result->{unixgroup_created} = 1;
    };

    $result->{group_user_added} = 0;
    if ($opts{user} ne "root") {
        try {
            $result->{unix_group}->adduser("username" => $opts{user},);
            $result->{group_user_added} = 1;
        }
        catch NACL::APISet::Exceptions::CommandFailedException with {

            # do nothing , the user has already been added into the group
        };
    }

    try {
        $result->{unix_group}->adduser("username" => "nfs",);
        $result->{group_nfs_user_added} = 1;
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        $result->{group_nfs_user_added} = 0;

        # do nothing , the user has already been added into the group
    };

    try {
        $result->{unix_group}->adduser("username" => "root",);
        $result->{group_root_user_added} = 1;
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        $result->{group_root_user_added} = 0;

        # do nothing , the user has already been added into the group
    };

    $Log->comment("Create the dns for the vserver");
    try {
        NACL::C::VserverServicesDns->create(
            command_interface => $Node_cmd,
            vserver           => "$opts{ filer_details }->{ filer_vserver }",
            domains           => [$opts{kdc}->{kdc_realm}],
            'name-servers'    => [$opts{kdc}->{kdc_ip}],
            state             => 'enabled',
            timeout           => "2",
            attempts          => "1"
        );
    }
    catch NACL::C::Exceptions::VserverServicesDns::AlreadyExists with {
        $Log->comment("AlreadyExists so Modify");
        NACL::C::VserverServicesDns->modify(
            command_interface => $Node_cmd,
            vserver           => "$opts{ filer_details }->{ filer_vserver }",
            domains           => [$opts{kdc}->{kdc_realm}],
            'name-servers'    => [$opts{kdc}->{kdc_ip}],
            state             => 'enabled',
            timeout           => "2",
            attempts          => "1"
        );
    };

    $Log->comment("Configure the Kerberos on the Vserver");
    my $KDC_vendor;
    my $realm_config;
    my %realm_opts = ();

    # this option when set to 0,is for Kerberos AD setup
    if ($opts{MIT_Setup}) {
        $KDC_vendor   = "Other";
        $realm_config = $::MIT_CONFIG_NAME;
    } else {
        $KDC_vendor                  = "Microsoft";
        $realm_config                = $::REALM_CONFIG_NAME;
        $realm_opts{'adserver-name'} = $opts{kdc}->{kdc_server} . "." . $opts{kdc}->{kdc_domain};
        $realm_opts{'adserver-ip'}   = $opts{kdc}->{kdc_ip};
    }

    # perform the kerberos setup
    $Log->comment("Configure the Kerberos realm on the Vserver");
    try {
        NACL::C::VserverServicesKerberosRealm->delete(
            command_interface => $Node_cmd,
            configname        => $realm_config,
        );
    }
    catch NACL::APISet::Exceptions::NoMatchingEntriesException with {

        #Ignore.
        } otherwise {
        my $exception = shift;
        croak("ERROR : Kerberos REALM delete for recreation failed : " . $exception->text());
        };

    #Create the Kerberos realm on the filer for AD setup
    NACL::C::VserverServicesKerberosRealm->create(
        command_interface     => $Node_cmd,
        configname            => $realm_config,
        realm                 => uc($opts{kdc}->{kdc_realm}),
        'kdc-vendor'          => $KDC_vendor,
        'kdc-ip'              => $opts{kdc}->{kdc_ip},
        'kdc-port'            => $::KDC_SERVICE_PORT,
        'clock-skew'          => $::CLOCK_SKEW,
        'adminserver-ip'      => $opts{kdc}->{kdc_ip},
        'adminserver-port'    => $::KADMIN_SERVICE_PORT,
        'passwordserver-ip'   => $opts{kdc}->{kdc_ip},
        'passwordserver-port' => $::PASSWD_SERVER_PORT,
        %realm_opts,
    );

    # Create kerberos config on the filer
    KRB_Filer_Kerberos_Config_Create(
        kdc           => $opts{kdc},
        filer_details => $opts{filer_details},
        user          => $opts{user},
        MIT_Setup     => $opts{MIT_Setup}
    );

    if (!$opts{MIT_Setup}) {
        $result = KRB_Filer_LDAP_create(
            filer_vserver => $opts{filer_details}->{filer_vserver},
            filer_name    => $opts{filer_details}->{filer_name},
            kdc           => $opts{kdc},
        );
    }

    $Log->exit();
    return $result;
} ## end sub KRB_Filer_Setup( % )

sub KRB_Filer_LDAP_create(%) {
    my %opts = validate(
        @_,
        {   filer_vserver => {type => SCALAR},
            filer_name    => {type => SCALAR},
            kdc           => {type => HASHREF},
        }
    );

    my $result;

    if ($opts{filer_name} eq '') {
        croak('Filer name was not passed into Ldap create module');
    }

    if ($opts{filer_vserver} eq '') {
        croak('Vserver name was not passed into Ldap create module');
    }

    # Perform LDAP setup, to incorporate non root user requirement
    # LDAP setup has to be made even for the MIT setup
    $Log->comment("Creating Ldap and ldap client on filer: $opts{ filer_name }");
    my $LdapClientConfig = "krb_AD_" . $opts{filer_name};

    my @ldap_servers = ($opts{kdc}->{kdc_ip});
    try {
        $result->{ldap_client} = NACL::C::VserverServicesLdapClient->find(
            command_interface => $Node_cmd,
            filter            => {
                vserver         => $opts{filer_vserver},
                "client-config" => $LdapClientConfig,
            }
        );
        $result->{ldap_client_created} = 0;
    }
    catch NACL::Exceptions::NoElementsFound with {
        my $DC = kerberos_create_dc(kdc_realm => $opts{kdc}->{kdc_realm});
        $result->{ldap_client} = NACL::C::VserverServicesLdapClient->create(
            command_interface     => $Node_cmd,
            vserver               => $opts{filer_vserver},
            "client-config"       => $LdapClientConfig,
            schema                => $::LDAP_SCHEMA,
            "min-bind-level"      => "simple",
            "ad-domain"           => $opts{kdc}->{kdc_fqn},
            'bind-as-cifs-server' => 'false',
            "bind-password"       => $::LDAP_PASSWD,
            "bind-dn"             => 'cn=' . $::LDAP_USER . ',' . $opts{kdc}->{kdc_ou} . ',' . $DC,
            "base-dn"             => $DC,
            "base-scope"          => "subtree",
            "user-dn"             => $opts{kdc}->{kdc_ou} . "," . $DC,
            "user-scope"          => "subtree",
            "group-scope"         => "subtree",
        );

        $result->{ldap_client}->modify(
            vserver         => $opts{filer_vserver},
            "servers"       => \@ldap_servers,
            "client-config" => $LdapClientConfig,
        );

        $result->{ldap_client}->modify(
            vserver                => $opts{filer_vserver},
            "preferred-ad-servers" => \@ldap_servers,
            "client-config"        => $LdapClientConfig,
        );

        $result->{ldap_client_created} = 1;
    };

    # Making sure the filer name to be used for the Cifs Server creation is within 15 characters
    # as Microsoft BIOS' requirement.
    my $CifsServer = $opts{filer_name} . "_" . $opts{filer_vserver};
    if (length($CifsServer) > 15) {
        $CifsServer = "kerb_$$";
    }
    try {
        $result->{cifs_server} = NACL::CS::VserverCifs->fetch(
            command_interface => $Node_cmd,
            filter            => {vserver => $opts{filer_vserver},}
        );
        $result->{cifs_server_created} = 0;
    }
    catch NACL::Exceptions::NoElementsFound with {
        try {
            $result->{cifs_server} = NACL::C::VserverCifs->create(
                command_interface       => $Node_cmd,
                vserver                 => $opts{filer_vserver},
                "cifs-server"           => $CifsServer,
                domain                  => uc($opts{kdc}->{kdc_realm}),
                "username"              => $opts{kdc}->{kdc_user},
                "password"              => $opts{kdc}->{kdc_passwd},
                ou                      => "CN=Computers",
                "reuse_machine_account" => "y",
            );
            $result->{cifs_server_created} = 1;
        }
        catch NATE::BaseException with {
            my $exception = shift;
            croak("Vserver cifs creation failed with error : " . $exception->text());
        };
    };

    try {
        $result->{ldap} = NACL::C::VserverServicesLdap->find(
            command_interface => $Node_cmd,
            filter            => {
                vserver         => $opts{filer_vserver},
            }
        );
        $result->{ldap_created} = 0;
        my $ldap_config = $result->{ldap}->get_one_state_attribute('client-config');
        if( $ldap_config ne $LdapClientConfig ){
            $result->{ldap}->modify(
                'client-config' => $LdapClientConfig,
            );
        }
    }
    catch NACL::Exceptions::NoElementsFound with {
        $result->{ldap} = NACL::C::VserverServicesLdap->create(
            command_interface => $Node_cmd,
            vserver           => $opts{filer_vserver},
            "client-config"   => $LdapClientConfig,
        );
        $result->{ldap_created} = 1;
    };

    return $result;
} ## end sub KRB_Filer_LDAP_create(%)

=head2 KRB_Filer_Kerberos_Config_Create

The kerberos config needs to be created on the filer which involves disabling the kerberos on the interface
and then enabling it using the spn provided.

KRB_Filer_Kerberos_Config_Create( kdc => \%<KDC-Details> , filer_details => \%<Filer-Details> , user => $<Username> );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item filer_details

(Required) : Reference to the Filer hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item user

(Required) : Username of the user whose entries are to be added into the files so that mount would succeed

=item MIT_Setup

(Optional) : If the subroutine is used during an MIT setup, then set the value to 1, Default value is 0

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub KRB_Filer_Kerberos_Config_Create( % ) {
    my %opts = validate(
        @_,
        {   filer_details => {type => HASHREF},
            kdc           => {type => HASHREF},
            user          => {type => SCALAR},
            MIT_Setup     => {
                type     => SCALAR,
                optional => 1,
                default  => 0,
                regex    => qr/[01]/
            },
        }
    );
    $Log->enter();

    #validate the kdc and filer hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_filer_hash($opts{filer_details});

    my $intf;
    my $Node_cmd    = NACL::C::Node->find();
    my $Api_Set_Obj = $Node_cmd->apiset();
    my $user;

    if (!$opts{MIT_Setup}) {
        $user = $opts{kdc}->{kdc_user};
    } else {
        $user = $opts{kdc}->{kdc_user} . "/" . $::MIT_KDC_ADMIN;
    }

    # Obtain the interface name to which mount_ip belongs to
    $intf = NACL::C::NetworkInterface->find(
        command_interface => $Node_cmd,
        filter            => {address => $opts{filer_details}->{filer_mount_ip}}
    );

    try {

        # disable the existing kerberos config if any
        NACL::C::VserverNfsKerberosConfig->modify(
            command_interface => $Node_cmd,
            vserver           => $opts{filer_details}->{filer_vserver},
            'lif'             => "$intf->{lif}",
            'kerberos'        => "disable",
            "admin-username"  => $user,
            "admin-password"  => $opts{kdc}->{kdc_passwd},
        );
    }
    catch NATE::BaseException with {
        my $exception = shift;
        $Log->comment("Kerberos already disabled on the lif " . $exception->text());
    };

    my $realm           = uc("$opts{ kdc }->{ kdc_realm }");
    my $hostname        = _obtain_ip_host_name($opts{filer_details}->{filer_mount_ip});
    my $spn             = "nfs/" . $hostname . "\@$realm";
    my $hyphen_hostname = join('-', (split(/[\.\@\/]/, $spn)));
    my $truncated_name  = substr(uc("$hyphen_hostname"), 0, 15);
    my $last_character  = substr($truncated_name, -1, 1);
    if ($last_character =~ qr/\-/) {    #remove trailing - if present
        chop($truncated_name);
    }

    my $append_to_cn = lc(kerberos_create_dc(kdc_realm => $opts{kdc}->{kdc_realm}));

    establish_kdc_connection($opts{kdc}->{kdc_server});
    eval { $KDC_Connection_Obj->execute(command => "dsrm -noprompt CN=$truncated_name,CN=Computers,$append_to_cn"); };
    if ($@) {
        if ($@ =~ /No superior reference has been/) {
            $Log->comment(qq{The entry is not present in the DC, proceding});
        } else {
            $Log->comment(qq{Unable to remove the spn in DC, error:$@});
        }
    }

    $Api_Set_Obj->execute_raw_command(command => "set -privilege admin -confirmations on");

    # Until component layer handles this:
    $Api_Set_Obj->execute_raw_command(
        command => "vserver nfs kerberos-config modify -vserver "
            . $opts{filer_details}->{filer_vserver}
            . " -lif "
            . $intf->{lif}
            . " -kerberos enabled -spn $spn -admin-username $user",
        additional_prompts => ["Password:"]
    );
    $Api_Set_Obj->execute_raw_command(
        command     => $opts{kdc}->{kdc_passwd},
        match_table => ["{y|n}" => "y"]
    );

    $Api_Set_Obj->execute_raw_command(command => "set -privilege admin -confirmations off");
    $Node_cmd->run(command => "version");

    NACL::C::VserverNfsKerberosConfig->find(
        command_interface => $Node_cmd,
        filter            => {
            vserver  => $opts{filer_details}->{filer_vserver},
            kerberos => 'enabled',
            lif      => $intf->{lif}
        }
    );

    $Log->exit();
    return 1;
} ## end sub KRB_Filer_Kerberos_Config_Create( % )

=head2 KRB_Cleanup

The setup which was created by the script would be destroyed based the mode of destruction.
Different modes of destruction are:

Mode            Value

ALL             Everything related to kerberos.
AD_CLEANUP      Only AD related stuff should be removed.
CLIENT_CLEANUP  Only Client configurations should be restored.
SERVER_CLEANUP  Only Filer's configurations should be restored.
NONE            Don't cleanup anything.

KRB_Cleanup(   filer_cleanup => \%<Filer-cleanup> , user => $<Username> , mode => $<level-of-destruction> ,
               filer_object => $<filer-object> , client_object => $<client-object> , kdc_object => $<kdc_object>
               kdc => \%<KDC-details> , client => \%<client-hash-details> , filer_details => \%<filer-hash-details> );

=over

=item Arguments

=over

=item kdc

(Required) : Reference to the KDC hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item client

(Required) : Reference to the Client hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item filer_details

(Required) : Reference to the Filer hash which contains various information as mentioned in the design document
             http://wikid.netapp.com/w/NOS_Automation/NFS/Kerberos_Library_Design_Document#AD_Setup

=item filer_cleanup

(Required) : Reference to the hash for what entities in filer were created by script and what entities were reused.
             This information is obtained after the call the KRB_Filer_Setup

=item mode

(Required) : As given in the description various level of setup can be destroyed and they can be given as
             "ALL" or "NONE" and so on.

=item user

(Required) : Username of the user whose entries are to be added into the files so that mount would succeed

=item client_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item kdc_object

(Required) : NACL::C::Client object has to be passed into the subroutine

=item filer_object

(Required) : NACL::C::Node object has to be passed into the subroutine

=item MIT_Setup

(Optional) : If the subroutine is used during an MIT setup, then set the value to 1, Default value is 0

=back

=item Return

1 on success or croaks on failure

=back

=cut

sub KRB_Cleanup( % ) {
    my %opts = validate(
        @_,
        {   kdc           => {type => HASHREF},
            client        => {type => HASHREF},
            filer_details => {type => HASHREF},
            filer_cleanup => {type => HASHREF | UNDEF},
            filer_object  => {type => OBJECT},
            client_object => {type => OBJECT},
            kdc_object    => {type => OBJECT},
            user          => {type => SCALAR, default => "root"},
            mode          => {type => SCALAR, default => $::KERB_CLEANUP, regex => qr/ALL|AD_CLEANUP|CLIENT_CLEANUP|SERVER_CLEANUP|NONE/},
            MIT_Setup => {
                type     => SCALAR,
                optional => 1,
                default  => 0,
                regex    => qr/[01]/
            },
        }
    );
    $Log->enter();

    #validate the kdc , filer and client hash that was passed into the subroutine
    validate_kdc_hash($opts{kdc});
    validate_filer_hash($opts{filer_details});
    validate_client_hash($opts{client});

    if ($opts{filer_object} eq ""
        || (!$opts{filer_object}->isa("NACL::C::Node"))) {
        $opts{filer_object} = $Node_cmd;
    }

    my @client_list = NACL::C::Client->find_unix_clients();

    # If client objects of client and kdc are not provided,
    # then create the objects instead of croaking the script
    my $CLIENTS = $opts{client}->{client_name} . "," . $opts{kdc}->{kdc_server};
    if (!$opts{MIT_Setup}) {

        if ($opts{kdc_object} eq ""
            || (!$opts{kdc_object}->isa("NACL::C::Client"))) {
            $opts{kdc_object} = NACL::C::Client->find_windows_clients();
        }

        if ($opts{client_object} eq ""
            || (!$opts{client_object}->isa("NACL::C::Client"))) {
            $opts{client_object} = $Client_cmd;
        }
    } else {

        if ($opts{kdc_object} eq ""
            || (!$opts{kdc_object}->isa("NACL::C::Client"))) {
            $opts{kdc_object} = $client_list[1];
        }

        if ($opts{client_object} eq ""
            || (!$opts{client_object}->isa("NACL::C::Client"))) {
            $opts{client_object} = $client_list[0];
        }

    }

    if ($opts{user} eq "") {
        $Log->comment("NULL value has been passed in for the user, into KRB_Client_Setup subroutine");
    }

    # Convert the string value into its equivalent Constant value.
    # All the constant values are specified in the beginning of libray.

    # @step Cleanup filer. Remove resources created for the test.
    if ($opts{mode} eq "NONE") {
        $Log->comment("User has requested not to cleanup the setup");
        return;
    }

    # Need root privilege for all of the operations.
    client_enter_sudo($opts{client_object});
    $opts{short_clientname} = (split('\.', $opts{client}->{client_name}))[0];

    if ($opts{mode} eq "ALL" || $opts{mode} eq "AD_CLEANUP") {
        if (!$opts{MIT_Setup}) {
            my $DC = kerberos_create_dc(kdc_realm => $opts{kdc}->{kdc_realm});
            $opts{kdc_object}->execute(command => "dsrm -noprompt \"cn=$opts{short_clientname},$opts{ kdc }->{ kdc_ou },$DC\"");

            # create root and keytab principals for the client,
            # mapping to the above created user.
            $Log->comment("Delete the keytab files on the KDC server");
            $opts{kdc_object}->execute(command => "cd /\ | del C:\\$opts{ client }->{ client_name }.$opts{ user }.keytab");
        } else {
            MIT_delete_client_principal_from_KDC(
                kdc        => $opts{kdc},
                client     => $opts{client},
                kdc_object => $opts{kdc_object},
                sudo_path  => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
            );
        }
    }

    if ($opts{mode} eq "ALL" || $opts{mode} eq "CLIENT_CLEANUP") {
        foreach my $_client (@client_list) {
            check_client_and_sudo($_client);
            $Log->step("Revert the /etc/hosts, /etc/passwd and /etc/group on the client");

            client_execute_ignore_error("\\mv /etc/group.backup /etc/group");
            client_execute_ignore_error("\\mv /etc/passwd.backup /etc/passwd");
            client_execute_ignore_error("\\mv /etc/hosts.backup /etc/hosts");

            if ($opts{MIT_Setup}) {
                MIT_delete_principal_from_client(
                    kdc           => $opts{kdc},
                    client        => $opts{client},
                    client_object => $_client,
                    sudo_path     => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
                );
            }

            if ($_client->type() =~ /unix-linux/) {
                client_execute_ignore_error("\\mv /etc/idmapd.conf.backup /etc/idmapd.conf");
                client_execute_ignore_error("\\mv /etc/default/nfs-common.backup /etc/default/nfs-common");
                client_execute_ignore_error("\\mv /etc/sysconfig/nfs.backup /etc/sysconfig/nfs");

                $Log->comment("Delete the krb5.conf and the keytab files on the client");
                client_execute_ignore_error("\\mv /etc/krb5.keytab.backup /etc/krb5.keytab");
                client_execute_ignore_error("\\mv /etc/krb5.conf.backup /etc/krb5.conf");

            } elsif ($_client->type() =~ /unix-solaris/) {
                client_execute_ignore_error("\\mv /etc/default/nfs.backup /etc/default/nfs");
                client_execute_ignore_error("\\mv /etc/nfssec.conf.backup /etc/nfssec.conf");

                $Log->comment("Delete the krb5.conf and the keytab files on the client");
                client_execute_ignore_error("\\mv /etc/krb5/krb5.keytab.backup /etc/krb5/krb5.keytab");
                client_execute_ignore_error("\\mv /etc/krb5/krb5.conf.backup /etc/krb5/krb5.conf");

            } else {
                croak("Invalid hosttype of the client: $opts{ client_object }->type()");
            }

            client_rpcgssd_restart(
                client_object => $_client,
                sudo_path     => $opts{client}->{sudo_path} ? $opts{client}->{sudo_path} : $default_values{sudo_path},
            );
            client_execute("kdestroy");
        }
    }

    if ($opts{mode} eq "ALL" || $opts{mode} eq "SERVER_CLEANUP") {
        $Log->comment("Delete the kerberos setup on the filer");
        my $Node_cmd      = $opts{filer_object};
        my $filer_cleanup = $opts{filer_cleanup};

        if (keys(%$filer_cleanup) == 0) {
            $Log->comment("Looks like the script execution failed before entering filer setup stage");
            return;
        }

        # delete the kerberos setup
        my $config_name;
        if ($opts{MIT_Setup}) {
            $config_name = $::MIT_CONFIG_NAME;
        } else {
            $config_name = $::REALM_CONFIG_NAME;
        }

        filer_kerberos_disable(
            kdc           => $opts{kdc},
            filer_details => $opts{filer_details}
        );

        NACL::C::VserverServicesKerberosRealm->delete(
            command_interface => $opts{filer_object},
            configname        => $config_name
        );

        $Log->comment("Delete the export policy, root and nfs user");

        my $result = $opts{filer_cleanup};

        if ($result->{export_policy_rule_created}) {
            $result->{export_policy_rule}->delete();
        }

        if ($result->{export_policy_created}) {
            $result->{export_policy}->delete();
        }

        if ($result->{unixuser_created}) {
            $result->{unixuser}->delete();
        }

        if ($result->{unixuser_nfs_created}) {
            $result->{unixuser_nfs}->delete();
        }

        if ($result->{unixuser_root_created}) {
            $result->{unixuser_root}->delete();
        }

        if ($result->{group_root_user_added}) {
            $result->{unix_group}->deluser(username => "root");
        }

        if ($result->{group_nfs_user_added}) {
            $result->{unix_group}->deluser(username => "nfs");
        }

        if ($result->{group_user_added}) {
            $result->{unix_group}->deluser(username => $opts{user});
        }

        if ($result->{unixgroup_created}) {
            $result->{unix_group}->delete();
        }

        if ($result->{ldap_client_created}) {
            $result->{ldap_client}->purge();
        }

        if ($result->{ldap_created}) {
            $result->{ldap}->purge();
        }

        if ($result->{cifs_server_created}) {
            $result->{cifs_server}->purge();
        }

    }
    $Log->comment("Cleanup Completed");

    $Log->exit();
    return 1;
} ## end sub KRB_Cleanup( % )

=head2 filer_ntp_config

This function is used to configure NTP server on the filer such that AD, Server and client are synced with the same time.

filer_ntp_config( filer_object => $<filer-Object> , ntp_server => $<NTP-Server-name> , timezone => $<TIMEZONE> );

=over

=item Arguments

=over

=item filer_object

(Required) :  NACL::C::Node object has to be passed into the subroutine

=item ntp_server

(Required) : NTP server with which the server has to be configured

=item timezone

(Required) : Timezone to which the filer has to be configured

=back

=item Return

1 on Success or croaks on failure

=back

=cut

sub filer_ntp_config( % ) {
    my %opts = validate(
        @_,
        {   filer_object => {type => OBJECT},
            ntp_server   => {type => SCALAR},
            timezone     => {type => SCALAR},
        }
    );
    $Log->enter();

    if (!$opts{filer_object}->isa("NACL::C::Node")) {
        croak("NACL::C::Node object had to be passed into the subroutine");
    }

    SystemServicesNtpConfig->modify(command_interface => $opts{filer_object}, "enabled" => "false");
    set_date(filer_object => $opts{filer_object});    #set the date on the filer
    SystemNodeDate->modify(command_interface => $opts{filer_object}, 'timezone' => $opts{timezone});
    SystemServicesNtpServer->create(
        command_interface => $opts{filer_object},
        node              => 'local',
        server            => $opts{ntp_server}
    );
    SystemServicesNtpConfig->modify(command_interface => $opts{filer_object}, "enabled" => "false");

    my $date = $opts{filer_object}->execute(command => "date");
    $Log->comment("=====================================================");
    $Log->comment("date on $opts{ filer_object }->{ hostname } : $date");
    $Log->comment("=====================================================");

    $Log->exit();
    return 1;
} ## end sub filer_ntp_config( % )

=head2 set_date

This function is used to set the localtime on the filer.

set_date( filer_object => $<filer-Object> );

=over

=item Arguments

=over

=item filer_object

(Required) :  NACL::C::Node object has to be passed into the subroutine

=back

=item Return

1 on Success or croaks on failure

=back

=cut

sub set_date() {
    my %opts = validate(@_, {filer_object => {type => OBJECT},});
    $Log->enter();

    if (!$opts{filer_object}->isa("NACL::C::Node")) {
        croak("NACL::C::Node object had to be passed into the subroutine");
    }

    my ($output, @vars, $time, $year, $month, $date, %h);

    $output = localtime;
    @vars = split(/\ /, $output);

    $time  = $vars[3];
    $month = "$vars[1]";
    $year  = $vars[4];
    $date  = $vars[2];

    #Thu Nov  3 12:34:24 2011
    %h = (
        Jan => 1,
        Feb => 2,
        Mar => 3,
        Apr => 4,
        May => 5,
        Jun => 6,
        Jul => 7,
        Aug => 8,
        Sep => 9,
        Oct => 10,
        Nov => 11,
        Dec => 12
    );

    try {
        NACL::C::SystemNodeDate->modify(
            command_interface => $opts{filer_object},
            "dateandtime"     => "$h{$month}\/$date\/$year $time",
        );
    }
    catch NATE::BaseException with {
        my $exception = shift;
        $Log->comment("Error while modifying system date on filer : " . $exception->text());
    };

    #$opts{ filer_object }->execute( command => "system node date modify -date $h{$month}\/$date\/$year $time" );

    $Log->exit();
    return 1;
} ## end sub set_date()

sub MIT_KDC_verify_principal($$$) {
    my ($principal, $kdc_object, $realm) = @_;
    $Log->enter();
    _enter_kadmin_prompt($kdc_object);

    # without the root/admin principal in the server we cannot proceed
    $principal = "$principal\@$realm";
    my $output = $kdc_object->execute(
        command            => "getprinc $principal",
        additional_prompts => ['kadmin.local']
    );

    if (   $output =~ /Principal does not exist/
        || $output !~ /$principal/) {
        croak("Principal:$principal not found in the MIT KDC Server");
    }
    _exit_kadmin_prompt($kdc_object);
    $Log->exit();
} ## end sub MIT_KDC_verify_principal($$$)

sub MIT_KDC_delete_principal($$) {

    my ($principal, $kdc_object, $realm) = @_;
    $Log->enter();
    my $present = 0;

    client_enter_sudo($kdc_object);
    _enter_kadmin_prompt($kdc_object);

    my $delete_princ     = "Are you sure you want to delete the principal ";
    my $principal_output = $kdc_object->execute(
        command            => "delprinc $principal",
        additional_prompts => ['kadmin.local'],
        match_table        => [$delete_princ => "yes",]
    );

    my $princ_msg = qq{Principal "$principal\@$realm" deleted};
    if ($principal_output !~ /$princ_msg/) {
        $Log->comment("Unable to delete principal:$principal from the server : $principal_output ");
    }

    _exit_kadmin_prompt($kdc_object);
    client_exit_sudo($kdc_object);
    $Log->exit();
} ## end sub MIT_KDC_delete_principal($$)

sub MIT_KDC_add_principal($$$$) {
    my ($principal, $kdc_object, $password, $realm) = @_;
    $Log->enter();
    client_enter_sudo($kdc_object);
    _enter_kadmin_prompt($kdc_object);
    $kdc_object->execute(
        command            => "addprinc -randkey $principal",
        additional_prompts => ['kadmin.local']
    );

    my $output = $kdc_object->execute(
        command            => "getprinc $principal",
        additional_prompts => ['kadmin.local']
    );

    if (   $output =~ /Principal does not exist/
        || $output !~ /$principal/) {
        croak("Principal:$principal could not be added into the MIT KDC Server");
    }

    if ($password) {
        my $password_output = $kdc_object->execute(
            command            => "cpw -pw $password $principal",
            additional_prompts => ['kadmin.local']
        );

        my $pwd_msg = qq{Password for "$principal\@$realm" changed};
        if ($password_output !~ /$pwd_msg/) {
            croak("Unable to change password for the principal:$principal in the server. Expected:$pwd_msg, got:$password_output");
        }
    }

    _exit_kadmin_prompt($kdc_object);
    client_exit_sudo($kdc_object);
    $Log->exit();
} ## end sub MIT_KDC_add_principal($$$$)

sub get_fqn($$$) {
    my ($host, $ip, $check) = @_;
    $Log->enter();
    my ($result, @entries, $domainname);

    $result = client_execute("host $ip");
    if ($result eq "") {
        croak(qq{No valid hostname found for the ip:$ip in  $result});
    }
    chomp($result);

    # Result looks like:
    # "61.186.225.10.in-addr.arpa domain name pointer proto-talapoin.rtp.netapp.com."
    #Need to pull out the last string, and then to remove the last dot.
    my @array = split(/ /, $result);
    my $fqn = $array[-1];
    @array = split(/\./, $fqn);
    if ($array[-1] eq "") {
        pop @array;
    }
    $fqn = join(".", @array);
    $Log->comment(qq{fqn = $fqn});
    if ($check) {
        if ($result =~ /$host\.([^ \t]*)/) {
            $fqn = $host . $1;
        } else {
            croak(qq{The hostname($result) from $ip doesnot even contain the hostname($host)});
        }
    }

    $Log->exit();
    return $fqn;
} ## end sub get_fqn($$$)

sub load_kerberos_module($) {
    my $cli = shift;
    $Log->enter();

    eval { client_execute("lsmod | grep auth_rpcgss", "", $cli); };
    if ($@) {
        eval { client_execute("insmod `find /lib -name rpcsec_gss_krb5.ko`", "", $cli); };
        if ($@) {
            croak(qq{The client doesnot support kerberos, all tests are invalid after this});
        }
    } else {
        $Log->comment(qq{Client already has the necessary kerberos module in the kernel});
        return;
    }

    $Log->exit();
} ## end sub load_kerberos_module($)

sub _restart_service($) {
    my $service = shift;
    $Log->enter();
    $Log->comment("Stop the $service services");

    # Procedure: Stop the rpcidmapd daemon, verify it and then issue start
    client_execute("/etc/init.d/$service stop");
    eval { my $service_status = client_execute("/etc/init.d/$service status"); };

    if (!$@) {
        croak("$service daemon could not be stopped on the client, there was no error");
    } elsif ($@ =~ /is stopped/) {
        $Log->comment("$service daemon is stopped on the client");
    } else {
        croak(qq{Unexpected error in obtaining $service status:  $@});
    }

    $Log->comment("Start the $service services");
    eval { client_execute("/etc/init.d/$service start"); };

    # Workaround for rpc.gssd. This service is not started when we use the above command.
    # We have to trigger it as below. Get rid of this code when the issue is fixed.
    if ($@) {
        if ($service eq "rpcgssd") {
            client_execute("/usr/sbin/rpc.gssd -v");
        }
    }

    my $rpcidmapd_status = client_execute("/etc/init.d/$service status");

    $Log->exit();
} ## end sub _restart_service($)

sub backup_files($) {
    my $file = shift;
    $Log->enter();

    use File::Basename;
    my $base = basename($file);
    eval { client_execute("\\ls -l $file"); };

    if ($@) {
        if ($@ =~ /No such file or directory/) {
            return;
        }
        croak(qq{Some unexpected error with ls: $@});
    }

    # So there is a file, and we need to copy.
    client_execute("\\cp -f $file $file.backup");
    client_execute("\\cp -f $file $::BACKUP_DIRECTORY/$base.backup");
    $Log->exit();
} ## end sub backup_files($)

sub client_execute_ignore_error($) {
    $Log->enter();
    eval { client_execute(shift); };
    if ($@) {
        $Log->comment("Ignoring the error:$@");
    }
    $Log->exit();
} ## end sub client_execute_ignore_error($)

sub get_hostname($) {
    my $output = client_execute("hostname", "", shift);
    $Log->enter();
    chomp($output);
    $Log->exit();
    return $output;
} ## end sub get_hostname($)

my $KDC_Server;

# return the kdc_connection_object
sub establish_kdc_connection($) {
    my $given_kdc = shift;
    $Log->enter();
    if ($given_kdc ne $KDC_Server || !$KDC_Connection_Obj) {
        $KDC_Connection_Obj = NACL::C::Client->new(client => $given_kdc);
        $KDC_Server = $given_kdc;
    }
    $Log->exit();
    return $KDC_Connection_Obj;
} ## end sub establish_kdc_connection($)

# To ensure that all the SPN names that we use are within 15 character limit,
# we use this function.
my %IP_HOST_NAME;
my $IP_COUNTER;

sub _obtain_ip_host_name($) {
    my $address = shift;
    $Log->enter();
    if (0) {
        if ($IP_HOST_NAME{$address}) {
            return $IP_HOST_NAME{$address};
        }

        $IP_COUNTER++;
        my $new_hostname = "KR" . $$ . "_$IP_COUNTER";
        $IP_HOST_NAME{$address} = $new_hostname;
        return $new_hostname;
    } else {
        my $hostname = get_fqn("junk", $address, 0);
        return $hostname;
    }
    $Log->exit();
} ## end sub _obtain_ip_host_name($)

sub _enter_kadmin_prompt($) {
    my $kdc_object = shift;
    $Log->enter();
    $kdc_object->execute(
        command            => "kadmin.local",
        additional_prompts => ['kadmin.local']
    );
    $Log->exit();
} ## end sub _enter_kadmin_prompt($)

sub _exit_kadmin_prompt($) {
    my $kdc_obj = shift;
    $Log->enter();
    my $output = $kdc_obj->execute(
        command            => "get_principal",
        additional_prompts => ['kadmin']
    );
    if ($output =~ /command not found/) {
        $Log->comment(qq{Already exited the kadmin});
        return;
    }

    $kdc_obj->execute(
        command            => "exit",
        additional_prompts => ['kadmin']
    );
    $Log->exit();
} ## end sub _exit_kadmin_prompt($)

sub MIT_KDC_add_current_user_spn($$) {
    my ($kdc_object, $client_object) = @_;
    $Log->enter();

    my $user = client_execute("whoami", "", $client_object);
    chomp($user);
    MIT_KDC_add_principal($user, $kdc_object, $::KDC_ADMIN_PASSWD, $::REALM);
    $Log->exit();
} ## end sub MIT_KDC_add_current_user_spn($$)

sub Client_kinit($;$$) {
    my ($client, $user, $password) = (shift, shift, shift);
    $Log->enter();
    param('KINIT',                  '-default', '/usr/bin/kinit');
    param('REALM',                  '-mesg',    'Kerberos Realm to use');
    param('USER_KERBEROS_PASSWORD', '-mesg',    'Kerberos Password to use');
    $password = $password ? $password : $::USER_KERBEROS_PASSWORD;
    my $command = "$::KINIT";
    if ($user) {
        $command .= " $user";
    }
    $client->execute(
        command     => $command,
        match_table => ["$::REALM:" => $password,]
    );
    $Log->exit();
} ## end sub Client_kinit($;$$)

sub get_root_principal($) {
    my $client_fqn = shift;
    return (qq{$::MIT_KDC_ROOT/$client_fqn});
}

sub get_host_principal($) {
    my $client_fqn = shift;
    return (qq{$::MIT_KDC_HOST/$client_fqn});
}

sub kdc_root_principal($) {
    my $realm = shift;
    return ($::MIT_KDC_ROOT . "@" . uc($realm));
}

sub kdc_admin_principal($) {
    my $realm = shift;
    return ($::MIT_KDC_ADMIN . "@" . uc($realm));
}

1;
