# this is basically -*- perl -*-

# $Id: //depot/prod/test/main/lib/nfs/NfsClient/4_1.pm#22 $
#
## @summary NfsClient - NFS client protocol class
## @author	dl-nfsqa-only (NFS QA Sunnyvale)
## @status	public
## @notes	Created by EP Network Storage Performance Lab, Inc.
## @pod here

=head1 NAME

NfsClient - NFS client protocol class

=head1 SYN

  use nfs::NfsClient;

  $nfs_obj = nfs::NfsClient->new(hostname => $host, version  => 4,
                                 proto => 'tcp');
  $nfs_obj->callback_register(version => 1);
  $nfs_obj->callback_process();
  $out = $nfs_obj->mount(path => '/vol/testvol');
  $nfs_obj->null();
  $out = $nfs_obj->exchange_id();
  $out = $nfs_obj->getdeviceinfo();
  $out = $nfs_obj->getattr();
  $out = $nfs_obj->setattr(attr => {FATTR4_MODE() => 0777});
  $out = $nfs_obj->lookup(name => 'file1');
  $out = $nfs_obj->access(access => ACCESS_READ);
  $opn = $nfs_obj->open(name => 'file1', access => OPEN4_SHARE_ACCESS_READ,
                                         deny   => OPEN4_SHARE_DENY_NONE);
  $out = $nfs_obj->read(offset => 0, count => 1024, stateid => $opn->{stateid});
  $out = $nfs_obj->open_downgrade(stateid => $opn->{stateid},
                                  access => OPEN4_SHARE_ACCESS_WRITE,
                                  deny => OPEN4_SHARE_DENY_NONE);
  $out = $nfs_obj->open_confirm(stateid => $opn->{stateid});
  $out = $nfs_obj->openattr();
  $out = $nfs_obj->close(stateid => $opn->{stateid});
  $out = $nfs_obj->lookupp();
  $out = $nfs_obj->readlink(fhandle => $fhandle);
  $out = $nfs_obj->mkdir(name => 'test_dir', attr => {FATTR4_MODE() => 0755});
  $out = $nfs_obj->symlink(name => 'sym_link', data => 'test_dir/file2');
  $out = $nfs_obj->mknod(name => 'dev_char', type => NFCHR, major=>123,
                         minor => 12, attr => {FATTR4_MODE() => 0666});
  $out = $nfs_obj->rename(fromdir => $f_fhandle, fromname => 'file3',
                          dodir   => $f_fhandle, toname   => 'file4');
  $out = $nfs_obj->remove(name => 'file4');
  $out = $nfs_obj->link(todir => $t_fhandle, name => 'hard_link1');
  $out = $nfs_obj->readdir();
  $out = $nfs_obj->readdirplus();
  $out = $nfs_obj->fsstat();
  $out = $nfs_obj->write(offset => $offset, data => $data);
  $out = $nfs_obj->commit();
  $out = $nfs_obj->delegreturn(stateid => $deleg_stateid);
  $out = $nfs_obj->delegpurge();
  $out = $nfs_obj->renew();
  $out = $nfs_obj->create(name => 'fifo', type => NFFIFO);
  $out = $nfs_obj->rmdir(name => 'fifo');
  $out = $nfs_obj->fsinfo();
  $out = $nfs_obj->secinfo(name => '.');
  $out = $nfs_obj->pathconf();
  $nfs_obj->umount(path => '/vol/testvol');
  $out = $nfs_obj->mount_dump();
  $out = $nfs_obj->mount_export();
  $out = $nfs_obj->create_session();
  $nfs_obj->current_fh_set(fhandle => $fhandle);
  $nfs_obj->sm_notify();

  # To lock a file on NFSv2 or NFSv3
  $lock = $nfs_obj->lock(offset => 32, count => 64);
  # To lock a file on NFSv4
  $lock = $nfs_obj->lock(offset => 32, count => 64, stateid => $opn->{stateid});

  # To unlock the file on NFSv2 or NFSv3
  $out = $nfs_obj->locku(offset => 32, count => 64);
  # To unlock the file on NFSv4
  $out = $nfs_obj->locku(offset => 32, count => 64,
                         stateid => $lock->{lock_stateid});

  # or simply use
  $out = $nfs_obj->locku(lockid => $lock->{lockid});
  # To unlock only a subregion of the lock
  $out = $nfs_obj->locku(lockid => $lock->{lockid}, offset => 50, count=>10);

  # To cancel a blocked lock on NFSv2 or NFSv3
  $out = $nfs_obj->lock_cancel(lockid => \%lockid);

  # To check whether a lock is available to the client on NFSv2 or NFSv3.
  $out = $nfs_obj->lock_test(offset => 32, count => 64, exclusive => 1);
  # To check whether a lock is available to the client
  # For NFSv2 and NFSv3 it calls method lock_test with exclusive set
  # to 1 if type is WRITE(W)_LT and set to 0 if type is READ(W)_LT
  $out = $nfs_obj->lockt(offset => 32, count => 64, type => WRITE_LT);

  # Release lock owner on NFSv4
  $out = $nfs_obj->release_lockowner();

  # Call a compound request
  @ops = ( { op => OP_PUTFH,   args => $fhandle },
           { op => OP_GETATTR, args => xdr_bitmap4(encode=>$attr) } );
  $out = $nfs_obj->compound( ops => \@ops, tag => 'getattr' );

  # Object description
  $rpc        = $nfs_obj->rpc();         # RPC object
  $mnt_fh     = $nfs_obj->mnt_fh();      # Filehandle of root of mount
  $curr_fh    = $nfs_obj->curr_fh();     # Current filehandle use as default
  $mnt_path   = $nfs_obj->mnt_path();    # Mount path
  $result     = $nfs_obj->result();      # Last failed results
  $clientid   = $nfs_obj->clientid();    # NFSv4 client id
  $seqid      = $nfs_obj->seqid();       # NFSv4 open sequence id
  $lock_seqid = $nfs_obj->lock_seqid();  # NFSv4 lock sequence id

  # Set NFSv4 open sequence id and return previous value
  $seqid_prev = $nfs_obj->seqid($seqid_new);

  # Set NFSv4 lock sequence id and return previous value
  $lock_seqid_prev = $nfs_obj->lock_seqid($lock_seqid_new);

  # Do we expect to wait for callback if we get a DELAY error for an op ?
  $nfs_obj->expect_callback_on_delay($flag);

=head1 DESCRIPTION

Provides an object interface for the NFS client protocol version
2, 3 and 4. All procedures for NFS version 2 and 3 are implemented
including some MOUNT protocol procedures to mount, unmount and dump
a list of mounts and exports on the NFS server. On NFSv4, only those
operations needed to have similar functionality with NFSv2 and v3
are implemented as of now.

Most methods are called with the same parameters for all three
versions of NFS. NFSv3 and v4 may require additional input parameters.
This should be noted in the documentation of each particular method.
In addition, some methods are not supported by all versions of NFS.
If this is the case the method croaks with an unsupported error message.

The format of the 'attr' option for setting the attributes of a
file system object is as follows. For NFSv2 and v3, the valid
options are: mode, size, uid, gid, atime and mtime. For NFSv4,
the valid options are any of the FATTR4_* attribute constants which
can be set. Note that there is a difference in the time structure:
NFSv2 is { seconds=>x, useconds=>y } while NFsv3 and v4 is
{ seconds=>x, nseconds=>y }.

When the 'attr' option is used to retrieve a list of attributes
for NFSv4, the option is a reference to an array of FATTR4_*
attribute constants. The best way to test this library is by running
the tests in nfs/ops.

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-nfsqa-only (NFS QA Sunnyvale)
  rraghav@netapp.com (NFS QA BTC).
  $Id: //depot/prod/test/main/lib/nfs/NfsClient/4_1.pm#22 $
  Last revised $Date: 2012/07/31 $

=head1 COPYRIGHT

  Copyright 2005, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package nfs::NfsClient::4_1;

use strict;
use warnings;

use nfs::Rpc;
use SPARSE::Util::Logger;
use Carp qw(croak);
use Params::Validate qw(validate SCALAR ARRAYREF HASHREF UNDEF);
use nfs::NfsXdr qw(xdr_access xdr_bitmap4 xdr_cb_compound xdr_cb_exit
  xdr_cb_info xdr_cb_pid xdr_cb_port
  xdr_cb_register xdr_commit xdr_compound xdr_compound_res xdr_cookie
  xdr_cookieverf xdr_create xdr_daemon_info xdr_dump xdr_export xdr_fattr4
  xdr_fhandle xdr_fsinfo xdr_fsstat xdr_getattr xdr_link xdr_lookup xdr_mkdir
  xdr_mknod xdr_mount xdr_nfstime
  xdr_opaque xdr_pathconf xdr_read xdr_readdir xdr_readdirplus xdr_readlink
  xdr_remove xdr_rename xdr_sattr xdr_setattr xdr_specdata xdr_stateid4
  xdr_symlink xdr_uint xdr_uint64 xdr_write xdr_name);
use nfs::NfsDef qw(nfs_error bitmap4_convert
  EXCLUSIVE FATTR4_FILEID FATTR4_FSID FATTR4_MODE FATTR4_NUMLINKS FATTR4_OWNER
  FATTR4_OWNER_GROUP FATTR4_RAWDEV FATTR4_SIZE FATTR4_TIME_ACCESS
  FATTR4_TIME_METADATA FATTR4_TIME_MODIFY FATTR4_TYPE FILE_SYNC
  FATTR4_NO_TRUNC FATTR4_LAYOUT_TYPE FATTR4_LAYOUT_BLKSIZE
  MOUNTPROC_DUMP MOUNTPROC_EXPORT MOUNTPROC_MNT MOUNTPROC_PATHCONF
  MOUNTPROC_UMNT NFBLK NFCHR NFDIR NFFIFO NFLNK NFREG NFS4ERR_DELAY
  NFS4ERR_DENIED NFS4ERR_GRACE NFSOCK NFSPROC3_ACCESS NFSPROC3_COMMIT
  NFSPROC3_CREATE NFSPROC3_FSINFO NFSPROC3_FSSTAT NFSPROC3_LINK
  NFSPROC3_LOOKUP NFSPROC3_MKDIR NFSPROC3_MKNOD NFSPROC3_PATHCONF
  NFSPROC3_READDIRPLUS NFSPROC3_REMOVE NFSPROC3_RENAME NFSPROC3_RMDIR
  NFSPROC3_SYMLINK NFSPROC3_WRITE NFSPROC_CREATE NFSPROC_GETATTR NFSPROC_LINK
  NFSPROC_LOOKUP NFSPROC_MKDIR NFSPROC_READ NFSPROC_READDIR NFSPROC_READLINK
  NFSPROC_REMOVE NFSPROC_RENAME NFSPROC_RMDIR NFSPROC_SETATTR NFSPROC_STATFS
  NFSPROC_SYMLINK NFSPROC_WRITE OP_ACCESS OP_CB_RECALL OP_CB_LAYOUTRECALL OP_CLOSE OP_COMMIT
  OP_CREATE OP_DELEGPURGE OP_DELEGRETURN OP_GETATTR OP_GETFH OP_LINK OP_LOCK
  OP_LOCKT OP_LOCKU OP_LOOKUP OP_LOOKUPP OP_OPEN OP_OPEN_CONFIRM OP_OPEN_DOWNGRADE
  OP_OPENATTR OP_PUTFH OP_PUTROOTFH OP_READ OP_READDIR OP_READLINK OP_RELEASE_LOCKOWNER
  OP_REMOVE OP_RENAME OP_RENEW OP_SAVEFH OP_SECINFO OP_SETATTR
  OP_SECINFO_NO_NAME OP_NVERIFY OP_SEQUENCE OP_VERIFY OP_WRITE OP_EXCHANGE_ID OP_CREATE_SESSION OPEN4_SHARE_ACCESS_READ
  OPEN4_SHARE_DENY_NONE READ_LT READW_LT UNCHECKED SP4_NONE SP4_MACH_CRED SP4_SSV
  WEBNFS3_FHANDLE WRITE_LT WRITEW_LT :NFS4DCLAIMS :NFS4EXCHANGE OP_DESTROY_SESSION OP_GETDEVICELIST
  OP_GETDEVICEINFO OP_LAYOUTGET OP_LAYOUTRETURN OP_DESTROY_CLIENTID OP_BIND_CONN_TO_SESSION OP_RECLAIM_COMPLETE
  :NFS4DELEG :NFS4OPEN
  OP_BACKCHANNEL_CTL OP_LAYOUTCOMMIT STATEID0 STATEID1);

use Net::IP qw(ip_is_ipv4 ip_is_ipv6);
use Socket;
use Data::Dumper;

use Options;
use Tharn;

my ($LOOPBACK);
my $Options = new Options;

BEGIN {
    # Die with a script error when an
    # input parameter fails validation
    Params::Validate::validation_options(
        on_fail => sub { my @msgs = @_;
                         chomp(@msgs);
                         Carp::confess(Tharn::script_result(@msgs)) }
    );
};

#---------------------------------------------------------------------
# Package local variables
#---------------------------------------------------------------------
my $sudo = '/usr/software/bin/sudo';
my $cb_lock_file = '/tmp/.cb_lock_file';

# Create a client id that is unique to the process, but does
# not change with every new instantiation
my $client_id = $$ + int(rand((1000000000)));
my $co_owner_id = $$ + int(rand((1000000000)));

# Map the callback procedures to its
# corresponding callback function
my %common_call_back = (
 1000 => \&xdr_cb_info,
 1001 => \&xdr_cb_pid,
 1002 => \&xdr_cb_port,
 1003 => \&xdr_cb_exit,
 2000 => \&xdr_cb_register,
);

my %call_back = (
    0 => sub {return},
    1 => \&xdr_cb_compound,
    %common_call_back
);
my %cb_call_back = (
    0 => sub {return},
    6 => \&xdr_cb_compound,
    %common_call_back
);

# Retry the NFS request for the following errors
# The NFS4ERR_DELAY -  a retry of the NFS request
my %retry_errors = (NFS4ERR_DELAY()   => 1,
                    NFS4ERR_GRACE()   => 2,);

#---------------------------------------------------------------------

=head2 new

  $nfs_obj = nfs::NfsClient->new(hostname => $host,
                                 version => 2|3|4
                                 [, proto => 'udp'|'tcp' ]
                                 [, mnt_ver => 1|2|3 ]
                                 [, mnt_proto => 'udp'|'tcp' ]
                                 [, port => $port ]
                                 [, resvport => 0|1 ]
                                 [, xid => $xid ]
                                 [, retry => $retry ]
                                 [, reconnect => 0|1 ]
                                 [, timeout => $timeout ]
                                 [, qsize => $qsize ]
                                 [, localaddr => $localaddr ]);

NFS object constructor. It creates the RPC connection object to the
server and initializes the class variables.  For NFSv4, the sequence
id's for both open and lock are initialized to zero, the callback
queue and the list of open delegations are defined and the client
identifier and verifier are set to a random number so every instance
of this object is a completely different client.

The following named arguments are accepted:

=over

=item Arguments

=over

=item hostname

(Required) : Hostname or ip address of server to connect.

=item proto

(Optional) : Protocol of connection: 'udp', 'tcp' (default is 'tcp').

=item port

(Optional) : Port number of server to connect
(default is 0 which attempts to connect to the portmapper to
get the port number).

=item resvport

(Optional) : Bind to a reserved port number on the local machine.
A reserved port number is less than or equal to 1024. If this
option is true an attempt to bind to a reserved port number
between 600 and 1023 is made. On most unix machines, root
is the only user able to bind to a reserved port.

=item xid

(Optional) : Initial transaction id (default is a random-generated number).

=item retry

(Optional) : Number of times to retry each RPC call if response is not
received within the timeout period (default is 0).

=item reconnect

(Optional) : Reconnect if TCP connection is dropped. This only works for
TCP connections and when the retry parameter is non-zero (default is 0)

=item timeout

(Optional) : Timeout value for connection (default is 30).

=item qsize

(Optional) : Size of RPC call queue.

=item localaddr

(Optional) : Hostname or ip address of local machine.

=item sec

(Optional) : The type of security to be used. If not specified, it uses "sys" authentication by default.
The valid values are none, sys, krb5, krb5i, krb5p. Please note as of writing this code, krb5p and krb5i
are not yet supported in c-Mode.

=back

=item Returns

NfsClient object

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub new($%) {
    my $class = shift;
    $Options->process();

    my (%opts) = validate (
        @_,
        {
            hostname  => { type => SCALAR },
            dstaddr   => { type => SCALAR, optional => 1 },
            minor_version   => { type => SCALAR, optional => 1,default => 1 },
            version   => { type => SCALAR,    regex => qr/^[234]$/ },
            resvport  => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 0 },
            proto     => { type => SCALAR,    regex => qr/^(udp|tcp)$/,
                                            default => 'tcp' },
            port      => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 0},
            xid       => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => int(rand(1000000000)) },
            retry     => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 0 },
            nfs_retry => {type    => SCALAR, regex   => qr/^\d+$/,
                                            default => 5},
            nfs_delay => {type    => SCALAR, regex   => qr/^\d+$/,
                                            default => 4},
            reconnect => { type => SCALAR,    regex => qr/^[01]$/,
                                            default => 0 },
            timeout   => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 30 },
            qsize     => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 0 },
            clientid  => { type => SCALAR,    regex => qr/^\d+$/,
                                           optional => 1,
                                            default => $client_id },
            localaddr => { type => SCALAR, optional => 1 },
            addr_type => { type => SCALAR, default => nfs::Rpc->addr_type()},
            cb        => { type => SCALAR,  optional => 1, default => 1 },
            rpc_vers  => { type => SCALAR,  optional => 1, default => 2 },
            sec      => { type => SCALAR, default => nfs::Rpc->auth_type(), regex => qr/^(sys|krb5|krb5i|krb5p|none)$/,},

        }
    );

    $Log->debug(qq{NFSC:new:hostname = $opts{hostname}\n});
    my ($self, %rpc_opts_h);

    # Pass local address if it is given
    $rpc_opts_h{localaddr} = $opts{localaddr} if defined($opts{localaddr});
    $Log->debug (qq{NFSC:new:localaddr = $rpc_opts_h{localaddr}}) if ($rpc_opts_h{localaddr});

    $opts{port} = $opts{port} ? $opts{port} : 2049;

    # Create an rpc object for NFS (100003)
    my $rpc = nfs::Rpc->clnt_create(hostname  => $opts{hostname},
                                    addr_type => $opts{addr_type},
                                    version   => $opts{version},
                                    resvport  => $opts{resvport},
                                    proto     => $opts{proto},
                                    port      => $opts{port},
                                    xid       => $opts{xid},
                                    retry     => $opts{retry},
                                    timeout   => $opts{timeout},
                                    qsize     => $opts{qsize},
                                    reconnect => $opts{reconnect},
                                    program   => 100003,
                                    sec       => $opts{sec},
                                    rpc_vers  => $opts{rpc_vers},
                                    %rpc_opts_h);

    # Make sure connection is established
    # Burt 178530 null procedure does not return void
    # it returns a null string of 4 bytes
    eval { $rpc->clnt_call(proc => 0, auth_type=>0) };
    croak $@ if ($@ and $@ !~ /4 extra bytes/);

    # Get local host name
    $opts{localname} = (POSIX::uname())[1];

    my %v4_1options;
    %v4_1options = (
           co_verifier    => time + int(rand(1000000000)),     ##TODO: We are keeping the co_verifier field to 100
           co_ownerid     => time + int(rand(1000000000)),
           seqid4         => 0,
           sequenceid4    => 0,
           clientid4      => $client_id,
    );

    $self = bless { %opts, %v4_1options,
                    rpc      => $rpc,
                    expect_callback_on_delay => 0,
                    }, $class;

    $rpc->{cb} = 1 if ($self->{cb} == 1);

    my %xdr_options = (
        proc  => 1,
        out   => \&xdr_compound_res);

    $self->{xdr_options} = \%xdr_options;

    $self->{minor_version} = 1;

    # Return NFS object
    return $self;
}

#---------------------------------------------------------------------
=head2 callback_register

  $nfs_obj->callback_register([ version => $version ]
                              [, timeout => $timeout ]);

Registers the callback service for NFSv4 using the given
version, the same protocol transport as the object
and using a program number from the transient range.
This function is only supported on NFSv4.

The following named arguments are accepted:

=over

=item Arguments

=over

=item version

(Optional) : Version number of callback service to register
(default is version 1).

=item timeout

(Optional) : Timeout value for connection (default is 30).

=back

=item Returns

Nothing

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub callback_register($%) {
    my $self = shift;
	$Log->enter(qq{self dump} . Dumper($self));

    my (%opts) =
		validate(@_,
        {
            version => { type => SCALAR, regex => qr/^\d+$/, default => 1 },
			addr_type => { type => SCALAR, regex => qr/inet|inet4|inet6/, default => $self->{addr_type}},
            timeout => { type => SCALAR, regex => qr/^\d+$/, default => 30 },
        }
    );

    # Create the callback service using the given version,
    # the same protocol transport as the NfsClient object,
    # and use a program number from the transient range
    $self->{callback} =
		nfs::Rpc->svc_create(version  => $opts{version},
							proto    => $self->{proto},
							addr_type=> $opts{addr_type},
							timeout  => $opts{timeout},
							port	=> $self->{rpc}->{sock}->sockport(),
							create_sock => 0,
							register => 1,
							dispatch => \%call_back);

	## Donot create a new socket for this connection, use the $self->{rpc}->{sock} only.
	$self->{callback}->{sock} = $self->{rpc}->{sock};
    $self->{callback}->{dispatch} = \%call_back;

	$Log->exit(qq{callback is:} . Dumper($self->{callback}));

    return;
}

#---------------------------------------------------------------------

=head2 callback_process

  $nfs_obj->callback_process([ timeout => $time_in_mseconds ]);

Process a pending callback. It waits until a callback is
received. It only supports the CB_RECALL callback.
This function is only supported on NFSv4.

=over

=item Arguments

=over

=item timeout

(Optional) : Timeout value in milliseconds to check if any
callback is pending. A value of 0 returns immediately if no
callback is currently pending. If argument is not specified
it waits until a callback is ready with no time limit.

=back

=item Returns

True if callback is received and processed correctly.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub callback_process($;%) {
    $Log->enter();
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            timeout   => { type => SCALAR, regex => qr/^\d+$/, optional => 1 },
        }
    );
    my $ver = $self->{version};
    my ($item, $out, %options);

    if ( ! defined($self->{callback}) ) {
        croak "Must use callback_register before calling this function";
    }

   	$Log->debug("callback_process: begin:" . Dumper($self->{callback}));
    # Add the timeout argument if it is defined
    $options{timeout} = $opts{timeout} if defined($opts{timeout});
    $options{accept}  = 0;
    if ($self->rpc->{hidden_callback_flag}) {
        delete $self->rpc->{hidden_callback_flag};
        return;
    }

    # Wait until a callback is received
    eval {
        $out = $self->{callback}->svc_getreqset(%options);
    };

    if ($@ && $@ =~ /Timeout waiting for data to be received/) {
        # Basically there is no cb message to process. Check if
        # a cb null was processed already. If not, croak.
        croak "No cb was received:$@" if (! $self->{rpc}->{cb_null});
        $Log->warn(qq{No callback was received, other than the null itself. This is not a problem usually. But if the program is expecting a callback, this is a problem.});
        return;
    }

    # Process callback queue
    if ($self->{cb_queue}) {
	    while ( @{$self->{cb_queue}} ) {
	        $Log->debug("We are in the callback queue \n");
	        $item = shift @{$self->{cb_queue}};
	        if ( $item->{op} == OP_CB_RECALL ) {
	            # Recall open delegation
	            $self->delegreturn(fhandle => $item->{args}{fhandle},
	                               stateid => $item->{args}{stateid});
	
	        } elsif ( $item->{op} == OP_CB_LAYOUTRECALL ) {
	            # Recall a layout
	            $self->layoutreturn();
	        }
	    }
	}

    return $out;
}

#---------------------------------------------------------------------

=head2 null

  $nfs_obj->null();

Calls the NULL procedure on the server.  This procedure takes
no arguments and returns nothing.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub null($;) {
    my $self = shift;
    my $rpc = $self->{rpc};

    # Call null procedure on server
    return $rpc->clnt_call(proc => 0);
}

#---------------------------------------------------------------------

=head2 getattr

  $out = $nfs_obj->getattr([ fhandle => $fhandle ]
                           [, attr => \@fattr ]);

Calls the GETATTR procedure on the server.
Retrieves the attributes of the specified file system object.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the object whose attributes
are to be retrieved (default is the current filehandle).

=item attr

(Optional) : A reference to an array of attributes to be retrieved.
Only used by NFSv4 (default is to get the same attributes as NFSv3).

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub getattr($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR,   default => $self->{curr_fh} },
            attr    => { type => ARRAYREF, default => [ FATTR4_TYPE,
                                                        FATTR4_SIZE,
                                                        FATTR4_FSID,
                                                        FATTR4_FILEID,
                                                        FATTR4_MODE,
                                                        FATTR4_NUMLINKS,
                                                        FATTR4_NO_TRUNC,
                                                        FATTR4_OWNER,
                                                        FATTR4_OWNER_GROUP,
                                                        FATTR4_RAWDEV,
                                                        FATTR4_TIME_ACCESS,
                                                        FATTR4_TIME_METADATA,
                                                        FATTR4_TIME_MODIFY,
                                                      ] },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Convert filehandle of object into XDR format
    $Log->debug(qq{calling xdr_fhandle with: $opts{fhandle}\n});
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);

    # Convert the list of attributes
    my $attr = bitmap4_convert(attrlist=>$opts{attr});
    # Create compound request
    my @ops;
    my $seq_arg = $self->_sequence();
    @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
             { op => OP_PUTFH,    args => $fhandle },
             { op => OP_GETATTR,  args => xdr_bitmap4(encode=>$attr) } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'getattr',
                                     ops => \@ops );

    # Create a copy of the fattr4 reference for ease of use
    $out->{fattr} = $out->{results}[2]{result}{fattr}{attr_vals};

    return $out;
}

#---------------------------------------------------------------------

=head2 setattr

  $out = $nfs_obj->setattr(attr => \%sattr
                           [, fhandle => $fhandle ]
                           [, ctime => \%nfstime3 ]);

Calls the SETATTR procedure on the server.
Changes one or more attributes of a file system object.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the object whose attributes
are to be changed (default is the current filehandle).

=item attr

(Required) : A reference to a hash of attributes to be changed.

=item ctime

(Optional) : A reference to an nfstime3 hash. If provided, the
server is to verify that this matches the ctime for the object.
If there is no match, the server preserves the object attributes
and returns an error. Not used in NFSv2.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------

sub setattr($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            attr    => { type => HASHREF },
            ctime   => { type => HASHREF, optional => 1 },
        }
    );


    my $retry = $opts{retry};
    my $retry_delay = $opts{delay};

    my (@caller_out) =  caller();
    $Log->debug(qq{setattr caller:@caller_out});
    my ($out, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my $min_ver = 0;

    if ($self->{minor_version} == 1) {
        $min_ver = 1;
        $seq_arg = $self->_sequence();
         }

    # Convert filehandle of object and
    # the sattr structure into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
    $Log->debug(qq{fhandle = $fhandle, opts{fhandle} = $opts{fhandle}\n});
    my $sattr   = xdr_sattr(encode=>$opts{attr}, version=>$ver);

    if ( $ver == 3 ) {
        if ( defined($opts{ctime}) ) {
            # Verify ctime
            $sattr .= xdr_uint(encode=>1) .
                      xdr_nfstime(encode=>$opts{ctime}, version=>$ver);
        } else {
            # Do not verify ctime
            $sattr .= xdr_uint(encode=>0);
        }
    }

    if ( $ver == 4 || $ver == 1) {
        my @ops;
        # Create compound request
        if ($min_ver == 1){
		@ops = (  { op => OP_SEQUENCE, args => $seq_arg } ,
			{ op => OP_PUTFH,   args => $fhandle } );
	} else {
        @ops = ( { op => OP_PUTFH,   args => $fhandle } );
	}
        if ( defined($opts{ctime}) ) {
            # Verify ctime
            my $ctime = { FATTR4_TIME_METADATA() => $opts{ctime} };
            push @ops, { op => OP_VERIFY, args => xdr_fattr4(encode=>$ctime) };
        }
        push @ops, { op => OP_SETATTR, args => $sattr };
        #retry block(Ref :Burt 503767)
		{
        # Call the compound procedure
        $out = $rpc->clnt_call(proc => 1,
                               arg  => xdr_compound(tag => 'setattr',
							minor => $min_ver,
                                                    ops => \@ops),
                               out  => \&xdr_compound_res);
		 # Retry call
            if ($retry-- and $out and $out->{status} == NFS4ERR_DELAY) {
               sleep $retry_delay;
               redo;
            }
        }
    } else {
        # Call correct procedure on server
        $out = $rpc->clnt_call(proc => NFSPROC_SETATTR,
                               arg  => $fhandle . $sattr,
                               out  => sub {xdr_setattr(version=>$ver, @_)});
    }

    # Check for errors
    $self->error_check(name=>'SETATTR', result=>$out);

    # Create a copy of the attrsset reference for ease of use
    if ( $ver == 4 || $ver == 1) {
        $out->{attrsset} = $out->{results}[1]{result}{attrsset};
    }

    return $out;
}

#---------------------------------------------------------------------

=head2 lookup

  $out = $nfs_obj->lookup(name => $name [, namelen => $namelen ]
                          [, dhandle => $fhandle ]);

Calls the LOOKUP procedure on the server.
Searches a directory for a specific name and returns the
filehandle for the corresponding file system object.
It sets the current filehandle.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory to search
(default is the current filehandle).

=item name

(Required) : The name of the object to find.

=item namelen

(Optional) : Force the length of name to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub lookup($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle => { type => SCALAR, default => $self->{curr_fh} },
            name    => { type => SCALAR },
            namelen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my ($out, $fhandle, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %len = ( len => $opts{namelen} ) if defined($opts{namelen});
    $seq_arg = $self->_sequence();

    # Convert filehandle of directory and
    # the name of object into XDR format
    my $dhandle = xdr_fhandle(encode=>$opts{dhandle}, version=>$ver);
    my $name    = xdr_opaque(encode=>$opts{name}, %len);

    # Create compound request
    my @ops = (  { op => OP_SEQUENCE, args => $seq_arg },
                 { op => OP_PUTFH,    args => $dhandle },
                 { op => OP_LOOKUP,   args => $name },
                 { op => OP_GETFH } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'lookup',
                                     ops => \@ops );

    # Create a copy of the filehandle for ease of use
    $out->{fhandle} = $out->{results}[-1]{result}{fhandle} ;

    # Set the current filehandle
    $self->current_fh_set(fhandle=>$out->{fhandle});

    return $out;
}

#---------------------------------------------------------------------

=head2 lookupp

  $out = $nfs_obj->lookupp([ dhandle => $fhandle ]);

Calls the LOOKUPP procedure on the server.
Gets the filehandle for the parent directory of the directory
entry specified by the given filehandle.
It is not supported on NFSv2 and v3; use lookup with '..' instead.
It sets the current filehandle.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory to search
(default is the current filehandle).

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub lookupp($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle => { type => SCALAR, default => $self->{curr_fh} },
        }
    );
    my ($out, $fhandle);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my $seq_arg = $self->_sequence();

    # Convert filehandle of directory into XDR format
    my $dhandle = xdr_fhandle(encode=>$opts{dhandle}, version=>$ver);

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
		{ op => OP_PUTFH,    args => $dhandle },
                { op => OP_LOOKUPP },
                { op => OP_GETFH } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'lookupp',
                                     ops => \@ops );

    # Get the filehandle of object to save it
    $out->{fhandle} = $out->{results}[-1]{result}{fhandle};

    # Set the current filehandle
    $self->current_fh_set(fhandle=>$out->{fhandle});

    return $out;
}

#---------------------------------------------------------------------

=head2 access

  $out = $nfs_obj->access(access => $access
                          [, fhandle => $fhandle ]);

Calls the ACCESS procedure on the server.
Determines the access rights that a user, as identified by the
credentials in the request, has with respect to a file system object.
This function is not supported on NFSv2.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the file system object to which access
is to be checked (default is the current filehandle).

=item access

(Required) : Bit mask of access permissions to check.
Valid access mode bits: ACCESS_READ, ACCESS_LOOKUP, ACCESS_MODIFY,
ACCESS_EXTEND, ACCESS_DELETE and ACCESS_EXECUTE.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub access($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            access  => { type => SCALAR,   regex => qr/^\d+$/ },
        }
    );
    my ($out, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    $seq_arg = $self->_sequence();

    # Convert filehandle of object and
    # the requested access bit mask into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
    my $access  = xdr_uint(encode=>$opts{access});

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                { op => OP_PUTFH,    args => $fhandle },
                { op => OP_ACCESS,   args => $access } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'access',
                                     ops => \@ops );

    return $out;
}

#---------------------------------------------------------------------

=head2 readlink

  $out = $nfs_obj->readlink([ fhandle => $fhandle ]);

Calls the READLINK procedure on the server.
Reads the pathname stored in a symbolic link.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of symbolic link
(default is the current filehandle).

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub readlink($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
        }
    );
    my ($out, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    $seq_arg = $self->_sequence();

    # Convert filehandle of object into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                { op => OP_PUTFH,    args => $fhandle },
                { op => OP_READLINK } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'readlink',
                                     ops => \@ops );

    # Create a copy of the link data reference for ease of use
    $out->{data} = $out->{results}[1]{result}{data};

    return $out;
}

#---------------------------------------------------------------------

=head2 mkdir

  $out = $nfs_obj->mkdir(name => $name [, namelen => $namelen ],
                         attr => \%sattr
                         [, dhandle => $fhandle ]);

Calls the MKDIR procedure on the server.
Creates a new directory.
This function is not supported on NFSv4;
use create instead.
It sets the current filehandle.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory in which the new directory
is to be created (default is the current filehandle).

=item name

(Required) : The name of the new directory.

=item attr

(Required) : The initial attributes for the new directory.

=item namelen

(Optional) : Force the length of name to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub mkdir($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle => { type => SCALAR, default => $self->{curr_fh} },
            name    => { type => SCALAR },
            attr    => { type => HASHREF },
            namelen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %len = ( len => $opts{namelen} ) if defined($opts{namelen});

    # marr TODO: confirm this
    if ( $ver == 4 || $ver == 1 ) {
        # MKDIR is not supported on NFSv4;
        # use CREATE instead
        croak "MKDIR is not supported on NFSv4";
    }

    # Convert filehandle of directory, name of directory
    # to create and initial attributes into XDR format
    my $args = xdr_fhandle(encode=>$opts{dhandle}, version=>$ver) .
               xdr_opaque(encode=>$opts{name}, %len) .
               xdr_sattr(encode=>$opts{attr}, version=>$ver);

    # Call correct procedure on server
    my $proc = ($ver == 2) ? NFSPROC_MKDIR : NFSPROC3_MKDIR;
    $out = $rpc->clnt_call(proc => $proc,
                           arg  => $args,
                           out  => sub {xdr_mkdir(version=>$ver, @_)});

    # Check for errors
    $self->error_check(name=>'MKDIR', result=>$out);

    # Set the current filehandle
    $self->current_fh_set(fhandle=>$out->{fhandle});

    return $out;
}

#---------------------------------------------------------------------

=head2 symlink

  $out = $nfs_obj->symlink(name => $name [, namelen => $namelen ],
                           data => $linktext [, datalen => $datalen ]
                           [, dhandle => $fhandle ]);

Calls the SYMLINK procedure on the server.
Creates a new symbolic link.
This function is not supported on NFSv4;
use create instead.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory in which the new symbolic
link is to be created (default is the current filehandle).

=item name

(Required) : The name of the new symbolic link.

=item data

(Required) : The string containing the symbolic link data.

=item namelen

(Optional) : Force the length of name to this number.

=item datalen

(Optional) : Force the length of data to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub symlink($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle => { type => SCALAR, default => $self->{curr_fh} },
            name    => { type => SCALAR },
            data    => { type => SCALAR },
            namelen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
            datalen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %nlen = ( len => $opts{namelen} ) if defined($opts{namelen});
    my %dlen = ( len => $opts{datalen} ) if defined($opts{datalen});

    croak "SYMLINK is not supported on NFSv4";

}

#---------------------------------------------------------------------

=head2 mknod

  $out = $nfs_obj->mknod(name => $name [, namelen => $namelen ],
                         type => $type,
                         attr => \%sattr
                         [, dhandle => $fhandle ]
                         [, major => $major, minor => $min ]);

Calls the MKNOD procedure on the server.
Creates a new special file: device, sockets or named pipes.
This function is not supported on NFSv2 and v4;
use create instead.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory in which the new special
file is to be created (default is the current filehandle).

=item name

(Required) : The name of the new special file.

=item type

(Required) : The type of special file to be created: NFBLK, NFCHR,
NFSOCK or NFFIFO.

=item attr

(Required) : The initial attributes for the new special file.

=item major

(Optional) : The major device number, only for type NFBLK or NFCHR.

=item minor

(Optional) : The minor device number, only for type NFBLK or NFCHR.

=item namelen

(Optional) : Force the length of name to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub mknod($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle => { type => SCALAR, default => $self->{curr_fh} },
            name    => { type => SCALAR },
            type    => { type => SCALAR, regex => qr/^\d+$/ },
            attr    => { type => HASHREF },
            major   => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
            minor   => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
            namelen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %len = ( len => $opts{namelen} ) if defined($opts{namelen});

    croak "MKNOD is not supported on NFSv$ver";

}

#---------------------------------------------------------------------

=head2 remove

  $out = $nfs_obj->remove(name => $name [, namelen => $namelen ]
                          [, dhandle => $fhandle ]);

Calls the REMOVE procedure on the server.
Removes an entry from a directory. If the entry in the directory
was the last reference to the corresponding file system object,
the object may be destroyed.
Removing a directory object is only supported on NFSv4.
For NFSv2 and v3 use rmdir instead.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory in which the entry
is to be removed (default is the current filehandle).

=item name

(Required) : The name of the entry to be removed.

=item namelen

(Optional) : Force the length of name to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub remove($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle => { type => SCALAR, default => $self->{curr_fh} },
            name    => { type => SCALAR },
            namelen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %len = ( len => $opts{namelen} ) if defined($opts{namelen});

    my $seq_arg = $self->_sequence();

    # Convert filehandle of directory and
    # name of object into XDR format
    my $dhandle = xdr_fhandle(encode=>$opts{dhandle}, version=>$ver);
    my $name    = xdr_opaque(encode=>$opts{name}, %len);

    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                { op => OP_PUTFH,    args => $dhandle },
                { op => OP_REMOVE,   args => $name } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'remove',
                                     ops => \@ops );

    return $out;
}

#---------------------------------------------------------------------

=head2 rename

  $out = $nfs_obj->rename(fromdir => $f_fhandle,
                          fromname => $o_name [, fromlen => $fromlen ],
                          todir => $t_handle,
                          toname => $n_name [, tolen => $tolen ]);

Calls the RENAME procedure on the server.
Renames the directory entry identified by 'fromname' in the
directory 'fromdir' to 'toname' in the directory 'todir'.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fromdir

(Required) : Filehandle of the directory from which the entry
is to be renamed.

=item fromname

(Required) : The name of the entry to be renamed.

=item todir

(Required) : Filehandle of the directory to which the entry
is to be renamed.

=item toname

(Required) : The new name for the object.

=item fromlen

(Optional) : Force the length of fromname to this number.

=item tolen

(Optional) : Force the length of toname to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub rename($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fromdir  => { type => SCALAR },
            fromname => { type => SCALAR },
            todir    => { type => SCALAR },
            toname   => { type => SCALAR },
            fromlen  => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
            tolen    => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my ($out, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %flen = ( len => $opts{fromlen} ) if defined($opts{fromlen});
    my %tlen = ( len => $opts{tolen} ) if defined($opts{tolen});

    $seq_arg = $self->_sequence();

    # Convert filehandle of the from directory, the original object name,
    # the filehandle of the to directory and the new name of object
    # into XDR format
    my $fromdir  = xdr_fhandle(encode=>$opts{fromdir}, version=>$ver);
    my $fromname = xdr_opaque(encode=>$opts{fromname}, %flen);
    my $todir    = xdr_fhandle(encode=>$opts{todir}, version=>$ver);
    my $toname   = xdr_opaque(encode=>$opts{toname}, %tlen);

    # Create compound request
    # The rename operation must use the saved
    # filehandle as the from directory and the
    # current filehandle as the to directory
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                { op => OP_PUTFH,    args => $fromdir },
                { op => OP_SAVEFH },
                { op => OP_PUTFH,    args => $todir },
                { op => OP_RENAME,   args => $fromname . $toname } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'rename',
                                     ops => \@ops );

    return $out;
}

#---------------------------------------------------------------------

=head2 link

  $out = $nfs_obj->link(todir => $t_fhandle,
                        name => $name [, namelen => $namelen ]
                        [, fhandle => $fhandle ]);

Calls the LINK procedure on the server.
Creates a hard link 'name' in the directory 'todir' for object
specified by 'fhandle'.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the existing file system object
(default is the current filehandle).

=item todir

(Required) : Filehandle of the directory in which the link
is to be created.

=item name

(Required) : The name that is to be associated with the created link.

=item namelen

(Optional) : Force the length of name to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub link($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            todir   => { type => SCALAR },
            name    => { type => SCALAR },
            namelen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %len = ( len => $opts{namelen} ) if defined($opts{namelen});
    my $seq_arg;

    $seq_arg = $self->_sequence();

    # Convert filehandle of the original object,
    # the filehandle of the to directory and the
    # name of hard link into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
    my $todir   = xdr_fhandle(encode=>$opts{todir}, version=>$ver);
    my $name    = xdr_opaque(encode=>$opts{name}, %len);

    # Create compound request
    # The link operation must use the saved filehandle
    # as the existing file system object and the
    # current filehandle as the to directory
    # where the link is to be created
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg},
                { op => OP_PUTFH,    args => $fhandle },
                { op => OP_SAVEFH },
                { op => OP_PUTFH,    args => $todir },
                { op => OP_LINK,     args => $name } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'link',
                                     ops => \@ops );

    return $out;
}

#---------------------------------------------------------------------

=head2 readdir

  $out = $nfs_obj->readdir([ dhandle => $fhandle ]
                           [, dircount => $dcount ]
                           [, maxcount => $mcount ]
                           [, cookie => $cookie ]
                           [, cookieverf => $cookieverf ]
                           [, attr => \@attr ]);

Calls the READDIR procedure on the server.
Retrieves a variable number of entries, in sequence, from a
directory and returns the name and file identifier for each,
with information to allow the client to request additional
directory entries in a subsequent READDIR request.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory to be read
(default is the current filehandle).

=item dircount

(Optional) : Maximum number of bytes of directory information returned
(default is 4096). Only used in NFSv4.

=item maxcount

(Optional) : Maximum size of the reply in bytes (default is 4096).

=item cookie

(Optional) : This should be set to 0 in the first request to read the
directory. On subsequent requests, it should be a cookie as returned
by the server (default is 0).

=item cookieverf

(Optional) : This should be set to a null string in the first request
to read the directory. On subsequent requests, it should be a
cookieverf as returned by the server (default is the null string).

=item attr

(Optional) : A reference to an array of attributes to be retrieved,
only used by NFSv4 (default is not to retrieve any attributes).

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub readdir($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle    => { type => SCALAR,   default => $self->{curr_fh} },
            maxcount   => { type => SCALAR,   default => 4096,
                                                regex => qr/^\d+$/ },
            dircount   => { type => SCALAR,   default => 4096,
                                                regex => qr/^\d+$/ },
            cookie     => { type => SCALAR,   default => 0,
                                                regex => qr/^\d+$/ },
            cookieverf => { type => SCALAR,   default => '' },
            attr       => { type => ARRAYREF, default => [] },
        }
    );
    my ($args, $out, $cookieverf, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Convert filehandle of directory to be read,
    # the cookie and the count into XDR format
    my $dhandle = xdr_fhandle(encode=>$opts{dhandle}, version=>$ver);
    my $cookie  = xdr_cookie(encode=>$opts{cookie}, version=>$ver);
    my $count   = xdr_uint(encode=>$opts{maxcount});

    # NFSv3 and v4 require a cookieverf structure
    $cookieverf = xdr_cookieverf(encode=>$opts{cookieverf});

    $seq_arg = $self->_sequence();

    # Convert the list of attributes
    my $attr = bitmap4_convert(attrlist=>$opts{attr});
    # Create arguments for compound request
    $args = $cookie . $cookieverf .
            xdr_uint(encode=>$opts{dircount}) . $count .
            xdr_bitmap4(encode=>$attr);

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
             { op => OP_PUTFH,   args => $dhandle },
             { op => OP_READDIR, args => $args }, );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'readdir',
                                     ops => \@ops );

    # Create a copy of the directory list reference for ease of use
    $out->{eof}        = $out->{results}[2]{result}{eof};
    $out->{cookieverf} = $out->{results}[2]{result}{cookieverf};
    $out->{list}       = $out->{results}[2]{result}{list};

    return $out;
}

#---------------------------------------------------------------------

=head2 readdirplus

  $out = $nfs_obj->readdirplus([ dhandle => $fhandle ]
                               [, dircount => $dcount ]
                               [, maxcount => $mcount ]
                               [, cookie => $cookie ]
                               [, cookieverf => $cookieverf ]);

Calls the READDIRPLUS procedure on the server.
Retrieves a variable number of entries, in sequence, from a
directory and returns complete information about each entry along
with information to allow the client to request additional
directory entries in a subsequent READDIRPLUS request.
This function is only supported on NFSv3. For NFSv4, use readdir
with the attr option set to get similar functionality.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory to be read
(default is the current filehandle).

=item dircount

(Optional) : Maximum number of bytes of directory information returned
(default is 4096).

=item maxcount

(Optional) : Maximum size of the reply in bytes (default is 4096).

=item cookie

(Optional) : This should be set to 0 in the first request to read the
directory. On subsequent requests, it should be a cookie as returned
by the server (default is 0).

=item cookieverf

(Optional) : This should be set to a null string in the first request
to read the directory. On subsequent requests, it should be a
cookieverf as returned by the server (default is the null string).

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub readdirplus($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle    => { type => SCALAR, default => $self->{curr_fh} },
            dircount   => { type => SCALAR, default => 4096,
                                              regex => qr/^\d+$/ },
            maxcount   => { type => SCALAR, default => 4096,
                                              regex => qr/^\d+$/ },
            cookie     => { type => SCALAR, default => 0,
                                              regex => qr/^\d+$/ },
            cookieverf => { type => SCALAR, default => 0 },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # READDIRPLUS is only supported on NFSv3
    # Use READDIR on NFSv4 to get similar results
    croak "READDIRPLUS is not supported on NFSv$ver";
}

#---------------------------------------------------------------------

=head2 fsstat

  $out = $nfs_obj->fsstat([ fhandle => $fhandle ]);

Calls the FSSTAT(v3)/STATFS(v2) procedure on the server.
Retrieves volatile file system state information.
This function is only supported on NFSv2 and v3.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle identifying an object in the file system
(default is the current filehandle).

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub fsstat($%) {
    my $self = shift;

    # FSSTAT is not supported on NFSv4
    croak "FSSTAT is not supported on NFSv4";

    return 0;
}

#---------------------------------------------------------------------

=head2 read

  $out = $nfs_obj->read(offset => $offset,
                        count => $count
                        [, fhandle => $fhandle ]
                        [, stateid => \%stateid|STATEID0|STATEID1 ]);

Calls the READ procedure on the server.
Reads the data from a given offset in a file returning the data.
For NFSv2 and v3, the file attributes are returned as well.
Must use open before calling this function for NFSv4,
unless the stateid is either STATEID0 or STATEID1.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the file to be read
(default is the current filehandle).

=item offset

(Required) : The position within the file at which the read is to begin.

=item count

(Required) : The number of bytes of data that are to be read.

=item stateid

(Optional) : Reference to a hash containing the stateid of the opened file.
The constants STATEID0 or STATEID1 can be used instead of the hash
for the special stateid of all 0's or all 1's respectively.
Only required for NFSv4.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub read($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle     => { type => SCALAR,          default => $self->{curr_fh} },
            offset      => { type => SCALAR,            regex => qr/^\d+$/ },
            count       => { type => SCALAR,            regex => qr/^\d+$/ },
            seqid_state => { type => SCALAR,         optional => 1, default => undef },
            stateid     => { type => HASHREF|SCALAR, optional => 1 },
        }
    );
    my ($out, $offset, $args);
    my $ver     = $self->{version};

    # Convert filehandle of file
    # and the count into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
    my $count   = xdr_uint(encode=>$opts{count});

    # Convert offset into XDR format
    $offset = xdr_uint64(encode=>$opts{offset});

    # Convert the stateid into XDR format and
    # append all other arguments to operation
    my $own_stateid;
    if (($opts{stateid} != STATEID0 || $opts{stateid} != STATEID1) && defined($opts{seqid_state})) {
        my %stateid_h = (seqid => $opts{seqid_state}, other => $opts{stateid}{other});
        $own_stateid = \%stateid_h;
    } else {
        $own_stateid = $opts{stateid};
    }
    my $seq_arg = $self->_sequence();	
    $args = xdr_stateid4(encode=>$own_stateid) . $offset . $count;

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                { op => OP_PUTFH,    args => $fhandle },
                { op => OP_READ,     args => $args }, );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'read',
                                     ops => \@ops );

    # Create a copy of the read data reference for ease of use
    $out->{eof}  = $out->{results}[-1]{result}{eof};
    $out->{data} = $out->{results}[-1]{result}{data};
    $out->{results}[-1]{result}{data} = \$out->{data};

    return $out;
}

#---------------------------------------------------------------------

=head2 write

  $out = $nfs_obj->write(offset => $offset,
                         data => $data [, datalen => $datalen ]
                         [, fhandle => $fhandle ]
                         [, stable => $stable_how ]
                         [, stateid => \%stateid|STATEID0|STATEID1 ]);

Calls the WRITE procedure on the server.
Writes the data to a given offset in a file.
Must use open before calling this function for NFSv4,
unless the stateid is either STATEID0 or STATEID1.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the file to be written
(default is the current filehandle).

=item offset

(Required) : The position within the file at which the write is to begin.

=item data

(Required) : The data to be written.

=item stable

(Optional) : How the write is to be performed: UNSTABLE, DATA_SYNC
or FILE_SYNC (default). Only used in NFSv3 and v4.

=item stateid

(Optional) : Reference to a hash containing the stateid of the opened file.
The constants STATEID0 or STATEID1 can be used instead of the hash
for the special stateid of all 0's or all 1's respectively.
Only required for NFSv4.

=item datalen

(Optional) : Force the length of data to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub write($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            offset  => { type => SCALAR,   regex => qr/^\d+$/ },
            data    => { type => SCALAR },
            stable  => { type => SCALAR, default => FILE_SYNC,
                                           regex => qr/^\d+$/ },
            stateid => { type => HASHREF|SCALAR, optional => 1 },
            datalen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my ($out, $offset, $stable, $args);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %len = ( len => $opts{datalen} ) if defined($opts{datalen});
    my @ops;

    # use stateid(0) if not given.
    $opts{stateid} = $opts{stateid} ? $opts{stateid} : 0x0 ;

    # Convert filehandle of file into XDR format
    my $fhandle = xdr_fhandle(encode => $opts{fhandle},
                             version => $ver);

    $offset = xdr_uint64(encode => $opts{offset});
    $stable = xdr_uint(encode => $opts{stable});

    # Convert the stateid into XDR format and
    # append all other arguments to operation
    $args = xdr_stateid4(encode => $opts{stateid}) . $offset . $stable .
              xdr_opaque(encode => $opts{data}, %len);

    my $seq_arg = $self->_sequence();
    @ops = (  { op => OP_SEQUENCE, args => $seq_arg },
              { op => OP_PUTFH, args => $fhandle },
              { op => OP_WRITE, args => $args }, );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'write',
                                     ops => \@ops );

    return $out;
}

#---------------------------------------------------------------------

=head2 commit

  $out = $nfs_obj->commit([ fhandle => $fhandle ]
                          [, offset => $offset ]
                          [, count => $count ]);

Calls the COMMIT procedure on the server.
Forces or flushes data to stable storage that was previously written
with a WRITE procedure call with the stable field set to UNSTABLE.
COMMIT is not supported on NFSv2 since all writes are committed.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the file to be committed
(default is the current filehandle).

=item offset

(Optional) : The position within the file at which the commit is to begin
(default is 0).

=item count

(Optional) : The number of bytes of data to flush
(default is 0, whole file).

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub commit($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            offset  => { type => SCALAR, default => 0, regex => qr/^\d+$/ },
            count   => { type => SCALAR, default => 0, regex => qr/^\d+$/ },
        }
    );
    my ($out, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    $seq_arg = $self->_sequence();

    # Convert filehandle of file, offset
    # and count into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
    my $offset  = xdr_uint64(encode=>$opts{offset});
    my $count   = xdr_uint(encode=>$opts{count});

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
             { op => OP_PUTFH,  args => $fhandle },
             { op => OP_COMMIT, args => $offset . $count }, );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'commit',
                                     ops => \@ops );

    return $out;
}

#---------------------------------------------------------------------

=head2 open

  $out = $nfs_obj->open(name => $name [, namelen => $namelen ]
                        [, dhandle => $fhandle ]
                        [, access => $access ]
                        [, deny => $deny ]
                        [, cr_mode => $cr_mode
                           [, attr => \%sattr ] [, verf => $verf ] ]
                        [, owner => $owner [, ownrlen => $ownrlen ] ]
                        [, retry => $retry ]
                        [, deleg => $deleg ]);

Calls the OPEN procedure on the server.
This function opens and/or creates regular files.
OPEN resembles LOOKUP in that it generates a filehandle for the
client to use.  Unlike LOOKUP though, OPEN creates server state on
the filehandle.  In normal circumstances, the client can only release
this state with a CLOSE operation.
It is not supported on NFSv2 and v3; use create instead.
It sets the current filehandle.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory in which the file is located
or the new file is to be created (default is the current filehandle).

=item name

(Required) : The name of the file to be opened or created.

=item owner

(Optional) : Owner of file locking state (default is the current process id).

=item cr_mode

(Optional) : Create mode: UNCHECKED, GUARDED or EXCLUSIVE
(default is to just open the file without creating it).

=item attr

(Optional) : The initial attributes for the new file.
It is only used when create mode is either UNCHECKED or GUARDED
(default is no initial attributes).

=item access

(Optional) : Share access mode of open: OPEN4_SHARE_ACCESS_READ (default),
OPEN4_SHARE_ACCESS_WRITE or OPEN4_SHARE_ACCESS_BOTH.

=item deny

(Optional) : Share deny mode of open: OPEN4_SHARE_DENY_NONE (default),
OPEN4_SHARE_DENY_READ, OPEN4_SHARE_DENY_WRITE or OPEN4_SHARE_DENY_BOTH.

=item retry

(Optional) : Number of times to retry the request if NFS4ERR_GRACE
is returned (default is 0). Each retry takes 10 seconds.

=item deleg

(Optional) : Number of times to retry the request if NFS4ERR_DELAY
is returned (default is 0). Each retry takes 1 second.
A callback is processed returning the delegation before retrying
for the first time.

=item verf

(Optional) : Create verifier. Only used if cr_mode is EXCLUSIVE.
Default is the current time.

=item namelen

(Optional) : Force the length of name to this number.

=item ownrlen

(Optional) : Force the length of owner to this number.

=item deleg_type

(Optional) : CLAIM_NULL (No special rights to file. Ordinary OPEN of the specified file.),
             CLAIM_PREVIOUS (Right to the file established by an open previous to serve reboot.  File identified by filehandle obtained at that time rather than by name.),
             CLAIM_DELEGATE_CUR (Right to file based on a delegation granted by the server. File is specified by name.) or
             CLAIM_DELEGATE_PREV (Right to file based on a delegation granted to a previous boot instance of the client.  File is specified by name.)

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub open($%) {
   $Log->enter();
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle => { type => SCALAR, default => $self->{curr_fh} },
            name    => { type => SCALAR },
            owner   => { type => SCALAR,   default => $$ },
            attr    => { type => HASHREF, optional => 1 },
            access  => { type => SCALAR,   default => OPEN4_SHARE_ACCESS_READ,
                                             regex => qr/^\d+$/ },
            deny    => { type => SCALAR,   default => OPEN4_SHARE_DENY_NONE,
                                             regex => qr/^\d+$/ },
            cr_mode => { type => SCALAR,  optional => 1, regex => qr/^\d+$/ },
            retry   => { type => SCALAR,   default => 0, regex => qr/^\d+$/ },
            verf    => { type => SCALAR,   default => time },
            # This is not used and is here for compatibility
            deleg   => { type => SCALAR,   default => 0, regex => qr/^\d+$/ },
            namelen => { type => SCALAR,  optional => 1, regex => qr/^\d+$/ },
            ownrlen => { type => SCALAR,  optional => 1, regex => qr/^\d+$/ },
            deleg_type => { type => SCALAR, default => CLAIM_NULL, regex => qr/^\d+$/ },

        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my $retry = $opts{retry};
    my %nlen = ( len => $opts{namelen} ) if defined($opts{namelen});
    my %olen = ( len => $opts{ownrlen} ) if defined($opts{ownrlen});
    my @ops;

    my $seq_arg = $self->_sequence();

    # Convert filehandle of directory and the name
    # of the new file into XDR format
    my $dhandle = xdr_fhandle(encode=>$opts{dhandle}, version=>$ver);

    my $name;
    # For CLAIM_PREVIOUS, the given filename is assumed to be the filehandle itself.
    # This will become the "current filehandle" during the open operation.
    $name = "";
    if ($opts{deleg_type} == CLAIM_PREVIOUS) {
        $name = xdr_fhandle(encode => $opts{name}, version => $ver);
    } else {
        $name = xdr_opaque(encode => $opts{name}, %nlen);
    }

    my $args;

    $args = xdr_uint(encode=>$self->{seqid4}++) .
            # Share access,
            xdr_uint(encode=>$opts{access}) .
            # Share deny,
            xdr_uint(encode=>$opts{deny}) .
            # Owner: client id
            $self->{clientid4} .
            #        owner
            xdr_opaque(encode=>$opts{owner}, %olen);

    # Open type
    if ( ! defined($opts{cr_mode}) ) {
        # OPEN4_NOCREATE
        $args .= xdr_uint(encode=>0);
    } else {
        # OPEN4_CREATE includes the create mode
        # and initial attributes if not EXCLUSIVE
        $args .= xdr_uint(encode=>1) .
                 xdr_uint(encode=>$opts{cr_mode});
        if ( $opts{cr_mode} == EXCLUSIVE ) {
            # Add verifier4 if exclusive create is requested
            # No initial attributes are set with option
            $args .= xdr_opaque(encode=>xdr_uint(encode=>$opts{verf}),
                                fixed=>1, maxlen=>8);
        } else {
            # UNCHECKED and GUARDED require initial attributes
            $args .= xdr_fattr4(encode=>$opts{attr});
        }
    }

    # set CLAIM type
    $args .= xdr_uint(encode=>$opts{deleg_type});

    if ($opts{deleg_type} == CLAIM_DELEGATE_CUR) {
        # Find deleg id
        my $found = 0;
        my $open_deleg;
        my $saved_arg;
        foreach $open_deleg (@{$self->{open_delegs}}) {
            if ($opts{name} eq $open_deleg->{name}) {
                $saved_arg = xdr_stateid4(encode => ($open_deleg->{stateid}));
                $found = 1;
            }
        }
        croak( "OPEN with CLAIM_DELEGATE_CUR: delegation-id is not found for file $opts{name}") if (!$found);
        $args .= $saved_arg;
        $args .= $name;
    } elsif ($opts{deleg_type} == CLAIM_PREVIOUS) {

        # We assume that the name is actually the filehandle itself!
        # For the next few operands that are coming up, dhandle is the current filehandle.
        # But in case of CLAIM_PREVIOUS, the current filehandle has to be set to the actual
        # handle of the file to be opened. So for further processing, we need to set the
        # dhandle to the actual filehandle ($name). Remaining operations are same as the
        # case of CLAIM_DELEGATE_CUR.
        $Log->debug("changing the dhandle to the name itself: $name\n");
        $dhandle = $name;
        if ($opts{access} == OPEN4_SHARE_ACCESS_READ) {
            $args .= xdr_uint(encode => OPEN_DELEGATE_READ);
        } elsif ($opts{access} == OPEN4_SHARE_ACCESS_WRITE) {
            $args .= xdr_uint(encode => OPEN_DELEGATE_WRITE);
        }
    ## end else [ if ($opts{deleg_type} ...
    } else {
        $args .= $name;
    }

    if($opts{deleg_type} == CLAIM_PREVIOUS) {
        @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                 { op => OP_PUTFH,    args => $dhandle },
                 { op => OP_OPEN,     args => $args } );
    }
    else {
        @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                 { op => OP_PUTFH,    args => $dhandle },
                 { op => OP_OPEN,     args => $args },
                 { op => OP_GETFH }, );
    }

    # Call the compound procedure
    $out = $self->process_operation( tag => 'open',
                                     ops => \@ops );

    # Set the current filehandle
    if($opts{deleg_type} != CLAIM_PREVIOUS) {
        $self->current_fh_set(fhandle=>$out->{results}[-1]{result}{fhandle});
    }

    # Create a copy of the stateid reference for ease of use
    $out->{stateid} = $out->{results}[2]{result}{stateid};

    # Save the filehandle for this open
    $out->{fhandle} = $self->{curr_fh};

    if ( exists($out->{results}[2]{result}{deleg}) ) {
        # Create a copy of the delegation stateid reference for ease of use
        $out->{deleg_stateid} = $out->{results}[2]{result}{deleg}{stateid};
        # Save it in a list of open delegations
        push @{$self->{open_delegs}}, {
            fhandle => $self->curr_fh(),
            stateid => $out->{deleg_stateid},
            name    => $opts{name},
        };
    }

    return $out;
}

#---------------------------------------------------------------------

=head2 open_downgrade

  $out = $nfs_obj->open_downgrade(stateid => $stateid
                                  [, access => $access ]
                                  [, deny => $deny ]
                                  [, fhandle => $fhandle ]);

Calls the OPEN_DOWNGRADE procedure on the server.
This function is used to adjust the share access and share deny
bits for a given open. This is necessary when a given open owner
opens the same file multiple times with different share access and
share deny flags. In this situation, a close of one of the opens may
change the appropriate share access and share deny flags to remove
bits associated with opens no longer in effect.
It is only supported on NFSv4.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the file to be closed
(default is the current filehandle).

=item stateid

(Required) : Reference to a hash containing the stateid of the opened file.

=item access

(Optional) : Share access mode to set: OPEN4_SHARE_ACCESS_READ (default),
OPEN4_SHARE_ACCESS_WRITE or OPEN4_SHARE_ACCESS_BOTH.

=item deny

(Optional) : Share deny mode to set: OPEN4_SHARE_DENY_NONE (default),
OPEN4_SHARE_DENY_READ, OPEN4_SHARE_DENY_WRITE or OPEN4_SHARE_DENY_BOTH.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub open_downgrade($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            seqid   => { type => SCALAR, default => $self->{seqid} },
            stateid => { type => HASHREF },
            access  => { type => SCALAR, default => OPEN4_SHARE_ACCESS_READ,
                                           regex => qr/^\d+$/ },
            deny    => { type => SCALAR, default => OPEN4_SHARE_DENY_NONE,
                                           regex => qr/^\d+$/ },
        }
    );
    my ($out, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    $seq_arg = $self->_sequence();

    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);

    # Encode the Stateid of open,
    my $args = xdr_stateid4(encode=>$opts{stateid}) .
               # Sequence id,
               xdr_uint(encode=>$opts{seqid}++) .
               # Share access,
               xdr_uint(encode=>$opts{access}) .
               # Share deny
               xdr_uint(encode=>$opts{deny});

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE,       args => $seq_arg },
                { op => OP_PUTFH,          args => $fhandle },
                { op => OP_OPEN_DOWNGRADE, args => $args } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'open dgrade',
                                     ops => \@ops );

    # Create a copy of the stateid reference for ease of use
    #The required op is at the end of the compound request, so pick the last op.
    $out->{stateid} = $out->{results}[-1]{result}{stateid};

    return $out;
}

#---------------------------------------------------------------------

=head2 open_confirm

  $out = $nfs_obj->open_confirm(stateid => $stateid
                                [, fhandle => $fhandle ]);

Calls the OPEN_CONFIRM procedure on the server.
This function is used to confirm the sequence id usage for the first
time that an open owner is used by a client.
It is only supported on NFSv4.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the opened file to be confirmed
(default is the current filehandle).

=item stateid

(Required) : Reference to a hash containing the stateid of the opened file.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub open_confirm($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            stateid => { type => HASHREF },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

  my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);

    # Encode the Stateid of open,
    my $args = xdr_stateid4(encode=>$opts{stateid}) .
               # Sequence id,
               xdr_uint(encode=>$self->{seqid4}++);

    # Create compound request
    my @ops = ( { op => OP_PUTFH,        args => $fhandle },
                { op => OP_OPEN_CONFIRM, args => $args } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'open conf',
                                     ops => \@ops );

    # Create a copy of the stateid reference for ease of use
    #The required op is at the end of the compound request, so pick the last op.
    $out->{stateid} = $out->{results}[-1]{result}{stateid};

    return $out;
}

#---------------------------------------------------------------------

=head2 openattr

  $out = $nfs_obj->openattr([ fhandle => $fhandle ]
                            [, createdir => 0|1 ]);

Calls the OPENATTR procedure on the server.
This function is used to obtain the filehandle of the named attribute
directory associated with the specified filehandle.
It is only supported on NFSv4.
It sets the current filehandle.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the object whose named attribute directory
is to be retrieved (default is the current filehandle).

=item creaetdir

(Required) : Reference to a hash containing the stateid of the opened file.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub openattr($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle   => { type => SCALAR, default => $self->{curr_fh} },
            createdir => { type => SCALAR, default => 0, regex => qr/^[01]$/ },
        }
    );
    my ($out, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    $seq_arg = $self->_sequence();

    # Encode the filehandle
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);

    # Encode the createdir flag
    my $args = xdr_uint(encode=>$opts{createdir});

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
		{ op => OP_PUTFH,    args => $fhandle },
                { op => OP_OPENATTR, args => $args },
                { op => OP_GETFH }, );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'openattr',
                                     ops => \@ops );

    # Set the current filehandle
    $self->current_fh_set(fhandle=>$out->{results}[-1]{result}{fhandle});

    # Save the filehandle for this openattr
    $out->{fhandle} = $self->{curr_fh};

    return $out;
}

#---------------------------------------------------------------------

=head2 close

  $out = $nfs_obj->close(stateid => $stateid
                         [, fhandle => $fhandle ]);

Calls the CLOSE procedure on the server.
This function closes a regular file.
The CLOSE operation releases share reservations for the regular or
named attribute file as specified by the current filehandle.  The
share reservations and other state information released at the server
as a result of this CLOSE is only associated with the supplied
stateid.  The sequence id provides for the correct ordering. State
associated with other OPENs is not affected.
It is only supported on NFSv4.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the file to be closed
(default is the current filehandle).

=item stateid

(Required) : Reference to a hash containing the stateid of the opened file.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub close($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            stateid => { type => HASHREF },
            owner   => { type => SCALAR, default => $$ },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my @ops;
    my ($args,$seq_id,$seq_arg);

    $seq_arg = $self->_sequence();

    # Convert filehandle of file
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);

    # Encode the sequence id,
    $args = xdr_uint(encode=>$self->{seqid4}++) .
        # Stateid of open
        xdr_stateid4(encode=>$opts{stateid});
    # Create compound request
    @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
             { op => OP_PUTFH, args => $fhandle },
             { op => OP_CLOSE, args => $args }, );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'close',
                                     ops => \@ops );

    return $out;
}

#---------------------------------------------------------------------

=head2 lock

  $out = $nfs_obj->lock(offset => $offset,
                        count => $count
                        [, lock_stateid => \%lock_stateid |
                           stateid => \%stateid
                           [, owner => $owner [, ownrlen => $ownrlen ] ] ]
                        [, fhandle => $fhandle ]
                        [, type => $lock_type ]
                        [, reclaim => 0|1 ]
                        [, caller_name => $caller_name ]
                        [, sm_state => $sm_state ]
                        [, cookie => $cookie ]
                        [, svid => $svid ]);

Calls the LOCK (NFSv4) procedure on the server.
This function requests a record lock for the byte range specified
by the offset and count parameters.

On NFSv4 only, to get a lock using an existing
lock stateid (from a previous lock, using the same owner) use the
lock_stateid parameter. On the other hand, to get a lock using a new owner,
the stateid of the opened file and the owner must be specified.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the file to be locked
(default is the current filehandle).

=item offset

(Required) : The position within the file at which the lock is to begin.

=item count

(Required) : The number of bytes of data that are to be locked.

=item type

(Optional) : The type of lock to get: READ_LT (default), WRITE_LT,
READW_LT, WRITEW_LT.

=item reclaim

(Optional) : True for a reclaim request (default is 0).

=item owner

(Optional) : Owner of file locking state (default is the current process id).

=item stateid

(Optional) : Reference to a hash containing the stateid of the opened file.
It is only supported on NFSv4.

=item lock_stateid

(Optional) : Reference to a hash containing the stateid of a lock owner.
It is only supported on NFSv4.

=item ownrlen

(Optional) : Force the length of owner to this number.

=item caller_name

(Optional) : Identifies the client making the request
(default is the client's local address or host name).
It is only supported on NFSv2 and NFSv3.

=item sm_state

(Optional) : The current value of the client's status monitor state number
(default is the status monitor state number of the object).
It is only supported on NFSv2 and NFSv3.

=item cookie

(Optional) : Cookie used to match asynchronous response with a request
(default is '').
It is only supported on NFSv2 and NFSv3.

=item svid

(Optional) : Identifier of the process making the request
(default is the current process id).
It is only supported on NFSv2 and NFSv3.

=back

=item Returns

Reference to a hash containing the status and output of procedure
as well as the lock_stateid and the lockid to be used in locku()
and lock_cancel().

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub lock($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle      => { type => SCALAR,   default => $self->{curr_fh} },
            caller_name  => { type => SCALAR,   default => $self->{mon_host} },
            sm_state     => { type => SCALAR,   default => $self->{sm_state} },
            cookie       => { type => SCALAR,   default => '' },
            offset       => { type => SCALAR,     regex => qr/^\d+$/ },
            count        => { type => SCALAR,     regex => qr/^\d+$/ },
            type         => { type => SCALAR,   default => READ_LT,
                                                  regex => qr/^\d+$/ },
            reclaim      => { type => SCALAR,   default => 0,
                                                  regex => qr/^[01]$/ },
            owner        => { type => SCALAR,   default => $$ },
            stateid      => { type => HASHREF, optional => 1 },
            lock_stateid => { type => HASHREF, optional => 1 },
            ownrlen      => { type => SCALAR,  optional => 1,
                                                  regex => qr/^\d+$/ },
            svid         => { type => SCALAR,   default => $$,
                                                  regex => qr/^\d+$/ },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    if (!defined($opts{ownerlen})) {
	$opts{ownerlen} = length $opts{owner} ;
     }
    my %len = ( len => $opts{ownrlen} ) if defined($opts{ownrlen});

    my $seq_arg = $self->_sequence();
    my $clientid = $self->{clientid4};

    # Convert filehandle of file into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);

    # Encode the locktype,
    my $args = xdr_uint(encode=>$opts{type}) .
               # Reclaim,
               xdr_uint(encode=>$opts{reclaim}) .
               # Offset,
               xdr_uint64(encode=>$opts{offset}) .
               # Length,
               xdr_uint64(encode=>$opts{count});

    # Encode the locker:
    if ( defined($opts{lock_stateid}) ) {
        # Encode known locker
        $args .= xdr_uint(encode=>0) .
                 xdr_stateid4(encode=>$opts{lock_stateid}) .
                 xdr_uint(encode=>$self->{lock_seqid}++);
    } else {
        # Encode new locker
        $args .= xdr_uint(encode => 1) ;
        $args .= xdr_uint(encode => $self->{seqid4}++) ;
        $args .= xdr_stateid4(encode => $opts{stateid}) ;
        $args .= xdr_uint(encode => $self->{lock_seqid}++) ;
        # Owner: client id
        $args .= $clientid ;
        # owner
        $args .= xdr_opaque(encode=>$opts{owner}, %len);
    }

    # Create compound request
    my @ops;

    @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
             { op => OP_PUTFH, args => $fhandle },
             { op => OP_LOCK,  args => $args }, );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'lock',
                                     ops => \@ops );

    $out->{lock_stateid} = $out->{results}[2]{result}{stateid};

    # Save lockid to be used on locku()
    $out->{lockid} = {
        fhandle => $opts{fhandle},
        stateid => $out->{lock_stateid},
        offset  => $opts{offset},
        count   => $opts{count},
        type    => $opts{type},
    };

    return $out;
}

#---------------------------------------------------------------------

=head2 locku

  $out = $nfs_obj->locku([ lockid => \%lockid ]
                         [, stateid => $lock_stateid ]
                         [, offset => $offset ]
                         [, count => $count ]
                         [, fhandle => $fhandle ]
                         [, type => $lock_type ]
                         [, caller_name => $caller_name ]
                         [, cookie => $cookie ]
                         [, owner => $owner ]
                         [, svid => $svid ]);

Calls the LOCKU (NFSv4) procedure on the server.
This function unlocks the record lock specified by the parameters.

The following named arguments are accepted:

=over

=item Arguments

=over

=item lockid

(Optional) : Reference to a hash of the lock to be unlocked as returned by
the lock() function. All other options can be used to overwrite options
in lockid.
If this argument is not specified the following options are required:
offset, count and stateid(NFSv4).

=item fhandle

(Optional) : Filehandle of the file to be unlocked
(default is the current filehandle).

=item stateid

(Optional) : Reference to a hash containing the stateid of a lock owner.
It is only supported on NFSv4.

=item offset

(Optional) : The position within the file at which the unlock is to begin.

=item count

(Optional) : The number of bytes of data that are to be unlocked.

=item type

(Optional) : The type of lock to release: READ_LT (default), WRITE_LT,
READW_LT, WRITEW_LT.

=item caller_name

(Optional) : Identifies the client making the request
(default is the client's local address or host name).
It is only supported on NFSv2 and NFSv3.

=item cookie

(Optional) : Cookie used to match asynchronous response with a request
(default is '').
It is only supported on NFSv2 and NFSv3.

=item owner

(Optional) : Owner of file locking state (default is the current process id).
It is only supported on NFSv2 and NFSv3.

=item svid

(Optional) : Identifier of the process making the request
(default is the current process id).
It is only supported on NFSv2 and NFSv3.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub locku($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle     => { type => SCALAR,  optional => 1 },
            caller_name => { type => SCALAR,  optional => 1 },
            cookie      => { type => SCALAR,   default => '' },
            lockid      => { type => HASHREF, optional => 1 },
            stateid     => { type => HASHREF, optional => 1 },
            owner       => { type => SCALAR,  optional => 1 },
            offset      => { type => SCALAR,  optional => 1,
                                                 regex => qr/^\d+$/ },
            count       => { type => SCALAR,  optional => 1,
                                                 regex => qr/^\d+$/ },
            type        => { type => SCALAR,  optional => 1,
                                                 regex => qr/^\d+$/ },
            svid        => { type => SCALAR,  optional => 1,
                                                 regex => qr/^\d+$/ },
        }
    );
    my ($out, $args, $fhandle, @ops, $nver, $lockid, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    $seq_arg = $self->_sequence();

    # Get lockid from options
    $lockid = $opts{lockid};

    if ( defined($lockid) ) {
        # Explicit options have precedence
        $opts{fhandle} = $lockid->{fhandle} if not defined($opts{fhandle});
        $opts{stateid} = $lockid->{stateid} if not defined($opts{stateid});
        $opts{offset}  = $lockid->{offset}  if not defined($opts{offset});
        $opts{count}   = $lockid->{count}   if not defined($opts{count});
        $opts{type}    = $lockid->{type}    if not defined($opts{type});
    } else {
        # Default values for unspecified options
        $opts{fhandle} = $self->{curr_fh} if not defined($opts{fhandle});
        $opts{type}    = READ_LT          if not defined($opts{type});
    }

    # Convert filehandle of file into XDR format
    $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);

    # Encode the locktype,
    $args = xdr_uint(encode=>$opts{type}) .
            # Lock sequence id,
            xdr_uint(encode=>$self->{lock_seqid}++) .
            # Lock stateid,
            xdr_stateid4(encode=>$opts{stateid}) .
            # Offset,
            xdr_uint64(encode=>$opts{offset}) .
            # Length,
            xdr_uint64(encode=>$opts{count});

    # Create compound request
    @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
             { op => OP_PUTFH,    args => $fhandle },
             { op => OP_LOCKU,    args => $args }, );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'locku',
                                     ops => \@ops );

    return $out;
}

#---------------------------------------------------------------------

=head2 lockt

  $out = $nfs_obj->lockt(offset => $offset,
                         count => $count
                         [, owner => $owner [, ownrlen => $ownrlen ] ]
                         [, fhandle => $fhandle ]
                         [, type => $lock_type ]
                         [, cookie => $cookie ]);

Calls the LOCKT (NFSv4) procedure on the server.
This function checks whether a lock is available to the client.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of the file to be tested for lock
(default is the current filehandle).

=item offset

(Required) : The position within the file at which the lock is to begin.

=item count

(Required) : The number of bytes of data that are to be locked.

=item type

(Optional) : The type of lock to test: READ_LT (default), WRITE_LT,
READW_LT, WRITEW_LT.

=item owner

(Optional) : Owner of file locking state (default is the current process id).

=item ownrlen

(Optional) : Force the length of owner to this number.

=item cookie

(Optional) : Cookie used to match asynchronous response with a request
(default is '').
It is only supported on NFSv2 and NFSv3.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub lockt($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR,  default => $self->{curr_fh} },
            cookie  => { type => SCALAR,  default => '' },
            offset  => { type => SCALAR,    regex => qr/^\d+$/ },
            count   => { type => SCALAR,    regex => qr/^\d+$/ },
            type    => { type => SCALAR,  default => READ_LT,
                                            regex => qr/^\d+$/ },
            owner   => { type => SCALAR,  default => $$ },
            ownrlen => { type => SCALAR, optional => 1,
                                            regex => qr/^\d+$/ },
        }
    );
    my ($out, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %len = ( len => $opts{ownrlen} ) if defined($opts{ownrlen});

    $seq_arg = $self->_sequence();

    # Convert filehandle of file into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);

    # Encode the locktype,
    my $args = xdr_uint(encode=>$opts{type}) .
               # Offset,
               xdr_uint64(encode=>$opts{offset}) .
               # Length,
               xdr_uint64(encode=>$opts{count}) .
               # Lock owner: client id
               $self->{clientid4} .
               #             owner
               xdr_opaque(encode=>$opts{owner}, %len);

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                { op => OP_PUTFH,    args => $fhandle },
                { op => OP_LOCKT,    args => $args }, );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'lockt',
                                     ops => \@ops );

    # Create a copy of the lock4denied structure reference for ease of use
    if ( $out && $out->{status} == NFS4ERR_DENIED ) {
        $out->{lock_denied} = $out->{results}[1]{result};
    }

    return $out;
}

#---------------------------------------------------------------------

=head2 release_lockowner

  $out = $nfs_obj->release_lockowner([, owner => $owner
                                        [, ownrlen => $ownrlen ] ]);

Calls the RELEASE_LOCKOWNER procedure on the server.
This function is used to notify the server that the lock owner is no
longer in use by the client. This allows the server to release cached
state related to the specified lock owner.
It is only supported on NFSv4.

The following named arguments are accepted:

=over

=item Arguments

=over

=item owner

(Optional) : Owner of file locking state (default is the current process id).

=item ownrlen

(Optional) : Force the length of owner to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub release_lockowner($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            owner   => { type => SCALAR,  default => $$ },
            ownrlen => { type => SCALAR, optional => 1,
                                            regex => qr/^\d+$/ },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %len = ( len => $opts{ownrlen} ) if defined($opts{ownrlen});

    # Encode the Lock owner: client id
    my $args = $self->{clientid4} .
               # owner
               xdr_opaque(encode=>$opts{owner}, %len);

    # Create compound request
    my @ops = ( { op => OP_RELEASE_LOCKOWNER, args => $args } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'rlse lockowner',
                                     ops => \@ops );

    return $out;
}

#---------------------------------------------------------------------

=head2 delegreturn

  $out = $nfs_obj->delegreturn(stateid => $deleg_stateid
                               [, fhandle => $fhandle ]);

Calls the DELEGRETURN procedure on the server.
This function returns an open delegation represented by the
current filehandle and stateid.
It is only supported on NFSv4.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Optional) : Filehandle of opened file (default is the current filehandle).

=item stateid

(Required) : Reference to a hash containing the stateid of the opened file.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub delegreturn($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            stateid => { type => HASHREF },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Convert filehandle of file
    my $fhandle = xdr_fhandle(encode => $opts{fhandle},
                              version => $ver);

    # Encode the stateid of open delegation
    my $args = xdr_stateid4(encode => $opts{stateid});

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE,    args => $self->_sequence() },
                { op => OP_PUTFH,       args => $fhandle },
                { op => OP_DELEGRETURN, args => $args } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'delegreturn',
                                     ops => \@ops );

    # Remove open delegation from list
    $self->deleg_remove(\%opts);

    return $out;
}

#---------------------------------------------------------------------

=head2 delegpurge

  $out = $nfs_obj->delegpurge();

Calls the DELEGPURGE procedure on the server.
This function purges all of the delegations awaiting recovery for a
given client. This is useful for clients which do not commit delegation
information to stable storage to indicate that conflicting requests need
not be delayed by the server awaiting recovery of delegation information.
It is only supported on NFSv4.

=over

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub delegpurge($%) {
    my $self = shift;
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Create compound request
    my @ops = ( { op => OP_DELEGPURGE, args => $self->{clientid4} } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'delegpurge',
                                     ops => \@ops );

    return $out;
}

#---------------------------------------------------------------------

=head2 renew

  $out = $nfs_obj->renew();

Calls the RENEW procedure on the server.
This function renews all leases which are currently held
by this client at the server.
It is only supported on NFSv4.

It returns a reference to a hash containing the status and output of procedure.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub renew($;) {
    my $self = shift;
    my ($out, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    $seq_arg = $self->_sequence();

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                { op => OP_RENEW,    args => $self->{clientid4} } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'renew',
                                     ops => \@ops );

    return $out;
}

#---------------------------------------------------------------------

=head2 create

  $out = $nfs_obj->create(name => $name [, namelen => $namelen ]
                          [, dhandle => $fhandle ]
                          [, type => $type ]
                          [, attr => \%sattr ]
                          [, cr_mode => $mode ]
                          [, data => $data [, datalen => $datalen ] ]
                          [, major => $major, minor => $minor ]);

Calls the CREATE procedure on the server.
This function creates different types of file system objects.
For NFSv2, it creates regular files, special device files,
named sockets or named pipes.
For NFSv3, it only creates regular files and accepts a creation
mode flag.
For NFSv4, it creates everything but a regular file including
directories, symbolic links, special device files, named
sockets and named pipes.
To have similar behavior with mkdir and open, it sets the current
filehandle if creating a regular file on NFSv2 and v3
and if creating a directory on NFSv4.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory in which the new file
is to be created (default is the current filehandle).

=item name

(Required) : The name that is to be associated with the created file.

=item type

(Optional) : The type of file to be created: NFREG (default), NFDIR,
NFBLK, NFCHR, NFLNK, NFSOCK or NFFIFO.

=item attr

(Optional) : The initial attributes for the new file
(default is no initial attributes).

=item cr_mode

(Optional) : Create mode: UNCHECKED (default), GUARDED, EXCLUSIVE.
Used only in NFSv3.

=item data

(Optional) : The string containing the symbolic link data.
Used only in NFSv4.

=item major

(Optional) : The major device number, only for type NFBLK and NFCHR.
Used in NFSv2 and v4.

=item minor

(Optional) : The minor device number, only for type NFBLK and NFCHR.
Used in NFSv2 and v4.

=item verf

(Optional) : Create verifier. Only used if cr_mode is EXCLUSIVE.
Default is the current time.

=item namelen

(Optional) : Force the length of name to this number.

=item datalen

(Optional) : Force the length of data to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub create($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle => { type => SCALAR, default => $self->{curr_fh} },
            name    => { type => SCALAR },
            type    => { type => SCALAR,  default => NFREG,
                                            regex => qr/^\d+$/ },
            attr    => { type => HASHREF, default => {} },
            cr_mode => { type => SCALAR,  default => UNCHECKED,
                                            regex => qr/^\d+$/ },
            data    => { type => SCALAR, optional => 1 },
            major   => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
            minor   => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
            verf    => { type => SCALAR,  default => time },
            namelen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
            datalen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my ($out, $args, $spec, $attr, $fhandle);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %nlen = ( len => $opts{namelen} ) if defined($opts{namelen});
    my %dlen = ( len => $opts{datalen} ) if defined($opts{datalen});
    my $seq_arg;

    $seq_arg = $self->_sequence();

    # Convert the major and minor numbers into XDR format
    # if this is a special device file
    if ( $opts{type} == NFBLK or $opts{type} == NFCHR ) {
        # For NFSv4 use the specdata structure
        $spec = xdr_specdata(encode => { specdata1 => $opts{major},
                                         specdata2 => $opts{minor} }
                            );
    }

    # Convert filehandle of directory and the name
    # of the new file into XDR format
    my $dhandle = xdr_fhandle(encode => $opts{dhandle}, version => $ver);
    my $name    = xdr_opaque(encode => $opts{name}, %nlen);

    # Use the fattr4 structure for the initial sattr
    # instead of using the sattr structure
    $attr = xdr_fattr4(encode=>$opts{attr});

    # Encode the object type
    $args = xdr_uint(encode=>$opts{type});
    if ( $opts{type} == NFLNK ) {
        # Add the linkdata if symbolic link is requested
        $args .= xdr_opaque(encode=>$opts{data}, %dlen);
    } elsif ( $opts{type} == NFBLK or $opts{type} == NFCHR ) {
        # Otherwise, add the specdata structure
        # for block or character special devices
        $args .= $spec;
    }
    # Finally, append the name of the new object
    # and the initial attributes
    $args .= $name . $attr;
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg},
                { op => OP_PUTFH,    args => $dhandle },
                { op => OP_CREATE,   args => $args },
                { op => OP_GETFH } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'create',
                                     ops => \@ops );

    # Create a copy of the filehandle for ease of use
    $out->{fhandle} = $out->{results}[3]{result}{fhandle};

    # Set the current filehandle
    if ( $opts{type} == NFDIR or $opts{type} == NFREG ) {
        $self->current_fh_set(fhandle=>$out->{fhandle});
    }

    return $out;
}

#---------------------------------------------------------------------

=head2 rmdir

  $out = $nfs_obj->rmdir(name => $name [, namelen => $namelen ]
                         [, dhandle => $fhandle ]);

Calls the RMDIR procedure on the server.
Removes a subdirectory from a directory.
This function is not supported on NFSv4;
use remove instead.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory from which the subdirectory
is to be removed (default is the current filehandle).

=item name

(Required) : The name of the subdirectory to be removed.

=item namelen

(Optional) : Force the length of name to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub rmdir($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle => { type => SCALAR, default => $self->{curr_fh} },
            name    => { type => SCALAR },
            namelen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %len = ( len => $opts{namelen} ) if defined($opts{namelen});

    # marr TODO: confirm this
    if ( $ver == 4 || $ver == 1 ) {
        # RMDIR is not supported on NFSv4
        # use REMOVE instead
        croak "RMDIR is not supported on NFSv4";
    }

    # Convert filehandle of directory and name of
    # directory to be removed into XDR format
    my $dhandle = xdr_fhandle(encode=>$opts{dhandle}, version=>$ver);
    my $name    = xdr_opaque(encode=>$opts{name}, %len);

    # Call correct procedure on server
    my $proc = $ver == 2 ? NFSPROC_RMDIR : NFSPROC3_RMDIR;
    $out = $rpc->clnt_call(proc => $proc,
                           arg  => $dhandle . $name,
                           out  => sub {xdr_remove(version=>$ver, @_)});

    # Check for errors
    $self->error_check(name=>'RMDIR', result=>$out);

    return $out;
}

#---------------------------------------------------------------------

=head2 secinfo

  $out = $nfs_obj->secinfo(name => $name [, namelen => $namelen ]
                          [, dhandle => $dhandle ]);

Calls the SECINFO procedure on the server.
Searches a directory for a specific name and returns a list of valid
RPC authentication flavors for the specific directory filehandle,
file name pair.
It is only supported on NFSv4.

The following named arguments are accepted:

=over

=item Arguments

=over

=item dhandle

(Optional) : Filehandle of the directory to search
(default is the current filehandle).

=item name

(Required) : The name of the object to get the SECINFO.

=item namelen

(Optional) : Force the length of name to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub secinfo($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle => { type => SCALAR, default => $self->{curr_fh} },
            name    => { type => SCALAR },
            namelen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my ($out, $fhandle, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %len = ( len => $opts{namelen} ) if defined($opts{namelen});

    $seq_arg = $self->_sequence();

    # Convert filehandle of directory and
    # the name of object into XDR format
    my $dhandle = xdr_fhandle(encode=>$opts{dhandle}, version=>$ver);
    my $name    = xdr_opaque(encode=>$opts{name}, %len);

    # Create compound request
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                { op => OP_PUTFH,    args => $dhandle },
                { op => OP_SECINFO,  args => $name } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'secinfo',
                                     ops => \@ops );

    # Create a copy of the secinfo reference for ease of use
    $out->{secinfo} = $out->{results}[1]{result}{secinfo};

    return $out;
}

#---------------------------------------------------------------------
# Internal function.
# Creates and returns an Rpc object for the MOUNT protocol used
# for NFS version 2 and 3 which requires a different protocol
# for the mount and umount operations as well as the PATHCONF
# procedure on NFSv2.
#---------------------------------------------------------------------
sub mount_rpc_get($;) {
    my $self = shift;
    my %rpc_opts_h;

    # Pass local address if it is given
    $rpc_opts_h{localaddr} = $self->{localaddr} if defined($self->{localaddr});

    # Create an rpc object for mountd (100005)
    my $rpc = nfs::Rpc->clnt_create(hostname => $self->{hostname},
                                    version  => $self->{mnt_ver},
                                    proto    => $self->{mnt_proto},
                                    port     => $self->{mnt_port},
                                    xid      => $self->{rpc}{xid},
                                    retry    => $self->{retry},
                                    resvport => $self->{resvport},
                                    timeout  => $self->{timeout},
                                    program  => 100005,
                                    %rpc_opts_h);

    # Save the mount port number so next time
    # the portmapper is not called to get it
    $self->{mnt_port} = $rpc->{port} if ! $self->{mnt_port};

    return $rpc;
}

#---------------------------------------------------------------------

=head2 mount

  $out = $nfs_obj->mount(path => $path [, pathlen => $pathlen ]);

Returns the root filehandle of the path to be mounted.
In NFSv4, the operation PUTROOTFH is issue to set the current
filehandle to the root of the server's file tree, then using LOOKUP
to traverse the given path and finally GETFH to get the filehandle for
the path to be mounted.

It saves the mountpoint filehandle on object's blessed hash
under the 'mnt_fh' key, as well as setting the current filehandle.

The following named arguments are accepted:

=over

=item Arguments

=over

=item path

(Required) : The pathname to be mounted.

=item pathlen

(Optional) : Force the length of path to this number.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub mount($%) {
    $Log->exit();
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            path            => { type => SCALAR },
            id              => { type => SCALAR, optional => 1 },
            pathlen         => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
            eia_flags       => { type => SCALAR , optional => 1, default => 0 },
            maxrequestsize  => {type => SCALAR , optional => 1, default => 20000 },
            maxresponsesize => {type => SCALAR , optional => 1, default => 20000 },
            maxrequests     => {type => SCALAR , optional => 1, default => 1000 },
            reclaim_complete=> {type => SCALAR, optional => 1, default => 1}
        }
    );
    my ($rpc, $out);
    my %len = ( len => $opts{pathlen} ) if defined($opts{pathlen});
    $Log->debug(qq{path:$opts{path}\n});
    my $seq_arg;
    my @ops;

    if ( $opts{eia_flags} ) {
        $self->exchange_id(eia_flags => $opts{eia_flags});
    } else {
        $self->exchange_id();
    }

    $self->create_session(maxrequestsize => $opts{maxrequestsize},
                         maxresponsesize => $opts{maxresponsesize},
                             maxrequests => $opts{maxrequests});

    $self->reclaim_complete() if ($opts{reclaim_complete} && !$self->{reclaim_complete});
    $seq_arg = $self->_sequence();
    # Mount on NFSv4 is implemented on the NFS protocol
    # so use the current Rpc object
    $rpc = $self->{rpc};

    # Create a list of directories for the path to be mounted
    $opts{path} =~ s{/}{};
    my @paths = split /\//, $opts{path};

    # Include SEQUENCE op
    @ops = ( { op => OP_SEQUENCE, args => $seq_arg},
             { op => OP_PUTROOTFH} );

    # traverse the path using LOOKUP
    push @ops, map { { op   => OP_LOOKUP,
                       args => xdr_opaque(encode=>$_, %len) }
                   } @paths;

    # and finally get the filehandle of the last LOOKUP operation
    push @ops, { op => OP_GETFH };

    # Call the compound procedure
    $out = $self->process_operation( tag => 'mount',
                                     ops => \@ops );

    # Create a copy of the filehandle for ease of use
    $out->{fhandle} = $out->{results}[-1]{result}{fhandle};

    ++$self->{cl_verifier};

    # Set current filehandle
    $self->current_fh_set(fhandle=>$out->{fhandle});

    # Save the root filehandle on the NfsClient object
    $self->{mnt_fh} = $out->{fhandle};

    # Save mount path
    $self->{mnt_path} = $opts{path};

    $Log->exit();
    return $out;
}

#---------------------------------------------------------------------

=head2 umount

  $nfs_obj->umount(path => $path [, pathlen => $pathlen ]);

For NFSv2 and v3, removes a MOUNT entry from the server's MOUNT list.
This procedure is an advisory call to the server as notification
that the client has unmounted the file system and will no longer
be using it. It uses the MOUNTPROC_UMNT procedure of the MOUNTD
protocol.

On the other hand, for NFSv4, this function does nothing since there
is no state associated with the root filehandle on the server for
any given path.

The following named arguments are accepted:

=over

=item Arguments

=over

=item path

(Required) : The pathname to be unmounted.

=item pathlen

(Optional) : Force the length of path to this number.

=back

=item Returns

Nothing

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub umount($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            path    => { type => SCALAR },
            pathlen => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
        }
    );
    my $rpc;
    my $ver = $self->{version};
    my %len = ( len => $opts{pathlen} ) if defined($opts{pathlen} );

    # Return open delegations
    foreach my $deleg ( @{$self->{open_delegs}} ) {
        $self->delegreturn(fhandle => $deleg->{fhandle},
                           stateid => $deleg->{stateid});
    }

    # No umount on NFSv4 so just return
    return;
}

#---------------------------------------------------------------------

=head2 mount_dump

  $out = $nfs_obj->mount_dump();

Calls the MOUNTPROC_DUMP procedure of the MOUNTD protocol.
Returns a list of clients and the exported file systems they
have mounted. Since the number of clients and mounted
file systems can be very long, clients should use this
call via a TCP connection.

=over

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub mount_dump($;) {
    my $self = shift;
    my $rpc = $self->mount_rpc_get();

    # Call correct procedure on server
    my $ret = $rpc->clnt_call(proc => MOUNTPROC_DUMP, out  => \&xdr_dump);
    $self->{rpc}{xid} = $rpc->{xid};

    return $ret;
}

#---------------------------------------------------------------------

=head2 mount_export

  $out = $nfs_obj->mount_export();

Calls the MOUNTPROC_EXPORT procedure of the MOUNTD protocol.
Returns a list of exported file systems and their export information.
Since the export list can be large, a TCP connection should be used
to retrieve this data.

=over

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub mount_export($;) {
    my $self = shift;
    my $rpc = $self->mount_rpc_get();

    # Call correct procedure on server
    my $ret = $rpc->clnt_call(proc => MOUNTPROC_EXPORT, out  => \&xdr_export);
    $self->{rpc}{xid} = $rpc->{xid};

    return $ret;
}

#---------------------------------------------------------------------

=head2 current_fh_set

  $nfs_obj->current_fh_set(fhandle => $fhandle);

Sets the current filehandle of the object.
All functions which require a filehandle uses this current
filehandle as the default.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fhandle

(Required) : Filehandle to set.

=back

=item Returns

The filehandle before being set.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub current_fh_set($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR },
        }
    );

    # Save the current filehandle on prev_fh and set
    # the current filehandle to the one provided
    $self->{prev_fh} = $self->{curr_fh};
    $self->{curr_fh} = $opts{fhandle};

    return $self->{prev_fh};
}

#---------------------------------------------------------------------
# Internal function.
# This function checks for errors on the results hash.
# If the status is not NFS_OK, the function croaks with
# the corresponding error.
# The error message has the following format:
#   'name' NFS?ERR_error_name: error_message
# The 'name' argument is just a string to identify which function
# has failed, usually is set to the procedure name,
# e.g., LOOKUP, CREATE, etc.
#---------------------------------------------------------------------
sub error_check($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            name   => { type => SCALAR },
            result => { type => HASHREF|UNDEF },
            type   => { type => SCALAR, default => 'nfs' },
        }
    );
    # Clear previous error result
    $self->{result} = undef;

    my ($package, $file, $line, $subr) = caller(0);
    my $callermsg = " ($package $subr called at $file line $line) ";

    # Check if result is defined
    croak "No results received from server: $callermsg" if !defined $opts{result};

    if ( $opts{result}{status} ) {
        # Procedure returned an error therefore save the
        # result on the object so it can be retrieved by
        # the calling process if needed
        $self->{result} = $opts{result};

        if ( $opts{type} eq 'nfs' ) {
            # Croak with the proper NFS error message
            croak $opts{name} . " " .
                  nfs_error( error=>$opts{result}{status},
                             version=>$self->{version} ) .
                  ": $callermsg";
        }
    }

    return;
}

#---------------------------------------------------------------------
# Internal function
# Remove the given delegation from the list of open delegations.
#---------------------------------------------------------------------
sub deleg_remove($$;) {
    my $self  = shift;
    my $deleg = shift;
    my $deleg_exists = 0;

    my $index = 0;
    # Search the specified delegation in list
    foreach my $open_deleg ( @{$self->{open_delegs}} ) {
        if ( $deleg->{fhandle} eq $open_deleg->{fhandle} ) {
            # Remove delegation from list
            splice @{$self->{open_delegs}}, $index, 1;
            $deleg_exists = 1;
            return;
        }
        ++$index;
    }
    $Log->warn("Attempted removal of non-existent delegation for " .
               "delegate " . Dumper($deleg));
}

#---------------------------------------------------------------------

=head2 compound

  $out = $nfs_obj->compound(ops => \@op_list
                            [, tag => $tag ]
                            [, num => $num ]
                            [, size => $size ]);

Calls a compound request on NFS version 4. The compound request is made
of all operations listed on the ops argument. Each operation is a hash
having the following:

  {
      op   => Operation number,
      args => XDR encoded arguments,
  }

The following named arguments are accepted:

=over

=item Arguments

=over

=item ops

(Required) : Reference to an array of operations to be included in the
compound request.

=item tag

(Optional) : Tag for the compound request. Default is 'compound'.

=item num

(Optional) : Use this as the number of operations to encode instead
of the actual number of operations encoded.

=item size

(Optional) : Truncate the arguments to the compound procedure to
this size.

=back

=item Returns

Reference to a hash containing the status and output of procedure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub compound($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            ops  => { type => ARRAYREF },
            tag  => { type => SCALAR,  default => 'perl synthetic client compound' },
            num  => { type => SCALAR, optional => 1 },
            size => { type => SCALAR, optional => 1 },
            excl_seq => { type => SCALAR,  optional => 1, default => '0' }
        }
    );
    my ($out, $arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my %args = (num => $opts{num}) if defined $opts{num};

    if (not $opts{excl_seq}){
        $self->{sequenceid4}++;
        my $xdr_sequence = xdr_opaque (encode => $self->{sessionid4}, fixed => 1, maxlen => 16).
                           xdr_uint(encode => $self->{sequenceid4}).xdr_uint(encode=>1).xdr_uint(encode=>8).xdr_uint(encode=>1);
        my $ops_ref = $opts{ops};
        $opts{ops} = [ {op => OP_SEQUENCE, args => $xdr_sequence}, @$ops_ref ];
    }


    # Encode compound arguments
    $args{minor} = $self->{minor_version};
    $arg = xdr_compound(tag => $opts{tag}, ops => $opts{ops}, %args);
    substr($arg, $opts{size}) = '' if defined $opts{size};

    # marr TODO: use process_operation inside this primitive compound?
    # Call the compound procedure
    $out = $rpc->clnt_call(proc => 1, arg => $arg, out => \&xdr_compound_res);

    # Check for errors
    $self->error_check(name=>'COMPOUND', result=>$out);

    return $out;
}

#---------------------------------------------------------------------

=head2 rpc

  $rpc_obj = $nfs_obj->rpc();

Returns the RPC object of the connection.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub rpc { return shift->{rpc} }

#---------------------------------------------------------------------

=head2 mnt_fh

  $mnt_fh = $nfs_obj->mnt_fh();

Returns the filehandle of the root of the mount point.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub mnt_fh { return shift->{mnt_fh} }

#---------------------------------------------------------------------

=head2 curr_fh

  $curr_fh = $nfs_obj->curr_fh();

Returns the current filehandle of the object. Most methods which accept
a filehandle have as a default this filehandle. It is set internally
with operations like LOOKUP, CREATE, OPEN, etc.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub curr_fh { return shift->{curr_fh} }

#---------------------------------------------------------------------

=head2 mnt_path

  $mnt_path = $nfs_obj->mnt_path();

Returns the pathname of the mount.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub mnt_path { return shift->{mnt_path} }

#---------------------------------------------------------------------

=head2 result

  $result = $nfs_obj->result();

Returns the last failed result. All methods croak when status is not
NFS?_OK and the results are saved so they can be retrieved using this
method.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub result { return shift->{result} }

#---------------------------------------------------------------------

=head2 clientid

  $clientid = $nfs_obj->clientid();

Returns the NFSv4 client id.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub clientid { return shift->{clientid4} }

#---------------------------------------------------------------------

=head2 seqid

  $seqid = $nfs_obj->seqid([ $seqid_new ]);

Gets or sets the current open sequence id for the NFS object.
Only useful for NFS version 4.

=over

=item Arguments

=over

=item <number>

(Optional) : Set the current open sequence id to this.

=back

=item Returns

The open sequence id value before being set.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub seqid($$;) {
    my ($self, $new_value) = @_;
    my $old = $self->{seqid4};

    if ( defined($new_value) ) {
        $self->{seqid4} = $new_value;
    }

    return $old;
}

#---------------------------------------------------------------------

=head2 lock_seqid

  $lock_seqid = $nfs_obj->lock_seqid([ $lock_seqid_new ]);

Gets or sets the current lock sequence id for the NFS object.
Only useful for NFS version 4.

=over

=item Arguments

=over

=item <number>

(Optional) : Set the current lock sequence id to this.

=back

=item Returns

The lock sequence id value before being set.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub lock_seqid($$;) {
    my ($self, $new_value) = @_;
    my $old = $self->{lock_seqid};

    if ( defined($new_value) ) {
        $self->{lock_seqid} = $new_value;
    }

    return $old;
}

#---------------------------------------------------------------------
# Do we expect a callback to come after we get a DELAY error for an OP ?
=head2 expect_callback_on_delay

    $flag = $nfs_obj->expect_callback_on_delay([ $flag ]);

Gets or sets the flag "expect_callback_on_delay" based on the argument.
Only useful for NFS version 4 and above.

=over

=item Arguments

=over

=item <flag>

(Optional) :  Set the current value to this number. If not provided, simply returns the current value of the flag in the nfs client object.

=back

=item Returns

The current flag value before being set is returned.

=back

=for html <hr />

=cut
sub expect_callback_on_delay($;$) {
    my ($self, $new_value) = @_;
    my $old = $self->{expect_callback_on_delay};

    if ( defined($new_value) ) {
        $self->{expect_callback_on_delay} = $new_value;
    }

    return $old;
}

#---------------------------------------------------------------------
# Destructor
# It unregisters the callback program for NFSv4
#---------------------------------------------------------------------
sub DESTROY {
    my $self = shift;

    if (($self->{version} == 4 || $self->{version} == 1)
                            && defined($self->{callback}) && $self->{minor_version} != 1) {
        eval { nfs::Rpc::pmap_unset(program => $self->{callback}{program},
                                    version => $self->{callback}{version},
                                    proto   => $self->{callback}{proto},
                                    port    => $self->{callback}{port}) };
    };
}

sub exchange_id($%) {
        $Log->enter();
        my $self = shift;
		$Log->enter();

        my (%opts) = validate (
        @_,
        {
            eia_flags => {type => SCALAR , default => EXCHGID4_FLAG_USE_PNFS_MDS},
            eia_state_protect => { type => SCALAR,   default => SP4_NONE },
            spo_must_enforce => { type => ARRAYREF, optional => 1  },
            spo_must_allow   =>  { type => ARRAYREF, optional => 1 },
	    id       => { type => SCALAR,  default => $self->{cl_id},
                                             regex => qr/^\d+$/ },
            verifier => { type => SCALAR,  default => $self->{cl_verifier},
                                             regex => qr/^\d+$/ },
            co_verifier => { type => SCALAR, default => $self->{co_verifier},
                                               regex => qr/^\d+$/ },
            idlen    => { type => SCALAR, optional => 1, regex => qr/^\d+$/ },
            impl_id  => { type => HASHREF,  optional => 1},
            nii_domain => { type => SCALAR, optional => 1,default => 'perl-synthetic-client' },
            nii_name => { type => SCALAR, optional =>1,default => '4.1-synthetic-client' },
            seconds => { type => SCALAR, optional =>1,default => 0 },
            nseconds => { type => SCALAR,optional =>1,default => 0 },
        }
    );

    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my $out;
    my $args;
    my $eia_state_protect;
    my %len = ( len => $opts{idlen} ) if defined($opts{idlen});

    # Convert client verifier into a string
    # and create the id string
    my $verifier = xdr_uint(encode=>$opts{co_verifier});
    # The id string is a constant and unique value for the client
    # it is created by concatenating both the ip addresses of
    # the client and server
    my $id = $rpc->{sock}->peeraddr() . $rpc->{sock}->sockaddr .
             xdr_uint(encode=>$self->{co_ownerid});

    # Create the client owner in the same way as SETCLIENTID
    my $eia_clientowner = xdr_opaque(encode=>$verifier, fixed=>1, maxlen=>8) .
               xdr_opaque(encode=>$id) ;

    if($opts{eia_state_protect} ==  SP4_MACH_CRED)
    {
        my $enforce = bitmap4_convert(attrlist=>$opts{spo_must_enforce});
        my $allow = bitmap4_convert(attrlist=>$opts{spo_must_allow});
        $eia_state_protect = xdr_uint (encode => $opts{eia_state_protect}) .
						 xdr_bitmap4 (encode => $enforce) .
						 xdr_bitmap4 (encode => $allow);
    }
    else
    {
	$eia_state_protect = xdr_uint (encode => $opts{eia_state_protect});
    }

    if( $opts{nii_domain} ||  $opts{nii_name} || $opts{seconds} ||  $opts{nseconds} )  {
        $opts{impl_id} = 1;
        my $nii_domain = xdr_opaque(encode=>$opts{nii_domain}) ;
        my $nii_name = xdr_opaque(encode=>$opts{nii_name}) ;
        my $nii_date = xdr_uint64(encode=>$opts{seconds}) . xdr_uint(encode=>$opts{nseconds});
        my $eia_client_impl_id =  xdr_uint(encode=>$opts{impl_id}) . $nii_domain . $nii_name . $nii_date ;

        $args =  $eia_clientowner ;
        $args .= xdr_uint (encode => $opts{eia_flags}) ;
        $args .= $eia_state_protect ;
        $args .= $eia_client_impl_id ;
    } else {
        $opts{impl_id} = 0 ;
        $args =  $eia_clientowner ;
        $args .= xdr_uint (encode => $opts{eia_flags}) ;
        $args .= $eia_state_protect ;
        $args .= xdr_uint (encode => $opts{impl_id});
      }
    
    my @ops = ( { op => OP_EXCHANGE_ID, args => $args } );

    $out = $self->process_operation( tag => 'exchange_id',
                                     ops => \@ops );

    $self->{clientid4} = $out->{results}->[0]->{result}->{clientid};
    $self->{seqid4} = $out->{results}->[0]->{result}->{seq_id};
    $out->{eir_flag} = $out->{results}->[0]->{result}->{eir_flag};
    $out->{state_protect} = $out->{results}->[0]->{result}->{state_pro};

    if($out->{state_protect} == SP4_MACH_CRED){

      $out->{spo_must_enforce} = $out->{results}->[0]->{result}->{spo_must_enforce};
      $out->{spo_must_allow}  = $out->{results}->[0]->{result}->{spo_must_allow};
    }
    $Log->exit();
    return $out;

}

sub process_operation {
   $Log->enter();
   my $self = shift;
   my (%args) = validate (
        @_,
        {
            tag   => { type => SCALAR,},
            ops   => { type => ARRAYREF,},
        }
    );

    $self->{pending_operation}->{tag} = $args{tag};
    $self->{pending_operation}->{ops} = $args{ops};

    my $xdr_options = $self->{xdr_options};
    my $rpc = $self->{rpc};
    my $p_op = $self->{pending_operation};
    $p_op->{error_retry} = 0;
    my $out = undef;

    my ($package, $file, $line, $subr) = caller(0);
    $Log->trace("Processing Operation: " . Dumper($p_op));
    my $callermsg = " ($package $subr called at $file line $line) ";
	# Clear previous error result
    $self->{result} = undef;

    {

       $out = $rpc->clnt_call(proc => $xdr_options->{proc},
			      auth_type => $rpc->{auth_type},
                              arg  => xdr_compound(tag   => $p_op->{tag},
                                                   minor => $self->{minor_version},
                                                   ops   => $p_op->{ops}),
                              out  => $xdr_options->{out});

        # If out is not defined, then we got no response
        if (not defined($out)) {
            croak "No results received from server: $p_op->{tag}" if !defined $out;

        # We should retry DELAY errors and other errors on the retry
        # error list
        } elsif (($out->{status}) and (exists $retry_errors{$out->{status}})) {
            $p_op->{error_retry}++;
            $Log->debug("retry $p_op->{error_retry} due to response status $out->{status}");
            my $quotient = $p_op->{error_retry} / 5;

            # Log a warning after every 5 retries
            if (int($quotient) == $quotient) {
                $Log->warn("Retried 5 times due to transient error: $out->{status}");
            }

            # sleep longer for NFS4ERR_GRACE
            my $grace_sleep = 30;
            my $delay_sleep = $self->{nfs_delay};
            my $sleep_time = ($out->{status} == NFS4ERR_GRACE) ? $grace_sleep : $delay_sleep;

            last if ($p_op->{error_retry} > $self->{nfs_retry});

            # If we are going to retry, we need to change some sequence numbers...
            # Change the sequence id
            $Log->debug("retry self->pendingoperation: ". Dumper($self->{pending_operation}));
            if ($p_op->{ops}->[0]->{op} == OP_SEQUENCE) {
                # If the SEQUENCE op is successful, we must increment the sequence number.
                # otherwise, don't.
                if ($out->{results}->[0]->{result}->{status} == 0) {
                    $p_op->{ops}->[0]->{args} = $self->_sequence();
                }
            }

            # process callback if the last reply was OPEN, and it comes with delay.
            if ($out->{results}->[-1]->{op} == OP_OPEN) {
                if ($out->{results}->[-1]->{result}->{status} == NFS4ERR_DELAY) {
                    if ($self->{expect_callback_on_delay}) {
                        $self->callback_process();
                    }
                }
            }
            sleep($sleep_time);
            redo;
        }

    }

   # If after so many attempts, we still have error, croak.
   if ( $out->{status} ) {
		$Log->debug("status error: " . Dumper($out));
        # save the result on the object so it can be retrieved by
        # the calling process if needed
        $self->{result} = $out;

        # Croak with the proper NFS error message
        croak uc($args{tag}) . " " .
            nfs_error(error   => $out->{status},
                      version => $self->{version}) .
                      ": $callermsg";
    }

   $p_op->{out} = $out;
   $self->{completed_operation} = $p_op;
   $self->{pending_operation}->{arg} = "";
   $self->{pending_operation}->{ops} = "";

   return $out;
}


sub destroy_clientid($%)
{
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            clientid4   => { type => SCALAR, default => $self->{clientid4} },
        }
    );

    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my $out;

    my $args = xdr_opaque (encode => $opts{clientid4}, fixed => 1, maxlen => 8);
    #my $args = $opts{clientid4};

    my @ops = ({ op => OP_DESTROY_CLIENTID, args => $args} );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'destroy_clientid',
                                     ops => \@ops );

    return $out;
} ## end sub destroy_clientid

sub create_session($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            clientid            =>  {type => SCALAR, optional => 1},
            seq_id              =>  {type => SCALAR, optional => 1},
            csa_flags           =>  {type => SCALAR, default => 2},
            headerpadsize       =>  {type => SCALAR, default => 0},
            maxrequestsize      =>  {type => SCALAR, default => 1000},
            maxresponsesize     =>  {type => SCALAR, default => 1000},
            maxresponsesize_c   =>  {type => SCALAR, default => 1000},
            maxoperations       =>  {type => SCALAR, default => 1000},
            maxrequests         =>  {type => SCALAR, default => 1000},
            rdma_ird            =>  {type => SCALAR, default => 0},
            headerpadsize_b     =>  {type => SCALAR, default => 0},
            maxrequestsize_b    =>  {type => SCALAR, default => 1000},
            maxresponsesize_b   =>  {type => SCALAR, default => 1000},
            maxresponsesize_c_b =>  {type => SCALAR, default => 1000},
            maxoperations_b     =>  {type => SCALAR, default => 1000},
            maxrequests_b       =>  {type => SCALAR, default => 100},
            rdma_ird_b          =>  {type => SCALAR, default => 0},
        }
    );

    my $rpc       = $self->{rpc};
    my $ver       = $self->{version};
    my $clientid  = defined ($opts{clientid}) ? $opts{clientid} : $self->{clientid4};
    my $seqid     = defined ($opts{seq_id}) ? $opts{seq_id} : $self->{seqid4};
    my $out;

    my $program = 10003;

    if ( defined($self->{callback}) ) {
            # Change the default values with
            # the real callback information
            $program =  $self->{callback}->{program};
    }

    $Log->enter('Gonna send program:' . $program);
    # Channel attrs
    my $ca_headerpadsize           = $opts{headerpadsize};
    my $ca_maxrequestsize          = $opts{maxrequestsize};
    my $ca_maxresponsesize         = $opts{maxresponsesize};
    my $ca_maxresponsesize_cached  = $opts{maxresponsesize_c};
    my $ca_maxoperations           = $opts{maxoperations};
    my $ca_maxrequests             = $opts{maxrequests};
    my $ca_rdma_ird                = $opts{rdma_ird};

    my $ca_headerpadsize_b           = $opts{headerpadsize_b};
    my $ca_maxrequestsize_b          = $opts{maxrequestsize_b};
    my $ca_maxresponsesize_b         = $opts{maxresponsesize_b};
    my $ca_maxresponsesize_cached_b  = $opts{maxresponsesize_c_b};
    my $ca_maxoperations_b           = $opts{maxoperations_b};
    my $ca_maxrequests_b             = $opts{maxrequests_b};
    my $ca_rdma_ird_b                = $opts{rdma_ird_b};
    my $csa_flags                    = $opts{csa_flags};


    $Log->debug("Value1: $ca_rdma_ird Value2: $ca_rdma_ird_b");
    my $channel_attrs = xdr_uint(encode => $ca_headerpadsize) .
                        xdr_uint(encode => $ca_maxrequestsize) .
                        xdr_uint(encode => $ca_maxresponsesize) .
                        xdr_uint(encode => $ca_maxresponsesize_cached) .
                        xdr_uint(encode => $ca_maxoperations) .
                        xdr_uint(encode => $ca_maxrequests) .
                        xdr_uint(encode => $ca_rdma_ird) ;

    my $backchannel_attrs = xdr_uint(encode => $ca_headerpadsize_b) .
                            xdr_uint(encode => $ca_maxrequestsize_b) .
                            xdr_uint(encode => $ca_maxresponsesize_b) .
                            xdr_uint(encode => $ca_maxresponsesize_cached_b) .
                            xdr_uint(encode => $ca_maxoperations_b) .
                            xdr_uint(encode => $ca_maxrequests_b) .
                            xdr_uint(encode => $ca_rdma_ird_b) ;

    my $args =  xdr_opaque(encode => $clientid, fixed => 1, maxlen => 8) .
                xdr_uint(encode => $seqid) .
                xdr_uint(encode => $csa_flags) .
                $channel_attrs .
                $backchannel_attrs .
                xdr_uint(encode => $program) .
                xdr_uint(encode => 1) .
                xdr_uint(encode => 0) ;

    my @ops = ( { op => OP_CREATE_SESSION, args => $args } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'create_session',
                                     ops => \@ops );
    #$self->callback_process();

    ## Get the session id and assign that to the rpc

    $self->{csr_flag}     = $out->{results}->[0]->{result}->{csr_flag};
    $self->{sessionid4}   = $out->{results}->[0]->{result}->{sessionid};
    #$self->{sequenceid4}      = $out->{results}->[0]->{result}->{seq_id} - 1;
    $self->{sequenceid4}      = 0;
    $self->{fwd_channel}->{ca_headerpadsize} = $out->{results}->[0]->{result}->{headerpadsize};
    $self->{fwd_channel}->{ca_maxrequestsize} = $out->{results}->[0]->{result}->{maxrequestsize};
    $self->{fwd_channel}->{ca_maxresponsesize} = $out->{results}->[0]->{result}->{maxresponsesize};
    $self->{fwd_channel}->{ca_maxresponsesize_cached} = $out->{results}->[0]->{result}->{maxresponsesize_c};
    $self->{fwd_channel}->{ca_maxoperations} = $out->{results}->[0]->{result}->{maxoperations};
    $self->{fwd_channel}->{ca_maxrequests} = $out->{results}->[0]->{result}->{maxrequests};
    $self->{fwd_channel}->{ca_rdma_ird} = $out->{results}->[0]->{result}->{rdma_ird};

    $self->{bck_channel}->{ca_headerpadsize} = $out->{results}->[0]->{result}->{headerpadsize_b};
    $self->{bck_channel}->{ca_maxrequestsize} = $out->{results}->[0]->{result}->{maxrequestsize_b};
    $self->{bck_channel}->{ca_maxresponsesize} = $out->{results}->[0]->{result}->{maxresponsesize_b};
    $self->{bck_channel}->{ca_maxresponsesize_cached} = $out->{results}->[0]->{result}->{maxresponsesize_c_b};
    $self->{bck_channel}->{ca_maxoperations} = $out->{results}->[0]->{result}->{maxoperations_b};
    $self->{bck_channel}->{ca_maxrequests} = $out->{results}->[0]->{result}->{maxrequests_b};
    $self->{bck_channel}->{ca_rdma_ird} = $out->{results}->[0]->{result}->{rdma_ird_b};

    return $out;
} ## end sub create_session

sub destroy_session($%)
{
    my $self = shift;
    $Log->enter();
    my (%opts) = validate (
        @_,
        {
            sessionid4   => { type => SCALAR, default => $self->{sessionid4} },
        }
    );

    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my $args = xdr_opaque (encode => $opts{sessionid4}, fixed => 1, maxlen => 16);

    my @ops = ( { op => OP_DESTROY_SESSION, args => $args} );

    # Call the compound procedure
    my $out = $self->process_operation( tag => 'destory_session',
                                        ops => \@ops );

    $Log->exit();
    return $out;
} # end sub destroy_session


sub getdevicelist($%)
{
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle    => { type => SCALAR, default => $self->{curr_fh} },
            layouttype => { type => SCALAR, default => 1,
                                            regex => qr/^\d+$/ },
            count      => { type => SCALAR, default => 10,
                                            regex => qr/^\d+$/ },
            cookie     => { type => SCALAR, default => 0,
                                            regex => qr/^\d+$/ },
            verif      => { type => SCALAR, default => 0,
                                            regex => qr/^\d+$/ },
        }
    );

    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my $fh  = xdr_fhandle (encode => $opts{fhandle}, version => 4);

    my $seq_arg = $self->_sequence();
    my $args =  xdr_uint (encode => $opts{layouttype}) .
                xdr_uint (encode => $opts{count}) .
                xdr_uint64 (encode => $opts{cookie}) .
                xdr_opaque (encode => $opts{verif}, fixed => 1, maxlen => 8);

    my @ops = ( { op => OP_SEQUENCE,      args => $seq_arg},
                { op => OP_PUTFH,         args => $fh},
                { op => OP_GETDEVICELIST, args => $args},
                { op => OP_GETFH } );

    # Call the compound procedure
    my $out = $self->process_operation( tag => 'getdevicelist',
                                     ops => \@ops );

    $self->{deviceid} = $out->{results}->[2]->{result}->{deviceid};

    return $out;
} ## end sub getdevicelist



sub getdeviceinfo($%)
{
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            deviceid => { type => SCALAR, default => $self->{deviceid} },
            layouttype => { type => SCALAR, default => 1 },
            count => { type => SCALAR, default => 1649000 },
            arr   => { type => ARRAYREF, default => [100, 100, 100] },
        }
    );
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my @arr = @{$opts{arr}};

    my $seq_arg = $self->_sequence();
    my $args =  xdr_opaque (encode => $opts{deviceid}, fixed => 1, maxlen => 16) .
                xdr_uint (encode => $opts{layouttype}) .
                xdr_uint (encode => $opts{count}) .
                xdr_bitmap4 (encode => $opts{arr}) ;

    my @ops = ( { op => OP_SEQUENCE,      args => $seq_arg },
                { op => OP_GETDEVICEINFO, args => $args } );

    # Call the compound procedure
    my $out = $self->process_operation( tag => 'getdeviceinfo',
                                     ops => \@ops );

    #Dereference for ease of use
    #$out->{ip_addr} = $out->{results}->[-1]->{result}->{dev_addr}->{da_addr_body};
    $out->{ip_addr} = $out->{results}->[-1]->{result}->{lif_ip};
    $out->{layout_dev} = $out->{results}->[-1]->{result}->{dev_layout};
    $out->{dev_index} = $out->{results}->[-1]->{result}->{dev_index};

    return $out;
} ## end sub getdeviceinfo


sub layoutget($%)
{
        my $self = shift;
        my (%opts) = validate (
        @_,
        {
            fhandle      => { type => SCALAR, default => $self->{curr_fh} },
            stateid      => { type => HASHREF },
            layout_avail => { type => SCALAR, default => 0 },
            layouttype   => { type => SCALAR, default => 0x1 },
            layoutiomode => { type => SCALAR, default => 2 },
            offset       => { type => SCALAR, default => 1000 },
            length       => { type => SCALAR, default => 10000 },
            min_length   => { type => SCALAR, default => 10000 },	
            max_count    => { type => SCALAR, default => 999 },
        }
    );

    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    my $fh = xdr_fhandle (encode => $opts{fhandle});

    my $seq_arg = $self->_sequence();
    my $args = xdr_uint (encode => $opts{layout_avail}) .
               xdr_uint (encode => $opts{layouttype}) .
               xdr_uint (encode => $opts{layoutiomode}) .
               xdr_uint (encode => $opts{layoutiomode}) .
               xdr_uint (encode => $opts{offset}) .
               xdr_uint64 (encode => $opts{length}) .
               xdr_uint64 (encode => $opts{min_length}) .
               xdr_stateid4 (encode => $opts{stateid}) .
               xdr_uint (encode => $opts{max_count}) ;


    my @ops = ( { op => OP_SEQUENCE,  args => $seq_arg},
                { op => OP_PUTFH,     args => $fh},
                { op => OP_LAYOUTGET, args => $args},
                { op => OP_GETFH } );

    # Call the compound procedure
    my $out = $self->process_operation( tag => 'layoutget',
                                     ops => \@ops );

     $out->{layout_stateid} = $out->{results}->[2]->{result}->{stateid};
     my $layout_info = $out->{results}->[2]->{result}->{layout_info};
     $out->{layout_fhandle} = $layout_info->{fh_layout};
     $out->{stripe_size} = (($layout_info->{nfsl_util}))&0xFFFFFFC0;
     $self->{stripe_size} = $out->{stripe_size};
     $out->{deviceid_l} = $layout_info->{deviceid_l};
     $out->{stripe_to_use} = $layout_info->{stripe_to_use};

    return $out;
} ## end sub layoutget


sub layoutreturn($%)
{
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle      => { type => SCALAR, default => $self->{curr_fh} },
            stateid      => { type => HASHREF, optional => 1 },
            reclaim      => { type => SCALAR, default => 0 },
            layouttype   => { type => SCALAR, default => 1 },
            layoutiomode => { type => SCALAR, default => 2 },
            offset       => { type => SCALAR, default => 0 },
            length       => { type => SCALAR, default => 0 },
            layoutreturn => { type => SCALAR, default => 3 },
            body         => { type => SCALAR, default => 0 },
        }
    );

    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my $fh = xdr_fhandle (encode => $opts{fhandle});

    my $seq_arg = $self->_sequence();
    my $args = xdr_uint (encode => $opts{reclaim}) .
               xdr_uint (encode => $opts{layouttype}) .
               xdr_uint (encode => $opts{layoutiomode}) ;

    if ($opts{layoutreturn} == 1 )
    {
        $args .=  xdr_uint(encode => $opts{layoutreturn}) .
                  xdr_uint64 (encode => $opts{offset}) .
                  xdr_uint64 (encode => $opts{length}) .
                  xdr_stateid4 (encode => $opts{stateid}) .
                  xdr_uint(encode => 0) .
                  xdr_opaque (encode => $opts{body}) ;
    }
    else {
	$args .=  xdr_uint(encode => $opts{layoutreturn});
    }

    my @ops = ( { op => OP_SEQUENCE,     args => $seq_arg},
                { op => OP_PUTFH,        args => $fh},
                { op => OP_LAYOUTRETURN, args => $args},
                { op => OP_GETFH } );

    # Call the compound procedure
    my $out = $self->process_operation( tag => 'layoutreturn',
                                     ops => \@ops );

    return $out;
} ## end sub layoutreturn


sub bind_conn_to_session($%)
{
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            sessionid          => { type => HASHREF, default => $self->{sessionid4}},
            channel_dir_client => { type => SCALAR, default => 3 },
            rdma_mode          => { type => SCALAR, default => 0 },
        }
    );

    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    my $args = xdr_opaque(encode => $opts{sessionid}, fixed => 1, maxlen => 16) .
               xdr_uint (encode => $opts{channel_dir_client}) .
               xdr_uint (encode => $opts{rdma_mode});
    my @ops = ({ op => OP_BIND_CONN_TO_SESSION, args => $args });

    # Call the compound procedure
    my $out = $self->process_operation( tag => 'bind_conn_to_session',
                                     ops => \@ops );

    return $out;
} ## end bind_conn_to_session


sub backchannel_ctl($%)
{
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            sessionid4   => { type => HASHREF, default => $self->{sessionid4}},
            stamp        => { type => SCALAR, default => 100000 },
            machine_name => { type => SCALAR,  default =>  $self->{localname} },
            uid          => { type => SCALAR, default => 0 },
            gid          => { type => SCALAR, default => 0 },
            gids         => { type => SCALAR, default => 0 },
        }
    );

    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    my $seq_arg = $self->_sequence();
    my $program =  $self->{callback}{program};

    my $arg = xdr_uint(encode => $program) .
              xdr_uint (encode => $opts{stamp}) .
	      xdr_opaque (encode => $opts{machine_name}) .
              xdr_uint (encode => $opts{uid}) .
              xdr_uint (encode => $opts{gid}) .
              xdr_uint (encode => $opts{gids}) ;

    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg},
                { op => OP_BACKCHANNEL_CTL, args => $arg } );

    # Call the compound procedure
    my $out = $self->process_operation( tag => 'backchannel_ctl',
                                     ops => \@ops );

    return $out;
} ## end sub backchannel_ctl



sub layoutcommit($%)
{
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle     => { type => SCALAR, default => $self->{curr_fh} },
            sessionid4  => { type => HASHREF, default => $self->{sessionid4}},
            offset      => { type => SCALAR, default => 1000 },
            length      => { type => SCALAR, default => 1000 },
            reclaim     => { type => SCALAR, default => 0 },
            stateid     => { type => HASHREF },
            newoffset   => { type => SCALAR, default => 0 },
            newtime     => { type => SCALAR, default => 0 },
            layouttype  => { type => SCALAR, default => 1 },
            lou_body    => { type => SCALAR, default => 0 },
        }
    );

    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    my $seq_arg = $self->_sequence();
    my $fh  = xdr_fhandle (encode => $opts{fhandle}, version => 4);

    my $args =  xdr_uint64 (encode => $opts{offset}) .
                xdr_uint64 (encode => $opts{length}) .
                xdr_uint (encode => $opts{reclaim}) .
                xdr_stateid4 (encode => $opts{stateid}) .
                xdr_uint (encode => $opts{newoffset}) .
                xdr_uint (encode => $opts{newtime}) .
                xdr_uint (encode => $opts{layouttype}) .
                xdr_uint (encode => 0) ;

    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg},
                { op => OP_PUTFH, args => $fh},
                { op => OP_LAYOUTCOMMIT, args => $args },
                { op => OP_GETFH } );

    # Call the compound procedure
    my $out = $self->process_operation( tag => 'layoutcommit',
                                     ops => \@ops );

    return $out;
} ## end sub layoutcommit


sub reclaim_complete {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            rc_one_fs => { type => SCALAR, default => 0 },
            fhandle   => { type => SCALAR, default => $self->{curr_fh} },
        },
    );

    my $rpc  = $self->rpc();
    my $out;
    my $vers = $self->{version};
    my $fhandle;
    my @ops;

    if ($opts{rc_one_fs}) {
        $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$vers);
    }
    my $reclaim_args = xdr_uint(encode=>$opts{rc_one_fs});
    # Create compound request:
    #   RECLAIM_COMPLETE to indicate that all reclaim is done.

    if($opts{rc_one_fs}) {
        @ops = (
            {op => OP_SEQUENCE,         args => $self->_sequence()},
            {op => OP_PUTFH,            args => $fhandle},
            {op => OP_RECLAIM_COMPLETE, args => $reclaim_args},
        );
    }
    else {
        @ops = (
            {op => OP_SEQUENCE,         args => $self->_sequence()},
            {op => OP_RECLAIM_COMPLETE, args => $reclaim_args},
       );

    }

    # Call the compound procedure
    $out = $self->process_operation( tag => 'reclaim complete',
                                     ops => \@ops );

    # Set a flag for future reference.
    if ($opts{rc_one_fs} == 0) {
        $self->{reclaim_complete} = 1;
    }

} ## end sub reclaim_complete


sub _sequence($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            sessionid4      => { type => HASHREF, default => $self->{sessionid4}},
            sequenceid4     => { type => SCALAR, default => $self->{sequenceid4}},
            slotid4         => { type => SCALAR, default => 1 },
            highest_slotid4 => { type => SCALAR, default => 1000 },
            cache_this      => { type => SCALAR, default => 0 },
        }
    );

    my $seq_arg = xdr_opaque(encode => $opts{sessionid4}, fixed => 1, maxlen => 16) .
                  xdr_uint (encode => ++$opts{sequenceid4}) .
                  xdr_uint (encode => $opts{slotid4}) .
                  xdr_uint (encode => $opts{highest_slotid4}) .
                  xdr_uint (encode => $opts{cache_this});

    $self->{sequenceid4}++;
    return  $seq_arg;
}

sub savefh($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
           fhandle => { type => SCALAR, default => $self->{curr_fh} },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    my $minor_version = 1;

    # Convert filehandle of object into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);

    # Create compound request
    my $seq_arg = $self->_sequence();
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                { op => OP_PUTFH,    args => $fhandle },
                { op => OP_SAVEFH } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'savefh',
                                     ops => \@ops );

    return $out;
} #end sub savefh

sub secinfo_no_name($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dhandle => { type => SCALAR, default => $self->{curr_fh} },
            style   => { type => SCALAR, regex => qr/^[01]$/ },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Convert filehandle of directory and
    # the style of object into XDR format
    my $dhandle = xdr_fhandle(encode=>$opts{dhandle}, version=>$ver);
    my $style   = xdr_uint(encode=>$opts{style});

    # Create compound request
    my $seq_arg = $self->_sequence();
    my @ops = ( { op => OP_SEQUENCE,        args => $seq_arg },
                { op => OP_PUTFH,           args => $dhandle },
                { op => OP_SECINFO_NO_NAME, args => $style } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'secinfo_no_name',
                                     ops => \@ops );

    # Create a copy of the secinfo_no_name reference for ease of use
    $out->{secinfo_no_name} = $out->{results}[1]{result}{secinfo_no_name};

    return $out;
} #end sub secinfo_no_name

sub nverify($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            attr    => { type => HASHREF },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Convert filehandle of object and
    # the sattr structure into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
    my $sattr   = xdr_fattr4(encode=>$opts{attr});

    # Create compound request
    my $seq_arg = $self->_sequence();
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                { op => OP_PUTFH,    args => $fhandle },
                { op => OP_NVERIFY,  args => $sattr } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'nverify',
                                     ops => \@ops );

    return $out;
} #end sub nverify

sub verify($%) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            attr    => { type => HASHREF },
        }
    );
    my $out;
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Convert filehandle of object and
    # the sattr structure into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
    my $sattr   = xdr_fattr4(encode=>$opts{attr});

    # Create compound request
    my $seq_arg = $self->_sequence();
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg },
                { op => OP_PUTFH,    args => $fhandle },
                { op => OP_VERIFY,   args => $sattr } );

    # Call the compound procedure
    $out = $self->process_operation( tag => 'verify',
                                     ops => \@ops );

    return $out;
} #end sub verify

#---------------------------------------------------------------------

=head2 putrootfh

  $nfs_obj->putrootfh();

Calls OP_PUTROOTFH and returns the filehandle returned by the server.

=item Returns

The root filehandle.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------


sub putrootfh($) {
    my $self = shift;
    my $rpc  = $self->{rpc};
    my $out;
    my $vers = $self->{version};
    my $minor_version=$self->{minor_version};

    # PUTROOTFH to get the filehandle of the public filehandle from the server

    my $seq_arg = $self->_sequence();


    # Create compound request:
    # Include SEQUENCE op
    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg},
                { op => OP_PUTROOTFH} );

    #   and finally get the filehandle that should represent the last putrootfh
    push @ops, {op => OP_GETFH};

    # Call the compound procedure
    $out = $self->process_operation(tag => 'putrootfh',
                                    ops => \@ops);

    # Create a copy of the filehandle for ease of use
    $out->{fhandle} = $out->{results}[-1]{result}{fhandle};

    # Save the current filehandle on prev_fh and set
    # the current filehandle to the one provided
    $self->{prev_fh} = $self->{curr_fh};
    $self->{curr_fh} = $out->{fhandle};
    $self->{root_fh} = $out->{fhandle};

    return $self->{root_fh};
} ## end sub putrootfh




##############################################################################
# protocol and version methods added by kartikr to support perforce ops scripts
###############################################################################

sub protocol { return shift->{proto} }

sub version { return shift->{version} }

sub hostname { return shift->{hostname} }

#---------------------------------------------------------------------

=head2 sequence

  $nfs_obj->sequence();

Calls OP_SEQUENCE and increments the sequenceId if successful.

=item Returns

The output of the call.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------

sub sequence($) {
    my $self = shift;
    my ($out, $seq_arg);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};
    $seq_arg = $self->_sequence();

    croak "Rpc object not defined! Will require debugging. Please contact the maintainer of the code.." if !defined $rpc;

    my @ops = ( { op => OP_SEQUENCE, args => $seq_arg } );

        # Call the compound procedure
    $out = $rpc->clnt_call(proc => 1,
                            arg  => xdr_compound(tag => 'sequence',
                            minor => $self->{minor_version},
                            ops => \@ops),
                            out  => \&xdr_compound_res
                        );

    # Check for errors
    $self->error_check(name=>'SEQUENCE', result=>$out);

    return $out;
}

1;

