# -*- perl -*-

## ---------------------------------------------------
## Copyright (c) 2008 NetApp, Inc., All Rights Reserved
## Permission is granted to use this script on a NetApp Product.
## Any other use, modification, or distribution is prohibited
## without prior written consent from NetApp, Inc.
## ---------------------------------------------------

package nfs::pNFS;

## ---------------------------------------------------
## @summary
##  Routines for pNFS 
##
## @description
##  This routine will be used to 
##  to setup a node in similar way the nfssetup function does for 7G system.
##
## @usage
##  use pNFS;
##
## @status      new
##
## @author      talukder@netapp.com
##

use strict;
use base qw(nfs::NfsClient);
use Exporter;
use Tharn;
use Carp;
use SPARSE::Util::Logger;
use nfs::NfsDef qw(:NFSCONSTANTS);
use Options;
use Tharn;
use SPARSE::Util::Logger;
use Params::Validate qw(validate SCALAR ARRAYREF HASHREF UNDEF);

use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);
#
#$VERSION = "1.00";
#
@ISA       = qw(Exporter nfs::NfsClient);

our @EXPORT_OK = qw(
  &check_and_empty_mhost
  &clear_deviceid_cache
  &get_deviceid_mhost
  &check_layoutreturn
  &_calculate_stripe
);

my $Options = new Options;


sub new($%;) {

    my $class = shift;
    $Options->process();

    my (%opts) = validate (
        @_,
        {
            hostname  => { type => SCALAR },
            dstaddr   => { type => SCALAR, optional => 1 },
            version   => { type => SCALAR,    regex => qr/^[234]$/ },
            resvport  => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 0 },
            proto     => { type => SCALAR,   default => 'tcp' },
            xid       => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => int(rand(1000000000)) },
            retry     => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 0 },
            reconnect => { type => SCALAR,    regex => qr/^[01]$/,
                                            default => 0 },
            timeout   => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 30 },
            qsize     => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 0 },
            localaddr => { type => SCALAR, optional => 1 },
            addr_type => { type => SCALAR, default => $Options->get(option=>"ADDR_TYPE") },
            min_ver   => {type => SCALAR,  optional => 1,
                                            default => 0 },
            eia_flags => {type => SCALAR , default => 1},
            eia_state_protect   => { type => SCALAR,   default => 0 },
            impl_id  => { type => HASHREF,optional =>1,default => {} },
	    csa_flags=>  {type => SCALAR, default => 'CREATE_SESSION4_FLAG_CONN_BACK_CHAN'},
	    path    =>  {type => SCALAR},
	    callback   => {type => SCALAR, default  => 0},

        }
    );
   my ($self) = nfs::NfsClient->new(hostname => $opts{hostname},
				     version => $opts{version},
				     proto   => $opts{proto},
				     minor_version   => $opts{min_ver});
   bless $self, $class;
   $Log->debug("Send the exchange_id op");
   $self->exchange_id (eia_flags => $opts{eia_flags},
		       eia_state_protect => $opts{eia_state_protect},
		       impl_id => $opts{impl_id});
   $self->create_session (csa_flags => $opts{csa_flags});
   if ($opts{callback} == 1) {
	#$self->callback_process();
	#$self->callback_process();
	}
   $self->mount(path => $opts{path}); 
   return $self;
}

sub read_pnfs($%,)
{
	my $self = shift;
	my %opts = validate (
	@_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            offset  => { type => SCALAR,   regex => qr/^\d+$/ },
            count   => { type => SCALAR,   regex => qr/^\d+$/ },
            stateid => { type => HASHREF|SCALAR, optional => 1 },
            inval_fh => { type => SCALAR, default => 0},
            inval_st => { type => SCALAR, default => 0}
        }
    );

    my ($out, $offset, $args, $op, $i, $read_fh, $state_id);
    my @ips;
    my @ip;
    my $rpc     = $self->{rpc};
    my $ver     = $self->{version};
    my $min_ver = $self->{minor_version};
    my $seq_id4 = $self->{seq_id4};
    my $my_xdr;

   $op = _calculate_stripe($self, %opts);
   @ip = @$op;
   if(!$opts{inval_fh}) {
	$read_fh = $self->{fhandle};
   }
   else {
   	$read_fh = $opts{fhandle};
   }
   if($opts{inval_st}) {
   	$state_id = $self->{state_id};
   }
   else {
   	$state_id = $opts{stateid};
   }
   logcomment ("No of lifs is " . $#ip);

   for (my $kl = 0; $kl <= $#ip; $kl++) {
	logcomment ("The ip addr for " . $kl . " th time is " . $ip[$kl]);
	}

    for ($i = 0; $i <= $#ip; $i++) {
        if ($i == 0) {
		# If this is the first DS then we need to read from the offset to the first stripe 
		logcomment ("We have matched zero and the ip address is " . $ip[$i] );
        if ($main::ADDR_TYPE eq "inet6") {
            my @values = split(/\./,$ip[$i]);
            logcomment ("We have matched zero and the ip address is value " . $values[0]);
            my %h1 = ( ds_ip => $values[0],
                       offset => $opts{offset},
                       fhandle => $read_fh,
                       count   => $self->{stripe_size},
                       stateid => $state_id );
            return nfs_read(%h1);
            }
        if ($main::ADDR_TYPE eq "inet4") {
            my %h1 = ( ds_ip => $ip[$i],
			   offset => $opts{offset},
			   fhandle => $read_fh,
			   count   => $self->{stripe_size},
			   stateid => $state_id );
		     return nfs_read(%h1);
        }
	} elsif ($i == $#ip) {
		logcomment ("We have matched last and the ip address is " . $ip[$i] );
                my %h1 = ( ds_ip => $ip[$i],
                           offset => $opts{offset} * $i,
                           fhandle => $read_fh,
                           count   => $self->{stripe_size},
                           stateid => $state_id );

		# If this is the last DS then we need to read from the penultimate stripe to the length
		return nfs_read(%h1);
	} else {
		# If this DS is in-between the first and last one then we need to read from the full stripe
		my %h1 = ( ds_ip => $ip[$i],
                           offset => $opts{offset} * $i,
                           fhandle => $read_fh,
                           count   => $self->{stripe_size},
                           stateid => $state_id );
		return nfs_read(%h1);
		}
		
	}
}


sub nfs_read(%,)
{
        my %opts = validate (
        @_,
        {
            ds_ip   => { type => SCALAR },
            offset  => { type => SCALAR,   regex => qr/^\d+$/ },
            fhandle => { type => SCALAR },
            count   => { type => SCALAR,   regex => qr/^\d+$/ },
            stateid => { type => HASHREF|SCALAR, optional => 1 },
        }
	);
	logcomment ("Using" . $opts{ds_ip} . " for reading data");
	my $ds_clnt_obj = nfs::NfsClient->new(hostname => $opts{ds_ip}, 
				      version  => 4,
				      minor_version  => 1,
				      proto    => 'tcp');
	my $exc_out = $ds_clnt_obj->exchange_id(eia_flags => EXCHGID4_FLAG_USE_PNFS_DS);
	my $session_out = $ds_clnt_obj->create_session();
	my $read_op = $ds_clnt_obj->read(fhandle => $opts{fhandle}, 	
			    		 stateid => $opts{stateid},  
					 count => $opts{count}, 
					 offset => $opts{offset});

return $read_op;

}

sub write_pnfs($%,)
{
        my $self = shift;
        my %opts = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            offset  => { type => SCALAR,   regex => qr/^\d+$/ },
            data   => { type => SCALAR,   default => 'abcdef'},
            stateid => { type => HASHREF|SCALAR, optional => 1 },
            inval_fh => { type => SCALAR, default => 0},
            inval_st => { type => SCALAR, default => 0}
        }
    );

    my ($out, $offset, $args, $op, $i, $write_fh, $state_id);
    my (@ips, @ip);
    my $rpc     = $self->{rpc};
    my $ver     = $self->{version};
    my $min_ver = $self->{minor_version};
    my $seq_id4 = $self->{seq_id4};
    my $my_xdr;

    $op = _calculate_stripe($self, %opts);
    @ip = @$op;

   if(!$opts{inval_fh}) {
        $write_fh = $self->{fhandle};
   }
   else {
        $write_fh = $opts{fhandle};
   }
   if($opts{inval_st}) {
        $state_id = $self->{state_id};
   }
   else {
        $state_id = $opts{stateid};
   }

   for ($i = 0; $i <= $#ip; $i++) {
        if ($i == 0) {
                # If this is the first DS then we need to read from the offset to the first stripe
                logcomment ("We have matched zero and the ip address is " . $ip[$i] );
        if ($main::ADDR_TYPE eq "inet6") {
            my @values = split(/\./,$ip[$i]);
            logcomment ("We have matched zero and the ip address is value " . $values[0]);
            my %h1 = ( ds_ip => $values[0],
                       offset => $opts{offset},
                       fhandle => $write_fh,
                       data   => $opts{data},
                       stateid => $state_id );
            return nfs_write(%h1);
            }

        if ($main::ADDR_TYPE eq "inet4") {
            my %h1 = ( ds_ip => $ip[$i],
                           offset => $opts{offset},
                           fhandle => $write_fh,
                           data   => $opts{data},
                           stateid => $state_id );
            return nfs_write(%h1);
            }
        } elsif ($i == $#ip) {
                logcomment ("We have matched last and the ip address is " . $ip[$i] );
                my %h1 = ( ds_ip => $ip[$i],
                           offset => $opts{offset} * $i,
                           fhandle => $write_fh,
                           data   => $opts{data},
                           stateid => $state_id );

                # If this is the last DS then we need to read from the penultimate stripe to the length
                return nfs_write(%h1);
        } else {
                # If this DS is in-between the first and last one then we need to read from the full stripe
                my %h1 = ( ds_ip => $ip[$i],
                           offset => $opts{offset} * $i,
                           fhandle => $write_fh,
                           data   => $opts{data},
                           stateid => $state_id );
                return nfs_write(%h1);
                }

        }
}


sub nfs_write($$%,)
{
        my %opts = validate (
        @_,
        {
            ds_ip   => { type => SCALAR },
            offset  => { type => SCALAR,   regex => qr/^\d+$/ },
            fhandle => { type => SCALAR},
            data   => { type => SCALAR},
            stateid => { type => HASHREF|SCALAR, optional => 1 },

        }
     );
        my $ds_clnt_obj = nfs::NfsClient->new(hostname => $opts{ds_ip},
                                      version  => 4,
                                      minor_version  => 1,
                                      proto    => 'tcp');
        my $exc_out = $ds_clnt_obj->exchange_id(eia_flags => EXCHGID4_FLAG_USE_PNFS_DS);
        my $session_out = $ds_clnt_obj->create_session();
        my $write_op = $ds_clnt_obj->write(fhandle => $opts{fhandle},
                                         stateid => $opts{stateid},
                                         data => $opts{data},
                                         offset => $opts{offset});

	return $write_op;

}



sub _calculate_stripe($%,)
{
	my $self = shift;
        my %opts = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            offset  => { type => SCALAR, regex => qr/^\d+$/ },
            count   => { type => SCALAR, regex => qr/^\d+$/, default => 10000 },
            data    => { type => SCALAR, optional => 1}, 
	    stateid => { type => HASHREF|SCALAR, optional => 1 },
            inval_fh => { type => SCALAR, default => 0},
	    inval_st => { type => SCALAR, default => 0}
        }
    );
	my ($layout, $device_id, $index, $first_index, $strip_size, $getdevinfo, $dev_index, $ip_addr, $low, $high, $ds_no, $i);
	my (@dev_i, @ipde, @ret_arr, $aaa_1);
	my $length = $opts{count};

	# Send LAYOUTGET op to get the device-id, the filehandle, the stripe size and the first stripe index

	$layout = $self->layoutget( fhandle => $opts{fhandle},
				    stateid => $opts{stateid},
				    length  => $opts{count},
				    offset  => $opts{offset});	
	$device_id = $layout->{deviceid_l};
	$self->{fhandle} = $layout->{layout_fhandle};
	$first_index = $layout->{stripe_to_use};
	$strip_size = $layout->{stripe_size};
	$self->{stripe_size} = $strip_size;
	$self->{state_id} = $layout->{layout_stateid};

	# Send GETDEVICEINFO op to get the device indices and the IP addresses corresponding to the device-id

	$getdevinfo = $self->getdeviceinfo ( deviceid => $device_id );
	$dev_index  = $getdevinfo->{dev_index};
	@dev_i = @$dev_index;
	$ip_addr    = $getdevinfo->{ip_addr};
	@ipde = @$ip_addr;
	# Store the IP addresses in an array

	for ($index=0; $index <= $#ipde; $index++) {
        	if ($ipde[$index] =~ /(\d+)(\.\d+)(\.\d+)(\.\d+)/) {
               		 $ipde[$index] = "$1$2$3$4";
        	}
	}
	
	# Calculate from which data servers the data has to be accessed
	$low  = int($opts{offset}/$strip_size);
	$high = int(($opts{offset}+$length)/$strip_size);

	logcomment ("Offset is " . $opts{offset} . " count is " . $length . " high is " . $high . " low is  " . $low);
	# Get the total no of data servers that will be required for this operation
	$ds_no = $high-$low;
	#$ds_no = $ds_no + 1;
	logcomment ("For this operation we will need " . $ds_no . " no of data servers");

	my $j = 0;
	my $i;
	
	for ($i = 0; $i <= $ds_no; $i++) {
		my $index_to_use = $first_index + $i;
		my $ds_to_use    = $dev_i[$index_to_use];
		$ret_arr[$j] = $ipde[$ds_to_use];
		if ($i > 4095) {
	                # Provide support for a read/write operation which is spanning accross more than 4096 stripes,
	                # we need to start from first_stripe_to_use after 4096 elements
			$i = 0;
			$ds_no = $ds_no - 4095;
		}
		$j++;
	}

 return \@ret_arr;
}



sub get_deviceid_mhost($) {
    my (@b, @c, $out, $out1);
    my $server = shift;
    #$Test->log("get_deviceid_mhost");
    #eval { $out = $server->cli(cmd => "set -privilege diagnostic -confirmations off; debug smdb table pNfsDeviceIdMappings show"); };
    eval { $out = $server->cli(cmd => "set -privilege diagnostic -confirmations off; nfs pnfs devices mappings show -vserver $main::VSERVER_GX_NAME"); };
    print ("Output from mappings show is: " . $out);
    my @a = split(/\n/, $out);
    for (my $i = 2; $i<= $#a; $i++) {
        push(@b,  split(/X;X/, $a[$i]));
    }
    print ("Output from mappings show is: " . @b);
    eval {$out1 = $server->cli(cmd => "set -privilege diagnostic -confirmations off; nfs pnfs devices show"); };
    my @a = split(/\n/, $out1);
    for (my $i = 2; $i<= $#a; $i++) {
        push(@c,  split(/X;X/, $a[$i]));
    }
    my %h1 = (deviceid => \@c,              devmap   => \@b);

    #$Test->log("pNfsDeviceId is " . $out);
    return \%h1;
}


sub clear_deviceid_mhost($;) {
    my ($server) = @_;
    eval { my $out = $server->cli(cmd => "set -privilege test; nfs pnfs devices delete -mid *"); };

    if ($@ && $@ !~ /There are no entries matching your query/) {
        croak ("Clearing deviceid mhost failed: $@");
    }
}

sub check_layoutreturn($$$;) {
     my ($client, $filehandle, $stateid) = @_;
     my $rpc = $client->{rpc};
     if ($rpc->{cb_layoutreturn} == 1) {
     #$Test->log("Sending LAYOUTRETURN");
     $client->layoutreturn(fhandle => $filehandle, stateid => $stateid);
     }
}


sub clear_deviceid_cache($) {

    #$Test->log("Clearing the device-id cache and vsid is $vsid");
     my $server = shift;
     my ($i, $x, @vs, $vs);
     my $vs = $server->cli(cmd => "set -privilege diagnostic -confirmations off; debug smdb table vserver_id_to_name show");
     my @a = split(/\n/, $vs);
     for ($i =0 ; $i <= $#a; $i++) {
        if ($a[$i] =~ /$main::VSERVER/) {
                $x = $a[$i];
           }
        }
     @vs = split(/X/, $x);
     #$Test->log("Vserver id is $vs[0]");

    eval { $server->cli(cmd => "set -privilege test -confirmations off; vserver nfs pnfs devices cache flush -node $main::FILER -vserver $main::VSERVER_GX_NAME"); };
    eval { $server->cli(cmd => "set -privilege test -confirmations off; vserver nfs pnfs devices cache flush -node $main::FILER -vserver $main::VSERVER_GX_NAME"); };
    if ($@ =~ /pNFS device cache for vserver not found/) {
        #$Test->log("The device-id cache is clear");
        }


}

sub check_and_empty_mhost($) {

    #$Test->log("Checking the device-id cache and if not empty clearing the data");
    my $server = shift;
    my $ret = get_deviceid_mhost($server);
        print "The return value is $ret \n";
    my $ret1 = $ret->{devmap};
    my $ret2 = $ret->{deviceid};
    my @arr = @$ret1;
    my @arr1 = @$ret2;
    print "Elements ot return are @arr \n";
    print "Elements ot return are @arr1 \n";
    if (@arr != undef || @arr1 != undef) {
        clear_deviceid_mhost($server);
    }
}


1;
