## @summary OOP Adapter for the lib/mount.pm library.
## @description Provides an object-oriented interface to NAS mounts.
## @dependencies variant of lib/mount.pm, lib/nfs/xmount.pm.
## @author  dl-maryv (FSS Protocols QA)
package nfs::rcalls;

use strict;
use Tharn;
use POSIX;
use nfs::say;

use vars qw(@ISA @EXPORT);

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(&rcalls_stat);

##############################################################################

sub rcalls_stat($$;) {
    return eval {
	my $path = shift;
	my $client = shift;

	if ($client) {
	    # put the path at the end so we can verify it if we want to.
	    my $sobj = Tharn::subtest("{
                    logresult('PASS', join(' ', stat(\"$path\"), \"$path\"))
                        or tstlog('-fatal', \$@);}", '-client', $client,
				  '-onexit', sub {}, '--', client => "");
	    my $result = Rslt::worst($sobj->rslts());
	    if ($result && $result->severity() >
                           NATE::Result::Warn->severity()) {
		die join("\n", $result->msgs());
	    }
	    my @r;
	    foreach my $m ($result->msgs()) {
		push(@r, split(/\s+/, $m));
	    }
	    if (@r == 14) {
		return @r;
	    }
	    die join("\n", $result->msgs());
	} else {
	    return (stat($path), $path) or die "$!";
	}
    };
}

1;
