# -*- perl -*-

## ---------------------------------------------------
## Copyright (c) 2008 NetApp, Inc., All Rights Reserved
## Permission is granted to use this script on a NetApp Product.
## Any other use, modification, or distribution is prohibited
## without prior written consent from NetApp, Inc.
## ---------------------------------------------------

package nfs::vStorageclient;

## ---------------------------------------------------
## @summary
##  Routines for vStorage synthetic client 
##
## @description
##  This routine will be used to 
##   generate various vStorage ops
##
## @usage
##  use vStorqgeclient;
##
## @status      new
##
## @author      talukder@netapp.com
##

use strict;
use Tharn;
use Carp;
#use Thread;
use Switch;
use SPARSE::Util::Logger;
use base qw(nfs::NfsClient::4_0);
use nfs::NfsDef qw(:NFSCONSTANTS);
use Options;
use Tharn;
use SPARSE::Util::Logger;
use nfs::Rpc;
use Params::Validate qw(validate SCALAR ARRAYREF HASHREF UNDEF OBJECT);
use nfs::NfsXdr qw(xdr_access xdr_bitmap4 xdr_cb_compound xdr_cb_exit
  xdr_cb_info xdr_cb_nlm_granted xdr_cb_nsm_notify xdr_cb_pid xdr_cb_port
  xdr_cb_register xdr_commit xdr_compound xdr_compound_res xdr_cookie
  xdr_cookieverf xdr_create xdr_daemon_info xdr_dump xdr_export xdr_fattr4
  xdr_fhandle xdr_fsinfo xdr_fsstat xdr_getattr xdr_link xdr_lookup xdr_mkdir
  xdr_mknod xdr_mount xdr_nfstime xdr_nlm_lock xdr_nlm_stat xdr_nlm_test
  xdr_opaque xdr_pathconf xdr_read xdr_readdir xdr_readdirplus xdr_readlink
  xdr_remove xdr_rename xdr_sattr xdr_setattr xdr_specdata xdr_stateid4
  xdr_symlink xdr_uint xdr_uint64 xdr_write xdr_name);
use Data::Dumper;

my $Options = new Options;
my %call_back = (
    0 => sub {return},
    6 => \&xdr_cb_compound,
);


sub new($%;) {

    my $class = shift;
    $Options->process();

    my (%opts) = validate (
        @_,
            {
            hostname  => { type => SCALAR },
            dstaddr   => { type => SCALAR, optional => 1 },
            version   => { type => SCALAR,    regex => qr/^[1]$/ },
            resvport  => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 0 },
            proto     => { type => SCALAR,    regex => qr/^(udp|tcp)$/,
                                            default => 'tcp' },
            mnt_ver   => { type => SCALAR,    regex => qr/^[123]$/,
                                           optional => 1 },
            mnt_proto => { type => SCALAR,    regex => qr/^(udp|tcp)$/,
                                           optional => 1 },
            nlm_ver   => { type => SCALAR,    regex => qr/^[1234]$/,
                                           optional => 1 },
            nlm_proto => { type => SCALAR,    regex => qr/^(udp|tcp)$/,
                                           optional => 1 },
            port      => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 0 },
            xid       => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => int(rand(1000000000)) },
            retry     => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 0 },
            reconnect => { type => SCALAR,    regex => qr/^[01]$/,
                                            default => 0 },
            timeout   => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 30 },
            qsize     => { type => SCALAR,    regex => qr/^\d+$/,
                                            default => 0 },
            localaddr => { type => SCALAR, optional => 1 },
            nlm       => { type => SCALAR,  default => 0 },
            nsm       => { type => SCALAR,  default => 0 },
            nsm_proto => { type => SCALAR,    regex => qr/^(udp|tcp)$/,
                                            default => 'udp' },
            addr_type => { type => SCALAR, default => $Options->get(option=>"ADDR_TYPE") },
            min_ver   => {type => SCALAR,  optional => 1,
                                            default => 0 },
            rpc_vers  => {type => SCALAR,  optional => 1,
                                            default => 2 }

        }
   );

    my ($self, %rpc_opts_h);

    my $rpc = nfs::Rpc->clnt_create(hostname  => $opts{hostname},
                                    version   => 1,
                                    resvport  => $opts{resvport},
                                    proto     => $opts{proto},
                                    port      => $opts{port},
                                    xid       => $opts{xid},
                                    retry     => $opts{retry},
                                    timeout   => $opts{timeout},
                                    qsize     => $opts{qsize},
                                    reconnect => $opts{reconnect},
                                    addr_type => $opts{addr_type},
                                    program   => 400010,
                                    %rpc_opts_h);

    eval { $rpc->clnt_call(proc => 0) };
    croak $@ if ($@ and $@ !~ /4 extra bytes/);

   my %v4options;

        %v4options = (
            seqid       => 0,
            lock_seqid  => 0,
            cb_queue    => \@nfs::NfsXdr::cb_queue,
            open_delegs => \@nfs::NfsXdr::open_delegs,
            cl_verifier => time + int(rand(1000000000)),
            cl_id       => time + int(rand(1000000000)),
        );

    $self = bless { %opts, %v4options,
                    rpc      => $rpc }, $class;

return $self;
}


sub copy_notify($%,) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
	    netid   => { type => SCALAR, default => 'tcp',
					 regex   => qr/tcp|tcp6/ },
	    naddr   => { type => SCALAR, regex   => qr/^[\.0-9a-fA-F\:]+$/ },
        }
    );
    my ($out, $args, @ops);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Convert filehandle of file
    # and the count into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
    $args = xdr_uint(encode => 2) .
            xdr_opaque(encode => $opts{netid}) .
            xdr_opaque(encode => $opts{naddr});

    @ops = ( { op => OP_PUTFH, args => $fhandle },
             { op => OP_COPY_NOTIFY,  args => $args },
             { op => OP_GETFH } );

    $out = $rpc->clnt_call(proc => 1,
                           arg  => xdr_compound(tag => 'copy_notify',
                                                ops => \@ops),
                           out  => \&xdr_compound_res);
    $Log->trace(qq{copy_notify output: } . Dumper($out));
    $self->error_check(name=>'COPY_NOTIFY', result=>$out);

    $self->{nloc_type} =  $out->{results}->[1]->{result}->{nloc_type};	
    if ($self->{nloc_type}[0] == 0){
	$self->{copy_host} = $out->{results}->[1]->{result}->{host_name};	
    } elsif ($self->{nloc_type}[0] == 1) {
	$self->{url} = $out->{results}->[1]->{result}->{url};	
    } else {
	$self->{t_proto} = $out->{results}->[1]->{result}->{t_proto};	
	$self->{t_addr} = $out->{results}->[1]->{result}->{t_addr};	
    }

    return $out;

}

#sub hole_punch($%,) {
#    my $self = shift;
#    my (%opts) = validate (
#        @_,
#        {
#            fhandle => { type => SCALAR,   default => $self->{curr_fh} },
#	    offset  => { type => SCALAR,   regex => qr/^\d+$/ },
#            length  => { type => SCALAR,   regex => qr/^\d+$/ },
#
#        }
#    );
#
#    my ($out, $offset, $args, @ops);
#    my $rpc     = $self->{rpc};
#    my $ver     = $self->{version};
#
#    # Convert filehandle of file
#    # and the count into XDR format
#    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
#    $args = xdr_uint64(encode => $opts{offset}) .
#	    xdr_uint64(encode => $opts{length});
#
#    @ops = ( { op => OP_PUTFH, args => $fhandle },
#             { op => OP_HOLE_PUNCH,  args => $args },
#             { op => OP_GETFH } );
#
#    $out = $rpc->clnt_call(proc => 1,
#                           arg  => xdr_compound(tag => 'hole_punch',
#                                                ops => \@ops),
#                           out  => \&xdr_compound_res);
#    $self->error_check(name=>'HOLE_PUNCH', result=>$out);
#
#    return $out;
#}

sub copy_revoke($%,) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            netid   => { type => SCALAR, default => 'tcp',
                                         regex   => qr/tcp|tcp6/ },
            naddr   => { type => SCALAR, regex   => qr/^[\.0-9a-fA-F\:]+$/ },
        }
    );
    my ($out, $args, @ops);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Convert filehandle of file
    # and the count into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
    $args = xdr_uint(encode => 2) .
            xdr_opaque(encode => $opts{netid}) .
            xdr_opaque(encode => $opts{naddr});

    @ops = ( { op => OP_PUTFH, args => $fhandle },
             { op => OP_COPY_REVOKE,  args => $args },
             { op => OP_GETFH } );

    $out = $rpc->clnt_call(proc => 1,
                           arg  => xdr_compound(tag => 'copy_revoke',
                                                ops => \@ops),
                           out  => \&xdr_compound_res);
    $self->error_check(name=>'COPY_REVOKE', result=>$out);

    return $out;

}

sub copy_abort($%,) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            stateid => { type => HASHREF|SCALAR },
        }
    );
    my ($out, $args, @ops);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Convert filehandle of file
    # and the count into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
    $args = xdr_stateid4(encode => $opts{stateid}) ;

    @ops = ( { op => OP_PUTFH, args => $fhandle },
             { op => OP_COPY_ABORT,  args => $args },
             { op => OP_GETFH } );

    $out = $rpc->clnt_call(proc => 1,
                           arg  => xdr_compound(tag => 'copy_abort',
                                                ops => \@ops),
                           out  => \&xdr_compound_res);
    $self->error_check(name=>'COPY_ABORT', result=>$out);

    return $out;

}

sub copy_status($%,) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            fhandle => { type => SCALAR, default => $self->{curr_fh} },
            stateid => { type => HASHREF|SCALAR, default => $self->{stateid} },
        }
    );
    my ($out, $args, @ops);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Convert filehandle of file
    # and the count into XDR format
    my $fhandle = xdr_fhandle(encode=>$opts{fhandle}, version=>$ver);
    $args = xdr_stateid4(encode => $opts{stateid}) ;

    @ops = ( { op => OP_PUTFH, args => $fhandle },
             { op => OP_COPY_STATUS,  args => $args },
             { op => OP_GETFH } );

    $out = $rpc->clnt_call(proc => 1,
                           arg  => xdr_compound(tag => 'copy_status',
                                                ops => \@ops),
                           out  => \&xdr_compound_res);
    $self->error_check(name=>'COPY_STATUS', result=>$out);

    return $out;

}


sub copy($%,) {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            dst_handle => { type => SCALAR, default => $self->{curr_fh} },
            src_handle => { type => SCALAR },
	    src_offset => { type => SCALAR, regex => qr/^\d+$/ , default => 0 },
	    dst_offset => { type => SCALAR, regex => qr/^\d+$/ , default => 0 },
	    length     => { type => SCALAR, regex => qr/^\d+$/ , default => 0 },
	    copy_flag  => { type => SCALAR, default => 4 },
	    component  => { type => SCALAR, optional => 1 },
            netid   => { type => SCALAR|ARRAYREF, default => $self->{t_proto},
                                          optional => 1 },
            naddr   => { type => SCALAR|ARRAYREF, optional => 1,
					 default => $self->{t_addr}},
            netlock => { type => SCALAR|ARRAYREF, default => $self->{nloc_type}, 
					 optional => 1 },
            copy_host=> { type => SCALAR|ARRAYREF, default => $self->{host_name},
					  optional => 1 },
            url     => { type => SCALAR|ARRAYREF, default => $self->{url},
					 optional => 1 },

        }
    );
    my ($out, $args, @ops, $elements, $i);
    my $rpc = $self->{rpc};
    my $ver = $self->{version};

    # Convert filehandle of file
    # and the count into XDR format
    my $len = length($opts{component});
    my $dst_fhandle = xdr_fhandle(encode=>$opts{dst_handle}, version=>$ver);
    my $src_fhandle = xdr_fhandle(encode=>$opts{src_handle}, version=>$ver);
	$opts{component} = 0;
	$args = xdr_uint64(encode => $opts{src_offset}) .
            xdr_uint64(encode => $opts{dst_offset}) .
            xdr_uint64(encode => $opts{length}) .
            xdr_uint(encode => $opts{copy_flag}) .
	    xdr_uint(encode => $opts{component});
    my @arr = $opts{netlock};
    $Log->debug("Array is @arr and url is $self->{url}[0]");
    $elements = $#arr;
    $args .= xdr_uint(encode => $elements+1) ;	# Number of elements in array
    for ($i = 0; $i <= $elements; $i++) {
			$Log->debug("Getting for $i th time and netloc is  $opts{netlock}");
            		$args .= xdr_uint(encode => $opts{netlock}[$i]) ;	# NL4_NETADDR
			switch ($opts{netlock}[$i]) {
				case 0 {
						$args .= xdr_opaque(encode => $opts{copy_host}[$i]);
					}
				case 1  {
						$args .= xdr_opaque(encode => $opts{url}[$i]);
					}
				case 2 {
						$args .= xdr_opaque(encode => $opts{netid}) .
				                          xdr_opaque(encode => $opts{naddr});
					}
				}
	}
						

    @ops = ( { op => OP_RENEW, args => $self->{clientid} },
	     { op => OP_PUTFH, args => $src_fhandle },
	     { op => OP_SAVEFH },
	     { op => OP_PUTFH, args => $dst_fhandle },
             { op => OP_COPY,  args => $args },
             { op => OP_GETFH } );

    $out = $rpc->clnt_call(proc => 1,
                           arg  => xdr_compound(tag => 'copy',
                                                ops => \@ops),
                           out  => \&xdr_compound_res);
    $self->error_check(name=>'COPY', result=>$out);
    $out->{stateid} = $out->{results}->[4]->{result}->{stateid}; 
    return $out;

}

return 1;
