## @summary SFTP routines for ONTAP 7.3.2 and later
## @author maheshkr@netapp.com or dl-nb-cifs-qa@netapp.com
## @status Shared

=head1 NAME

ntap_sftp  -  SFTP routines built over Net::SFTP module customized to NetApp Test Environment

=head1 SYNOPSIS

use ntap_sftp qw(sftp_login sftp_pwd sftp_full_path sftp_ls sftp_get sftp_mkdir sftp_put sftp_get_perm sftp_get_uid sftp_get_gid sftp_get_mtime sftp_get_atime sftp_get_size sftp_chgrp sftp_chown sftp_chmod sftp_cd sftp_rename sftp_rm sftp_rmdir sftp_ls_print);

=head1 DESCRIPTION

=head1 SEE-ALSO

Net::SFTP
Net::SFTP::Attributes
Net::SFTP::Constants

=head1 ROUTINES

=cut

package ntap_sftp;

use strict;
use Burt;
use Tharn;
require Exporter;
use Net::SFTP;
use Net::SFTP::Attributes;
use Net::SFTP::Constants qw( :fxp :flags SSH2_FXF_READ :status SSH2_FX_EOF :att);
use Net::SFTP::Util qw(fx2txt);
use SayWrap qw(sayEOF);
use vars qw(@ISA @EXPORT $SFTP_H $FILER $CLIENT $SFTP_USER $SFTP_PASSWD);

@ISA = ('Exporter');
@EXPORT = qw(sftp_login sftp_pwd sftp_ls sftp_full_path sftp_get sftp_mkdir sftp_put sftp_get_perm sftp_get_uid sftp_get_gid sftp_get_mtime sftp_get_atime sftp_get_size sftp_chgrp sftp_chown sftp_chmod sftp_cd sftp_rename sftp_rm sftp_rmdir sftp_ls_print);

##################################################################################################
########################## --------------ROUTINES---------------- ################################
##################################################################################################
## SFTP Login routine
sub sftp_login {
	$FILER = $_[0];
	$SFTP_USER = $_[1];
	$SFTP_PASSWD = $_[2];
	$SFTP_H = Net::SFTP->new($FILER, user=>$SFTP_USER, password=>$SFTP_PASSWD,debug => 1);
#	fx2txt($SFTP_H->status());
	return $SFTP_H;
 }

## get pwd path
sub sftp_pwd {
	$SFTP_H = $_[0];
	my $path = $SFTP_H->do_realpath(".");
	return $path;
}

## get Full Path
sub sftp_full_path {
	$SFTP_H = $_[0];
	my $rel_path = $_[1];
	my $full_path = $SFTP_H->do_realpath($rel_path);
	return $full_path;
}

## list routine
sub sftp_ls {
	$SFTP_H = $_[0];
	my $path = sftp_pwd($SFTP_H) . "/" . $_[1];
	my @handle = $SFTP_H->ls($path);
#use Data::Dumper;
#	logcomment(Dumper(@handle));
	return @handle;
}

## get routine
sub sftp_get {
	$SFTP_H = $_[0];
	my $file = sftp_pwd ($SFTP_H) . "/" . $_[1];
	my $local_file = $_[2];
	if(! defined($SFTP_H->get($file,$local_file) ) ) {
		return 1;
	} else {
		return 0;
	}
}

## put routine
sub sftp_put {
	$SFTP_H = $_[0];
	my $local_file = $_[1];
	my $abs_remote_file_path = sftp_pwd($SFTP_H) . "/" . $_[2];
	my $status = $SFTP_H->put($local_file, $abs_remote_file_path);
	sftp_ls_print($SFTP_H);
#	my $status2 = fx2txt($SFTP_H->status());
#	logcomment("SFTP PUT STATUS: $status2");
	if ($SFTP_H->status() != SSH2_FX_OK) {
		return 1;
	} else {
		return 0;
	}
}

## mkdir routine
sub sftp_mkdir {
	$SFTP_H = $_[0];
	my $dir = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $status = $SFTP_H->do_mkdir($dir);
	if ($status != SSH2_FX_OK) {
		return 1;
	} else {
		return 0;
	}
}

## get permissions
sub sftp_get_perm {
	$SFTP_H = $_[0];
	my $file = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $attrs = Net::SFTP::Attributes->new();
	$attrs = $SFTP_H->do_stat($file);
	my $perm = $attrs->perm;
	my $oct = sprintf("%o", $perm);
	$oct = $oct % 1000;
	logcomment("File $file has $oct permissions");
	return $oct;
}

## get uid
sub sftp_get_uid {
	$SFTP_H = $_[0];
	my $file = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $attrs = Net::SFTP::Attributes->new();
	$attrs = $SFTP_H->do_stat($file);
	my $uid = $attrs->uid;
	logcomment("$uid is the owner of the file $file");
	return $uid;
}

## get gid
sub sftp_get_gid {
	$SFTP_H = $_[0];
	my $file = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $attrs = Net::SFTP::Attributes->new();
	$attrs = $SFTP_H->do_stat($file);
	my $gid = $attrs->gid;
	logcomment("$gid is the group id of the file $file");
	return $gid;
}

## get modified time
sub sftp_get_mtime {
	$SFTP_H = $_[0];
	my $file = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $attrs = Net::SFTP::Attributes->new();
	$attrs = $SFTP_H->do_stat($file);
	my $mtime = $attrs->mtime;
	logcomment("$mtime is the last modified time (mtime) of the file $file");
	return $mtime;
}

## get size
sub sftp_get_size {
	$SFTP_H = $_[0];
	my $file = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $attrs = Net::SFTP::Attributes->new();
	$attrs = $SFTP_H->do_stat($file);
	my $size = $attrs->size;
	logcomment("$size is the size of the file $file");
	return $size;
}

## get last accessed time
sub sftp_get_atime {
	$SFTP_H = $_[0];
	my $file = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $attrs = Net::SFTP::Attributes->new();
	$attrs = $SFTP_H->do_stat($file);
	my $atime = $attrs->atime;
	logcomment("$atime is the last accessed time (atime) of the file $file");
	return $atime;
}

## remove directory
sub sftp_rmdir	{
	$SFTP_H = $_[0];
	my $path = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $status = $SFTP_H->do_rmdir($path);
	if($status != SSH2_FX_OK) {
		return 1;
	} else {
 		logcomment("deleted directory $path");
		return 0;
	}
}

## remove file
sub sftp_rm	{
	$SFTP_H = $_[0];
	my $file = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $status = $SFTP_H->do_remove($file);
	if ($status != SSH2_FX_OK) {
		return 1;
	} else {
		logcomment("deleted file $file");
		return 0;
	}
}

## change directory
sub sftp_cd {
	$SFTP_H = $_[0];
	my $path = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $dirhandle = $SFTP_H->do_opendir($path);
	return $dirhandle;
}

## rename - need to check what's the actual return status for success and failure
sub sftp_rename	{
	$SFTP_H = $_[0];
	my $old_path = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $new_path = sftp_pwd($SFTP_H) . "/" . $_[2];
	my $status = $SFTP_H->do_rename($old_path,$new_path);
	if (! defined($status)) {
		return 1;
	} else {
		return 0;
	}
}

## symlink
sub sftp_symlink_create	{
}

## chgrp
sub sftp_chgrp {
	$SFTP_H = $_[0];
	my $attr = Net::SFTP::Attributes->new();
	$attr->gid($_[2]);
	my $file = sftp_pwd($SFTP_H) . "/" . $_[1];
	if(! defined($SFTP_H->do_setstat($file, $attr))) {
		logresult("FAIL","chgrp of $file to $_[2] failed with error".$SFTP_H->status() );
		return 0;
	} else { logcomment("chgrp of $file to $_[2] succeeded"); 
		return 1;
	}	
}

## chown
sub sftp_chown {
	$SFTP_H = $_[0];
	my $attr = Net::SFTP::Attributes->new();
	$attr->uid($_[2]);
	my $file = sftp_pwd($SFTP_H) . "/" . $_[1];
	if(! defined($SFTP_H->do_setstat($file, $attr))) {
		logresult("FAIL","chown of $file to $_[2] failed with error".$SFTP_H->status() );
		return 0;
	} else {
		logcomment("chown of $file to $_[2] succeeded");
		return 1;
	}
}

## chmod - need to check what's the actual return status for success and failure
sub sftp_chmod {
	$SFTP_H = $_[0];
	my $attr = Net::SFTP::Attributes->new();
	$attr->perm ($_[2]);
	my $file = sftp_pwd($SFTP_H) . "/" . $_[1];
	my $status = $SFTP_H->do_setstat($file, $attr);
	my $status2 = fx2txt($SFTP_H->status());
	logcomment("CHMOD STATUS: $status2");
	if(! defined($status)) {
		return 0;
	} else {
		return 1;
	}
}

## Print the files for sftp ls command.
sub sftp_ls_print {
        $SFTP_H  = $_[0];
        my $sftp_pwd = sftp_pwd($SFTP_H);
        my $path = $sftp_pwd . "/" . $_[1];
	my @filelist;
        if (! defined $sftp_pwd) {
                logresult("FAIL", "Failed to get \"pwd\" for \"ls $path\".");
        } else {
                @filelist = sftp_ls($SFTP_H, $_[1]);
                if (!(@filelist)) {
                        logresult("FAIL", "Failed to get directory listing for \"ls $path\".");
                } else {
                        my $attr = Net::SFTP::Attributes->new();
                        logcomment("Contents of \"ls $path\":");
                        print ("PERMS\tUID\tGID\tSIZE\tMTIME\t\tFILENAME\n");
                        foreach my $file (@filelist) {
                                my $filename = $file->{filename};
                                $attr = $file->{a};
                                my $octal_perm = sprintf("%o", $attr->{perm});
				$octal_perm = $octal_perm % 1000;
                                print ("$octal_perm\t$attr->{uid}\t$attr->{gid}\t$attr->{size}\t$attr->{mtime}\t$filename\n");
                        }
                }
        }
}

##Devicing a method to convert timestamp to a readable format
##The unix time stamp is a way to track time as a running total of seconds. 
##This count starts at the Unix Epoch on January 1st, 1970. 
##Therefore, the unix time stamp is merely the number of seconds between a particular date and the Unix Epoch.
##This is very useful to computer systems for tracking and sorting dated information in dynamic and distributed applications both online and client side.

 sub convert_timestamp {


 }
