#Perl module for performing Takeover.Giveback operations and validation of IC connection throughout the process.
#by Weishi Huang, 09/25/2011
###############################################################################
#$Id: //depot/prod/test/Rsierranevada/lib/platval/Interconnect/CFO.pm#3 $
=head1 NAME

TOGB: Perl module for takeover and giveback operations

=head1 SYNOPSIS

use IConnect::CFO;
$togb_obj = TOGB->new();

=head1 REQUIRES

None

=head1 EXPORTS

None

=head1 DESCRIPTION

This module is designed to work for both c and 7 modes.

=head1 METHODS

=head2 do_takeover

=over 4

$togb_obj->do_takeover(
          con_obj => $con,
          mode => "c",
          graceful => 0,
          timeout => 60,
          );

required: con_obj, connection object of the filer on which takeover is issued.
          mode, filer mode, either 7 or c.

optional: graceful, is either 1 or 0 for yes and no. Default is 0 for ungraceful takeover.
          timeout, timeout following takeover command is issued. Default is 180 seconds.

=back

=head2 do_giveback

=over 4

=back

=head1 CLASS VARIABLES

=over 4

=None

=back

=head1 AUTHOR

Weishi Huang weishi@netapp.com, 9/28/2011

=cut
############################################################################

package platval::Interconnect::CFO;
use Carp;
use Params::Validate qw(:all);
use strict;

#############################################
sub new {
  my $class = shift;
  my $this = {};
  bless $this, $class;
  $this->{err} = '';
  return $this;
}

#=============================================================
## method: takeover on command (either forceful or unforceful)
#=============================================================
sub cfo_on_cmd {
  my $class = shift;
  my %in=validate(
        @_, {
          con_obj_a=>{type=>OBJECT},
          tran_obj_a=>{type=>OBJECT},
          con_obj_b=>{type=>OBJECT},
          tran_obj_b=>{type=>OBJECT},
          forceful=>{type=>SCALAR,
                optional=>1,
                default=>1,
                regex=>qr/^(1)|(0)$/,
               },
          debug=>{type=>SCALAR,
                optional=>1,
                default=>1,
                regex=>qr/^(1)|(0)$/,
               },
            },
          );

  my $cmd='cf takeover -f';
  if ($in{forceful} == 0) {
     $cmd = 'cf takeover';
  }
  print "Perform takeover by issing '$cmd'\n" unless !$in{debug};
  $in{con_obj_a}->execute_start(command=>$cmd);
  print "pausing 180 sec after '$cmd'\n" unless !$in{debug};
  sleep 180;
#for console connect, no need to expect any output.
# $in{con_obj_a}->execute_expect();
  my ($ct, $state_a, $state_b, $err, $out);
  $ct = 0;
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
	$in{con_obj_a}->execute(command=>"cf status");
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_A after TO=$state_a\n" unless !$in{debug};
    print "STATE_B after TO=$state_b\n" unless !$in{debug};
    if ($ct >= 600) {
      $err="Error: Was not ready for Giveback after waiting $ct sec";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_a =~ /CLI/ and $state_b =~ /TAKEN_OVER_WAITING_ONTAP/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless $in{debug};
    }
  }

# print "checking ic status after TO\n" unless !$in{debug};
# $class->check_ic_status(con_obj=>$in{con_obj_a}, stage=>'after TO');

  if ($in{forceful}) {
    $in{con_obj_a}->execute_start(command=>'cf giveback -f');
  } else {
   print "waiting 600 sec for auto giveback to complete..\n";;
   Tharn::snooze(600);
  }
  $ct = 0;
# $in{con_obj_a}->execute_expect();
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
	$in{con_obj_a}->execute(command=>"cf status");
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_A after GB=$state_a\n" unless !$in{debug};
    print "STATE_B after GB=$state_b\n" unless !$in{debug};
    if ($ct >= 60) {
      $err="Error: Giveback was not complete after waiting $ct sec: STAET_A=$state_a; STATE_B=$state_b";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_a =~ /CLI/ and $state_b =~ /PASSWORD|CLI/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless $in{debug};
    }
  }

  $class->_restore_2_cli($in{con_obj_a}, $in{tran_obj_b});
  return 0;
} ## end of cfo_on_cmd()


## method: takeover on reboot
#=============================================================
sub cfo_on_reboot {
  my $class = shift;
  my %in=validate(
        @_, {
          con_obj_a=>{type=>OBJECT},
          tran_obj_a=>{type=>OBJECT},
          con_obj_b=>{type=>OBJECT},
          tran_obj_b=>{type=>OBJECT},
          debug=>{type=>SCALAR,
                optional=>1,
                default=>1,
                regex=>qr/^(1)|(0)$/,
               },
            },
          );

  print "Perform 'reboot -t 0'\n" unless !$in{debug};
  $in{con_obj_a}->execute_start(command=>'reboot -t 0');
  $in{tran_obj_a}->set(timeout=>600);
  print "pausing 180 sec after reboot cmd...\n" unless !$in{debug};
  sleep 180;
# $in{con_obj_a}->execute_expect();
  my ($ct, $state_a, $state_b, $err, $out);
  $ct = 0;
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_A after reboot=$state_a\n" unless !$in{debug};
    print "STATE_B after reboot=$state_b\n" unless !$in{debug};
    if ($ct >= 600) {
      $err="Error: Giveback was not ready after waiting $ct sec";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_b =~ /CLI|PASSWORD/ and $state_a =~ /TAKEN_OVER_WAITING_ONTAP/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless !$in{debug};
    }
  }
  if ($state_b =~ /PASSWORD/) {
    $state_b = $in{tran_obj_b}->change_state(to=>'CLI');
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_B after change_state=$state_b, and should be CLI\n" unless !$in{debug};
  }

# print "checking ic status after TO\n" unless !$in{debug};
# $class->check_ic_status(con_obj=>$in{con_obj_b}, stage=>'after TO');

  print "Issue 'cf giveback -f'\n" unless !$in{debug};
  $in{con_obj_b}->execute_start(command=>'cf giveback -f');
  sleep 30;
# $in{con_obj_a}->execute_expect();
  $ct = 0;
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_A after GB=$state_a\n" unless !$in{debug};
    print "STATE_B after GB=$state_b\n" unless !$in{debug};
    if ($ct >= 60) {
      $err="Error: Giveback was not complete after waiting $ct sec: STAET_A=$state_a; STATE_B=$state_b";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_b =~ /CLI/ and $state_a =~ /PASSWORD|CLI/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless $in{debug};
    }
  }

  $class->_restore_2_cli($in{con_obj_b}, $in{tran_obj_a});
  return 0;
} ## end of cfo_on_reboot()

#=============================================================
## method: takeover on halt and then reboot
#=============================================================
sub cfo_on_halt {
  my $class = shift;
  my %in=validate(
        @_, {
          con_obj_a=>{type=>OBJECT},
          tran_obj_a=>{type=>OBJECT},
          con_obj_b=>{type=>OBJECT},
          tran_obj_b=>{type=>OBJECT},
          debug=>{type=>SCALAR,
                optional=>1,
                default=>1,
                regex=>qr/^(1)|(0)$/,
               },
          platform=>{type=>SCALAR,
                optional=>1,
                default=>'vsa',
               },
            },
          );

  my ($ct, $state_a, $state_b, $err, $out);

  print "issue halt\n" unless !$in{debug};
  $in{con_obj_a}->execute_start(command=>'halt -t 0');
  $in{tran_obj_a}->set(timeout=>600);
  print "pausing 180 sec after halt cmd...\n" unless !$in{debug};
  sleep 180;
  print "getting transit state after pausing 120 sec following halt cmd...\n" unless !$in{debug};
  $state_a = $in{tran_obj_a}->get_state(timeout=>600);
  print "STATE_A after halt=$state_a\n" unless !$in{debug};
#catch expected timeout at reboot menual
#For the time being, VSA platform (Model Name: DOvM200) requires entrance of Ctrl return following halt
  if ($state_a =~ /HALTED|FIRMWARE|LOADER_BANNER/) {
      if ($in{platform} =~ /^D0v/i) {
        print "This is vsa platform. Enter return to initiate reboot.\n" unless !$in{debug};
        $in{con_obj_a}->execute(command=>'\n', timeout=>900, prompts=>['Waiting for giveback']);
      } else {
        print "This is non-vsa platform. Enter boot_primary to continue.\n" unless !$in{debug};
        #$in{con_obj_a}->execute(command=>'boot_primary', timeout=>900, prompts=>['Waiting for giveback']);
        $in{tran_obj_a}->change_state(from => "FIRMWARE", to => "TAKEN_OVER_WAITING_ONTAP", timeout=>900);
      }
  } else {
      $err="Error: expected state was not HALTED|FIRMWARE|LOADER_BANNER state after 120 sec following halt cmd. \nObv state=$state_a";
      $class->{err}=$err;
      confess($err);
  }

  $ct = 0;
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_A after reboot=$state_a\n" unless !$in{debug};
    print "STATE_B after reboot=$state_b\n" unless !$in{debug};
    if ($ct >= 600) {
      $err="Error: Giveback was not ready after waiting $ct sec";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_b =~ /CLI|PASSWORD/ and $state_a =~ /TAKEN_OVER_WAITING_ONTAP/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless !$in{debug};
    }
  }
  if ($state_b =~ /PASSWORD/) {
    $state_b = $in{tran_obj_b}->change_state(to=>'CLI');
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_B after change_state=$state_b\n" unless !$in{debug};
  }

# print "checking ic status after TO\n" unless !$in{debug};
# $class->check_ic_status(con_obj=>$in{con_obj_b}, stage=>'after TO');

  print "Issue 'cf giveback -f' on filer B\n" unless !$in{debug};
  $in{con_obj_b}->execute_start(command=>'cf giveback -f');
  $ct = 0;
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
    print "STATE_A after GB=$state_a\n" unless !$in{debug};
    my $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_B after GB=$state_b\n" unless !$in{debug};
    if ($ct >= 60) {
      $err="Error: Giveback was not complete after waiting $ct sec: STAET_A=$state_a; STATE_B=$state_b";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_b =~ /CLI/ and $state_a =~ /PASSWORD|CLI/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless !$in{debug};
    }
  }

  $class->_restore_2_cli($in{con_obj_b}, $in{tran_obj_a});
  return 0;
} ## end of cfo_on_halt()


#=============================================================
## method: takeover on panic
#=============================================================
sub cfo_on_panic {
  my $class = shift;
  my %in=validate(
        @_, {
          con_obj_a=>{type=>OBJECT},
          tran_obj_a=>{type=>OBJECT},
          con_obj_b=>{type=>OBJECT},
          tran_obj_b=>{type=>OBJECT},
          debug=>{type=>SCALAR,
                optional=>1,
                default=>1,
                regex=>qr/^(1)|(0)$/,
               },
            },
          );

  my ($ct, $state_a, $state_b, $err, $out);

  $in{con_obj_a}->execute_start(command=>'priv set -q test');
  print "issue panic cmd on filer A" unless !$in{debug};
  $in{con_obj_a}->execute_start(command=>'priv set -q test;panic');
  $in{tran_obj_a}->set(timeout=>600);
  print "pausing 180 sec after panic cmd...\n" unless !$in{debug};
  sleep 180;
  print "getting transit state after pausing 120 sec following panic cmd...\n" unless !$in{debug};
  $state_a = $in{tran_obj_a}->get_state(timeout=>600);
  print "STATE_A after panic=$state_a\n" unless !$in{debug};

  $ct = 0;
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
    print "STATE_A after reboot=$state_a\n" unless !$in{debug};
    my $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_B after reboot=$state_b\n" unless !$in{debug};
    if ($ct >= 900) {
      $err="Error: Giveback was not complete after waiting $ct sec: STAET_A=$state_a; STATE_B=$state_b";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_b =~ /CLI|PASSWORD/ and $state_a =~ /TAKEN_OVER_WAITING_ONTAP/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless !$in{debug};
    }
  }
  if ($state_b =~ /PASSWORD/) {
    $state_b = $in{tran_obj_b}->change_state(to=>'CLI');
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_B after change_state=$state_b\n" unless !$in{debug};
  }

# print "checking ic status after TO\n" unless !$in{debug};
# $class->check_ic_status(con_obj=>$in{con_obj_b}, stage=>'after TO');

  print "Issue 'cf giveback -f' on filer B\n" unless !$in{debug};
  $in{con_obj_b}->execute_start(command=>'cf giveback -f');
  $ct = 0;
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_A after GB=$state_a\n" unless !$in{debug};
    print "STATE_B after GB=$state_b\n" unless !$in{debug};
    if ($ct >= 120) {
      $err="Error: Giveback was not complete after waiting $ct sec: STAET_A=$state_a; STATE_B=$state_b";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_b =~ /CLI/ and $state_a =~ /PASSWORD|CLI/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless !$in{debug};
    }
  }

  $class->_restore_2_cli($in{con_obj_b}, $in{tran_obj_a});
  return 0;
} ## end of cfo_on_panic()

#=============================================================
## method: takeover on power cycle
#=============================================================
sub cfo_on_powercmd {
  my $class = shift;
  my %in=validate(
        @_, {
          con_obj_a=>{type=>OBJECT},
          tran_obj_a=>{type=>OBJECT},
          con_obj_b=>{type=>OBJECT},
          tran_obj_b=>{type=>OBJECT},
          debug=>{type=>SCALAR,
                optional=>1,
                default=>1,
                regex=>qr/^(1)|(0)$/,
               },
           #powercmd=>{type=>SCALAR},
		   rlm_con_a=>{type=>OBJECT},
		   },
          );

  my ($ct, $state_a, $state_b, $err, $out);

  print "Starting to perform power cycle\n" unless !$in{debug};
  #$out =`$in{powercmd}`;
  $in{rlm_con_a}->execute(command=>'system power cycle', timeout=>900, prompts=>['This will cause a dirty shutdown of your appliance']);
  $in{rlm_con_a}->execute_start(command=>'y');
  print "power cycle is done\n" unless !$in{debug};
  #if ($out =~ /Rlm unknown timeout|Unable to login to RLM/i) {
  #  $err = "Rlm did not seem to setup for power cycle filer. Test aborted.\n$out\n";
  #  $class->{err}=$err;
  #  confess($err);
  #} 
  
  $in{tran_obj_a}->set(timeout=>600);
  print "pausing 180 sec after powercmd...\n" unless !$in{debug};
  sleep 180;
  print "getting transit state after pausing 120 sec following powercmd...\n" unless !$in{debug};
  $state_a = $in{tran_obj_a}->get_state(timeout=>600);
  print "STATE_A after panic=$state_a\n" unless !$in{debug};

  $ct = 0;
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_A after reboot=$state_a\n" unless !$in{debug};
    print "STATE_B after reboot=$state_b\n" unless !$in{debug};
    if ($ct >= 900) {
      $err="Error: Giveback was not complete after waiting $ct sec: STAET_A=$state_a; STATE_B=$state_b";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_b =~ /CLI|PASSWORD/ and $state_a =~ /TAKEN_OVER_WAITING_ONTAP/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless !$in{debug};
    }
  }
  if ($state_b =~ /PASSWORD/) {
    $state_b = $in{tran_obj_b}->change_state(to=>'CLI');
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_B after change_state=$state_b\n" unless !$in{debug};
  }

# print "checking ic status after TO\n" unless !$in{debug};
# $class->check_ic_status(con_obj=>$in{con_obj_b}, stage=>'after TO');

  print "Issue 'cf giveback -f' on filer B\n" unless !$in{debug};
  $in{con_obj_b}->execute_start(command=>'cf giveback -f');
  $ct = 0;
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
    print "STATE_A after GB=$state_a\n" unless !$in{debug};
    my $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_B after GB=$state_b\n" unless !$in{debug};
    if ($ct >= 120) {
      $err="Error: Giveback was not complete after waiting $ct sec";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_b =~ /CLI/ and $state_a =~ /PASSWORD|CLI/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless !$in{debug};
    }
  }

  $class->_restore_2_cli($in{con_obj_b}, $in{tran_obj_a});
  return 0;
} ## end of cfo_on_powercmd()

#=============================================================
## method: takeover on system reset
#=============================================================
sub cfo_on_reset {
  my $class = shift;
  my %in=validate(
        @_, {
          con_obj_a=>{type=>OBJECT},
          tran_obj_a=>{type=>OBJECT},
          con_obj_b=>{type=>OBJECT},
          tran_obj_b=>{type=>OBJECT},
          debug=>{type=>SCALAR,
                optional=>1,
                default=>1,
                regex=>qr/^(1)|(0)$/,
               },
		   rlm_con_a=>{type=>OBJECT},
		   },
          );

  my ($ct, $state_a, $state_b, $err, $out);

  print "Starting to perform system reset\n" unless !$in{debug};
  #$out =`$in{powercmd}`;
  $in{rlm_con_a}->execute(command=>'system reset', timeout=>900, prompts=>['This will cause a dirty shutdown of your appliance']);
  $in{rlm_con_a}->execute_start(command=>'y');
  print "system reset is done\n" unless !$in{debug};
  #if ($out =~ /Rlm unknown timeout|Unable to login to RLM/i) {
  #  $err = "Rlm did not seem to setup for power cycle filer. Test aborted.\n$out\n";
  #  $class->{err}=$err;
  #  confess($err);
  #} 
  
  $in{tran_obj_a}->set(timeout=>600);
  print "pausing 180 sec after reset...\n" unless !$in{debug};
  sleep 180;
  print "getting transit state after pausing 120 sec following reset...\n" unless !$in{debug};
  $state_a = $in{tran_obj_a}->get_state(timeout=>600);
  print "STATE_A after panic=$state_a\n" unless !$in{debug};

  $ct = 0;
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_A after reboot=$state_a\n" unless !$in{debug};
    print "STATE_B after reboot=$state_b\n" unless !$in{debug};
    if ($ct >= 900) {
      $err="Error: Giveback was not complete after waiting $ct sec: STAET_A=$state_a; STATE_B=$state_b";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_b =~ /CLI|PASSWORD/ and $state_a =~ /TAKEN_OVER_WAITING_ONTAP/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless !$in{debug};
    }
  }
  if ($state_b =~ /PASSWORD/) {
    $state_b = $in{tran_obj_b}->change_state(to=>'CLI');
    $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_B after change_state=$state_b\n" unless !$in{debug};
  }

# print "checking ic status after TO\n" unless !$in{debug};
# $class->check_ic_status(con_obj=>$in{con_obj_b}, stage=>'after TO');

  print "Issue 'cf giveback -f' on filer B\n" unless !$in{debug};
  $in{con_obj_b}->execute_start(command=>'cf giveback -f');
  $ct = 0;
  while (1) {
    $state_a = $in{tran_obj_a}->get_state(timeout=>600);
    print "STATE_A after GB=$state_a\n" unless !$in{debug};
    my $state_b = $in{tran_obj_b}->get_state(timeout=>600);
    print "STATE_B after GB=$state_b\n" unless !$in{debug};
    if ($ct >= 120) {
      $err="Error: Giveback was not complete after waiting $ct sec";
      $class->{err}=$err;
      confess($err);
    }
    if ($state_b =~ /CLI/ and $state_a =~ /PASSWORD|CLI/) {
      last;
    } else {
      sleep 10;
      $ct += 10;
      print "Pausing 10 sec before checking again. Elapsed time=$ct...\n" unless !$in{debug};
    }
  }

  $class->_restore_2_cli($in{con_obj_b}, $in{tran_obj_a});
  return 0;
} ## end of cfo_on_reset()

#=============================================================
## method: return latest err msg saved on object
#=============================================================
sub show_err {
  my $class = shift;
  return $class->{err};
}

#######################################################
#=============================================================
## method: check_ic_status
#=============================================================
sub check_ic_status {
  my $class = shift;
  my %in=validate(
        @_, {
          con_obj=>{type=>OBJECT},
          stage=>{type=>SCALAR,
                  regexp=>/^(before TO|after TO)$/,
                 },
          debug=>{type=>SCALAR,
                optional=>1,
                default=>1,
                regex=>qr/^(1)|(0)$/,
               },
             },
          );
  my ($out, $exp);
  if ($in{stage} =~ /before/) {
    $exp='Link 0:.+?up.+IC.+connection.+?up.+NVLOG.+?up';
  } else {
    $exp='Link 0:.+?up.+IC.+connection.+?up.+NVLOG.+?down';
  }

  $in{con_obj}->execute(command=>'priv set -q test');
  $out = $in{con_obj}->execute(command=>'ic status');
  if ($out !~ /$exp/igms) {
    return "Error: ic statu is not expected \($exp\): $out.";
  } else {
    print "Note: ic status is expected \($exp\): $out.\n" unless !$in{debug};
    return 0;
  }
}

#=============================================================
## method: check_cf_status
#=============================================================
sub check_cf_status {
  my $class = shift;
  my %in=validate(
        @_, {
          con_obj=>{type=>OBJECT},
          debug=>{type=>SCALAR,
                optional=>1,
                default=>1,
                regex=>qr/^(1)|(0)$/,
               },
             },
          );
  my ($out, $exp);
  $exp='\benabled\b';
  $out = $in{con_obj}->execute(command=>'cf status');
  if ($out !~ /$exp/igms) {
    return "Error: cf status is not expected \($exp\):\n$out.";
  } else {
    print "Note: cf status is expected \($exp\):\n$out.\n" unless !$in{debug};
    return 0;
  }
}
#=============================================
sub _restore_2_cli {
  my ($class, $con_obj, $tran_obj) = @_;
  my ($state_b, $out);
  $state_b = $tran_obj->change_state(to=>'CLI');
  $state_b = $tran_obj->get_state(timeout=>600);
  if ($state_b ne 'CLI') {
    confess("unable to get back to CLI state");
  }
  print "STATE_B after change_state=$state_b is verified.\n";

  print "Check cf status to ensure cf is enabled...";
  $out =  $class->check_cf_status(con_obj=>$con_obj);
  foreach (1..3) {
    if ($out) {
      print "pausing 10 sec before checking cf status again...";
      sleep 10;
      $out =  $class->check_cf_status(con_obj=>$con_obj);
    } else {
      last;
    }
  }
  if ($out) {
    confess("unable to get back to cf enabled status after TOGB and 3 trials.");
  }
} ## end of _restore_2_cli

1,
