#!/usr/bin/perl

## @summary Library of routines used in WFG and MBX clusters.
##
## @description
## This is a library to check the Waiting For Giveback prompt and check the
## options FromBootMenu,sanityCheck,startClientStress,startHammerStress,
## createVol. with various kinds of appliances (filers);

## @synopsis
## use ha;
##
## sanityCheck();
## startClientStress();
## startHammerStress();
## createVol();
## doFilersUp();
## isBrSetup();
## isRrSetup();
## getTimeout();
## cleanUp();
## filerPanic();
## clearError()
## @usage
##
## <HR><center><b>sanityCheck</b></center>
##
## <tt><b>sanityCheck($HostFilerA, $HostFilerB, $ConnFilerA,
## $ConnFilerB)</b></tt>
## <b>$HostFilerA, $HostFilerB, $ConnFilerA,$ConnFilerB</b> - host records or
## connection records for two filers which are a CFO pair.
##
## <HR><center><b>doFilersUp</b></center>
##
## <tt><b>doFilersUp($HostFilerA, $HostFilerB, $ConnFilerA,
## $ConnFilerB,$TIMEOUT)</b></tt>
## <b>$HostFilerA, $HostFilerB, $ConnFilerA,$ConnFilerB</b> - host records or
## connection records for two filers which are a CFO pair.
##
## <b>$timeout</b> - how many seconds (or an absolute time_t value) to
## wait for the wanted state to be reached before giving up with an
## error.  The default is -1 (infinite).
##
## <HR><center><b> startClientStress</b></center>
##
## <tt><b>startClientStress($HostFiler, $FILER_VOL, $FILERA,
##                          $DISK_COUNT, $CLIENT_FILE, $VolumeFlag) </b></tt>
## <b>$HostFiler, $FILER_VOL, $FILER</b> -
## host records or   connection records for two filers which are a CFO pair.
## <b>$DISK_COUNT</b> - 3 Disk is required for Nfs Stress.
## <b>$CLIENT_FILE</b> - the path of the n_dct.thpl file which generates
## the nfs stress
##	<b>$VolumeFlag</b> - reference variable to store new volume created
##
## <HR><><b>startHammerStress</b>
##
## <tt><b>startHammerStress($HostFiler, $FILER_VOL,
##									 $DISK_COUNT, $VolumeFlag)</b></tt>
##
## <b>$HostFiler, $FILER_VOL, $DISK_COUNT</b> - Host record
## <b>$FILER_VOL</b> - Volume used in filer.
## <b>$DISK_COUNT</b> - 3 Disk is required for Nfs Stress.
## <b>$VolumeFlag</b> - reference variable to store new volume created
##
## <HR><b>createVol</b>
##
## <tt><b>createVol($HostFiler, $volumeName, $DISK_COUNT, $VolumeFlag)</b></tt>
## <b>$HostFiler, $volumeName, $AGGR, $DISK_COUNT</b>- host record for filer.
## <b>$DISK_COUNT</b> - 3 Disk is required for Nfs Stress.
## <b>$volumeName</b>- volume name to be created.
## <b>$VolumeFlag</b> - reference variable to store new volume created
##
## <HR><b>setPowerState</b>
##
## <tt><b>setPowerState($connection, $state, $timeout)</b></tt>
## <b>$connection</b>- RLM connection for the filer.
## <b>$state</b> - power state("on", "off", "cycle").
## <b>$timeout</b>- Timeout.
## <HR><b>isBrSetup</b>
##
## <tt><b>isBrSetup($connection)</b></tt>
## <b>$connection</b>- Check connection for FIlERA and FILERB
##
## <HR><b>isRrSetup</b>
##
## <tt><b>isRrSetup($connection)</b></tt>
## <b>$connection</b>- Check connection for FIlERA and FILERB
##
## <HR><b>getTimeout</b>
##
## <tt><b>getTimeout($connection,$timeout)</b></tt>
## <b>$connection</b>- Check connection for FIlERA and FILERB
## <b>$timeout</b>- Check for IC/BR set up Timeout
##
## <HR><b>cleanUp($CLEANUP, $STRESS_FILER, $STRESS_CLIENT, $FILERA_VOL,
## $FILERB_VOL,$HostFilerA, $HostFilerB, $ConnFilerA, $ConnFilerB, $TIMEOUT,
## $FILERA, $FILERB, $ntest_pid_filerA, $ntest_pid_filerB,
##	$volume_flag_filerA, $volume_flag_filerB)</b>
##
## <b>$CLEANUP</b>- Cleanup and make filers up.
## <b>$STRESS_FILER</b>- Start Hammer on both the filers if STRESS_FILER enable.
## <b>$STRESS_CLIENT</b>-Start the CLIENT stress on on both the Filers if
## STRESS_CLIENT enable.
## <b>$FILERA_VOL</b>-Volume used for FILERA.
## <b>$FILERB_VOL</b>-Volume used for FILERB.
## <b>$HostFilerA, $HostFilerB, $ConnFilerA,$ConnFilerB</b> - host records or
## connection records for two filers which are a CFO pair.
## <b>$TIMEOUT</b>- Timeout
## <b>$FILERA</b>- FilerA used.
## <b>$FILERB</b>- FilerB used.
## <b>$ntest_pid_filerA</b>- process id for FilerA
## <b>$ntest_pid_filerB</b>-  process id for FilerB
## <b>$volume_flag_filerA</b>- variable to check new volume created for FILERA
## <b>$volume_flag_filerB</b>- variable to check new volume created for FILERB
##
## <HR><b>filerPanic</b>
## <tt><b>filerPanic($connection)</b></tt>
## <b>$connection</b>- filer connection object
##
## <HR><b>clearError</b>
## <tt><b>clearError($connection)</b></tt>
## <b>$connection</b>- filer connection object
##
## <HR><b>installAndDownloadBuild</b>
##
## <tt><b>installAndDownloadBuild($ref_connections, $build, $timeout)</b></tt>
## <b>$ref_connections</b>- Reference to array of connections.
## <b>$build</b> - New build to install.
## <b>$timeout</b>- Timeout.
##
package platval::Interconnect::ha;
use strict;
use vars qw(@ISA @EXPORT);
require Exporter;
@ISA = ('Exporter');
@EXPORT =('sanityCheck', 'startClientStress', 'startHammerStress', 'doFilersUp',
           'cleanUp', 'getTimeout', 'isBrSetup', 'isRrSetup', 'setPowerState','filerPanic',
           'createVol', 'installAndDownloadBuild','clearError');

use Tharn qw(say script_result host logcomment logresult param);
use boot;
use cluster;
use filer;
use RaidSqa;

use constant {
   AGGR_FLEX         => 1,
   UNSYNC_LOG_CNT    => 5,
   IC_MAX_TIMEOUT    => 200,
   BR_MAX_TIMEOUT    => 600,
   RR_MAX_TIMEOUT    => 600,
   DOWNLOAD_TIMEOUT  => 3600,
   YES               => "y\n",
   POWER_OFF         => "system power off",
   POWER_ON          => "system power on",
   POWER_STATUS      => "system power status",
   DOWNLOAD_PROMPT   =>
   "You can cancel this operation by hitting Ctrl-C in the next 6 seconds",
   PROMPT_SYS_OFF    =>
   "This will cause a dirty shutdown of your appliance.  Continue\\? \\[y/n\\]",
};


#-------------------------------------------------------------------------------
#  Check for following Requirements:->
#  1.check for cf enable, if disable make it enable.
#  2.check for Interconnect is connected or not. If not connected fatal error.
#  3.check weather cluster is licensed or not on filerB.
#     If not licensed fatal error.
#  4.check weather cluster is licensed or not on filerA.
#     If not licensed fatal error.
#  5.check weather version mismatch or not. If mismatch fatal error.
#  6.check for unsynchronized log.
#  7.once unsynchronized log is comes waiting for 180 sec and again checking
#  8.unsynchronized log is present or not, if present's log fatal error.
#  9.Disable auto giveback on both the filers.
#-------------------------------------------------------------------------------

sub sanityCheck($$$$)
{
   my ($HostFilerA, $HostFilerB, $ConnFilerA, $ConnFilerB )  = @_;
   my $TIMEOUT = BR_MAX_TIMEOUT;
   ## @step if filerA or filerB already in takeover,loader prompt make both
   ## filers up.
   doFilersUp($HostFilerA, $HostFilerB, $ConnFilerA, $ConnFilerB,$TIMEOUT);

   ## Moved the license check to doFilersUp() 
   #if(!filer_cluster_is_licensed($ConnFilerA) or
   #!filer_cluster_is_licensed($ConnFilerB))
   #{
   #        logresult('fatal', "Cluster is not licensed, Please license and reboot
   #            the filer and run the script again");
   #        goto fail;
   # }

   my $icstatus;
   ## @step check for Interconnect is connected or not.
  (undef, undef,$icstatus) = say($ConnFilerA, "ic status");

  if (($icstatus =~ /not connected/i) or
      ($icstatus =~ /waiting for connected/i))
   {
      logresult('fatal',"Interconnect link is down");
      goto fail;
   }

   my $afterA;
   (undef, undef, $afterA) = say ($ConnFilerA, "cf status");

   ## @step check weather version mimatch or not.
   if ($afterA =~ /version mismatch/i )
   {
      logresult('fatal', "Takeover not possible due to ontap version mismatch,
               please correct the version and re-run the script");
      goto fail;
   }
   ##	@step check for partner mailbox disks connectivity.
   if ($afterA =~ /partner mailbox disks not accessible or invalid/i)
   {
      logresult('fatal',"check for disk connectivity and re-run the \
      	script again");
      goto fail;
   }


   ## @step check for unsynchronized log.
   if (($afterA =~ /unsynchronized log/i) || ($afterA =~ /interconnect error/i)) 
   {
      # once unsynchronized log encounters  5 times sleep for 60 sec and check
      # still unsynchronized log is present or not,
      # if present's log fatal error.
      logcomment("unsynchronized log");
      my $unsync_logs=0;
      my $after;
      for (my $cnt=0;$cnt<UNSYNC_LOG_CNT;++$cnt)
      {
         sleep 60;
         (undef, undef, $after) = say ($ConnFilerA, "cf status");
         if (($after =~ /unsynchronized log/i) || ($afterA =~ /interconnect error/i)) 
         {
            $unsync_logs++;
         }
         else
         {
            # if unsynchronized log is not encounters at any loop iteration
            # coming out of loop.
            last;
         }
      }

      if($unsync_logs >= UNSYNC_LOG_CNT)
      {
         # After 5 min still unsynchronized log present reseting link and
         # checking once again status if log present through fatal error.
         say ($ConnFilerA, "ic reset link");
         sleep 10;
         (undef, undef, $after) = say ($ConnFilerA, "cf status");

         if (($after =~ /unsynchronized log/i) || ($afterA =~ /interconnect error/i)) 
         {
            logresult('fatal', "Takeover disabled because of unsynchronized \
            log");
            goto fail;
         }
      }
   }

   # Disable auto giveback on both the filers.
   filer_options_set($ConnFilerA, 'cf.giveback.auto.enable', 'off');
   filer_options_set($ConnFilerB, 'cf.giveback.auto.enable', 'off');
   return 1;

   fail:
   return 0;
}

#-------------------------------------------------------------------------------
#   Start client stress on Filer if FILERA_VOL exist or created successfully.
#-------------------------------------------------------------------------------
sub startClientStress($$$$$$$)
{
   my ($HostFiler, $FilerVol, $Filer, $DiskCount, $ClientFile, $ChildPid,
		$VolumeFlag) = @_;
   #----------------------------------------------------------------------------
   # Start NFS stress on Filer if FILERA_VOL exist or created successfully.
   #----------------------------------------------------------------------------
   my $result = 0;
   my $pid = -1;
   if(createVol($HostFiler, $FilerVol, $DiskCount, $VolumeFlag))   {
      #-------------------------------------------------------------------------
      # Do fork and execute the client file on clild process
      #-------------------------------------------------------------------------
      $pid = fork();
      if ($pid == 0) {
         ## Execute the NFS stress on child process.
         exec("ntest -hw -noconsole FILER=$Filer"
            . " FILER_vol=$FilerVol $ClientFile");
          logcomment ("Unable to start the Client stress on Filer : $Filer");
          exit(0);
      }
      elsif ($pid)  {
         ## Make a sleep on parent process so child process wake up.
         sleep(5);
      }
   }
   else {
      logcomment ("Unable to create volume");
   }

   ${$ChildPid} = $pid;
   return (($pid == -1) ? 0 :1);
}


#-------------------------------------------------------------------------------
# Start Hammer on FilerA if FILERA_VOL exist or created.
#-------------------------------------------------------------------------------
sub startHammerStress($$$$)
{
   my ($HostFiler, $FilerVol, $DiskCount,$VolumeFlag)  = @_;
   #----------------------------------------------------------------------------
   # Create a volume in filer if not exist and start the hammer stress on it.
   #----------------------------------------------------------------------------
   if(createVol($HostFiler, $FilerVol, $DiskCount, $VolumeFlag))   {
      ## Start Hammer on the volume.
      RaidSqa_hammer_start(host => $HostFiler, filer_vol => $FilerVol);
      return 1;
   }
   else {
      logcomment ("Unable to create volume");
   }
   return 0;
}

#-------------------------------------------------------------------------------
#  Create the volume if the volume "$volumeName" not exist in the filer
#  Return 1 if volume already exist or successfully created, else return 0.
#-------------------------------------------------------------------------------
sub createVol($$$$)
{
   my ($HostFiler, $VolumeName, $DiskCount, $VolumeFlag) = @_;

   ## Check volumeName already exist.
   ## If not exist create a flexible volume of same name.
   if (!RaidSqa_check_aggr_exist_anytype(host      => $HostFiler,
                                         filer_vol => $VolumeName)
      ) {
      logcomment("Volume : $VolumeName not exist. So creating new volume.");
      if (!RaidSqa_aggr_create(host       => $HostFiler,
                               filer_vol  => $VolumeName,
                               aggr_type  => AGGR_FLEX,
                               disk_count => $DiskCount)
         ) {
         logresult('FATAL',"$HostFiler : volume $VolumeName creation failed");
         return 0;
      }
      else {
         logcomment("Volume : $VolumeName creation sucessfull.");
         # $VolumeFlag is set to 1, if new volume is created
         ${ $VolumeFlag } = 1;
      }  ## end if (!RaidSqa_aggr_create...
   }  ## end if (RaidSqa_check_aggr_exist_anytype...

   return 1;
} ## end sub checkVol()

#-------------------------------------------------------------------------------
# If filerA or filerB already in takeover,loader prompt make both filers up.
#-------------------------------------------------------------------------------
sub doFilersUp($$$$$)
{
   my ($HostFilerA, $HostFilerB, $ConnFilerA, $ConnFilerB, $TIMEOUT)  = @_;
   $TIMEOUT = BR_MAX_TIMEOUT;
   my $status_a = get_cf_state($HostFilerA);
   my $status_b = get_cf_state($HostFilerB);

   logcomment("The status of FILERA: $status_a and FILERB : $status_b");
   
   ## If license doesnot exists, add license and reboot the filer
   if ( "NO_LICENSE" eq $status_a || "NO_LICENSE" eq $status_b ) {
       say($ConnFilerA,"license add KDQLGBN",'-eval_error');
       say($ConnFilerB,"license add KDQLGBN",'-eval_error');
       logcomment("Reboot Filer A");
       transit_to($ConnFilerA, 'FIRMWARE' , timeout=>$TIMEOUT) or
         die " transit to FIRMWARE state on FilerA failed...";
       transit_to($ConnFilerA, 'UP', timeout=>$TIMEOUT) or
         die " transit to UP state on FilerA failed...";

       logcomment("Reboot Filer B");
       transit_to($ConnFilerB, 'FIRMWARE' , timeout=>$TIMEOUT) or
         die " transit to FIRMWARE state on FilerB failed...";
       transit_to($ConnFilerB, 'UP', timeout=>$TIMEOUT) or
         die " transit to UP state on FilerB failed...";
   }
   elsif ( "NON_HAMODE" eq $status_a && "NON_HAMODE" eq $status_b) {
       set_ha_mode($HostFilerA,$HostFilerB ) or die "Failed to set filer to HA mode" ;
   }
   ## if filerA is in takeover mode, giveback and wait for SF_UP.
   elsif ("TAKEOVER" eq $status_a)
   {
      cf_giveback($ConnFilerA);
      wait_fw_state($ConnFilerA, "SF_UP", $TIMEOUT) or
      die " waiting to SF_UP state on FilerA  failed...";
   }
   ## if filerB is takeover mode, giveback and wait for SF_UP.
   elsif ("TAKEOVER" eq $status_b)
   {
      cf_giveback($ConnFilerB);
      wait_fw_state($ConnFilerB, "SF_UP", $TIMEOUT) or
      die " waiting to SF_UP state on FilerB  failed...";
   }
   ## if filerA is dead or unreach mode, making filerA to up state.
   elsif("DEAD" eq $status_a or "UNREACH" eq $status_a)
   {
      logcomment(" dead or unreached ");
      my $boot_state = boot_state($HostFilerA, timeout=>$TIMEOUT);
      unless ($boot_state eq "TAKEN_OVER_WAITING_ONTAP" or
      $boot_state eq "TAKEN_OVER_QUERY" )
      {
         transit_to($ConnFilerA, 'UP', timeout=>$TIMEOUT) or
         die " transit to UP state on FilerA failed...";
      }
   }
   ## if filerB is dead or unreach mode, making filerB to up state.
   elsif("DEAD" eq $status_b or "UNREACH" eq $status_b)
   {
      logcomment(" dead or unreached ");
      my $boot_state = boot_state($HostFilerB, timeout=>$TIMEOUT);
      unless ($boot_state eq "TAKEN_OVER_WAITING_ONTAP" or $boot_state eq
      "TAKEN_OVER_QUERY")
      {
         transit_to($ConnFilerB, 'UP', timeout=>$TIMEOUT) or
          die " transit to UP state on FilerA failed...";
      }
   }

   cf_enable($HostFilerA) or die $@;
   cf_enable($HostFilerB) or die $@;
   wait_state($HostFilerA, "UP", $TIMEOUT) or
   die "waiting to UP state of FilerA failed.";
   wait_state($HostFilerB, "UP", $TIMEOUT) or
   die "waiting to UP state of FilerB failed.";
   #cf_enable($HostFilerA) or die $@;
   #cf_enable($HostFilerB) or die $@;
}

#-------------------------------------------------------------------------------
# Check BR setup or IC setup
# Return 1 if BR setup else return 0.
#-------------------------------------------------------------------------------
sub isBrSetup($)  {
   my $connection = shift;
   my @br_builds = ("mainngN", "8\\.\\d+", "R8.0RCxN", "R8.0xN", "RboilermakerN");
   my(undef, undef, $after) = say ($connection, "version");
   logcomment("Value in After ===> $after");
   foreach (@br_builds) {
   logcomment("Version type -----------> $_");
      if ($after =~ /$_/) {
         return 1;
      }
   }
   return 0;
}

#-------------------------------------------------------------------------------
# Check RR setup
# Return 1 if RR setup else return 0.
#-------------------------------------------------------------------------------
sub isRrSetup($)  {
   my $connection = shift;

   my @rr_builds = ("rr-platstor.*","fsb.*","_rr_","mainN.*");
   my(undef, undef, $after) = say ($connection, "version");
   foreach (@rr_builds) {
      if ($after =~ /$_/) {
         return 1;
      }
   }
   return 0;
}

#-------------------------------------------------------------------------------
# Get Maximum timeout for any steps in IC/BR/RR setups if user not specified
#-------------------------------------------------------------------------------
sub getTimeout($$)   {
   my ($connection, $timeout) = @_;
   unless(${$timeout})  {
      if(isBrSetup($connection) ) {
        ${$timeout} = BR_MAX_TIMEOUT;
      }elsif(isRrSetup($connection)) {
        ${$timeout} = RR_MAX_TIMEOUT;
      }else {
        ${$timeout} = IC_MAX_TIMEOUT;
      }
   }
}

#-------------------------------------------------------------------------------
# Set Power State("on", "off, "cycle")
# Return 1 if state set successfully, else Return 0 as fail.
#-------------------------------------------------------------------------------
sub setPowerState($$$)  {
   my ($connection, $state, $timeout) = @_;
   my @arr_states = ("on", "off", "cycle");
   my $status = 0;

   if (exists {map { $_ => 1 } @arr_states}->{$state})
   {
      # Give the system power command
      if($state eq "on")  {
         say($connection, "system power $state", '-timeout', $timeout);
      }
      else  {
         say($connection, "system power ".$state, '-prompt', PROMPT_SYS_OFF,
                                                  '-timeout', $timeout);
         say($connection, YES, '-timeout', $timeout);
      }

      $status = 1;
      # Check the power status in a loop of max 10.
      if ($state ne "cycle")  {
         my $count=0;
         for(; $count < 10; ++$count) {
            my(undef, undef, $after) = say($connection, POWER_STATUS,
                                          '-timeout', $timeout);
            # Check "Turned on by Agent: yes" for on and
            # Check "Turned on by Agent: no"  for off.
            if($state eq "on")  {
               if($after =~ /Turned on by Agent: yes/){
                  last;
               }
            }
            else  {
               if($after =~ /Turned on by Agent: no/) {
                  last;
               }
            }
            sleep(10);
         }
         $status = (($count < 10) ? 1 : 0);
      }
      else  {
         # wait for the filer to start reboot.
         sleep(10);
      }
   }
   return $status;
}

#-------------------------------------------------------------------------------
# Install and download build to Filers
#-------------------------------------------------------------------------------
sub installAndDownloadBuild($$$) {
   my ($ref_connections, $build, $timeout) = @_;

   logcomment("do software install on Filers");
   foreach my $conn (@{$ref_connections}) {
      say($conn,     "software install $build -f",
         '-wait',    "software: installation of \\S* completed",
         '-timeout', $timeout);
   }

   logcomment("download the build image to Filers");
   # For save time we starts the downloads in all filers.
   for(my $count=1; $count <= @$ref_connections; ++$count) {
      my $connection = $ref_connections->[$count-1];
      say($connection, "download", '-wait', DOWNLOAD_PROMPT, '-noprompt',
                                   '-timeout', DOWNLOAD_TIMEOUT);
   }

   logcomment("Make sure in all filers have downloads completed");
   for(my $count=1; $count <= @$ref_connections; ++$count) {
      my $connection = $ref_connections->[$count-1];
      say($connection, "\r", '-timeout', DOWNLOAD_TIMEOUT);
   }
}

#-------------------------------------------------------------------------------
# CleanUp method
#-------------------------------------------------------------------------------
sub cleanUp($$$$$$$$$$$$$$$$)   {
   my ($CLEANUP, $STRESS_FILER, $STRESS_CLIENT, $FILERA_VOL, $FILERB_VOL,
       $HostFilerA, $HostFilerB, $ConnFilerA, $ConnFilerB, $TIMEOUT,
       $FILERA, $FILERB, $ntest_pid_filerA, $ntest_pid_filerB,
       $volume_flag_filerA, $volume_flag_filerB) = @_;

   if ($CLEANUP)  {
      logcomment(" CLEAN UP STARTED ");
      #-------------------------------------------------------------------------
      #  Do both FilerA and FilerB UP
      #-------------------------------------------------------------------------
      logcomment('Make both filers UP');
      doFilersUp($HostFilerA, $HostFilerB, $ConnFilerA, $ConnFilerB, $TIMEOUT);

      #-------------------------------------------------------------------------
      #  Stop Hammer Stress on both the filers
      #-------------------------------------------------------------------------
      if($STRESS_FILER)
      {
         logcomment("Abort hammer on Filer : $FILERA");
         RaidSqa_hammer_abort(host => $HostFilerA);

         logcomment("Abort hammer on Filer : $FILERB");
         RaidSqa_hammer_abort(host => $HostFilerB);
      }

      #-------------------------------------------------------------------------
      #  Stop Client Stress on both the filers
      #-------------------------------------------------------------------------
      if($STRESS_CLIENT)
      {
         ## Kill ntest child process for FilerA
         if($ntest_pid_filerA > 0) {
            unless( kill 'KILL', $ntest_pid_filerA )   {
               tstlog('-fatal', "Unable to kill child ntest process");
            }
         }

         ## Kill ntest child process for FilerB
         if($ntest_pid_filerB > 0) {
            unless( kill 'KILL', $ntest_pid_filerB )   {
               tstlog('-fatal', "Unable to kill child ntest process");
            }
         }
      }

      #-------------------------------------------------------------------------
      #  Destroy Volumes on both the filers
      #-------------------------------------------------------------------------
      ## Clean up the volumes if( $CLEANUP == 1)
      if( $CLEANUP == 1) {
        if ($volume_flag_filerA) {
            logcomment("Removing volume : $FILERA_VOL from $FILERA ");
            RaidSqa_aggr_remove(host      => $HostFilerA,
                                filer_vol => $FILERA_VOL,
                                aggr_type =>	AGGR_FLEX);

            logcomment("Zeroing the spare disk in Filer : $FILERA ");
            RaidSqa_disk_zero_spares(host=>$HostFilerA);
         }
         if ($volume_flag_filerB) {
            logcomment("Removing volume : $FILERB_VOL from $FILERB ");
            RaidSqa_aggr_remove(host      => $HostFilerB,
                                filer_vol => $FILERB_VOL,
                                aggr_type =>	AGGR_FLEX);

            logcomment("Zeroing the spare disk in Filer : $FILERB ");
            RaidSqa_disk_zero_spares(host=>$HostFilerB);
         }
      }  ## end if( $CLEANUP == 1)
      logcomment(" CLEAN UP FINISHED ");
   }  ## end if ($CLEANUP)
}

#-------------------------------------------------------------------------------
# Panic the filer
#-------------------------------------------------------------------------------
sub filerPanic($)
{
   my($fc) = shift;
   say ($fc, "priv set test; panic",-blind);
}
#-------------------------------------------------------------------------------
# clear all tp errors of a filer.
#-------------------------------------------------------------------------------
sub clearError($)
{
  my($fc) = shift;
  say ($fc, "priv set test; tp clear");
}

1;
