# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/main/ontap/main/cluster/ic/iwarp.pm
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.

## @summary
##  List of functions which will help rdmalib test automation easy.
##
## @description
##
##
## @synopsis
## ntest PREJOB=hw_assist_prejob.thpl FILERA=<filer1> FILERB=<filer2> option2.thpl
## @param FILERA Required: First filer in a clustered pair
## @param FILERB Required: Second filer in a clustered pair
## @dependencies Filer: FilerConfig=filerc
## @author sriramv@netapp.com


package iwarp;
use vars ('@ISA','@EXPORT','$NUMBER_OF_QP','$NUM_OF_LINKS');
require Exporter;
@ISA = ('Exporter');
@EXPORT =('get_connectx_net_info', 'get_filer_sysid', 'interconnect_type', 'verify_TenG_rnic',
	'get_iwarp_net_info','is_QP_given_state', 'get_QP_info', 
	'is_QP_listed', 'is_interconnect_UP', 'link_toggle_on', 'link_toggle_off', 'get_ic_config_info', 'disconnect_on', 'disconnect_off', 'all_QP_connected', 'verify_iwarp_link_UP', 'rdmalib_cleanup', 'is_ControlQP_with_validIP',
'number_of_link_UP', 'set_testpoint', 'verify_testpoint_hit','verify_connectx','get_ic_link_status_hash','get_rnic_qp_hash');



use Tharn qw(say script_result host logcomment logresult param);

use strict;
use cluster;
use cluster::Utils;
use boot;
use Sysconfig;
use SayWrap;

##The function gives filer sysid using sysconfig.
##There are so many functions which returns Filer Sysid, but most of them
##do return value with other parameters like partner's system id, serial
##number, etc. So I am designing function to return systemid assuming that
##this should report value on single node.

sub get_filer_sysid($)
{
	my ($conn) = @_;
	my (undef, undef, $out) = say($conn, "sysconfig", -swallow);
	$out =~ /.*System ID:\s+(\d+)/;
	return $1;
}


#This function returns interconnect type being used.
sub interconnect_type($)
{
	my ($conn) = @_;
	my (undef, undef, $out) = say($conn, "priv set -q test;ic config show", -swallow);
	$out =~ /Interconnect Type:\s+(\w+)/;
	return $1;

}

#This function will make sure 10G rnic is present in the system.


sub verify_TenG_rnic($)
{
	my ($conn) = @_;
	my $TenG_rnic_found = 0;
	
	my (undef, undef, $out) = say($conn, "priv set -q test;sysconfig -v", -swallow);
	chomp( my @outputlines = split('\n', $out));

	foreach my $outputline (@outputlines) {

		if ($outputline =~ /slot\s\d+:\sDual 10G Ethernet Controller/)
		{
			logcomment("10G RNIC found");
			$TenG_rnic_found = 1;
		}
	}

	return $TenG_rnic_found;
}
			
#This function will get RNIC interface and IP address info

sub get_iwarp_net_info($)
{

	my ($conn) = @_;
	my $iwar_interfaces;
	my @iwarp_ifs;
	my %interface_ip;
	my $failed_identify=0;

	my (undef, undef, $out) = say($conn, "priv set -q test;ic config show", -swallow);
	sleep 10;
	chomp( my @outputlines = split('\n', $out));

	for my $outputline (@outputlines) {

		if ($outputline =~ /BROADCAST/) {
			logcomment("The output line = $outputline");
			$outputline =~ /(\w+)/;
			if (!$1) {
				$failed_identify++;
			}
			push @iwarp_ifs, $1;
		}

		if ($outputline =~ /inet/) { 
			$outputline =~ /inet\s+(\d+.\d+.\d+.\d+)/;
			if (!$1) {
				$failed_identify++;
			}
			push @iwarp_ifs, $1;
		}

	}

	if ($failed_identify) {
		logcomment("Failed to identify iWARP interface and/or IP address");
	}

	logcomment("The iwarp interfaces : @iwarp_ifs");

	%interface_ip = @iwarp_ifs;

	return %interface_ip;

}


sub verify_iwarp_link_UP($$)
{
	my ($conn, $interface) = @_;
	my @outputlines;
	my @out;
	my @outputline;
    my $cnt;
	my (undef, undef, $out) = say($conn, "priv set -q test;ic status", -swallow);
	
	while($out =~ /Link\s+\d:\s+up/gsi){
	$cnt++;
	}
	
	return $cnt;
}
	

#Get all the QP info.


sub get_QP_info( $$ )
{
	my ($conn, $QP) = @_;
	my (undef, undef, $out) = say($conn, "priv set -q test;rnic -us", -swallow);
	my @rnic_array;

	chomp( my @outputlines = split('\n', $out));

	
	for my $outputline (@outputlines) {
		if ($outputline =~ /\d+\s+$QP\s+/) {
			if ($outputline =~ /Established/) {
				$outputline =~ /(\d+)\s+\w+\s+(\d+.\d+.\d+.\d+)/;
				push @rnic_array, $1;
				push @rnic_array, $2;
			}
		}
	}

	logcomment("RNIC array = @rnic_array");

	return @rnic_array;
}

#Verify QP is in given state.


sub is_QP_given_state($$$)
{
	my ($conn, $QP, $state) = @_;
	my (undef, undef, $out) = say($conn, "priv set -q test;rnic -us", -swallow);
	my @rnic_array;
	my $found_flag = 0;

	chomp( my @outputlines = split('\n', $out));

	
	for my $outputline (@outputlines) {
		if ($outputline =~ /\d+\s+$QP\s+/) {
			if ($outputline =~ /$state/) {
				$found_flag = 1;
			}
		}
	}

	return $found_flag;

}

sub is_ControlQP_with_validIP($$)
{
	my ($conn, $IP) = @_;
	my (undef, undef, $out) = say($conn, "priv set -q test;rnic -us", -swallow);

	my @rnic_array;
	my $found_flag = 0;

	chomp( my @outputlines = split('\n', $out));

	for my $outputline (@outputlines) {
                if ($outputline =~ /\d+\s+Control\s+/) {
                        if ($outputline =~ /$IP/) {
                                $found_flag = 1;
                        }
                }
        }

        return $found_flag;

}



#Verify given QP is listed in rnic -us

sub is_QP_listed($$)
{
	my ($conn, $QP) = @_;
	my (undef, undef, $out) = say($conn, "priv set -q test;rnic -us", -swallow);
	my @rnic_array;
	my $found_flag = 0;

	chomp( my @outputlines = split('\n', $out));

	
	for my $outputline (@outputlines) {
		if ($outputline =~ /\d+\s+$QP\s+/) {
			$found_flag = 1;
		}
	}

	return $found_flag;

}

#Verify atleast 1 link is up, interconnect RDMA status is up.


sub is_interconnect_UP($)
{
	my ($conn) = @_;
	my (undef, undef, $out) = say($conn, "priv set -q test;ic status", -swallow);
	my @rnic_array;
	my $link_up_flag = 0;
	my $rdma_up_flag = 0;

	chomp( my @outputlines = split('\n', $out));

	
	for my $outputline (@outputlines) {
		if ($outputline =~ /Link\s\d:\s+up/) {
			$link_up_flag = 1;
		}

		if ($outputline =~ /IC RDMA connection : up/) {
			$rdma_up_flag = 1;
		}
	}

	if ( $link_up_flag ne 1 || $rdma_up_flag ne 1 ) {
		return 0;
	}

	return 1;
}


#Get ic config info .

sub get_ic_config_info($)
{
	my ($conn) = @_;
	my (undef, undef, $out) = say($conn, "priv set -q test;ic config show", -swallow);
	my @rnic_array;
	my $local_system_id;
	my $partner_system_id;

	sleep 10;

	chomp( my @outputlines = split('\n', $out));

	
	for my $outputline (@outputlines) {
		if ($outputline =~ /Local System ID:/) {
			$outputline =~ /Local System ID:\s+(\d+)/;
			$local_system_id = $1;
		}

		if ($outputline =~ /Partner System ID:/) {
			$outputline =~ /Partner System ID:\s+(\d+)/;
			$partner_system_id = $1;
		}
	}

	return "$local_system_id $partner_system_id";

}

sub disconnect_on($)
{
	my ($conn) = @_;
	my (undef, undef, $out) = say($conn, "priv set -q test;rnic -ut disconnect_on -N Control", -swallow);

	sleep 30;

	(undef, undef, $out) = say($conn, "priv set -q test;rnic -ut disconnect_status", -swallow);

	if ($out !~ /test active/) {
		return 0;
	}
	return 1;
}

sub disconnect_off($)
{
	my ($conn) = @_;

	my (undef, undef, $out) = say($conn, "priv set -q test;rnic -ut disconnect_off -N Control", -swallow);

	sleep 30;

	(undef, undef, $out) = say($conn, "priv set -q test;rnic -ut disconnect_status -N Control", -swallow);

	if ($out !~ /test not active/) {
		return 0;
	}
	return 1;
}


sub link_toggle_on($)
{
	my ($conn) = @_;

	my (undef, undef, $out) = say($conn, "priv set -q test;rnic -ut link_toggle_on", -swallow);

	sleep 30;

	return 1;
}

sub link_toggle_off($)
{
	my ($conn) = @_;

	my (undef, undef, $out) = say($conn, "priv set -q test;rnic -ut link_toggle_off", -swallow);

	sleep 30;

	return 1;
}


sub all_QP_connected($)
{
	my ($conn) = @_;
	my (undef, undef, $out) = say($conn, "priv set -q test;rnic -us", -swallow);
	my @rnic_array;
	my $found_flag = 0;

	my $ret = is_QP_given_state($conn, "Control", "Established");
	if (!$ret) {
		return 0;
	}

	sleep 20;
	chomp( my @outputlines = split('\n', $out));

	
	for my $outputline (@outputlines) {
		logcomment("Each line of rnic -us is: $outputline");
		if ($outputline =~ /Estab/) {
			$found_flag = $found_flag + 1;
			logcomment("$outputline; number of QP: $found_flag");
		}
	}

	return $found_flag;

}

sub rdmalib_cleanup($$)
{

	my ($rshA,$rshB) = @_;

	logcomment("RDMAlib cleanup started");

	my (undef, undef, $out) = say($rshA, "priv set -q test;rnic -ut disconnect_off -N Control", -swallow);
	(undef, undef, $out) = say($rshB, "priv set -q test;rnic -ut disconnect_off -N Control", -swallow);
	(undef, undef, $out) = say($rshA, "priv set -q test;rnic -ut link_toggle_off", -swallow);
	(undef, undef, $out) = say($rshB, "priv set -q test;rnic -ut link_toggle_off", -swallow);
	#(undef, undef, $out) = say($rshA, "priv set -q test;hammer abort", -swallow);
	#(undef, undef, $out) = say($rshB, "priv set -q test;hammer abort", -swallow);

	logcomment("RDMAlib cleanup finished");
}

sub set_testpoint($$$)
{
	my ($conn, $module, $test_point) = @_;

	my (undef, undef, $out) = say($conn, "priv set -q test;tp error $module $test_point 1 1", -swallow);
	sleep 5;
	(undef, undef, $out) = say($conn, "priv set -q test;tp list", -swallow);

	chomp( my @outputlines = split('\n', $out));
	my $test_point_set = 0;
        for my $outputline (@outputlines) {
                if ($outputline =~ /Module $module id $test_point/) {
			$test_point_set = 1;
                }
        }

	return $test_point_set;
}


sub verify_testpoint_hit($$$)
{
	my ($conn, $module, $test_point) = @_;

	my ($undef, undef, $out) = say($conn, "priv set -q test;tp list", -swallow);

	chomp( my @outputlines = split('\n', $out));

	my $test_point_set = 1;
        for my $outputline (@outputlines) {
                if ($outputline =~ /Module $module id $test_point/) {
			$test_point_set = 0;
                }
        }

	return $test_point_set;
}

sub number_of_link_UP($)
{
	my ($conn) = @_;
	my (undef, undef, $out) = say($conn, "priv set -q test;ic status", -swallow);
	my $link_count = 0;

	chomp( my @outputlines = split('\n', $out));

	
	for my $outputline (@outputlines) {
		if ($outputline =~ /Link\s\d:\s+up/) {
			$link_count = $link_count + 1;
		}
	}
	return $link_count;
}

sub verify_connectx($)
{
	my ($conn) = @_;
	my @outputlines;
	my @out;
	my $outputline;
	my $connectx_found = 0;

	my (undef, undef, $out) = say($conn, "priv set -q test;sysconfig -v", -swallow);
	chomp( @outputlines = split('\n', $out));

	for my $outputline (@outputlines) {
		if ($outputline =~ /NVRAM8 ConnectX/) {
			$connectx_found = 1;
		}
	}
	return $connectx_found;
}

# This function returns rdma conenctions array from the "rnic -us" all 
# QPs  output

sub get_rnic_qp_hash($)
{
        my ($conn) = @_;
        my (undef, undef, $out) = say($conn, "priv set -q test; rnic -us", -swallow);
        my $rnic_qp_hash = {};
        my @array_lines = split(/\n/, $out);
        foreach (@array_lines) {
                trim($_);
                logcomment("\nline: $_\n");
                $rnic_qp_hash->{$2} = $1 if /^(\d+)\s+(\w+)\s+(\d+\.\d+\.\d+\.\d
+)\s+Established.*?/;
        }
        return $rnic_qp_hash;
}

# This function returns hash having link status info from "ic status" output
# Hash size gives no of links
# keys:Link 0,Link 1 values:up,down

sub get_ic_link_status_hash($)
{
        my ($conn) = @_;
        my (undef, undef, $out) = say($conn, "priv set -q test; ic status", -swallow);
        my $ic_link_status_hash = {};
        my @array_lines = split(/\n/, $out);
        #logcomment("\narray_lines: @array_lines\n");
        foreach (@array_lines) {
                trim($_);
                #logcomment("\nline: $_\n");

                if (/Link\s0:\s+up/) {
                        $ic_link_status_hash->{'Link 0'} = 'up';
                }
                if (/Link\s0:\s+down/) {
                        $ic_link_status_hash->{'Link 0'} = 'down';
                }
		if (/Link\s1:\s+up/) {
                        $ic_link_status_hash->{'Link 1'} = 'up';
                }
                if (/Link\s1:\s+down/) {
                        $ic_link_status_hash->{'Link 1'} = 'down';
                }

		#
		# IC RMDA connection status
		#

		if (/IC\sRDMA\sconnection\s:\sup/) {
			$ic_link_status_hash->{'RDMA connection'} = 'up';
		} else {
			$ic_link_status_hash->{'RDMA connection'} = 'down';
		}

		#
		# Local FM License
		#
		
		if (/Local\sFM\slicense\s:\senabled/) {
			$ic_link_status_hash->{'Local license'} = 'enabled';
		} else {
			$ic_link_status_hash->{'Local license'} = 'disabled';
		}

		#
		# Partner FM license
		#

		if (/Remote\sFM\slicense\s:\senabled/) {
			$ic_link_status_hash->{'Remote license'} = 'enabled';
		} else {
			$ic_link_status_hash->{'Remote license'} = 'disabled';
		}

        }
        logcomment("\n\n ****  $ic_link_status_hash->{'Link 0'} , $ic_link_status_hash->{'Link 1'}\n\n");
        return $ic_link_status_hash;
}

# This function returns trim of the string passed
sub trim($)
{
	my $string = shift;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}


sub get_connectx_net_info($)
{

	my ($conn) = @_;
	my $iwar_interfaces;
	my @iwarp_ifs;
	my %interface_ip;
	my $failed_identify=0;

	my (undef, undef, $out) = say($conn, "priv set test;ic config show", -swallow);
	sleep 10;
	chomp( my @outputlines = split('\n', $out));

	for my $outputline (@outputlines) {

		if ($outputline =~ /IP:/) { 
			$outputline =~ /(\w+):\s+IP:\s+(\d+.\d+.\d+.\d+)/;
			if (!$1) {
				$failed_identify++;
			}
			
			if (!$2) {
				$failed_identify++;
			}

			push @iwarp_ifs, $1;
			push @iwarp_ifs, $2;
		}

	}

	if ($failed_identify) {
		logcomment("Failed to identify CONNECTX interface and/or IP address");
	}

	logcomment("The connectx interfaces : @iwarp_ifs");

	%interface_ip = @iwarp_ifs;

	return %interface_ip;

}
