# $Id: //depot/prod/test/main/lib/platval/platsw/memerr.pm#1 $
#***********************************************************************************************************************************
## @summary 	Inject memory errors and validate the same.This library expected work only on BBA platforms.
##
## @description
## "BBA_MEI.pm": BBA_MEI library file implements two functions 
##    1. injectMemerr :- Inject CECC and UECC errors .This function will take 4 arguments.
##                        1.trigger 't' for Trgigger
##                                  'n' for NO -Trigger      
##                        2.Count  <number> : Number of errors to inject.
##                        3.Type    'c' for Correctable errors
##                                  'u' for Uncorrectable errors.
##                        4.Address <address> This is optional.If not specified it will inject at random address
##
##                        RETURN VALUE : DIMM number and Address
##
##   2.memerr_v       :- This function will return count of cecc errors in logger.
##                       RETURN VALUE: Returns hash contains count of cecc errors in logger,recent DIMM number, recent address,timestamp  
##
##
## @author              sbikkina@netapp.com
#***********************************************************************************************************************************
#!/usr/local/bin/perl

package BBA_MEI;

use warnings;
use NACL::APISet;
use NACL::C::Node;
use NACL::Transit;
use strict;

# Other module imports
use Tharn qw($Log);
use Data::Dumper;

###############################################################################
# Method:    new
# Objective: Constructor to create new Memory Error Injection object
# Details:   See POD documentation at the end of this file.
###############################################################################

sub new {

 my $class= shift;
 my $self = {
  _Api_set_obj=>shift,
  _Transit_obj=>shift,
  _platform=>shift,
  _total_dimms=>shift,
 };

 bless $self, $class;
 return $self;
}

###############################################################################
# Method       : injectMemerr
# Objective    : Method to inject CECC,UECC errors with and without trigger
# Return value : DIMM and Address values. DIMM and Address values are '-1' in case of unsuccesfull.
# Details      : See POD documentation at the end of this file.
###############################################################################

sub injectMemerr {

   my $self=shift;
   my ($cmd,$mei_output,$cecc_count,$Dimm,$raddr,$Api_obj,$Transit_obj,$filer_state);
   my ($trigger,$count,$type,$addr) = @_;

   $Log->enter();
   $addr = ($addr=~/^0x[0-9,A-F,a-f]/)? $addr:" " if(defined($addr));

   $Log->comment("Memmory Error injection on $self->{_platform} platform");
   $Log->comment("Error Injection Values trigger = $trigger Count = $count Type = $type Address = $addr");
   
   my @waitString_nt=("Injected correctable error","Injected uncorrectable error");
   my @waitString='PANIC';   
   
   $Api_obj= $self->{_Api_set_obj};
   $Transit_obj=$self->{_Transit_obj};
   $filer_state="SYSTEMSHELL"; 
 
   $cmd="-t" if($trigger=~/t/);
   $cmd= "sudo memerrinject"." ".$cmd." ".$count." ".$type; 
   $cmd= $cmd." ".$addr if($addr=~/^0x[0-9,A-F,a-f]/);
   #$cmd = ($addr=~/^0x[0-9,A-F,a-f]/)? '$cmd." ".$addr':'$cmd." ";

   $filer_state="FIRMWARE" if ($type=~ /u/);
   
   $Log->comment("Command going to execute $cmd");

   my $time=$$Api_obj->get_timeout();
   $$Api_obj->set_timeout("connectrec-timeout" =>'60');
   
   eval
    {
    if(($type=~/u/)&&($trigger=~/t/))
      {
      $$Api_obj->execute_raw_command(command=>$cmd,prompts => (\@waitString));
      }
    elsif(($type=~/c/)&& ($trigger=~/t/))
      { 
      $$Api_obj->execute_raw_command(command=>$cmd);
      sleep(10);
      }
    elsif(($trigger=~/n/)&&($type=~/c|u/))
      {
      $$Api_obj->execute_raw_command(command=>$cmd,prompts => (\@waitString_nt));
      $Log->comment("Injected Error type : $type without trigger");
      $Dimm=-1;
      $addr=-1;
      }
    };
   
   if($@)
     {
      # ADD code to handle supress and locked condition.
      $Log->comment(" Not able to inject errors");
      return;  
     }
   if(defined($Dimm))
     {
     return ($Dimm,$addr); 
     }

   $$Transit_obj->change_state(to => "$filer_state",append_output_to => \$mei_output);
    
    if($mei_output =~ /ECC error at DIMM\-(\d+)+(.*)ADDR\s(0[xX][0-9a-fA-F]+)/ig)
      {
        $Dimm=$1;
        $addr=$3;
        return ($Dimm,$addr);
      }
    else
      {
       $Log->comment(" Check BIOS injection lock and try again.This procedure will return -1 as DIMM value for unsuccessfull injections\n");
       $Dimm=-1;
       $addr=-1;
       return ($Dimm,$addr);
      }
}

###############################################################################
# Method       : memerr_v
# Objective    : Method to verify logger output.
# Return value : Returns hash of DIMM,Address,Time stamp and CECC count values of last reported CECC error in logger.
# Details      : See POD documentation at the end of this file.
###############################################################################

sub memerr_v {

   my $self=shift;
   my ($Api_obj,$Transit_obj,$memerr_op,$cecc_dimm,$cecc_addr,$cecc_timestamp,$cecc_count);
   my %rtn_op;
   
   $Api_obj= $self->{_Api_set_obj};
   $Transit_obj=$self->{_Transit_obj};
 
   $$Transit_obj->change_state(to =>'SYSTEMSHELL',username => 'naroot',password =>'netapp1!'); 
   $memerr_op=$$Api_obj->execute_raw_command(command=>"sudo memerr -v");

   if($memerr_op =~ /No CECC errors since boot/)
         {$cecc_count=0;}
   elsif($memerr_op =~/([0-9]+) error\(s\) occurred since boot/)
         {$cecc_count=$1;}
   else
        { $Log->comment(" memerr -v command not working as expected");}

   if($cecc_count && $memerr_op=~/DIMM-(\S+), Physical address:(.+)\, Time\: \[([0-9]+)\]\s+$/)
      {   
       $cecc_dimm=$1;
       $cecc_addr=$2;
       $cecc_timestamp=$3;
       %rtn_op=( _Dimm => "$cecc_dimm", _addr => "$cecc_addr", _timestamp => "$cecc_timestamp", _count => "$cecc_count" );
       return %rtn_op;
      }
    elsif($cecc_count == 0)
      {
       %rtn_op=( _Dimm => '0', _addr => '0', _timestamp => '0', _count => "$cecc_count");
       return %rtn_op; 
      }
}

1;    
