# $Id: //depot/prod/test/main/lib/platval/platsw/IBM.pm#1 $
#
#       Copyright (c) 2011 Network Appliance, Inc.
#       All Rights reserved
#
## @summary Routines for validating the Branding conversions on the filers.
##
## @description
##    This library contains individual routiens for ONTAP, RLM/SP
##    for analysing various tasks such as getting model information and branding information
##    making the Configuration tables with the given information and installing and updating
##    various SW and FW on Ontap,BIOS and SP/RLM and comparing the model-string at various events.
##
####################################################################################################
####################################################################################################
## Sub-Routines:
##
## 1. create_platform_hash
##      To Load The given Data and make a Hash for the new Platforms from the input files.
##
## 2. add_to_brand_names_hash
##      To Add the created Hash to the existing Brand_Names Hash
##
## 3. get_model_name
##      To Get the Model_Name from sysconfig -a output
##
## 4. get_model_type
##      To Get the Model_Type from sysconfig -a output
##
## 5. change_branding
##      To Change the Branding Information by entering to Maintainance Mode On given Inputs
##
## 6. change_banding_controller
##      To Change the Branding Information of the Controller
##
## 7. change_banding_midplane_1
##      To Change the Branding Information of the Midplane_1
##
## 8. change_banding_midplane_2
##      To Change the Branding Information of the Midplane_2
##
## 9. software_update
##      To do Software Update On the filer
##
##10. compare_model_name
##      To Compare the Model_Names before and after the Events
##
##11.compare_model_type
##      To Compare the Model_Type before and after the Events
##
##12.get_model_name_reboot_after_branding
##      To Get The Model_Name after Filer reboot after branding Conversion
##
##13.get_model_name_panic_after_branding
##      To Get The Model_Name after Filer pinics after branding Conversion
##
##14.get_model_name_power_cycle_after_branding
##      To Get The Model_Name after Filer power_cycle after branding Conversion
##
##15.get_model_name_flash_after_branding
##      To Get The Model_Name after Filer BIOS flash updates after branding Conversion
##
##16.get_model_name_sp_update_after_branding
##      To Get The Model_Name after Filer sp_updates after branding Conversion
##
##17.get_model_name_reset_backup_after_branding
##      To Get The Model_Name after Filer reset_backup after branding Conversion
##
##18.get_model_name_reset_primary_after_branding
##      To Get The Model_Name after Filer reset_primary after branding Conversion
##
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
####################################################################################################
require "tharnit_lib.pl";

use vars qw/%brand_names/;


%brand_names =  (       ABSOLUT  => {   ABSOLUT_L  => ["FAS6240","N7750T","IBM-2867-E11","IBM-2867-E21","IBM-2867-EN1","FAS6240"],
                                        ABSOLUT_M  => ["FAS6280","N7950T","IBM-2867-E12","IBM-2867-E22","IBM-2867-EN2","FAS6280"],
                                        ABSOLUT_V  => ["FAS6210","N7550T","IBM-2867-C10","IBM-2867-C20","IBM-2867-CN0","FAS6210"]},
                        CARNEGIE => {   CARNEGIE_L => ["FAS3210","N6210","IBM-2858-C10","IBM-2858-C20","IBM-2858-CN0","FAS3210"],
                                        CARNEGIE_M => ["FAS3240","N6240","IBM-2858-E11","IBM-2858-E21","IBM-2858-EN1","FAS3240"],
                                        CARNEGIE_H => ["FAS3270","N6270","IBM-2858-E12","IBM-2858-E22","IBM-2858-EN2","FAS3270"]},
                        VESPA    => {   VESPA_2U   => ["FAS2240-2","N3220","IBM-2857-A12","IBM-2857-A22","IBM","FAS"],
                                        VESPA_4U   => ["FAS2240-4","N3240","IBM-2857-A14","IBM-2857-A24","IBM","FAS"]},
                        RUCKUS   => {   RUCKUS     => ["FAS2220","N3150","IBM-2857-A15","IBM-2857-A25","IBM","FAS"]},
                        DELPHI   => {   DELPHI_L   => ["FAS3220","N6220","IBM-2858-E15","IBM-2858-E25","IBM-2858-EN5","FAS3220"],
                                        DELPHI_M   => ["FAS3250","N6250","IBM-2858-E16","IBM-2858-E26","IBM-2858-EN6","FAS3250"]}
                );


my $rsh;
my $run_local="";
my $host = host($FILER);
if($host->{hosttype} =~ /ng/){
$LOGIN="admin";
$RLM_LOGIN="admin";
$rsh = connect("$FILER#ssh");
$run_local="set test -c off;run local";
}else{
$rsh = connect("$FILER#rsh");
}

sub create_platform_hash($) {

        my $inputFile = shift;
        my %my_hash;
        my @hash_array;

        open (FILE, $inputFile) || die "Cannot Open the Input file $inputFile";

        while (<FILE>)  {
                chomp;
                next if ($_ =~ /^#/);
                push @hash_array,$_;
        }

        foreach (@hash_array) {
                my ($key,$value) = split(/=/,$_);
                $my_hash{$key} = $value;
        }

        return \%my_hash;
}



sub add_to_brand_names_hash($) {

        my $my_hash_ref = shift;

        my @my_hash_array;

        push @my_hash_array,"$my_hash_ref->{'NetApp_Model_Name'}";
        push @my_hash_array,"$my_hash_ref->{'IBM_Model_Name'}";
        push @my_hash_array,"$my_hash_ref->{'IBM_Model_Type_Single_Node'}";
        push @my_hash_array,"$my_hash_ref->{'IBM_Model_Type_Dual_Node'}";
        push @my_hash_array,"$my_hash_ref->{'IBM_Model_Name_To_Convert'}";
        push @my_hash_array,"$my_hash_ref->{'NetApp_Model_Name_To_Convert'}";

        $brand_names{$my_hash_ref->{'MODEL'}}->{$my_hash_ref->{'MODEL_TYPE'}} = \@my_hash_array;
}



sub get_model_name($) {

        my $fc = shift;

        say($fc,"$run_local version",'-swallow','-timeout',120);
        sleep 2;
        say($fc,"$run_local sp status",'-swallow','-timeout',120);
        sleep 2;
        my (undef,undef,$out,undef) = say($fc,"$run_local sysconfig -a",'-swallow','-timeout',120);
        sleep 10;

        my ($model_name) = $out =~ /Model\sName\:\s*([\w\d\-]+)/;
        my ($machine_type) = $out =~ /Machine\sType\:\s*([\w\d\-]+)/;
        my ($version) = $out =~ /([\w\s]+)Release\s/;

        return ($version,$model_name,$machine_type);
}



sub get_model_type($) {

        my $model_name = shift;

        foreach $key (keys %brand_names) {
                foreach $sub_key (keys %{$brand_names{$key}}) {
                        if(($model_name eq $brand_names{$key}{$sub_key}->[0]) || ($model_name eq $brand_names{$key}{$sub_key}->[1])) {
                                tstlog -comment,"$FILER with Model_Name: $model_name matched for \n MODEL       : $key \n TYPE        : $sub_key \n NETAPP Name : $brand_names{$key}{$sub_key}->[0] \n IBM Name    : $brand_names{$key}{$sub_key}->[1]";
                                return ($key,$sub_key);
                        }
                }
        }
}



sub change_branding($$$) {

        my $fc = shift;
        my $model = shift;
        my $model_type = shift;

        $conn = $fc;

        my $brand_convert = ($BRAND eq "IBM")? 4 : 5;
        my $model_name_converted = ($BRAND eq "IBM")? 1 : 0;

        enable_sldiag($conn);
        enable_maint_mfg();

        change_banding_controller($fc,$model,$model_type,$brand_convert);
        change_banding_midplane_1($fc,$model,$model_type,$brand_convert) if (($model ne "VESPA") || ($model ne "RUCKUS"));
        change_banding_midplane_2($fc,$model,$model_type,$brand_convert) if (($model eq "CARNEGIE") || ($model eq "DELPHI"));

        say($fc,"halt",'-timeout',120);
        say($fc,"bye",'-timeout',180);

        return $model_name_converted;
}



sub change_banding_controller($$$$) {

        my $fc = shift;
        my $model = shift;
        my $model_type = shift;
        my $brand = shift;

        say($fc,"sldiag system show 0",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system modify 0 product_name $brand_names{$model}{$model_type}->[$brand]",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system show 0",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system commit 0",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system show 0",-swallow,-prompt,'\*\>','-timeout',120);
}



sub change_banding_midplane_1($$$$) {

        my $fc = shift;
        my $model = shift;
        my $model_type = shift;
        my $brand = shift;

        say($fc,"sldiag system show 1",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system modify 1 product_name $brand_names{$model}{$model_type}->[$brand]",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system show 1",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system commit 1",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system show 1",-swallow,-prompt,'\*\>','-timeout',120);
}



sub change_banding_midplane_2($$$$) {

        my $fc = shift;
        my $model = shift;
        my $model_type = shift;
        my $brand = shift;

        say($fc,"sldiag system show 2",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system modify 2 product_name $brand_names{$model}{$model_type}->[$brand]",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system show 2",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system commit 2",-swallow,-prompt,'\*\>','-timeout',120);
        say($fc,"sldiag system show 2",-swallow,-prompt,'\*\>','-timeout',120);
}



sub software_update($$) {

        my $fc = shift;
        my $SW_PATH = shift;
                if($host->{hosttype}=~/ng/){
                say($fc,"system node image update -package $SW_PATH -replace-package true -setdefault true",-swallow,'-noprompt',-prompt,"Installation complete",'-timeout',1800);
                sleep 10;
                }else{
        say($fc,"software update $SW_PATH -f",-swallow,'-noprompt',-prompt,"software: installation of image.tgz completed",'-timeout',1800);
                }
        if($host->{hosttype}=~/ng/){
        say($fc,"system node reboot -node local -inhibit-takeover true",-swallow,-prompt, 'Are you sure you want to reboot the node','login',@prompts,'Password','-timeout',600);
                say($fc,"y",-swallow,'-noprompt',-prompt,'login',@prompts,'Password','-timeout',600);
                }else{
                say($fc,"reboot",-swallow,'-noprompt',-prompt,'login',@prompts,'Password','-timeout',600);
                }
}



sub compare_model_name($$) {

        my $model_name_before = shift;
        my $model_name_after = shift;

        my $result_model_name = ($model_name_before eq $model_name_after)? 1:0;

        return $result_model_name;
}



sub compare_model_type($$) {

        my $model_type_before = shift;
        my $model_type_after = shift;

        my $result_model_type = ($model_type_before eq $model_type_after)? 1:0;

        return $result_model_type;
}



sub get_model_name_reboot_after_branding($) {

        my $fc = shift;

        if($host->{hosttype}=~/ng/){
        say($fc,"system node reboot -node local -inhibit-takeover true",-swallow,-prompt, 'Are you sure you want to reboot the node','login',@prompts,'Password','-timeout',600);
                say($fc,"y",-swallow,'-noprompt',-prompt,'login',@prompts,'Password','-timeout',600);
                }else{
                say($fc,"reboot",-swallow,'-noprompt',-prompt,'login',@prompts,'Password','-timeout',600);
                }
        sleep 300;

        rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
        disconnect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
        $rsh = connect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
        my ($version_after_reboot, $model_name_after_reboot,$machine_type_after_reboot) = get_model_name($rsh);

        return ($version_after_reboot, $model_name_after_reboot,$machine_type_after_reboot);
}



sub get_model_name_panic_after_branding($) {

        my $fc = shift;

        say($fc,"$run_local priv set test;$run_local panic",-swallow,'-noprompt',-prompt,'login',@prompts,'Password','-timeout',1200);

        sleep 300;

        rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
                disconnect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
                $rsh = connect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
        my ($version_after_panic,$model_name_after_panic,$machine_type_after_panic) = get_model_name($rsh);

        return ($version_after_panic,$model_name_after_panic,$machine_type_after_panic);
}



sub get_model_name_power_cycle_after_branding($) {

        my $fc = shift;

        my (undef,undef,$out) = say($fc,"$run_local sp status");
        my ($ip) = $out =~/IP\sAddress\:\s*([\d\.]+)/;

        my $rlm_pt = rlm_make_conn_to_rlm($FILER,$ip,$RLM_LOGIN,$PASSWD);

        say($rlm_pt,"system power cycle",'-swallow','-noprompt','-wait',"Continue",'-timeout',90);
        say($rlm_pt,"y",'-timeout',90);

        sleep 300;

        disconnect($rlm_pt);

        rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);

        disconnect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
        $rsh = connect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
        my ($version_after_power_cycle, $model_name_after_power_cycle,$machine_type_after_power_cycle) = get_model_name($rsh);

        return ($version_after_power_cycle,$model_name_after_power_cycle,$machine_type_after_power_cycle);
}



sub get_model_name_flash_after_branding($$) {

        my $fc = shift;
        my $fh = shift;

        say($fc,"$run_local halt -f -t 0",'-timeout',120);

        say($fc,"ifconfig $fh->{'IFC'} -addr=$fh->{'default_ip'} -mask=$fh->{'MASK'} -gw=$fh->{'GATEWAY'}",'-timeout',120);
        say($fc,"flash $BIOS_PATH",'-timeout',900);

        say($fc,"bye",-swallow,'-noprompt',-prompt,'login',@prompts,'Password','-timeout',600);

        sleep 300;

        rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
        disconnect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
        $rsh = connect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
        my ($version_after_flash,$model_name_after_flash,$machine_type_after_flash) = get_model_name($rsh);

        return ($version_after_flash,$model_name_after_flash,$machine_type_after_flash);
}



sub get_model_name_sp_update_after_branding($) {

        my $fc = shift;

        my (undef,undef,$out) = say($rsh,"$run_local sp status");
        my ($ip) = $out =~/IP\sAddress\:\s*([\d\.]+)/;

        my $rlm_pt = rlm_make_conn_to_rlm($FILER,$ip,$RLM_LOGIN,$PASSWD);

        say($rlm_pt,"sp update $SP_PATH -f",'-swallow','-timeout',1200);

        disconnect($rlm_pt);

        say($fc,"$run_local sp reboot",'-swallow','-noprompt','-wait',"disconnected",'-timeout',90);
        say($fc,"y",'-timeout',90);

        sleep 180;

        rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);

        my ($version_after_sp_update,$model_name_after_sp_update,$machine_type_after_sp_update) = get_model_name($rsh);

        return ($version_after_sp_update,$model_name_after_sp_update,$machine_type_after_sp_update);
}



sub get_model_name_reset_backup_after_branding($) {

        my $fc = shift;

        my (undef,undef,$out) = say($rsh,"$run_local sp status");
        my ($ip) = $out =~/IP\sAddress\:\s*([\d\.]+)/;

        sleep 10;

        say($fc,"$run_local halt -f -t 0",-swallow,'-noprompt',-prompt,'login',@prompts,'Password','-timeout',300);

        my $rlm_pt = rlm_make_conn_to_rlm($FILER,$ip,$RLM_LOGIN,$PASSWD);

        say($rlm_pt,"system reset backup",'-swallow','-noprompt','-wait',"Continue",'-timeout',90);
        say($rlm_pt,"y",'-timeout',90);

        sleep 300;

        disconnect($rlm_pt);

        rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
                disconnect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
        $rsh = connect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
        my ($version_after_reset_backup,$model_name_after_reset_backup,$machine_type_after_reset_backup) = get_model_name($rsh);

        return ($version_after_reset_backup,$model_name_after_reset_backup,$machine_type_after_reset_backup);
}



sub get_model_name_reset_primary_after_branding($) {

        my $fc = shift;

        my (undef,undef,$out) = say($rsh,"$run_local sp status");
        my ($ip) = $out =~/IP\sAddress\:\s*([\d\.]+)/;

        my $rlm_pt = rlm_make_conn_to_rlm($FILER,$ip,$RLM_LOGIN,$PASSWD);

        say($rlm_pt,"system reset primary",'-swallow','-noprompt','-wait',"Continue",'-timeout',90);
        say($rlm_pt,"y",'-timeout',90);

        sleep 300;

        disconnect($rlm_pt);

        rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
                disconnect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
                $rsh = connect("$FILER#ssh") if($host->{hosttype} =~ /ng/);
        my ($version_after_reset_primary,$model_name_after_reset_primary,$machine_type_after_reset_primary) = get_model_name($rsh);

        return ($version_after_reset_primary,$model_name_after_reset_primary,$machine_type_after_reset_primary);
}

