# $Id: //depot/prod/test/main/lib/platval/platsw/memerr.pm#1 $
#***********************************************************************************************************************************
## @summary 	Inject memory errors and validate the same
##
## @description
## "memerr.pm" Currently if the error type is "c,ck,mbn" it injects and validates the error information while if the error type is   
## "u or mbmn" validates the dimm is disabled and appropriate memory size is reduced 
##
## @synopsis
## inject_mem_error
##
## @author              kakshay@netapp.com
#***********************************************************************************************************************************
#!/usr/local/bin/perl

use Tharn;
use boot;
use platval::platsw::util;
require "tharnit_lib.pl";

use strict;

use constant MAX_DIMM			=> 12;
use constant DEBUG			=> 1;
use constant TIMEOUT 			=> 800;
use constant SUCCESS                    => 1;
use constant FAILURE                    => 0;

our @prompts = ('LOADER(-\w+)?\*?>','CFE(-\w+)?\*?>',$FILER.'[\*>#%]','Selection','\*>');
#------------------------------------------------------------------------------------------------------------------
#Function Name  : inject_mem_error
#Parameters     : $error_type           -> Type of error to inject, it can be c,ck,mbn,u,mbmn
#                 $n_error              -> Number of errors to inject
#                 $trigger              -> To trigger immediately after being injected
#                 $mem_addr             -> To inject/trigger error at specified address
#
#Return Type    : SUCCESS		-> Returns the DIMM information details 
#                 FAILURE               -> 0
#
#Description    : Injects and triggers(optional) correctable, uncorrectable or user defined masks at specified or
#                 randomly selected memory location
#-------------------------------------------------------------------------------------------------------------------
sub inject_mem_error
{
   my $host = host($FILER);
   my ($conn,$error_type,$n_error,$trigger,$mem_addr) = @_;

   say($conn,"sudo sysctl debug.cecc_log_console=1",-prompt,$host->{'prompt_norm'});

   my $cmd_opts = "-t" if($trigger);
   $cmd_opts = $cmd_opts." ".$n_error." ".$error_type." ".$mem_addr;

   tstlog -comment,"Injecting: memerrinject $cmd_opts";
   if($error_type =~ /c/i || $error_type =~ /mbn/i || $error_type =~ /ck/i)
   {
      my @waitfor = ("at specified address","at a randomly-selected address","is not 16-byte aligned");

      tstlog -comment,"Introducing Correctable Memory Injection Errors";
      my $match; my $before; my $after; my $prompt;

      if($trigger)
      {
        if($error_type =~ /mbn/i)
        {
          say($conn,"sudo sysctl sysvar.memerrinject.arch.raw_mode=1",-prompt,$host->{'prompt_norm'},-swallow);
          say($conn,"sudo sysctl sysvar.memerrinject.arch.ecc_mask=0x00000003",-prompt,$host->{'prompt_norm'},-swallow);
          ($match,$before,$after,$prompt) = say($conn,"sudo memerrinject -t 1 c $mem_addr",-noprompt,"-wait",@waitfor,-timeout,80,-swallow);
        }
        elsif($error_type =~ /ck/i)
        {
          say($conn,"sudo sysctl sysvar.memerrinject.arch.raw_mode=1",-prompt,$host->{'prompt_norm'},-swallow);
          say($conn,"sudo sysctl sysvar.memerrinject.arch.ecc_mask=0x2f429",-prompt,$host->{'prompt_norm'},-swallow);
          ($match,$before,$after,$prompt) = say($conn,"sudo memerrinject -t 1 c $mem_addr",-noprompt,"-wait",@waitfor,-timeout,80,-swallow);
        }
        else
        {
          say($conn,"sudo sysctl sysvar.memerrinject.arch.raw_mode=0",-prompt,$host->{'prompt_norm'},-swallow);
          ($match,$before,$after,$prompt) = say($conn,"sudo memerrinject $cmd_opts",-noprompt,"-wait",@waitfor,-timeout,80,-swallow);
        }
      }
      else
      {
        say($conn,"sudo sysctl sysvar.memerrinject.arch.raw_mode=0",-prompt,$host->{'prompt_norm'},-swallow);
        ($match,$before,$after,$prompt) = say($conn,"sudo memerrinject $cmd_opts",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);
      }
      if($match =~ /is not 16-byte aligned/)
      {
        tstlog -comment, "Address is not 16-byte aligned";
        return;
      }
      my $err_info;
      for(my $i = 1; $i <= $n_error; $i++)
      {
         if($trigger)
         {
           ($match,undef,$err_info, undef) = say($conn,"",-noprompt,"-wait","ECC error at DIMM \\d+,ADDR 0x\\w+.",-timeout,130,-swallow);
           sleep 3;
           (undef,undef,$after,undef) = say($conn,"sudo memerr -v",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);

           my @at_addr = $match =~ /(0x\w+)/;
           tstlog -comment,"Error At Address: $at_addr[0]";

           return FAILURE if($after !~ /$at_addr[0]\b/);
         }
      } # End of For
      return $err_info;
   } # End of if($error_type =~ /c/i || $error_type =~ /mbn/i || $error_type =~ /ck/i)
   if($error_type =~ /u/i || $error_type =~ /mbmn/i)
   {
      my $after;
      my $dimm_id;
      my @prompts = ('LOADER(-\w+)?\*?>','CFE(-\w+)?\*?>',$FILER.'[\*>#%]');

      tstlog -comment,"Introducing Uncorrectable Memory Injection Errors";

      my (undef,undef,$memsize,undef) = say($conn,"sudo sysctl debug.installed_memsize",-prompt,$host->{'prompt_norm'},-swallow);
      (undef,$memsize) = split(/:/,$memsize);

      tstlog -comment, "Before Injecting Uncorrectable Error:$memsize";

      if($error_type =~ /mbmn/i)
      {
        say($conn,"sudo sysctl sysvar.memerrinject.arch.raw_mode=1",-prompt,$host->{'prompt_norm'},-swallow);
        say($conn,"sudo sysctl sysvar.memerrinject.arch.ecc_mask=0xAAAAAE3A",-prompt,$host->{'prompt_norm'},-swallow);
        (undef,undef,$after,undef) = say($conn,"sudo memerrinject -t 1 c $mem_addr",-prompt,@prompts,-timeout,400,-swallow);
      }
      else
      {
        say($conn,"sudo sysctl sysvar.memerrinject.arch.raw_mode=0",-prompt,$host->{'prompt_norm'},-swallow);
        (undef,undef,$after,undef) = say($conn,"sudo memerrinject $cmd_opts",-prompt,@prompts,-timeout,400,-swallow);
      }
      sleep 5;

      if($after =~ /is not 16-byte aligned/)
      {
        tstlog -comment, "Address is not 16-byte aligned";
        return;
      }
      if(($after =~ /ONTAP Detected Bad DIMM in slot: \d+/))
      {
         my $disabled_size;

         (undef,$dimm_id) = split(/:/,$&);
         if($ENABLE_DIMM)
         {
           ($disabled_size,undef) = enable_dimm($conn,$dimm_id);
           tstlog -comment, "DEBUG->size_before_enable:$disabled_size" if(DEBUG);
           tstlog -comment, "DEBUG->memsize:$memsize" if(DEBUG);
         }
         else
         {
           my $filer_state = get_filer_state($conn);
           $filer_state = goto_systemshell($conn) if($filer_state !~ "IN_SYSTEM_SHELL");

           if(($filer_state == SUCCESS || $filer_state =~ "IN_SYSTEM_SHELL"))
           {
             my (undef,undef,$disabled_size,undef) = say($conn,"sudo sysctl debug.installed_memsize",-prompt,$host->{'prompt_norm'},-swallow);
             (undef,$disabled_size) = split(/:/,$disabled_size);
           }
         }
         ($disabled_size < $memsize) ? tstlog '-comment',"DIMM $dimm_id Disabled" : tstlog '-fail', "Unable To Disable DIMM";

         sleep 10;
      }
      if($after =~ /Uncorrectable Machine Check Error/)
      {
         tstlog -comment,"Injected uncorrectable error"; # at a randomly-selected address";
      }
      return $dimm_id;
   } #End of if($error_type =~ /u/i || $error_type =~ /mbmn/i)
}#End of the Function

1;
