#!/usr/local/bin/perl
# $Id: $
#
#**********************************************************************************************************************************************
##
## @description
## "sp_agent.pm" helps to verify the "system watchdog", "read vgpio pin", "write to vgpio pin", and simulate emergency sensors error conditions
##
## @synopsis
## system_watchdog_verify($conn);
## read_vgpio($conn,$pin,$value);
## write_vgpio($conn,$pin,$value);
## emergency_sensor($conn,$sensor,$event);
##
## @dependencies     test/main/lib/platval/platsw/util.pm
##
## @status      public
##
## @author      vasant.patil@netapp.com
##
## @notes       Created by Platform Validation.
## 
#************************************************************************************************************************************************

use Tharn;
use strict;
use warnings;
use Switch;

use constant SUCCESS                    => 1;
use constant FAILURE                    => 0;

#*********************************************************************************************
## Function Name: system_watchdog_verify
## Parameters   : $conn           -> Filer connection
## Return Type  : SUCCESS         -> 1, if system watchdog is enabled
##                FAILURE         -> 0, if system watchdog is disabled
## Description  : Verify wheather system watchdog is enabled or disabled
##
#*********************************************************************************************

sub system_watchdog_verify 
{
my($Api_Set_Obj) = @_;
my $host = host($FILER);
my $str1 = "Response";
my $result = "21";

## Verify watchdog
tstlog -comment, "Verifying System Watchdog ";
#my ($match,$before,$out,$prompt) =  say($conn,"sudo sptool -r 6 25",-prompt,$host->{'prompt_norm'},-swallow); #akshay->Commented 

my $system_shell_prompt = $Api_Set_Obj->get_connection()->hostp()->id().'__prompt__>\s'; #akshay->Added
my $out = $Api_Set_Obj->execute_raw_command(command => 'sudo sptool -r 6 25','connectrec-prompts' => [$system_shell_prompt] );#akshay->Added


if($out =~ /$str1.*/)
{
  my (undef,$val) = split(/:/,$&);
  my ($byte0,$byte1,$byte2,$byte3) = split(/ /,$val);

	if($byte3 == $result)
	{
	tstlog -comment , "System Watchdog is Enabled\n";	
	return SUCCESS ;
	}
	else
	{
	tstlog -comment , "System Watchdog is Disabled\n";	
	return FAILURE ;
	}
}
else
{
tstlog -comment,"Invalid IPMI watchdog command or Command not supported on this platform\n"; 
return FAILURE ;
}

}#End of system_watchdog_verify Function

#*********************************************************************************************
## Function Name: write_vgpio
## Parameters   : $conn           -> Filer connection
##                $pin            -> vgpio pin number
##                $value          -> value/number to be written to vgpio
##
## Return Type  : N/A
##
## Description  : Writting input data to vgpio pin 
##
#*********************************************************************************************

sub write_vgpio 
{
my($Api_Set_Obj,$pin,$value) = @_;
my $host = host($FILER);

my $system_shell_prompt = $Api_Set_Obj->get_connection()->hostp()->id().'__prompt__>\s'; #akshay->Added

tstlog -comment, "Writing $value to $pin vgpio pin";
my $raw_output = $Api_Set_Obj->execute_raw_command(command => "sudo sysctl debug.vgpio.value=$value",'connectrec-prompts' => [$system_shell_prompt] ); #akshay-> Added
$raw_output = $Api_Set_Obj->execute_raw_command(command => "sudo sysctl debug.vgpio.write_pin=$pin",'connectrec-prompts' => [$system_shell_prompt] ); #akshay-> Added

#say($conn,"sudo sysctl debug.vgpio.value=$value",-prompt,$host->{'prompt_norm'},-swallow); #akshay->Commented 
#say($conn,"sudo sysctl debug.vgpio.write_pin=$pin",-prompt,$host->{'prompt_norm'},-swallow); #akshay->Commented 

} #End of write_vgpio function

#*********************************************************************************************
## Function Name: read_vgpio
## Parameters   : $conn           -> Filer connection
##                $pin            -> vgpio pin number
##                $value          -> dummy data to be written before reading the actual value 
##
## Return Type  : Return the data/value read from vgpio pin
##
## Description  : Reading data from vgpio pin
##
#*********************************************************************************************

sub read_vgpio
{
my($conn,$pin,$value) = @_;
my $host = host($FILER);

tstlog -comment, "Writing dummy data $value to debug.vgpio.value ";
say($conn,"sudo sysctl debug.vgpio.value=$value",-prompt,$host->{'prompt_norm'},-swallow);

tstlog -comment, "Reading $pin vgpio pin";
say($conn,"sudo sudo sysctl debug.vgpio.read_pin=$pin",-prompt,$host->{'prompt_norm'},-swallow);
my ($match,$before,$out,$prompt) = say($conn,"sudo sysctl debug.vgpio.value",-prompt,$host->{'prompt_norm'},-swallow);

my $str = "value:";
if($out =~ m/.*$str.*/)
{
  my ($rec_out,$ret) = split(/:/,$&);
  return $ret;
}

} #End of read_vgpio function

#*********************************************************************************************
## Function Name: emergency_sensor
## Parameters   : $conn           -> Filer connection
##                $sensor         -> Emergency temperatute sensor name
##                $event          -> sensor value (1 or 2) or state(failed) to be set
##
## Return Type  : $out            -> console output after the event trigger
##
## Description  : Simulating the emergency backup sensor policy conditions
##
#*********************************************************************************************

sub emergency_sensor
{
my($conn,$sensor,$event) = @_;
my $host = host($FILER);

if($event =~ "failed")
{
my $filer_state = get_filer_state($conn);
$filer_state = goto_systemshell($conn) if($filer_state !~ "IN_SYSTEM_SHELL");
my ($match,$before,$out,$prompt) = say($conn,"sudo envcmdtool set_val -n \"$sensor\" $event ", -prompt,$host->{'prompt_norm'},-timeout,60,-swallow);
return $out;
}
else
{
my ($match,$before,$out,$prompt) = say($conn,"environment chassis \"$sensor\" set-state $event", -prompt,$host->{'prompt_norm'},-timeout,60,-swallow);
}

}#End of function

1;

