#!/usr/local/bin/perl
# $Id: $
#
#**********************************************************************************************************************************************
##
## @description
## "sram.pm" helps to "intialize and validate sram region","writting sram data records" and "reading sram data records" 
##
## @synopsis
## intialize_validate_sram_region($conn,$region,$in);
## write_sram_records($conn,$region,$rec_type);
## read_sram_records($conn,$region);
##
## @dependencies     test/main/lib/platval/platsw/util.pm
##
## @status      public
##
## @author      vasant.patil@netapp.com
##
## @notes       Created by Platform Validation.
##
#************************************************************************************************************************************************

use Tharn;
use strict;
use warnings;
use Switch;

use constant SUCCESS                    => 1;
use constant FAILURE                    => 0;

#*********************************************************************************************
## Function Name: intialize_validate_sram_region
##
## Parameters   : $conn           -> Filer connection
##              : $region         -> 1=BIOS, 2=ONTAP 
##              : $in             -> 1 if want to initialize the sram region before validating it      
##
## Return Type  : SUCCESS         -> 1, if sram region is valid
##                FAILURE         -> 0, if sram region is invalid
##
## Description  : Initialize and validate the SRAM regions
##
#*********************************************************************************************

sub intialize_validate_sram_region
{
## Intialize aand validate the SRAM region
my($conn,$region,$in) = @_;  
my $host = host($FILER);
my $inval ="invalid";
my $reg = "Reg";


## Intialize the SRAM region
if($in == 1)
{
if($region == 1)
 {
 say($conn,"sudo ssram -g1 -m -z2",-prompt,$host->{'prompt_norm'},-swallow);
 tstlog -comment, "Intializing SRAM BIOS REGION\n";
 }
 else
 {
 tstlog -comment, "Intializing SRAM ONTAP REGION\n";
 }

say($conn,"sudo ssram -i -g $region",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);
}

## Validate the SRAM region
tstlog -comment, "Validating SRAM Region\n";
my ($match,$before,$after,$prompt) = say($conn,"sudo ssram -v -g $region",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);

if($after =~ m/$reg.*/)
{
 my (undef,undef,undef,$str1)= split(/ /,$&);

 if($str1 =~ $inval)
 {
  tstlog -comment, "REGION $region is invalid\n";
  return FAILURE;
 }
 else 
 {
  tstlog -comment, "REGION $region is valid\n";
  return SUCCESS;
 }
}

} ##End of intialize_validate_sram_region function

#*********************************************************************************************
## Function Name: write_sram_records
##
## Parameters   : $conn           -> Filer connection
##              : $region         -> 1=BIOS, 2=ONTAP
##              : $rec_type       -> SRAM record to be written (MESS,DIMM,CPU,PCI,SPD or ALL)
##
## Return Type  : NA
##
## Description  : Writting the SRAM data records in BIOS or ONTAP region
##
#*********************************************************************************************

sub write_sram_records
{
my($conn,$region,$rec_type) = @_;
my $host = host($FILER);
 if($region == 1)
 {
 say($conn,"sudo ssram -g1 -m -z2",-prompt,$host->{'prompt_norm'},-swallow); 
 tstlog -comment, "Writting Data records in SRAM BIOS region";
 }
 else
 {
 tstlog -comment, "Writting Data records in SRAM ONTAP region";
 } 

switch($rec_type)
    {
      case "MESS" 
      {
        say($conn,"sudo ssram -w -g $region -t 1 -l \"I am testing SRAM region \" -x 1 2",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);
      }       
       
     case "DIMM" 
     {
         say($conn,"sudo ssram -w -g $region -t 2 -x8 1 2 2 4 5 6 7 8",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);
     }
     
     case "CPU"
     {
         say($conn,"sudo ssram -w -g $region -t 3 -x 4 1 2 3 4",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);
     }

     case "PCI"
     {
         say($conn,"sudo ssram -w -g $region -t 4 -x6 1 2 3 4 5 6",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);
     }

     case "SPD"
     {
         say($conn,"sudo ssram -w -g $region -t 5 -x4 1 2 2 4",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);
     }

     case "ALL"
     {
        say($conn,"sudo ssram -w -g $region -t 1 -l \"I am testing SRAM region \" -x 1 2",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow); 
        say($conn,"sudo ssram -w -g $region -t 2 -x8 1 1 1 4 5 6 7 8",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);
        say($conn,"sudo ssram -w -g $region -t 3 -x 4 1 2 3 4",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);
        say($conn,"sudo ssram -w -g $region -t 4 -x6 1 2 3 4 5 6",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);  
        say($conn,"sudo ssram -w -g $region -t 5 -x4 1 1 1 4",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);
     }

    }#End of Switch

#@ step : verify the wriiten record

my ($match,$before,$after,$prompt) = say($conn,"sudo ssram -a -g $region",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);
my $str = "Rec";
return SUCCESS if($after =~ m/$str.*/);

} ## End of write_sram_records function

#*********************************************************************************************
## Function Name: read_sram_records
##
## Parameters   : $conn           -> Filer connection
##              : $region         -> 1=BIOS, 2=ONTAP
##
## Return Type  : $count          -> Total number of SRAM data records read
##
## Description  : Reading the SRAM data records from ONTAP or BIOS region
##
#*********************************************************************************************

sub read_sram_records
{
 my($conn,$region) = @_;
 my $host = host($FILER);
 if($region == 1)
 {
 say($conn,"sudo ssram -g1 -m -z2",-prompt,$host->{'prompt_norm'},-swallow);
 } ##end of if 1

my ($match,$before,$after,$prompt) = say($conn,"sudo ssram -a -g $region",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);

my $data = $after ;
my $str = "Rec";
my @record1 = split (/\n/,$data);

my $count = 0;
my $line;
my $mess_rec = "Rec(1)";
my $dimm_rec = "Rec(2)";
my $cpu_rec = "Rec(3)";
my $pci_rec = "Rec(4)";
my $spd_rec = "Rec(5)";


foreach $line(@record1)
{
if($line =~ m/$str.*/)
{
 $count++;
 my ($rec_out,undef) = split(/,/,$&);
 my ($str1,undef,$str2) = split(/ /,$rec_out);
 my ($match,$before,$read_back,$prompt) = say($conn,"sudo ssram -r -g $region -o $str2",-prompt,$host->{'prompt_norm'},-timeout,80,-swallow);

 ##@step verify that SRAM data record are read successfully
 if($read_back =~ m/$str.*/)
 {
  tstlog -comment, "\n Reading of SRAM data record at offset $str2 Complete successfully\n";
 }
 else
 {
  tstlog -comment, "\n FAILS to read back the SRAM data records at offset $str2 : Possibility of invalid offset\n";
 }

##@step Check the type of SRAM data record

  if($str1 eq $mess_rec)
  {
   tstlog -comment, "\nMessage SRAM Data Record  error: $read_back\n";
  }
  elsif($str1 eq $dimm_rec)
  {
   tstlog -comment, "\nDIMM SRAM Data Record  error: $read_back\n";
  }
  elsif($str1 eq $cpu_rec)
  {
   tstlog -comment, "\nCPU SRAM Data Record  error: $read_back\n";
  }
  elsif($str1 eq $pci_rec)
  {
   tstlog -comment, "\nPCI SRAM Data Record  error: $read_back\n";
  }
  elsif($str1 eq $spd_rec)
  {
   tstlog -comment, "\nSPD SRAM Data Record  error: $read_back\n";
  }
  else
  {
   tstlog -comment, "\n Unknown Data Record Error : $read_back\n";
  }

} ## End of 2nd if statement

} ## End of Foreach statement

##tstlog -comment, "\n Total Number of SRAM Data Records = $count\n\n";
return $count;

} ## end of read_sram_records function
1;



