# $Id: //depot/prod/test/main/lib/platval/platsw/pcie.pm
#***********************************************************************************************************************************
## @summary     Modules developed to support Platform software automation common functions for PCIe module.
##
## @description
## It contains common functions to be used for platform validation.
## goto_systemshell
## This is for going from any prompt in ONTAP to systemshell
##
## @synopsis
##
## @author jkonjeti@netapp.com, ravikira@netapp.com, jiljith@netapp.com, sbikkina@netapp.com
##
#***********************************************************************************************************************************

require "tharnit_lib.pl";

sub pcie_verify_led_status ($$$;$) {
        my $fc = shift;
	my $PCI_SLOT = shift;
        my $result_str =shift;          #either PASS or FAIL or WARN
        $result_str = "\U$result_str\E";
        my $corr_err = shift;
        logcomment ("Reading LED status");
        my (undef, undef, $led_status) = say ($fc, "sudo iic -a 0xe8 -b 1 -r 1 -o 1", '-swallow', -timeout, 80);
	(my $temp_a) = $led_status =~ /(\w+\:\s+\w+)/mg;
	$led_status = $&;
        logcomment ("LED Device Offset: $led_status");
        if (($led_status =~ /fe/i)&&($PCI_SLOT == 1)) {
                logresult ($result_str, "Slot 1 LED is turned ON");
                logcomment ("Turning OFF the Slot $PCI_SLOT LED");
                say ($fc, "echo 0 > /dev/led/pcislot_1", '-swallow', -timeout, 10);
        } elsif (($led_status =~ /fd/i)&&($PCI_SLOT == 2)) {
                logresult ($result_str, "Slot 2 LED is turned ON");
                logcomment ("Turning OFF the Slot $PCI_SLOT LED");
                say ($fc, "echo 0 > /dev/led/pcislot_2", '-swallow', -timeout, 10);
        } elsif ($led_status =~ /fc/i) {
                logresult ('FAIL', "Both Slot LEDs are turned ON");
		say ($fc, "echo 0 > /dev/led/pcislot_1", '-swallow', -timeout, 10);
                say ($fc, "echo 0 > /dev/led/pcislot_2", '-swallow', -timeout, 10);
        } elsif (($led_status =~ /fb/i)&&($PCI_SLOT == 3)) {	#valid only for BIMOTA and Agusta
                logresult ($result_str, "Slot 3 LED is turned ON");
                logcomment ("Turning OFF the Slot $PCI_SLOT LED");
                say ($fc, "echo 0 > /dev/led/pcislot_3", '-swallow', -timeout, 10);
        } elsif (($led_status =~ /f7/i)&&($PCI_SLOT == 4)) {	#valid only for BIMOTA and Agusta
                logresult ($result_str, "Slot 4 LED is turned ON");
                logcomment ("Turning OFF the Slot $PCI_SLOT LED");
                say ($fc, "echo 0 > /dev/led/pcislot_4", '-swallow', -timeout, 10);
	} elsif (($led_status =~ /ff/i) && ($PCI_SLOT == 0)) {	#valid only for onboard PLX swith ports plx87xx
		logresult ($result_str, "All LEDs are turned OFF as expected");
        } elsif (($led_status =~/ff/i) && ($corr_err =~ /PCI Express correctable error/i)) {
                logresult ('PASS', "All LEDs are turned OFF as expected");
        } else {
                logresult ('FAIL', "Some problem with the LED behavior, system is not working as expected");
		say ($fc, "echo 0 > /dev/led/pcislot_1", '-swallow', -timeout, 10);
                say ($fc, "echo 0 > /dev/led/pcislot_2", '-swallow', -timeout, 10);
		say ($fc, "echo 0 > /dev/led/pcislot_3", '-swallow', -timeout, 10);
		say ($fc, "echo 0 > /dev/led/pcislot_4", '-swallow', -timeout, 10);
        }
}

sub pcie_dump_ontap_ems_logs ($$) {
        my $fc = shift;
        my $err_type = shift;
        logcomment ("Dumping EMS Logs");
        say ($fc, "tail -50 /mroot/etc/log/ems", '-swallow', -timeout, 80);

        logcomment ("Dumping EMS logs specific to error");
        if ($err_type =~ /uerr/i) {
                say ($fc, "grep boot_reason -B1 /mroot/etc/log/ems | tail -3", '-swallow', -timeout, 80);
                say ($fc, "grep panic -B1 /mroot/etc/log/ems | tail -3", '-swallow', -timeout, 80);
                say ($fc, "grep stack_string -B1 -A1 /mroot/etc/log/ems | tail -3", '-swallow', -timeout, 80);
        } elsif ($err_type =~ /cerr/i) {
                say ($fc, "grep pcie_errors -B1 /mroot/etc/log/ems | tail -3", '-swallow', -timeout, 80);
        }
}

sub pcie_dump_sp_sel_logs ($) {
        my $r = shift;
        logcomment ("Dumping relevant SEL logs from SP");
        say ($r, "events newest 50", '-swallow','-timeout', 300);
        say ($r, "events search panic", '-swallow','-timeout', 300);
        say ($r, "events search email", '-swallow','-timeout', 300);
}

1;
