# $Id: //depot/prod/test/main/lib/platval/platsw/platsw.pm
#***********************************************************************************************************************************
## @summary     Modules developed to support Platform software automation common functions.
## 
## @description
## It contains common functions to be used for platform validation.
## goto_systemshell
## This is for going from any prompt in ONTAP to systemshell
##
## @synopsis
## 
## @author divya.bokaria@netapp.com, jkonjeti@netapp.com, ravikira@netapp.com, kakshay@netapp.com, jiljith@netapp.com
##
#***********************************************************************************************************************************

require "tharnit_lib.pl";
use rlm_ipv6;

sub get_system_config ($) {
        my $conn = shift;
        logcomment ("Getting the system configuration");
        say ($conn, "priv set test", -swallow);
        say ($conn, "sysconfig -a", -noprompt, -prompt, "$FILER".'(\-\d+)?\*\>', -swallow, '-timeout', 500);
        say ($conn, "sysconfig -P", -swallow, '-timeout', 500);
        say ($conn, "sysconfig -M", -swallow, '-timeout', 500);
        my(undef, undef, $output) = say ($conn, "sysconfig -ac", '-swallow', '-timeout', 500);
        if ($output !~ /There are no configuration errors/) {
                logresult ('WARN', "There are configuration errors in the system");
        }
        say ($conn, "vol status -r", '-swallow', '-timeout', 100);
        say ($conn, "df -Vh", '-swallow', '-timeout', 100);
        say ($conn, "df -Ah", '-swallow', '-timeout', 100);
        say ($conn, "sasadmin adapter_state", '-swallow', '-timeout', 100);
        my (undef, undef, $output) = say ($conn, "sasadmin expander_map", '-swallow', '-timeout', 100);
	say ($conn, "priv set", -swallow);
        return ($output);
}

sub dump_ontap_ems_logs ($$) {
        my $fc = shift;
        my $err_type = shift;
        logcomment ("Dumping EMS Logs");
        eval { say ($fc, "tail -50 /mroot/etc/log/ems", '-swallow', -timeout, 280) };

        logcomment ("Dumping EMS logs specific to error");
        if ($err_type =~ /uerr/i) {
           eval {
                say ($fc, "grep boot_reason -B1 /mroot/etc/log/ems | tail -3", '-swallow', -timeout, 280);      #In BBA platforms, during stress grep taking long time.Burt:677785
                say ($fc, "grep panic -B1 /mroot/etc/log/ems | tail -3", '-swallow', -timeout, 280);
                say ($fc, "grep stack_string -B1 -A1 /mroot/etc/log/ems | tail -3", '-swallow', -timeout, 280);
           };
        } elsif ($err_type =~ /cerr/i) {
           eval { say ($fc, "grep pcie_errors -B1 /mroot/etc/log/ems | tail -3", '-swallow', -timeout, 80) };
        }
}

sub start_stress ($$)
{
  my $conn = shift;
  my $host = shift;
  say($conn,"priv set test",-prompt,$host->{'prompt_norm'},-swallow, -timeout, 30);
  say($conn,"disk scrub start",-prompt,$host->{'prompt_norm'},-swallow);
  say($conn,"hammer -f -1 /vol/myvol/filename1 1234568",-prompt,$host->{'prompt_norm'},-swallow);
  say($conn,"hammer -f -1 /vol/myvol/filename2 1234568",-prompt,$host->{'prompt_norm'},-swallow);
  say($conn,"hammer -f -1 /vol/myvol/filename3 1234568",-prompt,$host->{'prompt_norm'},-swallow);
  #say($conn,"filersio asyncio_active 00 -r 100 4k 0 7g -1 5 /vol/myvol/file.90g  -create", -prompt,$host->{'prompt_norm'},-swallow);
  #say($conn,"iftest start -j -n e0a e0c e0b e0d e0i e0j e0k e0l", -prompt,$host->{'prompt_norm'},-swallow, -timeout, 300);  <- Stops at e0a itself if not connected
  say($conn,"filersio asyncio_active 50 -r 100 64k 0 10G 31536000 4 /vol/myvol/file1.90g  -create", -prompt,$host->{'prompt_norm'},-swallow);
  say($conn,"filersio asyncio_active 50 -r 100 64k 0 10G 31536000 4 /vol/myvol/file2.90g  -create", -prompt,$host->{'prompt_norm'},-swallow);
  sleep 10;

  my(undef,undef,$load,undef) = say($conn,"sysstat -x -c 10 1",-prompt,$host->{'prompt_norm'},-swallow, -timeout, 30);
#  tstlog -comment,"Load is:$load";
  sleep 30;
}

sub connect_filer_ssh {
	my $fs = connect("$FILER#$ssh++;conntype=ssh-persist; connhost=$FILER_IP; username=root; password=netapp1!");
	return $fs;
}

sub random_selection ($) {
        my @matrix = @{$_[0]};
        if ($RANDOM == 1) {
                logcomment ("Selecting devices and error types randomly from the list");
                my %combinations_hash;
                my @combinations_case_coverage_order;
                until (scalar (keys %combinations_hash) == scalar(@matrix)) {
                                my $index = int rand (scalar @matrix);
                                next if (exists $combinations_hash{$index});
                                $combinations_hash{$index} = '';
                                push @combinations_case_coverage_order, $matrix[$index];
                }
                @matrix = @combinations_case_coverage_order;
                logcomment ("Executing devices and error types randomly from the list @matrix");
        } else {
                logcomment ("Executing devices and error types Sequentially from the list");
        }
        return (\@matrix);
}

sub disconnect_filerBSD_console ($;$) {
        my $conn = shift;
	my $host = shift;
        say($conn,"exit", -swallow);
        say($conn,"exit", -swallow) unless($h->{hosttype} =~ /ng/);
}

sub disconnect_filerBSD_ssh ($) {
        my $conn = shift;
        disconnect_filerBSD_console ($conn);
        say($conn,"exit", -blind, -swallow);
}

sub verify_faults ($) {
	my $conn = shift;
	logcomment ("Verifying if any faults faults in the system");
	say ($conn, "show_faults");
}

sub goto_systemshell ($;$) {
        my $conn = shift;
	my $h = shift;
        if ($h->{hosttype} =~ /ng/) {
                say ($conn, "set diag -c off", -swallow);
        } else {
                say ($conn, "priv set test", -swallow);
        }
        say($conn,"systemshell",-noprompt,-prompt,"login:",-swallow,-timeout,30);
        say($conn,"diag",-noprompt,-prompt,"Password:",-swallow,-timeout,30);
        #my (undef,undef,$output)=say($conn,"netapp1!",-swallow, -noprompt, -prompt, "$FILER\%", -timeout,30);
        my (undef,undef,$output)=say($conn, "netapp1!", '-swallow', '-timeout', 30);
        if($output !~ /Warning\:  The system shell provides access to low-level/) {
                logresult('FAIL',"Could not connect to systemshell, please ensure login and password are correct\n");
        }
}

sub stress_cleanup ($;$) {
        my $conn = shift;
	my $FILER = shift;

        logcomment("Performing cleanup...");

        my ($match, undef, undef, $prompt) = say ($conn, " ", -swallow, -prompt, "^$FILER", "LOADER", "$SP");

        if ($prompt eq "$FILER") {
                ## @step As part of cleanup, stop the previously begun stress tests
                say ($conn, "priv set test",    -swallow);
                say ($conn, "filersio stop",    -swallow);
                say ($conn, "disk scrub stop",  -swallow);
                say ($conn, "hammer abort",     -swallow);
                say ($conn, "iftest stop",      -swallow);
                say ($conn, "priv set",      	-swallow);
        } else {
                logcomment("Filer is not at ONTAP prompt; unable to stop the stress tests");
        }
}

#-----------------------------------------------------------------------------------------
#Function Name  : start_iftest
#Parameters     : The two network interfaces we intend to run the test on - ETH0, ETH1
#Return Type    : NA
#Description    : Intended to make sure network for ports specified works correctly
#-----------------------------------------------------------------------------------------
sub start_iftest ($$$$)
{
  my $conn = shift; 
  my $host = shift;

  my $ETH0 = shift;
  my $ETH1 = shift;
 
  say($conn, "iftest start $ETH0 $ETH1", -prompt, $host->{'prompt_norm'},-swallow);

  # 'iftest status'-- incase of a failure-- is updated only after iftest times-out; so let us loop until it either starts or times-out
  while (1)
  {
  ($match, undef, $output, $prompt) = say($conn, "iftest status", -prompt,$host->{'prompt_norm'}, -swallow);

        # Match output of iftest for both conditions
        $output =~ m/Connector \#[0-9] has processed ([0-9]+) MB|iftest (is) not running/;

        logcomment("MegaBytes processed by iftest: $1");

        # if MegaBytes processed is > 0 MB, then iftest has successfully begun
        if ($1 ne '0'){
                        last;
                }

        sleep 10;
  }

  ($match, $before, $after, $prompt) = say($conn, "iftest status", -prompt,$host->{'prompt_norm'}, -swallow);

  if ($after =~ /iftest is not running/)
  {
        logresult("FAIL", "Could not start iftest stress on interfaces $ETH0 $ETH1");
  }
  else
  {
        logcomment("IFTest started on interfaces $ETH0 and $ETH1");
  }
}


__END__
sub goto_systemshell($;$$)
{       my $fc= rlm_make_conn_to_ontap($FILER);
        my $shell_connect = shift ;
        my $prompt,$output;
        $login_shell=$_[0] ? $_[0] : 'diag';
        $passwd_shell=$_[1] ? $_[1] : 'netapp1!';
        (undef,undef,undef,$prompt)=say($fc,"\n",-prompt,@prompts,@prompts_sp,'Service Processor Login:','Password:',-swallow,-timeout,30);
        if($shell_connect ==1)
        {
                if($prompt=~/$FILER\%/)
                {       logcomment("Filer is at systemshell\n");
                        return;
                }
                rlm_verify_filer_state($FILER);
                (undef,undef,undef,$prompt)=say($fc,"priv set test",-prompt,@prompts,-swallow,-timeout,30);
                (undef,undef,undef,$prompt)=say($fc,"systemshell",-noprompt,-prompt,"login:",-swallow,-timeout,30);
                (undef,undef,undef,$prompt)=say($fc,"$login_shell",-noprompt,-prompt,"Password:",-swallow,-timeout,30);
                (undef,undef,$output,$prompt)=say($fc,"$passwd_shell",-noprompt,-prompt,"$FILER\%",-swallow,-timeout,30);
                if($output=~ /Warning\:  The system shell provides access to low-level/)
                        {logcomment("Successfully came till systemshell\n");}
                else
 {logresult('FAIL',"Could not connect to systemshell, please ensure login and password are correct\n");}
        }
        if($shell_connect ==0)
        {       if($prompt=~/$FILER\%/)
                {       (undef,undef,$output,$prompt)=say($fc,"exit",-noprompt,-prompt,"$FILER\*\>",-swallow,-timeout,30);
                        logcomment("Successfully logged out of systemshell\n");
                }
                else
                {logcomment("Not in systemshell\n");}
        }
}
1;
