##$Id://depot/prod/test/Rsierranevada/lib/platval/platsw/sram.pm#1 $
##
## Library consists of all functions related to sram module.
##
## Objective:
##
## Common functions related to SRAM testcases.
##
## @author:swarup.kulkarni@mindteck.com;dl-pvt-platsw@netapp.com.
##
## Dependancy: Its requires libary called sp_agent.pm.
##

package sram;

use sp_agent;
use NACL::APISet;
use NACL::Transit;
use warnings;
use Tharn;
use Carp qw(croak);
use strict;
use NATE::Log qw(log_global);

#global variables used
our ($FILER,           $PASSWORD,      $TIMEOUT,            $Node,
     $Obj_systemshell, $Host,          $Conn,               $Obj_transit,
     $current_state,   $apiset_loader, $apiset_systemshell, $output,
     $Sp_agent,        $self,          $log);

sub new {
    my $class = shift;
    $FILER = shift;
    $log   = log_global();
    _init($FILER);
    my $ref = {};
    bless $ref, $class;
}

sub _init {
    my $FILER = shift;
    $Host        = host($FILER);
    $Conn        = connect("#console");
    $Node        = $Host->hosttype();
    $PASSWORD    = $Host->{'default_password'};
    $Obj_transit = NACL::Transit->new(name => $FILER);
    $Sp_agent    = sp_agent->new($FILER);
}

## function dumps ssram file from systemshell.
##
## Input: NA
##
## Output : an hash of multiple values.
##
sub sram_log {
    my ($output, @record, @msg, @value, %msg, $obj_systemshell);
    $self = shift;

    $Obj_transit->change_state(to => "SYSTEMSHELL", password => "$PASSWORD", timeout => '500');
    $obj_systemshell = $Sp_agent->Apiset_creation('Systemshell');

    $output =
      $obj_systemshell->execute_raw_command(command => "cat /mroot/etc/log/ssram/ssram.log",
                                            timeout => '100');

    @record = split("\n", $output);
    @record = grep { $_ !~ /^\s+$/ig } @record;

    foreach my $record (@record) {
        if ($record =~ /LOG/ig) {
            @value = $record =~ /ONTAP\:(.*)/ig;
            push(@{$msg{'msg'}}, @value);
        }

        if ($record =~ /DIMM/ig) {
            @value = $record =~ /ONTAP\:.*(\d+)/ig;
            push(@{$msg{'dimm'}}, @value);
        }

        if ($record =~ /CPU/ig) {
            @value = $record =~ /ONTAP\:+\s+socket\((\d).*core\((\d).*bank\((\d)/ig;
            push(@{$msg{'cpu'}}, @value);
        }

        if ($record =~ /PCI/ig) {
            @value = $record =~ /ONTAP\:.*slot\((\d)/ig;
            push(@{$msg{'pci'}}, @value);
        }

        if ($record =~ /SPD/ig) {
            @value = $record =~ /ONTAP\:.*slot\((\d)/ig;
            push(@{$msg{'spd'}}, @value);
        }
    } ## end foreach my $record (@record)
    return \%msg;
} ## end sub sram_log

##Function used to stimulate the different types of errors from BIOS.
##
##Input : command to simululate testharness errors.
##
##Output : command executed status.
##
sub testharness_set {
    my ($output, $command);
    $self    = shift;
    $command = shift;
    $Obj_transit->change_state(to => "FIRMWARE", timeout => '500');
    $apiset_loader = $Sp_agent->Apiset_creation('Firmware');
    $apiset_loader->execute_raw_command(command => "priv set test");
    $output = $apiset_loader->execute_raw_command(command => "$command", timeout => '200');
    return $output;
} ## end sub testharness_set

##To execute commands from freebsd
##
##Input : command to execute
##
##Output : command executed status
##
## Memerrinjection should be enabled in BIOS setup page.

sub Execute_command_from_freebsd {
    my ($output, $command);
    $self    = shift;
    $command = shift;

    $Obj_transit->change_state(to => "SYSTEMSHELL", password => "$PASSWORD", timeout => '500');
    $apiset_systemshell = $Sp_agent->Apiset_creation('Systemshell');
    $output = $apiset_systemshell->execute_raw_command(command => "$command", timeout => '300');
    return $output;
} ## end sub Execute_command_from_freebsd

## Function to get SRAM data for different types errors at both ONTAP and LOADER
## region($region) and from(systemshell or firmware)state.
##
## Input :region(0 or 1),state(FIRMWARE or SYSTEMSHELL) command.
##
## Output :0(failure) or 1(success) or 2(empty) and array of hashes
##
sub verify_record {
    my ($region,       $output, $tmp_output, @type_1, @type_2,
        @type_3,       @type_4, @type_5,     $i,      $count,
        $record_count, $state,  $command,    $type,   @present_lines,
        @record_count, @type1,  @type2,      @type3,  @type4,
        @type5);
    $self    = shift;
    $region  = shift;
    $state   = shift;
    $command = shift;

    undef @type1;
    undef @type2;
    undef @type3;
    undef @type4;
    undef @type5;

    $count = 0;

    $log->log("Verify data length of SRAM Region");
    $Obj_transit->change_state(to => "$state", timeout => '500');
    if ($state =~ /FIRMWARE/ig) {
        $type = 'Firmware';
    } else {
        $type = 'Systemshell';
    }

    $apiset_systemshell = $Sp_agent->Apiset_creation("$type");
    if ($type eq 'Firmware') {
        $apiset_systemshell->execute_raw_command(command => "priv set test");
    }
    $output = $apiset_systemshell->execute_raw_command(command => "$command");
    if ($output =~ /Invalid/ig) {
        croak("Not able to execute command");
    }
    @present_lines = split("\n", $output);

    @record_count = $output =~ /Rec/igs;
    $record_count = @record_count;

    if ($record_count == 0) {
        $log->log("No record exist");
        return 2;
    }

    $log->log("Find no. of entries for records");
    for (my $i = 0 ; $i <= $#present_lines ; $i++) {
        if ($present_lines[$i] =~ /Rec\(1/ig) {
            push(@type_1, $present_lines[$i]);
            push(@type_1, $present_lines[++$i]);
            $count++;
        }
        if ($present_lines[$i] =~ /Rec\(2/ig) {
            push(@type_2, $present_lines[$i]);
            push(@type_2, $present_lines[++$i]);
            push(@type_2, $present_lines[++$i]);
            $count++;
        }
        if ($present_lines[$i] =~ /Rec\(3/ig) {
            push(@type_3, $present_lines[$i]);
            push(@type_3, $present_lines[++$i]);
            $count++;
        }
        if ($present_lines[$i] =~ /Rec\(4/ig) {
            push(@type_4, $present_lines[$i]);
            push(@type_4, $present_lines[++$i]);
            push(@type_4, $present_lines[++$i]);
            $count++;
        }
        if ($present_lines[$i] =~ /Rec\(5/ig) {
            push(@type_5, $present_lines[$i]);
            push(@type_5, $present_lines[++$i]);
            $count++;
        }

    } ## end for (my $i = 0 ; $i <= ...

    $log->log("Find no of entries for records");
    $i = 0;
    if (@type_1) {
        foreach my $values (@type_1) {
            if ($values =~ /Rec\((\d)/ig) {
                $type1[$i]->{'Rec'} = $1;
            }
            if ($values =~ /\@+\s+([0-9a-z]+)/ig) {
                $type1[$i]->{'offset'} = $1;
            }
            if ($values =~ /len\((\d+)/ig) {
                $type1[$i]->{'length'} = $1;
            }
            if ($values =~ /tstamp\(([a-z0-9]+)/ig) {
                $type1[$i]->{'timestamp'} = $1;
            }
            if ($values =~ /msg\((.*?)\)/i) {
                $type1[$i]->{'message'} = $1;
            }
            $i++;
        } ## end foreach my $values (@type_1)
    } ## end if (@type_1)
    $i = 0;
    if (@type_2) {
        foreach my $values (@type_2) {
            if ($values =~ /Rec\((\d)/ig) {
                $type2[$i]->{'Rec'} = $1;
            }
            if ($values =~ /\@+\s+([0-9a-z]+)/ig) {
                $type2[$i]->{'offset'} = $1;
            }
            if ($values =~ /len\((\d+)/ig) {
                $type2[$i]->{'length'} = $1;
            }
            if ($values =~ /tstamp\(([a-z0-9]+)/ig) {
                $type2[$i]->{'timestamp'} = $1;
            }
            if ($values =~ /node\((\d)/i) {
                $type2[$i]->{'node'} = $1;
            }
            if ($values =~ /chan\((\d)/i) {
                $type2[$i]->{'chan'} = $1;
            }
            if ($values =~ /dimm\((\d)/i) {
                $type2[$i]->{'dimm'} = $1;
            }
            if ($values =~ /rank\(([a-z0-9]+)/i) {
                $type2[$i]->{'rank'} = $1;
            }
            if ($values =~ /bank\(([a-z0-9]+)/i) {
                $type2[$i]->{'bank'} = $1;
            }
            if ($values =~ /row\(([a-z0-9]+)/i) {
                $type2[$i]->{'row'} = $1;
            }
            if ($values =~ /col\(([a-z0-9]+)/i) {
                $type2[$i]->{'col'} = $1;
            }
            $i++;
        } ## end foreach my $values (@type_2)
    } ## end if (@type_2)
    $i = 0;
    if (@type_3) {
        foreach my $values (@type_3) {
            if ($values =~ /Rec\((\d)/ig) {
                $type3[$i]->{'Rec'} = $1;
            }
            if ($values =~ /\@+\s+([0-9a-z]+)/ig) {
                $type3[$i]->{'offset'} = $1;
            }
            if ($values =~ /len\((\d+)/ig) {
                $type3[$i]->{'length'} = $1;
            }
            if ($values =~ /tstamp\(([a-z0-9]+)/ig) {
                $type3[$i]->{'timestamp'} = $1;
            }
            if ($values =~ /node\((\d)/i) {
                $type3[$i]->{'node'} = $1;
            }
            if ($values =~ /core\((\d)/i) {
                $type3[$i]->{'core'} = $1;
            }
            if ($values =~ /bank\((\d)/i) {
                $type3[$i]->{'bank'} = $1;
            }
            if ($values =~ /error\((\d+)/i) {
                $type3[$i]->{'error'} = $1;
            }
            $i++;
        } ## end foreach my $values (@type_3)
    } ## end if (@type_3)

    $i = 0;
    if (@type_4) {
        foreach my $values (@type_4) {
            if ($values =~ /Rec\((\d)/ig) {
                $type4[$i]->{'Rec'} = $1;
            }
            if ($values =~ /\@+\s+([0-9a-z]+)/ig) {
                $type4[$i]->{'offset'} = $1;
            }
            if ($values =~ /len\((\d+)/ig) {
                $type4[$i]->{'length'} = $1;
            }
            if ($values =~ /tstamp\(([a-z0-9]+)/ig) {
                $type4[$i]->{'timestamp'} = $1;
            }
            if ($values =~ /bus\((\d)/i) {
                $type4[$i]->{'bus'} = $1;
            }
            if ($values =~ /slot\((\d)/i) {
                $type4[$i]->{'slot'} = $1;
            }
            if ($values =~ /func\((\d)/i) {
                $type4[$i]->{'func'} = $1;
            }
            if ($values =~ /pslot\((\d+)/i) {
                $type4[$i]->{'pslot'} = $1;
            }
            if ($values =~ /error\((\d+)/i) {
                $type4[$i]->{'error'} = $1;
            }
            if ($values =~ /action\((\d+)/i) {
                $type4[$i]->{'action'} = $1;
            }
            $i++;
        } ## end foreach my $values (@type_4)
    } ## end if (@type_4)

    $i = 0;
    if (@type_5) {
        foreach my $values (@type_5) {
            if ($values =~ /Rec\((\d)/ig) {
                $type5[$i]->{'Rec'} = $1;
            }
            if ($values =~ /\@+\s+([0-9a-z]+)/ig) {
                $type5[$i]->{'offset'} = $1;
            }
            if ($values =~ /len\((\d+)/ig) {
                $type5[$i]->{'length'} = $1;
            }
            if ($values =~ /tstamp\(([a-z0-9]+)/ig) {
                $type5[$i]->{'timestamp'} = $1;
            }
            if ($values =~ /node\((\d)/i) {
                $type5[$i]->{'node'} = $1;
            }
            if ($values =~ /chan\((\d)/i) {
                $type5[$i]->{'chan'} = $1;
            }
            if ($values =~ /dimm\((\d)/i) {
                $type5[$i]->{'dimm'} = $1;
            }
            if ($values =~ /error\((\d+)/i) {
                $type5[$i]->{'error'} = $1;
            }
            $i++;
        } ## end foreach my $values (@type_5)
    } ## end if (@type_5)

    if (@type1) {
        $log->log("Type 1 messages");
        foreach my $values (@type1) {
            while (my ($a, $b) = each(%{$values})) {
                $log->log("Fields for type_1 Key:$a and values:$b");
                if (!defined($b)) {
                    $log->log("Fields for type_1 Key:$a is not defined");
                    return 0;
                }
            }

        } ## end foreach my $values (@type1)
    } else {
        $log->log("No matching record for type 1");
        return 0;
    }

    if (@type2) {
        foreach my $values (@type2) {
            while (my ($a, $b) = each(%{$values})) {
                $log->log("Fields for type_2 Key:$a and values:$b");
                if (!defined($b)) {
                    $log->log("Fields for type_2 Key:$a is not defined");
                    return 0;
                }
            }
        }
    } else {
        $log->log("No matching record for type 2");
        return 0;
    }

    if (@type3) {
        foreach my $values (@type3) {
            while (my ($a, $b) = each(%{$values})) {
                $log->log("Fields for type_3 Key:$a and values:$b");
                if (!defined($b)) {
                    $log->log("Fields for type_3 Key:$a is not defined");
                    return 0;
                }
            }
        }
    } else {
        $log->log("No matching record for type 3");
        return 0;
    }

    if (@type4) {
        foreach my $values (@type4) {
            while (my ($a, $b) = each(%{$values})) {
                $log->log("Fields for type_4 Key:$a and values:$b");
                if (!defined($b)) {
                    $log->log("Fields for type_4 Key:$a is not defined");
                    return 0;
                }
            }
        }
    } else {
        $log->log("No matching record for type 4");
        return 0;
    }

    if (@type5) {
        foreach my $values (@type5) {
            while (my ($a, $b) = each(%{$values})) {
                $log->log("Fields for type_5 Key:$a and values:$b");
                if (!defined($b)) {
                    $log->log("Fields for type_5 Key:$a is not defined");
                    return 0;
                }
            }
        }
    } else {
        $log->log("No matching record for type 5");
        return 0;
    }

    if ($count == $record_count) {
        $log->log("No. of records matched");
        return (\@type1, \@type2, \@type3, \@type4, \@type5, 1);
    } else {
        $log->log("No. of records did not matched ");
        return 0;
    }
} ## end sub verify_record

## Function to initialise SRAM from systemshell.
##
## Input :region(0 or 1).
##
## Output :0(failure) or 1(success).
##
sub initialise_region {
    my ($region, $output);

    $self   = shift;
    $region = shift;

    $log->log("Initialise $region region");
    $Obj_transit->change_state(to => "SYSTEMSHELL", password => "$PASSWORD");
    $apiset_systemshell = $Sp_agent->Apiset_creation('Systemshell');
    $apiset_systemshell->execute_raw_command(command => "sudo ssram -i -g $region");

    $log->log("Verify $region is Intialised");
    $output = $apiset_systemshell->execute_raw_command(command => "sudo ssram -a -g $region");
    if ($output =~ /DataLen\((?=0)/ig) {
        $log->log("$region is Intialised properly");
        return 1;
    } else {
        $log->log("$region is not initialised properly");
        return 0;
    }
} ## end sub initialise_region

## Function to verify header of SRAM logs from systemshell.
##
## Input :region(0 or 1).
##
## Output :0(for initialised sram region) or 1(data records present).
##
sub verify_header {
    my ($region, $output);
    $self   = shift;
    $region = shift;

    $Obj_transit->change_state(to => "SYSTEMSHELL", password => "$PASSWORD");
    $apiset_systemshell = $Sp_agent->Apiset_creation("Systemshell");
    $output = $apiset_systemshell->execute_raw_command(command => "sudo ssram -g $region -h");
    if ($output !~ /DataLen+\((?=0)/ig) {
        $log->log("Data length is more than 0");
        return 1;
    } else {
        $log->log("Data length is zero -0");
        return 0;
    }
} ## end sub verify_header

## Function to write diffent records to sram region from systemshell.
##
## Input :region(0 or 1),$record_type(1,2,3,4,5),$msg(string),$size no_of_bytes,@data(bytes_written).
##
## Output :0(for wrong usage of command or inadequate params) or 1(data records written succesful).
##
sub sram_write_data {
    my ($region, $record_type, $msg, $size, $data, $output);
    $self        = shift;
    $region      = shift;
    $record_type = shift;
    $msg         = shift;
    $size        = shift;
    $data        = shift;

    $log->log("Write to $region region");
    $Obj_transit->change_state(to => "SYSTEMSHELL", password => "$PASSWORD");
    $apiset_systemshell = $Sp_agent->Apiset_creation('Systemshell');
    $output = $apiset_systemshell->execute_raw_command(
               command => "sudo ssram -g $region -t $record_type -l \"$msg\" -x $size -w @{$data}");
    if ($output =~ /usage/ig) {
        return 0;
    }
    return 1;
} ## end sub sram_write_data

## Function to get sram header output of sram region from systemshell.
##
## Input :region(0 or 1).
##
## Output :command output.
##
sub sram_size {
    my $output;
    $self = shift;
    my $region = shift;

    $Obj_transit->change_state(to => "SYSTEMSHELL", password => "$PASSWORD", timeout => '500');
    $apiset_systemshell = $Sp_agent->Apiset_creation('Systemshell');
    $output =
      $apiset_systemshell->execute_raw_command(command => "sudo ssram -h -g $region",
                                               timeout => '300');
    return $output;
} ## end sub sram_size
## Function to initialise SRAM from Loader.
##
## Input :region(0 or 1).
##
## Output :0(failure) or 1(success).
##
sub oem_initialise_bios {
    $self = shift;
    my $region = shift;
    my $output;

    $log->log("Reach to Loader");
    $Obj_transit->change_state(to => "FIRMWARE", timeout => '500');
    $apiset_loader = $Sp_agent->Apiset_creation('Firmware');
    $apiset_loader->execute_raw_command(command => "priv set test");
    $apiset_loader->execute_raw_command(command => "pelog -i -g=$region");

    $log->log("Verify $region is intialised");
    $output = $apiset_loader->execute_raw_command(command => "pelog -a -g=$region");
    if ($output =~ /DataLen\((?=0)/ig) {
        $log->log("$region is Intialised properly");
        return 1;
    } else {
        $log->log("$region is not initialised properly");
        return 0;
    }
} ## end sub oem_initialise_bios

## Function to make Bios or Ontap region readable or writable from FreeBsd.
##
## Input :region(0 or 1).
##
## Output :0(failure) or 1(success).
##
sub sram_access_mode {
    my ($region, $output, $access);
    $self   = shift;
    $region = shift;
    $access = shift;

    $log->log("Access region $region to $access Mode");
    $Obj_transit->change_state(to => "SYSTEMSHELL", password => "$PASSWORD");
    $apiset_systemshell = $Sp_agent->Apiset_creation('Systemshell');
    $output =
      $apiset_systemshell->execute_raw_command(command => "sudo ssram -g $region -m -z $access");
    if ($output =~ /Previous+\s+access+\s+mode/ig) {
        return 1;
    } else {
        return 0;
    }
} ## end sub sram_access_mode

## Function to make Bios or Ontap region readable or writable from loader.
##
## Input :region(0 or 1).
##
## Output :0(failure) or 1(success).
##
sub access_bios_mode {
    my ($region, $output, $access);
    $self   = shift;
    $region = shift;
    $access = shift;

    $log->log("Access region $region to $access Mode");
    $Obj_transit->change_state(to => "FIRMWARE");
    $apiset_systemshell = $Sp_agent->Apiset_creation('Firmware');
    $apiset_loader->execute_raw_command(command => "priv set test");
    $output = $apiset_loader->execute_raw_command(command => "pelog -m -z=$access -g=$region");
    if ($output =~ /Previous+\s+access+\s+mode/ig) {
        return 1;
    } else {
        return 0;
    }
} ## end sub access_bios_mode

## Function to write different records to sram region from Loader.
##
## Input :region(0 or 1),$record_type(1,2,3,4,5),$msg(string),$size no_of_bytes,@data(bytes_written).
##
## Output :0(for wrong usage of command or inadequate params) or 1(data records written succesful).
##
sub sram_write_data_bios {
    my ($region, $record_type, $msg, $size, $data);
    $self        = shift;
    $region      = shift;
    $record_type = shift;
    $msg         = shift;
    $size        = shift;
    $data        = shift;

    $log->log("Write to $region region");
    $Obj_transit->change_state(to => "FIRMWARE");
    $apiset_loader = $Sp_agent->Apiset_creation('Firmware');
    $apiset_loader->execute_raw_command(command => "priv set test");
    $output = $apiset_loader->execute_raw_command(
                    command => "pelog -g=$region -t=$record_type -l=\"$msg\" -x=$size -w @{$data}");
    if (($output =~ /Invalid/ig) || ($output =~ /command+\s+status/ig)) {
        return 0;
    }
    return 1;
} ## end sub sram_write_data_bios

## Function to read offset data from sram region for bios or ontap from systemshell.
##
## Input :region(0 or 1),offset(valid or invalid).
##
## Output :0(for wrong usage of command or inadequate params) or command output.
##
sub sram_read_data {
    my $output;
    $self = shift;
    my $region = shift;
    my $offset = shift;
    $log->log("Read back data from $offset for region:$region");
    $apiset_systemshell = $Sp_agent->Apiset_creation('Systemshell');
    $output =
      $apiset_systemshell->execute_raw_command(command => "sudo ssram -r -g $region -o $offset");
    if ($output =~ /usage/ig) {
        croak("Not able to Execute command");
    }
    return $output;
} ## end sub sram_read_data

1;
