# $Id: //depot/prod/test/main/lib/platval/platsw/util.pm#1 $

#***********************************************************************************************************************************
## @summary Modules to boot filer if in any other state(Loader,boot menu or main mode) to appropriate state 
##
## @description
## "util.pm" helps to get the state of the filer, boot and goto system shell
##
## @synopsis
## get_filer_state($conn);
## boot_ontap($conn);
## add_license($conn,"nfs");
## exportfs($conn);
## save_envi
##
## @author              kakshay@netapp.com
#***********************************************************************************************************************************

#!/usr/local/bin/perl

use Tharn;
use boot;
use Admin::License qw(is_licensed license);
require "tharnit_lib.pl";

use strict;

use constant DEBUG			=> 1;
use constant MAX_TIMEOUT		=> 400;
use constant ONTAP_PROMPT               => "IN_ONTAP";
use constant LOADER_CFE_PROMPT          => "IN_LOADER";
use constant SYS_SHELL_PROMPT           => "IN_SYSTEM_SHELL";
use constant MAINTENANCE_PROMPT         => "IN_MAINT";
use constant SP_BOOT_MENU               => "IN_BOOT_MENU";
use constant SUCCESS                    => 1;
use constant FAILURE                    => 0;
use constant GX_BOOT_LOC		=> "os/boot/gx_boot.thpl"; 

our @prompts = ('LOADER(-\w+)?\*?>','CFE(-\w+)?\*?>',$FILER.'[\*>#%]','Selection','\*>');

#*********************************************************************************************
#Function Name: get_filer_state
#Parameters   : $conn
#Return Type  : Current state of the filer.
#               - ONTAP Prompt
#               - Loader Prompt
#               - Maintenance Prompt
#               - System shell Prompt
#               - Boot Menu
#Description  : Retrive the current status of the filer(loader,maintenance,boot menu,ontap,system shell)
#*********************************************************************************************
sub get_filer_state
{
  my ($conn) = @_;
  my (undef,undef,$after,$prompt) = say($conn,"?",-prompt,@prompts,-timeout,2000,-swallow);

  return MAINTENANCE_PROMPT if($prompt =~ /^\*>$/);
  return SP_BOOT_MENU if($prompt =~ /Selection/);
  return ONTAP_PROMPT if($after =~ /sysconfig/);
  return LOADER_CFE_PROMPT if($after =~ /printenv/);
  #return SYS_SHELL_PROMPT if($after =~ /No match/); #uncomment later
  return SYS_SHELL_PROMPT if($after =~ /Command not found/ || $after =~ /No match/);
}

#*********************************************************************************************
#Function Name: boot_ontap
#Parameters   : $conn
#Return Type  : Current state of the filer.
#               SUCCESS->1 if able to boot
#               FAILURE->0 if unable to boot
#Description  : Boots the filer if in any other states(loader,boot menu or in main mode)
#*********************************************************************************************
sub boot_ontap
{
  my ($conn) = @_;

  my $filer_state = get_filer_state($conn); 

  if($filer_state !~ ONTAP_PROMPT)
  {
    tstlog -comment, "Going to ONTAP prompt\n";
    subtest(GX_BOOT_LOC, "TASK=reboot");

    $filer_state = get_filer_state($conn);
    return FAILURE if($filer_state !~ ONTAP_PROMPT || $filer_state !~ SYS_SHELL_PROMPT);
  }
  return SUCCESS;
}

#********************************************************************************************************
#Function Name: goto_systemshell
#Parameters   : $conn
#Return Type  : SUCCESS -> 1, if logged into to the systen shell
#               FAILURE -> 0, unable to login to system shell
#Description  : Login to the system shell, regardless of the state(loader,maintenance,boot menu,ontap,system shell)
#*********************************************************************************************************
sub goto_systemshell
{
  my ($conn) = @_;

  while(1)
  {
    my $filer_state = get_filer_state($conn);

    if($filer_state =~ LOADER_CFE_PROMPT || $filer_state =~ MAINTENANCE_PROMPT || $filer_state =~ SP_BOOT_MENU)
    {
      tstlog -comment, "IN LOADER/MAINTENANCE PROMPT/BOOT MENU";
      boot_ontap($conn);
      #tstlog -comment, "Booting Ontap";

      #transit_to($conn,"UP");
      #sleep 30;
      next;
    }
    if($filer_state =~ SYS_SHELL_PROMPT)
    {
       tstlog -comment, "IN SYSTEM SHELL";
       return SUCCESS;
    }
    if($filer_state =~ ONTAP_PROMPT)
    {
       tstlog -comment, "IN ONTAP PROMPT";
       tstlog -comment, "Going to system shell";

       my $h = host($FILER);

       my $freeBSD_user = $h->{'FREEBSD_USER'};
       my $freeBSD_pass = $h->{'FREEBSD_PASS'};

       say($conn,"priv set test; systemshell", -prompt,"login:");
       say($conn,$freeBSD_user,-prompt,"Password:");
       my (undef,undef,undef,$prompt) = say($conn,$freeBSD_pass,-prompt,@prompts);
       next;
    }
    return FAILURE;
  } #End of while(1)

}

#----------------------------------------------------------------------------------------------
sub wait_boot_ontap
{
   my ($conn) = @_;

   my $state;
   my $flag = 1;

   our @prompts = ('LOADER(-\w+)?\*?>','CFE(-\w+)?\*?>',$FILER.'[\*>#%]','Selection','\*>');

   do
   {
     my $match;
     if($flag)
     {
       my @waitstr = ('Starting AUTOBOOT press Ctrl-C to abort','login:','Password:','Login incorrect');

       my (undef,undef,undef,$undef) = say($conn,"?",'-noprompt');

       (($match,undef,undef,undef) = say($conn,"",'-noprompt','-wait',@waitstr,@prompts,'-timeout',MAX_TIMEOUT));

       if($match =~ "Starting AUTOBOOT press Ctrl-C to abort")
       {
         say($conn,"\x03",'-noprompt');
         $flag = 0;
         tstlog -comment, "DEBUG->FLAG:$flag" if(DEBUG);
       }

       subtest(GX_BOOT_LOC, "TASK=shutdown_cold") if($match =~ "Login:" || $match =~ "Password:" || $match =~ "Login incorrect");
     }# End of if($flag)

     $state = get_filer_state($conn);
     tstlog -comment,"DEBUG:state->$state" if(DEBUG);

     boot_ontap($conn) if($state =~ LOADER_CFE_PROMPT);
   }while($state !~ LOADER_CFE_PROMPT && $state !~ ONTAP_PROMPT && $state !~ SYS_SHELL_PROMPT);

   $state = get_filer_state($conn);
   if($state =~ SYS_SHELL_PROMPT || $state =~ ONTAP_PROMPT) #Need to do login to FreeBSD
   {
     tstlog -comment, "Succesfully Booted ONTAP";
     sleep 10;  #-------------------------------> Uncomment later

     ($state = goto_systemshell($conn)) if ($state =~ ONTAP_PROMPT);
      $state = get_filer_state($conn);

     return SUCCESS if($state =~ SYS_SHELL_PROMPT);
   } #End of
   else
   {
     tstlog -FAIL, "Unable to Boot ONTAP";
     return FAILURE;
   }
}
#***************************************************************************
#Function Name: save_envi
#Parameters   : $conn,connection handel to the filer
#               $env_value, environment variable and its corresponding value
#Return Type  : SUCCESS -> 1
#               FAILURE -> 0
#Description  : Sets an environment variable either in the loader/filer prompt
#***************************************************************************
sub save_envi
{
  my ($conn,$env_value) = @_;
  my $prompt;
  my $state = get_filer_state($conn);
  if($state =~ LOADER_CFE_PROMPT)
  {
    (undef,undef,undef,$prompt) = say($conn,"setenv $env_value",-prompt,@prompts);
  }
  elsif($state =~ SYS_SHELL_PROMPT)
  {
    say($conn,"logout", -blind, -noprompt);
    sleep 5;
    (my $match,undef,my $out,$prompt) = say($conn,"\n",-blind,-noprompt,-prompt,@prompts,-timeout,100);
    (undef,undef,undef,$prompt) = say($conn,"",-prompt,@prompts,-timeout,100);
    (undef,undef,undef,$prompt) = say($conn,"priv set test; bootargs set $env_value",-prompt,@prompts) if($prompt);
  }
  elsif($state =~ MAINTENANCE_PROMPT || $state =~ SP_BOOT_MENU)
  {
    #transit_to($conn,"UP");
    subtest(GX_BOOT_LOC, "TASK=normal");
    sleep 30;
    (undef,undef,undef,$prompt) = say($conn,"priv set test; bootargs set $env_value",-prompt,@prompts);
  }
  elsif($state =~ ONTAP_PROMPT)
  {
    (undef,undef,undef,$prompt) = say($conn,"priv set test; bootargs set $env_value",-prompt,@prompts);
  }
  return SUCCESS if($prompt);
}

#*********************************************************************************************
#Function Name: add_license
#Parameters   : $conn
#		$service -> license to add for the corresponding service
#Return Type    SUCCESS->1 if license add command successful
#               FAILURE->0 unable to add license
#Description  : Retrive the license code and add the same
#*********************************************************************************************
sub add_license
{
  my($conn,$service) = @_;

  say($conn,"priv set test");
  my (undef,undef,$after,undef) = say($conn,"secret_code 1 $service",-prompt,@prompts,-swallow);
  my @sc_buf = split("\n",$after);
  my (undef,$license_code) = split(/=/,$sc_buf[3]);

  (undef,undef,$after,undef) = say($conn,"license",-prompt,@prompts,-swallow);
   return SUCCESS if($after =~ /$license_code/);

  (undef,undef,$after,undef) = say($conn,"license add $license_code",-prompt,@prompts,-swallow);
  return SUCCESS if($after =~ /license has been installed/);
}

#*********************************************************************************************
#Function Name: exportfs
#Parameters   : $conn
#Return Type  : SUCCESS->1 if paths are successfully exported
#               FAILURE->0 unable to export
#Description  : Checks if paths are exported else adds nfs license and exports the same
#*********************************************************************************************
sub exportfs
{
  my ($conn) = @_;

  say($conn,"priv set test",-prompt,@prompts,-swallow);
  my(undef,undef,$after,undef) = say($conn,"exportfs",-prompt,@prompts,-swallow);

  #my $ret = add_license($conn,"nfs");
  license($conn, 'nfs') or return FAILURE; #using existing lib. 

  my @buf = split("\n",$after);
  return SUCCESS if(($buf[1] =~ /\/vol\/vol0\/home/) && ($buf[2] =~ /\/vol\/vol0/));


  say($conn,"exportfs -p sec=sys,rw,anon=0,nosuid /vol/vol0",-prompt,@prompts,-swallow);
  say($conn,"exportfs -p sec=sys,rw,anon=0,nosuid /vol/vol0/home",-prompt,@prompts,-swallow); 
  my(undef,undef,$after,undef) = say($conn,"exportfs",-prompt,@prompts,-swallow); 

  return SUCCESS if(($after =~ /\/vol\/vol0\/home/) && ($after =~ /\/vol\/vol0/));
}
1;
