# $Id: //depot/prod/test/main/lib/platval/platsw/vespa_memerr.pm#1 $
#***********************************************************************************************************************************
## @summary 	This is a library for invoking subroutines useful for Memory error injection testcases and also for SPD.
##
## @description
## "memerr.pm" This has various modules for address generation,testcase description etc for validating Memory error injection. 
##
## SUB ROUTINES ::
##
## 1.find_address_range
## This module helps in getting all the valid address ranges for a particular filer.
##
## 2.load_memerrinject_testcases
## This gives all the testcases for memory error injections which can be covered by automation.
##
## 3.execute_memerr_testcase
## This executes the listed testcases according to the parameters specified.
##
## 4.decimal_to_hex_alligned
## This is for getting the decimal number in hex format both and also in 64 byte alligned format.
## 5.inject_error
## This is for inputting the error command for proper error generation
## 
## 6.cecc_count_memerr_v
## This module gives the number of cecc errors reported and the last error address.
## 
## 7.generate_random_ecc_mask
## This is for generating a random ecc mask according to the condition that we want for correctable or uncorrectable error.
## 
## 8.generate_random_address
## This is for generating a random address within the ranges for the specific filer or outside the range if we want an incorrect address.
## 
## 9.verify_Testcases_MEI
## This is for selecting some testcases out of the 'load_memerrinject_testcases', but we are selecting all always.
## 
## 10.start_stress
## This is for running stress in the background.
##
## @synopsis
## use vespa_memerr.pm
##
## @keywords
## memerr,platsw
##
## @author              divyab@netapp.com
#***********************************************************************************************************************************
#!/usr/local/bin/perl


use vars qw/$addr_range_start_1,$addr_range_start_2,$addr_range_start_3,$addr_range_end_1,$addr_range_end_2,$addr_range_end_3,$Sleep_Time,$Logger_disable,$Err_DIMM/;
#use strict;

use constant MAX_DIMM			=> 12;
use constant DEBUG			=> 1;
use constant TIMEOUT 			=> 800;
use constant SUCCESS                    => 1;
use constant FAILURE                    => 0;
$Sleep_Time=30;

our @prompts = ('LOADER(-\w+)?\*?>','CFE(-\w+)?\*?>',$FILER.'[\*>#%]','Selection','\*>');

#*********************************************************************************************
#Function Name: find_address_range
#Parameters   : $FILER
#Return Type  : N/A 
#Description  : This module helps in getting all the valid address ranges for a particular filer.
#*********************************************************************************************

sub find_address_range($)
{	my $FILER= shift;
	my $fc = rlm_make_conn_to_ontap($FILER);
	say($fc,"priv set test",-prompt,@prompts,-swallow,-timeout,30);
	logcomment("Finding the proper address ranges for injecting errors\n");
	(undef,undef,$output,undef)=say($fc,"mhostexec sudo memerrinject -t 1 c -16",-prompt,@prompts,-swallow,-timeout,30);
	if($output=~/\[(.+)\, (.+)\)\s+\[(.+)\, (.+)\)\s+\[(.+)\, (.+)\)/)
	{	$addr_range_start_1=$1;
		$addr_range_end_1=$2;
		$addr_range_start_2=$3;
		$addr_range_end_2=$4;
		$addr_range_start_3=$5;
		$addr_range_end_3=$6;
	}
	else
		{logresult('FATAL',"Could not get proper address ranges\n");}

}
#*********************************************************************************************
#Function Name: load_memerrinject_testcases
#Parameters   : N/A
#Return Type  : N/A
#Description  : This gives all the testcases for memory error injections which can be covered by automation.
#*********************************************************************************************

sub load_memerrinject_testcases()
{
#	The fields of this hash are :: 
#	Testcase description
#	Error type : Correctable = "c",uncorrectable ="u",not specific=''
#	Address for errors : Random address=0, user defined address =1,user defined non 16 byte alligned address=-1,user defined out of range address=-2
#	ECC mask : Mask for correctable error=1, No Mask=0, Mask for uncorrectable error=2
#	Triggering of Errors : Error to be triggered='', Error not to be triggered="nt"
#	Does not apply for testcase 13-16.		
	%memerrinject_testcases=
	(	
	1=>["Testcase 1 : Inject correctable errors at random locations",c,0,0],
	2=>["Testcase 2 : Inject correctable errors at valid memory location",c,1,0],
	3=>["Testcase 3 : Inject uncorrectable errors at random locations",u,0,0],
	4=>["Testcase 4 : Inject uncorrectable errors at valid memory locations",u,1,0],
	5=>["Testcase 5 : Give a user-defined mask for correctable errors and inject errors at random locations",'NA',0,1],
	6=>["Testcase 6 : Give a user-defined mask for correctable errors and inject errors at valid memory locations",'NA',1,1],
	7=>["Testcase 7 : Give a user-defined mask for uncorrectable errors and inject errors at random locations",'NA',0,2],
	8=>["Testcase 8 : Give a user-defined mask for uncorrectable errors and inject errors at valid memory locations",'NA',1,2],
	9=>["Testcase 9 : Give a user-defined mask for correctable errors and inject errors at valid memory locations without trigger",'NA',1,1,"nt"],
	10=>["Testcase 10 : Give a user-defined mask for uncorrectable errors and inject errors at valid memory locations without trigger",'NA',1,2,"nt"],
	11=>["Testcase 11 : Inject correctable/uncorrectable error on a non-16byte alligned address",'NA',-1,0],
	12=>["Testcase 12 : Inject correctable/uncorrectable error on a out of range addres",'NA',-2,0],
	13=>["Testcase 13 : Change error detection polling time and inject correctable errors","1>2>5>6"], 
	14=>["Testcase 14 : Disable the logger and then enable it back and inject correctable errors","1>2>5>6"],
	15=>["Testcase 15 : Disable the logger and inject errors and then enable the logger back again","1>2>5>6"],
	16=>["Testcase 16 : Take input from user correctable or uncorrectable and the address","N/A"],
	);
}

#*********************************************************************************************
#Function Name: execute_memerr_testcase
#Parameters   : $testcase="testcase number",$FILER="filer connection handle"
#Return Type  : N/A
#Description  : This executes the listed testcases according to the parameters specified in the hash for testcases.
#*********************************************************************************************

sub execute_memerr_testcase($$)
{
	my $testcase= shift;
	my $FILER= shift;
	my $Mask,$Num_hex,$Num_64_alligned;
	my $fc = rlm_make_conn_to_ontap($FILER);
	if($testcase >=1 && $testcase<=12)
		{($err_command,$Mask,$Num_hex,$Num_64_alligned)=inject_error($memerrinject_testcases{$testcase});}
		logcomment("The error command line is :: $err_command\n");
	if($testcase==16)
	{
		if($ERR_COMMAND=~/^([c|u])\,(.+)/)
		{	$ERR_COMMAND="mhostexec sudo memerrinject -t $Number_of_Err $1 $2";
			$testcase=2 if($1 eq 'c');
			$testcase=4 if($1 eq 'u');
			$err_command=$ERR_COMMAND;
		}
		else
		{	logcomment("The format of the err command is wrong so skipping this testcase\n");
			return;
		}
	}
	if($testcase >=5 && $testcase <=10)
	{	say($fc,'mhostexec sudo sysctl sysvar.memerrinject.arch.raw_mode=1',-prompt,@prompts,-swallow,-timeout,30);
		say($fc,"mhostexec sudo sysctl sysvar\.memerrinject\.arch\.ecc_mask\=$Mask",-prompt,@prompts,-swallow,-timeout,30);
	}
	(undef,undef,$Date_before,undef)=say($fc,"mhostexec sudo date \+\%s",-prompt,@prompts,-swallow,-timeout,30);
	($Date_before = $1) if($Date_before =~ /([\d]+)\s+$/);
	logcomment("date before is :: $Date_before\n");
	if($testcase=~/^[1|2|5|6|9]$/)
	{	($err_count_before,$last_err_addr_before,$last_err_time_before)=cecc_count_memerr_v($FILER);
		$match_string='Injected(and triggered | )(.*)correctable error at (.+) address (.+) with ECC mask (\S+).'; 
		($Match,$Before,$After,$Prompt)=say($fc,"$err_command",-prompt,@prompts,'-wait',$match_string,-swallow,-timeout,60);
		$Match=~/Injected(and triggered | )(.*)correctable error at (.+) address (.+) with ECC mask (\S+)./;
		$Address_err=$4;
		$ECC_Mask=$5;
		logcomment ("The address where error was injected is : $Address_err\n The ECC_Mask is : $ECC_Mask\n");
		sleep($Sleep_Time);
		($err_count_after,$last_err_addr_after,$last_err_time_after)=cecc_count_memerr_v($FILER);
		if($testcase==9)
		{	logresult('FAIL',"Logger count increased even though error was not triggered\n") unless($err_count_after==$err_count_before);
			($Match,$Before,$After,$Prompt)=say($fc,"$err_command",-prompt,@prompts,'-wait',$match_string,-swallow,-timeout,60);
			$Match=~/Injected(and triggered | )(.*)correctable error at (.+) address (.+) with ECC mask (\S+)./;
                	$Address_err=$4;
                	$ECC_Mask=$5;
			sleep($Sleep_Time);
			($err_count_after,$last_err_addr_after,$last_err_time_after)=cecc_count_memerr_v($FILER);
		}
		if($Logger_disable==0)
		{	
			if($Num_hex)
                		{logresult('FAIL',"Adress of injection is not correct \n") unless(($Address_err=~/$Num_hex/i) && ($last_err_addr_after=~/$Num_64_alligned/i));}
                	#logcomment("$Address_err &$Num_hex n$last_err_addr_after v$Num_64_alligned");
			unless($err_count_after>$err_count_before && $err_count_after<=($err_count_before+$Number_of_Err))
			{logresult('FAIL',"Memerr is not reporting the number of errors properly\n");}
			logcomment("The date after injection of error is :$last_err_time_after\n The date before injection of error is :$Date_before\n");
			unless($last_err_time_after>$Date_before && $last_err_time_after<=($Date_before+90))
			{logresult('FAIL',"Time of error reported by Memerr -v is not correct\n");}
		}
		elsif($Logger_disable==1)
		{	if($err_count_after!=$err_count_before)
				{logresult('FAIL',"Error count is updating even when logger is disabled\n");}
		}
	}
	if($testcase=~/^([3|4|7|8]|10)$/)
	{	
		#Jiljith: Edited match_string to include hyphen (or space) after string 'DIMM'
		$match_string='PANIC  \: ECC error at DIMM[\s-](NV)?\d*: \S+,ADDR (\S)+,';
		undef $Address_err; 
		if($testcase==10)
		{	$match_string_1='Injected (.*)correctable error at (.+) address (.+) with ECC mask (\S+).';
                	($Match,$Before,$After,$Prompt)=say($fc,"$err_command",-prompt,@prompts,'-wait',$match_string_1,-swallow,-timeout,60);
                	$Match=~/Injected (.*)correctable error at (.+) address (.+) with ECC mask (\S+)./;
                	$Address_err=$3;
		}
		#Jagadeesh 5-Jul-11: Increased timeout from 360 to 960 as sometimes Dumpcore taking long time to reach LOADER fw prompt.
		($Match,$Before,$After,$Prompt)=say($fc,"$err_command",-prompt,@prompts,'-wait',$match_string,-swallow,-timeout,960);
		
		#Jiljith: Edited $Match to include hyphen or space, or 'NV' after string 'DIMM'
		$Match=~/PANIC  \: ECC error at DIMM[\s-](NV)?(\d*): \S+,ADDR (\S+),/;
		$Address_err=$3;
		$Err_DIMM=$2;
		#logcomment("Address_err=$Address_err and Num_64_alligned=$Num_64_alligned\n");
		if($Num_hex)
		{logresult('FAIL',"Address of injection is not correct \n") unless($Address_err=~/$Num_64_alligned/i);}
		rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
		start_stress() if $STRESS;
		say($fc,"priv set test",-prompt,@prompts,-swallow,-timeout,30);
	}
	if($testcase >=5 && $testcase <=10)
	{	($Match,$Before,$After,$Prompt)=say($fc,'mhostexec sudo sysctl sysvar.memerrinject.arch.raw_mode',-prompt,@prompts,-swallow,-timeout,30);
		if($After=~/sysvar.memerrinject.arch.raw_mode: 1/)
		{	($Match,$Before,$After,$Prompt)=say($fc,'mhostexec sudo sysctl sysvar.memerrinject.arch.raw_mode=0',-prompt,@prompts,-swallow,-timeout,30);
		}

	}	
	if($testcase==11)
	{	$match_string='Error: provided address \S+ is not 16-byte aligned.';
		($Match,$Before,$After,$Prompt)=say($fc,"$err_command",-prompt,@prompts,'-wait',$match_string,-swallow,-timeout,60);
        }
	if($testcase==12)
	{	$match_string='Error: provided address \S+ is not valid. Valid physical address ranges are as follows:';
		($Match,$Before,$After,$Prompt)=say($fc,"$err_command",-prompt,@prompts,'-wait',$match_string,-swallow,-timeout,60);
        }
	if($testcase=~/^(13|14|15)$/)
	{	my $i=1;
		while($i<7)
		{logcomment("Iteration number $i\n");
		if($testcase==13)
		{	$Sleep_Time=80;
			($Match,$Before,$After,$Prompt)=say($fc,'mhostexec sudo sysctl machdep.cecc_period_ms',-prompt,@prompts,-swallow,-timeout,30);
		        if($After=~/machdep.cecc_period_ms: (\d+)/)
				{logcomment("The polling period is : $1 ms\n");}
			else
				{logresult('FAIL',"The polling time was not displayed\n");}
			($Match,$Before,$After,$Prompt)=say($fc,'mhostexec sudo sysctl machdep.cecc_period_ms=60000',-prompt,@prompts,-swallow,-timeout,30);
			if($After=~/machdep.cecc_period_ms: \d+ -> 60000/)
				{logcomment("The polling time was changed successfully to 60 seconds\n");}
		}
		if($testcase==14 || $testcase==15)
		{	($Match,$Before,$After,$Prompt)=say($fc,'mhostexec sudo sysctl machdep.cecc_enabled=0',-prompt,@prompts,'-wait','machdep.cecc_enabled: \d \-\> 0',-swallow,-timeout,30);
			sleep (60);
		}
		if($testcase==14)
			{($Match,$Before,$After,$Prompt)=say($fc,'mhostexec sudo sysctl machdep.cecc_enabled=1',-prompt,@prompts,'-wait','machdep.cecc_enabled: \d \-\> 1',-swallow,-timeout,30);}
		if($i==3)
			{$i=$i+2;}
		if($testcase==15)
			{$Logger_disable=1;}
		logcomment("executing testcase $memerrinject_testcases{$i}->[0] as a subtest for this testcase\n");
		execute_memerr_testcase($i,$FILER);
		if($testcase==13)
		{	($Match,$Before,$After,$Prompt)=say($fc,'mhostexec sudo sysctl machdep.cecc_period_ms=10000',-prompt,@prompts,-swallow,-timeout,30);
			   if($After=~/machdep.cecc_period_ms: \d+ -> 10000/)
                                {logcomment("The polling time was changed back successfully to 10 seconds\n");}
                }
			
		if($testcase==15)
		{	$Logger_disable=0;
			($err_count_before,$last_err_addr_before,$last_err_time_before)=cecc_count_memerr_v($FILER);
			($Match,$Before,$After,$Prompt)=say($fc,'mhostexec sudo sysctl machdep.cecc_enabled=1',-prompt,@prompts,'-wait','machdep.cecc_enabled: \d \-\> 1',-swallow,-timeout,30);
			sleep (30);
			($err_count_after,$last_err_addr_after,$last_err_time_after)=cecc_count_memerr_v($FILER);
			logresult('FAIL',"Logger did not update properly after it was enabled\n") unless($err_count_after>$err_count_before);
		}$i++;
		}
			
	}
		
}	

#*********************************************************************************************
#Function Name: decimal_to_hex_alligned
#Parameters   : $Num="number in decimal format"
#Return Type  : $Num_hex="number in hex format",$Num_64_alligned="number in hex format and 64 byte alligned
#Description  : This is for getting the decimal number in hex format both and also in 64 byte alligned format.
#*********************************************************************************************

sub decimal_to_hex_alligned($)
{
	$Num= shift;
	my $Num_64_alligned,$Num_hex;
	(undef,undef,$After)=say($cc,"echo \"obase\=16\; $Num\" | bc");
	if($After=~/(\S+)/)
	{	$Num_hex=$1;
 	}
	$Num_64_alligned=$Num -($Num%64);
	(undef,undef,$After)=say($cc,"echo \"obase\=16\; $Num_64_alligned\" | bc");	
	if($After=~/(\S+)/)
        {	$Num_64_alligned=$1;
	logcomment("The number in hexadecimal format and 64byte alligned is : $Num_hex,$Num_64_alligned \n");
	}
	return ($Num_hex,$Num_64_alligned);
}	

#*********************************************************************************************
#Function Name: inject_error
#Parameters   : Testcase="This takes the testcase number from the hash as input"
#Return Type  : $err_command="The command for injecting error according to the testcase",$Mask="The mask to be set for the particular testcase",$Num_hex="number in hex format",$Num_64_alligned="number in hex format and 64 byte alligned
#Description  : This is for inputting the error command for proper error generation
#*********************************************************************************************

sub inject_error($)
{
	my ($title,$err_type,$err_addr,$ecc_mask,$err_trigger)=@{$_[0]};
	#logcomment(" The testcase being executed is : $title");
	my $address_inject,$Mask;
	my $err_command='mhostexec sudo memerrinject';
	if($err_trigger ne "nt")
		{$err_command=$err_command . ' -t';}
	$err_command= $err_command . " $Number_of_Err";
	if($err_type ne 'c' && $err_type ne 'u')
	{	$err_type=int rand(1.5);
		print("Generating c or u randomly\n");
		if($err_type == 1)
			{$err_type = 'c';}
		else
			{$err_type = 'u';}
	}
	$err_command= $err_command . " $err_type";
	#logcomment("$err_command");
	if($ecc_mask)
		{$Mask=generate_random_ecc_mask($ecc_mask);}
		$Mask='0x'.$Mask;
	if($err_addr==0)
		{	logcomment("$err_command,$address_inject");
			return ($err_command,$Mask);}
	elsif($err_addr==1)
		{$address_inject=generate_random_address(0);}
	elsif($err_addr==-1)
		{$address_inject=generate_random_address(1);}
	elsif($err_addr==-2)
		{$address_inject=generate_random_address(0,5);}
	$err_command= $err_command . " $address_inject";
	if($err_addr!=0)
		{($Num_hex,$Num_64_alligned)=decimal_to_hex_alligned($address_inject);}
	#logcomment("$err_command,$address_inject");
	return ($err_command,$Mask,$Num_hex,$Num_64_alligned);
}	

#*********************************************************************************************
#Function Name: cecc_count_memerr_v
#Parameters   : $FILER="filer connection handle"
#Return Type  : $err_count="number or errors given by logger",$last_err_addr="address of the last cecc error",$last_err_time="time of occurence of the last cecc error"
#Description  : This module gives the number of cecc errors reported and the last error address.
#*********************************************************************************************

sub cecc_count_memerr_v($)
{	my $FILER= shift;
	my $err_count,$last_err_addr,$last_err_time;
        my $fc = rlm_make_conn_to_ontap($FILER);
        say($fc,"priv set test",-prompt,@prompts,-swallow,-timeout,30);
	(undef,undef,$output,undef)=say($fc,"mhostexec memerr -v",-prompt,@prompts,-swallow,-timeout,60);
	if($output=~/([0-9]+) error\(s\) occurred since boot/)
		{$err_count=$1;}
	elsif($output=~/No CECC errors since boot/)
		{$err_count=0;}
	else
		{logresult('FATAL',"Memerr -v is not giving expected result\n");}
	if($err_count && $output=~/DIMM-(\S+), Physical address:(.+)\, Time\: \[([0-9]+)\]\s+$/)
	{	$last_err_addr=$2;
		$last_err_time=$3;
		$Err_DIMM=$1;
	}
	elsif($err_count)
		{logresult('FATAL',"Memerr -v is not properly showing the address and timestamp for cecc error locations\n");}
	return ($err_count,$last_err_addr,$last_err_time,$Err_DIMM);
}

#*********************************************************************************************
#Function Name: generate_random_ecc_mask
#Parameters   : $err_type="type of ecc-mask we want ot generate 0 is for correctable 1 is for uncorrectable error" -Optional parameter.
#Return Type  : ecc_mask="gives the ecc mask to be used"
#Description  : This is for generating a random ecc mask according to the condition that we want for correctable or uncorrectable error.
#*********************************************************************************************

sub generate_random_ecc_mask(;$)
{
	my $i;
	$err_type= $_[0] ? $_[0] : '1';	
	$loop=$err_type;
	my $random_half;
	my $ecc_mask;
	while($loop>0)
	{for($x=0;$x<=3;$x++)
	{$i=int rand(16);
	$i=sprintf("%x",$i);
	$random_half=$random_half.$i;
	}
	$loop--;
	}
	$ecc_mask=$random_half;	
	if($err_type==1)
	{	
	if($i){$ecc_mask=$ecc_mask."0000";}
	else {$ecc_mask="0000".$ecc_mask;}
	}	
	return $ecc_mask;
}

#*********************************************************************************************
#Function Name: generate_random_address
#Parameters   : $non_16_byte_alligned=" if this is one a non-16 byte address will be generated",$chose_addr_range="if it is 1 we chose from address range 1 if it is 2 we chose from address range 2 if it is 3 we chose from address range 3 if it is 4 we take an address for DIMM2 if it is 5 we take an out of range addres"
#Return Type  : $address_inject="address of injecting error"
#Description  : This is for generating a random address within the ranges for the specific filer or outside the range if we want an incorrect address.
#*********************************************************************************************

sub generate_random_address(;$$)
{	logcomment("Generating random addresses\n");
	($non_16_byte_alligned,$chose_addr_range)=@_;
	$address_range_1=hex("$addr_range_end_1") - hex("$addr_range_start_1");
	$address_range_2=hex("$addr_range_end_2") - hex("$addr_range_start_2");
	$address_range_3=hex("$addr_range_end_3") - hex("$addr_range_start_3");
	if(!$chose_addr_range) 
		{$chose_addr_range = int rand(2.5)+1;}
	if ($chose_addr_range == 1)
	{	logcomment("Taking addresses from range 1\n");
		$address_inject= int rand ($address_range_1);
		$address_inject=$address_inject + hex("$addr_range_start_1");
	}
	elsif($chose_addr_range ==2)
	{	logcomment("Taking addresses from range 2\n");
		$address_inject= int rand($address_range_2);
		$address_inject=$address_inject + hex("$addr_range_start_2");
	}
	elsif($chose_addr_range ==3)
	{	logcomment("Taking addresses from range 3\n");
		$address_inject= int rand($address_range_3);	
		$address_inject=$address_inject + hex("$addr_range_start_3");
        }
	elsif($chose_addr_range ==4)
	{logcomment("This is to be used only for VESPA for injecting errors in DIMM-2\n");
	$address_inject=hex("0x23FFFFFC1");
	}
	elsif($chose_addr_range ==5)
	{	logcomment("Taking addresses from out of range\n");
		$address_range_4=hex("$addr_range_start_2")-hex("$addr_range_end_1");
		$address_inject= int rand($address_range_4);
		$address_inject=$address_inject + hex("$addr_range_end_1");
	}
	else
	{logcomment("Wrong address range generated or selected\n");}
	if($non_16_byte_alligned==0)
		{$address_inject=$address_inject-($address_inject%16);}
	elsif($non_16_byte_alligned==1 && ($address_inject%16)==0)
		{$address_inject++;}
	#$address_inject=sprintf("%x","$address_inject");
	print(" Address where to be injected :: $address_inject\n");
	return  $address_inject;
	
}

#*********************************************************************************************
#Function Name: verify_Testcases_MEI
#Parameters   : N/A
#Return Type  : N/A
#Description  : This is for selecting some testcases out of the 'load_memerrinject_testcases', but we are selecting all always.
#*********************************************************************************************

sub verify_Testcases_MEI()
{	logcomment("Running all testcases for Memory Error Injections\n");
}

#*********************************************************************************************
#Function Name: start_stress
#Parameters   : N/A
#Return Type  : N/A
#Description  : This is for running stress in the background.
#*********************************************************************************************

sub start_stress
{
  say($fc,"priv set test",-prompt,@prompts,swallow);
  say($fc,"disk scrub start",-prompt,@prompts,-swallow);
  say($fc,"hammer -f -1 /vol/vol0/filename 1234",-prompt,@prompts,-swallow);
  say($fc,"filersio asyncio_active 00 -r 100 4k 0 7g 84000 5 /vol/vol0/file.90g -create",-prompt,$host->{'prompt_norm'},-swallow);
  sleep 10;


  my(undef,undef,$load,undef) = say($fc,"sysstat -c 1 1",-prompt,@prompts,-swallow);
  tstlog -comment,"Load is:$load";
}


1;
