# $Id: //depot/prod/test/main/lib/platval/platsw/spd.pm#1 $
#***********************************************************************************************************************************
## @summary     Modules developed to support validating MES region of the SPD
##
## @description
## "spd.pm" helps in validating the MES info when memory error are injected. Currently validates the signature, version, error counts
## when particular error is injected.
##
## @synopsis
##
## update_date_time
## enable_dimm
## read_spd
## write_spd
## init_spd
## read_spd_region
## unset_histo
## spd_val_correctable
## corr_mes_crc
## read_spd_uncorrectable
## rd_wr_spd
## get_filer_model
## populate_DIMM_number_addresses
## read_spd_data
## write_spd_data
## execute_spd_testcase
## @author              divyab@netapp.com
#***********************************************************************************************************************************
#!/usr/local/bin/perl

use Tharn;
use boot;
#use platval::platsw::util;
require "tharnit_lib.pl";

#use strict;
use vars qw($ERR_TYPE $N_ERR $TRIG $ADDR $ENABLE_DIMM $INIT_SPD $CORR_CRC_MES $DATE_CHANGE $DISABLE_CECC $REBOOT $POWER_CYCLE $UNSET_HISTG $READ $WRITE $DIMM $BYTES $OFFSET $WR_DATA $filer_model $MAX_DIMM @DIMM_address $Number_of_DIMM $BUS);


use constant ABSOLUT_V                  => "FAS6210";
use constant ABSOLUT_M                  => "FAS6280";
use constant ABSOLUT_L                  => "FAS6240";
use constant VESPA_2U                   => "FAS2240-2";
use constant VESPA_4U                   => "FAS2240-4";
use constant RUCKUS                     => "FAS2220";
use constant DEBUG                      => 1;
use constant TIMEOUT                    => 800;
use constant SIGNATURE                  => "MES";
use constant VERSION_NO                 => "0x1";
use constant SUCCESS                    => 1;
use constant FAILURE                    => 0;
use constant GX_BOOT_LOC                => "os/boot/gx_boot.thpl";
use constant MAX_OFFSET                 => 254;

param("ERR_TYPE","-default","c");
param("N_ERR","-default","1");
param("TRIG","-default","1");
param("ADDR","-default","");
param("ENABLE_DIMM","-default",0);
param("INIT_SPD","-default",0);
param("CORR_CRC_MES","-default",0);
param("DISABLE_CECC","-default",0);
param("DATE_CHANGE","-default","");
param("REBOOT","-default",0);
param("UNSET_HISTG","-default",0);
param("POWER_CYCLE","-default",0);
param("READ","-default",0);
param("WRITE","-default",0);
param("WR_DATA","-default",1);
param("DIMM","-default",2);
param("BYTES","-default",1);
param("OFFSET","-default",0);

our @prompts = ('LOADER(-\w+)?\*?>','CFE(-\w+)?\*?>',$FILER.'[\*>#%]','Selection','\*>');

#*********************************************************************************************
#Function Name: update_date_time
#Parameters   : $conn           -> Filer connection
#               $update_hm      -> Number of Hour/Minute to update
#Return Type  : NA
#Description  : This module is used to update the Hours before reading SPD
#*********************************************************************************************
sub update_date_time
{
  my ($conn,$update_hm) = @_;
  my $sec;
  my $host = host($FILER);

  my $HM = chop($update_hm);

  my(undef,undef,$curr_date_time,undef) = say($conn,"date",-prompt,$host->{'prompt_norm'},-swallow);
  tstlog -comment,"Current Date:$curr_date_time\nAddition of Date and Time to: $update_hm $HM" if(DEBUG);

  ($sec = ($update_hm * 60 *60)) if($HM =~ 'H');
  ($sec = ($update_hm * 60)) if($HM =~ 'M');

  my(undef,undef,$epoch_datetime,undef) = say($conn,"mhostexec sudo date +\%s",-prompt,$host->{'prompt_norm'},-swallow);

  ($epoch_datetime = $&) if($epoch_datetime =~ m/(\d+.)/);
  tstlog -comment, "EPOCH:$epoch_datetime\nSEC:$sec" if(DEBUG);

  $update_hm = (($epoch_datetime + $sec) - 10);
  tstlog -comment,"New Epoch:$update_hm" if(DEBUG);

  my(undef,undef,$YY,undef) = say($conn,"mhostexec sudo date -r $update_hm +\%Y",-prompt,$host->{'prompt_norm'},-swallow);
  ($YY = $&) if($YY =~ /(^[\d\S.]+)/m);

  my(undef,undef,$MM,undef) = say($conn,"mhostexec sudo date -r $update_hm +\%m",-prompt,$host->{'prompt_norm'},-swallow);
  ($MM = $&) if($MM =~ /(^[\d\S.]+)/m);

  my(undef,undef,$DD,undef) = say($conn,"mhostexec sudo date -r $update_hm +\%d",-prompt,$host->{'prompt_norm'},-swallow);
  ($DD = $&) if($DD =~ /(^[\d\S.]+)/m);

  my(undef,undef,$H,undef) = say($conn,"mhostexec sudo date -r $update_hm +\%H",-prompt,$host->{'prompt_norm'},-swallow);
  ($H = $&) if($H =~ /(^[\d\S.]+)/m);

  my(undef,undef,$M,undef) = say($conn,"mhostexec sudo date -r $update_hm +\%M",-prompt,$host->{'prompt_norm'},-swallow);
  ($M = $&) if($M =~ /(^[\d\S.]+)/m);

  my(undef,undef,$S,undef) = say($conn,"mhostexec sudo date -r $update_hm +\%S",-prompt,$host->{'prompt_norm'},-swallow);
  ($S = $&) if($S =~ /(^[\d\S.]+)/m);

  my $new_datetime = "$YY"."$MM"."$DD"."$H"."$M."."$S";

  tstlog -comment,"New Date/Time:$new_datetime" if(DEBUG);

  say($conn,"mhostexec sudo date $new_datetime",-prompt,$host->{'prompt_norm'},-swallow);
  sleep 10;

}
#*********************************************************************************************
#Function Name: enable_dimm
#Parameters   : $conn           -> Filer Connection
#               $dimm_id        -> Corresponding DIMM to enable
#Return Type  : $size_before_enable -> Size of the memory before enable
#               $size_after_enable  -> Size of the memory after enabling the DIMM
#Description  : Enable a specific DIMM
#*********************************************************************************************
sub enable_dimm
{
  my ($conn,$dimm_id) = @_;
  my $host = host($FILER);
  our @loader_prompts = ('LOADER(-\w+)?\*?>','CFE(-\w+)?\*?>'); #,$FILER.'[\*>#%]','Selection','\*>');

 # my $filer_state = get_filer_state($conn);
 # $filer_state = goto_systemshell($conn) if($filer_state !~ "IN_SYSTEM_SHELL");

  #if(($filer_state == SUCCESS || $filer_state =~ "IN_SYSTEM_SHELL"))
  #{
    my $size_before_enable;
    my $size_after_enable;

    my(undef,undef,$size_before_enable,undef) = say($conn,"mhostexec sudo sysctl debug.installed_memsize",-prompt,$host->{'prompt_norm'},-swallow);
    (undef,$size_before_enable) = split(/:/,$size_before_enable);

    say($conn,"mhostexec sudo spd -n $dimm_id -o 156 -w 1 0xc",-prompt,$host->{'prompt_norm'},-swallow);
    say($conn,"logout",-prompt,$host->{'prompt_norm'},-swallow);
    subtest(GX_BOOT_LOC, "TASK=reboot");

    #$filer_state = get_filer_state($conn);
    #$filer_state = goto_systemshell($conn) if($filer_state !~ "IN_SYSTEM_SHELL");

    #if($filer_state == SUCCESS || $filer_state =~ "IN_SYSTEM_SHELL")
    #{
      my(undef,undef,$size_after_enable,undef) = say($conn,"mhostexec sudo sysctl debug.installed_memsize",-prompt,$host->{'prompt_norm'},-swallow);
      (undef,$size_after_enable) = split(/:/,$size_after_enable);

      return ($size_before_enable,$size_after_enable);
    #}
  #}
}
#*********************************************************************************************
#Function Name: read_spd
#Parameters   : $conn           -> Filer connection
#               $dimm           -> Read operation from a specified DIMM
#               $offset         -> Read operation from specified offset
#               $n              -> Number of bytes to read
#
#Return Type  : $read_val       -> Returns the read byte
#Description  : Uses spd utility to read from the corresponding DIMM at a specific offset of bytes N
#*********************************************************************************************
sub read_spd
{
  my($conn,$dimm,$offset,$n) = @_;
  my $host = host($FILER);

  if($dimm)
  {
    my(undef,undef,$read_val,undef) = say($conn,"mhostexec sudo spd -n $dimm -o $offset -r $n",-prompt,$host->{'prompt_norm'},-swallow);
    (undef,$read_val) = split(/:/,$read_val);
    $read_val=~s/\-+\s+//;
    tstlog -comment, "DEBUG->Read Value:$read_val" if(DEBUG);
    return $read_val;
  }
}
#*********************************************************************************************
#Function Name: write_spd
#Parameters   : $conn           -> Filer connection
#               $dimm           -> Write operation from particular DIMM
#               $offset         -> Write to be performed at particular offset
#               $n              -> Number of bytes to write
#               $value          -> Value to be writen
#Return Type  : SUCCESS         -> 1, if write successful
#               FAILURE         -> 0, if write unsuccessful
#Description  : Writes n bytes on a particular DIMM at a known offset
#*********************************************************************************************
sub write_spd
{
  my ($conn,$dimm,$offset,$n,$value) = @_;
  my $read_val;

  my $host = host($FILER);

  if($dimm)
  {
    say($conn,"mhostexec sudo spd -n $dimm -o $offset -w $n $value",-prompt,$host->{'prompt_norm'},-swallow);
    $read_val = read_spd($conn,$dimm,$offset,$n);

    tstlog -comment, "DEBUG->read_val:$read_val" if(DEBUG);
    return SUCCESS if($read_val =~ $value);
  }
  else
  {
    $dimm = 2 if($filer_model =~ ABSOLUT_V);
    $dimm = 1 if($filer_model =~ RUCKUS || $filer_model =~ ABSOLUT_M || $filer_model =~ ABSOLUT_L || $filer_model=~VESPA_2U ||$filer_model=~VESPA_4U);

    while($dimm <= $MAX_DIMM)
    {
      say($conn,"mhostexec sudo spd -n $dimm -o $offset -w $n $value",-prompt,$host->{'prompt_norm'},-swallow);
      $read_val = read_spd($conn,$dimm,$offset,$n);

      tstlog -comment, "DEBUG->Value:$value" if(DEBUG);

      $read_val =~ s/\s+$//;
      return FAILURE if($value !~ /$read_val/i);

      $dimm+=2 if($filer_model =~ ABSOLUT_V);
      $dimm+=1 if($filer_model =~ RUCKUS || $filer_model =~ ABSOLUT_M || $filer_model =~ ABSOLUT_L || $filer_model =~VESPA_2U ||$filer_model=~VESPA_4U);
        #logcomment("dimm number is :: $dimm\n");
    } #End of while
    return SUCCESS;
  }
}

#*********************************************************************************************
#Function Name: init_spd
#Parameters   : $conn           -> Filer connection
#Return Type  : SUCCESS         -> 1, if initializing successful
#               FAILURE         -> 0, Unsuccessful in initializing
#Description  : Initialize the MES region on all the DIMM
#*********************************************************************************************
sub reinit_spd($;$)
{
  my $FILER = shift;
  my $DIMM_num = shift;
  my @DATA=('0x0e');
  my $i=0;
  if($DIMM_num)
  {     logcomment("Reinitializing DIMM $DIMM_num\n");
        write_spd_data($FILER,\@DATA,0,156,$DIMM_num);
        }
  else
  {     logcomment("Reinitializing all the DIMMs of the FILER\n");
        while($Number_of_DIMM>$i)
        {       write_spd_data($FILER,\@DATA,0,156,$DIMM_address[$i]);
                $i++;
        }$i=0;
  }

        #(undef,undef,$output,$prompt) =say($fc,"reboot",-swallow,-prompt, 'login',@prompts,'Password','-timeout', 300);
        my $host=host($FILER);
                if($host->{hosttype} =~ /ng/){
                say($fc,"exit",-noprompt);
                (undef,undef,$output,$prompt) =say( $fc, "set -c off;system node reboot -node local", -prompt, @prompts, -swallow, -timeout, 300 );
                }else{
                (undef,undef,$output,$prompt) =say( $fc, "reboot",-noprompt, -swallow, -timeout, 300 );
                 sleep 60;
        ##(undef,undef,$output,$prompt) =say( $fc, "reboot", -prompt, @prompts, -swallow, -timeout, 300 );
                }
        if($output =~/PANIC/)
        {logresult('FATAL',"A panic was encontered in between \n");}
        rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);

        say($fc,"priv set test", -swallow, -prompt, @prompts, '-timeout', 60);
        if($DIMM_num)
{               my( undef, undef, $spd_reinit, undef )
            = say ($fc, "mhostexec sudo spd -n $DIMM_num -a",
            -prompt, @prompts, -swallow, -timeout, 60 );
        if($spd_reinit=~/Flag 0xc[\s\S]+Log Cnt:0x0[\s]+UECC cnt:0x0[\s]+UECC0 addr:0x0[\s\S]+CECC cnt:0x0[\s]+CECC0 addr:0x0[\s\S]+Daily cnt:0x0\s+Weekly cnt: 0x0\s+Jan:0x0 Feb:0x0 Mar:0x0 April:0x1\s+May:0x0 Jun:0x0 July:0x0 Aug:0x0\s+Sept:0x0 Oct:0x0 Nov:0x0 Dec:0x0/)
        {logcomment("DIMM $DIMM_num successfully reinitialized\n");}
        else
        {logresult('FAIL',"DIMM not initialized properly\n");}
        }
        else
{               while($Number_of_DIMM>$i)
        { my( undef, undef, $spd_reinit, undef )
            = say( $fc, "mhostexec sudo spd -n $DIMM_address[$i] -a",
            -prompt, @prompts, -swallow, -timeout, 60 );
        if($spd_reinit=~/Flag 0xc[\s\S]+Log Cnt:0x0\s+UECC cnt:0x0\s+UECC0 addr:0x0[\s\S]+CECC cnt:0x0\s+CECC0 addr:0x0[\s\S]+Daily cnt: 0x0\s+Weekly cnt: 0x0\s+Jan:0x0 Feb:0x0 Mar:0x0 April:0x0\s+May:0x0 Jun:0x0 July:0x0 Aug:0x0\s+Sept:0x0 Oct:0x0 Nov:0x0 Dec:0x0/)
        {logcomment("DIMM $DIMM_address[$i] successfully reinitialized\n");}
        else
        {logresult('FAIL',"DIMM not initialized properly\n");}
        $i++;
        }}


}
#*********************************************************************************************
#Function Name: read_spd_region
#Parameters   : $conn           -> Filer Connection
#               $dimm           -> To read MES region from specified DIMM
#Return Type  : %spd_out        -> Returns a hash with all the fields of the MES
#Description  : Read the MES region on a particular DIMM based on the error type
#*********************************************************************************************
sub read_spd_region
{
  my($conn,$dimm) = @_;
  my %spd_out;
  my $host = host($FILER);

  my (undef,undef,$out,undef) = say($conn,"mhostexec sudo spd -n $dimm -a",-prompt,$host->{'prompt_norm'},-swallow);

  if($ERR_TYPE =~ /c|ck|mbn/)
  {
  %spd_out = $out =~ /.+.+(Sig)\:\s([<\w>]+)\s(Version)\:\s([\dx]+)\s+(Flag)\s([\wx]+).+(Log Cnt)\:\s?([\wx]+)\s+(UECC cnt)\:([\wx]+)\s+(UECC0 addr)\:([\wx]+).+\s+(UECC1 addr)\:([\wx]+).+(CECC cnt)\:([\wx]+).+\s(CECC0 addr)\:([\wx]+).+\s(CECC1 addr)\:([\wx]+).+\s(CECC last error time)?(?:\:)?([\w\s\d\:]+)?(?:.+)?(Serial)\:([\d\w]+).+.+\s(Daily cnt)\:\s+([\wx]+).+(Weekly cnt)\:\s+([\wx]+).+\s(Jan)\:([\wx]+)\s(Feb)\:([\wx]+)\s(Mar)\:([\wx]+)\s(April)\:([\wx]+).+\s(May)\:([\wx]+)\s(Jun)\:([\wx]+)\s(July)\:([\wx]+)\s(Aug)\:([\wx]+).+\s(Sept)\:([\wx]+)\s(Oct)\:([\wx]+)\s(Nov)\:([\wx]+)\s(Dec)\:([\wx]+)/s;
  }
  elsif($ERR_TYPE =~/u|mbmn/)
  {
  %spd_out = $out =~ /.+.+(Sig)\:\s([<\w>]+)\s(Version)\:\s([\dx]+)\s+(Flag)\s([\wx]+).+(Log Cnt)\:\s?([\wx]+)\s+(UECC cnt)\:([\wx]+)\s+(UECC0 addr)\:([\wx]+).+\s+(UECC1 addr)\:([\wx]+)/s;#.+(CECC cnt)\:([\wx]+).+\s(CECC0 addr)\:([\wx]+).+\s(CECC1 addr)\:([\wx]+).+\s(CECC last error time)\:([\w\s\d\:]+).+(Serial)\:([\d\w]+).+.+\s(Daily cnt)\:\s+([\wx]+).+(Weekly cnt)\:\s+([\wx]+).+\s(Jan)\:([\wx]+)\s(Feb)\:([\wx]+)\s(Mar)\:([\wx]+)\s(April)\:([\wx]+).+\s(May)\:([\wx]+)\s(Jun)\:([\wx]+)\s(July)\:([\wx]+)\s(Aug)\:([\wx]+).+\s(Sept)\:([\wx]+)\s(Oct)\:([\wx]+)\s(Nov)\:([\wx]+)\s(Dec)\:([\wx]+)/s;
  }

  return %spd_out;
}
#*********************************************************************************************
#Function Name:
#Parameters   :
#Return Type  :
#Description  :
#*********************************************************************************************
sub display_spd
{
  my (%details) = @_;

  foreach my $k (keys %details)
  {
    tstlog -comment,"DEBUG->:$k => $details{$k}";
  }
}
#*********************************************************************************************
#Function Name: unset_histo
#Parameters   : $conn           -> Filer Connection
#               $dimm           -> Disable CECC histogram
#Return Type  : NA
#Description  : Disable CECC histogram on that corresponding DIMM
#*********************************************************************************************
sub unset_histo
{
  my ($conn,$dimm) = @_;
  my $host = host($FILER);

  say($conn,"mhostexec sudo spd -n $dimm -o 156 -w 1 0x4",-prompt,$host->{'prompt_norm'},-swallow);
}
#*********************************************************************************************
#Function Name: spd_val_correctable
#Parameters   : $conn           -> Connection to Filer
#               $err_info       -> Memory error information like(DIMM slot, address, timestamp)
#Return Type  : NA
#Description  : Validates the fields of MES when correctable errors are injected
#*********************************************************************************************
sub spd_val_correctable
{
  my($conn,$err_info) = @_;
  my $host = host($FILER);

  tstlog -comment, "Validating SPD region for Correctable Error";
  my (undef,undef,$res,undef) = say($conn,"mhostexec sudo memerr -v",-prompt,$host->{'prompt_norm'},-swallow);

  my(@memerr_info) = split(/\n/,$res);

  tstlog -comment, "DEBUG->Correctable Memory Error :$memerr_info[$#memerr_info - 1]" if(DEBUG);
  my($dimm_slot,$addr,$err_time) = split(/,/,$memerr_info[$#memerr_info - 1]);

  (undef,$dimm_slot) = split(/:/,$dimm_slot);
  tstlog -comment, "DEBUG->DIMM Slot:$dimm_slot" if(DEBUG);

  (undef,$addr) = split(/:/,$addr);
  tstlog -comment, "DEBUG->Address:$addr" if(DEBUG);

  (undef,$err_time) = split(/:/,$err_time);
  $err_time =~ s/\s+$//;
  $err_time =~ s/[\[\]]//g;

  tstlog -comment, "DEBUG->Error Time:$err_time" if(DEBUG);

  my %spd_det_old = read_spd_region($conn,$dimm_slot);

  update_date_time($conn,"7H") if(!$REBOOT && !$POWER_CYCLE);
  if($REBOOT || $POWER_CYCLE)
  {
    #say($conn,"logout",-prompt,$host->{'prompt_norm'},-swallow);
    subtest(GX_BOOT_LOC, "TASK=reboot") if($REBOOT);
    subtest(GX_BOOT_LOC, "TASK=shutdown_cold") if($POWER_CYCLE);

    #my $filer_state = get_filer_state($conn);
    #$filer_state = goto_systemshell($conn) if($filer_state !~ "IN_SYSTEM_SHELL");
  }
  unset_histo($conn,$dimm_slot) if($UNSET_HISTG);

  my %spd_det_new = read_spd_region($conn,$dimm_slot);

  display_spd(%spd_det_new) if(DEBUG);

  if($spd_det_new{'Sig'} =~ SIGNATURE && $spd_det_new{'Version'} =~ VERSION_NO)
  {
    tstlog -comment, "Signature:$spd_det_new{'Sig'} Version:$spd_det_new{'Version'}";

    tstlog '-warn', "Error in Log Count:"  if(hex($spd_det_new{'Log Cnt'}) - hex($spd_det_old{'Log Cnt'}) != 1);
    tstlog '-warn', "Error in CECC Count:"  if(hex($spd_det_new{'CECC cnt'}) - hex($spd_det_old{'CECC cnt'}) != 1);

    my (undef,undef,$mem_err_time,undef) = say($conn,"mhostexec sudo date -r $err_time",-prompt,$host->{'prompt_norm'},-swallow);
    my @corr_memerr_time = split(/\n/,$mem_err_time);
    $corr_memerr_time[-1] =~ s/GMT[\s]//;
    $spd_det_new{'CECC last error time'} =~ s/\s+$//;

    tstlog -comment,"DEBUG->corr_memerr_time:$corr_memerr_time[-1]" if(DEBUG);
    tstlog -comment,"DEBUG->SPD_last err time:$spd_det_new{'CECC last error time'}" if(DEBUG);

    tstlog '-warn', "Time Mis-match" if($corr_memerr_time[-1] !~ $spd_det_new{'CECC last error time'});

    tstlog '-warn', "Daily/Weekly Count" if((hex($spd_det_new{'Daily cnt'}) - hex($spd_det_old{'Daily cnt'}) != 1) && (hex($spd_det_new{'Weekly cnt'}) - hex($spd_det_old{'Weekly cnt'}) != 1));

    $mem_err_time =~ s/\s+$//;

    my (undef,undef,$month,undef) = say($conn,"mhostexec sudo date -r $err_time +\%b",-prompt,$host->{'promt_norm'},-swallow);

    (undef,$month) = split(/\n/,$month);
    $month =~ s/\s+$//;

    $month = "April" if($month eq "Apr");
    $month = "Sept" if($month eq "Sep");
    $month = "July" if($month eq "Jul");

    my $new_month = hex($spd_det_new{$month});
    my $old_month = hex($spd_det_old{$month});

    tstlog -comment, "DEBUG->Month:$month" if(DEBUG);
    tstlog -comment, "DEBUG->spd_det_new{$month}:$new_month---->$spd_det_new{$month}" if(DEBUG);
    tstlog -comment, "DEBUG->spd_det_old{$month}:$old_month---->$spd_det_old{$month}" if(DEBUG);

    tstlog '-warn', "Error in Updating Month's Count" if(hex($spd_det_new{$month}) - hex($spd_det_old{$month}) != 1);
  }
  else
  {
    tstlog '-fail',"Wrong Signature";
  }
}
#*********************************************************************************************
#Function Name: corr_mes_crc
#Parameters   : $conn           -> Filer Connection
#Return Type  : Returns the address of the DIMM2
#Description  : Corrupt the Signature and CRC of DIMM2
#*********************************************************************************************
sub corr_mes_crc
{
  my($conn) = @_;
  my $host = host($FILER);

  my $flag = write_spd($conn,2,153,3,"01 02 03");
  if($flag)
  {
    say($conn,"mhostexec sudo iic -b 1 -a 0xa0 -o 150 -w 2 1 2",-prompt,$host->{'prompt_norm'},-swallow);  # For now its ok to hard code the values
    say($conn,"mhostexec sudo spd -n 2 -a",-prompt,$host->{'prompt_norm'},-swallow);
    return ("0x100000080");
  }
}
#*********************************************************************************************
#Function Name:
#Parameters   :
#Return Type  :
#Description  :
#*********************************************************************************************
sub spd_val
{
  my($conn,$error_type,$err_info) = @_;
  #my $host = host($FILER);

  if($error_type =~ /c/i)
  {
    spd_val_correctable($conn,$err_info);
  }
  elsif($error_type =~ /u/i)
  {
    #spd_val_uncorrectable($conn,$err_info);
  }
}

#*********************************************************************************************
#Function Name: read_spd_uncorrectable
#Parameters   : $conn           -> Filer connection
#Return Type  : %spd_contents   -> Returns the uncorrectable fields of the MES region
#Description  : Read the Uncorrectable info on all the DIMM when uncorrectable error is introduced
#*********************************************************************************************
sub read_spd_uncorrectable
{
  my ($conn) = @_;
  my $host = host($FILER);

  my %spd_contents;
  my $dimm;

  my(undef,undef,$filer_model,undef) = say($conn,"mhostexec sudo sysctl hw.system_model",-prompt,$host->{'prompt_norm'},-swallow);
  my(undef,undef,$filer_model,undef) = say($conn,"mhostexec sudo sysctl hw.system_model",-prompt,$host->{'prompt_norm'},-swallow);
  (undef,$filer_model) = split(/:/,$filer_model);

  tstlog -comment, "DEBUG->Filer Model:$filer_model" if(DEBUG);
  $filer_model =~ s/\s+$//;

  $dimm = 2 if($filer_model =~ ABSOLUT_V);
  $dimm = 1 if($filer_model =~ ABSOLUT_M || $filer_model =~ ABSOLUT_L);
  $dimm = 1 if($filer_model =~ RUCKUS || $filer_model=~ VESPA_2U || $filer_model=~ VESPA_4U);
  while($dimm <= $MAX_DIMM)
  {
    my %has = read_spd_region($conn,$dimm);
    $spd_contents{$dimm} = \%has;

    my $value = $spd_contents{$dimm}->{'Version'};
    tstlog -comment, "DEBUG->Version:$value" if(DEBUG);

    my $value = $spd_contents{$dimm}->{'Log Cnt'};
    tstlog -comment, "DEBUG->Log Cnt:$value" if(DEBUG);

    my $value = $spd_contents{$dimm}->{'UECC cnt'};
    tstlog -comment, "DEBUG->UECC cnt:$value" if(DEBUG);

    $dimm+=2 if($filer_model =~ ABSOLUT_V);
    $dimm+=1 if($filer_model =~ ABSOLUT_M || $filer_model =~ ABSOLUT_L);
  } #end of while

  #display_spd(%spd_contents);
  return %spd_contents;
}

#*********************************************************************************************
#Function Name: rd_wr_spd
#Parameters   : $conn           -> Filer Connection
#Return Type  : NA
#Description  : Perform read/write operation on corresponding DIMM
#*********************************************************************************************
sub rd_wr_spd
{
  my ($conn) = @_;

  for(my $offset=$OFFSET; $offset <= MAX_OFFSET; $offset++)
  {
    read_spd($conn,$DIMM,$offset,$BYTES) if($READ);
    if($WRITE)
    {
      my $flag = write_spd($conn,$DIMM,$offset,$BYTES,$WR_DATA);
      tstlog '-fail', "Write failed at offset $offset of bytes $BYTES" if(!$flag)
    }
  }
}

sub get_filer_model($)
{
        $FILER=shift;
        $fc = rlm_make_conn_to_ontap($FILER);
        say($fc,"priv set test",-prompt,$host->{'prompt_norm'},-swallow);
        (undef,undef,$filer_model,undef) = say($fc,"mhostexec sudo sysctl hw.system_model",-prompt,$host->{'prompt_norm'},-swallow);
        (undef,$filer_model) = split(/: /,$filer_model);
        $BUS=1 if($filer_model=~/(6210)|(6240)|(6280)/);
        $BUS=0 if($filer_model=~/(2240)|(2220)/);
}

sub populate_DIMM_number_addresses($)
{
        $FILER=shift;
        $fc = rlm_make_conn_to_ontap($FILER);
        say($fc,"priv set test",-prompt,$host->{'prompt_norm'},-swallow);
        (undef,undef,$output,$prompt)=say($fc,"mhostexec sudo spd -a",-prompt,@prompts,-swallow,-timeout,'90');
        while($output=~/Flag \S+ Id (\S+)/g)
                {push @DIMM_address, $1;}
        $Number_of_DIMM=scalar(@DIMM_address);
        logcomment("The addresses of the various DIMMs are ::@DIMM_address and The number of DIMMs are :: $Number_of_DIMM\n");
}
#sub read_spd_data(FILER,option,offset,dimm number,number of bytes(1,2),Data_ptr)
sub read_spd_data($;$$$$$)
{
        logcomment("In read spd data \n");
        $FILER=shift;
        $option=shift;
        my @output_iic;
        my $offset=shift;
        my $DIMM_num=shift;
        my $num_of_bytes=shift;
        $Data_ptr=shift;
        my $fc = rlm_make_conn_to_ontap($FILER);
        if($option==-1)
                {$offset= int rand(255) + 256 unless($offset);}
        $offset= int rand(254) unless($offset);
        $num_of_bytes= 1 + int rand (2) unless($num_of_bytes);
        $DIMM_num=$DIMM_address[int rand($Number_of_DIMM)] unless($DIMM_num);
        (undef,undef,$output,$prompt)=say($fc,"mhostexec sudo spd -n $DIMM_num -o $offset -r $num_of_bytes",-prompt,@prompts,-swallow,-timeout,'90');
        if($offset <0 || ($offset+$num_of_bytes>256))
        {       if($output=~/spd: Invalid offset/)
                {       logcomment("Offset is out of range \n");
                        return;
                }
                else
                {       $Num_of_Failures++;
                        logresult('FAIL',"Proper message was not generated for offset out of range \n");}
        }
        (undef,$output)=split(/:/,$output);
        $output=~s/\-+\s+//;
        (@output_spd)=split(' ',$output);
        undef $output;
        unless($Number_of_DIMM>2)
        {
        my $DIMM_add_iic=2*(hex($DIMM_num) -1) + 160;
        (undef,undef,$output,$prompt)=say($fc,"mhostexec sudo iic -b $BUS -a $DIMM_add_iic -o $offset -r $num_of_bytes",-prompt,@prompts,-swallow,-timeout,'90');
        if($num_of_bytes>2)
        {        unless($output=~/read operation failed/)
                 {      logcomment('FAIL',"Did not get proper failure message for reading >2 bytes through iic\n");
                        $Num_of_Failures++;
                                                          }
        }
        else
        {
        (undef,,$output)=split(/:/,$output);
        $output=~s/\-+\s+//;
        @output_iic=split(' ',$output);
        }
        logcomment( "Debug :: The iic read values are : @output_iic \n The spd read values are : @output_spd\n");
        }
        my $i=0;
        while(@output_spd>$i)
        {
                #logcomment("$output_iic[$i]:$output_spd[$i]::$Data_ptr->[$i]::$Data_ptr\n");
                if($num_of_bytes<=2 && $Number_of_DIMM<=2)
                {
                         unless($output_iic[$i]=~/$output_spd[$i]/i)
                        {       logresult('FAIL',"Read value is different for SPD and iic \n");
                                $Num_of_Failures++;
                        }
                }
                 if((scalar@{$Data_ptr}) && ($Data_ptr->[$i]!~/$output_spd[$i]/i))
                 {      logresult('FAIL',"Read value is different for SPD and the write values\n");
                        $Num_of_Failures++;
                 }
                $i++;
        }
        return (\@output_spd);
}

#sub write_spd_data(FILER,bytes,action,OFFSET,dimm number)
sub write_spd_data($$$;$$)
{       logcomment(" In write spd data \n");
        my $FILER=shift;
        my $Data_ptr=shift;
        my $action=shift;
        my $offset=shift;
        my $DIMM_num=shift;
        if($action==-2)
        {$offset= int rand(150) unless($offset);}
        else
        {$offset= int rand(100)+150  unless($offset);}
        $DIMM_num= $DIMM_address[int rand($Number_of_DIMM)] unless($DIMM_num);
        my $num_of_bytes=scalar(@{$Data_ptr});
        my @write_data=@{$Data_ptr};
        $output_spd = read_spd_data($FILER,0,$offset,$DIMM_num,$num_of_bytes);
        (undef,undef,$output,$prompt)=say($fc,"mhostexec sudo spd -n $DIMM_num -o $offset -w $num_of_bytes @write_data",-prompt,@prompts,-swallow,-timeout,'90');
        if($offset>=0 && $offset<150)
        {       if($output=~/spd: Offset\(\S+\) < MES/)
                        {return;}
                else
                {       logresult('FAIL',"Writing to read-only region did not generate proper error message \n");
                        $Num_of_Failures++;
                }
        }
        read_spd_data($FILER,0,$offset,$DIMM_num,$num_of_bytes,$Data_ptr);
        if($action==-1)
        {       logcomment("Writing back the original values\n");
                write_spd_data($FILER,$output_spd,0,$offset,$DIMM_num);
        }
}

#*********************************************************************************************
#Function Name: execute_spd_testcase
#Parameters   : FILER="the filer name",testcase=testcase number from 1-8,update_way=update way for cecc error
#Return Type  : N/A
#Description  : This tells how many errors were logged into a particular spd region.
#*********************************************************************************************
sub execute_spd_testcase($$;$) {
    my $FILER      = shift;
    my $testcase   = shift;
    my $update_way = shift;
    $fc = rlm_make_conn_to_ontap($FILER);
    ( undef, undef, $spd_before, undef ) = say( $fc, "mhostexec sudo spd -a",
        -prompt, @prompts, -swallow, -timeout, 60 );
    if ( $testcase =~ /[1-8]/ ) {
        execute_memerr_testcase( $testcase, $FILER );
    }
    if ( $testcase =~ /1|2|5|6/ ) {
        $update_way = int rand(3) unless ($update_way);
        if ( $update_way == 0 )
        {        update_date_time( $fc, "7H" );
                        sleep(90);
                say($fc,"date",-prompt, @prompts, -swallow, -timeout, 60 );
         }
        elsif ( $update_way == 1 ) {
            say( $fc, 'mhostexec sudo sysctl debug.mes.write=1',
                -prompt, @prompts, -swallow, -timeout, 60 );
        } elsif ( $update_way == 2 ) {
            say( $fc, 'reboot', -prompt, @prompts, -swallow, -timeout, 300 );
            rlm_verify_filer_state( $FILER, $LOGIN, $PASSWD );
            say( $fc, 'priv set test',
                -prompt, @prompts, -swallow, -timeout, 60 );
        } else {
            logcomment("Cannot update spd region as value set is wrong\n");
        }
        ( undef, undef, $last_err_time, $ERR_Dimm )
            = cecc_count_memerr_v($FILER);
        logcomment("Error dimm is : $ERR_Dimm\n");
        ( undef, undef, $spd_after, undef )
            = say( $fc, "mhostexec sudo spd -n $DIMM_address[$ERR_Dimm-1] -a",
            -prompt, @prompts, -swallow, -timeout, 60 );
        ( undef, undef, $Date_of_err, undef )
            = say( $fc, "mhostexec sudo date -r $last_err_time",
            -prompt, @prompts, -swallow, -timeout, 60 );
        $Dimm_number = $DIMM_address[ $ERR_Dimm - 1 ];
        if ( $spd_before =~ /(Id $Dimm_number [\s\S]*?\-\-\-+)/ ) {
 $spd_before = $&;
        } else {
            logcomment("Error while reading \n");
        }
        logcomment(
            "The spd before is : $spd_before and spd after is : $spd_after\n"
        );
        if ( $spd_before
            =~ /Log Cnt\:(\w+)[\s]+[\s\S]+CECC cnt\:(\w+)\s+CECC0 addr\:(\w+) bit\:(\w+)\s+CECC1 addr\:(\w+) bit\:(\w+)\s+(CECC last error time\:(.+))?[\s\S]+Daily cnt\: (\w+)\s+Weekly cnt\: (\w+)/
            )
        {
            logcomment("Regex 1 matched\n");
            $logcount_before           = $1;
            $cecc_cnt_before           = $2;
            $cecc0_addr_before         = $3;
            $cecc1_addr_before         = $5;
            $cecc_last_err_time_before = $8;
            $Daily_count_before        = $9;
            $weekly_count_before       = $10;
        }
        if ( $spd_after
            =~ /Log Cnt\:(\w+)[\s]+[\s\S]+CECC cnt\:(\w+)\s+CECC0 addr\:(\w+) bit\:(\w+)\s+CECC1 addr\:(\w+) bit\:(\w+)\s+(CECC last error time\:(.+))?[\s\S]+Daily cnt\: (\w+)\s+Weekly cnt\: (\w+)/
            )
        {
            logcomment("regex matched\n");
            $logcount_after           = $1;
            $cecc_cnt_after           = $2;
            $cecc0_addr_after         = $3;
            $cecc1_addr_after         = $5;
            $cecc_last_err_time_after = $8;
            $Daily_count_after        = $9;
            $weekly_count_after       = $10;
        }
        logresult( 'FAIL',
            "Logcount has not increased:logcount_before=$logcount_before and logcount_after=$logcount_after\n"
            )
            unless ( ( hex($logcount_before) + 1 ) eq hex($logcount_after) );
        logresult( 'FAIL',
            "cecc count has not increased:cecc_cnt_before=$cecc_cnt_before and cecc_cnt_after=$cecc_cnt_after\n"
            )
            unless ( hex($cecc_cnt_after) eq ( hex($cecc_cnt_before) + 1 ) );
        logresult( 'FAIL',
            "daily count has not increased:Daily_count_before=$Daily_count_before and Daily_count_after=$Daily_count_after\n"
 )
            unless (
            hex($Daily_count_after) eq ( hex($Daily_count_before) + 1 ) );
        logresult( 'FAIL',
            "weekly count has not increased:weekly_count_before=$weekly_count_before and weekly_count_after=$weekly_count_after\n"
            )
            unless (
            hex($weekly_count_after) == ( hex($weekly_count_before) + 1 ) );
        if($Date_of_err=~/([a-z]{3} [a-z]{3} [0-9]{2} [0-9]{2}\:[0-9]{2}\:[0-9]{2}).+( [0-9]{4})/i)
          {$Date_of_err=$1.$2;}
        logresult( 'FAIL', "time of error was not updated: cecc_last_err_time_after=$cecc_last_err_time_after and Date_of_err=$Date_of_err\n" )
         unless ( $cecc_last_err_time_after=~/$Date_of_err/ );
        if ( $cecc_cnt_before eq '0x0' ) {
            logresult( 'FAIL',
                "Not updating MES properly when address count before is 0" )
                unless ( ( $cecc0_addr_before ne $cecc0_addr_after )
                && ( $cecc1_addr_before eq $cecc1_addr_after ) );
        } elsif ( $cecc_cnt_before eq '0x1' ) {
            logresult( 'FAIL',
                "Not updating MES properly when address count before is 1" )
                unless ( ( $cecc0_addr_before eq $cecc0_addr_after )
                && ( $cecc1_addr_before ne $cecc1_addr_after ) );
        } else {
            logresult( 'FAIL',
                "Not updating MES properly when address count before is more than 1"
                )
                unless ( ( $cecc0_addr_before eq $cecc0_addr_after )
                && ( $cecc1_addr_before eq $cecc1_addr_after ) );
        }

    }
    if ( $testcase =~ /3|4|7|8/ ) {
        ( undef, undef, $spd_after, undef )
            = say( $fc, "mhostexec sudo spd -n $DIMM_address[$Err_DIMM-1] -a",
            -prompt, @prompts, -swallow, -timeout, 60 );
        $Dimm_number = $DIMM_address[ $Err_DIMM - 1 ];
        if ( $spd_before =~ /(Id $Dimm_number [\s\S]*?\-\-\-+)/ ) {
            $spd_before = $&;
        } else {
            logcomment("Error while reading \n");
        }
        logcomment(
            "The spd before is : $spd_before and spd after is : $spd_after\n"
        );
        if ( $spd_before
 =~ /Log Cnt\:(\w+)[\s]+UECC cnt\:(\w+)\s+UECC0 addr\:(\w+) (Time\:(.+))?\s+UECC1 addr\:(\w+) (Time\:(.+))?/
            )
        {
            #logcomment("Regex 1 matched\n");
            $logcount_before            = $1;
            $Uecc_cnt_before            = $2;
            $Uecc0_addr_before          = $3;
            $Uecc1_addr_before          = $6;
            $Uecc0_last_err_time_before = $4;
            $Uecc1_last_err_time_before = $7;
        }
        if ( $spd_after
            =~ /Log Cnt\:(\w+)[\s]+UECC cnt\:(\w+)\s+UECC0 addr\:(\w+) (Time\:(.+))?\s+UECC1 addr\:(\w+) (Time\:(.+))?/
            )
        {
            #logcomment("Regex 2 matched\n");
            $logcount_after            = $1;
            $Uecc_cnt_after            = $2;
            $Uecc0_addr_after          = $3;
            $Uecc1_addr_after          = $6;
            $Uecc0_last_err_time_after = $4;
            $Uecc1_last_err_time_after = $7;
        }
        logresult( 'FAIL',
            "Logcount has not increased:logcount_before=$logcount_before and logcount_after=$logcount_after\n"
            )
            unless ( ( hex($logcount_before) + 1 ) eq hex($logcount_after) );
        logresult( 'FAIL',
            "UECC count has not increased:Uecc_cnt_before=$Uecc_cnt_before and Uecc_cnt_after=$Uecc_cnt_after\n"
            )
            unless ( ( hex($Uecc_cnt_before) + 1 ) eq hex($Uecc_cnt_after) );
        if ( $Uecc_cnt_before eq '0x0' ) {
            logresult( 'FAIL',
                "Not updating MES properly when address count before is 0" )
                unless ( ( $Uecc0_addr_before ne $Uecc0_addr_after )
                && ( $Uecc1_addr_before eq $Uecc1_addr_after ) );
        } elsif ( $Uecc_cnt_before eq '0x1' ) {
            logresult( 'FAIL',
                "Not updating MES properly when address count before is 1" )
                unless ( ( $Uecc0_addr_before eq $Uecc0_addr_after )
                && ( $Uecc1_addr_before ne $Uecc1_addr_after ) );
        } else {
            logresult( 'FAIL',
                "Not updating MES properly when address count before is more than 1"
                )
 unless ( ( $Uecc0_addr_before eq $Uecc0_addr_after )
                && ( $Uecc1_addr_before eq $Uecc1_addr_after ) );
        }
    }
}

1;

