# -*- perl -*-
# $Id:
#
#	Copyright (c) 2009 Network Appliance, Inc.
#	All Rights reserved
##
## @summary This file provides common functions for RLM tests.
##
## @description
## RLM.pm is a common module for account lockout and password aging RLM tests.
##
## Sub-Routines:
## 1. DeleteUser
##  -	Deletes a user or all users except root.
## 2. SearchUser
##	-	Searches if a user account exists
## 3. LockUser
##  -	Locks a user account by entering wrong passwords.
## 4. ExpireUser
##  -	To expire a user account by password aging.
## 5. ChangeTime
##	-	Changes the date in the filer
## 6. RecoverUser
##  -	To Recover the locked/disabled user account
## 7. TestUser
##  -	To check if a user account is functional
## 8. CreateUser
##  -	Creates a user account.
## 9. UserLogin
##  -	Login as a specified user to the filer console
## 10. UpdateRlm
##  -	To Update RLM firmware
## 11. RLM_Reboot
##	-	To reboot RLM module
## 12. Is_RLM_Online
##  -	To Check if RLM is online
## 11. RLM_SSH_Command
##	-	To Execute a non-interactive ssh command
## 12. Multiple_User_Delete
##  -	To delete 4 Users, user1, user2, user3 and user4
## 13. Change_User_Password
##	-	To Change password for a specified User ID from root
## 14. Which_SP
##	-	To get the type of service processor in the specified filer
## 15. RLM_Version
##  -  To return the RLM version from rlm or Ontap console
## 16. Set_Unset_ipv4_ipv6_dhcp
##  -  Set/Unset the ipv6 address from Loader/ontap console along with
##	   Enabling/disabling dhcp.
## 17. RLM_Status
##	-  To get status of peritcular field from the output of command 'rlm status'
## 18. RLM_Update
##	-  Update the RLM image from rlm
## 19. RLM_Boot_Image
##  -  To boot RLM from primary and backup image
## 20. Create_Mod_Del_Role
##	-  Function to create ,modify and delete the role
## 21. Create_Mod_Del_Group
## 	-  Function to create ,modify and delete the group
## 22. Create_Mod_Del_User
##	-  Function to create ,modify and delete the user
## 23. SP_Test_User
## 	-  To check if a user account is functional on RLM /SP.
## 24. SP_Update
##  -  To update RLM/SP from ONTAP
## 25. SP_Version
##  -  To get version info for RLM/SP at ONTAP and RLM/SP prompt
## 26. Is_SP_Online
##	 -  To Check if RLM/SP is online
## 27. SP_Reboot
##	-	To reboot RLM/SP module
## 28. SP_Setup_Authlog
## -   To setup the /var/log/authlog for testing purpose
##
## @status private
##
## @author sandeepk@netapp.com, rohank@netapp.com, dl-platform-qa@netapp.com, mariat@netapp.com
##
## @notes Created by Mindteck (India) Ltd
##
#########################################################################################

package platval::service_processor::RLM;

use strict;
use warnings;
use Shell;
use Tharn;
use BTS_Config;
use Params::Validate qw(validate OBJECT SCALAR);

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	DeleteUser
	SearchUser
	LockUser
	ExpireUser
	ChangeTime
	RecoverUser
	TestUser
	CreateUser
	UserLogin
	UpdateRlm
	RLM_Reboot
	Is_RLM_Online
	RLM_SSH_Command
	Multiple_User_Delete
	Change_User_Password
	Which_SP
	RLM_Version
	Set_Unset_ipv4_ipv6_dhcp
	RLM_Status
	RLM_Update
	RLM_Boot_Image
	Create_Mod_Del_Role
	Create_Mod_Del_Group
	Create_Mod_Del_User
	SP_Test_User
	SP_Update
	SP_Version
   Is_SP_Online
   SP_Reboot
   SP_Setup_Authlog
   SP_root_connect
);

our $VERSION = '1.0';

my $ssh_conn_count = 1;
##############################     FUNCTIONS     ##############################

#----------------------------------------------------------------------------
# FUNCTION: To delete a User/all Users other than root
# INPUT	  :	$_[0] = Filer connection handle,
#			$_[1] = Pass one User name to delete a particular user OR
#					keyword "all" to delete all users except 'root' user.
# RETURN  :	NONE.
#----------------------------------------------------------------------------
sub DeleteUser($$)
{
	my $fc = $_[0] ;
	my $username = $_[1];
	my @user_attributes;
	my @users;
	my $out;
	my $user_exists = 0;

	if( $username eq "all" )
	{
		#List all users and delete all users but retain the "root" user - Default system administrator
		logcomment("Delete all users except root user");
		(undef, undef, $out, undef)=say( $fc, "useradmin user list", -swallow, -timeout, 300 );
		@user_attributes=split(/^\r/m,$out);
		#Check if there are other users than root
		shift(@user_attributes);
		if(($#user_attributes + 1) == 1){
			return 1;
		}
		foreach (@user_attributes)
		{
			@users = split(/\n/, $_ );
			shift(@users);
			$username = shift(@users);

			if(defined($username)){
				(undef,$username) = split(/: /, $username);
			}

			if( defined($username) ){
				logcomment("Delete user:- $username");
				#say( $fc, "useradmin user delete $username ", -swallow, -timeout, 30 );
				say( $fc, "useradmin user delete $username", -swallow, '-noprompt', '-wait', "has been deleted","User does not exist","User <$username> deleted", -timeout, 30);
				$user_exists = SearchUser($_[0],$username);
				if ($user_exists == 1){
					return 0;
				}
			}
		}
		return 1;
	}
	else
	{
		#Delete perticular user from the list
		logcomment("Delete user:- $username");
		$user_exists = SearchUser($_[0],$username);
		if ($user_exists == 0){
			return 1;
		}
		say( $fc, "useradmin user delete $username", -swallow, '-noprompt',
			'-wait', "has been deleted","User does not exist","User <$username> deleted", -timeout, 30);
		$user_exists = SearchUser($_[0],$username);
		if ($user_exists == 1){
			return 0;
		}else {
			return 1;
		}
	}
}

#----------------------------------------------------------------------------
# FUNCTION:	To search if a user account exists.
# INPUT   :	$_[0]=Filer connection handle, $_[1]= User name.
# RETURN  :	1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub SearchUser
{
	my($match, $before, $after, undef) = say ($_[0],
				"useradmin user list $_[1]", '-swallow', '-noprompt',
					'-wait', "Status","User does not exist",-timeout,100);

	if ($before =~ /Name: $_[1]/){
		logcomment("User $_[1] Exists");
		return 1;
	}else {
		logcomment("User $_[1] Does Not Exist");
		return 0;
	}
}

#----------------------------------------------------------------------------
# FUNCTION:	To Lock a user account.
# INPUT   :	$_[0]=Filer connection handle, $_[1]= Root Password,
#			$_[2]= User Name, $_[3]= No of passwd retries.
#		NOTE:	"1^&*9" is the wrong password, do not pass this as a valid
#				password to this function
# RETURN  :	1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub LockUser
{
	my ($cur_user, $cmd_res);
	my $msg_disable_root_success =".*root is disabled due to a manual change.*";
	# ."Review option security\.passwd\.rootaccess\.enable\..*";
    my $msg_disable_root_failure = "root user cannot modify this option";
      print "\$cur_user = $cur_user\n\$cmd_res = $cmd_res\n";
      print "\$_[2] = $_[2]\n";
      if ( $_[2] eq "root") {
		#logcomment("Code to lock root user needs to be added");
		### Added by Prabha

		## Check login id
		(undef, undef, $cur_user, undef) = say($_[0], "useradmin whoami");

      if( $cur_user =~ /root/ ){
         (undef, undef, $cmd_res, undef) = say($_[0],
                        "options security.passwd.rootaccess.enable off");
         if ($cmd_res !~ /$msg_disable_root_failure/g) {
            logcomment( "Cannot lock root user while logged in as root. ".
                  "Login as a different user and try");
            return 0;
         } else {
            return 1;
         }
      }
      if( $cur_user !~ /root/) {
	    	(undef, undef, $cmd_res, undef) = say($_[0],
	    					   "options security.passwd.rootaccess.enable off");
         (undef, undef, $cmd_res, undef) = say($_[0],
                        "options security.passwd.rootaccess.enable");
     		if ($cmd_res !~ /security.passwd.rootaccess.enable off/g) {
 	        	logcomment("Failed to disable root user while logged in ".
 	        			   "as $cur_user");
 	        	return 0;
     		} else {
	     		return 1;
	     	}
		}
	}
=comment # original code  start
    	if( $cur_user !~ /root/) {
	    	(undef, undef, $cmd_res, undef) = say($_[0],
	    					   "options security.passwd.rootaccess.enable off");
     		if ($cmd_res !~ /$msg_disable_root_success/g) {
 	        	logcomment("Failed to disable root user while logged in ".
 	        			   "as $cur_user");
 	        	return 0;
     		} else {
	     		return 1;
	     	}
		}
	}
=cut # original end
	#### End of Addition by Prabha
	else {
      print "\$_[0] = $_[0]\n\$_[3] = $_[3]\n";
		say($_[0],"options security.passwd.lockout.numtries $_[3]");
		say($_[0],"\4",-prompt,"login:",-swallow,-timeout,300);
		for (my $i=1; $i<=$_[3]; $i++)
		{
			if ($i != $_[3]){
				say($_[0],"$_[2]",-prompt,"Password:",-swallow,-timeout,300);
				say($_[0],"1^&*9",-prompt,"login:",-swallow,-timeout,300);
			}elsif ($i == $_[3]) {
				say($_[0],"$_[2]",-prompt,"Password:",-swallow,-timeout,300);
#				my (undef,undef,$after,undef) = say($_[0],"1^&*9",-prompt,"login:",-swallow,-timeout,300); #-wait,"account has been disabled",
                               my (undef,undef,$after,undef) = say($_[0],"1^&*9",-prompt,"login:",-timeout,500);
				# After the user account is locked, use root login to reach filer prompt
				say($_[0],"\r",-prompt,"login:",-swallow,-timeout,300);
				say($_[0],"root",-swallow,-prompt,"Password:",-timeout,200);
				eval {
					say($_[0],$_[1],-swallow,-timeout,20);
				};
				say($_[0],"\r", -swallow);
print "\$after = $after\n";
				if ($after =~ /too many invalid password attempts/ && $after =~ /account has been disabled/){
					 logcomment ("User account of $_[2] is locked/disabled");
					return 1;
				}else {
					logcomment("Failed to lock/disable User account of $_[2]");
					return 0;
				}
			}
		}
	}
}

#---------------------------------------------------------------------
# FUNCTION:	To expire a given user account
# INPUT	  :	$_[0]= Filer connection handle, $_[1]= Number of days,
#			$_[2]= User name, $_[3]= User password.
# RETURN  : 0 = Fail, 1 = PASS
#---------------------------------------------------------------------
sub ExpireUser ($$$$)
{
	my $fc=$_[0];
	my $day=$_[1];
	my $user=$_[2];
	my $passwd=$_[3];
	my $match;

	 #Change the time in the filer for password to expire on ONTAP
	ChangeTime($_[0],$_[1]);
	# Waiting for the RLM daily handler for password expiry
	ChangeTime($_[0],"1");
	say( $fc, "date 0159.56", -swallow, -timeout, 300 );
	sleep 20;

	#Logout from the filer
	say($_[0],"\4",-prompt,"login:",-swallow,-timeout,300);

	#Loginto the filer after the USER password has expired.
	if ( $_[2] =~ /root/ ){
		logcomment("Code to expire root user needs to be added");
	}else{
		logcomment("Verify password for $user has expired");
		say($_[0],$_[2],-prompt,"Password:",-swallow,-timeout,300);
      eval
      {
			(undef,undef,$match,undef) = say($_[0],$_[3],-noprompt,-prompt,"login:", -swallow,-timeout,300);
      };
      if($@)
      {
        # The 'date' command is executed to capture the 'expired password' message. (Refer - Burt 366706)
        my (undef,undef,undef,$prompt) = say($_[0],"date",-swallow, -noprompt, -prompt,".* expired password .*",-swallow,-timeout,300);
        if($prompt =~/user \'$_[2]\' has an expired password .*/)
        {
            	logcomment("Password of $user expired");
					return 1;
         }
      }
	}

	if ($match =~ /password has expired/){
		logcomment("Password of $user expired");
		return 1;
	}else {
		logcomment("Password of $user has not expired");
		return 0;
	}
}




#---------------------------------------------------------------------
# FUNCTION:	To change the date in the filer
# INPUT	  :	$_[0]= Filer connection handle, $_[1]= Number of days.
# RETURN  :	NONE.
#---------------------------------------------------------------------
sub ChangeTime($$)
{
	my ($d,$day,$hr,$sign,$time_const,$sleep_time,$fc,$cmd);
	$fc = $_[0];
	$time_const = '59.58';
	$sleep_time = 20;
	$day = $_[1];

	for($d = 0; $d < $day; $d++) {
		for($hr = 0; $hr < 24; $hr++) {
			sleep $sleep_time;
			$sign = ($hr >= 10) ? $hr.$time_const : "0$hr".$time_const;
			$cmd = "date $sign";
			say( $fc, $cmd, -swallow, -timeout, 300 );
		}
	}
	sleep $sleep_time;
}

#----------------------------------------------------------------------------
# FUNCTION:	To Recover the locked/disabled user account.
# INPUT   :	$_[0]= Filer connection handle, $_[1]= User Name,
#			$_[2]= User Password, $_[3]= Root Password, $_[4]= FILER,
#			$_[5]= RLM IP, $_[6]= RLM Login permit (0=No,1=Yes).
# RETURN  :	1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub RecoverUser
{
	my $user_active =1;
	say($_[0],"\4",-prompt,"login:",-swallow,-timeout,300);
	say($_[0],"root",-prompt,"Password:",-swallow,-timeout,300);
	eval {
		say($_[0],$_[3],-swallow,-timeout,20);
	};
	say($_[0],"\r",-swallow);
	say($_[0],"passwd",-prompt,"Login:",-swallow,-timeout,300);
	say($_[0],$_[1],-prompt,"New password:",-swallow,-timeout,300);
	say($_[0],$_[2],-prompt,"Retype new password:",-swallow,-timeout,300);
	my (undef,undef,$after,undef) = say($_[0],$_[2],-timeout,300);
print "\$after = $after\n";
	if ($after !~ /changed/) {
		logcomment("Failed to change password for user $_[1]");
		return 0;
	} else {
		logcomment("Password for user $_[1] changed");
	}

	$user_active = TestUser($_[0],$_[5],$_[1],$_[2],$_[3],$_[4],$_[6]);
	if ($user_active == 0) {
		logcomment("Succeeded to Recover $_[1]");
		return 1;
	} else {
		logcomment("Failed to Recover $_[1]");
		return 0;
	}
}

#----------------------------------------------------------------------------
# FUNCTION:	To check if a user account is functional.
# INPUT   :	$_[0]= Filer connection handle, $_[1]=SP/RLM IP, $_[2]= User Name,
#			$_[3]= User Password, $_[4]= Root Password, $_[5]= Filer,
#			$_[6]= RLM Login permit (0=No,1=Yes).
# RETURN  : 0 = Pass, $after = Reason for Failure
#----------------------------------------------------------------------------
sub TestUser
{
	my ($frb,$match,$after,$prompt);
	my $cons_login = 0;
	my $rlm_login = 0;
	my $passwd_expiry = 0;

	# Verify if the user can login to Filer
		say($_[0],"\4",-swallow,-prompt,"login:",-timeout,300);
		say($_[0],$_[2],-swallow, -prompt,"Password:",-timeout,300);

		if ($_[2] eq "root") {
			eval {
				($match,undef,undef,$prompt) = say($_[0],$_[4],-swallow,-prompt,-timeout,20);
			};
			eval { say($_[0],"\r", -swallow,-timeout, 15); };
		} else {
        eval{
          if($_[6] == 1)
          {
           ($match,undef,$after,$prompt) = say($_[0],$_[3],-swallow, -prompt,"login:",-swallow,-timeout,300);
          }
          else
          {
           ($match,undef,$after,$prompt) = say($_[0],$_[3],-swallow, -noprompt, -prompt,"login:",-swallow,-timeout,300);
          }
         };
         if($@)
         {
         	# The 'date' command is executed to capture the 'expired password' message. (Refer - Burt 366706)
         	(undef,undef,undef,$prompt) = say($_[0],"date",-swallow, -noprompt, -prompt,".* expired password .*",-swallow,-timeout,600);
         	if($prompt =~/user \'$_[2]\' has an expired password .*/)
         	{
           		$passwd_expiry = 1;
            	$after="password has expired";
         	}
         }
	  		if ($prompt eq "login:") {
				say($_[0],"root",-prompt,"Password:",-timeout,300);
				eval {
				($match,undef,$after,$prompt) = say($_[0],$_[4],-swallow,-prompt,-timeout,30);
				};
				say($_[0],"\r", -swallow);
				logcomment ("User $_[2] is restricted to login to Filer Console");
				$cons_login = 1;
			}
			if ($after =~ /password has expired/) {
				$passwd_expiry = 1;
			}
        }



	# Verify if the user can login to RLM. While calling this
	# function, specify if the user can login to RLM or not.
	# Depending on $_[6], the below loop is entered.
	if ($_[6] == 0) {
		if ($_[2] eq "root") {
			$frb = connect("$_[5]#ssh$ssh_conn_count;conntype=ssh-persist;connhost=$_[1];username=naroot;password=$_[4];prompts=Permission");
			$rlm_login = 1;
		} else {
			$frb = connect("$_[5]#ssh$ssh_conn_count;conntype=ssh-persist;connhost=$_[1];username=$_[2];password=$_[3];prompts=Permission");
			$rlm_login = 1;
		}
	}elsif ($_[6] == 1) {
		if ($_[2] eq "root"){
			$frb = connect("$_[5]#ssh$ssh_conn_count;conntype=ssh-persist;connhost=$_[1];username=naroot;password=$_[4]");
		} else {
			$frb = connect("$_[5]#ssh$ssh_conn_count;conntype=ssh-persist;connhost=$_[1];username=$_[2];password=$_[3]");
		}
		disconnect($frb);
		sleep 10;
	}
	$ssh_conn_count = ($ssh_conn_count+1);

	if (($rlm_login == 0) && ($cons_login == 0) && ($passwd_expiry == 0)){
		logcomment ("User $_[2] is active and operational");
		return 0;
	} elsif (($passwd_expiry == 1) && ($rlm_login == 1)){
      logcomment ("Password of $_[2] is Expired.");
      return $after;
   }
   elsif (($cons_login == 1) && ($rlm_login == 1)){
		logcomment ("After account is disabled, $_[2] is restricted for both Filer and RLM/SP Login");
		return $after;
	} elsif (($cons_login == 1) && ($rlm_login == 0)){
		logcomment ("$_[2] is restricted only for Filer Login");
		return $after;
	} elsif (($cons_login == 0) && ($rlm_login == 1)){
		logcomment ("$_[2] is restricted only for RLM/SP Login");
		return $after;
	} elsif (($passwd_expiry == 1) && ($cons_login == 0) && ($rlm_login == 0)){
		logcomment ("Password of $_[2] is Expired. Change it before attempting further actions");
		return $after;
	}
}

#----------------------------------------------------------------------------
# FUNCTION:	To create a user account.
# INPUT   :	$_[0]=Filer connection handle, $_[1]= User name, $_[2]= User Password,
#			$_[3]= Group name, $_[4]= Min no. of days for Password Expiry,
#			$_[5]= Max no. of days for Password Expiry, $_[6]= RLM IP,
#			$_[7]=FILER, $_[8]=Root Password, $_[9]=RLM Login permit(0=No,1=Yes).
# RETURN  :	1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub CreateUser
{
	my $user_exists = SearchUser($_[0],$_[1]);
	if ($user_exists == 1){
		logcomment("Could not add user <$_[1]>. Error: User is already present");
		return 0;
	} else {
		#say($_[0],"useradmin user add $_[1] -g $_[3] -m $_[4] -M $_[5]",-prompt,"New password:",-swallow,-timeout,300);
		#say($_[0],"$_[2]",-prompt,"Retype new password:",-swallow,-timeout,300);
		#say($_[0],"$_[2]",-prompt,"User <$_[1]> added",-swallow,-timeout,300);

		say($_[0],"useradmin user add $_[1] -g $_[3] -m $_[4] -M $_[5]",-swallow,-prompt,"New password:",-timeout,300);
		say($_[0],"$_[2]",-swallow,-prompt,"Retype new password:",-timeout,300);
		say($_[0],"$_[2]",-swallow,'-noprompt','-wait',"User <$_[1]> added",-timeout,300);

		logcomment("Added user <$_[1]> and need to check");
		# Check if the user is functional
		my $user_active = TestUser($_[0],$_[6],$_[1],$_[2],$_[8],$_[7],$_[9]);
		if ($user_active == 0) {
			logcomment("Succeeded to add $_[1]");
			return 1;
		} else {
			logcomment("Failed to add $_[1]");
			return 0;
		}
	}
}

#----------------------------------------------------------------------------
# FUNCTION:	To Login as a specified user to the filer console.
# INPUT   :	$_[0]= Filer connection handle, $_[1]= User Name,
#			$_[2]= User Password, $_[3] = Filer
# RETURN  :	NONE
#----------------------------------------------------------------------------
sub UserLogin {
   my ($prompt);
     (undef, undef, undef, $prompt) =
      say($_[0], "\4", -prompt, "login:", "Password:", -swallow, -timeout, 300);
	if ($prompt !~ /$_[3].*/){
		if($prompt =~ /login:.*/) {
			 (undef, undef, undef, $prompt) = say($_[0], $_[1], -prompt, "Password:", -swallow, -timeout, 300);
    	}
    	if($prompt =~ /Password:.*/){
       	eval { say($_[0], $_[2], -prompt, -swallow, -timeout, 20); };
       	say($_[0], "\r", -swallow);
      }
    }
}
#----------------------------------------------------------------------------
# FUNCTION:	To Update RLM firmware.
# INPUT   :	$_[0]= Filer connection handle, $_[1]= Image name with complete path.
# RETURN  :	NONE
#----------------------------------------------------------------------------
sub UpdateRlm
{
	my ($after, $prompt, $result);
	say($_[0], "rlm status", -swallow,'-timeout',100,
			'-fatal', "rlm status command time out");

	(undef, undef, $after, undef) = say($_[0],"software install $_[1] -f",
											-swallow,-timeout,300);
	if($after !~ /completed/) {
		logcomment("Software Install of RLM firmware Failed");
		return 0;
	}

	logcomment("Starting RLM update, takes more than 30 mins");
	(undef, undef, undef, $prompt) = say($_[0], "priv set advanced", '-swallow','-timeout',300);
	(undef, undef, undef, $prompt) = say($_[0], "rlm update -f",
								-prompt, "Do you want to reboot the RLM now",
						"RLM version is up-to-date",'-swallow','-timeout',4500);
	if ($prompt =~ /reboot the RLM now/) {
		say($_[0], "y",-prompt,'-timeout',300);
		$result = Is_RLM_Online($_[0]);
		if( $result == 0 ) {
			logcomment("RLM failed to come online after rlm update");
			return 0;
		}
		return 1;
	} else {
		logcomment("RLM firmware update Failed");
		return 0;
	}
}


#----------------------------------------------------------------------------
# FUNCTION:	To reboot RLM module.
# INPUT   :	$_[0]= Filer connection handle, $_[1]= root password,
#			$_[2]= Name of the Filer.
# RETURN  :	1 - Success, 0 - Failure
#----------------------------------------------------------------------------
sub RLM_Reboot
{
	my $out;
	my $result;
	my $fc = $_[0];
	my $passwd = $_[1];
	my $filer = $_[2];

	#Log-in as root user
	UserLogin($fc, "root", $passwd, $filer);

	say($fc,"rlm status",-prompt,-swallow,-timeout,300);
	say($fc,"rlm reboot",-prompt,"Do you want to reboot the RLM now.*",-swallow,-timeout,300);
	say($fc,"y",-prompt,-swallow,-timeout,300);
	(undef,undef,$out,undef)=say($fc,"rlm status",-prompt,-swallow,-timeout,300);
	if($out !~ /Status: Rebooting/) {
		return 2;
	}
	$result = Is_RLM_Online($fc);
	if( $result == 0 ) {
		return 0;
	}else{
		return 1;
	}
}

#----------------------------------------------------------------------------
# FUNCTION: Check if RLM is online.
# INPUT   : $_[0]= Filer connection handle
# RETURN  : 0 - Failure, 1 - Success.
#----------------------------------------------------------------------------
sub Is_RLM_Online
{
	my $timer = 0;
	my $status = 0;
	my $fc = $_[0];
	while($timer <= 16){
		my(undef, undef, $out) = say($fc, "rlm status", -prompt, -swallow, -timeout, 30);
		if($out =~ m/Online/) {
			$status = 1;
			$timer = 17;
		} else {
			sleep(15);
		}
		$timer = $timer + 1;
	}
	return $status;
}

#----------------------------------------------------------------------------
# FUNCTION:	To Execute a non-interactive ssh command.
# INPUT   :	$_[0]= Client IP, $_[1]= Client's root password, $_[2] = User Name,
#			$_[3] = User Password, $_[4] = RLM IP.
# RETURN  :	NONE
#----------------------------------------------------------------------------
sub RLM_SSH_Command
{
	my $ssh = connect("#rlogin;connhost=$_[0];password=$_[1];username=root;prompts=root");
	eval {
		say($ssh,"ssh -l $_[2] $_[4] events info",-prompt,"password",-swallow,-timeout,180);
	};
	if ($@) {
		say($ssh,"yes", -prompt, "password:", -timeout, 100); 
	}

	my (undef, undef, undef, $prompt) = say($ssh,$_[3],-prompt,"Latest record","Permission",-swallow,-timeout,180);
	if ($prompt eq "Permission") {
		say($ssh,"\3",-noprompt,-swallow,-timeout,300);
		return 0;
	} else {
		return 1;
	}
}

#---------------------------------------------------------------------
# FUNCTION: To delete 4 Users, user1, user2, user3 and user4
# INPUT	  : $_[0] = Filer connection handle
# RETURN  : 0 = Success, 1 = Failure
#---------------------------------------------------------------------
sub Multiple_User_Delete
{
	# Arguments - DeleteUser: Connection handle, all (specifies to delete
	# 				all users other than root)
	my @users = qw (user1 user2 user3 user4);
	foreach my  $user (@users){
		my $return = DeleteUser($_[0],$user);
		if ($return == 0){
			logcomment("Failed to delete user $user");
			return 1;
		}
	}
	return 0;
}

#---------------------------------------------------------------------
# FUNCTION:	As a root user, change password for a specified User ID
# INPUT   :	$_[0]= Filer connection handle, $_[1]= User Name,
#			$_[2]= User Password.
# RETURN  :	0 = Success, 1 = Failure
#---------------------------------------------------------------------
sub Change_User_Password
{
	my (undef, undef, undef, $prompt) = say($_[0], "passwd", -swallow, -prompt,
									"Login:", "New password:", -timeout, 100);
	if ($prompt eq "Login:"){
		say($_[0], $_[1], -swallow, -prompt, "New password:", -timeout, 100);
	}
	say($_[0], $_[2], -swallow, -prompt, "Retype new password:", -timeout, 100);
	my (undef, undef, $resp, undef) = say($_[0], $_[2], -timeout, 100);
	if ($resp !~ /changed/){
		logcomment("Failed to change password for user $_[1]");
		return 1;
	}else{
		return 0;
	}
}

#----------------------------------------------------------------------------
# FUNCTION:	To get the type of service processor in the specified filer.
# INPUT   :	$_[0]= Filer connection handle, $_[1]= Login Name,
#       	$_[2]= Filer Name, $_[3]= Password, $_[4] = Filer mode.
# RETURN  :	Service processor type (SP / RLM / BMC / UNKNOWN).
#----------------------------------------------------------------------------
sub Which_SP(%;)
{
	my (%opts) = validate (
		@_,
		{
			fc			=> { type => OBJECT },
			username	=> { type => SCALAR, default => 'root' },
			user_passwd => { type => SCALAR, default => 'netapp1!' },
			filer		=> { type => SCALAR },
			mode		=> { type => SCALAR, default => '7mode' },
		}
	);

	my ($response,$processor);
	my $not_found = "not found";
	my $no_rlm = "There is no RLM";
	my $not_supported = "NOT SUPPORTED";

	if(ONTAP_Login($opts{fc},$opts{username},$opts{user_passwd},$opts{filer})) {
		logcomment ("root user failed to get to ONTAP prompt");
	}
	if ($opts{mode} eq "10mode") {
		Dblade_10mode($opts{fc},"enter");
	}
	(undef,undef,$response) = say ($opts{fc},"sp status",-swallow,
	                                             -prompt,'-timeout',90);
	if($response =~ /$not_found/i) {
		(undef,undef,$response) = say ($opts{fc},"rlm status",-swallow,
	                                             -prompt,'-timeout',90);
		if(($response =~ /$not_found/i) || ($response =~ /$no_rlm/i)) {
			(undef,undef,$response) = say ($opts{fc},"bmc status",-swallow,
	                                             -prompt,'-timeout',90);
			if(($response =~ /$not_found/i) || ($response =~ /$not_supported/i)) {
				$processor = "UNKNOWN";
			} else {
				$processor = "BMC";
			}
		} else {
			$processor = "RLM";
		}
	} else {
		$processor = "SP";
	}
	if ($opts{mode} eq "10mode") {
		Dblade_10mode($opts{fc},"exit");
	}
	return $processor;
}

#-------------------------------------------------------------------------------
# FUNCTION	:	RLM_Version - return the RLM version from rlm or Ontap console
# INPUT		:	$_[0] - rlm/ontap connection handle
#				$_[1] - String "rlm" or "ontap"
#				$_[2] - mode of the filer
# RETURN	:	version string
#-------------------------------------------------------------------------------
sub RLM_Version {

	my $fc        = shift;
	my $interface = shift;
	my $mode	  = shift;
	my ($resp, $ret);

	if ($interface =~ /rlm/i){
		snooze (10);
		(undef, undef, $resp, undef) = say ($fc, "version");
	    if ($resp =~ /(.*Primary firmware.*)/) {
	        $ret = (split(" ", $1))[-1];
	    }
	}
	if ($interface =~ /ontap/i){
		if (($mode eq "7mode") || ($mode eq "7G")){
			(undef, undef, $resp, undef) = say ($fc, "rlm status");
		} elsif ($mode eq "10mode") {
			(undef, undef, $resp, undef) = say ($fc, "rlm status");
		}
		if ($resp =~ /(.*Firmware Version.*)/) {
		    $ret = (split(" ", $1))[-1];
		}
	}
	return $ret;
}

#-------------------------------------------------------------------------------
# FUNCTION :  Set/Unset the ipv6 address from Loader/ontap console along
# 			  with Enabling/disabling dhcp.
# INPUT	   :  fc   => 'filer connection'
#			  ipv6 => 'disable or enable'
#       	  dhcp => 'enable or disable'
#
# 			  if dhcp parameter key is 'disable' following parameter is required
# 			  rlm_ip => 'rlm ip' rlm_netmask => 'rlm netmask' rlm_GW => 'rlm GW'
#
#			  if ipv6 parameter key is 'enable' following parameter is required
# 		   	  rlm_IPV6 => 'rlm ipv6' rlm_IPV6_PREFIX => 'rlm_IPV6_PREFIX'
#		   	  rlm_IPV6_GW => 'rlm IPV6 GW'
# RETURN  :   1 - any error occurs during these operation
#-------------------------------------------------------------------------------
sub Set_Unset_ipv4_ipv6_dhcp {

	my(%opt)=@_;

	my $loader_prompt="LOADER.*";
	my $cfe_prompt="CFE.*";
	my $setup_cmd = "rlm setup";
	my $fc=$opt{fc};

	my $prompt = Filer_Prompt($fc);
	if( $prompt =~ /$loader_prompt|$cfe_prompt/){
		Set_Priv($fc,'advanced');
	}
	#Check for InPut parameter
	if($opt{dhcp} !~/enable/i && $opt{dhcp} !~/disable/i){

			print "dhcp option is not provided\n";
			return 1;
	}
	if($opt{dhcp} eq "disable" && ( !$opt{rlm_ip} ||
						!$opt{rlm_netmask} || !$opt{rlm_GW})){
			logcomment("dhcp option disabled but complete RLM ipv4".
												" info is not provided");
			return 1;
	}
	if($opt{ipv6} !~ /enable/i && $opt{ipv6} !~/disable/i){
			logcomment("Valid ipv6 option is not provided $opt{ipv6}");
		return 1;
	}
	if($opt{ipv6} eq "enable" && ( ! $opt{rlm_IPV6} ||
							!$opt{rlm_IPV6_PREFIX} || !$opt{rlm_IPV6_GW})){
			logcomment("ipv6 is enabled but complete".
									" RLM ipv6 info is not provided");
			return 1;
	}
	#Start rlm setup
	eval{
		say ($fc, $setup_cmd, -prompt, "Would you like to configure the RLM");
		say($fc, "y", -prompt, "Would you like to enable DHCP");
		if ($opt{dhcp} !~ /enable/i) {

		    say($fc, "n", -prompt, "Please enter the IP address for the RLM.*");
		    say($fc, "$opt{rlm_ip}", -prompt,
		    						"Please enter the netmask for the RLM.*");
			say($fc, "$opt{rlm_netmask}", -prompt,
					"Please enter the IP address for the RLM gateway.*");
			say($fc, "$opt{rlm_GW}", -prompt,
								"Do you want to enable IPv6 on the RLM.*");
		} else {
		    say ($fc, "y", -prompt, "Do you want to enable IPv6 on the RLM.*");
		}
		if ($opt{ipv6} =~/disable/i) {
			say($fc,"n", -prompt,"$loader_prompt|$cfe_prompt", -timeout, 60);
		}else {
		    say($fc, "y",-prompt,"Do you want to assign IPv6 global address.*");
		    say($fc, "y",-prompt,"Please enter the IPv6 address for the RLM.*");
		    say($fc, "$opt{rlm_IPV6}", -prompt,
		    					"Please enter the subnet prefix for the RLM.*");
		    say($fc, "$opt{rlm_IPV6_PREFIX}", -prompt,
		    					"Please enter the IPv6 Gateway for the RLM.*");
		    if ($prompt =~/$loader_prompt|$cfe_prompt/) {
		    	say($fc, $opt{rlm_IPV6_GW}, -prompt,
		      			"Do you want to enable IPv6 .*Address for the RLM.*");
		    	say($fc,"y",-prompt,$loader_prompt,$cfe_prompt, -timeout, 60);
		    } else {
		    	say($fc,"$opt{rlm_IPV6_GW}",-prompt,$loader_prompt,
		    										$cfe_prompt,-timeout,60);
		    }
		}
	};
	if($@){
		logcomment ("FAIL:Failed to set ipv6 $opt{ipv6}".
										" and dhcp $opt{dhcp} : $@");
		return 1;
	}
}

#------------------------------------------------------------------------------
# FUNCTION	: 	To get status of peritcular field from the output of
#				the command 'rlm status'.
# INPUT		:	$[0]	-	filer connection
#				$[1]	-	mode of filer ( 7mode or 10mode)
#				$[2]	-	major field (RLM, RLM IPv4 or RLM IPv6)
#				$[3]	- 	minor fields ( Status, Firmware Version etc.)
# RETURN	:   Value of a perticular field from the output of the
#				the command 'rlm status'.
#------------------------------------------------------------------------------
sub RLM_Status {

	my $fc = shift;
	my $mode = shift;
	my $major_field = shift;
	my $minor_field = shift;
	my $hash_key = "RLM";
	my %hash;
    my $after_rlm_status;
    my $loader_prompt="LOADER.*";
    my $cfe_prompt="CFE.*";

    $fc->discardbuff;

    my $prompt = Filer_Prompt($fc);

    if( $prompt =~ /$loader_prompt|$cfe_prompt/){
        Set_Priv($fc,"advanced");
        (undef, undef, $after_rlm_status) = say($fc, "rlm status", '-swallow', '-prompt', $loader_prompt, '-timeout', 30);
    }elsif (( $mode eq "7mode" ) || ( $mode eq "7G" )){
		(undef, undef, $after_rlm_status) = say($fc, "rlm status", '-swallow', '-timeout', 30);
	}elsif( $mode eq "10mode" ){
		(undef, undef, $after_rlm_status) = say($fc,
							"run local -c rlm status", '-timeout', 30);
	}

	my @rlm_status = split("\n", $after_rlm_status);
	shift(@rlm_status);

	foreach (@rlm_status) {
		 my @rlm_status_comp = split(": ", $_);
         foreach (@rlm_status_comp) {
             my $rlm_status_comp = $_;
             	if($rlm_status_comp =~ /Remote LAN Module.*Status/){
					$rlm_status_comp = "Status";
             	}

             $rlm_status_comp = "Firmware Version" if( $rlm_status_comp =~ /Firmware Version/ );
			 $rlm_status_comp = "Ethernet Link" if( $rlm_status_comp =~ /Ethernet Link/ );

             my $value = pop(@rlm_status_comp);
             $value =~ s/^\s+//;
             $value =~ s/\s+$//;

             $rlm_status_comp =~ s/^\s+//;
         	 $rlm_status_comp =~ s/\s+$//;

             if( $rlm_status_comp =~ /IPv4.*/ )
             		{
	             		$hash_key = "RLM IPv4";
	             		next;
             		}
             if( $rlm_status_comp =~ /IPv6.*/ )
             		{
	             		$hash_key = "RLM IPv6";
	             		$hash{$hash_key}{$rlm_status_comp} = $value if($value =~ /Disabled/);
	             		next;
             }
             next if( $rlm_status_comp =~ /Link Local Address/ );
             $hash{$hash_key}{$rlm_status_comp} = $value;
        }
}
    return $hash{$major_field}{$minor_field};
}

#------------------------------------------------------------------------------
#	FUNCTION	: 	Update the RLM image from rlm
#	INPUT		:	$[0]	-	Filer name
#					$[1]	-	filer connection
#					$[2]	-	rlm connection
#					$[3]	-	login password for rlm connection
#			 		$[4]	-	HTTP path of rlm image
#								(eg. http://172.17.0.10/rlm/)
#					$[5]	-	RLM version
#					$[6]	-	Filer operating mode( 7-mode or 10-mode)
#	RETURN		:  	0 - rlm update was successfull
#					1 - rlm update failed
#	NOTE		:	If RLM update is successfull the connection handle
#					is disconnected. The test script has to reconnect to
#					rlm console if needed.
#------------------------------------------------------------------------------
sub RLM_Update {

	my $FILER= shift;
	my $fc = shift;
	my $fr = shift;
	my $passwd = shift;
	my $http_url = shift;
	my $rlm_version = shift;
	my $mode = shift;
	my $image="/build/targets/RLM_FW.tar.gz";

    $fr->discardbuff;

    my(undef, undef, $after_rlm_update) = say($fr,
    "rlm update $http_url$rlm_version$image -f", '-swallow', '-timeout', 3600);

    if($after_rlm_update =~ m/RLM FW Update Successful/) {
        disconnect($fr);
        my $rlm_status = RLM_Reboot($fc, $passwd, $FILER);
        if($rlm_status == 0) {
        	logcomment("Warning: RLM failed to come online");
        	return 1;
        }
        $fr = connect("$FILER#ssh8;conntype=ssh-persist;connhost=$FILER-rlm;username=naroot;password=$passwd");
        $fr->discardbuff;
        my $version = RLM_Version($fr, "rlm", $mode);
        if($version eq $rlm_version) {
        	logcomment("force $rlm_version rlm firmware version downloaded ");
        } else {
            logcomment("cannot force download $rlm_version
            		   rlm firmware version");
            return 1;
        }
        disconnect($fr);
    } else {
    	logcomment("force rlm update $rlm_version unsuccessful");
        return 1;
    }
    return 0;
}

#---------------------------------------------------------------------------
# FUNCTION	: 	To boot RLM from primary and backup image
# INPUT		: 	$_[0] = Filer Connection handle,
#				$_[1] = Image string as primary/backup, $_[2] = Filer mode
# RETURN	: 	0 - Fail
#				1 - Pass
#				2 - Unknown Response from RLM
#---------------------------------------------------------------------------
sub RLM_Boot_Image {
	my ($fc, $image, $mode) = @_;
	my $prompt = ".*reboot the RLM now.*";
	my $result;

	if (($mode eq "7mode") || ($mode eq "7G")){
		say ($fc, "rlm reboot $image", -swallow, -prompt, "$prompt");
	} elsif ($mode eq "10mode") {
		say ($fc, "run local -c rlm reboot $image", -swallow, -prompt,
		"$prompt");
	}
	say ($fc, "y", -swallow);

	my (undef, undef, $out, undef) = say ($fc, "rlm status", -prompt, -swallow,
								  -timeout, 300);
	if($out !~ /Status: Rebooting/) {
		return 2;
	}
	$result = Is_RLM_Online($fc);
	if ( $result == 0 ) {
		return 0;
	} else {
		return 1;
	}
}

#------------------------------------------------------------------------------
#	FUNCTION	: 	Function to create ,modify and delete the role
#	INPUT		:	fc 		- 	Filer connection handel
#					name 	- 	name of the role
#					action 	- 	create/modify/delete
#					cap    	- 	Allowed capability[ <cap1>,<cap2>,...,<capn>]
#			 		mode	- 	7G/7mode/10mode
#	RETURN		:  	0 - role create/modify/delete successful.
#					1 - failed to create/modify/delete role.
#------------------------------------------------------------------------------

sub Create_Mod_Del_Role {

	my (%opt, $cmd, $response);
	%opt = @_;

	$cmd = "useradmin role" ;
	$cmd = "run local " . $cmd if ($opt{mode} eq '10mode' );
	#Add a role.
	if( $opt{action} =~/create/){
	  (undef,undef,$response)=say ($opt{'fc'},
	  "$cmd add $opt{'name'} -a  $opt{cap}", -swallow);
	  return 1 if($response !~ /Role \<$opt{'name'}\> added/);
	}
	#Modify role.
	if( $opt{action} =~/modify/){
	  (undef,undef,$response)=say ($opt{'fc'},
	   "$cmd modify $opt{'name'} -a  $opt{cap}", -swallow);
	  return 1 if($response !~ /Role \<$opt{'name'}\> modified/);
	}
	#Delete role.
	if( $opt{action} =~/[d]elete/){
	  (undef,undef,$response)=say ($opt{'fc'}, "$cmd delete $opt{'name'}",
	   -swallow);
	  return 1 if($response !~ /Role \<$opt{'name'}\> deleted/);
	}
    return 0;
}

#------------------------------------------------------------------------------
#	FUNCTION	: 	Function to create ,modify and delete the group
#	INPUT		:	fc 		- 	Filer connection handel
#					name 	- 	name of the group
#					action 	- 	create/modify/delete
#					role   	- 	Roles [ <role1>[,<role2>,...,<roleN>]
#			 		mode	- 	7G/7mode/10mode
#	RETURN		:  	0 - group create/modify/delete successful.
#					1 - failed to create/modify/delete group.
#------------------------------------------------------------------------------

sub Create_Mod_Del_Group {

	my(%opt,$cmd, $response);
	%opt=@_;
	$cmd = "useradmin group" ;
	$cmd = "run local " . $cmd if ($opt{mode} eq '10mode' );

	#Add group.
	if($opt{action} =~/create/){
	  (undef,undef,$response)=say ($opt{'fc'},
	  "$cmd add $opt{'name'} -r  $opt{role}", -swallow);
	  return 1 if($response !~ /Group \<$opt{'name'}\> added/);
	}
	#Modify group.
	if($opt{action} =~/modify/){
	  (undef,undef,$response)=say ($opt{'fc'},
	  "$cmd modify $opt{'name'} -r  $opt{role}", -swallow);
	  return 1 if($response !~ /Group \<$opt{'name'}\> modified/);
	}
	#Delete group.
	if($opt{action} =~/[d]elete/){
	  (undef,undef,$response)=say ($opt{'fc'}, "$cmd delete $opt{'name'}",
	   -swallow);
	  return 1 if($response !~ /Group \<$opt{'name'}\> deleted/);
	}
    return 0;
}

#------------------------------------------------------------------------------
#	FUNCTION	: 	Function to create ,modify and delete the user
#	INPUT		:	fc 		- 	Filer connection handel
#					name 	-	name of the user
#					passwd	-	password for the user
#					gname 	- group name
#					action 	- 	create/modify/delete
#			 		mode	- 	7G/7mode/10mode
#	RETURN		:  	0 - user create/modify/delete successfull.
#					1 - failed to create/modify/delete user.
#------------------------------------------------------------------------------

sub Create_Mod_Del_User {
	my(%opt,$cmd,$prompt,$response,$prmt);
	%opt=@_;
	$cmd = "useradmin user" ;
	$cmd = "run local " . $cmd if ($opt{mode} eq '10mode' );

	#Create user.
	if( $opt{action} =~/create/){
	  $prompt="New password:";
	  (undef,undef,$response)=say ($opt{'fc'},
	  "$cmd add $opt{'name'} -g  $opt{gname}",-swallow, -prompt, $prompt);

	  if($response !~ /User is already present/){
		  $prompt="Retype new password:";
		  (undef,undef,$response)=say ($opt{'fc'}, "$opt{passwd}", -swallow,
		   -prompt,$prompt);
		  (undef,undef,$response)=say ($opt{'fc'}, "$opt{passwd}", -swallow);
		  return 1 if($response !~ /User \<$opt{'name'}\> added./);
  	  }
	}

	#Modify user.
	if( $opt{action} =~/modify/){
	  (undef,undef,$response)=say ($opt{'fc'},
	   "$cmd modify $opt{'name'} -g  $opt{gname}", -swallow);
	  return 1 if($response !~ /User \<$opt{'name'}\> modified./);
	}
	#Delete user.
	if( $opt{action} =~/[d]elete/){
	  (undef,undef,$response)=say ($opt{'fc'},
	   "$cmd delete $opt{'name'}", -swallow);
	  return 1 if($response !~ /User \<$opt{'name'}\> deleted./);
	}
    return 0;
}

#----------------------------------------------------------------------------
# FUNCTION:    To check if a user account is functional on RLM /SP.
# INPUT   :    rlm_sp_ip   	-	RLM / SP IP
#              user        	- 	name of the user
#              passwd      	- 	user password
#              filer       	- 	filer name
#              login_permit - 	RLM / SP Login permit (0=No,1=Yes).
# RETURN  : 0 = Success, 1 = Failure
#----------------------------------------------------------------------------

sub SP_Test_User {
	my (%opt, $err, $fs);
	%opt = @_;
   $ssh_conn_count = ($ssh_conn_count+1);

	# Verify if the user can login to RLM/SP. While calling this
	# function, specify if the user can login to RLM/SP or not.
	# Depending on $opt{login_permit}, the below loop is entered.

	if ($opt{login_permit} == 0) {
	    eval{
	           $fs = connect ("$opt{filer}#rlm$ssh_conn_count;conntype=ssh-persist;connhost=$opt{rlm_sp_ip};username=$opt{user};password=$opt{passwd};prompts=Permission");
	    };
	    if ($@) {
	         $err = $@;
	         logcomment("Failed as connected to RLM/SP");
            disconnect($fs);
	         return 1;
	    } else {
	         return 0;
	    }
	} elsif ($opt{login_permit} == 1) {
	    eval {
	            $fs = connect ("$opt{filer}#rlm$ssh_conn_count;conntype=ssh-persist;connhost=$opt{rlm_sp_ip};username=$opt{user};password=$opt{passwd}");
		};
		if ($@) {
		   $err = $@;
		   logcomment("Failed to connect to RLM/SP");
		   return 1;
		} else {
         disconnect($fs);
         return 0;
		}
		snooze 10;
	}
}

#----------------------------------------------------------------------------
# FUNCTION:	To Update SP/RLM firmware.
# INPUT   :
#			fc			=> filer connection handle
#			build_path	=> absolute build path
#			sp_rlm		=> interface "RLM" or "SP"
#			flag		=> flag "c" of "f"
#			mode 		=> mode of the filer
#
# RETURN  :	1 = pass, 0 = fail
#----------------------------------------------------------------------------
sub SP_Update {

	my (%opt, $prossr, $status, $after, $prompt, $image, $cmd);
	%opt = validate (
		@_,
		{
			'fc'		 => { type => OBJECT },
			'build_path' => { type => SCALAR },
			'sp_rlm'	 => { type => SCALAR },
			'mode' 		 => { type => SCALAR, default => '7mode' },
			'flag'		 => { type => SCALAR, default => "" },
		},
	);
	$status = 0;
	$prossr = "rlm"
	if ($opt{sp_rlm} =~ /RLM/i);
	$prossr = "sp"
	if ($opt{sp_rlm} =~ /SP/i);

	$cmd = "software install $opt{build_path} -f";

	$cmd = "run local" . $cmd
	if ($opt{mode} eq "10mode");
	(undef, undef, $after, undef) = say($opt{fc}, $cmd, -swallow, -timeout,300);
	if($after !~ /completed/) {
		logcomment("Software Install of $prossr firmware Failed");
		return 0;
	}

	logcomment("Starting $prossr update, takes more than 30 mins");
	if($opt{flag} ne "") {
		if ($opt{mode} eq "10mode") {
			say ($opt{fc}, "run local priv set advanced", -swallow);
		} else {
			say ($opt{fc}, "priv set advanced", -swallow);
		}
		$cmd = "$prossr update -$opt{flag}";
		$cmd = "run local" . $cmd
		if ($opt{mode} eq "10mode");
	} else {
		$cmd = "$prossr update";
		$cmd = "run local" . $cmd
		if ($opt{mode} eq "10mode");
	}
   say($opt{fc}, $cmd, -prompt, ".*Firmware updated successfully.*",
                                          -swallow, -timeout, 4500);
   $cmd = "\r";
	$cmd = "run local" . $cmd
	if ($opt{mode} eq "10mode");
	(undef, undef, undef, $prompt) = say($opt{fc}, "\r", -prompt, ".*reboot.*",
							   						-swallow);
	if ($prompt =~ /Do you want to reboot/) {
		$cmd = "y";
		$cmd = "run local" . $cmd
		if ($opt{mode} eq "10mode");
      say($opt{fc}, $cmd ,-prompt,'-timeout',300);
	}

	$cmd = "$prossr status";
	$cmd = "run local" . $cmd
	if ($opt{mode} eq "10mode");
	for (0 .. 15) {
		my(undef, undef, $out) = say($opt{fc}, $cmd, -swallow);
		if($out =~ m/Online/) {
			$status = 1;
			last;
		}
		snooze (40);
	}
	if( $status == 0 ) {
		logcomment("$prossr failed to come online after rlm update");
		return 0;
	} else {
		logcomment("$prossr firmware update successful");
		return 1;
	}
}


#-------------------------------------------------------------------------------
# FUNCTION	:	SP_Version - return the RLM version from rlm or Ontap console
# INPUT   :
#			fcsp		=>  connection handle of filer or rlm or sp, from
#							where want to extract the version information
#			sp_rlm		=>	interface "RLM" or "SP"
#			interface	=>  "ontap" of "serviceprocessor"requied if taking
#							version information at ONTAP
#			mode		=>  mode of the filer
#
# RETURN  :	version string
#-------------------------------------------------------------------------------
sub SP_Version {

	my (%opt, $resp, $ret, $prossr, $cmd);
	%opt = validate (
		@_,
		{
			'fcsp'		 => { type => OBJECT },
			'sp_rlm'	 => { type => SCALAR },
			'interface'	 => { type => SCALAR },
			'mode' 		 => { type => SCALAR, default => '7mode' },
		},
	);

	$prossr = "rlm"
	if ($opt{sp_rlm} =~ /RLM/i);
	$prossr = "sp"
	if ($opt{sp_rlm} =~ /SP/i);

	if ($opt{interface} =~ /serviceprocessor/i){
		snooze (10);
		(undef, undef, $resp, undef) = say ($opt{fcsp}, "version");
	    if ($resp =~ /(.*Primary firmware.*)/) {
	        $ret = (split(" ", $1))[-1];
	    }
	}
	if ($opt{interface} =~ /ontap/i){
		$cmd = "$prossr status";
		$cmd = "run local" . $cmd
		if ($opt{mode} eq "10mode");
		(undef, undef, $resp, undef) = say ($opt{fcsp}, $cmd, -swallow);
		if ($resp =~ /(.*Firmware Version.*)/) {
		    $ret = (split(" ", $1))[-1];
		}
	}
	return $ret;
}

#----------------------------------------------------------------------------
# FUNCTION: Check if SP/RLM is online.
#
# INPUT   :
#			fc			=> filer connection handle
#			sp_rlm	=> interface "RLM" or "SP"
#			mode 		=> mode of the filer
#
# RETURN  : 0 - Failure, 1 - Success.
#----------------------------------------------------------------------------
sub Is_SP_Online {
print "Inside Is_SP_Online\n";
    my (%opt, $prossr, $timer, $status, $mode, $out);
    %opt = validate(@_,
                    {'fc'     => {type => OBJECT},
                     'sp_rlm' => {type => SCALAR},
                     'mode'   => {type => SCALAR, default => '7mode'},},);

    $prossr = "rlm"
      if ($opt{sp_rlm} =~ /RLM/i);
    $prossr = "sp"
      if ($opt{sp_rlm} =~ /SP/i);

    $timer  = 0;
    $status = 0;
    $mode   = $opt{mode};
    $out    = "";
    while ($timer <= 16) {
        if (($mode eq "7mode") || ($mode eq "7G")) {
           eval {
            (undef, undef, $out) =
              say($opt{fc}, "$prossr status", -prompt, -swallow, -timeout, 30); };
            if ($@)
            {
               # Filer prompt is not returned properly whe "rlm/sp status" is checked.
              	sleep(60);
              (undef, undef, $out) =
              say($opt{fc}, "$prossr status", -prompt, -swallow, -timeout, 30);
            }
        } elsif ($mode eq "10mode") {
            (undef, undef, $out) = say($opt{fc}, "run local $prossr status",
                                       -prompt, -swallow, -timeout, 30);
        }
        if ($out =~ m/Online/) {
            $status = 1;
            $timer  = 17;
        } else {
            sleep(15);
        }
        $timer = $timer + 1;
    } ## end while ($timer <= 16)
    return $status;
} ## end sub Is_SP_Online

#----------------------------------------------------------------------------
# FUNCTION:	To reboot RLM/SP module.
# INPUT   :
#			fc			=> filer connection handle
#			sp_rlm	=> interface "RLM" or "SP"
#			mode 		=> mode of the filer
#        filer		=> the name of filer
#        passwd	=> password of root user
# RETURN  : 0 - Failure, 1 - Success.
#----------------------------------------------------------------------------
sub SP_Reboot {
print "Inside SP_Reboot\n";
    my (%opt, $prossr, $out, $result, $mode, $fc, $cmd);
    %opt = validate(@_,
                    {'fc'     => {type => OBJECT},
                     'sp_rlm' => {type => SCALAR},
                     'mode'   => {type => SCALAR, default => '7mode'},
                     'filer'  => {type => SCALAR},
                     'passwd' => {type => SCALAR},},);

    $prossr = "rlm"
      if ($opt{sp_rlm} =~ /RLM/i);
    $prossr = "sp"
      if ($opt{sp_rlm} =~ /SP/i);

    $out="";
    $fc = $opt{fc};
    $mode = $opt{mode};
    #Log-in as root user
    UserLogin($opt{fc}, "root", $opt{passwd}, $opt{filer});

     if (($mode eq "7mode") || ($mode eq "7G")) {
       $cmd = "";
    } elsif ($mode eq "10mode") {
     $cmd = "run local";
    }

    say($fc, "$cmd $prossr status", -prompt, -swallow, -timeout, 30);
    sleep 5;
    say($fc, "$cmd $prossr reboot",
        -prompt, "Do you want to reboot the SP|RLM now.*",
        -swallow, -timeout, 300);
    say($fc, "y", -swallow, -prompt, "Console connection through the SP will now be dropped", -timeout, 30);
    sleep 30;
    (undef, undef, $out, undef) = say($fc, "$cmd $prossr status", -prompt, -swallow, -timeout, 100);
        (undef, undef, $out, undef) = say($fc, "$cmd $prossr status", -prompt, -swallow, -timeout, 100);
print "\$out = $out\n";
    if ($out !~ /Status: Rebooting/) {
        (undef, undef, $out, undef) = say($fc, "$cmd $prossr status", -prompt, -swallow, -timeout, 100);
print "\$out = $out\n";
            if ($out !~ /Status: Rebooting/) {
        return 2;
    }
    }

    $result = Is_SP_Online(fc => $fc, sp_rlm => $prossr, mode => $mode);
      if ($result == 0) {
        return 0;
    } else {
        return 1;
    } 
} ## end sub SP_Reboot

#----------------------------------------------------------------------------
# FUNCTION:	To set up the /var/log/authlog.
# INPUT   :
#        		$[0]	-	value =0 in setup , value =1 in cleanup
#					$[1]	-	connection handle for root sp/rlm login
#					$[2]	-  value =0 in setup, value =0/1 depending on script is passed. if passed, value=1
#					$[3]	-	scriptname
# RETURN  : NONE
#----------------------------------------------------------------------------
sub SP_Setup_Authlog($$$$) {
    my $setup        = shift @_;
    my $SP_ssh_root = shift @_;
    my $pass         = shift @_;
    my $scriptname   = shift @_;

    if ($setup == 1) {
        logcomment(
            "Setting up an temporary authlog log file for testing, by taking a backup of original authlog file\n"
        );
        my (undef, undef, $SP_response) =
          say($SP_ssh_root, "cd /var/log", '-swallow', '-noprompt', '-prompt',
              "#", '-timeout', 100);
        (undef, undef, $SP_response) =
          say($SP_ssh_root, "ls", '-swallow', '-noprompt', '-prompt', "#",
              '-timeout', 100);

        if ($SP_response =~ /\s+authlog\s+/i) {
            say($SP_ssh_root, "rm -rf authlog",
                '-swallow', '-noprompt', '-prompt', "#", '-timeout', 100);
            (undef, undef, $SP_response) =
              say($SP_ssh_root, "ls", '-swallow', '-noprompt', '-prompt', "#",
                  '-timeout', 100);
            if ($SP_response =~ /\s+authlog\s+/i) {
                logcomment("Failed to delete authlog file");
           }
            } ## end else [ if ($SP_response =~ /\s+authlog\s+/i)]
            logcomment("Creating a new authlog for testing purpose");
            (undef, undef, $SP_response) = say($SP_ssh_root, "touch authlog",
                      '-swallow', '-noprompt', '-prompt', "#", '-timeout', 100);
            (undef, undef, $SP_response) =
              say($SP_ssh_root, "ls", '-swallow', '-noprompt', '-prompt', "#",
                  '-timeout', 100);
            if ($SP_response =~ /\s+authlog\s+/i) {
                logcomment("New authlog file is created");
            } else {
                logcomment("Failed to create authlog file");
            }

    } ## end if ($setup == 1)
    elsif ($setup == 0) {
        my (undef, undef, $SP_response) =
          say($SP_ssh_root, "cd /var/log", '-swallow', '-noprompt', '-prompt',
              "#", '-timeout', 100);
        (undef, undef, $SP_response) =
          say($SP_ssh_root, "ls", '-swallow', '-noprompt', '-prompt', "#",
              '-timeout', 100);
        if ($SP_response =~ /\s+authlog\s+/i) {
            if ($pass == 1) {
                logcomment("Removing the temporarily created authlog");
                say($SP_ssh_root, "rm -rf authlog",
                    '-swallow', '-noprompt', '-prompt', "#", '-timeout', 100);
            } elsif ($pass == 0) {
                logcomment("Renaming authlog to authlog_$scriptname as the script is failed");
                say($SP_ssh_root, "mv authlog authlog_$scriptname",
                    '-swallow', '-noprompt', '-prompt', "#", '-timeout', 100);
            }
        } ## end if ($SP_response =~ /\s+authlog\s+/i)
        logcomment("Replacing the original authlog file");
         my (undef, undef, $SP_response) = say($SP_ssh_root, "mv authlog_org authlog",
            '-swallow', '-noprompt', '-prompt', "#", '-timeout', 100);
            if($SP_response =~/No such file or directory/)
            {
            	logcomment("Failed to locate authlog_org. Creating new authlog file");
              say($SP_ssh_root, "touch authlog",'-swallow', '-noprompt', '-prompt', "#", '-timeout', 100);
            }

        (undef, undef, $SP_response) =  say($SP_ssh_root, "ls authlog", '-swallow', '-noprompt', '-prompt', "#",
            '-timeout', 100);
        if ($SP_response !~ /\s+authlog\s+/i) {
            logcomment("Failed to locate authlog file");
        }

    } ## end elsif ($setup == 0)

} ## end sub sp_Setup_Authlog($,$,$,$)


#----------------------------------------------------------------------------
# FUNCTION:	To set up the /var/log/authlog.
# INPUT   :
#        		$[0]	-	Filer name
#					$[1]	-	filer connection
#					$[2]	-  rlm/sp ip
#					$[3]	-	rlm/sp
# RETURN  : NONE
#----------------------------------------------------------------------------
sub SP_root_connect($$$$) {
    my ($FILER, $fc, $rlm_ip, $sp_type) = @_;
    my $r;
    say($fc, "priv set test",
        '-swallow', '-prompt', "$FILER.*>", '-timeout', 60);
    say($fc, "$sp_type debugmode",
        '-swallow', '-prompt', "$FILER.*>", '-timeout', 60);
    my $Pwd = ($sp_type eq 'sp') ? 'superuser' : 'moondance';

    eval {
          $r = connect("$FILER#ssh-rlm". ++$ssh_conn_count
              . ";conntype=ssh-persist;connhost=$rlm_ip;username=root;password=$Pwd;prompts='\#'");
    };
    unless ($@) {
        return $r;
    } else {
       logresult('FATAL', "SSH to $rlm_ip failed : $@ \n");
    }
} ## end sub SP_root_connect



1;
