# $Id: //depot/prod/test/main/lib/platval/service_processor/SNMP.pm#1 $
#
#       Copyright (c) 2010 Network Appliance, Inc.
#       All Rights reserved
#
## @summary Routines for validating RLM/SP and Ontap IPv4 and IPv6 support for the feature SNMP.
##
## @description
##    This library contains individual routiens for ONTAP, RLM/SP
##    for analysing various tasks such as SP/RLM events, SP/RLM status -d,
##    Configuring and deleting of SNMP clients, Gathering IP's of SNMP clients,
##    Getting and Validating the SNMP statistics and Validating configuration from
##    both Ontap as well as SP/RLM.
##
#############################################################################################
#############################################################################################
## Sub-Routines:
##
## 1. load_ipv4_ipv6_SNMP_config_cases
##      To Load The IPv4 and IPv6 test cases and their expected results
##
## 2. snmp_test_results
##      To Validate the results of the tests for diff test cases
##
## 3. verify_SNMP_testcase_validity
##      To verify valid TestCase was sent or not
##
## 4. get_snmp_client_ipv4_ipv6_ips
##	To Check the IPv4 and IPv6 IP's for the Clients Passed
##
## 5. check_snmp_trap_config
##      To Gather the IPv4 and IPv6 Clients and their respective IP's as per the Test Cases
##
## 6. setup_ontap_snmp_trap_config
##	To Add the SNMP TrapHosts to Ontap
##
## 7. set_snmp_ontap_trap_rlm_config_status
##	To Set the environment to add SNMP Traphosts and RLM/SP Setup As per Test Cases	
##
## 8. get_snmp_trap_conf_ontap
##	To Gather SNMP TrapHosts Configuration from Ontap
##
## 9. test_snmp_trap_conf
##	To Validate the SNMP TrapHosts Configuration before and after the events
##
##10. get_snmp_statistics
##	To Gather SNMP statistics and configuration from RLM/SP status -d
##
##11. test_snmp_statistics
##	To Validate all the SNMP statistics before and after the events
##
##12. test_snmp_statistics_check
##	To Validate all the SNMP statistics before and after the particular event	
##
##13. get_snmp_events
##	To Gather SNMP events from SP events
##
##14. test_snmp_events_check
##	To Validate all the SNMP events before and after the particular event	
##
##15. delete_snmp_traphost
##	To Delete the Existed TrapHosts Configured 
##
#############################################################################################
#############################################################################################
require "tharnit_lib.pl";
#use rlm_ipv6;
use platval::service_processor::rlm_ipv6;

use vars qw/%snmp_test_cases/;

## Diff SNMP IPv4/IPv6 cobinations as test cases and Each Case contain
## SNMP Client conf either IPv4 or IPv6 or Both and same for RLM?SP also.
## First  Argument => Description of the Test Case
## Seoond Argument => RLM Conf to Be Set As per RLM_IPv6 Test Plan 
## Third  Argument => RLM/SP statistics from status -d 
## Fourth Argument => RLM/SP events from RLM/SP events
## Fifth  Argument => SNMP Client with IPv4 Configuration(1->Trap Should Be Received, 0-> Trap Should Not Be Received)
## Sixth  Argument => SNMP Client with IPv6 Configuration(1->Trap Should Be Received, 0-> Trap Should Not Be Received)
sub load_ipv4_ipv6_SNMP_config_cases {
      
	%snmp_test_cases = (
                1 => ["Case 1. SNMP clients are IPv4 address and $RLM_PROMPT have IPv4 address",2,1,1,1,0],
        );

        unless ($NO_IPv6) {
                %snmp_test_cases = (%snmp_test_cases,
                        2 => ["Case 2. SNMP clients are IPv4 address and $RLM_PROMPT have IPv6 address",4,1,1,0,1],
                        3 => ["Case 3. SNMP clients are IPv4 address and $RLM_PROMPT have both IPv4 and IPv6 addresses",5,1,1,1,1],
                        4 => ["Case 4. SNMP clients are both IPv4 and IPv6 address and $RLM_PROMPT have IPv4 address",2,1,1,1,0],
                        5 => ["Case 5. SNMP clients are both IPv4 and IPv6 address and $RLM_PROMPT have IPv6 address",4,1,1,0,1],
                        6 => ["Case 6. SNMP clients are both IPv4 and IPv6 address and $RLM_PROMPT have both IPv4 and IPv6 addresses",5,1,1,1,1],
			7 => ["Case 7. SNMP clients are IPv6 address and $RLM_PROMPT have IPv4 address",2,1,1,1,0],
                        8 => ["Case 8. SNMP clients are IPv6 address and $RLM_PROMPT have IPv6 address",4,1,1,0,1],
                        9 => ["Case 9. SNMP clients are IPv6 address and $RLM_PROMPT have both IPv4 and IPv6 addresses",5,1,1,1,1],
                );
        }
}

## Validating the Results of Statistics, Events and Trap information
## As per the above Hash table

sub snmp_test_results {

	my $case = shift;
	my $statistics_results = shift;
	my $events_results = shift;
	my $clients_to_configure = shift;
	my @client_results = @_;

	my $configure_client = 1;
	my $statistics = 1;
	my $events = 1;

	tstlog -comment,"Case:$case";

	tstlog -comment,"Client results:@client_results";

	unless($statistics_results == $snmp_test_cases{$case}->[2]) {
		logresult('FAIL',"Statistics are Not updated properly");
		$statistics = 0;
	}

	if ($RLM_PROMPT !~ /RLM/) {
		unless($events_results == $snmp_test_cases{$case}->[3]) {
			logresult('FAIL',"Events are Not updated properly");
			$events = 0;
		}
	}

	for (my $j = 0;$j < scalar(@{$clients_to_configure});$j++) {
		if($clients_to_configure->[$j] =~ /\.+/) {
			unless($client_results[$j] == $snmp_test_cases{$case}->[4]) {
				logresult('FAIL',"IPv4:Trap was Not generated for this Event \n $client_results[$j] NE $snmp_test_cases{$case}->[4]");
				$configure_client = 0;
			}
		} else {
			unless($client_results[$j] == $snmp_test_cases{$case}->[5]) {
				logresult('FAIL',"IPv6:Trap was Not generated for this Event \n $client_results[$j] NE $snmp_test_cases{$case}->[5]");
				$configure_client = 0;
			}
		}	
	}
	
	return $statistics,$events,$configure_client;
}


## Validating the value of $TEST_CASE passed as the Argument
sub verify_SNMP_testcase_validity {
	
	my $TEST_CASE = shift;
        my $no_ipv6;

        if ($NO_IPv6) {
                $no_ipv6 = $NO_IPv6;
                undef ($NO_IPv6);
                rlm_load_ipv4_ipv6_SNMP_config_cases();
        }

        if ($TEST_CASE > 0 && $TEST_CASE <= scalar(keys %snmp_test_cases)) {
                logresult ('NA', "Case: $TEST_CASE :- " . $snmp_test_cases{$TEST_CASE}->[0] . " can't be execuated as \"$no_ipv6\" ");
        } else {
                logresult ('NA', "Invalid test case/state: $TEST_CASE. See the tharndoc for more info");
        }
}


## Checking the IPv4 and IPv6 IP's for the Clients Passed
sub get_snmp_client_ipv4_ipv6_ips {

	my @clients_info = @_;
	my @client_conn;
	my %client_details;

	for (my $i = 0;$i < scalar(@clients_info);$i++) {

        	$ch = host($clients_info[$i]);
        	$client_conn[$i] = connect($clients_info[$i]."#rlogin");
        	my (undef,undef,$out) = say($client_conn[$i],"/sbin/ifconfig eth1 | grep inet");
        	logcomment ("output = $out");

        	($client_details{$clients_info[$i]}->{'ipv4'}) = $out =~ /inet\saddr\:([\d\.]+)/s;
        	($client_details{$clients_info[$i]}->{'ipv6'}) = $out =~ /inet6\saddr\:\s([\w\:]+)\/\d{1,3}\sScope\:Global/s;
	}

	tstlog -comment,"CLient IP's taken";

	return \%client_details;
}


## Gathering the IPv4 and IPv6 Clients and their respective IP's as per the Test Cases
sub check_snmp_trap_config {

        my $setup = shift;
	my $clients_list = shift;

	my @config_clients;
	my @config_v4_clients;

	tstlog -comment,"CLIENTS COUNT :".scalar(keys %{$clients_list});

	foreach my $i (keys %{$clients_list}) {
		if($setup == 1) {
			push(@config_clients,$clients_list->{$i}->{'ipv4'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
			tstlog -comment,"in 1st setup condition";
		} elsif($setup == 2) {
			push(@config_clients,$clients_list->{$i}->{'ipv4'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
                } elsif($setup == 3) {
			push(@config_clients,$clients_list->{$i}->{'ipv4'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
                } elsif($setup == 4) {
			push(@config_clients,$clients_list->{$i}->{'ipv4'});
			push(@config_clients,$clients_list->{$i}->{'ipv6'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
                } elsif($setup == 5) {
			push(@config_clients,$clients_list->{$i}->{'ipv4'});
			push(@config_clients,$clients_list->{$i}->{'ipv6'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
                } elsif($setup == 6) {
			push(@config_clients,$clients_list->{$i}->{'ipv4'});
			push(@config_clients,$clients_list->{$i}->{'ipv6'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
                } elsif($setup == 7) {
			push(@config_clients,$clients_list->{$i}->{'ipv6'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
                } elsif($setup == 8) {
			push(@config_clients,$clients_list->{$i}->{'ipv6'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
                } elsif($setup == 9) {
			push(@config_clients,$clients_list->{$i}->{'ipv6'});
			push(@config_v4_clients,$clients_list->{$i}->{'ipv4'});
                }
	}

	tstlog -comment,"Clients Info gathered: @config_clients";

	tstlog -comment,"Clients To Configure gathered: @config_v4_clients";


	return \@config_clients,\@config_v4_clients;
}


## Adding the SNMP TrapHosts to Ontap
sub setup_ontap_snmp_trap_config {

	my $FILER = shift;
	my $clients_list = shift;

	my $fc = rlm_make_conn_to_ontap($FILER);

	for(my $i = 0;$i < scalar(@{$clients_list});$i++) {
		say($fc,"snmp traphost add $clients_list->[$i]");
	}
}


## Set the environment to add SNMP Traphosts and RLM/SP Setup As per Test Cases 
sub set_snmp_ontap_trap_rlm_config_status {

	my $FILER = shift;
	my $clients_data = shift;
	my $case = shift;
	
	my ($config_client_data,$config_v4_client_data) = check_snmp_trap_config($case,$clients_data);
	setup_ontap_snmp_trap_config($FILER,$config_client_data);
        rlm_load_ipv4_ipv6_config_cases($FILER);
        logcomment ("$RLM_PROMPT is setting for the configuration. Testcase: " . $snmp_test_cases{$case}->[1] . "\n");
	sleep 10;
        rlm_setup($FILER,$test_states{$snmp_test_cases{$case}->[1]});
        rlm_verify_rlm_status($FILER,'ONTAP',$snmp_test_cases{$case}->[1]);

	return $config_client_data,$config_v4_client_data;
}


## Gather SNMP TrapHosts Configuration from Ontap
sub get_snmp_trap_conf_ontap {

	my $FILER = shift;

	my $fc = rlm_make_conn_to_ontap($FILER);

	my (undef,undef,$out) = say($fc,"snmp traphost",'-swallow','-timeout',30);

	my @trap_hosts = $out =~ /^\s+([\w\.\:]+)\s\([\w\.\:]\)\s\<[\w\.\:]\>/mg;

	return @trap_hosts;
}



## Validate the SNMP TrapHosts Configuration before and after the events
sub test_snmp_trap_conf {

	my ($before,$after) = @_;

	for(my $i = 0; $i < scalar(@{$before}); $i++) {
		if($before->[$i] ne $after->[$i]) {
			logresult('FATAL',"Trap Host IP Mismatched[$before->[$i] not equals $after->[$i]]");
		}
	}
}


## Gathering SNMP statistics and configuration from RLM/SP status -d
sub get_snmp_statistics {

        my $FILER = shift;

        my $fhA = rlm_make_host_to_ontap($FILER);

        tstlog -comment,"Checking SP Status -d Command for $FILER";

        my $ip;
	
	if($ipv4_ip) {
		$ip = $fhA->{'RLM_IP'};
	} elsif(($ipv6_global_ip !~ /Disabled|unknown/i) && $ipv6_global_ip && ($ipv6_global_ip ne $ipv6_gateway)) {
		$ip = $fhA->{'RLM_IPV6_IP'};
	} elsif($ipv6_router_ip) {
		$ip = $fhA->{'RLM_IPV6_ROUTER'};
	}

        my $rlm_pt = rlm_make_conn_to_rlm($FILER,$ip);

        my (undef,$out,undef,undef) = say($rlm_pt,"$RLM status -d",'-swallow',-noprompt,'-wait',@prompts_sp,'-timeout',180);

        my %snmp_status_d = $out =~ /trap\sdaemon.+(snmp enable)\s+(\d+).+(snmp trap power_off_via_$RLM)\s+(\d+).+(snmp trap power_cycle_via_$RLM)\s+(\d+).+(snmp trap reset_via_$RLM)\s+(\d+).+(snmp trap post_error)\s+(\d+).+(snmp trap abnormal_reboot)\s+(\d+).+(snmp trap l2_watchdog_timeout)\s+(\d+).+(snmp trap power_loss sent)\s+(\d+).+(snmp trap loss_of_heartbeat)\s+(\d+).+(snmp test trap)\s+(\d+).+(snmp test trap failure)\s+(\d+).+/s;

        my %snmp_conf_status_d = $out =~ /\/mnt\/logs\/etc\/trap_conf.+(trap_enable)\s+(\w+).+(version)\s+(\d+).+(num_traphosts)\s+(\d+).+/s;

	my @traphosts = $out =~ /traphost\[\d+\]\s([\w\.\:]+)/sg;

	my @trap_ip;

	for (my $i=0; $i < $snmp_conf_status_d{num_traphosts}; $i++) {
		push(@trap_ip,$traphosts[$i]);
	}

        disconnect($rlm_pt);

        return \%snmp_status_d,\%snmp_conf_status_d,@trap_ip;
}


## Validating all the SNMP statistics before and after the events
sub test_snmp_statistics {

        my ($before,$after) = @_;

        foreach my $k (keys %{$before}) {
                unless ($after->{$k} >= ($before->{$k})) {
                        logresult('FAIL', "Count Not Incremented \n After = $after->{$k} \n Before = $before->{$k}");
                }
        }

        logcomment ("snmp_statistics Incremented or As Expected");
}


## Validating all the SNMP statistics before and after the particular event
sub test_snmp_statistics_check {

        my ($before,$after,$msg) = @_;

        foreach my $k (keys %{$before}) {
                if ($k =~ /$msg/) {
                        if ($after->{$k} == ($before->{$k}+1)) {
                                logcomment ("PASS: Statistics are \n After = $after->{$k} \n Before = $before->{$k} \n");
                                return 1;
                        } else {
                                logcomment ("FAIL: Statistics are \n After = $after->{$k} \n Before = $before->{$k} \n");
                                return 0;
                        }
                }
        }
}


## Gathering SNMP events from SP events 
sub get_snmp_events {

        my $FILER = shift;

        my $fhA = rlm_make_host_to_ontap($FILER);

        tstlog -comment,"Checking SP/RLM Events Commands";

	my $ip;

        if($ipv4_ip) {
                $ip = $fhA->{'RLM_IP'};
        } elsif(($ipv6_global_ip !~ /Disabled|unknown/i) && $ipv6_global_ip && ($ipv6_global_ip ne $ipv6_gateway)) {
                $ip = $fhA->{'RLM_IPV6_IP'};
        } elsif($ipv6_router_ip) {
                $ip = $fhA->{'RLM_IPV6_ROUTER'};
        }

        my $rlm_pt = rlm_make_conn_to_rlm($FILER,$ip);

        my (undef,undef,$out) = say($rlm_pt, "events search SNMP",'-swallow','-timeout',120);

        disconnect($rlm_pt);

        return $out;
}


## Validating all the SNMP events before and after the particular event
sub test_snmp_events_check {

        my ($before,$after,$msg) = @_;

        my @out1 = $before =~ /Record.+SNMP $msg/mg;

        my @out2 = $after =~ /Record.+SNMP $msg/mg;

        if($out1[-1] ne $out2[-1]) {
                logcomment("PASS:\n Events Occured Are \n After :$out1[-1] \n Before :$out2[-1]");
                return 1;
        } else {
                logcomment("FAIL:\n Events Occured Are \n After :$out1[-1] \n Before :$out2[-1]");
                return 0;
        }
}


## Deleting the Existed TrapHosts Configured
sub delete_snmp_traphost {

    	my $FILER = shift;

	my $fc = rlm_make_conn_to_ontap($FILER);

    	my (undef,undef,$out) = say ($fc,"snmp traphost");
    	my @traphost_list = split (/\n/,$out);

    	foreach (@traphost_list) {
        	if ($_ =~ /\s+(\S+)\s+\S+\s+\S+/) {
            		say ($fc,"snmp traphost delete $1");
        	}
    	}
}
