# -*- perl -*-
# /* $Id$ */
#
# Copyright (c) 2011 NetApp, Inc.
# All rights reserved.
##
## @summary
## SP_CLI.pm is a common module for all the Service Processor CLI tests.
##
## @description
## This module provides common functions for use in testing Service Processor
## CLI related functionalities.
##
## Sub-routines:
##
## 1. SP_SEL_Messages
##    List of SP SEL messages
##
## 2. SP_SEL_Setup_Clear
##    Setup/Clear an alternative SP Sytem Event Log file for testing
##
## 3. Inspect_SP_SEL_logs
##    Inspecting Service Processor System Event Logs
##
## 4. SP_System_Reset
##    Reset the system using SP system reset command
##
## @synopsis
##      use SP_CLI;
## 
## @status To be reviewed
##
## @author pooja1netapp.com
##
###############################################################################

use strict;
use warnings;
use Tharn;
use BTS_Config qw(Filer_Prompt Set_Priv ONTAP_Login);
use SPARSE::Util::Logger qw(logStep logWarn);

#---------------------------------------------------------------------------
# Initialize and validate parameters
#---------------------------------------------------------------------------
use vars qw($FILER $MODE $LOGIN $PASSWD $RLM_PROMPT);

#---------------------------------------------------------------------------
# Required Parameters
#---------------------------------------------------------------------------
param('FILER', '-mesg', "Filer Name");

#------------------------------------------------------------------------------
# Optional Parameters
#------------------------------------------------------------------------------
param('MODE',   '-default', '7mode');
param('LOGIN',  '-default', "root");
param('PASSWD', '-default', "\r");

#---------------------------------------------------------------------
# Connection handle
#---------------------------------------------------------------------
our ($fc, $fh);

#----------------------------------------------------------------------------
# FUNCTION: List of SEL logs
# INPUT:  fc      - filer connection handle
#         cmd     - command to execute
# RETURN:   Output of the command
#----------------------------------------------------------------------------

sub Inspect_SP_SEL_logs($$)
{
        my ($Sp_response,@SP_response_1,$prompt);
        my $sel_found = 0;
        my $SP_CLI_SEL1 = "\[SP CLI\.notice\]";
        my $SP_CLI_SEL2 = "NOTICE: USER_TRIGGERED";
        $fc = $_[0];

        #Login to SP shell from filer
    logStep("Step 1: Logging into Service Processor");
        logStep("Step 1: logging into RLM/SP CLI");
    (undef, undef, undef, $prompt) =
      say($fc, "\cg", '-swallow', '-noprompt', '-prompt',
          'Service Processor Login',
          '-timeout', 10);

    if ($prompt =~ /Service Processor Login/i) {
        (undef, undef, undef, $prompt) =
          say($fc, "naroot", '-swallow', '-prompt', "$RLM_PROMPT $FILER.*> ",
              -timeout, 10);
    }
    if ($prompt =~ /Password/i) {
        (undef, undef, undef, $prompt) =
          say($fc, "$PASSWD", '-swallow', '-noprompt', '-prompt', '-timeout',
              10);
    }

        #Inspect the SEL events in SP
        (undef, undef, $Sp_response, $prompt) =
        say($fc, "events all", '-swallow', '-noprompt', '-prompt', "$RLM_PROMPT $FILER.*> ",
              -timeout, 300);
        @SP_response_1 = split(/\r\n/,$Sp_response);
        foreach (@SP_response_1) {
                if((($_ =~ /$SP_CLI_SEL1/g)&&($_ =~ /$_[1]/g)) ||
                   (($_ =~ /$SP_CLI_SEL2/)&&($_ =~ /$_[1]/)))  {
                        $sel_found = 1;
                }
        }
        if ($sel_found) {
                logresult("PASS","SEL events are generated successfully");
        }
        else {
                logresult("FAIL","Failed to generate an SEL event");
        }
}

#----------------------------------------------------------------------------
# FUNCTION: List of SEL messages
# INPUT:        NONE.
# RETURN:   Hash table
#----------------------------------------------------------------------------
sub SP_SEL_Messages()
{
        my %SEL_Messages = (
                'SEL_1' => ["system reset","SP_CLI-4"],
                'SEL_2' => ["system core","SP_CLI-5"],
                'SEL_3' => ["sp reboot","SP_CLI-7"],
                'SEL_4' => ["system reset backup","SP_CLI-9"],
                'SEL_5' => ["system reset current","SP_CLI-10"],
                'SEL_6' => ["system reset primary","SP_CLI-11"],
                'SEL_7' => ["system power off","SP_CLI-3"],
                'SEL_8' => ["system power on","SP_CLI-3"],
                'SEL_9' => ["system power cycle","SP_CLI-3"],
                'SEL_10' => '',

        );

        return (%SEL_Messages);
}

#----------------------------------------------------------------------------
# FUNCTION: List of ONTAP messages
# INPUT:        NONE.
# RETURN:   Hash table
#----------------------------------------------------------------------------
sub SP_ONTAP_Messages()
{
        my %ONTAP_Messages =
        ('ONTAP_1' => ["System rebooted after sp system reset","SP_CLI-9"],
        'ONTAP_2' => ["System rebooted after sp system reset","SP_CLI-10"],
        'ONTAP_3' => ["System rebooted after sp system reset","SP_CLI-11"],
        'ONTAP_4' => ["System rebooted after power-on","SP_CLI-3"],
        'ONTAP_5' => [],
        'ONTAP_6' => [],
        'ONTAP_7' => [],
        'ONTAP_8' => [],
        'ONTAP_9' => [],
        'ONTAP_10' => [],

        );

        return (%ONTAP_Messages);
}

#----------------------------------------------------------------------------
# FUNCTION: Setup/Clear an alternative SP Sytem Event Log file
# INPUT:        $_[0] = Filer Connection Handle,
#               $_[1] = 0/1 ; 0 = Setup an alternative SP SEL log file
#                                     1 = Clear an alternative SP SEL log file
# RETURN:   NONE.
#----------------------------------------------------------------------------

sub SP_SEL_Setup_Clear($$)
{
        my $Sp_response;
        my $match_string1 = "SP debugmode Entered\\!";
        my $match_string2 = "can\\'t cd to /mnt/logs/sel";
        #my $match_string3 = "y\\/n";
        my $match_string3 = "Do you want to reboot the SP now.*";
        my $root_password = "superuser";
        my $sp_reboot_command = "sp reboot";
        my $fc = $_[0];
        my $SEL_log = $_[1];
        my $prompt;

        #Set the filer to test privilege and then set the SP to debug mode in ONTAP^M
        # switching SP into test mode
        say($fc, "priv set test",'-swallow', '-prompt', "$RLM_PROMPT $FILER.*> ",'-timeout', 60);

        (undef, undef, $Sp_response, $prompt) =
        say($fc,"sp debugmode",'-swallow', '-noprompt', '-prompt', "$FILER.*> ",
              -timeout, 10);
        if($Sp_response =~ /$match_string1/) {
                logStep("Successfully entered to SP debug mode");
        }
        else {
                logresult("FAIL","Failed to enter into SP debug mode");
        }

        # Login to SP with root user
    (undef, undef, undef, $prompt) =
      say($fc, "\cg", '-swallow', '-noprompt', '-prompt',
          'Service Processor Login',
          '-timeout', 10);

    if ($prompt =~ /Service Processor Login/i) {
        (undef, undef, undef, $prompt) =
          say($fc, "root", '-swallow', '-noprompt','-prompt', 'Password',
              -timeout, 10);
    }
    if ($prompt =~ /Password/i) {
        (undef, undef, undef, $prompt) =
          say($fc, "superuser", '-swallow', '-noprompt', '-prompt', "#", '-timeout',
              10);
    }

        # This piece of code will either setup/clear an alternative SP SEL log file
        # for testing
        if($SEL_log == 0) {
                logStep("Setting up an alternative SP event log file for testing by".
                                                                "taking a backup of original SP SEL log file");

        (undef, undef, undef, $prompt) =
          say($fc, "cd /mnt/logs",'-swallow', '-noprompt', '-prompt', "#",
              -timeout, 10);
        (undef, undef, $Sp_response, $prompt) =
          say($fc, "ls",'-swallow', '-noprompt', '-prompt', "#",
              -timeout, 10);
                if($Sp_response =~ /sel/) {
        (undef, undef, $Sp_response, $prompt) =
          say($fc, "mv sel sel1",'-swallow', '-noprompt', '-prompt', "#",
              -timeout, 10);
        (undef, undef, $Sp_response, $prompt) =
          say($fc, "ls",'-swallow', '-noprompt', '-prompt', "#",
              -timeout, 10);

                        if($Sp_response !~ /sel1/) {
                                logresult("FAIL","Failed to rename sel directory to sel1");
                        }
                else {
                        logresult("PASS","System event log file is not present in /mnt/logs location");
                }

                }
        } elsif ($SEL_log == 1) {
                logStep("Clearing the alternative SEL log file which was used for".
                                                "testing and then backup the original SP SEL log file");
          say($fc, "rm -rf /mnt/logs/sel/*",'-swallow', '-noprompt', '-prompt', "#",
              -timeout, 10);
          say($fc, "rmdir /mnt/logs/sel/",'-swallow', '-noprompt', '-prompt', "#",
              -timeout, 10);
        (undef, undef, $Sp_response, $prompt) =
          say($fc, "cd /mnt/logs/sel/",'-swallow', '-noprompt', '-prompt', "#",
              -timeout, 10);


                if($Sp_response =~ /$match_string2/) {
                        logStep("Successfully deleted the alternative SEL log file")
                }
                else {
                        logresult("FAIL","Failed to delete the alternative SEL log file");
                }

          say($fc, "mv /mnt/logs/sel1 /mnt/logs/sel/",'-swallow', '-noprompt', '-prompt', "#",
              -timeout, 10);
        (undef, undef, $Sp_response, $prompt) =
          say($fc, "ls /mnt/logs/sel/",'-swallow', '-noprompt', '-prompt', "#",
              -timeout, 10);
                if($Sp_response =~ /sel/) {
                        logStep("Restored the original SEL file");
                }
                else {
                        logresult("FAIL","Failed to restore the original SEL file");
                }
        }

        # Reboot the RLM after setup/clear of an alternative SP SEL log
        logcomment("exiting from the sp debugmode");
        say($fc,"\cD",'-swallow', '-noprompt', -timeout, 60);
        say($fc,"\r",'-swallow', '-noprompt', -timeout, 60);
        logcomment("Rebooting the SP");
        (undef,undef,undef,$Sp_response)=say($fc,"$sp_reboot_command",-prompt,$match_string3, -timeout, 60);
                say($fc,"y",-swallow,-prompt,"now be dropped",'-timeout',60);
        snooze(120);
}

#--------------------------------------------------------------------------
# FUNCTION: Reset the system using SP system reset command
# INPUT:        $_[0] = Filer Connection Handle,
#               $_[1] = SP command,
# RETURN:   NONE.
#----------------------------------------------------------------------------
sub SP_System_Reset($$)
{
        my ($sp_response,$match,$prompt,$prmpt);
        my $mesg1 = "Continue.*";
        my $mesg2 = "Booting from Backup Image";
        my $login_prompt = "login:";
        my $fc = $_[0] ;
        my $command = $_[1] ;
        (undef, undef, $sp_response, $prompt) =
        say($fc, $command, '-swallow', '-noprompt', '-prompt', "$mesg1",
              -timeout, 10);
        say($fc, "y", '-swallow', '-noprompt', '-prompt', "$RLM_PROMPT $FILER.*> ",
              -timeout, 300);
    (undef,undef,$match,$prmpt) = say($fc,"\cD",-swallow,-prompt,$login_prompt,"LOADER.*",'-timeout',600);
    if($match eq $mesg2) {
            logStep("The $command resets the system successfully");
    }
    if($match =~/Booted from Backup/ &&  $prmpt=~/LOADER/i) {
            logStep("The $command resets the system successfully");
    }
    elsif($prmpt=~/login/ || $prmpt=~/$FILER/ || $prmpt=~/LOADER/i){
            logStep("The $command resets the system successfully");
    }
    else {
            logresult("FAIL","The $command failed to reset the system");
    }
    # Login to the filer after SP system reset command
    #if(ONTAP_Login($fc,$LOGIN,$PASSWD,$FILER)) {
    #        logresult("FAIL","Failed to login to the FILER");
    #}
}

1;	
