# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/platval/service_processor/SP_Stress_Utils.pm#2 $:
#
#	Copyright (c) 2009 Network Appliance, Inc.
#	All Rights reserved
##
## @summary This file provides common functions for Stress test in SP
##
## @description
## SP_Stress_Utils.pm is a common module for account lockout and password aging RLM tests.
##
## Sub-Routines:
##
## 1.start_stress  
##		-  Intended to make sure CPU time is fully used
##
## 2.SP_Update_from_SP
##		-  To Update SP/RLM firmware.
##
## 3.SP_System_ACP_Sensors_Show
##		-	To verify the 'system acp sensor show'
##
## 4.SP_System_Sensors
##		-	To verify the 'system sensors'
##
## 5.Sensor_Status_compare_SP
##		-	To compare the sensor status from SP
##
## 6.ONTAP_Storage_Show_ACP
##		-	To verify the 'storage show acp'
##
## 7.ONTAP_Environment_Shelf
##		-	To verify the 'environment shelf'
##
## 8.Sensor_Status_compare_ONTAP
##		-	To verify the sensor status from ONTAP
##
## 9.Sensor_Status_ONTAP
##		-	To compare the sensor status from ONTAP
##
## 10.Generate_SES_Traffic
##		-	To Generate SES Traffic
##
## 11.System_fru_show
##		-	Display the fru information
##
## 12.System_acp_show
##		-	Display the acp info
##
## 13.System_acp_model_show
##		-	Display the acp model info
##
## 14.Storage_show_acp
##	    -   Display the acp model info
##
##
## @status private
##
## @author anjanad@netapp.com, mariat@netapp.com
##
## @note
##
#########################################################################################

package platval::service_processor::SP_Stress_Utils;

use strict;
use warnings;
use Shell;
use Tharn;
use BTS_Config;
use platval::service_processor::rlm_ipv6;
use platval::service_processor::RLM;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

our %EXPORT_TAGS = (
    'all' => [
        qw(

          )]);
our @EXPORT_OK = (@{$EXPORT_TAGS{'all'}});

our @EXPORT = qw(
  start_stress
  SP_Update_from_SP
  SP_System_ACP_Sensors_Show
  SP_System_Sensors
  Sensor_Status_compare_SP
  ONTAP_Storage_Show_ACP
  ONTAP_Environment_Shelf
  Sensor_Status_compare_ONTAP
  Sensor_Status_ONTAP
  Generate_SES_Traffic
  System_fru_show
  System_acp_show
  System_acp_model_show
  Storage_show_acp
  Filer_prompt
  Validation_Items

  );

our $VERSION = '1.0';
my (@prompts, @prompts_sp) ;
##############################     FUNCTIONS     ##############################

sub Filer_prompt($) {
	my $FILER = shift @_;
	@prompts = ('LOADER(-\w+)?\*?>','CFE(-\w+)?\*?>',$FILER.'(::[^>#]*)?\*?[>#]');
	@prompts_sp = ("SP $FILER" . '\*?>', "SP". '\*?>');

}

#-------------------------------------------------------------------------------
# FUNCTION  : start_stress  -  Intended to make sure CPU time is fully used
# INPUT     : host - filer host handle
#           : conn - filer connection handle
# RETURN    : NA
#-------------------------------------------------------------------------------

sub start_stress($$)
{
  my ($host, $conn)=@_;
  say($conn,"priv set test",-prompt,$host->{'prompt_norm'},-swallow);
  say($conn,"disk scrub start",-prompt,$host->{'prompt_norm'},-swallow);
  say($conn,"hammer -f -1 /vol/vol0/filename 1234",-prompt,$host->{'prompt_norm'},-swallow);
  say($conn,"filersio asyncio_active 00 -r 100 4k 0 7g 84000 5 /vol/vol0/file.90g -create",-prompt,$host->{'prompt_norm'},-swallow);
  sleep 10;
  my(undef,undef,$load,undef) = say($conn,"sysstat -c 5 1",-prompt,$host->{'prompt_norm'},-swallow);
  logcomment("Load is:$load");

}


#---------------------------------------------------------------------
# FUNCTION:	To Update SP/RLM firmware.
# INPUT   :
#			fc			=> filer connection handle
#			sp_con   => sp/rlm connection handle
#			build_path	=> absolute build path
#			sp_rlm		=> interface "RLM" or "SP"
#
# RETURN  :	1 = pass, 0 = fail
#---------------------------------------------------------------------
sub SP_Update_from_SP {

    my (%opt, $prossr, $cmd, $out, $result);
    %opt = @_;

    $prossr = "rlm"
      if ($opt{sp_rlm} =~ /RLM/i);
    $prossr = "sp"
      if ($opt{sp_rlm} =~ /SP/i);

    $cmd = "$prossr update $opt{build_path} -f";

    say($opt{sp_con}, $cmd, -prompt, ".*Update Successful",
        -swallow, -timeout, 4500);

    $cmd = "$prossr reboot";
    say($opt{sp_con}, $cmd, -prompt, "Do you want to reboot the SP|RLM now",
        -timeout, 60);
    say($opt{sp_con}, "y", -noprompt);

    (undef, undef, $out, undef) =
      say($opt{fc}, "$prossr status", -prompt, -swallow, -timeout, 60);

    if ($out !~ /Status: Rebooting/) {
        return 2;
    }

    $result = 0;
    for (0 .. 15) {
        my (undef, undef, $out) = say($opt{fc}, "$prossr status", -swallow);
        if ($out =~ m/Online/) {
            $result = 1;
            last;
        }
        snooze(40);
    }

    # $result = Is_SP_Online(fc => $opt{fc}, sp_rlm => $prossr, mode => "7mode");

    logcomment("Value of - $result");

    if ($result == 0) {
        return 0;
    } else {
        return 1;
    }

} ## end sub SP_Update_from_SP

#---------------------------------------------------------------------
# FUNCTION:	To verify the 'system acp sensor show'
# INPUT   :
#			fc			=> sp connection handle
#
# RETURN  :	1 = pass, 0 = fail
#---------------------------------------------------------------------
sub SP_System_ACP_Sensors_Show {

    my $sp     = shift;
    my $result = 0;

    (undef, undef, $result, undef) = say($sp, "system acp sensors show",
                              '-swallow', '-prompt', @prompts, '-timeout', 120);
    if (   ($result !~ /Disk device installed element list/)
        && ($result !~ /Power Supply installed element list/)
        && ($result !~ /ACP installed element list/)
        && ($result !~ /SAS Expander Module installed element lis/)) {
        return 0;
    } else {
        return 1;
    }

} ## end sub SP_System_ACP_Sensors_Show

#---------------------------------------------------------------------
# FUNCTION:	To verify the 'system sensors'
# INPUT   :
#			fc			=> sp connection handle
#
# RETURN  :	1 = pass, 0 = fail
#---------------------------------------------------------------------
sub SP_System_Sensors {

    my $sp = shift;
    my $sensor_status;
    (undef, undef, $sensor_status, undef) = say($sp, "system sensors ",
                              '-swallow', '-timeout', 200);
    return $sensor_status;

}

#---------------------------------------------------------------------
# FUNCTION:	To compare the sensor status from SP
# INPUT   : $[0] - output of 'system sensors'
# RETURN  :	1 = pass, 0 = fail
#---------------------------------------------------------------------
sub Sensor_Status_compare_SP($) {
    my $sensor_status = shift;
    my (@sensors, @environ_sensor, @sensor_info, @pass_sensors, @warn_sensors,
        @fail_sensors, @discreate_sensors);
    my ($count, $sensor_value, $sensor_details, $pass_sens_val, $fail_sens_val,
        $warn_sens_val, $discreate_sens_val, $total_count);
    my $flag = 1;
    $count = 1;
    logcomment("Called Complare function");
    @environ_sensor = split(/\n/, $sensor_status);

    foreach $sensor_value (@environ_sensor) {
        $count++;
        next if ($count < 5);
       # last if ($count == scalar(@environ_sensor));
        last if ($count > scalar(@environ_sensor));
        @sensor_info = split(/\s*\|/, $sensor_value);
        push(@sensors, $sensor_info[0]);
        $sensor_details = $sensor_info[0] . " --- " . $sensor_info[3];
        if ($sensor_info[2] !~ /discrete/) {
            if ($sensor_info[3] =~ /\s*ok\s*/i) {
                push(@pass_sensors, $sensor_details);
            } elsif (   ($sensor_info[3] =~ /nc/i)
                     || ($sensor_info[3] =~ /cr/i)
                     || ($sensor_info[3] =~ /NA/i)) {
                push(@warn_sensors, $sensor_details);
            } else {
                push(@fail_sensors, $sensor_details);
            }
        } else {
            push(@discreate_sensors, $sensor_details);
        }
    } ## end foreach $sensor_value (@environ_sensor)

    if (scalar(@pass_sensors) > 0) {
        logresult("PASS", "Displayed proper status for below sensors ");
        for $pass_sens_val (@pass_sensors) {
            logcomment("$pass_sens_val");
        }
    }

    if (scalar(@warn_sensors) > 0) {
        logresult("WARN", "Displayed unexpected status for below sensors");
        for $warn_sens_val (@warn_sensors) {
            logcomment("$warn_sens_val");
        }
        $flag = 0;
    }
    if (scalar(@fail_sensors) > 0) {
        logresult("FAIL", "Failed to display proper status for below sensors");
        for $fail_sens_val (@fail_sensors) {
            logcomment("$fail_sens_val");
        }
        $flag = 0;
    }

    if (scalar(@discreate_sensors) > 0) {
        logresult("PASS", "Below sensors are discreate sensors");
        for $discreate_sens_val (@discreate_sensors) {
            logcomment("$discreate_sens_val");
        }
    }
    return $flag;

} ## end sub Sensor_Status_compare_SP($)

#---------------------------------------------------------------------
# FUNCTION:	To verify the 'storage show acp'
# INPUT   :
#			fc			=> filer connection handle
#
# RETURN  :	1 = pass, 0 = fail
#---------------------------------------------------------------------
sub ONTAP_Storage_Show_ACP {

    my $fc     = shift;
    my $result = 0;

    (undef, undef, $result, undef) = say($fc, "storage show acp",
                              '-swallow', '-prompt', @prompts, '-timeout', 120);
    if (   ($result !~ /Alternate Control Path:\s*Enabled/)
        && ($result !~ /ACP Status:\s*Active/)
        && ($result !~ /ACP Connectivity Status: Partial Connectivity/)) {
        return 0;
    } else {
        return 1;
    }

} ## end sub ONTAP_Storage_Show_ACP

#---------------------------------------------------------------------
# FUNCTION:	To verify the 'environment shelf'
# INPUT   :
#			fc			=> filer connection handle
#
# RETURN  :	1 = pass, 0 = fail
#---------------------------------------------------------------------
sub ONTAP_Environment_Shelf {

    my $fc     = shift;
    my $result = 0;

    (undef, undef, $result, undef) = say($fc, "environment shelf",
                              '-swallow', '-prompt', @prompts, '-timeout', 120);
    if (   ($result !~ /Environment for channel/)
        && ($result !~ /Number of shelves monitored: \d+\s*enabled: yes/)
        && ($result !~ /Environmental failure on shelves on this channel\? no/))
    {
        return 0;
    } else {
        return 1;
    }

} ## end sub ONTAP_Environment_Shelf

#---------------------------------------------------------------------
# FUNCTION:	To verify the sensor status from ONTAP
# INPUT   :
#			fc			=> filer connection handle
#
# RETURN  :	1 = pass, 0 = fail
#---------------------------------------------------------------------
sub Sensor_Status_ONTAP {

    my $fc = shift;
    my $val;
    my (@sensors,      @environ_sensor, @sensor_info,
        @pass_sensors, @warn_sensors,   @fail_sensors);
    my ($count,         $sensor_status, $sensor_value, $sensor_details,
        $pass_sens_val, $fail_sens_val, $warn_sens_val);

    (undef, undef, $sensor_status, undef) =
      say($fc, "environment chassis list-sensors",
          -prompt, @prompts, -timeout, 500);
    # sleep(5);
    return $sensor_status;

} ## end sub Sensor_Status_ONTAP

#---------------------------------------------------------------------
# FUNCTION:	To compare the sensor status from ONTAP
# INPUT   : $[0] - output of 'environment chassis list-sensors'
# RETURN  :	1 = pass, 0 = fail
#---------------------------------------------------------------------
sub Sensor_Status_compare_ONTAP($) {

    my $sensor_status = shift;
    logcomment("Called Complare function");
    my (@sensors,      @environ_sensor, @sensor_info,
        @pass_sensors, @warn_sensors,   @fail_sensors);
    my ($count,         $sensor_value,  $sensor_details, $pass_sens_val,
        $fail_sens_val, $warn_sens_val, $total_count);
    my $flag = 1;
    $count          = 1;
    @environ_sensor = split(/\n/, $sensor_status);
    $total_count    = scalar(@environ_sensor);
    foreach $sensor_value (@environ_sensor) {
        $count++;
        next if ($count < 6);
        last if ($count > $total_count);

        @sensor_info = split(/\s\s+/, $sensor_value);
        push(@sensors, $sensor_info[0]);
        $sensor_details = $sensor_info[0] . " --- " . $sensor_info[1];
        if ($sensor_info[1] =~
            /normal|OK|PRESENT|IPMI_HB_OK|UP|GOOD|HA|Standalone|PRIMARY|ON|SP_RESET/i
          ) {
            push(@pass_sensors, $sensor_details);
        } elsif ($sensor_info[1] =~
                 /init_failed|critlow|crithigh|warnlow|warnhigh|HIGH|LOW/i) {
            push(@warn_sensors, $sensor_details);
        } else {
            push(@fail_sensors, $sensor_details);
        }
    } ## end foreach $sensor_value (@environ_sensor)

    if (scalar(@pass_sensors) > 0) {
        logresult("PASS", "Displayed proper status for below sensors ");
        for $pass_sens_val (@pass_sensors) {
            logcomment("$pass_sens_val");
        }
    }

    if (scalar(@warn_sensors) > 0) {
        logresult("WARN", "Displayed unexpected status for below sensors ");
        $flag = 0;
        for $warn_sens_val (@warn_sensors) {
            logcomment("$warn_sens_val");
        }
    }

    if (scalar(@fail_sensors) > 0) {
        logresult("FAIL", "Failed to display proper status for below");
        $flag = 0;
        for $fail_sens_val (@fail_sensors) {
            logcomment("$fail_sens_val");
        }
    }
    return $flag;
} ## end sub Sensor_Status_compare_ONTAP($)

#---------------------------------------------------------------------
# FUNCTION:	To Generate SES Traffic
# INPUT   :
#			fc			=> filer connection handle
#
# RETURN  :	1 = pass, 0 = fail
#---------------------------------------------------------------------
sub Generate_SES_Traffic {

    my $fc     = shift;
    my $result = 0;
    my $flag   = 0;

    say($fc, " priv set test; tp error force ses_scan 1 1 ",
        -prompt, @prompts, -timeout, 120);
    (undef, undef, $result, undef) =
      say($fc, " tp list ", -prompt, @prompts, -timeout, 120);
    sleep(5);

} ## end sub Generate_SES_Traffic

#----------------------------------------------------------------------------
# FUNCTION :  Display the fru information
# INPUT	  :	$_[0] = Filer connection handle
# RETURN  :
#----------------------------------------------------------------------------
sub System_fru_show($$) {
my ($rlm_naroot, $fc)=@_;
    my ($fru_info_0,              @fru_info,
        $fru_infoo,               $product_name_0,
        $produt_name_info_0,      $product_partnum_0,
        $produt_partnum_info_0,   $product_version_0,
        $produt_version_info_0,   $product_ser_num_0,
        $produt_ser_num_info_0,   $product_asset_tag_0,
        $produt_asset_tag_info_0, $custom_0,
        $custom_info_0,           $fru_info_1,
        $product_name_1,          $produt_name_info_1,
        $product_partnum_1,       $produt_partnum_info_1,
        $product_ser_num_1,       $produt_ser_num_info_1,
        $custom_1,                $custom_info_1,
        $fru_info_2,              $product_name_2,
        $produt_name_info_2,      $product_partnum_2,
        $produt_partnum_info_2,   $product_version_2,
        $produt_version_info_2,   $product_ser_num_2,
        $produt_ser_num_info_2,   $custom_2,
        $custom_info_2,           @prompts,
        $fru_infoo_2,             @fru_info_2);

    (undef, undef, $fru_info_0, undef) = say($rlm_naroot, "priv set test;system fru show 0",
                                    -swallow, -prompt, @prompts, -timeout, 200);
    @fru_info = split(/\n/, $fru_info_0);
    foreach $fru_infoo (@fru_info) {
        if ($fru_infoo =~ /Product Name/i) {
            ($product_name_0, $produt_name_info_0) = split(/\s\s+/, $fru_infoo);
        }

        if ($fru_infoo =~ /Product PartNum/i) {
            ($product_partnum_0, $produt_partnum_info_0) =
              split(/\s\s+/, $fru_infoo);
        }

        if ($fru_infoo =~ /Product Version/i) {
            ($product_version_0, $produt_version_info_0) =
              split(/\s\s+/, $fru_infoo);
        }

        if ($fru_infoo =~ /Product SerNum/i) {
            ($product_ser_num_0, $produt_ser_num_info_0) =
              split(/\s\s+/, $fru_infoo);
        }

        if ($fru_infoo =~ /Product AssetTag/i) {
            ($product_asset_tag_0, $produt_asset_tag_info_0) =
              split(/\s\s+/, $fru_infoo);
        }

        if ($fru_infoo =~ /Custom/i) {
            ($custom_0, $custom_info_0) = split(/\s\s+/, $fru_infoo);
        }
    } ## end foreach $fru_infoo (@fru_info)
    if (($produt_name_info_0 =~ /FAS/i) && ($produt_ser_num_info_0 =~ /\w+/i)) {
        logresult('PASS',
                  "Fru_info: Controller inventory data displayed properly");
    } else {
        logresult('FAIL',
              "Fru Info: Controller inventory data has not displayed properly");
              Validation_Items($fc, $rlm_naroot);
    }

    (undef, undef, $fru_info_1, undef) = say($rlm_naroot, "priv set test;system fru show 1",
                                    -swallow, -prompt, @prompts, -timeout, 200);
    @fru_info = split(/\n/, $fru_info_1);
    foreach $fru_infoo (@fru_info) {
        if ($fru_infoo =~ /Product Name/i) {
            ($product_name_1, $produt_name_info_1) = split(/\s\s+/, $fru_infoo);
        }

        if ($fru_infoo =~ /Product PartNum/i) {
            ($product_partnum_1, $produt_partnum_info_1) =
              split(/\s\s+/, $fru_infoo);
        }

        if ($fru_infoo =~ /Product SerNum/i) {
            ($product_ser_num_1, $produt_ser_num_info_1) =
              split(/\s\s+/, $fru_infoo);
        }

        if ($fru_infoo =~ /Custom/i) {
            ($custom_1, $custom_info_1) = split(/\s\s+/, $fru_infoo);
        }
    } ## end foreach $fru_infoo (@fru_info)

    if (   ($produt_name_info_1 =~ /FAS/i)
        && ($produt_partnum_info_1 =~ /\w+/i)
        && ($produt_ser_num_info_1 =~ /\w+/i)) {
        logresult('PASS',
                  "Fru info: Midplane inventory data displayed properly");
    } else {
        logresult('FAIL',
                "Fru info: Midplane inventory data has not displayed properly");
                Validation_Items($fc, $rlm_naroot);
    }

    (undef, undef, $fru_info_2, undef) = say($rlm_naroot, "priv set test;system fru show 2",
                                    -swallow, -prompt, @prompts, -timeout, 200);
@fru_info_2 = split(/\n/, $fru_info_2);
    foreach $fru_infoo_2 (@fru_info_2) {
        if ($fru_infoo_2 =~ /Product Name/i) {
            ($product_name_2, $produt_name_info_2) = split(/\s\s+/, $fru_infoo_2);
        }

        if ($fru_infoo_2 =~ /Product PartNum/i) {
            ($product_partnum_2, $produt_partnum_info_2) =
              split(/\s\s+/, $fru_infoo_2);
        }

        if ($fru_infoo_2 =~ /Product Version/i) {
            ($product_version_2, $produt_version_info_2) =
              split(/\s\s+/, $fru_infoo_2);
        }

        if ($fru_infoo_2 =~ /Product SerNum/i) {
            ($product_ser_num_2, $produt_ser_num_info_2) =
              split(/\s\s+/, $fru_infoo_2);
        }

        if ($fru_infoo_2 =~ /Custom/i) {
            ($custom_2, $custom_info_2) = split(/\s\s+/, $fru_infoo_2);
        }
    } ## end foreach $fru_infoo (@fru_info)
    if (   ($produt_name_info_2 =~ /\w+/i)
        && ($produt_partnum_info_2 =~ /\w+/i)
        && ($produt_ser_num_info_2 =~ /\w+/i)) {
        logresult('PASS',
                  "Fru show: NVMEM Bat inventory data displayed properly");
    } else {
        logresult('FAIL',
               "Fru show: NVMEM Bat inventory data has not displayed properly");
               Validation_Items($fc, $rlm_naroot);
    }

    return ($produt_partnum_info_1, $produt_name_info_0,
            $produt_name_info_1,    $produt_ser_num_info_1);

} ## end sub System_fru_show($)

#----------------------------------------------------------------------------
# FUNCTION :  Display the acp info
# INPUT	  :	$_[0] = Filer connection handle
# RETURN  :
#----------------------------------------------------------------------------

sub System_acp_show($$) {

my ($rlm_naroot, $fc)=@_;    
my ($acp_info,             @acp_infoo,           $acp_inf,
        $chassis_serNum,       $chassis_serNum_info, $chassis_partNum,
        $chassis_partNum_info, $midplan_prod_id,     $midplan_prod_id_info,
        $midplan_serNum,       $midplan_serNum_info, $board_prod_id,
        $board_prod_id_info1,  $board_serNum,        $board_serNum_info,
        $acp_mac_address,      $board_serNum_info,   @prompts);

    (undef, undef, $acp_info, undef) = say($rlm_naroot, "priv set test;system acp show",
                                    -swallow, -prompt, @prompts, -timeout, 200);
    @acp_infoo = split(/\n/, $acp_info);
    foreach $acp_inf (@acp_infoo) {
        if ($acp_inf =~ /chassis serial number/i) {
            ($chassis_serNum, $chassis_serNum_info) = split(/\:/, $acp_inf);
        }
        if ($acp_inf =~ /chassis part number/i) {
            ($chassis_partNum, $chassis_partNum_info) = split(/\:/, $acp_inf);
        }
        if ($acp_inf =~ /midplane product id/i) {
            ($midplan_prod_id, $midplan_prod_id_info) = split(/\:/, $acp_inf);

        }
        if ($acp_inf =~ /midplane serial number/i) {
            ($midplan_serNum, $midplan_serNum_info) = split(/\:/, $acp_inf);

        }
        if ($acp_inf =~ /board product id/i) {
            ($board_prod_id, $board_prod_id_info1) = split(/\:/, $acp_inf);
        }
        if ($acp_inf =~ /board serial number/i) {
            ($board_serNum, $board_serNum_info) = split(/\:/, $acp_inf);

        }
        if ($acp_inf =~ /acp mac address/i) {
            ($acp_mac_address, $board_serNum_info) = split(/\:/, $acp_inf);
        }

    } ## end foreach $acp_inf (@acp_infoo)
    if (   ($chassis_serNum_info =~ /\w+/i)
        && ($midplan_prod_id_info =~ /\w+/i)
        && ($midplan_serNum_info  =~ /\w+/i)
        && ($board_prod_id_info1  =~ /\w+/i)
        && ($board_serNum_info    =~ /\w+/i)) {

        logresult('PASS', "acp info: VPD data displayed properly");
    } else {
        logresult('FAIL', "acp info: VPD data not displayed properly");
        Validation_Items($fc, $rlm_naroot);
    }

    if ($acp_info =~ /status\s+:\s+valid partner detected/i) {
        logresult('PASS', "acp info: Partner module detected successfully");
    } else {
        logresult('FAIL', "acp info: Partner module not detected");
        Validation_Items($fc, $rlm_naroot);
    }

    if (   ($acp_info =~ /cpld interface\s+:\s+passed/i)
        && ($acp_info =~ /expander eth interface\s+:\s+passed/i)) {
        logresult('PASS', "acp info: Self-test passed");
    } else {
        logresult('FAIL', "acp info: Self-test failed");
        Validation_Items($fc, $rlm_naroot);
    }

    return ($midplan_prod_id_info, $midplan_serNum_info, $board_prod_id_info1);

} ## end sub System_acp_show($)

#----------------------------------------------------------------------------
# FUNCTION :  Display the acp model info
# INPUT	  :	$_[0] = Filer Rlm connection handle
# RETURN  :
#----------------------------------------------------------------------------

sub System_acp_model_show($$) {
my ($rlm_naroot, $fc)=@_;    
my ($acp_model_info, @acp_model_infoo,     $acp_model_inf,
        $brand,          $brand_name_info,     $pcm_type,
        $pcm_type_info,  $midplane_prod_id,    $midplane_prod_id_info,
        $board_prod_id,  $board_prod_id_info2, $model,
        $model_info,     @prompts);

    (undef, undef, $acp_model_info, undef) =
      say($rlm_naroot, "priv set test;system acp model show",
          -swallow, -prompt, @prompts, -timeout, 200);
    @acp_model_infoo = split(/\n/, $acp_model_info);
    foreach $acp_model_inf (@acp_model_infoo) {
        if ($acp_model_inf =~ /branding/i) {
            ($brand, $brand_name_info) = split(/\:/, $acp_model_inf);
        }
        if ($acp_model_inf =~ /PCM type/i) {
            ($pcm_type, $pcm_type_info) = split(/\:/, $acp_model_inf);
        }
        if ($acp_model_inf =~ /midplane product id/i) {
            ($midplane_prod_id, $midplane_prod_id_info) =
              split(/\:/, $acp_model_inf);
        }
        if ($acp_model_inf =~ /board product id/i) {
            ($board_prod_id, $board_prod_id_info2) =
              split(/\:/, $acp_model_inf);

        }
        if ($acp_model_inf =~ /Model/i) {
            ($model, $model_info) = split(/\:/, $acp_model_inf);
        }

    } ## end foreach $acp_model_inf (@acp_model_infoo)

    if (   ($brand_name_info =~ /FAS/i)
        && ($midplane_prod_id_info =~ /\w+/)
        && ($board_prod_id_info2   =~ /\w+/)
        && ($acp_model_info        =~ /Model\s+:\s*FAS\w+/i)) {
        logresult('PASS', "acp model data displayed successfully");
    } else {
        logresult('FAIL', "acp model data not displayed successfully");
Validation_Items($fc, $rlm_naroot);    
}

    return ($midplane_prod_id_info, $brand_name_info,
            $board_prod_id_info2,   $model_info);

} ## end sub System_acp_model_show($)

#----------------------------------------------------------------------------
# FUNCTION :  Display the acp model info
# INPUT	  :	$_[0] = Filer connection handle
# RETURN  :
#----------------------------------------------------------------------------
sub Storage_show_acp($$) {
    my ($rlm_naroot, $fc)=@_;
    my ($storage_acp_info, @prompts);
    (undef, undef, $storage_acp_info, undef) =
      say($fc, "priv set test;storage show acp", -swallow, -prompt, @prompts, -timeout, 200);
    if (   ($storage_acp_info =~ /Alternate Control Path:\s+Enabled/i)
        && ($storage_acp_info =~ /Ethernet Interface:\s+e0P/i)
        && ($storage_acp_info =~ /ACP Status:\s+Active/i)) {
        logresult('PASS', "storage acp data verified successfully");
    } else {
        logresult('FAIL', "storage acp data is not correct");
Validation_Items($fc, $rlm_naroot);    
}

} ## end sub Storage_show_acp($)


#----------------------------------------------------------------------------
# FUNCTION :  Display the acp model info
#INPUT     : sp - Rlm connection handle
#           : fc - filer connection handle
# RETURN  :
#----------------------------------------------------------------------------

sub Validation_Items($$) {
logcomment("Executing validation items");
my ($fc, $rlm_naroot)=@_;
    say($fc, "sysconfig -v", -swallow,  -prompt, @prompts, -timeout, 100);
    say($fc, "ifconfig -a", -swallow,  -prompt, @prompts, -timeout, 60);
    say($rlm_naroot, "sp status -d", -swallow,  -prompt, @prompts, -timeout, 1000);
    say($rlm_naroot, "events newest 10", -swallow,  -prompt, @prompts, -timeout, 60);
    say($fc, "storage show acp", -swallow, -prompt, @prompts, -timeout, 100);
    say($fc, "environ chassis list-sensors",  -prompt, @prompts, -swallow, -timeout, 100);
    say($rlm_naroot, "system sensors", -swallow,  -prompt, @prompts, -timeout, 100);

} ## end sub Commands
1;
