# -*- perl -*-
# $Id: //depot/prod/test/main/lib/platval/service_processor/asup.pm
#
#       Copyright (c) 2009 Network Appliance, Inc.
#       All Rights reserved
#
## @summary Routines for validating SP ASUP testcases.
##
## @description
##    This library contains individual routines for ONTAP, SP and Boot Loader
##    for testing of ASUP module for both configuration and functionality related 
##    tests.ASUP scripts are  not fully automated user has to check the mail IDs configured to check if mails were received.
##
## Sub-Routines:
## 	1.events_search
##	To run the events search "string" on SP side and to collect the output in an Array.
##
##	2.EMS_specific 
##	This subroutine takes the specific string to be searched and the return Array of events_search as input and returns
##	the last two instances of the string in the array.
##
##	3.check_mailhosts
##	This subroutine takes a pointer to mailhosts to be used from the configuration file and checks if all are pinging 
##	if all mailhosts are not pinging it specifies the mailhosts which are not pinging and gives a fatal error
##
##	4.set_options
##	This will set the various options for ASUP in the options.autosupport field
##
##	5.check_options
##	This will check if the options we set for ASUP configuration is same as options.autosupport or not
##
##	6.check_mailhosts
##	This will check if sp status -d/rlm_config_from_filer options are matching with options autosupport options or not
##
##	7.Check_Result
##	This checks the last two EMS message logged for the particular ASUP before and after the running a particular testcase 
##	and determines if the testcase passed accordingly if an ASUP was generated or not.
##
##	8.set_priv
##	This sets the privilege as required for both SP and ONTAP side.
##
##	9.ping5
##	This will give the information if an IP or filer is pinging or not.	
##
##	10.SP_SIDE_ASUP_CHECK
##	This function checks if ASUP is generated or not for sp test autosupport,system power cycle,
##	system power off,system reset,system reset backup,system core,watchdog hang and watchdog_reset
##	
##	11.ASUP_Heartbeat_SP	
##	This will test if ASUP from SP is generated when heartbeat from ONTAP to SP is missing	
##
##	12.ASUP_Heartbeat_ONTAP
##	This wil test if ASUP from ONTAP is generated when heartbeat from SP to ONTAP is missing
##
##	13.load_functionality_testcases
##	This will load all testcases related to ASUP functionality.
##
##	14.verify_functionality_testcases
##	This will verify if all functionality testcases are valid.
##
##	15.load_SP_Config_testcases	
##	This will load all the SP setup testcases
##	
##	16.verify_SP_Config_testcases
##	This will verify if all the sp setup testcases are valid.
##
##	17.load_ASUP_options_testcases
##	This will load all the testcases related to options.autosupport
##
##	18.verify_ASUP_options_testcases
##	This will verify if all the options autosupport related testcases are valid.
##
## @synopsis
##      use asup;
##
## @keywords
##   sp,asup,functionality,ems,configuration,events,mailhost.   
##
## @author  divyab@netapp.com,dhivakar@netapp.com,jkonjeti@netapp.com, ravikira@netapp.com, dl-rlm_ipv6_validation@netapp.com, dl-sp-validation@netapp.com
##
######################################################################################################################################


require "tharnit_lib.pl";
our ($FILER,$fc,$PROMPT_RLM,$default_ip,$rlm_ssh_ip,$CLIENT,$CLIENT_CONN,$SP_ssh,%config_data, %ASUP_config_testcases,%SP_Config_testcases,$NO_IPv6,%functionality_testcases,@valid_cases,$Pass_Flag,$PollAgain,$PollEnable,$Global_Config,$Global_Setup,$Global_Func,$testcase,$From);

#rlm_load_ipv4_ipv6_config_cases($FILER); #rlm_ipv6.pm

use platval::service_processor::rlm_ipv6;


    my $h = rlm_make_host_to_ontap($FILER);
    rlm_load_tharnhost_data($FILER);
    $default_ip = $h->{'default_ip'};
## @name events_search
## This subroutine takes a single word as input for the "events search xxx" command in SP/RLM and return the array containing all the events given as output by the command.
sub events_search($) 
{
	my $search_string=shift;
	my $EMS_ASUP;
	$track=0;
	while($track<5)
	{	eval{(undef,undef,$EMS_ASUP) =say ($SP_ssh,"events search $search_string",'-swallow','-timeout', 1000);};
		if($@)
		{	sleep(120);
			logcomment("Sending events search command again as it failed the first time");
		}
		else
			{last;}
	$track++;
	}
	if($track==5)
	{logresult('FATAL',"Events search command failed five times");}	
		
	@retval=split(/\n/,$EMS_ASUP);
}

## @name EMS_specific
## This subroutine takes the Specific EMS message we want to search in the SP EMS and the array returned by events_search and gives the last and the secondlast element that 
## match the Specific EMS.
sub EMS_specific
{
	my($ASUP_title,@EMS);
	($ASUP_title,@EMS) =@_;
	my $count=0;
	my $count_After=0;
	my @EMS_After;
	while($EMS[$count])
	{
		if($EMS[$count] =~ /\Q$ASUP_title\E/)
		{
			$EMS_After[$count_After]=$EMS[$count];
			$count_After++;
		}
		$count++;
	}
	#logcomment("The ASUP messages extracted are :: $EMS_After[-1],$EMS_After[-2]\n");
	return $EMS_After[-1],$EMS_After[-2];
	
}

## @name check_mailhosts
## This subroutine takes a pointer to an array of mailhosts and checks if they are pinging or not.The Second argument is to be given as 1 if we are checking IPV6 mailhosts.
## It will pass if all mailhosts in the array are pinging , else fail.
sub check_mailhosts($;$)
{
        $Mailhost=shift @_;
        @Not_Pinging;
        $curr=0;
	my $ping_host;
	$Mailhost6_enable= shift @_;
        while($Mailhost->[$curr])
        {	
		if($Mailhost6_enable==1)
			{(undef, undef, $ping_host) = say ($cc,"ping6 -c 5 $Mailhost->[$curr]");}
		else
        	        {(undef, undef, $ping_host) = say ($cc,"ping -c 5 $Mailhost->[$curr]");}

                if($ping_host =~  /5\s+packets\s+transmitted\,\s+[1-5]\s+received\,\s+(0\%\s+loss|[1-9][0-9]\%\s+loss)/i
                        | $ping_host =~  /5\s+packets\s+transmitted\,\s+[1-5]\s+received\,\s+(0\%\s+packet loss|[1-9][0-9]\%\s+packet loss)/i)

                	{logcomment ("The mailserver $Mailhost->[$curr] is pinging\n");}
                else
                	{@Not_Pinging=(@Not_Pinging,$Mailhost->[$curr]);}
                $curr++;
        }
        if(@Not_Pinging)
        	{logresult('FAIL',"@Not_Pinging is not pinging \n");}
}

## @name set_options
## This sets the valus of options autosupport.It takes a pointer to an array as argument and the array should have values for the test case title,autosupport.enable,
## autosupport.content,autosupport.from,autosupport.mailhost,autosupport.to,autosupport.support.enable,autosupport.support.to in order.
sub set_options($)
{ 
	my($title,$enable,$content,$from,$Mailhost,$MailID,$enable_Netapp,$NETAPP_MAIL_ID);
	($title,$enable,$content,$from,$Mailhost,$MailID,$enable_Netapp,$NETAPP_MAIL_ID)=@{$_[0]};
	logcomment(" The options being set for the particular test case:: $title is ::\n$enable,$content,$from,$Mailhost,$MailID,$enable_Netapp,$NETAPP_MAIL_ID\n");
        $From = "$from\@netapp.com"; 
	say($fc, "options autosupport.to $MailID",'-prompt',"@prompts",'-swallow','-timeout', 300);
	say($fc,"options autosupport.enable $enable",'-prompt',"@prompts",'-swallow','-timeout', 300);
	say($fc,"options autosupport.content $content",'-prompt',"@prompts",'-swallow','-timeout', 300);
	say($fc,"options autosupport.mailhost $Mailhost",'-prompt',"@prompts",'-swallow','-timeout', 300);
	say($fc,"options autosupport.from $From",'-prompt',"@prompts",'-swallow','-timeout', 300);
	set_priv($fc,"test");
	say($fc,"options autosupport.support.enable $enable_Netapp",'-prompt',"@prompts",'-swallow','-timeout', 300);
	say($fc,"options autosupport.support.to $NETAPP_MAIL_ID",'-prompt',"@prompts",'-swallow','-timeout', 300);
}

## @name check_spstatusd
## This check the values of various fields in options.autosupport with spstatus -d /rlm_config_from_filer to check that the propagated values to SP are same as
## options.autosupport.Takes the array of values extracted from options .auotsupport as input. Options.auotsupport.to is not being checked.
## $Case_validity gives the case number so that if we want to skip any comparison for a particular testcase we can do so.
sub check_spstatusd
{	$case_validity= shift @_;
        $enable=shift @_;
	$content=shift @_;
	$from=shift @_;
	$Mailhost=shift @_;
	$MailID=shift @_;
	$enable_Netapp=shift @_;
	$NETAPP_MAIL_ID=shift @_;
	logcomment("The options we set from options autosupport are :: $enable,$content,$from,$Mailhost,$MailID,$enable_Netapp,$NETAPP_MAIL_ID\n");
	#my(undef,undef,$string)=say($SP_ssh,"sp status -d",'-swallow','-timeout', 600);
	#$string=~/cat \/mnt\/logs\/etc\/rlm_config_from_filer\s+((.+\s+)+)cat \/mnt\/logs\/etc\/rlm_config_to_filer/i;
	my (undef,undef,$string,$prompt)=say($fc,"rdfile //etc/rlm_config_from_filer",'-timeout', 100);
print "\$string = $string\n\$prompt = $prompt\n";
        my (undef,undef,$string,$prompt)=say($fc,"rdfile //etc/rlm_config_from_filer",'-timeout', 100);
print "\$string = $string\n\$prompt = $prompt\n";
        sleep 30;
	#$string="'".$1."'";
=comment
	$string=~/asup_enabled\s(\w+)\s+asup_to_netapp_enabled\s(\w+)\s+(.+\s+)+asup_from\s(\S+)\s+autosupport.content\s(\w+)/;
	$enable_sp=$1;
	$enable_Netapp_sp=$2;
	$from_sp=$4;
	$content_sp=$5;
	$string=~/mailhost\[0\]\s+(.*)\s*mailhost\[1\]\s+(.*)\s*mailhost\[2\]\s+(.*)\s*mailhost\[3\]\s+(.*)\s*mailhost\[4\]\s+(.*)\s*asup/;
	$Mailhost_sp=join(",",$1,$2,$3,$4,$5);
	$Mailhost_sp=~s/,,+|\s+//g;
	$Mailhost_sp=~s/,$//;
	$string =~/asup_to_netapp\[0\]\s+(.*)\s*(.+\s+)+asup_to_netapp\[1\]\s+(.*)\s*asup(.+\s+)+asup_to_netapp\[2\]\s+(.*)\s*asup(.+\s+)+asup_to_netapp\[3\]\s+(.*)\s*asup(.+\s+)+asup_to_netapp\[4\]\s+(.*)\s*asup(.+\s+)+tuples/;
	$NETAPP_MAIL_ID_sp=join(",",$1,$3,$5,$7,$9);
	$NETAPP_MAIL_ID_sp=~s/,,+|\s+//g;
	$NETAPP_MAIL_ID_sp=~s/,$//;
	$string =~/asup_to_local\[0\]\s+(.*)\s*cb_to_net(.+\s+)+asup_to_local\[1\]\s+(.*)\s*cb_to_net(.+\s+)+asup_to_local\[2\]\s+(.*)\s*cb_to_net(.+\s+)+asup_to_local\[3\]\s+(.*)\s*cb_to_net(.+\s+)+asup_to_local\[4\]\s+(.*)\s*cb_to_net(.+\s+)+tuples/;
	$MAIL_ID_sp=join(",",$1,$3,$5,$7,$9);
	$MAIL_ID_sp=~s/,,+|\s+//g;
	$MAIL_ID_sp=~s/,$//;
	$retval=1;
	logcomment("The values extracted from sp status -d are :: $enable_sp,$enable_Netapp_sp,$content_sp,$from_sp,$MAIL_ID_sp,$Mailhost_sp,$NETAPP_MAIL_ID_sp\n");
	if($enable ne $enable_sp)
	{
                logcomment ("ASUP enable field is not properly propagated to SP \n");
                $retval=0;
        }
	if($enable_Netapp ne $enable_Netapp_sp)
	{
		logcomment("Netapp MAIL ID enable field is not properly propagated to SP\n");
		$retval=0;
	}
	if($content_sp ne $content)
	{
		logcomment("Content field is not properly propagated to SP\n");
		$retval=0;
	}
	if($from_sp ne $from)
	{
		logcomment("From field is not properly propagated to SP\n");
		$retval=0;
	}
	if($MailID ne $MAIL_ID_sp)
	{
		logcomment("Mail IDs are not propagated to SP \n");
		$retval=0;
	}
	if($case_validity==16 || $case_validity==24)
		{logcomment("Not matching mailhosts for the testcase ::$case_validity\n");}
	else
	{
		if($Mailhost ne  $Mailhost_sp)
		{
			logcomment("Mailhosts are not porperly propagated to SP \n");
			$retval=0;
		}
	}
	#if($NETAPP_MAIL_ID ne $NETAPP_MAIL_ID_sp)
	#{
	#	logcomment("Netapp mail IDs are not properly propagated to SP\n");
	#	$retval=0;
	#}
=cut
	if($retval ==0)
        	{logresult('FAIL'," ASUP options from options autosupport and SP status -d are not the same\n");}
        else
       		{logresult ('PASS',"All ASUP options are same on options autosupport and SP status -d \n");}

	
}

## @name check_options
## This checks the options set by set_options subroutine with the output of options.autosupport.Autosupport.support.to is not being checked.
## Takes the testcase number and reference to an array as inputs.
sub check_options($$)
{	
	my($title,$enable,$content,$from,$Mailhost,$MailID,$enable_Netapp,$NETAPP_MAIL_ID);
        $case_validity =shift @_;
	($title,$enable,$content,$from,$Mailhost,$MailID,$enable_Netapp,$NETAPP_MAIL_ID)=@{$_[0]};
	my(undef,undef,$ASUP_INFO) = say ($fc, "options autosupport",'-swallow','-prompt',"@prompts",-timeout, 300);
	$ASUP_INFO =~ /autosupport\.to\s+(.+)/;
	my $flag=1;
	my @retval;

	if ($enable =~/^(yes|on|1|true)$/i)
                {$enable="on";}
        elsif($enable =~/^(0|off|false|no)$/i)
                {$enable="off";}
	
	$ASUP_INFO =~ /autosupport.enable\s+(\w+)/;
	if($enable ne $1)
	{
		logcomment ("ASUP enable is not set properly and the value in options autosupport is ::$1 \n");
		$flag=0;
	}
	@retval=($1);
	$ASUP_INFO =~ /autosupport.content\s+(\w+)/;
	if($content ne $1)
	{
		logcomment ("Content field is not set properly and the value in options autosupport is ::$1\n");
		$flag=0;
	}
	push(@retval,$1);
	$ASUP_INFO =~ /autosupport.from\s+(\S+)/;
print "\From = $From and \$1 = $1\n";
	if($From ne $1)
	{
		logcomment("From field is not set properly and the value in options autosupport is ::$1\n");
		$flag=0;
	}
	push(@retval,$1);
	$ASUP_INFO=~ /autosupport.mailhost\s+([\w\.\-\@\,\: ]+)/;	
	$temp =$1;
	$temp =~s/ \,|\, /\,/g;
	$temp=~s/\s+//g;
	push(@retval,$temp);
	if($Mailhost ne $temp)
	{
		logcomment ("Mailhost is not set properly and the value in options autosupport is ::$1\n");
		$flag=0;
	}
	$ASUP_INFO=~ /autosupport.to\s+([\w\.\-\@\, ]+)/;
	$temp =$1;
        $temp =~s/ \,|\, /\,/g;
	$temp=~s/\s+//g;
        push(@retval,$temp);
	if($case_validity ==17 ||$case_validity ==18)
		{logcomment("Skipping Mail ID validation with options autosupport for testcase ::$case_validity\n");}
	else
	{
		if($MailID ne $temp)
		{	
			logcomment ("Mail ID was not correctly set properly and the value in options autosupport is ::$1\n");
			$flag=0;
		}
	}
	$ASUP_INFO =~ /autosupport.support.enable\s+(\w+)/;
	if($enable_Netapp ne $1)
	{
		logcomment("Autosupport.support.enable field is not set properly and the value in options autosupport is ::$1\n");
		$flag=0;
        }
	push(@retval,$1);
	$ASUP_INFO =~/autosupport.support.to\s+([\w\.\-\@\, ]+)/;
        $temp =$1;
        $temp =~s/ \,|\, /\,/g;
	$temp=~s/\s+//g;
        push(@retval,$temp);
	if($flag ==0)
		{logresult('FAIL'," Check ASUP options before proceeding\n");}
	else
		{logresult ('PASS',"All ASUP options are set properly \n");}
	return @retval;
}

## @name set_date
##This checks the date command in ONTAP and sleeps for 1/2 minutes if the minute field is 58/59.
sub set_date
{	
	while(1)
	{
		my(undef,undef,$retval) = say ($fc,"date",'-swallow','-timeout', 300);
		$retval =~ /(Mon|Tue|Wed|Thu|Fri|Sat|Sun)\s+(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s+([0-3]?[0-9])\s+([0-2][0-9])\:[0-6][0-9]\:[0-6][0-9](.+(19|20)[0-9][0-9])/;
		if ($5 == 58)
			{sleep(120);}
		elsif($5 == 59)
			{sleep 60;}	
		else
			{return $retval;}
		
	}
	
}

## @name Check_Result
## This takes the last EMS/SEL generated before the testcase and the second last and the last EMS/SEL message after the test case as arguments.
## $No_EMS should be set to 1 if the testcase is not supposed to generate any EMS and $Pass_Flag1 is the global variable $Pass_Flag which tells if the test case should pass or
## fail. This subroutine will check if an EMS was generated for a particular test case or not and if the EMS/SEL is an ASUP EMS it can also check if ASUP sending passed or failed.
sub Check_Result($$$;$$)
{
	undef $No_EMS;
	undef $Pass_Flag1;
	undef $PollAgain;
	my($EMS_Before_Last,$EMS_After_Penultimate,$EMS_After_Last,$No_EMS,$Pass_Flag1)=@_;
	logcomment("The SEL logs we are checking are ::\nSEL generated Before test case execution ::$EMS_Before_Last\nLast 2 SEL generated after the test case execution::$EMS_After_Penultimate\n$EMS_After_Last\n");
	if($No_EMS == 1)
	{	if($EMS_Before_Last eq $EMS_After_Last)
			{logresult('PASS',"No SEL message was generated for this testcase\n");}
		elsif (%ASUP_options_testcases)
			{logresult('FAIL',"$SP_Config_testcases{$Global_Setup}->[0],$ASUP_options_testcases{$Global_Config}->[0],$functionality_testcases{$Global_Func}->[0]\nSEL log was generated for this test-case\n");}
		else
		{logresult('FAIL',"SEL message was generated for this test-case\n");}
	}
	else
	{
        if ($testcase == 6) {
        logcomment(" No SEL message is generated for the test case \n");
        } else {
		if($EMS_Before_Last eq $EMS_After_Penultimate && ($Pass_Flag1==2 || $Pass_Flag1==0))
		{	if($EMS_After_Last =~/Send Failed/i)
			{	if(%ASUP_options_testcases)
					{logresult('FAIL',"$SP_Config_testcases{$Global_Setup}->[0],$ASUP_options_testcases{$Global_Config}->[0],$functionality_testcases{$Global_Func}->[0]\nThe SEL message for the test case is :: $EMS_After_Last\n");}
				else
					{logresult('FAIL',"The SEL message for the test case is :: $EMS_After_Last\n");}
			}
			elsif($EMS_After_Last)
				{logresult('PASS',"The SEL message for the test case is :: $EMS_After_Last\n");}
			else
			{	if(%ASUP_options_testcases)
				{	if($PollEnable == 1)
                                	{       logcomment("No SEL message is generated for this testcase,checking again\n");
                                        	$PollAgain=1;
                                	}

	                                else
					{	logresult('FAIL',"$SP_Config_testcases{$Global_Setup}->[0],$ASUP_options_testcases{$Global_Config}->[0],$functionality_testcases{$Global_Func}->[0]\nNo SEL message is generated for this testcase\n");}
				}
				else
				{logresult('FAIL',"No SEL message is generated for this testcase\n");}
                        }                           
		}
		elsif($EMS_Before_Last eq $EMS_After_Penultimate && $Pass_Flag1==1)
		{
                $ASUP_SEARCH = $config_data{ASUP_SEARCH}->[$testcase];
         	 if($EMS_After_Last =~/Send Failed/i)
				 {logresult('PASS',"Negative TestCase::The SEL message for the test case is :: $EMS_After_Last\n");}
	        	elsif($EMS_After_Last)
				{logresult('FAIL',"$SP_Config_testcases{$Global_Setup}->[0],$ASUP_options_testcases{$Global_Config}->[0],$functionality_testcases{$Global_Func}->[0]\nNegative TestCase::The SEL message for the test case is :: $EMS_After_Last\n");
                        }
                        else
                        	{logresult('FAIL',"$SP_Config_testcases{$Global_Setup}->[0],$ASUP_options_testcases{$Global_Config}->[0],$functionality_testcases{$Global_Func}->[0]\nNegative TestCase::No SEL message is generated for this testcase\n");

                }
                }
		elsif($EMS_Before_Last eq $EMS_After_Last)
		{
        	if(%ASUP_options_testcases)
			 {	if($PollEnable == 1)
                                        {       logcomment("No SEL message is generated for this testcase,checking again\n");
                                                $PollAgain=1;
                                        }

                                else
					{logresult('FAIL',"$SP_Config_testcases{$Global_Setup}->[0],$ASUP_options_testcases{$Global_Config}->[0],$functionality_testcases{$Global_Func}->[0]\nNo SEL message was generated for this testcase\n");}
			 }
			else
				{logresult('FAIL',"No SEL message was generated for this testcase\n");}
		}
		else
		{	if(%ASUP_options_testcases)
				{logresult('FAIL',"$SP_Config_testcases{$Global_Setup}->[0],$ASUP_options_testcases{$Global_Config}->[0],$functionality_testcases{$Global_Func}->[0]\nMore than one SEL messages were generated in the meantime the last two EMS messages are :: $EMS_After_Penultimate\n,$EMS_After_Last\n");}
			else
			{logresult('FAIL',"More than one SEL messages were generated in the meantime the last two SEL messages are :: $EMS_After_Penultimate\n,$EMS_After_Last\n");}
		}
            }
        }
}

## @name valid_positive_testcase
## This subroutine checks if the options autosupport testcases should pass or fail according to SP setup and ASUP configuration and accordingly populates a matrix for each SP
##setup.
sub valid_positive_testcase($)
{	$SP_config_case=shift @_;
	undef @valid_cases;
	if($SP_config_case==1 || $SP_config_case==2)
		{@valid_cases=(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,17,18,19,20,21,22,27,28,29,31,33);}
	if($SP_config_case==3)
		{@valid_cases=(23,25,26,27,28,29,30,31,32,33);}
	if($SP_config_case==4)
		{@valid_cases=(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,17,18,19,20,21,22,23,25,26,27,28,29,30,31,32,33);}
}

## @name pass_flag
## This sets $Pass_Flag as true or false according to the Autosupport options test case number by checking the @valid_cases matrix for the particular sp setup.
sub pass_flag($)
{	$testcase=shift @_;
	undef $Pass_Flag;
	$track=0;
	logcomment("the valid test cases are :: @valid_cases\n");
	foreach(@valid_cases)
	{	if($valid_cases[$track]==$testcase)
		{	$Pass_Flag=2;
			logcomment("Setting the flag to 2 as it is a positive testcase\n");
			return;
		}
		else
			{$track++;}
	}
	$Pass_Flag=1;
}	

## @name set_priv
## This sets the privilege level for the filer or SP and returns true false if the value is set or not respectively.
sub set_priv($$)
{	
	my $connection=shift @_;
	my $priv_value=shift @_;
	say($connection,"priv set $priv_value",'-swallow','-timeout',30);
	my(undef,undef,$out_priv)=say($connection,"priv",'-swallow','-timeout',30);
	if($out_priv =~ /$priv_value/)
		{$retval=1;}
	else
		{$retval=0;}
	return $retval;
}  

## @name sp_SEL_setup_clear
## This function is used to setup an alternate log system for SP by passing a true value to the function or to restore the original log on passing a false value to this funtion.
sub sp_SEL_setup_clear($)
{	
	$setup=shift @_;
	if($setup==1)
	{	
		logcomment("Setting up an alternative SP event log file for testing by taking a backup of original SP SEL log file\n");
		my(undef,undef,$SP_response) = say($SP_ssh_root,"cd /mnt/logs",'-swallow','-noprompt','-prompt',"#",'-timeout',100);
		(undef,undef,$SP_response) = say($SP_ssh_root,"ls",'-swallow','-noprompt','-prompt',"#",'-timeout',100);
                if($SP_response =~ /\s+sel\s+/i) 
		{
                    say($SP_ssh_root,"mv sel sel1",'-swallow','-noprompt','-prompt',"#",'-timeout',100);
                    (undef,undef,$SP_response)= say($SP_ssh_root,"ls",'-swallow','-noprompt','-prompt',"#",'-timeout',100);
                    if($SP_response != /^sel1$/) 
		     	 {logcomment("Failed to rename sel directory to sel1");}
                }
                else 
	        	 {logcomment("System event log file is not present in /mnt/logs location");}
        } 
	elsif($setup==0)
	{
		logcomment("Clearing the alternative SEL log file which was used for testing and then backup the original SP SEL log file");
        	say($SP_ssh_root,"rm -rf /mnt/logs/sel/*",'-swallow','-noprompt','-prompt',"#",'-timeout',100);
        	say($SP_ssh_root,"rmdir /mnt/logs/sel/",'-swallow','-noprompt','-prompt',"#",'-timeout',100);
		(undef,undef,$SP_response)= say($SP_ssh_root,"cd /mnt/logs/sel/",'-swallow','-noprompt','-prompt',"#",'-timeout',100);
		if($SP_response=~ /can\'t cd to \/mnt\/logs\/sel/) 
			{logcomment("Successfully deleted the alternative SEL log file")}
        	else
			{logcomment("Failed to delete the alternative SEL log file");}

		say($SP_ssh_root,"mv /mnt/logs/sel1 /mnt/logs/sel/",'-swallow','-noprompt','-prompt',"#",'-timeout',100);
        	$SP_response = say($SP_ssh_root,"ls /mnt/logs/",'-swallow','-noprompt','-prompt',"#",'-timeout',100);
        	if($SP_response =~ /\s+sel\s+/i)
			{logcomment("Restored the original SEL file\n");}
               
         	else 
			{logcomment("Failed to restore the original SEL file");}
 
	}
}

## @name ping5
## This function is used to check if any connection (filer,sp client) is pinging or not.It returns 1 if pinging and 0 if not pinging.
sub ping5 ($$)
{

	($cc,$Ping_this)=@_;
	my(undef, undef, $ping_host) = say ($cc,"ping -c 5 $Ping_this");

	if($ping_host =~  /5\s+packets\s+transmitted\,\s+[1-5]\s+received\,\s+(0\%\s+loss|[1-9][0-9]\%\s+loss)/i 
		| $ping_host =~  /5\s+packets\s+transmitted\,\s+[1-5]\s+received\,\s+(0\%\s+packet loss|[1-9][0-9]\%\s+packet loss)/i)
		
	{	
		logcomment(" Filer ip $Ping_this is pinging\n");
		$retval=1;
	}
	else
	{	logcomment (" Filer ip $Ping_this is not pinging\n");
		$retval=0;
	}
	return $retval;
}

## @name SP_SIDE_ASUP_CHECK
## This function checks the ASUP generation for sp test autusupport,system power cycle,system power off,system reset,system reset backup,system core,watchdog hang,watchdog reset
## It first stores all EMS we need to check after giving the command for the test(only the last EMS of each type before running the test). Then it runs the command and checks
## for the EMS strings after the command and checks the result by the check_result subroutine. If we need to check for reminder mail generation a wait of 15 minutes is given.
sub SP_SIDE_ASUP_CHECK($$)
{	
	rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
	$testcase =shift @_;
	$case_validity=shift @_;
	@CONN=($fc,$SP_ssh,$SP_ssh,$SP_ssh,$SP_ssh,$SP_ssh,$fc,$fc);
	my($SP_COMMAND,$SP_ASUP,$ASUP_SEARCH,$Command_Conn)=($config_data{SP_COMMAND}->[$testcase],$config_data{SP_ASUP}->[$testcase],$config_data{ASUP_SEARCH}->[$testcase],$CONN[$testcase]);
        my $TEST = $config_data{ASUP_SEARCH}->[$testcase];
	logcomment("$SP_COMMAND\n$SP_ASUP\n$ASUP_SEARCH\n");
	undef @EMS;
	undef @EMS_Heartbeat;
	undef @EMS_Reboot;
	$Reboot_msg='[SP.critical]: Filer Reboots';
	@EMS=events_search($ASUP_SEARCH);
	unless($SP_COMMAND eq "sp test autosupport")
	{	@EMS_Reboot=events_search("Reboots");
		($Reboot_Before,undef)=EMS_specific($Reboot_msg,@EMS_Reboot);
	}
	if($SP_COMMAND ne "sp test autosupport")
	{	$REMINDER_ASUP =$SP_ASUP;
        	$REMINDER_ASUP =~ s/First notification/Reminder/;
		logcomment("REMINDER_ASUP=$REMINDER_ASUP,SP_ASUP=$SP_ASUP\n");
		$Heartbeat='[SP.critical]: Heartbeat stopped';
		@EMS_Heartbeat=events_search("Heartbeat");
		($EMS_Heartbeat_Before,undef)=EMS_specific($Heartbeat,@EMS_Heartbeat);
		($EMS_REMINDER_Before,undef)=EMS_specific($REMINDER_ASUP,@EMS);
	}
	($EMS_Before,undef)=EMS_specific($SP_ASUP,@EMS);
        sleep 300;
	#logresult('FAIL',"$FILER is not pinging please check ONTAP setup before proceeding\n") if(ping5($cc,$FILER)==0);

logresult('FAIL',"$FILER is not pinging please check ONTAP setup before proceeding\n") if(ping5($cc,$default_ip)==0);
	set_priv($fc,"test");
	say($fc,"bootargs set AUTOBOOT FALSE",'-swallow','-timeout',30);
	 if($Command_Conn ==$SP_ssh)
	{	say($SP_ssh,"$SP_COMMAND",'-noprompt','-wait','This will cause a dirty shutdown of your appliance','-swallow','-timeout',100);
		say($SP_ssh,"y",'-prompt',"SP $FILER\>",'-swallow','-timeout',30);
	}
        elsif($SP_COMMAND eq "watchdog wd_reset"){
        logcomment("Executing for watchdog reset\n");
        my (undef,undef,$output,$prompt) = say($fc,"priv set test; help watchdog", -noprompt, -prompt, "$FILER", @prompts, '-swallow','-timeout',30);
        if ($output =~ /unrecognized command/) {
                my (undef, undef, $out, $prompt) = say($fc, "priv set test; systemshell", -noprompt, -prompt, "login\:", @prompts, -timeout, 30);

                if ($out =~ /systemshell not found/) {
                         # Running the watchdog wd_reset command on filer
                         say($fc,"watchdog wd_reset",'-swallow','-blind','-noprompt','-timeout',900);
                } elsif ($prompt =~ /login\:/) {
                        say($fc,"diag",-noprompt, -prompt,"Password\:",'-timeout',90);
                        say($fc,"netapp1\!",'-timeout',90);

                        say($fc,"sudo sysctl debug.watchdog",'-timeout',90);
                        say($fc,"sudo sysctl debug.watchdog.disable=0",'-timeout',90);
                        say($fc,"sudo sysctl debug.watchdog.ignore_nmi=1",'-timeout',90);
                        say($fc,"sudo sysctl debug.watchdog.trigger_l2_timeout=1",'-timeout',90);
                        say($fc,"sudo sysctl debug.watchdog.hang_cpu=1",'-timeout',900,'-eval_error');
                } else {
        say($fc,"watchdog wd_reset", -prompt, 'DUMPCORE: END', -timeout, '900');}
        }
        }
	 elsif($SP_COMMAND ne "watchdog hang")
		{say($fc,"$SP_COMMAND",'-blind','-noprompt','-timeout',900);}
	else
		{
        logcomment("Executing for watchdog hang\n");
        my (undef,undef,$output,$prompt) = say($fc,"priv set test; help watchdog", -noprompt, -prompt, "$FILER", @prompts, '-swallow','-timeout',30);
        if ($output =~ /unrecognized command/) {
                my (undef, undef, $out, $prompt) = say($fc, "priv set test; systemshell", -noprompt, -prompt, "login\:", @prompts, -timeout, 30);

                if ($out =~ /systemshell not found/) {
                        # Running the watchdog hang command on node A
                        say($fc,"watchdog hang",-prompt, 'DUMPCORE: END', -timeout, '900');
                } elsif ($prompt =~ /login\:/) {
                        say($fc,"diag",-noprompt, -prompt,"Password\:",'-timeout',90);
                        say($fc,"netapp1\!",'-timeout',90);

                        say($fc,"sudo sysctl debug.watchdog",'-timeout',90);
                        say($fc,"sudo sysctl debug.watchdog.disable=0",'-timeout',90);
                        say($fc,"sudo sysctl debug.watchdog.ignore_nmi=0",'-timeout',90);
                        say($fc,"sudo sysctl debug.watchdog.trigger_l2_timeout=0",'-timeout',90);
                        say($fc,"sudo sysctl debug.watchdog.hang_cpu=1",'-swallow',-noprompt,'-wait', 'DUMPCORE: END', -timeout, '900');

        } else { 
	say($fc,"watchdog hang", -prompt, 'DUMPCORE: END', -timeout, '900');}
        }
        }
        sleep 120; 
	if ($SP_COMMAND ne "sp test autosupport")
		{sleep(90);}
	sleep(30);
	if($SP_COMMAND eq "system power off")
		{say($SP_ssh,"system power on",'-prompt',"@prompts",'-swallow','-timeout',100);}
	unless($SP_COMMAND eq "sp test autosupport")
	{	if(ping5($cc,$default_ip) != 0)
			{logresult('FAIL',"$FILER is still pinging after $SP_COMMAND\n");}
	}
        undef @EMS_Reboot;
	unless($SP_COMMAND eq "system power off" ||$SP_COMMAND eq "sp test autosupport" || $SP_COMMAND eq "system core")
	{	@EMS_Reboot=events_search("Reboots");
		($Reboot_After_Last,$Reboot_After_Penultimate)=EMS_specific($Reboot_msg,@EMS_Reboot);
		Check_Result($Reboot_Before,$Reboot_After_Penultimate,$Reboot_After_Last);
	}
	$PollEnable=1;
	$Iter_Result=0;
	while($Iter_Result <10)
	{	
		undef @EMS;
		@EMS=events_search($ASUP_SEARCH);
		($EMS_After_Last,$EMS_After_Penultimate)=EMS_specific($SP_ASUP,@EMS);
		Check_Result($EMS_Before,$EMS_After_Penultimate,$EMS_After_Last,0,$Pass_Flag);
		$Iter_Result++;
		if($PollAgain==1 && $Iter_result !=10)
		{	logcomment("Sleeping for 60 seconds\n");
			sleep(60);
		}
		if($PollAgain ==1 && $Iter_Result== 10)
			{logresult('FAIL',"$SP_Config_testcases{$Global_Setup}->[0],$ASUP_options_testcases{$Global_Config}->[0],$functionality_testcases{$Global_Func}->[0]\nNo SEL message is generated for this testcase\n");}
		elsif($PollAgain ==0)
			{last;}
	}
	$PollEnable=0;
	if($SP_COMMAND eq "system core" ||$SP_COMMAND eq "watchdog hang")
	{	$Temp_WAIT15=$WAIT15;
	 	$Temp_REMINDER=$REMINDER;
		$REMINDER=1;
		$WAIT15=1;
	} 
	logcomment("REMINDER=$REMINDER AND WAIT15=$WAIT15\n");


	if($SP_COMMAND ne "sp test autosupport" && $WAIT15==1)
	{	
		logcomment("Waiting for 15 minutes for reminder mail generation\n");
		if($REMINDER==0)
			{rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);}
		sleep(810);
		undef @EMS;
       		undef @EMS_Heartbeat;
		undef  @EMS_Reboot;
		if($SP_COMMAND eq "system power off" || $SP_COMMAND eq "system core")
		{	 @EMS_Reboot=events_search("Reboots");
	                ($Reboot_After_Last,$Reboot_After_Penultimate)=EMS_specific($Reboot_msg,@EMS_Reboot);
        	        Check_Result($Reboot_Before,$Reboot_After_Penultimate,$Reboot_After_Last);
		}
		@EMS=events_search($ASUP_SEARCH);
		@EMS_Heartbeat=events_search("Heartbeat");
		($EMS_REMINDER_After_Last,$EMS_REMINDER_After_Penultimate)=EMS_specific($REMINDER_ASUP,@EMS);
		($EMS_Heartbeat_Last,$EMS_Heartbeat_Penultimate)=EMS_specific($Heartbeat,@EMS_Heartbeat);
		if($REMINDER==0)
		{
			Check_Result($EMS_Heartbeat_Before,$EMS_Heartbeat_Penultimate,$EMS_Heartbeat_Last,1);	
			Check_Result($EMS_REMINDER_Before,$EMS_REMINDER_After_Penultimate,$EMS_REMINDER_After_Last,1);
		}
		else
		{	Check_Result($EMS_Heartbeat_Before,$EMS_Heartbeat_Penultimate,$EMS_Heartbeat_Last);
                        Check_Result($EMS_REMINDER_Before,$EMS_REMINDER_After_Penultimate,$EMS_REMINDER_After_Last,0,$Pass_Flag);
                }

#		elsif($REMINDER==0 && $EMS_REMINDER_Before eq $EMS_REMINDER_After_Last)
#			{logresult('PASS',"No reminder mail was generated as ONTAP is up again\n");}
#		else
#			{logresult('FAIL',"REMINDER MAIL was generated even when ONTAP is up\n");}
	}
	if($SP_COMMAND eq "system core" ||$SP_COMMAND eq "watchdog hang")
	{	$REMINDER=$Temp_REMINDER;
		$WAIT15=$Temp_WAIT15;
	}
	if($SP_COMMAND eq "system reset backup")
	{	say($SP_ssh,"system reset primary",'-noprompt','-wait','This will cause a dirty shutdown of your appliance','-swallow','-timeout',100);
                say($SP_ssh,"y",'-prompt',"SP $FILER\>",'-swallow','-timeout',300);
		sleep(75);
	}
	rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
}

## @name ASUP_Heartbeat_SP
## This checks for ASUP when heartbeat from ONTAP to SP is missing. Algorithm is similar to SP_SIDE_ASUP_CHECK subroutine.
sub ASUP_Heartbeat_SP($)
{
my $ASUP_SEARCH = "Heartbeat";
my $TEST = $config_data{ASUP_SEARCH}->[$testcase];
        logcomment("$SP_COMMAND\n$SP_ASUP\n$ASUP_SEARCH\n");
	$case_validity=shift @_;	
	logcomment("This will test if ASUP from SP is generated when heartbeat from ONTAP to SP  is missing\n");
	rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
	undef @EMS,;
	@EMS=events_search("HEARTBEAT");
	($EMS_Before,undef)=EMS_specific($config_data{Heartbeat_ASUP},@EMS);
	$Reminder_ASUP=$config_data{Heartbeat_ASUP};
	$Reminder_ASUP=~s/First notification/Reminder/;
	($EMS_REMINDER_Before,undef)=EMS_specific($REMINDER_ASUP,@EMS);
	set_priv($fc,"test");
	(undef,undef,$Heartbeat_Status)=say($fc,"sp heartbeat status",'-swallow','-timeout',30);
	if($Heartbeat_Status=~"Heartbeat to SP is enabled")
	{	say($fc,"sp heartbeat disable",'-swallow','-timeout',30);
	}
	elsif($Heartbeat_Status=~"Heartbeat to SP is disabled")
	{
		say($fc,"sp heartbeat enable",'-swallow','-timeout',30);
		sleep(60);
		say($fc,"sp heartbeat disable",'-swallow','-timeout',30);
	}
	(undef,undef,$Heartbeat_Status)=say($fc,"sp heartbeat status",'-swallow','-timeout',30);
	unless($Heartbeat_Status=~"Heartbeat to SP is disabled")
	{logresult('FAIL',"Could not disable Heartbeat to SP\n");}
	sleep(900);
	undef @EMS;
	@EMS=events_search("HEARTBEAT");
	($EMS_After_Last,$EMS_After_Penultimate)=EMS_specific($config_data{Heartbeat_ASUP},@EMS);
	      $PollEnable=1;
        $Iter_Result=0;
        while($Iter_Result <10)
        {
                undef @EMS;
#                @EMS=events_search($ASUP_SEARCH);
        @EMS=events_search("HEARTBEAT");
                 ($EMS_After_Last,$EMS_After_Penultimate)=EMS_specific($config_data{Heartbeat_ASUP},@EMS);
#                ($EMS_After_Last,$EMS_After_Penultimate)=EMS_specific($SP_ASUP,@EMS);
                Check_Result($EMS_Before,$EMS_After_Penultimate,$EMS_After_Last,0,$Pass_Flag);
                $Iter_Result++;
                if($PollAgain==1 && $Iter_result !=10)
                {       logcomment("Sleeping for 60 seconds\n");
                        sleep(60);
                }
                if($PollAgain ==1 && $Iter_Result== 10)
                        {logresult('FAIL',"$SP_Config_testcases{$Global_Setup}->[0],$ASUP_options_testcases{$Global_Config}->[0],$functionality_testcases{$Global_Func}->[0]\nNo SEL message is generated for this testcase\n");}
                elsif($PollAgain ==0)
                        {last;}
        }
        $PollEnable=0;
	if($WAIT15==1)
	{	logcomment("Waiting for 15 minutes to check reminder mail generation \n");
		if($REMINDER==0)
		{	say($fc,"sp heartbeat enable",'-swallow','-timeout',30);
			(undef,undef,$Heartbeat_Status)=say($fc,"sp heartbeat status",'-swallow','-timeout',30);
			unless($Heartbeat_Status=~"Heartbeat to SP is enabled")
			{logresult('FAIL',"Could not enable Heartbeat to SP");}
		}
		sleep(900);
		undef @EMS;
		($EMS_REMINDER_After_Last,$EMS_REMINDER_After_Penultimate)=EMS_specific($REMINDER_ASUP,@EMS);
		if($REMINDER==1)
			{Check_Result($EMS_REMINDER_Before,$EMS_REMINDER_After_Penultimate,$EMS_REMINDER_After_Last,0,$Pass_Flag);}
		else
			{Check_Result($EMS_REMINDER_Before,$EMS_REMINDER_After_Penultimate,$EMS_REMINDER_After_Last,1,$Pass_Flag);}
	}
	say($fc,"sp heartbeat enable",'-swallow','-timeout',30);
}

## @name ASUP_Heartbeat_ONTAP
## This checks for ASUP when heartbeat from ONTAP is missing and rebooting does not restore the heartbeat.Algorithm is similar to SP_SIDE_ASUP_CHECK.
sub ASUP_Heartbeat_ONTAP($)
{	$case_validity=shift @_;
	logcomment("This will test if ASUP from ONTAP is generated when heartbeat from SP to ONTAP goes missing\n");
	rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
	set_priv($fc,"test");
	say($fc,"options ems.autosuppress.enable off",'-swallow','-timeout',30);
	$waitstr1='sp.heartbeat.stopped:warning';
#	$waitstr2='\(SP HBT STOPPED\) CRITICAL';	
        $waitstr2='SP HBT STOPPED';
	say($fc,"sp debugmode",'-swallow','-timeout',30);
	$root_ssh =rlm_make_conn_to_rlm($FILER,$rlm_ssh_ip,'root','superuser');
	$Change_init='sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /etc/init.d/netapp_final_initscript';
	$Change_init_primary='sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /mnt/jffs2_rootfs_primary/etc/init.d/netapp_final_initscript';
	$Change_init_backup='sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /mnt/jffs2_rootfs_backup/etc/init.d/netapp_final_initscript';
	$Change_watcher='sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /etc/watcher.conf';
	$Change_watcher_primary='sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /mnt/jffs2_rootfs_primary/etc/watcher.conf';
	$Change_watcher_backup='sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /mnt/jffs2_rootfs_backup/etc/watcher.conf';
	say($root_ssh,'cat /etc/init.d/netapp_final_initscript',-swallow,'-timeout',60);
	say($root_ssh,"$Change_init",'-swallow','-timeout',30);
	say($root_ssh,'cat /etc/init.d/netapp_final_initscript','-swallow','-timeout',60);
	say($root_ssh,'cat /mnt/jffs2_rootfs_primary/etc/init.d/netapp_final_initscript','-swallow','-timeout',60);
	say($root_ssh,"$Change_init_primary",'-swallow','-timeout',30);
	say($root_ssh,'cat /mnt/jffs2_rootfs_primary/etc/init.d/netapp_final_initscript','-swallow','-timeout',60);
	say($root_ssh,'cat /mnt/jffs2_rootfs_backup/etc/init.d/netapp_final_initscript','-swallow','-timeout',60);
	say($root_ssh,"$Change_init_backup",'-swallow','-timeout',30);
	say($root_ssh,'cat /mnt/jffs2_rootfs_backup/etc/init.d/netapp_final_initscript','-swallow','-timeout',60);
	say($root_ssh,$Change_watcher,'-swallow','-timeout',30);
	say($root_ssh,$Change_watcher_primary,'-swallow','-timeout',30);
	say($root_ssh,$Change_watcher_backup,'-timeout',30);
	say($root_ssh,'cat /etc/watcher.conf','-swallow','-timeout',30);
	say($root_ssh,'cat /mnt/jffs2_rootfs_primary/etc/watcher.conf','-swallow','-timeout',30);
	say($root_ssh,'cat /mnt/jffs2_rootfs_backup/etc/watcher.conf','-swallow','-timeout',30);
	(undef,undef,$ID)=say($root_ssh,'ps - ef | grep watcher','-swallow','-timeout',60);
	if($ID =~/(\d+)[\s]?root[\s]+(\d+)[\s]?\w[\s]+\/usr\/local\/bin\/watcher/i)
		{say($root_ssh,"kill -9 $1",'-swallow','-timeout',30);}
	else
		{logresult('FAIL',"Failed to kill the watcher PID\n");}
	(undef,undef,$ID)=say($root_ssh,'ps - ef | grep r2fbeat','-swallow','-timeout',30);
	if($ID =~/(\d*)[\s]?root[\s]*(\d*)[\s]?\w[\s]*\/usr\/local\/bin\/r2fbeat/i)
		{say($root_ssh,"kill -9 $1",'-swallow','-timeout',30);}
        else
        	{logresult('FAIL',"Failed to kill the r2fbeat PID\n");}
	
	say($root_ssh,"ps",'-swallow','-timeout',100);
	eval
	{
		($EMS_Match1,$EMS_Before1,$EMS_Output1)=say($fc,"date",'-wait',$waitstr1,'-noprompt','-swallow','-timeout',650);	
	};
	if($@)
		{logresult('FAIL',"Heartbeat stopped message was not received : $@\n");}
	else
		{logresult('PASS',"ONTAP detected SP heartbeat loss :: $EMS_Match1\n");}
	logcomment("\nEMS_Match=$EMS_Match1,EMS_Before=$EMS_Before1,EMS_Output=$EMS_Output1\n");
	sleep(30);
	(undef,undef,$spstatus)=say($fc,"sp status",'-swallow','-timeout',300);
	if($spstatus=~/Rebooting/)
		{logcomment("ONTAP trying to reboot SP\n");}
	else
		{logresult('FAIL',"SP should be rebooted after detection of heartbeat loss\n");} 
	eval
		{($EMS_Match1,$EMS_Before1,$EMS_Output1)=say($fc,"date",'-wait',$waitstr2,'-noprompt','-swallow','-timeout',600);};
	if($@)
	{
		logresult('FAIL',"EMS for ASUP for heartbeat loss was not generated\n");
		
	}
	(undef,undef,$spstatus)=say($fc,"sp status",'-swallow','-timeout',300);	
	logcomment("\nEMS_Match=$EMS_Match1,EMS_Before=$EMS_Before1,EMS_Output=$EMS_Output1\n");
	if($spstatus=~ /pkt:\s+unknown/i || $spstatus=~ /pkt:\s+failed/i || $spstatus=~ /pkt:\s+degraded/i)
		{logcomment("No packet heartbeat \n");}
	else
		{logresult('FAIL',"Packet heartbeat is still received\n");}
	#say($root_ssh,'sed -i \'$a\/usr/local/bin/r2fbeat\ \' /mnt/jffs2_rootfs_primary/etc/watcher.conf','-swallow','-timeout',30);
	#say($root_ssh,'sed -i \'$a\/usr/local/bin/r2fbeat\ \' /mnt/jffs2_rootfs_backup/etc/watcher.conf','-swallow','-timeout',30);
	set_priv($fc,"test");
	disconnect($root_ssh);
	say($fc,"sp debugmode",'-swallow','-timeout',30);
	$root_ssh =rlm_make_conn_to_rlm($FILER,$rlm_ssh_ip,'root','superuser');
	$Change_watcher='sed -i \'s_\#\+/usr/local/bin/r2fbeat_/usr/local/bin/r2fbeat_\' /etc/watcher.conf';
	$Change_watcher_primary='sed -i \'s_\#\+/usr/local/bin/r2fbeat_/usr/local/bin/r2fbeat_\' /mnt/jffs2_rootfs_primary/etc/watcher.conf';
	$Change_watcher_backup='sed -i \'s_\#\+/usr/local/bin/r2fbeat_/usr/local/bin/r2fbeat_\' /mnt/jffs2_rootfs_backup/etc/watcher.conf';
	say($root_ssh,$Change_watcher,'-swallow','-timeout',30);
	say($root_ssh,$Change_watcher_primary,'-swallow','-timeout',30);
	say($root_ssh,$Change_watcher_backup,'-swallow','-timeout',30);
	$Change_init='sed -i \'s_\#\+/usr/local/bin/r2fbeat_/usr/local/bin/r2fbeat_\' /etc/init.d/netapp_final_initscript';
	$Change_init_primary='sed -i \'s_\#\+/usr/local/bin/r2fbeat_/usr/local/bin/r2fbeat_\' /mnt/jffs2_rootfs_primary/etc/init.d/netapp_final_initscript';
	$Change_init_backup='sed -i \'s_\#\+/usr/local/bin/r2fbeat_/usr/local/bin/r2fbeat_\' /mnt/jffs2_rootfs_backup/etc/init.d/netapp_final_initscript';
	say($root_ssh,$Change_init,'-swallow','-timeout',30);
	say($root_ssh,$Change_init_primary,'-swallow','-timeout',30);
	say($root_ssh,$Change_init_backup,'-swallow','-timeout',30);
	(undef,undef,$ID)=say($root_ssh,'ps - ef | grep watcher','-swallow','-timeout',30);
        if($ID =~/(\d*)[\s]?root[\s]*(\d*)[\s]?S[\s]*\/usr\/local\/bin\/watcher/i)
        	{say($root_ssh,"kill -9 $1",'-swallow','-timeout',30);}
        else
        	{logresult('FAIL',"Failed to kill the watcher PID\n");}
	$waitstr3='sp.heartbeat.resumed:info';
	eval
		{($EMS_Match1,$EMS_Before1,$EMS_Output1)=say($fc,"date",'-wait',$waitstr3,'-noprompt','-swallow','-timeout',200);};
	logcomment("\nEMS_Match=$EMS_Match1,EMS_Before=$EMS_Before1,EMS_Output=$EMS_Output1\n");
	sleep(40);
	(undef,undef,$spstatus)=say($fc,"sp status",'-swallow','-timeout',300);
	if($spstatus=~/online/i)
		{logcomment("SP is up again\n");}
	else
		{logresult('FAIL',"SP is not online\n");}
	say($root_ssh,"ps",'-swallow','-timeout',100);
	disconnect($root_ssh);
}

## @name Negative_ASUP_Cases
##This subroutine checks for non generation of ASUP for reboot/halt and user triggered Panic.

sub Negative_ASUP_Cases($)
{	$case_validity=shift @_;
	if($case_validity==10)
                {$SP_COMMAND="panic";} 
	elsif($case_validity==11)
                {$SP_COMMAND="reboot";}
	elsif($case_validity==12)
		{$SP_COMMAND="halt";}
        else
  	      {logresult("FATAL","Invalid testcase\n");}
	logcomment("This will test if ASUP is generated on issuing Panic,Halt or Reboot\n");
	rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
	@EMS=events_search("ASUP");
	($EMS_Before,undef)=$EMS[-1];
	set_priv($fc,"test");
        say($fc,"bootargs set AUTOBOOT TRUE",'-swallow','-timeout',30);
	if($SP_COMMAND eq "panic")
		{say($fc,"$SP_COMMAND", -prompt, 'DUMPCORE: END', -timeout, '900');
                sleep(450);}
	else
	{	say($fc,"$SP_COMMAND",'-blind','-noprompt','-timeout',300);
		sleep(450);
	}
	rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
	undef @EMS;
	@EMS=events_search("ASUP");
	$EMS_After_Last=$EMS[-1];
	$EMS_After_Penultimate=$EMS[-2];	
	Check_Result($EMS_Before,$EMS_After_Penultimate,$EMS_After_Last,1);
}

## @name load_functionality_testcases
## The hash contains all the testcases related to functionality.	
sub load_functionality_testcases
{
	%functionality_testcases=(
	0=>["Case 0. SP test autosupport"],
	1=>["Case 1. system power cycle"],
	2=>["Case 2. system power off"],
	3=>["Case 3. system reset current"],
	4=>["Case 4. system reset backup"],
	5=>["Case 5. system core"],
	6=>["Case 6. watchdog hang"],
	7=>["Case 7. watchdog reset"],
	8=>["Case 8. Heartbeat loss from ONTAP to SP"],
	9=>["Case 9. Heartbeat loss from SP to ONTAP"],
	10=>["Case 10.User Triggered Panic"],
	11=>["Case 11.User Triggered Reboot"],
	12=>["Case 12.User Triggered Halt"],
	);
}

## @name verify_functionality_testcases
## This subroutine lists any exceptions for the functionality testcases.
sub verify_functionality_testcases
{
	logcomment("All testcases in the functionality hash are valid \n");
}	

## @name load_SP_Config_testcases
## This hash contains the testcases related to sp setup.
sub load_SP_Config_testcases
{
	%SP_Config_testcases=(
	1=>["State 1. $RLM_PROMPT with static IPV4 addresses",2],
	2=>["State 2. $RLM_PROMPT with DHCP IPV4 addresses",3],
	);
	unless ($NO_IPv6) {
	%SP_Config_testcases=(%SP_Config_testcases,
	3=>["State 3. $RLM_PROMPT with IPV6 addresses",4],
	4=>["State 4. $RLM_PROMPT with static IPV4 and IPV6 addresses",5],
	);}
}

## @name verify_SP_Config_testcases
## This subroutine has the condition for uploading IPV6 testcases only if $NO_IPv6 is false. Thus other testcases wil be ignored for uploading.
sub verify_SP_Config_testcases($)
{ my $TEST_CASE = shift;
        my $no_ipv6;

        if ($NO_IPv6) 
        {
	        $no_ipv6 = $NO_IPv6;
                undef ($NO_IPv6);
                load_SP_Config_testcases();
        }

        if ($TEST_CASE > 0 && $TEST_CASE <= scalar(keys %SP_Config_testcases)) 
		{logresult ('NA', "Case: $TEST_CASE :- " . $SP_Config_testcases{$TEST_CASE}->[0] . " can't be executed as \"$no_ipv6\" ");}
        else
                {logresult ('NA', "Invalid test case/state: $TEST_CASE. See the tharndoc for more info");}

}

## @name load_ASUP_options_testcases
## This contais the options autosupport testcases hash.
sub load_ASUP_options_testcases(%)
{
	my %config_data=@_;
	%ASUP_options_testcases=

	(
 	1 => ["State 1.set one valid mail ID for local and netapp ID and  mailserver","on","complete","State1",$config_data{IPV4_MAILHOST}->[0],$config_data{MAIL_ID}->[0],"on",$config_data{NETAPP_MAIL_ID}->[0]],
 	2 => ["State 2.set two valid mail ID for local and netapp ID and a proper mailserver hostname","on","complete","State2",$config_data{NAME_IPV4_MAILHOST}->[0],"$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1]"],
 	3 => ["State 3.set the same two valid mail ID for local and netapp ID and a proper mailserver","on","complete","State3",$config_data{IPV4_MAILHOST}->[1],"$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[0]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0]"],
 	4 => ["State 4.set the five valid mail ID for local and netapp ID and a proper mailserver","on","complete","State4",$config_data{IPV4_MAILHOST}->[0],"$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
	5 => ["State 5.Set 2 wrong Mail ID for local and 2 correct Mail ID for netapp ID and a proper mailserver","on","complete","State5",$config_data{IPV4_MAILHOST}->[1],"$config_data{BAD_MAIL_ID}->[0],$config_data{BAD_MAIL_ID}->[1]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0]"],
	6 => ["State 6.Set 2 wrong Mail ID for the Netapp ID and 2 proper IDs for local MAIL ID and a proper mailserver","on","complete","State6",$config_data{IPV4_MAILHOST}->[2],"$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3]","on","$config_data{BAD_NETAPP_MAIL_ID}->[0],$config_data{BAD_NETAPP_MAIL_ID}->[1]"],
 	7 => ["State 7.Set a wrong followed by a right mail ID for local and all correct IDs for Netapp and a proper mailserver","on","complete","State7",$config_data{IPV4_MAILHOST}->[3],"$config_data{BAD_MAIL_ID}->[2],$config_data{MAIL_ID}->[3]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1]"],
 	8 => ["State 8.Set a wrong followed by a right mail ID for Netapp and all correct IDs for local and a proper mailserver","on","complete","State8",$config_data{IPV4_MAILHOST}->[2],"$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3]","on","$config_data{BAD_NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1]"],
 	9 => ["State 9.Set 4 wrong followed by a correct Mail ID for local and Netapp user and a proper mailserver","on","complete","State9",$config_data{IPV4_MAILHOST}->[0],"$config_data{BAD_MAIL_ID}->[0],$config_data{BAD_MAIL_ID}->[1],$config_data{BAD_MAIL_ID}->[2],$config_data{BAD_MAIL_ID}->[3],$config_data{MAIL_ID}->[0]","on","$config_data{BAD_NETAPP_MAIL_ID}->[0],$config_data{BAD_NETAPP_MAIL_ID}->[1],$config_data{BAD_NETAPP_MAIL_ID}->[2],$config_data{BAD_NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[0]"],
 	10=> ["State 10.Set the same correct MAIL ID for all five options in local and Netapp and a proper mailserver","on","complete","State10",$config_data{IPV4_MAILHOST}->[0],"$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[0]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0]"],
 	11=> ["State 11.Set a mixture of right and wrong MAIL ID for both netapp and local and a proper mailserver","on","complete","State11",$config_data{IPV4_MAILHOST}->[0],"$config_data{BAD_MAIL_ID}->[0],$config_data{BAD_MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{BAD_MAIL_ID}->[4]","on","$config_data{BAD_NETAPP_MAIL_ID}->[0],$config_data{BAD_NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{BAD_NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 	12=> ["State 12.Set a mixture of wrong,right Netapp Mail IDs and no local IDs and a proper mailserver","on","complete","State12",$config_data{IPV4_MAILHOST}->[0],".","on","$config_data{BAD_NETAPP_MAIL_ID}->[0],$config_data{BAD_NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{BAD_NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 	13=> ["State 13.Set a mixture of wrong right local mail ids and no netapp mail ids and a proper mailserver","yes","complete","State13",$config_data{IPV4_MAILHOST}->[0],"$config_data{BAD_MAIL_ID}->[0],$config_data{BAD_MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{BAD_MAIL_ID}->[4]","on","."],
 	14=> ["State 14.Set netapp mail option to off and send mail to correct IDs for both netapp and local and set a proper mailserver","on","complete","State14",$config_data{IPV4_MAILHOST}->[0],"$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1]","off","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 	15=> ["State 15.Set the content option to minimal and send mail to all five netapp and local users and set a proper mailserver","YES","minimal","State15",$config_data{IPV4_MAILHOST}->[0],"$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 	16=> ["State 16.Set the autosupport enable option to off and try to send mail to all five netapp and local IDs and set a proper mailserver","off","complete","State16",$config_data{IPV4_MAILHOST}->[0],"$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 	17=> ["State 17.Put a mixture of 5 right and wrong mail ids followed by a right mail ID for both local and Netapp IDs and set a proper mailserver","on","minimal","State17",$config_data{IPV4_MAILHOST}->[0],"$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{BAD_MAIL_ID}->[2],$config_data{BAD_MAIL_ID}->[3],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3]","on","$config_data{BAD_NETAPP_MAIL_ID}->[0],$config_data{BAD_NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4],$config_data{NETAPP_MAIL_ID}->[0]"],
 	18=> ["State 18.Put all 5 wrong mail ids followed by a right mail ID for both local and Netapp IDs and set a proper mailserver","true","minimal","State18",$config_data{IPV4_MAILHOST}->[0],"$config_data{BAD_MAIL_ID}->[0],$config_data{BAD_MAIL_ID}->[1],$config_data{BAD_MAIL_ID}->[2],$config_data{BAD_MAIL_ID}->[3],$config_data{BAD_MAIL_ID}->[4],$config_data{MAIL_ID}->[0]","on","$config_data{BAD_NETAPP_MAIL_ID}->[0],$config_data{BAD_NETAPP_MAIL_ID}->[1],$config_data{BAD_NETAPP_MAIL_ID}->[2],$config_data{BAD_NETAPP_MAIL_ID}->[3],$config_data{BAD_NETAPP_MAIL_ID}->[4],$config_data{NETAPP_MAIL_ID}->[0]"],
 	19=> ["State 19.Set 2 wrong followed by one right mailserver and try to send mail to all five local and netapp IDs","TRUE","complete","State19","$config_data{BAD_IPV4_MAILHOST}->[0],$config_data{BAD_IPV4_MAILHOST}->[1],$config_data{IPV4_MAILHOST}->[0]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 	20=> ["State 20.Set 3 wrong followed by one right mailserver and try to send mail to all five local and netapp IDs","1","complete","State20","$config_data{BAD_IPV4_MAILHOST}->[0],$config_data{BAD_IPV4_MAILHOST}->[1],$config_data{BAD_IPV4_MAILHOST}->[2],$config_data{IPV4_MAILHOST}->[0]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 	21=> ["State 21.Set all 3 right mailservers and try to send mail to all five local and netapp IDs","ON","complete","State21","$config_data{IPV4_MAILHOST}->[0],$config_data{IPV4_MAILHOST}->[1],$config_data{IPV4_MAILHOST}->[2]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
	22=> ["State 22.Set hostnames for IPV4 Mailhosts only(bad folowed by good)with all five local and Netapp IDs","1","complete","State22","$config_data{BAD_NAME_IPV4_MAILHOST}->[0],$config_data{BAD_NAME_IPV4_MAILHOST}->[1],$config_data{NAME_IPV4_MAILHOST}->[0]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],

	);

 	unless ($NO_IPv6) {
		%ASUP_options_testcases=(%ASUP_options_testcases,
 		23=> ["State 23.Set IPV6 mailhost and try to send mail to all five local and netapp IDs","on","complete",'State23',$config_data{IPV6_MAILHOST}->[0],"$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 		24=> ["State 24.Set 2 right IPV6 mailhost and disable ASUP and try to send mail to all five local and netapp IDs","0","complete",'State24',"$config_data{IPV6_MAILHOST}->[0],$config_data{IPV6_MAILHOST}->[1]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 		25=> ["State 25.Set 2 wrong IPV6 mailhost and one right IPV6 mailhost and try to send mail to all five local and netapp IDs","on","complete",'State25',"$config_data{BAD_IPV6_MAILHOST}->[0],$config_data{BAD_IPV6_MAILHOST}->[1],$config_data{IPV6_MAILHOST}->[0]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 		26=> ["State 26.Set 4 Wrong followed by a right IPV6 mailhost and try to send mail to all five local and netapp IDs","on","complete",'State26',"$config_data{BAD_IPV6_MAILHOST}->[0],$config_data{BAD_IPV6_MAILHOST}->[1],$config_data{BAD_IPV6_MAILHOST}->[2],$config_data{BAD_IPV6_MAILHOST}->[3],$config_data{IPV6_MAILHOST}->[0]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 		27=> ["State 27.Set one wrong, 3 right IPV4 mailhosts followed by an IPV6 mailhost with all five local and Netapp IDs","on","complete",'State27',"$config_data{BAD_IPV4_MAILHOST}->[0],$config_data{IPV4_MAILHOST}->[0],$config_data{IPV4_MAILHOST}->[1],$config_data{IPV4_MAILHOST}->[2],$config_data{IPV6_MAILHOST}->[0]","$config_data{MAIL_ID}->[0]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 		28=> ["State 28.Set 3 wrong IPV6 mailhosts followed by right IPV6 and IPV4 mailhost with all five local and Netapp IDs","on","complete",'State28',"$config_data{BAD_IPV6_MAILHOST}->[0],$config_data{BAD_IPV6_MAILHOST}->[1],$config_data{BAD_IPV6_MAILHOST}->[2],$config_data{IPV6_MAILHOST}->[0],$config_data{IPV4_MAILHOST}->[0]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 		29=> ["State 29.Set a mixture of IPV6 followed by IPV4 mailhosts  with all five local and Netapp IDs","on","complete",'State29',"$config_data{BAD_IPV6_MAILHOST}->[0],$config_data{IPV6_MAILHOST}->[0],$config_data{IPV4_MAILHOST}->[0]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
		30=> ["State 30.Set hostnames for IPV6 mailhosts only (bad followed by good)with all five local and Netapp IDs","on","complete",'State30',"$config_data{BAD_NAME_IPV6_MAILHOST}->[0],$config_data{BAD_NAME_IPV6_MAILHOST}->[1],$config_data{NAME_IPV6_MAILHOST}->[0]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 		31=> ["State 31.Set hostnames for a mixture of IPV4 and IPV6 mailhosts with all five local and netapp IDs","on","complete",'State31',"$config_data{NAME_IPV4_MAILHOST}->[0],$config_data{NAME_IPV6_MAILHOST}->[0]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 		32=> ["State 32.Set hostname for bad IPV4 followed by IPV6 mailhost with all five local and netapp IDs","on","complete",'State32',"$config_data{BAD_NAME_IPV4_MAILHOST}->[0],$config_data{NAME_IPV6_MAILHOST}->[0]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"],
 		33=> ["State 33.Set hostname for bad IPV6,IPV4 followed by good IPV4 and IPV6 mailhost with all five local and netapp IDs","on","complete",'State33',"$config_data{BAD_NAME_IPV6_MAILHOST}->[0],$config_data{BAD_NAME_IPV6_MAILHOST}->[1],$config_data{BAD_NAME_IPV4_MAILHOST}->[0],$config_data{NAME_IPV6_MAILHOST}->[0],$config_data{NAME_IPV4_MAILHOST}->[0]","$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]","on","$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"]
		);
	}
}

## @name verify_ASUP_options_testcases
## This contains the conditions when some of the options.autosupport testcases are to be eliminated form testing.
sub verify_ASUP_options_testcases($)
{
 my $TEST_CASE = shift;
        my $no_ipv6;

        if ($NO_IPv6)
        {       $no_ipv6 = $NO_IPv6;
                undef ($NO_IPv6);
                load_ASUP_options_testcases(%config_data);
        }

        if ($TEST_CASE > 0 && $TEST_CASE <= scalar(keys %ASUP_options_testcases)) 
	{
                logresult ('NA', "Case: $TEST_CASE :- " . $ASUP_options_testcases{$TEST_CASE}->[0] . " can't be execuated as \"$no_ipv6\" ");
        }
	else 
	{
                logresult ('NA', "Invalid test case/state: $TEST_CASE. See the tharndoc for more info");
        }
}
=from
## Hearbeat and timesync Functions
################################################################################
## Function Name : timeSync
## Return Value  : PASS or FAIL
## Description   : This function verifies whether Ontap and SP time are in sync
###############################################################################

sub timeSync {
   my(undef,$dateSP,$dateOntapi,@dateSP,@dateOntap);
   (undef,undef,$dateSP) = say ($SP_ssh,"date",'-swallow','-timeout', 300);
   (undef,undef,$dateOntap) = say ($fc,"date",'-swallow','-timeout',300);
   @dateSP = split(/\s*/,$dateSP);
   @dateOntap = split(/\s*/,$dateOntap);
   @timeSP = split(/:/,$dateSP[3]);
   @timeOntap = split(/:/,$dateOntap[3]);
   if (($dateSP[0] eq $dateOntap[0]) && ($dateSP[1] eq $dateOntap[1]) && ($dateSP[2] eq $dateOntap[2]) && ($timeSP[0] eq $timeOntap[0]) && ($timeSP[1] eq $timeSP[1]) && ($dateSP[4] eq $dateOntap[4]) && ($dateSP[5] eq $dateOntap[5])) {
      tstlog -PASS, "Ontap and SP time are in sync";
      return "PASS";
   } else {
     tstlog -FAIL, "Ontap and SP time are not in sync.Please check the log";
     return "FAIL";
   }
}
=cut
################################################################################
## Function Name : timeSync
## Return Value  : PASS or FAIL
## Description   : This function verifies whether Ontap and SP time are in sync
################################################################################

sub timeSync {
   my(undef,$dateSP,$dateOntapi,@dateSP,@dateOntap);
   (undef,undef,$dateSP) = say ($SP_ssh,"date",'-swallow',-prompt, @prompts_sp, '-timeout', 300);
   (undef,undef,$dateOntap) = say ($fc,"date",'-swallow','-timeout',300);
   @dateSP = $dateSP =~ /(\w+)\s+(\w+)\s+(\d+)\s+([\d\:]+)\:\d+\s+(\w+)\s+(\d+)/;
   @dateOntap = $dateOntap =~ /(\w+)\s+(\w+)\s+(\d+)\s+([\d\:]+)\:\d+\s+(\w+)\s+(\d+)/;
   for ($count=0;$count < scalar @dateOntap;$count++) {
       if ($dateSP[$count] ne $dateOntap[$count]) {
          tstlog -WARN, "Ontap and SP time are not in sync.Please check the log";
          return "FAIL";
       }
   }
   tstlog -PASS, "Ontap and SP time are in sync";
   return "PASS";

}

###########################################################################
## Function Name : stopSPToOntapHeartbeat
## Return Value  : PASS or FAIL
## Description   : This function deliberately stops SP heartbeat
###########################################################################

sub stopSPToOntapHeartbeat {
    my $rlm_ssh_ip = $_[0];
    tstlog -comment, "Deliberately stopping SP heartbeat";
    set_priv($fc,"test");
    say($fc,"options ems.autosuppress.enable off",'-swallow','-timeout',30);
    say($fc,"sp debugmode",'-swallow','-timeout',30);
    my $root_ssh =rlm_make_conn_to_rlm($FILER,$rlm_ssh_ip,'root','superuser');
    say($root_ssh,"sed -i \'\/r2fbeat\/d\' \/etc\/watcher.conf",'-swallow','-timeout',30);
    my (undef,undef,$ID)=say($root_ssh,'ps - ef | grep watcher','-swallow','-timeout',30);
    if ($ID =~/(\d*)[\s]?root[\s]*(\d*)[\s]?S[\s]*\/usr\/local\/bin\/watcher/i) {
        say($root_ssh,"kill -9 $1",'-swallow','-timeout',30);
    } else {
        tstlog -FATAL, "Failed to kill the watcher PID";
    }
    (undef,undef,$ID)=say($root_ssh,'ps - ef | grep r2fbeat','-swallow','-timeout',30);
    if ($ID =~/(\d*)[\s]?root[\s]*(\d*)[\s]?S[\s]*\/usr\/local\/bin\/r2fbeat/i) {
        say($root_ssh,"kill -9 $1",'-swallow','-timeout',30);
    } else {
        tstlog -FATAL, "Failed to kill the r2fbeat PID";
    }
    return "PASS";
}


##########################################################################
## Function Name : disableOntapToSPHeartbeat
## Return Value  : PASS or FAIL
## Description   : This function disables Ontap to SP heartbeat
###########################################################################
sub disableOntapToSPHeartbeat {
   tstlog -comment, "Disabling Ontap to SP heartbeat";
   set_priv($fc,"test");
   my (undef,undef,$heartbeatStatus)=say($fc,"sp heartbeat status",'-swallow','-timeout',30);
   if ($heartbeatStatus =~ /Heartbeat to SP is enabled/) {
     say($fc,"sp heartbeat disable",'-swallow','-timeout',30);
   } else {
     say($fc,"sp heartbeat enable",'-swallow','-timeout',30);
     sleep(60);
     say($fc,"sp heartbeat disable",'-swallow','-timeout',30);
   }
   (undef,undef,$heartbeatStatus) = say($fc,"sp heartbeat status",'-swallow','-timeout',30);
   if ($heartbeatStatus =~ /Heartbeat to SP is disabled/) {
     tstlog -comment, "Ontap to SP heartbeat is disabled";
     return "PASS";
   } else {
     tstlog -FATAL, "Could not disable Ontap to SP heartbeat";
   }
}


##########################################################################
## Function Name : enableOntapToSPHeartbeat
## Return Value  : PASS or FAIL
## Description   : This function enables to SP heartbeat
###########################################################################


sub enableOntapToSPHeartbeat {
   tstlog -comment, "Enabling Ontap to SP heartbeat";
   set_priv($fc,"test");
   my (undef,undef,$heartbeatStatus)=say($fc,"sp heartbeat enable",'-swallow','-timeout',30);
   if ($heartbeatStatus =~ /Heartbeat to SP enabled/) {
     tstlog -comment, "Ontap to SP heartbeat is enabled";
     return "PASS";
   } else {
     tstlog -FATAL, "Could not enable Ontap to SP heartbeat";
   }
}
1;
