# -*- perl -*-
# $Id:$
#
#       Copyright (c) 2010 NetApp, Inc.
#       All Rights reserved
#
## @summary Routines for validating SP ASUP testcases for 10mode.
##
## @description
##	This library contains individual routines for ONTAP, SP and Boot Loader
## for testing of ASUP module for both configuration and functionality related
## tests.ASUP scripts are not fully automated user has to check the mail IDs
## configured to check if mails were received.
##
## Sub-Routines:
##
##	1.check_options_10mode
##	This will check if the options we set for ASUP configuration is same as
## options.autosupport or not
##
##	2.set_priv_10mode
##	This sets the privilege as required for both SP and ONTAP side.
##
##	3.SP_Side_ASUP_Check_10mode
##	This function checks if ASUP is generated or not for sp test autosupport,
## system power cycle,system power off,system reset,system reset backup,
## system core,watchdog hang and watchdog_reset
##
##	4.ASUP_Heartbeat_SP_10mode
##	This will test if ASUP from SP is generated when heartbeat from ONTAP to SP
## is missing
##
##	5.load_ASUP_options_testcases_10mode
##	This will load all the testcases related to options.autosupport
##
## 6.check_spstatusd_10mode
## This function verify the configuration of asup.
##
## 7.set_options_10mode
## This sets the values of options autosupport.It takes a pointer to an array as
## argument and the array should have values for the test case title,-state,
## -remove-private-data,-from,-mail-hosts,-to,-support,-support-address in order.
##
## 8.rlm_setup_10mode
## To do "rlm setup" in ONTAP
##
## 9.Negative_ASUP_Cases
## This subroutine checks for non generation of ASUP for reboot/halt and user triggered Panic.
##
## 10.rlm_reboot_rlm_10mode
## Reboot's the RLM from ONTAP.
##
## 11.rlm_verify_rlm_status_10mode
##	Verifies "rlm status" output in ONTAP and SP
##
##	12.rlm_load_ipv4_config_cases.
## To load ipv4 test cases
##
## 13.load_SP_Config_testcases
## This hash contains the testcases related to sp setup.
##
## 14.valid_positive_testcase
## This subroutine checks if the options autosupport testcases should pass or fail
## according to SP setup and ASUP configuration and accordingly populates a matrix for each SP setup.
##
## 15.pass_flag
## This sets $Pass_Flag as true or false according to the Autosupport options test
## case number by checking the @valid_cases matrix for the particular sp setup.
##
## 16. disableOntapToSPHeartbeat_10mode
## This function disables Ontap to SP heartbeat in 10mode
##
## 17. enableOntapToSPHeartbeat_10mode
## This function enables to SP heartbeat in 10mode
##
## 18. stopSPToOntapHeartbeat_10mode
## This function deliberately stops SP heartbeat from 10mode
##
## @synopsis
##		use asup10mode;
##
## @keywords
##   sp,asup,functionality,ems,configuration,events,mailhost.
##
## @author  deepakb@netapp.com
##
######################################################################################################################################

require "tharnit_lib.pl";
use BTS_Config;
use platval::service_processor::asup;
use platval::service_processor::rlm_ipv6_10mode;
our ($FILER,                   $fc,                         $SP_ssh,
     %config_data,             %SP_Config_testcases_10mode, $NO_IPv6,
     %functionality_testcases, @valid_cases,                $Pass_Flag,
     $PollAgain,               $PollEnable,                 $Global_Config,
     $Global_Setup,            $Global_Func,                $LOGIN, 
     $PASSWD);

use vars
  qw/$LOADER %test_states $RLM $RLM_PROMPT $NO_IPv6 $IPv4_ONLY $RLM_IPV4_IP $RLM_IPV4_MASK $RLM_IPV4_GATEWAY $RLM_IPV6_IP $RLM_IPV6_PREFIX $RLM_IPV6_GATEWAY $RLM_IPV6_ROUTER $RLM_WGET_IPV4_HTTP_PATH $RLM_WGET_IPV6_HTTP_PATH $rlm_state_title $rlm_ipv4_dhcp $rlm_ipv4_ip $rlm_ipv4_netmask $rlm_ipv4_gateway $rlm_enable_ipv6 $rlm_assign_ipv6_global_addr $rlm_ipv6_global_ip $rlm_ipv6_subnet_prefix $rlm_ipv6_gateway $rlm_ipv6_router $ipv4_ip $ipv6_global_ip $ipv6_router_ip $ipv6_link_local_ip $ipv6_gateway $ipv6_status @prompts @prompts_sp $fw_version $dhcp_status $ipv4_mask $ipv4_gateway $ipv6_prefix $ssh_index $ontap_version/;

#-------------------------------------------------------------------------------
# FUNCTION: set_options_10mode
# This sets the values of options autosupport.It takes a pointer to an array as
# argument and the array should have values for the test case title,-state,
# -remove-private-data,-from,-mail-hosts,-to,-support,-support-address in order.
# INPUT: Pointer to an array for autosupport values.
# RETURN: NONE
#-------------------------------------------------------------------------------
sub set_options_10mode($) {
    my ($title, $enable, $content, $from, $Mailhost, $MailID, $enable_Netapp,
        $NETAPP_MAIL_ID);
    ($title, $enable, $content, $from, $Mailhost, $MailID, $enable_Netapp,
     $NETAPP_MAIL_ID) = @{$_[0]};
    set_priv_10mode($fc, "test");
    logcomment(
        " The options being set for the particular test case:: $title is ::\n$enable,$content,$from,$Mailhost,$MailID,$enable_Netapp,$NETAPP_MAIL_ID\n"
    );

    $cmd =
      "system autosupport modify -node local -state $enable -from $from -to  $MailID -mail-hosts  $Mailhost  -support $enable_Netapp -support-address $NETAPP_MAIL_ID -remove-private-data $content ";

    say($fc, "$cmd", '-prompt', "@prompts", '-swallow', '-timeout', 300);
} ## end sub set_options_10mode($)

#-------------------------------------------------------------------------------
# FUNCTION: check_spstatusd_10mode
# This check the values of various fields in options.autosupport with sp status -d
# /rlm_config_from_filer to check that the propagated values to SP are same as
# "system node autosupport show -instance".Takes the array of values extracted
# from options auotsupport as input. Options.autotsupport.to is not being checked.
# $Case_validity gives the case number so that if we want to skip any comparison for a particular testcase we can do so.
# INPUT: Array of values extracted from options autosupport.
# RETURN: NONE
#-------------------------------------------------------------------------------

sub check_spstatusd_10mode {
    $case_validity  = shift @_;
    $enable         = shift @_;
    $content        = shift @_;
    $from           = shift @_;
    $Mailhost       = shift @_;
    $MailID         = shift @_;
    $enable_Netapp  = shift @_;
    $NETAPP_MAIL_ID = shift @_;
    logcomment(
        "The options we set from options autosupport are :: $enable,$content,$from,$Mailhost,$MailID,$enable_Netapp,$NETAPP_MAIL_ID\n"
    );
    (undef, undef, $string) =
      say($fc, "run local rdfile //etc/rlm_config_from_filer",
          '-swallow', '-timeout', 100);

    #$string=~/asup_enabled\s(\w+)\s+asup_to_netapp_enabled\s(\w+)\s+(.+\s+)+asup_from\s+(\S+)\s+autosupport.content\s(\w+)/;
    $string =~ /asup_from\s+(\S+)\s+autosupport.content/;
    $enable_sp        = $enable;
    $enable_Netapp_sp = $enable_Netapp;
    $from_sp          = $1;
    $content_sp       = $content;
    $string =~
      /mailhost\[0\]\s+(.*)\s*mailhost\[1\]\s+(.*)\s*mailhost\[2\]\s+(.*)\s*mailhost\[3\]\s+(.*)\s*mailhost\[4\]\s+(.*)\s*asup/;
    $Mailhost_sp = join(",", $1, $2, $3, $4, $5);
    $Mailhost_sp =~ s/,,+|\s+//g;
    $Mailhost_sp =~ s/,$//;
    $string =~
      /asup_to_netapp\[0\]\s+(.*)\s*(.+\s+)+asup_to_netapp\[1\]\s+(.*)\s*asup(.+\s+)+asup_to_netapp\[2\]\s+(.*)\s*asup(.+\s+)+asup_to_netapp\[3\]\s+(.*)\s*asup(.+\s+)+asup_to_netapp\[4\]\s+(.*)\s*asup(.+\s+)+tuples/;
    $NETAPP_MAIL_ID_sp = join(",", $1, $3, $5, $7, $9);
    $NETAPP_MAIL_ID_sp =~ s/,,+|\s+//g;
    $NETAPP_MAIL_ID_sp =~ s/,$//;
    $string =~
      /asup_to_local\[0\]\s+(.*)\s*cb_to_net(.+\s+)+asup_to_local\[1\]\s+(.*)\s*cb_to_net(.+\s+)+asup_to_local\[2\]\s+(.*)\s*cb_to_net(.+\s+)+asup_to_local\[3\]\s+(.*)\s*cb_to_net(.+\s+)+asup_to_local\[4\]\s+(.*)\s*cb_to_net(.+\s+)+tuples/;
    $MAIL_ID_sp = join(",", $1, $3, $5, $7, $9);
    $MAIL_ID_sp =~ s/,,+|\s+//g;
    $MAIL_ID_sp =~ s/,$//;
    $retval = 1;
    logcomment(
        "The values extracted from sp status -d are :: $enable_sp,$enable_Netapp_sp,$content_sp,$from_sp,$MAIL_ID_sp,$Mailhost_sp,$NETAPP_MAIL_ID_sp\n"
    );

    if ($enable ne $enable_sp) {
        logcomment("ASUP enable field is not properly propagated to SP \n");
        $retval = 0;
    }
    if ($enable_Netapp ne $enable_Netapp_sp) {
        logcomment(
              "NetApp MAIL ID enable field is not properly propagated to SP\n");
        $retval = 0;
    }
    if ($content_sp ne $content) {
        logcomment("Content field is not properly propagated to SP\n");
        $retval = 0;
    }
print "\$from_sp = $from_sp\n\$from = $from\n";
    if ($from_sp ne $from) {
        logcomment("From field is not properly propagated to SP\n");
        $retval = 0;
    }
print "\$MailID = $MailID\n\$MAIL_ID_sp = $MAIL_ID_sp\n";
    if ($MailID ne $MAIL_ID_sp) {
        logcomment("Mail IDs are not propagated to SP \n");
        $retval = 0;
    }
    if ($case_validity == 14) {
        logcomment(
                  "Not matching mailhosts for the testcase ::$case_validity\n");
    } else {
        if ($Mailhost ne $Mailhost_sp) {
            logcomment("Mailhosts are not properly propagated to SP \n");
            $retval = 0;
        }
    }
#    if ($retval == 0) {
#        logresult(
#            'FAIL',
#            " ASUP options from options autosupport and SP status -d are not the same\n"
#        );
#    } else {
#        logresult(
#            'PASS',
#            "All ASUP options are same on options autosupport and SP status -d \n"
#        );
#    }

} ## end sub check_spstatusd_10mode

#-------------------------------------------------------------------------------
# FUNCTION: check_options_10mode
# This checks the options set by set_options subroutine with the output of options.autosupport,Autosupport.support.to is not being checked.
# INPUT: Takes the testcase number and reference to an array as inputs.
# RETURN: Return the array of autosupport values.
#-------------------------------------------------------------------------------

sub check_options_10mode($$) {
    my ($title, $enable, $content, $from, $Mailhost, $MailID, $enable_Netapp,
        $NETAPP_MAIL_ID);
    $case_validity = shift @_;
    ($title, $enable, $content, $from, $Mailhost, $MailID, $enable_Netapp,
     $NETAPP_MAIL_ID) = @{$_[0]};
print "\$from = $from \n";
    say($fc, "row 200", '-swallow', '-prompt', "@prompts", -timeout, 300);
    my (undef, undef, $ASUP_INFO) =
      say($fc, "system autosupport show -node local -instance",
          '-swallow', '-prompt', "@prompts", -timeout, 300);
    $ASUP_INFO =~ /State\: \s+(.+)/;
    my $flag = 1;
    my @retval;

    $ASUP_INFO =~ /State:\s+(\w+)/;
    if ($enable ne $1) {
        logcomment(
            "ASUP enable is not set properly and the value in options autosupport is ::$1 \n"
        );
        $flag = 0;
    }
    @retval = ($1);
#    $ASUP_INFO =~ /Removes Customer-Supplied Data\:\s+(\w+)/;
    $ASUP_INFO =~ /Remove Sensitive Data\:\s+(\w+)/;
    if ($content ne $1) {
        logcomment(
            "Content field is not set properly and the value in options autosupport is ::$1\n"
        );
        $flag = 0;
    }
    push(@retval, $1);
    $ASUP_INFO =~ /From Address\:\s+(\S+)/;
print "\$from = $from\n\$1 = $1\n";
    if ($from ne $1) {
        logcomment(
            "From field is not set properly and the value in options autosupport is ::$1\n"
        );
        $flag = 0;
    }
    push(@retval, $1);
    $ASUP_INFO =~ /SMTP Mail Hosts\:\s+(\d+\.\d+\.\d+\.\d+)/;

    $temp = $1;
#    $temp =~ s/ \,|\, /\,/g;
#    $temp =~ s/\s+//g;

    push(@retval, $temp);
print "\$Mailhost = $Mailhost\n\$temp = $temp \n";
    if ($Mailhost !~ /$temp/) {
        logcomment(
            "Mailhost is not set properly and the value in options autosupport is ::$1\n"
        );
        $flag = 0;
    }
    $ASUP_INFO =~
      /List of To Addresses\:\s+((\w+)\@\w+\.\w+)/;

    $temp = $1;
#    $temp =~ s/ \,|\, /\,/g;
#    $temp =~ s/\s+//g;

    push(@retval, $temp);
print "\$MailID = $MailID\n\$temp = $temp \n";
    if ($MailID !~ /$temp/) {
        logcomment(
            "Mail ID was not set properly and the value in options autosupport is ::$1\n"
        );
        $flag = 0;
    }

#    $ASUP_INFO =~ /Send AutoSupport to Support\:\s+(\w+)/;
    $ASUP_INFO =~ /Send AutoSupport Messages to Vendor Support\:\s+(\w+)/;
    if ($enable_Netapp ne $1) {
        logcomment(
            "Autosupport.support.enable field is not set properly and the value in options autosupport is ::$1\n"
        );
        $flag = 0;
    }
    push(@retval, $1);
    $ASUP_INFO =~ /Support Address\:\s+([\w\.\-\@\, ]+)/;
    $temp = $1;
    $temp =~ s/ \,|\, /\,/g;
    $temp =~ s/\s+//g;
    push(@retval, $temp);
    if ($flag == 0) {
        logresult('FAIL', " Check ASUP options before proceeding\n");
    } else {
        logresult('PASS', "All ASUP options are set properly \n");
    }
    return @retval;
} ## end sub check_options_10mode($$)

#-------------------------------------------------------------------------------
# @name set_priv_10mode
# This sets the privilege level for the filer.and returns true false if the value is set or not respectively.
# INPUT : $connection=filer connection handle,$priv_value=priv value.
# RETURN: Return 1 on success and 0 on failure.
#-------------------------------------------------------------------------------

sub set_priv_10mode($$) {
    my $connection = shift @_;
    my $priv_value = shift @_;
    say($connection, "set -privilege $priv_value -confirmations off",
        '-swallow', '-timeout', 30);
    my (undef, undef, $out_priv) =
      say($connection, "set", '-swallow', '-timeout', 30);
    $out_priv =~ /Privilege Level\:\s+(\w+)/;
    if ($1 eq $priv_value) {
        logcomment("Set privilege successfully");
        $retval = 1;
    } else {
        $retval = 0;
    }
    return $retval;
} ## end sub set_priv_10mode($$)

#-------------------------------------------------------------------------------
# FUNCTION: SP_Side_ASUP_Check_10mode
# This function checks the ASUP generation for sp test autusupport,system power cycle,system power off,
# system reset,system reset backup,system core,watchdog hang,watchdog reset.
# It first stores all EMS we need to check after giving the command for the
# test(only the last EMS of each type before running the test). Then it runs
# the command and checks for the EMS strings after the command and checks the result
# by the check_result subroutine. If we need to check for reminder mail generation a wait of 15 minutes is given.
# INPUT: $testcase=test case for asup configuration,$case_validity
# RETURN: NONE
#-------------------------------------------------------------------------------
sub SP_Side_ASUP_Check_10mode($$) {
    # cluster name is sometimes different from actual filer name in 10mode
    ($filer_prompt) = @{$fh->prompt_norm()};

    logcomment("Getting on to ONTAP prompt");
#    if (ONTAP_Login($fc, $LOGIN, $PASSWD, $filer_prompt)) {
#        logresult("FATAL", "Failed to get to ONTAP prompt");
#    }

    Check_Filer_State_CMode($FILER, $LOGIN, $PASSWD);

    my $testcase = shift @_;
    $case_validity = shift @_;
    @CONN = ($fc, $SP_ssh);
    my ($SP_COMMAND, $SP_ASUP, $ASUP_SEARCH, $Command_Conn) = (
                                         $config_data{SP_COMMAND}->[$testcase],
                                         $config_data{SP_ASUP}->[$testcase],
                                         $config_data{ASUP_SEARCH}->[$testcase],
                                         $CONN[$testcase]);
    logcomment("$SP_COMMAND\n*****$SP_ASUP\n*****$ASUP_SEARCH\n");
    undef @EMS;
    undef @EMS_Heartbeat;
    undef @EMS_Reboot;
    $Reboot_msg = '[SP.critical]: Filer Reboots';
    @EMS        = events_search($ASUP_SEARCH);

    unless ($SP_COMMAND eq "sp test autosupport") {
        @EMS_Reboot = events_search("Reboots");
        ($Reboot_Before, undef) = EMS_specific($Reboot_msg, @EMS_Reboot);
    }
    if ($SP_COMMAND ne "sp test autosupport") {
        $REMINDER_ASUP = $SP_ASUP;
        $REMINDER_ASUP =~ s/First notification/Reminder/;
        logcomment("REMINDER_ASUP=$REMINDER_ASUP,SP_ASUP=$SP_ASUP\n");
        $Heartbeat     = '[SP.critical]: Heartbeat stopped';
        @EMS_Heartbeat = events_search("Heartbeat");
        ($EMS_Heartbeat_Before, undef) =
          EMS_specific($Heartbeat, @EMS_Heartbeat);
        ($EMS_REMINDER_Before, undef) = EMS_specific($REMINDER_ASUP, @EMS);
    } ## end if ($SP_COMMAND ne "sp test autosupport")
    ($EMS_Before, undef) = EMS_specific($SP_ASUP, @EMS);
    logresult('FAIL',
           "$FILER is not pinging please check ONTAP setup before proceeding\n")
      if (ping5($cc, $FILER) == 0);
    set_priv_10mode($fc, "test");
    say($fc, "run local bootargs set AUTOBOOT FALSE",
        '-swallow', '-timeout', 30);
    if ($SP_COMMAND eq "sp test autosupport") {
        say($fc, "run local $SP_COMMAND", -prompt, @prompts, '-timeout', 120);
    }
    elsif ($Command_Conn == $SP_ssh) {
        say($SP_ssh, "$SP_COMMAND", '-noprompt', '-wait',
            'This will cause a dirty shutdown of your appliance',
            '-swallow', '-timeout', 100);
        say($SP_ssh, "y", '-prompt', "SP $FILER\>", '-swallow', '-timeout', 30);
    } elsif ($SP_COMMAND ne "watchdog hang") {
#        say($fc, "run local $SP_COMMAND", -prompt, @prompts, '-timeout', 120);
        say($SP_ssh, "$SP_COMMAND", '-noprompt', '-wait',
            'This will cause a dirty shutdown of your appliance',
            '-swallow', '-timeout', 100);
        say($SP_ssh, "y", '-prompt', "SP $FILER\>", '-swallow', '-timeout', 30);

#        say($SP_ssh, "$SP_COMMAND", -prompt, @prompts, '-timeout', 120);
        logcomment("sleeping for 320 seconds for asup generation");
        sleep 320;    #sleep for asup generation
    } else {
        logcomment("Executing for watchdog hang\n");
        say($fc, "run local watchdog hang",
            -prompt, 'DUMPCORE: END',
            -timeout, '900');
    }
    if ($SP_COMMAND ne "sp test autosupport") { sleep(90); }
    sleep(30);
    if ($SP_COMMAND eq "system power off") {
        say($SP_ssh, "system power on",
            '-prompt', "@prompts", '-swallow', '-timeout', 100);
    }
    unless ($SP_COMMAND eq "sp test autosupport") {
	sleep(90);
        if (ping5($cc, $FILER) != 0) {
            logresult('FAIL', "$FILER is still pinging after $SP_COMMAND\n");
        }
    }
    undef @EMS_Reboot;
    unless (   $SP_COMMAND eq "system power off"
            || $SP_COMMAND eq "sp test autosupport"
            || $SP_COMMAND eq "system core") {
        @EMS_Reboot = events_search("Reboots");
        ($Reboot_After_Last, $Reboot_After_Penultimate) =
          EMS_specific($Reboot_msg, @EMS_Reboot);
        Check_Result($Reboot_Before, $Reboot_After_Penultimate,
                     $Reboot_After_Last);
    }
    $PollEnable  = 1;
    $Iter_Result = 0;
    while ($Iter_Result < 10) {
        undef @EMS;
        @EMS = events_search($ASUP_SEARCH);
        ($EMS_After_Last, $EMS_After_Penultimate) =
          EMS_specific($SP_ASUP, @EMS);
        Check_Result($EMS_Before, $EMS_After_Penultimate, $EMS_After_Last, 0,
                     $Pass_Flag);
        $Iter_Result++;
        if ($PollAgain == 1 && $Iter_result != 10) {
            logcomment("Sleeping for 60 seconds\n");
            sleep(60);
        }
        if ($PollAgain == 1 && $Iter_Result == 10) {
            logresult(
                'FAIL',
                "$SP_Config_testcases_10mode{$Global_Setup}->[0],$ASUP_options_testcases_10mode{$Global_Config}->[0],$functionality_testcases{$Global_Func}->[0]\nNo SEL message is generated for this testcase\n"
            );
        } elsif ($PollAgain == 0) {
            last;
        }
    } ## end while ($Iter_Result < 10)
    $PollEnable = 0;
    if ($SP_COMMAND eq "system core" || $SP_COMMAND eq "watchdog hang") {
        $Temp_WAIT15   = $WAIT15;
        $Temp_REMINDER = $REMINDER;
        $REMINDER      = 1;
        $WAIT15        = 1;
    }
    logcomment("REMINDER=$REMINDER AND WAIT15=$WAIT15\n");

    if ($SP_COMMAND ne "sp test autosupport" && $WAIT15 == 1) {
        logcomment("Waiting for 15 minutes for reminder mail generation\n");
        if ($REMINDER == 0) {
    logcomment("Getting on to ONTAP prompt");
#    if (ONTAP_Login($fc, $LOGIN, $PASSWD, $filer_prompt)) {
#        logresult("FATAL", "Failed to get to ONTAP prompt");
#    }

     Check_Filer_State_CMode($FILER, $LOGIN, $PASSWD);
        }
        sleep(810);
        undef @EMS;
        undef @EMS_Heartbeat;
        undef @EMS_Reboot;
        if ($SP_COMMAND eq "system power off" || $SP_COMMAND eq "system core") {
            @EMS_Reboot = events_search("Reboots");
            ($Reboot_After_Last, $Reboot_After_Penultimate) =
              EMS_specific($Reboot_msg, @EMS_Reboot);
            Check_Result($Reboot_Before, $Reboot_After_Penultimate,
                         $Reboot_After_Last);
        }
        @EMS           = events_search($ASUP_SEARCH);
        @EMS_Heartbeat = events_search("Heartbeat");
        ($EMS_REMINDER_After_Last, $EMS_REMINDER_After_Penultimate) =
          EMS_specific($REMINDER_ASUP, @EMS);
        ($EMS_Heartbeat_Last, $EMS_Heartbeat_Penultimate) =
          EMS_specific($Heartbeat, @EMS_Heartbeat);
        if ($REMINDER == 0) {
            Check_Result($EMS_Heartbeat_Before, $EMS_Heartbeat_Penultimate,
                         $EMS_Heartbeat_Last, 1);
            Check_Result($EMS_REMINDER_Before, $EMS_REMINDER_After_Penultimate,
                         $EMS_REMINDER_After_Last, 1);
        } else {
            Check_Result($EMS_Heartbeat_Before, $EMS_Heartbeat_Penultimate,
                         $EMS_Heartbeat_Last);
            Check_Result($EMS_REMINDER_Before, $EMS_REMINDER_After_Penultimate,
                         $EMS_REMINDER_After_Last, 0, $Pass_Flag);
        }

    } ## end if ($SP_COMMAND ne "sp test autosupport"...
    if ($SP_COMMAND eq "system core" || $SP_COMMAND eq "watchdog hang") {
        $REMINDER = $Temp_REMINDER;
        $WAIT15   = $Temp_WAIT15;
    }
    if ($SP_COMMAND eq "system reset backup") {
        say($SP_ssh,
            "system reset primary",
            '-noprompt',
            '-wait',
            'This will cause a dirty shutdown of your appliance',
            '-swallow',
            '-timeout',
            100);
        say($SP_ssh, "y", '-prompt', "SP $FILER\>", '-swallow', '-timeout',
            300);
        sleep(75);
    } ## end if ($SP_COMMAND eq "system reset backup")
    logcomment("Getting on to ONTAP prompt");
#    if (ONTAP_Login($fc, $LOGIN, $PASSWD, $filer_prompt)) {
#        logresult("FATAL", "Failed to get to ONTAP prompt");
#    }

     Check_Filer_State_CMode($FILER, $LOGIN, $PASSWD);

} ## end sub SP_Side_ASUP_Check_10mode($$)

#-------------------------------------------------------------------------------
# FUNCTION: ASUP_Heartbeat_SP_10mode
# This checks for ASUP when heartbeat from ONTAP to SP is missing. Algorithm is similar to SP_SIDE_ASUP_CHECK subroutine.
# INPUT: $case_validity
# RETURN: NONE
#-------------------------------------------------------------------------------
sub ASUP_Heartbeat_SP_10mode($$) {
    $case_validity = shift @_;

    logcomment(
        "This will test if ASUP from SP is generated when heartbeat from ONTAP to SP  is missing\n"
    );

    logcomment("Getting on to ONTAP prompt");
#    if (ONTAP_Login($fc, $LOGIN, $PASSWD, $filer_prompt)) {
#        logresult("FATAL", "Failed to get to ONTAP prompt");
#    }

     Check_Filer_State_CMode($FILER, $LOGIN, $PASSWD);

    undef @EMS,;
    @EMS = events_search("HEARTBEAT");
    ($EMS_Before, undef) = EMS_specific($config_data{Heartbeat_ASUP}, @EMS);
    $Reminder_ASUP = $config_data{Heartbeat_ASUP};
    $Reminder_ASUP =~ s/First notification/Reminder/;
    ($EMS_REMINDER_Before, undef) = EMS_specific($REMINDER_ASUP, @EMS);
    set_priv_10mode($fc, "test");
    (undef, undef, $Heartbeat_Status) =
      say($fc, "run local sp heartbeat status", '-swallow', '-timeout', 30);

    if ($Heartbeat_Status =~ "Heartbeat to SP is enabled") {
        say($fc, "run local sp heartbeat disable", '-swallow', '-timeout', 30);
    } elsif ($Heartbeat_Status =~ "Heartbeat to SP is disabled") {
        say($fc, "run local sp heartbeat enable", '-swallow', '-timeout', 30);
        sleep(60);
        say($fc, "run local sp heartbeat disable", '-swallow', '-timeout', 30);
    }
    (undef, undef, $Heartbeat_Status) =
      say($fc, "run local sp heartbeat status", '-swallow', '-timeout', 30);
    unless ($Heartbeat_Status =~ "Heartbeat to SP is disabled") {
        logresult('FAIL', "Could not disable Heartbeat to SP\n");
    }
    sleep(900);
    undef @EMS;
    @EMS = events_search("HEARTBEAT");
    ($EMS_After_Last, $EMS_After_Penultimate) =
      EMS_specific($config_data{Heartbeat_ASUP}, @EMS);
    $PollEnable  = 1;
    $Iter_Result = 0;
    while ($Iter_Result < 10) {

        undef @EMS;
        @EMS = events_search("HEARTBEAT");    #$ASUP_SEARCH);
        ($EMS_After_Last, $EMS_After_Penultimate) =
          EMS_specific($config_data{Heartbeat_ASUP}, @EMS);
        Check_Result($EMS_Before, $EMS_After_Penultimate, $EMS_After_Last, 0,
                     $Pass_Flag);
        $Iter_Result++;
        if ($PollAgain == 1 && $Iter_result != 10) {
            logcomment("Sleeping for 60 seconds\n");
            sleep(60);
        }
        if ($PollAgain == 1 && $Iter_Result == 10) {
            logresult(
                'FAIL',
                "$SP_Config_testcases{$Global_Setup}->[0],$ASUP_options_testcases{$Global_Config}->[0],$functionality_testcases{$Global_Func}->[0]\nNo SEL message is generated for this testcase\n"
            );
        } elsif ($PollAgain == 0) {
            last;
        }
    } ## end while ($Iter_Result < 10)
    $PollEnable = 0;
    if ($WAIT15 == 1) {
        logcomment(
                 "Waiting for 15 minutes to check reminder mail generation \n");
        if ($REMINDER == 0) {
            say($fc, "run local sp heartbeat enable",
                '-swallow', '-timeout', 30);
            (undef, undef, $Heartbeat_Status) =
              say($fc, "run local sp heartbeat status",
                  '-swallow', '-timeout', 30);
            unless ($Heartbeat_Status =~ "Heartbeat to SP is enabled") {
                logresult('FAIL', "Could not enable Heartbeat to SP");
            }
        } ## end if ($REMINDER == 0)
        sleep(900);
        undef @EMS;
        ($EMS_REMINDER_After_Last, $EMS_REMINDER_After_Penultimate) =
          EMS_specific($REMINDER_ASUP, @EMS);
        if ($REMINDER == 1) {
            Check_Result($EMS_REMINDER_Before, $EMS_REMINDER_After_Penultimate,
                         $EMS_REMINDER_After_Last, 0, $Pass_Flag);
        } else {
            Check_Result($EMS_REMINDER_Before, $EMS_REMINDER_After_Penultimate,
                         $EMS_REMINDER_After_Last, 1, $Pass_Flag);
        }
    } ## end if ($WAIT15 == 1)
    say($fc, "run local sp heartbeat enable", '-swallow', '-timeout', 30);

} ## end sub ASUP_Heartbeat_SP_10mode($$)

#-------------------------------------------------------------------------------
# FUNCTION: Negative_ASUP_Cases
# This subroutine checks for non generation of ASUP for reboot/halt and user triggered Panic.
# INPUT: $case_validity
# RETURN: NONE
#-------------------------------------------------------------------------------
sub Negative_ASUP_Cases_10mode($) {
    $case_validity = shift @_;
    if    ($case_validity == 10) { $SP_COMMAND = "run local panic"; }
    elsif ($case_validity == 11) { $SP_COMMAND = "system node reboot $FILER"; }
    elsif ($case_validity == 12) {
        $SP_COMMAND = "system node halt $FILER -confirmations off ";
    } else {
        logresult("FATAL", "Invalid testcase\n");
    }
    logcomment(
         "This will test if ASUP is generated on issuing Panic,Halt or Reboot\n"
    );

    logcomment("Getting on to ONTAP prompt");
#    if (ONTAP_Login($fc, $LOGIN, $PASSWD, $filer_prompt)) {
#        logresult("FATAL", "Failed to get to ONTAP prompt");
#    }

    Check_Filer_State_CMode($FILER, $LOGIN, $PASSWD);

    @EMS = events_search("ASUP");
    ($EMS_Before, undef) = $EMS[-1];
    set_priv_10mode($fc, "test");
    say($fc, "run local bootargs set AUTOBOOT TRUE", '-swallow', '-timeout',
        30);
    if ($SP_COMMAND eq "panic") {
        say($fc, "$SP_COMMAND", -prompt, 'DUMPCORE: END', -timeout, '900');
    } else {
        say($fc, "$SP_COMMAND", '-blind', '-noprompt', '-timeout', 300);
        sleep(450);
    }
    logcomment("Getting on to ONTAP prompt");
#    if (ONTAP_Login($fc, $LOGIN, $PASSWD, $filer_prompt)) {
#        logresult("FATAL", "Failed to get to ONTAP prompt");
#    }

    Check_Filer_State_CMode($FILER, $LOGIN, $PASSWD);
    undef @EMS;
    @EMS                   = events_search("ASUP");
    $EMS_After_Last        = $EMS[-1];
    $EMS_After_Penultimate = $EMS[-2];
    Check_Result($EMS_Before, $EMS_After_Penultimate, $EMS_After_Last, 1);
} ## end sub Negative_ASUP_Cases_10mode($)

#-------------------------------------------------------------------------------
# FUNCTION: load_ASUP_options_testcases_10mode
# This contains the options autosupport testcases hash.
# INPUT: hash of asup_configuration.txt
# RETURN: NONE
#-------------------------------------------------------------------------------

sub load_ASUP_options_testcases_10mode(%) {
    my %config_data = @_;
    %ASUP_options_testcases_10mode = (
        1 => [
            "State 1.set one valid mail ID for local and netapp ID and  mailserver",
            "enable",
            "false",
            "State1\@netapp.com",
            $config_data{IPV4_MAILHOST}->[0],
            $config_data{MAIL_ID}->[0],
            "enable",
            $config_data{NETAPP_MAIL_ID}->[0]],
        2 => [
            "State 2.set two valid mail ID for local and netapp ID and a proper mailserver hostname",
            "enable",
            "false",
            "State2\@netapp.com",
            $config_data{NAME_IPV4_MAILHOST}->[0],
            "$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1]",
            "enable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1]"
        ],
        3 => [
            "State 3.set the same two valid mail ID for local and netapp ID and a proper mailserver",
            "enable",
            "false",
            "State3\@netapp.com",
            $config_data{IPV4_MAILHOST}->[1],
            "$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[0]",
            "enable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0]"
        ],
        4 => [
            "State 4.set the five valid mail ID for local and netapp ID and a proper mailserver",
            "enable",
            "false",
            "State4\@netapp.com",
            $config_data{IPV4_MAILHOST}->[0],
            "$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]",
            "enable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"
        ],
        5 => [
            "State 5.Set 2 wrong Mail ID for local and 2 correct Mail ID for netapp ID and a proper mailserver",
            "enable",
            "false",
            "State5\@netapp.com",
            $config_data{IPV4_MAILHOST}->[1],
            "$config_data{BAD_MAIL_ID}->[0],$config_data{BAD_MAIL_ID}->[1]",
            "enable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0]"
        ],
        6 => [
            "State 6.Set 2 wrong Mail ID for the Netapp ID and 2 proper IDs for local MAIL ID and a proper mailserver",
            "enable",
            "false",
            "State6\@netapp.com",
            $config_data{IPV4_MAILHOST}->[2],
            "$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3]",
            "enable",
            "$config_data{BAD_NETAPP_MAIL_ID}->[0],$config_data{BAD_NETAPP_MAIL_ID}->[1]"
        ],
        7 => [
            "State 7.Set a wrong followed by a right mail ID for local and all correct IDs for Netapp and a proper mailserver",
            "enable",
            "false",
            "State7\@netapp.com",
            $config_data{IPV4_MAILHOST}->[3],
            "$config_data{BAD_MAIL_ID}->[2],$config_data{MAIL_ID}->[3]",
            "enable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1]"
        ],
        8 => [
            "State 8.Set a wrong followed by a right mail ID for Netapp and all correct IDs for local and a proper mailserver",
            "enable",
            "false",
            "State8\@netapp.com",
            $config_data{IPV4_MAILHOST}->[2],
            "$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3]",
            "enable",
            "$config_data{BAD_NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1]"
        ],
        9 => [
            "State 9.Set 4 wrong followed by a correct Mail ID for local and Netapp user and a proper mailserver",
            "enable",
            "false",
            "State9\@netapp.com",
            $config_data{IPV4_MAILHOST}->[0],
            "$config_data{BAD_MAIL_ID}->[0],$config_data{BAD_MAIL_ID}->[1],$config_data{BAD_MAIL_ID}->[2],$config_data{BAD_MAIL_ID}->[3],$config_data{MAIL_ID}->[0]",
            "enable",
            "$config_data{BAD_NETAPP_MAIL_ID}->[0],$config_data{BAD_NETAPP_MAIL_ID}->[1],$config_data{BAD_NETAPP_MAIL_ID}->[2],$config_data{BAD_NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[0]"
        ],
        10 => [
            "State 10.Set the same correct MAIL ID for all five options in local and Netapp and a proper mailserver",
            "enable",
            "false",
            "State10\@netapp.com",
            $config_data{IPV4_MAILHOST}->[0],
            "$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[0]",
            "enable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[0]"
        ],
        11 => [
            "State 11.Set a mixture of right and wrong MAIL ID for both netapp and local and a proper mailserver",
            "enable",
            "false",
            "State11\@netapp.com",
            $config_data{IPV4_MAILHOST}->[0],
            "$config_data{BAD_MAIL_ID}->[0],$config_data{BAD_MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{BAD_MAIL_ID}->[4]",
            "enable",
            "$config_data{BAD_NETAPP_MAIL_ID}->[0],$config_data{BAD_NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{BAD_NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"
        ],
        12 => [
            "State 12.Set Netapp mail option to off and send mail to correct IDs for both netapp and local and set a proper mailserver",
            "enable",
            "false",
            "State12\@netapp.com",
            $config_data{IPV4_MAILHOST}->[0],
            "$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1]",
            "disable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"
        ],
        13 => [
            "State 13.Set the content option to minimal and send mail to all five netapp and local users and set a proper mailserver",
            "enable",
            "true",
            "State13\@netapp.com",
            $config_data{IPV4_MAILHOST}->[0],
            "$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]",
            "enable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"
        ],
        14 => [
            "State 16.Set the autosupport enable option to off and try to send mail to all five netapp and local IDs and set a proper mailserver",
            "disable",
            "false",
            "State16\@netapp.com",
            $config_data{IPV4_MAILHOST}->[0],
            "$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]",
            "enable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"
        ],
        15 => [
            "State 15.Set 2 wrong followed by one right mailserver and try to send mail to all five local and netapp IDs",
            "enable",
            "false",
            "State15\@netapp.com",
            "$config_data{BAD_IPV4_MAILHOST}->[0],$config_data{BAD_IPV4_MAILHOST}->[1],$config_data{IPV4_MAILHOST}->[0]",
            "$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]",
            "enable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"
        ],
        16 => [
            "State 16.Set 3 wrong followed by one right mailserver and try to send mail to all five local and netapp IDs",
            "enable",
            "false",
            "State16\@netapp.com",
            "$config_data{BAD_IPV4_MAILHOST}->[0],$config_data{BAD_IPV4_MAILHOST}->[1],$config_data{BAD_IPV4_MAILHOST}->[2],$config_data{IPV4_MAILHOST}->[0]",
            "$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]",
            "enable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"
        ],
        17 => [
            "State 17.Set all 3 right mailservers and try to send mail to all five local and netapp IDs",
            "enable",
            "false",
            "State17\@netapp.com",
            "$config_data{IPV4_MAILHOST}->[0],$config_data{IPV4_MAILHOST}->[1],$config_data{IPV4_MAILHOST}->[2]",
            "$config_data{MAIL_ID}->[0],$config_data{MAIL_ID}->[1],$config_data{MAIL_ID}->[2],$config_data{MAIL_ID}->[3],$config_data{MAIL_ID}->[4]",
            "enable",
            "$config_data{NETAPP_MAIL_ID}->[0],$config_data{NETAPP_MAIL_ID}->[1],$config_data{NETAPP_MAIL_ID}->[2],$config_data{NETAPP_MAIL_ID}->[3],$config_data{NETAPP_MAIL_ID}->[4]"
        ],);
} ## end sub load_ASUP_options_testcases_10mode(%)

#-------------------------------------------------------------------------------
# FUNCTION: rlm_setup_10mode
# To do "rlm setup" in ONTAP
# INPUT: $FILER, array of values of ipv4 config.
# RETURN: NONE
#-------------------------------------------------------------------------------
sub rlm_setup_10mode ($$;$) {
    my $FILER = shift @_;
    my $fc    = rlm_make_conn_to_ontap($FILER);
    ($rlm_state_title,  $rlm_ipv4_dhcp, $rlm_ipv4_ip,
     $rlm_ipv4_netmask, $rlm_ipv4_gateway) = @{$_[0]};
    my $firmware  = $_[1];
    my $max_tries = 10;
    logcomment("$rlm_state_title\n");

    logcomment("Configuring $RLM_PROMPT using \'$RLM setup\' in ONTAP");

    say($fc, " ", -swallow)
      ; #Sometimes when ontap prompt is not there then "sp setup" is getting failed
    say($fc, "run local $RLM setup",
        '-swallow', '-noprompt', '-wait',
        "Would you like to configure the $RLM_PROMPT");
    say($fc, "yes", '-swallow', '-noprompt', '-wait',
        "Would you like to enable DHCP on the $RLM_PROMPT LAN interface");
    my ($match) = say($fc, $rlm_ipv4_dhcp, '-swallow', -prompt, @prompts,
                      "Please enter the IP address (?:for|of) the $RLM_PROMPT");

    if ($rlm_ipv4_dhcp eq "no") {
        say($fc, $rlm_ipv4_ip, '-swallow', '-noprompt', '-wait',
            "Please enter the netmask (?:for|of) the $RLM_PROMPT");
        say($fc, $rlm_ipv4_netmask, '-swallow', '-noprompt', '-wait',
            "Please enter the IP address (?:for|of) the $RLM_PROMPT gateway");
        my (undef,undef,undef,$prompt) = say($fc, $rlm_ipv4_gateway, '-swallow',-prompt,'Do you want to enable IPv6 on the SP ?',@prompts);
        if($prompt =~ /Do you want to enable IPv6 on the SP/)
        {
	   say($fc, 'no', -swallow);
        }
    }

    logcomment("$RLM setup completed successfully");
} ## end sub rlm_setup_10mode ($$;$)

#-------------------------------------------------------------------------------
# FUNCTION: rlm_reboot_rlm_10mode
#  		Reboot's the RLM from ONTAP.
# INPUT : $FILER
# RETURN : NONE
#-------------------------------------------------------------------------------
sub rlm_reboot_rlm_10mode ($) {
    my $FILER = shift @_;
    my $fc    = rlm_make_conn_to_ontap($FILER);

    logcomment("Rebooting $RLM_PROMPT at ONTAP");
    say($fc,
        "run local $RLM reboot",
        '-prompt',
        "If your console connection is through the (?:$RLM_PROMPT|Service Processor|Remote LAN Module),it will be disconnected",
        '-timeout',
        30);
    say($fc, "y", '-prompt', @prompts, '-timeout', 300);

    sleep(140);

    my ($timer, $status) = (0, 0);
    $fc->discardbuff;
    while ($timer <= 20) {
        my (undef, undef, $out) = say($fc, "run local $RLM status",
                                      '-prompt', @prompts, '-timeout', 300);
        if ($out =~ /Online/) {
            $status = 1;
            $timer  = 21;
        }
        sleep(15);
        $timer = $timer + 1;
    } ## end while ($timer <= 20)
    unless ($status) {
        logresult('FATAL', "$RLM_PROMPT is not getting back to online state");
    }
} ## end sub rlm_reboot_rlm_10mode ($)

#-------------------------------------------------------------------------------
# FUNCTION: rlm_verify_rlm_status_10mode
#				Verifies "rlm status" output in ONTAP and SP
# INPUT: $conn=connection handler of filer, $mode=ONTAP or LOADER,$state,
# RETURN: NONE
#-------------------------------------------------------------------------------
sub rlm_verify_rlm_status_10mode ($$;$) {
    my ($conn, $mode, $state) = @_;
    if ($mode =~ /ONTAP|LOADER/i) {
        $conn = rlm_make_conn_to_ontap($conn);
    }

    logcomment("\"$RLM status\" in $mode \n");
    if ($mode eq $RLM_PROMPT) {
        say($conn, "run local version", -prompt, @prompts, -timeout, 300);
    }

    my $rlm_output;
    unless ($mode =~ /LOADER/i) {
        my (undef, undef, $output, $prompt) =
          say($conn, "run local $RLM status", -prompt, @prompts, -timeout, 300);
        $rlm_output = $output;
    } else {
        sleep(15);
        my @waitstrings = (
                 'Link\sLocal\s(?:IP|Address)(?:\s+IP\sAddress)?\:\s+([\w\:]+)',
                 'IPv6(?:\sconfiguration)?\:(?:\s+)?(Disabled)?');
        my ($match, $before, $output, $prompt) =
          say($conn, "run local $RLM status",
              '-swallow', '-wait', @waitstrings, -noprompt, '-prompt',
              @prompts);
        $rlm_output = $before . "\n" . $match . "\n" . $output;
    } ## end else

    logcomment("Verifying the $RLM status output in $mode...");
    if ($state) {
        if (rlm_extract_rlm_status_info($rlm_output, $mode, $state)) {
            logresult(
                'PASS',
                "$RLM status output looking as expected for the test state: $state"
            );
        } elsif (($ipv6_global_ip =~ /Disabled/i) && ($ipv6_router_ip)) {
            logresult(
                'WARN',
                "$RLM_PROMPT Global IPv6 expected, but RA seen and Global IP Disabled for the test state: $state"
            );
        } else {
            logresult('FATAL',
                    "State: $state FAILED due to incorrect $RLM status output");
        }
    } else {
        rlm_extract_rlm_status_info($rlm_output, $mode);
        $ipv4_ip        = 0 if ($ipv4_ip        =~ /Disabled|unknown/i);
        $ipv4_mask      = 0 if ($ipv4_mask      =~ /Disabled|unknown/i);
        $ipv4_gateway   = 0 if ($ipv4_gateway   =~ /Disabled|unknown/i);
        $ipv6_router_ip = 0 if ($ipv6_router_ip =~ /Disabled|unknown/i);
        $ipv6_status = 'Disabled' if ($NO_IPv6 && ($TEST_STATE <= $IPv4_ONLY));
    }
} ## end sub rlm_verify_rlm_status_10mode ($$;$)

#-------------------------------------------------------------------------------
# FUNCTION: rlm_load_ipv4_config_cases.
# To load ipv4 test cases
# INPUT: $FILER
# RETURN: NONE
#-------------------------------------------------------------------------------
sub rlm_load_ipv4_config_cases ($) {
    my $FILER = shift @_;

    rlm_load_tharnhost_data($FILER);

    %test_states = (
               1 => ["State 1. $RLM_PROMPT with no IPv4 or IPv6 addresses",
                     "no", '0.0.0.0', '0.0.0.0', '0.0.0.0', "no"],
               2 => ["State 2. $RLM_PROMPT with static IPv4 address only",
                     "no", $RLM_IPV4_IP, $RLM_IPV4_MASK, $RLM_IPV4_GATEWAY, "no"
                    ],
               3 => ["State 3. $RLM_PROMPT with dhcp IPv4 address only",
                     "yes", '', '', '', "no"],);
} ## end sub rlm_load_ipv4_config_cases ($)

#-------------------------------------------------------------------------------
# FUNCTION: load_SP_Config_testcases
# This hash contains the testcases related to sp setup.
# INPUT : NONE
# RETURN: NONE
#-------------------------------------------------------------------------------
sub load_SP_Config_testcases_10mode {
    %SP_Config_testcases_10mode = (
                    1 => ["State 1. $RLM_PROMPT with static IPV4 addresses", 2],
                    2 => ["State 2. $RLM_PROMPT with DHCP IPV4 addresses",   3],
                                  );

}

#-------------------------------------------------------------------------------
# FUNCTION: valid_positive_testcase
# This subroutine checks if the options autosupport testcases should pass or fail
#according to SP setup and ASUP configuration and accordingly populates a matrix for each SP setup.
# INPUT: array from load_SP_Config_testcases_10mode function
# RETURN: NONE
#-------------------------------------------------------------------------------
sub valid_positive_testcase_10mode($) {
    $SP_config_case = shift @_;
    undef @valid_cases;
    if ($SP_config_case == 1 || $SP_config_case == 2) {
        @valid_cases =
          (1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17);
    }

}

#-------------------------------------------------------------------------------
# FUNCTION:  pass_flag
# This sets $Pass_Flag as true or false according to the Autosupport options test
#case number by checking the @valid_cases matrix for the particular sp setup.
# INPUT: $testcase
# RETURN: NONE
#-------------------------------------------------------------------------------
sub pass_flag_10mode($) {
    my $testcase = shift @_;
    undef $Pass_Flag;
    $track = 0;
    logcomment("the valid test cases for 10mode are :: @valid_cases \n");

    foreach (@valid_cases) {
        if ($valid_cases[$track] == $testcase) {
            $Pass_Flag = 2;
            logcomment("Setting the flag to 2 as it is a positive testcase\n");
            return;
        } else {
            $track++;
        }
    }
    $Pass_Flag = 1;
} ## end sub pass_flag_10mode($)


##########################################################################
## Function Name : disableOntapToSPHeartbeat_10mode
## Return Value  : PASS or FAIL
## Description   : This function disables Ontap to SP heartbeat in 10mode
###########################################################################
sub disableOntapToSPHeartbeat_10mode {
    tstlog -comment, "Disabling Ontap to SP heartbeat";
    say($fc, "set -privilege test -confirmations off", -swallow, -timeout, 100);
    my (undef, undef, $heartbeatStatus) =
      say($fc, "run local sp heartbeat status", '-swallow', '-timeout', 30);
    if ($heartbeatStatus =~ /Heartbeat to SP is enabled/) {
        say($fc, "run local sp heartbeat disable", '-swallow', '-timeout', 30);
    } else {
        say($fc, "run local sp heartbeat enable", '-swallow', '-timeout', 30);
        sleep(60);
        say($fc, "run local sp heartbeat disable", '-swallow', '-timeout', 30);
    }
    (undef, undef, $heartbeatStatus) =
      say($fc, "run local sp heartbeat status", '-swallow', '-timeout', 30);
    if ($heartbeatStatus =~ /Heartbeat to SP is disabled/) {
        tstlog -comment, "Ontap to SP heartbeat is disabled";
        return "PASS";
    } else {
        tstlog -FATAL, "Could not disable Ontap to SP heartbeat";
    }
} ## end sub disableOntapToSPHeartbeat_10mode

##########################################################################
## Function Name : enableOntapToSPHeartbeat_10mode
## Return Value  : PASS or FAIL
## Description   : This function enables to SP heartbeat in 10mode
###########################################################################

sub enableOntapToSPHeartbeat_10mode {
    tstlog -comment, "Enabling Ontap to SP heartbeat";
    say($fc, "set -privilege test -confirmations off", -swallow, -timeout, 100);
    my (undef, undef, $heartbeatStatus) =
      say($fc, "run local sp heartbeat enable", '-swallow', '-timeout', 30);
    if ($heartbeatStatus =~ /Heartbeat to SP enabled/) {
        tstlog -comment, "Ontap to SP heartbeat is enabled";
        return "PASS";
    } else {
        tstlog -FATAL, "Could not enable Ontap to SP heartbeat";
    }
} ## end sub enableOntapToSPHeartbeat_10mode


###########################################################################
## Function Name : stopSPToOntapHeartbeat_10mode
## Return Value  : PASS or FAIL
## Description   : This function deliberately stops SP heartbeat from 10mode
###########################################################################

sub stopSPToOntapHeartbeat_10mode {
    my $rlm_ssh_ip = $_[0];
    tstlog -comment, "Deliberately stopping SP heartbeat";
    say($fc, "set -privilege test -confirmations off", -swallow, -timeout, 100);
    say($fc, "run local options ems.autosuppress.enable off",
        '-swallow', '-timeout', 30);
    say($fc, "run local sp debugmode", '-swallow', '-timeout', 30);
    my $root_ssh =
      rlm_make_conn_to_rlm($FILER, $rlm_ssh_ip, 'root', 'superuser');
    say($root_ssh, "sed -i \'\/r2fbeat\/d\' \/etc\/watcher.conf",
        '-swallow', '-timeout', 30);
    my (undef, undef, $ID) =
      say($root_ssh, 'ps - ef | grep watcher', '-swallow', '-timeout', 30);

    if ($ID =~ /(\d*)[\s]?root[\s]*(\d*)[\s]?S[\s]*\/usr\/local\/bin\/watcher/i)
    {
        say($root_ssh, "kill -9 $1", '-swallow', '-timeout', 30);
    } else {
        tstlog -FATAL, "Failed to kill the watcher PID";
    }
    (undef, undef, $ID) =
      say($root_ssh, 'ps - ef | grep r2fbeat', '-swallow', '-timeout', 30);
    if ($ID =~ /(\d*)[\s]?root[\s]*(\d*)[\s]?S[\s]*\/usr\/local\/bin\/r2fbeat/i)
    {
        say($root_ssh, "kill -9 $1", '-swallow', '-timeout', 30);
    } else {
        tstlog -FATAL, "Failed to kill the r2fbeat PID";
    }
    return "PASS";
} ## end sub stopSPToOntapHeartbeat_10mode

1;
