# -*- perl -*-
# $Id:
#
#	Copyright (c) 2011 Network Appliance, Inc.
#	All Rights reserved
##
## @summary This file provides common functions for RLM tests.
##
## @description
## console_redirection_10mode.pm is a common module for account lockout and password aging RLM tests.
##
## Sub-Routines:
##
## 1. SearchUser_10mode
## -   To search a user in 10mode
## 2. CreateUser_10mode
## -  Creates a user account in 10 mode
## 3. DeleteUser_10mode
## -   Deletes a user in 10 mode
## 4. Create_Mod_Del_User_10mode
## -   Function to create ,modify and delete the user in 10 mode.
## 5. UserLogin_10mode
## -	 Login as a specified user to the 10mode filer console
## 6. LockUser_10mode
## -	 Locks a user account by using lock option.
## 7. UnLockUser_10mode
## -	 To Recover the locked user account by using unlock option.
## 8. TestUser_10mode
## -	 To check if a user account is functional in 10 mode.
## 9. SP_Update_10mode
##	- 	 To update rlm/sp build from ONTAP for 10 mode filer.
## 10. Lock_Client
## -   To verify the client is blocked when login to RLM/SP is tried with
##     wrong password more number of times
## 11. Block_Unblock_Msg
## -   To verify the Block/Unblock message from the RLM/SP
##	12. Get_Events
## - 	 To verify the the user can connect to RLM/SP to get events
## 13. rlm_setup_10mode
## @status private
##
## @author  mariat@netapp.com
##
## @notes Created by Mindteck (India) Ltd
##
#########################################################################################

package platval::service_processor::console_redirection_10mode;

use strict;
use warnings;
use Shell;
use Tharn;
use BTS_Config;
use Params::Validate qw(validate OBJECT SCALAR);

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
   SearchUser_10mode
   CreateUser_10mode
   DeleteUser_10mode
   Create_Mod_Del_User_10mode
   UserLogin_10mode
   LockUser_10mode
   UnLockUser_10mode
   TestUser_10mode
   SP_Update_10mode
   Lock_Client
   Block_Unblock_Msg
   Get_Events

);

our $VERSION = '1.0';

my $ssh_conn_count = 1;
#----------------------------------------------------------------------------
# FUNCTION:	To search if a user account exists in 10 mode.
# INPUT   :	$_[0]=Filer connection handle, $_[1]= User name.
# RETURN  :	1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub SearchUser_10mode {
    my $output=0;
    my $cmd = "security login show -username $_[1] -application $_[2]";
    eval {
        $output = $_[0]->cli(cmd => $cmd);

    };
    if ($output =~ /.*$_[1]X;.*/) {
        logcomment("User $_[1] Exists");
        return 1;
    } elsif ($output =~ /There are no entries matching your query/) {
        logcomment("User $_[1] Does Not Exist");
        return 0;
    }
} ## end sub SearchUser_10mode


#----------------------------------------------------------------------------
# FUNCTION:	To create a user account in 10mode.
# INPUT		:	server 	- server connection handle
#					name 		- name of the user
#					passwd	- password for the user
#					role 		- role of the user
# RETURN  :	1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub CreateUser_10mode {
    my (%opt, $cmd);
    %opt = @_;
    my $user_exists = SearchUser_10mode($opt{'server'}, $opt{'name'}, "*",);
    if ($user_exists == 1) {
        logcomment(
                  "Could not add user <$_[1]>. Error: User is already present");
        return 0;
    } else {
        eval {
            $cmd =
              "security login create -username $opt{'name'} -application console -authmethod password -role $opt{'role'}";
            $opt{'server'}->cli(cmd => $cmd, noprompt => 1);
            $opt{'server'}->cli(cmd => $opt{'passwd'}, poke => $opt{'passwd'});
            $cmd =
              "security login create -username $opt{'name'} -application service-processor -authmethod password -role $opt{'role'}";
            $opt{'server'}->cli(cmd => $cmd);
        };
        if ($@) {
            logcomment("Failed to add $opt{'name'}");
            return 0;
        } else {
            logcomment("Succeeded to add $opt{'name'}");
            return 1;
        }
    } ## end else [ if ($user_exists == 1)]
} ## end sub CreateUser_10mode

#----------------------------------------------------------------------------
# FUNCTION: To delete a User/all Users other than root in 10 mode
# INPUT	  :	$_[0] = Filer connection handle,
#			$_[1] = Pass one User name to delete a particular user OR
#					keyword "all" to delete all users except 'root' user.
#			$_[2] = Application of each user OR '*' to delete user with all application
# RETURN  :	1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub DeleteUser_10mode($$$) {
    my $username    = $_[1];
    my $user_exists = 0;
    my $cmd;
    if ($username eq "all") {

        # <TBD>
        return 1;
    } else {

        #Delete perticular user from the list
        logcomment("Search the $username");
        $user_exists = SearchUser_10mode($_[0], $_[1], $_[2]);
        if ($user_exists == 0) {
            return 1;
        }
        $cmd = "security login delete -username $_[1] *";
        $_[0]->cli(cmd => $cmd);
        $user_exists = SearchUser_10mode($_[0], $_[1], $_[2]);
        if ($user_exists == 1) {
            logcomment("Failed to delete $_[1]");
            return 0;
        } else {
            logcomment("Succeeded to delete $_[1]");
            return 1;
        }
    } ## end else [ if ($username eq "all")]
} ## end sub DeleteUser_10mode($$$)

#------------------------------------------------------------------------------
#	FUNCTION	: 	Function to create ,modify and delete the user
#	INPUT		:	server 		- 	server connection handle
#					name 	-	name of the user
#					passwd	-	password for the user
#					application 	- application type like console, sp etc.
#					role - role of the user
#					action 	- 	create/modify/delete
#	RETURN		:  	0 - user create/modify/delete successfull.
#					1 - failed to create/modify/delete user.
#------------------------------------------------------------------------------
sub Create_Mod_Del_User_10mode {
    my (%opt, $cmd, $user_exists);
    %opt = @_;

    #Create user.
    if ($opt{action} =~ /create/) {
        $user_exists =
          SearchUser_10mode($opt{'server'}, $opt{'name'}, $opt{'application'});
        if ($user_exists == 1) {
            logcomment(
                  "Could not add user <$_[1]>. Error: User is already present");
            return 0;
        } else {
            eval {
                $cmd =
                  "security login create -username $opt{'name'} -application $opt{'application'} -authmethod password -role $opt{'role'}";
                $opt{'server'}->cli(cmd => $cmd, noprompt => 1);
                $opt{'server'}
                  ->cli(cmd => $opt{'passwd'}, poke => $opt{'passwd'});
            };
            if ($@) {
                logcomment("Failed to add $opt{'name'}");
                return 0;
            } else {
                logcomment("Succeeded to add $opt{'name'}");
                return 1;
            }
        } ## end else [ if ($user_exists == 1)]
    } ## end if ($opt{action} =~ /create/)

    #Modify user.
    if ($opt{action} =~ /modify/) {
        $user_exists =
          SearchUser_10mode($opt{'server'}, $opt{'name'}, $opt{'application'});
        if ($user_exists == 1) {
            logcomment(
                  "Could not add user <$_[1]>. Error: User is already present");
            return 0;
        } else {
            eval {
                $cmd =
                  "security login create -username $opt{'name'} -application $opt{'application'} -authmethod password -role $opt{'role'}";
                $opt{'server'}->cli(cmd => $cmd);
            };
            if ($@) {
                logcomment("Failed to modify $opt{'name'}");
                return 0;
            } else {
                logcomment("Succeeded to modify $opt{'name'}");
                return 1;
            }
        } ## end else [ if ($user_exists == 1)]
    } ## end if ($opt{action} =~ /modify/)

    #Delete user.
    if ($opt{action} =~ /[d]elete/) {
        logcomment("Delete user:- $opt{'name'}");
        $user_exists =
          SearchUser_10mode($opt{'server'}, $opt{'name'}, $opt{'application'});
        if ($user_exists == 0) {
            return 1;
        }
        $cmd = "security login delete -username $opt{'name'} -application $opt{'application'} -authmethod * ";
        $opt{'server'}->cli(cmd => $cmd);
        $user_exists =
          SearchUser_10mode($opt{'server'}, $opt{'name'}, $opt{'application'});
        if ($user_exists == 1) {
            logcomment("Failed to delete $opt{'name'} having application $opt{'application'}");
            return 0;
        } else {
            logcomment("Succeeded to delete $opt{'name'} having application $opt{'application'}");
            return 1;
        }
    } ## end if ($opt{action} =~ /[d]elete/)

    return 0;
} ## end sub Create_Mod_Del_User_10mode

#----------------------------------------------------------------------------
# FUNCTION:	To Login as a specified user to the 10 mode filer console.
#	INPUT		:	server 	- 	server connection handle
#					name 		-	name of the user
#					passwd	-	password for the user
#	RETURN	:  0 - user login successful.
#					1 - failed to login.
#----------------------------------------------------------------------------
sub UserLogin_10mode {

    my (%opt);
    %opt = @_;
    eval {
        $opt{'server'}->console(cmd => "exit", noprompt => 1);
        $opt{'server'}->console(cmd => $opt{'name'}, poke => $opt{'passwd'});
    };
    if ($@) {
        logcomment("Failed to login to filer console by $opt{'name'}");
        $opt{'server'}->console(cmd => "\r", noprompt => 1);
        $opt{'server'}->console(cmd => $opt{'admin_login'}, poke => $opt{'admin_passwd'});
       # $opt{'server'}->console(cmd => "admin",  noprompt => 1);
#        $opt{'server'}->console(cmd => "netapp1!");
        return 0;
    }
    return 1;
} ## end sub UserLogin_10mode


#----------------------------------------------------------------------------
# FUNCTION	:	To Lock a user account in 10 mode
# INPUT		:	server 	- 	server connection handle
#					name 		-	name of the user
# RETURN  	:	1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub LockUser_10mode {
    my (%opt);
    %opt = @_;
    eval {
        $opt{'server'}
          ->cli(cmd => "security login lock -username  $opt{'name'}");
    };
    if ($@) {
        logcomment("Failed to lock the $opt{'name'}");
        return 0;
    }
    return 1;
} ## end sub LockUser_10mode

#----------------------------------------------------------------------------
# FUNCTION:	To Recover the locked user account in 10 mode
# INPUT		:	server 	- 	server connection handle
#					name 		-	name of the user
# RETURN  :	1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub UnLockUser_10mode {
    my (%opt);
    %opt = @_;
    eval {
        $opt{'server'}
          ->cli(cmd => "security login unlock -username  $opt{'name'}");
    };
    if ($@) {
        logcomment("Failed to unlock the $opt{'name'}");
        return 0;
    }
    return 1;

} ## end sub UnLockUser_10mode

#----------------------------------------------------------------------------
# FUNCTION:	To check if a user account is functional in 10mode.
# INPUT		:	server 			- server connection handle
#					filer				- filer name
#					name 				- name of the user
#					passwd			- password for the user
#					admin_passwd	- admin password
#					RLM_IP   		- RLM/SP ip
#					RLM_Login  		- RLM Login permit (0=No,1=Yes).
# RETURN  : 1 = Pass, 0 = Failure
#----------------------------------------------------------------------------
sub TestUser_10mode {
    my (%opt);
    my $frb="";
    %opt = @_;

    my $cons_login = 0;
    my $rlm_login  = 0;

    # Verify if the user can login to console.
    $cons_login =
      UserLogin_10mode('server' => $opt{'server'},
                 'name'   => $opt{'name'},
                 'passwd' => $opt{'passwd'},
                 'admin_login' => $opt{'admin_login'},
                 'admin_passwd' => $opt{'admin_passwd'},);

    # Verify if the user can login to RLM. While calling this
    # function, specify if the user can login to RLM or not.
    # Depending on $opt{'RLM_Login'}, the below loop is entered.
    if ($opt{'RLM_Login'} == 0) {
        if ($opt{'name'} eq "admin") {
            $frb = connect(
                " $opt{'filer'}#ssh-sp$ssh_conn_count;conntype=ssh-persist;connhost=$opt{'RLM_IP'};username=admin;password=$opt{'admin_passwd'};prompts=Permission"
            );
        } else {
            $frb = connect(
                "$opt{'filer'}#ssh-sp$ssh_conn_count;conntype=ssh-persist;connhost=$opt{'RLM_IP'};username=$opt{'name'};password=$opt{'passwd'};prompts=Permission"
            );
        }
    } elsif ($opt{'RLM_Login'} == 1) {
        if ($opt{'name'} eq "admin") {
            $frb = connect(
                "$opt{'filer'}#ssh-sp$ssh_conn_count;conntype=ssh-persist;connhost=$opt{'RLM_IP'};username=admin;password=$opt{'admin_passwd'}"
            );


            $rlm_login = 1;
        } else {
            $frb = connect(
                "$opt{'filer'}#ssh-sp$ssh_conn_count;conntype=ssh-persist;connhost=$opt{'RLM_IP'};username=$opt{'name'};password=$opt{'passwd'}"
            );
            $rlm_login = 1;
        }
        disconnect($frb);
        sleep 10;
    } ## end elsif ($opt{'RLM_Login'} ...)
    $ssh_conn_count = ($ssh_conn_count + 1);

    if (($rlm_login == 1) && ($cons_login == 1)) {
        logcomment("User $opt{'name'} is active and operational");
        return 1;
    } elsif (($cons_login == 0) && ($rlm_login == 0)) {
        logcomment(
            "After account is disabled, $opt{'name'} is restricted for both Filer and RLM/SP Login"
        );
        return 0;
    } elsif (($cons_login == 0) && ($rlm_login == 1)) {
        logcomment("$opt{'name'} is restricted only for Filer Login");
        return 0;
    } elsif (($cons_login == 1) && ($rlm_login == 0)) {
        logcomment("$opt{'name'} is restricted only for RLM/SP Login");
        return 0;
    }
} ## end sub TestUser_10mode

#----------------------------------------------------------------------------
# FUNCTION:	To check if a user account is functional in 10mode.
# INPUT		:	fc 				- filer connection handle
#					build_path		- absolute build path
#					sp_rlm			- filer is sp / rlm
#					flag				- flag "c" of "f"
#					node_name   	- cluster name of filer
# RETURN  : 1 = Pass, 0 = Failure
#----------------------------------------------------------------------------

sub SP_Update_10mode {

    my (%opt, $prossr, $status, $after, $prompt, $cmd, $file_name);
    %opt =@_;
#    %opt = validate(@_,
#                    {'fc'         => {type => OBJECT},
#                     'build_path' => {type => SCALAR},
#                     'sp_rlm'     => {type => SCALAR},
#                     'flag'       => {type => SCALAR, default => ""},
#                     'node_name'  => {type => SCALAR},},);
    $status = 0;

    if ($opt{sp_rlm} =~ /RLM/i) {
        $prossr    = "rlm";
        $file_name = "RLM_FW.zip";
    }
    if ($opt{sp_rlm} =~ /SP/i) {
        $prossr    = "sp";
        $file_name = "SP_FW.zip";
    }

    $cmd =
        "system image get -node "
      . $opt{node_name}
      . " -package "
      . $opt{build_path}
      . " -replace-package true";

    (undef, undef, $after, undef) =
      say($opt{fc}, $cmd, -swallow, -timeout, 3000);
    if ($after !~ /completed/) {
        logcomment(" Software download of $prossr firmware Failed ");
        return 0;
    }
    $cmd = " run local software install $file_name";
    (undef, undef, $after, undef) =
      say($opt{fc}, $cmd, -swallow, -timeout, 3000);
    if ($after !~ /software: installation of $file_name completed/) {
        logcomment(" Software install of $prossr firmware Failed ");
        return 0;
    }

    logcomment(" Starting $prossr update, takes more than 30 mins ");
    if ($opt{flag} ne "") {
    	  say($opt{fc}, "set -privilege advanced -confirmations off", -swallow);
        #say($opt{fc}, " run local priv set advanced ", -swallow);
        $cmd = " run local $prossr update -$opt{flag} ";
    } else {
        $cmd = " run local $prossr update ";

    }
    say($opt{fc}, $cmd, -prompt, "Do you want to reboot the SP|RLM now.*",
        -swallow, -timeout, 4500);

    say($opt{fc}, "y", -prompt, -swallow, -timeout, 300);

    $cmd = "run local $prossr status";
    for (0 .. 15) {
        my (undef, undef, $out) = say($opt{fc}, $cmd, -swallow);
        if ($out =~ m/Online/) {
            $status = 1;
            last;
        }
        snooze(40);
    }
    if ($status == 0) {
        logcomment("$prossr failed to come online after rlm update");
        return 0;
    } else {
        logcomment("$prossr firmware update successful");
        return 1;
    }
} ## end sub SP_Update_10mode

#------------------------------------------------------------------------------
# FUNCTION: To verify the client is blocked when login to RLM/SP is tried with
# wrong password more number of times
# INPUT: $_[0]=client connection handle
#			$_[1]= User name
#			$_[2]= invalid user password
#			$_[3]= rlm/sp ip
#			$_[4]= rlm/sp type
# RETURN:       1 - on Sucess, 0 - on Failure
#------------------------------------------------------------------------------
sub Lock_Client {
    my ($con, $user, $inv_pwd, $rlm_ip, $RLM_PROMPT) = @_;
    my ($prmpt, $cmd, $result);
    $cmd = "ssh $user" . "@" . "$rlm_ip";
    foreach (0 .. 5) {
        eval {
            eval { (undef, undef, $result, $prmpt) =
              say($con, "$cmd", -prompt, "password:", -timeout, 100);};
              if($@)
              {
               (undef, undef, $result, $prmpt) =
              say($con, "yes", -prompt, "password:", -timeout, 100);
              }
            foreach (0 .. 2) {
                if ($prmpt =~ /password:/) {
                    (undef, undef, $result, $prmpt) =
                      say($con, $inv_pwd, -prompt, "password:", -timeout, 100);
                } else {
                    last;
                }
            }
            snooze 30;
        };
        if ($@) {
            logcomment("Failed to connect to $RLM_PROMPT: $@");

        } else {
            logcomment(  "Successfully refused the connection to "
                       . "$RLM_PROMPT with invalid passwd");

        }
    } ## end foreach (0 .. 5)
} ## end sub Lock_Client

#------------------------------------------------------------------------------
# FUNCTION: To verify the Block/Unblock message from the RLM/SP
# INPUT: $_[0]= filer name
#			$_[1]= User name
#			$_[2]= user password
#			$_[3]= rlm/sp ip
#        $_[4]= rlm/sp type
#			$_[5]= whether IPv4/IPv6 format
#        $_[6]= command to be executed (like event - events newest 10, status - rlm/sp status -d)
#        $_[7]= message to be verified (like Block, Unblock)
#			$_[8]= IP for the first client
#			$_[9]= IP for the second client (optional)
#			$_[10]= IP for the third client (optional)
# RETURN:       1 - on Sucess, 0 - on Failure
#------------------------------------------------------------------------------
sub Block_Unblock_Msg($$$$$$$$$;$$) {

    my ($filer,      $user,       $pwd, $rlm_ip,
        $sp_type,    $ip_format,  $cmd, $block_unblock_msg,
        $client_ip1, $client_ip2, $client_ip3) = @_;
    my $result;
    $ssh_conn_count = ($ssh_conn_count+1);
    # Login to the RLM/SP with "ssh naroot@<RLM/SP_IP>" and
    my $fr = connect(  "$filer#ssh$ssh_conn_count;conntype=ssh-persist;connhost=$rlm_ip;"
                     . "username=$user;password=$pwd");

    if ($cmd eq "events") {
        (undef, undef, $result, undef) = say($fr, "events newest 10");
        if (($client_ip1) && ($client_ip2) && ($client_ip3)) {
            return 0
              if (
                 ($result !~ /\[auth.notice\]: $block_unblock_msg $client_ip1/g)
                 && ($result !~
                     /\[auth.notice\]: $block_unblock_msg $client_ip2/g)
                 && ($result !~
                     /\[auth.notice\]: $block_unblock_msg $client_ip3/g));
        } elsif (($client_ip1) && ($client_ip2)) {
            return 0
              if (
                 ($result !~ /\[auth.notice\]: $block_unblock_msg $client_ip1/g)
                 && ($result !~
                     /\[auth.notice\]: $block_unblock_msg $client_ip2/g));
        } elsif ($client_ip1) {
            return 0
              if (
                 $result !~ /\[auth.notice\]: $block_unblock_msg $client_ip1/g);
        }
    } elsif ($cmd eq "status") {
        (undef, undef, $result, undef) =
          say($fr, $sp_type . " status -d", -timeout, 800);

        # Burt :439356 - The 'Block/Unblock IPv6 client_ip in firewall' wont be
        # displayed in 'sp status -d' for an SP filer.
        if (($sp_type =~ /RLM/i) || (($sp_type =~ /SP/i) && ($ip_format =~ /IPv6/))) {
            if (($client_ip1) && ($client_ip2) && ($client_ip3)) {
                return 0
                  if (
                    ($result !~
                     /root: \- $block_unblock_msg.*? $ip_format $client_ip1 in firewall/g
                    )
                    && ($result !~
                        /root: \- $block_unblock_msg.*? $ip_format $client_ip2 in firewall/g
                       )
                    && ($result !~
                        /root: \- $block_unblock_msg.*? $ip_format $client_ip3 in firewall/g
                       ));
            } elsif (($client_ip1) && ($client_ip2)) {
                return 0
                  if (
                    ($result !~
                     /root: \- $block_unblock_msg.*? $ip_format $client_ip1 in firewall/g
                    )
                    && ($result !~
                        /root: \- $block_unblock_msg.*? $ip_format $client_ip2 in firewall/g
                       ));

            } elsif ($client_ip1) {
                return 0
                  if ($result !~
                    /root: \- $block_unblock_msg.*? $ip_format $client_ip1 in firewall/g
                  );
            }
        }
     } ## end elsif ($cmd eq "status")

    disconnect($fr);
    return 1;
} ## end sub Block_Unblock_Msg


#------------------------------------------------------------------------------
# FUNCTION: To verify the the user can connect to RLM/SP to get events
# INPUT: $_[0]= cliet connection handle
#			$_[1]= User name
#			$_[2]= user password
#			$_[3]= rlm/sp ip
#        $_[4]= rlm/sp prompt
#			$_[5]= filer name
# RETURN:       1 - on Sucess, 0 - on Failure
#------------------------------------------------------------------------------
sub Get_Events {
    my ($cc, $user, $pwd, $rlm_ip, $RLM_PROMPT, $FILER) = @_;
    my $cmd = "ssh $user" . "@" . "$rlm_ip";
    eval {
        eval { say($cc, "$cmd", -prompt, "password:", -timeout, 100); };
        if ($@) {
            say($cc, "yes", -prompt, "password:", -timeout, 100);
        }
        say($cc, $pwd, -noprompt, -prompt, "$RLM_PROMPT $FILER.*>", -timeout,
            100);
        say($cc, "events newest 5",
            -noprompt, -prompt, "$RLM_PROMPT $FILER.*>",
            -timeout, 100);
        say($cc, "exit", -prompt, -timeout, 100);
    };
    if ($@) {
        return 0;
    } else {
        return 1;
    }
} ## end sub Get_Events


1;
