# -*- perl -*-
# $Id: //depot/prod/test/main/lib/platval/service_processor/firmware_update.pm
#       Copyright (c) 2009 Network Appliance, Inc.
#       All Rights reserved
#
## @summary Routines for validating SP/RLM fw update testcases.
##
## @NOTE : SP and RLM is used interchangeably, the library supports both.
##
## @description
## This library contains individual routines for ONTAP, SP and Boot Loader
## for testing of firmware update module for firware upgrade/downgrade of SP from both SP and ONTAP.
##
## Sub-Routines:
## 1.get_path_version 
## This function is for getting the various version and builds(SP fw and ONTAP) for the base or for upgrading.
## If build path is not specified it is populated by using the version and the location of the build to be used.
## 
## 2.load_Testcases_fw_Update
## This loads the test cases to be tested. If LOADER =1 it does not load any ONTAP testcases and if the build locations are not pingable then also it does
## not load testcases. If test hash is empty it returns FATAL error.
##
## 3.get_sp_version
## This returns the booted,primary and the backup SP fw versions as per the 'version' command.
##
## 4.verify_Testcases_fw_Update
## This is for using the testcase generation subroutine 'rlm_get_test_case_execution_order'. We can specify testcases we want ot skip following some
## condition in this routine.
##
## 5.check_sp_version
## This checks the version from SP fw with respect to the testcase expectation.
## 6.SP_Base_Conditions
## This restores the base conditions for SP fw versions before the start of any testcase.
##
## 7.sp_reboot
## This is for SP reboot from ONTAP/SP/LOADER from primary or backup fw as per the testcase requirement.
##
## 8.upgrade_statement
## This gives the SP upgrade statement for both FW and ONTAP side.
##
## 9.Corrupt_verify_Primary_backup
## This functions is for corrupting the primary or backup SP fw. This also checks for restoring the fw after the updation.
##
## 10.check_status_version
## This is for checking the SP fw version with the version string from ONTAP status.
##
## 11.assign_version
## This takes the binary values extracted from the csv files for base primary and backup versions and replaces with actual versions.
##
## 12.Execute_Test_Case
## This is for full execution of a single testcase.It takes all the values for the test case and checks all validation items before and after fw update
## and gives the results by verifying with the expectation.
##
## @dependencies
## rlm_ipv6.pm and asup.pm
##
## @synopsis
##      use firmware_update;
##
## @keywords
## firmware update,upgrade,downgrade,sp,reboot,corrupt,build,image,rlm
##
## @author :  divyab@netapp.com,jkonjeti@netapp.com, ravikira@netapp.com, dl-rlm_ipv6_validation@netapp.com, dl-sp-validation@netapp.com
##
######################################################################################################################################


## The global parameter list for the scripts using this library. 
our (%Testcases_fw_Update,%TestHash_ontap,%TestHash_fw,$BasePathFw,$BasePathOntap,$UpgradePathOntap,$UpgradePathFw,$Base1PathFw,$Base1PathOntap,$Flag_fw,$Flag_Ontap,$RLM_FW_Image,$SP_FW_Image,$RLM_ONTAP_Image,$SP_ONTAP_Image,$LOADER_ONTAP,$SP_ssh,$ACP_SP);

## For negative test cases we need to use SP fw images for RLM and RLM fw images for SP.Paths used are specified below.
$RLM_FW_Image='http://10.57.83.10/engineering/rlse/rlm/4.0/build/targets/RLM_FW.tar.gz';
$RLM_ONTAP_Image='http://10.57.83.10/engineering/rlse/rlm/4.0/build/targets/RLM_FW.zip';
$SP_FW_Image='http://10.57.83.10/engineering/rlse/sp/2.0x7/Build.cfgnetapp/Output/SP_FW.tar.gz';
$SP_ONTAP_Image='http://10.57.83.10/engineering/rlse/sp/2.0x7/Build.cfgnetapp/Output/SP_FW.zip';

## @name get_path_version
## This function takes version and path as specified in parameters and also an optional argument(for base backup version). If only version is specified it
## takes the path from the SITE as given in parameters and returns both ONTAP and SP fw update paths. The optional argument is for backup version. If nothing
## is specified for backup the version is same as upgrade version.


sub acp_present()
{	(undef,undef,$version_sp)=say($SP_ssh,"version",'-prompt',"@prompts_sp",'-swallow','-timeout', 300);
	if($version_sp=~/Booted primary ACP version/)
	{	logcomment("ACP is present on SP\n");
		$ACP_SP=1;
	}	
	else
		{logcomment("ACP is not present on SP\n");}
}
sub get_path_version($$;$)
{
	my $VERSION=shift;
	my $PATH=shift;
	my $optional=shift;
	print"version =$VERSION and path =$PATH and optional=$optional\n";
	if($PATH)
	{
        	if($VERSION)
        	{       $PathFw=$PATH;
                	$PathOntap=$PATH;
          	        if($PATH=~/tar\.gz/)
				{$PathOntap=~s/tar\.gz/zip/;}
                	 else
				{$PathFw=~s/zip/tar\.gz/;}
       		}
        	else
 	       		{logresult('FATAL',"Please specify the VERSION if you want to give the PATH\n");}
	}
	elsif($PATH==0 && $VERSION !=0)
	{

        	$link1=('http://10.57.83.10/engineering/rlse/').($RLM).('/') if($SITE =~/SVL/i);
                $link1=('http://10.57.83.10/engineering/rlse/').($RLM).('/') if($SITE =~/BTC/i);
      	  	$link1=('http://10.98.128.210/engineering/rlse/').($RLM).('/') if($SITE=~/ptc/i);
        	$link1=('http://10.227.64.33/x/eng/rlse/').($RLM).('/') if($SITE=~/rtp/i);
		$link2='/build/targets/RLM_FW.tar.gz'   if($RLM eq 'rlm');
        	$link2='/Build.cfgnetapp/Output/SP_FW.tar.gz' if($RLM eq 'sp');
        	$link3=$link2;
        	$link3=~ s/tar\.gz/zip/;
        	$PathFw=($link1) . ($VERSION) . ($link2);
        	$PathOntap=($link1) . ($VERSION) .($link3);
	}
	elsif($optional)
	{	logcomment("Taking backup fw base values as same as upgrade values as nothing was specified\n");
		$PathFw=$UpgradePathFw;
		$PathOntap=$UpgradePathOntap;
		$VERSION=$UPGRADE_VERSION;
		return ($PathFw,$PathOntap,$VERSION);
	}
	else
	{       logresult('FATAL',"Please specify VERSION");}
	
	$Flag_fw=1 if!(-e $PathFw);

        $Flag_Ontap=1 if!(-e $PathOntap);

	return ($PathFw,$PathOntap);
}

## @name load_Testcases_fw_Update
## This loads the testcases according to path accessiblility of the various versions and LOADER value.
sub load_Testcases_fw_Update()
{	logcomment("Flag_fw value :: $Flag_fw \n Flag_Ontap value :: $Flag_Ontap\n");
	 %Testcases_fw_Update=(%Testcases_fw_Update,%TestHash_fw);	
	if($LOADER==0)
	{
		logcomment("Adding all ONTAP test cases in the testbucket\n");
		%Testcases_fw_Update=(%Testcases_fw_Update,%TestHash_ontap);
		
	}
	elsif($LOADER!=0 ||  $Flag_Ontap!=0)
	{logcomment("Not Adding ONTAP testcases as the link is broken or LOADER =1\n LOADER=$LOADER Flag_Ontap=$Flag_Ontap Flag_fw=$Flag_fw\n");
	}
}

## @name get_sp_version
## This returns the booted from, booted version, primary version and backup version from SP fw 'version' command.
sub get_sp_version()
{	(undef,undef,$version_sp)=say($SP_ssh,"version",'-prompt',"@prompts_sp",'-swallow','-timeout', 300);
	if(($ACP_SP==1) && ($version_sp =~/Booted (.+) firmware version (.+)\r\nBooted (.+) ACP version (.+)\r\nPrimary firmware version (.+)\r\nPrimary ACP version (.+)\r\nBackup firmware version (.+)\r\nBackup ACP version (.+)\r\n/))
		{return ($1,$2,$5,$7);}
	elsif(($ACP_SP==0) && ($version_sp =~/Booted (.+) firmware version (.+)\r\nPrimary firmware version (.+)\r\nBackup firmware version (.+)\r\n/)) 		{return ($1,$2,$3,$4);}
	else
	{logresult('FATAL',"Could not get proper output for version command");}
}

## @name verify_Testcases_fw_Update
## This is for using the testcase generation subroutine 'rlm_get_test_case_execution_order'.
sub verify_Testcases_fw_Update
{ logcomment(" Not omitting any test cases\n");}

## @name check_sp_version
## This takes a set of booted from,booted with ,primary and backup versions for SP fw and checks if it matches with output from 'get_sp_version'.
sub check_sp_version($$$$)
{
	my ($Check_Booted_With,$Check_Current,$Check_Primary,$Check_Backup)=@_;
	my $Check=0;
	($Actual_Booted_With,$Actual_Booted,$Actual_Primary,$Actual_Backup)=get_sp_version();
	$Check=1 if($Check_Backup ne $Actual_Backup);
	$Check=10+$Check if($Check_Primary ne $Actual_Primary);
	$Check=100+$Check if($Check_Current ne $Actual_Booted);
	$Check=1000+$Check if($Check_Booted_With ne $Actual_Booted_With);
	logresult('FATAL'," Result=$Check,'1' in units place means mismatch for base version,'1'in tens place is mismatch for primary version,'1'in hundreds place is mismatch for the booted version and '1' in the thousands place means mismatch for booted with backup/primary") if($Check !=0);
	logcomment("Both the versions look the same\n");
}

## @name SP_Base_Conditions
## This restores the base condition for every testcase.The base primary and backup version is same for every testcase but it may be booted with primary
## or backup which is decided according to the input variable Booted_With.
sub SP_Base_Conditions($$)
{
	my ($Booted_With)=shift @_;
	my ($Upgrade_from)= shift @_;
	$Current_Base_Version = $BASE_VERSION if($Booted_With eq 'primary');
	$Current_Base_Version = $BASE_VERSION_1 if($Booted_With eq 'backup');

	(undef,undef,$Actual_Primary,$Actual_Backup)=get_sp_version();
 	if($Actual_Backup ne $BASE_VERSION_1)
	{	if($Actual_Primary ne $BASE_VERSION_1){
	say($SP_ssh,"$RLM update $Base1PathFw -f -p",'-prompt',"@prompts_sp",'-swallow','-timeout', 300);
			(undef,undef,$Actual_Primary,undef)=get_sp_version();
			logresult('FATAL',"Could not put the required version in the primary:Actual_Primary=$Actual_Primary,BASE_VERSION_1=$BASE_VERSION_1") if($Actual_Primary ne $BASE_VERSION_1);
		}
		say($SP_ssh,"$RLM update $Base1PathFw -f -b",'-prompt',"@prompts_sp",'-swallow','-timeout', 300);
		sp_reboot(1);
		(undef,undef,undef,$Actual_Backup)=get_sp_version();
		logresult('FATAL',"Could not put required version in the backup") if($Actual_Backup ne $BASE_VERSION_1);
	}
	if($Actual_Primary ne $BASE_VERSION)
	{
		say($SP_ssh,"$RLM update $BasePathFw -f -p",'-prompt',"@prompts_sp",'-swallow','-timeout', 300);
	}
	($Actual_Booted_With,$Actual_Booted,$Actual_Primary,undef)=get_sp_version();
	logresult('FATAL',"Could not put the required version in the primary\n") if($Actual_Primary ne $BASE_VERSION);
	if($Booted_With ne $Actual_Booted_With || $Actual_Booted ne $Current_Base_Version) 
		 {	logcomment("Booted_With=$Booted_With Actual_Booted_With=$Actual_Booted_With Actual_Booted=$Actual_Booted Current_Base_Version=$Current_Base_Version");
			sp_reboot("1",$Booted_With);}
	check_sp_version($Booted_With,$Current_Base_Version,$BASE_VERSION,$BASE_VERSION_1);
	logcomment("Successfully restored the base conditions\n");
}

## @name sp_reboot
## This is a generic subroutine for sp reboot.It takes $Upgrade_from as compulsory argument which decides if reboot is from sp,loader or ONTAP.Second
##argument specifies if we want to boot with backup or primary.If it is not specifed only 'sp reboot' is executed.
sub sp_reboot($;$$)
{
	my $Upgrade_from = shift @_;
	my $Booted_With=shift @_;	
	my $SP_User=shift @_;
#	$Booted_With="\L$Booted_With\E";
	logcomment("Upgrade_from=$Upgrade_from ,Booted with=$Booted_With\n");
	if($Upgrade_from==0)
	{	disconnect($SP_ssh);
		if (defined $Booted_With)
			{say($fc,"$RLM reboot $Booted_With",'-prompt', 'y\/n', '-timeout', 30);}
		else
			{say($fc,"$RLM reboot",'-prompt', 'y\/n', '-timeout', 30);}	
	 say($fc, "y", '-blind', '-timeout', 300);
                sleep(160);
        }
	if($Upgrade_from==1)
	{	if (defined $Booted_With)
		{       disconnect($SP_ssh);
                say($fc,"priv set test",-swallow,-timeout,100);
                if($LOADER ==1)
		{say($fc,"$RLM reset $Booted_With",'-prompt','Continue? [y/n]','-timeout', 30);}
		else
		{say($fc,"$RLM reboot $Booted_With",'-prompt','y\/n', '-timeout', 30);}
                say($fc, "y",'-blind', '-timeout', 300);
	        }
	
		else	
		{       
			say($SP_ssh,"$RLM reboot",'-noprompt','-prompt','Continue\? \[y\/n\]','-timeout', 30);
        	        say($SP_ssh, "y", '-blind', '-timeout', 120);
        	}
	
                sleep(160);
        }

	my ($timer,$status) = (0,0);
    	$fc->discardbuff;
	while($timer <= 50)
	{	my(undef, undef, $out) = say($fc, "$RLM status", '-prompt', @prompts, '-timeout', 300);
         	if($out =~ /Online/)
		{	$status = 1;
            		$timer = 51;
          	} 
		elsif ($Upgrade_from && $out =~ /Link Local Address|IPv6\:\s+(?:Disabled|unknown)/) 
		{
            		$status = 1;
            		$timer = 51;
		}
		 else 
			{sleep(15);}
          	$timer = $timer + 1;
	}
 	unless ($status) 
		{logresult ('FATAL', "$RLM_PROMPT is not getting back to online state");}
		
	logcomment("sleeping for 60 seconds after sp reboot\n");
	sleep(60);
	disconnect($SP_ssh);
	if ($SP_User)
	{	
		$SP_ssh=rlm_make_conn_to_rlm ($FILER, $ipv4_ip,$SP_User , $PASSWD);
	} else {
		$SP_ssh=rlm_make_conn_to_rlm ($FILER, $ipv4_ip, 'naroot', $PASSWD);
	}
}
## @name upgrade_statement
## This subroutine is for executing the sp update command with the proper paths and options.$Upgrade_From decides if we want ot upgrade from SP or ONTAP.
##$Upgrade_Option can be 1-5 for SP and 1-3 for ONTAP.$Upgrade decides if we want to use the regular image or do negative testing or reboot during upgrade.
##$PASS checks the console messages with expected output. 
sub upgrade_statement($$$$)
{	my($Upgrade_From,$Upgrade_Option,$Upgrade,$PASS)=@_;
	my $Prompt;
	logcomment("Upgrade_From=$Upgrade_From,Upgrade_Option=$Upgrade_Option,Upgrade=$Upgrade\n");
	if($Upgrade_From==1)
	{	if($Upgrade != -1)
		{$UseImage=$UpgradePathFw;}
		elsif($RLM=~/sp/i)
		{$UseImage=$RLM_FW_Image;}
		elsif($RLM=~/rlm/i)
		{$UseImage=$SP_FW_Image;}
		if($Upgrade_Option==1)
			{(undef,undef,$Output)=say($SP_ssh,"$RLM update $UseImage",-prompt,@prompts,-timeout,300);}
		elsif($Upgrade_Option==2)
			{(undef,undef,$Output)=say($SP_ssh,"$RLM update $UseImage -f",-prompt,@prompts,-timeout,300);}
		elsif($Upgrade_Option==3)
			{(undef,undef,$Output)=say($SP_ssh,"$RLM update $UseImage -c",-prompt,@prompts,-timeout,300);}
		elsif($Upgrade_Option==4)
			{(undef,undef,$Output)=say($SP_ssh,"$RLM update $UseImage -f -p",-prompt,@prompts,-timeout,300);}
		elsif($Upgrade_Option==5)
		 	{(undef,undef,$Output)=say($SP_ssh,"$RLM update $UseImage -f -b",-prompt,@prompts,-timeout,300);}
		if($PASS == 1)
			{logresult('FATAL',"Please check update process") unless($Output=~/Installing package/&& $Output=~/Reprocessing configuration state/ && $Output=~/$RLM FW Update Successful/i && $Output=~/Use "$RLM reboot" command to reboot into the new firmware/i);}
		elsif($PASS == 0)
#			{logresult('FATAL',"Please check update process,updating wrong image properly") unless($Output=~/FW package install failed,version doesn't support existing flash type....FAILED/ && $Output=~/ERROR : $RLM_PROMPT FW Update Unsuccessful/);}
                       {logresult('FATAL',"Please check update process,updating wrong image properly") unless($Output=~/FW package install failed, the new version is not compatible with this platform.....FAILED/ && $Output=~/ERROR : $RLM_PROMPT FW Update Unsuccessful/);}
	}
	elsif($Upgrade_From==0)
	{ 	if($Upgrade != -1)
                {$UseImage=$UpgradePathOntap;}
		elsif($RLM=~/sp/i)
                {$UseImage=$RLM_ONTAP_Image;}
                elsif($RLM=~/rlm/i)
                {$UseImage=$SP_ONTAP_Image;}
		(undef,undef,$Output)=say($fc,"software install $UseImage -f",-prompt,@prompts,-timeout,300);
		if($PASS ==1 && $Upgrade != -1)
		{logresult('FATAL',"Software installation did not happen properly") unless($Output=~/Please type \'$RLM update\' to load the new SP firmware/i);}
		if($PASS ==0 && $Upgrade == -1)
		{logresult('FATAL',"Software installation did not fail on wrong update") unless($Output=~/software: installation of (.+)was not successful/);}
		undef $Output;
		@prompts1=("Do you want to reboot the $RLM_PROMPT now","$RLM_PROMPT firmware update failed");
		if($Upgrade !=2 && $Upgrade !=3)
		{	if($Upgrade_Option==1)
				{(undef,undef,$Output,$Prompt)=say($fc,"$RLM update",-prompt,@prompts1,-swallow,-timeout,1620);
			}
			elsif($Upgrade_Option==2)
				{(undef,undef,$Output,$Prompt)=say($fc,"$RLM update -f",-prompt,@prompts1,-swallow,-timeout,2000);}
			elsif($Upgrade_Option==3)
				{(undef,undef,$Output,$Prompt)=say($fc,"$RLM update -c",-prompt,@prompts1,-swallow,-timeout,1620);}		
			logcomment("Output=$Output and Prompt=$Prompt\n");
			if($Prompt=~/Do you want to reboot the $RLM_PROMPT now/ && $Upgrade != -1)
				{say($fc,"n",-prompt,@prompts,-timeout,300);}
			elsif($Output=~/$RLM_PROMPT version is up-to-date/)
			{logcomment("Version is up to date\n");}
			#elsif($Upgrade == -1)
			#{}
			elsif($PASS==0)
			{logcomment("This is supposed to fail\n");}
			else
			{logresult('FATAL',"Unexpected output on sp update from ONTAP");}
		}
		if($Upgrade ==2 || $Upgrade ==3)
		{	say($fc,"bootargs set AUTOBOOT TRUE",'-swallow','-timeout',30);
			if($Upgrade_Option==1)
			{(undef,undef,$Output,$Prompt)=say($fc,"$RLM update",-blind,-timeout,900);}
			elsif($Upgrade_Option==2)
			 {(undef,undef,$Output)=say($fc,"$RLM update -f",-blind,-timeout,900);}
			elsif($Upgrade_Option==3)
			{(undef,undef,$Output)=say($fc,"$RLM update -c",-blind,-timeout,900);}
			sleep(200);
			if($Upgrade ==2)
			{	say($SP_ssh,"system power cycle",'-noprompt','-prompt','This will cause a dirty shutdown of your appliance.  Continue\? \[y\/n\]',-swallow,-timeout,100);
				say($SP_ssh,"y",'-noprompt','-prompt',@prompts_sp,-swallow,-timeout,100);
				sleep(60);
				for($i=0;$i<10;$i++)
				{	eval{(undef,undef,undef,$p)=say($fc,'',-swallow,-timeout,30);};
					unless($@)
					{if ($p =~ /login|$FILER/i)
					{	rlm_verify_filer_state($FILER,$LOGIN,$PASSWD);
						last;
					}}
					sleep(30);
				}
					
			}
			if($Upgrade ==3)
			{	sp_reboot(1);}
		}
	}
	
}


## @name Corrupt_verify_Primary_backup
## This is for corrupting primary or backup image. if $Corrupt is 0 menas nothing should be corrupted, 1 means primary corrupt,-1 means backup corrupt and 
## for 2 this function checks for a non corrupt state.
sub Corrupt_verify_Primary_backup($)
{	my $Corrupt=shift @_;
	my $Corrupt_var=0;
	if($Corrupt==1)
	{$Corrupt_var="PRIMARY";}
	elsif($Corrupt==-1)
	{$Corrupt_var="BACKUP";}
	logcomment("Corrupting $Corrupt_var ...\n");
	say($fc,"priv set test",-swallow,-timeout,100);
	say($fc,"sp debugmode",-swallow,-timeout,100);
	say($SP_ssh,"priv set test",'-noprompt','-prompt',@prompts_sp,-swallow,-timeout,100);
	say($SP_ssh,"$RLM shell",-swallow,'-noprompt','-prompt','\$',-timeout,100);
	say($SP_ssh,"fw_setenv INSTALLING_BACKUP 0",-swallow,'-noprompt','-prompt','\$',-timeout,100) if($Corrupt != -1);
	say($SP_ssh,"fw_setenv INSTALLING_PRIMARY 0",-swallow,'-noprompt','-prompt','\$',-timeout,100) if($Corrupt != 1);
	say($SP_ssh,"fw_setenv INSTALLING_$Corrupt_var 1",-swallow,'-noprompt','-prompt','\$',-timeout,100) if($Corrupt_var);
	my(undef,undef,$Output,undef)=say($SP_ssh,"fw_printenv",'-noprompt','-prompt','\$',-swallow,-timeout,100);
	if($Corrupt==0 || $Corrupt==2)
	{	unless($Output=~/INSTALLING_BACKUP\=0/ && $Output=~/INSTALLING_PRIMARY\=0/)
			{logresult('FATAL',"The backup and primary variables could not be put to 0: $Output");}
	}
	if($Corrupt==1)
	{	unless($Output=~/INSTALLING_BACKUP=0/ && $Output=~/INSTALLING_PRIMARY=1/)
			{logresult('FATAL',"The primary and backup corruption variables could not be properly set :$Output");}
	}
	if($Corrupt==-1)
	{	unless($Output=~/INSTALLING_BACKUP=1/ && $Output=~/INSTALLING_PRIMARY=0/)
		{logresult('FATAL',"The primary and backup corruption variables could not be properly set :$Output");}
	}
	say($SP_ssh,"exit",,-swallow,'-noprompt','-prompt',@prompts_sp,-timeout,100);
}

## @name check_status_version
## This is for checking the SP fw version with the version string from ONTAP status.
sub check_status_version($$)
{
	$version_sp=shift @_;
	$status_version_sp=shift @_;
	if($version_sp ne $status_version_sp)
	{logresult('FATAL',"Version command and status shows different sp version\n");}
	logcomment("Both the version are same\n");
}

## @name assign_version
## This takes the binary values extracted from the csv files for base primary and backup versions and replaces with actual versions.
sub assign_version($)
{
	my $Binary_Version=shift @_;
	if($Binary_Version==1)
	{$Binary_Version=$UPGRADE_VERSION;}
	elsif($Binary_Version==0)
	{$Binary_Version=$BASE_VERSION;}
	elsif($Binary_Version ==-1)
	{$Binary_Version=$BASE_VERSION_1;}
	else
	{logresult('FATAL',"Unknown version");}
	return $Binary_Version;
}

## @name Execute_Test_Case
## This is the actual SP fw upgradation test case execution subroutine. It takes the testcase number as argument and gets all values from hash table.
## It restores the base condition for SP and corrupts any image if needed. then it checks for validation items like date,SEL logs for fw update and SP RESET,
## version from sp status. After performing sp upgrade it checks all versions again and reboots SP. All validation items are again checked.
sub Execute_Test_Case($)
{
	my $Key=$_[0];	#@_
	my ($TestCase_Name,$Booted_With,$Upgrade,$Upgrade_from,$Upgrade_Option,$Corrupt,$Current_Version_AU,$Primary_Version_AU,$Backup_Version_AU,$Current_Version_AR,$Primary_Version_AR,$Backup_Version_AR,$PASS)=@{$Testcases_fw_Update{$Key}};
	logcomment("TestCase ::key=$Key: @{$Testcases_fw_Update{$Key}} \n");
	#logcomment("Current_Version_AU=$Current_Version_AU Primary_Version_AU=$Primary_Version_AU Backup_Version_AU=$Backup_Version_AU\n");
	if($Current_Version_AR ne $Primary_Version_AR)
	{$check_booted="backup";}
	else
	{$check_booted="primary";}

	$Current_Version_AU=assign_version($Current_Version_AU);
	$Primary_Version_AU=assign_version($Primary_Version_AU);
	$Backup_Version_AU=assign_version($Backup_Version_AU);
	$Current_Version_AR=assign_version($Current_Version_AR);
	$Primary_Version_AR=assign_version($Primary_Version_AR);
	$Backup_Version_AR=assign_version($Backup_Version_AR);
	logcomment("Expected values of the versions after update before reboot are given below\nBooted_Version_After_Update=$Current_Version_AU Primary_Version_After_Update=$Primary_Version_AU Backup_Version_After_Update=$Backup_Version_AU\n");
	logcomment("Expected values of the versions after update and reboot are given below\nBooted_Version_After_Reboot=$Current_Version_AR Primary_Version_After_Reboot=$Primary_Version_AR Backup_Version_After_Reboot=$Backup_Version_AR\n");
	Corrupt_verify_Primary_backup(0);
	if($Corrupt == 0)
		{SP_Base_Conditions($Booted_With,$Upgrade_from);}
	else
		{SP_Base_Conditions("primary",$Upgrade_from);}
	rlm_verify_rlm_status($FILER,$LOADER_ONTAP, $RLM_STATE);
	(undef,$Version_sp,undef,undef)=get_sp_version();	
	check_status_version($Version_sp,$fw_version);
	if($Corrupt)
	{	Corrupt_verify_Primary_backup($Corrupt);
		sp_reboot(1);
		sleep(60);
		rlm_verify_rlm_status($FILER,$LOADER_ONTAP, $RLM_STATE);
	        (undef,$Version_sp,undef,undef)=get_sp_version();
        	check_status_version($Version_sp,$fw_version);
		if($Corrupt==1)
		{check_sp_version("backup",$BASE_VERSION_1,$BASE_VERSION,$BASE_VERSION_1);}
		elsif($Corrupt==-1)
		{check_sp_version("primary",$BASE_VERSION,$BASE_VERSION,$BASE_VERSION_1);}
	}
	@EMS=events_search("install");	
	($EMS_Install_before,undef)=EMS_specific("Firmware install successful",@EMS);
	undef @EMS;
	@EMS=events_search("Reset");
	($EMS_reboot_before,undef)=EMS_specific("$RLM_PROMPT Reset",@EMS);
	(undef,undef,$dateSP) = say ($SP_ssh,"date",'-swallow', -noprompt, -prompt, @prompts_sp, '-timeout', 300);
	logcomment("Date before updating is :: $dateSP\n");
	upgrade_statement($Upgrade_from,$Upgrade_Option,$Upgrade,$PASS);
	check_sp_version($Booted_With,$Current_Version_AU,$Primary_Version_AU,$Backup_Version_AU);
	undef @EMS;
	@EMS=events_search("install");
	($EMS_Install_After2,$EMS_Install_After1)=EMS_specific("Firmware install successful",@EMS);		
	logcomment("$EMS_Install_before,$EMS_Install_After1,$EMS_Install_After2");
	if($PASS ==1)
	{Check_Result($EMS_Install_before,$EMS_Install_After1,$EMS_Install_After2);}
	else
	{Check_Result($EMS_Install_before,$EMS_Install_After1,$EMS_Install_After2,1);}
	if($Upgrade==3)
	{	undef @EMS;
       		@EMS=events_search("Reset");
        	($EMS_reboot_after2,$EMS_reboot_after1)=EMS_specific("$RLM_PROMPT Reset",@EMS);
		Check_Result($EMS_reboot_before,$EMS_reboot_after1,$EMS_reboot_after2);
		$EMS_reboot_before=$EMS_reboot_after2;
	}
	if($Booted_With eq "backup")
	{sp_reboot($Upgrade_from,"primary");}
	else
	{sp_reboot($Upgrade_from);}
	check_sp_version($check_booted,$Current_Version_AR,$Primary_Version_AR,$Backup_Version_AR);
	undef @EMS;
	@EMS=events_search("Reset");
	($EMS_reboot_after2,$EMS_reboot_after1)=EMS_specific("$RLM_PROMPT Reset",@EMS);
	Check_Result($EMS_reboot_before,$EMS_reboot_after1,$EMS_reboot_after2);
	rlm_verify_rlm_status($FILER,$LOADER_ONTAP, $RLM_STATE);
	check_status_version($Current_Version_AR,$fw_version);
	(undef,undef,$dateSP) = say ($SP_ssh,"date",'-swallow', -noprompt, -prompt, @prompts_sp, '-timeout', 300);
        logcomment("Date after updating is :: $dateSP\n");
        Corrupt_verify_Primary_backup(2);
	logresult('PASS',"The testcase :$Key has passed");
}

1;
