# -*- perl -*-
# $id: //depot/prod/test/main/lib/platval/service_processor/spcli.pm
#
## Copyright (c) 2009 Network Appliance, Inc.
## All Rights reserved
##
## @summary Sub-routines for system forensics commands and  testcases.
#3 @description
## This module contains all the necessary functions for executing the system forensics test cases
## 
##
## @author dhivakar@netapp.com
##############################AA########################################################################################

require "tharnit_lib.pl";
our ($FILER,$fc,$PROMPT_RLM,$rlm_ssh_ip,$CLIENT,$CLIENT_CONN,$SP_ssh,%config_data, %ASUP_config_testcases,%SP_Config_testcases,$NO_IPv6,%functionality_testcases,@valid_cases,$Pass_Flag,$Global_Config,$Global_Setup,$Global_Func);



###########################################################################
# Function Name : forensicsEnable
# Arguments     : No Arguments
# Return Value  : PASS or FAIL
# Description   : This subroutine enables system forensics and verifies
#                 whether it is enabled
###########################################################################

sub forensicsEnable {
   my $commandOutput;
   tstlog -comment, "Enabling system forensics";
   (undef,undef,$commandOutput,undef) = say($SP_ssh,"priv set test",'-swallow','-timeout',30);
   (undef,undef,$commandOutput,undef) = say($SP_ssh,"system forensics enable",'-swallow','-timeout',30);
   (undef,undef,$commandOutput,undef) = say($SP_ssh,"system forensics show",'-swallow','-timeout',30);
   if ($commandOutput =~ /Forensics enabled: true/) {
      tstlog -comment, "System Forensics enabled successfully";
   } else {
      tstlog -FATAL, "System Forensics could not be enabled";
   }
}


###########################################################################
# Function Name : forensicsDisable
# Arguments     : No Arguments
# Return Value  : PASS or FAIL
# Description   : This subroutine disables system forensics and verifies
#                 whether it is disabled
###########################################################################

sub forensicsDisable {
   my $commandOutput;
   tstlog -comment, "Disabling system forensics";
   (undef,undef,$commandOutput,undef) = say($SP_ssh,"priv set test",'-swallow','-timeout',30);
   (undef,undef,$commandOutput,undef) = say($SP_ssh,"system forensics disable",'-swallow','-timeout',30);
   (undef,undef,$commandOutput,undef) = say($SP_ssh,"system forensics show",'-swallow','-timeout',30);
   if ($commandOutput =~ /Forensics enabled: false/) {
      tstlog -comment, "System Forensics disabled successfully";
   } else {
      tstlog -FATAL, "System Forensics could not be disabled";
   }
}


###########################################################################
# Function Name : forensicsSetTime
# Arguments     : time to be set
# Return Value  : PASS or FAIL
# Description   : This subroutine sets the specified time-limit
###########################################################################

sub forensicsSetTime($) {
   my $commandOutput;
   tstlog -comment, "Setting the specified time limit $_[0] seconds";
   (undef,undef,$commandOutput,undef) = say($SP_ssh,"priv set test",'-swallow','-timeout',30);
   (undef,undef,$commandOutput,undef) = say($SP_ssh,"system forensics set-time-limit $_[0]",'-swallow','-timeout',30);
   (undef,undef,$commandOutput,undef) = say($SP_ssh,"system forensics show",'-swallow','-timeout',30);
   if ($commandOutput =~ /Forensics time limit: $_[0]/) {
      tstlog -comment, "The specified time limit is set successfully";
   } else {
      tstlog -FATAL, "The specified time limit could not be set";
   }
}


###########################################################################
# Function Name : forensicsLogClear
# Arguments     : Conn to SP root
# Return Value  : PASS or FAIL
# Description   : This subroutine clears the forensics logs
###########################################################################

sub forensicsLogClear($) {
   my ($root_ssh) = @_;
   my $commandOutput;
   tstlog -comment, "Clearing the forensics logs";
  (undef,undef,$commandOutput,undef) = say($SP_ssh,"system forensics log clear",'-prompt',@prompts_sp,'-swallow','-timeout',30);
  (undef,undef,$commandOutput,undef) = say($root_ssh,"ls /mnt/logs/forensics",'-prompt',@prompts_sp,'-swallow','-timeout',30);
   my $logDump = forensicsLogDump();
   if (($logDump =~ /No forensics logs available/) && ($commandOutput !~ /forensics_logs/) && ($commandOutput !~ /.tar/)) {
	  tstlog -comment, "forensics log clear command executed successfully";
   } else {
      tstlog -FATAL, "forensics log clear command failed";
   }
}


###########################################################################
# Function Name : forensicsLogDump
# Arguments     : No Arguments
# Return Value  : PASS or FAIL
# Description   : This subroutine dumps the most recent forensics output
###########################################################################

sub forensicsLogDump {
   my $commandOutput;
   tstlog -comment, "Dumping the most recent forensics output";
   (undef,$a,$commandOutput,undef) = say($SP_ssh,"system forensics log dump","system forensics log dump",'-prompt',@prompts_sp,'-swallow','-timeout',30);
   return $commandOutput;
}


###########################################################################
# Function Name : forensicsConfigUpdate
# Arguments     : Conn to SP root,PATH to xml config file
# Return Value  : PASS or FAIL
# Description   : This subroutine updates the config file
###########################################################################

sub forensicsConfigUpdate($$) {
   my ($root_ssh,$CONFIG_FILE) = @_;
   my ($commandOutput,$output);
   tstlog -comment, "Updating the forensics input config xml file";
  (undef,undef,$commandOutput,undef) = say($SP_ssh,"system forensics config update $CONFIG_FILE",'-prompt',@prompts_sp,'-swallow','-timeout',30);
  if ($commandOutput =~ /forensics.xml        100%/) {
     tstlog -PASS, "Updated the xml config file";
  } else {
     tstlog -FATAL, "system forensics config update command failed";
  }
  tstlog -comment, "Checking the directory to see if the config file is actually copied";
  (undef,undef,$output,undef) =  say($root_ssh,"ls \/mnt\/logs\/etc\/",'-swallow','-timeout',30);
  if ($output =~ /forensics.xml/) {
     tstlog -PASS, "The xml config file is updated successfully";
  } else {
     tstlog -FAIL, "system forensics config update command failed";
  }
}
###########################################################################
# Function Name : forensicsConfigDump
# Arguments     : Conn to SP root
# Return Value  : PASS or FAIL
# Description   : This subroutine dumps the config file.If there is a user
#                 updated config file, that will be dumped otherwise the 
#                 default config file will be dumped.
###########################################################################

sub forensicsConfigDump($) {
   my ($root_ssh) = @_;
   my ($commandOutput,$output,$match);
   tstlog -comment, "Dumping the config files";
  (undef,undef,$commandOutput,undef) = say($SP_ssh,"system forensics config dump",'-prompt',@prompts_sp,'-swallow','-timeout',30);
   #print " The command output is \n $commandOutput \n test";
  (undef,undef,$output,undef) =  say($root_ssh,"ls \/mnt\/logs\/etc\/",'-swallow','-timeout',30);
  if ($output =~ /forensics.xml/) {
     (undef,$match,$output,undef) =  say($root_ssh,"cat \/mnt\/logs\/etc\/forensics.xml","cat \/mnt\/logs\/etc\/forensics.xml",'-swallow','-timeout',30);
     #print "The output is \n $output \n test";
     if ($commandOutput =~ /\Q$output\E/) {
        tstlog -PASS, "The updated config file is dumped successfully";
     } else {
        tstlog -FATAL, "The updated config file is not dumped correctly";
     }
  } else {
     (undef,$match,$output,undef) =  say($root_ssh,"cat \/etc\/forensics\/forensics.xml","cat \/etc\/forensics\/forensics.xml",'-swallow','-timeout',30);
     if ($commandOutput =~ /\Q$output\E/) {
        tstlog -PASS, "The default config file is dumped successfully";
     } else {
        tstlog -FATAL, "The default config file is not dumped correctly";
     }
  }
}
###########################################################################
# Function Name : forensicsConfigClear
# Arguments     : Conn to SP root
# Return Value  : PASS or FAIL
# Description   : This subroutine clears the updated config file
###########################################################################

sub forensicsConfigClear($) {
   my ($root_ssh) = @_;
   my ($commandOutput,$output);
   tstlog -comment, "Clearing the config files";
  (undef,undef,$commandOutput,undef) = say($SP_ssh,"system forensics config clear",'-prompt',@prompts_sp,'-swallow','-timeout',30);
  (undef,undef,$output,undef) =  say($root_ssh,"ls \/mnt\/logs\/etc\/",'-swallow','-timeout',30);
  if ($output =~ /forensics.xml/) {
     tstlog -FATAL, "Not able to delete the user uploaded input forensics config file";
  } else {
     tstlog -PASS, "User uploaded input forensics config file is successfully deleted";
  }
}


###########################################################################
# Function Name : triggerL2Timeout
# Arguments     : Filer connection handle
# Description   : This subroutine triggers L2 timeout by ignoring NMI and 
#                 hanging the controller
###########################################################################

sub triggerL2Timeout($) {
   my ($conn) = @_;
   tstlog -comment, "Triggering L2 timeout by ignoring NMI and hanging controller";
   (undef,undef,$commandOutput,undef) = say($conn,"sudo sysctl debug.watchdog.disable=0",'-swallow','-timeout',30);
   (undef,undef,$commandOutput,undef) = say($conn,"sudo sysctl debug.watchdog.ignore_nmi=1",'-swallow','-timeout',30);
   (undef,undef,$commandOutput,undef) = say($conn,"sudo sysctl debug.watchdog.trigger_l2_timeout=1",'-swallow','-timeout',30);
   (undef,undef,$commandOutput,undef) = say($conn,"sudo sysctl debug.watchdog.hang_cpu=1",'-swallow','-timeout',30);
}


###########################################################################
# Function Name : checkOutputXMLs
# Arguments     : connection handle to SP root shell
# Description   : This subroutine checks whether the number of output files
#                 created are same as that of input xml files and also 
#                 whether a timestamp file is created
###########################################################################

sub checkOutputXMLs($) {
   my ($root_ssh) = @_;
   my ($inputFiles,$outputFiles,@inputFiles,@outputFiles,$line,@line,@line1);
   my $finalInputString = "final";
   my $finalOutputString = "final";
   tstlog -comment, "Listing the input xml files";
   say($root_ssh,"cd \/etc\/forensics\/",'-swallow','-timeout',30);
   (undef,undef,$inputFiles,undef)= say($root_ssh,"ls *.xml",'-swallow','-timeout',30);  
   tstlog -comment, "Listing the output xml files";
   say($root_ssh,"cd \/mnt\/logs\/forensics\/forensics_logs\/",'-swallow','-timeout',30);
   (undef,undef,$outputFiles,undef)= say($root_ssh,"ls",'-swallow','-timeout',30);
   if ($outputFiles !~ /timestamp/) {
      tstlog -FAIL, "timestamp file is not present in the output xml file directory";
   }
   (undef,undef,$outputFiles,undef)= say($root_ssh,"ls *.xml",'-swallow','-timeout',30);
   if ($inputFiles eq $outputFiles) {
      tstlog -PASS, "The output xml file corresponding to the input xml file is successfully created along with a timestamp file";
   } else {
      tstlog -FAIL, "The output xml file is not created properly";
   }

}


###########################################################################
# Function Name : checkLogRotation
# Arguments     : iteration value,total iterations
# Description   : This subroutine checks whether proper forensics log files
#                 are created and the log .tar files are rotated and only the
#                 three most recent files are maintained.
###########################################################################

sub checkLogRotation($$$) {
   my ($iterationValue,$totalIterations,$root_ssh) = @_;
}


1;
