# -*- perl -*-
# $Id:
#
#	Copyright (c) 2010 NetApp, Inc.
#	All Rights reserved
##
## @summary
## This file provides common functions for testing Hardware Assist in 10 mode.
##
## @description
## hw_assist_10mode.pm is a common module for to enable and test Hardware Assist
##
## Sub-Routines:
##
##  1. load_ipv4_ipv6_HW_ASSIST_config_cases
##  -  Function to load the HW_ASSIST_config_cases
##
##  2. check_cluster_setup
##  -  To verify the filers are in cluster setup.
##
##  3. check_ontap_rlm_config_status
##  -  To verify ontap and rlm/sp configuration details.
##
##  4. set_cf_hwassist_partner_ip
##  -  To set the hwassist partner ip
##
##  5. set_hwassist_enable
##  -  To enable/disable the hwassist
##
##  6. verify_hwassist_test
##  -  To verfiy the hwassist feature
##
##  7. run_hwassist_test
##  -  To run the hwassist test
##
##  8. get_hwassist_details
##  -  To get the hwassist details
##
##  9. cf_hwassist_stats
##  -  To get the hwassist stats
##
##  10. cf_hwassist_partner_details
##   -  To get the hwassist partner details
##
##  11. cf_hwassist_events
##   -  To get hwassisit events
##
##  12. cf_hwassist_statistics
##   -  To get the hwassist statistics from RLM/SP
##
##  13. test_hwassist_get_partner_details
##   -  To compare the hwasssist partner details
##
##  14. send_cmd_to_filer
##  -  Function to send filer for generating hwassist
##
##  15. get_info_about_partner
##  -  To the information about the partner state  after hwassist
##
##  16. test_hwassist_stats_check
##   -  To verify the hwassist stats before and after hw_assist
##
##  17. test_hwassist_statistics_check
##   -  To verify the hwassist statistics before and after hw_assist
##
##  18. test_hwassist_events_check
##   -  To verify the hwassist events before and after hw_assist
##
##  19. doFilersUp
##   -  To bring up the filers
##
##  20. check_ontap_config_status
##   -  Verify the ONTAP configuration status
##
##  21. hwassist_test_results
##   -  Verify the hwassit test results
##
##  22. cf_status_precheck_10mode
##   -  Function to check the status of the filer and cf status in 10mode
##
##  23. get_cf_state_10mode
##   -  Function to get the cf status in 10mode
##
##  24. cf_enable_10mode
##   -  Function to enable the cf status in 10mode
##
##  25. cf_giveback_10mode
##   -  Function to giveback the filer in 10mode
##
##
## @synopsis
##		use hw_assist_10mode;
##
##	@status	review
##
## @author  mariat@netapp.com, rbasani@netapp.com
##

use platval::service_processor::rlm_ipv6_10mode;
use Server;
use boot;
use platval::service_processor::asup10mode;

#-------------------------------------------------------------------------------
# FUNCTION	: Function to load the HW_ASSIST_config_cases.
# INPUT		: NONE
# RETURN		: NONE

# Description of each elements of the %hw_test_cases hash  are below:
# First  Argument => Description of the Test Case
# Seoond Argument => RLM Conf to Be Set As per RLM_IPv6 Test Plan
# Third  Argument => RLM/SP hwassist stats from ONTAP
# Fourth Argument => RLM/SP hwassist statistics from status -d
# Fifth  Argument => RLM/SP hwassist events from RLM/SP events

#-------------------------------------------------------------------------------
sub load_ipv4_ipv6_HW_ASSIST_config_cases {
    %hw_test_cases = (
        1 => [
            "Case 1. hw_assist partners are IPv4 addresses and $RLM_PROMPT's have IPv4 addresses",
            2,
            1,
            1,
            1],);

} ## end sub load_ipv4_ipv6_HW_ASSIST_config_cases

#-------------------------------------------------------------------------------
# FUNCTION	: To verify the filers are in cluster setup.
# INPUT		: server connection handle
# RETURN		: NONE
#-------------------------------------------------------------------------------
sub check_cluster_setup {

    logcomment("Verify the filers are in a cluster setup");

    my $Server   = shift;
    my @Filers   = $Server->servers();
    my $sfo_show = $Server->storage_failover_show();

    # Verify the NODES parameter has filer names.
    logcomment("Verify the NODES parameter has filer names");
    if (scalar(@Filers) < 2) {
        logresult("FATAL",
                  "Two filers must be specified for the NODES parameter");
    }

    # Verify alteast two filers are configured in the cluster
    logcomment("Verify alteast two filers are configured in the cluster ");
    my $Cluster = $Server->cluster_show();
    if (scalar(@{$Cluster}) < 2) {
        logresult("FATAL",
                  "This test requires at least two filers in the cluster");
    }

    # Ensure that the two filers are in the cluster and that they are
    # online
    logcomment("Verify the filers are online state");
    $found = 0;
    for (my $count = 0 ; $count <= (scalar(@{$Cluster}) - 1) ; $count++) {
        ($found++)
          if (   (grep($_ eq $$Cluster[$count]{filer}, @Filers))
              && ($$Cluster[$count]{health} eq "true"));
    }
    if ($found < 2) {
        logresult("FATAL",
                " The filers specified are either not in quorum or not online");
    }

    # Ensure that SFO is enabled in both the filers
    logcomment("Verify the SFO is enabled in both the filers");
    for (my $i = 0 ; $i <= (scalar(@Filers) - 1) ; $i++) {
        logcomment(
            "SFO Show for $$sfo_show[$i]{'node-name'}: $$sfo_show[i]{'possible'} , $$sfo_show[i]{'state'}"
        );
        if (   $$sfo_show[$i]{'possible'} =~ /true/
            && $$sfo_show[$i]{'state'}           =~ /connected/
            && $$sfo_show[$i]{'interconnect-up'} =~ /true/
            && $$sfo_show[$i]{'enabled'}         =~ /true/) {
            logcomment(
                "Storage Failover is enabled in $$sfo_show[$i]{'node-name'} filer as expected"
            );
        } else {
            logresult(
                "FATAL",
                "Storage failover is disabled in either of the filers. Kindly enable SFO in both filers inorder to proceed with this testcase"
            );
        }
    } ## end for (my $i = 0 ; $i <= ...)

    # Enable AUTOBOOT and cf.giveback.auto.enable in both filers
    logcomment("Enable AUTOBOOT and cf.giveback.auto.enable in both filers");
    $Server->set_execution_server(-Server => $Filers[0]);
    $Server->cli(cmd => "set -privilege test -confirmations off");
    $Server->cli(cmd => "run local options cf.giveback.auto.enable on");
    $Server->cli(cmd => "run local bootargs set AUTOBOOT true");
    $Server->cli(cmd => "system autosupport modify -node $Filers[0] -state disable");

    $Server->set_execution_server(-Server => $Filers[1]);
    $Server->cli(cmd => "set -privilege test -confirmations off");
    $Server->cli(cmd => "run local options cf.giveback.auto.enable on");
    $Server->cli(cmd => "run local bootargs set AUTOBOOT true");
    $Server->cli(cmd => "system autosupport modify -node $Filers[1] -state disable");

} ## end sub check_cluster_setup

#-------------------------------------------------------------------------------
# FUNCTION	: To verify ontap and rlm/sp configuration details.
# INPUT		: FilerA
#				: FilerB
#				: testcase number
#				: server connection handle
# RETURN		: NONE
#-------------------------------------------------------------------------------
sub check_ontap_rlm_config_status {

    logcomment("Set the $RLM_PROMPT details depending on the test case");

    my ($FILERA, $FILERB, $case, $Server) = @_;

    check_cluster_setup($Server);

    $self_partner_ip = check_ontap_config_status($FILERA, $case);
    $part_partner_ip = check_ontap_config_status($FILERB, $case);

    rlm_load_ipv4_ipv6_config_cases($FILERA);
    rlm_setup_10mode($FILERA, $test_states{$hw_test_cases{$case}->[1]});
    rlm_reboot_rlm_10mode($FILERA);
    sleep(60);
    rlm_verify_rlm_status_10mode($FILERA, 'ONTAP', $hw_test_cases{$case}->[1]);

    rlm_load_ipv4_ipv6_config_cases($FILERB);
    rlm_setup_10mode($FILERB, $test_states{$hw_test_cases{$case}->[1]});
    rlm_reboot_rlm_10mode($FILERB);
    sleep(60);
    rlm_verify_rlm_status_10mode($FILERB, 'ONTAP', $hw_test_cases{$case}->[1]);

    set_cf_hwassist_partner_ip($FILERA, $part_partner_ip, $Server);
    set_cf_hwassist_partner_ip($FILERB, $self_partner_ip, $Server);

} ## end sub check_ontap_rlm_config_status

#-------------------------------------------------------------------------------
# FUNCTION	: To set the hwassist partner ip
# INPUT		: Filer name
#				: partner ip
#				: server connection handle
# RETURN		: NONE
#-------------------------------------------------------------------------------
sub set_cf_hwassist_partner_ip {

    my ($filer, $ip, $Server) = @_;
    logcomment("Set $filer with the hwassist partner ip as $ip");
    $Server->storage_failover_modify(node                  => $filer,
                                     "hwassist-partner-ip" => $ip);
    $Server->set_execution_server(-Server => $filer);
    $Server->storage_failover_show();
    $Server->cli(cmd => "run local cf hw_assist status");

} ## end sub set_cf_hwassist_partner_ip

#-------------------------------------------------------------------------------
# FUNCTION	: To enable/disable the hwassist
# INPUT		: Filer name
#				: true/false
#				: server connection handle
# RETURN		: NONE
#-------------------------------------------------------------------------------
sub set_hwassist_enable {

    # storage failover modify -node f3040-177-59 -hwassist true
    my ($filer, $msg, $Server) = @_;
    logcomment("Set the hwassist option to $msg in $filer");
    $Server->storage_failover_modify(node       => $filer,
                                     "hwassist" => $msg);
    $Server->set_execution_server(-Server => $filer);
    $Server->storage_failover_show(node   => $filer,
                                   fields => "hwassist");

} ## end sub set_hwassist_enable

#-------------------------------------------------------------------------------
# FUNCTION	: To verfiy the hwassist feature
# INPUT		: FilerA
#				: FilerB
#				: testcase
#				: server connection handle
# RETURN		: NONE
#-------------------------------------------------------------------------------
sub verify_hwassist_test {

    my ($self, $part, $case, $Server, $cmd) = @_;

    #Enabling the hw_assist on both the filers
    set_hwassist_enable($self, "true", $Server);
    set_hwassist_enable($part, "true", $Server);

    sleep 5;

    run_hwassist_test($self, $part, $case, 1, $Server, $cmd);

    cf_status_precheck_10mode($self, $part);
    doFilersUp($self, $part);

    if ($STATE == 1) {
        logcomment("Enabling the hw_assist on FilerA and Disabling On FilerB");

        set_hwassist_enable($self, "true",  $Server);
        set_hwassist_enable($part, "false", $Server);

        sleep 5;

        run_hwassist_test($self, $part, $case, 2, $Server, $cmd);

        cf_status_precheck_10mode($self, $part);
        doFilersUp($self, $part);

        logcomment("Enabling the hw_assist on FilerB and Disabling On FilerA");

        set_hwassist_enable($self, "false", $Server);
        set_hwassist_enable($part, "true",  $Server);

        sleep 5;

        run_hwassist_test($self, $part, $case, 3, $Server, $cmd);

        cf_status_precheck_10mode($self, $part);
        doFilersUp($self, $part);

        logcomment("Disabling the hw_assist on Both the filers");

        set_hwassist_enable($self, "false", $Server);
        set_hwassist_enable($part, "false", $Server);

        sleep 5;

        run_hwassist_test($self, $part, $case, 4, $Server, $cmd);

        cf_status_precheck_10mode($self, $part);
        doFilersUp($self, $part);
    } ## end if ($STATE == 1)

    logresult(PASS, "Test Successful");

} ## end sub verify_hwassist_test

#-------------------------------------------------------------------------------
# FUNCTION	: To run the hwassist test
# INPUT		: FilerA
#				: FilerB
#				: testcase
#				: server connection handle
# RETURN		: NONE
#-------------------------------------------------------------------------------
sub run_hwassist_test {

    my ($self, $part, $case, $type, $Server, $cmd) = @_;

    my ($eventsA1,          $eventsA2);
    my ($eventsB1,          $eventsB2);
    my ($statsA1,           $statsA2);
    my ($statsB1,           $statsB2);
    my ($statusA1,          $statusA2);
    my ($statusB1,          $statusB2);
    my ($confA1,            $confA2);
    my ($confB1,            $confB2);
    my ($partner_detailsA1, $partner_detailsA2);
    my ($partner_detailsB1, $partner_detailsB2);
    my ($hw_event, $ems_cmd, $invert);
    my ($hw_ems_before_result, $hw_ems_after_result);

    logcomment(
        "Verify the hwassist stats, hwassist statstics and hwassist events BEFORE the HW_ASSIST takeover for $self"
    );
    ($statsA1, $statusA1, $confA1, $eventsA1, $partner_detailsA1) =
      get_hwassist_details($self, $Server);

    logcomment(
        "Verify the hwassist stats, hwassist statstics and hwassist events BEFORE the takeover for $part"
    );
    ($statsB1, $statusB1, $confB1, $eventsB1, $partner_detailsB1) =
      get_hwassist_details($part, $Server);

    # Checking the configuration of the partner details from RLM/SP
    $result = test_hwassist_get_partner_details($partner_detailsA1, $confA1);

    if ($result == 1) {
        logcomment(
            "hwassist_partner details are same as Expected before test for $self"
        );
    } else {
        logresult(FAIL,
            "hwassist_partner details are Changed and Not as Expected before test for $self"
        );
    }

    $result =  test_hwassist_get_partner_details($partner_detailsB1, $confB1);

    if ($result == 1) {
        logcomment(
            "hwassist_partner details are same as Expected before test for $part"
        );
    } else {
        logresult(FAIL,
            "hwassist_partner details are Changed and Not as Expected before test for $part"
        );
    }

    # the type =3 and type =4 wont produce the "SChkHWassistTkOvr" as hwassist is disabled on FilerA
    # watchdog hang, watchdog wd_reset, reboot, halt, cf takeover, panic - wont produce "SChkHWassistTkOvr"
    if (($type == 3) || ($type == 4) || ($cmd eq "watchdog hang") ||
    		($cmd eq "cf takeover") || ($cmd eq "panic") || ($cmd eq "reboot") ||
         ($cmd eq "cf hw_assist test") || ($cmd eq "halt")) {
        $invert = 1;
    } else {
        $invert = 0;
    }


    logcomment(
        "Verify the 'SChkHWassistTkOvr' event from  /mroot/etc/log/ems BEFORE the HW_ASSIST takeover"
    );
    $ems_cmd = " grep -r 'SChkHWassistTkOvr' /mroot/etc/log/ems | wc -l";
    my $hw_ems_before_result = $Server->shell(cmd => $ems_cmd);
    logcomment(
        " No: of 'SChkHWassistTkOvr' events in ems log before takeover = $hw_ems_before_result"
    );

    # Send the command to the Filer. Start the test into background
    # and store the returned object into @subtests array.
    logcomment("Running the test command");
    push @subtests,
      subtest(\&send_cmd_to_filer, '-bg', '-runid', 'send_cmd_to_filer', '--',
              $self, $part, $cmd, $Server);

    # Wait for both tests to finish.
    Subtest::wait_finish(subtests => [@subtests]);

   sleep 30;

    push @subtests,
      subtest(\&get_info_about_partner, '-runid', 'get_info_from_partner', '--',
              $self, $part, $cmd, $Server);

    # time needed for giving back.
    sleep(120);

    logcomment(
        "Verify the 'SChkHWassistTkOvr' event from  /mroot/etc/log/ems AFTER the HW_ASSIST takeover"
    );
    $hw_ems_after_result = $Server->shell(cmd => $ems_cmd);
    logcomment(
        " No: of 'SChkHWassistTkOvr' events in ems log after takeover = $hw_ems_after_result"
    );

    logcomment(
        "Verify the hwassist stats, hwassist statstics and hwassist events AFTER the takeover for $self"
    );
    ($statsA2, $statusA2, $confA2, $eventsA2, $partner_detailsA2) =
      get_hwassist_details($self, $Server, 0);

    logcomment(
        "Verify the hwassist stats, hwassist statstics and hwassist events AFTER the HW_ASSIST takeover for $part"
    );
    ($statsB2, $statusB2, $confB2, $eventsB2, $partner_detailsB2) =
      get_hwassist_details($part, $Server);

    my ($result, $result1, $result2, $result3,) = 0;

    if ($cmd eq "system power off") {
        $hw_event = "power_off_via_$RLM";
    }
    elsif($cmd eq "system power cycle") {

      $hw_event = "power_cycle_via_$RLM";
    }
    elsif($cmd eq "system reset") {

      $hw_event = "reset_via_$RLM";
    }
    elsif(($cmd eq "watchdog hang") || ($cmd eq "reboot") || ($cmd eq "halt") || ($cmd eq "cf takeover") || ($cmd eq "panic")) {

      $hw_event = "abnormal_reboot";
    }
    elsif($cmd eq "watchdog wd_reset") {

      $hw_event = "l2_watchdog_reset";
    }
     elsif($cmd eq "cf hw_assist test") {

      $hw_event = "test";
    }



    if ($cmd eq "cf hw_assist test") {
       # Checking the configuration of the partner details from RLM/SP for hwassist test commad
    	$result = test_hwassist_get_partner_details($partner_detailsB1, $confB2);
    	if ($result == 1) {
        	logcomment(
            "hwassist_partner details are same as Expected before and after test for $part"
        );
    	} else {
        	logresult(FAIL,
            "hwassist_partner details are Changed and Not as Expected before and after test for $part"
        );
    	}

   	 $result = test_hwassist_get_partner_details($partner_detailsA1, $confA2);
    	if ($result == 1) {
        	logcomment(
            "hwassist_partner details are same as Expected before and after test for $self"
        );
    	} else {
        	logresult(FAIL,
            "hwassist_partner details are Changed and Not as Expected before and after test for $self"
        );
    	}
    } ## end if ($cmd eq "cf hw_assist test")


   logcomment(
        "Compare the details of the hwassist stats, hwassist statstics ,hwassist events and SChkHWassistTkOvr before and after HW_ASSIST takeover "
    );

    # need to uncomment once the "storage failover hwassist show" alert events are displayed properly.
    # $result1 = test_hwassist_stats_check($statsB1, $statsB2, $hw_event);


    # if ($RLM_PROMPT !~ /RLM/) {
    $result3 = test_hwassist_events_check($eventsA1, $eventsA2, $hw_event);
    # }

    if ($cmd eq "watchdog wd_reset")
    {
     $hw_event = "l2_watchdog_timeout";   # name of the event is different for stats and statistics for watchdog wd_reset
    }
    $result2 = test_hwassist_statistics_check($statusA1, $statusA2, $hw_event);

    #Validating the results with the expected results
    ($result1, $result2, $result3) =
      hwassist_test_results($case, $type, $result1, $result2, $result3);

    if ($result2 == 1) {
        logcomment("hwassist_statistics Incremented or as Expected");
    }elsif(($result3 == 1) && ($cmd eq "watchdog hang") || ($cmd eq "reboot") || ($cmd eq "halt") || ($cmd eq "cf takeover") || ($cmd eq "panic")) {
      logcomment("hwassist_events not generated or as Expected");
    } else {
        logresult('FAIL',
                  "hwassist_statistics are Not Incremented or as not Expected");
    }

    # if ($RLM_PROMPT !~ /RLM/) {
    if ($result3 == 1) {
        logcomment("hwassist_events generated or as Expected");
    }elsif(($result3 == 1) && ($cmd eq "watchdog hang") || ($cmd eq "reboot") || ($cmd eq "halt") || ($cmd eq "cf takeover") || ($cmd eq "panic")) {
      logcomment("hwassist_events not generated or as Expected"); 
            } else {
        logresult('FAIL',
                  "hwassist_events are Not generated or as not Expected");
    }

    # }

=comment
     # need to uncomment once the "storage failover hwassist show" alert events are displayed properly.
    if ($result1 == 1) {
        logresult('PASS', "$RLM hwassist for event:$cmd Happened");
    } else {
        logresult('FATAL', "$RLM hwassist for event:$cmd NOT Happened");
    }
=cut

	# If $invert=1, the  SChkHWassistTkOvr msg is not produced in ems log, as the hwassist is disabled in FilerA
    if ($invert) {
        if ($hw_ems_after_result > $hw_ems_before_result) {
            logresult('FAIL',
                      "Observed the Unexpected string 'SChkHWassistTkOvr'");
        } else {
            logresult(
                'PASS',
                "SChkHWassistTkOvr did not observed in ems log after takeover \n"
            );
        }
    } else {
        if ($hw_ems_after_result > $hw_ems_before_result) {
            logresult(
                'PASS',
                " The 'SChkHWassistTkOvr' ems event was generated in ems log as Expected"
            );
        } else {
            logresult(
                'FAIL',
                " The 'SChkHWassistTkOvr' ems event was NOT generated in ems log"
            );
        }
    } ## end else [ if ($invert) ]

} ## end sub run_hwassist_test

#-------------------------------------------------------------------------------
# FUNCTION	: To get the hwassist details
# INPUT		: FilerA
#				: server connection handle
#				: ignore_case
# RETURN		: NONE
#-------------------------------------------------------------------------------
#sub get_hwassist_details($;$) {
sub get_hwassist_details($$;$) {

    my $FILER       = shift;
    my $Server      = shift;
    my $ignore_case = defined($_[0]) ? $_[0] : '1';

    my $fc = rlm_make_conn_to_ontap($FILER);

    my ($stats, $partner_details);
    if ($ignore_case) {

        # Capturing the info from the "cf hw_assist stats" command
        $stats = cf_hwassist_stats($FILER, $Server);

        # Capturing the info from the "cf.hw_assist" commands
        $partner_details = cf_hwassist_partner_details($FILER, $Server);
    }

    # Capturing the info from the "RLM/SP status -d" command
    my ($status, $conf) = cf_hwassist_statistics($FILER);

    my $events;

    # if ($RLM_PROMPT !~ /RLM/) {
    # Capturing the info from the "events" command
    $events = cf_hwassist_events($FILER);

    #  }

    return ($stats, $status, $conf, $events, $partner_details);

} ## end sub get_hwassist_details($$;$)

#-------------------------------------------------------------------------------
# FUNCTION	: To get the hwassist stats
# INPUT		: FilerA
#				: server connection handle
# RETURN		: NONE
#-------------------------------------------------------------------------------
sub cf_hwassist_stats {
    my $FILER  = shift;
    my $Server = shift;

    logcomment("Taking the cf_hwassist_stats for $FILER");

    $Server->set_execution_server(-Server => $FILER);

    my $hw_assist_events =
      $Server->storage_failover_hwassist_show(node => $FILER);

    my %hw_assist_stats;

    ($hw_assist_stats{'power_loss'}) = $$hw_assist_events[0]{'power_loss'};
    ($hw_assist_stats{'l2_watchdog_reset'}) =
      $$hw_assist_events[0]{'watchdog-reset'};

    if ($RLM_PROMPT =~ /RLM/) {
        ($hw_assist_stats{"power_off_via_$RLM"}) =
          $$hw_assist_events[0]{'power-off-via-rlm'};
        ($hw_assist_stats{"power_cycle_via_$RLM"}) =
          $$hw_assist_events[0]{'power-cycle-via-rlm'};
        ($hw_assist_stats{"reset_via_$RLM"}) =
          $$hw_assist_events[0]{'reset-via-rlm'};
    } elsif ($RLM_PROMPT =~ /SP/) {
        ($hw_assist_stats{"power_off_via_$RLM"}) =
          $$hw_assist_events[0]{'power-off-via-sp'};
        ($hw_assist_stats{"power_cycle_via_$RLM"}) =
          $$hw_assist_events[0]{'power-cycle-via-sp'};
        ($hw_assist_stats{"reset_via_$RLM"}) =
          $$hw_assist_events[0]{'reset-via-sp'};
    }

    ($hw_assist_stats{'post_error'}) = $$hw_assist_events[0]{'post-error'};
    ($hw_assist_stats{'abnormal_reboot'}) =
      $$hw_assist_events[0]{'abnormal-reboot'};
    ($hw_assist_stats{'loss_of_heartbeat'}) =
      $$hw_assist_events[0]{'loss-of-heartbeat'};
    ($hw_assist_stats{'test'})       = $$hw_assist_events[0]{'test'};
    ($hw_assist_stats{'keep-alive'}) = $$hw_assist_events[0]{'keep-alive'};

    return \%hw_assist_stats;
} ## end sub cf_hwassist_stats

#-------------------------------------------------------------------------------
# FUNCTION	: To get the hwassist partner details
# INPUT		: FilerA
#				: server connection handle
# RETURN		: NONE
#-------------------------------------------------------------------------------
sub cf_hwassist_partner_details {

    my ($FILER, $Server) = @_;

    logcomment("Taking the hwassist_partner_details for $FILER");
    $Server->set_execution_server(-Server => $FILER);
    my $hw_details =
      $Server->storage_failover_show(node   => $FILER,
                                     fields => "hwassist",
                                     fields => "hwassist-partner-port",
                                     fields => "hwassist-partner-ip",
                                     fields => "hwassist-health-check-interval"
                                    );

    my %hw_assist_partner_conf;
    ($hw_assist_partner_conf{'enable'}) = $$hw_details[0]{'hwassist'};
    ($hw_assist_partner_conf{'address'}) =
      $$hw_details[0]{'hwassist-partner-ip'};
    ($hw_assist_partner_conf{'port'}) =
      $$hw_details[0]{'hwassist-partner-port'};
    ($hw_assist_partner_conf{'health_check_interval'}) =
      $$hw_details[0]{'hwassist-health-check-interval'};
    $Server->cli(cmd => "set -privilege test -confirmations off");
    my $hw_ss_result =
      $Server->cli(cmd => "run local cf test_hw_assist ss get");
    %hw_assist_partner_conf =
      (%hw_assist_partner_conf, $hw_ss_result =~ /(Local ss)\s+(\w+)/);

    my $hw_assist_partner = \%hw_assist_partner_conf;
    logcomment(
        "hwassist_partner_details for $FILER are
         \n enable -
          $hw_assist_partner->{'enable'} \n address -
          $hw_assist_partner->{'address'} \n port -
          $hw_assist_partner->{'port'} \n health_check_interval -
          $hw_assist_partner->{'health_check_interval'} \n Local ss -
          $hw_assist_partner->{'Local ss'} \n "
              );

    return \%hw_assist_partner_conf;

} ## end sub cf_hwassist_partner_details

#-------------------------------------------------------------------------------
# FUNCTION	: To get the hwassist events
# INPUT		: FilerA
# RETURN		: The hwassist events from the RLM/SP
#-------------------------------------------------------------------------------
sub cf_hwassist_events {
    my $FILER = shift;

    logcomment(" Taking the hwassist_events for $FILER ");
    my $fh = rlm_make_host_to_ontap($FILER);

    tstlog -comment, " Checking SP / RLM Events Commands ";

    if ($ipv4_ip) {
        $ip = $fh->{'RLM_IP'};
    } elsif (   ($ipv6_global_ip !~ /Disabled|unknown/i)
             && $ipv6_global_ip
             && ($ipv6_global_ip ne $ipv6_gateway)) {
        $ip = $fh->{'RLM_IPV6_IP'};
    } elsif ($ipv6_router_ip) {
        $ip = $fh->{'RLM_IPV6_ROUTER'};
    }

    my $rlm_pt = rlm_make_conn_to_rlm($FILER, $ip);

    my (undef, undef, $out) =
      say($rlm_pt, " events search hwassist ", '-swallow', '-timeout', 120);

    disconnect($rlm_pt);

    return $out;
} ## end sub cf_hwassist_events

#-------------------------------------------------------------------------------
# FUNCTION	: To get the hwassist statistics from RLM/SP
# INPUT		: FilerA
# RETURN		: hwassist statistics from the RLM/SP
#-------------------------------------------------------------------------------
sub cf_hwassist_statistics {

    my $FILER = shift;

    logcomment(" Taking the cf_hwassist_statistics for $FILER ");
    my $fh = rlm_make_host_to_ontap($FILER);

    logcomment(" Checking $RLM Status -d Command for $FILER ");

    if ($ipv4_ip) {
        $ip = $fh->{'RLM_IP'};
    }

    my $rlm_pt = rlm_make_conn_to_rlm($FILER, $ip);

    my (undef, $out, undef, undef) =
      say($rlm_pt, "$RLM status -d ",
          '-swallow', -noprompt, '-wait', "$RLM_PROMPT $FILER.*>",
          '-timeout', 180);

    my %hwassist_status_d = $out =~
      /trap\sdaemon.+(takeover enable)\s+(\d+).+(takeover trap power_off_via_$RLM)\s+(\d+).+(takeover trap power_cycle_via_$RLM)\s+(\d+).+(takeover trap reset_via_$RLM)\s+(\d+).+(takeover trap post_error)\s+(\d+).+(takeover trap abnormal_reboot)\s+(\d+).+(takeover trap l2_watchdog_timeout)\s+(\d+).+(takeover trap power_loss)\s+(\d+).+(takeover trap loss_of_heartbeat)\s+(\d+).+(takeover test trap)\s+(\d+).+(takeover test trap failure)\s+(\d+).+/s;

    my %hwassist_conf_status_d;

    ($hwassist_conf_status_d{'takeover_enable'}) =
      $out =~ /\/mnt\/logs\/etc\/takeover_conf.+takeover_enable\s+(\w+)/s;
    ($hwassist_conf_status_d{'partner_ip'}) =
      $out =~ /\/mnt\/logs\/etc\/takeover_conf.+partner_ip\s+([\w\.\:]+)/s;
    ($hwassist_conf_status_d{'partner_port'}) =
      $out =~ /\/mnt\/logs\/etc\/takeover_conf.+partner_port\s+(\d+)/s;
    ($hwassist_conf_status_d{'hello_interval'}) =
      $out =~ /\/mnt\/logs\/etc\/takeover_conf.+hello_interval\s+(\d+)/s;
    ($hwassist_conf_status_d{'repeat_count'}) =
      $out =~ /\/mnt\/logs\/etc\/takeover_conf.+repeat_count\s+(\d+)/s;
    ($hwassist_conf_status_d{'secret'}) =
      $out =~ /\/mnt\/logs\/etc\/takeover_conf.+secret\s+(\w+)/s;
    ($hwassist_conf_status_d{'ip_family'}) =
      $out =~ /\/mnt\/logs\/etc\/takeover_conf.+ip_family\s+(\d+)/s;
    ($hwassist_conf_status_d{'partner_ip6'}) =
      $out =~ /\/mnt\/logs\/etc\/takeover_conf.+partner_ip6\s+([\w\:]+)/s;

    disconnect($rlm_pt);

    return \%hwassist_status_d, \%hwassist_conf_status_d;
} ## end sub cf_hwassist_statistics

#-------------------------------------------------------------------------------
# FUNCTION	: Compare the hwassist partner details
# INPUT		: hwassist details before hwassist takeover
#           : hwassist details after hwassist takeover
# RETURN		: 0 for fail/ 1 for pass
#-------------------------------------------------------------------------------
sub test_hwassist_get_partner_details {

    my ($before, $after) = @_;

    logcomment("$before->{'enable'} \n $before->{'address'} \n $before->{'port'} \n $before->{'health_check_interval'} \n $before->{'Local ss'} \n ");

    logcomment("$after->{'takeover_enable'} \n $after->{'partner_ip'} \n $after->{'partner_port'} \n $after->{'hello_interval'} \n $after->{'secret'} \n $after->{'ip_family'} \n $after->{'repeat_count'} ");

    #unless($before->{'enable'} eq $after->{'takeover_enable'}) {
    unless ((   ($before->{'enable'} eq "true") && ($after->{'takeover_enable'} eq "on"))
            || (($before->{'enable'} eq "false") && ($after->{'takeover_enable'} eq "off"))
      ) {
        logcomment(" Partner Details Are Mismatching(1) \n Before = $before->{'enable'} \n After = $after->{'takeover enable'} \n ");
        return 0;
    } ## end unless ((($before->{'enable'...})))

    unless (   ($before->{'address'} eq $after->{'partner_ip'})
            || ($before->{'address'} eq $after->{'partner_ip6'})) {
        logcomment(" Partner Details Are Mismatching(2) \n Before = $before->{'address'} \n After = $after->{'partner_ip'} \n ");
        return 0;
    }

    unless ($before->{'port'} eq $after->{'partner_port'}) {
        logcomment(" Partner Details Are Mismatching(3) \n Before = $before->{'port'} \n After = $after->{'partner_port'} \n ");
        return 0;
    }

    unless ($before->{'health_check_interval'} eq $after->{'hello_interval'}) {
        logcomment(" Partner Details Are Mismatching(4) \n Before = $before->{'health_check_interval'} \n After = $after->{'hello_interval'} \n ");
        return 0;
    }

    unless ($before->{'Local ss'} eq $after->{'secret'}) {
        logcomment(" Partner Details Are Mismatching(5) \n Before = $before->{'Local ss'} \n After = $after->{'secret'} \n ");
        return 0;
    }

    unless (($after->{'ip_family'} == 2) || ($after->{'ip_family'} == 10)) {
        logcomment(" Partner Details Are Mismatching(6) \n IP_FAMILY = $after->{'ip_family'} \n ");
        return 0;
    }

    unless ($after->{'repeat_count'} == 2) {
        logcomment(" Partner Details Are Mismatching(7) \n REPEAT_COUNT = $after->{'repeat_count'} \n ");
        return 0;
    }

    return 1;
} ## end sub test_hwassist_get_partner_details

#-------------------------------------------------------------------------------
# FUNCTION	: Send the command to the filer
# INPUT		: FilerA
#				: FilerB
#				: command to be executed
# RETURN    : None
#-------------------------------------------------------------------------------
sub send_cmd_to_filer {
    my ($self, $part, $msg, $Server) = @_;

    logcomment("Value of self = $self");
    logcomment("Value of part = $part");
    my ($ip, $cmd);

    rlm_load_tharnhost_data($self);

    my $fh = rlm_make_host_to_ontap($self);

    if ($ipv4_ip) {
        $ip = $fh->{'RLM_IP'};
    }

    if (   ($msg eq "system power off")
        || ($msg eq "system power cycle")
        || ($msg eq "system reset")) {
        my $rlm_pt = rlm_make_conn_to_rlm($self, $ip);
        if ($msg eq "system reset") {
            say($rlm_pt, "system reset", '-prompt', "Continue", '-timeout', 90);
            say($rlm_pt, "y", '-timeout', 90);

            sleep(120);
        } elsif ($msg eq "system power cycle") {
            say($rlm_pt, "system power cycle",
                '-prompt', "Continue", '-timeout', 90);
            say($rlm_pt, "y", '-timeout', 90);
        } elsif ($msg eq "system power off") {
            say($rlm_pt, "system power off",
                '-prompt', "Continue", '-timeout', 90);
            say($rlm_pt, "y", '-timeout', 90);

            sleep 60;

            say($rlm_pt, "system power on", '-timeout', 90);
        }

        # @step objective:
        # This step is reading the trap received by trapd daemon on the client
        # and deciding whether it is correct or wrong
        sleep(120);

        disconnect($rlm_pt);

    } elsif (($msg eq "watchdog hang") || ($msg eq "watchdog wd_reset")) {
        $Server->set_execution_server(-Server => $self);
        $cmd = "sudo sysctl debug.watchdog";
        $Server->shell(cmd => $cmd);
        $cmd = "sudo sysctl debug.watchdog.disable=0";
        $Server->shell(cmd => $cmd);
        if (($msg eq "watchdog hang")) {
            $cmd = "sudo sysctl debug.watchdog.ignore_nmi=0";
            $Server->shell(cmd => $cmd);
            $cmd = "sudo sysctl debug.watchdog.trigger_l2_timeout=0";
            $Server->shell(cmd => $cmd);
        } elsif ($msg eq "watchdog wd_reset") {
            $cmd = "sudo sysctl debug.watchdog.ignore_nmi=1";
            $Server->shell(cmd => $cmd);
            $cmd = "sudo sysctl debug.watchdog.trigger_l2_timeout=1";
            $Server->shell(cmd => $cmd);
        }
        $cmd = "sudo sysctl debug.watchdog.hang_cpu=1";
        $Server->shell(cmd => $cmd);

        sleep(120);
    } elsif ($msg eq "reboot") {

        #$Server->system_reboot(node => $self);
        $Server->system_node_reboot(node => $self);
    } elsif ($msg eq "halt") {
      $Server->system_node_halt(node => $self);
      sleep 60;
    } elsif ($msg eq "cf takeover") {
        $Server->set_execution_server(-Server => $part);
        $Server->storage_failover_takeover(ofnode => $self);
        sleep 60;
    }
    elsif ($msg eq "panic") {
        $Server->set(privilege => "test");
        $Server->set_execution_server(-Server => $part);
        $Server->cli(cmd => "system node run -node $self -command panic", noprompt => 1);
        sleep 60;

        #added the below code as the to make sure ONTAP prompt.
        $Server->cli(cmd => "\cD");

    }
    elsif ($msg eq "cf hw_assist test") {
        $Server->storage_failover_hwassist_test(node => $self);
    }

} ## end sub send_cmd_to_filer
#-------------------------------------------------------------------------------
# FUNCTION	: To the information about the partner state
# INPUT		: FilerA
#				: FilerB
#				: server filer handle
# RETURN		: None
#-------------------------------------------------------------------------------
sub get_info_about_partner {

    my ($self, $part, $cmd, $Server) = @_;

    my $sfo_show = $Server->storage_failover_show();

    #Capturing the CF state of the filer(UP / TAKEOVER / UNREACH / DEAD)
    logcomment("State of $part is $$sfo_show[1]{'reason'}");

    # Takeover wont happen for test command in hwassist
    if ($cmd ne "cf hw_assist test") {
    	if ($$sfo_show[1]{'reason'} !~ /In takeover/i) {
        logresult('FAIL',
                  "$part should have taken over $self, seems like it hasn\'t.");
   	 } else {
        	logresult(
            	'PASS',
            	"No mismatches, and node $part in a takeover state.\n $part has properly taken over $self"
        	);
    	}
    }

} ## end sub get_info_about_partner

#-------------------------------------------------------------------------------
# FUNCTION	: To verify the hwassist stats before and after hw_assist
# INPUT		: hwassist stats details before hw_assist takeover
#				: hwassist stats details after hw_assist takeover
#				: hwassist event msg (eg: power_off_via_rlm)
# RETURN    : 1 for pass, 0 for fail
#-------------------------------------------------------------------------------
sub test_hwassist_stats_check {
    my ($before, $after, $msg) = @_;

    foreach my $k (keys %{$before}) {
        if ($k =~ /$msg/) {
            if ($after->{$k} == ($before->{$k} + 2)) {
                logcomment(
                    "PASS\n After = $after->{$k} \n Before = $before->{$k} \n");
                return 1;
            } elsif($after->{$k} == ($before->{$k})&& $msg =~ /abnormal_reboot/) {
                # elseif Codition to check negative testcases
                                logcomment ("PASS\n After = $after->{$k} \n Before = $before->{$k} \n");
                                return 1;
                           }else {
                logcomment(
                    "FAIL\n After = $after->{$k} \n Before = $before->{$k} \n");
                return 0;
            }
        } ## end if ($k =~ /$msg/)
    } ## end foreach my $k (keys %{$before...})
} ## end sub test_hwassist_stats_check

#-------------------------------------------------------------------------------
# FUNCTION	: To verify the hwassist statistics before and after hw_assist
# INPUT		: hwassist statistics details before hw_assist takeover
#				: hwassist statistics details after hw_assist takeover
#				: hwassist event msg (eg: power_off_via_rlm)
# RETURN    : 1 for pass, 0 for fail
#-------------------------------------------------------------------------------
sub test_hwassist_statistics_check {
    my ($before, $after, $msg) = @_;
     foreach my $k (keys %{$before}) {
        if ($k =~ /$msg/) {
            if ($after->{$k} == ($before->{$k} + 1)) {
                logcomment(
                    "PASS\n After = $after->{$k} \n Before = $before->{$k} \n");
                return 1;
            } elsif($after->{$k} == ($before->{$k}) && $msg =~ /abnormal_reboot/) {
                # elseif Codition to check negative testcases
                                logcomment ("PASS\n After = $after->{$k} \n Before = $before->{$k} \n");
                                return 1;
                           } else {
                logcomment(
                    "FAIL\n After = $after->{$k} \n Before = $before->{$k} \n");
                return 0;
            }
        } ## end if ($k =~ /$msg/)
    } ## end foreach my $k (keys %{$before...})
} ## end sub test_hwassist_statistics_check

#-------------------------------------------------------------------------------
# FUNCTION	: To verify the hwassist events before and after hw_assist
# INPUT		: hwassist events details before hw_assist takeover
#				: hwassist events details after hw_assist takeover
#				: hwassist event msg (eg: power_off_via_rlm)
# RETURN    : 1 for pass, 0 for fail
#-------------------------------------------------------------------------------
sub test_hwassist_events_check {
    my ($before, $after, $msg) = @_;

    my @out1 = $before =~ /Record.+hwassist $msg/mg;

    my @out2 = $after =~ /Record.+hwassist $msg/mg;

    if ($out1[-1] ne $out2[-1]) {
        logcomment(
            "PASS\nEvents Occured Are \nAfter :$out1[-1] \n Before :$out2[-1]");
        return 1;
    } elsif ($out1[-1] eq $out2[-1] && $msg =~ /abnormal_reboot/)  {
        # To check negative testcases
                logcomment("PASS\nEvents Occured Are \nAfter :$out1[-1] \n Before :$out2[-1]");
                return 1;
                }else {
        logcomment(
            "FAIL\nEvents Occured Are \nAfter :$out1[-1] \n Before :$out2[-1]");
        return 0;
    }
} ## end sub test_hwassist_events_check

#-------------------------------------------------------------------------------
# FUNCTION	: To bring up the filers
# INPUT		: FilerA
#				: FilerB
# RETURN    : NONE
#-------------------------------------------------------------------------------
sub doFilersUp {

    my ($FILERA, $FILERB) = @_;

    $fhA = host("$FILERA", '-fatal', "Cannot allocate $FILERA");
    $fcA = connect("#console", '-fatal', "Cannot connect to $FILERA");
    $fhB = host("$FILERB", '-fatal', "Cannot allocate $FILERB");
    $fcB = connect("#console", '-fatal', "Cannot connect to $FILERB");

    my $boot_stateA = boot_state($fhA);
    my $boot_stateB = boot_state($fhB);
    logcomment(
        " State of filers \n $FILERA - $boot_stateA \n State of $FILERB - $boot_stateB"
    );

    eval {

        if ("UP" ne $boot_stateA) {
            transit_to($fcA, 'UP');
        }
        if ("UP" ne $boot_stateB) {
            transit_to($fcB, 'UP');
        }

      } ## end eval

} ## end sub doFilersUp

#-------------------------------------------------------------------------------
# FUNCTION	: Verify the ONTAP configuration status
# INPUT		: Filer name
#				: Test case number
# RETURN    : partner ip is returned
#-------------------------------------------------------------------------------
sub check_ontap_config_status {
    my $FILERA = shift;
    my $setup  = shift;

    my $hwassist_partner_ip;

    my $fcA = rlm_make_conn_to_ontap($FILERA);
    my $fhA = rlm_make_host_to_ontap($FILERA);

    my $self_default_ip = $fhA->default_ip;

    logcomment("Default IPv4_IP of $FILERA = $self_default_ip");

    unless ($self_default_ip) {
        logresult(
            'SCRIPT',
            "To run this script, filer tharnhost file should have the following fileds: default_ip"
        );
    }

    my (undef, undef, $out) = say($fcA, "\r\n", '-swallow', '-timeout', 60);
    sleep 2;
    my (undef, undef, $out) =
      say($fcA, "run local ifconfig -a", '-swallow', '-timeout', 60);

    # Only one test case is applicable to HW_ASSIST in 10 mode
    if ($setup == 1) {
        if ($out =~ /$self_default_ip/) {
            $hwassist_partner_ip = $self_default_ip;
        } else {
            logcomment("Setup Proper IPv4 IP for FILER");
        }
    }

    logcomment(
        "The hw_assist IP to assign to the partner of $FILERA : $hwassist_partner_ip"
    );
    return $hwassist_partner_ip;
} ## end sub check_ontap_config_status

#-------------------------------------------------------------------------------
# FUNCTION	: Verify hwassist the test results
# INPUT		: test case number
#				: test case type
#				: stats results
#				: statistics results
#           : events results
# RETURN    : the information of stats, statistics and events
#-------------------------------------------------------------------------------
sub hwassist_test_results {

    my $case               = shift;
    my $type               = shift;
    my $stats_results      = shift;
    my $statistics_results = shift;
    my $events_results     = shift;

    my $stats      = 1;
    my $statistics = 1;
    my $events     = 1;

    tstlog -comment, "Case:$case \n Type:$type";

    # type =3 and type =4 wont produce the hwassist stats, statistsics and events,
    # as those are the negative test case.
    if (($type == 3) || ($type == 4)) {
        $hw_test_cases{$case}->[2] = 0;
        $hw_test_cases{$case}->[3] = 0;
        $hw_test_cases{$case}->[4] = 0;
    }

    # uncomment when the stat issue is resolved.
    #    unless ($stats_results == $hw_test_cases{$case}->[2]) {
    #        logresult('FAIL', "Stats are Not updated properly");
    #        $stats = 0;
    #    }

    unless ($statistics_results == $hw_test_cases{$case}->[3]) {
        logresult('FAIL', "Statistics are Not updated properly");
        $statistics = 0;
    }

    #if ($RLM_PROMPT !~ /RLM/) {
    unless ($events_results == $hw_test_cases{$case}->[4]) {
        logresult('FAIL', "Events are Not updated properly");
        $events = 0;
    }

    # }

    return $stats, $statistics, $events;
} ## end sub hwassist_test_results

#-------------------------------------------------------------------------------
# FUNCTION	: Function to check the status of the filer and cf status in 10mode
# INPUT		: FilerA
#				: FilerB
#				:
# RETURN    : NONE
#-------------------------------------------------------------------------------
sub cf_status_precheck_10mode ( $$ ) {
    my ($filerA, $filerB) = @_;

    logcomment("Verify the Status of the filers");

    my $fhostA = rlm_make_host_to_ontap($filerA);
    my $fconnA = rlm_make_conn_to_ontap($filerA);
    my $fhostB = rlm_make_host_to_ontap($filerB);
    my $fconnB = rlm_make_conn_to_ontap($filerB);

    my $boot_stateA = boot_state($fhostA);
    my $boot_stateB = boot_state($fhostB);

    logcomment(
        "Boot state of filers \n $filerA - $boot_stateA \n $filerB - $boot_stateB"
    );

    if ($boot_stateA eq "FIRMWARE" && $boot_stateB eq "FIRMWARE") {
        logcomment(
              "Both nodes $filerA and $filerB are down, rebooting both nodes.");

        transit_to($fconnA, 'UP');
        transit_to($fconnB, 'UP');

        logcomment("Sleeping for 300 seconds");
        sleep(300);

        $boot_stateA = boot_state($fhostA);
        $boot_stateB = boot_state($fhostB);
    } elsif ($boot_stateA eq "FIRMWARE" && $boot_stateB eq "UP") {
        logcomment(
                  "$filerA is down and $filerB is UP, rebooting node $filerA.");

        transit_to($fconnA, 'UP');

        logcomment("Sleeping for 300 seconds");
        sleep(300);

        $boot_stateA = boot_state($fhostA);
        $boot_stateB = boot_state($fhostB);
    } elsif ($boot_stateA eq "UP" && $boot_stateB eq "FIRMWARE") {
        logcomment(
                  "$filerB is down and $filerA is UP, rebooting node $filerB.");

        transit_to($fconnB, 'UP');

        logcomment("Sleeping for 300 seconds");
        sleep(300);

        $boot_stateA = boot_state($fhostA);
        $boot_stateB = boot_state($fhostB);
    } ## end elsif ($boot_stateA eq "UP"...)

    my $state_of_nodeA = get_cf_state_10mode($filerA);
    my $state_of_nodeB = get_cf_state_10mode($filerB);

    logcomment(
        "State of filers \n $filerA - $state_of_nodeA \n $filerB - $state_of_nodeB"
    );
    logcomment(
        "Boot state of filers \n $filerA - $boot_stateA \n $filerB - $boot_stateB"
    );

    if ($boot_stateA eq "" || $boot_stateB eq "") {
        logresult(
            'FATAL',
            "Either or both of the nodes $filerA and $filerB could not be brought up, please look into it."
        );
    }

    if ($boot_stateA eq "FIRMWARE" && $boot_stateB eq "FIRMWARE") {
        logresult(
            'FATAL',
            "Either or both of the nodes $filerA and $filerB could not be brought up, please look into it."
        );
    } elsif ($state_of_nodeA eq "UP" && $state_of_nodeB eq "UP") {
        logcomment("Both filers are up");
    } elsif ($state_of_nodeA eq "DISABLED") {
        logcomment("cf is disabled in $filerA, enabling it");
        cf_enable_10mode($filerA);
    } elsif ($state_of_nodeA eq "TAKEOVER") {
        if ($boot_stateB eq "FIRMWARE") {
            logcomment("Node $filerB is down, rebooting it");
            rlm_reboot_rlm_10mode($filerB);
            logcomment("Sleeping for 300 seconds");
            sleep(300);
        } elsif ($boot_stateB eq "TAKEN_OVER_WAITING_ONTAP") {
            logcomment("Node $filerB has been takenover, giving it back");
            cf_giveback_10mode($filerA);
            sleep 60;
        } else {
            logresult('WARN', "Check the status of the filers.");
        }
    } elsif ($state_of_nodeB eq "TAKEOVER") {
        if ($boot_stateA eq "FIRMWARE") {
            logcomment("Node $filerA is down, rebooting it");
            rlm_reboot_rlm_10mode($filerA);
            logcomment("sleeping for 300 seconds");
            sleep(300);
        } elsif ($boot_stateA eq "TAKEN_OVER_WAITING_ONTAP") {
            logcomment("Node $filerA has been takenover, giving it back");
            cf_giveback_10mode($filerB);
            sleep 60;
        } else {
            logresult('WARN', "Check the status of the filers.");
        }
    } else {
        logresult(
            'WARN',
            "Unknown state of nodes $filerA and $filerB.  Please check the filers."
        );
    }
} ## end sub cf_status_precheck_10mode ( $$ )

#-------------------------------------------------------------------------------
# FUNCTION	: Function to get the cf status in 10mode
# INPUT		: FilerA
#				:
#				:
# RETURN    : return the state of the filer
#-------------------------------------------------------------------------------
sub get_cf_state_10mode ( $;$ ) {
    my ($filer, $rate) = @_;
    $rate = $rate || 10;
    my $conn = rlm_make_conn_to_ontap($filer);

    my $cur_state;

    while (1) {
        my $retstr;

        (undef, undef, $retstr) =
          say($conn, "run local cf status", '-eval_error');
        return 'UNREACH'  if ($@);
        return 'DISABLED' if ($retstr =~ /Cluster disabled/);
        return 'DISABLED' if ($retstr =~ /Controller Failover disabled/);
        return 'DISABLED' if ($retstr =~ /takeover disabled/);
        return 'DISABLED' if ($retstr =~ /has disabled takeover by/);

        last
          if (   length($retstr)
              && ($retstr !~ /attempting takeover/)
              && ($retstr !~ /giveback in progress/));

        sleep $rate;
    } ## end while (1)

    my $out;
    do {
        (undef, undef, $out) =
          say($conn, "run local cf monitor", '-eval_error');
        if ($out =~ /(UP|TAKEOVER|DEAD)/) {
            $cur_state = $1;
        }
    } while ($out =~ /Try again/ && !defined($cur_state));

    if (!defined($cur_state)) {
        return 'UNREACH';
    } else {
        return $cur_state;
    }
} ## end sub get_cf_state_10mode ( $;$ )

#-------------------------------------------------------------------------------
# FUNCTION	: Function to enable the cf status in 10mode
# INPUT		: Filer name

# RETURN    : 1 on success
#-------------------------------------------------------------------------------
sub cf_enable_10mode ( $ ) {
    my ($filer) = @_;
    my $conn = rlm_make_conn_to_ontap($filer);
    say($conn, "run local cf enable", '-eval_error');
    return ($@) ? undef : 1;
}

#-------------------------------------------------------------------------------
# FUNCTION	: Function to giveback the filer in 10mode
# INPUT		: Name of the filer
#				: options for giveback like -f
# RETURN    : 1 on success
#-------------------------------------------------------------------------------
sub cf_giveback_10mode ( $;% ) {
    my ($filer, %options) = @_;
    return eval {
        my $fhost = rlm_make_host_to_ontap($filer);
        my $fconn = rlm_make_conn_to_ontap($filer);
        my $do_boot =
          exists($options{boot})
          ? $options{boot}
          : ($fhost->hosttype =~ /^filer-simulator/);
        my $force = (!exists $options{force} || $options{force}) ? '-f' : '';

        # If the cf command fails due to vfiler failing to run, retry the
        # command until sucessfull
            my (undef, undef, $output,$prompt)  =
              say($fconn, "storage failover giveback -fromnode $filer -require-partner-waiting true -override-vetoes true", '-prompt', 'continue');
            logcomment("***************promt is $prompt******");
            if ($prompt =~ /continue/) { 
                 say($fconn,"y",'-blind'); 
 }
        die $@ if $@;
        if ($do_boot) {
            my $partner = $options{partner};
            if (!$partner) {
                die
                  "'partner' option required for giveback() when 'boot' option is true\n";
            }
            my (@partner) = (ref($partner) eq 'ARRAY') ? @$partner : ($partner);
            &boot(@partner) or die $@;
        }
        return 1;
    };
} ## end sub cf_giveback_10mode ( $;% )

1;
