
package platval::service_processor::ontap_backup_policy;

use strict;
use warnings;

use TCD;
use Tharn;
use NACL::APISet;
use NACL::Transit;
use NACL::C::SystemNodeServiceProcessorImage;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
    Clear_SEL
    Clear_EMS
    Set_Debug_Level
    Get_Platform
    Get_Reboot_Time
    Get_Default_Wait_Time
    Events_Search
    Stop_SP_To_Ontap_Heartbeat
    Verify_Debug_Message
    Detect_Heartbeat_Loss
    Verify_SP_Reboot
    Verify_SP_Online
    Check_Sensor_State
    $msg_flag
    $reboot_flag
    $exit_flag
    $curr_timestamp1
    $curr_sec1
);

our $msg_flag = 0;
our $reboot_flag = 0;
our $exit_flag = 0;
our $curr_timestamp1;
our $curr_sec1;

#----------------------------------------------------------------------------
# FUNCTION:     To clear all the SEL logs
# INPUT   :     SP Root Shell API object
# RETURN  :     None
#----------------------------------------------------------------------------

sub Clear_SEL($) {

    my $APISetSPRoot = shift;
    $APISetSPRoot->sel_clear();
}

#----------------------------------------------------------------------------
# FUNCTION:     To clear all the ems messages
# INPUT   :     SP System Shell API object
# RETURN  :     None
#----------------------------------------------------------------------------

sub Clear_EMS($) {

    my $APISetSystemShell = shift;
    $APISetSystemShell->execute_raw_command('command' => "echo > /mroot/etc/log/ems");
}

#----------------------------------------------------------------------------
# FUNCTION:     To set the debug levels
# INPUT   :     CLI API object,filer mode,value
# RETURN  :     None
#----------------------------------------------------------------------------

sub Set_Debug_Level($$$) {

    my ($api_set_obj_cli,$mode,$value) = @_ ;
    my $priv_cmd = "priv set test";
    my $sp_st_mgr_dbg_cmd = "dbg level sp_state_manager=$value";
    my $sp_ipmi_hbt_dbg_cmd = "dbg level sp_ipmi_hbt=$value";
    if ($mode eq "CMode"){
           $priv_cmd = "set -privilege test -confirmations off";
           $sp_st_mgr_dbg_cmd = "run local " . $sp_st_mgr_dbg_cmd;
           $sp_ipmi_hbt_dbg_cmd = "run local " . $sp_ipmi_hbt_dbg_cmd;
    }
    $api_set_obj_cli->execute_raw_command('command' => "$priv_cmd");
    $api_set_obj_cli->execute_raw_command('command' => "$sp_st_mgr_dbg_cmd"); 
    $api_set_obj_cli->execute_raw_command('command' => "$sp_ipmi_hbt_dbg_cmd");
}

#----------------------------------------------------------------------------
# FUNCTION:     To get the platform name
# INPUT   :     SP Root Shell API object
# RETURN  :     Platform name
#----------------------------------------------------------------------------

sub Get_Platform($) {

    my $APISetSPRoot = shift;
    my $platform = $APISetSPRoot->execute_raw_command(command => "cat /var/run/platform");
    tstlog -comment,"Platform is $platform";
    return $platform;
}

#----------------------------------------------------------------------------
# FUNCTION:     To get the reboot time for the specified platform
# INPUT   :     Platform name
# RETURN  :     Reboot time in seconds
#----------------------------------------------------------------------------

sub Get_Reboot_Time($) {
    my $platform = shift;
    my $rbt_time;
    if($platform =~ /Buell|Bimota|Agusta/i) {
        $rbt_time = 180;
    } elsif($platform =~ /Vespa|Alpine|Tioga|Tenaya/i) {
        $rbt_time = 120;
    }
    return $rbt_time;
}

#----------------------------------------------------------------------------
# FUNCTION:     To get default wait time for the specified platform
# INPUT   :     Platform name,String = "packet" or "ipmi"
# RETURN  :     default wait time in seconds
#----------------------------------------------------------------------------

sub Get_Default_Wait_Time($$) {

    my ($platform,$str) = @_;
    my ($msg_wait_sec,$reboot_wait_sec,$online_wait_sec,$total_wait_sec,@default_wait_time) = 0;
    if ($str eq "packet") {     ## for packet heartbeat test case
        if ($platform =~ /Buell|Bimota|Agusta/i) {
            $msg_wait_sec = 540;
            $reboot_wait_sec = 0;
            $online_wait_sec = 180;
        } elsif($platform =~ /Vespa|Alpine|Tioga|Tenaya/i) {
            $msg_wait_sec = 540;
            $reboot_wait_sec = 0;
            $online_wait_sec = 120;
        }
    } elsif ($str eq "ipmi") {   ## for ipmi heartbeat test case
        if ($platform =~ /Buell|Bimota|Agusta/i) {
            $msg_wait_sec = 20;
            $reboot_wait_sec =120;
            $online_wait_sec = 180;
        } elsif($platform =~ /Vespa|Alpine|Tioga|Tenaya/i) {
            $msg_wait_sec = 20;
            $reboot_wait_sec = 0;
            $online_wait_sec = 120;
        }
    }
    $total_wait_sec = ($msg_wait_sec+$reboot_wait_sec+$online_wait_sec);
    push (@default_wait_time,$msg_wait_sec,$reboot_wait_sec,$online_wait_sec,$total_wait_sec);

    return @default_wait_time;
}

#----------------------------------------------------------------------------
# FUNCTION:     Search for the events
# INPUT   :     Event string
# RETURN  :     flag = 0 or flag = 1
#----------------------------------------------------------------------------

sub Events_Search($$){

    my ($APISetSP,$event_string) = @_;
    tstlog -comment, "EVENT - $event_string";
    my @Sel_Events = ("SP Reset Externally","SP Reset Internally","Heartbeat started");
    my $found = 0;
    $APISetSP->api_reconnect_session() if !$APISetSP->api_is_session_alive();
    my $sel_response = $APISetSP->execute_raw_command('command' => "events search $event_string");
    foreach my $res (@Sel_Events) {
        if ($sel_response =~ /$res/) {
            $found++;
            tstlog -pass, "SEL event \"$res\" is generated successfully";
            last;
        }
    }
    return $found;
}

#----------------------------------------------------------------------------
# FUNCTION:     To Stop heartbeat from SP to ONTAP
# INPUT   :     file handler,SP root shell handler,mode,string = "miss" or "loss"
# RETURN  :
#----------------------------------------------------------------------------

sub Stop_SP_To_Ontap_Heartbeat($$$$) {
    
    my ($api_set_obj_cli,$APISetSPRoot,$mode,$string) = @_;
    tstlog -comment, "Deliberately stopping SP heartbeat";
    my $priv_cmd = "priv set test";
    my $autosuppress_cmd = "options ems.autosuppress.enable off";
    my $sp_debug_cmd = "sp debugmode";
    my $ID;
    if ($mode eq "CMode") {
        $priv_cmd = "set -privilege test -confirmations off";
        $autosuppress_cmd = "run local " . $autosuppress_cmd;
        $sp_debug_cmd = "run local " . $sp_debug_cmd;
    }
    $api_set_obj_cli->execute_raw_command('command' => "$priv_cmd");
    $api_set_obj_cli->execute_raw_command('command' => "$autosuppress_cmd");
    $api_set_obj_cli->execute_raw_command('command' => "$sp_debug_cmd");

    $APISetSPRoot->execute_raw_command(command => "sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /etc/watcher.conf");
    if ($string =~ /loss/) {
        $APISetSPRoot->execute_raw_command(command => "sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /mnt/jffs2_rootfs_primary/etc/watcher.conf");
        $APISetSPRoot->execute_raw_command(command => "sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /mnt/jffs2_rootfs_backup/etc/watcher.conf");
        $APISetSPRoot->execute_raw_command(command => "sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /etc/init.d/netapp_final_initscript");
        $APISetSPRoot->execute_raw_command(command => "sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /mnt/jffs2_rootfs_primary/etc/init.d/netapp_final_initscript");
        $APISetSPRoot->execute_raw_command(command => "sed -i \'s_/usr/local/bin/r2fbeat_#&_\' /mnt/jffs2_rootfs_backup/etc/init.d/netapp_final_initscript");    
    }
    
    $ID = $APISetSPRoot->execute_raw_command(command => "ps - ef | grep watcher");
    if ($ID =~/(\d*)[\s]?root[\s]*(\d*)[\s]?S[\s]*\/usr\/local\/bin\/watcher/i) {
        $APISetSPRoot->execute_raw_command(command => "kill -9 $1");
    } else {
        tstlog -FATAL, "Failed to kill the watcher PID";
        return 0;
    }
    $ID = $APISetSPRoot->execute_raw_command(command => "ps - ef |grep r2fbeat");
    if ($ID =~/(\d*)[\s]?root[\s]*(\d*)[\s]?S[\s]*\/usr\/local\/bin\/r2fbeat/i) {
        $APISetSPRoot->execute_raw_command(command => "kill -9 $1");
    } else {
        tstlog -FATAL, "Failed to kill the r2fbeat PID";
        return 0;
    }
    return 1;
}

#----------------------------------------------------------------------------
# FUNCTION:     To verify the debug messages
# INPUT   :     Transit object handler,platform name
# RETURN  :     1-pass, 0-fail
#----------------------------------------------------------------------------

sub Verify_Debug_Message($$) {

    my ($Transit_obj,$platform) = @_;
    tstlog -comment, "Verify the debug message on the console";
    my $debug_msg_BBA = "Resetting reboot timer to 180 secs";
    my $debug_msg_vespa = "Resetting reboot timer to 120 secs";
    my $out;
    $Transit_obj->get_state(timeout => 120,append_output_to => \$out);
    if (($platform =~ /Buell|Bimota|Agusta/i) && ($out =~ /$debug_msg_BBA/ )) {
        tstlog -pass, "Appropriate debug message \"$debug_msg_BBA\" for the platform " .
                      "$platform is displayed successfully";
    } elsif (($platform =~ /Vespa|Alpine|Tioga|Tenaya/i) && ($out =~ /$debug_msg_vespa/)) {
        tstlog -fail, "Appropriate debug message \"$debug_msg_vespa\" for the platform " .
                      "$platform is displayed successfully";
        return 0;
    } else {
        tstlog -fail, "Debug message is not displayed";
        return 0;
    }
    return 1;
}

#----------------------------------------------------------------------------
# FUNCTION:     To detect whether ONTAP detects heartbeat loss
# INPUT   :     Systemshell handler,ems message,default wait time,current time in seconds
# RETURN  :     Current time in seconds
#----------------------------------------------------------------------------

sub Detect_Heartbeat_Loss($$$$) {

    my ($APISetSystemShell,$ems_msg,$msg_wait_sec,$sec0) = @_;
    my $msg_received_in_sec = 0;
    $curr_timestamp1 = $APISetSystemShell->execute_raw_command('command' => "date");
    $curr_timestamp1 =~  m/.*\w+\s+\w+\s+\d+\s+(\d+):(\d+):(\d+)\s+\w+\s+\d+.*/i;
    $curr_sec1 = ($1*3600)+($2*60)+($3);
    if ($msg_flag == 0) {
        my $ems_search = $APISetSystemShell->execute_raw_command('command' => "cat /mroot/etc/log/ems | grep \"$ems_msg\"");  
        if ($ems_search =~ /$ems_msg/) { 
            $curr_timestamp1 = $APISetSystemShell->execute_raw_command('command' => "date");
            $curr_timestamp1 =~  m/.*\w+\s+\w+\s+\d+\s+(\d+):(\d+):(\d+)\s+\w+\s+\d+.*/i; 
            $curr_sec1 = ($1*3600)+($2*60)+($3);
            $msg_received_in_sec = ($curr_sec1 - $sec0);
            if(($msg_received_in_sec >= ($msg_wait_sec-5)) && ($msg_received_in_sec <= ($msg_wait_sec+60))) {
                $msg_flag = 1;
                tstlog -pass, "ONTAP detected heartbeat loss from SP successfully in approximately $msg_received_in_sec secs";
                return $curr_sec1;
            } else {
                tstlog -fail, "ONTAP did not detect heartbeat loss from SP";
                return 0;
            }
        } 
    sleep(10);
    }
    return $curr_sec1;
}

#----------------------------------------------------------------------------
# FUNCTION:     To verify if SP reboots
# INPUT   :     Node,Systemshell object handler,reboot time,current time in seconds
# RETURN  :     Current time in seconds
#----------------------------------------------------------------------------

sub Verify_SP_Reboot ($$$$) {
 
    my ($node,$APISetSystemShell,$reboot_wait_sec,$sec1) = @_;
    my $rebooted_time_in_sec = 0;
    my ($response,$sp_status);
    $curr_timestamp1 = $APISetSystemShell->execute_raw_command('command' => "date");
    $curr_timestamp1 =~  m/.*\w+\s+\w+\s+\d+\s+(\d+):(\d+):(\d+)\s+\w+\s+\d+.*/i;
    $curr_sec1 = ($1*3600)+($2*60)+($3);
    $response = NACL::CS::SystemNodeServiceProcessor->fetch(command_interface => $node);
    $sp_status = $response->status();
    tstlog -comment, "SP Status : $sp_status";
    if (($sp_status =~ /rebooting/i) && ($reboot_flag == 0)) {
        $curr_timestamp1 = $APISetSystemShell->execute_raw_command('command' => "date");
        $curr_timestamp1 =~  m/.*\w+\s+\w+\s+\d+\s+(\d+):(\d+):(\d+)\s+\w+\s+\d+.*/i;
        $curr_sec1 = ($1*3600)+($2*60)+($3);
        $rebooted_time_in_sec = ($curr_sec1 - $sec1);
        if ($reboot_wait_sec == 0) {
            if ($rebooted_time_in_sec <= 60) {
                tstlog -pass, "ONTAP rebooted SP successfully";
                $reboot_flag = 1;
                return $curr_sec1;
            }
        } else {
            #if (($rebooted_time_in_sec >= $reboot_wait_sec) && ($rebooted_time_in_sec <= ($reboot_wait_sec+5))) {
            if (($rebooted_time_in_sec >= $reboot_wait_sec)) {
                tstlog -pass, "ONTAP waited approximately for $rebooted_time_in_sec secs before rebooting SP";
                $reboot_flag = 1;
                return $curr_sec1; 
            } 
        } 
    }
    return $curr_sec1;
}

#----------------------------------------------------------------------------
# FUNCTION:     To verify if SP came online
# INPUT   :     Node,Systemshell object handler,time taken for SP to come online,
#               current time in seconds
# RETURN  :     Current time in seconds
#----------------------------------------------------------------------------

sub Verify_SP_Online ($$$$) {

    my ($node,$APISetSystemShell,$online_wait_sec,$sec2) = @_;
    my $online_time_in_sec = 0;
    my ($response,$sp_status);
    $curr_timestamp1 = $APISetSystemShell->execute_raw_command('command' => "date");
    $curr_timestamp1 =~  m/.*\w+\s+\w+\s+\d+\s+(\d+):(\d+):(\d+)\s+\w+\s+\d+.*/i;
    $curr_sec1 = ($1*3600)+($2*60)+($3);
    $response = NACL::CS::SystemNodeServiceProcessor->fetch(command_interface => $node);
    $sp_status = $response->status();
    tstlog -comment, "SP Status : $sp_status";
    if (($sp_status =~ /online/i) && ($reboot_flag == 1)) {
        $curr_timestamp1 = $APISetSystemShell->execute_raw_command('command' => "date");
        $curr_timestamp1 =~  m/.*\w+\s+\w+\s+\d+\s+(\d+):(\d+):(\d+)\s+\w+\s+\d+.*/i;
        $curr_sec1 = ($1*3600)+($2*60)+($3);
        $online_time_in_sec = ($curr_sec1 - $sec2);
        if ($online_time_in_sec <= $online_wait_sec) {
            tstlog -pass, "SP came up online successfully within $online_time_in_sec secs";
            $reboot_flag = 1;
            $exit_flag = 1;
            return $curr_sec1;
        }
    }    
    return $curr_sec1;
}

#----------------------------------------------------------------------------
# FUNCTION:     To check the sensor state
# INPUT   :     Systemshell object handler,sensor name
# RETURN  :     Parsed output of the command "envcmdtool get_state -n <sensor>"
#----------------------------------------------------------------------------

sub Check_Sensor_State($$) {
  
    my ($APISetSystemShell,$sensor) = @_;
    $sensor = "\"" . $sensor . "\"";
    $APISetSystemShell->execute_raw_command( command => "sudo /bin/csh");
    my $response = $APISetSystemShell->envcmdtool_get_state('name' => $sensor);
    my $parsed_output = $response->get_parsed_output();
    
    return $parsed_output;
}

1;
