# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/platval/service_processor/rlm_ipv6.pm#5 $
#
#       Copyright (c) 2009 Network Appliance, Inc.
#       All Rights reserved
#
## @summary Routines for validating RLM/SP ipv4 and ipv6.
##
## @description
##    This library contains individual routiens for ONTAP, RLM/SP and Boot Loader
##    for analysing various tasks such as rlm/sp setup, rlm/sp status, rlm/sp status -d, 
##    ping, ping6, ssh interactive mode, ssh non-interactive mode, wget, etc.
##
## Sub-Routines:
## 1. rlm_find_rlm_sp_prompt
##	To find filer is having RLM/SP prompt with the help of tharnhost file
##
## 2. rlm_ontap_login
##	To boot filer into ONTAP prompt either from LOADER or ONTAP login prompt
##
## 3. rlm_load_tharnhost_data
##	To load values of RLM IPv4 and IPv6 IP's from tharnhost data
##
## 4. rlm_load_ipv4_ipv6_config_cases
##	To load ipv4/ipv6 configuration testcase to a hash. If ONTAP don't have IPv6 support then it will get loaded only IPv4 cases
##
## 5. rlm_verify_ipv4_ipv6_testcase_validity
##	To verify the validity of the given test case / test state
##
## 6. rlm_is_ipv6_enable
##	To enable/disable ip.v6 option in ONTAP
##
## 7. rlm_is_ipv6_ra_enable
##	To enable/disable ip.v6.ra_enable option in ONTAP
##
## 8. rlm_is_valid_ipv6_image
##	To verify valid rlm firmware image which supports both ipv4 and ipv6
##
## 9. rlm_verify_filer_state
##	To boot filer into ONTAP/LOADER prompt either from LOADER/ONTAP/SP
##
## 10. rlm_setup
##	To setup rlm in ONTAP and LOADER
##
## 11. rlm_verify_rlm_status
##	To verify "rlm status" output in ONTAP/SP
##
## 12. rlm_extract_rlm_status_info
##      To extract all fields and corresponding values from "rlm status" output in ONTAP
##
## 13. rlm_verify_rlm_status_values
##	To verify "rlm status" values with testcase/teststate expected result
##
## 14. rlm_verify_rlm_status_d
##	To verify "rlm status -d" output in ONTAP and RLM/SP
##
## 15. rlm_verify_rlm_status_rlm
##	To verify "rlm status" output in RLM
##
## 16. rlm_reboot_rlm
##	To reboot rlm/SP either from ONTAP/LOADER
##
## 17. rlm_ping
##	To ping IPv4/IPv6 address using various options such as: sudo, -a, -c, -f, -l, -s
##
## 18. rlm_wget
##	To download files from webhosts: http://<ipv4-address> and http://<ipv6-address>
##
## 19. rlm_make_conn_to_rlm
##	To connect to RLM/SP either as naroot or root user
##
## 20. rlm_ssh_interactive_mode
##	To SSH into RLM/SP with all IPv4 and IPv6 addresses
##
## 21. rlm_ssh_non_interactive_mode
##	To SSH into RLM with all IPv4 and IPv6 addresses by non-interactive mode: IPv4 option: -v, IPv6 option: -v -6
##
## 22. rlm_validation_items
##	To validate all the items suchs as: rlm status, reboot, rlm status -d, ssh, wget, ping 
##
## 23. rlm_make_conn_to_ontap
##	To create console connection for filer either from FILER variable or by using existing connection handler
##
## 24. rlm_make_host_to_ontap
##	To create host connection for filer either from FILER variable or by using existing connection handler
##
## 25. rlm_ctrl_g_login
##	To login to SP via Ctrl+g option
##
## 25. rlm_ctrl_g_logout
##	To logout from SP incase if user logins via Ctrl+g option
##
## 26. rlm_get_test_case_execution_order
##	To get the Random / sequence / series / range of test cases list.
##
## 27. Validation_Items
##      Execute the commands 'sp status -d' and 'events newest 100'.
##
## 28. get_filer_state
##      Get the filer state and return the state.
##
## 29. Get_SP_ROOT_IPMI_Result
##	Return the result after executing the ipmitool command
##
## 30. Get_SP_IPMI_Result
##     Return the result after executing the sp ipmi raw command
##
## 31. Reboot_Time
##     Return the defaukt reboot time for particular platform
##
## @synopsis
##	use rlm_ipv6;
##
## @keywords
##      rlm setup, rlm status, ping, ping6, wget, loader, ontap, ssh, reboot, ipv6, ipv4, ra, router
##
## @author jkonjeti@netapp.com, ravikira@netapp.com, divyab@netapp.com, dl-rlm_ipv6_validation@netapp.com, dl-sp-validation@netapp.com
##
######################################################################################################################################

require "tharnit_lib.pl";
undef $CLIENT;
undef $CLIENT_CONN;
use boot qw(transit_to reboot halt boot reboot_to);
use NACL::Transit;

use vars qw/$LOADER %test_states $RLM $RLM_PROMPT $NO_IPv6 $IPv4_ONLY $RLM_IPV4_IP $RLM_IPV4_MASK $RLM_IPV4_GATEWAY $RLM_IPV6_IP $RLM_IPV6_PREFIX $RLM_IPV6_GATEWAY $RLM_IPV6_ROUTER $RLM_WGET_IPV4_HTTP_PATH $RLM_WGET_IPV6_HTTP_PATH $rlm_state_title $rlm_ipv4_dhcp $rlm_ipv4_ip $rlm_ipv4_netmask $rlm_ipv4_gateway $rlm_enable_ipv6 $rlm_assign_ipv6_global_addr $rlm_ipv6_global_ip $rlm_ipv6_subnet_prefix $rlm_ipv6_gateway $rlm_ipv6_router $ipv4_ip $ipv6_global_ip $ipv6_router_ip $ipv6_link_local_ip $ipv6_gateway $ipv6_status @prompts @prompts_sp $fw_version $dhcp_status $ipv4_mask $ipv4_gateway $ipv6_prefix $ssh_index $ontap_version $FILER_MODE/;

param ('RLM_PROMPT',   '-default',  "SP");
param ('RLM_WGET_IPV4_HTTP_PATH',   '-default',  "http://10.57.61.40/~jkonjeti/hop_release_rlm_images/maytag_kern.2d"); 
param ('RLM_WGET_IPV6_HTTP_PATH',   '-default',  "http://fd20:8b1e:b255:4164:214:5eff:fe95:24d8/files/maytag_kern.2d");
param ('CLIENT',   '-default',  "localhost");
#param ('NO_IPv6',   '-default',  "1");

## @name rlm_find_rlm_sp_prompt ($;$)
## @example
## rlm_find_rlm_sp_prompt($FIELR);
## rlm_find_rlm_sp_prompt($FIELRA, $FILERB);
## @description
## This function will finds filers remote module with the help of tharnhost file. 
## And also it will create global varialbes (@prompts & @prompts_sp) which will have respective prompts data.
sub rlm_find_rlm_sp_prompt ($;$) {
	my $FILER = shift @_;
	my $h = rlm_make_host_to_ontap($FILER);
	$RLM_PROMPT = $h->{'PROMPT_RLM'};
#print "RLM_PROMPT = $RLM_PROMPT\n";

        if ($RLM_PROMPT =~ /rlm|sp/i) {
                $RLM_PROMPT = "\U$&\E"; #Upper Case. i.e RLM / SP
                $RLM = "\L$&\E";        #Lower Case. i.e rlm / sp
        } else {
                #logresult ('SCRIPT', "To run this script \'PROMPT_RLM\' field should have value either \'RLM\' or\'SP\' in the filer tharnhost($FILER) file");
                $RLM_PROMPT = "SP";
                $RLM = "sp";
        }

#print "RLM_PROMPT = $RLM_PROMPT\n";

	@prompts = ('LOADER(-\w+)?\*?>','CFE(-\w+)?\*?>',$FILER.'(::[^>#]*)?\*?[>#]',$FILER.'.*');
	#@prompts_sp = ($RLM_PROMPT. '.*>');
	@prompts_sp = ("$RLM_PROMPT $FILER" . '\*?>', $RLM_PROMPT. '\*?>');

	my ($FILERB, $hB, $RLM_PROMPTB);
	if ($_[0]) {
	   $FILERB = shift @_;
	   $hB = rlm_make_host_to_ontap($FILERB);
#	   $RLM_PROMPTB = $hB->{'PROMPT_RLM'};
           $RLM_PROMPTB = $RLM_PROMPT;

	   if ($RLM_PROMPT !~ /$RLM_PROMPTB/i) {
		logresult ('SCRIPT', "To run this script \'PROMPT_RLM\' field should have value either \'RLM\' or\'SP\' in the filer tharnhost($FILERB) file");
	   }
	   push @prompts, $FILERB.'(::[^>#]*)?(\(takeover\))?\*?[>#]', $FILER.'(::[^>#]*)?(\(takeover\))?\*?[>#]';
	   push @prompts_sp, "$RLM_PROMPT $FILERB" . '\*?>';
	}
}

## @name rlm_ontap_login ($;$$)
## @example
## rlm_ontap_login ($FILER);
## rlm_ontap_login ($FILER, $Uname, $Pwd);
## @description
## If the filer is at CFE/BIOS/LOADER firmware prompt or ontap login prompt, then it will boot it into ONTAP. 
sub rlm_ontap_login ($;$$) {
        my $FILER = shift;
        my $Uname = $_[0] ? $_[0] : 'root';
        my $Pwd = $_[1] ? $_[1] : '';

        my $fc = rlm_make_conn_to_ontap($FILER);
	my $h = rlm_make_host_to_ontap($FILER);

	logcomment ("Bringing filer: $FILER to ONTAP prompt");
	if (!transit_to($h, 'UP', username=>$Uname, password=>$Pwd)) {
		logresult ('FATAL', "Incorrect login credentials: Login: $Uname, Password: $Pwd");
	}
	sleep 30;

	eval { (undef, undef, undef, $prompt) = say($fc," ", -swallow, -prompt, @prompts, '-timeout', 10) };
	$prompt = "" if ($@);
	return $prompt;

#	if ($prompt =~ /$FILER/) {
#		return $prompt;
#	}
}

## @name rlm_load_tharnhost_data ($)
## @example
## rlm_load_tharnhost_data ($FILER);
## @description
## This will load the tharnhost data (RLM_IP, RLM_MASK, RLM_GATEWAY, RLM_IPV6_IP, RLM_IPV6_PREFIX, 
## RLM_IPV6_GATEWAY, RLM_IPV6_ROUTER, RLM_WGET_IPV4_HTTP_PATH, RLM_WGET_IPV6_HTTP_PATH) into global variables.
sub rlm_load_tharnhost_data ($) {
	my $FILER = shift @_;
	my $h = rlm_make_host_to_ontap($FILER);

	#Pruthvi getting the Mode (7M or CM) of the FILER
	$FILER_MODE = $h->{'hosttype'};

	$RLM_IPV4_IP = $h->{'RLM_IP'};
	$RLM_IPV4_MASK = $h->{'RLM_MASK'};
	$RLM_IPV4_GATEWAY = $h->{'RLM_GATEWAY'};
	$RLM_IPV6_IP = $h->{'RLM_IPV6_IP'};

#print "RLM_IPV6_IP = $RLM_IPV6_IP\n";

        if ($RLM_IPV6_IP =~ /\S+/) {
        logcomment ("RLM_IPV6_IP is loaded from tharnhost file");
        } else {
                $RLM_IPV6_IP = "fd20:8b1e:b255:c000::000";
        }

#print "RLM_IPV6_IP = $RLM_IPV6_IP\n";

	$RLM_IPV6_PREFIX = $h->{'RLM_IPV6_PREFIX'};

#print "RLM_IPV6_PREFIX = $RLM_IPV6_PREFIX\n";

        if ($RLM_IPV6_PREFIX =~ /\d+/) {
        logcomment ("RLM_IPV6_PREFIX is loaded from tharnhost file");
        } else {
                $RLM_IPV6_PREFIX = "64";
        }

#print "RLM_IPV6_PREFIX = $RLM_IPV6_PREFIX\n";

	$RLM_IPV6_GATEWAY = $h->{'RLM_IPV6_GATEWAY'};

#print "RLM_IPV6_GATEWAY = $RLM_IPV6_GATEWAY\n";

        if ($RLM_IPV6_GATEWAY =~ /\S+/) {
        logcomment ("RLM_IPV6_GATEWAY is loaded from tharnhost file");
        } else {
                $RLM_IPV6_GATEWAY = "fd20:8b1e:b255:c000::0";
        }

#print "RLM_IPV6_GATEWAY = $RLM_IPV6_GATEWAY\n";

	$RLM_IPV6_ROUTER = $h->{'RLM_IPV6_ROUTER'};

#print "RLM_IPV6_ROUTER = $RLM_IPV6_ROUTER\n";

        if ($RLM_IPV6_ROUTER =~ /\S+/) {
        logcomment ("RLM_IPV6_ROUTER is loaded from tharnhost file");
        } else {
                $RLM_IPV6_ROUTER = "fd20:8b1e:b255:c099:2a0:98ff:fe13:9ffa";
        }
	
#print "RLM_IPV6_ROUTER = $RLM_IPV6_ROUTER\n";

#	$RLM_WGET_IPV4_HTTP_PATH = $h->{'RLM_WGET_IPV4_HTTP_PATH'};
#	$RLM_WGET_IPV6_HTTP_PATH = $h->{'RLM_WGET_IPV6_HTTP_PATH'};
#	$CLIENT = $h->{'CLIENT'};

#	unless ($RLM_IPV4_IP && $RLM_IPV4_MASK && $RLM_IPV4_GATEWAY && $RLM_IPV6_IP && $RLM_IPV6_PREFIX && $RLM_IPV6_GATEWAY && $RLM_IPV6_ROUTER && $RLM_WGET_IPV4_HTTP_PATH && $RLM_WGET_IPV6_HTTP_PATH) {
#		logresult ('SCRIPT', "To run this script, filer tharnhost file should have the following fileds: RLM_IP, RLM_MASK, RLM_GATEWAY, RLM_IPV6_IP, RLM_IPV6_PREFIX, RLM_IPV6_GATEWAY, RLM_IPV6_ROUTER, RLM_WGET_IPV4_HTTP_PATH, RLM_WGET_IPV6_HTTP_PATH");
#	}
	unless ($RLM_IPV4_IP && $RLM_IPV4_MASK && $RLM_IPV4_GATEWAY) {
		my $pre_requisites = 'To run this script, filer tharnhost file should have the following fileds: RLM_IP, RLM_MASK, RLM_GATEWAY';
		unless ($NO_IPv6) {
			$pre_requisites .= 'RLM_IPV6_IP, RLM_IPV6_PREFIX, RLM_IPV6_GATEWAY, RLM_IPV6_ROUTER, RLM_WGET_IPV4_HTTP_PATH, RLM_WGET_IPV6_HTTP_PATH';
		}
		logresult ('SCRIPT', "$pre_requisites");
	}

	unless ($NO_IPv6) {
		unless ($RLM_IPV6_IP && $RLM_IPV6_PREFIX && $RLM_IPV6_GATEWAY && $RLM_IPV6_ROUTER && $RLM_WGET_IPV4_HTTP_PATH && $RLM_WGET_IPV6_HTTP_PATH) {
			logresult ('SCRIPT', "To run this script, filer tharnhost file should have the following fileds: RLM_IPV6_IP, RLM_IPV6_PREFIX, RLM_IPV6_GATEWAY, RLM_IPV6_ROUTER, RLM_WGET_IPV4_HTTP_PATH, RLM_WGET_IPV6_HTTP_PATH");
        	}
	}
}

## @name rlm_load_ipv4_ipv6_config_cases ($)
## @example
## rlm_load_ipv4_ipv6_config_cases ($FILER);
## @description
## This will load all the ipv4/ipv6 testcases into an hash, i.e %test_states.
## If ONTAP don't have IPv6 support then it loads only IPv4 testcases.
sub rlm_load_ipv4_ipv6_config_cases ($) {
    my $FILER = shift @_;

    rlm_load_tharnhost_data($FILER);

    %test_states = (
        1 => ["State 1. $RLM_PROMPT with no IPv4 or IPv6 addresses", "no", '0.0.0.0', '0.0.0.0', '0.0.0.0', "no"],
        2 => ["State 2. $RLM_PROMPT with static IPv4 address only", "no", $RLM_IPV4_IP, $RLM_IPV4_MASK, $RLM_IPV4_GATEWAY, "no"],
        3 => ["State 3. $RLM_PROMPT with dhcp IPv4 address only", "yes", '', '', '', "no"],
    );

    unless ($NO_IPv6) {
      %test_states = (%test_states,
        4 => ["State 4. $RLM_PROMPT with IPv6 address only", "no", '0.0.0.0', '0.0.0.0', '0.0.0.0', "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY],
        5 => ["State 5. $RLM_PROMPT with static IPv4 and IPv6 addresses", "no", $RLM_IPV4_IP, $RLM_IPV4_MASK, $RLM_IPV4_GATEWAY, "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY],
        6 => ["State 6. $RLM_PROMPT with dhcp IPv4 and IPv6 addresses", "yes", '', '', '', "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY],
        7 => ["State 7. $RLM_PROMPT with static IPv4 and no Global IPv6 address", "no", $RLM_IPV4_IP, $RLM_IPV4_MASK, $RLM_IPV4_GATEWAY, "yes", "no"],
        8 => ["State 8. $RLM_PROMPT with DHCP IPv4 and no Global IPv6 address", "yes", '', '', '', "yes", "no"],
	9 => ["State 9. $RLM_PROMPT with no IPV4 and no Global IPV6 address", "no", '0.0.0.0', '0.0.0.0', '0.0.0.0', "yes", "no"],
	10 => ["State 10. $RLM_PROMPT with no IPV4 and Duplicate the RA address to Global IPV6 address", "no", '0.0.0.0', '0.0.0.0', '0.0.0.0', "yes", "yes", $RLM_IPV6_ROUTER, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY],
	11 => ["State 11. $RLM_PROMPT with static IPV4 and Duplicate the RA address to Global IPV6 address", "no", $RLM_IPV4_IP, $RLM_IPV4_MASK, $RLM_IPV4_GATEWAY, "yes", "yes", $RLM_IPV6_ROUTER, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY],
	12 => ["State 12. $RLM_PROMPT with DHCP IPV4 and Duplicate the RA address to Global IPV6 address", "yes", '', '', '', "yes", "yes", $RLM_IPV6_ROUTER, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY],
	13 => ["State 13. $RLM_PROMPT with no IPV4 and Duplicate the RA address to Gateway IPV6 address", "no", '0.0.0.0', '0.0.0.0', '0.0.0.0', "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX, $RLM_IPV6_ROUTER],
	14 => ["State 14. $RLM_PROMPT with static IPV4 and Duplicate the RA address to Gateway IPV6 address", "no", $RLM_IPV4_IP, $RLM_IPV4_MASK, $RLM_IPV4_GATEWAY, "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX, $RLM_IPV6_ROUTER],
	15 => ["State 15. $RLM_PROMPT with DHCP IPV4 and Duplicate the RA address to Gateway IPV6 address", "yes", '', '', '', "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX, $RLM_IPV6_ROUTER],
	16 => ["State 16. $RLM_PROMPT with no IPV4 and Duplicate the RA address to Global & Gateway IPV6 addresses", "no", '0.0.0.0', '0.0.0.0', '0.0.0.0', "yes", "yes", $RLM_IPV6_ROUTER, $RLM_IPV6_PREFIX, $RLM_IPV6_ROUTER],
	17 => ["State 17. $RLM_PROMPT with static IPV4 and Duplicate the RA address to Global & Gateway IPV6 addresses", "no", $RLM_IPV4_IP, $RLM_IPV4_MASK, $RLM_IPV4_GATEWAY, "yes", "yes", $RLM_IPV6_ROUTER, $RLM_IPV6_PREFIX, $RLM_IPV6_ROUTER],
	18 => ["State 18. $RLM_PROMPT with DHCP IPV4 and Duplicate the RA address to Global & Gateway IPV6 address", "yes", '', '', '', "yes", "yes", $RLM_IPV6_ROUTER, $RLM_IPV6_PREFIX, $RLM_IPV6_ROUTER],
	19 => ["State 19. $RLM_PROMPT with no IPV4 and Duplicate Global address to Gateway IPV6 address", "no", '0.0.0.0', '0.0.0.0', '0.0.0.0', "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX,  $RLM_IPV6_IP],
	20 => ["State 20. $RLM_PROMPT with static IPV4 and Duplicate Global address to Gateway IPV6 address", "no", $RLM_IPV4_IP, $RLM_IPV4_MASK, $RLM_IPV4_GATEWAY, "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX,  $RLM_IPV6_IP],
	21 => ["State 21. $RLM_PROMPT with DHCP IPV4 and Duplicate Global address to Gateway IPV6 address", "yes", '', '', '', "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX,  $RLM_IPV6_IP],
	22 => ["State 22. $RLM_PROMPT with no IPV4 and Duplicate Gateway IPv6 address to Global IPv6 address", "no", '0.0.0.0', '0.0.0.0', '0.0.0.0', "yes", "yes", $RLM_IPV6_GATEWAY, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY],
	23 => ["State 23. $RLM_PROMPT with static IPV4 and Duplicate Gateway IPv6 address to Global IPv6 address", "no", $RLM_IPV4_IP, $RLM_IPV4_MASK, $RLM_IPV4_GATEWAY, "yes", "yes", $RLM_IPV6_GATEWAY, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY],
	24 => ["State 24. $RLM_PROMPT with DHCP IPV4 and Duplicate Gateway IPv6 address to Global IPv6 address", "yes", '', '', '', "yes", "yes", $RLM_IPV6_GATEWAY, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY],
	25 => ["State 25. $RLM_PROMPT with no IPv4 and no Router IPv6 address", "no", '0.0.0.0', '0.0.0.0', '0.0.0.0', "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY, "off"],
	26 => ["State 26. $RLM_PROMPT with static IPv4 and no Router IPv6 address", "no", $RLM_IPV4_IP, $RLM_IPV4_MASK, $RLM_IPV4_GATEWAY, "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY, "off"],
	27 => ["State 27. $RLM_PROMPT with DHCP IPv4 and no Router IPv6 address", "yes", '', '', '', "yes", "yes", $RLM_IPV6_IP, $RLM_IPV6_PREFIX, $RLM_IPV6_GATEWAY, "off"],
	28 => ["State 28. $RLM_PROMPT with no IPv4 and no Global and no Router IPv6 addresses", "no", '0.0.0.0', '0.0.0.0', '0.0.0.0', "yes", "no", '', '', '', "off"],
	29 => ["State 29. $RLM_PROMPT with static IPv4 and no Global and no Router IPv6 addresses", "no", $RLM_IPV4_IP, $RLM_IPV4_MASK, $RLM_IPV4_GATEWAY, "yes", "no", '', '', '', "off"],
	30 => ["State 30. $RLM_PROMPT with dhcp IPv4 and no Global and no Router IPv6 addresses", "yes", '', '', '', "yes", "no", '', '', '', "off"],
     );
   }
}

## @name rlm_verify_ipv4_ipv6_testcase_validity($)
## @example
## rlm_verify_ipv4_ipv6_testcase_validity($TEST_STATE);
## @description
## This will validate whether the given testcase/teststate no. is valid or not.
sub rlm_verify_ipv4_ipv6_testcase_validity($) {
   my $TEST_STATE = shift @_;
   my $no_ipv6;
   if ($NO_IPv6) {
        $no_ipv6 = $NO_IPv6;
        undef ($NO_IPv6);
        rlm_load_ipv4_ipv6_config_cases($FILER);
   }

   if ($TEST_STATE > 0 && $TEST_STATE <= scalar(keys %test_states)) {
        logresult ('NA', "Case: $TEST_STATE :- " . $test_states{$TEST_STATE}->[0] . " can't be execuated as \"$no_ipv6\" ");
   } else {
        logresult ('NA', "Invalid test case/state: $TEST_STATE. See the tharndoc for more info");
   }
}

## @name rlm_is_ipv6_enable ($)
## @example
## rlm_is_ipv6_enable($FILER);
## @description
## Enables/disables ipv6 feature (ip.v6 option) in ONTAP
sub rlm_is_ipv6_enable ($) {
	my $FILER = shift @_;
	my $fc = rlm_make_conn_to_ontap($FILER);

	logcomment ("Enabling ipv6 option ...");
	my(undef, undef, $output) = say($fc, "options ip.v6.enable");
	unless($output =~ /[\w\.]+\s+on/) {
           my(undef, undef, $output1) = say($fc, "options ip.v6.enable on");
	   my(undef, undef, $output2) = say($fc, " ");	# unless ($output =~ /^[\w\.]+\s+on/m);
	   my $output = $output1 . $output2;

	   if ($output =~ /IPv6 is not supported in Data ONTAP 8\.0/) {
		$NO_IPv6=$&;
		$IPv4_ONLY = 3;	#The last ipv4 testcase number from the hash %test_states
		logresult('WARN', "IPv6 feature will not be validated as \"$&\" release");
	   } else {
		my(undef, undef, $output) = say($fc, "options ip.v6.enable");

		unless($output =~ /[\w\.]+\s+on/) {
			logresult('CONFIG', "Could not find ipv6 feature on ONTAP -- Impossible to continue");
		}
	   }
	} else {
	   logcomment("Option IPV6 already enabled ...");
	}
}

## @name rlm_is_ipv6_ra_enable($;$)
## @example
## rlm_is_ipv6_ra_enable ($FILER);
## rlm_is_ipv6_ra_enable ($FILER, 'off'); 
## @description
## Enables/disables router address (ip.v6.ra_enable option) in ONTAP.
sub rlm_is_ipv6_ra_enable ($;$) {
	my $FILER = shift @_;
	my $flag = shift @_ || 'on';
	
	my $fc = rlm_make_conn_to_ontap($FILER);

	if ($flag eq 'on') {
        	logcomment ("Enabling IPV6 router assigned address ...");
	} else {
		logcomment ("Disabling IPV6 router assigned address ...");
	}
        my(undef, undef, $output) = say($fc, "options ip.v6.ra_enable");
	my ($val) = $output =~ /ip\.v6\.ra_enable\s+(on|off)/;

	unless ($val) {
		logresult('CONFIG', "Could not find option to Enable/Disable Router Assigned Address feature on ONTAP -- Impossible to continue")
	}

	if ($flag eq $val) {
		logcomment ("IPv6 Router Assigned Address already powered \U$val\E");
	} else {
		say($fc, "options ip.v6.ra_enable $flag");
		my(undef, undef, $output) = say($fc, "options ip.v6.ra_enable");
		unless ($output =~ /ip\.v6\.ra_enable\s+$flag/) {
		   logresult('FATAL', "Option to enable/disalbe Router Assigned Address not working properly on ONTAP");
		}
	}
}

## @name rlm_is_valid_ipv6_image ($)
## @example
## rlm_is_valid_ipv6_image ($FILER);
## @description 
## Verifies RLM is having valid firmware image which supports IPv6 feature or not. This function is RLM specific.
sub rlm_is_valid_ipv6_image ($) {
	my $FILER = shift @_;
	my $fc = rlm_make_conn_to_ontap($FILER);

	logcomment ("Validating the existed $RLM_PROMPT FW image");
	say($fc, "priv set test", -prompt, @prompts, -timeout, 300) if($LOADER);
       my(undef, undef, $output) = say($fc, "$RLM status", -timeout, 300);
#	my(undef, undef, $output) = say($fc, "$RLM status", -prompt, @prompts, -timeout, 300);
	if ($output =~ /Firmware\sVersion\:\s+(4\.[\d\w]+)/) {
		logcomment ("$RLM_PROMPT is having supported ipv6 image : $1");
	} else {
		logresult ('CONFIG', "To run this script $RLM should have 4.x image");
	}
}

## @name rlm_verify_filer_state ($;$$)
## @example
## rlm_verify_filer_state ($FILER)
## rlm_verify_filer_state ($FILER, $Uname, $Pwd)
## @description
## Verifies filer state whether its ONTAP/LOADER, and will take necessary action.
sub rlm_verify_filer_state ($;$$) {
	my $FILER = shift @_;
	my $Uname = ($_[0]) ? $_[0] : 'root';
	my $Pwd = ($_[1]) ? $_[1] : '';

	my $h = rlm_make_host_to_ontap($FILER);
	my $fc = rlm_make_conn_to_ontap($FILER);
################################################################ Added by deepakkb
    my @prompt_var = @{$h->prompt_norm()};
    @{$h->prompt_norm()} = undef ;
    $obj_tr = NACL::Transit->new(name => $FILER);
    tstlog -comment, "Check for the filer state and make it UP";
    my $Current_state = $obj_tr->get_state();
    tstlog -comment, "Current state is $Current_state ";
    if($Current_state ne 'CLI')
    {
	    $obj_tr->change_state(to => 'CLI');
    } 
    @{$h->prompt_norm()} =  @prompt_var;
#################################################################
	logcomment ("Checking state of the FILER: $FILER");
	say($fc,"\cc", -swallow, -blind, -noprompt);
	push @prompts_sp, '#';

	my $after, $prompt;
	while (1) {
		(undef, undef, $after, $prompt) = say($fc," ", -swallow, -noprompt, -prompt, @prompts_sp, @prompts, 'Service Processor Login', 'login', 'Password','-timeout',580);
		if(($after =~ /blacksmith|Part$/) && ($prompt =~ /#/)) {	# To supress the stress messages on the console, also to suppress any DIMM errors on the console
		   pop @prompts_sp;
		} else {
		   last;
		}
	}
	logcomment ("Filer is at prompt: $prompt, bringing to the desired state");

	if (($prompt =~ /Password/) && ($prompt !~ /Login incorrect/)) {
		while (1) {
		    eval {(undef, undef, $after, $prompt) = say($fc," ", -swallow, -noprompt, -prompt, 'smith #', @prompts_sp, @prompts, 'Service Processor Login', 'login', 'Login incorrect', 'Login inc') };
		    if ( ($prompt =~ /smith #/) || (($after =~ /Part$/) && ($prompt =~ /#/)) ) {
			pop @prompts_sp if ($prompts_sp[-1] eq '#');
			next;
		    }
		    last unless ($@);
		    sleep 10;
		}
		logcomment ("Filer is at prompt: $prompt, retrying to bring the desired state");
	}

	if ($prompt =~ /$RLM_PROMPT $FILER.*|Service Processor Login/) {
		logcomment ("FILER: $FILER is at $RLM_PROMPT console");
		pop @prompts_sp if ($prompts_sp[-1] eq '#');
		$prompt = rlm_ctrl_g_logout($FILER);
	}

	if ($prompt =~ /login/i) {
		logcomment ("FILER: $FILER is at ONTAP login prompt");
		pop @prompts_sp if ($prompts_sp[-1] eq '#');
		$prompt = rlm_ontap_login($FILER, $Uname, $Pwd);
	}
	
        if ($prompt =~ /#/) {
                logcomment ("FILER: $FILER is at $RLM_PROMPT root shell");
                $prompt = rlm_ctrl_g_logout($FILER);
		pop @prompts_sp if ($prompts_sp[-1] eq '#');
        }

	unless ($prompt) {
		if (!transit_to($h, 'UP', username=>$Uname, password=>$Pwd)) {
		   sleep 30;
		   (undef, undef, undef, $prompt) = say($fc," ", -swallow, -prompt, @prompts, 'Password', 'Login incorrect', 'login');
		   unless (($prompt =~ /$FILER/) || ($prompt =~ /Password|Login incorrect|login/i)) {
			  logresult ('FATAL', "The filer is unable to boot up normally.");
		   }
		   if ($prompt =~ /Password|Login incorrect|login/i) {
			  if (!transit_to($h, 'UP', username=>$Uname, password=>$Pwd)) {
				logresult ('FATAL', "The filer is unable to boot up normally.");
			  }
		   }
		}
		sleep 30;
		(undef, undef, undef, $prompt) = say($fc," ", -swallow, -prompt, @prompts);
	}


	if ($prompt =~ /LOADER/) {
		logcomment ("FILER: $FILER is at LOADER firmware");
		say($fc,"version", -prompt, @prompts);
		if ($LOADER == 0) {
			logcomment ("Booting filer: $FILER to ONTAP");
			if (!transit_to($h, 'UP', username=>$Uname, password=>$Pwd)) {
			   sleep 60;	#transit_to fails, when prompt jumbles or console spam with many messages.
			   (undef, undef, undef, $prompt) = say($fc," ", -swallow, -prompt, @prompts, 'Password', 'Login incorrect', 'login');
			   unless (($prompt =~ /$FILER/) || ($prompt =~ /Password|Login incorrect|login/i)) {
			      logresult ('FATAL', "The filer is unable to boot up normally.");
			   }
			   if ($prompt =~ /Password|Login incorrect|login/i) {
			      if (!transit_to($h, 'UP', username=>$Uname, password=>$Pwd)) {
				logresult ('FATAL', "The filer is unable to boot up normally.");
			      }
                           }
			}
			sleep 80;
			logcomment ("Filer: $FILER is at ONTAP prompt");
			say($fc,"version", -prompt, @prompts);
		}
	} elsif ($prompt =~ /$FILER\*?>/) {
		logcomment ("Filer: $FILER is at ONTAP prompt");
		say($fc,"version", -prompt, @prompts);
		if ($LOADER == 1) {
			logcomment ("Halting the filer to Loader prompt");
			if (!halt($fc)) {
			   logresult ('FATAL', "Can't take the filer to FIRMWARE");
			}
			sleep 80;
			logcomment ("Taken to LOADER firmware successfully");
			say($fc,"version", -prompt, @prompts);
		}
	} elsif ($prompt =~ /CFE/) {
		logresult ('CONFIG', "This script supports only LOADER\n");
	} 
	elsif ($prompt =~ /$FILER.*%/) { #Pruthvi added code to take FILER to ONTAP prompt if it is at FREEBSD	
        logcomment("$FILER is in system shell.Exiting from system shell");
        say($fc, "exit",-swallow,-blind,-noprompt);
	rlm_verify_filer_state($FILER,$Uname,$Pwd);
      }

}

## @name rlm_setup ($$;$)
## @example
## rlm_setup ($FILER, $test_states{$case});
## rlm_setup ($FILER, $test_states{$case}, "LOADER");
## @description
## To do "rlm setup" in ONTAP and LOADER.
sub rlm_setup ($$;$){
	my $FILER = shift @_;
	my $fc = rlm_make_conn_to_ontap($FILER);
	my $fh = rlm_make_host_to_ontap($FILER); 
	($rlm_state_title, $rlm_ipv4_dhcp, $rlm_ipv4_ip, $rlm_ipv4_netmask, $rlm_ipv4_gateway, $rlm_enable_ipv6, $rlm_assign_ipv6_global_addr, $rlm_ipv6_global_ip, $rlm_ipv6_subnet_prefix, $rlm_ipv6_gateway, $rlm_ipv6_router) = @{$_[0]};
	my $firmware = $_[1];
	my $max_tries = 10;

	logcomment ("$rlm_state_title\n");

	my %prompts_setup = (
		"Would you like to configure the $RLM_PROMPT"				=>	"yes",
		"Would you like to enable DHCP on the $RLM_PROMPT LAN interface"	=>	"$rlm_ipv4_dhcp",
		"Please enter the IP address (?:for|of) the $RLM_PROMPT gateway"	=>	"$rlm_ipv4_gateway",
		"Please enter the IP address (?:for|of) the $RLM_PROMPT"		=>	"$rlm_ipv4_ip",
		"Please enter the netmask (?:for|of) the $RLM_PROMPT"			=>	"$rlm_ipv4_netmask",
		"Do you want to enable IPv6 on the $RLM_PROMPT"				=>	"$rlm_enable_ipv6",
		"Do you want to assign IPv6 global address"				=>	"$rlm_assign_ipv6_global_addr",
                "Please enter the IPv6 address for the $RLM_PROMPT"                     =>      "$rlm_ipv6_global_ip",
                "Please enter the subnet prefix for the $RLM_PROMPT"                    =>      "$rlm_ipv6_subnet_prefix",
                "Please enter the IPv6 Gateway for the $RLM_PROMPT"                     =>      "$rlm_ipv6_gateway",
	);
	
	unless ($firmware) {
		if ($rlm_ipv6_router) {
			rlm_is_ipv6_ra_enable($FILER, 'off');
		}

		logcomment ("Configuring $RLM_PROMPT using \'$RLM setup\' in ONTAP");

		say ($fc, " ", -swallow, -noprompt, -prompt, @prompts);	#Sometimes when ontap prompt is not there then "sp setup" is getting failed
		say ($fc, "$RLM setup\r", -nocr, -blind, -swallow);
		for (my $tries=0; $tries<=$max_tries; $tries++) {
			my (undef, undef, undef, $prompt) = say ($fc, "", -noprompt, -prompt, [keys %prompts_setup, @prompts], -timeout, 500);
			last if ($prompt =~ /LOADER(-\w+)?\*?>|CFE(-\w+)?\*?>|$FILER[\*>]/);
			foreach my $key (keys %prompts_setup) {
				if ($prompt =~ /$key/) {
					my $value = $prompts_setup{$key};
					say ($fc, "$value\r", -nocr, -blind, -swallow);
					delete $prompts_setup{$key};
					last;
				}
			}
		}
	} else {
		my $rlm_ipv6_router_on_off;
                if ($rlm_ipv6_router) {
                        $rlm_ipv6_router_on_off='no';
                } else {
			$rlm_ipv6_router_on_off='yes';
		}

		logcomment ("Configuring $RLM_PROMPT using \'$RLM setup\' in $firmware");

		say ($fc, "$RLM setup", '-swallow', '-noprompt', '-wait', "Would you like to configure the $RLM_PROMPT");
		say ($fc, "yes", '-swallow', '-noprompt', '-wait', "Would you like to enable DHCP on the $RLM_PROMPT LAN interface");
		my($match) = say ($fc, $rlm_ipv4_dhcp, '-swallow', '-noprompt', '-wait', "Please enter the IP address for the $RLM_PROMPT", "Do you want to enable IPv6 on the $RLM_PROMPT");
		if ($rlm_ipv4_dhcp eq "no") {
			say ($fc, $rlm_ipv4_ip, '-swallow', '-noprompt', '-wait', "Please enter the netmask for the $RLM_PROMPT");
			say ($fc, $rlm_ipv4_netmask, '-swallow', '-noprompt', '-wait', "Please enter the IP address for the $RLM_PROMPT gateway");
			say ($fc, $rlm_ipv4_gateway, '-swallow', '-noprompt', '-wait', "Do you want to enable IPv6 on the $RLM_PROMPT");
		}
		if ($rlm_enable_ipv6 eq "yes") {
			if ($rlm_assign_ipv6_global_addr eq "yes") {
				say ($fc, $rlm_enable_ipv6, '-swallow', '-noprompt', '-wait', "Do you want to assign IPv6 global address");
				say ($fc, $rlm_assign_ipv6_global_addr, '-swallow', '-noprompt', '-wait', "Please enter the IPv6 address for the $RLM_PROMPT");
				say ($fc, $rlm_ipv6_global_ip, '-swallow', '-noprompt', '-wait', "Please enter the subnet prefix for the $RLM_PROMPT");
				say ($fc, $rlm_ipv6_subnet_prefix, '-swallow', '-noprompt', '-wait', "Please enter the IPv6 Gateway for the $RLM_PROMPT");
				say ($fc, $rlm_ipv6_gateway, '-swallow', '-noprompt', '-wait', "Do you want to enable IPv6 Router Advertisement Address for the $RLM_PROMPT");
				say ($fc, $rlm_ipv6_router_on_off, '-swallow', '-prompt', @prompts, -timeout, 500);
			} else {
				say ($fc, $rlm_enable_ipv6, '-swallow', '-noprompt', '-wait', "Do you want to assign IPv6 global address");
				say ($fc, $rlm_assign_ipv6_global_addr, '-swallow', '-noprompt', '-wait', "Do you want to enable IPv6 Router Advertisement Address for the $RLM_PROMPT");
                                say ($fc, $rlm_ipv6_router_on_off, '-swallow', '-prompt', @prompts, -timeout, 500);
			}
		} else {
			say ($fc, $rlm_enable_ipv6, '-swallow');
		}
	}
	logcomment ("$RLM setup completed successfully");
}

## @name rlm_verify_rlm_status ($$;$)
## @example
## rlm_verify_rlm_status ($FILER, "LOADER");
## rlm_verify_rlm_status ($FILER, "ONTAP", $case);
## @description
## Verifies "rlm status" output in ONTAP or LOADER or SP
sub rlm_verify_rlm_status ($$;$) {
	my ($conn, $mode, $state) = @_;
	my @waitstrings = ('Link\sLocal\s(?:IP|Address)(?:\s+IP\sAddress)?\:\s+([\w\:]+)', 'IPv6(?:\sconfiguration)?\:(?:\s+)?(Disabled)?');	#jkonjeti 06Feb13: moved this array from else statement
	if ($mode =~ /ONTAP|LOADER/i) {
		$conn = rlm_make_conn_to_ontap($conn);
	}

	logcomment ("\"$RLM status\" in $mode \n");
	if ($mode eq $RLM_PROMPT) {
		say ($conn, "version", -prompt, @prompts, -timeout, 300);
	}

	my $rlm_output;
	unless ($mode =~ /LOADER/i) {
#	   my(undef, undef, $output, $prompt) = say($conn, "$RLM status", -prompt, @prompts, -timeout, 300);	#jkonjeti 06Feb13: Commented these 2 lines
#	   $rlm_output = $output;
	   #jkonjeti 06Feb13: Added these 2 lines. Sometimes "sp status" from ONTAP is taking long time and in between if ONTAP warning message occurs say function is failing.
           my($match, $before, $output, $prompt) = say($conn, "$RLM status", '-swallow', '-wait', @waitstrings, -timeout, 300);	
	   $rlm_output = $before . "\n" . $match . "\n" . $output;
	} else {
	   sleep(15);
#	   my @waitstrings = ('Link\sLocal\s(?:IP|Address)(?:\s+IP\sAddress)?\:\s+([\w\:]+)', 'IPv6(?:\sconfiguration)?\:(?:\s+)?(Disabled)?');
#	   my($match, $before, $output, $prompt) = say($conn, "$RLM status", '-swallow', '-wait', @waitstrings, -noprompt, '-prompt', @prompts);
          my($match, $before, $output, $prompt) = say($conn, "$RLM status", '-swallow', '-wait', @waitstrings, -noprompt, -timeout, 300);
	   $rlm_output = $before . "\n" . $match . "\n" . $output;
        }

	logcomment ("Verifying the $RLM status output in $mode...");
	if ($state) {
		if (rlm_extract_rlm_status_info($rlm_output, $mode, $state)) {
			logresult ('PASS', "$RLM status output looking as expected for the test state: $state");
		} elsif (($ipv6_global_ip =~ /Disabled/i) && ($ipv6_router_ip)) {
			logresult ('WARN', "$RLM_PROMPT Global IPv6 expected, but RA seen and Global IP Disabled for the test state: $state");
		} else {
			logresult ('FATAL', "State: $state FAILED due to incorrect $RLM status output");
		}
	} else {
		rlm_extract_rlm_status_info($rlm_output, $mode);
	        $ipv4_ip = 0 if ($ipv4_ip =~ /Disabled|unknown/i);
        	$ipv4_mask = 0 if ($ipv4_mask =~ /Disabled|unknown/i);
	        $ipv4_gateway = 0 if ($ipv4_gateway =~ /Disabled|unknown/i);
	        $ipv6_router_ip = 0 if ($ipv6_router_ip =~ /Disabled|unknown/i);
	        $ipv6_status = 'Disabled' if ($NO_IPv6 && ($TEST_STATE <= $IPv4_ONLY));
	}
}

## @name rlm_extract_rlm_status_info ($;$$)
## @example
## rlm_extract_rlm_status_info ($rlm_output, $mode, $state);
## @description
## Extracts IPV4 and IPv6 fields and corresponding values from "rlm status" output in ONTAP/SP/LOADER and sends return value to the subroutine "rlm_verify_rlm_status". 
sub rlm_extract_rlm_status_info ($;$$) {
	my ($rlm_status_output, $mode, $TEST_STATE) = @_;
print "\$rlm_status_output = $rlm_status_output\n\$mode = $mode\n\$TEST_STATE = $TEST_STATE\n";
	($fw_version, $dhcp_status, $ipv4_ip, $ipv4_mask, $ipv4_gateway, $ipv6_status, $ipv6_global_ip, $ipv6_prefix, $ipv6_gateway, $ipv6_router_ip, $ipv6_link_local_ip) = ();
	
	my %rlm_status_hash;

	if ($rlm_status_output =~ /Status:\s(\w+)/i) {
		$rlm_status_hash{'Status'} = $1;
	}

	if (($mode =~ /ONTAP/i) && ($rlm_status_hash{'Status'} !~ /online/i)) {
		logresult ('FATAL', "$RLM is in: \'" . $rlm_status_hash{'Status'} . "\', but it should be ONLINE");  
	}

	if ($rlm_status_output =~ /Firmware\sVersion\:\s+([\w\.]+).+Ethernet\sLink\:\s+(\w+)/s) {
		($rlm_status_hash{'Firmware Version'}, $rlm_status_hash{'Ethernet Link'}) = ($1, $2);
	}

	if ($rlm_status_output =~ /IP\sAddress\:\s+([\d\.Disabledunknown]+).+Netmask\:\s+([\d\.Disabledunknown]+)\s+Gateway\:\s+([\d\.Disabledunknown]+)/is) {
		($rlm_status_hash{'IP Address'}, $rlm_status_hash{'Netmask'}, $rlm_status_hash{'Gateway:'}) = ($1, $2, $3);
	} elsif ($rlm_status_output =~ /IPv4(?:\sconfiguration)?\:\s+(Disabled)/) {
		$rlm_status_hash{'IP Address'} = $rlm_status_hash{'Netmask'} = $rlm_status_hash{'Gateway:'} = $1;
	}

	if ($rlm_status_output =~ /Global(?:\sAddress\s+)?\sIP(?:\sAddress)?\:\s+([\w\:]+).+Prefix\sLength\:\s+(\d{1,3})?.+Gateway(?:\sAddress)?\:\s+([abcdefDisabledunknown0-9\:]+)/is) {
		($rlm_status_hash{'Global IP'}, $rlm_status_hash{'Prefix Length'}, $rlm_status_hash{'Gateway'} ) = ($1, $2, $3);
	} elsif ($rlm_status_output =~ /Global\s(?:IP|Address)\:\s+(Disabled)/) {
		$rlm_status_hash{'Global IP'} = $1;
	}

	$rlm_status_hash{'DHCP'} = $1 if ($rlm_status_output =~ /DHCP\:\s+(\w+)/);
	$rlm_status_hash{'IPv6 configuration'} = $1 if ($rlm_status_output =~ /IPv6(?:\sconfiguration)?\:(?:\s+)?(Disabled)?/);
	$rlm_status_hash{'Link Local IP'} = $1 if ($rlm_status_output =~ /Link\sLocal\s(?:IP|Address)(?:\s+IP\sAddress)?\:\s+([\w\:]+)/);
	$rlm_status_hash{'Router Assigned IP'} = $1 if ($rlm_status_output =~ /Router(?:\sAssigned|\sAdvertisement)?\s(?:IP|Address)(?:\s+IP\sAddress)?\:\s+([\w\:]+)/);

	$fw_version = $rlm_status_hash{'Firmware Version'};
	$dhcp_status = $rlm_status_hash{'DHCP'};
	$ipv4_ip = $rlm_status_hash{'IP Address'};
	$ipv4_mask = $rlm_status_hash{'Netmask'};
	$ipv4_gateway = $rlm_status_hash{'Gateway:'};
	$ipv6_status = $rlm_status_hash{'IPv6 configuration'};
	$ipv6_global_ip = $rlm_status_hash{'Global IP'};
	$ipv6_prefix = $rlm_status_hash{'Prefix Length'};
	$ipv6_gateway = $rlm_status_hash{'Gateway'};
	$ipv6_router_ip = $rlm_status_hash{'Router Assigned IP'};
	$ipv6_link_local_ip = $rlm_status_hash{'Link Local IP'};

	if ($rlm_status_hash{'Ethernet Link'} !~ /up/i) {
		logresult ('FATAL', "$RLM ethernet link is: " . $rlm_status_hash{'Ethernet Link'});
	}

print "\$TEST_STATE = $TEST_STATE\n";
	if ($TEST_STATE) {
		my $ret = rlm_verify_rlm_status_values($TEST_STATE);
print "\$ret = $ret\n";
		return ($ret);
	}
}

## @name rlm_verify_rlm_status_values ($_
## @example
## rlm_verify_rlm_status_values ($case_no);
## @description
## Validates "sp/rlm status" output with the corresponding testcase/teststate.
sub rlm_verify_rlm_status_values ($) {
print "Inside RLM verify rlm_status_values \n";
	my $TEST_STATE = shift @_;
print "\$TEST_STATE = $TEST_STATE\n";
	logcomment ("Firmware Version	= $fw_version");
	logcomment ("IPV4 DHCP Status	= $dhcp_status");
	logcomment ("IPV4 IP Addr	= $ipv4_ip");
	logcomment ("IPV4 Netmask	= $ipv4_mask");
	logcomment ("IPV4 Gateway	= $ipv4_gateway");
	logcomment ("IPV6 Status	= $ipv6_status");
	logcomment ("Global IP		= $ipv6_global_ip");
	logcomment ("Prefix Length	= $ipv6_prefix");
	logcomment ("Gateway IPV6 IP	= $ipv6_gateway");
	logcomment ("Router Assigned IP = $ipv6_router_ip");
	logcomment ("Link Local IP	= $ipv6_link_local_ip");
print "\$ipv4_ip = $ipv4_ip\n\$ipv4_mask = $ipv4_mask\n\$ipv4_gateway = $ipv4_gateway\n\$ipv6_router_ip = $ipv6_router_ip\n\$ipv6_status = $ipv6_status\n";
        $ipv4_ip = 0 if ($ipv4_ip =~ /Disabled|unknown/i);
        $ipv4_mask = 0 if ($ipv4_mask =~ /Disabled|unknown/i);
        $ipv4_gateway = 0 if ($ipv4_gateway =~ /Disabled|unknown/i);
	$ipv6_router_ip = 0 if ($ipv6_router_ip =~ /Disabled|unknown/i);
	$ipv6_status = 'Disabled' if ($NO_IPv6 && ($TEST_STATE <= $IPv4_ONLY));
print "\$dhcp_status = $dhcp_status\n";
print "\$rlm_ipv4_ip = $rlm_ipv4_ip\n\$rlm_ipv4_netmask = $rlm_ipv4_netmask\n\$rlm_ipv4_gateway = $rlm_ipv4_gateway\n";
	unless ($dhcp_status =~ /$rlm_ipv4_dhcp/i) {
		return(0) if ($ipv4_ip && $ipv4_mask && $ipv4_gateway);
	}
	if ($TEST_STATE == 1) {
#		unless (($ipv4_ip == 0) && ($ipv4_mask == 0) && ($ipv4_gateway == 0) && ($ipv6_status =~ /Disabled/i)) {
               unless (($ipv4_ip == 0) && ($ipv4_mask == 0) && ($ipv4_gateway == 0)) {
			return 0;
		}
	} elsif ($TEST_STATE == 2) {
#		unless (($ipv4_ip eq $rlm_ipv4_ip) && ($ipv4_mask eq $rlm_ipv4_netmask) && ($ipv4_gateway eq $rlm_ipv4_gateway) && ($ipv6_status =~ /Disabled/i)) {
               unless (($ipv4_ip eq $rlm_ipv4_ip) && ($ipv4_mask eq $rlm_ipv4_netmask) && ($ipv4_gateway eq $rlm_ipv4_gateway)) {
			return 0;
		}
	} elsif ($TEST_STATE == 3) {
		unless ($ipv4_ip && $ipv4_mask && $ipv4_gateway && ($ipv6_status =~ /Disabled/i)) {
			return 0;
		}
	} elsif ($TEST_STATE == 4 || $TEST_STATE == 13 || $TEST_STATE == 19 || $TEST_STATE == 22) {
		unless (($ipv4_ip == 0) && ($ipv4_mask == 0) && ($ipv4_gateway == 0) && ($ipv6_global_ip =~ /$rlm_ipv6_global_ip/i) && ($ipv6_prefix == $rlm_ipv6_subnet_prefix) && ($ipv6_gateway =~ /$rlm_ipv6_gateway/i) && $ipv6_router_ip && $ipv6_link_local_ip) {
			return 0;
		}
	} elsif ($TEST_STATE == 5 || $TEST_STATE == 14 || $TEST_STATE == 20 || $TEST_STATE == 23) {
		unless (($ipv4_ip eq $rlm_ipv4_ip) && ($ipv4_mask eq $rlm_ipv4_netmask) && ($ipv4_gateway eq $rlm_ipv4_gateway) && ($ipv6_global_ip =~ /$rlm_ipv6_global_ip/i) && ($ipv6_prefix == $rlm_ipv6_subnet_prefix) && ($ipv6_gateway =~ /$rlm_ipv6_gateway/i) && $ipv6_router_ip && $ipv6_link_local_ip) {
			return 0;
		}
	} elsif ($TEST_STATE == 6 || $TEST_STATE == 15 || $TEST_STATE == 21 || $TEST_STATE == 24) {
		unless ($ipv4_ip && $ipv4_mask && $ipv4_gateway && ($ipv6_global_ip =~ /$rlm_ipv6_global_ip/i) && ($ipv6_prefix == $rlm_ipv6_subnet_prefix) && ($ipv6_gateway =~ /$rlm_ipv6_gateway/i) && $ipv6_router_ip && $ipv6_link_local_ip) {
			return 0;
		}
	} elsif ($TEST_STATE == 7) {
		unless (($ipv4_ip eq $rlm_ipv4_ip) && ($ipv4_mask eq $rlm_ipv4_netmask) && ($ipv4_gateway eq $rlm_ipv4_gateway) && ($ipv6_global_ip =~ /Disabled/i) && $ipv6_router_ip && $ipv6_link_local_ip) {
			return 0;
		}
	} elsif ($TEST_STATE == 8) {
		unless ($ipv4_ip && $ipv4_mask && $ipv4_gateway && ($ipv6_global_ip =~ /Disabled/i) && $ipv6_router_ip && $ipv6_link_local_ip) {
			return 0;
		}
	} elsif ($TEST_STATE == 9) {
		unless (($ipv4_ip == 0) && ($ipv4_mask == 0) && ($ipv4_gateway == 0) && ($ipv6_global_ip =~ /Disabled/i) && $ipv6_router_ip && $ipv6_link_local_ip) {
			return 0;
		}
	} elsif ($TEST_STATE == 10 || $TEST_STATE == 16 || $TEST_STATE == 25) {
                unless (($ipv4_ip == 0) && ($ipv4_mask == 0) && ($ipv4_gateway == 0) && ($ipv6_global_ip =~ /$rlm_ipv6_global_ip/i) && ($ipv6_prefix == $rlm_ipv6_subnet_prefix) && ($ipv6_gateway =~ /$rlm_ipv6_gateway/i) && !$ipv6_router_ip && $ipv6_link_local_ip) {
                        return 0;
                }
        } elsif ($TEST_STATE == 11 || $TEST_STATE == 17 || $TEST_STATE == 26) {
                unless (($ipv4_ip eq $rlm_ipv4_ip) && ($ipv4_mask eq $rlm_ipv4_netmask) && ($ipv4_gateway eq $rlm_ipv4_gateway) && ($ipv6_global_ip =~ /$rlm_ipv6_global_ip/i) && ($ipv6_prefix == $rlm_ipv6_subnet_prefix) && ($ipv6_gateway =~ /$rlm_ipv6_gateway/i) && !$ipv6_router_ip && $ipv6_link_local_ip) {
                        return 0;
                }
        } elsif ($TEST_STATE == 12 || $TEST_STATE == 18 || $TEST_STATE == 27) {
                unless ($ipv4_ip && $ipv4_mask && $ipv4_gateway && ($ipv6_global_ip =~ /$rlm_ipv6_global_ip/i) && ($ipv6_prefix == $rlm_ipv6_subnet_prefix) && ($ipv6_gateway =~ /$rlm_ipv6_gateway/i) && !$ipv6_router_ip && $ipv6_link_local_ip) {
                        return 0;
                }
        } elsif ($TEST_STATE == 28) {
#		unless (($ipv4_ip == 0) && ($ipv4_mask == 0) && ($ipv4_gateway == 0) && ($ipv6_global_ip =~ /Disabled/i) && !$ipv6_router_ip && ($ipv6_link_local_ip !~ /unknown|Disabled/i)) {
               unless (($ipv4_ip == 0) && ($ipv4_mask == 0) && ($ipv4_gateway == 0) && ($ipv6_global_ip =~ /Disabled/i) ) {
                        return 0;
                }
	} elsif ($TEST_STATE == 29) {
                unless (($ipv4_ip eq $rlm_ipv4_ip) && ($ipv4_mask eq $rlm_ipv4_netmask) && ($ipv4_gateway eq $rlm_ipv4_gateway) && ($ipv6_global_ip =~ /Disabled/i) && !$ipv6_router_ip && $ipv6_link_local_ip) {
                        return 0;
                }
        } elsif ($TEST_STATE == 30) {
                unless ($ipv4_ip && $ipv4_mask && $ipv4_gateway && ($ipv6_global_ip =~ /Disabled/i) && !$ipv6_router_ip && $ipv6_link_local_ip) {
                        return 0;
                }
        } 
}

## @name rlm_verify_rlm_status_d ($$)
## @example
## rlm_verify_rlm_status_d ($FILER, $mode)
## @description
## Verifies "rlm/sp status -d" output in ONTAP and RLM/SP.
sub rlm_verify_rlm_status_d ($$) {
	my ($conn, $mode) = @_;
	if ($mode =~ /ONTAP/i) {
		$conn = rlm_make_conn_to_ontap($conn);
	}elsif ($mode =~ /RLM|SP/i) {
		$mode = $RLM_PROMPT;
	}

	logcomment ("Analysing the output of \'$RLM status -d\' in $mode");
	say($conn, "priv set test", -timeout, 100);
	#my(undef, undef, $output, $prompt) = say($conn, "$RLM status -d", -swallow, -noprompt, -prompt, @prompts_sp, @prompts, -timeout, 3600);
#	my(undef, undef, $output, $prompt) = say($conn, "$RLM status -d", -swallow, -noprompt, '-wait', 'ip -6 addr', -timeout, 1500, -prompt, @prompts_sp, @prompts);
#	my($before, $after, $output, $prompt) = say($conn, "$RLM status -d", -swallow, -noprompt, '-wait', 'ip -6 addr', $FILER, -prompt, @prompts_sp, @prompts, -timeout, 900);
	#my($before, $after, $output, $prompt) = say($conn, "$RLM status -d", -swallow, -noprompt, '-wait', 'ip -6 addr', $FILER, -prompt, @prompts_sp, @prompts, -timeout, 900);
	#my($before, $after, $output, $prompt) = say($conn, "$RLM status -d", -swallow, -noprompt, '-wait', 'ip -6 addr', $FILER, -prompt, $FILER, "SP $FILER", -timeout, 900);
	my($before, $after, $output, $prompt) = say($conn, "$RLM status -d", -swallow, -noprompt, '-wait', 'ip -6 addr', $FILER, -prompt, @prompts, @propmts_sp, -timeout, 1500);
	$output = $after . $output;
#	my(undef, undef, $output, $prompt) = say($conn, "$RLM status -d", -swallow, -noprompt, -prompt, @prompts, -timeout, 3600);
	my $result = 1;

	logcomment ("Checking the /mnt/logs/etc/rlm_config_xtd_info");

	my %rlm_status_d_hash = $output =~ /\/mnt\/logs\/etc\/rlm_config_xtd_info.+(\b\w+_ipv6)\s([\w\:]+).+(\b\w+_ipv6)\s([\w\:]+).+(\b\w+_ipv6)\s([\w\:]+).+(\b\w+_ipv6)\s([\w\:]+)/s;
	my ($lla, $ra, $ga, $gwa) = ($rlm_status_d_hash{'link_ipv6'}, $rlm_status_d_hash{'router_ipv6'}, $rlm_status_d_hash{'global_ipv6'}, $rlm_status_d_hash{'gateway_ipv6'});

	if ($ipv6_status =~ /Disabled/i) {
		unless (($lla == 0) || ($ra == 0) || ($ga == 0) || ($gwa == 0)) {
			$result = 0;
		}
	} else {
		if ($ipv6_global_ip =~ /Disabled|unknown/i) {
			unless (($lla =~ /$ipv6_link_local_ip/) || ($ra =~ /$ipv6_router_ip/) || ($ga == 0) || ($gwa == 0)) {
				$result = 0;
			}
		} else {
			unless (($lla =~ /$ipv6_link_local_ip/) || ($ra =~ /$ipv6_router_ip/) || ($ga =~ /$ipv6_global_ip/) || ($gwa =~ /$ipv6_gateway/)) {
				$result = 0;
			}
		}
	}


        foreach ($output =~ /errors\:(\d+)/g) {
                if ($_ > 0) {
                   logresult ('WARN', "\"$RLM status\" showing error packets: $1");
                   $result = 0;
                   last;
                }
        }

	if ($result) {
		logresult ('PASS', "The output of /mnt/logs/etc/rlm_config_xtd_info looking as expected");
	} else {
		logresult ('FATAL', "The output of /mnt/logs/etc/rlm_config_xtd_info is different w.r.t '$RLM status', hence '$RLM status -d' output is wrong");
	}

        logcomment ("Checking the \"ip -6 addr\" section in $RLM status -d output");
        if ($RLM =~ /SP/i) {
		($ra) = $output =~ /ip\s\-6\saddr.+inet6\s([\w\:]+)\/\d+\sscope\sglobal\sdynamic/s;
		($ga) = $output =~ /ip\s\-6\saddr.+inet6\s([\w\:]+)\/\d+\sscope\sglobal\s+/s;
		($lla) = $output =~ /ip\s\-6\saddr.+inet6\s([\w\:]+)\/\d+\sscope\slink/s;		
#                my ($lla, $ra, $ga) = ($rlm_status_d_hash{'link'}, $rlm_status_d_hash{'global dynamic'}, $rlm_status_d_hash{'global'});

                unless (($lla =~ /$ipv6_link_local_ip/) || ($ra =~ /$ipv6_router_ip/) || ($ga =~ /$ipv6_global_ip/)) {
                                $result = 0 unless ($ipv6_status =~ /Disabled/i);
                }
        }

	if ($result) {
                logresult ('PASS', "The output of \"ip -6 addr\" section looking as expected");
        } else {
                logresult ('FATAL', "The output of \"ip -6 addr\" section is different w.r.t '$RLM status', hence '$RLM status -d' output is wrong");
        }

	logresult ('PASS', "'$RLM status -d' output looking as expected");
}

## @name rlm_verify_rlm_status_rlm ($$)
## @example
## rlm_verify_rlm_status_rlm ($rlm_conn, $state_no);
## @description
## Verifies "rlm/sp status" output in RLM/SP.
sub rlm_verify_rlm_status_rlm ($$) {
	my ($r, $state) = @_;
	my ($v4ip, $ra, $lla, $ga);
	my $result = 1;

	logcomment ("\"$RLM status\" in $RLM_PROMPT \n");
	my(undef, undef, $output) = say($r, "$RLM status", -timeout, 180);
	logcomment ("Verifying the $RLM status output in $RLM_PROMPT...");

	if ($ipv4_ip) {
		($v4ip) = $output =~ /inet\saddr\:([\w\.]+)/s;
		unless ($v4ip =~ /$ipv4_ip/) {
			$result = 0;
		}
	}

	unless ($ipv6_status =~ /Disabled/i) {
		if (($ipv6_global_ip =~ /Disabled|unknown/i) && $ipv6_router_ip) {
			my @rlm_ip_arr = $output =~ /inet6\saddr\:\s([\w\:]+)\/\d+\sScope\:(\w+).+inet6\saddr\:\s([\w\:]+)\/\d+\sScope\:(\w+)/s;

                        if ($rlm_ip_arr[1] =~ /Link/i ) {
                                ($ra, $lla) = @rlm_ip_arr[2,0];
                        } else {
                                ($ra, $lla) = @rlm_ip_arr[0,2];
                        }

			unless (($ra =~ /$ipv6_router_ip/) && ($lla =~ /$ipv6_link_local_ip/)) {
				logresult ('WARN', "Incorrect Router IP or Link Local IP in the $RLM status output");
				$result = 0;
			}

		} elsif (($ipv6_global_ip !~ /Disabled|unknown/i) && !$ipv6_router_ip) {
			my @rlm_ip_arr = $output =~ /inet6\saddr\:\s([\w\:]+)\/\d+\sScope\:(\w+).+inet6\saddr\:\s([\w\:]+)\/\d+\sScope\:(\w+)/s;

			if ($rlm_ip_arr[1] =~ /Link/i ) {
				($ga, $lla) = @rlm_ip_arr[2,0];
			} else {
				($ga, $lla) = @rlm_ip_arr[0,2];
			}

			unless (($ga =~ /$ipv6_global_ip/) && ($lla =~ /$ipv6_link_local_ip/)) {
				logresult ('WARN', "Incorrect Global IP or Link Local IP in the $RLM status output");
				$result = 0;
			}

		} elsif (($ipv6_global_ip =~ /Disabled|unknown/i) && !$ipv6_router_ip) {
			($lla) = $output =~ /inet6\saddr\:\s([\w\:]+)/s;
			unless ($lla =~ /$ipv6_link_local_ip/) {
				logresult ('WARN', "Incorrect Link Local IP in the $RLM status output");
				$result = 0;
			}
		} else {
			my @rlm_ip_arr = $output =~ /inet6\saddr\:\s([\w\:]+)\/\d+\sScope\:(\w+).+inet6\saddr\:\s([\w\:]+)\/\d+\sScope\:(\w+).+inet6\saddr\:\s([\w\:]+)\/\d+\sScope\:(\w+)/s;

			for (my $i=1; $i < scalar(@rlm_ip_arr); $i=$i+2) {
				if ($rlm_ip_arr[$i] =~ /Link/i) {
					$lla = $rlm_ip_arr[$i-1];
					splice (@rlm_ip_arr, $i-1, 2);
					last;
				}
			}

			if ($rlm_ip_arr[0] =~ /$ipv6_global_ip/ && $rlm_ip_arr[1] =~ /Global/i) {
				($ga, $ra) = @rlm_ip_arr[0,2];
			} else {
				($ga, $ra) = @rlm_ip_arr[2,0];
			}

			unless (($ga =~ /$ipv6_global_ip/i) && ($ra =~ /$ipv6_router_ip/) && ($lla =~ /$ipv6_link_local_ip/)) {
				logresult ('WARN', "Incorrect Global IP or Router IP or Link Local IP in the $RLM status output");
				$result = 0;
			}
		}
	}

	my @pkt_errors = $output =~ /errors\:(\d+)/g;
	foreach (@pkt_errors) {
		if ($_ > 0) {
		   logresult ('WARN', "\"$RLM status\" showing error packets: $1");
		   $result = 0;
		   last;
		}
	}

	if ($result){
		logresult ('PASS', "$RLM output is as expected in the test state: $state");
	} else {
		logresult ('FATAL', "State: $state FAILED due to incorrect $RLM status output");
	}
}

## @name rlm_reboot_rlm ($)
## @example
## rlm_reboot_rlm ($FILER)
## @description
## Reboot's RLM/SP either from ONTAP / LOADER.
sub rlm_reboot_rlm ($) {
    my $FILER = shift @_;
    my $fc = rlm_make_conn_to_ontap($FILER);

    if ($LOADER) {
    	logcomment ("Rebooting $RLM_PROMPT at LOADER");
    } else {
	logcomment ("Rebooting $RLM_PROMPT at ONTAP");
    }

    say($fc, "priv set test", '-prompt', @prompts) if($LOADER);
    say($fc, "$RLM reboot", '-prompt', "If your console connection is through the (?:$RLM_PROMPT|Service Processor|Remote LAN Module),it will be disconnected", '-timeout', 30);
    say($fc, "y", '-prompt', @prompts, '-timeout', 300);

    sleep(300);

    my ($timer,$status) = (0,0);
    $fc->discardbuff;
    while($timer <= 20){
         my(undef, undef, $out) = say($fc, "$RLM status", '-timeout', 300);
#         my(undef, undef, $out) = say($fc, "$RLM status", '-prompt', @prompts, '-timeout', 300);
         if($out =~ /Online/){
            $status = 1;
            $timer = 21;
	  } elsif ($LOADER && $out =~ /Link Local Address|IPv6\:\s+(?:Disabled|unknown)/) {
            $status = 1;
            $timer = 21;
          } else {
            sleep(15);
          }
          $timer = $timer + 1;
    }

    unless ($status) {
        logresult ('FATAL', "$RLM_PROMPT is not getting back to online state");
    }
}

## @name rlm_ping ($$;$$)
## @example
## rlm_ping ($client_conn, $ping_ip);
## rlm_ping ($client_conn, $ping_ip, $title, $icmp_options);
## @description
## Validates ping/ping6 functionality for IPv4/IPv6 addresses with various options (-a, -c, -f, -l, -s, sudo) and lists the summary.
sub rlm_ping ($$;$$) {
	my $cc = shift @_;
	my $ping_ip = shift @_;
	my ($item_title, $options) = @{$_[0]};
	my $SUDO = $_[1];

	my ($ping_title, $ping6_option) = ($ping_ip =~ /^[\d\.]+$/) ? ('PING') : ('PING6', '-I eth0');
	my $ping_cmd = "\L$ping_title\E";

	$SUDO ? logcomment ("SUDO $ping_title: $item_title \n") : logcomment ("$ping_title: $item_title \n");
	
	sleep 3;
	#my (undef, undef, $output) = say($cc, "$SUDO $ping_cmd @{$options} $ping6_option $ping_ip -c 6");
	#my (undef, undef, $output) = say($cc, "$SUDO $ping_cmd @{$options} $ping6_option $ping_ip -c 17");
	my (undef, undef, $output) = say($cc, "$SUDO $ping_cmd @{$options} $ping6_option $ping_ip -c 18");
	
	my $option = join(" ", @{$options});
	
	unless ($option eq '-f') {
		if ($output =~ /(\d+)\%\spacket\sloss/) {
	   		if ($1 > 0) {
				logresult ('FATAL', "pinging to ip-address: $ping4_ip not responding");
		        } else {
				logresult ('PASS', "pinging to ip-address: $ping4_ip succedded");
		   	}
		}
	} else {
                if ($output =~ /(\d+)\%\spacket\sloss/) {
                        if ($1 > 0) {
                                logresult ('PASS', "pinging to ip-address: $ping4_ip not responding");
                        } else {
                                logresult ('FATAL', "pinging to ip-address: $ping4_ip succedded");
                        }
                }
	}
}

## @name rlm-wget ($$$)
## @example
## rlm_wget ($FILER, $rlm_conn, $rlm_ip);
## @description
## Downloads files from IPv4 and IPv6 webhosts.
sub rlm_wget ($$$) {
   my ($FILER, $r, $ip) = @_;
   my $fc = rlm_make_conn_to_ontap($FILER);

   logcomment ("WGET: Downloading files using wget command...");
   say ($fc, "priv set test", -prompt, @prompts);
   say ($fc, "$RLM debugmode", -prompt, @prompts);
   say ($r, "priv set test");
   say ($r, "$RLM shell", -prompt, '\$');

   my $wget_http_path = ($ip eq $ipv4_ip) ? $RLM_WGET_IPV4_HTTP_PATH : $RLM_WGET_IPV6_HTTP_PATH;

   my ($file) = $wget_http_path =~ /\/([\w\.]+)$/;
   my (undef, undef, $o) = say ($r, "wget $wget_http_path",  -prompt, '\$', '-timeout',3000);

   if($o =~ /$file\s+100%/s) {
        logresult ('PASS', "File: $file downloaded successfully");
   } elsif ($o =~ /Proxy Error/) {
	logresult ('FAIL', "Can't download file: $file, because of the proxy error");
   } else {
        logresult ('FATAL', "wget failed to download file: $file");
   }

   sleep (1);
   say ($r, "rm -f $file", -prompt, '\$');
}

## @name rlm_make_conn_to_rlm ($$;$$)
## @example
## rlm_make_conn_to_rlm ($FILER, $rlm_ip);
## rlm_make_conn_to_rlm ($FILER, $rlm_ip, $Uname, $Pwd);
## @description
## To establish a connection to RLM IP via SSH and returns the connection handle.
sub rlm_make_conn_to_rlm ($$;$$) {
	my ($FILER, $ip) = @_;
        my $Uname = $_[2] ? $_[2] : 'naroot';
        my $Pwd = $_[3] ? $_[3] : 'netapp1!';	
	my ($r, $old_pwd);

        if ($Uname eq "root") {
		my $h = rlm_make_host_to_ontap($FILER);
		#my ($tharn_prompt) = @{$h->prompt_norm()};
		#logcomment ("prompt data at tharnhost: $data");
		#$h->prompt_norm(['#']);
		#($data) = @{$h->prompt_norm()};
		#logcomment ("prompt data modified at tharnhost: $data ");
		push @{$h->prompt_norm()}, '#';
		$old_pwd = $Pwd;
#print "\$RLM_PROMPT = $RLM_PROMPT\n";
		$Pwd = ($RLM_PROMPT eq 'SP') ? 'superuser' : 'moondance';
		eval { $r = connect("$FILER#ssh" . ++$ssh_index . ";conntype=ssh-persist;connhost=$ip;username=$Uname;password=$Pwd") };

		unless ($@) {
			return $r;
		} else {
			$Pwd = $old_pwd;
			my $fc = rlm_make_conn_to_ontap($FILER);	
        	        my (undef, undef, undef, $prompt) = say ($fc, " ", '-swallow', '-prompt', @prompts, '-timeout', 10);
                	if ($prompt =~ /LOADER/) {
				my $h = rlm_make_host_to_ontap($FILER);
				logcomment ("Booting FILER to ONTAP to set \"$RLM debugmode\" ");
                        	$Uname = "root";
                        	if(!transit_to($h, 'UP', username=>$Uname, password=>$Pwd)) {
                                	logresult ('FATAL', "The filer is unable to boot up normally.");
                        	}
                        	sleep 80;
                        	(undef, undef, undef, $prompt) = say ($fc, "version", '-swallow', '-prompt', @prompts, '-timeout', 10);
                	}

	                if ($prompt =~ /$FILER/) {
        	                say ($fc, "priv set test", '-swallow', '-prompt', @prompts, '-timeout', 10);
                	        say ($fc, "$RLM debugmode", '-swallow', '-prompt', @prompts, '-timeout', 10);
	                }

	                $Pwd = ($RLM_PROMPT eq 'SP') ? 'superuser' : 'moondance';
	        }
	}

	if ($ip) {
		eval { $r = connect("$FILER#ssh" . ++$ssh_index . ";conntype=ssh-persist;connhost=$ip;username=$Uname;password=$Pwd") };
		unless ($@) {
			return $r;
		} else {
			logresult ('FATAL', "SSH to $ip failed : $@ \n");
		}
	}
}

## @name rlm_ssh_interactive_mode ($;$$)
## @example
## rlm_ssh_interactive_mode ($FILER);
## rlm_ssh_interactive_mode ($FILER, $Uname, $Pwd);
## @description
## Tests the connectivity of RLM/SP IPv4 and IPv6 addresses via SSH [INTERACTIVE MODE].
sub rlm_ssh_interactive_mode ($;$$) {
	my $FILER = shift @_;
        my $Uname = $_[0] ? $_[0] : 'naroot';
        my $Pwd = $_[1] ? $_[1] : '';	

	if ($ipv4_ip || $ipv6_global_ip || $ipv6_router_ip) {
		logcomment ("SSH into $RLM_PROMPT with all the IPv4 and IPv6 addresses [INTERACTIVE MODE] \n");
	} else {
		logcomment ("[INTERACTIVE MODE]: No IP is avaliable to do SSH \n"); 	
	}

	my $r;
	foreach my $ssh_interactive_ip ($ipv4_ip, $ipv6_global_ip, $ipv6_router_ip) {
		if ($ssh_interactive_ip && ($ssh_interactive_ip !~ /Disabled|unknown/i)) {
			next if ($ssh_interactive_ip =~ /$ipv6_gateway/i);
			my $r = rlm_make_conn_to_rlm ($FILER, $ssh_interactive_ip, $Uname, $Pwd);
			my (undef, undef, $ver) = say ($r, "version");
			$ver =~ /Booted (primary|backup) firmware version/;
			unless ($1 =~ /primary/) {
				logresult ('FAIL', "$1 seen, primary should come");
			}
			rlm_wget($FILER, $r, $ssh_interactive_ip) unless($LOADER);
			disconnect($r);
			logresult ('PASS', "SSH to $ssh_interactive_ip succeeded \n");
		}
	}
}

## @name rlm_ssh_non_interactive_mode ($;$$)
## @example
## rlm_ssh_non_interactive_mode ($client_conn);
## rlm_ssh_non_interactive_mode ($client_conn, $Uname, $Pwd);
## @description
## Tests the connectivity of RLM/SP IPv4 and IPv6 addresses via SSH using options '-v' for IPv4 and '-v -6' for IPv6 [NON-INTERACTIVE MODE].
sub rlm_ssh_non_interactive_mode ($;$$) {
	my $cc = shift @_;
        my $Uname = $_[0] ? $_[0] : 'naroot';
        my $Pwd = $_[1] ? $_[1] : '';
	
	if ($ipv4_ip || $ipv6_global_ip || $ipv6_router_ip) {
		logcomment ("Run a command into $RLM_PROMPT for all IPv4 and IPv6 addresses [NON-INTERACTIVE MODE] \n");
	} else {
		logcomment ("[NON-INTERACTIVE MODE]: No IP is avaliable to run a command into $RLM_PROMPT \n");
        }

	foreach my $ssh_non_interactive_ip ($ipv4_ip, $ipv6_global_ip, $ipv6_router_ip) {
		if ($ssh_non_interactive_ip && ($ssh_non_interactive_ip !~ /Disabled|unknown/i)) {
			next if ($ipv6_gateway =~ /$ssh_interactive_ip/);
			my $option = ($ssh_non_interactive_ip =~ /$ipv4_ip/) ? '-v' : '-v -6';
			my ($prompt, $output) = say($cc, "ssh $option $Uname\@$ssh_non_interactive_ip version", '-noprompt', '-wait', 'password:', 'Are you sure you want to continue connecting', 'prompt__>', '-timeout', 100);
			($prompt, $output) = say($cc, "yes", '-noprompt', '-wait', 'password:', 'prompt__>', '-timeout', 30) if ($prompt  =~ /Are you sure you want to continue connecting/);
			($output) = say($cc, "$Pwd") if ($prompt  =~ /password:/i);
			if ($output !~ /No route to host/s) {
				logresult ('PASS', "Connection to ssh naroot\@$ssh_non_interactive_ip succeeded \n");
			} else {
				logresult ('FATAL', "Connection to ssh naroot\@$ssh_non_interactive_ip failed in non-interactive mode \n");
			}
		}
	}
}

## Validate RLM checkpoints all together. Covers "rlm status", "reboot", "rlm status -d", "ping", "ping -6", "wget", "ssh"
sub rlm_validation_items ($$$;$$) {
	logcomment ("VALIDATION ITEMS ...");
	my ($FILER, $cc, $case) = @_;
        my $Uname = ($_[3] eq 'root') ? 'naroot' : $_[3];
        my $Pwd = $_[4];

	my $fc = rlm_make_conn_to_ontap($FILER);

	#1
	sleep(5); #sp status taking time to display once after sp setup.
	$LOADER ? rlm_verify_rlm_status($FILER, 'LOADER', $case) : rlm_verify_rlm_status($FILER, 'ONTAP', $case);

	#2
        my $rlm_ssh_ip;
        if ($ipv4_ip) {
                $rlm_ssh_ip = $ipv4_ip;
        } elsif (($ipv6_global_ip !~ /Disabled|unknown/i) && $ipv6_global_ip && ($ipv6_global_ip ne $ipv6_gateway)) {
                $rlm_ssh_ip = $ipv6_global_ip;
        } elsif ($ipv6_router_ip) {
                $rlm_ssh_ip = $ipv6_router_ip;
        }

        my $r;
        if ($rlm_ssh_ip) {
                $r = rlm_make_conn_to_rlm ($FILER, $rlm_ssh_ip, $Uname, $Pwd);
                if ($RLM_PROMPT =~ /RLM/) {
                        rlm_verify_rlm_status_rlm($r, $case);
                } else {
                        rlm_verify_rlm_status($r, $RLM_PROMPT, $case);
                }
	#3
                rlm_verify_rlm_status_d($r, 'RLM');
                disconnect($r);
        }

	#4
	rlm_reboot_rlm($FILER);
	sleep(60);	#Sometimes "rlm status" giving ontap prompt immediately, to overcome this issue, sleeping for some time.
	$LOADER ? rlm_verify_rlm_status($FILER, 'LOADER', $case) : rlm_verify_rlm_status($FILER, 'ONTAP', $case);

	#5
#	rlm_verify_rlm_status_d($FILER, 'ONTAP') unless ($LOADER);

	#6 Sometimes DHCP IP many change once after SP reboot. This code will handle to consider the changed IP
	if ($ipv4_ip && ($dhcp_status =~ /yes/i)) {
		$rlm_ssh_ip = $ipv4_ip;
	}

	if ($rlm_ssh_ip) {
		$r = rlm_make_conn_to_rlm ($FILER, $rlm_ssh_ip, $Uname, $Pwd);
		if ($RLM_PROMPT =~ /RLM/) {
			rlm_verify_rlm_status_rlm($r, $case);
		} else {
			rlm_verify_rlm_status($r, $RLM_PROMPT, $case);
		}
		rlm_verify_rlm_status_d($r, 'RLM');
		disconnect($r);
	}

	#7
	%ping_items = (
		1	=> ['Ping both IPv4 and IPv6 addresses'],
		2	=> ['Ping flood both IPv4 and IPv6 addresses', ['-f']],
		3	=> ['Ping IPv4 and IPv6 addresses with a preloaded burst of packets', ['-l 3']],
		4	=> ['Change the size of the data packet for ICMPv4 / ICMPv6 and ping IPv4 and IPv6 addresses', ['-s 1000']],
		5	=> ['Ping IPv4 and IPv6 addresses with limited ICMPv4 / ICMPv6 packets', ['-c 10']],
		6	=> ['Figure out ICMPv4 / ICMPv6 Node Information Node Address query, than a echo-request', ['-a']],
		7	=> ['Try a combination of -f, -a, -s, -l with a single ping6 command', ['-l 3', '-s 1000', '-a']],
	);

	foreach my $key (sort keys %ping_items) {
		foreach my $ip ($ipv4_ip, $ipv6_global_ip, $ipv6_router_ip) {	#, $ipv6_link_local_ip) {
			if ($ip && ($ip !~ /Disabled|unknown/i) && ($ip !~ /$ipv6_gateway/i)) {
				rlm_ping ($cc, $ip, $ping_items{$key});
				rlm_ping ($cc, $ip, $ping_items{$key}, 'sudo');
			}
		}
	}

	#8: SSH INTERACTIVE MODE
	rlm_ssh_interactive_mode($FILER, $Uname, $Pwd);

	#9: SSH NON-INTERACTIVE MODE
	rlm_ssh_non_interactive_mode($cc, $Uname, $Pwd);

	if ($rlm_ipv6_router && !$LOADER) {
		rlm_is_ipv6_ra_enable($FILER, 'on');
	}
}

# convert $spec (which might be a hostspec or connspec) to a
# connection object (to console, if not specified)

sub rlm_make_conn_to_ontap ( $ ) {
    my ($spec) = @_;
    if (ref($spec) =~ /Connectrec/) {
        return $spec;
    } elsif (ref($spec) =~ /Hostrec/) {
        return connect($spec->{id} . "#console");
    } else {
        return connect($spec . "#console");
    }
}

# convert $spec (which might be a hostspec or connspec) to a
# host object

sub rlm_make_host_to_ontap ( $ ) {
    my ($spec) = @_;
    if (ref($spec) =~ /Connectrec/) {
        return $spec->{hostp};
    } else {
        return host($spec);
    }
}

sub rlm_ctrl_g_login ($;$$) {
        my $FILER = shift;
        my $Uname = $_[0] ? $_[0] : 'naroot';
        my $Pwd = $_[1] ? $_[1] : '';
	my $old_pwd;

        my $fc = rlm_make_conn_to_ontap($FILER);

        if ($Uname =~ /^root$/i) {
                my (undef, undef, undef, $prompt) = say ($fc, " ", '-swallow', '-prompt', @prompts, '-timeout', 10);
                if ($prompt =~ /LOADER/) {
                        $Uname = "root";
                        if(!transit_to($h, 'UP', username=>$Uname, password=>$Pwd)) {
                                logresult ('FATAL', "The filer is unable to boot up normally.");
                        }
			sleep 80;
                        (undef, undef, undef, $prompt) = say ($fc, "version", '-swallow', '-prompt', @prompts, '-timeout', 10);
                }

                if ($prompt =~ /$FILER/) {
                        say ($fc, "priv set test", '-swallow', '-prompt', @prompts, '-timeout', 10);
                        say ($fc, "$RLM debugmode", '-swallow', '-prompt', @prompts, '-timeout', 10);
                }

                $Pwd = 'superuser';
        }

        my (undef, undef, undef, $prompt) = say ($fc, "\cg", '-swallow', '-noprompt', '-prompt', 'Service Processor Login', '-timeout', 10);

	push @prompts_sp, '#';
        if ($prompt =~ /Service Processor Login/i) {
                (undef, undef, undef, $prompt) = say ($fc, "$Uname", '-swallow', '-noprompt', '-prompt', @prompts_sp, 'Password', '-timeout', 10);
        }
        if ($prompt =~ /Password/i) {
                (undef, undef, undef, $prompt) = say ($fc, "$Pwd", '-swallow','-noprompt', '-prompt', @prompts_sp, 'Service Processor Login', '-timeout', 10);
        }
	pop @prompts_sp;

        if ($prompt =~ /$RLM_PROMPT.*>/) {
                logcomment ("Filer is at $RLM_PROMPT console");
        } elsif (($prompt =~ /\#/) && ($Uname eq 'root')) {
                logcomment ("Filer is at $RLM_PROMPT root cnosole");
        } elsif ($prompt =~ /Service Processor Login/) {
                logresult ('FATAL', "Incorrect login credentials: Login: $Uname, Password: $Pwd");
        } else {
                logresult ('FATAL', "Can not take to $RLM_PROMPT console");
        }
}

sub rlm_ctrl_g_logout ($) {
        my $FILER = shift;
        my $fc = rlm_make_conn_to_ontap($FILER);
        logcomment ("Logging out from the $RLM_PROMPT console");
        say($fc, "\cd", '-swallow', '-noprompt', '-timeout', 10);
        eval { (undef, undef, undef, $prompt) = say($fc, " ", '-swallow', '-prompt', @prompts, '-timeout', 10) };
	$prompt = "" if ($@);
        return $prompt;
}

sub rlm_get_test_case_execution_order ($$$;$$) {
#	my %matrix_args = (
#		Test_Case => "",
#		COMBINATIONS => "",
#		test_cases_hash => \%test_states,
#		Test_Case_Validity_Func => \&rlm_verify_ipv4_ipv6_testcase_validity,
#		RANDOM => 0,
#	);

	my $Test_Case = shift @_;
	my $COMBINATIONS = shift @_;
	my $test_cases_hash = shift @_;
	my $Test_Case_Validity_Func = shift @_;
	my $RANDOM = $_[0] ? $_[0] : 0;
	my $RANGE = 0;
	my @matrix;

	if ($Test_Case =~ /ALL/i && !$COMBINATIONS) {
		$COMBINATIONS = join (",", sort {$a<=>$b} keys %{$test_cases_hash});
	} elsif ($Test_Case > 0 && $Test_Case <= scalar(keys %{$test_cases_hash}) && !$COMBINATIONS) {
		$COMBINATIONS = $Test_Case;
	} elsif (!$COMBINATIONS) {
		$Test_Case_Validity_Func->($Test_Case);
	}

	@matrix = $COMBINATIONS =~ /(\d+)\s*(-|\.\.)\s*(\d+)/g;
	if (($matrix[1] =~ m{-|\.\.}) && (scalar(@matrix) == 3)) {
		splice(@matrix, 1,1);
		$RANGE = 1;
	} else {
		@matrix = $COMBINATIONS =~ /[\s]*(\d+)[\s]*/g;
	}	

	if ($RANGE==1) {
		if ($matrix[0]>$matrix[1]) {
			@matrix = ($matrix[1], $matrix[0]);
		}
		@matrix = ($matrix[0]..$matrix[1]);
	}	

#	foreach my $c (@matrix) {
#	   unless (exists($test_cases_hash->{$c})) {
#		logresult ('INFO', "Invalid state: $c in the combination list");
#	   }
#	}
	
	for (my $i=$#matrix; $i>=0; $i--) {
		unless (exists($test_cases_hash->{$matrix[$i]})) {
			logresult ('INFO', "Invalid state: $matrix[$i] in the combination list");
			splice(@matrix, $i, 1) unless ($NO_IPv6);
		}
	}	

	if ($NO_IPv6) {
		my @skipping_cases;
		for (my $i=$#matrix; $i>=0; $i--) {
			if ($matrix[$i] > $IPv4_ONLY) {
				push @skipping_cases, splice(@matrix, $i, 1);
			}
		}
		if (scalar(@skipping_cases)) {
			@skipping_cases = reverse @skipping_cases;
			logresult ('WARN', "Following test states/cases will be omitted from execution as \"$NO_IPv6\": @skipping_cases");
		}
	}

	unless (scalar(@matrix)) {
		logresult ('NA', "None of the test cases are valid: $COMBINATIONS. See the tharndoc for more info");
	}

	if (scalar(@matrix) > 1) {
		if ($RANDOM == 1) {
			logcomment ("Selecting test states randomly from the list");
			my %combinations_hash;
			my @combinations_case_coverage_order;
			until (scalar (keys %combinations_hash) == scalar(@matrix)) {
				my $index = int rand (scalar @matrix);
				next if (exists $combinations_hash{$index});
				$combinations_hash{$index} = '';
				push @combinations_case_coverage_order, $matrix[$index];
			}
			@matrix = @combinations_case_coverage_order;
			logcomment ("Executing test cases/states Randomly");
		} else {
			logcomment ("Executing test cases/states Sequentially");
		}
	}

	return (\@matrix);
}

sub get_filer_state($)
{
  my $conn=shift;
#  push(@prompts,"$FILER%");
  my (undef,undef,$after,$prompt) = say($conn,"?",-prompt,@prompts,-swallow);

        logcomment(" get_filer_state ; after = $after");
        logcomment(" get_filer_state ; prompt = $prompt");

  return MAINTENANCE_PROMPT if($prompt =~ /^\*>$/);
  return SP_BOOT_MENU if($prompt =~ /Selection/);
  return ONTAP_PROMPT if($after =~ /sysconfig/);
  return LOADER_CFE_PROMPT if($after =~ /printenv/);
  return SYS_SHELL_PROMPT if($after =~ /No match/);
} ## end sub get_filer_state

sub Validation_Items($$) {
        logcomment("Executing validation items");
        my ($fc, $rlm_naroot) = @_;
        say($rlm_naroot, "sp status -d", -swallow,  -prompt, @prompts, -timeout, 1500);
        say($rlm_naroot, "events newest 100", -swallow,  -prompt, @prompts, -timeout, 300);
        $filer_state=get_filer_state($fc);
        if ($filer_state =~ "ONTAP_PROMPT")
        {
           say($fc, "priv set test;sp status -d", -swallow,  -prompt, @prompts, -timeout, 1500);
        }

} ## end sub Validation_Items


#------------------------------------------------------------------------------
# FUNCTION: Return the result after executing the sp ipmi raw command
# INPUT:  fc      - filer connection handle
#         FILER   - filer name
#         user    - user name by which sp should be connected
#         pwd     - password for the particular user.
#         cmd     - command to execute
# RETURN: Output of the command
#------------------------------------------------------------------------------
sub Get_SP_ROOT_IPMI_Result($$$$$$;$) {

    my ($fc, $FILER, $user, $pwd, $cmd, $wait_result) = @_;
    my ($prompt, $result);
    
	my (undef, undef, undef, $prompt) =
      say($fc, "\cg", '-swallow', '-noprompt', '-prompt',
          'Service Processor Login',
          '-timeout', 10);

    if ($prompt =~ /Service Processor Login/i) {
        (undef, undef, undef, $prompt) =
          say($fc, "$user", '-swallow', '-prompt', "Password:", -timeout, 10);
    }
    if ($prompt =~ /Password/i) {
        (undef, undef, undef, $prompt) =
          say($fc, "$pwd", '-swallow', '-noprompt', '-prompt', "#", '-timeout',
              10);
    }

    (undef, undef, undef, $prompt) = say($fc, "cd /usr/local/bin",
                       '-swallow', '-noprompt', '-prompt', "#", '-timeout', 10);

    if ($wait_result) {
        (undef, undef, $result, undef) =
          say($fc, "$cmd", '-swallow', '-prompt', "# ", '-timeout', 60);

    } else {
        (undef, undef, $result, undef) =
          say($fc, "$cmd", '-swallow', '-noprompt', '-timeout', 10);
        sleep(120);

    }

    say($fc, "\cd", '-swallow', '-noprompt', '-timeout', 10);
    say($fc, " ", '-swallow', '-prompt', @prompts, '-timeout', 10);

    return $result;

} ## end sub Get_SP_ROOT_IPMI_Result($$$$$$)

#------------------------------------------------------------------------------
# FUNCTION: Return the result after executing the sp ipmi raw command
# INPUT:  fc      - filer connection handle
#         FILER   - filer name
#         user    - user name by which sp should be connected
#         pwd     - password for the particular user.
#         cmd     - command to execute
# RETURN: Output of the command
#------------------------------------------------------------------------------
sub Get_SP_IPMI_Result($$$$$) {

    my ($fc, $FILER, $user, $pwd, $cmd) = @_;
    my ($prompt, $result);

    ## Enter into sp debugmode
    logStep("Entering into sp debugmode");
    say($fc, "priv set test",
    '-swallow', '-prompt', "$FILER.*> ",
    '-timeout', 60);

    say($fc, "sp debugmode",
        '-swallow', '-prompt', "$FILER.*> ",
        '-timeout', 60);

    my (undef, undef, undef, $prompt) =
      say($fc, "\cg", '-swallow', '-noprompt', '-prompt',
          'Service Processor Login',
          '-timeout', 10);

    if ($prompt =~ /Service Processor Login/i) {
        (undef, undef, undef, $prompt) =
          say($fc, "$user", '-swallow', '-prompt', "$RLM_PROMPT $FILER.*>",'Password:',
              -timeout, 10);
    }
    if ($prompt =~ /Password/i) {
        (undef, undef, undef, $prompt) =
          say($fc, "$pwd", '-swallow', '-noprompt', '-timeout', 10);
    }

    say($fc, "priv set test",
        '-swallow', '-prompt', "$RLM_PROMPT $FILER.*>",
        '-timeout', 10);

    (undef, undef, $result, undef) =
      say($fc, "$cmd", '-swallow', '-prompt', "$RLM_PROMPT $FILER.*>",
          '-timeout', 60);

    say($fc, "\cd", '-swallow', '-noprompt', '-timeout', 10);
    say($fc, " ", '-swallow', '-prompt', @prompts, '-timeout', 10);

    return $result;

} ## end sub Get_SP_IPMI_Result($$$$$)

sub Reboot_Time($) {

    my $Platform = shift;
    my $Time;
    if ($Platform =~ /Buell|Bimota/) {
    $Time = 180;
    } elsif ($Platform =~ /Vespa|Carnegie|Absolut/) {
    $Time = 120;
    }
    return $Time;
}
1;
