

package platval::service_processor::sp_autoupdate;

use strict;
use warnings;

use TCD;
use Tharn;
use NACL::APISet;
use NACL::Transit;
use Data::Dumper;
use Time::Local;
use NACL::C::Node;
use NACL::C::SystemNodeImage;
use NACL::CS::SystemNodeServiceProcessorImage;
use NACL::CS::SystemNodeServiceProcessor;
use NACL::CS::SystemNodeServiceProcessorImageUpdateProgress;
use NACL::C::SystemNodeServiceProcessorImage;
use Time::Zone;
use Time::Local;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
  sp_update_from_sp
  enable_disable_autoupdate
  Copy_SPFW
  Add_Time_To_Periodic_Check_Time
  Check_Periodic_Time
  sleep_time
  Monitor_Update_Progress
  SP_Auto_Update_Not_Triggered
  Pre_Update_Check 
  Get_Firmware_Path 
  $Retry_Count
);

our ($Retry_Count);

#----------------------------------------------------------------------------
# FUNCTION:     To update SP Firmware from SP CLI.
# INPUT   :     $_[0]= SP Connection handle, 
#               $_[1]= $Node,
#               $_[2]= SP version.
# RETURN  :     1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub sp_update_from_sp($$$) {		
	my($response, $parsed_output);	
	my ($APISetSP, $Node, $url, $version) = @_;
	
    $APISetSP->set_timeout('timeout' => 600);	
	$APISetSP->api_reconnect_session() if !$APISetSP->api_is_session_alive();
	my $return = enable_disable_autoupdate($Node, "false");
	unless($return){
		return 0;		
	}	
#	$response = $APISetSP->execute_raw_command( command => "sp update $url -f" );
#    if($response =~ /SP FW Update Successful/i) {
eval{
            my $prompts_answers = ['.*Do you want to reboot the SP now.*' => 'y',];
          $response = $APISetSP->execute_raw_command( command => "sp update $url -f", 'connectrec-match_table' => $prompts_answers,);
      };    
    if($response =~ /sp reboot/i) {
        eval {
            my @prompts_answers = ( '.*The Service Processor will be rebooted.*' => 'y', );
            $APISetSP->sp_reboot('connectrec-match_table' => \@prompts_answers, ); 
      };          
    }
    	sleep(150);
    	my ($timer,$status,$time_taken) = (0,0,120);
    	while($timer <= 20) { 
    		$response = NACL::CS::SystemNodeServiceProcessor->fetch(command_interface => $Node, filter => {node => "local"});    		
    		if($response->status() =~ /Online/i){
    			$status = 1;
    		    $timer = 21;
    		}else {
    			sleep(30);
    		}
    		$timer += 1;
    		$time_taken += 30;
    	}
    	unless ($status) {
    		tstlog -comment, "SP is not getting back to online state even after waiting for $time_taken seconds approximately ";
    		return 0;
    	}
    	tstlog -comment, "SP is online : Took $time_taken seconds approximately to come online ";
    	$APISetSP->api_reconnect_session() if !$APISetSP->api_is_session_alive();
    	$response = NACL::CS::SystemNodeServiceProcessor->fetch(command_interface => $Node, filter => {node => "local"}); 
    	my $updated_version = $response->fw_version();        
        if($updated_version eq $version) {
        	$Log->comment("SP has been updated with firmware version : $version");
        }else {
        	tstlog -comment, "SP has not been updated with firmware version : $version";
        	return 0;
        }
#   else {
#  	tstlog -comment, "SP Firmware was not updated successfully";
# 	return 0;
#}    
    return 1;	
}

#----------------------------------------------------------------------------
# FUNCTION:     Enable or Disable autoupdate
# INPUT   :     $_[0]= Component ref, 
#               $_[1]= true/false.               
# RETURN  :     1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub enable_disable_autoupdate($$){
	my $Node = $_[0];
	my $desired_state = $_[1];	
	my $response = NACL::CS::SystemNodeServiceProcessorImage->fetch(command_interface => $Node, filter => {node => "local"});
	my $auto_update = $response->autoupdate();
	tstlog -info, "Initally Auto-Update has been set to $auto_update";
	if($auto_update ne $desired_state){
		NACL::C::SystemNodeServiceProcessorImage->modify(command_interface => $Node, node => "local", autoupdate => $desired_state);
		sleep(10);
		$response = NACL::CS::SystemNodeServiceProcessorImage->fetch(command_interface => $Node,filter => {node => "local"});
		$auto_update = $response->autoupdate();
		tstlog -info, "After modifying Auto-Update field has been set to $auto_update";
		if($auto_update ne $desired_state){
			tstlog -comment, "ERROR : Autoupdate is not set to $desired_state";
			return 0;
		}		
	}
	
	return 1;	
		
}

#----------------------------------------------------------------------------
# FUNCTION:     copy sp fw to /mroot/etc/software
# INPUT   :     $_[0]= ONTAP Connection handle, 
#               $_[1]= sp fw version.               
# RETURN  :     1 = Success, 0 = Failure
#----------------------------------------------------------------------------
sub Copy_SPFW($$){	
	my($APISetSystemshell, $url) = @_;	
	$APISetSystemshell->set_timeout('timeout' => 600);
	$APISetSystemshell->execute_raw_command(command => "cd /mroot/etc/software/");	 
	my $response = $APISetSystemshell->execute_raw_command(command => "wget $url");
	unless($response =~ /100%/){
		tstlog -comment, "Unable to copy sp firmware to /mroot/etc/software";
		return 0;
	}
	return 1;
}

sub Add_Time_To_Periodic_Check_Time($$){
	my @current = @{$_[0]};
	my $add = $_[1];
	
	my %months = (Jan => 1, Feb => 2, Mar => 3, Apr => 4, May => 5, Jun => 6, Jul => 7,
	              Aug => 8, Sep => 9, Oct => 10, Nov => 11, Dec => 12);
	$current[0] = $months{$current[0]} if exists $months{$current[0]}; 	 
	if($current[5] < 59){
		$current[5] = 59;
	}          
	my $current = timelocal($current[5], $current[4], $current[3], $current[1], $current[0] - 1, $current[2]);
	
	my ($sec,$min,$hour,$mday,$month,$year) = localtime($current + $add * 60);
	$year += 1900;
    $month = $month + 1;
    
    my @periodic_check = ($month, $mday, $year, $hour, $min, $sec);
    foreach (@periodic_check){
    	if($_ < 10){
    		$_ = 0 . $_;
    	}
    	
    }      
    my $grep_time = join ':' , ($periodic_check[3], $periodic_check[4]);
	
	return (\@periodic_check , $grep_time);
	
}

sub Check_Periodic_Time($){
	my ($response, @current_time);	
	$response = shift;
	my @response = split /\n/, $response;
	foreach(@response){
		if(/.+?\w{3}\s(\w{3}\s\d{2}\s\d{4}\s\d\d:\d\d:\d\d).+?Periodic.*/i){
			@current_time = split /[:\s]/, $1;			
		}
	}	
	if(@current_time){
		return \@current_time;
	}else{
		return 0;
	}
	
}

sub sleep_time($$){
	my @date = @{$_[0]};
	
	my @periodic_time = @{$_[1]};
    my %months = (Jan => 1, Feb => 2, Mar => 3, Apr => 4, May => 5, Jun => 6, Jul => 7,
	              Aug => 8, Sep => 9, Oct => 10, Nov => 11, Dec => 12);

$date[1] = $months{$date[1]} if exists $months{$date[1]};

my $date = timelocal($date[5], $date[4], $date[3], $date[2], $date[1]-1, $date[-1]);
my $periodic_time = timelocal($periodic_time[5], $periodic_time[4], $periodic_time[3], $periodic_time[1], $periodic_time[0]-1, $periodic_time[2]);

my $elapsed = $periodic_time - $date;

return $elapsed;
	
}

#----------------------------------------------------------------------------
# FUNCTION:     Check auto-update is not triggered when auto-update is turned off.
# INPUT   :     $_[0]= ONTAP Connection handle, 
#               $_[1]= periodic_check_time.               
# RETURN  :     1 = Success, 0 = Failure
#----------------------------------------------------------------------------

sub SP_Auto_Update_Not_Triggered($$){
	my $APISetSystemshell = $_[0];
	my $periodic_check_time = $_[1];
	my ($date_response, @date, $sleep_time, $periodic_time, $grep_time, $wait_time);
	my $count_down = 10;
	my $return = 1;
	my $update_started = "false";
	$wait_time = 3;	
	$periodic_time = $periodic_check_time;		
	while($count_down > 0){
		tstlog -info, "COUNT DOWN : $count_down";
		if($wait_time == 3){
			($periodic_time, $grep_time) = Add_Time_To_Periodic_Check_Time($periodic_check_time, $wait_time);
		}else{
			($periodic_time, $grep_time) = Add_Time_To_Periodic_Check_Time($periodic_time, $wait_time);
		}
		
		$date_response = $APISetSystemshell->execute_raw_command(command => "date");
		@date = split /[:\s]+/, $date_response;
		$sleep_time = sleep_time(\@date, $periodic_time);
		$sleep_time = $sleep_time;
		tstlog -comment, "sleeping for $sleep_time seconds";		    
		sleep($sleep_time);
		
		my $response = $APISetSystemshell->execute_raw_command(command => "cat /mroot/etc/log/mlog/servprocd.log | grep $grep_time");
		my $out = Check_Periodic_Time($response);
	    if($out){
	    	$periodic_check_time = $out;
	    }
	    
	    my @response = split /\n/, $response;
	    foreach(@response){
	    	if(/SP auto-update is currently disabled/){
	    		tstlog -info, "SP auto-update is Disabled.\n $_"; 	    		
	    		last;   		
	    	}elsif(/New SP Firmware package not available/){
	    		tstlog -info, "New SP Firmware package not available\n $_"; 	    		
	    		last;
	    	}elsif(/SP: Firmware update started./){
	    		tstlog -fail, "SP auto-update is Enabled and SP Firmware update has been triggered.\n $_";
	    		$update_started = "true";
	    		last; 
	    	}
	    }
	    if($update_started eq "true"){
	    	$return = 0;
	    	last;
	    }else{
	    	$wait_time = 1;
	    	$count_down--;
	    }    
	}
	return $return;	
}

#----------------------------------------------------------------------------
# FUNCTION:     Verify pre-update check.
# INPUT   :     $_[0]= ONTAP Connection handle, 
#               $_[1]= periodic_check_time.               
# RETURN  :     1 = Success, 0 = Failure
#----------------------------------------------------------------------------

sub Pre_Update_Check($$$$$){
	
	my ($date_response, @date, $sleep_time, $periodic_time, $grep_time, $wait_time);
	my $count_down = 7;
	my ($APISetSystemshell, $Node, $periodic_check_time, $wait_time, $update_type) = @_;
	my $return = 1;
		
	$periodic_time = $periodic_check_time;
	
	while($count_down > 0){
		$APISetSystemshell->api_reconnect_session() if !$APISetSystemshell->api_is_session_alive();
		log->comment("COUNT DOWN : $count_down");
		if($wait_time == 3){
			($periodic_time, $grep_time) = Add_Time_To_Periodic_Check_Time($periodic_check_time, $wait_time);
		}else{
			($periodic_time, $grep_time) = Add_Time_To_Periodic_Check_Time($periodic_time, $wait_time);
		}
		
		$date_response = $APISetSystemshell->execute_raw_command(command => "date");
		@date = split /[:\s]+/, $date_response;
		$sleep_time = sleep_time(\@date, $periodic_time);
		$sleep_time = $sleep_time;
		tstlog -comment, "sleeping for $sleep_time seconds";		    
		sleep($sleep_time);
		
		my $response = NACL::CS::SystemNodeServiceProcessorImage->fetch(command_interface => $Node, filter => {node => "local"});
	    my $image_status = $response->status();
	    tstlog -comment, "sp autoupdate status : $image_status";
	
	    $response = NACL::CS::SystemNodeServiceProcessorImageUpdateProgress->fetch(command_interface => $Node, filter => {node => "local"});
	    my $in_progress = $response->in_progress();
	    my $percentage_done = $response->percent_done();
	    tstlog -comment, "sp autoupdate in progress : $in_progress and percentage done is $percentage_done";
	
	
	    $response = NACL::CS::SystemNodeServiceProcessor->fetch(command_interface => $Node, filter => {node => "local"});
	    my $sp_status = $response->status();
        tstlog -comment, "SP Status : $sp_status";
        
        $response = $APISetSystemshell->execute_raw_command(command => "cat /mroot/etc/log/mlog/servprocd.log | grep $grep_time");
	    my $out = Check_Periodic_Time($response);
	    if($out){
	    	$periodic_check_time = $out;
	    }
	    
	    my @response = split /\n/, $response;
	    my $failed_event = 0;		
		my $maximum_retries = 0;
		my $new_fw_avalible = 0;
		my $auto_update_enabled = 0;
		foreach(@response){
			if(/SP firmware update from  (\S+?) to (\S+?) has been triggered|SP firmware transfer is in progress/){
				tstlog -info, "SP Update has been triggered sucessufully. \n $_";								
			}elsif(/Maximum SP auto-update retries value has been reached : 3/){
				tstlog -info, "SP Auto-update failed. \n $_";
				$maximum_retries = 1;
				last;
			}elsif(/ERR: Servprocd::SpUpdate:/){
				tstlog -fail, "$_";
				$failed_event = 1;
			}elsif(/auto_update_retry_count : (\d)/){
				tstlog -info, "Current retry count is $1. \n $_";
				$Retry_Count = $1;
			}elsif(/New SP Firmware package not available/){
	    		tstlog -info, "New SP Firmware package not available\n $_"; 
	    		$new_fw_avalible++;	    		
	    		last;
			}elsif((/SP Autoupdate is NOT Enabled/i) && ($update_type =~ /Auto-Updating/i)){
				tstlog -info, "SP Auto Update is not enabled. \n $_";
				$auto_update_enabled = 1;
				last;
			}	
		}
		if($maximum_retries){
			$return = 0;
			last;
		}elsif($failed_event){
			$wait_time = 3;
		}elsif($new_fw_avalible == 3){
			$return = 0;
			last;
		}elsif($auto_update_enabled){
			$return = 0;
			last;
		}else{
			$wait_time = 1;
			$count_down--;
		}
	}
	
	return $return;	
}


# Input : $_[0] = systemshell connection handle
#         $_[1] = $Node
#         $_[2] = $periodic_check_time
#
#
sub Monitor_Update_Progress($$$$$$;$){	
	my $count_down = 60;
	my ($skip_count_previous,      $sp_status,        $grep_time,
	    $skip_count,               $image_status,     $percentage_done,
	    $in_progress,              @errors,           $response,
	    $fw_version);
	my ($APISetSystemshell, $Node, $periodic_check_time, $wait_time, $update_type, $update_to_fw, $optional) = @_;	
	my ($auto_reboot_flag,         $reboot_ems,                  $reboot_servprocd, 
	    $failed_event_flag,        $err_update_engine_servprocd, $err_update_engine_ems,
	    $update_started_flag,      $update_start_ems,            $update_start_servprocd,
	    $post_update_check_flag,   $post_check_servprocd,        $post_check_ems,
	    $update_completed_flag,    $update_completed_ems,        $update_completed_servprocd);
	    
	$auto_reboot_flag=$reboot_ems=$reboot_servprocd=$failed_event_flag=$err_update_engine_servprocd=$err_update_engine_ems=
	$update_started_flag=$update_start_ems=$update_start_servprocd=$post_update_check_flag=$post_check_servprocd=
	$post_check_ems=$update_completed_flag=$update_completed_ems=$update_completed_servprocd= "false";

	my $end_time  = time() + 2000; #(in sec)
	my $exit_flag = 0;	
	my $error_flag = 0;
	my $periodic_time = $periodic_check_time;
	my %verified = ('update_started' => "false",
	                'post_update_check' => "false",
	                'auto_reboot'=> "false",
	                'update_completed' => "false");
	                
	while($count_down > 0){
		$APISetSystemshell->api_reconnect_session() if !$APISetSystemshell->api_is_session_alive();
		tstlog -comment, "COUNT DOWN : $count_down";
		if($wait_time == 3){
			($periodic_time, $grep_time) = Add_Time_To_Periodic_Check_Time($periodic_check_time, $wait_time);
		}else{
			($periodic_time, $grep_time) = Add_Time_To_Periodic_Check_Time($periodic_time, $wait_time);
		}
		
		my $date_response = $APISetSystemshell->execute_raw_command(command => "date");
		my @date = split /[:\s]+/, $date_response;
		my $sleep_time = sleep_time(\@date, $periodic_time);
		$sleep_time = $sleep_time + 10;
		tstlog -comment, "sleeping for $sleep_time seconds";		    
		sleep($sleep_time);
		
		#Verifying Otap CLI content
		$response = NACL::CS::SystemNodeServiceProcessorImage->fetch(command_interface => $Node, filter => {node => "local"});
	    $image_status = $response->status();

	    my $ems_temp = $APISetSystemshell->execute_raw_command(command => "cat /mroot/etc/log/ems | grep sp_ -B 2 -A 2");
	    if($ems_temp =~ /SP Firmware update has been successfully scheduled|SP firmware update from.*has been triggered/i)
	    {
            	$update_started_flag = "true";
                tstlog -comment, "SP Update was sucessfully triggered. \n $_";
                $update_start_ems = "true";


		while (time() < $end_time)
            	{
                	$response = NACL::CS::SystemNodeServiceProcessorImageUpdateProgress->fetch(command_interface => $Node, filter => {node => "local"});
	                $in_progress = $response->in_progress();
        	        $percentage_done = $response->percent_done();
                	tstlog -info, "sp autoupdate progress in percentage : $percentage_done";
	                tstlog -info, "SP update in progress : $in_progress";
        	        if($in_progress =~ /failed/)
                	{
                        	tstlog -fail, "SP auotupdate showing as fail";
                                last;
                        }
                        elsif($percentage_done =~ /100/)
                        {
                                tstlog -info, "SP auotupdate showing 100 percentage complete";
                                last;
                        }
			sleep(100);
            	}
            }

            my $ems_temp = $APISetSystemshell->execute_raw_command(command => "cat /mroot/etc/log/ems | grep sp_ -B 2 -A 2");

	    if($ems_temp=~ /SP:\s+Firmware\s+update\s+completed/)
	    {
            	$update_completed_flag = "true";
                tstlog -comment, "SP firmware update completed \n $_";
                $update_completed_ems = "true";
	    }

	    $response = NACL::CS::SystemNodeServiceProcessor->fetch(command_interface => $Node, filter => {node => "local"});
	    $sp_status = $response->status(); 
	    $fw_version = $response->fw_version();	     
	    
	    #verifying ems.
	    my $response_ems = $APISetSystemshell->execute_raw_command(command => "cat /mroot/etc/log/ems | grep sp_ -B 2 -A 2");
		my @response_ems = split /<LR/, $response_ems;	
		foreach (@response_ems){
			if(/SP:\s+?Triggering\s+?auto-reboot\s+?to\s+?complete\s+?firmware\s+?update/i){
				$auto_reboot_flag = "true";
				tstlog -comment, "EMS has been generated for SP automatic reboot. \n $_";						
				$reboot_ems = "true";
			}elsif(/SP Firmware update has been successfully scheduled|SP firmware update from.*has been triggered/i){
				$update_started_flag = "true";
				tstlog -comment, "SP Update was sucessfully triggered. \n $_";
				$update_start_ems = "true";
			}elsif(/sp_upd_unexpt_evt/){
				$failed_event_flag = "true";
				if((defined $optional) && $optional =~ /update-engine-err/){
					tstlog -comment, "Update failed after injecting update-engine-err\n $_";
					$err_update_engine_ems = "true";
				}else{
					tstlog -fail, "Unexpected error occured while sp update in progress\n $_";					
					push @errors, $_;
					$error_flag = 1;
				}
			}elsif(/post-update check.*(failed|passed)/i){
				$post_update_check_flag = "true";
				my $post_result_ems = $1;
				if((defined $optional) && ($optional =~ /post-update-err/) && ($post_result_ems =~ /failed/i)){
					tstlog -info, "SP PostUpdate check $post_result_ems as expected after injecting error\n $_";
					$post_check_ems = "true";					
				}elsif((defined $optional) && ($optional =~ /post-update-err/) && ($post_result_ems !~ /failed/i)){
					tstlog -fail, "SP PostUpdate check $post_result_ems. Not displaying expected result after injecting error\n $_";
					push @errors, $_;
					$error_flag = 1;
				}elsif($post_result_ems =~ /passed/i){
					tstlog -info, "SP PostUpdate check $post_result_ems\n $_";
					$post_check_ems = "true"; 
				}else{
					tstlog -fail, "SP PostUpdate check $post_result_ems.";
					push @errors, $_;
					$error_flag = 1;
				}
			}elsif(/SP:\s+Firmware\s+update\s+completed/){
				$update_completed_flag = "true";
	    	    tstlog -comment, "SP firmware update completed \n $_";
	    	    $update_completed_ems = "true";
	        }elsif(/Maximum SP auto-update retries value has been reached./){
				if((defined $optional) && ($optional =~ /update-engine-err/)){
					tstlog -comment, "Reached maximum retries after injecting update-engine-err\n $_";
					$exit_flag = 3;
				}else{
					tstlog -fail, "Reached maximum retries update has been blocked\n $_";					
					push @errors, $_;
					$error_flag = 1;
					$exit_flag = 3;
				}
			}
		} 
		
		sleep(35);
		#Verifying sevprocd		
	    
	    
        
		
		#Validation
                if($exit_flag == 3){
                        last;
                }elsif(($auto_reboot_flag eq "true") &&  ($verified{'auto_reboot'} eq "false")){
                        if(($reboot_ems eq "true")){
                                tstlog -comment, "ems log was generated for Auto Reboot";
                        }elsif(($reboot_ems ne "true") ){
                                push @errors, "ems log was generated for Auto Reboot";
                                $error_flag = 1;
                        }
			elsif($reboot_ems ne "true"){
				push @errors, "ems not generated for Auto Reboot" ;
				$error_flag = 1;
			}
			if(($sp_status =~ /rebooting|unknown/i)){
				tstlog -comment, "Displaying sp status as expected";
			}else{
				tstlog -comment, "Not displaying sp status as expected";
			}
			$verified{'auto_reboot'} = "true";									
		}elsif(($failed_event_flag eq "true") && ($optional =~ /update-engine-err/)){
			if(($err_update_engine_ems eq "true")){
				tstlog -comment, "ems logs were generated for failed event after injecting error";
			}elsif(($err_update_engine_ems ne "true")){
				push @errors, "ems logs not generated for failed event after injecting error";
				$error_flag = 1;
			}elsif($err_update_engine_ems ne "true"){
				push @errors, "ems not generated for failed event after enjecting error" ;
				$error_flag = 1;
			}				
		}elsif(($update_started_flag eq "true") && ($verified{'update_started'} eq "false")){
			if(($update_start_ems eq "true") ){
				tstlog -comment, "ems logs were generated for pre update check";
			}elsif(($update_start_ems ne "true")){
				push @errors, " ems logs not generated for pre update check";
				$error_flag = 1;
			}elsif($update_start_ems ne "true"){
				push @errors, "ems not generated for pre update check" ;
				$error_flag = 1;
			}	
			$verified{'update_started'} = "true";
			if(($verified{'update_started'} eq "true") && $optional =~ /exit_on_update_start/){
				last;
			}
		}elsif(($post_update_check_flag eq "true") && ($verified{'post_update_check'} eq "false")){
			if(($post_check_ems eq "true")){
				tstlog -comment, " ems  log was generated for postupdate check";
			}elsif(($post_check_ems ne "true")){
				push @errors, "ems not generated for postupdate check";
				$error_flag = 1;
			}elsif($post_check_ems ne "true"){
				push @errors, "ems not generated for post update check" ;
				$error_flag = 1;
			}
			#verifing ontal cli 
			if($sp_status !~ /online/i){
	        	tstlog -info, "SP STATUS : $sp_status \n SP has come online after auto reboot : $sp_status.";
	        }elsif($sp_status =~ /offline/i){
	        	push @errors, "SP STATUS : $sp_status \n SP went offline after auto reboot.";
	        	$error_flag = 1;
	        }
	        if($fw_version =~ /$update_to_fw/i){
	        	tstlog -info, "Firmware installed : $fw_version \n SP is displaying the updated version";
	        }else{
	        	push @errors, "Firmware installed : $fw_version \n SP id not displaying the expected version";
	        	$error_flag = 1;
	        }
	        
	        if($image_status =~ /installed/i){
	        	tstlog -info, "Image Update Status : $image_status. \n Displaying expected image update status after post update check.";
	        }else{
	        	push @errors, "Image Update Status : $image_status. \n Not displaying expected image update status after post update check.";
	        	$error_flag = 1;
	        }
			$verified{'post_update_check'} = "true";			
			last;			
		}elsif(($update_completed_flag eq "true") && ($verified{'update_completed'} eq "false")){
			if(($update_completed_ems eq "true") ){
				tstlog -comment, "ems log was generated after completing update and before auto-reboot";
			}elsif($update_completed_ems ne "true"){
				push @errors, "EMS not generated after completing update before auto-reboot";
				$error_flag = 1;
			}
			$verified{'update_completed'} = "true";
		}
		$wait_time = 1;
		$count_down--;
	}	
	return ($error_flag, \@errors); 
    
}

sub Get_Firmware_Path($$){
	my $version = shift;
	my $path = shift;
	my $path_fw = $path;
	my $path_ontap = $path;
	my ($link1,$link2,$link3);
	
	if($path){
		if($path =~ /tar\.gz/){
			$path_ontap =~ s/tar\.gz/zip/;
		}else{
			$path_fw =~ s/zip/tar\.gz/;
		}
	}elsif((!defined($path)) && $version){
		$link1 = 'http://10.60.132.98/engineering/rlse/sp/';
		$link2 = '/Build.cfgnetapp/Output/SP_FW.zip';
		$link3 = '/Build.cfgnetapp/Output/SP_FW.tar.gz';
		
		$path_ontap = $link1 . $version . $link2;
		$path_fw = $link1 . $version . $link3;
		
	}
	
	return ($path_ontap, $path_fw);
	
}


1;
