# $Id: //depot/prod/test/main/lib/platval/virgo/iomem.pm $
#***********************************************************************************************************************************
## @summary   Routines to validate the Flash Cache 2 functionality 
##
## @description
## This library file contains the individual routines for ONTAP to perform various tasks such as intial setup for Flash Cashe 2 card, 
## start and stop I/O traffic, Retrieving the Falsh Cache 2 card details, Getting the available Falsh Cache 2 slots, Setting up a debug
## level, Reporting the statistics for FAL etc.
##
## @synopsis
## virgo2_setup();
## start_IO_traffic();
## stop_IO_traffic();
## get_available_virgo2_slot();
## get_fal_stats();
## set_fal_dbg_level();
##
## @status      public
##
## @author	makubhai@netapp.com
##
## @notes       Created by Platform Validation.
#***********************************************************************************************************************************
#!/usr/local/bin/perl

require "tharnit_lib.pl";


#*********************************************************************************************
## Function Name: virgo2_setup
##
## Parameters   : $APISet7mode	-> NACL TRANSIT OBJECT
##
## Return Type  : NONE
##
## Description  : Setup the Flash Cache 2 to type flash and check for configuration if present
##
#*********************************************************************************************

sub virgo2_setup {
        $Test->step("Verifying the System Type is set to 'flash'");
        
	my $response = $APISet7mode->iomem_system_current_get();
        $APISet7mode->iomem_system_current_set ('state' => 'flash') if((my $system_type = $response->get_processed_output()) =~ /DRAM/);
	sleep 2;

        $Test->step("Dumping the current bootargs");
        $APISet7mode->execute_raw_command(command=>"priv set test; bootargs dump");

        $Test->step("Verifying no Configuration Errors present");
        my $response = $APISet7mode->sysconfig('check-expansion-cards'=>'1');
        tstlog -fail, "There is a configuration error in this system" if((my $system_type = $response->get_processed_output()) !~ /no configuration errors/);
	sleep 1;
}


#*********************************************************************************************
## Function Name: start_IO_traffic
##
## Parameters   : $APISet7mode  -> NACL TRANSIT OBJECT
##
## Return Type  : NONE
##
## Description  : Start the I/O traffic such as Hammer, Filersio
##
#*********************************************************************************************

sub start_IO_traffic {
        $Test->step("Starting the I/O traffic using hammer, filersio etc..");

        my $cmd = 'priv set test; hammer -f -1 /vol/vol0/ham.1 10000000';
        $APISet7mode->execute_raw_command(command => $cmd);
        sleep 10;
        $cmd = 'priv set test; hammer -f -1 /vol/vol0/ham.2 1000000';
        $APISet7mode->execute_raw_command(command => $cmd);
        sleep 10;
        $cmd = 'priv set test; filersio asyncio_active 00 -r 100 4k 0 7g 840000 5 /vol/vol0/file.90g -create';
        $APISet7mode->execute_raw_command(command => $cmd);
        sleep 10;
}


#*********************************************************************************************
## Function Name: stop_IO_traffic
##
## Parameters   : $APISet7mode  -> NACL TRANSIT OBJECT
##
## Return Type  : NONE
##
## Description  : Stop the I/O traffic such as Hammer, Filersio
##
#*********************************************************************************************

sub stop_IO_traffic {
        my $cmd = "priv set test; hammer abort";
        $APISet7mode->execute_raw_command(command => $cmd);
        sleep 10;
        $cmd = "priv set test; filersio stop";
        $APISet7mode->execute_raw_command(command => $cmd);
        sleep 10;
}


#*********************************************************************************************
## Function Name: get_card_details
##
## Parameters   : $APISet7mode  -> NACL TRANSIT OBJECT
##		: $slot		-> Flash Cache 2 slot number
##
## Return Type  : %card_info 	-> Returns a reference to the Hash which contains key and value
##				   for card information such as no of blocks, no of planes etc.
##
## Description  : Reading the card details from SMF table for the respective flash cache 2 card
##
#*********************************************************************************************

sub get_card_details{
    my ($line, @keys, @fru, @values, $no_of_keys, $i, %card_info);
    $cmd = "priv set test";
    $response_obj = $APISet7mode->execute_raw_command(command => $cmd);
    $cmd = "priv set test; smf show iomem_fru_driver_2";
    $response_obj = $APISet7mode->execute_raw_command(command => $cmd);
    sleep 5;
    #$Test->nlog(Dumper(@parsed_output));
    foreach $line (split("\n",$response_obj)) {
        if($line =~ /slot/) {
            @keys = split(/\s+/,$line);
            next;
        }
        @fru = split(/\s+/,$line);
        if($fru[0] == $slot) {@values = @fru;last;};
    }
    my $count = scalar(@fru);
    $no_of_keys = scalar(@keys);
	tstlog -comment, "Keys: $no_of_keys Values: $count Contents in FRU: @fru";
    for ($i=0; $i< $no_of_keys; $i++) {
        $card_info{$keys[$i]} = $values[$i];
    }

    return \%card_info;
}


#*********************************************************************************************
## Function Name: get_available_virgo2_slot
##
## Parameters   : $APISet7mode  	-> NACL TRANSIT OBJECT
##
## Return Type  : @available_card_slot	-> Returns an array of all the available Flash Cache 2 cards in system 
##
## Description  : List the available Flash Cache 2 cards in system using sysconfig 
##
#*********************************************************************************************

sub get_available_virgo2_slot {
        my ($response, @parsed_output, @available_card_slot, $i, $device_name);
        $response = $APISet7mode->sysconfig();
        @parsed_output = @{$response->get_parsed_output()};
        #$Test->nlog(Dumper(@parsed_output));
        for (my $i=0; $i<scalar (@{$parsed_output[0]->{'slot_information'}}); $i++) {
                $device_name = $parsed_output[0]->{'slot_information'}[$i]->{'device'};
                if ($device_name =~ /Flash Cache 2/){
                        push(@available_card_slot,"$parsed_output[0]->{'slot_information'}[$i]->{'slot_number'}");
                        #tstlog(-comment, "Flash Cache 2:$parsed_output[0]->{'slot_information'}[$i]->{'slot_number'} Device Name: $device_name");
		}
        }
	return @available_card_slot;
}


#*********************************************************************************************
## Function Name: set_fal_dbg_level
##
## Parameters   : $APISet7mode  	-> NACL TRANSIT OBJECT
##		  $dbg_fal_level 	-> FAL debug level
## Return Type  : Returns value zero if no input parameter provided and debug level already
##		  set to default value
##
## Description  : Set up the FAL debug level to provided $dbg_fal_level else to default value
##
#*********************************************************************************************

sub set_fal_dbg_level {
	$Response = $APISet7mode->bootargs_get('arg'=>"dbg_fal_level");
	@parsed_output = @{$Response->get_parsed_output()};
        if(!$dbg_fal_level) {
        	return 0 if("$parsed_output[0]->{'dbg_fal_level'}" eq "5");
         	$APISet7mode->bootargs_set('arg'=>"dbg_fal_level",'value'=>"5");
               	$Test->step("Rebooting the filer to get effect default value of FAL Debug Level");
		$Obj->reboot();
                sleep 200;
     	}else{
		return 0 if("$parsed_output[0]->{'dbg_fal_level'}" eq "$dbg_fal_level");
		$APISet7mode->bootargs_set('arg'=>"dbg_fal_level",'value'=>"$dbg_fal_level");
            	$Test->step("Rebooting the filer to get effect of the specified FAL Debug Level");
		$Obj->reboot();
              	sleep 200;
   	}
}


#*********************************************************************************************
## Function Name: get_fal_stats
##
## Parameters   : $APISet7mode  -> NACL TRANSIT OBJECT
##
## Return Type  : Returns a reference to Hash of FAL statistics
##
## Description  : Retrieve the FAL statistics using stats show
##
#*********************************************************************************************

sub get_fal_stats {
	my ($stats, $line, $key, $val, %fal_stats);
	$stats = $APISet7mode->execute_raw_command(command => "priv set test; stats show fal");	
	foreach $line (split("\n",$stats)) {
		if($line =~ /fal/) {
			$line =~ s/fal://g;
			($key, $val) = split /:\s*/, $line, 2;
			$fal_stats{$key} = $val if defined $val;
			#tstlog -comment,"Value derived for the $key => $fal_stats{$key}";
		}
	}
	return \%fal_stats;
}

#*********************************************************************************************
## Function Name: start_nfs
##
## Parameters   : $APISet7mode  -> NACL TRANSIT OBJECT
##
## Return Type  : Returns a reference to Hash of FAL statistics
##
## Description  : Retrieve the FAL statistics using stats show
##
#*********************************************************************************************

sub start_nfs {
        $Test->step("Starting the NFS traffic using sio tool from client");

	my ($cmd, $host_ip, $out);	
	$host_ip = $Host->{'default_ip'};

	$Test->nlog("Creating a volume named 'myvol'");
        $cmd = "priv set test; vol create myvol 3";
   	$out = $APISet7mode->execute_raw_command(command=>"$cmd");
	sleep 50 if($out !~ /volume 'myvol' already exists/);

        $Test->nlog("Exporting the newly created volume before mount");
        $cmd = "priv set test; exportfs -io sec=sys,rw,anon=0,nosuid /vol/myvol";
        $APISet7mode->execute_raw_command(command=>"$cmd");
        sleep 3;
	$APISet7mode->execute_raw_command(command=>"priv set test; exportfs");
	sleep 2;

	$Test->nlog("Creating a File to start NFS traffic");
	$cmd = "priv set test; mkfile 100m /vol/myvol/testfile";
	$APISet7mode->execute_raw_command(command=>"$cmd");
        sleep 3;
	$APISet7mode->execute_raw_command(command=>"priv set test; options interface.blocked.mgmt_data_traffic off");

	$Test->nlog("Creating a directory to mount filer at client");
	$APISetClient->execute_raw_command(command=>"sudo mkdir -p /t/$FILER/vol/myvol");

	$Test->nlog("Mounting the filer at client");
	$APISetClient->execute_raw_command(command=>"sudo mount -o hard,intr,rsize=8192,wsize=8192  $host_ip:/vol/myvol /t/$FILER/vol/myvol");
	sleep 3;

	$Test->nlog("Changing the mode of the file");
	$APISetClient->execute_raw_command(command=>"cd /t/$FILER/vol/myvol");
	$APISetClient->execute_raw_command(command=>"sudo chmod 777 /t/$FILER/vol/myvol/testfile");
	sleep 3;

	$Test->nlog("Starting the NFS traffic on testfile of volume 'myvol'");
	$APISetClient->execute_raw_command(command=>"sudo /usr/software/test/bin/sio 50 100 4k 0 100m 50000 5 /t/$FILER/vol/myvol/testfile &");
	$APISetClient->execute_raw_command(command=>"\n");
        sleep 5;
	
	$cmd = "priv set test; nfsstat 1 10";
        $APISet7mode->execute_raw_command(command=>"$cmd");
        sleep 10;

}

#*********************************************************************************************
## Function Name: stop_nfs
##
## Parameters   : $APISet7mode  -> NACL TRANSIT OBJECT
##
## Return Type  : Returns a reference to Hash of FAL statistics
##
## Description  : Retrieve the FAL statistics using stats show
##
#*********************************************************************************************

sub stop_nfs {
        $Test->step("Stopping the NFS traffic");
	
        my ($cmd, $host_ip, $Response);
	$Test->nlog("Identifying and killing the NFS process to stop the traffic");
	$APISetClient->execute_raw_command(command=>"ps -C sio | sudo kill -9 `awk 'FNR==2{print $1}'`");
	sleep 10;
	
	$Test->nlog("Unmounting the Filer from client");
	$APISetClient->execute_raw_command(command=>"sudo umount -f -l /t/$FILER/vol/myvol");
	sleep 5;
	$APISetClient->execute_raw_command(command=>"sudo rm -rf /t/$FILER");
	sleep 2;

	$Test->nlog("Offlining the volume");
	$Response = $APISet7mode->vol_offline('volume'=>"myvol");
	sleep 5;
	
	$Test->nlog("Destroying the volume 'muvol'");
	$APISet7mode->vol_destroy('volume'=>"myvol",'force'=>"1");
	sleep 5;
}
