<?php
/**
 *  $Id: //depot/prod/test/Rsierranevada/lib/proxy.php#1 $
 *
 *  This script is used to test HTTP requests from a libcurl client on the same
 *  filer that receives the request.
 *
 *  1. Copy me to the filer under /mroot/etc/www/dev/.
 *  2. Enable the supdiag web service and create access.
 *     vserver services web modify supdiag -enabled true -vserver <node>
 *     vserver services web access create supdiag <role> -vserver <cserver>
 *  3. Invoke this script with an HTTP request. The response from the localhost
 *     request will be proxied back to the remote client.
 *
 *  Usage:
 *  To make this request using the filer as a client
 *  http://<LIF>/uri/path
 *  make this request from the remote client
 *  http://<node>/support/dev/mroot/proxy.php/<LIF>/uri/path
 *
 *  If <LIF> is "local", it will use "127.0.0.1:937".
 *  If <LIF> is "host", it will use the same host as the remote request.
 *
 *  Use HTTP basic authentication to access supdiag.
 *  Use the Proxy-Authorization header to have the local client use HTTP
 *  authentication.
 */

$path = $_SERVER["PATH_INFO"];
if (!$path) {
    http_send_status(400);
    exit("Path info required");
}

$parts = explode("/", ltrim($path, "/"), 2);
$path = $parts[1];
$host = $parts[0];
if ($host == "local") {
    $host = "127.0.0.1:937";
}
else if ($host == "host") {
    $host = $_SERVER["HTTP_HOST"];
}

$url = "http://$host/$path";
eval('$method = HTTP_METH_'.$_SERVER["REQUEST_METHOD"].';');
$req_body = http_get_request_body();

$headers = array();
$options = array();
foreach (apache_request_headers() as $header => $value) {
    if (preg_match('/^Test-Proxy-(.*)/i', $header, $matches)) {
        $headers[$matches[1]] = $value;
    }
    if (strcasecmp($header, "Proxy-Authorization") === 0 &&
        preg_match('/^Basic +(.*)/i', $value, $matches))
    {
        $options["httpauth"] = base64_decode($matches[1]);
    }
}

try {
    $req = new HttpRequest($url, $method, $options);
    $req->setQueryData($_SERVER["QUERY_STRING"]);
    $req->setHeaders($headers);
    if ($method === HTTP_METH_POST) {
        $req->setRawPostData($req_body);
    }
    else if ($method === HTTP_METH_PUT) {
        $req->setRawPutData($req_body);
    }

    $req->send();

    header_remove();
    http_send_status($req->getResponseCode());
    foreach ($req->getResponseHeader() as $header => $value) {
        header("$header: $value");
    }
    echo $req->getResponseBody();
}
catch (HttpException $ex) {
    http_send_status(502);
    echo $ex->getMessage();
}

?>
