# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/scsi_scate.pm#1 $
# 
# Copyright (c) 2005 Network Appliance, Inc.
# All rights reserved.
#
# 

# Define local parameters and their defaults
# Identify these param variables in a 'use vars' statement

# use statements (which may include common param definitions):
use strict;
use Burt;
require Exporter;

# use vars qw(
our(@ISA, @EXPORT, $scateServer, $ssConn, $mpFaultFile, $scateClient1, $client1Path, $client1Path2, $scateClient2, $client2Path, $scateClient3, $client3Path, $scsiVersion, $scenarioType);

@ISA = ('Exporter');
@EXPORT = qw( scateRunTest );

# Define local variables and their initial values
param('RUNLENGTH', '-default',"360");
param('scateServer','-default',"rtpqa-sun1000-1");
param('ssConn','-default',"#rsh");
param('scateClient1','-default',"");
param('client1Path','-default',"");
param('client1Path2','-default',"");
param('scateClient2','-default',"");
param('client2Path','-default',"");
param('scateClient3','-default',"");
param('client3Path','-default',"");
param('mpFaultFile','-default',"");
param('scsiVersion','-default',"scsi3");
param('scenarioType','-default',"non_reboot");

## @name ::scateRunTest
## @description 
##   execute the SCATE SCSI scenario as passed in.
## @synopsis
## 	scateRunTest()
## @arg $scateClient1, $client1Path, $scateClient2, $client2Path, $scateClient3, $client3Path, $scsiVersion, $scenarioType Required: Host with SUNWtscsi pkg installed. 
## @return none
## @example
##    scateRunTest()
##
##
sub scateRunTest($;$) {
  # scsiVersion and scenarioType are to be passed into the script.
  $scsiVersion = shift @_;
  $scenarioType = shift @_;

  my ($tcName,$tcDesc,$bootType,$tcType,$assertNum);
  # use the same testcase name as in the listtestcases() sub
  # to enable per testcase specific logging in nuffo.
  #
  $tcName = join('_',$scsiVersion,$scenarioType);
  $tcName =~ s/:/_/g;
  $tcName =~ s/-/_thru_/g;
  
  ($bootType,$tcType,$assertNum) = split(/:/,$scenarioType);
  $tcDesc = "Execute run_test for $scsiVersion $bootType scenario, testcase $tcType and assertion num $assertNum";
  # now call testcase_start to mark each time this sub is called.
  testcase_start($tcName, $tcDesc);
 
  # Connect to CLIENT, the SCATE SCSI server
  my $ch = host ($scateServer);
  my $cc = connect ($scateServer.$ssConn, -timeout, 10);
  
  my ($returnValue, $cmd2Xec, $cmdParms, $cmdVariant, @hostList, @pathList);
  
  if($client1Path) {
    push @pathList, $client1Path;
  }
  if($client1Path2) {
    push @pathList, $client1Path2;
  }
  if($client2Path) {
    push @pathList, $client2Path;
  }
  if($client3Path) {
    push @pathList, $client3Path;
  }
  
  if($scsiVersion eq "scsi3") {
      if(@pathList != 3) {
        logresult("CONFIG", "Must specify 3 paths for SCSI3!!");
        exit;
      }
  }
  elsif($scsiVersion eq "scsi2") {
    if(@pathList < 2) {
      logresult("CONFIG", "Must specify 2 paths for SCSI2!!");
      exit;
    }
  }
  elsif($scsiVersion eq "devid") {
    if(@pathList < 2) {
      logresult("CONFIG", "Must specify atleast 2 paths for DEVID!!");
      exit;
    }
  }
   
  # this is the main shell script that will lauch the SCATE
  # scsi tests. What follows is the parm building for this script..
  $cmd2Xec = "/opt/SUNWtscsi/tset_bin/run_test ";
  
  # build the hosts list from all valid hostnames passed in...
  if($scateClient1) {
    push @hostList,$scateClient1;
  }
  if($scateClient2) {
    push @hostList,$scateClient2;
  }
  if($scateClient3) {
    push @hostList,$scateClient3;
  }
  
  if(@hostList == 0) {
    logresult("CONFIG", "Valid number of hosts is 1 or 2 or 3 ");
    exit;
  }
  elsif(@hostList == 1) {
    if($scsiVersion eq "scsi3") {
      $cmdParms = "-n \"$hostList[0]:$client1Path,$hostList[0]:$client2Path,$hostList[0]:$client3Path\" ";
    }
    elsif($scsiVersion eq "scsi2") {
      $cmdParms = "-n \"$hostList[0]:$client1Path,$hostList[0]:$client2Path\" ";
    }
    elsif($scsiVersion eq "devid") {
      $cmdParms = "-n \"$hostList[0]:$client1Path,$client1Path2\" ";
    }
  }
  elsif(@hostList == 3) {
    if($scsiVersion eq "scsi3") {
      $cmdParms = "-n \"$scateClient1:$client1Path,$scateClient2:$client2Path,$scateClient3:$client3Path\" ";
    }
    else {
      logresult("CONFIG", "If 3 hosts are specified then scsi version should be SCSI3");
      exit;
    }
  }
  elsif(@hostList == 2) {
    if($scsiVersion eq "scsi2") {
      $cmdParms = "-n \"$scateClient1:$client1Path,$scateClient2:$client2Path\" ";
    }
    else {
      logresult("CONFIG", "If 2 hosts are specified then scsi version should be SCSI2");
      exit;
    }
  }
  
  # if testing with multi pathing faults then the filename has to
  # be specified.
  if($mpFaultFile ne "") {
    $cmd2Xec .= "-f ".$mpFaultFile." ";
  }
  
  $cmdVariant = $scsiVersion.":".$scenarioType;
  $cmd2Xec .= $cmdParms.$cmdVariant;
  # logcomment(" printing the cmd to execute: $cmd2Xec");

  # $cmd2Xec = "/opt/SUNWtscsi/tset_bin/run_test -f /opt/SUNWtscsi/targPortDown.scate -n \"$scateClient1:$client1Path,$scateClient2:$client2Path,$scateClient3:$client3Path\" $scsiVersion:$scenarioType";
  
  (undef, undef, $returnValue) = say($cc, $cmd2Xec, '-timeout', "3600");
     
  ## @step Verify:
       my ($junk, $testLogLoc);
       my @logFileNames = split(/\r\n/,$returnValue); # can use filer_cli_split
       foreach(@logFileNames) {
         if($_ =~ /testlog/) {
  	   ($junk, $testLogLoc) = split(/\//,$_,2);
  	   say($cc, "cat \/$testLogLoc", '-timeout', "360");
         }
         ##	Check the returnValue to see if SCATE reported
         ##	a failure..... 
     	 if($_ =~ /\d+: - PASS/) {
       	   logresult("PASS", "SCATE SCSI $scenarioType PASSED");
     	 }
    	 elsif($_ =~ /\d+: - FAIL/) {
      	   logresult("FAIL", abort=>0, "SCATE SCSI $scenarioType FAILED");
     	 }
    	 elsif($_ =~ /error/i) {
      	   logresult("INSPECT", abort=>0, "SCATE SCSI $scenarioType Returned an error.....investigate");
     	 }
	 elsif($_ =~ /\d+: - unresolved|Illegal|Invalid/i) {
       	   logresult("INSPECT", abort=>0, "SCATE SCSI $scenarioType Returned Invalid result", "Investigate");
         }
       }
  # Disconnect from client
  disconnect($cc);
}

1;
