# Hey emacs, this is basically -*- perl -*-
# $Id: $

## @summary Setup Routines for filers
## @author dl-central-libs
## @status public
## @pod here


=head1 NAME

setup - Filer setup routines.

=head1 DESCRIPTION

This lib is deprecated w.r.t cifs routines. Use CIFS::Setup instead.

=head1 ROUTINES

=cut

package setup;

use strict;


use Tharn;
use Admin::License qw(license is_licensed);
use Utils qw(make_conn make_host);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use CIFS::Setup qw(cifs_setup);
use boot qw(reboot boot wait_for_rsh);

BEGIN
{
    my @cifs_subs = qw(
        is_cifs_setup
        cifs_setup
        );
    my @cf_subs = qw(
        is_cf_setup
        is_filer_cf_capable
        is_a_filer_cf_capable
        );
    my @dafs_subs = qw(
        is_filer_dafs_capable
        is_dafs_setup
        dafs
        create_dafs_raw_dev
        viping
        );
    my @san_subs = qw(
        is_filer_fcp_capable
        is_fcp_setup
        fcp_setup
        is_iscsi_setup
        iscsi_setup
        is_veritas_installed
        );
    my @ftp_subs = qw(
        ftp_setup
        );
    my @http_subs = qw(
        http
        );
    my @nfs_subs = qw(
        is_nfs_setup
        nfs_setup
        mount_vol
        );
    my @misc_subs = qw(
        filelist
        ping_filer
    );

	Module::Common::setup_export_tags(
		cifs  => [ @cifs_subs ],
		cf    => [ @cf_subs ],
		dafs  => [ @dafs_subs ],
		san   => [ @san_subs ],
		ftp   => [ @ftp_subs ],
		http  => [ @http_subs ],
		nfs   => [ @nfs_subs ],
		misc  => [ @misc_subs ]
	);
}


=head2 is_cifs_setup

This routine is deprecated. Use CIFS::Setup::is_cifs_setup instead.
Refer CIFS::Setup::is_cifs_setup for details

=cut

sub is_cifs_setup ( $ ) {

    my ($filer) = @_;
    logresult('warn', "This routine setup::is_cifs_setup is deprecated. Use CIFS::Setup::is_cifs_setup");
    return(CIFS::Setup::is_cifs_setup(host=>$filer));
}


=head2 cifs_setup(%opts)

This routine is deprecated. Use CIFS::Setup::cifs_setup instead.
Refer CIFS::Setup::cifs_setup for details

=cut

sub cifs_setup {

    my (%args) = @_;
    logresult('warn', "This routine setup::cifs_setup is deprecated. Use CIFS::Setup::cifs_setup instead");

    # Convert some args around to make it backwards compatible

    # setup::cifs_setup used a named param 'filer'
    # CIFS::Setup::cifs_setup now uses a named param 'host'
    # Copy 'filer' to 'host'
    $args{host} = $args{filer};
    delete $args{filer};

    # fill up auth-type based on old style args, ldap, nis, domain and workgroup
    # 'auth-type' will default to 'workgroup' in CIFS::Setup::cifs_setup
    # if none of ldap, nis, domain and workgroup are specified
    if (!exists($args{'auth-type'})) {
        # croak if more than one of ldap,nis,workgroup,domain is specified
        if ($args{ldap} || $args{nis}) {
            $args{'auth-type'} = 'passwd';
            if ($args{ldap}) {
                $args{ldap_setup} = $args{ldap};
                delete $args{ldap};
            }
            if ($args{nis}) {
                logresult('warn',"NIS configure is not part of cifs_setup. Use Network::Nis::nis_configure");
                delete $args{nis};
            }
        }
        if ($args{domain}) {
            if ($args{'auth-type'} eq 'passwd') {
                croak('Both domain and ldap or nis are given. ".
                        "Pls specify only one method of user authentication')
            } else {
                $args{'auth-type'} = 'ad';
            }
        }
        if ($args{workgroup}) {
            if ($args{'auth-type'} eq 'passwd') {
                croak('Both workgroup and ldap or nis are given. ".
                        "Pls specify only one method of user authentication')
            } elsif ($args{'auth-type'} eq 'domain') {
                croak('Both workgroup and domain are given. ".
                        "Pls specify only one method of user authentication')
            } else {
                $args{'auth-type'} = 'workgroup';
            }
        }
    }

    # Convert 'protocol' to 'security-style'
    # also change the content i.e. 'multi' to 'multiprotocol'
    if (!exists($args{'security-style'}) && exists($args{protocol})) {
        $args{protocol} = 'multiprotocol' if ($args{protocol} eq 'multi');
        $args{'security-style'} = $args{protocol};
        delete $args{protocol};
    }

    # Convert 'ldap' to 'ldap_setup'
    if (!exists($args{ldap_setup}) && exists($args{ldap})) {
        $args{ldap_setup} = $args{ldap};
        delete $args{ldap};
    }

    # Convert 'netbios_name' to 'server-name'
    if (!exists($args{'server-name'}) && exists($args{netbios_name})) {
        $args{'server-name'} = $args{netbios_name};
        delete $args{netbios_name};
    }

    return eval {
        CIFS::Setup::cifs_setup(%args);
    };
}


=head2 is_cf_setup($filer,$partner)

Verify if filer is setup for clustering

=over

=item Arguments

=over 

=item filer

(Required) : hostspec or host object of filer.


=item partner

(Required) : hostspec or host object of partner.

=back

=back

=for html <hr />

=cut

sub is_cf_setup ( $$ ) {

    my ($filer, $partner) = @_;

    return((is_a_cf_setup($filer) && is_a_cf_setup($partner)) ? 1 : 0);
}

sub is_a_cf_setup ( $ ) {

    my ($filer) = @_;

    my $conn = make_conn($filer);
    my $out = sayEOF($conn, "cf status");
    return(($out =~ /(must be licensed|disabled)/) ? 0 : 1);
}


=head2 is_filer_cf_capable($filer,$partner)

Verify if filer/partner has cluster capability


=over

=item Arguments

=over

=item filer

(Required) : hostspec or host object of filer.


=item partner

(Required) : hostspec or host object of partner.

=back

=back

=for html <hr />

=cut

sub is_filer_cf_capable ( $$ ) {

    my ($filer,$partner) = @_;

    return((is_a_filer_cf_capable($filer) && is_a_filer_cf_capable($partner)) ? 1 : 0);
}

sub is_a_filer_cf_capable ( $ ) {

    my ($filer) = @_;

    my $conn = make_conn($filer);
    my $out = sayEOF($conn, "sysconfig -v");
    return(($out =~ /(Cluster Interconnect|Interconnect HBA|VI Cluster Adapter|Interconnect adapter)/) ? 1 : 0);
}



=head2 is_filer_dafs_capable($filer)

Verify if filer has dafs VI/IP interface

=over

=item Arguments

=over

=item filer

(Required) : hostspec or host object of filer.

=back

=back

=for html <hr />

=cut

sub is_filer_dafs_capable ( $ ) {

    my ($filer) = @_;

    my $conn = make_conn($filer);

    my $out = sayEOF($conn, "ifconfig -a");
    return(($out !~ /VI\/IP/) ? 0 : 1);
}



=head2 is_dafs_setup($filer)

check if DAFS is already setup on filer

=over

=item Arguments

=over

=item $filer

(Required) : hostspec or host object of filer.

=back

=item Returns 

1 if dafs is setup

0 otherwise

=back

=for html <hr />

=cut

sub is_dafs_setup($) {

    my ($filer) = @_;

    my $conn = make_conn($filer);
    my $out = sayEOF($conn, "dafs status -n");

    return(($out =~ /DAFS service is started.*State:\s+ATTACHED/s) ? 1 : 0);
}



=head2 dafs($filer,$dafs_filer_vol,$filer_vi_dev,%opts)

setup dafs on filer

=over

=item Arguments

=over

=item filer

(Required) : hostspec of filer

=item dafs_filer_vol

(Required) : volume to add to dafs

=item filer_vi_dev

(Required) : VI/IP interface on filer (e.g. vf3, e5)

=item partner_vi_dev

(Optonal) : VI/IP interface on partner (e.g. vf3, e5)

=back

=item Notes

TBD Modify this routine to work for multiple vol

TBD Modify this routine to exit gracefully

=back

=for html <hr />

=cut

sub dafs($$$;$) {

    my ($filer, $dafs_filer_vol, $filer_vi_dev, $partner_vi_dev) = @_;

    my $h = host($filer);
    my $c = connect("#telnet");

    my $disable_timeout=7200;

    my (undef, undef, $output) = say($c, "ifconfig $filer_vi_dev");
    logresult('CONFIG', "Interface $filer_vi_dev not present on $filer") if ($output =~ /no such interface/);
    logresult('CONFIG', "$filer_vi_dev on $filer not type VI Interface") if ($output !~ /VI\/IP/);
    logresult('CONFIG', "Link not up for $filer_vi_dev on $filer") if ($output !~ /auto-1000sx-fd-up/);

    # stop and license DAFS if necessary
    say($c,"dafs stop"); # what return value to check ???? 
    license($c, "dafs") or logresult('FATAL', "Could not license dafs : $@");

    use constant RE_NTWKQUERY => "Enter a network device for DAFS, or 'q' when done\:";
    use constant RE_PARTNERNTWKQUERY => "Enter a partner network device for DAFS, or 'q' when done\:";
    use constant RE_EXPORTQUERY => "Do you want to export all filer volumes through DAFS\:";
    use constant RE_FILERVOL => "Enter a filer volume/path to export, or 'q' when done\:";
    use constant RE_CLIENTAUTH => "Do you want privileged clients to require authentication\:";
    use constant RE_IDLETIME => "Enter idle timeout for sessions, or 0 to disable timeout\:";

    say($c,"dafs setup",'-blind');

    # wait for following "dafs setup" prompts, and act on them, until the
    # prompt coming back is one of the builtin say prompts (i.e. the
    # filer's normal prompt, showing that dafs setup is done)

    my $first=1;
    while (1) {
        my $prompt;
        (undef, undef, undef, $prompt) = 
            say($c,"",-prompt,[RE_NTWKQUERY,
                                    RE_PARTNERNTWKQUERY,RE_EXPORTQUERY,
                                    RE_FILERVOL,RE_CLIENTAUTH,RE_IDLETIME]);

        if ($prompt =~ RE_NTWKQUERY) {
            if ($first) {
                say($c,$filer_vi_dev,'-blind');
                $first--;
            } else {
                say($c,"q",'-blind');
                $first++;
            }
        } elsif ($prompt =~ RE_PARTNERNTWKQUERY) {
            if ($first) {
                if (defined($partner_vi_dev)) { 
                    say($c,$partner_vi_dev,'-blind');
                    $first--;
                } else {
                    say($c,"q",'-blind');
                }
            } else {
                say($c,"q",'-blind');
                $first++;
            }
        } elsif ($prompt =~ RE_EXPORTQUERY) {
            say($c,"n",'-blind');
        } elsif ($prompt =~ RE_FILERVOL) {
            if ($first) {
                say($c,"$dafs_filer_vol",'-blind');
                $first--;
            } else {
                say($c,"q",'-blind');
                $first++;
            }
        } elsif ($prompt =~ RE_CLIENTAUTH) {
            say($c,"n",'-blind');
        } elsif ($prompt =~ RE_IDLETIME) {
            say($c,$disable_timeout,'-blind');
        } else {
            # builtin prompt, i.e. filer's normal prompt
            last;
        }
    }
    (undef, undef, $output) = say($c, "dafs status -n");
    disconnect("#telnet");
    if ($output =~ /$filer_vi_dev.*State: ATTACHED/ ) {
        if (defined($partner_vi_dev)) {
            if ($output !~ /$partner_vi_dev.*State: INACTIVE PARTNER/) {
                logresult("INSPECT", "DAFS SETUP FAILED for PARTNER");
            } 
        }
        logresult('PASS', "DAFS SETUP COMPLETE");
    } else {
        logresult('FATAL', "DAFS SETUP FAILED");
    }

}


=head2 is_filer_fcp_capable($filer)

check if filer is fcp capable

=over

=item Arguments

=over

=item filer

(Required) : Hostspec of filer

=back

=back

=for html <hr />

=cut

sub is_filer_fcp_capable($) {

    my ($filer) = @_;

    my $conn = make_conn($filer);

    # check sysconfig for Fibre Channel Target Host Adapter
    my $out = sayEOF($conn, "sysconfig");
    return(($out !~ /Fibre Channel Target Host Adapter/) ? 0 : 1);
}



=head2 is_fcp_setup($filer)

check if FCP is already setup on filer

=over

=item Arguments

=over

=item filer

(Required) : Hostspec of filer

=back

=item Returns

1 if FCP is setup

0 otherwise

=back

=for html <hr />

=cut

sub is_fcp_setup($) {

    my ($filer) = @_;

    my $conn = make_conn($filer);

    # check if fcp is running
    my $out = sayEOF($conn, "fcp status");
    return(0) if ($out =~ /(service.*not running|not licensed)/);

    # check if nodename is not all zeros
    # dont know of a better way to verify if fcp is REALLY running
    $out = sayEOF($conn, "fcp nodename");
    return(($out =~ /^Fibre Channel nodename: .*(0000000000000000)/) ? 0 : 1);
}


=head2 is_iscsi_setup($filer)

check if iSCSI is already setup on filer

=over

=item Arguments

=over

=item filer

(Required) : Hostspec of filer

=back

=item Returns

1 if iSCSI is setup

0 otherwise

=back

=for html <hr />

=cut

sub is_iscsi_setup($) {

    my ($filer) = @_;

    my $conn = make_conn($filer);

    # check if iSCSI is running
    my $out = sayEOF($conn, "iscsi status");
    return(0) if ($out =~ /(service.*not running|not licensed)/);

    # check if nodename is not all zeros
    # dont know of a better way to verify if iSCSI is REALLY running
    $out = sayEOF($conn, "iscsi nodename");
    return(($out =~ /^iSCSI target nodename: iqn.1992-08.com.netapp.\S+/) ? 1 : 0);

}


=head2 iscsi_setup($filer)

setup iSCSI on filer

=over

=item Arguments

=over

=item filer

(Required) : Hostspec of filer

=back

=back

=for html <hr />

=cut

sub iscsi_setup ( $ ) {

    my ($filer) = @_;

    my $conn = make_conn($filer);

    # license iSCSI
    license($conn, 'iscsi') or die $@;

    # check if iSCSI is started
    # if not started then start it
    my $out = sayEOF($conn, "iscsi status");
    sayEOF($conn, "iscsi start")
        if ($out =~ /service.*not running/);
    
    # check if iscsi is started now
    return(is_iscsi_setup($filer));
}

=head2 fcp_setup($filer)

setup fcp on filer

=over

=item Arguments

=over

=item filer

(Required) : Hostspec of filer

=back

=item obj

(Required) : Hostobject of filer 

=back

=back

=for html <hr />

=cut

sub fcp_setup ( $ ) {

    my ($filer) = @_;

    my $host = make_host($filer);
    my $conn = make_conn($filer);

    # license fcp
    license($conn, "fcp") or die $@;

    my %hash = %$host;

    #
    #After a fresh install target port configuration is erased,hence
    #it is required to reconfigure the targets
    #
    my @targets= grep(/^TARGET\d_NAME/, keys(%hash));
    if (@targets > 0) {
       foreach my $target(@targets) {
          sayEOF($conn, "priv set diag;fcadmin offline $hash{$target}");
          sayEOF($conn, "priv set diag;fcadmin config -t target $hash{$target}");
       }
       reboot($host);
    } else {
      logcomment("Target names not defined on the nate host file");
    }


    #
    #The partner filer communicates the WWNN nodename to the filer.
    #It waits for the partner host to be up,because the partner host
    #could be in-middle of a reboot or install.
    #
    if ($hash{part_host}) {
        my $partner_host = host($hash{part_host});
        logcomment("Waiting on partner to respond with the maximum timeout of 100 minutes ..");
        wait_for_rsh($partner_host,timeout=>6000,delta=>600);
        my $partner_conn = make_conn($partner_host);
        sayEOF($partner_conn, "priv set diag; vtic send_names");

        #
        #rsh to the current filer fails intermittently,hence adding
        #some timeout before issuing any command
        #
        wait_for_rsh($host,timeout=>600,delta=>60);
        sayEOF($conn, "priv set diag; vtic send_names");

    }
    
    # check if fcp is started
    # if not started then start it
    my $out = sayEOF($conn, "fcp status");
    sayEOF($conn, "fcp start")
        if ($out =~ /service.*not running/);
    # check if fcp is started now
    return(is_fcp_setup($filer));
}



=head2 ftp_setup($filer,$ftpuser,$ftppass,$mount_point)

Setup filer for FTP

Will setup FTP on the filer. Will not work if cifs is not setup on filer
maybe change this to work for Anonymous FTP. Also instead of appending
the user/passwd details to /etc/passwd check if the user exists and if so 
do not append

=over

=item Arguments

=over

=item filer

(Required) : Hostspec of filer

=item ftpuser

(Required) : FTP user name

=item ftppass

(Required) : FTP password

=item mountpoint

(Required) : filer mountpoint

=item home_dir

(Optional) : ftp home directory for the user. Default : "/"

=back


=item Notes

will not work from remote client

=back

=for html <hr />

=cut

sub ftp_setup( $$$$;$ ) {

    use cat qw(do_shell append_unique_line);

    logcomment("Setting Up FTP");
    my ($filer,$ftpuser,$ftppass,$mount_point,$home_dir) = @_;

    $home_dir = "/" if (!defined($home_dir));

    # Create a connection to use
    my $c = make_conn($filer);

    # Turn the FTP service on if necessary
    sayEOF($c,"options ftpd.enable on") if grep_data($c,'options ftpd.enable','off');

    # Create a Password a uid and a gid to be used for FTP.
    my $uid = $>;
    my $gid = $) + 0;
    my $after = sayEOF($c,"cifs passwd $ftppass");
    $after =~ m/password\s+is\s+(\S+)/;

    # Append the password to the "/etc/password" file
    my $filename = "$mount_point/etc/passwd";
    cat::do_shell("/usr/local/bin/r touch $filename ; /usr/local/bin/r chmod 0666 $filename");
    cat::append_unique_line($filename,"${ftpuser}:${1}:${uid}:${gid}::$home_dir:/bin/false");

    logcomment("FTP setup complete");
}


=head2 http($filer,$http_vol,$mount_point)

Setup filer for http.

=over

=item Arguments

=over

=item filer

(Required) : Hostspec of filer

=item http_vol

(Required) : http volume

=item mountpoint

(Required) : filer mountpoint

=back

=back

=for html <hr />

=cut

sub http( $$$ ) {

    use POSIX qw(strftime);

    return eval {
    logcomment("Setting up HTTP");
    my ($filer,$http_vol,$mount_point) = @_;

    # Create A connection record
    my $c = make_conn($filer);

    # License the HTTP service if necessary.
    license($c, "http") or die $@;

    # Turn the HTTP service on if necessary.
    sayEOF($c,"options httpd.enable on") if grep_data($c,'options httpd.enable','off');

    # Create a http Directory on the Volume.
    # TBD move this out of setup
    my $dir = "httpstress_" . POSIX::strftime("%y:%m:%d-%H:%M:%S",localtime(time));
    do_shell("/usr/local/bin/r mkdir $mount_point/$dir ; /usr/local/bin/r chmod 0777 $mount_point/$dir");

    # Create some Files to be read via HTTP protocol
    # TBD move this out of setup
    create_files($mount_point,$dir);

    # Set the "httpd.rootdir" options to this.
    my $out = sayEOF($c,"options httpd.rootdir /vol/$http_vol/$dir");
    logresult('warn', "http setup : $out") if ($out); 
    # will get a warning if the rootdir is not accessible
    };
}



=head2 is_nfs_setup($spec)

Check if NFS is running on filer


=over

=item Arguments

=over

=item spec

(Required): Hostspec of filer

=back

=item Returns

0 if NFS is NOT setup

1 if NFS is setup

=back

=for html <hr />

=cut

sub is_nfs_setup($) {

    my ($spec) = @_;

    eval {
    my $conn = make_conn($spec);
    my $out = sayEOF($conn, "nfs status");
    return(($out =~ /NOT running/i) ? 0: 1);
    };
}


=head2 nfs_setup($filer,%opts)

Will setup nfs on the filer


=over

=item Arguments

=over

=item filer

(Required) : Hostspec of filer


=item path

(Optional) : path to export


=item export

(Optional) : flag to denote if the entry is to added to /etc/exports

=back


=item Notes 

TBD - Use ExportFS.pm

=back

=for html <hr />

=cut

sub nfs_setup( $;% ) {

    use mount qw(mount umount);
    use cat qw(append_unique_line);
    use Volume qw(vol_root);

    my ($filer,%args) = @_;

    eval {
    my $path = defined($args{path}) ? $args{path} : '';
    my $export = defined($args{export}) ? $args{export} : 1;

    my $conn = make_conn($filer);

    # license nfs
    license($conn, "nfs") or die $@;
    # turn nfs on
    my $out = sayEOF($conn, "nfs on");

    # check if running
    die ("Unable to setup nfs")
        if (!is_nfs_setup($filer));

    if ($path) {
        # export the path / qtree
        $out = sayEOF($conn, "priv set -q advanced; exportfs -i -o anon=0 $path");
        die("Volume $path not accessible on $filer")
            if ($out =~ /no such directory/);

        # check if exportfs really worked
        $out = sayEOF($conn,"priv set -q advanced; exportfs");
        die("Volume $path does not exist or is offline on $filer")
            if ($out !~ /$path/);

        if ($export) {
            # get root volume of filer
            my $root_vol = vol_root($conn);
            die("Unable to get root volume on $filer")
                if (!$root_vol);

            # export root vol if not exported
            $out = sayEOF($conn,"priv set -q advanced; exportfs");
            if ($out !~ /\/vol\/$root_vol/) {
                $out = sayEOF($conn,
                    "priv set -q advanced; exportfs -i -o anon=0 /vol/$root_vol");
                # shld work no need to check
            }

            # mount root vol of filer
            my $mount_point = mount($filer, "/vol/$root_vol");
            die("Unable to mount $filer:/vol/$path")
                if (!$mount_point);

            my $ch = host("localhost");
            my $hosttype = $ch->{hosttype};
            my $cc = connect("#rsh");
            my $sudo = '/usr/local/bin/sudo';
            my $dir;
            if ($hosttype =~ /^unix-(hpux|solaris)/) {
                $dir = '/usr/bin';
            } elsif ($hosttype =~ /^unix-linux/) {
                $dir = '/bin';
            } else {
                die("$hosttype of localhost not supported");
            }
            # append unique entry into /etc/exports
            my $etc_exports = "$mount_point/etc/exports";
            say($cc, "$sudo $dir/touch $etc_exports && echo _''OK_","_OK_",
                '-fatal', "Cannot touch file $etc_exports");
            say($cc, "$sudo $dir/chmod 0666 $etc_exports && echo _''OK_","_OK_",
                '-fatal', "Cannot chmod file $etc_exports");
            append_unique_line($etc_exports,"$path -anon=0");

            # umount root volume
            umount($mount_point)
                or logresult('WARN', "Could not umount $mount_point");
        }
    }
    };
}



=head2 mount_vol($filer,$filer_ip,$volume,%opts)

This routine will mount the volume. Will license nfs, enable nfs, export the 
volume if not done. update /etc/exports if $export is specified.
Will perform the mount on local client itself.


=over

=item Arguments

=over

=item filer

(Required) : Hostspec of filer


=item filer_ip

(Required) : ip address of filer


=item volume

(Required) : filer volume


=item export 

(Optional) : flag denoting if entry to be added to /etc/exports


=item client

(Optional) : not implemented

=back


=item Notes

TBD add remote client stuff

TBD default to root volume if not specified

TBD default to default ip if filer_ip not specified


=item Bugs

cannot mount_vol of non-root volume with export=1

=back

=for html <hr />

=cut

sub mount_vol($$$%) {

    use cat qw(do_shell append_unique_line);
    use mount qw(mount);
    use ExportFS qw(export_path);

    my ($filer,$filer_ip,$volume,%args) = @_;
    my $sudo = '/usr/local/bin/sudo';

    my $export = defined($args{export}) ? $args{export} : 1;
    #my $client = defined($args{client}) ? $args{client} : 'localhost';

    # Create A connection Record
    my $c = make_conn($filer);
    
    # License nfs if not already
    license($c, "nfs");
    # TBD check the return value

    # Turn service on if not already
    sayEOF($c,"priv set -q advanced; nfs on") 
        if (grep_data($c,'nfs','NOT running'));

    # Export the Volume if not already. (The volume needs to be present)
    # default export options anon=0
    export_path($c, "/vol/$volume", OPTIONS=>"anon=0", WRITE_ETC=>1) or 
        logresult('fatal', "Unable to export $volume :$@");

    # Now try to mount the volume.
    for (my $i=0;$i<=5;$i++) {
        my $mount_point = mount($filer_ip,
                                "/vol/$volume");
                                #client=>$client);
        return $mount_point if ($mount_point);

        logcomment("$@. Retrying in 10 secs");
        sleep (10);
    }
    logresult('fatal',"Exiting. Could not mount $filer_ip $volume");
}

sub grep_data($$$) {

    my ($conn,$command,$data) = @_;    
    my (undef,undef,$after) = say ($conn,"priv set -q advanced;$command");
    my @data = split(/\n/,$after);
    my $regex = string2regex($data);
    return 1 if grep {/$regex/} @data;
}

sub create_files($$) {

    use POSIX;
    my ($mount_point,$subdir) = @_;
    my @files = filelist();
    foreach my $file(@files){
    logcomment("creating file $file");
        my ($fname,$size) = split ('_',$file);
        my $byte = chr(80);
        my $fullname = "$mount_point/$subdir/$file";
        my $fh = new FileHandle "$fullname",O_WRONLY|O_CREAT;
        $fh ? syswrite($fh,$byte x $size,$size) : last;
    }
}

sub filelist {

  my @files;
  foreach my $size (0, 1, 2, 5, 10, 15, 20, 25, 50, 75, 100, 200, 500, 1000, 4096) {
      my $name = "file_$size";
      push @files,$name;
    }
  return @files;
}


=head2 viping($client,$filer,%opts)

Check CLIENT / FILER connectivity via the VI/IP interface

=over

=item Arguments

=over

=item client

(Required) : Hostspec of client

=item filer

(Required) : Hostspec of filer

=item client_vi_dev

(Optional) : Client VI/IP device

=item client_utils_dir

(Optional) : Location of utils. Default: /opt/NTAPutils/bin

=back


=item Returns

1 on success

0 on failure

=back

=for html <hr />

=cut

sub viping ( $$;% ) {

    my ($client,$filer,%args) = @_;

    my $client_utils_dir = defined($args{client_utils_dir}) ? $args{client_utils_dir} : "/opt/NTAPutils/bin";

    my $cc = make_conn($client);
    my $output = sayEOF($cc, "$client_utils_dir/viping $filer");
    if ($output =~ /Name not found/) { 
        logcomment("Unable to viping $filer from $client");
        return 0;
    }

    foreach my $line (split(/\n/,$output)) {
        my ($vinc, $local_ip, $remote_ip, $remote_host, 
                $dummy, $connected, $linkup, $error) = split(/\s+/,$line);
        next if ((defined($args{client_vi_dev})) && ($vinc !~ /$args{client_vi_dev}/));
        if (($remote_host =~ /$filer/) && ($connected =~ /YES/) && ($linkup =~ /YES/)) {
            return 1;
        }
    }
    return 0;
} 



=head2 ping_filer($filer_ip,%opts)

Ping a filer

=over

=item Arguments

=over

=item filer_ip

(Required) : IP Address of filer to ping

=item client

(Optional) : Hostspec of client to ping from. Default: localhost

=item timeout

(Optional) : time in sec to wait till failure is determined. Default: 20

=item ping_path

(Optional) : path where ping binary lives.

=back

=back

=for html <hr />

=cut

sub ping_filer($%) {

    my ($filer,%args) = @_;

    my $client = defined($args{client}) ? $args{client} : 'localhost';

    eval {
        my $subtest_obj = subtest( sub{sub_ping($filer,%args)},
                        '-nobg',
                        '-ifscript', '-ok',
                        '-iffatal', '-ok',
                        '-runid', "ping-$filer-$client");
        my $worst = Rslt::worst($subtest_obj->ownRslts());
        return((($worst) && ($worst->type() =~ /SCRIPT|FATAL/)) ? 0 : 1);
    }
}

sub sub_ping($%) {

    my ($filer,%args) = @_;

    eval {
        my $client = defined($args{client}) ? $args{client} : 'localhost';
        my $timeout = defined($args{timeout}) ? $args{timeout} : 20;

        my $ch = host($client);
        my $hosttype = $ch->{hosttype};
        my ($default_path, $ping_arg);
        if ($hosttype =~ /linux/) {
            $default_path = '/bin';
            $ping_arg = "-c 5";
        } elsif ($hosttype =~ /solaris/) {
            $default_path = '/usr/sbin';
        } elsif ($hosttype =~ /windows/) {
            $default_path = 'c:\windows\system32';
           # $default_path = 'c:\winnt\system32'; WIN NT
        } else {
            die("Unknown hosttype $hosttype");
        }
        my $ping_path = defined($args{ping_path}) ? $args{ping_path} : $default_path;

        my $cc = make_conn($client);
        if ($hosttype =~ /^windows/) {
            #my (undef, undef, $out) = say($cc, "cmd.exe /c $ping_path\\ping $filer", 
            my (undef, undef, $out) = say($cc, "cmd.exe /c ping $filer", 
                                        '-timeout', $timeout);
            addresult(($out =~ /(could not find host|Unknown host|Bad IP address|Request timed out)/i) ? 
                                    'FATAL' : 'PASS');
        } else {
            my (undef, undef, $out) = say($cc, "$ping_path/ping $ping_arg $filer", 
                                        '-timeout', $timeout);
            die($out) if ($out =~ /command not found/i);
            addresult(($out =~ /(unknown host|no answer)/i) ? 'FATAL' : 'PASS');
        }
    }
}


=head2 create_dafs_raw_dev($filer,$client,%opts)

create a file via dafs


=over

=item Arguments

=over

=item filer 

(Required) : Hostspec of filer

=item client

(Required) : Hostspec of client

=item file_size

(Optional) : size of file to create. Default: 50000

=item file_name

(Optional) : name of file to create.  Default: dafsfile

=item filer_vol

(Optional) : name of volume on filer.  Default: root volume

=item client_ddafs_dir

(Optional) : path to ddafs dir on client.  Default: /opt/NTAPddafs/bin

=back

=back

=for html <hr />

=cut

sub create_dafs_raw_dev ( $$;% ) {

    use Volume qw(vol_root vol_exists vol_status);

    my ($filer, $client, %args) = @_;
    my $sudo = '/usr/local/bin/sudo';

    my $file_size = defined($args{file_size}) ? $args{file_size} : 50000;
    my $file_name = defined($args{file_name}) ? $args{file_name} : 'dafsfile';
    my $client_ddafs_dir = defined($args{client_ddafs_dir}) ? $args{client_ddafs_dir} : '/opt/NTAPddafs/bin';
    #-----------------------------------------------------
    # If filer_vol not defined use root volume
    # if defined then check if it exists and is online
    #-----------------------------------------------------
    my $fc = make_conn($filer);

    my $filer_vol;
    if (!defined($args{filer_vol})) {
        $filer_vol = vol_root($fc);
    } else {
        if (vol_exists($fc, $args{filer_vol})) {
            my $obj = vol_status($fc, $args{filer_vol})
                            or die($@);
            my $state = $obj->state();
            if (defined($state) && $state eq 'online') {
                $filer_vol = $args{filer_vol};
            } else {
                die("$args{filer_vol} is not online\n");
            }
        } else {
            die("$args{filer_vol} does not exist\n");
        }
    }


    #--------------------------------------------
    # DAFS export volume if not exported
    #--------------------------------------------
    my $path = "/vol/$filer_vol";
    my ($dafs_export_name,$enabled,$mode);

    # first check if its already exported
    my (undef, undef, $output) = say($fc,"dafs export list");
    if (($dafs_export_name, $enabled, $mode) = ($output =~ 
        /Name:\s+([\w\/]+).*Export point:\s+$path[^\/].*Enabled:\s+(true|false).*Read\/Write Mode:\s+(read_only|read_write)/s)) {
        if ($dafs_export_name ne $path) {
            logcomment("$path is already dafs exported as $dafs_export_name and not as $path");
        } else {
            logcomment("$path is already dafs exported as $path");
        }

        # check if export is enabled and ready for r/w
        logresult('CONFIG',"$path is DAFS exported in read-only mode.")
            if ($mode =~ /read_only/);
        logresult('CONFIG',"$path is DAFS exported but not enabled.")
            if ($enabled =~ /false/);

    } else {
        # path is not exported, so export now
        $dafs_export_name=$path;
        (undef, undef, $output) = say($fc,"dafs export create $dafs_export_name $path");
        logresult('FATAL', "DAFS export of $path failed. $output")
            if ($output =~ /((already exported)|(Duplicate export name))/);

        (undef, undef, $output) = say($fc,"dafs export list $dafs_export_name");
        logresult('FATAL', "DAFS export of $path failed")
            if ($output =~ /Export .* was not found/);
    }


    my $cc = make_conn($client);
    my $conn_timeout = ($cc->{timeout} < 500) ? 500 : $cc->{timeout};


    #---------------------------------------------------------------------
    # @step Check if entry for the FILER is present in Client's /etc/ddafs_servers
    # have to subtest client this... part
    # not complete..................
    #--------------------------------------------------------------------
    #my $filename = "/etc/ddafs_servers";
    #cat::do_shell("$sudo touch $filename; $sudo chmod 0666 $filename");
    #cat::append_unique_line("$filename","$filer");
    

    #-----------------------------------------
    # Probe the filers for ddafs files
    #-----------------------------------------
    (undef, undef, $output) = say($cc,
            "$sudo $client_ddafs_dir/ddafs probe -v open",
            -timeout, $conn_timeout);
    # how to check if this failed.... I dont know the failures for this

    $path .= "/ddafs";   # all ddafs files have to exist in dir 'ddafs'
    my $raw_dev_link = "/dev/ddafs/$filer/$filer_vol/$file_name";


    #--------------------------------------------
    # check if raw device already exists
    #--------------------------------------------
    (undef,undef,$output) = say($cc,"$sudo $client_ddafs_dir/ddafs list", 
                -timeout, $conn_timeout);
    if ($output =~ /$raw_dev_link/) {
        logcomment("$raw_dev_link already exists and is open");
    } else {
        #--------------------------------------------
        # Add a raw dev on dafs client
        #  this can be done in 2 ways
        #  1. using filer command 'mkfile' and then 
        #     using ddafs open from client
        #  2. using client ddafs command 'ddafs add'
        #  We will do it using the second way
        #--------------------------------------------

        # Add the raw dev
        (undef, undef, $output) = say($cc,
            "$sudo $client_ddafs_dir/ddafs add -v -f $file_size $filer $path/$file_name",
                                -timeout, $conn_timeout);
        # don't know if this is right way to check for failures...
        logresult('FATAL', "Add of raw dev link $raw_dev_link failed") 
            if ($output =~ /(fail|Invalid pathname)/i);

        # open the raw dev if it already exists
        # this happens sometimes if ddafs probe -v open fails to detect
        # bug ???
        if ($output =~ /already exists/i) {
            (undef, undef, $output) = say($cc, 
                "$sudo $client_ddafs_dir/ddafs open -v -fd -h $filer $path/$file_name $raw_dev_link",
                                -timeout, $conn_timeout);
        }

        # check if addition of raw dev link succeeded
        (undef, undef, $output) = say($cc,"$sudo $client_ddafs_dir/ddafs list", 
                                -timeout, $conn_timeout);
        logresult('FATAL',"Failure in opening Raw Dev Link $raw_dev_link") 
            if ($output !~ /$raw_dev_link/);

    }

    # now ddafs raw-device is created, opened and ready for use
    return($raw_dev_link);
}




=head2 is_veritas_installed($client,%opts)

check if veritas is installed on client

=over

=item Arguments

=over

=item client

(Required) : hostspec of client

=item path

(Optional) : path where veritas binary vxdisk could live. Default: /usr/sbin

=back

=back

=for html <hr />

=cut

# better way is to check if the package is installed.... but not sure which
# ones
#tada-c63.lab.netapp.com% pkginfo | grep -i veritas
#system      VRTSfsdoc      VERITAS File System Documentation
#application VRTSfspro      VERITAS File System Management Services Provider
#system      VRTSlic        VERITAS Licensing Utilities
#application VRTSob         VERITAS Enterprise Administrator Service
#application VRTSobgui      VERITAS Enterprise Administrator
#application VRTSvlic       VERITAS License Utilities
#system      VRTSvmdoc      VERITAS Volume Manager (user documentation)
#system      VRTSvmman      VERITAS Volume Manager, Manual Pages
#application VRTSvmpro      VERITAS Volume Manager Management Services Provider
#system      VRTSvxfs       VERITAS File System
#system      VRTSvxvm       VERITAS Volume Manager, Binaries

sub is_veritas_installed ( $% ) {

    my ($client,%args) = @_;
    my $sudo ="/usr/local/bin/sudo";

    my $path = defined($args{path}) ? $args{path} : '/usr/sbin';

    my $cc = make_conn($client);
    my (undef, undef, $out) = say($cc, "$sudo $path/vxdisk list");
    return(($out =~ /command not found/i) ? 0 : 1);
}



1;
