## @summary SLDIAG library functions to be used by NATE tests
## @status public
## @author c-dl-pavana@netapp.com

#------------------------------------------------------------------------------
# Subroutines

use boot;
use Switch;
require "tharnit_lib.pl";
use vars qw/$conn $host $FILER $FILER_CONN $BOOT_MODE $NETBOOT $ONTAP_MODE/;
our @prompts = ('LOADER(-\w+)?\*?>','CFE(-\w+)?\*?>',$FILER.'[\*>#]','Selection','\*>',$FILER.'[\:\:\>#]','Login incorrect','Password','login');
#our @prompts = ('LOADER(-\w+)?\*?>','CFE(-\w+)?\*?>',$FILER.'[\*>#]','Selection','\*>',$FILER.'[\:\:\>#]');

#*********************************************************************************************
#Function Name: disable_device_type
#Parameters   : Device Type, NULL
#Return Type  : Returns the Zero on success and Error No on Failure
#Description  : Disables the given device; Disables all if no parameter is passed.
#*********************************************************************************************

sub disable_device_type{
    my($dev_type) = @_;
    my @info;
    if ($dev_type)
    {
        say($conn, "sldiag device modify -dev $dev_type -selection disable");
    }
    else
    {
        my(undef,undef,$output,undef) = say($conn,"sldiag device show",-swallow);
        @info = split(/\n/, $output);
	
        foreach my $line (@info)
        {
            if ($line =~ /enabled/)
            {
                my @array = split(/\s+/, $line);
                say($conn, "sldiag device modify -dev $array[0] -name $array[1] -selection disable");
            }
        }
    }
    #PRUTHVI added the below check when $dev_type is not given
    #my @states = get_device_type_state($dev_type);
    my @states;
    if ($dev_type)
	{
		@states = get_device_type_state($dev_type);
	} else {
		@states = get_device_type_state();
	}

    foreach my $device_state (@states) {
      if($device_state =~ /enable/){
        return 301;
      } else {
        next;
      }
    }
    return 0;
}

#*********************************************************************************************
#Function Name: disable_device_name
#Parameters   : Device Name, NULL 
#Return Type  : Returns the Zero on success and Error No on Failure 
#Description  : Disables the given device; Disables all if no parameter is passed.
#*********************************************************************************************

sub disable_device_name{
  
   my($dev_name) = @_; 
   my @info;

   if ($dev_name)
   {
	say($conn, "sldiag device modify  -name $dev_name -selection disable");
   }
   else
   {
        my(undef,undef,$output,undef) = say($conn,"sldiag device show");
        @info = split(/\n/, $output);
        foreach my $line (@info)
        {
	    logcomment ("disable_device_name line=$line");
            if ($line =~ /enabled/)
            {
		logcomment ("line=$line");
                my @array = split(/\s+/, $line);
		logcomment ("$array[0],$array[1]");
                say($conn, "sldiag device modify -dev $array[0] -name $array[1] -selection disable");
            }
        }
    }

    my $state = get_device_state($dev_name);
    if($state =~ /disable/){
      return 0;
    } else {
      return 301;
    }
}

#*********************************************************************************************
#Function Name: enable_device_type
#Parameters   : Device Type, NULL
#Return Type  : Returns the Zero on success and Error No on Failure
#Description  : Enables the given device; Enables all if no parameter is passed.
#*********************************************************************************************

sub enable_device_type{
    my($dev_type) = @_;
    my @info;
    if ($dev_type)
    {  
      # the if body piece of code added to handle to enable all the storage disks 
	if ($dev_type =~ "storage") 
       {
         my @devices = get_device_names("storage");

            #enable_device_("storage");
              	foreach my $dev (@devices) 
                {
                  if ( $dev =~ /0[a-l]|[1-12][a-d]/ )  #modified by pradeepn
                   {
                      say( $conn, "sldiag device modify -dev storage -name $dev.* -selection enable",-timeout,1000);
                   }
                }

       } else { 
        say($conn, "sldiag device modify -dev $dev_type -selection enable");
       }
    }
    else
    {
        my(undef,undef,$output,undef) = say($conn,"sldiag device show",-swallow);#pruthvi added swallow
        @info = split(/\n/, $output);
	tstlog -comment, "inside else output=$output ~~~~~~~~~~~~~~~~~~~~~";
        foreach my $line (@info)
        {
	   
	    tstlog -comment, "inside foreach line=$line ~~~~~~~~~~~~~~~~";
            if ($line =~ /disabled/)
            {
                my @array = split(/\s+/, $line);
                say($conn, "sldiag device modify -dev $array[0] -selection enable");
            }

        }
    }
 
    my @states; 
    #my @states = get_device_type_state($output);
    if ($dev_type) # PRUTHVI added the loop to check when dev_type not given
	{	 @states = get_device_type_state($dev_type);
        } else	{
		 @states = get_device_type_state();
	}

    foreach my $state (@states) {
      if($state =~ /disable/){
        tstlog -comment, "state =$state";
        return 301;
      } else {
        next;
      }
    }
    return 0;
}

#*********************************************************************************************
#Function Name: enable_device_name
#Parameters   : Device Name, NULL
#Return Type  : Returns the Zero on success and Error No on Failure
#Description  : Enables the given device; Enables all if no parameter is passed.
#*********************************************************************************************

sub enable_device_name{
    my($dev_name) = @_;
    my @info;
    if ($dev_name)
    {
        say($conn, "sldiag device modify -name $dev_name -selection enable");
    }
    else
    {
        my(undef,undef,$output,undef) = say($conn,"sldiag device show");
        @info = split(/\n/, $output);
        foreach my $line (@info)
        {
            if ($line =~ /disabled/)
            {
                my @array = split(/\s+/, $line);
                say($conn, "sldiag device modify -dev $array[0] -name $array[1] -selection enable");
            }

        }
    }

    my $state = get_device_state($dev_name);
    if($state =~ /enable/){
      return 0;
    } else {
      return 301;
    }
}

#*********************************************************************************************
#Function Name: disable_device_type_only
#Parameters   : Device Type
#Return Type  : Returns the Zero on success and Error No on Failure
#Description  : Disables the given device and Enables remaining all 
#*********************************************************************************************

sub disable_device_type_only{
    my($dev_type) = @_;

        say($conn,"sldiag device modify -selection enable");

        if($dev_type) {
          say($conn, "sldiag device modify -selection disable -dev $dev_type");
        } else {
          say($conn, "sldiag device modify -selection disable -dev *");
        }

	my @states = get_device_type_state($dev_type);
	foreach my $state (@states) {
	  if($state =~ /enable/){
            return 301;
      	  } else {
            next;
      	  }
        }
	return 0;
}

#*********************************************************************************************
#Function Name: disable_device_name_only
#Parameters   : Device Name
#Return Type  : Returns the Zero on success and Error No on Failure
#Description  : Disables the given device and Enables remaining all
#*********************************************************************************************

sub disable_device_name_only{
    my($dev_name) = @_;

        say($conn,"sldiag device modify -selection enable");

        if($dev_name) {
          say($conn, "sldiag device modify -selection disable -name $dev_name");
        } else {
          say($conn, "sldiag device modify -selection disable -dev *");
        }

        my $state = get_device_state($dev_name);
        if($state =~ /disable/){
          return 0;
        } else {
          return 301;
        }
}

#*********************************************************************************************
#Function Name: enable_device_type_only
#Parameters   : Device Type
#Return Type  : Returns the Zero on success and Error No on Failure 
#Description  : Enables the given device and Disables remaining all
#*********************************************************************************************

sub enable_device_type_only{
    my($dev_type) = @_;
	
	say($conn,"sldiag device modify -selection disable");

	if($dev_type) {
	# the if body piece of code added to handle to enable all the storage disks
        	if ($dev_type =~ "storage")
            	{
         	my @devices = get_device_names("storage");

            	#enable_device_("storage");
                foreach my $dev (@devices)
                {
                  if ( $dev =~ /[0-5][a-d]/ )
                   {
                      say( $conn, "sldiag device modify -dev storage -name $dev.* -selection enable",-timeout,1000);
                   }
                }

       		} else {
        	say($conn, "sldiag device modify -dev $dev_type -selection enable");
       		}
	      say($conn, "sldiag device modify -selection enable -dev $dev_type");
	} else {
	  say($conn, "sldiag device modify -selection enable dev *");
	}

        my @states = get_device_type_state($dev_type);
        foreach my $state (@states) {
          if($state =~ /disable/){
            return 301;
          } else {
            next;
          }
        }
	return 0;
}

#*********************************************************************************************
#Function Name: enable_device_name_only
#Parameters   : Device Name
#Return Type  : Returns the Zero on success and Error No on Failure
#Description  : Enables the given device and Disables remaining all
#*********************************************************************************************

sub enable_device_name_only{
    my($dev_name) = @_;

        say($conn,"sldiag device modify -selection disable");

        if($dev_name) {
          say($conn, "sldiag device modify -selection enable -name $dev_name");
        } else {
          say($conn, "sldiag device modify -selection enable -dev *");
        }

        my $state = get_device_state($dev_name);
        if($state =~ /enable/){
          return 0;
        } else {
          return 301;
        }
}

#*********************************************************************************************
#Function Name:disable_adaptors
#Parameters   : NULL
#Return Type  : Disables fcal and sas adaptor devices if present 
#*********************************************************************************************
sub disable_adaptors {
	my $str;
	my @adaptors_list=('sas','fcal'); # can be added to this array as and when the new adaptors comes
	foreach my $adaptor (@adaptors_list){

		if(check_device($adaptor)){ ## checking for existence of the device
			say($conn, "sldiag device  modify -dev $adaptor -selection disable");
			(undef,undef,$str,undef)=say($conn, "sldiag device show -dev $adaptor");
			my @str_array=split('\n',$str);
			foreach my $line( @str_array){
				next if($line=~/-------------|DEV|NAME|sldiag|^\s*$/);
				if($line!~/\bdisabled\b/){
					return -1; #error status
				}
			} ## end of inner foreach statement
       
		}
	} # end of outer foreach statement 
	say($conn,"sldiag device show");
	return 0; # success return value

}

#*********************************************************************************************
#Function Name:enable_adaptors
#Parameters   : NULL
#Return Type  : Enables fcal and sas adaptor devices if present
#*********************************************************************************************

sub enable_adaptors {
 my $str;
    my @adaptors_list=('sas','fcal'); # can be added to this array as and when the new adaptors comes
    foreach my $adaptor (@adaptors_list){

			if(check_device($adaptor)){ ## checking for existence of the device
            say($conn, "sldiag device  modify -dev $adaptor -selection enable");
            (undef,undef,$str,undef)=say($conn, "sldiag device show -dev $adaptor");
            my @str_array=split('\n',$str);
            foreach my $line( @str_array){
                next if($line=~/-------------|DEV|NAME|sldiag|^\s*$/);
                if($line!~/\benabled\b/){
                    return -1; #error status
                }
            } ## end of inner foreach statement

        }
    } # end of outer foreach statement
	say($conn,"sldiag device show");

    return 0; # success return value

}


#*********************************************************************************************
#Function Name: clear_devicestatus
#Parameters   : NULL
#Return Type  : No Return 
#Description  : Clears sldiag device status list 
#*********************************************************************************************

sub clear_devicestatus{
    my(undef,undef,undef,undef) = say($conn, "sldiag device clearstatus");
} 

#*********************************************************************************************
#Function Name: clear_wait_devicestatus
#Parameters   : NULL
#Return Type  : Returns Zero on Success
#Description  : Clears all status logs and waits until all messages are cleared 
#*********************************************************************************************

sub clear_wait_devicestatus {
   while (1){
        my(undef,undef,$output,$prompt) = say($conn, "sldiag device clearstatus");
        if ($output =~ /SLDIAG: Cannot clear status while tests are still running/){
          sleep 10;
          next;
        }else {
          return 0;
        }
    }
}

#*********************************************************************************************
#Function Name: stop_clear_alltests
#Parameters   : Device Type
#Return Type  : Returns the value returned by clear_wait_devicestatus subroutine 
#Description  : Stops all tests belongs to the device, if nothing is passed then stops all tests 
#*********************************************************************************************

sub stop_clear_alltests{
    my($dev_type) = @_;
    if ($dev_type)
    {
        my(undef,undef,$output,undef) = say($conn, "sldiag device stop -dev $dev_type");
	return 308 if($output =~ /This feature is not implemented/);
    }
    else
    {
        my(undef,undef,$output,undef) = say($conn, "sldiag device stop",-swallow);#PRUTHVI added swallow
    }
    return clear_wait_devicestatus();
}
#*********************************************************************************************
#Function Name: run_test 
#Parameters   : Device Type, Device Name
#Return Type  : Returns the Error no if No Devices were selected and null on successful execution  
#Description  : Start running the given test on given device. If only dev is sent then tests on that device 
#*********************************************************************************************

sub run_test{
    my( $dev_type, $dev_name) = @_;
    my $output;
    if ($dev_type && $dev_name) {
	(undef,undef,$output,undef) = say($conn, "sldiag device run -dev $dev_type -name $dev_name");
    } elsif ($dev_type) {
       (undef,undef,$output,undef) = say($conn, "sldiag device run -dev $dev_type");
    } elsif ($dev_name) {
       (undef,undef,$output,undef) = say($conn, "sldiag device run -name $dev_name");
    } else {
      my(undef,undef,$output,undef) = say($conn, "sldiag device run");
    }

    (undef,undef,$output,undef) = say($conn, " ");
    if($output =~ /No Devices were selected|SLDIAG: Cannot run while tests are still running/) {
      return 304;
    }
} 

#*********************************************************************************************
#Function Name: sldiag_version 
#Parameters   : NULL
#Return Type  : Returns the version of sldiag 
#Description  : Get the version string of SLDIAG 
#*********************************************************************************************

sub sldiag_version{
    my(undef,undef,$output,$prompt) = say($conn,"\r\r\r"); 
    if(($ONTAP_MODE eq 10) && (($prompt =~ /login/) or ($prompt =~ /Login incorrect/)))
    {
	say($conn, "\r", '-wait', "login", -noprompt, -timeout, 500);
        say($conn, "admin", '-wait', "Password", -noprompt, -timeout, 50);
        say($conn, "netapp1!");
	say($conn, "run local", -noprompt);
    }
	
    my(undef,undef,$output,undef) = say($conn,"version",-swallow);
    return $output;
}


#*********************************************************************************************
#Function Name: check_device
#Parameters   : device type
#Return Type  : Returns true(1)if device is present on the filer if not,returns false(0)
#Description  : checks for particular device
#*********************************************************************************************

sub check_device {
 my ($dev) =@_;
my (undef,undef,$output) = say($conn,"sldiag de sh -dev $dev",-swallow,-timeout,100);
 if($output=~/\b$dev\b/)
  {
    return 1;
  }
 else
  {
   return 0;
  }
}



#*********************************************************************************************
#Function Name: get_device_types 
#Parameters   : NULL
#Return Type  : Returns all device types  
#Description  : This function returns array of all devices supported by sldiag 
#*********************************************************************************************

sub get_device_types{
        my @array;
        my (undef,undef,$output,undef) = say($conn, "sldiag device types");
	my @info = split(/\n/, $output);
        foreach my $line (@info)
        { 
          next if($line =~ /sldiag device|DEV|----------|$FILER/);
	  $line=~/\w+/;
          push(@array, $&);
        }
    return @array;
}

#*********************************************************************************************
#Function Name: get_device_names Dev:venkat2
#Parameters   : Device Type or NULL
#Return Type  : Returns all device names of the device type send. Returns all incase input parameter is NULL 
#Description  : This function returns list of all devices supported by sldiag 
#*********************************************************************************************

sub get_device_names{
    my ($dev_type) = @_;
    logcomment("Inside get_device_names");
$dev_type='*' unless($dev_type);
(undef,undef,$sldiag_out,undef)=say($conn,"sldiag device show -dev $dev_type", -swallow);
my @sldiag_devices;
foreach my $sldiag_line(split(/\n/,$sldiag_out))
{
        #next if(($sldiag_line !~ /$dev_type/) or ($sldiag_line =~ /sldiag/));
        next if($sldiag_line =~ /sldiag|DEV|----------|$FILER|SLDIAG override set|Initializing/);
        #tstlog -comment, "LINE:$sldiag_line";
        my ($temp1,$device_name,@temp)=split(/\s+/,$sldiag_line);
                push(@sldiag_devices,$device_name);
}
logcomment("Exiting get_device_names");
return @sldiag_devices;
}

#*********************************************************************************************
#Function Name: get_device_state Dev:venkat2
#Parameters   : Device Name
#Return Type  : Returns device state 'enabled' or 'disabled'
#Description  : This function returns state of the device.
#*********************************************************************************************
sub get_device_state{
	my ($dev_name)=@_;
	(undef,undef,$sldiag_out,undef)=say($conn,"sldiag device show -name $dev_name",-swallow);#PRUTHVI added swallow
	$sldiag_out =~ /enabled|disabled/;
	return $&;
}

#*********************************************************************************************
#Function Name: get_device_type_state Dev:asad
#Parameters   : Device Type
#Return Type  : Returns an array which contains device type and device state as 'enabled' or 'disabled'
#Description  : This function returns states of the same type devices.
#*********************************************************************************************
sub get_device_type_state{
        my ($dev_type)=@_;
        my $sldiag_out;
        if(!$dev_type) {
          (undef,undef,$sldiag_out,undef)=say($conn,"sldiag device show",-swallow);
        } else {
          (undef,undef,$sldiag_out,undef)=say($conn,"sldiag device show -dev $dev_type",-swallow);
        }
        my @array = $sldiag_out =~ /(\w+).\s+.+(enabled|disabled)/ig;
	 
        return @array;
}

#*********************************************************************************************
#Function Name: enable_sldiag Dev:venkat2
#Parameters   : Console Handle
#Return Type  : Returns 0 on success and ERR no on failure
#Description  : This function checks availability of sldiag on the filer and enables if available
#*********************************************************************************************
#sub enable_sldiag {
sub enable_sldiag (;$){ ## added for Ravi - dharmesh
	tstlog -comment, " Inside enable_sldiag BOOT_MODE=$BOOT_MODE ONTAP_MODE=$ONTAP_MODE";
        $conn = shift if (@_); ## added for Ravi -dharmesh
	@sld_prompts=$conn->prompts();
        push(@sld_prompts,@prompts);
        $conn->prompts(@sld_prompts);
	param("ONTAP_MODE","-default",""); #Added by pradeep to handle 10 mode 15122009
	&sldiag_version;

	param("BOOT_MODE","-default","");
	param("NETBOOT","-default","");
	
	my $current_state = &get_filer_state;
	logcomment("current boot mode is $current_state");
	if ($current_state =~ /FREE BSD/)
	{
		(undef,undef,$sldiag_out,undef)=say($conn,"run -node $FILER");
	        return 355 if($sldiag_out !~ /Type 'exit' or 'Ctrl-D' to return to the CLI/);
		$current_state = UP;
	}
	if(($BOOT_MODE) or ($current_state eq 'NOWHERE') or ($current_state eq 'FIRMWARE')) #Added last condition by venkat on 23112009
	{
		logcomment("BOOT_MODE is set to $BOOT_MODE or filer is in NOWHERE state.");
		&set_boot_mode if(($current_state ne $BOOT_MODE) or ($current_state eq 'FIRMWARE')); #Added last condition by venkat on 23112009
	}

	#&set_boot_mode if($current_state ne $BOOT_MODE);#Added by venkat to fix running sldiag commands at loader prompt. 20091013
	tstlog -comment, "calling get_filer_state";
	$current_state = &get_filer_state;
	logcomment("current boot mode is $current_state");
        if ($current_state =~ /FREE BSD/) #Added by pradeep to handle 10 mode 17122009
        {
                (undef,undef,$sldiag_out,undef)=say($conn,"run -node $FILER");
                return 355 if($sldiag_out !~ /Type 'exit' or 'Ctrl-D' to return to the CLI/);
                $current_state = UP;
        }
	if($current_state =~ /UP/)
	{
		tstlog -comment, "In enable_sldiag current_state=$current_state";
		(undef,undef,$sldiag_out,undef)=say($conn,"priv set test",-swallow);#PRUTHVI
		say($conn,"priv set advanced") if($sldiag_out =~ /admin/);
		say($conn,"\r",-swallow);# PRUTHVI added as sometimes sldiag_out has junk value
		(undef,undef,$sldiag_out,undef)=say($conn,"sldiag device show",-swallow,-timeout,300);
		return 355 if($sldiag_out =~ /sldiag not found/);
		tstlog -comment, " sldiag_out--$sldiag_out--"; #PRUTHVI
		if($sldiag_out =~ /Only authorized/)
		{
			tstlog -comment, "inside password checking"; ## PRUTHVI
			my $password_string=&compose_password;
			say($conn,"sldiag mfg", '-wait',"mode password:", -noprompt);
			(undef,undef,$sldiag_out,undef)=say($conn,$password_string);
			tstlog -comment, "PASSWORD RESULT:$sldiag_out";
			return 355 unless($sldiag_out =~ /Manufacturing mode is enabled/);
		} else { #sometimes sldiag_out is having junk value.so adding this loop to double check
			say($conn," ",-swallow);
			(undef,undef,$sldiag_out,undef)=say($conn,"sldiag device show",-swallow,-timeout,300);
			return 355 if($sldiag_out =~ /sldiag not found/);
			tstlog -comment, " sldiag_out --> $sldiag_out <--"; #PRUTHVI
	                if($sldiag_out =~ /Only authorized/)
                		{
                        	tstlog -comment, "inside password checking-2"; ## PRUTHVI
                        	my $password_string=&compose_password;
                        	say($conn,"sldiag mfg", '-wait',"mode password:", -noprompt);
                        	(undef,undef,$sldiag_out,undef)=say($conn,$password_string);
                        	tstlog -comment, "PASSWORD RESULT:$sldiag_out";
                       	 	return 355 unless($sldiag_out =~ /Manufacturing mode is enabled/);
				}
		}

	}

	(undef,undef,$sldiag_out,undef)=say($conn,"sldiag de show",-swallow, -timeout,300);
	if($sldiag_out =~ /Only authorized/)
	{
		logcomment (" second loop");
		my $password_string=&compose_password; #added by PRUTHVI
		say($conn,"sldiag mfg", '-wait',"mode password:", -noprompt);
		#(undef,undef,$sldiag_out,undef)=say($conn,"ctymfg");
		#PRUTHVI added the below lines
                (undef,undef,$sldiag_out,undef)=say($conn,$password_string);
		tstlog -comment, "PASSWORD RESULT:$sldiag_out";
		return 355 unless($sldiag_out =~ /Manufacturing mode is enabled/);
	} else {
		&set_default_options;
		stop_clear_alltests();
		return 0;
	      }
tstlog -comment, " exiting enable_sldiag";
}


#*************************************************************************************************************************************************************
#Function Name: enable_maint_mfg  Dev:pradeepn
#Parameters   : NULL
#Description : Enables the mfg mode if it is not enabled. If mfg mode is already enabled, function comes out with return value  355 and  performing no operation
#***************************************************************************************************************************************************************

sub enable_maint_mfg {
 my($match,$before,$after)=say($conn ,"sldiag mfg", -swallow, -noprompt, '-wait', 'Please enter manufacturing mode password', 'Manufacturing mode is enabled');
        say($conn," ");    
         if($match!~/Manufacturing mode is enabled/)
            {           
             my $password_string=&compose_password;
                        say($conn,"sldiag mfg", '-wait',"mode password:", -noprompt);
                        (undef,undef,$sldiag_out,undef)=say($conn,$password_string);
                        tstlog -comment, "PASSWORD RESULT:$sldiag_out";
                        return 355 unless($sldiag_out =~ /Manufacturing mode is enabled/);
            }
  }


#*********************************************************************************************
#Function Name: get_test_state  Dev:venkat2
#Parameters   : Device Type, Test Name
#Return Type  : Returns test state 'enabled' or 'disabled'. Returns 'undef' if test is not available
#Description  : This function returns state of the test
#*********************************************************************************************
sub get_test_state{
	my ($dev_type,$test_name)=@_;
	(undef,undef,$sldiag_out,undef)=say($conn,"sldiag test show -dev $dev_type");
	foreach(split(/\n/,$sldiag_out))
	{
		next unless(/$test_name/);
		/enabled|disabled/;
		return $&;
	}
	return 'undef';
}

#*********************************************************************************************
#Function Name: get_device_tests_state  Dev:asad
#Parameters   : Device Type
#Return Type  : Returns an array of test state which contains test index and test state as 'enabled' or 'disabled'. 
#Description  : This function returns array of state of the tests
#*********************************************************************************************
sub get_device_tests_state{
        my ($dev_type)=@_;
        my $sldiag_out;
        if(!$dev_type) {
          (undef,undef,$sldiag_out,undef)=say($conn,"sldiag test show");
        } else {
          (undef,undef,$sldiag_out,undef)=say($conn,"sldiag test show -dev $dev_type");
        }
        my @array = $sldiag_out =~ /(\w+).\s+.+(enabled|disabled)/ig;
        return @array;
}


#*********************************************************************************************
#Function Name: store_options  Dev:pradeepn  
#Parameters   : Device Type
#Return Type  : Returns an array of options and option values of a device
#Description  : This function returns array of options and option values of a device 
#*********************************************************************************************


sub store_options{
my ($dev_type)=@_;
my(undef,undef,$option_out,undef)=say($conn,"sldiag option show -dev $dev_type",-swallow);

my @option_outs=split(/\n/,$option_out);

my @stored_options;
my $flag=1;
foreach my $option (@option_outs)
  {
	next if($option=~/^\s*$|---------|sldiag option|OPTION NAME/);
	my @array=split(/\s+/,$option);
	my $scalar= @array;
	if($scalar == 3)  # when 3 fields are present, the 2nd field indicates the device name , so option should be set to that particular device
   	  {
      		$array[1]=~s/(\d+)/ $1/;
      		$array[$scalar-1]=$array[$scalar-1].' '.$array[1];
      		$flag=0;
       		# tstlog -comment, "\n $option\n";
      		push(@stored_options,$array[0]);
      		push(@stored_options,$array[$scalar-1]);
  	  }

  }
foreach my $option1 (@option_outs)
  {
	if ($flag==0){last;}
	next if($option1=~/^\s*$|---------|sldiag option|OPTION NAME/);
	my @array=split(/\s+/,$option1);
	my $scalar= @array;

	 if( $scalar == 2)
        	{
      	        	push(@stored_options,$array[0]);
                        push(@stored_options,$array[$scalar-1]);

                 } 

  # tstlog -comment , "\n\n $array2[0]\n $array2[$scalar-1]\n";  
  }
return @stored_options;

}  


#*********************************************************************************************
#Function Name: restore_options  Dev:pradeepn
#Parameters   : stored options  value array
#Return Type  : NULL
#Description  : before calling this function, store_options function must be called to save an array 
#                of options and option values , the options are  reset to the stored values
#********************************************************************************************

sub restore_options{
if(scalar(@_)=~0)
 {
   return 305;
 }
my (@array_options)=@_;
my $i=0;
my $scalar=@array_options;
for($i=0; $i<$scalar; $i+=2)
 {
  set_option($array_options[$i],$array_options[$i+1]);
 }
}

#*********************************************************************************************
#Function Name: get_test_index  Dev:venkat2
#Parameters   : Device Type, Test Name
#Return Type  : Returns the index number of the given test. Returns 'undef' if given test name invalid
#Description  : This function returns index number of the test
#*********************************************************************************************
sub get_test_index{
	my ($dev_type,$test_name)=@_;
	logcomment ("get_test_index dev_type=$dev_type-- test_name=$test_name");
	(undef,undef,$sldiag_out,undef)=say($conn,"sldiag test show -dev $dev_type",-swallow);
	foreach(split(/\n/,$sldiag_out))
	{
		next unless(/$test_name/);
		($index,@temp)=split(/\s+/);
		$index =~ /\d*/;
		logcomment ("index=$index");
		return $&;
	}
	return 'undef';
}

#*********************************************************************************************
#Function Name: get_option 
#Parameters   : Option Name
#Return Type  : Option value, returns error no in case insufficient argument passed.
#Description  : iThis function gives the value of a given option name.
#*********************************************************************************************

sub get_option{
    if(scalar(@_)=~0){
      return 305;
    }
    my ($option_name) = @_;
    my(undef,undef,$output,undef) = say($conn, "sldiag option show $option_name");
    return 303 if($output =~ /ERROR: specified option could not be found or processed/);
    @info = split(/\n/, $output);
    foreach my $line (@info)
    {
        my(@arr) = split(/ \s+/,$line);
	if ((scalar(@arr)=~2)&&($arr[0]=~"$option_name")){
		return $arr[-1];
	}
	elsif((scalar(@arr)=~3)&&((my $optname = "$arr[0] $arr[1]")=~"$option_name")){
    		return $arr[-1];
	}
    }
}

#*********************************************************************************************
#Function Name: set_option
#Parameters   : Option Name, Option value
#Return Type  : 0 on success, Error No on Failure. Returns error no in case of insufficient parameters. 
#Description  : Sets the passed value to the passed option 
#*********************************************************************************************

sub set_option{
  my $option_name;
  my @option_values;
 
    if(scalar(@_)=~0){
      return 305;
    }
     if ((scalar(@_))>2)  #added by pradeepn  used when option value consists of device name and a value. Its applicable in nvram(margin)options.
     {
        ($option_name, @option_values)= @_;
        $option_value ="@option_values";   # concatinating device name and option value 
        tstlog -comment, "\n\n ******************************************\n";
      } 
     else
   {
    ($option_name, $option_value) = @_;
    }
    my $ret_value = get_option($option_name);
    if ($ret_value eq $option_value)
    {
        tstlog -comment, "$option_name is already set to $option_value";
	return 0;
    }
    else
    {
	my(undef,undef,$output,undef) = say($conn, "sldiag option modify $option_name $option_value",-swallow,-timeout,300);
	if ($output =~ /out of range|too long|input is wrong|Syntax error|[Ii]ncorrect|Illegal setting|exceeds the range|Invalid|Wrong options|ERROR: specified option could not be found or processed|enter a valid value/i)
	{
           return 303;
	} else {
	   return 0;
	}
    }
}

#*********************************************************************************************
#Function Name: disable_test
#Parameters   : Device Type or NULL
#Return Type  : 0 on success, Error No on Failure
#Description  : Disable the tests on given device;Disables all if no parameter is passed
#*********************************************************************************************

sub disable_test{
    my($dev_type, $test_name) = @_;
    if ($dev_type)
    {
        if($dev_type && $test_name) {
           my $index = get_test_index($dev_type, $test_name);
           say($conn, "sldiag test modify -dev $dev_type -index $index -selection disable");
           my $state = get_test_state($dev_type, $test_name);
           return 0 if($state =~ /disable/);
        } else {
       	   say($conn, "sldiag test modify -dev $dev_type -selection disable");
	}
    }
    else
    {
        my(undef,undef,$output,undef) = say($conn,"sldiag test show");
        my @info = split(/\n/, $output);
        foreach my $line (@info)
        {
            if ($line =~ /enabled/)
            {
                my @array = split(/\s+/, $line);#PRUTHVI 
                say($conn, "sldiag test modify -index $array[0] -dev $array[1] -selection disable");
            }
        }
        #return 0;
    }
    my @states;
    #my @states = get_device_tests_state($dev_type); 
    my $count=0;
    #PRUTHVI added the below condition to check when the dev_type is not given
    if($dev_type)
  	{
		@states = get_device_tests_state($dev_type);
	} else {
		@states = get_device_tests_state();
	}
    foreach my $test_state (@states) {
      if($test_state =~ /enable/){
        return 302;
      } else {
        next;
      }
    }
    return 0;
}

#*********************************************************************************************
#Function Name: enable_test
#Parameters   : Device Type or NULL
#Return Type  : 0 on success, Error No on Failure
#Description  : Enable the tests on given device;Enables all if no parameter is passed
#*********************************************************************************************
    
sub enable_test{
    my($dev_type, $test_name) = @_;
    if ($dev_type)
    {
	if($test_name) {
           my $index = get_test_index($dev_type, $test_name);
           say($conn, "sldiag test modify -dev $dev_type -index $index -selection enable");
	   my $state = get_test_state($dev_type, $test_name);
	   return 0 if($state =~ /enable/);
	} else {
           say($conn, "sldiag test modify -dev $dev_type -selection enable");
	}
    }
    else
    {
        my(undef,undef,$output,undef) = say($conn,"sldiag test show",-swallow,-timeout,100);
        my @info = split(/\n/, $output);
        foreach my $line (@info)
        {
            if ($line =~ /disabled/)
            {
                my @array = split(/ +/, $line);
                say($conn, "sldiag test modify -index $array[0] -dev $array[1] -selection enable");
            }
        }
        #return 0;#PRUTHVI commented this
    }

    #PRUTHVI added the below check when $dev_type is not given
    #my @states = get_device_tests_state($dev_type);
    my @states;
   if ($dev_type)
  	{
    		 @states = get_device_tests_state($dev_type);
	} else {
    		 @states = get_device_tests_state();
	}
		
    foreach my $test_state (@states) {
      if($test_state =~ /disable/){
        return 302;
      } else {
        next;
      }
    }
    return 0;
}

#*********************************************************************************************
#Function Name: disable_test_only
#Parameters   : Device Type, Index
#Return Type  : Returns the Zero on success and Error No on Failure as well in case of insufficient parameters
#Description  : Disables the given tests and enables remaining all 
#*********************************************************************************************

sub disable_test_only{
    my($dev_type, $test_name) = @_;

        return 305 if(!($dev_type && $test_name));
        say($conn,"sldiag test modify -dev * -selection enable");

        my $index = get_test_index($dev_type, $test_name);
        say($conn, "sldiag test modify -dev $dev_type -index $index -selection disable");

        my $state = get_test_state($dev_type, $test_name);
        if ($state =~/disable/){
          return 0;
        } else {
          return 302;
        }
}

#*********************************************************************************************
#Function Name: enable_test_only
#Parameters   : Device Type, test name
#Return Type  : Returns the Zero on success and Error No on Failure as well in case of insufficient parameters
#Description  : Enables the given tests and disables remaining all 
#*********************************************************************************************

sub enable_test_only{
    my($dev_type, $test_name) = @_;
        return 305 if(!($dev_type && $test_name));
        say($conn,"sldiag test modify -dev * -selection disable");

        my $index = get_test_index($dev_type, $test_name);
        say($conn, "sldiag test modify -dev $dev_type -index $index -selection enable");

        my $state = get_test_state($dev_type, $test_name);
        if ($state =~/enable/){
          return 0;
        } else {
          return 302;
        }
}

#*********************************************************************************************
#Function Name: stop_test
#Parameters   : Device Type, Device Name
#Description  : Stop the given test on given device
#*********************************************************************************************

sub stop_test{
    my( $dev_type, $dev_name) = @_;
    if ($dev_type && $dev_name)
    {
        my(undef,undef,$output,undef) = say($conn, "sldiag device stop -dev $dev_type -name $dev_name", '-noprompt', '-wait', "<SLDIAG:_ALL_TESTS_COMPLETED>", -timeout,1000);
    }
    elsif ($dev_type)
    {
       my(undef,undef,$output,undef) = say($conn, "sldiag device stop -dev $dev_type", '-noprompt', '-wait', "<SLDIAG:_ALL_TESTS_COMPLETED>", -timeout,1000);
    }
    elsif ($dev_name)
    {
       my(undef,undef,$output,undef) = say($conn, "sldiag device stop -name $dev_name", '-noprompt', '-wait', "<SLDIAG:_ALL_TESTS_COMPLETED>", -timeout,1000);
    } else {
       my(undef,undef,$output,undef) = say($conn, "sldiag device stop", '-noprompt', '-wait', "<SLDIAG:_ALL_TESTS_COMPLETED>", -timeout,1000);
	}
}

#*********************************************************************************************
#Function Name: get_test_status 
#Parameters   : Device Type, Device Name
#Return Type  : Status of the Test. Returns error no in case of insifficient parameters.
#Description  : Get the status of the device test
#*********************************************************************************************

sub get_test_status{
    my($dev_type, $dev_name) = @_;
    
    if ($dev_type && $dev_name) {
    	my (undef,undef,$output,undef) = say($conn, "sldiag device status -dev $dev_type -name $dev_name",-swallow);#pruthvi added swallow
    		if ($output =~ /ERROR: specified device could not be found or processed/) {
      			return 306;
    		}
   	my @array = $output =~ /(\w+).+\s+.+(passed|running|aborted|failed|pending|time|full|error|Completed)/ig; 
    	return $array[-1];
    } elsif($dev_type) {
	my (undef,undef,$output,undef) = say($conn, "sldiag device status -dev $dev_type",-swallow);
                if ($output =~ /ERROR: specified device could not be found or processed/) {
                        return 306;
                }
        my @array = $output =~ /(\w+).+\s+.+(passed|running|aborted|failed|pending|time|full|error|Completed)/ig;
        return $array[-1];
    } else {
	say($conn, "sldiag device status");
    }
}

#***************************************************************************************************
#Function Name: run_test_wait
#Parameters   : Device Type, Device Name
#Return Type  : Returns the Status of a test
#Description  : Start running the given test on given device and wait till the completion of a test
#***************************************************************************************************

sub run_test_wait{
    my($dev_type, $dev_name, $timeout) = @_;
    my $prompt;
    $timeout = 300 if(!$timeout);
    tstlog -comment, "TIMEOUT of the script: $timeout";
    if ($dev_type && $dev_name) {
        (undef,undef,undef,$prompt) = say($conn, "sldiag device run -dev $dev_type -name $dev_name",-swallow, '-noprompt','-prompt', "<SLDIAG:_ALL_TESTS_COMPLETED>|SLDIAG: No tests are enabled|No Devices were selected", -timeout, $timeout);
    } elsif ($dev_type) {
       (undef,undef,undef,$prompt) = say($conn, "sldiag device run -dev $dev_type",-swallow, '-noprompt', '-prompt', "<SLDIAG:_ALL_TESTS_COMPLETED>|SLDIAG: No tests are enabled|No Devices were selected", -timeout, $timeout);
    } elsif ($dev_name) {
       (undef,undef,undef,$prompt) = say($conn, "sldiag device run -name $dev_name",-swallow, '-noprompt', '-prompt', "<SLDIAG:_ALL_TESTS_COMPLETED>|SLDIAG: No tests are enabled|No Devices were selected", -timeout, $timeout);
    } else {
      (undef,undef,undef,$prompt) = say($conn, "sldiag device run",-swallow, '-noprompt', '-prompt', "<SLDIAG:_ALL_TESTS_COMPLETED>|SLDIAG: No tests are enabled|No Devices were selected");
    }
    #Below validation check is added to verify either devices or tests are not enabled
    return 301 if($prompt =~ /No Devices were selected|SLDIAG: No tests are enabled/);
}

#***************************************************************************************************
#Function Name: mfg_sta_validate #Dev:venkatn
#Parameters   : Device Type, Device Name
#Return Type  : Returns nothing. Just reports all syntax and order errors in the log.
#Description  : This function to validate syntax and order of device status in mfg style
#***************************************************************************************************
sub mfg_sta_validate{

my ($dev_type, $dev_name) = @_;

say($conn,"sldiag option modify display-style mfg");

if ($dev_type && $dev_name)
{
        (undef,undef,$sldiag_out,undef)=say($conn,"sldiag device status -dev $dev_type -name $dev_name");
} elsif ($dev_type)
{
        (undef,undef,$sldiag_out,undef)=say($conn,"sldiag device status -dev $dev_type",-timeout,300);
} else {
        (undef,undef,$sldiag_out,undef)=say($conn,"sldiag device status");
}
@style_headers=('TEST START','DEVTYPE','NAME','START DATE','STATUS','STARTING','END DATE','LOOP','TEST END');
$connounter=0;
foreach (split(/\n/,$sldiag_out))
{
        #switch ($_) {
        #        case /TEST START/ { tstlog -fail, "TEST START syntax failed" unless(/-+/) }
        #        case /TEST END/ {tstlog -fail, "TEST END syntax failed" unless(/-+/)}
        #        case /DEVTYPE/ {tstlog -fail, "DEVTYPE syntax failed" unless(/nic/)}
        #        case /NAME/ {tstlog -fail, "NAME syntax failed" unless(/NIC details|Configure NIC Media Type|Configure NIC Flow Control|NIC Packet Test/)}
        #        case /END DATE/ {tstlog -fail, "END DATE syntax failed" unless(/\w{3}\s\w{3}\d?\d?:\d?\d:\s\w{3}\s\w{4}/)}
        #        case /START DATE/ {tstlog -fail, "START DATE syntax failed" unless(/\w{3}\s\w{3}\d?\d?:\d?\d:\s\w{3}\s\w{4}/)}
        #        case /STATUS/ {tstlog -fail, "START DATE syntax failed" unless(/[Cc]ompleted|COMPLETED|[Pp]assed|PASSED|[Ff]ailed|FAILED|[Aa]borted|ABORTED|[Rr]unning|RUNNING/)}
        #        case /LOOP/ {tstlog -fail, "LOOP syntax failed" unless(/\d*\/\d*/)}
        #        else {tstlog -comment "Generic statement found:$_"}
        #}
#ABOVE SWITCH IS NOT WORKING. BELOW CODE IS TEMPARORY WORK AROUND TO RELACE ABOVE SWITCH CASE

        if(/TEST START/) { tstlog -fail, "TEST START syntax failed" unless(/-+/) }
        elsif(/TEST END/) {tstlog -fail, "TEST END syntax failed" unless(/-+/)}
       elsif(/DEVTYPE/) {tstlog -fail, "DEVTYPE syntax failed" unless(/cna|acp|acpm|fcache|nic|mem|nvmem|volume|interconect|bootmedia/)}
        #elsif(/NAME/) {tstlog -fail, "NAME syntax failed" unless(/NIC details|Configure NIC Media Type|Configure NIC Flow Control|NIC Packet Test/)}
        elsif(/END DATE/) {tstlog -fail, "END DATE format failed. Should be in Day Mon DD HH:MM:SS ZON YYYY format" unless(/\w{3}\s\w{3}\s\d{0,1}\s?\d{0,1}\W\d{0,1}\S\d{0,1}\S\w{1,3}\S\w{2,4}/)}
        elsif(/START DATE/) {tstlog -fail, "START DATE format failed. Should be in Day Mon DD HH:MM:SS ZON YYYY format" unless(/\w{3}\s\w{3}\s\d{0,1}\s?\d{0,1}\W\d{0,1}\S\d{0,1}\S\w{1,3}\S\w{2,4}/)}
        elsif(/STATUS/) {tstlog -fail, "START DATE syntax failed" unless(/[Cc]ompleted|COMPLETED|[Pp]assed|PASSED|[Ff]ailed|FAILED|[Aa]borted|ABORTED|[Rr]unning|RUNNING/)}
        elsif(/LOOP/) {tstlog -fail, "LOOP syntax failed" unless(/\d*\/\d*/)}
    #    else {  if(/[Pp]anic|PANIC/) { tstlog -fatal, "PANIC FOUND";}
     #           elsif (/[Ee]rror|ERROR|[Ff]ailed|FAILED/) {tstlog -warn, "FAIL or ERROR String found";}
      #          else { }#For Future
       # }

        if(/TEST START|DEVTYPE|NAME|START DATE|STATUS|STARTING|END DATE|LOOP|TEST END/)
        {
                tstlog -fail, "$_ header is not in right order" unless(/$style_header[$connounter]/);
                #tstlog -pass, "$_ header is in right order" if(/$style_header[$connounter]/);
                $connounter++;
                $connounter=0 if($connounter == 9);
        }
}
}

#***************************************************************************************************
#Function Name: is_test_failed 
#Parameters   : Device Type
#Return Type  : Returns One on success and Zero on failure
#Description  : This function to validate whether all the tests has been PASSED/FAILED
#***************************************************************************************************
sub is_test_failed{
    my($dev_type) = @_;
    set_option("display-style", "default");
    my $output, $flag;
    if ($dev_type)
    {
        (undef,undef,$output,undef) = say($conn, "sldiag device status -dev $dev_type");
    }
    else
    {
      say($conn,,-swallow);#pruthvi added to clear the buffer
      (undef,undef,$output,undef) = say($conn, "sldiag device status",-swallow);
    }

tstlog -comment, "#############$output#############";
    my @info = split(/\d+\/\d+/, $output);
    $info[-1] =~ s/\s+$//;
    foreach my $line (@info)
    {
       next unless ($line);
       my @array = $line =~ /(\w+).+\s+.+(passed|running|aborted|failed|pending|time|full|error|Completed)/ig;
         tstlog -comment, "\n$array[-1]\n";
         if ($array[-1] =~ /PASSED|[Pp]assed|[Cc]ompleted|COMPLETED/i){
         $flag=1;
       } else {
           $flag=0;
           last;
       }
    }
    return $flag;
}

#***************************************************************************************************
#Function Name: set_default_options
#Parameters   : Device Type or NULL
#Return Type  : No Return
#Description  : Sets all values to default values. Sets only specified device type options is dev type passed else sets all generic options to their default values.
#***************************************************************************************************
sub set_default_options{
#  my($dev_type)=@_; #Waiting the default values from developers

#As of now supports only gereric options
%option_values=("display-style","default","display-dest","log","errstop","enable","loop","1","signaldone","enable","seed","unset");
foreach(keys(%option_values))
{
	$opt_val=get_option($_);
	chomp($opt_val);
	chop($opt_val) if($opt_val =~ /enabled|disabled/);
	set_option($_,$option_values{$_}) if($opt_val ne $option_values{$_});
}


}

#***************************************************************************************************
#Function Name: get_test_list  Author:venkat2
#Parameters   : Device Type or NULL
#Return Type  : all tests of the given device type. returns all tests if NULL passed
#Description  : Returns the list of all tests of the passed device type. returns all tests if NULL passed
#***************************************************************************************************
sub get_test_list1{

my ($dev_type)=@_;
$dev_type='*' unless($dev_type);
(undef,undef,my $sld_out,undef) = say($conn, "sldiag test show -dev $dev_type");
my @tests;
foreach(split(/\n/,$sld_out))
{
	next if(/INDEX|-+/);
        if(/serviceProcessor|environmentals/)
         {}
          (undef,my $dev_type_filer,my $pre_test,undef)=split(/\s\s\s+/);
	(undef,my $dev_type_filer,my $pre_test,undef)=split(/\s\s\s+/);
	if($pre_test =~ /disable|enabled/) {
		($test,undef)=split(/disabled|enabled/,$pre_test);
	} else {
		$test=$pre_test;
	}
	push(@tests,$test);
}
return @tests;
}

#***************************************************************************************************
#Function Name: get_test_list  Author:pradeepn
#Parameters   : Device Type or NULL
#Return Type  : all tests of the given device type. returns all tests if NULL passed
#Description  : Returns the list of all tests of the passed device type. returns all tests if NULL passed
#***************************************************************************************************
sub get_test_list{

my ($dev_type)=@_;
$dev_type='*' unless($dev_type);
(undef,undef,my $sld_out,undef) = say($conn, "sldiag test show -dev $dev_type");
my @tests;
foreach(split(/\n/,$sld_out))
{
        next if(/INDEX|-+/);
        my @test_name;
         my @test_line_output=split(/\s+/);
          splice(@test_line_output,0,2);
         foreach my $val (@test_line_output)
         {
          if($val=~/disable|enable/) { last; }
             push(@test_name,$val);
         }
         $test="@test_name" ;
        push(@tests,$test);
}
return @tests;
}




#************************************************************************************************************************
#Function Name: test_reorder
#Parameters   : Device Type, Index and action(enable/disable) 
#Return Type  : Return Error No if test index does not belong to the specified device type and/or insufficient parameter
#Description  : Reorders the test execution precedance for a specified test indexes of respective device type.
#************************************************************************************************************************
sub test_reorder {
  
  my ($dev_type, $index, $action) = @_;
  my $output;
  return 305 if(!($dev_type, $index));
  if($action) {
    (undef,undef,$output,undef) = say($conn, "sldiag test reorder -dev $dev_type -index $index -selection $action");
  } else {
    (undef,undef,$output,undef) = say($conn, "sldiag test reorder -dev $dev_type -index $index -selection disable");
  }
    if($output =~ /index is required|ERROR: a specified test index was previously specified|All test ordering has been cleared|ERROR: a specified test index does not belong to the specified device type|missing a required value/) {
      return 307;
    } else {
      return 0;
    }
}
#**********************************************************************************************************************
#Function Name	:get_boot_mode
#Parameters	:NONE
#Return Type	: 0 on success and ERROR No on Failure
#Description	:Checks the filer state and brings in to the required state based on BOOT_MODE and NETBOOT parameters
#**********************************************************************************************************************
sub get_boot_mode {

	tstlog -comment, "Checking boot mode";
	if($NETBOOT)
	{
		say($conn,"halt", -prompt,'LOADER', -noprompt, -timeout, 300);
		say($conn,"ifconfig e0a -auto", -prompt, 'LOADER', -noprompt, -timeout, 100);
		if($BOOT_MODE eq 'MAINT')
		{
			say($conn,"netboot $NETBOOT",'-wait', "Press CTRL", -noprompt, -timeout, 300);
                	say($conn,"\cC", '-wait', "Selection", -noprompt);
                	my (undef,undef,undef,$prompt) = say($conn,"5", -prompt,"Continue");
                	say($conn,"yes") if($prompt =~/Continue/);
		} else {
			say($conn, "netboot $NETBOOT", '-wait', "Selection", -noprompt, -timeout, 300);	
			say($conn, "1", -timeout, 300);
		}
	} else {

		if($BOOT_MODE eq 'MAINT')
		{
			tstlog -comment, "Taking filer to MAINT mode";
			say($conn, "halt -f -t 0", '-wait', 'LOADER', -noprompt, -timeout, 300);
			say($conn,"boot_ontap",'-wait', "Press CTRL", -noprompt, -timeout, 300);
                	say($conn,"\cC", '-wait', "Selection", -noprompt);
                        my (undef,undef,undef,$prompt) = say($conn,"5", -prompt,"Continue");
                        say($conn,"yes") if($prompt =~/Continue/);
		} else {
			tstlog -comment, "Taking filer to NORMAL mode";
			say($conn, "halt -f -t 0", '-wait', 'LOADER', -noprompt, -timeout, 300);
			say($conn, "boot_ontap", -timeout, 300);	
		}
	}
}

#**********************************************************************************************************************
#Function Name	:get_filer_state
#Parameters	:NONE
#Return Type	:any one of the following based on filer state MAINT,FIRMWARE,UP,BOOT MENU, FREE BSD,NOWHERE
#Description	:Returns the state of the filer
#**********************************************************************************************************************

sub get_filer_state {
sleep 5; #Pruthvi

say($conn,"\r ",-swallow);
#(undef,undef,$sldiag_out,undef)=say($conn,"vol status",-swallow);
my (undef,undef,$sldiag_out,$prompt)=say($conn,"vol status",-swallow);
	tstlog -comment, "BOOT MODE is $BOOT_MODE";
	logcomment (" sldiag_out=$sldiag_out--prompt=$prompt--"); 
        if ($sldiag_out =~ /Please use|Use/)# changed by pradeepn output of vol status command on absolut gives only USe volume 
        {
#Maintanace mode
		return 'MAINT';		
	} elsif($sldiag_out =~ /Invalid command/) {

#LOADER prompt
		return 'FIRMWARE'; 
	} elsif($sldiag_out =~ /ERROR/) {
#If filer is in FreeBSD mode
		return 'FREE BSD';
	} elsif($sldiag_out =~ /Volume State/) {
#If filer is in ontap mode
		return 'UP';
	} elsif(($sldiag_out =~ /Please choose/) or ($sldiag_out =~ /Please make/)) { #Added by pradeep to handle 10 mode 15122009
#If filer stuck in special boot menu
		return 'BOOT MENU';
	} elsif(($sldiag_out =~ /incorrect/i) or ($prompt =~ /login/i)){#PRUTHVI 
#If filer is at Login  prompt
		 say($conn,"netapp1!");
		 say($conn,"\r\r\n");
		 sleep 1;
		 return 'UP';
	}
	  elsif($sldiag_out =~ /sleeping for a bit/){
		sleep 10;
		say($conn, "netapp1!");
		return 'UP';
	}
           else {
#I dont know where the filer is landed
		logcomment (" get_filer_state else");
		return 'NOWHERE';
	}
}

#********************************************************************************************************************
sub set_boot_mode {

	tstlog -comment, " inside set_boot_mode";
	$current_mode=&get_filer_state;
	if($BOOT_MODE)
	{
		&boot_filer if(($BOOT_MODE ne $current_mode) or ($current_mode ne 'NOWHERE'));
	} else {
		if($current_mode eq 'FIRMWARE')
		{
			$BOOT_MODE='MAINT';
			&boot_filer 
		}
	}
}


#********************************************************************************************************************
sub boot_filer {
	$current_mode=&get_filer_state;
	tstlog -comment, "Inside boot_filer current_mode=$current_mode";
	say($conn,"halt", -prompt, "LOADER", -noprompt, -timeout, 300) if(($current_mode eq 'MAINT') or ($current_mode eq 'UP'));
	if($current_mode eq 'BOOT MENU') 
	{
		if ($ONTAP_MODE eq 10)#Added by pradeep to handle 10 mode 15122009
		{
			say($conn,"maint", '-wait',"WARNING",-noprompt);
			say($conn,"halt", -prompt, "LOADER", -noprompt, -timeout, 300);
		}
		else
		{
			say($conn,"5", '-wait',"Continue",-noprompt);
	        	say($conn,"yes");
			say($conn,"halt", -prompt, "LOADER", -noprompt, -timeout, 300);
		}
	} 
	
	$boot_cmd='boot_ontap';
	if($NETBOOT)
	{
		$boot_cmd="netboot $NETBOOT";
		say($conn,'ifconfig e0a -auto', -prompt, "LOADER");
	}
	if($BOOT_MODE eq 'MAINT')
	{
		tstlog -comment, "boot_filer MAINT checking";
		if ($ONTAP_MODE eq 10) #Added by pradeep to handle 10 mode 15122009
		{
	                say($conn,"$boot_cmd",'-wait', "Press Ctrl-C", -noprompt, -timeout, 500);
	                say($conn,"\cC", '-wait', "Selection", -noprompt, -timeout, 1000);
         	        say($conn,"5", 'wait',"WARNING", -noprompt, -timeout, 500);
		}
		else
		{
			tstlog -comment, "boot_filer else";
			say($conn,"$boot_cmd", -prompt, "Press Ctrl-C|Press CTRL-C", -timeout, 500);
			say($conn,"\cC", '-wait', "Selection", -noprompt, -timeout, 1000);
                	my (undef,undef,undef,$prompt) = say($conn,"5", -prompt,"Continue", -timeout, 500);
	                say($conn,"yes") if($prompt =~/Continue/);
		}
	} else {
		if($NETBOOT)
		{
			say($conn,"$boot_cmd", -prompt, "Selection", -noprompt, -timeout, 500);
			say($conn,"1",-timeout, 300);
		} else {
                        tstlog -comment, "Taking filer to NORMAL mode"; #Added by pradeep to handle filer to normal mode 15122009
			if ($ONTAP_MODE eq 10)
			{
				tstlog -comment, " Inside boot_filer cluster mode checking";
				my(undef,undef,$output,$prompt) = say($conn,"\n",-timeout,50);
				if ($prompt =~ /LOADER/i)
				{
					say($conn, "boot_ontap", '-wait', "login", -noprompt, -timeout, 500);
				 	my(undef,undef,$output,$prompt) = say($conn, "admin", -timeout, 50);
			         	if ($prompt =~ /Password/)
			         	{
		                 		say($conn, "netapp1!");
			         	}
				}
                                say($conn,"run local",-timeout,20);
			}
			else
			{
				my (undef,undef,undef,$prompt) = say($conn,"$boot_cmd", -prompt,"Password", -timeout, 500);
   				say($conn,"netapp1!") if ($prompt =~ /Password/);
			}
		} 	
	}
	
}

#**********************************************************************************************

sub compose_password
{
	tstlog -comment, "inside compose_password";
	sleep 5;
	say($conn,"\n",-swallow);
	(undef,undef,my $version_string,undef)=say($conn, "sldiag version",-swallow);
	$version_string=~/\w\.\d{2}\.\d\d/;
	my $vers_str1=$&;
	my $sub_string1=substr $vers_str1, 0, 1;
	my $sub_string2=substr $vers_str1, 2, 2;
	my $sub_string3=substr $vers_str1, 5, 2;
	$version_string=$sub_string1.$sub_string2.$sub_string3;	
	(undef,undef,my $date_string,undef)=say($conn, "sldiag date");
	tstlog -comment, "date is $date_string";
	$date_string =~ /:\d\d:/;
	$date_string=$&;
	$date_string=~/\d\d/;
	$date_string=$&;
	my $password_string="cty".$version_string."mfg".$date_string;
	tstlog -comment, "pass:$password_string";
	return $password_string;
}

#**********************************************************************************************************************
#Function Name  :analyze_test_status
#Parameters     :devtype, devname ,error_type_flag(yes or no)
#Return Type    : Returns nothing
#Description    : Will analyze the test status and prints passed,failed or aborted  
# example  analyze_test_status( "volume","vol0","yes");
#**********************************************************************************************************************

sub analyze_test_status 
{

  my($dev_type, $dev_name, $error_type_flag) = @_;
  
  if ($dev_type) 
  {
    switch ($dev_type)
    {
	case "volume"	{ analyze_vol_status ($dev_type, $dev_name , $error_type_flag) }
 	case "storage"  { tstlog -comment,"storage not implemented yet" }
	case "mem"      { tstlog -comment,"mem detailed status not implemented yet" }
        else            { tstlog -comment, "User didnot send any dev_type" }
    }

   } 
}
#display vol status alone
sub analyze_vol_status 
{
        tstlog -comment, "Inside analyze_test_status";
  my ($dev_type, $dev_name, $error_type_flag) = @_;
  my $output ;
  if ($dev_type && $dev_name) 
  {
        (undef,undef,$output,undef) = say($conn, "sldiag device status -dev $dev_type -name $dev_name");
        tstlog -comment, "both params passed DEV_TYP: $dev_type and DEV_NAM: $dev_name";
                if ($output =~ /ERROR: specified device could not be found or processed/) 
		{
                        return 306;
                }
  } else {
        (undef, undef , $output, undef ) = say( $conn, "sldiag device status -dev volume");
  }
    if ($output =~ /SLDIAG:\sNo\slog\smessages\sare\spresent/) 
    {
      tstlog -fail, "No Logs present for device VOLUME" ;
    } else {
      my @lines = split (/\d+\/\d+/ , $output);
        foreach my $line ( @lines ) 
        {
         #capturing device ,testname and status
         my ($dev, $test, $status) = $line =~ /(\w+).[\s|\t]+(FIO\sStress[\s\(th\d+\)]*)\s.+(passed|running|aborted|failed|pending|time|full|error|Completed|ERROR)/i;
         if($dev && $test && $status ) 
         {
                 if ($status =~ /PASSED/i) 
		 {
                 tstlog -pass ,"DEV:$dev TEST:$test STATUS:$status ";
		 } elsif ($status =~ /ERROR/i) {
                 tstlog -fail ,"DEV:$dev TEST:$test STATUS:$status ";
		 } elsif ( $status =~ /Aborted/i ) {
		         tstlog -comment, "DEV:$dev TEST:$test STATUS:$status";
		   }
         }
        }
      }

}

