package ssl_ca::third_party_ca;

use strict;
use warnings;
use base 'Exporter';
use File::Basename qw(dirname);
use Tharn qw($Log);
use NATE::Inc qw(find_file);


our @EXPORT = qw();
our @EXPORT_OK =
  qw( $verisign_cn $verisign_ca $verisign_cert $verisign_private_key @verisign_chain @verisign_sr_chain @verisign_cn_chain @verisign_ca_chain
  $godaddy_cn $godaddy_ca $godaddy_cert $godaddy_private_key @godaddy_chain @godaddy_sr_chain @godaddy_cn_chain @godaddy_ca_chain
  $thawte_cn $thawte_ca $thawte_cert $thawte_private_key @thawte_chain @thawte_sr_chain @thawte_cn_chain @thawte_ca_chain
  $networksolutions_cn $networksolutions_ca $networksolutions_cert $networksolutions_private_key @networksolutions_chain @networksolutions_sr_chain @networksolutions_cn_chain @networksolutions_ca_chain
);

my $path = dirname(find_file(__FILE__)) . '/certs';

sub read_cert_file {
    local $/ = undef;
    my $filepath = shift;
    if ( open( CERT, $filepath ) ) {
        my $value = <CERT>;
        close(CERT);
        return $value;
    }
    else {
        $Log->error("Error: could not open $filepath");
        return "";
    }
}

##################################################
########## Verisign Server Certificate ###########
##################################################

our $verisign_cn = "mot-cert-02.rtp.netapp.com";
our $verisign_ca = "\"VeriSign Class 3 Secure Server CA - G3\"";

my $certfile = $path . "/verisign/cert.pem";
our $verisign_cert = read_cert_file($certfile);

my $keyfile = $path . "/verisign/private_key.pem";
our $verisign_private_key = read_cert_file($keyfile);

my $rootcertfile = $path . "/verisign/rootcert.pem";
our $verisign_root_cert = read_cert_file($rootcertfile);

my $intercert1file = $path . "/verisign/intercert1.pem";
our $verisign_inter_cert1 = read_cert_file($intercert1file);

my $intercert2file = $path . "/verisign/intercert2.pem";
our $verisign_inter_cert2 = read_cert_file($intercert2file);

our @verisign_chain =
  ( $verisign_root_cert, $verisign_inter_cert1, $verisign_inter_cert2 );
our @verisign_sr_chain = (
    "3C9131CB1FF6D01B0E9AB8D044BF12BE",
    "6ECC7AA5A7032009B8CEBCF4E952D491",
    "1B093B786096DA37BBA4519446C89678"
);
our @verisign_cn_chain = (
    "Class3PublicPrimaryCertificationAuthority",
    "VeriSignClass3SecureServerCA-G3",
    "VeriSignClass3PublicPrimaryCertificationAuthority-G5"
);
our @verisign_ca_chain = (
    "\"VeriSign, Inc.\"",
    "\"VeriSign Class 3 Public Primary Certification Authority - G5\"",
    "\"VeriSign, Inc.\""
);

##################################################
########## GoDaddy Server Certificate ############
##################################################

our $godaddy_cn = "mot-cert-11.rtp.netapp.com";
our $godaddy_ca = "\"Go Daddy Secure Certification Authority\"";

$certfile = $path . "/godaddy/cert.pem";
our $godaddy_cert = read_cert_file($certfile);

$keyfile = $path . "/godaddy/private_key.pem";
our $godaddy_private_key = read_cert_file($keyfile);

$rootcertfile = $path . "/godaddy/rootcert.pem";
our $godaddy_root_cert = read_cert_file($rootcertfile);

$intercert1file = $path . "/godaddy/intercert1.pem";
our $godaddy_inter_cert1 = read_cert_file($intercert1file);

our @godaddy_chain =
  ( $godaddy_root_cert, $godaddy_inter_cert1 );

our @godaddy_sr_chain = ( "00", "0301" );

our @godaddy_cn_chain = (
    "GoDaddyClass2CertificationAuthority",
    "GoDaddySecureCertificationAuthority"
);
our @godaddy_ca_chain = (
    "\"The Go Daddy Group, Inc.\"",
    "\"The Go Daddy Group, Inc.\""
);

##################################################
########## Thawte Server Certificate #############
##################################################

our $thawte_cn = "mot-cert-01.rtp.netapp.com";
our $thawte_ca = "\"Thawte DV SSL CA\"";

$certfile = $path . "/thawte/cert.pem";
our $thawte_cert = read_cert_file($certfile);

$keyfile = $path . "/thawte/private_key.pem";
our $thawte_private_key = read_cert_file($keyfile);

$rootcertfile = $path . "/thawte/rootcert.pem";
our $thawte_root_cert = read_cert_file($rootcertfile);

$intercert1file = $path . "/thawte/intercert1.pem";
our $thawte_inter_cert1 = read_cert_file($intercert1file);

$intercert2file = $path . "/thawte/intercert2.pem";
our $thawte_inter_cert2 = read_cert_file($intercert2file);

our @thawte_chain =
  ( $thawte_root_cert, $thawte_inter_cert1, $thawte_inter_cert2 );
our @thawte_sr_chain = (
    "36122296C5E338A520A1D25F4CD70954",
    "3365500879AD73E230B9E01D0D7FAC91",
    "7610128A17B682BB3A1F9D1A9A35C092"
);
our @thawte_cn_chain =
  ( "ThawtePremiumServerCA", "thawtePrimaryRootCA", "ThawteDVSSLCA" );
our @thawte_ca_chain = (
    "\"Thawte Premium Server CA\"",
    "\"Thawte Premium Server CA\"",
    "\"thawte Primary Root CA\""
);

##########################################################
########## NetworkSolutions Server Certificate ###########
##########################################################

our $networksolutions_cn = "mot-cert-02.rtp.netapp.com";
our $networksolutions_ca = "\"Network Solutions Certificate Authority\"";

$certfile = $path . "/networksolutions/cert.pem";
our $networksolutions_cert = read_cert_file($certfile);

$keyfile = $path . "/networksolutions/private_key.pem";
our $networksolutions_private_key = read_cert_file($keyfile);

$rootcertfile = $path . "/networksolutions/rootcert.pem";
our $networksolutions_root_cert = read_cert_file($rootcertfile);

$intercert1file = $path . "/networksolutions/intercert1.pem";
our $networksolutions_inter_cert1 = read_cert_file($intercert1file);

$intercert2file = $path . "/networksolutions/intercert2.pem";
our $networksolutions_inter_cert2 = read_cert_file($intercert2file);

our @networksolutions_chain = (
    $networksolutions_root_cert, 
    $networksolutions_inter_cert1,
    $networksolutions_inter_cert2
);
our @networksolutions_sr_chain = (
    "01",
    "484BACF1AAC7D71343D1A27435499725",
    "10E776E8A65A6E377E050306D43C25EA"
);
our @networksolutions_cn_chain = (
    "AddTrustExternalCARoot", 
    "UTN-USERFirst-Hardware",
    "NetworkSolutionsCertificateAuthority"
);
our @networksolutions_ca_chain = (
    "\"AddTrust External CA Root\"",
    "\"AddTrust External CA Root\"",
    "\"UTN-USERFirst-Hardware\""
);

1;
