# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Metrocluster Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Metrocluster

=head1 DESCRIPTION

C<NACL::C::Metrocluster> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP Metrocluster.  A related class is
L<NACL::CS::Metrocluster|lib-NACL-CS-Metrocluster-pm>, which represents
the past state of an ONTAP Metrocluster.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::Metrocluster;

use strict;
use warnings;

use NACL::CS::Metrocluster;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use Tharn qw($Log);

=head1 METHODS

=head2 new

  my $Metrocluster = NACL::C::Metrocluster->new(
                                   command_interface => $ci,
								  );

Return a new Metrocluster component object with the given attributes.

=cut

=head2 switchback

    NACL::C::Metrocluster->switchback(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to switch back storage and client access.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'override-vetoes' => $string >>

Applicable for CMode CLI.

=item C<< 'f' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub switchback {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( override-vetoes f
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_switchback_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit();
} ## end sub switchback

sub _switchback_cmode_cli {
    $Log->enter();
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( override-vetoes f
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_switchback',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_switchback(%cmode_args);
    $Log->exit();
} ## end sub _switchback_cmode_cli

=head2 switchover

    NACL::C::Metrocluster->switchover(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
   
(Class method)
This method is used to switch over storage and client access.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'node-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'force-nvfail-all' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'force' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=item C<< 'retry-failed-nodes' => [ $retry-failed-node1, $retry-failed-node2, ... ] >>

Nodes to Switchover
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< 'override-vetoes' => $string >>

Override All Vetoes
Applicable for CMode CLI

=item C<< 'forced-on-disaster' => $string >>

Force a switchover on disaster
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub switchover {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            "retry-failed-nodes" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( node-uuid node force-nvfail-all

                    force

                    override-vetoes forced-on-disaster
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_switchover_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit();
} ## end sub switchover

sub _switchover_cmode_cli {
    $Log->enter();
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node-uuid node force-nvfail-all

            force
            extended_query
            override-vetoes forced-on-disaster

            retry-failed-nodes)
    ];
    my %cmode_args;
    foreach my $array_element (qw(retry-failed-nodes)) {
        if (defined $opts{$array_element}) {
            $opts{$array_element} = join ',', @{$opts{$array_element}};
        }
    }

    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_switchover',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_switchover(%cmode_args);
    $Log->exit();
} ## end sub _switchover_cmode_cli

=head2 assign

    NACL::C::Metrocluster->assign(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
   
(Class method)
This method is used to assign a node to a location.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'node-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'location' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub assign {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( node-uuid location node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_assign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit();
} ## end sub assign

sub _assign_cmode_cli {
    $Log->enter();
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node-uuid location node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_assign',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_assign(%cmode_args);
    $Log->exit();
} ## end sub _assign_cmode_cli

=head2 check

    NACL::C::Metrocluster->check(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to check the MetroCluster setup.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'verbose' => $string >>

Applicable for CMode CLI.

=item C<< 'level' => $string >>

Applicable for CMode CLI.

=item C<< 'break-on-failure' => $string >>

Applicable for CMode CLI.

=item C<< 'precondition-for' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub check {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( verbose level break-on-failure precondition-for
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_check_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit();
} ## end sub check

sub _check_cmode_cli {
    $Log->enter();
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( verbose level break-on-failure precondition-for
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_check',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_check(%cmode_args);
    $Log->exit();
} ## end sub _check_cmode_cli

=head2 heal

    NACL::C::Metrocluster->heal(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to heal aggregates and partner roots.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'phase' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=item C<< 'bypass-nodes-not-switched-over' => $string >>


Allow heal aggr operation even though partially switched over, only switched over nodes participates the operation
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub heal {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( phase

                    bypass-nodes-not-switched-over
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_heal_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit();
} ## end sub heal

sub _heal_cmode_cli {
    $Log->enter();
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( phase
            extended_query
            bypass-nodes-not-switched-over
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_heal',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_heal(%cmode_args);
    $Log->exit();
} ## end sub _heal_cmode_cli

=head2 disable

    NACL::C::Metrocluster->disable(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
   
(Class method)
This method is used to disable mirroring between DR partners.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'node-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( node-uuid node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_disable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit();
} ## end sub disable

sub _disable_cmode_cli {
    $Log->enter();
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node-uuid node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_disable',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_disable(%cmode_args);
    $Log->exit();
} ## end sub _disable_cmode_cli

=head2 wipeconfig

    NACL::C::Metrocluster->wipeconfig(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
   
(Class method)
This method is used to remove the specified node and its HA, DR, and DR-auxiliary partners from the MetroCluster configuration.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'node-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub wipeconfig {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( node-uuid node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_wipeconfig_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit();
} ## end sub wipeconfig

sub _wipeconfig_cmode_cli {
    $Log->enter();
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node-uuid node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_wipeconfig',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_wipeconfig(%cmode_args);
    $Log->exit();
} ## end sub _wipeconfig_cmode_cli

=head2 modify

    NACL::C::Metrocluster->modify(
                                command_interface => $ci,
	                            'cluster' => $cluster,
	                            'node' => $node,
	                            'cluster-ordinal' => $cluster_ordinal,
                                ...);
	                             
								 or

    $Metrocluster_obj->modify(...);

(Class or instance method)

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cluster' => $cluster >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'cluster-ordinal' => $cluster_ordinal >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume-count-limit' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'cluster' => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< 'cluster-ordinal' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=item C<< 'node-name' => $string >>

Node to Change the Option on possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'node-object-limit' => $string >>

Refresh Configuration possible value(s) are, on,off
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter();
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( volume-count-limit

                    cluster node cluster-ordinal

                    node-name

                    node-object-limit
                    )
            ),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit();
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter();
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( cluster volume-count-limit node cluster-ordinal

            cluster node cluster-ordinal
            extended_query
            node-name

            node-object-limit
            )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'metrocluster_modify',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_modify(%cmode_args);
    $Log->exit();
} ## end sub _modify_cmode_cli

=head2 enable

    NACL::C::Metrocluster->enable(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
   
(Class method)
This method is used to enable mirroring between DR partners.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'node-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'refresh' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node-name' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub enable {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( node-uuid refresh node

                    node-name
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_enable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit();
} ## end sub enable

sub _enable_cmode_cli {
    $Log->enter();
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node-uuid refresh node

            node-name
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_enable',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_enable(%cmode_args);
    $Log->exit();
} ## end sub _enable_cmode_cli

=head2 unassign

    NACL::C::Metrocluster->unassign(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
   
(Class method)
This method is used to unassign a node from its assigned location.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'node-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string>>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub unassign {
    $Log->enter();
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( node-uuid node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_unassign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit();
} ## end sub unassign

sub _unassign_cmode_cli {
    $Log->enter();
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node-uuid node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_unassign',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_unassign(%cmode_args);
    $Log->exit();
} ## end sub _unassign_cmode_cli

=head2 configure

    NACL::C::Metrocluster->configure(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to configure partners and start DR mirroring for the node.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'refresh' => $string >>

Refresh Configuration possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'node-name' => $string >>

Node to Configure possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "metrocluster configure" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub configure {
    $Log->enter();
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( refresh node-name
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'metrocluster_configure',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_configure_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Metrocluster::configure',
    );

}

sub _configure_cmode_cli {
    $Log->enter();
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'metrocluster_configure',);
    $Log->exit();
}

=head2 unconfigure

    NACL::C::Metrocluster->unconfigure(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
   
(Class method)
This method is used to remove the specified node and its HA, DR, and DR-auxiliary partners from the MetroCluster configuration.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)
Node to Unconfigure
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "metrocluster unconfigure" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub unconfigure {
    $Log->enter();
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'metrocluster_unconfigure',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_unconfigure_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Metrocluster::unconfigure',
    );

}

sub _unconfigure_cmode_cli {
    $Log->enter();
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'metrocluster_unconfigure',);
    $Log->exit();
}

sub _primary_keys_validate_spec {
    return (
        'cluster' => {type => SCALAR},
        'node'    => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

sub get_one_state_attribute {
    $Log->enter();
    my $error
        = "Since NACL::C::Metrocluster doesn't have any primary key attributes, "
        . "use \"fetch\" instead of \"get_one_state_attribute\" to get the attribute";
    $Log->exit();
    NATE::BaseException->throw($error);
}

sub state {
    $Log->enter();
    my $error
        = "Since NACL::C::Metrocluster doesn't have any primary key attributes, "
        . "use \"fetch\" instead of \"state\"";
    $Log->exit();
    NATE::BaseException->throw($error);
}

sub find {
    $Log->enter();
    my $error
        = "Since NACL::C::Metrocluster doesn't have any primary key attributes, "
        . "use \"fetch\" instead of \"find\"";
    $Log->exit();
    NATE::BaseException->throw($error);
}

1;
