# $Id: //depot/prod/test/Rsierranevada/lib/torture_lib.pm#1 $

## @summary     Defines random RAID fiji filters for disk failed or offline torture test.
## @description Defines random RAID fiji filters on various fiji points for disk failed or offline torture test. 
## raidv2_generate_filter() is invoked from the torture test scripts to generate random fiji filters for various fiji points. 
## Combines previous torture_lib and torture_lib_do (disk offline) -- only difference was disk error type and few editing errors.
## Disk errors are E_DISK_FAILED to fail disk, or E_DISK_AGRSV_TIMEOUT to cause disk offline.
## #author kmacy@netapp.com
## @author schulz@netapp.com, dl-spe-tag@netapp.com

package torture_lib;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = (raidv2_generate_filter, TORTURE_DEBUG);
use strict;

# Old comments:
# injection points that need to be preceeded by errors:  "raid.disk.read.retry.done", "raid.readerr.child.done"
# add specification of what events to preceed what rules
# don't have more than one outstanding disk fail on a raidgroup at one time
# set rules up so that events are likely to occur
# unused for raid.tetrisio.event "err_handle"?

# Called by raidv2_generate_filter() to pick random points with their variations.
sub raidv2_pick_injection_point($$$) {
	my ($sim, $type, $disk_error) = @_;	# is sim, torture type, disk error
	
	# Failures that can be scoped within a given io-type
	#	my @SCOPABLE_INJECTION_POINTS = ( [ "raid.disk.read.done", "rw", "disk" ],
	#					  [ "raid.disk.write.done", "rw", "disk" ],
	#[ "raid.cksum.verify.buf", "rw" ], takes a different set of error codes
	#					);
	#	my @NONSCOPABLE_INJECTION_POINTS = ( [ "raid.label.read.done", "volop", "disk" ],
	#					     [ "raid.label.write.done", "volop", "disk" ]
	#					     );

	# these are not used, what are they
	my @RETRY_INJECTION_POINTS = ( 
		[ "raid.diskio.event",          [ "read_retry" ],    "rw", "disk" ],
		[ "raid.readerr.event",         [ "readerr_start" ], "rw", "rgid" ],
		[ "raid.readerr.child.done",    [ ],                 "rw", "disk" ],
		[ "raid.mirror.readerr.event",  [ "readerr_start" ], "rw", "rgid" ],
		[ "raid.tetrisio.event",        [ "err_handle" ],    "rw", "disk" ]
		);
		#tetrisio.event "rmt_clear_done", "xor_done", "rmt_write_done", "pzero_log", "nvram_commit", 
		# unused "wack_plex_mismatch", "err_handle"

	# Could probably combine SIM and non-SIM INJECTION_POINTS someday
	
	my @SIM_MIRRORED_INJECTION_POINTS = ( 
		[ "raid.tetrisio.event",   [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.tetrisio.event",   [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull" ]],"scoping", "disk" ],
		[ "raid.mirrorio.event",   [ "readio_start", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirrorio.event",   [ "verify", "verify", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]],    "scoping", "disk" ],
		[ "raid.scrubio.event",    [ "start_reads", "scrub", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]],  "scoping", "disk" ],
		[ "raid.scrubio.event",    [ "start_writes", "scrub", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.rgio.event",       [ "readio_start",  "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
#		[ "raid.reparityio.event", [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ] ],  "scoping", "disk" ],
#		[ "raid.reparityio.event", [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.verifyio.event", [ "start_reads", "verify", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.verifyio.event", [ "start_writes", "reconstruct", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		# the first can only be exercised when reconstruct is going on on another disk add can_fatal field?
#		[ "raid.reconsio.event", [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.reconsio.event", [ "start_writes", "reconstruct", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		# this one is also complicated because it is only going to be writing to the destination plex
#		[ "raid.mirror.resyncio.event", [ "start_reads", "resync", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.resyncio.event", [ "start_writes", "resync", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.disk.read.done", [ "rw", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "sideaffect", "disk" ],
		[ "raid.disk.write.done", [ "rw", [ "$disk_error", "disk.simpull", "disk.offline" ]], "sideaffect", "disk" ],
		[ "raid.label.read.done", [ "volop", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "sideaffect", "disk" ],
		[ "raid.label.write.done", [ "volop", [ "$disk_error", "disk.simpull", "disk.offline" ]], "sideaffect", "disk" ],
#		[ "raid.disk.unfail.done", [ "unfail" ], "sideaffect", "disk" ],
#		[ "raid.cksum.event",      [ "cksum_rmt_write_done" ], "rw" ],	
#		[ "raid.cksum.buf.read.done", [ "cksum_rmt_write_done", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.diskio.event", [ "read_retry", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN","$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		# ask if this is in a read context
		[ "raid.mirrorio.event", [ "err_handle", "rw", "raid.disk.write.done",[ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirrorio.event", [ "wack_plex_mismatch", "rw", "raid.disk.read.done",[ "E_DISK_REASSIGN","$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.readerr.event", [ "readerr_start", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.stripereplayio.event", [ "start_parity_writes", "rw", "raid.disk.write.done",[ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		# what context should this happen in?
		[ "raid.tetrisio.event",   [ "nvram_commit", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",   [ "err_handle", "rw", "raid.disk.read.done", [ "$disk_error", "disk.simpull", "disk.offline", "E_DISK_RECOMMEND_REASSIGN" ]],"scoping", "disk" ],
		# read or write ?
		[ "raid.tetrisio.event",   [ "rmt_clear_done", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.mirror.resyncio.event",[ "start_reads", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.resyncio.event",[ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.mirror.cpio.event",    [ "start_reads", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.cpio.event",    [ "start_writes", "rw", "raid.disk.write.done",[ "$disk_error", "disk.simpull", "disk.offline" ] ], "scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "xor_done", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "pzero_log", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		);

	# @MIRRORED_INJECTION_POINTS for non-sim were not defined:  copied @SIM_MIRRORED_INJECTION_POINTS, removed disk.simpull's 
	my @MIRRORED_INJECTION_POINTS = ( 
		[ "raid.tetrisio.event",   [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.tetrisio.event",   [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error" ]],"scoping", "disk" ],
		[ "raid.mirrorio.event",   [ "readio_start", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirrorio.event",   [ "verify", "verify", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]],    "scoping", "disk" ],
		[ "raid.scrubio.event",    [ "start_reads", "scrub", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]],  "scoping", "disk" ],
		[ "raid.scrubio.event",    [ "start_writes", "scrub", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.rgio.event",       [ "readio_start",  "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "scoping", "disk" ],
#		[ "raid.reparityio.event", [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ] ],  "scoping", "disk" ],
#		[ "raid.reparityio.event", [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.verifyio.event", [ "start_reads", "verify", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.verifyio.event", [ "start_writes", "reconstruct", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		# the first can only be exercised when reconstruct is going on on another disk add can_fatal field?
#		[ "raid.reconsio.event", [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.reconsio.event", [ "start_writes", "reconstruct", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		# this one is also complicated because it is only going to be writing to the destination plex
#		[ "raid.mirror.resyncio.event", [ "start_reads", "resync", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.resyncio.event", [ "start_writes", "resync", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.disk.read.done", [ "rw", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "sideaffect", "disk" ],
		[ "raid.disk.write.done", [ "rw", [ "$disk_error", "disk.offline" ]], "sideaffect", "disk" ],
		[ "raid.label.read.done", [ "volop", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "sideaffect", "disk" ],
		[ "raid.label.write.done", [ "volop", [ "$disk_error", "disk.offline" ]], "sideaffect", "disk" ],
#		[ "raid.disk.unfail.done", [ "unfail" ], "sideaffect", "disk" ],
#		[ "raid.cksum.event",      [ "cksum_rmt_write_done" ], "rw" ],	
#		[ "raid.cksum.buf.read.done", [ "cksum_rmt_write_done", "rw", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.diskio.event", [ "read_retry", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN","$disk_error", "disk.offline" ]], "scoping", "disk" ],
		# ask if this is in a read context
		[ "raid.mirrorio.event", [ "err_handle", "rw", "raid.disk.write.done",[ "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirrorio.event", [ "wack_plex_mismatch", "rw", "raid.disk.read.done",[ "E_DISK_REASSIGN","$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.readerr.event", [ "readerr_start", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.stripereplayio.event", [ "start_parity_writes", "rw", "raid.disk.write.done",[ "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		# what context should this happen in?
		[ "raid.tetrisio.event",   [ "nvram_commit", "rw", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",   [ "err_handle", "rw", "raid.disk.read.done", [ "$disk_error", "disk.offline", "E_DISK_RECOMMEND_REASSIGN" ]],"scoping", "disk" ],
		# read or write ?
		[ "raid.tetrisio.event",   [ "rmt_clear_done", "rw", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.mirror.resyncio.event",[ "start_reads", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.resyncio.event",[ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.mirror.cpio.event",    [ "start_reads", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.mirror.cpio.event",    [ "start_writes", "rw", "raid.disk.write.done",[ "$disk_error", "disk.offline" ] ], "scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "xor_done", "rw", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "pzero_log", "rw", "raid.disk.write.done", [ "$disk_error", "disk.offline" ]],"scoping", "disk" ],
		);

	my @SIM_INJECTION_POINTS = ( 
		[ "raid.tetrisio.event",   [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.tetrisio.event",   [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.scrubio.event",    [ "start_reads", "scrub", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]],  "scoping", "disk" ],
		[ "raid.scrubio.event",    [ "start_writes", "scrub", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.rgio.event",       [ "readio_start",  "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
#		[ "raid.reparityio.event", [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ] ],  "scoping", "disk" ],
#		[ "raid.reparityio.event", [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		# the first can only be exercised when reconstruct is going on on another disk add can_fatal field?
#		[ "raid.reconsio.event", [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.reconsio.event", [ "start_writes", "reconstruct", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		# this one is also complicated because it is only going to be writing to the destination plex
#		[ "raid.disk.read.done", [ "rw", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "sideaffect", "disk" ],
		[ "raid.disk.write.done", [ "rw", [ "$disk_error", "disk.simpull", "disk.offline" ]], "sideaffect", "disk" ],
		[ "raid.label.read.done", [ "volop", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "sideaffect", "disk" ],
		[ "raid.label.write.done", [ "volop", [ "$disk_error", "disk.simpull", "disk.offline" ]], "sideaffect", "disk" ],
#		[ "raid.disk.unfail.done", [ "unfail" ], "sideaffect", "disk" ]
		#[ "raid.cksum.event",      [ "cksum_rmt_write_done" ], "rw" ],	
#		[ "raid.cksum.buf.read.done", [ "cksum_rmt_write_done", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.diskio.event", [ "read_retry", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN","$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		# ask if this is in a read context
		[ "raid.mirrorio.event", [ "err_handle", "rw", "raid.disk.write.done",[ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.stripereplayio.event", [ "start_parity_writes", "rw", "raid.disk.write.done",[ "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		# what context should this happen in?
		[ "raid.tetrisio.event",   [ "nvram_commit", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",   [ "err_handle", "rw", "raid.disk.read.done", [ "$disk_error", "disk.simpull", "disk.offline", "E_DISK_RECOMMEND_REASSIGN" ]],"scoping", "disk" ],
		# read or write ?
		[ "raid.tetrisio.event",   [ "rmt_clear_done", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error", "disk.simpull", "disk.offline" ]], "scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "xor_done", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",       [ "pzero_log", "rw", "raid.disk.write.done", [ "$disk_error", "disk.simpull", "disk.offline" ]],"scoping", "disk" ],
		);

	my @INJECTION_POINTS = ( 
#		[ "raid.cksum.buf.read.done", [ "cksum_rmt_write_done", "rw", "raid.disk.write.done", [ "$disk_error" ]],"scoping", "disk" ],
		[ "raid.diskio.event", [ "read_retry", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN","$disk_error" ]], "scoping", "disk" ],
		# ask if this is in a read context
		[ "raid.mirrorio.event", [ "err_handle", "rw", "raid.disk.write.done",[ "$disk_error" ]], "scoping", "disk" ],
		[ "raid.mirrorio.event", [ "wack_plex_mismatch", "rw", "raid.disk.read.done",[ "E_DISK_REASSIGN","$disk_error" ]], "scoping", "disk" ],
		[ "raid.mirror.readerr.event", [ "readerr_start", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]], "scoping", "disk" ],
		[ "raid.stripereplayio.event", [ "start_parity_writes", "rw", "raid.disk.write.done",[ "$disk_error" ]], "scoping", "disk" ],
		# what context should this happen in?
		[ "raid.tetrisio.event",   [ "nvram_commit", "rw", "raid.disk.write.done", [ "$disk_error" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",   [ "err_handle", "rw", "raid.disk.read.done", [ "$disk_error", "E_DISK_RECOMMEND_REASSIGN" ]],"scoping", "disk" ],
		# read or write ?
		[ "raid.tetrisio.event",   [ "rmt_clear_done", "rw", "raid.disk.write.done", [ "$disk_error" ]],"scoping", "disk" ],
		[ "raid.mirror.resyncio.event",   [ "start_reads", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]], "scoping", "disk" ],
		[ "raid.mirror.resyncio.event",   [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error" ]],"scoping", "disk" ],
		[ "raid.mirror.cpio.event",   [ "start_reads", "rw", "raid.disk.read.done",[ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]], "scoping", "disk" ],
		[ "raid.mirror.cpio.event",   [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error" ]], "scoping", "disk" ],
		[ "raid.tetrisio.event",      [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]], "scoping", "disk" ],
		[ "raid.tetrisio.event",      [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",      [ "xor_done", "rw", "raid.disk.write.done", [ "$disk_error" ]],"scoping", "disk" ],
		[ "raid.tetrisio.event",      [ "pzero_log", "rw", "raid.disk.write.done", [ "$disk_error" ]],"scoping", "disk" ],
		[ "raid.mirrorio.event",   [ "readio_start", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]], "scoping", "disk" ],
		[ "raid.mirrorio.event",   [ "verify", "verify", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]],    "scoping", "disk" ],
		[ "raid.scrubio.event",    [ "start_reads", "scrub", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]],  "scoping", "disk" ],
		[ "raid.scrubio.event",    [ "start_writes", "scrub", "raid.disk.write.done", [ "$disk_error" ]], "scoping", "disk" ],
		[ "raid.rgio.event",       [ "readio_start",  "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]], "scoping", "disk" ],
#		[ "raid.reparityio.event", [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ] ],  "scoping", "disk" ],
#		[ "raid.reparityio.event", [ "start_writes", "rw", "raid.disk.write.done", [ "$disk_error" ]], "scoping", "disk" ],
		[ "raid.mirror.verifyio.event", [ "start_reads", "verify", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]], "scoping", "disk" ],
		[ "raid.mirror.verifyio.event", [ "start_writes", "reconstruct", "raid.disk.write.done", [ "$disk_error" ]], "scoping", "disk" ],
		# the first can only be exercised when reconstruct is going on on another disk add can_fatal field?
#		[ "raid.reconsio.event", [ "start_reads", "rw", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]], "scoping", "disk" ],
		[ "raid.reconsio.event", [ "start_writes", "reconstruct", "raid.disk.write.done", [ "$disk_error" ]], "scoping", "disk" ],
# no mirror events or actions on non-mirrored
#		# this one is also complicated because it is only going to be writing to the destination plex
#		[ "raid.mirror.resyncio.event", [ "start_reads", "resync", "raid.disk.read.done", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]], "scoping", "disk" ],
#		[ "raid.mirror.resyncio.event", [ "start_writes", "resync", "raid.disk.write.done", [ "$disk_error" ]], "scoping", "disk" ],
		[ "raid.disk.read.done", [ "rw", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]], "sideaffect", "disk" ],
		[ "raid.disk.write.done", [ "rw", [ "$disk_error" ]], "sideaffect", "disk" ],
		[ "raid.label.read.done", [ "volop", [ "E_DISK_RECOMMEND_REASSIGN", "$disk_error" ]], "sideaffect", "disk" ],
		[ "raid.label.write.done", [ "volop", [ "$disk_error" ]], "sideaffect", "disk" ],
#		[ "raid.disk.unfail.done", [ "unfail" ], "sideaffect", "disk" ]
#		[ "raid.cksum.event",      [ "cksum_rmt_write_done" ], "rw" ],	
		);

	my($filter_info_ref) = "";
	my $random_fiji;
	if ($sim && $type eq "MIRRORED") {
		$random_fiji = int(rand($#SIM_MIRRORED_INJECTION_POINTS + 1));
		$filter_info_ref = $SIM_MIRRORED_INJECTION_POINTS[$random_fiji];
		#print "\$random_fiji is $random_fiji out of " . (scalar @SIM_MIRRORED_INJECTION_POINTS) . "\n";
	} elsif (!$sim && $type eq "MIRRORED") {
		$random_fiji = int(rand($#MIRRORED_INJECTION_POINTS + 1));
		$filter_info_ref = $MIRRORED_INJECTION_POINTS[$random_fiji];
		#print "\$random_fiji is $random_fiji out of " . (scalar @MIRRORED_INJECTION_POINTS) . "\n";
	} elsif ($sim) {
		$random_fiji = int(rand($#SIM_INJECTION_POINTS + 1));
		$filter_info_ref = $SIM_INJECTION_POINTS[$random_fiji];
		#print "\$random_fiji is $random_fiji out of " . (scalar @SIM_INJECTION_POINTS) . "\n";
	} else {
		$random_fiji = int(rand($#INJECTION_POINTS + 1));
		$filter_info_ref = $INJECTION_POINTS[$random_fiji];
		#print "\$random_fiji is $random_fiji out of " . (scalar @INJECTION_POINTS) . "\n";
	}
	print "$sim, $type \n";
	my @filter_info = @$filter_info_ref;
	if ($#filter_info != 3) {
		die "bad filter info, last index is $#filter_info, expected 3";
		#tstlog -fatal, "bad filter info, length is $#filter_info, expected 4";
	}
	return @filter_info;
}

# Selects random fiji, returns raid action and anonymous subroutine reference for inserting current vol and disk into filter. 
sub raidv2_generate_filter($$$) {
	my ($sim, $type, $disk_error) = @_;	# is sim, torture type, disk error
	my ($ip, $point_info, $category, $target_type) = raidv2_pick_injection_point($sim, $type, $disk_error);
	my ($filter_str, @combo)  = ("", (), ());	# $filter_str not used
	
	my $RAIDV2_RAID4_GROUP_CHECK = "(eq (mon_state (parent_id \@obj disk) group) NORMAL)";
	my $RAIDV2_RAIDRD_GROUP_CHECK = "(|| (eq (mon_state (parent_id \@obj disk) group) NORMAL) (eq (mon_state (parent_id \@obj disk) group) DEGRADED))"; 
	my $RAIDV2_MIRRORED_CHECK =   "(eq (mon_state (parent_id (parent_id (parent_id \@obj disk) group) plex) mirror) NORMAL)";
	my $CHECK;
	if ($type eq "MIRRORED") {
		$CHECK = $RAIDV2_MIRRORED_CHECK;
	} elsif ($type eq "RAIDRD") {
		$CHECK = $RAIDV2_RAIDRD_GROUP_CHECK;
	} else {
		$CHECK = $RAIDV2_RAID4_GROUP_CHECK;
	}
	
	# Create anonymous subroutine for inserting volume and disk phrases into filter.
	if ($category eq "scoping") {
		my ($type, $action, $sa_injection_point, $errors) = @$point_info; 	# fiji point param @type, raid action, fiji point, errors
		my @errorlist = @$errors;
		my $error_code = $errorlist[int(rand($#errorlist + 1))];
		my $filter_gen = sub($$) {
			my ($volume, $disk) = @_;
			my %target_name_by_type = ();
			$target_name_by_type{"vol"} = $volume;
			$target_name_by_type{"disk"} =  $disk; 
			my $scope_filter = "($ip (&& (eq \@type $type) (eq \@$target_type $target_name_by_type{$target_type})) (nil))";
			my $sa_filter = "";
			if ($error_code eq "disk.simpull" || $error_code eq "disk.offline") {
				$sa_filter = "($sa_injection_point (&& (eq \@disk $disk) $CHECK) ($error_code $disk))";
			} else {
				$sa_filter = "($sa_injection_point (&& (eq \@disk $disk) $CHECK) (return (buf_Errno $error_code)))";
			}
			return "(scope raidiosubs (nil) $scope_filter $sa_filter)";
		};
		push @combo, $action;
		push @combo, $filter_gen;
	} elsif ($category eq "sideaffect") {
		my ($action, $errors) = @$point_info;	# raid action, errors
		my @errorlist = @$errors;
		my $error_code = $errorlist[int(rand($#errorlist + 1))];
		my $filter_gen = sub($$) {
			my ($volume, $disk) = @_;
			my %target_name_by_type = ();
			$target_name_by_type{"vol"} = $volume;
			$target_name_by_type{"disk"} =  $disk; 
			my $sa_filter = "";
			my $in_exp =  "(eq \@$target_type $target_name_by_type{$target_type})"; 
			my $cexp = $in_exp;
			if ($target_type eq "disk") {
				$cexp = "(&& $CHECK $in_exp)";
			}
			if ($error_code eq "disk.simpull" || $error_code eq "disk.offline") {
				#$sa_filter = "($ip $cexp (disk.simpull $disk))"; 
				$sa_filter = "($ip $cexp ($error_code $disk))"; 
			} else {
				$sa_filter = "($ip $cexp (return (buf_Errno $error_code)))"; 
			}
			return $sa_filter;
		};
		push @combo, $action;
		push @combo, $filter_gen;
	} else {
		die "major badness";
		#tstlog -fatal, "bad category $category";
	}
	
	return @combo;
}
