package CR::uACLCommonParams;

## @summary
##  Common parameters used in uACL tests.

use Params::Common;
use Options;

my $Options = new Options;
$Options->parameters(
    duplication_ok => 1,
    definitions => [
        {   name          => "TIMEOUT",
            description   => "Set the command timeout value",
            spec          => "s",
            default       => 600,
            greater_equal => 0,
        },
        {   name        => "VSERVER",
            description => "Vserver name",
            spec        => "s",
            default     => "vs0",
        },
        {   name        => "FLEXVOL_VSERVERS",
            description => "Flex Volume Vserver name",
            spec        => "s",
            default     => "vs1",
        },
        {   name        => "FLEXCACHE_VSERVERS",
            description => "Flex Cache Volume Vserver Name",
            spec        => "s",
            default     => "vs2",
        },
        {   name        => "CVOL_JUNCTION_PATH",
            description => "Junction path for the InfiniteVol",
            spec        => "s",
            default     => "/NS",
        },
        {   name         => "NFS_PROTOCOL",
            description  => "List of NFS protocols to enable",
            spec         => "s@",
            default      => ["v3"],
            valid_values => [ "v3", "v4.0", "v4.1", "v4.1-pnfs" ],
        },
        {   name         => "V4_ACL",
            description  => "NFSv4.0 ACL Support",
            spec         => "s",
            default      => "disabled",
            valid_values => [ "enabled", "disabled" ],
        },
        {   name        => "V4_ID_DOMAIN",
            description => "NFSv4 ID Mapping Domain",
            spec        => "s",
            default     => "WIN.SMOKE",
        },
        {   name         => "V4_1_ACL",
            description  => "NFSv4.1 ACL Support",
            spec         => "s",
            default      => "disabled",
            valid_values => [ "enabled", "disabled" ],
        },
        {   name => "AD_SERVER_USER",
            description =>
                "AD server admin name, needed for remove cifs vserver",
            spec    => "s",
            default => "adadmin",
        },
        {   name => "AD_SERVER_PASSWORD",
            description =>
                "AD server admin password, needed for remove cifs vserver",
            spec    => "s",
            default => "adadmin",
        },
        {   name        => "CIFS_DOMAIN",
            description => "Domain name for CIFS server",
            spec        => "s",
            default     => "CNC",
        },
        {   name        => "CIFS_DOMAIN_LONG",
            description => "Domain name for CIFS server",
            spec        => "s",
            default     => "CNC.SPIN.ENG.NETAPP.COM",
        },
        {   name => "AD_SERVER_IP",
            description =>
                "List of IP addresss of the Active Directory servers",
            spec    => "s@",
            default => [ "10.98.203.74", "10.98.202.208" ],
        },
        {   name        => "CIFS_SERVER",
            description => "CIFS server name",
            spec        => "s",
        },
        {   name           => "CIFS_SHARE",
            description    => "CIFS SHARE name",
            spec           => "s",
            default        => "cifs_share",
        },
        {   name        => "CIFS_SUPERUSERS",
            description => "CIFS Super User",
            spec        => "s@",
            default     => [],
        },
        {   name        => "LOCAL_UNIX_USERS",
            description => "Local unix user",
            spec        => "s",
            default =>
                'root:0:0:"Local Root",nobody:65534:65534:nobody,smoke:18811:30:smoke',
        },
        {   name        => "LOCAL_UNIX_GROUPS",
            description => "Local unix group",
            spec        => "s",
            default     => 'root:0,nobody:65534,engr:30',
        },
        {
            name        => "NS_SWITCH",
            description => "NS Switches",
            spec        => "s@",
            default     => ['file','nis'],
        },
        {   name => "DEBUG_CMD",
            description =>
                "command for gathering debug info in a diag_post_test",
            spec => "s",
        },
        {
            name        => 'CR_CIFS_SMB_SESSION',
            description => 'CIFS SMB session for which test should be run.'
                           . ' Valid values are: SMB1, SMB2, SMB2_1, SMB3.',
            default     => 'SMB2_1',
        },
        {  name => 'NIS_DOMAIN',
            description =>
                'NIS Domain name, if not provide, will try to get it from site info files',
            spec    => 's',
            default => 'netapp.com',
        },
        {  name => 'NIS_SERVERS',
            description =>
                'NIS Domain Servers IP, if not provide, will try to get it from site info files',
            spec    => 's@',
            default => ['10.60.132.113'],
        },
        {   name        => 'CR_CIFS_SHARE',
            description => 'CIFS SHARE name',
            spec        => 's',
            default     => 'cifs_share',
        },
        {   name        => "SETUP_DONE",
            description => "To create users on filers for uACL testing",
            default     => 1,
            regex       => qr/0|1/,
        },
        {   name        => 'ACL_UNIX_USER',
            description => 'ACL UNIX USER',
            spec        => 's',
            default     => 'uaclnfsuser',
        },
        {   name        => "ACL_CIFS_USER",
            description => "",
            spec        => "s",
            default     => 'aclcifsuser',
        },
        {   name        => "UACL_GRP",
            description => "",
            spec        => "s",
            default     => 'uaclgrp',
        },
        {   name        => "CIFS_GRP",
            description => "",
            spec        => "s",
            default     => 'smokeACLgrp',
        },
        {   name        => "VERBOSE",
            description => 'Set Debug messages for uACL libraries',
            spec        => "boolean",
            default     => 0,
        },
    ]
);

$Options->process();
1;
