# $Id://depot/prod/test/main/lib/uACL/uACLSetup.pm#1$
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary uACLSetup - Creates Basic uACL setup
## @author yadag@netapp.com
## @status private

=head1 NAME

uACL::uACLSetup <This set of functions create default setup for CR uACL testing>

=head1 DESCRIPTION

C<uACL::uACLSetup> provides methods for ACL tests .

=head1 METHODS

=cut

package uACLSetup;
use strict;
use warnings;
no warnings 'redefine';
use Tharn;
use uACL::uACLCommonParams;
use Options;
use Data::Dumper;
use Params::Common;
use NACL::C::Vserver;
use NACL::C::VserverNfs;
use NACL::C::VserverCifs;
use NACL::CS::VserverCifs;
use NACL::C::VserverNameMapping;
use NACL::C::VserverGroupMapping;
use NACL::C::VserverServicesUnixUser;
use NACL::Exceptions::NoElementsFound;
use NACL::C::VserverServicesUnixGroup;
use NACL::C::VserverCifsUsersAndGroupsLocalUser;
use NACL::C::VserverCifsUsersAndGroupsLocalGroup;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamValueException;

use Params::Validate
    qw[validate_with ARRAYREF BOOLEAN HASHREF SCALAR OBJECT ];

our (
    $Node, $grp,   @engr, @insuser, @root,    $Vserver,
    $name, @group, $user, $ID,      $PARAMS,
    $Options,      $GRPNAME,        $NFSUSER,
    $CIFSUSER,     $CIFS_GRP,       $CIFS_DOMAIN,
);

sub useUACLSetup {
    $Options = new Options;
    $Options->process();
    $GRPNAME = $Options->get( option => 'UACL_GRP' );
    $NFSUSER = $Options->get( option => 'ACL_UNIX_USER' );
    $CIFSUSER = $Options->get( option => 'ACL_CIFS_USER' );
    $CIFS_GRP = $Options->get( option => 'CIFS_GRP' );
    $CIFS_DOMAIN = $Options->get( option => 'CIFS_DOMAIN' );
    $Vserver = $Options->get( option => 'VSERVER' );
}

sub create_unix_groups {
    $Log->enter();
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            vserver => { type => SCALAR, optional => 1, default => 'vs0' },
            count   => {
                type     => SCALAR,
                optional => 1,
                default  => 10,
                regex    => qr/\d+/
            },
        },
        allow_extra => 1,
    );
    my $vserver = $args{vserver};
    $Log->comment("uACL::uACLSetup: Creating Unix Groups ...");
    my $max_grps = ( $args{count} > 50 ) ? 50 : $args{count};

    for ( my $gid_counter = 1; $gid_counter <= $max_grps; $gid_counter++ ) {
        try {
            my $gid    = 900 + $gid_counter;
            my $ug_obj = NACL::C::VserverServicesUnixGroup->create(
                command_interface => $self->{NODES}->[0],
                'vserver'         => $vserver,
                'name'            => $GRPNAME . $gid_counter,
                'id'              => $gid,
            );
        } ## end try

        # Catching the Exception if the Group already Created
        catch NACL::APISet::Exceptions::CommandFailedException with {
            my $err_msg = shift;
            if ( $err_msg->text() =~ /command failed: duplicate entry/ ) {
                $Log->comment( " uACL::uACLSetup: Group " 
                        . $GRPNAME
                        . $gid_counter
                        . " already created " );
            } ## end if ( $err_msg->text() ...
        };
    } ## end for ( my $gid_counter =...
    $Log->exit();
    return 1;

} ## end sub create_unix_groups

sub add_unix_users {
    $Log->comment(
        " uACL::uACLSetup: Creating unix users and Adding them to appropriate groups"
    );
    $Log->enter();
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            vserver => { type => SCALAR, optional => 1, default => 'vs0' },
            count   => {
                type     => SCALAR,
                optional => 1,
                default  => 10,
                regex    => qr/\d+/
            },
        },
        allow_extra => 1,
    );
    my $vserver = $args{vserver};
    $Log->comment("uACL::uACLSetup: Creating Unix users ...");

    my $gid = 0;
    for ( my $uid_counter = 1; $uid_counter <= $args{count}; $uid_counter++ )
    {
        my $uid      = 390000 + $uid_counter;
        my $name     = $NFSUSER . $uid_counter;
        my $fullname = $name;
        $Log->step(
            " uACL::uACLSetup: Adding user $name, UID: $uid, GID $gid, Full name: $fullname if user does not exist"
        );

        try {
            NACL::C::VserverServicesUnixUser->create(
                command_interface => $self->{NODES}->[0],
                'user'            => $name,
                'vserver'         => $vserver,
                'primary-gid'     => $gid,
                'id'              => $uid,
                'full-name'       => $fullname,
            );
        } ## end try
        catch NACL::APISet::Exceptions::CommandFailedException with {
            my $err_msg = shift;
            if ( $err_msg->text() =~ /command failed: duplicate entry/ ) {
                $Log->comment(
                    " uACL::uACLSetup: User $name already created ");
            }
        };

        $Log->step(
            "uACL::uACLSetup: Adding users to groups uaclgrp3 & uaclgrp6");
        foreach my $grp ( "uaclgrp3", "uaclgrp6" ) {
            try {
                NACL::C::VserverServicesUnixGroup->adduser(
                    'command_interface' => $self->{NODES}->[0],
                    'name'              => $grp,
                    'vserver'           => $vserver,
                    'username'          => $name
                );
            } ## end try
            catch NACL::APISet::Exceptions::CommandFailedException with {
                my $err_msg = shift;
                if ( $err_msg->text() =~ /command failed: duplicate entry/ ) {
                    $Log->comment(
                        "User $name already added to the group $grp");
                }
            };
        } ## end foreach my $grp ( "uaclgrp3"...
    } ## end for ( my $uid_counter =...

    $Log->exit();
    return 1;
} ## end sub add_unix_users

sub verver_group_mapping_create {

    $Log->enter();
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            vserver => { type => SCALAR, optional => 1, default => 'vs0' },
        },
        allow_extra => 1,
    );
    my $vserver = $args{vserver};
    $Log->comment("uACL::uACLSetup: Creating the group-mapping ");

    try {
        NACL::C::VserverGroupMapping->create(
            'command_interface' => $self->{NODES}->[0],
            'vserver'           => $vserver,
            'position'          => "901",
            'direction'         => 'win-unix',
            'pattern'           => $CIFS_GRP,
            'replacement'       => 'root'

        );
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $err_msg = shift;
        if ( $err_msg->text()
            =~ /command failed: The pattern $CIFS_GRP already exists/ )
        {
            $Log->comment(
                " uACL::uACLSetup: Group Mapping already done for this Pattern "
            );
        } ## end if ( $err_msg->text() ...
    };
    $Log->exit();
    return 1;

} ## end sub verver_group_mapping_create

sub vserver_name_mapping_create {
    $Log->enter();
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            vserver => { type => SCALAR, optional => 1, default => 'vs0' },
            count   => {
                type     => SCALAR,
                optional => 1,
                default  => 10,
                regex    => qr/\d+/
            },
            domain =>
                { type => SCALAR, default =>  $Options->get( option => 'CIFS_SERVER') },
        },
        allow_extra => 1,
    );
    my $vserver = $args{vserver};
    my $domain  = $args{domain};

    if ( $domain =~ /(\d{1,3}\.){3,3}\d{1,3}/ ) {
        $Log->comment(
            " uACL::uACLSetup: CIFS server name $domain is an IP Address, finding corresponding CIFS server name"
        );
        my $fetch_output;

        try {
            $fetch_output = NACL::CS::VserverCifs->fetch(
                command_interface => $self->{NODES}->[0],
                filter            => { 'vserver' => $vserver, },
                requested_fields  => ['cifs-server'],
            );
        } ## end try
        catch NACL::Exceptions::NoElementsFound with {
            my $err_msg = shift;
            $Log->comment(
                "No Valid CIFS configuraiton found for vserver $vserver, \nError $err_msg"
            );
            return $TCD::CONFIG;
        };

        $domain = $fetch_output->cifs_server();
        $Log->comment(" uACL::uACLSetup: CIFS server name is $domain");
    } ## end if ( $domain =~ /(\d{1,3}\.){3,3}\d{1,3}/)

    $Log->comment(
        " uACL::uACLSetup: Creating the Name-mapping for linux/win login ...");
    $Log->comment(
        " uACL::uACLSetup: Deleteing existing name mapping");
    try {
        NACL::C::VserverNameMapping->delete(
            'command_interface' => $self->{NODES}->[0],
            'vserver'           => $vserver,
            'position'          => 1,
            'direction'         => 'win-unix',
        );
    } catch NACL::APISet::Exceptions::CommandFailedException with {
    };

    try {
        NACL::C::VserverNameMapping->delete(
            'command_interface' => $self->{NODES}->[0],
            'vserver'           => $vserver,
            'position'          => 1,
            'direction'         => 'unix-win',
        );
    } catch NACL::APISet::Exceptions::CommandFailedException with {
    };

    my $win_login       = $CIFS_DOMAIN . '\\\\' .
                             $Options->get( option => 'CIFS_USER' );
    my $client_obj = NACL::C::Client->find();
    my $nfs_login = $client_obj->hostrec()->{connection_fields}{ssh}{username};
    try {
        NACL::C::VserverNameMapping->create(
            'command_interface' => $self->{NODES}->[0],
            'vserver'           => $vserver,
            'position'          => 1,
            'direction'         => 'win-unix',
            'pattern'           => $win_login,
            'replacement'       => $nfs_login,
        );
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $err_msg = shift;
        if ( $err_msg->text()
            =~ /command failed: The pattern .* already exists/ )
        {

            $Log->comment(
                " uACL::uACLSetup: win-unix Name Mappping already done for $win_login"
            );
        } ## end if ( $err_msg->text() ...
    }
    try {
        NACL::C::VserverNameMapping->create(
            'command_interface' => $self->{NODES}->[0],
            'vserver'           => $vserver,
            'position'          => 1,
            'direction'         => 'unix-win',
            'pattern'           => $nfs_login,
            'replacement'       => $win_login,
        );
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $err_msg = shift;
        if ( $err_msg->text()
            =~ /command failed: The pattern .* already exists/ )
        {
            $Log->comment(
                " uACL::uACLSetup: unix-win Name Mappping already done for $nfs_login"
            );
        } ## end if ( $err_msg->text() ...
    };

    $Log->comment(
        " uACL::uACLSetup: Creating the Name-mapping for uACL users ...");

    for ( my $index = 1; $index <= $args{count}; $index++ ) {
        my $position_index = 1025 - $index;
        my $win_user       = $domain . '\\\\' . $CIFSUSER . $index;
        my $nfs_user       = $NFSUSER . $index;

        try {
            NACL::C::VserverNameMapping->create(
                'command_interface' => $self->{NODES}->[0],
                'vserver'           => $vserver,
                'position'          => $position_index,
                'direction'         => 'win-unix',
                'pattern'           => $win_user,
                'replacement'       => $nfs_user,
            );
        } ## end try
        catch NACL::APISet::Exceptions::CommandFailedException with {
            my $err_msg = shift;
            if ( $err_msg->text()
                =~ /command failed: The pattern .* already exists/ )
            {

                $Log->comment(
                    " uACL::uACLSetup: win-unix Name Mappping already done for $win_user"
                );
            } ## end if ( $err_msg->text() ...
        } ## end with
        catch NACL::APISet::Exceptions::InvalidParamValueException with {
            my $err_msg = shift;
            if ( $err_msg->text() =~ /"1025" is an invalid value for field/ )
            {
                $Log->comment(
                    " uACL::uACLSetup: Maximum limit of name-mappings that can be created is reached"
                );
            } ## end if ( $err_msg->text() ...

        };

        try {
            NACL::C::VserverNameMapping->create(
                'command_interface' => $self->{NODES}->[0],
                'vserver'           => $vserver,
                'position'          => $position_index,
                'direction'         => 'unix-win',
                'pattern'           => $nfs_user,
                'replacement'       => $win_user,
            );
        } ## end try
        catch NACL::APISet::Exceptions::CommandFailedException with {
            my $err_msg = shift;
            if ( $err_msg->text()
                =~ /command failed: The pattern .* already exists/ )
            {
                $Log->comment(
                    " uACL::uACLSetup: unix-win Name Mappping already done for $nfs_user"
                );
            } ## end if ( $err_msg->text() ...
        };

    } ## end for ( my $index = 1; $index...

    $Log->exit();
    return 1;
} ## end sub vserver_name_mapping_create

sub create_local_cifs_users {

    $Log->enter();
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            vserver => { type => SCALAR, optional => 1, default => 'vs0' },
            count   => {
                type     => SCALAR,
                optional => 1,
                default  => 10,
                regex    => qr/\d+/
            },
        },
        allow_extra => 1,
    );

    my $vserver = $args{vserver};

    for ( my $index = 1; $index <= $args{count}; $index++ ) {
        try {
            my $output = NACL::C::VserverCifsUsersAndGroupsLocalUser->create(
                'command_interface'           => $self->{NODES}->[0],
                'vserver'                     => $vserver,
                'user-name'                   => $CIFSUSER . $index,
                'user-cannot-change-password' => 'true',
                'password'                    => 'netapp1!',
                'is-account-disabled'         => 'false',
                'password-never-expires'      => 'true'
            );
        } ## end try
        catch NACL::APISet::Exceptions::CommandFailedException with {
            my $err_msg = shift;
            if ( $err_msg->text()
                =~ /Reason: The specified name already exists/ )
            {
                $Log->comment( " uACL::uACLSetup: Local CIFS user " 
                        . $CIFSUSER
                        . $index
                        . " already exists" );
            } ## end if ( $err_msg->text() ...
        };
    } ## end for ( my $index = 1; $index...
    $Log->exit();
    return 1;
} ## end sub create_local_cifs_users

sub create_local_cifs_groups {

    $Log->enter();
    my $self = shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            vserver => { type => SCALAR, optional => 1, default => 'vs0' },
            count   => {
                type     => SCALAR,
                default  => 1,
                regex    => qr/\d+/
            },
        },
        allow_extra => 1,
    );

    my $vserver = $args{vserver};

    for ( my $index = 0; $index < $args{count}; $index++ ) {
        try {
            my $ind = ($index) ? ($index) : "";
            my $output = NACL::C::VserverCifsUsersAndGroupsLocalGroup->create(
                'command_interface'           => $self->{NODES}->[0],
                'vserver'                     => $vserver,
                'group-name'                   => $CIFS_GRP . $ind,
            );
        } ## end try
        catch NACL::APISet::Exceptions::CommandFailedException with {
            my $err_msg = shift;
            if ( $err_msg->text()
                =~ /Reason: The specified name already exists/ )
            {
                $Log->comment( " uACL::uACLSetup: Local CIFS Group "
                        . $CIFS_GRP
                        . $index
                        . " already exists" );
            } ## end if ( $err_msg->text() ...
        };
    } ## end for ( my $index = 1; $index...
    $Log->exit();
    return 1;
} ## end sub create_local_cifs_groups

1;
