package vldbUtil;

#use strict;
use Tharn;
use Data::Dumper;
use NACL::C::Node;
use NACL::APISet;
use NACL::STask::Aggregate;
use NACL::STask::Volume;
use Server;
use Connectrec qw();
use NACL::C::ClusterRing;

use base 'Exporter';
our @EXPORT = qw(create_hash
  inject_discrepancies
  inject_discrepancies_in_partner_node
  inject_discrepancy_aggr_VLDB_dup_WAFL
  online_offline_vldb
  create_dup_volname_discrepancy
  create_flexclone_parent_volume
  _delete_volumes_in_wafl
  _deleteAggr
  _delete_volumes_in_vldb
  _delete_volumes_in_vldb_from_vvolTable
  _get_DSID_from_vol_name
  _get_MSID
  _delete_aggr_in_wafl
  _aggrexist
  _vserverexist
  _vserver_exists
  _get_aggr_from_volume
  _findNodeFromAggr
  _create_dup_vols
  _vserver_name_to_id
  _cleanup
  compare_vreport_list
  test
);
my ( @Nodes_array, $Api_Set_Obj_ngsh, $Api_Set_Obj_bsd, $node, $Filer, $ci,
    $volPrefix );

sub _config {
    @Nodes_array = NACL::C::Node->find( command_interface => $ci );
    $Test->nlog("nodes in the cluster are :\n");
    foreach (@Nodes_array) {
        $Test->nlog("$_->{node}\n");
    }
    return 1;
} ## end sub _config
################################################################################
sub init {

    #    my $invocant = shift;
    #    my $class         = ref($invocant) || $invocant;
    #    my %args          = @_;
    #    my $self          = {@_
    #			 };
    #    bless $self,$class;
    $ci               = shift;
    $Api_Set_Obj_ngsh = $ci->apiset(
        category  => "Node",
        interface => "CLI",
        set       => "CMode"
    );
    $Api_Set_Obj_bsd = $ci->apiset(
        category  => "Node",
        interface => "CLI",
        set       => "Systemshell"
    );
    $Filer     = shift;
    $Test      = shift;
    $volPrefix = shift;
    _config();
    print "this is constructor for vldbUtil\n";

    #return $self;
} ## end sub init

sub test {
    $Api_Set_Obj_ngsh->storage_aggregate_show();
    print "you are in vldbUtil::test\n";
}

sub vldbtest {
    my $args = @_;

    $Api_Set_Obj_bsd->execute_raw_command( command => "vldbtest help", );
}

#sub vreport_show{
#    my $args = @_;
#    my @debug_vreport_obj = NACL::C::DebugVreport->find(command_interface => $node);
#
#}
#sub vreport_fix{
#    my @debug_vreport_obj = NACL::C::DebugVreport->find(command_interface => $node);
#    foreach my $obj (@debug_vreport_obj){
#    }
#
#
#}

sub _deleteAggr {
    my ($AggrPrefix) = @_;

    #delete the aggregates that were created by this test

    eval {
        $output =
          $Filer->storage_aggregate_show( aggregate => "$AggrPrefix*", );
    };
    return $TCD::PASS if ($@);
    my $result = $TCD::PASS;

    my $aggr_cnt = scalar @{$output};
    $Test->nlog("Found $aggr_cnt aggregate(s) matching name:'${AggrPrefix}*'");

    for ( my $i = 0 ; $i < $aggr_cnt ; $i++ ) {
        my $aggregate = $$output[$i]{aggregate};

        $Test->nlog("deleting aggregate $aggregate");
        eval {
            my $del_output =
              $Filer->storage_aggregate_delete( aggregate => $aggregate, );
        };
        if ($@) {
            $Test->nlog("Failed to delete aggregate '$aggregate': ($@)");

        }
    } ## end for ( my $i = 0; $i < $aggr_cnt...

    return $result;
} ## end sub _deleteAggr

sub _delete_volumes_in_wafl {
    my $flexVolume = shift;
    my $aggregate  = shift;
    my $vserver    = shift;
    my $node;
    if ( !defined($vserver) ) {
        $vserver = 'vs0';
    }
    if ( defined($aggregate) ) {

        $node = _findNodeFromAggr($aggregate);
        $Test->nlog("add $node to the servers list\n");
        $Filer->add_nodes( nodes => $node, );
        $Filer->set_execution_server( -Server => $node );
    } ## end if ( defined ($aggregate...
    eval {
        $output = $Filer->volume_show(
            vserver  => $vserver,
            volume   => "$flexVolume",
            instance => "*"
        );
    };
    return $TCD::FAIL if ($@);
    my $result = $TCD::PASS;

    my $vol_cnt = scalar @{$output};
    $Test->nlog("Found $vol_cnt volumes matching name:'$flexVolume'");
    $Filer->set_execution_server( -Server => $node );

    for ( my $i = 0 ; $i < $vol_cnt ; $i++ ) {
        my $volume  = $$output[$i]{volume};
        my $dsid    = $$output[$i]{dsid};
        my $hexDsid = sprintf( "0x%08x", $dsid );

        $Test->nlog("\nOfflining volume $volume from WAFL");
        $output = $Filer->shell( cmd => "zsmcli d-volume-offline id=$hexDsid" );
        if ( $output !~ /passed/ ) {
            $Test->nlog("$output");
            $Test->nlog("Failed to offline volume in WAFL '$volume': ($@)");
            return $TCD::FAIL;
        }
        else {
            $Test->nlog("PASS\n");
        }

        $Test->nlog("\nDeleting volume $volume from WAFL");
        $output = $Filer->shell( cmd => "zsmcli d-volume-destroy id=$hexDsid" );
        if ( $output !~ /passed/ ) {
            $Test->nlog("$output");
            $Test->nlog("Failed to delete volume in WAFL '$volume': ($@)");
            return $TCD::FAIL;
        }
        else {
            $Test->nlog("PASS\n");
        }

    } ## end for ( my $i = 0; $i < $vol_cnt...
    return $result;
} ## end sub _delete_volumes_in_wafl

sub _delete_volumes_in_vldb {
    my $flexVolume = shift;
    my $vserver    = shift;
    if ( !defined($vserver) ) {
        $vserver = 'vs0';
    }
    eval {
        $Test->nlog("Came to check if the volume present on vserver 'vs0'");
        $output = $Filer->volume_show(
            vserver  => $vserver,
            volume   => "$flexVolume",
            instance => "*"
        );
        $Test->nlog("volume show output = $output");
    };
    return $TCD::FAIL if ($@);
    my $result = $TCD::PASS;

    my $vol_cnt = scalar @{$output};
    $Test->nlog("Found $vol_cnt volumes matching name:'$flexVolume'");

    for ( my $i = 0 ; $i < $vol_cnt ; $i++ ) {
        my $volume = $$output[$i]{volume};
        my $dsid   = $$output[$i]{dsid};

        $Test->nlog("Deleting volume $volume from vldb");
        eval {
            my $cmd =
"set -privilege diag; set -confirmations off; debug smdb table deleteVolume run -volDsid $dsid -deleteJunction true ";
            $output = $Filer->shell( cmd => "ngsh -c \"$cmd\"" );
        };
        if ($@) {
            $Test->nlog("Failed to delete volume in vldb '$volume': ($@)");
            return $TCD::FAIL;
        }
    } ## end for ( my $i = 0; $i < $vol_cnt...
    return $result;
} ## end sub _delete_volumes_in_vldb

# This method will be used for creation of discrepancies per created
# volumes. The command invoked would be:
# debug smdb table vvolTable delete -volDsid <DSID> -force true
sub _delete_volumes_in_vldb_from_vvolTable {
    my $flexVolume = shift;
    my $vserver    = shift;
    if ( !defined($vserver) ) {
        $vserver = 'vs0';
    }
    eval {
        $output = $Filer->volume_show(
            vserver  => $vserver,
            volume   => "$flexVolume",
            instance => "*"
        );
    };
    return $TCD::FAIL if ($@);
    my $result = $TCD::PASS;

    my $vol_cnt = scalar @{$output};
    $Test->nlog("Found $vol_cnt volumes matching name:'$flexVolume'");

    for ( my $i = 0 ; $i < $vol_cnt ; $i++ ) {
        my $volume = $$output[$i]{volume};
        my $dsid   = $$output[$i]{dsid};

        $Test->nlog("Deleting volume $volume from vldb");
        eval {

            # Delete it from the vvolTable with a force option. This is
            # the only way to create junction discrepancy for every
            # volume that it created
            my $cmd =
"set -privilege diag; set -confirmations off; debug smdb table vvolTable delete -volDsid $dsid -force true";
            $output = $Filer->shell( cmd => "ngsh -c \"$cmd\"" );
        };
        if ($@) {
            $Test->nlog("Failed to delete volume in vldb '$volume': ($@)");
            return $TCD::FAIL;
        }
    } ## end for ( my $i = 0; $i < $vol_cnt...
    return $result;
} ## end sub _delete_volumes_in_vldb_from_vvolTable

# sub _snapmirror_create_initialize{
#
#	my $srcpath = $cluster_name[0]->{'name'}.":\/\/$vserver\/$SrcVolume";
#	my $destpath = $cluster_name[0]->{'name'}.":\/\/$vserver\/$DestVolume";
#	my $result = $TCD::PASS;
#		eval{
#		$Filer->snapmirror_create('destination-path' => $destpath,
#								   'source-path' => $srcpath,);
#		$Filer->snapmirror_initialize('destination-path' =>$destpath,);
#		};
#		if($@){
#			$Test->nlog(" snapmirror creation/initialization failed\n");
#			return $TCD::FAIL;
#		}
#
#	return $result;
#	}
#
sub _get_DSID_from_vol_name {
    my $flexVolume = shift;
    eval {
        $output = $Filer->volume_show(
            vserver  => $vserver,
            volume   => "$flexVolume",
            instance => "*"
        );
    };
    return $TCD::FAIL if ($@);

    $Test->nlog("Found volume:'$flexVolume' and MSID : $$output[0]{dsid}");
    return $$output[0]{dsid};
} ## end sub _get_DSID_from_vol_name

sub _get_MSID {
    my $flexVolume = shift;
    $output = $Filer->volume_show(
        vserver  => $vserver,
        volume   => "$flexVolume",
        instance => "*"
    );

    $Test->nlog("Found volume:'$flexVolume' and MSID : $$output[0]{msid}");

    return $$output[0]{msid};

} ## end sub _get_MSID

sub _delete_aggr_in_wafl {
    my $aggregate = shift;
    my $node      = shift;
    $Test->nlog("add $node to the servers list\n");
    $Filer->add_nodes( nodes => $node, );
    $Filer->set_execution_server( -Server => $node );
    $Test->nlog("Offlining Aggregate $aggregate from WAFL");

    $output =
      $Filer->shell( cmd => "zsmcli aggr-offline aggregate=$aggregate" );
    if ( $output !~ /passed/ ) {
        $Test->nlog("$output");
        $Test->nlog("Failed to offline aggregate in WAFL '$aggregate': ($@)");
        return $TCD::FAIL;
    }
    else {
        $Test->nlog("PASS\n");
    }

    $Test->nlog("\nDeleting aggregate $aggregate from WAFL");
    $output =
      $Filer->shell( cmd => "zsmcli aggr-destroy aggregate=$aggregate" );
    if ( $output !~ /passed/ ) {
        $Test->nlog("$output");
        $Test->nlog("Failed to delete aggregate in WAFL '$aggregate': ($@)");
        return $TCD::FAIL;
    }
    else {
        $Test->nlog("PASS\n");
    }
    $node = $Nodes[0];
    $Filer->set_execution_server( -Server => $node );

} ## end sub _delete_aggr_in_wafl

sub _aggrexist {
    my $aggr = shift;
    eval { $Filer->storage_aggregate_show( aggregate => $aggr ); };
    return 0 if ($@);

    return 1;
} ## end sub _aggrexist

sub _vserverexist {

    my $vserver = shift;
    eval { $Filer->vserver_show( vserver => $vserver ); };
    return 0 if ($@);

    return 1;
} ## end sub _vserverexist

sub _vserver_exists {
    my $vserver = shift;
    try {
        my $response = $Api_Set_Obj_ngsh->vserver_show( vserver => $vserver, );
        if ($response) {
            return 1;
        }
        else {
            return 0;
        }
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        return 0;
    };
} ## end sub _vserver_exists

sub _get_aggr_from_volume {
    my $volume = shift;
    my $aggregate;
    $output = $Filer->storage_aggregate_show( aggregate => "$aggregate", );
    $aggregate = $$output[0]{aggregate};
    $Test->nlog("volume is hosted on  $aggregate ");

    return $aggregate;
} ## end sub _get_aggr_from_volume

sub _findNodeFromAggr {
    my $aggregate = shift;
    $output = $Filer->storage_aggregate_show( aggregate => "$aggregate", );
    my $result = $TCD::PASS;
    my $node   = $$output[0]{nodes};
    $Test->nlog("aggregate $aggregate is on $node ");
    return $node;
} ## end sub _findNodeFromAggr

sub _create_dup_vols {
    my $by_name_id     = shift;
    my $same_diff_node = shift;
    my @aggregates;
    my $result;

    #check if there are atleast two aggregate on each node
    my @aggr_list;
    foreach (@nodes) {

#        $response = $Api_Set_Obj_ngsh->storage_aggregate_show(nodes=> $_->{node},
#						 root => "false",);
#        my @output = @{$response->get_parsed_output()};
#	my $aggr_cnt = scalar @output;
#	if($aggr_cnt != 0){
#	    $Test->nlog("Found $aggr_cnt aggregate(s) on the node '$_->{node}'");
#	    push(@aggr_list,$output[0]->{aggregate})
#	}else{
        for ( my $loopvar = 0 ; $loopvar < 2 ; $loopvar++ ) {

            #create a new aggregate
            my $aggregate = $volPrefix . $loopvar;
            $Test->nlog("creating Aggregate '$aggregate'");
            NACL::STask::Aggregate->create(
                command_interface => $_,
                if_exists         => 'reuse',
                aggregate         => $aggregate,
            );
            push( @aggr_list, $aggregate );

            #}
        } ## end for ( my $loopvar = 0; ...

    } ## end foreach (@nodes)
    if ( !_vserver_exists('vs0') ) {
        $vserver = $volPrefix . 'vserver';
        $Test->nlog("creating vserver '$vserver'");
        NACL::C::Vserver->create(
            command_interface           => $Nodes_array[0],
            vserver                     => $vserver,
            rootvolume                  => "${vserver}_root",
            aggregate                   => $aggr_list[0],
            'ns-switch'                 => 'file',
            'rootvolume-security-style' => 'mixed',
        );
    } ## end if ( !_vserver_exists ...

    if ( $by_name_id eq 'name' ) {

        #create duplicate volumes by name
        #this is not implemented now
        #if($same_diff_node eq 'same'){
        ##get two aggregates on the same node
        #}else{
        ##get two aggregates from two different nodes
        #}
    }
    elsif ( $by_name_id eq 'dsid' ) {

        #create duplicate volumes by dsid
        if ( $same_diff_node eq 'same' ) {

    #create a volume 'V' on an aggregate 'A' on node 'N' and delete it from VLDB
            $volume = 'voldupbydsidone';
            NACL::STask::Volume->create(
                command_interface => $Nodes_array[0],
                vserver           => $vserver,
                if_exists         => 'reuse',
                aggregate         => $aggr_list[0],
                volume            => $volume,
            );
            _delete_volumes_in_vldb($volume);

            #make the aggregate 'A' offline
            NACL::STask::Aggregate->modify(
                command_interface => $Nodes_array[0],
                aggregate         => $aggr_list[0],
                state             => 'offline',
                'unmount-volumes' => 'true',
            );

            #decrease nextIdTable's dsid value by '1'
            $result = _modify_nextIdTable( 'dsid', 1 );
            return $TCD::FAIL if ( $result != $TCD::PASS );

      #create a new volume 'V' on the same node 'N' in a different aggregate 'B'
            $volume = 'voldupbydsidtwo';
            NACL::STask::Volume->create(
                command_interface => $Nodes_array[0],
                vserver           => $vserver,
                if_exists         => 'reuse',
                aggregate         => $aggr_list[1],
                volume            => $volume,
            );

            #bring back the aggregate 'A' online
            NACL::STask::Aggregate->modify(
                command_interface => $Nodes_array[0],
                aggregate         => $aggr_list[0],
                state             => 'online',
            );
        }
        else {

    #get two aggregates from two different nodes
    #create a volume 'V' on an aggregate 'A' on node 'N' and delete it from VLDB
            $volume = 'voldupbydsidone';
            NACL::STask::Volume->create(
                command_interface => $Nodes_array[0],
                vserver           => $vserver,
                if_exists         => 'reuse',
                aggregate         => $aggr_list[0],
                volume            => $volume,
            );
            _delete_volumes_in_vldb($volume);

            #make the aggregate 'A' offline
            NACL::STask::Aggregate->modify(
                command_interface => $Nodes_array[0],
                aggregate         => $aggr_list[0],
                state             => 'offline',
            );

            #decrease nextIdTable's dsid value by '1'
            $result = _modify_nextIdTable( 'dsid', 1 );
            return $TCD::FAIL if ( $result != $TCD::PASS );

      #create a new volume 'V' on the  on node 'N2' in a different aggregate 'C'
            my $volume = 'voldupbydsidtwo';
            NACL::STask::Volume->create(
                command_interface => $Nodes_array[0],
                vserver           => $vserver,
                if_exists         => 'reuse',
                aggregate         => $aggr_list[2],
                volume            => $volume,
            );

            #bring back the aggregate 'A' online
            NACL::STask::Aggregate->modify(
                command_interface => $Nodes_array[0],
                aggregate         => $aggr_list[0],
                state             => 'online',
            );
        } ## end else [ if ( $same_diff_node eq...
    } ## end elsif ( $by_name_id eq 'dsid')
    return ( $vserver, $volume );
} ## end sub _create_dup_vols
##sub vreport_show{
##
##
##}
##sub vrepair_fix{
##
##}
#sub _modify_nextIdTable{
#    my $field  = shift;
#    my $value  = shift;
#    my $dsid;
#    eval {
#        my $cmd = "set -privilege diag; set -confirmations off; debug smdb table nextIdTable show";
#        my $output = $Filer->cli(cmd=>$cmd);
#        my @array_of_hashes = create_hash($output);
#        $dsid = $array_of_hashes[0]{$field};
#        $dsid = $dsid-$value;
#        $cmd = "set -privilege diag; set -confirmations off;debug smdb table nextIdTable modify -$field $dsid";
#        $output = $Filer->shell(cmd=>"ngsh -c \"$cmd\"");
#    };
#    if ($@) {
#        $Test->nlog("Failed to modify nextIdTable: ($@)");
#        return $TCD::FAIL;
#    }
#    return $TCD::PASS;
#}
#
#
#
sub _vserver_name_to_id {
    my $vserver = shift;

    #debug smdb table vserver_id_to_name show -name vs0
    my $cmd = "debug smdb table vserver_id_to_name show -name $vserver";
    my $output = $Filer->cli( cmd => $cmd );
    my @array_of_hashes = create_hash($output);
    $Test->nlog("$array_of_hashes[0]{id}");
    return $array_of_hashes[0]{id};
} ## end sub _vserver_name_to_id

# This function will make the VLDB process online/offline on the requested
# node based on the operation request of the caller.
# 1. 'operation == 0' indicates offlining VLDB
# 2. 'operation == 1' indicates making VLDB online.
# 'node' is taken as one of the parameter.
sub online_offline_vldb {
    my $node      = shift;
    my $operation = shift;

    my $host = host($node);
    my $conn = Connectrec->new(
        host     => $host,
        username => 'diag',
        password => 'netapp1!',
        template => 'ssh-persist',
        login    => 1,
    );
    my $apiset = NACL::APISet->new(
        connection => $conn,
        category   => 'Node',
        interface  => 'CLI',
        set        => 'Systemshell',
    );

    if ( $operation == 0 ) {
        $Test->nlog( "\n** Offline VLDB in node:" . $node . "**\n" );
        $apiset->spmctl( handle => 'vldb', stop => 1 );
    }
    else {
        $Test->nlog( "\n** Make VLDB online in node:" . $node . "**\n" );
        $apiset->spmctl( handle => 'vldb', exec => 1 );
        for ( my $count = 0 ; $count < 3 ; $count++ ) {
            $Test->nlog("Verifying the vldb status after 30 seconds\n");
            sleep 30;

            my @cluster_ring =
              NACL::C::ClusterRing->find( command_interface => $ci );
            foreach my $ring (@cluster_ring) {
                my $state = $ring->state();
                next if ( $state->unitname !~ /vldb/ );
                if ( ( $state->online =~ /offline/ ) && ( $count == 2 ) ) {
                    NATE::BaseException->throw("Could not online vldb");
                }
            }
        }
    }
    return $TCD::PASS;
} ## end sub online_offline_vldb

sub create_flexclone_parent_volume {

    # Create a new aggregate
    my $aggregate = shift;
    my $volume    = shift;

    # Create vserver 'vs0' if not present
    $Test->nlog("\nCreate vserver 'vs0' if it doesn't exist\n");
    if ( !_vserver_exists('vs0') ) {
        $vserver = $Prefix . 'vserver';
        $Test->nlog("creating vserver '$vserver'");
        NACL::C::Vserver->create(
            command_interface           => $Nodes_array[0],
            vserver                     => $vserver,
            rootvolume                  => "${vserver}_root",
            aggregate                   => $Nodes_array[0]->{node},
            'ns-switch'                 => 'file',
            'rootvolume-security-style' => 'mixed',
        );
    } ## end if ( !_vserver_exists ...

    # Create volume for these new aggregate
    $Test->nlog("Creating a new aggregate '$aggregate'");

    NACL::STask::Aggregate->create(
        command_interface => $Nodes_array[0],
        if_exists         => 'reuse',
        aggregate         => $aggregate,
    );

    $Test->nlog("Aggregate '$aggregate' is now created");

    #create volume vol<> for aggregate aggr1<>
    $Test->nlog("Creating Parent volume '$volume' for aggregate '$aggregate'");

    my %other_opts = ( size => '20m', );
    NACL::STask::Volume->create(
        command_interface => $Nodes_array[0],
        vserver           => $vserver,
        if_exists         => 'reuse',
        aggregate         => $aggregate,
        volume            => $volume,
        %other_opts,
    );

    $Test->nlog(
        "Parent Volume '$volume' for aggregate '$aggregate' is now created");
} ## end sub create_flexclone_parent_volume

# This methode will create discrepancy with duplicate volume name
sub create_dup_volname_discrepancy {

    # Create a new aggregate
    my $volume;
    my $aggregate;
    my $aggregate2;
    my %Nodes_aggr;
    my %Nodes_aggr2;

    # Create vserver 'vs0' if not present
    $Test->nlog("\nCreate vserver 'vs0' if it doesn't exist\n");
    if ( !_vserver_exists('vs0') ) {
        $vserver = $Prefix . 'vserver';
        $Test->error("vserver 'vs0' doesn't exist");
        $Test->nlog("creating vserver '$vserver'");
        NACL::C::Vserver->create(
            command_interface           => $Nodes_array[0],
            vserver                     => $vserver,
            rootvolume                  => "${vserver}_root",
            aggregate                   => $Nodes_array[0]->{node},
            'ns-switch'                 => 'file',
            'rootvolume-security-style' => 'mixed',
        );
    } ## end if ( !_vserver_exists ...

    # Create volumes for these new aggregates
    for ( my $i = 0 ; $i < scalar @Nodes_array ; $i++ ) {

        # create aggregate: 'aggr1<>' for all nodes
        $aggregate = $volPrefix . "aggr1$i";
        $Test->nlog("Creating a new aggregate '$aggregate'");
        NACL::STask::Aggregate->create(
            command_interface => $Nodes_array[$i],
            if_exists         => 'reuse',
            aggregate         => $aggregate,
        );
        push( @{ $Nodes_aggr{ $node->{node} } }, $output[0]->{aggregate} );

        $Test->nlog("Aggregate '$aggregate' is now created");

        #create volume vol<> for aggregate aggr1<>
        $volume = $volPrefix . "vol$i";
        $Test->nlog("Creating volume '$volume' for aggregate '$aggregate'");

        my %other_opts = ( size => '20m', );
        NACL::STask::Volume->create(
            command_interface => $Nodes_array[$i],
            vserver           => $vserver,
            if_exists         => 'reuse',
            aggregate         => $aggregate,
            'junction-path'   => undef,
            volume            => $volume,
            %other_opts,
        );

        $Test->nlog(
            "Volume '$volume' for aggregate '$aggregate' is now created");

        # Now delete the volumes in the VLDB for the newly created
        # volumes
        $Test->nlog("Delete '$volume' from the VLDB");
        my $return_val = _delete_volumes_in_vldb( $volume, $vserver );
        $Test->nlog("Return value = $return_val");

        # Now Offline the aggregate
        $Test->nlog("\nOffline Aggregate '$aggregate'");
        NACL::C::StorageAggregate->offline(
            command_interface => $Nodes_array[$i],
            aggregate         => $aggregate,
            "unmount-volumes" => "true",
        );

        # create aggregate: 'aggr2<>' for all nodes
        $aggregate2 = $volPrefix . "aggr2$i";
        $Test->nlog("\nCreating a new aggregate '$aggregate2'");

        NACL::STask::Aggregate->create(
            command_interface => $Nodes_array[$i],
            if_exists         => 'reuse',
            aggregate         => $aggregate2,
        );
        push( @{ $Nodes_aggr2{ $node->{node} } }, $output[0]->{aggregate} );

        #create volume vol<> for aggregate aggr2<>
        $Test->nlog("creating volume '$volume' for aggregate '$aggregate2'");

        NACL::STask::Volume->create(
            command_interface => $Nodes_array[$i],
            vserver           => $vserver,
            if_exists         => 'reuse',
            aggregate         => $aggregate2,
            'junction-path'   => undef,
            volume            => $volume,
            %other_opts,
        );
        $Test->nlog("Volume '$volume' for aggregate '$aggregate2' is created");

        # Now make aggr1<> online
        $Test->nlog("Make the Aggregate '$aggregate' online.\n");
        NACL::C::StorageAggregate->online(
            command_interface => $Nodes_array[$i],
            aggregate         => $aggregate,
        );
        $Test->nlog("Aggregate '$aggregate' is now online.\n");
    } ## end for ( my $i = 0; $i < scalar...
    return 1;
} ## end sub create_dup_volname_discrepancy

sub inject_discrepancies {

    #volume
    #create some volumes on one volume on each node
    my $volume;
    my $aggregate = $volPrefix . "aggr1";
    my @aggregates;

    #check if there are atleast one aggregate on each node
    my @aggr_list;
    my %Nodes_aggr;
    foreach my $node (@Nodes_array) {
        $response = $Api_Set_Obj_ngsh->storage_aggregate_show(
            nodes => $node->{node},
            root  => "false",
        );
        my @output   = @{ $response->get_parsed_output() };
        my $aggr_cnt = scalar @output;
        if ( $aggr_cnt != 0 ) {
            $Test->nlog(
                "Found $aggr_cnt aggregate(s) on the node '$node->{node}'");
            push( @{ $Nodes_aggr{ $node->{node} } }, $output[0]->{aggregate} );
        }
        else {

            #create a new aggregate
            $Test->nlog("creating Aggregate '$aggregate'");
            NACL::STask::Aggregate->create(
                command_interface => $node->{node},
                if_exists         => 'reuse',
                aggregate         => $aggregate,
            );
            push( @{ $Nodes_aggr{ $node->{node} } }, $output[0]->{aggregate} );
        } ## end else [ if ( $aggr_cnt != 0 )

    } ## end foreach my $node (@Nodes_array)
##1. create a 2 node cluster -checked as part of setup()
##2. create a vserver (vserver setup)
    if ( !_vserver_exists('vs0') ) {
        $vserver = $Prefix . 'vserver';
        $Test->nlog("creating vserver '$vserver'");
        NACL::C::Vserver->create(
            command_interface           => $Nodes_array[0],
            vserver                     => $vserver,
            rootvolume                  => "${vserver}_root",
            aggregate                   => $Nodes_array[0]->{node},
            'ns-switch'                 => 'file',
            'rootvolume-security-style' => 'mixed',
        );
    } ## end if ( !_vserver_exists ...
    for ( my $i = 0 ; $i < scalar @Nodes_array ; $i++ ) {
        $volume = $volPrefix . "vol$i";
        $Test->nlog("creating volume '$volume' with junction-path '\/$volume'");

 #    my $Volume = NACL::C::Volume->create(command_interface => $Nodes_array[0],
 #                                          volume            => $volume,
 #                                          vserver           => $vserver,
 #                                          aggregate         => $aggr_list[0],
 #                                          'junction-path'   => "/$volume",
 #					  size              => '2g',
 #                                          );
        my %other_opts = (
            'junction-path' => "/$volume",
            size            => '20m',
        );
        NACL::STask::Volume->create(
            command_interface => $Nodes_array[$i],
            vserver           => $vserver,
            if_exists         => 'reuse',
            aggregate         => $Nodes_aggr{ $Nodes_array[$i]->{node} }->[0],
            volume            => $volume,
            %other_opts,
        );
        if ( $i % 2 == 0 ) {
            _delete_volumes_in_vldb( $volume, $vserver );
        }
        else {
            _delete_volumes_in_wafl( $volume,
                $Nodes_aggr{ $Nodes_array[$i]->{node} }->[0], $vserver );
        }
    } ## end for ( my $i = 0; $i < scalar...

    #junction
    #aggregate
    #snapmirror
    return 1;
} ## end sub inject_discrepancies

# This method will inject aggrgate discrepancy in VLDB with duplicate
# aggregates in WAFL.
sub inject_discrepancy_aggr_VLDB_dup_WAFL {
    my $aggregate = shift;

    # create aggrgate in Node 1
    $Test->nlog(
        "Creating a new aggregate '$aggregate' in node '$Nodes_array[0]'.\n");

    NACL::STask::Aggregate->create(
        command_interface => $Nodes_array[0],
        if_exists         => 'reuse',
        aggregate         => $aggregate,
    );

    try {
        @Response = NACL::CS::StorageAggregate->fetch(
            command_interface => $ci,
            filter            => {
                state     => "online",
                aggregate => $aggregate,
            },
            requested_fields => ['uuid'],

        );
    }
    catch NATE::BaseException with {

        my $exception_object = shift;
        $Test->nlog(
"No aggregates available with name . $aggregate . Error: . $exception_object . .\n"
        );
        $flag_err = 1;
    };

    # Fail the TC if the flag_err is 1
    if ($flag_err) {
        $Test->error(
"*** No aggregates with name . $aggregate .  available in the cluster ***"
        );
        return;

        #	return $TCD::FAIL;
    }

    foreach my $c_obj (@Response) {
        $UUID = $c_obj->{uuid};
    }

    # Delete aggregate in VLDB
    $cmd = "debug smdb table aggrTable delete -aggrID $UUID -force true";
    eval {
        $output = $Api_Set_Obj_ngsh->execute_raw_command( command => $cmd );
    };
    if ($@) {
        $Test->error(
"*** Failed to delete the aggrTable entry for the aggregate . $aggregate . ***\n"
        );
        return;
    }

    # create aggregate in Node 2
    $Test->nlog(
        "Creating a new aggregate '$aggregate' in node '$Nodes_array[1]'.\n");

    NACL::STask::Aggregate->create(
        command_interface => $Nodes_array[1],
        if_exists         => 'reuse',
        aggregate         => $aggregate,
    );

    # Now you have aggregate which is in VLDB and is duplicate in WAFL,
    # Both nodes have the same WAFL entry.
}

# This method will inject volume discrepancies on the partner node
sub inject_discrepancies_in_partner_node {

    #create some volumes on one volume on each node
    my $volume;
    my $aggregate = $volPrefix . "aggr2";
    my @aggregates;

    #check if there are atleast one aggregate on each node
    my @aggr_list;
    my %Nodes_aggr;
    foreach my $node (@Nodes_array) {
        $response = $Api_Set_Obj_ngsh->storage_aggregate_show(
            nodes => $node->{node},
            root  => "false",
        );
        my @output   = @{ $response->get_parsed_output() };
        my $aggr_cnt = scalar @output;
        if ( $aggr_cnt != 0 ) {
            $Test->nlog(
                "Found $aggr_cnt aggregate(s) on the node '$node->{node}'");
            push( @{ $Nodes_aggr{ $node->{node} } }, $output[0]->{aggregate} );
        }
        else {

            #create a new aggregate
            $Test->nlog("creating Aggregate '$aggregate'");
            NACL::STask::Aggregate->create(
                command_interface => $node,
                if_exists         => 'reuse',
                aggregate         => $aggregate,
            );
            push( @{ $Nodes_aggr{ $node->{node} } }, $output[0]->{aggregate} );
        } ## end else [ if ( $aggr_cnt != 0 )

    } ## end foreach my $node (@Nodes_array)
    if ( !_vserver_exists('vs0') ) {
        $vserver = $Prefix . 'vserver';
        $Test->nlog("creating vserver '$vserver'");
        NACL::C::Vserver->create(
            command_interface           => $Nodes_array[0],
            vserver                     => $vserver,
            rootvolume                  => "${vserver}_root",
            aggregate                   => $Nodes_array[0]->{node},
            'ns-switch'                 => 'file',
            'rootvolume-security-style' => 'mixed',
        );
    } ## end if ( !_vserver_exists ...
    for ( my $i = 0 ; $i < scalar @Nodes_array ; $i++ ) {
        $volume = $volPrefix . "vol1$i";
        $Test->nlog("creating volume '$volume' with junction-path '\/$volume'");

        my %other_opts = (
            'junction-path' => "/$volume",
            size            => '20m',
        );
        NACL::STask::Volume->create(
            command_interface => $Nodes_array[1],
            vserver           => $vserver,
            if_exists         => 'reuse',
            aggregate         => $Nodes_aggr{ $Nodes_array[1]->{node} }->[0],
            volume            => $volume,
            %other_opts,
        );
        _delete_volumes_in_vldb( $volume, $vserver );
    } ## end for ( my $i = 0; $i < scalar...
    return 1;
} ## end sub inject_discrepancies_in_partner_node

sub _cleanup {
    my $tmp_result = _deleteVolumes( "*", $volPrefix );
    $result = $tmp_result if ( $tmp_result != $TCD::PASS );

    #delete aggregates
    $tmp_result = _deleteAggr($volPrefix);
    $result = $tmp_result if ( $tmp_result != $TCD::PASS );

    #delete vserver
    $tmp_result = _deleteVserver($volPrefix);
    $result = $tmp_result if ( $tmp_result != $TCD::PASS );
} ## end sub _cleanup

sub _deleteVolumes {
    my ( $vserver, $Prefix ) = @_;
    my $error;

    #delete the volumes that were created by this test
    try {
        $response = $Api_Set_Obj_ngsh->volume_show(
            vserver => $vserver,
            volume  => "$Prefix*",
            vsroot  => 'false',
        );
        $output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $error = $exception_object->get_error_message();
    };
    if ( $error =~ /no entries/ ) {
        return $TCD::PASS;
    }
    my $result  = $TCD::PASS;
    my $vol_cnt = scalar @{$output};
    $Test->nlog("Found $vol_cnt volumes matching name:'${Prefix}*'");

    for ( my $i = 0 ; $i < $vol_cnt ; $i++ ) {
        my $volume  = $$output[$i]{volume};
        my $vserver = $$output[$i]{vserver};

        $Test->nlog("deleting volume $volume");
        NACL::STask::Volume->purge(
            command_interface => $Nodes_array[0],
            volume            => $volume,
            vserver           => $vserver,
        );
    } ## end for ( my $i = 0; $i < $vol_cnt...

    return $result;
} ## end sub _deleteVolumes

sub _deleteVserver {
    my $Prefix = shift;
    my $error;
    my $result = $TCD::PASS;
    try {
        $response = $Api_Set_Obj_ngsh->vserver_show( vserver => "$Prefix*", );
        $output = $response->get_parsed_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $error = $exception_object->get_error_message();
    };
    if ( $error =~ /no entries/ ) {
        return $TCD::PASS;
    }
    my $vserver_cnt = scalar @{$output};
    $Test->nlog("Found $vserver_cnt vserver(s) matching name:'${vserver}*'");

    for ( my $i = 0 ; $i < $vserver_cnt ; $i++ ) {
        my $vserver = $$output[$i]{vserver};

        $Test->nlog("deleting vserver $vserver");

        #delete root volume

        try {
            $Test->nlog("offlining and deleting root vol '${vserver}_root' ");
            $Api_Set_Obj_ngsh->volume_offline(
                vserver => $vserver,
                volume  => "${vserver}_root",
            );
            $Api_Set_Obj_ngsh->volume_delete(
                vserver => $vserver,
                volume  => "${vserver}_root",
            );
            $response =
              $Api_Set_Obj_ngsh->vserver_delete( vserver => $vserver, );
        } ## end try
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $Log->comment( "Text is :" . $exception_object->text() );
            $result = $TCD::FAIL;
        };
    } ## end for ( my $i = 0; $i < $vserver_cnt...
    return $result;
} ## end sub _deleteVserver

sub compare_vreport_list {
    my $first_array_ref  = shift;
    my $second_array_ref = shift;
    my @first_array      = @$first_array_ref;
    my @second_array     = @$second_array_ref;
    if ( scalar @first_array == scalar @second_array ) {
        $Test->nlog("vreport list count matched\n");
    }
    else {
        $Test->nlog("vreport list count did not match");
        return 0;
    }
    $Log->comment("comparing the values between vreport  and cache");
    for ( my $i = 0 ; $i < scalar @first_array ; $i++ ) {
        my $match = 0;
      INNER: for ( my $j = 0 ; $j < scalar @second_array ; $j++ ) {
            if ( $first_array[$i]->{type} eq $second_array[$j]->{type} ) {
                $Log->comment("$first_array[$i]->{type}");
                if ( $first_array[$i]->{object} eq $second_array[$j]->{object} )
                {
                    $Log->comment("$first_array[$i]->{object}");
                    $Log->comment("All matched");
                    $match = 1;
                }
                else {
                    $Log->comment(
"not matched :($first_array[$i]->{type} --> $second_array[$j]->{type})"
                    );
                    next INNER;
                } ## end else [ if ( $first_array[$i]->...
            }
            else {
                $Log->comment(
"not matched :$first_array[$i]->{object} --> $second_array[$j]->{object}"
                );
                next INNER;
            } ## end else [ if ( $first_array[$i]->...
        } ## end for ( my $j = 0; $j < scalar...
        if ( $match == 0 ) {
            $Log->comment(" didn't match");
            return 0;
        }
    } ## end for ( my $i = 0; $i < scalar...
    return 1;
} ## end sub compare_vreport_list
1;
