# -*- perl -*-
#  $Id;

## @summary waflStat library -- modification subroutines
##
## @description
##    This package defines routines which attempt to perform wafl stat*
##    operations that actually display wafl statistics.
##    just gathering information from the filer and returning it.
##
##
## @dependencies
##    lib/SayWrap.pm
##    FSR/FSRutils.pm
##
## @bugs
##
## @author dl-waflqa-automation@netapp.com
## @status design 

package waflStats;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(wafl_stats
		wafl_susp
		statit_b
		statit_e
		statit_en
		statit_er);
	 


use SayWrap;
use FSR::FSRutils;
use Tharn;
use Utils qw(make_conn_and_complain);
use strict;
use Carp qw(croak);
use SPARSE::Util::Logger qw( logWarn logTrc logStep logDbg logInfo );
use NaElement;


## @name ::wafl_stats
## @usage wafl_stats($filer)
## @synopsis
##   It display all wafl statistics. 
##    Returns success or failure.

sub wafl_stats($) {
	my ($fc) = @_;
	my $expected_output="WAFL statistics over";
	my $text = sayEOF($fc, "priv set test; wafl stats");
	if($text =~ /$expected_output/){
		return 1;}
	else{
		return 0;
  }
}
## @name ::wafl_susp
## @usage wafl_susp($fc)
## @synopsis
##   It display all wafl statistics.
##    Returns success or failure.

sub wafl_susp($){
	my ($fc) = @_;
        my $text = sayEOF($fc, "priv set test; wafl susp");
        if($text =~ /Found \d+ suspended message\(s\)\./){
                return 1;}
        else{
                return 0;
  }
}
## @name ::statit_b
##
## @description
##  It takes the beginning snapshots of performance stats.
##
## @synopsis
##  statit_b($conn)
##
## @arg $conn required: valid connection 
##
## @example
##  statit_b($conn);
##

sub statit_b($){
	my ($fc) = @_;
        my $text = sayEOF($fc, "priv set test; statit -b");
	if($text){
                return 0 ;}
        else{
                return 1;
  }
}

## @name ::statit_e
##
## @description
##  It takes the end snapshots of performance stats.
##
## @synopsis
##  statit_e($conn)
##
## @arg $conn required: valid connection 
##
## @example
##  statit_e($conn);
##

sub statit_e($){
        my ($fc) = @_;
	#Need to start statit -b to begin snap shot first
	my $text = sayEOF($fc, "priv set test; statit -b");
        my $text = sayEOF($fc, "priv set test; statit -e");
        if($text =~ /CPU Statistics/){
                return 1 ;}
        else{
                return 0;
  }
}
## @name ::statit_en
##
## @description
##  It appends nfsstat -c output to the report.
##
## @synopsis
##  statit_en($conn)
##
## @arg $conn required: valid connection 
##
## @example
##  statit_en($conn);
##

sub statit_en($){
        my ($fc) = @_;
        #Need to start statit -b to begin snap shot first
        my $text = sayEOF($fc, "priv set test; statit -b");
        my $text = sayEOF($fc, "priv set test; statit -en");
        if($text =~ /NFS Detail Statistics/){
                return 1 ;}
        else{
                return 0;
  }
}
## @name ::statit_er
##
## @description
##  It reports wafl statistics  counts over intervals instead of per second rates. 
##
## @synopsis
##  statit_er($conn)
##
## @arg $conn required: valid connection 
##
## @example
##  statit_er($conn);
##

sub statit_er($){
        my ($fc) = @_;
        #Need to start statit -b to begin snap shot first
        my $text = sayEOF($fc, "priv set test; statit -b");
        my $text = sayEOF($fc, "priv set test; statit -er");
        if($text !~ /per second/){
                return 1 ;}
        else{
                return 0;
  }
}


1;
