# -*- perl -*-
#  $Id: //depot/prod/ontap/main/test/lib/waflobjCreator#2 $

## @summary Library routines to create different kinds of wafl file types
## @author  neelima@netapp.com
## @status  testme
##


package waflobjCreator; 
use strict;
use mount qw(mount);
use MapShare qw(map_cifs_share);
use SAN;
#use SAN qw(create_lun);
use QtreeCmds;
use Module::Common qw(validate_mixed);
use Params::Validate
	qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF);
use base qw(Exporter);
use Tharn;
use Utils::Units qw(:convert);
use Utils qw(make_conn_and_complain);
use filer qw(filer_file_make);
use misc qw(cli_localhost);

our $Validation; 

BEGIN
{
	$Validation = 1;
	my @unix_subs   = qw(
		create_directory
		check_directory
		create_fifo
	        create_symlink
		create_hardlink
		create_dev
		create_file
		check_file
		check_filesize
		change_mode
		rename_obj
		delete_object
		create_socket
		create_lun
		create_deep_directory
		create_wide_directory

	);
	my @windows_subs = qw(
		get_acls
		create_acls
		create_ntstream
		verify_ntstream
		create_fileonWin
		create_dironWin
		share_driveonNT
		create_cifsshare
		delete_cifsshare
		delete_NTobject
		delete_NTfile
		connectWinClient
	);
	
	my @filer_subs = qw(
		create_level_file
	);

	#These need to be deprecated, please don't use, if used, please change to
	#using vvol.pm or qtreeCmds.pm 
	my @deprecated_subs = qw(
		check_vol
		create_vol
		check_qtree
		create_qtree
		set_security_qtree

	);
	our %EXPORT_TAGS = (
		unix     => [@unix_subs],
		windows  => [@windows_subs],
		filer    => [@filer_subs],
		deprecate => [@deprecated_subs]
	);
	$EXPORT_TAGS{all} = [ map{ @{$_} } values %EXPORT_TAGS ];
	Exporter::export_ok_tags('all');

}

## @name ::create_directory

## @usage create_directory ($dir,%args);

## @description This routine will create a directory $dir with 

##              permissions 0777 by default

## @arg mode Optional : mode of the directory

##              Default is 0777

## @arg client Optional: Defaults to localhost

## @returns (returns 1 if succeeds else dies with error in $@)

sub create_directory {
	my($dir, %args) = validate_mixed(
		@_,
		[ { type => SCALAR } ],
		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
			mode => { type => SCALAR, optional => 1, default => 0777 },
		        timeout => { type => SCALAR, optional => 1, default => 3600 }
		        
		}
	);
	return eval{
		 my $testobj= subtest('{
			my($dir, $mode) = @_;
			my $dirre = quotemeta($dir);
			#using info, since do not want to log failure 
			mkdir($dir,$mode) or return logresult(type=>"INFO",abort=>1, "$!");
			return logresult(type=>"PASS");
		}',
		'-runid',"create_dir",
		'-client',$args{client},
		'-timeout',$args{timeout},
		'-onexit',sub{}, #don't pass results to parent
		'--',$dir, $args{mode}
		);
		parse_results($testobj) or die("$@");
		if($Validation){
			check_directory($dir,client=>$args{client}) or die("$@");
		}
		return(1);
       };
}	

## @name ::check_directory

## @usage check_directory ($dirpath,client=>"tada-c11");

## @description This routine will check for the existence of a directory

## @arg client Optional: Defaults to localhost

## @returns (returns 1 if succeeds else returns 0)

sub check_directory{
	my($dir, %args) = validate_mixed(
		@_,
		[ { type => SCALAR } ],
		{
		 client => { type => SCALAR, optional  => 1, default => 'localhost'},
		 timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	return eval{
		my $testobj= subtest('{
			if(-d $_[0]){ 
				return logresult(type=>"PASS" , "Directory exists");
			}else{
			      	return logresult(type=>"INFO","Not a directory/Directory does not exist");
			}}',  
			'-runid',"checkdir",
			'-client',$args{client},
                        '-timeout',$args{timeout},
			'-onexit',sub{},
			'--',$dir);
		#parse subtest results
		parse_results($testobj) or return(0);
		return(1);
	};
}

## @name ::create_fifo

## @usage create_fifo($filepath,client=>$client)

## @description This routine will create a pipe

## @arg mode Optional: Defaults to 0777

## @arg client Optional: Defaults to localhost

## @returns (1 if succeeds else dies with error in $@)

sub create_fifo {
	my($filepath, %args) = validate_mixed(
		@_,
		[ { type => SCALAR } ],
		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
		        mode => { type => SCALAR, optional => 1, default => 0777 },
                        timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	return eval{
		my $testobj= subtest('{
			use POSIX qw(mkfifo);
			my($filepath, $mode)=@_;
			if(-p $filepath){
				return logresult(type=>"INFO" , "pipe exists");
			}elsif (-e $filepath) {
				return logresult(type=>"INFO","File exists but  not a FIFO \"named pipe\".");
			}else {
			mkfifo($filepath, $mode) or return logresult(type=>"INFO","cannot create a named pipe $filepath(undef from mkfifo)"); 
			return logresult(type=> "PASS"); 
		}}',           
		'-runid',"fifo",
		'-client',$args{client},
		'-timeout',$args{timeout},
		'-onexit',sub{},
		'--',$filepath,$args{mode});
		#parse subtest results
		parse_results($testobj) or die("$@");
		return(1);
	};  	
}

## @name ::create_symlink

## @usage create_symlink($filepath,$symlinkpath,client=>$client)

## @description This routine will create a symlink to a file

## @arg client Optional: Defaults to localhost

## @returns (1 if succeeds else dies with error in $@)

sub create_symlink {
	my($filepath,$symlinkpath, %args) = validate_mixed(
		@_,
		[ { type => SCALAR },{ type => SCALAR } ],
		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
         	        timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	return eval{
		my $testobj= subtest('{
			my($filepath, $symlinkpath)=@_;
			if(-l $symlinkpath){
				return logresult(type=>"INFO" , "symlink exists");
			}elsif (! -e $filepath) {
				return logresult(type=>"INFO","src file to create symlink does not exist");
			}else {
				symlink($filepath, $symlinkpath) or return logresult(type=>"INFO","Could not create a symlink $symlinkpath $!"); 
				return logresult(type=> "PASS","created a symlink"); 
			}
			}',           
			'-runid',"symlink",
			'-client',$args{client},
			'-onexit',sub{},		
			'-timeout',$args{timeout},
			'--',$filepath,$symlinkpath);
		#parse subtest results
		parse_results($testobj) or die("$@");
		return(1);
	};  	
}



## @name ::create_socket

## @usage create_socket($sockpath,client=>$client)

## @description This routine will create  a socket

## @arg client Optional: Defaults to localhost

## @returns (1 if succeeds else dies with error in $@)

sub create_socket {
	my($sockpath, %args) = validate_mixed(
		@_,
		[ { type => SCALAR } ],
		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
         	        timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	return eval{
		my $testobj=subtest('{
			use Socket;
			unlink $_[0];
			socket(SERVER,PF_UNIX,SOCK_STREAM,0)
				or return  logresult(type=>"INFO", "Cannot create a socket or already existing: $!");
			bind(SERVER,sockaddr_un("$_[0]"))
				or return  logresult(type=>"INFO", "Cannot bind  socket or already existing: $!");
			return logresult(type=>"PASS", "Socket file created");
			}',
			'-runid',"createsocket",
			'-client',$args{client},
			'-timeout',$args{timeout},
			'-onexit',sub{}, 
			'--',$sockpath);
		#parse subtest results
		parse_results($testobj) or die("$@");
		return(1);
	};	 
}

## @name ::create_hardlink

## @usage create_hardlink($sockpath,client=>$client)

## @description This routine will create  a link

## @arg $filepath Required: src file to create a link

## @arg $hardlinkpath Required: link path
## If 'count' is passed, $hardlinkpath will be used as the basename for hardlink creation

## @arg client Optional: Defaults to localhost

## @arg count Optional: Number of hardlinks to create, defaults to 1. Uses $hardlinkpath as base name and appends numbers 0 to $count-1 to the hardlink

## @returns (1 if succeeds else dies with error in $@)

sub create_hardlink {
	my($filepath, $hardlinkpath, %args) = validate_mixed(
	#my($filepath, $hardlinkpath, $count, %args) = validate_mixed(
		@_,
		[ { type => SCALAR },{ type => SCALAR } ],
		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
                        count  => { type => SCALAR, optional  => 1, default => 1},
         	        timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	return eval {
		my $testobj= subtest('{
                        use misc qw(cli_localhost);
			my($filepath, $hardlinkpath, $count)=@_;
			if (! -e $filepath) {
				return logresult(type=>"INFO","src file to create symlink does not exist");
			}else {
                                if ($count > 1) {
                                    for(my $i = 0; $i < $count; $i++) {
				        link($filepath, $hardlinkpath.$i) or return logresult(type=>"INFO","Could not create a hardlink $hardlinkpath$i $!"); 
                                    }
                                    my $output = cli_localhost("ls -l $filepath"); 
                                    logcomment($output);
				    return logresult(type=> "PASS","created $count hardlinks"); 
                                } else {
				    link($filepath, $hardlinkpath) or return logresult(type=>"INFO","Could not create a hardlink $hardlinkpath $!"); 
				    return logresult(type=> "PASS","created a hardlink"); 
                                }
			}
			}',           
			'-runid',"hardlink",
			'-client',$args{client},
			'-timeout',$args{timeout},
			'-onexit',sub{},		
			'--',$filepath,$hardlinkpath,$args{count});
			#parse subtest results
			parse_results($testobj) or die("$@");
			return(1);
	};  	
}

## @name ::create_dev

## @usage create_dev($chardev_path,1,1,client=>$client)

## @usage create_dev($chardev_path,1,2,type=>"b");
## @description This routine will create character device file

## @arg $filepath Required: src file to create a link

## @arg $majordev Required: link path

## @arg $minordev Required: Defaults to localhost

## @arg type Optional:  type=>'c' creates a char special file
##                      type=>'b' creates a block special file

## @arg client Optional: Defaults to localhost
##                      client=>'tada-c11' will create from this client

## @returns (1 if succeeds else dies with error in $@)

## @dependencies Works only on unix kind of filesystems.
##              Right now this works only on linux and
##              solaris clients.

##              

sub create_dev {
	my($filepath, $majordev, $minordev, %args) = validate_mixed(
		@_,
		[ { type => SCALAR },{ type => SCALAR },{ type => SCALAR }  ],
		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
			type => {type => SCALAR, optional => 1, default =>'c'},
			mode => {type => SCALAR, optional =>1, default =>0777},
         	        timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	return eval {
		my $hclient= host($args{client});
		my $mknod;
		if($hclient->hosttype =~ /linux/){
			$mknod = "/bin/mknod";
		}elsif($hclient->hosttype =~ /solaris/){
			$mknod = "/usr/sbin/mknod";
		}else {
			$mknod ="mknod";
		}
		my $cclient= connect('#rlogin','inherit=0');
		my $cmd = "$mknod $filepath $args{type} $majordev $minordev";   
		say($cclient,"sudo $cmd && echo _''OK_","_OK_",'-eval_error','-timeout',$args{timeout});
		die "Command \"$cmd\" failed: $@" if $@;
		return(1);
	};
}
	

## @name ::delete_object

## @usage delete_object($objectPath,client=>$client);

## @description This routine deletes a dir/file on a Unix machine

## @arg purge Optional: purge=>1, (uses delete_loacl_tree which deletes all the files
##                                 in the directory and deletes the dir)
##                      purge=>0 (uses rmdir which will delete the dir
##                                only if it is empty)
##                      Defaults to purge=>1

## @arg client Optional: Defaults to localhost
##                      client=>'tada-c11' will create from this client

## @returns (1 if succeeds else dies with error in $@)

sub delete_object {
	my($objpath, %args) = validate_mixed(
		@_,
		[ { type => SCALAR } ],

		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
			purge => {type => SCALAR, optional => 1, default => 1}, # default: tries to remove the directories or files in the objpath
         	        timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	return eval {
		my $testobj= subtest('{
		use DataManip::local qw(delete_local_tree);
		my($objpath,$purge)=@_;
                my $result =0 ;
		if (! -e $objpath) {
			return logresult(type=>"INFO","$objpath does not exist");
		} else {
			if($purge){
				delete_local_tree($objpath); 
                                if($@){
					return logresult(type=>"INFO",abort=>1,"$!"); 
				}	
			}else{
				rmdir($objpath);
				if($!){
					return logresult(type=>"INFO",abort=>1,"$!"); 
				}	
			}
	                        return logresult(type=> "PASS","removed the directory"); 
		}}',
		'-runid',"rmdir",
		'-client',$args{client},
		'-onexit',sub{},		
		'-timeout',$args{timeout},
		'--',$objpath,
		$args{purge});
		#parse subtest results
		parse_results($testobj) or die("$@");
		return(1);
	};
}

## @name ::change_mode

## @usage change_mode($filepath,$mode, client=>$client);

## @description This routine changes the mode of the file/dir

## @arg client Optional: Defaults to localhost
##                      client=>'tada-c11' will use this client

## @returns (1 if succeeds else dies with error in $@)
sub change_mode {
	my($filepath, $mode, %args) = validate_mixed(
		@_,
		[ { type => SCALAR },{ type => SCALAR }  ],

		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
         	        timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	return eval {
		my $testobj= subtest('{
		my($filepath, $mode)=@_;
		if (! -e $filepath) {
			return logresult(type=>"INFO",abort=>1,"file does not exisit");
		} else {
			chmod($mode, $filepath) or return logresult(type=>"INFO","$!"); 
			return logresult(type=> "PASS","changed mode"); 
		}}',           
		'-runid',"chmod",
		'-client',$args{client},
		'-onexit',sub{},		
		'-timeout',$args{timeout},		
		'--',$filepath,$mode);
		#parse subtest results
		parse_results($testobj) or die("$@");
		return(1);
	};
}  
		
## @name ::rename_obj

## @usage rename_obj($src,$dst,client=>$client);

## @description This routine changes the name of the src to dst

## @arg client Optional: Defaults to localhost
##                      client=>'tada-c11' will use this client

## @returns (1 if succeeds else dies with error in $@)

sub rename_obj {
	my($src, $dst, %args) = validate_mixed(
		@_,
		[ { type => SCALAR },{ type => SCALAR }  ],

		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
         	        timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	return eval {
		my $testobj= subtest('{
		my($src, $dst)=@_;
		if (! -e $src) {
			return logresult(type=>"INFO","src does not exisit");
		} else {
			rename($src, $dst) or return logresult(type=>"INFO","$!"); 
			return logresult(type=> "PASS","renamed $src"); 
		}}',           
		'-runid',"rename",
		'-client',$args{client},
		'-onexit',sub{},		
		'-timeout',$args{timeout},		
		'--',$src,$dst);
		#parse subtest results
		parse_results($testobj) or die("$@");
		return(1);
	};

}

## @name ::create_file()
##
## @description Creates a file of specified size 
## 
## @arg $filepath Required : absolute path of the file to be created
## 
## @arg $size Required : size of the file to be created.
##                       size can be in(b,k,m,g,t) 
##                        50m -50 mega bytes file
##                        10k -10 kilo bytes file
##                         
##
## @arg offset Optional: Will write random chars from offset byte of the file
##                 offset=>50 , first 50 bytes are holes and after that 
##                 random chars will be written to the file.
##                 Defaults to offset=>0
##
## @arg client Optional: Defaults to localhost
##                      client=>'tada-c11' will use this client
##
## @example create_file($filepath, $size, %options)
##
## @returns 1 on success, error in $@
#  (Original author: Remi)

sub create_file {
	my($filepath, $size, %args) = validate_mixed(
		@_,
		[ { type => SCALAR },{ type => SCALAR }  ],

		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
			offset => { type => SCALAR, optional  => 1, default => 0},
         	        timeout => { type => SCALAR, optional => 1, default => 7200 }

		}
	);
	return eval {
		my $size_in_bytes = convert_to_bytes($size) or die("$@");
		my $testobj= subtest('{
			use DataManip::local qw(writeFile);
			my($filepath, $size_in_bytes, $offset)=@_;
			if (-e $filepath) {
				return logresult(type=>"INFO","File exists");
			}
                        logcomment("Creating file at $filepath  of size $size_in_bytes");
			writeFile($filepath, $size_in_bytes, OFFSET => $offset) 
				or return logresult(type=>"INFO","$!");
                         return logresult(type=> "PASS","created  file $filepath"); 
		}',   
		'-runid',"createfile",
		'-client',$args{client},
		'-timeout',$args{timeout},				    
		'-onexit',sub{},		
		'--',$filepath,
		$size_in_bytes,
		$args{offset}
		);
		#parse subtest results
		parse_results($testobj) or die("$@");
		return(1);
	};
}

## @name ::check_file
##
## @usage check_file($filepath,client=>$client);
##
## @description This routine checks if the file exists or not
##
## @arg client Optional: Defaults to localhost
##                      client=>'tada-c11' will use this client

## @returns (1 if succeeds else 0)

sub check_file {
	my($filepath, %args) = validate_mixed(
		@_,
		[ { type => SCALAR }  ],

		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},

         	        timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	return eval{ 
		my $testobj=subtest('{
			if(-e $_[0]){
				return addresult(type=>"PASS" , "File exists");
			}else{
				return addresult(type=>"INFO","File not found");
			}
			}',
			'-runid',"checkfile",
			'-client',$args{client},
			'-timeout',$args{timeout},
			'-onexit',sub{},
			'--',$filepath);
		#parse subtest results
		parse_results($testobj) or return(0);
		return(1);

	};
}

## @name ::check_filesize
## @usage check_filesize($filepath,$filesize,client=>$client);

## @arg $filepath (Required):path to the file which needs to be checked.

## @arg $filesize (Optional):size of the file that needs to be checked against
## @arg client Optional: Defaults to localhost
##                      client=>'tada-c11' will use this client

## @description This routine checks the size of the file

## @returns (1 if succeeds else dies)

sub check_filesize{
	my($filepath, $filesize, %args) = validate_mixed(
		@_,
		[ { type => SCALAR },{ type => SCALAR }  ],

		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
         	        timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	$filesize=convert_to_bytes($filesize);
	return eval{ 
		my $testobj=subtest('{ 
			my $present_size = (-s "$_[0]");
			my $verify_size = $_[1];
			if($present_size == $verify_size ){
				return addresult(type=>"PASS" , "File size is right");
			}else{
				return addresult(type=>"INFO","File size incorrect Current size is $present_size, Expected size is $verify_size in bytes");
			}
			}',
			'-runid',"checksize",
			'-client',$args{client},
			'-timeout',$args{timeout},			
			'-onexit',sub{},		
			'--',$filepath,$filesize);
	 	#parse subtest results
		parse_results($testobj) or die("$@");
		return(1);
	};
}




#Subroutines Used locally
#parses the subtest results
sub parse_results{
	my ($subtestObj) = @_;
	my $resultstr = "";
	return eval {
		my @results = $subtestObj->own_results();
		foreach my $result(@results){
			my $type = $result->type();
			if ($type =~ /PASS/) {
			    return(1);
			} else {
			    $resultstr .= join(' ',$result->msgs());
			}
		    }
		die "$resultstr";
	    };
}

## @name ::create_cifsshare

## @usage create_cifsshare($filer,$share_name,$share_path)

## @description This routine will create a cifs share

## @returns (1 if succeeds else dies with error in $@)

sub create_cifsshare {	
	my $cfiler = make_conn_and_complain(shift);
	my($share_name, $share_path, %args) = validate_mixed(
		@_,
		[{ type => SCALAR },{ type => SCALAR }  ],
		{timeout => { type => SCALAR, optional => 1, default => 3600 }
}
	);
	return eval{
		my (undef, undef, $after)=say($cfiler,"cifs shares -add $share_name $share_path",'-timeout',$args{timeout});
		if (length($after)) { 
			die $after;
		} else {
			return(1);
		}
	};    
}

## @name ::delete_cifsshare

## @usage delete_cifsshare($filer,$share_name)

## @description This routine will delete cifs share

## @returns (1 if succeeds else dies with error in $@)

sub delete_cifsshare {
	my $cfiler = make_conn_and_complain(shift);
	my($share_name,%args) = validate_mixed(
		@_,
		[ { type => SCALAR }  ],
		{timeout => { type => SCALAR, optional => 1, default => 3600 }
		}
	);
	return eval{
		my (undef, undef, $after)= say($cfiler,"cifs shares -delete $share_name",'-timeout',$args{timeout});
		if (length($after)) {
		    die $after;
		} else {
		    return(1);
		}
	};    
}

## @name ::create_ntstream

## @usage create_ntstream($share_drive,$streamfile,\@streams,$win_client 
##			  [,writestr=>$string])

## @description This routine creates files with streams

## @returns (1 if succeeds else dies with error in $@)

## @dependencies rshd needs to be installed on $win_client 

sub create_ntstream
{
	my($share_drive,$streamfile,$streams,$win_client, %opts) = @_;
	my($winConn,$rsh_timeout)=connectWinClient($win_client);
	my $stream_path=$share_drive . "\\" . $streamfile;
	my $string = $opts{writestr};

	return eval{
		foreach my $stream(@$streams) {
			logcomment("$stream");
			$string = "This is a Stream"
				unless defined($string);
			my $cmd="echo $string $stream > $stream_path:$stream";
			my (undef,undef,$after) = say($winConn,
						      "cmd.exe /c $cmd",
						      '-timeout',$rsh_timeout);
			if(length($after)){
				die($after);
			}
		}
		return (1);
	};
}

## @name ::verify_ntstream

## @usage verifyntstream($share_drive,$streamfile,\@streams,$win_client 
##			  [,readstr=>$string])

## @description This routine reads streams associated with NT files

## @returns (1 if succeeds else dies with error in $@)

## @dependencies rshd needs to be installed on $win_client 

sub verify_ntstream
{
	my($share_drive, $streamfile, $streams, $win_client, %opts) = @_;
	$win_client = "localhost"
		unless defined($win_client);
	my($winConn,$rsh_timeout) = connectWinClient($win_client);
	my $stream_path = $share_drive . "\\" . $streamfile;
	my $string = $opts{readstr};

	$string = "This is a Stream"
		unless defined($string);

	foreach my $stream(@$streams) {
		my $cmd = "type $stream_path:$stream";
		my (undef, undef, $after) = say($winConn, "cmd.exe /c $cmd",
						'-timeout', $rsh_timeout);	
		
		logcomment("after=$after");
		die "expected string doesn't match the stream data"
			if ($after !~ /$string/);
	}
	return 1;
}


## @name ::share_driveonNT

## @usage share_driveonNT($cifs_share,$filer_ip,$win_client,$cifs_user,$cifs_password);

## @description This routine maps the  share to PC

## @returns (1 if succeeds else dies with error in $@)

## @dependencies rshd needs to be installed on $win_client 

## @notes this needs to be deprecated, just use map_cifs_share

sub share_driveonNT($$$$$%){
    my($cifs_share,$filer_ip,$win_client,$cifs_user,$cifs_password,%args)=@_;
    return eval{
	my $share= map_cifs_share(client=>$win_client, 
                            server=>$filer_ip, 
                            share=>$cifs_share,
                            workgroup=>undef,
                            username=>$cifs_user,
                            password=>$cifs_password,
                            timeout=>$args{timeout}) 
            or die("Unable to map the share '$cifs_share' : $@");
	return($share);
    };
}                       

## @name ::create_lun

## @usage create_lun($filer,$lunpath,$client,format=>0);

## @description This routine create a lun

## @returns (1 if succeeds else dies with error in $@)

## @notes This needs to be deprecated, and just use SAN::create_lun

sub create_lun($$$;%){
	my($filer,$lunpath,$client,%args)=@_;
	my @lun_dev;
	my $format= defined($args{format}) ? $args{format} : '1';

	if( defined($args{lun_size})){
		@lun_dev=SAN::create_lun($filer,$client, protocol=>'fcp',
					lun_path=>$lunpath,lun_size=>$args{lun_size},format=>$format);
	}else{
		@lun_dev=SAN::create_lun($filer,$client,protocol=>'fcp',lun_path=>$lunpath,
                    format=>$format);
	}
	#pick the first raw lun device 
	return($lun_dev[0]);
}

## @name ::create_acls

## @usage create_acls($aclfile,$aclchangeStr,$win_client);

## @description This routine sets acls to the file

## @returns (1 if succeeds else dies with error in $@)

## @dependencies rshd needs to be installed on $win_client 

sub create_acls{
	my($aclfile,$aclchangeStr,$win_client,%args) = @_;
	return eval{
		my($winConn,$rsh_timeout)=connectWinClient($win_client);
		my $timeout = defined($args{timeout}) ? $args{timeout} : $rsh_timeout;
		my $cmd="cacls $aclfile $aclchangeStr";
		my (undef,undef,$after)= say($winConn,"cmd.exe /c $cmd",'-timeout',$timeout);
		if ($after !~ /processed/){
			die($after);
		}
		return(1);
	};
}

## @name ::get_acls

## @usage get_acls($aclfile,$win_client, %args);

## @description Command executes 'cacls' command on windows client
##              and returns the output to the user.

## @returns (command output if succeeds else dies with error in $@)
##
##           If $args{want} and/or $args{dont_want} are defined,
##           returns 1 if output contains $args{want} and doesnt contain
##           $args{dont_want}. Otherwise dies.

## @dependencies rshd needs to be installed on $win_client

## @notes User can process output, or this function can do it.
sub get_acls{
	my ($aclfile, $win_client, %args) = @_;
	return eval {
		my($winConn,$rsh_timeout)=connectWinClient($win_client);
		my $timeout = defined($args{timeout}) ? $args{timeout} : $rsh_timeout;
		my $cmd="cacls $aclfile";
		my (undef, undef, $output) = say($winConn, "cmd.exe /c $cmd", '-timeout', $timeout);
		if (defined($args{want}) || defined($args{dont_want})) {
		# we process result
			return _process_output($output, %args);
		}
		# caller processes output themselves
		return $output;
	};
}


## @name ::_process_output

## @usage _process_output($output, %args);

## @description Checks $output for presense $args{want} (if defined)
##              and for absense of $args{dont_want} (if defined)

## @returns (1 if succeeds else dies with error in $@)
sub _process_output{
	my ($output, %args) =@_;
	return eval {
		if ($args{want}) {
			if ($output !~ $args{want}) {
				die("the following output did not have string"
				    ." \' $args{want} \': $output");
			}
		}
		if ($args{dont_want}) {
			if ($output =~ $args{dont_want}) {
				die ("the following output had string"
				     ." \' $args{dont_want} \' : $output");
			}
		}
		return 1;
	};
}


## @name ::create_dironWin

## @usage create_dironWin($dirpath,$win_client);

## @description This routine create a directory on the PC

## @returns (1 if succeeds else dies with error in $@)

## @dependencies rshd needs to be installed on $win_client 
##TDB: Use create_directory instead and change the scripts or libs which use this 
## routine 
sub create_dironWin{
	my ($dirpath,$win_client,%args)=@_;
	my($winConn,$rsh_timeout) = connectWinClient($win_client);
	my $timeout = defined($args{timeout}) ? $args{timeout} : $rsh_timeout;
	return eval{
		my(undef,undef,$after)=say($winConn,"cmd.exe /c mkdir $dirpath",'-timeout',$timeout);
		if (length($after)) {
			die $after;
		} else {
			return(1);
		}
	};
}

## @name ::create_fileonWin

## @usage create_fileonWin($filepath,$win_client[,writestr=>$string]);

## @description This routine create a file on the PC

## @returns (1 if succeeds else dies with error in $@)

## @dependencies rshd needs to be installed on $win_client 
## TBD: Use create_file instead and change the scripts or libs which 
## use this routine 
sub create_fileonWin{
	my ($filename,$win_client,%args)=@_;
	my $string = $args{writestr};
	return eval{
		$string = "This is a file"
			unless defined($string);
		my $cmd = "echo $string > $filename";
		my($winConn,$rsh_timeout) = connectWinClient($win_client);
		my $timeout = defined($args{timeout}) ? $args{timeout} : $rsh_timeout;
		my(undef,undef,$after)=say($winConn,"cmd.exe /c $cmd",'-timeout',$timeout);
		if (length($after)) {
			die $after;
		} else {
			return(1);
		}
	};
}

#returns a connection object
sub connectWinClient{
	my $win_client=shift;
	my $ch = host($win_client);
	my $cc = connect("#rsh");
	my $rsh_timeout = ($cc->{timeout} < 6000) ? 6000 : $cc->{timeout};
	return($cc,$rsh_timeout);
} 

## @name ::delete_NTobject

## @usage delete_NTobject($objectPath,$win_client);

## @description This routine deletes a dir on the PC

## @returns (1 if succeeds else dies with error in $@)

## @dependencies rshd needs to be installed on $win_client 

sub delete_NTobject{
	my($objectPath,$win_client,%args)=@_;
	my($winConn,$rsh_timeout) = connectWinClient($win_client);
	return eval{
		my($winConn,$rsh_timeout) = connectWinClient($win_client);
		my $timeout = defined($args{timeout}) ? $args{timeout} : $rsh_timeout;
		my(undef,undef,$after)=  say($winConn,"cmd.exe /c rmdir /s /q $objectPath",'-timeout',$timeout);
		if (length($after)) {
			die $after;
		} else {
			return(1);
		}
	};
}

## @name ::delete_NTfile

## @usage delete_NTfile($objectPath,$win_client);

## @description This routine deletes a dir/file on the PC

## @returns (1 if succeeds else dies with error in $@)

## @dependencies rshd needs to be installed on $win_client 

sub delete_NTfile{
	my($objectPath,$win_client,%args)=@_;
	my($winConn,$rsh_timeout) = connectWinClient($win_client);
	return eval{
		my($winConn,$rsh_timeout) = connectWinClient($win_client);
		my $timeout = defined($args{timeout}) ? $args{timeout} : $rsh_timeout;
		my(undef,undef,$after)=  say($winConn,"cmd.exe /c del /q $objectPath",'-timeout',$timeout);
		if (length($after)) {
			die $after;
		} else {
			return(1);
		}
	};
}

## @name ::create_deep_directory

## @usage create_deep_directory($dirpath,client=>$client,depth=>'3');

## @description This routine will create a deep level directory in the 
##              specified path

## @arg depth  : no of levels of directories.
## @arg client Optional: Defaults to localhost
##                      client=>'tada-c11' will use this client
## TBD:This routine does not work as expected, when it reaches the limit of 256 characters 
## logic needs to be changed so that the directory creation is broken down and we 'cd' to the
## depth of directory and create the new directory w.r.to depth specified by the user. 
##@returns (1 if setting succeeds else dies with error in $@)

sub create_deep_directory  {
	my($dirpath, %args) = validate_mixed(
		@_,
		[ { type => SCALAR } ],
		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
			depth => { type => SCALAR, optional => 1, default => '10' },
		        timeout => { type => SCALAR, optional => 1, default => 3600 }


		}
	);
	return eval {
		create_directory($dirpath,client=>$args{client},timeout=>$args{timeout}) or die("$@");
		my $level = "dlevel";
		for (my $i=0;$i<$args{depth};++$i) {
			$level = $level . $i . "/";
			create_directory("$dirpath/$level",client=>$args{client},timeout=>$args{timeout}) or die("$@");
		}
		return(1);
	};
}

## @name ::create_wide_directory

## @usage create_wide_directory($dirpath ,client=>$client,width=>$width);

## @description This routine will create a $width directories in one level

## @arg client Optional: Defaults to localhost
##                      client=>'tada-c11' will use this client
##
## @returns (1 if succeeds else dies with error in $@)
sub create_wide_directory {
	my($dirpath, %args) = validate_mixed(
		@_,
		[ { type => SCALAR } ],
		{
			client => { type => SCALAR, optional  => 1, default => 'localhost'},
			width => { type => SCALAR, optional => 1, default => '10' },
		        timeout => { type => SCALAR, optional => 1, default => 3600 }

		}
	);
	return eval {
		create_directory($dirpath,client=>$args{client},timeout=>$args{timeout}) or die("$@");
		for (my $i=0;$i<$args{width}; ++$i) {
			create_file("$dirpath/file$i","1k",client=>$args{client},timeout=>$args{timeout}) or die("$@");
		}
		return(1);
	};
}

## @name ::create_level_file
## @usage create_level_file(coon=>filer_conn,
##			pathname=><filename>,
##			vol_type=><flex or trad>
##			level=><indirect level, Optional, default 1)>,
##			extra=><additional bytes to add, optional, default 0>
##			)
## @description This routine uses the filer cmd 'mkfile' to create a file
##	at pathname, with the specified indirect level and extra bytes
##	Max level is restricted to 3 to prevent large files from being created
## @returns (1 if successful else dies with error in $@)

use constant FLEX_INODE_INDIRECT_COUNT => 8;
use constant FLEX_INDIRECT_COUNT => 510;
use constant TRAD_INODE_INDIRECT_COUNT => 16;
use constant TRAD_INDIRECT_COUNT => 1024;
use constant BLOCK_SIZE => 4096;

sub create_level_file {
	my (%args) = validate(@_,
	{	conn => { type => HASHREF, optional => 0 },
		pathname => { type => SCALAR, optional => 0 },
		vol_type => { type => SCALAR, optional => 0,
					regex => qr/flex|trad/ },
		level => { type => SCALAR, optional => 1, default => 1 },
		extra => { type => SCALAR, optional => 1, default => 0 }
	} );

    die "must be between 0 and 3"
	if $args{level} < 0 || $args{level} > 3; # Level 0 to 3 only

    my $fsize = 0;
    if ( $args{level}) {
	my $cnt = 1;
	$fsize = ($args{vol_type} eq "flex"
			? FLEX_INODE_INDIRECT_COUNT
			: TRAD_INODE_INDIRECT_COUNT
		) * BLOCK_SIZE;
	while ( $cnt++ < $args{level} ) {
		$fsize *= ($args{vol_type} eq "flex"
				? FLEX_INDIRECT_COUNT
				: TRAD_INDIRECT_COUNT );
	}
    };
    $fsize += $args{extra};

    my $info = "level $args{level}, size $fsize, file $args{pathname}";
    logcomment("Creating $info");
    die "File creation failed for $info"
	unless filer_file_make($args{conn}, $args{pathname}, $fsize);
    logcomment("File created successfully");
    return 1;
}



## @name ::check_vol
## @usage check_vol($filer, $vol)
## @description - check if vol exists
## @returns (1 if vol exists, 0 otherwise)
## @notes (DEPRECATED, instead use Volume.pm)

sub check_vol($$) {
	my($filer,$vol)=@_;
	host($filer);
	my $cfiler = connect("#rsh");
	my(undef,undef,$after)=say($cfiler,"vol status $vol");
	if ($after =~ /vol\s+status:\s+No\s+volume\s+named\s+.*\s+exists/) {
		return 0;
	}else{
		return 1;
	} 
}

## @name ::create_vol
## @usage create_vol($filer, $vol, $no_of_disks)
## @description This routine will create the volume, and export the created volume 
## @returns (1 if vol create and export succeed, die otherwise)
## @dependencies The filer has to have atleast $noofdisks spare disks
## @notes (DEPRECATED, instead use Volume.pm)

sub create_vol($$$;%) {
	my ($filer,$vol,$noofdisks)= @_;
	my $h=host($filer);
	my $cfiler = connect("#rsh");
	return eval{
		vol_create($filer, $vol, options=>$noofdisks) or  die($@);
		my(undef,undef,$after)=say($cfiler,"exportfs -o anon=0 /vol/$vol");
		if(length($after)){
			die($after);
		}
		return 1;
	};
}

## @name ::check_qtree
## @usage check_qtree($filer, $vol, $qtree)
## @description This routine will check if qtree exists
## @returns (1 if qtree exists, 0 otherwise)
## @notes (DEPRECATED, instead use QtreeCmds.pm)

sub check_qtree($$$) {
	my($filer,$vol,$qtree)=@_;
	host($filer);
	my $cfiler = connect("#rsh");
	my(undef,undef,$after)=say($cfiler,"qtree status $vol");
	my @lines = split(/\n/ , $after);
	for my $line (@lines) {
		if ($line =~ /$vol\s+$qtree\s+.*/) {
			return 1;
		}
	}
	return 0;
}
		
## @name ::create_qtree
## @usage create_qtree($filer,$vol,$qtree)
## @description This routine will create a qtree $qtree
## @returns (1 if create succeeds else dies with error in $@)
## @notes (DEPRECATED, instead use QtreeCmds.pm)

sub create_qtree($$$) {
	my($filer,$vol,$qtree)=@_;
	my $h=host($filer);
	my $cfiler = connect("#rsh");
	my $qtreepath="/vol/$vol/$qtree";
	return eval{
		qtreeCreate($cfiler,$qtreepath) or die("$@");
		return 1;
	};
}
		 

## @name ::set_security_qtree
## @usage set_security_qtree($filer, $qtreePath, $security)
## @description This routine will set the security style of the qtree
## @returns (1 if setting succeeds else dies with error in $@)
## @notes (DEPRECATED, instead use QtreeCmds.pm)

sub set_security_qtree($$$) {
	my($filer,$qtreePath,$security)=@_;
	host($filer);
	my $cfiler = connect("#rsh");
	return eval{
		my(undef,undef,$after)=say($cfiler,"qtree security $qtreePath $security");
		if (length($after)){
			die $after;
		}else {
			return 1;
		}
	};
}

1;

