#$Id 
##
## @summary XantUtil.pm - XANT library for commands and utilities
##
## @status TESTME
##
## @author govindar@netapp.com
## @pod here

=head1 NAME

Xantutil

=head1 DESCRIPTION

        This library use boot.pm library to get various state of filer.
        This contains all maintaince mode functions.
        The some of the RaidSqa functions will not work in the maintenance mode. 
        To support copycore added XantUtil_get_aggr_list,XantUtil_aggr_offline functions in the XantUtil  

=head1 SYNOPSIS

        package XantUtil;

        use XantUtil.pm;

        my $host = host($FILER);

        ##FUNCTIONS RELATED TO COMMANDS ##
        XantUtil_boot_up(host=>$host,  conntype=>$conntype, filer_state=>$state,max_try=>$max_try);
        XantUtil_run_cmd(host=>$host,  conntype=>$conntype, cmd=>$cmd, options=>$options, prompt=>$prompt, timeout=>$timeout);
        ## FUNCTIONS RELATED TO AGGR/VOL ##
        XantUtil_get_aggr_list(host=>$host, filermode=>$filermode, conntype=>$conntype,root_vol=>$rootvol );
        XantUtil_aggr_offline(host=>$host, filermode=>$filermode, conntype=>$conntype, volname => $filer_vol);
        ## FUNCTIONS RELATED TO DISKS/MINI DISKS##
        XantUtil_get_disk_list(host=>$host, filermode=>$filermode, conntype=>$conntype,mini_disk=>"no");
        XantUtil_delete_mini_disks(host=>$host, filermode=>$filermode, conntype=>$conntype, diskcount=>$diskcount);
        XantUtil_create_mini_disk(host=>$host, filermode=>$filermode, conntype=>$conntype,disksize=>$disksize,disklist=>\@disk_list);
        
    
=head1 ROUTINES

=cut

package xant::XantUtil;
use strict;
no strict "subs";
use warnings;
require Exporter;

BEGIN {
use Tharn;
use boot;
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF CODEREF UNDEF);
use Utils qw(make_conn);
}

use vars qw(@ISA @EXPORT);
@ISA = qw(Exporter) ;
use constant MAX_TIMEOUT => 300;
@EXPORT = qw (
    XantUtil_boot_up
    XantUtil_run_cmd
    XantUtil_get_aggr_list
    XantUtil_aggr_offline
    XantUtil_get_disk_list
    XantUtil_delete_mini_disks
    XantUtil_create_mini_disk
);



#------------------------------------------------------------------------------

=head2 XantUtil_boot_up

XantUtil_boot_up(host=>$host,  conntype=>$conntype, filer_state=>$state,max_try=>$max_try);

This subroutine try to bring the filer to mentioned state.

It returns 1 if success 0 otherwise

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item conntype

(Optional): Connection type to use. Default is "#console". 

=item filer_state

(Required): The desination state of filer.

=item max_try

(Optional): Give the maximum no of time to bring state. Default is 2.

=back

=cut

#------------------------------------------------------------------------------

sub XantUtil_boot_up {
        my (%opts) = validate(@_, {
                                host => { type=>OBJECT, isa=>'Hostrec' },
                                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                                filer_state => { type=>SCALAR },
                                max_try => { type=>SCALAR,default=>1,optional=>1 },
                                cmode => {type =>SCALAR,default=>0,optional=>1},
                                }
                        );
        my $host = $opts{host};
        my $conntype = $opts{conntype};
        my $STATE = $opts{filer_state};
        my $max_try = $opts{max_try};
        my $cmode =$opts{cmode};
        
my $conn_console = make_conn($host,$conntype);
my $boot_state = boot_state($conn_console);
my $bcount=0;
while($boot_state !~ /$STATE/)
{
    logcomment("XantUtil.pm: Check if filer is $STATE");
    logcomment("XantUtil.pm: Trying to bring the filer to state $bcount attempt");
    $boot_state = boot_state($conn_console);
    logcomment("XantUtil.pm: Filer is in the state $boot_state");
    if ($boot_state eq "$STATE") {
        logcomment("XantUtil.pm: filer is $STATE");
        last;
    } else {
        if($cmode) {
            eval {RaidSqa_transit_to(host=>$host, state_name=>"$STATE")};
        }else {
            eval {transit_to($conn_console, "$STATE", timeout => MAX_TIMEOUT)}; 
        }
        sleep 30;
    }
    $bcount++;
    if($bcount > $max_try) {
        logresult("WARN","XantUtil.pm: Could not bring filer to mentioned state no of try=$bcount");
        return 0;
    }
}
return 1;
}

#------------------------------------------------------------------------------

=head2 XantUtil_run_cmd

XantUtil_run_cmd(host=>$host,  conntype=>$conntype, cmd=>$cmd, options=>$options, prompt=>$prompt, timeout=>$timeout);

This subroutine runs any filer comamnd.

It returns o/p of the command if any

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item conntype

(Optional): Connection type to use. Default is "#console". 

=item cmd

(Optional): filer command.

=item options

(Optional): gives options to be used while executing the filer command.

=item prompt

(Optional): gives prompt to be wait for in case of executing say command.

=item timeout

(Optional): timeout value in secs.

=back

=cut

#------------------------------------------------------------------------------

sub XantUtil_run_cmd {
my (%opts) =
      validate(@_,
               {host      => {type => OBJECT, isa     => 'Hostrec'},
                conntype  => {type => SCALAR, default => "#console", optional => 1},
                cmd     => {type => SCALAR},
                options => {type => SCALAR, optional => 1},
                prompt => {type => SCALAR, optional => 1},
                timeout => {type => SCALAR, default => 180, optional => 1},});
                
    my $host      = $opts{host};
    my $conntype  = $opts{conntype};
    my $cmd       = $opts{cmd};
    my $options   = $opts{options};
    my $prompt   = $opts{prompt};
    my $timeout   = $opts{timeout};
    my $conn = make_conn($host, $conntype);
    my $output;

    if ((!defined($options))&&(!defined($prompt))) {
        (undef, undef, $output) = say($conn, "$cmd", -timeout, $timeout);
    } elsif ((defined($options))&&(!defined($prompt))) {
        (undef, undef, $output) = say($conn, "$cmd", "$options", -timeout, $timeout);
    } elsif ((!defined($options))&&(defined($prompt))) {
        (undef, undef, $output) = say($conn, "$cmd", -prompt, $prompt, -timeout, $timeout);
    } else {
        (undef, undef, $output) = say($conn, "$cmd", -prompt, $prompt, $options, -timeout, $timeout);
    }

    return $output;
}

#------------------------------------------------------------------------------

=head2 XantUtil_get_aggr_list

XantUtil_get_aggr_list(host=>$host, filermode=>$filermode, conntype=>$conntype,root_vol=>$rootvol );

This subroutine by default gets list of aggr/tradvol in a given filer.

It returns an array on success

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#console". For filermode "MAINT" it is "#console" by default.

=item root_vol

(Optional): if root=>1 then Get the root volname. Default is 0. 
=back

=cut

#------------------------------------------------------------------------------


sub XantUtil_get_aggr_list {
        my (%opts) = validate(@_, {
                                host => { type=>OBJECT, isa=>'Hostrec' },
                                conntype => {type=>SCALAR,default=>"#console",optional=>1},
                                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                                root_vol => {type => SCALAR, default=>0,optional => 1},
                                }
                        );
        my $host = $opts{host};
        my $conntype = $opts{conntype};
        my $root_vol = $opts{root_vol};
        my $filermode = $opts{filermode};
        my ($cmd,@outlines,@aggr_list,$rootvol,$output,$prompt)=();
        $cmd="raid_config info listaggr";
        logcomment("XantUitl.pm: Trying to get all the aggreate ");
        if( $filermode ne "normal") {
             $prompt=qq/^\\*>/;
             $conntype=qq/#console/;
        }
        
        $output=XantUtil_run_cmd(host=>$host,cmd=>$cmd,conntype=>$conntype ,prompt=>$prompt );
        
        @outlines=split(/\n|\n\r/,$output);
        foreach my $line(@outlines) {
            next if($line !~ /=/);
            if($line =~/raidname=(\w+).*(diskroot=1)/) {
                $rootvol=$1;
                return $rootvol if($root_vol);
            }elsif($line =~/raidname=(\w+)/) {
                push(@aggr_list,$1);
            }
        }
        logcomment("XantUtil.pm: The aggr lists are @aggr_list");
        return @aggr_list;

}

#------------------------------------------------------------------------------

=head2 XantUtil_aggr_offline

XantUtil_aggr_offline(host=>$host, filermode=>$filermode, conntype=>$conntype, volname => $filer_vol);

This subroutine offline the test aggregate or traditional volume.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#console". For filermode "MAINT" it is "#console" by default.

=item volname

(Required) : Name of aggregate  or traditional volume to be offlined.

=back

=cut

#------------------------------------------------------------------------------


sub XantUtil_aggr_offline {
        my (%opts) = validate(@_, {
                                host => { type=>OBJECT, isa=>'Hostrec' },
                                conntype => {type=>SCALAR,default=>"#console",optional=>1},
                                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                                volname => {type => SCALAR},
                                }
                        );
        my $host = $opts{host};
        my $conntype = $opts{conntype};
        my $filermode = $opts{filermode};
        my $volname = $opts{volname};
        my ($cmd,$output,@outlines,@aggr_list,$rootvol,$prompt)=();
        if( $filermode ne "normal") {
             $prompt=qq/^\\*>/;
             $conntype=qq/#console/;
        }
        $cmd="aggr offline $volname";
        logcomment("XantUtil.pm: Trying to offline the volume $volname");
        $output=XantUtil_run_cmd(host=>$host,cmd=>$cmd,conntype=>$conntype ,prompt=>$prompt );
        return 1;
}

#------------------------------------------------------------------------------

=head2 XantUtil_get_disk_list

XantUtil_get_disk_list(host=>$host, filermode=>$filermode, conntype=>$conntype,mini_disk=>"no");

This subroutine get the list of disks other than root volume disk.

It returns arryref on success and 0 on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#console".  

=item mini_disk

(optional) : Specifies the flag to "yes" to get mini_disk. Default is no.

=back

=cut

#------------------------------------------------------------------------------


sub XantUtil_get_disk_list {
        my (%opts) = validate(@_, {
                                host => { type=>OBJECT, isa=>'Hostrec' },
                                conntype => {type=>SCALAR,default=>"#console",optional=>1},
                                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                                mini_disk => { type =>SCALAR, default=>"no",optional=>1},
                                }
                        );
        my $host = $opts{host};
        my $conntype = $opts{conntype};
        my $filermode = $opts{filermode};
        my $mini_disk = $opts{mini_disk};
        my ($cmd,$output,@outlines,$prompt,@mini_disk,@root_disks,@all_disks,$line,$rootvol,%hash,%hash1)=();
        if( $filermode ne "normal") {
             $prompt=qq/^\\*>/;
             $conntype=qq/#console/;
             
        }
        # Get the root volume disks 
        $rootvol=XantUtil_get_aggr_list(host=>$host,conntype=>$conntype,filermode =>$filermode, root_vol=>1);
        logcomment("XantUtil.pm: The root volume is $rootvol");
        $cmd="raid_config info listdisk";
        $output=XantUtil_run_cmd(host=>$host,cmd=>$cmd,conntype=>$conntype ,prompt=>$prompt );
        @outlines=split(/\n|\n\r/,$output);
        #raid_name=0b.40,name=0b.40
        foreach  $line(@outlines) {
                if($line=~/\/$rootvol\/(.*),name=(\w+\.\d+),/) {
                    push(@root_disks,$2);
            } elsif($line =~/raid_name=(\w+\.\d+),name=(\w+\.\d+)/) {
                   #push (@all_disks,$1) if($1 eq $2);
            }
        }
        $cmd="mini_disk list";
        logcomment("XantUtil.pm: Trying to get Normal disk or mini disk list");
        $output=XantUtil_run_cmd(host=>$host,cmd=>$cmd,conntype=>$conntype ,prompt=>$prompt );
        @outlines=split(/\n|\n\r/,$output);
        #raid_name=0b.40,name=0b.40
        foreach  $line(@outlines) {
            if($line =~/Disk\s(.*):\s+is configured as a (\d+\w+\w?) minidisk/i) {
                logcomment("XantUtil.pm: The minidisk disk name is $1 and size is $2");
                push(@mini_disk,$1);
            }elsif($line=~ /Disk\s(.*):\s+is configured as a NORMAL disk/i) {
                logcomment("XantUtil.pm: The Normal disk name is $1 ");
                push (@all_disks,$1);
            }
        }
        @hash1{@all_disks}=();
        @hash{@mini_disk} = ();
        foreach (@root_disks) {
                delete $hash{$_};
                delete $hash1{$_};
        }
        @mini_disk = sort keys %hash;
        @all_disks = sort keys %hash1;
        logcomment("XantUtil.pm: The root volume $rootvol disks are @root_disks"); 
        logcomment("XantUtil.pm: The minidisk disk list is @mini_disk");
        if($mini_disk eq "yes") {
            return @mini_disk;
        }else {
            return @all_disks;
        }
}

#------------------------------------------------------------------------------

=head2 XantUtil_delete_mini_disks

XantUtil_delete_mini_disks(host=>$host, filermode=>$filermode, conntype=>$conntype);

This subroutine delete all the mini disks.

It returns 0 no mini disks is present 1 for deleted mini disk successfully.

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#console".  

=back

=cut

#------------------------------------------------------------------------------


sub XantUtil_delete_mini_disks {
        my (%opts) = validate(@_, {
                                host => { type=>OBJECT, isa=>'Hostrec' },
                                conntype => {type=>SCALAR,default=>"#console",optional=>1},
                                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                                diskcount => { type =>SCALAR, default =>0, optional =>1},
                                }
                        );
        my $host = $opts{host};
        my $conntype = $opts{conntype};
        my $filermode = $opts{filermode};
        my $diskcount = $opts{diskcount};
        my ($cmd,$output, $prompt,%mini_disk,@tmp_mini_disk,@mini_disk_arr)=();
        if( $filermode ne "normal") {
             $prompt=qq/^\\*>/;
             $conntype=qq/#console/;
        }

        @tmp_mini_disk=XantUtil_get_disk_list(host=>$host,conntype=>$conntype ,filermode=>$filermode, mini_disk=>"yes");
        if(($#tmp_mini_disk == -1) &&($#tmp_mini_disk < $diskcount)) {
            logresult('WARN',"XantUtil.pm: There is no mini disk present in the filer");
            return 0;
        }
        if($diskcount) {
            @mini_disk_arr=splice (@tmp_mini_disk,0,$diskcount);
        }else {
            @mini_disk_arr=@tmp_mini_disk;
        }
        $cmd="mini_disk delete @mini_disk_arr";
        my $mprompt='Are you sure you want to continue?';
        logcomment("XantUtil.pm: Trying to delete the all mini disks @mini_disk_arr");
        $output=XantUtil_run_cmd(host=>$host,cmd=>$cmd,conntype=>$conntype ,prompt=>$mprompt );
        $cmd="yes";
        $mprompt='Would you like to zero newly deleted minidisks now?';
        $output=XantUtil_run_cmd(host=>$host,cmd=>$cmd,conntype=>$conntype ,prompt=>$mprompt );
        $cmd="no";
        $output=XantUtil_run_cmd(host=>$host,cmd=>$cmd,conntype=>$conntype ,prompt=>$prompt );
        return 1;
}


#------------------------------------------------------------------------------

=head2 XantUtil_create_mini_disk

XantUtil_create_mini_disk(host=>$host, filermode=>$filermode, conntype=>$conntype,disksize=>$disksize,disklist=>\@disk_list);

This subroutine will create a mini disks in the given disk size with count.

It returns 1 

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#console".  

=item disksize

(Required) : Specifies the size of the disks in "MB". 

=item disk_list

(Required) : Specifies array of disks to create mini disks.


=back

=cut

#------------------------------------------------------------------------------


sub XantUtil_create_mini_disk {
        my (%opts) = validate(@_, {
                                host => { type=>OBJECT, isa=>'Hostrec' },
                                conntype => {type=>SCALAR,default=>"#console",optional=>1},
                                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                                disksize => {type => SCALAR},
                                disklist => {type => ARRAYREF},
                                }
                        );
        my $host = $opts{host};
        my $conntype = $opts{conntype};
        my $filermode = $opts{filermode};
        my $disksize = $opts{disksize};
        my $disklist_ref = $opts{disklist};
        my ($rootvol,$prompt,$cmd,$output);
        my @disklist=@$disklist_ref;
        if( $filermode ne "normal") {
             $prompt=qq/^\\*>/;
             $conntype=qq/#console/;
        }
        $disksize="$disksize"."MB";
        
        $cmd="mini_disk create $disksize @disklist";
        my $mprompt='Are you sure you want to continue?';
        logcomment("XantUtil.pm: Trying to create mini_disk of size $disksize @disklist");
        $output=XantUtil_run_cmd(host=>$host,cmd=>$cmd,conntype=>$conntype ,prompt=>$mprompt );
        $cmd="yes";
        $mprompt='Would you like to zero newly created minidisks now?';
        $output=XantUtil_run_cmd(host=>$host,cmd=>$cmd,conntype=>$conntype ,prompt=>$mprompt );
        if($output =~/Unable to create minidisk on disk(.*)/) {
            logcomment("Fail to create minidisk(s)");
            return 0;
        }
        $cmd="no";
        $output=XantUtil_run_cmd(host=>$host,cmd=>$cmd,conntype=>$conntype ,prompt=>$prompt );
        return 1;
}


1;
