# -*- Perl -*-
# $Id:$
##
##	Copyright (c) 2009 NetApp.
##	All Rights reserved
##
## @summary ESSUtils.pm is a common module for all the ESS tests
##
## @description
## This module provides common functions for use in testing ESS functionality.
## Functions:
## 1. Existing_Nodes
##  - To get the number and names of the nodes connected.
## 2. Sensors_Show
##	- To Verify the if the output from sensors show command is in the
##	  expected format.
##
## @status private
##
## @author sandeepk@netapp.com,dl-platform-qa@netapp.com
##
#########################################################################################

package ESSUtils;

use strict;
use warnings;
use Shell;
use Tharn;
use SPARSE::Util::Logger qw(logStep logWarn);

require Exporter;
our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

our %EXPORT_TAGS = (
    'all' => [
        qw(

            )
    ]
);

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
    Existing_Nodes
    Sensors_Show
);

############### Function Prototypes ##########
sub Existing_Nodes($$);
sub Sensors_Show($$$);

############### Package Version ############
our $VERSION = '1.0';

############### Local Variables ############

##############################     FUNCTIONS     ##############################

#-------------------------------------------------------------------------------
# FUNCTION:	To get the number and names of the nodes connected.
# INPUT   : $_[0] = Filer connection handle, $_[1] = Filer Name.
# RETURN  : %node_info = number of nodes available and their names.
#-------------------------------------------------------------------------------
sub Existing_Nodes($$) {
    my $fc = $_[0];
    my $response;
    my $node_list;
    my $total_nodes;
    my @tmprslt;
    my @arr;

    ( undef, undef, $response, undef )
        = say( $fc, "run -node \t", -swallow, '-timeout', 300 );
    say( $fc, "\3", -swallow, '-timeout', 300 );
    say( $fc, "\4", -swallow, -noprompt, '-timeout', 30 );

    @tmprslt = split( /\n/, $response );

    if ( $tmprslt[1] !~ /$_[1]/ ) {
        $total_nodes = 1;
        $node_list   = $tmprslt[0];
        chomp $node_list;
        $node_list =~ s/run -node //;
        $node_list =~ s/\s+$//;
    } else {
        @arr         = split( /\s{2,}/, $tmprslt[1] );
        $node_list   = $arr[1];
        $total_nodes = @arr;
    }

    logcomment("Total number of Nodes available : $total_nodes");
    logcomment("Node names are : $node_list");
    my %node_info = (
        TOTALNODES => $total_nodes,    # Total number of nodes available
        NODELIST   => $node_list
    );                                 # Node names
    return (%node_info);
} ## end sub Existing_Nodes($$)

#-------------------------------------------------------------------------------
# FUNCTION:	To Verify the if the output from sensors show command is in the
#			expected format.
# INPUT   : $_[0] = Filer connection handle, $_[1] = Filer Name,
#           $_[2] = Filer Prompt
# RETURN  : 0 = Success, 1 = Failure.
#-------------------------------------------------------------------------------
sub Sensors_Show($$$) {
    my $filer_prompt = $_[2];
    my $fc           = shift;
    my $filer        = shift;
    my $ref_out_1    = "Node Sensor";
    my $ref_out_2    = "State Value/Units Crit-Low Warn-Low Warn-Hi Crit-Hi";
    my ( $response1, $response2, @tmprslt, $prompt, $i );
    my %node_info = Existing_Nodes( $fc, $filer );
    my $tot_nodes = $node_info{TOTALNODES};
    my $node_list = $node_info{NODELIST};
    my @nodes     = split( /\s/, $node_list );

    ( undef, undef, $response1, $prompt )
        = say( $fc, "system node environment sensors show",
        -swallow, -prompt, "Press <space> to page down",
        $filer_prompt, '-timeout', 300
        );
    @tmprslt = split( /\n/, $response1 );
    if (   ( $tmprslt[1] =~ /$ref_out_1\s*$ref_out_2/ )
        && ( $tmprslt[2] !~ /[A-Za-z0-9]/ )
        && ( $tmprslt[3] =~ /$filer/ )
        && ( $tmprslt[4] =~ /[A-Za-z0-9]/ )
        && ( $tmprslt[5] =~ /\s*[A-Za-z0-9]/ ) ) {
        logStep("Output Format Matched");
    } else {
        logStep("Output Format Unmatched");
        return 1;
    }
    if ( $tot_nodes == 1 ) {
        say( $fc, "q", -swallow, -prompt, $filer_prompt, '-timeout', 300 );
        return 0;
    } else {
        for ( $i = 1; $i < $tot_nodes; $i++ ) {
            if ( $response1 =~ /$nodes[$i]/ ) {
                logStep("$nodes[$i] is found");
            }
        }
    }
    eval {
        while ( $prompt !~ /$filer_prompt/ ) {
            ( undef, undef, $response2, $prompt )
                = say( $fc, " ", -swallow, -prompt,
                "Press <space> to page down",
                $filer_prompt, '-timeout', 300 );

            my $j = 1;
            for ( $j = 1; $j < $tot_nodes; $j++ ) {
                if ( $response2 =~ /$nodes[$j]/ ) {
                    logStep("$nodes[$j] is found");
                }
            }
        }
    };
    if ($@) {
        return 1;
    }
} ## end sub Sensors_Show($$$)
1;

