#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/BLAST/SanBlaze.pm#1 $

## @summary Provide an interface to a SAN Blaze device
## @author evan@netapp.com
## @status private
## @pod here

package SanBlaze;

# Compiler directives.
use strict;

# Module import.
use Utils qw(make_host make_conn);
use Time::HiRes qw(gettimeofday);
use Tharn;
use Error;
use Data::Dumper;

use BLAST::SanBlaze::Port;

use vars qw(@ISA);

$SanBlaze::TRUE = 0;
$SanBalze::FALSE = 1;

@ISA = qw(Error);

sub new {
    my ($proto, %args) = @_;
    my $class = ref($proto) || $proto;
    my (@output, $ports);
    my $self = {};
    bless $self,$class;
    
    foreach (keys %args) {
        if (/-?host$/) {
            length($self->{host} = $args{$_})
              or do {
                $self->_set_err("SanBlaze::new() invalid value for \"-host\":", "$args{$_}");
                return $self;
              };
        } elsif (/^-?connection$/) {
            length($self->{conn} = $args{$_})
              or do {
                $self->_set_err("SanBlaze::new() invalid value for \"-connection\":", "$args{$_}");
                return $self;
              };
        } elsif (/^-?conn_type$/) {
            length($self->{conn_type} = $args{$_})
              or do {
                $self->_set_err("SanBlaze::new() invalid value for \"-conn_type\":", "$args{$_}");
                return $self;
              };
        } elsif (/^-?user$/) {
            length($self->{user} = $args{$_})
              or do {
                $self->_set_err("SanBlaze::new() invalid value for \"-user\":", "$args{$_}");
                return $self;
              };
        } elsif (/^-?passwd$/) {
            length($self->{passwd} = $args{$_})
              or do {
                $self->_set_err("SanBlaze::new() invalid value for \"-passwd\":", "$args{$_}");
                return $self;
              };
        } elsif (/-?timeout$/) {
            length($self->{timeout} = $args{$_})
              or do {
                $self->_set_err("SanBlaze::new() invalid value for \"-timeout\":", "$args{$_}");
                return $self;
              };
        } elsif (/^-?ports$/) {
            length($ports = $args{$_})
              or do {
                $self->_set_err("SanBlaze::new() invalid value for \"-passwd\":", "$args{$_}");
                return $self;
              };
        }
    }

    # Verify mandatory parameters.
    unless (defined $self->{host} and length $self->{host}) {
        $self->_set_err("SanSwitch::new mandatory parameter \"host\"", "not specified");
        return $self;
    }

    # Set default values
    $self->{user} = "vlun" if (!defined($self->{user}));
    $self->{password} = "SANBlaze" if (!defined($self->{password}));
    $self->{timeout} = 600 if (!defined($self->{timeout}));
    
    # Determine if a connection has been provided
    if (!defined($self->{conn})) {
        # Connect to the host and store the connection object
        $self->{conn} = $self->_connect($self->{host},
                                      $self->{user},
                                      $self->{passwd},
                                      $self->{timeout},
                                      $self->{conn_type});
        if ($self->error) {
            # Error message is already set, return object so caller can get the error.
            return $self;
        }
    }

    if (!defined($ports)) {
        @output = $self->execute("ls -1 --color=no /port*");
        if ($self->error) {
            return $self;
        }
    } else {
        @output = split(/,/,$ports);
        for (my $i=0; $i<@output; $i++) {
            $output[$i] = "port" . $output[$i];
        }
    }

    foreach my $port (@output) {
        # Remove starting slash and trailing @ symbol
        $port =~ s/\@$//;
        $port =~ s/^\///;

        my $portobj = Port->new(-name => $port, -parent => $self);
        push (@{$self->{ports}}, $portobj);
    }

    return $self;
}

sub get_port {
    my ($self, $name) = @_;

    if (!defined($name)) {
        $self->_set_err("SanBlaze::get_port() mandatory paramter name not specified");
        return;
    }

    foreach my $port (@{$self->{ports}}) {
        if ($port->{name} =~ /$name/) {
            return $port;
        }
    }

    $self->_set_err("SanBlaze::get_port() no ports matching '$name' found");
    return;
}

sub get_ports {
    my ($self) = @_;

    return $self->{ports};
}
   

sub getNumberOfInitiators {
    my ($self, $port) = @_;
    
    if ($port !~ /\d+/) {
        $self->_set_err("SanBlaze::getNumberOfInitiators() invalid port value");
        return undef;
    }

    my ($status, @output) = $self->execute("cat /iport${port}/port | grep NumberOfInitiators");
    return (split(/=/, $output[0]))[1];
}


sub execute {
    my ($self, $cmd) = @_;
    my ($status, @output, $errmsg, $t0, $t1);

    # Clear errors.
    $self->_clear_err;

    unless (length $cmd) {
        $self->_set_err("Error: SanBlaze::execute() invalid CLI command");
        return [];
    }   
        
    # Init execution timer
    $t0 = gettimeofday;

    # Execute the command
    ($status, @output) = &_exec($self->{conn}, $cmd, $self->{timeout});
              
    # Get the post execution time and save the command time
    $t1 = gettimeofday;
    $self->{command_time} = $t1 - $t0;

    if ($status > 0) {
        $self->_set_err("Error: SanBlaze::execute() :", @output);
        return (wantarray()) ? @output : \@output;
    }

    return (wantarray()) ? @output : \@output;
}

### P R I V A T E   M E T H O D S ###

sub _connect {
    my ($self, $hostname, $user, $passwd, $timeout, $conn_type) = @_;
    my ($connect, $conn_str, $errmsg, $host, $id, $hostInfo,);

    $host = Utils::make_host($hostname);

    if ($conn_type eq "THARNHOST") {
        $conn_type = "";

        # Check to see if any connection types are defined,
        # if not return the default value
        if (!defined($host->{connection_fields})) {

            # Set the $conn_type to the default of telnet
            $conn_type = "telnet";
        }

        # Go throught each remainder field and set $conn_type
        # IF rsh is defined, set that. This means that if the
        # conn type of rsh is defined in the tharnhost file,
        # we set $conn_type to that. To not use rsh, comment
        # out the conn type of rsh from the tharnhost file.
        # This will then check to see if the next available
        # conn type is defined in the tharnhost file.
        # The order of connection type is:
        # 1) rsh
        # 2) ssh
        # 3) telnet
        # 4) rlogin
        # 5) console
          elsif (defined($host->{connection_fields}->{rsh})) {
            $conn_type = "rsh";
        } elsif (defined($host->{connection_fields}->{ssh})) {
            $conn_type = "ssh";
        } elsif (defined($host->{connection_fields}->{telnet})) {
            $conn_type = "telnet";
        } elsif (defined($host->{connection_fields}->{rlogin})) {
            $conn_type = "rlogin";
        } elsif (defined($host->{connection_fields}->{console})) {
            $conn_type = "console";
        }
    } elsif ($conn_type eq "") {
        $conn_type = "telnet";
    } else {
        # Default $conn_type to the specified value user passed in
    }

    if (defined($user) and length($user)) {
        $host->{connection_fields}{$conn_type}{username} = $user;
    }

    if (defined($passwd) and length($passwd)) {
        $host->{connection_fields}{$conn_type}{password} = $passwd;
    }

    $conn_str =
        $hostname . "#"
      . $conn_type . ";"
      . "username="
      . $host->{connection_fields}{$conn_type}{username} . ";"
      . "password="
      . $host->{connection_fields}{$conn_type}{password} . ";"
      . "timeout="
      . $self->{timeout}
      . ";inherit=0;";

    $connect = Tharn::connect(
        $conn_str,
        -eval => sub {
            if (@_) {
                my $errmsg = join("", @_);
                $errmsg =~ s/\n//g;
                $self->_set_err($errmsg);

                Tharn::logresult("CONFIG", abort => 0, "\n@_");

                $self->{connected} = 0;
                return $self;
            }
        });

    # Switch to superuser mode
    Tharn::say($connect, "su", -noprompt , -prompt , "assword:", '-timeout', $self->{timeout});
    Tharn::say($connect, "SANBlaze", '-timeout', $self->{timeout});

    $self->{connected} = 1;
    return $connect;
}

sub _exec {
    my ($connection, $cmd, $timeout) = @_;

    my ($env_output, $output, @output, $status);
    my ($match, $before, $after, $sentinel, $default_timeout);
    my ($cmdlog, $original_cmd);

    $default_timeout = "600";
    $cmdlog = 1;
    @output = ();

    if (!defined($cmd) or $cmd =~ /^\s*$/) {
        &Tharn::tstlog('-warn', "Command passed to SanBlaze::_exec() was empty");
        $status = 1;
        return($status, @output);
    }

    # set the default timeout if not specified
    $timeout = $default_timeout if (!defined($timeout) or $timeout =~ m/^\s*$/);

    $original_cmd = $cmd;

    # Force the STRERR to show up in the STDOUT
    $cmd = $cmd . " 2>&1";
    
    # add to the command the echo of status to get that information from
    # the unix shell. Also add the sentinel string DoNe to command so that
    # we can match on that string.
    $sentinel = 'DoNe';
    $cmd = $cmd . ";echo \"rstatus=\$\?\" ;echo $sentinel";

    # clear buffers
    $connection->discardbuff();

    # execute the passed command. Use just timeout since we check earlier
    # now if it was not passed and set the default if that is the case.
    ($match, $output, $after) =  &Tharn::say($connection, $cmd, $sentinel,
                                             '-noprompt',
                                             '-timeout', $timeout,
                                             '-ignore', "echo $sentinel");
    # strip output and remove CR/LF
    chomp($output);
    $output =~ s/\r//g;

    # remove first line 
    #$cmd =~ s/"rstatus=\$\?" ;echo DoNe//g;
    #$output =~ s/"rstatus=\$\?" ;//g;

    # strip sentinel and then status from output
    $output =~ s/\n$sentinel.*//;

    # strip any telnet ctrl characters that get
    # added to the output (this problem seems 
    # to only occur with certain platforms.
    $output =~ s/ \cH//g;

    # locate status.  Remember that commands
    # echo themselves so make sure you don't 
    # get the command line 
    ($status) = $output =~ /rstatus=(\d+)/;

    $output =~ s/rstatus=\d+//;

    # create an array from the output
    @output = split(/\n/, $output);

    ## We need to determine if command is multiline 
    ## and exists in the output.  IMPORTANT: Not all
    ## commands echo back their input so don't forget to
    ## check before modifying the output.
    #my @cmds = split(/\n/, $cmd);

    ## If output contains second line of cmd 
    ## then cmd was echoed to output and is 
    ## multiline
    #if (defined $cmds[1] and length $cmds[1]) {
        ## Remove continuation character
    #    $cmds[1] =~ s/\\//g;
    #    if ($output[0] =~ /$cmds[1]/) {
          ## determine size of command and strip
          ## from there.
    #      @output = @output[@cmds-1..@output-1];
    #    }
    #}
    
    ## Go through the output and remove any line that has the command in it
    ## We do this, because none of the function that call execute expect
    ## to have to filter out their cmd from the returned output.
    my @output_sorted;
    foreach my $line (@output) {
        if ($line =~ /$original_cmd/) {
            next;
        }

        push (@output_sorted, $line);
    }

    @output = @output_sorted;

    return($status, @output);
}

