#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/BLAST/SanBlaze/Port.pm#1 $

## @summary Provide an interface to a port on a SAN Blaze device
## @author evan@netapp.com
## @status private
## @pod here

package Port;

# Compiler directives.
use strict;

# Module import.
use Error;
use BLAST::SanBlaze;
use Data::Dumper;
use vars qw(@ISA);

@ISA = qw(Error);

$SanBlaze::LUN0 = 1;
$SanBlaze::ALL_LUNS = 2;
$SanBlaze::REPORT_LUNS = 3;

sub new {
    my ($class, %args) = @_;
    my ($self);

    $self = {};

    # bless the object upfront in case an error occurs.
    bless $self, $class;

    foreach (keys %args) {
        if (/-?parent$/) {
            if (ref($args{$_}) !~ "SanBlaze") {
                $self->_set_err("Error: Port::_new() " .
                        "Invalid value for argument \"-parent\"");
                return $self;
            }
            $self->{parent} = $args{$_};
        } elsif (/^-?name$/) {
            length($self->{name} = $args{$_})
              or do {
                $self->_set_err("Port::new() invalid value for \"-name\":", "$args{$_}");
                return $self;
              };
        }
    }

    if (!defined($self->{parent})) {
        $self->_set_err("Port::new() mandatory parameter \"-parent\" not specified");
        return $self;
    }

    if (!defined($self->{name})) {
        $self->_set_err("Port::new() mandatory parameter \"-name\" not specified");
        return $self;
    }

    $self->updatePortInfo();

    #$self->{targets} = $self->updateTargetInfo();
    return $self;
}

sub updatePortInfo {
    my ($self) = @_;

    # Get /port#/port values
    my ($status, @output) = $self->{parent}->execute("cat /$self->{name}/port");

    foreach my $line (@output) {
        if ($line =~ /NumLUNsTarget/) {
            next;
        }
        my ($key, $value) = split(/=/, $line);
        $self->{port}{$key} = $value;
    }

    # Get /iport#/port values
    my ($status, @output) = $self->{parent}->execute("cat /i$self->{name}/port");

    foreach my $line (@output) {
        my ($key, $value) = split(/=/, $line);
        $self->{iport}{$key} = $value;
    }
}

#sub updateTargetInfo {
#    my ($self) = @_;

    
#}

sub get_wwpn {
    my ($self) = @_;

    return $self->{port}{WWPN};
}
sub offline {
    my ($self) = @_;
    my @output;
    my $count = 0;
    my $port_timeout = 30;

    $self->{parent}->execute("echo Offline > /$self->{name}/port");

    if ($self->{parent}->error) {
        return $SanBlaze::FALSE;
    }
    
    do {
        $self->updatePortInfo();
        $count++;
        sleep(1);
    } while ($self->{port}{PortState} !~ /Offline/i && $count < $port_timeout);

    # Port state does not appear to be accurately reflected so
    # assume it is good for now.
    return $SanBlaze::TRUE;

    if ($count == $port_timeout) {
        $self->_set_err("Port::offline() port did not go offline");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub online {
    my ($self) = @_;
    my @output;
    my $count = 0;
    my $port_timeout = 30;

    $self->{parent}->execute("echo Online > /$self->{name}/port");

    if ($self->{parent}->error) {
        return $SanBlaze::FALSE;
    }

    do {
        $self->updatePortInfo();
        $count++;
        sleep(1);
    } while ($self->{port}{PortState} !~ /Online/i && $count < $port_timeout);

    # Port state does not appear to be accurately reflected so
    # assume it is good for now.
    return $SanBlaze::TRUE;

    if ($count == $port_timeout) {
        $self->_set_err("Port::online() port did not go online");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub set_initiator_count {
    my ($self, $inits) = @_;
    my $count = 0;
    my $port_timeout = 30;

    $inits = $inits - 1;
    $self->{parent}->execute("echo NumAliases=$inits > /$self->{name}/port");

    do {
        $self->updatePortInfo();
        $count++;
        sleep(1);
    } while($self->{iport}{NumberOfInitiators} != ($inits+1) && $count < $port_timeout);

    if ($count == $port_timeout) {
        $self->_set_err("Port::set_initiator_count() error setting number of initiators");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub stop_tests {
    my ($self) = @_;

    $self->{parent}->execute("echo StopTests > /i$self->{name}/port");

    if ($self->{parent}->error) {
        $self->_set_err("Port::stop_tests() error while stopping tests");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub pause_tests {
    my ($self) = @_;

    $self->{parent}->execute("echo PauseTests > /i$self->{name}/port");

    if ($self->{parent}->error) {
        $self->_set_err("Port::pause_tests() error while pausing tests");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub unpause_tests {
    my ($self) = @_;

    $self->{parent}->execute("echo UnPauseTests > /i$self->{name}/port");

    if ($self->{parent}->error) {
        $self->_set_err("Port::unpause_tests() error while resuming tests");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub set_oui {
    my ($self, $oui) = @_;

    $self->{parent}->execute("echo OUI=$oui > /$self->{name}/port");

    if ($self->{parent}->error) {
        $self->_set_err("Port::set_oui() error while setting OUI to $oui");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub probe {
    my ($self, $value, $initiator) = @_;

    if ($value < 1 || $value > 3) {
        $self->_set_err("Port::probe() invalid probe value specified");
        return $SanBlaze::FALSE;
    }

    if (!defined($initiator)) {
        # If an initiator number is not provided, use all
        $initiator = "-1";
    }

    # Adding the initiator is undocumented in V4.0.1 but works
    $self->{parent}->execute("echo Probe=$value,$initiator > /i$self->{name}/port");

    if ($self->{parent}->error) {
        $self->_set_err("Port::probe() error while reprobing for devices");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub set_speed {
    my ($self, $speed) = @_;

    $self->{parent}->execute("echo PortSpeed=$speed > /$self->{name}/port");

    if ($self->{parent}->error) {
        $self->_set_err("Port::set_speed() error while setting speed to $speed");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub set_topology {
    my ($self, $topology) = @_;

    if ($topology !~ /(Auto|PointToPoint|Loop)/i) {
        $self->_set_err("Port::set_topology() invalid topology value specified");
        return $SanBlaze::FALSE;
    }

    $self->{parent}->execute("echo Topology=$topology> /$self->{name}/port");

    if ($self->{parent}->error) {
        $self->_set_err("Port::set_topology() error while setting topology to $topology");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub link_reset {
    my ($self) = @_;

    $self->{parent}->execute("echo LinkReset > /$self->{name}/port");

    if ($self->{parent}->error) {
        $self->_set_err("Port::link_reset() error while performing link reset");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub start_test {
    my ($self, %args) = @_;
    my ($type, $threads, $blocks, $ios_per_thread, $random, $opcode_size, $paused, $pattern, $initiator);

    $type = "Compare";
    $threads = "64";
    $blocks = -1; #random
    $ios_per_thread = "0"; #run forever
    $random = 1;
    $opcode_size = 0; #random
    $paused = 0;
    $pattern = 0; #random
    $initiator = -1; #all

    foreach (keys %args) {
        if (/-?type$/) {
            length($type = $args{$_})
              or do {
                $self->_set_err("Port::start_test() invalid value for \"-type\":", "$args{$_}");
                return ($Controller::FALSE);
              };
        } elsif (/^-?threads$/) {
            length($threads = $args{$_})
              or do {
                $self->_set_err("Port::start_test() invalid value for \"-threads\":", "$args{$_}");
                return ($Controller::FALSE);
              };
        } elsif (/^-?blocks$/) {
            length($blocks= $args{$_})
              or do {
                $self->_set_err("Port::start_test() invalid value for \"-blocks\":", "$args{$_}");
                return ($Controller::FALSE);
              };
        } elsif (/^-?ios_per_thread$/) {
            length($ios_per_thread = $args{$_})
              or do {
                $self->_set_err("Port::start_test() invalid value for \"-ios_per_thread\":", "$args{$_}");
                return ($Controller::FALSE);
              };
        } elsif (/^-?random$/) {
            length($random = $args{$_})
              or do {
                $self->_set_err("Port::start_test() invalid value for \"-random\":", "$args{$_}");
                return ($Controller::FALSE);
              };
        } elsif (/^-?opcode_size$/) {
            length($opcode_size = $args{$_})
              or do {
                $self->_set_err("Port::start_test() invalid value for \"-opcode_size\":", "$args{$_}");
                return ($Controller::FALSE);
              };
        } elsif (/^-?pattern$/) {
            length($pattern = $args{$_})
              or do {
                $self->_set_err("Port::start_test() invalid value for \"-pattern\":", "$args{$_}");
                return ($Controller::FALSE);
              };
        } elsif (/^-?initiator$/) {
            length($initiator = $args{$_})
              or do {
                $self->_set_err("Port::start_test() invalid value for \"-initiator\":", "$args{$_}");
                return ($Controller::FALSE);
              };
        } elsif (/^-?paused$/) {
            length($paused = $args{$_})
              or do {
                $self->_set_err("Port::start_test() invalid value for \"-paused\":", "$args{$_}");
                return ($Controller::FALSE);
              };
        }
    }

    if ($type ne "Read" and $type ne "Write" and $type ne "Compare") {
        $self->_set_err("Port::start_test() invalid type ($type) specified");
        return ($Controller::FALSE);
    }

    # Continue on error and attempt recovery default command retry is 5, let's bump this up to 10
    $self->{parent}->execute("echo OnError=Continue > /i$self->{name}/port");
    $self->{parent}->execute("echo Retry=10 > /i$self->{name}/port");

    if ($type eq "Write" or $type eq "Compare") {
        $self->{parent}->execute("echo AlwaysWriteEnabled=1 > /i$self->{name}/port");
    }

    $self->{parent}->execute("echo $type,$threads,$blocks,$ios_per_thread,$random,$opcode_size,$paused,$pattern,$initiator > /i$self->{name}/port");
    
    if ($self->{parent}->error) {
        $self->_set_err("Port::start_test() error while trying to start test",
            join(/\n/, $self->{parent}->errmsg));
        return ($Controller::FALSE);
    }

    return ($Controller::TRUE);
}

sub reset_wwn {
    my ($self) = @_;

    $self->{parent}->execute("echo ResetWWN > /$self->{name}/port");

    if ($self->{parent}->error) {
        $self->_set_err("Port::reset_wwn() error while resetting WWN");
        return $SanBlaze::FALSE;
    } else {
        return $SanBlaze::TRUE;
    }
}

sub get_initiators {
    my ($self) = @_;
    my @inits;
    my ($status, @output) = $self->{parent}->execute("cat /$self->{name}/port | grep NumLUNsTarget");

    foreach my $line (@output) {
        # Do an ungreedy match as the line is comma seperated with multiple key=value pairs
        if ($line =~ /WWPN=(\S+?),/) {
            push (@inits, $1);
        }
    }

    return \@inits;
}
