#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/BLAST/Test.pm#1 $

## @summary Common subroutines used in the SAN Unit Test suite
## @author evan@netapp.com
## @status private
## @pod here

=pod

=head1 NAME

Test.pm

=head1 DESCRIPTION

This package contains common subroutines used in the SAN Unit Test suite

=head1 METHODS

=cut

package Test;

##########################################################################

=head2 C<check_error()>

=over 4

=item * Description

Checks an object for any errors that it may have set. The object must be
inheriting from the Error library. If an error is detected, it will write
a failure message to the nate test log. The test will then exit with a
failure.

=item * Arguments

=over 4

=item 1 Object
 (Required)

A reference to the object to be checked.

=back

=item * Returns

None.

=back

=cut

##########################################################################
sub check_error {
    my $obj = shift;
    if ($obj->error()) {
        my $errmsg = join('\n', $obj->errmsg());
        &main::tstlog('-fatal', "Errors encountered in object",
                "Error Message = $errmsg");
    }
}

##########################################################################

=head2 C<check_ras()>

=over 4

=item * Description

Checks RAS entry labels against a template.

=item * Arguments Hash

=over 4

=item 1 -ras_array
 (Required)

The array of RAS entries to check.

=item 2 -template
 (Required)

The array template of labels to check against.

=item 3 -ignore_label
 (Required)

Comma or pipe seperated list of labels to ignore.

=back

=item * Returns

The number of mismatched labels detected. If the check was successful,
this would be zero.

=back

=cut

##########################################################################
sub check_ras {
    my (%args) = @_;
    my (@ras_filtered, @ras_array, @template, $ignore_label);
    my $failcount = 0;

    foreach (keys %args) {
        if (/-?ras_array/) {
            @ras_array = @{$args{$_}};
        }
        if (/-?template/) {
            @template = @{$args{$_}};
        }
        if (/-?ignore_label/) {
            $ignore_label = $args{$_};
            $ignore_label =~ s/,/|/g;
            #print "ignore label: $ignore_label\n";
        }
    }

    if ( !defined(@ras_array) || !defined(@template) ) {
        return 1;
    }

    foreach (@ras_array) {
        if ($_->{label} !~ /$ignore_label/) {
            #print "$_->{label}\n";
            push(@ras_filtered, $_);
        }# else {
            #print "ignoring $_->{label}\n";
        #}
    }
    for (my $i=0; $i<@template; $i++) {
        if ($ras_filtered[$i]->{label} ne $template[$i]) {
            $failcount++;
        }
    }

    return $failcount;
}

sub split_first {
    my ($list, $delim, $index) = @_;

    $delim = ":" if (!defined($delim));
    $index = 0 if (!defined($index));

    my @items = split(/$delim/, $list);
 
    return $items[$index];
}

sub remove_overrides {
    my ($value, $delim) = @_;

    if (!defined($delim)) {
	return (split(/{/, $value))[0];
    } else {
	my @temp = split(/$delim/, $value);
	my @array = ();

	foreach (@temp) {
	    push (@array, (split(/{/, $_))[0]);
	}

	return join($delim, @array);
    }
}

=pod

=head1 AUTHOR

Evan Chiu (evan@netapp.com)

=head1 COPYRIGHT

Copyright (c) 2008 Network Appliance, Inc. All rights reserved.

=cut

1;
