# -*- perl -*-
#  $Id: //depot/prod/test/Rsierranevada/lib/Dense.pm#1 $
#
## @summary 
##    Library routines for SIS (Single Instance Stroage) volume 
##
## @author dl-dpgqa-automation@netapp.com
## @author rani@netapp.com
## @author karthiks@netapp.com
## @status in-progress
## @pod here

=head1 NAME

Dense - Basic routines for SIS volumes 

=head1 DESCRIPTION

This module defines routines for SIS volumes.  

=head1 FUNCTIONS

SIS Core funtions

=cut

package Dense;

BEGIN {
    use base qw(Exporter);
    my  @conf = qw(
                   dense_config
                   dense_config_unsched
                   dense_toggle
                   dense_toggle_license
                   );

    my @action = qw(
                    dense_check
                    dense_clone
                    dense_off_undo
                    dense_share
                    dense_share_vp3
                    dense_start
                    dense_stop 
                    dense_undo
                    dense_unpack
                    dense_clone_start
                    dense_clone_stop
                    dense_revert_to
		    revert_sis_volumes
                    ); 

    my @status = qw(
                    dense_stat
                    dense_status
                    dense_clone_status
					dense_get_state
                    );
    my @state = qw(
                   dense_get_config
                   dense_get_progress
                   dense_get_last_op_size
                   dense_get_stat_item
                   dense_get_status_item
                   dense_is_licensed
                   dense_set_license
                   dense_remove_license
                   dense_deleted_inode_hash_state
                   );

    my @util   = qw(
                    get_changelog_opt
                    get_dense_vols
                    get_changelog_info
                    parse_dense_check
                    get_share_vols
                    parse_clone_status
					clone_clear_status
					clone_clear_failed
                    is_dense_flag
                    set_filer_sync
                    check_test_privilege
					remove_leading_trailing_spaces
					dense_create_checkpoint
                    dense_num_gatherer_scanners
					dense_sort_gen
					dense_sort_run
					dense_vol_limit
                    );


    my @verify = qw(
                    dense_verify_changelog
                    dense_verify_changelog_opt
                    dense_verify_config
                    dense_verify_flag
                    dense_verify_stage
                    dense_verify_stat
                    dense_verify_state
                    dense_verify_status
                    dense_verify_status_error
                    dense_wait_for
                    dense_wait_for_stage
                    dense_wait_for_status
		    dense_wait_for_progress
                    dense_wait_for_field
                    dense_clone_wait_for_finish
                    dense_verify_checkpoint
                    );

    my @comp   = qw(
                    $ENABLE_COMPRESSION
					$DEFAULT_DLOG_SIS_FILES
					$DENSE_ONTAP_VERSION
                    $RR_BRANCH
                    );

    our %EXPORT_TAGS = ( conf   => [ @conf ],
                         action => [ @action ],
                         status => [ @status ],
                         state  => [ @state ],
                         util   => [ @util ], 
                         verify => [ @verify ],
			 comp   => [ @comp ]
                       );

    $EXPORT_TAGS{'all'} = [@conf, @status, @action, @state, @util, @verify, @comp];

    Exporter::export_ok_tags('all');
}#BEGIN #1

use strict;
use SPARSE::Util::Logger;
use filer;
use Burt;
use Tharn qw(connect host say logcomment logresult param tstlog addresult fatal_result);
use Utils qw(make_conn);
use Data::Dumper qw(Dumper);
use Dense::Status; 
use Dense::Constant::Command;
use Dense::Constant::Message;
use Filer::Version;
use Params::Validate qw(validate validate_pos OBJECT SCALAR BOOLEAN HASHREF ARRAYREF UNDEF);
use FSR::FSRutils;
use Date::Manip;
use Time::ParseDate;
use snaplock qw(system_time_set system_time_get);
use VFILER::Vfiler;
use Volume qw(volume_options_list_info);
use SnapCmds;
use Filer::Version qw(get_wafl_version);

BEGIN {
Params::Validate::validation_options(
    on_fail => sub {
    	my @msgs = @_; 
	chomp(@msgs);
        Carp::confess(Tharn::script_result(@msgs))
	}
   );
} 
#BEGIN #2

use FS::Util::filer_messages;
use Volume qw (vol_list vol_status);
use vars qw($ENABLE_COMPRESSION $DEFAULT_DLOG_SIS_FILES $DENSE_ONTAP_VERSION $RR_BRANCH);

=head2 dense_vol_limit(host=>$host, aggr=>0, release=>$release, model=>$model, partner=>$partner)

Get dense volume size limit from the filer

=over 4

=item host

Filer HostSpec

=item aggr
optional default is dense_aggr
aggr name on which this vol is present

=item release
optional

Filer release version

=item model
optional

find size limit on this model

=item partner

0  = run command in regular mode or 1 = run command in takeover mode on partner

=item Returns:

Dense volume size limit on the filer for the aggr type

=over 4

Return message returned by cli 

=back 

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut


sub dense_vol_limit($@) {
	my (%opts) = validate(@_, {
			host => {type=>SCALAR},
			aggr => {type=>SCALAR, default=>"dense_aggr"},
			release => {type=>SCALAR, optional=>1},
			model => {type=>SCALAR, optional=>1},
			partner => {type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/} 
			} 
		      );

	my ($host) = $opts{host};
	my ($conn) = make_conn($host,"#rsh");
	my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);
        my $large_aggr = Dense::Utils::dense_is_larger_aggr(conn=>$conn, aggr_name=>$opts{aggr});
	my $cmd;

#        if (((defined $opts{release}) && ($opts{release} eq "RR")) || $DENSE_ONTAP_VERSION eq "RR") {
#            die "Volume limits for 64bit Aggrs in RR is not finalized yet" if($large_aggr);
            return("14716g") if(!$large_aggr);
#        }

### Using hash from Messages_* till sis max_vol_size gets fixed
         my $model = ((defined $opts{model})?$opts{model}:filer_model_get($conn));
         logcomment("Model is $model");
         return((GetMessage(host=>$host, code=>"LA_VOL_SIZE_LIMIT"))->{$model});
### burt406659 - sis max_vol_size not giving the proper limits. Enable below code after that burt is fixed.
#	if ($opts{partner}) {
#		$cmd = 	"partner " . $DENSE_TEST_CMD . 
#			"partner " . DENSE_VOL_LIMIT_NAME . $opts{model};
#	} else {
#		$cmd = $DENSE_TEST_CMD . DENSE_VOL_LIMIT_NAME . $opts{model};
#	}
#
#	my ($return_text, $result_code) = filer_cli_say($conn, $cmd);
#
#	# No errror is return
#	if ($result_code eq cmdOkReturn) {
#                my ($model, $limit, $lalimit);
#                if($return_text =~ /^Max dense volume size for (.*) is (.*) or .*(but on larger aggregates we support..(.*) max dense vol size)?$/ms) {
#                   ($model, $limit, $lalimit) = ($1, $2, $4);
#                   $limit =~ s/\s+//;
#                   $lalimit =~ s/\s+//;
#                   (defined $opts{model}) && ($opts{model} ne $model) && die("filer gives vol limits for different model $model than the one asked for $opts{model}");
#                   logcomment("Model: <<$model>> Limit: <<$limit>> LA Limit: <<$lalimit>>");
#                   return($large_aggr?$lalimit:$limit);
#                }
#		die("$return_text");
#	} else {
#		die("$return_text\n\n");
#	}
}

=head2 dense_check(host=>$host, path_name=>$path_name, cmd_opt=>$cmd_opt, partner=>$partner)

Verify and update the fingerprint database. 

=over 4

=item host

Filer HostSpec

=item path_name

Path name, volume name in /vol/volName or aggr name

=item cmd_opt

-c or -v option

=item partner

0  = run command in regular mode or 1 = run command in takeover mode on partner

=item Returns:

Success

=over 4

Return message returned by cli 

=back 

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut


sub dense_check($@) {
	my (%opts) = validate(@_, {
			host => {type=>SCALAR},
			path_name => {type=>SCALAR},
			cmd_opt => {type=>SCALAR, optional=>1},
			partner => {type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/} 
			} 
		      );

	my ($host) = $opts{host};
	my ($path_name) = $opts{path_name}; 
	my ($cmd_opt) = $opts{cmd_opt};

	my ($conn) = make_conn($host,"#rsh");
	my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);

	my %returnInfo = ();

	my $cmd;

	if ($opts{partner}) {
		if ($cmd_opt) {
			$cmd = 	"partner"." " . $DENSE_TEST_CMD . 
				"partner". " " .  DENSE_CHECK_NAME .  
				$cmd_opt . " " . $path_name;
		} else {
			$cmd = 	"partner"." " . $DENSE_TEST_CMD . 
				"partner". " " . DENSE_CHECK_NAME . $path_name;
		} 
	} else {
		if ($cmd_opt) {
			$cmd = $DENSE_TEST_CMD . DENSE_CHECK_NAME .  
				$cmd_opt . " " . $path_name;
		} else{
			$cmd = $DENSE_TEST_CMD . DENSE_CHECK_NAME . $path_name;
		}
	}

	my ($return_text, $result_code) = filer_cli_say($conn, $cmd);

	# No errror is return
	if (($cmd_opt =~ /\-c/) || ($result_code eq cmdOkReturn)) {
		return $return_text;
	} else {
		die("$return_text\n\n");
	}
}

=head2 dense_config(host=>$host, path_name=>$path_name, partner=>$partner, schedule_string, test_mode, time_order, ignore_undef, throttle, compression, unsupported)

Used to configure the schedule of de-duplication process of a volume.

=over 4

=item host

Filer HostSpec

=item path_name

Path name, volume name in /vol/volName

=item unsupported

Optional, string of anything. For negative test most of time.

=item schedule_string

Optional, string of schedule if you do not want to spcify in day, hour or 
minute options (requested). If this is not null, day, hour, munite, time 
order, ignore_undef are ignored all together, only the schedule_string 
will be passed. Default is null. 

=item test_mode

Optional, either 0 or 1. If test_mode=>1, will invoke command under priv 
set test. Otherwise, invoke command normally. Default is test_mode=>0.

=item throttle

Optional, a string. Ontap supports on and off, default is on if not specified.

=item Returns:

Success

=over 4

Return 1

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_config($@)
{
	my (%opts) = validate(@_, { host => { type=>SCALAR|HASHREF },
			path_name => { type=>SCALAR },
			schedule_string => { type=>SCALAR | UNDEF, default=>"", optional=>1 },
                        schedule => { type=>SCALAR | UNDEF, default=>"", optional=>1 },
			test_mode => { type=>SCALAR, default=>"0", optional=>1, regex=>qr/^(0|1)$/ }, 
			time_order => { type=>SCALAR | UNDEF, optional=>1 },
			ignore_undef => { type=>SCALAR | UNDEF, optional=>1 },
			throttle => { type=>SCALAR, default=>"on", optional=>1 },
            'inline-compression' =>  { type=>SCALAR, optional=>1 },
            'background-compression' => { type=>SCALAR, optional=>1 },
			unsupported => { type=>SCALAR | UNDEF, optional=>1 },
			partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ },
			mbs => {type=>SCALAR | UNDEF , optional=>1 },	 
			'verify-trigger-rate' => {type=>SCALAR , optional=>1 }	} 
			);

	my ($host) = $opts{host};
	my ($path_name) = $opts{path_name};
	my ($sched_string) = $opts{schedule_string};
        my ($sched) = $opts{schedule};
	my ($test_mode) = $opts{test_mode};
	my ($time_order) = $opts{time_order};
	my ($ignore_undef) = $opts{ignore_undef};
	my ($throttle) = $opts{throttle};
	my ($inline_compression) = $opts{'inline-compression'};
    my ($background_compression) = $opts{'background-compression'};
	my ($unsupported) = $opts{unsupported};
    my ($mbs) = $opts{mbs};
    my $verify_rate = $opts{'verify-trigger-rate'};
	my ($conn) = make_conn($host,"#rsh");
	my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);

	my $cmd;
    
	## if host is in BR  we can set the MBS options as part of config otherwise not 
	if(($opts{mbs}) && ($DENSE_ONTAP_VERSION != /BR/)){
	   die("MBS value can be set in sis config only in BR and not in other versions");
	}
	if ( $opts{partner} ) {
		$cmd = "partner" . " ". $DENSE_TEST_CMD . "partner" . " ".  DENSE_CONFIG_NAME; 
	} else {
		$cmd = $DENSE_TEST_CMD . DENSE_CONFIG_NAME; 
	}

    $inline_compression = $opts{'inline-compression'} ? $opts{'inline-compression'} : "false";
	$background_compression = $opts{'background-compression'} ? $opts{'background-compression'} : "false";


# sched_string is specified, should skip the time construct part
	if ($sched_string){
		$cmd = $cmd . " -s " . $sched_string;
	}elsif($sched){
                 $cmd = $cmd . " -s " . $sched;
        } 
        elsif($opts{mbs}){
	   $cmd = $cmd." -m ".$mbs;
	}elsif($opts{'verify-trigger-rate'}){
       $cmd = $cmd." -r ".$verify_rate;
    } elsif(($opts{'background-compression'}) || ($opts{'inline-compression'})){
	   $cmd = $cmd." -C ".$background_compression." -I ".$inline_compression;
	}

# add path_name
	$cmd = $cmd . " $path_name";

	my $return_text;
	my $result_code;

	if ($opts{test_mode} && $opts{test_mode} == 1){
		($return_text, $result_code) = filer_cli_test_say($conn, $cmd);
	}else{
		($return_text, $result_code) = filer_cli_say($conn, $cmd);
	}

	# result_code returned is 0 when dense config is not found (No Config entry found.) hence this is not reported as failure. 
	if($return_text =~ GetMessage(host=>$host, code=>"DENSE_CONFIG_NOT_FOUND")){
		$result_code = 1;
	}
# No error is return
	if( $result_code eq cmdOkReturn){
		return 1;
	}else{
		die ("$return_text\n");
	}
}

=head2 dense_config_unsched(host=>$host, path_name=>$path_name, schedule_string=>$schedule_string, partner=>$partner)

Used to configure the schedule of de-duplication process of a volume.

=over 4

=item host

Filer HostSpec

=item path_name

Path name, volume name in /vol/volName

=item schedule_string

optional, schedule string for unscheduling, defaults to "-"

=item Returns:

Success

=over 4

Return 1 

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_config_unsched($@)
{
    my (%opts) = validate(@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR }, 
                                schedule_string => { type=>SCALAR, default=>"-", optional=>1 },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } }
			 );

    my ($host) = $opts{host};
    my ($path_name) = $opts{path_name};
    my ($schedule_string) = $opts{schedule_string};
    my ($partner) = $opts{partner};

    eval { 
	dense_config(host=>$host, path_name=>$path_name, schedule_string=>$schedule_string, partner=>$partner);
    };
	
    if ($@){
	die("$@\n");
    }else{
	return 1;
    }
}


=head2 dense_off_undo(host=>$host, path_name=>$path_name, partner=>$partner)

Convert a dense volume to a normal volume. Kick off dense off, then 
dense_undo, will not wait the process to finish.

=over 4

=item host

Filer HostSpec

=item path_name

Valid path name (for phase one, volume name)

=item force

Set to 1 will force sis undo to run even when volume is full, debug only option, defaults to 0.

=item Returns:

Success

=over 4

Return Success message
The dedup operation on volume "/vol/spring" is unconfigured, and all the 
data sharings in the volume are reverted.

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_off_undo($@) 
{
   my (%opts) = validate(@_, { host => { type=>SCALAR },
                               path_name => { type=>SCALAR },
			       force => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ }, 
		               partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } }
                         );

    my ($host) = $opts{host};
    my ($path_name) = $opts{path_name};

    # dense off first
    eval {
	dense_toggle(host=>$host, path_name=>$path_name, state=>DENSE_OFF_NAME, partner=>$opts{partner});
    };

    # Want die here instead of toggle function
    if ($@) {
	die("$@\n");
    }

    eval {
	dense_undo(host=>$host, path_name=>$path_name, partner=>$opts{partner}, force=>$opts{force});
    }; 

    # check result
    if($@) { 
	die("$@\n");
    }
}

=head2 dense_start(host=>$host, path_name=>$path_name, nosync=>$nosync, scan=>$scan, reuse_ckpt=>$reuse_ckpt partner=>$partner)

Manually kick off the de-duplication process on a valid path.
In phrase one, only volume is supported.

=over 4

=item host

Filer HostSpec

=item path_name

Path name, Volume name for phase one, in /vol/volName

=item nosync

Optional arguments, defaults to 0

=item scan

Optional arguments, if set to 1 uses the -s -f option with sis start, defaults to 0

=item queue

Optional arguments, if set to 1 uses the -q -f option with sis start to queue a sis operation, defaults to 0
[applicable when already a sis is running]

=item reuse_ckpt

Optional arguments, if set to 1 uses the -p option [applicable only for Gathering checkpoint] with sis start, defaults to 0

=item Returns:

Success

=over 4

Return success message. 

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_start($@) 
{
   my (%opts) = validate(@_, { host => { type=>SCALAR|HASHREF },
                               path_name => { type=>SCALAR },
			       nosync => { type=>SCALAR, default=>"0", optional=>1 },
			       scan => { type=>SCALAR, default=>"0", optional=>1 },
                               queue => {type => SCALAR, default => "0", optional => 1},           
                               reuse_ckpt => { type=>SCALAR, default=>"0", optional=>1 },
                               del_ckpt => { type=>SCALAR, default=>"0", optional=>1 },
			       partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ },
                               compression  => { type=>BOOLEAN, optional=>1}, 
                               'build-metadata' => { type=>SCALAR, default=>"0", optional=>1 },
                               'o' => {type=>SCALAR, default=>"0", optional=>1} }
                        );

    my ($host) = $opts{host};
    my ($path_name) = $opts{path_name};
    my ($conn) = make_conn($host,"#rsh");
    my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);

    # Check for the lack of test privilege
  
    # Run sync to ensure the all new writes have been change-logged
    # unless nosync option is specified 
    if (!$opts{nosync}) {
        filer_cli_say($conn, "priv set -q diag; sync; sync", 300);
    }

    my $cmd;
    
    if( $opts{reuse_ckpt} && $opts{del_ckpt} ) {
        die("Cannot use both reuse and delete checkpoint options at the same time\n");
    }

    my $options;

    if( $opts{scan} ) {
	$options = " -s -f ";
        if( $opts{'build-metadata'} ) {
            $options = "-s -m -f ";
        }
        #sis start -o option for old gathering style
        if( $opts{'o'} ) {
            $options = "-s -o -f ";
        } 
	if( $opts{reuse_ckpt} ) {
            $options = " -s -f -p ";
        }
	if( $opts{del_ckpt} ) {
            $options = " -s -f -d ";
        }
        if( $opts{compression} ) {
            $options = " -s -C -f ";
        }
    } elsif ($opts{queue}) {
             $options = " -q -f ";
    } else {
	$options = " ";
        if( $opts{reuse_ckpt} ) {
            die ("Invalid use of Reuse checkpoint flag with sis start\n");
        }
        if( $opts{del_ckpt} ) {
            $options = " -d -f ";
        }
    }

    if ( $opts{partner} ) {
        $cmd = "partner" . " " . $DENSE_TEST_CMD . "partner" . " " . DENSE_START_NAME . $options . $path_name;
    } else {
	$cmd = $DENSE_TEST_CMD . DENSE_START_NAME . $options . $path_name;
    }

    my ($return_text, $result_code) = filer_cli_say($conn, $cmd);

    # No errror is return
    if( $result_code eq cmdOkReturn){
	return $return_text;
    }else{
	die("$return_text\n");
    }
}

=head2 dense_stat(host=>$host, block=>$block, long=>$long, diag_mode=>$diag_mode, path=>$path, partner=>$partner)

Display statistic of all the dense volumes.
    
=over 4

=item host

Filer HostSpec

=item block

Optional, either 0 or 1. If set to 1, will use -b to display in block format. 
Default is 0.  

=item long

Optional, either 0 or 1. If set to 1, will use -l long list display. 
Default is 0. 

=item diag_mode

Optional, either 0 or 1. If set to 1, will run in diag mode. Default is 0.

=item path

Optional, a path, should be volume name for first phase.  If specified paths, 
only status on those path will be displayed. Use , to seperate different path 
names. If not specified, all dense paths will be displayed.

=item Returns:

Success

=over 4

Return an array of hash table containing stat in key value pair.
Actually it is a hash of a hash. You can get info as the following.

For a short form (no -l) and %stat_info is returned

$stat_info{"/vol/vol0"}{Allocated} contains allocated volume space 
including the unit: 352 MB 
$stat_info{"/vol/flex_vol"}{"Saving"} contains total saving on the 
volume including the unit: 176 MB 
$stat_info{$path_name}{"%Saved"} contains number of percentage: 33 % 

For a long form (-l), %stat_info is returned

$stat_info{$path_name}{Allocated} contains allocated volume space 
including the unit: 90178 
$stat_info{"/vol/flex_summer"}{"Saving"} contains saved by sharing: 
23577 

=back

Failure

=over 4

Return with an error message in C<$@> with result code

=back

=back

=cut

sub dense_stat($@)
{
      my (%opts) = validate(@_, { host => { type=>SCALAR },
                                  block => { type=>SCALAR, default=>"0", optional=>1, regex=>qr/^(0|1)$/ },
                                  long => { type=>SCALAR, default=>"0", optional=>1, regex=>qr/^(0|1)$/ },
                                  verbose => { type=>SCALAR, default=>"0", optional=>1, regex=>qr/^(0|1)$/ },
                                  diag_mode => { type=>SCALAR, default=>"0", optional=>1, regex=>qr/^(0|1)$/ }, 
                                  path=> { type=>SCALAR, default=>"", optional=>1 },
				  partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                         );

    my ($host) = $opts{host};
    my ($path) = $opts{path};
    my ($long) = $opts{long};
    my ($verbose) = $opts{verbose};
    my ($block) = $opts{block};
    my ($diag_mode) = $opts{diag_mode};
    my ($version) = getwv(host=>$host);

    my ($conn) = make_conn($host,"#rsh");
    my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);
 
    my %stat = ();

    my $cmd;

    if ($opts{partner}) {
    	$cmd = "partner" . " " . $DENSE_TEST_CMD . "partner" . " ". DENSE_STAT_NAME; 
    } else {
    	$cmd = $DENSE_TEST_CMD . DENSE_STAT_NAME; 
    }
    my @process = ();

    # Check if long list option is specified
    if ( $opts{long} && $opts{long} == 1 ){
	$cmd = $cmd . "-l ";
	$long = 1;
    }

    # Check if long list option is specified
    if ( $opts{verbose} && $opts{verbose} == 1 ){
	$cmd = $cmd . "-v ";
	$verbose = 1;
    }
    
	# Check if block option is specified
    if ( $opts{block} && $opts{block} == 1 ){
	$cmd = $cmd . "-b ";
	$block = 1;
    }

    # Check if diag_mode option is specified
    if ( $opts{diag_mode} && $opts{diag_mode} == 1 ){
	$diag_mode = 1;
    }

    # Check if there is volume specified
    if ( defined $opts{path} ){
	my @paths = split (/,/, $opts{path});
	    if ( scalar(@paths) > 0){
		foreach my $single (@paths) {
		    $cmd = $cmd . $single;
		} # end for
	    } # end if scalar
    }

    my $return_text;
    my $result_code; 

    # Issue command
    if ( $diag_mode ){
	($return_text, $result_code) = filer_cli_diag_say($conn, $cmd);
    }else{
	($return_text, $result_code) = filer_cli_say($conn, $cmd);
    }
    
    # Parse the return value and return approiately 
    # Process return value
    if($result_code eq cmdOkReturn){
        if ($return_text !~ $Dense::Constant::Message::Dconst{$version}{"DENSE_STAT_NOT_FOUND"}){
	    @process = filer_cli_split($return_text);
	
	    # Process return value
	    # Cannot Parse detail status from diag mode output - if ( $diag_mode == 1 || $long == 1 ){
	    if ( $long == 1 ){
	        %stat = Dense::Status::dense_parse_stat_detail(stat_array=>\@process);
	    }else{
	        %stat = Dense::Status::dense_parse_stat(stat_array=>\@process);
	    }
        }
    }else{
	die("$return_text\n");
    }
    
    return %stat;
}


=head2 dense_status(host=>$host, long=>$long, diag_mode=>$diag_mode, path=>$path, partner=>$partner)

Display status of all the dense volumes and de-duplication operations running 
on them.

=over 4

=item host

Filer HostSpec

=item long

Optional. If specified as 1, long list will be displayed. All others will 
display short list. 

=item diag_mode

Optional.  If value is 1,  filer_cli_diag_say() is used to query the dense
status command in diag mode.  The diag mode will have additional status
fields.

=item path

Optional. If specified paths, only status on those path will be displayed. 
Use , to seperate different path names.

If not specified, all dense paths will be displayed.

=item Returns:

Success

=over 4

Return a hash containing requested status
Actually it is a hash of a hash. If you think we should simply it, I will 
do that.

For short form (without -l) with %status_info returned

$status_info{$volume_path}{State} contains volume_paths current state. 
$status_info{$volume_path}{Status} contains Status of the volume. 

For long form (with -l) with %status_info return

$status_info{$volume_path}{Progress} to get progress information
$status_info{$volume_path}{"Last Processed Size"} to get size information 

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=item Example:

=over 4

dense_status($srcConnection, $opts{long}=>1, $opts{path}=>volumeOne,volumeTwo);
Same as: dense status -l volumeOne volumeTwo

dense_status($srcConnection, $opts{long}=>1);
Same as: dense status -l 

dense_status($srcConnection); 
Same as: dense status

dense_status($srcConnection, $opts{path}=>volumeOne);
Same as: dense status volumeOne

dense_status($srcConnection, $opts{path}=>volumeOne,volumeTwo);
Same as: dense status volumeOne volumeTwo

=back

=back

=cut

sub dense_status($@)
{
      my (%opts) = validate(@_, { host => { type=>SCALAR },
                                  long => { type=>SCALAR, default=>"0", optional=>1, regex=>qr/^(0|1)$/ },
                                  diag_mode => { type=>SCALAR, default=>"0", optional=>1, regex=>qr/^(0|1)$/ }, 
                                  path=> { type=>SCALAR, default=>"", optional=>1 },
				  partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } ,
				  expected_error => { type => SCALAR | ARRAYREF | UNDEF, optional => 1 } }
		);

    my ($host) = $opts{host};
    my ($path) = $opts{path};
    my ($long) = $opts{long};
    my ($diag_mode) = $opts{diag_mode};
    my @expected_errors = ();

    my $errs = $opts{expected_error};
    if( defined $opts{expected_error} )
    {
	    if( ref( $errs ) eq "ARRAY" )
	    {
		@expected_errors = @{$errs};
	    }
	    else
	    {
		push @expected_errors, $errs;
	    }
    
            logcomment("Expected Error: [@expected_errors]");
    }


    my ($conn) = make_conn($host,"#rsh");
    my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);
    my $version = getwv(host=>$host);

    my %status = ();
    my $cmd;

    if ($opts{partner}) {
    	$cmd = "partner" . " " . $DENSE_TEST_CMD . "partner" . " " . DENSE_STATUS_NAME; 
    } else {
    	$cmd = $DENSE_TEST_CMD . DENSE_STATUS_NAME; 
    }
    my @process = ();

    # Check if long list is specified
    if ( defined $opts{long} && $opts{long} == 1 ){
	$cmd = $cmd . "-l ";
	$long = 1; 
    }
    
    # Check if there is volume specified
    if ( defined $opts{path} ){
	my @paths = split (/,/, $opts{path}); 
	if ( scalar(@paths) > 0){
	    foreach my $single (@paths) {
		$cmd = $cmd . $single;
	    } # end for
	} # end if scalar
    }


    # Issue command
    my $return_text;
    my $result_code;
    if ( defined $opts{diag_mode} && $opts{diag_mode} == 1 ){
        ($return_text, $result_code) = filer_cli_diag_say($conn, $cmd);
    } else {
        ($return_text, $result_code) = filer_cli_say($conn, $cmd);
    }
    
    # Parse the return value and return approiately  
    # Process return value 
    if($result_code eq cmdOkReturn){ 
        if ($return_text !~ GetMessage( code => "DENSE_STATUS_NOT_FOUND", host => $host) )
        {
            @process = filer_cli_split($return_text);
   
	    # Process return value 
	    # 
	    if ( $long == 0 ){
	        %status = Dense::Status::dense_parse_status(status_array=>\@process);
	    }else{
	        %status = Dense::Status::dense_parse_status_detail(status_array=>\@process); 
	    }
        } else {
	    $status{Status} = "$return_text";
	}	
    }elsif( $#expected_errors >= 0 )
    {
        my $found = 0;
        foreach my $err ( @expected_errors )
        {
            logcomment("Checking msg [$err] against [$return_text]");
            $found = 1 if( $return_text =~ /$err/ );

	    $status{Status} = "$return_text";
	    last if( $found );
        }
        die("$return_text\n") if( !$found );
    }else{
       
	die("$return_text\n");
    }
    
    return %status;
}

# need inhancement 
=head2 dense_stop(host=>$host, path_name=>$path_name, partner=>$partner)

Abort the de-duplication process or inflation process on path.
Will return after kick off the operation. It will not wait for the process to 
finish.

=over 4

=item host

Filer HostSpec

=item path_name

Path name (for phase one, volume name), should be in format /vol/volumename

=item Returns:

Success

=over 4

Return success message: The operation on "/vol/spring/" is being stopped.

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_stop($@) 
{
   my (%opts) = validate(@_, { host => { type=>SCALAR|HASHREF },
                               path_name => { type=>SCALAR },
			       partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                         );

    my ($host) = $opts{host};
    my ($path_name) = $opts{path_name};
    my ($conn) = make_conn($host,"#rsh");
    my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);
    my $cmd;

    if ($opts{partner}) {
    	$cmd = "partner" . " ". $DENSE_TEST_CMD . "partner" . " " . DENSE_STOP_NAME . $path_name;  
    } else {
    	$cmd = $DENSE_TEST_CMD . DENSE_STOP_NAME . $path_name;  
    }

    my ($return_text, $result_code) = filer_cli_say($conn, $cmd);

    # No errror is return
    if( $result_code eq cmdOkReturn){
	return $return_text; 
    }elsif($return_text =~ /currently idle/i)
    {
	return $return_text;
    }else{
	die("$return_text\n");
    }
}


=head2 dense_toggle(host=>$host, path_name=>$path_name, state=>$state, partner=>$partner)

Turn dense on or off, depending on the value pass in.

=over 4

=item host

Filer HostSpec

=item path_name

Path name, example, flex_vol. It should only be volume name in phase one

=item state

String, either on or off

=item Returns:

Success

=over 4

Return Success message 

The operation has already been enabled for this path.
The operation has already been disabled for this path.

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_toggle($@) 
{
   my (%opts) = validate(@_, { host => { type=>SCALAR|HASHREF },
                               path_name => { type=>SCALAR },
                               state => { type=>SCALAR, optional=>1 },
			       partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                         );
    
    my ($host) = $opts{host};
    my ($path_name) = $opts{path_name};
    my ($state) = $opts{state};

    my ($conn) = make_conn($host,"#rsh");
    my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);

    my $cmd;

    if ($opts{partner}) {
    	if ($state =~ "on") {
		$cmd = "partner" . " " . $DENSE_TEST_CMD . "partner" . " ". DENSE_ON_NAME . "$path_name";
    	} else {
		$cmd = "partner" . " " . $DENSE_TEST_CMD . "partner" . " " . DENSE_OFF_NAME . "$path_name";
    	}
    } else {
	if ($state =~ "on") {
                $cmd = $DENSE_TEST_CMD . DENSE_ON_NAME . "$path_name";
        } else {
                $cmd = $DENSE_TEST_CMD . DENSE_OFF_NAME . "$path_name";
        }
    }

    my ($return_text, $result_code) = filer_cli_say($conn, $cmd); 

    # No errror is return
    if( $result_code eq cmdOkReturn ){
	return $return_text;
    }else{
	die("$return_text\n");
    }
}

=head2 dense_undo(host=>$host, path_name=>$path_name, inode=>$inum, rm_sharing=>wrong, partner=>$partner)

Convert a dense volume to a normal volume. Kick off sis undo for an inode if specified, 
will not wait for the process to finish. 

=over 4

=item host

Filer HostSpec

=item path_name

Valid path name (for phase one, volume name)

=item inode

inode number of the filer to be reverted

=item wrongorall

Set to 1 will make all the sharings to be removed including wrong ones, defaults to 0. Set to 0 removes only the wrong sharings

=item onlylogwrong

Set to 1 will only log wrong sharings and won't remove the sharings, defaults to 0.

=item force

Set to 1 will force sis undo to run even when volume is full, debug only option, defaults to 0.

=item Returns:

Success

=over 4

Return Success message 
The dedup operation on volume "/vol/spring" is unconfigured, and all the data 
sharings in the volume are reverted.

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_undo($@) 
{
   my (%opts) = validate(@_, { host			=> { type=>SCALAR },
                               path_name	=> { type=>SCALAR },
							   inode		=> { type=>SCALAR, optional=>1},
							   rm_sharing	=> { type=>SCALAR, optional=>1, regex=>qr/^(wrong|all)$/},
							   log_wrong	=> { type=>BOOLEAN, optional=>1},
							   force		=> { type=>BOOLEAN, optional=>1}, 
							   partner		=> { type=>BOOLEAN, optional=>1},
							   compression  => { type=>BOOLEAN, optional=>1} } 
                        );

	my $path_name		= $opts{path_name};
	my $conn 			= make_conn($opts{host},"#rsh");
    my $DENSE_TEST_CMD	= check_test_privilege('host' => $conn);
    my $cmd;

    if ($opts{partner}) {
    	$cmd = "partner " . $DENSE_TEST_CMD . "partner ". DENSE_UNDO_NAME . $path_name;
    } else {
    	$cmd = $DENSE_TEST_CMD . DENSE_UNDO_NAME . $path_name;
    }

	$cmd .= " -i $opts{inode}" if ($opts{inode});

	if ($opts{rm_sharing} && $opts{log_wrong}) {
		die("rm_sharing and log_wrong options can not be used together. Either remove sharing or log the wrong sharing");
	}
	
	$cmd .= " -t all" if($opts{rm_sharing} eq "all");
	$cmd .= " -t wrong" if($opts{rm_sharing} eq "wrong");
	$cmd .= " -t wrong -d" if($opts{log_wrong});
    $cmd .= " -C" if ($opts{compression});
	
    if($opts{force}) {
        if($DENSE_TEST_CMD) {
			$cmd = $cmd . " -f";
		} else {
			logcomment("WARNING: Cannot use -f flag with sis undo on a non-debug build");	
		}
    }

    my ($return_text, $result_code) = filer_cli_diag_say($conn, $cmd);

    # check result 
    if( $result_code eq cmdOkReturn ){
		return $return_text; 
    } else {
		die("$return_text\n");
    }
}

=head2 dense_sort_gen(conn=>$conn, path=>$path, num=>$num, partner=>$partner)

Generate fingerprint files with out any data. This is an test-mode only command

=over 4

=item conn

Filer Connection

=item path

Valid path name

=item num

number of fingerprint files to create

=item Returns:

nothing

=over 4

=back

Failure

=over 4

dies with an error message

=back

=back

=cut

sub dense_sort_gen($@)
{
	my (%opts) = validate(@_, { conn	=> { type=>HASHREF },
								path	=> { type=>SCALAR },
								num		=> { type=>SCALAR, regex=>qr/^\d+$/ },
								timeout => { type=>SCALAR, optional=>1 },
								partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                       );
    my $DENSE_TEST_CMD	= check_test_privilege('host' => $opts{conn});
    my $cmd;

    if ($opts{partner}) {
    	$cmd = "partner " . $DENSE_TEST_CMD . "partner ". DENSE_SORT_NAME . "gen $opts{num} $opts{path}";
    } else {
    	$cmd = $DENSE_TEST_CMD . DENSE_SORT_NAME . "gen $opts{num} $opts{path}";
    }

    $opts{timeout} = 3600 unless $opts{timeout};

	my ($text, $result) = filer_cli_test_say($opts{conn}, $cmd, $opts{timeout});
	
	die(DENSE_SORT_NAME."gen command failed saying $text") if ($result ne cmdOkReturn);
}

=head2 dense_sort_run(conn=>$conn, ipath=>$input_path, opath=>$output_path, num=>$num, workers=>$num_workers, partner=>$partner)

Sort fingerprint file created with gen. This is an test-mode only command

=over 4

=item conn

Filer Connection

=item ipath

Valid input path name

=item opath

Valid output path name

=item num

number of fingerprint files to create

=item workers

number of workers, default takes it as 2

=item Returns:

memory allocated for this sort

=over 4

=back

Failure

=over 4

dies with an error message

=back

=back

=cut

sub dense_sort_run($@)
{
	my (%opts) = validate(@_, { conn	=> { type=>HASHREF },
								ipath	=> { type=>SCALAR },
								opath	=> { type=>SCALAR },
								num		=> { type=>SCALAR, regex=>qr/^\d+$/ },
								workers	=> { type=>SCALAR, default=>2, regex=>qr/^\d+$/ },
								timeout => { type=>SCALAR, optional=>1 },
								partner => { type=>SCALAR, default=>0, regex=>qr/^(0|1)$/ } } 
                       );
    my $DENSE_TEST_CMD	= check_test_privilege('host' => $opts{conn});
    my ($cmd, $lines);
	my $pat = "for memory ";

	my $conn = $opts{conn};

    if ($opts{partner}) {
    	$cmd = "partner " . $DENSE_TEST_CMD . "partner ". DENSE_SORT_NAME . "run $opts{num} $opts{workers} $opts{ipath} $opts{opath}";
     } else {
    	$cmd = $DENSE_TEST_CMD . DENSE_SORT_NAME . "run $opts{num} $opts{workers} $opts{ipath} $opts{opath}";
    }

    $opts{timeout} = 3600 unless $opts{timeout};

	my ($match) = say($opts{conn}, $cmd, '-noprompt', '-wait', "$pat.*\.\r", '-timeout', $opts{timeout});
	die(DENSE_SORT_NAME."run command failed to get requested memory in the form \"Request for memory (<num of buckets>, <mem allocated>): <vol>\", but got \"$match\"") if ($match !~ /$pat\((\d+), (\d+)\): (.*)\./);
	logcomment("memory allocated to sis sort run for $1 records in $3 volume is $2");
	
	return($2);
}

=head2 dense_unpack(host=>$host, tar_file=>$tar_file, unpack_dir=>$unpack_dir, avg_share_size=>$avg_share_size, incremental=>$incremental, timeout=>$timeout, partner=>$partner)

Do a VP3 unpack of a tar file into smaller files in a directory.
This is a wrapper for the "dense unpack" command.

=over 4

=item host

Filer HostSpec

=item tar_file

Full path of the tar file to be unpacked, e.g. "/vol/dense_vol/TEST/donor".

=item unpack_dir

Full path of the directory that contains the smaller unpacked files.

=item avg_share_size

Average number of shared blocks.

=item Following are Options for dense unpack operation. Optional.

=item incremental

Perform an incremental unpack.

=item timeout

Timeout value for the "dense share" command. Optional, default is 3600 seconds.

=item Returns:

Success

=over 4

A hash with these keys:
common: SHARE_ERR
non-incremental: HOLE_FILL_SIZE
incremental: SHARE_SIZE, HOLE_PUNCH_SIZE

=back

Failure

=over 4

Return with an error message in C<$@> with result code

=back

=back

=cut

sub dense_unpack($@)
{
  my (%opts) = validate(@_, { host => { type=>SCALAR },
			      tar_file => { type=>SCALAR },
			      unpack_dir => { type=>SCALAR },
			      avg_share_size => { type=>SCALAR },
			      incremental => { type=>SCALAR, optional=>1 },
			      timeout => { type=>SCALAR, optional=>1 },
			      partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                       );

	my $host=$opts{host};
	my $tar_file=$opts{tar_file};
	my $unpack_dir=$opts{unpack_dir};
	my $avg_share_size=$opts{avg_share_size};
	
	my ($conn)=make_conn($host,"#rsh");
        my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);
    	my $cmd;

    my $options = "";
    if ( defined $opts{incremental} && $opts{incremental} == 1 ){
    	$options .= " -i "; 
    }
    $opts{timeout} = 7200 unless $opts{timeout};

    my %results = ();

   if ($opts{partner}) {
    	$cmd = "partner" . " " . $DENSE_TEST_CMD . "partner" . " ". DENSE_UNPACK_NAME . $options . $tar_file . " " . $unpack_dir .  " " . $avg_share_size;
   } else {
    	$cmd = $DENSE_TEST_CMD . DENSE_UNPACK_NAME . $options . $tar_file . " " . $unpack_dir .  " " . $avg_share_size;
   }

    my ($return_text, $result_code) = filer_cli_say($conn, $DENSE_TEST_CMD . $cmd, {timeout=>$opts{timeout}});

    # check result
    if ( $result_code eq cmdOkReturn ) {
	# parse the output and return the result as an array
	# XXX - need to handle possible error messages
	my @pairs = split " ", $return_text;

	foreach my $pair (@pairs) {
	    my ($key, $value) = split /=/, $pair;
	    $results{$key} = $value;
	}
        return %results;
    } else {
        die("$return_text\n");
    }
}


=head2 dense_share(host=>$host, donor=>$donor, dfbn=>$dfbn, recipient=>$recipient, rfbn=>$rfbn, blk_cnt=>$blk_cnt, r_hole=>$r_hole, d_hole=>$d_hole, maxref=>$maxref, no_cmp=>$no_cmp, timeout=>$timeout, partner=>$partner)

Share a range of blocks from donor file to recipient file.

=over 4

=item host

Filer HostSpec

=item donor

Full path of the donor file, e.g. "/vol/dense_vol/TEST/donor".

=item dfbn

Start FBN in the donor file.

=item recipient

Full path of the recipient file, e.g. "/vol/dense_vol/TEST/recipient".

=item rfbn

Start FBN in the recipient file.

=over 4

Options for dense share operation. Optional.

=item blk_cnt

Number of blocks in the range. Default is 1.

=item no_cmp

No data block comparison when this is true, so the blocks with
different content can be shared.

=item hole

Allow holes in the ranges from the donor file and the recipient file
if this is true.

=item maxref

Won't fail the share operation if the donor block hits the maximum
ref_count. The recipient will make a copy of the block instead of
sharing.

=item timeout

Timeout value for the "dense share" command. Optional, default is 3600 seconds.

=item Returns:

Success

=over 4

Return Success message

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_share($@)
{
    my (%opts) = validate(@_, { host => { type=>SCALAR },
				donor => { type=>SCALAR },
				dfbn => { type=>SCALAR },
				recipient => { type=>SCALAR },
				rfbn => { type=>SCALAR },
				blk_cnt => { type=>SCALAR, default=>"1", optional=>1 },
				no_cmp => { type=>SCALAR|UNDEF, optional=>1 },
				r_hole => { type=>SCALAR|UNDEF, optional=>1 },
				d_hole => { type=>SCALAR|UNDEF, optional=>1 },
				maxref => { type=>SCALAR|UNDEF, optional=>1 },
				timeout => { type=>SCALAR|UNDEF, optional=>1 }, 
				retval => { type=>SCALAR|UNDEF, optional=>1 },
				rettxt => { type=>SCALAR|UNDEF, optional=>1 },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
			 );

    my ($host)=$opts{host};
    my ($donor)=$opts{donor};
    my ($dfbn)=$opts{dfbn}; 
    my ($recipient)=$opts{recipient}; 
    my ($rfbn)=$opts{rfbn};

    my ($conn) = make_conn($host,"#rsh");
    my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);
    if (!$DENSE_TEST_CMD) {
        logresult('FAIL',
                  "Block sharing only available in builds with test privilege");
    }

    my $options = "";
    my $cmd;
    my $opt2    = "";
    my $opt3    = "";
	$cmd = "priv set test; sis share -h";
	my ($ret,$ret_code) = filer_cli_say($conn,  $cmd, {timeout=>$opts{timeout}});
	if($ret =~/-dhole/ && $ret=~/-rhole/){
		if ( defined $opts{no_cmp} && $opts{no_cmp} == 1 ){
			$options .= " -nocmp "; 
		}
		if ( defined $opts{d_hole} && $opts{d_hole} == 1 ){
			$options .= " -dhole ";
		}
		if ( defined $opts{r_hole} && $opts{r_hole} == 1 ){
			$options .= " -rhole ";
		}
		if ( defined $opts{maxref} && $opts{maxref} == 1 ){
			$options .= " -maxref ";
		}
		if( defined $opts{dfbn}){
		$opt2 = " -ds " . $dfbn ;
		}
		if( defined $opts{rfbn}){
		$opt3 = " -rs " . $rfbn ;
		}
		$opts{blk_cnt} = 1 unless $opts{blk_cnt};
		$opts{timeout} = 3600 unless $opts{timeout};
		
		if ($opts{partner}) {
			$cmd = "partner" . " " . DENSE_TEST_CMD . "partner" . " ". DENSE_SHARE_NAME . $options . " -df " . $donor . " " . $opt2 . " " . " -rf ". $recipient . " " . $opt3 . " " . " -nblks " . $opts{blk_cnt};
		} else {
		$cmd = DENSE_TEST_CMD . DENSE_SHARE_NAME . $options . " -df " . $donor . " " . $opt2 . " " . " -rf ". $recipient . " " . $opt3 . " " . " -nblks " . $opts{blk_cnt}; 
		}
	}else{
		if ( defined $opts{no_cmp} && $opts{no_cmp} == 1 ){
			$options .= " -c "; 
		}
		if ( defined $opts{d_hole} && $opts{d_hole} == 1 ){
			$options .= " -d ";
		}
		if ( defined $opts{r_hole} && $opts{r_hole} == 1 ){
			$options .= " -r ";
		}
		if ( defined $opts{maxref} && $opts{maxref} == 1 ){
			$options .= " -m ";
		}
		$opts{blk_cnt} = 1 unless $opts{blk_cnt};
		$opts{timeout} = 3600 unless $opts{timeout};
		
		if ($opts{partner}) {
			$cmd = "partner" . " " . DENSE_TEST_CMD . "partner" . " ". DENSE_SHARE_NAME . $options . $donor . " " . $dfbn . " " . $recipient . " " . $rfbn . " " . $opts{blk_cnt};
		} else {
		$cmd = DENSE_TEST_CMD . DENSE_SHARE_NAME . $options . $donor . " " . $dfbn . " " . $recipient . " " . $rfbn . " " . $opts{blk_cnt};
		}
	}

    my ($return_text, $result_code) = filer_cli_say($conn,  DENSE_TEST_CMD . $cmd, {timeout=>$opts{timeout}});

    # Default the expected return code to cmdOkReturn.
    if (!defined($opts{retval})) {
	logcomment("No return value is expected, defaulting to cmdOkReturn=". cmdOkReturn);
    	$opts{retval} = cmdOkReturn;
    }

    # Check against the expected return value.
    if ($result_code ne $opts{retval}) {
	#die("Unexpected return value: $result_code, $opts{retval}, $return_text\n");
    	die("$return_text\n");
    }

    # Match the returned text.
    if ($opts{rettxt} && $return_text !~ $opts{rettxt}) {
    	die("Mismatched return message: $return_text\n");
    }
}

=head2 dense_share_vp3(host=>$host, donor=>$donor, dfbn=>$dfbn, recipient=>$recipient, rfbn=>$rfbn,  blk_cnt=>$blk_cnt, r_hole=>$r_hole, d_hole=>$d_hole, maxref=>$maxref, no_cmp=>$no_cmp, timeout=>$timeout, partner=>$partner)

Wraqpper function of dense_share() for VP3 related testing.
In this function, the options no_cmp, hole, and maxref are always set to true.

=over 4

Same as dense_share(), except that the blk_cnt of the shared range is an 
optional argument to the function instead of an element inside the opts.

=back

=cut

sub dense_share_vp3($@)
{
    my (%opts) = validate(@_, { host => { type=>SCALAR },
                                donor => { type=>SCALAR },
                                dfbn => { type=>SCALAR },
                                recipient => { type=>SCALAR },
                                rfbn => { type=>SCALAR },
                                blk_cnt => { type=>SCALAR, default=>"1", optional=>1 },
                                no_cmp => { type=>SCALAR, default=>"1", optional=>1 },
                                r_hole => { type=>SCALAR|UNDEF, optional=>1 },
                                d_hole => { type=>SCALAR|UNDEF, optional=>1 },
                                maxref => { type=>SCALAR, default=>"1", optional=>1 },
                                timeout => { type=>SCALAR|UNDEF, optional=>1 },
                                retval => { type=>SCALAR|UNDEF, optional=>1 },
                                rettxt => { type=>SCALAR|UNDEF, optional=>1 },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                         );

    my ($host)=$opts{host};
    my ($donor)=$opts{donor};
    my ($dfbn)=$opts{dfbn};
    my ($recipient)=$opts{recipient};
    my ($rfbn)=$opts{rfbn};

    return 
    	dense_share(host=>$host, donor=>$donor, dfbn=>$dfbn, recipient=>$recipient, rfbn=>$rfbn, blk_cnt=>$opts{blk_cnt}, no_cmp=>$opts{no_cmp}, r_hole=>$opts{r_hole}, d_hole=>$opts{d_hole},  maxref=>$opts{maxref}, timeout=>$opts{timeout}, retval=>$opts{retval}, rettxt=>$opts{rettxt}, partner=>$opts{partner});
}

=head2 dense_clone(host=>$host, source=>$source, clone=>$clone, options=>$option, timeout=>$timeout, partner=>$partner)

Clone all vbns from source file and clone file.

=over 4

=item host

Filer HostSpec

=item source

Full path of source file

=item clone

Full path of clone file

=item timeout

Optional, default is 3600 seconds.

=item Returns:
      
Success

=over 4
       
Return message from cli on success

=back
       
Failure

=over 4
       
Return error message from cli on failure

=back
       
=back

=cut

sub dense_clone($@)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR },
				 source => { type=>SCALAR },
				 clone => { type=>SCALAR },
				 options => { type=>SCALAR, optional=>1 },
				 timeout => { type=>SCALAR, default=>"3600", optional=>1 },
				 partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } }
			  );

    my ($host)=$opts{host};
    my ($source)=$opts{source};
    my ($clone)=$opts{clone};
    my ($timeout)=$opts{timeout};
    my ($options)=$opts{options};

    my ($conn) = make_conn($host,"#rsh");
    my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);
    if (!$DENSE_TEST_CMD) {
        logresult('FAIL',
                  "Block cloning only available in builds with test privilege");
    }
    my $cmd;

    $timeout = 3600 unless $timeout;

    if(defined($options)) {
	$options = " " . $options . " ";
    } else {
	$options = "";
    }

    if($opts{partner}) {
    	$cmd = "partner" . " ". DENSE_TEST_CMD . "partner" . " ". DENSE_CLONE_NAME . $options . $source . " " . $clone;
    } else {
    	$cmd = DENSE_TEST_CMD . DENSE_CLONE_NAME . $options . $source . " " . $clone;
    }

    my ($return_text, $result_code) = filer_cli_say($conn, DENSE_TEST_CMD . $cmd, {timeout=>$timeout});

    # check result
    if ( $result_code eq cmdOkReturn ) {
        return $return_text;
    } else {
        die("$return_text\n");
    }
}

=head2 dense_start_clone(host=>$host, source=>$path_name, clone=> $clone partner=>$partner 
                          options => '-n' timeout => 36000, src_fbn => 20, dst_fbn => 40, fbn_cnt => 20)

start clone operation

=over 4

=item host

Filer HostSpec

=item source

clone source

=item clone

clone destination

=item partner


=item options

the -n and -l options

=item timeout

Timeout

=item src_fbn

starting fbn number of the source

=item dst_fbn

starting fbn number of the destination

=item fbn_cnt

The number of fbn that need to cloned

=item Returns:

Success

=over 4

=back

=back

=cut


sub dense_clone_start($@)
{

    my (%opts) = validate (@_, { host => { type=>SCALAR | HASHREF },
                                 source => { type=>SCALAR },
                                 clone => { type=>SCALAR },
                                 options => { type=>SCALAR, optional=>1 },
                                 timeout => { type=>SCALAR, default=>"3600", optional=>1 },
                                 partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ }, 
                                 src_fbn => { type=>SCALAR, optional=>1 },
                                 dst_fbn => { type=>SCALAR, optional => 1},
                                 fbn_cnt => {type=>SCALAR, optional => 1} 
                                }
                          );

    my $host = $opts{host};
    my $source = $opts{source};
    my $clone = $opts{clone};
    my $timeout = $opts{timeout};
    my $options = $opts{options};
    my $src_fbn = $opts{src_fbn};
    my $dst_fbn = $opts{dst_fbn};
    my $fbn_cnt = $opts{fbn_cnt};

    my ($conn) = make_conn($host,"#rsh");
    my $cmd;
    my $cmd_suffix;
    $timeout = 3600 unless $timeout;

    if(defined($options)) {
        $options = " " . $options . " ";
    } else {
        $options = " ";
    }

    if($src_fbn && $dst_fbn && $fbn_cnt) {
       $cmd_suffix = $src_fbn . ':' . $dst_fbn . ':' . $fbn_cnt;
       $options = $options . " " . '-r' ;
    }

    $cmd = DENSE_CLONE_START . " " .  $source . " " . $clone . $options . $cmd_suffix;
    my ($return_text, $result_code) = filer_cli_advanced_say($conn, $cmd, {timeout=>$timeout});

    # check result
    if($return_text =~ m/Clone operation started successfully. ID:\s*(\d+)/) {
        return($1);
    } else { 
        die("Unable to start clone operation: $return_text");
    }
}

=head2 dense_clone_stop(host=>$host, vol=>$path_name, id => $id )

start clone operation

=over 4

=item host

Filer HostSpec

=item vol 

volume name

=item  ID

ID got using the clone start command

=item Returns:

Success

=over 4

=back

=back

=cut

sub dense_clone_stop($@)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR | HASHREF },
                                 vol => { type=>SCALAR },
                                 id  => { type=>SCALAR },
                                 timeout => { type=>SCALAR, default=>"3600", optional=>1 },
                                 partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ },
                                }
                          );

    my $host = $opts{host};
    my $vol = $opts{vol};
    my $id = $opts{id};
    my $timeout = $opts{timeout};
    
    my ($conn) = make_conn($host,"#rsh");
    my $cmd;
    $timeout = 3600 unless $timeout;

    $cmd = DENSE_CLONE_STOP . " " .  $vol . " " . $id;

    my ($return_text, $result_code) = filer_cli_advanced_say($conn, $cmd, {timeout=>$timeout});
    
    #wait for a couple of seconds before you get the status
    sleep(5);
    my %status_info = dense_clone_status( host => $host, vol => $vol, id =>$id);
    if($status_info{$id}{'State'} =~ m/failed/) {
       return(1);
    } else {
       die("Unable to stop the clone\n");
    }
}

=head2 dense_clone_wait_for_finish(host=>$host, vol=>$path_name, id=> $id )

wait for clone operation to finish

=over 4

=item host

Filer HostSpec

=item vol

volume name

=item  ID

ID got using the clone start command

=item Returns:

Success

=over 4

=back

=back

=cut


sub dense_clone_wait_for_finish($@)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR | HASHREF },
                                 vol => { type=>SCALAR },
                                 id  => { type=>SCALAR },
                                 timegap => { type=>SCALAR, default =>"5", optional =>1},
                                 partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ },
                                }
                          );
    my $host = $opts{host};
    my $vol = $opts{vol};
    my $id = $opts{id};
    my $timegap = $opts{timegap};
    my %status_info;

    my $conn = make_conn($host,"#rsh");
    my $cmd;
    %status_info = dense_clone_status( host => $host, vol => $vol, id =>$id);
    while($status_info{$id}{'State'} =~ m/running/) {
       sleep($timegap);
       %status_info = dense_clone_status( host => $host, vol => $vol, id =>$id);
    }
    if($status_info{$id}{'State'} =~ m/failed/) {
       return(0);
    } else {
      return(1); 
    }
}


=head2 dense_get_config(host=>$host, path_name=>$path_name, partner=>$partner)

Get config info of the filer or a path 

=over 4

=item host

Filer HostSpec

=item path_name

Optional, valid path name as /vol/volName. If no specified,
all config entries are returned. 

=item Returns:

Success

=over 4

Return config info of the path_name. It might return a null,
if no path config info is found.

=back

Failure

=over 4

Return error in C<$@>

=back

=back

=cut

sub dense_get_config($@)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR },
				 path_name => { type=>SCALAR },
				 partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
			  );
    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name}; 
    my ($version) = getwv(host=>$host);

    my ($conn) = make_conn($host,"#rsh");
    my $DENSE_TEST_CMD = check_test_privilege('host' => $conn);

    my %list = ();
    my @process = ();
    my $cmd;

    if($opts{partner}) {
    	$cmd = "partner" . " ". $DENSE_TEST_CMD . "partner" . " ". DENSE_CONFIG_NAME; 
    } else {
    	$cmd = $DENSE_TEST_CMD . DENSE_CONFIG_NAME; 
    }
    
    my ($return_text, $result_code) = filer_cli_say($conn, $cmd);
  
    if( $result_code eq cmdOkReturn){
        # parse the return text 
        if ($return_text !~ GetMessage( code => "DENSE_CONFIG_NOT_FOUND", host => $host) ){
            @process = filer_cli_split($return_text);
            %list = Dense::Status::dense_parse_config(config_ref=>\@process); 
        }
    }else{
        die("$return_text\n");
    }
    
    # return whole config list if no path_name is specified
    if($path_name){
        return $list{$path_name}; 
    }else{
        return %list;
    }
}


=head2 dense_get_progress(host=>$host, path_name=>$path_name, partner=>$partner)

Get progress number in KB from dense stat -l $path_name. 

=over 4

=item host

Filer HostSpec

=item path_name

Valid path name

=item Returns:

Success 

=over 4

Return with numbers in KB, it might return 0 KB. If the path is not 
configured, it will die with error "No value found.".

=back

Failure 

=over 4

Return error in C<$@> 

=back

=back

=cut

sub dense_get_progress($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );
    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};

    my ($conn) = make_conn($host,"#rsh");

    my $value = eval{
	dense_get_status_item(host=>$host, item_name=>"Progress", path_name=>$path_name, partner=>$opts{partner});  
    };
	
    if ($@){
	die("$@\n");
    }else{
	return Dense::Status::dense_kb_atoi(kb=>$value);
    }
}


=head2 dense_get_last_op_size(host=>$host, path_name=>$path_name, partner=>$partner)

Get last operation size in KB from dense stat -l $path_name. 

=over 4

=item host

Filer HostSpec

=item path_name

Valid path name

=item Returns:

Success 

=over 4

Return with numbers in KB, it might return 0 KB. If the path is not 
configured, it will die with error "No value found.".

=back

Failure 

=over 4

Return error in C<$@> 

=back

=back

=cut

sub dense_get_last_op_size($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );
    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};

    my ($conn) = make_conn($host,"#rsh");
	
    my $value = eval{
	dense_get_status_item(host=>$host, item_name=>"Last Operation Size", path_name=>$path_name, partner=>$opts{partner});  
    };
	
    if ($@){
	die("$@\n");
    }else{
	return Dense::Status::dense_kb_atoi(kb=>$value);
    }
}


=head2 dense_get_stat_item(host=>$host, item_name=>$item_name, path_name=>$path_name, partner=>$partner)

Get item values from dense stat -l $path_name. If serval items are needed, 
use dense_stat to retrieve data.

=over 4

=item host

Filer HostSpec

item itme_name

String, dedup stat item name, such as Shared, Allocated, Path and etc. Please 
provide the same name as how it is displayed in the output. If there is 
space in it, use double quote, "Total Processed".

=item path_name

Valid path name (for phase one, volume name)

=item Returns:

Successfully get the item

=over 4

Return a string of the value that the path specified contains, it can be empty.

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_get_stat_item($@) 
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                item_name => { type=>SCALAR }, 
                                path_name => { type=>SCALAR },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );
    my ($host)=$opts{host};
    my ($item_name)=$opts{item_name};
    my ($path_name)=$opts{path_name};

    my ($conn) = make_conn($host,"#rsh");

    # get status
    my %stat = eval{
	dense_stat(host=>$host, path=>$path_name, long=>1, partner=>$opts{partner});
    };

    if ($@){
	die("$@\n");
    }else{
	my $current_value = $stat{$path_name}{$item_name};
	return $current_value;
    }
}

=head2 dense_get_status_item(host=>$host, item_name=>$item_name, path_name=>$path_name, partner=>$partner)

Get item values from dense status -l $path_name. If serval items are needed, 
you should use dense_status to retrieve data. 

=over 4

=item host

Filer HostSpec

=item itme_name

String, dedup status item name, such as State, Status, Progress, Schedule, 
Compression and etc.. Please provide the same name as how it is displayed 
in the output. If there is space in it, use double quote, "Last Processed 
Size". 

=item path_name

Valid path name (for phase one, volume name)

=item Returns:

Successfully get the item  

=over 4

Return a string of the value that the path specified contains, it can be 
empty. 

=back

Failure 

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_get_status_item($@) 
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                item_name => { type=>SCALAR },
                                path_name => { type=>SCALAR },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );
    my ($host)=$opts{host};
    my ($item_name)=$opts{item_name};
    my ($path_name)=$opts{path_name};
    
    my ($conn) = make_conn($host,"#rsh");

    # get status
    my %status ;
    eval {
	 %status = dense_status(host=>$host, path=>$path_name, long=>1, partner=>$opts{partner});
    };
	
    if ($@){
	die("$@\n");
    }else{
	my $current_value = $status{$path_name}{$item_name};
	return $current_value;    
    }
}


=head2 dense_get_status_internal(host=>$host, path_name=>$path_name, partner=>$partner)

Get current dense status of the path specified. Return an be ACTIVE, IDLE, 
UNDOING or NOSTATUS.
    
=over 4

=item host

Filer HostSpec

=item path_name

Valid path name (for phase one, volume name), in /vol/volumename

=item Returns:

Success

=over 4

Return one of the following INITIALIZING, ACTIVE, IDLE, UNDOING or NOSTATUS.

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut
    
sub dense_get_status_internal($@) 
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );
    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    
    my ($conn) = make_conn($host,"#rsh");

    my $current_status;
    my $current_op_error;

    # get status
    my %status = eval { 
	dense_status(host=>$host, long=>1, diag_mode=>1, path=>$path_name, partner=>$opts{partner});
    };

    if ($@){
	die("$@\n"); 
    }else{
	$current_status   = $status{$path_name}{Status};
	$current_op_error = $status{$path_name}{"Last Operation Error"};
	if ( $current_status =~ DENSE_STATUS_IDLE){
	    return ("IDLE", $current_op_error);
	}elsif ( $current_status =~ DENSE_STATUS_ACTIVE){
	    return ("ACTIVE", $current_op_error);
	}elsif ( $current_status =~ DENSE_STATUS_UNDOING){
	    return ("UNDOING",  $current_op_error);
	}elsif ( $current_status =~ DENSE_STATUS_INIT) {
	    return ("INITIALIZING", $current_op_error);
	}elsif ( $current_status =~ DENSE_STATUS_PENDING) {
	    return ("PENDING", $current_op_error);
	}elsif (! $current_status ){
	    return ("NOSTATUS", "-");
	}elsif ( $current_status =~ /Downgrading/) {
		return ("DOWNGRADING", $current_op_error);
	}else{
	    # unknown things found
	    # does this OK 
	    die("Unknown status was found $current_status\n");
	}
    }
}

=head2 dense_get_state_internal(host=>$host, path_name=>$path_name, partner=>$partner)

Get current dense state for path_name specified, return can be one of the 
following ENABLED, DISABLED and NOSTATUS.

=over 4

=item host

Filer HostSpec

=item path_name

Valid path name (for phase one, volume name)

=item Returns:

Success

=over 4

Return one of the following ENABLED, DISABLED and NOSTATUS.

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_get_state_internal($@) 
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ }, 
				expected_error => { type=>SCALAR | UNDEF, optional=>1 } } 
                          );
    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    
    my ($conn) = make_conn($host,"#rsh");

    my $expected_error = $opts{expected_error};

    # get status
    my %status = eval {
	dense_status(host=>$host, long=>1, path=>$path_name, partner=>$opts{partner}, expected_error=>$expected_error);
    };
    
    if ($@){
	die("Unexpected operation error: $@\n");
    }else{

	if ( defined($expected_error) && ($status{Status} =~ /$expected_error/) ) {
	    return $status{Status};
	}

	my $current_state;

	$current_state = $status{$path_name}{State};
	
	if ( $current_state eq DENSE_STATE_DISABLED){
	    return "DISABLED";
	}elsif ( $current_state eq DENSE_STATE_ENABLED){
	    return "ENABLED";      
	}elsif ( ! $current_state ){
	    return "NOSTATUS";
	}else{
	    die("Unknown state was found $current_state\n");
	}
    }
}

sub dense_get_progress_internal($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
                                partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ },
                                expected_error => { type=>SCALAR | UNDEF, optional=>1 } }
                          );
    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};

    my ($conn) = make_conn($host,"#rsh");

    my $expected_error = $opts{expected_error};

    # get status
    my %status = eval {
        dense_status(host=>$host, long=>1, path=>$path_name, partner=>$opts{partner}, expected_error=>$expected_error);
    };

    if ($@){
        die("Unexpected operation error: $@\n");
    }else{

            return $status{$path_name}{Progress};
    }
}

=head2 dense_is_licensed(host=>$host)

Find out if dense is licensed.

=over 4

=item host

Filer HostSpec

=item Returns: 

If dense license exists in filer

=over 4 

Return 1. 

=back

If dense license does not exist in filer

=over 4

Return 0.

=back

=back

=cut

sub dense_is_licensed($@) 
{
   my (%opts) = validate (@_, { host => { type=>SCALAR|HASHREF } }
                          );
    my ($host)=$opts{host};
    
    my ($conn) = make_conn($host,"#rsh");

    # in filer.pm there is a function called filer_service_license
    # to add license to the filer.
    # 
    my $return;
    eval {
    	$return = filer_service_is_licensed($conn, DENSE_NAME);
    };
    if($@) {
	my $ERR_UNSUP_PLATFORM = "license: This license is only supported either on NearStore platforms or on platforms operating in NearStore mode.";
	if($@ =~ /$ERR_UNSUP_PLATFORM/) {	
		die("Cannot license DENSE_NAME on this platform\n");	
	}
	else {
		die("Error licensing DENSE_NAME: $@\n");
	}
    	## $return = filer_service_is_licensed($conn, DENSE_OLD_NAME);
    }
	
    if ($return == 0 ){
	return 0;
    } else {
	# Do not care site license or else. If it is licensed, return 1
	return 1;
    }
}


=head2 dense_set_license(host=>$host, enforce_platform=>$enforce_platform)

Add a dense license for the filer

=over 4

=item host

Filer HostSpec

=item OPTIONS

=item enforce_platform

Optional, whether we enforce the nearstore platform requirement for SIS.

=item Returns:

If the filer is licensed

=over 4

Return 1.

=back

If the filer fails to license

=over 4

Return 0.

=back

=back

=cut

sub dense_set_license($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR|HASHREF }, 
				enforce_platform => { type=>SCALAR, default=>"0", optional=>1 } }
                          );
    my ($host)=$opts{host};
    my ($enforce_platform)=$opts{enforce_platform};
    my ($version) = getwv(host=>$host);
    
    my ($conn) = make_conn($host,"#rsh");

	my $license_set = dense_is_licensed(host=>$host);
	my $filer_model = filer_model_get($conn);

	if ($filer_model =~ /^FAS/) {
		filer_service_license($conn, "nearstore_option");
	} elsif ($filer_model =~ /^N/) {
                 filer_service_license($conn, DENSE_NAME);
	} elsif (!$opts{enforce_platform} && $filer_model !~ /^R/) {
		filer_options_set($conn, $Dense::Constant::Message::Dconst{$version}{"DENSE_OPT_ALLOW_PLATFORMS"}, $Dense::Constant::Message::Dconst{$version}{"DENSE_OPT_ON"},0);
	}

	if (!$license_set) {
		eval{
			filer_service_license($conn, DENSE_NAME);
                       # Changes for Licensing compression
                       if($ENABLE_COMPRESSION)
                       {
                       		filer_service_license($conn, COMPRESSION_NAME, COMPRESSION_LICENSE_CODE);
                       }
		};
		if($@) {
			##filer_service_license($conn, DENSE_OLD_NAME);
		        my $ERR_UNSUP_PLATFORM = "license: This license is only supported either on NearStore platforms or on platforms operating in NearStore mode.";
		        if($@ =~ /$ERR_UNSUP_PLATFORM/) {
               			 die("Cannot license DENSE_NAME on this platform\n");
        		}
		        else {
               			 die("Error licensing DENSE_NAME: $@\n");
        		}
    		}

			# Changes for Licensing compression
                       if($ENABLE_COMPRESSION)
                       {
                                filer_service_license($conn, COMPRESSION_NAME, COMPRESSION_LICENSE_CODE);
                       }
	}
	logcomment("dense_set_license(): ONTAP Version - $DENSE_ONTAP_VERSION");

	## In RR and above version, options "licensed_feature.a_sis.enable" should also be turned ON for sis to work
	if( $DENSE_ONTAP_VERSION eq "RR" ){
	    ## As currently this options change is not propagated to semainrr/main/waflmainrr, setting the options inside an eval. eval will be removed when mainN has this code
	    eval {
		Dense::Utils::set_sis_options(conn=>$conn, option_name=>"licensed_feature.a_sis.enable", value=>"on");
	    };
	}

	return dense_is_licensed(host=>$host);
}

=head2 dense_remove_license(host=>$host)

Remove a dense license from the filer

=over 4

=item host

Filer HostSpec

=item Returns:

If the filer is licensed

=over 4

Return 1.

=back

If the filer fails to license

=over 4

Return 0.

=back

=back

=cut

sub dense_remove_license($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR } } );

    my ($host)=$opts{host};
    
    my ($conn) = make_conn($host,"#rsh");

	my $license_set = dense_is_licensed(host=>$host);

	if ($license_set) {
		eval{
			filer_service_unlicense($conn, DENSE_NAME);
		};
		if($@) {
			## filer_service_unlicense($conn, DENSE_OLD_NAME);
        		my $ERR_UNSUP_PLATFORM = "license: This license is only supported either on NearStore platforms or on platforms operating in NearStore mode.";
		        if($@ =~ /$ERR_UNSUP_PLATFORM/) {
               			 die("Cannot license DENSE_NAME on this platform\n");
        		}
		        else {
               			 die("Error licensing DENSE_NAME: $@\n");
		        }
    		}
	}

	return dense_is_licensed(host=>$host);
}


=head2 dense_toggle_license(host=>$host, action=>$action, code=>$code)

License or unlicense dense. 

=over 4

=item host

Filer HostSpec

=item $action

Optional, either 1 or 0 to indicate set or remove the dense license. 
Default is 1.

=item Code

Optional, license code for dense. Default is empty.

=item Returns:

If successfully set or remove dense license

=over 4

Return 1.

=back

If fail to set or remove dense license 

=over 4

Return 0.

=back

=back

=cut

sub dense_toggle_license($@) 
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
				action => { type=>SCALAR, default=>"1", optional=>1, regex=>qr/^(0|1)$/ },
				code => { type=>SCALAR, default=>"", optional=>1 } }	
		          );

    my ($host)=$opts{host};
    my ($code)=$opts{code};
    my ($action)=$opts{action};

    my ($conn) = make_conn($host,"#rsh");

    # default would be failure
    my $return_value = 0;

    # Add license to the filer.

    if($action == 0){
	# unlicense dense
	eval{
		filer_service_unlicense($conn, DENSE_NAME);
	};
	if($@){
		## filer_service_unlicense($conn, DENSE_OLD_NAME);
		my $ERR_UNSUP_PLATFORM = "license: This license is only supported either on NearStore platforms or on platforms operating in NearStore mode.";
	        if($@ =~ /$ERR_UNSUP_PLATFORM/) {
       		         die("Cannot unlicense DENSE_NAME on this platform\n");
        	}
	        else {
       		         die("Error unlicensing DENSE_NAME: $@\n");
        	}

	}
    }else{
	#license dense
	eval{
		filer_service_license($conn, DENSE_NAME, $code);
	};
	if($@){
		## filer_service_license($conn, DENSE_OLD_NAME, $code);
		my $ERR_UNSUP_PLATFORM = "license: This license is only supported either on NearStore platforms or on platforms operating in NearStore mode.";
	        if($@ =~ /$ERR_UNSUP_PLATFORM/) {
       		         die("Cannot license DENSE_NAME on this platform\n");
        	}
	        else {
       		         die("Error licensing DENSE_NAME: $@\n");
        	}
	}
    }

    # Check if license is set
    my $value = dense_is_licensed(host=>$host);

    # unlicense dense                    here is license dense
    if (($action == 0 && $value ==0) || ($action == 1 && $value == 1)){
	$return_value = 1;
    }
	
    return $return_value;
}

=head2 dense_verify_changelog(host=>$host, path_name=>$path_name, string_one=>$string_one, string_two=>$string_two, partner=>$partner, exact=>1)

Verify strings in changlog matche what we expected. 
When both string_one and string_two are digits, we will
compare them with numbers in change logs. 
When one of the string is not digit, we will compare either string 
to DENSE_METAFILE_NONEXIST, when the volume is 
not a sis volume and sis metafile should not exist. 

Call them like this 
dense_verify_changelog(host=>$host, path_name=>$path_name, string_one=>0, string_two=>202);
Both numbers will be verified
or
dense_verify_changelog(host=>$conn, path_name=>$path, string_one=>0, string_two=>DENSE_METAFILE_NONEXIST);
dense_verify_changelog(host=>$conn, path_name=>$path, string_one=>DENSE_METAFILE_NONEXIST, string_two=>2);
The number on the above two cases is ignored  

=over 4

=item host

Filer HostSpec

=item path_name

Volume name as in /vol/volName

=item string_one

First expected value to verify against

=item string_two

Second expected value to verify against

=item exact

optional, defaults to 0, if set to 1 does a exact compare instead of comparing to a range of values (old implementation)

=item Returns:

Success

=over 4

Return 1

=back

No match is found 

=over 4

Return error in C<$@>

=back

=back

=cut

sub dense_verify_changelog($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR }, 
                                string_one => { type=>SCALAR },
                                string_two => { type=>SCALAR },
                                exact => { type=>SCALAR, default=>0 },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );

    my ($host)=$opts{host};
    my ($exact)=$opts{exact};
    my ($path_name)=$opts{path_name};
    my ($one)=$opts{string_one};
    my ($two)=$opts{string_two};

    my ($conn) = make_conn($host,"#rsh");

    my $hostspec = host($host);
    my $ver = getwv(host=>$host);
    my($changelog0_name, $changelog1_name);

    # Number of times to check for changlog information
    my $retries = 1;

    $changelog0_name = $Dense::Constant::Message::Dconst{$ver}{"clog0_name"};
    $changelog1_name = $Dense::Constant::Message::Dconst{$ver}{"clog1_name"};

    logcomment("DEBUG: Changelog 0 is $changelog0_name and Changelog 1 is $changelog1_name");

    my $done = 0;
    my $log_one;
    my $log_two;

    while ( $retries-- )
    {
	    my %log = get_changelog_info(host=>$host, 
				path_name=>$path_name, 
				partner=>$opts{partner});

	    $log_one = $log{$changelog0_name};
	    $log_two = $log{$changelog1_name};

            logcomment("DEBUG: Expected Values - one=$one, two=$two");
            logcomment("DEBUG: Obtained Values - one=$log_one, two=$log_two");

	    # Check if any of them has more than one digit
	    if ($one =~ /\d+/ && $two =~ /\d+/){
		# Check the value
		###################################################
		# Positive 
		#if ( (($log_one >= $one) && ($log_two >= $two)) ||
		#     (($log_one >= $two) && ($log_two >= $one)) ) {
		####################################################
		if ($exact) {
		    logcomment("Doing a exact compare of changelog records");

		    if ( ($log_one == $one) && ($log_two == $two) ) {
		        ## Is changelog.1 equal to value 1 and changelog.0 equal to value 0
			$done = 1;
		    } elsif ( ($log_two == $one) && ($log_one == $two) ) {
		        ## Is changelog.0 equal to value 1 and changelog.1 equal to value 0
			$done = 1;
		    }
		} else {
      		    if ( ($log_one >= $one || $log_two >= $two) && ($log_one >= $two || $log_two >= $one) ) {
			$done = 1;
		    }
		}
	    }else{
		# If none has digit, check error message
		#################################################
		#if ( (($log_one =~ /$one/) && (! $log_two )) ||
		#     ((! $log_one) && ($log_two =~ /$two/)) ) {
		################################################
		if ( ( $log_one !~ /$one/ || $log_two) && 
		     ( $log_one || $log_two !~ /$two/) ) { 
		    logcomment("Change log work as expected");
			$done = 2;
		}else{
			$done = 3;
		}
	    }
    } # while

    if( !$done )
    {
    	die("Number in changelog is not what we expected : Changelog0:$log_one, Changelog1:$log_two\n");
    } elsif ( $done == 3 )
    {
    	die("Message in changelog is not what we expected: Changelog0:$log_one, Changelog1:$log_two\n");
    }
    
    return 1;
}


=head2 dense_verify_changelog_opt(host=>$host, vol_name=>$vol_name, verify_against=>$verify_against, partner=>$partner)

Verify if the config of the value volume is what it should be

=over 4

=item host

Filer HostSpec

=item vol_name

Volume name as volName

=item verify_against

Value to verify against

=item Returns:

If sis_logging returned from vol options and verify against are the same

=over 4

Return 1

=back

If sis_logging returned from vol options and verify against are the not same

=over 4

Return error in C<$@>

=back

=back

=cut

sub dense_verify_changelog_opt($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                vol_name => { type=>SCALAR },
                                verify_against => { type=>SCALAR },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );

    my ($host)=$opts{host};
    my ($vol_name)=$opts{vol_name};
    my ($verify_against)=$opts{verify_against};

    my ($conn) = make_conn($host,"#rsh");

    my $opt_as =  get_changelog_opt(host=>$host, volname=>$vol_name, partner=>$opts{partner});

    if( $opt_as eq $verify_against){
       return 1;
    }else{
       die("Sis logging $opt_as on $vol_name does not match $verify_against\n");
    }
}

=head2 dense_verify_config(host=>$host, path_name=>$path_name, verify_against=>$verify_against, partner=>$partner)

Verify if the config of the value volume is what it should be 

=over 4

=item host

Filer HostSpec

=item path_name

Volume path name as /vol/volName

=item verify_against

Value to verify against

=item Returns:

If config value returned from dense_get_config and verify against are the same  

=over 4

Return 1

=back

If config value returned from dense_get_config and verify against are not the same  

=over 4

Return error in C<$@>

=back

=back

=cut

sub dense_verify_config($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
                                verify_against => { type=>SCALAR },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );
    
    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    my ($verify_against)=$opts{verify_against};

    my ($conn) = make_conn($host,"#rsh");

    my $config_as =  dense_get_config(host=>$host, path_name=>$path_name, partner=>$opts{partner});
    $config_as =~ s/^\s+//;
    $config_as =~ s/\s+$//;

    if( $config_as eq $verify_against){ 
       return 1;
    }else{
       die("Config entry $config_as on $path_name does not match $verify_against\n");
    }
}

=head2 dense_verify_flag(host=>$host, vol_name=>$vol_name, state=>$state, partner=>$partner)

Verify if the volume has dense flag by checking vol status 

=over 4

=item host

Filer HostSpec

=item $vol_name

Valid volume name in volname

=item state

One of the following, ON or OFF 

=item Returns:

If verify ON and volume's status doesn't have dense flag 

=over 4

Return error in C<$@>

=back

If verify OFF and volume's status has dense flag 

=over 4

Return error in C<$@>

=back

=back

=cut

sub dense_verify_flag($@) 
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                vol_name => { type=>SCALAR },
                                state => { type=>SCALAR, regex=>qr/^(ON|OFF)$/ },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );
    
    my ($host)=$opts{host};
    my ($vol_name)=$opts{vol_name};
    my ($state)=$opts{state};

    my ($conn) = make_conn($host,"#rsh");

    if( ($state eq "ON") && ( !is_dense_flag(host=>$host, volname=>$vol_name) ) ){
       die("Path $vol_name has no dense flag\n"); 
    }

    if( ($state eq "OFF") && is_dense_flag(host=>$host, volname=>$vol_name) ){
       die("Path $vol_name has dense flag\n"); 
    }
}

=head2 dense_verify_stage(host=>$host, path_name=>$path_name, verify_against=>$verify_against, partner=>$partner)

Verify if the path become to the wanted status

=over 4

=item host

Filer HostSpec

=item $path_name

Valid path in /vol/volname

=item verify_against

One of the following stage, GATHERING, SORTING, SAVING, DONE, DASH, CHECKING_P1, 
CHECKING_P2  

=item Returns:

Success

=over 4

Return 1

=back

Failure

=over 4

Return 0.

=back

=back

=cut

sub dense_verify_stage($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
                                verify_against => { type=>SCALAR },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );

    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    my ($verify_against)=$opts{verify_against};

    my ($conn) = make_conn($host,"#rsh");

    my ($return_string, $op_error) = dense_get_stage_internal(host=>$host, path_name=>$path_name, partner=>$opts{partner});
	
    logcomment("Current state=[$return_string] matches required state [$verify_against]");

    # check if we want to fatal it here
    if ($return_string eq $verify_against)
    {
	logcomment("Current state=[$return_string] matches required state [$verify_against]");
        return 1;
    } else {
        # do not match
        die("$verify_against does not match curent stage $return_string\n");
    }
}


=head2 dense_verify_state(host=>$host, path_name=>$path_name, state=>$state, partner=>$partner)

Verify if the path become to the wanted state

=over 4

=item host

Filer HostSpec

=item $path_name

Valid path

=item state

One of the following, ENABLE, DISABLE, NOSTATUS

=item Returns:

Success

=over 4

Return 1

=back

Failure

=over 4

Return error in C<$@>

=back

=back

=cut

sub dense_verify_state($@) 
{

# state => { type=>SCALAR, regex=>qr/^(ENABLE|DISABLE|NOSTATUS)$/ } }

   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
                                state => { type=>SCALAR|ARRAYREF },
				expected_error => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } , 
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );

    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    my ($state)=$opts{state};
    my $type = ref($opts{state});

    my ($conn) = make_conn($host,"#rsh");

    my ($expected_error);  

    if( $opts{expected_error} ) {
	if($type eq "ARRAY") {
            die "[dense_verify_state]: State cannot be an ARRAYREF when expected_error is set to 1\n";
        } else {
	    $expected_error = $state;
       }	
    }


    my $return_string = dense_get_state_internal(host=>$host, path_name=>$path_name, partner=>$opts{partner}, expected_error=>$expected_error);


    ## If we expect an error returned from dense_get_state internal, check if the return string is same as the expected error
    if( $opts{expected_error} ) { 
        return 1 if ($return_string =~ /$expected_error/); 
    }

    my @verify_states = ();

    if( $type eq "ARRAY" )
    {
	@verify_states = @{$opts{state}};
    }else{
	push @verify_states, $opts{state};
    }

    # verify the current state against the verify one 
    foreach my $st (@verify_states)
    {
	return 1 if ($return_string eq $st);
    }
    
    die("$state does not match curent state $return_string\n");
}

=head2 dense_verify_status(host=>$host, path_name=>$path_name, verify_against=>$verify_against, partner=>$partner)

Verify if the path become to the wanted status 

=over 4

=item host

Filer HostSpec

=item $path_name

Valid path in /vol/volname

=item verify_against

One of the following status, INITIALIZING, IDLE, ACTIVE, UNDOING, NOSTATUS

=item Returns:

Success 

=over 4

Return 1 

=back

Failure 

=over 4

Return error in C<$@>

=back

=back

=cut

sub dense_verify_status($@) {
    # state => { type=>SCALAR, regex=>qr/^(IDLE|ACTIVE|UNDOING|NOSTATUS|INITIALIZING)$/ } }
    my (%opts) = validate (@_,  { 
                                host            => { type=>SCALAR },
                                path_name       => { type=>SCALAR },
                                verify_against  => { type=>SCALAR|ARRAYREF },
				                partner         => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } 
                                } 
                          );

    my ($host)           = $opts{host};
    my ($path_name)      = $opts{path_name};
    my ($verify_against) = $opts{verify_against};
    my ($conn)           = make_conn( $host, "#rsh" );

    my ($return_status, $op_error) = dense_get_status_internal(host=>$host, path_name=>$path_name, partner=>$opts{partner});

    my $found         = 0;
    my $state_to_find = ();
    if (ref($opts{verify_against}) eq "ARRAY") {
        $state_to_find = $opts{verify_against};
    } else {
        push (@$state_to_find, $opts{verify_against});
    }

    logcomment("DEBUG: dense_verify_status: looking for [@$state_to_find], obtained=[$return_status]");

    foreach my $st (@$state_to_find) {
        if ( $return_status =~ /$st/i ) {
            logcomment("DEBUG: dense_verify_status: Found required state, reqd=$st, obtained=$return_status");
            $found = 1;
            last;
        }

    }

    # check if we want to fatal it here
    if ($found) {
        return 1;
    } else {
        # do not match, add error at the end
        logcomment("dense_verify_status [$verify_against] does not match [$return_status:$op_error]");
        die "$verify_against does not match current status $return_status: $op_error\n";
    }
}

sub dense_verify_progress($@)
{
# state => { type=>SCALAR, regex=>qr/^(IDLE|ACTIVE|UNDOING|NOSTATUS|INITIALIZING)$/ } }
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
                                verify_against => { type=>SCALAR|ARRAYREF },
                                partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } }
                          );

    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    my ($verify_against)=$opts{verify_against};

    my ($conn) = make_conn($host,"#rsh");

    my %status = dense_status(host=>$host, long=>1, path=>$path_name, partner=>$opts{partner}) or die "Sis status failed : $@";
    my $return_status = $status{$path_name}{Progress}; 
    #my ($return_status, $op_error) = dense_get_progress_internal(host=>$host, path_name=>$path_name, partner=>$opts{partner});

    my $found = 0;
    my @state_to_find = ();
    if( ref($opts{verify_against}) eq "ARRAY" )
    {
        @state_to_find = $opts{verify_against};
    }
    else
    {
       push @state_to_find, $opts{verify_against};
    }

    logcomment("dense_verify_progress: looking for [$verify_against], obtained=[$return_status]");

    foreach my $st (@state_to_find)
    {
         if( $return_status =~ /$st/i )
         {
                logcomment("dense_verify_status: Found required state, reqd=$st, obtained=$return_status");
                $found = 1;
                last;
         }
    }

    # check if we want to fatal it here
    if ($found){
        return 1;
    } else {
        # do not match, add error at the end
        logcomment("dense_verify_progress [$verify_against] does not match [$return_status]");
        die "$verify_against does not match current progress $return_status\n";
    }
}


=head2 dense_verify_status_error(host=>$host, path_name=>$path_name, verify_error=>$verify_error, partner=>$partner)

Verify if the path status has the correct error string 

=over 4

=item host

Filer HostSpec

=item $path_name

Valid path in /vol/volname

=item verify_error

Error string to verify against Last Operation Error

=item Returns:

Success

=over 4

Return 1

=back

Failure

=over 4

Return error in C<$@>

=back

=back

=cut

sub dense_verify_status_error($@)
{
	my (%opts) = validate (@_, { 
			host => {type=>SCALAR},
			path_name => {type=>SCALAR},
			verify_error => {type=>SCALAR},
			partner => {type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/}
			} 
			);

	my ($host) 		= $opts{host};
	my ($version) 		= getwv(host=>$host);
	my ($path_name) 	= $opts{path_name};
	my ($conn) 		= make_conn($host,"#rsh");
	
	my ($verify_error) 	= GetMessage( code => "$opts{verify_error}", host => $host);
	my ($return_status, $op_error) = dense_get_status_internal(
						host=>$host, 
						path_name=>$path_name, 
						partner=>$opts{partner}
						);
        
       logcomment("Expected Error [$verify_error], obtained error [$return_status], op_error [$op_error]");
	# check if we want to fatal it here
	if ($op_error eq $verify_error){
		return 1;
	} else {
               if ( $opts{verify_error} eq "DENSE_STOP" ) { ## Since in BR "sis stop" can result in "read metafile error"
                       my ($verify_error_new) = GetMessage( code => "DENSE_METAFILE_READ", host => $host);
                       if ($op_error eq $verify_error_new) {
                               return 1;
                       } else {
                               die("$verify_error and $verify_error_new does not match current error $op_error:in status $return_status\n");
                       }
               } else {
                       die("$verify_error does not match current error $op_error:in status $return_status\n");
               }
       }

}

=head2 dense_verify_stat(host=>$host, path=>$path, num_shared=>$num_shared, num_saved=>$num_saved, num_dfiles=>$num_dfiles, partner=>$partner)

Verify Shared, Saving, and # Dense Files of a dense volume
against specified values. The percent saved will also verified
based on the numbers got from the dense stat command.

=over 4

=item host 

Filer HostSpec, required.

=item path 

Full path of a volume, /vol/<volname>, required.

=item num_shared 

Expected number of shared blocks, required.

=item num_saved 

Expected number of saved blocks by sharing, required.

=item num_dfiles 

Expected number of dense files in the volume, required.

=item Returns:

Shared

Saved

Dense Files

=back

=cut

sub dense_verify_stat($@) {
    my (%opts) = validate( @_, {
            host       => { type => SCALAR },
            path       => { type => SCALAR },
            num_shared => { type => SCALAR },
            num_saved  => { type => SCALAR },
            num_dfiles => { type => SCALAR },
            partner    => { type => SCALAR, optional => 1, default => 0, regex => qr/^(0|1)$/ }
        }
    );

    my ($host)       = $opts{host};
    my ($path)       = $opts{path};
    my ($num_shared) = $opts{num_shared};
    my ($num_saved)  = $opts{num_saved};
    my ($num_dfiles) = $opts{num_dfiles};
    my $long         = 1;
    my $block        = 1;
    my $diag_mode    = 1;

    my ($conn) = make_conn( $host, "#rsh" );
    my $volname;
    if ( $path =~ /\/vol\/(\w+)/ ) {
        $volname = $1;
    }
    logcomment("Volume name is $volname");

    # sync the volume before checking the stat
    set_filer_sync( host => $host );

    # use dense stat -l -b
    my %stat_info = dense_stat(host=>$host, block=>$block, long=>$long, diag_mode=>$diag_mode, path=>$path, partner=>$opts{partner});
    
    my $shared = $stat_info{$path}{"Shared"};
    ($shared, undef) = split /\s+/, $shared;
    ($shared == $num_shared) 
	    or die("Incorrect shared blocks: $shared vs $num_shared\n");
    
    my $saved = $stat_info{$path}{"Saving"};
    ($saved, undef) = split /\s+/, $saved;
    ($saved == $num_saved)
	    or die("Incorrect saved by sharing count: $saved vs $num_saved\n");

    logcomment("Version is $DENSE_ONTAP_VERSION and DEFAULT_DLOG_SIS_FILES= $DEFAULT_DLOG_SIS_FILES");
    ## Verify if the volume option no_delete_log is present for that volume, if present, add dlog_mfiles to the sis files count
    my $dlog_mfiles = $DEFAULT_DLOG_SIS_FILES; ## Number of delete log overflow files/volume
    # my $dlog_option = "no_delete_log"; 

    # my %vol_opts = Volume::volume_options_list_info(host=>host($host), volume=>$volname);
    # my $dlog = 1 if (exists $vol_opts{$dlog_option});
    # if($dlog) {
        $num_dfiles = $num_dfiles + $dlog_mfiles;
    # }

        my @metafiles = (
            'fingerprint.vvol.delta', 'fingerprint.vvol.delta.old',
            'clogdir.0/bucket0',      'clogdir.1/bucket0'
        );
        my $inode_dense_flag;
        my $file_path;
        my %vol = ( 'name' => $volname );
        logcomment('Setting wafl_metadata_visible flag');
        Dense::Utils::set_metafile_visible(host => $host);
        foreach my $metafile (@metafiles) {
            $inode_dense_flag = 0;
            $file_path        = "/vol/$volname/metadir/dense/$metafile";
            eval {
                $inode_dense_flag = Dense::Utils::check_inode_dense_flag(
                    conn          => $conn,
                    vol           => \%vol,
                    file_name     => $file_path,
                    private_inode => 1,
                );
            };
            if ($@) {
                logcomment("Inode flag for $metafile is not available: $@");
            } else {
                logcomment("Inode flag for: $metafile is = $inode_dense_flag");
            }
            $num_dfiles += $inode_dense_flag;
        }
    my $dfiles = $stat_info{$path}{"SIS Files"};
    ($dfiles <= $num_dfiles)
    	    or die("Incorrect number of SIS files: $dfiles vs $num_dfiles\n");

    my $pct_saved = $stat_info{$path}{"%Saved"};
    my $total = $stat_info{$path}{"Allocated"} + $saved;
    my $pct = int ($saved * 100 / $total);
    ($pct_saved eq $pct . " %")
	or die("Incorrect total % saved: $pct_saved vs $pct\n");
}

=head2 dense_wait_for_field(host=>$host, path_name=>$path_name, wait_on=>$wait_on, wait_for=>$wait_for, time_gap=>$time_gap, time_out=>$time_out, expected_error=>$expected_error, partner=>$partner)

Verify if the path become to the wanted status or stage on the wait_for string 

=over 4

=item host

Filer HostSpec

=item path_name

Valid path

=item wait_on

What field you want to wait for,

=item wait_for

One of the following for STATUS, INITIALIZING, IDLE, ACTIVE, UNDOING, NOSTATUS, DONE, DASH
One of the following for STAGE, GATHING, SORTING, SAVING, CHECKING_P1, CHECKING_P2,
DONE, DASH

=item time_gap

Optional, time in seconds. Wait how long to check status again, default is
2 seconds.

=item time_out

Optional, time in seconds. In order not to loop forever. Default is 3600
which is 2 hours for now. No code for this yet... TBD.

=item expected_error

Optional, error string to check on "Current Operation Error". Default is "-".

=item Returns:

Success

=over 4

Return 1

=back

Failure

=over 4

Return Error message in C<@>

=back

=back

=cut

sub dense_wait_for_field($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
                                wait_on => { type=>SCALAR },
                                wait_for => { type=>SCALAR },
                                time_gap => { type=>SCALAR, optional=>1, default=>"2" },
                                time_out => { type=>SCALAR, optional=>1, default=>"300" }, 
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );

    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    my ($wait_on)=$opts{wait_on};
    my ($wait_for)=$opts{wait_for};
    my ($time_gap)=$opts{time_gap};
    my ($time_out)=$opts{time_out};
    my (%status_info, $field, $value);
    my ($conn) = make_conn($host,"#rsh");

    # some default value
    my $wait_time = 2;

    # set parameter if passed in
    if ( $opts{time_gap} && ($opts{time_gap} > 0) ){
        $wait_time = $opts{time_gap};
    }

    if ( $opts{time_out} && ($opts{time_out} > 0) ){
        $time_out = $opts{time_out};
    }

    while ( ($time_out > 1) && ($value ne $wait_for) ) {
	logcomment("Current time-out is $time_out");
        %status_info = dense_status(host=>$host, long=>1, path=>$path_name, partner=>$opts{partner});
        (defined $status_info{$path_name}{$wait_on}) || die "$wait_on field not present in long dense_status output";
        $field =$wait_on;
        $value = $status_info{$path_name}{$field};

        # wait sometime
        sleep($wait_time);
	$time_out -= ( $wait_time + 1 );
    } # end while

    if ( $time_out > 1 ){
        return 1;
    } else {
        die("Either time out or wait fail\n");
    }
}

=head2 dense_wait_for(host=>$host, path_name=>$path_name, wait_on=>$wait_on, wait_for=>$wait_for, time_gap=>$time_gap, time_out=>$time_out, expected_error=>$expected_error, partner=>$partner)

Verify if the path become to the wanted status or stage on the wait_for string 

=over 4

=item host

Filer HostSpec

=item path_name

Valid path

=item wait_on

What area you want to wait for, for now it is either STAGE or STATUS

=item wait_for

One of the following for STATUS, INITIALIZING, IDLE, ACTIVE, UNDOING, NOSTATUS, DONE, DASH
One of the following for STAGE, GATHING, SORTING, SAVING, CHECKING_P1, CHECKING_P2,
DONE, DASH

=item time_gap

Optional, time in seconds. Wait how long to check status again, default is
2 seconds.

=item time_out

Optional, time in seconds. In order not to loop forever. Default is 3600
which is 2 hours for now. No code for this yet... TBD.

=item expected_error

Optional, error string to check on "Current Operation Error". Default is "-".

=item Returns:

Success

=over 4

Return 1

=back

Failure

=over 4

Return Error message in C<@>

=back

=back

=cut

sub dense_wait_for($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
				status => { type=>SCALAR, optional=>1, default=>"IDLE" },
				stage => { type=>SCALAR, optional=>1 },
				progress => { type=>SCALAR, optional=>1 },
                                wait_on => { type=>SCALAR, optional=>1, regex=>qr/^(STAGE|STATUS|PROGRESS)$/ },
                                wait_for => { type=>SCALAR, optional=>1 },
                                time_gap => { type=>SCALAR, optional=>1, default=>"2" },
                                time_out => { type=>SCALAR, optional=>1, default=>"300" }, 
                                expected_error => { type=>SCALAR, optional=>1, default=>"-" },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );

    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    my ($wait_on)= !defined($opts{wait_on}) ? ($opts{stage}? "STAGE" : "STATUS") : $opts{wait_on};
    my ($wait_for)= !defined($opts{wait_for}) ? ($opts{stage} ? $opts{stage} : $opts{status}) : $opts{wait_for};
    $wait_on = defined($opts{progress}) ? "PROGRESS" : $wait_on;
    $wait_for = defined($opts{progress}) ? $opts{progress} : $wait_for;
    my ($time_gap)=$opts{time_gap};
    my ($time_out)=$opts{time_out};
    my ($expected_error)=$opts{expected_error};

    my ($conn) = make_conn($host,"#rsh");

    # some default value
    my $wait_time = 2;
    my $check = 0;

    # set parameter if passed in
    if ( $opts{time_gap} && ($opts{time_gap} > 0) ){
        $wait_time = $opts{time_gap};
    }

    if ( $opts{time_out} && ($opts{time_out} > 0) ){
        $time_out = $opts{time_out};
    }
    
    logcomment("Current time-out is $time_out");
    logcomment("Values are : wait_on = $wait_on, wait_for = $wait_for");


    if ( $opts{expected_error}){
        $expected_error = $opts{expected_error};
    }

    # get current time
    # ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

    # wait for the stage or time out
    # or if there is unexpected operation error, the will return
    # the error and break the loop, such as no space left error
    while ( $check == 0 ) {
        # wait sometime
        sleep($wait_time);
        eval{
            if($wait_on eq "STAGE"){
                dense_verify_stage(host=>$host, path_name=>$path_name, verify_against=>$wait_for, partner=>$opts{partner});
            }elsif($wait_on eq "STATUS"){
		logcomment("DEBUG: dense_wait_for calling dense_verify_status");
                #dense_verify_status($conn, $path_name, $wait_for, $expected_error);
                dense_verify_status(host=>$host, path_name=>$path_name, verify_against=>$wait_for, partner=>$opts{partner});
            }elsif($wait_on eq "PROGRESS"){
		logcomment("dense_wait_for calling dense_verify_progress");
                dense_verify_progress(host=>$host, path_name=>$path_name, verify_against=>$wait_for, partner=>$opts{partner});
	    }
        };

	logcomment("Got Error: $@");

        if (! $@){
            $check = 1;
        }else{
            if ($@ =~ /^Unexpected operation error/){
		logcomment("[$@] matched [Unexpected operation error]");
                die("$@\n");
            }
        }

	$time_out -= ( $wait_time + 1 );
	if ( $time_out < 1 )
	{
		logcomment("Time-out Expired, exitting the loop now, value=$time_out");
		$check = 2; # to be on the safer side
		last;
	}
	else
	{
		logcomment("Current time-out is $time_out");
	}
    } # end while

    if ( $check == 1 ){
        return 1;
    } else {
        # really odd here
        die("Either time out or wait fail\n");
    }
}

=head2 dense_wait_for_stage(host=>$host, path_name=>$path_name, stage=>$stage, time_gap=>$time_gap, time_out=>$time_out, expected_error=>$expected_error, partner=>$partner)

Verify if the path become to the wanted status

=over 4

=item host

Filer HostSpec

=item $path_name

Valid path

=item stage

Stage on one of the following, GATHING, SORTING, SAVING, DONE, CHECKING_P1, 
CHECKING_P2, DASH

=item time_gap

Optional, time in seconds. Wait how long to check status again, default is
2 seconds.

=item time_out

Optional, time in seconds. In order not to loop forever. Default is 3600
which is 2 hours for now. No code for this yet... TBD.

=item expected_error

Optional, error string to check on "Current Operation Error". Default is "-".

=item Returns:

Success

=over 4

Return 1

=back

Failure

=over 4

Return Error message in C<@>

=back

=back

=cut

sub dense_wait_for_stage($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
                                stage => { type=>SCALAR },
                                time_gap => { type=>SCALAR, optional=>1, default=>"2" },
                                time_out => { type=>SCALAR, optional=>1, default=>"3600" },
                                expected_error => { type=>SCALAR, optional=>1, default=>"-" },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );

    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    my ($stage)=$opts{stage};
    my ($time_gap)=$opts{time_gap};
    my ($time_out)=$opts{time_out};
    my ($expected_error)=$opts{expected_error};

    dense_wait_for(host=>$host, path_name=>$path_name, wait_on=>"STAGE", wait_for=>$stage, time_gap=>$time_gap, time_out=>$time_out, expected_error=>$expected_error, partner=>$opts{partner});
}


=head2 dense_wait_for_status(host=>$host, path_name=>$path_name, status=>$status, time_gap=>$time_gap, time_out=>$time_out, expected_error=>$expected_error, partner=>$partner)

Verify if the path become to the wanted status

=over 4

=item host

Filer HostSpec

=item $path_name

Valid path

=item status

On of the following, INITIALIZING, IDLE, ACTIVE, UNDOING, NOSTATUS

=item %options

=over 4

=item time_gap

Optional, time in seconds. Wait how long to check status again, default is
2 seconds.

=item time_out

Optional, time in seconds. In order not to loop forever. Default is 3600
which is 2 hours for now. No code for this yet... TBD.

=item expected_error

Optional, error string to check on "Current Operation Error". Default is "-".

=item Returns:

Success

=over 4

Return 1

=back

Failure

=over 4

Return Error message in C<@>

=back

=back

=cut

sub dense_wait_for_status($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
                                status => { type=>SCALAR },
                                time_gap => { type=>SCALAR, optional=>1, default=>"2" },
                                time_out => { type=>SCALAR, optional=>1, default=>"300" },
                                expected_error => { type=>SCALAR, optional=>1, default=>"-" },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );

    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    my ($status)=$opts{status};
    my ($time_gap)=$opts{time_gap};
    my ($time_out)=$opts{time_out};
    my ($expected_error)=$opts{expected_error};

    dense_wait_for(host=>$host,path_name=>$path_name,wait_on=>"STATUS",wait_for=>$status,time_gap=>$time_gap,time_out=>$time_out,expected_error=>$expected_error, partner=>$opts{partner});
}

=head2 dense_wait_for_status(host=>$host, path_name=>$path_name, status=>$status, time_gap=>$time_gap, time_out=>$time_out, expected_error=>$expected_error, partner=>$partner)

Verify if the path become to the wanted status

=over 4

=item host

Filer HostSpec

=item $path_name

Valid path

=item status

On of the following, INITIALIZING, IDLE, ACTIVE, UNDOING, NOSTATUS

=item %OPTIONS

=item time_gap 

Optional, time in seconds. Wait how long to check status again, default is 
2 seconds.

=item time_out

Optional, time in seconds. In order not to loop forever. Default is 3600 
which is 2 hours for now. No code for this yet... TBD.

=item Returns:

Success

=over 4

Return 1

=back

Failure

=over 4

Return Error message in C<@> 

=back

=back

=cut

sub dense_wait_for_status($@) 
{
   my (%opts) = validate (@_, { 
                                host            => { type=>SCALAR },
                                path_name       => { type=>SCALAR },
                                status          => { type=>SCALAR|ARRAYREF },
                                time_gap        => { type=>SCALAR, optional=>1, default=>2 },
                                time_out        => { type=>SCALAR, optional=>1, default=>3600 },
				                expected_error  => { type=>SCALAR, optional=>1, default=>"-" },
				                partner         => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } 
                
                              } 
                          );

    my ($host, $path_name, $wait_for, $time_gap, $time_out, $conn, $check, $wait_time);

    $host      = $opts{host};
    $path_name = $opts{path_name};
    $wait_for  = $opts{status};
    $time_gap  = $opts{time_gap};
    $time_out  = $opts{time_out};
    $conn      = make_conn( $host, "#rsh" );
    $check     = 0;
    $wait_time = $time_gap;

    logcomment("Starting with time-out=$time_out");

    # wait for the status or time out
    # or if there is unexpected operation error, the will return
    # the error and break the loop, such as no space left error 
    while ( $check == 0 ) {
        # wait sometime
        sleep($wait_time);

        eval {
            logcomment("dense_wait_for_status calling dense_verify_status");
            dense_verify_status(host => $host, path_name => $path_name, verify_against => $wait_for, partner => $opts{partner});
        };
        ## logcomment("Got Error: $@");

        my @Err = split( /:/, $@ );

        ## logcomment("Operation Error is $Err[1]");

        if ( !$@ ) {
            $check = 1;
        } else {
            if ( $Err[1] !~ /^(\s*)-(\s*)/ ) {
                logcomment("Dying because of error: $@");
                die("$@\n");
            }
        }

        $time_out -= ( $wait_time + 1 );
        if ( $time_out <= 0 ) {
            logcomment("Current time-out=$time_out, Exceed time-out value");
            $check = 2;    # to ensure it exits the loop
            last;
        } else {
            logcomment("Current time-out=$time_out");
        }

    }    # end while

    if ( $check == 1 ) {
        return 1;
    } else {
        # really odd here
        die("Either time out or wait fail\n");
    }
}

=head2 get_dense_vols(host=>$host)

Get volumes that has dense type under vol status

=over 4

=item host

Filer HostSpec

=item Returns:

Success

=over 4

Return the array of volume names, it can be null

=back

Failure

=over 4

Return error in C<$@>

=back

=back

=cut

sub get_dense_vols(%)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR } } );

    my ($host) = $opts{host};
    my ($conn) = make_conn($host,"#rsh");
    my @dense_vols = ();

    my @vols = vol_list($conn);

    foreach my $single (@vols){
        if ( is_dense_flag(host=>$host, volname=>$single) ){
            push (@dense_vols,$single);
        }
    }

    return @dense_vols;
}

=head2 get_changelog_info(host=>$host, path_name=>$path_name, partner=>$partner)

Get the changelog option value of vol options
Since this is a readonly option, no sis_logging
is abled to add to vol options

Sample returns:
Checking changelog.0 ...  828072 records
Checking changelog.1 ...  0 records

Checking changelog.0 ...
Checking changelog.1 ... Metafile does not exist: /vol/dense_vol_purge

=over 4

=item host

Filer HostSpec

=item $path_name

Path of the volume as /vol/volName

=item Returns:

Success

=over 4

Return a hash of changelog, it can be null or a string
$log{changelog.0} and its value in blocks 
$log{changelog.1} and its value in blocks 

log{changelog.0} would have 828072
log{changelog.1} would have 0 
or 
log{changelog.0} would have null
log{changelog.1} would have "Metafile does not exist" 

=back

Failure

=over 4

Return error in C<$@>

=back

=back

=cut

sub get_changelog_info($@)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR },
                                 path_name => { type=>SCALAR },
				 partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );

    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    my ($version) = getwv(host=>$host);

    my ($conn)=make_conn($host,"#rsh");

    my %log = (); 

    my $return_text = dense_check(host=>$host, path_name=>$path_name, cmd_opt=>'-c', partner=>$opts{partner}); 
    ## logcomment("DEBUG: return text is $return_text");
 
    # parse the return text
    my @process = filer_cli_split($return_text);
    ## logcomment("DEBUG: process is @process");
    my $hostspec = host($host);
    my ($clog_0_start,$clog_1_start) = (0,0);
    my ($clog_0_done,$clog_1_done) = (0,0);
    if ( $version eq "GB" ) {
           if (scalar(@process) > 0){
		   ## logcomment("DEBUG: line is $line");
                   # Checking changelog.0 ...  0 records\n
                foreach my $line (@process){
                    if ($line =~ /^Checking\s(changelog.0)/) {
                        $clog_0_start = 1;
                        if ($line =~ /(\d+)\s+records$/) {
                            print("***************changelog.0 is:$1*********\n");
                            $log{"changelog.0"} = $1;
                            $clog_0_done = 1;
                        }
                        next;
                    }
                    if ($clog_0_start && !$clog_0_done) {
                        if ($line =~ /(\d+)\s+records$/) {
                            print("***************changelog.0 is:$1*********\n");
                            $log{"changelog.0"} = $1;
                            $clog_0_done = 1;
                        }
                        next;
                    }
                    if ($line =~ /^Checking\s(changelog.1)/) {
                        if (!$clog_0_done) {
                            $log{"changelog.0"} = GetMessage(code => "DENSE_METAFILE_NONEXIST", host => $host);
                            print("***************changelog.0 is:$log{$1}*********\n");
                        }
                        $clog_1_start = 1;
                        if ($line =~ /(\d+)\s+records$/) {
                            print("***************changelog.1 is:$1*********\n");
                            $log{"changelog.1"} = $1;
                            $clog_1_done = 1;
                        }
                        next;
                    }
                    if ($clog_1_start && !$clog_1_done) {
                        if ($line =~ /(\d+)\s+records$/) {
                            print("***************changelog.1 is:$1*********\n");
                            $log{"changelog.1"} = $1;
                            $clog_1_done = 1;
                            return %log;
                        }
                        next;
                    }
                    if (($line =~ /Number of changelog buckets/) && !$clog_1_done) {
                        $log{"changelog.1"} = GetMessage(code => "DENSE_METAFILE_NONEXIST", host => $host);
                        print("***************changelog.1 is:$log{$1}*********\n");
                    }
                } # End for each
           }
    } else {
            ## Changelog names have been changed in IC
            if (scalar(@process) > 0){
		    ## logcomment("DEBUG: line is $line");
                    # Checking clogdir.0 ...  0 records\n
		foreach my $line (@process){
		    if ($line =~ /^Checking\s(clogdir.0)/) {
			$clog_0_start = 1;
			if ($line =~ /(\d+)\s+records$/) {
			    print("***************clogdir.0 is:$1*********\n");
			    $log{"clogdir.0"} = $1;
			    $clog_0_done = 1;
			}
			next;
		    }
		    if ($clog_0_start && !$clog_0_done) {
			if ($line =~ /(\d+)\s+records$/) {
			    print("***************clogdir.0 is:$1*********\n");
			    $log{"clogdir.0"} = $1;
			    $clog_0_done = 1;
			}
			next;
		    }
		    if ($line =~ /^Checking\s(clogdir.1)/) {
			if (!$clog_0_done) {
			    $log{"clogdir.0"} = GetMessage(code => "DENSE_METAFILE_NONEXIST", host => $host);
			    print("***************clogdir.0 is:$log{$1}*********\n");
			}
			$clog_1_start = 1;
			if ($line =~ /(\d+)\s+records$/) {
			    print("***************clogdir.1 is:$1*********\n");
			    $log{"clogdir.1"} = $1;
			    $clog_1_done = 1;
			}
			next;
		    }
		    if ($clog_1_start && !$clog_1_done) {
		        if ($line =~ /(\d+)\s+records$/) {
			    print("***************clogdir.1 is:$1*********\n");
			    $log{"clogdir.1"} = $1;
			    $clog_1_done = 1;
			    return %log;
			}
			next;
		    } 
		    if (($line =~ /Number of changelog buckets/) && !$clog_1_done) {
			$log{"clogdir.1"} = GetMessage(code => "DENSE_METAFILE_NONEXIST", host => $host);
		        print("***************clogdir.1 is:$log{$1}*********\n");
		    }	
		} # End for each
            }
    }

    return %log; 
}

sub parse_dense_check ($@) {
    my (%opts) = validate( @_, {
            host        => { type => SCALAR },
            path_name   => { type => SCALAR },
            downgraded  => { type => SCALAR, optional => 1,default => 0, regex => qr/^(0|1)$/ },
            verbose     => { type => SCALAR, optional => 1,default => 0, regex => qr/^(0|1)$/ },
            partner     => { type => SCALAR, optional => 1,default => 0, regex => qr/^(0|1)$/ }
        }
    );

    my ( $host, $path_name, $downgraded, $version, $conn, $return_text, $hostspec, $cmd_opt);
    my ( @process, %parsed_output, $ver_cli_output, @ver_keys, $ver_key, %ret_output);

    $host      = $opts{'host'};
    $path_name = $opts{'path_name'};
    $downgraded = $opts{'downgraded'};
    $version   = getwv( host => $host );

    $conn = make_conn( $host, "#rsh" );

    $cmd_opt = $opts{'verbose'} ? "-cv" : "-c";
    $return_text = dense_check(
          host      => $host,
          path_name => $path_name,
          cmd_opt   => $cmd_opt,
          partner   => $opts{partner}
    );

    # parse the return text
    @process = filer_cli_split($return_text);
    $hostspec = host($host);

    $ver_cli_output = _split_parse_dense_check(\@process);
    @ver_keys = keys(%$ver_cli_output);
    foreach $ver_key (@ver_keys) {
        if ($ver_key =~/active/i) {
            $parsed_output{$ver_key} = _extract_parse_dense_check($ver_cli_output->{$ver_key}, $version);
            $parsed_output{detail} = _extract_parse_dense_check_detail($ver_cli_output->{$ver_key} ) if ($opts{'verbose'});
        } elsif ($downgraded) {
            $parsed_output{$ver_key} = _extract_parse_dense_check(\@process, $ver_key);
            $parsed_output{$ver_key}{detail} = _extract_parse_dense_check_detail($ver_cli_output->{$ver_key} ) if ($opts{'verbose'});
        } else {
            logcomment("Ignoring key : $ver_key");
        }
    }

    logcomment ("After complete parsing = ".Dumper(\%parsed_output));

    return \%parsed_output;

}

sub _split_parse_dense_check($) {
    my ($cli_output) = @_;

    my (%splited, $line, $stage, @tmp_holder);
    $stage = 'active';
    foreach $line (@$cli_output) {
        if ($line =~ /^Checking\s+Downgraded\s+temporary\s+metafiles\s+\((\S+)\)/) {
            push(@{$splited{$stage}}, @tmp_holder);
            @tmp_holder = ();
            $stage = $1;
            push(@tmp_holder, $line);
        } else {
            push(@tmp_holder, $line);
        }
    }
    push(@{$splited{$stage}}, @tmp_holder);
    return \%splited;
}

sub _extract_parse_dense_check($$) {
    my ($cli_output, $ver) = @_;
    
    my (%ver_map, $version, $cl_0_name, $cl_1_name, @element_list, %element_loc, $gather, $dup_sorted);
	my ($fingerprint,$fp_next,$fp_vvol,$fp_vvol_next,$fp_vvol_delta,$fp_vvol_delta_old,$fp_vvol_temp,$fp_vvol_gen,$fp_stale,$fp_stale_sorted,$fp_stale_sorted_old, $stale_fbns,$stale_fbns_old,$stale_fbns_out,$fingerprint_rr,$fingerprint_vvol_rr,$fingerprint_vvol_delta_rr,$clogdir_0_rr,$clogdir_1_rr, $fingerprint_br, $clogdir_0_br, $clogdir_1_br);
    my ($element, $element_cnt, %element_key, $line, $present_stage, %ret_output);
    my (@dlines,@list,@list1);

    %ver_map = (
            '7.2'   => 'gb',
            '7.3'   => 'ic',
            '8.0'   => 'br',
            '8.1'   => 'rr',
            );
    if ( $ver =~ /\d+/ ) {
        $version = $ver_map{$ver};
    } else {
        $version = lc($ver);
    }

    $fingerprint			= 'fingerprint';
	$fp_next				= 'fingerprint.next';
	$fp_vvol				= 'fingerprint.vvol';
	$fp_vvol_next			= 'fingerprint.vvol.next';
	$fp_vvol_delta			= 'fingerprint.vvol.delta';
	$fp_vvol_delta_old		= 'fingerprint.vvol.delta.old';
	$fp_vvol_temp			= 'fingerprint.vvol.temp';
	$fp_vvol_gen			= 'fingerprint.vvol.gen';
	$fp_stale				= 'fingerprint.stale';
	$fp_stale_sorted		= 'fingerprint.stale.sorted';
	$fp_stale_sorted_old	= 'fingerprint.stale.sorted.old';
	$gather					= 'gather';
	$dup_sorted				= 'duplicate.sorted';
	$cl_0_name				= 'clogdir.0';
        $cl_1_name				= 'clogdir.1';
        $stale_fbns             = 'stale_fbns';
        $stale_fbns_old         = 'stale_fbns.old';
        $stale_fbns_out         = 'stale_fbns.out';
    $fingerprint_rr         = 'fingerprint.rr';
    $fingerprint_vvol_rr    = 'fingerprint.rr.vvol';
    $fingerprint_vvol_delta_rr = 'fingerprint.rr.vvol.delta';
    $clogdir_0_rr            = 'clogdir.0.rr';
    $clogdir_1_rr           = 'clogdir.1.rr';
    $fingerprint_br         = 'fingerprint.br';
    $clogdir_0_br           = 'clogdir.0.br';
    $clogdir_1_br           = 'clogdir.1.br';

	
    if ( $version eq 'gb' ) {
        $cl_0_name = 'changelog.0';
        $cl_1_name = 'changelog.1';
    } 
	
    if ( $ver =~ /\d+/ ) {
        $fingerprint = $fingerprint.'.'.$version.'.rv';
        $fp_vvol = $fp_vvol.'.'.$version.'.rv';
        $fp_vvol_delta = $fp_vvol_delta.'.'.$version.'.rv';
        $cl_0_name = $cl_0_name.'.'.$version.'.rv';
        $cl_1_name = $cl_1_name.'.'.$version.'.rv';
        print "Dowgrade clogs are $cl_0_name , $cl_1_name";
        @dlines = grep(/$version/, @$cli_output);
    } else {
        @dlines = @$cli_output;
    }

    %element_key = (
            $fingerprint        => 'fp',
			$fp_next			=> 'fp_next',
			$fp_vvol			=> 'fp_vvol',
			$fp_vvol_next		=> 'fp_vvol_next',
			$fp_vvol_delta		=> 'fp_vvol_delta',
			$fp_vvol_delta_old	=> 'fp_vvol_delta_old',
			$fp_vvol_temp		=> 'fp_vvol_temp',
			$fp_vvol_gen		=> 'fp_vvol_gen',
			$fp_stale			=> 'fp_stale',
			$fp_stale_sorted	=> 'fp_stale_sorted',
			$fp_stale_sorted_old=> 'fp_stale_sorted_old',
            $cl_0_name          => 'cl_0',
            $cl_1_name          => 'cl_1',
            $gather				=> 'gather',
            $dup_sorted			=> 'dedup_sort',
            $stale_fbns         => 'stale_fbns',
            $stale_fbns_old     => 'stale_fbns.old',
            $stale_fbns_out     => 'stale_fbns.out',
        $fingerprint_rr         => 'fp_rr',
    $fingerprint_vvol_rr    => 'fp_vvol_rr',
    $fingerprint_vvol_delta_rr => 'fp_vvol_delta_rr',
    $clogdir_0_rr            => 'cl_0_rr',
    $clogdir_1_rr            => 'cl_1_rr',
        $fingerprint_br         => 'fp_br',
        $clogdir_0_br           => 'cl_0_br',
            $clogdir_1_br           => 'cl_1_br',
            
            );

    foreach $line (@dlines) {
        if ( $line =~ /^Checking/ ) {
            if ( $line =~ /^Checking\s+(\S+)\s+/ ) {
                my $got_stage = $1;
                $present_stage = "";
                if ($element_key{$got_stage}) {
                    $present_stage = $got_stage;
                    if ( $line =~ /(\d+)\s+records/i ) {
			            $ret_output{ $element_key{$present_stage} } = $1;
                        $present_stage = "";
                    }
                }
            }
        } elsif ($line =~ /(\d+)\s+records/i ) {
            if ($present_stage) {
                $ret_output{ $element_key{$present_stage} } = $1;
            }
        } elsif ($line =~ /Number of Segments:\s+(\d+)/ ) {
            $ret_output{'total_segments'} = $1;
        } elsif ($line =~ /Number of Records:/ ) {
            @list = map {/(\d+)/} split (/\s+/,$line);
        } elsif ($line =~ /^\s+\d+/) {
          @list1 =map {/(\d+)/} split (/\s+/,$line);
          push (@list, @list1);
        }
        if (($#list >= 0) && ($ret_output{'total_segments'} == ($#list + 1))) {
                $ret_output{'segment_list'} = \@list;
            }

    }

    return \%ret_output;
}

sub _extract_parse_dense_check_detail($) {
    my ($cli_output) = @_;

    my ($line,%ret_output,%element_key,$got_stage,$segment);
    %element_key = (
         'fingerprint'                  => 'fp',
                 'fingerprint.next'             => 'fp_next',
                 'fingerprint.vvol'             => 'fp_vvol',
                 'fingerprint.vvol.next'        => 'fp_vvol_next',
                 'fingerprint.vvol.delta'       => 'fp_vvol_delta',
                 'fingerprint.vvol.delta.old'   => 'fp_vvol_delta_old',
                 'fingerprint.vvol.temp'        => 'fp_vvol_temp',
                 'fingerprint.vvol.gen'         => 'fp_vvol_gen',
                 'fingerprint.stale'            => 'fp_stale',
                 'fingerprint.stale.sorted'     => 'fp_stale_sorted',
                 'fingerprint.stale.sorted.old' => 'fp_stale_sorted_old',
                 'gather'                       => 'gather',
                 'duplicate.sorted'             => 'dedup_sort',
                 'clogdir.0'                    => 'cl_0',
         'clogdir.1'                    => 'cl_1',
         'stale_fbns'                   => 'stale_fbns',
                'fingerprint.rr'            => 'fp_rr',
                'fingerprint.vvol.rr.rv'       => 'fp_vvol_rr_rv',
                'fingerprint.vvol.delta.rr.rv' => 'fp_vvol_delta_rr_rv',
                'fingerprint.rr.vvol'       => 'fp_vvol_rr',
                'fingerprint.rr.vvol.delta' => 'fp_vvol_delta_rr',
                'clogdir.0.rr'              => 'cl_0_rr',
                'clogdir.1.rr'              => 'cl_1_rr',
                'clogdir.0.rr.rv'           => 'cl_0_rr_rv',
                'clogdir.1.rr.rv'           => 'cl_1_rr_rv',
                'fingerprint.br'            => 'fp_br',
                'clogdir.1.br'              => 'cl_1_br',
                'clogdir.0.br'              => 'cl_0_br',

         );

    foreach $line (@$cli_output) {
        if ( $line =~ /Number of|contents of/ ) {
            next;
        } elsif ( $line =~ /^Checking\s+(\S+)\s+/ ) {
            $got_stage = $1;
            next;
        } elsif ($line =~ /Contents of Segment:\s+(\d+)/ ) {
            $segment = $1;
            next;
        }
        if ( $got_stage eq "stale_fbns") {
            if ( $line =~ /\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/ ) {
                push @{$ret_output{$element_key{'stale_fbns'}}->{inode}},$1;
                push @{$ret_output{$element_key{'stale_fbns'}}->{wi_gen}},$2;
                push @{$ret_output{$element_key{'stale_fbns'}}->{fbn_src}},$3;
                push @{$ret_output{$element_key{'stale_fbns'}}->{fbn_dst}},$4;
                push @{$ret_output{$element_key{'stale_fbns'}}->{cp_cnt}},$5;
            }
        }
        if ( $line =~ /^\s*(\w+)\s+(\d+)\s+(\d+)$/ ) {
            print "Line obtained is $line \n";
            if ($got_stage eq "fingerprint") {
                push @{$ret_output{$element_key{$got_stage}}->{$segment}->{fp}},$1;
                push @{$ret_output{$element_key{$got_stage}}->{$segment}->{vvbn}},$2;
                push @{$ret_output{$element_key{$got_stage}}->{$segment}->{cp_cnt}},$3;
            } elsif ($got_stage eq "fingerprint.vvol" || $got_stage eq "fingerprint.stale.sorted") {
                push @{$ret_output{$element_key{$got_stage}}->{fp}},$1;
                push @{$ret_output{$element_key{$got_stage}}->{vvbn}},$2;
                push @{$ret_output{$element_key{$got_stage}}->{cp_cnt}},$3;
            } else {
                push @{$ret_output{$element_key{$got_stage}}->{fp}},$1;
                push @{$ret_output{$element_key{$got_stage}}->{vvbn}},$2;
                push @{$ret_output{$element_key{$got_stage}}->{cp_cnt}},$3;
            }
        } elsif ( $line =~ /^\s*(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)$/ ) {
            print "Line obtaines is $line \n";

                if ($got_stage eq "fingerprint.rr" || $got_stage eq "fingerprint") {
                    push @{$ret_output{$element_key{$got_stage}}->{$segment}->{fp}},$1;
                    push @{$ret_output{$element_key{$got_stage}}->{$segment}->{inode}},$2;
                    push @{$ret_output{$element_key{$got_stage}}->{$segment}->{fbn}},$3;
                    push @{$ret_output{$element_key{$got_stage}}->{$segment}->{wi_gen}},$4;
                    push @{$ret_output{$element_key{$got_stage}}->{$segment}->{cp_cnt}},$5;
                    push @{$ret_output{$element_key{$got_stage}}->{$segment}->{compress_attempted}},$6;
                } elsif ($got_stage eq "fingerprint.rr.vvol" || $got_stage eq "fingerprint.rr.vvol.delta" || $got_stage eq "clodir.0.rr" || $got_stage eq "clogdir.1.rr" || $got_stage eq "fingerprint.vvol" || $got_stage eq "clogdir.0" || $got_stage eq "fingerprint.vvol.delta" || $got_stage eq "clogdir.1" ) {
                    push @{$ret_output{$element_key{$got_stage}}->{fp}},$1;
                    push @{$ret_output{$element_key{$got_stage}}->{inode}},$2;
                    push @{$ret_output{$element_key{$got_stage}}->{fbn}},$3;
                    push @{$ret_output{$element_key{$got_stage}}->{wi_gen}},$4;
                    push @{$ret_output{$element_key{$got_stage}}->{cp_cnt}},$5;
                    push @{$ret_output{$element_key{$got_stage}}->{compress_attempted}},$6;
                } elsif ($got_stage eq "fingerprint.vvol.rr.rv" ) {
                    push @{$ret_output{$element_key{$got_stage}}->{fp}},$1;
                    push @{$ret_output{$element_key{$got_stage}}->{inode}},$2;
                    push @{$ret_output{$element_key{$got_stage}}->{fbn}},$3;
                    push @{$ret_output{$element_key{$got_stage}}->{wi_gen}},$4;
                    push @{$ret_output{$element_key{$got_stage}}->{cp_cnt}},$5;
                    push @{$ret_output{$element_key{$got_stage}}->{compress_attempted}},$6;
                } else {
                    push @{$ret_output{$element_key{$got_stage}}->{fp}},$1;
                    push @{$ret_output{$element_key{$got_stage}}->{inode}},$2;
                    push @{$ret_output{$element_key{$got_stage}}->{fbn}},$3;
                    push @{$ret_output{$element_key{$got_stage}}->{wi_gen}},$4;
                    push @{$ret_output{$element_key{$got_stage}}->{cp_cnt}},$5;
                    push @{$ret_output{$element_key{$got_stage}}->{compress_attempted}},$6;
                }
        } elsif ( $line =~ /^\s*(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)$/ ) {
            print "Line obtaines is $line \n";
            if ($got_stage eq "clogdir.0" || $got_stage eq "clogdir.1") {
                push @{$ret_output{$element_key{$got_stage}}->{fp}},$1;
                push @{$ret_output{$element_key{$got_stage}}->{inode}},$2;
                push @{$ret_output{$element_key{$got_stage}}->{fbn}},$3;
                push @{$ret_output{$element_key{$got_stage}}->{vvbn}},$4;
                push @{$ret_output{$element_key{$got_stage}}->{wi_gen}},$5;
                push @{$ret_output{$element_key{$got_stage}}->{cp_cnt}},$6;
                push @{$ret_output{$element_key{$got_stage}}->{compress_attempted}},$7;
                push @{$ret_output{$element_key{$got_stage}}->{unprocessed}},$8;
            } else {
                push @{$ret_output{$element_key{$got_stage}}->{fp}},$1;
                push @{$ret_output{$element_key{$got_stage}}->{inode}},$2;
                push @{$ret_output{$element_key{$got_stage}}->{fbn}},$3;
                push @{$ret_output{$element_key{$got_stage}}->{vvbn}},$4;
                push @{$ret_output{$element_key{$got_stage}}->{wi_gen}},$5;
                push @{$ret_output{$element_key{$got_stage}}->{cp_cnt}},$6;
                push @{$ret_output{$element_key{$got_stage}}->{compress_attempted}},$7;
                push @{$ret_output{$element_key{$got_stage}}->{unprocessed}},$8;
            }
        }

        }

    return \%ret_output;
}

=head2 get_changelog_opt(host=>$host, volname=>$vol_name, partner=>$partner)

Get the changelog option value of vol options 
Since this is a readonly option, no sis_logging  
is abled to add to vol options

=over 4

=item host

Filer HostSpec

=item Returns:

Success

=over 4

Return the array of volume names, it can be null

=back

Failure

=over 4

Return error in C<$@>

=back

=back

=cut

sub get_changelog_opt($@)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR },
                                 volname => { type=>SCALAR },
				 partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
                          );

    my ($host)=$opts{host};
    my ($vol_name)=$opts{volname};

    my ($conn)=make_conn($host,"#rsh");
    my $cmd;

    if($opts{partner}) {
    	$cmd = "partner". " ". "vol options " . $vol_name;
    } else {
    	$cmd = "vol options " . $vol_name;
    }

    my $opt_value;

    my ($return_text, $result_code) = filer_cli_test_say($conn, $cmd);

    # Can't check result_code, it doesn't work 
    if ($return_text){
        my @opts = split (/,/, $return_text);

        if (scalar(@opts) > 0){
            @opts = grep(/sis_logging/, @opts);
            my @values = split (/=/, $opts[0]);
            $opt_value = $values[1];
        } # end if scalar
    }

    return $opt_value;
}

=head2 get_share_vols(host=>$host, partner=>$partner)

Get the dense volumes that have shared blocks

=over 4

=item host

Filer HostSpec

=item Returns:

Success

=over 4

Return the array of volume names, it can be null

=back

Failure

=over 4

Return error in C<$@>

=back

=back

=cut

sub get_share_vols($@)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR }, 
				 partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } }
		 );

    my ($host) = $opts{host};

    my ($conn) = make_conn($host,"#rsh");
    my @share_vols = ();

    my %stat_info = dense_stat(host=>$host, long=>1, partner=>$opts{partner});

    #if (! \%stat_info )
    foreach my $vol_name (keys %stat_info){
        my $save_value = $stat_info{$vol_name}{"Shared"};
        # get saving number
        if (Dense::Status::dense_kb_atoi(kb=>$save_value) > 0 ){
            push (@share_vols,$vol_name);
        }
    }
    return @share_vols;
}

=head2 is_dense_flag(host=>$host, volname=>$volName)

Check if the volume has dense flag or not

=over 4

=item host

Filer HostSpec

=item volName

Volume name 

=item partner

partner mode

=item Returns:

Dense flag is set 

=over 4

Return 1 if the volume is set to dense

=back

Dense flag is not set 

=over 4

Return 0 if the volume is not set to dense

=back

=back

=cut

sub is_dense_flag($@)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR },
				 volname => { type=>SCALAR },
				 partner => { type=>SCALAR, optional=>1, default=>0 } } 
			  );

    my ($host)=$opts{host};
    my ($volname)=$opts{volname};
    my ($partner)=$opts{partner};

    my ($conn)=make_conn($host,"#rsh");
    my $status;

    if($partner) {
	my ($return_text, $result_code) = filer_cli_say($conn, "partner vol status /vol/$volname");
	$status = $return_text;	
    } else {

    	# check volume is dense
    	my $status_obj = vol_status($conn, $volname);
	    if (! $status_obj){
        	die("No vol status for $volname\n");
    	}
    	$status = $status_obj->status();
    }   

    if ( $status =~ /sis/ ){
        return 1;    
    }else{
        return 0; 
    }
   
}

=head2 set_filer_sync(host=>$host)

Issue "sync sync" to make sure data flush to the disk.

=over 4

=item host

Hostspec, required.

=item Returns:

Successful

=over 4

Return 1

=back

Failure

=over 4

Error

=back

=back

=cut

sub set_filer_sync($@)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR } });

    my ($host)=$opts{host};

    my ($conn)=make_conn($host,"#rsh");

    my ($return_text, $result_code) = filer_cli_say($conn, "priv set -q diag; sync;sync", {timeout=>300});

##     ## $result_code is always 255 
##     if( $result_code eq cmdOkReturn){
##        return 1; 
##     }else{
##        die("Sync failed: $return_text\n");
##     }
}

=head2 dense_get_stage_internal(host=>$host, path_name=>$path_name, partner=>$partner)

Get current dense stages of the path specified. Return an be GATHERING, SORTING,

SAVING, DONE, DASH and NOSTATUS
  
=over 4

=item host

Filer HostSpec

=item path_name

Valid path name (for phase one, volume name), in /vol/volumename

=item Returns:

=over 4

Success

=over 4

Return one of the following GATHERING, SORTING, SAVING, DONE, DASH and NOSTATU
S.

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_get_stage_internal($@)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR },
				 path_name => { type=>SCALAR },
				 partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } 
			  );

    my ($host) = $opts{host};
    my ($path_name) = $opts{path_name};

    my ($conn) = make_conn($host,"#rsh");

    my $current_stage;
    my $current_op_error;

    # get status
    my %status = eval {
        dense_status(host=>$host, long=>1, diag_mode=>1, path=>$path_name, partner=>$opts{partner});
    };

    if ($@){
        die("Unexpected operation error : $@\n");
    }else{
        $current_stage   = $status{$path_name}{"Stage"};
        $current_op_error = $status{$path_name}{"Current Operation Error"};
        if ( $current_stage =~ DENSE_STAGE_GATHERING){
            return ("GATHERING", $current_op_error);
        }elsif ( $current_stage =~ DENSE_STAGE_SORTING){
            return ("SORTING", $current_op_error);
        }elsif ( $current_stage =~ DENSE_STAGE_SAVING){
            return ("SAVING",  $current_op_error);
        }elsif ( $current_stage =~ DENSE_STAGE_CHECKING_P0){
            return ("CHECKING_P0",  $current_op_error);
        }elsif ( $current_stage =~ DENSE_STAGE_CHECKING_P1){
            return ("CHECKING_P1",  $current_op_error);
        }elsif ( $current_stage =~ DENSE_STAGE_CHECKING_P2){
            return ("CHECKING_P2",  $current_op_error);
        }elsif ( $current_stage =~ DENSE_STAGE_DONE){
            return ("DONE",  $current_op_error);
        }elsif ( $current_stage =~ DENSE_STAGE_DASH){
            return ("DASH",  $current_op_error);
        }elsif (! $current_stage ){
            return ("NOSTATUS", "-");
        }else{
            # unknown things found
            # does this OK
            die("Unknown stage was found $current_stage\n");
        }
    }
}

=head2 dense_revert_to()

Start sis revert_to on the supplied volume on the filer

=over 4

=item host

Filer HostSpec

=item path_name

Path name, Volume name for phase one, in /vol/volName

=item Validate

Optional, if set to 1, upgrades metafiles before reverting them again if needed and checks that sis revert_to happened on the volume, defaults to 1

=item Returns:

Success

=over 4

Return 1

=back

Failure

=over 4

Return with an error text, if validate is set
Die with error, if validate is not set

=back

=back

=cut
sub dense_revert_to {

   my (%opts) = validate(@_, { host => { type=>SCALAR },
                               path_name => { type=>SCALAR },
                               version => { type=>SCALAR, optional=>1 },
                               validate => { type=>SCALAR, default=>"0" },
                               timeout => { type=>SCALAR, default=>"600" },
                               count => { type=>SCALAR, default=>"10" },
                               cleanup => { type=>SCALAR, optional=>1, default=>0},
                               'clean-up' => { type=>SCALAR, optional=>1, default=>0},
                               partner => { type=>SCALAR, default=>0, regex=>qr/^(0|1)$/ } }
                         );

    my $host=$opts{host};
    my $path_name=$opts{path_name};
    host($host);
    my $conn=connect("#rsh-socket");
    my $validate=$opts{validate};
    my $version=$opts{version};
    my $cleanup=$opts{'clean-up'};
    my $cleanstr="-cleanup";
    my $cmd;
    my $err;

    if ($opts{cleanup}) {
        $cleanup = $opts{cleanup}; 
    }

    if ( $cleanup ) {

        if ( $opts{partner} ) {
            $cmd = "partner" . " " . DENSE_REVERT_NAME . " " . $cleanstr . " " . $path_name;
        } else {
            $cmd = DENSE_REVERT_NAME . " " . $cleanstr . " " . $path_name;
        }
    } else {
         if ( $opts{partner} ) {
            $cmd = "partner" . " " . DENSE_REVERT_NAME . " " . $version . " " . $path_name;
        } else {
            $cmd = DENSE_REVERT_NAME . " " . $version . " " . $path_name;
        }
    }

    ## If Validate is set, verify that sis revert_to happens on the volume, otherwise just return command error/ or 1 if success
    my (@strs, $tail, $done, $count, $txt, $error);

    ## Messages that are expected after sis revert_to
    @strs = ("'sis revert_to' command has succeeded on volume $path_name",
         "sis 'revert_to' command has failed on volume $path_name :Previously downgraded SIS metafiles exists",
         "sis 'revert_to' command has failed on volume $path_name :SIS metafiles upgrade not yet done ",
         "Operation is currently active: $path_name",
         "Operation is not enabled: $path_name");

    $done = 0;
    $error = 0;
    # Create a new instance of a filer_messages object where logFilePath will be "/etc/messages"
    # Start nfs service 
#    filer_service_start($conn, "nfs");
 #   $tail = filer_messages->new($host, \@strs, logFilePath=>'/etc/messages');


    my ($return_text, $result_code) = filer_cli_say($conn, $cmd);

    $return_text =~ s/\n//;

    my $msg_busy = GetMessage(code=>"DENSE_BUSY", host=>$host);


 	my $msg_conf = GetMessage(code=>"DENSE_NOT_CONFIGURED", host=>$host);

    if($return_text =~ /$msg_busy/) {
        logcomment("SIS operation is currently active on volume $path_name");
    $error=1;
        if( $validate) {
        dense_wait_for_status(host=>$host, path_name=>$path_name, status=>"IDLE", time_gap=>10, time_out=>$opts{timeout});
            dense_revert_to(host=>$host, version=>$version, count=>5, path_name=>$path_name, validate=>1);
        $err="DENSE_BUSY";
        $error=0;
    }
    $validate=0;
    }

    if($return_text =~ /$msg_conf/) {
        logcomment("SIS is not configured or enabled on volume $path_name");
    $validate=0;
    $error=1;
    }

    if($validate) {
         # Create a new instance of a filer_messages object where logFilePath will be "/etc/messages"
        # Start nfs service 
        filer_service_start($conn, "nfs");
        $tail = filer_messages->new($host, \@strs, logFilePath=>'/etc/messages');

        while( ($count <= $opts{count}) && (!$done) ) {
            my $lines = $tail->get_lines();
            if ( $tail->triggered() ) {
                $done = 1;
        my @match = $tail->get_matched_lines();
        $txt = $tail->dumpMatches(@match);
        logcomment("Found message: $txt");
        }
            sleep 5;
            $count++;
        }
	if (!$done) {
            die("$cmd did not finish in time. Timeout occured: $opts{count} iterations\n");
        }

    if($txt=~ /GetMessage(code=>"DENSE_REVERT_SUCCESS", host=>$host)/) {
            logcomment("sis revert_to was successful on volume $path_name: $txt");
    }

    if ($txt =~ /GetMessage(code=>"DENSE_UPGRADE_NOT_DONE", host=>$host)/) {
            logcomment("sis revert_to failed on volume $path_name: $txt. Running sis start on $path_name");
            dense_start(host=>$host, path_name=>$path_name);
            dense_wait_for_status(host=>$host, path_name=>$path_name, status=>"IDLE", time_gap=>10, time_out=>$opts{timeout});
        dense_revert_to(host=>$host, version=>$version, count=>5, path_name=>$path_name, validate=>1);
        $err="DENSE_UPGRADE_NOT_DONE";
        }

        if ($txt =~ /GetMessage(code=>"DENSE_DOWNGRADE_EXIST", host=>$host)/) {
            logcomment("Downgraded metafiles already exist on volume $path_name: $return_text");
        $err="DENSE_DOWNGRADE_EXIST";
        }

    }

    # No errror is return
    if(!$error){
        return 1;
    }elsif ($err) {
    logcomment("sis revert_to returned non fatal error: $err");
    return $err;
    } else{
        die("sis revert_to failed with error: $return_text\n");
    }

}

=head2 revert_sis_volumes(host=>$host, revert_to=>$release)

=over 4

=item host

Hostspec, required.

=item revert_to

Release to revert to(7.2 or 7.3), required.

=item undo

Run sis undo for volumes that cannot be reverted, optional, set to 0

=item retry_count

Number of retries for processing revert_to output, optional, set to 5

=item sis_timeout 

Timeout value for sis revert_to command, default value is 600, optional

=item timeout 

Timeout for revert_to command on filer, default value is 1200, optional

=item Returns:

=over 4

Success: 1

Failure: die on failure

=back

=cut

sub revert_sis_volumes {

    my (%opts) = validate (@_, { host => { type=>SCALAR }, 
			         revert_to => { type=>SCALAR },
			         undo => { type=>SCALAR, optional=>1, default=>0 }, 
			         retry_count => { type=>SCALAR, optional=>1, default=>5 }, 
			         sis_timeout => { type=>SCALAR, default=>600, optional=>1 },
			         timeout => { type=>SCALAR, default=>900, optional=>1 } }
			  );

    my $host = $opts{host};
    host($host);
    my ($conn) = connect("#telnet");
    my $version = $opts{revert_to};
    my $retry = $opts{retry_count};
    my @output;
    my $rvt_needed=1;

    # see what has to be done for the revert process.
    my $filer_revert_cmd = "revert_to -f $version";
    my $acls_warning_prompt = qr/Do you wish to continue\? \[yes\/no\]\?/;
    my $revert_failed_prompt = qr/Please address the above condition[s]?, then try \"revert_to\" again\./;

    logcomment("Attempting to revert to $version .. ");
    my ($before, $match, $after, $received_prompt) = say($conn, "$filer_revert_cmd",
                                                         '-noprompt', '-prompt', $acls_warning_prompt, $revert_failed_prompt,
                                                         '-ignore', "echo ignore",
                                                         '-comment', "Attempted $filer_revert_cmd",
                                                         '-timeout', $opts{timeout});
    # if warned to revert ACLs, respond with 'yes'
    if ($received_prompt =~ /$acls_warning_prompt/i) {
        ($before, $match, $after, $received_prompt) = say($conn, "yes",
                                                          '-noprompt', '-prompt', $revert_failed_prompt,
                                                          '-ignore', "echo ignore",
                                                          '-comment', "Attempted 'yes'");
    }
    if ($received_prompt !~ /$revert_failed_prompt/i) {
        die("Error: Could not obtain revert_to output on $host\n");
    }

    # convert "$after" into an array
    @output = split("\n", $after);

    while ( ($retry > 0) && ($rvt_needed) ) {

        # Error out if undo is not set and sis volumes cannot be reverted 
	my $undo_message = "because volume size or maxfiles exceeded max allowed for SIS";
        if (grep(/$undo_message/, @output)) {
	    if($opts{undo}) {
		## Find all volumes that need to be undone
		foreach (@output) {
		    if($_ =~ /sis undo (\/vol\/(\S+))/) {
			my $path = $1;
			$path =~ s/"//;
			dense_off_undo(host=>$host, path_name=>$path);	
			dense_wait_for_status(host=>$host, path_name=>$path, status=>'NOSTATUS', time_out=>$opts{timeout});
		    }
	        }
	    } else {
                die("Error: Cannot revert filer $host to $version. ASIS will have to be disabled on some or all volumes\n");
	    }
        }

        # Revert SIS metafiles, if needed
	my $revert_message = "Please revert SIS metafiles on Volume";
        if (grep(/$revert_message/, @output)) {
	
            # Get List of all Dense volumes
            my @dense_vols = get_dense_vols(host=>$host);

            foreach my $vol (@dense_vols) {
		my $MAX = $opts{sis_timeout} / 5;
		my $vol_path = "/vol/" . $vol;
		my $ret_val;
		eval {
		    $ret_val = dense_revert_to(host=>$host, version=>$version, count=>$MAX, path_name=>$vol_path, validate=>1);
		};
		if($@) {
		    die("dense_revert_to returned with an error: $@\n");
		}
	    }	    
	
        }

        logcomment("Attempting to revert to $version .. ");
        ($before, $match, $after, $received_prompt) = say($conn, "$filer_revert_cmd",
                                                         '-noprompt', '-prompt', $acls_warning_prompt, $revert_failed_prompt,
                                                         '-ignore', "echo ignore",
                                                         '-comment', "Attempted $filer_revert_cmd",
                                                         '-timeout', $opts{timeout});
        # if warned to revert ACLs, respond with 'yes'
        if ($received_prompt =~ /$acls_warning_prompt/i) {
            ($before, $match, $after, $received_prompt) = say($conn, "yes",
                                                              '-noprompt', '-prompt', $revert_failed_prompt,
                                                              '-ignore', "echo ignore",
                                                              '-comment', "Attempted 'yes'");
        }
        if ($received_prompt !~ /$revert_failed_prompt/i) {
            die("Error: Could not obtain revert_to output on $host\n");
        }


        logTrc("Response to cmd $filer_revert_cmd was $match $before \'$after\' $received_prompt.. ");
        
	# convert "$after" into an array
        @output = split("\n", $after);
	$rvt_needed = grep(/$revert_message/, @output);
 	$rvt_needed?logcomment("sis revert_to is needed"):logcomment("sis revert_to is not needed");
	$retry--;
    }

    if(!$rvt_needed) {
	logcomment("sis revert_to was successful on all sis volumes");
	return 1;
    }
    if($rvt_needed) {
	die("sis revert_to was not successful on all sis volumes. Too many retries\n");
    }

}

=head2 check_test_privilege(host=>$host)

Check for the test privilege and if exists use priv set -q test; in filer commands.

=over 4

=item host

Hostspec, required.

=item Returns:

=over 4

Success: The content of the DENSE_TEST_CMD constant

Failure: A null string

=back

=cut

sub check_test_privilege {
   
    my (%opts) = validate (@_, { host => { type=>SCALAR|HASHREF }, });
    my ($conn) = make_conn($opts{'host'},"#rsh");
    my $output;

    eval { (undef, undef, $output) = say($conn, 'priv set test'); };
    if ($output =~ /unrecognized privilege/) {
	chomp($output);
	logcomment($output);
	return '';
    } else {
	return DENSE_TEST_CMD;
    }
}


=head2 dense_clone_status(host=>$host, vol=>$vol, id=> $id)

Get the dense clone status

=over 4

=item host

Hostspec, required

=item vol

volume name

=item id

The clone ID

=item Returns:

%status_info. Hash of a Hash.The keys are (ID,field values)

=back

=cut

sub dense_clone_status {
        
    my (%opts) = validate (@_, { host => { type=>SCALAR|HASHREF }, 
                                 vol => { type => SCALAR},
				 id => { type=>SCALAR, optional=>1, default=>0 } }
			  );
    my $host = $opts{'host'};
    my $id;
    if($opts{'id'}) {
        $id = $opts{'id'};
    } else {
        $id = " ";
    }
    my $vol = $opts{'vol'};
    my ($conn) = make_conn($host,"#rsh");
    
	my ($status_array, $return_code) = filer_cli_advanced_say($conn, "clone status $vol $id");
        my @to_process = split(/\n/, $status_array);

        my $num = scalar(@to_process);
        my %status_info;
        my $key;
	my $index = 0;

        foreach my $line (@to_process) {

        # ID: 21
        # Source: /vol/p3vol/file.0
        # Destination: /vol/p3vol/file.0_clone.15
        # Block ranges: 
        # State: failed (Not enough disk space for the operation)
        # Type: file
        # 
        # ID: 84
        # Source: /vol/temp/lun1
        # Destination: /vol/temp/lun1_clone
        # Block ranges: 
        # State: failed (Not enough disk space for the operation)
        # Type: lun

                # It starts with an empty string
                if ( $line =~ m/\w+/){
			if($line =~ /^\s*$/) {
				next;
			}
                      my($field, $value) = split(":",$line);
                      $field = remove_leading_trailing_spaces($field);
                      $value = remove_leading_trailing_spaces($value);
                       
                      if( $field =~ m/ID/ ){
                           $key = $value;
                           next;
                      }  
                      if($field =~ m/State/) {
                          # split on '(' and remove the trailing ')" using chop
                          my @tmp_arr = split(/\(/,$value);
                          chop($tmp_arr[1]);
                          $status_info{$key}{'State'} = $tmp_arr[0];
                          $status_info{$key}{'Status'} = $tmp_arr[1];
                          next;
                      }
                      $status_info{$key}{$field} = $value;
                }
        } # end foreach
        return %status_info;
}

sub parse_clone_status {

    my (%opts) = validate (@_, { host => { type=>SCALAR|HASHREF },
                                 id => { type=>SCALAR, optional=>1, default=>0 } }
                          );
    my $host = $opts{'host'};
    my $id = $opts{'id'};
    my ($conn) = make_conn($host,"#rsh");

        my ($status_array, $return_code) = filer_cli_advanced_say($conn, "clone status $id");
        my @to_process = split(/\n/, $status_array);

        my $num = scalar(@to_process);
        my %status_info = ();
        my $key="UNKNOWN";
        my $index = 0;

        foreach my $line (@to_process) {

        ## ID: 21
        ## Source: /vol/p3vol/file.0
        ## Destination: /vol/p3vol/file.0_clone.15
        ## Block ranges:
        ## State: failed (Not enough disk space for the operation)
        ## Type: file
        ##
        ## ID: 84
        ## Source: /vol/temp/lun1
        ## Destination: /vol/temp/lun1_clone
        ## Block ranges:
        ## State: failed (Not enough disk space for the operation)
        ## Type: lun

                # It starts with an empty string
                if ( $line ne ""){

                        if($line =~ /^\s*$/) {
                                next;
                        }
                        # Begin with non-white space character
                        if ($line =~ /^\s*(\S.+):\s+(.+?)\s*$/) {
                                my $field = $1;
                                my $data = $2;

                                ## logcomment("DEBUG: Field is $field and Data is $data");

                                # The top level key is the index
                                if ( $field eq "ID"){
                                        $key = $index;
                                        $index++;
                                }
                                $status_info{$key}{$field} = $data;

                        }else {
                                logresult("FAIL", "ERROR: Unparsable line in clone status output: $line");
                        }
                }

        } # end foreach

        return %status_info;
}

sub clone_clear_status {

    my (%opts) = validate (@_, { host => { type=>SCALAR|HASHREF },
                                 id => { type=>SCALAR } }
                          );
    my $host = $opts{'host'};
    my $id = $opts{'id'};
    my ($conn) = make_conn($host,"#rsh");

        my ($return_text, $result_code) = filer_cli_advanced_say($conn, "clone clear $id");

	if($return_text =~ /No such clone operation/) {
		logcomment("WARNING: Clone ID does not exist");
	}
	
        ($return_text, $result_code) = filer_cli_advanced_say($conn, "clone status $id");
	if(!$return_text) {
		logcomment("Successfully cleared clone status entry for ID: $id");
	}else {
		logcomment("WARNING: clone status entry for ID: $id was not cleared");
	}

}

sub clone_clear_failed {

    my (%opts) = validate (@_, { host => { type=>SCALAR|HASHREF },
                                 status => { type=>HASHREF } }
                          );
    my $host = $opts{'host'};
    my $statusref = $opts{'status'};
    my %status = %$statusref;
    my ($conn) = make_conn($host,"#rsh");

	foreach my $key (keys %status) {

		if($status{$key}{'State'} =~ /failed/i) {	
	
        		my $id = $status{$key}{'ID'};
        		logcomment("ID to be cleared is $id and State is $status{$key}{'State'}");
       			clone_clear_status(host=>$host, id=> $id); 

		}
	}

}


=head2 dense_create_checkpoint(host=>$host, path_name=>$path_name, stage=>$stage, checkpoint_timeout=>$checkpoint_timeout, sleep=>$sleep)

Create dense checkpoint for a particular stage and wait at that checkpoint for checkpoint_timeout value

=over 4

=item host

Filer HostSpec

=item $path_name

Valid path in /vol/volname

=item stage

One of the following stage, GATHERING, SORTING, DEDUPING_P1, DEDUPING_P2, CHECKING_P1

=item checkpoint_timeout 

Waits at the restart checkpoint stage for the specficied time

=item sleep 

Sleep n seconds before fiji is hit. Use this option based on volume size.

=item Returns:

Success

=over 4

Return hash value of checkpoint stats information 

=back

Failure

=over 4

Return 0.

=back

=back

=cut

sub dense_create_checkpoint($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
                                stage => { type=>SCALAR },
                                sub_stage => { type=>SCALAR, optional=>1 },
                                vf_id => { type=>SCALAR, optional=>1 },
                                scan => { type=>SCALAR, optional=>1, default=>0 },
                                checkpoint_timeout => { type=>SCALAR, optional=>1, default=>60 },
                                sleep => { type=>SCALAR, optional=>1, default=>0 } }
                          );

    my ($host) = $opts{host};
    my ($path_name) = $opts{path_name};
    my ($stage) = $opts{stage};
    my ($vf_id) = $opts{vf_id};
    my ($scan) = $opts{scan};
    my ($checkpoint_timeout) = $opts{checkpoint_timeout}; 
    my ($sleep) = $opts{sleep};
    my $sub_stage = $opts{sub_stage};
    my $stage_next = "";
    my $stage_expected = "";
    my %checkpoint_stats = ();
    my $output = "";

    # Restart checkpoint is not applicable for the stage CHECKING_P2
    die ("Cannot have restart checkpoint for [$stage] \n") if ($stage =~ /CHECKING_P2/);

    # Compute the next stage for a given stage. 
    my %dense_stages = (
	GATHERING 	=> "Gathering",
	SORTING 	=> "Sorting",
	DEDUPING_P1 	=> "Sorting",
	DEDUPING_P2 	=> "Saving",
	DEDUPING_SHARING=> "Saving",
	CHECKING_P1 	=> "Checking_P1"
	);
    my %checkpoint_stages = (
	GATHERING 	=> "Gathering_phase2",
	SORTING 	=> "Sorting",
	DEDUPING_P1 	=> "Saving_pass1",
	DEDUPING_P2 	=> "Saving_pass2",
	DEDUPING_SHARING=> "Saving_sharing",
	CHECKING_P1 	=> "Checking_pass2"
    );
    my %fiji_stages = (
	GATHERING 	=> "GATHERING",
	SORTING 	=> "SORTING",
	DEDUPING_P1 	=> "SORTING",
	DEDUPING_P2 	=> "DEDUPING_P1",
	DEDUPING_SHARING=> "DEDUPING_SHARING",
	CHECKING_P1 	=> "CHECKING_P1"
    );
    my @path = split(/\//, $path_name);
    my $vol_name = $path[2];

    my ($conn) = make_conn($host, "#rsh");

    # Set the fiji on the filer and start sis with scan set to 1

   
    my $checkpoint_stage;
    my $fiji_set_stage = undef;
    $checkpoint_stage = $checkpoint_stages{$stage};

    my $fiji_stage = "DENSE_STAGE_".$fiji_stages{$stage};
    if ($sub_stage eq "BUCKET_SORT") {

        $output = filer_cli_test_say($conn, "fiji filter \"(repeat 10 (dense.delay (eq \@FUNC_NAME bucket_sort_worker_sort)(sleep $checkpoint_timeout)))\"");
    
    } elsif ($stage eq "DEDUPING_SHARING" ) {
    
        $output = filer_cli_test_say($conn, "fiji filter \"(dense.delay.sharing(eq \@path /vol/$vol_name)(sleep $checkpoint_timeout))\"");
    
    } else {
        $output = filer_cli_test_say($conn, "fiji filter \"(repeat 10 (dense.sleep.stage (&& (eq \@sleep_stage $fiji_stage) (eq \@vol $vol_name)) (return $checkpoint_timeout)))\"");
    
    }
    if ( $output !~ /fiji: filter set/ ) { 
        die "Unable to set the fiji filter for stage [$stage] on [$vol_name] \n";
    }
    
    if ( $stage !~ /CHECKING/ ) {
        eval {
            if ( defined($vf_id) ) {
                Dense::dense_vfiler::vfiler_dense_start(host=>$host, path=>$path_name, vf_id=>$vf_id, scan=>$scan);
            } else {
                dense_start(host=>$host, path_name=>$path_name, scan=>$scan);
            }
        };
        if ($@){ die("$@\n"); } 
    }
    my (%status_info, $ckpt_time, $stage_curr, $checkpoint_stage_curr, $status);
 
    if (($stage eq "GATHERING") || ($sub_stage eq "BUCKET_SORT")) {

        dense_wait_for_stage(host=>$host, path_name=>$path_name, stage=>$stage, time_out=>600);

	# Increase the system time by 1 hour for gatherer checkpoint to be created

	 my ($raworigsysdate, $time, $date, $newdate);
         $raworigsysdate = system_time_get($conn);
	 $time = parsedate($raworigsysdate);
	 $time = $time + 3660;
	 $date = &ParseDateString("epoch $time");
	 $newdate = substr($date, 2, length($newdate)-3);
	 $newdate =~ s/://;
	 system_time_set($conn, $newdate, "FALSE");
     filer_cli_say($conn, "priv set -q diag; sync; sync", 300);
	 # Wait till the checkpoint gets created

         my ($count, $flag) = (0, 0);

         if ( defined($vf_id) ) {
             %status_info = Dense::dense_vfiler::vfiler_dense_status(host=>$host, vf_id=>$vf_id, path=>$path_name);
             $stage_curr = $status_info{$vf_id}{$path_name}{'Stage'};
             $ckpt_time = $status_info{$vf_id}{$path_name}{'Checkpoint Time'};
         } else {
             %status_info = dense_status(host=>$host, long=>1, path=>$path_name);
             $stage_curr = $status_info{$path_name}{'Stage'};
             $ckpt_time = $status_info{$path_name}{'Checkpoint Time'};
         }
        
         logcomment("Expected Dense stage = $dense_stages{$stage} , Expected checkpoint_stage = $checkpoint_stages{stage}");
         until (($stage_curr ne $dense_stages{$stage})  ||  ($count == 40) ) {
             if ($ckpt_time !~ /No Checkpoint/ ) {
                 $flag = 1;
                 last;
             }
             logcomment ("Debug[$count]: Waitng ...");
             $count++;
             sleep 2;
             if ( defined($vf_id) ) {
                 %status_info = Dense::dense_vfiler::vfiler_dense_status(host=>$host, vf_id=>$vf_id, path=>$path_name);
                 $stage_curr = $status_info{$vf_id}{$path_name}{'Stage'};
                 $ckpt_time = $status_info{$vf_id}{$path_name}{'Checkpoint Time'};
             } else {
                 %status_info = dense_status(host=>$host, long=>1, path=>$path_name);
                 $stage_curr = $status_info{$path_name}{'Stage'};
                 $ckpt_time = $status_info{$path_name}{'Checkpoint Time'};
             }
             logcomment("Expected Dense stage = $dense_stages{$stage} , Got stage = $stage_curr");
         }

         if ( ($count == 40) || (!$flag) ) {
             die ("Restart Checkpiont is not created after 1 hour of Gathering/Sorting phase \n");
         }

         if ( defined($vf_id) ) {
             Dense::dense_vfiler::vfiler_dense_stop(host=>$host, vf_id=>$vf_id, path=>$path_name);
         } else {
             dense_stop(host=>$host, path_name=>$path_name);
            }
         dense_wait_for_status(host=>$host, path_name=>$path_name, status=>"IDLE");

    } else {
        if ($stage =~ /CHECKING_P1/) { 
            dense_wait_for_status(host=>$host, path_name=>$path_name, status=>"IDLE");
            eval {
                Dense::dense_vfiler::vfiler_dense_check(host=>$host, vf_id=>$vf_id, path=>$path_name) if ( defined($vf_id) );
                dense_check(host=>$host, path_name=>$path_name);
            };
            if ($@){ die("$@\n"); }
        }

        # Make sure dense has entered the required "stage" and "checkpoint stage"

        if ( defined($vf_id) ) {
            %status_info = Dense::dense_vfiler::vfiler_dense_status(host=>$host, vf_id=>$vf_id, path=>$path_name);
            $status = $status_info{$vf_id}{$path_name}{'Status'};
            $stage_curr = $status_info{$vf_id}{$path_name}{'Stage'};
            $checkpoint_stage_curr = $status_info{$vf_id}{$path_name}{'Checkpoint Stage'};
        } else { 
            %status_info = dense_status(host=>$host, long=>1, path=>$path_name);
            $status = $status_info{$path_name}{'Status'};
            $stage_curr = $status_info{$path_name}{'Stage'};
            $checkpoint_stage_curr = $status_info{$path_name}{'Checkpoint Stage'};
        }

        logcomment("Dense stage is :$stage_curr ,Checkpoint stage is :$checkpoint_stage_curr, Expecting dense stage as $stage_expected"); 
        sleep($sleep);

        until ( ( ($stage_curr =~ /$dense_stages{$stage}/i) && ($checkpoint_stage_curr =~ /$checkpoint_stages{$stage}/i) ) || ($status eq "Idle" ) ) {
            if ( defined($vf_id) ) {
                %status_info = Dense::dense_vfiler::vfiler_dense_status(host=>$host, vf_id=>$vf_id, path=>$path_name);
                $status = $status_info{$vf_id}{$path_name}{'Status'};
                $stage_curr = $status_info{$vf_id}{$path_name}{'Stage'};
                $checkpoint_stage_curr = $status_info{$vf_id}{$path_name}{'Checkpoint Stage'};
            } else {
                %status_info = dense_status(host=>$host, long=>1, path=>$path_name);
                $status = $status_info{$path_name}{'Status'};
                $stage_curr = $status_info{$path_name}{'Stage'};
                $checkpoint_stage_curr = $status_info{$path_name}{'Checkpoint Stage'};
            }
       	    logcomment("Actual dense stage is :$stage_curr ,Checkpoint stage :$checkpoint_stage_curr, SIS status =$status");
            logcomment("Expected dense stage :$dense_stages{$stage} Checkpoint stage = $checkpoint_stages{$stage}");
        }

        if ($status ne "Idle") {
            if ( defined($vf_id) ) {
                Dense::dense_vfiler::vfiler_dense_stop(host=>$host, vf_id=>$vf_id, path=>$path_name);
            } else {
                dense_stop(host=>$host, path_name=>$path_name);
            }
            dense_wait_for_status(host=>$host, path_name=>$path_name, status=>"IDLE");
        } else {
            die "Restart Checkpiont is not created for the stage [$stage] \n";
        }
    }

    filer_cli_test_say($conn, "fiji clear");
    # Get checkpoint information and fill the hash
    if ( defined($vf_id) ) {
        %status_info = Dense::dense_vfiler::vfiler_dense_status(host=>$host, vf_id=>$vf_id, path=>$path_name);
        logcomment("Checking status_info output");
        print Dumper(\%status_info);
        $checkpoint_stats{'Checkpoint Op Type'} = $status_info{$vf_id}{$path_name}{'Checkpoint Op Type'};
        $checkpoint_stats{'Checkpoint Stage'} = $status_info{$vf_id}{$path_name}{'Checkpoint Stage'};
        $checkpoint_stats{'Checkpoint Sub-stage'} = $status_info{$vf_id}{$path_name}{'Checkpoint Sub-stage'};
        $checkpoint_stats{'Checkpoint Progress'} = $status_info{$vf_id}{$path_name}{'Checkpoint Progress'};
        $checkpoint_stats{'Checkpoint Magic'} = $status_info{$vf_id}{$path_name}{'Checkpoint Header Information'}{'Checkpoint Magic'};
        $checkpoint_stats{'Checkpoint Checksum'} = $status_info{$vf_id}{$path_name}{'Checkpoint Header Information'}{'Checkpoint Checksum'};
        $checkpoint_stats{'Checkpoint Version'} = $status_info{$vf_id}{$path_name}{'Checkpoint Header Information'}{'Checkpoint Version'};
        $checkpoint_stats{'Checkpoint Size'} = $status_info{$vf_id}{$path_name}{'Checkpoint Header Information'}{'Checkpoint Size'};
        logcomment ("dense_create_checkpoint : Checkpoint stats are \n");
        print Dumper(\%checkpoint_stats);

        if ( $status_info{$vf_id}{$path_name}{'Checkpoint Stage'} eq $checkpoint_stages{$stage}) {
            return (\%checkpoint_stats);
        } else {
            die "Checkpoint is created for different stage : $status_info{$vf_id}{$path_name}{'Checkpoint Stage'}";
        }
    } else {
        %status_info = dense_status(host=>$host, long=>1, path=>$path_name);
        logcomment("Checking status_info output");
        print Dumper(\%status_info);
        $checkpoint_stats{'Checkpoint Op Type'} = $status_info{$path_name}{'Checkpoint Op Type'};
        $checkpoint_stats{'Checkpoint Stage'} = $status_info{$path_name}{'Checkpoint Stage'};
        $checkpoint_stats{'Checkpoint Sub-stage'} = $status_info{$path_name}{'Checkpoint Sub-stage'};
        $checkpoint_stats{'Checkpoint Progress'} = $status_info{$path_name}{'Checkpoint Progress'};
        $checkpoint_stats{'Checkpoint Magic'} = $status_info{$path_name}{'Checkpoint Header Information'}{'Checkpoint Magic'};
        $checkpoint_stats{'Checkpoint Checksum'} = $status_info{$path_name}{'Checkpoint Header Information'}{'Checkpoint Checksum'};
        $checkpoint_stats{'Checkpoint Version'} = $status_info{$path_name}{'Checkpoint Header Information'}{'Checkpoint Version'};
        $checkpoint_stats{'Checkpoint Size'} = $status_info{$path_name}{'Checkpoint Header Information'}{'Checkpoint Size'};
        logcomment ("dense_create_checkpoint : Checkpoint stats are \n");
        print Dumper(\%checkpoint_stats);

        if ( $status_info{$path_name}{'Checkpoint Stage'} eq $checkpoint_stages{$stage}) {    
    	    return (\%checkpoint_stats);
        } else {
    	    die("Checkpoint is created for different stage : $status_info{$path_name}{'Checkpoint Stage'}");
        }
    }
}

=head2 dense_verify_checkpoint(host=>$host, path_name=>$path_name, checkpoint_stats=>checkpoint_stats)

Verify the existance of dense checkpoint for a particular stage

=over 4

=item host

Filer HostSpec

=item $path_name

Valid path in /vol/volname

=item checkpoint_stats

Hash value of checkpoint stats obtained from dense_create_checkpoint

=item Returns:

Success

=over 4

Return 1

=back

Failure

=over 4

Return 0.

=back

=back

=cut

sub dense_verify_checkpoint($@)
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
                                checkpoint_stats => { type=>HASHREF } }
                          );

    my ($host) = $opts{host};
    my ($path_name) = $opts{path_name};
    my $checkpoint_stats_ref = $opts{'checkpoint_stats'};
    my %checkpoint_stats = %$checkpoint_stats_ref;
    my %checkpoint_stats_curr = ();

    my ($conn) = make_conn($host, "#rsh");
    my %status_info = dense_status(host=>$host, long=>1, path=>$path_name);

    # Get checkpoint information and fill the hash

    $checkpoint_stats_curr{'Checkpoint Op Type'} = $status_info{$path_name}{'Checkpoint Op Type'};
    $checkpoint_stats_curr{'Checkpoint Stage'} = $status_info{$path_name}{'Checkpoint Stage'};
    $checkpoint_stats_curr{'Checkpoint Sub-stage'} = $status_info{$path_name}{'Checkpoint Sub-stage'};
    $checkpoint_stats_curr{'Checkpoint Progress'} = $status_info{$path_name}{'Checkpoint Progress'};
    $checkpoint_stats_curr{'Checkpoint Magic'} = $status_info{$path_name}{'Checkpoint Header Information'}{'Checkpoint Magic'};
    $checkpoint_stats_curr{'Checkpoint Checksum'} = $status_info{$path_name}{'Checkpoint Header Information'}{'Checkpoint Checksum'};
    $checkpoint_stats_curr{'Checkpoint Version'} = $status_info{$path_name}{'Checkpoint Header Information'}{'Checkpoint Version'};
    $checkpoint_stats_curr{'Checkpoint Size'} = $status_info{$path_name}{'Checkpoint Header Information'}{'Checkpoint Size'};

    # Make sure the values of checkpoint_stats and checkpoint_stats_curr are same
    # Hash comparision logic 
     
    my (@checkpoint_stats, @checkpoint_stats_curr, $equal);
    @checkpoint_stats = sort keys %checkpoint_stats;
    @checkpoint_stats_curr = sort keys %checkpoint_stats_curr;
    $equal = 0;

    if ($#checkpoint_stats == $#checkpoint_stats_curr) {
        $equal++;
        for (0..$#checkpoint_stats) { 
            $equal--, last if $checkpoint_stats[$_] ne $checkpoint_stats_curr[$_] 
        }
        if ($equal) {
            for (@checkpoint_stats) { 
               $equal--, last if $checkpoint_stats{$_} ne $checkpoint_stats_curr{$_} 
            }
        }
    }
   
    logcomment ("Previous checkpoint stats \n");
    print Dumper(\%checkpoint_stats);
    logcomment ("Current checkpoint stats \n");
    print Dumper(\%checkpoint_stats_curr);

    return 1 if ($equal);

    die("Checkpoint validation failed. Current checkpoint stats did not match with previous one \n");
}

=head2 dense_num_gatherer_scanners(conn=>$conn, num=>$num)

To retrive or set the 'options sis.num_gatherer_scanners' value

=over 4

=item conn

Filer Connection object, required.

=item num

-if no value passed, it will return current 'sis.num_gatherer_scanners' value

-if value is passed, it will set the 'sis.num_gatherer_scanners' to the passed value

Valid $num value ranges between : 1-64

=item Returns

Success

=over4

-if no value passed in num, it will return current 'sis.num_gatherer_scanners' value

-if value is passed in num, it will return 1

=back

Failure

=over4

Return error text from cli

=back

=back

=cut

sub dense_num_gatherer_scanners(%)
{
    my %opts = validate (@_, {  conn => { type=>HASHREF },
                                num => { type=>SCALAR, optional=>1 } }
                        );
    my $conn = $opts{conn};
    my $num = $opts{num};

    my $cmd = 'options sis.num_gatherer_scanners';
    my ($return_text, $return_code, $current_num);
    
    ($return_text, $return_code) = filer_cli_say($conn, $cmd);
        if ($return_text =~ /sis.num_gatherer_scanners\s+(\d+)/) {
            $current_num = $1;
            logcomment("Current num_gatherer_scanners value is: $current_num");
        }else {
            die("$return_text \n");
	}
    if ($num) {
        logcomment("Setting num_gatherer_scanners value to: $num");
	$cmd = "options sis.num_gatherer_scanners $num";
	($return_text, $return_code) = filer_cli_say($conn, $cmd);
        my $current_num = dense_num_gatherer_scanners(conn=>$conn);
            if ($current_num == $num) {
                logcomment("num_gatherer_scanners value set to: $num");
                return 1;
            }else {
                die("$return_text - Value could not be set to: $num \n");
            }
    }else {
        return $current_num;
    }
}

=head2 dense_deleted_inode_hash_state(conn=>$conn, state=>$state)

To retrive or set the 'sis.deleted_inode_hash' state (on|off)

=over 4

=item conn

Filer Connection object, required

=item state

-if no value passed, it will return current 'sis.deleted_inode_hash' state

-if value is passed, it will set the 'sis.deleted_inode_hash' to the passed state

- Value must be 'on' or 'off'

=item Returns:

Success

=over 4

-if no value passed in state, it will return current 'sis.deleted_inode_hash' state

-if value is passed in state, it will return 1 

=back

Failure

=over 4

Return error text from cli

=back

=back

=cut

sub dense_deleted_inode_hash_state(%)
{
    my %opts = validate (@_, { conn => { type=>HASHREF },
                               state => { type=>SCALAR, optional=>1, regex=>qr/^(on|off)$/i } }
                        );
    my $conn = $opts{conn};
    my $state = $opts{state};
    
    unless (filer_kernel_is_debug($conn)) {
        die("Filer Kernel is not Debug - 'sis.deleted_inode_hash' not supported \n");
    }

    my $cmd = 'options sis.deleted_inode_hash';
    my ($return_text, $return_code, $current_state);
    ($return_text, $return_code) = filer_cli_say($conn, $cmd);
        if ($return_text =~ /sis.deleted_inode_hash\s+(on|off)/) {
            $current_state = $1;
            logcomment("Current sis.deleted_inode_hash state is: $current_state");
        }else {
            die("$return_text \n");
        }
    if ($state) {
        logcomment("Setting sis.deleted_inode_hash state to: $state");
        $cmd = "options sis.deleted_inode_hash $state";
        ($return_text, $return_code) = filer_cli_say($conn, $cmd);
        my $current_state = dense_deleted_inode_hash_state(conn=>$conn);
            if ($current_state =~ /$state/i) {
                logcomment("sis.deleted_inode_hash state set to: $current_state");
                return 1;
            }else {
                die("$return_text - sis.deleted_inode_hash could not be set to: $state \n");
            }
    }else {
        return $current_state;
    }
}

# ----------------------------------------------------
# Utility routine to strip trailing and leading spaces
# ----------------------------------------------------
=head2 remove_leading_trailing_spaces()

Inpurt:
	Scalar variable OR array reference

	Return value:

	Return the scalar variable (For Scalar input)
	Return the array reference (For array reference input)

=cut

sub remove_leading_trailing_spaces
{
    ### Check input type
    my $refType = ref(@_[0]);

    if($refType eq "ARRAY"){
        my @array_input = @{@_[0]};
        foreach (@array_input){
                s/^\s+//;
                s/\s+$//;
        }
        return(\@array_input);
    }else{
        my $input_val = @_[0];
        $input_val =~ s/^\s+//;
        $input_val =~ s/\s+$//;
        return($input_val);
    }
}


=head2 dense_get_state(host=>$host, path_name=>$path_name, partner=>$partner)

Wrapper around dense_get_state_internal
Get current dense state for path_name specified, return can be one of the 
following ENABLED, DISABLED and NOSTATUS.

=over 4

=item host

Filer HostSpec

=item path_name

Valid path name (for phase one, volume name)

=item Returns:

Success

=over 4

Return one of the following ENABLED, DISABLED and NOSTATUS.

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_get_state($@) 
{
   my (%opts) = validate (@_, { host => { type=>SCALAR },
                                path_name => { type=>SCALAR },
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ }, 
				expected_error => { type=>SCALAR | UNDEF, optional=>1 } } 
                          );
    my ($host)=$opts{host};
    my ($path_name)=$opts{path_name};
    
    my ($conn) = make_conn($host,"#rsh");

    my $expected_error = $opts{expected_error};

    # get status
    my $return_string;
	my %status = eval {
	$return_string = dense_get_state_internal(host=>$host, path_name=>$path_name, 
		partner=>$opts{partner}, expected_error=>$expected_error);
    };
    
    if ($@){
	die("$@\n");
    }else{
		return $return_string;
	}
}

#----------------------------------------------------------------------
# The testme routine to test the exported functions of this library
#
# ntest PKGNAME=Dense SUBNAME=testme util/test_pkg.thpl
#
# The routine can also be invoked with many optional parameters, such as:
#
# ntest PKGNAME=Dense SUBNAME=testme FUNCTION_NAME=dense_check util/test_pkg.thpl
#
#----------------------------------------------------------------------

sub testme() {

	package main;

	our ($FILER, $FUNCTION_NAME, $FILER_VOL, $FILER_CONN, $PARTNER);

	param('FILER',       '-mesg',    "Filer to test");
	param('FILER_CONN','-default',"#rsh");
	param('FUNCTION_NAME',   '-default',    "all");
        param('FILER_VOL',   '-default', 'vol0');
        param('PARTNER',   '-default', '0');

	package Dense;
	
	logcomment("Testing on FILER $FILER, Function $FUNCTION_NAME");
	my $ret;
	my $path_name = "/vol/".$FILER_VOL;

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_check"){
		
		# Test dense_check without any cmd options
		eval {
		$ret = dense_check(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
		};
	
		if ($@ =~ /Connection timed out/) {
			logresult('pass', "Expected fail. Check filer and connection type");
		} elsif ($@) {
			logresult('fail', "$@");
		} else {
			## print Dumper($ret);
		       	logresult('pass', "Got expected result"); 
		}
		
		# Test dense_check with cmd option -c
		eval {
		$ret = dense_check(host=>$FILER, path_name=>$path_name, cmd_opt=>"-c", partner=>$PARTNER);
		};
	
		if ($@ =~ /Connection timed out/) {
			logresult('pass', "Expected fail. Check filer and connection type");
		} elsif ($@) {
			logresult('fail', "$@");
		} else {
			## print Dumper($ret);
		       	logresult('pass', "Got expected result"); 
		}

		# Test dense_check with cmd option -v
		eval {
		$ret = dense_check(host=>$FILER, path_name=>$path_name, cmd_opt=>"-v", partner=>$PARTNER);
		};
	
		if ($@ =~ /Connection timed out/) {
			logresult('pass', "Expected fail. Check filer and connection type");
		} elsif ($@) {
			logresult('fail', "$@");
		} else {
			## print Dumper($ret);
		       	logresult('pass', "Got expected result"); 
		}
	}

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_config"){

		# Test dense_config with only required parameters
		eval {
			$ret = dense_config(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
		};

		if ($@ =~ /Connection timed out/) {
       	                 logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

		# Test dense_config with schedule_string set to a valid value
       		 eval {
		        $ret = dense_config(host=>$FILER, path_name=>$path_name, schedule_string=>'sun-sat@1-23', partner=>$PARTNER);
       		 };

	        if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

		# Test dense_config with schedule_string set to a invalid value
       		 eval {
         	        $ret = dense_config(host=>$FILER, path_name=>$path_name, schedule_string=>'sun-sat@1-23@~0-59', partner=>$PARTNER);
	         };

        	if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

		# Test dense_config with schedule_string set to a valid value in test mode
       		 eval {
		        $ret = dense_config(host=>$FILER, path_name=>$path_name, schedule_string=>'sun-sat@1-23@~0-59', test_mode=>"1", partner=>$PARTNER);
        	 };

	        if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

        }
	
       if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_config_unsched"){

		# Test dense_config_unsched with only required params
		eval {
                        $ret = dense_config_unsched(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	
		# Reset schedule on volume
		dense_config(host=>$FILER, path_name=>$path_name, schedule_string=>'sun-sat@1-23', partner=>$PARTNER);	

		# Test dense_config_unsched with all params
		eval {
			$ret = 	dense_config_unsched(host=>$FILER, path_name=>$path_name, schedule_string=>"-", partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

		# Reset schedule on volume
                dense_config(host=>$FILER, path_name=>$path_name, schedule_string=>'sun-sat@1-23', partner=>$PARTNER);

                # Test dense_config_unsched with invalid sched_string
                eval {
                        $ret = dense_config_unsched(host=>$FILER, path_name=>$path_name, schedule_string=>"\$", partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

	}
	
	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_toggle"){

		# Turn dense off on a sis volume
		 eval {
			$ret = dense_toggle(host=>$FILER, path_name=>$path_name, state=>"off", partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

		# Turn sis back on 
		 eval {
			$ret = dense_toggle(host=>$FILER, path_name=>$path_name, state=>"on", partner=>$PARTNER)
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

	}

        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_undo"){

                # Undo SIS on a volume
		eval {
                        $ret = dense_undo(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_off_undo"){

		# Turn off SIS and undo the volume
		eval {
			$ret = dense_off_undo(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	}
	
	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_start"){

		# Start SIS on the volume without -s option
		eval {
			$ret = dense_start(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

		# Start SIS on the volume with the -s option
		eval {
			$ret = dense_start(host=>$FILER, path_name=>$path_name, nosync=>"0", scan=>"1", partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	}
        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_stat"){

		# Get stats for all volumes
		eval {
			$ret = dense_stat(host=>$FILER, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

		# Get stats for a single volume
		eval {
			$ret = dense_stat(host=>$FILER, path=>$path_name, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }


		# Get stats in block format and long format
		eval {
			$ret = dense_stat(host=>$FILER, block=>"1", long=>"1", path=>$path_name, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }


		# Get stats in diag mode
		eval {
			$ret = dense_stat(host=>$FILER, diag_mode=>"1", path=>$path_name, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	}

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_status"){

                # Get status for all volumes
                eval {
			$ret = dense_status(host=>$FILER, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

                # Get status for a single volume
                eval {
			$ret = dense_status(host=>$FILER, path=>$path_name, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }


                # Get status in long format
                eval {
			$ret = dense_status(host=>$FILER, long=>1, path=>$path_name, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }


                # Get status in diag mode
                eval {
                        $ret = dense_status(host=>$FILER, diag_mode=>"1", path=>$path_name, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_stop"){

                # Get status for all volumes
                eval {
			$ret = dense_stop(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
                 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	}
	
        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_unpack"){

		my $tar_file = "/vol/".$FILER_VOL."/TEST/donor";
		my $unpack_dir = "/vol/".$FILER_VOL."/TEST/";

		eval {
			$ret = dense_unpack(host=>$FILER, tar_file=>$tar_file, unpack_dir=>$unpack_dir, avg_share_size=>"20", partner=>$PARTNER)
		};     
                 
                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	
	}
	
	 if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_share"){

		my $donor = $path_name . "/fileA";
		my $recipient = $path_name . "/fileB";

		eval {
			$ret = dense_share(host=>$FILER, donor=>$donor, dfbn=>2, recipient=>$recipient, rfbn=>3, partner=>$PARTNER);
                };    

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	}

	 if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_share_vp3"){
         
                my $donor = $path_name . "/fileA";
                my $recipient = $path_name . "/fileB";

                eval {
                        $ret = dense_share_vp3(host=>$FILER, donor=>$donor, dfbn=>2, recipient=>$recipient, rfbn=>3, no_cmp=>1, maxref=>1, r_hole=>1, partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
		
		eval {
                        $ret = dense_share_vp3(host=>$FILER, donor=>$donor, dfbn=>2, recipient=>$recipient, rfbn=>3, no_cmp=>1, maxref=>1, d_hole=>1, partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }
	
	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_clone"){

                my $source = $path_name . "/fileA";
                my $clone = $path_name . "/fileA_clone";

                eval {
			$ret = dense_clone(host=>$FILER, source=>$source, clone=>$clone, timeout=>500, partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	}
	
	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_get_config"){

		eval {
			my %hash = dense_get_config(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
			Dense::Status::print_hash(hashofhash=>\%hash);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
	}
	
	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_get_progress"){

		eval {
			my $value = dense_get_progress(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
			print "\n Progress: $value\n";
		};

		if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }
	
        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_get_last_op_size"){

                eval {
			my $value = dense_get_last_op_size(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
			print "\nLast op size: $value\n";
		};
		
		if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }
	
        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_get_stat_item"){

		my $item_name = "SIS Files";

                eval {
                        my $value = dense_get_stat_item(host=>$FILER, item_name=>$item_name, path_name=>$path_name, partner=>$PARTNER);
			print "\n $item_name: $value\n";
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }
	
	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_get_status_item"){

                my $item_name = "Blocks De-duplicated";

                eval {
			my $value = dense_get_status_item(host=>$FILER, item_name=>$item_name, path_name=>$path_name, partner=>$PARTNER);
			print "\n $item_name: $value\n";
		};

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }
	
	 if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_get_status_internal"){

                eval {
                        my $value = dense_get_status_internal(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
			print "\n STATUS: $value\n";
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }
	
         if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_get_state_internal"){
        
                eval {
                        my $value = dense_get_state_internal(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
			print "\n STATE: $value\n";
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }
	
         if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_is_licensed"){

                eval {
			$ret = dense_is_licensed(host=>$FILER);
		};

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
	                ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

         if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_set_license"){

		# Test without enforce platform
                eval {
			logcomment("Set License without enforce platform");
                        $ret = dense_set_license(host=>$FILER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

		# Remove License in between
		dense_remove_license(host=>$FILER);

		# Test with enforce platform
                eval {
                        logcomment("Set License with enforce platform");
                        $ret = dense_set_license(host=>$FILER, enforce_platform=>1);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

        }

        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_remove_license"){

                eval {
                        $ret = dense_remove_license(host=>$FILER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	}
	
        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_toggle_license"){

		# Remove License
                eval {
                        $ret = dense_toggle_license(host=>$FILER, action=>"0", code=>"");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

                # Set License
                eval {
                        $ret = dense_toggle_license(host=>$FILER, code=>"WQQQZGI");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_verify_changelog"){

                eval {
                        $ret = dense_verify_changelog(host=>$FILER, path_name=>$path_name, string_one=>"6", string_two=>"0", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	}

        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_verify_changelog_opt"){
                   
                eval {
                        $ret = dense_verify_changelog_opt(host=>$FILER, vol_name=>$FILER_VOL, verify_against=>"off", partner=>$PARTNER);
                }; 
                   
                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }  
		
		 eval {
                        $ret = dense_verify_changelog_opt(host=>$FILER, vol_name=>$FILER_VOL, verify_against=>"on", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }          

        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_verify_config"){

                eval {
                        $ret = dense_verify_config(host=>$FILER, path_name=>$path_name, verify_against=>"-", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	}
	
	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_verify_flag"){

                eval {
                        $ret = dense_verify_flag(host=>$FILER, vol_name=>$FILER_VOL, state=>"ON");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

                eval {
                        $ret = dense_verify_flag(host=>$FILER, vol_name=>$FILER_VOL, state=>"OFF");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_verify_stage"){

                eval {
                        $ret = dense_verify_stage(host=>$FILER, path_name=>$path_name, verify_against=>"DASH", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

		eval {
                        $ret = dense_verify_stage(host=>$FILER, path_name=>$path_name, verify_against=>"GATHERING", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_verify_state"){

                eval {
                        $ret = dense_verify_state(host=>$FILER, path_name=>$path_name, state=>"ENABLED", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

                eval {
                        $ret = dense_verify_state(host=>$FILER, path_name=>$path_name, state=>"NOSTATUS", partner=>$PARTNER); 
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_verify_status"){

                eval {
                        $ret = dense_verify_status(host=>$FILER, path_name=>$path_name, verify_against=>"UNDOING", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

                eval {
                        $ret = dense_verify_status(host=>$FILER, path_name=>$path_name, verify_against=>"IDLE", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_verify_status_error"){

                eval {
                        $ret = dense_verify_status_error(host=>$FILER, path_name=>$path_name, verify_error=>"-", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

                eval {
                        $ret = dense_verify_status_error(host=>$FILER, path_name=>$path_name, verify_error=>"Write to metafile failed", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_verify_stat"){

                eval {
                        $ret = dense_verify_stat(host=>$FILER, path=>$path_name, num_shared=>"20592", num_saved=>"0", num_dfiles=>"4", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
	}

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_wait_for"){

                eval {
                        $ret=dense_wait_for(host=>$FILER, path_name=>$path_name, wait_on=>"STAGE", wait_for=>"DONE", time_gap=>5, time_out=>60, expected_error=>"-", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_wait_for_status"){

                eval {
                        $ret = dense_wait_for_status(host=>$FILER, path_name=>$path_name, status=>"ACTIVE", time_gap=>1, time_out=>60, expected_error=>"-", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_wait_for_stage"){

                eval {
                        $ret = dense_wait_for_stage(host=>$FILER, path_name=>$path_name, stage=>"SAVING", time_gap=>1, time_out=>60, expected_error=>"-", partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "get_dense_vols"){

                eval {
                        $ret = get_dense_vols(host=>$FILER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "get_changelog_info"){

                eval {
                        $ret = get_changelog_info(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);

                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "get_changelog_opt"){

                eval {
                        $ret = get_changelog_opt(host=>$FILER, volname=>$FILER_VOL, partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "get_share_vols"){

                eval {
                        my @arr = get_share_vols(host=>$FILER, partner=>$PARTNER);
			print ("\nVolumes with shared blocks are @arr\n");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "is_dense_flag"){

                eval {
                        $ret = is_dense_flag(host=>$FILER, volname=>$FILER_VOL);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

		if ($ret) { logcomment("$FILER_VOL is a dense volume"); }

		eval {
			$ret = is_dense_flag(host=>$FILER, volname=>$FILER_VOL, partner=>1);
		};

		if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
		
		if ($ret) { logcomment("$FILER_VOL is a dense volume"); }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_get_stage_internal"){

                eval {
                        $ret = dense_get_stage_internal(host=>$FILER, path_name=>$path_name, partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        ## print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

} ## End of Testme Routine

=head1 SEE ALSO

    Where additional information can be found.

=head1 BUGS

    Description of bugs or what the library does not do and why

=head1 STATUS

    Shared

=head1 ATTRIBUTES

    ONTAP_COMMANDS = volume, df, useradmin

=head1 AUTHOR

    Rani Palanisami (rani@netapp.com)
	$Id: //depot/prod/test/Rsierranevada/lib/Dense.pm#1 $
	Last revised $Date: 2013/01/22 $

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

1;
