#Hey emacs, this is basically -*- perl -*-
#$Id:$
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.

## @summary        Library that defines general purpose utility routines
## @description
##        File copying, socket operations, miscellaneous parsing, simulator
##      booting functions
##
## @author dl-waflqa@netapp.com (FSS WAFL Quality)

package FS::Util::Utilities;

use strict;
no strict "subs";

use Module::Common qw(validate_mixed);

BEGIN {
    my $VERSION = '0.02';
    my @utils   = qw(
        &uStatus
        &uSystem
        &uDate
        &uReadHeaders
        &uChkVals
        &uStrHi
        &uDiffDFValues

        &verify_say
        &verify_log
        &purge_log
        &count_log_lines
        &read_log_from_line

        &uCopy
        &uSeqStr &uSendSeq &uReadSeq

        &uGetSrcIP
        &uGetFQDN
        &uSoShutdownReader
        &uSoShutdownWriter
        &file_reserve
        &lun_reserve
        &pick_random_number

        &get_firmware_env
        &set_firmware_env
        &unset_firmware_env
        &revert_to

        &file_stat_hash

        &acceptable_rm_path
        &deltree
        &recursive_rm_path_check
        &delfile

        &util_unlink
        &util_rmdir
        &util_mkdir
        &util_rename
        &util_chmod
        &util_mkfile
        &util_link
        &util_filesize
        &util_stat
        &util_exists
        &disk_zero_spares

        &nfs_cifs_mkfile
        &nfs_cifs_copy
        &nfs_cifs_rename
        &nfs_cifs_rdonly
        &nfs_cifs_exists
        &nfs_cifs_rd_write
        &nfs_cifs_append
        &nfs_cifs_overwrite
        &nfs_cifs_delete
        &nfs_cifs_mkdir

        &lun_create
        &lun_destroy
        &lun_exists
        &lun_clone
        &lun_clone_split
        &lun_clone_split_status
        &lun_share
        &lun_rescan
        &lun_list_info
        &lun_clone_list_info
        &dump_wait
        &restore_wait

        &uSeqData
        &uShiftData
        &uRandData
        &uSendData
        &uReadData
        &uVerifyData
        &uReadVerifyData
        &uSendVerifyData
        &uHexDump
        &cifs_chown_file
        &cifs_file_getowner
        &write_file
        &get_mibfile_location
        &access_file_blocks
        &version_check_ic2
        &version_check_ic3
        &version_check_br1_and_higher
        &access_file_blocks_and_verify_ems
        &dumpblock
        &set_wafl_scan_speed
        &get_wafl_scan_speed
        &compare_counters
        &options_get
        &options_set
        &get_qualified_image_name
        &is_dual_inode
        &filersio_asyncio_active
        &filersio_asyncio_active
        &filer_sio_status
        &get_vbns
        &sv_filer_service_license
        &sv_filer_service_unlicense
    );

    my @switches = qw(ON OFF);

    Module::Common::setup_export_tags(
        switches => [@switches],
        utils    => [@utils]
    );
} ## end BEGIN

use boot;
use Carp;
use IO::Handle;
use IO::Socket::INET;
use Socket;
use POSIX qw(:sys_wait_h strftime);
use POSIX;
use SPARSE::Util::Logger;
use SayWrap qw(sayEOF);
use config_base qw(host_is_alive);
use SPARSE::Constants;
use Params::Validate
    qw(validate BOOLEAN SCALAR HASHREF ARRAYREF SCALARREF CODEREF OBJECT UNDEF);
use Utils qw(make_conn);
use Utils::Units qw(convert_to_bytes);
use Tharn;
use Data::Dumper;
use Mail::Sendmail;
use FileManip qw(stat_file setattr_file get_file_size test_file);
use Sys::Hostname;
use Filer::Version
    qw(get_wafl_version WV_VB_MIN WV_ROLLINGROCK_MIN WV_BOILERMAKER_MIN is_ontapng_version);
use filer qw(filer_kernel_is_debug filer_cli_diag_say filer_vol_file_exists
    filer_cli_say filer_cli_split filer_showfh filer_cli_test_say filer_model_get
    filer_service_license);
use TestEnv::MinWin qw(:all);
use Zapi;
use ZapiErrno;
use NaElement;
use Utils qw(make_host);
use CLI::FileIO qw(rdfile wrfile);
use SnapCmds qw(snapshot_status);
use misc qw(cli);
use NACL::APISet::Version;
use Admin::License qw( is_licensed unlicense );

my $sudo_cmd
    = defined($main::SUDO_PATH) ? $main::SUDO_PATH : "/usr/local/bin/sudo";

###########################
my $perlcmd_path = "/usr/software/bin/perl"
    ; #Changing path from /usr/local/bin/perl, as this path is no more supported by Engsup
##########################
use constant ON  => "on";
use constant OFF => "off";

# ------------------------------------------------------------------------

my $RM_NOTIFY_LIST = [qw(dl-waflqa@netapp.com)];

# ------------------------------------------------------------------------
#= Miscellaneous utilities

#= Convert process exit status to a legible error string - or undef if 0
#= :status like $?
#= >undef|result string
sub uStatus($) {
    my $status = shift;
    my $tmp;

    if ( WIFEXITED($status) ) {
        $tmp = WEXITSTATUS($status);
        if ($tmp) {
            return $tmp;
        }
        return undef;
    } ## end if ( WIFEXITED($status...

    if ( WIFSIGNALED($status) ) {
        $tmp = WTERMSIG($status);
        return $tmp;
    }

    return $status;
} ## end sub uStatus($)

#= Run a shell command and report its exit status or verify its output
#= :command        command to run
#= :[cond]        output success expr "=~ /good/" or "!~ /fail/"
#= >undef|error string
sub uSystem($;$) {
    my ( $cmd, $exp ) = @_;

    # check the command for attempts to recursively delete root (dies
    # if such an attempt is detected)
    recursive_rm_path_check($cmd);

    logTrc("Running command: $cmd");

    if ( !defined($exp) ) {
        my $pid = fork();
        croak("fork failed: $!") unless defined($pid);

        if ( $pid == 0 ) {    # child
            exec($cmd);

            #croak("exec($cmd): $!");
        }

        if ( ( my $p = waitpid( $pid, 0 ) ) != $pid ) {
            croak("waitpid found wrong child: $p  expect $pid");
        }

        return uStatus($?);
    } ## end if ( !defined($exp) )

    # $exp is defined from here on.
    # grab stout and stderr
    open( RES, "$cmd 2>&1 |" ) or croak("could not start $cmd: $!");
    my @res = <RES>;
    close(RES);
    logDbg("$cmd: \"@res\"");

    # check against expected output
    my $exc   = $@;    # preserve any current exception
    my $match = 0;
    if ( !@res ) {
        croak("bad result from \"$cmd\": \"$_\"");
    }
    for (@res) {
        if ( eval("q{$_} $exp") ) {
            $match = 1;
            last;
        }
    } ## end for (@res)
    croak("bad result from \"$cmd\": \"@res\"") if $match == 0;

    $@ = $exc;
    return undef;
} ## end sub uSystem($;$)

#= Generate date format based on epoch seconds
#= :time        epoch time eg time()+30
#= >date string
sub uDate($) {
    return strftime( "%a, %d %b %Y %H:%M:%S GMT", gmtime( $_[0] ) );
}

#= Diff 'df' values
#= :val1         hash ref 1
#= :val2        hash ref 2
#= :[pcdrift]  max acceptable percentage deviation among val1 and val2
sub uDiffDFValues($$;$) {
    my ( $val1, $val2, $pcdrift ) = @_;
    $pcdrift = 10 if ( !( defined($pcdrift) ) );

    for my $attr ( keys( %{$val1} ) ) {
        if ( $val1->{$attr} =~ /(\d+)%/ ) {
            $val1->{$attr} = $1;
        }
        if ( $val2->{$attr} =~ /(\d+)%/ ) {
            $val2->{$attr} = $1;
        }
    } ## end for my $attr ( keys( %{...

    logTrc( "comparing " . Dumper($val1) . " and " . Dumper($val2) );

    for my $attr ( keys( %{$val1} ) ) {
        if ( ( ( $val1->{$attr} - $val2->{$attr} ) / $val1->{$attr} ) * 100
            > $pcdrift )
        {
            croak(    "discrepancy in df values: $val1->{$attr} and"
                    . " $val2->{$attr} differ by more than"
                    . " $pcdrift%" );
        } ## end if ( ( ( $val1->{$attr...
    } ## end for my $attr ( keys( %{...

    return undef;
} ## end sub uDiffDFValues($$;$)

sub verify_say($$;$$$) {
    my ( $filer, $cmd, $must, $mustnot, $opts ) = @_;

    my $fh = host($filer);
    my $fc = Tharn::connect("#rsh");
    $fc->maxbufsize( 20 * 1024 * 1024 );
    logTrc(   "say to $filer: \"$cmd\""
            . ( $must    ? "\nmust match \"$must\""        : "" )
            . ( $mustnot ? "\nmust not match \"$mustnot\"" : "" ) );

    my ( undef, undef, $reply ) = say( $fc, $cmd );
    logDbg("reply was \"$reply\"");

    # check the reply for must-match or must-not-match
    if ( $must && $reply !~ /$must/ || $mustnot && $reply =~ /$mustnot/ ) {
        croak("unexpected result from \"$cmd\": \"$reply\"");
    }
    return $reply;
} ## end sub verify_say($$;$$$)

# Cleanup syslog file (/etc/messages = /etc/log/messages).
sub purge_log($;%) {
    my ( $filer, $opts ) = @_;

    my $fc     = make_conn($filer);
    my $pfiler = $fc->hostp()->{PFILER};
    if ($pfiler) {
        $filer = $pfiler;
    }
    my $logfile = $opts->{log} || "/etc/messages";
    wrfile( $fc, $logfile, "" );
    if ($@) {
        die("purge_log:  Error from 'wrfile': $@");
    }
} ## end sub purge_log($;%)

# Count number of lines in logfile
sub count_log_lines($;%) {
    my ( $filer, $opts ) = @_;

    my $fc     = make_conn($filer);
    my $pfiler = $fc->hostp()->{PFILER};
    if ($pfiler) {
        $filer = $pfiler;
    }
    my $logfile = $opts->{log} || "/etc/messages";
    my $text = rdfile( $fc, $logfile );
    if ($@) {
        die("count_log_lines:  Error from 'rdfile': $@");
    }
    my @linelist = split( /\n/, $text );
    my $cnt = scalar(@linelist);
    return ($cnt);
} ## end sub count_log_lines($;%)

# Read logfile starting at line xxx. Returns array of lines
sub read_log_from_line($$;%) {
    my ( $filer, $linenum, $opts ) = @_;

    my $fc     = make_conn($filer);
    my $pfiler = $fc->hostp()->{PFILER};
    if ($pfiler) {
        $filer = $pfiler;
    }
    my $logfile = $opts->{log} || "/etc/messages";
    my $text = rdfile( $fc, $logfile );
    if ($@) {
        die("read_log_from_line:  Error from 'rdfile': $@");
    }
    return ( split( /\n/, $text ) );
} ## end sub read_log_from_line($$;%)

sub verify_log($$;$) {
    my ( $filer, $pattern, $opts ) = @_;

    my $hostrec = host($filer);
    my $pfiler  = $hostrec->{PFILER};
    if ($pfiler) {
        $filer = $pfiler;
    }

    my $datestring = $opts->{date} || verify_say( $filer, "date" );
    my $logfile = $opts->{log} || "/etc/log/messages";
    my $timeout = defined( $opts->{timeout} ) ? $opts->{timeout} : CMDTIMEOUT;
    my $reftimeout
        = defined( $opts->{reftimeout} ) ? $opts->{reftimeout} : 40;
    my $endtime = time() + $timeout;

    my %months = (
        "Jan" => "0",
        "Feb" => "1",
        "Mar" => "2",
        "Apr" => "3",
        "May" => "4",
        "Jun" => "5",
        "Jul" => "6",
        "Aug" => "7",
        "Sep" => "8",
        "Oct" => "9",
        "Nov" => "10",
        "Dec" => "11",
    );

    my ( $day, $mth, $date, $time, $zone, $year )
        = split( /\s+/, $datestring );
    my ( $hour, $min, $sec ) = split( /:/, $time );

    # every logfile seems to have its own timestamp format
    my $timefmt = '^';
    if ( $logfile eq "/etc/log/snapmirror" ) {
        $timefmt .= '\w+\s+';
    }
    $timefmt
        .= $day . '\s+' 
        . $mth . '\s+' 
        . $date . '\s+'
        . '(\d+):(\d+):(\d+)\s+'
        . $zone . '.*';

    use Time::Local;
    my $reftime
        = timegm( $sec, $min, $hour, $date, $months{$mth}, $year - 1900 );

    # messages can be as old of (t - $reftimeout) sec
    $reftime -= $reftimeout;

    while ( time() < $endtime ) {

        my @string = split(
            /\n/,
            verify_say(
                $filer, "rdfile $logfile",
                undef,  "^$logfile: No such file "
            )
        );
        logTrc("searching $logfile for \"$pattern\"");
        foreach my $line (@string) {

            logInfo("log line: $line");

            my ( $loghr, $logmin, $logsec ) = ( $line =~ /$timefmt/ );

            if ( !defined($loghr) ) {
                logInfo("skip timestamp: $line");
                next;
            }
            my $logtime
                = timegm( $logsec, $logmin, $loghr, $date, $months{$mth},
                $year - 1900 );

            if ( $logtime > $reftime && ( $line =~ /$pattern/ ) ) {
                logTrc(   "$filer: pattern \'$pattern\'"
                        . " found in $logfile @ $line" );
                return 1;
            }
        } ## end foreach my $line (@string)
        sleep(5);
    } ## end while ( time() < $endtime)

    croak("$filer: pattern \'$pattern\' not found in $logfile");
} ## end sub verify_log($$;$)

# ------------------------------------------------------------------------
#= String utilities

#= Read file of name=value pairs and construct hash
#= :filename|undef        file of name-value pairs
#= :option-list|undef        list of name-value pairs
#= :name-list|undef        verify names
#= >hash of params
sub uParams($$$) {
    my ( $file, $opt, $names ) = @_;
    my %params;

    if ($file) {
        open( FILE, $file ) or croak("Bad params file $file\n");
        while (<FILE>) {
            next if (/^#|^$/);
            my ( $name, $value ) = /(\w+)=(.*)$/;
            defined($name) or croak("Bad param line \"$_\"\n");
            $params{$name} = $value;
        } ## end while (<FILE>)
        close(FILE);
    } ## end if ($file)
    if ($opt) {
        for ( split( /,/, $opt ) ) {
            my ( $name, $value ) = /(\w+)=(.*)$/;
            defined($name) or croak("Bad name-value \"$_\"\n");
            $params{$name} = $value;
        }
    } ## end if ($opt)
    if ($names) {
        my %valid;
        for (@$names) {
            $valid{$_} = 1;
        }
        for ( keys(%params) ) {
            defined( $valid{$_} )
                or croak("Unknown param $_ [@{$names}]\n");
        }
    } ## end if ($names)

    return \%params;
} ## end sub uParams($$$)

#= Read and parse headers from stream
#= :fh                input stream
#= :hdr|undef        validation hash {hdr=>value}
#= >hash of header-values
sub uReadHeaders($;$) {
    my ( $fh, $chkHdr ) = @_;
    logDbg( "read headers fd=" . fileno($fh) );
    my %hdrs;

    my $MSGHDR = '^([^:]+):' . '\s*' . '(.*)\r\n$';    # '
    while (<$fh>) {
        logTrc($_);
        last if (/^\r\n$/);

        next if ( $_ !~ /$MSGHDR/o );
        $hdrs{$1} = $2;
    } ## end while (<$fh>)
    uChkVals( \%hdrs, $chkHdr );
    return \%hdrs;
} ## end sub uReadHeaders($;$)

#= Verify hash
#= :data        input hash
#= :chk                validation hash
sub uChkVals($$) {
    my ( $data, $chk ) = @_;

    return unless ( defined($chk) );    # optional
    for ( keys( %{$chk} ) ) {
        logDbg("check $_ is $chk->{$_}");
        $data->{$_} eq $chk->{$_}
            or croak( "'$_' has bad value [$data->{$_}], "
                . "expecting [$chk->{$_}]\n" );
    } ## end for ( keys( %{$chk} ) )
} ## end sub uChkVals($$)

#= Highlight multiline string for display
#= :str
#= :[prefix]
sub uStrHi($;$) {
    my ( $msg, $pfx ) = @_;
    $pfx = "> " unless defined($pfx);

    chomp($msg);
    return $pfx . join( "\n$pfx", split( /\n/, $msg ) ) . "\n";
} ## end sub uStrHi($;$)

# ------------------------------------------------------------------------
#= Data movers
#= Copy byte streams to and fro.

my $BUFSIZ = 4096;

#= Copy all data until EOF
#= :fh-in        input stream
#= :fh-out        output stream
#= >bytes copied
sub copyUntilEOF($$) {
    my ( $r, $w ) = @_;
    my $rlen = 0;
    my $buf;
    while ( my $buflen = read( $r, $buf, $BUFSIZ ) ) {
        $rlen += $buflen;

        my $wrote = syswrite( $w, $buf );
        if ( !defined($wrote) || $wrote != $buflen ) {
            croak("bad write $wrote $! [$buflen]\n");
        }
    } ## end while ( my $buflen = read...
    return $rlen;
} ## end sub copyUntilEOF($$)

use Fcntl qw( F_GETFL F_SETFL O_NONBLOCK );

#= Set/ unset blocking mode on stream
#= :fh                stream
#= :flags|0        set flags or enable O_NONBLOCK
#= >old flags
sub blocking($$) {
    my ( $fh, $flags ) = @_;

    my $oflags = fcntl( $fh, F_GETFL, 0 )
        or croak("fcntl F_GETFL failed: $!\n");
    if ( !$flags ) {

        # no flags:- set non-blocking mode
        $flags = $oflags | O_NONBLOCK;
    }
    fcntl( $fh, F_SETFL, $flags ) or croak("fcntl failed: $!\n");
    return $oflags;
} ## end sub blocking($$)

#= Copy all outstanding data
#= :fh-in        input stream
#= :fh-out        output stream
#= >bytes copied
sub copyUntilEmpty($$) {
    my ( $r, $w ) = @_;

    my $rlen = 0;
    my $flags = blocking( $r, 0 );

    while (1) {
        my $buf;

        my $buflen = read( $r, $buf, $BUFSIZ );
        last if ( $buflen == 0 );
        $rlen += $buflen;

        my $wrote = syswrite( $w, $buf );
        if ( !defined($wrote) || $wrote != $buflen ) {
            croak("bad write $wrote $! [$buflen]\n");
        }

    } ## end while (1)
    blocking( $r, $flags );
    return $rlen;
} ## end sub copyUntilEmpty($$)

#= Copy data from one file descriptor to another
#= :fh-in        input stream
#= :len                "all", "pending" or byte count
#= :fh-out        output stream
#= >bytes copied
sub uCopy($$$) {
    my ( $r, $len, $w ) = @_;
    logDbg(
        "copy data: " . fileno($r) . " -> " . fileno($w) . " $len bytes" );

    return copyUntilEOF( $r, $w ) if ( $len eq "all" );
    return copyUntilEmpty( $r, $w ) if ( $len eq "pending" );
    croak("bad length to read $len\n") unless ( $len =~ /\d+/ );

    my $rlen = 0;
    while ( $len > 0 ) {
        my $buf;
        read( $r, $buf, ( $len > $BUFSIZ ) ? $BUFSIZ : $len )
            or croak("copy read failed: $rlen $!\n");
        $rlen += length($buf);

        my $wlen = syswrite( $w, $buf );
        ( defined($wlen) and $wlen == length($buf) )
            or croak("copy write failed: $wlen $!\n");

        $len -= length($buf);
    } ## end while ( $len > 0 )
    return $rlen;
} ## end sub uCopy($$$)

# ------------------------------------------------------------------------
#= Data sequences
#= A sequence is a printable ASCII byte pattern which can be validated
#= given the initial seed.  It is typically used in place of raw data.
#= We use 80 bytes of ASCII data.

my $seq = "0123456789abcdef"    # 16 * 5 == 80 bytes
    . "ghijklmnopqrstuv"
    . "wxyzABCDEFGHIJKL"
    . "MNOPQRSTUVWXYZ@#"
    . "&%+-*/=?<([{}])>";

# extract sequences from this buffer
my $seqBuf = $seq x ( $BUFSIZ / length($seq) + 1 );
my $seqBufLen = length($seqBuf);

#= Get sequence substring
#= :offset        sequence offset
#= :len                bytes
#= >sequence substring
sub uSeqStr($$) {
    my $seqoff = $_[0] % length($seq);
    my $str    = substr( $seqBuf, $seqoff, $_[1] );
    my $len    = $_[1] - length($str);
    while ( $len > $seqBufLen ) {
        $str .= $seqBuf;
        $len -= $seqBufLen;
    }
    $str .= substr( $seqBuf, 0, $len );
    return $str;
} ## end sub uSeqStr($$)

#= Write a sequence string to a socket
#= :offset        sequence offset
#= :len                byte count
#= :fh                output stream
#= >final sequence offset or error msg expected
sub uSendSeq($$$;$) {
    my ( $seqoff, $len, $w, $err ) = @_;

    logDbg(   "SendSeq: seqoff=$seqoff ["
            . ( $seqoff % length($seq) )
            . "] len=$len fd="
            . fileno($w) );
    while ($len) {
        my $towrite = ( $len > $BUFSIZ ) ? $BUFSIZ : $len;

        my $str = &uSeqStr( $seqoff, $towrite );
        logInfo("Seq write @ $seqoff: \"$str\"");

        my $wlen = syswrite( $w, &uSeqStr( $seqoff, $towrite ) );
        if ( !defined($wlen) ) {
            return $! if ( defined($err) );
            croak("seq write failed: $!\n");
        }

        $len -= $wlen;
        $seqoff += $wlen;
    } ## end while ($len)
    return $seqoff;
} ## end sub uSendSeq($$$;$)

#= Read and verify a sequence
#= :fh                input stream
#= :offset        sequence offset
#= :len                "all" or byte count
#= >final sequence offset
sub uReadSeq($$$) {
    my ( $r, $seqoff, $len ) = @_;

    $len = -1 if ( $len eq "all" );
    logDbg(   "ReadSeq: seqoff=$seqoff ["
            . ( $seqoff % length($seq) )
            . "] len=$len fd="
            . fileno($r) );
    while ($len) {
        my $buf;
        read( $r, $buf, ( $len > $BUFSIZ or $len == -1 ) ? $BUFSIZ : $len );
        my $rlen = length($buf);

        if ( $rlen == 0 ) {
            last if ( $len == -1 );
            croak("seq read failed: seqoff=$seqoff $!");
        }
        $len -= $rlen unless ( $len == -1 );
        logInfo("Seq read @ $seqoff: \"$buf\"");

        # check first byte
        my $rseq = substr( $buf, 0, 1 );
        my $xseq = &uSeqStr( $seqoff, 1 );
        ( $rseq eq $xseq )
            or croak("seq mismatch \"$rseq\" != \"$xseq\"");

        $seqoff += $rlen;
    } ## end while ($len)
    return $seqoff;
} ## end sub uReadSeq($$$)

# ------------------------------------------------------------------------
#= Networking Operations

#= Try to connect to a host and find the source ip-address of the connected
#= socket.
#= :target        name or ipaddr
#= :port        port
#= >stringified ipaddr
sub uGetSrcIP($$) {
    my ( $target, $tgtport ) = @_;

    my $socket = IO::Socket::INET->new("$target:$tgtport")
        or croak("couldn't connect to '$target:$tgtport': $!");

    my $saddr = getsockname($socket);
    close($socket);
    my ( $port, $ipaddr ) = sockaddr_in($saddr);
    return inet_ntoa($ipaddr);
} ## end sub uGetSrcIP($$)

#= Look up host name and extract FQDN (for netapp.com) from response
#= :name        hostname
#= >FQDM
sub uGetFQDN($) {
    my $host = $_[0];
    my ( $name, $aliases ) = gethostbyname($host);

    for ( ( $name, split( ' ', $aliases ) ) ) {
        return $_ if /\.netapp\.com/;
    }
    croak("cannot get FQDN for $host");
} ## end sub uGetFQDN($)

#= Socket Operations
#= closing read ends, write ends etc.

#= shutdown read end of a socket
sub uSoShutdownReader($) {
    my ($socket) = $_[0];
    logDbg( "shutdown read end fd=" . fileno($socket) );
    shutdown( $socket, 0 );
}

#= shutdown write end of a socket
sub uSoShutdownWriter($) {
    my ($socket) = $_[0];
    logDbg( "shutdown write end fd=" . fileno($socket) );
    shutdown( $socket, 1 );
}

#= enable/disable reservation for the specified file
#= :conn        filer connection
#= :file        file to set/get the reservation flag on
#= [args]        a hash of options - keys are "reservation" and "timeout"
#=                possible values of $args{reservation} are ON and OFF
#=                $args{timeout} is integer timeout for filer commands.
#= >ON|OFF
sub file_reserve {
    my ($conn) = make_conn(shift);
    my ($expr) = ON . "|" . OFF;
    my ( $file, %args ) = validate_mixed(
        @_,
        [ { type => SCALAR } ],
        {   reservation => {
                type     => SCALAR,
                optional => 1,
                default  => "",
                regex    => qr/^($expr|)$/
            },

            timeout => {
                type     => SCALAR,
                optional => 1,
                default  => 3600,
                regex    => qr/^\d+$/    # got to look like a number
            }
        }
    );

    my ($reservation) = $args{reservation};

    my ($cmd) = "file reservation $file "
        . (
        ( $reservation eq "" )
        ? ""
        : ( ( $reservation eq ON ) ? "enable" : "disable" )
        );

    logTrc("Running command: $cmd");

    my ($text) = sayEOF( $conn, $cmd, timeout => $args{timeout} );

    if ( $text =~ /space reservations for file $file: (on|off)/ ) {
        my ($result) = $1;
        return $result if ( $reservation eq "" );
        return $result if ( $reservation eq $result );
    }

    croak("Parse error in file_reserve(): $text");
} ## end sub file_reserve

#= enable/disable reservation for the specified file
#= :conn        filer connection
#= :lun         lun to set/get the reservation flag on
#= [args]        a hash of options - keys are "reservation" and "timeout"
#=                possible values of $args{reservation} are ON and OFF
#=                $args{timeout} is integer timeout for filer commands.
#= >ON|OFF
sub lun_reserve {
    my ($conn) = make_conn(shift);
    my ($expr) = ON . "|" . OFF;

    my ( $file, %args ) = validate_mixed(
        @_,
        [ { type => SCALAR } ],
        {   reservation => {
                type     => SCALAR,
                optional => 1,
                default  => "",
                regex    => qr/^($expr|)$/
            },

            timeout => {
                type     => SCALAR,
                optional => 1,
                default  => 3600,
                regex    => qr/^\d+$/    # got to look like a number
            }
        }
    );

    my ($reservation) = $args{reservation};

    my ($cmd) = "lun set reservation $file "
        . (
        ( $reservation eq "" )
        ? ""
        : ( ( $reservation eq ON ) ? "enable" : "disable" )
        );

    logTrc("Running command: $cmd");

    my ($text) = sayEOF( $conn, $cmd, timeout => $args{timeout} );

    if ($reservation) {

        # If setting reservation status, no reply is expected
        # from 'lun set reservation' command, but system messages may
        # be picked up depending on connection type. If the command
        # is not empty or contains something other than system messages
        # report an error.
        croak("Error in command reply: $text")
            if ( $text && $text !~ /(fill|overwrite) reservation/ );

        # Call command again without enable/disable to
        # retrieve the reservation status
        $cmd = "lun set reservation $file";
        $text = sayEOF( $conn, $cmd, timeout => $args{timeout} );
    } ## end if ($reservation)

    if ( $text
        =~ /space reservation for LUN $file \(inode \d+\): (enabled|disabled)/i
        )
    {
        my ($result) = ( $1 eq 'enabled' ) ? 'on' : 'off';
        return $result if ( $reservation eq "" );
        return $result if ( $reservation eq $result );
    } ## end if ( $text =~ ...

    croak("Parse error in lun_reserve(): $text");
} ## end sub lun_reserve

## @name ::pick_random_number
## @arg from - required. lower boundary
## @arg to - required. upper boundary
sub pick_random_number {
    my ( $from, $to ) = @_;
    croak("upper boundary is less than lower boundary")
        if ( $to <= $from );

    my $number;
    my $range = $to - $from;
    $number = ( int( rand($range) ) );
    $number = $from + $number;
    logDbg("pick_random_number returning $number");
    return $number;
} ## end sub pick_random_number

## @name ::get_firmware_env
## @description This routine will boot the hardware filer to the firmware
##              prompt and then query for the required configuration variables.
##
## @synopsis
##    use FS::Util::Utilities;
##    get_firmware_env($FILER, env_hash=>\%hashref,
##                     [timeout=>..., final_state=>..]);
##
## @arg hostspec Required. hostspec of filer,
##               eg. 'milford;hosttype=unix-solaris'
## @arg env_hash Required. Hash reference to the hash containing the names of
##               the configuration variables as keys to the hash.
## @arg timeout  Optional. Timeout value. Defaults: 1200 secs
## @arg final_state Optional. The final state of the hardware filer to be
##                  booted to. The acceptable values are those constants
##                  found in boot.pm,eg. FIRMWARE, BOOT_MENU, UP.
##                  Defaults to 'UP'.
##
## @notes If something happens while the function tries to get the
##        configuration variables, it will attempt to reboot the filer
##        back to the original state before croaking.
##
## @examples
##    <blockquote>
##    use FS::Util::Utilities;
##
##    # find the values for the following configuration variables
##    # if you put any values in this hash, they will be overwritten
##    # by whatever values taken from the firmware ok prompt.
##    my %get_env = (
##        'dont-dump-core?' => '',
##        'gdb-on-panic?'   => ''
##    );
##
##    # get the configuration variables' values and then reboot the filer
##    # back to the 'UP' state (as defined in boot.pm).
##    get_firmware_env($FILER, env_hash=>\%get_env,
##                     final_state=>'UP',
##                     timeout=>1200) or die $@;
##    </blockquote>
##
sub get_firmware_env {
    my $hostspec = shift;
    my $hostrec  = host($hostspec);
    my $conn     = connect('#console');

    my (%opts) = validate(
        @_,
        {   env_hash => { type => HASHREF, optional => 0 },
            timeout  => {
                type     => SCALAR,
                optional => 1,
                default  => 1200
            },
            final_state => {
                type     => SCALAR,
                optional => 1,
                default  => 'UP'
            },
        }
    );

    # NA to simulators
    if ( $hostrec->hosttype() eq 'filer-simulator' ) {
        return 1;
    }

    # get the current boot state. if something stupid happens here,
    # we'll reboot back to the original boot state so that they won't
    # be stuck in the firmware prompt.

    #if the filer is gb or newer, we dont need to halt it.
    # use 'bootargs dump' command instead
    my $bootstate = boot_state($hostrec) or croak $@;
    my $err;
    my $cmd;
    if (   ( $bootstate eq UP )
        && get_wafl_version($hostrec) >= WV_VB_MIN
        && filer::filer_kernel_is_debug($conn) )
    {
        $cmd = "priv set -q test; bootargs print";
    } else {
        $cmd = "printenv";

        # my $bootstate = boot_state($hostrec) or croak $@;
        my $err;

        # boot into firmware prompt
        eval {
            transit_to( $hostrec, 'FIRMWARE', timeout => $opts{'timeout'} )
                or die $@;
        };
        goto GET_ERROR if ( $err = $@ );
    } ## end else [ if ( ( $bootstate eq UP...

    foreach my $env ( keys %{ $opts{env_hash} } ) {
        my $q_env  = quotemeta($env);
        my $output = (
            say( $conn, "$cmd $env", '-prompt',
                $hostrec->{prompt_fw}, '-timeout', $opts{'timeout'}
            )
        )[2];

        # strip out the 'echo' associated with #console and the
        # line and carriage returns
        $output =~ s/printenv $q_env|\r\n//g;

        $output =~ /$q_env\s*=?\s*(.*$)/;
        $opts{env_hash}->{$env} = $1;

    } ## end foreach my $env ( keys %{ $opts...

    # boot back into whatever state user requested for
    eval {
        transit_to( $hostrec, $opts{'final_state'},
            timeout => $opts{'timeout'} )
            or die $@;
    };

    goto GET_ERROR if ( $err = $@ );
    return 1;

GET_ERROR:

    # if something happens here, there's nothing much we can do abt.
    transit_to( $hostrec, $bootstate, timeout => $opts{'timeout'} )
        or croak $@;

    # now we have booted back, croak with the reasons why we died
    croak $err;
} ## end sub get_firmware_env

## @name ::set_firmware_env
## @description This routine will boot the hardware filer to the firmware
##              prompt and then set the required configuration variables
##              with the desired values.
##
## @synopsis
##    use FS::Util::Utilities;
##    set_firmware_env($FILER, env_hash=>\%hashref,
##                     [timeout=>..., final_state=>..]);
##
## @arg hostspec Required. hostspec of filer,
##               eg. 'milford;hosttype=unix-solaris'
## @arg env_hash Required. Hash reference to the hash containing the names of
##               the configuration variables as keys to the hash and the
##               values of each configuration to be set.
## @arg timeout  Optional. Timeout value. Defaults: 1200 secs
## @arg final_state Optional. The final state of the hardware filer to be
##                  booted to. The acceptable values are those constants
##                  found in boot.pm,eg. FIRMWARE, BOOT_MENU, UP.
##                  Defaults to 'UP'.
##
## @notes Do *NOT* use this function to clear a configuration variable.
##        Use unset_firmware_env() to unset a configuration variable.
##
##        If something happens while the function tries to set the
##        configuration variables, it will attempt to reboot the filer
##        back to the original state before croaking.
##
##
## @examples
##    <blockquote>
##    use FS::Util::Utilities;
##
##    # find the values for the following configuration variables
##    my %set_env = (
##        'dont-dump-core?' => 'true',
##        'gdb-on-panic?'   => 'true'
##    );
##
##    # set the configuration variables' values and then reboot the filer
##    # back to the 'UP' state (as defined in boot.pm).
##    set_firmware_env($FILER, env_hash=>\%set_env,
##                     final_state=>'UP',
##                     timeout=>1200) or die $@;
##    </blockquote>
##
sub set_firmware_env {
    my $hostspec = shift;
    my $hostrec  = host($hostspec);
    my $conn     = connect('#console');

    my (%opts) = validate(
        @_,
        {   env_hash => { type => HASHREF, optional => 0 },
            timeout  => {
                type     => SCALAR,
                optional => 1,
                default  => 1200
            },
            final_state => {
                type     => SCALAR,
                optional => 1,
                default  => 'UP'
            },
        }
    );

    # NA to simulators
    if ( $hostrec->hosttype() eq 'filer-simulator' ) {
        return 1;
    }

    # get the current boot state. if something stupid happens here,
    # we'll reboot back to the original boot state so that they won't
    # be stuck in the firmware prompt.
    my $bootstate = boot_state($hostrec) or croak $@;
    my $err;

    my $cmd;
    my $get_cmd;
    my $unset_final_state;
    my @say_args;
    if (   ( $bootstate eq UP )
        && get_wafl_version($hostrec) >= WV_VB_MIN
        && filer::filer_kernel_is_debug($conn) )
    {
        @say_args          = ();
        $cmd               = "priv set -q test; bootargs set";
        $get_cmd           = "priv set -q test; bootargs print";
        $unset_final_state = 'UP';
    } else {

        $cmd               = "setenv";
        $get_cmd           = "printenv";
        $unset_final_state = 'FIRMWARE';
        @say_args          = (
            '-noprompt', '-prompt', $hostrec->{prompt_fw}, '-timeout',
            $opts{'timeout'}
        );

        # boot into firmware prompt
        eval {
            transit_to( $hostrec, 'FIRMWARE', timeout => $opts{'timeout'} )
                or die $@;
        };
        goto SET_ERROR if ( $err = $@ );
    } ## end else [ if ( ( $bootstate eq UP...
    my $tries = 0;
    foreach my $env ( keys %{ $opts{env_hash} } ) {
        my $q_env = quotemeta($env);

        # if we are trying to set the firmware variable to a null string or
        # "*** Undefined ***",
        # it really means that we want to 'unsetenv' the variable
        if (   ( $opts{env_hash}->{$env} eq '' )
            or ( $opts{env_hash}->{$env} =~ /Undefined/i ) )
        {
            unset_firmware_env(
                $hostspec,
                env_hash    => { $env => $opts{env_hash}->{$env} },
                final_state => $unset_final_state
            ) or die $@;
        } else {

            # try for 3 times. why do we do this? rebooting takes
            # forever. if it fails once, give it 2 more chance to
            # make amends.
            while ( $tries < 3 ) {
                say( $conn, "$cmd $env $opts{env_hash}->{$env}", @say_args );

                # verify that it was set
                my $output = ( say( $conn, "$get_cmd $env", @say_args ) )[2];

                # strip out the 'echo' associated with #console and the
                # line and carriage returns
                $output =~ s/$get_cmd $q_env|\r+\n//g;

                my $value;
                if ( $output =~ /$q_env\s*={0,}\s*(.*)$/ ) {
                    $value = $1;
                }

                if ( $value ne "$opts{'env_hash'}->{$env}" ) {

                    # didn't managed to set. try again for 2 more times
                    $tries++;
                } else {

                    # as long $tries is not 3, we're ok (see test below)
                    $tries = 99;
                }
            } ## end while ( $tries < 3 )

            if ( $tries == 3 ) {
                transit_to( $hostrec, $bootstate,
                    timeout => $opts{'timeout'} )
                    or croak $@;
                croak "Unable to set firmware config variable '$env' after "
                    . "3 tries";
            } else {
                $tries = 0;
            }
        } ## end else [ if ( ( $opts{env_hash}...
    } ## end foreach my $env ( keys %{ $opts...

    # boot back into whatever state user requested for
    eval {
        transit_to( $hostrec, $opts{'final_state'},
            timeout => $opts{'timeout'} )
            or die $@;
    };

    goto SET_ERROR if ( $err = $@ );
    return 1;

SET_ERROR:

    # if something happens here, there's nothing much we can do abt.
    transit_to( $hostrec, $bootstate, timeout => $opts{'timeout'} )
        or croak $@;

    # now we have booted back, croak with the reasons why we died
    croak $err;
} ## end sub set_firmware_env

## @name ::unset_firmware_env
## @description This routine will boot the hardware filer to the firmware
##              prompt and unset the required configuration variables.
##
## @synopsis
##    use FS::Util::Utilities;
##    unset_firmware_env($FILER, env_hash=>\%hashref,
##                       [timeout=>..., final_state=>..]);
##
## @arg hostspec Required. hostspec of filer,
##               eg. 'milford;hosttype=unix-solaris'
## @arg env_hash Required. Hash reference to the hash containing the names of
##               the configuration variables as keys to the hash.
## @arg timeout  Optional. Timeout value. Defaults: 1200 secs
## @arg final_state Optional. The final state of the hardware filer to be
##                  booted to. The acceptable values are those constants
##                  found in boot.pm,eg. FIRMWARE, BOOT_MENU, UP.
##                  Defaults to 'UP'.
##
## @notes If something happens while the function tries to unset the
##        configuration variables, it will attempt to reboot the filer
##        back to the original state before croaking.
##
## @examples
##    <blockquote>
##    use FS::Util::Utilities;
##
##    # find the values for the following configuration variables
##    my %unset_env = (
##        'dont-dump-core?' => 'true',
##        'gdb-on-panic?'   => 'true'
##    );
##
##    # unset the configuration variables' values and then reboot the filer
##    # back to the 'UP' state (as defined in boot.pm).
##    unset_firmware_env($FILER, env_hash=>\%unset_env,
##                       final_state=>'UP',
##                       timeout=>1200) or die $@;
##    </blockquote>
##
sub unset_firmware_env {
    my $hostspec = shift;
    my $hostrec  = host($hostspec);
    my $conn     = connect('#console');

    my (%opts) = validate(
        @_,
        {   env_hash => { type => HASHREF, optional => 0 },
            timeout  => {
                type     => SCALAR,
                optional => 1,
                default  => 1200
            },
            final_state => {
                type     => SCALAR,
                optional => 1,
                default  => 'UP'
            },
        }
    );

    # NA to simulators
    if ( $hostrec->hosttype() eq 'filer-simulator' ) {
        return 1;
    }

    # get the current boot state. if something stupid happens here,
    # we'll reboot back to the original boot state so that they won't
    # be stuck in the firmware prompt.
    my $bootstate = boot_state($hostrec) or croak $@;
    my $err;

    # boot into firmware prompt
    eval {
        transit_to( $hostrec, 'FIRMWARE', timeout => $opts{'timeout'} )
            or die $@;
    };
    goto UNSET_ERROR if ( $err = $@ );

    my $tries = 0;
    foreach my $env ( keys %{ $opts{env_hash} } ) {
        my $q_env = quotemeta($env);

        # try for 3 times. why do we do this? rebooting takes
        # forever. if it fails once, give it 2 more chance to
        # make amends.
        while ( $tries < 3 ) {
            say( $conn, "unsetenv $env",
                '-noprompt', '-prompt', $hostrec->{prompt_fw}, '-timeout',
                $opts{'timeout'} );

            # verify that it was unset
            my $output = (
                say( $conn, "printenv $env",
                    '-prompt',  $hostrec->{prompt_fw},
                    '-timeout', $opts{'timeout'}
                )
            )[2];

            # strip out the 'echo' associated with #console and the
            # line and carriage returns
            $output =~ s/printenv $q_env|\r\n//g;

            my $value;
            if ( $output =~ /$q_env\s*=?\s*(.*)$/ ) {
                $value = $1;
            }

            # when unset, value should be an empty string or *** Undefined ***
            if ( ( $value ne "" ) and ( $value !~ /Undefined/i ) ) {

                # didn't managed to set. try again for 2 more times
                $tries++;
            } else {

                # as long $tries is not 3, we're ok (see test below)
                $tries = 99;
            }
        } ## end while ( $tries < 3 )

        if ( $tries == 3 ) {

            # failed, bail out. first go back to original boot state
            transit_to( $hostrec, $bootstate, timeout => $opts{'timeout'} )
                or croak $@;
            croak "Unable to unset firmware config variable '$env' after "
                . "3 tries";
        } else {
            $tries = 0;
        }
    } ## end foreach my $env ( keys %{ $opts...

    # boot back into whatever state user requested for
    eval {
        transit_to( $hostrec, $opts{'final_state'},
            timeout => $opts{'timeout'} )
            or die $@;
    };

    goto UNSET_ERROR if ( $err = $@ );
    return 1;

UNSET_ERROR:

    # if something happens here, there's nothing much we can do abt.
    transit_to( $hostrec, $bootstate, timeout => $opts{'timeout'} )
        or croak $@;

    # now we have booted back, croak with the reasons why we died
    croak $err;
} ## end sub unset_firmware_env

## @name ::revert_to
## @description This routine aims to provide the revert functionality
##                for filers
##
## @synopsis
##    use FS::Util::Utilities qw(revert_to);
##    revert_to($FILER, $version,
##                       [timeout=>..., force=>'1|0']);
##
## @arg hostspec Required. hostspec of filer,
##               eg. 'milford;hosttype=filer'
## @arg        version  Required version to revert the filer to
## @arg timeout  Optional Timeout value. Defaults: 1200 secs
## @arg force    Optional forces a revert using the '-f' option to revert
##
## @arg notes         Note that this is really a start to address the reverting
##                 functionality addition into libraries
sub revert_to($$%) {
    my ( $hostspec, $version, %opts ) = validate_mixed(
        @_,
        [ { type => SCALAR }, { type => SCALAR } ],
        {   timeout => { type => SCALAR, optional => 1, default => 1200 },
            force   => { type => SCALAR, optional => 1, default => 0 }
        }
    );

    my $hostrec = host($hostspec);
    my $conn = connect( '#console', '-timeout', $opts{timeout} );

    my $cmd = "revert_to ";
    $cmd .= "-if "
        if ( $opts{force} );
    $cmd .= "$version";

    logTrc("attempting to revert to $version .. ");
    my ( $match, $before, $reply, $prompt )
        = say( $conn, $cmd, -timeout, $opts{timeout} );

    logTrc("response to cmd $cmd was $match $before \'$reply\' $prompt.. ");

    # Simple failure case that can be handled currently
    if (   $reply =~ /CIFS must be terminated|NFS must be turned off/
        && $opts{force} )
    {
        sayEOF( $conn, "cifs terminate;license delete nfs" )
            or croak($@);
        logTrc("cifs and nfs have been turned off, trying to revert..");
        revert_to(
            $hostspec, $version,
            force   => $opts{force},
            timeout => $opts{timeout}
        ) or croak($@);
    } elsif ( $reply
        =~ /You are about to revert the system to work with Data ONTAP $version/
        )
    {
        logTrc(   "reverting seems to be successful, assuming auto-boot "
                . "true, the filer should be able to boot back up "
                . "please check the status of your filer... " );
    } else {

        # XXX - Other cases like deleting existing snapshots probably need to
        #        be left to the user
        croak("can't revert.. reply was \'$reply\'");
    } ## end else [ if ( $reply =~ /CIFS must be terminated|NFS must be turned off/...

# XXX-TODO - what if the filer is not set for autoboot? need to
# handle those cases ...
#logTrc("attempt transition to UP state");
# we need to make sure that transit_to() does not use info from tharnhost file,
# so explicitly set bootimage and ontapdir to undef.
    transit_to(
        $conn, UP,
        timeout   => $opts{timeout},
        username  => 'root',
        netboot   => 0,
        bootimage => '',
        ontapdir  => undef
    ) or croak $@;
} ## end sub revert_to($$%)

## @name ::file_stat_hash
## @arg file - required: the file to stat
## @description This routine does a stat of the given file and returns a
## reference to a hash containing the names and values returned.
sub file_stat_hash {
    my ($path) = @_;
    croak("no such file: '$path'") if ( !-e $path );

    my @names
        = qw(dev ino mode nlink uid gid rdev size atime mtime ctime blksize blocks);
    my @values = stat($path);

    my %hash;
    for ( my $i = 0; $i < scalar(@names); $i++ ) {
        $values[$i] = scalar( localtime( $values[$i] ) )
            if ( $names[$i] =~ /time/ );
        $hash{ $names[$i] } = $values[$i];
    }

    return \%hash;
} ## end sub file_stat_hash

## @name ::acceptable_rm_path
## @arg path - required: the path to validate
## @description This routine returns false if the given path contains any bare
## slashes or repeated slases. It is meant to be used to validate a path before
## attempting to recursively remove it.
sub acceptable_rm_path {
    my ($path) = @_;

    my @statinfo = stat($path);

    # per Tap (tapl@netapp.com): the inode number of a root directory of a
    # filesystem is 2 for Linux's ext2/ext3, Solaris's UFS and IBM's JFS.'
    return ( ( ( $path ne '/' ) && ( $path !~ m%//+% ) )
            || $statinfo[1] > 3 );

} ## end sub acceptable_rm_path

## @name ::_parse_recursive_rm_target_paths
## @arg $str - required: the command string to parse
## @description This routine considers the given string as a unix shell command line
## and examines it for any embedded recursive rm commands. For each such command found,
## the target paths are collected and the overall list returned to the caller.
sub _parse_recursive_rm_target_paths {
    my ($str) = @_;

    croak script_result( __PACKAGE__
            . "::_parse_recursive_rm_target_paths: cannot check an empty string."
    ) if ( !$str );

    my @target_paths;

    # remove any embedded newlines (and any preceeding backslashes)
    $str =~ s/\\?\n//g;

    # split the line into separate commands, using three
    # command delimiters: '&&', '||', ';'
    my @commands = split( /\&\&|\|\||;/, $str );
    @commands = ($str) if ( !scalar(@commands) );

    # trim spaces from end of each command
    chomp(@commands);

    # trim spaces from beginning of each command
    @commands = grep( $_ =~ s/^\s*//, @commands );

    my $orig_value = $";
    $" = "\n\t";
    logDbg("commands found:\n\t@commands\n");
    $" = $orig_value;

    # this sub-pattern matches a space, a slash or the beginnning of a line.
    #   -> the matched text is NOT captured.
    my $rm_prefix = '(?:/|^|\s)';

 # this sub-pattern matches any group of command line args which are preceeded
 # by either one or two hyphens and which don't contain the letter 'r'.
 #   -> the matched text is NOT captured.
    my $arg_group_pattern = '(?:\s+-{1,2}[^r\s]+)*';

    foreach my $cmd (@commands) {

    # check whether the command includes 'rm -r'
    #   note: the regular expression below says
    #     - look for the $rm_prefix pattern (the matched text is NOT captured)
    #     - followed by the string 'rm'
    #     - optionally followed by an 'arg_group_pattern' as defined above
    #     - followed by an arg group pattern that DOES contain the
    #     letter 'r'
    #     - optionally followed by an 'arg_group_pattern' as defined above
    #     - followed by some whitespace
    #     - followed by a string of characters which is the LIST OF PATHS
    #     TARGETED BY THE EMBEDDED 'rm' COMMAND AND IS CAPTURED IN $1.
    #     - optionally followed by some whitespace
    #     - followed by the END-OF-LINE
        if ( $cmd
            =~ m%${rm_prefix}rm\s+${arg_group_pattern}-{1,2}\S*r\S*${arg_group_pattern}\s+(.+)\s*$%
            )
        {

            # get the paths targeted by the current recursive rm command
            my @paths = split( /\s+/, $1 );

           # remove outermost pair of enclosing quotes from each path (if any)
            my @trimmed;
            while ( my $path = pop(@paths) ) {
                my $tmp = $path;

                # remove enclosing single quotes?
                $tmp =~ s/^'(.*)'/$1/;

                # or, remove enclosing double quotes?
                $tmp =~ s/^"(.*)"/$1/ if ( length($tmp) == length($path) );

                push( @trimmed, $tmp );
            } ## end while ( my $path = pop(@paths...

            # add the target paths to our list
            push( @target_paths, @trimmed );
        } ## end if ( $cmd =~ ...
    } ## end foreach my $cmd (@commands)

    return @target_paths;
} ## end sub _parse_recursive_rm_target_paths

## @name ::deltree
## @arg $path - required: the path to be removed
## @arg conn - optional: a named parameter referencing a connection
## object through which the delete should be done.
## @arg timeout - optional: a named parameter specifying the timeout.
## this parameter is only meaningful when the 'conn' parameter is also
## given.
## @description This routine recursively removes the given path using
## system(). The target path is first verified to make sure that it
## doesn't map to the filesystem root (/). The function dies with a
## NATE script result if the path doesn't exist or is unacceptable.
## If the system() command is actually executed, then the exit value,
## signal_num and dumped_core values found in $? are returned. For
## more information about these values, see 'perldoc -f system'.
##
## Options:
##	cc:	'client connection'	Connection to NFS client
##	   If given, then this connection is used to execute the 'rm'
##	   command. Otherwise client is assumed to be 'localhost',
##	   and the 'rm' command command is executed here.
##
##	timeo:	'command timeout'	Timeout for 'rm' command.
##	    Default is 300 sec.
##
##	retain_dir:  'retain_directory'
##	    If '1', then all directories and files under the given
##	    directory are deleted, but the directory itself is
##	    retained. Otherwise	the directory is deleted also --
##	    (equiv. to 'rm -rf $DIR/*'). Default is '0', so directory
##	    is deleted.
##
sub deltree {
    my ( $path, %opts ) = @_;

    my $cc         = $opts{conn};
    my $timeo      = $opts{timeout} || 300;
    my $retain_dir = $opts{retain_dir};

    if ( !defined($cc) && ( !-e $path ) ) {
        croak script_result(
            __PACKAGE__ . "::deltree: no such path: '$path'" );
    }

    if ($retain_dir) {
        if ( $path =~ /[^\/\*]$/ ) {
            $path .= '/*';
        } elsif ( $path =~ /\/$/ ) {
            $path .= '*';
        }
    } ## end if ($retain_dir)

    my $cmd = "rm -rf \"$path\"";

    # this call dies if the path maps to the root directory
    recursive_rm_path_check($cmd);

    if ( !defined($cc) ) {
        my $failed = system("$sudo_cmd $cmd");
        return ( !$failed );
    } else {
        my $match = ( say( $cc, $cmd, '-timeout', $timeo ) )[2];

        # ignore 'stty:' messages as not errors
        $match =~ s/^stty: standard input: .*$//m;
        chomp($match);
        croak("deltree: client rm command failed: $match")
            if ( $match =~ /error|permission|denied/ );
        return 1;
    } ## end else [ if ( !defined($cc) )
} ## end sub deltree

## @name ::delfile
## @arg $path - required: the path to be removed
## @arg conn - optional: a named parameter referencing a connection
## object through which the delete should be done.
## @arg timeout - optional: a named parameter specifying the timeout.
## this parameter is only meaningful when the 'conn' parameter is also
## given.
## @description This routine removes the file specified by the given path.
sub delfile {
    my ( $path, %opts ) = @_;

    my $cc = $opts{conn};
    my $timeo = $opts{timeout} || 300;

    if ( !defined($cc) && ( !-e $path ) ) {
        croak script_result(
            __PACKAGE__ . "::delfile: no such path: '$path'" );
    }

    my $cmd = "$sudo_cmd rm -f \"$path\"";

    if ( !defined($cc) ) {
        my $failed = system($cmd);
        return ( !$failed );
    } else {
        my $match = ( say( $cc, $cmd, '-timeout', $timeo ) )[2];
        croak("delfile: client rm command failed: $match") if ($match);
        return 1;
    }
} ## end sub delfile

## @name ::recursive_rm_path_check
## $cmd - required: a unix command line string which may contain a recursive
## rm command.
## @description This routine checks the given string for instances of recursive
## rm's. Each path targeted by such a command is checked using acceptable_rm_path()
## to see if it is acceptable or not. When an unacceptable path is found, the
## function dies with a script result.
sub recursive_rm_path_check($) {
    my ($cmd) = @_;

    # get the list of paths targeted by any embedded recursive rm commands
    my @recursive_rm_target_paths = _parse_recursive_rm_target_paths($cmd);

    # log a trace message
    my $orig_value = $";
    $" = "\n\t";
    logTrc("recursive rm paths found:\n\t@recursive_rm_target_paths\n")
        if (@recursive_rm_target_paths);
    $" = $orig_value;

    # check the recursive rm paths to (hopefully) prevent attempts
    # to remove the root
    foreach my $path (@recursive_rm_target_paths) {
        if ( acceptable_rm_path($path) ) {
            logTrc("acceptable rm path: '$path'");
        } else {

            # build a stack backtrace
            my $backtrace;
            for ( my $i = 1; 1; $i++ ) {
                my ( $package, $file, $line ) = caller $i;

                # bail when we get to the bottom of the stack
                last if ( !defined($package) );

                $backtrace .= ${file} . " line $line\n";
            } ## end for ( my $i = 1; 1; $i++)

            # get the current hostname
            my $host = hostname();

            # build our message
            my $msg
                = "\nA script you were running on host '$host' passed an unacceptable path ($path) for a recursive remove command. Refer to the stacktrace below.\n\n\n$backtrace\n";
            my $subject
                = 'WARNING: A SCRIPT ATTEMPTED TO RECURSIVELY DELETE ROOT';
            my $tester = "$ENV{LOGNAME}\@netapp.com";

            # send mail
            my %mail = (
                To      => $tester,
                Cc      => join( ',', @{$RM_NOTIFY_LIST} ),
                From    => $tester,
                Subject => $subject,
                Message => "$msg"
            );

            logDbg(
                "sending notification email:\n" . Dumper( \%mail ) . "\n" );

            sendmail(%mail) or warn $Mail::Sendmail::error;

            croak script_result( __PACKAGE__
                    . "::recursive_rm_path_check: can't continue - unacceptable recursive rm path: '$path'"
            );
        } ## end else [ if ( acceptable_rm_path...
    } ## end foreach my $path (@recursive_rm_target_paths)
} ## end sub recursive_rm_path_check($)

## @name ::util_unlink
## @arg $srcpath - required: the target path to the file
## @arg conn - optional: a named parameter referencing a connection
##  object through which the delete should be done.
## @arg timeout - optional: a named parameter specifying the timeout.
##  This parameter is only meaningful when the 'conn' parameter is also
##  given.
## @description create a link between the two files on the given client.
## returns 1 on success, 0 and $@ on failure.

sub util_unlink {
    my ( $srcpath, %opts ) = @_;

    my $cc = $opts{conn};
    my $timeo = $opts{timeout} || 300;

    my ( $bs, $count ) = ( 1, 0 );

    my $ret;
    if ( !defined($cc) ) {
        $ret = uSystem("unlink $srcpath");
        return $@ ? 0 : 1;
    } else {    # use client connection
        my ( $match, undef, $after ) = say( $cc, "unlink $srcpath" );
        if ( $after =~ /permission|denied|error|fail|cannot/i ) {
            $@ = $after;
            return 0;
        }
    } ## end else [ if ( !defined($cc) )
    return 1;
} ## end sub util_unlink

## @name ::util_rmdir
## @arg $path - required: the directory path to be created
## @arg conn - optional: a named parameter referencing a connection
## object through which the delete should be done.
## @arg timeout - optional: a named parameter specifying the timeout.
## this parameter is only meaningful when the 'conn' parameter is also
## given.
## @description 'rmdir -p' on the given path on the given client.
## returns 1 on success, 0 and $@ on failure.

sub util_rmdir {
    my ( $path, %opts ) = @_;

    my $cc = $opts{conn};
    my $timeo = $opts{timeout} || 300;

    if ( !defined($cc) ) {
        return !uSystem("$sudo_cmd rmdir -p $path");
    } else {
        my ( $match, undef, $after ) = say( $cc, "$sudo_cmd rmdir -p $path" );
        if ( $after =~ /permission|denied|error|fail|cannot/i ) {
            $@ = $after;
            return 0;
        } else {
            return 1;
        }
    } ## end else [ if ( !defined($cc) )
} ## end sub util_rmdir

## @name ::util_mkdir
## @arg $path - required: the directory path to be created
## @arg conn - optional: a named parameter referencing a connection
## object through which the delete should be done.
## @arg timeout - optional: a named parameter specifying the timeout.
## this parameter is only meaningful when the 'conn' parameter is also
## given.
## @description 'mkdir -p' on the given path on the given client.
## returns 1 on success, 0 and $@ on failure.

sub util_mkdir {
    my ( $path, %opts ) = @_;

    my $cc = $opts{conn};
    my $timeo = $opts{timeout} || 300;

    if ( !defined($cc) ) {
        return !uSystem("mkdir -p $path");
    } else {
        my ( $match, undef, $after ) = say( $cc, "mkdir -p $path" );
        if ( $after =~ /permission|denied|error|fail|cannot/i ) {
            $@ = $after;
            return 0;
        } else {
            return 1;
        }
    } ## end else [ if ( !defined($cc) )
} ## end sub util_mkdir

## @name ::util_rename
## @arg $path1 - required: the source path
## @arg $path2 - required: the destination path
## @arg conn - optional: a named parameter referencing a connection
## object through which the delete should be done.
## @arg timeout - optional: a named parameter specifying the timeout.
## this parameter is only meaningful when the 'conn' parameter is also
## given.
## @description 'mv $path1 $path2' on the given path on the given client.
## returns 1 on success, 0 and $@ on failure.
## @notes this function only correctly works with plain files and
## directories.

sub util_rename {
    my ( $path1, $path2, %opts ) = @_;

    my $cc = $opts{conn};
    my $timeo = $opts{timeout} || 300;

    if ( !defined($cc) ) {
        return rename( $path1, $path2 );
    } else {
        my ( undef, undef, $ret )
            = say( $cc,
            "test -f '$path2' -o -d '$path2' && echo target exists" );
        if ( $ret =~ /target exists/ ) {
            logWarn("rename target '$path2' exists before rename");
        }
        my ( $match, undef, $after )
            = say( $cc, "mv $path1 $path2", -timeout, $timeo );
        ( undef, undef, $ret )
            = say( $cc, "test -f '$path1' -o -d '$path1' && echo not moved" );
        if ( $ret =~ /not moved/ ) {
            $@ = $after;
            return 0;
        } else {
            return 1;
        }
    } ## end else [ if ( !defined($cc) )
} ## end sub util_rename

## @name ::util_chmod
## @arg $mode - required: the mode bits to change to
## @arg $path - required: the target path
## @arg conn - optional: a named parameter referencing a connection
## object through which the delete should be done.
## @arg timeout - optional: a named parameter specifying the timeout.
## this parameter is only meaningful when the 'conn' parameter is also
## given.
## @description 'chmod' on the given path on the given client.
## returns 1 on success, 0 and $@ on failure.

sub util_chmod {
    my ( $mode, $path, %opts ) = @_;

    my $cc = $opts{conn};
    my $timeo = $opts{timeout} || 300;

    if ( !defined($cc) ) {
        logDbg("changing to mode $mode for file $path");
        return chmod( $mode, $path );
    } else {
        $mode = sprintf( "0%o", $mode );
        logDbg("changing to mode $mode for file $path");
        my ( $match, undef, $after ) = say( $cc, "chmod $mode $path" );
        if ( $after =~ /permission|denied|error|fail|cannot|can't change/i ) {
            $@ = $after;
            return 0;
        } else {
            return 1;
        }
    } ## end else [ if ( !defined($cc) )
} ## end sub util_chmod

## @name ::util_mkfile
## @arg $path - required: the target path to create file
## @arg size - optional: a named parameter giving the desired size.
##  created contents are all zeros (or random). exclusive with content
## @arg content - optional: a named parameter with the initial contents
##  of the file. Exclusive with size, and overrides.
## @arg mode - optional: a named parameter with the initial mode bits.
## @arg conn - optional: a named parameter referencing a client connection
##  object through which the create/write file should be done.
## @arg timeout - optional: a named parameter specifying the timeout.
##  This parameter is only meaningful when the 'conn' parameter is also
##  given.
## @arg random - optional: if true, copy $size bytes from /dev/urandom.
##  Will fail if /dev/urandom doesn't exist.
## @description create a file on the given path on the given client.
##  If neither size nor content is supplied, default is that the initial
##  content is the path.
## returns 1 on success, 0 and $@ on failure.

sub util_mkfile {
    my ( $path, %opts ) = @_;

    my $cc      = $opts{conn};
    my $timeo   = $opts{timeout} || 300;
    my $size    = $opts{size};
    my $mode    = $opts{mode};
    my $content = $opts{content} || ( defined($size) ? undef : $path );
    my $random  = $opts{random} ? "/dev/urandom" : "/dev/zero";

    my ( $bs, $count ) = ( 1, 0 );
    if ( defined($size) ) {
        if ( $size == 0 ) {
            $count = 0;
            $bs    = 1;
        } elsif ( $size % 4096 == 0 ) {
            $bs    = 4096;
            $count = $size / 4096;
        } elsif ( $size % 512 == 0 ) {
            $bs    = 512;
            $count = $size / 512;
        } else {
            $count = 1;
            $bs    = $size;
        }
    } ## end if ( defined($size) )
    my $ret;
    if ( !defined($cc) ) {
        if ( $content or $size == 0 ) {
            open( OUT, ">$path" ) or $@ = $!, return 0;
            print OUT $content if $content;
            close(OUT);
            if ($mode) {
                return chmod( $mode, $path );
            }
        } else {
            $ret = uSystem("dd if=$random of=$path count=$count bs=$bs");
            $@ = $ret unless $ret =~ /records out/;
        }
        return $@ ? 0 : 1;
    } else {    # use client connection
        if ($content) {
            my ( $match, undef, $after )
                = say( $cc, "echo '$content' > $path" );
            if ( $after =~ /permission|denied|error|fail|cannot/i ) {
                $@ = $after;
                return 0;
            }
        } elsif ( $size == 0 ) {
            my ( $match, undef, $after ) = say( $cc, "touch $path" );
            if ( $after =~ /permission|denied|error|fail|cannot/i ) {
                $@ = $after;
                return 0;
            }
        } else {
            my ( $match, undef, $after )
                = say( $cc, "dd if=$random of=$path count=$count bs=$bs" );
            if ( $after =~ /permission|denied|error|fail|cannot/i ) {
                $@ = $after;
                return 0;
            }
        } ## end else [ if ($content)
        if ($mode) {
            return util_chmod( $mode, $path, conn => $cc );
        }
    } ## end else [ if ( !defined($cc) )
    return 1;
} ## end sub util_mkfile

## @name ::util_link
## @arg $srcpath - required: the target path to create file
## @arg $refpath - required: a named parameter giving the desired size.
##  created contents are all zeros (or random). exclusive with content
## @arg $type - Option: Describes the type of link "s" for symbolic or "h" for hard
## @arg conn - optional: a named parameter referencing a connection
##  object through which the delete should be done.
## @arg timeout - optional: a named parameter specifying the timeout.
##  This parameter is only meaningful when the 'conn' parameter is also
##  given.
## @description create a link between the two files on the given client.
## returns 1 on success, 0 and $@ on failure.

sub util_link {
    my ( $srcpath, $refpath, %opts ) = @_;

    my $type  = $opts{type} || "h";
    my $cc    = $opts{conn};
    my $timeo = $opts{timeout} || 300;

    my ( $bs, $count ) = ( 1, 0 );

    # if user specified "h" for a hardlink then just blank it out
    #  if they specified an "s" for sym link then leave it alone
    $type =~ s/h//i;
    my $ret;
    if ( !defined($cc) ) {
        $ret = uSystem("ln $type $srcpath $refpath");
        return $@ ? 0 : 1;
    } else {    # use client connection
        my ( $match, undef, $after ) = say( $cc, "ln $srcpath $refpath" );
        if ( $after =~ /permission|denied|error|fail|cannot/i ) {
            $@ = $after;
            return 0;
        }
    } ## end else [ if ( !defined($cc) )
    return 1;
} ## end sub util_link

## @name ::util_filesize
## @arg $path - required: the target path to create file
## @arg conn - optional: a named parameter referencing a connection
##  object through which the delete should be done.
## @description return the size of the file at path on (optional) client.
## returns size on success, die on failure.

sub util_filesize {
    my ( $path, %opts ) = @_;

    my $size;
    my $cc = $opts{conn};
    if ($cc) {
        ( undef, undef, $size )
            = say( $cc, "$perlcmd_path -le 'print 0+(stat(qq($path)))[7];'" );
    } else {
        $size = ( stat($path) )[7];
    }
    chomp($size);
    return $size;
} ## end sub util_filesize

## @name ::util_stat
## @arg $path - required: the target path to create file
## @arg conn - optional: a named parameter referencing a connection
##  object through which the delete should be done.
## @description return the stat structure of the file at path on
## (optional) client as an array.
## returns stat array on success, die on failure.
## For non-existent file, an empty array is returned.
## Array structure:
## ($dev, $ino. $mode, $nlink, $uid, $gid, $rdev, $size, $atime $mtime, $ctime,
##   $blksize, $blocks)

sub util_stat {
    my ( $path, %opts ) = @_;
    my @ret_arr;
    my @stat_arr;
    my $cc = $opts{conn};
    if ($cc) {
        my $str;
        ( undef, undef, $str ) = say( $cc,
            "$perlcmd_path -le '\@s=stat(qq($path));print join(qq( ),\@s);'"
        );

        # Check to see if there are no rsh connection messages.
        # If there are, ignore them to obtain the actual file stats
        @stat_arr = split(/\n/, $str);
        foreach my $each_stat_arr(@stat_arr){
            $each_stat_arr =~ s/\r|\n//g;
            if($each_stat_arr =~ /^\d+(\s\d+)+$/){
                $str = $each_stat_arr;
                last;
            }
        }    
        @ret_arr = split( /\s+/, $str );
    } else {
        @ret_arr = stat($path);
    }
    chomp(@ret_arr);
    return @ret_arr;
} ## end sub util_stat

## @name ::util_exists
## @arg $path - required: the target path to create file
## @arg conn - optional: a named parameter referencing a connection
##  object through which the delete should be done.
## @description Returns "1" if the given path points to an existing file/dir/link/etc.
## (optional) client as an array.
## returns stat "1" or "0" on success, die on failure.

sub util_exists {
    my ( $path, %opts ) = @_;
    my @arr = util_stat( $path, %opts );
    my $exists = ( ( scalar(@arr) > 0 ) ? 1 : 0 );
    logDbg(   "util_exists  Line: " 
            . __LINE__
            . "   Path: \"$path\"   Conn: \"$opts{conn}\"   Returning: \"$exists\""
    );
    return ($exists);
} ## end sub util_exists

## @name ::disk_zero_spares
## @arg $conn - required: a connection object to filer
## @description Starts a disk zeroing process; does NOT wait for completion

sub disk_zero_spares {
    my ($conn) = @_;

    my ( undef, undef, $ret ) = say( $conn, "disk zero spares" );
    return;
} ## end sub disk_zero_spares

## @name ::nfs_cifs_mkfile
## @arg $path 	     - required: path of the file to be created
## @arg $protocol    - optional: nfs|cifs. protocol to be used for the mkfile.
##			default - nfs
## @arg size 	     - optional: a named parameter giving the desired size.
##  			created contents are all zeros (or random). exclusive
##			with content
## @arg content	     - optional: a named parameter with the initial contents
##  			of the file. Exclusive with size, and overrides.
## @arg mode         - optional: a named parameter with the initial mode bits.
##		       If cifs is being used, the mode bits - +R | -R
##		       (R - readonly)
## @arg client       - optional: a named parameter referencing a nfs connection
##		       object or cifs client through which the mkfile should be
##		       done. The variable is treated as a nfs/cifs depending
##		       on the value passed in for protocol.
## @arg timeout      - optional: a named parameter specifying the timeout.
##  		       This parameter is only meaningful when the 'conn'
##		       parameter is also given.
## @arg random       - optional: if true, copy $size bytes from /dev/urandom.
## 		       Will fail if /dev/urandom doesn't exist.
## @description      create a file on the given path on the given client.
##  		     If neither size nor content is supplied, default is that
##		     the initial content is the path.
## returns 1 on success, 0 and $@ on failure.
sub nfs_cifs_mkfile {
    my ( $path, %opts ) = @_;
    my $protocol = $opts{protocol};

    if ( $protocol and $protocol eq 'cifs' ) {
        my $cifs_client = $opts{client};
        logTrc("Creating the file $path using CIFS");
        if ( !defined($cifs_client) ) {
            croak("cifs_client needed for cifs mkfile");
        }
        my $mode = $opts{mode};
        my $content = $opts{content} || $path;

        my $output
            = execute_cmd_special( $cifs_client, "echo '$content'> $path" );
        if ($output) {
            $@ = $output;
            return 0;
        }

        if ( $mode and $mode eq "+R" ) {
            return nfs_cifs_rdonly(
                $path,
                protocol    => "cifs",
                cifs_client => $cifs_client
            );
        } ## end if ( $mode and $mode eq...
        return 1;
    } else {
        logTrc("Creating the file $path using NFS");
        my $nfs_client = $opts{client};
        return util_mkfile( $path, conn => $nfs_client, %opts );
    }
} ## end sub nfs_cifs_mkfile

## @name ::nfs_cifs_copy
## @arg $from_path   - required: path of the source file
## @arg $to_path     - required: path of the destination file
## @arg $protocol    - optional: nfs|cifs. a named parameter referencing the
##		       protocol to be used for the copy. default - nfs
## @arg client       - optional: a named parameter referencing a nfs connection
##		       object or cifs client through which the copy should be
##		       done. The variable is treated as a nfs/cifs depending
##		       on the value passed in for protocol.
## @description      copy a file using either nfs or cifs.
sub nfs_cifs_copy {
    my ( $from_path, $to_path, %opts ) = @_;
    my $protocol = $opts{protocol};

    if ( $protocol and $protocol eq 'cifs' ) {
        my $cifs_client = $opts{client};
        if ( !defined($cifs_client) ) {
            croak("cifs_client needed for cifs copy");
        }
        logTrc("Copying the file using CIFS");
        execute_cmd( $cifs_client, "copy $from_path $to_path" );
    } else {
        logTrc("Copying the file using NFS");
        my $nfs_client = $opts{client};
        if ( !defined($nfs_client) ) {
            uSystem("cp $from_path $to_path");
        } else {
            say( $nfs_client, "cp $from_path $to_path" );
        }
    } ## end else [ if ( $protocol and $protocol...
} ## end sub nfs_cifs_copy

## @name ::nfs_cifs_rename
## @arg $from_file   - required: name of the source file
## @arg $to_file     - required: name of the destination file
## @arg $path	     - required: path for the source and destination file
## @arg $protocol    - optional: nfs|cifs. a named parameter referencing the
##		       protocol to be used for the copy. default - nfs
## @arg client       - optional: a named parameter referencing a nfs connection
##		       object or cifs client through which the copy should be
##		       done. The variable is treated as a nfs/cifs client
##		       depending on the value passed in for protocol.
## @description      rename a file using either nfs or cifs.
##			returns 1 on success. 0 on failure.
sub nfs_cifs_rename {
    my ( $from_file, $to_file, $path, %opts ) = @_;
    my $protocol = $opts{protocol};

    if ( $protocol and $protocol eq 'cifs' ) {
        my $cifs_client = $opts{client};
        if ( !defined($cifs_client) ) {
            croak("cifs_client needed for cifs copy");
        }
        logTrc("Renaming the file using CIFS");
        my $output
            = execute_cmd( $cifs_client, "rename $path$from_file $to_file" );
        if ($output) {
            $@ = $output;
            return 0;
        } else {
            return 1;
        }
    } else {
        logTrc("Renaming the file using NFS");
        my $nfs_client = $opts{client};
        return util_rename( "$path$from_file", "$path$to_file",
            conn => $nfs_client );
    } ## end else [ if ( $protocol and $protocol...
} ## end sub nfs_cifs_rename

## @name ::nfs_cifs_exists
## @arg $path        - required: path of the file or directory to be checked
## @arg $protocol    - optional: nfs|cifs. a named parameter referencing the
##		       protocol to be used for the attribute change.
##		       default - nfs
## @arg client       - optional: a named parameter referencing a nfs connection
##		       object or cifs client through which the access
##		       done. The variable is treated as a nfs/cifs depending
##		       on the value passed in for protocol.
# @description      Check for existance of a file or directory
##		     returns 1 on success 0 on failure
sub nfs_cifs_exists {
    my ( $path, %opts ) = @_;
    my $protocol = $opts{protocol};
    logDbg(   "Enter nfs_cifs_exists  Line: " 
            . __LINE__
            . "\n\nPath: \"$path\"  Protocol: \"$protocol\"   Client: \"$opts{client}\""
    );
    if ( $protocol and $protocol eq 'cifs' ) {
        my $cifs_client = $opts{client};
        if ( !defined($cifs_client) ) {
            croak("cifs_client needed for cifs operation");
        }
        logDbg(   "nfs_cifs_exists   Line: " 
                . __LINE__
                . "  Check file \"$path\" using CIFS" );
        my $output = execute_cmd( $cifs_client, "attrib $path" );
        if ( !$output || $output =~ /not found/i ) {
            return 0;
        } else {
            return 1;
        }
    } else {
        logDbg(   "nfs_cifs_exists   Line: " 
                . __LINE__
                . "  Check file \"$path\" using NFS -- calling 'util_exists'"
        );
        my $nfs_client = $opts{client};
        return ( util_exists( $path, conn => $nfs_client ) );
    } ## end else [ if ( $protocol and $protocol...
} ## end sub nfs_cifs_exists

## @name ::nfs_cifs_rdonly
## @arg $path        - required: path of the file to be made read-only
## @arg $protocol    - optional: nfs|cifs. a named parameter referencing the
##		       protocol to be used for the attribute change.
##		       default - nfs
## @arg client       - optional: a named parameter referencing a nfs connection
##		       object or cifs client through which the chmod should be
##		       done. The variable is treated as a nfs/cifs depending
##		       on the value passed in for protocol.
# @description      make the file readonly.
##		     returns 1 on success 0 on failure
sub nfs_cifs_rdonly {
    my ( $path, %opts ) = @_;
    my $protocol = $opts{protocol};

    if ( $protocol and $protocol eq 'cifs' ) {
        my $cifs_client = $opts{client};
        if ( !defined($cifs_client) ) {
            croak("cifs_client needed for cifs operation");
        }
        logTrc("Making the file read-only using CIFS");
        my $output = execute_cmd( $cifs_client, "attrib +R $path" );
        if ($output) {
            $@ = $output;
            return 0;
        } else {
            return 1;
        }
    } else {
        logTrc("chmod file readonly using NFS");
        my $nfs_client = $opts{client};
        return util_chmod( 0444, $path, conn => $nfs_client );
    }
} ## end sub nfs_cifs_rdonly

## @name ::nfs_cifs_rd_write
## @arg $path        - required: path of the file to be made read-write
## @arg $protocol    - optional: nfs|cifs. a named parameter referencing the
##		       protocol to be used for the attribute change.
##		       default - nfs
## @arg client       - optional: a named parameter referencing a nfs connection
##		       object or cifs client through which the copy should be
##		       done. The variable is treated as a nfs/cifs depending
##		       on the value passed in for protocol.
## @description      make the file writable.
##		     returns 1 on success 0 on failure
sub nfs_cifs_rd_write {
    my ( $path, %opts ) = @_;
    my $protocol = $opts{protocol};

    if ( $protocol and $protocol eq 'cifs' ) {
        my $cifs_client = $opts{client};
        if ( !defined($cifs_client) ) {
            croak("cifs_client needed for cifs operation");
        }
        logTrc("Making the file read-write using CIFS");
        my $output = execute_cmd( $cifs_client, "attrib -R $path" );
        if ($output) {
            $@ = $output;
            return 0;
        } else {
            return 1;
        }
    } else {
        logTrc("chmod file read-write using NFS");
        my $nfs_client = $opts{client};
        return util_chmod( 0666, $path, conn => $nfs_client );
    }
} ## end sub nfs_cifs_rd_write

## @name ::nfs_cifs_append
## @arg $to_file     - required: name of the destination file
## @arg $append_file - optional: for nfs only. Used if you want to append the
##		       contents of one file to another. If append_file is
##		       passed, "cat append_file >> to_file" is done.
## @arg $content     - optional: Used if we want to append some content
##		       to a file. Can be used in cifs or nfs.
##		       Command used to append is echo content >> to_file
## @arg $protocol    - optional: nfs|cifs. a named parameter referencing the
##		       protocol to be used for the copy. default - nfs
## @arg client       - optional: a named parameter referencing a nfs connection
##		       object or cifs client through which the copy should be
##		       done. The variable is treated as a nfs/cifs client
##		       depending on the value passed in for protocol.
## @description      append to a file using either nfs or cifs.
##			returns 1 on success. 0 on failure.
sub nfs_cifs_append {
    my ( $to_path, %opts ) = @_;
    my $protocol = $opts{protocol};
    my $content  = $opts{content};

    if ( $protocol and $protocol eq 'cifs' ) {
        my $cifs_client = $opts{client};
        if ( !defined($cifs_client) ) {
            croak("cifs_client needed for cifs copy");
        }
        if ( !defined($content) ) {
            croak("No content passed to append !! Check caller");
        }
        logTrc("Appending to  the file using CIFS");
        execute_cmd_special( $cifs_client, "echo $content >>  $to_path" );
    } else {
        logTrc("Appending to the file using NFS");
        my $nfs_client  = $opts{client};
        my $append_file = $opts{append_file};
        my $command;

        if ( defined($append_file) ) {
            $command = "cat $append_file >> $to_path";
        } else {
            $command = "echo $content >> $to_path";
        }
        if ( !defined($nfs_client) ) {
            print("Using uSystem");
            uSystem("$command");
        } else {
            print("using say");
            say( $nfs_client, $command );
        }
    } ## end else [ if ( $protocol and $protocol...
} ## end sub nfs_cifs_append

## @name ::nfs_cifs_overwrite
## @arg $file	     - required: path of the file to be overwritten
## @arg $content     - required: content to overwrite.
## @arg $protocol    - optional: nfs|cifs. a named parameter referencing the
##		       protocol to be used for the overwrite. default - nfs
## @arg client       - optional: a named parameter referencing a nfs connection
##		       object or cifs client through which the overwrite should
##		       be done. The variable is treated as a nfs/cifs client
##		       depending on the value passed in for protocol.
## @description      overwrite the contents of  a file using either nfs or cifs.
##			returns 1 on success. 0 on failure.
sub nfs_cifs_overwrite {
    my ( $file, $content, %opts ) = @_;
    my $protocol = $opts{protocol};

    if ( $protocol and $protocol eq 'cifs' ) {
        my $cifs_client = $opts{client};
        if ( !defined($cifs_client) ) {
            croak("cifs_client needed for cifs copy");
        }
        logTrc("Overwriting the file using CIFS");
        my $output
            = execute_cmd_special( $cifs_client, "echo $content > $file" );
        if ($output) {
            $@ = $output;
            return 0;
        } else {
            return 1;
        }
    } else {
        logTrc("Overwriting the file using NFS");
        my $nfs_client = $opts{client};
        if ( !defined($nfs_client) ) {
            uSystem("echo '$content' > $file");
        } else {
            say( $nfs_client, "echo '$content' > $file" );
        }
    } ## end else [ if ( $protocol and $protocol...
} ## end sub nfs_cifs_overwrite

## @name ::nfs_cifs_delete
## @arg $file        - required: path of the file
## @arg protocol     - optional: nfs|cifs. a named parameter referencing the
##		       protocol to be used for the delete. default - nfs
## @arg client       - optional: a named parameter referencing a nfs connection
##		       object or cifs client through which the delete should be
##		       done. The variable is treated as a nfs/cifs client
##		       depending on the value passed in for protocol.
## @description      delete a file using either nfs or cifs.
##
sub nfs_cifs_delete {
    my ( $file, %opts ) = @_;
    my $protocol = $opts{protocol};

    if ( $protocol and $protocol eq 'cifs' ) {
        my $cifs_client = $opts{client};
        if ( !defined($cifs_client) ) {
            croak("cifs_client needed for cifs copy");
        }
        logTrc("Deleting the file using CIFS");
        my $output = execute_cmd( $cifs_client, "del $file" );
        if ($output) {
            $@ = $output;
            return 0;
        } else {
            return 1;
        }
    } else {
        logTrc("Deleting the file using NFS");
        my $nfs_client = $opts{client};
        return delfile( $file, conn => $nfs_client );
    }
} ## end sub nfs_cifs_delete

## @name ::nfs_cifs_mkdir
## @arg $path - required: the directory path to be created
## @arg protocol - optional: nfs|cifs. a named parameter referencing
##                 the protocol to use. default = nfs
## @arg client  - optional: a named parameter referencing a connection
##                object through which the mkdir should be done.
## @arg timeout - optional: a named parameter specifying the timeout.
##                this parameter is only meaningful when the 'client'
##                parameter is also given.
## @description 'mkdir -p' on the given path on the given client.
##              returns 1 on success, 0 and $@ on failure.

sub nfs_cifs_mkdir($@) {
    my ( $path, %opts ) = @_;
    my $protocol = $opts{protocol};

    if ( $protocol and $protocol eq 'cifs' ) {
        my $cifs_client = $opts{client};
        if ( !defined($cifs_client) ) {
            croak("cifs_client needed for cifs operations");
        }
        my $cmd = q(cmd /c x:\\perl\\bin\\perl.exe -MFile::Path -le )
            . qq("eval {mkpath(qw($path),1);}; print \$@ if \$@;");
        logTrc("Creating directory $path using CIFS");
        my $cc = make_conn($cifs_client);
        my ( undef, undef, $output ) = say( $cc, $cmd );
        logDbg("CIFS mkpath($path) returned: $output");

        # verify that directory was created
        my $output2 = execute_cmd( $cifs_client, "dir $path" );
        if ( $output2 =~ /File Not Found/ ) {
            $@ = $output;
            return 0;
        } else {
            return 1;
        }
    } else {
        logTrc("Creating directory $path using NFS");
        my $nfs_client = $opts{client};
        return util_mkdir( $path, conn => $nfs_client );
    }
} ## end sub nfs_cifs_mkdir($@)

## @name ::lun_create
## @arg conn - reqauired. named param the connection object to the FILER
## @arg lun_path - required: named parameter the path of the LUN to be created
## @arg from_file - optional: file from which to create the LUN. it is an error
## for both from_file and size to be defined.
## @arg size - optional: named parameter LUN size, default 5m
## @arg ostype - optional: named parameter ostype, default 'linux'
## @arg reservation - optional: named parameter reservation type
## @description Create an empty, unmounted LUN at lun_path on FILER.
##              returns 1 on success, dies on failure;

sub lun_create(%) {
    my (%opts) = @_;

    my $from_file = $opts{from_file};
    my $size      = $opts{size};
    my $ostype    = $opts{ostype} ? $opts{ostype} : 'linux';
    my $reuse     = $opts{reuse} ? $opts{reuse} : $::SETUP_DONE;
    my $path      = $opts{lun_path};

    # build the command string based on options given
    my $cmd = "lun create";
    if ( defined($from_file) ) {
        if ( defined($size) ) {
            die("from_file and size are mutually exclusive options: from_file = '$from_file', size = '$size'."
            );
        }

        # here, from_file must be defined (and size is not)
        $cmd .= " -f $from_file";
    } else {

        # set default size
        $size = '4m' if ( !defined($size) );
        $cmd .= " -s $size";
    } ## end else [ if ( defined($from_file...

    #
    $cmd .= " -t $ostype ";

    #
    if ( $opts{reservation} ) {
        $cmd .= "-o noreserve ";
    }

    # finally, add the lun path
    $cmd .= $path;

    my ( $output, $output2 );
    ( undef, undef, $output2 ) = say( $opts{conn}, "lun show $path" );
    if ( $output2 !~ /No such LUN exists/i ) {

        # LUN already exists
        if ( $reuse eq "0" ) {
            die "LUN $path already exists and shouldn't";
        } elsif ( $reuse eq 'preclean' ) {
            logDbg("PRECLEAN by destroying LUN $path");
            ( undef, undef, $output )
                = say( $opts{conn}, "lun destroy -f $path" );
        } else {
            return 1;    # reuse case
        }
    } ## end if ( $output2 !~ /No such LUN exists/i)

    logTrc("creating LUN $path");
    ( undef, undef, $output ) = say( $opts{conn}, $cmd );
    die $output
        if ( $output =~ /no space left on device|too many|already exists/i );

    ( undef, undef, $output2 ) = say( $opts{conn}, "lun show $path" );
    die "$opts{lun_path}: $output2" if $output2 =~ /No such LUN exists/i;

    return 1;
} ## end sub lun_create(%)

## @name ::lun_destroy
## @arg conn - required. named param the connection object to the FILER
## @arg lun_path - required: named parameter the path of the LUN to be created
## @arg force - optional: add '-f' to the command
## @description destroy a LUN at lun_path on FILER.
##              returns 1 on success, die on failure

sub lun_destroy(%) {
    my (%opts) = @_;

    my $force = $opts{force} ? "-f" : "";
    my ( undef, undef, $output )
        = say( $opts{conn}, "lun destroy $force $opts{lun_path}" );

    my ( undef, undef, $output2 )
        = say( $opts{conn}, "lun show $opts{lun_path}" );
    die "$opts{lun_path}: $output2" if $output2 !~ /No such LUN exists/i;

    return 1;
} ## end sub lun_destroy(%)

## @name ::lun_exists
## @arg conn - required, named param the connection object to the FILER
## @arg lun_path - required: named parameter the path of the LUN to be created
## @description destroy a LUN at lun_path on FILER.
##              returns 1 if the LUN exists, 0 if doesn't

sub lun_exists(%) {
    my (%opts) = @_;

    my $output;
    ( undef, undef, $output )
        = say( $opts{conn}, "lun show $opts{lun_path}" );
    return $output =~ /No such LUN exists/i ? 0 : 1;
} ## end sub lun_exists(%)

## @name ::lun_clone
## @arg conn - reqauired. named param the connection object to the FILER
## @arg clone_path - required: named parameter the path of the LUN clone
##                   to be created
## @arg lun_path - required: named parameter the path of the base LUN
## @arg snap_name - required: named parameter of base snapshot
## @arg reservation - optional: named parameter reservation type
## @description Create a clone at clone_path of a LUN at lun_path on FILER.
##              returns 1 on success, dies on failure;

sub lun_clone(%) {
    my (%opts) = @_;

    my $clone_path = $opts{clone_path};
    my $base_path  = $opts{lun_path};
    my $snap_name  = $opts{snap_name};

    my ( $output, $output2 );
    logTrc(
        "creating LUN clone $clone_path from LUN $base_path at snap $snap_name"
    );
    my $cmd = "lun clone create $clone_path ";
    if ( $opts{reservation} ) {
        $cmd .= "-o noreserve ";
    }
    $cmd .= "-b $base_path $snap_name";
    ( undef, undef, $output ) = say( $opts{conn}, $cmd );
    die $output
        if ( $output =~ /no space left on device|too many/i );

    ( undef, undef, $output2 ) = say( $opts{conn}, "lun show $clone_path" );
    die "$opts{lun_path}: $output2" if $output2 =~ /No such LUN exists/i;

    return 1;
} ## end sub lun_clone(%)

## @name ::lun_clone_split
## @arg conn - required. named param the connection object to the FILER
## @arg lun_path - required: named parameter the path of the clone
## @arg completion - optional: named parameter "start" or "wait" until finished (default)
## @description Split a clone at clone_path of a LUN at lun_path on FILER.
##              returns 1 on success, dies on failure;

sub lun_clone_split(%) {
    my (%opts) = @_;

    my $clone_path = $opts{lun_path};
    my $wait = $opts{completion} ? $opts{completion} : "wait";

    my ( $output, $output2 );
    logTrc("splitting LUN-clone $clone_path");
    my $cmd = "lun clone split start $clone_path ";
    ( undef, undef, $output ) = say( $opts{conn}, $cmd );
    die($output)
        if ( $output =~ /no space left on device|too many/i );

    return 1 unless $wait;

    do {
        ( undef, undef, $output2 )
            = say( $opts{conn}, "lun clone split status $clone_path" );
        die "$clone_path: $output2" if $output2 =~ /No such LUN exists/i;
        } while ( $output2 !~ /lun clone split status: .* LUN is not a clone/
        && $output2
        !~ /lun clone split status: .* LUN clone is not being split/ );

    return 1;
} ## end sub lun_clone_split(%)

## @name ::lun_clone_split_status
## @arg conn - required. named param the connection object to the FILER
## @arg lun_path - required: named parameter the path of the clone
## @description return status of Split a clone at clone_path of a LUN at lun_path on FILER.
##              returns string on success, dies on failure;

sub lun_clone_split_status(%) {
    my (%opts) = @_;

    my $clone_path = $opts{lun_path};

    my $output2;

    ( undef, undef, $output2 )
        = say( $opts{conn}, "lun clone split status $clone_path" );
    die "$clone_path: $output2" if $output2 =~ /No such LUN exists/i;

    return $output2;
} ## end sub lun_clone_split_status(%)

## @name ::lun_share
## @arg conn - required. named param the connection object to the FILER
## @arg lun_path - required: named parameter the path of the LUN
## @arg mode - optional: named param none|read|write|all. default none
## @description return status of 'lun share' lun_path on FILER.
##              returns 1 on success, dies on failure;

sub lun_share(%) {
    my (%opts) = @_;

    my $clone_path = $opts{lun_path};
    my $mode = $opts{mode} ? $opts{mode} : 'none';

    my $output;

    ( undef, undef, $output )
        = say( $opts{conn}, "lun share $clone_path $mode" );
    die "$clone_path: $output" if $output =~ /No such LUN exists/i;

    return 1;
} ## end sub lun_share(%)

## @name ::lun_rescan
## @arg conn - required. named param the connection object to the FILER
## @arg lun_path - required: named parameter the path of the LUN
## @arg mode - optional: named param none|read|write|all. default none
## @description return status of 'lun share' lun_path on FILER.
##              returns 1 on success, dies on failure;

sub lun_rescan(%) {
    my (%opts) = @_;

    my $output;
    my $cmd = "priv set -q diag; lun rescan $opts{volume}";
    ( undef, undef, $output ) = say( $opts{conn}, $cmd );
    die "lun rescan $opts{volume}: $output" if $output =~ /No such volume/i;

    return 1;
} ## end sub lun_rescan(%)

# zapi can fail under so many circumstances, so shut it up
# since we dont want FAIL results showing up from libs
sub _fail_default {
    my $msg = shift;
    logDbg("zapi failed: $msg ($@)");
}

## @name ::lun_list_info
## @usage lun_list_info(conn=>$conn, %options)
## @arg path (optional): path to LUN. if empty, all LUNs
## @arg get_backing (optional, boolean): return name of backing snapshot for LUN clone
## @description return status of ZAPI lun-list-info.
##              returns array of hashes on success, dies on failure

sub lun_list_info(%) {
    my (%opts) = @_;

    my $zcmd = new NaElement("lun-list-info");
    $zcmd->child_add( new NaElement( "path", $opts{path} ) )
        if $opts{path};
    $zcmd->child_add(
        new NaElement( "get-clone-backing-snapshot", $opts{get_backing} ) )
        if $opts{get_backing};

    my $zapi = Zapi->new( $opts{conn}->hostp()->default_ip(),
        "root", $opts{conn}->hostp()->default_password() );
    $zapi->{fail} = \&_fail_default;

    my $xo = $zapi->invoke_elem( $zcmd, "any" );
    logDbg( "lun-list-info ZAPI returns: " . $xo->toString() );

    my @results;
    if ( !$xo->results_ok() ) {
        if ( $xo->name() eq "results" ) {
            die
                "lun-list-info FAILed: status=>$xo->attr_get('status'), errno=>$xo->attr_get('errno'), reason=>$xo->attr_get('reason')";
        } else {
            die "lun-list-info FAILed: unknown reason";
        }
    } ## end if ( !$xo->results_ok(...
    my $vo = $xo->child_get("luns");
    if ($vo) {
        my $num_elems = $vo->child_max_number();
        logDbg("child-get(luns) returned $num_elems children");
        for ( my $i = 0; $i < $num_elems; $i++ ) {
            my %out = ();
            my $vl  = $vo->child_get_nth($i);
            $out{"path"} = $vl->child_get("path")->{content};
            $out{"size"} = $vl->child_get("size")->{content};
            $out{"serial-number"}
                = $vl->child_get("serial-number")->{content};
            $out{"block-size"}  = $vl->child_get("block-size")->{content};
            $out{"read-only"}   = $vl->child_get("read-only")->{content};
            $out{"online"}      = $vl->child_get("online")->{content};
            $out{"mapped"}      = $vl->child_get("mapped")->{content};
            $out{"share-state"} = $vl->child_get("share-state")->{content};
            $out{"backing-snapshot"}
                = $vl->child_get("backing-snapshot")->{content}
                if $vl->child_get("backing-snapshot");
            $out{"clone-backing-snapshot"}
                = $vl->child_get("clone-backing-snapshot")->{content}
                if $vl->child_get("clone-backing-snapshot");
            $out{"multiprotocol-type"}
                = $vl->child_get("multiprotocol-type")->{content};
            $out{"uuid"}      = $vl->child_get("uuid")->{content};
            $out{"device-id"} = $vl->child_get("device-id")->{content}
                if $vl->child_get("device-id");
            $out{"is-space-reservation-enabled"}
                = $vl->child_get("is-space-reservation-enabled")->{content};
            push( @results, {%out} );
        } ## end for ( my $i = 0; $i < $num_elems...
    } ## end if ($vo)
    return (@results);
} ## end sub lun_list_info(%)

## @name ::lun_clone_list_info
## @usage lun_clone_list_info(conn=>$conn, volume=>$volume, snapshot=>$snapshot)
## @description Lists all LUN clones in specified snapshot.
##              returns array of strings (paths) on success,
##                      array or hash of error info on failure

sub lun_clone_list_info(%) {
    my (%opts) = @_;

    my $zcmd = new NaElement("lun-clone-list-info");
    $zcmd->child_add( new NaElement( "volume",   $opts{volume} ) );
    $zcmd->child_add( new NaElement( "snapshot", $opts{snapshot} ) );

    my $zapi = Zapi->new( $opts{conn}->hostp()->default_ip(),
        "root", $opts{conn}->hostp()->default_password() );
    $zapi->{fail} = \&_fail_default;

    my $xo = $zapi->invoke_elem( $zcmd, "any" );
    logDbg( "lun-clone-list-info ZAPI returns: " . $xo->toString() );

    my @results;
    if ( !$xo->results_ok() ) {
        if ( $xo->name() eq "results" ) {
            die
                "lun-clone-list-info FAILed: status=>$xo->attr_get('status'), errno=>$xo->attr_get('errno'), reason=>$xo->attr_get('reason')";
        } else {
            die "lun-clone-list-info FAILed: unknown reason";
        }
    } ## end if ( !$xo->results_ok(...
    my $vo = $xo->child_get("lun-clone-list");
    if ($vo) {
        my $num_elems = $vo->child_max_number();
        logDbg("child-get(lun-clone-list) returned $num_elems children");
        for ( my $i = 0; $i < $num_elems; $i++ ) {
            my $vl = $vo->child_get_nth($i);
            my $vp = $vl->child_get("path")->{content};
            push( @results, $vp );
        }
    } ## end if ($vo)
    return (@results);
} ## end sub lun_clone_list_info(%)

## @name ::dump_wait
## @arg filer - required. the hostobj of the FILER
## @arg tree - required. the path of filesystem to backup
## @arg opts - array of options+arguments to dump
## @description dump the filer tree at tree, wait for it to finish
##              returns 1 on success, dies on failure;

sub dump_wait {
    my ( $filer, $tree, @opts ) = @_;

    my $host = host($filer);
    my $fc = connect( "#telnet", '-timeout', $main::CMDTIMEOUT );
    my $output;
    my $cmd = "dump @opts $tree";
    ( undef, undef, $output ) = say( $fc, $cmd, "DUMP IS DONE" );
    die "dump: $output" if $output =~ /No such volume/i;

    return 1;
} ## end sub dump_wait

## @name ::restore_wait
## @arg filer - required. the hostobj of the FILER
## @arg opts - array of options+arguments to dump
## @description restore the filer tree, wait for it to finish
##              returns 1 on success, dies on failure;

sub restore_wait {
    my ( $filer, @opts ) = @_;

    my $host = host($filer);
    my $fc = connect( "#telnet", '-timeout', $::CMDTIMEOUT );
    my $output;
    my $cmd = "restore @opts";
    ( undef, undef, $output ) = say( $fc, $cmd, "RESTORE IS DONE" );
    die "restore: $output" if $output =~ /No such volume/i;

    return 1;
} ## end sub restore_wait

#=  "Barrel Shift" Data right or left. Result is returned. Original buffer is unchanged.
#=
#=   All args are 'keyword' args
#=
#= :data		data to shift. Required.
#= :count		shift count. Required.  + for left shift, - for right shift
#= :data_offset		Optional. offset into 'data' string to use. Default is 0.
#= :data_length		Optional. Length of 'data' string to use. Default is length($data)
sub uShiftData(%) {
    my (%args) = validate(
        @_,
        {   data        => { type => SCALAR },
            count       => { type => SCALAR },
            data_offset => { type => SCALAR, default => 0, optional => 1 },
            data_length => { type => SCALAR, default => 0, optional => 1 },
        }
    );

    my ( $data, $count, $data_offset, $data_length )
        = ( $args{data}, $args{count}, $args{data_offset},
        $args{data_length} );

    logDbg(
        "uShiftData:   count: $count  data_offset: $data_offset  data_length: $data_length"
    );
    if ( $data_offset || $data_length ) {
        if ($data_length) {
            my $data_len = length($data);
            $data_offset %= $data_len;
            my $max_len = $data_len - $data_offset;
            $data_length = $max_len if ( $data_length > $max_len );
            $data = substr( $data, $data_offset, $data_length );
        } else {
            $data = substr( $data, $data_offset );
        }
    } ## end if ( $data_offset || $data_length)
    $data_length = length($data);
    return ("") if ( $data_length == 0 );
    $count %= $data_length;
    my $shift_str;
    if ( $count == 0 ) {
        return ($data);
    } elsif ( $count < 0 ) {
        my $rem_count = $data_length - $count;
        $shift_str
            = substr( $data, $rem_count ) . substr( $data, 0, $rem_count );
    } else {
        $shift_str = substr( $data, $count ) . substr( $data, 0, $count );
    }
    return ($shift_str);
} ## end sub uShiftData(%)

#=  Generate data string from "random" bytes.
#=
#=   All args are 'keyword' args
#=
#= :length		Required. Length of 'data' string to generate.
#= :allowed_bytes	Optional. If given, a string giving the byte values allowed in the
#=			    string. Default is any value from 0 to 255.
#= :buffer_ref		Optional. SCALAR REF to existing buffer/string to be used
#=                          for output. Default is to construct new buffer/string.
#= :buffer_offset	Optional. Offset into buffer to begin copying to. Optional.
#=                          Default is zero. Ignored unless "buffer_ref" is specified.

sub uRandData(%) {
    my (%args) = validate(
        @_,
        {   length => { type => SCALAR },
            allowed_bytes =>
                { type => SCALAR, default => undef, optional => 1 },
            buffer_ref => {
                type     => SCALARREF | UNDEF,
                default  => undef,
                optional => 1
            },
            buffer_offset => { type => SCALAR, default => 0, optional => 1 },
        }
    );

    my ( $length, $allowed_bytes, $buffer_ref, $buffer_offset ) = (
        $args{length},     $args{allowed_bytes},
        $args{buffer_ref}, $args{buffer_offset}
    );

    #logDbg("uRandData\nLength: $length\nAllowed Bytes: \"$allowed_bytes\"" .
    #	"\nBuffer Offset: $buffer_offset");

    my $buffer = "";
    if ( !$buffer_ref ) {
        $buffer_ref    = \$buffer;
        $buffer_offset = 0;
    }
    my $buff_len = length($$buffer_ref);
    $buffer_offset = $buff_len if ( $buffer_offset > $buff_len );

    my $ofs = $buffer_offset;
    for ( my $cnt = 0; $cnt < $length; $cnt++, $ofs++ ) {
        my $char;
        if ($allowed_bytes) {
            $char = substr( $allowed_bytes,
                int( rand( length($allowed_bytes) ) ), 1 );
        } else {
            $char = chr( int( rand(256) ) );
        }

        #substr($$buffer_ref, $ofs, 0) = $char;
        substr( $$buffer_ref, $ofs ) = $char;

        # Debug
        my $cval = ord($char);
        if ( $cval > 32 && $cval < 255 ) {

            #logDbg("Count: $cnt   Char: ".sprintf("%02X\t%c", $cval, $cval));
        } else {

         #logDbg("Count: $cnt   Char: ".sprintf("%02X\t%c", $cval, ord(' ')));
        }
    } ## end for ( my $cnt = 0; $cnt...

  #logDbg("Exit uRandData:\n\n".uHexDump(data=>$$buffer_ref, print_ascii=>1));
    return ($$buffer_ref);
} ## end sub uRandData(%)

#=  Generate data string which is composed of smaller substrings of a given "seed" data string.
#=  This fcn is used to create large randomized data blocks/strings from smaller ones, such
#=  as those created by uRandData.
#=
#=   All args are 'keyword' args
#=
#= :length		Required. Length of 'data' string to generate.
#= :data		Required. The string from which the various substrings
#=			    will be generated.
#= :min_substr_len		Optional. Minimum length of substrings. Default is 1.
#= :max_substr_len		Optional. Minimum length of substrings. Default is length($data)/2
#= :buffer_ref		Optional. SCALAR REF to existing buffer/string to be used
#=                          for output. Default is to construct new buffer/string.
#= :buffer_offset	Optional. offset into buffer to begin copying to. Optional.
#=                          Default is zero. Ignored unless "buffer_ref" is specified.

sub uSeqData(%) {
    my (%args) = validate(
        @_,
        {   length         => { type => SCALAR },
            data           => { type => SCALAR },
            min_substr_len => { type => SCALAR, default => 1, optional => 1 },
            max_substr_len => { type => SCALAR, default => 0, optional => 1 },
            buffer_ref => {
                type     => SCALARREF | UNDEF,
                default  => undef,
                optional => 1
            },
            buffer_offset => { type => SCALAR, default => 0, optional => 1 },
        }
    );

    my ( $length, $data, $min_substr_len, $max_substr_len, $buffer_ref,
        $buffer_offset )
        = (
        $args{length}, $args{data}, $args{min_substr_len},
        $args{max_substr_len}, $args{buffer_ref}, $args{buffer_offset}
        );

#logDbg("uSeqData\nLength: $length\nData Length: ".length($data)."\nMin Substr Len: " .
#	"$min_substr_len\nMax Substr Len: $max_substr_len\nBuffer Offset: $buffer_offset");

    my $data_len               = length($data);
    my $default_min_substr_len = 1;
    my $default_max_substr_len = int( length($data) / 2 );
    if ( !$max_substr_len || $max_substr_len > $length ) {
        $max_substr_len = $default_max_substr_len;
    }
    if ( !$min_substr_len || $min_substr_len > $max_substr_len ) {
        $min_substr_len = $max_substr_len;
    }
    my $buffer = "";
    if ( !$buffer_ref ) {
        $buffer_ref    = \$buffer;
        $buffer_offset = 0;
    }
    my $buff_len = length($$buffer_ref);
    $buffer_offset = $buff_len if ( $buffer_offset > $buff_len );

    my $buff_ofs      = $buffer_offset;
    my $rem_length    = $length;
    my $sub_len_range = ( $max_substr_len - $min_substr_len );

#logDbg("uSeqData:  max_substr_len: $max_substr_len  min_substr_len: $min_substr_len  " .
# 	"sub_len_range: $sub_len_range");
    while ( $rem_length > 0 ) {
        if ( $rem_length < $max_substr_len ) {
            $sub_len_range  = ( $rem_length - 1 );
            $min_substr_len = 1;
        }
        my $sub_len = $min_substr_len + int( rand( 1 + $sub_len_range ) );
        my $sub_ofs = int( rand( 1 + $data_len - $sub_len ) );
        my $sub     = substr( $data, $sub_ofs, $sub_len );

#croak("uSeqData:  sub_len_range: $sub_len_range  rem_length: $rem_length  ".
#	"sub_len: $sub_len  length(sub): ".length($sub)); if (length($sub) > $rem_length);
        substr( $$buffer_ref, $buff_ofs ) = $sub;
        $buff_ofs += $sub_len;
        $rem_length -= $sub_len;

# logDbg("Data: \"$data\"   sub_len: $sub_len  sub_ofs: $sub_ofs   " .
# 	"buff_ofs: $buff_ofs   rem_length: $rem_length  length(): ".length($$buffer_ref));
    } ## end while ( $rem_length > 0 )
    return ($$buffer_ref);
} ## end sub uSeqData(%)

#=  Verify content of "test" block against "master" block.
#=
#=  All args are 'keyword' args
#=
#= :data	      "master" copy of data. Required.
#= :buffer            buffer to verify. Required.
#= :length	      length of comparison. Optional. Default is
#=		            min( length(buff), length(data) )
#= :data_offset       offset into 'data' for start of comparison.
#=                          Optional. Default is  0.
#= :buffer_offset     offset into 'data' for start of comparison.
#=                          Optional. Default is 0.
#= :croak_on_miscompare      If '1', then croak if comparison fails. If '0' then return '0'
#=                           if comparison fails. Default is '1'.
sub uVerifyData(%) {
    my (%args) = validate(
        @_,
        {   data          => { type => SCALAR },
            buffer        => { type => SCALAR },
            length        => { type => SCALAR, default => 0, optional => 1 },
            data_offset   => { type => SCALAR, default => 0, optional => 1 },
            buffer_offset => { type => SCALAR, default => 0, optional => 1 },
            croak_on_miscompare =>
                { type => SCALAR, default => 1, optional => 1 },
        }
    );

    my ( $data, $buffer, $length, $data_offset, $buffer_offset,
        $croak_on_miscompare )
        = (
        $args{data}, $args{buffer}, $args{length}, $args{data_offset},
        $args{buffer_offset}, $args{croak_on_miscompare}
        );

    my $data_len   = length($data);
    my $buffer_len = length($buffer);
    my $max_len1
        = ( $data_len > $data_offset ) ? ( $data_len - $data_offset ) : 0;
    my $max_len2
        = ( $buffer_len > $buffer_offset )
        ? ( $buffer_len - $buffer_offset )
        : 0;
    my $max_length = ( $max_len1 < $max_len2 ) ? $max_len1 : $max_len2;
    $length = $max_length if ( !$length || ( $length > $max_length ) );
    logDbg(   "\nuVerifyData\nlength: $length\nmax_length: $max_length\n"
            . "data_offset: $data_offset\ndata_length: $data_len\n"
            . "buffer_offset: $buffer_offset\nbuffer_length: $buffer_len\n"
            . "croak_on_miscompare: $croak_on_miscompare" );

#logDbg("\nuVerifyData  Line: ".__LINE__."\n\ndata:\t\"$data\"\nbuffer:\t\"$buffer\"");

    if ( $max_length == 0 ) {
        croak(    "uVerifyData Line: " 
                . __LINE__ . "\n"
                . "length: $length\nmax_length: $max_length\n"
                . "data_offset: $data_offset\ndata_length: $data_len\n"
                . "buffer_offset: $buffer_offset\nbuffer_length: $buffer_len"
        );
    } ## end if ( $max_length == 0 )
    my $data_substr   = substr( $data,   $data_offset,   $length );
    my $buffer_substr = substr( $buffer, $buffer_offset, $length );

    my $verify_result = undef;
    if ( $buffer_substr ne $data_substr ) {
        $verify_result = 0;
        logDbg("uVerfiyData miscompare");
        logDbg(   "uVerifyData miscompare\n\ndata_offset: $data_offset  "
                . "buffer_offset: $buffer_offset  length: $length   max_length: $max_length"
        );
        if ( $length < 1200 ) {
            logDbg(
                "\nExpected:\n\n"
                    . uHexDump(
                    data        => $data_substr,
                    offset      => $data_offset,
                    length      => $length,
                    print_ascii => 1
                    )
                    . "\n\nActual:\n\n"
                    . uHexDump(
                    data        => $buffer_substr,
                    offset      => $buffer_offset,
                    length      => $length,
                    print_ascii => 1
                    )
            );
        } ## end if ( $length < 1200 )
        if ($croak_on_miscompare) {
            croak("uVerfiyData miscompare");
        }
    } else {
        $verify_result = 1;
    }

    logDbg("uVerifyData:  Returning $verify_result");
    return ($verify_result);
} ## end sub uVerifyData(%)

#= Read and return arbitrary data from a filehandle
#=
#=  All args are 'keyword' args
#=
#= :rd              Required. Input filehandle.
#= :length	    Required. length to read.
#= :blksz           Optional. I/O block size to use. I/O ops are split into
#=                      a sequence of smaller ones, such that all ops have length
#=                      blksz or less. If 0 (zero), then I/O ops are not split.
#= :buffer_ref      SCALAR REF to existing buffer/string to be used
#=                       for temp. storage of input. Optional. Default
#=                       is to construct new buffer/string.
#= :buffer_offset   offset into buffer to begin copying to. Optional.
#=                          Default is zero. Ignored unless "buffer_ref" is specified.
sub uReadData(%) {
    my (%args) = validate(
        @_,
        {   rd     => { type => GLOBREF },
            length => { type => SCALAR },
            blksz  => { type => SCALAR, default => 0, optional => 1 },
            buffer_ref => {
                type     => SCALARREF | UNDEF,
                default  => undef,
                optional => 1
            },
            buffer_offset => { type => SCALAR, default => 0, optional => 1 },
        }
    );

    my ( $rd, $length, $blksz, $buffer_ref, $buffer_offset ) = (
        $args{rd}, $args{length}, $args{blksz}, $args{buffer_ref},
        $args{buffer_offset}
    );

    $blksz ||= $length;

    logDbg(
        "uReadData:  length: $length  blksz: $blksz  buffer_offset: $buffer_offset  fd="
            . fileno($rd) );

    my $buffer = "";
    if ( !$buffer_ref ) {
        $$buffer_ref   = \$buffer;
        $buffer_offset = 0;
    }
    my $buff_len = length($$buffer_ref);
    $buffer_offset = $buff_len if ( $buffer_offset > $buff_len );

    my $buff_ofs    = $buffer_offset;
    my $rem_length  = $length;
    my $length_read = 0;
    my $blknum      = 0;
    while ($rem_length) {
        my $to_read
            = ( $blksz && $blksz < $rem_length ) ? $blksz : $rem_length;
        logInfo("Seq read $to_read bytes @ offset $buff_ofs ...");

        logDbg(
            "uReadData  Calling sysread  length: $to_read  ofs: $buff_ofs");
        my $rdlen = sysread( $rd, $$buffer_ref, $to_read, $buff_ofs );
        logDbg("uReadData  Returned from sysread  rdlen: $rdlen");
        if ( !defined($rdlen) ) {
            if ( $! == EINTR ) {
                $rdlen = 0;
                next;
            }
            croak("seq read failed: $!\n");
        } elsif ( !$rdlen ) {
            logWarn("uReadData: no data read by sysread: $!");
            croak("uReadData: no data read by sysread: $!");
        } elsif ( $rdlen != $to_read ) {
            logWarn(  "uReadData -- Bytes read:  Expected=$to_read  "
                    . "Actual=$rdlen\n$!\nBlknum: $blknum" );
            croak(    "uReadData -- Bytes read:  Expected=$to_read  "
                    . "Actual=$rdlen\n$!\nBlknum: $blknum" );
        } ## end elsif ( $rdlen != $to_read)
        logDbg("uReadData: $rdlen bytes read");
        $rem_length -= $rdlen;
        $buff_ofs    += $rdlen;
        $length_read += $rdlen;
        $blknum++;
    } ## end while ($rem_length)
    logDbg(
        "Exit uReadData  Bytes read: Expected=$length  Actual=$length_read");
    return ($$buffer_ref);
} ## end sub uReadData(%)

#= Read and verify arbitrary data from a filehandle
#=
#=  All args are 'keyword' args
#=
#= :rd              Required. Input filehandle.
#= :data            Required. data (string) to be comared against file contents.
#= :length	    length to write. Optional. Default is length($data)
#= :data_offset     Offset into data of first byte to write. Optional.
#=                       Default is zero.
#= :blksz           Optional. I/O block size to use. I/O ops are split into
#=                      a sequence of smaller ones, such that all ops have length
#=                      blksz or less. If 0 (zero), then I/O ops are not split.
#= :buffer_ref      SCALAR REF to existing buffer/string to be used
#=                       for temp. storage of input. Optional. Default
#=                       is to construct new buffer/string.
#= :croak_on_miscompare  If '1', then croak if comparison fails. If '0' then return '0' if comparison fails.
#=                          Default is one.
#= :length_read_ref	Optional. If given, then is a reference to a scalar, which will be used to store
#=			    the total number of bytes read.
#= :return	     Returns '0' if match failed, or '1' if match succeeded. If 'read' failed, or if
#=		         croak_on_miscompare is set and match failed, then 'croaks'
sub uReadVerifyData(%) {
    my (%args) = validate(
        @_,
        {   rd          => { type => GLOBREF },
            data        => { type => SCALAR },
            length      => { type => SCALAR, default => 0, optional => 1 },
            data_offset => { type => SCALAR, default => 0, optional => 1 },
            blksz       => { type => SCALAR, default => 0, optional => 1 },
            buffer_ref => {
                type     => SCALARREF | UNDEF,
                default  => undef,
                optional => 1
            },
            croak_on_miscompare =>
                { type => SCALAR, default => 1, optional => 1 },
            length_read_ref =>
                { SCALARREF | UNDEF, default => undef, optional => 1 },
        }
    );

    my ( $rd, $data, $length, $data_offset, $blksz, $buffer_ref,
        $croak_on_miscompare, $length_read_ref )
        = (
        $args{rd},                  $args{data},
        $args{length},              $args{data_offset},
        $args{blksz},               $args{buffer_ref},
        $args{croak_on_miscompare}, $args{length_read_ref}
        );
    my $data_len   = length($data);
    my $max_length = ( $data_len - $data_offset );
    $blksz ||= $length;

    logDbg(   "\nuReadVerifyData\nlength: $length\nmax_length: $max_length\n"
            . "data_offset: $data_offset\ndata_length: $data_len\n" . "fd="
            . fileno($rd)
            . "\nblksz: $blksz\ncroak_on_miscompare: $croak_on_miscompare" );

    $length = $max_length if ( !$length || ( $length > $max_length ) );
    my $buffer = "";
    if ( !$buffer_ref ) {
        $$buffer_ref = \$buffer;
    }
    my $length_read = "";
    if ( !$length_read_ref ) {
        $$length_read_ref = \$length_read;
    }

    my $data_ofs   = $data_offset;
    my $buff_ofs   = 0;
    my $rem_length = $length;
    $$length_read_ref = 0;
    my $verify_result = 1;
    my $blknum        = 0;
    while ($rem_length) {
        my $to_read
            = ( $blksz && $blksz < $rem_length ) ? $blksz : $rem_length;
        logInfo("Seq read $to_read bytes @ data_offset $buff_ofs ...");
        logDbg(
            "uReadVerifyData  Calling sysread  length: $to_read  ofs: $buff_ofs"
        );
        my $rdlen = sysread( $rd, $$buffer_ref, $to_read, $buff_ofs );
        logDbg("uReadVerifyData  Returned from sysread  rdlen: $rdlen");
        $$length_read_ref += $rdlen;
        if ( !defined($rdlen) ) {
            if ( $! == EINTR ) {
                $rdlen = 0;
                next;
            }
            croak("seq read failed: $!\n");
        } elsif ( !$rdlen ) {
            logWarn("uReadVerifyData: no data read by sysread: $!");
            croak("uReadVerifyData: no data read by sysread: $!");
        } elsif ( $rdlen != $to_read ) {
            logWarn(  "uReadVerifyData -- Expected to read $to_read bytes  "
                    . "Actual bytes read: $rdlen\n$!\nBlknum: $blknum" );
            croak(    "uReadVerifyData -- Expected to read $to_read bytes  "
                    . "Actual bytes read: $rdlen\n$!\nBlknum: $blknum" );
        } ## end elsif ( $rdlen != $to_read)
        logDbg("uReadVerifyData: $rdlen bytes read");

        #$ength = $rdlen;
        logDbg(   "\nuReadVerifyData  Calling uVerifyData\nblkum: $blknum\n"
                . "length: $rdlen\ndata_offset: $data_ofs\n"
                . "buffer_offset: $buff_ofs\ndata:\t\"$data\"\nbuffer:\t\"$$buffer_ref\""
        );
        $verify_result = uVerifyData(
            data                => $data,
            buffer              => $$buffer_ref,
            length              => $rdlen,
            data_offset         => $data_ofs,
            buffer_offset       => $buff_ofs,
            croak_on_miscompare => $croak_on_miscompare
        );

        logDbg("uReadVerifyData  Returned from uVerifyData: $verify_result");
        if ( !$verify_result ) {
            logDbg(
                "uReadVerifyData miscompare ($$length_read_ref bytes previously matched)"
            );
            if ($croak_on_miscompare) {
                logWarn("uReadVerifyData miscompare");
                croak("uReadVerifyData miscompare");
            }
        } ## end if ( !$verify_result )
        $rem_length -= $rdlen;
        $data_ofs += $rdlen;
        $buff_ofs += $rdlen;
        $blknum++;
    } ## end while ($rem_length)
    if ( !$verify_result ) {
        logDbg("Exit uReadVerifyData -- match failed");
    } else {
        logDbg("Exit uReadVerifyData -- match succeeded");
    }
    logDbg(   "Exit uReadVerifyData  Verify result: $verify_result  "
            . "Bytes read: Expected=$length  Actual=$$length_read_ref" );
    return ($verify_result);
} ## end sub uReadVerifyData(%)

#= Write arbitrary data to a filehandle
#=
#=  All args are 'keyword' args
#=
#= :wrt             Required. output filehandle.
#= :data            Reqquired. data to write (string).
#= :length	    length to write. Optional. Default is length($data)
#= :data_offset     Offset into data of first byte to write. Optional.
#=                      Default is zero.
#= :blksz           Optional. I/O block size to use. I/O ops are split into
#=                      a sequence of smaller ones, such that all ops have length
#=                      blksz or less. If 0 (zero), then I/O ops are not split.
sub uSendData(%) {
    my (%args) = validate(
        @_,
        {   wrt         => { type => GLOBREF },
            data        => { type => SCALAR },
            length      => { type => SCALAR, default => 0, optional => 1 },
            data_offset => { type => SCALAR, default => 0, optional => 1 },
            blksz       => { type => SCALAR, default => 0, optional => 1 },
        }
    );

    my ( $wrt, $data, $length, $data_offset, $blksz ) = (
        $args{wrt}, $args{data}, $args{length}, $args{data_offset},
        $args{blksz}
    );

    my $max_length = ( length($data) - $data_offset );
    $blksz ||= $length;
    logDbg(
        "uSendData:  data_offset: $data_offset  length: $length  max_length: $max_length"
            . "  blksz= $blksz  fd="
            . fileno($wrt) );
    $length = $max_length if ( !$length || ( $length > $max_length ) );

    my $ofs            = $data_offset;
    my $rem_length     = $length;
    my $length_written = 0;
    my $blknum         = 0;
    while ($rem_length) {
        my $to_write
            = ( $blksz && $blksz < $rem_length ) ? $blksz : $rem_length;
        logInfo("Seq write $to_write bytes @ data_offset $ofs ...");
        logDbg("uSendData  Calling syswrite  length: $to_write  ofs: $ofs");
        my $wlen = syswrite( $wrt, $data, $to_write, $ofs );
        logDbg("uSendData  Returned from syswrite  length: $wlen");
        if ( !defined($wlen) ) {
            if ( $! == EINTR ) {
                $wlen = 0;
                next;
            }
            croak("uSendData: syswrite failed: $!");
        } elsif ( !$wlen ) {
            croak("uSendData: no data written by syswrite: $!");
        } elsif ( $wlen != $to_write ) {
            logWarn(  "uSendData:  Bytes written:  Expected=$to_write  "
                    . "Actual=$wlen\n$!\nBlknum: $blknum" );
            croak(    "uSendData:  Bytes written:  Expected=$to_write  "
                    . "Actual=$wlen\n$!\nBlknum: $blknum" );
        } ## end elsif ( $wlen != $to_write)
        logDbg("uSendData: $wlen bytes written");
        logInfo("... $wlen bytes written");
        $rem_length -= $wlen;
        $ofs            += $wlen;
        $length_written += $wlen;
        $blknum++;
    } ## end while ($rem_length)
    logDbg(
        "Exit uSendData  Bytes written: Expected=$length  Actual=$length_written"
    );
    return ($length_written);
} ## end sub uSendData(%)

#= Write arbitrary data to a filehandle and read it back, verify correct
#=
#=  All args are 'keyword' args
#=
#= :fh              Required. input/output filehandle Must be open for both reading and writing
#= :data            Required. Data to write and read (verify) (string).
#= :length	    length to write. Optional. Default is length($data)
#= :data_offset     Offset into data of first byte to write. Optional.
#=                      Default is zero.
#= :blksz           Optional. I/O block size to use. I/O ops are split into
#=                      a sequence of smaller ones, such that all ops have length
#=                      blksz or less. If 0 (zero), then I/O ops are not split.
#= :buffer_ref      SCALAR REF to existing buffer/string to be used
#=                       for temp. storage of input. Optional. Default
#=                       is to construct new buffer/string.
#= :croak_on_miscompare  If '1', then croak if comparison fails. If '0' then return '0' if comparison fails.
#=                          Default is one.
#= :length_read_ref	Optional. If given, then is a reference to a scalar, which will be used to store
#=			    the total number of bytes read.
#= :length_written_ref	Optional. If given, then is a reference to a scalar, which will be used to store
#=			    the total number of bytes read.
#= :return	     Returns '0' if match failed, or '1' if match succeeded. If 'read' failed, or if
#=		         croak_on_miscompare is set and match failed, then 'croaks'
sub uSendVerifyData(%) {
    my (%args) = validate(
        @_,
        {   fh          => { type => GLOBREF },
            data        => { type => SCALAR },
            length      => { type => SCALAR, default => 0, optional => 1 },
            data_offset => { type => SCALAR, default => 0, optional => 1 },
            blksz       => { type => SCALAR, default => 0, optional => 1 },
            buffer_ref => {
                type     => SCALARREF | UNDEF,
                default  => undef,
                optional => 1
            },
            croak_on_miscompare =>
                { type => SCALAR, default => 1, optional => 1 },
            length_read_ref =>
                { SCALARREF | UNDEF, default => undef, optional => 1 },
            length_written_ref =>
                { SCALARREF | UNDEF, default => undef, optional => 1 },
        }
    );

    my ($fh,                  $data,            $length,
        $data_offset,         $blksz,           $buffer_ref,
        $croak_on_miscompare, $length_read_ref, $length_written_ref
        )
        = (
        $args{fh},                  $args{data},
        $args{length},              $args{data_offset},
        $args{blksz},               $args{buffer_ref},
        $args{croak_on_miscompare}, $args{length_read_ref},
        $args{length_written_ref}
        );

    my $data_len   = length($data);
    my $max_length = ( $data_len - $data_offset );
    $blksz ||= $length;
    my $fpos = tell($fh);

    logDbg(   "\nuSendVerifyData\nlength: $length\nmax_length: $max_length\n"
            . "data_offset: $data_offset\ndata_length: $data_len\n" . "fd="
            . fileno($fh)
            . "\nfpos: $fpos\nblksz: $blksz\n"
            . "croak_on_miscompare: $croak_on_miscompare" );

    $length = $max_length if ( !$length || ( $length > $max_length ) );
    if ( defined($buffer_ref) ) {
        logDbg("uSendVerifyData   buffer_ref is set");
    } else {
        logDbg("uSendVerifyData   buffer_ref is not set");
    }

    my $buffer = "";
    if ( !$buffer_ref ) {
        $$buffer_ref = \$buffer;
    }
    my $length_read = "";
    if ( !$length_read_ref ) {
        $$length_read_ref = \$length_read;
    }
    my $length_written = "";
    if ( !$length_written_ref ) {
        $$length_written_ref = \$length_written;
    }

    logDbg("uSendVerifyData:  Calling uSendData");
    $length_written = uSendData(
        wrt         => $fh,
        data        => $data,
        length      => $length,
        data_offset => $data_offset,
        blksz       => $blksz
    );

    logDbg("uSendVerifyData:  Returned from uSendData");
    $$length_written_ref = $length_written;
    if ( $length_written != $length ) {
        logWarn(  "uSendVerifyData:  uSendData Wrote: $length_written bytes  "
                . "Expected: $length bytes\n$!" );
        croak(    "uSendVerifyData:  uSendData Wrote: $length_written bytes  "
                . "Expected: $length bytes\n$!" );
    } ## end if ( $length_written !=...

    sysseek( $fh, $fpos, 0 );
    logDbg("uSendVerifyData:  Calling uReadVerifyData");
    logDbg(
        "\nuSendVerifyData Calling uReadVerifyData\nlength: $length\ndata_offset: $data_offset\n"
            . "blksz: $blksz\ndata:\t\"$data\"\nbuffer:\t\"$$buffer_ref\"" );
    my $verify_result = uReadVerifyData(
        rd                  => $fh,
        data                => $data,
        length              => $length,
        data_offset         => $data_offset,
        buffer_ref          => $buffer_ref,
        croak_on_miscompare => $croak_on_miscompare,
        length_read_ref     => $length_read_ref,
        blksz               => $blksz
    );

    logDbg(
        "uSendVerifyData:  Returned from uReadVerifyData: Verify result: $verify_result"
            . "  Length Read: $$length_read_ref" );
    $length_read = $$length_read_ref;
    if ( $$length_read_ref != $length ) {
        logWarn(
            "uSendVerifyData:  uReadVerify Read: $$length_read_ref bytes  Expected: $length bytes\n$!"
        );
        croak(
            "uSendVerifyData:  uReadVerify Read: $$length_read_ref bytes  Expected: $length bytes\n$!"
        );
    } ## end if ( $$length_read_ref...
    if ( !$verify_result ) {
        logDbg("Exit uSendVerifyData -- match failed");
    } else {
        logDbg("Exit uSendVerifyData -- match succeeded");
    }
    logDbg(   "Exit uSendVerifyData  Verify result: $verify_result\n"
            . "Bytes written:\tExpected=$length  Actual=$$length_written_ref\n"
            . "Bytes read:\tExpected=$length  Actual=$$length_read_ref" );
    return ($verify_result);
} ## end sub uSendVerifyData(%)

#= Dump string/data in hex format.Write arbitrary data to a filehandle and read it back, verify correct
#=
#=  All args are 'keyword' args
#=
#= :data               Required. string/data to be dumped.
#= :base_addr          Base address. Optional. Default is zero.
#= :offset             Offset into data/string to begin dumping. Optional. Default is 0.
#= :length             Length to dump. Optional. Default is entire data/string.
#= :print_ascii        If '1', then also print ASCII char of each byte. Optional. Default is zero.
#= :line_bytes         Number of bytes/chars per line. Optional. Default is 16.
sub uHexDump(%) {
    my (%args) = validate(
        @_,
        {   data        => { type => SCALAR },
            base_addr   => { type => SCALAR, default => 0, optional => 1 },
            length      => { type => SCALAR, default => 0, optional => 1 },
            offset      => { type => SCALAR, default => 0, optional => 1 },
            line_bytes  => { type => SCALAR, default => 16, optional => 1 },
            print_ascii => { type => SCALAR, default => 0, optional => 1 },
        }
    );

    my ( $data, $base_addr, $length, $offset, $line_bytes, $print_ascii ) = (
        $args{data},   $args{base_addr},  $args{length},
        $args{offset}, $args{line_bytes}, $args{print_ascii}
    );

    $base_addr  ||= 0;
    $line_bytes ||= 16;
    $offset     ||= 0;
    my $ofs        = $offset;
    my $rem_length = length($data);
    if ( $length && $length < $rem_length ) {
        $rem_length = $length;
    }
    my $out = "";
    my $char_line;
    my $hex_line;
    my $addr = $base_addr;
    while ( $rem_length > 0 ) {
        my $addr_str = sprintf( "0x%04X", $addr );
        $hex_line   = "${addr_str}:\t";
        $char_line  = "${addr_str}:\t";
        $line_bytes = $rem_length if ( $line_bytes > $rem_length );
        for ( 0 .. ( $line_bytes - 1 ) ) {
            my $cval = ord( substr( $data, $ofs, 1 ) );
            $hex_line .= sprintf( "\t%02X", $cval );
            if ( $cval > 32 && $cval < 255 ) {
                $char_line .= sprintf( "\t%c", $cval );
            } else {
                $char_line .= sprintf( "\t%c", ord(" ") );
            }
            $ofs++;
            $rem_length--;
            $addr++;
        } ## end for ( 0 .. ( $line_bytes...
        $hex_line  .= "\n";
        $char_line .= "\n";
        $out       .= $hex_line;
        if ($print_ascii) {
            $out .= $char_line;
        }
        $out .= "\n";
    } ## end while ( $rem_length > 0 )
    return ($out);
} ## end sub uHexDump(%)

=head2 cifs_chown_file (%opts)

	Set owner of file using CIFS

	All args are 'keyword' args

	Requires 'subinacl' to be available on CIFS client
	at:  "x:\bin\subinacl.exe"

=over 4

=item  client

	Required	CIFS client ('hostspec' string, or 'HOSTREC' or 'CONNECTREC' object)

=item user

	Required	CIFS id of new owner

=item path

	Required	Windows path of file to have new owner

=item recurseive

	Optional	Default is '0' (zero). If '1', and if 'path' is
			a directory, then also 'chown' all (recursively)
			contained files and dirs.

=back

=cut

sub cifs_chown_file(%) {
    my (%opts) = validate_mixed(
        @_,
        [],
        {   client    => { type => SCALAR | HASHREF | OBJECT },
            user      => { type => SCALAR },
            path      => { type => SCALAR },
            recursive => { type => SCALAR, optional => 1, default => 0 },
        }
    );
    return eval {
        my ( $path, $user, $client )
            = ( $opts{path}, $opts{user}, $opts{client} );

        logDbg( "Enter cifs_chown_file\n\n" . Dumper( \%opts ) );
        my $cifsconn = make_conn( $client, "#rsh" );
        my $cifs_hostname = $cifsconn->hostp()->hostname();
        my $cifs_cmmd;

        if ( $opts{recursive} ) {
            $cifs_cmmd
                = "x:\\bin\\subinacl.exe /subdirectories $path /setowner\=$user";
        } else {
            $cifs_cmmd = "x:\\bin\\subinacl.exe /file $path /setowner\=$user";
        }

        my $output = execute_cmd( $cifs_hostname, $cifs_cmmd );

        chomp($output);
        $output =~ s/\s+/ /g;
        $output =~ s/^\s*//g;
        $output =~ s/\s*$//g;
        $output =~ s%[^:\w\ \t=\\\/\-\\._]%%g;

        if ( $output !~ /is the new owner/ ) {
            die(      "cifs_chown_file:  Could not change owner of\n"
                    . "file \"$path\" to \"$user\":\n\n$output" );
        }
        return (1);
    };
} ## end sub cifs_chown_file(%)

=head2 cifs_file_getowner (%opts)

	Get owner of file using CIFS

	All args are 'keyword' args

	Requires 'subinacl' to be available on CIFS client
	at:  "x:\bin\subinacl.exe"

=over 4

=item  client

	Required	CIFS client ('hostspec' string, or 'HOSTREC' or 'CONNECTREC' object)

=item path

	Required	Windows path of file whose owner is to be checked

=back

=cut

sub cifs_file_getowner(%) {
    my (%opts) = validate_mixed(
        @_,
        [],
        {   client => { type => SCALAR | HASHREF | OBJECT },
            path   => { type => SCALAR },
        }
    );
    return eval {
        my ( $path, $client ) = ( $opts{path}, $opts{client} );

        logDbg( "Enter cifs_file_getowner\n\n" . Dumper( \%opts ) );
        my $cifsconn = make_conn( $client, "#rsh" );
        my $cifs_hostname = $cifsconn->hostp()->hostname();
        my $cifs_cmmd;

        $cifs_cmmd = "x:\\bin\\subinacl.exe /file $path /display\=owner";

        my $output = execute_cmd( $cifs_hostname, $cifs_cmmd );

        chomp($output);
        $output =~ s/^\s*//;
        $output =~ s/\s*$//;
        $output =~ s%[^:\w\ \t=\\\/\-\\._]%%g;
        $output =~ s/\s+/ /g;
        if ( $output =~ /=\s*(\S+)/ ) {
            $output = $1;
        }
        return ($output);
    };
} ## end sub cifs_file_getowner(%)

#= write_file()
#=
#= Replacement for "mkfile", both "filer CLI" and "client/NFS based"
#=
#=  All args are 'keyword' args
#=
#=  Returns: status: 1 => success, 0 => failed.
#=
#=  This function strives to be "backwards compatible" with several other functions which it is
#=  intended to replace. This has resulted in a long options list, and several alternative "styles"
#=  of use. Also, there are a few "aliases" for option keywords, such as "path"/"file", and
#=  "content"/"contents".
#=
#=  The library fcn. FileManip::write_file is used to do all 'client-based' (NFS or CIFS) write
#=  operations. Only 'filer-based' writes are done directly by this fcn.
#=
#= :filer               Optional. Filer HOSTSPEC, HOSTREC, or CONNECTREC.
#=
#= :client              Optional. NFS Client HOSTSPEC, HOSTREC, or CONNECTREC (or string 'localhost').
#=
#= :cifs_client         Optional. CIFS Client HOSTSPEC, HOSTREC, or CONNECTREC
#=
#= :agent		Optional. One of "localhost", "client", "cifs_client", or "filer".
#=			Allows more than one of filer, client, and/or cifs_client options to be specified.
#=
#=  Either a 'path' or 'file' argument must be specified
#= :path                Required. Path to file to be created/wriiten
#= :file                Synonnym for 'path'
#=
#= :timeout		Optional.
#=
#= :size		Required. Size of files. e.g. "10m", "100k" "50g"
#=
#= :numfiles		Optionsal. If given, then number of files to create. Default is 1.
#=			Files will be named <PATH>_1, <PATH>_2, <PATH>_3
#=
#= :dont_overwrite	Optional. If '1', then new files are created, but existing files are not
#=			overwritten. Default is '1'.
#=
#= :content	     - optional: a named parameter with the initial contents
#=  			of the file. Exclusive with size, and overrides.
#=
#= :contents	     - A synonym for 'content'
#=
#= :debug_verbose    - Optional.  If '1', then more debug/diagnostic messages are given.
#=			  Default is '0'.
#=
#=
#= At most one of "filer" or "client" or "cifs_client" should be specified, unless the "agent"
#= option is also speciified:
#=
#=	Option		Agent Option		Effect
#=	--------------------------------------------------------------------------------------------------
#=	filer		"filer"		==>	Filer CLI "mkfile" will be used to create/write to file(s)
#=	client		"client"	==>	NFS client will be used to create/write to file(s)
#=	cifs_ client	"cifs_cleint"	==>	CIFS client will be used to create/write to file(s)
#=	(neither)	"localhost"	==>	Local NFS client will be used to create/write to file(s)
#=
#= If "filer" is specified, then the "path" should be of form "/vol/<VOLNAME>/..." (filer relative).
#= Otherwise, "path" should be for the mounted path on the specified client (or local client).
#=
#= As an alternative to the "path" option, a "vol_path" option may be specified instead. For a "filer"
#= type invocation, this has the same meaning as the "path" option. Howver, with any other type of
#= invocation, a "mountpoint" option must also be given in addition to the "vol_path" option. The
#= "client relative" path is then determined by stripping the "/vol/<VOL_NAME>" prefix from the vol_path,
#= and appending the result to the mountpoint. Not that for cifs clients, the mountpoint is a mapped drive,
#= e.g. "D:". The motivation for this is to easily switch between "filer" and non-filer agents, without having
#= to change the form of the path option to adopt to/from "filer relative" vs. "client relative" paths.
#= The "mountpoint" option is ignored for "filer" type invocations.
#=
#=	Option		Agent Option		Path options allowed
#=	--------------------------------------------------------------------------------------------------
#=	filer		"filer"			path || vol_path (mountpoint allowed but is ignored)
#=	client		"client"		path || (vol_path && mountpoint)
#=	cifs_client	"cifs_client"		path || (vol_path && mountpoint)
#=	(none)		"localhost"		path || (vol_path && mountpoint)
#=
#=
#=
#=         "Filer Only" Args -- The following args are ONLY valid if 'filer' option is specified.
#=
#= :step_size 	   - optioneal. The step-size-in-blocks specifies that mkfile should create files
#= 			with holes in them. mkfile writes one block of zeros for each
#=			step-size-in-blocks blocks. For example, if you use a step-size-in-blocks
#=			of 3, the resulting file will have one block of zeros followed by two empty
#=			(unallocated) blocks, followed by a block of zeros and 2 more empty blocks,
#=			and so on. The default step-size-in-blocks is 1. The step_size option can
#=			only be used in "filer-type" invocations, and in conjunction with the
#=			"numfiles" option.
#=
#=
#=
#=         "Client Only" Args -- The following args are NOT valid if 'filer' option is specified.
#=
#= :buffer_size	     - Optional. Size of data buffer to be used for writing to the file.
#=			Large files need to be written in multiple operations, due to
#=			limitatitations on max string length imposed by memory constraints.
#=			If 'buffer_size' is given, and file size is larger than buffer_size,
#=			then multiple write operations will be used to write to the file,
#=			and all operations will be in increments of "buffer_size" or less.
#=
#=			The 'write_data_fcn' option may be used in conjunction with the
#=			'buffer_size' option, to allow for more flexible data string
#=			creation. If used with 'contents' option, and if buffer_size is
#=			less than size of "contents" string, then multiple write operations
#=			will be used to write the file.
#=
#=			Defualt is 10Mbyte.
#=
#=
#= :file_list	     - optional. If given, an ARRAYREF. The paths of all files written will be
#=                       appended to this list. The list should be initialized (e.g to "empty list")
#=                       prior to calling this fcn.
#=
#= :uid		     - Optional. If given, this UID will be assigned to the file(s).
#=
#= :gid		     - Optional. If given, this GID will be assigned to the file(s).
#=
## :mode             - optional: a named parameter with the initial mode bits.
#=		       If cifs is being used, the mode bits - +R | -R
#=		       (R - readonly)
#=
#= :fstat	     - Optional.
#=			  	If '0', then scalar status is returned --
#=					1 -> success, 0 => failure.
#=			  	If '1', then list of attrs is returned (@attrs)
#=			  	Default is '0'.
#=
#= :verify	     - Optional.  If '1', then verify existance and size of files written.
#=			  Default is '0'.
#=
#= :catch_errors     - Optionsal.  If '1', then some errors (those that occur during the 'write' operation)
#=				         are caught, and '$@' is set with an error message.
#=					 Errors such as invalid parameter values are not caught.
#=				   Otherwise, the routine will 'die' on all errors.
#=				   Default: '0' (die on error).
#=
#=
#= :write_data_fcn   - optional: Valid only if 'buffer_size' option is also given.
#=			If given, a "function ref" for a function which takes
#=			the following keyword options/args, and returns a "data string"
#=			which will be written to the file. The write_data_fcn is just
#=			responsible for creating the "data string" to be written, it
#=			does not itself perform any I/O.
#=
#=			If no "write_data_fcn" is specified, then a simple "default" fcn.
#=			is used. The behavior of the "default" fcn is not specified, it
#=			may simply return a block of identical chars.
#=
#=			"content", "contents", and "write_data_fcn" are all mutually
#=			exclusive options.
#=
#=			The following options will be passed to the fcn, it can ignore any except
#=			"write_len". The other options are useful for passing 'state' to the
#=			write_data_fcn, to enable it to return different values of "data string"
#=			when called multiple times (but this is not required).
#=
#=				write_len:	Length of data to write, in bytes. The "write_data_fcn"
#=					        is required to honor this option.
#=
#=				'Convenience Options' -- may be ignored.
#=
#=				buffer_size:	Buffer size, in bytes. For all write
#=					        operations except the last one, 'write_len'
#=					        will equal 'buffer_size'. For the last write,
#=						'write_len' may be less than the buffer size
#=					        (file_length mod buffer_size).
#=
#=				file_size:	Size (in bytes) of file to be created.
#=
#=				file_path:	Absoluete path of file to be created.
#=
#=				count:		Number of write operations prior to this one
#=						(i.e. zero-based 'invocation' count)
#=						    first call to 'write_data_fcn'   ==> 0,
#=						    second call "     "   "    "     ==> 1, etc.
#=
#=				end_count:	The count value for the last write operation.
#=
#=				offset:		The starting address of the file into which the
#=						data is to be written for this write operation. (bytes).
#=
#=				start_offset:	The offset into the file for the first write operation.
#=
#=				special_volume_options:	 See 'special_volume_options' arg for 'write_file'
#=						         below.
#=
#=				io_size:	The 'prefered I/O size' (block size) of the
#=						filesystem hosting the file(s) to be written.
#=
#=				write_data_args  Only applies if 'write_data_args' option was provided for
#=						 'write_file'.  User-defined data. Is commonly a HASHREF
#=						 for user-defined info to be used by the write_data_fcn,
#=						 byt may be any scalar value chosen by the user.
#=
#=				scramble	 (see below)
#=
#=
#= :write_data_args  - Optional. Ignored unless 'write_data_fcn' option is also given. If supplied, this
#=		       value is passed to the 'write_data_fcn' via the keyword 'write_data_args'.
#=			This arg is not used by the 'write_file' fcn. itself. It may be any scalar
#=			value, but would typically be a HASHREF containing various args to be
#=			used by the 'write_data_fcn'.
#=
#=
#= :scramble         - optional:
#=				1) For user-defined write_data_fcn, the behavior is determined by that fcn.
#=				   -- the fcn may simply ignore this arg.
#=
#=				2. For 'default_write_data_fcn', the behavior is the following:
#=			 	    'Approximates'  behavior of 'SCRAMBLE' parameter of DataManip::populate"
#=				    function.   See: http://wikid.netapp.com/w/DataManip for more info on
#=				    "populate" use of "SCRAMBLE". The main difference is that for 'scramble'
#=				    values > 1, 'populate' treats this value as an absolute number, whereas
#=				    here the number is treated as a percent value.
#=
#=				    > If scramble is '0.0' (default):
#=					Then every block written is identical. (100% de-dupable and compressible)
#=
#=				    > If scramble is in range: 0.0 - 1.0:
#=					Then 'scramble' represents the fraction of blocks that will be unique
#=					(diffferent from every other block of the given file). The fraction of
#=					"common" (de-dupable) blocks in the file is (1 - scramble)
#=						e.g.    scramble=0.15  ==>  15% of blocks will be unique
#=					-- Compressible, de-dup resistance depends on 'scramble' value
#=
#=				    > If scramble is greater than 1.0:
#=					Then the 'int' part of scramble gives the percentage of random bytes
#=					in each block.
#=						e.g.    scramble=30    ==>  30% of bytes in each block will be random
#=					-- De-dup resistant, compressiblity depends on 'scramble' value.
#=
#=
#= :special_volume_options  - Optional.  If given, this is a HASHREF with zero or more
#=				"special volume options" specified. If not supplied,
#=				then the corresponding global parameter values are used.
#=				That is:
#=					$default_special_volume_options = {
#=						compression_enable    => $::COMPRESSION_ENABLE,
#=						asis_enable           => $::ASIS_ENABLE,
#=						encryption_enable     => $::ENCRYPTION_ENABLE,
#=						large_aggr	      => $::LARGE_AGGR,
#=					};
#=				If a 'special_volume_options' HASHREF arg is supplied, then the
#=				'default_special_volume_options' above is used to supply any
#=				fields not set in the HASHREF.
#=
#=				These options are not used by the 'write_fcn' function itself,
#=				but are passed on to the 'write_data_fcn'.
#=
sub write_file(%) {

    my (%args) = validate(
        @_,
        {

            #=========  'Common' (filer or client) options:  ==========
            filer =>
                { type => SCALAR | HASHREF | OBJECT | UNDEF, optional => 1 },
            client =>
                { type => SCALAR | HASHREF | OBJECT | UNDEF, optional => 1 },
            cifs_client =>
                { type => SCALAR | HASHREF | OBJECT | UNDEF, optional => 1 },
            agent      => { type => SCALAR,         optional => 1 },
            path       => { type => SCALAR,         optional => 1 },
            file       => { type => SCALAR,         optional => 1 },
            vol_path   => { type => SCALAR | UNDEF, optional => 1 },
            mountpoint => { type => SCALAR | UNDEF, optional => 1 },

            #
            # Exactly 1 of 'size', 'contents', or 'content' must be given
            size     => { type => SCALAR, optional => 1 },
            content  => { type => SCALAR, optional => 1 },
            contents => { type => SCALAR, optional => 1 },

            #
            numfiles => { type => SCALAR | UNDEF, optional => 1 },
            dont_overwrite =>
                { type => SCALAR | UNDEF, optional => 1, default => 0 },
            timeout => { type => SCALAR | UNDEF, optional => 1 },
            debug_verbose => { type => BOOLEAN, optional => 1, default => 0 },

            #=========  'Client Only' options:  ==========
            catch_errors => { type => BOOLEAN, optional => 1, default => 0 },
            mode         => { type => SCALAR,  optional => 1 },
            uid          => { type => SCALAR,  optional => 1 },
            gid          => { type => SCALAR,  optional => 1 },
            offset       => { type => SCALAR,  optional => 1 },
            buffer_size => { type => SCALAR, optional => 1, default => "2m" },
            file_list => { type => ARRAYREF, optional => 1 },
            user      => { type => SCALAR,   optional => 1 },
            fstat     => { type => BOOLEAN,  optional => 1, default => 1 },
            verify    => { type => BOOLEAN,  optional => 1, default => 0 },
            scramble => { type => SCALAR, optional => 1, default => undef },

    #
    # 'write_data_args' if given, can be any type. Used with 'write_data_fcn'.
            write_data_fcn  => { type     => CODEREF, optional => 1 },
            write_data_args => { optional => 1 },

            #
            #
            #  'random' option is deprecated -- use 'scramble' option instead
            random => { type => BOOLEAN, optional => 1 },    # Deprecated
                                                             #
            special_volume_options =>
                { type => HASHREF, optional => 1, default => {} },

            #=========  'Filer Only' options:  ==========
            step_size => { type => SCALAR, optional => 1 },
        }
    );

#===========================================================================================
    my $default_write_data_fcn = sub(%) {
        my (%args) = @_;

        # -------------- 'generic' write_data_fcn args -------------
        my $write_len = $args{write_len};    # Length of data to write
                                             #
             #    Contract:  write_data_fcn must return string/buffer of
             #		length 'write_len' bytes
             #	        'write_len' will always be <= 'buffer_size'
             #
          #    #  If 'file_size' is greater than 'buffer_size', then this fcn. will
          #       will be invoked multiple times.
          #
          #    *  'write_len' will be equal to 'buffer_size' for all invocations
          #       except (possibly) the last, when 'buffer_size' will be equal to
          #       'file_size' mod 'buffer_size.
          #
          #    #  Total length wriiten prior to this invocation is equal to
          #       'count' times 'buffer_size'
          #
          #    *  Following args are for 'convenience' of user-defined 'write_data_fcn',
          #       they may be ignored.
          #
        my $buffer_size = $args{buffer_size};   # user specified 'buffer_size'
        my $file_size   = $args{file_size};     # user specified 'file_size'
        my $offset      = $args{offset};        # offset into file (bytes)
        my $start_offset
            = $args{start_offset};    # offset into file of first invocation
        my $count
            = $args{count}; # "invocation count" (zero based) for this invocation
        my $end_count
            = $args{end_count};    # "invocation count" of final invocation.
        my $special_volume_options
            = $args{special_volume_options}; # See 'special_volume_options' param, above.
            #   Currently: asis_enable, compression_enable,
            #   encryption_enable, large_aggr.
        my $io_size = $args{io_size};    # 'prefered I/O size'
        my $scramble
            = $args{scramble};    # (see info for 'scramble' param above)
        my $file_path     = $args{file_path};    # Path of file to be written.
        my $debug_verbose = $args{debug_verbose};

        # ------- 'custom' (user-defined) write_data_fcn args -------
        my $write_data_args = $args{write_data_args};
        my $contents        = $write_data_args->{contents};
        my $contents_len    = undef;

  #  Most of following code is for implementing behavior related to 'scramble'
  #  parameter.
        my ( $msg, $msg1 );
        my $num_key_char = 2;

        #  'orig_blk_size' gives size of (filer) data block
        #      (except for small values of 'write_len')
        #  'curr_blk_size' equals 'orig_blk_size', except possibly for last
        #      pass thru 'for' loop starting around line 4205.
        my $orig_blk_size
            = ( ( $io_size < $write_len ) ? $io_size : $write_len );
        my $curr_blk_size = $orig_blk_size;
        my $blk_buff_len;
        my $blk_buff = '';
        my $rtn_buff = '';
        my $RAND     = undef;
        my $common_key;
        my $n_rand;
        my $n_width;
        my $key;
        my $n_read;
        my $saved_data_args;
        my $saved_data_args_copy;

        if ( $count == 0 ) {
            delete( $write_data_args->{saved_data_args} );
        }

        $scramble = 0 if ($contents);
        if ( !defined($scramble) ) {
            if ( $special_volume_options->{compression_enable} ) {
                $scramble = 20;
                logDbg(   "default_write_data_fcn  Line: " 
                        . __LINE__
                        . "\n\n'compression_enable' special_volume_option is set --\n"
                        . "   Setting (undefined) 'scramble' value to \"$scramble\""
                );
            } elsif ( $special_volume_options->{asis_enable} ) {
                $scramble = 0.4;
                logDbg(   "default_write_data_fcn  Line: " 
                        . __LINE__
                        . "\n\n'asis_enable' special_volume_option is set --\n"
                        . "   Setting (undefined) 'scramble' value to \"$scramble\""
                );
            } elsif ( $args{random} ) {
                $scramble = 30;
                logDbg(   "default_write_data_fcn  Line: " 
                        . __LINE__
                        . "\n\n'random' option is set --\n"
                        . "   Setting (undefined) 'scramble' value to \"$scramble\""
                );
            } else {
                $scramble = 0;
            }
        } elsif ( !$scramble ) {
            $scramble = 0;
        } elsif ( int($scramble) > 1.0 ) {
            $scramble = int($scramble);
            $scramble = 100 if ( $scramble > 100 );
        } elsif ( $scramble < 0.0 ) {
            $scramble = 0;
        }

        my $args_copy = {%args};
        if ( $contents && length($contents) > 100 ) {
            my $len = length($contents);
            $args_copy->{contents} = "## Data Length: $len bytes ##";
        } elsif ($contents) {
            $args_copy->{contents} = $contents;
        } else {
            $args_copy->{contents} = "";
        }
        $args_copy->{write_data_args} = "{  write_data_args ... }";
        if ($debug_verbose) {
            logDbg(   "default_write_data_fcn  Line: " 
                    . __LINE__
                    . "   Args:\n\n"
                    . Dumper($args_copy)
                    . "\nScramble (final value):\t$scramble" );
        } ## end if ($debug_verbose)

#  I.  if 'contents' is given, then just return the appropriate substring of 'contents'....
        if ($contents) {
            my $contents_offset;
            my $contents_len;
            if ( $write_data_args->{saved_data_args} && ( Y $count > 0 ) ) {

                #  Get saved values -- set during previous invocation
                $saved_data_args = $write_data_args->{saved_data_args};
                $contents_offset = $saved_data_args->{contents_offset};
                $contents_len    = $saved_data_args->{contents_len};
                $contents        = $saved_data_args->{contents};
            } else {
                $contents_offset                    = 0;
                $saved_data_args->{contents}        = $contents;
                $saved_data_args->{contents_offset} = $contents_offset;
                $contents_len                       = length($contents);
                $saved_data_args->{contents_len}    = $contents_len;
            } ## end else [ if ( $write_data_args->...
            if ( $contents_offset == $contents_len ) {
                logDbg(   "default_write_data_fcn  Line: " 
                        . __LINE__
                        . "\ncontents_offset=$contents_offset\n"
                        . "equals  contents_len=$contents_len\n ... so exiting"
                );
                return ("");
            } elsif ( $contents_offset > $contents_len ) {
                die(      "default_write_data_fcn  Line: " 
                        . __LINE__
                        . "\n\nContents_offset=$contents_offset\n"
                        . "    is greater than \nContents_len=$contents_len"
                );
            } elsif ( ( $write_len + $contents_offset ) > $contents_len ) {
                $write_len = $contents_len - $contents_offset;
            }
            $rtn_buff = substr( $contents, $contents_offset, $write_len );
            $saved_data_args->{contents_offset} += $write_len;
            my $rtn_buff_len = length($rtn_buff);
            if ($debug_verbose) {
                logDbg(   "default_write_data_fcn  Line: " 
                        . __LINE__ . "\n\n"
                        . "Contents_offset:   \"$contents_offset\"\n"
                        . "Contents_len:      \"$contents_len\"\n"
                        . "Write_len:         \"$write_len\"\n"
                        . "length(rtn_buff):  \"$rtn_buff_len\"\n\n... Returning rtn_buff\n\n"
                        . "" );
            } ## end if ($debug_verbose)
            return ($rtn_buff);
        } ## end if ($contents)

#  II.  if 'contents' is not given, then we need to generate appropriate data,
#        including "scrambling" when appropriate.

        #  First Initialize local args.
        #  Open 'random' filehandle 'RAND' for use later.
        sysopen( $RAND, "/dev/urandom", O_RDONLY )
            or die( "default_write_data_fcn  Line: " 
                . __LINE__
                . "\nCould not open '/dev/urandom' for reading: $!" );

        unless ( defined($RAND) ) {
            die(      "default_write_data_fcn  Line: " 
                    . __LINE__
                    . "\n\n'/dev/urandom' filehandle 'RAND' is undefined" );
        }
        unless ( stat(*$RAND) ) {
            die(      "default_write_data_fcn  Line: " 
                    . __LINE__
                    . "\n\n'Stat' failed on '/dev/urandom' filehandle 'RAND'"
            );
        } ## end unless ( stat(*$RAND) )

        if ( $write_data_args->{saved_data_args} && ( $count > 0 ) ) {

            #  Get saved values -- set during previous invocation
            $saved_data_args = $write_data_args->{saved_data_args};
            $common_key      = $saved_data_args->{common_key};
            $n_rand          = $saved_data_args->{n_rand};
            $n_width         = $saved_data_args->{n_width};
            $blk_buff        = $saved_data_args->{blk_buff};
        } else {

            #  Compute values and store these in 'saved_data_args' for
            #  subsequent incocations.
            $saved_data_args                    = {};
            $write_data_args->{saved_data_args} = $saved_data_args;
            $saved_data_args->{orig_blk_size}   = $orig_blk_size;
            $saved_data_args->{curr_blk_size}   = $curr_blk_size;
            $saved_data_args->{file_path}       = $file_path;

            #  Initialize remaining vars --
            #     a. Vars used for 'scramble' value 0 < x < 1
            #     b. Vars used for 'scramble' values x > 1
            if ( $scramble && ( $scramble > 0.0 ) && ( $scramble <= 1.0 ) ) {

                #  Vars for scrable values  x < 1
                $n_read
                    = sysread( *$RAND, ( $common_key = '' ), $num_key_char );
                if ( $n_read != $num_key_char ) {
                    die(      "default_write_data_fcn  Line: " 
                            . __LINE__
                            . "\n\nError reading from /dev/urandom\n"
                            . "Expected to read $num_key_char bytes, "
                            . "actually read $n_read bytes:\n> $!\n" );
                } ## end if ( $n_read != $num_key_char)
                $saved_data_args->{num_key_char} = $num_key_char;
                $saved_data_args->{common_key}   = $common_key;
            } elsif ( $scramble && ( $scramble > 1.0 ) ) {

                #  Vars for scrable values  x > 1
                $n_rand = ( $curr_blk_size * int($scramble) ) / 100;
                if (   ( $n_rand - int($n_rand) > .4 )
                    && ( $n_rand < $curr_blk_size ) )
                {
                    $n_rand += 1;
                }
                $n_rand = int($n_rand);
                $saved_data_args->{n_rand} = $n_rand;
                $n_width = int( ( $curr_blk_size - 1 ) / $n_rand );
                $saved_data_args->{n_width} = $n_width;
            } ## end elsif ( $scramble && ( $scramble...

            #  Generate single 'random' block 'blk_buff' which will be used as
            #  starting point for all blocks to be written, with changes to
            #  individual blocks determined by value of 'scramble' parameter.
            $blk_buff = '';
            $rtn_buff = '';
            for (
                ( my $rem_to_read = $curr_blk_size, my $total_read = 0 );
                ( $rem_to_read > 0 );
                ( $rem_to_read -= $n_read, $total_read += $n_read )
                )
            {
                my $tmp_buff = '';

    # $n_read = sysread(*$RAND, substr($blk_buff, $total_read), $rem_to_read);
                $n_read = sysread( *$RAND, $tmp_buff, $rem_to_read );
                unless ( defined($n_read) && ( $n_read > 0 ) ) {
                    $msg
                        = "default_write_data_fcn  Line: " 
                        . __LINE__
                        . "\n\nError reading from /dev/urandom\n"
                        . "Expected to read $curr_blk_size bytes, "
                        . "actually read $n_read bytes:\n> $!\n";
                    logDbg($msg);
                    die($msg);
                } ## end unless ( defined($n_read) ...
                $blk_buff .= $tmp_buff;
            } ## end for ( ( my $rem_to_read...
            if ( length($blk_buff) != $curr_blk_size ) {
                die(      "default_write_data_fcn  Line: " 
                        . __LINE__
                        . "\n\nExpected  length(blk_buff)="
                        . length($blk_buff) . "\n"
                        . "    to equal curr_ blk_size=$curr_blk_size\n" );
            } ## end if ( length($blk_buff)...

            #  Store copy of 'blk_buff' (data/string) in 'saved_data_args'
            $saved_data_args->{blk_buff} = $blk_buff;

            #  -- Finished generating 'blk_buff'
        } ## end else [ if ( $write_data_args->...

      #  -- Finished initializing args, or reading them from 'saved_data_args'

        $blk_buff_len                    = length($blk_buff);
        $saved_data_args->{blk_buff_len} = $blk_buff_len;
        $saved_data_args_copy            = {%$saved_data_args};
        $saved_data_args_copy->{blk_buff}
            = "< BLOCK BUFFER  ($blk_buff_len bytes) .... >";
        if ( $args{debug_verbose} ) {
            logDbg(   "default_write_data_fcn  Line: " 
                    . __LINE__
                    . "\n\nSaved Data Args:\n\n"
                    . Dumper($saved_data_args_copy) );
            logDbg(   "default_write_data_fcn  Line: " 
                    . __LINE__
                    . "\n\nI/O Size:\t$io_size\nWrite Len:\t$write_len\n"
                    . "Orig blk size:\t$orig_blk_size\nCurr blk size:\t$curr_blk_size\n"
                    . "length(blk_buff):\t"
                    . length($blk_buff) . "\n"
                    . "N_rand:\t$n_rand\nN_width:\t$n_width\nNum key char:\t$num_key_char\n"
                    . "Scramble:\t$scramble" );
        } ## end if ( $args{debug_verbose...
        if ( length($blk_buff) != $curr_blk_size ) {
            die(      "default_write_data_fcn  Line: " 
                    . __LINE__
                    . "\n\nExpected  length(blk_buff)="
                    . length($blk_buff) . "\n"
                    . "    to equal curr_ blk_size=$curr_blk_size\n" );
        } ## end if ( length($blk_buff)...

       #   Modify 'blk_buff' as determined by 'scramble' value -- three cases:
       #     a. 'scramble' value 0 < x < 1
       #     b. 'scramble' values x > 1
       #     c. 'scramble' value == 0.0
       #
       #   Loop for inserting 'random' bytes in blocks
       #   Handle data in 'curr_blk_size' chunks
        for (
            (   my $rem_chars   = $write_len,
                my $blkcnt      = 0,
                my $rand_blkcnt = 0, my $wrlen = $curr_blk_size
            );
            ( $wrlen > 0 );
            (   $rem_chars -= $wrlen,
                $blkcnt++,
                $wrlen = ( ( $io_size < $rem_chars ) ? $io_size : $rem_chars )
            )
            )
        {

            $curr_blk_size
                = ( $wrlen < $curr_blk_size ) ? $wrlen : $curr_blk_size;

            if ( $args{debug_verbose} ) {
                logDbg(   "default_write_data_fcn  Line: " 
                        . __LINE__
                        . "\n\nTop-of-loop\n\nBlk cnt:\t$blkcnt\nRem chars:\t$rem_chars\n"
                        . "Write len\t$write_len\nOrig blk size:\t$orig_blk_size\n"
                        . "Wrlen:\t$wrlen\nCurr blk size:\t$curr_blk_size\n"
                        . "Rand blk cnt:\t$rand_blkcnt" );
            } ## end if ( $args{debug_verbose...

            if ( $curr_blk_size < $orig_blk_size ) {

                #  Set 'curr_blk_size' for last pass thru loop
                my $old_blk_buff_len = length($blk_buff);

                # $blk_buff = substr($blk_buff, $curr_blk_size) = '';
                $blk_buff = substr( $blk_buff, 0, $curr_blk_size );
                $blk_buff_len = length($blk_buff);
                if ( $args{debug_verbose} ) {
                    logDbg(   "default_write_data_fcn  Line: " 
                            . __LINE__
                            . "\n\nCalling 'substr' on:\n"
                            . "String:     blk_buff\n"
                            . "Length:     $old_blk_buff_len\n"
                            . "Offest:     $curr_blk_size (curr_blk_size)\n"
                            . "Orig blk size: $orig_blk_size\n\n"
                            . "New blk_buff len: $blk_buff_len" );
                } ## end if ( $args{debug_verbose...

                # $saved_data_args->{blk_buff} = $blk_buff;
                $saved_data_args->{curr_blk_size} = $curr_blk_size;
            } ## end if ( $curr_blk_size < ...

            if ( $blk_buff_len != $curr_blk_size ) {
                die(      "default_write_data_fcn  Line: " 
                        . __LINE__
                        . "\n\nExpected length(blk_buff)=$blk_buff_len\n"
                        . "   to equal  curr_blk_size=$curr_blk_size\n" );
            } ## end if ( $blk_buff_len != ...
            if ( $args{debug_verbose} ) {
                logDbg(   "default_write_data_fcn  Line: " 
                        . __LINE__
                        . "\n\nBlk_cnt:\t$blkcnt\nRand_blkcnt:\t$rand_blkcnt\n"
                        . "Rem_chars:\t$rem_chars\nWrlen:\t$wrlen\n" );
            } ## end if ( $args{debug_verbose...

            if ( $scramble && ( $scramble > 0.0 ) && ( $scramble <= 1.0 ) ) {

                #  a. 'scramble' value 0 < x < 1
                last unless ( $wrlen > $num_key_char );

                # my $rand_val = rand();
                my $rand_val = ( ord( getc(*$RAND) ) / 255 );
                if ( $rand_val < $scramble ) {
                    $rand_blkcnt++;
                    $n_read = sysread( *$RAND, ( $key = '' ), $num_key_char );
                    if ( $n_read != $num_key_char ) {
                        $msg
                            = "default_write_data_fcn  Line: " 
                            . __LINE__
                            . "\n\nError reading from /dev/urandom\n"
                            . "Expected to read $num_key_char bytes, "
                            . "actually read $n_read bytes:\n> $!\n";
                        logDbg($msg);
                        die($msg);
                    } ## end if ( $n_read != $num_key_char)
                    if ( $args{debug_verbose} ) {
                        logDbg(   "default_write_data_fcn  Line: " 
                                . __LINE__
                                . "\n'Random' block added..." );
                    }

                } else {
                    $key = $common_key;
                }
                substr( $blk_buff, 0, $num_key_char ) = $key;

                if ( $args{debug_verbose} ) {
                    logDbg(
                              "default_write_data_fcn  Line: " 
                            . __LINE__
                            . "\n\nScramble:\t$scramble\nBlk_cnt:\t$blkcnt\n"
                            . "Rand_blkcnt: $rand_blkcnt\nRand Val:\t$rand_val\n"
                            . "Rand ratio: "
                            . sprintf( "%.3f",
                            ( $rand_blkcnt / ( 1 + $blkcnt ) ) )
                    );
                } ## end if ( $args{debug_verbose...

                #  End of case: 'scramble' value 0 < x < 1
            } elsif ( $scramble && ( $scramble > 1.0 ) ) {

                #  b. 'scramble' values x > 1
                if ( $args{debug_verbose} ) {
                    logDbg(   "default_write_data_fcn  Line: " 
                            . __LINE__ . "\n\n"
                            . "Scramble:\t$scramble\nWrite Len:\t$write_len\n"
                            . "I/O Size:\t$io_size\nCurr Block size:\t$curr_blk_size\n"
                            . "Wrlen:\t$wrlen\nN_rand:\t$n_rand\nN_width:\t$n_width"
                    );
                } ## end if ( $args{debug_verbose...
                if ( $curr_blk_size < $orig_blk_size ) {
                    $n_rand = ( $curr_blk_size * int($scramble) ) / 100;
                    if (   ( $n_rand - int($n_rand) > .4 )
                        && ( $n_rand < $curr_blk_size ) )
                    {
                        $n_rand += 1;
                    }
                    $n_rand = int($n_rand);
                } ## end if ( $curr_blk_size < ...
                if ( $n_rand >= 1 ) {
                    $n_width = int( ( $curr_blk_size - 1 ) / $n_rand );
                    if ( $args{debug_verbose} ) {
                        logDbg(   "default_write_data_fcn  Line: " 
                                . __LINE__
                                . "\n\nN_rand:\t$n_rand\nN_width:\t$n_width\n"
                        );
                    } ## end if ( $args{debug_verbose...
                    for (
                        ( my $ofst = 0, my $rndcnt = 0 );
                        ( $rndcnt < $n_rand && $ofst < $curr_blk_size );
                        ( $rndcnt++, $ofst += $n_width )
                        )
                    {
                        if ( $args{debug_verbose} ) {

                     # logDbg("default_write_data_fcn  Line: ".__LINE__.
                     #         "\n\nN_rand:\t$n_rand\nRnd_cnt:\t$rndcnt\n".
                     #         "Wrtlen:\tOffset:\t$ofst\nN_width:\t$n_width");
                        } ## end if ( $args{debug_verbose...
                        my $chr = getc(*$RAND);
                        substr( $blk_buff, $ofst, 1 ) = $chr;
                    } ## end for ( ( my $ofst = 0, my...
                } ## end if ( $n_rand >= 1 )

            } else {

                #  c. 'scramble' value == 0.0
                #  End of case: 'scramble' values x > 1
                #  No aditional action needed for 'non-scrambled' case.
                #  -- just use 'blk_buff' as-is
            } ## end else [ if ( $scramble && ( $scramble...

            $rtn_buff .= $blk_buff;
            if ( $args{debug_verbose} ) {
                logDbg(   "default_write_data_fcn  Line: " 
                        . __LINE__
                        . "\n\n  Blk cnt:\t$blkcnt\nLength(rtn_buff):\t"
                        . length($rtn_buff)
                        . " bytes" );
            } ## end if ( $args{debug_verbose...

        }    # for (...)

       #  Close 'RAND' filehandle before exiting, each invocation re-opens it.
        if ( $count == $end_count ) {
            close(*$RAND) if ($RAND);
        }
        logDbg(   "default_write_data_fcn  Line: " 
                . __LINE__
                . "\n\n Returning:  "
                . length($rtn_buff)
                . " bytes" );
        return ($rtn_buff);
    };

#===========================================================================================

#=================================================================================================
    my $write_file_aux = sub() {

#
#       %args
#	          offset, $buffer_size, file_list, fstat, rtn_fstat, special_volume_options
#	          contents, size_bytes, io_size, write_data_fcn, write_data_args,
#

        return eval {
            my ($argsref) = @_;
            my ( $msg, $msg1, $rtn_status );

            my $write_file_subr_args = $argsref->{write_file_subr_args};
            my $file_path            = $write_file_subr_args->{file};

            my $dont_overwrite  = $argsref->{dont_overwrite};
            my $size_bytes      = $argsref->{size_bytes};
            my $raw_offset      = $argsref->{offset} || 0;
            my $raw_buffer_size = $argsref->{buffer_size},
                my $file_list   = $argsref->{file_list};
            my $contents               = $argsref->{contents};
            my $scramble               = $argsref->{scramble};
            my $io_size                = $argsref->{io_size};
            my $write_data_fcn         = $argsref->{write_data_fcn};
            my $write_data_args        = $argsref->{write_data_args};
            my $rtn_fstat              = $argsref->{rtn_fstat};
            my $special_volume_options = $argsref->{special_volume_options};
            my $debug_verbose          = $argsref->{debug_verbose};

            my $fstat    = $argsref->{fstat};
            my @attrs    = ();
            my $truncate = 0;

            if ($debug_verbose) {
                my $fake_contents = "";
                my $len           = length($contents);
                if ( $contents && $len > 100 ) {
                    $fake_contents = "## Data Length: $len ##";
                } elsif ($contents) {
                    $fake_contents = $contents;
                } else {
                    $fake_contents = "";
                }
                my $args_tmp = {%$argsref};
                $args_tmp->{contents} = $fake_contents;
                logDbg(   "Enter write_file_aux  Line: " 
                        . __LINE__ . "\n"
                        . Dumper($args_tmp) );
            } ## end if ($debug_verbose)

            my $write_file_subr_args_copy = {%$write_file_subr_args};
            delete( $write_file_subr_args->{contents} );
            my $size_bytes_aux = $size_bytes;
            if ( $size_bytes_aux == 0 ) {
                logDbg(   "write_file_aux  Line: " 
                        . __LINE__
                        . "\nZero 'size_bytes' given, will use '1' instead, and then truncate to zero"
                );
                $size_bytes_aux = 1;
                $truncate       = 1;
            } ## end if ( $size_bytes_aux ==...

            if ( !$raw_buffer_size ) {
                die(      "Utilities::write_file_aux:  Line: " 
                        . __LINE__
                        . "\n'buffer_size' must be defined, and greater than zero"
                );
            } ## end if ( !$raw_buffer_size)
            my $buffer_size = convert_to_bytes($raw_buffer_size);
            if ( !$buffer_size || $buffer_size <= 0 ) {
                die(      "Utilities::write_file_aux  Line: " 
                        . __LINE__
                        . "\nInvalid 'buffer_size' value: \"$raw_buffer_size\""
                );
            } ## end if ( !$buffer_size || ...
            my $start_offset = convert_to_bytes($raw_offset);
            if (   !defined($start_offset)
                || ( $start_offset ne '0' && !$start_offset )
                || ( $start_offset < 0 ) )
            {
                die(      "Utilities::write_file_aux  Line: " 
                        . __LINE__
                        . "\nInvalid 'offset' value: \"$raw_offset\"" );
            } ## end if ( !defined($start_offset...

            my $end_count = int( $size_bytes_aux / $buffer_size ) - 1;
            my $rem_size  = $size_bytes_aux % $buffer_size;
            $end_count++ if ($rem_size);

            my $len_to_write = (
                $rem_size
                ? ( ( $end_count * $buffer_size ) + $rem_size )
                : ( ( $end_count + 1 ) * $buffer_size )
            );

            $msg1
                = "    File Path:        \"$file_path\"\n"
                . "    dont_overwrite:   "
                . ( $dont_overwrite ? 1 : 0 ) . "\n"
                . "    File Size:        $size_bytes_aux\n\nOpts:\n"
                . "    (Start) Offset:   $raw_offset\n"
                . "    Buffer Size:      $raw_buffer_size\n"
                . "    End Count:        $end_count\n"
                . "    Rem Size:         $rem_size\n"
                . "    Length to write (total) $len_to_write\n" . "";

            if ( $argsref->{debug_verbose} ) {
                logDbg(   "write_file_aux  Line: " 
                        . __LINE__
                        . "\n\n$msg1 \n\n'write_file_subr' args:\n"
                        . Dumper($write_file_subr_args) );
            } ## end if ( $argsref->{debug_verbose...

            if ( $len_to_write != $size_bytes_aux ) {
                $msg
                    = "write_file_aux  Line: " 
                    . __LINE__
                    . "\n\n$msg1 \n\n"
                    . "Internal 'write_file' error:\n"
                    . "'Length_to_be_written' =\n"
                    . "       ( (end_count=$end_count * buffer_size=$buffer_size)  +"
                    . "  rem_size=$rem_size )\n"
                    . "                       = $len_to_write\n\n"
                    . "     does not equal 'File_Size'=$size_bytes_aux";
                logcomment($msg);
                die($msg);
            } ## end if ( $len_to_write != ...

            my %write_data_fcn_args = (
                special_volume_options => $special_volume_options,
                io_size                => $io_size,
                scramble               => $scramble,
                write_data_args        => $write_data_args,
                start_offset           => $start_offset,
                end_count              => $end_count,
                buffer_size            => $buffer_size,
                file_size              => $size_bytes_aux,
                file_path              => $file_path,
                debug_verbose          => $debug_verbose,
            );

            if ($dont_overwrite
                && FileManip::stat_file(
                    file               => $file_path,
                    client             => $write_file_subr_args->{client},
                    okay_if_not_exists => 1
                )
                )
            {
                $msg
                    = "Utilities::write_file_aux  Line: " 
                    . __LINE__
                    . "\n'dont_overwrite' is set and file "
                    . "$file_path exists, not overwriting";
                logDbg("write_file_aux  Error: $msg");

                #return($msg);
                die($msg);
            } else {
                my $wrlen;
                my $data_str_ref = \$write_file_subr_args->{contents};
                my $count        = 0;
                my $len_written  = 0;
                my $offset       = $start_offset;
                for (
                    $count = 0;
                    $count <= $end_count;
                    ( $count++, $offset += $wrlen, $len_written += $wrlen )
                    )
                {
                    $wrlen = (
                        ( ( $count == $end_count ) && $rem_size )
                        ? $rem_size
                        : $buffer_size
                    );

                    my $msg1
                        = "Buffer_size:     $buffer_size\n"
                        . "File size        $size_bytes_aux\n"
                        . "Count:           $count\n"
                        . "End_count:       $end_count\n"
                        . "Offset:          $offset\n"
                        . "Start_offset:    $start_offset\n"
                        . "Bytes written:   $len_written\n"
                        . "Writelen:        $wrlen\n"
                        . "File Path:       \"$file_path\"";

                    if ($debug_verbose) {
                        logDbg(   "write_file_aux  Line: " 
                                . __LINE__
                                . "  Top-of-loop ('count')\n\n$msg1" );
                    }

                    if ( $wrlen <= 0 ) {
                        $msg
                            = "write_file_aux  Line: " 
                            . __LINE__
                            . "\n\n$msg1 \n\n"
                            . "Internal 'write_file' error:\n"
                            . "    'Write Len'=$wrlen is <= 0";
                        logcomment($msg);
                        die($msg);
                    } ## end if ( $wrlen <= 0 )
                    if ( $offset != $start_offset + $len_written ) {
                        $msg
                            = "write_file_aux  Line: " 
                            . __LINE__
                            . "\n\n$msg1 \n\n"
                            . "Internal 'write_file' error:\n\n"
                            . "  Offest=$offset  !=\n"
                            . "     ( Start_Offset=$start_offset  +  "
                            . "len_written=$len_written) =   "
                            . ( $start_offset + $len_written );
                        logcomment($msg);
                        die($msg);
                    } ## end if ( $offset != $start_offset...
                    if ( ( $offset + $wrlen )
                        > ( $start_offset + $size_bytes_aux ) )
                    {
                        $msg
                            = "write_file_aux  Line: " 
                            . __LINE__
                            . "\n\n$msg1 \n\n"
                            . "Internal 'write_file' error:\n\n"
                            . "  Quantity_A =  ( 'Offset'=$offset\n"
                            . "                    +  'WriteLen'=$wrlen )  = "
                            . ( $offset + $len_written + $wrlen ) . "\n\n"
                            . "  ... is greater than ...\n\n"
                            . "  Quantity_B =  ( 'Start Offset'=$start_offset\n"
                            . "                    +  'File Size'=$size_bytes_aux )  = "
                            . ( $start_offset + $size_bytes_aux ) . "\n";
                        logcomment($msg);
                        die($msg);
                    } ## end if ( ( $offset + $wrlen...

                    $write_data_fcn_args{write_len} = $wrlen;
                    $write_data_fcn_args{offset}    = $offset;
                    $write_data_fcn_args{count}     = $count;

                    my $write_data_fcn_args_copy = {%write_data_fcn_args};
                    $write_data_fcn_args_copy->{write_data_args} = "{ .... }";
                    if ( $argsref->{debug_verbose} ) {
                        logDbg(   "write_file_aux  Line: " 
                                . __LINE__
                                . "\nCalling write_data_fcn ....\n\n"
                                . Dumper($write_data_fcn_args_copy) );
                    } ## end if ( $argsref->{debug_verbose...
                    $$data_str_ref = &$write_data_fcn(%write_data_fcn_args);
                    if ( $argsref->{debug_verbose} ) {
                        logDbg(   "write_file_aux  Line: " 
                                . __LINE__
                                . "\n.... Returned from write_data_fcn" );
                    }

          # if (!$$buff_ref || ref($$buff_ref) || length($$buff_ref) < $wrlen)
                    if (   !defined($$data_str_ref)
                        || !$$data_str_ref
                        || ref($$data_str_ref) )
                    {
                        $msg
                            = "write_file_aux  Line: " 
                            . __LINE__
                            . "\n\n$msg1 \n\n"
                            . "Error from 'write_data_fcn'\n"
                            . "  -- did not return valid string: " . "\""
                            . (
                            defined($$data_str_ref)
                            ? $$data_str_ref
                            : 'UNDEF'
                            )
                            . "\"\n\nwrite_data_fcn_args:\n\n"
                            . Dumper($write_data_fcn_args_copy);
                        logcomment($msg);
                        die($msg);
                    } ## end if ( !defined($$data_str_ref...

                    # my $data_len = length($$buff_ref);
                    my $data_len = length($$data_str_ref);
                    if ( $data_len > $wrlen ) {
                        $msg
                            = "write_file_aux  Line: " 
                            . __LINE__
                            . "\n\n$msg1 \n"
                            . "Data length:\t$data_len\n\n"
                            . "Error in data returned from 'write_data_fcn' --\n"
                            . "   Expected:  'data_length'=$data_len\n"
                            . "                  ==  'write length'=$wrlen";
                        logcomment($msg);
                        die($msg);
                    } elsif ( $data_len > $wrlen ) {
                        $msg
                            = "write_file_aux  Line: " 
                            . __LINE__
                            . "\n\n$msg1 \n"
                            . "Data length:\t$data_len\n\n"
                            . "Unexpected length returned from 'write_data_fcn' --\n"
                            . "   Expected:  'data_length'=$data_len\n"
                            . "                  ==  'write length'=$wrlen";
                        logcomment($msg);
                    } else {
                        if ( $argsref->{debug_verbose} ) {
                            $msg
                                = "write_file_aux  Line: " 
                                . __LINE__
                                . "\n\n$msg1 \n"
                                . "Data length:\t$data_len";
                            logDbg($msg);
                        } ## end if ( $argsref->{debug_verbose...
                    } ## end else [ if ( $data_len > $wrlen)
                    my $subr_args_contents_len
                        = length( $write_file_subr_args_copy->{contents} );
                    $write_file_subr_args_copy->{contents}
                        = "## Data Length: $subr_args_contents_len ##";

                    # $write_file_subr_args->{contents} = $data_str;

                    $write_file_subr_args->{offset} = $offset;

                    # $write_file_subr_args_copy->{contents} = $data_str;

                    if ( $fstat && ( $count == $end_count ) ) {
                        $write_file_subr_args->{fstat} = 1;
                        if ( $argsref->{debug_verbose} ) {
                            logDbg(   "write_file_aux  Line: " 
                                    . __LINE__
                                    . "\nCalling FileManip::write_file:\n\n"
                                    . Dumper($write_file_subr_args_copy) );
                        } ## end if ( $argsref->{debug_verbose...
                        @attrs
                            = FileManip::write_file(%$write_file_subr_args);
                        if ( !@attrs ) {
                            $msg
                                = "write_file_aux  Line: " 
                                . __LINE__
                                . "FileManip::write_file failed: $@";
                            logcomment($msg);
                            die($msg);
                        } ## end if ( !@attrs )

                    } else {
                        $write_file_subr_args->{fstat} = 0;
                        if ( $argsref->{debug_verbose} ) {
                            logDbg(   "write_file_aux  Line: " 
                                    . __LINE__
                                    . "\nCalling FileManip::write_file:\n\n"
                                    . Dumper($write_file_subr_args_copy) );
                        } ## end if ( $argsref->{debug_verbose...
                        $rtn_status
                            = FileManip::write_file(%$write_file_subr_args);
                        if ( !$rtn_status ) {

                            #last unless (@attrs);
                            $msg
                                = "write_file_aux  Line: " 
                                . __LINE__
                                . "\n\n'write_file' Args:\n\n"
                                . Dumper($write_file_subr_args_copy) . "\n\n"
                                . "FileManip::write_file failed: $@";
                            logcomment($msg);
                            die($msg);
                        } ## end if ( !$rtn_status )
                        if ( $argsref->{debug_verbose} ) {
                            logDbg(   "Utilities::write_file_aux  Line: " 
                                    . __LINE__
                                    . "\nReturned $rtn_status from FileManip::write_file\n"
                                    . "File Path: \"$file_path\"" );
                        } ## end if ( $argsref->{debug_verbose...
                    } ## end else [ if ( $fstat && ( $count...

                    if ($truncate) {

                 #  Truncate file to zero length using FileManip::setattr_file
                        my %setattr_file_hash = (
                            file   => $write_file_subr_args->{file},
                            client => $write_file_subr_args->{client},
                            size   => 0,
                        );
                        if ( $argsref->{debug_verbose} ) {
                            logDbg(   "write_file_aux  Line: " 
                                    . __LINE__
                                    . "\nCalling FileManip::setattr_file to "
                                    . "truncate file to zero-length:\n\n"
                                    . Dumper($write_file_subr_args_copy) );
                        } ## end if ( $argsref->{debug_verbose...
                        $rtn_status
                            = FileManip::setattr_file(%setattr_file_hash);
                        if ( !$rtn_status ) {
                            $msg
                                = "write_file_aux  Line: " 
                                . __LINE__
                                . "\n\n'write_file' Args:\n\n"
                                . Dumper($write_file_subr_args_copy) . "\n\n"
                                . "FileManip::setattr_file failed: $@";
                            logcomment($msg);
                            die($msg);
                        } ## end if ( !$rtn_status )
                        if ( $argsref->{debug_verbose} ) {
                            logDbg(   "Utilities::write_file_aux  Line: " 
                                    . __LINE__
                                    . "\nReturned $rtn_status from FileManip::setattr_file\n"
                                    . "File Path: \"$file_path\"" );
                        } ## end if ( $argsref->{debug_verbose...
                    } ## end if ($truncate)

                }    # for ($count = 0 .... )

                if ( $fstat && ref($rtn_fstat) eq 'HASH' ) {
                    my %fst = ();
                    (   $fst{dev},   $fst{ino},   $fst{mode},
                        $fst{nlink}, $fst{uid},   $fst{gid},
                        $fst{rdev},  $fst{size},  $fst{atime},
                        $fst{mtime}, $fst{ctime}, $fst{blksize},
                        $fst{blocks}
                    ) = @attrs;
                    if ( $argsref->{debug_verbose} ) {
                        logDbg(   "Utilities::write_file_aux  Line: " 
                                . __LINE__
                                . "\nReturned attrs from FileManip::write_file\n"
                                . "File Path: \"$file_path\"\n\n"
                                . Dumper( \%fst ) );
                    } ## end if ( $argsref->{debug_verbose...
                    $rtn_fstat->{$file_path} = {%fst};
                } ## end if ( $fstat && ref($rtn_fstat...
            } ## end else [ if ( $dont_overwrite &&...

            if ( ref($file_list) eq 'ARRAY' ) {
                push( @$file_list, $file_path );
            }

            my $errmsg = $@;
            die($errmsg) if ($errmsg);
            if ( $argsref->{debug_verbose} ) {
                logDbg(   "Utilities::write_file_aux  Line: " 
                        . __LINE__
                        . "\nReturning\nErrmsg: \"$errmsg\"" );
            }

            #die($errmsg);
            return ($errmsg);
        };

    };    # End sub write_file_aux()
     #=================================================================================================

    return eval {

        my %orig_args = (%args);
        my %args_copy = (%args);
        %args = (%orig_args);
        $args_copy{filer}  = "Filer"  if ( $args{filer} );
        $args_copy{Client} = "Client" if ( $args{client} );
        logDbg(   "Enter Utilities::write_file  Line: " 
                . __LINE__
                . "  Opts:\n\n"
                . Dumper( \%args_copy ) );

        #        my %client_opts = ();
        #        foreach ( qw( contents mode random uid gid offset user ) ) {
        #		if (exists($args{$_})) {
        #			$client_opts{$_} = $args{$_};
        #		}
        #	}
        my $vol_path   = $args{vol_path};
        my $mountpoint = $args{mountpoint};
        my $abs_path;
        if ( exists( $args{path} ) ) {
            $abs_path = $args{path};
        } else {
            $abs_path = $args{file};
        }

        if ( $abs_path && ( $args{vol_path} || $args{mountpoint} ) ) {
            die("Utilities::write_file: If 'path' option is given then neither "
                    . "'vol_path' or 'mountpoint' options should also be given"
            );
        }

        my ( $single_file, $numfiles );
        if ( $args{numfiles} ) {
            $numfiles    = convert_to_bytes( $args{numfiles} );
            $single_file = ( $numfiles <= 1 );
        } else {
            $single_file = 1;
            $numfiles    = undef;
        }
        my $step_size      = convert_to_bytes( $args{step_size} );
        my $timeout        = $args{timeout};
        my $num_opts       = 0;
        my $dont_overwrite = $args{dont_overwrite};
        my $proto;
        my $catch_errors = $args{catch_errors};
        my $msg;
        my $errmsg;
        my $agent = $args{agent};
        my $rtn_status;
        my @attrs;
        my $fstat  = $args{fstat};
        my $verify = $args{verify};

        my $offset = $args{offset};
        my $size_bytes;
        my $contents;

        $num_opts = 0;
        $num_opts++ if ( defined( $args{size} ) );
        $num_opts++ if ( defined( $args{contents} ) );
        $num_opts++ if ( defined( $args{content} ) );
        if ( $num_opts != 1 ) {
            die("Utilities::write_file: Exactly 1 of 'size', 'contents', or \n"
                    . "'content' options may be specified" );
        }
        if ( defined( $args{size} ) ) {
            $size_bytes = convert_to_bytes( $args{size} );
            if ( $size_bytes !~ /^\d+$/ ) {
                die(      "Utilities::write_file  Line: " 
                        . __LINE__
                        . "\nInvalid 'size' value: \"$args{size}\"" );
            }
        } elsif ( defined( $args{contents} ) || defined( $args{content} ) ) {
            $contents = $args{contents} || $args{content};
            if ( ref($contents) ) {
                die(      "Utilities::write_file  Line: " 
                        . __LINE__
                        . "\n'contents' must be a scalar" );
            }
            if ( $contents eq "" ) {
                die(      "Utilities::write_file  Line: " 
                        . __LINE__ . "\n"
                        . "'contents' must not be empty string" );
            }
            $size_bytes = length($contents);
        } else {
            die(      "Utilities::write_file  Line: " 
                    . __LINE__
                    . "\nNo 'size' or 'contents' options given" );
        }

        if ( $size_bytes == 0 && defined($offset) && $offset != 0 ) {
            die(      "Utilities::write_file  Line: " 
                    . __LINE__
                    . "\n'offset' must be zero if 'size' is zero" );
        }
        $args{size_bytes} = $size_bytes;
        $args{contents}   = $contents;

        my $write_data_fcn  = $args{write_data_fcn};
        my $write_data_args = $args{write_data_args};
        if ( !$write_data_fcn ) {
            $write_data_fcn = $default_write_data_fcn;
            $write_data_args = { contents => $contents };
        }
        my %default_special_volume_options = (
            compression_enable => $::COMPRESSION_ENABLE,
            asis_enable        => $::ASIS_ENABLE,
            encryption_enable  => $::ENCRYPTION_ENABLE,
            large_aggr         => $::LARGE_AGGR,
        );
        $args{write_data_fcn}  = $write_data_fcn;
        $args{write_data_args} = $write_data_args;
        my $special_volume_options = $args{special_volume_options};
        unless ($special_volume_options) {
            $special_volume_options = {};
        }

        # $special_volume_options = { %{$args{special_volume_options}} };
        $special_volume_options = $args{special_volume_options};
        $special_volume_options = {%$special_volume_options};
        while ( my ( $k, $v ) = each(%default_special_volume_options) ) {
            if ( defined($v) && !defined( $special_volume_options->{$k} ) ) {
                $special_volume_options->{$k} = $v;
            }
        }
        $args{special_volume_options} = $special_volume_options;

        my $rtn_fstat = {};
        $args{rtn_fstat} = $rtn_fstat;
        if ($dont_overwrite) {
            logWarn("Utilities::write_file:  'dont_overwrite' is set");
        }
        my ( $filer, $client, $cifs_client, $client_agent );

        if ($agent) {
            if ( $agent !~ /localhost|client|cifs_client|filer/ ) {
                die("Utilities::write_file: Unexpected value for 'agent' option: \"$agent\""
                );
            }
            if ( $args{debug_verbose} ) {
                logDbg(   "Utilities::write_file  Line: " 
                        . __LINE__
                        . "   Agent option given: \"$agent\"" );
            }
            if ( $agent eq 'filer' ) {
                $filer = ( $args{filer} || $::FILER );
                if ( $args{debug_verbose} ) {
                    logDbg(   "Utilities::write_file  Line: " 
                            . __LINE__ . "\n"
                            . "Agent is 'filer': $filer" );
                }
            } elsif ( $agent eq 'client' ) {
                $client = ( $args{client} || $::CLIENT );
                if ( $args{debug_verbose} ) {
                    logDbg(   "Utilities::write_file  Line: " 
                            . __LINE__ . "\n"
                            . "Agent is 'client': $client" );
                }
            } elsif ( $agent eq 'cifs_client' ) {
                $client = ( $args{cifs_client} || $::CIFS_CLIENT );
                if ( $args{debug_verbose} ) {
                    logDbg(   "Utilities::write_file  Line: " 
                            . __LINE__ . "\n"
                            . "Agent is 'cifs_client': $client" );
                }
            } elsif ( $agent eq 'localhost' ) {
                $client = 'localhost';
                if ( $args{debug_verbose} ) {
                    logDbg(   "Utilities::write_file  Line: " 
                            . __LINE__ . "\n"
                            . "Agent is 'localhost'" );
                }
            } ## end elsif ( $agent eq 'localhost')

        } else {

            # '$agent' not defined...
            $num_opts = 0;
            $num_opts++ if ( $args{filer} );
            $num_opts++ if ( $args{client} );
            $num_opts++ if ( $args{cifs_client} );
            if ( $num_opts > 1 ) {
                die(      "Utilities::write_file: At most one of 'filer' or\n"
                        . "'client', or 'cifs_client'\noptions may be specified\n"
                        . "when 'agent' is not specified." );
            }
            if ( $args{filer} ) {
                $agent = 'filer';
                if ( $args{debug_verbose} ) {
                    logDbg(   "Utilities::write_file  Line: " 
                            . __LINE__ . "\n"
                            . "No 'agent' specified, using 'filer'" );
                }
            } elsif ( $args{client} ) {
                $client = $args{client};
                $agent  = 'client';
                if ( $args{debug_verbose} ) {
                    logDbg(   "Utilities::write_file  Line: " 
                            . __LINE__ . "\n"
                            . "No 'agent' specified, using '$agent': $client"
                    );
                } ## end if ( $args{debug_verbose...
            } elsif ( $args{cifs_client} ) {
                $cifs_client = $args{cifs_client};
                $agent       = 'cifs_client';
                if ( $args{debug_verbose} ) {
                    logDbg(   "Utilities::write_file  Line: " 
                            . __LINE__ . "\n"
                            . "No 'agent' specified, using '$agent': $client"
                    );
                } ## end if ( $args{debug_verbose...
            } else {
                $agent  = 'localhost';
                $client = 'localhost';
                if ( $args{debug_verbose} ) {
                    logDbg(   "Utilities::write_file  Line: " 
                            . __LINE__ . "\n"
                            . "No 'agent', 'filer', 'client' or 'cifs_client' specified,\n"
                            . "Using agent: 'localhost'" );
                } ## end if ( $args{debug_verbose...
            } ## end else [ if ( $args{filer} )
        } ## end else [ if ($agent)

        if ( $args{debug_verbose} ) {
            logDbg(
                      "Utilities::write_file  Line: " 
                    . __LINE__ . "\n"
                    . "Agent is 'filer': $filer\n\n"
                    . "'client'       Option: \""
                    . ( defined( $args{client} ) ? $args{client} : "UNDEF" )
                    . "\"\n"
                    . "'cifs_client'  Option: \""
                    . (
                    defined( $args{cifs_client} )
                    ? $args{cifs_client}
                    : "UNDEF"
                    )
                    . "\"\n"
                    . "'agent'        Option: \""
                    . ( defined( $args{agent} ) ? $args{agent} : "UNDEF" )
                    . "\"\n"
                    . "'filer'        Option: \""
                    . ( defined( $args{filer} ) ? $args{filer} : "UNDEF" )
                    . "\"\n\n"
                    . "'client'       \""
                    . ( defined($client) ? $client : "UNDEF" ) . "\"\n"
                    . "'cifs_client'  \""
                    . ( defined($cifs_client) ? $cifs_client : "UNDEF" )
                    . "\"\n"
                    . "'agent'        \""
                    . ( defined($agent) ? $agent : "UNDEF" ) . "\"\n"
                    . "'filer'        \""
                    . ( defined($filer) ? $filer : "UNDEF" ) . "\"\n"
                    . "'client_agent' \""
                    . ( defined($client_agent) ? $client_agent : "UNDEF" )
                    . "\""
            );
        } ## end if ( $args{debug_verbose...

        if ( $agent eq 'filer' ) {

            if ( $contents && length($contents) > 524 ) {
                $agent = undef;
            } else {
                $filer = $args{filer} || $::FILER;
            }

            unless ( $abs_path || $vol_path ) {
                die("Utilities::write_file: Either 'path' or 'vol_path' options must be given"
                );
            }
            if ( !$abs_path ) {
                $abs_path = $vol_path;
            }
            my $fc = eval { make_conn($filer) };
            die("Utilities::write_file: 'make_conn' failed: $@")
                if ( $@ || !$fc );
            my $fname = $fc->hostp()->hostname();
            my $size  = $args{size};

            #my ($vol, $rel_path) = ($abs_path =~ m%/vol/([^/]+)/(.*)%);
            my ( undef, undef, $vol, $rel_path )
                = split( m%/%, $abs_path, 4 );
            if ( $args{debug_verbose} ) {
                logDbg(   "Utilities::write_file  Line: " 
                        . __LINE__ . "\n\n"
                        . "Filer: $fname\nPath: $abs_path\nSize: $args{size}\n"
                        . "Single File: $single_file\nNum Files: $numfiles\n"
                        . "Path: $abs_path\nVolume: $vol\nRel. Path: $rel_path\n"
                        . "Timeout: $timeout" );
            } ## end if ( $args{debug_verbose...
            if ( $dont_overwrite
                && filer_vol_file_exists( $fc, $vol, $rel_path ) )
            {
                logDbg(   "Utilities::write_file  Line: " 
                        . __LINE__
                        . "   'dont_overwrite' is set and file exists, returning...\n"
                        . "File: $abs_path" );
                return (1);
            } ## end if ( $dont_overwrite &&...
            my $base_cmd;
            my $cmd_type;
            if ($contents) {
                $base_cmd = "wrfile -a $abs_path \"$contents\"";
                $cmd_type = 'wrfile';
            } else {
                $base_cmd = "mkfile $size $abs_path";
                $cmd_type = 'mkfile';
                if ( defined($numfiles) ) {
                    $base_cmd .= " $numfiles";
                    if ($step_size) {
                        $base_cmd .= " $step_size";
                    }
                } elsif ( defined($step_size) ) {

#die("Utilities::write_file: Step-size option allowed only if numfiles option is also given");
                    logWarn(
                        "Utilities::write_file: Step-size option given, \n"
                            . "but not numfiles option" );
                    $numfiles = 1;
                } ## end elsif ( defined($step_size...
            } ## end else [ if ($contents)
            my $txt;

            # my $txt = filer_cli_diag_say($fc, $base_cmd);
            my $priv_cmd = "priv set -q diag; $base_cmd";
            if ($timeout) {
                logDbg(   "Utilities::write_file  Line: " 
                        . __LINE__
                        . "    Calling 'sayEOF' on filer --\nCmmd: \"$base_cmd\"\n"
                        . "Timeout: \"$timeout\"" );
                $txt = sayEOF( $fc, $priv_cmd, timeout => $timeout );
            } else {
                logDbg(   "Utilities::write_file  Line: " 
                        . __LINE__
                        . "\n\nCalling 'sayEOF' on filer --\nCmmd: \"$base_cmd\"\n(no timeout given)"
                );
                $txt = sayEOF( $fc, $priv_cmd );
            } ## end else [ if ($timeout)
            if ( $args{debug_verbose} ) {
                logDbg(   "Utilities::write_file  Line: " 
                        . __LINE__
                        . "\n\nReply:\n$txt" );
            }

            # Output should be either blank
            # or part of mkfile_exceptions_list
            # refer burt412173 and burt554496
            $rtn_status = _is_mkfile_output_ok( output => $txt );

            logDbg(   "Utilities::write_file  Line: " 
                    . __LINE__
                    . "  Returned from filer_cli_diag_say\n\nStatus: $rtn_status  Text: \"$txt\""
            );
            logDbg(   "Exit Utilities::write_file  Line: " 
                    . __LINE__
                    . "   Return status: $rtn_status" );
            return ($rtn_status);
        } ## end if ( $agent eq 'filer')

        #  'agent' is NOT filer...

        if ( $agent eq 'localhost' ) {
            $client_agent = 'localhost';
        } elsif ( $agent eq 'cifs_client' ) {
            $client_agent = $cifs_client;
        } else {
            $client_agent = $client;
        }

        unless ($client_agent) {
            die(      "Utilities::write_file Line: " 
                    . __LINE__
                    . "    'client_agent' not defined\n\n"
                    . "'client'        Option: \""
                    . ( defined( $args{client} ) ? $args{client} : "UNDEF" )
                    . "\"\n"
                    . "'cifs_client'   Option: \""
                    . (
                    defined( $args{cifs_client} )
                    ? $args{cifs_client}
                    : "UNDEF"
                    )
                    . "\"\n"
                    . "'agent'         Option: \""
                    . ( defined( $args{agent} ) ? $args{agent} : "UNDEF" )
                    . "\"\n"
                    . "'filer'         Option: \""
                    . ( defined( $args{filer} ) ? $args{filer} : "UNDEF" )
                    . "\"\n\n"
                    . "'client'        \""
                    . ( defined($client) ? $client : "UNDEF" ) . "\"\n"
                    . "'cifs_client'   \""
                    . ( defined($cifs_client) ? $cifs_client : "UNDEF" )
                    . "\"\n"
                    . "'agent'         \""
                    . ( defined($agent) ? $agent : "UNDEF" ) . "\"\n"
                    . "'filer'         \""
                    . ( defined($filer) ? $filer : "UNDEF" ) . "\"\n"
                    . "'client_agent'  \""
                    . ( defined($client_agent) ? $client_agent : "UNDEF" )
                    . "\""
            );
        } ## end unless ($client_agent)

        my ( $vol, $rel_path );
        unless ( $abs_path || ( $vol_path && $mountpoint ) ) {
            die( "Utilities::write_file: Either 'path' or both of 'vol_path' "
                    . "and 'mountpoint' options must be given" );
        }
        if ( $rel_path && $rel_path =~ m%\\% ) {
            $rel_path =~ s%\\+%\/%g;
        }
        if ( $abs_path && $abs_path =~ m%\\% ) {
            $abs_path =~ s%\\+%\/%g;
        }
        if ( !$abs_path ) {
            ( undef, undef, $vol, $rel_path ) = split( m%/%, $vol_path, 4 );
            $abs_path = $mountpoint . '/' . $rel_path;
        }
        $abs_path =~ s%\/+%\/%g;
        unless ( $abs_path =~ m%^([^\/]+:)?(\/[^\/]+)+% ) {
            die( "Utilities::write_file: Bad format for 'path': \"$abs_path\""
            );
        }

        my @write_file_opts
            = qw( contents oflags fstat mode uid gid offset user );
        my %write_file_subr_args = ( client => $client_agent );
        for my $x (@write_file_opts) {
            if ( exists( $args{$x} ) && defined( $args{$x} ) ) {
                $write_file_subr_args{$x} = $args{$x};
            }
        }
        $args{write_file_subr_args} = \%write_file_subr_args;

        #	$abs_path =~ m%^(([^\/]+)?(\/[^\/]+)*)/[^\/]+$%;
        my $dir_path = $abs_path;
        $dir_path =~ s%\/[^\/]+$%%;
        if ( $dir_path =~ /:$/ ) {
            $dir_path .= '/';
        }

        if ( $args{debug_verbose} ) {
            logDbg(   "Utilities::write_file  Line: " 
                    . __LINE__
                    . "\n\nAgent: \"$agent\"\nClient: \"$client\"\n"
                    . "vol_path: \"$vol_path\"\nvolume: \"$vol\"\nrel_path: \"$rel_path\"\n"
                    . "mountpoint: \"$mountpoint\"\nabs_path: \"$abs_path\"\nDir path: \"$dir_path\""
            );
        } ## end if ( $args{debug_verbose...

        if ( $dir_path !~ /:/ ) {

      #  Unless using CIFS, check for existence of directory to be written to.
            my %stat_file_args = (
                file     => $dir_path,
                client   => $client_agent,
                rtn_hash => 1
            );
            if ( $args{debug_verbose} ) {
                logDbg(   "Utilities::write_file  Line: " 
                        . __LINE__
                        . "\n\nCalling FileManip::stat_file\n\n"
                        . Dumper( \%stat_file_args ) );
            } ## end if ( $args{debug_verbose...
            my %dir_stat_info = FileManip::stat_file(%stat_file_args);
            logDbg(   "Utilities::write_file  Line: " 
                    . __LINE__
                    . "\n\nDir Path: \"$dir_path\""
                    . "\n\nStats:\n\n"
                    . Dumper( \%dir_stat_info ) );
            my $io_size = $dir_stat_info{io_size};
            $args{io_size} = $io_size;
        } ## end if ( $dir_path !~ /:/ )
        $args{io_size} ||= 512;

        if ( $verify && $args{offset} ) {
            logWarn(  "Utilities::write_file:  Line: " 
                    . __LINE__
                    . "\n'verify' is set, and 'offset' is non-zero for file \"$abs_path\"\n"
                    . "Will verify existance only" );
        } ## end if ( $verify && $args{...

        my $verify_okay;
        my $file_size;
        my $msg1;
        if ($single_file) {
            $write_file_subr_args{file} = $abs_path;

            logDbg(   "Utilities::write_file  Line: " 
                    . __LINE__
                    . "\nWriting single file --  Path: $abs_path" );
            if ( $args{debug_verbose} ) {
                logDbg(   "Utilities::write_file  Line: " 
                        . __LINE__
                        . "     Calling 'write_file_aux'" );
            }
            $errmsg = &$write_file_aux( \%args );
            if ($errmsg) {
                if ($catch_errors) {
                    return eval { die($errmsg) };
                } else {
                    die($errmsg);
                }
            } ## end if ($errmsg)
            if ( $args{debug_verbose} ) {
                logDbg(   "Utilities::write_file  Line: " 
                        . __LINE__
                        . "    .... Returned from 'write_file_aux'" );
            }
            logDbg(   "Utilities::write_file  Line: " 
                    . __LINE__
                    . "\n  -- Finished writing single file" );
            if ($verify) {
                $msg1 = "";
                logDbg(   "Utilities::write_file  Line: " 
                        . __LINE__
                        . "\nverifying file \"$abs_path\"" );
                if ( $args{offset} ) {
                    $verify_okay = test_file(
                        file   => $abs_path,
                        test   => 'e',
                        client => $client
                    );
                    if ($verify_okay) {
                        $msg1 = "";
                    } else {
                        $msg1
                            = "Error creating file: \"$abs_path\" -- file does not exist";
                    }
                } else {
                    my $actual_size = get_file_size(
                        file   => $abs_path,
                        client => $client_agent
                    );
                    $verify_okay = ( $actual_size == $size_bytes );
                    if ($verify_okay) {
                        $msg1 = "";
                    } else {
                        $msg1
                            = "Bad size for created file \"$abs_path\"\n"
                            . "Expected Size: \"$size_bytes\" bytes\n"
                            . "Actual size:   \"$actual_size\" bytes";
                    } ## end else [ if ($verify_okay)
                } ## end else [ if ( $args{offset} )
                unless ($verify_okay) {
                    $msg
                        = "write_file  Line: " 
                        . __LINE__
                        . "\nVerify failed for file: \"$abs_path\"\n$msg1";
                    logcomment($msg);
                    if ($catch_errors) {
                        return eval { die($msg) };
                    } else {
                        die($msg);
                    }
                } ## end unless ($verify_okay)
            } ## end if ($verify)
        } else {
            logDbg(   "Utilities::write_file  Line: " 
                    . __LINE__
                    . "\nWriting multiple files:\nNum Files: $numfiles\nBase Path: $abs_path"
            );
            for ( my $cnt = 1; $cnt <= $numfiles; $cnt++ ) {
                my $cnt_path = $abs_path . "_" . $cnt;
                $write_file_subr_args{file} = $cnt_path;

                if ( $args{debug_verbose} ) {
                    logDbg(   "Utilities::write_file  Line: " 
                            . __LINE__
                            . "    Calling 'write_file_aux'  Count: $cnt ...."
                    );
                } ## end if ( $args{debug_verbose...
                $errmsg = &$write_file_aux( \%args );
                if ($errmsg) {
                    if ($catch_errors) {
                        return eval { die($errmsg) };
                    } else {
                        die($errmsg);
                    }
                } ## end if ($errmsg)
                if ( $args{debug_verbose} ) {
                    logDbg(   "Utilities::write_file  Line: " 
                            . __LINE__
                            . "    .... Returned from 'write_file_aux'  Count: $cnt"
                    );
                } ## end if ( $args{debug_verbose...
                if ($verify) {
                    if ( $args{debug_verbose} ) {
                        logDbg(   "Utilities::write_file  Line: " 
                                . __LINE__
                                . "\nverifying file \"$cnt_path\"" );
                    }
                    $msg1 = "";
                    if ( $args{soffset} ) {
                        $verify_okay = test_file(
                            file   => $cnt_path,
                            test   => 'e',
                            client => $client
                        );
                        if ($verify_okay) {
                            $msg1 = "";
                        } else {
                            $msg1 = "Error creating file: \"$cnt_path\"\n"
                                . "    -- file does not exist";
                        }
                    } else {
                        my $actual_size = get_file_size(
                            file   => $cnt_path,
                            client => $client
                        );
                        $verify_okay = ( $actual_size == $size_bytes );
                        if ($verify_okay) {
                            $msg1 = "";
                        } else {
                            $msg1
                                = "Bad size for created file \"$cnt_path\"\n"
                                . "Expected Size: \"$size_bytes\" bytes\n"
                                . "Actual size:   \"$actual_size\" bytes";
                        } ## end else [ if ($verify_okay)
                    } ## end else [ if ( $args{soffset} )
                    unless ($verify_okay) {
                        $msg
                            = "write_file  Line: " 
                            . __LINE__
                            . "\nVerify failed for file: \"$cnt_path\"\n$msg1";
                        logcomment($msg);
                        if ($catch_errors) {
                            return eval { die($msg) };
                        } else {
                            die($msg);
                        }
                    } ## end unless ($verify_okay)
                } ## end if ($verify)
            } ## end for ( my $cnt = 1; $cnt...
            logDbg(   "Utilities::write_file  Line: " 
                    . __LINE__
                    . "\n  -- Finished writing multiple files" );
        } ## end else [ if ($single_file)

        if ($fstat) {
            logDbg( "Exit Utilities::write_file  Line: " . __LINE__ );

            #		logDbg("Exit Utilities::write_file  Line: ".__LINE__.
            #		       "\nReturn attrs:\n\n".Dumper($rtn_fstat));
            return ($rtn_fstat);
        } else {
            logDbg(   "Exit Utilities::write_file  Line: " 
                    . __LINE__
                    . "\nReturn status: $rtn_status" );
            return (1);
        } ## end else [ if ($fstat)

    };
} ## end sub write_file(%)

#= Get the netapp.mib file path for BR or IC
#= :fc                filer connection
#= :vers              version of filer
#= >mib file path

sub get_mibfile_location($$;) {
    my ( $fc, $version ) = @_;
    my $mib_file_path = "/x/eng/rlse/$version/files/netapp.mib";
    if ( is_ontapng_version($fc) ) {
        $mib_file_path
            = "/x/eng/rlse/ontapng/$version/ontap/files/netapp.mib";
    }
    return $mib_file_path;
} ## end sub get_mibfile_location($$;)
#######################################################################################################
## @name ::access_file_blocks(conn => $conn, path => $path, cmd => $cmd,
##                                          msgtype => $msgtype, nfs_client => $client)
##
## @summary
##     Return the output by accessing  the user data from nfs, or execute
##     the command on the filer and return the output depending on the msgtype.
##
## @description
##     access_file_blocks - Verify the msgtype and execute the command either in filer or in nfs and
##     return the output to the caller.
##
##
## @arg $conn Required:
##      conn => filer connection object.
##
## @arg $path Required:
##      path => path for the file to verify corruption for
##
## @arg $msgtype Required:
##      msgtype => The type of the message (abortable or non-abortable)
##
## @arg $cmd Optional:
##      cmd => Any cmd is acceptable which is having a vol_path as an argument.
##      Default: rdfile => if msgtype => 'non-abortable'
##      Default: cat => if msgtype => 'abortable'
##
## @arg $client Optional:
##      nfs_client => Client to work on.
##      Default: localhost
##
## @arg $timeout Optional:
##      timeout => Timeout for rdfile.
##      Default: 300
##
## @arg $private Optional:
##      private => To access a private file throgh dumpblock.
##      Default: undef
##
## @arg $blockno Optional:
##      blockno => Blockno of the file to access throgh dumpblock.
##      Default: 0
##
## @arg $fileid Optional:
##      fileid => Inode number of the file to pass to dumpblock.
##      Default: Finds and uses the fileid from the path provided.
##
## @arg $snapid Optional:
##      snapid => snapid of the volume to pass to dumpblock.
##      Default: Finds and uses the snapid from the path provided.
##
## @return values
##      Returns the message returned by client if the msg type is 'abortable'
##                      or
##      Returns the output of the filer in case of 'non-abortable'.
##
## @example my ($msg) = access_file_blocks(conn => $conn,
##                           	           path => "/vol/testvol/testfile",
##					   msgtype => "abortable",
##                                         cmd => "ls");
##

sub access_file_blocks(%) {
    my (%opts,         $conn,       $client_conn, $vol,
        $path,         $nfs_client, $cmd,         $cmd_to_exec,
        $path_of_file, $msgtype,    $abort_msg,   $non_abort_msg,
        $mnt,          $tmout,      $snap,        $private,
        $blockno,      $fileid,     $snapid,      $level,
        $str
    );

    %opts = validate(
        @_,
        {   conn       => { type => OBJECT },
            path       => { type => SCALAR },
            nfs_client => {
                type     => SCALAR | OBJECT,
                optional => 1,
                default  => 'localhost'
            },
            msgtype => { type => SCALAR, default  => 'abortable' },
            cmd     => { type => SCALAR, optional => 1 },
            timeout => { type => SCALAR, optional => 1 },
            private => { type => SCALAR, optional => 1 },
            blockno => { type => SCALAR, optional => 1 },
            fileid  => { type => SCALAR, optional => 1 },
            snapid  => { type => SCALAR, optional => 1 },
            level   => { type => SCALAR, optional => 1 },
        }
    );

    $conn       = $opts{conn};
    $path       = $opts{path};
    $msgtype    = $opts{msgtype};
    $nfs_client = $opts{nfs_client};
    $cmd        = $opts{cmd};
    $tmout      = $opts{timeout};
    $blockno    = $opts{blockno};
    $private    = ( $opts{private} == 1 ) ? '1' : undef;
    $fileid     = $opts{fileid};
    $snapid     = $opts{snapid};
    $level      = $opts{level};

    if ( !defined($tmout) ) {
        $tmout = 300;
    }

    if ( !defined($blockno) ) {
        $blockno = 0;
    }
    if ( $nfs_client =~ m!Connectrec=HASH! ) {
        $client_conn = $nfs_client;
    } else {
        host($nfs_client);
        $client_conn = connect("#rsh");
    }

    if ( $msgtype eq 'abortable' ) {
        $cmd = defined $opts{cmd} ? $opts{cmd} : 'cat';

        if ( $path =~ m!/vol/(.*?)/(.*)! ) {
            $vol          = $1;
            $path_of_file = $2;
        } else {
            die("Path provided is not correct or not exists.");
        }

        ## mounting on the volume
        $mnt = FS::Util::Setup::mount_vol(
            client    => $client_conn,
            filer_vol => $vol,
            mountopts => 'actimeo=0,noac,hard,intr'
        ) or die("Mount failed :$@");

        ## Exec the cmd
        $cmd_to_exec = "$mnt/$path_of_file";
        ( undef, undef, $abort_msg, undef )
            = say( $client_conn, "$cmd $cmd_to_exec", '-timeout', $tmout );

        ## Unmounting the volume
        FS::Util::Setup::umount_vol( $mnt, client => $client_conn )
            or die("Umount failed :$@");

        return $abort_msg;
    } elsif ( $msgtype eq 'non-abortable' ) {

        $cmd = defined $opts{cmd} ? $opts{cmd} : 'rdfile';

        my $host = $conn->{hostp};
        my $vers = get_wafl_version($host);
        my $ic3  = version_check_ic3( conn => $conn );

        # The following condition is to ensure that dumpblock command is used
        # only for RR and IC (IC3 onwards) excluding BR0.

        if ( ( $vers >= WV_ROLLINGROCK_MIN || $ic3 ) && $cmd eq 'rdfile' ) {
            my $type = 'file';
            if ( $path =~ m!/vol/(.*?)/(metadir\/|).snapshot/(.*?)/(.*)! ) {
                $vol          = $1;
                $snap         = $3;
                $path_of_file = $4;
                if ( $vers < WV_ROLLINGROCK_MIN ) {
                    $type = 'info';
                }

            } elsif ( $path =~ m!/vol/(.*?)/(metadir\/|)(.*)! ) {
                $vol          = $1;
                $path_of_file = $2;
            } else {
                die("Path provided is not correct or not exists.");
            }

            # noabort should be appeneded only for RR builds
            my $no_abort = ( $vers < WV_ROLLINGROCK_MIN ) ? 0 : 1;

            if ( !defined($fileid) ) {
                $str
                    = 'dumpblock(conn => $conn, block_type => $type, volume => $vol, file   => $path, blockno => $blockno, noabort => $no_abort,';
            } else {
                $str
                    = 'dumpblock(conn => $conn, block_type => $type, volume => $vol, fileid  => $fileid, blockno => $blockno, noabort => $no_abort,';
            }

# snapid should be appeneded only for RR builds. If using dumpblock info, it can be appended for >ic3 and >br1 builds

            if ( $vers >= WV_ROLLINGROCK_MIN || $type eq 'info' ) {
                if ( !defined($snapid) && defined($snap) ) {
                    $str .= 'snapname => $snap,';
                } elsif ( defined($snapid) && defined($snap) ) {
                    $str .= 'snapid => $snapid,';
                } elsif ( defined($snapid) ) {
                    $str .= 'snapid => $snapid,';
                }
                if ( defined($level) ) {
                    $str .= 'level => $level,';
                }
            } ## end if ( $vers >= WV_ROLLINGROCK_MIN...

         # -private should be appended only for BR.1 and RR.0 builds and above

            if ( ( $vers >= WV_BOILERMAKER_MIN ) && defined($private) ) {
                $str .= 'private => $private,';
            }

            $str .= ');';

            $non_abort_msg = eval $str;

            return $non_abort_msg;
        } else {
            ( undef, undef, $non_abort_msg, undef )
                = say( $conn, "$cmd $path", '-timeout', $tmout );
            return $non_abort_msg;
        }
    } else {
        die("Invalid msgtype: $msgtype");
    }

} ## end sub access_file_blocks(%)
############################################################################################
## @name ::compare_counters(conn => $conn, stat_value => $stat_value,
##                           stat_new_value => $stat_new_value)
##
## @summary
##     Compare two REP stat values that were taken between filer operations
##     and return 1 if the stat was updated otherwise return 0.
##
## @description
##     Compare REP stat value and return if the stat was modified between
##     operations.
##
## @arg $conn Required:
##      conn => filer connection object.
##
## @arg $stat_value Required:
##      stat_value => Value before operation
##
## @arg $stat_new_value Required:
##      stat_new_value => Value after operation
##
## @return values
##      Returns 1 if the stat was modified
##                      or
##      Returns 0
##
## @example my ($res) = compare_counters(conn => $conn,
##                           	         stat_value => $value,
##					 stat_new_value => $new_value);
##
##
sub compare_counters(%) {

    my ( %opts, $conn, $stat_value, $stat_new_value );

    %opts = validate(
        @_,
        {   conn           => { type => OBJECT },
            stat_value     => { type => SCALAR },
            stat_new_value => { type => SCALAR },
        }
    );

    $conn           = $opts{conn};
    $stat_value     = $opts{stat_value};
    $stat_new_value = $opts{stat_new_value};

    if ( $stat_value < $stat_new_value ) {
        return 1;
    } else {
        return 0;
    }
} ## end sub compare_counters(%)
############################################################################################
## @name ::version_check_ic2(conn => $conn)
##
## @summary
##     Verify the version of the filer is ironcity_2 or not.
##
## @description
##     version_check_ic2 - Verify the version of the filer is ironcity_2 or ironcity_1
##
## @arg $conn Required:
##      conn => filer connection object.
##
## @return value
##      Return 1 if the version is ironcity_2 else return 0.
##
##
sub version_check_ic2(%) {

    my ( %opts, $conn );

    %opts = validate( @_, { conn => { type => OBJECT }, } );

    $conn = $opts{conn};

    my $output = sayEOF( $conn, "priv set -q diag; file check " );

    if ( $output =~ /-S/ ) {
        return 1;
    } else {
        return 0;
    }
} ## end sub version_check_ic2(%)

############################################################################################
## @name ::version_check_ic3(conn => $conn)
##
## @summary
##     Verify the version of the filer is ironcity_3 or not.
##
## @description
##     version_check_ic3 - Verify the version of the filer is ironcity_3 or not
##
## @arg $conn Required:
##      conn => filer connection object.
##
## @return value
##      Return 1 if the version is ironcity_3 else return 0.
##
##
sub version_check_ic3(%) {

    my ( %opts, $conn );

    %opts = validate( @_, { conn => { type => OBJECT }, } );

    $conn = $opts{conn};

    my $output = sayEOF( $conn, "priv set -q diag; vol vbncheck " );
    if ( $output =~ /-maps/ ) {
        return 1;
    } else {
        return 0;
    }
} ## end sub version_check_ic3(%)

############################################################################################
## @name ::version_check_br1_and_higher(host_obj => $host_obj)
##
## @summary
##     Verify the version of the filer is BR.1 release or higher.
##
## @description
##     Verify the version of the filer using
##      get_wafl_version() and NACL::APISet::Version
##      to determine if filer version is BR.1 or higher
##
## @arg $host_obj Required:
##      host_obj => filer host object.
##
## @return value
##      Return 1 if the version is BR.1 or higher else return 0.
##
##
sub version_check_br1_and_higher(%) {

    my ( %opts, $host_obj );
    my ( $hosttype, $shell_type, $version_manager, $wafl_ver, $vers );

    %opts = validate( @_, { host_obj => { type => OBJECT } } );

    $host_obj = $opts{host_obj};

    $hosttype = $host_obj->hosttype();
    if ( $hosttype =~ /filer/ ) {
        $shell_type = ( $hosttype =~ /ng/ ) ? "CMode" : "7Mode";
    } else {
        die("Invalid hosttype for FILER and it is $hosttype");
    }

    $version_manager = NACL::APISet::Version->new(
        hostobj    => $host_obj,
        category   => "Node",
        shell_type => $shell_type
    );

    $vers
        = $version_manager->get_version_attribute( "attribute" => "release" );
    $Log->debug("Filer version obtained is: $vers");

    $wafl_ver = get_wafl_version($host_obj);
    if ( $wafl_ver < WV_BOILERMAKER_MIN || $vers eq 'Boilermaker__8.0.0' ) {
        $Log->debug("version_check_br1_and_higher return: 0");
        return 0;
    } else {
        $Log->debug("version_check_br1_and_higher return: 1");
        return 1;
    }

} ## end sub version_check_br1_and_higher(%)

############################################################################################
## @name ::access_file_blocks_and_verify_ems(%)
##
## @summary
##     Wrapper function of access_file_blocks which will provide ems error message for
##     the pattern provided along with abortable and non-abortable messages.
##
## @description
##     Function is only applicable for the ems triggering activities.
##     Returns ems error messages along with abortable and non-abortable messages
##
## @arg $conn Required:
##      conn => filer connection object.
##
## @arg $pattern Required:
##      pattern => pattern to match with ems log.
##
## @arg $path Required:
##      path => path for the file to verify corruption for
##
## @arg $msgtype Required:
##      msgtype => The type of the message (abortable or non-abortable)
##
## @arg $cmd Optional:
##      cmd => cmd to trigger execute.
##      Default: rdfile => if msgtype => 'non-abortable'
##      Default: cat => if msgtype => 'abortable'
##
## @arg $client Optional:
##      nfs_client => Client to work on.
##      Default: localhost
##
## @arg $timeout Optional:
##      timeout => Timeout for rdfile.
##      Default: 300
## @arg $debug Optional:
##      debug => 1 implies , the output of 'ems event status' command will
##      be shown in the log for the debugging purpose while verifying the ems.
##
## @return
##      Returns the ems error message for the pattern provided and also,
##      returns the abortable/non-abortable error message from the ems
##      triggering event.
##
##
sub access_file_blocks_and_verify_ems(%) {

    my ( %opts, $conn, $pattern, $emsLog, $ems_msg, $msg, $tmout, $debug );

    %opts = validate(
        @_,
        {   conn       => { type => OBJECT },
            path       => { type => SCALAR },
            nfs_client => {
                type     => SCALAR | OBJECT,
                optional => 1,
                default  => 'localhost'
            },
            msgtype => { type => SCALAR, default  => 'abortable' },
            cmd     => { type => SCALAR, optional => 1 },
            pattern => { type => SCALAR },
            timeout => { type => SCALAR, optional => 1 },
            debug => { type => SCALAR, optional => 1, default => 0 },
        }
    );

    $conn    = $opts{conn};
    $pattern = $opts{pattern};
    $tmout   = $opts{timeout};
    $debug   = $opts{debug};

    if ( !defined($tmout) ) {
        $tmout = 300;
    }

    ## Start capturing the EMS
    $emsLog = LogReader::EMS->new($conn);

    if ( $debug == 1 ) {
        $emsLog->start_tail(
            pattern => $pattern,
            debug   => $debug
        );
    } else {
        $emsLog->start_tail( pattern => $pattern );
    }
    if ( defined $opts{cmd} ) {
        $msg = &access_file_blocks(
            conn       => $conn,
            path       => $opts{path},
            msgtype    => $opts{msgtype},
            nfs_client => $opts{nfs_client},
            cmd        => $opts{cmd},
            timeout    => $tmout
        );
    } else {
        $msg = &access_file_blocks(
            conn       => $conn,
            path       => $opts{path},
            msgtype    => $opts{msgtype},
            nfs_client => $opts{nfs_client},
            timeout    => $tmout
        );
    } ## end else [ if ( defined $opts{cmd...

    ## Stop capturing the EMS
    $emsLog->stop_tail( [$pattern] );

    $ems_msg = $emsLog->find_events($pattern);

    if ( @{$ems_msg} ) {
        $Log->trace("Lib Detected data corruption");
    } else {
        $Log->trace("Lib Did not detect data corruption");
    }

    return ( $msg, $ems_msg );
} ## end sub access_file_blocks_and_verify_ems(%)
#######################################################

## @name ::dumpblock(conn       => $fc,
##                   block_type => $block_type,
##                   volume     => $volume,
##                   type       => 1,
##                   file       => $file,
##                   fileid     => $fileid,
##                   private    => 1,
##                   blockno    => $blockno,
##                   level      => $level,
##                   snapname   => $snapname,
##                   snapid     => $snapid,
##                   dbn        => $dbn,
##                   vbn        => $vbn,
##                   diskno     => $diskno,
##                   cksum      => 1,
##                   block      => $block)
##
## @summary
##
##     Execute the dumpblock command on the filer and returns the output
##
## @description
##
##     This function runs the dumpblock command on the filer. The command
##     usage for dumpblock is:
##
##     dumpblock ifile [-A | -V] <vol-name> <fileid> [<snapid> [-b]]
##       - dump block of inofile containing that inode
##     dumpblock file [-A | -V] <vol-name> <fileid> [-private]  <blockno> [-no_abort] [<snapid>]
##       - dump data block <blockno> from file inode # <fileid> <snapid>
##     dumpblock vbn [-A | -V] <vol-name> <vbn>
##       - dump block with the specified vbn on the specified volume/aggregate
##     dumpblock dbn [-c] <disk_name> <dbn>
##       - dump block with the specified dbn on the specified disk
##     dumpblock info [-A | -V] <vol-name> <fileid> [-private] <blockno>
##                                                  [<level>] [<snapid>]
##       - dump information about the specified block
##
##     The output of the command is parsed and stored in hash. A reference to
##     the hash is returned on success.
##
## @arg %opts Required
##
##     This hash contains all the parameters that are passed to the dumpblock
##     command. Individual parameters are specified below:
##
##     conn => connspec Required
##     The connection spec using which the commands in this function will be
##     executed
##
##     block_type => 'dumpblock type' Required
##     The dumpblock type that has to be executed. Valid values are ifile, file,
##     vbn, dbn or info.
##
##     volume => 'volume or aggr name' Optional
##     The volume name on which the dumpblock command has to be run.
##
##     type => 'aggr|vol' Optional
##     This value denotes if the volume parameter is an aggregate or volume.
##     This will include the -A option if the value is 'aggr'. Default is 'vol'
##
##     file => 'filename' Optional
##     The filename of the file against which dumpblock has to be run.
##     If filename is provided and fileid is not provided, the fileid will
##     be obtained by calling filer_showfh.
##
##     fileid => 'fileid' Optional
##     The fileid of the file against which dumpblock has to be run.
##     The filename parameter can be provided instead of this, and the function
##     will take care of finding the fileid.
##
##     noabort  => '-no_abort' Optional
##     Option to run dumpblock to trigger non-abortable messages.
##     Only this option works on RR or later builds.
##
##     private  => '-private' Optional
##     Private option to access the private/meta level information.a
##     Only this option works on BR.1 or later builds.
##
##     blockno => 'blockno' Optional
##     Block number of the file for which dumpblock is being run.
##
##     level => 'level' Optional
##     Block level of the file being used for dumpblock
##
##     snapname => 'snapshot name' Optional
##     Name of the snapshot, for which the snapid will be passed to dumpblock.
##     This function will find out the snapid of the provided snapname and pass
##     it to dumpblock
##
##     snapid => 'snapid' Optional
##     Snapid that will be passed to dumpblock. Snapname parameter can be used
##     instead of this, the function will get the required snapid and pass
##     it to dumpblock
##
##     dbn => 'dbn' Optional
##     The value of dbn that is to be passed to dumpblock
##
##     vbn => 'virtual block number' Optional
##     The virtual block number which will be passed to dumpblock
##
##     diskno => 'disk number' Optional
##     The disk identifier that will be passed to dumpblock
##
##     cksum => '0|1' Optional
##     The '-c' option will be passed to dumpblock if this parameter is set to 1
##     This is valid only for 'dumpblock dbn'
##
##     block => '0|1' Optional
##     Setting this parameter to 1 will set the flag '-b' while calling
##     dumpblock. This is valid only for
##     'dumpblock ifile'
##
## @return value
##     Returns a hashref that contains the content of the block that was read
##     using dumpblock. Also contains diskid, VBN and other values that are
##     returned by dumpblock.
##
##     On failure, 0 will be returned and a message will be stored in $@
##
## @example
##     my $dump = dumpblock(conn => $Fc,
##                          block_type => 'info',
##                          volume => 'testvol',
##                          fileid => '101',
##                          blockno => '0',
##                          level => '0') or die($@);
##     The above example will run the following command on the filer:
##     dumpblock info testvol 101 0 0

sub dumpblock(%) {

    my %opts = validate(
        @_,
        {   conn       => { type => OBJECT },
            block_type => {
                type  => SCALAR,
                regex => qr/^(ifile|file|vbn|dbn|info)$/
            },
            volume => { type => SCALAR, optional => 1 },
            type   => {
                type     => SCALAR,
                optional => 1,
                default  => 'vol'
            },
            file     => { type => SCALAR, optional => 1 },
            fileid   => { type => SCALAR, optional => 1 },
            noabort  => { type => SCALAR, optional => 1 },
            private  => { type => SCALAR, optional => 1 },
            blockno  => { type => SCALAR, optional => 1 },
            level    => { type => SCALAR, optional => 1 },
            snapname => { type => SCALAR, optional => 1 },
            snapid   => { type => SCALAR, optional => 1 },
            dbn      => { type => SCALAR, optional => 1 },
            vbn      => { type => SCALAR, optional => 1 },
            diskno   => { type => SCALAR, optional => 1 },
            cksum    => { type => SCALAR, optional => 1 },
            block    => { type => SCALAR, optional => 1 }
        }
    );

    my ($block_type, $cmd,    $conn,     $volume, $fileid,   $blockno,
        $level,      $status, $noabort,  $snapid, $dbn,      $vbn,
        $diskno,     $cksum,  @info_arr, $block,  $dump_out, $private
    );

    $conn       = $opts{conn};
    $block_type = $opts{block_type};
    $volume
        = ( $opts{type} eq 'aggr' ) ? '-A ' . $opts{volume} : $opts{volume};
    $fileid  = $opts{fileid};
    $noabort = ( $opts{noabort} == 1 ) ? '-no_abort' : undef;
    $private = ( $opts{private} == 1 ) ? '-private' : undef;
    $blockno = $opts{blockno};
    $level   = $opts{level};
    $snapid  = $opts{snapid};
    $dbn     = $opts{dbn};
    $vbn     = $opts{vbn};
    $diskno  = $opts{diskno};
    $cksum   = ( $opts{cksum} == 1 ) ? '-c' : '';
    $block   = ( $opts{block} == 1 ) ? '-b' : '';

    my $host = $conn->{hostp};
    my $vers = get_wafl_version($host);

    # Getting the fileid, if filename is supplied instead of fileid
    if ( !defined( $opts{fileid} ) && defined( $opts{file} ) ) {
        my %showfh = filer_showfh( $conn, $opts{file} );
        $fileid = hex( $showfh{fileid} );
    }

    # Getting the snapid, if snapname is specified instead of snapid
    if ( !defined( $opts{snapid} ) && defined( $opts{snapname} ) ) {
        my @snaps = snapshot_status( $conn, $volume );
        foreach my $snaprec (@snaps) {
            if ( $opts{snapname} eq $$snaprec{name} ) {
                $snapid = $$snaprec{snapid};
                last;
            }
        } ## end foreach my $snaprec (@snaps)
    } ## end if ( !defined( $opts{snapid...

    # Building the command
    $cmd = "dumpblock $block_type ";

    if ( $block_type eq 'info' ) {
        if ( defined($private) ) {
            $cmd .= "$volume $fileid $private $blockno $level $snapid";
        } else {
            $cmd .= "$volume $fileid $blockno $level $snapid";
        }
    } elsif ( $block_type eq 'dbn' ) {
        $cmd .= "$cksum $diskno $dbn";
    } elsif ( $block_type eq 'vbn' ) {
        $cmd .= "$volume $vbn";
    } elsif ( $block_type eq 'file' ) {
        if ( $vers >= WV_BOILERMAKER_MIN ) {
            if ( $vers >= WV_ROLLINGROCK_MIN ) {
                $cmd .= "$volume $fileid $private $blockno $noabort $snapid";
            } else {
                $cmd .= "$volume $fileid $private $blockno";
            }
        } else {
            $cmd .= "$volume $fileid $blockno";
        }
    } elsif ( $block_type eq 'ifile' ) {
        $cmd .= "$volume $fileid $snapid $block";
    }
    $dump_out = filer_cli_say( $conn, "priv set -q diag; $cmd" );

    # If there is a syntax error, return 0.

    if ( $dump_out =~ m/dumpblock|block not found/ ) {
        $@ = $dump_out;
        return 0;
    }

    # Parse the output based on the $block_type

    if ( $block_type eq 'info' ) {

        my %dump_hash;

        # Storing the output of dumpblock into an array
        my @arr = split( /\n/, $dump_out );

        # Replacing ^M characters with ''
        @arr = map { s/\cM//g; $_ } @arr;

        # Saving the status of dumpblock. This will be added to the hash as
        # 'status'
        $status = shift @arr;

        foreach my $str (@arr) {

            # Splitting the contents of @arr as single elements in @info_arr
            push( @info_arr, split( / /, $str ) );
        }

        %dump_hash = @info_arr;
        $dump_hash{status} = $status;

        return \%dump_hash;

    } elsif ( $block_type =~ m/dbn|vbn|file|ifile/ ) {
        my $dump_hash = _parse_dumpblock($dump_out);
        return $dump_hash;
    }
} ## end sub dumpblock(%)

sub _parse_dumpblock($) {
    my ($dump_out) = @_;
    my ( %dump_hash, @dmp );

    # Storing the output of dumpblock into an array
    @dmp = split( /\n/, $dump_out );

    # Replacing ^M characters with ''
    @dmp = map { s/\cM//g; $_ } @dmp;

    # Parsing the first line. This line contains diskno, dbn, vbn etc.
    # These values will be placed into the hash dump_hash

    # Looking for comma separated list of parameters
    if ( $dmp[0] =~ m/,/ ) {

        # Using comma as the delimiter for parameter-value pairs
        my @spl = split( /,/, $dmp[0] );
        foreach (@spl) {
            m/(\w+)\s+=?\s*([\w\.]+)/;
            $dump_hash{$1} = $2;
        }
    } ## end if ( $dmp[0] =~ m/,/ )

    # Saving the contents of the block into the hash
    $dump_hash{block_contents} = \@dmp;
    return ( \%dump_hash );
} ## end sub _parse_dumpblock($)

############################################################################################
## @name ::set_wafl_scan_speed(conn => $conn, speed => $speed)
##
## @summary
##     Sets wafl scan speed
##
## @description
##      sets the wafl scan speed to $speed if given,
##      otherwise set the speed to 10000
##
## @arg $conn Required:
##      conn => filer connection object.
##
## @arg $speed Optional:
##      speed => scanner speed
##      Default: 10000
##
## @return value
##      an array of return value and error messgae where,
##      return value=1 and error message="", if successful
##      return value=0 and error message="error", if failed
##
##
sub set_wafl_scan_speed(%) {

    my ( %opts, $fconn, $speed, $output );

    my ( $ret_val, $err_msg );

    %opts = validate(
        @_,
        {   conn  => { type => OBJECT },
            speed => { type => SCALAR, default => 10000 }
        }
    );

    $fconn = $opts{conn};

    $speed = $opts{speed};

    $output = sayEOF( $fconn, "priv set -q test; wafl scan speed $speed" );

    if ($output) {

        # got error while setting scanner speed
        $ret_val = 0;
        $err_msg = "setting of wafl scan speed failed, error: $output";
    } else {

        # set the speed successfully
        $ret_val = 1;
        $err_msg = "";
    } ## end else [ if ($output)

    return ( $ret_val, $err_msg );

} ## end sub set_wafl_scan_speed(%)

############################################################################################
## @name ::get_wafl_scan_speed(conn => $conn)
##
## @summary
##     Gets wafl scan speed
##
## @description
##     get_wafl_scan_speed - retrieve wafl scan speed
##
## @arg $conn Required:
##      conn => filer connection object.
##
## @return value
##      an array of scanner speed and error message where,
##      return value=scanner_speed and error message="", if successful
##      return value=-1 and error message="error", if failed
##
##
sub get_wafl_scan_speed(%) {

    my ( %opts, $fconn, $output );

    my ( $scanner_speed, $err_msg );

    %opts = validate( @_, { conn => { type => OBJECT } } );

    $fconn = $opts{conn};

    $output = sayEOF( $fconn, "priv set -q test; wafl scan speed" );

    if ( $output =~ m/WAFL\s+scan\s+speed\s+is\s+([\d]+)/i ) {
        $scanner_speed = $1;
        $err_msg       = "";
    } else {

        # error case
        $scanner_speed = -1;
        $err_msg       = "Could not retrieve wafl scan speed, error: $output";
    } ## end else [ if ( $output =~ m/WAFL\s+scan\s+speed\s+is\s+([\d]+)/i)
    return ( $scanner_speed, $err_msg );

} ## end sub get_wafl_scan_speed(%)

############################################################################################
## @name ::options_get($conn,$option)
## @summary Get the value of the option, even the option has been set with
##          multiple values delimited by a space. This function is different
##          from filer_options_get as filer_options_get will return only one
##          value for the option though it has been set with multiple values.
## @arg $conn Required: The filer connection name where this function is executed
## @arg $option Required: The name of the option
## @return Returns the value for the options given.

sub options_get {
    my ( $conn, $option ) = @_;
    my $cmd = "options $option";
    my ( $text, $result ) = filer_cli_say( $conn, $cmd, { retry => 'ok' } );

    my $value = "";
    my @val   = "";
    my @info  = filer_cli_split($text);
    foreach my $line (@info) {
        if ( $line =~ /$option / ) {

            #Replacing warning message with nothing if executed on
            #cluster setup.
            $line =~ s/\(value might be overwritten in takeover\)//gi;
            @val = split( /\s+/, $line );
            shift(@val);
            $value = join( ' ', @val );
        } ## end if ( $line =~ /$option /)
    } ## end foreach my $line (@info)
    return $value;

} ## end sub options_get
################################################################################
## @name ::options_set($conn,$option,$value)
## @summary Set the value to the given option(Multiple values should be
##          delimited by space). This function is different from
##          filer_options_set as filer_options_set will set multiple values
##          to an option only if values provided inside quotes and fails if
##          verify => 1, as it inturn calls filer_options_get to get the value
##          for the option to match with the pattern set and which will not
##          match because filer_options_get returns only one value even muliple
##          values set for that particular option.
## @arg $conn Required: The filer connection name where this function is executed
## @arg $option Required: The name of the option
## @arg $value Required: The value to set the option to
## @return If the value set properly on the filer returns 1, else croack on error.

sub options_set {
    my ( $conn, $option, $value ) = @_;
    my $cmd = "options $option \'$value\'";
    my ( $text, $result ) = filer_cli_say( $conn, $cmd, { retry => 'ok' } );
    my $check_value = options_get( $conn, $option );
    if ( $value ne $check_value ) {
        croak
            "$option is $check_value, but should have been changed to $value";
    }
    return 1;
} ## end sub options_set

#####################################################################
#Function : get_qualified_image_name()
#Description: validates the image name and return the full qualified
#   name in case of short name (eg. mainngN etc) given as input
#Parameter: $build_name
#Returns:  Return the image name on success, otherwise dies
#####################################################################
sub get_qualified_image_name ($) {

    my ($build_name) = @_;

    my ($image_name);

    my ($link_found) = 0;

    my (@dir_list_to_search) = qw ( /rlse/DOT
        /rlse
        /rlse/ontapng
    );

    foreach my $dir (@dir_list_to_search) {

        logcomment("Check $build_name in $dir\n");

        $image_name = readlink "$dir/$build_name";

        if ($image_name) {
            logcomment("Got link: $image_name\n");
            $image_name =~ /(.*\/)+(.*)/;
            $image_name = $2;
            $link_found = 1;
            last;
        } else {
            logcomment("$build_name not found in dir: $dir, $!\n");
        }
    } ## end foreach my $dir (@dir_list_to_search)

    if ($link_found) {
        logcomment("Image name returned: $image_name\n");
        return $image_name;
    } else {
        die("Invalid image name: $build_name\n");
    }
} ## end sub get_qualified_image_name ($)
########################################################################
## @name ::is_dual_inode()
##
## @summary
##      Check the current version on the filer supports
##      private inode(Dual inode feauture)
##
## @description
##      Verify the dual inode feature supported by the current
##      version on the filer using maxfiles -private command
##      which give maxfiles on private space.
##
## @synopsis is_dual_inode($conn)
##
##
## @arg $conn Required:
##      filer connection object.
##
## @return value
##      return value=1 , if version supports dual inode feature
##      return value=0 , if version didnt support dual inode feature
##
sub is_dual_inode($) {
    my ($conn) = @_;
    my $cmd = "priv set -q advanced;maxfiles -private";
    my $output;

    $output = sayEOF( $conn, "$cmd" );
    if ( $output !~ /private files/ ) {
        return 0;
    }
    return 1;
} ## end sub is_dual_inode($)
########################################################################
## @name ::filersio_asyncio_active
##
## @summary
##      filersio is a workload generator that is built into ONTAP.
##      The I/Os are always launched asynchronously
##
## @description
##      This is a thread based workload.
##      It keeps the number of I/Os being serviced by WAFL at any given point in time at a constant.
##
##
##
## @synopsis filersio_asyncio_active($Filer_connection, $read_pct, $rand_pct, $blk_size, $start_byte, $end_byte, $run_time, $num_active, $file)
##
##
## @arg $conn Required:
##      filer connection object.
##
## @return value
##      return value=1 , if Successfull
##      return value=0 , if unsuccessfull
##
sub filersio_asyncio_active {
    my ($Filer_connection, $read_pct,   $rand_pct,
        $blk_size,         $start_byte, $end_byte,
        $run_time,         $num_active, $file
    ) = @_;
    my ($filer_sio_cmd);
    eval {

        # Creates $file_name if it doesan't exist and runs asynchronously
        $Log->comment("<--------- Run filersio asyncio_active --------->\n");
        $filer_sio_cmd
            = "filersio asyncio_active $read_pct -r $rand_pct $blk_size $start_byte $end_byte $run_time $num_active $file -create";
        filer_cli_test_say( $Filer_connection, $filer_sio_cmd )
            or die
            "filersio_asyncio_active: filersio command failed. Command is: $filer_sio_cmd";
    };
    if ($@) {
        return 0;
    }
    return 1;

} ## end sub filersio_asyncio_active
########################################################################
## @name ::filersio_asyncio_pause
##
## @summary
##      filersio is a workload generator that is built into ONTAP.
##      The I/Os are always launched asynchronously
##
## @description
##      This is an I/O based workload.
##      It keeps the I/Os generated per second at a constant.
##
##
##
## @synopsis filersio_asyncio_pause($Filer_connection, $read_pct, $rand_pct, $blk_size, $start_byte, $end_byte, $run_time, $num_active, $pause_count, $file)
##
##
## @arg $conn Required:
##      filer connection object.
##
## @return value
##      return value=1 , if Successfull
##      return value=0 , if unsuccessfull
##
sub filersio_asyncio_pause {
    my ($Filer_connection, $read_pct, $rand_pct, $blk_size,
        $start_byte,       $end_byte, $run_time, $num_active,
        $pause_count,      $file
    ) = @_;
    my ($filer_sio_cmd);
    eval {
        $Log->comment("<--------- Run filersio asyncio_pause --------->\n");
        $filer_sio_cmd
            = "filersio asyncio_pause $read_pct -r $rand_pct $blk_size $start_byte $end_byte $run_time $num_active $pause_count $file";
        filer_cli_test_say( $Filer_connection, $filer_sio_cmd )
            or die
            "filersio_asyncio_pause: filersio command failed. Command is: $filer_sio_cmd";
    };
    if ($@) {
        return 0;
    }
    return 1;

} ## end sub filersio_asyncio_pause
########################################################################
## @name ::filer_sio_status
##
## @summary
##      filersio is a workload generator that is built into ONTAP.
##      The I/Os are always launched asynchronously
##
## @description
##      Print the status of a running workload.
##
##
##
## @synopsis filer_sio_status($Filer_connection, "filersio status")
##
##
## @arg $conn Required:
##      filer connection object.
##
## @return value
##      return value=1 , if Successfull
##      return value=0 , if unsuccessfull
##
########## filer_sio_status ##################################################
sub filer_sio_status {
    my ( $Filer_conection, $scan ) = @_;
    my ( $filer_sio_cmd, $result );
    eval {
        $Log->comment("<--------- Run filer_sio_status  --------->\n");
        $filer_sio_cmd = "filersio status";

        my $status = filer_cli_test_say( $Filer_conection, $filer_sio_cmd )
            or die
            "filersio_asyncio_status: filersio command failed. Command is: $filer_sio_cmd";

        $result = ( $status =~ /$scan/g );
        disconnect($Filer_conection);
    };
    if ($@) {
        return 0;
    }
    return $result;
} ## end sub filer_sio_status

############################################################################
# This function checks if the output returned by mkfile is OK
# Accepts the mkfile output
# Returns 1 on success and 0 on failure
# Refer burt412173 and burt554496
############################################################################
sub _is_mkfile_output_ok {
    my %options = @_;
    my $output  = $options{output};
    #########################################################
    # Normally mkfile returns blank output
    # But with the recent builds it returns some text
    # This array will define all the messages that are
    # considered to be OK (refer burt412173 and burt554496)
    # As and when we see exceptions, this array needs to
    # be updated
    #########################################################
    my @mkfile_exceptions_list
        = ( "Consistency points disabled by test command",
            "file created",
          );

    if ( $output eq "" ) {

        # return success if its blank output
        return 1;
    }
    foreach my $mkfile_exception (@mkfile_exceptions_list) {

        # if the output has the message which is part of
        # of the exception list, the return success
        if ( $output =~ /$mkfile_exception/ ) {
            return 1;
        }
    } ## end foreach my $mkfile_exception...

    # if output is neither blank, nor part of exception list
    # then return failure
    return 0;
} ## end sub _is_mkfile_output_ok

################################################################################
# Function:    get_vbns
# Description: Obtains the vbns for a specified file
# Parameters:  $path - path of the file for which vbns are to be obtained
# Returns:     Returns list of vbns on success and dies on failure
################################################################################

sub get_vbns {

    my ($path) = @_;
    my ( $found, @vbnlist, $output, @lines );
    $found = 0;

    #wafl showvbns will take some time to return the correct pvbn of a file
    #hence using wafl flush cache

    logcomment("wafl showvbns $path");
    $output = cli( $::FILER,
        "priv set -q diag; wafl flush cache; wafl showvbns $path" );

    @lines = split( /\n/, $output );

    foreach my $line (@lines) {
        if ( $line =~ /fbn\s(\d+)\sis\svbn\s(\d+),\svvbn\s(\d+)/ ) {
            push @vbnlist, $3;
        } else {
            $found = 1;
        }
    } ## end foreach my $line (@lines)
    if ($found) {
        die("Failed to get vvbns: $@");
    }

    return (@vbnlist);

} ## end sub get_vbns

################################################################################
# Function:   	sv_filer_service_license
# Description: 	Install sv_ontap_pri and sv_ontap_sec licenses based on the
#			   	the following licensing work flows:
#
#              	a) For legacy (non FAS2040) hw, install sv_ontap_pri and
#     		 	sv_ontap_sec separately as needed.
#
#				b) For FAS2040, install snaprestore if you want sv_ontap_pri;
#				install snapmirror if you want sv_ontap_sec.
#
#				c) For new hw (Carnegie/Absolut and VSIM/VSA), install
#				sv_ontap_sec to get sv_ontap_pri.
#
# Parameters: 	$fc - Filer connection
#				$service - license to install
# Returns:     	Returns 1 on success and dies on failure
################################################################################

sub sv_filer_service_license {

    my ( $fc, $service ) = @_;

    my $filer_model = filer_model_get($fc);

    if ( $service eq 'sv_ontap_pri' ) {
        if (   ( $filer_model =~ "DOvM" )
            || ( $filer_model =~ "FCvM" )
            || ( $filer_model =~ "SIMBOX" )
            || ( $filer_model =~ "FAS32" )
            || ( $filer_model =~ "FAS62" ) 
            || ( $filer_model =~ "FAS22" ) )
        {
            logcomment("license sv_ontap_sec if needed");
            if ( !is_licensed( $fc, 'sv_ontap_sec' ) ) {
                filer_service_license( $fc, 'sv_ontap_sec' ) or die $@;
            }
        } elsif ( $filer_model =~ "FAS2040" ) {
            logcomment(
                "license snaprestore if needed, sv_ontap_pri will be enabled"
            );
            if ( !is_licensed( $fc, 'snaprestore' ) ) {
                filer_service_license( $fc, 'snaprestore' ) or die $@;
            }

            if ( !is_licensed( $fc, 'sv_ontap_pri' ) ) {
                die("sv_ontap_pri license is not enabled");
            }

        } else {
            logcomment("license sv_ontap_pri if needed");
            if ( !is_licensed( $fc, 'sv_ontap_pri' ) ) {
                filer_service_license( $fc, 'sv_ontap_pri' ) or die $@;
            }
        } ## end else [ if ( ( $filer_model =~...
    } ## end if ( $service eq 'sv_ontap_pri')

    elsif ( $service eq 'sv_ontap_sec' ) {
        if ( $filer_model =~ "FAS2040" ) {

            logcomment(
                "license snapmirror if needed, sv_ontap_sec will be enabled");
            if ( !is_licensed( $fc, 'snapmirror' ) ) {
                filer_service_license( $fc, 'snapmirror' ) or die $@;
            }

            if ( !is_licensed( $fc, 'sv_ontap_sec' ) ) {
                die("sv_ontap_sec license is not enabled");
            }
        } ## end if ( $filer_model =~ "FAS2040")

        elsif ( !is_licensed( $fc, 'sv_ontap_sec' ) ) {
            filer_service_license( $fc, 'sv_ontap_sec' ) or die $@;
        }
    } else {
        die( "Only sv_ontap_pri or sv_ontap_sec service can be licensed here"
        );
    }

    return 1;

} ## end sub sv_filer_service_license

################################################################################
# Function:   	sv_filer_service_unlicense
# Description: 	Uninstall sv_ontap_pri and sv_ontap_sec licenses based on the
#			   	the following licensing work flows:
#
#              	a) For legacy (non FAS2040) hw, install sv_ontap_pri and
#     		 	sv_ontap_sec separately as needed.
#
#				b) For FAS2040, install snaprestore if you want sv_ontap_pri;
#				install snapmirror if you want sv_ontap_sec.
#
#				c) For new hw (Carnegie/Absolut and VSIM/VSA), install
#				sv_ontap_sec to get sv_ontap_pri.
#
# Parameters: 	$fc - Filer connection
#				$service - license to uninstall
# Returns:     	Returns 1 on success and dies on failure
################################################################################
sub sv_filer_service_unlicense {

    my ( $fc, $service ) = @_;

    my $filer_model = filer_model_get($fc);
    if ( $service eq 'sv_ontap_pri' ) {
        if (   ( $filer_model =~ "DOvM" )
            || ( $filer_model =~ "FCvM" )
            || ( $filer_model =~ "SIMBOX" )
            || ( $filer_model =~ "FAS32" )
            || ( $filer_model =~ "FAS62" ) 
            || ( $filer_model =~ "FAS22" ) )
        {
            logcomment("unlicense sv_ontap_sec if needed");
            if ( is_licensed( $fc, 'sv_ontap_sec' ) ) {
                unlicense( $fc, 'sv_ontap_sec' ) or die $@;
            }
        } elsif ( $filer_model =~ "FAS2040" ) {
            logcomment("unlicense snaprestore if needed");
            if ( is_licensed( $fc, 'snaprestore' ) ) {
                unlicense( $fc, 'snaprestore' ) or die $@;
            }

            logcomment("unlicense sv_ontap_pri if needed");
            if ( is_licensed( $fc, 'sv_ontap_pri' ) ) {
                unlicense( $fc, 'sv_ontap_pri' ) or die $@;
            }

        } else {
            logcomment("unlicense sv_ontap_pri if needed");
            if ( is_licensed( $fc, 'sv_ontap_pri' ) ) {
                unlicense( $fc, 'sv_ontap_pri' ) or die $@;
            }
        } ## end else [ if ( ( $filer_model =~...
    } ## end if ( $service eq 'sv_ontap_pri')

    elsif ( $service eq 'sv_ontap_sec' ) {
        if ( $filer_model =~ "FAS2040" ) {

            logcomment("unlicense snapmirror if needed");
            if ( is_licensed( $fc, 'snapmirror' ) ) {
                unlicense( $fc, 'snapmirror' ) or die $@;
            }
        } ## end if ( $filer_model =~ "FAS2040")
        elsif ( is_licensed( $fc, 'sv_ontap_sec' ) ) {
            unlicense( $fc, 'sv_ontap_sec' ) or die $@;
        }
    } else {
        die("Only sv_ontap_pri or sv_ontap_sec service can be unlicensed here"
        );
    }

    return 1;

} ## end sub sv_filer_service_unlicense

# ------------------------------------------------------------------------
1;

# ------------------------------------------------------------------------
