#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/IFT/IFTSetup.pm#1 $
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.

## @summary        Setup library specifically for the WAFL QA Integreated
## Functional test suite.
## @description
##             Client queries, stats
##
## @author bundy

use strict;

package IFTFilerSetup;
use Tharn;
use Aggregate qw(aggr_create aggr_purge aggr_exists aggr_status);
use Volume qw(vol_create vol_create_flex vol_purge vol_exists vol_status
  vol_options vol_online vol_root);
use SayWrap qw(sayEOF);
use ExportFS qw(export_path unexport_path list_exports);
use Admin::License qw(license unlicense is_licensed);
use Utils qw(make_conn);
use Data::Dumper;
use SPARSE::Util::Logger qw(logStep logWarn logDbg logError);
use setup qw(iscsi_setup fcp_setup is_fcp_setup);
use filer qw(filer_service_stop);
use Quota qw(setQuotaList enableQuota disableQuota
               writeQuotaFile readQuotaFile setSingleQuota deleteQuota
               getQuotaReport enableQuota disableQuota);
use QtreeCmds
  qw(qtreeList qtreeStatus qtreeCreate qtreeDelete qtreePresent qtreeSecurity);
use Users qw(createUsers createGroups readPasswdFile readGroupFile
  delPasswdEntry delGroupEntry);
use Fcntl qw(:DEFAULT :flock);

# Need to increase default timeout when doing qtree status, vol status, etc.
my $MIN_CMDTIMEOUT = 120;
$main::CMDTIMEOUT = $MIN_CMDTIMEOUT if ($main::CMDTIMEOUT < $MIN_CMDTIMEOUT);

### use SanClient;
### use SanClient::Host::LinuxHost;
### use SanClient::Host::SolarisHost;

use IFT::IFTClient;

use constant MAX_AGGRS => 100;
use constant MAX_VVOLS => 500;
use constant MAX_TVOLS => 100;        # ???
use constant REUSE     => 'reuse';
use constant CREATE    => 'create';
our $RETRIES = 3;
our $AGGR    = "aggr";
our $FLEX    = "flex";
our $TRAD    = "trad";
our $LUNS    = "LUNs";
our $QTREES  = "qtrees";
our $ACK = "ACK";
our $NCK = "NCK";
our $PURGE   = "purge";
our($fs);

my $MIN_CMDTIMEOUT = 360;
$main::CMDTIMEOUT = $MIN_CMDTIMEOUT if ($main::CMDTIMEOUT < $MIN_CMDTIMEOUT);

## new is called with the class (IFTFilerSetup::new) and a filer info hash
## that contains
##  fname = full hostspec of filer i.e. u43;hosttype=filer

sub new( $$ ) {
    my ( $class, $filer, %opts ) = @_;
    my $self = {};

    #    print "DBGCEB: filer = ",Dumper($filer);
    #    getc; #DBGCEB

    _initialize_filer_object( $self, $filer, %opts );
    connect_to_filer( $self, $filer );
    return $self if ( $self->{error} );

#    if (!$self->{fc} || ref($self->{fc}) !~ /Connectrec/) {
#        $self->{error} = "IFTSetup::new:  Could not connect to filer: $self->{hostname}";
#    }

    bless $self, $class;
    return $self;
}

sub connect_to_filer {
    my $self    = shift;
    my $retries = $RETRIES;
    my $text    = undef;
    my $fspec   = undef;
    my $fname;

    if ( $self->{fc} && ref( $self->{fc} ) =~ /Connectrec/ ) {
        $fspec = $self->{fc};
    } elsif ( $self->{fh} && ref( $self->{fh} ) =~ /Hostrec/ ) {
        $fspec = $self->{fh};
    } elsif ( $self->{fullname} ) {
        $fspec = $self->{fullname};
    } else {
        $self->{error} =
          "connect_to_filer:  No 'fc', 'fh', or 'fullname' "
          . "values given for filer";
        return;
    }
    while ($retries) {
        $self->{error} = undef;
        if ( $self->{conn} ) {
            eval {
                $self->{fc} = make_conn( $fspec, $self->{conn} );
                $text = sayEOF( $self->{fc}, "hostname" );
            };
        } else {
            eval {
                $self->{fc} = make_conn($fspec);
                $text = sayEOF( $self->{fc}, "hostname" );
            };
        }
        last if ( $text eq $self->{fname} );
        $self->{error} = "Cannot connect to $self->{fname}: $@";
        logresult( 'INFO', "connect_to_filer:  $self->{fname}   ERROR: $@" );
        $retries--;
        sleep 90;
    }

    if ( !$self->{error} ) {
        $self->{fc}->timeout($self->{timeout})
            if (defined($self->{timeout}));

        my $fh = $self->{fc}->hostp();
        $self->{fh} = $fh;
        my $hostname = $fh->hostname();
        $hostname =~ s/\..*//;
        $self->{fname}    = $hostname;
        $self->{hostname} = $hostname;
        $self->{hosttype} = $fh->hosttype();
        $self->{ipaddr}   = $self->{fh}->default_ip();
    }

    return;
}

sub test_filer {
    my $self  = shift;
    my $error = 0;
    eval { sayEOF( $self->{fc}, "hostname" ) or $error = 1; };
    $error = 1 if $@;
    return if !$error;
    $self->connect_to_filer();
    return;
}

sub process_error {
    my $self  = shift;
    my $error = shift;
    $self->test_filer();
    if ( defined $self->{error} ) {
        $self->{error} = $error;
        return;
    }
}

sub _initialize_filer_object {
    my $self     = shift;
    my $filer    = shift;
    my %opts     = (@_);
    my $licenses = undef;

    if ( !$filer ) {
        if ( !keys(%opts) ) {
           $self->{error} = "IFTSetup::new:  FILER undefined";
           return;
        }
        $filer = {%opts};
    }
    if ( !ref($filer) ) {
       $self->{fullname} = $filer;
    } elsif ( ref($filer) =~ /IFTFilerSetup/ ) {
        $self->{fullname} = $filer->{fullname} if (exists $filer->{fullname});
        $self->{fullname} = $filer->{fname} if (!$self->{fullname});
        $self->{fname}    = $filer->{fname};
    } elsif ( ref($filer) eq "HASH" ) {
        if ( $filer->{fullname} ) {
            $self->{fullname} = $filer->{fullname};
        } elsif ( $filer->{hostname} ) {
            $self->{fullname} = $filer->{hostname};
        } else {
            $self->{fullname} = $filer->{fname};
        }
    }
    if (!exists $self->{fname}) {
       my ( $fname, @rest ) = split ( /;/, $self->{fullname} );
       $self->{fname} = $fname;
    }

    if ( ref($filer) && $filer->{licenses} ) {
        $self->{cfginfo}{licenses} = $filer->{licenses};
    } else {
        $self->{cfginfo}{licenses} = $opts{licenses}
          if ( exists $opts{licenses} );
    }

    if ( !ref( $self->{cfginfo}{licenses} ) ) {
        my @lic_list = split ( /\W/, $self->{cfginfo}{licenses} );
        $self->{cfginfo}{licenses} = [@lic_list];
    }

    my $fields = $self->{fullname};
    my ( $filer, $rest ) = ( $fields =~ /(\w+)((\;.*)*)/ );
    foreach ( split ( /;/, $rest ) ) {
        if (/(\w+)=(.*)/) {
            $self->{$1} = $2;
        }
    }
    $self->{fname} = $filer if ( !exists $self->{fname} );
    $self->{conn}  = $opts{conn};
    $self->{conn}  = $main::FILER_CONN if ( !$self->{conn} );
    my $hostp = host( $self->{fullname} );
    if ( !$hostp ) {
        $self->{error} =
"_initialize_filer_object:  host() failed for filer: \"$self->{fullname}\": $@";
    }
    $self->{hosttype}               = $hostp->hosttype();
    $self->{hostname}               = $hostp->hostname();
    $self->{default_ip}             = $hostp->default_ip();
    $self->{cfginfo}{$AGGR}{prefix} =
      ( exists $opts{aggr_prefix} )
      ? $opts{aggr_prefix}
      : $main::AGGR_PREFIX;
    $self->{cfginfo}{$FLEX}{prefix} =
      ( exists $opts{flex_prefix} )
      ? $opts{flex_prefix}
      : $main::FLEX_PREFIX;
    $self->{cfginfo}{$TRAD}{prefix} =
      ( exists $opts{trad_prefix} )
      ? $opts{trad_prefix}
      : $main::TRAD_PREFIX;
    $self->{cfginfo}{$AGGR}{count} =
      ( exists $opts{aggr_count} )
      ? $opts{aggr_count}
      : int($main::AGGR_COUNT);
    $self->{cfginfo}{$FLEX}{count} =
      ( exists $opts{flex_count} )
      ? $opts{flex_count}
      : int($main::FLEX_COUNT);
    $self->{cfginfo}{$TRAD}{count} =
      ( exists $opts{trad_count} )
      ? $opts{trad_count}
      : int($main::TRAD_COUNT);
    $self->{cfginfo}{mode} =
      ( exists $opts{mode} )
      ? $opts{mode}
      : $main::MODE;
    $self->{cfginfo}{$FLEX}{size} =
      ( exists $opts{flex_size} )
      ? $opts{flex_size}
      : $main::FLEX_SIZE;
    $self->{cfginfo}{$TRAD}{size} =
      ( exists $opts{trad_size} )
      ? $opts{trad_size}
      : $main::TRAD_SIZE;
    $self->{cfginfo}{$AGGR}{size} =
      ( exists $opts{aggr_size} )
      ? $opts{aggr_size}
      : $main::AGGR_SIZE;
    $self->{cfginfo}{$AGGR}{raid} =
      ( exists $opts{aggr_raid} )
      ? $opts{aggr_raid}
      : $main::AGGR_RAID;
    $self->{cfginfo}{$TRAD}{raid} =
      ( exists $opts{trad_raid} )
      ? $opts{aggr_raid}
      : $main::TRAD_RAID;
    $self->{cfginfo}{$LUNS}{count} =
      ( exists $opts{lun_count} )
      ? $opts{lun_count}
      : $main::LUN_COUNT;
    $self->{cfginfo}{$LUNS}{prefix} =
      ( exists $opts{lun_prefix} )
      ? $opts{lun_prefix}
      : $main::LUN_PREFIX;
    $self->{cfginfo}{$LUNS}{size} =
      ( exists $opts{lun_size} )
      ? $opts{lun_size}
      : $main::LUN_SIZE;
    $self->{cfginfo}{DO_QTREES} =
      ( exists $opts{qtrees} )
      ? $opts{qtrees}
      : $main::QTREES;
    $self->{cfginfo}{$QTREES}{count} =
      ( exists $opts{qtree_count} )
      ? $opts{qtree_count}
      : $main::QTREE_COUNT;
    $self->{cfginfo}{$QTREES}{prefix} =
      ( exists $opts{qtree_prefix} )
      ? $opts{qtree_prefix}
      : $main::QTREE_PREFIX;
    $self->{cfginfo}{quotas} =
      ( exists $opts{quotas} )
      ? $opts{quotas}
      : $main::QUOTAS;
    $self->{cfginfo}{containers}{types} = $opts{containers}
      if ( exists $opts{containers} );

    $self->{timeout} =
      ( exists $opts{CMDTIMEOUT} )
      ? $opts{CMDTIMEOUT}
      : $main::CMDTIMEOUT;

    return;
}

# ------------------------------------------------------------------------

sub read_config_file {
    my $self    = shift;
    my $cfgfile = shift;
    my (%opts) = @_;
    if ( !-e $cfgfile ) {
        die ("read_config_file:  File not found: \"$cfgfile\"");
    }
    my $pkg = $opts{pkg};
    my $config_hash = {};
    open( CFG, "<$cfgfile" )
      or die ("read_config_file:  Could not open \"$cfgfile\" for reading: $!");

    while (<CFG>) {
        chomp($_);
        $_ =~ s/^\s*//;
        $_ =~ s/\s*$//;
        if ( !$_ || $_ =~ /^#/ ) {
            next;
        }
        if ( $_ !~ /^(\w+)\s*=\s*(.*)/ ) {
            die ("read_config_file:  Invalid entry: \"$_\"");
        }
        my $name = $1;
        my $val = $2;
        my $var;
        $config_hash->{$name} = $val;
        if ($pkg) {
           $var = "\$" . $pkg . "::" . $name;
        } else {
           $var = "\$" . $name;
        }
        eval("$var = \"$val\"");
    }
    close(CFG);
    return ($config_hash);
}

# ------------------------------------------------------------------------

sub setup_licenses {
    my $self = shift;
    $self->{error} = undef;
    if ( !exists $self->{cfginfo}{licenses} || !$self->{cfginfo}{licenses} ) {
        $self->{error} = "No licenses were defined";
        return;
    }
    foreach my $license ( @{ $self->{cfginfo}{licenses} } ) {
        $self->{licenses}{$license} = 0;
        logresult( 'INFO', "Checking license: $license" );
        if ( !is_licensed( $self->{fc}, $license ) ) {
            logresult( 'INFO', "Licensing: $license" );
            license( $self->{fc}, $license )
              or $self->{error} = "self->{fname}: cannot set $license";
            last if ( $self->{error} );
            $self->{licenses}{$license} = 1;
        }
    }
    return;
}

sub setup_space_mgmt {
    my $self  = shift;
    my @guars = ( "volume", "file" );

    my $count = 10;
    $self->get_existing_containers() if ( !exists $self->{containers} );
    if ( !$self->{cfginfo}{$FLEX}{count} ) {
        $self->{error} =
          warn_result(
            "setup_space_mgmt:$self->{fname} " . "no flex vols exist." );
        return;
    }

    foreach my $vol ( keys %{ $self->{containers}{volume} } ) {
        next if ( $self->{containers}{volume}{type} ne "flex" );
        $count--;
        next if ( $count > 0 );
        my $idx = rand( ($#guars) );
        logcomment( "setup_space_mgmt:$self->{fname}:$vol setting guarantee to"
              . " $guars[$idx]" );
        vol_options( $self->{fc}, $vol, guarantee => $guars[$idx] )
          or $self->{error} =
          "setup_space_mgmt:$self->{fname} "
          . "unable to set $vol guarantee to $guars[$idx]: $@";
        $self->{containers}{volume}{$vol}{guarantee} = $guars[$idx]
          if ( !$self->{error} );

        if ( ( $guars[$idx] eq 'file' ) && ( !$self->{error} ) ) {
            my $fract = rand(100);
            logcomment( "setup_space_mgmt:$self->{fname}:$vol setting "
                  . " fractional reserve to $fract" );
            vol_options( $self->{fc}, $vol, fractional_reserve => $fract )
              or $self->{error} =
              "setup_space_mgmt:$self->{fname} "
              . "unable to set $vol fractional_reserve to $fract: $@";
            $self->{containers}{volume}{$vol}{freserve} = $fract
              if ( !$self->{error} );
        }

        if ( $self->{error} ) {
            logWarn( $self->{error} );
            $self->{error} = "";
        } else {
            logcomment( "setup_space_mgmt:$self->{fname}:$vol successfully set "
                  . "space mgmt." );
        }
    }
}

sub get_existing_containers {
    my $self    = shift;
#    my @mytypes = ( $AGGR, $TRAD, $FLEX, $QTREES, $LUNS );
    my @mytypes = ( $AGGR, $TRAD, $FLEX );
    my $types   = $self->{cfginfo}{containers}{types};
    my $subs    = {
        $AGGR   => \&_get_existing_aggrs,
        $TRAD   => \&_get_existing_tradvols,
        $FLEX   => \&_get_existing_flexvols,
        $QTREES => \&_get_existing_qtrees,
        $LUNS   => \&_get_existing_luns,
    };
    my $key         = undef;
    my $export_vols = undef;

    $types = \@mytypes if ( !$types );

    foreach my $type ( @{$types} ) {
        if ( ( $type eq $TRAD ) or ( $type eq $FLEX ) ) {
            $key         = "volume";
#            $export_vols = $self->{cfginfo}{$FLEX}{export}
#              || $self->{cfginfo}{$TRAD}{export};
#            $export_vols = 1 unless ( defined($export_vols) );
        } else {
            $key = $type;
        }

        $self->{containers}{count}{$type} = $subs->{$type}($self);
        return if ( $self->{error} );
        foreach my $thing ( keys %{ $self->{containers}{$type} } ) {
            if (exists $self->{containers}{$key}{$thing}{$AGGR}) {
               $self->{containers}{$AGGR}
                 {$self->{containers}{$key}{$thing}{$AGGR}}{kids}{$type}{$thing}
                    = 1;
            }
            if (exists $self->{containers}{$key}{$thing}{volume}) {
                $self->{containers}{volume}
                 {$self->{containers}{$key}{$thing}}{kids}{$type}{$thing}= 1;
            }
            if (exists $self->{containers}{$key}{$thing}{qtree}) {
                $self->{containers}{qtree}
                 {$self->{containers}{$key}{$thing}}{kids}{$type}{$thing}= 1;
            }
        }

        # DBGCEB put these back
               $self->_export_vols();
#                   unless (($export_vols == 0) && ($key ne "containers"));
    }

    $self->{containers}{count}{volume} =
        $self->{containers}{count}{flex} +  $self->{containers}{count}{trad};

   # if the container is contained in something else, add the container
   # and type to the parent container.
    my @types = ("volume", $QTREES, $LUNS);
    foreach my $type (@{$types}) {
     next if (!exists $self->{containers}{$type});
     foreach my $thing (keys %{$self->{containers}{$type}}) {
            if (exists $self->{containers}{$type}{$thing}{$AGGR}) {
               $self->{containers}{$AGGR}
               {$self->{containers}{$type}{$thing}{$AGGR}}{kids}{$type}{$thing}
                     = 1;
            }
            if (exists $self->{containers}{$type}{$thing}{volume}) {
                $self->{containers}{volume}
                 {$self->{containers}{$type}{$thing}}{kids}{$type}{$thing}= 1;
            }
            if (exists $self->{containers}{$type}{$thing}{qtree}) {
                $self->{containers}{$QTREES}
                 {$self->{containers}{$type}{$thing}}{kids}{$type}{$thing}= 1;
            }
     }
   }
    return;
}

# sub setup_aggreagates
# input: $self - IFTFilerSetup obj
# options: $opt{mode} or $self->{cfginfo}{mode} - PURGE will purge existing
#          aggregates
#          $opt{aginfo} or $self->{cfginfo}{aggr} - cfg info hash for aggrs
#              {count} - number of aggregates to create
#              {prefix} - aggr prefix
#              {size} - size of aggregates
#              {raid} - raid type

sub setup_aggregates {
    my $self    = shift;
    my $voltype = $AGGR;
    $self->{error} = undef;
    my $aginfo = $self->{cfginfo}{$voltype};
    my $mode    = $self->{cfginfo}{mode};
    my $needed  = $aginfo->{count};
    if ($needed > MAX_AGGRS) {
        $needed = MAX_AGGRS;
    }

    $self->_get_existing_aggrs();
    return 0 if ( $self->{error} );

    if ( $mode eq $PURGE ) {
        $self->_purge_aggregates();
        return 0 if ( $self->{error} );
    }
    my $num_existing = ( keys %{ $self->{containers}{$voltype} } );
    my $new = $needed - $num_existing;
    if ($new > 0) {
        $aginfo->{new} = $new;
        $self->_create_aggregates();
        return   if ( $self->{error} );
    }
    
    return;
}

sub cleanup_aggrs {
    my $self = shift;
    foreach my $aggr ( keys %{ $self->{containers}{$AGGR} } ) {
        next if ( !exists $self->{containers}{$AGGR}{new} );
        next if ( !aggr_exists( $self->{fc}, $aggr ) );
        eval { aggr_purge( $self->{fc}, $aggr ); };
        logWarn("Unable to purge $self->{fname}:$aggr:$@") if $@;
    }
}

sub cleanup_flexvols {
    my $self = shift;
    foreach my $vol ( keys %{ $self->{containers}{$FLEX} } ) {
        next if ( !exists $self->{containers}{$FLEX}{new} );
        next if ( !vol_exists( $self->{fc}, $vol ) );
        eval { vol_purge( $self->{fc}, $vol ); };
        logWarn("Unable to purge $self->{fname}:$vol:$@") if $@;
    }
}

sub _purge_aggregates {
    my $self   = shift;
    my $error  = undef;
    my $prefix = $self->{cfginfo}{$AGGR}{prefix};
    logcomment( "_purge_aggregates:$self->{fname}: purge aggr with "
          . "prefix $prefix" );
    foreach my $aggr ( keys %{ $self->{containers}{$AGGR} } ) {
        next if ( ($prefix) && ( $aggr !~ /$prefix/ ) );
        my $retries = $RETRIES;
      RETRYIT:
        eval { aggr_purge( $self->{fc}, $aggr ); };
        $error =
          "purge_aggregates:$self->{fname}: could not purge aggregates:$@"
          if $@;

        if ($error) {
            $self->process_error($error);
            $error = undef;
            return if ( $self->{error} );
            $retries--;
            goto RETRYIT if $retries;
            $self->{error} = $error;
            return;
        }
        delete $self->{containers}{aggrs}{$aggr};
        $self->{containers}{count}{$AGGR}--;
    }
    return 1;
}

sub _create_aggregates {
    my $self   = shift;
    my $aginfo = $self->{cfginfo}{$AGGR};

    my $prefix = $aginfo->{prefix} ? $aginfo->{prefix} : 'testag';
    my $count  = $aginfo->{count}  ? $aginfo->{count}  : 1;
    logcomment("_create_aggregates:$self->{fname}:Creating $count Aggregates");
    my $raidtype = $aginfo->{raid} ? $aginfo->{raid} : 'raid_dp';
    my $size     = $aginfo->{size} ? $aginfo->{size} : 3;
    my $cntr     = 0;
    my $error    = undef;
    for ( 1 .. $count ) {
        my $name    = $prefix . ( time() );
        my $retries = $RETRIES;
      RETRYIT:
        eval {
            aggr_create( $self->{fc}, $name, $size, raidtype => $raidtype );
        };
        $error = sprintf "%s: cannot create %s :$@", $self->{fname}, $name
          if $@;

        if ($error) {
            $self->process_error($error);
            $error = undef;
            return if ( $self->{error} );
            $retries--;
            goto RETRYIT if $retries;
            $self->{error} = $error;
            return;
        }

#        print "DBGCEB:ERROR:_create_aggregates:$self->{fname}:$self->{error}\n"
#            if $self->{error};

        $cntr++;
        $self->{containers}{count}{$AGGR} += 1;
        logcomment("$self->{fname}:created aggr $name: $cntr aggrs created");
        my $newaggr = {};
        $newaggr->{size} = $size;
        $newaggr->{raid} = $raidtype;
        $newaggr->{new}  = 1;
        $self->{containers}{$AGGR}{$name} = $newaggr;
        $self->{containers}{count}{$AGGR}++;
    }
    return;
}

sub setup_flex_vols {
    my $self    = shift;
    my $voltype = $FLEX;
    $self->{error} = undef;
    my $volinfo = $self->{cfginfo}{$voltype};
    my $mode    = $self->{cfginfo}{mode};
    my $needed  = $volinfo->{count};
    if ($needed > MAX_VVOLS) {
        $needed = MAX_VVOLS
    }

logresult('INFO', "setup_flex_vols  Line: ".__LINE__."needed: $needed");
    # processing existing matching flexvols
    $self->_get_existing_flexvols();
        return 0 if ( $self->{error} );
    if ( $mode eq $PURGE ) {
        $self->_purge_volumes($voltype);
        return 0 if ( $self->{error} );
    }
    my $num_existing = ( keys %{ $self->{containers}{$voltype} } );
    my $new = $needed - $num_existing;
    logresult('INFO', "setup_flex_vols  Line: ".__LINE__."   needed: $needed  num_existing: $num_existing  new: $new");
    if ($new > 0) {
        # get existing matching aggrs
        $self->_get_existing_aggrs();
        my $eaggr_count = ( keys %{ $self->{containers}{$AGGR} } );
        if ( !$eaggr_count ) {
            $self->setup_aggregates();
            $eaggr_count = ( keys %{ $self->{containers}{$AGGR} } );
        }
        if ( !$eaggr_count ) {
            $self->{error} = "setup_flex_vols: No aggregates available";
            return;
        }
        my $per_aggr = int( $new / $eaggr_count );
        my $rem = $new % $eaggr_count;
        # create new flexvols
        my @aggrs = keys( %{ $self->{containers}{$AGGR} }); 
#logresult('INFO', "setup_flex_vols  Line: ".__LINE__."   new: $new  eaggr_count: $eaggr_count  per_aggr: $per_aggr  rem: $rem"); sleep(10);
        if ($per_aggr) {
            foreach my $aggr ( @aggrs ) {
#logresult('INFO', "setup_flex_vols  Line: ".__LINE__."   creating $per_aggr flexvol(s) in aggr $aggr"); sleep(5);
                $self->_create_flexvols( $per_aggr, $aggr );
                return 0 if $self->{error};
                $rem -= $per_aggr;
	    }
        }
        if ($rem) {
            foreach my $aggr ( @aggrs ) {
                last unless ($rem);
#logresult('INFO', "setup_flex_vols  Line: ".__LINE__."   REM: $rem   creating 1 flexvol in aggr $aggr"); sleep(5);
                $self->_create_flexvols( 1, $aggr );
                return 0 if $self->{error};
                $rem--;
            }
        }
    }
    $self->_export_vols();
    return;
}

sub _get_existing_aggrs {
    my $self     = shift;
    return ( $self->_get_existing_aggrs_tradvols($AGGR) );
}

sub _get_existing_flexvols {
    my $self    = shift;
    return ( $self->_get_existing_vols_aux($FLEX) );
}

sub _get_existing_tradvols {
    my $self    = shift;
#    return ( $self->_get_existing_vols_aux($TRAD) );
    return ( $self->_get_existing_aggrs_tradvols($TRAD) );
}

sub _get_existing_vols_aux {
    my $self    = shift;
    my $voltype = shift;
    my $error   = undef;

    #    print "\nDBGCEB: _get_existing_vols_aux\n";
    my $prefix = $self->{cfginfo}{$voltype}{prefix}
      if ( exists $self->{cfginfo}{$voltype}{prefix} );
    $prefix = "test" . $voltype unless ($prefix);

    my $cmd     = "vol status";
    my $retries = $RETRIES;
    my $text    = undef;
  RETRYIT:
    eval { $text = sayEOF( $self->{fc}, $cmd ); };
    $error = "$self->{fname}: " . "vol status command failed: $@"
      if ( ($@) || ( !$text ) );

    if ($error) {
        $self->process_error($error);
        $error = undef;
        return if ( $self->{error} );
        $retries--;
        goto RETRYIT if $retries;
        return 0;
    }

    return if $self->{error};
    my $flex_count = 0;
    my $trad_count = 0;

    foreach my $type ( ($AGGR, $FLEX, $TRAD) ) {
       if ( !ref($self->{cfginfo}{$type}{existing}) ) {
           $self->{cfginfo}{$type}{existing} = {};
           $self->{containers}{$type} = $self->{cfginfo}{$type}{existing};
           $self->{containers}{count}{$type} = 0;
       }
    }
    if ( !ref($self->{containers}{volume}) ) {
        $self->{containers}{volume} = {};
        $self->{containers}{count}{volume} = 0;
    }

    my ( $str, @lines, $name, $line, @text, );
    @lines = split ( /\n/, $text );

    foreach $line (@lines) {
        if ( $line !~ /online|offline/ ) {
            $str .= $line;
            push @text, $str;
            $str = undef;
        } elsif ( $line =~ /guarantee/ ) {
            push @text, $str if ($str ne undef);
            $str = $line;
            push @text, $str;
            $str = undef;
        } else {
            push @text, $str if ($str ne undef);
            $str = $line;
        }
    }
    push @text, $str if ($str ne undef);

    my $dbgcnt = 0;    #DBGCEB
    foreach $line (@text) {

        my ( $type, $raid, $status, $opts );
               print "DBGCEB:_get_existing_vols_aux:line = $line\n";
        next if ($line =~ /^Volume/);

        if (
#        (
        $line =~ /(\w+)\s+(online|offline)\s*(raid\w+)\,\s+(\w+)/)
 #       ||
 #       (
 #           $line =~ /^
 #           (\w+)\s+
 #           (online|offline)\s+
 #           (raid[\_\w]+)\,\s+
 #           (\w+)\s+
 #               /x
 #         ))
        {
            $name   = $1;
            $status = $2;
            $raid   = $3;
            $type   = $4;
        } else {
            next;
        }
        next if (!$name);
        next if ($voltype && $type ne $voltype);
        next if ($prefix && $name !~ /^$prefix/);
        if ( $status eq "offline" ) {
            vol_online( $self->{fc}, $name )
                or logWarn( "_get_existing_vols_aux:$self->{fname}:$name"
                      . "cannot be onlined: $@" );
            }
        my $newvol;
        if ( ref($self->{cfginfo}{$type}{existing}{$name}) ) {
            $newvol = $self->{cfginfo}{$type}{existing}{$name};
        } else {
            $newvol = {};
            $self->{cfginfo}{$type}{existing}{$name} = $newvol;
        }
        $newvol->{status}  = $status;
        $newvol->{raid}    = $raid;
        $newvol->{name}    = $name;
        $newvol->{opts}    = $opts;
        $newvol->{type}    = $type;
        $newvol->{path}    = "/vol/$name";
        $newvol->{sm_stat} = $1
          if ( $line =~ /snapmirrored\=(\w+)/ );
        $newvol->{lw_protect} = $1
          if ( $line =~ /lost_write_protect\=(\w+)/ );

        $self->{containers}{volume}{$name} = $newvol;
        if ($type eq $FLEX) {
            $flex_count++;
        } else {
            $trad_count++;
        }
        my $vol = $name;
        $cmd = "vol status $vol";
        my $text    = undef;
        my $retries = $RETRIES;
      RETRYIT:
        eval { $text = sayEOF( $self->{fc}, $cmd ); };
        $error = "$self->{fname}: " . "vol status command failed: $@"
          if ( ($@) || ( !$text ) );
        if ($error) {
            $self->process_error($error);
            $error = undef;
            return if ( $self->{error} );
            $retries--;
            goto RETRYIT if $retries;
            logWarn($error);
            next;
        }

        while ( $text =~ /\GContaining\sAggregrate\:\s\'(\w)/smgx ) {
            if ($1 =~ "N/A") {
                $newvol->{$AGGR} = undef;
            } else {
                $newvol->{$AGGR} = $1;
            }
            last;
        }
    }
    if (!$voltype) {
        $self->{containers}{count}{$FLEX} = $flex_count;
        $self->{containers}{count}{$TRAD} = $trad_count;
    } elsif ($voltype eq $FLEX) {
        $self->{containers}{count}{$FLEX} = $flex_count;
    } else {
        $self->{containers}{count}{$TRAD} = $trad_count;
    }
    #   $Data::Dumper::Maxdepth = 2;
}

sub _purge_volumes {
    my $self    = shift;
    my $voltype = shift;
    my $prefix  = $self->{cfginfo}{$voltype}{prefix};
    logcomment( "_purge_volumes:$self->{fname}:purging $voltype volumes:"
          . " prefix = $prefix" );
    foreach my $vol ( keys %{ $self->{containers}{$voltype} } ) {
        next if ( ($prefix) && ( $vol !~ /$prefix/ ) );
        my $retries = $RETRIES;
      RETRYIT:
        my $error = undef;
        eval { vol_offline( $self->{fc}, $vol ); };
        $error = "_purge_volumes:$self->{fname}: " . "could not offline $vol:$@"
          if $@;
        if ($error) {
            $self->process_error($error);
            $error = undef;
            return if ( $self->{error} );
            $retries--;
            goto RETRYIT if $retries;
            $self->{error} = $error;
            return;
        }

        vol_destroy( $self->{fc}, $vol )
          or $error =
          "_purge_volumes:$self->{fname}: " . "could not destroy $vol:$@";
        if ($error) {
            $self->process_error($error);
            $error = undef;
            return if ( $self->{error} );
            $retries--;
            goto RETRYIT if $retries;
            $self->{error} = $error;
            return;
        }
        delete $self->{containers}{$voltype}{$vol};
        delete $self->{containers}{volume}{$vol};
        $self->{containers}{count}{volume}--;
        $self->{containers}{count}{$voltype}--;
    }
}

sub _create_flexvols {
    my $self    = shift;
    my $count   = shift;
    my $aggr    = shift;
    my $volinfo = $self->{cfginfo}{$FLEX};
    my $error   = undef;
    logcomment("Creating $count flex vols");

#    print "DBGCEB:_create_flexvols:$self->{fname}: volinfo = ", Dumper($volinfo);

    my $prefix = $volinfo->{prefix} ? $volinfo->{prefix} : 'flexvol';
    my $size   = $volinfo->{size}   ? $volinfo->{size}   : '100m';
    $self->{error} = sprintf "_create_vols: not a valid flex vol size ", $size
      if ( $size !~ /\d+m/ );
    return $self->{error} if $self->{error};
    my $guarantee = $volinfo->{guarantee} ? $volinfo->{guarantee} : 'none';
    my $cntr = 0;
    for ( 1 .. $count ) {
        my $name    = $prefix . ( time() );
        my $retries = $RETRIES;
      RETRYIT:
        eval {
            vol_create_flex( $self->{fc}, $name, $aggr, $size,
                guarantee => "none" );
        };
        $error = sprintf "%s: cannot create %s :$@", $self->{fname}, $name
          if $@;
        if ($error) {
            "DBGCEB: error = $error\n";
#            getc;    #DBGCEB
            if ( $error =~ /Maximum number of flexible volumes exceeded/ ) {
                $self->{error} = $error;
                return;
            }
            $self->process_error($error);
            $error = undef;
            return if ( $self->{error} );
            $retries--;
            goto RETRYIT if $retries;
            $self->{error} = $error;
            return;
        }

        return $self->{error} if $self->{error};
        $cntr++;
        logcomment(
"$self->{fname}:created flex vol $name: $cntr volumes on $aggr created"
        );

        my $newvol = {};
        $newvol->{size}      = $size;
        $newvol->{$AGGR}     = $aggr;
        $newvol->{guarantee} = $guarantee;
        $newvol->{type}      = $FLEX;
        $newvol->{new}       = 1;
        $newvol->{$name}     = {};
        $newvol->{path}      = "/vol/$name";
        $self->{containers}{$FLEX}{$name} = $newvol;
        $self->{containers}{volume}{$name} = $newvol;
        $self->{containers}{count}{volume}++;
        $self->{containers}{count}{$FLEX}++;
    }
    return;
}

sub setup_trad_vols {
    my $self    = shift;
    my $voltype = $TRAD;
    $self->{error} = undef;
    my $volinfo = $self->{cfginfo}{$voltype};
    my $mode    = $self->{cfginfo}{mode};
    my $needed  = $volinfo->{count};
    if ($needed > MAX_TVOLS) {
        $needed = MAX_TVOLS;
    }

    $self->_get_existing_tradvols();
    if ( $mode eq $PURGE ) {
        $self->_purge_volumes($voltype);
        return 0 if ( $self->{error} );
    }
    my $num_existing = ( keys %{ $self->{containers}{$voltype} } );
    my $new = $needed - $num_existing;
    return 0 if ( $self->{error} );

    $volinfo->{new} = $new;
    if ($new > 0) {
        $self->_create_tradvols();
        return 0   if ( $self->{error} );
    }
    $self->_export_vols();
    return;
}

sub _get_existing_aggrs_tradvols {
    my $self     = shift;
    my $voltype  = shift;
    my $error    = undef;

    foreach my $type ( ($AGGR, $FLEX, $TRAD) ) {
       if ( !ref($self->{cfginfo}{$type}{existing}) ) {
           $self->{cfginfo}{$type}{existing} = {};
           $self->{containers}{$type} = $self->{cfginfo}{$type}{existing};
           $self->{containers}{count}{$type} = 0;
       }
    }
    if ( !ref($self->{containers}{volume}) ) {
        $self->{containers}{volume} = {};
        $self->{containers}{count}{volume} = 0;
    }

    my $prefix = $self->{cfginfo}{$voltype}{prefix}
      if ( exists $self->{cfginfo}{$voltype}{prefix} );
    logcomment("_get_existing_aggrs_tradvols: $voltype  prefix: \"$prefix\"");
    my $cmd     = "aggr status";
    my $retries = $RETRIES;
  RETRYIT:

    my $text = undef;
    eval { $text = sayEOF( $self->{fc}, $cmd ); };
    $error = "$self->{fname}: $cmd failed: $@"
      if ( ($@) || ( !$text ) );

    if ($error) {
        $self->process_error($error);
        $error = undef;
        return if ( $self->{error} );
        $retries--;
        goto RETRYIT if $retries;
        $self->{error} = $error;
        return 0;

    }

    return if ( $self->{error} );
    my $trad_count = 0;
    my $aggr_count = 0;
    my ( $str, @lines, $name, $line, $type, $raid, $opts, $status, @text, );
    @lines = split ( /\n/, $text );

    foreach $line (@lines) {
        if ( $line =~ /online|offline|restricted/ ) {
            push @text, $str if ($str);
            $str = $line;
            next;
        }
        $str .= $line;
    }
    push @text, $str if ($str);

    foreach $line (@text) {
        if (
            $line =~ /^
            \s*(\w+)\s+
            (online|offline|restricted)\s+
            (raid[\_\w]+)\,\s+
            (\w+)
                /x
          )
        {
            $name   = $1;
            $status = $2;
            $raid   = $3;
            $type   = $4;
            $opts   = $5;
        } else {
            next;
        }
        next if ( $voltype && $type ne $voltype );
        next if ( ($prefix) && ( $name !~ /^$prefix/ ) );

        if ($type eq $AGGR) {
            $aggr_count++;
        } else {
            $trad_count++;
        }

        my $newaggr;
        if ( ref($self->{cfginfo}{$type}{existing}{$name}) ) {
            $newaggr = $self->{cfginfo}{$type}{existing}{$name};
        } else {
            $newaggr = {};
            $self->{cfginfo}{$type}{existing}{$name} = $newaggr;
        }
        $newaggr->{status}  = $status;
        $newaggr->{raid}    = $raid;
        $newaggr->{name}    = $name;
        $newaggr->{opts}    = $opts;
        $newaggr->{type}    = $type;
        $newaggr->{path}    = "/vol/$name"
            if ($type eq $TRAD);
        $newaggr->{sm_stat} = $1
          if ( $line =~ /snapmirrored\=(\w+)/ );
        $newaggr->{lw_protect} = $1
          if ( $line =~ /lost_write_protect\=(\w+)/ );
        if ($type eq $TRAD) {
            $self->{containers}{volume}{$name} = $newaggr;
        } else {
            $self->{containers}{$AGGR}{$name} = $newaggr;
        }
    }
    if (!$voltype) {
        $self->{containers}{count}{$AGGR} = $aggr_count;
        $self->{containers}{count}{$TRAD} = $trad_count;
    } elsif ($voltype eq $AGGR) {
        $self->{containers}{count}{$AGGR} = $aggr_count;
    } else {
        $self->{containers}{count}{$TRAD} = $trad_count;
    }
}

sub _create_tradvols {
    my $self    = shift;
    my $volinfo = $self->{cfginfo}{$TRAD};
    my $error   = undef;

    my $prefix = $volinfo->{prefix} ? $volinfo->{prefix} : 'testag';
    my $count  = $volinfo->{new}    ? $volinfo->{new}    : 1;
    logcomment("Creating $count tradvols");
    my $raidtype = $volinfo->{raid} ? $volinfo->{raid} : 'raid_dp';
    my $size     = $volinfo->{size} ? $volinfo->{size} : 3;
    my $cntr     = 0;
    logcomment(
        "_create_tradvols:$self->{fname}: creating $count $TRAD " . "volume" );

    for ( 1 .. $count ) {
        my $name    = $prefix . ( time() );
        my $retries = $RETRIES;
        my $rc      = undef;
      RETRYIT:
        my $count = $RETRIES - $retries;
        logcomment("_create_tradvols: retrying trad vol create: $retries")
          if $count;
        eval {
            $rc =
              vol_create( $self->{fc}, $name, $size, raidtype => $raidtype );
        };

        $error = sprintf "%s: cannot create %s :$@", $self->{fname}, $name
          if ( $@ || $rc != 1 );
        if ($error) {
            $self->process_error($error);
            $error = undef;
            return if ( $self->{error} );
            $retries--;
            goto RETRYIT if $retries;
            $self->{error} = $error;
            return;
        }

        $cntr++;
        logcomment( "_create_tradvols: $self->{fname}:created tradvol $name: "
              . "$cntr tradvols "
              . "created" );

        my $newvol = {};
        $newvol->{new}     = 1;
        $newvol->{size}    = $size;
        $newvol->{raid}    = $raidtype;
        $newvol->{type}    = $TRAD;
        $newvol->{path}    = "/vol/$name";
        $newvol->{created} = 1;
        $self->{containers}{volume}{$name} = $newvol;
        $self->{containers}{$TRAD}{$name} = $newvol;
        $self->{containers}{count}{volume}++;
        $self->{containers}{count}{$TRAD}++;

        eval {
            export_path( $self->{fc},
                "$self->{containers}{volume}{$name}{path}" );
        };
        $error = sprintf "%s: Cannot export %s: $@", $self->{fname},
          $self->{containers}{volume}{$name}{path}
          if ($@);
        logWarn($error);

    }
    return;
}

sub _export_vols {
    my $self  = shift;
    my $error = undef;
    logStep("_export_vols:$self->{fname}:export volumes");
    my $vols = $self->{containers}{volume};
    foreach my $vol ( keys %{$vols} ) {
        next if ( exists $vols->{path} );
        my $retries = 3;
      RETRYIT:
        my $vol_path = "/vol/$vol";
        $vols->{$vol}{path} = $vol_path;
        eval { export_path( $self->{fc}, "$vol_path" ); };
        $error = sprintf "%s: Cannot export %s: $@", $self->{fname}, $vol_path
          if $@;
        if ($error) {
            $self->process_error($error);
            $error = undef;
            if ( defined $self->{error} ) {
                logWarn( $self->{error} );
            } else {
                $retries--;
                goto RETRYIT if $retries;
                logWarn( $self->{error} );
            }
        }
        $vols->{$vol}{export} = 1;
    }
}


##############################################################################################

##############################################################################################

##########################################
#
#     USE statements & initialization
#
#########################################

#   use QtreeCmds qw(qtreeList qtreeStatus qtreeCreate qtreeDelete qtreePresent);
#   use blocks_filer qw(filer_lun_exists filer_luns_get filer_lun_share_all filer_lun_create filer_lun_destroy);
#   use Users qw(createUsers createGroups readPasswdFile readGroupFile delPasswdEntry delGroupEntry);
#   use Quota;
#

##################################################
#
#        Start Qtree section
#
##################################################

#   ( -- at end of get_existing_containers -- )
#   >
#   >    # get qtrees
#   >    $self->_get_existing_qtrees();
#   >    $self->{containers}{$QTREES} = $self->{cfginfo}{$QTREES}{existing};
#   >
#   >
#       # return $containers;


sub _get_existing_volumes {
    my $self = shift;
    my $existing_volumes = { %{$self->{containers}{volume}} };
    if (!$self->{root_vol}) {
        $self->{root_vol} = vol_root($self->{fc});
    }
    while ( my($vname, $vinfo) = each(%$existing_volumes)) {
        delete($existing_volumes->{$vname}) unless ($vname && $vinfo->{type} =~ /(trad|flex)/);
    }
    foreach my $voltype (($TRAD, $FLEX)) {
        # while ( my($vname, $vinfo) = each(%{$self->{existing}{$voltype}}) )
        while ( my($vname, $vinfo) = each(%{$self->{containers}{$voltype}}) ) {
            next unless ($vname);
            next unless $vinfo->{type} =~ /(trad|flex)/;
            $existing_volumes->{$vname} = $vinfo;
        }
    }
    return ($existing_volumes);
}


sub _get_vol_ref {
    my $self = shift;
    my $vol  = shift;
    my ($volhash, $volname, $volpath);
    if ( !$vol ) {
        $volhash = undef;
    } elsif ( ref($vol) ) {
        $volhash = $vol;
        $volname = $volhash->{name};
        $volpath = $volhash->{path};
    } else {
#        my $existing_volumes = $self->{containers}{volume};
        my $existing_volumes = $self->_get_existing_volumes();
        my $v;
        if ($vol =~ m%/vol/(.+)%) {
            $volname = $1;
            if ( $existing_volumes->{$volname} ) {
                $volhash = $existing_volumes->{$volname};
                $volpath = $volhash->{path};
            }
        } else {
            $volname = $vol;
            if ( $existing_volumes->{$volname} ) {
                $volhash = $existing_volumes->{$volname};
                $volpath = $volhash->{path};
            }
        }
    }
    $volhash = undef unless (defined($volhash));
###    return ($volhash, $volname, $volpath);
    return ($volname, $volhash, $volpath);
}


sub _get_lun_ref {
    my $self = shift;
    my $lun  = shift;
    my (%opts) = @_;
    my $qtree_opt = $opts{qtree};
    my ($qtname, $qthash, $qtpath, $lunpath, $lunhash, $lunname);
    if ($qtree_opt) {
        ($qthash, $qtname, $qtpath) = $self->_get_qtree_ref($qtree_opt);
    }

    if ( !$lun ) {
        $lunhash = undef;
    } elsif ( ref($lun) ) {
        $lunhash = $lun;
        $lunname= $lunhash->{lun};
        $lunpath= $lunhash->{path};
    } else {
        if ($lun =~ m%/vol/([^/]+)/([^/]+)/([^/]+)%) {
            $lunpath = $lun;
            $lunname = $3;
        } elsif ($qtpath) {
            $lunname = $lun;
            $lunpath = "$qtpath/$lunname";
        } else {
        }
        $self->_get_existing_luns()
            unless (defined($self->{containers}{luns}));
        my $existing_luns = $self->{containers}{luns};
        if ($lunpath && $existing_luns->{$lunpath}) {
            $lunhash = $existing_luns->{$lunpath};
            $lunname= $lunhash->{lun};
            $lunpath= $lunhash->{path};
        }
        $lunhash = undef unless (defined($lunhash));
    }
    return ($lunhash, $lunname, $lunpath);
}


sub _get_qtree_ref {
    my $self = shift;
    my $qtree  = shift;
    my (%opts) = @_;
    my $vol_opt = $opts{vol};
    my ($volname, $volhash, $volpath, $qtpath, $qthash, $qtname);
    if ($vol_opt) {
        ($volname, $volhash, $volpath) = $self->_get_vol_ref($vol_opt);
    }

    if ( !$qtree ) {
        $qthash = undef;
    } elsif ( ref($qtree) ) {
        $qthash = $qtree;
        $qtname= $qthash->{qtree};
        $qtpath= $qthash->{path};
    } else {
        if ($qtree =~ m%/vol/([^/]+)/([^/]+)%) {
            $qtpath = $qtree;
            $qtname = $2;
        } elsif ($volpath) {
            $qtname = $qtree;
            $qtpath = "$volpath/$qtname";
        } else {
        }
        $self->_get_existing_qtrees()
            unless (defined($self->{containers}{qtrees}));
        my $existing_qtrees = $self->{containers}{qtrees};
        if ($qtpath && $existing_qtrees->{$qtpath}) {
            $qthash = $existing_qtrees->{$qtpath};
            $qtname= $qthash->{qtree};
            $qtpath= $qthash->{path};
        }
        $qthash = undef unless (defined($qthash));
    }
    return ($qthash, $qtname, $qtpath);
}
















sub setup_qtrees {
    my $self = shift;
    my (%opts) = (@_);
    $self->{cfginfo}{$QTREES} = {} unless ($self->{cfginfo}{$QTREES});
    my $qtinfo = $self->{cfginfo}{$QTREES};
    my $force  = ( exists $opts{force} ) ? $opts{force} : 0;
    my $needed = ( exists $opts{count} ) ? $opts{count} : $qtinfo->{count};
    return if (!$needed && !$force);
    $qtinfo->{export} = $opts{export}
        unless (defined($qtinfo->{export}));
    my $export = $qtinfo->{export};

    if ( !defined( $qtinfo->{existing} ) || $force ) {
        $qtinfo->{existing}          = {};
        $qtinfo->{count}             = 0;
        $qtinfo->{new}               = 0;
        $self->{cfginfo}{$QTREES}    = $qtinfo;
        $self->{containers}{$QTREES} = $qtinfo->{existing};
        $self->_get_existing_qtrees( export => $export );  #DBGCEB put this back
    }

    my $mode   = $qtinfo->{mode} || $opts{mode};

    if ( $mode eq $PURGE ) {
        $self->_purge_qtrees();
        return if ( $self->{error} );
    }
    my $num_existing = keys( %{ $qtinfo->{existing} } );
    my $new          = undef;
    $new = $opts{new} if ( !defined($new) && defined( $opts{new} ) );
    $new = $needed - $num_existing if ( !defined($new) && defined($needed) );
    $new = 0 unless ( defined($new) );

logresult('INFO', "setup_qtrees  Needed: $needed    Num Existing: $num_existing   New: $new");
    if ( $new > 0 ) {
        $qtinfo->{new} = $new;
        $self->_create_qtrees(%opts);
    } else {
        $qtinfo->{new} = 0;
    }
    if ($export) {
        $self->_export_qtrees();
    }
    return 1;
}

sub _get_existing_qtrees {
    my $self = shift;
    my (%opts) = (@_);
    my $qtinfo = $self->{cfginfo}{$QTREES};
    $self->setup_qtrees(%opts) unless ($qtinfo);
    $qtinfo->{existing} = {} unless ( $qtinfo->{existing} );
    my $existing         = $qtinfo->{existing};
    my $export           = $qtinfo->{export};
    $export = $opts{export} unless ( defined($export) );
    my $exported_hash = list_exports($self->{fc});

    my @qtpath_list = eval{ qtreeList($self->{fc}) };

    if ($@ || scalar(@qtpath_list) == 0) {
       if ($@ && $@ !~ /parsing/) {
            $self->{error} = "_get_existing_qtrees: Error from qtreeList: $@";
            logresult('FATAL', $self->{error});
            return 0;
        } else {
            logresult('WARN', "_get_existing_qtrees: Error from qtreeList: $@");
            @qtpath_list = ();
            my $out = sayEOF($self->{fc}, "qtree status", timeout=>$MIN_CMDTIMEOUT);
            my $qcount = `sudo rsh $self->{fname} qtree status| grep qtree|wc -l`;
            chomp($out);
            my @lines = grep(/qtree/, split(/\n/, $out));
            foreach (@lines) {
                if (/^(\w+)\s+(\w+)/) {
                    my $qtpath = "/vol/" . $1 . "/" . $2;
                    push(@qtpath_list, $qtpath);
                }
            }
        }
    }

    $qtinfo->{count} = 0;
    foreach my $qtpath (@qtpath_list) {
        if ( $qtpath !~ m%/vol/([^\/]+)/([^\/]+)$% ) {
            $self->{error} =
              "_get_existing_qtrees: Can't parse qtree path: \"$qtpath\"";
            logWarn( $self->{error} );
            return 0;
        }
        my ( $vol, $qtree ) = ( $1, $2 );
        if (!$self->{root_vol}) {
            $self->{root_vol} = vol_root($self->{fc});
        }
        next if ($vol eq $self->{root_vol});
        my $qthash = {};
        $existing->{$qtpath} = $qthash;
        $qthash->{volume}    = $vol;
        $qthash->{qtree}     = $qtree;
        $qthash->{name}      = $qtree;
        $qthash->{qtpath}    = $qtpath;
        $qthash->{path}      = $qtpath;
        $qthash->{mp_type}   = 'qtree';
        $qthash->{is_exported} = defined($exported_hash->{$qtpath});
        $qthash->{export} = $export if ( defined($export) );
        my $volhash;
        ($vol, $volhash) = $self->_get_vol_ref($vol);
        $volhash->{$QTREES} = {} unless ( $volhash->{$QTREES} );
        $volhash->{$QTREES}{$qtpath} = $qtpath;

        # my $status = qtreeStatus($self->{fc}, $qtpath);
        # if ($@) {
        #     $self->{error} = "qtreeStatus: $@";
        #     return 0;
        # }
        if ( $qthash->{export} ) {
            $self->_export_qtree($qtpath);
        }
#        elsif ( $qthash->{is_exported} ) {
#             $self->_unexport_qtree($qtpath);
#        }

        $qtinfo->{count}++;
    }
logresult('INFO', "Exiting get_existing_qtrees   Count: $qtinfo->{count}");
    $self->{containers}{$QTREES} = $qtinfo->{existing};
    return ( $qtinfo->{count} );
}

sub _purge_qtree {
    my $self = shift;
    my ($qtree) = @_;
    my $qtinfo           = $self->{cfginfo}{$QTREES};
    my $existing         = $qtinfo->{existing};
    my $luninfo          = $self->{cfginfo}{$LUNS};
    my ( $qthash, $qt, $qtpath );

    if ( ref($qtree) ) {
        $qthash = $qtree;
        $qtpath = $qthash->{qtpath};
        $qtree  = $qthash->{name};
    } else {
        $qtpath = $qtree;
        $qthash = $existing->{$qtpath};
        logresult( 'INFO', "Checking for qtpath: \"$qtpath\"" );
        if ( !$qthash ) {
            $self->{error} = "_purge_qtree: Could not find qtree $qtpath";
            logWarn( $self->{error} );
            return 0;
        }
        $qtree = $qthash->{qtree};
    }

    my @luns = keys( %{ $qthash->{$LUNS} } );
    $self->_purge_lun_list( \@luns );
    my $qtinfo = $self->{cfginfo}{$QTREES};
    my $qthash = $existing->{$qtpath};
    next unless ($qthash);
    my $vol = $qthash->{volume};
    my $qt  = $qthash->{qtree};
    if ( qtreePresent( $self->{fc}, $qtpath ) ) {
        $self->_unexport_qtree($qtpath);
logresult('INFO', "purge_qtree Calling qtreeDelete");
        eval { qtreeDelete($self->{fc}, $qtpath) };
logresult('INFO', "purge_qtree Returned from qtreeDelete");
    }
    if ($@) {
        $self->{error} = "_purge_qtrees: Could not delete qtree $qtpath: $@";
logresult('WARN', $self->{error});
        return 0;
    }
    if ( qtreePresent( $self->{fc}, $qtpath ) ) {
        $self->{error} = "_purge_qtrees: Could not delete qtree $qtpath: $@";
logresult('WARN', $self->{error});
        return 0;
    }
    delete( $existing->{$qtpath} );
    $qtinfo->{count}--;
    my $volhash;
    ($vol, $volhash) = $self->_get_vol_ref($vol);
    $volhash->{$QTREES} = {} unless ( $volhash->{$QTREES} );
    delete( $volhash->{$QTREES}{$qtpath} );
    return (1);
}

sub _purge_qtree_list {
    my $self = shift;
    my ($qtree_list) = @_;
    my $qtinfo           = $self->{cfginfo}{$QTREES};
    my $existing         = $qtinfo->{existing};
    if ( !ref($qtree_list) || ref($qtree_list) eq "HASH" ) {
        $qtree_list = [$qtree_list];
    }
    foreach my $qtpath (@$qtree_list) {
        $self->_purge_qtree($qtpath);
    }

$self->{error} = undef; ##################################

}

sub _purge_qtrees {
    my $self     = shift;
    my $qtinfo   = $self->{cfginfo}{$QTREES};
    my $existing = $qtinfo->{existing};
    logcomment("_purge_qtrees: $self->{fname}");
    my $qtree_list = [ keys( %{ $qtinfo->{existing} } ) ];
    return ( $self->_purge_qtree_list($qtree_list) );
}

sub _create_qtree_for_volume {
    my $self = shift;
    my ( $vol, $qtree, %opts ) = @_;
    my $qtinfo           = $self->{cfginfo}{$QTREES};
    my $existing         = $qtinfo->{existing};
    my $volhash          = undef;
    my $export           = $qtinfo->{export};
    $export = $opts{export} unless ( defined($export) );
    my $security = $qtinfo->{security};
    $security = $opts{security} unless ( defined($security) );
    my $volhash;
    my $new_qtrees = {};

    if ( !$vol ) {
        $self->{error} = "_create_qtree_for_volume: no volume given";
        logWarn( $self->{error} );
        return 0;
    }
    ($vol, $volhash) = $self->_get_vol_ref($vol);
    if (!$volhash) {
        $self->{error} = "_create_qtree_for_volume: no such volume: $vol";
        logWarn( $self->{error} );
        return 0;
    }

    if ( !vol_exists( $self->{fc}, $vol ) ) {
        logWarn
          ("_create_qtree_for_volume: volume: $vol has been deleted");
        return 0;
    }

    # my($state, $status, $ag, %options) = vol_status($self->{fc}, $vol);
    # if ($state !~ /online/) {
    # }

    my $time = time();
    logcomment("Creating qtree for volume $vol on $self->{fname}");
    my $qtpath = "/vol/$vol/$qtree";

    if (defined($existing->{$qtpath})) {
        my $ctr = 1;
        my $base_qtpath = $qtpath;
        while (defined($existing->{$qtpath})) {
            $qtpath = $base_qtpath . "_" . $ctr;
            $ctr++;
        }
    }

    qtreeCreate( $self->{fc}, $qtpath )
      or $self->{error} =
"_create_qtree_for_volume: cannot create qtree $qtpath in volume $vol: $@";
    if ( $self->{error} ) {
        logWarn( $self->{error} );
        $self->{error} = undef;
        return;
    }
    if ( !qtreePresent( $self->{fc}, $qtpath ) ) {
        logWarn( "_create_qtree_for_volume: cannot create qtree $qtpath "
              . "in volume $vol: $@" );
    }
    if ($security) {
        qtreeSecurity( $self->{fc}, $qtpath, $security );
    }
    logcomment("$self->{fname}:created qtree $qtree in volume $vol");
    my $qthash = {};
    $new_qtrees->{$qtpath} = $qthash;
    $existing->{$qtpath} = $qthash;
    $qthash->{volume}    = $vol;
    $qthash->{qtree}     = $qtree;
    $qthash->{name}      = $qtree;
    $qthash->{qtpath}    = $qtpath;
    $qthash->{path}      = $qtpath;
    $qthash->{mp_type}   = 'qtree';
    $qthash->{export} = $export if ( defined($export) );
    $qthash->{is_exported} = 0;
    $qtinfo->{count}++;
    $volhash->{$QTREES}{$qtpath} = $qtpath;
    $self->{containers}{$QTREES}{$qtree} = $qthash;
    $self->_export_qtree($qtpath) if ($qthash->{export});
    return ($new_qtrees);
}

sub _create_qtrees_for_volume {
    my $self = shift;
    my ( $vol, $count, %opts ) = @_;
    my $qtinfo           = $self->{cfginfo}{$QTREES};
    my $existing         = $qtinfo->{existing};
    my $prefix           = $opts{prefix} || $qtinfo->{prefix} || 'qtree';
    my $volhash;
    my $export = $qtinfo->{export};
    $export = $opts{export} unless ( defined($export) );

    my $time = time();
    logcomment("Creating $count qtrees for volume $vol on $self->{fname}");
    my $qcount = 0;
    my $new_qtrees = {};
    for ( my $cntr = 0 ; $cntr < $count ; $cntr++ ) {
        my $qt = "${prefix}_${time}_${cntr}";
        my $newx = $self->_create_qtree_for_volume( $vol, $qt );
        return(0) if ($self->{error});
        $new_qtrees = { %$new_qtrees, %$newx };
    }
    return $new_qtrees;
}

sub _create_qtrees {
    my $self = shift;
    my (%opts) = @_;
    my $qtinfo = $self->{cfginfo}{$QTREES};
    my $needed = ( exists $opts{new} ) ? $opts{new} : $qtinfo->{new};
    my $new_qtree_count = $qtinfo->{count} + $needed;
    return if !$needed;

#    my $existing_volumes = $self->{containers}{volume};
    my $existing_volumes = $self->_get_existing_volumes();
    my @vol_names_tmp        = keys(%$existing_volumes);
    my $qcount = 0;
    my $new_qtrees = {};

    if (!$self->{root_vol}) {
        $self->{root_vol} = vol_root($self->{fc});
    }
    my @vol_names = ();
    foreach (@vol_names_tmp) {
        next if (!$_);
        next if ($_ eq $self->{root_vol});
        push(@vol_names, $_);
    }
    my $nvols = scalar(@vol_names);
    print "DBGCEB:_create_qtrees: volumes = ", @vol_names, "\n";
    if ( $nvols == 0 ) {
        $self->{error} = "create_qtrees: no volumes exist on $self->{fname}";
        logresult( 'FATAL',
            "create_qtrees: no volumes exist on $self->{fname}" );
        return (0);
    }
#    my $qt_per_vol = int($needed / $nvols);
#    my $rem = $needed % $nvols;
#    foreach my $vol (@vol_names) {
#        $self->_create_qtrees_for_volume($vol, $qt_per_vol, %opts);
#        return(0) if ($self->{error});
#    }

    for (my $i=0; $qtinfo->{count} < $new_qtree_count; $i = ($i + 1) % $nvols) {
        my $vol = $vol_names[$i];
        my $newx = $self->_create_qtrees_for_volume($vol, 1, %opts);
        return(0) if ($self->{error});
        $new_qtrees = { %$new_qtrees, %$newx };
    }
    $qtinfo->{new} = 0;

    return (1);
}

sub _export_thing {
    my $self   = shift;
    my $path  = shift;
    my $info = shift;
    my $error = undef;
    logStep("_export_thing:$self->{fname}:export thing");
    my $retries = $RETRIES;

    return unless $info->{export};
  RETRYIT:
    eval { export_path( $self->{fc}, "$path" ); };
    if ($@) {
        $self->{error} =
          "_export_thing: export_path failed for \"$path\": $@";
    }
    if ($error) {
        $self->process_error($error);
        $error = undef;
        if ( defined $self->{error} ) {
            logWarn( $self->{error} );
        } else {
            $retries--;
            goto RETRYIT if $retries;
            logWarn( $self->{error} );
        }
    }

#    if (!$self->_is_exported($path)) {
#         $self->{error} = "_export_thing: Failed to export path: \"$path\"";
#         logWarn( $self->{error} );
#         return;
#    }

    $info->{is_exported} = 1;
}


sub _unexport_thing {
    my $self   = shift;
    my $path = shift;;
    my $info = shift;

    if ( !$info->{is_exported} ) {
        return;
    }

    my $error = undef;
    logStep("_unexport_thing:$self->{fname}:unexport thing");
    my $retries = $RETRIES;
  RETRYIT:
    eval { unexport_path( $self->{fc}, "$path" ); };
    if ($@) {
        $error = sprintf("%s: Cannot unexport %s: $@", $self->{fname}, $path);
        logWarn( $self->{error} );
        return ($self->{error}); 
    }

    if ($error) {
        $self->process_error($error);
        $error = undef;
        if ( defined $self->{error} ) {
            logWarn( $self->{error} );
        } else {
            $retries--;
            goto RETRYIT if $retries;
            logWarn( $self->{error} );
        }
    }
    $info->{is_exported} = 0;
}

sub _export_qtree {
    my $self   = shift;
    my $qtree  = shift;
    my $qtinfo = $self->{cfginfo}{$QTREES};
    my ( $qt, $qthash, $qtpath );
    my $qtrees = $qtinfo->{existing};

    if ( ref($qtree) ) {
        $qthash = $qtree;
        $qtpath = $qthash->{qtpath};
        $qtree  = $qthash->{name};
    } else {
        $qtpath = $qtree;
        $qthash = $qtrees->{$qtpath};
        if ( !$qthash ) {
            $self->{error} = "_export_qtree: Could not find qtree $qtpath";
logresult('WARN', $self->{error});
            return 0;
        }
        $qtree = $qthash->{qtree};
    }
    if ( !$qthash->{export} || $qthash->{is_exported} ) {
        return;
    }
    return( $self->_export_thing($qtpath, $qthash) );
}

sub _unexport_qtree {
    my $self   = shift;
    my $qtree  = shift;
    my $qtinfo = $self->{cfginfo}{$QTREES};
    my ( $qt, $qthash, $qtpath );
    my $qtrees = $qtinfo->{existing};

    if ( ref($qtree) ) {
        $qthash = $qtree;
        $qtpath = $qthash->{qtpath};
        $qtree  = $qthash->{name};
    } else {
        $qtpath = $qtree;
        $qthash = $qtrees->{$qtpath};
        if ( !$qthash ) {
            $self->{error} = "_export_qtree: Could not find qtree $qtpath";
logresult('WARN', $self->{error});
            return 0;
        }
        $qtree = $qthash->{qtree};
    }
    if ( !$qthash->{is_exported} ) {
        return;
    }
    return( $self->_unexport_thing($qtpath, $qthash) );
}

sub _export_qtree_list {
    my $self       = shift;
    my $qtree_list = shift;
    my $error      = undef;
    return if ( !$qtree_list );
    logcomment("_export_qtree_list:$self->{fname}:export qtrees");
    foreach my $qtx (@$qtree_list) {
        $self->_export_qtree($qtx);
    }
}

sub _export_qtrees {
    my $self  = shift;
    my $error = undef;
    logStep("_export_qtrees:$self->{fname}:export qtrees");
    my $qtrees     = $self->{containers}{$QTREES};
    my @all_qtrees = keys %{$qtrees};
    $self->_export_qtree_list( \@all_qtrees );
}

sub _is_exported {
    my $self    = shift;
    my $path    = shift;
    my $exports = list_exports( $self->{fc} );
    return ( defined( $exports->{$path} ) );
}

##################################################
#
#        End Qtree section
#
##################################################

##################################################
#
#        Start LUN section
#
##################################################

use blocks_filer qw(filer_luns_get);
use blocks_filer qw(filer_fcp_adapters_get);
use blocks_filer qw(filer_lun_show_get);
use blocks_filer qw(filer_lun_igroups_get);
use blocks_filer qw(filer_igroup_lun_lunid_get);
use blocks_filer qw(filer_lun_share_all);
use blocks_filer qw(filer_lun_share_none);
use blocks_filer qw(filer_lun_exists);
use blocks_filer qw(filer_lun_destroy);
use blocks_filer qw(filer_lun_create);
use blocks_filer qw(filer_lun_map_low);
use blocks_filer qw(filer_igroup_lun_lunid_get);
use blocks_filer qw(filer_lun_online);
use blocks_filer qw(filer_lun_offline);
use blocks_filer qw(filer_fcp_nodename_get);

sub lun_online {
    my $self    = shift;
    my $lunpath = shift;

    # blocks_filer::filer_lun_online
    eval { filer_lun_online( $self->{fc}, $lunpath ); };
    return (1);
}

sub lun_offline {
    my $self    = shift;
    my $lunpath = shift;

    # blocks_filer::filer_lun_offline
    eval { filer_lun_offline( $self->{fc}, $lunpath ); };
    return (1);
}

#   #(at end of get_existing_containers)
#   >
#   >    # get LUNs
#   >    $self->_get_existing_luns();
#   >    $self->{containers}{$LUNS} = $self->{cfginfo}{$LUNS}{existing};
#   >
#       # return $containers;

sub setup_luns {
    my $self = shift;
    my (%opts) = (@_);
    $self->{cfginfo}{$LUNS} = {} unless ($self->{cfginfo}{$LUNS});
    my $luninfo = $self->{cfginfo}{$LUNS};
    my $force  = ( exists $opts{force} ) ? $opts{force} : 0;
logresult('INFO', "Enter setup_luns  Opts:\n\n".Dumper(%opts));

    my $mode   = $self->{mode} || $opts{mode};
    my $export = $luninfo->{export} || $opts{export};
    $export = $opts{export} unless ( defined($export) );
    my $needed = ( exists $opts{count} ) ? $opts{count} : $luninfo->{count};
    return if (!$needed && !$force);

    if ( !defined( $luninfo->{existing} ) || $force ) {
        $luninfo->{existing}       = {};
        $luninfo->{count}          = 0;
        $luninfo->{new}            = 0;
        $self->{cfginfo}{$LUNS}    = $luninfo;
        $self->{containers}{$LUNS} = $luninfo->{existing};
        $self->_get_existing_luns();
    }

    my $existing = $luninfo->{existing};
    $self->{containers}{$LUNS} = $existing;

    if ( $mode eq $PURGE ) {
        $self->_purge_luns();
        return if ( $self->{error} );
    }
    my $num_existing = keys( %{ $luninfo->{existing} } );
    my $new          = undef;
    $new = $opts{new} if ( !defined($new) && defined( $opts{new} ) );
    $new = $needed - $num_existing if ( !defined($new) && defined($needed) );
    $new = 0 unless ( defined($new) );
# logresult('INFO', "setup_luns  Needed: $needed    Num Existing: $num_existing   New: $new   Opts:\n\n".Dumper(%opts));
#     if ( $new > 0 ) {
#         $luninfo->{new} = $new;
#         $self->_create_luns(%opts);
#     } else {
#         $luninfo->{new} = 0;
#     }

    return (1);
}

sub _setup_spec_luns {
    my $self    = shift;
    my $lunproto = shift;
    my (%opts) = (@_);

    my %opts_copy = %opts;
    $opts_copy{count} = 0;
    $opts_copy{force} = 1;

    $self->{cfginfo}{$LUNS} = {} unless ($self->{cfginfo}{$LUNS});
    my $luninfo = $self->{cfginfo}{$LUNS};
    my $spec_luninfo = $luninfo->{$lunproto};

    my $force = ( exists $opts{force} ) ? $opts{force} : 0;
    my $needed = ( exists $opts{count} ) ? $opts{count} : 
                   ($spec_luninfo->{count} || $luninfo->{count});
    return if (!$needed && !$force);

    if ( !defined( $luninfo->{existing} ) ) {
        $self->setup_luns(force=>1, %opts_copy);
    }
    $luninfo->{$lunproto} = {} unless ($luninfo->{$lunproto});
    $spec_luninfo = $luninfo->{$lunproto};
    if ( !defined($spec_luninfo->{existing}) || $force )
    {
        $spec_luninfo->{existing} = {};
        $spec_luninfo->{count}    = 0;
        $spec_luninfo->{new}      = 0;
        $self->_get_existing_spec_luns();
    }

    my $map_luns = $opts{map_luns};
    $map_luns = 1 unless ( defined($map_luns) );

    my $spec_existing = $spec_luninfo->{existing};
    my $mode          = $self->{mode} || $opts{mode};

    if ( $mode eq $PURGE ) {
        $self->_purge_spec_luns($lunproto);
        return (0) if ( $self->{error} );
    }
    my $num_existing = keys(%$spec_existing);
    my $new          = undef;
    $new = $opts{new} if ( !defined($new) && defined( $opts{new} ) );
    $new = $needed - $num_existing if ( !defined($new) && defined($needed) );
    $new = 0 unless ( defined($new) );
logresult('INFO', "setup_spec_luns ($lunproto)  Needed: $needed  Num Existing: $num_existing  New: $new");
    if ( $new > 0 ) {
        $spec_luninfo->{new} = $new;
        $self->_create_spec_luns($lunproto, %opts);
    } else {
        $spec_luninfo->{new} = 0;
    }

    my $igrpinfo      = $self->{cfginfo}{igroups};
    my $spec_igrpinfo = $igrpinfo->{proto};
    if (   !$spec_igrpinfo
        || !$spec_igrpinfo->{existing}
        || !keys( %{ $spec_igrpinfo->{existing} } ) )
    {
        $self->_setup_spec_igroup($lunproto);
    }
    my %map_opts = %opts;
    $self->_map_spec_luns( $lunproto, %map_opts );
    return 1;
}

sub setup_nfs_luns {
    my $self = shift;
    my (%opts) = @_;
    return ( $self->_setup_spec_luns( "nfs", %opts ) );
}

sub setup_iscsi_luns {
    my $self = shift;
    my (%opts) = @_;
    return ( $self->_setup_spec_luns( "iscsi", %opts ) );
}

sub setup_fcp_luns {
    my $self = shift;
    my (%opts) = @_;
    return ( $self->_setup_spec_luns( "fcp", %opts ) );
}


sub _get_existing_luns {
    my $self            = shift;
    my $luninfo         = $self->{cfginfo}{$LUNS};
    my $existing        = $luninfo->{existing};
    my $existing_qtrees = $self->{containers}{$QTREES};

    # blocks_filer::filer_luns_get
    my @lunpath_list = filer_luns_get( $self->{fc} );
    if ($@) {
        $self->{error} = "lunList: $@";
        return 0;
    }
    $luninfo->{count}    = 0;
    $luninfo->{existing} = {};
    my $existing = $luninfo->{existing};
    foreach my $lunpath (@lunpath_list) {
        my $lunhash = {};
        if ( $lunpath !~ m%/vol/([^\/]+)/([^\/]+)/([^\/]+)$% ) {
            $self->{error} =
              "_get_existing_luns: Can't parse lun path: \"$lunpath\"";
logresult('WARN', $self->{error});
        }
        my ( $vol, $qtree, $lun ) = ( $1, $2, $3 );
        my $qtpath = "/vol/$vol/$qtree";
        if ( !$existing_qtrees->{$qtpath} ) {
            $self->{error} = "_get_existing_luns: no such qtree: $qtpath";
logresult('WARN', $self->{error});
        }
        my $qthash = $existing_qtrees->{$qtpath};
        if ( $qthash->{export} ) {
        }

        $qthash->{export}       = 0;
        $lunhash->{qtree}       = $qtree;
        $lunhash->{volume}      = $vol;
        $lunhash->{lun}         = $lun;
        $lunhash->{name}        = $lun;
        $lunhash->{mp_type}     = 'lun';
        $lunhash->{lunpath}     = $lunpath;
        $lunhash->{path}        = $lunpath;
        $lunhash->{is_exported} = $self->_is_exported($lunpath);

        $self->lun_online($lunpath);

        $existing->{$lunpath}      = $lunhash;
        $luninfo->{count}++;
        $qthash->{$LUNS} = {} unless ( $qthash->{$LUNS} );
        $qthash->{$LUNS}{$lunpath} = $lunhash;
    }

    $self->{containers}{$LUNS} = $luninfo->{existing};
    return ( $luninfo->{count} );
}


sub _get_existing_spec_luns {
    my $self            = shift;
    my $luninfo         = $self->{cfginfo}{$LUNS};
    my $existing_luns   = $luninfo->{existing};
    my $igrpinfo      = $self->{cfginfo}{igroups};
    my $existing_igroups   = $igrpinfo->{existing};
    for my $type ( ( "iscsi", "fcp", "nfs" ) ) {
        $luninfo->{$type} = {} unless ( $luninfo->{$type} );
        $luninfo->{$type}{existing} = {};
        $luninfo->{$type}{count}    = 0;
    }
    while ( my ($lunpath, $lunhash) = each(%$existing_luns) ) {
        if (!$lunhash->{igroups} || !$lunhash->{igroups_by_proto}) {
###           $self->_get_existing_lun_maps(igroups=>$igroups, luns=>$luns);
            $self->_get_existing_lun_maps();
        }
        my ($spec_luninfo, $spec_existing_luns);
        my $lunproto;
        if (keys(%{$lunhash->{igroups_by_proto}}) > 1) {
                $self->{error} = "_get_existing_spec_igroups: Conflicting protocols for LUN \"$lunpath\"";
                logWarn($self->{error});
        }
        foreach my $igname (keys(%{$lunhash->{igroups}})) {
            my $ighash = $existing_igroups->{$igname};
            if (!$ighash) {
                $self->{error} = "_get_existing_spec_igroups: Can't find igroup: \"$igname\"";
                logError($self->{error});
                next;
            }
        }
        # blocks_filer::filer_lun_share_none
        filer_lun_share_none( $self->{fc}, $lunpath );
        # blocks_filer::filer_lun_offline
        $self->lun_offline( $self->{fc}, $lunpath );
        foreach my $proto (keys(%{$lunhash->{igroups_by_proto}})) {
            $spec_luninfo = $luninfo->{$lunproto};
            $spec_existing_luns = $spec_luninfo->{existing};
            $spec_existing_luns->{$lunpath} = $lunhash;
            $spec_luninfo->{count}++;
            if ( $proto eq "nfs" ) {
                # blocks_filer::filer_lun_share_all
                filer_lun_share_all( $self->{fc}, $lunpath );
            } elsif ( $proto eq "iscsi" ) {
                # blocks_filer::filer_lun_online
                $self->lun_online($lunpath);
             } elsif ( $lunproto eq "fcp" || $lunproto eq "fc" ) {
                # blocks_filer::filer_lun_online
                $self->lun_online($lunpath);
             } else {
             }
         }
    }
}


sub _purge_lun_list {
    my $self = shift;
    my ($lun_list) = @_;
    my $luninfo          = $self->{cfginfo}{$LUNS};
    my $igrpinfo         = $self->{cfginfo}{igroups};
    my $existing         = $luninfo->{existing};
    my $existing_igroups = $igrpinfo->{existing};
    my $existing_qtrees  = $self->{containers}{$QTREES};
    logcomment("_purge_luns: $self->{fname}");

    if ( !ref($lun_list) || ref($lun_list) eq "HASH" ) {
        $lun_list = [$lun_list];
    }
    foreach my $lunpath (@$lun_list) {
        my $lunhash = $existing->{$lunpath};
        next unless ($lunhash);
        my $vol     = $lunhash->{volume};
        my $qtree   = $lunhash->{qtree};
        my $lun     = $lunhash->{lun};
        my $lunproto = $lunhash->{proto};

        # blocks_filer::filer_lun_exists
        if ( filer_lun_exists( $self->{fc}, $lunpath ) ) {
            my @igroups = keys( %{ $lunhash->{igroup} } );
            foreach my $ig (@igroups) {
                $self->_unmap_igroup_lun( $lun, $ig );
            }

            $self->lun_offline( $self->{fc}, $lunpath );

            # blocks_filer::filer_lun_share_none
            filer_lun_share_none( $self->{fc}, $lunpath );

            # blocks_filer::filer_lun_destroy
            filer_lun_destroy( $self->{fc}, $lunpath );
        }

        # blocks_filer::filer_lun_exists
        if ( filer_lun_exists( $self->{fc}, $lunpath ) ) {
            $self->{error} = "_purge_luns: Could not destroy lun $lunpath: $@";
logresult('WARN', $self->{error});
            return 0;
        }
        my $spec_luninfo  = $luninfo->{$lunproto};
        my $spec_existing = $spec_luninfo->{existing};
        delete( $existing->{$lunpath} );
        delete( $spec_existing->{$lunpath} );
        $luninfo->{count}--;
        $spec_luninfo->{count}--;

        my $qtpath = "/vol/$vol/$qtree";
        my $qthash = $existing_qtrees->{$qtpath};
        if ( !defined($qthash) ) {
            $self->{error} = "_purge_luns: Qtree not found: $qtpath";
logresult('WARN', $self->{error});
            return (0);
        }
        $qthash->{$LUNS} = {} unless ( $qthash->{$LUNS} );
        delete( $qthash->{$LUNS}{$lunpath} );
    }
    return (1);
}

sub _purge_luns {
    my $self     = shift;
    my $luninfo  = $self->{cfginfo}{$LUNS};
    my $existing = $luninfo->{existing};
    logcomment("_purge_luns: $self->{fname}");
    my $lun_list = [ keys( %{ $luninfo->{existing} } ) ];
    return ( $self->_purge_lun_list($lun_list) );
}

sub _purge_spec_luns {
    my $self          = shift;
    my $lunproto       = shift;
    my $luninfo       = $self->{cfginfo}{$LUNS};
    my $spec_luninfo  = $luninfo->{$lunproto};
    my $spec_existing = $spec_luninfo->{existing};
    logcomment("_purge_spec_luns: $self->{fname}");
    my $lun_list = [ keys(%$spec_existing) ];
    return ( $self->_purge_lun_list($lun_list) );
}

sub _purge_nfs_luns {
    my $self = shift;
    my (%opts) = @_;
    return ( _purge_spec_luns( "nfs", %opts ) );
}

sub _purge_iscsi_luns {
    my $self = shift;
    my (%opts) = @_;
    return ( _purge_spec_luns( "iscsi", %opts ) );
}

sub _purge_fcp_luns {
    my $self = shift;
    my (%opts) = @_;
    return ( _purge_spec_luns( "fcp", %opts ) );
}

sub _create_luns_for_qtree {
    my $self = shift;
    my ( $qtpath, $count, $lunproto, %opts ) = @_;

    if ($qtpath !~ m%/vol/[^/]+/[^/]+%) {
        $self->{error} = "_create_luns_for_qtree: bad qtree path: \"$qtpath\"";
logresult('FATAL', $self->{error});
        return ( 0 );
    }
    my $luninfo = $self->{cfginfo}{$LUNS};
    $lunproto = 'fcp' if ( $lunproto eq 'fc' );
    if ( $lunproto !~ /nfs|iscsi|fcp/ ) {
        $self->{error} = "_create_luns_for_qtree: unknown lun type: $lunproto";
logresult('WARN', $self->{error});
        return ( 0 );
    }

    my $ostype = $opts{ostype};
    logWarn("create_luns_for_qtree: No 'ostype' given") unless ($ostype);

    my $existing        = $luninfo->{existing};
    my $spec_luninfo    = $luninfo->{$lunproto};
    my $spec_existing   = $spec_luninfo->{existing};
    my $existing_qtrees = $self->{containers}{$QTREES};
    my $prefix          = $opts{prefix}
      || $spec_luninfo->{prefix}
      || $luninfo->{prefix}
      || 'lun';
    my $size = $opts{size}
      || $spec_luninfo->{size}
      || $luninfo->{size}
      || '10m';

    my $qthash;
    if ( !$qtpath ) {
        $self->{error} = "_create_luns_for_qtree: no qtpath given";
logresult('WARN', $self->{error});
        return 0;
    } elsif ( ref($qtpath) ) {
        $qthash = $qtpath;
        $qtpath = $qthash->{qtpath};
    } elsif ( $existing_qtrees->{$qtpath} ) {
        $qthash = $existing_qtrees->{$qtpath};
    } else {
        $self->{error} = "_create_luns_for_qtree: no such qtree: $qtpath";
logresult('WARN', $self->{error});
        return 0;
    }
    if ( $qthash->{export} ) {
logresult('WARN', "_create_luns_for_qtree: qtree $qtpath has \"export\" option set.");
    }
    $qthash->{export} = 0;
    my $qtree = $qthash->{qtree};
    my $vol   = $qthash->{volume};
    my $time  = time();
    logcomment("Creating $count luns for qtree $qtpath on $self->{fname}");
    my $new_luns = {};
    for ( my $cntr = 0 ; $cntr < $count ; $cntr++ ) {
        my $lun     = "${prefix}_${time}_${cntr}";
        my $lunpath = "$qtpath/$lun";
        if (defined($existing->{$lunpath})) {
            my $ctr = 1;
            my $base_lunpath = $lunpath;
            while (defined($existing->{$lunpath})) {
                $lunpath = $base_lunpath . "_" . $ctr;
                $ctr++;
           }
        }

        # blocks_filer::filer_lun_create
        filer_lun_create( $self->{fc}, $lunpath, $size, $ostype );

        # blocks_filer::filer_lun_exists
        if ( !filer_lun_exists( $self->{fc}, $lunpath) ) {
            $self->{error} =
"_create_luns_for_qtree: cannot create lun $lunpath on $self->{fname}: $@";
logresult('WARN', $self->{error});
            return ( $self->{error} );
        }
        logcomment("$self->{fname}:created lun $lun in qtree $qtpath");
        my $lunhash = {};
        $lunhash->{qtree}   = $qtree;
        $lunhash->{name}    = $lun;
        $lunhash->{lun}     = $lun;
        $lunhash->{volume}  = $vol;
        $lunhash->{size}    = $size;
        $lunhash->{proto}   = $lunproto;
        $lunhash->{lunpath} = $lunpath;
        $lunhash->{path}    = $lunpath;
        $lunhash->{mp_type} = 'lun';

        $self->lun_online($lunpath);
        if ( $lunproto eq "nfs" ) {

            # blocks_filer::filer_lun_share_all
            filer_lun_share_all( $self->{fc}, $lunpath );
        } elsif ( $lunproto eq "iscsi" ) {

            # blocks_filer::filer_lun_share_none
            filer_lun_share_none( $self->{fc}, $lunpath );
        } elsif ( $lunproto eq "fcp" || $lunproto eq "fc" ) {

            # blocks_filer::filer_lun_share_none
            filer_lun_share_none( $self->{fc}, $lunpath );
        } else {
            $self->{error} =
              "_get_existing_luns: unknown lun type: \"$lunproto\"";
logresult('WARN', $self->{error});
             return ( $self->{error} );
        }

        $new_luns->{$lunpath}      = $lunhash;
        $existing->{$lunpath}      = $lunhash;
        $spec_existing->{$lunpath} = $lunhash;
        $luninfo->{count}++;
        $spec_luninfo->{count}++;

        $qthash->{$LUNS} = {} unless ( $qthash->{$LUNS} );
        $qthash->{$LUNS}{$lunpath} = $lunhash;
    }
    return ($new_luns);
}

sub _map_igroup_lun {
    my $self = shift;
    my ( $igroup, $lun, %opts ) = @_;
    my $luninfo        = $self->{cfginfo}{$LUNS};
    my $igrpinfo       = $self->{cfginfo}{igroups};
    my $existing_luns  = $luninfo->{existing};
    my $existing_igrps = $igrpinfo->{existing};
    my ( $luninfo, $lunpath, $igrp_info, $ig_name );
    if ( !ref($lun) ) {
        $lunpath = $lun;
        $luninfo = $existing_luns->{$lunpath};
    } else {
        $luninfo = $lun;
        $lunpath = $luninfo->{lunpath};
    }
    $luninfo->{igroups} = {} unless ($luninfo->{igroups});
    $luninfo->{igroups_by_proto} = {} unless ($luninfo->{igroups_by_proto});
    my $lun_igroups = $luninfo->{igroups};
    my $lun_igroups_by_proto = $luninfo->{igroups_by_proto};

    if ( !ref($igroup) ) {
        $ig_name   = $igroup;
        $igrp_info = $existing_igrps->{$ig_name};
    } else {
        $igrp_info = $igroup;
        $ig_name   = $igrp_info->{name};
    }
    my $igproto = $igrp_info->{name};
    $lun_igroups_by_proto->{$igproto} = {} unless ($lun_igroups_by_proto);
    if ( keys(%$lun_igroups_by_proto) && !exists($lun_igroups_by_proto->{$igproto}) ) {
        $self->{error} =
"_map_lun_to_igroup: igroup $ig_name type: $igproto does not match that of LUN $lunpath";
logresult('WARN', $self->{error});
    }

    # blocks_filer::filer_lun_igroups_get
    my @igroups_for_lun = filer_lun_igroups_get( $self->{fc}, $lunpath );
    if ( defined( $luninfo->{igroups}{$ig_name} ) ) {
    }
    if ( grep( /^$ig_name$/, @igroups_for_lun ) ) {
        $self->{error} = "LUN $lunpath   is already mapped to igroup  $ig_name";
        logresult( 'FATAL',
            "LUN $lunpath   is already mapped to igroup  $ig_name" );
        return;
    }
    $igrp_info->{$LUNS} = {} unless ( $igrp_info->{$LUNS} );

    # blocks_filer::filer_lun_map_low
    my $lunid = filer_lun_map_low( $self->{fc}, $lunpath, $ig_name );

    # blocks_filer::filer_igroup_lun_lunid_get
    my $lunid = filer_igroup_lun_lunid_get( $self->{fc}, $ig_name, $lunpath );
    $lun_igroups->{$ig_name} = $lunid;
    $lun_igroups_by_proto->{$igproto}{$ig_name} = $lunid;
    $igrp_info->{$LUNS} = {} unless ( $igrp_info->{$LUNS} );
    $igrp_info->{$LUNS}{$lunpath} = $lunid;
}

sub _map_igroup_luns {
    my $self = shift;
    my ( $igroup, $lun_list, %opts ) = @_;
    foreach my $lun (@$lun_list) {
        $self->_map_igroup_lun( $igroup, $lun, %opts );
    }
}

sub _map_spec_luns {
    my $self = shift;
    my ( $type, %opts ) = @_;
    my $luninfo               = $self->{cfginfo}{$LUNS};
    my $spec_luninfo          = $luninfo->{$type};
    my $spec_existing_luns    = $spec_luninfo->{existing};
    my $igrpinfo              = $self->{cfginfo}{igroups};
    my $spec_igrpinfo         = $igrpinfo->{$type};
    my $spec_existing_igroups = $spec_igrpinfo->{existing};
    my $lun_list              = [ keys(%$spec_existing_luns) ];
    my $igrp_list             = [ keys(%$spec_existing_igroups) ];
    my $igroup                = $opts{igroup};
    if ( !$igroup ) {
        $igroup = $$igrp_list[0];
    }
    if ( !defined($igroup) ) {
        $self->{error} = "_map_spec_luns: no igroup found for type \"$type\"";
        logresult( 'INFO',
            "_map_spec_luns  igrpinfo:\n\n" . Dumper($igrpinfo) );
        logresult( 'FATAL',
            "_map_spec_luns: no igroup found for type \"$type\"" );
        return ( $self->{error} );
    }
    my ( $ig_name, $ig_hash );
    if ( ref($igroup) ) {
        $ig_hash = $igroup;
        $ig_name = $ig_hash->{name};
    } else {
        $ig_name = $igroup;
        $ig_hash = $spec_existing_igroups->{$ig_name};
    }
    if ( $ig_hash->{proto} ne $type ) {
        $self->{error} = "_map_spec_luns: igroup $ig_name is not type $type";
        logresult( 'INFO',
            "_map_spec_luns: igroup $ig_name is not type $type" );
    }
    $self->_map_igroup_luns( $ig_name, $lun_list, %opts );
}

sub _unmap_igroup_lun {
    my $self = shift;
    my ( $lun, $igroup, %opts ) = @_;
    my $luninfo        = $self->{cfginfo}{$LUNS};
    my $igrpinfo       = $self->{cfginfo}{igroups};
    my $existing_luns  = $luninfo->{existing};
    my $existing_igrps = $igrpinfo->{existing};
    my ( $luninfo, $lunpath, $igrp_info, $ig_name );
    if ( !ref($lun) ) {
        $lunpath = $lun;
        $luninfo = $existing_luns->{$lunpath};
    } else {
        $luninfo = $lun;
        $lunpath = $luninfo->{lunpath};
    }

    if ( !ref($igroup) ) {
        $ig_name   = $igroup;
        $igrp_info = $existing_igrps->{$ig_name};
    } else {
        $igrp_info = $igroup;
        $ig_name   = $igrp_info->{name};
    }
    if ( $luninfo->{proto} ne $igrp_info->{proto} ) {
        $self->{error} =
"_map_lun_to_igroup: igroup $ig_name type $igrp_info->{proto} does not match  LUN $lunpath type $luninfo->{proto}";
logresult('WARN', $self->{error});
        return ( $self->{error} );
    }

    # blocks_filer::filer_lun_unmap
    filer_lun_unmap( $self->{fc}, $lunpath, $ig_name );
    delete( $luninfo->{igroups}{$ig_name} );
    delete( $igrp_info->{$LUNS}{$lunpath} );
}

sub _unmap_igroup_luns {
    my $self = shift;
    my ( $igroup, %opts ) = @_;
    my $igrpinfo       = $self->{cfginfo}{igroups};
    my $luninfo        = $self->{cfginfo}{$LUNS};
    my $existing_luns  = $luninfo->{existing};
    my $existing_igrps = $igrpinfo->{existing};
    my ( $ig_name, $igrp_info );
    if ( !ref($igroup) ) {
        $ig_name   = $igroup;
        $igrp_info = $existing_igrps->{$ig_name};
    } else {
        $igrp_info = $igroup;
        $ig_name   = $igrp_info->{name};
    }
    if ( !$igrp_info->{$LUNS} ) {
        return;
    }
    foreach my $lun ( keys( %{ $igrp_info->{$LUNS} } ) ) {
        $self->_unmap_igroup_lun( $lun, $igrp_info, %opts );
    }
}


sub _create_spec_luns {
    my $self = shift;
    my ( $lunproto, %opts ) = @_;
    my $luninfo      = $self->{cfginfo}{$LUNS};
    my $spec_luninfo = $luninfo->{$lunproto};
    my $existing_qtrees = $self->{containers}{$QTREES};
    my @qtree_paths     = keys(%$existing_qtrees);
    my $needed       = ( exists $opts{new} ) ? $opts{new} : $spec_luninfo->{new};
    my $new_lun_count = $luninfo->{count} + $needed;
    return if !$needed;
    $lunproto = 'fcp' if ( $lunproto eq 'fc' );
    if ( $lunproto !~ /nfs|iscsi|fcp/ ) {
        $self->{error} = "_create_spec_luns: unknown lun type: $lunproto";
logresult('FATAL', $self->{error});
        return ( 0 );
    }

    my $nqtrees = scalar(@qtree_paths);
    if ( $nqtrees == 0 ) {
        logresult( 'FATAL', "create_luns: no qtrees exist on $self->{fname}" );
        $self->{error} = "create_luns: no qtrees exist on $self->{fname}";
logresult('FATAL', $self->{error});
        return (0);
    }
#    my $lun_per_qtree = int( $needed / $nqtrees );
#    my $rem           = $needed % $nqtrees;
#    foreach my $qtpath (@qtree_paths) {
#        $self->_create_luns_for_qtree( $qtpath, $lun_per_qtree, $lunproto, %opts );
#        return (0) if ( $self->{error} );
#    }
    my $new_luns = {};
    for (my $i=0; $luninfo->{count} < $new_lun_count; $i = ($i + 1) % $nqtrees) {
        my $qtpath = $qtree_paths[$i];
        my $newx = $self->_create_luns_for_qtree($qtpath, 1, $lunproto, %opts);
        return(0) if ($self->{error});
        $new_luns = { %$new_luns, %$newx };
    }
    $luninfo->{new} = 0;
    return ($new_luns);
}

sub _create_iscsi_luns {
    my $self = shift;
    my (%opts) = @_;
    $opts{ostype} = "linux" unless ($opts{ostype});
    return ( _create_spec_luns( "iscsi", %opts ) );
}

sub _create_fcp_luns {
    my $self = shift;
    my (%opts) = @_;
    $opts{ostype} = "solaris" unless ($opts{ostype});
    return ( _create_spec_luns( "fcp", %opts ) );
}

sub _create_luns {
    my $self = shift;
    my (%opts) = @_;
    my $luninfo = $self->{cfginfo}{$LUNS};
    my $needed  = ( exists $opts{new} ) ? $opts{new} : $luninfo->{new};
    my $new_luns = {};
    for my $lunproto ( ( 'nfs', 'fcp', 'iscsi' ) ) {
        $opts{new} = $needed;
        my $newx = $self->_create_spec_luns( $lunproto, %opts );
        if ( $self->{error} ) {
            return (0);
        }
        $new_luns = { %$new_luns, %$newx };
    }
    return ($new_luns);
}

##################################################
#
#        End LUN section
#
##################################################

##################################################
#
#        Start IGroup section
#
##################################################

use blocks_filer qw(filer_igroup_show_get);
use blocks_filer qw(filer_igroup_is_fcp);
use blocks_filer qw(filer_igroup_is_iscsi);
use blocks_filer qw(filer_igroup_show_get);
use blocks_filer qw(filer_igroup_luns_get);
use blocks_filer qw(filer_igroup_is_defined);
use blocks_filer qw(filer_igroup_nodenames_get);
use blocks_filer qw(filer_igroup_destroy);
use blocks_filer qw(filer_igroup_node_add);
use blocks_filer qw(filer_igroup_node_remove);
use blocks_filer qw(filer_fcp_adapters_get);
use blocks_filer qw(filer_fcp_adapter_get);

use blocks_filer_2 qw(igroup_add);
use blocks_filer_2 qw(igroup_remove);
use blocks_filer_2 qw(igroup_create);

sub setup_igroups {
    my $self = shift;
    my (%opts) = @_;
    $self->{cfginfo}{igroups} = {} unless ($self->{cfginfo}{igroups});
    my $igrpinfo = $self->{cfginfo}{igroups};
    my $needed = ( exists $opts{count} ) ? $opts{count} : $igrpinfo->{count};
    my $force      = ( exists $opts{force} ) ? $opts{force} : 0;
    return if (!$needed && !$force);

    my $clientinfo = $opts{clients};

    if ( !defined( $igrpinfo->{existing} ) || $force ) {
        $igrpinfo->{existing}     = {};
        $igrpinfo->{count}        = 0;
        $igrpinfo->{new}          = 0;
        $self->{cfginfo}{igroups} = $igrpinfo;
        $self->_get_existing_igroups(%opts);
        if ($clientinfo) {
            my $setup_iscsi = 1;
            my $setup_fcp   = 1;
            while ( my ( $ig_name, $ig_hash ) =
                each( %{ $igrpinfo->{existing} } ) )
            {
                my $ig_type = $ig_hash->{proto};
                if ( $ig_type eq 'iscsi' && $setup_iscsi ) {
                    $self->setup_iscsi();
                    $setup_iscsi = 0;
                }
                if ( $ig_type eq 'fcp' && $setup_fcp ) {
                    $self->setup_fcp();
                    $setup_fcp = 0;
                }
                $self->_get_igroup_initiator_info(
                    $ig_name, $clientinfo,
                    setup_iscsi => 0,
                    setup_fcp   => 0
                );

#                $self->remove_igroup_clients($ig_name, $clientinfo);
#                $self->add_matching_clients_to_igroup($ig_name, $clientinfo, setup_iscsi=>0, setup_fcp=>0);
            }
        }
    }

    my $existing = $igrpinfo->{existing};

    my $mode = $self->{mode} || $opts{mode};
    if ( $mode eq $PURGE ) {
        $self->_purge_igroups();
        return if ( $self->{error} );
    }
    my $num_existing = keys( %{ $igrpinfo->{existing} } );
    my $new          = undef;
    $new = $opts{new} if ( !defined($new) && defined( $opts{new} ) );
    $new = $needed - $num_existing if ( !defined($new) && defined($needed) );
    $new = 0 unless ( defined($new) );
    if ( $new > 0 ) {
        $igrpinfo->{new} = $new;
        $self->_create_igroups(%opts);
    } else {
        $igrpinfo->{new} = 0;
    }
    if ($clientinfo) {
        $self->populate_igroups($clientinfo);
    }
    return 1;
}

sub populate_igroups {
    my $self       = shift;
    my $clientinfo = shift;
    my $igrpinfo   = $self->{cfginfo}{igroups};
    while ( my ( $ig_name, $ig_hash ) = each( %{ $igrpinfo->{existing} } ) ) {
        $self->add_matching_clients_to_igroup( $ig_hash, $clientinfo );
    }
}


sub _setup_spec_igroup {
    my $self   = shift;
    my $igproto = shift;
    $igproto = 'fcp' if ( $igproto eq 'fc' );
    my (%opts) = @_;
    $opts{spec_setup} => 1;
    my %opts_copy = %opts;
    $opts_copy{count} = 0;
    $opts_copy{force} = 1;

    $self->{cfginfo}{igroups} = {} unless ($self->{cfginfo}{igroups});
    my $igrpinfo = $self->{cfginfo}{igroups};
    my $spec_igrpinfo = $igrpinfo->{$igproto};
    my $needed = $opts{count} || $spec_igrpinfo->{count} || 1;
    if ( !$igrpinfo->{existing} ) {
        $self->setup_igroups(%opts_copy);
    }

    if ( $igproto !~ /iscsi|fcp/ ) {
        $self->{error} = "_create_igroup: unkown igroup type: $igproto";
logresult('WARN', $self->{error});
        return ( $self->{error} );
    }

    $igrpinfo->{$igproto} = {} unless ($igrpinfo->{$igproto});
    $spec_igrpinfo = $igrpinfo->{$igproto};
    if ( !defined( $spec_igrpinfo->{existing} ) ) {
        $spec_igrpinfo->{existing} = {};
        $spec_igrpinfo->{count}    = 0;
        $spec_igrpinfo->{new}      = 0;
    }

    if (   !$spec_igrpinfo
        || !$spec_igrpinfo->{existing}
        || !keys( %{ $spec_igrpinfo->{existing} } ) )
    {
        $self->create_igroup( $igproto, %opts );
    }

    return 1;
}


sub setup_iscsi_igroup {
    my $self = shift;
    my (%opts) = (@_);
    return ( $self->_setup_spec_igroup( "iscsi", %opts ) );
}

sub setup_fcp_igroup {
    my $self = shift;
    my (%opts) = (@_);
    return ( $self->_setup_spec_igroup( "fcp", %opts ) );
}

sub _get_igroup_type {
    my $self     = shift;
    my $igrp     = shift;
    my $igrpinfo = $self->{cfginfo}{igroups};
    my $existing = $igrpinfo->{existing};
    my ( $ig_name, $ig_hash );
    if ( !ref($igrp) ) {
        $ig_name = $igrp;
        $ig_hash = $existing->{$igrp};
    } elsif ( ref($igrp) eq "HASH" ) {
        $ig_hash = $igrp;
        $ig_name = $ig_hash->{name};
    }

    # blocks_filer::filer_igroup_is_fcp
    # blocks_filer::filer_igroup_is_iscsi
    if ( filer_igroup_is_fcp( $self->{fc}, $ig_name ) ) {
        return ('fcp');
    } elsif ( filer_igroup_is_iscsi( $self->{fc}, $ig_name ) ) {
        return ('iscsi');
    } else {
        return (undef);
    }
}


sub _filer_get_lun_maps ($) {
  my $self = shift;
  my $ig_hash = {};
  my $lun_hash = {};

  my $cmd = "lun show -m";
  my($text) = sayEOF($self->{fc}, $cmd);

   my ($lunpath, $igroup, $lunid, $protocol);
  foreach my $line (split(/\n/, $text)) {
    if ($line =~ /^LUN Path/i || $line =~ /^\s*$/) {
      next;
    } elsif ($line =~ /^---/) {
      next;
    }  elsif ($line =~ /^\s*$/ or $line =~ /^===== /) {
      next; # vfiler header lines
    } elsif (($line =~ /lun show\s?:/) || ($line =~ /usage:/)) {
      $self->{error} = "_get_lun_maps: Unexpected message from ($cmd): $line";
logError($self->{error});
      next;
    }
    $line =~ s/\s+$//;
    my @matches;
###    my ($lunpath, $igroup, $lunid, $protocol) = ($line =~ m%^(/vol/\S+)\s+(\S+)\s+(\d+)\s+(\w+)$%);
    if ( (@matches = ($line =~ m%^(/vol/\S+)\s+(\S+)\s+(\d+)\s+(\w+).*$%)) && $matches[0] ) {
       ($lunpath, $igroup, $lunid, $protocol) = @matches;
###    } elsif ( (@matches = ($line =~ m%^\s+(\S+)\s+(\d+)\s+(\w+).*$%)) && $matches[0] ) {
    } elsif ( (@matches = ($line =~ m%^\s+(\S+)\s+(\d+)\s+(\w+).*$%)) && $matches[0] ) {
       ($igroup, $lunid, $protocol) = @matches;
    } else {
      $self->{error} = "_get_lun_maps: Can't parse line: \"$line\"";
logError($self->{error});
      next;
    }
    # Initialize the next sub-hash if necessary
    $lun_hash->{$lunpath} = {}
       unless ($lun_hash->{$lunpath});
    $ig_hash->{$igroup} = {}
       unless ($ig_hash->{$igroup});
    $ig_hash->{$igroup}{$lunpath} = $lunid;
    $lun_hash->{$igroup}{$lunpath} = $lunid;
  } # end foreach
logDbg("Exit filer_get_lun_maps\n\n".Dumper($ig_hash));
  return ($ig_hash, $lun_hash);
}


sub _get_existing_lun_maps {
    my $self          = shift;
    my (%opts)        = @_;
    my $igrpinfo      = $self->{cfginfo}{igroups};
    my $luninfo       = $self->{cfginfo}{$LUNS};
    my $existing_luns = $luninfo->{existing};
    my $existing_igrps = $igrpinfo->{existing};
    my $luns_to_check;
    my $igrps_to_check;

    if ($opts{luns}) {
        if (ref($opts{luns}) =~ /HASH/) {
           $luns_to_check = $opts{luns};
        } elsif (ref($opts{luns}) =~ /ARRAY/) {
           $luns_to_check = {};
           foreach (@{$opts{luns}}) { $luns_to_check->{$_} = 1; }
###           map( { $luns_to_check->{$a} = 1 } @{$opts{luns}} );
        } else {
           $self->{error} = "_get_existing_lun_maps:  'luns' option must be arrayref or hashref";
           logError($self->{error});
        }
    }

    if ($opts{igroups}) {
        if (ref($opts{igroups}) =~ /HASH/) {
           $igrps_to_check = $opts{igroups};
        } elsif (ref($opts{igroups}) =~ /ARRAY/) {
           $igrps_to_check = {};
           foreach (@{$opts{igroups}}) { $igrps_to_check->{$_} = 1; }
###           map( { $igrps_to_check->{$a} = 1 } @{$opts{igroups}} );
        } else {
           $self->{error} = "_get_existing_lun_maps:  'igroups' option must be arrayref or hashref";
           logError($self->{error});
        }
    }

    $self->setup_igroups(force=>1, count=>0)
       unless ($existing_igrps);
    $self->setup_luns(force=>1, count=>0)
       unless ($existing_luns);
    my ($igrp_maps, $lun_maps) = $self->_filer_get_lun_maps();
    while ( my ($ig, $igmaps) = each(%$igrp_maps) ) {
        my $ighash = $existing_igrps->{$ig};
        if (!$ighash) {
            $self->{error} = "_get_existing_lun_maps:  No igroup found for name: $ig";
            logError($self->{error});
            next;
        }
        $ighash->{$LUNS} = {} unless ($ighash->{$LUNS});
        my $igproto = $ighash->{proto};
        while ( my($lunpath, $lunid) =  each(%$igmaps) ) {
            my $lunhash = $existing_luns->{$lunpath};
            if (!$lunhash) {
                $self->{error} = "_get_existing_lun_maps:  No LUN found for lunpath: $lunpath";
                logError($self->{error});
                next;
            }
            $lunhash->{igroups} = {} unless ($lunhash->{igroups});
            $lunhash->{igroups_by_proto} = {} unless ($lunhash->{igroups});
            my $lun_igroups = $lunhash->{igroups};
            my $lun_igroups_by_proto = $lunhash->{igroups_by_proto};
            $lun_igroups_by_proto->{$igproto} = {} unless ($lunhash->{igroups});
            $lun_igroups->{$ig} = $lunid;
            if ( keys(%$lun_igroups_by_proto) && !exists($lun_igroups_by_proto->{$igproto}) ) {
                $self->{error} = "_get_existing_lun_maps:  Conflicting types for lunpath: $lunpath";
                logWarn($self->{error});
                next;
            }
            $lun_igroups_by_proto->{$igproto}{$ig} = $lunid;
        }
    }
}


sub _get_existing_igroups {
    my $self          = shift;
    my (%opts)        = @_;
    my $igrpinfo      = $self->{cfginfo}{igroups};
    my $get_luns      = $opts{get_luns} || 1;
    my $clientinfo    = $opts{clients};
    # blocks_filer::filer_igroup_show_get
    my %igroup_hash = filer_igroup_show_get( $self->{fc}, undef );

logresult('INFO', "get_existing_igroups IGROUP_HASH:\n\n".Dumper(\%igroup_hash));

    my @igroup_list = keys(%igroup_hash);

    $igrpinfo->{existing} = {};
    my $existing = $igrpinfo->{existing};
    for my $type ( ( "iscsi", "fcp" ) ) {
        $igrpinfo->{$type} = {} unless ( $igrpinfo->{$type} );
        $igrpinfo->{$type}{existing} = {};
        $igrpinfo->{$type}{count}    = 0;
    }
    while ( my ( $ig_name, $igrp_info ) = each(%igroup_hash) ) {
        my $igproto         = $igrp_info->{proto};
        my $igrp_hash      = {};

        # my $igrpproto = $self->_get_igroup_type($ig_name);
        my $igrpproto = $igrp_info->{'initiator-group-type'};
        $igrpproto = lc($igrpproto);
        $igrpproto = 'fcp' if ( $igrpproto eq 'fc' );
        $igrpinfo->{$igrpproto} = {} unless ( $igrpinfo->{$igrpproto} );
        my $initiator_str = $igrp_info->{Nodename};
        my @initiators = split ( /\s+/, $initiator_str );
        $igrp_hash->{initiators} = {};
        my $spec_igrpinfo = $igrpinfo->{$igrpproto};
        my $spec_existing = $spec_igrpinfo->{existing};
#        if ($get_luns) {
#             # blocks_filer::filer_igroup_luns_get
#             my @lunpath_list = filer_igroup_luns_get( $self->{fc}, $ig_name );
#             $igrp_hash->{$LUNS} = {};
#             foreach my $lunpath (@lunpath_list) {
#                 my $luninfo = $existing_luns->{$lunpath};
# 
#                 # blocks_filer::filer_igroup_lun_lunid_get
#                 my $lunid =
#                   filer_igroup_lun_lunid_get( $self->{fc}, $ig_name, $lunpath );
#                 $igrp_hash->{$LUNS}{$lunpath} = $lunid;
#             }
#         }
        $igrp_hash->{igroup}       = $ig_name;
        $igrp_hash->{name}         = $ig_name;
        $igrp_hash->{proto}        = $igrpproto;
        $existing->{$ig_name}      = $igrp_hash;
        $spec_existing->{$ig_name} = $igrp_hash;
        $igrpinfo->{count}++;
        $spec_igrpinfo->{count}++;
    }
    if ($get_luns) {
        my $luninfo       = $self->{cfginfo}{$LUNS};
        my $existing_luns = $luninfo->{existing};
        $self->setup_luns(force=>1, count=>0) unless ($existing_luns);
        my $luns = [ keys(%$existing_luns) ];
        my $igroups = [ keys(%$existing) ];
        $self->_get_existing_lun_maps(igroups=>$igroups, luns=>$luns);
    }
    return ( $igrpinfo->{count} );
}


sub _get_fcp_target_info {
    my $self       = shift;
    $self->{fcp} = {} unless ($self->{fcp});
    $self->{fcp}{adapters} = {} unless ($self->{fcp}{adapters});
    $self->{fcp}{adapters_by_portname} = {} unless ($self->{fcp}{adapters});
###    $self->{fcp}{adapters} = filer_fcp_adapters_get($self->{fc});
    my $out = sayEOF($self->{fc}, "fcp show adapter -v");
    my @lines = split(/\n/, $out);
    my $adapter;
    for (my $line = shift(@lines); @lines; $line = shift(@lines)) {
         my ($key, $val) = ($line =~ /(\w+):\s+(.*)/);
         $key = lc($key);
         $key =~ s/\s+/_/;
         if ($key eq 'slot') {
              $self->{fcp}{adapters_by_portname}{$adapter->{portname}} = $adapter if ($adapter && $adapter->{portname});
              $adapter = {};
              $adapter->{$key} = $val;
              $self->{fcp}{adapters}{$adapter->{slot}} = $adapter if ($adapter && $adapter->{slot});
         } elsif ($key eq 'portname' || $key eq 'nodename') {
              if ($val =~ /\S+\s+\((\w+)\)/) {
                  $val = $1;
              }
              $adapter->{$key} = $val;
         } else {
              $adapter->{$key} = $val;
         }
     }
     $self->{fcp}{adapters}{$adapter->{slot}} = $adapter if ($adapter && $adapter->{slot});
     $self->{fcp}{adapters_by_portname}{$adapter->{portname}} = $adapter if ($adapter && $adapter->{portname});
logresult('WARN', "FILER TARGET INFO:\n\n".Dumper($self->{fcp}{adapters_by_portname}));
}


sub _purge_one_igroup {
    my $self       = shift;
    my $igrp       = shift;
    my $clientinfo = shift;
    my (%opts) = @_;
    my $ig_name;
    my $ig_hash;
    my $igrpinfo = $self->{cfginfo}{igroups};
    my $existing = $igrpinfo->{existing};
    logcomment("_purge_igroup_list: $self->{fname}");

    if ( !ref($igrp) ) {
        $ig_name = $igrp;
        $ig_hash = $existing->{$igrp};
    } elsif ( ref($igrp) eq "HASH" ) {
        $ig_hash = $igrp;
        $ig_name = $ig_hash->{name};
    }
    if ( !$ig_hash ) {
        return (1);
    }
    my $igrpproto      = $ig_hash->{proto};
    my $spec_igrpinfo = $igrpinfo->{igrpproto};
    my $spec_existing = $spec_igrpinfo->{existing};
    $self->_unmap_igroup_luns($ig_name);
    ### my @client_names = keys(%{$ig_hash->{clients}});
    my @clients = ();
    foreach my $cl_name ( keys( %{ $ig_hash->{clients} } ) ) {
        my $cl_hash = $clientinfo->{clients}{$cl_name};
        if ( !$cl_hash ) {
            next;
        }
        my $cl_igroups = $cl_hash->{igroups};
        delete( $cl_igroups->{$ig_name} );
    }

    # blocks_filer::filer_igroup_is_defined
    if ( filer_igroup_is_defined( $self->{fc}, $ig_name ) ) {

        # blocks_filer::filer_igroup_destroy
        filer_igroup_destroy( $self->{fc}, $ig_name );
    }

    # blocks_filer::filer_igroup_is_defined
    if ( filer_igroup_is_defined( $self->{fc}, $ig_name ) ) {
        $self->{error} =
          "_purge_igroups: Could not destroy igroup $ig_name: $@";
logresult('WARN', $self->{error});
        return 0;
    }
    delete( $existing->{$ig_name} );
    delete( $spec_existing->{$ig_name} );
    $igrpinfo->{count}--;
    $spec_igrpinfo->{count}--;
    return (1);
}

sub _purge_igroup_list {
    my $self       = shift;
    my $igrp_list  = shift;
    my $clientinfo = shift;
    my (%opts) = @_;
    my $igrpinfo = $self->{cfginfo}{igroups};
    my $existing = $igrpinfo->{existing};
    logcomment("_purge_igroup_list: $self->{fname}");
    if ( !ref($igrp_list) || ref($igrp_list) eq "HASH" ) {
        $igrp_list = [$igrp_list];
    }
    foreach my $igroup (@$igrp_list) {
        $self->_purge_one_igroup( $igroup, $clientinfo, %opts );
        if ( $self->{error} ) {
        }
    }
    return (1);
}

sub _purge_igroups {
    my $self       = shift;
    my $clientinfo = shift;
    my (%opts) = @_;
    my $igrpinfo = $self->{cfginfo}{igroups};
    my $existing = $igrpinfo->{existing};
    logcomment("_purge_igroups: $self->{fname}");
    my $igrp_list = [ keys( %{ $igrpinfo->{existing} } ) ];
    return ( $self->_purge_igroup_list( $igrp_list, $clientinfo, %opts ) );
}

sub _purge_spec_igroups {
    my $self   = shift;
    my $igproto = shift;
    $igproto = 'fcp' if ( $igproto eq 'fc' );
    my $clientinfo = shift;
    my (%opts) = @_;
    my $igrpinfo      = $self->{cfginfo}{igroups};
    my $spec_igrpinfo = $igrpinfo->{$igproto};
    my $spec_existing = $spec_igrpinfo->{existing};
    logcomment("_purge_spec_igroups: $self->{fname}");
    my $igrp_list = [ keys(%$spec_existing) ];
    return ( $self->_purge_igroup_list( $igrp_list, $clientinfo, %opts ) );
}

sub _purge_iscsi_igroups {
    my $self       = shift;
    my $clientinfo = shift;
    my (%opts) = @_;
    return ( $self->_purge_spec_igroups( "iscsi", $clientinfo, %opts ) );
}

sub _purge_fcp_igroups {
    my $self       = shift;
    my $clientinfo = shift;
    my (%opts) = @_;
    return ( $self->_purge_spec_igroups( "fcp", $clientinfo, %opts ) );
}


sub _get_iscsi_initiator_nodename {
    my $self           = shift;
    my $initiator_name = shift;
    my $hash           = $self->_get_iscsi_initiator_hash();
    my $wwpn_nodename  = $hash->{$initiator_name};
    if ( !$wwpn_nodename ) {
        $self->{error} =
"get_iscsi_initiator_nodename:  No iscsi nodename defined for initiator: $initiator_name";
        logresult( 'FATAL',
"get_iscsi_initiator_nodename:  No iscsi nodename defined for initiator: $initiator_name"
        );
        return;
    }
    logresult( 'INFO',
"get_iscsi_initiator_nodename   client: \"$initiator_name\"    wwpn_nodename: \"$wwpn_nodename\""
    );
    return ($wwpn_nodename);
}


sub _get_fcp_initiator_nodename {
    my $self           = shift;
    my $initiator_name = shift;
    my $hash           = $self->_get_fcp_initiator_hash();
    my $wwpn_nodename  = $hash->{$initiator_name};
    if ( !$wwpn_nodename ) {
        $self->{error} =
"get_fcp_initiator_nodename:  No fcp nodename defined for initiator: $initiator_name";
logresult( 'FATAL', $self->{error});
        return;
    }
    logresult( 'INFO',
"get_fcp_initiator_nodename   client: \"$initiator_name\"    wwpn_nodename: \"$wwpn_nodename\""
    );
    return ($wwpn_nodename);
}


sub _get_iscsi_initiator_hash {
    my $self           = shift;
    my $initiator_hash = {};
    my $out            = sayEOF( $self->{fc}, "iscsi initiator show" );
    $out =~ s/^(\s\n)*//;
    my @lines = split ( /\n/, $out );
    foreach my $line (@lines) {
        $line =~ s/^\s*//;
        my $initiator_name = undef;
        my $wwpn_nodename  = undef;
        my ( $tsih, $tpgroup, $host_and_initiator ) = split ( /\s+/, $line, 3 );
        next if ( $line !~ /^\d+\s+\d+\s+(.*)/ );
        if ( $host_and_initiator !~ m%^([\w\.]+) \(([^\s\/]+)% ) {
            $self->{error} =
"_get_iscsi_initiator_hash:  Could not parse \"host/initiator\" field: \"$host_and_initiator\"";
logresult('WARN', $self->{error});
            return ( $self->{error} );
        }
        $initiator_name = $1;
        $wwpn_nodename  = $2;
        $initiator_name =~ /^([^\.]+)/;
        $initiator_name = $1;
        $initiator_hash->{$initiator_name} = $wwpn_nodename;
    }
    return ($initiator_hash);
}


sub _get_initiator_nodename {
    my $self           = shift;
    my $initiator_type = shift;
    my $initiator      = shift;
    my $clientinfo     = shift;
    my (%opt) = @_;
    my $wwpn_nodename;
    my $initiator_name;
    my $cli_info;

    if ( ref($initiator) ) {
        $cli_info       = $initiator;
        $initiator_name = $cli_info->{hostname};
    } else {
        $initiator_name = $initiator;
        $cli_info       = $clientinfo->{$initiator_name};
    }
    if ( $initiator_type eq 'iscsi' ) {
        if ( $opt{setup_iscsi} ne "0" ) {
            $self->setup_iscsi();
        }
        $clientinfo->_initialize_iscsi( $initiator_name, $self );
        $wwpn_nodename =
          $clientinfo->_get_iscsi_initiator_nodename($initiator_name);
    } elsif ( $initiator_type eq 'fcp' ) {
        if ( $opt{setup_fcp} ne "0" ) {
            $self->setup_fcp();
        }
        $clientinfo->_initialize_fcp( $initiator_name, $self );
        my @wwpn_list =
          $clientinfo->_get_fcp_initiators( $initiator_name, $self );
        $wwpn_nodename = $wwpn_list[0];
    }
    return ($wwpn_nodename);
}

sub _get_igroup_initiator_info {
    my $self       = shift;
    my $igroup     = shift;
    my $clientinfo = shift;
    my (%opt) = @_;
    my $igrpinfo         = $self->{cfginfo}{igroups};
    my $existing_igroups = $igrpinfo->{existing};
    my ( $ig_name, $ig_hash );
    if ( !ref($igroup) ) {
        $ig_name = $igroup;
        $ig_hash = $existing_igroups->{$igroup};
    } elsif ( ref($igroup) eq "HASH" ) {
        $ig_hash = $igroup;
        $ig_name = $ig_hash->{name};
    }
    my $ig_type = $ig_hash->{proto};

    if ( $ig_type eq 'iscsi' && $opt{setup_iscsi} ne "0" ) {
        $self->setup_iscsi();
    } elsif ( $ig_type eq 'fcp' && $opt{setup_fcp} ne "0" ) {
        $self->setup_fcp();
    }
    my $initiator_hash = $ig_hash->{initiators};
    foreach my $client ( keys( %{ $clientinfo->{clients} } ) ) {
        my $wwpn = $self->_get_initiator_nodename(
            $ig_type, $client, $clientinfo,
            setup_fcp   => 0,
            setup_iscsi => 0
        );
        $initiator_hash->{$wwpn} = $client;
    }
    foreach my $wwpn ( keys(%$initiator_hash) ) {
        if ( !$initiator_hash->{$wwpn} ) {
            logresult( 'WARN',
"_get_igroup_initiator_info: Could not get client in igroup $ig_name for WWPN \"$wwpn\""
            );
            logresult( 'WARN',
"_get_igroup_initiator_info: removing unknown nodename in igroup $ig_name for WWPN \"$wwpn\""
            );

            # blocks_filer_2::igroup_remove()
            my ( $text, $result ) = igroup_remove(
                CONN        => $self->{fc},
                IGROUP_NAME => $ig_name,
                NODES       => [$wwpn],
                FORCE       => 'true'
            );
            delete( $initiator_hash->{$wwpn} );
        }
    }
}


sub add_matching_clients_to_igroup {
    my $self       = shift;
    my $igroup     = shift;
    my $clientinfo = shift;
    my (%opt) = @_;
    my $igrpinfo         = $self->{cfginfo}{igroups};
    my $existing_igroups = $igrpinfo->{existing};
    my ( $ig_name, $ig_hash, $ig_type );
    if ( !ref($igroup) ) {
        $ig_name = $igroup;
        $ig_hash = $existing_igroups->{$igroup};
    } elsif ( ref($igroup) eq "HASH" ) {
        $ig_hash = $igroup;
        $ig_name = $ig_hash->{name};
    }
    $ig_type = $ig_hash->{proto};
    while ( my ( $cl_name, $cl_hash ) = each( %{ $clientinfo->{clients} } ) ) {
        next unless ( $cl_hash->{type}{$ig_type} );
        next if ( $ig_hash->{initiators}{$cl_name} );
        $self->add_igroup_client( $ig_name, $cl_hash, $clientinfo, %opt );
    }
}

sub add_igroup_client {
    my $self       = shift;
    my $igroup     = shift;
    my $client     = shift;
    my $clientinfo = shift;
    my (%opt) = @_;
    my $igrpinfo      = $self->{cfginfo}{igroups};
    my $existing      = $igrpinfo->{existing};
    my $spec_existing = $igrpinfo->{fcp}{existing};
    my ( $ig_name, $ig_hash, $ig_type );

    if ( !ref($igroup) ) {
        $ig_name = $igroup;
        $ig_hash = $existing->{$igroup};
        $ig_type = $ig_hash->{proto};
    } elsif ( ref($igroup) eq "HASH" ) {
        $ig_hash = $igroup;
        $ig_name = $ig_hash->{name};
    }
    my $ig_type = $ig_hash->{proto};
    my ( $cl_name, $cl_hash );
    if ( !ref($client) ) {
        $cl_name = $client;
        $cl_hash = $clientinfo->{clients}{$cl_name};
        if ( !$cl_hash ) {
            $self->{error} =
              "_add_igroup_client: Could not find client: $cl_name";
logresult('WARN', $self->{error});
            return ( $self->{error} );
        }
    } else {
        $cl_hash = $client;
        $cl_name = $cl_hash->{hostname};
    }
    if ( !$cl_hash->{type}{$ig_type} ) {
        $self->{error} =
"add_igroup_client: protocol type \'$ig_type\' not valid for client: $cl_name";
logresult('WARN', $self->{error});
        return ( $self->{error} );
    }
    my $hosttype = $cl_hash->{hosttype};
    if ( $hosttype !~ /(solaris|linux|windows)/i ) {
    }
    my $ostype        = $1;
    my $wwpn_nodename =
      $self->_get_initiator_nodename( $ig_type, $cl_name, $clientinfo, %opt );
    if ( !$wwpn_nodename ) {
        $self->{error} =
"_add_igroup_client: No $ig_type initiator name defined for client $cl_name";
logresult('WARN', $self->{error});
        return ( $self->{error} );
    }
    if ( $ig_hash->{initiators}{$cl_name} ) {
        return;

# $self->{error} = "_add_igroup_client: Node $cl_name is already a member of igroup $ig_name";
# logresult('WARN', $self->{error});
# return( $self->{error} );
    }

    # blocks_filer::filer_igroup_node_add
    # blocks_filer_2::igroup_add()
    my ( $text, $result ) = igroup_add(
        CONN        => $self->{fc},
        IGROUP_NAME => $igroup,
        NODES       => [$wwpn_nodename]
    );

    $cl_hash->{igroups} = {} unless ( $cl_hash->{igroups} );
    $cl_hash->{igroups}{$ig_type} = {} unless ( $cl_hash->{igroups}{$ig_type} );
    $cl_hash->{igroups}{$ig_type}{ $self->{fname} } = $ig_name;
    $ig_hash->{initiators} = {} unless ( $ig_hash->{initiators} );
    $ig_hash->{initiators}{$cl_name} = $wwpn_nodename;

    return (1);
}

sub remove_igroup_client {
    my $self       = shift;
    my $igroup     = shift;
    my $client     = shift;
    my $clientinfo = shift;
    my (%opt) = @_;
    my $igrpinfo      = $self->{cfginfo}{igroups};
    my $existing      = $igrpinfo->{existing};
    my $spec_existing = $igrpinfo->{fcp}{existing};
    my ( $ig_name, $ig_hash, $ig_type );

    if ( !ref($igroup) ) {
        $ig_name = $igroup;
        $ig_hash = $existing->{$igroup};
        $ig_type = $ig_hash->{proto};
    } elsif ( ref($igroup) eq "HASH" ) {
        $ig_hash = $igroup;
        $ig_name = $ig_hash->{name};
    }
    my $ig_type = $ig_hash->{proto};
    my ( $cl_name, $cl_hash );
    if ( !ref($client) ) {
        $cl_name = $client;
        $cl_hash = $clientinfo->{clients}{$cl_name};
        if ( !$cl_hash ) {
            $self->{error} =
              "_remove_igroup_client: Could not find client: $cl_name";
logresult('WARN', $self->{error});
            return ( $self->{error} );
        }
    } else {
        $cl_hash = $client;
        $cl_name = $cl_hash->{hostname};
    }
    if ( !$cl_hash->{type}{$ig_type} ) {
        $self->{error} =
"remove_igroup_client: protocol type \'$ig_type\' not valid for client: $cl_name";
logresult('WARN', $self->{error});
        return ( $self->{error} );
    }
    my $hosttype = $cl_hash->{hosttype};
    if ( $hosttype !~ /(solaris|linux|windows)/i ) {
    }
    my $ostype       = $1;
    my @client_list  = ();
    my $client_found = 0;
    while ( my ( $wwpn, $init_client ) = keys( %{ $ig_hash->{initiators} } ) ) {

        if ( $cl_name eq $init_client ) {
            $client_found = 1;
        }

        # blocks_filer::filer_igroup_node_remove
        # blocks_filer_2::igroup_remove()
        my ( $text, $result ) = igroup_remove(
            CONN        => $self->{fc},
            IGROUP_NAME => $igroup,
            NODES       => [$wwpn],
            FORCE       => 1
        );
    }
    if ( !$client_found ) {
        $self->{error} =
"_remove_igroup_client: Node $cl_name is not a member of igroup $ig_name";
logresult('WARN', $self->{error});
        return ( $self->{error} );
    }
    if ( $cl_hash->{igroups}{$ig_type}{$ig_name} ) {
        delete( $cl_hash->{igroups}{$ig_type}{$ig_name} );
    }
    return (1);
}

sub remove_igroup_clients {
    my $self       = shift;
    my $igroup     = shift;
    my $clientinfo = shift;
    my (%opt) = @_;
    my $igrpinfo      = $self->{cfginfo}{igroups};
    my $existing      = $igrpinfo->{existing};
    my $spec_existing = $igrpinfo->{fcp}{existing};
    my ( $ig_name, $ig_hash, $ig_type );

    if ( !ref($igroup) ) {
        $ig_name = $igroup;
        $ig_hash = $existing->{$igroup};
        $ig_type = $ig_hash->{proto};
    } elsif ( ref($igroup) eq "HASH" ) {
        $ig_hash = $igroup;
        $ig_name = $ig_hash->{name};
    }
    my @initiators = keys( %{ $igrpinfo->{initiators} } );
    foreach my $client (@initiators) {
        $self->remove_igroup_client( $igroup, $client, $clientinfo );
    }
}

sub create_fcp_igroup {
    my $self = shift;
    my (%opts) = @_;
    return $self->create_igroup( "fcp", %opts );
}

sub create_iscsi_igroup {
    my $self = shift;
    my (%opts) = @_;
    return $self->create_igroup( "iscsi", %opts );
}

sub create_igroup {
    my $self   = shift;
    my $igproto = shift;
    my (%opts) = @_;
    my $ig_name = $opts{name};
    $ig_name = "$igproto" . time() unless ($ig_name);
    $ig_name =~ s/\..*//;
    my $igrpinfo      = $self->{cfginfo}{igroups};
    my $spec_igrpinfo = $igrpinfo->{$igproto};
    my $existing      = $igrpinfo->{existing};
    my $spec_existing = $spec_igrpinfo->{existing};
    my $new_igroups = {};
    if ( $existing->{$ig_name} ) {
        $self->{error} = "create_igroup: igroup \"$ig_name\" already exists";
logresult('WARN', $self->{error});
        return ( 0 );
    }
    $igproto = 'fcp' if ( $igproto eq 'fc' );
    if ( $igproto !~ /(fcp|iscsi)/ ) {
        $self->{error} = "create_igroup: unknown igroup type: $igproto";
logresult('WARN', $self->{error});
        return ( 0 );
    }
    my $igrp_hash = {};
    $igrp_hash->{igroup} = $ig_name;
    $igrp_hash->{name}   = $ig_name;
    $igrp_hash->{proto}  = $igproto;

    # blocks_filer_2::igroup_create
    my ( $text, $result ) = igroup_create(
        CONN        => $self->{fc},
        IGROUP_NAME => $ig_name,
        TYPE        => $igproto
    );

    $new_igroups->{$ig_name}      = $igrp_hash;
    $existing->{$ig_name}         = $igrp_hash;
    $spec_existing->{$ig_name}    = $igrp_hash;
    $igrpinfo->{count}++;
    $spec_igrpinfo->{count}++;
    return ($new_igroups);
}


##################################################
#
#        End IGroup section
#
##################################################

###################################################################################################

##################################################
#
#        Start user/group section
#
##################################################

sub setup_groups {
    my $self = shift;
    my (%opts) = @_;
    $self->{cfginfo}{groups} = {} unless ($self->{cfginfo}{groups});
    my $grpinfo = $self->{cfginfo}{groups};
    my $needed = ( exists $opts{count} ) ? $opts{count} : $grpinfo->{count};
    my $force = ( exists $opts{force} ) ? $opts{force} : 0;
    return if (!$needed && !$force);


    if ( !defined( $grpinfo->{existing} ) || $force ) {
        $grpinfo->{existing}        = {};
        $grpinfo->{existing_by_gid} = {};
        $grpinfo->{count}           = 0;
        $grpinfo->{new}             = 0;
        $self->{cfginfo}{groups}    = $grpinfo;
        $self->_get_existing_groups();
    }
    my $existing_groups        = $grpinfo->{existing};
    my $existing_groups_by_gid = $grpinfo->{existing_by_gid};
    my $mode = $self->{mode} || $opts{mode};

    if ( $mode eq $PURGE ) {
        logWarn("setup_groups calling purge_groups");
        $self->_purge_groups();
        return if ( $self->{error} );
    }
    my $num_existing = keys( %{ $grpinfo->{existing} } );
    my $new          = undef;
    $new = $opts{new} if ( !defined($new) && defined( $opts{new} ) );
    $new = $needed - $num_existing if ( !defined($new) && defined($needed) );
    $new = 0  unless ( defined($new) );
    if ( $new > 0 ) {
        $grpinfo->{new} = $new;
        $self->_create_groups(%opts);
    } else {
        $grpinfo->{new} = 0;
    }

logWarn("Exit setup_groups EXISTING_GROUPS:\n\n".Dumper($existing_groups)."\n\nEXISTING_USERS:\n\n".Dumper($grpinfo->{existing}));
    return 1;
}

sub setup_users {
    my $self = shift;
    my (%opts) = @_;

    $self->{cfginfo}{users} = {} unless ($self->{cfginfo}{users});
    my $userinfo = $self->{cfginfo}{users};

    my $needed = ( exists $opts{count} ) ? $opts{count} : $userinfo->{count};
    my $force = ( exists $opts{force} ) ? $opts{force} : 0;
    return if (!$needed && !$force);

    if ( !defined( $userinfo->{existing} ) || $force ) {
        $userinfo->{existing}        = {};
        $userinfo->{existing_by_uid} = {};
        $userinfo->{count}           = 0;
        $userinfo->{new}             = 0;
        $self->{cfginfo}{users}   = $userinfo;
        $self->_get_existing_users();
    }

    $self->setup_groups(force=>1)
         unless ( keys(%{$self->{cfginfo}{groups}{existing}}) );
    my $existing_users         = $userinfo->{existing};
    my $existing_users_by_uid  = $userinfo->{existing_by_uid};
    my $grpinfo                = $self->{cfginfo}{groups};
    my $existing_groups        = $grpinfo->{existing};
    my $existing_groups_by_gid = $grpinfo->{existing_by_gid};
    my $mode                   = $userinfo->{mode} || $opts{mode};

    if ( $mode eq $PURGE ) {
        logWarn("setup_users calling purge_users");
        $self->_purge_users();
        return if ( $self->{error} );
    }
    my $num_existing = keys( %{ $userinfo->{existing} } );
    my $new          = undef;
    $new = $opts{new} if ( !defined($new) && defined( $opts{new} ) );
    $new = $needed - $num_existing if ( !defined($new) && defined($needed) );
    $new = 0 unless ( defined($new) );
logresult('INFO', "setup_users  Needed: $needed   Num existing: $num_existing   New: $new");
    if ( $new > 0 ) {
        $userinfo->{new} = $new;
        $self->_create_users(%opts);
    } else {
        $userinfo->{new} = 0;
    }
    return 1;
}

sub _get_existing_groups {
    my $self                   = shift;
    $self->setup_groups(force=>1)
        unless (defined($self->{cfginfo}{groups}{existing}));
    $self->setup_users(force=>1)
        unless (defined($self->{cfginfo}{users}{existing}));
    my $groupinfo              = $self->{cfginfo}{groups};
#    my $existing_groups        = $groupinfo->{existing};
#    my $existing_groups_by_gid = $groupinfo->{existing_by_gid};
    my $existing_groups        = {};
    my $existing_groups_by_gid = {};
    $groupinfo->{existing} = $existing_groups;
    $groupinfo->{existing_by_gid} = $existing_groups_by_gid;
    my $group_hash             = readGroupFile( $self->{fc} );

    if ($@) {
        $self->{error} = "readGroupFile: $@";
logresult('WARN', $self->{error});
        return 0;
    }
    my $count = 0;
    while ( my ( $gname, $ginfo ) = each(%$group_hash) ) {
#        my $grphash = {};
        my $grphash = { %$ginfo };
        foreach (keys(%$grphash)) {
            delete($grphash->{$_}) unless (defined($grphash->{$_}));
        }
        my $gname = $ginfo->{group};
        my $gid = $ginfo->{gid};
        $grphash->{name} = $gname;
        $existing_groups->{$gname} = $grphash;
        if ( !defined( $grphash->{gid} ) ) {
            $self->{error} = "_get_existing_groups: no gid for group: $gname";
logresult('WARN', $self->{error});
            return 0;
        }
        $existing_groups_by_gid->{$gid} = $grphash;
        $grphash->{users}               = {};
        $grphash->{users_by_uid}        = {};
        $count++;
    }

#    $groupinfo->{existing}        = $existing_groups;
#    $groupinfo->{existing_by_gid} = $existing_groups_by_gid;
    $groupinfo->{count}           = $count;
    return ($count);
}


sub _get_existing_users {
    my $self                   = shift;
    $self->setup_groups(force=>1)
        unless ( keys(%{$self->{cfginfo}{groups}{existing}}) );
    $self->setup_users(force=>1)
        unless (defined($self->{cfginfo}{users}{existing}));
    my $groupinfo              = $self->{cfginfo}{groups};
    my $userinfo               = $self->{cfginfo}{users};
#    my $existing_users         = $userinfo->{existing};
#    my $existing_users_by_uid  = $userinfo->{existing_by_uid};
    my $existing_users        = {};
    my $existing_users_by_uid = {};
    $userinfo->{existing} = $existing_users;
    $userinfo->{existing_by_uid} = $existing_users_by_uid;
    my $existing_groups        = $groupinfo->{existing};
    my $existing_groups_by_gid = $groupinfo->{existing_by_gid};

    my $user_hash = readPasswdFile( $self->{fc} );
    if ($@) {
        $self->{error} = "readPasswdFile: $@";
logresult('WARN', $self->{error});
        return 0;
    }
    my $count = 0;
    while ( my ( $login, $uinfo ) = each(%$user_hash) ) {
#        my $uhash = { };
        my $uhash = { %$uinfo };
        foreach (keys(%$uhash)) {
            delete($uhash->{$_}) unless (defined($uhash->{$_}));
        }
        my $uid   = $uinfo->{uid};
        my $group = $uinfo->{group};
        my $gid   = $uinfo->{gid};
        $uhash->{name} = $group;
        if ( !defined( $uinfo->{uid} ) ) {
            $self->{error} = "_get_existing_users: no uid for user: $login";
logresult('WARN', $self->{error});
            return 0;
        }
        $existing_users_by_uid->{$uid} = $uhash;
        $existing_users->{$login} = $uhash;
        my $grphash;
        if ( $group ) {
            $grphash = $existing_groups->{$group};
            if ( !$grphash ) {
                logWarn("_get_existing_users: no group found for group name: $group");
            }
        } elsif ( $gid ) {
            $grphash = $existing_groups_by_gid->{$gid};
            if ( !$grphash ) {
                logWarn("_get_existing_users: no group found for group gid: $gid");
            }
        } else {
            $self->{error} =
              "_get_existing_users: no group or gid given for user: $login";
logresult('WARN', $self->{error});
#            return 0;
        }
        $uhash->{group}                = $grphash->{group};
        $uhash->{gid}                  = $grphash->{gid};
        $grphash->{users}              = {} unless ( $grphash->{users} );
        $grphash->{users}{$login}       = $uid;
        $grphash->{users_by_uid}       = {} unless ( $grphash->{users_by_uid} );
        $grphash->{users_by_uid}{$uid} = $login;
        $count++;
    }

#    $userinfo->{existing}        = $existing_groups;
#    $userinfo->{existing_by_uid} = $existing_users_by_uid;
    $userinfo->{count}           = $count;
    return ($count);
}

sub _purge_group_list {
    my $self = shift;
    my ($group_list) = @_;
    my $grpinfo                = $self->{cfginfo}{groups};
    my $existing_groups        = $grpinfo->{existing};
    my $existing_groups_by_gid = $grpinfo->{existing_by_gid};
    logcomment("_purge_groups: $self->{fname}");
    if ( !ref($group_list) || ref($group_list) eq "HASH" ) {
        $group_list = [$group_list];
    }
    foreach my $group (@$group_list) {
        ### while (my ($gname, $gname) = each(%$existing))
        my ($grphash, $gname, $gid) = $self->_get_group_ref($group);
        next unless ($grphash);
        delGroupEntry( $self->{fc}, $gname );
        if ($@) {
            $self->{error} = "_purge_groups: Error from delGroupEntry: $@";
logresult('WARN', $self->{error});
            return (0);
        }
        delete( $existing_groups->{$gname} );
        delete( $existing_groups_by_gid->{ $grphash->{gid} } );
        $grpinfo->{count}--;
    }
#    $grpinfo->{existing}        = $existing_groups;
#    $grpinfo->{existing_by_gid} = $existing_groups_by_gid;
    return (1);
}

sub _purge_groups {
    my $self                   = shift;
    my $grpinfo                = $self->{cfginfo}{groups};
    my $existing_groups        = $grpinfo->{existing};
    my $existing_groups_by_gid = $grpinfo->{existing_by_gid};
    logcomment("_purge_groups: $self->{fname}");
    my $group_list = [ keys(%$existing_groups) ];
    return ( $self->_purge_group_list($group_list) );
}

sub _purge_user_list {
    my $self = shift;
    my ($user_list) = @_;
    my $grpinfo                = $self->{cfginfo}{groups};
    my $userinfo                  = $self->{cfginfo}{users};
    my $existing_users         = $userinfo->{existing};
    my $existing_users_by_uid  = $userinfo->{existing_by_uid};
    my $existing_groups        = $grpinfo->{existing};
    my $existing_groups_by_gid = $grpinfo->{existing_by_gid};

    my $userlist_hash = {};
    foreach (@$user_list) {
        $userlist_hash->{$_} = 1;
    }

    logcomment("_purge_users: $self->{fname}");
    if ( !ref($user_list) || ref($user_list) eq "HASH" ) {
        $user_list = [$user_list];
    }
# delPasswdList( $self->{fc}, $userlist_hash);
# logError("Returned from "delPasswdList");

    foreach my $user (@$user_list) {
        ### while (my ($login, $user) = each(%$existing_users))
        my ($uhash, $login, $uid) = $self->_get_user_ref($user);
        next unless ($uhash);
        my $gname = $uhash->{group};
        delPasswdEntry( $self->{fc}, $login );
        if ($@) {
            $self->{error} = "_purge_users: Error from delPasswdEntry: $@";
logresult('WARN', $self->{error});
            return (0);
        }
logWarn("Deleted user \"$login\"");
        delete( $existing_users->{$login} );
        delete( $existing_users_by_uid->{$uid} );
        $userinfo->{count}--;
        my $grphash = $existing_groups->{$gname};
        delete( $grphash->{users}{$login} );
        delete( $grphash->{users_by_uid}{$uid} );
    }
#    $userinfo->{existing}        = $existing_groups;
#    $userinfo->{existing_by_uid} = $existing_users_by_uid;
    return (1);
}

sub _purge_users {
    my $self                   = shift;
    my $grpinfo                = $self->{cfginfo}{groups};
    my $userinfo                  = $self->{cfginfo}{users};
    my $existing_users         = $userinfo->{existing};
    my $existing_users_by_uid  = $userinfo->{existing_by_uid};
    my $existing_groups        = $grpinfo->{existing};
    my $existing_groups_by_gid = $grpinfo->{existing_by_gid};

    logcomment("_purge_users: $self->{fname}");
    my $user_list = [ keys(%$existing_users) ];
    ### while (my ($login, $user) = each(%$existing_users))
    return ( $self->_purge_user_list($user_list) );
}


sub _create_users_for_group {
    my $self = shift;
    my $group = shift;
    my $count = shift;
    my (%opts) = @_;
    $self->setup_groups(force=>1)
        unless ( keys(%{$self->{cfginfo}{groups}{existing}}) );
    $self->setup_users(force=>1)
        unless ( keys(%{$self->{cfginfo}{users}{existing}}) );

    my $grpinfo                = $self->{cfginfo}{groups};
    my $userinfo               = $self->{cfginfo}{users};
    my $existing_users         = $userinfo->{existing};
    my $existing_users_by_uid  = $userinfo->{existing_by_uid};
    my $existing_groups        = $grpinfo->{existing};
    my $existing_groups_by_gid = $grpinfo->{existing_by_gid};
    my $min_uid = ( exists $opts{min_uid} ) ? $opts{min_uid} : $userinfo->{min_uid};
    $min_uid = 10001 unless ($min_uid);
    my $prefix = $opts{prefix} || $grpinfo->{prefix} || 'user_';
    $prefix .= "_" unless ($prefix =~ /_$/);
    my $error;

    my ($grphash, $gname, $gid) = $self->_get_group_ref($group);
    if (!defined($grphash)) {
        $error = "_create_users_for_group: no such group: $gname";
        logWarn($error);
logresult('FATAL', $error);
        $self->{error} = $error;
        return 0;
    }
    if ( !$gname ) {
        $error = "_create_users_for_group: no group given";
        logWarn($error);
logresult('FATAL', $error);
        $self->{error} = $error;
        return 0;
    } 

    my $time = time();
    logcomment("Creating $count users for group $gname on $self->{fname}");
    my $uid_cntr = 0;
    my $new_users = {};
    for ( my $cntr = 0 ; $cntr < $count ; $cntr++, $uid_cntr++ ) {
#        my $login      = "${prefix}_${time}_${uid_cntr}";
#        my ($olduser) = $self->_get_user_ref($login);
#        my %users_hash =
#             createUsers( $self->{fc}, 1, login => $login, group => $gname, min_uid => $min_uid );
        my %users_hash =
          createUsers( $self->{fc}, 1, login_prefix=>$prefix, group=>$gname, min_uid=>$min_uid );
        my ($login) = keys(%users_hash);
        my ($user) = values(%users_hash);
        if ($@) {
            $self->{error} =
"_create_users_for_group: Error from createUsers grp=$gname: $@";
logresult('WARN', $self->{error});
            return (0);
        }
        if ( !$user ) {
            $self->{error} =
"_create_users_for_group: Error from createUsers grp=$gname -- no user created";
logresult('WARN', $self->{error});
            return (0);
        }
#        my $uhash = {}; 
        for (keys(%$user)) {
            delete($user->{$_}) unless (defined($user->{$_}));
        }
        my $uhash = { %$user };
        my $uid   = $user->{uid};
        logcomment("$self->{fname}:created user $login in group $gname");
        $new_users->{$login}           = $uhash;
        $existing_users->{$login}      = $uhash;
        $existing_users_by_uid->{$uid} = $uhash;
        $uhash->{group}                = $gname;
        $userinfo->{count}++;
        $grphash->{users}        = {} unless ( $grphash->{users} );
        $grphash->{users_by_uid} = {} unless ( $grphash->{users_by_uid} );
        $grphash->{users}{$login}       = $uid;
        $grphash->{users_by_uid}{$uid}  = $login;
    }
    return ( $new_users );
logresult('INFO', "Exit create_users_for_group   NEW USERS: \n\n".Dumper($new_users));
}


sub _create_groups {
    my $self = shift;
    my (%opts) = @_;
    $self->setup_users(force=>1)
        unless (defined($self->{cfginfo}{groups}{existing}));
    my $grpinfo                = $self->{cfginfo}{groups};
    my $existing_groups        = $grpinfo->{existing};
    my $existing_groups_by_gid = $grpinfo->{existing_by_gid};
    my $needed = ( exists $opts{new} ) ? $opts{new} : $grpinfo->{new};
    return unless ($needed);
    my $min_gid = ( exists $opts{min_gid} ) ? $opts{min_gid} : $grpinfo->{min_gid};
    $min_gid = 1001 unless ($min_gid);
    my $prefix = $opts{prefix} || $grpinfo->{prefix} || 'group_';
    $prefix .= "_" unless ($prefix =~ /_$/);
    my $time = time();
    my $users_per_group = (defined($opts{users_per_group})) ? $opts{users_per_group}
                              : $grpinfo->{users_per_group};
    my $group_only = defined($opts{group_only}) ? $opts{group_only} : 1;

    logcomment("Creating $needed groups on $self->{fname}");

    my $new_groups = {};
    for ( my $cntr = 0 ; $cntr < $needed ; $cntr++ ) {
#        my $gname = "${prefix}_${time}_${cntr}";
#        my ($oldgrp) = $self->_get_group_ref($gname);
#        my %groups_hash = createGroups( $self->{fc}, 1, group => $gname, min_gid => $min_gid);
        my %groups_hash = createGroups( $self->{fc}, 1, group_prefix => $prefix, min_gid => $min_gid);
        if ($@) {
            $self->{error} =
              "_create_groups: Error from createGroups: $@";
logresult('WARN', $self->{error});
            return (0);
        }
        my ($gname) = keys(%groups_hash);
        my ($group) = values(%groups_hash);
        if ( !$group ) {
            $self->{error} =
              "_create_groups: Error from createGroups -- no group created";
logresult('WARN', $self->{error});
            return (0);
        }
#        my $ghash = {};
        my $ghash = { %$group };
        my $gid   = $group->{gid};
        logcomment("$self->{fname}:created group $gname");
        $new_groups->{$gname}            = $ghash;
        $existing_groups->{$gname}       = $ghash;
        $existing_groups_by_gid->{$gid}  = $ghash;
        $grpinfo->{count}++;
        if ($users_per_group && !$group_only) {
            $opts{users_per_group} = $users_per_group;
            delete($opts{new});
            while ( my($gname,$ghash) = each(%$new_groups) ) {
                my $group_users = $ghash->{users};
                my $num_group_users = scalar(keys(%$group_users));
                my $needed = $users_per_group - $num_group_users;
                $self->_create_users_for_group($gname, $needed, %opts);
            }
        }
    }

    $grpinfo->{new} = 0;
    return ($new_groups);
}


sub _create_users {
    my $self = shift;
    my (%opts) = @_;
    my $groupinfo              = $self->{cfginfo}{groups};
    my $userinfo               = $self->{cfginfo}{users};
    my $existing_users         = $userinfo->{existing};
    my $existing_users_by_uid  = $userinfo->{existing_by_uid};
    my $existing_groups        = $groupinfo->{existing};
    my $existing_groups_by_gid = $groupinfo->{existing_by_gid};
    my $users_per_group = $opts{users_per_group} || $groupinfo->{users_per_group};

    my $needed = $opts{new} || $userinfo->{new};
    my $new_users = {};
    if ($users_per_group) {
        foreach my $grphash (values(%$existing_groups)) {
            my $group_existing_users = $grphash->{users};
            my $num_group_existing_users = scalar(keys(%$group_existing_users));
            my $group_needed = $users_per_group - $num_group_existing_users;
            my $newx = $self->_create_users_for_group($grphash, $group_needed, %opts);
            return(0) if ($self->{error});
            $new_users = { %$new_users, %$newx };
        }
    } elsif ($needed) {
        my $new_user_count = $userinfo->{count} + $needed;
        my @grp_names = keys(%$existing_groups);
    
        my $ngrps = scalar(@grp_names);
        if ( $ngrps == 0 ) {
            $self->{error} = "create_users: no groups exist on $self->{fname}";
    logresult('WARN', $self->{error});
            return (0);
        }
    #    my $users_per_grp = int( $needed / $ngrps );
    #    my $rem           = $needed % $ngrps;
    #    foreach my $grp (@grp_names) {
    #        my $newx = $self->_create_users_for_group( $grp, $users_per_grp, %opts );
    #        return (0) if ( $self->{error} );
    #    }
    
        for (my $i=0; $userinfo->{count} < $new_user_count; $i = ($i + 1) % $ngrps) {
            my $grp = $grp_names[$i];
            my $newx = $self->_create_users_for_group($grp, 1, %opts);
            return(0) if ($self->{error});
            $new_users = { %$new_users, %$newx };
        }
        $userinfo->{new} = 0;
    }
    return ($new_users);
}


my $_saved_user_uid_;
my $_saved_user_gid_;

sub RestoreUser {
    my $self = shift;
    return unless ($_saved_user_uid_);
    $> = $_saved_user_uid_;
    $) = $_saved_user_gid_;
    $_saved_user_uid_ = undef;
    $_saved_user_gid_ = undef;
    return;
}

sub SetUser {
    my $self = shift;
    my $user = shift;
    my ($uinfo, $ulogin, $uid) = $self->_get_user_ref($user);
    my $gid;
    $_saved_user_uid_ = $>;
    $_saved_user_gid_ = $);
    $> = $uid;
    $) = $gid;
}

sub _get_user_ref {
    my $self = shift;
    my $user = shift;
    my $userinfo               = $self->{cfginfo}{users};
    my $existing_users         = $userinfo->{existing};
    my $existing_users_by_uid  = $userinfo->{existing_by_uid};
    my $ulogin = undef;
    my $uinfo = undef;
    my $uid = undef;
    if (ref($user)) {
        $uinfo = $user;
        $ulogin = $uinfo->{login};
        $uid = $uinfo->{uid};
    } elsif ($user =~ /^\d+/) {
        if (exists($existing_users_by_uid->{$user})) {
            $uid = $user;
            $uinfo = $existing_users_by_uid->{$uid};
            $ulogin = $uinfo->{login};
        }
    } else {
        if (exists($existing_users->{$user})) {
            $ulogin = $user;
            $uinfo = $existing_users->{$ulogin};
            $uid = $uinfo->{uid};
        }
    }
    return ($uinfo, $ulogin, $uid);
}

sub _get_group_ref {
    my $self = shift;
    my $group = shift;
    my $groupinfo               = $self->{cfginfo}{groups};
    my $existing_groups         = $groupinfo->{existing};
    my $existing_groups_by_gid  = $groupinfo->{existing_by_gid};
    my $gname = undef;
    my $ginfo = undef;
    my $gid = undef;

    if (ref($group)) {
        $ginfo = $group;
        $gname = $ginfo->{group};
        $gid = $ginfo->{gid};
    } elsif ($group =~ /^\d+/) {
        if (exists($existing_groups_by_gid->{$group})) {
            $gid = $group;
            $ginfo = $existing_groups_by_gid->{$gid};
            $gname = $ginfo->{group};
        } else {
logWarn("get_group_ref:  No such group:  group=\"$group\"");
        }
    } else {
        if (exists($existing_groups->{$group})) {
            $gname = $group;
            $ginfo = $existing_groups->{$gname};
            $gid = $ginfo->{gid};
        } else {
logWarn("get_group_ref:  No such group:  gid=\"$group\"");
        }
    }
    return ($ginfo, $gname, $gid);
}




##################################################
#
#        End user/group section
#
##################################################

###################################################################################################

##################################################
#
#        Start Quota section
#
##################################################


sub enable_default_volume_quotas {
    my $self = shift;
    my $existing_volumes = $self->_get_existing_volumes();
    my @quotas = ();
    foreach my $vol (keys(%$existing_volumes)) {
        push( @quotas,  { target =>"*", type => "user\@/vol/$vol" } );
        push( @quotas,  { target =>"*", type => "group\@/vol/$vol" } );
        push( @quotas,  { target =>"*", type => "tree\@/vol/$vol" } );
    }
    setQuotaList( $self->{fc}, \@quotas );
    foreach my $vol (keys(%$existing_volumes)) {
        enableQuota($self->{fc}, $vol, nowait=>1, dontcheck=>1);
    }
}

sub disable_all_quotas {
    my $self = shift;
    my (%opts) = (@_);
    my $existing_volumes = $self->_get_existing_volumes();
    foreach my $vol (keys(%$existing_volumes)) {
        eval { disableQuota($self->{fc}, $vol, %opts) };
    }
}

sub enable_all_quotas {
    my $self = shift;
    my (%opts) = (@_);
    my $existing_volumes = $self->_get_existing_volumes();
    foreach my $vol (keys(%$existing_volumes)) {
        eval { enableQuota($self->{fc}, $vol, %opts) };
    }
}

sub disable_quotas_for_vol {
    my $self = shift;
    my $vol = shift;
    my (%opts) = (@_);
    my ($volname, $volhash, $volpath) = $self->_get_vol_ref($vol);
    eval { disableQuota($self->{fc}, $volname, %opts) };
}

sub enable_quotas_for_vol {
    my $self = shift;
    my $vol = shift;
    my (%opts) = (@_);
    my ($volname, $volhash, $volpath) = $self->_get_vol_ref($vol);
    eval { enableQuota($self->{fc}, $volname, %opts) };
}

sub delete_all_quotas {
    my $self = shift;
    writeQuotaFIle($self->{fc});
}


sub setup_quotas {
    my $self = shift;
    my (%opts) = @_;

logresult('INFO', "Enter setup_quotas  OPTS: \n\n".Dumper(\%opts));

    $self->{cfginfo}{quotas} = {} unless ($self->{cfginfo}{quotas});
    $self->{cfginfo}{quotas} = {} unless ( ref($self->{cfginfo}{quotas}) eq 'HASH' );
    my $qqinfo = $self->{cfginfo}{quotas};
    my $users_per_qtree = $opts{users_per_qtree} || $qqinfo->{users_per_qtree};
    my $groups_per_qtree = $opts{groups_per_qtree} || $qqinfo->{groups_per_qtree};
    my $users_per_group = $opts{users_per_group} || $qqinfo->{users_per_group};

    my $qtreeinfo = $self->{cfginfo}{qtrees};
    $self->setup_qtrees(force=>1)
      unless ( keys(%{$qtreeinfo->{existing}}) );

    my $groupinfo = $self->{cfginfo}{groups};
    $self->setup_groups(force=>1)
      unless ( keys(%{$groupinfo->{existing}}) );

    my $userinfo = $self->{cfginfo}{users};
    $self->setup_users(force=>1 )
      unless ( keys(%{$userinfo->{existing}}) );

    $self->disable_all_quotas();
    my $force = ( exists $opts{force} ) ? $opts{force} : 0;
    if ( !defined {$qqinfo} || !defined( $qqinfo->{existing} ) || $force ) {
        $qqinfo->{existing}       = {};
        $qqinfo->{count}          = 0;
        $qqinfo->{new}            = 0;
        $self->{cfginfo}{quotas} = $qqinfo;
        $self->_get_existing_quotas();
    }
    my $mode = $self->{mode} || $opts{mode};

    if ( $mode eq $PURGE ) {
logresult('INFO', "setup_quotas  Calling _purge_quotas");
        $self->_purge_quotas();
        return if ( $self->{error} );
    }
    if ($users_per_qtree) {
        $self->_create_quotas_for_users_per_qtree(%opts);
    } elsif ($groups_per_qtree) {
        $self->_create_quotas_for_groups_per_qtree(%opts);
    }
#logresult('INFO', "Exit setup_quotas  CURRENT:\n\n".Dumper($qqinfo->{existing}));

    $self->enable_all_quotas(reenable=>1);
    return 1;
}


sub _get_existing_quotas {
    my $self             = shift;
    my $qqinfo            = $self->{cfginfo}{quotas};
    my $userinfo         = $self->{cfginfo}{users};
    my $groupinfo        = $self->{cfginfo}{groups};
    my $existing_users   = $userinfo->{existing};
    my $existing_groups  = $groupinfo->{existing};
    my $existing_quotas  = $qqinfo->{existing};
    my $existing_qtrees  = $self->{containers}{$QTREES};
    my $error;

    my $quota_list = readQuotaFile( $self->{fc} );
    if ($@) {
        $self->{error} = "_get_existing_quotas: Error from readQuotaFile: $@";
logresult('FATAL', $self->{error});
        return 0;
    }
    my $count = 0;

    foreach my $quota (@$quota_list) {
        my $qqhash = {%$quota};

        $qqhash = $self->_fillout_quota_info($qqhash);
        my $existing;
        if ( !$qqhash->{type} ) {
            $self->{error} = "_get_existing_quotas: quota 'type' is undefined";
logresult('FATAL', $self->{error});
            return 0;
        }
        if ( !$qqhash->{target} ) {
            $self->{error} =
              "_get_existing_quotas: quota 'target' is undefined";
logresult('FATAL', $self->{error});
            return 0;
        }

        foreach (keys(%$qqhash)) {
            delete($qqhash->{$_}) unless (defined($qqhash->{$_}));
        }
        my $type      = $qqhash->{type};
        my $basetype  = $qqhash->{basetype};
        my $target    = $qqhash->{target};
        my $quota_key = $qqhash->{qkey};
        $existing->{$quota_key} = $qqhash;
        my $default = $qqhash->{default};
        my $volpath    = $qqhash->{volpath};
        my $vol        = $qqhash->{volname};
        my $qtname     = $qqhash->{treename};
        my $qtpath     = $qqhash->{treepath};
        my $path       = $qtpath || $volpath;
        my $qthash;
        my $volhash;
        my $userhash;
        my $grphash;

        $qqhash->{volume}  = $vol;
        $qqhash->{qtpath}  = $qtpath;
        $qqhash->{volpath} = $volpath;
        $qqhash->{qtree}   = $qtname;
        $qqhash->{qkey}     = $quota_key;

        if ( !$qqhash->{volname} ) {
            $self->{error} =
              "_get_existing_quotas: No volume defined for quota:\n\n".Dumper($quota);
logresult('FATAL', $self->{error});
            return 0;
        }

        ($vol, $volhash) = $self->_get_vol_ref($vol);
        if ( !$volhash ) {
            logWarn("_get_existing_quotas: no existing volume: $vol");
            next;
        }
        $volhash->{quotas} = {} unless ( $volhash->{quotas} );
        $volhash->{quotas}{existing} = {} unless ( $volhash->{quotas}{existing} );
        $volhash->{quotas}{existing}{$quota_key} = $qqhash;
        $volhash->{quotas}{$basetype} = {} unless ( $volhash->{quotas}{$basetype} );
        $volhash->{quotas}{$basetype}{$target} = $qqhash;
        $existing_quotas->{$quota_key} = $qqhash;
        if ($qtname) {
            if ( !$qtpath ) {
                $self->{error} =
                  "_get_existing_quotas: quota 'treepath' is undefined";
                return 0;
            }
            ($qthash, $qtname, $qtpath) = $self->_get_qtree_ref($qtname, vol=>$vol);
            if ( !$qthash ) {
                logWarn("_get_existing_quotas: no existing qtree: $qtpath");
logresult('FATAL', "get_existing_quotas  LINE: ".__LINE__);
                next;
            }
            $qthash->{quotas} = {} unless ( $qthash->{quotas} );
            $qthash->{quotas}{existing} = {} unless ( $qthash->{quotas}{existing} );
            $qthash->{quotas}{existing}{$quota_key} = $qqhash;
            $qthash->{quotas}{$basetype} = {} unless ( $qthash->{quotas}{$basetype} );
            $qthash->{quotas}{$basetype}{$target} = $qqhash;
        }

        if ($basetype ne 'tree' && $basetype ne 'user' && $basetype ne 'group') {
            $self->{error} =
              "_get_existing_quotas: unknown basetype: $basetype";
logresult('FATAL', $self->{error});
            return 0;
        }
        if ( $basetype =~ /group/ && !$default ) {
            my $groupname = $qqhash->{groupname};
            my $grphash = $existing_groups->{$groupname};
            if ( !$grphash ) {
               logWarn("_get_existing_quotas: no existing group: $groupname");
logresult('FATAL', "get_existing_quotas  LINE: ".__LINE__);
               next;
            }
            $grphash->{quotas} = {} unless ( $grphash->{quotas} );
            $grphash->{quotas}{$path} = $qqhash;
        } elsif ( $basetype =~ /user/ && !$default ) {
            my $username = $qqhash->{username};
            my $uhash = $existing_users->{$username};
            if ( !$uhash ) {
                logWarn("_get_existing_quotas: no existing user: \"$username\"");
logresult('FATAL', "get_existing_quotas  LINE: ".__LINE__);
                next;
            }
            $uhash->{quotas} = {} unless ( $uhash->{quotas} );
            $uhash->{quotas}{$path} = $qqhash;

            #  Set 'uid', 'gid'  fields of qtree for use during quota tests.
            $qthash->{uid} = $uhash->{uid};
            $qthash->{gid} = $uhash->{gid};

        }
        $count++;
    }

    $qqinfo->{existing} = $existing_quotas;
    $qqinfo->{count}    = $count;
    return ($count);
}


sub _delete_quotas_for_vol {
    my $self = shift;
    my (%opts) = (@_);
    my $vol = shift;
    my ($volname, $volhash, $volpath) = $self->_get_vol_ref($vol);
    eval { removeQuotaEntry( $self->{fc}, $volname, %opts) };
}


sub _delete_quota {
     my $self  = shift;
     my $quota = shift;
     my $qqinfo = $self->{cfginfo}{quotas};
     my $existing_quotas = $qqinfo->{existing};
     my ($qqhash, $qkey);
     if (ref($quota)) {
        $qqhash = $quota;
        $qkey = $qqhash->{qkey};
     } else {
        $qkey = $quota;
        $qqhash = $existing_quotas->{$qkey};
     }
     if (!$qkey || !$qqhash) {
     }
     my $qqhash     = $existing_quotas->{$qkey};
     my $target    = $qqhash->{target};
     my $type      = $qqhash->{type};
     my $basetype  = $qqhash->{basetype};
     my $quota_key = "$type:$target";
     deleteQuota( $self->{fc}, target => $target, type => $type );
     if ($@) {
         $self->{error} = "_purge_quotas: Error from deleteQuota: $@";
         return (0);
     }
     delete( $existing_quotas->{$quota_key} );
     $qqinfo->{count}--;

     my $treepath = $qqhash->{treepath};
     my $volpath  = $qqhash->{volpath};
     my $volname  = $qqhash->{volname};
     my $qtname   = $qqhash->{treename};
     my $qkey     = $qqhash->{treepath};
     my $path     = $treepath || $volpath;
     my $volhash;
     my $qthash;
     my $volpath;
     my $qtpath;

     my $volhash;
     ($volname, $volhash) = $self->_get_vol_ref($volname);
     if ( !$volhash ) {
         logWarn("_purge_quotas: no existing volume: $volname");
     }
     $volhash->{quotas} = {} unless ( $volhash->{quotas} );
     $volhash->{quotas}{existing} = {} unless ( $volhash->{quotas}{existing} );
     delete($volhash->{quotas}{existing}{$quota_key});
     $volhash->{quotas}{$basetype} = {} unless ( $volhash->{quotas}{$basetype} );
     delete($volhash->{quotas}{$basetype}{$quota_key});
     if ( $qqhash->{treename} ) {
         if ( !$treepath ) {
             $self->{error} =
               "_purge_quotas: quota 'treepath' is undefined";
             return 0;
         }
         ($qthash, $qtname, $qtpath) = $self->_get_qtree_ref($qtname, vol=>$volname);
         if ( !$qthash ) {
             logWarn("_purge_quotas: no existing qtree: $qtname");
         }

         $qthash->{quotas} = {} unless ( $qthash->{quotas} );
         $qthash->{quotas}{existing} = {} unless ( $qthash->{quotas}{existing} );
         delete($qthash->{quotas}{existing}{$quota_key});
         $qthash->{quotas}{$basetype} = {} unless ( $qthash->{quotas}{$basetype} );
         delete($qthash->{quotas}{$basetype}{$quota_key});
     }

     if ( $basetype =~ /tree/ ) {
     } elsif ( $basetype =~ /group/ ) {
         my $groupname = $qqhash->{groupname};
         my ($grphash, undef) = $self->_get_group_ref($groupname);
         if ( !$grphash ) {
             logWarn("_purge_quotas: no existing group: $groupname");
         }
         $grphash->{quotas} = {} unless ( $grphash->{quotas} );
         delete( $grphash->{quotas}{$path} );
     } elsif ( $basetype =~ /user/ ) {
         my $username = $qqhash->{username};
         my ($uhash, undef) = $self->_get_user_ref($username);
         if ( !$uhash ) {
             logWarn("_purge_quotas: no existing user: $username");
         }
         $uhash->{quotas} = {} unless ( $uhash->{quotas} );
         delete( $uhash->{quotas}{$path} );
     } else {
         $self->{error} =
           "_purge_quotas: unknown basetype: $basetype";
         return 0;
     }
    $self->enable_quotas_for_volume($volname, reenable=>1);
}


sub _delete_quotas_for_vol {
     my $self  = shift;
     my $vol = shift;
     my ($volname, $volhash) = $self->_get_vol_ref($vol);
     my $vol_quotas = $volhash->{quotas}{existing};
     return unless (!$vol_quotas);
     foreach my $qkey (keys(%$vol_quotas)) {
        $self->_delete_quota($qkey);
    }
}


sub _purge_quotas {
    my $self             = shift;
    my $qqinfo           = $self->{cfginfo}{quotas};
    my $userinfo         = $self->{cfginfo}{users};
    my $groupinfo        = $self->{cfginfo}{groups};
    my $existing_quotas  = $qqinfo->{existing};

    logcomment("_purge_quotas: $self->{fname}");
    foreach my $qkey (keys(%$existing_quotas)) {
        $self->_delete_quota($qkey);
    }
#    $self->enable_all_quotas(reenable=>1);
    return (1);
}


sub _random_disk_quota {
    my $quota_info = $a;
    my $max = $quota_info->{max_disk} || "100m";
    my $min = $quota_info->{min_disk} || "10m";
    my ($min_val, $min_suffix) = ($min =~ /^(\d+)([A-Za-z]?)$/);
    my ($max_val, $max_suffix) = ($max =~ /^(\d+)([A-Za-z]?)$/);
    if ($min_suffix ne $max_suffix) {
        logError("_random_disk_quota  max_disk: \"$max\" and min_disk: \"$min\" have different suffixes");
    }
    my $val = int(rand($max_val - $min_val) + $min_val);
    return ($val . $max_suffix);
logresult('FATAL', "_random_disk_quota");
}

sub _random_files_quota {
    my $quota_info = $a;
    my $max = $quota_info->{max_files} || 100;
    my $min = $quota_info->{min_files} || 10;
    my $val = int(rand($max - $min) + $min);
    return ($val);
logresult('FATAL', "_random_files_quota");
}


sub _fillout_quota_info {
    my $self = shift;
    my $qqhash = shift;
#logresult('INFO', "Enter _fillout_quota_info:\n\n".Dumper($qqhash));
    my $qqinfo = $self->{cfginfo}{quotas};
    my $new_quotas = {};
    my $type = undef;
    my $basetype = undef;
    if (defined($qqhash->{type})) {
        $type = $qqhash->{type};
        $basetype = $type;
        if ($basetype =~ /^(\w+)@/) {
            $basetype = $1;
        }
    } elsif (defined($qqhash->{basetype})) {
        $basetype = $qqhash->{basetype};
    } elsif (defined($qqhash->{user})) {
        $basetype = 'user';
    } elsif (defined($qqhash->{group})) {
        $basetype = 'user';
    } else {
        $basetype = 'tree';
    }
    if ($basetype ne 'tree' && $basetype ne 'user' && $basetype ne 'group') {
         $self->{error} =
           "_fillout_quota_info: unknown basetype: $basetype";
logError($self->{error});
         return 0;
    }
    my $target = undef;
    my $default = undef;
    my $usrhash = undef;
    my $user = undef;
    my $group = undef;
    my $grphash = undef;
    if (defined($qqhash->{target})) {
        $target = $qqhash->{target};
    } elsif (defined($qqhash->{id})) {
        $target = $qqhash->{id};
    } elsif ($qqhash->{id}) {
        $target = $qqhash->{id};
    } elsif (defined($qqhash->{user}) && $basetype eq 'user') {
        $target = $qqhash->{user};
    } elsif (defined($qqhash->{group}) && $basetype eq 'group') {
        $target = $qqhash->{group};
    } elsif ($qqhash->{$default}) {
        $target = "*";
        $default = 1;
    } else {
        $target = "*";
        $default = 1;
    }
    $default = ($target eq "*") ? 1 : 0 unless (defined($default));
    if ($basetype eq 'user' && !$default) {
        ($usrhash, $user) = $self->_get_user_ref($target);
        if ( !$usrhash ) {
            $self->{error} =
               "_fillout_quota_info: no existing user: $qqhash->{user}";
logError($self->{error});
            return 0;
        }
        $target = $user;
    } elsif ($basetype eq 'group' && !$default) {
        ($grphash, $group) = $self->_get_group_ref($target);
        if ( !$grphash ) {
            $self->{error} =
              "_fillout_quota_info: no existing group: $qqhash->{group}";
logError($self->{error});
            return 0;
        }
        $target = $group;
    }
    if (!defined($default)) {
        $default = ($target eq "*");
    }

    my $qtname = undef;
    my $qthash = undef;
    my $qtpath = undef;
    my $volname = undef;
    my $volhash = undef;
    my $volpath = undef;
    my $path;
    my $qtree = $qqhash->{tree} || $qqhash->{qtree};
    my $vol;
    if ($basetype eq 'tree' && $target =~ m%(/vol/.+)%) {
        $path = $1;
    } elsif ($type =~ m%@(/vol/.+)%) {
        $path = $1;
    } else {
        $path = $qqhash->{path};
    }
    if ($path && $path =~ m%(/vol/[^/]+)%) {
        $vol = $1;
        if ($path =~ m%(/vol/[^/]+/[^/]+)%) {
             $qtree = $1;
        }
    } elsif ($qqhash->{vol}) {
        $vol = $qqhash->{vol};
    }
    if ($qtree && !$vol) {
       ($qthash, $qtname, $qtpath) = $self->_get_qtree_ref($qtree);
       if ($qthash) {
           $vol = $qthash->{volume};
       }
    }
    if (!$vol) {
         $self->{error} =
           "_fillout_quota_info: no volume given";
logError($self->{error});
         return 0;
    }
    ($volname, $volhash, $volpath) = $self->_get_vol_ref($vol);
    if (!$volhash) {
         $self->{error} =
           "_fillout_quota_info: volume not found: $vol";
logWarn($self->{error});
	 next;
    }
    if ($qtree && !$qthash) {
       ($qthash, $qtname, $qtpath) = $self->_get_qtree_ref($qtree, vol=>$volname);
       if (!$volhash) {
              $self->{error} =
               "_fillout_quota_info: volume not found: $vol";
logWarn($self->{error});
	      next;
        }
    }
    $path = $qtpath || $volpath;
    if ($basetype eq 'tree') {
        if ($default) {
            $type = "tree\@$volpath";
        } else {
            $type = 'tree';
            $target = $path;
        }
    } elsif (!$target) {
    } else {
        $type = "$basetype\@$path";
        $qqhash->{$basetype} = $target;
    }
    if ($basetype eq 'user' && !$default) {
        $user = $target;
        ($usrhash, $user) = $self->_get_user_ref($user);
        if ( !$usrhash ) {
            $self->{error} =
               "_fillout_quota_info: no existing user: \"$user\"";
logError($self->{error});
            return 0;
        }
    } elsif ($basetype eq 'group' && !$default) {
        $group = $target;
        ($grphash, $group) = $self->_get_group_ref($group);
        if ( !$grphash ) {
            $self->{error} =
              "_fillout_quota_info: no existing group: $group";
logError($self->{error});
            return 0;
        }
    }

    my $dlimit_fun;
    my $dlimit = (defined($qqhash->{disk})) ? $qqhash->{disk} :
                      (defined($qqhash->{dlimit})) ? $qqhash->{dlimit} : "-";
    my $flimit = (defined($qqhash->{files})) ? $qqhash->{files} :
                      (defined($qqhash->{flimit})) ? $qqhash->{flimit} : "-";
    my $quota_key = "$type:$target";
    my $usermap = undef;
    if (exists($qqhash->{usermap}) && defined($qqhash->{usermap})) {
        $usermap = ($qqhash->{usermap} == 1) ? 1 : 0;
    }
    $qqhash->{qkey} = $quota_key;
    $qqhash->{disk} = $dlimit;
    $qqhash->{dlimit} = $dlimit;
    $qqhash->{flimit} = $flimit;
    $qqhash->{volname} = $volname;
    $qqhash->{volpath} = $volpath;
    delete($qqhash->{vol});
    delete($qqhash->{tree});
    delete($qqhash->{qtree});
    if ($qtname) {
        $qqhash->{qtname} = $qtname;
        $qqhash->{treename} = $qtname;
        $qqhash->{qtpath} = $qtpath;
        $qqhash->{treepath} = $qtpath;
    } else {
        delete($qqhash->{qtname});
        delete($qqhash->{treename});
        delete($qqhash->{qtpath});
        delete($qqhash->{treepath});
    }   

    $qqhash->{basetype} = $basetype;
    $qqhash->{type} = $type;
    $qqhash->{path} = $path;
    $qqhash->{target} = $target;
    $qqhash->{id} = $target;
#logresult('INFO', "_fillout_quota_info  Returning:\n\n".Dumper($qqhash));
    return ($qqhash);
}


sub _create_quota {
    my $self = shift;
    my $qqhash = shift;
    my (%opts) = shift;
logresult('INFO', "Enter _create_quota:\n\n".Dumper($qqhash));
    my $qqinfo = $self->{cfginfo}{quotas};
    my $new_quotas = {};

    my $min_files = $opts{min_files};
    my $max_files = $opts{max_files};
    my $min_disk = $opts{min_disk};
    my $max_disk = $opts{max_disk};
    $qqhash = $self->_fillout_quota_info($qqhash);
    my ($volname, $volhash, $volpath) = $self->_get_vol_ref($qqhash->{volname});
    my ($qthash, $qtname, $qtpath) = $self->_get_qtree_ref($qqhash->{treepath});
    my ($grphash, $group);
    if ($qqhash->{group}) {
        ($grphash, $group) = $self->_get_group_ref($qqhash->{group});
    }
    my ($usrhash, $user);
    if ($qqhash->{user}) {
        ($usrhash, $user) = $self->_get_user_ref($qqhash->{user});
    }
    my $dlimit = $qqhash->{dlimit};
    if (ref($dlimit) eq 'CODE') {
        my $dlimit_fun = $dlimit;
        my $qqh = { %$qqhash };
        $qqh->{max_disk} = $max_disk;
        $qqh->{min_disk} = $min_disk;
logresult('INFO', "Calling dlimit_fun:");
        $dlimit = &$dlimit_fun($qqh);
    }
    my $flimit = $qqhash->{flimit};
    if (ref($flimit) eq 'CODE') {
        my $flimit_fun = $flimit;
        my $qqh = { %$qqhash };
        $qqh->{max_files} = $max_files;
        $qqh->{min_files} = $min_files;
logresult('INFO', "Calling dlimit_fun:");
        $flimit = &$flimit_fun($qqh);
    }
    my $type = $qqhash->{type};
    my $target = $qqhash->{target};
    my $quota_key = $qqhash->{qkey};
    my $basetype = $qqhash->{basetype};
    my $path = $qqhash->{path};
    my $usermap = undef;
    if (exists($qqhash->{usermap}) && defined($qqhash->{usermap})) {
        $usermap = ($qqhash->{usermap} == 1) ? 1 : 0;
    }

    my $qq = { target=>$target, type=>$type, disk=>$dlimit, files=>$flimit };
    $qq->{usermap} = $qqhash->{usermap} if (defined($qqhash->{usermap}));

    setSingleQuota( $self->{fc}, %$qq )
      or $self->{error} =
      "_create_quota: cannot create quota type: ".
            "$type  target: $target:   filer: $self->{fname}: $@";
    if ($self->{error}) {
logWarn($self->{error});
logresult('FATAL', "create_quota  Line: ".__LINE__."    $self->{error}");
        return ( $self->{error} );
    }

logresult('INFO', "_create_quota: Enabling quotas for volume: $volname");
    enableQuota($self->{fc}, $volname, reenable=>1)
      or $self->{error} =
      "_create_quota: cannot (re)enable quotas for volume $volname ".
             "filer: $self->{fname}: $@";
    if ($self->{error}) {
logWarn($self->{error});
logresult('FATAL', "create_quota  Line: ".__LINE__."    $self->{error}");
        return ( $self->{error} );
    }

     $qqinfo->{count}++;
     $new_quotas->{$quota_key} = $qqhash;
     $qqinfo->{existing} = {} unless ($qqinfo->{existing});
     $qqinfo->{existing}{$quota_key} = $qqhash;
     $volhash->{quotas} = {} unless ($volhash->{quotas});
     $volhash->{quotas}{existing} = {} unless ($volhash->{quotas}{existing});
     $volhash->{quotas}{existing}{$quota_key} = $qqhash;
     $volhash->{quotas}{$basetype} = {} unless ($volhash->{quotas}{$basetype});
     $volhash->{quotas}{$basetype}{$target} = $qqhash;
     if ($qthash) {
         $qthash->{quotas}{existing}{$quota_key} = $qqhash;
         $qthash->{quotas}{$basetype} = {} unless ( $qthash->{quotas}{$basetype} );
         $qthash->{quotas}{$basetype}{$target} = $qqhash;
     }
    
     if ( $grphash ) {
         $grphash->{quotas} = {} unless ( $grphash->{quotas} );
         $grphash->{quotas}{$path} = $qqhash;
     } elsif ( $usrhash ) {
         $usrhash->{quotas} = {} unless ( $usrhash->{quotas} );
         $usrhash->{quotas}{$path} = $qqhash;

         #  Set 'uid', 'gid'  fields of qtree for use during quota tests.
         $qthash->{uid} = $usrhash->{uid};
         $qthash->{gid} = $usrhash->{gid};
     }
logresult('INFO', "Exit _create_quota  Returning:\n\n".Dumper($new_quotas));
     return $new_quotas;
}



#    _create_quotas_for_volume
#
#    Create one or more quote entries for users, groups, and/or qtree.
sub _create_quotas_for_volume {
    my $self = shift;
    my $vol_in = shift;
    my (%opts) = @_;

logresult('INFO', "Enter create_quotas_for_volume  VOL: \"$vol_in\"  OPTS:\n\n".Dumper(%opts));

    my $qqinfo            = $self->{cfginfo}{quotas};
    my $users            = defined($opts{users}) ? $opts{users} : undef;
    my $groups           = defined($opts{groups}) ? $opts{groups} : undef;
    my $qtrees           = defined($opts{qtrees}) ? $opts{qtrees} : undef;
    my $qtdlimit         = $opts{qtree_dlimit} || $qqinfo->{qtree_dlimit} || "-";
    my $qtflimit         = $opts{qtree_flimit} || $qqinfo->{qtree_flimit} || "-";
    my $udlimit          = $opts{user_dlimit} || $qqinfo->{user_dlimit} || "-";
    my $uflimit          = $opts{user_flimit} || $qqinfo->{user_flimit} || "-";
    my $gdlimit          = $opts{group_dlimit} || $qqinfo->{group_dlimit} || "-";
    my $gflimit          = $opts{group_flimit} || $qqinfo->{group_flimit} || "-";
    my $usermap          = $opts{usermap} || $qqinfo->{usermap} || 0;
    my $group_only       = $opts{group_only};
    my $user_only        = $opts{user_only};

    my @target_list = ();
    if ( !$users && !$groups && !$qtrees ) {
        my $self->{error} =
"_create_quotas_for_volume: at least one of 'qtrees', 'users', 'groups' options must be given";
        return ( $self->{error} );
    }
    if ( !$users && !$groups ) {
        my $tgt = {};
        $tgt->{tgt_type} = 'tree';
        @target_list = ($tgt);
    }
    if ( !$vol_in ) {
        $self->{error} = "_create_quotas_for_volume: no volume given";
        logError($self->{error});
        return ( $self->{error} );
    }
    my ($vol, $volhash) = $self->_get_vol_ref($vol_in);
    if (!$volhash) {
        $self->{error} = "_create_quotas_for_volume: no such volume: $vol";
        logError($self->{error});
        return ( $self->{error} );
    }
    my $volpath = "/vol/$vol";

    if ($users) {
        if ( ref($users) eq "ARRAY" ) {
        } elsif ( ref($users) eq "HASH" ) {
            $users = [ keys(%$users) ];
        } else {
            $self->{error} =
              "_create_quotas_for_volume: invalid 'users' value";
            logError($self->{error});
            return ( $self->{error} );
        }
    }
    if ($users && @$users) {
        foreach my $usr (@$users) {
            my ($uhash, $uname) = $self->_get_user_ref($usr);
            my $tgt = { };
            $tgt->{tgt_id} = $usr;
            $tgt->{tgt_type} = 'user';
            push ( @target_list, $tgt );
        }
    }

    if ($groups) {
        if ( ref($groups) eq "ARRAY" ) {
        } elsif ( ref($groups) eq "HASH" ) {
            $groups = [ keys(%$groups) ];
        } else {
            $self->{error} =
              "_create_quotas_for_volume: invalid 'groups' value";
            logError($self->{error});
            return ( $self->{error} );
        }
     }
     if ($groups && @$groups) {
        foreach my $grp (@$groups) {
            my ($ghash, $gname) = $self->_get_group_ref($grp);
            if (!$user_only) {
                my $tgt = { };
                $tgt->{tgt_id} = $grp;
                $tgt->{tgt_type} = 'group';
                push ( @target_list, $tgt );
            }
            if (!defined($users) && !$group_only) {
                my %opts_copy = %opts;
                my @users_for_group = keys(%{$ghash->{users}});
                $opts_copy{users} = \@users_for_group;
                $opts_copy{groups} = [];
                $self->_create_quotas_for_volume($vol, %opts_copy);
            }
        }
    }

    my @qtrees_or_vol;

    if ( $qtrees ) {
        my ($qthash, $qtpath);
        if ( ref($qtrees) eq "ARRAY" ) {
        } elsif ( ref($qtrees) eq "HASH" ) {
            $qtrees = [ keys(%$qtrees) ];
        } else {
            $self->{error} =
              "_create_quotas_for_volume: invalid 'qtrees' value";
            logError($self->{error});
            return ( $self->{error} );
        }
    }

    if ( $qtrees && @$qtrees ) {
        @qtrees_or_vol = ();
        foreach my $qt (@$qtrees) {
            my ($qthash, $qtname, $qtpath) = $self->_get_qtree_ref($qt, vol=>$vol);
            if (!$qthash) {
                $self->{error} = "_create_quotas_for_volume: no such qtree: $qtname";
                logError($self->{error});
                return ( $self->{error} );
             }
            $qthash->{quotas} = {} unless ( $qthash->{quotas} );
            $qthash->{quotas}{existing} = {} unless ( $qthash->{quotas} );
            push(@qtrees_or_vol, $qthash);
        }
    } else {
        @qtrees_or_vol = ($volhash);
    }
    my $new_quotas = {};
    foreach my $qthash (@qtrees_or_vol) {
        my $path = $qthash->{path};
        my $qqhash = {};
        $qqhash->{path} = $path;
        $qqhash->{usermap} = $usermap if (defined($usermap));
        foreach my $tgt (@target_list) {
            my $basetype = $tgt->{tgt_type};
            if ( $basetype eq 'user' ) {
                $qqhash->{user} = $tgt->{tgt_id};
                $qqhash->{disk} = $udlimit;
                $qqhash->{files} = $uflimit;
                $qqhash->{type} = "user";
            } elsif ( $basetype eq 'group' ) {
                $qqhash->{group} = $tgt->{tgt_id};
                $qqhash->{disk} = $gdlimit;
                $qqhash->{files} = $gflimit;
                $qqhash->{type} = "group";
            } elsif ( $basetype eq 'tree' ) {
                $qqhash->{disk} = $qtdlimit;
                $qqhash->{files} = $qtflimit;
                $qqhash->{type} = "tree";
            }
            my $new_quota_hash = $self->_create_quota($qqhash, %opts);
            $new_quotas = { %$new_quotas, %$new_quota_hash };
        }
logresult('INFO', "Exit create_quotas_for_volume   New Quotas:\n\n".Dumper($new_quotas));
    }
    return ( $new_quotas );
}


sub _create_quotas_for_qtree {
    my $self = shift;
    my $qtree = shift;
    my (%opts) = @_;

    my ($qthash, $qtname, $qtpath) = $self->_get_qtree_ref($qtree);
    if ( !$qthash ) {
        $self->{error} =
          "_create_quotas_for_qtree: could not find qtree: $qtpath";
        return ( $self->{error} );
    }
    $opts{qtrees} = [ $qtname ];
    my ($vol, $volhash, $volpath) = $self->_get_vol_ref($qthash->{volume});
    return ( $self->_create_quotas_for_volume( $vol, %opts ) );
}


# sub _create_quotas {
#     my $self = shift;
#     my (%opts) = @_;
#     my $qinfo    = $self->{cfginfo}{quotas};
#     my $existing = $qinfo->{existing};
#     my $needed   = $qinfo->{new};
#     return (0) if ( $needed <= 0 );
# 
#     my $volhash     = $self->{containers}{volume};
#     my $qtreehash   = $self->{containers}{qtrees};
#     my @vol_names = keys(%$volhash);
#     my @qtree_names = keys(%$qtreehash);
#     foreach my $vol (@vol_names) {
#         $self->_create_quotas_for_volume( $vol, %opts );
#         return (0) if ( $self->{error} );
#     }
#     $qinfo->{new} = 0;
#     return (1);
# }


# sub _create_quotas_for_client {
#     my $self = shift;
#     my $client = shift;
#     my $clientinfo = shift;
#     my (%opts) = @_;
#     my $users_per_group = $opts{users_per_group} ||
#                              $self->{cfginfo}{users_per_group} || 1;
#     my $groups_per_qtree = $opts{groups_per_qtree} ||
#                              $self->{cfginfo}{groups_per_qtree} || 1;
#     my $qtrees_per_client = $opts{qtrees_per_client} ||
#                              $self->{cfginfo}{qtrees_per_client} || 1;
#     my $existing_qtrees = $self->{containers}{$QTREES};
#     my $existing_users = $self->{containers}{$QTREES};
#     my $existing_groups = $self->{containers}{$QTREES};
# }


sub _create_quotas_for_groups_per_qtree {
    my $self = shift;
    my (%opts) = @_;

    $self->{cfginfo}{quotas} = {} unless ($self->{cfginfo}{quotas});
    my $qqinfo = $self->{cfginfo}{quotas};

    my $qtreeinfo = $self->{cfginfo}{qtrees};
    $self->setup_qtrees(force=>1)
      unless ( keys(%{$qtreeinfo->{existing}}) );
    my $existing_qtrees = $qtreeinfo->{existing};
    my $num_qtrees = scalar(keys(%$existing_qtrees));
    my @qtree_list = keys(%$existing_qtrees);

    my $groups_per_qtree = $opts{groups_per_qtree} || 1;
    my $users_per_group = $opts{users_per_group} || 1;
    my $groupinfo = $self->{cfginfo}{groups};
    $self->setup_groups(force=>1)
      unless ( keys(%{$groupinfo->{existing}}) );
    my $existing_groups = $groupinfo->{existing};
    my @group_list = keys(%$existing_groups);

    my $users_per_group = $opts{users_per_group} || 1;
    my $userinfo = $self->{cfginfo}{users};
    $self->setup_users(force=>1)
      unless ( keys(%{$userinfo->{existing}}) );
    my $existing_users =  $userinfo->{existing};
    my @user_list = keys(%$existing_users);

    $opts{group_only} = 0;
    $opts{user_only} = 1;
    delete($opts{group_only});
    for my $qthash (values(%$existing_qtrees)) {
        my $qtree_existing_group_quotas = $qthash->{quotas}{group};
        my $needed = $groups_per_qtree - scalar(keys(%$qtree_existing_group_quotas));
        my $new_groups = $self->_create_groups(new=>$needed, users_per_group=>$users_per_group, group_only=>0);
        $opts{groups} = $new_groups;
        $self->_create_quotas_for_qtree($qthash, %opts);
    }
}


sub _create_quotas_for_users_per_qtree {
    my $self = shift;
    my (%opts) = @_;

logresult('INFO', "Enter _create_quotas_for_users_per_qtree");
    $self->{cfginfo}{quotas} = {} unless ($self->{cfginfo}{quotas});
    my $qqinfo = $self->{cfginfo}{quotas};

    my $qtreeinfo = $self->{cfginfo}{qtrees};
    $self->setup_qtrees(force=>1)
      unless ( keys(%{$qtreeinfo->{existing}}) );
    my $existing_qtrees = $qtreeinfo->{existing};
    my $num_qtrees = scalar(keys(%$existing_qtrees));
    my @qtree_list = keys(%$existing_qtrees);

    my $users_per_qtree = $opts{users_per_qtree} || 1;
    my $groupinfo = $self->{cfginfo}{groups};
    $self->setup_groups(force=>1)
      unless ( keys(%{$groupinfo->{existing}}) );
    my $existing_groups = $groupinfo->{existing};
    my @group_list = keys(%$existing_groups);

    my $users_per_group = $opts{users_per_group} || 1;
    my $userinfo = $self->{cfginfo}{users};
    $self->setup_users(force=>1)
      unless ( keys(%{$userinfo->{existing}}) );
    my $existing_users =  $userinfo->{existing};
    my @user_list = keys(%$existing_users);

    $opts{group_only} = 0;
    $opts{user_only} = 1;
    delete($opts{group_only});
    for my $qthash (values(%$existing_qtrees)) {
        my $qtree_existing_user_quotas = $qthash->{quotas}{user};
        my $needed = $users_per_qtree - scalar(keys(%$qtree_existing_user_quotas));
#logresult('INFO', "_create_quotas_for_users_per_qtree  Line: ".__LINE__."   qtree: \"$qthash->{path}\"  NEEDED: $needed  USERS_PER_QTREE: $users_per_qtree   EXISTING: ". scalar(keys(%$qtree_existing_user_quotas)));

### logresult('INFO', "_create_quotas_for_users_per_qtree  Line: ".__LINE__."      QTREE:\n\n".Dumper($qthash));
        next if ($needed <= 0);

### logresult('FATAL', "_create_quotas_for_users_per_qtree  Line: ".__LINE__."      QTREE:\n\n".Dumper($qthash));
        my $group_for_quotas;
        my $new_users;
        if ( scalar(keys(%$qtree_existing_user_quotas)) < 1 ) {
            my $new_groups = $self->_create_groups(new=>1);
            my @grp_keys = keys(%$new_groups);
            $group_for_quotas = $grp_keys[0];
        } else {
            my @vals = values(%$qtree_existing_user_quotas);
            my $qq = $vals[0];
            my $uname = $qq->{username};
            my ($uhash, $login, $uid) = $self->_get_user_ref($uname);
            $group_for_quotas = $uhash->{group};
        }
        my $new_users = $self->_create_users_for_group($group_for_quotas, $needed, %opts);
        $opts{users} = $new_users;
#logresult('INFO', "_create_quotas_for_users_per_qtree  Line: ".__LINE__."   Calling _create_quotas_for_qtree  OPTS:\n\n".Dumper(%opts));
        $self->_create_quotas_for_qtree($qthash, %opts);
#logresult('INFO', "_create_quotas_for_users_per_qtree  Line: ".__LINE__."   Returned from _create_quotas_for_qtree");
    }
#logresult('INFO', "Exit _create_quotas_for_users_per_qtree");
}


sub _get_qtree_quota_users_and_groups {
    my $self = shift;
    my $qtree = shift;
    my ($qthash) = $self->_get_qtree_ref($qtree);
    if (!$qthash) {
    }
    my $users_hash = {};
    my $groups_hash = {};
    my ($user_quotas) = $qthash->{quotas}{user};
    foreach $qthash (values(%$user_quotas)) {
        my $user = $qthash->{target};
        my ($uhash, $login, $uid) = $self->_get_user_ref($user);
        $users_hash->{$user} = $uhash;
    }
    my ($group_quotas) = $qthash->{quotas}{group};
    foreach $qthash (values(%$group_quotas)) {
        my $group = $qthash->{target};
        my ($grphash, $gname, $gid) = $self->_get_group_ref($group);
        $groups_hash->{$group} = $grphash;
    }
    return { users=>$users_hash, groups=>$groups_hash };
}


sub quota_info_for_qtree {
    my $self     = shift;
    my $qtree    = shift;
    my ($qthash, $qtname, $qtpath) = $self->_get_qtree_ref($qtree);
    my $quota_report = getQuotaReport($self->{fc});
    if (!$qthash) {
        return undef;
    }
    my $qtree_users_and_groups = $self->_get_qtree_quota_users_and_groups($qtree);
    my @qtree_users = keys(%{$qtree_users_and_groups->{users}});
    if (scalar(@qtree_users) > 1) {
logError("quota_info_for_qtree: Qtree $qtpath has multiple user quotas: @qtree_users");
    }
    my $qtree_user = $qtree_users[0];
    my $qr;
#logresult('INFO', "quota_info_for_qtree  Line: ".__LINE__. "    Looking for Qtree: $qtname   User: $qtree_user");
    foreach (@$quota_report) {
#logresult('INFO', "quota_info_for_qtree  Line: ".__LINE__."\n\n".Dumper($_));
        if ($_->{tree} eq $qtname && $_->{type} eq "user" && $_->{target} eq $qtree_user) {
            $qr = $_;
            last;
        }
  logcomment("couldn't find $qtree_user");
    }
    if (!$qr) {
logError("quota_info_for_qtree: No quota report info for qtree $qtpath");
        return undef;
    }
    my $curr_size = 1024 * $qr->{kbytes_used};
    my $max_size=  1024 * $qr->{kbytes_limit} || "-";
    my $curr_files = $qr->{files_used};
    my $max_files = $qr->{files_limit} || "-";
    my $quota_info = { curr_files=>$curr_files, max_files=>$max_files, 
                           curr_size=>$curr_size, max_size=>$max_size};
logresult('INFO', "Exit quota_info_for_qtree:  $qtname  User:$qtree_user Returning:\n\n".Dumper($quota_info)."\n\nQR:\n\n".Dumper($quota_info));
return ( $quota_info );
}


########################################################
#
#      Quota Support Functions for NfsFuncs.pm
#

sub _initialize_subtest_quotas {
    my $self     = shift;
    my $nfsobj   = shift;
    my $server   = shift;

    $self->{client_quotas} = {}
        unless ($self->{client_quotas});
}

#

sub _start_quota_op {
    my $self     = shift;
    my $nfsobj   = shift;
    my $server   = shift;
}

sub _get_mountpoint_to_user_hashtable {
}

sub _user_for_mountpoint {
}

#
#      End Quota Support Functions for NfsFuncs.pm
#
########################################################


########################################################
#
#      Quota Functions for NfsFuncs.pm
#
#      -- to be added to NfsFuncs.pm
#

sub _start_quota_op {
    my $self = shift;
    my $server = shift;
    my $quotas = $self->{cfginfo}{quotas};
    return 
       unless ($quotas);
    my $filer = $quotas->{filer};
    my $op_key = $filer->_start_quota_op($self, $server);
    return ( $op_key );
}

sub _process_quota_limits {
    my $self = shift;
    my $var = shift;
    my $mp = shift;
    my $op_key = shift;

    my $self = shift;
    my $quotas = $self->{cfginfo}{quotas};
    return 
       unless ($quotas);
    my $filer = $quotas->{filer};

    my ($file_size) = ($var =~ /FILE_SIZE\=(\d*)/);
    my ($file_count) = ($var =~ /FILE_COUNT\=(\-?\d*)/);
    my ($success) = ($var !~ /FAIL/);
    my $qtree = $self->{mounts}{$mp};
    $qtree->{size_used} += int($file_size);
    $qtree->{files_used} += int($file_count);

    # now check the quota report for this user
    # compare quota report with used
    my $error = $filer->_process_quota_limits($op_key, $mp,
                                              $file_size, $file_count,
                                              $success);

    # $self->{error} if error (but don't kill test)
    return;
}

#
#      End Quota Functions for NfsFuncs.pm
#
########################################################


########################################################
#
#      OLD Quota Support Functions
#

use constant MKFILE         => 1;
use constant RMFILE         => 2;
use constant MKDIR          => 3;
use constant RMDIR          => 4;
use constant APPENDFILE     => 5;
use constant TRUNCFILE      => 6;
use constant ALLOW          => 7;
use constant DISALLOW       => 8;
use constant PASS           => 9;
use constant FAIL           => 10;

sub InitializeQuotaTest {
    my $self = shift;
    my $client = shift;
    my $subtest_id = shift;

    $self->{quota_tests} = {}
        unless ($self->{quota_tests});
    my $quota_tests = $self->{quota_tests};
    $quota_tests->{$client} = {}
        unless ($quota_tests->{$client});
    my $client_quota_tests = $quota_tests->{$client};
    if ($client_quota_tests->{$subtest_id}) {
        logresult('FATAL', "InitializeQuotaTest:  subtest $subtest_id already initialized for client $client -- Filer: $self->{fname}");
    }
    my $quota_test = {};
    $client_quota_tests->{$subtest_id} = $quota_test;
}

sub BeforeFileOp {
    my $self = shift;
    my $client = shift;
    my $subtest_id = shift;
    my $op_type = shift;
    my $path = shift;
    my $size = shift;

    my ($key, $expected_result);
    return ($key, $expected_result);
}

sub AfterFileOp {
    my $self = shift;
    my $key = shift;
    my $result = shift;
    my $clientinfo = shift;

    my $status;
    return ($status);
}

#
#      End OLD Quota Support Functions
#
########################################################


########################################################
#
#       Mutex Functions
#

sub _acquire_mutex {
    my $self = shift;
    if (!$self->{mutex_file}) {
        $self->_get_mutex_file();
    }
    open(MUTEX, ">$self->{mutex_file}")
        or logresult('FATAL', "Can't open file $self->{mutex_file}: $!");

    flock(MUTEX, LOCK_EX)
        or logresult('FATAL', "Can't lock mutex file $self->{mutex_file}: $!");
    $self->{mutex_active} = 1;
}

sub _release_mutex {
    my $self = shift;
    if (!$self->{mutex_file}) {
        return;
    }
    flock(MUTEX, LOCK_UN)
        or logresult('FATAL', "Can't release mutex file $self->{mutex_file}: $!");
    close(MUTEX)
         or logresult('FATAL', "Can't close mutex file $self->{mutex_file}: $!");
}

sub _get_mutex_file {
     my $self = shift;
     $self->{mutex_file} = "/tmp/IFTFilerSetup_mutex_" . $self->{fname};
     open(MUTEX, ">$self->{mutex_file}")
         or logresult('FATAL', "Can't create mutex file $self->{mutex_file}: $!");
     print MUTEX `date`
         or logresult('FATAL', "Can't write to mutex file $self->{mutex_file}: $!");
     close(MUTEX)
         or logresult('FATAL', "Can't close mutex file $self->{mutex_file}: $!");
}

sub _rm_mutex_file {
    my $self = shift;
    if (!$self->{mutex_file}) {
        return;
    }
    unlink($self->{mutex_file});
}

sub test_mutex {
    my $self = shift;
    logresult('INFO', "Start test_mutex");
    sleep(10);
    logresult('INFO', "Parent waiting on mutex ...");
    $self->_acquire_mutex();
    logresult('INFO', "... Parent acquired mutex");
    my $pid = fork();
    if ($pid) {
        sleep(10);
        logresult('INFO', "Parent releasing mutex ...");
        $self->_release_mutex();
        waitpid($pid, 0);
    } else {
        logresult('INFO', "Child waiting on mutex ...");
        $self->_acquire_mutex();
        logresult('INFO', "... Child acquired mutex");
        sleep(5);
        logresult('INFO', "Child releasing mutex ...");
        $self->_release_mutex();
        sleep(20);
    }
    logresult('INFO', "Finished test_mutex");
}

#
#       End Mutex Functions
#
########################################################

    

##################################################
#
#        End Quota section
#
##################################################

##############################################
#
#         iSCSI / FCP Setup
#
##############################################

sub get_filer_fcp_nodename {
    my $self = shift;
    # blocks_filer::filer_fcp_nodename_get
    my $nodename = filer_fcp_nodename_get($self->{fc});
    return ($nodename);
}

sub setup_iscsi {
    my $self = shift;
    my $clientinfo = shift;
    $self->{iscsi} = {} unless ($self->{iscsi});
    filer_service_stop( $self->{fc}, 'iscsi' );
    iscsi_setup( $self->{fc} );
    sleep(30);
    foreach my $client (keys(%{$clientinfo->{clients}})) {
###         $clientinfo->_initialize_fcp( $client, $self );
         $clientinfo->_configure_fcp( $client, $self );
    }
    sleep(30);
    $self->{iscsi}{initiators} = $self->_get_iscsi_initiator_hash();
    $self->{iscsi}{initialized} = 1;
    return $self->{error} if $self->{error};
}

sub setup_fcp {
    my $self = shift;
    my $clientinfo = shift;
    $self->{fcp} = {} unless ($self->{fcp});
    filer_service_stop( $self->{fc}, 'fcp' );
    fcp_setup( $self->{fc} );
    sleep(30);
    is_fcp_setup( $self->{fc} )
      or $self->{error} = sprintf "%s: Cannot setup FCP on filer %s",
      $self->{fname}, $self->{fc};
    $self->{fcp} = {} unless ($self->{fcp});
    $self->_get_fcp_target_info();
    foreach my $client (keys(%{$clientinfo->{clients}})) {
###         $clientinfo->_initialize_fcp( $client, $self );
         $clientinfo->_configure_fcp( $client, $self );
    }
    sleep(30);
    $self->{fcp}{initiators} = $self->_get_fcp_initiator_hash();
    $self->{fcp}{initialized} = 1;
    return $self->{error} if $self->{error};
}

sub setup_fcp_adapter {
    my $self    = shift;
    my $adapter = shift;
    my (%opts) = (@_);
    my $mediatype = $opts{mediatype};
    my $partner   = $opts{partner};

    # blocks_filer::filer_fcp_adapter_offline
    filer_fcp_adapter_offline( $self->{fc}, $adapter );
    if ($mediatype) {

        # blocks_filer::filer_fcp_adapter_mediatype_set
        filer_fcp_adapter_mediatype_set( $self->{fc}, $adapter, $mediatype );
    }
    if ( defined($partner) ) {
        if ( $partner eq 'default' ) {

            # blocks_filer::filer_fcp_adapter_partneradapter_default
            filer_fcp_adapter_partneradapter_default( $self->{fc}, $adapter );
        } elsif ( !$partner || $partner eq 'none' ) {

            # blocks_filer::filer_fcp_partneradapter_remove
            filer_fcp_adapter_partneradapter_remove( $self->{fc}, $adapter );
        } else {

            # blocks_filer::filer_fcp_adapter_partneradapter_set
            filer_fcp_adapter_partneradapter_set( $self->{fc}, $adapter,
                $partner );
        }
    }

    # blocks_filer::filer_fcp_adapter_online
    filer_fcp_adapter_offline( $self->{fc}, $adapter );
}

##############################################
#
#         End FCP Setup
#
##############################################

##############################################################################################

##############################################################################################

package IFTIPCSetup;
use IO::Socket;

#use IO::Handle;
use Net::hostent;    # for OO version of gethostbyaddr
use Data::Dumper;
our $PORT      = 9000;            # pick something not in use
our $HOST      = "localhost";
our $GETSTATUS = "get status";
our $SETSTATUS = "set status ";
our $EXIT      = "exit";
our $testinfo  = {};
our $filerinfo = {};

sub new {
    my $class = shift;
    my %args  = @_;
    return if ( !exists $args{op} );
    return if ( !exists $args{filers} );
    my $self = {};
    $self->{filers} = $args{filers};
    $self->{op}     = $args{op};
    foreach my $f ( keys %{ $self->{filers} } ) {
        $self->{filers}{ $self->{filers}{$f}{fname} }{status} = undef;
    }
    bless $self, $class;
    if ( $self->{op} eq "start" ) {
        my $pid = fork() or die "status server not up (cannot fork)";
        if ( !$pid ) {
            $self->start_status_server();
            return;
        } else {
            $self->{pid} = $pid;
            return $self;
        }
    } elsif ( $self->{op} eq "conn" ) {
        $self->{server} = $self->connect_to_status_server();
        return if ( !$self->{server} );
        return $self;
    } else {
        return;
    }
}

sub start_status_server {
    my $self   = shift;
    my $server = IO::Socket::INET->new(
        Proto     => 'tcp',
        LocalPort => $PORT,
        Listen    => SOMAXCONN,
        Reuse     => 1
    );
    die "Cannot connect $!" if ( !$server );

    IO::Socket->output_record_separator("\n");

    my $sread  = new IO::Select();
    my $swrite = new IO::Select();
    $swrite->add($server);
    $sread->add($server);
    while (1) {
        print "waiting on a client to connect or ready to read...\n";
        my ($rdsock) = IO::Select->select($sread);
        my $monitor = {};

        foreach my $sock (@$rdsock) {
            if ( $sock == $server ) {
                my $news = $sock->accept();
                $sread->add($news);
                print "got a new one!\n";
                next;
            }
            print "reading socket...\n";
            my $cmd = <$sock>;
            chomp $cmd;
            print "I got the command: $cmd\n";
            if ( $cmd eq $GETSTATUS ) {
                my $status = "status ";
                foreach my $f ( keys %{ $self->{filers} } ) {
                    $status .= " $f $self-{filers}{$f}{status} ";
                }
                print $sock $status;
            } elsif ( $cmd =~ /$SETSTATUS\s*(\*|\w+)\s(\w+)/ ) {
                my $filer  = $1;
                my $status = $2;
                if ( $filer eq "all" ) {
                    foreach my $f ( keys %{ $self->{filers} } ) {
                        $self->{filers}{$f}{status} = $status;
                    }
                } else {
                    $self->{filers}{$filer}{status} = $status;
                }
                print "setting status $filer $status\n";
                print $sock "status OK";
            } elsif ( $cmd =~ /get\smonitor\s(\w+)/ ) {
                if ( $monitor->{$1} eq "on" ) {
                    print $sock "on";
                } else {
                    print $sock "off";
                }
            } elsif ( $cmd =~ /set\smonitor\s(w+)\s(\w+)/ ) {
                my $filer = $1;
                my $cond  = $2;
                $monitor->{$filer}{cond} = $cond;
                if ( $cond eq "on" ) {
                    push @{ $monitor->{$filer}{socks} }, $sock;
                } else {
                    my @temp = ();
                    foreach my $s ( @{ $monitor->{$filer}{socks} } ) {
                        next if ( $s == $sock );
                        push @temp, $s;
                    }
                    $monitor->{$filer}{socks} = [@temp];
                }
            } elsif ( $cmd =~ /mon\s\w+\s(.*)/ ) {
                my $filer = $1;
                my $text  = $2;
                foreach my $s ( @{ $monitor->{$filer}{socks} } ) {
                    print $s $text;
                }
            } elsif ( $cmd eq $EXIT ) {
                my $socks = $sread->handles;
                print "closing down the sockets...\n";
                foreach my $s (@$socks) {
                    close $s;
                }
                print "adios!\n";
                exit(0);
            } elsif ( $cmd =~ /TEST/ ) {
                do_test_info($cmd);
            } elsif ( $cmd =~ /FILER/ ) {
                do_filer_info($cmd);
            }
        }

    }
}

sub connect_to_status_server {
    my $self   = shift;
    my $host   = $HOST;
    my $port   = $PORT;
    my $handle = undef;
    return if ( $self->{server} );

  CONNECT:

    $handle = IO::Socket::INET->new(
        Proto    => "tcp",
        PeerAddr => $host,
        PeerPort => $port
      )
      or die "can't connect to port $port on $host: $!";
    $handle->autoflush(1);    # so output gets there right away
    IO::Socket->output_record_separator("\n");

    if ( !$handle ) {
        $self->{error} = "Cannot connect to status server: $!";
    }
    $self->{server} = $handle;
    return;
}

sub get_status {
    my $self  = shift;
    my %args  = @_;
    my $filer = $args{filer};
    $filer = "all" if ( !$filer );
    my $cmd    = "get status $filer";
    my $handle = $self->{server};
    print $handle $cmd;
    my $line = undef;
    my $stat = undef;

    while ( $line = <$handle> ) {
        next if ( $line !~ /status/ );
        $stat = 1;
        if ( $filer eq "all" ) {
            $line =~ /~status\s(\w+)\s(\w+)\s(\w+)\s(\w+)$/;
            $self->{filers}{$1}{status} = $2;
            $self->{filers}{$3}{status} = $4;
        } else {
            $line =~ /status\s\w+\s(\w+)/;
            $self->{filers}{$filer}{status} = $1;
        }
    }
    $self->{error} = "could not retrieve status" if ( !$stat );
    return;
}

sub set_status {
    my $self   = shift;
    my %args   = @_;
    my $filer  = $args{filer};
    my $status = $args{status};
    my $handle = $self->{server};
    $filer = "all" if ( !$filer );
    my $cmd = "set status $filer $status";

    print $handle $cmd;
    my $line = undef;
    my $stat = undef;
    while ( $line = <$handle> ) {
        next if ( $line !~ /OK/ );
        $stat = 1;
    }
    if ( $filer eq "all" ) {
        foreach my $f ( keys %{ $self->{filers} } ) {
            $self->{filers}{$f}{status} = $status;
        }
    } else {
        $self->{filers}{$filer}{status} = $status;
    }
    $self->{error} = "could not retrieve status" if ( !$stat );
    return;
}

sub do_test_info {
    my $sock    = shift;
    my $cmd     = shift;
    my $cmdinfo = {};
    my $end     = 0;
    while ($cmd) {
        my ( $k, $v );
        for my $field ( split /;/, $cmd ) {
            if ( $field eq "END" ) {
                $end = 1;
                next;
            }
            my ( $k, $v ) = split /:/, $field;
            $cmdinfo->{$k} = $v;
        }
        last if ($end);
        $cmd = <$sock>;
    }

    if ( !exists $cmdinfo->{TEST} ) {
        print $sock "NAK";
        return;
    }
    my $test = $cmdinfo->{TEST};
    delete $cmdinfo->{TEST};
    foreach my $key ( keys %{$cmdinfo} ) {
        $testinfo->{$test}{$key} = $cmdinfo->{$key};
    }
    log_test_info($test);

}

sub log_test_info {
    my $test = shift;
}

sub start_monitor {
    my $self = shift;

    foreach my $key ( keys %{ $self->{filers} } ) {
        $self->{monitor}{$key}{pid} = fork() or die "Unable to start monitor";
        if ( !exists $self->{monitor}{$key}{pid} ) {
            $self->monitor_filer($key);
        }
    }
}

sub monitor_filer {
    my $filer = shift;
    return;
}

# ------------------------------------------------------------------------
1;

# ------------------------------------------------------------------------
