## $Id: $
## @author      pravin.salgond@netapp.com
## @POD         For documentation see POD section of each package       (#TODO)
#==================================================================================
package RaidAzcsUtils;
use Tharn qw($Log);
use RaidSqav3 qw(AUTOLOAD);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF CODEREF UNDEF);

#-------------------------------------------------------------------------------
sub check_spares {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $aggr_create = $self->{tc_hash}->{aggr_create};
    my $disk_count = $aggr_create->{disk_count};
    my $mirror = $aggr_create->{mirror}; 
    my $checksum_type = $aggr_create->{checksum_type};
    my @disk_list = ();

    if($checksum_type) {
        $Log->comment("$self->{test}: Get spares by checksum_type: $checksum_type");
        @disk_list = RaidSqa_get_disk_list_by_checksum_type(api_set_obj => $api_set_obj,
                                                            spare_only => "yes",
                                                            checksum_type => $checksum_type);
    } else {
        $Log->comment("$self->{test}: Get all spare disks");
        @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                            spare_only => "yes");
    }
    
    $Log->comment("$self->{test}: All spares: @disk_list");
    
    #===============================================================
    $Log->step("Check if sufficient spare disks are available");
    my ($required_disks);
    if ($mirror) {
        $required_disks = $disk_count / 2;
        foreach my $pool ("pool0","pool1") {
            @pool_disk_list = RaidSqa_get_disk_list_by_pool(api_set_obj => $api_set_obj,
                                                         spare_only => "yes",
                                                         disk_list => \@disk_list,
                                                         pool_name  => $pool);
            if( scalar(@pool_disk_list) < $required_disks ) {
                $Log->comment("$self->{test}: spare disk count from $pool: $disk_count");
                die("$self->{test}: Sufficient spares are not available in $pool");
            }
        }
    } else {
        if( scalar(@disk_list) < $disk_count ) {
                $Log->comment("$self->{test}: Required disk count: $disk_count, available disks: @disk_list");
                die("$self->{test}: Sufficient spares are not available for aggr create");
        }
    }
    $Log->comment("$self->{test}: Enough spares disks are available to create volume");
}

#-------------------------------------------------------------------------------
sub aggr_ok {
    my $self = shift;
    my $tc_hash = $self->{tc_hash};
    my $api_set_obj = $self->{api_set_obj};
    my $aggr_create = $tc_hash->{aggr_create};

    die("$self->{test}: aggr_create is mandetory field in TC hash") if(!defined($aggr_create));
    if(RaidSqa_check_existing_aggr_ok(api_set_obj   => $api_set_obj,
                                      filer_vol     => $aggr_create->{filer_vol},
                                      aggr_type     => $aggr_create->{aggr},
                                      raid_type     => $aggr_create->{raid_type},
                                      mirror        => $aggr_create->{mirror},
                                      storage_type  => $aggr_create->{storage_type},
                                      aggr_bits     => $aggr_create->{aggr_bits},
                                      checksum_type => $aggr_create->{checksum_type},
                                      )) {
        $Log->comment("$self->{test}: Aggr $aggr_create->{filer_vol} is ok for this test");
        return 1;
    } else {
        $Log->comment("$self->{test}: Aggr $aggr_create->{filer_vol} is not ok for this test");
        return 0;
    }
}

#-------------------------------------------------------------------------------
sub aggr_exists {
    my $self = shift;
    my $tc_hash = $self->{tc_hash};
    my $api_set_obj = $self->{api_set_obj};
    my $aggr_create = $tc_hash->{aggr_create};

    die("$self->{test}: aggr_create is mandetory field in TC hash") if(!defined($aggr_create));
    if(RaidSqa_check_aggr_exist_anytype(api_set_obj=>$api_set_obj, filer_vol => $aggr_create->{filer_vol})) {
        return 1;
    } else {
        return 0;
    }
}

#---------------------------------------------------------------------
sub aggr_create {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $aggr_create = $tc_hash->{aggr_create};
    my $new_aggr_required = (exists $aggr_create->{new_aggr_required}) ? $aggr_create->{new_aggr_required} : 1;
    my $filer_vol     = $aggr_create->{filer_vol};
    my %aggr_create_args = (api_set_obj   => $api_set_obj,
                            filer_vol     => $filer_vol,
                            raid_type     => $aggr_create->{raid_type},
                            raid_size     => $aggr_create->{raid_size},
                            aggr_type     => $aggr_create->{aggr},
                            aggr_bits     => $aggr_create->{aggr_bits},
                            mirror        => $aggr_create->{mirror},
                            vserver       => $aggr_create->{vserver},
                        );
    
    if(exists $aggr_create->{checksum_type}) {
        if($self->is_cmode()) {
            $aggr_create_args{checksum_type} = $aggr_create->{checksum_type};
        } elsif($aggr_create->{checksum_type} eq "zoned") {
            $Log->comment("Adding -z options for zcsv1 aggr");
            $aggr_create_args{more_options} = " -z";
        } else {
            $aggr_create_args{checksum_type} = $aggr_create->{checksum_type};
        }
    }
    
    if(exists $aggr_create->{disk_count}) {
        $aggr_create_args{disk_count} = $aggr_create->{disk_count};
    } elsif(defined($aggr_create->{disk_list})) {
        $aggr_create_args{disk_list} = $aggr_create->{disk_list};
        $aggr_create_args{disk_list} = $aggr_create->{disk_list1} if(defined($aggr_create->{disk_list1}));
    }
    
    my (@group_list);
    die("$self->{test}: No filer_vol defined") if(!defined($filer_vol));
    #define test id to append to volume name if volume already exists
    $Log->comment("$self->{test}: creating testvolume $filer_vol");
    my $aggr_exists = $self->aggr_exists();
    if($aggr_exists && !$new_aggr_required) {
        $aggr_exists = $self->aggr_ok();
    }

    $self->aggr_destroy() if($aggr_exists and $new_aggr_required);
    $self->check_spares() if($new_aggr_required);

    #---------------------------------------------------------------
    $Log->comment("Create test volume");
    #---------------------------------------------------------------    
    if(!$new_aggr_required && $aggr_exists) {
        $Log->comment("using existing aggr with name $filer_vol");
        $self->aggr_reuse();
    } else {        
        if (!RaidSqa_aggr_create(%aggr_create_args)) {
            die("$self->{test}: aggr create failed to create $filer_vol");
        }
    }

    $Log->comment("$self->{test}: Volume $filer_vol created successfully");
    @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
    $tc_hash->{group_list} = [@group_list];

    #Update disk rgdn for all disks in each raid group
    foreach my $group (@group_list) {
        my @disk_list = RaidSqa_get_disk_list(api_set_obj=>$api_set_obj, group_name => $group);
        foreach my $disk (@disk_list) {
            my $rgdn = RaidSqa_get_disk_rgdn(api_set_obj=>$api_set_obj, disk_name=>$disk);
            $tc_hash->{rgdn_info}->{$group}->{$rgdn} = $disk;
            $Log->comment("$self->{test}: group $group, disk $disk, rgdn $rgdn");
        }
    }
    $Log->comment("Dumper:: create_testvol tc_hash ". Dumper($tc_hash));
    $self->{tc_hash} = $tc_hash;
} ## end sub aggr_create

#-------------------------------------------------------------------------------
sub aggr_destroy {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $aggr_type = $tc_hash->{aggr_create}->{aggr};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};

    #destroy aggr
    my $result = RaidSqa_aggr_remove(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol, aggr_type=>$aggr_type);
    if(!$result) {
        die("$self->{test}: Aggr destroy was not successful for filer_vol $filer_vol");
    }
}

#-------------------------------------------------------------------------------
sub get_optional_storage_types {
    my $self = shift;
    my %args = validate(@_, {storage_type => {type=>SCALAR}});
    my @storage_type_groups = (
        ['ATA', 'SATA', 'BSAS'],
        ['FCAL', 'SAS'],
    );
    my @valid_storage_types = ('ATA', 'SATA', 'BSAS', 'FCAL', 'SAS');

    if(! grep /$args{storage_type}/,  @valid_storage_types) {
        die("$self->{test}: Input storage type is not one of valid storage types");
    }

    foreach my $ref (@storage_type_groups) {
        if(grep(/$args{storage_type}/, @$ref)) {
            return @$ref;
        }
    }
    die("$self->{test}: Optional storage type could not be found for $args{storage_type}");
}

#-------------------------------------------------------------------------------
sub is_cmode {
    my $self = shift;
    return ($self->{api_set_obj}->{mode} =~ /cmode/i) ? 1 : 0;
}

#---------------------------------------------------------------------
sub dummyvol {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};

    my %args = @_ ;
    my $dummyvol = $args{dummyvol};
    my $action = $args{action};
    my $unmount = $args{unmount};
    my $mount = $args{mount};
    my $mountpoint = $args{mountpoint};
    my $vserver = $args{vserver};

    if(defined($unmount) && !defined($mountpoint)) {
        die("$self->{test}: Mountpoint is not defined for unmount action");
    }

    if((defined($action) or defined($mount)) && !defined($dummyvol)) {
        die("$self->{test}: Mountvol is not defined");
    }

    #NOTE: sequence of if conditions are important here
    if(defined($unmount)) {
        umount($mountpoint);
    }

    if(defined $action) {
        if($action eq "create") {
            if(!RaidSqa_check_aggr_exist_anytype(api_set_obj=>$api_set_obj, filer_vol => $dummyvol)) {
                if (!RaidSqa_aggr_create(api_set_obj   => $api_set_obj,
                                            filer_vol     => $dummyvol,
                                            raid_type     => 'raid4',
                                            raid_size     => 2,
                                            aggr_type     => 1,
                                            disk_count    => 2,
                                            vserver       => $vserver)) {
                    die("$self->{test}: Aggr create failed to create $dummyvol");
                }
            }
        }

        if($action eq "destroy") {
             if (!RaidSqa_aggr_remove(api_set_obj   => $api_set_obj,
                                    filer_vol     => $dummyvol,
                                    aggr_type     => 1,
                                    disk_count    => 2,
                                    vserver       => $vserver)) {
                die("$self->{test}: Aggr remove failed to delete $dummyvol");
            }
        }
    }

    if(defined($mount)) {
        my $mountpoint = RaidSqa_aggr_mount(api_set_obj=>$api_set_obj, filer_vol=>$dummyvol);
        if(!$mountpoint) {
            die("$self->{test}: Aggr mount failed for volume $dummyvol")
        } else {
            return $mountpoint;
        }
    }
}

#-------------------------------------------------------------------------------
#Total levels 5, 1: min, 5max
#All dumpers should be moved to 5
#All variable values should be moved to 5
#1 will hold only log comments which provides additional information of step completion successfully
sub debug {
    my $self = shift;
    my $comment = shift;
    my $debug_level = shift;
    $debug_level = (defined($debug_level)) ? $debug_level : 1;
    if($self->{debug} && $self->{debug} >= $debug_level) {
        $Log->comment("DEBUG::$comment");
    }
}

#----------------------------------------------------------------------------------
sub step {
    my $self = shift;
    my @args = @_;
    # if(!defined($self->{test_object})) {
        # $self->{test_object} = (defined($main::Test)) ? $main::Test : $Log;
    # }
    # my $test_obj = $self->{test_object};
    # $test_obj->step(@args);
	$Log->step(@args);
}

#-------------------------------------------------------------------------------
sub disk_assign {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $disks = $self->{testdb}->{disk_unassign};
    $disks = $self->{disk_unassign} if(!defined($disks));

    foreach my $pool (0, 1) {
        if(scalar(@{$disks->{"pool".$pool}})) {
            my $result = RaidSqa_disk_assign(api_set_obj => $api_set_obj, disk_list=>$disks->{"pool".$pool}, pool=>$pool);
            if(!$result) {
                die("$self->{test}: Disks Could not re-assign to pool $pool");
            }
        }
    }
}

#-------------------------------------------------------------------------------
sub set_flags {
    my $self = shift;
    my $flags = $self->{tc_hash}->{test_flags};
    my $api_set_obj = $self->{api_set_obj};
    
    if(!ref($flags)) {
        $Log->warn("$self->{test}: Flags are not defined in tc_hash");
        return;
    }
    
    foreach my $flag (keys %$flags) {
        $flags->{$flag}->{original} = RaidSqa_get_setflag(api_set_obj => $api_set_obj, flag_name => $flag);
        RaidSqa_setflag(api_set_obj => $api_set_obj, flag_name => $flag, flag_value=> $flags->{$flag}->{required});
    }
    $self->{tc_hash}->{test_flags} = $flags;
}

#-------------------------------------------------------------------------------
sub reset_flags {
    my $self = shift;
    my $flags = $self->{tc_hash}->{test_flags};
    my $api_set_obj = $self->{api_set_obj};
    
    if(!ref($flags)) {
        $Log->warn("$self->{test}: Flags are not defined in tc_hash");
        return;
    }
    
    foreach my $flag (keys %$flags) {
        RaidSqa_setflag(api_set_obj => $api_set_obj, flag_name => $flag, flag_value=> $flags->{$flag}->{original});
    }
}

#-------------------------------------------------------------------------------
sub set_options {
    my $self = shift;
    my $options = $self->{tc_hash}->{test_options};
    my $api_set_obj = $self->{api_set_obj};
    
    if(!ref($options)) {
        $Log->warn("$self->{test}: options are not defined in tc_hash");
        return;
    }
    
    foreach my $option (keys %$options) {
        $options->{$option}->{original} = RaidSqa_get_options(api_set_obj => $api_set_obj, option_name => $option);
        RaidSqa_set_options(api_set_obj => $api_set_obj, option_name => $option, option_value=> $options->{$option}->{required});
    }
    $self->{tc_hash}->{test_options} = $options;
}

#-------------------------------------------------------------------------------
sub reset_options {
    my $self = shift;
    my $options = $self->{tc_hash}->{test_options};
    my $api_set_obj = $self->{api_set_obj};
    
    if(!ref($options)) {
        $Log->warn("$self->{test}: Options are not defined in tc_hash");
        return;
    }
    
    foreach my $option (keys %$options) {
        RaidSqa_set_options(api_set_obj => $api_set_obj, option_name => $option, option_value=> $options->{$option}->{original});
    }
}

#-------------------------------------------------------------------------------
sub set_aggr_options {
    my $self = shift;
    my %opts = validate(@_, {filer_vol=>{type=>SCALAR}});
    my $aggr_options = $self->{tc_hash}->{test_aggr_options};
    my $api_set_obj = $self->{api_set_obj};
    
    if(!ref($aggr_options)) {
        $Log->warn("$self->{test}: Aggr options are not defined in tc_hash");
        return;
    }
    
    foreach my $option (keys %$aggr_options) {
        $aggr_options->{$option}->{original} = RaidSqa_get_options(api_set_obj => $api_set_obj, option_name => $option);
        RaidSqa_aggr_options(api_set_obj => $api_set_obj, filer_vol=>$opts{filer_vol}, 
                             options => $option, value=> $aggr_options->{$option}->{required});
    }
    $self->{tc_hash}->{test_options} = $aggr_options;
}

#-------------------------------------------------------------------------------
sub reset_aggr_options {
    my $self = shift;
    my %opts = validate(@_, {filer_vol=>{type=>SCALAR}});
    my $aggr_options = $self->{tc_hash}->{test_options};
    my $api_set_obj = $self->{api_set_obj};
    
    if(!ref($aggr_options)) {
        $Log->warn("$self->{test}: Aggr options are not defined in tc_hash");
        return;
    }
    
    foreach my $option (keys %$aggr_options) {
        RaidSqa_aggr_options(api_set_obj => $api_set_obj, filer_vol=>$opts{filer_vol}, 
                             options => $option, value=> $aggr_options->{$option}->{original});
    }
}

#-------------------------------------------------------------------------------
sub get{
    my $self = shift;
    my $key_name = shift;
    my $key_mapping = $self->{testdb}->{key_mapping};

    if(!grep /$key_name/, keys %$key_mapping) {
        die("\n$self->{test}: Undefined key access: $key_name");
    }

    my $map = $key_mapping->{$key_name};
    my $value = eval($map);
    $self->debug("get $key_name: value = $value");
    return $value;
}

#-------------------------------------------------------------------------------
sub set {
    my $self = shift;
    my $key_name = shift;
    my $key_value = shift;
    my $key_mapping = $self->{testdb}->{key_mapping};

    if(!grep /$key_name/, keys %$key_mapping) {
        die("\n$self->{test}: Undefined key access: $key_name");
    }

    my $map = $key_mapping->{$key_name};
    my $eval_string = "$map = \"$key_value\"";
    eval($eval_string);
    die("$self->{test}: set value $key_value for key $key_name failed: $@") if($@);
}

#-------------------------------------------------------------------------------
sub get_hash {
    my $self = shift;
    my $key_name = shift;
    my $key_mapping = $self->{testdb}->{key_mapping};

    if(!grep /$key_name/, keys %$key_mapping) {
        die("\n$self->{test}: Undefined key access: $key_name");
    }

    my $map = $key_mapping->{$key_name};
    my $value = eval($map);
    if(ref $value !~ /HASH/) {
        die("$self->{test}: Key $key_name does not have hash reference: $value");
    } else {
        $self->debug("$self->{test}: get_hash, return value for $key_name: ".Dumper($value), 5) if();
    }
    return $value;
}

#-------------------------------------------------------------------------------
sub set_hash {
    my $self = shift;
    my $key_name = shift;
    my $key_value = shift;
    my $key_mapping = $self->{testdb}->{key_mapping};

    if(!grep /$key_name/, keys %$key_mapping) {
        die("\n$self->{test}: Undefined key access: $key_name");
    }

    if(uc ref $key_value ne 'HASH') {
        die("$self->{test}: Value to key $key_name should be a hash reference: ".$key_value);
    }

    my $map = $key_mapping->{$key_name};
    my %hash = %$key_value;
    my $eval_string = $map.' = \%hash';
    eval($eval_string);
    if($@) {
        die("$self->{test}: set_hash setting hash reference to key $key_name failed: $@");
    } else {
        return 1;
    }
}

#-------------------------------------------------------------------------------
sub get_object {
    my $self = shift;
    my $key_name = shift;
    my $key_mapping = $self->{testdb}->{key_mapping};

    if(!grep /$key_name/, keys %$key_mapping) {
        die("\n$self->{test}: Undefined key access: $key_name");
    }

    my $map = $key_mapping->{$key_name};
    my $value = eval($map);
    my $object_type = ref $value;
    if($object_type =~ /ARRAY|SCALAR|REF|CODE|HASH/) {
        die("$self->{test}: This key does not have object reference: $value");
    }
    return $value;
}

#----------------------------------------------------------------------------------
sub get_spares_by_rc_parameter {
    my $self = shift;
    my %args = validate(@_, {
                                parameter_name  => {type => SCALAR},
                                parameter_value => {type => SCALAR},
                                disk_list       => {type => ARRAYREF|UNDEF, default=>undef, optional=>1},
                                refresh         => {type => SCALAR, optional => 1, default => 0},
                            }
                        );
    if($args{refresh}) {
        $self->update_spare_info();
    }
    my $parameter_name = $args{parameter_name};
    my $parameter_value = $args{parameter_value};
    my $spares_info = $self->get_hash('test_spares_info');
    my @disk_list = (defined($args{disk_list})) ? @{$args{disk_list}} : keys %{$spares_info};    
    my @return_disk_list = ();
    foreach my $disk (@disk_list) {
        my $object = $self->get_rc_object(disk_name => $disk);
        my $value = $object->parameter($parameter_name);        
        if(defined($value) && ("$value" eq "$parameter_value")) {
            push(@return_disk_list, $disk);
        } 
        # else {
            # $self->debug("$self->{test}: Disk $disk does not have $parameter_name set to $parameter_value: ".$value, 5);
        # }
    }
    return @return_disk_list;
}

#----------------------------------------------------------------------------------
sub get_spares_by_filter {
    my $self = shift;
    my %args = validate(@_, {
                                storage_type    => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                                checksum_type   => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                                bps             => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                                rpm             => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                                disk_list       => {type=>ARRAYREF|UNDEF, default=>undef, optional=>1},
                                refresh         => {type=>SCALAR, optional=>1, default=>0},
                            });
    if($args{refresh}) {
        $self->update_spare_info();
    }

    my $spares_info = $self->get_hash('test_spares_info');
    my @disk_list = (defined($args{disk_list})) ? @{$args{disk_list}} : keys %{$spares_info};
    if(defined($args{storage_type})) {
        @disk_list = $self->get_spares_by_rc_parameter(
                        parameter_name   => 'disk_type',
                        parameter_value  => $args{storage_type},
                        disk_list        => \@disk_list
                    );
        $self->debug("$self->{test}: get_spares_by_filter: disks of storage type $args{storage_type}: @disk_list");
    }
    if(defined($args{checksum_type})) {
        @disk_list = $self->get_spares_by_rc_parameter(
                        parameter_name   => 'checksum_type',
                        parameter_value  => $args{checksum_type},
                        disk_list        => \@disk_list
                    );
        $self->debug("$self->{test}: get_spares_by_filter: disks of checksum type $args{checksum_type}: @disk_list");
    }

    if(defined($args{bps})) {
        @disk_list = $self->get_spares_by_rc_parameter(
                        parameter_name   => 'bps',
                        parameter_value  => $args{bps},
                        disk_list        => \@disk_list
                    );
        $self->debug("$self->{test}: get_spares_by_filter: disks of bps $args{bps}: @disk_list");
    }

    if(defined($args{rpm})) {
        @disk_list = $self->get_spares_by_rc_parameter(
                        parameter_name   => 'rpm',
                        parameter_value  => $args{rpm},
                        disk_list        => \@disk_list
                    );
        $self->debug("$self->{test}: get_spares_by_filter: disks of rpm $args{rpm}: @disk_list");
    }
    return @disk_list;
}

#----------------------------------------------------------------------------------
sub get_rc_object {
    my $self = shift;
    my %args = (@_, {
                    disk_name => {type=>SCALAR | UNDEF, default => undef, optional=>1},
                    filer_vol => {type=>SCALAR | UNDEF, default => undef, optional=>1},
                    group_name => {type=>SCALAR | UNDEF, default => undef, optional=>1},
                    plex_name => {type=>SCALAR | UNDEF, default => undef, optional=>1},
                });

    my @defined_input_params = grep defined($args{$_}), keys %args;
    my $count = scalar(@defined_input_params);
    die("$self->{test}: get_rc_object: At least one parameter should be passed to subroutine") if($count == 0);
    die("$self->{test}: get_rc_object: Only one parameter should be passed to subroutine") if($count > 1);

    my $api_set_obj = $self->{api_set_obj};
    my $raid_object = $self->get_object('test_rc_object');
    my $object = undef;
    if(defined($args{disk_name})) {
        $object = $raid_object->disk($args{disk_name});
        if(!defined($object)){
            my $disk_name = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_name=>$args{disk_name});
            $object = $raid_object->disk($disk_name);
        }
        $Log->warn("$self->{test}: Could not get object for disk $args{disk_name}") if(!defined($object));
    } elsif(defined($args{plex_name})) {
        $object = $raid_object->plex($args{plex_name}) if(defined($args{plex_name}));
        $Log->warn("$self->{test}: Could not get object for plex $args{plex_name}") if(!defined($object));
    } elsif(defined($args{group_name})) {
        $object = $raid_object->raid_group($args{group_name});
        $Log->warn("$self->{test}: Could not get object for group $args{group_name}") if(!defined($object));
    } elsif(defined($args{filer_vol})) {
        $object = $raid_object->aggregate($args{filer_vol});
        $object = $raid_object->vol($args{filer_vol}) if(!defined($object));
        $Log->warn("$self->{test}: Could not get object for aggr $args{filer_vol}") if(!defined($object));
    }
    #TODO:: Check for object
    return $object;
}

sub sort_disks_by_used_size {
    my $self = shift;
    my %opts = (@_, {
                    disk_list => {type=>HASHREF},
                    ascending => {type=>SCALAR | UNDEF, default => undef, optional=>1},
                });
    my @disk_list = @{$opts{disk_list}};
    my $ascending = $opts{ascending};
    my $disks_size_mapping = {};
    my @sorted_disks = ();
    
    foreach my $disk (@disk_list) {
        my $disk_obj = $self->get_rc_object(disk_name => $disk);
        if(!defined($disk_obj)) {next;}
        my $used_sz = $disk_obj->parameter('used_sz');
        if(!defined($disks_size_mapping->{$used_sz})) {
            $disks_size_mapping->{$used_sz} = [$disk];
        } else {
            push(@{$disks_size_mapping->{$used_sz}}, $disk);
        }        
    }
    
    if(defined($ascending)) {
        my @sizes = keys %$disks_size_mapping;
        @sizes = sort {$a <=> $b} @sizes;
        if($ascending == 0) { @sizes = reverse @sizes ;}
        foreach my $size (@sizes) {
            push(@sorted_disks, $size);
            push(@sorted_disks, $disks_size_mapping->{$size});            
        }
        my %size_disk_hash = @sorted_disks;
        $Log->comment("TODO::Remove: Dumper: ".Dumper(\%size_disk_hash));
        return \%size_disk_hash;
    } else {
        $Log->comment("TODO::Remove: Dumper disks_size_mapping: ".Dumper($disks_size_mapping));
        return $disks_size_mapping;
    }
}

#-------------------------------------------------------------------------------
sub update_rc_object {
    my $self = shift;
    my $rc_object = $self->get_object('test_rc_object');
    $rc_object->refresh();
    $rc_object->all_aggregates();
    $self->{testdb}->{rc_object} = $rc_object;
}

#-------------------------------------------------------------------------------
sub update_spare_info {
    my $self = shift;
    $self->update_rc_object();
    my $rc_obj = $self->get_hash('test_rc_object');
    my %spare_disks = $rc_obj->spare_disks();
    $self->set_hash(test_spares_info => \%spare_disks);
}

sub get_aggr_show_space {
    my $self = shift;
    my %opts = validate(@_, {
                    filer_vol => {type => SCALAR|UNDEF, default=>undef, optional=>1},
                    option    => {type => SCALAR|UNDEF, default=>undef, optional=>1},
                });
    my $filer_vol = $opts{filer_vol};
    my $option = $opts{option};
    my $aggr_show_space = {};
    
    if(defined($option) && $option !~ /^-[hkmgtb]/i ) {
        die("get_aggr_show_space: Error: Invalid option is provided: $option");
    }
    
    my $api_set_obj = $self->{api_set_obj};
    my $output = RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd=> "priv set -q diag; aggr show_space $option $filer_vol");
    my @lines  = split(/\r*\n/, $output);
    my @keys = ('Total space', 'WAFL reserve', 'Snap reserve', 'Usable space', 'BSR NVLOG', 'A-SIS', 'Smtape');
    
    foreach my $index (0..$#lines) {
        if($lines[$index] =~ /Aggregate (\w+)/){        
            my $aggr = $1;
            if(defined $filer_vol and $filer_vol ne $aggr) {next;}
            my $line = $lines[$index+3];
            $line =~ s/^\s+//;
            my @values = split(/\s+/, $line);
            foreach my $keyid (0..$#keys){
                $values[$keyid] =~ /(\d+)(.B)?/;
                $aggr_show_space->{$filer_vol}->{$keys[$keyid]} = $1;
            }
            if(defined $filer_vol and $filer_vol eq $aggr) {last;}
        }
    }
    return $aggr_show_space;
}


#==================================================================================
package AzcsErrors;

#Complier Directives
use strict;
use warnings;

# Test Directives
use vars qw(@ISA);
use TCD;
use Tharn qw ($Log);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidSqav3 qw(AUTOLOAD);
use RaidLW;
use RaidInfo;
use Data::Dumper;
use Utils qw(make_conn);

BEGIN {
    @ISA = qw(RaidAzcsUtils);
}
#---------------------------------------------------------------------
# new AzcsErrors(api_set_obj=>$api_set_obj, debug=>$debug)
sub new {
    my $self = shift if(scalar(@_) % 2);
    my %self = validate(@_, {
                                api_set_obj => {type => OBJECT, isa=> "NACL::APISet"},
                                debug       => {type => SCALAR, default=>0, optional=>1},
                            }
                        );
    $self = \%self;

    my $api_set_obj = $self->{api_set_obj};
    my $conntype = ($api_set_obj->{mode} eq "CMode") ? "#ssh" : "#rsh";

    if(defined($api_set_obj->{hostobj})) {
        $self->{conn_obj} = make_conn($api_set_obj->{hostobj}, $conntype);
    } elsif (defined $api_set_obj->{connection}) {
        $self->{conn_obj} = $api_set_obj->{connection};
    }

    if(!defined($self->{conn_obj})) {
        die("AzcsErrors::new: Could not create connection object");
    }

    #Define io_hash
    $self->{io_hash}->{wafl} = \&waflio;
    $self->{io_hash}->{reparity} = \&reparityio;
    $self->{io_hash}->{scrub} = \&scrubio;
	$self->{io_hash}->{recons} = \&reconsio;
	$self->{io_hash}->{lzresync} = \&lzresyncio;
	$self->{io_hash}->{l1resync} = \&l1resyncio;
	$self->{io_hash}->{verify} = \&verifyio;
	$self->{io_hash}->{sdc} = \&sdcio;
	$self->{io_hash}->{media_scrub} = \&media_scrubio;

    bless $self;
    return $self;
}

#---------------------------------------------------------------------
sub run_test {
        my $self = shift;
        my %args = @_ ;
        $self->{test} = $args{test};
        $self->{tc_hash} = $args{tc_hash};

        my $api_set_obj = $self->{api_set_obj};
        my $tc_hash = $self->{tc_hash};
        my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
        my $aggr_type = $tc_hash->{aggr_create}->{aggr};


        eval {
            $self->debug("Dumper:: tc_hash ". Dumper($tc_hash));
            $self->debug("Dumper:: tc_hash ". Dumper($self->{io_hash}));
            #==============================================
            $Log->step("Create a test aggr with name $filer_vol");
            $self->aggr_create();

            my $cksum_type = RaidSqa_get_aggr_cksum_type(api_set_obj=>$api_set_obj, filer_vol => $filer_vol);

            if($cksum_type !~ /advanced_zoned/) {
                die("$self->{test}: This test is applicable only for azcs aggr: $cksum_type");
            }

            #==============================================
            $Log->step("Create a test file with name $tc_hash->{filename}");
            $self->file_create();

            #==============================================
            $Log->step("Populate data to tc data structure");
            $self->populate_data();

			#==============================================
            $Log->step("Settig plex to required state");
            $self->set_plex_state();
			
            #==============================================
            $Log->step("Capture ems mark");
            $self->{tc_hash}->{ems_mark} = RaidSqa_ems_log_get_mark(api_set_obj=>$api_set_obj);

            #==============================================
            $Log->step("Inject error");
            $self->inject_error();

            #==============================================
            $Log->step("Set rg state to $tc_hash->{rg_state}");
            $self->set_rg_state();

            #=========================================================
            $Log->step("flush cache");
            $self->flush_cache();

            #=========================================================
            $Log->step("Run RAID IO of $tc_hash->{io_type} type on aggr $filer_vol");
			$self->debug("Dumper:: io_hash ". Dumper(($self->{io_hash})));
            $self->raid_io(io_type => $tc_hash->{io_type});
            sleep(30);

            #=========================================================
            $Log->step("Wait for zone scrub to complete");
            RaidSqa_wait_for_scrub_completion(api_set_obj => $api_set_obj, group_name => $tc_hash->{group_name});
			
			if ( $tc_hash->{zone_scrub_check} =~ /true|1/i) {
				$Log->step("Checking for zone scrub results");
				$self->check_zone_scrub();
			}

            #=========================================================
            $Log->step("Validate test results");
            $self->verify_results();

            #=========================================================
            #Wait for any recons if required
            $self->reset_rg_state();
			$self->reset_plex_state();

            #=========================================================
            $Log->step("delete test file");
            $self->file_delete();

            #=========================================================
            $Log->step("flush cache");
            $self->flush_cache();

            #=========================================================
            ## run wafliron to make aggregate clean
            $Log->step("Run wafliron on aggregate if it has been marked wafl inconsistent");
            RaidSqa_aggr_wafliron_start(api_set_obj=>$api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type);
            sleep(60);
        };

        if($@) {
            $self->{error} = $@;
            #=========================================================
            $Log->step("$self->{test}: cleanup");
            RaidSqa_aggr_wafliron_start(api_set_obj=>$api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type);
            sleep(60);
            $self->file_delete();
            $self->flush_cache();
            #=========================================================
            die("FAIL: $self->{error}");
        }
}

#---------------------------------------------------------------------
sub file_create {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};

    my $zone_count = $tc_hash->{zone_count};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
    my ($cmd, $file_block_max, $fbn, $inode);
    my $no_of_rgs = scalar(@{$tc_hash->{group_list}});
    my $file_size = int(5 * $no_of_rgs * ($zone_count/2)) + 1;
    $file_size = $file_size."M";
    my $counter = 0;

    my $result = RaidSqa_file_create(api_set_obj=>$api_set_obj, file_name=>$tc_hash->{filename}, file_size=>$file_size);
    if(!$result) {
        die("$self->{test}: File $tc_hash->{filename} of size $file_size could not be created");
    }

    #save md5sum of the file
    $tc_hash->{file_data}->{md5sum_before} = RaidSqa_get_md5sum(api_set_obj=>$api_set_obj, file_name=>$tc_hash->{filename});
    sleep(10);
    #==================================================================
    # Sync the data written to disk
    $cmd = "priv set test; sync; wafl flush cache";
    RaidSqa_filer_cmd (api_set_obj => $api_set_obj, cmd => $cmd);

    #==================================================================
    # Find out file size and get max blocks in file
    $cmd = "priv set test; slist $tc_hash->{filename}";
    my $output = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd => $cmd);
    my @slist_output = split(/\n/, $output);
    $self->debug("$self->{test}: slist output : @slist_output ");
    foreach my $temp_line (@slist_output) {
        if ($temp_line =~ /(\d+)\s+(\d+)\s+$tc_hash->{filename}/) {
            my $inode = $1 ; my $file_block_max = $2;
            $file_block_max = int($file_block_max / 4096);
            $self->debug("$self->{test}: inode: $inode, file_block_max: $file_block_max");
            $tc_hash->{inode} = $inode;
            $tc_hash->{file_block_max} = $file_block_max;
            last;
        }
    }

    #=====================================================================
    #Get FBN0 ref fbn
    my @fbn0_data = RaidSqa_get_file_block_info(api_set_obj=>$api_set_obj,
                                             filer_vol=>$filer_vol,
                                             file_name=>$tc_hash->{filename},
                                             block_number=>0);

    my $dumpblock = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>"priv set diag;dumpblock dbn -c $fbn0_data[0] $fbn0_data[1]");
    my $test_group = RaidSqa_get_disk_group_name(api_set_obj=>$api_set_obj, disk_name=>$fbn0_data[0]);
	my $test_plex = RaidSqa_get_disk_plex_name(api_set_obj=>$api_set_obj, disk_name=>$fbn0_data[0]);

    if(defined($test_group)){
        $tc_hash->{test_group} = $test_group;
    } else {
        die("$self->{test}: Test group could not be found");
    }
	if(defined($test_plex)){
        $tc_hash->{test_plex} = $test_plex;
    } else {
        die("$self->{test}: Test plex name could not be found");
    }
    #This might need fix
    if($dumpblock =~ /0x0020:\s+0x(.{8})/m) {
        my $fbn0 = $1;
        $tc_hash->{fbn0} = hex($fbn0);
        $self->debug("$self->{test}: FBN found is : $fbn0", 5);
        $self->debug("$self->{test}: FBN is : $tc_hash->{fbn0}", 5);
    }

    my $cksum_block = $fbn0_data[3];
    my @zone_list = ();
    if(defined($zone_count)) {
        foreach (0..14*$zone_count) {       #Max try 15
				#if($self->validate_zone_for_file_data(tc_hash=>$tc_hash, cksum_block=>$cksum_block)) {
                push(@zone_list, $cksum_block);
				#}
            last if(scalar(@zone_list) == $zone_count);
            $cksum_block += 64;
        }
    }
    if(scalar(@zone_list) != $zone_count) {
        die("$self->{test}: zone containing complete data could not be found");
    }
    $Log->comment("$self->{test}: Zone array : @zone_list");
    $tc_hash->{file_data}->{zone_list} = \@zone_list;
    $Log->comment("Dumper:: file_create: tc_hash ". Dumper($tc_hash));
    $self->{tc_hash} = $tc_hash;
}

#---------------------------------------------------------------------
sub file_delete {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $cmd = "priv set test; rm $tc_hash->{filename}";
    RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>$cmd);
}

#---------------------------------------------------------------------
sub set_rg_state {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $test_group = $tc_hash->{test_group};
    my $group_state = $tc_hash->{rg_state};

    $Log->comment("$self->{test}: Required fsm state for group is $group_state");
    return if(!defined($group_state) or uc $group_state eq "NORMAL");

    #TODO:: Add code for degraded recons
    #die("$self->{test}: No data structure for disks_state") if(!defined($tc_hash->{disks_state}));
if ( defined($tc_hash->{disks_state}) )
	{
    foreach my $disk_state_hash (@{$tc_hash->{disks_state}}) {
        my $disk_state = lc $disk_state_hash->{disk_state};
        if(!defined($disk_state)) {
            die("$self->{test}: disk_state is mandatory value in disks_state hash");
        }
        my $options = "" ; my $result;
        $options = "-i" if($disk_state eq "fail");
        if($disk_state =~ "fail") {
            $result = RaidSqa_disk_fail(api_set_obj=>$api_set_obj, disk_name => $disk_state_hash->{disk_name}, options => $options);
        } elsif ($disk_state eq "offline") {
            RaidSqa_disk_offline(api_set_obj=>$api_set_obj, disk_name=>$disk_state_hash->{disk_name});
        } elsif ($disk_state eq "custom") {
            my $err_info = $disk_state_hash->{err_info};
            #TODO:: Add code for fiji, cmds etc
        }
        if(!$result) {
            die("$self->{test}: Change disk state to $disk_state: Failed");
        }
    }
}
if ( defined($tc_hash->{disks_state_plex0}) )
	{
		foreach my $disk_state_hash (@{$tc_hash->{disks_state_plex0}}) {
			my $disk_state = lc $disk_state_hash->{disk_state};
			if(!defined($disk_state)) {
				die("$self->{test}: disk_state is mandatory value in disks_state_plex0 hash");
			}
			my $options = "" ; my $result;
			$options = "-i" if($disk_state eq "fail");
			if($disk_state =~ "fail") {
				$result = RaidSqa_disk_fail(api_set_obj=>$api_set_obj, disk_name => $disk_state_hash->{disk_name}, options => $options);
			} elsif ($disk_state eq "offline") {
				RaidSqa_disk_offline(api_set_obj=>$api_set_obj, disk_name=>$disk_state_hash->{disk_name});
			} elsif ($disk_state eq "custom") {
				my $err_info = $disk_state_hash->{err_info};
				#TODO:: Add code for fiji, cmds etc
			}
			if(!$result) {
				die("$self->{test}: Change disk state to $disk_state: Failed");
			}
		}
	}
	if ( defined($tc_hash->{disks_state_plex1}) )
	{
		foreach my $disk_state_hash (@{$tc_hash->{disks_state_plex1}}) {
			my $disk_state = lc $disk_state_hash->{disk_state};
			if(!defined($disk_state)) {
				die("$self->{test}: disk_state is mandatory value in disks_state_plex1 hash");
			}
			my $options = "" ; my $result;
			$options = "-i" if($disk_state eq "fail");
			if($disk_state =~ "fail") {
				$result = RaidSqa_disk_fail(api_set_obj=>$api_set_obj, disk_name => $disk_state_hash->{disk_name}, options => $options);
			} elsif ($disk_state eq "offline") {
				RaidSqa_disk_offline(api_set_obj=>$api_set_obj, disk_name=>$disk_state_hash->{disk_name});
			} elsif ($disk_state eq "custom") {
				my $err_info = $disk_state_hash->{err_info};
				#TODO:: Add code for fiji, cmds etc
			}
			if(!$result) {
				die("$self->{test}: Change disk state to $disk_state: Failed");
			}
		}
	}
    if($group_state =~ /RECONS/) {
        RaidSqa_setflag(api_set_obj=>$api_set_obj, flag_name=>"raid_disable_new_recons", flag_value=>0);
    }


    #============================================
    #Check FSM state of the filer
    my $result = RaidSqa_check_group_fsm_state(api_set_obj => $api_set_obj, group_name => $test_group, fsm_state=>uc $group_state);
    if(!$result) {
        die("$self->{test}: Group $test_group is not in $group_state");
    }
}
#---------------------------------------------------------------------
sub set_plex_state {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $test_plex = $tc_hash->{test_plex};
	my $io_type = $tc_hash->{io_type};
	my $mirror = $tc_hash->{aggr_create}->{mirror};
	my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
	
	my $offline_plex = '';

    $Log->comment("$self->{test}: Offlining plex other than $test_plex if required");
    return if (lc $io_type eq 'verify' or (!$mirror) );	
	return if (	defined($tc_hash->{stripes_plex1}) );
	
	my @plex_list = RaidSqa_get_plex_list(api_set_obj => $api_set_obj, filer_vol => $filer_vol); 
	
	foreach my $temp_plex (@plex_list) {
		next if ( $temp_plex eq $test_plex);
		$offline_plex = $temp_plex if ( $temp_plex ne $test_plex);
	}
	
	$Log->comment("$self->{test}: Offlining plex $offline_plex");
	
	if ( RaidSqa_plex_offline( api_set_obj => $api_set_obj, plex_name => $offline_plex) ){
		$Log->comment("$self->{test}: Successfully Offlined plex $offline_plex");	
		$tc_hash->{offline_plex} = $offline_plex;
	} else {
		die("$self->{test}: could not offline plex $offline_plex");	
	}	
}

#-----------------------------------------------------------------------------
sub reset_rg_state {
    my $self =  shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $test_group = $tc_hash->{test_group};
    my $group_state = $tc_hash->{rg_state};

    if($group_state =~ /DEGRAD/i) {
        RaidSqa_setflag(api_set_obj=>$api_set_obj, flag_name => "raid_disable_new_recons", flag_value => 0);
        sleep(15);
    }

    #If any of disks' ownership was removed, should be reassigned here
    #TODO::
    # my $reassign_disk = $tc_hash->{disks_removed};
    # foreach my $index ("0", "1")  {
        # if(defined ($reassign_disk->{"pool".$index})) {
            # my @disk_list = @{$reassign_disk->{"pool".$index}};
            # RaidSqa_disk_assign(api_set_obj=>$api_set_obj, disk_list=>\@disk_list, pool=>$index);
        # }
    # }
    RaidSqa_wait_for_reconstruction_completion(api_set_obj => $api_set_obj);
}
#-----------------------------------------------------------------------------
sub reset_plex_state {
    my $self =  shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
	my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
	my $aggr_type = $tc_hash->{aggr_create}->{aggr};
    my $test_plex = $tc_hash->{test_plex};
    my $offline_plex = $tc_hash->{offline_plex};
	
	if ( RaidSqa_check_aggr_mirror_state(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type, mirror_state => 'DEGRADED') )
	{
		$Log->comment("$self->{test} : aggr mirror state is DEGRADED : onlining plex");
		RaidSqa_plex_online(api_set_obj => $api_set_obj, plex_name => $offline_plex);	
		RaidSqa_wait_for_plex_resync_completion(api_set_obj => $api_set_obj, plex_name => $offline_plex, resync_level => '1');
	} else {
		$Log->comment("$self->{test} : aggr mirror state is  not DEGRADED : no need to offline plex");
	}
	

   
}


#-----------------------------------------------------------------------
sub aggr_destroy {
    my $self =  shift;
    my $api_set_obj = $self->{api_set_obj};
    my $filer_vol = $self->{tc_hash}->{aggr_create}->{filer_vol};
    my $aggr_type = $self->{tc_hash}->{aggr_create}->{aggr};
    my $result = RaidSqa_aggr_remove(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type=>$aggr_type);
    if(!$result) {
        die("$self->{test}: Could not remove aggr $filer_vol");
    }
}

#---------------------------------------------------------------------
sub aggr_reuse {
    my $self =  shift;
    my $api_set_obj = $self->{api_set_obj};
    my $filer_vol = $self->{tc_hash}->{aggr_create}->{filer_vol};
    my $aggr_type = $self->{tc_hash}->{aggr_create}->{aggr};

    #Offline aggr
    my $result = RaidSqa_aggr_offline(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type);
    if(!$result) {
        die("$self->{test}: Aggr offline failed for $filer_vol");
    }

    #Online aggr
    $result = RaidSqa_aggr_online(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type);
    if(!$result) {
        die("$self->{test}: Aggr online failed for $filer_vol");
    }
}

#---------------------------------------------------------------------
sub inject_error {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};

    my $error = {};
    my ($inject_disk, $inject_dbn);

if (exists($tc_hash->{stripes}) ){
		
    foreach my $stripe (@{$tc_hash->{stripes}}) {
        my $data_dbn = $stripe->{dbn};
        my $checksum_block = $stripe->{checksum_block};
        $self->dump_stripe(dbn=>$data_dbn, checksum_block=>$checksum_block, group_name=>$tc_hash->{test_group}) if($tc_hash->{dump_stripe});

        foreach my $err_info (@{$stripe->{errors}}) {
            my $error_type = $err_info->{error_type};
            my $disk_name = $err_info->{disk_name};
            my $disk_type = $err_info->{disk_type};
            my $dd_option = ($err_info->{inject_at} eq "data_block") ? "-c" : "";
            my $block_type = ($err_info->{inject_at} eq "data_block") ? "data" : "checksum";
            $inject_dbn = ($err_info->{inject_at} eq "data_block") ? $data_dbn : $checksum_block;

            $Log->comment("$self->{test}: Injecting $error_type error on $disk_name $inject_dbn");
            if($error_type eq "media") {
                my $result = RaidSqa_media_error_inject(api_set_obj => $api_set_obj, disk_name =>$disk_name, dbn => $inject_dbn);
                if(!$result) {
                    die("$self->{test}: Could not inject media error on $disk_name at $inject_dbn");
                }
            } elsif($error_type eq "checksum")  {
                my %error_info = %{$err_info->{ckse_info}};

                my $rlw = RaidLW->new(api_set_obj     => $api_set_obj,
                                      disk_name       => $disk_name,
                                      dbn             => $inject_dbn,
                                      filer_vol       => $filer_vol,
                                      disk_type       => $disk_type,
                                      disk_block_type => $block_type,
                                      dd_option       => $dd_option
                                      );
                $rlw->inject_checksum_error(disk_name   => $disk_name,
                                            dbn         => $inject_dbn,
                                            disk_type   => $disk_type,
                                            mountpoint  => $tc_hash->{mountpoint},
                                            inject_hash => \%error_info);

            } elsif($error_type eq "zero_block") {
                my $cmd = "priv set test; zeroblock dbn $disk_name $inject_dbn";
                my $output = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>$cmd);
                if($output =~ /zeroblock dbn: expected/im) {
                    die("$self->{test}: zero block error on $disk_name at $inject_dbn failed: $output");
                }
            }
            $Log->comment("$self->{test}: Inject $error_type error on $disk_name $inject_dbn : Sucessful");
        }
    }
}
if (exists($tc_hash->{stripes_plex0}) ){
		
		foreach my $stripe (@{$tc_hash->{stripes_plex0}}) {
			my $data_dbn = $stripe->{dbn};
			my $checksum_block = $stripe->{checksum_block};
			$self->dump_stripe(dbn=>$data_dbn, checksum_block=>$checksum_block, group_name=>$tc_hash->{test_group}) if($tc_hash->{dump_stripe});

			foreach my $err_info (@{$stripe->{errors}}) {
				my $error_type = $err_info->{error_type};
				my $disk_name = $err_info->{disk_name};
				my $disk_type = $err_info->{disk_type};
				my $dd_option = ($err_info->{inject_at} eq "data_block") ? "-c" : "";
				my $block_type = ($err_info->{inject_at} eq "data_block") ? "data" : "checksum";
				$inject_dbn = ($err_info->{inject_at} eq "data_block") ? $data_dbn : $checksum_block;

				$Log->comment("$self->{test}: Injecting $error_type error on $disk_name $inject_dbn");
				if($error_type eq "media") {
					my $result = RaidSqa_media_error_inject(api_set_obj => $api_set_obj, disk_name =>$disk_name, dbn => $inject_dbn);
					if(!$result) {
						die("$self->{test}: Could not inject media error on $disk_name at $inject_dbn");
					}
				} elsif($error_type eq "checksum")  {
					my %error_info = %{$err_info->{ckse_info}};

					my $rlw = RaidLW->new(api_set_obj     => $api_set_obj,
										  disk_name       => $disk_name,
										  dbn             => $inject_dbn,
										  filer_vol       => $filer_vol,
										  disk_type       => $disk_type,
										  disk_block_type => $block_type,
										  dd_option       => $dd_option
										  );
					$rlw->inject_checksum_error(disk_name   => $disk_name,
												dbn         => $inject_dbn,
												disk_type   => $disk_type,
												mountpoint  => $tc_hash->{mountpoint},
												inject_hash => \%error_info);

				} elsif($error_type eq "zero_block") {
					my $cmd = "priv set test; zeroblock dbn $disk_name $inject_dbn";
					my $output = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>$cmd);
					if($output =~ /zeroblock dbn: expected/im) {
						die("$self->{test}: zero block error on $disk_name at $inject_dbn failed: $output");
					}
				}
				$Log->comment("$self->{test}: Inject $error_type error on $disk_name $inject_dbn : Sucessful");
			}
		}
	}
	if (exists($tc_hash->{stripes_plex1}) ){
		
		
		
		## if ( scalar (@{$tc_hash->{stripes_plex1}}) == 0 )
		## if (@{$tc_hash->{stripes_plex1}} )
		
		foreach my $stripe (@{$tc_hash->{stripes_plex1}}) {
			my $data_dbn = $stripe->{dbn};
			my $checksum_block = $stripe->{checksum_block};
			$self->dump_stripe(dbn=>$data_dbn, checksum_block=>$checksum_block, group_name=>$tc_hash->{test_group}) if($tc_hash->{dump_stripe});

			foreach my $err_info (@{$stripe->{errors}}) {
				my $error_type = $err_info->{error_type};
				my $disk_name = $err_info->{disk_name};
				my $disk_type = $err_info->{disk_type};
				my $dd_option = ($err_info->{inject_at} eq "data_block") ? "-c" : "";
				my $block_type = ($err_info->{inject_at} eq "data_block") ? "data" : "checksum";
				$inject_dbn = ($err_info->{inject_at} eq "data_block") ? $data_dbn : $checksum_block;

				$Log->comment("$self->{test}: Injecting $error_type error on $disk_name $inject_dbn");
				if($error_type eq "media") {
					my $result = RaidSqa_media_error_inject(api_set_obj => $api_set_obj, disk_name =>$disk_name, dbn => $inject_dbn);
					if(!$result) {
						die("$self->{test}: Could not inject media error on $disk_name at $inject_dbn");
					}
				} elsif($error_type eq "checksum")  {
					my %error_info = %{$err_info->{ckse_info}};

					my $rlw = RaidLW->new(api_set_obj     => $api_set_obj,
										  disk_name       => $disk_name,
										  dbn             => $inject_dbn,
										  filer_vol       => $filer_vol,
										  disk_type       => $disk_type,
										  disk_block_type => $block_type,
										  dd_option       => $dd_option
										  );
					$rlw->inject_checksum_error(disk_name   => $disk_name,
												dbn         => $inject_dbn,
												disk_type   => $disk_type,
												mountpoint  => $tc_hash->{mountpoint},
												inject_hash => \%error_info);

				} elsif($error_type eq "zero_block") {
					my $cmd = "priv set test; zeroblock dbn $disk_name $inject_dbn";
					my $output = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>$cmd);
					if($output =~ /zeroblock dbn: expected/im) {
						die("$self->{test}: zero block error on $disk_name at $inject_dbn failed: $output");
					}
				}
				$Log->comment("$self->{test}: Inject $error_type error on $disk_name $inject_dbn : Sucessful");
			}
		}
	}
}
#---------------------------------------------------------------------
sub populate_data {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $raid_type = $tc_hash->{aggr_create}->{raid_type};
	my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};

    my @zone_list = @{$tc_hash->{file_data}->{zone_list}};

    my $rgdn_offset = ($raid_type eq "raid_dp") ? 1 : 0 ;
    my $test_group = $tc_hash->{test_group};
		my $test_group_plex1 = '';
	my @group_list = @{$tc_hash->{group_list}};	
	
	foreach my $temp_group (@group_list) {
		if ($temp_group eq $test_group) {
			next;
		} else {
			$test_group_plex1 = $temp_group;
		}			
	}
	
		
	
   
   	
    if(defined($tc_hash->{disks_state})) {
        my $disk_count = scalar(@{$tc_hash->{disks_state}}) - 1;
        foreach my $index (0..$disk_count) {
            my $disk_id = $tc_hash->{disks_state}->[$index]->{disk_id};
            my $disk_type = $tc_hash->{disks_state}->[$index]->{disk_type};
            my $rgdn;
            if($disk_type eq "data") {
                $rgdn = $disk_id + $rgdn_offset;
            } elsif( $disk_type eq "parity") {
                $rgdn = $rgdn_offset;
            } elsif($disk_type eq "dparity") {
                $rgdn = 0;
            }
            if(defined($tc_hash->{rgdn_info}->{$test_group}->{$rgdn})) {
                $tc_hash->{disks_state}->[$index]->{disk_name} = $tc_hash->{rgdn_info}->{$test_group}->{$rgdn};
            } else {
                die("$self->{test}: disk for rgdn $rgdn is not found in group $test_group");
            }
        }
    }
	if(defined($tc_hash->{disks_state_plex0})) {
        my $disk_count = scalar(@{$tc_hash->{disks_state_plex0}}) - 1;
        foreach my $index (0..$disk_count) {
            my $disk_id = $tc_hash->{disks_state_plex0}->[$index]->{disk_id};
            my $disk_type = $tc_hash->{disks_state_plex0}->[$index]->{disk_type};
            my $rgdn;
            if($disk_type eq "data") {
                $rgdn = $disk_id + $rgdn_offset;
            } elsif( $disk_type eq "parity") {
                $rgdn = $rgdn_offset;
            } elsif($disk_type eq "dparity") {
                $rgdn = 0;
            }
            if(defined($tc_hash->{rgdn_info}->{$test_group}->{$rgdn})) {
                $tc_hash->{disks_state_plex0}->[$index]->{disk_name} = $tc_hash->{rgdn_info}->{$test_group}->{$rgdn};
            } else {
                die("$self->{test}: disk for rgdn $rgdn is not found in group $test_group");
            }
        }
    }

	if(defined($tc_hash->{disks_state_plex1})) {
        my $disk_count = scalar(@{$tc_hash->{disks_state_plex1}}) - 1;
		my $test_plex = $tc_hash->{test_plex};
		
        foreach my $index (0..$disk_count) {
            my $disk_id = $tc_hash->{disks_state_plex1}->[$index]->{disk_id};
            my $disk_type = $tc_hash->{disks_state_plex1}->[$index]->{disk_type};
            my $rgdn;
            if($disk_type eq "data") {
                $rgdn = $disk_id + $rgdn_offset;
            } elsif( $disk_type eq "parity") {
                $rgdn = $rgdn_offset;
            } elsif($disk_type eq "dparity") {
                $rgdn = 0;
            }
            if(defined($tc_hash->{rgdn_info}->{$test_group_plex1}->{$rgdn})) {
                $tc_hash->{disks_state_plex1}->[$index]->{disk_name} = $tc_hash->{rgdn_info}->{$test_group_plex1}->{$rgdn};
            } else {
                die("$self->{test}: disk for rgdn $rgdn is not found in group $test_group_plex1");
            }
        }
    }
	
	my @error_disks = ();
	my @failed_disks = ();
	
	
	

    if(defined($tc_hash->{stripes})) {
        my $stripe_count = scalar(@{$tc_hash->{stripes}}) - 1;
        foreach my $index (0..$stripe_count) {
            my $inject_hash = $tc_hash->{stripes}->[$index];
            my $stripe_offset = $inject_hash->{stripe_offset};
            my $zone_id = $inject_hash->{zone_id};
            $tc_hash->{stripes}->[$index]->{checksum_block} = $zone_list[$zone_id];
            $tc_hash->{stripes}->[$index]->{dbn} = $zone_list[$zone_id] - 32 + $stripe_offset;

            my $err_index = 0;
            foreach my $err_info (@{$inject_hash->{errors}}) {
                my $disk_id = $err_info->{disk_id};
                my $disk_type = $err_info->{disk_type};
                my $rgdn;
                if($disk_type eq "data") {
                    $rgdn = $disk_id + $rgdn_offset;
                } elsif( $disk_type eq "parity") {
                    $rgdn = $rgdn_offset;
                } elsif($disk_type eq "dparity") {
                    $rgdn = 0;
                }
                if($disk_type=~/parity/) {
                    if(defined($err_info->{ckse_info}->{gen_count}) &&
                       !defined($err_info->{ckse_info}->{gen_count})) {
                       die("$self->{test}: Disk rgdn is required to inject gen_count error on parity disks");
                    } elsif(defined($err_info->{ckse_info}->{gen_count})) {
                        $tc_hash->{stripes}->[$index]->{errors}->[$err_index]->{ckse_info}->{disk_rgdn} += $rgdn_offset;
                    }
                }

                my $disk_name = $tc_hash->{rgdn_info}->{$test_group}->{$rgdn};
                if(defined($disk_name)) {
                    $tc_hash->{stripes}->[$index]->{errors}->[$err_index]->{disk_name} = $disk_name;
                } else {
                    die("$self->{test}: disk for rgdn $rgdn is not found in group $test_group");
                }
                $err_index++;
				push (@error_disks, $disk_name);
            }
        }
		$tc_hash->{rgdn_info}->{error_disks} = \@error_disks;
    }
if(defined($tc_hash->{stripes_plex0})) {
        my $stripe_count = scalar(@{$tc_hash->{stripes_plex0}}) - 1;
        foreach my $index (0..$stripe_count) {
            my $inject_hash = $tc_hash->{stripes_plex0}->[$index];
            my $stripe_offset = $inject_hash->{stripe_offset};
            my $zone_id = $inject_hash->{zone_id};
            $tc_hash->{stripes_plex0}->[$index]->{checksum_block} = $zone_list[$zone_id];
            $tc_hash->{stripes_plex0}->[$index]->{dbn} = $zone_list[$zone_id] - 32 + $stripe_offset;

            my $err_index = 0;
            foreach my $err_info (@{$inject_hash->{errors}}) {
                my $disk_id = $err_info->{disk_id};
                my $disk_type = $err_info->{disk_type};
								
                my $rgdn;
                if($disk_type eq "data") {
                    $rgdn = $disk_id + $rgdn_offset;
                } elsif( $disk_type eq "parity") {
                    $rgdn = $rgdn_offset;
                } elsif($disk_type eq "dparity") {
                    $rgdn = 0;
                }
                if($disk_type=~/parity/) {
                    if(defined($err_info->{ckse_info}->{gen_count}) &&
                       !defined($err_info->{ckse_info}->{gen_count})) {
                       die("$self->{test}: Disk rgdn is required to inject gen_count error on parity disks");
                    } elsif(defined($err_info->{ckse_info}->{gen_count})) {
                        $tc_hash->{stripes_plex0}->[$index]->{errors}->[$err_index]->{ckse_info}->{disk_rgdn} += $rgdn_offset;
                    }
                }

                my $disk_name = $tc_hash->{rgdn_info}->{$test_group}->{$rgdn};
                if(defined($disk_name)) {
                    $tc_hash->{stripes_plex0}->[$index]->{errors}->[$err_index]->{disk_name} = $disk_name;
                } else {
                    die("$self->{test}: disk for rgdn $rgdn is not found in group $test_group");
                }
                $err_index++;
				push (@error_disks, $disk_name);
            }
        }
		$tc_hash->{rgdn_info}->{error_disks} = \@error_disks;
    }
	
	logcomment("Aka : reached populate : checking for stripes");
    if(defined($tc_hash->{stripes_plex1})) {
		
        my $stripe_count = scalar(@{$tc_hash->{stripes_plex1}}) - 1;
        foreach my $index (0..$stripe_count) {
            my $inject_hash = $tc_hash->{stripes_plex1}->[$index];
            my $stripe_offset = $inject_hash->{stripe_offset};
            my $zone_id = $inject_hash->{zone_id};
            $tc_hash->{stripes_plex1}->[$index]->{checksum_block} = $zone_list[$zone_id];
            $tc_hash->{stripes_plex1}->[$index]->{dbn} = $zone_list[$zone_id] - 32 + $stripe_offset;

            my $err_index = 0;
            foreach my $err_info (@{$inject_hash->{errors}}) {
                my $disk_id = $err_info->{disk_id};
                my $disk_type = $err_info->{disk_type};
				
                my $rgdn;
                if($disk_type eq "data") {
                    $rgdn = $disk_id + $rgdn_offset;
                } elsif( $disk_type eq "parity") {
                    $rgdn = $rgdn_offset;
                } elsif($disk_type eq "dparity") {
                    $rgdn = 0;
                }
                if($disk_type=~/parity/) {
                    if(defined($err_info->{ckse_info}->{gen_count}) &&
                       !defined($err_info->{ckse_info}->{gen_count})) {
                       die("$self->{test}: Disk rgdn is required to inject gen_count error on parity disks");
                    } elsif(defined($err_info->{ckse_info}->{gen_count})) {
                        $tc_hash->{stripes_plex1}->[$index]->{errors}->[$err_index]->{ckse_info}->{disk_rgdn} += $rgdn_offset;
                    }
                }

                my $disk_name = $tc_hash->{rgdn_info}->{$test_group_plex1}->{$rgdn};
                if(defined($disk_name)) {
                    $tc_hash->{stripes_plex1}->[$index]->{errors}->[$err_index]->{disk_name} = $disk_name;
                } else {
                    die("$self->{test}: disk for rgdn $rgdn is not found in group $test_group_plex1");
                }
                $err_index++;
				push (@error_disks, $disk_name);
            }
        }
		$tc_hash->{rgdn_info}->{error_disks} = \@error_disks;
    }
	
	logcomment("Aka : reached populate : checking for ems");
	if(defined($tc_hash->{stripes})) {
		if(defined($tc_hash->{ems}) ){
			foreach my $event (keys %{$tc_hash->{ems}}) {
				if($tc_hash->{ems}->{$event}->{key}=~ /blockNum|dbn/) {
					my @stripe_ids = @{$tc_hash->{ems}->{$event}->{stripes}};
					my $ems_at = $tc_hash->{ems}->{$event}->{value};
					foreach my $index (0..$#stripe_ids){
						my $key = ( $ems_at eq "data_block") ? 'dbn' : 'checksum_block';
						my $ems_block = $tc_hash->{stripes}->[$index]->{$key};
						$tc_hash->{ems}->{$event}->{stripes}->[$index] = $ems_block;
					}
				} elsif($tc_hash->{ems}->{$event}->{key} =~ /fileid/) {
					$tc_hash->{ems}->{$event}->{value} = $tc_hash->{inode};
				} elsif($tc_hash->{ems}->{$event}->{key} =~ /vol/) {
					$tc_hash->{ems}->{$event}->{value} = $tc_hash->{aggr_create}->{filer_vol};
				} elsif($tc_hash->{ems}->{$event}->{key} =~ /rg/) {
					$tc_hash->{ems}->{$event}->{value} = $tc_hash->{test_group};
				}
			}
		}
	}
	if(defined($tc_hash->{stripes_plex0})) {
		if(defined($tc_hash->{ems}) ){
            $Log->comment("tc hash of ems". Dumper $tc_hash->{ems});
			foreach my $event (keys %{$tc_hash->{ems}}) {
				if($tc_hash->{ems}->{$event}->{key}=~ /blockNum|dbn/) {
					my @stripe_ids = @{$tc_hash->{ems}->{$event}->{stripes_plex0}} if(defined($tc_hash->{ems}->{$event}->{stripes_plex0}));
					my $ems_at = $tc_hash->{ems}->{$event}->{value};
					foreach my $index (0..$#stripe_ids){
						my $key = ( $ems_at eq "data_block") ? 'dbn' : 'checksum_block';
						my $ems_block = $tc_hash->{stripes_plex0}->[$index]->{$key};
						$tc_hash->{ems}->{$event}->{stripes_plex0}->[$index] = $ems_block;
					}
				} elsif($tc_hash->{ems}->{$event}->{key} =~ /fileid/) {
					$tc_hash->{ems}->{$event}->{value} = $tc_hash->{inode};
				} elsif($tc_hash->{ems}->{$event}->{key} =~ /vol/) {
					$tc_hash->{ems}->{$event}->{value} = $tc_hash->{aggr_create}->{filer_vol};
				} elsif($tc_hash->{ems}->{$event}->{key} =~ /rg/) {
					$tc_hash->{ems}->{$event}->{value} = $tc_hash->{test_group};
				}
			}
		}
	}

	if(defined($tc_hash->{stripes_plex1})) {
		if(defined($tc_hash->{ems}) ){
			foreach my $event (keys %{$tc_hash->{ems}}) {
				if($tc_hash->{ems}->{$event}->{key}=~ /blockNum|dbn/) {
					my @stripe_ids = @{$tc_hash->{ems}->{$event}->{stripes_plex1}} if(defined($tc_hash->{ems}->{$event}->{stripes_plex1}));
					my $ems_at = $tc_hash->{ems}->{$event}->{value};
					foreach my $index (0..$#stripe_ids){
						my $key = ( $ems_at eq "data_block") ? 'dbn' : 'checksum_block';
						my $ems_block = $tc_hash->{stripes_plex1}->[$index]->{$key};
						$tc_hash->{ems}->{$event}->{stripes_plex1}->[$index] = $ems_block;
					}
				} elsif($tc_hash->{ems}->{$event}->{key} =~ /fileid/) {
					$tc_hash->{ems}->{$event}->{value} = $tc_hash->{inode};
				} elsif($tc_hash->{ems}->{$event}->{key} =~ /vol/) {
					$tc_hash->{ems}->{$event}->{value} = $tc_hash->{aggr_create}->{filer_vol};
				} elsif($tc_hash->{ems}->{$event}->{key} =~ /rg/) {
					$tc_hash->{ems}->{$event}->{value} = $tc_hash->{test_group};
				}
			}
		}
	}
	
	logcomment("Aka : reached populate : done : dumping hash");
    $self->debug("Dumper:: populate_data::tc_hash ". Dumper($tc_hash));
    #$self->{tc_hash} = $tc_hash;
}

#---------------------------------------------------------------------
sub dump_stripe {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};

    my %args = @_;
    my $dbn = $args{dbn};
    my $checksum_block = $args{checksum_block};
    my $group_name = $args{group_name};

    my @disk_list = RaidSqa_get_disk_list(api_set_obj=>$api_set_obj, group_name => $group_name);

    $Log->comment("$self->{test}: Dump stripe: $dbn");
    foreach my $disk (@disk_list) {
        RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>"priv set diag; dumpblock dbn -c $disk $dbn");
    }
}

#---------------------------------------------------------------------
sub verify_results {
    my $self =shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};

    $Log->comment("$self->{test}: Verify field ems: ".Dumper($tc_hash->{ems}));
    $self->check_ems_events();

    $self->check_md5sum();

    #Check for pseudo bad and unverified blocks
    foreach my $stripe (@{$tc_hash->{stripes}}) {
		my $dbn = $stripe->{dbn};
        my $checksum_block = $stripe->{checksum_block};
        $self->dump_stripe(dbn => $dbn, group_name=>$tc_hash->{test_group}) if($tc_hash->{dump_stripe});
        $self->dump_stripe(dbn => $checksum_block, group_name=>$tc_hash->{test_group}) if($tc_hash->{dump_stripe});
        foreach my $err_info (@{$stripe->{errors}}) {
            my $disk_name = $err_info->{disk_name};
            my $bit_set = $err_info->{bit_set};
            $Log->comment("$self->{test}: Verify field bit_set: $bit_set");
            if(defined($bit_set)) {
                my $result = 1;
                if($bit_set eq "pseudo_bad"){
                    $result  = RaidSqa_check_disk_block_is_pseudo(api_set_obj => $api_set_obj, disk_name => $disk_name, dbn => $dbn);
                } elsif ($bit_set eq "unverified"){
                    $result  = RaidSqa_check_disk_block_is_unverified(api_set_obj => $api_set_obj, disk_name => $disk_name, dbn => $dbn);
                } elsif ($bit_set eq "dump"){
                    my $cmd = "priv set test; dumpblock dbn -c $disk_name $dbn";
                    RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd => $cmd);
                } elsif($bit_set eq "none") {
                    my $pseudo_bit = RaidSqa_check_disk_block_is_pseudo(api_set_obj => $api_set_obj,
                                                                        disk_name => $disk_name, dbn => $dbn);
                    my $unverified_bit = RaidSqa_check_disk_block_is_unverified(api_set_obj => $api_set_obj,
                                                                                disk_name => $disk_name, dbn => $dbn);

                    if($pseudo_bit or $unverified_bit) {
                        die("$self->{test}: pseudo bad or unverified bit is not set to 0");
                    }
                } else {
                    die("$self->{test}: Unrecognised value set for field bit_set");
                }
                if(!$result) {
                    die("$self->{test}: block $dbn @ $disk_name is not marked as $bit_set");
                }
            }
        }
    }
	
	foreach my $stripe (@{$tc_hash->{stripes_plex0}}) {
        my $dbn = $stripe->{dbn};
        my $checksum_block = $stripe->{checksum_block};
        $self->dump_stripe(dbn => $dbn, group_name=>$tc_hash->{test_group}) if($tc_hash->{dump_stripe});
        $self->dump_stripe(dbn => $checksum_block, group_name=>$tc_hash->{test_group}) if($tc_hash->{dump_stripe});
        foreach my $err_info (@{$stripe->{errors}}) {
            my $disk_name = $err_info->{disk_name};
            my $bit_set = $err_info->{bit_set};
            $Log->comment("$self->{test}: Verify field bit_set: $bit_set");
            if(defined($bit_set)) {
                my $result = 1;
                if($bit_set eq "pseudo_bad"){
                    $result  = RaidSqa_check_disk_block_is_pseudo(api_set_obj => $api_set_obj, disk_name => $disk_name, dbn => $dbn);
                } elsif ($bit_set eq "unverified"){
                    $result  = RaidSqa_check_disk_block_is_unverified(api_set_obj => $api_set_obj, disk_name => $disk_name, dbn => $dbn);
                } elsif ($bit_set eq "dump"){
                    my $cmd = "priv set test; dumpblock dbn -c $disk_name $dbn";
                    RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd => $cmd);
                } elsif($bit_set eq "none") {
                    my $pseudo_bit = RaidSqa_check_disk_block_is_pseudo(api_set_obj => $api_set_obj,
                                                                        disk_name => $disk_name, dbn => $dbn);
                    my $unverified_bit = RaidSqa_check_disk_block_is_unverified(api_set_obj => $api_set_obj,
                                                                                disk_name => $disk_name, dbn => $dbn);

                    if($pseudo_bit or $unverified_bit) {
                        die("$self->{test}: pseudo bad or unverified bit is not set to 0");
                    }
                } else {
                    die("$self->{test}: Unrecognised value set for field bit_set");
                }
                if(!$result) {
                    die("$self->{test}: block $dbn @ $disk_name is not marked as $bit_set");
                }
            }
        }
    }
	
	foreach my $stripe (@{$tc_hash->{stripes_plex1}}) {
        my $dbn = $stripe->{dbn};
        my $checksum_block = $stripe->{checksum_block};
        $self->dump_stripe(dbn => $dbn, group_name=>$tc_hash->{test_group}) if($tc_hash->{dump_stripe});
        $self->dump_stripe(dbn => $checksum_block, group_name=>$tc_hash->{test_group}) if($tc_hash->{dump_stripe});
        foreach my $err_info (@{$stripe->{errors}}) {
            my $disk_name = $err_info->{disk_name};
            my $bit_set = $err_info->{bit_set};
            $Log->comment("$self->{test}: Verify field bit_set: $bit_set");
            if(defined($bit_set)) {
                my $result = 1;
                if($bit_set eq "pseudo_bad"){
                    $result  = RaidSqa_check_disk_block_is_pseudo(api_set_obj => $api_set_obj, disk_name => $disk_name, dbn => $dbn);
                } elsif ($bit_set eq "unverified"){
                    $result  = RaidSqa_check_disk_block_is_unverified(api_set_obj => $api_set_obj, disk_name => $disk_name, dbn => $dbn);
                } elsif ($bit_set eq "dump"){
                    my $cmd = "priv set test; dumpblock dbn -c $disk_name $dbn";
                    RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd => $cmd);
                } elsif($bit_set eq "none") {
                    my $pseudo_bit = RaidSqa_check_disk_block_is_pseudo(api_set_obj => $api_set_obj,
                                                                        disk_name => $disk_name, dbn => $dbn);
                    my $unverified_bit = RaidSqa_check_disk_block_is_unverified(api_set_obj => $api_set_obj,
                                                                                disk_name => $disk_name, dbn => $dbn);

                    if($pseudo_bit or $unverified_bit) {
                        die("$self->{test}: pseudo bad or unverified bit is not set to 0");
                    }
                } else {
                    die("$self->{test}: Unrecognised value set for field bit_set");
                }
                if(!$result) {
                    die("$self->{test}: block $dbn @ $disk_name is not marked as $bit_set");
                }
            }
        }
    }

    $Log->comment("$self->{test}: Verify field raid_config: ".Dumper($tc_hash->{raid_config}));
    $self->check_raid_config_flags();
}

#---------------------------------------------------------------------
sub check_md5sum {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};

    if(exists $tc_hash->{file_data}->{md5sum_match}) {
        my $test = $tc_hash->{file_data}->{md5sum_match};
        #save md5sum of the file
        $tc_hash->{file_data}->{md5sum_after} =
            RaidSqa_get_md5sum(api_set_obj=>$api_set_obj, file_name=>$tc_hash->{filename});
        my $before = $tc_hash->{file_data}->{md5sum_before};
        my $after = $tc_hash->{file_data}->{md5sum_after};

        if($test == 1 &&  $before ne $after) {
            $Log->comment("$self->{test}: md5sum: before: $before, after: $after");
            die("$self->{test}: md5sum of test file does not match after test");
        }
    }
}

#---------------------------------------------------------------------
sub check_raid_config_flags {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $rc_hash = $self->{tc_hash}->{raid_config};

    #Create raidInfo object
    $Log->comment("$self->{test}: Create raid_config object:");
    my $robj = new RaidInfo($self->{conn_obj});
    die ("$self->{test}: Raid_config object create: failed") if(!defined($robj));

    foreach my $type ("aggr", "plex", "group") {
        foreach my $name (keys %{$rc_hash->{$type}}) {
            my $obj = $self->get_raid_obj(raid_obj=>$robj, type=>$type, name=>$name);
            foreach my $parameter (keys %{$rc_hash->{$type}->{$name}}) {
                my $act_value = $obj->parameter($parameter);
                my $exp_value = $rc_hash->{$type}->{$name}->{$parameter};
                if($self->compare(exp_value=>$exp_value, act_value=>$act_value)){
                    $Log->comment("$self->{test}: $type parameter $parameter is set to $act_value");
                } else {
                    die("$self->{test}: $type parameter $parameter is not set to $exp_value, actual value: $act_value");
                }
            }
        }
    }
}

#---------------------------------------------------------------------
sub check_ems_events {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $ems_hash =  $self->{tc_hash}->{ems};
    my $ems_mark =  $self->{tc_hash}->{ems_mark};

    my $ref_events = RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj, timestamp => $ems_mark);
    my $ref_ids = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj, timestamp => $ems_mark);

    #=====================================================================
    $Log->step("$self->{test}: Print Events occured during the test");
    foreach my $event (keys %{$ref_events}){
        $Log->comment("$self->{test}: Event $event has occured during test");
    }

    foreach my $event (keys %{$ems_hash}) {
        if(!defined($ref_events->{$event})){
            $Log->warn("$self->{test}: Event $event did not occur for this test");
        } else {
            my $ids = $ref_events->{$event};
            my $key = $ems_hash->{$event}->{key};
            my @values = ();
            if($key =~ /blocknum|dbn/i) {
                @values = @{$ems_hash->{$event}->{stripes}} if (defined $ems_hash->{$event}->{stripes});
            } elsif($key =~ /rg|vol|fileid/i) {
                @values = ($ems_hash->{$event}->{value});
            }

            foreach my $value (@values) {
                my $found = 0;
                foreach my $id (@{$ids}) {
                    $self->debug("$self->{test}: Exp:: $value, Act:: $ref_ids->{$id}->{$key}", 5);
                    if ($ref_ids->{$id}->{$key} eq "$value") {
                        $found++;
                    }
                }
                if($found == 0) {
                    $Log->warn("$self->{test}: Event $event occured $found times for value $value");
                } else {
                    $Log->comment("$self->{test}: Event $event occured $found times for value $value");
                }
            }
        }
    }
	foreach my $event (keys %{$ems_hash}) {
        if(!defined($ref_events->{$event})){
            $Log->warn("$self->{test}: Event $event did not occur for this test");
        } else {
            my $ids = $ref_events->{$event};
            my $key = $ems_hash->{$event}->{key};
            my @values = ();
            if($key =~ /blocknum|dbn/i) {
                @values = @{$ems_hash->{$event}->{stripes_plex0}} if (defined $ems_hash->{$event}->{stripes_plex0});
            } elsif($key =~ /rg|vol|fileid/i) {
                @values = ($ems_hash->{$event}->{value});
            }

            foreach my $value (@values) {
                my $found = 0;
                foreach my $id (@{$ids}) {
                    $self->debug("$self->{test}: Exp:: $value, Act:: $ref_ids->{$id}->{$key}", 5);
                    if ($ref_ids->{$id}->{$key} eq "$value") {
                        $found++;
                    }
                }
                if($found == 0) {
                    $Log->warn("$self->{test}: Event $event occured $found times for value $value");
                } else {
                    $Log->comment("$self->{test}: Event $event occured $found times for value $value");
                }
            }
        }
    }
	foreach my $event (keys %{$ems_hash}) {
        if(!defined($ref_events->{$event})){
            $Log->warn("$self->{test}: Event $event did not occur for this test");
        } else {
            my $ids = $ref_events->{$event};
            my $key = $ems_hash->{$event}->{key};
            my @values = ();
            if($key =~ /blocknum|dbn/i) {
                @values = @{$ems_hash->{$event}->{stripes_plex1}} if (defined $ems_hash->{$event}->{stripes_plex1});
            } elsif($key =~ /rg|vol|fileid/i) {
                @values = ($ems_hash->{$event}->{value});
            }

            foreach my $value (@values) {
                my $found = 0;
                foreach my $id (@{$ids}) {
                    $self->debug("$self->{test}: Exp:: $value, Act:: $ref_ids->{$id}->{$key}", 5);
                    if ($ref_ids->{$id}->{$key} eq "$value") {
                        $found++;
                    }
                }
                if($found == 0) {
                    $Log->warn("$self->{test}: Event $event occured $found times for value $value");
                } else {
                    $Log->comment("$self->{test}: Event $event occured $found times for value $value");
                }
            }
        }
    }
}

#---------------------------------------------------------------------
sub validate_zone_for_file_data {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};

    my %opts = @_ ;
    my $cksum_block = $opts{cksum_block} ;
    my $fbn0 = $tc_hash->{fbn0};
    my $inode = $tc_hash->{inode};
    my $test_group  = $tc_hash->{test_group};

    my $dbnl = $cksum_block - 32;
    my $dbnh = $cksum_block + 31;
    my @data_disks = RaidSqa_get_disk_list_by_raid_type(api_set_obj=>$api_set_obj, group_name=>$test_group, raid_type=>"data");

    $self->debug("$self->{test}: FBN 0: $fbn0");
    $self->debug("$self->{test}: cksum_dbn: $cksum_block dbnl: $dbnl dbnh: $dbnh");

    my ($fbnl, $fbnh);
    foreach my $data_disk (@data_disks) {
        my $dumpblockl = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>"priv set test; dumpblock dbn -c $data_disk $dbnl");
        if($dumpblockl =~ /0x0020: 0x(.{8})/m) {
            $fbnl = hex($1);
            $Log->comment("$self->{test}: FBN L: $fbnl");
            if($fbnl != 0) {
                $fbnl = $fbnl - $fbn0;
                return if ($fbnl < 0) ;
                $Log->comment("$self->{test}: FBN l: $fbnl");

                my $info = RaidSqa_get_dumpblock_info(api_set_obj=>$api_set_obj,filer_vol=> $filer_vol, fileid=>$inode, fbn=>$fbnl);
                if($info) {
                    if($info->{dbn} ne $dbnl) {
                        $Log->comment("$self->{test}: $dbnl does not belong to file $tc_hash->{filename}");
                        return 0;
                    }
                }
            } else {
                return 0;
            }
        }

        my $dumpblockh = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>"priv set test; dumpblock dbn -c $data_disk $dbnh");
        if($dumpblockh =~ /0x0020: 0x(.{8})/m) {
            $fbnh = hex($1);
            $Log->comment("$self->{test}: FBN H: $fbnh");
            if($fbnh != 0) {
                $fbnh = $fbnh - $fbn0;
                return if ($fbnh < 0);
                my $info = RaidSqa_get_dumpblock_info(api_set_obj=>$api_set_obj,filer_vol=>$filer_vol, fileid=>$inode, fbn=>$fbnh);
                if($info) {
                    if($info->{dbn} ne $dbnh) {
                        $Log->comment("$self->{test}: $dbnh does not belong to file $tc_hash->{filename}");
                        return 0;
                    }
                }
            } else {
                return 0;
            }
        }
    }
    $Log->comment("$self->{test}: zone with checksum dbn $cksum_block is completely filled with file data");
    return 1;
}

#---------------------------------------------------------------------
sub flush_cache {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};

    RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd => "priv set test;wafl flush cache");
    RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd => "priv set test;raid_flush_zcsv2_cache $filer_vol");
}

#---------------------------------------------------------------------
sub raid_io {
    my $self = shift;
    my %args = validate(@_,{io_type => {type => SCALAR}});
    my $io_type = $args{io_type};
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};

    $self->debug("$self->{test}: Executing io $io_type");

    if(defined $self->{io_hash}->{$io_type} ) {
        return $self->{io_hash}->{$io_type}->($self);
    } else {
        die ("$self->{test}: Unrecongnized RAID IO type: $io_type");
    }
}

#---------------------------------------------------------------------
sub waflio {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};

    $Log->step("Executing WAFL IO on file $tc_hash->{filename}");
    RaidSqa_file_nfs_read(api_set_obj => $api_set_obj, file_name=>$tc_hash->{filename});
}

#---------------------------------------------------------------------
sub reparityio {
    my $self = shift;
    my $tc_hash = $self->{tc_hash};
    my $api_set_obj = $self->{api_set_obj};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
    my $aggr_type = $tc_hash->{aggr_create}->{aggr};

    # Destroy the volume
    $Log->comment("$self->{test}: Destroy and undestroy vol/aggr $filer_vol");
    $Log->comment("$self->{test}: Destroy vol/aggr $filer_vol");
    RaidSqa_aggr_remove(api_set_obj => $api_set_obj,
                        filer_vol => $filer_vol,
                        aggr_type => $aggr_type);

    if (RaidSqa_check_aggr_exist(api_set_obj      => $api_set_obj,
                                 filer_vol => $filer_vol,
                                 aggr_type => $aggr_type)
      ) {
        die("$self->{test}: Volume $filer_vol could not be removed");
    }
    sleep(60);

    #===============================================================
    # Undestroy the destroyed volume for reparity to start
    $Log->comment("$self->{test}: Undestroy vol/aggr $filer_vol");
    if (!RaidSqa_aggr_undestroy(api_set_obj => $api_set_obj,, filer_vol => $filer_vol, aggr_type => $aggr_type)) {
        die("$self->{test}: Failed to undestroy the volume $filer_vol");
    }
    sleep(5);

    #===============================================================
    # Check for reparity on vol/aggr filer_vol
    my @group_list = @{$self->{tc_hash}->{group_list}};
    $Log->comment("$self->{test}: Check if reparity is running on groups @group_list");
    foreach my $group (@group_list) {
        if (!RaidSqa_check_group_reparity_in_progress(api_set_obj => $api_set_obj, group_name => $group)){
            die("$self->{test}: Failed as reparity is not running on group $group");
        }
    }

    #Wait for reparity to get over
    RaidSqa_wait_for_reparity_completion(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol);
}

#---------------------------------------------------------------------
sub scrubio {
    my $self = shift;
    my $tc_hash = $self->{tc_hash};
    my $api_set_obj = $self->{api_set_obj};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
    my $test_group = $tc_hash->{test_group};

    my $result = RaidSqa_aggr_scrub_start(api_set_obj=>$api_set_obj, group_name=>$test_group);
    if($result) {
        $Log->comment("$self->{test}: Scrub start on raid group $test_group: ok");
    } else {
        $Log->comment("$self->{test}: Scrub start on raid group $test_group: failed");
    }

    sleep(30);  #to avoid more raid_config logs
    RaidSqa_wait_for_scrub_completion(api_set_obj => $api_set_obj,  group_name => $test_group);
    my $scrub_report = RaidSqa_get_scrub_report_by_group(api_set_obj=>$api_set_obj, group_name=>$test_group);
    $self->debug("$self->{test}: Scrub report: ".Dumper($scrub_report));
}

#---------------------------------------------------------------------
sub reconsio {
    my $self = shift;
    my $tc_hash = $self->{tc_hash};
    my $api_set_obj = $self->{api_set_obj};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
    my $test_group = $tc_hash->{test_group};
	my $test_plex = $tc_hash->{test_plex};
	
	if ($tc_hash->{rg_state} =~ /normal/i) {		
	
		my @unused_disks = ();
		$self->debug("Dumper:: populate_data::tc_hash ". Dumper($tc_hash));
		
		my @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj, group_name => $test_group);
		$Log->comment("Aka : reached reconsio :  disk list : @disk_list");
		
		my @error_disks = @{$tc_hash->{rgdn_info}->{error_disks}};
		$Log->comment("Aka : reached reconsio :  error disk list : @error_disks");
		
		$Log->comment("Aka : reached reconsio :  unused disk list : @unused_disks");
		my @failed_disk_list = ();
		my @offlined_disk_list = ();
		my @group_list =  RaidSqa_get_group_list(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
		foreach my $rg_name (@group_list){
	        $rg_name =~ /\/$filer_vol\/(plex\d)\/(rg\d+)/;    
	        my $plex = $1;
	        my $rg = $2;
			@failed_disk_list =    RaidSqa_get_disk_list_by_fsm_state(api_set_obj => $api_set_obj, fsm_state => "OFFLINE", group_name =>  $rg_name );
			@offlined_disk_list =  RaidSqa_get_disk_list_by_fsm_state(api_set_obj => $api_set_obj, fsm_state => "FAILED", group_name =>  $rg_name);
			$Log->comment("Aka : reached reconsio : rg_name ; $rg_name, group : $rg, plex : $plex, failed_disk_list = @failed_disk_list, offlined_disks : @offlined_disk_list");
		}
		@failed_disk_list =    RaidSqa_get_disk_list_by_fsm_state(api_set_obj => $api_set_obj, fsm_state => "OFFLINE", group_name =>  $test_group );
		@offlined_disk_list =  RaidSqa_get_disk_list_by_fsm_state(api_set_obj => $api_set_obj, fsm_state => "FAILED", group_name =>  $test_group);
		
		my @available_disks = ();
		foreach my $disk (@disk_list) {
			if (grep /$disk/ , @error_disks) {
				next;
			} else {
					push (@unused_disks, $disk);
			}			
		}
		foreach my $disk (@unused_disks) {
			if ( (grep /$disk/ , @failed_disk_list) || (grep /$disk/ , @offlined_disk_list) ){
				next;
			} else {
					push (@available_disks, $disk);
			}			
		}
	    $Log->comment("Aka : reached reconsio :failed_disk_list = @failed_disk_list, offlined_disks : @offlined_disk_list error_disks; @error_disks, available_disks: @available_disks");
		$Log->comment("$self->{test}: reconsio : failing disk : $available_disks[0]");
		if ( RaidSqa_disk_fail(api_set_obj => $api_set_obj, disk_name => $available_disks[0] , options => "-i") ) {
			$Log->comment("$self->{test}: reconsio : successfully failed disk : $available_disks[0]");
		} else {
			die("$self->{test}: reconsio : could not fail disk : $available_disks[0]");
		}
	}	
	RaidSqa_setflag(api_set_obj => $api_set_obj, flag_name => 'raid_disable_new_recons', flag_value => '0');
	my $timeout_RECONS = 120;
	my $group_fsm_state = RaidSqa_get_group_fsm_state(api_set_obj => $api_set_obj, group_name => $test_group);
	while ( ($group_fsm_state !~ /recons/i) && ($timeout_RECONS > 0) ){
		sleep 2;		
		$timeout_RECONS -= 10;
		$group_fsm_state = RaidSqa_get_group_fsm_state(api_set_obj => $api_set_obj, group_name => $test_group);
		
	}
	$group_fsm_state = RaidSqa_get_group_fsm_state(api_set_obj => $api_set_obj, group_name => $test_group);
	if ($group_fsm_state =~ /recons/i ) {
		$Log->comment("$self->{test}: reconsio : test group $test_group, recons state : $group_fsm_state");
		RaidSqa_wait_for_reconstruction_completion(api_set_obj => $api_set_obj,  group_name => $test_group);
	} else {
		die("$self->{test}: reconsio : RECONS did not start");
	}
}





#---------------------------------------------------------------------
sub sdcio {
    my $self = shift;
    my $tc_hash = $self->{tc_hash};
    my $api_set_obj = $self->{api_set_obj};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
    my $test_group = $tc_hash->{test_group};
	
	$self->debug("Dumper:: populate_data::tc_hash ". Dumper($tc_hash));
    
}

#---------------------------------------------------------------------
sub media_scrubio {
    my $self = shift;
    my $tc_hash = $self->{tc_hash};
    my $api_set_obj = $self->{api_set_obj};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
    my $test_group = $tc_hash->{test_group};
	
	$self->debug("Dumper:: populate_data::tc_hash ". Dumper($tc_hash));
	$Log->comment("$self->{test}: Running media scrub on aggregate");
	
	if (RaidSqa_aggr_media_scrub_start(api_set_obj => $api_set_obj, filer_vol => $filer_vol)
	) {
		$Log->comment("$self->{test}: successfully started media scrub on $filer_vol");
	} else {
		die(" $self->{test}: could not start media scrub on $filer_vol");
	}
	
	my $output = RaidSqa_aggr_media_scrub_status(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
	
	my $timeout_SCRUB = 300;
	
	while ( ($output =~ /\.* is \d+\% complete/i) && ($timeout_SCRUB > 0) ){
		sleep 2;
		$output = RaidSqa_aggr_media_scrub_status(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
		$timeout_SCRUB -= 5;
	}
	
	$Log->comment("$self->{test}: completed media scrub on $filer_vol");
    
}

#---------------------------------------------------------------------
sub verifyio {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
	my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
	my $test_plex  = $tc_hash->{test_plex};
	my $aggr_type = $tc_hash->{aggr_create}->{aggr};

    $Log->step("Executing VERIFY IO on aggr $filer_vol");
	
	die (" $self->{test}: aggr mirror state is not NORMAL : cannot start verify ") 
		if (!RaidSqa_check_aggr_mirror_state(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type, mirror_state => 'NORMAL') );
		
	
	if (RaidSqa_aggr_verify_start( api_set_obj => $api_set_obj, filer_vol => $filer_vol) ){
		  $Log->comment("$self->{test}: verify IO started on vol/aggr $filer_vol");
	} else {
		 die("$self->{test}: verify IO could not be started on $filer_vol");
	}
	
	RaidSqa_wait_for_verify_completion( api_set_obj => $api_set_obj, filer_vol => $filer_vol);   
}

#---------------------------------------------------------------------
sub lzresyncio {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
	my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
	my $aggr_type = $tc_hash->{aggr_create}->{aggr};
	my $test_plex = $tc_hash->{test_plex};
	my $new_plex = '';
	
    $Log->step("Executing LZRESYNC IO on aggr $filer_vol");
	die (" $self->{test}: aggr mirror state is not UNMIRRORED : cannot start lzresync ") 
		if (!RaidSqa_check_aggr_mirror_state(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type, mirror_state => 'UNMIRRORED') );
		
	#die (" $self->{test}: aggr mirror state is not NORMAL : cannot start lzresync ") 
	#	if (!RaidSqa_check_aggr_mirror_state(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type, mirror_state => 'UNMIRRORED') );

	if ( RaidSqa_aggr_mirror(api_set_obj => $api_set_obj, filer_vol => $filer_vol) ){
		$Log->comment(" $self->{test}: successfully executed aggr mirror command; lzresync will start");
		my @plex_list = RaidSqa_get_plex_list(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
		foreach my $temp_plex (@plex_list) {
			next if ($temp_plex eq $test_plex);
			$new_plex = $temp_plex if ($temp_plex ne $test_plex);
		}
	} else {
		die ("$self->{test}: aggr mirror could not be executed for $filer_vol");
	}
	
	if ( RaidSqa_check_plex_resyncing(api_set_obj => $api_set_obj, plex_name => $new_plex, resync_level => 0 )) {
		$Log->comment(" $self->{test}: plex $new_plex resync level 0 is in progress");		
	}	 
	RaidSqa_wait_for_plex_resync_completion(api_set_obj => $api_set_obj, plex_name => $new_plex, resync_level => '0');
}


#---------------------------------------------------------------------
sub l1resyncio {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
	my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
	my $aggr_type = $tc_hash->{aggr_create}->{aggr};
	my $test_plex = $tc_hash->{test_plex};
	my $offline_plex = $tc_hash->{offline_plex};
	
	
    $Log->step("$self->{test}:Executing L1RESYNC IO on aggr $filer_vol");
	
	die (" $self->{test}: aggr mirror state is UNMIRRORED : cannot start l1resync on a non-sync aggregate") 
		if (RaidSqa_check_aggr_mirror_state(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type, mirror_state => 'UNMIRRORED') );
		
	#die (" $self->{test}: aggr mirror state is not DEGRADED : cannot start l1resync ") 
	#	if (!RaidSqa_check_aggr_mirror_state(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type, mirror_state => 'DEGRADED') );

	if (!defined($tc_hash->{stripes_plex1}) ) {
		die (" $self->{test}: there is no offline plex defined : cannot start L1 resync") if (!defined($offline_plex));
	} elsif (defined($tc_hash->{stripes_plex1}) ) { 		
		my @plex_list = RaidSqa_get_plex_list(api_set_obj => $api_set_obj, filer_vol => $filer_vol); 	
		foreach my $temp_plex (@plex_list) {
			next if ( $temp_plex eq $test_plex);
			$offline_plex = $temp_plex if ( $temp_plex ne $test_plex);
		}		
		$Log->comment("$self->{test}: Offlining plex $offline_plex");		
		if ( RaidSqa_plex_offline( api_set_obj => $api_set_obj, plex_name => $offline_plex) ){
			$Log->comment("$self->{test}: Successfully Offlined plex $offline_plex");	
			$tc_hash->{offline_plex} = $offline_plex;
		} else {
			die("$self->{test}: could not offline plex $offline_plex");	
		}	
		sleep 30;
	} else {
		$Log->comment("$self->{test}: sub l1resyncio");
	}
	
	$Log->comment("$self->{test}: onlining plex $offline_plex to start L1 RESYNC");	
	if(! RaidSqa_plex_online(api_set_obj => $api_set_obj, plex_name => $offline_plex) ) {
		die (" $self->{test}: could not online plex $offline_plex");
	} else {
		$Log->comment("$self->{test}: plex $offline_plex is made online: L1 RESYNC will start");
	}
	
	if ( RaidSqa_check_plex_resyncing(api_set_obj => $api_set_obj, plex_name => $offline_plex, resync_level => 1 )) {
		$Log->comment(" $self->{test}: plex $offline_plex resync level 0 is in progress");
		RaidSqa_wait_for_plex_resync_completion(api_set_obj => $api_set_obj, plex_name => $offline_plex, resync_level => '1');		
	}	 
}


sub update_disk_info {
	my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
	my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
	
	
	
	
}
sub get_raid_obj {
    my $self = shift;
    #Put access control code
    my %args = @_;      #being internal code, there wont be any validation
    my $raid_obj = $args{raid_obj};
    my $type     = $args{type};
    my $name     = $args{name};

    my $child_obj = undef;
    if($type =~ /aggr/i) {
        $child_obj = $raid_obj->aggregate($name);
    } elsif($type =~ /plex/i) {
        $child_obj = $raid_obj->plex($name);
    } elsif($type =~ /group/i) {
        $child_obj = $raid_obj->raid_group($name);
    } elsif($type =~ /disk/i) {
        $child_obj = $raid_obj->disk($name);
    }

    if(!defined($child_obj)) {
        die("$self->{test}:$type with name $name does not exist");
    }

    return $child_obj;
}

#---------------------------------------------------------------------
sub compare {
    my $args = shift;
    #Put access control code
    my %args = @_;      #being internal code, there wont be any validation
    my $act_value = $args{act_value};
    my $exp_value = $args{exp_value};

    my $eval_string = "0";
    my $result = undef;    my $string = undef;
    if($exp_value =~ /([<=>]{1,2})(\d+)$/) {
        $string = "($act_value $1 $2) ? 1 : 0";
    } else {
        $string = "($exp_value eq $act_value) ? 1 : 0";
    }
    return eval($string);
}

#---------------------------------------------------------------------
sub get_tc_data {
    my $self = shift;
    return $self->{tc_hash};
}

#---------------------------------------------------------------------
sub set_tc_data {
    my $self = shift;
    my %args = validate(@_, {tc_hash => {type => HASHREF}});
    #Code to validate tc data structure
    $self->{tc_hash} = $args{tc_hash};
}

sub check_zone_scrub {
	my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};  
    my $ems_mark =  $self->{tc_hash}->{ems_mark};
	
	my @stripe_list = @{$tc_hash->{stripes}};
	my @checksum_dbn_list =  ();
	foreach my $stripe_list (@stripe_list) {
		my $checksum_dbn = $stripe_list->{checksum_block};
		push(@checksum_dbn_list, $checksum_dbn);
	}
	logcomment("AZCSUTILS : EMS DUMP : checksum_dbn_list : @checksum_dbn_list");
	my @begin_dbns = RaidSqa_ems_get_value_by_key(api_set_obj => $api_set_obj, eventtype => "raid.rg.scrub.start.zone", key => 'begin' , timestamp => $ems_mark);
	logcomment("AZCSUTILS : EMS DUMP : return value : @begin_dbns");
	my @end_dbns = RaidSqa_ems_get_value_by_key(api_set_obj => $api_set_obj, eventtype => "raid.rg.scrub.start.zone", key => 'end' , timestamp => $ems_mark);
	logcomment("AZCSUTILS : EMS DUMP : return value : @end_dbns");	
	
	foreach my $checksum_dbn (@checksum_dbn_list) {
		my $check_dbn = $checksum_dbn - 32;
		logcomment("AZCSUTILS : EMS DUMP :zone scrub begin message for zone $checksum_dbn is found") if ( grep /^$check_dbn$/, @begin_dbns);
		$check_dbn = $checksum_dbn + 31;
		logcomment("AZCSUTILS : EMS DUMP :zone scrub end message  for zone $checksum_dbn is found") if ( grep /^$check_dbn$/, @end_dbns);
	}	
}
#====================================================================
#                           POD section                             =
#====================================================================
# All documentation can be found here (where is it, ohh yet to be added)
#TODO
#==================================================================================

package AzcsRbnDbn;

use vars qw(@ISA);
use strict;
use warnings;
use Tharn qw($Log);
use RaidSqav3 qw(AUTOLOAD);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidLW;
use RaidInfo;
use Data::Dumper;
use Utils;

BEGIN {
    @ISA = qw(RaidAzcsUtils);
}

#-------------------------------------------------------------------------------
# new AzcsRbnDbn(api_set_obj=>$api_set_obj, tc_hash => $tc_hash, test=>RAID_ZCSv2_rbn_dbn_01)
sub new {
    my $self = shift if(scalar(@_) % 2);
    my %self = validate(@_, {
                                api_set_obj => {type => OBJECT, isa=> "NACL::APISet"},
                                debug       => {type => SCALAR, default=>0, optional=>1},
                            });
    $self = \%self;
    my $api_set_obj = $self->{api_set_obj};
    my $host;
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        my $conntype = ($host->{hosttype} =~ /ng/) ? "#ssh" : "#rsh";
        $self->{conn_obj} = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $self->{conn_obj} = $api_set_obj->{connection};
    }
    bless $self;
    return $self;
}

#-------------------------------------------------------------------------------
sub run_test {
    my $self = shift;
    my %args = validate(@_, {
                                tc_hash     => {type => HASHREF},
                                test        => {type => SCALAR, default=>"AzcsRbnDbn", optional=>1},
                            });
    $self->{tc_hash} = $args{tc_hash};
    $self->{test} = $args{test};
    my $api_set_obj = $self->{api_set_obj};

    my $tc_hash = $self->{tc_hash};
    my @steps = qw(aggr_create disk_prefail disk_fail disk_replace aggr_mirror create_file aggr_scrub aggr_destroy);
    $self->{spares_disk_sizes} = $self->get_spares_disk_sizes();
    $Log->comment("$self->{test}: Dumper::TC_HASH ".Dumper($self->{tc_hash}));

    eval {
        $Log->step("$self->{test}: Clear all fiji filters from previous tests");
        RaidSqa_fiji_clear(api_set_obj=>$api_set_obj, fiji_point=>'all');
        foreach my $step (@steps) {
            if(defined($tc_hash->{$step})) {
                $Log->step("$self->{test}: Start execution of step $step");
                no strict "refs";
                $self->$step();
                use strict "refs";
                $Log->step("$self->{test}: End execution of step $step");
            }
        }

        if($tc_hash->{aggr_create}->{aggr_cleanup}) {
            $self->aggr_destroy();
        }
    };

    if($@) {
        $self->{error} = $@;
        if($tc_hash->{aggr_create}->{aggr_cleanup}) {
            $self->aggr_destroy();
        }
        $Log->comment("$self->{test}: Dumper::TC_HASH ".Dumper($self->{tc_hash}));
        die($self->{error});
    }
}

#-------------------------------------------------------------------------------
sub aggr_create {
    my $self = shift;
    my $tc_hash = $self->{tc_hash};
    my $api_set_obj = $self->{api_set_obj};
    my $aggr_create = $tc_hash->{aggr_create};
    my $match = ($aggr_create->{abort_operation}) ? "equal" : "less";

    #create an aggr
    my @aggr_disk_list = ();
    my %aggr_create_args = (
                            api_set_obj   => $self->{api_set_obj},
                            filer_vol     => $aggr_create->{filer_vol},
                            raid_type     => $aggr_create->{raid_type},
                            raid_size     => $aggr_create->{raid_size},
                            aggr_type     => $aggr_create->{aggr_type},
                            aggr_bits     => $aggr_create->{aggr_bits},
                            mirror        => $aggr_create->{mirror},
                            checksum_type => $aggr_create->{checksum_type},
                            vserver       => $aggr_create->{vserver},
    );

    #NOTE: If aggr_reuse is set to 1, it can not be used to compare vol disks with spares
    # as we are not aware of sizes when disks were spares, hence return here
    if($self->aggr_exists()){
        if($aggr_create->{aggr_reuse} == 1) {
            if($aggr_create->{check_vol_disks}) {
                die("$self->{test}:If aggr_reuse is set to 1, it can not be used to compare vol disks with spares");
            }
            if($self->aggr_ok()) {
                return;
            } else {
                $self->aggr_destroy();
                #save spares disk sizes again
                $self->{spares_disk_sizes} = $self->get_spares_disk_sizes();
            }
        } else {
            $self->aggr_destroy();
            #save spares disk sizes again
            $self->{spares_disk_sizes} = $self->get_spares_disk_sizes();
        }
    }

    if($aggr_create->{abort_operation} && exists($aggr_create->{disk_count}))  {
        my @spare_list = RaidSqa_get_disk_list_by_checksum_type(api_set_obj=>$api_set_obj,
                                                                spare_only=>"yes",
                                                                checksum_type=>$aggr_create->{checksum_type});
        if(scalar(@spare_list) < $aggr_create->{disk_count}) {
            die("$self->{test}: There are not enough spares for test: current_count: ".scalar(@spare_list));
        }

        my @disk_list = @spare_list[0..$aggr_create->{raid_size}-1];
        $aggr_create_args{disk_list} = \@disk_list;
        @aggr_disk_list = (@disk_list);
        if($aggr_create->{mirror}) {
            my @disk_list1 = @spare_list[$aggr_create->{raid_size}..(2*$aggr_create->{raid_size})-1];
            $aggr_create_args{disk_list1} = \@disk_list1;
            push (@aggr_disk_list, @disk_list1);
        }
    } elsif(exists($aggr_create->{disk_count})) {
        $aggr_create_args{disk_count} = $aggr_create->{disk_count};
        $self->check_spares();
    } elsif (exists($aggr_create->{disk_list})) {
        $aggr_create_args{disk_list} = $aggr_create->{disk_list};
        $aggr_create_args{disk_list1} = $aggr_create->{disk_list1} if(defined($aggr_create->{disk_list1}));
    }

    $Log->comment("$self->{test}: Creating aggr with below args: ".Dumper(\%aggr_create_args));
    if($aggr_create->{abort_operation}) {
        #set fiji to reboot filer
        $self->set_fiji_filter();
        eval {
            my $result = RaidSqa_aggr_create(%aggr_create_args);
            die("$self->{test}: Aggr with name $aggr_create->{filer_vol} could not be created") if(!$result);
        };
        if($@) {
            $aggr_create->{error} = $@;
        } else {
            $Log->warn("$self->{test}: Filer reboot: failed");
        }

        RaidSqa_transit_to(api_set_obj=>$api_set_obj, state_name=>"UP");
        sleep(5);

        if($self->aggr_exists()) {
            die("$self->{test}: Filer did not reboot before aggr creation");
        }
    } else {
        my $result = RaidSqa_aggr_create(%aggr_create_args);
        if(!$result) {
            die("$self->{test}: Aggr with name $aggr_create->{filer_vol} could not be created");
        }
        @aggr_disk_list = RaidSqa_get_disk_list(api_set_obj=>$api_set_obj, filer_vol=>$aggr_create->{filer_vol});
    }

    if($aggr_create->{check_vol_disks}) {
        if(!scalar(@aggr_disk_list)) {
            die("$self->{test}: No disks available in aggr $aggr_create->{filer_vol}");
        }

        #get size of vol disks
        my $aggr_disks_sizes = $self->get_disk_size(disk_list => \@aggr_disk_list);
        my $spares_disk_sizes = $self->{spares_disk_sizes};
        $self->debug("$self->{test}: Dumper aggr_disk_sizes: ".Dumper($aggr_disks_sizes));
        $self->debug("$self->{test}: Dumper spares_disk_sizes: ".Dumper($spares_disk_sizes));

        #compare size
        foreach my $disk (@aggr_disk_list) {
            my $aggr_size = $aggr_disks_sizes->{$disk};
            my $spare_size = $spares_disk_sizes->{$disk};
            die("$self->{test}: Could not find size for aggr disk $disk") if(!defined($aggr_size));
            die("$self->{test}: Could not find size for spares disk $disk") if(!defined($spare_size));
            $self->compare_disk_size(size1 => $aggr_size, size2 => $spare_size, match=>$match, disk_name=>$disk);
        }
    }
}

#-------------------------------------------------------------------------------
sub aggr_disk_list{
my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $aggr_create = $self->{tc_hash}->{aggr_create};
    my $aggr_add = $self->{tc_hash}->{aggr_add};
    my $filer_vol = $aggr_create->{filer_vol};
    my $spares_disk_sizes = $self->{spares_disk_sizes};
	#{tc_hash}->{aggr_disk_list};
	
    my @aggr_disk_list = $self->{tc_hash}->{aggr_disk_list};
 @aggr_disk_list = RaidSqa_get_disk_list(api_set_obj=>$api_set_obj, filer_vol=>$aggr_create->{filer_vol});
 return @aggr_disk_list;
 }

#-------------------------------------------------------------------------------
sub aggr_add {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $aggr_create = $self->{tc_hash}->{aggr_create};
    my $aggr_add = $self->{tc_hash}->{aggr_add};
    my $filer_vol = $aggr_create->{filer_vol};
    my $spares_disk_sizes = $self->{spares_disk_sizes};

    #Get current size and increment raidsize by 1
    my $current_raid_size = RaidSqa_get_aggr_options(api_set_obj=>$api_set_obj, filer_vol => $filer_vol, options=>"raidsize");
    RaidSqa_aggr_options(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol, options=>"raidsize", value=> $current_raid_size + 1);
    my $checksum_type = (defined $aggr_create->{checksum_type}) ? $aggr_create->{checksum_type} :
                        RaidSqa_get_aggr_cksum_type(api_set_obj=>$api_set_obj, filer_vol => $filer_vol);

    #Find a disk to be added to aggregate
    my ($disk_pool0, $disk_pool1);
    my @spare_disks = RaidSqa_get_disk_list_by_checksum_type(api_set_obj=>$api_set_obj, checksum_type =>$checksum_type, spare_only=>'yes');
    if(!scalar(@spare_disks)) {
        die("$self->{test}: Spare disks with checksum type '$checksum_type' does not exist");
    }

    my %aggr_add_args = (api_set_obj => $api_set_obj, filer_vol => $filer_vol);
    if(!$aggr_create->{mirror}) {
        $disk_pool0 = shift @spare_disks;
        $self->{tc_hash}->{disk_add}->{disk_name0} = $disk_pool0;
        $aggr_add_args{disk_list} = [$disk_pool0];
        $self->{aggr_add_disk_list} = [$disk_pool0];
    } else {
        my @pool0_disks = RaidSqa_get_disk_list_by_pool(api_set_obj=>$api_set_obj, pool_name=>'pool0', disk_list=>\@spare_disks);
        my @pool1_disks = RaidSqa_get_disk_list_by_pool(api_set_obj=>$api_set_obj, pool_name=>'pool1', disk_list=>\@spare_disks);
        $disk_pool0 = shift @pool0_disks;
        $disk_pool1 = shift @pool1_disks;
        $aggr_add_args{disk_list} = [$disk_pool0];
        $aggr_add_args{disk_list1} = [$disk_pool1];
        $self->{tc_hash}->{disk_add}->{disk_name0} = $disk_pool0;
        $self->{tc_hash}->{disk_add}->{disk_name1} = $disk_pool1;
        $self->{aggr_add_disk_list} = [$disk_pool0, $disk_pool1];
    }

    if($aggr_add->{abort_operation}) {
        #set fiji to reboot filer
        $self->set_fiji_filter();
        eval {
            RaidSqa_aggr_add(%aggr_add_args);
        };
        if($@) {
            $aggr_add->{error} = $@;
        } else {
            $Log->warn("$self->{test}: Filer reboot: failed");
        }

        RaidSqa_transit_to(api_set_obj=>$api_set_obj, state_name=>"UP");
        sleep(5);
    } else {
        my $result =  RaidSqa_aggr_add(%aggr_add_args);
        if($result) {
            $Log->comment("$self->{test}: Aggr add: ok");
        } else {
            die("$self->{test}: Aggr add: failed");
        }
    }

    if($aggr_add->{check_vol_disks}) {
        my $match = ($aggr_add->{abort_operation}) ? "equal" : "less";
        my $add_add_disk_sizes = $self->get_disk_size(disk_list=>$self->{aggr_add_disk_list});
        foreach my $disk (@$self->{aggr_add_disk_list}) {
            my $aggr_size = $add_add_disk_sizes->{$disk};
            my $spare_size = $spares_disk_sizes->{$disk};
            die("$self->{test}: Could not find size for aggr disk $disk") if(!defined($aggr_size));
            die("$self->{test}: Could not find size for spares disk $disk") if(!defined($spare_size));
            $self->compare_disk_size(size1 => $aggr_size, size2=>$spare_size, match=>$match, disk_name=>$disk);
        }
    }
}

#-------------------------------------------------------------------------------
sub aggr_mirror {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $aggr_create = $self->{tc_hash}->{aggr_create};
    my $aggr_mirror = $self->{tc_hash}->{aggr_mirror};
    my ($old_plex, $new_plex);
    my @mirrored_disks = ();

    $self->debug("aggr_mirror: Dumper:".Dumper($aggr_create), 5);
    my $filer_vol = (defined($aggr_mirror->{filer_vol})) ? $aggr_mirror->{filer_vol} : $self->{tc_hash}->{aggr_create}->{filer_vol};
    die("$self->{test}: Filer vol is not defined") if(!defined($filer_vol));

    my @plex_list = RaidSqa_get_plex_list(api_set_obj => $api_set_obj, filer_vol=>$filer_vol);
    if(scalar(@plex_list) == 2) {
        die("$self->{test}: Aggr $filer_vol is already mirrored");
    } else {
        $old_plex = shift @plex_list;
    }

    my @aggr_disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
    my $aggr_disk_count = scalar(@aggr_disk_list);
    my $get_disk_list = ($aggr_create->{checksum_type}) ? \&RaidSqa_get_disk_list_by_checksum_type:
                                                          \&RaidSqa_get_disk_list;
    my @disk_list = $get_disk_list->(api_set_obj    => $api_set_obj,
                                     spare_only     => 'yes',
                                     checksum_type  => $aggr_create->{checksum_type});

    if(scalar(@disk_list) < $aggr_disk_count) {
        die("$self->{test}: sufficient spare disks are not available: $aggr_disk_count: ".scalar(@disk_list));
    }
    @mirrored_disks = @disk_list[0..$aggr_disk_count-1];
    if($aggr_mirror->{abort_operation}) {
        #set fiji to reboot filer
        $self->set_fiji_filter();
        eval {
            RaidSqa_aggr_mirror(api_set_obj => $api_set_obj, filer_vol => $filer_vol, disk_list => \@mirrored_disks);
        };
        if($@) {
            $aggr_mirror->{error} = $@;
        } else {
            $Log->warn("$self->{test}: Filer reboot: failed");
        }

        RaidSqa_transit_to(api_set_obj=>$api_set_obj, state_name=>"UP");
        sleep(5);
    } else {
        my $result = RaidSqa_aggr_mirror(api_set_obj => $api_set_obj, filer_vol => $filer_vol, disk_list => \@mirrored_disks);
        if(!$result) {
            die("$self->{test}: Aggr mirrored failed for $filer_vol");
        } else {
            my @plex_list = RaidSqa_get_plex_list(api_set_obj => $api_set_obj, filer_vol=>$filer_vol);
            $new_plex = ($plex_list[0] eq $old_plex) ? $plex_list[1] : $plex_list[0];
            @mirrored_disks = RaidSqa_get_disk_list(api_set_obj => $api_set_obj, plex_name => $new_plex) if($aggr_mirror->{check_vol_disks});
        }
    }
    if($aggr_mirror->{check_vol_disks}) {
        my $mirrored_disks_sizes = $self->get_disk_size(disk_list => \@mirrored_disks);

        #get size of spares disks
        my $spares_disk_sizes = $self->{spares_disk_sizes};

        #compare size
        foreach my $disk (@mirrored_disks) {
            my $aggr_size = $mirrored_disks_sizes->{$disk};
            my $spare_size = $spares_disk_sizes->{$disk};
            die("$self->{test}: Could not find size for aggr disk $disk") if(!defined($aggr_size));
            die("$self->{test}: Could not find size for spares disk $disk") if(!defined($spare_size));
            $self->compare_disk_size(size1 => $aggr_size, size2=>$spare_size, match=>"less", disk_name=>$disk);
        }
    }
}

#-------------------------------------------------------------------------------
sub aggr_destroy {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $aggr_destroy = $tc_hash->{aggr_destroy};
    my $aggr_type = $tc_hash->{aggr_create}->{aggr};

    $Log->comment("$self->{test}: aggr_destroy: start");

    my $filer_vol = (defined($aggr_destroy->{filer_vol})) ? $aggr_destroy->{filer_vol} : $tc_hash->{aggr_create}->{filer_vol};
    die("$self->{test}: Name for aggr is not defined") if(!defined($filer_vol));

    #get a list of disks in aggr
    my @aggr_disk_list = RaidSqa_get_disk_list(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol);

    #get size of disks in aggr
    my $aggr_disks_sizes = $self->get_disk_size(disk_list => \@aggr_disk_list);

    #destroy aggr
    my $result = RaidSqa_aggr_remove(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol, aggr_type=>$aggr_type);
    if(!$result) {
        die("$self->{test}: Aggr destroy was not successful for filer_vol $filer_vol");
    }

    #Check disk_size before zeroing
    my $spares_disk_sizes_non_zeroed = $self->get_disk_size(disk_list => \@aggr_disk_list);

    #zero spares disks
    RaidSqa_disk_zero_spares(api_set_obj => $api_set_obj);
    sleep(50); #To avoid more raid_config logs
    RaidSqa_wait_for_spare_disks_zeroing_completion(api_set_obj=>$api_set_obj);

    #get size of spares disks
    my $spares_disk_sizes_zeroed = $self->get_disk_size(disk_list => \@aggr_disk_list);

    if($aggr_destroy->{check_vol_disks}) {
        foreach my $disk (@aggr_disk_list) {
            my $aggr_size = $aggr_disks_sizes->{$disk};
            my $spare_size1 = $spares_disk_sizes_non_zeroed->{$disk};
            my $spare_size2 = $spares_disk_sizes_zeroed->{$disk};
            die("$self->{test}: Could not find size for aggr disk $disk") if(!defined($aggr_size));
            die("$self->{test}: Could not find size for spares disk $disk") if(!defined($spare_size1) or !defined($spare_size2));
            $self->compare_disk_size(size1 => $aggr_size, size2=>$spare_size1, match=>"less",disk_name=>$disk);
            $self->compare_disk_size(size1 => $aggr_size, size2=>$spare_size2, match=>"less",disk_name=>$disk);
        }
    }
    $Log->comment("$self->{test}: aggr_destroy: end");
}

#-------------------------------------------------------------------------------
sub aggr_scrub {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $aggr_scrub = $self->{tc_hash}->{aggr_scurb};
    my $aggr_create = $self->{tc_hash}->{aggr_create};

    $self->fill_volume();
    if($aggr_scrub->{do_recons}) {
        my @disk_list = RaidSqa_get_disk_list(api_set_obj=>$api_set_obj, filer_vol=>$aggr_create->{filer_vol});
        RaidSqa_setflag(api_set_obj=>$api_set_obj, flag_name => 'raid_disable_new_recons', flag_value => 0);
        my $result = RaidSqa_disk_fail(api_set_obj=>$api_set_obj, disk_name=>$disk_list[0], options=>"-i");
        if(!$result) {
            die("$self->{test}: Disk fail for $disk_list[0]: failed");
        } else {
            RaidSqa_wait_for_reconstruction_completion(api_set_obj => $api_set_obj, filer_vol => $aggr_create->{filer_vol});
        }
    }

    my $result = RaidSqa_aggr_scrub_start(api_set_obj=>$api_set_obj, filer_vol=>$aggr_create->{filer_vol});
    if(!$result) {
        die("$self->{test}: Scrub could not be started on $aggr_scrub->{filer_vol}");
    }

    #==================================================
    # wait for scrub to complete
    RaidSqa_wait_for_scrub_completion(api_set_obj=>$api_set_obj, filer_vol=>$aggr_scrub->{filer_vol});

    #==================================================
    #check scrub report for all groups in aggr
    my @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj, filer_vol=>$aggr_scrub->{filer_vol});
    my $test_group = shift @group_list;
    my $ref_report = RaidSqa_get_scrub_report_by_group(api_set_obj => $api_set_obj, group_name=>$test_group);

    if(ref $ref_report) {
        foreach my $error (keys %$ref_report) {
            my $comment = "$self->{test}: Scrub reported $ref_report->{$error} $error errors";
            if($ref_report->{$error} > 0) {
                die($comment);
            } else {
                $Log->comment($comment);
            }
        }
    }

    my $cmd = "priv set diag; rm $aggr_scrub->{filename}";
    RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd => $cmd);
}

#-----------------------------------------------------------------
sub fill_volume {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $aggr_create = $self->{tc_hash}->{aggr_create};
    my $filer_vol = $aggr_create->{filer_vol};
    my $filename = "/vol/$filer_vol/fill_volume";
    $self->{aggr_scrub}->{filename} = $filename;

    #get size of volume
    my $output = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd => "priv set diag; df $filer_vol");
    my @lines = split(/\r*\n/, $output);
    my @words = ();
    foreach my $line (@lines) {
        if($line =~ m%^/vol/$filer_vol/\s+%) {
            @words = split(/\s+/, $line);
            last;
        }
    }

    my $aggr_size = $words[3] - 20;
    die("$self->{test}: Could not get aggr size: $aggr_size") if(!defined($aggr_size));
    my $cmd = "priv set diag; mkfile $aggr_size $filename";
    RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd => $cmd);
}

#-------------------------------------------------------------------------------
sub disk_prefail {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    $self->populate_test_data(step=>"disk_prefail");

    #================================================
    #init variables
    my ($vol_disk_size,  $spare_disk_size, $size,
        $target_disk_name, $target_disk_size, $result);

    my $current_fsm_state = undef;
    my $spares_disk_sizes = $self->{spares_disk_sizes};

    my $disk_hash = $self->{tc_hash}->{disk_prefail};
    $self->debug("$self->{test}: disk_prefail:: disk_hash: ".Dumper($disk_hash));
    my $disk_name = $disk_hash->{disk_name};
    my $match = ($disk_hash->{abort_operation} == 1) ? "equal" : "less";
    my $fsm_state_old_disk = ($disk_hash->{abort_operation} == 1) ? "INUSE" : "FAILED";

    my $ref = $self->get_disk_size(disk_list=>[$disk_name]);
    $vol_disk_size = $ref->{$disk_name};
    $self->debug("$self->{test}: disk_size for $disk_name: $vol_disk_size");

    #================================================
    #prefail a disk
    $result = RaidSqa_disk_fail(api_set_obj=>$api_set_obj, disk_name=>$disk_name);
    if(!$result) {
        die("$self->{test}: disk prefail for $disk_name: failed");
    } else {
        $Log->comment("$self->{test}: disk prefail for $disk_name: ok");
    }
    sleep(5);

    #================================================
    #Check if sdc started on disk
    $result = RaidSqa_check_disk_sdc_start(api_set_obj=>$api_set_obj, disk_name =>$disk_name);
    if(!$result) {
        die("$self->{test}: sdc for $disk_name: failed");
    } else {
        $Log->comment("$self->{test}: sdc for $disk_name: ok");
    }

    #================================================
    #get target disk
    $target_disk_name = RaidSqa_get_disk_sdc_target(api_set_obj=>$api_set_obj, disk_name => $disk_name);
    if(!$target_disk_name) {
        die("$self->{test}: Find target disk: failed, $target_disk_name");
    } else {
        $Log->comment("$self->{test}: Find target disk: ok, $target_disk_name");
    }

    #================================================
    if($disk_hash->{abort_operation}) {
        $result = RaidSqa_disk_unfail(api_set_obj=>$api_set_obj, disk_name=>$disk_name);
        if(!$result) {
            die("$self->{test}: sdc unfail for $disk_name: failed");
        }

        $result = RaidSqa_check_disk_sdc_start(api_set_obj=>$api_set_obj, disk_name =>$disk_name);
        if(!$result) {
            $Log->comment("$self->{test}: sdc abort for $disk_name: ok");
        } else {
            die("$self->{test}: sdc abort for $disk_name: failed");
        }
    }

    #================================================
    #Check size of the target disk
    if($disk_hash->{check_new_disk}) {
        $ref = $self->get_disk_size(disk_list=>[$target_disk_name]);
        $target_disk_size = $ref->{$target_disk_name};
        my $spare_size = $self->{spares_disk_sizes}->{$target_disk_name};
        $self->compare_disk_size(size1 => $target_disk_size, size2=>$spare_size, match=>"$match",
                                disk_name=>$target_disk_name);
    }

    #================================================
    #Wait for SDC to complete
    RaidSqa_wait_for_disk_sdc_completion(api_set_obj=>$api_set_obj, disk_name=>$disk_name);

    #================================================
    #Check fsm state of the failed disk
    $result = RaidSqa_check_disk_fsm_state(api_set_obj=>$api_set_obj, disk_name=>$disk_name, fsm_state=>"$fsm_state_old_disk");
    if(!$result) {
        die("$self->{test}: FSM state of failed disk is not $fsm_state_old_disk");
    }

    #================================================
    #Check size of the failed disk
    if($disk_hash->{check_old_disk}) {
        $Log->comment("$self->{test}: Compare size of old disk after sdc completion");
        $ref = $self->get_disk_size(disk_list=>[$disk_name]);
        my $disk_size = $ref->{$disk_name};
        $self->compare_disk_size(size1 => $vol_disk_size, size2 => $disk_size, match => "$match",
                                 disk_name=>$disk_name);
    }

    #================================================
    if($disk_hash->{old_disk_final_state}) {
        my $makespare_option = ($disk_hash->{opt_makespare}) ? " -s" : "";
        my $fsm_state = $disk_hash->{old_disk_final_state};

        if($fsm_state =~ /(NON)?ZEROED/ && !$disk_hash->{abort_operation}) {
            $result = RaidSqa_disk_unfail(api_set_obj=>$api_set_obj, disk_name=>$disk_name, options=>$makespare_option);
            die("$self->{test}: sdc unfail for $disk_name: failed") if(!$result);
        }

        if($fsm_state eq "ZEROED")  {
            RaidSqa_disk_zero_spares(api_set_obj=>$api_set_obj);
            RaidSqa_wait_for_spare_disks_zeroing_completion(api_set_obj=>$api_set_obj, disk_list=>[$disk_name]);
        }

        $ref = $self->get_disk_size(disk_list=>[$disk_name]);
        my $spare_size = $ref->{$disk_name};
        $self->compare_disk_size(size1 => $vol_disk_size, size2 => $spare_size, match => "$match",
                                 disk_name=>$disk_name);
    }
}

#-------------------------------------------------------------------------------
sub disk_fail {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    $self->populate_test_data(step=>"disk_fail");

    my ($vol_disk_size,  $spare_disk_size, $size,
        $new_disk_name, $new_disk_size, $result);

    my $disk_hash = $self->{tc_hash}->{disk_fail};
    $self->debug("$self->{test}: disk_fail:: disk_hash: ".Dumper($disk_hash));
    my $match = "less";
    my $disk_name = $disk_hash->{disk_name};
    my $disk_type = $disk_hash->{disk_type};
    my $spares_disk_sizes = $self->{spares_disk_sizes};

    my $ref = $self->get_disk_size(disk_list=>[$disk_name]);
    $vol_disk_size = $ref->{$disk_name};
    my $test_group = $self->{test_group};

    # NOTE: Please do not alter sequence of steps,
    #       recons may get over if more delays is added
    #================================================
    #fail a disk
    $result = RaidSqa_disk_fail(api_set_obj=>$api_set_obj, disk_name=>$disk_name, options=>"-i");
    if(!$result) {
        die("$self->{test}: disk fail for $disk_name: failed");
    } else {
        $Log->comment("$self->{test}: disk fail for $disk_name: ok");
    }

    #================================================
    #get new disk
    my $parent_state = "RECONS".uc $disk_hash->{disk_type};
    my @disk_list = RaidSqa_get_disk_list_by_parent_state(api_set_obj => $api_set_obj, group_name => $test_group, parent_state => $parent_state);
    $new_disk_name = shift @disk_list;
    if(!$new_disk_name) {
        die("$self->{test}: Find recons disk: failed, $new_disk_name");
    } else {
        $Log->comment("$self->{test}: Find recons disk: ok, $new_disk_name");
    }

    #================================================
    #Check size of the target disk
    if($disk_hash->{check_new_disk}) {
        $ref = $self->get_disk_size(disk_list=>[$new_disk_name]);
        $new_disk_size = $ref->{$new_disk_name};
        my $spare_size = $self->{spares_disk_sizes}->{$new_disk_name};
        $self->compare_disk_size(size1 => $new_disk_size, size2=>$spare_size, match=>"less",
                                disk_name=>$new_disk_name);
    }

    #================================================
    if($disk_hash->{fail_recons_disk}) {
        $result = RaidSqa_disk_fail(api_set_obj=>$api_set_obj, disk_name=>$new_disk_name, options=>"-i");
        if(!$result) {
            die("$self->{test}: disk fail for $new_disk_name: failed");
        } else {
            $Log->comment("$self->{test}: disk fail for $new_disk_name: ok");
        }

        #Compare size of new disk with its size when it was spare
        $ref = $self->get_disk_size(disk_list=>[$new_disk_name]);
        $new_disk_size = $ref->{$new_disk_name};
        my $spare_size = $self->{spares_disk_sizes}->{$new_disk_name};
        $self->compare_disk_size(size1 => $new_disk_size, size2=>$spare_size, match=>"equal",
                                disk_name=>$new_disk_name);
    }

    #================================================
    #Wait for recons to complete
    RaidSqa_wait_for_reconstruction_completion(api_set_obj => $api_set_obj);

    #================================================
    #Check fsm state of the failed disk
    $result = RaidSqa_check_disk_fsm_state(api_set_obj => $api_set_obj, disk_name => $disk_name, fsm_state => "FAILED");
    if(!$result) {
        die("$self->{test}: FSM state of failed disk is not 'FAILED'");
    }

    #================================================
    #Check size of the failed disk
    if($disk_hash->{check_old_disk}) {
        $Log->comment("$self->{test}: Compare size of old disk after sdc completion");
        $ref = $self->get_disk_size(disk_list=>[$disk_name]);
        my $disk_size = $ref->{$disk_name};
        $self->compare_disk_size(size1 => $vol_disk_size, size2 => $disk_size, match => "less",
                                 disk_name=>$disk_name);
    }

    #================================================
    if($disk_hash->{old_disk_final_state}) {
        my $makespare_option = ($disk_hash->{opt_makespare}) ? " -s" : "";
        my $fsm_state = $disk_hash->{old_disk_final_state};

        if($fsm_state =~ /(NON)?ZEROED/ && !$disk_hash->{abort_operation}) {
            $result = RaidSqa_disk_unfail(api_set_obj=>$api_set_obj, disk_name=>$disk_name, options=>$makespare_option);
            die("$self->{test}: sdc unfail for $disk_name: failed") if(!$result);
        }

        if($fsm_state eq "ZEROED")  {
            RaidSqa_disk_zero_spares(api_set_obj=>$api_set_obj);
            RaidSqa_wait_for_spare_disks_zeroing_completion(api_set_obj=>$api_set_obj, disk_list=>[$disk_name]);
        }

        $ref = $self->get_disk_size(disk_list=>[$disk_name]);
        my $spare_size = $ref->{$disk_name};
        $self->compare_disk_size(size1 => $vol_disk_size, size2 => $spare_size, match => "less",
                                 disk_name=>$disk_name);
    }
}

#-------------------------------------------------------------------------------
sub disk_replace {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    $self->populate_test_data(step=>"disk_replace");

    #================================================
    #init variables
    my ($vol_disk_size,  $spare_disk_size, $size,
        $target_disk_name, $target_disk_size, $result);
    my $current_fsm_state = undef;
    my $spares_disk_sizes = $self->{spares_disk_sizes};

    my $disk_hash = $self->{tc_hash}->{disk_replace};
    $self->debug("$self->{test}: disk_replace:: disk_hash: ".Dumper($disk_hash));
    my $disk_name = $disk_hash->{disk_name};
    my $match = ($disk_hash->{abort_operation} == 1) ? "equal" : "less";
    my $fsm_state_old_disk = ($disk_hash->{abort_operation} == 1) ? "INUSE" : "NOTZEROED";

    my $ref = $self->get_disk_size(disk_list=>[$disk_name]);
    $vol_disk_size = $ref->{$disk_name};
    $self->debug("$self->{test}: vol disk_size for $disk_name: $vol_disk_size");
    my @possible_replacing_disks = keys %$spares_disk_sizes;
    
    #Get a pool of $disk_name
    $Log->comment("$self->{test}: Finding pool for disk $disk_name");
    my @disk_list = RaidSqa_get_disk_list_by_pool(api_set_obj=>$api_set_obj, pool_name =>"pool0", disk_list=>[$disk_name]);    
    if(scalar(@disk_list)) {
        $Log->comment("$self->{test}: Using pool0 disks finding a replacing disk");
        @possible_replacing_disks = RaidSqa_get_disk_list_by_pool(api_set_obj=>$api_set_obj, pool_name =>"pool0", disk_list=>\@possible_replacing_disks);
    } else {
        $Log->comment("$self->{test}: Using pool1 disks for finding replacing disk");
        @possible_replacing_disks = RaidSqa_get_disk_list_by_pool(api_set_obj=>$api_set_obj, pool_name =>"pool1", disk_list=>\@possible_replacing_disks);
    }
    
    foreach my $disk (@possible_replacing_disks) {
        $self->debug("$self->{test}: spare_disk: $disk size: $spares_disk_sizes->{$disk}", 5);
        if($spares_disk_sizes->{$disk} > $vol_disk_size && $disk ne $disk_name) {
            $target_disk_name = $disk;
            last;
        }
    }

    $self->debug("$self->{test}: Disk $target_disk_name will be used for disk replace");
    die("$self->{test}: Target disk for disk replace could not be found") if(!defined($target_disk_name));

    #================================================
    #replace a disk
    $result = RaidSqa_disk_replace_start(api_set_obj    =>$api_set_obj,
                                         disk_name      => $disk_name,
                                         replacing_disk => $target_disk_name);
    sleep(5);

    #================================================
    #Check if sdc started on disk
    if(!$result) {
        die("$self->{test}: disk replace failed on $disk_name");
    } else {
        $Log->comment("$self->{test}: disk replace successful on $disk_name"); 
    }

    #================================================
    if($disk_hash->{abort_operation}) {
        my $cmd = "priv set -q diag; disk replace stop $disk_name";
        RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>$cmd);

        $result = RaidSqa_check_disk_sdc_start(api_set_obj => $api_set_obj, disk_name => $disk_name);
        if(!$result) {
            $Log->comment("$self->{test}: sdc abort for $disk_name: ok");
        } else {
            die("$self->{test}: sdc abort for $disk_name: failed");
        }
    }

    #================================================
    #Check size of the target disk
    if($disk_hash->{check_new_disk}) {
        $ref = $self->get_disk_size(disk_list=>[$target_disk_name]);
        $target_disk_size = $ref->{$target_disk_name};
        my $spare_size = $self->{spares_disk_sizes}->{$target_disk_name};
        $self->compare_disk_size(size1 => $target_disk_size, size2=>$spare_size, match=>"$match",
                                disk_name=>$target_disk_name);
    }

    #================================================
    #Wait for SDC to complete
    RaidSqa_wait_for_disk_sdc_completion(api_set_obj=>$api_set_obj, disk_name=>$disk_name);

    #================================================
    #Check fsm state of the failed disk
    $result = RaidSqa_check_disk_fsm_state(api_set_obj=>$api_set_obj, disk_name=>$disk_name, fsm_state=>"$fsm_state_old_disk");
    if(!$result) {
        die("$self->{test}: FSM state of failed disk is not $fsm_state_old_disk");
    }

    #================================================
    #Check size of the failed disk
    if($disk_hash->{check_old_disk}) {
        $Log->comment("$self->{test}: Compare size of old disk after sdc completion");
        $ref = $self->get_disk_size(disk_list=>[$disk_name]);
        my $disk_size = $ref->{$disk_name};
        $self->compare_disk_size(size1 => $vol_disk_size, size2 => $disk_size, match => "$match",
                                 disk_name=>$disk_name);
    }

    #================================================
    if($disk_hash->{old_disk_final_state}) {
        my $fsm_state = $disk_hash->{old_disk_final_state};

        if($fsm_state eq "ZEROED")  {
            RaidSqa_disk_zero_spares(api_set_obj=>$api_set_obj);
            RaidSqa_wait_for_spare_disks_zeroing_completion(api_set_obj=>$api_set_obj, disk_list=>[$disk_name]);
        }

        $ref = $self->get_disk_size(disk_list=>[$disk_name]);
        my $spare_size = $ref->{$disk_name};
        $self->compare_disk_size(size1 => $vol_disk_size, size2 => $spare_size, match => "$match",
                                 disk_name=>$disk_name);
    }

}

#TODO: Test cases are not yet finalized
sub create_file {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    $Log->warn("$self->{test}: Test cases are not yet defined");
}

#-------------------------------------------------------------------------------
sub get_spares_disk_sizes {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my @spares_disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj, spare_only => "yes");
    $self->debug("$self->{test}: get_spares_disk_sizes: spare list: @spares_disk_list");
    return $self->get_disk_size(disk_list=>\@spares_disk_list);
}

#-------------------------------------------------------------------------------
sub get_disk_size {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};

    my %args = @_;
    my $disk_list = $args{disk_list};
    my @disk_list = @{$disk_list};
    $self->debug("$self->{test}: get_disk_size:: input_disk_list: @disk_list");
    my $disk_sizes = {};

    my $conn  = $self->{conn_obj};
    my $robj = new RaidInfo($conn);

    foreach my $disk (@disk_list) {
        my $dobj = $robj->disk($disk);
        my $disk_size = undef;
        if(defined $dobj) {
            $disk_sizes->{$disk} = $dobj->parameter("used_sz");
        }
    }
    $self->debug("$self->{test}: Dumper: disk_sizes ".Dumper($disk_sizes));
    return $disk_sizes;
}

#-------------------------------------------------------------------------------
sub compare_disk_size {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my %args = validate(@_,{
                        size1     => {type => SCALAR},
                        size2     => {type => SCALAR},
                        match     => {type => SCALAR, default => "equal", optional=>1},
                        disk_name => {type => SCALAR | UNDEF, default => undef, optional=>1},
                        });
    my $size1     = $args{size1};
    my $size2     = $args{size2};
    my $match     = $args{match};
    my $disk_name = $args{disk_name};

    my $op = {
                less => "<",
                more => ">",
                equal => "=="
             };

    my $expr = "($size1 $op->{$match} $size2) ? 1 : 0";
    my $op_str = ($match eq 'equal') ? "to" : "than";
    $self->debug("$self->{test}: Eval expression : $expr", 5);
    my $result = eval($expr);
    my $disk_info = defined($disk_name) ? "for $disk_name" : "";
    if($result) {
        $Log->comment("$self->{test}: Disk's size1 $size1 $disk_info is $match $op_str size2 $size2");
    } else {
        die("$self->{test}: Disk's size1 $disk_info is not $match $op_str $size2");
    }
}

#-------------------------------------------------------------------------------
sub set_fiji_filter {
    my $self = shift;
    my $filer_vol = $self->{tc_hash}->{aggr_create}->{filer_vol};
    if(!defined($filer_vol)) {
        die("$self->{test}: Filer vol name is required to set fiji");
    }
    my $fiji_point = "(raid.labelio.L1done (eq \@vol $filer_vol)(reboot))";
    my $result = RaidSqa_fiji_filter(api_set_obj => $self->{api_set_obj},
                                     fiji_point  => $fiji_point);
    if(!$result) {
        die("$self->{test}: Set Fiji filter: failed");
    }
}

#-------------------------------------------------------------------------------
sub populate_test_data {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $aggr_create = $self->{tc_hash}->{aggr_create};
    my %args = @_;
    my $step = $args{step};

    if(!defined($aggr_create)) {
        $self->debug("$self->{test}: Populate_data: ".Dumper($self->{tc_hash}));
        die("$self->{test}: aggr_create information is required");
    }
    my $filer_vol = $aggr_create->{filer_vol};
    my $raid_type = $aggr_create->{raid_type};
    my @group_list = RaidSqa_get_group_list(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol);
    my $test_group = shift @group_list if(scalar(@group_list));
    if(defined($test_group)) {
        $self->debug("$self->{test}: Test group: $test_group");
        $self->{test_group} = $test_group;
    } else {
        die("$self->{test}: Could not find test group");
    }

    if($step eq "aggr_scurb") {

    }

    if($step =~ /^disk_prefail|disk_fail|disk_replace$/) {
        my $disk_type = $tc_hash->{$step}->{disk_type};

        die("$self->{test}: filer_vol parameter is required") if(!defined($filer_vol));
        die("$self->{test}: disk_type can not be dparity for raid4") if($raid_type eq "raid4" && $disk_type eq "dparity");
        my @disk_list = RaidSqa_get_disk_list_by_raid_type(api_set_obj=>$api_set_obj,
                                                           group_name=>$test_group,
                                                           raid_type=>$disk_type);

        $self->debug("$self->{test}: disk list for group $test_group: @disk_list");
        my $disk_name = shift @disk_list;
        if($disk_name) {
            $self->{tc_hash}->{$step}->{disk_name} = $disk_name;
        } else {
            die("$self->{test}: Could not find disk for test");
        }
    }
    $self->debug("$self->{test}: populate_test_data: Dumper: tc_hash".Dumper($self->{tc_hash}));
}

#=============================================================================
package AzcsAggrAdd;

use vars qw(@ISA);
use strict;
use warnings;
use Tharn qw($Log);
use RaidSqav3 qw(AUTOLOAD);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidLW;
use RaidInfo;
use Data::Dumper;
use Utils;

BEGIN {
    @ISA = qw(RaidAzcsUtils);
}

#---------------------------------------------------------------------
sub new {
    my $self = shift if(scalar(@_) % 2);
    my %self = validate(@_, {
                                api_set_obj => {type => OBJECT, isa=> "NACL::APISet"},
                                debug       => {type => SCALAR, default=>0, optional=>1},
                            });
    $self = \%self;
    $self->{debug} = 5;
    my $api_set_obj = $self->{api_set_obj};
    my $host;
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        my $conntype = ($host->{hosttype} =~ /ng/) ? "#ssh" : "#rsh";
        $self->{conn_obj} = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $self->{conn_obj} = $api_set_obj->{connection};
    }
    bless $self, 'AzcsAggrAdd';
    $self->{test_object} = (defined($main::Test)) ? $main::Test : $Log;
    $self->init();
    return $self;
}

#---------------------------------------------------------------------
sub init {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $conn_obj = $self->{conn_obj};
    my $rc_object = new RaidInfo($conn_obj);

    #init test data base
    $self->{testdb}->{key_mapping} = {
        filer_vol                    => '$self->{tc_hash}->{aggr_create}->{filer_vol}',
        aggr_checksum_type           => '$self->{tc_hash}->{aggr_create}->{checksum_type}',
        aggr_type                    => '$self->{tc_hash}->{aggr_create}->{aggr_type}',
        aggr_raid_type               => '$self->{tc_hash}->{aggr_create}->{raid_type}',
        aggr_mirror                  => '$self->{tc_hash}->{aggr_create}->{mirror}',
        aggr_add_params              => '$self->{tc_hash}->{aggr_add}->{command_params}',
        aggr_add_hash                => '$self->{tc_hash}->{aggr_add}->{command_hash}',
        aggr_add_output              => '$self->{tc_hash}->{test_result}->{actual}->{command_output}',
        aggr_add_status              => '$self->{tc_hash}->{test_result}->{actual}->{aggr_add_status}',
        aggr_add_result              => '$self->{tc_hash}->{test_result}',
        aggr_add_command             => '$self->{tc_hash}->{aggr_add}->{command}',
        ingore_other_spares          => '$self->{tc_hash}->{aggr_add}->{ingore_other_spares}',
        test_rc_object               => '$self->{testdb}->{rc_object}',
        test_aggr_info               => '$self->{testdb}->{aggr_info}',
        test_spares_info             => '$self->{testdb}->{spare_info}',
        enable_optional_storage_type => '$self->{tc_hash}->{aggr_add}->{enable_optional_storage_type}',
    };

    $self->{testdb}->{aggr_info} = undef;
    $self->{testdb}->{spare_info} = undef;
    $self->{testdb}->{rc_object} = undef;
    $self->{testdb}->{added_disks} = [];
    $self->{testdb}->{disk_unassign}->{pool0} = [];
    $self->{testdb}->{disk_unassign}->{pool1} = [];

    #create raid_config object and assign to $self
    $self->{testdb}->{rc_object} = $rc_object;
}

#---------------------------------------------------------------------
sub run_test {
    my $self = shift;
    my %args = validate(@_, {
                                tc_hash     => {type => HASHREF},
                                test        => {type => SCALAR, default=>"AzcsRbnDbn", optional=>1},
                            });

    $self->{tc_hash} = $args{tc_hash};
    $self->{test} = $args{test};
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my ($opt_azcs_value);

    eval {
    
        #If aggr checksum type is zoned, we will disable raid.azcs.enable option to create zcsv1 disks
        if(lc $tc_hash->{aggr_create}->{checksum_type} eq "zoned") {
           $opt_azcs_value = RaidSqa_get_options(api_set_obj=>$api_set_obj, option_name=>"raid.azcs.enable");
           RaidSqa_set_options(api_set_obj=>$api_set_obj, option_name=>"raid.azcs.enable", option_value=>"off");
        }
    
        $self->step("AggrAdd: Setup testbed for test execution");
        $self->setup();

        if(lc $tc_hash->{aggr_create}->{checksum_type} eq "zoned") {
           RaidSqa_set_options(api_set_obj=>$api_set_obj, option_name=>"raid.azcs.enable", option_value=>$opt_azcs_value);
        }
        
        $self->step("AggrAdd: Populate test case data required for this test");
        $self->populate_test_data();

        $self->step("AggrAdd: Setup argument hash for aggr add hash");
        $self->setup_aggr_add_hash();
        
        $self->step("AggrAdd: Execute aggr add command");
        $self->execute_aggr_add_cmd();
        sleep(2);
        RaidSqa_filer_cmd(api_set_obj => $self->{api_set_obj}, cmd=>'aggr status -r');

        $self->step("AggrAdd: Save test results");
        $self->save_test_results();

        $self->step("AggrAdd: Validate test results");
        $self->validate_test_results();

        $self->step("AggrAdd: Cleanup testbed setup");
        $self->cleanup();
    };

    if($@) {
        $self->{error} = $@;
        $Log->comment("FAIL: $self->{error}");        
        $self->cleanup();
        die("FAIL: $self->{error}");
    }
}

#---------------------------------------------------------------------
sub setup {
    my $self = shift;
    my $setup = $self->{tc_hash}->{setup};
    my $aggr_add_opts = $self->{tc_hash}->{aggr_add};
    my $filer_vol = $self->{tc_hash}->{aggr_create}->{filer_vol};

    if(defined($setup->{set_flags}) && $setup->{set_flags} == 1) {
        $self->step("AggrAdd: Setup flags for this test case");
        $self->set_flags();
    }

    if(defined($setup->{set_options}) && $setup->{set_options} == 1) {
        $self->step("AggrAdd: Setup options for this test case");
        $self->set_options();
    }
    
    if(defined($setup->{spare_setup}) && $setup->{spare_setup} == 1) {
        $self->step("AggrAdd: Setup spares for test execution");
        $self->setup_spares();
        RaidSqa_filer_cmd(api_set_obj => $self->{api_set_obj}, cmd=>'aggr status -s');
    }

    #If disk_size is mentioned as disk_list_option for aggr add, we should use preferred disk as disk list
    if(ref($aggr_add_opts->{command_params}->{disk_list})) {
        my $defined_size_type = 0;
        foreach my $spare_type (@{$aggr_add_opts->{disk_list_options}}) {
            $defined_size_type = 1 if($spare_type->{size} =~ /less|more/i);
        }
        if($defined_size_type == 1) {
            $Log->comment("AggrAdd: Setup: Updating disklist and removing disk count from aggr_create");
            my $disk_count = $self->{tc_hash}->{aggr_create}->{disk_count};
            my $mirror = $self->{tc_hash}->{aggr_create}->{mirror} or 0;
            my $disks_per_pool = $disk_count / ($mirror+1);
            my $storage_type = $self->{tc_hash}->{aggr_create}->{storage_type};
            my $checksum_type = $self->{tc_hash}->{aggr_create}->{checksum_type};

            $self->update_spare_info();
            my $spare_info = $self->get_hash('test_spares_info');
            my @spare_disks = keys %$spare_info;
            @spare_disks = $self->get_spares_by_filter(disk_list => \@spare_disks, checksum_type=>$checksum_type, storage_type=>$storage_type);
            my $disk_by_sizes = $self->sort_disks_by_used_size(disk_list=>\@spare_disks);
            my ($disk_count_by_size, $pref_disk_size) = (0,0);
            my @disk_list = ();
            foreach my $size (keys %$disk_by_sizes) {
                @disk_list = @{$disk_by_sizes->{$size}};
                if($disk_count_by_size < scalar(@disk_list)) {
                    $pref_disk_size = $size;
                    $disk_count_by_size = scalar(@disk_list);
                }
            }
            if(@disk_list < $disk_count) {
                die("AggrAdd: Setup: No enough spares are available of preferred size: $pref_disk_size");
            }
            
            $self->{tc_hash}->{aggr_create}->{disk_list} = [@disk_list[0..$disks_per_pool-1]];

            if($mirror) {
                $self->{tc_hash}->{aggr_create}->{disk_list1} = [@disk_list[$disks_per_pool..(2*$disks_per_pool-1)]];
            }
            delete $self->{tc_hash}->{aggr_create}->{disk_count};
            delete $self->{tc_hash}->{aggr_create}->{checksum_type};
        }
        $Log->comment("AggrAdd: Dumper: tc_hash: ".Dumper($self->{tc_hash}));
    }

    RaidSqa_filer_cmd(api_set_obj => $self->{api_set_obj}, cmd => "aggr status -r");

    #if this flag is not set, It will create aggr by default
    if(!defined($setup->{aggr_create}) or $setup->{aggr_create} != 0) {
        $self->step("AggrAdd: Create a test aggregate");
        $self->aggr_create();
        
        $self->step("AggrAdd: Set aggr options");
        if(!defined($setup->{set_aggr_options}) or $setup->{set_aggr_options} != 0) {
            $self->set_aggr_options(filer_vol=>$filer_vol);
        }
        
        $self->step("AggrAdd: Add raid groups to test aggregate");
        $self->aggr_add_groups();
        RaidSqa_filer_cmd(api_set_obj => $self->{api_set_obj}, cmd=>'aggr status -r');
    }
}

#---------------------------------------------------------------------
sub cleanup {
    my $self = shift;
    my $cleanup = $self->{tc_hash}->{cleanup};
    my $filer_vol = $self->{tc_hash}->{aggr_create}->{filer_vol};
    RaidSqa_filer_cmd(api_set_obj=>$self->{api_set_obj}, cmd=>"priv set -q diag; aggr status -r");

    if($cleanup->{aggr_remove} == 1) {
        $self->step("AggrAdd: cleanup test aggr");
        $self->aggr_destroy();
    } elsif($cleanup->{reset_aggr_options}) {
        $self->reset_aggr_options(filer_vol => $filer_vol);
    }
    
    if($cleanup->{disk_assign} == 1) {
        $self->step("AggrAdd: Reassign all disks removed during the test");
        $self->disk_assign();
    }
    if($cleanup->{disk_unfail} == 1) {
        $self->step("AggrAdd: Unfail all disks");
        RaidSqa_disk_unfail(api_set_obj=>$self->{api_set_obj}, disk_name=>'all');
    }
    if($cleanup->{reset_options} == 1) {
        $self->step("AggrAdd: cleanup, reset all options used in this test to their original values");
        $self->reset_options();
    }

    if($cleanup->{reset_flags} == 1) {
        $self->step("AggrAdd: cleanup, reset all flags used in this test to their original values");
        $self->reset_flags();
    }

    if($cleanup->{zero_spares} == 1) {
        $self->step("AggrAdd: Cleanup, start spare disks zeoring");
        RaidSqa_disk_zero_spares(api_set_obj=>$self->{api_set_obj});
    }
}

#--------------------------------------------------------------------
sub setup_spares {
    my $self = shift;
    my $tc_hash = $self->{tc_hash};
    my $api_set_obj = $self->{api_set_obj};
    my $required_spares = $self->{tc_hash}->{spare_setup};
    my $mirror = $self->{tc_hash}->{aggr_create}->{mirror};
    
    $Log->comment("$self->{test}: setup_spares: update spare info");
    $self->update_spare_info();

    #get all spares disks
    my $index = 0;
    my @disks_to_assign = ();
    my @disks_to_remove = ();
    my $spares_info = $self->get_hash('test_spares_info');
    my @spare_disk_list = keys %$spares_info;
    my @all_spare_disks = @spare_disk_list;
    foreach my $spare_type (@$required_spares) {
        my $min_count = defined($spare_type->{min_spare_count}) ? $spare_type->{min_spare_count} : -1;
        my $max_count = defined($spare_type->{max_spare_count}) ? $spare_type->{max_spare_count} : -1;
        $self->debug("$self->{test}: min_spare_count: $min_count", 3);
        $self->debug("$self->{test}: max_spare_count: $max_count", 3);
        my %get_spare_filter = (
            checksum_type => $spare_type->{checksum_type},
            bps => $spare_type->{bps},
            rpm => $spare_type->{rpm},
        );

        foreach my $key (keys %get_spare_filter) {
            delete $get_spare_filter{$key} if(!defined($get_spare_filter{$key}) or uc $get_spare_filter{$key} eq 'ANY');
        }
        @spare_disk_list = $self->get_spares_by_filter(%get_spare_filter);        
        die('$self->{test}: Enough spares of given criteria are not available') if("$min_count" ne "-1" && scalar(@spare_disk_list) < $min_count * ($mirror+1));
        
        #FOR storage type we need to consider the optional storage type
        if(defined($spare_type->{storage_type}) && uc $spare_type->{storage_type} ne 'ANY') {
            my @disk_list = $self->get_spares_by_rc_parameter(parameter_name   => 'disk_type',
                                                                parameter_value  => $spare_type->{storage_type},
                                                                disk_list        => \@spare_disk_list);
            $self->debug("$self->{test}: Storage_type: $spare_type->{storage_type}, disk_count: ".scalar(@disk_list));
            if("$min_count" ne -1 && scalar(@disk_list) < $min_count*($mirror+1)) {
                if(1 != $self->is_enable_optional_storage_type()) {
                    die("$self->{test}: Enough spares of storage_type $spare_type->{storage_type} are not available");
                } else {
                    $Log->warn("$self->{test}: Enough spares of storage_type $spare_type->{storage_type} are not available");
                    $Log->comment("$self->{test}: Check alternate storage type");
                    my @optional_storage_type = $self->get_optional_storage_types(storage_type => $spare_type->{storage_type});
                    my @temp_disk_list = ();
                    foreach my $storage_type (@optional_storage_type) {
                        @temp_disk_list = $self->get_spares_by_rc_parameter( parameter_name   => 'disk_type',
                                                                             parameter_value  => $storage_type,
                                                                             disk_list        => \@spare_disk_list);
                        $self->debug("$self->{test}: setup_spares: Storage_type: $storage_type, disk_count: ".scalar(@temp_disk_list));
                        if(scalar(@temp_disk_list) >= $min_count) {
                            $Log->comment("$self->{test}: Using optional storage type $storage_type for this test");
                            last;
                        }
                    }
                    if(scalar @temp_disk_list < $min_count*($mirror+1)) {
                        die("$self->{test}: Enough spare disks of required storage type could not found for this test");
                    } else {
                        if($mirror) {
                            #Check for each pool having sufficient disks for mirror enabled test
                            foreach my $pool (0 ,1) {
                                my @pool_disks = $self->get_spares_by_rc_parameter( parameter_name   => 'pool',
                                                                                     parameter_value  => $pool,
                                                                                     disk_list        => \@spare_disk_list);
                                die("$self->{test}: Pool $pool does not have enough spares of $spare_type->{storage_type} $spare_type->{checksum_type}")
                                if(scalar(@pool_disks) < $min_count);
                            }
                        }
                        @spare_disk_list = @temp_disk_list;
                    }
                }
            } else {
                @spare_disk_list = @disk_list;
            }
        }
        #Sort based on pool
        if("$max_count" ne "-1" && scalar(@spare_disk_list)) {
            my @local_disk_list = ();
            foreach my $pool (0 ,1) {
                my @pool_disks = $self->get_spares_by_rc_parameter( parameter_name   => 'pool',
                                                                     parameter_value  => $pool,
                                                                     disk_list        => \@spare_disk_list);
                if($mirror) {
                    push(@local_disk_list, @pool_disks[0..$max_count-1]);
                } else {
                    push(@local_disk_list, @pool_disks);
                }
            }
            @spare_disk_list = @local_disk_list[0..($max_count*($mirror+1))-1];
        }
        push(@disks_to_assign, @spare_disk_list);
        $self->{spare_setup}->[$index++]->{spare_disk_list} = \@spare_disk_list;
    }

    #Find the disks whose ownership is to be removed
    if(!$self->get('ingore_other_spares')) {        #This value by default should be 1
        my %disk_hash = map {$_, 1} @disks_to_assign;
        @disks_to_remove = grep (!defined($disk_hash{$_}), @all_spare_disks);
        foreach my $pool (0,1) {
            my @pool_disks = $self->get_spares_by_rc_parameter(parameter_name   => 'pool',
                                                                parameter_value  => $pool,
                                                                disk_list        => \@disks_to_remove);
            $self->{testdb}->{disk_unassign}->{"pool".$pool} = \@pool_disks;
        }
    }
    if(scalar(@disks_to_remove)) {
        my $result = RaidSqa_disk_remove_ownership(api_set_obj=>$api_set_obj, disk_list=>\@disks_to_remove);
        if(!$result) {
            die("$self->{test}: Disk Remove_ownership failed");
        }
    }
    $self->debug("$self->{test}: Removed ownership for @disks_to_remove");
    sleep(2);
}

# AzcsAggrAdd::populate_test_data
#--------------------------------------------------------------------
sub populate_test_data {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};

    if(!defined($api_set_obj)) {
       die("$self->{test}: Api set object is not defined".Dumper($self));
    }

    $Log->comment("$self->{test}: Populate test data");
    $self->update_spare_info();

    #==================================================================================================
    #Save existing aggr disks info
    #{plex => {rg} => [disks];
    my $filer_vol = $self->get('filer_vol');
    my $aggr_info = {};
    my @all_data_disks = ();
    my @plex_list = RaidSqa_get_plex_list(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol);
    foreach my $plex (@plex_list) {
        my @group_list = RaidSqa_get_group_list(api_set_obj=>$api_set_obj, plex_name => $plex);
        foreach my $rg (@group_list) {
            my @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj, group_name => $rg);
            push(@all_data_disks, RaidSqa_get_disk_list_by_raid_type(api_set_obj => $api_set_obj, group_name => $rg, raid_type=>'data'));
            $aggr_info->{$plex}->{$rg} = [@disk_list];
        }
    }
    
    #=============================================================================
    #update aggr add params
    my $aggr_add_params = $self->get_hash('aggr_add_params');
    $aggr_add_params->{force} = 1 if(!defined($aggr_add_params->{force}));
    $self->set('ingore_other_spares') = 1 if(!defined($self->get('ingore_other_spares')));

    #If group name is defined in aggr_add_params, we will consider only that raid group for populating disks
    if($aggr_add_params->{group_name} =~ /rg\d+/){
        my @group_list = RaidSqa_get_group_list(api_set_obj=>$api_set_obj);
        @group_list = grep (/$aggr_add_params->{group_name}/, @group_list);
        @all_data_disks =  RaidSqa_get_disk_list_by_raid_type(api_set_obj => $api_set_obj, group_name => $group_list[0], raid_type=>'data');
    }
    #Sort the disks by size and get the disk used (size => [disk_list])
    my $sorted_data_disks = $self->sort_disks_by_used_size(disk_list => \@all_data_disks, ascending=>1);
    my @asc_disk_sizes = keys %$sorted_data_disks;
    
    #If incomplete raid group, we need to make sure that disk_count is alway less that raid_size

    #Update disk list if defined as empty array hash
    if(uc ref $aggr_add_params->{disk_list} && scalar(@{$aggr_add_params->{disk_list}}) == 0) {
        if(defined($aggr_add_params->{storage_type})) {
            $Log->warn("$self->{test}: Storage type and disk list can not be used together, ingorning storage type");
            $aggr_add_params->{storage_type} = undef;
        }
        if(defined($aggr_add_params->{checksum_type})) {
            $Log->warn("$self->{test}: Checksum type and disk list can not be used together, ingorning checksum type");
            $aggr_add_params->{checksum_type} = undef;
        }
        my @spare_disks = keys %{$self->get_hash('test_spares_info')};
        if(defined($self->{tc_hash}->{aggr_add}->{disk_list_options})) {
            my @opts = @{$self->{tc_hash}->{aggr_add}->{disk_list_options}};
            foreach my $index (0..$#opts) {
                my $opt = $opts[$index];
                my $disk_count = (defined($opt->{disk_count})) ? $opt->{disk_count} : $aggr_add_params->{disk_count};
                die("$self->{test}: Disk count parmaeter is required for finding disk list") if(!defined($disk_count));
                my @disk_list = $self->get_spares_by_filter (
                                            checksum_type => $opt->{checksum_type},
                                            storage_type  => $opt->{storage_type},
                                            bps           => $opt->{bps},
                                            disk_list     => \@spare_disks,
                                        );
                
                my @valid_disks = ($aggr_add_params->{force}) ? @disk_list : $self->get_spares_by_rc_parameter (
                                                                                        parameter_name => 'pool', 
                                                                                        parameter_value => '0', 
                                                                                        disk_list => \@disk_list
                                                                                    );
                
                #Possible - less, more
                if($opt->{size} =~ /(less|more)/) {
                    my @used_disks = (@{$aggr_add_params->{disk_list}});
                    push(@used_disks, @{$aggr_add_params->{disk_list1}}) if(defined $aggr_add_params->{disk_list1});
                    $Log->comment("$self->{test}: Used disks: @used_disks");
                    my %used_disks = map {$_ => 1 } @used_disks;
                    my @unused_spare_disks = grep !defined($used_disks{$_}), @valid_disks;
                    my $sorted_spares = $self->sort_disks_by_used_size(disk_list=>\@unused_spare_disks, ascending=>1);
                    my @asc_spare_sizes = keys %$sorted_spares;
                    my @sorted_spare_disk_list = ();
                    $Log->comment("$self->{test}: Sorted disk sizes: @asc_spare_sizes");
                    foreach my $size (@asc_spare_sizes) {
                        if(exists $sorted_spares->{$size}) {
                            push(@sorted_spare_disk_list, @{$sorted_spares->{$size}});
                        }
                    }

                    if(!@sorted_spare_disk_list) {
                        die("$self->{test}: no disks found after sorting disks based on size");
                    }
                    
                    #if disk size is mentioned as 'less':                    
                    if($opt->{size} =~ /(less)/) {
                        # Find disks which is lower in size than lowest size data disk
                        if($asc_spare_sizes[0] < $asc_disk_sizes[0]) {
                            @valid_disks = @sorted_spare_disk_list;
                        }
                    } elsif($opt->{size} =~ /(more)/) {
                        # Find disks which is greater in size than greatest size data disk
                        if($asc_spare_sizes[$#asc_spare_sizes] > $asc_disk_sizes[$#asc_disk_sizes]) {
                            @valid_disks = reverse @sorted_spare_disk_list;
                        }
                    }
                }
                
                if(scalar(@valid_disks) < $disk_count) {
                    die("$self->{test}: populate_test_data: There are not enough spares for disk add operation");
                }
                
                $Log->comment("$self->{test}: populate_test_data: spares that will be used for disk add: @valid_disks");
                push(@{$aggr_add_params->{disk_list}}, @valid_disks[0..$disk_count-1]);
                @valid_disks = @valid_disks[$disk_count..$#valid_disks];
                if($self->{tc_hash}->{aggr_create}->{mirror}) {
                    my @valid_disks1 = ($aggr_add_params->{force}) ? @valid_disks : $self->get_spares_by_rc_parameter (
                                                                                            parameter_name => 'pool', 
                                                                                            parameter_value => '1', 
                                                                                            disk_list => \@disk_list
                                                                                        );
                    if(scalar(@valid_disks1) < $disk_count) {
                        die("$self->{test}: populate_test_data: There are not enough spares for disk add operation");
                    } else {
                        $Log->comment("$self->{test}: populate_test_data: spares that will be used for disk add: @valid_disks1");
                        push(@{$aggr_add_params->{disk_list1}}, @valid_disks1[0..$disk_count-1]);
                    }
                }
            }
        } else {
            die("$self->{test}: disk_list_options hash is mandetory for finding disk list");
        }
    }
    
    #=================================================================================================
    if($aggr_add_params->{disk_size} eq "define") {
        my $disk_size = shift @asc_disk_sizes;
        $aggr_add_params->{disk_size} = $disk_size."m";
    }
    
    $self->set_hash(aggr_add_params => $aggr_add_params);
    $self->set_hash(test_aggr_info => $aggr_info);
    #==================================================================================================
    $self->debug("$self->{test}: populate_test_data: tc_hash Dumper: ".Dumper($self->{tc_hash}));
}

#--------------------------------------------------------------------
sub aggr_add_groups {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $add_raid_groups = $self->{tc_hash}->{add_raid_groups};
    my $filer_vol = $self->get('filer_vol');
    my $aggr_create = $self->{tc_hash}->{aggr_create};

    my @old_group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
    $self->debug("$self->{test}: aggr_add_groups: old_group_list: @old_group_list");

    my @rgs = keys %$add_raid_groups;
    @rgs = sort @rgs;
    foreach my $rg (@rgs) {
        my $rg_info = $add_raid_groups->{$rg};
        die("$self->{test}: Disk count is required for aggr add operation") if(!defined($rg_info->{disk_count}));
        my %aggr_add_args = (api_set_obj => $api_set_obj,
                            filer_vol   => $self->get('filer_vol'),
                            group_name  => 'new',
                        );
        my $options  = (defined($rg_info->{checksum_type})) ? "-f -c $rg_info->{checksum_type}" : '-f';
        $options .= " -T $rg_info->{storage_type}" if(defined($rg_info->{storage_type}));
        $aggr_add_args{options} = $options;

        my $aggr_raid_size = RaidSqa_get_aggr_options(api_set_obj => $api_set_obj, filer_vol => $filer_vol, options => 'raidsize');
        if($rg_info->{type} eq 'incomplete') {
            if($rg_info->{disk_count} >= $aggr_raid_size) {
                die("$self->{test}: add_raid_groups: can not create incomplete raid group as disk_count is not less than raid_size of aggr");
            }
        } elsif($rg_info->{type} eq 'complete') {
            if($rg_info->{disk_count} < $aggr_raid_size) {
                die("$self->{test}: add_raid_groups: can not create complete raid group as disk_count is less than raid_size of aggr");
            }
        }
        
        my @spare_list = $self->get_spares_by_filter(
                            checksum_type => $rg_info->{checksum_type}, 
                            storage_type  => $rg_info->{storage_type},
                            bps           => $rg_info->{bps}, 
                            rpm           => $rg_info->{rpm}
                        );
        my @disk_list  = @spare_list[0..$rg_info->{disk_count}-1];
        my @disk_list1 = @spare_list[$rg_info->{disk_count}..(2*$rg_info->{disk_count}-1)] if($aggr_create->{mirror});

        if (defined($rg_info->{disk_list}) and scalar(@{$rg_info->{disk_list}}) == 0) {            
            $aggr_add_args{disk_list}   = \@disk_list;
            $aggr_add_args{disk_list1}  = \@disk_list1;
        } elsif(defined($rg_info->{disk_count})) {
            $aggr_add_args{disk_size}   = $rg_info->{disk_size} if(defined $rg_info->{disk_size});
            $aggr_add_args{disk_count}  = ($aggr_create->{mirror} + 1) * $rg_info->{disk_count};
        } else {
            die("$self->{test}: Either disk_count or disk_list should be mentioned");
        }

        my $result = RaidSqa_aggr_add(%aggr_add_args);
        if(!$result) {
            if(!defined($rg_info->{disk_list}) && defined($rg_info)) {
                $Log->warn("$self->{test}: Aggr add with disk count failed, trying 'aggr add' with disk list and -f option");
                $aggr_add_args{disk_count} = undef;
                $aggr_add_args{disk_list}   = \@disk_list;
                $aggr_add_args{disk_list1}  = \@disk_list1;
                $result = RaidSqa_aggr_add(%aggr_add_args);
            }
        }
        
        if(!$result) {
            die("$self->{test}: Aggr add failed for raid_group $rg");
        }

        #Wait for aggr disks to complete zeoring
        RaidSqa_wait_for_aggr_disks_zeroing_completion(api_set_obj => $api_set_obj, filer_vol => $self->get('filer_vol'));

        my @group_list = RaidSqa_get_group_list(api_set_obj=>$api_set_obj, filer_vol=>$self->get('filer_vol'));
        $self->debug("$self->{test}: aggr_add_groups: new_group_list: @group_list");

        if(scalar(@group_list) == scalar(@old_group_list)) {
            die("$self->{test}: Raid group could not be added to ".$self->get('filer_vol'));
        }

        foreach my $group (@group_list) {
            if(!grep /$group/, @old_group_list) {
                $Log->comment("$self->{test}: new group $group added to test volume");
                $self->{tc_hash}->{add_raid_groups}->{$rg}->{rg_name} = $group;
                @old_group_list = @group_list;
            }
        }
    }
    $self->debug("$self->{test}: aggr_add_groups: Dumper: ".Dumper($self->{tc_hash}->{add_raid_groups}));
}

#--------------------------------------------------------------------
sub execute_aggr_add_cmd {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $aggr_add_hash = $self->get_hash('aggr_add_hash');
    my $aggr_add_params = $self->get_hash('aggr_add_params');

    my ($response, $output, $command);
    my %args = (api_set_obj => $api_set_obj);
    my @disk_list = ();
    push(@disk_list, @{$aggr_add_params->{disk_list}}) if(defined($aggr_add_params->{disk_list}));
    push(@disk_list, @{$aggr_add_params->{disk_list1}}) if(defined($aggr_add_params->{disk_list1}));
    $args{disk_list} = \@disk_list if(scalar(@disk_list));
    RaidSqa_wait_for_spare_disks_zeroing_completion(%args);

    if($self->is_cmode()) {
        try {
            $api_set_obj->set_privilege("privilege-level" => 'diag');
            if($self->is_cmode()) {
                $response = $api_set_obj->storage_aggregate_add_disks(%$aggr_add_hash);
            } else {
                $response = $api_set_obj->aggr_add(%$aggr_add_hash);
            }
            $output   = $response->get_raw_output();
            $command  = $response->get_command();
        } catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $response = $exception_object->get_response_object();
            $output   = $response->get_raw_output();
            $command  = $response->get_command();
            $Log->comment("$self->{test}: Aggr add failed with output");
        }
    } else {    #TODO:: Remove this and use nacl object for both 7mode and Cmode
        $command = $self->get('aggr_add_command');
        $output = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>$command);
        if($output =~ /^usage:\r*\n/){
            die("$self->{test}: Command syntax is not correct for aggr add");
        }
    }
    $self->set('aggr_add_output'=>$output);
    $self->set('aggr_add_command'=>$command);
}

#--------------------------------------------------------------------
sub save_test_results {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $filer_vol = $self->get('filer_vol');
    my $aggr_add_result = $self->get_hash('aggr_add_result');
    my $test_results = $aggr_add_result->{actual};
    my (@new_group_list, @new_disk_list, @old_disk_list, @old_group_list);

    #Command execution status
    my $aggr_add_output = $self->get('aggr_add_output');
    if($aggr_add_output =~ /Addition of \d (disk|disks) to the aggregate has completed/) {
        $test_results->{aggr_add_status} = 1;
    } else {
        $test_results->{aggr_add_status} = 0;
    }

    #Update checksum type of aggr
    $test_results->{final_checksum_type} = RaidSqa_get_aggr_cksum_type(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol);
    my $old_rg_count = 0;
    my $aggr_info = $self->get_hash('test_aggr_info');
    foreach my $plex (keys %$aggr_info) {
        my @rgs = keys %{$aggr_info->{$plex}};
        push(@old_group_list, @rgs);
        $old_rg_count += scalar(@rgs);
        foreach my $rg (@rgs) {
            push (@old_disk_list, @{$aggr_info->{$plex}->{$rg}});
        }
    }
    my %old_disk_list = map {$_ => 1} @old_disk_list;
    my %old_group_list = map {$_ => 1} @old_group_list;
    my @disk_list = RaidSqa_get_disk_list(api_set_obj   => $api_set_obj, filer_vol => $filer_vol);
    my @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
    my $new_rg_count = scalar(@group_list);
    @new_disk_list = grep !defined($old_disk_list{$_}), @disk_list;
    @new_group_list = grep !defined($old_group_list{$_}), @group_list;

    $test_results->{new_rg_count} = $new_rg_count - $old_rg_count;
    $test_results->{added_disk_list} = [@new_disk_list];
    $test_results->{added_group_list} = [@new_group_list];
    $test_results->{aggr_bits} = RaidSqa_get_aggr_bits(api_set_obj => $api_set_obj, filer_vol=>$filer_vol);
    $self->debug("$self->{test}: actual test results: DUMPER".Dumper($test_results));
    $aggr_add_result->{actual} = $test_results;
    $self->set_hash(aggr_add_result=>$aggr_add_result);
}

#--------------------------------------------------------------------
sub validate_test_results {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    $self->update_rc_object();

    my $aggr_add_result = $self->get_hash('aggr_add_result');
    my $filer_vol = $self->get('filer_vol');
    my $act_results = $aggr_add_result->{actual};
    my $exp_results = $aggr_add_result->{expected};
    my @new_disk_list = @{$act_results->{added_disk_list}};
    my @new_group_list = @{$act_results->{added_group_list}};
    my $aggr_add_params = $self->get_hash('aggr_add_params');

    #==========================================================
    #Validate aggr add status
    if(defined($exp_results->{aggr_add_status})) {
        if($act_results->{aggr_add_status} == $exp_results->{aggr_add_status}) {
            $Log->comment("$self->{test}: Validate aggr add status: ok");
        } else {
            $self->debug("$self->{test}: Dumper: test results: ".Dumper($aggr_add_result));
            die("$self->{test}: Validate aggr add status: failed");
        }
    }
    
    #==========================================================
    #Validate aggr add command output
    if(defined($exp_results->{command_output})) {
        if($act_results->{command_output} =~ /$exp_results->{command_output}/i) {
            $Log->comment("$self->{test}: Validate aggr command output: ok");
        } else {
            $self->debug("$self->{test}: Dumper: test results: ".Dumper($aggr_add_result));
            die("$self->{test}: Validate aggr command output: failed");
        }
    }

    #==========================================================
    #validate new rg count
    if(defined($exp_results->{new_rg_count})) {
        my $min_rg_count = $exp_results->{new_rg_count}->[0];
        if($min_rg_count != -1 && $act_results->{new_rg_count} < $min_rg_count) {
            $self->debug("$self->{test}: Dumper: test results: ".Dumper($aggr_add_result));
            die("$self->{test}: new rg count is less that minimum expected");
        }

        my $max_rg_count = $exp_results->{new_rg_count}->[1];
        if(defined($max_rg_count) && $max_rg_count != -1 && $act_results->{new_rg_count} > $max_rg_count) {
            $self->debug("$self->{test}: Dumper: test results: ".Dumper($aggr_add_result));
            die("$self->{test}: new rg count is more that expected");
        }
        $Log->comment("$self->{test}: new rg count: ok");
    }

    #==========================================================
    #Validate final checksum type of aggr after aggr add operation
    if(defined($exp_results->{final_checksum_type})) {
        if($act_results->{final_checksum_type} eq $exp_results->{final_checksum_type}) {
            $Log->comment("$self->{test}: Aggr final checksum type: ok");
        } else {
            $self->debug("$self->{test}: Dumper: test results: ".Dumper($aggr_add_result));
            die("$self->{test}: Aggr final checksum type: failed");
        }
    }

    #==========================================================
    #Check aggr bits 
    if($aggr_add_params->{'64bit_upgrade'} == 1 && $aggr_add_params->{'opt_upgrade'} ne "check") {
        #Check aggr size and update aggr_bits to 64 if size is more than 16TB
        my $aggr_space = $self->get_aggr_show_space(filer_vol=>$filer_vol, option => "-t");
        my $size = $aggr_space->{'Total space'};
        if(int($size) >= 16) {
            $exp_results->{aggr_bits} = 64;
        } else {
            $exp_results->{aggr_bits} = 32;
        }
    }
    
    if(defined($exp_results->{aggr_bits})) {
        if($act_results->{aggr_bits} eq $exp_results->{aggr_bits}) {
            $Log->comment("$self->{test}: Aggr bits: ok");
        } else {
            $self->debug("$self->{test}: Dumper: test results: ".Dumper($aggr_add_result));
            die("$self->{test}: Add bits: failed");
        }
    }

    #==========================================================
    #Check the types of the disks added
    if($exp_results->{aggr_add_status}) {
        my @possible_storage_types = @{$exp_results->{storage_type}} if(defined($exp_results->{storage_type}));
        my @possible_bps = @{$exp_results->{bps}} if(defined($exp_results->{bps}));
        foreach my $disk (@new_disk_list) {
            my $disk_obj = $self->get_rc_object(disk_name => $disk);
            die("$self->{test}: validate_test_results: Could not get rc object for disk $disk") if(!defined($disk_obj));
            if(scalar @possible_storage_types) {
                my $disk_storage_type = $disk_obj->parameter("disk_type");
                if(!grep /$disk_storage_type/, @possible_storage_types) {
                    die("$self->{test}: Storage type of newly added disk $disk is not one of allowed storage_type: $disk_storage_type");
                }
            }

            if(scalar @possible_bps) {
                my $disk_bps = $disk_obj->parameter("bps");
                if(!grep /$disk_bps/, @possible_bps) {
                    die("$self->{test}: BPS of newly added disk $disk is not one of allowed value: $disk_bps");
                }
            }
        }
    }

    #==========================================================
}

#--------------------------------------------------------------------
sub setup_aggr_add_hash {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $params = $self->get_hash('aggr_add_params');
    my $mirror = $self->{tc_hash}->{aggr_create}->{mirror};
    #my $self->{testdb}->{aggr_add_hash} = \%aggr_add_hash;
    my %aggr_add_hash = ();

    if($self->is_cmode()) {
        $aggr_add_hash{aggregate} = $self->get('filer_vol');
        $aggr_add_hash{'force-spare-pool'} = $params->{force} if(defined $params->{force});
        $aggr_add_hash{'checksum'} = $params->{checksum_type} if(defined $params->{checksum_type});
        $aggr_add_hash{'raid-group'} = $params->{group_name} if(defined $params->{group_name});
        $aggr_add_hash{'disk-type'} = $params->{storage_type} if(defined $params->{storage_type});
        if(defined($params->{disk_list})) {
            $aggr_add_hash{'disks'} = join(", ", @{$params->{disk_list}});
        } elsif(defined($params->{disk_count})) {
            $aggr_add_hash{'disk-size'} = $params->{disk_size} if(defined $params->{disk_size});
            $aggr_add_hash{'disk-count'} = ( $mirror + 1) * $params->{disk_count};
        }
        $self->set_hash(aggr_add_hash => \%aggr_add_hash);
    } else {
        my $cmd = 'priv set -q diag; aggr add '. $self->get('filer_vol'). " ";
        $cmd .= " -f" if($params->{force} == 1);
        $cmd .= " -c $params->{checksum_type}" if(defined($params->{checksum_type}));
        $cmd .= " -g $params->{group_name}" if(defined($params->{group_name}));
        $cmd .= " -T $params->{storage_type}" if(defined($params->{storage_type}));

        if(defined($params->{disk_list})) {
            my $disk_args = join(" ", @{$params->{disk_list}});
            my $disk_args1 = join(" ", @{$params->{disk_list1}}) if(defined($params->{disk_list1}));
            $cmd .= " -d ".$disk_args;
            $cmd .= " -d ".$disk_args1 if(defined($disk_args1));
        } elsif(defined($params->{disk_count})) {
            my $disk_count = ($mirror + 1) * $params->{disk_count};
            $cmd .= " $disk_count";
            if(defined($params->{disk_size})) {$cmd .= "\\\@$params->{disk_size}";}
        }
        $Log->comment("TODO:: REMOVE: CMD: $cmd");
        $self->set(aggr_add_command => $cmd);

        #TODO, use api_set_obj to execute the command
        # $aggr_add_hash{aggregate} = $self->get('filer_vol');
        # $aggr_add_hash{'force-spare-pool'} = $params->{force} if(defined $params->{force});
        # $aggr_add_hash{'checksum'} = $params->{checksum_type} if(defined $params->{checksum_type});
        # $aggr_add_hash{'raid-group'} = $params->{group_name} if(defined $params->{group_name});
        # $aggr_add_hash{'disk-type'} = $params->{storage_type} if(defined $params->{storage_type});
        # if(defined($params->{disk_list})) {
            # $aggr_add_hash{'disks'} = $params->{disk_list};
        # } elsif(defined($params->{disk_count})) {
            # $aggr_add_hash{'disk-size'} = $params->{disk_size} if(defined $params->{disk_size});
            # $aggr_add_hash{'disk-count'} = $params->{disk_count};
        # }
    }
}

#--------------------------------------------------------------------
#helper methods:
#----------------------------------------------------
sub is_enable_optional_storage_type {
    my $self = shift;
    my $enabled = $self->get('enable_optional_storage_type');
    return defined($enabled) ? $enabled : 0 ;
}


#==================================================================================
package AzcsAggrCreate;

use vars qw(@ISA);
use strict;
no strict 'refs'; 
use warnings;
use Tharn qw($Log);
use RaidSqav3 qw(AUTOLOAD);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidLW;
use RaidInfo;
use Data::Dumper;
use Utils;

BEGIN {
    @ISA = qw(RaidAzcsUtils);
}

#-------------------------------------------------------------------------------
sub new {
    my $self = shift if(scalar(@_) % 2);
    my %self = validate(@_, {
                                api_set_obj => {type => OBJECT, isa=> "NACL::APISet"},
                                debug       => {type => SCALAR, default=>0, optional=>1},
                            });
	#$self->{aggr_create_command} = '';
    $self = \%self;
    my $api_set_obj = $self->{api_set_obj};
    my $host;
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        my $conntype = ($host->{hosttype} =~ /ng/) ? "#ssh" : "#rsh";
        $self->{conn_obj} = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $self->{conn_obj} = $api_set_obj->{connection};
    }
    bless $self, 'AzcsAggrCreate';
    $self->init();
    return $self;
}

#-------------------------------------------------------------------------------
sub init {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $conn_obj = $self->{conn_obj};
    my $rc_object = new RaidInfo($conn_obj);
    
	 #init test data base
    $self->{testdb}->{key_mapping} = {
        filer_vol                    => '$self->{tc_hash}->{aggr_info}->{filer_vol}',
        aggr_checksum_type           => '$self->{tc_hash}->{aggr_info}->{checksum_type}',
        aggr_type                    => '$self->{tc_hash}->{aggr_info}->{aggr_type}',
        aggr_raid_type               => '$self->{tc_hash}->{aggr_info}->{raid_type}',
        aggr_mirror                  => '$self->{tc_hash}->{aggr_info}->{mirror}',        
        test_rc_object               => '$self->{testdb}->{rc_object}',
        test_aggr_info               => '$self->{testdb}->{aggr_info}',	
        test_spares_info             => '$self->{testdb}->{spare_info}',     
		aggr_create_command			 => '$self->{testdb}->{aggr_create_command}',
		aggr_create_output			 => '$self->{testdb}->{aggr_create_output}',
		
		};

    $self->{testdb}->{aggr_info} = undef;
    $self->{testdb}->{spare_info} = undef;
    $self->{testdb}->{rc_object} = undef;

    #create raid_config object and assign to $self
    $self->{testdb}->{rc_object} = $rc_object; 
}

#-------------------------------------------------------------------------------
sub run_test {
    my $self = shift;
    my %args = validate(@_, {
                                tc_hash     => {type => HASHREF},
                                test        => {type => SCALAR, default=>"AzcsAggrCreate", optional=>1},
                            });
                            
    $self->{tc_hash} = $args{tc_hash};
    $self->{test} = $args{test};
    my $tc_hash = $self->{tc_hash};
    
    eval {
		$self->step("Aggr_Create : Calling function for setup"); 
        $self->setup();
		$self->step("Aggr_Create : populating test data");
		$self->populate_test_data();		
		$self->step("Aggr_Create : Executing aggregate create command");
		$self->execute_aggr_create_cmd();
		$self->step("Aggr_Create : Saving execution status and aggr info");
		$self->save_test_results();
		$self->step("Aggr_Create : Validating test results");
		$self->validate_test_results();       
    };
    
    if($@) {
		$self->{error} = $@;		
		die("FAIL: $self->{error}");
    }
}


#-------------------------------------------------------------------------------
sub setup {
	my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
	my $tc_hash = $self->{tc_hash};
	
	$Log->comment(" $self->{test} : Aggr Create: setup : updating total disk count");
	$self->set_reqd_disk_count();
	
	## TODO : any kind of pre-checks if required	
    # if only one disk type defined, only that should be available in filer
    # if mirror=1 and pool is undef, check in both the pools
    # if count is specified, check for number of spares available
    # handling FC/SAS   
	if (defined($tc_hash->{filer_disk_condition}) ) {
		$Log->comment("$self->{test}: setup : checking for filer disk condition");
        my @all_spares = RaidSqa_get_disk_list(api_set_obj => $api_set_obj, spare_only => 'yes');
        my @spares_left = ();
        
        ## populate disk list of each type
		foreach my $disk_type_hash (keys(%{$tc_hash->{filer_disk_condition}})) {
			my %disk_hash = %{$tc_hash->{filer_disk_condition}->{$disk_type_hash}};
			my $disk_type = "$disk_hash{'type'}". " ". "$disk_hash{'bps'}";
			$Log->comment("$self->{test}: setup : checking for disk of type $disk_type");
			my @param_disk_list=();           
            @param_disk_list = $self->get_spares_by_filter(storage_type    => $disk_hash{'type'},														
															bps             => $disk_hash{'bps'},
															rpm             => $disk_hash{'rpm'},
															disk_list       => \@all_spares,
															refresh         => 1,);
            if (defined($disk_hash{'size'})){
				@param_disk_list = $self->get_spares_by_rc_parameter(parameter_name  => 'size',
																parameter_value => $disk_hash{'size'},
                                                                disk_list       => \@param_disk_list,
																refresh => 1, );
			}
            $tc_hash->{filer_disk_condition}->{$disk_type_hash}->{disk_list} = \@param_disk_list ;
            foreach my $temp_disk (@all_spares) {
                next if (grep /$temp_disk/,@param_disk_list);
                push (@spares_left, $temp_disk) if (!grep /$temp_disk/,@param_disk_list);
            }
            $Log->comment("Aka : setup : param_disk_list = @param_disk_list, left spares : @spares_left");
            @all_spares = @spares_left;            
		}		
        
        #remove all spares that are not required
        foreach my $temp_disk (@all_spares) {
            $Log->comment("aggr create : setup : removing disks @all_spares as they are not required for current test");
            RaidSqa_disk_remove_ownership(api_set_obj => $api_set_obj, disk_list => \@all_spares);
            $tc_hash->{cleanup}->{disk_reassign} = 1;
            $tc_hash->{cleanup}->{removed_disk_list} = \@all_spares;
        }
        
        ## check for pool condition
        
        ## check for max/equal/sufficient condition	
		foreach my $disk_type_hash (keys(%{$tc_hash->{filer_disk_condition}})) {
			## reference disk type = $disk_Type_hash
			my %ref_disk_hash = %{$tc_hash->{filer_disk_condition}->{$disk_type_hash}};
			if ($ref_disk_hash{'is_sufficient'}) {
				my $req_disk_count = $tc_hash->{aggr_info}->{total_disk_count};
				my @disk_array = @{$ref_disk_hash{disk_list}};
				if (scalar @disk_array >= $req_disk_count){
					$Log->comment(" $self->{test} : Aggr Create: setup : sufficient spares available");
				} 
			}
			if ($ref_disk_hash{'is_max'}) {
				$Log->comment(" $self->{test} : Aggr Create: setup :checking for max");
				my $count = scalar (@{$ref_disk_hash{disk_list}});
				my $max_flag = 1;
				foreach my $temp_disk_hash (keys(%{$tc_hash->{filer_disk_condition}})) {
					next if ($temp_disk_hash eq $disk_type_hash);
					$max_flag = 0 if (scalar @{$tc_hash->{filer_disk_condition}->{$temp_disk_hash}->{disk_list}} >= $count);					
				}
				if ($max_flag){
					$Log->comment(" $self->{test} : Aggr Create: setup : disk type is maximum in number");
				} else {
					die(" $self->{test} : Aggr Create: setup : disk type is not maximum in number");
				}
			}
			if ($ref_disk_hash{'is_equal'}) {
				$Log->comment(" $self->{test} : Aggr Create: setup : checking for equal");
				my $count = scalar (@{$ref_disk_hash{disk_list}});
				my $equal_flag = 1;
				foreach my $temp_disk_hash (keys(%{$tc_hash->{filer_disk_condition}})) {
					next if ($temp_disk_hash eq $disk_type_hash);
					$equal_flag = 0 if ( ($tc_hash->{filer_disk_condition}->{$temp_disk_hash}->{is_equal} == 1) && 
									(scalar @{$tc_hash->{filer_disk_condition}->{$temp_disk_hash}->{disk_list}} >= $count) );					
				}
				if ($equal_flag){
					$Log->comment(" $self->{test} : Aggr Create: setup : expected disk types are equal in number");
				} else {
					die(" $self->{test} : Aggr Create: setup : some disk types expected to be equal in number are not equal");
				}
			}			
		}
	} else {
		$Log->comment("$self->{test}: setup : filer disk condition not specified - directly proceeding with test");
	}
	
	$Log->comment("$self->{test} : Aggr Create: setup : dumping hash");
    $self->debug("Dumper:: populate_data::tc_hash ". Dumper($tc_hash));
		
}

#-------------------------------------------------------------------------------
sub set_reqd_disk_count {
	my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
	my $tc_hash = $self->{tc_hash};
	
	my $disk_count = 0;
	$Log->comment("$self->{test} : set_reqd_disk_count : getting total disk count information");
	foreach my $plex_info_key (keys %{$tc_hash->{aggr_info}->{disk_info}}) {		
		my @plex_info_array = @{$tc_hash->{aggr_info}->{disk_info}->{$plex_info_key}}; 
				
		## $disk_set is a reference to an anonymous hash
		foreach my $disk_set (@plex_info_array) {
			my $type_count = $disk_set->{disk_count} ;
			$disk_count += $type_count;
		}
	}	
	$Log->comment("$self->{test} : set_reqd_disk_count : Total number of disks required : $disk_count ");
	$tc_hash->{aggr_info}->{total_disk_count} = $disk_count;	
}

#-------------------------------------------------------------------------------
sub populate_test_data {
	my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
	my $tc_hash = $self->{tc_hash};
	
	my $param_string = '';
	my $aggr_create_command = '';
	my @aggr_create_params = ();
	my $disk_info_ref = '';
	
	
	$Log->comment(" $self->{test}: populate_test_data : object info".Dumper($self->{tc_hash}));
	## get aggr create or vol create
	$param_string = $self->get_aggr_create_string($tc_hash->{aggr_info}->{aggr_type});	
	push(@aggr_create_params, $param_string); 
	## get param value for all other keys
	my $aggr_name = $tc_hash->{aggr_info}->{filer_vol};			
	push(@aggr_create_params, $aggr_name);
	
	
	foreach my $param_key (keys %{$tc_hash->{aggr_info}} ) {
		logcomment("AZCS_UTILS : AGGR_CREATE: working on KEY : $param_key");
		next if ($param_key eq 'aggr_type');
		next if ($param_key eq 'filer_vol');
				
		if ($param_key ne 'disk_info') {
			$param_string =  $self->get_string_for_param($param_key, $tc_hash->{aggr_info}->{$param_key});		
			push(@aggr_create_params, $param_string);
		} else {
			#get reference to disk info hash
			$disk_info_ref = $tc_hash->{aggr_info}->{disk_info};									
		}
	}
	$aggr_create_command = join(" ",@aggr_create_params);	
	$param_string = $self->get_disk_info();	
	$aggr_create_command = "$aggr_create_command" . " $param_string";		
	
	## update TC hash with the command to be used
	$tc_hash->{aggr_create_command} = $aggr_create_command;	
	## update object with command string	
	$self->{tc_hash}->{aggr_create_command} = $aggr_create_command;
	
}

#-------------------------------------------------------------------------------
sub get_aggr_create_string {
	my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
	my $tc_hash = $self->{tc_hash};
	
	my $aggr = $tc_hash->{aggr_info}->{aggr_type};
	$Log->comment("$self->{test}: get_aggr_create_string : Got value of aggr type: $aggr");
	my $aggr_string = '';
	if($self->is_cmode()) {
		$aggr_string = ($aggr == 1) ? "storage aggregate create" : "volume create";
	} else {
		$aggr_string = ($aggr == 1) ? "aggr create" : "vol create";
	}
	$Log->comment("$self->{test}: get_aggr_create_string :  returning string : $aggr_string");
	return $aggr_string;
}

#-------------------------------------------------------------------------------
sub get_string_for_param {
	my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
	
	
	my $param_name = $_[0];
	my $param_value = $_[1];
	my $param_string = "#####";
	$Log->comment("$self->{test}: get_string_for_param :  param_name : $param_name, param_value : $param_value");
	my %param_hash_7mode =  (								
								raid_type => 	{ 'undef' => "",
												 'default' => "-t raid_dp",
												 'raid_dp'	=> "-t raid_dp",
												 'raid4' => "-t raid4",
												 '4'	=> "-t raid4",
												 'dp'	=> "-t raid_dp",
												 },
								raid_size => { 'undef' => "",
												 'default' => "", ## TODO : need an entire logic for addressing this
												},
								aggr_bits => { 'undef' => "",
												 'default' => "-B 64",
												 '32'		=> "-B 32",
												 '64'		=> "-B 64",
												},
								aggr_mirror => { 'undef' => "",
											  '1'     => "-m",
											  '0'	  => "",
											 },
								checksum_type => { 'undef' => "",
													 'block' => "-c block",
													 'zoned' => "-c zoned",
													 'advanced_zoned' => "-c advanced_zoned",
													 'mixed' => "",
													 'hybrid' => "",													 
													},
								vserver	=> { 'undef' => "",
											 'default' => "vs0"												
											},
								disk_type => { 'undef' => "",
												 'ata'	=> "-T ATA",
												 'bsas' => "-T BSAS",
												 'fcal' => "-T FCAL",
												 'sas' => "-T SAS",
												 'sata' => "-T SATA",
												 'ssd' => "-T SSD",
												 'lun' => "", ## TODO : need to handle this
											},
								disk_rpm => {	'undef' => "",
												'else' => "-R integer", ## TODO: need to handle this
											  },
							);
	my %param_hash_Cmode =  (								
								raid_type => 	{ 'undef' => "",
												 'default' => "raid_dp",
												 'raid_dp'	=> "raid_dp",
												 'raid4' => "raid4",
												 '4'	=> "raid4",
												 'dp'	=> "raid_dp",
												 },
								raid_size => { 'undef' => "",
											   'default' => "", 
												},
								aggr_bits => { 'undef' => "",
												'default' => "64",
											 '32'		=> "32",
											 '64'		=> "64",
												},
								aggr_mirror => { 'undef' => "",
											  '1'     => "",
											  '0'	  => "",
											 },
								checksum_type => { 'undef' => "",
													 'block' => "block",
													 'zoned' => "zoned",
													 'advanced_zoned' => "advanced_zoned",
													 'mixed' => "",
													 'hybrid' => "",													 
													},
								vserver	=> { 'undef' => "",
											 'default' => "vs0"												
											},
								disk_type => { 'undef' => "",
												 'ata'	=> "ATA",
												 'bsas' => "BSAS",
												 'fcal' => "FCAL",
												 'sas' => "SAS",
												 'sata' => "SATA",
												 'ssd' => "SSD",
												 'lun' => "", ## TODO : need to handle this
											},
								disk_rpm => {	'undef' => "",
												'else' => "integer", ## TODO: need to handle this
											  },
							);
	$param_name = lc $param_name;	
	$param_value = lc $param_value;	
	if($self->is_cmode()) {
		#parse through C-mode hash, return key value pair
		# or call function to create another hash, let tc_data have reference to that hash
	} else {
		foreach my $aggr_keys (keys %param_hash_7mode) {
			if ($aggr_keys eq  $param_name) {
				$param_string = $param_hash_7mode{$aggr_keys}->{$param_value};
			}
			if ($param_name eq 'raid_size'){
				$param_string = "-r $param_value" if ($param_value =~ /\d+/);			
			} elsif ($param_name eq 'disk_rpm'){
				$param_string = "-R $param_value" if ($param_value =~ /\d+/ );			
			} elsif ($param_name eq 'vserver' && ($self->is_cmode()) ) {
				$param_string = "-vserver $param_value" if ($param_value ne undef && $param_value ne "default" );						
			} else {
				## value of param string should not be undef in that case
				# if (!defined ($param_string)){
					# my @possible_param_values = keys %$aggr_keys ;
					  # die("$self->{test}: Could not get string value - Possible reason : Incorrect value for parameter $param_name : 
							# Sent : $param_value , expected : @possible_param_values");
				# }
			}
			last if ($param_string ne "#####");
		}
	}
	$param_string = " " if ($param_string eq "#####");
	$Log->comment("$self->{test}: get_string_for_param : returning string $param_string");
	return $param_string;	
}

#-------------------------------------------------------------------------------
sub get_disk_info {
	my $self = shift;
	my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    
	
	my $disk_info = $tc_hash->{aggr_info}->{disk_info};
	my %disk_info_hash = %$disk_info;
	
	## if mirrored, both plexes should be defined
	if ( (scalar keys %disk_info_hash != 2) && ($tc_hash->{aggr_info}->{mirror} == 1) ){
		die("$self->{test}: mirror config requires information for both plex");
	}
	

	
	my @disk_list1 = ();
	my @disk_list2 = ();
	

	my $param_string = '';
	my $disk_count = 0 ;
	
	
	my @plex0_set = @{$disk_info_hash{plex0}};
	
	## $disk_set is a reference to an anonymous hash
	foreach my $disk_set (@plex0_set) 
	{
		my @param_disk_list = ();		
		
			
		#if disk list is undef, disk count is to be used
		if (!defined($disk_set->{disk_list}) )
		{				
			$param_string = $disk_set->{disk_count} ;			
			# before setting this check if other params are defined and corresponding spares are available
			if (defined ($disk_set->{disk_type} )) {
				@param_disk_list = $self->get_spares_by_rc_parameter(parameter_name  => 'disk_type',
																parameter_value => $disk_set->{disk_type},
																refresh => 1, );
				warn("$self->{test}: insufficient spares available of specified disk storage_type : $disk_set->{disk_type} :
					Aggr creation might fail"	)	 if (scalar @param_disk_list < $param_string);
			}
			if (defined ($disk_set->{disk_size} ))  {
				@param_disk_list = $self->get_spares_by_rc_parameter(parameter_name  => 'used_sz',
																parameter_value => $disk_set->{disk_size},
																refresh => 1, );
				warn("$self->{test}: insufficient spares available of specified size : $disk_set->{disk_size} :
					Aggr creation might fail"	)	 if (scalar @param_disk_list < $param_string);
			}
			if (defined ($disk_set->{disk_rpm} ))  {
				@param_disk_list = $self->get_spares_by_rc_parameter(parameter_name  => 'rpm',
																parameter_value => $disk_set->{disk_rpm},
																refresh => 1, );
				warn("$self->{test}: insufficient spares available of specified rpm : $disk_set->{disk_rpm} :
					Aggr creation might fail"	)	 if (scalar @param_disk_list < $param_string);
			}
			if (defined ($disk_set->{disk_bps} )) {
				@param_disk_list = $self->get_spares_by_rc_parameter(parameter_name  => 'bps',
																parameter_value => $disk_set->{disk_bps},
																refresh => 1, );
				warn("$self->{test}: insufficient spares available of specified bps : $disk_set->{disk_bps} :
					Aggr creation might fail"	)	 if (scalar @param_disk_list < $param_string);
			}
			if (defined ($disk_set->{pool} )) {
				@param_disk_list = $self->get_spares_by_rc_parameter(parameter_name  => 'pool',
																parameter_value => $disk_set->{pool},
																refresh => 1, );
				warn("$self->{test}: insufficient spares available in specified pool : $disk_set->{pool} :
					Aggr creation might fail")	 if (scalar @param_disk_list < $param_string);
			}
			
			$disk_count = $disk_count+$param_string;
		} else {
			## if disk_list is set to an empty array, we need to get the disks based on the params defined
			@param_disk_list =  $self->get_spares_by_rc_parameter(parameter_name  => 'pool',
																parameter_value => $disk_set->{pool},
																refresh => 1, );
			
			@param_disk_list = $self->get_spares_by_filter(storage_type    => $disk_set->{disk_type},
															checksum_type   => $disk_set->{checksum_type},
															bps             => $disk_set->{disk_bps},
															rpm             => $disk_set->{disk_rpm},
															disk_list       => @param_disk_list,
															refresh         => 1,);
			die("$self->{test}: disk list is empty - cannot proceed with aggr create") if (scalar @param_disk_list == 0);
			die("$self->{test}: insufficient disks found with specified parameters - required: $disk_set->{disk_count} , available - @param_disk_list ")
																													if (scalar @param_disk_list == 0);
			$disk_count = $disk_set->{disk_count};
			@disk_list1 = splice(@param_disk_list, 0, $disk_count);
			$param_string = join(" ", @disk_list1);
			$param_string = "-d" . "$param_string";
		}
		
	}	
	return $param_string;
	
}


#--------------------------------------------------------------------
sub execute_aggr_create_cmd {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    
	
    #TODO:: Set priviledge to diag before adding disks
    my ($response, $output, $command);
	
    #try {
        $response = $api_set_obj->set_privilege("privilege-level" => 'diag');
        if($self->is_cmode()) {            
            # $response = $api_set_obj->storage_aggregate_create(%$aggr_create_args);
            # $output   = $response->get_raw_output();
            # $command  = $response->get_command();
        } else {
            #Time being, we need to execute the 
            #$response = $api_set_obj->aggr_add(%$aggr_add_args);
            # $output   = $response->get_raw_output();
            # $command  = $response->get_command();
            
           # $command = $self->get('aggr_create_command');
		    $command = $tc_hash->{aggr_create_command};
            $output = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>$command);
        }
    # } catch NACL::APISet::Exceptions::ResponseException with {
        # # my $exception_object = shift;
        # # $response = $exception_object->get_response_object();
        # # $output   = $response->get_raw_output();
        # # $command  = $response->get_command();
        # # $Log->comment("$self->{test}: Aggr create failed with output");
    # }
	$Log->comment("$self->{test}: execute_aggr_create_cmd : object info".Dumper($self->{tc_hash}));
	#logcomment(" $self->{test}: AGGR_UTILS : AGGR_CREATE : here :output: $output");
    #$self->set('aggr_create_output'=>$output);
	#$self->set('aggr_create_command'=>$command);
    #$self->set('aggr_create_output'=>$output);
	#$tc_hash->{actual_output}->{output_string} = $output;
	#$self->$tc_hash->{actual_output}->{output_string} = $output;
	
	$tc_hash->{actual_output}->{output_string} = $output;
	$self->{tc_hash}->{actual_output}->{output_string} = $output;
	## update object with command string
	## function : setAggrCreatecommand();
	#$self->{tc_hash}->{aggr_create_command} = $aggr_create_command;
	
	$Log->comment("$self->{test}: execute_aggr_create_cmd : object info".Dumper($self->{tc_hash}));
	
}   

#--------------------------------------------------------------------
sub save_test_results {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};	
	
	
    my $filer_vol = $tc_hash->{aggr_info}->{filer_vol};	
	$Log->comment("$self->{test}: save_test_results : filer_vol : $filer_vol");
	if (RaidSqa_check_aggr_exist_anytype(api_set_obj => $api_set_obj, filer_vol => $filer_vol, )) {
		$tc_hash->{actual_output}->{aggr_create_success} = 1;	
	} else {
		$tc_hash->{actual_output}->{aggr_create_success} = 0;	
	}
	
	my $aggr_rc_object = $self->get_rc_object(filer_vol => $filer_vol);
	$Log->comment("$self->{test}: save_test_results : aggr info".Dumper($aggr_rc_object));	
	
	my $parameter_value = $aggr_rc_object->parameter('aggr_format');	
	$tc_hash->{actual_output}->{aggr_bits} = $parameter_value;	
		
	$parameter_value = $aggr_rc_object->parameter('raidtree_type');
	$tc_hash->{actual_output}->{raid_type} = $parameter_value;
		
	$parameter_value = $aggr_rc_object->parameter('cksum_type');
	$tc_hash->{actual_output}->{checksum_type} = $parameter_value ;
		
	$parameter_value = $aggr_rc_object->parameter('mirror_state');	
	$tc_hash->{actual_output}->{mirrored} = $parameter_value;	
	
	if ( ($tc_hash->{expected_output}->{raid_type})  =~ /raid_dp|dp/i){
		$tc_hash->{expected_output}->{raid_type} = 'raidrd';
	}
		
	$Log->comment("$self->{test}: save_test_results : object info".Dumper($self->{tc_hash}));
}

#--------------------------------------------------------------------
sub validate_test_results {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
	my $tc_hash = $self->{tc_hash};
		
    #Comapare the output if the expected output is defined
    #Check the types of the disks added
    #check aggr_bits
	$Log->comment("$self->{test}: validate_test_results : object info".Dumper($self->{tc_hash}->{actual_output}));
	
	if ($tc_hash->{expected_output}->{aggr_create_success} ){
		if ($tc_hash->{expected_output}->{mirrored} eq $tc_hash->{actual_output}->{mirrored} ) {
			$Log->comment("$self->{test}: validate_test_results : mirror state match");
		}
		if ($tc_hash->{expected_output}->{aggr_bits} eq $tc_hash->{actual_output}->{aggr_bits} ) {
			$Log->comment("$self->{test}: validate_test_results : aggr bits match");
		}
		if ($tc_hash->{expected_output}->{checksum_type} eq $tc_hash->{actual_output}->{checksum_type} ) {
			$Log->comment("$self->{test}: validate_test_results : checksum type match");
		}
		if ($tc_hash->{expected_output}->{raid_type} eq $tc_hash->{actual_output}->{raid_type} ) {
			$Log->comment("$self->{test}: validate_test_results : raid type matches");
		}		
		if ($tc_hash->{expected_output}->{check_rg_count}){
			if ($tc_hash->{expected_output}->{rg_count} eq $tc_hash->{actual_output}->{rg_count} ) {
				$Log->comment("$self->{test}: validate_test_results : rg count match");		
			}
		}
	} 
	
	if ($tc_hash->{expected_output}->{check_output_string}){
		if ($tc_hash->{expected_output}->{output_string} eq $tc_hash->{actual_output}->{output_string} ) {
			$Log->comment("$self->{test}: validate_test_results : output string match");
		}
	}	
	
} 


package AzcsAggrMirror;

use vars qw(@ISA);
use strict;
use warnings;
use Tharn qw($Log);
use RaidSqav3 qw(AUTOLOAD);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidLW;
use RaidInfo;
use Data::Dumper;
use Utils;
use Storage::SASUtils qw(get_disks);

BEGIN {
    @ISA = qw(RaidAzcsUtils);
}

#-------------------------------------------------------------------------------
sub new {
    my $self = shift if(scalar(@_) % 2);
    my %self = validate(@_, {
                                api_set_obj => {type => OBJECT, isa=> "NACL::APISet"},
                                debug       => {type => SCALAR, default=>0, optional=>1},
                            }
                        );
    $self = \%self;

    my $api_set_obj = $self->{api_set_obj};
    my $conntype = ($api_set_obj->{mode} eq "CMode") ? "#ssh" : "#rsh";

    if(defined($api_set_obj->{hostobj})) {
        $self->{conn_obj} = make_conn($api_set_obj->{hostobj}, $conntype);
    } elsif (defined $api_set_obj->{connection}) {
        $self->{conn_obj} = $api_set_obj->{connection};
    }

    if(!defined($self->{conn_obj})) {
        die("AzcsAggrMirror ::new: Could not create connection object");
    }

    bless $self, 'AzcsAggrMirror';
	$self->init();
    return $self;
}

#-------------------------------------------------------------------------------
sub init {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $conn_obj = $self->{conn_obj};
    my $rc_object = new RaidInfo($conn_obj);

    #init test data base
    $self->{testdb}->{key_mapping} = {
        filer_vol                    => '$self->{tc_hash}->{aggr_create}->{filer_vol}',
        aggr_checksum_type           => '$self->{tc_hash}->{aggr_create}->{checksum_type}',
		aggr_bps                     => '$self->{tc_hash}->{aggr_create}->{bps}',
        aggr_type                    => '$self->{tc_hash}->{aggr_create}->{aggr_type}',
        aggr_raid_type               => '$self->{tc_hash}->{aggr_create}->{raid_type}',
        aggr_mirror                  => '$self->{tc_hash}->{aggr_create}->{mirror}',
		test_spares_info             => '$self->{testdb}->{spare_info}',
		test_rc_object               => '$self->{testdb}->{rc_object}',
        test_aggr_info               => '$self->{testdb}->{aggr_info}',
		command_status               => '$self->{tc_hash}->{test_results}->{actual}->{command_status}',
        test_results                 => '$self->{tc_hash}->{test_results}',	
		};

    $self->{testdb}->{aggr_info} = undef;
    $self->{testdb}->{spare_info} = undef;
    $self->{testdb}->{rc_object} = undef;

    #create raid_config object and assign to $self
    $self->{testdb}->{rc_object} = $rc_object;
}

#-------------------------------------------------------------------------------
sub run_test {
    my $self = shift;
    my %args = validate(@_, {
                                tc_hash     => {type => HASHREF},
                                test        => {type => SCALAR, default=>"AzcsAggrMirror", optional=>1},
                            });

    $self->{tc_hash} = $args{tc_hash};
    $self->{test} = $args{test};
    my $tc_hash = $self->{tc_hash};

    eval {
        $self->step("AzcsAggrMirror: Setup testbed for test execution");
		## create aggregate as part of setup
        $self->setup();
		
		## create aggregate as part of setup
		#$self->step("AzcsAggrMirror: create a test aggregate");
        

        $self->step("AzcsAggrMirror: add raid group for mixed checksum if required");
        #my $source_disk = $self->get_source_disk();
		if (defined ($tc_hash->{mcs_aggr_info})) {
			## TODO :: add code to handle MCS
		}				
			
		$self->step("AzcsAggrMirror: perform aggr mirror");
        $self->aggr_mirror();				

		$Log->comment("AzcsAggrMirror : save test results");
        $self->save_test_results();

        $self->step("AzcsAggrMirror: Validate test results");
        $self->validate_test_results();

        $self->step("AzcsAggrMirror: Cleanup testbed setup");
        #$self->cleanup();
    };

    if($@) {
        $self->{error} = $@;
        die("FAIL: $self->{error}");
    }
}

#-------------------------------------------------------------------------------------------------
sub setup {
    my $self = shift;
	my $tc_hash = $self->{tc_hash};
	 my $api_set_obj = $self->{api_set_obj};
    my $setup = $self->{tc_hash}->{setup};    
	my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
	
	## check license	
	$Log->comment("AzcsAggrMirror : setup: checking for syncmirror license");
	if ( $tc_hash->{disk_setup_info}->{syncmirror_check} eq 'true' ){
		 if (RaidSqa_check_license( api_set_obj     => $api_set_obj,
                                     service  => "syncmirror_local")) {
			$Log->comment("AzcsAggrMirror : setup; sync-mirror service is available");
		} else {
			if ( $tc_hash->{disk_setup_info}->{syncmirror_install} eq 'true' ) {
				$Log->comment("AzcsAggrMirror : setup: trying to install syncmirror license");
				RaidSqa_license_add_by_service(api_set_obj => $api_set_obj, service => 'syncmirror_local');
			} else {
				die("AzcsAggrMirror : setup: sync-mirror not licensed : trying to install ");
			}
		}	              
	}
	
	## check disk status 
	my @spare_info = keys(%{$tc_hash->{disk_setup_info}->{spare_disk_info}});
	logcomment("AzcsAggrMirror : setup : spares_info : @spare_info");	
	
	foreach my $spare_info (@spare_info) {
		my $spare_info_ref = $tc_hash->{disk_setup_info}->{spare_disk_info}->{$spare_info};
		logcomment("AzcsAggrMirror : setup : hash : %hashval");	
		$Log->comment(" AzcsAggrMirror : setup : hash :".Dumper($tc_hash->{disk_setup_info}->{spare_disk_info}->{$spare_info}));
		$self->setup_spares(spare_info_hash => $spare_info_ref);
	} 
	
	## create aggregate
	$Log->comment("AzcsAggrMirror : setup: creating aggregate $tc_hash->{aggr_create}->{filer_vol} ");
	$self->aggr_create();
	
	
	## check aggr checksum type
	if ( RaidSqa_get_aggr_cksum_type(api_set_obj => $api_set_obj, filer_vol => $filer_vol)  eq ($tc_hash->{aggr_create}->{checksum_type} )) {
		$Log->comment("AzcsAggrMirror : setup: checksum type of aggregate is as required");		
	} else {
		die(" AzcsAggrMirror : setup: checksum type is not as required");
	}
	
	
	## check for aggr add spares
	if (defined($tc_hash->{mcs_aggr_info}) ){		
		$Log->comment("AzcsAggrMirror : setup : Performing aggr add for mixed checksum aggregates");
		$self->create_mcs_aggr();	
	}	
	
	## TODO::
	## check for aggr mirror spares
	## setup spares as required
	
}



#-------------------------------------------------------------------------------
sub create_mcs_aggr {
	my $self = shift;
	my $tc_hash = $self->{tc_hash};
	my $api_set_obj = $self->{api_set_obj};
	
    my $aggr_add_result = 0;
	my $aggr_add_command = '';
	if ($self->is_cmode()) {
		## insert c-mode command
	} else {
		$aggr_add_command = ($tc_hash->{aggr_create}->{aggr} == 1) ? "aggr add" : "vol add";
	}	
	my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
	
    
	die ("$self->{test} :: cannot specify both disk list and checksum type both") 
		if ( (defined ($tc_hash->{aggr_add_info}->{checksum_type})) && (defined ($tc_hash->{aggr_add_info}->{disk_list})));		
	$tc_hash->{aggr_add_info}->{disk_count} =  $tc_hash->{aggr_create}->{disk_count} if (!defined($tc_hash->{aggr_add_info}->{disk_count}) );
	if (defined($tc_hash->{aggr_add_info}->{checksum_type})){
		my $response;
		my $output;
		my $command;
		
		$Log->comment(" $self->{test}: create_mcs_aggr : using checksum type to create MCS aggregate");
		my $disk_count = $tc_hash->{aggr_add_info}->{disk_count};
		my $checksum_type = $tc_hash->{aggr_add_info}->{checksum_type};		
		$aggr_add_command = "$aggr_add_command" . "  $filer_vol" . " -c $checksum_type" . " $disk_count";		
		## Execute the aggr add command
		if($self->is_cmode()) {
			# try {
				# $api_set_obj->set_privilege("privilege-level" => 'diag');
				# $response = $api_set_obj->storage_aggregate_add_disks(%$aggr_add_hash);
				# $output   = $response->get_raw_output();				
			# } catch NACL::APISet::Exceptions::ResponseException with {
				# my $exception_object = shift;
				# $response = $exception_object->get_response_object();
				# $output   = $response->get_raw_output();
				# $command  = $response->get_command();
				# $Log->comment("$self->{test}: Aggr add failed with output");
			# }
		} else {    
			## 7mode command
			$output = RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd => $aggr_add_command);	
            $aggr_add_result =  1 if ($output =~ /has completed/i);          
		}		
	} elsif (defined($tc_hash->{aggr_add_info}->{disk_list})) {
			my @disk_list = ();
		if (defined($tc_hash->{aggr_add_info}->{pool}) ) {
			@disk_list = RaidSqa_get_disk_list_by_pool(api_set_obj => $api_set_obj,  pool_name => $tc_hash->{aggr_add_info}->{pool}, spare_only => 'yes');
		} else {
			@disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj, spare_only => 'yes');
		}
		die (" AzcsAggrMirror : create_mcs_aggr: insufficient spares for aggr add") if (scalar @disk_list < ($tc_hash->{aggr_add_info}->{disk_count}) );
		@disk_list = $self->get_spares_by_filter(storage_type => $tc_hash->{aggr_add_info}->{disk_type} ,
												checksum_type => $tc_hash->{aggr_add_info}->{checksum_type} ,
												bps => $tc_hash->{aggr_add_info}->{disk_bps}, 
												rpm => $tc_hash->{aggr_add_info}->{disk_rpm},
												disk_list => \@disk_list,
												refresh => 1);		
		#die (" AzcsAggrMirror : setup: insufficient spares for aggr add as per required params") if (scalar @disk_list < ($tc_hash->{aggr_add_info}->{disk_count}) );
		@disk_list = splice(@disk_list, 0, $tc_hash->{aggr_add_info}->{disk_count});
		die (" AzcsAggrMirror : create_mcs_aggr: insufficient spares for aggr add as per required params") if (scalar @disk_list < ($tc_hash->{aggr_add_info}->{disk_count}) );
		my $options = "-f"  if ($tc_hash->{aggr_add_info}->{force_add});
		my ($output, $result) = RaidSqa_aggr_add(api_set_obj => $api_set_obj, filer_vol => $filer_vol, group_name => $tc_hash->{aggr_add_info}->{group_name}, disk_list => \@disk_list, options => $options);
		$Log->comment(" AzcsAggrMirror : create_mcs_aggr : aggr add output : $output");
		if ($result) {
			$Log->comment(" AzcsAggrMirror : create_mcs_aggr : aggr add successful");
            $aggr_add_result = 1;
		} else {
			die(" AzcsAggrMirror : create_mcs_aggr : aggr add failure");
		}
	} else {
		die(" AzcsAggrMirror : create_mcs_aggr : failure in aggr add");		
	}
    
    ## waiting for aggr add to complete 
    if ($aggr_add_result) {
        $Log->comment(" AzcsAggrMirror : create_mcs_aggr : updating raid_group info");
        #my @group_list = @{$tc_hash->{group_list}};
        my @new_group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
        $tc_hash->{group_list} = [@new_group_list];

        #Update disk rgdn for all disks in each raid group
        foreach my $group (@new_group_list) {
            my @disk_list = RaidSqa_get_disk_list(api_set_obj=>$api_set_obj, group_name => $group);
            foreach my $disk (@disk_list) {
                my $rgdn = RaidSqa_get_disk_rgdn(api_set_obj=>$api_set_obj, disk_name=>$disk);
                $tc_hash->{rgdn_info}->{$group}->{$rgdn} = $disk;
                $Log->comment("$self->{test}: group $group, disk $disk, rgdn $rgdn");
            }
        }
    }
    
}

#-------------------------------------------------------------------------------
sub aggr_mirror {	
	my $self = shift;
	my $tc_hash = $self->{tc_hash};
	 my $api_set_obj = $self->{api_set_obj};
	
	## read all info
	my @options = ();
	if (defined ($tc_hash->{aggr_mirror_info}->{cmd_options} ) ){
		my $cmd_options = $tc_hash->{aggr_mirror_info}->{cmd_options};
		my @options = split(/,/,$cmd_options) ;
	}
	
	
	
	my $aggr_mirror_command = '';
	## create aggr_mirror command
	if ($tc_hash->{aggr_create}->{aggr} ) {
		$aggr_mirror_command = "aggr mirror ";
	} else {
		$aggr_mirror_command = "vol mirror ";
	}
	my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
	$aggr_mirror_command .= " $filer_vol";
	
	$aggr_mirror_command .= " -f" if (grep /^\-f$/, @options);
	$aggr_mirror_command .= " -d" if (grep /^\-d$/, @options);
	
	if ( (grep /^\-d$/, @options) && (defined ($tc_hash->{aggr_mirror_info}->{disk_list}) ) ) {
		my @pool_disk_list = RaidSqa_get_disk_list_by_pool(api_set_obj => $api_set_obj, pool_name => $tc_hash->{aggr_mirror_info}->{disk_info}->{pool}, spare_only => 'yes');
		my @spare_list =  $self->get_spares_by_filter (
										bps           => $tc_hash->{aggr_mirror_info}->{disk_info}->{bps},
										rpm			  => $tc_hash->{aggr_mirror_info}->{disk_info}->{rpm},
										storage_type  => $tc_hash->{aggr_mirror_info}->{disk_info}->{type},										
										disk_list     => \@pool_disk_list,
									);
		die (" AzcsAggrMirror : aggr_mirror : insufficient spares for aggr mirror ") if (scalar @spare_list < $tc_hash->{aggr_create}->{disk_count} );
		@spare_list = splice(@spare_list,0,$tc_hash->{aggr_create}->{disk_count});
		my $disk_list = join(" ",@spare_list);
		$aggr_mirror_command .= " $disk_list";
	} 
	
	## execute command
	$Log->comment("AzcsAggrMirror : aggr_mirror : executing command: $aggr_mirror_command");	
   if($self->is_cmode()) {
		$Log->warn("AzcsAggrMirror : aggr_mirror : aggr mirror is not supported for C-mode currently");	
    } else {    #TODO:: Remove this and use nacl object for both 7mode and Cmode
        #$command = $self->get('aggr_add_command');
        my $output = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>$aggr_mirror_command);
		$tc_hash->{test_results}->{actual}->{output_string} = $output;
        $tc_hash->{test_results}->{actual}->{command_status} = 1 if ($output =~ /has completed/ig);
        RaidSqa_wait_for_aggr_disks_zeroing_completion(api_set_obj => $api_set_obj, filer_vol => $filer_vol) if ($output =~ /has completed/ig);
        RaidSqa_wait_for_plex_resync_completion(api_set_obj => $api_set_obj, filer_vol => $filer_vol) if ($output =~ /has completed/ig);
    }
	$Log->comment(" $self->{test}: aggr_mirror : object info".Dumper($self->{tc_hash}));	
}


#-------------------------------------------------------------------------------
sub setup_spares {
	my $self = shift;
    my %args = validate(@_, {
								spare_info_hash     => {type => HASHREF},								
                            });
    
	my $tc_hash = $self->{tc_hash};	
	my $api_set_obj = $self->{api_set_obj};
	my $spare_info_ref = $args{spare_info_hash};	
	my %spare_info_hash = %$spare_info_ref;
	
	$Log->comment(" $self->{test}: setup : object info".Dumper($self->{tc_hash}));
	
	my $pool = $spare_info_hash{pool};	
	$pool = "pool0" if (!defined($pool) ) ;
	
	my @pool_disk_list = RaidSqa_get_disk_list_by_pool(api_set_obj => $api_set_obj, pool_name => $pool, spare_only => 'yes');	
	my @reqd_spare_list =  $self->get_spares_by_filter (
										bps           => $spare_info_hash{bps},										
										storage_type  => $spare_info_hash{disk_type},
										checksum_type => $spare_info_hash{checksum_type}, 
										disk_list     => \@pool_disk_list,										
									);	
	if (defined($spare_info_hash{count})) {
		if (scalar @reqd_spare_list >= $spare_info_hash{count} ) {
			$Log->comment("AzcsAggrMirror : Setup : setup_spares : spares are present as required");
			$Log->comment("AzcsAggrMirror : Setup : setup_spares list : @reqd_spare_list");
			my @spare_info_disk_list = splice(@reqd_spare_list, 0, $spare_info_hash{count});
			#return @spare_info_disk_list;
		} else {
				## check in other pool if spares are available
				my $other_pool = '';
				if ($pool eq "pool0") {
					$other_pool = "pool1";
				} else {
					$other_pool = "pool0";
				}
				## try to setup spares accordingly
				@pool_disk_list = RaidSqa_get_disk_list_by_pool(api_set_obj => $api_set_obj, pool_name => $other_pool, spare_only => 'yes');
				my @other_pool_spare_list =  $self->get_spares_by_filter (
										bps           => $spare_info_hash{bps},										
										storage_type  => $spare_info_hash{disk_type},
										checksum_type => $spare_info_hash{checksum_type}, 
										disk_list     => \@pool_disk_list,										
										);
				if (scalar @other_pool_spare_list >= $spare_info_hash{count} ) {
					$Log->comment("AzcsAggrMirror : Setup : setup_spares : assigning sparesfrom one pool to other");
					## find number of disks to assign 
					## disk remove ownership for them
					## assign to the first pool
					## make sure there are required number of disks in this pool as well
					#my @spare_info_disk_list = splice(@reqd_spare_list, 0, $spare_info_hash{count});
					#return @spare_info_disk_list;
				}
				
		}		
	}
}

#-------------------------------------------------------------------------------
sub cleanup {
    my $self = shift;
    my $cleanup = $self->{tc_hash}->{cleanup};

    if($cleanup->{aggr_remove} == 1) {
        $self->step("AzcsAggrMirror : cleanup test aggr");
        $self->aggr_destroy();
    }
	
    if($cleanup->{reset_options} == 1) {
        $self->step("AzcsAggrMirror : cleanup, reset all options used in this test to their original values");
        $self->reset_options();
    }

    if($cleanup->{zero_spares} == 1) {
        $self->step("AzcsAggrMirror : Cleanup, start spare disks zeoring");
        RaidSqa_disk_zero_spares(api_set_obj=>$self->{api_set_obj});
    }
}


#--------------------------------------------------------------------
sub save_test_results {
    my $self = shift;
	my $tc_hash = $self->{tc_hash};	
	$self->update_rc_object();
    my $api_set_obj = $self->{api_set_obj};
    my $filer_vol = $self->get('filer_vol');		
	my $aggr_obj = $self->get_rc_object(filer_vol => $filer_vol);
	my $command_status_output = 1 if($aggr_obj->parameter('mirror_state_name') eq "mirrored") ;
	my $test_results  = $tc_hash->{test_results};	
	$test_results->{actual}->{command_status} = $command_status_output;
    if($command_status_output) {
		$test_results->{actual}->{checksum_type}  = RaidSqa_get_aggr_cksum_type(api_set_obj    => $api_set_obj,filer_vol => $filer_vol);		
    } else {
       $Log->comment("$self->{test}: aggr mirror failed.");
	   $test_results->{actual}->{checksum_type}  = RaidSqa_get_aggr_cksum_type(api_set_obj    => $api_set_obj,filer_vol => $filer_vol);
    }
    $self->debug("$self->{test}: actual test results: DUMPER".Dumper($command_status_output));
}

#--------------------------------------------------------------------
sub validate_test_results {
    my $self = shift;
	my $tc_hash = $self->{tc_hash};
	my $filer_vol = $self->get('filer_vol');
    my $api_set_obj = $self->{api_set_obj};   
	my $test_results =$tc_hash->{test_results};
	
	if($test_results->{expected}->{command_success}) {
		if($test_results->{actual}->{command_status}) {
			$Log->comment("$self->{test}: validate_test_results: aggr successfully mirrored.");
			if($test_results->{actual}->{checksum_type} eq $test_results->{expected}->{checksum_type}) {
				$Log->comment("$self->{test}: validate_test_results: aggr mirrored, with checksum : $test_results->{actual}->{checksum_type}.");
			} else {
				die("$self->{test}: validate_test_results:  Actual and Expected parameters not matching".Dumper($self->{tc_hash}));
			}
		} else {
			die("$self->{test}: validate_test_results:  aggr mirror not happened.".Dumper($self->{tc_hash}));
		}
	}
	else
	{
		if($test_results->{expected}->{command_success} eq $test_results->{actual}->{command_status}) {
			$Log->comment("$self->{test}: validate_test_results:aggr mirror not happened as expected");
			if($test_results->{actual}->{checksum_type} eq $test_results->{expected}->{checksum_type}) {
				$Log->comment("$self->{test}: validate_test_results: aggr mirror not happened as expected, with checksum : $test_results->{actual}->{checksum_type}.");
			} else {
				die("$self->{test}: validate_test_results:  aggr mirror not happened as expected, Actual and Expected parameters not matching".Dumper($self->{tc_hash}));
			}
		}
	}
    
    if ($test_results->{expected}->{check_output_string}) {
        if ($test_results->{expected}->{output_string} eq $test_results->{actual}->{output_string} ){
            $Log->comment("$self->{test}: validate_test_results: aggr mirror command output matches");
        } else {
            die("$self->{test}: validate_test_results: aggr mirror command output does not matches 
                    expected:$test_results->{expected}->{output_string} actual: $test_results->{actual}->{output_string}");
        }
        
    }
}

#==================================================================================
package AzcsDiskReplace;

use vars qw(@ISA);
use strict;
use warnings;
use Tharn qw($Log);
use RaidSqav3 qw(AUTOLOAD);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidLW;
use RaidInfo;
use Data::Dumper;
use Utils;
use Storage::SASUtils qw(get_disks);

BEGIN {
    @ISA = qw(RaidAzcsUtils);
}

#-------------------------------------------------------------------------------
sub new {
    my $self = shift if(scalar(@_) % 2);
    my %self = validate(@_, {
                                api_set_obj => {type => OBJECT, isa=> "NACL::APISet"},
                                debug       => {type => SCALAR, default=>0, optional=>1},
                            }
                        );
    $self = \%self;

    my $api_set_obj = $self->{api_set_obj};
    my $conntype = ($api_set_obj->{mode} eq "CMode") ? "#ssh" : "#rsh";

    if(defined($api_set_obj->{hostobj})) {
        $self->{conn_obj} = make_conn($api_set_obj->{hostobj}, $conntype);
    } elsif (defined $api_set_obj->{connection}) {
        $self->{conn_obj} = $api_set_obj->{connection};
    }

    if(!defined($self->{conn_obj})) {
        die("AzcsDiskReplace::new: Could not create connection object");
    }

    #Define io_hash
    $self->{io_hash}->{wafl} = \&waflio;    
    $self->{io_hash}->{scrub} = \&scrubio;
	
    bless $self, 'AzcsDiskReplace';
	$self->init();
    return $self;
}

sub init {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $conn_obj = $self->{conn_obj};
    my $rc_object = new RaidInfo($conn_obj);

    #init test data base
    $self->{testdb}->{key_mapping} = {
        filer_vol                    => '$self->{tc_hash}->{aggr_create}->{filer_vol}',
        aggr_checksum_type           => '$self->{tc_hash}->{aggr_create}->{checksum_type}',
		aggr_bps                     => '$self->{tc_hash}->{aggr_create}->{bps}',
        aggr_type                    => '$self->{tc_hash}->{aggr_create}->{aggr_type}',
        aggr_raid_type               => '$self->{tc_hash}->{aggr_create}->{raid_type}',
        aggr_mirror                  => '$self->{tc_hash}->{aggr_create}->{mirror}',
		source_disk          	     => '$self->{tc_hash}->{get_source_disk}',
		target_disk          	     => '$self->{tc_hash}->{get_target_disk}',
        disk_replace_status          => '$self->{tc_hash}->{disk_replace_result}->{actual}->{disk_replace_status}',
        disk_replace_result          => '$self->{tc_hash}->{disk_replace_result}',
        execute_disk_replace_cmd     => '$self->{tc_hash}->{execute_disk_replace_cmd}->{command}',
        test_rc_object               => '$self->{testdb}->{rc_object}',
        test_aggr_info               => '$self->{testdb}->{aggr_info}',
		ingore_other_spares          => '$self->{tc_hash}->{aggr_add}->{ingore_other_spares}',
        test_spares_info             => '$self->{testdb}->{spare_info}',
       
		};

    $self->{testdb}->{aggr_info} = undef;
    $self->{testdb}->{spare_info} = undef;
    $self->{testdb}->{rc_object} = undef;

    #create raid_config object and assign to $self
    $self->{testdb}->{rc_object} = $rc_object;
}

#-------------------------------------------------------------------------------
sub run_test {
    my $self = shift;
    my %args = validate(@_, {
                                tc_hash     => {type => HASHREF},
                                test        => {type => SCALAR, default=>"AzcsRbnDbn", optional=>1},
                            });

    $self->{tc_hash} = $args{tc_hash};
    $self->{test} = $args{test};
    my $tc_hash = $self->{tc_hash};

    eval {
        $self->step("Disk_Replace: Setup testbed for test execution");
        $self->setup();
		
		$self->step("Disk_Replace: create a test aggregate");
        $self->aggr_create();
		
		if (defined($tc_hash->{mcs_aggr_create_info})){
			## call MCS creation function
			&AzcsAggrMirror::create_mcs_aggr();
		}


        $self->step("Disk_Replace: Get Source Disk For Replace");
        my $source_disk = $self->get_source_disk();
		
		$tc_hash->{disk_to_replace} = $source_disk;
		$self->step("Disk_Replace: Get Target Disk For Replace");
        my $target_disk = $self->get_target_disk();
		$tc_hash->{replacement_disk} = $target_disk;
        
		
		$self->step("Disk_Replace: start io on testvolume");
        $self->scrubio();
		
		$Log->comment("Disk Replace : tc_hash :".Dumper($self->{tc_hash}));
        $self->step("Disk_Replace: Execute Disk_Replace command");
        $self->execute_disk_replace_cmd();
		
		
        sleep(2);
        RaidSqa_filer_cmd(api_set_obj => $self->{api_set_obj}, cmd=>'aggr status -r');

        $self->step("Disk_Replace: Save test results");
        $self->save_test_results();

        $self->step("Disk_Replace: Validate test results");
        $self->validate_test_results();

        $self->step("Disk_Replace: Cleanup testbed setup");
        $self->cleanup();
    };

    if($@) {
        $self->{error} = $@;
        die("FAIL: $self->{error}");
    }
}

#-------------------------------------------------------------------------------------------------
sub setup {
    my $self = shift;
	my $tc_hash = $self->{tc_hash};
    my $setup = $self->{tc_hash}->{setup};
	

    if(exists($setup->{set_options}) && $setup->{set_options} == 1) {
        $self->step("Disk_Replace: Setup options for this test case");
        $self->set_options();
    }
	
}

#-------------------------------------------------------------------------------
sub cleanup {
    my $self = shift;
    my $cleanup = $self->{tc_hash}->{cleanup};

    if($cleanup->{aggr_remove} == 1) {
        $self->step("Disk_Replace: cleanup test aggr");
        $self->aggr_destroy();
    }
	
    if($cleanup->{reset_options} == 1) {
        $self->step("Disk_Replace: cleanup, reset all options used in this test to their original values");
        $self->reset_options();
    }

    if($cleanup->{zero_spares} == 1) {
        $self->step("Disk_Replace: Cleanup, start spare disks zeoring");
        RaidSqa_disk_zero_spares(api_set_obj=>$self->{api_set_obj});
    }
}

#--------------------------------------------------------------------
sub aggr_create {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
    my $tc_hash = $self->{tc_hash};
    my $aggr_create = $tc_hash->{aggr_create};
    my $new_aggr_required = (exists $aggr_create->{new_aggr_required}) ? $aggr_create->{new_aggr_required} : 1;
    my $filer_vol     = $tc_hash->{aggr_create}->{filer_vol};

    die("$self->{test}: No filer_vol defined") if(!defined($filer_vol));
    #define test id to append to volume name if volume already exists
    $Log->comment("$self->{test}: creating testvolume $filer_vol");
    my $aggr_exists = $self->aggr_exists();
    if($aggr_exists && !$new_aggr_required) {
        $aggr_exists = $self->aggr_ok();
    }

    $self->aggr_destroy() if($aggr_exists and $new_aggr_required);
   # $self->check_spares() if($new_aggr_required);

    #---------------------------------------------------------------
    $Log->comment("Create test volume");
    #---------------------------------------------------------------
	$Log->comment(" $self->{test}: AGGR_UTILS : DISK_REPLACE : object info".Dumper($self->{tc_hash}));
	## get spare disk list based on bps,rpm, checksum type, : get_spares_by_filter
	my $bps           = $tc_hash->{aggr_create}->{bps};
	my $storage_type  = $tc_hash->{aggr_create}->{storage_type};
	my $disk_count    = $tc_hash->{aggr_create}->{disk_count};
	my $checksum_type = $tc_hash->{aggr_create}->{checksum_type};
		#	logcomment("Checking for bps: $bps, checksum_type: $checksum_type, storage_type; $storage_type ,  disk_count : $disk_count");
							
			my @group_list = $self->get_spares_by_filter (
						        storage_type  => $storage_type,
								bps           => $bps,
								refresh       => 1,
                            );
			$Log->comment("$self->{test}: the disk list is : @group_list");
			if(scalar @group_list < $disk_count)
			{
				die("$self->{test}: Insufficient spares to create $filer_vol");
			}
			
			my @disk_list = splice(@group_list, 0, $disk_count);
			$Log->comment("$self->{test}: the disk list is :  @disk_list");
			my %aggr_create_args = (api_set_obj   => $api_set_obj,
                            filer_vol     => $filer_vol,
                            raid_type     => $aggr_create->{raid_type},
                            raid_size     => $aggr_create->{raid_size},
                            aggr_type     => $aggr_create->{aggr},
							storage_type  => $aggr_create->{storage_type},
                            disk_list     => \@disk_list,
                            aggr_bits     => $aggr_create->{aggr_bits},
                            mirror        => $aggr_create->{mirror},
                            vserver       => $aggr_create->{vserver},
                          );
	
	
        #$self->aggr_create(%aggr_create_args);
		if (!RaidSqa_aggr_create(%aggr_create_args)) {
            die("$self->{test}: aggr create failed to create $filer_vol");
        }
    $Log->comment("$self->{test}: Volume $filer_vol created successfully");
    @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
    $tc_hash->{group_list} = [@group_list];

    #Update disk rgdn for all disks in each raid group
    foreach my $group (@group_list) {
        my @disk_list = RaidSqa_get_disk_list(api_set_obj=>$api_set_obj, group_name => $group);
        foreach my $disk (@disk_list) {
            my $rgdn = RaidSqa_get_disk_rgdn(api_set_obj=>$api_set_obj, disk_name=>$disk);
            $tc_hash->{rgdn_info}->{$group}->{$rgdn} = $disk;
            $Log->comment("$self->{test}: group $group, disk $disk, rgdn $rgdn");
        }
    }
   
} ## end sub aggr_create

#--------------------------------------------------------------------
# AzcsDiskReplace::get_source_disk for replace
#--------------------------------------------------------------------
sub get_source_disk {

    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
	my $tc_hash = $self->{tc_hash};
	my $filer_vol = $tc_hash->{source_disk}->{filer_vol};

    if(!defined($api_set_obj)) {
       die("$self->{test}: Api set object is not defined".Dumper($self->{tc_hash}));
    }

    $Log->comment("$self->{test}: get_source_disk");
    $self->update_spare_info();

	#=============================================================================
	#get_source_disk for disk replace
                
	my @source_disk_list = RaidSqa_get_disk_list(api_set_obj=>$api_set_obj, filer_vol => $filer_vol);
	#==============================================================================================
	# check the disk is correct to replace
	#===============================================================================================			
	if(!scalar(@source_disk_list)) {
	   die("$self->{test}: get_source_disk: There are not enough spares for disk replace operation");
	} else 	{
		$Log->comment("$self->{test}: get_source_disk: volume disk list for disk_replace: @source_disk_list");
	}
	$Log->comment("$self->{test}: get_source_disk: validating disk parameters against required values");
	
	
	my $disk_rc_object = $self->get_rc_object(disk_name => $source_disk_list[0]);
	# foreach my $param_key (keys (%{$tc_hash->{source_disk}})){
		# next if ($param_key eq 'filer_vol');
		# if (defined ($tc_hash->{source_disk}->{$param_key} ) ){
			# my $param_value = $disk_rc_object->parameter($param_key);
			# if ( $param_value eq $tc_hash->{source_disk}->{$param_key}) {
				# $Log->comment("$self->{test}: get_source_disk: value for param $param_key [$param_value] matching for disk $source_disk_list[0]");
			# } else {
				# warn(" $self->{test}:  get_source_disk: value mismatch in required and actual param values for disk $source_disk_list[0] ");				
			# }
		# }
	# }	
	
	$Log->comment("$self->{test}: get_source_disk: Found source disk : $source_disk_list[0]");
	
	## validate disk params as per tc_hash
	return $source_disk_list[0];
}	
	
#--------------------------------------------------------------------
# AzcsDiskReplace::get_target_disk for replace
#--------------------------------------------------------------------
sub get_target_disk {
    my $self = shift;
    my $api_set_obj = $self->{api_set_obj};
	my $tc_hash = $self->{tc_hash};

    if(!defined($api_set_obj)) {
       die("$self->{test}: Api set object is not defined".Dumper($self->{tc_hash}));
    }

    $Log->comment("$self->{test}: get_target_disk");
    $self->update_spare_info();
	
	#=============================================================================
	#get_target_disk for disk replace
  	my $checksum_type = $self->{tc_hash}->{target_disk}->{checksum_type};
	my $storage_type = $self->{tc_hash}->{target_disk}->{storage_type};
	my $bps = $self->{tc_hash}->{target_disk}->{bps};
	## use get_spares_by_filter
	my @target_disk_list = $self->get_spares_by_filter(
								checksum_type => $checksum_type,
						        storage_type  => $storage_type,
								bps           => $bps,
								refresh       => 1,
                            );
	my $replacement_disk = '';	
	my $value_mismatch = 0;
	#==============================================================================================
		# check the disk is correct to replace
	#===============================================================================================	
    if(scalar @target_disk_list < 0) {
		die("$self->{test}: get_target_disk: There are not enough spares for disk replace operation");
    } else {
		$Log->comment("$self->{test}: get_target_disk: spares that will be used for disk_replace: @target_disk_list");
		foreach my $target_disk (@target_disk_list) {
			my $disk_rc_object = $self->get_rc_object(disk_name => $target_disk);
			
			# foreach my $param_key (keys %{$tc_hash->{target_disk}}){				
				# if (defined ($tc_hash->{target_disk}->{$param_key} ) ){
					# my $param_value = $disk_rc_object->parameter($param_key);
					# if ( $param_value eq $tc_hash->{source_disk}->{$param_key}) {
						# $Log->comment("$self->{test}: get_target_disk: value for param $param_key [$param_value] matching for disk $target_disk");
					# } else {
						# warn(" $self->{test}:  get_target_disk: value mismatch in required and actual param values for disk $target_disk ");	
						# $value_mismatch++;
					# }
				# }
			# }
			if ($value_mismatch == 0) {
				$replacement_disk = $target_disk;
				last;
			}
		}
    }
	$Log->comment("$self->{test}: get_target_disk: Found replacement disk : $replacement_disk");
	if(!$replacement_disk)
	{
	die("$self->{test}:could not found replacement disk for disk replace");
	}
	return $replacement_disk;		
}

#--------------------------------------------------------------------
		# Run some io on the volume
#--------------------------------------------------------------------
sub scrubio {
    my $self = shift;
    my $tc_hash = $self->{tc_hash};
    my $api_set_obj = $self->{api_set_obj};
    my $filer_vol = $tc_hash->{aggr_create}->{filer_vol};
   

    my $result = RaidSqa_aggr_scrub_start(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol);
    if($result) {
        $Log->comment("$self->{test}: Scrub start on $filer_vol: ok");
    } else {
        $Log->comment("$self->{test}: Scrub start on raid filer_vol: failed");
    }
}

#--------------------------------------------------------------------
#replace a disk
#--------------------------------------------------------------------
sub execute_disk_replace_cmd {
			my $self = shift;
			my $api_set_obj = $self->{api_set_obj};
			my $tc_hash = $self->{tc_hash};
			
			my ($result,$disk_name,$target_disk_name);
			$disk_name = $tc_hash->{disk_to_replace};
			$target_disk_name = $tc_hash->{replacement_disk};
			
			$result = RaidSqa_disk_replace_start(api_set_obj    =>$api_set_obj,
                                         disk_name      => $disk_name,
                                         replacing_disk => $target_disk_name,);
    sleep(5);
	 RaidSqa_wait_for_disk_sdc_completion(api_set_obj=>$api_set_obj, disk_name=>$disk_name);
	# if($result)
	# {
		# $Log->comment("$self->{test}:disk replaced successfully");
	# }
	# else
	# {
	# die("$self->{test}:Disk replace failed".Dumper($self));
	# }
	
	$self->set('disk_replace_status'=>$result);
	sleep 120;
	
}


#--------------------------------------------------------------------
sub save_test_results {
    my $self = shift;
	my $tc_hash = $self->{tc_hash};
	
	$self->update_rc_object();
    my $api_set_obj = $self->{api_set_obj};
    my $filer_vol = $self->get('filer_vol');
    my $disk_replace_status = $self->get('disk_replace_status');
	my $disk_replace_result =$tc_hash->{disk_replace_result};
	
	#$disk_replace_result->{actual}->{disk_replace_status} = $disk_replace_status;	 
    if($disk_replace_status) {
		my $replace_disk = $tc_hash->{replacement_disk};
		my $disk_obj = $self->get_rc_object(disk_name => $replace_disk);
		## find bps, storage_type and checksum_type of disk and update actual hash o/p
		$disk_replace_result->{actual}->{bps} = $disk_obj->parameter("bps");
		$disk_replace_result->{actual}->{storage_type} = $disk_obj->parameter("storage_type");
		$disk_replace_result->{actual}->{checksum_type} = $disk_obj->parameter("checksum_type");
    } else {
       $Log->comment("$self->{test}:disk replace is not success");
    }
	
    #$self->debug("$self->{test}: actual test results: DUMPER".Dumper($disk_replace_status));

}
#--------------------------------------------------------------------
sub validate_test_results {
    my $self = shift;
	my $tc_hash = $self->{tc_hash};
    my $api_set_obj = $self->{api_set_obj};   

	my $disk_replace_result =$tc_hash->{disk_replace_result};
       
	if($disk_replace_result->{expected}->{disk_replace_status}) {
		if ($disk_replace_result->{expected}->{check_spares}) {
				my @spares = RaidSqa_get_disk_list(api_set_obj    =>$api_set_obj,
													spare_only => "yes",);
				$Log->comment("$self->{test}: validate_test_results: spares are @spares");
				my $spares; 
				my $disk_to_replace =  $tc_hash->{disk_to_replace};
				my $back_to_spare =0;
				foreach $spares(@spares) {
					if($spares eq $disk_to_replace)
					{
						$back_to_spare =1;
						$Log->comment("$self->{test}: validate_test_results: disk successfully returned to spares");
						last;
					}
			}
			if(!$back_to_spare)
			{
				die("$self->{test}: disk is not successfully returned to spares");
			}
		} else {
			if(!($disk_replace_result->{actual}->{checksum_type} eq $disk_replace_result->{expected}->{checksum_type}) &&($disk_replace_result->{actual}->{storage_type} eq $disk_replace_result->{expected}->{storage_type})
				&& ($disk_replace_result->{actual}->{bps} eq $disk_replace_result->{expected}->{bps}))
			{
			  die("$self->{test}: Actual and Expected parameters not matching");
			} else {
			 $Log->comment("$self->{test}:disk replace is not happened");
			}
		}  
	} else {
		if($disk_replace_result->{actual}->{disk_replace_status} eq $disk_replace_result->{expected}->{disk_replace_status})
				{
					$Log->comment("$self->{test}: validate_test_results:disk replace not happened as expected");
				}
	}
}

1;



