#$Id: //depot/prod/test/Rsierranevada/lib/RaidErrUtils.pm#4 $

## Please refer to following link for data structure definitions for ErrHandling tests
## http://wikid.netapp.com/w/RAID_SQA/NB/RAID_Error_Handling_Automation/Merging_Lib
## @change : naushad@netapp.com Burt 708025 : Added plex3 in plex validation code since When non-sync aggr is mirrored plex0 and plex3 are created whereas when mirrored aggr is created plex0 and plex1 is created

package RaidErrUtils;

use strict;
use warnings;
use vars qw(@ISA);

# Module Imports
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidSqav3 qw(AUTOLOAD);
use RaidLW;
use NACL::APISet;
use Tharn qw($Log);
use Data::Dumper;
use List::Util qw[min max];
use Storable qw(dclone);

BEGIN {
    @ISA = qw(commonUtils);
}
require Exporter;

# Constructor for RaidErrUtils
    sub new {
        my $self = undef;
        $self = shift if(scalar(@_) % 2);
        my (%opts) = validate(@_, {
            api_set_obj   => {type => OBJECT, isa => 'NACL::APISet'},
            tc_data       => {type => HASHREF},
            io_type       => {type => SCALAR|UNDEF, default => undef, optional => 1},
            filer_vol     => {type => SCALAR|UNDEF, default => undef, optional => 1},
            vserver       => {type => SCALAR,       default => "vs0", optional => 1},
            mirror        => {type => SCALAR|UNDEF, default => undef, optional => 1},
            aggr          => {type => SCALAR,       default => 1,     optional => 1},
            aggr_bits     => {type => SCALAR,       default => 64,    optional => 1},
            checksum_type => {type => SCALAR|UNDEF, default => undef, optional => 1},
            timeout       => {type => SCALAR,       default => 120,   optional => 1},
        });

        $self = {};
        $self->{api_set_obj} = $opts{api_set_obj};
        $self->{timeout} = $opts{timeout};
        $self->{tc_data} = $opts{tc_data};
        $self->{io_type} = $opts{io_type};
        $self->{mirror}  = $opts{mirror};

        #Override all predefined parameters by userdefined parameters
        foreach my $index (0..scalar(@{$self->{tc_data}->{aggregates}})-1) {
            if(defined($opts{vserver}))   { $self->{tc_data}->{aggregates}->[$index]->{vserver}   = $opts{vserver};  }
            if(defined($opts{aggr}))      { $self->{tc_data}->{aggregates}->[$index]->{aggr}      = $opts{aggr};     }
            if(defined($opts{aggr_bits})) { $self->{tc_data}->{aggregates}->[$index]->{aggr_bits} = $opts{aggr_bits};}
            if(defined($opts{mirror}))    { $self->{tc_data}->{aggregates}->[$index]->{mirror}    = $opts{mirror};   }

            if(defined($opts{checksum_type})) {
                $self->{tc_data}->{aggregates}->[$index]->{checksum_type} = $opts{checksum_type};
                foreach my $rgindex (0..scalar(@{$self->{tc_data}->{aggregates}->[$index]->{rgs}})-1) {
                    $self->{tc_data}->{aggregates}->[$index]->{rgs}->[$rgindex]->{checksum_type} = $opts{checksum_type};
                }
            }
        }

        $self->{aggr_info}              = {};
        $self->{file_info}              = {};
        $self->{spare_disks}            = {};
        $self->{is_start_from_scratch}  = 1;

        bless $self;
        $Data::Dumper::Indent = 1;
        return $self;
    }

    sub print_tc_headers {
        my $self = shift;
        my $tc_data = $self->get_tc_data();
        $Log->comment("*"x80);
        $Log->comment("Assertion: ".$tc_data->{assertion});
        $Log->comment("*"x80);
        $Log->comment("TC Datastructure: \n".Dumper($tc_data));
        $Log->comment("*"x80);
    }

    # sub _utils_freezer {
        # my $self = shift;

    # }
    # Algorithm for validation:
    # total_spare_requirements = []
    # Validate AggrCreate for every hash entry in aggregates
    #   plex_count = get_plex_count(filer_vol, error_info)
    #   Create new AggrCreate object with aggregates hash, io_type, aggr, aggr_bits, checksum_type, plex_count
    #       (CHANGE: aggr, aggr_bits, checksum_type is changed in constructor for RaidErrUtils)
    #       override the raidio with the io_type passed by user
    #   Call validate function to check
    #       override aggr, aggr_bits, checksum_type if they are passed as constructor
    #       presence of filer_vol, vserver, mirror, raidio, raid_type, raid_size, aggr, aggr_bits, checksum_type, hybrid_aggr, plex_offline, plex_offline_always,
    #           ignore_lw_validation, mirror_during_resyncio, rg, aggr_options
    #       decide its values based on default/optional requirement and the param passed with new()
    #       Repeat this validation for all member-fields mentioned above
    #       validate raid_size and assign numeric values for expected string values(min/max/default)
    #       At any point if default value is not provided or if wrong value is provided, then assign the default value.
    #
    #   total_spare_requirements = $aggr_create->get_spare_requirements(total_spare_requirements)
    # Validate SpareDisks
    #   SpareDisks->new(spare_setup_info)
    #   spare_disks->compare(total_spare_requirements) # incore comparison of two data structures
    #       $self->update() # update spare_setup_info for required additional spares than what is already mentioned in spare_setup_info.
    #   spare_disk->validate()  # check for spares on filer and fail test if spare requirements are not met
    #
    # Validate ErrorInfo
    #   Add extra function to validate error_info as validate_error_info()
    #

    sub validate_tc_data {
        my $self = shift;
        my %opts = validate(@_, {
            ignore_error_info => {type => SCALAR, default=>0, optional=>1},
            ignore_aggr_info  => {type => SCALAR, default=>0, optional=>1},
            ignore_spare_info => {type => SCALAR, default=>0, optional=>1},
        });

        my $tc_data           = $self->get_tc_data();
        my $api_set_obj       = $self->{api_set_obj};
        my $ignore_error_info = $opts{ignore_error_info};
        my $ignore_aggr_info  = $opts{ignore_aggr_info};
        my $ignore_spare_info = $opts{ignore_spare_info};
        my @valid_raidio      = ('waflio', 'scrubio', 'reconsio', 'resyncio', 'verifyio', 'reparityio');

        $self->print_tc_headers();
        $Log->comment("Starting validation of testcase data...");

        #===================================================================
        #validate test assertion
        $Log->comment("Starting validation of test assertion....");
        if(!defined($tc_data->{assertion})) {
            die("Validated test assertion: Error: Test assertion is not defined for test case");
        }
        $Log->comment("Validated test assertion: $tc_data->{assertion}");
        $Log->comment("Validated test assertion: OK");

        #===================================================================
        #validate ignore_validation
        $Log->comment("Starting validation of ignore_validation....");
        if(!defined($tc_data->{ignore_validation})) {
            $Log->comment("Validated test assertion: Info: ignore_validation is not defined. Changing to default - 0");
            $tc_data->{ignore_validation} = 0;
        }
        $Log->comment("Validated ignore_validation: $tc_data->{ignore_validation}");
        $Log->comment("Validated ignore_validation: OK");

        #===================================================================
        #validate start_recons_always
        $Log->comment("Starting validation of start_recons_always....");
        if(!defined($tc_data->{start_recons_always})) {
            $Log->comment("Validated test assertion: Info: start_recons_always is not defined. Changing to default - 0");
            $tc_data->{start_recons_always} = 0;
        }
        $Log->comment("Validated start_recons_always: $tc_data->{start_recons_always}");
        $Log->comment("Validated start_recons_always: OK");

        my $total_spare_requirements = [];
        my $mirror = 0;

        #Skip aggr_info validation if required
        if($ignore_aggr_info == 0) {
            die("validate_tc_data: Error: No aggregates present in tc_data") if(ref($tc_data->{aggregates}) !~ /ARRAY/i);
            foreach my $index (0..scalar(@{$tc_data->{aggregates}})-1) {
                my $filer_vol = $tc_data->{aggregates}->[$index]->{filer_vol};
                my $plex_count = $self->get_error_info_plex_count(filer_vol => $filer_vol, error_info => $tc_data->{error_info}->{$filer_vol});

                #=============================================================
                #Create aggr_create object and validate data structure
                my $aggr = AggrCreate->new(api_set_obj=>$self->{api_set_obj}, aggregate_data=>$tc_data->{aggregates}->[$index],
                                            io_type=>$self->{io_type}, plex_count=>$plex_count);

                $aggr->validate_aggr_data();
                $self->set_aggregate(filer_vol => $filer_vol, object => $aggr);
                if($aggr->is_mirror()) { $mirror++; }

                if($ignore_error_info == 0) {
                    #=============================================================
                    #validate error inject data
                    $self->validate_error_info(filer_vol => $filer_vol);
                    my $file_create_obj = $self->init_fileCreate(filer_vol=>$filer_vol);
                    $aggr->set_fileCreate(fileCreate => $file_create_obj);
                }

                #=============================================================
                #Update the spare requirements with spare requirements for $filer_vol
                $total_spare_requirements = $aggr->get_spare_requirements(total_spare_requirements => $total_spare_requirements);
            }
        }

        if($ignore_spare_info == 0) {
            #=============================================================
            #Validate spare disks data
            if($mirror) {$mirror = 1};
            if(!defined($tc_data->{setup_info}->{spare_setup})) {
                $tc_data->{setup_info}->{spare_setup} = {
                    ignore_removing_ownership => 1,
                    enable_optional_storage_type => 1,
                    mirror => $mirror,
                };
            }
            my $spare_disks = SpareDisks->new(api_set_obj => $self->{api_set_obj}, setup_info => $tc_data->{setup_info}->{spare_setup}, mirror=>$mirror);
            $spare_disks->compare_spare_setup_info(total_spare_requirements => $total_spare_requirements);
            $spare_disks->validate_spare_data();

            $self->set_spare_disks(object=>$spare_disks);
        }
        $Log->comment("Validated tc_data: OK");
        $Log->comment("Valdiate_tc_data: Debug: Dumper: ".Dumper($self->get_tc_data()));
    }

    sub validate_error_info {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => SCALAR});
        my $filer_vol  = $opts{filer_vol};
        my $aggr       = $self->get_aggregate(filer_vol => $filer_vol);
        my $error_info = $self->get_error_info(filer_vol => $filer_vol);
        my $tc_data    = $self->get_tc_data();
        my $raidio     = $aggr->get_raidio();
        my $raid_size  = $aggr->get_raid_size();
        my $raid_type  = $aggr->get_raid_type();
        my $min_raid_size = ($raid_type eq "raid_dp") ? 5 : 3;
        my ($plex_count, $disk_count, $rg_count);

        #=============================================================================
        # Check for plexes
        $Log->comment("Starting validation of plexes....");
        my @plex_list = grep (/^plex(\d)$/i, keys %$error_info);
        $plex_count = scalar(@plex_list);

        if($plex_count == 0){
            die("Validated plexes: Error: plex data is missing");
        }elsif($plex_count > 2){
            die("Validated plexes: Error: more than 2 plexes defined") ;
        }

        # if($plex_count < 2 && grep(/verifyio/, @$raidio)) {
            # die("Validated plexes: Error: 2 plexes are required for verifyio");
        # }

        $Log->comment("Validated plexes: @plex_list");
        $Log->comment("Validated plexes: OK");

        #=============================================================================
        # Check for plexes
        $Log->comment("Starting validation of rg_count....");
        foreach my $plex (@plex_list){
            my @rg_list = grep (/rg/i, keys %{$error_info->{$plex}});

            if(scalar(@rg_list) > scalar(@{$aggr->get_rgs()})) {
                die("Validated rg_count: Error: raid groups data exceed rg count in aggr- ".scalar(@{$aggr->get_rgs()}));
            }
            $Log->comment("Validated rg_count: plex - $plex, rg_list - @rg_list");
        }
        $Log->comment("Validated rg_count: OK");

        #=============================================================================
        # Check for error inject data
        $Log->comment("Validating error inject data...");
        $self->set_error_type(type => 'recover');
        my $total_disk_fail_offline_count = 0;

        foreach my $plex (@plex_list){
            my @rg_list = grep (/rg/i, keys %{$error_info->{$plex}});
            foreach my $rg_index (0..scalar(@rg_list)-1){
                my $rg = $rg_list[$rg_index];
                if(exists($error_info->{$plex}->{$rg}->{stripe_gap})) {
                    my $stripe_gap = $error_info->{$plex}->{$rg}->{stripe_gap};
                    if(!$stripe_gap =~ /\d+/) {
                        die("Validated error inject data: Error: stripe_gap should be numeric - Actual value - $stripe_gap");
                    }
                }
                $Log->comment("Validated error inject data: stripe_gap for plex - $plex, rg - $rg is ".
                               $error_info->{$plex}->{$rg}->{stripe_gap});

                my @err_list = grep(/^\s*(zones|disk_fail|disk_offline)\s*$/i, keys %{$error_info->{$plex}->{$rg}});
                if(!scalar(@err_list)){
                    die("Validated error inject data: Error: incomplete err inject data for plex - $plex, rg - $rg");
                }

                my $disk_fail_offline_count = 0;
                my %disks_hash = ();
                foreach my $err_type (@err_list){
                    my @errors = @{$error_info->{$plex}->{$rg}->{$err_type}};
                    if(!scalar(@errors)) {
                        die("Validated error inject data: Error: $err_type mentioned but data missing for plex - $plex, rg - $rg");
                    }

                    if($err_type eq "disk_offline" or $err_type eq "disk_fail"){
                        foreach my $errid (0..$#errors){
                            if(exists ${$errors[$errid]}{disk_type}){
                                my $disk_type = ${$errors[$errid]}{disk_type} ;
                                if($disk_type eq "DATA" or $disk_type eq "PARITY" or $disk_type eq "DPARITY"){
                                    if($aggr->get_raid_type() =~ /raid4/ and $disk_type eq "DPARITY"){
                                        die("Validated error inject data: Error: DPARITY mentioned for raid4 at plex - $plex, rg - $rg, err_type - $err_type");
                                    }
                                    $disk_fail_offline_count = $disk_fail_offline_count +1 ;
                                    $disks_hash{$disk_type} = !exists $disks_hash{$disk_type} ? 1 : $disks_hash{$disk_type}+1;
                                }else{
                                    die("Validated error inject data: Error: invalid disk_type mentioned for plex - $plex, rg - $rg, err_type - $err_type");
                                }
                            }else{
                                die("Validated error inject data: Error: disk_type missing for - $plex, rg - $rg, err_type - $err_type");
                            }
                        }
                    }
                }

                if($disk_fail_offline_count > $min_raid_size-1){
                    die("Validated error inject data: Error: disks to be failed/offlined is greater than raid tolerance - $min_raid_size for plex - $plex, rg- $rg");
                }
                $total_disk_fail_offline_count += $disk_fail_offline_count;
                $Log->comment("Validated error inject data: Info: Disks to fail is within limits of raid_size - $raid_size and raid_type - ".$aggr->get_raid_type());

                #Check if disk fail/offline count exceeds than allowed number of disk fail/offline for scrubio
                my $disk_fail_offline_threshold = ($raid_type =~ /raid_dp/) ? 1 : 0;
                if($disk_fail_offline_count > $disk_fail_offline_threshold && (grep /scrubio/, @$raidio)) {
                    die("Validated error inject data: Error: disk fail/offline count exceeds the number allowed for scrubio for plex: $plex, rg: $rg");
                }

                if(ref($error_info->{$plex}->{$rg}->{zones})) {
                    my @zones = @{$error_info->{$plex}->{$rg}->{zones}};
                    my $rgs = $aggr->get_rgs();

                    #If aggr is hybrid_enabled and RG is ssd tier then no error should be injected on this RG.
                    if(scalar @zones > 0 && $aggr->get_hybrid_aggregate() && $rgs->[$rg_index]->{ssd_tier}) {
                        die("Validated error inject data: Error: Error can not be injected on SSD tier RG in hybrid aggr");
                    }

                    foreach my $zoneid (0..$#zones) {
                        if(!exists ${$error_info->{$plex}->{$rg}->{zones}->[$zoneid]}{zone_same_plex_dbn}){
                            my $default_zone_same_plex_dbn = $plex_count - 1;
                            $Log->comment("Validated error inject data: Info: zone_same_plex_dbn not defined for plex - $plex, rg - $rg Changing to default - $default_zone_same_plex_dbn");
                            $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{zone_same_plex_dbn} = $default_zone_same_plex_dbn;
                        } else {
                            if($plex_count == 1 && $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{zone_same_plex_dbn} == 1) {
                                die("Validated error inject data: Error: zone_same_pelx_dbn should be 0 when there is only one plex");
                            }
                        }

                        if(!defined($error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes})) {next;}
                        my @stripes = @{$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}};
                        foreach my $stripeid (0..$#stripes){
                            my $stripe_err_count = 0;
                            my %stripes_disks_hash = ();

                            # Check for type
                            if(!exists ${$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]}{type} ){
                                $Log->comment("Validated error inject data: Info: error type not defined for plex - $plex, rg - $rg, stripe - ".($stripeid+1)." Changing to default - recover");
                                $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{type} = 'recover';
                            }

                            # Check for stripe_same_plex_dbn
                            if(!exists ${$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]}{stripe_same_plex_dbn} ){
                                my $default_stripe_same_plex_dbn = $plex_count - 1;
                                $Log->comment("Validated error inject data: Info: stripe_same_plex_dbn not defined for plex - $plex, rg - $rg, stripe - ".($stripeid+1)." Changing to default - $default_stripe_same_plex_dbn");
                                $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{stripe_same_plex_dbn} = $default_stripe_same_plex_dbn;
                            } else {
                                if($plex_count == 1 && $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{stripe_same_plex_dbn} == 1) {
                                    die("Validated error inject data: Error: stripe_same_plex_dbn should be 0 when there is only one plex");
                                }
                            }

                            my @stripe_errs = grep(/(checksum|media|parity_inconsistency|zeroblock)/, keys %{$stripes[$stripeid]});
                            if(!scalar(@stripe_errs)) {
                                die("Validated error inject data: Error: incomplete err inject data for plex - $plex, rg - $rg, stripe - ".($stripeid+1));
                            }

                            foreach my $stripe_err_type (@stripe_errs){
                                my @stripe_err_ids = @{$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err_type}};
                                my @used_disk_ids = ();
                                foreach my $stripe_err_id (0..$#stripe_err_ids){
                                    my @stripe_offsets = ();
                                    my $disk_type = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err_type}->[$stripe_err_id]->{disk_type};
                                    my $bit_set = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err_type}->[$stripe_err_id]->{bit_set};
                                    my $stripe_offset = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err_type}->[$stripe_err_id]->{stripe_offset};

                                    if(defined($stripe_offset) and $stripe_offset > 63 or $stripe_offset < 0) {
                                        die("Validated error inject data: Error: Invalid value set for stripe offset: actual - $stripe_offset, expected - 0 to 63");
                                    } elsif(defined($stripe_offset) && grep /$stripe_offset/, @stripe_offsets) {
                                        $Log->comment("Validated error inject data: Info:stripe_offset - $stripe_offset, used stripe_offsets: @stripe_offsets");
                                        die("Validated error inject data: Error: Same stripe offset can not be used for two stripes in zone");
                                    }

                                    if(!defined($bit_set)) {
                                        $Log->comment("Validated error inject data: Info: bit_set is not defined for \n".
                                                       "filer_vol: $filer_vol, plex: $plex, rg: $rg, error: $stripe_err_type - Changing it to default 'dump'");
                                        $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err_type}->[$stripe_err_id]->{bit_set} = "dump";
                                        $bit_set = "dump";
                                    }
                                    if($bit_set !~ /(dump|none|pseudo_bad|unverified)/) {
                                        $Log->comment("Validated error inject data: Info: filer_vol: $filer_vol, plex: $plex, rg: $rg, error: $stripe_err_type");
                                        die("Validated error inject data: Error: bit_set is not set to one of valid value: $bit_set");
                                    }

                                    if($disk_type eq "DATA" or $disk_type eq "PARITY" or $disk_type eq "DPARITY"){
                                        if($raid_type =~ /raid4/ and $disk_type eq "DPARITY"){
                                            die("Validated error inject data: Error: DAPRITY mentioned for raid4 at plex - $plex, rg - $rg, err_type - $stripe_err_type");
                                        }
                                        $stripe_err_count = $stripe_err_count +1 ;
                                        $stripes_disks_hash{$disk_type} = !exists $stripes_disks_hash{$disk_type} ? 1 : $stripes_disks_hash{$disk_type}+1;
                                    }else{
                                        die("Validated error inject data: Error: invalid disk_type mentioned for plex - $plex, rg - $rg, err_type - $stripe_err_type: $disk_type");
                                    }

                                    #Check for disk_id is not used already
                                    if($disk_type eq "DATA"){
                                        my $disk_id = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err_type}->[$stripe_err_id]->{disk_id};
                                        if(grep /$disk_id/, @used_disk_ids) {
                                            die("Validated error inject data: Error: disk id $disk_id is already used by other error in this stripe");
                                        } else {
                                            push(@used_disk_ids, $disk_id);
                                        }
                                    }

                                    if($stripe_err_type eq "media") {
                                        my $inject_method = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err_type}->[$stripe_err_id]->{inject_method};
                                        if(defined ($inject_method) && $inject_method !~ /fiji|disk_mung/) {
                                            die("Validated error inject data: Error: Inject Method is not correct for media error - $inject_method, valid methods - fiji, disk_mung");
                                        } elsif(!defined($inject_method)) {
                                            $Log->comment("Validated error inject data: Info: Inject method is not defined for media error, setting it to default  - 'fiji'");
                                            $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err_type}->[$stripe_err_id]->{inject_method} = 'fiji';
                                        }
                                    }

                                    #Check for rgdn value
                                    if($stripe_err_type eq "checksum") {
                                        my $error = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err_type}->[$stripe_err_id];
                                        if(exists $error->{disk_rgdn}) {
                                            my @disk_rgdn = split(/,/, $error->{disk_rgdn});
                                            foreach my $rgdn_no (@disk_rgdn) {
                                                if($rgdn_no > $raid_size-1) {
                                                    die("validate_tc_data: Error: disk rgdn is more than max rgdn: max_rgdn: ".($raid_size -1)."rgdn: $rgdn_no");
                                                }
                                            }
                                        }
                                    }
                                }

                                $self->compare_disks_list(failing_disks=>\%disks_hash, raid_type => $aggr->get_raid_type(),
                                                          stripes_disks=>\%stripes_disks_hash, raid_size=>$raid_size);
                                $Log->comment("Validated error inject data: Stripe data validated for plex - $plex, rg - $rg");
                            }

                            if(${$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]}{type} eq 'unrecover'){
                                $self->set_error_type(type => 'unrecover');
                                $Log->comment("Validated error inject data: marking global type as \"unrecover\"".
                                              "as one of the stripe is unrecover at plex - $plex, rg - $rg");
                            }

                            if(${$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]}{type} eq 'recover' and ($stripe_err_count+$disk_fail_offline_count) > $min_raid_size-1){
                                # die("Validated error inject data: Error: error injection on this stripe is greater than recover tolerance - $min_raid_size for plex - $plex, rg- $rg");
                                if($aggr->is_mirror() && ($stripe_err_count+$disk_fail_offline_count) > $raid_size){
                                    die("Validated error inject data: Error: error injection on this stripe is greater than raid size - $raid_size for plex - $plex, rg- $rg");
                                } elsif($aggr->is_mirror() == 0) {
                                    die("Validated error inject data: Error: error injection on this stripe is greater than recover tolerance - $min_raid_size for plex - $plex, rg- $rg");
                                }
                            }

                            if(${$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]}{type} eq 'unrecover' and ($stripe_err_count+$disk_fail_offline_count) > $raid_size){
                                die("Validated error inject data: Error: error injection on this stripe is greater than raid size - $raid_size for plex - $plex, rg- $rg");
                            }
                        }
                    }
                }
                $Log->comment("Validated error inject data: error list for plex - $plex, rg - $rg is @err_list");
            }
        }

        if ($total_disk_fail_offline_count == 0 and ($tc_data->{start_recons_always} == 1 or grep(/reconsio/, @$raidio))) {
            die("Validated error inject data: Error: 1 disk has to be failed/offlined in one of RG for reconsio. Actual disk failed/offlined count - $total_disk_fail_offline_count");
        } elsif($total_disk_fail_offline_count > 0 and grep(/reparityio/, @$raidio)) {
            die("Validated error inject data: Error: disk should not be failed/offlined in any RG for reparityio. Actual disk failed/offlined count - $total_disk_fail_offline_count");
        } else {
            $Log->comment("Validated error inject data: Info: Total disk fail/offline count - $total_disk_fail_offline_count");
        }
        $Log->comment("Validated error inject data: OK");
        $self->set_tc_data(tc_data => $tc_data);
    }

    sub init_fileCreate {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => {type=>SCALAR}});
        my $filer_vol = $opts{filer_vol};
        my $aggr = $self->get_aggregate(filer_vol=>$filer_vol);
        my $rgs  = $aggr->get_rgs();
        my $file_info = {};

        my $error_info = $self->get_error_info(filer_vol=>$filer_vol);
        my @plex_list = sort grep /plex/, keys %$error_info;
        my $plex0 = $plex_list[0];

        my $rg_index = 0;
        foreach my $rg (sort grep /rg/, keys(%{$error_info->{$plex0}})) {
            #If aggr is hybrid_enabled and RG is ssd_tier then File will not be created for that RG
            if($rgs->[$rg_index]->{ssd_tier} == 1 && $aggr->get_hybrid_aggregate() == 1) {
                $rg_index++;
                next;
            }
            my $rg_name = "/$filer_vol/$plex0/$rg";
            my $stripe_gap = $self->get_stripe_gap(filer_vol=>$filer_vol, rg=>$rg);
            if($stripe_gap > 0){
                $file_info->{$filer_vol}->{$rg_name}->{stripe_gap} = $stripe_gap;
                my $unique_stripe_count = $self->get_unique_stripe_count(filer_vol=>$filer_vol, rg=>$rg);
                $file_info->{$filer_vol}->{$rg_name}->{unique_stripe_count} = $unique_stripe_count;
                $file_info->{$filer_vol}->{$rg_name}->{required_file_size} = $self->compute_required_file_size(
                    filer_vol    => $filer_vol,
                    stripe_count => $unique_stripe_count,
                    stripe_gap   => $stripe_gap
                );
            } else {
                my $zones = $self->get_unique_zones(filer_vol=>$filer_vol, rg=>$rg);
                $file_info->{$filer_vol}->{$rg_name}->{zones} = $zones if(ref($zones) =~/array/i) ;
                $file_info->{$filer_vol}->{$rg_name}->{required_file_size} = $self->compute_required_file_size(
                    filer_vol  => $filer_vol,
                    zone_count => scalar(@$zones)
                ) if(ref($zones) =~/array/i);
            }

            $rg_index++;
        }

        my $file_create_obj  = FileCreate->new(api_set_obj=>$self->{api_set_obj}, aggregate=>$aggr, file_info=>$file_info);
        return $file_create_obj;
    }

    # N.Seshadri: This subroutine does following tasks:
    #   compares if stripe error is injected on failing/offlined disks
    #   If data disks
    sub compare_disks_list{
        my $self = shift;
        my (%opts) = validate(@_,{
                                raid_type        => {type=>SCALAR},
                                failing_disks    => {type=>HASHREF, default=>undef, optional=>1},
                                stripes_disks    => {type=>HASHREF, default=>undef, optional=>1},
                                raid_size        => {type=>SCALAR, optional=>1},
                             });
        my %failing_disks = %{$opts{failing_disks}};
        my %stripes_disks = %{$opts{stripes_disks}};
        my $raid_size     = $opts{raid_size};
        my $tc_data       = $self->get_tc_data();
        my $parity_disks  = $opts{raid_type} =~ /raid4/ ? 1 : 2;

        foreach my $disk_type ("PARITY", "DPARITY"){
            if(exists $failing_disks{$disk_type} and exists $stripes_disks{$disk_type}){
                die("Validated error inject data: Error: invalid error injected on failing/offlining $disk_type disk");
            }
        }

        $failing_disks{DATA} = exists $failing_disks{DATA} ? $failing_disks{DATA} : 0;
        $stripes_disks{DATA} = exists $stripes_disks{DATA} ? $stripes_disks{DATA} : 0;
        if(exists $failing_disks{DATA} or exists $stripes_disks{DATA}){
            if(($failing_disks{DATA}+$stripes_disks{DATA}) > ($raid_size-$parity_disks)){
                die("Validated error inject data: Error: (Failed/Offlined disks + stripe error ) - ".
                    "(".$failing_disks{DATA}."+".$stripes_disks{DATA}.") = ".
                    ($failing_disks{DATA}+$stripes_disks{DATA}).
                    " on data disks exceed total data disks - ".($raid_size-$parity_disks));
            }
        }
    }

    sub setup_spares {
        my $self = shift;
        my $spares_object = $self->get_spares_object();
        $spares_object->setup_spares();
    }

    sub get_spare_disks {
        my $self = shift;
        return $self->{spare_disks};
    }

    sub set_spare_disks {
        my $self = shift;
        my %opts = validate(@_, {
            object => {type => OBJECT, isa=>"SpareDisks"},
        });
        $self->{spare_disks} = $opts{object};
    }

    sub create_aggr {
        my $self = shift;
        my %opts = validate(@_, {
            filer_vol_list => {type => ARRAYREF, default=>[$self->get_filer_vol_list()], optional=>1},
        });

        my $api_set_obj = $self->{api_set_obj};
        my $tc_data = $self->get_tc_data();
        my @filer_vol_list = @{$opts{filer_vol_list}};
        my @new_filer_vol_list = ();
        # Enable system wide raid_lost_write option before creating aggregate
        RaidSqa_set_raid_lw_opt(api_set_obj  => $api_set_obj,option_value => 'on');

        foreach my $filer_vol (@filer_vol_list) {
            my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
            if($aggr->exists()) {
                my $filer_vol = $aggr->get_filer_vol();
                my $filer_vol1 = $filer_vol."$$";
                $Log->comment("create_aggr: Info: User defined filer_vol - $filer_vol already exists, ".
                              "hence creating vol with new name - $filer_vol1");

                $self->update_aggr_name(old_filer_vol => $filer_vol, new_filer_vol => $filer_vol1);
            }

            my $aggr_create_sequence = $aggr->get_aggr_create_sequence();
            if(defined($aggr_create_sequence)) {
                my @aggr_create_sequence = split(/,\s*/,$aggr_create_sequence);
                if ($aggr_create_sequence[0] =~ /^m$/i) {
                    @aggr_create_sequence[1,0] = @aggr_create_sequence[0,1];
                }

                $aggr->create();
                my @rgs = @{$aggr->get_rgs()};
                shift @aggr_create_sequence;
                shift @rgs;
                my $hybrid_enable_flag = 0;
                my $mirrored_after_aggr_create = 0 ;

                #my $rg_index = 0;
                foreach my $create_sequence (@aggr_create_sequence) {
                    if (($create_sequence =~ /^s/i) && (!$hybrid_enable_flag)) {
                        $aggr->enable_hybrid_aggr();
                        $hybrid_enable_flag = 1;
                    }
                    if ($create_sequence =~ /^m$/i) {
                        if (!RaidSqa_aggr_mirror(api_set_obj=>$api_set_obj, filer_vol=>$aggr->get_filer_vol())) {
                            die("create_aggr: Error: Failed to mirror aggr-".$aggr->get_filer_vol());
                        }
                        sleep(10);
                        my $timeout_resync = time() + 3600;
                        my $resync_state = RaidSqa_check_aggr_mirror_state(api_set_obj => $api_set_obj,filer_vol => $aggr->get_filer_vol(),aggr_type => $aggr->get_aggr(),mirror_state => 'RESYNC');
                        while (($resync_state) && ($timeout_resync > time())) {
                            $resync_state = RaidSqa_check_aggr_mirror_state(api_set_obj => $api_set_obj,filer_vol => $aggr->get_filer_vol(),aggr_type => $aggr->get_aggr(),mirror_state => 'RESYNC');
                            sleep (30);
                        }
                        $mirrored_after_aggr_create = 1;
                        next;
                    }
                    $aggr->add_raid_groups(raid_group => [shift @rgs], mirror => $aggr->get_mirror() || $mirrored_after_aggr_create);
                }

                # Check aggr checksum_type as mixed in case bcs and azcs rg's are present.
                if (($aggr_create_sequence =~ /\bha\b/i) && ($aggr_create_sequence =~ /\bhb\b/i)) {
                    $Log->comment("create_aggr: Info: Check aggr-".$aggr->get_filer_vol()." checksum_type as \'mixed\'");
                    my $aggr_checksum_type = RaidSqa_get_aggr_cksum_type(api_set_obj => $api_set_obj, filer_vol => $aggr->get_filer_vol());
                    if ($aggr_checksum_type eq "mixed") {
                        $Log->comment("create_aggr: Info: aggr-".$aggr->get_filer_vol()." checksum_type-$aggr_checksum_type as expected");
                    } else {
                        die("create_aggr: Error: checksum_type for Volume-".$aggr->get_filer_vol().";Expected-mixed ; Actual-$aggr_checksum_type");
                    }
                }
                # Check if aggr is marked as hybrid in case ssd disks are added to aggr.
                if ($aggr_create_sequence =~ /\bs/i) {
                    # get raidInfo object
                    my $raid_info_object = $self->get_raid_info_object();
                    my $aggr_obj = $raid_info_object->aggregate($aggr->get_filer_vol());
                    if (defined($aggr_obj)) {
                        if ($aggr_obj->parameter("hybrid") == 1) {
                            $Log->comment("create_aggr: Info: aggr-".$aggr->get_filer_vol()." is hybrid aggregate");
                        } else {
                            die("create_aggr: Error: Failed to create aggr-".$aggr->get_filer_vol()."as hybrid aggr");
                        }
                    } else {
                        $Log->comment("create_aggr: Error: Failed to create RaidInfo object for filer_vol-".$aggr->get_filer_vol());
                    }
                }
            } else {
                $aggr->create();
                $aggr->enable_hybrid_aggr();
                $aggr->add_raid_groups();
            }
            $self->refresh_tc_data(filer_vol => $aggr->get_filer_vol());
            push(@new_filer_vol_list, $aggr->get_filer_vol());
        }
        #This is required to inform caller of this function about changes in the names in aggregates
        return \@new_filer_vol_list;
    }

    sub refresh_tc_data {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol => {type => SCALAR},
                    });

        my $aggr = $self->get_aggregate(filer_vol => $opts{filer_vol});
        $aggr->refresh_aggr_info();
        $self->populate_error_info_disks(filer_vol=>$opts{filer_vol});
    }

    sub get_filer_vol_list {
        my $self = shift;

        my @aggr_names = keys %{$self->{aggr_create_hash}};
        return @aggr_names;
    }

    sub cleanup_aggr {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol_list => {type => ARRAYREF, default=>[$self->get_filer_vol_list()], optional=>1},
                    });

        my $api_set_obj = $self->{api_set_obj};

        #Clear all fijis
        $Log->comment("cleanup_aggr: Info: Clear all fiji points");
        RaidSqa_fiji_clear(api_set_obj => $api_set_obj, fiji_point => "all");

        #for each aggregate, delete the aggregate
        $Log->comment("cleanup_aggr: Info: Remove all aggregates");
        foreach my $filer_vol (@{$opts{filer_vol_list}}) {
            my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
            if(!defined $aggr) { next ; }
            $aggr->destroy();
        }

        #Start disk zeroing
        $Log->comment("cleanup_aggr: Info: Start disk zeroing after aggr delete");
        RaidSqa_disk_zero_spares(api_set_obj => $api_set_obj);
    }

    sub create_test_file {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => {type=>SCALAR}});
        my $filer_vol = $opts{filer_vol};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        $aggr->create_test_file();
    }

    sub collate_file_info {
        my $self = shift;
        my (%opts) = validate(@_, {
                        filer_vol => {type=>SCALAR},
                    });
        my $filer_vol = $opts{filer_vol};
        my $api_set_obj = $self->{api_set_obj};

        my $error_info = $self->get_error_info(filer_vol => $filer_vol);
        my $aggr = $self->get_aggregate(filer_vol=>$filer_vol);
        $aggr->refresh_checksum_type();
        my $file = $aggr->get_fileCreate();
        my $file_info = $file->get_file_info();

        my @plex_list = sort grep /^plex\d$/, keys %$error_info;
        foreach my $rg (grep /^rg\d+/, keys %{$error_info->{$plex_list[0]}}) {
            my $rg_name = "/$filer_vol/$plex_list[0]/$rg";
            next if(!exists($file_info->{$filer_vol}->{$rg_name}));
            # my $dbn_data = $file_info->{dbn};
            if($self->get_stripe_gap(filer_vol=>$filer_vol, rg=>$rg)){
                my @dbn_list = sort {$a <=> $b} keys %{$file_info->{$filer_vol}->{$rg_name}->{dbn}};
                my $dbn_data = $file_info->{$filer_vol}->{$rg_name}->{dbn};
                my $dbn_pointer = 0;
                foreach my $zoneid (0..scalar(@{$error_info->{$plex_list[0]}->{$rg}->{zones}})-1){
                    my $zone = $error_info->{$plex_list[0]}->{$rg}->{zones}->[$zoneid];
                    foreach my $stripeid (0..scalar(@{$zone->{stripes}})-1) {
                        $self->set_stripe_dbn(filer_vol=>$filer_vol, plex=>$plex_list[0], rg=>$rg, zoneid=>$zoneid, stripeid=>$stripeid, dbn=>$dbn_list[$dbn_pointer], dbn_data=>$dbn_data);
                        if(exists $error_info->{$plex_list[1]}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]) {
                            if($self->is_stripe_same_plex_dbn(filer_vol=>$filer_vol, rg=>$rg, zoneid=>$zoneid, stripeid=>$stripeid)){
                                $self->set_stripe_dbn(filer_vol=>$filer_vol, plex=>$plex_list[1], rg=>$rg, zoneid=>$zoneid, stripeid=>$stripeid, dbn=>$dbn_list[$dbn_pointer], dbn_data=>$dbn_data);
                            } else {
                                $dbn_pointer++;
                                $self->set_stripe_dbn(filer_vol=>$filer_vol, plex=>$plex_list[1], rg=>$rg, zoneid=>$zoneid, stripeid=>$stripeid, dbn=>$dbn_list[$dbn_pointer], dbn_data=>$dbn_data);
                            }
                        }
                        $dbn_pointer++;
                    }
                }
            } else {
                my $zone_cksum_dbn = $file_info->{$filer_vol}->{$rg_name}->{zone_cksum_dbn};
                my $dbn_data = $file_info->{$filer_vol}->{$rg_name}->{dbn};
                my $cksum_dbn_pointer = 0;
                if(ref($error_info->{$plex_list[0]}->{$rg}->{zones}) =~ /array/i) {
                    foreach my $zoneid (0..scalar(@{$error_info->{$plex_list[0]}->{$rg}->{zones}})-1) {
                        $self->set_zone_dbn(filer_vol=>$filer_vol, plex=>$plex_list[0], rg=>$rg, zoneid=>$zoneid, zone_cksum_dbn=>$zone_cksum_dbn->[$cksum_dbn_pointer], dbn_data=>$dbn_data);
                        if(exists $error_info->{$plex_list[1]}->{$rg}->{zones}->[$zoneid]) {
                            if($self->is_zone_same_plex_dbn(filer_vol=>$filer_vol, rg=>$rg, zoneid=>$zoneid)){
                                $self->set_zone_dbn(filer_vol=>$filer_vol, plex=>$plex_list[1], rg=>$rg, zoneid=>$zoneid, zone_cksum_dbn=>$zone_cksum_dbn->[$cksum_dbn_pointer], dbn_data=>$dbn_data);
                            } else {
                                $cksum_dbn_pointer++;
                                $self->set_zone_dbn(filer_vol=>$filer_vol, plex=>$plex_list[1], rg=>$rg, zoneid=>$zoneid, zone_cksum_dbn=>$zone_cksum_dbn->[$cksum_dbn_pointer], dbn_data=>$dbn_data);
                            }
                        }
                        $cksum_dbn_pointer++;
                    }
                }
            }
        }
    }

    # Algorithm
        # For a given filer_vol, traverse each rg in %file_info
        #     read file_name and fileid
        #     generate md5sum for this file and save it
        #     now traverse through each dbn value
        #        populate its fields
        #        get all disks related to this rg from aggr_info and call get_stripes to populate stripe checksum
        # At the end gather all files list in /etc/crash as pi_files_list entry denoting ARRAYREF
    sub save_validation_data {
        my $self =shift;
        my (%opts) = validate(@_, {
                        filer_vol               => {type=>SCALAR},
                        context                 => {type=>SCALAR},
                        save_stripe_checksum    => {type=>SCALAR, default=>1, optional=>1},
                    });

        my $filer_vol = $opts{filer_vol};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $file = $aggr->get_fileCreate();
        my $context = $opts{context};
        my $api_set_obj = $self->{api_set_obj};

        if($context !~ /^(before|after)$/) {
            die("save_validation_data: Invalid param value for param context: $context");
        }

        $Log->comment("save_validation_data: Debug: filer_vol - $filer_vol");
        my $raidlw = new RaidLW(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol, mount=>1, unmount=>0);

        foreach my $rg_name ($file->get_rg_list()){
            my $file_name = $file->get_test_file_name(rg_name=>$rg_name);
            $Log->comment("save_validation_data: reading md5sum for file - $file_name, rg_name - $rg_name");
            next if ($file_name !~ /\w/i);
            my $md5sum = RaidSqa_get_md5sum(api_set_obj=>$api_set_obj, file_name => $file_name);
            $file->set_md5sum(rg_name => $rg_name, md5sum => $md5sum, context => $context);
            $Log->comment("save_validation_data: Info: md5sum for file - $file_name is $md5sum");
            if(!$md5sum){
                $Log->comment("save_validation_data: Info: As md5sum is 0, probably reading file failed: $md5sum");
            }

            if($opts{save_stripe_checksum} == 1) {
                foreach my $dbn ($file->get_dbn_list(rg_name => $rg_name)){
                    $rg_name =~ /\/$filer_vol\/(plex\d)\/(rg\d+)/;
                    my ($plex, $rg) = ($1, $2);
                    my $disks = $aggr->get_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>'other');
                    my $media_disks = $self->get_error_injected_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, error_type=>'media', dbn=>$dbn);

                    my %disk_hash = map {$_, 1 } @$media_disks;
                    my @non_media_stripe_disks = grep(!$disk_hash{$_}, @$disks);

                    $Log->comment("save_validation_data: Info: stripe checksum is collected for file - $file_name at plex - $1, rg - $2, dbn - $dbn".
                                  "\nDisks on this stripe are @non_media_stripe_disks.  \nNote: These are only non-failed, non-media-error or online disks");

                    $file->set_stripe_checksum( rg_name=>$rg_name, dbn=>$dbn, context=>$context,
                                                        stripe_checksum=>$raidlw->get_stripes(disk_list=>\@non_media_stripe_disks, dbn=>$dbn));
                    $raidlw->print_stripes(disk_list=>$disks, dbn=>$dbn);
                }
            }
        }

        $Log->comment("save_validation_data: Info: gathering files list from /etc/crash");
        $file->save_pi_files_list(context=>$context);

        $Log->comment("save_validation_data: Info: aggr status");
        foreach my $option ("-v", "-r") {
            RaidSqa_get_aggr_status(api_set_obj=>$api_set_obj, options=>$option);
        }
        $file->print_file_info();
        my $ems_mark =$aggr->get_ems_marker();
        if($ems_mark) {
            my $ref_events = RaidSqa_ems_log_dump_by_type(api_set_obj => $self->{api_set_obj}, timestamp => $ems_mark);
            my @ems_events = keys %$ref_events;
            $Log->comment("save_validate_data: Info: Ems events occured in this run: \n". join("\n", @ems_events));
        }
    }

    sub prepare_config_state {
        my $self =shift;
        my (%opts) = validate(@_, {
                            raidio_type => {type=>SCALAR},
                            filer_vol   => {type=>SCALAR},
                        });

        my $raidio_type = $opts{raidio_type};
        my $filer_vol = $opts{filer_vol};
        my $api_set_obj = $self->{api_set_obj};

        my $error_info = $self->get_error_info(filer_vol=>$filer_vol);
        my @plex_list = grep (/^plex(\d)$/i, keys %$error_info);

        #Since disk offline will fail if there is any disk failed/reconstructing
        # we will offline all disks first and then will fail disks
        foreach my $plex (@plex_list){
            my @rg_list = grep (/rg/i, keys %{$error_info->{$plex}});
            foreach my $rg (@rg_list){
                my @err_list = grep(/^\s*(disk_offline)\s*$/i, keys %{$error_info->{$plex}->{$rg}});
                if(scalar(@err_list)){
                    $Log->comment("inject_error: Info: disk_offline error exists at plex - $plex, rg - $rg. Disabling Recons");
		    RaidSqa_setflag(api_set_obj => $api_set_obj,flag_name   => 'raid_disable_new_recons',flag_value  => 1);
                    if (exists $error_info->{$plex}->{$rg}->{disk_offline}){
                        my @errors = @{$error_info->{$plex}->{$rg}->{disk_offline}};
                        foreach my $disk_offline_id (0..$#errors){
                            my $disk_name = $error_info->{$plex}->{$rg}->{disk_offline}->[$disk_offline_id]->{disk_name};
                            my $disk_type = $error_info->{$plex}->{$rg}->{disk_offline}->[$disk_offline_id]->{disk_type};
                            $Log->comment("inject_error: Info: offlining disk - $disk_name, disk_type - $disk_type, disk_offline_index - $disk_offline_id");

                            if (!RaidSqa_disk_offline(api_set_obj=>$api_set_obj, disk_name=>$disk_name)){
                                sleep(3);
                                if (!RaidSqa_check_disk_fsm_state(api_set_obj=>$api_set_obj, disk_name=>$disk_name, fsm_state=>"OFFLINE")){
                                    die("inject_error: Info: disk_offline operation failed for disk - $disk_name");
                                }
                            }
                        }
                    }
                }
            }
        }

        foreach my $plex (@plex_list){
            my @rg_list = grep (/rg/i, keys %{$error_info->{$plex}});
            foreach my $rg (@rg_list){
                my @err_list = grep(/^\s*(disk_fail)\s*$/i, keys %{$error_info->{$plex}->{$rg}});
                if(scalar(@err_list)){
                    $Log->comment("inject_error: Info: disk_fail error exists at plex - $plex, rg - $rg. Disabling Recons");
		    RaidSqa_setflag(api_set_obj => $api_set_obj,flag_name   => 'raid_disable_new_recons',flag_value  => 1);
                    if(exists $error_info->{$plex}->{$rg}->{disk_fail}){
                        my @errors = @{$error_info->{$plex}->{$rg}->{disk_fail}};
                        foreach my $disk_fail_id (0..$#errors){
                            my $disk_name = $error_info->{$plex}->{$rg}->{disk_fail}->[$disk_fail_id]->{disk_name};
                            my $disk_type = $error_info->{$plex}->{$rg}->{disk_fail}->[$disk_fail_id]->{disk_type};
                            my $immediate = (lc $error_info->{$plex}->{$rg}->{disk_fail}->[$disk_fail_id]->{type} eq "immediate") ? '-i' : "";
                            my $fsm_state = ($immediate eq "-i") ? "FAILED" : "INUSE";
                            $Log->comment("inject_error: Info: failing disk - $disk_name, disk_type - $disk_type, disk_fail_index - $disk_fail_id");
                            if (!RaidSqa_disk_fail(api_set_obj=>$api_set_obj, disk_name=>$disk_name, options=>$immediate)){
                                sleep(3);
                                if (!RaidSqa_check_disk_fsm_state(api_set_obj=>$api_set_obj, disk_name=>$disk_name, fsm_state=>$fsm_state)){
                                    die("inject_error: Info: disk_fail operation failed for disk - $disk_name");
                                }
                            }
                        }
                    }
                }
            }
        }

        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $plex_to_offline = $aggr->get_plex_offline();

        if($plex_to_offline !~ /undef/){
            if($aggr->get_plex_offline_always() == 1 or $raidio_type =~ /resyncio/i) {
                # Offline this plex
                my $result = RaidSqa_plex_offline(api_set_obj => $api_set_obj, plex_name=>"/$filer_vol/$plex_to_offline");
                if($result == 0){
                    die("prepare_config_state: Error: Plex offline failed. plex - /$filer_vol/$plex_to_offline");
                }
            }
        }
    }

	# Algorithm:
        # For all plexes
            # For all rgs
                # if disk_fail or disk_offline exist:
                    # then disable recons
                # if disk_fail:
                    # fail_disk(disk_name)
                # elsif disk_offline:
                    # offline_disk(disk_name)
                # elsif stripes:
                    # newDBN = dbn
                    # if inject_at == checksum_blk:
                    # if media:
                        # inject_media_error(disk_name, newDBN)
                    # elsif checksum:
                        # inject_checksum(disk_name, newDBN, disk_type, checksum_hash)
                    # elsif zeroblock:
                        # inject_zeroblock(disk_name, newDBN)
                    # elsif parity_inconsistency:
                        # inject_pi_error(disk_name, newDBN)
    sub inject_error{
        my $self =shift;
        my %opts = validate(@_, {filer_vol => {type => SCALAR}});

        my $filer_vol = $opts{filer_vol};
        my $aggr      = $self->get_aggregate(filer_vol => $filer_vol);
        my $error_info = $self->get_error_info(filer_vol => $filer_vol);
        my $api_set_obj = $self->{api_set_obj};

        my @plex_list = grep (/^plex(\d)$/i, keys %$error_info);
        foreach my $plex (@plex_list){
            my @rg_list = grep (/rg/i, keys %{$error_info->{$plex}});
            foreach my $rg (@rg_list){
                if(!exists $error_info->{$plex}->{$rg}->{zones}) { next ;}
                my @zones = @{$error_info->{$plex}->{$rg}->{zones}};
                my $raidlw = new RaidLW(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol, mount=>1, unmount=>0);
                foreach my $zoneid (0..$#zones) {
                    my $zone_info = $zones[$zoneid];
                    if (!exists $zone_info->{stripes}) { next ;}
                    my @errors = @{$zone_info->{stripes}};
                    foreach my $stripeid (0..$#errors){
                        my $stripe_info = $errors[$stripeid];
                        my $dbn = $stripe_info->{dbn};
                        my $cksum_dbn = $stripe_info->{cksum_dbn};
                        my $newDBN = $dbn;
                        my $disk_name;
                        
                        my @stripe_err_list = keys %{$stripe_info};
                        foreach my $stripe_err (grep(/(media|parity_inconsistency|zeroblock)/, @stripe_err_list) ){
                            if(exists $stripe_info->{$stripe_err}){

                                # traverse each of media, pi, zeroblock and array within it.
                                my @stripe_err_arr = @{$stripe_info->{$stripe_err}};
                                foreach my $errid (0..$#stripe_err_arr){
                                    if($stripe_info->{$stripe_err}->[$errid]->{inject_at} eq 'checksum_blk'){
                                        $newDBN = $cksum_dbn;
                                        $Log->comment("inject_error: Info: inject $stripe_err error requested on checksum_blk - $newDBN");
                                    }
                                    if($stripe_info->{$stripe_err}->[$errid]->{disk_name} ne ''){
                                        $disk_name = $stripe_info->{$stripe_err}->[$errid]->{disk_name};
                                    }
                                    my $disk_type = $stripe_info->{$stripe_err}->[$errid]->{disk_type};
                                    $Log->comment("inject_error: Info: inject $stripe_err error. disk_name - $disk_name, disk_type - $disk_type, ".
                                            "dbn - $dbn, cksum_dbn - $cksum_dbn, injected dbn - $newDBN");

                                    #set fiji repeat count for media errors if defined so.
                                    my $fiji_repeat_count = $stripe_info->{$stripe_err}->[$errid]->{fiji_repeat_count};
                                    $raidlw->set(key => "fiji_repeat_count", value => $fiji_repeat_count)
                                        if ($stripe_err =~ /media/i && defined($fiji_repeat_count));
                                    $raidlw->set(key => "disk_name", value => $disk_name);
                                    $raidlw->set(key => "dbn",       value => $newDBN);
                                    my $err_method = "inject_".$stripe_err."_error";

                                    no strict 'refs';
                                    if($stripe_err eq 'media') {
                                        my $inject_method = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err}->[$errid]->{inject_method};
                                        $Log->comment("inject_error: Info: Media Error Injection Method: $inject_method");
                                        $raidlw->$err_method(inject_method => $inject_method);
                                    } else {
                                        $raidlw->$err_method();
                                    }
                                    use strict 'refs';
                                }
                            }
                        }

                        if(exists $stripe_info->{checksum}){
                            my @stripe_err_arr = @{$stripe_info->{checksum}};
                            foreach my $errid (0..$#stripe_err_arr){
                                # traverse every checksum entry in that stripe
                                my $disk_block_type = "data";
                                if($stripe_info->{checksum}->[$errid]->{inject_at} eq 'checksum_blk'){
                                    $newDBN = $cksum_dbn;
                                    $disk_block_type = "checksum";
                                    $Log->comment("inject_error: Info: inject checksum error requested on checksum_blk - $newDBN");
                                }
                                if ($stripe_info->{checksum}->[$errid]->{disk_name} ne ''){
                                    $disk_name = $stripe_info->{checksum}->[$errid]->{disk_name};
                                }
                                my $disk_type = $stripe_info->{checksum}->[$errid]->{disk_type};
                                $Log->comment("inject_error: Info: inject checksum error. disk_name - $disk_name, disk_type - $disk_type, ".
                                        "dbn - $dbn, cksum_dbn - $cksum_dbn, injected dbn - $newDBN");
                                $raidlw->set(key=>"disk_name",      value=>$disk_name);
                                $raidlw->set(key=>"dbn",            value=>$newDBN);
                                $raidlw->set(key=>"disk_type",      value=>$disk_type);
                                $raidlw->set(key=>"disk_block_type",value=>$disk_block_type);
                                my $data_disk_rgdn;

                                # To inject gencount error on parity/dparity disks need to have data disk_rgdn value
                                if ($disk_type =~ /parity/i) {
                                    my $inject_hash = $stripe_info->{checksum}->[$errid];
                                    if (exists($inject_hash->{gen_count})) {
                                        # Check for data disk_rgdn value if provided
                                        if (!exists($inject_hash->{disk_rgdn})){
                                            my $fileCreate = $self->get_aggregate(filer_vol => $filer_vol)->get_fileCreate();
                                            my $file_info = $fileCreate->get_file_info();
                                            my $rg_name = "/$filer_vol/$plex/$rg";
                                            my $data_disk = $file_info->{$filer_vol}->{$rg_name}->{dbn}->{$newDBN}->{disk_name};
                                            my $aggr_info_disk_rgdn_mapping = $aggr->get_aggr_info_disk_rgdn_mapping(plex_name=>"/$filer_vol/$plex", rg_name=>$rg_name);
                                            my $data_disk_rgdn = $aggr_info_disk_rgdn_mapping->{$data_disk};
                                            $inject_hash->{disk_rgdn} = $data_disk_rgdn;
                                            $Log->comment("inject_error: Info: inject checksum error. disk_name - $disk_name, disk_type - $disk_type, ".
                                                            "dbn - $dbn, cksum_dbn - $cksum_dbn, injected dbn - $newDBN, data_disk - $data_disk, data_disk_rgdn - $data_disk_rgdn");
                                        }
                                    }
                                }
                                $raidlw->inject_checksum_error(inject_hash=>$stripe_info->{checksum}->[$errid]);
                            }
                        }
                    }
                }
            }
        }
    }

    sub save_ems_marker {
        my $self = shift;
        my (%opts) = validate(@_, {filer_vol => {type=>SCALAR}});
        my $aggr = $self->get_aggregate(filer_vol => $opts{filer_vol});

        $Log->comment("set_ems_marker: Info: saving ems marker timestamp...");
        $aggr->set_ems_marker(ems_marker=>RaidSqa_ems_log_get_mark(api_set_obj => $self->{api_set_obj}));
        $Log->comment("set_ems_marker: Info: saved ems marker for aggr $opts{filer_vol} - ".$aggr->get_ems_marker());
    }

    sub populate_error_info_disks {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol => {type => SCALAR},
                    });

        my $tc_data = $self->get_tc_data();
        my $filer_vol = $opts{filer_vol};
        my $error_info = $self->get_error_info(filer_vol => $filer_vol);
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);

        my @plex_list = grep (/^plex(\d)$/i, keys %$error_info);
        # my $plex_count = scalar(@plex_list);

        foreach my $plex (@plex_list){
            my @rg_list = grep (/rg/i, keys %{$error_info->{$plex}});
            foreach my $rg (@rg_list){
                $Log->comment("populate_error_info_disks: Info: Disks to fail on plex - $plex, rg - $rg are:\n".
                                Dumper($self->get_disks_to_fail(filer_vol => $filer_vol, plex=>$plex ,rg=>$rg)));

                #=======================================================
                # For every plex/rg in tc_data, update aggr_info for disks that are getting failed/offlined
                # also update tc_data for disk_name entries of disk_fail/disk_offline tags
                my @err_list = grep(/^\s*(disk_fail|disk_offline)\s*$/i, keys %{$error_info->{$plex}->{$rg}});
                foreach my $err_type (@err_list){
                    my @errors = @{$error_info->{$plex}->{$rg}->{$err_type}};
                    foreach my $errid (0..$#errors){
                        my $disk_type = ${$errors[$errid]}{disk_type} ;
                        my $fail_type = ${$errors[$errid]}{type} if($err_type =~ /fail/);
                        my $disksref = $aggr->get_aggr_info_disks(plex=>$plex, rg=>$rg, disk_type=>$disk_type);
                        foreach my $disk (@{$disksref}){
                            my $fail_disk_list = $aggr->get_aggr_info_disks(plex=>$plex, rg=>$rg, disk_type=>'fail');
                            my $prefail_disk_list = $aggr->get_aggr_info_disks(plex=>$plex, rg=>$rg, disk_type=>'fail');
                            my $offline_disk_list = $aggr->get_aggr_info_disks(plex=>$plex, rg=>$rg, disk_type=>'offline');

                            my @arr = ();
                            @arr = @{$fail_disk_list} if defined($fail_disk_list);
                            @arr = (@arr, @{$prefail_disk_list}) if defined($prefail_disk_list);
                            @arr = (@arr, @{$offline_disk_list}) if defined($offline_disk_list);
                            if(!grep/$disk/, @arr){
                                my $disk_err_type;
                                if(defined($fail_type) && $fail_type =~ /prefail/) {
                                    $disk_err_type = "prefail" ;
                                } elsif($err_type =~ /disk_(fail|offline)/) {
                                    $disk_err_type = $1;
                                }
                                $aggr->add_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>$disk_err_type, disks=>[$disk]);
                                # TODO (FIXED): update tc_data for disk_name as well
                                $error_info->{$plex}->{$rg}->{$err_type}->[$errid]->{disk_name} = $disk;
                                last;
                            }
                        }
                    }
                }

                my $otherdisksref = $aggr->get_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>'all');
                my $fail_disk_list = $aggr->get_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>'fail');
                my $prefail_disk_list = $aggr->get_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>'prefail');
                my $offline_disk_list = $aggr->get_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>'offline');

                my @arr = ();
                @arr = @{$fail_disk_list} if defined($fail_disk_list);
                @arr = (@arr, @{$prefail_disk_list}) if defined($prefail_disk_list);
                @arr = (@arr, @{$offline_disk_list}) if defined($offline_disk_list);
                $aggr->remove_aggr_info_disks(plex=>$plex, rg=>$rg, disk_type=>'other', disks=>\@arr);

                #=======================================================
                # For every plex/rg in tc_data, update tc_data for disks that are getting error injected on stripes
                if(ref($error_info->{$plex}->{$rg}->{zones}) =~ /ARRAY/i) {
                    my @zone_list = @{$error_info->{$plex}->{$rg}->{zones}};
                    foreach my $zoneid (0..$#zone_list) {
                        if(!defined($zone_list[$zoneid]->{stripes})) {next;}
                        my @stripe_list = @{$zone_list[$zoneid]->{stripes}};
                        foreach my $stripeid (0..$#stripe_list) {
                            my @other_disks = @{$aggr->get_aggr_info_disks(plex=>$plex, rg=>$rg, disk_type=>'other')};
                            my @err_type_list = grep(/(checksum|media|parity_inconsistency|zeroblock)/, keys %{$stripe_list[$stripeid]});
                            my $arr_index = 0;
                            foreach my $err_type (@err_type_list) {
                                my @errors = @{$stripe_list[$stripeid]->{$err_type}};    #all errors of type $err_type, e.g media
                                foreach my $errorid (0..$#errors) {
                                    my @disk_list = @{$aggr->get_aggr_info_disks(plex=>$plex, rg=>$rg, disk_type=>$errors[$errorid]->{disk_type})};
                                    my %disk_hash = map {$_, 1} @other_disks;
                                    @disk_list = grep (defined($disk_hash{$_}), @disk_list);
                                    my $disk_id = (defined($errors[$errorid]->{disk_id})) ? $errors[$errorid]->{disk_id} : $arr_index++;
                                    my $array_index = ($errors[$errorid]->{disk_type} eq 'DATA') ?  $disk_id : 0;
                                    $errors[$errorid]->{disk_name} = $disk_list[$array_index];
                                }
                                $stripe_list[$stripeid]->{$err_type} = \@errors;
                            }
                        }
                        $zone_list[$zoneid]->{stripes} = \@stripe_list;
                    }
                    $error_info->{$plex}->{$rg}->{zones} = \@zone_list;
                }
                #=======================================================
            } #End, each rg
        } #End, each plex

        $Log->comment("populate_error_info_disks: Debug: Dumper error_info ". Dumper($error_info));
        $self->set_error_info(error_info=>$error_info, filer_vol=>$filer_vol);
    }

    sub is_start_from_scratch {
        my $self = shift;
        my (%opts) = validate(@_, {
                                filer_vol => {type=>SCALAR},
                                raidio_type => {type=>SCALAR},
                                set_next => {type=>SCALAR, default=>0, optional=>1},
                            });
        my $filer_vol = $opts{filer_vol};
        my $raidio_type = $opts{raidio_type};
        my $set_next = $opts{set_next};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);

        if($set_next == 0 and $self->{is_start_from_scratch} == 1){
            return 1;
        }

        # If type eq "unrecover" start aggr from scratch
        if($self->get_error_type() eq "unrecover" ){
            $Log->comment("is_start_from_scratch: Info : starting test from scratch as error type is \"unrecover\"");
            $self->{is_start_from_scratch} = 1;
            return 1;
        }

        # If raidio_type eq "reconsio" or "resyncio" start aggr from scratch
        if($set_next == 1 and $raidio_type =~ /(reconsio|resyncio)/i){
            $Log->comment("is_start_from_scratch: Info : starting test from scratch as raidio type is \"$raidio_type\"");
            $self->{is_start_from_scratch}  = 1;  # TODO: Add methods set_start_from_scratch() and get_start_from_scratch()
            return 1;
        }

        #If disk_fail/disk_offline or media type error is injected, then start from scratch
        my $error_info = $self->get_error_info(filer_vol => $filer_vol);   # TODO: Rename this to $error_info from method get_error_info()
        my @plex_list = grep (/^plex(\d)$/i, keys %$error_info);
        foreach my $plex (@plex_list){
            my @rg_list = grep (/rg/i, keys %{$error_info->{$plex}});
            foreach my $rg (@rg_list){
                my $disk_fail_err = $self->has_error_type(filer_vol => $filer_vol, plex=>$plex, rg=>$rg, error_type=>"disk_fail");
                my $disk_offline_err = $self->has_error_type(filer_vol => $filer_vol, plex=>$plex, rg=>$rg, error_type=>"disk_offline");
                if($disk_fail_err or $disk_offline_err){
                    $Log->comment("is_start_from_scratch: Info : starting test from scratch as current test had \"disk_fail\" or \"disk_offline\".");
                    $self->{is_start_from_scratch} = 1;
                    return 1;
                }

                $Log->comment("is_start_from_scratch: Info: plex - $plex, rg - $rg, error_info -".Dumper($error_info));
                my @zone_list = @{$error_info->{$plex}->{$rg}->{zones}} if(ref($error_info->{$plex}->{$rg}->{zones}) =~ /ARRAY/i);
                foreach my $zone (@zone_list) {
                    my @stripes = @{$zone->{stripes}};
                    foreach my $stripe (@stripes) {
                        if(grep(/(media)/, keys %$stripe)){
                            $Log->comment("is_start_from_scratch: Info : starting test from scratch as current test had \"media\" error injected");
                            $self->{is_start_from_scratch} = 1;
                            return 1;
                        }

                        #If error injection is on checksum block, then start from scratch
                        if('advanced_zoned' eq $aggr->get_checksum_type()) {
                            my $err_match = "(checksum|parity_inconsistency|zeroblock)";
                            my @err_types = grep(/$err_match/, keys %$stripe);
                            my ($err_on_data_blk, $err_on_checksum_blk) = (0, 0);
                            foreach my $err_type (@err_types) {
                                my @errors = @{$stripe->{$err_type}};
                                foreach my $error (@errors) {
                                    if($error->{inject_at} eq "checksum_blk") {$err_on_checksum_blk = 1};
                                    if($error->{inject_at} eq "data_blk") {$err_on_data_blk = 1};
                                }
                                if($err_on_checksum_blk == 1 && $err_on_data_blk == 0) {
                                    $Log->comment("is_start_from_scratch: Info : starting test from scratch as error injection occurs on checksum block");
                                    $self->{is_start_from_scratch} = 1;
                                    return 1;
                                }
                            }
                        }
                    }
                }
            }
        }

        $self->{is_start_from_scratch} = 0;
        return 0;
    }

    sub get_tc_data{
        my $self = shift;

        return $self->{tc_data};
    }

    sub set_tc_data {
        my $self =shift;
        my (%opts) = validate(@_, {
                            tc_data => {type=>HASHREF|UNDEF, default=>undef, optional=>1},
                        });

        $self->{tc_data} = $opts{tc_data};
    }

    sub get_injected_err_count {
        my $self =shift;
        my %opts = validate(@_, {
                        filer_vol => {type=>SCALAR},
                        rg_name=>{type=>SCALAR},
                        error=>{type=>SCALAR}
                    });

        my $filer_vol = $opts{filer_vol};
        my $rg_name = $opts{rg_name};
        my $error = $opts{error};
        my $error_info = $self->get_error_info(filer_vol => $filer_vol);

        my %err_map_hash = ("pi" => "parity_inconsistency",  "media_err" => "media",);

        $rg_name =~ /\/$filer_vol\/(plex\d)\/(rg\d+)/;
        my $plex = $1;
        my $rg = $2;
        $Log->comment("get_injected_err_count: Debug: rg_name - $rg_name, plex - $plex, rg - $rg, error - $error");

        my (@zones, @errors);
        if(ref($error_info->{$plex}->{$rg}->{zones}) =~ /ARRAY/i){
            @zones = @{$error_info->{$plex}->{$rg}->{zones}};
        }

        my $err_count = 0;
        foreach my $zoneid(0..$#zones) {
            if(exists $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes})  {
                @errors = @{$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}};
            }
            foreach my $stripeid (0..$#errors){
                $Log->comment("get_injected_err_count: Debug: rg_name - $rg_name, plex - $plex, rg - $rg, error - $error, stripeid - $stripeid");
                if(exists $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$err_map_hash{$error}}) {
                    my @stripe_err_ids = @{$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$err_map_hash{$error}}};
                    $err_count += scalar(@stripe_err_ids);
                }
            }
        }
        $Log->comment("get_injected_err_count: Debug: Total error injected on rg - $rg_name, error_type - $error is $err_count");
        return $err_count;
    }

    sub get_inject_at_array {
        my $self =shift;
        my %opts = validate(@_, {
                        filer_vol => {type=>SCALAR},
                        rg_name=>{type=>SCALAR},
                        error=>{type=>SCALAR}
                    });

        my $filer_vol = $opts{filer_vol};
        my $rg_name = $opts{rg_name};
        my $error = $opts{error};
        my $error_info = $self->get_error_info(filer_vol => $filer_vol);

        my %err_map_hash = ("pi" => "parity_inconsistency",  "media_err" => "media",);

        $rg_name =~ /\/$filer_vol\/(plex\d)\/(rg\d+)/;
        my $plex = $1;
        my $rg = $2;
        $Log->comment("get_injected_err_count: Debug: rg_name - $rg_name, plex - $plex, rg - $rg, error - $error");

        my (@zones, @errors, @inject_at_values);
        if(exists $error_info->{$plex}->{$rg}->{zones}){
            @zones = @{$error_info->{$plex}->{$rg}->{zones}};
        }

        foreach my $zoneid(0..$#zones) {
            if(exists $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes})  {
                @errors = @{$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}};
            }
            foreach my $stripeid (0..$#errors){
                $Log->comment("get_injected_err_count: Debug: rg_name - $rg_name, plex - $plex, rg - $rg, error - $error, stripeid - $stripeid");
                if(exists $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$err_map_hash{$error}}) {
                    foreach my $error (@{$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$err_map_hash{$error}}}) {
                        push(@inject_at_values, $error->{inject_at});
                    }
                }
            }
        }
        $Log->comment("get_injected_err_count: Debug: Inject at values for $error - @inject_at_values");
        return @inject_at_values;
    }

    sub compare_validation_data {
        my $self =shift;
        my %opts = validate(@_, {
                        filer_vol =>{type=>SCALAR},
                        compare_stripe_checksum =>{type=>SCALAR, default=>1, optional=>1},
                    });

        my $filer_vol = $opts{filer_vol};
        my $api_set_obj = $self->{api_set_obj};
        my $type = $self->get_error_type();

        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $file = $aggr->get_fileCreate();

        my $tc_data = $self->get_tc_data();

        # Validation-1: Compare rg list
        my @rg_list = sort( $file->get_rg_list());

        #===========================================================================================
        my $raidlw = new RaidLW(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol, mount=>1, unmount=>0);
        # Validation-2: Compare checksum stripes info
        foreach my $id (0..$#rg_list){
            my @dbn_list = $file->get_dbn_list(rg_name => $rg_list[$id]);
            if(!scalar(@dbn_list)){
                $Log->comment("compare_validation_data: Info: Expected dbn field not found for rg - $rg_list[$id] from file_info: @dbn_list");
                next;
            }

            if($opts{compare_stripe_checksum} == 1) {
                foreach my $dbn_number (@dbn_list){
                    my $stripe_checksum_before = $file->get_stripe_checksum(rg_name=>$rg_list[$id], dbn=>$dbn_number, context=>'before');
                    my $stripe_checksum_after  = $file->get_stripe_checksum(rg_name=>$rg_list[$id], dbn=>$dbn_number, context=>'after');
                    if(!defined $stripe_checksum_before){
                        die("compare_validation_data: Error: stripe_checksum field not found for rg - $rg_list[$id] from info before error inject");
                    }

                    if(!defined $stripe_checksum_after){
                        die("compare_validation_data: Error: stripe_checksum field not found for rg - $rg_list[$id] from info after error inject");
                    }

                    $Log->comment("compare_validation_data: Info: comparing checksum stripe on rg - $rg_list[$id], dbn - $dbn_number ");
                    my $msg_match       = "compare_validation_data: Info: Checksum on stripes before error inject matched with that of after error recovery";
                    my $msg_mismatch    = "compare_validation_data: Info: Checksum on stripes before error inject did not match with that of after error recovery";
                    my $compare_result  = $file->compare_stripe_checksum(
                                            before_error_inject => $stripe_checksum_before,
                                            after_error_inject  => $stripe_checksum_after,
                                            error_type          => $type
                                        );

                    if($type eq "recover"){
                        if($compare_result)  {$Log->comment($msg_match);}
                        if(!$compare_result) {die($msg_mismatch);}
                    } elsif($type eq "unrecover") {
                        if(!$compare_result)  {$Log->comment($msg_mismatch);}
                        if($compare_result && ($aggr->get_ignore_lw_validation() == 0)) {die($msg_match);}
                    }
                }
            }
        }

        #===========================================================================================
        # Validation-3: Compare md5sum and other individual file related fields
        my $skip_md5sum = 0;
        if(!$self->has_parity_inconsistency(filer_vol => $filer_vol)) {
            my $ems_by_event = RaidSqa_ems_log_dump_by_type(api_set_obj => $self->{api_set_obj}, timestamp => $aggr->get_ems_marker());
            $skip_md5sum = 1 if(defined($ems_by_event->{'raid.rg.scrub.rewrite.parity'}));
        }

        if($skip_md5sum == 0) {
            foreach my $id (0..$#rg_list){

                #compare file names
                my $filename = $file->get_test_file_name(rg_name => $rg_list[$id]);

                #compare md5sum
                my $md5sum_before = $file->get_md5sum(rg_name => $rg_list[$id], context=>'before');
                my $md5sum_after = $file->get_md5sum(rg_name => $rg_list[$id], context=>'after');

                $Log->comment("compare_validation_data: Info: Test is of type - $type");
                $Log->comment("compare_validation_data: Info: md5sum before - $md5sum_before, md5sum after - $md5sum_after");
                if($md5sum_before ne $md5sum_after){
                    my $msgString ="md5sum doesn't match before and after.  Before error inject, file_name - $filename, md5sum - $md5sum_before,".
                        " obtained: file_name - $filename, md5sum - $md5sum_after";
                    if($type eq "recover"){
                        die("compare_validation_data: Error: ".$msgString);
                    } else {
                        $Log->comment("compare_validation_data: Info: ".$msgString);
                    }
                } else {
                    if($type eq "recover"){
                        $Log->comment("compare_validation_data: Info: md5sum matched before and after.  file_name - $filename");
                    } else {
                        #Note: For RLW multierror tests, data/parity block will be rewritten, so for unrecover there might not be md5sum mismatch
                        $Log->warn("compare_validation_data: Error: md5sum matched before and after while not expected for type - $type.".
                            " file_name - $filename");
                    }
                }
            }
        }

        # Validation-4: Compare pi_files_list
        my $new_pi_files = $file->is_new_pi_files_created();

        # TODO:   If parity_inconsistency is injected then $new_pi_files > 0 should pass, else it should fail TC.
        if(ref($new_pi_files) && scalar(@{$new_pi_files}) ){
            $Log->warn("compare_validation_data: Error: New PI files are created.  PI files - @{$new_pi_files}");
        }
    }

    #=================================== Methods to manipulate zone/stripe/error related info  ===================================
    sub get_error_info {
        my $self = shift;
        my (%opts) = validate(@_, {
                        filer_vol => {type=>SCALAR},
                    });

        $self->{tc_data}->{error_info}->{$opts{filer_vol}};
    }

    sub set_error_info {
        my $self = shift;
        my (%opts) = validate(@_, {
                        filer_vol       => {type=>SCALAR},
                        error_info => {type=>HASHREF},
                    });

        my $tc_data = $self->get_tc_data();
        $tc_data->{error_info}->{$opts{filer_vol}} = $opts{error_info};
        $self->set_tc_data(tc_data => $tc_data);
    }

    sub delete_error_info {
        my $self = shift;
        my (%opts) = validate(@_, {
                        filer_vol       => {type=>SCALAR},
                    });

        my $tc_data = $self->get_tc_data();
        delete $tc_data->{error_info}->{$opts{filer_vol}};
        $self->set_tc_data(tc_data => $tc_data);
    }

    sub get_error_info_zone_count {
        my $self = shift;
        my (%opts) = validate(@_, {
                        filer_vol => {type=>SCALAR},
                        plex => {type=>SCALAR},
                        rg => {type=>SCALAR},
                    });
        my $filer_vol = $opts{filer_vol};
        my $plex = $opts{plex};
        my $rg = $opts{rg};

        my $error_info = $self->get_error_info(filer_vol => $filer_vol);
        if(exists($error_info->{$plex}->{$rg}->{zones})){
            return scalar( @{$error_info->{$plex}->{$rg}->{zones}});
        }else{
            return 0;
        }
    }

    sub get_error_info_stripe_count {
        my $self = shift;
        my (%opts) = validate(@_, {
                        zoneid  => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                        filer_vol => {type=>SCALAR},
                        plex => {type=>SCALAR},
                        rg => {type=>SCALAR},
                    });
        my $filer_vol = $opts{filer_vol};
        my $zoneid = $opts{zoneid};
        my $plex = $opts{plex};
        my $rg = $opts{rg};
        my $stripe_count = 0;

        if(!defined($opts{zoneid})){
            my $zone_count = $self->get_error_info_zone_count(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg);
            foreach my $zoneid (0..$zone_count-1){
                $stripe_count += $self->get_error_info_stripe_count(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, zoneid=>$zoneid);
            }
        } else {
            my $error_info = $self->get_error_info(filer_vol => $filer_vol);
            if(exists($error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes})){
                 $stripe_count = scalar( @{$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}});
            }
        }
        return $stripe_count;
    }

    sub get_error_info_stripes{
        my $self = shift;
        my (%opts) = validate(@_, {
            filer_vol  => {type=>SCALAR},
            plex       => {type=>SCALAR},
            rg         => {type=>SCALAR},
            zone_index => {type=>SCALAR, default => 0, optional => 1},
        });

        my $filer_vol = $opts{filer_vol};
        my $plex = $opts{plex};
        my $rg = $opts{rg};

        my $error_info = $self->get_error_info(filer_vol => $filer_vol);
        if(exists($error_info->{$plex}->{$rg}->{zones}->[$opts{zone_index}]->{stripes})) {
            return $error_info->{$plex}->{$rg}->{zones}->[$opts{zone_index}]->{stripes};
        }
        return {};
    }

    sub is_stripe_same_plex_dbn {
        my $self = shift;
        my (%opts) = validate(@_, {
                            filer_vol => {type=>SCALAR},
                            rg        => {type=>SCALAR},
                            zoneid    => {type=>SCALAR},
                            stripeid  => {type=>SCALAR},
                        });
        my $filer_vol = $opts{'filer_vol'};
        my $rg = $opts{'rg'};
        my $zoneid = $opts{'zoneid'};
        my $stripeid = $opts{'stripeid'};
        my $error_info = $self->get_error_info(filer_vol=>$filer_vol);
        my @plex_list = sort grep /^plex\d+$/, keys %$error_info;

        my $stripe_same_plex_dbn = 0;
        foreach my $plex (@plex_list) {
            if(defined $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{stripe_same_plex_dbn}) {
                $stripe_same_plex_dbn += $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{stripe_same_plex_dbn};
            }
        }
        return ($stripe_same_plex_dbn > 0) ? 1 : 0;
    }

    sub is_zone_same_plex_dbn {
        my $self = shift;
        my (%opts) = validate(@_, {
                            filer_vol => {type=>SCALAR},
                            rg        => {type=>SCALAR},
                            zoneid    => {type=>SCALAR},
                        });
        my $filer_vol = $opts{'filer_vol'};
        my $rg = $opts{'rg'};
        my $zoneid = $opts{'zoneid'};
        my $error_info = $self->get_error_info(filer_vol=>$filer_vol);
        my @plex_list = sort grep /^plex\d+$/, keys %$error_info;

        my $zone_same_plex_dbn = 0;
        foreach my $plex (@plex_list) {
            if(defined $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{zone_same_plex_dbn}) {
                $zone_same_plex_dbn += $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{zone_same_plex_dbn};
            }
        }
        return ($zone_same_plex_dbn > 0) ? 1 : 0;
    }

    sub get_max_zone_count {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => {type=>SCALAR}});
        my $filer_vol = $opts{filer_vol};
        my $error_info = $self->get_error_info(filer_vol => $filer_vol);
        my @plex_list = grep (/plex/, keys %$error_info);
        my @zone_counts = ();
        foreach my $plex (@plex_list) {
            my @rg_list = grep (/rg\d+/, keys %{$error_info->{$plex}});
            foreach my $rg (@rg_list) {
                push (@zone_counts, scalar(@{$error_info->{$plex}->{$rg}->{zones}}))
                    if(exists($error_info->{$plex}->{$rg}->{zones}));
            }
        }
        $Log->comment("get_max_zone_count: Debug: zone_counts for each rg: @zone_counts");
        @zone_counts = sort {$b <=> $a} @zone_counts;
        return $zone_counts[0];
    }

    sub get_disks_to_fail {
        my $self = shift;
        my (%opts) = validate(@_, {
                        filer_vol => {type=>SCALAR},
                        plex      => {type=>SCALAR},
                        rg        => {type=>SCALAR},
                    });

        my $filer_vol = $opts{filer_vol};
        my $plex = $opts{plex};
        my $rg = $opts{rg};
        my $error_info = $self->get_error_info(filer_vol => $filer_vol);

        my @err_list = grep(/^\s*(disk_fail|disk_offline)\s*$/i, keys %{$error_info->{$plex}->{$rg}});

        my %disks_hash = ();
        foreach my $err_type (@err_list){
            my @errors = @{$error_info->{$plex}->{$rg}->{$err_type}};
            foreach my $errid (0..$#errors){
                if(exists ${$errors[$errid]}{disk_type}){
                    my $disk_type = ${$errors[$errid]}{disk_type} ;
                    if($disk_type eq "DATA" or $disk_type eq "PARITY" or $disk_type eq "DPARITY"){
                        $disks_hash{$err_type}->{$disk_type} = !exists $disks_hash{$err_type}->{$disk_type} ? 1 : $disks_hash{$err_type}->{$disk_type}+1;
                    }
                }
            }
        }
        return \%disks_hash;
    }

    sub set_stripe_dbn {
        my $self = shift;
        my (%opts) = validate(@_, {
            filer_vol => {type=>SCALAR},
            plex      => {type=>SCALAR},
            rg        => {type=>SCALAR},
            zoneid    => {type=>SCALAR},
            stripeid  => {type=>SCALAR},
            dbn       => {type=>SCALAR},
            dbn_data  => {type=>HASHREF},
        });

        my $filer_vol = $opts{filer_vol};
        my $plex      = $opts{plex};
        my $rg        = $opts{rg};
        my $stripeid  = $opts{stripeid};
        my $zoneid    = $opts{zoneid};
        my $dbn       = $opts{dbn};
        my $dbn_data  = $opts{dbn_data};

        my $error_info = $self->get_error_info(filer_vol => $filer_vol);
        $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{dbn} = $dbn;
        $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{cksum_dbn} = $dbn_data->{$dbn}->{cksum_dbn};
        $self->set_error_info(filer_vol => $filer_vol, error_info=>\%{$error_info});
        $self->set_ems_data(%opts);
    }

    sub set_zone_dbn {
        my $self = shift;
        my (%opts) = validate(@_, {
                        filer_vol => {type=>SCALAR},
                        plex      => {type=>SCALAR},
                        rg        => {type=>SCALAR},
                        zoneid    => {type=>SCALAR},
                        dbn_data  => {type=>HASHREF},
                        zone_cksum_dbn  => {type=>ARRAYREF},
                    });

        my $filer_vol = $opts{filer_vol};
        my $plex      = $opts{plex};
        my $rg        = $opts{rg};
        my $zoneid    = $opts{zoneid};
        my $zone_cksum_dbn = $opts{zone_cksum_dbn};
        my $dbn_data  = $opts{dbn_data};
        my $cksum_dbn = $dbn_data->{$zone_cksum_dbn->[0]}->{cksum_dbn};

        my $error_info = $self->get_error_info(filer_vol => $filer_vol);
        my $zone = $error_info->{$plex}->{$rg}->{zones}->[$zoneid];
        foreach my $stripeid (0..scalar(@{$zone->{stripes}})-1) {
            my $stripe = $zone->{stripes}->[$stripeid];
            my $stripe_offset = (defined $zone->{stripes}->[$stripeid]->{stripe_offset}) ? $zone->{stripes}->[$stripeid]->{stripe_offset} : $stripeid;
            my $compute_dbn = $cksum_dbn - 32 + $stripe_offset;
            $Log->comment("set_zone_dbn: Error: zone_cksum_dbn array - computed_dbn - $compute_dbn, zone_cksum_dbn - @$zone_cksum_dbn");
            if(!grep /$compute_dbn/, @$zone_cksum_dbn){
                die("set_zone_dbn: Error: Invalid zone_cksum_dbn array - computed_dbn - $compute_dbn, zone_cksum_dbn - @$zone_cksum_dbn");
            }
            $self->set_stripe_dbn(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, zoneid=>$zoneid, stripeid=>$stripeid, dbn=>$compute_dbn, dbn_data=>$dbn_data);
        }
    }

    sub set_ems_data {
        my $self = shift;
        my (%opts) = validate(@_, {
                        filer_vol  => {type=>SCALAR},
                        plex       => {type=>SCALAR},
                        rg         => {type=>SCALAR},
                        stripeid   => {type=>SCALAR},
                        zoneid     => {type=>SCALAR},
                        dbn        => {type=>SCALAR},
                        dbn_data   => {type=>HASHREF},
                    });
        my $filer_vol    = $opts{filer_vol};
        my $plex         = $opts{plex};
        my $rg           = $opts{rg};
        my $stripe_index = $opts{stripeid};
        my $zone_index   = $opts{zoneid};
        my $dbn          = $opts{dbn};
        my $cksum_dbn    = $opts{dbn_data}->{$dbn}->{cksum_dbn};

        my $error_info = $self->get_error_info(filer_vol => $filer_vol);
        my @err_types = grep (/(media|checksum|parity_inconsistancy|zeroblock)/, keys %{$error_info->{$plex}->{$rg}->{zones}->[$zone_index]->{stripes}->[$stripe_index]});
        foreach my $error_type (@err_types) {
            my @errors = @{$error_info->{$plex}->{$rg}->{zones}->[$zone_index]->{stripes}->[$stripe_index]->{$error_type}};
            foreach my $errorid (0..$#errors) {
                my $ems = $errors[$errorid]->{ems};
                my $copy_ems = {};      #To avoid deep hash copy problem
                if(defined($ems)) {
                    foreach my $event (keys %$ems) {
                        foreach my $ems_key (keys %{$ems->{$event}}) {
                            #Update values based on keywords
                            if($ems_key =~ /raidio/i) {next;}
                            my $ems_value = $ems->{$event}->{$ems_key};
                            if($ems_value !~ /^(DBN|CKSUM_DBN|CKSUM_DBN\+|RG_NAME|FILER_VOL|FILEID)$/) {
                                die("set_ems_data: Error: Unknow value type : event - $event, key - $ems_key, value - $ems_value");
                            }

                            if($ems_value eq "DBN")  {$ems_value = $dbn;}
                            if($ems_value eq "CKSUM_DBN")  {$ems_value = $cksum_dbn;}
                            if($ems_value eq "CKSUM_DBN+")  {$ems_value = $cksum_dbn+1;}
                            if($ems_value eq "RG_NAME")  {$ems_value = "/$filer_vol/$plex/$rg";}
                            if($ems_value eq "FILER_VOL")  {$ems_value = $filer_vol;}

                            if($ems_value eq "FILEID")  {
                                my $aggr = $self->get_aggregate(filer_vol=>$filer_vol);
                                my $file = $aggr->get_fileCreate();
                                $ems_value = $file->get_test_fileid(rg_name => "/$filer_vol/$plex/$rg");
                            }
                            $copy_ems->{$event}->{$ems_key} = $ems_value;
                            $Log->comment("set_ems_data: Info: event - $event, key - $ems_key, value - $ems_value");
                        }
                    }
                }
                $error_info->{$plex}->{$rg}->{zones}->[$zone_index]->{stripes}->[$stripe_index]->{$error_type}->[$errorid]->{ems} = $copy_ems;
            }
        }

        $self->set_error_info(filer_vol => $filer_vol, error_info=>\%{$error_info});
        $Log->comment("set_ems_data: Debug:  Updated ems data: ".Dumper($error_info));
    }

    sub has_error_type {
        # This function checks error type in tc_data for a given plex and rg.
        my $self =shift;
        my (%opts) = validate(@_,{
                                filer_vol => {type=>SCALAR},
                                plex => {type=>SCALAR},
                                rg => {type=>SCALAR},
                                error_type => {type=>SCALAR},
                             });
        my $tc_data = $self->get_tc_data();
        my $filer_vol = $opts{filer_vol};
        my $plex = $opts{plex};
        my $rg = $opts{rg};
        my $error_type = $opts{error_type};
        my $error_info = $self->get_error_info(filer_vol => $filer_vol);

        my @err_list = grep(/^\s*($error_type)\s*$/i, keys %{$error_info->{$plex}->{$rg}});
        return (scalar(@err_list) > 0);
    }

    sub get_error_injected_disks {
        my $self = shift;
        my (%opts) = validate(@_,{
                                filer_vol => {type=>SCALAR},
                                plex => {type=>SCALAR},
                                rg => {type=>SCALAR},
                                error_type => {type=>SCALAR},
                                dbn => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                             });
        my %dbn_disk_hash = ();
        my @all_disk_list = ();
        my $error_info = $self->get_error_info(filer_vol => $opts{filer_vol});
        foreach my $zone (@{$error_info->{$opts{plex}}->{$opts{rg}}->{zones}}){
            foreach my $stripe (@{$zone->{stripes}}) {
                if(grep /$opts{error_type}/, keys %$stripe){
                    my $dbn = $stripe->{dbn};
                    $dbn_disk_hash{$dbn} = [];
                    foreach my $error (@{$stripe->{$opts{error_type}}}) {
                        push(@{$dbn_disk_hash{$dbn}}, $error->{disk_name});
                        push(@all_disk_list, $error->{disk_name});
                    }
                }
            }
        }

        if(exists($opts{dbn})){
            my @disk_list = (exists $dbn_disk_hash{$opts{dbn}}) ? @{$dbn_disk_hash{$opts{dbn}}} : ();
            return \@disk_list;
        } else {
            return  \@all_disk_list;
        }
    }

    sub get_stripe_gap {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol   => {type=>SCALAR},
                        rg          => {type=>SCALAR}
                    });

        my $filer_vol = $opts{filer_vol};
        my $rg = $opts{rg};
        my $error_info = $self->get_error_info(filer_vol=>$filer_vol);
        my @plex_list = sort grep (/plex/, keys %$error_info);

        foreach my $plex (@plex_list) {
            if( defined($error_info->{$plex}->{$rg}->{stripe_gap}) &&
                $error_info->{$plex}->{$rg}->{stripe_gap} =~ /\d+/) {
                return $error_info->{$plex}->{$rg}->{stripe_gap};
            }
        }
        return 0;
    }

    #In case of recover errors if only one PI is injected,
    #then scrub IO will not be able to determine which block is culprit
    #In such a case, it will rewrite parity causing error to be propogated

    #incase of mirror it should recover from other plex?

    sub has_parity_inconsistency {
        my $self = shift;
        my (%opts) = validate(@_,{
            filer_vol => {type=>SCALAR},
        });

        my $error_info = $self->get_error_info(filer_vol => $opts{filer_vol});
        my @plexes = keys %$error_info;

        foreach my $plex (@plexes) {
            my @rgs = keys %{$error_info->{$plex}};
            foreach my $rg (@rgs) {
                my @zones = @{$error_info->{$plex}->{$rg}->{zones}} if(exists($error_info->{$plex}->{$rg}->{zones}));
                foreach my $zone (@zones) {
                    my @stripes = @{$zone->{stripes}};
                    foreach my $stripe (@stripes) {
                        if(ref($stripe->{parity_inconsistency})) {
                            return 1 if(scalar(@{$stripe->{parity_inconsistency}}) > 0);
                        }
                    }
                }
            }
        }
    }

    #=================================== Methods related to raid IOs  ===================================
    sub run_raidio {
        my $self =shift;
        my %opts = validate(@_, {
                                raidio_type =>{type=>SCALAR},
                                filer_vol =>{type=>SCALAR},
                            });
        my $filer_vol = $opts{filer_vol};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $api_set_obj = $self->{api_set_obj};
        my $raidio_type = $opts{raidio_type};

        $self->flush_cache(filer_vol => $filer_vol, checksum_type => $aggr->get_checksum_type(), aggr_type => $aggr->get_aggr());
        my $raidio_method = "run_".$raidio_type;
        $Log->comment("run_raidio: Info: executing $raidio_type...");

        no strict 'refs';
        $self->$raidio_method(filer_vol => $filer_vol);
        use strict 'refs';
    }

    sub run_waflio {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => {type=>SCALAR}});
        my $filer_vol = $opts{filer_vol};
        my $api_set_obj = $self->{api_set_obj};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $file = $aggr->get_fileCreate();

        my $tc_data = $self->get_tc_data();
        my $type = $self->get_error_type();

        foreach my $rg_name ($file->get_rg_list()){
            my $file_name = $file->get_test_file_name(rg_name=>$rg_name);
            $Log->comment("run_waflio: Info: executing WAFL IO on file - $file_name");
            my $result = RaidSqa_file_nfs_read(api_set_obj=>$api_set_obj, file_name=>$file_name);
            if(!$result) {
                if($type eq "recover"){
                    die("run_waflio: Error: read error of file - $file_name.");
                } else {
                    $Log->warn("run_waflio: Error: read error of file - $file_name.");
                }
            }
        }
        $Log->comment("run_waflio: Info: completed WAFL IO on all files");
    }

    sub run_reparityio {
        my $self =shift;
        my %opts = validate(@_, {filer_vol => {type=>SCALAR}});
        my $filer_vol = $opts{filer_vol};

        my $api_set_obj = $self->{api_set_obj};
        my $aggr = $self->get_aggregate(filer_vol=>$filer_vol);
        my $file = $aggr->get_fileCreate();

        # Destroy the volume
        $Log->comment("run_reparityio: Info: executing reparity on aggregate - $filer_vol");
        $Log->comment("run_reparityio: Info: destroy vol/aggr - $filer_vol");
        $file->create_mkfile(count=>5);   # creating file of 50m x 5.  May be add more if required for reparity to be effective.

        #get rtid for aggregate
        $Log->comment("run_reparityio: Info: retrieve aggr raidtreeID - $filer_vol");
        my $aggr_rtid = RaidSqa_get_aggr_rtid(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
        $Log->comment("raidtreeID - $aggr_rtid");


        RaidSqa_aggr_remove(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr->get_aggr());
        if (RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr->get_aggr()) ) {
            die("run_reparityio: Error: aggr destroy failed - $filer_vol.");
        }

        #save ems marker
        $Log->comment("run_reparityio: Info: setting ems marker");
        my $mark_ems = RaidSqa_ems_log_get_mark(api_set_obj => $api_set_obj);

        # Undestroy the destroyed volume for reparity to start
        $Log->comment("run_reparityio: Info: undestroy vol/aggr - $filer_vol");
        if (!RaidSqa_aggr_undestroy(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr->get_aggr(), raidtree_id => $aggr_rtid)) {
            die("run_reparityio: Error: aggr undestroy failed - $filer_vol.");
        }

        # Check for reparity on vol/aggr FILER_VOL
        sleep(10);
        my @rg_list = $aggr->get_aggr_info_rgs();

        $Log->comment("run_reparityio: Info: check if reparity is running on groups - @rg_list");
        my $event_ems = RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj, timestamp => $mark_ems);
        my $ref_ids = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj,timestamp => $mark_ems);
        foreach my $rg_name (@rg_list) {
            if (!RaidSqa_check_group_reparity_in_progress(api_set_obj => $api_set_obj, group_name => $rg_name)){
                # Check ems events; should not be any event related to raid.rg.reparity.start
                $Log->comment("run_reparityio: Info: Checking ems event related to reparity start - raid.rg.reparity.start");
                my $ems_event_found = 0;
                my $event_ids = $event_ems->{'raid.rg.reparity.start'};
                if ($event_ids) {
                    foreach my $eventid (@$event_ids) {
                        my $rg = $ref_ids->{$eventid}->{rg};
                        if ( $rg eq $rg_name ) {
                            $Log->comment("run_reparityio: Info: Event - raid.rg.reparity.start found for rg - $rg");
                            $ems_event_found = 1;
                            last;
                        }
                    }
                } else {
                    die("run_reparityio: Error: reparity ems event \"raid.rg.reparity.start\" not found on group - $rg_name ");
                }
                if($ems_event_found == 0){
                    die("run_reparityio: Error: reparity is not running on group - $rg_name");
                }
            }
        }
        $Log->comment("run_reparityio: Info: Reparity was running on groups - @rg_list");
        RaidSqa_wait_for_reparity_completion(api_set_obj => $api_set_obj, filer_vol=>$filer_vol);  # Bhagwan barose
        $self->run_wafliron(filer_vol => $filer_vol);
        $Log->comment("run_reparityio: Info: completed reparityio on filer_vol - $filer_vol");
    }

    sub run_scrubio {
        my $self = shift;
        my (%opts) = validate(@_, {
                            filer_vol => {type=>SCALAR},
                            rg_name   => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                            validation_step => {type=>SCALAR, default=>0, optional=>1},
                        });

        my $filer_vol = $opts{filer_vol};
        my $api_set_obj = $self->{api_set_obj};
        my $rg_name = $opts{rg_name};
        my $validation_step = $opts{validation_step};

        if(defined $rg_name){
            $filer_vol = undef;
        }

        $Log->comment("run_scrubio: Info: Starting scrub on aggr - $filer_vol");
        my ( $output, $error ) = RaidSqa_aggr_scrub_start(api_set_obj => $api_set_obj, filer_vol => $filer_vol, group_name=>$rg_name);
        if ( !$error ) {
            if($validation_step){
                $Log->comment("run_scrubio: Info: starting scrub failed for aggr - $filer_vol, in validation step");
            } else {
                die("run_scrubio: Info: starting scrub failed for aggr - $filer_vol");
            }
        } else {
            $Log->comment("run_scrubio: Info: scrub started on aggr - $filer_vol");
            # Wait for scrub to complete.
            $Log->comment("run_scrubio: Info: waiting for scrub to complete on aggr - $filer_vol");
            RaidSqa_wait_for_scrub_completion(api_set_obj => $api_set_obj, filer_vol   => $filer_vol, group_name=>$rg_name);
            $Log->comment("run_scrubio: Info: completed scrubio on filer_vol - $filer_vol");
        }
    }

    sub run_wafliron {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => {type=>SCALAR}});
        my $filer_vol = $opts{filer_vol};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $api_set_obj = $self->{api_set_obj};

        RaidSqa_aggr_wafliron_start(api_set_obj=>$api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr->get_aggr());
        my $count = 0;
        while (RaidSqa_aggr_wafliron_status(api_set_obj=>$api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr->get_aggr())) {
            $Log->comment("run_wafliron: Info: Waiting for wafliron to complete. looping - $count...");
            $count += 1;
            sleep 30;
        }
    }

    sub run_verifyio {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => {type=>SCALAR}});
        my $filer_vol = $opts{filer_vol};
        my $api_set_obj = $self->{api_set_obj};

        # Get plex count to make sure it is mirrored volume
        $Log->comment("run_verifyio: Info: Starting verify on aggr - $filer_vol");
        my $error = RaidSqa_aggr_verify_start(api_set_obj => $api_set_obj,filer_vol=>$filer_vol);
        if (!$error ) {
            die("run_verifyio: Info: starting verify failed for aggr - $filer_vol");
        }

        $Log->comment("run_verifyio: Info: waiting for verify to complete on aggr - $filer_vol");
        RaidSqa_wait_for_verify_completion(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
        return 1;
        $Log->comment("run_verifyio: Info: completed run_verifyio on filer_vol - $filer_vol");
    }

    sub run_reconsio {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => {type=>SCALAR}});
        my $filer_vol = $opts{filer_vol};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $api_set_obj = $self->{api_set_obj};

        my ($enable_new_recons_flag, $old_flag_value) = (0, 0);
        foreach my $rg_name ($aggr->get_aggr_info_rgs()){
            $rg_name =~ /\/$filer_vol\/(plex\d)\/(rg\d+)/;
            my $plex = $1;
            my $rg = $2;

            my @failed_disk_list =     @{$aggr->get_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>'fail')};
            my @offlined_disk_list = @{$aggr->get_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>'offline')};

            if (scalar(@failed_disk_list) > 0 or scalar(@offlined_disk_list) > 0) {
                # Enable reconstruction
                $Log->comment("run_reconsio: Info: start recons on filer_vol - $filer_vol");
                if($enable_new_recons_flag == 0){
                    $old_flag_value = RaidSqa_get_setflag(api_set_obj => $api_set_obj,flag_name   => 'raid_disable_new_recons');
                    $Log->comment("run_reconsio: Info: Enabling recons to start recons on failed disks");
                    RaidSqa_setflag(api_set_obj => $api_set_obj,flag_name   => 'raid_disable_new_recons',flag_value  => 0);
                    $enable_new_recons_flag =1;
                }
            }

            if (scalar(@offlined_disk_list) > 0) {
                $Log->comment("run_reconsio: Info: Onlining disks to start recons on offlined disks");
                foreach my $offline_disk (@offlined_disk_list) {
                    RaidSqa_disk_online(api_set_obj => $api_set_obj, disk_name =>$offline_disk);
                }
            }
        }

        sleep(10);
        $Log->comment("run_reconsio: Info: Wait for recons to complete on filer_vol - $filer_vol");
        RaidSqa_wait_for_reconstruction_completion(api_set_obj => $api_set_obj,filer_vol => $filer_vol);

        #update reconstructed disks details to aggr_info
        if ($enable_new_recons_flag == 1 ) {
            $Log->comment("run_reconsio: Info: Updating reconstructed disks details to aggr_info...");
            $aggr->get_reconstructing_disks();

            #before existing, we need to reset flag 'raid_disable_new_recons' so that next raidIO will have correct config
            RaidSqa_setflag(api_set_obj => $api_set_obj,flag_name => 'raid_disable_new_recons',flag_value => $old_flag_value);
        }

        $Log->comment("run_reconsio: Info: completed run_reconsio on filer_vol - $filer_vol");
    }

    sub run_resyncio {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => {type=>SCALAR}});
        my $filer_vol = $opts{filer_vol};
        my $api_set_obj = $self->{api_set_obj};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $offline_plex = $aggr->get_plex_offline();

        if (!$offline_plex) {
            die("run_resyncio: Error: resyncio failed as no offlined plex found");
        }
        $Log->comment("run_resyncio: Info: start resyncio by onlining plex $offline_plex");
        my $error = RaidSqa_plex_online(api_set_obj => $api_set_obj, plex_name => "/$filer_vol/$offline_plex");
        if ( !$error ) {
            die("run_resyncio: Error: Plex online operation failed for plex /$filer_vol/$offline_plex");
        }
        sleep(5);
        RaidSqa_wait_for_plex_resync_completion(api_set_obj => $api_set_obj, plex_name => "/$filer_vol/$offline_plex", resync_level => 1);
        $Log->comment("run_resyncio: Info: completed run_resyncio on filer_vol - $filer_vol for plex - /$filer_vol/$offline_plex");
    }

    sub run_replayio {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => {type=>SCALAR}});
        my $filer_vol = $opts{filer_vol};
        my $api_set_obj = $self->{api_set_obj};

        # Panic - through nv_trash
        $Log->comment("run_replayio: Started");
        RaidSqa_nv_trash(api_set_obj=>$api_set_obj);

        # Wait for the code dump to happen
        sleep 1320;  # Temporarily using sleep instead we should be using APIs to wait for the codedump to complete

        # Wait for the filer to boot up
        RaidSqa_transit_to(api_set_obj=>$api_set_obj, state_name => "UP", timeout => 2400);

        sleep 150;  # Temporarily using sleep instead we should be using APIs to wait for the codedump to complete

        # Wait for reparity to complete on filer_vol
        RaidSqa_wait_for_reparity_completion(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol);
        $Log->comment("run_replayio: Completed");
    }

    sub run_nvfile_replayio {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => {type=>SCALAR}});
        $Log->comment("run_nvfile_replayio: Started");
        my $filer_vol = $opts{filer_vol};
        my $api_set_obj = $self->{api_set_obj};
        my $aggr_type = 0;

        my $cmd = "priv set test;setflag wafl_metadata_visible 1;ls /vol/vol0/metadir/nvsave/";

        # Panic - through nv_trash
        RaidSqa_nv_trash(api_set_obj=>$api_set_obj);

        # Wait for the code dump to happen
        sleep 1320;  # Temporarily using sleep instead we should be using APIs to wait for the codedump to complete

        # Wait for the filer to boot up
        RaidSqa_transit_to(api_set_obj=>$api_set_obj, state_name => "UP");

        # Bring the volume offline
        RaidSqa_aggr_offline(api_set_obj => $api_set_obj, filer_vol=> $filer_vol, aggr_type => $aggr_type);

        # Check for the nvsave file.
        RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd => $cmd);

        # Check to make sure that the vol is offline.
        RaidSqa_get_vol_status(host=> $api_set_obj->{hostobj}, filer_vol=>$filer_vol);

        sleep 150;  # Temporarily using sleep instead we should be using APIs to wait for the codedump to complete

        # Bring the volume back online
        RaidSqa_aggr_online(api_set_obj => $api_set_obj, filer_vol=>$filer_vol, aggr_type => $aggr_type);

        # Wait for reparity to complete on filer_vol
        RaidSqa_wait_for_reparity_completion(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol);
        $Log->comment("run_nvfile_replayio: Completed");
    }


    #=================================== Methods to support validation of raidios ===================================
    sub validate_raidio {
        my $self =shift;
        my %opts = validate(@_, {
                        filer_vol => {type=>SCALAR},
                        raidio_type => {type=>SCALAR}, # TODO: This is not used for now, so remove later
                    });

        my $filer_vol = $opts{filer_vol};
        my $raidio_type = $opts{raidio_type};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $error_info => $self->get_error_info(filer_vol => $filer_vol);
        my $api_set_obj = $self->{api_set_obj};
        my $tc_data = $self->get_tc_data();
        my $ems_marker=$aggr->get_ems_marker();
        my $ems_data_hash = undef;

        $self->flush_cache(filer_vol=>$filer_vol, checksum_type => $aggr->get_checksum_type(), aggr_type => $aggr->get_aggr());
        # validating raid lost write flags settings
        $Log->comment("validate_raidio: Info : validate raid_lost_write flags settings for raidio - $raidio_type");
        $self->validate_raid_lost_write_flags(filer_vol => $filer_vol);

        my @ems_data_list = ();
        my @plex_list = grep (/^plex(\d)$/i, keys %$error_info);
        foreach my $plex (@plex_list){
            my @rg_list = grep (/rg/i, keys %{$error_info->{$plex}});
            foreach my $rg (@rg_list){
                my $newDBN = undef;
                if (exists $error_info->{$plex}->{$rg}->{zones}){
                    my @zones = @{$error_info->{$plex}->{$rg}->{zones}};
                    foreach my $zoneid (0..$#zones){
                        if (exists $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}){
                            my @errors = @{$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}};
                            foreach my $stripeid (0..$#errors){
                                my $dbn = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{dbn};
                                my $cksum_dbn = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{cksum_dbn};
                                $newDBN = $dbn;

                                foreach my $stripe_err (grep(/(media|parity_inconsistency|zeroblock|checksum)/, keys %{$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]}) ){
                                    $Log->comment("validate_raidio: Info: stripe_err - $stripe_err");
                                    if(exists $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err}){

                                        # traverse each of media, pi, zeroblock and array within it.
                                        my @stripe_err_arr = @{$error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err}};
                                        foreach my $errid (0..$#stripe_err_arr){
                                            my $ems = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err}->[$errid]->{ems};
                                            if(defined($ems)){
                                                push(@ems_data_list, $ems);
                                            }
                                            my $disk_name = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err}->[$errid]->{disk_name};
                                            my $disk_type = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err}->[$errid]->{disk_type};
                                            my $bit_set = $error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err}->[$errid]->{bit_set};

                                            if($error_info->{$plex}->{$rg}->{zones}->[$zoneid]->{stripes}->[$stripeid]->{$stripe_err}->[$errid]->{inject_at} eq 'checksum_blk'){
                                                $newDBN = undef;
                                            }
                                            $Log->comment("validate_raidio: Info: validating multierror flags at - disk_name - $disk_name, disk_type - $disk_type,".
                                                          " dbn - $dbn, cksum_dbn - $cksum_dbn, injected dbn - $newDBN for raidio - $raidio_type");
                                            $self->validate_multierror_flags(filer_vol => $filer_vol, error_type=>$stripe_err, disk_name=>$disk_name, disk_type=>$disk_type, dbn=>$newDBN);
                                            $self->validate_bit_set(disk_name => $disk_name, dbn => $dbn, bit_set => $bit_set);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        if (scalar(@ems_data_list) > 0) {
            $Log->comment("validate_raidio: Info : validating ems events for raidio - $raidio_type");
            $self->validate_ems_event(ems_mark=>$ems_marker, ems_data_list=>\@ems_data_list, raidio_type => $raidio_type);
        } else {
            $Log->comment("validate_raidio: Info : No ems data is defined: ".Dumper(\@ems_data_list));
        }

        # Validate scrubio and its summary report to match injected errors and detected errors
        # scrub reports 64 checksum errors for 1 media_err if its injected on checksum block

        if($raidio_type eq "scrubio" && $aggr->get_checksum_type() eq 'block'){     #TODO: For advanced_zone, media errors are not reported in scrub report
            # my @scrub_error_summary = ("pi", "cksum_err", "media_err","lw_err");
            my @scrub_error_summary = ("pi", "media_err");
            $Log->comment("validate_raidio: Info : validating scrubio report for all groups");
            my @group_list  = $aggr->get_aggr_info_rgs();

            foreach my $rg_name (@group_list) {
                $Log->comment("validate_raidio: Info : verifying scrub summary report for group - $rg_name");
                my $ref_report = RaidSqa_get_scrub_report_by_group(api_set_obj => $self->{api_set_obj}, group_name  => $rg_name );
                if($ref_report == 0){
                    $Log->comment("validate_raidio: Warn: some error encountered during getting scrub report for group - $rg_name");
                    next;
                }

                foreach my $error (@scrub_error_summary) {
                    my $expected_injected_err_count = $self->get_injected_err_count(filer_vol=> $filer_vol, rg_name=>$rg_name, error=>$error);
                    my @inject_at_array = $self->get_inject_at_array(filer_vol=> $filer_vol, rg_name=>$rg_name, error=>$error);
                    my @checksum_block_errors = grep /checksum_blk/, @inject_at_array ;
                    $expected_injected_err_count -= scalar(@checksum_block_errors) if($error eq "media_err");

                    #for PI error, it might different values in report based on which disks errors were injected on.
                    if(($error eq "media_err" && int($ref_report->{$error}) < $expected_injected_err_count) or
                        ($error eq "pi" && $expected_injected_err_count > 0 && int($ref_report->{$error}) == 0)){

                        die("validate_raidio: Error : Scrub summary report failed, $error value mismatch.  ".
                            "Expected - $expected_injected_err_count, obtained $ref_report->{$error} for group $rg_name");
                    } else {
                        $Log->comment("validate_raidio: Info : Scrub summary report passed for $error. ".
                                      "Detected same as injected - $expected_injected_err_count for group $rg_name");
                    }
                }
            }
        }

        #Run scrub in end to make sure no errors are hit and check for scrub summary report.
        $Log->comment("validate_raidio: Info : validate scrub summary report at end of each test for raidio - $raidio_type");
        if($aggr->is_rg_degraded() == 0){
            #$self->validate_scrub_summary(raidio_type=>$raidio_type, filer_vol => $filer_vol);
        }
    }

    sub validate_raid_lost_write_flags {
        my $self  = shift;
        my %opts = validate(@_, {
                                filer_vol=>{type=>SCALAR},
                            });
        my $filer_vol   = $opts{filer_vol};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $tc_data     = $self->get_tc_data();
        my $api_set_obj = $self->{api_set_obj};
        my @group_list  = $aggr->get_aggr_info_rgs(filer_vol => $filer_vol);

        my ($raid_lw_allowed,$raid_lw_enabled, $raid_lost_write, $group_raid_lw_scrub);
        my $ems_data_hash = undef;

        # Check for system wide and aggr wide option.
        my $rlw_system_wide_value = RaidSqa_get_options(api_set_obj=>$api_set_obj,option_name => 'raid.lost_write.enable');
        my $rlw_aggr_wide_value   = RaidSqa_get_aggr_options(api_set_obj=>$api_set_obj,filer_vol => $filer_vol,options =>'raid_lost_write');

        my $system_wide_lw_option = $tc_data->{setup_info}->{global_options_settings}->{"raid.lost_write.enable"}->{required_val};
        my $aggr_wide_lw_option   = $aggr->{aggr_options}->{"raid_lost_write"};

        if ($aggr_wide_lw_option eq 'off') { # check what param is passed from tc_data
                if ($rlw_aggr_wide_value eq "off") {
                    $Log->comment("validate_raid_lost_write_flags: Info: option \"raid_lost_write\" is same as expected value - $rlw_aggr_wide_value");
                } else {
                    die("validate_raid_lost_write_flags: Error: option \"raid_lost_write\" setting. Expected value - off, Actual - $rlw_aggr_wide_value");
                }
                # check following values in aggr and group using raid_config
                $raid_lw_allowed     = 0;
                $raid_lw_enabled     = 0;
                $raid_lost_write     = "off";
                $group_raid_lw_scrub = "DISABLED";
        } elsif($system_wide_lw_option eq 'off') {
                if ($rlw_system_wide_value eq "off") {
                    $Log->comment("validate_raid_lost_write_flags: Info: option \"raid.lost_write.enable\" is same as expected value - $rlw_system_wide_value");
                } else {
                    die("validate_raid_lost_write_flags: Error: option \"raid.lost_write.enable\" setting.  Expected value - off, Actual - $rlw_system_wide_value");
                }
                # check following values in aggr and group using raid_config
                $raid_lw_allowed     = 1;
                $raid_lw_enabled     = 0;
                $raid_lost_write     = "inoperative";
                $group_raid_lw_scrub = "INACTIVE";
        } else {
            if ($rlw_aggr_wide_value eq "on" and $rlw_system_wide_value eq "on") {
                $Log->comment("validate_raid_lost_write_flags: Info: option \"raid_lost_write and raid.lost_write.enable\" is same as expected value - $rlw_aggr_wide_value");
            } else {
                die("validate_raid_lost_write_flags: Error: option \"raid_lost_write or raid.lost_write.enable\" is not same as expected value. Expected - on");
            }
            # check following values in aggr and group using raid_config
            $raid_lw_allowed     = 1;
            $raid_lw_enabled     = 1;
            $raid_lost_write     = "on";
            $group_raid_lw_scrub = "DONE";
        }

        # check aggr flags related to RLW
        $Log->comment("validate_raid_lost_write_flags: Info : Check for aggr flags related to RLW for volume $filer_vol");
        $Log->comment("validate_raid_lost_write_flags: Info : Expected flags. raid_lw_allowed - $raid_lw_allowed, raid_lw_enabled - $raid_lw_enabled, raid_lost_write - $raid_lost_write");
        my $result = RaidSqa_check_aggr_lw_flags(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol,
                                                 raid_lw_allowed=>$raid_lw_allowed, raid_lw_enabled=>$raid_lw_enabled,
                                                 raid_lost_write=>$raid_lost_write);
        if (!$result) {
            die("validate_raid_lost_write_flags: Error : Raid Lost Write flags for aggr are not set to proper values as expected");
        }

        # check group flags related to RLW
        $Log->comment("validate_raid_lost_write_flags: Info : Check for group flags related to RLW for groups @group_list");
        $result = RaidSqa_check_group_lw_flags(api_set_obj=>$api_set_obj, group_list=>\@group_list, group_raid_lw_scrub=>$group_raid_lw_scrub);
        if (!$result) {
            die("validate_raid_lost_write_flags: Error : Raid Lost Write flags for group are not set to proper values as expected. Expected group_raid_lw_scrub - $group_raid_lw_scrub");
        }
        return 1;
    }

    sub validate_ems_event {
        my $self = shift;
        my (%opts) = validate(@_, {
                ems_mark      => {type => SCALAR},
                ems_data_list => {type => ARRAYREF},
                raidio_type   => {type => SCALAR},
            }
        );

        my $ems_mark      = $opts{'ems_mark'};
        my @ems_data_list = @{$opts{'ems_data_list'}};
        my $raidio        = $opts{'raidio_type'};

        $Log->comment("validate_ems_event: Info : validating ems events");
        $Log->comment("validate_ems_event: Debug : ems data: ".Dumper(\@ems_data_list));
        # Get list of events and their id's.
        my $ref_events = RaidSqa_ems_log_dump_by_type(api_set_obj => $self->{api_set_obj}, timestamp => $ems_mark);
        my $ref_ids = RaidSqa_ems_log_dump_by_id(api_set_obj => $self->{api_set_obj}, timestamp   => $ems_mark);

        foreach my $ems_data (@ems_data_list) {
            foreach my $event (keys %$ems_data){
                if(!grep /$raidio/, @{$ems_data->{$event}->{'raidio'}}){ next; }
                $Log->comment("validate_ems_event: Info: Check for ems event - $event");
                delete $ems_data->{$event}->{'raidio'};
                $Log->comment("validate_ems_event: Debug: ems data - ".Dumper($ems_data->{$event}));
                if(!exists $ref_events->{$event}){
                    die("validate_ems_event: Error: Ems event $event has not occured for raidioio - $raidio");
                }
                my @ems_ids = @{$ref_events->{$event}};
                my $event_occured = 0;
                foreach my $id (@ems_ids) {
                    my $match = 1;
                    foreach my $key (keys %{$ems_data->{$event}}){
                        if($ref_ids->{$id}->{$key} ne $ems_data->{$event}->{$key}) {
                            $match = 0;
                            last;
                        }
                    }
                    if($match == 1) {
                        $event_occured = 1;
                        $Log->comment("validate_ems_event: Info: EMS event $event has occured for given ems_data for raidio - $raidio");
                        last;
                    }
                }
                if($event_occured == 0){
                    die("validate_ems_event: Error: Ems event $event has not occured for raidioio - $raidio");
                }
            }
        }
    }

    sub validate_bit_set {
        my $self = shift;
        my %opts = validate(@_, {
                        disk_name => {type => SCALAR},
                        dbn       => {type => SCALAR},
                        bit_set   => {type => SCALAR, default=>"dump", optional => 1},
                    });

        my $bit_set = $opts{bit_set};

        $Log->comment("validate_bit_set: Info: Expected bit_set - $bit_set");
        my $cmd = "priv set -q diag; dumpblock dbn -c $opts{disk_name} $opts{dbn}";
        my $output = RaidSqa_filer_cmd(api_set_obj => $self->{api_set_obj}, cmd=>$cmd);
        my @lines = split(/\r*\n/, $output);
        my $bit_set_line = undef;
        foreach my $line (@lines) {
            if($line =~ /0x0020:/) {
                $bit_set_line = $line;
                last;
            }
        }

        my @_4byte_words = split(/ /, $bit_set_line);
        my $value = hex($_4byte_words[4]);
        my $pseudo_bad = $value & 0x00000001;
        my $unverified = $value & 0x00000002;

        if($bit_set eq "dump") {
            return;
        }

        if($bit_set eq "unverified") {
            if($unverified == 1) {
                $Log->comment("validate_bit_set: Info: Block $opts{dbn} on disk $opts{disk_name} is set to unverified");
                return;
            }
            die("validate_bit_set: Error: Block $opts{dbn} on disk $opts{disk_name} is set to unverified");
        }

        if($bit_set eq "pseudo_bad") {
            if($pseudo_bad == 1) {
                $Log->comment("validate_bit_set: Info: Block $opts{dbn} on disk $opts{disk_name} is set to pseudo_bad");
                return;
            }
            die("validate_bit_set: Error: Block $opts{dbn} on disk $opts{disk_name} is not set to pseudo_bad");
        }

        if($bit_set eq "none") {
            if($pseudo_bad == 1) { die("validate_bit_set: Error: Block $opts{dbn} on disk $opts{disk_name} is set to pseudo_bad");}
            if($unverified == 1) { die("validate_bit_set: Error: Block $opts{dbn} on disk $opts{disk_name} is set to unverified");}
            $Log->comment("validate_bit_set: Info: Block $opts{dbn} on disk $opts{disk_name} is not set to pseudo_bad or unverified");
        }
    }

    sub validate_scrub_summary {
        my $self  = shift;
        my %opts = validate(@_, {
                                raidio_type => {type => SCALAR},
                                filer_vol   => {type => SCALAR},
                            });
        my $filer_vol   = $opts{filer_vol};
        my $raidio_type = $opts{raidio_type};
        my $api_set_obj = $self->{api_set_obj};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $error_info = $self->get_error_info(filer_vol=>$filer_vol);
        my @group_list  = $aggr->get_aggr_info_rgs(filer_vol=>$filer_vol);

        # NOTE: See if there are any stripe errors on rgs that doesn't have disk_fail/disk_offline, but raidio_type is reconsio.
        # In such case scrub summary may detect the left over undetected errors and that should be ok.

        my $scrub_shows_errors = 0;
        my @plex_list = grep (/^plex(\d)$/i, keys %$error_info);
        foreach my $plex (@plex_list){
            my @rg_list = grep (/rg/i, keys %{$error_info->{$plex}});
            foreach my $rg (@rg_list){
                my $stripe_err = $self->has_error_type(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, error_type=>"zones");
                my $disk_fail_err = $self->has_error_type(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, error_type=>"disk_fail");
                my $disk_offline_err = $self->has_error_type(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, error_type=>"disk_offline");

                if($stripe_err and !($disk_fail_err or $disk_offline_err) and $raidio_type eq "reconsio"){
                    $scrub_shows_errors = 1;
                    last;
                }
            }
            if($scrub_shows_errors){
                last;
            }
        }

        $Log->comment("validate_scrub_summary: Info : At the end of test re-running scrub to verify we have no more errors");
        $self->run_scrubio(validation_step=>1, filer_vol=>$filer_vol);
        #-------------------------------------------------------
        # Check scrub summary report for all groups.
        #-------------------------------------------------------
        my @scrub_error_summary = ("pi", "cksum_err", "media_err","lw_err");
        $Log->comment("validate_scrub_summary: Info : Check scrub summary report for all groups");

        foreach my $rg_name (@group_list) {
            $Log->comment("validate_scrub_summary: Info : verifying scrub summary report for group - $rg_name");
            my $ref_report = RaidSqa_get_scrub_report_by_group(api_set_obj => $self->{api_set_obj}, group_name  => $rg_name );
            $Log->comment("validate_scrub_summary: Debug: Value of ref_report is ".Dumper($ref_report));
            foreach my $error (@scrub_error_summary) {
                if ($ref_report and $ref_report->{$error} ) {
                    if($scrub_shows_errors){
                        $Log->comment("validate_scrub_summary: Warn : Scrub summary report has $error seen $ref_report->{$error} times for group $rg_name.  \n".
                                      "This is ok as testcase did not have any recons(disk_fail/disk_offline) in a rg with stripes error for raidio_type - reconsio.");
                    } else {
                        die("validate_scrub_summary: Error : Scrub summary report failed, $error occurred for $ref_report->{$error} times for group $rg_name");
                    }
                } else {
                    $Log->comment("validate_scrub_summary: Info : Scrub summary report passed for $error: $error did not occur for group $rg_name");
                }
            }
        } ## end foreach my $rg_name (@group_list)
    }

    sub validate_multierror_flags {
        my $self = shift;
        my (%opts) = validate(@_, {
                                    filer_vol => {type=>SCALAR},
                                    error_type => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                                    disk_name => {type=>SCALAR|UNDEF, default=>undef},
                                    disk_type => {type=>SCALAR|UNDEF, default=>undef},
                                    dbn => {type=>SCALAR|UNDEF, default=>undef},

                                });
        my $filer_vol = $opts{filer_vol};
        my $api_set_obj = $self->{api_set_obj};
        my $aggr = $self->get_aggregate(filer_vol => $filer_vol);
        my $tc_data = $self->get_tc_data();
        my $type = $self->get_error_type();

        my $error_type = $opts{error_type};
        my $disk_name  = $opts{disk_name};
        my $disk_type  = $opts{disk_type};
        my $dbn = $opts{dbn};

        my $error_values = {};
        my $value_recover = 0;

        if($aggr->get_ignore_lw_validation() == 0){
            $Log->comment("validate_multierror_flags: Info: Checking for has_bad_blocks on filer_vol - $filer_vol, disk - $disk_name, dbn - $dbn");
            $error_values->{aggr_has_bad_blocks} = RaidSqa_get_aggr_has_bad_blocks(api_set_obj=>$api_set_obj,filer_vol=>$filer_vol);

            $Log->comment("validate_multierror_flags: Info: Checking for bad_block_count on filer_vol - $filer_vol, disk - $disk_name, dbn - $dbn");
            $error_values->{aggr_bad_block_count} = RaidSqa_get_aggr_bad_block_count(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol);

            $Log->comment("validate_multierror_flags: Info: Checking for tetris_opt_disabled on filer_vol - $filer_vol, disk - $disk_name, dbn - $dbn");
            $error_values->{aggr_tetris_opt_disabled} = RaidSqa_get_aggr_tetris_opt_disabled(api_set_obj=>$api_set_obj,filer_vol=>$filer_vol);

            if ($disk_type =~ /data/i && defined($dbn)) {
                $Log->comment("validate_multierror_flags: Info: Checking for pseudo_bad bit for filer_vol - $filer_vol, disk - $disk_name, dbn - $dbn");
                $error_values->{disk_block_is_pseudo} = RaidSqa_check_disk_block_is_pseudo(api_set_obj=>$api_set_obj,disk_name=>$disk_name, dbn=>$dbn);
            }
        }

        #TODO (FIXED: 25-March-11), add check for dbn and disk_type
        if ($error_type eq "media" && defined($dbn) && $disk_type =~ /data/i) {
            $Log->comment("validate_multierror_flags: Info: Checking for disk block is zeroed_out for filer_vol - $filer_vol, disk - $disk_name, dbn - $dbn");
            $error_values->{disk_block_is_zeroed_out} = RaidSqa_check_disk_block_is_zeroed_out(api_set_obj=>$api_set_obj, disk_name =>$disk_name, dbn=>$dbn);
            $Log->comment("validate_multierror_flags: debug: ".Dumper($error_values));
         }

        foreach my $error (keys %{$error_values} ) {
            if ($type eq "recover") {
                if ($error_values->{$error} == $value_recover) {
                    $Log->comment("validate_multierror_flags: Info: required \"recoverable\" error param $error is set to $value_recover");
                } else {
                    die("validate_multierror_flags: Error: checking during \"recoverable\" scenario - error param - $error, Expected - $value_recover, Actual - ".$error_values->{$error});
                }
            } else {
                if($aggr->get_ignore_lw_validation() == 0){
                    if ($error_values->{$error} > $value_recover) {
                        $Log->comment("validate_multierror_flags: Error: required \"unrecoverable\" error param $error is set to ".$error_values->{$error});
                    } else {
                        die("validate_multierror_flags: Error: required \"unrecoverable\" error param $error should not be $value_recover");
                    }
                }
            }
        }
        return 1;
    }

    #=================================== Internal search methods  ===================================
    sub update_aggr_name {
        my $self = shift;
        my %opts = validate(@_, {
                        old_filer_vol => {type => SCALAR},
                        new_filer_vol => {type => SCALAR},
                    });

        #update all references of $filer_vol with new name

        #update tc data
        my $tc_data = $self->get_tc_data();
        foreach my $index (0..scalar(@{$tc_data->{aggregates}})-1) {
            if($tc_data->{aggregates}->[$index]->{filer_vol} eq $opts{old_filer_vol}) {
                $tc_data->{aggregates}->[$index]->{filer_vol} = $opts{new_filer_vol};
            }
        }

        #update error_info
        my $error_info = $self->get_error_info(filer_vol => $opts{old_filer_vol});
        $self->set_error_info(filer_vol => $opts{new_filer_vol}, error_info => $error_info);
        $self->delete_error_info(filer_vol => $opts{old_filer_vol});

        #update aggregate objects and file create objects
        my $aggr_object = $self->get_aggregate(filer_vol => $opts{old_filer_vol});
        $aggr_object->set_filer_vol(filer_vol => $opts{new_filer_vol});
        my $fileCreate = $aggr_object->get_fileCreate();
        my $file_info = $fileCreate->get_file_info();

        foreach my $filer_vol (keys %$file_info) {
            if($filer_vol eq $opts{old_filer_vol}) {
                my $rg_info = $file_info->{$filer_vol};

                #update rg_names with new name
                foreach my $rg_name (keys %$rg_info) {
                    my $new_rg_name = $rg_name;
                    $new_rg_name =~ s/$opts{old_filer_vol}/$opts{new_filer_vol}/g;
                    $rg_info->{$new_rg_name} = $rg_info->{$rg_name};
                    delete $rg_info->{$rg_name};
                }

                $file_info->{$opts{new_filer_vol}} = $rg_info;
                delete $file_info->{$filer_vol};
            }
        }

        $fileCreate->set_file_info(file_info => $file_info);
        $fileCreate->{aggregate} = $aggr_object;
        $aggr_object->set_fileCreate(fileCreate => $fileCreate);
        $self->set_aggregate(filer_vol => $opts{new_filer_vol}, object => $aggr_object);
        $self->delete_aggregate(filer_vol => $opts{old_filer_vol});
    }

    sub get_error_type {
        my $self = shift;

        my $type = lc $self->{tc_data}->{error_info}->{type};
        if($type ne 'recover' && $type ne 'unrecover') {
            $type = 'recover';
        }
        return $type;
    }

    sub set_error_type {
        my $self = shift;
        my (%opts) = validate(@_, {type => {type=>SCALAR}});

        $self->{tc_data}->{error_info}->{type} = $opts{type};
    }

    sub get_ems_marker{
        my $self =shift;
        my (%opts) = validate(@_, {filer_vol => {type=>SCALAR}});
        my $aggr = $self->get_aggregate(filer_vol => $opts{filer_vol});

        return $aggr->get_ems_marker();
    }

    sub get_aggregate {
        my $self = shift;
        my %opts = validate(@_, {
            filer_vol   => {type => SCALAR},
        });

        if(exists $self->{aggr_create_hash}->{$opts{filer_vol}}) {
            return $self->{aggr_create_hash}->{$opts{filer_vol}};
        }

        my @aggr_create_vols = keys %{$self->{aggr_create_hash}};
        $Log->comment("get_aggregate: Dumper: aggr_create_hash keys - @aggr_create_vols");
        die("get_filer_vol: Error: Object for aggregate with name ".$opts{filer_vol}." does not exist");
    }

    sub set_aggregate {
        my $self = shift;
        my %opts = validate(@_, {
            filer_vol => {type => SCALAR},
            object => {type => OBJECT, isa=>'AggrCreate'},
        });

        $self->{aggr_create_hash}->{$opts{filer_vol}} = $opts{object};
    }

    sub delete_aggregate {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol   => {type => SCALAR},
                    });

        delete $self->{aggr_create_hash}->{$opts{filer_vol}};
    }

    sub get_raidio_list {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol => {type=>SCALAR, default=>'all', optional=>1},
                    });

        my $aggr = $self->get_aggregate(filer_vol => $opts{filer_vol});
        return  @{$aggr->get_raidio()};
    }

    sub raidio_exists {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol => {type=>SCALAR, default=>'all', optional=>1},
                        raidio_type => {type=>SCALAR},
                    });
        my @raidio_list = $self->get_raidio_list(filer_vol => $opts{filer_vol});
        my $return      = (grep /$opts{raidio_type}/, @raidio_list) ? 1 : 0;
        if($return) {$Log->comment("raidio_exists: Info: Raid IO $opts{raidio_type} is valid for filer_vol $opts{filer_vol}");}
        return $return;
    }

    sub get_use_stripe_offset {
        my $self = shift;
        my %opts = validate(@_, {filer_vol => {type => SCALAR}});

        my $error_info = $self->get_error_info(filer_vol => $opts{filer_vol});
        foreach my $plex (keys %$error_info) {
            foreach my $rg (keys %$plex) {
                foreach my $zone (@{$rg->{zones}}) {
                    foreach my $stripe (@{$zone->{stripes}}){
                        if(exists $stripe->{stripe_offset}) {
                            return 1;
                        }
                    }
                }
            }
        }
    }

    sub get_error_info_plex_count {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol => {type => SCALAR},
                        error_info => {type => HASHREF|UNDEF, default=>undef, optional=>1},
                    });
        my $filer_vol = $opts{filer_vol};
        my $error_info = $opts{error_info};
        if(!defined($error_info)) {
            $self->get_error_info(filer_vol=>$filer_vol);
        }

        my @plex_list = grep(/plex\d+/, keys %$error_info);
        return scalar(@plex_list);
    }

    sub compute_required_file_size {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol  => {type=>SCALAR},
                        stripe_gap => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                        stripe_count => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                        zone_count   => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                    });

        my $stripe_gap = $opts{stripe_gap};
        my $stripe_count = $opts{stripe_count};
        my $zone_count = $opts{zone_count};
        if(!defined($stripe_count) && !defined($zone_count)) {
            die("compute_required_file_size: Error: Either stripe count or zone count should be defined");
        } elsif(defined($stripe_count) && !defined($stripe_gap)) {
            die("compute_required_file_size: Error: Stripe_gap should be defined when stripe_count is defined");
        }


        my $aggr = $self->get_aggregate(filer_vol=>$opts{filer_vol});
        my $rgs = $aggr->get_rgs();
        my $parity_disk_count = ($aggr->get_raid_type() eq "raid_dp") ? 2 : 1;
        my $data_disk_count = 0 ;
        foreach my $rg (@$rgs) {
            #If aggr is hybrid_enabled and RG is SSD tier, ignore disk count for RG
            if($aggr->get_hybrid_aggregate() == 1 && $rg->{ssd_tier} == 1) {
                next;
            }
            $data_disk_count += ($rg->{disk_count} - $parity_disk_count);
        }

        $Log->comment("compute_required_file_size: Debug: data_disk_count: $data_disk_count");

        my $file_size = $data_disk_count * 4096 * 64 * 10;
        if(defined($stripe_count)) {
            $file_size *= ($stripe_count * $stripe_gap);
        }  elsif($zone_count > 0) {
            $file_size *= ($zone_count);
        }

        $file_size = '7864320' if($file_size <= 0);     #default 4096 * 64 * 10 * 3
        return $file_size;
    }

    sub get_unique_stripe_count {
        my $self = shift;
        my %opts = validate(@_, {
            filer_vol   => {type=>SCALAR},
            rg          => {type=>SCALAR}
        });

        my $filer_vol = $opts{filer_vol};
        my $rg = $opts{rg};
        my $error_info = $self->get_error_info(filer_vol=>$filer_vol);
        my @plex_list = sort grep /plex/, keys %$error_info;
        my $stripes = {};
        my $unique_stripe_count = 0;

        foreach my $plex (@plex_list) {
            my @zones = @{$error_info->{$plex}->{$rg}->{zones}} if(ref($error_info->{$plex}->{$rg}->{zones}) =~ /ARRAY/i);
            $stripes->{$plex} = [] if(!@zones);
            foreach my $zone (@zones) {
                if(defined($stripes->{$plex})) {
                    push(@{$stripes->{$plex}}, @{$zone->{stripes}});
                } else {
                    $stripes->{$plex} = [@{$zone->{stripes}}];
                }
            }
        }
        if(scalar(@plex_list) == 1) {
            $Log->comment("get_unique_stripe_count: Info: Filer_vol - $filer_vol, rg - $rg, unique_stripe_count - ".scalar(@{$stripes->{$plex_list[0]}}));
            return scalar(@{$stripes->{$plex_list[0]}});
        }

        #Find Total stripes
        my $total_stripe_count =  scalar(@{$stripes->{$plex_list[0]}})+ scalar(@{$stripes->{$plex_list[1]}});
        my ($plex0_same_stripe_count, $plex1_same_stripe_count);

        foreach my $stripe (@{$stripes->{$plex_list[0]}}) {
            if($stripe->{'stripe_same_plex_dbn'} == 1) {
                $plex0_same_stripe_count += 1;
            }
        }

        foreach my $stripe (@{$stripes->{$plex_list[1]}}) {
            if($stripe->{'stripe_same_plex_dbn'} == 1) {
                $plex1_same_stripe_count += 1;
            }
        }

        $unique_stripe_count = $total_stripe_count - min($plex0_same_stripe_count, $plex1_same_stripe_count);
        $Log->comment("get_unique_stripe_count: Info: Filer_vol - $filer_vol, rg - $rg, unique_stripe_count - $unique_stripe_count");
        return $unique_stripe_count;
    }

    sub get_unique_zones {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol   => {type=>SCALAR},
                        rg          => {type=>SCALAR}
                    });

        my $filer_vol = $opts{filer_vol};
        my $rg = $opts{rg};
        my $error_info = $self->get_error_info(filer_vol=>$filer_vol);
        my @plex_list = sort grep /plex/, keys %$error_info;
        my $zones = [];
        my ($plex0_same_zone_dbn, $plex1_same_zone_dbn) = (0,0);
        my $total_zone_count = 0;

        foreach my $plex (@plex_list) {
            $total_zone_count += scalar(@{$error_info->{$plex_list[0]}->{$rg}->{'zones'}})
                if(ref($error_info->{$plex_list[0]}->{$rg}->{'zones'}));
        }

        return 0 if ($total_zone_count == 0);       #if there are no zones, we should not go further


        #plex0 zone_same_plex_dbn count
        foreach my $zone (@{$error_info->{$plex_list[0]}->{$rg}->{zones}}) {
            if($zone->{'zone_same_plex_dbn'} == 1) {
                $plex0_same_zone_dbn += 1;
            }
        }

        #plex1 zone_same_plex_dbn count
        foreach my $zone (@{$error_info->{$plex_list[1]}->{$rg}->{zones}}) {
            if($zone->{'zone_same_plex_dbn'} == 1) {
                $plex1_same_zone_dbn += 1;
            }
        }

        my $zoneid = 0;
        my $unique_zone_count = (@plex_list == 1) ? $total_zone_count :
                                $total_zone_count - max($plex0_same_zone_dbn, $plex1_same_zone_dbn);

        $Log->comment("get_unique_zones: Info: total_zone_count - $total_zone_count, unique_zone_count - $unique_zone_count, max_zone_count - max($plex0_same_zone_dbn, $plex1_same_zone_dbn)");
        for(my $index=0; $index < $unique_zone_count; $index++){
            $Log->comment("get_unique_zones: Info: filer_vol - $filer_vol, rg - $rg, zoneid - $zoneid");
            $zones->[$index] = [];
            if($self->is_zone_same_plex_dbn(filer_vol=>$filer_vol, rg=>$rg, zoneid=>$zoneid)){
                foreach my $plex (@plex_list) {
                    my $zone = $error_info->{$plex}->{$rg}->{zones}->[$zoneid];
                    foreach my $stripeid (0..scalar(@{$zone->{stripes}})-1) {
                        my $stripe_offset = $zone->{stripes}->[$stripeid]->{stripe_offset};
                        if(!defined($stripe_offset)) {
                            $stripe_offset = $stripeid;
                        }
                        if(!grep /$stripe_offset/, @{$zones->[$index]}) {
                            push(@{$zones->[$index]}, $stripe_offset);
                        }
                    }
                }
            } else {
                #push plex0 into an array, increment index and then push plex1
                foreach my $plexid (0,1) {
                    my $zone = $error_info->{$plex_list[$plexid]}->{$rg}->{zones}->[$zoneid];
                    if(ref $zone) {
                        $Log->comment("get_unique_zones: Debug: Dumper: ".Dumper($zone));
                        $index += $plexid;
                        $zones->[$index] = [];
                        foreach my $stripeid (0..scalar(@{$zone->{stripes}})-1) {
                            my $stripe_offset = $zone->{stripes}->[$stripeid]->{stripe_offset};
                            if(!defined($stripe_offset)) {
                                $stripe_offset = $stripeid;
                            }
                            if(!grep /$stripe_offset/, @{$zones->[$index]}) {
                                push(@{$zones->[$index]}, $stripe_offset);
                            }
                        }
                    }
                }
            }
            $zoneid++;
        }

        return $zones;
    }

    sub dump_tc_data {
        my $self = shift;
        my %opts = validate(@_, {
            ignore_aggr_info => {type => SCALAR, default => 0, optional => 1},
            ignore_file_info => {type => SCALAR, default => 0, optional => 1},
        });


        my $tc_data = dclone($self->get_tc_data());

        #remove api_set_obj
        delete $tc_data->{api_set_obj} if(exists $tc_data->{api_set_obj});

        #fix aggr objects
        foreach my $index (0..(scalar(@{$tc_data->{aggregates}})-1) ) {
            delete $tc_data->{aggregates}->[$index]->{api_set_obj}
                if(exists($tc_data->{aggregates}->[$index]->{api_set_obj}));
        }

    }
## ======================= End of RaidErrUtils =======================

package AggrCreate;
use strict;
use warnings;
use vars qw(@ISA);

# Module Imports
use TCD;
use Data::Dumper;
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidSqav3 qw(AUTOLOAD);
use Tharn qw($Log);
use NACL::APISet;
use Storable qw(dclone);

BEGIN {
    @ISA = qw(commonUtils);
}

    sub new {
        my $self = undef;
        $self = shift if(scalar(@_) % 2);
        my %opts = validate(@_, {
            api_set_obj    => {type => OBJECT, isa => "NACL::APISet"},
            aggregate_data => {type => HASHREF},
            io_type        => {type => SCALAR|UNDEF, default=>undef, optional=>1},
            plex_count     => {type => SCALAR},
        });

        $self = dclone($opts{aggregate_data});
        $self->{api_set_obj} = $opts{api_set_obj};
        $self->{io_type}     = $opts{io_type};
        $self->{plex_count}  = $opts{plex_count};

        bless $self, 'AggrCreate';
        return $self;
    }

    sub exists {
        my $self = shift;
        my $filer_vol = $self->get_filer_vol();
        my $api_set_obj = $self->{api_set_obj};

        $Log->comment("aggr::exists: Info: check existence of $filer_vol");
        if (RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => 0)
            or RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => 1)) {
            return 1;
        }
    }

    sub create {
        my $self = shift;
        my $filer_vol            = $self->get_filer_vol();
        my $api_set_obj          = $self->{api_set_obj};
        my $mirror               = $self->is_mirror();
        my @rgs                  = @{$self->get_rgs()};
        my $aggr_create_sequence = $self->get_aggr_create_sequence();
        $mirror                  = 0 if (defined($aggr_create_sequence));

        $Log->comment("create_aggr: Debug: Mirror value - $mirror");
        $Log->comment("create_aggr: Info: Creating new test volume - ".$filer_vol);

        if (!RaidSqa_aggr_create(
                api_set_obj   => $api_set_obj,
                filer_vol     => $filer_vol,
                raid_type     => $self->get_raid_type(),
                raid_size     => $self->get_raid_size(),
                aggr_type     => $self->get_aggr(),
                aggr_bits     => $self->get_aggr_bits(),
                vserver       => $self->get_vserver(),
                disk_count    => $rgs[0]->{disk_count} * ($mirror+1),
                checksum_type => $rgs[0]->{checksum_type},
                storage_type  => $rgs[0]->{storage_type},
                mirror        => $mirror)) {
            die("create: Error: Aggr create failed to create $filer_vol");
        } else {
            $Log->comment("create: Info: Aggr $filer_vol created successfully");
        }
    }

    sub destroy {
        my $self = shift;
        my $api_set_obj = $self->{api_set_obj};
        # Reset the value of 'raid_disable_new_recons' for disks to get Online successfully
        RaidSqa_setflag(api_set_obj => $api_set_obj,flag_name => 'raid_disable_new_recons',flag_value => 0);
        foreach my $aggr_type ( 0, 1 ) {
            # RaidSqa_test_init( host => $Host,  filer_vol => $filer_vol, aggr_type => $aggr_type);
            RaidSqa_test_init( api_set_obj => $api_set_obj, filer_vol => $self->get_filer_vol(), aggr_type => $aggr_type, destroy_inconsistent_aggrs=>1);
        }
        $self->set_aggr_info(aggr_info => {});
        if($self->exists()) { die ("destroy: Error: Aggr remove failed for aggr - ".$self->get_filer_vol()); }
    }

    sub add_raid_groups {
        my $self = shift;
        my (%opts) = validate(@_,{
                                raid_group => {type=>ARRAYREF | UNDEF, default=>undef, optional=>1},
                                mirror     => {type=>SCALAR, default=>$self->get_mirror(), optional=>1},
                            });

        my $api_set_obj = $self->{api_set_obj};
        my $raid_group  = $opts{raid_group};
        my $mirror      = $opts{mirror};
        my $filer_vol   = $self->get_filer_vol();

        my @rgs = ();
        if (defined($raid_group)) {
            @rgs = @$raid_group;
        } else {
            @rgs = @{$self->get_rgs()};
            shift @rgs;     #Ignore first rg as it is used to create an aggregate
        }

        foreach my $rg_info (@rgs) {
            if(!defined($rg_info->{disk_count})){
                die("add_raid_groups: Error: Disk count is required for aggr add operation") ;
            }
            my $options  = (defined($rg_info->{checksum_type})) ? "-f -c $rg_info->{checksum_type}" : '-f';
            $options .= " -T $rg_info->{storage_type}" if(defined($rg_info->{storage_type}));

            my $result = RaidSqa_aggr_add(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol,
                            group_name=>'new', disk_count=> int($rg_info->{disk_count}) * ($mirror+1), options=>$options);

            if(!$result) {      #TODO: result returned by RaidSqa is not correct, it returns true even in case of failure
                die("add_raid_groups: Aggr add failed");
            }
        }

        #Wait for aggr disks to complete zeroing
        RaidSqa_wait_for_aggr_disks_zeroing_completion(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
        #resize the volume so that files will be created on each raid group
        my $usable_size = $self->get_aggr_usable_space();
        my $cmd = "priv set -q diag; vol size $filer_vol $usable_size";
        RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd => $cmd);
    }
    # # TODO : Melan #####################################################################################################
    # sub aggr_mirror_split {
        # my $self = shift;
        # my (%opts) = validate(@_,{
                                # filer_vol => {type=>SCALAR, default=>$self->get_filer_vol(), optional=>1},
                                # splitted_vol => {type=>SCALAR | UNDEF, default=>undef, optional=>1}
                             # });
        # my $filer_vol = $opts{filer_vol};
        # my $splitted_vol = $opts{splitted_vol};
        # $splitted_vol = $filer_vol."_split" if (!defined($splitted_vol));

        # my @plex_list = $self->get_aggr_info_plexes(filer_vol=>$filer_vol);
        # $Log->comment("Dumper: plex_list".Dumper(\@plex_list));
        # if (@plex_list < 2) {
            # die("aggr_mirror_split: Need atleast two plexes for mirror split");
        # }
        # $Log->comment("aggr_mirror_split: Split mirrored aggregate-$filer_vol using plex-$plex_list[1] into splitvol-$splitted_vol");
        # my $return = RaidSqa_aggr_split(api_set_obj => $self->{api_set_obj},plex_name=>$plex_list[1], splitted_vol=>$splitted_vol);
        # if (!$return) {
            # die("aggr_mirror_split: Failed to split mirrored aggregate-$filer_vol using plex-$plex_list[1]");
        # }
        # # TODO : Update tc_data with new split vol and and its disk_list
        # # TODO : get plex list for split vol
        # # TODO : get rg list for split vol
        # # TODO : get disk list based on plex and rg
        # # TODO : Update tc_data
    # }

    # sub aggr_mirror_rejoin {
        # my $self = shift;
        # my (%opts) = validate(@_,{
                                # filer_vol => {type=>SCALAR, default=>$self->get_filer_vol(), optional=>1},
                                # splitted_vol => {type=>SCALAR, default=><TODO : Get it from tc_data>, optional=>1},
                                # disk_list => {type=>ARRAYREF, optional=>1},
                             # });
        # my $filer_vol = $opts{filer_vol};
        # my $splitted_vol = $opts{splitted_vol};
        # my $disk_list = $opts{disk_list};
        # my $return = RaidSqa_aggr_mirror(api_set_obj => $self->{api_set_obj}, filer_vol=>$filer_vol, splitted_vol=>$splitted_vol,disk_list=>$disk_list);
        # if (!$return) {
            # die("aggr_mirror_rejoin: Failed to rejoin splitvol-$splitted_vol with aggregate-$filer_vol");
        # }
        # # TODO : Update tc_data by removing split vol and and its disk_list and update filer_vol data with new plex list/rg list/disk list
    # }
    # #####################################################################################################

    sub enable_hybrid_aggr {
        my $self = shift;
        my $filer_vol = $self->get_filer_vol();

        if($self->is_hybrid_aggregate()) {
            # if(!$self->check_hybrid_aggr_bootargs()) {
                # die("enable_hybrid_aggr: Error: Bootargs options for hybrid_aggregates is not set to 'on'");
            # }
            # if(!$self->check_hybrid_aggr_license()) {
                # die("enable_hybrid_aggr: Error: License for hybrid_aggregates is not installed");
            # }
            if(!RaidSqa_aggr_options(
                    api_set_obj => $self->{api_set_obj},
                    filer_vol   => $filer_vol,
                    options     => 'hybrid_enabled',
                    value       => 'on')){
                die("enable_hybrid_aggr: Error: Set option \"hybrid_enabled\" to \"on\" for aggr $filer_vol failed");
            }
        } else {
            $Log->comment("enable_hybrid_aggregate: Info: This aggr is not a hybrid aggr");
        }
    }

    # sub check_hybrid_aggr_bootargs {
        # my $self = shift;
        # my $bootarg = "bootarg.dblade.hybrid_aggregates";
        # my $output = RaidSqa_filer_cmd(api_set_obj => $self->{api_set_obj}, cmd=>"bootargs print $bootarg");
        # if($output =~ /$bootarg.*(true|1)/) {
            # return 1;
        # } else {
            # return 0;
        # }
    # }

    # sub check_hybrid_aggr_license {
        # my $self = shift;
        # if(RaidSqa_check_license(api_set_obj=>$self->{api_set_obj}, service=>'hybrid_aggregates')){
            # return 1;
        # } else {
            # return 0;
        # }
    # }

    sub apply_aggr_options {
        my $self = shift;

        my $aggr_options = $self->get_aggr_options();
        foreach my $option (keys %{$aggr_options}) {
            if(!RaidSqa_aggr_options(
                    api_set_obj => $self->{api_set_obj}, filer_vol   => $self->get_filer_vol(),
                    options     => $option, value       => $aggr_options->{$option})) {
                die("set_aggr_options: Setting option $option to $aggr_options->{option} failed for". $self->get_filer_vol());
            }
        }
    }

    sub refresh_aggr_info {
        my $self = shift;
        my $api_set_obj = $self->{api_set_obj};
        my $filer_vol   = $self->get_filer_vol();

        $Log->comment("refresh_aggr_info: Info: updating tc_data for filer_vol - $filer_vol");

        # create aggr_info stucture
        my %aggr_info = ();
        my @plex_list = RaidSqa_get_plex_list(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol);
        foreach my $plex (@plex_list){
            my @group_list = RaidSqa_get_group_list(api_set_obj=>$api_set_obj, plex_name=>$plex);
            foreach my $group (@group_list) {
                my %group_disks = ();
                $group_disks{data_disk_list} = [RaidSqa_get_disk_list_by_raid_type(api_set_obj=>$api_set_obj, group_name=>$group, raid_type=>"data")];
                $group_disks{parity_disk_list} = [RaidSqa_get_disk_list_by_raid_type(api_set_obj=>$api_set_obj, group_name=>$group, raid_type=>"parity")];
                $group_disks{dparity_disk_list} = [RaidSqa_get_disk_list_by_raid_type(api_set_obj=>$api_set_obj, group_name=>$group, raid_type=>"dparity")];
                foreach my $disk_list (keys(%group_disks)) {
                    # adds all types of disks to super set - disk_list
                    push(@{$group_disks{disk_list}}, @{$group_disks{$disk_list}});
                    # Initially update 'other_disk_list' same as 'disk_list'
                    push(@{$group_disks{other_disk_list}}, @{$group_disks{$disk_list}});
                }

                # update disk rgdn details to aggr_info
                $group_disks{disk_rgdn_mapping} = $self->get_disks_rgdn(disks => $group_disks{disk_list} );
                $aggr_info{$filer_vol}->{$plex}->{$group} = \%group_disks;
            }
        }

        $Log->comment("refresh_aggr_info: Info: Dumper aggr_info: ".Dumper(\%aggr_info));
        $self->set_aggr_info(aggr_info=> \%aggr_info);
    }

    sub get_disk_raid_group {
        my $self = shift;
        my (%opts) = validate(@_,{
            aggr_info => {type=>HASHREF, default=>$self->get_aggr_info(), optional=>1},
            disk_name => {type=>SCALAR}
        });

        my $aggr_info = $opts{aggr_info};
        my $disk_name = $opts{disk_name};
        my $group_name = undef;

        $Log->comment("get_disk_raid_group: Debug: Input disk name - ". $disk_name);

        foreach my $aggr (keys %$aggr_info) {
            my @plex_list = $self->get_aggr_info_plexes(aggr_info=>$aggr_info, filer_vol=>$aggr);
            foreach my $plex (@plex_list) {
                my @rg_list = $self->get_aggr_info_rgs(aggr_info=>$aggr_info, filer_vol=>$aggr, plex=>$plex);
                foreach my $rg (@rg_list) {
                    next if(ref($aggr_info->{$aggr}->{$plex}->{$rg}->{disk_list}) !~ /ARRAY/i);
                    my @disk_list = @{$aggr_info->{$aggr}->{$plex}->{$rg}->{disk_list}};
                    if(grep /$disk_name/, @disk_list) {
                        $group_name = $rg;
                    }
                }
            }
        }

        if(defined($group_name)) {
            return $group_name;
        } else {
            $Log->comment("get_disk_raid_group: Debug: aggr_info - ".Dumper($aggr_info));
            die("get_disk_raid_group: Error: Raid group name could not be found for disk - $disk_name");
        }
    }

    sub get_aggr_info_plexes{
        my $self =shift;
        my (%opts) = validate(@_,{
                                aggr_info => {type=>HASHREF, default=>$self->get_aggr_info(), optional=>1},
                                filer_vol => {type=>SCALAR, default=>$self->get_filer_vol(), optional=>1},
                             });
        my $aggr_info = $opts{aggr_info};
        my $filer_vol = $opts{filer_vol};

        my @plex_list = keys %{$aggr_info->{$filer_vol}};
        return @plex_list;
    }

    sub get_aggr_info_rgs {
        my $self =shift;
        my (%opts) = validate(@_,{
                                aggr_info => {type=>HASHREF, default=>$self->get_aggr_info(), optional=>1},
                                filer_vol => {type=>SCALAR, default=>$self->get_filer_vol(), optional=>1},
                                plex => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                             });
        my $aggr_info = $opts{aggr_info};
        my $filer_vol = $opts{filer_vol};
        my $plex = $opts{plex};

        my @rg_list = ();

        # foreach my $plex ( keys %{$aggr_info{$filer_vol}}){
        if(!defined($plex)){
            foreach $plex ( $self->get_aggr_info_plexes(aggr_info=>$aggr_info, filer_vol=>$filer_vol)){
                push(@rg_list, keys %{$aggr_info->{$filer_vol}->{$plex}});
            }
        }else{
            push(@rg_list, keys %{$aggr_info->{$filer_vol}->{$plex}});
        }
        return @rg_list;
    }

    sub get_aggr_info_disks {
        my $self = shift;
        my (%opts) = validate(@_, {
            filer_vol  => {type=>SCALAR|UNDEF, default=>$self->get_filer_vol()},
            plex    => {type=>SCALAR},      # plex format : plex0, plex1
            rg  => {type=>SCALAR},          # rg format    : rg0, rg1
            disk_type  => {type=>SCALAR},   # disks_type format: all, data, parity, dparity, fail, offline, reconstruct
        });

        my $filer_vol = $opts{filer_vol};
        my $plex = $opts{plex};
        my $rg = $opts{rg};
        my $disk_type = $opts{disk_type};
        my $aggr_info = $self->get_aggr_info();

        $Log->comment("get_aggr_info_disks: Info: Input, $filer_vol, $plex, $rg, disk_type - $disk_type");
        $disk_type = ($disk_type eq "all") ? "disk_list" : lc $disk_type."_disk_list";

        my @plexes = grep(/^\/$filer_vol\/$plex$/, keys %{$aggr_info->{$filer_vol}});
        my @rgs = grep(/^\/$filer_vol\/$plex\/$rg$/, keys %{$aggr_info->{$filer_vol}->{$plexes[0]}});
        my $retval = [];
        if(ref($aggr_info->{$filer_vol}->{$plexes[0]}->{$rgs[0]}->{$disk_type}) =~ /ARRAY/i){
            $retval = $aggr_info->{$filer_vol}->{$plexes[0]}->{$rgs[0]}->{$disk_type};
        }

        $Log->warn("get_aggr_info_disks: Warning: disks not found of type - $disk_type") if(!scalar(@$retval));
        return $retval;
    }

    sub add_aggr_info_disks {
        my $self = shift;
        my (%opts) = validate(@_, {
                                filer_vol  => {type=>SCALAR|UNDEF, default=>$self->get_filer_vol()},
                                plex    => {type=>SCALAR},      # plex format : plex0, plex1
                                rg  => {type=>SCALAR},          # rg format    : rg0, rg1
                                disk_type  => {type=>SCALAR},   # disks_type format: all, data, parity, dparity, fail, offline, reconstruct
                                disks => {type=>ARRAYREF}
                                });

        my $filer_vol = $opts{filer_vol};
        my $plex = $opts{plex};
        my $rg = $opts{rg};
        my $disk_type = $opts{disk_type};
        my @disks = @{$opts{disks}};
        my $aggr_info = $self->get_aggr_info();

        $disk_type = ($disk_type eq "all") ? "disk_list" : lc $disk_type."_disk_list";

        my @plexes = grep(/^\/$filer_vol\/$plex$/, keys %{$aggr_info->{$filer_vol}});
        my @rgs = grep(/^\/$filer_vol\/$plex\/$rg$/, keys %{$aggr_info->{$filer_vol}->{$plexes[0]}});
        my $disk_type_ref = !defined($aggr_info->{$filer_vol}->{$plexes[0]}->{$rgs[0]}->{$disk_type}) ? [] :
                                 $aggr_info->{$filer_vol}->{$plexes[0]}->{$rgs[0]}->{$disk_type};

        push(@{$disk_type_ref}, @disks);
        $aggr_info->{$filer_vol}->{$plexes[0]}->{$rgs[0]}->{$disk_type} = $disk_type_ref;
        $self->set_aggr_info(aggr_info=>$aggr_info);
    }

    sub remove_aggr_info_disks{
        my $self = shift;
        my (%opts) = validate(@_, {
                                filer_vol  => {type=>SCALAR|UNDEF, default=>$self->get_filer_vol()},
                                plex       => {type=>SCALAR},       # plex format : plex0, plex1
                                rg         => {type=>SCALAR},       # rg format    : rg0, rg1
                                disk_type  => {type=>SCALAR},       # disks_type format: all, data, parity, dparity, fail, offline, reconstruct
                                disks      => {type=>ARRAYREF}
                                });

        my $filer_vol = $opts{filer_vol};
        my $plex = $opts{plex};
        my $rg = $opts{rg};
        my $disk_type = $opts{disk_type};
        my %remove_disks = map {$_, 1} @{$opts{disks}};
        # my $aggr_info = \%aggr_info;#self->get_aggr_info();
        my $aggr_info = $self->get_aggr_info();

        $disk_type = ($disk_type eq "all") ? "disk_list" : lc $disk_type."_disk_list";

        my @plexes = grep(/^\/$filer_vol\/$plex$/, keys %{$aggr_info->{$filer_vol}});
        my @rgs = grep(/^\/$filer_vol\/$plex\/$rg$/, keys %{$aggr_info->{$filer_vol}->{$plexes[0]}});
        my $disk_type_ref = $aggr_info->{$filer_vol}->{$plexes[0]}->{$rgs[0]}->{$disk_type};
        if(!defined($disk_type_ref)) {$disk_type_ref = []};

        $disk_type_ref = [grep(!$remove_disks{$_}, @$disk_type_ref)];

        $aggr_info->{$filer_vol}->{$plexes[0]}->{$rgs[0]}->{$disk_type} = $disk_type_ref;
        $self->set_aggr_info(aggr_info=>$aggr_info);
    }

    sub get_disk_list {
        # TODO: Rarely used . See if this function can be deprecated
        # instead use get_aggr_info_disks
        my $self =shift;
        my (%opts) = validate(@_,{
                            aggr_info => {type=>HASHREF, default=>$self->get_aggr_info(), optional=>1},
                            disk_type=>{type=>SCALAR},
                            raid_group=>{type=>SCALAR},
                         });

        my $aggr_info = $opts{aggr_info};
        my $disk_type = $opts{disk_type};
        my $raid_group = $opts{raid_group};
        my @disk_list = ();

        $disk_type = ($disk_type eq "all") ? "" : lc $disk_type."_";
        my $key = "$disk_type"."disk_list";

        foreach my $aggr (keys %$aggr_info) {
            foreach my $plex (keys %{$aggr_info->{$aggr}}) {
                if(defined($aggr_info->{$aggr}->{$plex}->{$raid_group})) {
                    # @disk_list = $plex->{$raid_group}->{$key};
                    return $aggr_info->{$aggr}->{$plex}->{$raid_group}->{$key};
                }
            }
        }
        return @disk_list;
    }

    sub get_disk_type {
        my $self = shift;
        my (%opts) = validate(@_,{
                                aggr_info => {type=>HASHREF, default=>$self->get_aggr_info(), optional=>1},
                                disk_name => {type=>SCALAR}
                            });

        my $disk_name = $opts{disk_name};
        my $aggr_info = $opts{aggr_info};

        foreach my $aggr (keys %$aggr_info) {
            # my @plex_list = keys %{$aggr_info->{$aggr}};
            my @plex_list = $self->get_aggr_info_plexes(aggr_info=>$aggr_info, filer_vol=>$aggr);
            foreach my $plex (@plex_list) {
                my @rg_list = keys %{$aggr_info->{$aggr}->{$plex}};
                foreach my $rg (@rg_list) {
                    foreach my $disk_type ("data", "parity", "dparity") {
                        my @disk_list = @{$aggr_info->{$aggr}->{$plex}->{$rg}->{"$disk_type"."_disk_list"}};
                        if(grep /^$disk_name$/, @disk_list) {
                            return $disk_type;
                        }
                    }
                }
            }
        }
    }

    sub get_reconstructing_disks {
        my $self =shift;
        my $api_set_obj = $self->{api_set_obj};
        my %aggr_info = %{$self->get_aggr_info()};
        my $filer_vol = $self->get_filer_vol();

        # X = Get disk list before in a particular plex/rg before recons
        # Y = Get all disks in a particular plex/rg after recons
        # F = Failed disk
        # X1 = X - F  # no need to discount offlined disks as they are not replaced.
        # Z = Y - X1   # these are the newly reconstructed disks
        # call get_disk_rgdn() for each of Z and then map it to F.
        # return new old mapping in format - {new=>old}

        foreach my $rg_name ($self->get_aggr_info_rgs()){
            $rg_name =~ /\/$filer_vol\/(plex\d)\/(rg\d+)/;
            my $plex = $1;
            my $rg = $2;

            my @all_disks_after_recons = RaidSqa_get_disk_list(api_set_obj=>$api_set_obj, group_name => $rg_name);
            my @failed_disks = @{$self->get_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>'fail')};
            my @non_failed_disks = @{$self->get_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>'other')};

            # Remove non-failed disks from all_disks_after_recons
            foreach my $disk_to_remove (@non_failed_disks){
                @all_disks_after_recons = @{$self->remove_array_item(array=>\@all_disks_after_recons, item=>$disk_to_remove)};
            }

            $Log->comment("get_reconstructing_disks: Debug: new reconstruced disks for plex - $plex, rg - $rg: ".Dumper(\@all_disks_after_recons));

            # @all_disks_after_recons is now the list of new reconstructed disks
            my $aggr_info_disk_rgdn_mapping = $self->get_aggr_info_disk_rgdn_mapping(plex_name=>"/$filer_vol/$plex", rg_name=>$rg_name);
            my $new_recons_disks_rgdn = $self->get_disks_rgdn(disks=>\@all_disks_after_recons);

            $Log->comment("get_reconstructing_disks: Debug: aggr_info_disk_rgdn_mapping for plex - $plex, rg - $rg: ".Dumper($aggr_info_disk_rgdn_mapping));
            $Log->comment("get_reconstructing_disks: Debug: new reconstructed disks rgdn for plex - $plex, rg - $rg: ".Dumper($new_recons_disks_rgdn));

            my %reconstruct_disk_mapping = ();
            foreach my $new_recons_disk (@all_disks_after_recons){
                my $new_rgdn = $new_recons_disks_rgdn->{$new_recons_disk};
                foreach my $failed_disk (@failed_disks){
                    my $old_rgdn = $aggr_info_disk_rgdn_mapping->{$failed_disk};
                    if($old_rgdn == $new_rgdn){
                        $reconstruct_disk_mapping{$new_recons_disk} = $failed_disk;
                    }
                }
            }
            $self->set_aggr_info_reconstruct_disk_mapping(plex_name=>"/$filer_vol/$plex", rg_name=>$rg_name, reconstruct_disk_mapping=>\%reconstruct_disk_mapping);
        }
        $Log->comment("get_reconstructing_disks: Debug: updated aggr_info for reconstructing disks: ".Dumper($self->get_aggr_info()));
    }

    sub get_aggr_info_disk_rgdn_mapping{
        my $self =shift;
        my (%opts) = validate(@_,{
                                aggr_info => {type=>HASHREF, default=>$self->get_aggr_info(), optional=>1},
                                filer_vol => {type=>SCALAR, default=>$self->get_filer_vol(), optional=>1},
                                plex_name => {type=>SCALAR},
                                rg_name => {type=>SCALAR},
                             });

        my $aggr_info = $opts{aggr_info};
        my $filer_vol = $opts{filer_vol};
        my $plex_name = $opts{plex_name};
        my $rg_name = $opts{rg_name};

        if(ref($aggr_info->{$filer_vol}->{$plex_name}->{$rg_name}->{disk_rgdn_mapping}) =~ /HASH/i){
            return $aggr_info->{$filer_vol}->{$plex_name}->{$rg_name}->{disk_rgdn_mapping};
        }
        return {};
    }

    sub set_aggr_info_reconstruct_disk_mapping{
        my $self =shift;
        my (%opts) = validate(@_,{
                                aggr_info => {type=>HASHREF, default=>$self->get_aggr_info(), optional=>1},
                                filer_vol => {type=>SCALAR, default=>$self->get_filer_vol(), optional=>1},
                                plex_name => {type=>SCALAR},
                                rg_name => {type=>SCALAR},
                                reconstruct_disk_mapping => {type=>HASHREF},
                             });

        my $aggr_info = $opts{aggr_info};
        my $filer_vol = $opts{filer_vol};
        my $plex_name = $opts{plex_name};
        my $rg_name = $opts{rg_name};
        my $reconstruct_disk_mapping = $opts{reconstruct_disk_mapping};

        $aggr_info->{$filer_vol}->{$plex_name}->{$rg_name}->{reconstruct_disk_mapping} = $reconstruct_disk_mapping;
        $self->set_aggr_info(aggr_info=> \%{$aggr_info});
        return {};
    }

    sub get_disks_rgdn {
        my $self =shift;
        my (%opts) = validate(@_, {
                                    disks => {type=>ARRAYREF},
                                });

        my $api_set_obj = $self->{api_set_obj};
        my @disks = @{$opts{disks}};
        my $raid_info_object = $self->get_raid_info_object();

        my %disks_rgdn_mapping = ();
        foreach my $disk (@disks){
            my $disk_obj = $raid_info_object->disk("$disk");
            if(!defined($disk_obj)) {
                my $active_disk_name = $self->get_disk_active_path(disk_name=>$disk, raid_object=>$raid_info_object);
                $disk_obj = $raid_info_object->disk("$active_disk_name");
                if(!defined($disk_obj)){ die("get_disks_rgdn: Error: disk with name $disk not found"); }
            }
            $Log->comment("get_disks_rgdn: Info: getting disk rgdn for disk - $disk");
            $disks_rgdn_mapping{$disk} = $disk_obj->parameter("disk_rgdn");
        }
        return \%disks_rgdn_mapping;
    }

    sub is_rg_degraded{
        my $self = shift;
        my (%opts) = validate(@_, {
                                filer_vol => {type=>SCALAR, default=>$self->get_filer_vol(), optional=>1},
                                raid_type => {type => SCALAR, default=>$self->get_raid_type(), optional=>1},
                                rg_name => {type => SCALAR | UNDEF, default=>undef, optional=>1},
                            });
        my $filer_vol = $opts{filer_vol};
        my $rg_name = $opts{rg_name};
        my $raid_type = $opts{raid_type};

        my @group_list = ($rg_name);
        if(! defined $rg_name){
            @group_list  = $self->get_aggr_info_rgs();
        }

        foreach my $rg_name ( @group_list ){
            $rg_name =~ /\/$filer_vol\/(plex\d)\/(rg\d+)/;
            my $plex = $1;
            my $rg = $2;
            my $failed_disks = $self->get_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>'fail');
            my $offlined_disks = $self->get_aggr_info_disks(filer_vol=>$filer_vol, plex=>$plex, rg=>$rg, disk_type=>'offline');

            #my $sum_of_disks = scalar(@{$failed_disks}) + scalar(@{$offlined_disks});
            my $sum_of_disks = 0;
            $sum_of_disks = scalar(@{$failed_disks}) if (defined $failed_disks);
            $sum_of_disks += scalar(@{$offlined_disks}) if (defined $offlined_disks);
            if($raid_type eq 'raid4'){
                $sum_of_disks >= 1 ? return 1 : next;
            }
            if($raid_type eq 'raid_dp'){
                $sum_of_disks >= 2 ? return 1 : next;
            }
        }
        return 0;
    }

    sub get_disk_count {
        my $self = shift;
        my %opts = validate(@_, {
                        plex_name => {type => SCALAR|UNDEF, default => undef, optional => 1},
                        rg_name => {type => SCALAR|UNDEF, default => undef, optional => 1},
                    });

        my $filer_vol = $self->get_filer_vol();
        my $aggr_info = $self->get_aggr_info();

        my @rg_list = ();

        if(defined($opts{rg_name})) {
            @rg_list = ($opts{rg_name});
        } elsif(defined($opts{plex_name})) {
            @rg_list = keys %{$aggr_info->{$filer_vol}->{$opts{plex_name}}};
        } else {
            foreach my $plex_name (keys %{$aggr_info->{$filer_vol}}) {
                push(@rg_list, keys %{$aggr_info->{$filer_vol}->{$plex_name}});
            }
        }

        my @disk_list = ();
        foreach my $rg_name (@rg_list) {
            $rg_name =~ /(\/$filer_vol\/plex\d+)\/(rg\d+)/;
            my $plex_name = $1;
            push(@disk_list, @{$aggr_info->{$filer_vol}->{$plex_name}->{$rg_name}->{disk_list}});
        }
        $Log->comment("get_disk_count: Info: filer_vol: $filer_vol, disk_list: @disk_list");
        return scalar(@disk_list);
    }

    sub get_plex_count{
        my $self = shift;
        my $aggr_info = $self->get_aggr_info();
        my $filer_vol = $self->get_filer_vol();
        return keys %{$aggr_info->{$filer_vol}}
    }


    #TODO: Algo:
        #   - get rgs
        #   - Foreach rg
        #       - get_disk_count
        #       - prepare a hash with all parameters similar to ones in spare_setup_info
        #       - traverse through each hash in total_spare_requirements and check if this set of requirement is already defined
        #       - if defined, add disk_count for this rg to required_disk_count of already defined hash
        #           - Note: We will not consider as spare_setup_info is defined non-mirror case, and aumatically checks for other pool if mirror
        #       - Else, add this new requirement to total_spare_requirements array
    sub get_spare_requirements {
        my $self = shift;
        my %opts = validate(@_, {total_spare_requirements => {type => ARRAYREF}});
        my @requirements = @{$opts{total_spare_requirements}};
        if(!defined $opts{total_spare_requirements}) {
            $opts{total_spare_requirements} = [];
        }

        my @params = ('storage_type', 'checksum_type');
        my @rgs = @{$self->get_rgs()};

        foreach my $rg (@rgs){
            my $hash_entry = 0;
            foreach my $index (0..scalar(@requirements)-1) {
                my $match = 1;
                foreach my $key (@params) {
                    if($rg->{$key} ne $requirements[$index]->{$key}) {$match = 0;}
                }
                if(!$match){ next;}
                $hash_entry = 1;
                $Log->comment("get_spare_requirements: Debug: Updating spare requirement count by "
                                .$rg->{disk_count}." for requirement .".Dumper($requirements[$index]));

                $requirements[$index]->{min_spare_count} += $rg->{disk_count};
                #PRA: requirement
                    # Do we need to update max spare count also???
                if(exists($requirements[$index]->{max_spare_count})) {
                    $requirements[$index]->{max_spare_count} += $rg->{disk_count};
                }
            }

            #PRA: Added new log comments to track what are we adding here
            if(!$hash_entry) {
                $Log->comment("get_spare_requirements: Info: Adding new hash entry to spares");
                my $rg_requirement = {};
                foreach my $key (@params) {
                    if(defined($rg->{$key})) {
                        $rg_requirement->{$key} = $rg->{$key};
                        $Log->comment("get_spare_requirements: Info: new hash entry, $key, $rg->{$key}");
                    }
                }
                $Log->comment("get_spare_requirements: Info: new hash entry, min & max disk count, $rg->{'disk_count'}");
                $rg_requirement->{'min_spare_count'} = $rg->{'disk_count'};
                $rg_requirement->{'max_spare_count'} = $rg->{'disk_count'};
                push(@requirements, $rg_requirement);
            }
        }
        return \@requirements;
    }

#================= File create subroutines ===============

    #input parameters required:
       # zone_count     -
       # stripe_gap     -
       # stripe_count   - For BCS aggrs where there are no zone, for zoned_checksum this is not applicable
    #This method create a file object and then creates
    sub create_test_file {
        my $self = shift;
        my $file = $self->get_fileCreate();
        $file->create_test_file();
    }

    sub get_rg_count {
        my $self = shift;
        my @rg_list = $self->get_aggr_info_rgs();
        return scalar(@rg_list);
    }

    sub validate_aggr_data {
        my $self = shift;
        my $api_set_obj = $self->{api_set_obj};
        my $plex_count = $self->{plex_count};
        $Log->comment("plex_count - $plex_count");
        $plex_count = 1 if ((!defined($plex_count)) || ($plex_count == 0));
        $Log->comment("plex_count - $plex_count");
        my ($default);

        #======================================================
        #validate filer_vol
        $Log->comment("Starting validation of filer_vol....");
        my $filer_vol = $self->get_filer_vol();
        if(!defined($filer_vol)) {
            die("Validated filer_vol: Error: filer_vol is not defined: $filer_vol");
        }
        $Log->comment("Validated filer_vol: $filer_vol");
        $Log->comment("Validated filer_vol: OK");

        #======================================================
        #validate vserver
        $Log->comment("Starting validation of vserver....");
        my $vserver = $self->get_vserver();
        if(!defined($vserver)) {
            $vserver = "vs0";
            $self->set_vserver(vserver=>$vserver);
            $Log->comment("Validated vserver: Info: vserver is not defined. Changing to default - $vserver");
        }
        $Log->comment("Validated vserver: $vserver");
        $Log->comment("Validated vserver: OK");


        #======================================================
        #validate mirror
        $Log->comment("Starting validation of mirror....");
        my $mirror = $self->get_mirror();
        if(!defined($mirror)) {
            $mirror = ($plex_count == 2) ? 1 : 0;               #PRA: For any number of plex other than two, we should not have mirror set true
            $Log->comment("Validated mirror: Info: mirror is not defined. Changing to default - $mirror");
        }
        # elsif($mirror != ($plex_count-1) && ($plex_count > 0)) {  #PRA: plex_count - 1 should only be done when plex_count > 0, to avoid ending up into negative number
            # $Log->comment("Validated mirror: Info: value for mirror is not correct - $mirror. Changing to default - ".($plex_count - 1));
            # $mirror = $plex_count - 1;
        # }

        $self->set_mirror(mirror=>$mirror);
        $Log->comment("Validated mirror: $mirror");
        $Log->comment("Validated mirror: OK");

        #======================================================
        #validate raidio

        $Log->comment("Starting validation of raidio....");
        my $raidio = $self->get_raidio();
        my $io_type = $self->get_io_type();
        my @io_list = ();
        if(defined($io_type)) {@io_list = split(/,/, $io_type);}
        my $raidio_comment = undef;
        if(!(defined($raidio) or defined($io_type))) {
            die("Validated raidio: Info: raidio and io_type both are not defined");
        } elsif(defined($io_type)){
            $raidio = \@io_list;
            $Log->comment("Validated raidio: Info: io_type is defined. Changing to default - $io_type");
        }

        #validate all raidios are valid
        my @valid_raidio = ('waflio', 'scrubio', 'reconsio', 'resyncio', 'verifyio', 'reparityio', 'replayio', 'nvfile_replayio');
        my %raidio_hash = map {$_, 1} @valid_raidio;
        my @invalid_raidio = grep (!$raidio_hash{$_}, @$raidio);
        @valid_raidio = grep ($raidio_hash{$_}, @$raidio);
        #PRA: Fix
            # If we have invalid raidio as 0, it should not die, as we may not want to run any raidio
        if(scalar(@$raidio) == scalar(@invalid_raidio) && @invalid_raidio != 0) {
            die("Validated raidio: Error: None of raidio in raidio_list are valid: @invalid_raidio");
        } elsif(scalar(@invalid_raidio)) {
            $Log->comment("Validated raidio: Error: Some of raidio in raidio_list are invalid: @invalid_raidio");
        } else {
            $Log->comment("Validated raidio: raidio is valid: @valid_raidio");
        }

        $raidio = \@valid_raidio;
        $self->set_raidio(raidio=>$raidio);
        $Log->comment("Validated raidio: ".join(" ",@{$raidio}));
        $Log->comment("Validated raidio: OK");

        #======================================================
        #validate raid_type
        $Log->comment("Starting validation of raid_type....");
        my $raid_type = $self->get_raid_type();
        if(!defined($raid_type)) {
            $raid_type = "raid_dp";
            $Log->comment("Validated raid_type: Info: raid_type is not defined. Changing to default - $raid_type");
        } elsif ($raid_type !~ /^(raid_dp|raid4)$/) {
            $Log->comment("Validated raid_type: Info: raid_type is not correct, $raid_type. Changing to default - raid_dp");
            $raid_type = "raid_dp";
        }
        $self->set_raid_type(raid_type=>$raid_type);
        $Log->comment("Validated raid_type: $raid_type");
        $Log->comment("Validated raid_type: OK");

        #======================================================
        # validate checksum_type
        $Log->comment("Starting validation of checksum_type....");
        my $checksum_type = $self->get_checksum_type();
        if(!defined($checksum_type)) {
            $checksum_type = $self->get_checksum_type(rg_index => 0);
        }

        if(!defined($checksum_type)) {
            $checksum_type = 'block';
            $Log->comment("Validated checksum_type: Info: checksum_type is not defined. Changing to - block");
        }

        if($checksum_type !~ /^\s*(block|zoned|advanced_zoned|mixed)\s*$/i) {die("Validated checksum_type: Error: checksum_type is not correct - $checksum_type");}
        $self->set_checksum_type(checksum_type=>$checksum_type);
        $Log->comment("Validated checksum_type: $checksum_type");
        $Log->comment("Validated checksum_type: OK");

        #======================================================
        #validate raid_size
        $Log->comment("Starting validation of raid_size....");
        my $raid_size = $self->get_raid_size();

        my $pref_storage_type = RaidSqa_get_preferred_storage_type(api_set_obj=>$api_set_obj, checksum_type=>$checksum_type);
        my $max_raid_size = RaidSqa_get_raid_size_value(api_set_obj=>$api_set_obj, raid_size=>"max", raid_type=>$raid_type, storage_type=>$pref_storage_type);
        my $min_raid_size = ($raid_type eq "raid_dp") ? 3 : 2;

        if(!defined($raid_size)) {
            $raid_size = $min_raid_size;
        } elsif($raid_size =~ /^(min|default|max)$/) {
            $raid_size = RaidSqa_get_raid_size_value(api_set_obj=>$api_set_obj, raid_size=>$raid_size, raid_type=>$raid_type, storage_type=>$pref_storage_type);
        } elsif($raid_size < $min_raid_size || $raid_size > $max_raid_size) {
            die("Validated raid_size: Error: Invalid raid_size: $raid_size. Expected range [$min_raid_size-$max_raid_size]");
        }

        $self->set_raid_size(raid_size=>$raid_size);
        $Log->comment("Validated raid_size: $raid_size");
        $Log->comment("Validated raid_size: OK");

        #======================================================
        #validate aggr
        $Log->comment("Starting validation of aggr....");
        my $aggr = $self->get_aggr();
        if(!defined($aggr)) {
            $aggr = 1;
            $Log->comment("Validated aggr: Info: aggr is not defined. Changing to default - $aggr");
        } elsif($aggr != 0 and $aggr != 1) {
            $Log->comment("Validated aggr: Info: aggr is not correct, $aggr. Changing to default - 1");
            $aggr = 1;
        }
        $self->set_aggr(aggr=>$aggr);
        $Log->comment("Validated aggr: $aggr");
        $Log->comment("Validated aggr: OK");

        #======================================================
        #validate aggr_bits
        $Log->comment("Starting validation of aggr_bits....");
        my $aggr_bits = $self->get_aggr_bits();
        if(!defined($aggr_bits)) {
            $aggr_bits = 64;
            $Log->comment("Validated aggr_bits: Info: aggr_bits is not defined. Changing to default - $aggr_bits");
        } elsif($aggr_bits != 64 and $aggr_bits != 32) {
            $Log->comment("Validated aggr_bits: Info: aggr_bits is not correct, $aggr_bits. Changing to default - 64");
            $aggr_bits = 64;
        }
        $self->set_aggr_bits(aggr_bits=>$aggr_bits);
        $Log->comment("Validated aggr_bits: $aggr_bits");
        $Log->comment("Validated aggr_bits: OK");

        #======================================================
        # validate plex_offline
        $Log->comment("Starting validation of plex_offline....");
        my $plex_offline = $self->get_plex_offline();
        if(!defined($plex_offline)) {
            $Log->comment("Validated plex_offline: Info: plex_offline is not defined. Changing to default - 'undef'");
            $plex_offline = 'undef';
        }elsif($plex_offline !~ /^\s*(plex0|plex1|plex3|undef)\s*$/i) {
            $Log->comment("Validated plex_offline: Info: plex_offline is not valid - $plex_offline.  Changing to default - 'undef'");
            $plex_offline = 'undef';
        } elsif($plex_offline =~ /plex/ && !$self->is_mirror()) {
            $Log->warn("Validated plex_offline: Info: plex_offline should be undef when aggr is not mirrored");
        }
        $self->set_plex_offline(plex_offline => $plex_offline);
        $Log->comment("Validated plex_offline: $plex_offline");
        $Log->comment("Validated plex_offline: OK");

        #======================================================
        # validate plex_offline_always
        $Log->comment("Starting validation of plex_offline_always....");
        $default = (grep /resyncio/, @{$self->get_raidio()}) ? 1 : $self->is_mirror();
        my $plex_offline_always = $self->get_plex_offline_always();
        if(!defined($plex_offline_always)) {
            $plex_offline_always = $default;
            $Log->comment("Validated plex_offline_always: Info: plex_offline_always is not defined. Changing to default - $default");
        }elsif($plex_offline_always != 0 and $plex_offline_always != 1) {
            $Log->comment("Validated plex_offline_always: Info: plex_offline_always is not valid - $plex_offline_always.  Changing to default - $default");
            $plex_offline_always = $default;
        }elsif($plex_offline_always == 1 and !$self->is_mirror()) {
            die("Validated plex_offline_always: Error: plex_offline_always should not be 1 when aggr is not mirrored");
        }
        $self->set_plex_offline_always(plex_offline_always => $plex_offline_always);
        $Log->comment("Validated plex_offline_always: $plex_offline_always");
        $Log->comment("Validated plex_offline_always: OK");

        #======================================================
        # validate mirror_during_resyncio
        $Log->comment("Starting validation of mirror_during_resyncio....");

        my $mirror_during_resyncio = $self->get_mirror_during_resyncio();
        my $defined_resyncio = (grep /resyncio/, @{$self->get_raidio()}) ? 1 : 0;

        #TODO: Change name from $default to some meaningful name
        $default = 0;
        if($defined_resyncio == 1) { $default = ($self->is_mirror()) ? 0 : 1;}

        if(!defined($mirror_during_resyncio)) {
            $mirror_during_resyncio = $default;
            $Log->comment("Validated mirror_during_resyncio: Info: mirror_during_resyncio is not defined. Changing to default - $default");
        }
        if($mirror_during_resyncio == 0 && $defined_resyncio == 1) {
            if(!$self->is_mirror()) {
                die("Validated mirror_during_resyncio: Error: For non-mirror aggr with resyncio, mirror_during_resyncio should be 1: $mirror_during_resyncio");
            }
            if($self->get_plex_offline() =~ /undef/) {
                die("Validated mirror_during_resyncio: Error: For resycnio, plex_offline can not be undef when mirror_during_resync is 0");
            }
        }
        $self->set_mirror_during_resyncio(mirror_during_resyncio => $mirror_during_resyncio);
        $Log->comment("Validated mirror_during_resyncio: $mirror_during_resyncio");
        $Log->comment("Validated mirror_during_resyncio: OK");

        #======================================================
        # validate rg data
        $Log->comment("Validating rg data...");
        my $rgs = $self->get_rgs();
        if(!defined($rgs) or !scalar(@$rgs)) {
            die("Validated rg data: Error: At least one rg should be defined in an aggregate");
        }
        foreach my $index (0..scalar(@$rgs)-1) {
            my $rg = $rgs->[$index];
            if(!defined($rg->{checksum_type})) {
                $rg->{checksum_type} = $self->get_checksum_type();
                $Log->comment("Validated rg_data: Info: checksum_type is not defined. Changing to - ".$self->get_checksum_type());
            }

            #This validation is not correct, it might fail for mixed checksum type aggreagates
            #preferred storage type rg0 may not be correct for rg1, we will will it blank

            #TODO: Make sure that aggr_add code does not fail due to this change
            #      Remove this code after few ErrHandling runs are successful

            # if(!defined($rg->{storage_type})) {
                # $rg->{storage_type} = $pref_storage_type;
                # $Log->comment("Validated rg_data: Info: storage_type is not defined. Changing to - $pref_storage_type");
            # }

            if(!defined($rg->{disk_count})) {
                $rg->{disk_count} = $min_raid_size;
                $Log->comment("Validated rg_data: Info: disk_count is not defined. Changing to - $min_raid_size");
            }

            if(!defined($rg->{ssd_tier})) {
                $rg->{ssd_tier} = 0;
                $Log->comment("Validated rg_data: Info: ssd_tier is not defined. Changing to default - 0");
            }
            $rgs->[$index] = $rg;
        }
        $self->set_rgs(rgs => $rgs);
        $Log->comment("Validated rg data: ".Dumper($rgs));
        $Log->comment("Validated rg data: OK");

        #======================================================
        # validate hybrid_aggregate
        $Log->comment("Validating hybrid_aggregate...");
        my $hybrid_aggr = $self->get_hybrid_aggregate();
        if(!defined($hybrid_aggr)) {
            $hybrid_aggr = $self->is_rgs_support_hybrid();
            $Log->comment("Validated hybrid_aggregate: Info: hybrid_aggregate is not defined. Changing to default - ".$hybrid_aggr);
        }

        my $rgs_has_ssd_tier = $self->is_rgs_support_hybrid();
        if($hybrid_aggr != $rgs_has_ssd_tier ) {
            if($rgs_has_ssd_tier) {
                die("Validated hybrid_aggregate: Error: Some of RGs have ssd_tier set to 1 when hybrid_aggregate is not set to 1");
            } else {
                die("Validated hybrid_aggregate: Error: None of RG has ssd_tier set to 1 when hybrid_aggregate is set to 1");
            }
        }
        $Log->comment("Validated hybrid_aggregate - $hybrid_aggr");
        $Log->comment("Validated hybrid_aggregate: OK");

        #======================================================
        # validate aggr_create_sequence
        $Log->comment("Validating aggr_create_sequence...");
        my $aggr_create_sequence = $self->get_aggr_create_sequence();
        if(defined($aggr_create_sequence)) {
            my @aggr_create_sequence = split(/,\s*/,$aggr_create_sequence);

            # There should not be any value other than ha,hb,sa,sb,m in aggr_create_sequence.
            if (grep !/(^ha$|^hb$|^sa$|^sb$|^m$)/, @aggr_create_sequence) {
                die("Validated aggr_create_sequence: Error: There should not be any value other than ha,hb,sa,sb,m in aggr_create_sequence.");
            }

            # There should be 0 or 1 instance of "m" in aggr_create_sequence.
            if ((grep /^m$/, @aggr_create_sequence) > 1) {
                die("Validated aggr_create_sequence: Error: There is more than 1 instance of 'm' in aggr_create_sequence");
            }

            # "sa" or "sb" should not be first value in aggr_create_sequence.
            if ($aggr_create_sequence[0] =~ /^s/) {
                die("Validated aggr_create_sequence: Error: 'sa' or 'sb' should not be first value in aggr_create_sequence");
            }

            # There should be atleast one "ha" or "hb" in aggr_create_sequence.
            if (!grep /(^ha$|^hb$)/, @aggr_create_sequence) {
                die("Validated aggr_create_sequence: Error: There should be atleast one 'ha' or 'hb' in aggr_create_sequence.");
            }

            # if "m" is the first entry then "sa" and "sb" should not be second entry.
            if ($aggr_create_sequence[0] eq "m" && $aggr_create_sequence[1] =~ /^s/) {
                die("Validated aggr_create_sequence: Error: if 'm' is the first entry then 'sa' and 'sb' should not be second entry.");
            }

            #======================================================
            # validate rg based on aggr_create_sequence
            $Log->comment("Validating rg based on aggr_create_sequence...");
            my $rgs         = $self->get_rgs();
            my @all_rgs     = @$rgs;
            my @rg_sequence = ();

            foreach my $aggr_sequence_value (@aggr_create_sequence) {
                if ($aggr_sequence_value =~ /^m$/) {
                    next;
                }
                my $return_value = $self->check_rg_for_aggr_create_sequence($aggr_sequence_value,\@all_rgs);
                $Log->comment("Validated aggr_create_sequence: return index - $return_value");
                if ($return_value ne "false") {
                    $Log->comment("Validated aggr_create_sequence: ".Dumper($all_rgs[$return_value]));
                    push(@rg_sequence,$all_rgs[$return_value]);

                    $Log->comment("Validated aggr_create_sequence: before remove ".Dumper(\@all_rgs));
                    # remove this index value from all_rgs so that modified array list is passed for next sequence
                    my %group_list = map { $_, 1 } $all_rgs[$return_value];
                    @all_rgs       = grep(!$group_list{$_}, @all_rgs);
                    $Log->comment("Validated aggr_create_sequence: after remove ".Dumper(\@all_rgs));
                } else {
                    die("Validated aggr_create_sequence: Error: rgs data is not available for aggr_create_sequence-$aggr_sequence_value.");
                }
            }
            $self->set_rgs(rgs => \@rg_sequence);
            my $rgs_sequence = $self->get_rgs();
            $Log->comment("Validated aggr_create_sequence: Dumnper ".Dumper($rgs_sequence));
        }
        $Log->comment("Validated aggr_create_sequence: OK");

    }

    sub get_aggr_usable_space {
        my $self = shift;
        my $filer_vol = $self->get_filer_vol();
        my $api_set_obj = $self->{api_set_obj};
        my $mode = ($api_set_obj->{set} eq "CMode");

        my $key_skip_snapshot = ($api_set_obj->{set} eq "CMode") ? 'skip-snapshot-lines'  : 'supress-snapshot-lines' ;
        my $key_aggr_name     = ($api_set_obj->{set} eq "CMode") ? 'aggregate'            : 'name' ;
        my $key_aggr_switch   = ($api_set_obj->{set} eq "CMode") ? 'aggregates'           : 'aggregate' ;
        my $key_megabytes     = ($api_set_obj->{set} eq "CMode") ? 'megabyte'             : 'scale-megabytes' ;
        my $available_space   = ($api_set_obj->{set} eq "CMode") ? 'available-space'      : 'avail' ;

        my $response = $api_set_obj->df($key_aggr_switch => 'true', $key_aggr_name=> $filer_vol, $key_megabytes => 'true', $key_skip_snapshot => 'true');
        my $parsed_output = $response->get_parsed_output();
        my $available_size  = $parsed_output->[0]->{$available_space};
        $parsed_output->[0]->{$available_space} =~ /^(\d+)(\w)B/i;
        my ($size, $prefix) = ($1, $2);
        $size = int(0.8 * $size);
        return $size.$prefix;
    }

    sub get_ems_marker {
        my $self = shift;

        return $self->{ems_marker};
    }

    sub set_ems_marker {
        my $self = shift;
        my %opts = validate(@_, {ems_marker => {type=>SCALAR}});

        $self->{ems_marker} = $opts{ems_marker};
    }

    sub is_rgs_support_hybrid {
        my $self = shift;
        my @rgs = @{$self->get_rgs()};

        my $ssd_tier = 0;
        foreach my $index (0..scalar(@rgs)-1) {
            my $rg = $rgs[$index];
            if($rg->{ssd_tier} == 1) {
                $ssd_tier++;
                if(defined($rg->{storage_type}) && $rg->{storage_type} !~ /^B?SSD$/) {
                    die("is_rgs_support_hybrid: Error: storage type for ssd_tier RGs should be either SSD or BSSD. Actual storage_type - ".$rg->{storage_type});
                } elsif(!defined($rg->{storage_type})) {
                    $rg->{storage_type} = "SSD";
                    $Log->comment("is_rgs_support_hybrid: Info: storage_type for ssd_tier RG is not defined - Changing it to SSD");
                }
            }
            $rgs[$index] = $rg;
        }

        if($ssd_tier > 0) {
            if($rgs[0]->{ssd_tier} == 1 && $ssd_tier != scalar(@rgs)) {
                $Log->comment("is_rgs_support_hybrid: Debug: ssd_tier = $ssd_tier, rg_count: ".scalar(@rgs));
                die("is_rgs_support_hybrid: Error: Condition failed; Either first group should be HDD for hybrid aggr or all RGs should be SSD for Bamboo aggregate.");
            }
            if($ssd_tier == scalar(@rgs)) {
                return 0;
            }
            return 1;
        } else {
            return 0;
        }
    }

    sub check_rg_for_aggr_create_sequence {
        my $self = shift;
        # Use validate function
        my ($sequence_value,$remaining_rgs) = @_;
        my $checksum_type;
        if ($sequence_value =~ /(^ha$|^sa$)/) {
            $checksum_type = 'advanced_zoned';
        } elsif ($sequence_value =~ /(^hb$|^sb$)/) {
            $checksum_type = 'block';
        }

        my $found_rg = 0;
        my $return_index = 0;
        $Log->comment("check_rg_for_aggr_create_sequence: Debug: sequence value - $sequence_value");
        $Log->comment("check_rg_for_aggr_create_sequence: Debug: dumper ".Dumper($remaining_rgs));
        foreach my $index (0..scalar(@$remaining_rgs)-1) {
            my $rg = $remaining_rgs->[$index];
            $return_index = $index;
            $Log->comment("check_rg_for_aggr_create_sequence: Debug: each rg dumper ".Dumper($rg));
            $Log->comment("check_rg_for_aggr_create_sequence: Debug: Expected - $checksum_type; Actual -  ".$rg->{checksum_type});
            if ($rg->{checksum_type} eq "$checksum_type") {
                if (($sequence_value =~ /^s/i) && ($rg->{storage_type} =~ /^B?SSD$/)) {
                    $found_rg++;
                    last;
                } elsif (($sequence_value =~ /^h/i) && ($rg->{storage_type} !~ /^B?SSD$/)) {
                    $found_rg++;
                    last;
                }
            } else {
                next;
            }
        }
        if ($found_rg) {
            $Log->comment("check_rg_for_aggr_create_sequence: Debug:  return index value - $return_index");
            return ($return_index);
        } else {
            return 'false';
        }
    }



#=================================== get/set methods for AggrCreate member fields =============================

    sub get_aggr_info{
        my $self = shift;

        return $self->{aggr_info};
    }

    sub set_aggr_info {
        my $self =shift;
        my (%opts) = validate(@_, {
                            aggr_info => {type=>HASHREF|UNDEF, default=>undef, optional=>1},
                        });

        $self->{aggr_info} = $opts{aggr_info};
    }

    sub get_filer_vol {
        my $self = shift;
        return $self->{filer_vol};
    }

    sub set_filer_vol {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol => {type=>SCALAR},
                    });

        $self->{filer_vol} = $opts{filer_vol};

    }

    sub get_vserver {
        my $self = shift;

        return $self->{vserver};

    }

    sub set_vserver {
        my $self = shift;
        my %opts = validate(@_, {
                        vserver => {type=>SCALAR},
                    });
        $self->{vserver} = $opts{vserver};
    }

    sub get_mirror {
        my $self = shift;

        return $self->{mirror};
    }

    sub is_mirror {
        my $self = shift;

        return $self->get_mirror();
    }

    sub set_mirror {
        my $self = shift;
        my %opts = validate(@_, {
                        mirror => {type=>SCALAR},
                    });
        $self->{mirror} = $opts{mirror};
    }

    sub get_raidio {
        my $self = shift;

        return $self->{raidio};
    }

    sub set_raidio {
        my $self = shift;
        my %opts = validate(@_, {
                        raidio => {type=>ARRAYREF},
                    });
        $self->{raidio} = $opts{raidio};
    }

    sub get_raid_type {
        my $self = shift;

        return $self->{raid_type};
    }

    sub set_raid_type {
        my $self = shift;
        my %opts = validate(@_, {
                        raid_type => {type=>SCALAR},
                    });
        $self->{raid_type} = $opts{raid_type};
    }

    sub get_raid_size {
        my $self = shift;

        return $self->{raid_size};
    }

    sub set_raid_size {
        my $self = shift;
        my %opts = validate(@_, {
                        raid_size => {type=>SCALAR},
                    });
        $self->{raid_size} = $opts{raid_size};
    }

    sub get_aggr {
        my $self = shift;

        return $self->{aggr};
    }

    sub set_aggr {
        my $self = shift;
        my %opts = validate(@_, {
                        aggr => {type=>SCALAR},
                    });
        $self->{aggr} = $opts{aggr};
    }

    sub get_aggr_bits {
        my $self = shift;

        return $self->{aggr_bits};
    }

    sub set_aggr_bits {
        my $self = shift;
        my %opts = validate(@_, {
                        aggr_bits => {type=>SCALAR},
                    });
        $self->{aggr_bits} = $opts{aggr_bits};
    }

    sub get_checksum_type {
        my $self = shift;

        return $self->{checksum_type};
    }

    sub set_checksum_type {
        my $self = shift;
        my %opts = validate(@_, {
                        checksum_type => {type=>SCALAR},
                    });
        $self->{checksum_type} = $opts{checksum_type};
    }

    sub refresh_checksum_type{
        my $self= shift;

        my $checksum_type = RaidSqa_get_aggr_cksum_type(api_set_obj=>$self->{api_set_obj}, filer_vol => $self->get_filer_vol(new_name=>1));
        $self->set_checksum_type(checksum_type => $checksum_type);
    }

    sub is_hybrid_aggregate {
        my $self = shift;

        return $self->get_hybrid_aggregate();
    }

    sub get_hybrid_aggregate {
        my $self = shift;

        return $self->{hybrid_aggregate};
    }

    sub set_hybrid_aggregate {
        my $self = shift;
        my %opts = validate(@_, {
                        hybrid_aggregate => {type=>SCALAR},
                    });
        $self->{hybrid_aggregate} = $opts{hybrid_aggregate};
    }

    sub get_aggr_create_sequence {
        my $self = shift;
        return $self->{aggr_create_sequence};
    }

    sub get_plex_offline {
        my $self = shift;
        return $self->{plex_offline};
    }

    sub set_plex_offline {
        my $self = shift;
        my %opts = validate(@_, {
            plex_offline => {type=>SCALAR},
        });
        $self->{plex_offline} = $opts{plex_offline};
    }

    sub get_plex_offline_always {
        my $self = shift;

        return $self->{plex_offline_always};
    }

    sub set_plex_offline_always {
        my $self = shift;
        my %opts = validate(@_, {
            plex_offline_always => {type=>SCALAR},
        });
        $self->{plex_offline_always} = $opts{plex_offline_always};
    }

    sub get_mirror_during_resyncio {
        my $self = shift;

        return $self->{mirror_during_resyncio};
    }

    sub set_mirror_during_resyncio {
        my $self = shift;
        my %opts = validate(@_, {
            mirror_during_resyncio => {type=>SCALAR},
        });
        $self->{mirror_during_resyncio} = $opts{mirror_during_resyncio};
    }

    sub get_ignore_lw_validation {
        my $self = shift;

        return $self->{ignore_lw_validation};
    }

    sub set_ignore_lw_validation {
        my $self = shift;
        my %opts = validate(@_, {
            ignore_lw_validation => {type=>SCALAR},
        });
        $self->{ignore_lw_validation} = $opts{ignore_lw_validation};
    }

    sub get_rgs {
        my $self = shift;

        return $self->{rgs};
    }

    sub set_rgs {
        my $self = shift;
        my %opts = validate(@_, {
            rgs => {type=>ARRAYREF},
        });
        $self->{rgs} = $opts{rgs};
    }

    sub get_aggr_options {
        my $self = shift;

        return $self->{aggr_options};
    }

    sub set_aggr_options {
        my $self = shift;
        my %opts = validate(@_, {
            aggr_options => {type=>HASHREF},
        });
        $self->{aggr_options} = $opts{aggr_options};
    }

    sub get_aggr_info {
        my $self = shift;

        return $self->{aggr_info};
    }

    sub set_aggr_info {
        my $self = shift;
        my %opts = validate(@_, {
            aggr_info => {type=>HASHREF},
        });
        $self->{aggr_info} = $opts{aggr_info};
    }

    sub get_plex_count {
        my $self = shift;

        return $self->{plex_count};
    }

    sub set_plex_count {
        my $self = shift;
        my %opts = validate(@_, {
            plex_count => {type=>SCALAR},
        });
        $self->{plex_count} = $opts{plex_count};
    }

    sub get_io_type {
        my $self = shift;

        return $self->{io_type};
    }

    sub set_io_type {
        my $self = shift;
        my %opts = validate(@_, {
            io_type => {type=>SCALAR},
        });
        $self->{io_type} = $opts{io_type};
    }

    sub get_fileCreate {
        my $self = shift;

        return $self->{fileCreate};
    }

    sub set_fileCreate {
        my $self = shift;
        my %opts = validate(@_, {
            fileCreate => {type => OBJECT, isa => 'FileCreate'},
        });

        $self->{fileCreate} = $opts{fileCreate};
    }


=cut
#DataStructure:
%file_info = (
    '$FILER_VOL' => {                     # this is a string representing filer name
        'rg_name' => {                    # this is a string in /vol/plex/rg format
            file_name => '',
            fileid => '',
            file_size => '',
            md5sum => 'value',
            dbn => {
                'dbn_1' => {              # this is a integer value regresenting LBA or dbn
                    disk_name =>'',
                    cksum_dbn => '',
                    fbn => '',
                    stripe_checksum => {
                        'disk_name1' => checksum_obj  # this is string representing string name
                        'disk_name2' => checksum_obj
                    },
                },
                # dbn_n...
            },
        },
    },
    pi_files_list => [],                  # array ref to list of files in /etc/crash listing it as pi files
);
=cut
#===================================================================================================
package FileCreate;
use strict;
use warnings;
use vars qw(@ISA);
use Storable qw(dclone);

# Module Imports
use TCD;
use Data::Dumper;
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidSqav3 qw(AUTOLOAD);
use RaidInfo;
use Tharn qw($Log);
use NACL::APISet;
use List::Util qw[min max];
use vars qw(@ISA $ZONE_SIZE);

    BEGIN {
        @ISA = qw(commonUtils);
    }

    sub new {
        my $self = undef;
        $self = shift if(scalar(@_) % 2);
        my %self = validate(@_, {
                        api_set_obj  => {type => OBJECT, isa => "NACL::APISet"},
                        aggregate    => {type => OBJECT, isa => "AggrCreate"},
                        file_info    => {type => HASHREF},
                    });
        $self = \%self;
        my $api_set_obj = $self->{api_set_obj};
        bless $self, 'FileCreate';
        $ZONE_SIZE = 64;            #represents the zone size
        return $self;
    }

    sub get_zone_count{
        my $self = shift;

        return $self->{zone_count};

    }

    sub create_mkfile {
        my $self = shift;
        my %opts = validate(@_, {
                        filer_vol=>{type=>SCALAR, default=>$self->{aggregate}->get_filer_vol(new_name=>1), optional=>1},
                        size=>{type=>SCALAR, default=>"10m", optional=>1},
                        count=>{type=>SCALAR, default=>"1", optional=>1},
                    });
        my $api_set_obj = $self->{api_set_obj};

        my $filer_vol = $opts{filer_vol};
        my $aggr = $self->{aggregate};
        my $mkfile_name = "/vol/$filer_vol/mkfile_".int(rand(100))."_";
        my $size = $opts{size};
        my $count = $opts{count};
        my $cmd  = "priv set -q diag; mkfile $size $mkfile_name $count";

        $Log->comment("create_mkfile: Info: Creating few temporary file(s)...");
        RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd => $cmd);
        $self->flush_cache(filer_vol => $filer_vol, checksum_type=>$aggr->get_checksum_type(), aggr_type => $aggr->get_aggr());
    }

    sub get_file_size{
        my $self =shift;
        my (%opts) = validate(@_, {
                            file_name  => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                        });

        my $api_set_obj = $self->{api_set_obj};
        my $file_name = $opts{file_name};
        my $cmd = "priv set -q diag; slist $file_name";
        my $output = RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd => $cmd);
        my @slist_output = split(/\n/, $output);

        foreach my $line (@slist_output) {
            if($line =~ /\d+\s+(\d+)\s+$file_name/){
                my $file_size = int($1);
                return $file_size;
            }
        }
    }

    sub get_test_file_size {
        my $self =shift;
        my (%opts) = validate(@_, {
                            filer_vol => {type=>SCALAR, default=>$self->{aggregate}->get_filer_vol(), optional=>1},
                            rg_name => {type=>SCALAR},
                        });
        my $filer_vol = $opts{filer_vol};
        my $rg_name = $opts{rg_name};
        my $file_info = $self->get_file_info();
        return $file_info->{$filer_vol}->{$rg_name}->{file_size};
    }

    sub set_test_file_size {
        my $self =shift;
        my (%opts) = validate(@_, {
                                    filer_vol => {type=>SCALAR, default=>$self->{aggregate}->get_filer_vol(), optional=>1},
                                    rg_name => {type=>SCALAR},
                                    file_size => {type=>SCALAR},
                                });
        my $filer_vol = $opts{filer_vol};
        my $rg_name = $opts{rg_name};
        my $file_size = $opts{file_size};

        my $file_info = $self->get_file_info();
        $file_info->{$filer_vol}->{$rg_name}->{file_size} = $file_size;;
        return $self->set_file_info(file_info=>\%{$file_info});
    }

    sub get_test_file_name {
        my $self =shift;
        my (%opts) = validate(@_, {
                        filer_vol => {type=>SCALAR, default=>$self->{aggregate}->get_filer_vol(), optional=>1},
                        rg_name => {type=>SCALAR},
                    });

        my $filer_vol = $opts{filer_vol};
        my $rg_name = $opts{rg_name};
        my $file_info = $self->get_file_info();
        return $file_info->{$filer_vol}->{$rg_name}->{file_name};
    }

    sub set_test_file_name {
        my $self =shift;
        my (%opts) = validate(@_, {
                                    filer_vol => {type=>SCALAR, default=>$self->{aggregate}->get_filer_vol(), optional=>1},
                                    rg_name => {type=>SCALAR},
                                    file_name => {type=>SCALAR},
                                });
        my $filer_vol = $opts{filer_vol};
        my $rg_name = $opts{rg_name};
        my $file_name = $opts{file_name};

        my $file_info = $self->get_file_info();
        $file_info->{$filer_vol}->{$rg_name}->{file_name} = $file_name;;
        return $self->set_file_info(file_info=>\%{$file_info});
    }

    sub get_file_info {
        my $self = shift;

        return $self->{file_info};
    }

    sub set_file_info {
        my $self =shift;
        my (%opts) = validate(@_, {
                            file_info => {type=>HASHREF|UNDEF, default=>undef, optional=>1},
                        });

        $self->{file_info} = $opts{file_info};
    }

    sub get_test_fileid {
        my $self =shift;
        my (%opts) = validate(@_, {
                            filer_vol => {type=>SCALAR, default => $self->{aggregate}->get_filer_vol(), optional=>1},
                            rg_name => {type=>SCALAR},
                        });
        my $filer_vol = $opts{filer_vol};
        my $rg_name = $opts{rg_name};
        my $file_info = $self->get_file_info();
        return $file_info->{$filer_vol}->{$rg_name}->{fileid};
    }

    sub set_test_fileid {
        my $self =shift;
        my (%opts) = validate(@_, {
                            filer_vol => {type=>SCALAR, default=>$self->{aggregate}->get_filer_vol(), optional=>1},
                            rg_name => {type=>SCALAR},
                            fileid => {type=>SCALAR},
                        });
        my $filer_vol = $opts{filer_vol};
        my $rg_name = $opts{rg_name};
        my $fileid = $opts{fileid};

        my $file_info = $self->get_file_info();
        $file_info->{$filer_vol}->{$rg_name}->{fileid} = $fileid;
        return $self->set_file_info(file_info=>\%{$file_info});
    }

    sub get_md5sum {
        my $self =shift;
        my (%opts) = validate(@_, {
                        filer_vol => {type=>SCALAR, default=>$self->{aggregate}->get_filer_vol(), optional=>1},
                        rg_name   => {type=>SCALAR},
                        context   => {type=>SCALAR},
                    });

        my $file_info = $self->get_file_info();
        return $file_info->{$opts{filer_vol}}->{$opts{rg_name}}->{md5sum}->{$opts{context}};
    }

    sub set_md5sum {
        my $self =shift;
        my (%opts) = validate(@_, {
                        filer_vol => {type=>SCALAR, default=>$self->{aggregate}->get_filer_vol(), optional=>1},
                        rg_name   => {type=>SCALAR},
                        md5sum    => {type=>SCALAR},
                        context   => {type=>SCALAR},
                    });

        my $file_info = $self->get_file_info();
        $file_info->{$opts{filer_vol}}->{$opts{rg_name}}->{md5sum}->{$opts{context}} = $opts{md5sum};
        $self->set_file_info(file_info => $file_info);
    }

    sub get_stripe_checksum {
        my $self =shift;
        my (%opts) = validate(@_, {
                            filer_vol => {type=>SCALAR, default => $self->{aggregate}->get_filer_vol(), optional=>1},
                            rg_name => {type=>SCALAR},
                            dbn     => {type=>SCALAR},
                            context => {type=>SCALAR},
                        });

        my $file_info = $self->get_file_info();
        # return $file_info->{$opts{filer_vol}}->{$opts{rg_name}}->{stripe_checksum}->{$opts{context}};
        # return $file_info->{$opts{filer_vol}}->{$opts{rg_name}}->{dbn}->{$opts{dbn}}->{stripe_checksum}->{$opts{context}};
        return $self->{stripe_checksum}->{$opts{context}}->{$opts{filer_vol}}->{$opts{rg_name}}->{$opts{dbn}};
    }

    sub set_stripe_checksum {
        my $self =shift;
        my (%opts) = validate(@_, {
                        filer_vol => {type=>SCALAR, default => $self->{aggregate}->get_filer_vol(), optional=>1},
                        rg_name => {type=>SCALAR},
                        dbn     => {type=>SCALAR},
                        context => {type=>SCALAR},
                        stripe_checksum => {type=>HASHREF},
                    });

        $self->{stripe_checksum}->{$opts{context}}->{$opts{filer_vol}}->{$opts{rg_name}}->{$opts{dbn}} = $opts{stripe_checksum};
        # my $file_info = $self->get_file_info();
        # $file_info->{$opts{filer_vol}}->{$opts{rg_name}}->{stripe_checksum}->{$opts{context}} = $opts{stripe_checksum};
        # $file_info->{$opts{filer_vol}}->{$opts{rg_name}}->{dbn}->{$opts{dbn}}->{stripe_checksum}->{$opts{context}} = $opts{stripe_checksum};
        # $self->set_file_info(file_info => $file_info);
    }

    sub get_dbn_list {
        my $self =shift;
        my (%opts) = validate(@_, {
                        filer_vol => {type=>SCALAR, default => $self->{aggregate}->get_filer_vol(), optional=>1},
                        rg_name => {type=>SCALAR},
                    });

        my $file_info = $self->get_file_info();
        return keys %{$file_info->{$opts{filer_vol}}->{$opts{rg_name}}->{dbn}};
    }

    sub get_rg_list {
        my $self =shift;
        my (%opts) = validate(@_, {
                            filer_vol => {type=>SCALAR, default => $self->{aggregate}->get_filer_vol(), optional=>1},
                        });

        my $file_info = $self->get_file_info();
        my @rg_list = grep (/rg/i, keys %{$file_info->{$opts{filer_vol}}});
        $Log->comment("get_rg_list: Info: rg_list - @rg_list");
        return @rg_list;
    }

    sub get_max_fbn {
        my $self = shift;
        my (%opts) = validate(@_, {
                filer_vol => {type=>SCALAR, default=>$self->{aggregate}->get_filer_vol(), optional=>1},
                rg_name => {type=>SCALAR},
                });
        my $filer_vol = $opts{filer_vol};
        my $rg_name = $opts{rg_name};
        my $file_info = $self->get_file_info();
        my $file_size = $file_info->{$filer_vol}->{$rg_name}->{file_size};
        # my $file_size = 4096*6;
        return int($file_size/4096);
    }

    sub getDBN {
        my $self = shift;
        my (%opts) = validate(@_, {
            filer_vol  => {type => SCALAR, default=>$self->{aggregate}->get_filer_vol(), optional=>1},
            fbn        => {type=>SCALAR},
            dbn        => {type=>SCALAR},
            maxfbn     => {type=>SCALAR},
            stripe_gap => {type=>SCALAR},
            fileid     => {type=>SCALAR},
        });

        my $fbn         = $opts{fbn};
        my $dbn         = $opts{dbn};
        my $maxfbn      = $opts{maxfbn};
        my $stripe_gap  = $opts{stripe_gap};
        my $fileid      = $opts{fileid};
        my $filer_vol   = $opts{filer_vol};
        my $api_set_obj = $self->{api_set_obj};

        my $newDbn = 0;
        my $dumpblock_info = {};
        while($fbn < $maxfbn){
            # $newDbn = int(rand($maxfbn+1));
            $dumpblock_info = RaidSqa_get_dumpblock_info(api_set_obj => $api_set_obj, filer_vol=>$filer_vol, fileid=>$fileid, fbn=>$fbn);
            $newDbn = $dumpblock_info->{dbn};
            if($newDbn == $dbn + $stripe_gap) {
                last;
            }
            $Log->comment("getDBN: Debug: fbn - $fbn, dbn - $dbn, stripe_gap - $stripe_gap, new dbn - $newDbn");
            $fbn += 1;
        }
        if($fbn >= $maxfbn){
            die("getDBN: Error: Did not find required dbn: fbn($fbn) > maxfbn($maxfbn)");
        }
        # print "FOUND: dbn - $dbn, stripe_gap - $stripe_gap, new dbn - $newDbn, fbn - $fbn\n";
        return {fbn=>$fbn, newDBN=>$newDbn, dumpblock_info=>$dumpblock_info};
    }

    sub compare_stripe_checksum {
        my $self =shift;
        my %opts = validate(@_, {
                        before_error_inject => {type => HASHREF},
                        after_error_inject  => {type => HASHREF},
                        error_type          => {type => SCALAR},
                    });

        my $aggr                = $self->{aggregate};
        my $api_set_obj         = $self->{api_set_obj};
        my $before_error_inject = $opts{before_error_inject};
        my $after_error_inject  = $opts{after_error_inject};
        my $error_type          = $opts{error_type};

        foreach my $disk (keys %{$before_error_inject}){
            my $cksum_obj_before = $before_error_inject->{$disk};
            my $cksum_obj_after = $after_error_inject->{$disk};

            #determine value for ignore_genid_mismatch, if 1, we will ignore the genid comparison
            my $rg_name = $aggr->get_disk_raid_group(disk_name=>$disk);
            $rg_name =~ /\/.*\/(plex\d+)\/(rg\d+)/;
            my ($plex, $rg) = ($1 , $2);
            my $data_disks = $aggr->get_aggr_info_disks(plex=>$plex, rg=>$rg, disk_type=>'data');
            my $ignore_genid_mismatch = (scalar(@$data_disks) == 1) ? 1 : 0;

            # compare this checksum object with other after error inject
            my $disk_type = $aggr->get_disk_type(disk_name => $disk);
            my $compare_result = $cksum_obj_before->compare(checksum_object=>$cksum_obj_after, disk_type=>$disk_type, ignore_genid_mismatch=>$ignore_genid_mismatch);
            if(scalar(keys %{$compare_result}) > 0){

                #BURT: 559471, There would be gen_count and emb_checksum mismatch for certain RLW errors a
                #              See BURT description for more details
                if($error_type eq "recover") {
                    my @mismatch_fields = keys %$compare_result;
                    if(!grep (!/(embedded_checksum|gen_count)/, @mismatch_fields) && scalar(@mismatch_fields) == 2) {
                       $Log->warn("compare_stripe_checksum: Info: gen_count is not fixed, parity/data drive is fixed instead of data/parity".
                                   Dumper($compare_result));
                    } else {
                        die("compare_stripe_checksum: Error: Following fields mismatched for disk - $disk: ".Dumper($compare_result));
                    }
                } else {
                    $Log->comment("compare_stripe_checksum: Info: Following fields mismatched for disk - $disk: ".Dumper($compare_result));
                    return 0;
                }
            }
        }
        return 1;
    }

    sub is_new_pi_files_created {
        my $self = shift;

        my @new_pi_files = ();
        my $file_info = $self->get_file_info();
        my @files_before = @{$file_info->{pi_files_list}->{before}};
        my @files_after = @{$file_info->{pi_files_list}->{after}};

        # compare two arrays and check for difference
        my %file_hash = map { $_, 1 } @files_before;
        my @diff_file_list = grep( !$file_hash{$_}, @files_after );
        @new_pi_files = grep(/^parity_inconsistencies.*/, @diff_file_list);
        if (scalar(@new_pi_files) > 0) {
            $Log->comment("is_new_pi_files_created: Error : New PI files are created -  @new_pi_files");
            return \@new_pi_files;
        } else {
            $Log->comment("is_new_pi_files_created: Info : New PI files are not generated");
            return [];
        }
    }

    sub list_pi_files {
        my $self = shift;
        my $cmd = 'priv set -q diag;ls /etc/crash';
        my $filelist = RaidSqa_filer_cmd(api_set_obj=>$self->{api_set_obj}, cmd=>$cmd);
        my @files = split(/\r?\n/, $filelist);

        for(my $i=0; $i < scalar(@files);$i++){
            # Trim leading and trailing blank spaces in file names
            $files[$i]  =~ s/^\s+//;
            $files[$i]  =~ s/\s+$//;
        }

        return \@files;
    }

    sub save_pi_files_list {
        my $self = shift;
        my %opts = validate(@_, {context=>{type=>SCALAR}});
        my $file_info = $self->get_file_info();
        $file_info->{pi_files_list}->{$opts{context}} = $self->list_pi_files();
        $self->set_file_info(file_info => $file_info);
    }

    sub print_file_info {
        my $self = shift;

        #remove stripe checksum_content before
        my $file_info = dclone($self->get_file_info());
        # $Log->comment("print_file_info: Debug: dumper file_info: ".Dumper($file_info));
        # # $file_info->{$opts{filer_vol}}->{$opts{rg_name}}->{dbn}->{$opts{dbn}}->{stripe_checksum}->{$opts{context}} = $opts{stripe_checksum};
        # foreach my $filer_vol (keys %$file_info) {
            # next if(!defined($file_info->{$filer_vol}));
            # foreach my $group (keys %{$file_info->{$filer_vol}}) {
                # $Log->comment("print_file_info: Debug: dumper file_info: ".Dumper($file_info->{$filer_vol}->{$group}->{'dbn'}));
                # my $dbn_info = $file_info->{$filer_vol}->{$group}->{'dbn'};
                # next if(!defined($dbn_info));
                # foreach my $dbn (keys %{$dbn_info}) {
                    # foreach my $context ('after', 'before') {
                        # my $stripe_checksum = $dbn_info->{$dbn}->{stripe_checksum}->{$context};
                        # next if(!defined($stripe_checksum));
                        # foreach my $disk (keys %$stripe_checksum) {
                            # $stripe_checksum->{$disk}->set_dd_block(dd_block => []);
                            # $stripe_checksum->{$disk}->{checksum}->{data_4k} = [];
                        # }
                        # $dbn_info->{$dbn}->{$context} = $stripe_checksum;
                    # }
                # }
                # $file_info->{$filer_vol}->{$group}->{dbn} = $dbn_info;
            # }
        # }
        $Log->comment("print_file_info: Debug: dumper file_info: ".Dumper($file_info));
    }

    sub get_max_file_size {
        my $self = shift;
        my $file_info = $self->get_file_info();
        my $filer_vol = $self->{aggregate}->get_filer_vol();
        my @rg_names = keys %{$file_info->{$filer_vol}};
        my @file_sizes = ();
        my $max_file_size = 0;
        foreach my $rg_name (@rg_names) {
            if($file_info->{$filer_vol}->{$rg_name}->{required_file_size} > $max_file_size) {
                $max_file_size = $file_info->{$filer_vol}->{$rg_name}->{required_file_size};
            }
        }

        $max_file_size = '7864320' if($max_file_size <= 0 );
        return $max_file_size;
    }

    sub create_test_file {
        my $self =shift;

        my $api_set_obj     = $self->{api_set_obj};
        my $aggr            = $self->{aggregate};
        my $filer_vol       = $aggr->get_filer_vol();
        my $max_file_size   = $self->get_max_file_size();
        my $maxfbn          = int($max_file_size/4096);
        my $file_info       = $self->get_file_info();
        my @rg_names        = keys %{$file_info->{$filer_vol}};
        my ($file_properties, $zone_cksum_dbn);

        $Log->comment("create_test_file: Debug: max_file_size: $max_file_size");

        if($max_file_size > 1024) {
            my @suffix = ("K", "M", "G");
            my $suffix = "K";
            foreach (0..2) {
                if($max_file_size > 1024) {
                    $max_file_size = int($max_file_size / 1024);
                    $suffix = $suffix[$_];
                }
            }
            $max_file_size = $max_file_size . "$suffix";
        }

        $Log->comment("create_test_file: Debug: file_info before file create: ".Dumper($self->get_file_info()));
        my $count = 1;
        while(scalar(@rg_names)) {
            my $file_name = "/vol/$filer_vol/rg_$count";
            my $file_data = {};
            $count += 1;
            $Log->comment("create_test_file: Info: create test file - $file_name of size $max_file_size");
            my $result = RaidSqa_file_create(api_set_obj=>$api_set_obj, file_name=>$file_name, file_size=>$max_file_size, vserver=>$aggr->get_vserver());
            die ("create_test_file: Error: File create with $file_name failed") if($result == 0 );

            $self->flush_cache(filer_vol => $filer_vol, checksum_type=>$aggr->get_checksum_type(), aggr_type => $aggr->get_aggr());
            $Log->comment("create_test_file: Info: File create with name $file_name successful");

            $file_data->{'file_name'} = $file_name;
            $file_data->{'fileid'} = RaidSqa_get_fileid(api_set_obj=>$api_set_obj, file_name=>$file_name);
            $file_data->{'dumpblock_info0'} = RaidSqa_get_dumpblock_info(api_set_obj => $api_set_obj, filer_vol=>$filer_vol, fileid=>$file_data->{'fileid'}, fbn=>0);
            $file_data->{'rg_name'} = $aggr->get_disk_raid_group(disk_name=>$file_data->{'dumpblock_info0'}->{disk});
            $file_data->{'maxfbn'} = $maxfbn;
            $file_data->{'file_size'} = $max_file_size;

            $Log->comment("create_test_file: Debug: file_data - ".Dumper($file_data));
            my $rg_name = $file_data->{rg_name};

            #If this file does not reside on one of the RG's we want to inject error on [i.e, @rg_names] we will skip gettting file properties
            next if(!grep /$rg_name/, @rg_names);

            if(defined $file_info->{$filer_vol}->{$rg_name}->{'stripe_gap'}){
                my $unique_stripe_count = $file_info->{$filer_vol}->{$rg_name}->{'unique_stripe_count'};
                my $stripe_gap = $file_info->{$filer_vol}->{$rg_name}->{'stripe_gap'};
                $file_properties =  $self->get_file_properties(file_data=>$file_data, stripe_gap=>$stripe_gap, unique_stripe_count=>$unique_stripe_count);
            } else {
                my $zones = $file_info->{$filer_vol}->{$rg_name}->{'zones'};
                if(!defined($zones)) {
                    die("create_test_file: Error: Zones or stripe_gap is not defined for this RG in file_info- $rg_name");
                }
                ($file_properties, $zone_cksum_dbn) =  $self->get_file_properties(file_data=>$file_data, zones=>$zones);
            }

            if(ref $file_properties) {
                $self->update_file_info(file_data=>$file_data, file_properties=>$file_properties, zone_cksum_dbn=>$zone_cksum_dbn);
                @rg_names = @{$self->remove_array_item(array=>\@rg_names, item=>$rg_name)};
                $Log->comment("create_test_file: info: File create successful file rg - $rg_name");
            }
        }
        $self->create_mkfile();

        # #Code to run hammer
        $Log->comment("create_test_file: Info: Starting hammer...");
        RaidSqa_hammer_start(api_set_obj => $api_set_obj, filer_vol => $filer_vol);
        $Log->comment("create_test_file: Debug: file_info after file create: ".Dumper($self->get_file_info()));
    }

    sub get_file_properties {
        my $self = shift;
        my %opts = validate(@_, {
            file_data           => {type=>HASHREF},
            stripe_gap          => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
            unique_stripe_count => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
            zones               => {type=>ARRAYREF|UNDEF, default=>undef, optional=>1},
        });

        my $file_data       = $opts{file_data};
        my $stripe_gap      = $opts{stripe_gap};
        my $zones           = $opts{zones};
        my $file_properties = {};
        my $zone_cksum_dbn  = [];
        my $cksum_dbn0      = $file_data->{dumpblock_info0}->{cksum_dbn};
        my $filer_vol       = $self->{aggregate}->get_filer_vol();
        my $api_set_obj     = $self->{api_set_obj};
        my $vserver         = $self->{aggregate}->{vserver};
        my $unique_stripe_count = $opts{unique_stripe_count};

        if(!defined($stripe_gap) && !defined($zones)) {
            die("get_file_properties: Error: One of stripe_gap with unique_stripe_count or zones should be defined");
        } elsif(defined($stripe_gap) && (!defined($unique_stripe_count))) {
            die("get_file_properties: Error: unique_stripe_count should be defined when stripe_gap is defined");
        }

        $file_data->{rg_name} =~ /\/.+\/(plex\d+)\/(rg\d+)/;
        my ($plex, $rg) = ($1, $2);
        my $data_disks = $self->{aggregate}->get_aggr_info_disks(plex=>$plex, rg=>$rg, disk_type=>'data');
        my $scramble_dbn_info = RaidSqa_get_scramble_dumpblock_info(api_set_obj => $api_set_obj, fileid => $file_data->{fileid}, filer_vol => $filer_vol, vserver => $vserver);

        if(defined $stripe_gap) {
            my @stripes = $self->get_stripes_dumpblock_data(
                stripe_gap  => $stripe_gap,
                file_data   => $file_data,
                data_disks  => $data_disks,
                scramble_dbn_info   => $scramble_dbn_info,
                unique_stripe_count => $unique_stripe_count,
            );

            $Log->comment("get_file_properties: Info: Full Stripes containig file data - @stripes");
            return 0 if(scalar(@stripes) < $unique_stripe_count);

            #this is static calculation and may have to be changed if zone size changes in future
            if(RaidSqa_get_group_checksum_type(api_set_obj=>$api_set_obj, group_name=>$file_data->{rg_name}) =~ /advanced_zoned/) {
                foreach my $stripe (@stripes) {
                    my $cksum_dbn = ((int(($stripe - $cksum_dbn0)/$ZONE_SIZE))*$ZONE_SIZE) + $cksum_dbn0;
                    if($stripe < $cksum_dbn0) {
                        $cksum_dbn -= $ZONE_SIZE if(($cksum_dbn - $stripe) > ($ZONE_SIZE/2));
                    } elsif ($stripe > $cksum_dbn0) {
                        $cksum_dbn += $ZONE_SIZE if(($stripe - $cksum_dbn) > ($ZONE_SIZE/2 - 1));
                    }
                    $file_properties->{$stripe}->{cksum_dbn} = $cksum_dbn;
                }
                # map { $file_properties->{$_}->{cksum_dbn} = ((int(($_ - $cksum_dbn0)/$ZONE_SIZE))*$ZONE_SIZE) + $cksum_dbn0} @stripes;
            } else {
                map { $file_properties->{$_}->{cksum_dbn} = $_} @stripes;
            }

            map { $file_properties->{$_}->{disk_name} = $data_disks->[0]} @stripes;
            $Log->comment("Dumper: file_properties ".Dumper($file_properties));
            return $file_properties;
        } else {
            my @cksum_dbns = $self->get_zones_cksum_dbn(
                file_data   => $file_data,
                zone_count  => scalar(@$zones),
                data_disks  => $data_disks,
                scramble_dbn_info => $scramble_dbn_info
            );

            $Log->comment("get_file_properties: Info: Full zones containig file data - @cksum_dbns");
            return 0 if(scalar(@cksum_dbns) < scalar(@$zones));

            foreach my $index (0..scalar(@cksum_dbns)-1) {
                $zone_cksum_dbn->[$index] = [];
                foreach my $offset (@{$zones->[$index]}) {
                    my $dbn = $cksum_dbns[$index] - 32 + $offset;
                    $file_properties->{$dbn}->{cksum_dbn} = $cksum_dbns[$index];
                    push @{$zone_cksum_dbn->[$index]}, $dbn;
                }
            }
            return $file_properties, $zone_cksum_dbn;
        }
    }

    #returns a hash in dbn=>dumpblock_hash format
    sub get_stripes_dumpblock_data {
        my $self = shift;
        my %opts = validate(@_, {
            file_data           => {type => HASHREF},
            stripe_gap          => {type => SCALAR},
            data_disks          => {type => ARRAYREF},
            unique_stripe_count => {type => SCALAR},
            scramble_dbn_info   => {type => HASHREF},
        });

        my $file_data           = $opts{file_data};
        my $stripe_gap          = $opts{stripe_gap};
        my $data_disks          = $opts{data_disks};
        my $scramble_dbn_info   = $opts{scramble_dbn_info};
        my $unique_stripe_count = $opts{unique_stripe_count};
        my $iter                = 1;
        my @stripes             = ();
        my $rg_name             = $file_data->{rg_name};

        my @dbn_list = keys %$scramble_dbn_info;
        @dbn_list = sort {$a <=> $b} @dbn_list;
        $Log->comment("get_stripe_dumpblock_data: Info: file dbn list  : ".@dbn_list);
        my $dbn = shift @dbn_list;

        $Log->comment("get_stripe_dumpblock_data: Info: unique_stripe_count: $unique_stripe_count");
        $Log->comment("get_stripe_dumpblock_data: Info: rg_name - $rg_name, data disks : ".@$data_disks);

        while($dbn < $dbn_list[-1] or $iter < @dbn_list) {
            my @disks_occupied = map {$_->{disk}} @{$scramble_dbn_info->{$dbn}};
            if(scalar(@disks_occupied) == scalar(@$data_disks)) {
                my %hash = map {$_, 1} @$data_disks;
                my @diff = grep !$hash{$_}, @disks_occupied;
                if(scalar(@diff) == 0) {
                    #this stripe contains user data blocks on all given disks
                    push(@stripes, $dbn);
                    return @stripes if(scalar(@stripes) >= $unique_stripe_count)
                }
            }
            $dbn += $stripe_gap;
            $iter++;
        }

        die("get_stripe_dumpblock_data: Error: File does not contain $unique_stripe_count full stripes");
    }

    sub get_zones_cksum_dbn {
        my $self = shift;
        my %opts = validate(@_, {
            file_data   => {type=>HASHREF},
            zone_count  => {type=>SCALAR},
            data_disks  => {type=>ARRAYREF},
            scramble_dbn_info => {type=>HASHREF},
        });

        my $file_data           = $opts{file_data};
        my $zone_count          = $opts{zone_count};
        my $data_disks          = $opts{data_disks};
        my $scramble_dbn_info   = $opts{scramble_dbn_info};
        my $cksum_dbn           = $file_data->{'dumpblock_info0'}->{cksum_dbn};
        my @zone_list           = ();

        foreach (0..15) {
            my $valid_zone  = 1;
            my @dbn_list    = ($cksum_dbn - 32, $cksum_dbn + 31);
            foreach my $dbn (@dbn_list) {
                my @disks_occupied = map {$_->{disk}} @{$scramble_dbn_info->{$dbn}};
                if(scalar(@disks_occupied) == scalar(@$data_disks)) {
                    my %hash = map {$_, 1} @$data_disks;
                    my @diff = grep !$hash{$_}, @disks_occupied;
                    if(scalar(@diff) != 0) {
                        $valid_zone = 0;
                        last;
                    }
                }
            }
            push(@zone_list, $cksum_dbn) if($valid_zone == 1);
            if(scalar @zone_list == $zone_count) {last;}
            $cksum_dbn += 64;
        }
        return @zone_list;
    }

    sub update_file_info {
        my $self = shift;
        my %opts = validate(@_, {
            file_data       => {type=>HASHREF},
            file_properties => {type=>HASHREF},
            zone_cksum_dbn  => {type=>ARRAYREF|UNDEF, default=>undef, optional=>1},
        });

        my $file_data       = $opts{file_data};
        my $file_properties = $opts{file_properties};
        my $zone_cksum_dbn  = $opts{zone_cksum_dbn};
        my $rg_name         = $file_data->{rg_name};
        my $filer_vol       = $self->{aggregate}->get_filer_vol();
        my $file_info       = $self->get_file_info();


        #Update the file data
        $file_info->{$filer_vol}->{$rg_name}->{file_name} = $file_data->{file_name};
        $file_info->{$filer_vol}->{$rg_name}->{fileid} = $file_data->{fileid};
        $file_info->{$filer_vol}->{$rg_name}->{maxfbn} = $file_data->{maxfbn};

        #Update file properties
        $file_info->{$filer_vol}->{$rg_name}->{dbn} = $file_properties;

        #update zones_dbn info if defined
        if(defined($zone_cksum_dbn)) {
            $file_info->{$filer_vol}->{$rg_name}->{zone_cksum_dbn} = $zone_cksum_dbn;
        }
        $self->set_file_info(file_info=>$file_info);
    }


=cut
my $spare_setup = {
    mirror  => 0,
    ignore_removing_ownership => 1, # possible values - 1,0.  If 1, other spares than required will be kept as it is
                                    # Default: 1, Optional.
    enable_optional_storage_type => 1,  # possible values - 1,0.  If 1, pick equivalent storage type upon encountering
                                        # insufficient disks for mentioned storage type. Default: 1, Optional.
    spare_setup_info => [
        {
            storage_type => "value",    # possible values - ATA, FCAL, etc.,
                                        # Default: ANY, which means it can be of any storage type. Optional
            bps => "value", # possible values: 512, 520, 4096, 4160.  Optional and should have value if it is mentioned
            checksum_type => 'block',   #possible values - block, zoned, advanced_zoned.  Default: block. Optional
            min_spare_count => "value", # some integer value > 0. Optional
            max_spare_count => "value", # some integer value > 0. Optional
            # spare_disk_list => [ ],   # This is internal variable and will be populated at runtime
        },
        # Repeat above hashref if there are more requirements for types of spares
    ],
    # spare_disk_info   => {    # Internally stored info
        # 'disk_name' => {
            # pool          => 0,
            # checksum_type => "block",
            # disk_type     => "ATA",
            # bps           => '512',
            # rpm           => '1500',
            # used_sz       => '',
        # }, ...
    # },
    # ownership_removed_disk_info => {
        # pool0 => \@pool0_disk_list,   # Disks which were removed as part of test from pool0
        # pool1 => \@pool1_disk_list,   # Disks which were removed as part of test from pool1
    # }
}
=cut
#==================================================================
package SpareDisks;
use strict;
use warnings;
use Storable qw(dclone);

# Module Imports
use TCD;
use Data::Dumper;
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidSqav3 qw(AUTOLOAD);
use RaidInfo;
use Tharn qw($Log);
use NACL::APISet;
use vars qw(@ISA);

    BEGIN {
        @ISA = qw(commonUtils);
    }

    sub new {
        my $self = undef;
        $self = shift if(scalar(@_) % 2);
        my %opts = validate(@_, {
                        api_set_obj       => {type => OBJECT, isa => "NACL::APISet"},
                        setup_info        => {type => HASHREF|UNDEF, default=>undef, optional=>1},
                        mirror            => {type => SCALAR},
                    });
        $self = (defined($opts{setup_info})) ? dclone($opts{setup_info}) : {};
        $self->{api_set_obj} = $opts{api_set_obj};
        $self->{mirror} = $opts{mirror};
        bless $self, 'SpareDisks';
        $self->refresh_spare_disk_info();
        return $self;
    }

    sub refresh_spare_disk_info {
        my $self = shift;
        my %opts = validate(@_,{
                        raid_object => {type => OBJECT, isa=>'RaidInfo', default=>$self->get_raid_info_object(), optional=>1},
                    });

        my %spare_disks = $opts{raid_object}->spare_disks();
        my @all_spare_disks = keys %spare_disks;
        my @param_list      = qw(pool checksum_type disk_type bps rpm used_sz);
        my $spare_disk_info = {};

        foreach my $disk_name (@all_spare_disks) {
            my $disk_object = $opts{raid_object}->disk($disk_name);
            next if(!defined($disk_object));

            foreach my $param (@param_list){
                $spare_disk_info->{$disk_name}->{$param} = $disk_object->parameter($param);
            }
        }
        $self->set_spare_disk_info(spare_disk_info => $spare_disk_info);
    }

    sub get_optional_storage_types {
        my $self = shift;
        my %opts = validate(@_, {storage_type => {type=>SCALAR}});

        my @storage_type_groups = (
            ['ATA', 'SATA', 'BSAS'],
            ['FCAL', 'SAS'],
            ['SSD'],
        );
        my @valid_storage_types = ('ATA', 'SATA', 'BSAS', 'FCAL', 'SAS', 'SSD');

        #Validate input storage type
        if(! grep /$opts{storage_type}/,  @valid_storage_types) {
            die("get_optional_storage_types: Error: Input storage type is not one of valid storage types");
        }

        #Find array to which input storage type belongs to
        foreach my $ref (@storage_type_groups) {
            if(grep(/$opts{storage_type}/, @$ref)) {
                return @$ref;
            }
        }
        die("get_optional_storage_types: Error: Optional storage type could not be found for $opts{storage_type}");
    }

    # Sorts inptu disk list based on storage type,
    # and returns data strucure with storage_type as keys and array ref of disks as values
    # e.g:
        # $return = {
            # 'storage_type1' = ['disk1','disk2'],
            # 'storage_type2' = ['disk1','disk2'],
        # }

    #===============================================================================================
    sub sort_disks_by_property {
        my $self = shift;
        my %opts = validate(@_, {
                        disk_list       => {type => ARRAYREF, default => [$self->get_all_spares()], optional=>1},
                        disk_property   => {type => SCALAR },
                    });

        my $disk_property   = $opts{disk_property};
        my $sorted_disks    = {};
        my $spare_disk_info = $self->get_spare_disk_info();

        foreach my $disk_name (@{$opts{disk_list}}) {
            next if(!defined($spare_disk_info->{$disk_name}));       #Q: Shall we put die statement here
            my $property_value = $spare_disk_info->{$disk_name}->{$disk_property};

            next if(!defined($property_value)); #Skip this disk if does not have any value set for the property
            $sorted_disks->{$property_value} = [] if(!defined($sorted_disks->{$property_value}));
            push(@{$sorted_disks->{$property_value}}, $disk_name);
        }

        return $sorted_disks;
    }

    #===============================================================================================
    sub get_disk_list_by_filter {
        my $self =shift;
        my (%opts) = validate(@_,{
                        checksum_type => {type => SCALAR|UNDEF, default=>undef, optional=>1},
                        disk_type     => {type => SCALAR|UNDEF, default=>undef, optional=>1},
                        bps           => {type => SCALAR|UNDEF, default=>undef, optional=>1},
                        rpm           => {type => SCALAR|UNDEF, default=>undef, optional=>1},
                        pool          => {type => SCALAR|UNDEF, default=>undef, optional=>1},
                        used_sz       => {type => SCALAR|UNDEF, default=>undef, optional=>1},
                        disk_list     => {type => ARRAYREF    , default=> [$self->get_all_spares()], optional=>1},
                     });

        my @disk_list           = @{$opts{disk_list}};
        my @filtered_disk_list  = ();
        my $spare_disk_info = $self->get_spare_disk_info();

        #delete params which are not required
        delete $opts{disk_list};
        my @params = keys %opts;
        map {delete $opts{$_} if(!defined($opts{$_}))} @params;

        foreach my $disk_name (@disk_list) {
            my $result = 1;
            if(!defined($spare_disk_info->{$disk_name})){
                $Log->comment("get_disk_list_by_filter: Debug: disk_name is not present in spare disk list");
                next;
            }

            foreach my $param (keys %opts) {
                if( $opts{$param} ne $spare_disk_info->{$disk_name}->{$param}){
                    $result = 0;
                }
            }
            push(@filtered_disk_list, $disk_name) if($result);
        }
        return \@filtered_disk_list;
    }

    #===============================================================================================
    sub get_all_spares {
        my $self = shift;

        my $spare_disk_info = $self->get_spare_disk_info();
        my @all_spare_disks =  keys %$spare_disk_info;
        return @all_spare_disks;
    }

    #===============================================================================================
    sub optional_storage_type_enabled {
        my $self = shift;

        return $self->{enable_optional_storage_type};
    }

    #===============================================================================================
    sub is_ignore_removing_ownership {
        my $self = shift;
        return $self->{ignore_removing_ownership};
    }

    #===============================================================================================
    sub get_spare_setup_info {
        # This method does "get" operation for Input data structure - "spare_setup_info"
        my $self = shift;
        if(defined($self->{spare_setup_info})) {
            return $self->{spare_setup_info};
        } else {
            return [];      #return empty array if not defined
        }
    }

    #===============================================================================================
    sub set_spare_setup_info {
        # This method does "set" operation for Input data structure - "spare_setup_info"
        my $self = shift;
        my %opts = validate(@_, {spare_setup_info => {type => ARRAYREF}});

        $self->{spare_setup_info} = $opts{spare_setup_info};
    }

    #===============================================================================================
    sub get_spare_disk_info {
        my $self = shift;

        if(!ref($self->{spare_disk_info})) {
            $self->refresh_spare_disk_info();
        }
        return $self->{spare_disk_info};
    }

    #===============================================================================================
    sub set_spare_disk_info {
        my $self = shift;
        my %opts = validate(@_, {spare_disk_info => {type => HASHREF}});

        $self->{spare_disk_info} = $opts{spare_disk_info};
    }

    #===============================================================================================
    sub get_ownership_removed_disk_info {
        my $self = shift;

        return $self->{ownership_removed_disk_info};
    }

    #===============================================================================================
    sub set_ownership_removed_disk_info {
        my $self = shift;
        my %opts = validate(@_, {
                        ownership_removed_disk_info => {type => HASHREF},
                    });

        $self->{ownership_removed_disk_info} = $opts{ownership_removed_disk_info};
    }

    #===============================================================================================
    sub reassign_disks {
        my $self = shift;

        my $ownership_removed_disk_info = $self->get_ownership_removed_disk_info();
        foreach my $pool (0 , 1) {
            next if(ref($ownership_removed_disk_info->{'pool'.$pool}) !~ /ARRAY/i);
            if(scalar(@{$ownership_removed_disk_info->{'pool'.$pool}}) > 0) {
                RaidSqa_disk_assign(
                    api_set_obj => $self->{api_set_obj},
                    disk_list   => $ownership_removed_disk_info->{'pool'.$pool},
                    pool        => $pool,
                    timeout     => $self->{timeout}
                );
            }
        }
    }

    #===============================================================================================
    #This subroutine compares the input spare_setup_info with spare_requirement_hash passed as argument
    sub compare_spare_setup_info {
        my $self = shift;
        my %opts = validate(@_, {
                        total_spare_requirements => {type=>ARRAYREF},
                    });
        my $total_spare_requirements = $opts{total_spare_requirements};
        my $spare_setup_info = $self->get_spare_setup_info();

        foreach my $spare_requirement (@$total_spare_requirements) {
            foreach my $index (0..scalar(@$spare_setup_info)-1) {
                my $spare_setup_hash = $spare_setup_info->[$index];
                if(!defined($spare_setup_hash->{min_spare_count})){
                    $spare_setup_hash->{min_spare_count} = -1;
                }
                if($self->compare_spare_hash(
                            spare_setup_hash =>$spare_setup_hash,
                            spare_requirement =>$spare_requirement)){
                    my $required_disk_count = $spare_requirement->{min_spare_count};
                    if($required_disk_count > $spare_setup_hash->{min_spare_count}) {
                        $spare_setup_hash->{min_spare_count} = $required_disk_count;
                    }
                    if(defined($spare_setup_hash->{max_spare_count}) &&
                        $required_disk_count > $spare_setup_hash->{max_spare_count}) {

                        $spare_setup_hash->{max_spare_count} = $required_disk_count;
                    }
                } else {
                    push(@$spare_setup_info, $spare_requirement);
                }
            }
        }
        $self->set_spare_setup_info(spare_setup_info=>$spare_setup_info);
    }

    #===============================================================================================
    #Checks if same kind of parameters are defined in two hashes passed as arguments to this subroutine
    sub compare_spare_hash {
        my $self = shift;
        my %opts = validate(@_, {
                        spare_setup_hash => {type=>HASHREF},
                        spare_requirement => {type=>HASHREF},
                    });

        my $spare_setup_hash  = $opts{spare_setup_hash};
        my $spare_requirement = $opts{spare_requirement};
        my @keys = ('storage_type', 'checksum_type');
        foreach my $key (@keys) {
            if($spare_setup_hash->{$key} ne $spare_requirement->{$key}) {return 0;}
        }
        return 1;
    }

    #===============================================================================================
    #This subroutine structure checks if spares disks are present on filer as per defined in spare_setup_info
    #Algorithm:
        # - refresh_spare_disk_info()
        # - Foreach spare_type
            # - Check if enough spares are available of given type
            # - If not die, else push used disks into an array
        # - Find unsued disks and unassign disks if ignore_removing_disks is set to 0.
    sub validate_spare_data {
        my $self = shift;
        my %opts = validate(@_, {
                        mirror => {type => SCALAR, default=>$self->{mirror}, optional =>1},
                    });

        my $mirror = $opts{mirror};
        my $api_set_obj = $self->{api_set_obj};

        #===================================================================
        #validate ignore_removing_ownership
        $Log->comment("Starting validation of ignore_removing_ownership....");
        if(!defined($self->{ignore_removing_ownership})) {
            $Log->comment("Validated ignore_removing_ownership: Info: ignore_removing_ownership is not defined. Changing to default - 1");
            $self->{ignore_removing_ownership} = 1;
        }
        $Log->comment("Validated ignore_removing_ownership: $self->{ignore_removing_ownership}");
        $Log->comment("Validated ignore_removing_ownership: OK");

        #===================================================================
        #validate enable_optional_storage_type
        $Log->comment("Starting validation of enable_optional_storage_type ....");
        if(!defined($self->{enable_optional_storage_type })) {
            $Log->comment("Validated enable_optional_storage_type : Info: enable_optional_storage_type  is not defined. Changing to default - 1");
            $self->{enable_optional_storage_type } = 1;
        }
        $Log->comment("Validated enable_optional_storage_type : $self->{enable_optional_storage_type }");
        $Log->comment("Validated enable_optional_storage_type : OK");

        my $spare_setup_info = $self->get_spare_setup_info();
        $self->refresh_spare_disk_info();
        my @all_spare_disks =  $self->get_all_spares();

        #===========================================================================================
        #get all spares disks
        my @disks_to_assign = ();
        my @disks_to_remove = ();
        foreach my $index (0..scalar(@$spare_setup_info)-1) {
            my $spare_requirement = $self->check_spares(spare_requirement => $spare_setup_info->[$index], mirror=>$mirror);
            push(@disks_to_assign, @{$spare_requirement->{spare_disk_list}});
            $spare_setup_info->[$index] = $spare_requirement;
        }

        #===========================================================================================
        #Find the disks whose ownership is to be removed
        if(!$self->is_ignore_removing_ownership()) {        #This value by default should be 1
            my %disk_hash = map {$_, 1} @disks_to_assign;
            @disks_to_remove = grep (!defined($disk_hash{$_}), @all_spare_disks);
            my $pool_disk_list = $self->sort_disks_by_property(disk_list=> \@disks_to_remove, disk_property => 'pool');
            $self->set_ownership_removed_disk_info(ownership_removed_disk_info => $pool_disk_list);
        }

        #===========================================================================================
        #Remove ownership of disks which are not required
        if(scalar(@disks_to_remove)) {
            $Log->comment("setup_spares: Info: Disks will be removed - @disks_to_remove");
            $Log->comment("setup_spares: Info: disable disk auto assign");
            RaidSqa_set_options(api_set_obj => $api_set_obj, option_name => "disk.auto_assign", option_value => "off");
            if( !RaidSqa_disk_remove_ownership(api_set_obj=>$api_set_obj, disk_list=>\@disks_to_remove)) {
                $Log->comment("setup_spares: Error: Disk Remove ownership failed - @disks_to_remove");
                # try again with individual disk -> this could be because of timing issue
                foreach my $disk (@disks_to_remove) {
                    die("setup_spares: Error: Disk Remove ownership failed - $disk")
                        if(!RaidSqa_disk_remove_ownership(api_set_obj=>$api_set_obj, disk_list=>[$disk]));
                }
            }
            sleep(2);
        }
        $self->set_spare_setup_info(spare_setup_info => $spare_setup_info);
    }

    # This subroutine checks if required spares are available, or removes if additional spares other than required are present in pool
    # Algorithm:
        # - Get all spares disks from both pools
        # - Apply filter of 'bps', checksum_type', 'rpm' as per mentioned
        # - Apply filter of storage_type to these already filtered spare disks
        # - if sufficient disks not are present, optional storage_type (if input flag 'enable_optional_storage_type' is 1)
        # - Checck if any of other storage type meets this criteria
        # - Sorts these disks by pool and check enough spares are present in each pool
    sub check_spares {
        my $self = shift;
        my %opts = validate(@_, {
                        spare_requirement => {type => HASHREF},
                        mirror => {type => SCALAR, default=>$self->{mirror}, optional =>1},
                    });
        my $spare_requirement       = $opts{spare_requirement};
        my $mirror                  = $opts{mirror};
        my $aggr_create_sequence    = $self->{aggr_create_sequence};
        my @all_spare_disks         =  $self->get_all_spares();
        $mirror = 1 if ($aggr_create_sequence =~ /\bm\b/i);

        #get min and max spare count
        # -1 represents ignore min_count
        my $min_count = defined($spare_requirement->{min_spare_count}) ? $spare_requirement->{min_spare_count} * ($mirror+1) : -1;
        my $max_count = defined($spare_requirement->{max_spare_count}) ? $spare_requirement->{max_spare_count} * ($mirror+1) : -1;

        #===========================================================================================
        $Log->comment("setup_spares: Debug: min_spares_required: $min_count, max_spares_required: $max_count");
        my %filter_criteria = (disk_list => \@all_spare_disks);
        foreach my $param ('checksum_type', 'bps', 'rpm') {
            if(exists($spare_requirement->{$param})) {
                $filter_criteria{$param} = $spare_requirement->{$param};
            }
        }

        #===========================================================================================
        my @valid_disk_list = ();
        if("$min_count" ne "-1") {
            @valid_disk_list = @{$self->get_disk_list_by_filter(%filter_criteria)};
            if(scalar(@valid_disk_list) < $min_count) {
                $Log->comment("setup_spares: Debug: valid_disk_list: @valid_disk_list");
                die("setup_spares: Error: Enough spare disks of required criteria could not found");
            }

            my $storage_disk_list = $self->sort_disks_by_property(disk_list=>\@valid_disk_list, disk_property => 'disk_type');

            #Filter disks by storage type
            if(defined($spare_requirement->{storage_type}) && uc $spare_requirement->{storage_type} ne 'ANY') {
                my $storage_type = $spare_requirement->{storage_type};
                my @optional_storage_type = $self->get_optional_storage_types(storage_type => $storage_type);
                my @valid_storage_type = ($self->optional_storage_type_enabled()) ? ($storage_type) : @optional_storage_type;
                my @disk_list = ();
                foreach my $storage_type (@valid_storage_type) {
                    if(!ref $storage_disk_list->{$storage_type}) {
                        next;
                    }
                    @disk_list = @{$storage_disk_list->{$storage_type}};
                    if(scalar(@disk_list) >= $min_count) {
                        $Log->comment("setup_spares: Info: Using optional storage type $storage_type");
                        $spare_requirement->{storage_type} = $storage_type;
                        last;
                    }
                }
                if(scalar(@disk_list) < $min_count) {
                        $Log->comment("setup_spares: Info: required disk count - $min_count, disk available - ".scalar(@disk_list));
                        die("setup_spares: Error: Enough spare disks of required storage type could not found - ".$spare_requirement->{storage_type});
                }
                @valid_disk_list = @disk_list;
            }
        } else {
            @valid_disk_list = @all_spare_disks;
        }

        #Check if disks are enough in each pool
        my $pool_disk_list = $self->sort_disks_by_property(disk_list => \@valid_disk_list, disk_property => 'pool');

        $Log->comment("setup_spares: Info: list of disks in each pool ".Dumper($pool_disk_list));
        my @spare_disk_list = ();
        if($mirror) {
            foreach my $pool (0, 1) {
                if(scalar(@{$pool_disk_list->{$pool}}) < $min_count/2) {
                    die("setup_spares: Error: There are not enough spares disks in pool$pool");
                }
                if("$max_count" ne "-1") { $pool_disk_list->{$pool} = [@{$pool_disk_list->{$pool}}[0..$max_count/2-1]];}
            }
            @spare_disk_list = (@{$pool_disk_list->{pool0}}, @{$pool_disk_list->{pool1}});
        } else {
            #Sort based on pool
            @spare_disk_list = @{$pool_disk_list->{0}} if(exists($pool_disk_list->{0}));
            push(@{$pool_disk_list->{1}}) if(exists($pool_disk_list->{1}));
            @spare_disk_list = @spare_disk_list[0..$max_count-1] if("$max_count" ne "-1");
        }
        $spare_requirement->{spare_disk_list} = \@spare_disk_list;
        return $spare_requirement;
    }

=cut
This package contains the common subroutines which can be used by all other package by inheriting this package
syntax:
    package test;
    use vars qw(@ISA);
    BEGIN {
        @ISA = qw (Utils);
    }
=cut
package commonUtils;
use Tharn qw($Log);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidSqav3 qw(AUTOLOAD);

    sub flush_cache {
        my $self =shift;
        my %opts = validate(@_, {
                        api_set_obj   => {type => OBJECT, isa=>'NACL::APISet', default=>$self->{api_set_obj}, optional=>1},
                        filer_vol     => {type => SCALAR},
                        checksum_type => {type => SCALAR},
                        aggr_type     => {type => SCALAR, default=>1, optional=>1},
                    });
        my $filer_vol = $opts{filer_vol};
        my $api_set_obj = $opts{api_set_obj};
        my $checksum_type = $opts{checksum_type};
        my $aggr_type     = $opts{aggr_type};

        $Log->comment("Flusing cache");
        if($checksum_type eq "advanced_zoned" or $checksum_type eq "mixed") {
            RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>"priv set -q test; raid_flush_zcsv2_cache $filer_vol");
        }

        RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>"priv set -q test; wafl flush cache", timeout => 120);

        my $was_hammer_running = 0;
        if(RaidSqa_hammer_status(api_set_obj => $api_set_obj) == 1){
            RaidSqa_hammer_pause(api_set_obj => $api_set_obj);
            $was_hammer_running = 1;
        }

        #TODO: Is this required for flushing cache? This would detect errors unnecessarily in case of advanced_zoned aggrs
        RaidSqa_filer_cmd(api_set_obj=>$api_set_obj, cmd=>"priv set -q diag; vol offline $filer_vol; vol online $filer_vol", timeout => 120);
        if (!RaidSqa_aggr_offline(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type)) {
            die("flush_cache: Error: failed to offline the vol/aggr $filer_vol");
        }
        sleep(2);
        if (!RaidSqa_aggr_online(api_set_obj => $api_set_obj, filer_vol => $filer_vol, aggr_type => $aggr_type)) {
            die("flush_cache: Error: failed to online the vol/aggr $filer_vol");
        }
        if($was_hammer_running == 1){
            RaidSqa_hammer_restart(api_set_obj => $api_set_obj);
        }
    }

    sub remove_array_item {
        my $self = shift;
        my (%opts) = validate(@_, {
                        array => {type=>ARRAYREF},
                        item  => {type=>SCALAR},
                    });
        my $arrayref = $opts{array};
        my $item = $opts{item};

        my $index = 0;
        while ($index <= $#{$arrayref} ) {
            my $value = ${$arrayref}[$index];
            if ( $value eq $item ) {
                splice @{$arrayref}, $index, 1;
            } else {
                $index++;
            }
        }
        return $arrayref;
    }

    sub get_raid_info_object {
        my $self = shift;
        my $api_set_obj = $self->{api_set_obj};
        my $conntype = ($api_set_obj->{set} eq "CMode") ? "#ssh" : "#rsh";
        my $conn_object = undef;

        if(defined($api_set_obj->{connection})) {
            $conn_object = $api_set_obj->{connection};
        } elsif(defined($api_set_obj->{hostobj})) {
            $conn_object = make_conn($api_set_obj->{hostobj}, $conntype);
        } else {
            die("get_raid_info_object: Error: Create connection object failed");
        }

        my $raid_object = new RaidInfo($conn_object);
        if(!defined($raid_object)){
            die("get_raid_info_object: Error: Create RaidInfo object failed");
        }
        return $raid_object;
    }

    sub is_vsim {
        my $self = shift;
        my $api_set_obj = $self->{api_set_obj};
        my $host = undef;
        if(defined($api_set_obj->{hostobj})) {
            if($api_set_obj->{hostobj}->{hosttype} =~ /sim/) {  return 1 };
        } elsif(defined($api_set_obj->{connection})) {
            if($api_set_obj->{connection}->{hostp}->{hosttype} =~ /sim/) {  return 1};
        }
    }

    sub get_disk_active_path {
        my $self = shift;
        my %opts = validate(@_, {
                        raid_object => {type => OBJECT, isa=>'RaidInfo', deafault=>$self->get_raid_info_object(), optional=>1},
                        disk_name   => {type => SCALAR},
                    });

        my $raid_object = $opts{raid_object};
        my $disk_name   = $opts{disk_name};
        my %disks_hash  = $raid_object->all_disks();

        if(exists $disks_hash{$disk_name}) {
            return $disk_name;
        } else {
            foreach my $disk (keys %disks_hash){
                if($disk_name eq $disks_hash{$disk}{"path_name2"}){
                    return $disk;
                }
            }
        }
    }



1;

