#$Id: //depot/prod/test/Rsierranevada/lib/RaidSqa_Cmodev3.pm#10 $
##
## @summary $pkg_name - RAID SQA library for commands and utilities for Cmode
##
## @status TESTME
##
## @author pravin.salgond@netapp.com; akanksha.sharma@netapp.com
##
## @change Test library for implementing NACL complaint code across raidsqa_cmode
## @change 2012-02-28 vivienne@netapp.com: Burt 577669 Added new subroutine RaidSqa_get_aggr_rtid
## @change 2012-08-13 msenthil@netapp.com: Burt 563902 Added label_buildtree related APIs
## @change 2013-02-11 naushad.syed@netapp.com: Burt 671291 Modified RaidSqa_aggr_options() to change aggr state from cluster shell
## @change 2013-02-12 naushad.syed@netapp.com: Burt 672607 Modified RaidSqa_aggr_add() for passing disklist for disk addition
## @change 2013-02-25 anupama.ganji@netapp.com: Burt 680142 Modified RaidSqa_check_aggr_after_isolate() for passing dual_path names
##
## @pod here

package RaidSqa_Cmodev3;
use strict;
no strict "subs";
use warnings;
use RaidInfo;
use Carp qw(confess croak);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use Tharn qw(say script_result $Log logcomment logresult param);
use Module::Common qw(validate_mixed);
use mount;
use Utils qw(make_conn);
use Data::Dumper;
use filer;
use boot;
use Shell qw(chmod sudo cat);
use Digest::MD5;

use NACL::APISet;
use NACL::Transit;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::Transit::Exceptions::TransitException qw(:try);
use NACL::STask::Aggregate;


BEGIN {
    Params::Validate::validation_options(
        on_fail => sub {
            my @msgs = @_;
            chomp(@msgs);
            Carp::confess(Tharn::script_result(@msgs));
        });
}

use vars qw(@ISA @EXPORT $CLIENT @Nodes $Node);
my $RaidSqa_TIMEOUT = 300;
my $RaidSqa_CLIENT = defined($CLIENT) ? $CLIENT : 'localhost';
my $pkg_name         = "RaidSqa_Cmodev3.pm";
my @RAIDSQA_STORAGE_TYPES = qw(FCAL SAS FSAS ATA BSAS SATA MSATA SSD LUN);

#This has contains all v2.0 licenses
my %DB_LICENSE = (
    base             => 'ETYYFLXUQUMADFAAAAAAAAAAAAAA',
    cdmi             => 'WSQFRWDUSAFDAAAAAAAAAAAAAAAA',
    cifs             => 'MJXMPWDUSAFDAAAAAAAAAAAAAAAA',
    fcp              => 'KYSQQWDUSAFDAAAAAAAAAAAAAAAA',
    flexclone        => 'GCKYSWDUSAFDAAAAAAAAAAAAAAAA',
    iscsi            => 'YDVBQWDUSAFDAAAAAAAAAAAAAAAA',
    nfs              => 'APZXOWDUSAFDAAAAAAAAAAAAAAAA',
    snaplock         => 'ERFCUWDUSAFDAAAAAAAAAAAAAAAA',
    snapmanagersuite => 'QLDRUWDUSAFDAAAAAAAAAAAAAAAA',
    snapmirror       => 'UHMJSWDUSAFDAAAAAAAAAAAAAAAA',
    snapprotectapps  => 'CGBGVWDUSAFDAAAAAAAAAAAAAAAA',
    snaprestore      => 'INOURWDUSAFDAAAAAAAAAAAAAAAA',
    snapvault        => 'SWHNTWDUSAFDAAAAAAAAAAAAAAAA'
);

#Export subroutines
@ISA    = qw(Exporter);
@EXPORT = qw (
  RaidSqa_aggr_create
  RaidSqa_flexvol_create
  RaidSqa_flexvol_resize
  RaidSqa_get_aggr_show_space
  RaidSqa_aggr_add
  RaidSqa_aggr_rename
  RaidSqa_aggr_offline
  RaidSqa_aggr_online
  RaidSqa_aggr_restrict
  RaidSqa_aggr_destroy
  RaidSqa_aggr_remove
  RaidSqa_aggr_mount
  RaidSqa_aggr_scrub_start
  RaidSqa_aggr_scrub_suspend
  RaidSqa_aggr_scrub_resume
  RaidSqa_aggr_scrub_stop
  RaidSqa_aggr_scrub_status
  RaidSqa_aggr_wafliron_start
  RaidSqa_aggr_wafliron_commit
  RaidSqa_aggr_wafliron_review
  RaidSqa_aggr_wafliron_status
  RaidSqa_aggr_media_scrub_start
  RaidSqa_aggr_media_scrub_stop
  RaidSqa_aggr_media_scrub_status
  RaidSqa_aggr_verify_stop
  RaidSqa_test_init
  RaidSqa_media_error_inject
  RaidSqa_media_scrub_enable
  RaidSqa_media_scrub_disable
  RaidSqa_filer_cmd
  RaidSqa_file_create
  RaidSqa_file_nfs_read
  RaidSqa_get_md5sum
  RaidSqa_dd
  RaidSqa_log
  RaidSqa_savecore
  RaidSqa_nv_trash
  RaidSqa_disk_online
  RaidSqa_disk_offline
  RaidSqa_disk_fail
  RaidSqa_disk_unfail
  RaidSqa_disk_remove
  RaidSqa_disk_mung
  RaidSqa_disk_zero_spares
  RaidSqa_disk_assign
  RaidSqa_disk_remove_ownership
  RaidSqa_disk_replace_start
  RaidSqa_diskbypass
  RaidSqa_find_active_path_disk_name
  RaidSqa_check_license
  RaidSqa_license_add_by_code
  RaidSqa_license_add_by_service
  RaidSqa_license_delete
  RaidSqa_fiji_show
  RaidSqa_check_fiji
  RaidSqa_get_fiji_slot
  RaidSqa_fiji_clear
  RaidSqa_fiji_filter
  RaidSqa_fiji_nvfilter
  RaidSqa_fiji_nvclear
  RaidSqa_hammer_start
  RaidSqa_hammer_pause
  RaidSqa_hammer_restart
  RaidSqa_hammer_abort
  RaidSqa_hammer_status
  RaidSqa_check_hammer_running
  RaidSqa_get_options
  RaidSqa_set_options
  RaidSqa_get_aggr_options
  RaidSqa_aggr_options
  RaidSqa_get_setflag
  RaidSqa_setflag
  RaidSqa_set_raid_lw_opt
  RaidSqa_set_aggr_raid_lost_write
  RaidSqa_get_root_vol
  RaidSqa_get_root_aggr
  RaidSqa_get_fileid
  RaidSqa_get_file_block_info
  RaidSqa_get_dumpblock_info
  RaidSqa_get_raid_size_value
  RaidSqa_get_scrub_report_by_group
  RaidSqa_get_preferred_storage_type
  RaidSqa_get_vserver_root
  RaidSqa_get_vserver_ip_addr
  RaidSqa_get_volume_junc_path
  RaidSqa_get_aggr_list
  RaidSqa_get_aggr_status
  RaidSqa_get_aggr_type
  RaidSqa_get_aggr_cksum_type
  RaidSqa_get_aggr_raidtree_type
  RaidSqa_get_aggr_storage_type
  RaidSqa_get_aggr_fsm_state
  RaidSqa_get_aggr_mirror_state
  RaidSqa_get_aggr_fsid
  RaidSqa_get_aggr_uuid
  RaidSqa_get_aggr_rtid
  RaidSqa_get_aggr_bits
  RaidSqa_get_aggr_context_parity_valid
  RaidSqa_get_aggr_has_bad_blocks
  RaidSqa_get_aggr_bad_block_count
  RaidSqa_get_aggr_tetris_opt_disabled
  RaidSqa_get_aggr_raid_lw_enabled
  RaidSqa_get_aggr_raid_lw_allowed
  RaidSqa_get_vol_status
  RaidSqa_get_plex_list
  RaidSqa_plex_destroy
  RaidSqa_get_one_plex_of_state
  RaidSqa_get_group_list
  RaidSqa_get_group_context_parity_valid
  RaidSqa_get_group_fsm_state
  RaidSqa_get_group_checksum_type
  RaidSqa_get_group_list_by_fsm_state
  RaidSqa_get_group_raid_lw_scrub
  RaidSqa_get_group_recons_suspended_block
  RaidSqa_get_group_scrub_suspended_block
  RaidSqa_get_disk_list
  RaidSqa_get_disk_list_by_fsm_state
  RaidSqa_get_disk_list_by_parent_state
  RaidSqa_get_disk_list_by_raid_type
  RaidSqa_get_disk_list_by_storage_type
  RaidSqa_get_disk_list_by_disk_type
  RaidSqa_get_disk_list_by_checksum_type
  RaidSqa_get_disk_list_by_pool
  RaidSqa_get_disk_list_by_rpm
  RaidSqa_get_disk_fsm_state
  RaidSqa_get_disk_parent_state
  RaidSqa_get_disk_storage_type
  RaidSqa_get_disk_aggr_name
  RaidSqa_get_disk_plex_name
  RaidSqa_get_disk_group_name
  RaidSqa_get_disk_rgdn
  RaidSqa_get_disk_used_size
  RaidSqa_get_disk_sdc_target
  RaidSqa_get_disk_other_path_name
  RaidSqa_get_disk_offline_ref_count
  RaidSqa_get_invalid_diskid
  RaidSqa_get_bypassed_disk_list
  RaidSqa_check_existing_aggr_ok
  RaidSqa_check_aggr_exist
  RaidSqa_check_aggr_exist_anytype
  RaidSqa_check_aggr_state_name
  RaidSqa_check_aggr_fsm_state
  RaidSqa_check_aggr_mirror_state
  RaidSqa_check_aggr_after_isolate
  RaidSqa_check_aggr_after_unisolate
  RaidSqa_check_aggr_lw_flags
  RaidSqa_check_plex_exist
  RaidSqa_check_plex_fsm_state
  RaidSqa_check_plex_state_name
  RaidSqa_check_plex_resyncing
  RaidSqa_check_group_exist
  RaidSqa_check_group_scrub_in_progress
  RaidSqa_check_group_recons_in_progress
  RaidSqa_check_group_reparity_in_progress
  RaidSqa_check_group_scrub_is_suspended
  RaidSqa_check_group_recons_is_suspend
  RaidSqa_check_group_is_running_context_scrub
  RaidSqa_check_group_lw_flags
  RaidSqa_check_disk_exist
  RaidSqa_check_disk_fsm_state
  RaidSqa_check_disk_parent_state
  RaidSqa_check_disk_is_prefailed
  RaidSqa_check_disk_sdc_start
  RaidSqa_check_disk_is_doing_optimized_recons
  RaidSqa_check_disk_block_is_zeroed_out
  RaidSqa_check_disk_block_is_pseudo
  RaidSqa_check_disk_block_is_unverified
  RaidSqa_check_disk_after_isolate
  RaidSqa_check_disk_after_unisolate
  RaidSqa_check_raid_size_valid_range
  RaidSqa_check_ioc_support
  RaidSqa_wait_for_scrub_completion
  RaidSqa_wait_for_aggr_disks_zeroing_completion
  RaidSqa_wait_for_reparity_completion
  RaidSqa_wait_for_reconstruction_completion
  RaidSqa_wait_for_disk_sdc_completion
  RaidSqa_wait_for_spare_disks_zeroing_completion
  RaidSqa_get_labelmaint_list
  RaidSqa_labelmaint_isolate
  RaidSqa_labelmaint_unisolate
  RaidSqa_get_label_vuinfo
  RaidSqa_get_label_examine
  RaidSqa_get_label_buildtree_cmd_string
  RaidSqa_label_buildtree
  RaidSqa_get_storage_failover_field
  RaidSqa_enable_storage_failover_status
  RaidSqa_enable_ha
  RaidSqa_disable_ha
  RaidSqa_get_ha_state
  RaidSqa_verify_storage_failover_status
  RaidSqa_set_storage_failover_field
  RaidSqa_giveback
  RaidSqa_takeover
  RaidSqa_wait_for_cf_status
  RaidSqa_wait_for_storage_failover_state
  RaidSqa_ems_log_get_mark
  RaidSqa_ems_log_dump_by_type
  RaidSqa_ems_log_dump_by_id
  RaidSqa_ems_log_check
  RaidSqa_ems_get_value_by_key
  RaidSqa_transit_to
  RaidSqa_boot_state
  RaidSqa_systemshell_cmd
  RaidSqa_set_snmp_community
  RaidSqa_validate
  RaidSqa_root_recovery
  RaidSqa_create_hybrid_aggr
  RaidSqa_print_status
  RaidSqa_disk_simpull
  RaidSqa_disk_simpush
  RaidSqa_get_pulled_disks
  RaidSqa_get_storage_type_family
  RaidSqa_sort_disks
  RaidSqa_get_mixed_storage_type_disks
  );

#====================================================
# TAG :: AGGR OPS
#====================================================

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_create

RaidSqa_aggr_create(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, disk_count=>$disk_count, raid_type=>$raid_type, raid_size=>$raid_size, mirror=>$mirror, storage_type=>$storage_type, rpm=>$rpm, aggr_type=>$aggr_type, aggr_bits=>$aggr_bits, node => $node);

-or-

RaidSqa_aggr_create(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, disk_list=>\@disk_list, disk_list1=>\@disk_list1, raid_type=>$raid_type, raid_size=>$raid_size, mirror=>$mirror, storage_type=>$storage_type, rpm=>$rpm, aggr_type=>$aggr_type, node => $node);

-or-

RaidSqa_aggr_create(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, disk_count=>$disk_count, raid_type=>$raid_type, raid_size=>$raid_size, mirror=>$mirror, storage_type=>$storage_type, aggr_type=>$aggr_type, rpm=>$rpm, more_options=>$more_options, node => $node);

This subroutine create the test aggregate.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate  or traditional volume to be created.

=item options

(Optional) : We can use this to define all options needed for vol/aggr create if we do not want to specify all the paremeters to this function instead want to create options in the script.

=item disk_count

(Optional) : Number of disk to be used for volume/aggegate creation. Need to define at least on parameter from disk_count and disk_list for volume creation, if none is defined, it will use disk_count as 6

=item disk_list

(Optional) : Reference to the list of disks to be used for volume creation. Need to define at least one parameter from disk_count or disk_list. But can not defined both.

=item disk_list1

(Optional) : Reference to the list of 2nd set of disks. Need to define this if mirror is 1 and disk_list is also defined. Can not define if disk_count is defined. For Cmode as mirror is not supported, we will just ignore this.

=item raid_type

(Optional): Raid type of the volume/aggregate to be created. By default it is raid_dp. Other options is raid4.

=item raid_size

(Optional): Raid size of the volume/aggregate to be created. By default it is default. Other values can be min, max or user_defined count but that should be in allowed range for a given storage_type.

=item mirror

(Optional): 1 for mirror volume and 0 for nonmirror. Default is 0. For Cmode as mirror is not supported, we will set it to 0

=item storage_type

(Optional):  Disk storage type. Can be ATA, SAS, FCAL. If nothing is defined  than it tries to find the prefered storage type and use that type of disk.

=item checksum_type

(Optional):  Aggr checksum type. Can be block, zoned, advanced_zoned, default is block

=item rpm

(Optional):  Disk rpm type. Can be 7200, 10000, 15000. If nothing is defined than it use any rpm disks to create volume/aggr.

=item aggr_type

(Optional): 1 for aggregate and 0 for traditional. Default it is 1. For Cmode as traditional is not supported, we will set it to 1

=item more_options

(Optional) : We can use this to define more options needed for vol/aggr create.

=item aggr_bits

(Optional): 32 for normal volume/aggr, 64 for larger aggregate. Default it is 64.

=item fake_la

(Optional): 1 for fake larger aggregate of size > 16TB and 0 for not.

=item fake_rg_max_count

(Optional): Default is 3, it controls the env variable needed to create fake larger aggregate.

=item fake_vol_size

(Optional): The value of fake larger flex volume in TB.

=item vserver

(Optional): Name of vserver for volume to be created, default is vs0

=item security_style

(Optional): Security style of volume. default is mixed.

=item hybrid

(Optional): Set this value if aggregate created has to be converted to HyA.Default is 0. Possible values: 1, 0.

=item hybrid_tier

(Optional): Set this value if tests are to be run on particular tier of HyA.Default is HDD. Possible values: SSD, HDD.

=item hya_storage_type

(Optional):  Disk storage type. Default value: undef.Possible values: SSD, BSSD. If not defined, then one of the type of SSD disk will be chosen.
=item unix_permissions

(Optional): unix permission of the volume, default is 0755

=item mirror_skip_config_checks

(Optional): Possible values 0,1. Default is 1.
            If 1, options raid.mirror_skip_config_checks will be set to ON before aggregate is mirrored
            If 0 then no action is taken

=item ssd_raid_type

(Optional): Possible values: raid4,raid_dp.Default is undef
             This option when set to different value other than RAID_TYPE will create MRT aggr.If undef is set, aggregate created will be of raidtype as RAID_TYPE.
             For eg : if RAID_TYPE=raid_dp and SSD_RAID_TYPE=raid4; aggregate created will be of Mixed Raid Type.

=item node

(Optional): Node on which the aggregate is to be created
Implementation still pending in function

## Added following two params:
node - to create aggregate on a particular node, default is undef
vol_reqd - if voluem inside aggregate needs to be created, default is 1

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_create {

    my (%opts) =
      RaidSqa_validate(@_,
        {
            api_set_obj   => {type => OBJECT, isa     => 'NACL::APISet'},
            conntype      => {type => SCALAR, default => "#ssh", optional => 1},
            filer_vol     => {type => SCALAR},
            mirror        => {type => SCALAR, default => 0,     optional => 1},
            disk_count    => {type => SCALAR   | UNDEF, default => undef, optional => 1},
            disk_size     => {type => SCALAR   | UNDEF, default => undef, optional => 1},
            disk_list     => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
            disk_list1    => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
            raid_size     => {type => SCALAR, default  => "default", optional => 1},
            raid_type     => {type => SCALAR, default  => "raid_dp", optional => 1},
            storage_type  => {type => SCALAR   | UNDEF, default => undef, optional => 1},
            checksum_type => {type => SCALAR   | UNDEF, default => undef, optional => 1},
            aggr_type     => {type => SCALAR, default => 1,     optional => 1},
            aggr_bits     => {type => SCALAR, default => 64,    optional => 1},
            rpm           => {type => SCALAR   | UNDEF, default  => undef, optional => 1},
            snaplock      => {type => SCALAR, default => 0,     optional => 1},
            more_options  => {type => SCALAR   | UNDEF, default  => undef, optional => 1},
            la            => {type => SCALAR, default => 0,     optional => 1},
            fake_la       => {type => SCALAR, default => 0,     optional => 1},
            fake_vol_size => {type => SCALAR, default => 24,    optional => 1},
            vserver       => {type => SCALAR, default => "vs0", optional => 1},
            vol_reqd      => {type => SCALAR, default => 1,     optional => 1},
            allow_same_carrier => {type => SCALAR, default => undef,       optional => 1},
            node          => {type => SCALAR | ARRAYREF, default => "local", optional => 1},
            hybrid        => {type => SCALAR, default => 0,     optional => 1},
            hybrid_tier   => {type => SCALAR, default => "HDD", optional => 1},
            hya_storage_type  => {type => SCALAR | UNDEF, default => undef, optional => 1},
            ssd_raid_type     => {type => SCALAR | UNDEF, default => undef, optional => 1},
            security_style    => {type => SCALAR, default => "mixed", optional => 1},
            desirable_layout  => {type => SCALAR,         default => 1,     optional => 1},
            unix_permissions  => {type => SCALAR, default => "0777",  optional => 1},
            fake_rg_max_count => {type => SCALAR, default => 3,       optional => 1},
            mirror_skip_config_checks => {type => SCALAR, default => 1, optional => 1},
            timeout           => {type => SCALAR, default => 300,       optional => 1},
        }
    );

    my $api_set_obj       = $opts{api_set_obj};
    my $conntype          = $opts{conntype};
    my $filer_vol         = $opts{filer_vol};
    my $disk_count        = $opts{disk_count};
    my $disk_size         = $opts{disk_size};
    my $disk_list         = $opts{disk_list};
    my $disk_list1        = $opts{disk_list1};
    my $raid_type         = $opts{raid_type};
    my $raid_size         = $opts{raid_size};
    my $mirror            = $opts{mirror};
    my $storage_type      = $opts{storage_type};
    my $checksum_type     = $opts{checksum_type};
    my $rpm               = $opts{rpm};
    my $aggr_type         = $opts{aggr_type};
    my $snaplock          = $opts{snaplock};
    my $more_options      = $opts{more_options};
    my $aggr_bits         = $opts{aggr_bits};
    my $fake_la           = $opts{fake_la};
    my $fake_rg_max_count = $opts{fake_rg_max_count};
    my $fake_vol_size     = $opts{fake_vol_size};
    my $vserver           = $opts{vserver};
    my $security_style    = $opts{security_style};
    my $unix_permissions  = $opts{unix_permissions};
    my $node              = $opts{node};
    my $vol_reqd          = $opts{vol_reqd};
    my $la                = $opts{la};
    my $hybrid            = $opts{hybrid};
    my $hybrid_tier       = $opts{hybrid_tier};
    my $hya_storage_type  = $opts{hya_storage_type};
    my $ssd_raid_type     = $opts{ssd_raid_type};
    my $desirable_layout  = $opts{desirable_layout};
    my $allow_same_carrier= $opts{allow_same_carrier};
    my $timeout           = $opts{timeout};
    my $mirror_skip_config_checks = $opts{mirror_skip_config_checks};

    my ($response_obj, $raw_output, $fake_add_disk_count, $host, $conn);
    my (@parsed_output, @fake_disk_add_list, @disk_list, @disk_list1, %aggr_create_hash);
    my $filer       = $api_set_obj->hostobj->hostname();

    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    ## other variables - defined for more options passed
    if((defined $checksum_type) && lc $checksum_type ne "undef") {
        $checksum_type = lc $checksum_type;
        if(!(grep /$checksum_type/, ("block", "zoned", "advanced_zoned"))) {
            $Log->comment("$pkg_name: Unrecognized checksum_type is passed to subroutine: $checksum_type");
            return 0;
        }
    }
    if($snaplock) {
        $Log->comment("$pkg_name: Snaplock is not supported for Cmode, hence ingoring snaplock");
    }

    ## other variables - defined for more options passed
    my $cksum_style;
    if((defined $checksum_type)&& ($checksum_type ne "undef")) {
        if(grep /$checksum_type/, ("block", "zoned", "advanced_zoned")) {
            $cksum_style         = lc $checksum_type;
        } else {
            $Log->comment("$pkg_name: Unrecognized checksum_type is passed to subroutine: $checksum_type");
            return 0;
        }
    }

    #Fail for incorrect values of subroutine params
    if (!$aggr_type) {
        $Log->comment("$pkg_name: trad_vol is not supported for Cmode, need to pass aggr_type as 1");
        return 0;
    }

    my $ignore_inconsistent = ($more_options =~ /ignore-inconsistent/) ? "on"      : undef;
    my $snap_autodelete     = ($more_options =~ /snap-autodelete/)     ? "on"      : undef;
    my $thorough_scrub      = ($more_options =~ /thorough-scrub/)      ? "on"      : undef;
    my $volume_style        = ($more_options =~ /volume-style/)        ? "striped" : undef;
    my $vol_type            = ($more_options =~ /voltype/)             ? "striped" : undef;
    my $allow_mixed         = ($more_options =~ /-f/)                  ? "true"    : undef;

    # dereference array of disk_list if specified
    @disk_list = @$disk_list if ($disk_list);
    @disk_list1 = @$disk_list1 if ($disk_list1);

    # use default disk_count if disk_list and disk_count both are not given, also check that user did not specified both together
    if ((!$disk_count) && (!@disk_list)) {
        $disk_count = 6;
    } elsif (($disk_count) && (@disk_list)) {
        $Log->comment(
            "$pkg_name: Need to provide only one parameter disk_count or disk_list to aggr create argument"
        );
        return 0;
    }

    # Get preferred storage type if not specified by user
    if (!defined($storage_type)) {
        $Log->comment("$pkg_name: Getting storage type, as not defined");
        $storage_type = RaidSqa_get_preferred_storage_type(api_set_obj => $api_set_obj, checksum_type=>$checksum_type, timeout => $timeout);
        $Log->comment("$pkg_name: Preferred storage type is $storage_type");
        $desirable_layout = 0;
    } else {
        if(!grep /$storage_type/i, @RAIDSQA_STORAGE_TYPES) {
            $Log->comment("$pkg_name: Invalid storage type defined: $storage_type");
            return 0;
        }
    }

    #Set 'raid.mirror_skip_config_checks' to ON to allow aggr create/mirror [BURT #525715]
    #Info: SN onwards, first mirror aggr creation on node fails if there are disks from same adapter in each pool
    #      However for testing purpose, we can skip this check by setting option  'raid.mirror_skip_config_checks' to on.

    if($mirror == 1 && $mirror_skip_config_checks == 1) {
        $Log->comment("$pkg_name: Setting value of an option to 'ON' - 'raid.mirror_skip_config_checks'");
        if(!RaidSqa_set_options(
            api_set_obj  => $api_set_obj,
            option_name  => 'raid.mirror_skip_config_checks',
            option_value => 'on',
            timeout      => $timeout,
            node         => $node
        )) {
            $Log->warn("$pkg_name: Setting value of an option to 'ON' failed - 'raid.mirror_skip_config_checks'");
            #return 0;
        }
    }

    # get raid_size value for min/max/default raid_size and if a user defined size is given than check that it is in valid range
    my %raid_size_args = (
        api_set_obj  => $api_set_obj,
        raid_size    => $raid_size,
        raid_type    => $raid_type,
        storage_type => $storage_type,
        timeout      => $timeout,
        node         => $node
    );

    if ($raid_size =~ /\D/) {
        $raid_size = RaidSqa_get_raid_size_value(%raid_size_args);
        $Log->comment("$pkg_name: using raid size as $raid_size");
    } else {
        if (!RaidSqa_check_raid_size_valid_range(%raid_size_args)) {
            $Log->comment("$pkg_name: raid_size $raid_size is not in the valid range");
            return 0;
        }
    } ## end else [ if ($raid_size =~ /\D/)

    #Check values of parameter requuired for fake_la
    if ($fake_la) {
        if ($aggr_bits != 64) {
            $Log->comment("$pkg_name: fake_la is supported only for 64-bit aggregates");
            return 0;
        }
        if ($fake_rg_max_count < 3) {
            $Log->comment("$pkg_name: for fake_la value of fake_rg_max_count should be >= 3");
            return 0;
        }
        if ($fake_vol_size < 24) {
            $Log->comment("$pkg_name: For fake_la, value of fake_vol_size should be >= 24");
            return 0;
        }
        if ($raid_size <= 3) {
            $Log->comment("$pkg_name: Increasing raid size by 2 for fake_la");
            $raid_size = $raid_size + 2;
        }

        try {
            #-------------------------------------------------------------------
            #TODO:: To fix this code if $node if array ref
            #
            $api_set_obj->set(privilege => "test");
            my @commands = (
                "bootargs set bootarg.dblade.fake_large_aggr true",
                "bootargs set raidtop_fake_rg_max_count $fake_rg_max_count",
            );

            foreach my $command (@commands) {
                $api_set_obj->system_node_run(
                                node    => $node,
                                command => $command,
                                'connectrec-timeout' => $timeout,
                            );
            }
        } catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            _handle_nacl_exception($exception_object);
            return 0;
        };

    } ## end if ($fake_la)

    if ((defined($rpm)) && (lc $allow_mixed eq "true")) {
        $Log->comment("$pkg_name: rpm value can not be passed when allow-mixed is set to true");
        return 0;
    }

    if ($disk_list) {
        if($mirror == 1 && !defined($disk_list1)) {
            $Log->comment("$pkg_name: disk_list1 is required for mirrorred aggr");
        }

        if ($fake_la) {
            my $index = ($raid_type eq "raid_dp") ? 3 : 2;
            @fake_disk_add_list = @disk_list[$index .. $#disk_list];
            @disk_list          = @disk_list[0 .. ($index-1)];
            if ($#fake_disk_add_list <= -1) {
                $Log->comment("$pkg_name: Number of disks available for addition are less than required for fake_la");
                return 0;
            }
        } ## end if ($fake_la)
    } else {
         $disk_count = $raid_size * (1 + $mirror) if ($disk_count eq "default");
        if ($fake_la) {
            my $fake_la_min_disk_count = ($raid_type eq "raid_dp") ? 3 : 2;
            $fake_add_disk_count = $disk_count - $fake_la_min_disk_count;
            $disk_count          = $fake_la_min_disk_count;
            if ($fake_add_disk_count <= 0) {
                $Log->comment("$pkg_name: Number of disks available for addition are less than required");
                return 0;
            }
        } ## end if ($fake_la)

        # my @spares_list =
          # RaidSqa_get_disk_list_by_storage_type(api_set_obj  => $api_set_obj,
                                                # spare_only   => "yes",
                                                # storage_type => $storage_type);
        # if(defined($checksum_type)&& ($checksum_type ne "undef")) {
            # @spares_list =
                  # RaidSqa_get_disk_list_by_checksum_type(api_set_obj  => $api_set_obj,
                                    # conntype      => $conntype,
                                    # disk_list     => \@spares_list,
                                    # checksum_type => $checksum_type);
        # }
        # my $spares_count = @spares_list;

        # # create a disk list from the disk_count, so that it can be passed to vol_create_args using -d option
        # my @spare_used_list = ();
        # for (my $n = $disk_count ; $n > 0 ; $n--) {
            # $spare_used_list[$n] = shift @spares_list;
        # }
    } ## end else [ if ($disk_list)

    $Log->comment("$pkg_name: create testing aggr/volume $filer_vol");

    if ($fake_la) {
        $Log->comment(
            "$pkg_name: creating fake_la with 1st minimun number of disk and".
            " will add remaining disk after creating a hole"
        );
    }

    # here we are actually creating the aggregate
    # two blocks here -- for disk_count and disk-list

    %aggr_create_hash = (
        'aggregate'          => $filer_vol,
        'raidtype'           => $raid_type,
        'maxraidsize'        => $raid_size,
        'connectrec-timeout' => 2*$timeout,
    );

    #TODO :: add this options and check why is it getting stuck
    $aggr_create_hash{'block-type'} = $aggr_bits if(defined($aggr_bits));
    $aggr_create_hash{'ignore-inconsistent'} = $ignore_inconsistent if(defined($ignore_inconsistent));
    $aggr_create_hash{'snap-autodelete'} = $snap_autodelete if(defined($snap_autodelete));
    $aggr_create_hash{'thorough-scrub'} = $thorough_scrub if(defined($thorough_scrub));
    $aggr_create_hash{'volume-style'} = $volume_style if(defined($volume_style));
    #$aggr_create_hash{'voltype'} = $vol_type if(defined($vol_type));
    $aggr_create_hash{'mirror'}       = "true" if($mirror == 1 && !defined($disk_list));
    $aggr_create_hash{'disktype'} = $storage_type if(!defined($disk_list) && defined($storage_type));

    if(defined($checksum_type) && !defined($disk_list) && ($checksum_type ne "undef")) {
        $aggr_create_hash{chksumstyle} = $cksum_style;
    }

    #Adding nodes information
    my (@nodes, $param_node_value);

    if (ref($node) =~ /array/i) {
        @nodes = @$node;
        $param_node_value = join(",", @nodes);
    } else {
        @nodes = ($node);
        $param_node_value = $node;
    }

    if (!defined($disk_list)) {
        $aggr_create_hash{nodes} = $param_node_value;
    }

    if (defined($disk_count)) {
        $aggr_create_hash{'diskcount'} = $disk_count;
        if ((defined($disk_size)) && (defined($rpm))) {
            ## we may have to find the list of particular disk size here
            $aggr_create_hash{'disksize'} = $disk_size;
            $aggr_create_hash{'diskrpm'}  = $rpm;
        } else {
            $aggr_create_hash{'allow-mixed'} = $allow_mixed;
            $aggr_create_hash{'disksize'} = $disk_size if defined($disk_size);
            $aggr_create_hash{'diskrpm'} = $rpm if defined($rpm);
        }
    } else {
        ## disk-list code
        $aggr_create_hash{'allow-mixed'} = $allow_mixed;
        $disk_list[0] = "$filer:".$disk_list[0];
        $aggr_create_hash{'disklist'}    = join(",$filer:", @disk_list);
        $disk_list1[0] = "$filer:".$disk_list1[0] if(@disk_list1);
        $aggr_create_hash{'mirror-disklist'} = join(",$filer:", @disk_list1) if(@disk_list1);
        $aggr_create_hash{'allow-same-carrier'} = 'true' if (defined($allow_same_carrier));
    }

    #This is not applicable anymore, keeping it commented if older script needs it
    if($more_options =~ /-z/) {
        $aggr_create_hash{'pass_through'} = $more_options;
    }

    #---------------------------------------------------------------------------
    #Create aggregate
    try {
        $api_set_obj->storage_aggregate_create(%aggr_create_hash);
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };

    my $raid_obj;
    foreach my $node (@nodes) {

        # Check test aggr/volume got created
        sleep 30;
        $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $aggr_obj = $raid_obj->aggregate($filer_vol);
        if (!$aggr_obj) {
            $Log->comment("$pkg_name: Aggr creation appears to have failed");
            return 0;
        }

        # wait for zeroing to finish
        RaidSqa_wait_for_aggr_disks_zeroing_completion(api_set_obj => $api_set_obj,
                                                       filer_vol   => $filer_vol,
                                                       timeout     => $timeout,
                                                       node        => $node);
    }

    ## TODO : Fix this part for $node being array ref
    ## This code is for FAKE_LA -- needs to be ported
    if ($fake_la) {
        $Log->comment(
            "$pkg_name: for fake_la creating a hole of $fake_vol_size TB and than adding remaining disks "
        );
        my $disk_size = int($fake_vol_size * 1024 * 1024 * 256);
        my %aggr_add_args = (
            api_set_obj => $api_set_obj,
            conntype    => $conntype,
            filer_vol   => $filer_vol,
            options     => "-s $disk_size -f",
            timeout     => $timeout,
            node        => $node
        );

        if ($disk_list) {
            $aggr_add_args{disk_list} = \@fake_disk_add_list;
        } else {
            $aggr_add_args{disk_count} = $fake_add_disk_count;
        }
        RaidSqa_aggr_add(%aggr_add_args);
    } ## end if ($fake_la)

    # Create flex volume in case test volume is aggr, having same name of aggregate with 3/4th of its size, except for fake_la
    $raid_obj->refresh();
    my $junc_path = "/" . "$filer_vol";
    my $aggr_obj = $raid_obj->aggregate($filer_vol);
    if (!$aggr_obj) {
        $Log->comment("$pkg_name: aggr $filer_vol does not exist");
        return 0;
    }

    if ($vol_reqd) {
        my %vol_create_args = (
            vserver              => $vserver,
            volume               => $filer_vol,
            aggregate            => $filer_vol,
            'security-style'     => $security_style,
            'unix-permissions'   => $unix_permissions,
            'junction-path'      => $junc_path,
            'connectrec-timeout' => 2*$timeout,
        );

        if ($fake_la) {
            my $aggr_size_in_gb        = $aggr_obj->parameter("raidsize") * 4 / (1024 * 1024);
            my $fake_vol_size_in_bytes = $fake_vol_size * 1024 * 1024 * 1024 * 1024;
            my $aggr_size_needed_in_gb = ($fake_vol_size_in_bytes * 3) / (100 * 1024 * 1024 * 1024);

            if ($aggr_size_in_gb < $aggr_size_needed_in_gb) {
                #TODO: fix "-s none" part of server->cli command
                #$server->cli(cmd=>"volume create -vserver $vserver -volume $filer_vol -s none -aggregate $filer_vol -size $fake_vol_size_in_bytes  -security-style $security_style -unix-permissions $unix_permissions -junction-path $junc_path");
                $vol_create_args{size} = $fake_vol_size_in_bytes;
            } else {
                # Either use normal mode or use more disks to create fake_la volumes
                $Log->comment(
                    "$pkg_name: aggr $filer_vol size is $aggr_size_in_gb GB and to create fake_la of flex vol size $fake_vol_size TB, need aggr of size $aggr_size_needed_in_gb GB"
                );
                return 0;
            }
        } else {
            #TODO:: Fix: BURT: Insufficient space to enforce guarantee
            #my $size = 3072 * $aggr_obj->parameter("raidsize");
            my $size = 2500 * $aggr_obj->parameter("raidsize");
            $vol_create_args{size} = $size;
        }

        #Create volume
        try {
            $api_set_obj->volume_create(%vol_create_args);
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            _handle_nacl_exception($exception_object);
            return 0;
        };
        sleep 5;

        $raid_obj->refresh();
        my $vol_obj = $raid_obj->volume("$filer_vol");

        if (!$vol_obj) {
            $Log->comment("$pkg_name: Flexvol creation failed");
            return 0;
        } else {
            $Log->comment("$pkg_name: Flexvol creation is successful");
            #return 1;
        }
    } else {
        $Log->comment(
            "$pkg_name: Test did not request for creation of flex vol - aggregate is created; returning 1"
        );
        #return 1;
    }
    # Check for desirable layout in case storage_type is MSATA
    if ($storage_type =~ /MSATA/i) {
        if ($desirable_layout) {
            my $desirable_layout_flag = RaidSqa_check_desirable_layout(api_set_obj => $api_set_obj,
                                                                       timeout     => $timeout,
                                                                       filer_vol => $filer_vol);
            if (!$desirable_layout_flag) {
                die("$pkg_name: Aggr-$filer_vol is not in desired layout");
            }
        }
    } ## end if ($storage_type =~ /MSATA/i)
    if ($hybrid) {
        my $vol_disk_count;
        if (!defined($disk_count)) {
            my @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                                    conntype    => $conntype,
                                                    node        => $node,
                                                    timeout     => $timeout,
                                                    filer_vol   => $filer_vol);
            $vol_disk_count = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                                    conntype    => $conntype,
                                                    timeout     => $timeout,
                                                    node        => $node,
                                                    group_name  => $group_list[0]);

            $raid_size = $vol_disk_count if ($opts{raid_size} =~ /\D/);
        } else {
            $vol_disk_count = $disk_count;
            if ($opts{raid_size} =~ /\D/) {
                if ($mirror) {
                    $raid_size = ($vol_disk_count / 2);
                } else {
                    $raid_size = $vol_disk_count;
                }
            }
        } ## end else [ if (!defined($disk_count...
        $Log->comment("$pkg_name: DISK_COUNT for creating Hybrid aggr - $vol_disk_count");
        $Log->comment("$pkg_name: RAID_SIZE for creating Hybrid aggr - $raid_size");
        $Log->comment("$pkg_name: SSD_RAID_TYPE for creating Hybrid aggr - $ssd_raid_type");

        # Function call to create Hybrid aggr
        if (RaidSqa_create_hybrid_aggr(api_set_obj      => $api_set_obj,
                                       node             => $node,
                                       conntype         => $conntype,
                                       filer_vol        => $filer_vol,
                                       disk_count       => $vol_disk_count,
                                       raid_size        => $raid_size,
                                       mirror           => $mirror,
                                       raid_type        => $raid_type,
                                       hya_storage_type => $hya_storage_type,
                                       ssd_raid_type    => $ssd_raid_type,
                                       timeout          => $timeout,
                                       aggr_type        => $aggr_type)
          ) {
            $Log->comment("$pkg_name: Hybrid Aggr/Volume creation is successful\n");
            return 1;
        } else {
            $Log->comment("$pkg_name: Hybrid Aggr/Volume creation appears to have failed.\n");
            return 0;
        }
    }
    $Log->comment("$pkg_name: printing sys aggr and vol status after aggr creation");
    RaidSqa_print_status(api_set_obj => $api_set_obj, type => $filer_vol);
    return 1;
} ## end sub RaidSqa_aggr_create

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_add

RaidSqa_aggr_add(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, group_name=>$group_name, disk_count=>$disk_count, disk_size=>$disk_size, disk_list=>$disk_list, disk_list1=>$disk_list1, options=>$options, node=>$node, timeout=>$timeout);

This subroutine add disks to an existing aggregate. You can use either disk_count or disk_count and disk_size or disk_list. Also if group_name is provided than it will add disk to that particular raidgroup. options is used to use either "-n" to get a command string instaed of actually adding or "-f" to force add disks. As mirror is not supported on Cmode there is no significance of disk_list1

It returns the command execution status(0/1) and output message.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate  or traditional volume to be created.

=item group_name

(Optional) : Provide this if want to add disk this particular raid group. But disks will be added if raidsize allows to add disk to that particular raid group.

=item disk_count

(Optional) : Number of of disk to be add

=item disk_size

(Optional) : Size of the disks to be used for addition. This can be used only when disk_count is also defined. but can not be used along with disk_list.

=item disk_list

(Optional) : Reference to the list of disks to be add. Need to define at least one parameter from disk_count or disk_list. But can not defined both.

=item disk_list1

(Optional) : This is needed fro mirror aggr, but as it is not supported in cmode, we will ignore this.

=item options

(Optional):  This can have 2 values "-f" to force add disks and "-n" to get a comamnd string instaed of actually adding disks.

=item node

(Optional): Node name on which the operation is to be performed

=item timeout

(Optional): timeout value in secs.

=item ssd_raid_type

(Optional): Possible values: raid4,raid_dp.Default is undef
             This option when set to different value other than RAID_TYPE will create MRT aggr.If undef is set, aggregate created will be of raidtype as RAID_TYPE.
             For eg : if RAID_TYPE=raid_dp and SSD_RAID_TYPE=raid4; aggregate created will be of Mixed Raid Type.


=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_add {

    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR},
                        group_name => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        disk_count => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        disk_size  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        disk_list  => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        disk_list1 => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        ssd_raid_type => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        options    => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        node       => {type => SCALAR, default => "local", optional => 1},
                        timeout     => {type => SCALAR, default => 1200, optional => 1},});
    my $api_set_obj     = $opts{api_set_obj};
    my $conntype        = $opts{conntype};
    my $filer_vol       = $opts{filer_vol};
    my $group_name      = $opts{group_name};
    my $disk_count      = $opts{disk_count};
    my $disk_size       = $opts{disk_size};
    my $disk_list       = $opts{disk_list};
    my $disk_list1      = $opts{disk_list1};
    my $options         = $opts{options};
    my $node            = $opts{node};
    my $timeout         = $opts{timeout};
    my $ssd_raid_type   = $opts{ssd_raid_type};
    my $filer           = $api_set_obj->hostobj->hostname();

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }
    my @disk_list    = @$disk_list if (defined($disk_list));
    my @disk_lists_1 = @$disk_list1 if (defined($disk_list1));
    # check that user did not specified both disk_count and disk_list together
    if (defined($disk_count) && (@disk_list)) {
        $Log->comment(
            "$pkg_name: Need to provide only one parameter disk_count or disk_list to aggr add argument"
        );
        return 0;
    }

    my %aggr_add_disks_hash;
    $aggr_add_disks_hash{'aggregate'} = $filer_vol;
    $aggr_add_disks_hash{'raidgroup'} = $group_name if (defined($group_name));

    if (defined($ssd_raid_type)) {
        # check if aggr is hybrid aggr
        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $aggr_obj = $raid_obj->aggregate("$filer_vol");
        if (defined($aggr_obj)) {
            if ($aggr_obj->parameter("hybrid") == 1) {
                $Log->comment("$pkg_name: Aggr/Volume disk addition is unsuccessful; $filer_vol is hybrid");
                my $error = 'Mixing RAID types is allowed only while adding SSD disks for the first time to a hybrid-enabled aggregate';
                return ($error, 0);
            } else {
                if (RaidSqa_get_aggr_options(api_set_obj=>$api_set_obj, filer_vol => $filer_vol, options => "hybrid_enabled") eq "on") {
                    $aggr_add_disks_hash{'raidtype'} = $ssd_raid_type;
                }
            }
        }
    }

    if (defined($disk_count)) {
        $aggr_add_disks_hash{'diskcount'} = $disk_count;
    } elsif (defined($disk_list) ) {
        my @disklist = ();
        foreach  (@disk_list) {
            push(@disklist, "$filer:".$_);
        }
        @disk_list = ();
        @disk_list = @disklist;
        $Log->comment("$pkg_name: Disks to be added are @disk_list");
        my $disk_list =join(',',@disk_list);
        $aggr_add_disks_hash{'disklist'} = $disk_list;
        if (defined($disk_list1)) {
            my @disklist1 = ();
            foreach  (@disk_lists_1) {
                push(@disklist1, "$filer:".$_);
            }
            @disk_lists_1 = ();
            @disk_lists_1 = @disklist1;
            $Log->comment("$pkg_name: Mirrored disks to be added are @disk_lists_1");
            my $disk_list1 =join(',',@disk_lists_1);
            $aggr_add_disks_hash{'mirror-disklist'} = $disk_list1;
        }
    }

    # Check in case param 'options' have values like checksum; raidtype; group; extract the values and overwrite the hash.
    if (defined($options)) {
        #Check for group name
        if ($options =~ /\-g\s+(\w+)/) {
            $aggr_add_disks_hash{'raidgroup'} = $1;
        }
        #Check for disktype
        if ($options =~ /\-T\s+(\w+)/) {
            $aggr_add_disks_hash{'disktype'} = $1;
        }
        #Check for raidtype
        if ($options =~ /\-t\s+(\w+)/) {
            $aggr_add_disks_hash{'raidtype'} = $1;
        }
        #Check for checksum type
        if ($options =~ /\-c\s+(\w+)/) {
            $aggr_add_disks_hash{'chksumstyle'} = $1;
        }
    }

    my ($output, $error);
    my $result = 1;
    try {
        my $response = $api_set_obj->storage_aggregate_add_disks(%aggr_add_disks_hash);
        $output = $response->get_raw_output();
        $Log->comment("$pkg_name: Aggr/Volume disk addition is successful\n");
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $Log->comment("$pkg_name: Aggr/Volume disk addition is unsuccessful\n");
        $result = 0;
    };

    sleep 20;

    my @output_strings =(
        "needs? to be zeroed before .* can be added",
        "Addition of \\d+ disks? to the aggregate has completed",
        "\\d+ disks? have been added to the aggregate.\\s+ \\d+ disk needs? to be zeroed before"
        #TODO: Do we need to add more here?
    );
    my $regex = join "|", @output_strings;

    if(($output != "") and ($output !~ /$regex/mi)) {
        $Log->comment("$pkg_name: Disk add to aggregate was not successful, output - $output");
        $result = 0;
    }

    # wait for zeroing to finish
    RaidSqa_wait_for_aggr_disks_zeroing_completion(api_set_obj => $api_set_obj,
                                                   conntype    => $conntype,
                                                   filer_vol   => $filer_vol,
                                                   node        => $node);
    return ($output, $result);

} ## end sub RaidSqa_aggr_add

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_rename

RaidSqa_aggr_rename(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, new_vol=>$new_vol, node=>$node);

This subroutine rename the test aggregate.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate to be renamed.

=item new_vol

(Required) : New name of aggregate.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_rename {
    my (%opts) =
      RaidSqa_validate(@_, {
        api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
        filer_vol => {type => SCALAR},
        new_vol   => {type => SCALAR},
        node      => {type => SCALAR, default => "local", optional => 1},
        timeout   => {type => SCALAR, default => 300, optional => 1},
    });

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $timeout     = $opts{timeout};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $new_vol     = $opts{new_vol};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my ($output, $error);
    try {
        my $response =
          $api_set_obj->storage_aggregate_rename(aggregate => $filer_vol, newname => $new_vol, 'connectrec-timeout'=> $timeout);
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
    };

    sleep 5;
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate($new_vol);

    # Verifying if aggregate object exists for the renamed aggregate
    if ($aggr_obj) {
        $Log->comment("$pkg_name: Aggr rename was successful");
        return 1;
    } else {
        $Log->comment("$pkg_name: Aggr rename failed with exception: $error");
        return 0;
    }
} ## end sub RaidSqa_aggr_rename

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_offline

RaidSqa_aggr_offline(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, aggr_type => $aggr_type, node => $node);

This subroutine offline the test aggregate.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate to be offlined.

=item aggr_type

(Required): 1 for aggregate and 0 for traditional. Default it is 1. For cmode traditional volume is not supported, hence we  will set it to 1 if provided as 0.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_offline {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol   => {type => SCALAR},
                        aggr_type   => {type => SCALAR, default => 1, optional => 1},
                        timeout     => {type => SCALAR, default => 600, optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $aggr_type   = $opts{aggr_type};
    my $timeout     = $opts{timeout};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if (!$aggr_type) {
        $Log->comment("$pkg_name: aggr_type as 0 is not supported in Cmode hence chnaging it to 1");
        $aggr_type = 1;
    }
    my $response_obj;
    my @vol    = ();
    my $result = 1;

    try {
        my $response = $api_set_obj->volume_show(aggregate => $filer_vol, 'connectrec-timeout' => $timeout);
        my $ref_vol = $response->get_parsed_output();
        @vol = @$ref_vol;
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
    };

    try {
        $Log->comment("$pkg_name: offlining aggregate $filer_vol");

        # check aggr/volume exists
        if (!RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                      filer_vol   => $filer_vol,
                                      aggr_type   => $aggr_type,
                                      timeout     => $timeout,
                                      node        => $node)
          ) {
            $Log->comment("$pkg_name: aggr $filer_vol does not exists");
            return 0;
        } else {

            ##BURT 475670 : Aggr does not get offlined if set confirmations is set to off due to burt
            # if (@vol > 0) {
                # RaidSqa_hammer_pause(api_set_obj => $api_set_obj);
                # foreach my $vol (@vol) {
                    # $api_set_obj->volume_unmount(volume  => $vol->{volume},
                                                 # vserver => $vol->{vserver});
                    # $Log->comment("$pkg_name: unmounted the volume:$vol->{volume}");
                    # $api_set_obj->volume_offline(volume  => $vol->{volume},
                                                 # vserver => $vol->{vserver});
                    # $Log->comment("$pkg_name: offlined the volume:$vol->{volume}");
                # } ## end foreach my $vol (@vol)
            # } ## end if (@vol > 0)

            RaidSqa_hammer_pause(api_set_obj => $api_set_obj, conntype => $conntype, node => $node, timeout => $timeout);
            $Log->comment("$pkg_name: Offlining aggregate");
            my $response_obj = $api_set_obj->set(confirmations => "on");

            my $aggr_offline_prompt = ['.*unmount all of the volumes hosted by the aggregate' => 'Y'];
            $response_obj =
              $api_set_obj->storage_aggregate_modify(aggregate => $filer_vol,
                                                     state     => "offline",
                                                     'connectrec-match_table' => $aggr_offline_prompt,
                                                     'connectrec-timeout' => $timeout,
                                                     );

            $api_set_obj->set(confirmations => "off");
            sleep 10;

            # check aggr came offline  successfully
            if (RaidSqa_check_aggr_state_name(api_set_obj => $api_set_obj,
                                              filer_vol   => $filer_vol,
                                              state_name  => "offline",
                                              aggr_type   => $aggr_type,
                                              timeout     => $timeout,
                                              node        => $node)
              ) {
                $Log->comment("$pkg_name: aggr $filer_vol successfully offlined");
            } else {
                $Log->comment("$pkg_name: aggr $filer_vol could not be offlined");
                $result = 0;
            }
        } ## end else [ if (!RaidSqa_check_aggr_exist...
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $output           = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    #If its not reset to off, other command executions will fail
    if($result == 0) {
        try {
            $api_set_obj->set(confirmations => "off");
        } catch NACL::APISet::Exceptions::ResponseException with {
        };
    }

    return $result;
} ## end sub RaidSqa_aggr_offline

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_online

RaidSqa_aggr_online(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, aggr_type => $aggr_type, node => $node);

This subroutine online the test aggregate.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate to be onlined.

=item aggr_type

(Required): 1 for aggregate and 0 for traditional. Default it is 1. For cmode traditional is not supported, hence will set to 1 if 0 provided.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_online {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        aggr_type => {type => SCALAR, default => 1, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $aggr_type   = $opts{aggr_type};
    my $node        = $opts{node};
    my $response_obj;

    # create connection
    my ($host, $conn);
    my $result = 1;
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if (!$aggr_type) {
        $Log->comment(
             "$pkg_name: As traditional volume is not supported, hence changing the aggr_type to 1");
        $aggr_type = 1;
    }

    try {
        $Log->comment("$pkg_name: Onlining aggregate $filer_vol");

        # check aggr/volume exists
        if (RaidSqa_check_aggr_state_name(api_set_obj => $api_set_obj,
                                          filer_vol   => $filer_vol,
                                          aggr_type   => $aggr_type,
                                          state_name  => "online",
                                          node        => $node)
          ) {
            $Log->warn("$pkg_name: aggr $filer_vol is already in online state so no need to online once again");
            return 1;
        } else {

            # online aggr/volume
            $response_obj =
              $api_set_obj->storage_aggregate_modify(aggregate => $filer_vol,
                                                     state     => "online");
            $Log->comment(
                 "$pkg_name : Response parsed output:" . Dumper($response_obj->get_parsed_output()));

            # check aggr came online successfully
            if (RaidSqa_check_aggr_state_name(api_set_obj => $api_set_obj,
                                              filer_vol   => $filer_vol,
                                              aggr_type   => $aggr_type,
                                              state_name  => "online",
                                              node        => $node)
              ) {
                $Log->comment("$pkg_name: aggr $filer_vol successfully onlined");
            } else {
                $Log->comment("$pkg_name: aggr $filer_vol could not be onlined");
                return 0;
            }
        }
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $output           = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    return $result if(!$result);

    #Online and mount all volumes in this aggr
    try {
        my $response = $api_set_obj->volume_show(aggregate => $filer_vol);
        my $ref_vol = $response->get_parsed_output();
        my @vol = @$ref_vol;
        if (@vol > 0) {
            foreach my $vol (@vol) {
                $api_set_obj->volume_online(volume  => $vol->{volume},
                                             vserver => $vol->{vserver});
                $Log->comment("$pkg_name: onlined the volume: $vol->{volume}");

                $api_set_obj->volume_mount(volume  => $vol->{volume},
                                           'junction-path' => "/".$vol->{volume},
                                           vserver => $vol->{vserver});
                $Log->comment("$pkg_name: mounted the volume: $vol->{volume}");
            } ## end foreach my $vol (@vol)
        } ## end if (@vol > 0)
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $exception_text = $exception_object->text();
        if($exception_text =~ /There are no entries matching your query/) {
            $Log->comment("$pkg_name: There are no volumes in aggr");
            $result = 1
        } else {
            _handle_nacl_exception($exception_object);
            $result = 0;
        }
    }
    return $result;
} ## end sub RaidSqa_aggr_online

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_restrict

RaidSqa_aggr_restrict(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, aggr_type => $aggr_type, node=>$node);

This subroutine restrict the test aggregate.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate to be restricted.

=item aggr_type

(Required): 1 for aggregate and 0 for traditional. Default it is 1. For cmode traditional is not supported, hence will set to 1 if 0 provided.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_restrict {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        aggr_type => {type => SCALAR, default => 1, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $aggr_type   = $opts{aggr_type};
    my $node        = $opts{node};

    my $response = undef;
    if (!$aggr_type) {
        $Log->comment(
                "$pkg_name: Traditional volume is not supported, hence changing the aggr_type to 1");
        $aggr_type = 1;
    }

    if (!RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  filer_vol   => $filer_vol,
                                  aggr_type   => $aggr_type,
                                  node        => $node)
      ) {
        $Log->comment("$pkg_name: aggr $filer_vol does not exists");
        return 0;
    } else {
        RaidSqa_hammer_pause(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);

        try {
            $Log->comment("$pkg_name: Offlining aggregate");
            $api_set_obj->set(confirmations => "on");
            my $aggr_restrict_prompt = ['.*unmount all of the volumes hosted by the aggregate' => 'Y'];

            $response =
              $api_set_obj->storage_aggregate_modify(aggregate => $filer_vol,
                                                     state     => 'restricted',
                                                     'connectrec-match_table' => $aggr_restrict_prompt);
            $api_set_obj->set(confirmations => "off");
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            my $output           = _handle_nacl_exception($exception_object);
        };

        if (RaidSqa_check_aggr_state_name(api_set_obj => $api_set_obj,
                                          conntype    => $conntype,
                                          filer_vol   => $filer_vol,
                                          state_name  => "restricted",
                                          aggr_type   => $aggr_type,
                                          node        => $node)
          ) {
            $Log->comment("$pkg_name: Aggregate $filer_vol restricted successfully");
            return 1;
        } else {
            $Log->comment("$pkg_name: Aggregate $filer_vol could not be restricted");
            return 0;
        }
    } ## end else [ if (!RaidSqa_check_aggr_exist...
} ## end sub RaidSqa_aggr_restrict

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_destroy

RaidSqa_aggr_destroy(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, aggr_type => $aggr_type, node=>$node);

This subroutine delete an aggregate.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate to be destroyed.

=item aggr_type

(Required): 1 for aggregate and 0 for traditional. Default it is 1. For cmode traditional is not supported, hence will set to 1 if 0 provided.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_destroy {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        aggr_type => {type => SCALAR, default => 1, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1}});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $aggr_type   = $opts{aggr_type};
    my $node        = $opts{node};
    my $result      = 1;

    if (!$aggr_type) {
        $Log->comment(
                    "$pkg_name: Traditional volume is not supported, hence changing aggr_type to 1");
        $aggr_type = 1;
    }

    my (@vol, $vol_error, $response);
    if (RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                 conntype    => $conntype,
                                 filer_vol   => $filer_vol,
                                 aggr_type   => $aggr_type)
      ) {
        $Log->comment("$pkg_name: Printing sys aggr and vol status before destroying aggregate");
        RaidSqa_print_status(api_set_obj => $api_set_obj, type => $filer_vol);
        try {
            @Nodes = NACL::C::Node->find();
            map {$Log->comment($_->node()."\n")} @Nodes;
            $Node = $Nodes[0] ;
        }
        catch NATE::BaseException with {
            my $exception = shift;
            _print_exception($exception);
            $Log->comment("$pkg_name: Filer Connection Failed : ". $exception->text());
            $result = $TCD::CONFIG;
        };
        try {
            NACL::STask::Aggregate->purge(
                command_interface => $Node,
                aggregate => $filer_vol
            );
            $Log->comment("$pkg_name: aggregate destroy succeed $result");
            $result = 1;
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            _handle_nacl_exception($exception_object);
            $Log->comment("$pkg_name: Error occured during aggregate destroy");
            $result = 0;
        };
    } ## end if (RaidSqa_check_aggr_exist...
    return $result;
} ## end sub RaidSqa_aggr_destroy

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_remove

RaidSqa_aggr_remove(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, aggr_type => $aggr_type, node=>$node);

This subroutine delete an aggregate.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate to be destroyed.

=item aggr_type

(Required): 1 for aggregate and 0 for traditional. Default it is 1. For cmode traditional is not supported, hence will set to 1 if 0 provided.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_remove {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        aggr_type   => {type => SCALAR, default => 1, optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},
                        destroy_inconsistent_aggrs=>{type => SCALAR, default => 0, optional => 1}});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $aggr_type   = $opts{aggr_type};
    my $node        = $opts{node};
    ## TODO need to add check for wafl inconsistent/partial volumes and handle extra prompt while deleteing.
    my $destroy_inconsistent_aggrs = $opts{destroy_inconsistent_aggrs};

    my $result      = 1;

    if (!$aggr_type) {
        $Log->comment(
                    "$pkg_name: Traditional volume is not supported, hence changing aggr_type to 1");
        $aggr_type = 1;
    }

    if (RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                 conntype    => $conntype,
                                 filer_vol   => $filer_vol,
                                 aggr_type   => $aggr_type,
                                 node        => $node)
      ) {
        if (!RaidSqa_check_aggr_state_name(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           filer_vol   => $filer_vol,
                                           state_name  => 'online',
                                           aggr_type   => $aggr_type,
                                           node        => $node)
          ) {
            if (!RaidSqa_aggr_online(api_set_obj => $api_set_obj,
                                     conntype    => $conntype,
                                     filer_vol   => $filer_vol,
                                     aggr_type   => $aggr_type,
                                     node        => $node)
              ) {
                $Log->comment(
                    "$pkg_name: aggr $filer_vol is not online, hence can not delete it as it can have flex vols in it"
                );
                return 0;
            } ## end if (!RaidSqa_aggr_online...
        } ## end if (!RaidSqa_check_aggr_state_name...

        $Log->comment("$pkg_name: Printing sys aggr and vol status before removing aggregate");
        RaidSqa_print_status(api_set_obj => $api_set_obj, type => "all");

        # pause hammer before starting aggr/volume offline
        RaidSqa_hammer_pause(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);
        sleep 10;

        my @vol;
        try {
            my $response = $api_set_obj->volume_show(aggregate => $filer_vol);
            my $ref_vol = $response->get_parsed_output();
            @vol = @$ref_vol;
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            my $exception        = _handle_nacl_exception($exception_object);
            if ($exception =~ /There are no entries matching your query/) {
                try {
                    my $response = $api_set_obj->storage_aggregate_delete(aggregate => $filer_vol);
                }
                catch NACL::APISet::Exceptions::ResponseException with {
                    my $exception_object = shift;
                    _handle_nacl_exception($exception_object);
                    $result = 0;
                };
            } ## end if ($exception =~ /There are no entries matching your query/)
        };

        if (!$result) {

            # Restart hammer after aggr/volume offline
            RaidSqa_hammer_restart(api_set_obj => $api_set_obj,
                                   conntype    => $conntype,
                                   node        => $node);
            return $result;
        }

        try {
            for my $vol (@vol) {
                $api_set_obj->volume_unmount(volume => $vol->{volume}, vserver => $vol->{vserver});
                $Log->comment("$pkg_name: unmounted the volume:$vol->{volume}");
                $api_set_obj->volume_offline(volume => $vol->{volume}, vserver => $vol->{vserver});
                $Log->comment("$pkg_name: offlined the volume:$vol->{volume}");
                $api_set_obj->volume_delete(volume => $vol->{volume}, vserver => $vol->{vserver});
                $Log->comment("$pkg_name: deleted the volume:$vol->{volume}");
            }
            $api_set_obj->storage_aggregate_delete(aggregate => $filer_vol);
        } ## end try
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            _handle_nacl_exception($exception_object);
            $result = 0;
        };

        if (!$result) {

            # Restart hammer after aggr/volume offline
            RaidSqa_hammer_restart(api_set_obj => $api_set_obj,
                                   conntype    => $conntype,
                                   node        => $node);
            sleep(5);
            return $result;
        } else {
            if (!RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                          conntype    => $conntype,
                                          filer_vol   => $filer_vol,
                                          aggr_type   => $aggr_type,
                                          node        => $node)
              ) {
                $Log->comment("$pkg_name: Aggr $filer_vol removed successfully");
            } else {
                $Log->comment("$pkg_name: Aggr $filer_vol could not be removed");
                $result = 0;
            }
        } ## end else [ if (!$result)

        # Restart hammer after aggr/volume offline
        RaidSqa_hammer_restart(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);
        sleep(5);
        return $result;
    } else {
        $Log->comment("$pkg_name: Aggr $filer_vol does not exist, no need to remove");
        return 1;
    }
} ## end sub RaidSqa_aggr_remove

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_aggr_mount

RaidSqa_aggr_mount(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node, filer_vol=>$filer_vol, client=>$client);

This subroutines mounts the given volume

It returns mountpoint on which volume is mounted

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item node

(Optional) : node name on which volume filer_vol is created, default local

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional): Volume to be mounted

=item client

(Optional): Client on which volume is to be mounted

=item check_license

(Optional): If set to 1, then checks for NFS license. Default: 0.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_mount {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR,},
                        check_license => {type => SCALAR, default=>1, optional=>1},
                        vserver   => {type => SCALAR, default => "vs0", optional => 1},
                        client    => {type => SCALAR, default => $RaidSqa_CLIENT, optional => 1}});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $client      = $opts{client};
    my $filer_vol   = $opts{filer_vol};
    my $check_license = $opts{check_license};
    my $vserver     = $opts{vserver};

    # create connection
    my ($host, $conn, $cmd);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
        $host = $conn->{hostp};
    }

    if($check_license == 1){
        my $result = RaidSqa_check_license(api_set_obj => $api_set_obj,
                                       node        => $node,
                                       conntype    => $conntype,
                                       service     => "nfs");

    if(!$result) {
            $Log->comment("$pkg_name: NFS license is not installed, installing nfs license");
            if (!RaidSqa_license_add_by_service(api_set_obj => $api_set_obj,
                                        node        => $node,
                                        conntype    => $conntype,
                                        service     => "nfs")
               ) {
                $Log->comment("$pkg_name: NFS license could not be added");
                return 0;
            }
        }
    }

    try {
        $api_set_obj->vserver_nfs_create(vserver => $vserver);
    }
    catch NATE::BaseException with {};
    try {
        $api_set_obj->vserver_export_policy_rule_create(vserver      => $vserver,
                                                        policyname   => "default",
                                                        clientmatch  => "0.0.0.0/0",
                                                        rorule       => "any",
                                                        rwrule       => "any",
                                                        superuser    => "any",
                                                        anon         => 0,
                                                        "allow-suid" => "false");
    } ## end try
    catch NATE::BaseException with {};

    try {
        $api_set_obj->vserver_services_unix_user_create(vserver       => $vserver,
                                                        user          => "root",
                                                        id            => 0,
                                                        "primary-gid" => 0);
    }
    catch NATE::BaseException with {};

    try {
        $api_set_obj->vserver_services_unix_group_create(vserver => $vserver,
                                                         name    => "root",
                                                         id      => 0);
    }
    catch NATE::BaseException with {};

    try {
        $api_set_obj->volume_modify(vserver => $vserver, volume => $filer_vol, 'unix-permissions' => "---rwxrwxrwx");
    }
    catch NATE::BaseException with {};

    my $junction_path = RaidSqa_get_volume_junc_path(api_set_obj => $api_set_obj, filer_vol => $filer_vol);

    if (!$junction_path) {
        $Log->comment("$pkg_name: Junction path not found for " . $filer_vol);
        return 0;
    } else {
        $Log->comment("$pkg_name: Junction path for $filer_vol is : $junction_path");
    }

    my $vserver_ip = RaidSqa_get_vserver_ip_addr(api_set_obj => $api_set_obj, vserver => $vserver);
    if (!$vserver_ip) {
        $Log->comment("$pkg_name: Could not find vserver IP for vserver $vserver");
        return 0;
    } else {
        $Log->comment("$pkg_name: Vserver IP for $vserver is : $vserver_ip");
    }

    my $mountpoint = mount($vserver_ip, $junction_path, retries => 5, client => $client);
    if (defined($mountpoint)) {
        $Log->comment("$pkg_name: Mounted aggr $filer_vol on path $mountpoint");
        return $mountpoint;
    } else {
        $Log->comment("$pkg_name: aggr $filer_vol could not be mounted");
        return 0;
    }
} ## end sub RaidSqa_aggr_mount

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_aggr_scrub_start

RaidSqa_aggr_scrub_start(api_set_obj=>$api_set_obj,  conntype=>$conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, node=>$node);

This subroutine by default start scrub on a given filer. But if any of the options from filer_vol, or group_name is specified than it start scrub on that particular raid object.
plex_name is not supported in Cmode

It returns the command execution status(0/1) and output message.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggr.

=item plex_name

(Optional) : Name of plex. plex_name is not supported in Cmode, we just ignore it.

=item group_name

(Optional) : Name of group.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_scrub_start {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        node       => {type => SCALAR,         default => "local", optional => 1},}
                      );

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my @group_list;
    my $result = 1;
    my ($output);
    my %args = (action => 'start');

    if ($filer_vol) {
        @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             filer_vol   => $filer_vol);
        $args{aggregate} = $filer_vol;
    } elsif ($plex_name) {
        $Log->comment(
                "$pkg_name: In CMode we can not provide plex_name to scrub command, hence ignoring");
    } elsif ($group_name) {
        (undef, $filer_vol, $plex_name, my $group) = split("/", $group_name);
        $args{aggregate} = $filer_vol;
        push (@group_list, $group_name);
        $args{raidgroup} = "$plex_name/$group";
    } else {
        @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj, conntype => $conntype);
        $args{aggregate} = '*';
    }

    try {
        my $response = $api_set_obj->storage_aggregate_scrub(%args);
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    if (!(@group_list)) {
        return ($output, 1);
    }

    foreach my $group (@group_list) {
        my $group_obj = $raid_obj->raid_group("$group");
        if (!$group_obj) {
            $Log->comment("$pkg_name: Raid group $group does not exist");
            return ($output, 0);
        }
        if ($group_obj->parameter("scrub_progress") eq "-1") {
            $Log->comment("$pkg_name: scrub not started on raid_group $group");
            return ($output, 0);
        } else {
            $Log->comment("$pkg_name: scrub started on raid_group $group");
        }
    } ## end foreach my $group (@group_list)
    return ($output, 1);
} ## end sub RaidSqa_aggr_scrub_start

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_scrub_suspend

RaidSqa_aggr_scrub_suspend(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, node => $node);

This subroutine by default suspend all scrub on a given filer. But if any of the options from filer_vol, or group_name is specified than it suspend scrub on that particular raid object.
plex_name is not supported in Cmode

It returns the command execution status(0/1) and output message.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggr or trad vol.

=item plex_name

(Optional) : Name of plex. plex_name is not supported in Cmode, we just ignore it.

=item group_name

(Optional) : Name of group.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_scrub_suspend {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        node       => {type => SCALAR,         default => "local", optional => 1},}
                      );
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};

    #my $o;

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my @group_list = ();
    my ($response, $output, $result);
    my %args = (action => 'suspend');

    if ($filer_vol) {
        @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             filer_vol   => $filer_vol,
                                             node        => $node);
        $args{aggregate} = "$filer_vol";
    } elsif ($plex_name) {
        $Log->comment(
                "$pkg_name: In cmode we can not provide plex_name to scrub command, hence ignoring");
    } elsif ($group_name) {
        (undef, $filer_vol, $plex_name, my $group) = split("/", $group_name);
        $args{aggregate} = $filer_vol;
        push (@group_list, $group_name);
        $args{raidgroup} = "$plex_name/$group";
    } else {
        @group_list =
          RaidSqa_get_group_list(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);
        $args{aggregate} = '*';
    }

    $result = 1;
    try {
        $response = $api_set_obj->storage_aggregate_scrub(%args);
        $output   = Dumper($response->get_raw_output());
        sleep 10;
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    # Check if the group list is valid
    if (!(@group_list)) {
        return ($output, 1);
    }

    foreach my $group (@group_list) {
        my $group_obj = $raid_obj->raid_group("$group");
        if (!$group_obj) {
            $Log->comment("$pkg_name: Raid group $group does not exist");
            return ($output, 0);
        }
        if ($group_obj->parameter("scrub_progress") ne "-1") {
            $Log->comment("$pkg_name: scrub could not suspend on raid_group $group");
            return ($output, 0);
        } else {
            $Log->comment("$pkg_name: scrub suspended on raid_group $group");
        }
    } ## end foreach my $group (@group_list)
    return ($output, 1);

} ## end sub RaidSqa_aggr_scrub_suspend

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_scrub_resume

RaidSqa_aggr_scrub_resume(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, node => $node);

This subroutine by default resume scrub on a given filer. But if any of the options from filer_vol, or group_name is specified than it resume scrub on that particular raid object.
plex_name is not supported in Cmode

It returns the command execution status(0/1) and output message.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggr or trad vol.

=item plex_name

(Optional) : Name of plex. plex_name is not supported in Cmode, we just ignore it.

=item group_name

(Optional) : Name of group.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_scrub_resume {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        node       => {type => SCALAR,         default => "local", optional => 1},}
                      );
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my %args = (action => 'resume');
    my @group_list;
    my ($response, $output);
    if ($filer_vol) {
        @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             filer_vol   => $filer_vol,
                                             node        => $node);
        $args{aggregate} = "$filer_vol";
    } elsif ($plex_name) {
        $Log->comment(
                "$pkg_name: In cmode we can not provide plex_name to scrub command, hence ignoring");
    } elsif ($group_name) {

        (undef, $filer_vol, $plex_name, my $group) = split("/", $group_name);
        $args{aggregate} = $filer_vol;
        push (@group_list, $group_name);
        $args{raidgroup} = "$plex_name/$group";
    } else {
        @group_list =
          RaidSqa_get_group_list(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);
        $args{aggregate} = '*';
    }

    my $result = 1;
    try {
        $response = $api_set_obj->storage_aggregate_scrub(%args);
        $output   = Dumper($response->get_raw_output());

        #TODO
        my $parsed_output = Dumper($response->get_parsed_output());
        $Log->comment("$pkg_name: Parsed Output is $parsed_output");
        $Log->comment("$pkg_name: Output is $output");
        sleep 20;

        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

        if (!(@group_list)) {
            return ($output, 1);
        }

        foreach my $group (@group_list) {
            my $group_obj = $raid_obj->raid_group("$group");
            if (!$group_obj) {
                $Log->comment("$pkg_name: Raid group $group does not exist");
                return ($output, 0);
            }
            if ($group_obj->parameter("scrub_progress") eq "-1") {
                $Log->comment("$pkg_name: scrub could not resume on raid_group $group");
                return ($parsed_output, 0);
            } else {
                $Log->comment("$pkg_name: scrub resumed on raid_group $group");
            }
        } ## end foreach my $group (@group_list)
        return ($parsed_output, 1);
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };
    return ($output, $result);
} ## end sub RaidSqa_aggr_scrub_resume

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_scrub_stop

RaidSqa_aggr_scrub_stop(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, node=>$node);

This subroutine by default stop all scrub on a given filer. But if any of the options from filer_vol, or group_name is specified than it stop scrub on that particular raid object.
plex_name is not supported in Cmode

It returns the command execution status(0/1) and output message.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggr .

=item plex_name

(Optional) : Name of plex. plex_name is not supported in Cmode, we just ignore it.

=item group_name

(Optional) : Name of group.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_scrub_stop {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        node       => {type => SCALAR,         default => "local", optional => 1},}
                      );

    my $o;
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $result      = 1;

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my (@group_list, $response, $output);

    my %args = (action => 'stop');
    my $error = undef;

    try {
        if (defined($filer_vol)) {
            @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                                 conntype    => $conntype,
                                                 filer_vol   => $filer_vol,
                                                 node        => $node);
            $args{aggregate} = "$filer_vol";
        } elsif (defined($plex_name)) {
            $Log->comment(
                "$pkg_name: In cmode we can not provide plex_name to scrub command, hence ignoring");
        } elsif (defined($group_name)) {
            (undef, $filer_vol, $plex_name, my $group) = split("/", $group_name);
            $args{aggregate} = $filer_vol;
        push (@group_list, $group_name);
        $args{raidgroup} ="$plex_name/$group";
        } else {
            @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                                 conntype    => $conntype,
                                                 node        => $node);
            $args{aggregate} = '*';
        }
        $response = $api_set_obj->storage_aggregate_scrub(%args);
        $output   = Dumper($response->get_raw_output());
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    sleep 20;

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    if (!(@group_list)) {
        return ($output, 1);
    }

    foreach my $group (@group_list) {
        my $group_obj = $raid_obj->raid_group("$group");
        if (!$group_obj) {
            $Log->comment("$pkg_name: Raid group $group does not exist");
            return ($output, 0);
        }
        if ($group_obj->parameter("scrub_progress") ne "-1") {
            $Log->comment("$pkg_name: scrub could not stop on raid_group $group");
            return ($output, 0);
        } else {
            $Log->comment("$pkg_name: scrub stopped on raid_group $group");
        }
    } ## end foreach my $group (@group_list)
    return ($output, $result);
} ## end sub RaidSqa_aggr_scrub_stop

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_scrub_status

RaidSqa_aggr_scrub_status(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, node => $node);

This subroutine by default provide scrub status on a given filer. But if any of the options from filer_vol, or group_name is specified than it start scrub on that particular raid object.
plex_name is not supported in Cmode

It returns scrub status o/p

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggr .

=item plex_name

(Optional) : Name of plex. plex_name is not supported in Cmode, we just ignore it.

=item group_name

(Optional) : Name of group.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_scrub_status {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        node       => {type => SCALAR,         default => "local", optional => 1},}
                      );
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $status;

    my %scrub_status_hash = (action => 'status');
    if ($filer_vol) {
        $scrub_status_hash{'aggregate'} = $filer_vol;
    } elsif ($plex_name) {
        $Log->comment(
                "$pkg_name: In cmode we can not provide plex_name to scrub command, hence ignoring");
    } elsif ($group_name) {
        (undef, $filer_vol, $plex_name, my $group) = split("/", $group_name);
        $scrub_status_hash{'aggregate'} = $filer_vol;
        $scrub_status_hash{'raidgroup'} = "$plex_name/$group";
    } else {
        $scrub_status_hash{'aggregate'} = '*';
    }
    try {
        my $response_obj = $api_set_obj->storage_aggregate_scrub(%scrub_status_hash);
        $status = $response_obj->get_raw_output();
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $status = _handle_nacl_exception($exception_object);
    };
    return $status;
} ## end sub RaidSqa_aggr_scrub_status

#------------------------------------------------------------------------------
=head2 RaidSqa_aggr_verify_status

RaidSqa_aggr_verify_status(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, node=>$node);

This subroutines checks the verify status of mirrored aggregate.

It returns command output (Output is reformatted to match that of 7mode)

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggr or trad vol.

=item node

(Optional) : Name of the node on which filer_vol resides

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_verify_status {
    my (%opts) =
      RaidSqa_validate(@_,
            {
                api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                filer_vol   => {type => SCALAR | UNDEF, default => undef, optional => 1},

            }
        );
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my ($response, $output, $parsed_output);
    if(!defined($filer_vol)) {
        $filer_vol = "*";
    }

    #Note: To make this subroutine compatible with 7mode, we need to modify return value according to 7mode
    #       - If filer_vol is mentioned and there is no verify in progress - return undef
    #       - If filer_vol is not mentioned and there are no verify in progress - return "aggr verify status: no verifies in progress"
    #       - If filer_vol is mentioned and aggr verify is running - return raw_output
    #       - If filer_vol is not mentioned and aggr verify is running for some of aggregates- return raw_output

    try {
        $response = $api_set_obj->storage_aggregate_verify(aggregate => $filer_vol, action=>"status", );
        $output = $response->get_raw_output();
        $parsed_output = $response->get_parsed_output();
        $Log->comment("DEBUG:: Remove: parsed output: aggr verify: ".Dumper($parsed_output));
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $output = _handle_nacl_exception($exception_object);
        return "";
    };

    #TODO
    if($output =~ /Percentage Completed:\d+%/ or $filer_vol eq "*") {
        return $output;
    } else {
        return "";
    }

} ## end sub RaidSqa_aggr_verify_status

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_verify_start

RaidSqa_aggr_verify_start(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, plex_number=>$plex_number, node=>$node);

This subroutine start aggregate verify on given filer_vol with target plex if defined.
If filer_vol is not defined, it will start aggregate verify on all aggregates in filer

It returns 1 on pass and 0 on fail

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggr or trad vol.

=item plex_number

(Optional) : Number of plex.

=item node

(Optional) : Name of the node on which aggregate verify is to be started

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_verify_start {
    my (%opts) =
    RaidSqa_validate(@_,
        {
            api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
            conntype    => {type => SCALAR, default => "#ssh", optional => 1},
            node        => {type => SCALAR, default => "local", optional => 1},
            filer_vol   => {type => SCALAR | UNDEF, default => undef, optional => 1},
            plex_number => {type => SCALAR | UNDEF, default => undef, optional => 1},

        }
    );

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_number = $opts{plex_number};
    my $node         = $opts{node};
    my ($response, $output, $parsed_output);

    $filer_vol = "*" if(!defined($filer_vol));

    my %cmd_args = (aggregate => $filer_vol, action=>"start");
    $plex_number = ($plex_number =~ /^\d$/) ? "plex".$plex_number : $plex_number;
    if(defined($plex_number)) {
        $cmd_args{plex_to_fix} = $plex_number;
    }

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    try {
        $response = $api_set_obj->storage_aggregate_verify(%cmd_args);
        $parsed_output = $response->get_parsed_output();
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
    };

    # if($output =~ /not/ || $output =~ /already running/ ) {
    #      return 0;
    # } elsif ($output =~ /start mirror verification/) {
    #      return 1;
    # }

    # need to enable following code once burt 249002 got fixed
    #sleep 10;
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");

    if ($aggr_obj->parameter("verifypct") eq "-1") {
        sleep 5;
        $raid_obj->refresh();
        if ($aggr_obj->parameter("verifypct") eq "-1") {
            $Log->comment("$pkg_name: verify could not start on aggr $filer_vol");
            return 0;
        } else {
            $Log->comment("$pkg_name: verify started on aggr $filer_vol");
            return 1;
        }

    } else {
        $Log->comment("$pkg_name: verify started on aggr $filer_vol");
        return 1;
    }
}

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_mirror

RaidSqa_aggr_mirror(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, options=>$options);

-or-

RaidSqa_aggr_mirror(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, disk_list=>\@disk_list);

This subroutine mirror the test aggregate using either using a list of disks or just use option "-n" to get the command line needed to create a mirror volume.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate  or traditional volume to be created.

=item options

(Optional) : There is only one options "-n" availabel which givesthe complete comamnd line needed to create a covert a unmirror volume to mirror

=item disk_list

(Optional) : Reference to the list of disks to be used for mirror volume creation from unmirror.

=item splitted_vol

(Optional) : Name of splitted_vol volume which was created by vol split command from the test volume when it was mirrored.

=item mirror_skip_config_checks

(Optional): Possible values 0,1. Default is 1.
            If 1, options raid.mirror_skip_config_checks will be set to ON before mirrored aggregate is created
            If 0 then no action is taken

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_mirror {

    my (%opts) =
      RaidSqa_validate(@_,
       {
            api_set_obj      => {type => OBJECT, isa     => 'NACL::APISet'},
            conntype         => {type => SCALAR, default => "#ssh", optional => 1},
            filer_vol        => {type => SCALAR},
            disk_list        => {type => ARRAYREF, optional => 1},
            allow_mixed_rpm  => {type => SCALAR|UNDEF, default=> undef, optional => 1},
            node             => {type => SCALAR, default => "local", optional => 1},
            mirror_skip_config_checks  => {type => SCALAR, default=> 1, optional => 1},
        }
    );
    my $api_set_obj = $opts{api_set_obj};
    my $conntype  = $opts{conntype};
    my $filer_vol = $opts{filer_vol};
    my $disk_list = $opts{disk_list};
    my $allow_mixed_rpm = $opts{allow_mixed_rpm};
    my $mirror_skip_config_checks = $opts{mirror_skip_config_checks};
    my $node      = $opts{node};
    my @disk_list;
    my ($response, $output, $parsed_output);
    my $filer       = $api_set_obj->hostobj->hostname();

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my %cmd_args = (aggregate => $filer_vol, 'ignore-pool-checks' => "true");
    if(defined($disk_list)) {
        $disk_list[0] = "$filer:".$disk_list[0];
        $cmd_args{'mirror-disklist'} = join(",$filer:", @$disk_list);
    }

    if(defined($allow_mixed_rpm)){
        if($allow_mixed_rpm !~ /(true|false)/i) {
            $Log->comment("$pkg_name: allow mixed rpm should be true or false: $allow_mixed_rpm");
            return 0;
        }
        $cmd_args{'allow-mixed-rpm'}=$allow_mixed_rpm;
    }

    #Set 'raid.mirror_skip_config_checks' to ON to allow aggr mirror [BURT #525715]
    #Info: SN onwards, first mirror aggr creation on node fails if there are disks from same adapter in each pool
    #      However for testing purpose, we can skip this check by setting option  'raid.mirror_skip_config_checks' to on.

    if($mirror_skip_config_checks == 1) {
        $Log->comment("$pkg_name: Setting value of an option to 'ON' - 'raid.mirror_skip_config_checks'");
        if(!RaidSqa_set_options(
            api_set_obj  => $api_set_obj,
            option_name  => 'raid.mirror_skip_config_checks',
            option_value => 'on',
            node         => $node
        )) {
            $Log->warn("$pkg_name: Setting value of an option to 'ON' failed - 'raid.mirror_skip_config_checks'");
            #return 0;
        }
    }


    #Burt: 539887: Aggr mirror fails if free_space_realloc option is set to on.
    #This code needs to be removed once BURT #540046 is fixed
    if('on' eq RaidSqa_get_aggr_options(
                    api_set_obj => $api_set_obj,
                    filer_vol   => $filer_vol,
                    options     => 'free_space_realloc',
                    node        => $node)){

        if(!RaidSqa_aggr_options(
            api_set_obj  => $api_set_obj,
        filer_vol   => $filer_vol,
            options  => 'free_space_realloc',
            value => 'off',
            node         => $node)) {
            $Log->comment("$pkg_name: Error: Disable free_space_realloc failed - $filer_vol");
            return 0;
        }
        $Log->comment("$pkg_name: Info: free_space_realloc options 'free_space_realloc is set to off");
    }

    try {
        $response = $api_set_obj->storage_aggregate_mirror(%cmd_args);
        $parsed_output = $response->get_parsed_output();
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
    };

    sleep 20;
    # Check test aggr/volume mirror got created
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");

    if ($aggr_obj->parameter("plex_count") ne "2") {
        $Log->comment("$pkg_name: Aggr/Volume mirror creation has failed.\n");
        return 0;
    }

    # wait for zeroing to finish
    RaidSqa_wait_for_aggr_disks_zeroing_completion(api_set_obj => $api_set_obj,
                                                   conntype  => $conntype,
                                                   node      => $node,
                                                   filer_vol => $filer_vol);
    $Log->comment("$pkg_name: Aggr/Volume got mirrored but may be still doing resync\n");
    return 1;
} ## end sub RaidSqa_aggr_mirror

#------------------------------------------------------------------------------

=head2 RaidSqa_plex_offline

RaidSqa_plex_offline(api_set_obj=>$api_set_obj, conntype=>$conntype, plex_name => $plex_name);

This subroutine offline the plex of a mirror volume/aggr.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item plex_name

(Required) : Name of plex to offline.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_plex_offline {
    my (%opts) =
      RaidSqa_validate(@_,
       {
            api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
            conntype    => {type => SCALAR, default => "#ssh", optional => 1},
            plex_name   => {type => SCALAR},
            node        => {type => SCALAR, default => "local", optional => 1},
        }
    );

    my $api_set_obj      = $opts{api_set_obj};
    my $node = $opts{node};
    my $conntype  = $opts{conntype};
    my $plex_name = $opts{plex_name};
    $plex_name =~ /\/(.*)\/(plex\d+)/i;
    my $aggregate = $1;
    my $plex = $2;
    my ($response, $output, $parsed_output);

    $Log->comment("$pkg_name: Offlining plex $plex_name");
    try {
        $response = $api_set_obj->storage_aggregate_plex_offline(aggregate=>$aggregate, plex=>$plex);
        my $parsed_output = $response->get_parsed_output();
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
    };

    # Check that plex got offlined successfully
    if (RaidSqa_check_plex_state_name(api_set_obj => $api_set_obj,
                                      conntype   => $conntype,
                                      plex_name  => $plex_name,
                                      state_name => "offline")
      ) {
        $Log->comment("$pkg_name: plex $plex_name successfully offlined");
        return 1;
    } else {
        $Log->comment("$pkg_name: plex $plex_name could not be offlined");
        return 0;
    }
} ## end sub RaidSqa_plex_offline

#------------------------------------------------------------------------------

=head2 RaidSqa_plex_online

RaidSqa_plex_online(api_set_obj=>$api_set_obj, conntype=>$conntype, plex_name => $plex_name);

This subroutine online the plex of a mirror volume/aggr.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item plex_name

(Required) : Name of plex to online.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_plex_online {
    my (%opts) =
      RaidSqa_validate(@_,
       {
            api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
            conntype    => {type => SCALAR, default => "#ssh", optional => 1},
            plex_name   => {type => SCALAR},
            node        => {type => SCALAR, default => "local", optional => 1},
        }
    );

    my $api_set_obj      = $opts{api_set_obj};
    my $node = $opts{node};
    my $conntype  = $opts{conntype};
    my $plex_name = $opts{plex_name};
    $plex_name =~ /\/(.*)\/(plex\d+)/i;
    my $aggregate = $1;
    my $plex = $2;
    my ($response, $output, $parsed_output);

    $Log->comment("$pkg_name: Onlining plex $plex_name");

    try {
        $response = $api_set_obj->storage_aggregate_plex_online(aggregate=>$aggregate, plex=>$plex);
        my $parsed_output = $response->get_parsed_output();
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
    };

    # check plex got online successfully
    if (RaidSqa_check_plex_state_name(api_set_obj => $api_set_obj,
                                      conntype   => $conntype,
                                      plex_name  => $plex_name,
                                      state_name => "online")
      ) {
        $Log->comment("$pkg_name: plex $plex_name successfully onlined");
        return 1;
    } else {
        $Log->comment("$pkg_name: plex $plex_name could not be onlined");
        return 0;
    }
} ## end sub RaidSqa_plex_online

#------------------------------------------------------------------------------

=head2 RaidSqa_get_plex_resync_level

RaidSqa_get_plex_resync_level(api_set_obj=>$api_set_obj, conntype=>$conntype, plex_name => $plex_name, node=>$node);

This subroutine gets resync level of a given plex if it is 0 or 1.

It returns resync level of given plex

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item plex_name

(Required) : Name of plex.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_plex_resync_level {
    my (%opts) =
      RaidSqa_validate(@_,
       {
            api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
            conntype    => {type => SCALAR, default => "#ssh", optional => 1},
            plex_name   => {type => SCALAR},
            node        => {type => SCALAR, default => "local", optional => 1},
        }
    );

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $plex_name   = $opts{plex_name};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # get list of groups in a specified plex
    my $raid_obj          = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $plex_obj          = $raid_obj->plex("$plex_name");
    my $plex_resync_level = $plex_obj->parameter("resync_level");
    return $plex_resync_level;
} ## end sub RaidSqa_get_plex_resync_level

#------------------------------------------------------------------------------

=head2 RaidSqa_wait_for_plex_resync_completion

RaidSqa_wait_for_plex_resync_completion(api_set_obj=>$api_set_obj, conntype=>$conntype, plex_name => $plex_name, resync_level=>$resync_level, timeout=>$timeout);

This subroutine wait till the resync (level 0 or level1) in a particular plex gets completed.

It returns nothing

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item plex_name

(Required) : Name of plex to be checked .

=item resync_level

(Required) : 1 for level1 and 0 for level0.

=item timeout

(Optional): Value of timeout in secs to wait while checking the status again.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_wait_for_plex_resync_completion {
    my (%opts) =
      RaidSqa_validate(@_,
       {
            api_set_obj  => {type => OBJECT, isa     => 'NACL::APISet'},
            conntype     => {type => SCALAR, default => "#ssh", optional => 1},
            filer_vol    => {type => SCALAR | UNDEF, default => undef, optional => 1},
            plex_name    => {type => SCALAR | UNDEF, default => undef, optional => 1},
            resync_level => {type => SCALAR | UNDEF, default => undef, optional => 1},
            timeout      => {type => SCALAR, default => "10", optional => 1},
            node         => {type => SCALAR, default => "local", optional => 1},
        }
    );
    my $api_set_obj  = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $filer_vol     = $opts{filer_vol};
    my $node         = $opts{node};
    my $plex_name    = $opts{plex_name};
    my $resync_level = $opts{resync_level};
    my $timeout      = $opts{timeout};

    my $plex_obj     = undef;

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }


    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    #my $plex_obj = $raid_obj->plex("$plex_name");

    my @plex_list = ($plex_name) if(defined($plex_name));
    if (defined($filer_vol) && !scalar(@plex_list)) {
        my $aggr_obj = $raid_obj->aggregate("$filer_vol");
        my %plex_list  = $aggr_obj->all_plexes();
        @plex_list = keys(%plex_list);
    }

    foreach my $plex (@plex_list) {
        $plex_obj = $raid_obj->plex("$plex");
        $Log->comment("$pkg_name: resyncing is going on plex $plex");
        $plex_name = $plex;
        last if($plex_obj->parameter('resyncing') eq '1');
    }

    if (!defined $plex_obj) {
        $Log->comment("no plex is in resyncing state");
        return 0;
    }
    $resync_level = $plex_obj->parameter("resync_level") if (!defined $resync_level);

    $Log->comment("$pkg_name: resyncing is going on plex $plex_name");
    if ($plex_obj->parameter("resync_level") eq $resync_level) {
        $Log->comment("$pkg_name: level $resync_level resyncing is going on plex $plex_name");
    } else {
        $Log->comment("$pkg_name: resyncing is not level $resync_level as expected on plex $plex_name");
        return 0;
    }

    # need to fix following after fix of burt 244854
    sleep 30;
    my $plex_resync_progress = $plex_obj->parameter("resync_progress");
    while ($plex_obj->parameter("resyncing") eq "1") {
        $Log->comment(
            "$pkg_name: resync is still $plex_resync_progress % on plex $plex_name, waiting for $timeout seconds"
        );
        sleep $timeout;
        $raid_obj->refresh();
        $plex_obj             = $raid_obj->plex("$plex_name");
        $plex_resync_progress = $plex_obj->parameter("resync_progress");
    }
    return 1;

} ## end sub RaidSqa_wait_for_plex_resync_completion

#------------------------------------------------------------------------------

=head2 RaidSqa_wait_for_verify_completion

RaidSqa_wait_for_verify_completion(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, timeout=>$timeout);

This subroutine by default wait till the verify get completed on on all raid groups in filer. But if any of the options from filer_vol is specified than it check verify completion on that particular raid object.

It returns nothing

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Use filer_vol if we want to check scrub progress on that particular aggr/trad volume.

=item timeout

(Optional): Value of timeout in secs to wait while checking the status again.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_wait_for_verify_completion {
    my (%opts) =
      RaidSqa_validate(@_,
       {
            api_set_obj => {type => OBJECT,       isa     => 'NACL::APISet'},
            conntype    => {type => SCALAR,       default => "#ssh", optional => 1},
            filer_vol   => {type => SCALAR|UNDEF, default => undef, optional => 1},
            timeout     => {type => SCALAR,       default => "10",  optional => 1},
            node        => {type => SCALAR,       default => "local", optional => 1},
        }
    );
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $timeout     = $opts{timeout};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    my @aggr_list;
    if (defined($filer_vol)) {
        @aggr_list = "$filer_vol";
    } else {
        @aggr_list =
          RaidSqa_get_aggr_list(api_set_obj => $api_set_obj, conntype => $conntype);
    }

    while (1) {
        my $verify_in_progress = 0;
        foreach my $aggr (@aggr_list) {
            my $aggr_obj             = $raid_obj->aggregate("$aggr");
            my $aggr_verify_progress = $aggr_obj->parameter("verifypct");
            if ($aggr_verify_progress != -1) {
                $Log->comment("$pkg_name: aggr verify still $aggr_verify_progress % on aggr $aggr");
                $verify_in_progress = 1;
            }
        }
        last if($verify_in_progress == 0);
        sleep $timeout;
        $raid_obj->refresh();
    } ## end while ($verify_in_progress...
} ## end sub RaidSqa_wait_for_verify_completion

#--------------------------------------------------------------------------------------------------

=head2 RaidSqa_aggr_wafliron_start
RaidSqa_aggr_wafliron_start(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine starts wafliron on the test aggregate .

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object


=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_wafliron_start {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        oc        => {type => SCALAR, default => 0, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $oc          = $opts{oc};
    my $node        = $opts{node};
    my ($output, $cmd, $response);

    $Log->comment("$pkg_name: Starting wafliron on aggr $filer_vol");

    #Starting wafliron on test aggr/volume
    $cmd = "aggr wafliron start $filer_vol";
    if ($oc) {
        $cmd = $cmd . " -o";
    }

    try {
        $response =
          $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; $cmd \"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
    };

    if ($output =~ /aggr wafliron start $filer_vol: ./) {
        $Log->comment("$pkg_name: Wafliron started on $filer_vol successfully");
        return 1;
    } else {
        return 0;
    }
} ## end sub RaidSqa_aggr_wafliron_start

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_wafliron_commit

RaidSqa_aggr_wafliron_commit(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine commits wafliron on the test aggregate.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_wafliron_commit {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $node        = $opts{node};
    my ($response, $result, $output);

    $Log->comment("$pkg_name:Commit wafliron on aggr $filer_vol");

    $result = 1;
    try {
        $response = $api_set_obj->system_node_run(
                                 node    => $node,
                                 command => "\"priv set -q diag; aggr wafliron commit $filer_vol \""
        );
        $output = $response->get_raw_output();
        if ($output =~ /Committed change log files for aggregate $filer_vol/) {
            $Log->comment("$pkg_name:Wafliron committed the chages successfully");
            $result = 1;
        } else {
            $Log->comment("$pkg_name:Wafliron commit is not successful");
            $result = 0;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        $result = 0;
    }
    return $result;
} ## end sub RaidSqa_aggr_wafliron_commit

#--------------------------------------------------------------------------------------------------

=head2 RaidSqa_aggr_wafliron_review
RaidSqa_aggr_wafliron_review(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine review wafliron on the test aggregate .

It returns the output string of the review command .

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object


=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_wafliron_review {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $node        = $opts{node};
    my ($output, $response);

    $Log->comment("$pkg_name: Review wafliron on aggr $filer_vol");

    try {

        # Review wafliron on test aggr/volume
        $response = $api_set_obj->system_node_run(
                                 node    => $node,
                                 command => "\"priv set -q diag; aggr wafliron review $filer_vol \""
        );
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        $output = 0;
    };
    return $output;
} ## end sub RaidSqa_aggr_wafliron_review

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_wafliron_status

RaidSqa_aggr_wafliron_status(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine checks the status of wafliron on the test aggregate.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object


=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_wafliron_status {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $node        = $opts{node};
    my ($output, $result, $response);

    $Log->comment("$pkg_name:Checking Status of wafliron on aggr $filer_vol");

    #Starting wafliron on test aggr/volume

    try {
        $response = $api_set_obj->system_node_run(
                                 node    => $node,
                                 command => "\"priv set -q diag; aggr wafliron status $filer_vol \""
        );
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
    };

    if ($output =~ /not currently/) {
        return 0;
    } elsif ($output =~ /(active)|(wafliron in optional commit mode has completed on aggregate)/) {
        $Log->comment("$pkg_name:Wafliron is completed successfully on $filer_vol");
        return 1;
    } else {
        $Log->comment("$pkg_name:Wafliron status could not be identified: $output");
        return 0;
    }
} ## end sub RaidSqa_aggr_wafliron_status

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_media_scrub_start

RaidSqa_aggr_media_scrub_start(api_set_obj => $api_set_obj,conntype => $conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, disk_name=>$disk_name, node=>$node);

This subroutine by default start media_scrub on a given filer. But if any of the options from filer_vol, plex_name, group_name or disk_name is specified than it start media_scrub on that particular raid object.

It returns 1 on pass and 0 on fail

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item filer_vol

(Optional) : Name of aggr or trad vol.

=item plex_name

(Optional) : Name of plex.

=item group_name

(Optional) : Name of group.

=item disk_name

(Optional) : Name of disk.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_media_scrub_start {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        disk_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        node       => {type => SCALAR,         default => "local", optional => 1},}
                      );
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $disk_name   = $opts{disk_name};
    my $node        = $opts{node};
    my (@group_list, $output, $media_scrub_start_cmd);
    my $result = 1;

    my $media_scrub_option = RaidSqa_get_options(api_set_obj => $api_set_obj,
                                                 conntype    => $conntype,
                                                 option_name => 'raid.media_scrub.enable',
                                                 node        => $node);
    if ($media_scrub_option =~ /on/i) {
        $Log->comment("$pkg_name: Media scrubbing is currently enabled on this system");
    } else {
        $Log->comment("$pkg_name: Media scrubbing is currently not enabled on this system.");
        return 0;
    }

    if ($filer_vol) {
        if (RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                     conntype    => $conntype,
                                     filer_vol   => $filer_vol,
                                     node        => $node)
          ) {
            @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                                 conntype    => $conntype,
                                                 filer_vol   => $filer_vol,
                                                 node        => $node);
        } else {
            $Log->comment("$pkg_name: Aggregate $filer_vol does not exist");
            return 0;
        }
        $media_scrub_start_cmd = "aggr media_scrub start $filer_vol";
    } elsif ($plex_name) {
        if (RaidSqa_check_plex_exist(api_set_obj => $api_set_obj,
                                     conntype    => $conntype,
                                     plex_name   => $plex_name,
                                     node        => $node)
          ) {
            @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                                 conntype    => $conntype,
                                                 plex_name   => $plex_name,
                                                 node        => $node);
        } else {
            $Log->comment("$pkg_name: Plex $plex_name does not exist");
            return 0;
        }
        $media_scrub_start_cmd = "aggr media_scrub start $plex_name";
    } elsif ($group_name) {
        @group_list            = ($group_name);
        $media_scrub_start_cmd = "aggr media_scrub start $group_name";
    } elsif ($disk_name) {
        my $disk_active_path_name = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                                                       conntype    => $conntype,
                                                                       disk_name   => $disk_name,
                                                                       node        => $node);
        if ($disk_active_path_name) {
            $Log->comment("$pkg_name: Using path name $disk_active_path_name for any operations on $disk_name");
            $disk_name = $disk_active_path_name;
        } else {
            $Log->comment(
                 "$pkg_name: The disk_name $disk_name specified does not exist. Please check again");
            return (0);
    }

        @group_list            = $disk_name;
        $media_scrub_start_cmd = "aggr media_scrub start -s $disk_name";
    } else {
        my @aggr_list =
          RaidSqa_get_aggr_list(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);
        foreach my $aggr_list (@aggr_list) {
            if (RaidSqa_check_aggr_state_name(api_set_obj => $api_set_obj,
                                              conntype    => $conntype,
                                              filer_vol   => $aggr_list,
                                              state_name  => "online",
                                              node        => $node)
              ) {
                @group_list =
                  RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                         conntype    => $conntype,
                                         filer_vol   => $aggr_list,
                                         node        => $node);
            } ## end if (RaidSqa_check_aggr_state_name...
        } ## end foreach my $aggr_list (@aggr_list)
        $media_scrub_start_cmd = "aggr media_scrub start";
    } ## end else [ if ($filer_vol)
    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; $media_scrub_start_cmd \"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    if (($output =~ /No/) || ($output =~ /Invalid/) || ($output =~ /Illegal/)) {
        $result = 0;
    }

    foreach my $group_list (@group_list) {
        if ($output !~ /enabled background media scrub of $group_list/) {
            $Log->comment("$pkg_name: Media scrub not enabled for group $group_list");
            $result = 0;
        }
    }
    return $result;
} ## end sub RaidSqa_aggr_media_scrub_start

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_media_scrub_stop

RaidSqa_aggr_media_scrub_stop(api_set_obj => $api_set_obj,conntype => $conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, disk_name=>$disk_name, node=>$node);

This subroutine by default stop all media scrub on a given filer. But if any of the options from filer_vol, plex_name, group_name, or disk_name is specified than it stop media_scrub on that particular raid object.

It returns 1 on pass and 0 on fail

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggr or trad vol.

=item plex_name

(Optional) : Name of plex.

=item group_name

(Optional) : Name of group.

=item disk_name

(Optional) : Name of disk.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_media_scrub_stop {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        disk_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        node       => {type => SCALAR,         default => "local", optional => 1},}
                      );
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $disk_name   = $opts{disk_name};
    my $node        = $opts{node};
    my (@group_list, $output, $media_scrub_stop_cmd);
    my $result = 1;

    my $media_scrub_option = RaidSqa_get_options(api_set_obj => $api_set_obj,
                                                 conntype    => $conntype,
                                                 option_name => 'raid.media_scrub.enable');
    if ($media_scrub_option =~ /on/i) {
        $Log->comment("$pkg_name: Media scrubbing is currently enabled on this system");
    } else {
        $Log->comment("$pkg_name: Media scrubbing is currently not enabled on this system.");
        return 0;
    }

    if ($filer_vol) {
        if (RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                     conntype    => $conntype,
                                     filer_vol   => $filer_vol,
                                     node        => $node)
          ) {
            @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                                 conntype    => $conntype,
                                                 filer_vol   => $filer_vol,
                                                 node        => $node);
        } ## end if (RaidSqa_check_aggr_exist...
        $media_scrub_stop_cmd = "aggr media_scrub stop $filer_vol";
    } elsif ($plex_name) {
        if (RaidSqa_check_plex_exist(api_set_obj => $api_set_obj,
                                     conntype    => $conntype,
                                     plex_name   => $plex_name,
                                     node        => $node)
          ) {
            @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                                 conntype    => $conntype,
                                                 plex_name   => $plex_name,
                                                 node        => $node);
        } ## end if (RaidSqa_check_plex_exist...
        $media_scrub_stop_cmd = "aggr media_scrub stop $plex_name";
    } elsif ($group_name) {
        @group_list           = $group_name;
        $media_scrub_stop_cmd = "aggr media_scrub stop $group_name";
    } elsif ($disk_name) {
        my $disk_active_path_name = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                                                       conntype    => $conntype,
                                                                       disk_name   => $disk_name,
                                                                       node        => $node);
        if ($disk_active_path_name) {
            $Log->comment("$pkg_name: Using path name $disk_active_path_name for any operations on $disk_name");
            $disk_name = $disk_active_path_name;
        } else {
            $Log->comment(
                 "$pkg_name: The disk_name $disk_name specified does not exist. Please check again");
            return (0);
        }

        @group_list           = $disk_name;
        $media_scrub_stop_cmd = "aggr media_scrub stop -s $disk_name";
    } else {
        my @aggr_list =
          RaidSqa_get_aggr_list(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);
        foreach my $aggr_list (@aggr_list) {
            if (RaidSqa_check_aggr_state_name(api_set_obj => $api_set_obj,
                                              conntype    => $conntype,
                                              filer_vol   => $aggr_list,
                                              state_name  => "online",
                                              node        => $node)
              ) {
                @group_list =
                  RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                         conntype    => $conntype,
                                         filer_vol   => $aggr_list,
                                         node        => $node);
            } ## end if (RaidSqa_check_aggr_state_name...
        } ## end foreach my $aggr_list (@aggr_list)
        $media_scrub_stop_cmd = "aggr media_scrub stop";
    } ## end else [ if ($filer_vol)

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; $media_scrub_stop_cmd \"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    if (($output =~ /No/) || ($output =~ /Invalid/) || ($output =~ /Illegal/)) {
        $result = 0;
    }

    foreach my $group_list (@group_list) {
        if ($output !~ /disabled background media scrub of $group_list/) {
            $Log->comment("$pkg_name: Media Scrub not disabled for group $group_list");
            $result = 0;
        }
    }
    return $result;
} ## end sub RaidSqa_aggr_media_scrub_stop

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_media_scrub_status

RaidSqa_aggr_media_scrub_status(api_set_obj => $api_set_obj, conntype => $conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, disk_name=>$disk_name, node=>$node);

This subroutine by default provide media_scrub status on a given filer. But if any of the options from filer_vol, plex_name, group_name or disk_name is specified than it provide media_scrub status on that particular raid object.

It returns nothing but just the media_scrub status o/p

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggr or trad vol.

=item plex_name

(Optional) : Name of plex.

=item group_name

(Optional) : Name of group.

=item disk_name

(Optional) : Name of disk.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_media_scrub_status {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        disk_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        node       => {type => SCALAR,         default => "local", optional => 1},}
                      );
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $disk_name   = $opts{disk_name};
    my $node        = $opts{node};
    my $output;
    my $media_scrub_cmd;
    my $result = 1;

    if ($filer_vol) {
        $media_scrub_cmd = "aggr media_scrub status $filer_vol";
    } elsif ($plex_name) {
        $media_scrub_cmd = "aggr media_scrub status $plex_name";
    } elsif ($group_name) {
        $media_scrub_cmd = "aggr media_scrub status $group_name";
    } elsif ($disk_name) {
        my $disk_active_path_name = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                                                       conntype    => $conntype,
                                                                       disk_name   => $disk_name,
                                                                       node        => $node);
        if ($disk_active_path_name) {
            $Log->comment("$pkg_name: Using path name $disk_active_path_name for any operations on $disk_name");
            $disk_name = $disk_active_path_name;
        } else {
            $Log->comment(
                 "$pkg_name: The disk_name $disk_name specified does not exist. Please check again");
            return (0);
        }

        $media_scrub_cmd = "aggr media_scrub status -s $disk_name";
    } else {
        $media_scrub_cmd = "aggr media_scrub status";
    }
    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; $media_scrub_cmd \"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };
    return $output;
} ## end sub RaidSqa_aggr_media_scrub_status

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_verify_stop

RaidSqa_aggr_verify_stop(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, node=>$node);

This subroutine start aggregate verify on given filer_vol with target plex if defined.
If filer_vol is not defined, it will start aggregate verify on all aggregates in filer

It returns 1 on pass and 0 on fail

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggr or trad vol.

=item node

(Optional) : Name of the node on which aggregate verify is to be started

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_verify_stop {
    my (%opts) =
    RaidSqa_validate(@_,
        {
            api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
            conntype    => {type => SCALAR, default => "#ssh", optional => 1},
            node        => {type => SCALAR, default => "local", optional => 1},
            filer_vol   => {type => SCALAR | UNDEF, default => undef, optional => 1},
        }
    );

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $node         = $opts{node};
    my ($response, $output, $parsed_output);

    $filer_vol = "*" if(!defined($filer_vol));

    my %cmd_args = (aggregate => $filer_vol, action=>"stop");

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    try {
        $response = $api_set_obj->storage_aggregate_verify(%cmd_args);
        $parsed_output = $response->get_parsed_output();
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
    };

    # need to enable following code once burt 249002 got fixed
    #sleep 10;
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");

    $filer_vol = "all" if(!defined($opts{filer_vol}));

    if ($aggr_obj->parameter("verifypct") ne "-1") {
        sleep 5;
        $raid_obj->refresh();
        if ($aggr_obj->parameter("verifypct") ne "-1") {
            $Log->comment("$pkg_name: verify could not be stopped on aggr - $filer_vol");
            return 0;
        } else {
            $Log->comment("$pkg_name: verify stop on aggr - $filer_vol");
            return 1;
        }

    } else {
        $Log->comment("$pkg_name: Aggr verify stopped on aggr - $filer_vol");
        return 1;
    }
} ## end sub RaidSqa_aggr_verify_stop

#------------------------------------------------------------------------------

=head2 RaidSqa_test_init

RaidSqa_test_init(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, aggr_type=>$aggr_type, node=>$node);

This subroutine is called before starting any tests, this will do following:
a. stop hammer,
b. unisolate any disk in olm,
c. online all offline disks
d. unfail all failed disks,
e. remove a particular aggregate having same name as required for test agregate,

It returns nothing.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate to be checked.

=item aggr_type

(Required): 1 for aggregate and 0 for traditional. Default it is 1. For cmode traditional volume is not supported, hence we  will set it to 1 if provided as 0.

=item node

(Optional): Node name on which the operation is to be performed

=item destroy_inconsistent_aggrs

(Optional) : If set to 1, wafl inconsistent volumes will be removed.Default is 0.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_test_init {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        aggr_type => {type => SCALAR, default => 1, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},
                        destroy_inconsistent_aggrs=>{type => SCALAR, default => 0, optional => 1}});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $aggr_type   = $opts{aggr_type};
    my $node        = $opts{node};
    my $destroy_inconsistent_aggrs = $opts{destroy_inconsistent_aggrs};

    if (!$aggr_type) {
        $Log->comment("$pkg_name: aggr_type as 0 is not supported in Cmode hence chnaging it to 1");
        $aggr_type = 1;
    }

    # abort hammer on filer
    RaidSqa_hammer_abort(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);

    # Unisolate all disk if any disk is in OLM
    RaidSqa_labelmaint_unisolate(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);

    # online all offline disk if any disk
    RaidSqa_disk_online(api_set_obj => $api_set_obj,
                        conntype    => $conntype,
                        disk_name   => "all",
                        node        => $node);

    # Unfail all disk to spare on filer
    RaidSqa_disk_unfail(api_set_obj => $api_set_obj,
                        conntype    => $conntype,
                        disk_name   => "all",
                        options     => "-s",
                        node        => $node);

    # remove the aggr if any exist with specified name
    RaidSqa_aggr_remove(api_set_obj => $api_set_obj,
                        conntype    => $conntype,
                        filer_vol   => $filer_vol,
                        aggr_type   => $aggr_type,
                        node        => $node,
                        destroy_inconsistent_aggrs => $destroy_inconsistent_aggrs);
} ## end sub RaidSqa_test_init

#------------------------------------------------------------------------------

=head2 RaidSqa_media_error_inject

RaidSqa_media_error_inject(api_set_obj => $api_set_obj, conntype=>$conntype, disk_name =>$disk_name, dbn => $dbn, node => $node);

This subroutine injects media error on given disk at given dbn depending on storage type.

It returns 1 on success.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item dbn

(Required) : dbn no to inject media error.

=item disk_name

(Required): Name of disk to inject media erro.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_media_error_inject {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        dbn       => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $node        = $opts{node};

    #Check if disk name exists or has an equivalent path_name2.
    if (defined($disk_name)) {
        my $disk_name1 =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        if ($disk_name1) {
            $Log->comment("$pkg_name: Using path name $disk_name1 for any operations on $disk_name");
            $disk_name = $disk_name1;
        } else {
            $Log->comment(
                 "$pkg_name: The disk_name $disk_name specified does not exist. Please check again");
            return (0);
        }
    } ## end if (defined($disk_name...
    my $dbn = $opts{dbn};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $storage_type =
      RaidSqa_get_disk_storage_type(api_set_obj => $api_set_obj,
                                    disk_name   => $disk_name,
                                    node        => $node);
    if (!$storage_type) {
        return 0;
    }
    if ($storage_type =~ /ata|bsas|sata|fsas/i) {
        $Log->comment("$pkg_name: Injecting media error through fiji.");
        my $media_error_1 =
          '(repeat 100000 (raid.disk.read.done.check (eq @disk' . " $disk_name)(return 1)))";
        my $media_error_2 =
            '((raid.disk.read.done.buf (&&(eq @disk'
          . " $disk_name) (eq " . '@bno'
          . " $dbn))(return (buf_Errno E_DISK_BAD_SECTOR_ERROR))))";
        if (!RaidSqa_fiji_filter(api_set_obj => $api_set_obj,
                                 fiji_point  => $media_error_1,
                                 node        => $node)
          ) {
            $Log->comment("$pkg_name: Media error 1 injection on disk $disk_name failed");
            return 0;
        }
        if (!RaidSqa_fiji_filter(api_set_obj => $api_set_obj,
                                    fiji_point  => $media_error_2,
                                    node        => $node)
          ) {
            $Log->comment("$pkg_name: Media error 2 injection on disk $disk_name failed");
            return 0;
        }
            $Log->comment("$pkg_name: Media error injected successfully using fiji");
            return 1;
    } elsif ($storage_type =~ /FCAL|SAS/i) {
        $Log->comment("$pkg_name: Injecting media error through disk_mung.");
        my $sector = 8 * $dbn;
        RaidSqa_disk_mung(api_set_obj => $api_set_obj,
                          disk_name   => $disk_name,
                          sector      => $sector,
                          node        => $node);
        $Log->comment("$pkg_name: Media error injected successfully using disk_mung");
        return 1;
    } else {
        logresult("$pkg_name: Disk has unknown storage type.Its storage type is $storage_type.");
        return 0;
    }
} ## end sub RaidSqa_media_error_inject

#------------------------------------------------------------------------------

=head2 RaidSqa_media_scrub_enable

RaidSqa_media_scrub_enable(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);

This subroutine enable the media_scrub on filer by turning on the raid options raid.media.scrub.enable

It returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_media_scrub_enable {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    try {
        my ($output, $output1, @output1);

        my $response =
          $api_set_obj->system_node_run(
                               node    => $node,
                               command => "\"priv set -q diag; options raid.media_scrub.enable on\""
          );
        $output = $response->get_raw_output();
        $response = $api_set_obj->system_node_run(
                                 node    => $node,
                                 command => "\" priv set -q diag; options raid.media_scrub.enable\""
        );
        $output1 = $response->get_raw_output();
        @output1 = split(/\r?\n/, $output1);
        if (grep(/on/, @output1)) {
            $Log->comment(
                "$pkg_name: Media scrubbing is currently enabled on this system - checking with aggr status output"
            );

            #return 1;
        } else {
            $Log->comment(
                "$pkg_name: Media scrubbing is currently not enabled on this system- checking with aggr status output"
            );

            #return 0;
        }

        $response = $api_set_obj->system_node_run(
                                          node    => $node,
                                          command => "\"priv set -q diag; aggr media_scrub status\""
        );
        $output1 = $response->get_raw_output();
        sleep 5;
        @output1 = split(/\r?\n/, $output1);
        if (grep(/\d+\%\s+complete/, @output1)) {
            $Log->comment("$pkg_name: Media scrubbing is currently enabled on this system");
            return 1;
        } else {
            $Log->comment("$pkg_name: Media scrubbing is currently not enabled on this system");
            return 0;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $output           = _handle_nacl_exception($exception_object);
        return 0;
    };
} ## end sub RaidSqa_media_scrub_enable

#------------------------------------------------------------------------------

=head2 RaidSqa_media_scrub_disable

RaidSqa_media_scrub_disable(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);

This subroutine disable the media_scrub on filer by turning off the raid options raid.media.scrub.enable

It returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_media_scrub_disable {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my ($output, $output1, @output1);
    try {
        my $response =
          $api_set_obj->system_node_run(
                             node    => $node,
                             command => "\" priv set -q diag; options raid.media_scrub.enable off\""
          );
        $output = $response->get_raw_output();
        sleep 5;
        $response = $api_set_obj->system_node_run(
                                          node    => $node,
                                          command => "\"priv set -q diag; aggr media_scrub status\""
        );
        $output1 = $response->get_raw_output();
        sleep 5;
        @output1 = split(/\r?\n/, $output1);
        if (grep(/Media scrubbing is currently disabled on this system/, @output1)) {
            $Log->comment("$pkg_name: Media scrubbing is currently disabled on this system");
            return 1;
        } else {
            $Log->comment("$pkg_name: Media scrubbing is currently not disabled on this system");
            return 0;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        return 0;
    };
} ## end sub RaidSqa_media_scrub_disable

#------------------------------------------------------------------------------

=head2 RaidSqa_filer_cmd

RaidSqa_filer_cmd(api_set_obj => $api_set_obj, conntype=>$conntype, cmd=>$cmd, timeout=>$timeout, node => $node, node=>$node);

This subroutine run run any filer comamnd where we are not expecting o/p for analysis, just want to run the command

It returns o/p of the command if any

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item cmd

(Optional): filer command.

=item options

(Optional): gives options to be used while executing the filer command.

=item prompt

(Optional): gives prompt to be wait for in case of executing say command.

=item timeout

(Optional): timeout value in secs.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_filer_cmd {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        cmd     => {type => SCALAR},
                        options => {type => SCALAR, optional => 1},
                        prompt  => {type => SCALAR, optional => 1},
                        timeout => {type => SCALAR, default => $RaidSqa_TIMEOUT, optional => 1},
                        node    => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $cmd         = $opts{cmd};
    my $options     = $opts{options};
    my $prompt      = $opts{prompt};
    my $timeout     = $opts{timeout};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }
    my $conn_console = make_conn($host, "#console");
    my $output;

    try {
        if ((!defined($options)) && (!defined($prompt))) {
            my $response =
                $api_set_obj->system_node_run(node    => $node,
                                            command => "\"priv set -q diag; $cmd \"");
            $output = $response->get_raw_output();
        } elsif ((defined($options)) && (!defined($prompt))) {
            (undef, undef, $output) =
              say($conn_console, "$cmd", "$options", -timeout, $timeout);
        } elsif ((!defined($options)) && (defined($prompt))) {
            (undef, undef, $output) = say($conn_console, "$cmd", -prompt, $prompt, -timeout, $timeout);
        } else {
            (undef, undef, $output) = say($conn_console, "$cmd", -prompt, $prompt, $options, -timeout, $timeout);
        }
        return $output;
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $output           = _handle_nacl_exception($exception_object);
        return 0;
    };
} ## end sub RaidSqa_filer_cmd

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_file_create

RaidSqa_file_create(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node, file_name=>$file_name, file_size=>$file_size, append=>$append);

This subroutines mounts volume and creates the file of given size (max 1G)

The following arguments are accepted:

=over

=item api_set_obj

(Required): Hostspec to filer

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional) : node on which file $file_name is created

=item file_name

(Required): Qulified name of the file to be created. eg. /vol/vol_name/file_name

=item file_size

(Required): Size of the file to be created (e.g 10k, 20m, 1g)
max file size that can be created at a time is 1g

=item append

(Optional): if 1, data is appended to existing file

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_file_create {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        file_name   => {type => SCALAR},
                        vserver     => {type => SCALAR, default => "vs0", optional => 1},
                        file_size   => {type => SCALAR},
                        client      => {type => SCALAR, default => $RaidSqa_CLIENT, optional => 1},
                        append      => {type => SCALAR, default => 0, optional => 1}});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $file_size   = $opts{file_size};
    my $file_name   = $opts{file_name};
    my $append      = $opts{append};
    my $vserver     = $opts{vserver};
    my $filer_vol   = "";
    my $filename    = "";
    my $string      = join("", ("a" .. "z", 0 .. 9));
    my $loop        = 1;
    my $result      = 1;
    if ($file_size =~ /^(\d+)(k|m|g)?b?$/i) {
        my $key   = $2;
        my $value = $1;
        if ($value > 1024) {
            $Log->comment("$pkg_name: Digit part of size should be less than 1024");
            return 0;
        }
        if ($key =~ /g/i && $value > 1) {
            $Log->comment("$pkg_name: Maximum supported file size is 1G");
            $Log->comment(
                 "$pkg_name: To create file more that 1G, recall this function in loop for same file"
            );
            return 0;
        }
        if ($key =~ /g/i) {
            $loop = 1000;
            $string = $string x (1048576 / 36);    #36 is string size
        } elsif ($key =~ /m/i) {
            $loop = $value;
            $string = $string x (1048576 / 36);
        } elsif ($key =~ /k/i) {
            $string = $string x (($value * 1024) / 36);
        }
    } else {
        $Log->comment("$pkg_name: unknown file_size format");
        return 0;
    }

    #max supported size is 1G, for more than that you can recall this function,
    #it will append to existing file

    if ($file_name =~ m(^/vol/([^/]+)/(.+)$)) {
        $filer_vol = $1;
        $filename  = $2;
    } elsif ($file_name =~ m(^/(.+)$)) {
        $filer_vol =
          RaidSqa_get_root_vol(api_set_obj => $api_set_obj, node => $node, conntype => $conntype,);
        $filename = $1;
    }

    my $mountpoint = RaidSqa_aggr_mount(api_set_obj => $api_set_obj,
                                        node        => $node,
                                        conntype    => $conntype,
                                        filer_vol   => $filer_vol,
                       vserver     => $vserver);
    if (!$mountpoint) {
        $Log->comment("$pkg_name: aggr $filer_vol could not be mounted :: $mountpoint");
        return 0;
    }

    my $fullFileName = "$mountpoint/$filename";
    eval {
        if ($append && -e $fullFileName) {
            sudo("chmod 777 $fullFileName");
            open(FILE, ">>$fullFileName") or die("Could not open file $fullFileName : $!");
        } else {
            sudo("touch $fullFileName");
            sudo("chmod 777 $fullFileName");
            open(FILE, ">$fullFileName") or die("Could not open file $fullFileName : $!");
        }

        $Log->comment("$pkg_name: Creating file $fullFileName of size $file_size");
        foreach (1 .. $loop) {
            print FILE "$string\n" or die("Could not write to file $fullFileName: $!");
        }
    };

    if ($@) {
        $Log->comment("$pkg_name: $@");
        my $size = -s $fullFileName;
        $Log->comment("$pkg_name: Unknown error detected. File created of size: $size");
        $result = 0;
    }
    my $size = -s $fullFileName;
    $Log->comment("$pkg_name: File created of size : $size");
    sleep(5);

    close(FILE);
    umount($mountpoint);
    return $result;
} ## end sub RaidSqa_file_create

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_file_nfs_read

RaidSqa_file_nfs_read(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node, file_name=>$file_name);

This subroutines mounts the given volume and then reads file to NFS

The following arguments are accepted:

=over

=item api_set_obj

(Required): Hostspec to filer

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional) : node on which file $file_name is created

=item file_name

(Required): Absolute path of file name to be read from client eg. /vol/vol_name/file_name

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_file_nfs_read {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        vserver   => {type => SCALAR, default => "vs0", optional => 1},
                        file_name => {type => SCALAR}});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $file_name   = $opts{file_name};
    my $filer_vol   = "";
    my $filename    = "";
    my $counter    = 0;
    my $result      = 1;
    my $vserver     = $opts{vserver};

    if ($file_name =~ m(^/vol/([^/]+)/(.+)$)) {
        $filer_vol = $1;
        $filename  = $2;
    } elsif ($file_name =~ m(^/(.+)$)) {
        $filer_vol =
          RaidSqa_get_root_vol(api_set_obj => $api_set_obj, node => $node, conntype => $conntype,);
        $filename = $1;
    }

    my $mountpoint = RaidSqa_aggr_mount(api_set_obj => $api_set_obj,
                                        node        => $node,
                                        conntype    => $conntype,
                                        filer_vol   => $filer_vol,
                                        vserver     => $vserver);
    if (!$mountpoint) {
        $Log->comment("$pkg_name: aggr $filer_vol could not be mounted :: $mountpoint");
        return 0;
    }

    my $fullFileName = "$mountpoint/$filename";

    eval {
        open(FILE, "<$fullFileName") or die("Could not open file $fullFileName : $!");
        while(<FILE>) {
            $counter++ if(defined $_);
        }
        if($counter) {
            $Log->comment("$pkg_name: Read $counter lines from file $fullFileName");
        } else {
            die("Could not read from file $fullFileName");
        }
    };

    if ($@) {
        $Log->comment("$pkg_name: $@");
        $result = 0;
    }
    sleep(5);

    close(FILE);
    umount($mountpoint);
    return $result;
} ## end sub RaidSqa_file_nfs_read

#------------------------------------------------------------------------------

=head2 RaidSqa_get_md5sum

RaidSqa_get_md5sum(api_set_obj=>$api_set_obj, conntype=>$conntype, file_name=>$file_name);

Subroutine to return the md5sum of a given file

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item file_name

(Optional): full qualified name (w.r.t Filer) of the file for which md5sum is to be generated.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_md5sum {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype      => {type => SCALAR, default => "#ssh", optional => 1},
                        vserver   => {type => SCALAR, default => "vs0", optional => 1},
                        file_name     => {type => SCALAR}}
                        );

    my $api_set_obj         = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $file_name    = $opts{file_name};
    my $vserver     = $opts{vserver};

    my ($filer_vol, $filename);

    # Extract filename from complete path
    if($file_name =~ m(^/vol/([^/]+)/(.+)$)) {
        $filer_vol = $1;
        $filename = $2;
    } elsif($file_name =~ m(^/(.+)$)) {
        $filer_vol = RaidSqa_get_root_vol(api_set_obj=>$api_set_obj);
        $filename = $1;
    }

    $Log->comment("$pkg_name: Working on md5sum at -\nVolume name - $filer_vol \nFilename - $filename");

    $Log->comment("$pkg_name: mount the aggr $filer_vol");
    my $mountpoint = RaidSqa_aggr_mount(api_set_obj=>$api_set_obj, filer_vol=>$filer_vol, vserver => $vserver);
    if(!$mountpoint) {
        $Log->comment("$pkg_name: aggr $filer_vol could not be mounted");
    }

    my $md5checksum;
    eval{
        my $fullFileName = "$mountpoint/$filename";
        open(FILE, $fullFileName ) or die "Error reading file $fullFileName \n";
        $Log->comment("$pkg_name: Computing md5 checksum");
        my $md5obj = Digest::MD5->new();
        $md5obj->addfile(*FILE);
        $md5checksum = $md5obj->hexdigest;
        close(FILE);
    };
    if($@) {
        $Log->comment("$pkg_name: $@");
        return 0;
    }


    $Log->comment("$pkg_name: Volume name - $filer_vol \nFilename - $filename \nmd5sum - $md5checksum");

    umount($mountpoint);
    return($md5checksum);
}

#------------------------------------------------------------------------------

=head2 RaidSqa_dd

RaidSqa_dd(api_set_obj => $api_set_obj, conntype=>$conntype, if=>$if, of=$of, din=$din, bin=>$bin, dout=>$dout, bout=$bout, count=$count, options=>$options, node => $node);

This subroutine run dd command to corrupt the disk blocks. Need a non-promoted debug build.

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item if

(Required): Name of input file.

=item of

(Required): Name of output file.

=item din

(Required): name of source disk.

=item bin

(Optional): Block number of source disk (wafl block 8k size).

=item dout

(Required): name of destination disk.

=item bout

(Optional): Block number of destination disk (wafl block 8k size).

=item count

(Optional): Number of block to be copied.

=item node

(Optional): Node name on which the operation is to be performed

=item options

(Optional): additional options to be passed to dd command.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_dd {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        if          => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        of          => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        din         => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        bin         => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        dout        => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        bout        => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        count       => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        options     => {type => SCALAR | UNDEF, default => "-c",   optional => 1},
                        retry_count => {type => SCALAR,         default => 3,   optional => 1},
                        node        => {type => SCALAR,         default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $if          = $opts{if};
    my $of          = $opts{of};
    my $din         = $opts{din};
    my $bin         = $opts{bin};
    my $dout        = $opts{dout};
    my $bout        = $opts{bout};
    my $count       = $opts{count};
    my $options     = $opts{options};
    my $retry_count     = $opts{retry_count};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $cmd_args;
    if (defined($if)) {
        if (defined($of) && defined($count)) {
            $cmd_args = "dd if=$if of=$of count=$count";
        } elsif (defined($of)) {
            $cmd_args = "dd if=$if of=$of";
        } elsif (defined($dout) && defined($bout) && defined($count)) {
            $cmd_args = "dd if=$if dout=$dout bout=$bout count=$count";
        }
    } elsif (defined($din) && defined($bin)) {
        if (defined($of) && defined($count)) {
            $cmd_args = "dd din=$din bin=$bin of=$of count=$count";
        } elsif (defined($of)) {
            $cmd_args = "dd din=$din bin=$bin of=$of";
        } elsif (defined($dout) && defined($bout) && defined($count)) {
            $cmd_args = "dd din=$din bin=$bin dout=$dout bout=$bout count=$count";
        }
    } else {
        $Log->comment("$pkg_name: right parameter not defined to dd command");
        return 0;
    }

    #Options will be of no use where input and output source is file
    if ($options && !(defined($of) && defined($if))) {
        $cmd_args .= " $options";
    }


    my $result = 1;
    my $output = undef;
    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; $cmd_args \"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    $Log->comment("$pkg_name: DD command output - $output");
    if($output =~ /Undefined error:\s*280/im) {
        #retry for 3 times else panic
        if($retry_count > 0) {
            sleep(30);
            $opts{retry_count} -= 1;
            RaidSqa_dd(%opts);
        }

        # #Panic node after 3+1 retries
        # $Log->comment("$pkg_name: Error: BURT669013: panicking filer to get the core for dd error message");
        # eval {
            # #Eval to avoid script failure due to prompt timeout after panic
            # RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd => 'priv set test; panic -d');
        # };
    }

    return $result;
} ## end sub RaidSqa_dd

#------------------------------------------------------------------------------

=head2 RaidSqa_log

RaidSqa_log(api_set_obj=>$api_set_obj, conntype=>$conntype, marker=>$marker, node => $node);

This subroutine log a marker in syslog file.

Return 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item marker

(Required): Name of the marker (message) which we want to add to log for marking purpose.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_log {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        marker => {type => SCALAR},
                        node   => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $marker      = $opts{marker};
    my $node        = $opts{node};
    my $result      = 1;
    my $output;

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; log $marker\" ");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };
    return $result;
} ## end sub RaidSqa_log

#------------------------------------------------------------------------------

=head2 RaidSqa_savecore

RaidSqa_savecore(api_set_obj=>$api_set_obj, conntype=>$conntype, options=>$options, node=>$node);

This subroutine run savecore command with various options provided by user.

It returns output of command.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item options

(Optional): Options of savecore command.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_savecore {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        options => {type => SCALAR | UNDEF, default => undef, optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $options     = $opts{options};

    my ($output);
    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; savecore $options \"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
    };
    return $output;
} ## end sub RaidSqa_savecore

#------------------------------------------------------------------------------

=head2 RaidSqa_nv_trash

RaidSqa_nv_trash(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node);

This subroutine Trash nvram to cause a panic with nvram loss so that there is reparity on boot.

It returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_nv_trash {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        timeout     => {type => SCALAR, default => 1200, optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $timeout     = $opts{timeout};
    my $conntype    = $opts{conntype};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $output;

    try {
        my $response =
          $api_set_obj->system_node_run(
                            node                  => $node,
                            command               => "\"priv set -q test; nv_trash toc all 0x12345678 \"",
                            'connectrec-timeout'  => $timeout
                        );
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;

        #TODO:: Need to change the way it handles
        # We expect here the failure as this code will not return prompt.
        $output = _handle_nacl_exception($exception_object);
    };
} ## end sub RaidSqa_nv_trash

#------------------------------------------------------------------------------

=head2 RaidSqa_disk_online

RaidSqa_disk_online(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name =>$disk_name,node=>$node);

This subroutine online one disk or all offline disks if disk name is given as "all".

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to online. if all than online all offline disks in filer

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_disk_online {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $node        = $opts{node};

    my (@offlined_disks, $host, $conn, $output);
    my $result = 1;
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    if ($disk_name =~ /all/i) {
        @offlined_disks =
          RaidSqa_get_disk_list_by_fsm_state(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             fsm_state   => "OFFLINE",
                                             node        => $node);
    } else {
        if (!RaidSqa_check_disk_exist(api_set_obj => $api_set_obj,
                                      conntype    => $conntype,
                                      disk_name   => $disk_name,
                                      node        => $node,)
          ) {
            $Log->comment("$pkg_name: Disk $disk_name does not exist.");
            return 0;
        }
        @offlined_disks = ($disk_name);
    } ## end else [ if ($disk_name =~ /all/i)
    @offlined_disks = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@offlined_disks);
    foreach my $disk (@offlined_disks) {
        try {
            my $response =
              $api_set_obj->system_node_run(node    => $node,
                                            command => "\"priv set -q diag; disk online $disk \" ");
            $output = $response->get_raw_output();
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            $result = 0;
        };
        sleep 10;
    } ## end foreach my $disk (@offlined_disks)

    $raid_obj->refresh();
    @offlined_disks = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@offlined_disks);
    foreach my $disk (@offlined_disks) {
        my $disk_obj;
        $disk_obj = $raid_obj->disk("$disk");

        # check disk got onlined
        if ($disk_obj->parameter("fsm_state") eq "OFFLINE") {
            $Log->comment("$pkg_name: Disk $disk still offlined.");
            $result = 0;
        } elsif (   ($disk_obj->parameter("fsm_state") eq "RECONS")
                 || ($disk_obj->parameter("fsm_state") eq "INUSE")
                 || ($disk_obj->parameter("fsm_state") eq "ZEROED")
                 || ($disk_obj->parameter("fsm_state") eq "NOTZEROED")) {
            $Log->comment("$pkg_name: disk $disk got online successfully");
        } else {
            $Log->comment("$pkg_name: Disk $disk could not be onlined.");
            $result = 0;
        }
    } ## end foreach my $disk (@offlined_disks)
    return $result;
} ## end sub RaidSqa_disk_online

#------------------------------------------------------------------------------

=head2 RaidSqa_disk_offline

RaidSqa_disk_offline(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, options=>$options, node=>$node);

This subroutine offline the disk.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to offline.

=item options

(Optional) : bdfu or timeout.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_disk_offline {

    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        options   => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $options     = $opts{options};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if (!RaidSqa_check_disk_exist(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  disk_name   => $disk_name,
                                  node        => $node,)
      ) {
        $Log->comment("$pkg_name: Disk $disk_name does not exist.");
        return 0;
    }

    my ($output, $disk_offline_cmd);
    my $result = 1;

    $disk_name =
      RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                         conntype    => $conntype,
                                         disk_name   => $disk_name,
                                         node        => $node);
    if (!defined($options)) {
        $disk_offline_cmd = "disk offline $disk_name";
    } else {
        $disk_offline_cmd = "disk offline -t $options $disk_name";
    }
    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; $disk_offline_cmd\" ");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    sleep 10;
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }

    # check disk got offlined
    if ($disk_obj->parameter("fsm_state") eq "OFFLINE") {
        $Log->comment("$pkg_name: Disk $disk_name got offline successfully");
    } else {
        $Log->comment("$pkg_name: Disk $disk_name could not be offlined.");
        $result = 0;
    }
    return $result;
} ## end sub RaidSqa_disk_offline

#------------------------------------------------------------------------------

=head2 RaidSqa_disk_fail

RaidSqa_disk_fail(api_set_obj => $api_set_obj, conntype=>$conntype, disk_name => $disk_name, options=>$options, node => $node);

This subroutine fail/prefail the disk.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to fail/prefail.

=item options

(Optional) : need to give as "-i" for disk fail otherwise disk will prefail.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_disk_fail {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        options   => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $options     = $opts{options};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $response_obj;

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    if (!defined($disk_obj)) {
        $Log->comment("$pkg_name: Could not find disk with name $disk_name");
        return 0;
    }
    my $output;
    try {

        # checking that specified disk is a file system disk before failing it
        if ($disk_obj->parameter("fsm_state") eq "INUSE") {
            my $response =
              $api_set_obj->system_node_run(
                                node    => $node,
                                command => "\"priv set -q diag; disk fail $options -f $disk_name \""
              );
            $output = $response->get_raw_output();

            ## TODO : better parsing of output and put comments based on that - this is not working now
            #my $immediate  = ($options eq "-i") ? "true" : "false";
            #$response_obj = $api_set_obj->storage_disk_fail(disk => $disk_name,
            #                                                immediate => $immediate);
            #$Log->comment("$pkg_name: Response parsed output:\n" . Dumper($response_obj->get_parsed_output()));
        } else {
            $Log->comment(
                    "$pkg_name: disk $disk_name is not a file system disk, so can not fail/prefail");
            return 0;
        }

        sleep(5);
        $raid_obj->refresh();
        $disk_obj = $raid_obj->disk("$disk_name");

        if (!$disk_obj) {
            $disk_name =
              RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                                 conntype    => $conntype,
                                                 disk_name   => $disk_name,
                                                 node        => $node);
            $disk_obj = $raid_obj->disk("$disk_name");
            if (!$disk_obj) {
                $Log->comment("$pkg_name: Disk with name $disk_name could not be found");
                return 0;
            }
        } ## end if (!$disk_obj)

        # check disk got failed/prefailed successfully
        if (!defined($options)) {
            if ($disk_obj->parameter("prefail") eq "1") {
                $Log->comment("$pkg_name: disk $disk_name got prefailed successfully");
                return 1;
            } else {
                $Log->comment("$pkg_name: Disk $disk_name could not be prefailed.");
                return 0;
            }

        } elsif ($options eq "-i") {
            if ($disk_obj->parameter("fsm_state") eq "FAILED") {
                $Log->comment("$pkg_name: disk $disk_name got failed successfully");
                return 1;
            } else {
                $Log->comment("$pkg_name: Disk $disk_name could not be failed.");
                return 0;
            }
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };

} ## end sub RaidSqa_disk_fail

#------------------------------------------------------------------------------

=head2 RaidSqa_disk_unfail

RaidSqa_disk_unfail(api_set_obj => $api_set_obj, conntype=>$conntype, disk_name=>$disk_name, options=>$options, node => $node);

This subroutine unfail one failed disk or all failed disks. If no options given than it is just use "disk unfail" otherwise we can pass options "-s" to this function to make disk as spare in all cases.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of failed disk to unfail, if all than it will unfail all failed disks.

=item options

(Optional) : we can pass options "-s" if we want to spare out the failed disk.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_disk_unfail {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        options   => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $options     = $opts{options};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $response_obj;

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    my $output;

    my @disks_to_be_unfailed = ();
    my %failed_disks_hash    = ();

    if ($disk_name eq "all") {
        my %failed_disks_hash = $raid_obj->failed_disks();
        @disks_to_be_unfailed = keys(%failed_disks_hash);
        if (!@disks_to_be_unfailed) {
            $Log->comment("$pkg_name: No failed Disk in system.");
            return 1;
        }
    } elsif (defined($disk_name)) {
        my $disk_name1 =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        if ($disk_name1) {
            $Log->comment("$pkg_name: Using path name $disk_name1 for any operations on $disk_name");
            $disk_name = $disk_name1;
        } else {
            $Log->comment(
                 "$pkg_name: The disk_name $disk_name specified does not exist. Please check again");
            return 0;
        }
        @disks_to_be_unfailed = ($disk_name);
    } ## end elsif (defined($disk_name...

    my $result = 1;
    try {

        # if disk_name is given as "all" than unfail all disks using specified options like "-s" to make it spare or without "-s" to assimilate to previous state

        $Log->comment("$pkg_name: Unfailing failed disks: @disks_to_be_unfailed");
        foreach my $disk (@disks_to_be_unfailed) {

            my $state = ($options eq "-s") ? "spare" : "unfail";

            #Currently CDEFs are not defined for "storage_disk_unfail hence using system node run"
            my $response =
              $api_set_obj->system_node_run(
                                      node    => $node,
                                      command => "\"priv set -q diag; disk unfail $options $disk \""
              );
            my $output = $response->get_raw_output();

            # $response = $api_set_obj->storage_disk_modify(state=>$state, pass_through=>$disk);
            # $output = $response->get_raw_output();
            #$response_obj => $api_set_obj->storage_disk_unfail(disk => $disk, '-s' => "true");
            #$Log->comment("$pkg_name: Response parsed output:\n" . Dumper($response_obj->get_parsed_output()));

            sleep 10;
        }
        $raid_obj->refresh();
        sleep 10;
        foreach my $disk (@disks_to_be_unfailed) {
            my ($disk_obj, $disk_path1, $disk_path2);
            if ($disk_name eq "all") {
                $disk_path1 = $failed_disks_hash{$disk}{"path_name1"};
                $disk_path2 = $failed_disks_hash{$disk}{"path_name2"};
            } else {
                $disk_path1 = $disk;
            }
            if (!($disk_obj = $raid_obj->disk("$disk_path1"))) {
                $disk_obj = $raid_obj->disk("$disk_path2");
            }

            if (!defined($disk_obj)) {
                $Log->comment("$pkg_name: Disk $disk could not be found in raid_config output");
                $result = 0;
                return 0;
            }

            if ($disk_obj->parameter("fsm_state") eq "FAILED") {
                $Log->comment("$pkg_name: Disk $disk could not be unfailed.");
                $result = 0;
                return 0;
            } else {
                $Log->comment("$pkg_name: Disk $disk successfully unfailed.");
            }
        } ## end foreach my $disk (@disks_to_be_unfailed)

        if ($disk_name eq "all") {
            $raid_obj->refresh();
            %failed_disks_hash = $raid_obj->failed_disks();
            my @failed_disks_list = keys(%failed_disks_hash);

            # check all disk got unfailed
            if (!@failed_disks_list) {
                $Log->comment("$pkg_name: There is no failed disk in the system");
                return 1;
            } else {
                $Log->comment("$pkg_name: Filer still have some failed disks: @failed_disks_list");
                return 0;
            }
        } ## end if ($disk_name eq "all")
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };
    return 1;
} ## end sub RaidSqa_disk_unfail

#------------------------------------------------------------------------------

=head2 RaidSqa_disk_remove

RaidSqa_disk_remove(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine remove the one spare disk or all spare disks from spare list and move them to broken list nonpersistantly.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of spare disk to remove, if all than it will remove all spare disks.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_disk_remove {
    my (%opts) =
      RaidSqa_validate(@_,
               {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                node        => {type => SCALAR, default => "local", optional => 1},
                conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};

    # create connection
    my ($host, $conn, $node_name, $output);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my ($result, @disks_to_be_removed);
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    # removing all spare disks from raid if disk_name is given as "all"
    if ($disk_name eq "all") {
        my %spare_disks_hash = $raid_obj->spare_disks();
        my @spare_disks_list = keys(%spare_disks_hash);

        if (!@spare_disks_list) {
            $Log->comment("$pkg_name: No spare Disk in system.");
            return 1;
        } else {
            @disks_to_be_removed = @spare_disks_list;
        }
    } else {
        my $disk_name = @disks_to_be_removed = ($disk_name);
    }

    $Log->comment("$pkg_name: Removing disks : @disks_to_be_removed");
    @disks_to_be_removed = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disks_to_be_removed);
    foreach my $disk (@disks_to_be_removed) {
        if (!RaidSqa_check_disk_exist(api_set_obj => $api_set_obj,
                                      conntype    => $conntype,
                                      disk_name   => $disk,
                                      node        => $node)
          ) {
            next;
        }

        try {

            #TODO :: Test and raise burt if not working
            #$api_set_obj->set(privilege=>"test");
            #my $response = $api_set_obj->storage_disk_remove(disk=>$disk);
            my $response =
              $api_set_obj->system_node_run(node    => $node,
                                            command => "\"priv set -q diag; disk remove $disk\"");
            $output = $response->get_raw_output();
        } ## end try
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_obj = shift;
            $output = _handle_nacl_exception($exception_obj);
        };
    } ## end foreach my $disk (@disks_to_be_removed)

    $result = 1;
    foreach my $disk (@disks_to_be_removed) {
        if (!RaidSqa_check_disk_fsm_state(api_set_obj => $api_set_obj,
                                          conntype    => $conntype,
                                          disk_name   => $disk,
                                          fsm_state   => "FAILED")
          ) {
            $Log->comment("$pkg_name: disk $disk is still not in failed (admin removed) state");
            $result = 0;
        } else {
            $Log->comment("$pkg_name: spare disk $disk removed from spare list");
        }
    } ## end foreach my $disk (@disks_to_be_removed)
    return $result;
} ## end sub RaidSqa_disk_remove

#------------------------------------------------------------------------------

=head2 RaidSqa_disk_mung

RaidSqa_disk_mung(api_set_obj => $api_set_obj, conntype=>$conntype, disk_name=>$disk_name, sector=>$sector, bits=>$bits, node => $node);

This subroutine mung a sector(block) of disks. Need a non-promoted debug build.

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required): Name of the disk.

=item sector

(Required): sector number of disk.

=item bits

(Optional): Number of consecutive bits to mung.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_disk_mung {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        sector    => {type => SCALAR},
                        bits      => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $sector      = $opts{sector};
    my $bits        = $opts{bits};
    my $node        = $opts{node};

    my $output;
    my $disk_active_path_name = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                                                   conntype    => $conntype,
                                                                   disk_name   => $disk_name,
                                                                   node        => $node);
    if ($disk_active_path_name) {
        $Log->comment("$pkg_name: Using path name $disk_active_path_name for any operations on $disk_name");
        $disk_name = $disk_active_path_name;
    } else {
        $Log->comment(
             "$pkg_name: The disk_name $disk_name specified does not exist. Please check again");
        return (0);
    }

    try {
        my $response =
          $api_set_obj->system_node_run(
                              node    => $node,
                              command => "\"priv set -q test; disk_mung $disk_name $sector $bits \""
          );
        $output = $response->get_raw_output();
        if ($output =~ /scsi_write_long succeeded on block $sector/) {
            return 1;
        } else {
            return 0;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };

} ## end sub RaidSqa_disk_mung

#--------------------------------------------------------------------------------

=head2 RaidSqa_disk_zero_spares

RaidSqa_disk_zero_spares(api_set_obj=>$api_set_obj, conntype=>$conntype, node => $node);

This subroutine start zeroing on all notzeroed spares.

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_disk_zero_spares {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $return      = 1;

    $Log->comment("$pkg_name: starting zeroing in spare disk");
    try {
        my $response = $api_set_obj->storage_disk_zerospares(owner => $node);
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $output           = _handle_nacl_exception($exception_object);
        $return = 0;
    };
    return $return;
} ## end sub RaidSqa_disk_zero_spares

#------------------------------------------------------------------------------

=head2 RaidSqa_disk_assign

RaidSqa_disk_assign(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name=>$disk_name, disk_list=>\@disk_list, pool=>$pool,node=>$node);

This subroutine assign one or more disk to filer in defined pool.

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Optional): name of disk. (but any one of disk_list or disk_name is required)

=item disk_list

(Optional): refrence to the list of disk (but any one of disk_list or disk_name is required)

=item pool

(Required): pool 0 or 1 .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_disk_assign {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype  => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR | UNDEF,   default => undef,   optional => 1},
                        disk_list => {type => ARRAYREF | UNDEF, default => undef,   optional => 1},
                        pool      => {type => SCALAR,           default => 0,       optional => 1},
                        node      => {type => SCALAR,           default => "local", optional => 1},
                        timeout   => {type => SCALAR,   default => $RaidSqa_TIMEOUT, optional => 1},
    });
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $timeout     = $opts{timeout};
    my $disk_name   = $opts{disk_name};
    my $disk_list   = $opts{disk_list};
    my @disk_list;
    my $pool   = $opts{pool};
    my $node   = $opts{node};
    my $result = 1;

    if (!defined($disk_name) && !defined($disk_list)) {
        $Log->comment("$pkg_name: None of the parameter disk_list or disk_name defined, need to define one of them");
        return 0;
    } elsif (defined($disk_name) && defined($disk_list)) {
        $Log->comment("$pkg_name: Both of the parameter disk_list or disk_name defined, need to define only one of them");
        return 0;
    }

    my $output;
    if (defined($disk_name)) {
        @disk_list = ($disk_name);
    } elsif (defined($disk_list)) {
        @disk_list = @$disk_list;
    }

    try {
        my $response = $api_set_obj->system_node_run (
            node    => $node,
            command => "\"priv set -q diag; disk assign -f -p $pool @disk_list \" ",
            'connectrec-timeout' => $timeout
        );
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };
    return $result;
} ## end sub RaidSqa_disk_assign

#------------------------------------------------------------------------------

=head2 RaidSqa_disk_remove_ownership

RaidSqa_disk_remove_ownership(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name=>$disk_name, disk_list=>\@disk_list, node=>$node);

This subroutine remove_ownership of one or more disk from filer .

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Optional): name of disk. (but any one of disk_list or disk_name is required)

=item disk_list

(Optional): refrence to the list of disk (but any one of disk_list or disk_name is required)

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_disk_remove_ownership {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_list => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        disk_name => {type => SCALAR | UNDEF,   default => undef, optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $disk_list   = $opts{disk_list};
    my $node        = $opts{node};
    my $filer       = $api_set_obj->hostobj->hostname();

    my ($result, $output, @disk_list, $disk_string);

    if (!defined($disk_name) && !defined($disk_list)) {
        $Log->comment(
            "$pkg_name: None of the parameter $disk_list or $disk_name defined, need to define one of them"
        );
        return 0;
    } elsif (defined($disk_name) && defined($disk_list)) {
        $Log->comment(
            "$pkg_name: Both of the parameter $disk_list or $disk_name defined, need to define only one of them"
        );
        return 0;
    }

    if (defined($disk_name)) {
        @disk_list = ($disk_name);
    } elsif (defined($disk_list)) {
        @disk_list = @$disk_list;
    }

    $result = 1;
    $disk_list[0] = "$filer:".$disk_list[0];
    $disk_string = join(",$filer:", @disk_list);
    try {
        my $response =
          $api_set_obj->storage_disk_removeowner(disk => $disk_string, force => "true");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        $result = 0;
    };
    return $result;
} ## end sub RaidSqa_disk_remove_ownership

#------------------------------------------------------------------------------

=head2 RaidSqa_disk_replace_start

RaidSqa_disk_replace_start(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, replacing_disk => $replacing_disk, options=>$options, node=>$node);

This subroutine replace the disk with specified disk.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to be replaced.

=item replacing_disk

(Required) : Name of disk to replace .

=item options

(Optional) : The option -m allows mixing disks with different characteristics.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_disk_replace_start {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name      => {type => SCALAR},
                        replacing_disk => {type => SCALAR},
                        options        => {type => SCALAR | UNDEF, default => undef, optional => 1},
                       });

    my $api_set_obj    = $opts{api_set_obj};
    my $node           = $opts{node};
    my $conntype       = $opts{conntype};
    my $disk_name      = $opts{disk_name};
    my $replacing_disk = $opts{replacing_disk};
    my $options        = $opts{options};

    my ($host, $conn, $response, $output, $result);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }
    $result = 1;

    my @disk_list = ($disk_name, $replacing_disk);
    @disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disk_list);
    ($disk_name, $replacing_disk) = @disk_list;
    #Check if disk name exists or has an equivalent path_name2.

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $Log->comment("$pkg_name: Disk with name $disk_name could not be found");
    }

    # checking that specified disk is a file system disk before replacing it
    if ($disk_obj->parameter("fsm_state") eq "INUSE") {
        try {
            $response =
              $api_set_obj->system_node_run(
                          node    => $node,
                          command => "\"disk replace start -f $options $disk_name $replacing_disk\""
              );
            $output = $response->get_raw_output();
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            $result = 0;
        };
    } else {
        $Log->comment("$pkg_name: disk $disk_name is not a file system disk, so can not replace");
        return 0;
    }

    return (0) if (!$result);

    #Check if disk name exists after replace or has an equivalent path_name2.
    $raid_obj->refresh();

    $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    # check disk got replaced successfully
    if ($disk_obj->parameter("replace") eq "1") {
        $Log->comment("$pkg_name: disk $disk_name got replaced successfully");
        return 1;
    } else {
        $Log->comment("$pkg_name: Disk $disk_name could not be replaced with disk $replacing_disk");
        return 0;
    }
} ## end sub RaidSqa_disk_replace_start

#------------------------------------------------------------------------------

=head2 RaidSqa_diskbypass

RaidSqa_diskbypass(api_set_obj => $api_set_obj, conntype => $conntype, error=>$error, disk_id=>$disk_id, portAorB=>$portAorB,node => $node);

This Subroutine to bypass/unbypass a disk. Bypasses/unbypasses an ESH drive port.

It returns a 1 on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item error

(Required) :  Error type to inject for bypassing drives.Following types of errors can be injected:
               "0x02" => "FORCE BYPASS".
               "0xa0" => "Clock Delta Fault Bypass".
               "0xa1" => "CRC Fault Bypass".
               "0xa2" => "WRD Fault Bypass".
               "0xa3" => "Increment CRC error +1".
               "0xa4" => "Increment CRC error +2".
               "0xa5" => "Increment WRD error +1000".
=item disk_id

(Required) : disk id to be bypassed.

=item portAorB

(Optional) : To bypass port A or port B of the drive. "A" or "B", if not defined that will bypass/unbypass both ports

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_diskbypass {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        error    => {type => SCALAR},
                        disk_id  => {type => SCALAR},
                        portAorB => {type => SCALAR, optional => 1},
                        node     => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj    = $opts{api_set_obj};
    my $conntype       = $opts{conntype};
    my $node           = $opts{node};
    my $service        = $opts{service};
    my $error          = $opts{error};
    my $diskidraw_list = $opts{disk_id};
    my $portAorB       = $opts{portAorB};

    my ($output, $line, $prompt, $a, $b, $i, $diskid, $adapid, $shelfid, $adshelf);
    my (@info, @port, @diskid, @channel1, @shelf1, @shelves, @shelf, @wwn, @wwn2, @wwn1, @wwnmap);
    my (%wwnmap, %portid, %wwnmap1);
    my $result = 1;
    my $n      = 0;

    try {
        my $response =
          $api_set_obj->system_node_run(node => $node, command => "\"storage show hub\" ");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    my @hub = split(/\r?\n/, $output);

    #### First create an array of adapters connected to hub

    my @channels = grep /Channel/, @hub;
    while ($n <= $#channels) {
        my @hub1 = split(/\s+/, $channels[$n]);
        $channel1[$n] = $hub1[1];
        $n++;
    }

    #### Create an array of  shelf ids of disks
    $n = 0;

    @shelves = grep /Shelf id:/, @hub;
    while ($n <= $#shelves) {
        @shelf = split(/\s+/, $shelves[$n]);
        $shelf1[$n] = $shelf[2];
        $n++;
    }

    #### Create an array of wwn nos.

    $n = 0;

    @wwn = grep /UID/, @hub;
    while ($n <= $#wwn) {
        @wwn2 = split(/\s+/, $wwn[$n]);
        $wwn1[$n] = $wwn2[2];
        $n++;
    }

    #### Create a hash table of adapters , shelves and wwns

    $i = 0;

    while ($i <= $#channel1) {
        $adshelf = $channel1[$i] . "." . $shelf1[$i];
        %wwnmap  = ($adshelf, $wwn1[$i]);
        %wwnmap1 = (%wwnmap1, %wwnmap);
        $i++;
    }
    foreach my $diskidraw (split " ", $diskidraw_list) {
        @diskid = split(/\./, $diskidraw);
        $diskid = $diskid[1];
        $adapid = $diskid[0];
        if ($diskid <= 13) {
            $shelfid = 0;
        } elsif ($diskid >= 16 && $diskid <= 29) {
            $shelfid = 1;
        } elsif ($diskid >= 32 && $diskid <= 45) {
            $shelfid = 2;
        } elsif ($diskid >= 48 && $diskid <= 61) {
            $shelfid = 3;
        } elsif ($diskid >= 64 && $diskid <= 77) {
            $shelfid = 4;
        } elsif ($diskid >= 80 && $diskid <= 93) {
            $shelfid = 5;
        } elsif ($diskid >= 96 && $diskid <= 109) {
            $shelfid = 6;
        } else {
            $shelfid = 7;
        }

        my $diskkey = "$adapid" . "." . "$shelfid";

        ### Check for ESH or ESH2

        try {
            my $response =
              $api_set_obj->system_node_run(node => $node, command => "\"environ shelf $adapid\" ");
            $output = $response->get_raw_output();
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            $result = 0;
        };

        my @eshtype = split(/\r?\n/, $output);
        my $tt = "no";
        my $eshtype1;
        foreach my $eshtype (@eshtype) {
            if ($eshtype =~ /Shelf:\s+$shelfid/) {
                $tt = "yes";
            }
            if (($eshtype =~ /Module type/) && ($tt eq "yes")) {
                $eshtype1 = "$eshtype";
                goto LABEL;
            }
        }

      LABEL: my $diskwwn = 0;

        $diskwwn = $wwnmap1{$diskkey};

        my $j;

        for ($i = 0 ; $i <= 13 ; $i++) {
            $j = $i;
            while ($j <= 112 + $i) {
                if ($eshtype1 =~ /ESH;/) {
                    $portid{$j} = $i + 2;
                }

                if ($eshtype1 =~ /ESH2/) {
                    $portid{$j} = $i + 4;
                }

                #MTK_EDIT: Added the below code for ESH4 model type shelfs
                if ($eshtype1 =~ /ESH4/) {
                    $portid{$j} = $i + 4;
                }
                $j = $j + 16;
            } ## end while ($j <= 112 + $i)
        } ## end for ($i = 0 ; $i <= 13 ...
        my $port = $portid{$diskid};

        if ($diskwwn eq '') {
            logresult("FATAL", "$diskidraw is not a hub disk");
        } else {
            if ($portAorB eq "") {
                @port = ('A', 'B');
            } else {
                @port = ($portAorB);
            }

            #print "WWN for disk: $diskidraw is $diskwwn and it's port no is
            if ($error eq "0x02") {
                $prompt = "fcadmin: hub request in progress to shelf UID: $diskwwn";
            } else {
                $prompt = "\n";
            }
            foreach my $port_to_use (@port) {
                my $cmd_to_execute =
                  "fcadmin hub pctrl -w $diskwwn -p $port -v $error -l $port_to_use";
#                RaidSqa_filer_cmd(api_set_obj => $api_set_obj,
#                                  node        => $node,
#                                  cmd         => $cmd_to_execute,
#                                  prompt      => $prompt);
                $api_set_obj->system_node_run(node    => $node,
                                              command => $cmd_to_execute);
                sleep 30;
            }
            sleep 30;
        } ## end else [ if ($diskwwn eq '')
    } ## end foreach my $diskidraw (split...
    return (1);
} ## end sub RaidSqa_diskbypass

#------------------------------------------------------------------------------

=head2 RaidSqa_find_active_path_disk_name

RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, conntype=>$conntype, disk_name =>$disk_name, node => $node);

This subroutine finds the active disk name of a given disk.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Optional): name of disk. (disk name is discarded when disk_list is passed)

=item disk_list

(Optional): refrence to the list of disk (only disk_list is comsidered when disk_list and disk_name are passed)

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_find_active_path_disk_name {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR | UNDEF,   default => undef,   optional => 1},
                        disk_list => {type => ARRAYREF | UNDEF, default => undef,   optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $disk_list   = $opts{disk_list};
    my @disk_list;
    my $node        = $opts{node};

    if (!defined($disk_name) && !defined($disk_list)) {
        die(
            "$pkg_name: None of the parameter disk_list or disk_name defined, need to define one of them"
        );
    } elsif (defined($disk_name) && defined($disk_list)) {
        $Log->comment(
            "$pkg_name: Both of the parameter disk_list or disk_name defined, considering disk_list alone"
        );
    }

    if (defined($disk_list)) {
        @disk_list = @$disk_list;
    } else {
        @disk_list = ($disk_name);
    }

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my %disks_hash = $raid_obj->all_disks();
    my $a = 0;
    my $disk;
    my @disk_active_path;
    foreach $disk (@disk_list) {
        if (exists $disks_hash{$disk}) {
            $disk_active_path[$a] = $disk;
            $a++;
        } else {
            foreach my $key (keys %disks_hash) {
                if ($disk eq $disks_hash{$key}{"path_name2"}) {
                    $disk_active_path[$a] = $disks_hash{$key}{"name"};
                    $a++;
                }
            }
        } ## end else [ if (exists $disks_hash...
    }
    if (defined($disk_name)) {
        return $disk_active_path[0];
    } elsif (defined($disk_list)) {
        return @disk_active_path;
    }

} ## end sub RaidSqa_find_active_path_disk_name

#------------------------------------------------------------------------------

=head2 RaidSqa_check_license

RaidSqa_check_license(api_set_obj=>$api_set_obj, conntype=>$conntype, service=>$service);

This subroutine check that filer is having license for a particular service.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item service

(Required): Name of the service for which we want to check license.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_license {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        service => {type => SCALAR},
                        node    => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $service     = $opts{service};
    my $node        = $opts{node};
    my ($response, $output, $result);

    $Log->comment("$pkg_name: Inputs: node - $node, service - $service");
    #-----------------------------------------------------------------------------------------------
    my @default_available_license = qw(syncmirror_remote syncmirror_local cf);
    if(grep /^$service$/i, @default_available_license) {
        $Log->comment("$pkg_name: License is no longer required - $service");
        return 1;
    }

    #-----------------------------------------------------------------------------------------------
    my @possible_licenses = keys %DB_LICENSE;
    if(!defined($DB_LICENSE{lc $service})) {
        $Log->comment("$pkg_name: Name of service is not valid - $service");
        return 0;
    }

    $result = 1;
    try {
        #-------------------------------------------------------------------------------------------
        my %cmd_args = (package => $service);
        $response = $api_set_obj->license_show(%cmd_args);
        $output = $response->get_parsed_output();
        $Log->comment("Dumper: License: ".Dumper($output));
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        $result = 0;
    };

    return $result if($result == 0);

    #-----------------------------------------------------------------------------------------------
    # #There will/should be only one entry when -owner is mentioned along.
    my $serial_number = undef;
    foreach my $entry (@$output) {
        if($entry->{package} =~ /$service/i) {
            $serial_number = $entry->{'serial-number'};
            last;
        }
    }

    #-----------------------------------------------------------------------------------------------
    if(defined($serial_number)) {
        $Log->comment("$pkg_name: License '$service' is present, serial - $serial_number");
        return 1;
    } else {
        $Log->comment("$pkg_name: License is not present - $service");
        return 0;
    }
} ## end sub RaidSqa_check_license
#------------------------------------------------------------------------------
=head2 RaidSqa_get_license_serial_number

RaidSqa_get_license_serial_number(api_set_obj=>$api_set_obj, conntype=>$conntype, service=>$service);

Returns serial-number for given license [Only for Cmode]

It returns numeric serial number

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item service

(Required): Name of the service for which we want to get serial number for.

=back

=cut

#------------------------------------------------------------------------------
sub RaidSqa_get_license_serial_number {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        service => {type => SCALAR},
                        node    => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $service     = $opts{service};
    my $node        = $opts{node};
    my ($response, $output, $result);

    $Log->comment("$pkg_name: Inputs: node - $node, service - $service");
    #-----------------------------------------------------------------------------------------------
    my @default_available_license = qw(syncmirror_remote syncmirror_local cf);
    if(grep /^$service$/i, @default_available_license) {
        $Log->comment("$pkg_name: License is no longer required - $service");
        return undef;
    }

    #-----------------------------------------------------------------------------------------------
    my @possible_licenses = keys %DB_LICENSE;
    if(!defined($DB_LICENSE{lc $service})) {
        $Log->comment("$pkg_name: Name of service is not valid - $service");
        return undef;
    }

    $result = 1;
    try {
        #-----------------------------------------------------------------------------------------------
        my %cmd_args = (package => $service);

        $response = $api_set_obj->license_show(%cmd_args);
        $output = $response->get_parsed_output();

        $Log->comment("Dumper: License: ".Dumper($output));
    } catch NACL::APISet::Exceptions::ResponseException with {
        die("Exception: License show");     #TODO
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        $result = 0;
    };

    return undef if($result == 0);

    #-----------------------------------------------------------------------------------------------
    # #There will/should be only one entry when -owner is mentioned along.
    my $serial_number = undef;
    foreach my $entry (@$output) {
        if($entry->{package} =~ /$service/i) {
            $serial_number = $entry->{'serial-number'};
            last;
        }
    }

    #-----------------------------------------------------------------------------------------------
    if(defined($serial_number)) {
        $Log->comment("$pkg_name: License '$service' is present, serial - $serial_number");
        return $serial_number;
    } else {
        $Log->comment("$pkg_name: License is not present - $service");
        return undef;
    }
}


#------------------------------------------------------------------------------

=head2 RaidSqa_license_add_by_code

RaidSqa_license_add_by_code(api_set_obj=>$api_set_obj, conntype=>$conntype, code=>$code, node => $node);

This subroutine add license using a particular code. It does not check the platform or any thing.

return 1 on success.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item code

(Required): Particular code for license to add.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_license_add_by_code {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        code => {type => SCALAR},
                        node => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $code        = $opts{code};

    my $output;
    my $result = 1;

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag ;license add $code\"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };
    return $result;
} ## end sub RaidSqa_license_add_by_code

#------------------------------------------------------------------------------

=head2 RaidSqa_license_add_by_service

RaidSqa_license_add_by_service(api_set_obj=>$api_set_obj, conntype=>$conntype, service=>$service,node => $node);

This subroutine add license for a particular service. Need a non-promoted debug build as 1st get code for that particular service for particular platform and than add that code.

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item service

(Required): Name of the service for which we want to add license.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_license_add_by_service {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        service => {type => SCALAR},
                        node    => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $service     = $opts{service};

    #-----------------------------------------------------------------------------------------------
    my @default_available_license = qw(syncmirror_remote syncmirror_local cf);
    if(grep /$service/i, @default_available_license) {
        $Log->comment("$pkg_name: License is no longer required - $service");
        return 1;
    }

    #-----------------------------------------------------------------------------------------------
    my @possible_licenses = keys %DB_LICENSE;
    if(!defined($DB_LICENSE{lc $service})) {
        $Log->comment("$pkg_name: Name of service is not valid - $service");
        return 0;
    }

    my $license_code = $DB_LICENSE{lc $service};

    try {
        $api_set_obj->system_license_add('license-code' => $license_code);
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
    };

    if (!RaidSqa_check_license(api_set_obj => $api_set_obj,
                              conntype    => $conntype,
                              service     => $service,
                              node        => $node)) {
        $Log->comment("$pkg_name: License add failed - $service");
        return 0;
    }
    return 1;
} ## end sub RaidSqa_license_add_by_service

#------------------------------------------------------------------------------

=head2 RaidSqa_license_delete

RaidSqa_license_delete(api_set_obj=>$api_set_obj, conntype=>$conntype, service=>$service, node => $node);

This subroutine delete license for a particular service.

Return 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item service

(Required): Name of the service for which we want to delete license.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_license_delete {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        service => {type => SCALAR},
                        node    => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $service     = $opts{service};

    my ($response, $output, $result);

    #-----------------------------------------------------------------------------------------------
    my @default_available_license = qw(syncmirror_remote syncmirror_local cf);
    if(grep /$service/i, @default_available_license) {
        $Log->comment("$pkg_name: License is no longer present - $service");
        return 1;
    }

    #-----------------------------------------------------------------------------------------------
    my @possible_licenses = keys %DB_LICENSE;
    if(!defined($DB_LICENSE{lc $service})) {
        $Log->comment("$pkg_name: Name of service is not valid - $service");
        return 0;
    }

    $result = 1;
    my $serial_number = RaidSqa_get_license_serial_number (
        api_set_obj => $api_set_obj,
        service     => $service,
    );

    if(!defined($serial_number)) {
        $Log->comment("$pkg_name: Serial number could not be retrieved, changing it to * - $service");
        $serial_number = '*';
    }

    try {
        my $response = $api_set_obj->system_license_delete(
            package => $service,
            'serial-number' => $serial_number
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };


    if (!$result or RaidSqa_check_license(api_set_obj => $api_set_obj,
                              conntype    => $conntype,
                              service     => $service,
                              node        => $node,)) {
        $Log->comment("$pkg_name: License delete failed - $service");
        $result = 0;
    }
    return $result;
} ## end sub RaidSqa_license_delete

#------------------------------------------------------------------------------

=head2 RaidSqa_fiji_show

RaidSqa_fiji_show(api_set_obj=>$api_set_obj, conntype=>$conntype,node => $node);

This subroutine gets o/p of fiji show command and show all fiji filters/nvfilter set on the filer.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_fiji_show {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $output;
    my $result = 1;

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; fiji show\" ");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };
    my @fiji_show = split(/\r?\n/, $output);
    return @fiji_show;
} ## end sub RaidSqa_fiji_show

#------------------------------------------------------------------------------

=head2 RaidSqa_check_fiji

RaidSqa_check_fiji(api_set_obj => $api_set_obj, conntype => $conntype, fiji_point=>$fiji_point, node => $node);

This subroutine check that filer is having fiji set.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item fiji_point

(Required): Name of the fiji_point for which we want to check fiji.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_fiji {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        fiji_point => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $fiji_point  = $opts{fiji_point};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $output;
    my $line;

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; fiji show \"");
        $output = $response->get_raw_output();
        my @info = split(/\r?\n/, $output);
        if (index($output, $fiji_point) ne "-1") {
            $Log->comment("$pkg_name: fiji filter/nvfilter $fiji_point is set");
            return 1;
        } else {
            $Log->comment("$pkg_name: fiji filter/nvfilter $fiji_point is not set");
            return 0;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };
} ## end sub RaidSqa_check_fiji

#------------------------------------------------------------------------------

=head2 RaidSqa_get_fiji_slot

RaidSqa_get_fiji_slot(api_set_obj => $api_set_obj, conntype=>$conntype, fiji_point=>$fiji_point, node => $node);

This subroutine get the fiji slot numbers for a particular fiji point.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item fiji_point

(Required) : Name of fiji_point .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_fiji_slot {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        fiji_point => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $fiji_point  = $opts{fiji_point};
    my $conntype    = $opts{conntype};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $output;
    my $line;

    my $n = 0;
    my @fiji_slot;

    #
    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; fiji show \"");
        $output = $response->get_raw_output();
        my @info = split(/\r?\n/, $output);
        foreach $line (@info) {
            my (@data) = split(": ", $line);
            if (index($line, "$fiji_point") ne "-1") {
                if (index($data[0], " (nvram)") ne "-1") {
                    substr($data[0], index($data[0], " (nvram)")) = "";
                }
                $fiji_slot[$n] = $data[0];
                $n++;
            }
        } ## end foreach $line (@info)
        return @fiji_slot;
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
    };
} ## end sub RaidSqa_get_fiji_slot

#------------------------------------------------------------------------------

=head2 RaidSqa_fiji_clear

RaidSqa_fiji_clear(api_set_obj => $api_set_obj, conntype=>$conntype, fiji_point=>$fiji_point, node => $node);

This subroutine clear a defined fiji filter from the filer. Use all  as fiji point for clearing all fiji filter

Return 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item fiji_point

(Required): Name of the fiji_point which we want to clear from the filer.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_fiji_clear {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        fiji_point => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $fiji_point  = $opts{fiji_point};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $output;
    my $slot;

    try {
        if ($fiji_point eq "all") {
            my $response =
              $api_set_obj->system_node_run(node    => $node,
                                            command => "\"priv set -q diag; fiji clear all \"");
            $output = $response->get_raw_output();
        } else {
            my @fiji_slot = RaidSqa_get_fiji_slot(api_set_obj => $api_set_obj,
                                                  fiji_point  => $fiji_point,
                                                  node        => $node);
            foreach $slot (@fiji_slot) {
                my $response =
                  $api_set_obj->system_node_run(node    => $node,
                                                command => "\"priv set -q diag; fiji clear $slot \""
                                               );
                $output = $response->get_raw_output();
            }
        } ## end else [ if ($fiji_point eq "all")
        if (RaidSqa_check_fiji(api_set_obj => $api_set_obj,
                               fiji_point  => $fiji_point,
                               node        => $node)
          ) {
            return 0;
        } else {
            return 1;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };
} ## end sub RaidSqa_fiji_clear

#------------------------------------------------------------------------------

=head2 RaidSqa_fiji_filter

RaidSqa_fiji_filter(api_set_obj => $api_set_obj, conntype=>$conntype, fiji_point=>$fiji_point, node => $node);

This subroutine add a defined fiji filter to the filer.

Return 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item fiji_point

(Required): Name of the fiji_point which we want to add to the filer.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_fiji_filter {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        fiji_point => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $fiji_point  = $opts{fiji_point};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    try {
        my $response =
          $api_set_obj->system_node_run(
                                       node    => $node,
                                       command => "\"priv set -q diag; fiji filter '$fiji_point' \""
          );
        my $output = $response->get_raw_output();
        if (RaidSqa_check_fiji(api_set_obj => $api_set_obj,
                               fiji_point  => $fiji_point,
                               node        => $node)
          ) {
            return 1;
        } else {
            return 0;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };

} ## end sub RaidSqa_fiji_filter

#------------------------------------------------------------------------------

=head2 RaidSqa_fiji_nvfilter

RaidSqa_fiji_nvfilter(api_set_obj=>$api_set_obj, conntype=>$conntype, fiji_point=>$fiji_point, node =>$node);

This subroutine add a defined fiji nvfilter to the filer.

Return 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item fiji_point

(Required): Name of the fiji_point which we want to add to the filer.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_fiji_nvfilter {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        fiji_point => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $fiji_point  = $opts{fiji_point};
    my $node        = $opts{node};
    my $result      = 1;
    my $output;
    try {
        my $response =
          $api_set_obj->system_node_run(
                                       node    => $node,
                                       command => "\"priv set -q diag; fiji nvfilter $fiji_point\" "
          );
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    if (!RaidSqa_check_fiji(api_set_obj => $api_set_obj,
                            conntype    => $conntype,
                            fiji_point  => $fiji_point,
                            node        => $node)
      ) {
        $result = 0;
    }
    return $result;
} ## end sub RaidSqa_fiji_nvfilter

#------------------------------------------------------------------------------

=head2 RaidSqa_fiji_nvclear

RaidSqa_fiji_nvclear(api_set_obj => $api_set_obj,conntype => $conntype, fiji_point=>$fiji_point, node=>$node);

This subroutine nvclear a defined fiji nvfilter from the filer. Give fiji_point as all if want to delete all fiji filters.

Return 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item fiji_point

(Required): Name of the fiji_point which we want to nvclear from the filer.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_fiji_nvclear {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        fiji_point => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $fiji_point  = $opts{fiji_point};
    my $node        = $opts{node};

    my $output;
    my (@fiji_slot, $slot);
    my $result = 1;

    if ($fiji_point =~ /all/i) {
        @fiji_slot = ('all');
    } else {
        @fiji_slot = RaidSqa_get_fiji_slot(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           fiji_point  => $fiji_point,
                                           node        => $node);
    }
    foreach $slot (@fiji_slot) {
        try {
            my $response =
              $api_set_obj->system_node_run(node    => $node,
                                            command => "\"priv set -q diag; fiji nvclear $slot\" ");
            $output = $response->get_raw_output();
            if ($output =~ /filter cleared/i) {
                $Log->comment("$pkg_name : fiji: filter cleared (nvram only) from slot $slot");
            } else {
                $Log->comment("$pkg_name : fiji: filter not cleared (nvram only) from slot $slot");
                $result = 0;
            }
        } ## end try
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            $result = 0;
        };
    } ## end foreach $slot (@fiji_slot)
    return $result;
} ## end sub RaidSqa_fiji_nvclear

#------------------------------------------------------------------------------

=head2 RaidSqa_hammer_start

RaidSqa_hammer_start(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, iterations=>$iterations, file_name=>$file_name, file_size=>$file_size, node => $node);

This subroutine start one hammer one a specified flex or trad volume with specified parameter on the filer.

return 1 if hammer started as needed and 0 if not.

The following arguments are accepted:

=over

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required): Name of the test flex or trad volume.

=item iterations

(Optional): Number of iterations to run hammer. Default is "-1" which means always.

=item file_name

(Optional): Name of file to be created by hammer. Default is "HAMMER_9001"

=item file_size

(Optional): size of file to be created by hammer. Default is "-1" which means max capacity of flex or trad volume.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_hammer_start {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR},
                        iterations => {type => SCALAR, default => "-1", optional => 1},
                        file_name  => {type => SCALAR, default => "HAMMER_001", optional => 1},
                        file_size  => {type => SCALAR, default => "-1", optional => 1},
                        node       => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $iterations  = $opts{iterations};
    my $file_name   = $opts{file_name};
    my $file_size   = $opts{file_size};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if (!defined($node)) {
        $node = "local";
    }

    # checking every 5 sec for a max time up to 5 minutes for hammer to start
    my ($hammer_status) = "foo";
    my ($wait_interval) = 5;
    my ($time_waited)   = 0;

    #TODO: Change to 300 after fix
    my ($max_wait) = 30;    # wait 2 minutes for hammer to abort

    my $file_name1 = $file_name . $$;
    my $result     = 1;

    try {
        my $response =
          $api_set_obj->system_node_run(
             node => $node,
             command =>
               "\"priv set -q diag; hammer -f $iterations /vol/$filer_vol/$file_name1 $file_size \""
          );
        my $o = $response->get_raw_output();

        while ($hammer_status !~ "/vol/$filer_vol/$file_name1" && $time_waited < $max_wait) {
            my $response =
              $api_set_obj->system_node_run(node    => $node,
                                            command => "\"priv set -q diag; hammer status \"");
            $hammer_status = $response->get_raw_output();
            $time_waited += $wait_interval;
            sleep $wait_interval;
        }
        if ($hammer_status =~ "/vol/$filer_vol/$file_name1") {
            $Log->comment("$pkg_name: hammer start on $filer_vol with filename $file_name1");
        } else {
            $Log->comment(
                 "$pkg_name: waited $time_waited seconds for hammer to start, but it could not start"
            );
            $result = 0;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        $result = 0;
    };
    return $result;
} ## end sub RaidSqa_hammer_start

#------------------------------------------------------------------------------

=head2 RaidSqa_hammer_pause

RaidSqa_hammer_pause(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node);

This subroutine pause all hammers on the filer.

return 1 if hammer pause and 0 if not.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_hammer_pause {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};

    # checking every 5 sec for a max time up to 5 minutes for hammer pause
    my ($hammer_status) = "foo";
    my ($wait_interval) = 5;
    my ($time_waited)   = 0;

    #TODO
    #my ($max_wait)      = 300;     # wait 5 minutes for hammer to abort
    my ($max_wait) = 10;    # wait 5 minutes for hammer to abort

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; hammer pause\"");
        my $output = $response->get_raw_output();
        sleep(5);
        $Log->comment("$pkg_name: output is $output");
        while ($hammer_status ne "" && $time_waited < $max_wait) {
            my $response =
              $api_set_obj->system_node_run(node    => $node,
                                            command => "\"priv set -q diag; hammer status\"");
            $hammer_status = $response->get_raw_output();
            $time_waited += $wait_interval;
            sleep $wait_interval;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
    };

    if ($hammer_status eq "") {
        $Log->comment("$pkg_name: Hammer paused");
        return 1;
    } else {
        $Log->comment(
               "$pkg_name: waited $time_waited seconds for hammer to pause, but it failed to pause");
        return 0;
    }
} ## end sub RaidSqa_hammer_pause

#------------------------------------------------------------------------------

=head2 RaidSqa_hammer_restart

RaidSqa_hammer_restart(host=>$host, conntype=>$conntype, node=>$node);

This subroutine restart all hammers on the filer.

return 1 if any hammer restart, 0 if no hammer restart.

The following arguments are accepted:

=over

=item host

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_hammer_restart {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};

    # checking every 5 sec for a max time up to 5 minutes for hammer  to restart
    my ($hammer_status) = " ";
    my ($wait_interval) = 5;
    my ($time_waited)   = 0;

    #TODO: change to 300 after fix
    my ($max_wait) = 10;    # wait 5 minutes for hammer to abort

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; hammer restart\"");
        my $output = $response->get_raw_output();
        sleep(5);
        $Log->comment("$pkg_name: output is $output");

        while ($hammer_status ne "" && $time_waited < $max_wait) {
            my $response =
              $api_set_obj->system_node_run(node    => $node,
                                            command => "\"priv set -q diag; hammer status\"");
            my $output = $response->get_raw_output();
            $hammer_status = $response->get_raw_output();

            #TODO DEBUG::
            $Log->comment("$pkg_name: Hammer status is :: $hammer_status");
            $time_waited += $wait_interval;
            sleep $wait_interval;
        } ## end while ($hammer_status ne ...
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
    };

    if ($hammer_status eq "") {
        $Log->comment(
            "$pkg_name: waited $time_waited seconds for hammer to restart, but it failed or there is no paused hammer to restart"
        );
        return 0;
    } else {
        $Log->comment("$pkg_name: Hammer restared ");
        return 1;
    }
} ## end sub RaidSqa_hammer_restart

#------------------------------------------------------------------------------

=head2 RaidSqa_hammer_abort

RaidSqa_hammer_abort(api_set_obj => $api_set_obj, conntype=>$conntype, node => $node);

This subroutine abort all hammers on the filer.

return 1 if hammer abort and 0 if not.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_hammer_abort {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # checking every 5 sec for a max time up to 5 minutes for hammer abort
    my ($hammer_status) = "foo";
    my ($wait_interval) = 5;
    my ($time_waited)   = 0;

    #TODO : Change to 300 after fix
    my ($max_wait) = 10;    # wait 5 minutes for hammer to abort

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; hammer abort\"");
        $hammer_status = $response->get_raw_output();

        while ($hammer_status ne "" && $time_waited < $max_wait) {
            my $response =
              $api_set_obj->system_node_run(node    => $node,
                                            command => "\"priv set -q diag; hammer status \"");
            $hammer_status = $response->get_raw_output();
            $time_waited += $wait_interval;
            sleep $wait_interval;
        }
        if ($hammer_status eq "") {
            $Log->comment("$pkg_name: Hammer aborted");
            return 1;
        } else {
            $Log->comment(
                 "$pkg_name: waited $time_waited seconds for hammer to abort, but it failed to abort"
            );
            return 0;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };
} ## end sub RaidSqa_hammer_abort

#------------------------------------------------------------------------------

=head2 RaidSqa_hammer_status

RaidSqa_hammer_status(api_set_obj=>$api_set_obj, conntype=>$conntype, node => $node);

This subroutine give status of hammers on the filer.

return 1 if hammer is running else 0 if no hammer is running.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item node

(Optional): Node name on which the operation is to be performed

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_hammer_status {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $result      = 1;
    my $output;

    # TODO Need to check for more redundant functions like RaidSqa_hammer_status and RaidSqa_check_hammer_running

    # hammer status
    $Log->comment("$pkg_name: hammer status");
    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; hammer status\" ");
        $output = $response->get_raw_output();
        if ($output =~ /blacksmith/) {
            $Log->comment("$pkg_name: hammer is running");
        } else {
            $Log->comment("$pkg_name: hammer is not running");
            $result = 0;
        }
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };
    return $result;
} ## end sub RaidSqa_hammer_status

#------------------------------------------------------------------------------

=head2 RaidSqa_check_hammer_running

RaidSqa_check_hammer_running(api_set_obj => $api_set_obj, conntype=>$conntype, hammer_number=>$hammer_number, node => $node);

This subroutine check that any or a particular hammer is running.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item hammer_number

(Optional): default is any, but we can check for a particular hammer number

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_hammer_running {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj   => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype      => {type => SCALAR, default => "#ssh", optional => 1},
                        hammer_number => {type => SCALAR, default => "any", optional => 1},
                        node          => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj   = $opts{api_set_obj};
    my $node          = $opts{node};
    my $conntype      = $opts{conntype};
    my $hammer_number = $opts{hammer_number};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my ($check, $output, $result);
    if ($hammer_number eq "any") {
        $check = "blacksmith";
    } else {
        $check = "blacksmith #" . $hammer_number;
    }

    $result = 1;
    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; hammer status \"");
        $output = $response->get_raw_output();
        if ($output =~ /$check/) {
            $Log->comment("$pkg_name: Hammer number: $hammer_number is running");
        } else {
            $Log->comment("$pkg_name: Hammer number: $hammer_number is not running");
            $result = 0;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        $result = 0;
    };
    return $result;
} ## end sub RaidSqa_check_hammer_running

#------------------------------------------------------------------------------

=head2 RaidSqa_get_options

RaidSqa_get_options(api_set_obj=>$api_set_obj, conntype=>$conntype, option_name=>$option_name);

This subroutine get value of an options.

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item option_name

(Required): Name of the option to get value.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_options {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        option_name => {type => SCALAR},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $option_name = $opts{option_name};
    my $node        = $opts{node};

    my $result = 1;
    my $output;
    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; options $option_name \"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    my $opt_value;
    if ($output =~ /$option_name\s+(\w+)/) {
        $opt_value = $1;
    }

    return $opt_value;
} ## end sub RaidSqa_get_options

#------------------------------------------------------------------------------

=head2 RaidSqa_set_options

RaidSqa_set_options(api_set_obj=>$api_set_obj, conntype=>$conntype, option_name=>$option_name, option_value=>$option_value, node => $node);

This subroutine set options on filer using options command.

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item option_name

(Required): Name of the option to change.

=item option_value

(Required): Value of the option to change.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_set_options {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        option_name  => {type => SCALAR},
                        option_value => {type => SCALAR},
                        node         => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj  = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $option_name  = $opts{option_name};
    my $option_value = $opts{option_value};
    my $node         = $opts{node};
    my $result       = 1;

    my $output;
    try {
        my $response =
          $api_set_obj->system_node_run(
                              node    => $node,
                              command => "\"priv set -q diag; options $option_name $option_value \""
          );
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        return 0;
    };

    if ($option_value eq RaidSqa_get_options(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             option_name => $option_name)
      ) {
        $Log->comment("$pkg_name: options $option_name is set to $option_value");
        return 1;
    } else {
        $Log->comment("$pkg_name: options $option_name is not set to $option_value");
        return 0;
    }
} ## end sub RaidSqa_set_options

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_options

RaidSqa_get_aggr_options(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, options=>$options);

This subroutine gets value of an options of a particular volume/aggergate.

It returns scalar value.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item options

(Required) : Name of options to get the value.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_options {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        options   => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $options     = $opts{options};
    my $node        = $opts{node};
    my $result      = 1;
    my $output;

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag;aggr options $filer_vol \"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    my @options_raw = split(/\r?\n/, $output);

    my $options_all;
    my $line;
    my $line1;
    foreach $line (@options_raw) {
        $options_all = $options_all . $line;
    }

    my @options_list = split(/, /, $options_all);
    my %options_hash;
    foreach $line1 (@options_list) {
        (my $key, my $value) = split(/=/, $line1);
        $options_hash{$key} = $value;
    }
    return $options_hash{$options};
} ## end sub RaidSqa_get_aggr_options

#------------------------------------------------------------------------------

=head2 RaidSqa_aggr_options

RaidSqa_aggr_options(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, options=>$options, value=>$value, node=>$node);

This subroutine set various options specified by options parameter to a value specified by value parameter for a given aggregate.

It returns 1 on pass and 0 on fail

The following arguments are accepted:

=over

=item host

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggr.

=item options

(Required) : Name of options or trad set or change.

=item value

(Required) : value of the options.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_aggr_options {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        options   => {type => SCALAR},
                        value     => {type => SCALAR},
                        storage_type  => {type => SCALAR| UNDEF, default => undef, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj      = $opts{api_set_obj};
    my $conntype         = $opts{conntype};
    my $filer_vol        = $opts{filer_vol};
    my $options          = $opts{options};
    my $value            = $opts{value};
    my $node             = $opts{node};
    my $storage_type     = $opts{storage_type};
    my $response         = undef;
    my $output           = undef;

    my $aggr_options_cmd = "aggr options $filer_vol $options $value";
    if (defined($storage_type) && ($options eq "raidtype")) {
        my %aggr_modify_hash = (
        'aggregate'          => $filer_vol,
        'raidtype'           => $value,
        'disktype'           => $storage_type
        );
        try {
            $response = $api_set_obj->storage_aggregate_modify(%aggr_modify_hash);
            $output = $response->get_raw_output();
            $Log->comment("$pkg_name: Aggr options has changed successfully");
        }
    } else {
        try {
            $response = $api_set_obj->system_node_run(
                              node    => $node,
                              command => "\"priv set -q diag; $aggr_options_cmd\""
            );
        }
    }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output           = _handle_nacl_exception($exception_object);
        };
    if (RaidSqa_get_aggr_options(api_set_obj => $api_set_obj,
                                 conntype    => $conntype,
                                 filer_vol   => $filer_vol,
                                 options     => $options,
                                 node        => $node) eq "$value"
      ) {
        return 1;
    } else {
        return 0;
    }
} ## end sub RaidSqa_aggr_options

#------------------------------------------------------------------------------

=head2 RaidSqa_get_setflag

RaidSqa_get_setflag(api_set_obj => $api_set_obj, conntype => $conntype, flag_name => $flag_name, node => $node);

This subroutine get value of a setflag using printflag command. Need a non-promoted debug build.

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item flag_name

(Required): Name of the flag to get value for.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_setflag {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        flag_name => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $flag_name   = $opts{flag_name};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my ($output, $ret_value);
    try {

        # get value of a specified flag using printflag command
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; printflag $flag_name \"");
        my $output = $response->get_raw_output();
        my @output1 = split(/\r?\n/, $output);

        #TODO:: Remove
        $Log->comment("$pkg_name: Test lines are " . join(":::", @output1));
        my ($flag, $value);
        foreach my $line (@output1) {
            if ($line =~ /$flag_name/) {
                ($flag, $value) = split(/ = /, $line);
                last;
            }
        }
        $ret_value = $value;
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output    = _handle_nacl_exception($exception_object);
        $ret_value = "";
    };

    $Log->comment("$pkg_name: Flag value for $flag_name is set to $ret_value");
    return $ret_value;
} ## end sub RaidSqa_get_setflag

#------------------------------------------------------------------------------

=head2 RaidSqa_setflag

RaidSqa_setflag(api_set_obj=>$api_set_obj, conntype=>$conntype, flag_name=>$flag_name, flag_value=>$flag_value);

This subroutine set flags using setflag command. Need a non-promoted debug build.

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item flag_name

(Required): Name of the flag to add.

=item flag_value

(Required): Value of the flag to add.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_setflag {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        flag_name  => {type => SCALAR},
                        flag_value => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $flag_name   = $opts{flag_name};
    my $flag_value  = $opts{flag_value};
    my $node        = $opts{node};

    # Adding specified flag using setflag command
    my $output;
    my $result = 1;

    try {
        my $response =
          $api_set_obj->system_node_run(
                                  node    => $node,
                                  command => "\"priv set -q diag; setflag $flag_name $flag_value \""
          );
        $output = $response->get_raw_output();
        $Log->comment("$pkg_name: setflag $flag_name $flag_value set");
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        $result = 0;
    };
    return $result;

} ## end sub RaidSqa_setflag

#------------------------------------------------------------------------------

=head2 RaidSqa_set_raid_lw_opt

RaidSqa_set_raid_lw_opt(api_set_obj=>$api_set_obj, conntype=>$conntype, option_value=>$option_value, node=>$node);

This subroutine enables/disables the system wide option raid_lw_opt.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh"

=item option_value

(Optional): Value of option raid_lw_opt to be set

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_set_raid_lw_opt {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        option_value => {type => SCALAR},
                        node         => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj  = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $option_value = $opts{option_value};
    my $node         = $opts{node};

    my $option_name1 = "raid.lost_write.enable";
    my $option_name2 = "raid.lost_write.interactive.enable";

    my $value_raid_lost_write =
      RaidSqa_get_options(api_set_obj => $api_set_obj,
                          conntype    => $conntype,
                          option_name => $option_name1,
                          node        => $node);
    if ($value_raid_lost_write eq $option_value) {
        $Log->comment("$pkg_name: options $option_name1 already set to $option_value");
    } else {

        # Get value for option raid.lost_write.interactive.enable to set it back to original value
        # $value_lw_interactive_option holds orginal value of system option raid.lost_write.interactive.enable
        my $value_lw_interactive_option =
          RaidSqa_get_options(api_set_obj => $api_set_obj,
                              conntype    => $conntype,
                              option_name => $option_name2,
                              node        => $node);
        $Log->comment("$pkg_name: LW Interactive option is set to $value_lw_interactive_option");

        # $reset_interactive_flag is used to indicate if resetting the value of lw interactive option is required.
        # If 1, lw interactive option will be reset to its orignal value
        my $reset_interactive_flag = undef;

        # Disable option raid.lost_write.interactive.enable
        if ($value_lw_interactive_option ne "off") {
            $Log->comment("$pkg_name: Setting lw interactive option to off");
            RaidSqa_set_options(api_set_obj  => $api_set_obj,
                                conntype     => $conntype,
                                option_name  => $option_name2,
                                option_value => "off",
                                node         => $node);
            $reset_interactive_flag = 1;
            sleep(5);
        } ## end if ($value_lw_interactive_option...
        RaidSqa_set_options(api_set_obj  => $api_set_obj,
                            conntype     => $conntype,
                            option_name  => $option_name1,
                            option_value => $option_value,
                            node         => $node);
        sleep(5);
        $value_raid_lost_write =
          RaidSqa_get_options(api_set_obj => $api_set_obj,
                              conntype    => $conntype,
                              option_name => $option_name1,
                              node        => $node);

        $Log->comment(
             "$pkg_name: raid_lost_write : $value_raid_lost_write and option_value : $option_value");
        if ($value_raid_lost_write ne $option_value) {
            $Log->comment("$pkg_name: options $option_name1 could not set to $option_value");
            return (0);
        } else {
            $Log->comment("$pkg_name: options $option_name1 set to $option_value");
        }

        # Reset the value for options raid.lost_write.interactive.enable
        if (defined($reset_interactive_flag)) {
            RaidSqa_set_options(api_set_obj  => $api_set_obj,
                                conntype     => $conntype,
                                option_name  => $option_name2,
                                option_value => $value_lw_interactive_option,
                                node         => $node);
        }
    } ## end else [ if ($value_raid_lost_write...
    return (1);
} ## end sub RaidSqa_set_raid_lw_opt

#------------------------------------------------------------------------------

=head2 RaidSqa_set_aggr_raid_lost_write

RaidSqa_set_aggr_raid_lost_write(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, option_value=>$option_value, node=>$node);

This subroutine enables/disables the aggr option raid_lost_write.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(required) : Name of vol/aggr on/for which raid_lost_write option should be disabled.

=item option_value

(Optional): Value of option raid_lost_write to be set for aggr $filer_vol

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_set_aggr_raid_lost_write {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        option_value => {type => SCALAR},
                        filer_vol    => {type => SCALAR},
                        node         => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj  = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $option_value = $opts{option_value};
    my $filer_vol    = $opts{filer_vol};
    my $node         = $opts{node};

    my $option_name1 = 'raid_lost_write';
    my $option_name2 = "raid.lost_write.interactive.enable";

    my $value_raid_lost_write =
      RaidSqa_get_aggr_options(api_set_obj => $api_set_obj,
                               conntype    => $conntype,
                               filer_vol   => $filer_vol,
                               options     => $option_name1,
                               node        => $node);

    if ($value_raid_lost_write eq $option_value) {
        $Log->comment("$pkg_name: options $option_name1 for $filer_vol is set to $option_value");
    } else {

        # Get value for option raid.lost_write.interactive.enable to set it back to original value
        # $value_lw_interactive_option holds orginal value of system option raid.lost_write.interactive.enable
        my $value_lw_interactive_option =
          RaidSqa_get_options(api_set_obj => $api_set_obj,
                              conntype    => $conntype,
                              option_name => $option_name2,
                              node        => $node);

        # $reset_interactive_flag is used to indicate if resetting the value of lw interactive option is required.
        # If 1, lw interactive option will be reset to its orignal value
        my $reset_interactive_flag = undef;

        # Disable option raid.lost_write.interactive.enable
        if ($value_lw_interactive_option eq "on") {
            RaidSqa_set_options(api_set_obj  => $api_set_obj,
                                conntype     => $conntype,
                                option_name  => $option_name2,
                                option_value => "off",
                                node         => $node);
            $reset_interactive_flag = 1;
            sleep(5);
        }

        # -f is required even if interactive option is off
        RaidSqa_aggr_options(api_set_obj => $api_set_obj,
                             conntype    => $conntype,
                             filer_vol   => $filer_vol,
                             options     => $option_name1,
                             value       => "$option_value -f",
                             node        => $node);
        sleep(5);
        $value_raid_lost_write =
          RaidSqa_get_aggr_options(api_set_obj => $api_set_obj,
                                   conntype    => $conntype,
                                   filer_vol   => $filer_vol,
                                   options     => $option_name1,
                                   node        => $node);

        if ($value_raid_lost_write ne $option_value) {
            $Log->comment(
                   "$pkg_name: options $option_name1 for $filer_vol could not set to $option_value");
            return (0);
        } else {
            $Log->comment("$pkg_name: options $option_name1 for $filer_vol set to $option_value");
        }

        # Reset the value for options raid.lost_write.interactive.enable
        if (defined($reset_interactive_flag)) {
            RaidSqa_set_options(api_set_obj  => $api_set_obj,
                                conntype     => $conntype,
                                option_name  => $option_name2,
                                option_value => $value_lw_interactive_option,
                                node         => $node);
        }
    } ## end else [ if ($value_raid_lost_write...
    return (1);
} ## end sub RaidSqa_set_aggr_raid_lost_write

#------------------------------------------------------------------------------

=head2 RaidSqa_get_root_vol

RaidSqa_get_root_vol(api_set_obj => $api_set_obj, conntype=>$conntype, node => $node);

This subroutine by gets root volume name in a given filer.

It returns an scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_root_vol {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my %all_vol  = $raid_obj->all_volumes();
    my @all_vol  = keys(%all_vol);
    my $vol;
    foreach $vol (@all_vol) {
        my $vol_obj = $raid_obj->volume("$vol");
        if (!$vol_obj) {
            next;
        }
        if ($vol_obj->parameter("root") eq "1") {
            return $vol;
        }
    }
} ## end sub RaidSqa_get_root_vol

#------------------------------------------------------------------------------

=head2 RaidSqa_get_root_aggr

RaidSqa_get_root_aggr(api_set_obj => $api_set_obj,  conntype=>$conntype, node=>$node);

This subroutine by gets root aggr name in a given filer.

It returns an scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_root_aggr {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my %all_aggr = $raid_obj->all_aggregates();
    my @all_aggr = keys(%all_aggr);
    my $aggr;
    foreach $aggr (@all_aggr) {
        my $aggr_obj = $raid_obj->aggregate("$aggr");
        if (!$aggr_obj) {
            next;
        }
        if ($aggr_obj->parameter("root") eq "1") {
            $Log->comment("$pkg_name: Aggregate $aggr is an root aggregate");
            return $aggr;
        }
    } ## end foreach $aggr (@all_aggr)
} ## end sub RaidSqa_get_root_aggr

#------------------------------------------------------------------------------

=head2 RaidSqa_get_fileid

RaidSqa_get_fileid(api_set_obj => $api_set_obj, conntype=>$conntype, file_name=>$file_name, node => $node);

This subroutine get fileid of a file.

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item file_name

(Required): Name of the file_name to get id.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_fileid {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        file_name => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $file_name   = $opts{file_name};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $entry;
    my $file_id;

    try {

        #(undef,undef,$output)=say($conn, "priv set -q test; showfh $file_name", -timeout, $RaidSqa_TIMEOUT);
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q test; showfh $file_name\"");
        my $output = $response->get_raw_output();

        my @file_info = split(/ /, $output);
        foreach $entry (@file_info) {
            if ($entry =~ /fileid=(.*)/) {
                $file_id = $1;
                $Log->comment("$pkg_name: fileid of file $file_name is $file_id");
                last;
            }
        }

        if (!$file_id) {
            logresult("FATAL", "$pkg_name: Could not get the fileid of file $file_name ");

        } else {
            return $file_id;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };

} ## end sub RaidSqa_get_fileid

#------------------------------------------------------------------------------

=head2 RaidSqa_get_file_block_info

RaidSqa_get_file_block_info(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, file_name=>$file_name, block_number=>$block_number, node => $node);

This subroutine get disk and dbn number for a file block.

Does not return anything.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item file_name

(Required): Name of the file_name .

=item block_number

(Required): the block number of file .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_file_block_info {
    my (%opts) =
      RaidSqa_validate(@_, {
        api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
        filer_vol    => {type => SCALAR},
        file_name    => {type => SCALAR},
        block_number => {type => SCALAR},
        node         => {type => SCALAR, default => "local", optional => 1},
    });

    my $api_set_obj  = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $filer_vol    = $opts{filer_vol};
    my $file_name    = $opts{file_name};
    my $block_number = $opts{block_number};
    my $node         = $opts{node};

    # create connection
    my ($host, $conn, $output, $entry);

    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $file_id =  RaidSqa_get_fileid(api_set_obj => $api_set_obj, file_name => $file_name, node => $node);

    try {
        my $response = $api_set_obj->system_node_run(
            node    => $node,
            command => "\"priv set -q diag; dumpblock info $filer_vol $file_id $block_number\""
        );
        my $o = $response->get_raw_output();
        my @block_info = split(/\r?\n/, $o);
        $Log->comment("$pkg_name: vbn is @block_info");
        my ($disk_id, $dbn_no, $vbn_no, $cksum_dbn);
        foreach $entry (@block_info) {
            if ($entry =~ /disk (.*)/) {
                $disk_id = $1;
                $Log->comment("$pkg_name: diskid is $disk_id");
                next;
            }
            if ($entry =~ /^dbn (.*)/) {
                $dbn_no = $1;
                $Log->comment("$pkg_name: dbn is $dbn_no");
                next;
            }
            if ($entry =~ /pvbn (.*)/) {
                $vbn_no = $1;
                $Log->comment("$pkg_name: vbn is $vbn_no");
                next;
            }
            if ($entry =~ /cksum_dbn (.*)/) {
                $cksum_dbn = $1;
                $Log->comment("$pkg_name: cksum_dbn is $cksum_dbn");
                next;
            }

        } ## end foreach $entry (@block_info)
        if (!$disk_id || !$dbn_no || !$vbn_no) {
            logresult(
                "FATAL",
                "$pkg_name: Could not get the disk_id  or dbn_no or vbn_no of file $file_name, block $block_number "
            );
        } else {
            if (defined($cksum_dbn)) {
                return ($disk_id, $dbn_no, $vbn_no, $cksum_dbn);
            } else {
                return ($disk_id, $dbn_no, $vbn_no);
            }
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };

} ## end sub RaidSqa_get_file_block_info

#-------------------------------------------------------------------------------------------------------------------------------------

=head2 RaidSqa_get_dumpblock_info

RaidSqa_get_dumpblock_info(api_set_obj  => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, fileid=>$fileid, fbn=>$fbn);

This subroutine returns hash structure of dumpblock info for a given fileid and fbn

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required): filer_vol, volume on which file resides

=item fileid

(Required): fileid of the file

=item fbn

(Required): File block number for which dumpblock info is required.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_dumpblock_info {
    my (%opts) = RaidSqa_validate(@_, { api_set_obj => { type=>OBJECT, isa=>'NACL::APISet' },
                    conntype    =>  { type=>SCALAR, default=>"#ssh", optional=>1 },
                                    filer_vol   => { type=>SCALAR },
                                    fileid      => { type=>SCALAR },
                                    fbn         => { type=>SCALAR },
                    node        => {type => SCALAR, default => "local", optional => 1},
                                }
                         );
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $fileid      = $opts{fileid};
    my $fbn         = $opts{fbn};
    my $node        = $opts{node};
    my @param_list = (  'disk',
                'dbn',
                'cksum_dbn',
                'pvbn',
                'pvbn-from-parent',
                'pvbn-from-container',
                'vvbn',
                'pvbn-of-parent',
                'pvbn-of-parent-from-container',
                'vvbn-of-parent',
                'fileid-of-container-l1',
                'fileid-of_container-l1-is-private',
                'fbn-of-container-l1',
                'ckse-dbn',
                'ckse-checksum',
                'ckse-buftreeid',
                'ckse-fbn',
                'ckse-cp-count',
                'ckse-notflexvol-flag',
                'fbn-of-inofile',
                'pvbn-of-inofile',
                'pvbn-of-inofile-from-parent',
                'pvbn-of-inofile-from-container',
                'vvbn-of-inofile'
                    );

    my $CMDTIMEOUT = 200;
    my %params_hash = map {$_=>''} @param_list;
    my $output;

    try {
        my $response =
        $api_set_obj->system_node_run(node    => $node,
                          command => "\"priv set -q diag;dumpblock info $filer_vol $fileid $fbn\" ");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        return 0;
    };

    if ($output=~ /block not found|inode has only|is invalid/g) {
            $Log->comment("$pkg_name: dumpblock info failed. invalid parameters: $output ");
            return(0);
    }
    my @raw_output= split(/\r*\n/, $output);
    foreach my $line (@raw_output) {
        my ($key, $value) = split(/\s+/, $line);
        if (exists $params_hash{$key}) {
            $params_hash{$key} = $value;
        }
    }
    return (\%params_hash);
}

#------------------------------------------------------------------------------

=head2 RaidSqa_get_scramble_dumpblock_info

RaidSqa_get_scramble_dumpblock_info(api_set_obj  => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, fileid=>$fileid, vserver=>$vserver, node => $node);

This subroutine returns hash structure of parsed output for scramble dumpblocks command

Below is sample entry in returned data structure
        '11953' => [
             {
               'level' => 0,
               'dbn' => '11953',
               'disk' => '0c.10',
               'fbn' => '8566'
             },
             {
               'level' => 0,
               'dbn' => '11953',
               'disk' => '0c.11',
               'fbn' => '8647'
             },
             {
               'level' => 0,
               'dbn' => '11953',
               'disk' => '0b.12',
               'fbn' => '8716'
             },
             {
               'level' => 0,
               'dbn' => '11953',
               'disk' => '0b.13',
               'fbn' => '8837'
             }
           ],

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required): filer_vol, volume on which file resides,
            Note that currently this subroutine works only for flexvol

=item fileid

(Required): fileid of the file

=item vserver

(Required): Name of vserver to which filer_vol belongs.

=item timeout

(Required): timeout for command execution

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_scramble_dumpblock_info {
    my (%opts) = RaidSqa_validate(@_, {
        api_set_obj  => {type => OBJECT, isa     => 'NACL::APISet'},
        conntype     => {type => SCALAR, default => "#ssh", optional => 1},
        fileid       => {type => SCALAR},
        filer_vol    => {type => SCALAR},
        vserver      => {type => SCALAR, default => 'vs0',   optional => 1},
        timeout      => {type => SCALAR, default => $RaidSqa_TIMEOUT, optional => 1},
        node         => {type => SCALAR, default => 'local', optional => 1},
    });

    my $api_set_obj = $opts{api_set_obj};
    my $fileid      = $opts{fileid};
    my $filer_vol   = $opts{filer_vol};
    my $node        = $opts{node};
    my $vserver     = $opts{vserver};
    my $scramble_info = {};
    my ($output);

    my $vserver_root = RaidSqa_get_vserver_root(api_set_obj => $api_set_obj, vserver => $vserver);
    if(!$vserver_root) {
        $Log->comment("$pkg_name: root volume was not found - $vserver");
        return {};
    }

    my $cmd = "\"priv set test; scramble dumpblocks -v -S 0 $filer_vol $fileid\"";
    my $filervol_file = "WAFL_dumpblocks.$filer_vol.$fileid";

    try {
        my $response = $api_set_obj->system_node_run(node => $node, command => $cmd, 'connectrec-timeout' => $opts{timeout});
        $output = $response->get_raw_output();
        $Log->comment("$pkg_name: Scramble dumpblock output - $output");
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return {};
    };

    my $output_file = ($output =~ /scramble dumpblocks: output being saved to (.*)\r*\n/im) ? $1 : "";
    $Log->comment("$pkg_name: Scramble dumpblock output_file - $output_file");
    return {} if($output_file eq "");

    #For Cmode, copy this file to $filer_vol
    RaidSqa_dd(api_set_obj => $api_set_obj, if => $output_file, of => "/vol/$filer_vol/".$filervol_file);
    my $mountpoint = RaidSqa_aggr_mount(api_set_obj => $api_set_obj, filer_vol => $filer_vol, vserver => $vserver);
    if(!$mountpoint) {
        $Log->comment("$pkg_name: Error: Mount failed - $filer_vol, vserver - $vserver");
        return {};
    }

    $filervol_file = $mountpoint."/".$filervol_file;
    $Log->comment("$pkg_name: Info: Parse output file - $filervol_file");
    return _parse_scramble_dumpblock_file(file_name => $filervol_file);
}

#------------------------------------------------------------------------------

=head2 RaidSqa_get_raid_size_value

RaidSqa_get_raid_size_value(api_set_obj => $api_set_obj, conntype=>$conntype, raid_size=>$raid_size, raid_type=>$raid_type, storage_type=>$storage_type, node => $node, override =>$override);

This subroutine gets value of raid_size for min/max/default for a given raid_type and storage_type.

It returns a scalar $raid_size on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item raid_size

(Required) : Raid size of the volume/aggregate to be created.  No default. Any value from min, max or default.

=item raid_type

(Required) : Raid type of the volume/aggregate to be created. No default. Can be raid4 or raid_dp.

=item storage_type

(Required) : Disk storage type. Can be  SCSI, FCAL, ATA, EATA, LUN, SAS, SATA, SSD, BSAS, XSAS, XATA, FSAS.

=item node

(Optional):  Node name on which the operation is to be performed

=item override

(Optional): override 'on' refers to the value of the hidden options raid.raid4.raidsize.override and raid.raid_dp.raidsize.override which are 'off' by default

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_raid_size_value {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        raid_size    => {type => SCALAR},
                        raid_type    => {type => SCALAR},
                        storage_type => {type => SCALAR},
                        override     => {type => SCALAR, default => "off", optional => 1},
                        node         => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj  = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $raid_size    = $opts{raid_size};
    my $raid_type    = $opts{raid_type};
    my $storage_type = $opts{storage_type};
    my $override     = lc $opts{override};
    my $node         = $opts{node};

    my @storage_types = ("SCSI", "FCAL", "ATA", "EATA", "LUN", "SAS", "SATA", "SSD", "BSAS", "XSAS", "XATA","BSSD", "MSATA", "FSAS");
    my @raid_types    = ("raid_dp", "raid4", "raid0");
    my %hash          = ();
    my $index         = 0;
    my $entry         = 0;
    my $entry_for     = "";
    my $raid_size_value = undef;

    if (!(grep /$raid_type/i, @raid_types)) {
        $Log->comment(
             "$pkg_name: Unrecongnized value for raid_type: $raid_type, should be one of @raid_types"
        );
        return 0;
    }

    if ($raid_size =~ /^(max|min|default)$/i) {
        if ($raid_size =~ /min/) {
            return 1 if ($raid_type eq "raid0");
            return 2 if ($raid_type eq "raid4");
            return 3 if ($raid_type eq "raid_dp");
        }
    } else {
        $Log->comment(
              "$pkg_name: Unrecongnized value for raid_size: $raid_size, should be min|max|default");
        return 0;
    }

    if (!(grep /$storage_type/i, @storage_types)) {
        $Log->comment(
            "$pkg_name: Unrecongnized value for raid_type: $storage_type, should be one of @storage_types"
        );
        return 0;
    }
    #defining some default values for older builds where raid_config command is not available
    if($raid_type eq "raid0") {
        if($storage_type =~/SCSI|FCAL|ATA|EATA|LUN|SAS|SATA|SSD|BSAS|XSAS|XATA|BSSD|FSAS/){
            $raid_size_value = 8 if ($raid_size eq "default");
            if($storage_type =~/SCSI|FCAL|LUN|SAS|SSD|XSAS|BSSD/){
                $raid_size_value = 26 if ($raid_size eq "max");
            }elsif($storage_type =~/ATA|EATA|XATA|BSAS|SATA|FSAS/){
                $raid_size_value = 14 if ($raid_size eq "max");
            }
        }
    }elsif($raid_type eq "raid4") {
        if($storage_type =~ /SCSI|FCAL|LUN|SAS|SSD|XSAS|BSSD/) {
                $raid_size_value = 8 if($raid_size eq "default");
                $raid_size_value = 14 if($raid_size eq "max");
        } elsif($storage_type =~/ATA|EATA|XATA|BSAS|SATA|FSAS/){
            $raid_size_value = 7 if ($raid_size eq "default");
            $raid_size_value = 7 if ($raid_size eq "max");
        }
    } elsif($raid_type eq "raid_dp") {
        if($storage_type =~/SCSI|FCAL|LUN|SAS|XSAS/){
            $raid_size_value = 16 if ($raid_size eq "default");
            $raid_size_value = 28 if ($raid_size eq "max");
        }elsif($storage_type =~/SSD|BSSD/){
            $raid_size_value = 23 if ($raid_size eq "default");
            $raid_size_value = 28 if ($raid_size eq "max");
        }elsif($storage_type =~/ATA|BSAS|XATA|EATA|SATA|MSATA|FSAS/){
            $raid_size_value = 14 if ($raid_size eq "default");
            $raid_size_value = 20 if ($raid_size eq "max");
        }
    }

    if ($override =~ /^(on|off)$/i) {
        $override = "override_" . $override;
    } else {
        $Log->comment(
                  "$pkg_name: Unrecongnized value for override option: $override, should be on/off");
        return 0;
    }

    my $output;
    try {
        my $response =
          $api_set_obj->system_node_run(
                                 node    => $node,
                                 command => "\"priv set -q diag; raid_config group_size_override \""
          );
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
    };

    my @output = split(/\r*\n/, $output);

    #get index from where table starts
    for ($index = 0 ; $index <= $#output ; $index++) {
        if ($output[$index] =~ /Disk Type/) {
            last;
        }
    }
    $index += 2;

    for (; $index <= $#output ; $index++) {
        my @line = split(/\s+/, $output[$index]);
        $Log->comment("Dumper:: Array " . Dumper(\@line));
        if (grep /$line[0]/, @storage_types) {
            if (grep /$line[1]/, @raid_types) {
                $entry                                                   = 1;
                $entry_for                                               = uc $line[0];
                $hash{$entry_for}->{$line[1]}->{override_off}->{default} = $line[2];
                $hash{$entry_for}->{$line[1]}->{override_off}->{max}     = $line[3];
                $hash{$entry_for}->{$line[1]}->{override_on}->{default}  = $line[4];
                $hash{$entry_for}->{$line[1]}->{override_on}->{max}      = $line[5];
            } else {
                $entry = 0;
                next;
            }
        } elsif ($line[0] ne "") {
            $Log->comment("$pkg_name: unknown disk_type found: $line[0]");
            $entry     = 0;
            $entry_for = "";
            next;
        } elsif ($entry) {
            if ((grep /$line[1]/, @raid_types)) {
                $hash{$entry_for}->{$line[1]}->{override_off}->{default} = $line[2];
                $hash{$entry_for}->{$line[1]}->{override_off}->{max}     = $line[3];
                $hash{$entry_for}->{$line[1]}->{override_on}->{default}  = $line[4];
                $hash{$entry_for}->{$line[1]}->{override_on}->{max}      = $line[5];
            } else {
                next;
            }
        } ## end elsif ($entry)
    } ## end for (; $index <= $#output...

    my $value = $hash{$storage_type}->{$raid_type}->{$override}->{$raid_size};
    if($value =~ /\d+/) {
        $Log->comment(
            "$pkg_name: $raid_size raid_size for $storage_type for $raid_type is $value ");
        return $value;
    } elsif(defined($raid_size_value)) {
        $Log->comment("$pkg_name: $raid_size raid_size for $storage_type for $raid_type is $raid_size_value ");
        return $raid_size_value;
    } else {
        $Log->comment("$pkg_name: Disks of storage type $storage_type are not present");
        return 0;
    }
} ## end sub RaidSqa_get_raid_size_value

#------------------------------------------------------------------------------

=head2 RaidSqa_get_scrub_report_by_group

RaidSqa_get_scrub_report_by_group(api_set_obj => $api_set_obj, conntype=>$conntype, group_name=>$group_name, node=>$node);

This subroutine gets scrub report (PI/checksum/media/lw error) of a particular raid group.

It returns reference to a hash on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid group to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_scrub_report_by_group {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $group_obj = $raid_obj->raid_group("$group_name");
    if (!$group_obj) {
        $Log->comment("$pkg_name: group with name $group_name is not found");
        return 0;
    }
    my %hash;

    # check if filer is having a flex vol of specified name
    if ($group_obj->parameter("scrub_progress") ne "-1") {
        $Log->comment("$pkg_name: Scrub is still running on raid group $group_name");
        return 0;
    } else {
        $hash{pi}        = $group_obj->parameter("scrub_pi");
        $hash{cksum_err} = $group_obj->parameter("scrub_cksum_err");
        $hash{media_err} = $group_obj->parameter("scrub_media_err");
        $hash{lw_err}    = $group_obj->parameter("scrub_lw_err");
    }
    return \%hash;
} ## end sub RaidSqa_get_scrub_report_by_group

#------------------------------------------------------------------------------

=head2 RaidSqa_get_preferred_storage_type

RaidSqa_get_preferred_storage_type(api_set_obj=>$api_set_obj, conntype=>$conntype);

This subroutine gets preferred storage type in a mix environment based upon which storage type is having more spare disks available.

It returns an scalar $storage_type on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item checksum_type

(Optional): If defined, Disks having checksum_type will be considerd to determine storage type

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_preferred_storage_type {
    my (%opts) = RaidSqa_validate(@_, {
        api_set_obj     => {type => OBJECT, isa     => 'NACL::APISet'},
        conntype        => {type => SCALAR, default => "#ssh", optional => 1},
        checksum_type   => {type => SCALAR | UNDEF, default => undef, optional => 1},
        node            => {type => SCALAR, default => "local", optional => 1},
    });

    my $api_set_obj     = $opts{api_set_obj};
    my $conntype        = $opts{conntype};
    my $checksum_type   = $opts{checksum_type};
    my $node            = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj    = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my %spare_disks = $raid_obj->spare_disks();
    my @spare_disks = keys(%spare_disks);
    my (%disk_by_types,%disk_type_count);
    my $storage_type = ($checksum_type eq 'advanced_zoned') ? 'ATA' : 'FCAL';

    # get spare based upon storage type
    @spare_disks = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@spare_disks);
    foreach my $disk (@spare_disks) {
        my $disk_obj = $raid_obj->disk("$disk");
        next if (!defined($disk_obj));
        my $disk_type = $disk_obj->parameter("disk_type");

        if (defined($checksum_type) && ($checksum_type ne "undef")) {
            my $disk_cksum = $disk_obj->parameter("checksum_type");
            next if (defined($disk_cksum) && $disk_cksum ne $checksum_type);
        }

        if(ref($disk_by_types{$disk_type})) {
            push(@{$disk_by_types{$disk_type}}, $disk);
        } else {
            $disk_by_types{$disk_type} = [$disk];
        }
    }

    foreach my $disk_type (keys %disk_by_types) {
        $disk_type_count{scalar(@{$disk_by_types{$disk_type}})} = $disk_type;
    }

    $Log->comment("$pkg_name: disk_type_count: " . Dumper(\%disk_type_count));
    my @counts = sort { $a <=> $b } keys %disk_type_count;
    $storage_type = $disk_type_count{$counts[$#counts]} if($counts[$#counts] > 0);
    $Log->comment("$pkg_name: Storage type is $storage_type");
    return $storage_type;
} ## end sub RaidSqa_get_preferred_storage_type

#------------------------------------------------------------------------------

=head2 RaidSqa_get_vserver_root

RaidSqa_get_vserver_root(api_set_obj => $api_set_obj, vserver=>$vserver);

This subroutine gets root volume of vserver.

It returns name of root volume for a given vserver on success.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item vserver

(Required) : Name of the vserver.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_vserver_root {
    my (%opts) = RaidSqa_validate(@_, {
        api_set_obj  => {type => OBJECT, isa     => 'NACL::APISet'},
        conntype     => {type => SCALAR, default => "#ssh", optional => 1},
        vserver      => {type => SCALAR, default => 'vs0',   optional => 1},
    });

    my $api_set_obj = $opts{api_set_obj};
    my $vserver     = $opts{vserver};
    my ($output, $response, $rootvol);

    try {
        $response = $api_set_obj->vserver_show(vserver => $vserver);
        $output   = $response->get_parsed_output();
        $rootvol     = $output->[0]->{rootvolume};
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };

    $Log->comment("$pkg_name : root volume for vserver $vserver - $rootvol");
    return $rootvol;
}


#------------------------------------------------------------------------------

=head2 RaidSqa_get_vserver_ip_addr

RaidSqa_get_vserver_ip_addr(api_set_obj => $api_set_obj, conntype=>$conntype, vserver=>$vserver, node => $node);

This subroutine gets the ip address of vserver.

It returns a ip address on success.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item vserver

(Required) : Name of the vserver.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_vserver_ip_addr {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        vserver => {type => SCALAR},
                        node    => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $vserver     = $opts{vserver};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }
    try {
        my $response = $api_set_obj->network_interface_show('vserver' => "$vserver");
        my $ref_ip   = $response->get_parsed_output();
        my @ni_all   = @$ref_ip;

        my $ip = "";
        foreach my $ni (@ni_all) {
            if (exists($ni->{address})) {
                $ip = $ni->{address};
                last;
            }
        }
        return $ip;
    } ## end try
    catch NATE::BaseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };
}    # end get_ip_addr

#------------------------------------------------------------------------------

=head2 RaidSqa_get_volume_junc_path

RaidSqa_get_volume_junc_path(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);


This subroutine gets a junction path for a given volume.

It returns a junction path on success.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate volume.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_volume_junc_path {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my ($response, $jp, $result);

    $result = 1;
    try {
        $response = $api_set_obj->volume_show('volume' => "$filer_vol");
        my $ref_jp = $response->get_parsed_output();
        my @vols   = @$ref_jp;
        foreach my $vol (@vols) {
            if (exists($vol->{'junction-path'})) {
                $jp = $vol->{'junction-path'};
                last;
            }
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
    };


    $Log->comment("$pkg_name: Junction path for $filer_vol is $jp");
    if($jp =~ /\s*-\s*/i) {
        $Log->comment("$pkg_name: remount volume with junc_path /$filer_vol");
        try {
            $api_set_obj->volume_mount(volume => $filer_vol, 'junction-path' => "/$filer_vol");
            $jp = "/$filer_vol";            #This assignment to $jp should be done only after successful volume mount.
        } catch  NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            _handle_nacl_exception($exception_object);
        }
    }

    return ($jp) ? $jp : 0;
}    # end get_junc_path

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_list

RaidSqa_get_aggr_list(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node);

This subroutine by default gets list of aggr/tradvol in a given filer.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_list {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj  = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my %aggr_list = $raid_obj->all_aggregates();
    my @aggr_list = keys(%aggr_list);

    return @aggr_list;
} ## end sub RaidSqa_get_aggr_list

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_status

RaidSqa_get_aggr_status(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, options=>$options);

This subroutine gets o/p of aggr status command for all aggr/one aggr/or-with different options.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggregate or traditional volume to get aggr status for. If not given, it will give o/p of all aggr. By default it is
undefined.

=item options

(Optional) : Any options with vol status o/p for a aggr or all aggr. By default it is "-r".

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_status {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        options   => {type => SCALAR,         default => undef,   optional => 1},
                        node      => {type => SCALAR,         default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $options     = $opts{options};
    my $node        = $opts{node};

    my $result = 1;
    my $output;
    try {
        my $response =
          $api_set_obj->system_node_run(
                                node    => $node,
                                command => "\"priv set -q diag; aggr status $options $filer_vol \" "
          );
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    my @aggr_status = split(/\r?\n/, $output);
    return @aggr_status;
} ## end sub RaidSqa_get_aggr_status

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_type

RaidSqa_get_aggr_type(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine gets info whether a particular volume/aggergate is aggregate or traditional volume.

It returns scalar value.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_type {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $vol_obj  = $raid_obj->volume("$filer_vol");
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!$aggr_obj || !$vol_obj) {
        $Log->comment("$pkg_name: Aggr or flex vol with name $filer_vol does not exist");
        return 0;
    }

    if ($aggr_obj) {
        return $aggr_obj->parameter("store_type");
    } else {
        return $vol_obj->parameter("store_type");
    }

} ## end sub RaidSqa_get_aggr_type

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_cksum_type

RaidSqa_get_aggr_cksum_type(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine gets checksum type info of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_cksum_type {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!$aggr_obj) {
        $Log->comment("$pkg_name: Aggr with name $filer_vol does not exist");
    }
    return $aggr_obj->parameter("cksum_type");
} ## end sub RaidSqa_get_aggr_cksum_type

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_raidtree_type

RaidSqa_check_aggr_raidtree_type(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine gets raidtree_type of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_raidtree_type {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!$aggr_obj) {
        $Log->comment("$pkg_name: Aggr with name $filer_vol does not exist");
    }
    return $aggr_obj->parameter("raidtree_type");
} ## end sub RaidSqa_get_aggr_raidtree_type

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_storage_type

RaidSqa_check_aggr_storage_type(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine gets storage_type of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_storage_type {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj       = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj       = $raid_obj->aggregate("$filer_vol");
    my %aggr_disk_hash = $aggr_obj->all_disks();
    my @aggr_disk_list = keys(%aggr_disk_hash);
    foreach my $disk (@aggr_disk_list) {
        my $disk_obj = $aggr_obj->disk("$disk");
        if (!$disk_obj) {
            next;
        } else {
            return $disk_obj->parameter("disk_type");
        }
    }
} ## end sub RaidSqa_get_aggr_storage_type

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_fsm_state

RaidSqa_get_aggr_fsm_state(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node => $node);

This subroutine gets fsm_state of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_fsm_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!$aggr_obj) {
        $Log->comment("$pkg_name: Aggr with name $filer_vol does not exist");
    }
    return $aggr_obj->parameter("fsm_state");
} ## end sub RaidSqa_get_aggr_fsm_state

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_mirror_state

RaidSqa_check_aggr_mirror_state(host=>$host, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine gets mirror_state of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_mirror_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!$aggr_obj) {
        $Log->comment("$pkg_name: Aggr with name $filer_vol does not exist");
    }
    return $aggr_obj->parameter("mirror_state");
} ## end sub RaidSqa_get_aggr_mirror_state

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_fsid

RaidSqa_get_aggr_fsid(api_set_obj=>$api_set_obj, node=>$node, conntype=>$conntype, filer_vol=>$filer_vol, node=>$node);

This subroutine returns fsid parameter from raid_config ouput for given aggr

It returns fsid of a given aggr

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(required) : Name of vol/aggr who's fsid is to be returned

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_fsid {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $vol_obj  = $raid_obj->volume("$filer_vol");
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!(defined($aggr_obj) or defined($vol_obj))) {
        $Log->comment("$pkg_name: vol/aggr $filer_vol does not exist");
    }
    if ($aggr_obj) {
        return $aggr_obj->parameter("fsid");
    } else {
        return $vol_obj->parameter("fsid");
    }
} ## end sub RaidSqa_get_aggr_fsid

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_uuid

RaidSqa_get_aggr_uuid(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine gets value of flag uuid of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_uuid {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!defined($aggr_obj)) {
        $Log->comment("$pkg_name: Could not get aggr object for $filer_vol");
        return 0;
    } else {
        return $aggr_obj->parameter("uuid");
    }
} ## end sub RaidSqa_get_aggr_uuid

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_rtid

RaidSqa_get_aggr_rtid(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine gets value of flag raidtreeID of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_rtid {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!defined($aggr_obj)) {
        $Log->comment("$pkg_name: Could not get aggr object for $filer_vol");
        return 0;
    } else {
        return $aggr_obj->parameter("raidtreeID");
    }
} ## end sub RaidSqa_get_aggr_rtid

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_bits

RaidSqa_check_aggr_bits(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine gets aggr_bits of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_bits {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!$aggr_obj) {
        $Log->comment("$pkg_name: Aggr with name $filer_vol does not exist");
        return 0;
    }
    return $aggr_obj->parameter("aggr_format");

} ## end sub RaidSqa_get_aggr_bits

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_context_parity_valid

RaidSqa_check_aggr_context_parity_valid(hapi_set_obj => $api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node => $node);

This subroutine gets value of flag context_parity_valid of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_context_parity_valid {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!$aggr_obj) {
        $Log->comment("$pkg_name: Aggr with name $filer_vol does not exist");
        return 0;
    }
    return $aggr_obj->parameter("context_parity_valid");
} ## end sub RaidSqa_get_aggr_context_parity_valid

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_has_bad_blocks

RaidSqa_get_aggr_has_bad_blocks(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node => $node);

This subroutine gets value of flag has_bad_blocks of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_has_bad_blocks {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!$aggr_obj) {
        $Log->comment("$pkg_name: Aggr with name $filer_vol does not exist");
        return 0;
    }
    return $aggr_obj->parameter("has_bad_blocks");
} ## end sub RaidSqa_get_aggr_has_bad_blocks

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_bad_block_count

RaidSqa_get_aggr_bad_block_count(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node => $node);

This subroutine gets value of flag bad_block_count of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_bad_block_count {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!$aggr_obj) {
        $Log->comment("$pkg_name: Aggr with name $filer_vol does not exist");
        return 0;
    }
    return $aggr_obj->parameter("bad_block_count");
} ## end sub RaidSqa_get_aggr_bad_block_count

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_tetris_opt_disabled

RaidSqa_get_aggr_tetris_opt_disabled(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node => $node);

This subroutine gets value of flag tetris_opt_disabled of a particular volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_tetris_opt_disabled {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (!$aggr_obj) {
        $Log->comment("$pkg_name: Aggr with name $filer_vol does not exist");
        return 0;
    }
    return $aggr_obj->parameter("tetris_opt_disabled");
} ## end sub RaidSqa_get_aggr_tetris_opt_disabled

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_raid_lw_enabled

RaidSqa_get_aggr_raid_lw_enabled(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol);

This function gets the value for parameter raid_lw_enabled
raid_lw_enabled : possible values: 0 or 1: if 1, aggr will be lost write protected

It returns value of raid_lw_enabled.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required): Name of volume for which parameter value to be extracted.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_raid_lw_enabled {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $vol_obj = $raid_obj->aggregate($filer_vol);

    if (!defined($vol_obj)) {
        $Log->comment("$pkg_name: Aggr of name $filer_vol does not exist");
        return 0;
    } else {
        my $value = $vol_obj->parameter('raid_lw_enabled');
        $Log->comment("$pkg_name: raid_lw_enabled is set to $value");
        return $value;
    }
} ## end sub RaidSqa_get_aggr_raid_lw_enabled

#------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_raid_lw_allowed

RaidSqa_get_aggr_raid_lw_allowed(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol);

This function gets the value for parameter raid_lw_allowed
raid_lw_allowed : possible values: 0 or 1: if 1 indicates that lost write protection can be turned 'ON' on that volume

It returns value of raid_lw_allowed.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required): Name of volume for which parameter value to be extracted.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_aggr_raid_lw_allowed {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $vol_obj = $raid_obj->aggregate($filer_vol);

    if (!defined($vol_obj)) {
        $Log->comment("$pkg_name: Aggr of name $filer_vol does not exist");
        return 0;
    } else {
        my $value = $vol_obj->parameter('raid_lw_allowed');
        $Log->comment("$pkg_name: raid_lw_allowed is set to $value");
        return $value;
    }
} ## end sub RaidSqa_get_aggr_raid_lw_allowed

#------------------------------------------------------------------------------

=head2 RaidSqa_get_vol_status

RaidSqa_get_vol_status(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, options => $options, node => $node );

This subroutine gets o/p of vol status command for all volumes/one volume/or-with different options.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggregate or traditional volume to get vol status for. If not given, it will give o/p of all volumes. By default it is
undefined.

=item options

(Optional) : Any options with vol status o/p for a volume or all volumes. By default it is "-r".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_vol_status {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        options   => {type => SCALAR,         default => undef,   optional => 1},
                        node      => {type => SCALAR,         default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $options     = $opts{options};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    try {

        # get o/p of vol status command with specified options
        my $response =
          $api_set_obj->system_node_run(
                                  node    => $node,
                                  command => "\"priv set -q diag; vol status $options $filer_vol \""
          );
        my $output = $response->get_raw_output();
        my @vol_status = split(/\r?\n/, $output);
        return @vol_status;
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $output           = _handle_nacl_exception($exception_object);
        my @error            = ($output);
        return @error;
    };
} ## end sub RaidSqa_get_vol_status

#------------------------------------------------------------------------------

=head2 RaidSqa_get_plex_list

RaidSqa_get_plex_list(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, node => $node);

This subroutine by default gets list of all plexes in a given filer. If filer_vol is given than it gives plex in that volume/aggregate.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_plex_list {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @plex_list;
    if ($filer_vol) {
        my $aggr_obj  = $raid_obj->aggregate("$filer_vol");
        my %plex_list = $aggr_obj->all_plexes();
        @plex_list = keys(%plex_list);
    } else {
        my %plex_list = $raid_obj->all_plexes();
        @plex_list = keys(%plex_list);
    }
    return @plex_list;
} ## end sub RaidSqa_get_plex_list

#----------------------------------------------------------------------------------------
=head2 RaidSqa_plex_destroy

RaidSqa_plex_destroy(api_set_obj=>$api_set_obj, node=>$node, conntype=>$conntype, plex_name => $plex_name, node=>$node,);

This subroutine destroy the plex of a mirror volume/aggr.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh"

=item plex_name

(Required) : Name of plex to destroy.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_plex_destroy {
    my (%opts) =
      RaidSqa_validate(@_, {
                api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                node        => {type => SCALAR, default => "local", optional => 1},
                plex_name   => {type => SCALAR},
            }
        );

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $plex_name   = $opts{plex_name};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    $plex_name =~ /\/(.*)\/(plex\d+)/i;
    my $aggregate = $1;
    my $plex = $2;
    my $output;

    $Log->comment("$pkg_name: destroying plex $plex_name");

    # check given plex is offline before starting destroy
    if (!RaidSqa_check_plex_state_name(api_set_obj => $api_set_obj,
                                       conntype    => $conntype,
                                       plex_name   => $plex_name,
                                       state_name  => "offline")
      ) {
        $Log->comment("$pkg_name: Can not destroy plex $plex_name as it is not offline");
        return 0;
    } else {

        # destroy given plex
        try {
            my $response = $api_set_obj->storage_aggregate_plex_delete(
                                            aggregate => $aggregate,
                                            plex      => $plex,
                                        );
            $output = $response->get_raw_output();
        } catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
        };

        # Check that plex got destroyed successfully
        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $plex_obj = $raid_obj->plex("$plex_name");

        if (!defined($plex_obj)) {
            $Log->comment("$pkg_name: plex destroy successful - $plex_name");
            return 1;
        } else {
            $Log->comment("$pkg_name: plex destroy failed - $plex_name");
            return 0;
        }
    } ## end else [ if (!RaidSqa_check_plex_state_name...

} ## end sub RaidSqa_plex_destroy

#-------------------------------------------------------------------------------------------------------------------------------------

=head2 RaidSqa_get_one_plex_of_state

RaidSqa_get_one_plex_of_state (api_set_obj=>$api_set_obj, node=>$node, conntype=>$conntype, filer_vol => $filer_vol, plex_state => $plex_state);

This subroutine returns the name of the plex which is in given state.
It returns the name of the plex which is in given state else 0.

The following arguments are accepted:
=over

=item api_set_obj
(Required): An NACL::APISet object

=item conntype
(Optional): Connection type to use. Default is "#ssh".

=item plex_state
(Required) : state of the plex

=item filer_vol
(Required) : Name of the mirrored volume.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_one_plex_of_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR},
                        plex_state => {type => SCALAR},});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_state  = $opts{plex_state};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my @plexes = RaidSqa_get_plex_list(api_set_obj => $api_set_obj,
                                       node        => $node,
                                       conntype    => $conntype,
                                       filer_vol   => $filer_vol);

    my $normal_plex;
    my $resync_plex;
    my $offlined_plex;

    foreach my $plex (@plexes) {
        my $online_state = 0;
        if (RaidSqa_check_plex_state_name(api_set_obj => $api_set_obj,
                                          node        => $node,
                                          conntype    => $conntype,
                                          plex_name   => $plex,
                                          state_name  => 'online',)
          ) {
            $online_state = 1;
        }

        # get list of groups in a specified plex
        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $plex_obj = $raid_obj->plex("$plex");
        if (!$plex_obj) {
            next;
        }
        my $parent_state_resync = $plex_obj->parameter("parent_state_resync");

        if ($online_state == 1 && $parent_state_resync == 0) {
            $normal_plex = $plex;
        } elsif ($online_state == 1 && $parent_state_resync == 1) {
            $resync_plex = $plex;
        } elsif ($online_state == 0) {
            $offlined_plex = $plex;
        }
    } ## end foreach my $plex (@plexes)

    if (uc $plex_state eq "NORMAL") {
        return $normal_plex;
    } elsif (uc $plex_state eq "RESYNC") {
        return $resync_plex;
    } elsif (uc $plex_state eq "OFFLINED") {
        return $offlined_plex;
    } else {
        return 0;
    }
} ## end sub RaidSqa_get_one_plex_of_state

#------------------------------------------------------------------------------

=head2 RaidSqa_get_group_list

RaidSqa_get_group_list(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, node=>$node);

This subroutine by default gets list of raid groups in a given filer. If any one of filer_vol or plex_name is given than it gives list of raid groups in that.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggregate or traditional volume.

=item plex_name

(Optional) : Name of plex.

=item node

(Optional): Node name on which the operation is to be performed

=item hybrid

(Optional): Set this value if aggregate created has to be converted to HyA.Default is 0. Possible values: 1, 0.

=item hybrid_tier

(Optional): Set this value if tests are to be run on particular tier of HyA.Default is HDD. Possible values: SSD, HDD.
=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_group_list {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        plex_name => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        node      => {type => SCALAR,         default => "local", optional => 1},
                        hybrid_tier => {type => SCALAR,         default => "HDD", optional => 1},
                        hybrid      => {type => SCALAR,         default => 0,     optional => 1}});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $node        = $opts{node};
    my $hybrid_tier = lc $opts{hybrid_tier};
    my $hybrid      = $opts{hybrid};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @group_list;
    if ($filer_vol) {
        my $aggr_obj = $raid_obj->aggregate("$filer_vol");

        # Check if the aggr_obj is valid
        if (!($aggr_obj)) {
            return @group_list;
        }
        my %group_list = $aggr_obj->all_raid_groups();
        @group_list = keys(%group_list);
    } elsif ($plex_name) {
        my $plex_obj = $raid_obj->plex("$plex_name");

        # Check if the plex_obj is valid
        if (!($plex_obj)) {
            return @group_list;
        }
        my %group_list = $plex_obj->all_raid_groups();
        @group_list = keys(%group_list);
    } else {
        my %group_list = $raid_obj->all_raid_groups();
        @group_list = keys(%group_list);

    }
    #   Add code to return group list based on hybrid_tier value
    if ($hybrid) {
        my (@group_list_HDD, @group_list_SSD);
        foreach my $group_name (@group_list) {
            my @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                                  node        => $node,
                                                  conntype   => $conntype,
                                                  group_name => $group_name);
            if (@disk_list) {
                my $storage_type = RaidSqa_get_disk_storage_type(api_set_obj => $api_set_obj, node => $node, disk_name => $disk_list[0]);
                if ($storage_type =~ /ssd/i) {
                    push(@group_list_SSD, $group_name);
                } else {
                    push(@group_list_HDD, $group_name);
                }
            } else {
                $Log->comment("$pkg_name: Failed to get disk list from group - $group_name\n");
                return (0);
            }
        } ## end foreach my $group_name (@group_list)
        if ($hybrid_tier eq "ssd") {
            return (@group_list_SSD);
        } else {
            return (@group_list_HDD);
        }
    } else {
        return @group_list;
    }
    #return @group_list;
} ## end sub RaidSqa_get_group_list

#------------------------------------------------------------------------------

=head2 RaidSqa_get_group_context_parity_valid

RaidSqa_get_group_context_parity_valid(api_set_obj=>$api_set_obj, conntype=>$conntype, group_name => $group_name, node=>$node);

This subroutine gets value of flag context_parity_valid of a particular group in volume/aggregate.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of group_name to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_group_context_parity_valid {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $group_obj = $raid_obj->raid_group("$group_name");
    if (!$group_obj) {
        $Log->comment("$pkg_name: Raid group with name $group_name does not exit");
        return 0;
    }
    return $group_obj->parameter("context_parity_protected");
} ## end sub RaidSqa_get_group_context_parity_valid

#------------------------------------------------------------------------------

=head2 RaidSqa_get_group_fsm_state

RaidSqa_get_group_fsm_state(api_set_obj=>$api_set_obj, conntype=>$conntype, group_name=>$group_name, node=>$node);

This subroutine gets fsm_state of a particular raid group.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid_group to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_group_fsm_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $group_obj = $raid_obj->raid_group("$group_name");
    if (!$group_obj) {
        $Log->comment("$pkg_name: Raid group with name $group_name does not exit");
        return 0;
    }
    return $group_obj->parameter("fsm_state");
} ## end sub RaidSqa_get_group_fsm_state

#------------------------------------------------------------------------------

=head2 RaidSqa_get_group_checksum_type

RaidSqa_get_group_checksum_type(api_set_obj=>$api_set_obj, conntype=>$conntype, group_name=>$group_name, node=>$node);

This subroutine gets checksum type of a particular raid group.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid_group to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_group_checksum_type {
    my (%opts) =
      RaidSqa_validate(@_, {
        api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
        node        => {type => SCALAR, default => "local", optional => 1},
        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
        group_name  => {type => SCALAR},
    });
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $group_obj = $raid_obj->raid_group("$group_name");
    if (!$group_obj) {
        $Log->comment("$pkg_name: Raid group with name $group_name does not exit");
        return 0;
    }
    return $group_obj->parameter("checksum_type");
} ## end sub RaidSqa_get_group_checksum_type

#------------------------------------------------------------------------------

=head2 RaidSqa_get_group_list_by_fsm_state

RaidSqa_get_group_list_by_fsm_state(api_set_obj=>$api_set_obj, conntype=>$conntype, fsm_state=>$fsm_state, filer_vol=>$filer_vol, plex_name=>$plex_name, node=>$node);

This subroutine by default gets list of raid groups of particular fsm_state in a given filer. If any one of filer_vol or plex_name is given than it gives list of raid groups of particular fsm_state in that.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item fsm_state

(Required) : Fsm_state of raidgroup, we want the list of.

=item filer_vol

(Optional) : Name of aggregate or traditional volume.

=item plex_name

(Optional) : Name of plex.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_group_list_by_fsm_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        fsm_state => {type => SCALAR},
                        filer_vol => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        plex_name => {type => SCALAR | UNDEF, default => undef, optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $fsm_state   = $opts{fsm_state};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @group_list;
    if ($filer_vol) {
        my $aggr_obj   = $raid_obj->aggregate("$filer_vol");
        my %group_list = $aggr_obj->all_raid_groups();
        @group_list = keys(%group_list);
    } elsif ($plex_name) {
        my $plex_obj   = $raid_obj->plex("$plex_name");
        my %group_list = $plex_obj->all_raid_groups();
        @group_list = keys(%group_list);
    } else {
        my %group_list = $raid_obj->all_raid_groups();
        @group_list = keys(%group_list);

    }
    my $group;
    my $n = 0;
    my @group_fsm_state;
    foreach $group (@group_list) {
        my $group_obj = $raid_obj->raid_group("$group");
        if (!$group_obj) {
            $Log->comment("$pkg_name: raid group with name $group does not exist");
            next;
        }
        if ($group_obj->parameter("fsm_state") eq $fsm_state) {
            $group_fsm_state[$n] = $group;
            $n++;
        }
    } ## end foreach $group (@group_list)
    return @group_fsm_state;
} ## end sub RaidSqa_get_group_list_by_fsm_state

#------------------------------------------------------------------------------

=head2 RaidSqa_get_group_raid_lw_scrub

RaidSqa_get_group_raid_lw_scrub(api_set_obj=>$api_set_obj, conntype=>$conntype, group_name=>$group_name);

This function gets the value for parameter group_raid_lw_scrub
group_raid_lw_scrub represents the LW state of the group.
Possible values: DISABLED, ABORTED, INACTIVE, DONE
DISABLED: Parameter will have this value when aggr wide options is OFF
ABORTED : Parameter will have this value when lw scrub detects non zero gen id on disk
INACTIVE: Parameter will have this value when at least once system wide LW option was turned off and no scrub is run on that group ever since.
DONE    : When LW scrub completes and does not find any gen id inconsistancy, it will mark raid group as DONE and this group will be LW protected

It returns value of group_raid_lw_scrub.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required): Name of raid group for which parameter value to be extracted.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_group_raid_lw_scrub {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $group_obj = $raid_obj->raid_group($group_name);

    if (!defined($group_obj)) {
        $Log->comment("$pkg_name: Group name $group_name does not exist");
        return 0;
    } else {
        my $value = $group_obj->parameter('group_raid_lw_scrub');
        $Log->comment("$pkg_name: group_raid_lw_scrub is set to $value");
        return $value;
    }
} ## end sub RaidSqa_get_group_raid_lw_scrub

#------------------------------------------------------------------------------

=head2 RaidSqa_get_group_recons_suspended_block

RaidSqa_get_group_recons_suspended_block(api_set_obj => $api_set_obj,  conntype=>$conntype, group_name=>$group_name, node => $node);

This subroutine gets recons_suspended_block of a particular raid group.

It returns scalar.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid_group to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_group_recons_suspended_block {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $group_obj = $raid_obj->raid_group("$group_name");
    if (!$group_obj) {
        $Log->comment("$pkg_name: Raid group with name $group_name does not exit");
        return 0;
    }
    return $group_obj->parameter("recons_suspended_block");
} ## end sub RaidSqa_get_group_recons_suspended_block

#------------------------------------------------------------------------------

=head2 RaidSqa_get_group_scrub_suspended_block

RaidSqa_get_group_scrub_suspended_block(api_set_obj=>$api_set_obj, node=>$node, conntype=>$conntype, group_name=>$group_name, node=>$node);

This subroutine returns suspended block number for a group

It returns value of parameter scrub_suspended_block for a given group_name
It returns -1 if scrub is not suspended
It returns 0 if group does not exist

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid group whose parameter value is to be returned

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_group_scrub_suspended_block {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check a test group exists
    if (!RaidSqa_check_group_exist(api_set_obj => $api_set_obj,
                                   conntype    => $conntype,
                                   group_name  => $group_name,
                                   node        => $node)
      ) {
        $Log->comment("$pkg_name: Raid group $group_name does not exist");
        return 0;
    } else {
        my $raid_obj  = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $group_obj = $raid_obj->raid_group("$group_name");
        if (!$group_obj) {
            $Log->comment("$pkg_name: Raid group with name $group_name does not exit");
            return 0;
        }
        return $group_obj->parameter("scrub_suspended_block");
    }
} ## end sub RaidSqa_get_group_scrub_suspended_block

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_list

RaidSqa_get_disk_list(api_set_obj=>$api_set_obj, conntype=>$conntype, spare_only=>$spare_only, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name);

This subroutine by default gets list of all disks recognized by raid in a given filer. By specifing any one of the options from spare_only, filer_vol, plex_name or group_name, we can get the disk list in that particular raid object.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item spare_only

(Optional): Its values are "yes" or "no". If "yes" than provide spare disks only.

=item filer_vol

(Optional): Name of aggr/volume of which we want to get the disk list.

=item plex_name

(Optional): Name of plex of which we want to get the disk list.

=item group_name

(Optional): Name of raid group of which we want to get the disk list.

=item hybrid

(Optional): Set this value if aggregate created has to be converted to HyA.Default is 0. Possible values: 1, 0.

=item hybrid_tier

(Optional): Set this value if tests are to be run on particular tier of HyA.Default is HDD. Possible values: SSD, HDD.

=item storage_type

(Optional) : Name of storage_type to get the list of disk. Possible values are ATA, FCAL, SAS, BSAS, MSATA, FSAS.
=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_list {
    my (%opts) =
      RaidSqa_validate(
        @_,
           {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
            node       => {type => SCALAR,         default => "local", optional => 1},
             conntype   => {type => SCALAR, default => "#ssh", optional => 1},
             spare_only => {type => SCALAR, default => "no", optional => 1},
             filer_vol   => {type => SCALAR | UNDEF, default => undef, optional => 1},
             plex_name   => {type => SCALAR | UNDEF, default => undef, optional => 1},
             group_name  => {type => SCALAR | UNDEF, default => undef, optional => 1},
             hybrid      => {type => SCALAR,         default => 0,     optional => 1},
             hybrid_tier => {type => SCALAR,         default => "HDD", optional => 1},
             desirable_layout => {type => SCALAR,    default => 0,     optional => 1},
             storage_type     => {type => SCALAR | UNDEF, default => undef, optional => 1},
             bps              => {type => SCALAR | UNDEF, default => undef, optional => 1},
             chan_name        => {type => SCALAR | UNDEF, default => undef, optional => 1},
             checksum_type    => {type => SCALAR | UNDEF, default => undef, optional => 1},
             copy_progress    => {type => SCALAR | UNDEF, default => undef, optional => 1},
             copy_target_id   => {type => SCALAR | UNDEF, default => undef, optional => 1},
             disk_rgdn        => {type => SCALAR | UNDEF, default => undef, optional => 1},
             disk_type        => {type => SCALAR | UNDEF, default => undef, optional => 1},
             fsm_state        => {type => SCALAR | UNDEF, default => undef, optional => 1},
             host_adapter     => {type => SCALAR | UNDEF, default => undef, optional => 1},
             optimized_recons => {type => SCALAR | UNDEF, default => undef, optional => 1},
             parent_state     => {type => SCALAR | UNDEF, default => undef, optional => 1},
             path_name1       => {type => SCALAR | UNDEF, default => undef, optional => 1},
             path_name2       => {type => SCALAR | UNDEF, default => undef, optional => 1},
             pool             => {type => SCALAR | UNDEF, default => undef, optional => 1},
             prefail          => {type => SCALAR | UNDEF, default => undef, optional => 1},
             raid_name        => {type => SCALAR | UNDEF, default => undef, optional => 1},
             raid_type        => {type => SCALAR | UNDEF, default => undef, optional => 1},
             replace          => {type => SCALAR | UNDEF, default => undef, optional => 1},
             rpm              => {type => SCALAR | UNDEF, default => undef, optional => 1},
             serialno         => {type => SCALAR | UNDEF, default => undef, optional => 1},
             shelf            => {type => SCALAR | UNDEF, default => undef, optional => 1},
             vendor           => {type => SCALAR | UNDEF, default => undef, optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $spare_only  = $opts{spare_only};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $storage_type = $opts{storage_type};
    my $node        = $opts{node};
    my $hybrid      = $opts{hybrid};
    my $hybrid_tier = lc $opts{hybrid_tier};
    my $desirable_layout = $opts{desirable_layout};

    my %temp = %opts;

    my ($host, $conn, $robj);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @disk_list;
    if ($spare_only =~ /yes/i) {
        my %all_spare_disks = $raid_obj->spare_disks();
        my @spare_disks     = keys(%all_spare_disks);
        if ($desirable_layout) {
            my %carrier_disk_mapping;
            foreach my $disk_name (@spare_disks) {
                my $disk_object = $raid_obj->disk($disk_name);
                next if (!defined($disk_object));
                if (!$disk_object->parameter("is-multidisk-carrier")) {
                    next;
                }
                my $carrier = $disk_object->parameter("carrier");
                if (exists($carrier_disk_mapping{$carrier})) {
                    next;
                } else {
                    $carrier_disk_mapping{$carrier} = $disk_name;
                }
            } ## end foreach my $disk_name (@spare_disks)
            @disk_list = (values(%carrier_disk_mapping));
        } else {
            @disk_list = @spare_disks;
        }
    } else {
        if ($filer_vol) {
            $robj = $raid_obj->aggregate($filer_vol);
            if (!$robj) {
                $robj = $raid_obj->volume($filer_vol);
            }
            $Log->comment("$pkg_name: Aggr $filer_vol does not exist") if (!$robj);
            return 0 if (!$robj);
        } elsif ($plex_name) {
            $robj = $raid_obj->plex($plex_name);
            $Log->comment("$pkg_name: Plex $filer_vol does not exist") if (!$robj);
            return 0 if (!$robj);
        } elsif ($group_name) {
            $robj = $raid_obj->raid_group($group_name);
            $Log->comment("$pkg_name: Raid group $group_name does not exist") if (!$robj);
            return 0 if (!$robj);
        } else {
            $robj = $raid_obj;
            $Log->comment("$pkg_name: Could not create RaidInfo object") if (!$robj);
        }
        my %disk_list = $robj->all_disks();
        @disk_list = keys(%disk_list);
        #return @disk_list;
    } ## end else [ if ($spare_only =~ /yes/i)
    if(defined $storage_type) {
        my $a = 0;
        my @disk_storage_type;
        foreach my $disk (@disk_list) {
            my $disk_obj = $raid_obj->disk("$disk");
            if ($disk_obj->parameter("disk_type") eq $storage_type) {
                    $disk_storage_type[$a] = $disk;
                    $a++;
            }
        } #end of for
        #assign new disk list
        @disk_list = @disk_storage_type;
    }
    foreach ('api_set_obj', 'node', 'conntype', 'plex_name', 'group_name', 'spare_only', 'filer_vol', 'hybrid', 'hybrid_tier', 'storage_type', 'desirable_layout') {
        delete($temp{$_});
    }

    # Filter disk list based on attributes passed to the function.
    my @attributes = keys(%temp);
    my @new_disk_list;
    foreach my $attr (@attributes) {
        if (defined($temp{$attr})) {
            foreach my $disk_name (@disk_list) {
                my $disk_obj = $raid_obj->disk("$disk_name");
                if ($disk_obj->parameter("$attr") eq $temp{$attr}) {
                    push(@new_disk_list, $disk_name);
                }
            }
            @disk_list     = @new_disk_list;
            @new_disk_list = ();
        } else {
            next;
        }
    } ## end foreach my $attr (@attributes)
    if ($hybrid) {
        my (@disk_list_SSD, @disk_list_HDD);
        if (@disk_list) {
            foreach my $disk_name (@disk_list) {
                my $storage_type = RaidSqa_get_disk_storage_type(api_set_obj => $api_set_obj, node => $node,conntype => $conntype, disk_name => $disk_name);
                if (lc($storage_type) =~ /ssd/i) {
                    push(@disk_list_SSD, $disk_name);
                } else {
                    push(@disk_list_HDD, $disk_name);
                }
            }
            if ($hybrid_tier eq "ssd") {
                return (@disk_list_SSD);
            } else {
                return (@disk_list_HDD);
            }
        } ## end if (@disk_list)
        else {
            return (@disk_list);
        }
    } else {
        return (@disk_list);
    }
} ## end sub RaidSqa_get_disk_list

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_list_by_fsm_state

RaidSqa_get_disk_list_by_fsm_state(api_set_obj=>$api_set_obj, conntype=>$conntype, fsm_state =>$fsm_state, spare_only=>$spare_only, disk_list=>$disk_list, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, node=>$node);

This subroutine by default gets list of disks of a given fsm_state in a given filer. By specifing any one of the options from disk_list, spare_only, filer_vol, plex_name or group_name, we can get the disk list of a given fsm_state from that particular raid object.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item fsm_state

(Required) : Name of fsm_state to get the list of disk.  Possible values are OFFLINE, INUSE, RECONS and more

=item spare_only

(Optional): Its values are "yes" or "no". If "yes" than provide spare disks only.

=item disk_list

(Optional) : List of disks from which need to get disks of a given fsm_state.

=item filer_vol

(Optional): Name of aggr/volume of which we want to get the disk list.

=item plex_name

(Optional): Name of plex of which we want to get the disk list.

=item group_name

(Optional): Name of raid group of which we want to get the disk list.

=item node

(Optional): Node name on which the operation is to be performed

=item storage_type

(Optional) : Name of storage_type to get the list of disk. Possible values are ATA, FCAL, SAS, BSAS, MSATA, FSAS.
=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_list_by_fsm_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        fsm_state  => {type => SCALAR},
                        spare_only => {type => SCALAR, default => "no", optional => 1},
                        disk_list  => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        storage_type     => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        node       => {type => SCALAR, default => "local", optional => 1},
                        desirable_layout => {type => SCALAR, default => 0, optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $fsm_state   = $opts{fsm_state};
    my $spare_only  = $opts{spare_only};
    my $disk_list   = $opts{disk_list};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $storage_type = $opts{storage_type};
    my $group_name  = $opts{group_name};
    my $node        = $opts{node};
    my $desirable_layout = $opts{desirable_layout};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @disk_list;
    my @disk_fsm_state;

    if (!$disk_list) {
        @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           spare_only  => $spare_only,
                                           filer_vol   => $filer_vol,
                                           plex_name   => $plex_name,
                                           group_name  => $group_name,
                                           node        => $node,
                                           desirable_layout => $desirable_layout);
    } else {
        @disk_list = @$disk_list;
    }

    my $a = 0;
    my $disk;
    @disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disk_list);
    foreach $disk (@disk_list) {
        my $disk_obj = $raid_obj->disk("$disk");
        if ($disk_obj->parameter("fsm_state") eq $fsm_state) {
            $disk_fsm_state[$a] = $disk;
            $a++;
        }
    } ## end foreach $disk (@disk_list)
    if(defined $storage_type) {
        my $a = 0;
        my @disk_storage_type;
        foreach my $disk (@disk_fsm_state) {
            my $disk_obj = $raid_obj->disk("$disk");
            if ($disk_obj->parameter("disk_type") eq $storage_type) {
                    $disk_storage_type[$a] = $disk;
                    $a++;

            }
        } #end of for
        #assign new disk list
        @disk_fsm_state = @disk_storage_type;
    }
    return @disk_fsm_state;
} ## end sub RaidSqa_get_disk_list_by_fsm_state

#--------------------------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_list_by_parent_state

RaidSqa_get_disk_list_by_parent_state(api_set_obj => $api_set_obj, conntype=>$conntype, parent_state =>$parent_state, spare_only=>$spare_only, disk_list=>$disk_list, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, node => $node);

This subroutine by default gets list of disks of a given parent_state in a given filer. By specifing any one of the options from disk_list, spare_only, filer_vol, plex_name or group_name, we can get the disk list of a given parent_state from that particular raid object.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item fsm_state

(Required) : Name of fsm_state to get the list of disk.  Possible values are OFFLINE, INUSE, RECONS and more

=item spare_only

(Optional): Its values are "yes" or "no". If "yes" than provide spare disks only.

=item disk_list

(Optional) : List of disks from which need to get disks of a given fsm_state.

=item filer_vol

(Optional): Name of aggr/volume of which we want to get the disk list.

=item plex_name

(Optional): Name of plex of which we want to get the disk list.

=item group_name

(Optional): Name of raid group of which we want to get the disk list.

=item node

(Optional): Node name on which the operation is to be performed

=item hybrid

(Optional): Set this value if aggregate created has to be converted to HyA.Default is 0. Possible values: 1, 0.

=item hybrid_tier

(Optional): Set this value if tests are to be run on particular tier of HyA.Default is HDD. Possible values: SSD, HDD.

=item storage_type

(Optional) : Name of storage_type to get the list of disk. Possible values are ATA, FCAL, SAS, BSAS, MSATA, FSAS.

=item storage_type

(Optional) : Name of storage_type to get the list of disk. Possible values are ATA, FCAL, SAS, BSAS, MSATA, FSAS.


=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_list_by_parent_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        parent_state => {type => SCALAR},
                        spare_only   => {type => SCALAR, default => "no", optional => 1},
                        disk_list    => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        filer_vol    => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        plex_name    => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        hybrid       => {type => SCALAR, default => 0, optional => 1},
                        hybrid_tier  => {type => SCALAR, default => "HDD", optional => 1},
                        group_name   => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        storage_type     => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        node         => {type => SCALAR, default => "local", optional => 1},
                        desirable_layout => {type => SCALAR, default => 0, optional => 1},});
    my $api_set_obj  = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $parent_state = $opts{parent_state};
    my $spare_only   = $opts{spare_only};
    my $disk_list    = $opts{disk_list};
    my $filer_vol    = $opts{filer_vol};
    my $plex_name    = $opts{plex_name};
    my $group_name   = $opts{group_name};
    my $storage_type = $opts{storage_type};
    my $node         = $opts{node};
    my $hybrid       = $opts{hybrid};
    my $hybrid_tier  = $opts{hybrid_tier};
    my $desirable_layout = $opts{desirable_layout};
    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @disk_list;
    my @disk_parent_state;

    if (!$disk_list) {
        @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           spare_only  => $spare_only,
                                           filer_vol   => $filer_vol,
                                           plex_name   => $plex_name,
                                           group_name  => $group_name,
                                           hybrid      => $hybrid,
                                           hybrid_tier => $hybrid_tier,
                                           node        => $node,
                                           desirable_layout => $desirable_layout);
    } else {
        @disk_list = @$disk_list;
    }

    my $a = 0;
    my $disk;
    @disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disk_list);
    foreach $disk (@disk_list) {
        my $disk_obj = $raid_obj->disk("$disk");

        if (!$disk_obj) {
            $Log->comment("$pkg_name: Disk with name $disk does not exist");
            next;
        }
        if ($disk_obj->parameter("parent_state") =~ "$parent_state") {
            $disk_parent_state[$a] = $disk;
            $a++;
        }
    } ## end foreach $disk (@disk_list)
    if(defined $storage_type) {
        my $a = 0;
        my @disk_storage_type;
        foreach my $disk (@disk_parent_state) {
            my $disk_obj = $raid_obj->disk("$disk");
            if ($disk_obj->parameter("disk_type") eq $storage_type) {
                    $disk_storage_type[$a] = $disk;
                    $a++;

            }
        } #end of for
        #assign new disk list
        @disk_parent_state = @disk_storage_type;
    }
    return @disk_parent_state;
} ## end sub RaidSqa_get_disk_list_by_parent_state

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_list_by_raid_type

RaidSqa_get_disk_list_by_raid_type(api_set_obj => $api_set_obj, conntype=>$conntype, raid_type =>$raid_type, spare_only=>$spare_only, disk_list=>$disk_list, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, node => $node);

This subroutine by default gets list of disks of a given raid_type in a given filer. By specifing any one of the options from disk_list, spare_only, filer_vol, plex_name or group_name, we can get the disk list of a given raid_type from that particular raid object.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item raid_type

(Required) : Name of raid_type to get the list of disk. Possible values are data, parity, dparity

=item spare_only

(Optional): Its values are "yes" or "no". If "yes" than provide spare disks only.

=item disk_list

(Optional) : List of disks from which need to get disks of a given fsm_state.

=item filer_vol

(Optional): Name of aggr/volume of which we want to get the disk list.

=item plex_name

(Optional): Name of plex of which we want to get the disk list.

=item group_name

(Optional): Name of raid group of which we want to get the disk list.

=item node

(Optional): Node name on which the operation is to be performed

=item hybrid

(Optional): Set this value if aggregate created has to be converted to HyA.Default is 0. Possible values: 1, 0.

=item hybrid_tier

(Optional): Set this value if tests are to be run on particular tier of HyA.Default is HDD. Possible values: SSD, HDD.

=item storage_type

(Optional) : Name of storage_type to get the list of disk. Possible values are ATA, FCAL, SAS, BSAS, MSATA, FSAS.
=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_list_by_raid_type {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        raid_type  => {type => SCALAR},
                        spare_only => {type => SCALAR, default => "no", optional => 1},
                        disk_list  => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        hybrid      => {type => SCALAR, default => 0, optional => 1},
                        hybrid_tier => {type => SCALAR, default => "HDD", optional => 1},
                        storage_type     => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        desirable_layout => {type => SCALAR, default => 0, optional => 1},
                        node       => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $raid_type   = $opts{raid_type};
    my $spare_only  = $opts{spare_only};
    my $disk_list   = $opts{disk_list};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $storage_type = $opts{storage_type};
    my $node        = $opts{node};
    my $hybrid      = $opts{hybrid};
    my $hybrid_tier = $opts{hybrid_tier};
    my $desirable_layout = $opts{desirable_layout};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @disk_list;
    my @disk_raid_type;
    if (!defined($node)) {
        $node = "local";
    }

    if (!$disk_list) {
        @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           spare_only  => $spare_only,
                                           filer_vol   => $filer_vol,
                                           plex_name   => $plex_name,
                                           group_name  => $group_name,
                                           hybrid      => $hybrid,
                                           hybrid_tier => $hybrid_tier,
                                           node        => $node,
                                           desirable_layout => $desirable_layout);
    } else {
        @disk_list = @$disk_list;
    }

    my $a = 0;
    my $disk;
    @disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disk_list);
    foreach $disk (@disk_list) {
        my $disk_obj = $raid_obj->disk("$disk");

        if (!$disk_obj) {
            $Log->comment("$pkg_name: Disk with name $disk does not exist");
            next;
        }
        if ($disk_obj->parameter("raid_type") eq $raid_type) {
            $disk_raid_type[$a] = $disk;
            $a++;
        }
    } ## end foreach $disk (@disk_list)
    if(defined $storage_type) {
        my $a = 0;
        my @disk_storage_type;
        foreach my $disk (@disk_raid_type) {
            my $disk_obj = $raid_obj->disk("$disk");
            if ($disk_obj->parameter("disk_type") eq $storage_type) {
                    $disk_storage_type[$a] = $disk;
                    $a++;
            }
        } #end of for
        #assign new disk list
        @disk_raid_type = @disk_storage_type;
    }

    return @disk_raid_type;
} ## end sub RaidSqa_get_disk_list_by_raid_type

#----------------------------------------------------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_list_by_storage_type

RaidSqa_get_disk_list_by_storage_type(api_set_obj=>$api_set_obj, conntype=>$conntype, storage_type =>$storage_type, spare_only=>$spare_only, disk_list=>$disk_list, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name);

This subroutine by default gets list of disks of a given storage_type in a given filer. By specifing any one of the options from disk_list, spare_only, filer_vol, plex_name or group_name, we can get the disk list of a given storage_type from that particular raid object.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item storage_type

(Required) : Name of storage_type to get the list of disk. Possible values are ATA, FCAL, SAS, BSAS, FSAS.

=item spare_only

(Optional): Its values are "yes" or "no". If "yes" than provide spare disks only.

=item disk_list

(Optional) : List of disks from which need to get disks of a given fsm_state.

=item filer_vol

(Optional): Name of aggr/volume of which we want to get the disk list.

=item plex_name

(Optional): Name of plex of which we want to get the disk list.

=item group_name

(Optional): Name of raid group of which we want to get the disk list.


=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_list_by_storage_type {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        storage_type => {type => SCALAR},
                        spare_only   => {type => SCALAR, default => "no", optional => 1},
                        disk_list    => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        filer_vol    => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        plex_name    => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        group_name   => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        desirable_layout => {type => SCALAR, default => 0, optional => 1},
                        node         => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj  = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $storage_type = $opts{storage_type};
    my $spare_only   = $opts{spare_only};
    my $disk_list    = $opts{disk_list};
    my $filer_vol    = $opts{filer_vol};
    my $plex_name    = $opts{plex_name};
    my $group_name   = $opts{group_name};
    my $node         = $opts{node};
    my $desirable_layout = $opts{desirable_layout};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @disk_list;
    my @disk_storage_type = ();

    if (!$disk_list) {
        @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           spare_only  => $spare_only,
                                           filer_vol   => $filer_vol,
                                           plex_name   => $plex_name,
                                           group_name  => $group_name,
                                           desirable_layout => $desirable_layout,
                                           node        => $node);
    } else {
        @disk_list = @$disk_list;
    }

    my $disk;
    @disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disk_list);
    foreach $disk (@disk_list) {
        my $disk_obj = $raid_obj->disk("$disk");

        if (!$disk_obj) {
            $Log->comment("$pkg_name: Disk with name $disk does not exist");
            next;
        }
        if ($disk_obj->parameter("disk_type") eq $storage_type) {
            push(@disk_storage_type, $disk);
        }
    } ## end foreach $disk (@disk_list)
    return @disk_storage_type;
} ## end sub RaidSqa_get_disk_list_by_storage_type

#----------------------------------------------------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_list_by_disk_type

RaidSqa_get_disk_list_by_disk_type(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_type =>$disk_type, spare_only=>$spare_only, disk_list=>$disk_list, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name);

This subroutine by default gets list of disks of a given disk_type in a given filer. By specifing any one of the options from disk_list, spare_only, filer_vol, plex_name or group_name, we can get the disk list of a given disk_type from that particular raid object.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_type

(Required) : Name of disk_type to get the list of disk. Possible values are ATA, FCAL, SAS, BSAS, SSD, FSAS.

=item spare_only

(Optional): Its values are "yes" or "no". If "yes" than provide spare disks only.

=item disk_list

(Optional) : List of disks from which need to get disks of a given fsm_state.

=item filer_vol

(Optional): Name of aggr/volume of which we want to get the disk list.

=item plex_name

(Optional): Name of plex of which we want to get the disk list.

=item group_name

(Optional): Name of raid group of which we want to get the disk list.


=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_list_by_disk_type {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_type => {type => SCALAR},
                        spare_only   => {type => SCALAR, default => "no", optional => 1},
                        disk_list    => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        filer_vol    => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        plex_name    => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        group_name   => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        desirable_layout => {type => SCALAR, default => 0, optional => 1},
                        node         => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj  = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $disk_type = $opts{disk_type};
    my $spare_only   = $opts{spare_only};
    my $disk_list    = $opts{disk_list};
    my $filer_vol    = $opts{filer_vol};
    my $plex_name    = $opts{plex_name};
    my $group_name   = $opts{group_name};
    my $node         = $opts{node};
    my $desirable_layout = $opts{desirable_layout};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @disk_list;
    my @disk_list_by_disk_type;


    if (!$disk_list) {
        @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           spare_only  => $spare_only,
                                           filer_vol   => $filer_vol,
                                           plex_name   => $plex_name,
                                           group_name  => $group_name,
                                           desirable_layout => $desirable_layout,
                                           node        => $node);
    } else {
        @disk_list = @$disk_list;
    }

    my $disk;
    @disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disk_list);
    foreach $disk (@disk_list) {
        my $disk_obj = $raid_obj->disk("$disk");

        if (!$disk_obj) {
            $Log->comment("$pkg_name: Disk with name $disk does not exist");
            next;
        }
        if ($disk_obj->parameter("disk_type") eq $disk_type) {
            push(@disk_list_by_disk_type, $disk);
        }
    } ## end foreach $disk (@disk_list)
    return @disk_list_by_disk_type;
} ## end sub RaidSqa_get_disk_list_by_disk_type

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_list_by_checksum_type

RaidSqa_get_disk_list_by_checksum_type(api_set_obj=>$api_set_obj, conntype=>$conntype, checksum_type =>$checksum_type, pool_name=>$pool_name, spare_only=>$spare_only, disk_list=>$disk_list, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, node=>$node);

This subroutine by default gets list of disks of a given checksum type in a given filer. By specifing any one of the options from disk_list, pool_name, spare_only, filer_vol, plex_name or group_name, we can get the disk list of a given checksum_type from that particular raid object.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item checksum_type

(Optional) : Name of the required checksumtype, to get the list of disk.
Possible values are block, zoned.

(Optional) : bps value of the required checksumtype, to get the list of disk.
Possible value is 520 or 512.

= item pool_name

(Optional): Name of the spare pool from which to get the list of given checksum_type disks.

=item spare_only

(Optional): Its values are "yes" or "no". If "yes" than provide spare disks of given checksum_type only.

=item disk_list

(Optional) : List of disks from which need to get disks of a given checksum_type.

=item filer_vol

(Optional): Name of aggr/volume of which we want to get the disk list of given checksum_type.

=item plex_name

(Optional): Name of plex of which we want to get the disk list of given checksum_type .

=item group_name

(Optional): Name of raid group of which we want to get the disk list of given checksum_type.

=item node

(Optional): Node name on which the operation is to be performed

=item storage_type

(Optional) : Name of storage_type to get the list of disk. Possible values are ATA, FCAL, SAS, BSAS, MSATA, FSAS.
=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_list_by_checksum_type {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj   => {type => OBJECT, isa      => 'NACL::APISet'},
                        node          => {type => SCALAR, default  => "local", optional => 1},
                        conntype      => {type => SCALAR, default  => "#ssh", optional => 1},
                        checksum_type => {type => SCALAR, optional => 1},
                        bps           => {type => SCALAR, optional => 1},
                        pool_name     => {type => SCALAR, default  => undef, optional => 1},
                        spare_only    => {type => SCALAR, default  => "no", optional => 1},
                        disk_list  => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF,   default => undef, optional => 1},
                        plex_name  => {type => SCALAR | UNDEF,   default => undef, optional => 1},
                        storage_type     => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        desirable_layout => {type => SCALAR, default => 0, optional => 1},
                        group_name => {type => SCALAR | UNDEF,   default => undef, optional => 1},}
                      );
    my $api_set_obj   = $opts{api_set_obj};
    my $node          = $opts{node};
    my $conntype      = $opts{conntype};
    my $checksum_type = $opts{checksum_type};
    my $bps           = $opts{bps};
    my $pool_name     = $opts{pool_name};
    my $spare_only    = $opts{spare_only};
    my $disk_list     = $opts{disk_list};
    my $filer_vol     = $opts{filer_vol};
    my $plex_name     = $opts{plex_name};
    my $storage_type = $opts{storage_type};
    my $group_name    = $opts{group_name};
    my $desirable_layout = $opts{desirable_layout};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @disk_list;
    my @disk_checksum_type = ();
    my %cksum_bps_mapping  = ();
    $cksum_bps_mapping{"zoned"}          = 512;
    $cksum_bps_mapping{"block"}          = 520;
    $cksum_bps_mapping{"advanced_zoned"} = 512;

    if ($checksum_type !~ /^(block|zoned|advanced_zoned)$/) {
        $Log->comment("$pkg_name: $checksum_type is unknown checksum type");
        return @disk_checksum_type;
    }

    my $bps_required = $cksum_bps_mapping{$checksum_type};
    if (!$disk_list) {
        @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           spare_only  => $spare_only,
                                           filer_vol   => $filer_vol,
                                           plex_name   => $plex_name,
                                           group_name  => $group_name,
                                           desirable_layout => $desirable_layout,
                                           node        => $node);
    } else {
        @disk_list = @$disk_list;
    }

    my ($parameter_name, $parameter_value);
    my $disk_obj = $raid_obj->disk("$disk_list[0]");
    if (!$disk_obj) {
         $disk_list[0] = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_name => $disk_list[0]);
         $disk_obj = $raid_obj->disk("$disk_list[0]");
    }

    if(defined($disk_obj)) {
        if($disk_obj->parameter("checksum_type")) {
            $parameter_name = "checksum_type";
            $parameter_value = "$checksum_type";
        } elsif($disk_obj->parameter("bps")) {
            $parameter_name = "bps";
            $parameter_value = $bps_required;
        }
    } else {
        $Log->comment("$pkg_name: Disk with name $disk_list[0] does not exist");
        return 0;
    }

    my $a = 0;
    my $disk;
    if ($pool_name) {
        my $b = 0;
        my @disks_pool0_checksum_type;
        my @disks_pool1_checksum_type;
        @disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disk_list);
        foreach $disk (@disk_list) {
            my $disk_obj = $raid_obj->disk("$disk");

            if (!$disk_obj) {
                $Log->comment("$pkg_name: Disk with name $disk does not exist");
                next;
            }
            if ($disk_obj->parameter($parameter_name) eq $parameter_value) {
                if ($disk_obj->parameter("pool") eq "0") {
                    $disks_pool0_checksum_type[$a] = $disk;
                    $a++;
                } elsif ($disk_obj->parameter("pool") eq "1") {
                    $disks_pool1_checksum_type[$b] = $disk;
                    $b++;
                }
            }
        } ## end foreach $disk (@disk_list)

        if ($pool_name =~ /pool0|Pool0|POOL0/) {
            return @disks_pool0_checksum_type;
        } elsif ($pool_name =~ /pool1|Pool1|POOL1/) {
            return @disks_pool1_checksum_type;
        }
    } else {
        @disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disk_list);
        foreach $disk (@disk_list) {
            my $disk_obj = $raid_obj->disk("$disk");

            if (!$disk_obj) {
                $Log->comment("$pkg_name: Disk with name $disk does not exist");
                next;
            }
            if ($disk_obj->parameter($parameter_name) eq $parameter_value) {
                $disk_checksum_type[$a] = $disk;
                $a++;
            }
        } ## end foreach $disk (@disk_list)

        if(defined $storage_type) {
            my $a = 0;
            my @disk_storage_type;
            foreach my $disk (@disk_checksum_type) {
                my $disk_obj = $raid_obj->disk("$disk");
                if ($disk_obj->parameter("disk_type") eq $storage_type) {
                        $disk_storage_type[$a] = $disk;
                        $a++;

                }
            } #end of for
            #assign new disk list
            $Log->comment("$pkg_name: Disks after filtering by storage type - @disk_storage_type");
            @disk_checksum_type = @disk_storage_type;
        }
        return @disk_checksum_type;
    } ## end else [ if ($pool_name)
} ## end sub RaidSqa_get_disk_list_by_checksum_type

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_list_by_pool

RaidSqa_get_disk_list_by_pool(api_set_obj=>$api_set_obj, conntype=>$conntype, pool_name =>$pool_name, spare_only=>$spare_only, disk_list=>$disk_list, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, node=>$node);

This subroutine by default gets list of disks in a given pool recognized by raid in a given filer. By specifing any one of the options from disk_list, spare_only, filer_vol, plex_name or group_name, we can get the disk list in a given pool from that particular raid object.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item pool_name

(Required) : Name of pool to get the list of disks. Options are "pool0" or "pool1".

=item spare_only

(Optional): Its values are "yes" or "no". If "yes" than provide spare disks only.

=item disk_list

(Optional) : List of disks from which need to get disks from a pool.

=item filer_vol

(Optional): Name of aggr/volume of which we want to get the disk list.

=item plex_name

(Optional): Name of plex of which we want to get the disk list.

=item group_name

(Optional): Name of raid group of which we want to get the disk list.

=item node

(Optional): Node name on which the operation is to be performed

=item storage_type

(Optional) : Name of storage_type to get the list of disk. Possible values are ATA, FCAL, SAS, BSAS, MSATA, FSAS.
=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_list_by_pool {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        pool_name  => {type => SCALAR},
                        spare_only => {type => SCALAR, default => "no", optional => 1},
                        disk_list  => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        storage_type     => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        desirable_layout => {type => SCALAR, default => 0, optional => 1},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $pool_name   = $opts{pool_name};
    my $spare_only  = $opts{spare_only};
    my $disk_list   = $opts{disk_list};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $storage_type = $opts{storage_type};
    my $desirable_layout = $opts{desirable_layout};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @disk_list;

    if (!$disk_list) {
        @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           spare_only  => $spare_only,
                                           filer_vol   => $filer_vol,
                                           plex_name   => $plex_name,
                                           group_name  => $group_name,
                                           desirable_layout => $desirable_layout);
    } else {
        @disk_list = @$disk_list;
    }

    my $a = 0;
    my $b = 0;
    my ($disk, @disks_pool0, @disks_pool1);
    @disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disk_list);
    foreach $disk (@disk_list) {
        my $disk_obj = $raid_obj->disk("$disk");

        if (!$disk_obj) {
            $Log->comment("$pkg_name: Disk with name $disk does not exist");
            next;
        }
        if ($disk_obj->parameter("pool") eq "0") {
            $disks_pool0[$a] = $disk;
            $a++;
        } elsif ($disk_obj->parameter("pool") eq "1") {
            $disks_pool1[$b] = $disk;
            $b++;
        }
    } ## end foreach $disk (@disk_list)
    if(defined $storage_type) {
        my $a = 0;
        my @disk_storage_type;
        my @disk_list;
        if ($pool_name =~ /pool0|Pool0|POOL0/) {
            @disk_list = @disks_pool0;
        } elsif ($pool_name =~ /pool1|Pool1|POOL1/) {
            @disk_list = @disks_pool1;
        }
        foreach $disk (@disk_list) {
            my $disk_obj = $raid_obj->disk("$disk");
            if ($disk_obj->parameter("disk_type") eq $storage_type) {
                    $disk_storage_type[$a] = $disk;
                    $a++;
            }
        } #end of for
        #assign new disk list
        if ($pool_name =~ /pool0|Pool0|POOL0/) {
             @disks_pool0 = @disk_storage_type;
        } elsif ($pool_name =~ /pool1|Pool1|POOL1/) {
             @disks_pool1 = @disk_storage_type;
        }
    }

    if ($pool_name =~ /pool0/i) {
        return @disks_pool0;
    } elsif ($pool_name =~ /pool1/i) {
        return @disks_pool1;
    }
} ## end sub RaidSqa_get_disk_list_by_pool

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_list_by_rpm

RaidSqa_get_disk_list_by_rpm(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_rpm =>$disk_rpm, spare_only=>$spare_only, disk_list=>$disk_list, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, node=>$node);

This subroutine by default gets list of disks of a given disk_rpm in a given filer. By specifing any one of the options from disk_list, spare_only, filer_vol, plex_name or group_name, we can get the disk list of a given disk_rpm from that particular raid object.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_rpm

(Required) : Value of disk_rpm to get the list of disk. Possible values are 7200, 10000, 15000.

=item spare_only

(Optional): Its values are "yes" or "no". If "yes" than provide spare disks only.

=item disk_list

(Optional) : List of disks from which need to get disks of a given fsm_state.

=item filer_vol

(Optional): Name of aggr/volume of which we want to get the disk list.

=item plex_name

(Optional): Name of plex of which we want to get the disk list.

=item group_name

(Optional): Name of raid group of which we want to get the disk list.


=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_list_by_rpm {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_rpm   => {type => SCALAR},
                        spare_only => {type => SCALAR, default => "no", optional => 1},
                        disk_list  => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        node       => {type => SCALAR, default => "local", optional => 1},
                        storage_type     => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        desirable_layout => {type => SCALAR, default => 0, optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef, optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $disk_rpm    = $opts{disk_rpm};
    my $spare_only  = $opts{spare_only};
    my $disk_list   = $opts{disk_list};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $storage_type = $opts{storage_type};
    my $desirable_layout = $opts{desirable_layout};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @disk_list;
    my @list_disk_rpm = ();

    if (!$disk_list) {
        @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           spare_only  => $spare_only,
                                           filer_vol   => $filer_vol,
                                           plex_name   => $plex_name,
                                           group_name  => $group_name,
                                           desirable_layout => $desirable_layout,
                                           node        => $node);
    } else {
        @disk_list = @$disk_list;
    }
    @disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disk_list);
    foreach my $disk (@disk_list) {

        if ($disk) {
            my $disk_obj = $raid_obj->disk("$disk");

            if (!$disk_obj) {
                $Log->comment("$pkg_name: Disk with name $disk does not exist");
                next;
            }
            #remove leading space which is present for rpm 7200 and 5400
            my $diskrpm_var = $disk_obj->parameter("rpm");
            $diskrpm_var =~ s/^\s//;
            if ($diskrpm_var eq $disk_rpm) {
                push(@list_disk_rpm, $disk);
            }
        } else {
            $Log->comment("$pkg_name: The disk_name $disk does not exist");
            return (0);
        }
    } ## end foreach my $disk (@disk_list)
    if(defined $storage_type) {
        my $a = 0;
        my @disk_storage_type;
        foreach my $disk (@list_disk_rpm) {
            my $disk_obj = $raid_obj->disk("$disk");
            if ($disk_obj->parameter("disk_type") eq $storage_type) {
                    $disk_storage_type[$a] = $disk;
                    $a++;
            }
        } #end of for
        #assign new disk list
        @list_disk_rpm = @disk_storage_type;
    }
    return @list_disk_rpm;
} ## end sub RaidSqa_get_disk_list_by_rpm

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_fsm_state

RaidSqa_get_disk_fsm_state(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine get the fsm state of a particular disk in system.

It returns a scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_fsm_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $disk_name   = $opts{disk_name};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    #Check if disk name exists or has an equivalent path_name2.
    $disk_name =
      RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                         conntype    => $conntype,
                                         disk_name   => $disk_name,
                                         node        => $node);

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }

    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk $disk_name does not exist");
        return 0;
    } else {
        my $disk_fsm_state = $disk_obj->parameter("fsm_state");
        return $disk_fsm_state;
    }
} ## end sub RaidSqa_get_disk_fsm_state

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_parent_state

RaidSqa_get_disk_parent_state(host=>$host, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine get the parent_state state of a particular disk in system.

It returns a scalar on success

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk .

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_parent_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $disk_name   = $opts{disk_name};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    #Check if disk name exists or has an equivalent path_name2.
    if (defined($disk_name)) {
        my $disk_name1 =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        if ($disk_name1) {
            $Log->comment("$pkg_name: Using path name $disk_name1 for any operations on $disk_name");
            $disk_name = $disk_name1;
        } else {
            $Log->comment(
                 "$pkg_name: The disk_name $disk_name specified does not exist. Please check again");
            return (0);
        }
    } ## end if (defined($disk_name...

    my $raid_obj          = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj          = $raid_obj->disk("$disk_name");

    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    my $disk_parent_state = $disk_obj->parameter("parent_state");
    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk $disk_name does not exist in filer");
        return 0;
    } else {
        return $disk_parent_state;
    }

} ## end sub RaidSqa_get_disk_parent_state

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_storage_type

RaidSqa_get_disk_storage_type(api_set_obj => $api_set_obj, conntype=>$conntype, disk_name => $disk, node => $node);

This subroutine gets the storage_type of the given disk.

It returns the storage type of disk on success otherwise returns 0.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Optional): Name of disk whose storage type is required.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_storage_type {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR | UNDEF},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    #Check if disk name exists or has an equivalent path_name2.
    if (defined($disk_name)) {
        my $disk_name1 =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        if ($disk_name1) {
            $Log->comment("$pkg_name: Using path name $disk_name1 for any operations on $disk_name");
            $disk_name = $disk_name1;
        } else {
            $Log->comment(
                 "$pkg_name: The disk_name $disk_name specified does not exist. Please check again");
            return (0);
        }
    } ## end if (defined($disk_name...

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @disk_list;
    my %disk_storage_type = ();

    if (!$disk_name) {
        $Log->comment("$pkg_name: disk_name is the required parameter.");
        return 0;
    }

    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $Log->comment("$pkg_name: Disk with name $disk_name does not exist");
        return 0;
    }
    my $storage_type = $disk_obj->parameter("disk_type");
    return $storage_type;
} ## end sub RaidSqa_get_disk_storage_type

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_aggr_name

RaidSqa_get_disk_aggr_name(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine get the aggr/vol name of a particular disk in system.

It returns a scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_aggr_name {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $disk_name   = $opts{disk_name};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk with name $disk_name does not exist");
        return 0;
    }

    my $disk_raid_name  = $disk_obj->parameter("raid_name");
    my $disk_group_rgid = $disk_obj->parameter("group_rgid");
    my $disk_plex_pid   = $disk_obj->parameter("plex_pid");

    my $disk_name_length       = length $disk_name;
    my $disk_group_rgid_length = length $disk_group_rgid;
    my $disk_plex_pid_length   = length $disk_plex_pid;
    my $disk_raid_name_length  = length $disk_raid_name;
    my $substr_length =
      $disk_raid_name_length -
      $disk_name_length -
      $disk_group_rgid_length -
      $disk_plex_pid_length - 10;
    my $disk_aggr_name = substr($disk_raid_name, 1, $substr_length);
    return $disk_aggr_name;

} ## end sub RaidSqa_get_disk_aggr_name

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_plex_name

RaidSqa_get_disk_plex_name(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine get the plex name of a particular disk in system.

It returns a scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_plex_name {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $disk_name   = $opts{disk_name};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    $disk_name =
      RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                         conntype    => $conntype,
                                         disk_name   => $disk_name,
                                         node        => $node);

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");

    if (!$disk_obj) {
        $Log->comment("$pkg_name: Disk $disk_name does not exist");
        return 0;
    }
    my $disk_raid_name  = $disk_obj->parameter("raid_name");
    my $disk_group_rgid = $disk_obj->parameter("group_rgid");

    my $disk_name_length       = length $disk_name;
    my $disk_group_rgid_length = length $disk_group_rgid;
    my $disk_raid_name_length  = length $disk_raid_name;
    my $substr_length = $disk_raid_name_length - $disk_name_length - $disk_group_rgid_length - 4;
    my $disk_plex_name = substr($disk_raid_name, 0, $substr_length);
    return $disk_plex_name;

} ## end sub RaidSqa_get_disk_plex_name

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_group_name

RaidSqa_get_disk_group_name(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine get the group name of a particular disk in system.

It returns a scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_group_name {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $disk_name   = $opts{disk_name};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    my $disk_raid_name;
    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk with name $disk_name does not exist");
        return 0;
    } else {
        $disk_raid_name = $disk_obj->parameter("raid_name");
    }

    my $disk_name_length      = length $disk_name;
    my $disk_raid_name_length = length $disk_raid_name;
    my $substr_length         = $disk_raid_name_length - $disk_name_length - 1;
    my $disk_group_name       = substr($disk_raid_name, 0, $substr_length);
    return $disk_group_name;

} ## end sub RaidSqa_get_disk_group_name

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_rgdn

RaidSqa_get_disk_rgdn(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine get the disk rgdn value of a particular disk in system..

It returns a scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_rgdn {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    $disk_name =
      RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                         conntype    => $conntype,
                                         disk_name   => $disk_name,
                                         node        => $node);
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $Log->comment("$pkg_name: Disk $disk_name does not exist in filer");
        return 0;
    } else {
        my $disk_rgdn = $disk_obj->parameter("disk_rgdn");
        return $disk_rgdn;
    }
} ## end sub RaidSqa_get_disk_rgdn

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_used_size

RaidSqa_get_disk_used_size(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine get the used_size of a particular disk in MB .

It returns a scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_used_size {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $disk_name   = $opts{disk_name};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    #Check if disk name exists or has an equivalent path_name2.
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }

    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk with name $disk_name does not exist");
        return 0;
    }

    my $disk_used_sz = $disk_obj->parameter("used_sz");
    return $disk_used_sz;
} ## end sub RaidSqa_get_disk_used_size

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_sdc_target

RaidSqa_get_disk_sdc_target(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine get the sdc target disks of a particular disk being copied in system.

It returns a scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_sdc_target {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $disk_name   = $opts{disk_name};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $disk;
    my $raid_obj  = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my %disk_hash = $raid_obj->all_disks();
    my @disk_list = keys(%disk_hash);

    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk with name $disk_name does not exist");
        return 0;
    } else {
        my $disk_copy_target_id = $disk_obj->parameter("copy_target_id");
        foreach $disk (@disk_list) {
            my $disk_obj1 = $raid_obj->disk("$disk");
            if ($disk_obj1->parameter("id") eq $disk_copy_target_id) {
                return $disk;
            }
        }
    }
} ## end sub RaidSqa_get_disk_sdc_target

#------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_other_path_name

RaidSqa_get_disk_other_path_name(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine get the other path name of a particular disk in system.

It returns a scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_other_path_name {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $disk_name   = $opts{disk_name};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk with name $disk_name does not exist");
        return 0;
    } else {
        my $disk_path_name2 = $disk_obj->parameter("path_name2");
        my $disk_path_name1 = $disk_obj->parameter("path_name1");
        if ($disk_path_name2 eq $disk_name) {
            return $disk_path_name1;
        } else {
            return $disk_path_name2;
        }
    }
} ## end sub RaidSqa_get_disk_other_path_name

#-------------------------------------------------------------------------------------------------------------------------------------

=head2 RaidSqa_get_disk_offline_ref_count

RaidSqa_get_disk_offline_ref_count (api_set_obj=>$api_set_obj, node=>$node, conntype=>$conntype, disk_name => $disk_name);

This subroutine get the disk offline reference count of a particular disk in system..

It returns a scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item node

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk .

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_disk_offline_ref_count {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }

    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk $disk_name does not exist in filer");
        return 0;
    } else {
        return $disk_obj->parameter("offline_ref");
    }
} ## end sub RaidSqa_get_disk_offline_ref_count

#------------------------------------------------------------------------------

=head2 RaidSqa_get_invalid_diskid

Get invalid disk id's for creation of temporary root volume

RETURNS
        0 on failure
    A single invalid disk or
    reference to an invalid disk list on success

ARGUMENTS

=over

=item conn

(Required): Connection object. Use rsh connection as there might
        be issues with console connection.

=item No. of disks

(Optional):  No of invalid disk id's needed. Defaults to 1.

=item node

(Optional): Node name on which the operation is to be performed

=back

EXAMPLE

        my $ret = RaidSqa_get_invalid_diskid(conn => $conn,
                     num_disks => $no_of_disks, node=>$node);

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_invalid_diskid {

    my %opts = RaidSqa_validate(@_,
                                {conn      => {type => OBJECT, isa      => "Connectrec"},
                                 num_disks => {type => SCALAR, optional => 1, default => 1},
                                 node      => {type => SCALAR, optional => 1, default => "local"},}
                               );

    my $conn         = $opts{conn};
    my $node         = $opts{node};
    my $num_of_disks = $opts{num_disks};

    my ($count, @disk_list, $disk_id, $i, $raid_obj, %spare_disks, @filer_disks_list);

#    $raid_obj         = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    $raid_obj         = new RaidInfo($conn, node => $node);
    %spare_disks      = $raid_obj->spare_disks();
    @filer_disks_list = keys %spare_disks;
    $i                = 1;

    while ($i <= $num_of_disks) {
        $disk_id = rand(200);
        $count = grep(/^$disk_id$/, @filer_disks_list);
        if (0 == $count) {
            push(@disk_list, $disk_id);
        }
        $i++;
    }

    if (0 == scalar(@disk_list)) {
        $Log->comment("$pkg_name: Failed to generate invalid disk id's");
        return 0;
    }

    if (1 == scalar(@disk_list)) {
        return ($disk_id);
    } else {
        return (\@disk_list);
    }
} ## end sub RaidSqa_get_invalid_diskid

#------------------------------------------------------------------------------

=head2 RaidSqa_get_bypassed_disk_list

RaidSqa_get_bypassed_disk_list (api_set_obj=>$api_set_obj, node=>$node, conntype=>$conntype);
This subroutine returns a list of bypassed disks on the filer

It returns a list of all disks bypassed on success and null on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype
(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_bypassed_disk_list {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my @bypassed_disks_list;
    my $output;

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag;sysconfig -v\"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
    }

    my @unformatted_output = split(/\r?\n/, $output);

    foreach my $line (@unformatted_output) {
        if ($line =~ /Disk (.*) in Shelf.*is bypassed/) {
            push @bypassed_disks_list, $1;
        }
    }
    $Log->comment("$pkg_name: bypassed disk list: @bypassed_disks_list");
    return @bypassed_disks_list;
}    ##end sub RaidSqa_get_bypassed_disk_list

#------------------------------------------------------------------------------

=head2 RaidSqa_check_existing_aggr_ok

RaidSqa_check_existing_aggr_ok(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, aggr_type=>$aggr_type, raid_type=>$raid_type, raid_size=>$raid_size, mirror=>$mirror, storage_type=>$storage_type, disk_count=>$disk_count, aggr_bits=>$aggr_bits, aggr_fsm_state=>$aggr_fsm_state, aggr_state_name=>$aggr_state_name, aggr_wafl_online=>$aggr_wafl_online, aggr_wafl_mounted=>$aggr_wafl_mounted, aggr_ignore_inconsist=>$aggr_ignore_inconsist, aggr_snapmirrored=>$aggr_snapmirrored, all_group_fsm_state=>$all_group_fsm_state, all_group_sdc=>$all_group_sdc, node => $node);

This subroutine checks the existing aggr/vol if it is in the state required to run the test.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate  or traditional volume to be created.

=item raid_type

(Optional): Raid type of the volume/aggregate

=item aggr_type

(Optional): aggr type of the volume/aggregate

=item raid_size

(Optional): Raid size of the volume/aggregate.

=item mirror

(Optional): 1 for mirror volume and 0 for nonmirror.

=item storage_type

(Optional):  Disk storage type. Can be ATA, SAS, FCAL, BSAS, FSAS.

=item disk_count

(Optional):  total number of disk in aggr.

=item aggr_bits

(Optional):  type of aggr, 32 or 64 bits

=item aggr_fsm_state

(Optional): aggr fsm state, default is "NORMAL".

=item aggr_state_name

(Optional): aggr state name, default is "online".

=item aggr_wafl_online

(Optional): aggr wafl_online state, default is "1".

=item aggr_wafl_mounted

(Optional): aggr wafl_mounted state, default is "1".

=item aggr_ignore_inconsist

(Optional): aggr ignore_inconsist state, default is "0".

=item aggr_snapmirrored

(Optional): aggr snapmirrored state, default is "0".

=item all_group_fsm_state

(Optional): all groups fsm state, default is "NORMAL".

=item all_group_sdc

(Optional): all groups sdc state, default is "0".

=item node

(Optional): Node name on which the operation is to be performed

=item hybrid

(Optional): Set this value to check if aggr is HyA.Default is undef. Possible values:undef,1, 0.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_existing_aggr_ok {
    my (%opts) = RaidSqa_validate(
        @_,
        {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
         conntype    => {type => SCALAR, default => "#ssh", optional => 1},
         filer_vol             => {type => SCALAR},
         aggr_type             => {type => SCALAR | UNDEF, default => undef, optional => 1},
         raid_type             => {type => SCALAR | UNDEF, default => undef, optional => 1},
         raid_size             => {type => SCALAR | UNDEF, default => undef, optional => 1},
         mirror                => {type => SCALAR | UNDEF, default => undef, optional => 1},
         storage_type          => {type => SCALAR | UNDEF, default => undef, optional => 1},
         disk_count            => {type => SCALAR | UNDEF, default => undef, optional => 1},
         aggr_bits             => {type => SCALAR | UNDEF, default => undef, optional => 1},
         aggr_fsm_state        => {type => SCALAR, default => "NORMAL", optional => 1},
         aggr_state_name       => {type => SCALAR, default => "online", optional => 1},
         aggr_wafl_online      => {type => SCALAR, default => "1", optional => 1},
         aggr_wafl_mounted     => {type => SCALAR, default => "1", optional => 1},
         aggr_ignore_inconsist => {type => SCALAR, default => "0", optional => 1},
         aggr_snapmirrored     => {type => SCALAR, default => "0", optional => 1},
         all_group_fsm_state   => {type => SCALAR, default => "NORMAL", optional => 1},
         all_group_sdc         => {type => SCALAR, default => "0", optional => 1},
         node                  => {type => SCALAR, default => "local", optional => 1},
         hybrid                => {type => SCALAR | UNDEF, default => undef, optional => 1}
        });

    my $api_set_obj           = $opts{api_set_obj};
    my $node                  = $opts{node};
    my $conntype              = $opts{conntype};
    my $filer_vol             = $opts{filer_vol};
    my $aggr_type             = $opts{aggr_type};
    my $raid_type             = $opts{raid_type};
    my $raid_size             = $opts{raid_size};
    my $mirror                = $opts{mirror};
    my $storage_type          = $opts{storage_type};
    my $disk_count            = $opts{disk_count};
    my $aggr_bits             = $opts{aggr_bits};
    my $aggr_fsm_state        = $opts{aggr_fsm_state};
    my $aggr_state_name       = $opts{aggr_state_name};
    my $aggr_wafl_online      = $opts{aggr_wafl_online};
    my $aggr_wafl_mounted     = $opts{aggr_wafl_mounted};
    my $aggr_ignore_inconsist = $opts{aggr_ignore_inconsist};
    my $aggr_snapmirrored     = $opts{aggr_snapmirrored};
    my $all_group_fsm_state   = $opts{all_group_fsm_state};
    my $all_group_sdc         = $opts{all_group_sdc};
    my $hybrid                = $opts{hybrid};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    my $vol_obj = $raid_obj->aggregate("$filer_vol");
    if (!$vol_obj) {
        $Log->comment("$pkg_name: Aggr with name $filer_vol does not exist");
        return 0;
    }
    my %vol_rg_hash = $vol_obj->all_raid_groups();
    my @vol_rg_list = keys(%vol_rg_hash);

    my %vol_disk_hash = $vol_obj->all_disks();
    my @vol_disk_list = keys(%vol_disk_hash);
    my $disk_obj      = $vol_obj->disk("$vol_disk_list[0]");

    my $store_type       = $vol_obj->parameter("store_type");
    my $fsm_state        = $vol_obj->parameter("fsm_state");
    my $state_name       = $vol_obj->parameter("state_name");
    my $wafl_online      = $vol_obj->parameter("wafl_online");
    my $wafl_mounted     = $vol_obj->parameter("wafl_mounted");
    my $ignore_inconsist = $vol_obj->parameter("ignore_inconsist");
    my $snapmirrored     = $vol_obj->parameter("snapmirrored");
    my $mirror_state     = $vol_obj->parameter("mirror_state");
    my $raidtree_type    = $vol_obj->parameter("raidtree_type");
    my $raidtree_size    = $vol_obj->parameter("size");
    my $number_of_disks  = $vol_obj->parameter("disk_count");
    my $aggr_format      = $vol_obj->parameter("aggr_format");

    #$Log->comment("$pkg_name: Verifying if existing aggr/trad vol $filer_vol is normal/online/mounted and not inconsistent/snapmirror destination");

    $Log->comment("$pkg_name: Verifying if the existing $filer_vol have required fsm state ");
    if ($fsm_state ne $aggr_fsm_state) {
        $Log->comment(
             "$pkg_name: Existing $filer_vol does not has the $aggr_fsm_state fsm state as required "
        );
        return 0;
    } else {
        $Log->comment(
                    "$pkg_name: Existing $filer_vol has the $aggr_fsm_state fsm state as required ");
    }

    $Log->comment("$pkg_name: Verifying if the existing $filer_vol have required state name ");
    if ($state_name ne $aggr_state_name) {
        $Log->comment(
            "$pkg_name: Existing $filer_vol does not has the $aggr_state_name state name state as required"
        );
        return 0;
    } else {
        $Log->comment(
                  "$pkg_name: Existing $filer_vol has the $aggr_state_name state name as required ");
    }

    $Log->comment(
                 "$pkg_name: Verifying if the existing $filer_vol have required wafl_online state ");
    if ($wafl_online ne $aggr_wafl_online) {
        $Log->comment(
            "$pkg_name: Existing $filer_vol does not has the $aggr_wafl_online wafl online state as required"
        );
        return 0;
    } else {
        $Log->comment(
             "$pkg_name: Existing $filer_vol has the $aggr_wafl_online wafl online state as required"
        );
    }

    $Log->comment(
                "$pkg_name: Verifying if the existing $filer_vol have required wafl_mounted state ");
    if ($wafl_mounted ne $aggr_wafl_mounted) {
        $Log->comment(
            "$pkg_name: Existing $filer_vol does not has the $aggr_wafl_mounted wafl mounted state as required"
        );
        return 0;
    } else {
        $Log->comment(
            "$pkg_name: Existing $filer_vol has the $aggr_wafl_mounted wafl mounted state as required"
        );
    }

    $Log->comment(
             "$pkg_name: Verifying if the existing $filer_vol have required ignore_inconsist state");
    if ($ignore_inconsist ne $aggr_ignore_inconsist) {
        $Log->comment(
            "$pkg_name: Existing $filer_vol does not has the $aggr_ignore_inconsist ignore_inconsist state as required"
        );
        return 0;
    } else {
        $Log->comment(
            "$pkg_name: Existing $filer_vol has the $aggr_ignore_inconsist ignore_inconsist state as required"
        );
    }

    $Log->comment(
                "$pkg_name: Verifying if the existing $filer_vol have required snapmirrored state ");
    if ($snapmirrored ne $aggr_snapmirrored) {
        $Log->comment(
            "$pkg_name: Existing $filer_vol does not has the $aggr_snapmirrored snapmirrored state as required"
        );
        return 0;
    } else {
        $Log->comment(
            "$pkg_name: Existing $filer_vol has the $aggr_snapmirrored snapmirrored state as required"
        );
    }

    #if ($fsm_state ne "NORMAL" || $state_name ne "online" || $wafl_online ne "1" || $wafl_mounted ne "1" || $ignore_inconsist ne "0" || $snapmirrored ne "0") {
    #        $Log->comment("$pkg_name: Existing aggr/trad vol $filer_vol is not in ONLINE/NORMAL state");
    #        return 0;
    #}

    #$Log->comment("$pkg_name: Verifying if existing aggr/trad vol $filer_vol raidgroups are in normal state");
    #
    #        my $group;
    #        foreach $group (@vol_rg_list) {
    #                my $group_obj = $vol_obj->raid_group("$group") ;
    #                if ($group_obj->parameter("fsm_state") ne "NORMAL" || $group_obj->parameter("disk_copy_source_id") ne "-1" || $group_obj->parameter("disk_copy_target_id") ne "-1") {
    #                        $Log->comment("$pkg_name: Not all the raidgroups are in normal state");
    #                        return 0;
    #                }
    #        }
    my $group;
    foreach $group (@vol_rg_list) {
        $Log->comment(
            "$pkg_name: Verifying if existing aggr/trad vol $filer_vol raidgroup $group is in required fsm state"
        );
        my $group_obj = $vol_obj->raid_group("$group");
        if (!$group_obj) {
            $Log->comment("$pkg_name: Raid group with name $group does not exist");
            next;
        }
        if ($group_obj->parameter("fsm_state") ne $all_group_fsm_state) {
            $Log->comment("$pkg_name: raidgroup $group is not in $all_group_fsm_state fsm state");
            return 0;
        }
        $Log->comment(
            "$pkg_name: Verifying if existing aggr/trad vol $filer_vol raidgroup $group is in required sdc state"
        );
        if (   $group_obj->parameter("disk_copy_source_id") ne "-1"
            || $group_obj->parameter("disk_copy_target_id") ne "-1") {
            $Log->comment("$pkg_name: raidgroup $group is not in required sdc state");
            return 0;
        }
    } ## end foreach $group (@vol_rg_list)

    if (defined($raid_type)) {
        $Log->comment(
            "$pkg_name: Verifying if existing aggr/trad vol $filer_vol is matching the RAID_TYPE requirement for the test"
        );
        if ($raid_type eq "raid4") {
            if (!($raidtree_type =~ /raid4/)) {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required type RAID4"
                );
                return 0;
            }
        } elsif ($raid_type eq "raid_dp") {
            if (!($raidtree_type =~ /raidrd/)) {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required type RAID-DP"
                );
                return 0;
            }
        }
    } ## end if (defined($raid_type...

    if (defined($storage_type)) {
        $Log->comment(
            "$pkg_name: Verifying if existing aggr/trad vol $filer_vol is matching the storage_type requirement for the test"
        );

        if ($storage_type eq "ATA") {
            if ($disk_obj->parameter("disk_type") ne "ATA") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required storage type ATA"
                );
                return 0;
            }
        } elsif ($storage_type eq "FCAL") {
            if ($disk_obj->parameter("disk_type") ne "FCAL") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required storage type FCAL"
                );
                return 0;
            }
        } elsif ($storage_type eq "BSAS") {
            if ($disk_obj->parameter("disk_type") ne "BSAS") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required storage type BSAS"
                );
                return 0;
            }

        } elsif ($storage_type eq "SAS") {
            if ($disk_obj->parameter("disk_type") ne "SAS") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required storage type SAS"
                );
                return 0;
            }
        } elsif ($storage_type eq "SSD") {
            if ($disk_obj->parameter("disk_type") ne "SSD") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required storage type SSD"
                );
                return 0;
            }
        } elsif ($storage_type eq "BSSD") {
            if ($disk_obj->parameter("disk_type") ne "BSSD") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required storage type BSSD"
                );
                return 0;
            }
        } elsif ($storage_type eq "MSATA") {
            if ($disk_obj->parameter("disk_type") ne "MSATA") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required storage type MSATA"
                );
                return 0;
            }
        } elsif ($storage_type eq "FSAS") {
            if ($disk_obj->parameter("disk_type") ne "FSAS") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required storage type FSAS"
                );
                return 0;
            }
        }
    } ## end if (defined($storage_type...

    if (defined($aggr_type)) {
        $Log->comment(
            "$pkg_name: Verifying if the existing $filer_vol is having the same aggr type as required"
        );
        if ($aggr_type) {
            if ($store_type ne "aggr") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required type aggr type"
                );
                return 0;
            } else {
                $Log->comment(
                    "$pkg_name: Verifying if existing aggr $filer_vol contains the required flex vol for the test"
                );

                my $flex_obj = $raid_obj->volume("$filer_vol");

                if (!$flex_obj) {
                    $Log->comment(
                        "$pkg_name: Aggregate $filer_vol does not contain the required flex volume");
                    return 0;
                } elsif ($flex_obj->parameter("state_name") ne "online") {
                    $Log->comment(
                        "$pkg_name: Aggregate $filer_vol does not contain an online required flex volume"
                    );
                    return 0;
                }
            } ## end else [ if ($store_type ne "aggr")

        } else {
            if ($store_type ne "trad") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required type aggr type"
                );
                return 0;
            }
        }
    } ## end if (defined($aggr_type...

    if (defined($raid_size)) {
        if ($raid_size =~ /\D/) {
            $raid_size =
              RaidSqa_get_raid_size_value(api_set_obj  => $api_set_obj,
                                          raid_size    => $raid_size,
                                          raid_type    => $raid_type,
                                          storage_type => $storage_type,
                                          node         => $node);
        }

        $Log->comment(
            "$pkg_name: Verifying if the existing $filer_vol is having the same raid size as required"
        );
        if ($raid_size != $raidtree_size) {
            $Log->comment(
                      "$pkg_name: Existing $filer_vol does not have the same raid size as required");
            return 0;
        } else {
            $Log->comment("$pkg_name: Existing $filer_vol has the same raid size as required");
        }
    } ## end if (defined($raid_size...

    if (defined($mirror)) {
        $Log->comment(
            "$pkg_name: Verifying if the existing $filer_vol is having the same mirror state as required"
        );
        if ($mirror) {
            if ($mirror_state ne "NORMAL") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required type mirror state"
                );
                return 0;
            }
        } else {
            if ($mirror_state ne "UNMIRRORED") {
                $Log->comment(
                    "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required type mirror state"
                );
                return 0;
            }
        }
    } ## end if (defined($mirror))

    if (defined($disk_count)) {
        $Log->comment(
            "$pkg_name: Verifying if the existing $filer_vol is having the same number of disks as required"
        );
        if ($disk_count ne $number_of_disks) {
            $Log->comment(
                "$pkg_name: Existing $filer_vol does not have the same number of disks as required");
            return 0;
        } else {
            $Log->comment("$pkg_name: Existing $filer_vol has the same number of disks as required");
        }
    } ## end if (defined($disk_count...

    if (defined($aggr_bits)) {
        $Log->comment(
            "$pkg_name: Verifying if the existing $filer_vol is having the same aggr bits type as required"
        );
        if (!($aggr_format =~ /$aggr_bits/)) {
            $Log->comment(
                "$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required aggr bits type"
            );
            return 0;
        }
    } ## end if (defined($aggr_bits...

    if (defined($hybrid)) {
        $Log->comment("$pkg_name: Verifying if the existing $filer_vol is hybrid aggr \n");
        if ($vol_obj->parameter("hybrid") != $hybrid) {
            $Log->comment("$pkg_name: Existing aggr/trad vol $filer_vol is not matching to its required hybrid type.\n");
            return 0;
        }
    }

    $Log->comment(
               "$pkg_name: Existing aggr/trad vol $filer_vol meets all the requirement for testing");
    return 1;
} ## end sub RaidSqa_check_existing_aggr_ok

#------------------------------------------------------------------------------

=head2 RaidSqa_check_aggr_exist

RaidSqa_check_aggr_exist(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, aggr_type=>$aggr_type, node => $node);

This subroutine checks that a particular aggergate exist in filer.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate to be checked if it exist in in filer.

=item aggr_type

(Optional): 1 for aggregate and 0 for traditional. Default it is 1. For Cmode as traditional is not supported, we will set it to 1

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_aggr_exist {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        aggr_type => {type => SCALAR, default => 1, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $aggr_type   = $opts{aggr_type};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if (!$aggr_type) {
        $Log->comment(
                "$pkg_name: Traditional volume is not supported, hence changing the aggr_type to 1");
        $aggr_type = 1;
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate($filer_vol);

    # check if a specified aggr/tradvol/flex exist in filer or not
    if ($aggr_obj) {
        $Log->comment("$pkg_name: aggregate $filer_vol exist");
        return 1;
    } else {
        $Log->comment("$pkg_name: aggregate $filer_vol does not exist");
        return 0;
    }
} ## end sub RaidSqa_check_aggr_exist

#------------------------------------------------------------------------------

=head2 RaidSqa_check_aggr_exist_anytype

RaidSqa_check_aggr_exist_anytype(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, node=>$node);

This subroutine checks that any flex/trad or aggr of partricular name exist in filer or not.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked if it exist in in filer.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_aggr_exist_anytype {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    my $vol_obj  = $raid_obj->aggregate("$filer_vol");
    my $flex_obj = $raid_obj->volume("$filer_vol");

    # check that a aggr or tradvol or flex of specified name exists or not
    if ((!$vol_obj) && (!$flex_obj)) {
        $Log->comment("$pkg_name: no trad or aggregate or flex with name $filer_vol exists");
        return 0;
    } elsif ($vol_obj->parameter("store_type") eq 'aggr') {
        $Log->comment("$pkg_name: aggregate $filer_vol exist");
        return 1;
    } elsif ($vol_obj->parameter("store_type") eq 'trad') {
        $Log->comment("$pkg_name: A trad vol $filer_vol exists");
        return 1;
    } elsif ($flex_obj->parameter("store_type") eq 'flex') {
        $Log->comment("$pkg_name: trad vol $filer_vol exist");
        return 1;
    }
} ## end sub RaidSqa_check_aggr_exist_anytype

#------------------------------------------------------------------------------

=head2 RaidSqa_check_aggr_state_name

RaidSqa_check_aggr_state_name(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, state_name=>$state_name,  aggr_type =>$aggr_type, node=>$node);

This subroutine checks that a particular aggergate is state_name(online/offline/restricted) is as defined by user.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate to be checked if it is online.

=item state_name

(Required) : State name of aggr/volume (offline/online/restricted).

=item aggr_type

(Optional): 1 for aggregate and 0 for traditional. Default it is 1. For Cmode as traditional is not supported, we will set it to 1

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_aggr_state_name {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR},
                        state_name => {type => SCALAR},
                        aggr_type  => {type => SCALAR, default => 1, optional => 1},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $state_name  = $opts{state_name};
    my $aggr_type   = $opts{aggr_type};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if (!$aggr_type) {
        $Log->comment(
             "$pkg_name: As traditional volume is not supported, hence changing the aggr_type to 1");
        $aggr_type = 1;
    }

    # check a test aggr/volume is having specified state
    if (!RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  filer_vol   => $filer_vol,
                                  aggr_type   => $aggr_type,
                                  node        => $node)
      ) {
        $Log->comment("$pkg_name: Aggregate $filer_vol does not exist");
        return 0;
    } else {
        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $vol_obj = $raid_obj->aggregate("$filer_vol");

        if (!$vol_obj) {
            $Log->comment("$pkg_name: Aggr $filer_vol does not exist");
            return 0;
        }

        if ($vol_obj->parameter("state_name") ne $state_name) {
            $Log->comment("$pkg_name: state name for $filer_vol is not $state_name as expected");
            return 0;
        } else {
            $Log->comment("$pkg_name: state name for $filer_vol is $state_name as expected");
            return 1;
        }
    } ## end else [ if (!RaidSqa_check_aggr_exist...
} ## end sub RaidSqa_check_aggr_state_name

#------------------------------------------------------------------------------

=head2 RaidSqa_check_aggr_fsm_state

RaidSqa_check_aggr_fsm_state(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, fsm_state=>$fsm_state, aggr_type=>$aggr_type, node => $node);

This subroutine checks fsm_state for a particular aggergate.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate to be checked if it is restricted.

=item fsm_state

(Required) : fsm state to be checked against.

=item aggr_type

(Required): 1 for aggregate and 0 for traditional. Default it is 1. For cmode traditional volume is not supported, hence we  will set it to 1 if provided as 0.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_aggr_fsm_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        fsm_state => {type => SCALAR},
                        aggr_type => {type => SCALAR, default => 1, optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $fsm_state   = $opts{fsm_state};
    my $aggr_type   = $opts{aggr_type};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if (!$aggr_type) {
        $Log->comment("$pkg_name: aggr_type as 0 is not supported in Cmode hence chnaging it to 1");
        $aggr_type = 1;
    }

    # check a test aggr/volume is having specified mirror state
    if (!RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                  filer_vol   => $filer_vol,
                                  aggr_type   => $aggr_type,
                                  node        => $node)
      ) {
        return 0;
    } else {

        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $vol_obj = $raid_obj->aggregate("$filer_vol");

        if ($vol_obj->parameter("fsm_state") ne $fsm_state) {
            $Log->comment("$pkg_name: fsm state for $filer_vol is not $fsm_state as expected");
            return 0;
        } else {
            $Log->comment("$pkg_name: fsm state for $filer_vol is $fsm_state as expected");
            return 1;
        }
    } ## end else [ if (!RaidSqa_check_aggr_exist...
} ## end sub RaidSqa_check_aggr_fsm_state

#------------------------------------------------------------------------------

=head2 RaidSqa_check_aggr_mirror_state

RaidSqa_check_aggr_mirror_state(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, aggr_type=>$aggr_type,mirror_state=>$mirror_state, node=>$node);

This subroutine checks mirror_state for a particular volume/aggergate.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked if it is restricted.

=item aggr_type

(Required) : 1 for aggregate and 0 for traditional.

=item mirror_state

(Required) : mirror state to be checked against.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_aggr_mirror_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol    => {type => SCALAR},
                        aggr_type    => {type => SCALAR, default => 1, optional => 1},
                        mirror_state => {type => SCALAR, default => "UNMIRRORED", optional => 1},
                        node         => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj  = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $filer_vol    = $opts{filer_vol};
    my $mirror_state = $opts{mirror_state};
    my $aggr_type    = $opts{aggr_type};
    my $node         = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if (!$aggr_type) {
        $Log->comment(
             "$pkg_name: As traditional volume is not supported, hence changing the aggr_type to 1");
        $aggr_type = 1;
    }

    # check a test aggr/volume is having specified mirror state
    if (!RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  filer_vol   => $filer_vol,
                                  aggr_type   => $aggr_type,
                                  node        => $node)
      ) {
        return 0;
    } else {

        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $vol_obj = $raid_obj->aggregate("$filer_vol");

        if ($vol_obj->parameter("mirror_state") ne $mirror_state) {
            $Log->comment("$pkg_name: Mirror state for $filer_vol is not $mirror_state as expected");
            return 0;
        } else {
            $Log->comment("$pkg_name: Mirror state for $filer_vol is $mirror_state as expected");
            return 1;
        }
    } ## end else [ if (!RaidSqa_check_aggr_exist...
} ## end sub RaidSqa_check_aggr_mirror_state

#------------------------------------------------------------------------------

=head2 RaidSqa_check_aggr_after_isolate

RaidSqa_check_aggr_after_isolate(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, vol_disk_list=>\@vol_disk_list, other_path_disklist=>\@other_path_disklist, node=>$node);

This subroutine checks an aggregate after isolate that its disks are in olm list and aggr is not in aggr list.

It returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required): Name of aggregate.

=item vol_disk_list

(Required): Reference to the array of disks in aggr/volume.

=item other_path_disklist

(Optional): Reference to the array of disks in aggr/volume with dual path names.

=item aggr_type

(Required): 1 for aggregate and 0 for traditional. Default it is 1.
            For cmode traditional volume is not supported, hence we  will set it to 1 if provided as 0.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_aggr_after_isolate {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol     => {type => SCALAR},
                        vol_disk_list => {type => ARRAYREF},
                        aggr_type     => {type => SCALAR, default => 1, optional => 1},
                        node          => {type => SCALAR, default => "local", optional => 1},
                        other_path_disklist => {type => ARRAYREF | UNDEF, default => undef, optional => 1},});
    my $api_set_obj   = $opts{api_set_obj};
    my $conntype      = $opts{conntype};
    my $filer_vol     = $opts{filer_vol};
    my $vol_disk_list = $opts{vol_disk_list};
    my $other_path_disklist = $opts{other_path_disklist};
    my $aggr_type     = $opts{aggr_type};
    my $node          = $opts{node};
    my @vol_disk_list = @$vol_disk_list;
    my @other_path_disklist = ();

    if (defined ($other_path_disklist)) {
        @other_path_disklist = @$other_path_disklist;
    }

    if (!$aggr_type) {
        $Log->comment("$pkg_name: aggr_type as 0 is not supported in Cmode hence chnaging it to 1");
        $aggr_type = 1;
    }

    my @labelmaint_list =
      RaidSqa_get_labelmaint_list(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  node        => $node);

    $Log->comment("$pkg_name: checking that aggr $filer_vol does not exists after isolate");

    if (RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                 conntype    => $conntype,
                                 filer_vol   => $filer_vol,
                                 aggr_type   => $aggr_type,
                                 node        => $node)
      ) {
        return 0;
    } else {
        $Log->comment("$pkg_name: checking that all disks of aggr $filer_vol are in OLM list");
        my $disk;
        foreach $disk (@vol_disk_list) {
            if (grep /$disk/, @labelmaint_list) {
                $Log->comment("$pkg_name: disk $disk is in labelmaint list");
            } else {
                $Log->comment("$pkg_name: checking $disk using dual_path name, if specified");
                if (grep /$disk/, @other_path_disklist) {
                    $Log->comment("$pkg_name: checked using dual_path of disk $disk, it is in labelmaint list");
                }  else {
                    die ("$pkg_name: disk $disk is not in labelmaint list");
                }
            }
        }
        return 1;
    } ## end else [ if (RaidSqa_check_aggr_exist...
} ## end sub RaidSqa_check_aggr_after_isolate

#------------------------------------------------------------------------------

=head2 RaidSqa_check_aggr_after_unisolate

RaidSqa_check_aggr_after_unisolate(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, vol_disk_list=>\@vol_disk_list, aggr_type=>$aggr_type, node=>$node);

This subroutine checks an aggregate after unisolate that its disks are not olm list and aggr is in aggr list as offline.

It returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object


=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required): Name of aggregate.

=item vol_disk_list

(Required): Reference to the array of disks in aggr/volume.

=item aggr_type

(Required): 1 for aggregate and 0 for traditional. Default it is 1. For cmode traditional volume is not supported, hence we  will set it to 1 if provided as 0

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_aggr_after_unisolate {
    my (%opts) = RaidSqa_validate(
        @_,
        {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
         conntype    => {type => SCALAR, default => "#ssh", optional => 1},
         filer_vol     => {type => SCALAR},
         vol_disk_list => {type => ARRAYREF},
         aggr_type     => {type => SCALAR, default => 1, optional => 1},
         node          => {type => SCALAR, default => "local", optional => 1},

        });
    my $api_set_obj   = $opts{api_set_obj};
    my $conntype      = $opts{conntype};
    my $filer_vol     = $opts{filer_vol};
    my $vol_disk_list = $opts{vol_disk_list};
    my $node          = $opts{node};
    my @vol_disk_list = @$vol_disk_list;
    my $aggr_type     = $opts{aggr_type};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if (!$aggr_type) {
        $Log->comment("$pkg_name: aggr_type as 0 is not supported in Cmode hence chnaging it to 1");
        $aggr_type = 1;
    }

    my @labelmaint_list =
      RaidSqa_get_labelmaint_list(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  node        => $node);

    $Log->comment("$pkg_name: checking that aggr $filer_vol comes back after unisolate");

    if (!RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  filer_vol   => $filer_vol,
                                  aggr_type   => $aggr_type,
                                  node        => $node)
      ) {
        return 0;
    } else {

        $Log->comment("$pkg_name: checking that aggr $filer_vol is offline");

        if (RaidSqa_check_aggr_state_name(api_set_obj => $api_set_obj,
                                          conntype    => $conntype,
                                          filer_vol   => $filer_vol,
                                          state_name  => "offline",
                                          aggr_type   => $aggr_type,
                                          node        => $node)
          ) {
            $Log->comment("$pkg_name: aggr $filer_vol is offline as expected");
        } else {
            $Log->comment("$pkg_name: aggr $filer_vol is not offline as expected");
            return 0;
        }

        $Log->comment("$pkg_name: checking that no disks of aggr $filer_vol are not in OLM list");

        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $disk;
        @vol_disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@vol_disk_list);
        foreach $disk (@vol_disk_list) {

            my $disk_obj = $raid_obj->disk($disk);
            my $disk_2nd = $disk_obj->parameter("path_name2");

            if ($disk_2nd) {
                if ((grep /$disk/, @labelmaint_list) || (grep /$disk_2nd/, @labelmaint_list)) {
                    $Log->comment("$pkg_name: disk $disk is still in labelmaint list");
                    return 0;
                } else {
                    $Log->comment("$pkg_name: disk $disk is not in labelmaint list");
                }

            } else {
                if (grep /$disk/, @labelmaint_list) {
                    $Log->comment("$pkg_name: disk $disk is still in labelmaint list");
                    return 0;
                } else {
                    $Log->comment("$pkg_name: disk $disk is not in labelmaint list");
                }
            }
        } ## end foreach $disk (@vol_disk_list)
        return 1;
    } ## end else [ if (!RaidSqa_check_aggr_exist...
} ## end sub RaidSqa_check_aggr_after_unisolate

#------------------------------------------------------------------------------

=head2 RaidSqa_check_aggr_lw_flags

RaidSqa_check_aggr_lw_flags(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, raid_lw_allowed=>$raid_lw_allowed, raid_lw_enabled=>$raid_lw_enabled, raid_lost_write=>$raid_lost_write, node=>$node);

This function checks the value for raid_config flags raid_lw_allowed, raid_lw_enabled, raid_lost_write against expected value sent as arg to function

Flags and defination:
raid_lw_allowed : possible values: 0 or 1: if 1 indicates that lost write protection can be turned 'ON' on that volume
raid_lw_enabled : possible values: 0 or 1: if 1, aggr will be lost write protected
raid_lost_write : possible values: on, off, upgrading, partial, upgrade partial, aborted, inoperative

on              : This indicates that aggr is cuurently LW protected
off             : This indicates that aggr is not LW protected
partial         : This indicates that some of raid groups are LW protected and some raid groups are in aborted state (See flag desc group_raid_lw_scrub for group lw states)
upgrade partial : This indicates that some of raid groups are LW protected and some of raid group are still in INACTIVE state
upgrading       : This indicates that all raid groups that aggr are in INACTIVE state
aborted         : This indicated that all raid groups that aggr are in aborted state
inoperative     : This indicates that aggr wide lw protection is on, but system wide LW protection is OFF. In this case aggr will not have protection against lost write

It returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required): Name of aggr/volume for which flag values are to be tested.

=item raid_lw_allowed

(Optional): Expected value of raid_lw_allowed raid_config flag.

=item raid_lw_enabled

(Optional): Expected value of raid_lw_enabled raid_config flag.

=item raid_lost_write

(Optional): Expected value of raid_lost_write option for aggr.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_aggr_lw_flags {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        raid_lw_allowed =>
                          {type => SCALAR | UNDEF, default => undef, optional => 1},
                        raid_lw_enabled =>
                          {type => SCALAR | UNDEF, default => undef, optional => 1},
                        raid_lost_write =>
                          {type => SCALAR | UNDEF, default => undef, optional => 1},
                        node => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj     = $opts{api_set_obj};
    my $conntype        = $opts{conntype};
    my $filer_vol       = $opts{filer_vol};
    my $raid_lw_allowed = $opts{raid_lw_allowed};
    my $raid_lw_enabled = $opts{raid_lw_enabled};
    my $raid_lost_write = $opts{raid_lost_write};
    my $node            = $opts{node};
    my $result          = 1;

    $Log->comment("$pkg_name: Testing aggr flags for $filer_vol");

    if (defined($raid_lw_allowed)) {
        $Log->comment("$pkg_name: Expected Values: raid_lw_allowed: $raid_lw_allowed");
        my $flag_value =
          RaidSqa_get_aggr_raid_lw_allowed(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           filer_vol   => $filer_vol,
                                           node        => $node);
        if ($flag_value == $raid_lw_allowed) {
            $Log->comment("$pkg_name: raid_lw_allowed is set to $raid_lw_allowed as expected");
        } else {
            $Log->comment(
                "$pkg_name: raid_lw_allowed is not set to expected value: exp: $raid_lw_allowed and actual: $flag_value"
            );
            $result = 0;
        }
    } ## end if (defined($raid_lw_allowed...

    if (defined($raid_lw_enabled)) {
        $Log->comment("$pkg_name: Expected Values: raid_lw_enabled: $raid_lw_enabled");
        my $flag_value =
          RaidSqa_get_aggr_raid_lw_enabled(api_set_obj => $api_set_obj,
                                           conntype    => $conntype,
                                           filer_vol   => $filer_vol,
                                           node        => $node);
        if ($flag_value == $raid_lw_enabled) {
            $Log->comment("$pkg_name: raid_lw_enabled is set to $raid_lw_enabled as expected");
        } else {
            $Log->comment(
                "$pkg_name: raid_lw_enabled is not set to expected value: exp: $raid_lw_enabled and actual: $flag_value"
            );
            $result = 0;
        }
    } ## end if (defined($raid_lw_enabled...

    if (defined($raid_lost_write)) {
        $Log->comment("$pkg_name: Expected Values: raid_lost_write: $raid_lost_write");

        #using aggr status -v
        my $cmd = "aggr status -v $filer_vol";
        my $response =
          $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; $cmd \"");
        my $output = $response->get_raw_output();
        my @options_raw = split(/\r?\n/, $output);

        my $options_all;
        my $line;
        my $line1;
        my $lw_state;
        foreach $line (@options_raw) {
            $options_all = $options_all . $line;
        }

        my @options_list = split(/[,\s+] /, $options_all);
        foreach $line1 (@options_list) {
            if ($line1 =~ /.*rlw_(\w+).*/) {
                $lw_state = $1;
                last;
            }
        }

        if ($lw_state eq $raid_lost_write) {
            $Log->comment("$pkg_name: raid_lost_write is set to $raid_lost_write as expected");
        } else {
            $Log->comment(
                "$pkg_name: rlw status for $filer_vol is not set to expected value: exp: $raid_lost_write and actual: $lw_state"
            );
            $result = 0;
        }
    } ## end if (defined($raid_lost_write...

    return $result;
} ## end sub RaidSqa_check_aggr_lw_flags

#------------------------------------------------------------------------------

=head2 RaidSqa_check_plex_exist

RaidSqa_check_plex_exist(api_set_obj=>$api_set_obj, node=>$node, conntype=>$conntype, plex_name => $plex_name, node=>$node);

This subroutine checks that a particular plex exist in filer.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item plex_name

(Required) : Name of plex to be checked if it exist in in filer.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_plex_exist {
    my (%opts) =
      RaidSqa_validate(@_,
               {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                node        => {type => SCALAR, default => "local", optional => 1},
                conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                plex_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $plex_name   = $opts{plex_name};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    my $plex_obj = $raid_obj->plex("$plex_name");

    # check a specified plex exist in filer
    if (!$plex_obj) {
        $Log->comment("$pkg_name: plex $plex_name does not exist in the filer");
        return 0;
    } else {
        $Log->comment("$pkg_name: plex $plex_name exist in the filer");
        return 1;
    }
} ## end sub RaidSqa_check_plex_exist

#------------------------------------------------------------------------------

=head2 RaidSqa_check_plex_fsm_state

RaidSqa_check_plex_fsm_state(api_set_obj=>$api_set_obj, node=>$node, conntype=>$conntype, plex_name => $plex_name, fsm_state=>$fsm_state);

This subroutine checks fsm state of a particular raid plex

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item plex_name

(Required) : Name of raid plex to be checked

=item fsm_state

(Required) : fsm_state to be checked against.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_plex_fsm_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        plex_name => {type => SCALAR},
                        fsm_state => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $plex_name   = $opts{plex_name};
    my $fsm_state   = $opts{fsm_state};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check a test plex exists and is having specified fsm_state
    if (!RaidSqa_check_plex_exist(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  plex_name   => $plex_name)
      ) {
        return 0;
    } else {
        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $plex_obj = $raid_obj->plex("$plex_name");

        if ($plex_obj->parameter("fsm_state") eq "$fsm_state") {
            $Log->comment("$pkg_name: raid plex $plex_name is in $fsm_state as expected");
            return 1;
        } else {
            $Log->comment("$pkg_name: raid plex $plex_name is not in $fsm_state as expected");
            return 0;
        }
    } ## end else [ if (!RaidSqa_check_plex_exist...
} ## end sub RaidSqa_check_plex_fsm_state

#------------------------------------------------------------------------------

=head2 RaidSqa_check_plex_state_name

RaidSqa_check_plex_state_name(api_set_obj=>$api_set_obj, node=>$node, conntype=>$conntype, plex_name => $plex_name, state_name=>$state_name);

This subroutine checks that a particular plex state is offline or online.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item plex_name

(Required) : Name of plex to be checked if it is offline.

=item state_name

(Required) : state_name of plex. Either offline or online.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_plex_state_name {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        plex_name  => {type => SCALAR},
                        state_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $plex_name   = $opts{plex_name};
    my $state_name  = $opts{state_name};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $plex_obj = $raid_obj->plex("$plex_name");

    # check test plex exists in filer and it is offline
    if (!RaidSqa_check_plex_exist(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  plex_name   => $plex_name,
                                  node        => $node)
      ) {
        return 0;
    } elsif ($state_name eq "online") {
        if ($plex_obj->parameter("online") ne "1") {
            return 0;
        } else {
            $Log->comment("$pkg_name: Plex $plex_name is online");
            return 1;
        }
    } elsif ($state_name eq "offline") {
        if ($plex_obj->parameter("online") ne "0") {
            return 0;
        } else {
            $Log->comment("$pkg_name: Plex $plex_name is offline");
            return 1;
        }
    }
} ## end sub RaidSqa_check_plex_state_name

#------------------------------------------------------------------------------

=head2 RaidSqa_check_plex_resyncing

RaidSqa_check_plex_resyncing(api_set_obj=>$api_set_obj, node=>$node, conntype=>$conntype, plex_name => $plex_name, resync_level=>$resync_level);

This subroutine checks that a particular plex state is offline or online.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item plex_name

(Required) : Name of raid plex to be checked.

=item resync_level

(Required) : resync_level to be checked against.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_plex_resyncing {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj  => {type => OBJECT, isa     => 'NACL::APISet'},
                        node         => {type => SCALAR, default => "local", optional => 1},
                        conntype     => {type => SCALAR, default => "#ssh", optional => 1},
                        plex_name    => {type => SCALAR},
                        resync_level => {type => SCALAR},});

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $plex_name   = $opts{plex_name};
    my $resync_level = $opts{resync_level};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $plex_obj = $raid_obj->plex("$plex_name");

    if(!defined($plex_obj)) {
        $Log->comment("$pkg_name: plex with name $plex_name does not exist");
        return 0;
    } else {
        if ($plex_obj->parameter("resyncing") eq "1") {
            if ($plex_obj->parameter("resync_level") eq "$resync_level") {
                $Log->comment("$pkg_name: raid plex $plex_name is resyncing with level $resync_level as expected");
                return 1;
            } else {
                $Log->comment("$pkg_name: raid plex $plex_name is not resyncing with level $resync_level");
                return 0;
            }
        } else {
            $Log->comment("$pkg_name: raid plex $plex_name is not resyncing ");
            return 0;
        }
    }
}  ## end sub RaidSqa_check_plex_resyncing

#------------------------------------------------------------------------------

=head2 RaidSqa_check_group_exist

RaidSqa_check_group_exist(api_set_obj => $api_set_obj, conntype=>$conntype, group_name => $group_name, node => $node);

This subroutine checks that a particular raid group exist in filer.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of the raid group to be checked if it exist in in filer.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_group_exist {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $group_obj = $raid_obj->raid_group("$group_name");

    # check a specified group exist in filer
    if (!$group_obj) {
        $Log->comment("$pkg_name: no raid group with name $group_name exist in the filer");
        return 0;
    } else {
        $Log->comment("$pkg_name: raid group with name $group_name exist in the filer");
        return 1;
    }
} ## end sub RaidSqa_check_group_exist

#------------------------------------------------------------------------------

=head2 RaidSqa_check_group_fsm_state

RaidSqa_check_group_fsm_state(api_set_obj => $api_set_obj, conntype=>$conntype, group_name => $group_name, fsm_state=>$fsm_state, node => $node);

This subroutine checks fsm state of a particular raid group

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid group to be checked

=item fsm_state

(Required) : fsm_state to be checked against.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_group_fsm_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},
                        fsm_state  => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};
    my $fsm_state   = $opts{fsm_state};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check a test group exists and is having specified fsm_state
    if (!RaidSqa_check_group_exist(api_set_obj => $api_set_obj,
                                   group_name  => $group_name,
                                   node        => $node)
      ) {
        return 0;
    } else {

        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $group_obj = $raid_obj->raid_group("$group_name");

        if ($group_obj->parameter("fsm_state") eq "$fsm_state") {
            $Log->comment("$pkg_name: raid group $group_name is in $fsm_state as expected");
            return 1;
        } else {
            $Log->comment("$pkg_name: raid group $group_name is not in $fsm_state as expected");
            return 0;
        }
    } ## end else [ if (!RaidSqa_check_group_exist...
} ## end sub RaidSqa_check_group_fsm_state

#------------------------------------------------------------------------------

=head2 RaidSqa_check_group_scrub_in_progress

RaidSqa_check_group_scrub_in_progress(api_set_obj=>$api_set_obj, conntype=>$conntype, group_name=>$group_name, node=>$node);

This subroutine checks that raid group scrub is running or not

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid group to be checked

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_group_scrub_in_progress {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check a test group exists and is having specified fsm_state
    if (!RaidSqa_check_group_exist(api_set_obj => $api_set_obj,
                                   conntype    => $conntype,
                                   group_name  => $group_name,
                                   node        => $node)
      ) {
        return 0;
    } else {

        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $group_obj = $raid_obj->raid_group("$group_name");

        if (   ($group_obj->parameter("scrub_progress") ne "-1")
            && ($group_obj->parameter("scrub_suspended_block") eq "-1")) {
            $Log->comment("$pkg_name: raid group $group_name scrub is in progress");
            return 1;
        } else {
            $Log->comment("$pkg_name: raid group $group_name scrub is not in progress");
            return 0;
        }
    } ## end else [ if (!RaidSqa_check_group_exist...
} ## end sub RaidSqa_check_group_scrub_in_progress

#------------------------------------------------------------------------------

=head2 RaidSqa_check_group_recons_in_progress

RaidSqa_check_group_recons_in_progress(api_set_obj=>$api_set_obj, conntype=>$conntype, group_name=>$group_name, node=>$node);

This subroutine checks that raid group recons is running or not

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid group to be checked

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_group_recons_in_progress {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check a test group exists and is having specified fsm_state
    if (!RaidSqa_check_group_exist(api_set_obj => $api_set_obj,
                                   conntype    => $conntype,
                                   group_name  => $group_name,
                                   node        => $node)
      ) {
        return 0;
    } else {

        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $group_obj = $raid_obj->raid_group("$group_name");

        if (   ($group_obj->parameter("fsm_state") =~ /RECONS/)
            && ($group_obj->parameter("recons_progress") ne "-1")
            && ($group_obj->parameter("recons_suspended_block") eq "-1")) {
            $Log->comment("$pkg_name: raid group $group_name reconstruction is in progress");
            return 1;
        } else {
            $Log->comment("$pkg_name: raid group $group_name reconstruction is not in progress");
            return 0;
        }
    } ## end else [ if (!RaidSqa_check_group_exist...
} ## end sub RaidSqa_check_group_recons_in_progress

#------------------------------------------------------------------------------

=head2 RaidSqa_check_group_reparity_in_progress

RaidSqa_check_group_reparity_in_progress(api_set_obj=>$api_set_obj, conntype=>$conntype, group_name=>$group_name, node=>$node);

This subroutine checks that raid group reparity is running or not

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid group to be checked

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_group_reparity_in_progress {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check a test group exists and is having specified fsm_state
    if (!RaidSqa_check_group_exist(api_set_obj => $api_set_obj,
                                   conntype    => $conntype,
                                   group_name  => $group_name,
                                   node        => $node)
      ) {
        return 0;
    } else {
        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $group_obj = $raid_obj->raid_group("$group_name");

        if ($group_obj->parameter("reparity_progress") ne "-1") {
            $Log->comment("$pkg_name: Reparity on raid_group $group_name is in progress");
            return 1;
        } else {
            $Log->comment("$pkg_name: Reparity on raid_group $group_name is not in progress");
            return 0;
        }
    } ## end else [ if (!RaidSqa_check_group_exist...
} ## end sub RaidSqa_check_group_reparity_in_progress

#------------------------------------------------------------------------------

=head2 RaidSqa_check_group_scrub_is_suspended

RaidSqa_check_group_scrub_is_suspended(api_set_obj => $api_set_obj, conntype=>$conntype, group_name=>$group_name, node => $node);

This subroutine checks that raid group scrub is suspended or not

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid group to be checked

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_group_scrub_is_suspended {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check a test group exists and is having specified fsm_state
    if (!RaidSqa_check_group_exist(api_set_obj => $api_set_obj,
                                   group_name  => $group_name,
                                   node        => $node)
      ) {
        return 0;
    } else {

        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $group_obj = $raid_obj->raid_group("$group_name");

        if (   ($group_obj->parameter("scrub_progress") eq "-1")
            && ($group_obj->parameter("scrub_suspended_block") ne "-1")) {
            $Log->comment("$pkg_name: raid group $group_name scrub is in suspend state");
            return 1;
        } else {
            $Log->comment("$pkg_name: raid group $group_name scrub is not in suspend state");
            return 0;
        }
    } ## end else [ if (!RaidSqa_check_group_exist...
} ## end sub RaidSqa_check_group_scrub_is_suspended

#------------------------------------------------------------------------------

=head2 RaidSqa_check_group_recons_is_suspend

RaidSqa_check_group_recons_is_suspend(api_set_obj=>$api_set_obj, conntype=>$conntype, group_name=>$group_name, node=>$node);

This subroutine checks that raid group recons is suspend or not

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid group to be checked

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_group_recons_is_suspend {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check a test group exists and is having specified fsm_state
    if (!RaidSqa_check_group_exist(api_set_obj => $api_set_obj,
                                   conntype    => $conntype,
                                   group_name  => $group_name,
                                   node        => $node)
      ) {
        return 0;
    } else {

        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $group_obj = $raid_obj->raid_group("$group_name");

        if (   ($group_obj->parameter("fsm_state") =~ /RECONS/)
            && ($group_obj->parameter("recons_suspended_block") ne "-1")) {
            $Log->comment("$pkg_name: raid group $group_name reconstruction is in suspend state");
            return 1;
        } else {
            $Log->comment(
                         "$pkg_name: raid group $group_name reconstruction is not in suspend state");
            return 0;
        }
    } ## end else [ if (!RaidSqa_check_group_exist...
} ## end sub RaidSqa_check_group_recons_is_suspend

#------------------------------------------------------------------------------

=head2 RaidSqa_check_group_is_running_context_scrub

RaidSqa_check_group_is_running_context_scrub(api_set_obj=>$api_set_obj, conntype=>$conntype, group_name=>$group_name, node=>$node);

This subroutine checks that raid group is running context scrub

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_name

(Required) : Name of raid group to be checked

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_group_is_running_context_scrub {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $group_name  = $opts{group_name};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check a test group exists and is having specified fsm_state
    if (!RaidSqa_check_group_exist(api_set_obj => $api_set_obj,
                                   conntype    => $conntype,
                                   group_name  => $group_name,
                                   node        => $node)
      ) {
        return 0;
    } else {

        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my $group_obj = $raid_obj->raid_group("$group_name");
        if (   ($group_obj->parameter("context_scrub") eq "1")
            && ($group_obj->parameter("context_parity_protected") eq "0")
            && ($group_obj->parameter("scrub_progress") ne "-1")
            && ($group_obj->parameter("scrub_suspended_block") eq "-1")) {

            $Log->comment("$pkg_name: raid group $group_name is running context scrub");
            return 1;
        } else {
            $Log->comment("$pkg_name: raid group $group_name scrub is not running context scrub");
            return 0;
        }
    } ## end else [ if (!RaidSqa_check_group_exist...
} ## end sub RaidSqa_check_group_is_running_context_scrub

#------------------------------------------------------------------------------

=head2 RaidSqa_check_group_lw_flags

RaidSqa_check_group_lw_flags(api_set_obj=>$api_set_obj, conntype=>$conntype, group_list=>\@group_list, group_raid_lw_scrub=>$group_raid_lw_scrub);

This function checks the value for raid_config flag against expected group_raid_lw_scrub provided as arg to function
Parameter group_raid_lw_scrub represents the LW state of the group.
Possible values: DISABLED, ABORTED, INACTIVE, DONE
DISABLED: Parameter will have this value when aggr wide options is OFF
ABORTED : Parameter will have this value when lw scrub detects non zero gen id on disk
INACTIVE: Parameter will have this value when at least once system wide LW option was turned off and no scrub is run on that group ever since.
DONE    : When LW scrub completes and does not find any gen id inconsistancy, it will mark raid group as DONE and this group will be LW protected

It returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item group_list

(Required): reference to an array containing a list of group for which flag value is to be checked.

=item group_raid_lw_scrub

(Required): Expected value of group_raid_lw_scrub raid_config flag.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_group_lw_flags {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        group_list          => {type => ARRAYREF},
                        group_raid_lw_scrub => {type => SCALAR},
                        node                => {type => SCALAR, default => "local", optional => 1},}
                      );

    my $api_set_obj         = $opts{api_set_obj};
    my $conntype            = $opts{conntype};
    my $group_list          = $opts{group_list};
    my $group_raid_lw_scrub = $opts{group_raid_lw_scrub};
    my $node                = $opts{node};
    my $result              = 1;

    foreach my $group (@$group_list) {
        my $flagval =
          RaidSqa_get_group_raid_lw_scrub(api_set_obj => $api_set_obj,
                                          conntype    => $conntype,
                                          group_name  => $group);
        if ($flagval eq $group_raid_lw_scrub) {
            $Log->comment("$pkg_name: $group has group_raid_lw_scrub set to $flagval");
        } else {
            $Log->comment(
                "$pkg_name: $group has group_raid_lw_scrub not set to $group_raid_lw_scrub :: $flagval"
            );
            $result = 0;
        }
    } ## end foreach my $group (@$group_list)
    return $result;
} ## end sub RaidSqa_check_group_lw_flags

#------------------------------------------------------------------------------

=head2 RaidSqa_check_disk_exist

RaidSqa_check_disk_exist(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine checks that a particular disk exist in system.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to be checked if it exist in filer or not.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_disk_exist {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $disk_name   = $opts{disk_name};
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check if specified disk exists in raid
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk $disk_name does not exist in filer");
        return 0;
    } else {
        $Log->comment("$pkg_name: disk $disk_name does exist in filer");
        return 1;
    }

} ## end sub RaidSqa_check_disk_exist

#------------------------------------------------------------------------------

=head2 RaidSqa_check_disk_fsm_state

RaidSqa_check_disk_fsm_state(api_set_obj => $api_set_obj, conntype=>$conntype, disk_name => $disk_name, fsm_state=>$fsm_state, node => $node);

This subroutine checks a particular disk's fsm state in system.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to be checked.

=item fsm_state

(Required) : FSM state of the disk to be checked against. Like INUSE|ZEROED|ZEROING|NOTZEROED|OFFLINE|FAILED|

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_disk_fsm_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        fsm_state => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $disk_name   = $opts{disk_name};
    my $fsm_state   = $opts{fsm_state};
    my $conntype    = $opts{conntype};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check if specified disk exist in specific fsm_state
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    # checking for NULL disk_obj
    if (!($disk_obj)) {
        $Log->comment("$pkg_name: Invalid disk_object! ");
        return 0;
    }

    my $disk_fsm_state = $disk_obj->parameter("fsm_state");

    if ($disk_fsm_state eq $fsm_state) {
        $Log->comment("$pkg_name: disk $disk_name is in $fsm_state fsm state");
        return 1;
    } else {
        $Log->comment(
            "$pkg_name: disk $disk_name fsm state is not $fsm_state instead it is: $disk_fsm_state");
        return 0;
    }
} ## end sub RaidSqa_check_disk_fsm_state

#------------------------------------------------------------------------------

=head2 RaidSqa_check_disk_parent_state

RaidSqa_check_disk_parent_state(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, parent_state=>$parent_state, node=>$node);

This subroutine checks a particular disk's parent_state state in system.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object


=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to be checked.

=item parent_state

(Required) : parent_state of the disk to be checked against. Like DATA|PARITY|DPARITY|RECONSDATA|RECONSPARITY|RECONSDPARITY

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_disk_parent_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name    => {type => SCALAR},
                        parent_state => {type => SCALAR},
                        node         => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj  = $opts{api_set_obj};
    my $disk_name    = $opts{disk_name};
    my $parent_state = $opts{parent_state};
    my $conntype     = $opts{conntype};
    my $node         = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check if specified disk exist in specific parent_state
    my $raid_obj          = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj          = $raid_obj->disk("$disk_name");

    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    my $disk_parent_state = $disk_obj->parameter("parent_state");
    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk $disk_name does not exist in filer");
        return 0;
    } elsif ($disk_parent_state eq $parent_state) {
        $Log->comment("$pkg_name: disk $disk_name is in $parent_state parent state");
        return 1;
    } else {
        $Log->comment(
            "$pkg_name: disk $disk_name parent state is not $parent_state instead it is: $disk_parent_state"
        );
        return 0;
    }

} ## end sub RaidSqa_check_disk_parent_state

#------------------------------------------------------------------------------

=head2 RaidSqa_check_disk_is_prefailed

RaidSqa_check_disk_is_prefailed(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name =>$disk_name, node=>$node);

This subroutine checks that a particular disk is in prefail state.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_disk_is_prefailed {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $disk_name   = $opts{disk_name};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check if specified disk exist in specific fsm_state
    my $raid_obj           = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj           = $raid_obj->disk("$disk_name");

    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    my $disk_prefail_state = $disk_obj->parameter("prefail");
    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk $disk_name does not exist in filer");
        return 0;
    } elsif ($disk_prefail_state eq "1") {
        $Log->comment("$pkg_name: disk $disk_name is in prefail state");
        return 1;
    } else {
        $Log->comment("$pkg_name: disk $disk_name is not in prefail state");
        return 0;
    }

} ## end sub RaidSqa_check_disk_is_prefailed

#------------------------------------------------------------------------------

=head2 RaidSqa_check_disk_sdc_start

RaidSqa_check_disk_sdc_start(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name =>$disk_name, node=>$node);

This subroutine checks that sdc started on a particular prefailed disk.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object


=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_disk_sdc_start {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $disk_name   = $opts{disk_name};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check if specified disk exist in specific fsm_state
    my $raid_obj          = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj          = $raid_obj->disk("$disk_name");

    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    my $disk_sdc_progress = $disk_obj->parameter("copy_progress");
    if (!$disk_obj) {
        $Log->comment("$pkg_name: disk $disk_name does not exist in filer");
        return 0;
    } elsif ($disk_sdc_progress ne "-1") {
        $Log->comment("$pkg_name: sdc started on disk $disk_name ");
        return 1;
    } else {
        $Log->comment("$pkg_name: sdc didn't started on disk $disk_name ");
        return 0;
    }
} ## end sub RaidSqa_check_disk_sdc_start

#------------------------------------------------------------------------------

=head2 RaidSqa_check_disk_is_doing_optimized_recons

RaidSqa_check_disk_is_doing_optimized_recons(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node=>$node);

This subroutine checks a particular disk's is doing optimized reconstruction.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to be checked.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_disk_is_doing_optimized_recons {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $disk_name   = $opts{disk_name};
    my $conntype    = $opts{conntype};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # check if specified disk exist in specific fsm_state
    my $raid_obj             = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_obj             = $raid_obj->disk("$disk_name");

    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    my $disk_optimized_state = $disk_obj->parameter("optimized_recons");
    my $disk_parent_state    = $disk_obj->parameter("parent_state");
    if (!defined($disk_obj)) {
        $Log->comment("$pkg_name: disk $disk_name does not exist in filer");
        return 0;
    } elsif (($disk_optimized_state) && ($disk_parent_state =~ /RECONS/)) {
        $Log->comment("$pkg_name: disk $disk_name is in optimized reconstruction state");
        return 1;
    } else {
        $Log->comment("$pkg_name: disk $disk_name is not in optimized reconstruction state");
        return 0;
    }

} ## end sub RaidSqa_check_disk_is_doing_optimized_recons

#------------------------------------------------------------------------------

=head2 RaidSqa_check_disk_block_is_zeroed_out

RaidSqa_check_disk_block_is_zeroed_out(api_set_obj => $api_set_obj, conntype=>$conntype, disk_name =>$disk_name, dbn=>$dbn, node => $node);

This subroutine checks that a particular disk blocks number (dbn) is zeroed out.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to be checked.

=item dbn

(Required) : Disk block number (dbn) of the disks to be checked for zeroed out.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_disk_block_is_zeroed_out {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        dbn       => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $dbn         = $opts{dbn};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my ($output, $hex_value);

    my $result = 1;
    try {
        my $response =
          $api_set_obj->system_node_run(
                                 node    => $node,
                                 command => "\"priv set -q diag; dumpblock dbn -c $disk_name $dbn\""
          );
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_obj = shift;
        $output = _handle_nacl_exception($exception_obj);
        $result = 0;
    };

    if ($result) {
        my @output_array = split(/\r?\n/, $output);
        foreach my $line (@output_array) {
            if ($line =~ /0x0000:/) {
                my @line1 = split(" ", $line);
                $hex_value = hex($line1[2]);
                last;
            }
        }
        if ($hex_value == 0x02000001) {
            $Log->comment("$pkg_name: dbn $dbn in disk $disk_name is zeroed out");
            $result = 1;
        } else {
            $Log->comment("$pkg_name: dbn $dbn in disk $disk_name is not zeroed out");
            $result = 0;
        }
    } else {
        $Log->comment("$pkg_name: dumpblock command could not be executed");
        return $result;
    }
    return $result;
} ## end sub RaidSqa_check_disk_block_is_zeroed_out

#------------------------------------------------------------------------------

=head2 RaidSqa_check_disk_block_is_pseudo

RaidSqa_check_disk_block_is_pseudo(api_set_obj => $api_set_obj,  conntype=>$conntype, disk_name =>$disk_name, dbn=>$dbn, node => $node);

This subroutine checks that a particular disk blocks number (dbn) is pseudo.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to be checked.

=item dbn

(Required) : disk block number (dbn) of the disks to be checked for pseudo.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_disk_block_is_pseudo {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        dbn       => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $disk_name   = $opts{disk_name};
    my $dbn         = $opts{dbn};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    try {
        my $response =
          $api_set_obj->system_node_run(
                                 node    => $node,
                                 command => "\"priv set -q diag; dumpblock dbn -c $disk_name $dbn\""
          );
        my $output              = $response->get_raw_output();
        my @output_array        = split(/\r?\n/, $output);
        my @context_cksum_lines = grep(/0x0020: /, @output_array);
        my @line3               = split(" ", $context_cksum_lines[0]);
        my @bit_to_comp_in_hex  = split(//, $line3[4]);
        my $bit_to_comp_in_bin  = sprintf "%04b", hex($bit_to_comp_in_hex[9]);
        my @last_bit_to_comp    = split(//, $bit_to_comp_in_bin);

        if ($last_bit_to_comp[3] == 1) {
            $Log->comment("$pkg_name: dbn $dbn in disk $disk_name is a pseudo bad block");
            return 1;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };
} ## end sub RaidSqa_check_disk_block_is_pseudo

#------------------------------------------------------------------------------

=head2 RaidSqa_check_disk_block_is_unverified

RaidSqa_check_disk_block_is_unverified(api_set_obj => $api_set_obj,  conntype=>$conntype, disk_name =>$disk_name, dbn=>$dbn, node => $node);

This subroutine checks that a particular disk blocks number (dbn) is unverified.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk to be checked.

=item dbn

(Required) : disk block number (dbn) of the disks to be checked for unverified.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_disk_block_is_unverified {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        dbn       => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $disk_name   = $opts{disk_name};
    my $dbn         = $opts{dbn};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    try {
        my $response =
          $api_set_obj->system_node_run(
                                 node    => $node,
                                 command => "\"priv set -q diag; dumpblock dbn -c $disk_name $dbn\""
          );
        my $output       = $response->get_raw_output();
        my @output_array = split(/\r?\n/, $output);
        my @line3        = split(" ", $output_array[4]);

        my @bit_to_comp_in_hex = split(//, $line3[4]);
        my $bit_to_comp_in_bin = sprintf "%04b", hex($bit_to_comp_in_hex[9]);
        my @bit_to_comp = split(//, $bit_to_comp_in_bin);

        if ($bit_to_comp[2] == 1) {
            $Log->comment("$pkg_name: dbn $dbn in disk $disk_name is unverified block");
            return 1;
        }
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };
} ## end sub RaidSqa_check_disk_block_is_unverified

#------------------------------------------------------------------------------

=head2 RaidSqa_check_disk_after_isolate

RaidSqa_check_disk_after_isolate(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name=>$disk_name, node=>$node);

This subroutine checks a disk after isolate that it is not in filer raid recognised disk list but in OLM.

It returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object


=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required): Name of disk which was isolated.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_disk_after_isolate {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my @labelmaint_list =
      RaidSqa_get_labelmaint_list(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  node        => $node);

    $Log->comment(
        "$pkg_name: checking that disk $disk_name is not in filer raid recognized list, but instaed in OLM"
    );

    if (RaidSqa_check_disk_exist(api_set_obj => $api_set_obj,
                                 conntype    => $conntype,
                                 disk_name   => $disk_name,
                                 node        => $node)
      ) {
        $Log->comment("$pkg_name: disk $disk_name is still exist in filer raid recognized list");
        return 0;
    } else {
        if (grep /$disk_name/, @labelmaint_list) {
            $Log->comment("$pkg_name: disk $disk_name is in labelmaint list");
            return 1;
        } else {
            $Log->comment("$pkg_name: disk $disk_name is not in labelmaint list");
            return 0;
        }
    }
} ## end sub RaidSqa_check_disk_after_isolate

#------------------------------------------------------------------------------

=head2 RaidSqa_check_disk_after_unisolate

RaidSqa_check_disk_after_unisolate(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name=>$disk_name, node=>$node);

This subroutine checks a disk after unisolate that it is not olm list but in filer raid recognized list.

It returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required): Name of disk which got unisolated.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_disk_after_unisolate {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    my @labelmaint_list =
      RaidSqa_get_labelmaint_list(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  node        => $node);

    $Log->comment(
        "$pkg_name: checking that disk $disk_name is in filer raid recognized list, but not in labelmaint list"
    );

    if (!RaidSqa_check_disk_exist(api_set_obj => $api_set_obj,
                                  conntype    => $conntype,
                                  disk_name   => $disk_name,
                                  node        => $node)
      ) {
        $Log->comment("$pkg_name: disk $disk_name is does not exist in filer raid recognized list");
        return 0;
    } else {
        $Log->comment("$pkg_name: disk $disk_name is in filer raid recognized list as expected");
        my $disk_obj = $raid_obj->disk($disk_name);
        my $disk_2nd = $disk_obj->parameter("path_name2");
        if ($disk_2nd) {
            if ((grep /$disk_name/, @labelmaint_list) || (grep /$disk_2nd/, @labelmaint_list)) {
                $Log->comment("$pkg_name: disk $disk_name is in labelmaint list");
                return 0;
            } else {
                $Log->comment("$pkg_name: disk $disk_name is not in labelmaint list");
                return 1;
            }

        } else {
            if (grep /$disk_name/, @labelmaint_list) {
                $Log->comment("$pkg_name: disk $disk_name is in labelmaint list");
                return 0;
            } else {
                $Log->comment("$pkg_name: disk $disk_name is not in labelmaint list");
                return 1;
            }
        }

    } ## end else [ if (!RaidSqa_check_disk_exist...

} ## end sub RaidSqa_check_disk_after_unisolate

#------------------------------------------------------------------------------

=head2 RaidSqa_check_raid_size_valid_range

RaidSqa_check_raid_size_valid_range(api_set_obj => $api_set_obj, conntype=>$conntype, raid_size=>$raid_size, raid_type=>$raid_type, storage_type=>$storage_type, node => $node, override => $override);

This subroutine checks that a particular raid_size is in valid range for a given raid_type and storage_type.

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item raid_size

(Required) : Raid size of the volume/aggregate to be created.  No default. Any user_defined count.

=item raid_type

(Required) : Raid type of the volume/aggregate to be created. No default. cane be raid4 or raid_dp.

=item storage_type

(Required) : Disk storage type. Can be  SCSI, FCAL, ATA, EATA, LUN, SAS, SATA, SSD, BSAS, XSAS, XATA, FSAS.

=item node

(Optional):  Node name on which the operation is to be performed

=item override

(Optional): override 'on' refers to the value of the hidden options raid.raid4.raidsize.override and raid.raid_dp.raidsize.override which are 'off' by default

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_raid_size_valid_range {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        raid_size    => {type => SCALAR},
                        raid_type    => {type => SCALAR},
                        storage_type => {type => SCALAR},
                        override     => {type => SCALAR, default => "off", optional => 1},
                        node         => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj  = $opts{api_set_obj};
    my $conntype     = $opts{conntype};
    my $raid_size    = $opts{raid_size};
    my $raid_type    = $opts{raid_type};
    my $storage_type = $opts{storage_type};
    my $override     = lc $opts{override};
    my $node         = $opts{node};
    my @storage_types = ("SCSI", "FCAL", "ATA", "EATA", "LUN", "SAS", "SATA", "SSD", "BSAS", "XSAS", "XATA", "MSATA", "FSAS");
    my @raid_types    = ("raid_dp", "raid4", "raid0");
    if (!(grep /$storage_type/i, @storage_types)) {
        $Log->comment(
        "$pkg_name: Unrecongnized value for storage_type: $storage_type, should be one of @storage_types"
        );
        return 0;
    }
    if (!(grep /$raid_type/i, @raid_types)) {
        $Log->comment(
             "$pkg_name: Unrecongnized value for raid_type: $raid_type, should be one of @raid_types"
        );
        return 0;
    }
    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }
    my $min_raid_size = RaidSqa_get_raid_size_value(api_set_obj  => $api_set_obj,
                                                    storage_type => $storage_type,
                                                    raid_type    => $raid_type,
                                                    raid_size    => "min",
                                                    override     => $override);
    my $max_raid_size = RaidSqa_get_raid_size_value(api_set_obj  => $api_set_obj,
                                                    storage_type => $storage_type,
                                                    raid_type    => $raid_type,
                                                    raid_size    => "max",
                                                    override     => $override);
    # checks that raid_size specified by user is in valide range as range are different for raid4/raidp for different type disks ATA/FCAL/SAS
    if ($raid_size >= $min_raid_size && $raid_size <= $max_raid_size) {
        $Log->comment(
            "$pkg_name: raid size:$raid_size for disktype: $storage_type disks and raidtype: $raid_type is in valid range ($min_raid_size-$max_raid_size)"
        );
        return 1;
    } else {
        $Log->comment(
            "$pkg_name: raid size:$raid_size for disktype: $storage_type disks and raidtype: $raid_type is not in valid range ($min_raid_size-$max_raid_size), can not continue test, rerun test with correct raid_size value"
        );
        return 0;
    }
} ## end sub RaidSqa_check_raid_size_valid_range

#------------------------------------------------------------------------------

=head2 RaidSqa_check_ioc_support

RaidSqa_check_ioc_support(api_set_obj=>$api_set_obj, node => $node, conntype=>$conntype,);

checks for ioc on root support in the build.

RETURNS
    0 on failure
    1 on success
ARGUMENTS

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_check_ioc_support {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $result      = 1;

    my $ret = RaidSqa_get_setflag(api_set_obj => $api_set_obj,
                                  flag_name   => "wafl_iron_optional_commit_root_enabled",
                                  node        => $node);

    if ((!defined($ret)) || ($ret eq "")) {
        $Log->comment("$pkg_name: IOC on root is not supported on current build");
        $result = 0;
    }

    $Log->comment("$pkg_name: IOC on root is supported on current build");
    return $result;
} ## end sub RaidSqa_check_ioc_support

#------------------------------------------------------------------------------

=head2 RaidSqa_wait_for_scrub_completion

RaidSqa_wait_for_scrub_completion(api_set_obj => $api_set_obj,  conntype=>$conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, timeout=>$timeout, node=>$node);

This subroutine by default wait till the scrub get completed on on all raid groups in filer. But if any of the options from filer_vol, plex_name or group_name is specified than it check scrub completion on that particular raid object.

It returns nothing

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Use filer_vol if we want to check scrub progress on that particular aggr/trad volume.

=item plex_name

(Optional) : Use plex_name if we want to check scrub progress on that particular plex.

=item group_name

(Optional) : Use group_name if we want to check scrub progress on that particular raid group.

=item timeout

(Optional): Value of timeout in secs to wait while checking the status again.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_wait_for_scrub_completion {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        node       => {type => SCALAR,         default => "local", optional => 1},}
                      );
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    my @group_list;
    if (defined($filer_vol)) {
        @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             filer_vol   => $filer_vol,
                                             node        => $node);
    } elsif (defined($plex_name)) {
        @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             plex_name   => $plex_name,
                                             node        => $node);
    } elsif (defined($group_name)) {
        push @group_list, $group_name;
    } else {
        @group_list =
          RaidSqa_get_group_list(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);
    }

    my $scrub_in_progress = 1;
    while ($scrub_in_progress == 1) {
        $scrub_in_progress = 0;
        my $group;
        foreach $group (@group_list) {
            my $group_obj = $raid_obj->raid_group("$group");
            if (!$group_obj) {
                $Log->comment("$pkg_name: Raid group with name $group does not exist");
                next;
            }
            my $group_scrub_progress = $group_obj->parameter("scrub_progress");
            if ($group_scrub_progress != -1) {
                $Log->comment("$pkg_name: scrub still $group_scrub_progress % on raid_group $group");
                $scrub_in_progress = 1;
                sleep(20);
            }
        } ## end foreach $group (@group_list)
        $raid_obj->refresh();
    } ## end while ($scrub_in_progress...
} ## end sub RaidSqa_wait_for_scrub_completion

#------------------------------------------------------------------------------

=head2 RaidSqa_wait_for_aggr_disks_zeroing_completion

RaidSqa_wait_for_aggr_disks_zeroing_completion(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol => $filer_vol, timeout=>$timeout, node => $node);

This subroutine wait till the disk zeroing in a particular aggregate/volume gets completed.

It returns nothing

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Required) : Name of aggregate or traditional volume to be checked .

=item timeout

(Optional): Value of timeout in secs to wait while checking the status again.

=item node

(Optional):  Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_wait_for_aggr_disks_zeroing_completion {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol => {type => SCALAR},
                        timeout   => {type => SCALAR, default => "10", optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $timeout     = $opts{timeout};
    my $node        = $opts{node};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    my %disklist = $aggr_obj->all_disks();
    my @disklist = keys(%disklist);
    my $count    = 1;
    my $disk;
    while ($count > 0) {
        $raid_obj->refresh();
        $aggr_obj = $raid_obj->aggregate("$filer_vol");
        %disklist = $aggr_obj->all_disks();
        @disklist = keys(%disklist);
        $count    = 0;
        $Log->comment("$pkg_name: disklist in test volume/aggr is: @disklist ");

        foreach $disk (@disklist) {
            my $disk_obj = $aggr_obj->disk("$disk");
            if (!$disk_obj) {
                $Log->comment("$pkg_name: disk with name $disk does not exist");
                next;
            }
            if (   $disk_obj->parameter("fsm_state") eq "ZEROING"
                || $disk_obj->parameter("fsm_state") eq "NOTZEROED") {
                $Log->comment("$pkg_name: disk $disk in test volume/aggr is not zeroed yet ");
                $count++;
            }
        } ## end foreach $disk (@disklist)
        $Log->comment("$pkg_name: waiting for $count disks to zero");
    last if($count == 0);
        sleep $timeout;
    } ## end while ($count > 0)
} ## end sub RaidSqa_wait_for_aggr_disks_zeroing_completion

#------------------------------------------------------------------------------

=head2 RaidSqa_wait_for_reparity_completion

RaidSqa_wait_for_reparity_completion(api_set_obj=>$api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, timeout=>$timeout, node=>$node);

This subroutine by default wait till the reparity gets completed on all raidgroups in filer. But if any of the options from filer_vol, plex_name or group_name is specified than it check reparity completion on that particular raid object.

It returns nothing

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Use filer_vol if we want to check reparity progress on that particular aggr/trad volume.

=item plex_name

(Optional) : Use plex_name if we want to check reparity progress on that particular plex.

=item group_name

(Optional) : Use group_name if we want to check reparity progress on that particular raid group.

=item timeout

(Optional): Value of timeout in secs to wait while checking the status again.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_wait_for_reparity_completion {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        timeout    => {type => SCALAR,         default => "10",  optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $timeout     = $opts{timeout};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj  = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj  = $raid_obj->aggregate("$filer_vol");
    my $plex_obj  = $raid_obj->plex("$plex_name");
    my $group_obj = $raid_obj->raid_group("$group_name");

    my @group_list;
    if (defined($aggr_obj)) {
        @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             filer_vol   => $filer_vol,
                                             node        => $node);
    } elsif (defined($plex_obj)) {
        @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             plex_name   => $plex_name,
                                             node        => $node);
    } elsif (defined($group_obj)) {
        @group_list = "$group_name";
    } else {
        @group_list =
          RaidSqa_get_group_list(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);
    }

    my $reparity_in_progress = 1;
    while ($reparity_in_progress == 1) {
        $reparity_in_progress = 0;
        my $group;
        foreach $group (@group_list) {
            my $group_obj = $raid_obj->raid_group("$group");
            if (!$group_obj) {
                $Log->comment("$pkg_name: Raid group with name $group does not exist");
                next;
            }
            my $group_reparity_progress = $group_obj->parameter("reparity_progress");
            if ($group_reparity_progress != -1) {
                $Log->comment(
                        "$pkg_name: reparity still $group_reparity_progress % on raid_group $group");
                $reparity_in_progress = 1;
            }
        } ## end foreach $group (@group_list)
        sleep $timeout;
        $raid_obj->refresh();
    } ## end while ($reparity_in_progress...
} ## end sub RaidSqa_wait_for_reparity_completion

#------------------------------------------------------------------------------

=head2 RaidSqa_wait_for_reconstruction_completion

RaidSqa_wait_for_reconstruction_completion(api_set_obj => $api_set_obj, conntype=>$conntype, filer_vol=>$filer_vol, plex_name=>$plex_name, group_name=>$group_name, disk_name=>$disk_name, timeout=>$timeout, node=>$node);

This subroutine by default wait till all reconstructions in a given filer get completed. But if any of the options from filer_vol, plex_name or group_name or disk_name is specified than it check reconstruction completion on that particular raid object.

It returns nothing

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item filer_vol

(Optional) : Name of aggr/trad volume to be checked .

=item plex_name

(Optional) : Name of plex to be checked .

=item group_name

(Optional) : Name of raid group to be checked .

=item disk_name

(Optional) : Name of disk to be checked .

=item timeout

(Optional): Value of timeout in secs to wait while checking the status again.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_wait_for_reconstruction_completion {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        plex_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        disk_name  => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        timeout    => {type => SCALAR,         default => "10",    optional => 1},
                        node       => {type => SCALAR,         default => "local", optional => 1},}
                      );

    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $filer_vol   = $opts{filer_vol};
    my $plex_name   = $opts{plex_name};
    my $group_name  = $opts{group_name};
    my $disk_name   = $opts{disk_name};
    my $timeout     = $opts{timeout};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my @group_list;
    if (defined($filer_vol)) {
        @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             filer_vol   => $filer_vol,
                                             node        => $node);
    } elsif (defined($plex_name)) {
        @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             plex_name   => $plex_name,
                                             node        => $node);
    } elsif (defined($group_name)) {
        push @group_list, $group_name;
    } else {
        @group_list =
          RaidSqa_get_group_list(api_set_obj => $api_set_obj, conntype => $conntype, node => $node);
    }

    my $recons_in_progress = 1;
    if (defined($disk_name)) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             disk_name   => $disk_name,
                                             node        => $node);
        while ($recons_in_progress == 1) {
            $recons_in_progress = 0;
            my $disk_obj = $raid_obj->disk("$disk_name");
            if (!$disk_obj) {
                $Log->comment("$pkg_name: Disk with name $disk_name does not exist");
                return 0;
            }
            if ($disk_obj->parameter("parent_state") =~ /RECONS/) {
                $Log->comment("$pkg_name: disk $disk_name is still doing reconstruction ");
                $recons_in_progress = 1;
            }
            sleep $timeout;
            $raid_obj->refresh();
        } ## end while ($recons_in_progress...

    } else {

        while ($recons_in_progress == 1) {
            $recons_in_progress = 0;
            my $group;
            foreach $group (@group_list) {
                my $group_obj = $raid_obj->raid_group("$group");
                if (!$group_obj) {
                    $Log->comment("$pkg_name: raid group with name $group does not exist");
                    next;
                }
                my $group_recons_progress = $group_obj->parameter("recons_progress");
                if ($group_recons_progress != -1) {
                    $Log->comment(
                        "$pkg_name: reconstruction still $group_recons_progress % on raid_group $group"
                    );
                    $recons_in_progress = 1;
                }
            } ## end foreach $group (@group_list)
            sleep $timeout;
            $raid_obj->refresh();
        } ## end while ($recons_in_progress...
    } ## end else [ if (defined($disk_name...
} ## end sub RaidSqa_wait_for_reconstruction_completion

#------------------------------------------------------------------------------

=head2 RaidSqa_wait_for_disk_sdc_completion

RaidSqa_wait_for_disk_sdc_completion(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name=>$disk_name, timeout=>$timeout, node=>$node);

This subroutine wait till the sdc on a particular disk get completed.

It returns nothing

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_name

(Required) : Name of disk on which sdc is running.

=item timeout

(Optional): Value of timeout in secs to wait while checking the status again.


=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_wait_for_disk_sdc_completion {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},
                        timeout   => {type => SCALAR, default => "10", optional => 1},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my $timeout     = $opts{timeout};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    my $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
        $disk_name =
          RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
        $disk_obj = $raid_obj->disk("$disk_name");
    }
    if (!$disk_obj) {
        $Log->comment("$pkg_name: Disk with name $disk_name does not exist");
        return 0;
    }

    my $disk_sdc_progress = $disk_obj->parameter("copy_progress");
    while ($disk_sdc_progress ne "-1") {
        $Log->comment("$pkg_name: sdc still $disk_sdc_progress % on disk $disk_name");
        sleep $timeout;
        $raid_obj->refresh();
        $disk_obj = $raid_obj->disk("$disk_name");
    if (!$disk_obj) {
            $disk_name =
                    RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj,
                                             conntype    => $conntype,
                                             disk_name   => $disk_name,
                                             node        => $node);
            $disk_obj = $raid_obj->disk("$disk_name");
        }
        if (defined($disk_obj)) {
            $disk_sdc_progress = $disk_obj->parameter("copy_progress");
        } else {
            last;
        }
    } ## end while ($disk_sdc_progress...
} ## end sub RaidSqa_wait_for_disk_sdc_completion

#------------------------------------------------------------------------------

=head2 RaidSqa_wait_for_spare_disks_zeroing_completion

RaidSqa_wait_for_spare_disks_zeroing_completion(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_list=>\@disk_list, timeout=>$timeout, node=>$node);

This subroutine wait till the disk zeroing on list of spare disks get completed.

It returns nothing

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_list

(Optional) : list of spare disk for which we want to check and wait till zeroing completion. Default is undefined.

=item timeout

(Optional): Value of timeout in secs to wait while checking the status again.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_wait_for_spare_disks_zeroing_completion {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT,   isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR,   default => "#ssh", optional => 1},
                        disk_list   => {type => ARRAYREF, default => undef, optional => 1},
                        timeout     => {type => SCALAR,   default => "10", optional => 1},
                        node        => {type => SCALAR,   default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_list   = $opts{disk_list};
    my $timeout     = $opts{timeout};
    my $node        = $opts{node};

    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if (!RaidSqa_get_disk_list_by_fsm_state(api_set_obj => $api_set_obj,
                                            conntype    => $conntype,
                                            spare_only  => "yes",
                                            fsm_state   => "ZEROING",
                                            node        => $node)
      ) {
        $Log->comment("$pkg_name: There is no spare disk in the systen which is zeroing");
        return 0;
    } else {
        my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
        my @disk_list = ();

        if (!defined($disk_list)) {
            @disk_list =
              RaidSqa_get_disk_list_by_fsm_state(api_set_obj => $api_set_obj,
                                                 conntype    => $conntype,
                                                 spare_only  => "yes",
                                                 fsm_state   => "ZEROING",
                                                 node        => $node);
        } else {
            @disk_list = @$disk_list;
        }

        # keep on checking until all spare disks are zeroed
        my $count = 1;
        my $disk;
        while ($count > 0) {
            $raid_obj->refresh();
            $count = 0;
            $Log->comment("$pkg_name: list of spare disks is: @disk_list ");
            @disk_list = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_list => \@disk_list);
            foreach $disk (@disk_list) {
                my $disk_obj = $raid_obj->disk("$disk");
                if (!$disk_obj) {
                    $Log->comment("$pkg_name: Disk with name $disk does not exist");
                    next;
                }
                if ($disk_obj->parameter("fsm_state") eq "ZEROING") {
                    $Log->comment("$pkg_name: disk $disk is not zeroed yet ");
                    $count++;
                }
            } ## end foreach $disk (@disk_list)
        } ## end while ($count > 0)
        $Log->comment("$pkg_name: waiting for $count disks to zero");
        sleep $timeout;
    } ## end else [ if (!RaidSqa_get_disk_list_by_fsm_state...
} ## end sub RaidSqa_wait_for_spare_disks_zeroing_completion

#------------------------------------------------------------------------------

=head2 RaidSqa_get_labelmaint_list

RaidSqa_get_labelmaint_list(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node);

This subroutine gets an array of disks which are in OLM.

It returns an array of olm disk on success.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_labelmaint_list {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $output;
    my $result = 1;

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; labelmaint list\"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };

    my @olm_list = split(/\r?\n\s/, $output);
    $Log->comment("$pkg_name: Here is the list of disk in olm: @olm_list");
    return @olm_list;

} ## end sub RaidSqa_get_labelmaint_list

#------------------------------------------------------------------------------

=head2 RaidSqa_labelmaint_isolate

RaidSqa_labelmaint_isolate(api_set_obj=>$api_set_obj, conntype=>$conntype, arg_list=>\@arg_list, node=>$node);

This subroutine isolate a list of disks/volumes/or-mix of them for OLM.

It returns nothing

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item arg_list

(Required): Reference to a list of argument need to be isolated. The list can have all disks, all volume name or mix of both.

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_labelmaint_isolate {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        arg_list => {type => ARRAYREF},
                        node     => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $arg_list    = $opts{arg_list};
    my $node        = $opts{node};

    my @arg_list = @$arg_list;
    my ($response, $output);
    my @error_strings = (
        "doesn't exist or is ineligible",
        "No disk or aggregate named",
        "Only non-filesystem disks or offline aggregates may be isolated into Online Maintenance Mode"
    );

    my $regex = join "|", @error_strings;
    # labelmaint isolate a list of aggr/volume and spare list
    if (!@arg_list) {
        $Log->comment("$pkg_name: Need to define at least one parameter to this funtion");
        return 0;
    } else {
        $Log->comment("$pkg_name: Isolating @arg_list");
        try {
            $response =
              $api_set_obj->system_node_run(
                                     node    => $node,
                                     command => "\"priv set -q diag; labelmaint isolate @arg_list\""
              );
            $output = $response->get_raw_output();
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
        };
    } ## end else [ if (!@arg_list)

    if ($output =~ /($regex)/mi) {
        return 0;
    } else {
        return 1;
    }
} ## end sub RaidSqa_labelmaint_isolate

#------------------------------------------------------------------------------

=head2 RaidSqa_labelmaint_unisolate

RaidSqa_labelmaint_unisolate(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node);

This subroutine unisolate all disk from olm to normal mode.

It returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_labelmaint_unisolate {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};

    my $output;
    $Log->comment("$pkg_name: Unisolating any disks in OLM...");

    try {
        my $response =
          $api_set_obj->system_node_run(node    => $node,
                                        command => "\"priv set -q diag; labelmaint unisolate\"");
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
    };

    my @olm_list = RaidSqa_get_labelmaint_list(api_set_obj => $api_set_obj,
                                               conntype    => $conntype,
                                               node        => $node);
    if (grep /No disks in online maintenance/, @olm_list) {
        $Log->comment("$pkg_name: There is no disk in labelmaint list");
        return 1;
    } else {
        $Log->comment("$pkg_name: There is still disk in labelmaint list");
        return 0;
    }
} ## end sub RaidSqa_labelmaint_unisolate

#------------------------------------------------------------------------------
=head2 RaidSqa_get_label_examine

RaidSqa_get_label_examine(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_name => $disk_name, node => $node_name);

This subroutine returns (hashref) output of command 'label examine' in key-value pair format.

The following arguments are accepted:

=over

=item api_set_obj

(Required): Hostspec to filer

=item node

(Optional): Node on which the aggregate is to be created
            Implementation still pending in function

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item disk_name

(Required)):  name of the disk

=back

=cut
#------------------------------------------------------------------------------

sub RaidSqa_get_label_examine {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name   => {type => SCALAR}});
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};
    my $conntype    = $opts{conntype};
    my $disk_name   = $opts{disk_name};
    my ($output, $result);
    $result = 1;
    try {
        my $response =
          $api_set_obj->system_node_run(
                     node    => $node,
                     command => "\"priv set -q diag; label examine $disk_name -a\"",
          );
        $output = $response->get_raw_output();
        $Log->comment("$pkg_name: label examine cmd is successful\n");
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $Log->comment("$pkg_name: label examine failed ");
        $result = 0;
    };
    return {} if (!$result);

    my @output = split(/\r*\n/, $output);

    #get index from where table starts
    my $index;
    for ($index = 0 ; $index <= $#output ; $index++) {
        if ($output[$index] =~ /label_crc/) {
            last;
        }
    }
    my @arr = split /,/, $output[$index];
    my @output_hash=();
    map {push(@output_hash, split(/=/,$_))} @arr;
    my %output_hash = @output_hash;
    return \%output_hash;

} ## end sub RaidSqa_get_label_examine

#-----------------------------------------------------------------------------

=head2 RaidSqa_get_label_vuinfo

RaidSqa_get_label_vuinfo(api_set_obj=>$api_set_obj, conntype=>$conntype, node => $node disk_list=>\@disk_list);

This subroutines parses the output of label vuinfo command and returns values for different parameters.

It returns a reference to a hash

ref_hash= {
            disk => {
                        error               => <ErrorString>  #This will be defined if it is not able to executed command on disk
                        group_raid_lw_scrub => <value>
                        lw_disk_signature   => <value>
                        cxt_parity          => <value>
                        ha_policy           => <value>
                        <param_name>        => <value>
                    }
          }

The following arguments are accepted:

=over


=item api_set_obj

(Required): A NACL::APISet object

=item node

(Optional): Node name on which the operation is to be performed

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item disk_list

(required) : list of disks for which vuinfo command output is required

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_label_vuinfo {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => "NACL::APISet"},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_list => {type => ARRAYREF},
                        node      => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my @disk_list   = @{$opts{disk_list}};
    my $result      = 1;

    my @param_list = ('group_raid_lw_scrub', 'lw_disk_signature',
                      'cxt_parity',          'ha_policy',
                      'Raidtree',            'magic',
                      'version',             'time',
                      'ID',                  'type',
                      'ndisks',              'striping',
                      'max groupsize',       'tricky_heritage',
                      'cksum type',          'nplex',
                      'plexid',              'ngroup',
                      'rgid',                'ndisk',
                      'rgdn',                'global id',
                      'vol_flag',            'disk_cksum_type',
                      'cksum_zone_size',     'cksum_offset',
                      'vr_start',            'size');
    my %label_vuinfo_output = ();

    foreach my $disk (@disk_list) {
        my $output = '';
        try {
            my $response =
              $api_set_obj->system_node_run(node    => $node,
                                            command => "\"priv set -q diag; label vuinfo $disk\" ");
            $output = $response->get_raw_output();
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            $result = 0;
        };
        if ($output =~ /(Unable to read undestroy information)|(No undestroy information found)/m) {
            $Log->comment("$pkg_name: Label vuinfo command failed for $disk");
            $label_vuinfo_output{$disk}->{error} = $output;
            next;
        }
        my @lines = split(/\r?\n/, $output);
        foreach my $line (@lines) {
            foreach my $param (@param_list) {
                if ($line =~ /$param\s(\w+)/) {
                    $label_vuinfo_output{$disk}->{$param} = $1;
                }
            }
        }
    } ## end foreach my $disk (@disk_list)

    $Log->comment("$pkg_name: " . Dumper(\%label_vuinfo_output));
    if (!(keys %label_vuinfo_output)) {
        $result = 0;
    } else {
        $result = \%label_vuinfo_output;
    }
    return $result;
} ## end sub RaidSqa_get_label_vuinfo
#------------------------------------------------------------------------------

=head2 RaidSqa_get_label_buildtree_disk_args

RaidSqa_get_label_buildtree_disk_args(api_set_obj=>$api_set_obj, filer_vol => $filer_vol);

It returns a disk string in a format which is required in label buildtree command
In Scalar context, it returns disk string required to be used in buildtree command

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item filer_vol

(Required) : Name of aggregate  or traditional volume whose disk string is to be created.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_label_buildtree_cmd_string {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        filer_vol => {type => SCALAR},});
    my $api_set_obj    = $opts{api_set_obj};
    my $filer_vol      = $opts{filer_vol};
    my %aggr_disk_list = ();

    my @plex_list = RaidSqa_get_plex_list(api_set_obj => $api_set_obj,
                                          filer_vol => $filer_vol);
    my @buildtree_disk_list = ();
    my ($plex_index, $rg_index);
    foreach my $plex (@plex_list) {
        $plex_index = undef;
        $rg_index   = undef;
        if ($plex =~ /plex(\d+)/) {
            $plex_index = $1;
            my @group_list =
              RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                     plex_name => $plex);
            foreach my $group (@group_list) {
                if ($group =~ /rg(\d+)/) {
                    $rg_index = $1;
                    my @disk_list =
                      RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                            group_name => $group);
                    my %disk_rgdn = ();
                    foreach my $disk (@disk_list) {
                        my $disk_rgdn =
                          RaidSqa_get_disk_rgdn(api_set_obj => $api_set_obj,
                                                disk_name => $disk);
                        $disk_rgdn{$disk_rgdn} = $disk;
                    }
                    my @buildtree_disk_list = ();
                    foreach my $rgdn (sort keys %disk_rgdn) {
                        push(@buildtree_disk_list, $disk_rgdn{$rgdn});
                    }
                    $aggr_disk_list{$plex_index}->{$rg_index} = \@buildtree_disk_list;
                } else {
                    $Log->comment("RadSqa_Cmodev3.pm Could not find rg_index for group $group");
                    return 0;
                }
            } ## end foreach my $group (@group_list)
        } else {
            $Log->comment("RadSqa_Cmodev3.pm Could not find plex index for plex $plex");
            return 0;
        }
    } ## end foreach my $plex (@plex_list)

    $Log->comment("RadSqa_Cmodev3.pm aggr_disk_list: " . Dumper(\%aggr_disk_list));
    my $disk_arg    = "";
    my @plex_indexs = sort { $a <=> $b } keys %aggr_disk_list;
    my $first_index = shift @plex_indexs;
    foreach my $plex_index (sort { $a <=> $b } keys %aggr_disk_list) {
        if ($plex_index !~ /$first_index/) {
            $disk_arg .= " -p";
        }
        foreach my $rg_index (sort { $a <=> $b } keys %{$aggr_disk_list{$plex_index}}) {
            if ($rg_index !~ /0/) {
                $disk_arg .= " -r";
            }
            $disk_arg .= " " . join(" ", @{$aggr_disk_list{$plex_index}->{$rg_index}});
        }
    } ## end foreach my $plex_index (sort...
    $Log->comment("RadSqa_Cmodev3.pm disk_arg: $disk_arg");
    return $disk_arg;
} ## end sub RaidSqa_get_label_buildtree_cmd_string

#------------------------------------------------------------------------------

=head2 RaidSqa_label_buildtree

RaidSqa_label_buildtree(api_set_obj=>$api_set_obj, filer_vol => $filer_vol, disk_args=>$disk_args, fsid=>$fsid, raid_type=>$raid_type, aggr_bits=>$aggr_bits, aggr_type => $aggr_type, options=>$options, hybrid=>$hybrid, checksum_type=>$checksum_type);

This function executes buildtree command on the given disk list

It returns 1 on success and 0 on failure

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item filer_vol

(Required) : Name of aggregate  or traditional volume to be onlined.

=item disk_args

(Required) : disk argument to be sent to buildtree command

=item fsid

(Required) : fsid value of test volume

=item aggr_bits

(Required) : 32 for 32-bit aggr/volume, 64 for 64-bit aggr/volume

=item raid_type

(Required) : Raid type parameter (possible values: raid_dp, raid4)

=item options

(Required) : Other options to be passed to buildtree command (e.g -lw)

=item aggr_type

(Required): 1 for aggregate and 0 for traditional. Default it is 1.

=item checksum_type

(Optional): default b. a for advanced_zoned, b for block, m for mixed, z for zoned

=item hybrid
(Optional): default 0. 1 for hybrid, 0 for non-hybrid

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_label_buildtree {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj   => {type => OBJECT, isa     => 'NACL::APISet'},
                        filer_vol     => {type => SCALAR},
                        disk_args     => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        fsid          => {type => SCALAR},
                        checksum_type => {type => SCALAR, default =>"b", optional => 1},
                        aggr_type     => {type => SCALAR, default => 1, optional => 1},
                        aggr_bits     => {type => SCALAR, default => "32", optional => 1},
                        raid_type     => {type => SCALAR, default => "raid_dp", optional => 1},
                        hybrid        => {type => SCALAR, default => 0,  optional => 1},
                        options       => {type => SCALAR | UNDEF, default => undef, optional => 1},});

    my $api_set_obj   = $opts{'api_set_obj'};
    my $filer_vol     = $opts{'filer_vol'};
    my $options       = $opts{'options'};
    my $disk_args     = $opts{'disk_args'};
    my $fsid          = $opts{'fsid'};
    my $aggr_type     = $opts{'aggr_type'};
    my $aggr_bits     = $opts{'aggr_bits'};
    my $raid_type     = $opts{'raid_type'};
    my $opt_aggr      = ($aggr_type) ? "-a" : "";
    my $opt_aggr_bits = ($aggr_bits eq "64") ? "-B" : "";
    my $opt_hybrid    = ($opts{'hybrid'}) ? "-H" : "";
    my $checksum_type;

    if($opts{'checksum_type'}=~/^(block|b)$/i) {
        $checksum_type = "b";
    } elsif ($opts{'checksum_type'}=~/^(advanced_zoned|a)$/i) {
        $checksum_type = "a";
    } elsif ($opts{'checksum_type'}=~/^(mixed|m)$/i) {
        $checksum_type = "m";
    } elsif ($opts{'checksum_type'}=~/^(zoned|z)$/i) {
        $checksum_type = "z";
    } else {
        $Log->comment("$pkg_name: invalid checksum ".$opts{'checksum_type'});
        return 0;
    }

    if (defined($disk_args)) {
        my $cmd = "priv set -q diag;label buildtree -n $filer_vol -f $fsid -c $checksum_type -t $raid_type $opt_hybrid $opt_aggr_bits $opt_aggr $options -F $disk_args";
        RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd => $cmd);
        sleep(5);
        $Log->comment("RadSqa_Cmodev3.pm label buildtree is executed successfully");
        return 1;
    } else {
        $Log->comment("RadSqa_Cmodev3.pm disk_args is mandatory for non-interactive buildtree command");
        return 0;
    }
} ## end sub RaidSqa_label_buildtree

#------------------------------------------------------------------------------

=head2 RaidSqa_get_storage_failover_field

RaidSqa_get_storage_failover_field(api_set_obj=>$api_set_obj, field_name=>$field_name, conntype=>$conntype, node=>$node);

This subroutine enables storage failover policy on Cmode filer

It returns the value for field defined by arg field_name

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item field_name

(Optional): Name of field in storage failover command whose value is to be returned

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_storage_failover_field {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        field_name => {type => SCALAR},
                        node       => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $field_name  = $opts{field_name};
    my $node        = $opts{node};

    my $result = 1;
    my ($output, $field_value);

    try {
        my $response = $api_set_obj->storage_failover_show(node => $node, fields => $field_name);
        $output      = $response->get_parsed_output();
        $field_value = $output->[0]->{$field_name};
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        return 0;
    };
    $Log->comment("$pkg_name: Field $field_name is set to $field_value");
    return $field_value;

} ## end sub RaidSqa_get_storage_failover_field

#------------------------------------------------------------------------------

=head2 RaidSqa_set_storage_failover_field

RaidSqa_set_storage_failover_field(api_set_obj=>$api_set_obj,  conntype=>$conntype, node=>$node);

This subroutine enables storage failover policy on Cmode filer

It returns the command execution status(0/1)

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".
=item field_name
(Required): storage failover field to set.

=item field_value
(Required): storage failover field value.

=item node
(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------
sub RaidSqa_set_storage_failover_field {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        field_name  => {type => SCALAR},
                        field_value       => {type => SCALAR},
                        node        => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $field       = $opts{field_name};
    my $value       = $opts{value};
    my $result      = 1;
    my ($output, $status);

    $status = RaidSqa_get_storage_failover_field(api_set_obj => $api_set_obj,
                                                 field_name  => $field,
                                                 conntype    => $conntype,
                                                 node        => $node);
    if ($status eq $value) {
   $Log->comment("$pkg_name: Storage failover field is already set to $status");
        return 1;
    } else {
        try {
            my $response = $api_set_obj->storage_failover_modify(node => $node, "$field"=>"$value");
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            return 0;
        };
    } ## end else [ if ($status eq "true")

    $status = RaidSqa_get_storage_failover_field(api_set_obj => $api_set_obj,
                                                 field_name  => $field,
                                                 conntype    => $conntype,
                                                 node        => $node);

    if ($status eq $value) {
        $Log->comment("$pkg_name: Storage failover field is set to $status");
   $Log->comment("$pkg_name: Storage failover field is set to $status");
        return 1;
    } else {
        $Log->comment("$pkg_name: Storage failover field could not be set to $status");
        return 0;
    }
} ## end sub RaidSqa_set_storage_failover_field

#------------------------------------------------------------------------------
=head2 RaidSqa_enable_storage_failover_status

RaidSqa_enable_storage_failover_status(api_set_obj=>$api_set_obj,  conntype=>$conntype, node=>$node);

This subroutine enables storage failover policy on Cmode filer

It returns the command execution status(0/1)

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_enable_storage_failover_status {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};

    my $result = 1;
    my ($output, $status);

    $status = RaidSqa_get_storage_failover_field(api_set_obj => $api_set_obj,
                                                 field_name  => 'enabled',
                                                 conntype    => $conntype,
                                                 node        => $node);
    if ($status eq "true") {
        $Log->comment("$pkg_name: Storage failover is already enabled");
        return 1;
    } else {
        try {
            my $response = $api_set_obj->storage_failover_modify(node => $node, enabled => "true");
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            return 0;
        };
    } ## end else [ if ($status eq "true")

    $status = RaidSqa_get_storage_failover_field(api_set_obj => $api_set_obj,
                                                 field_name  => 'enabled',
                                                 conntype    => $conntype,
                                                 node        => $node);

    if ($status eq "true") {
        $Log->comment("$pkg_name: Storage failover is enabled");
        return 1;
    } else {
        $Log->comment("$pkg_name: Storage failover could not be enabled");
        return 0;
    }
} ## end sub RaidSqa_enable_storage_failover_status

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_enable_ha

RaidSqa_enable_ha(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node);

This subroutine enables cluster ha on a given node

It returns the command execution status(0/1)

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_enable_ha {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $field_name  = $opts{field_name};
    my $node        = $opts{node};

    my $result = 1;
    my ($output, $status);

    $status = RaidSqa_get_ha_state(api_set_obj => $api_set_obj, conntype => $conntype);
    if ($status) {
        $Log->comment("$pkg_name: HA is already enabled");
        return 1;
    } else {
        try {
            my $response = $api_set_obj->cluster_ha_modify(configured => "true");
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            return 0;
        };
    } ## end else [ if ($status)

    $status = RaidSqa_get_ha_state(api_set_obj => $api_set_obj, conntype => $conntype);

    if ($status) {
        $Log->comment("$pkg_name: Cluster HA is enabled");
        return 1;
    } else {
        $Log->comment("$pkg_name: Cluster HA could not be enabled");
        return 0;
    }
} ## end sub RaidSqa_enable_ha

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_disable_ha

RaidSqa_disable_ha(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node);

This subroutine disables cluster ha on a given node

It returns the command execution status(0/1)

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_disable_ha {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $field_name  = $opts{field_name};
    my $node        = $opts{node};

    my $result = 1;
    my ($output, $status);

    $status = RaidSqa_get_ha_state(api_set_obj => $api_set_obj, conntype => $conntype);
    if (!$status) {
        $Log->comment("$pkg_name: HA is already disabled");
        return 1;
    } else {
        try {
            my $response = $api_set_obj->cluster_ha_modify(configured => "false");
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            return 0;
        };
    } ## end else [ if (!$status)

    $status = RaidSqa_get_ha_state(api_set_obj => $api_set_obj, conntype => $conntype);

    if (!$status) {
        $Log->comment("$pkg_name: Cluster HA is disabled");
        return 1;
    } else {
        $Log->comment("$pkg_name: Cluster HA could not be disabled");
        return 0;
    }
} ## end sub RaidSqa_disable_ha

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_get_ha_state

RaidSqa_get_ha_state(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node);

This subroutine disables cluster ha on a given node

It returns the command execution status(0/1)

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_ha_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $field_name  = $opts{field_name};
    my $node        = $opts{node};

    my $result = 1;
    my ($output, $state);

    $state = 0;
    try {
        my $response = $api_set_obj->cluster_ha_show();
        $output = $response->get_parsed_output();
        my $configured = $output->[0]->{'configured'};
        $state = ($configured eq "true") ? 1 : 0;
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        $state  = 0;
        return 0;
    };

    if ($state) {
        $Log->comment("$pkg_name: Cluster HA is configured");
        return 1;
    } else {
        $Log->comment("$pkg_name: Cluster HA is not configured");
        return 0;
    }
} ## end sub RaidSqa_get_ha_state

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_verify_storage_failover_status

RaidSqa_verify_storage_failover_status(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node);

This subroutine enables storage failover policy on Cmode filer

It returns 1 if storage failover state is set to connected else 0

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_verify_storage_failover_status {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};

    my ($output, $field_value);

    my $state = RaidSqa_get_storage_failover_field(api_set_obj => $api_set_obj,
                                                   field_name  => "state",
                                                   conntype    => $conntype,
                                                   node        => $node);
    if ($state eq "connected") {
        $Log->comment("$pkg_name: Storage failover state is set to connected");
        return 1;
    } else {
        $Log->comment("$pkg_name: Storage failover state is not set to connected");
        return 0;
    }
} ## end sub RaidSqa_verify_storage_failover_status

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_giveback

RaidSqa_giveback(api_set_obj=>$api_set_obj, conntype=>$conntype, fromnode=>$node);

This subroutine enables storage failover policy on Cmode filer

It returns 1 if giveback is successful

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item fromnode

(Optional): Node name used for giveback

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_giveback {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        fromnode    => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $fromnode    = $opts{fromnode};

    my $result = 0;
    my ($output, $field_value);

    try {
        my $response =
          $api_set_obj->storage_failover_giveback(fromnode          => $fromnode,
                                                  'override-vetoes' => "true");
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        return 0;
    };

    for (my $count = 0 ; $count < 100 ; $count++) {
        my $state =
          RaidSqa_get_storage_failover_field(api_set_obj => $api_set_obj,
                                             field_name  => "state",
                                             conntype    => $conntype,
                                             node        => $fromnode);
        if ($state eq "connected") {
            $Log->comment("$pkg_name: Storage failover giveback successful");
            $result = 1;
            last;
        } else {
            sleep(30);
        }
    } ## end for (my $count = 0 ; $count...

    if ($result) {
        return 1;
    } else {
        $Log->comment("$pkg_name: Storage failover giveback failed");
        return 0;
    }

} ## end sub RaidSqa_giveback

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_takeover

RaidSqa_takeover(api_set_obj=>$api_set_obj, conntype=>$conntype, ofnode=>$node);

This subroutine enables storage failover policy on Cmode filer

It returns 1 if giveback is successful

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item ofnode

(Optional): Node name used for giveback

=item option

(Optional): additional option to be passed to takeover command

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_takeover {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        option => {type => SCALAR | UNDEF, default => undef,   optional => 1},
                        ofnode => {type => SCALAR,         default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $ofnode      = $opts{ofnode};
    my $option      = $opts{option};

    my $result = 0;
    my ($output, $field_value);

    try {
        my $response =
          $api_set_obj->storage_failover_takeover(ofnode => $ofnode, option => $option);
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        return 0;
    };

    for (my $count = 0 ; $count < 100 ; $count++) {
        my $state =
          RaidSqa_get_storage_failover_field(api_set_obj => $api_set_obj,
                                             field_name  => "state",
                                             conntype    => $conntype,
                                             node        => 'local');
        if ($state eq "takeover") {
            $Log->comment("$pkg_name: Storage failover takeover successful");
            $result = 1;
            last;
        } else {
            sleep(30);
        }
    } ## end for (my $count = 0 ; $count...

    if ($result) {
        return 1;
    } else {
        $Log->comment("$pkg_name: Storage failover giveback failed");
        return 0;
    }

} ## end sub RaidSqa_takeover

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_wait_for_cf_status

RaidSqa_wait_for_cf_status(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node);

This subroutine enables storage failover policy on Cmode filer

It returns 1 if giveback is successful

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name used for giveback

=item message

(Optional): Message expected for cf status command

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_wait_for_cf_status {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        message => {type => SCALAR},
                        node    => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $message     = $opts{message};

    my ($output, $field_value);

    my $result        = 1;
    my $state_reached = 0;

    #OUTER:
    for (my $count = 0 ; $count < 100 ; $count++) {
        last if (!$result);
        try {

            #TODO : cf_status cdefs is not defined
            my $output =
              $api_set_obj->execute_raw_command(command => "cf status -node $node -fields state");
            if ($output =~ /$message/) {
                $Log->comment("$pkg_name: Message \"$message\" is found in cf status output");
                $state_reached = 1;

                #last OUTER;
                last;
            } else {
                sleep(30);
            }
        } ## end try
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            $result = 0;
            return 0;
        };
    } ## end for (my $count = 0 ; $count...

    if ($state_reached && $result) {
        return 1;
    } else {
        $Log->comment("$pkg_name: Message \"$message\" not found in cf status output");
        return 0;
    }
} ## end sub RaidSqa_wait_for_cf_status

#----------------------------------------------------------------------------------------------------------

=head2 RaidSqa_wait_for_storage_failover_state

RaidSqa_wait_for_storage_failover_state(api_set_obj=>$api_set_obj, conntype=>$conntype, node=>$node);

This subroutine enables storage failover policy on Cmode filer

It returns 1 if giveback is successful

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL API object

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=item node

(Optional): Node name used for giveback

=item state

(Optional): Expected state of storage failover

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_wait_for_storage_failover_state {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        state => {type => SCALAR,},
                        node  => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $node        = $opts{node};
    my $state       = $opts{state};

    my $state_reached = 0;

    for (my $count = 0 ; $count < 100 ; $count++) {
        my $ostate =
          RaidSqa_get_storage_failover_field(api_set_obj => $api_set_obj,
                                             field_name  => 'state',
                                             conntype    => $conntype,
                                             node        => $node);
        if ($state eq $ostate) {
            $Log->comment("$pkg_name: Storage failover state reached to $state");
            $state_reached = 1;
            last;
        } else {
            sleep(30);
        }
    } ## end for (my $count = 0 ; $count...

    if ($state_reached) {
        return 1;
    } else {
        $Log->comment("$pkg_name: Storage failover state did not reach to state \"$state\"");
        return 0;
    }
} ## end sub RaidSqa_wait_for_storage_failover_state

#------------------------------------------------------------------------------

=head2 RaidSqa_ems_log_get_mark

This routine returns the current mark(timestamp in format <n>t) of the event
log

my $ret = RaidSqa_ems_log_get_mark(api_set_obj=>$api_set_obj);
my $ret = RaidSqa_ems_log_get_mark(api_set_obj=>$api_set_obj, timeout=>60);

The following named arguments are accepted,

=over

=item api_set_obj

(Required) : a NACL API object

=item timeout

(Optional) : Timeout value for the say. Default : 1200

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_ems_log_get_mark {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        timeout     => {type => SCALAR, default => 1200, optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $timeout     = $opts{timeout};
    my $node        = $opts{node};

    try {
        $api_set_obj->system_node_run(node    => $node,
                                      command => "options ems.log.dupsuppress.enable off");
        $api_set_obj->system_node_run(node    => $node,
                                      command => "options ems.autosuppress.enable off");
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;

        #Do not handle failure as failure is expected here
    };

    # To Disable ems duplicate events
    my $output;
    try {
        my $response =
          $api_set_obj->system_node_run(node                 => $node,
                                        command              => "ems log dump 1s",
                                        'connectrec-timeout' => $timeout);

        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $output = _handle_nacl_exception($exception_object);
        return 0;
    };


    $Log->comment("$pkg_name: ems log dump 1s output: $output");

    my (@lines) = split(/\r*\n/, $output);
    @lines = grep /<NR/, @lines;

    #Use header (NR) only to get the timestamp
    my %emslog_nr = emslog_parseNR($lines[0]);
    $Log->comment("$pkg_name: Timestamp is : " . $emslog_nr{t} . "t");
    return ($emslog_nr{t} . "t");
} ## end sub RaidSqa_ems_log_get_mark

#------------------------------------------------------------------------------

=head2 RaidSqa_ems_log_dump_by_type

This routine returns pointer to hash table having event name as key and
array of event ids as value.

my $event_by_type = RaidSqa_ems_log_dump_by_type(api_set_obj=>$api_set_obj);
OR
my $mark = RaidSqa_ems_log_get_mark(api_set_obj=>$api_set_obj);
my $event_by_type = RaidSqa_ems_log_dump_by_type(api_set_obj=>$api_set_obj, timestamp=>$mark);
$event = $event_by_type->{"dfu.firmwareDownloading"};
if ($event eq "") {
    croak "Disk firmware download is not started yet";
}

$event = $event_by_type->{"dfu.firmwareUpToDate"};
if ($event eq "") {
    croak "Disk firmware download didn't complete in given time";
}

The output looks like:
$VAR1 = {
          'fci.adapter.offlining' => [
                                       '1133838353/531',
                                       '1133838353/539'
                                     ],
          'fci.adapter.offline' => [
                                     '1133838353/532',
                                     '1133838353/540'
                                   ],
          'fci.adapter.onlining' => [
                                      '1133838353/536',
                                      '1133838353/543'
                                    ],
          'unowned.disk.reminder' => [
                                       '1133838353/529'
                                     ],
          'monitor.shelf.fault' => [
                                     '1133838353/527',
                                     '1133838353/528'
                                   ],
          'fci.adapter.online' => [
                                    '1133838353/537',
                                    '1133838353/544'
                                  ],
          'kern.syslog.msg' => [
                                 '1133838353/530',
                                 '1133838353/533',
                                 '1133838353/534',
                                 '1133838353/535',
                                 '1133838353/538',
                                 '1133838353/542'
                               ],
          'config.NotMultiPath' => [
                                     '1133838353/541'
                                   ],
          'kern.uptime.filer' => [
                                   '1133838353/526'
                                 ]
        };

The following named arguments are accepted,

=over

=item api_set_obj

(Required) : NACL API object

=item timestamp

(Optional) : Get the event from timestamp Default : ""(means only header)

Timestamp semantic can be :

<n>s    Last n seconds
<n>m    Last n minutes
<n>h    Last n hours
<n>d    Last n days
<n>t    Since timestamp <n>

=item log

(Optional) : The logfile to get events from. Default : "" (means current)

=item verbose

(Optional) : Set this Option to get the verbose output. Default is Yes.

=item timeout

(Optional) : Timeout value for the say. Default : 1200

=item ems_file

(Optional) : The ems_file to get events from. Default : "none"

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_ems_log_dump_by_type {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        timestamp   => {type => SCALAR, default => "", optional => 1},
                        log         => {type => SCALAR, default => "", optional => 1},
                        verbose     => {type => SCALAR, default => "yes", optional => 1},
                        timeout     => {type => SCALAR, default => 1200, optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        ems_file    => {type => SCALAR, default => "none", optional => 1},});

    my $emslog_entry;
    my @emslog_events;
    my @emslog_event_lines;
    my %emslog_event_by_id;
    my %emslog_event_by_type;
    my %emslog_nr;
    my @lines;
    my $verbose     = " ";
    my $output      = " ";
    my $api_set_obj = $opts{api_set_obj};
    my $timeout     = $opts{timeout};
    my $node        = $opts{node};

    if ($opts{verbose} eq "no") {
        $verbose = "-noverbose";
    }

    if ($opts{ems_file} eq "none") {
        my $cmd = "ems log dump $opts{log} $opts{timestamp}";
        try {
            my $response =
              $api_set_obj->system_node_run(node                 => $node,
                                            command              => "$cmd",
                                            'connectrec-timeout' => $timeout);
            $output = $response->get_raw_output();
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            return 0;
        };
        @lines = split(/\r*\n/, $output);
    } else {
        my $result = 1;
        open(FILE, "< $opts{ems_file}") or $result = 0;
        if (!$result) {
            $Log->comment("$pkg_name: Could not open file $opts{ems_file} for reading: $!");
            return 0;
        }
        while (<FILE>) {
            push @lines, $_;
        }
    } ## end else [ if ($opts{ems_file} eq...

    foreach my $line (@lines) {
        if (!emslog_parse($line, \%emslog_nr, \$emslog_entry)) {
            next;
        }
        emslog_process_line($emslog_entry,        \@emslog_events, \@emslog_event_lines,
                            \%emslog_event_by_id, \%emslog_event_by_type);
        $emslog_entry = "";
    }
    return (\%emslog_event_by_type);
} ## end sub RaidSqa_ems_log_dump_by_type

#------------------------------------------------------------------------------

=head2 RaidSqa_ems_log_dump_by_id

This routine returns pointer to hash table having event info by id

my $ret = RaidSqa_ems_log_dump_by_id(api_set_obj=>$api_set_obj);
my $ret = RaidSqa_ems_log_dump_by_id(api_set_obj=>$api_set_obj, timeout=>300, timestamp=>23456788t);

The output looks like:
$VAR1 = {
          '1133838353/530' => {
                                'vfilerName' => '',
                                'header' => {
                                              'n' => 'raid-u23',
                                              'o' => 'rc',
                                              'p' => 6,
                                              's' => 'OK',
                                              't' => '1134007568',
                                              'd' => '08Dec2005 02:06:08',
                                              'vf' => '',
                                              'id' => '1133838353/530'
                                            },
                                'msg' => 'Taking adapter 6a offline at administrator&apos;s request.&nl;',
                                'type' => 'kern.syslog.msg',
                                'attr' => 'msg,vfilerName'
                              },
          '1133838353/531' => {
                                'header' => {
                                              'n' => 'raid-u23',
                                              'o' => 'rc',
                                              'p' => 6,
                                              's' => 'OK',
                                              't' => '1134007568',
                                              'd' => '08Dec2005 02:06:08',
                                              'vf' => '',
                                              'id' => '1133838353/531'
                                            },
                                'adapterName' => '6a',
                                'type' => 'fci.adapter.offlining',
                                'attr' => 'adapterName'
                              },
          .
          .
          .
}


The following named arguments are accepted,

=over

=item api_set_obj

(Required) : NACL API object

=item timestamp

(Optional) : Get the event from this timestamp Default : ""(means only header)

Timestamp semantic can be :

<n>s    Last n seconds
<n>m    Last n minutes
<n>h    Last n hours
<n>d    Last n days
<n>t    Since timestamp <n>

=item log

(Optional) : The logfile to get events from. Default : "" (means current)

=item verbose

(Optional) : Set this Option to get the verbose output. Default is Yes.

=item timeout

(Optional) : Timeout value for the say. Default : 1200: Increasing the default
value as the log file might be quite big and can take longer to dump the file

=item ems_file

(Optional) : The ems_file to get events from. Default : "none"

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_ems_log_dump_by_id {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        timestamp   => {type => SCALAR, default => "", optional => 1},
                        log         => {type => SCALAR, default => "", optional => 1},
                        verbose     => {type => SCALAR, default => "yes", optional => 1},
                        timeout     => {type => SCALAR, default => 1200, optional => 1},
                        node        => {type => SCALAR, default => "local", optional => 1},
                        ems_file    => {type => SCALAR, default => "none", optional => 1},});

    my $emslog_entry;
    my @emslog_events;
    my @emslog_event_lines;
    my %emslog_event_by_id;
    my %emslog_event_by_type;
    my %emslog_nr;
    my @lines;
    my $verbose = " ";
    my $output;
    my $api_set_obj = $opts{api_set_obj};
    my $timeout     = $opts{timeout};
    my $node        = $opts{node};

    if ($opts{verbose} eq "no") {
        $verbose = "-noverbose";
    }

    if ($opts{ems_file} eq "none") {
        my $cmd = "ems log dump $opts{log} $opts{timestamp}";
        try {
            my $response =
              $api_set_obj->system_node_run(node                 => $node,
                                            command              => "$cmd",
                                            'connectrec-timeout' => $timeout);
            $output = $response->get_raw_output();
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $output = _handle_nacl_exception($exception_object);
            return 0;
        };
        @lines = split(/\r*\n/, $output);
    } else {
        my $result = 1;
        open(FILE, "< $opts{ems_file}") or $result = 0;
        if (!$result) {
            $Log->comment("$pkg_name: Could not open file $opts{ems_file} for reading: $!");
            return 0;
        }
        while (<FILE>) {
            push @lines, $_;
        }
    } ## end else [ if ($opts{ems_file} eq...

    foreach my $line (@lines) {
        if (!emslog_parse($line, \%emslog_nr, \$emslog_entry)) {
            next;
        }
        emslog_process_line($emslog_entry,        \@emslog_events, \@emslog_event_lines,
                            \%emslog_event_by_id, \%emslog_event_by_type);
        $emslog_entry = "";
    }
    return (\%emslog_event_by_id);
} ## end sub RaidSqa_ems_log_dump_by_id

#------------------------------------------------------------------------------

=head2 RaidSqa_ems_log_check

This routine returns 0 if a given event on given objectname is not found from a specified timestamp otherwise returns <n> for the n number of instances found in the log.

my $mark = RaidSqa_ems_log_get_mark(api_set_obj=>$api_set_obj);
my $ret = RaidSqa_ems_log_check(api_set_obj=>$api_set_obj, eventtype=>"fci.adapter.offline", key=>"adapterName" value=>"6a", timestamp=>$mark);
if ($ret == 0) {
    $Log->comment("The specified event is not found in the ems log");
} else {
    $Log->comment("Found the specified ems event $ret times in the ems log");
}

The following named arguments are accepted,

=over

=item api_set_obj

(Required) : NACL API object

=item eventtype

(Required) : The type of events to look into, like "fci_adapter_offline"...

=item key

(Required) : What is the key (the objectname for a given event type) to look into the event
message. For example, in the following ems message key is "adapterName":

<LR d="07Dec2005 18:45:31" n="raid-u23" t="1133981131" id="1133838353/404" p="6" s="OK" o="ispfc_init_2" vf="">
<fci_adapter_offline_1
        adapterName="6a"/>
</LR>

=item value

(Required) : What is the value (the objectname for a given event type) to look into for the
given event type and key. To check for the above ems message the value would be "6a".

=item timestamp

(Required) : Get the event from this timestamp.

Timestamp semantic can be :

<n>s    Last n seconds
<n>m    Last n minutes
<n>h    Last n hours
<n>d    Last n days
<n>t    Since timestamp <n>

=item log

(Optional) : The logfile to get events from. Default : "" (means current)

=item timeout

(Optional) : Timeout value for the say. Default : 1200: Increasing the default
value as the log file might be quite big and can take longer to dump the file

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_ems_log_check {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa => 'NACL::APISet'},
                        eventtype   => {type => SCALAR},
                        key         => {type => SCALAR},
                        value       => {type => SCALAR},
                        timestamp   => {type => SCALAR},
                        log     => {type => SCALAR, default => "",      optional => 1},
                        node    => {type => SCALAR, default => "local", optional => 1},
                        timeout => {type => SCALAR, default => 1200,    optional => 1},});

    my $event_by_type;
    my $event_by_id;
    my $api_set_obj = $opts{api_set_obj};
    my $node        = $opts{node};

    if (defined($opts{log})) {
        $event_by_type =
          RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj,
                                       timestamp   => $opts{timestamp},
                                       timeout     => $opts{timeout},
                                       log         => $opts{log});
    } else {
        $event_by_type =
          RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj,
                                       timestamp   => $opts{timestamp},
                                       timeout     => $opts{timeout});
    }

    if (defined($opts{log})) {
        $event_by_id = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj,
                                                  timestamp   => $opts{timestamp},
                                                  timeout     => $opts{timeout},
                                                  log         => $opts{log});
    } else {
        $event_by_id = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj,
                                                  timestamp   => $opts{timestamp},
                                                  timeout     => $opts{timeout});
    }

    my $event = $event_by_type->{"$opts{eventtype}"};
    if (!defined($event)) {
        return (0);
    } else {
        my $found;
        $found = 0;
        foreach my $id (@{$event}) {
            if ($event_by_id->{$id}{$opts{key}} eq "$opts{value}") {
                ++$found;
            }
        }
        return ($found);
    } ## end else [ if (!defined($event))
} ## end sub RaidSqa_ems_log_check

#------------------------------------------------------------------------------

=head2 RaidSqa_ems_get_value_by_key

RaidSqa_ems_get_value_by_key(api_set_obj => $api_set_obj, eventtype=>$eventtype, key => $key, timestamp => $timestamp, log=>$log, timeout=>$timeout);

This subroutine return the ems value for key mentioned by $key

It returns an scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item eventtype

(Required): Type of event for which key is passed  ex: "raid.rg.scrub.resume"

=item key

(Required): The key name for which value is to be returned

=item timestamp

(Required): The timestamp from where event is to be searched
Timestamp semantic can be :
<n>s    Last n seconds
<n>m    Last n minutes
<n>h    Last n hours
<n>d    Last n days
<n>t    Since timestamp <n>

=item log

(Optional):

=item timeout

(Optional): Default : 1200, timeout before command returns

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_ems_get_value_by_key {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa => 'NACL::APISet'},
                        eventtype   => {type => SCALAR},
                        key         => {type => SCALAR},
                        timestamp   => {type => SCALAR},
                        log     => {type => SCALAR, default => "",      optional => 1},
                        node    => {type => SCALAR, default => "local", optional => 1},
                        timeout => {type => SCALAR, default => 1200,    optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $timestamp   = $opts{timestamp};
    my $timeout     = $opts{timeout};
    my $log         = $opts{log};
    my $eventtype   = $opts{eventtype};

    my $event_by_type;
    my $event_by_id;

    if (defined($log)) {
        $event_by_type =
          RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj,
                                       timestamp   => $timestamp,
                                       timeout     => $timeout,
                                       log         => $log);
    } else {
        $event_by_type =
          RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj,
                                       timestamp   => $timestamp,
                                       timeout     => $timeout);
    }

    if (defined($log)) {
        $event_by_id = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj,
                                                  timestamp   => $timestamp,
                                                  timeout     => $timeout,
                                                  log         => $log);
    } else {
        $event_by_id = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj,
                                                  timestamp   => $timestamp,
                                                  timeout     => $timeout);
    }

    my $event = $event_by_type->{$eventtype};
    if (!defined($event)) {
        return (0);
    } else {
        my @values = ();
        foreach my $id (@{$event}) {
            push(@values, $event_by_id->{$id}{$opts{key}});
        }
        if (wantarray) {
            return @values;
        } else {
            return $values[0];
        }
    } ## end else [ if (!defined($event))
} ## end sub RaidSqa_ems_get_value_by_key

#------------------------------------------------------------------------------

=head2 RaidSqa_transit_to

RaidSqa_transit_to(api_set_obj=>$api_set_obj, conntype=>$conntype, state_name => $state_name, api_reconnect=>$api_reconnect);


This subroutine transits the filer into the specified state.

Returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#console".

=item state_name

(Required) : Name of state name to be transit. Valid transit states are 'UP', 'CLI', 'FIRMWARE',
             'MAINT', 'BOOT_MENU', 'IBM', 'LOGIN', 'CFE', 'REBOOT', 'SHELL'

=item api_reconnect

(Optional): Boolean value Default 1, If this is set to 1, apiset object will be reinited
            after transit to UP or equivalent state

=item node

(Optional): Node name on which the operation is to be performed

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_transit_to {
    my (%opts) =
      RaidSqa_validate(@_,{
            api_set_obj     => {type => OBJECT, isa     => 'NACL::APISet', optional => 1},
            host            => {type => OBJECT, isa     => 'Hostrec',      optional => 1},
            node            => {type => SCALAR, default => "local",        optional => 1},
            conntype        => {type => SCALAR, default => "#console",     optional => 1},
            timeout         => {type => SCALAR, default => 1200,           optional => 1},
            api_reconnect   => {type => SCALAR, default => 1,              optional => 1},
            state_name      => {type => SCALAR},
        }
    );

    my $api_set_obj   = $opts{api_set_obj};
    my $host          = $opts{host};
    my $node          = $opts{node};
    my $conntype      = $opts{conntype};
    my $timeout       = $opts{timeout};
    my $api_reconnect = $opts{api_reconnect};
    my $state_name    = uc $opts{state_name};
    my $reconnect_required = 0;
    my ($name, $user_state_name);
    $user_state_name = $state_name;

    if(!(defined($host) or defined($api_set_obj))) {
        $Log->comment("$pkg_name: api_set_obj is not defined");
        return 0;
    }

    my @valid_transit_states = (
        'UP',       'CLI',      'LOGIN',    'MAINT',
        'IBM',      'FIRMWARE', 'CFE',      'BOOT_MENU',
        'REBOOT',   'SHELL',
    );

    if(!grep /$state_name/, @valid_transit_states) {
        $Log->comment("$pkg_name: state_name $state_name is not one of the supported state");
        return 0;
    }

    #---------------------------------------------------------------------------
    #Translate state name to effective Cmode state name for transit_to()
    my %effective_cmode_states = (
        'UP' => 'CLI',
        'IBM' => 'FIRMWARE',
        'CFE' => 'FIRMWARE',
    );

    if(exists($effective_cmode_states{$state_name})) {
        $Log->comment("$pkg_name: Old state name: $state_name");
        $state_name = $effective_cmode_states{$state_name};
        $Log->comment("$pkg_name: New state name for Cmode: $state_name");
    }

    #---------------------------------------------------------------------------
    #create transit object
    if(defined($api_set_obj)) {
        $name = (exists $api_set_obj->{hostobj}) ? $api_set_obj->{hostobj} :
                                                   $api_set_obj->{connection};
    } elsif(defined($host)) {
        $name = $host;
    }

    my $transit_obj  = NACL::Transit->new(name => $name);

    #---------------------------------------------------------------------------
    #reboot or change state as per user defined value ($state_name);
    try {
        #my $current_state = $transit_obj->get_state(timeout=>$timeout);
        #$Log->comment("$pkg_name: Current filer state is: $current_state");
        if($state_name =~ /REBOOT/) {
            $transit_obj->reboot(timeout=>$timeout);
        } else {
            $transit_obj->change_state(to=>$state_name, timeout=>$timeout);
        }

        my $end_state = $transit_obj->get_state();
        $Log->comment("$pkg_name: Filer reached to state: $end_state");
        if($end_state ne $state_name) {
            $Log->comment("$pkg_name: Filer did not reach to desired state");
            return 0;
        }
    } catch NACL::Transit::Exceptions::TransitException with {
        my $exception_object = shift;
        my $output = _handle_nacl_exception($exception_object);
        return 0;
    };

    #---------------------------------------------------------------------------
    #If state_name is one of UP|CLI|LOGIN|REBOOT,
    #we need to reinit api_set_obj
    if($api_reconnect == 1 && $state_name =~ /^(UP|CLI|LOGIN|REBOOT|SHELL)$/) {
        $reconnect_required = 1;
    }

    try {
        if ($reconnect_required == 1) {
            $Log->comment("$pkg_name: api_set_obj reconnect is required");
            if (!$api_set_obj->api_is_session_alive()) {

                $Log->comment("Explicit disconnect of SSH sessions");
                Tharn::disconnect("#ssh",'-ok');

                $api_set_obj->api_reconnect_session();
                $Log->comment("$pkg_name: APISet sessions reconn");
            }
        }
    } catch NATE::BaseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
    };
    $Log->comment("$pkg_name: Transit to state $user_state_name - success");
    return 1;
} ## end sub RaidSqa_transit_to

#-------------------------------------------------------------------------------------
=head2 RaidSqa_boot_state

RaidSqa_boot_state(api_set_obj => $api_set_obj, conntype => $conntype);

returns the state of the filer.

The following arguments are accepted:

=over

=item api_set_obj

(Required): A NACL::APISet object

=item transient

(Required): If set to 1, It returns the boot state immedietely without waiting for filer to reach a known boot state.

=item conntype

(Optional): Connection type to use. Default is "#ssh".

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_boot_state {
    my (%opts) =
      RaidSqa_validate(@_,{
        api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
        timeout     => {type => SCALAR, default => 1200, optional => 1},
        transient   => {type => SCALAR, default => 1, optional => 1},
    });

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $timeout     = $opts{timeout};
    my $transient     = $opts{transient};

    my ($response, $output, $name, $state_name);

        #---------------------------------------------------------------------------
    #create transit object
    if(defined($api_set_obj)) {
        $name = (exists $api_set_obj->{hostobj}) ? $api_set_obj->{hostobj} :
                                                   $api_set_obj->{connection};
    }

    my $transit_obj  = NACL::Transit->new(name => $name);
    $transit_obj->set(timeout => $timeout);


    try {
        #we do not want this function to wait for any regonizable. It should return immediately the state of filer
        #Further state changes can be or rather should be achieved using RaidSqa_transit_to()
        $state_name =  $transit_obj->get_state(transient=>$transient);
    } catch NACL::Transit::Exceptions::TransitException with {
        my $exception_object = shift;
        my $output = _handle_nacl_exception($exception_object);
        return 0;
    };

    if(defined($state_name)) {
        return $state_name;
    } else {
        $Log->comment("$pkg_name: State of the filer was not found");
        return 0;
    }
} ## end sub RaidSqa_boot_state
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------

=head2 RaidSqa_root_recovery

RaidSqa_root_recovery(api_set_obj=>$api_set_obj, reboot_required=>$reboot_required);

This function will do the root recovery process. Login to diag mode, executing shell commands & reboot the filer.
Returns nothing

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item node

(Optional): Node name on which the operation is to be performed

=item reboot_required

(Optional): Default 1, If this param is set to 1, filer is rebooted after root_recovery is complete

=item timeout

(Optional): timeout used before giving up

=back

=cut
#------------------------------------------------------------------------------

sub RaidSqa_root_recovery {
     my (%opts) =
      RaidSqa_validate(@_,{
                api_set_obj     => {type => OBJECT, isa => 'NACL::APISet'},
                node            => {type => SCALAR, default => "local", optional => 1},
                timeout         => {type => SCALAR, default => 1200, optional => 1},
                reboot_required => {type => SCALAR, default => 1,    optional => 1},
            }
        );
    my $api_set_obj     = $opts{api_set_obj};
    my $node            = $opts{node};
    my $timeout         = $opts{timeout};
    my $reboot_required = $opts{reboot_required};

    try {
        #-----------------------------------------------------------------------
        #Create transit object
        my $name  = (exists $api_set_obj->{hostobj}) ? $api_set_obj->{hostobj} :
                                                       $api_set_obj->{connection};

        my $transit_obj  = NACL::Transit->new(name => $name);
        my $current_state = $transit_obj->get_state(timeout=>$timeout);
        $Log->comment("$pkg_name: Current filer state is: $current_state");

        #-----------------------------------------------------------------------
        # Get console connection created by transit object
        my %res = $transit_obj->get('conn');
        my $console_conn = $res{'conn'};

        #-----------------------------------------------------------------------
        # Change state to systemshell
        if ($current_state ne "SYSTEMSHELL") {
            $Log->comment("$pkg_name: Change filer state to SYSTEMSHELL");
            $transit_obj->change_state( to => "SYSTEMSHELL");
        }

        my $apiset_shell = NACL::APISet->new(
                                connection  => $console_conn,
                                category    => 'Node',
                                interface   => 'CLI',
                                set         => 'Systemshell',
                                cdef_path   => $api_set_obj->{cdef_path},
                            );

        my @shell_commands = (
            'kenv bootarg.init.boot_recovery',
            'sudo sysctl kern.bootargs=--bootarg.init.boot_recovery',
            'ls /mroot/etc/cluster_config/monitor_mroot.nvfail',
            'sudo rm /mroot/etc/cluster_config/monitor_mroot.nvfail',
        );

        foreach my $command (@shell_commands) {
            # As we are executing commands through console connections,
            # we are getting timeouts due to DBG messages on console
            # Hence running in try - catch block without validation
            # Also, the prompt is set to .* as we may not always get right prompt
            # due to debug or console messages
            try {
                $apiset_shell->execute_raw_command(command => $command,
                                    'connectrec-additional_prompts' => ['.*']);
                sleep(5);
            } catch NACL::APISet::Exceptions::ResponseException with {
                $Log->warn("$pkg_name: Execution of command '$command' looks to be failed");
            }
        }

        $transit_obj->change_state( to => "CLI");

        #-----------------------------------------------------------------------
        #Reboot filer after root recovery
        if($reboot_required == 1) {
            RaidSqa_transit_to(api_set_obj=>$api_set_obj, state_name => "REBOOT");
            RaidSqa_transit_to(api_set_obj=>$api_set_obj, state_name => "CLI");
        }
    } catch NACL::Transit::Exceptions::TransitException with {
        my $exception_object = shift;
        my $output = _handle_nacl_exception($exception_object);
        return 0;
    };
}

#------------------------------------------------------------------------------

=head2 RaidSqa_systemshell_cmd

RaidSqa_systemshell_cmd(api_set_obj=>$api_set_obj,cmd => $cmd);

This function will run the command in filer systemshell.
It returns o/p of the systemshell command if any

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item cmd

(Optional):  filer systemshell command.

=item timeout

(Optional): timeout value in secs.

=back

=cut
#-----------------------------------------------------------------------------

sub RaidSqa_systemshell_cmd {
     my (%opts) =
      RaidSqa_validate(@_,{
                api_set_obj => {type => OBJECT, isa => 'NACL::APISet', optional => 1},
                cmd         => {type => SCALAR},
                timeout     => {type => SCALAR, default => 1200, optional => 1},
            }
        );
    my $api_set_obj = $opts{api_set_obj};
    my $cmd    = $opts{cmd};
    my $timeout = $opts{timeout};
    my $output;
    my $name  = (exists $api_set_obj->{hostobj}) ? $api_set_obj->{hostobj}
                                                   : $api_set_obj->{connection};

    my $transit_obj  = NACL::Transit->new(name => $name);
    try {
        my $current_state = $transit_obj->get_state(timeout=>$timeout);
        $Log->comment("$pkg_name: Current filer state is: $current_state");
        my $apiset_shell = NACL::APISet->new(
                                hostobj  => $api_set_obj->{hostobj},
                                category    => 'Node',
                                interface   =>'CLI',
                                set         =>'Systemshell',
                                cdef_path   => $api_set_obj->{cdef_path},
                            );

    # As we are executing commands through console connections,
    # we are getting timeouts due to DBG messages on console
    # Hence running in try - catch block without validation

    $output = $apiset_shell->execute_raw_command(command => $cmd);
        $transit_obj->change_state( to => "CLI");
    return $output;
    } catch NACL::Transit::Exceptions::TransitException with {
        my $exception_object = shift;
        my $output = _handle_nacl_exception($exception_object);
        return 0;
    };
} ## end RaidSqa_systemshell_cmd

#------------------------------------------------------------------------------

=head2 RaidSqa_set_snmp_community

RaidSqa_set_snmp_community( api_set_obj => $api_set_obj, access => $access, community => $community);

This subroutine sets the snmp community settings to given access type and community name.

It returns 1 on success and 0 on failure.

The following arguments are accepted:

=over

=item api_set_obj

(Required): NACL::APISet object

=item access

(Optional) : Specifies the access type to be set. Default is 'ro'.

=item community

(Optional): Specifies the community name to be set. Default is 'public'.

=back

=cut

#------------------------------------------------------------------------------
sub RaidSqa_set_snmp_community{
    my (%opts) =
        RaidSqa_validate(
           @_,
           {
               api_set_obj => { type => OBJECT, isa =>"NACL::APISet", optional => 1},
               access      => { type => SCALAR, default => "ro", optional => 1},
               community   => { type => SCALAR, default => "public", optional => 1},
           }
        );
    my $api_set_obj    = $opts{api_set_obj};
    my $access_type    = $opts{access};
    my $community_name = $opts{community};

    #Get current snmp settings.
    my ( $result, $output) = ( undef, undef);
    $result = $api_set_obj->system_snmp_community_show();
    $output = $result->get_parsed_output();
    $Log->comment("$pkg_name: Setup: SNMP community settings: ".Dumper($result));
    $Log->comment("$pkg_name: Setup: Access & Community: " . lc($output->[0]->{access}) . " & " . lc($output->[0]->{'community-name'}));
    #Check if the entry is already added.
    if ( ( lc($output->[0]->{access})         eq lc($access_type)    ) and
         ( lc($output->[0]->{'community-name'}) eq lc($community_name) )
       ) {
         $Log->comment("$pkg_name: Setup: snmp community is already set to 'ro public'");
         return (1);
    } else {
        #If there is no entry present, We can just add new one.
        if ( (not defined($output->[0]->{access})) &&
             (not defined($output->[0]->{'community-name'}))
           ) {
           $Log->comment("$pkg_name: Setup: snmp community is not set.");
           $api_set_obj->system_snmp_community_add( 'community-name' => lc($community_name), type => lc($access_type));
        } else {
           #If an entry is present but it is not 'ro public' then fix it
           $Log->comment("$pkg_name: Setup: snmp community is not set to ro public. Make it 'ro public'.");
           $api_set_obj->system_snmp_community_delete(
                                'community-name' => $output->[0]->{'community-name'},
                                 type => $output->[0]->{access});
           $api_set_obj->system_snmp_community_add(
                                'community-name' => "$community_name",
                                 type => "$access_type");
        } #end if

        $result = $api_set_obj->system_snmp_community_show();
        $output = $result->get_parsed_output();
        #Final check to see if the settings are good.
        if ( ( lc($output->[0]->{access})    eq lc($access_type)    ) and
             ( lc($output->[0]->{'community-name'}) eq lc($community_name) ) ) {
            $Log->comment("$pkg_name: Setup: snmp community setup is successful.");
            return (1);
        } else {
            $Log->comment("$pkg_name: Setup: snmp community setup did not succeed.");
            return (0);
        } #end if
   } #end outer-if
} ## end RaidSqa_set_snmp_community

#------------------------------------------------------------------------------


=head2 RaidSqa_get_disk_list_by_carrier

RaidSqa_get_disk_list_by_carrier(host=>$host, filermode=>$filermode, conntype=>$conntype,carrier=>$carrierid);

This subroutine returns the disks as array present in a carrier identified by carrrier_id

It returns a array on success

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item carrier_id

(Required): This represents the carrier_id as shown in raid_config output representing the carrier.

=back

=cut

#----------------------------------------------------------------------------------

sub RaidSqa_get_disk_list_by_carrier {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet', optional => 1},
                        node        => {type => SCALAR, default => "local",        optional => 1},
                        filermode => {type => SCALAR, default => "normal", optional => 1},
                        conntype  => {type => SCALAR, default => "#ssh", optional => 1},
                        carrier => {type => SCALAR},});

    my $api_set_obj      = $opts{api_set_obj};
    my $node              = $opts{node};
    my $filermode = $opts{filermode};
    my $conntype  = $opts{conntype};
    my $carrier   = $opts{carrier};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }
    if ($filermode ne "normal") {
        $conntype = "#console";
    }

    #my $conn      = make_conn($host, $conntype);
    my $raid_obj  = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my %all_disks = $raid_obj->all_disks();
    my @all_disks = keys(%all_disks);
    my @carrier_disks;

    foreach my $disk_name (@all_disks) {
        my $disk_object = $raid_obj->disk($disk_name);
        next if (!defined($disk_object));
        if ($disk_object->parameter("is-multidisk-carrier")) {
            if ($disk_object->parameter('carrier') eq "$carrier") {
                push(@carrier_disks, $disk_name);
            }
        }
    }
    return (@carrier_disks);
} ## end sub RaidSqa_get_disk_list_by_carrier

#------------------------------------------------------------------------------
=head2 RaidSqa_get_disk_carrier

RaidSqa_get_disk_carrier(host=>$host, filermode=>$filermode, conntype=>$conntype,disk_name=>$disk_name);

This subroutine returns the carrier_id for a mentioned disk_name.

It returns a scalar on success

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item disk_name

(Required): This represents the disk_name as shown in raid_config output.

=back

=cut

#----------------------------------------------------------------------------------
sub RaidSqa_get_disk_carrier {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet', optional => 1},
                        node        => {type => SCALAR, default => "local",        optional => 1},
                        filermode => {type => SCALAR, default => "normal", optional => 1},
                        conntype  => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_name => {type => SCALAR},});

    my $api_set_obj      = $opts{api_set_obj};
    my $node              = $opts{node};
    my $filermode = $opts{filermode};
    my $conntype  = $opts{conntype};
    my $disk_name = $opts{disk_name};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }
    if ($filermode ne "normal") {
        $conntype = "#console";
    }

    #my $conn        = make_conn($host, $conntype);
    my $raid_obj    = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $disk_object = $raid_obj->disk("$disk_name");
    if (!defined($disk_object)) {
        $Log->comment("RadSqa_Cmodev3.pm Disk - $disk_name not found");
        return (0);
    } else {
        if (!$disk_object->parameter("is-multidisk-carrier")) {
            $Log->comment("RadSqa_Cmodev3.pm Disk - $disk_name is not Boab disk");
            return (0);
        }
        return ($disk_object->parameter("carrier"));
    }
} ## end sub RaidSqa_get_disk_carrier

#------------------------------------------------------------------------------

=head2 RaidSqa_get_carrier_disks_mapping

RaidSqa_get_carrier_disks_mapping(host=>$host, filermode=>$filermode, conntype=>$conntype,forward=>1);

This subroutine returns deep-hash values for disks and carrier mapping..

It returns a hash reference on success

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item forward
customizes the return hash structure for carrier and disk name mapping.  Possible values 0,1. Default: 1.
If forward = 1, maps carrier id to array of disks
If forward = 0, maps disk name to carrier id.

=back

=cut

#----------------------------------------------------------------------------------

sub RaidSqa_get_carrier_disks_mapping {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet', optional => 1},
                        node        => {type => SCALAR, default => "local",        optional => 1},
                        filermode => {type => SCALAR, default => "normal", optional => 1},
                        conntype  => {type => SCALAR, default => "#ssh", optional => 1},
                        forward   => {type => SCALAR, default => 1, optional => 1},
                        spare_only      => {type => SCALAR, default => "no", optional => 1},});

    my $api_set_obj      = $opts{api_set_obj};
    my $node              = $opts{node};
    my $filermode   = $opts{filermode};
    my $conntype    = $opts{conntype};
    my $forward     = $opts{forward};
    my $spare_only  = $opts{spare_only};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if ($filermode ne "normal") {
        $conntype = "#console";
    }

    #my $conn      = make_conn($host, $conntype);
    my $raid_obj  = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my %all_disks = $raid_obj->all_disks();
    my @all_disks = keys(%all_disks);
    my (%carrier_disk_mapping, %disk_carrier_mapping);
    my @spare_disks = RaidSqa_get_disk_list (api_set_obj => $api_set_obj, spare_only=>$spare_only);
    foreach my $disk_name (@all_disks) {
        my $disk_object = $raid_obj->disk($disk_name);
        next if (!defined($disk_object));
        if (!$disk_object->parameter("is-multidisk-carrier")) {
            next;
        }

        if(defined $spare_only && ($spare_only =~ /yes/i)) {
            if (!(grep /$disk_name/, @spare_disks)) {
                next;
            }
        }
        my $carrier = $disk_object->parameter("carrier");
        if ($forward == 0) {
            $disk_carrier_mapping{$disk_name} = $carrier;
        } else {
            if (exists($carrier_disk_mapping{$carrier})) {
                push(@{$carrier_disk_mapping{$carrier}}, $disk_name);
            } else {
                $carrier_disk_mapping{$carrier} = [$disk_name];
            }
        }
    } ## end foreach my $disk_name (@all_disks)
    if ($forward == 0) {
        return (\%disk_carrier_mapping);
    } else {
        return (\%carrier_disk_mapping);
    }
} ## end sub RaidSqa_get_carrier_disks_mapping

#------------------------------------------------------------------------------

=head2 RaidSqa_check_desirable_layout

RaidSqa_check_desirable_layout(host=>$host, filermode=>$filermode, conntype=>$conntype,filer_vol=>$filer_vol,group_name=>$group_name);

This subroutine returns 0/1. Returns 1 if it is in desirable layout..

It returns a scalar on success

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item filer_vol

(Optional): Represents the aggregate for which desirable layout has to be validated. This checks for all RGs in a aggregate.

=item group_name

(Optional): Represents the raid group for which desirable layout has to be validated.

=back

=cut

#----------------------------------------------------------------------------------

sub RaidSqa_check_desirable_layout {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet', optional => 1},
                        node        => {type => SCALAR, default => "local",        optional => 1},
                        filermode => {type => SCALAR, default => "normal", optional => 1},
                        conntype  => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol  => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        group_name => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        });

    my $api_set_obj      = $opts{api_set_obj};
    my $node              = $opts{node};
    my $filermode  = $opts{filermode};
    my $conntype   = $opts{conntype};
    my $filer_vol  = $opts{filer_vol};
    my $group_name = $opts{group_name};
    my $desirable_layout = 0;

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    if (!defined($filer_vol) && (!defined($group_name))) {
        $Log->comment("RadSqa_Cmodev3.pm Param filer_vol or group_name missing in the function call");
        return (0);
    }
    if ($filermode ne "normal") {
        $conntype = "#console";
    }
    my (@group_list, @disk_list, @carrier_list, @undesired_groups);
    if (defined($filer_vol)) {
        @group_list = RaidSqa_get_group_list(api_set_obj => $api_set_obj,
                                             node        => $node,
                                             filermode => $filermode,
                                             conntype  => $conntype,
                                             filer_vol => $filer_vol);
    } elsif (defined($group_name)) {
        if (!RaidSqa_check_group_exist(api_set_obj => $api_set_obj,
                                       node        => $node,
                                       filermode  => $filermode,
                                       conntype   => $conntype,
                                       group_name => $group_name)
          ) {
            die("RadSqa_Cmodev3.pm Param group name: $group_name does not exists");
        }
        @group_list = $group_name;
    } ## end elsif (defined($group_name...

    #$conn = make_conn($host, $conntype);
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    $raid_obj->all_disks();
    foreach my $group (@group_list) {
        @disk_list = RaidSqa_get_disk_list(api_set_obj => $api_set_obj,
                                           node        => $node,
                                           filermode  => $filermode,
                                           conntype   => $conntype,
                                           group_name => $group);

        foreach my $disk_name (@disk_list) {
            my $disk_object = $raid_obj->disk($disk_name);
            if ($disk_object->parameter("is-multidisk-carrier")) {
                push(@carrier_list, $disk_object->parameter("carrier"));
            } else {
                $Log->comment("RadSqa_Cmodev3.pm Disk - $disk_name is not Boab disk");
                return (0);
            }
        }
        my %hash_carrier;
        $hash_carrier{$_}++ for @carrier_list;
        my @hash_carrier = keys(%hash_carrier);
        if (scalar(@carrier_list) == scalar(@hash_carrier)) {
            $desirable_layout = 1;
        } else {
            push(@undesired_groups, $group);
            $desirable_layout = 0;
         last;
        }
        (@carrier_list, @disk_list) = ();
    } ## end foreach my $group (@group_list)
    if ($desirable_layout) {
        $Log->comment("RadSqa_Cmodev3.pm Group - @group_list is in desired layout");
    } else {
        $Log->comment("RadSqa_Cmodev3.pm Group - @undesired_groups is not in desired layout");
    }
    return ($desirable_layout);
} ## end sub RaidSqa_check_desirable_layout

#------------------------------------------------------------------------------


#------------------------------------------------------------------------------

=head2 RaidSqa_create_hybrid_aggr

RaidSqa_create_hybrid_aggr(api_set_obj=>$api_set_obj, filermode=>$filermode, conntype=>$conntype, filer_vol=>$filer_vol, disk_count=>$disk_count, mirror=>$mirror,raid_type=>$raid_type,raid_size=>$raid_size);

This subroutine creates Hybrid aggr by adding new raid_group to existing aggr.

It returns 1 on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item disk_count

(Optional) : Number of of disk(SSD/BSSD) to be added to create HyA aggr.Default is 3.

=item raid_size

(Optional): Raid size of the new rg to be added. By default it is 3. Other values can be min, max or user_defined count but that should be in allowed range for a given storage_type.

=item mirror

(Optional): 1 for mirror volume and 0 for nonmirror. Default is 0.

=item filer_vol

(Optional): Name of aggr/volume which will be used to create HyA aggr.

=item aggr_type

(Optional): 1 for aggregate and 0 for traditional. Default it is 1.

=item hya_storage_type

(Optional):  Disk storage type. Default value: undef.Possible values: undef,SSD, BSSD. If not defined, then one of the type of SSD disk will be chosen.

=item ssd_raid_type

(Optional): Possible values: raid4,raid_dp.Default is undef
             This option when set to different value other than RAID_TYPE will create MRT aggr.If undef is set, aggregate created will be of raidtype as RAID_TYPE.
             For eg : if RAID_TYPE=raid_dp and SSD_RAID_TYPE=raid4; aggregate created will be of Mixed Raid Type.

=back

=cut

#------------------------------------------------------------------------------
sub RaidSqa_create_hybrid_aggr {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet', optional => 1},
                        node        => {type => SCALAR, default => "local",        optional => 1},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol        => {type => SCALAR},
                        disk_count       => {type => SCALAR, default => 3, optional => 1},
                        raid_size        => {type => SCALAR, default => 3, optional => 1},
                        raid_type        => {type => SCALAR, default => "raid_dp", optional => 1},
                        aggr_type        => {type => SCALAR, default => "1", optional => 1},
                        mirror           => {type => SCALAR, default => 0, optional => 1},
                        ssd_raid_type    => {type => SCALAR | UNDEF, default => undef, optional => 1},
                        hya_storage_type => {type => SCALAR | UNDEF, default => undef, optional => 1}});

    my $api_set_obj      = $opts{api_set_obj};
    my $filermode        = $opts{filermode};
    my $node              = $opts{node};
    my $filer_vol        = $opts{filer_vol};
    my $disk_count       = $opts{disk_count};
    my $mirror           = $opts{mirror};
    my $raid_size        = $opts{raid_size};
    my $raid_type        = $opts{raid_type};
    my $aggr_type        = $opts{aggr_type};
    my $conntype        = $opts{conntype};
    my $ssd_raid_type    = $opts{ssd_raid_type};
    my $hya_storage_type = $opts{hya_storage_type};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    # Check if aggr exists.

    if (!RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
                                  filer_vol   => $filer_vol,
                                  aggr_type   => $aggr_type,
                                  node        => $node)
      ) {
        $Log->comment("$pkg_name: aggr $filer_vol does not exists");
        return 0;
    }

    # Validate hya_storage_type
    my ($disk_count_ssd_pool0, $disk_count_ssd_pool1, $disk_count_bssd_pool0, $disk_count_bssd_pool1, $disk_count_total_pool0, $disk_count_total_pool1) = 0;

    $disk_count_ssd_pool0 = RaidSqa_get_disk_list(api_set_obj => $api_set_obj, node => $node,disk_type => "SSD", spare_only => "yes", pool => "0");
    $Log->comment("$pkg_name: spare disk count of storage type SSD from pool0  - $disk_count_ssd_pool0");
    $disk_count_bssd_pool0 = RaidSqa_get_disk_list(api_set_obj => $api_set_obj, node => $node,disk_type => "BSSD", spare_only => "yes", pool => "0");
    $Log->comment("$pkg_name: spare disk count of storage type BSSD from pool0  - $disk_count_bssd_pool0");

    $disk_count_ssd_pool1 = RaidSqa_get_disk_list(api_set_obj => $api_set_obj, node => $node,disk_type => "SSD", spare_only => "yes", pool => "1");
    $Log->comment("$pkg_name: spare disk count of storage type SSD from pool1  - $disk_count_ssd_pool1");
    $disk_count_bssd_pool1 = RaidSqa_get_disk_list(api_set_obj => $api_set_obj, node => $node,disk_type => "BSSD", spare_only => "yes", pool => "1");
    $Log->comment("$pkg_name: spare disk count of storage type BSSD from pool1  - $disk_count_bssd_pool1");

    $disk_count_total_pool0 = $disk_count_ssd_pool0 + $disk_count_bssd_pool0;
    $disk_count_total_pool1 = $disk_count_ssd_pool1 + $disk_count_bssd_pool1;

    if (!($disk_count_total_pool0 + $disk_count_total_pool1)) {
        $Log->comment("$pkg_name: spare disk of storage type SSD/BSSD not found");
        return (0);
    }

    my $mixed_disk_not_allowed_flag =
      RaidSqa_get_options(api_set_obj  => $api_set_obj,
                          node         => $node,
                          conntype    => $conntype,
                          option_name => "raid.disktype.enable");

    # Get storage_type in case it is not defined (SSD or BSSD)
    if (!defined($hya_storage_type)) {
        if ($mixed_disk_not_allowed_flag =~ /off/) {
            $hya_storage_type = "SSD";
        } else {
            $hya_storage_type = (($disk_count_ssd_pool0 + $disk_count_ssd_pool1) >= ($disk_count_bssd_pool0 + $disk_count_bssd_pool1)) ? "SSD" : "BSSD";
        }
    }
    $Log->comment("$pkg_name: hya_storage_type-$hya_storage_type");

    # Validate raid_size

    if (!RaidSqa_check_raid_size_valid_range(api_set_obj  => $api_set_obj,
                                             raid_size    => $raid_size,
                                             raid_type    => $raid_type,
                                             storage_type => $hya_storage_type,
                                             node         => $node)
      ) {
        $Log->comment("$pkg_name: raid_size $raid_size is not in the valid range");
        return 0;
    }

    $Log->comment("$pkg_name: raid_size-$raid_size");
    if ($mirror) {
        if ($disk_count < (2 * $raid_size)) {
            $disk_count = 2 * $raid_size;
        }
    }

    if (defined($ssd_raid_type) && ($ssd_raid_type eq "raid_dp")) {
        $Log->comment("RadSqa_Cmodev3.pm Checking for disk_count for raid_dp");
        if (($disk_count < 3) && (!$mirror)) {
            $disk_count = 3;
        } elsif (($disk_count < 6) && ($mirror)) {
            $disk_count = 6;
        }
    }

    $Log->comment("$pkg_name::disk_count value for mirror=$mirror-$disk_count");

    # Validate diskcount
    if ($mixed_disk_not_allowed_flag =~ /off/) {
        if ($mirror) {
            if (($disk_count_total_pool0 >= ($disk_count / 2)) && ($disk_count_total_pool1 >= ($disk_count / 2))) {
                $Log->comment("$pkg_name: sufficient spare disk of storage type SSS/BSSD available for creating mirrored HyA aggr");
            } else {
                $Log->comment("$pkg_name: sufficient spare disk of storage type SSS/BSSD not available for creating mirrored HyA aggr Expected-" . ($disk_count / 2) . " in each pool");
                return (0);
            }
        } else {
            if (($disk_count_total_pool0 >= $disk_count) || ($disk_count_total_pool1 >= $disk_count)) {
                $Log->comment("$pkg_name: sufficient spare disk of storage type SSS/BSSD available for creating non-mirrored HyA aggr");
            } else {
                $Log->comment("$pkg_name: sufficient spare disk of storage type SSS/BSSD not available for creating non-mirrored HyA aggr Expected-$disk_count in each pool");
                return (0);
            }
        }
    } else {
        if ($mirror) {
            if (uc($hya_storage_type) eq "SSD") {
                if (($disk_count_ssd_pool0 >= ($disk_count / 2)) && ($disk_count_ssd_pool1 >= ($disk_count / 2))) {
                    $Log->comment("$pkg_name: sufficient spare disk of storage type SSD available");
                } else {
                    $Log->comment("$pkg_name: sufficient spare disk of storage type SSD not available Expected-" . ($disk_count / 2) . " in each pool");
                    return (0);
                }
            } else {
                if (($disk_count_bssd_pool0 >= ($disk_count / 2)) && ($disk_count_bssd_pool1 >= ($disk_count / 2))) {
                    $Log->comment("$pkg_name: sufficient spare disk of storage type BSSD available");
                } else {
                    $Log->comment("$pkg_name: sufficient spare disk of storage type BSSD not available Expected-" . ($disk_count / 2) . " in each pool");
                    return (0);
                }
            }
        } else {
            if (uc($hya_storage_type) eq "SSD") {
                if (($disk_count_ssd_pool0 >= $disk_count) || ($disk_count_ssd_pool1 >= $disk_count)) {
                    $Log->comment("$pkg_name: sufficient spare disk of storage type SSD available");
                } else {
                    $Log->comment("$pkg_name: sufficient spare disk of storage type SSD not available Expected-$disk_count in each pool");
                    return (0);
                }
            } else {
                if (($disk_count_bssd_pool0 >= $disk_count) || ($disk_count_bssd_pool1 >= $disk_count)) {
                    $Log->comment("$pkg_name: sufficient spare disk of storage type BSSD available");
                } else {
                    $Log->comment("$pkg_name: sufficient spare disk of storage type BSSD not available Expected-$disk_count in each pool");
                    return (0);
                }
            }
        } ## end else [ if ($mirror)
    } ## end else [ if ($mixed_disk_not_allowed_flag...

    if (RaidSqa_get_aggr_options(api_set_obj => $api_set_obj,node => $node, filer_vol => $filer_vol, options => "hybrid_enabled") eq "off") {
        $Log->comment("$pkg_name: Enable option hybrid_enabled on filer_vol - $filer_vol to create hybrid aggr");
        if (!RaidSqa_aggr_options(api_set_obj => $api_set_obj,node => $node,conntype => $conntype, filer_vol => $filer_vol, options => "hybrid_enabled", value => "on")) {
            $Log->comment("$pkg_name: Failed to enable option hybrid_enabled on filer_vol - $filer_vol");
            return (0);
        }
    }
    my %aggr_add = (
                api_set_obj => $api_set_obj,
                filer_vol  => $filer_vol,
                disk_count => $disk_count,
                node       => $node,
                options    => "-c block -T $hya_storage_type -f",
            );

    $aggr_add{ssd_raid_type} = $ssd_raid_type if (defined($ssd_raid_type) && ($ssd_raid_type !~ /undef/i));

    $Log->comment("$pkg_name: Add new raidgroup to filer_vol - $filer_vol to create hybrid aggr");
    if (!RaidSqa_aggr_add(%aggr_add)) {
        $Log->comment("$pkg_name: Addition of disk to vol/aggr $filer_vol to create new raid group failed");
        return (0);
    } else {
        $Log->comment("$pkg_name: Added new raid group to vol/aggr $filer_vol");
    }

    # check if aggr is hybrid aggr
    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $aggr_obj = $raid_obj->aggregate("$filer_vol");
    if (defined($aggr_obj)) {
        if ($aggr_obj->parameter("hybrid") == 1) {
            return (1);
        } else {
            return (0);
        }
    } else {
        $Log->comment("$pkg_name: Failed to create RaidInfo object for filer_vol - $filer_vol\n");
        return (0);
    }
} ## end sub RaidSqa_create_hybrid_aggr


#------------------------------------------------------------------------------

=head2 RaidSqa_flexvol_create

RaidSqa_flexvol_create(api_set_obj=>$api_set_obj, filermode=>$filermode, conntype=>$conntype,aggr_name=>$aggr_name,vol_name=>$vol_name,vol_size=>$vol_size);

This subroutine returns 0/1. Returns 1 if flexvol is created.

It returns a scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item aggr_name

(Required): Represents the aggregate to which flexible volume will be added.

=item vol_name

(Required): Name of the flexible volume to be created.

=item vol_size

(Optional): %age of aggr_size to be used as vol_size.Default is 75% of aggr size.

=back

=cut

#----------------------------------------------------------------------------------

sub RaidSqa_flexvol_create {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype  => {type => SCALAR, default => "#ssh", optional => 1},
                        aggr_name        => {type => SCALAR},
                        vol_name         => {type => SCALAR},
                        node             => {type => SCALAR, default => "local",        optional => 1},
                        vol_size         => {type => SCALAR, default => 75, optional => 1},
                        vserver           => {type => SCALAR, default => "vs0",   optional => 1},
                        security_style    => {type => SCALAR, default => "mixed", optional => 1},
                        unix_permissions  => {type => SCALAR, default => "0777",  optional => 1},
                    });

    my $api_set_obj      = $opts{api_set_obj};
    my $conntype         = $opts{conntype};
    my $aggr_name        = $opts{aggr_name};
    my $vol_name         = $opts{vol_name};
    my $vol_size         = $opts{vol_size};
    my $node             = $opts{node};
    my $vserver          = $opts{vserver};
    my $security_style   = $opts{security_style};
    my $unix_permissions = $opts{unix_permissions};

    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    $Log->comment("$pkg_name: Check if aggr-$aggr_name exists");
    my $aggr_obj = $raid_obj->aggregate("$aggr_name");
    if (!$aggr_obj) {
        $Log->comment("$pkg_name: Failed to create flexible volume;aggr-$aggr_name doesnot exists.\n");
        return 0;
    }

    $Log->comment("$pkg_name: Check if volume-$vol_name exists");
    my $vol_obj = $raid_obj->volume("$vol_name");
    if ($vol_obj) {
        $Log->comment("$pkg_name: Flex volume-$vol_name already exists.\n");
        return 0;
    }

   # validate vol_size value;should be numeric
    if ($vol_size !~ /^\d+$/) {
        $Log->comment("$pkg_name: vol_size should not be numeric value only.\n");
        return 0;
    }

    # Check if vol_size is not more than 100%
    if ($vol_size > 100) {
        $Log->comment("$pkg_name: vol_size should not be more than 100%; provide lesser value.\n");
        return 0;
    }


    my ($vol_space,$space_format);
    $Log->comment("$pkg_name: Get the available space from aggregate-$aggr_name");
    my $ref_available_space = RaidSqa_get_aggr_show_space(api_set_obj=>$api_set_obj,filer_vol=>$aggr_name,options=>'m');
    if (ref($ref_available_space) ne "HASH") {
        $Log->comment("$pkg_name: Failed to get aggr show_space data for $aggr_name");
        return 0;
    } else {
        $vol_space = $ref_available_space->{'available'};
        $vol_size = $vol_size/100;
        $vol_space = $vol_space * $vol_size;
        if ($vol_space =~ /(\d+)\./) {
            $vol_space = $1;
        }
        if ($vol_space < 20) {
            $vol_space = 20;
        }
        $space_format = $vol_space . "m";
    }

    my $junc_path = "/" . "$vol_name";

    my %vol_create_args = (
        vserver              => $vserver,
        volume               => $vol_name,
        aggregate            => $aggr_name,
        'security-style'     => $security_style,
        'unix-permissions'   => $unix_permissions,
        'junction-path'      => $junc_path,
        size                 => $space_format,
        'connectrec-timeout' => 2*$RaidSqa_TIMEOUT
    );

   try {
        $api_set_obj->volume_create(%vol_create_args);
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        return 0;
    };
    sleep 5;

    $raid_obj->refresh();
    $vol_obj = $raid_obj->volume("$vol_name");
    if (!$vol_obj) {
        $Log->comment("$pkg_name: Flexible vol-$vol_name creation failed.\n");
        return 0;
    }
    $Log->comment("$pkg_name: Flexible vol-$vol_name created successfully");
    sleep(60);
    return 1;
}

#------------------------------------------------------------------------------

=head2 RaidSqa_flexvol_resize

RaidSqa_flexvol_resize(api_set_obj=>$api_set_obj,filermode=>$filermode, conntype=>$conntype,vol_name=>$vol_name,new_size=>$new_size,options=>$options);

This subroutine returns 0/1. Returns 1 if flexvol resizing is done.

It returns a scalar on success

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item vol_name

(Required): Name of the flexible volume to be created.

=item new_size

(Required): The size can be expressed as an absolute value, or as an incremental
            change by setting grow/shrink param to 1 to grow or shrink the volume size.

=item options

(Optional): Default is k for kilobytes.Possible values : 'k', 'm', 'g', and 't' indicates the
            desired units, namely 'kilobytes', 'megabytes', 'gigabytes' and 'terabytes' (respectively).

=back

=cut

#----------------------------------------------------------------------------------

sub RaidSqa_flexvol_resize {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype  => {type => SCALAR, default => "#ssh", optional => 1},
                        vol_name         => {type => SCALAR},
                        new_size         => {type => SCALAR},
                        node             => {type => SCALAR, default => "local",        optional => 1},
                        vserver           => {type => SCALAR, default => "vs0",   optional => 1},
                        options  => {type => SCALAR, default => 'k', optional => 1},
                    });
    my $api_set_obj      = $opts{api_set_obj};
    my $conntype         = $opts{conntype};
    my $node             = $opts{node};
    my $vserver          = $opts{vserver};
    my $vol_name         = $opts{vol_name};
    my $new_size         = $opts{new_size};
    my $options          = $opts{options};

    $Log->comment("$pkg_name: Check if volume-$vol_name exists");
    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }

    my $raid_obj = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);
    my $vol_obj = $raid_obj->volume("$vol_name");
    if (!$vol_obj) {
        $Log->comment("$pkg_name: Flex volume-$vol_name doesnot exists.\n");
        return 0;
    }

    # Validate new_size value
    if ($new_size !~ /^[\+\-]?\d{1,}$/) {
        $Log->comment("RadSqa_Cmodev3.pm new_size param should be integer with optional +/-");
      return 0;
    }
    $new_size = $new_size."$options";

    my %vol_resize_args = (vserver            => $vserver,
                           volume             => $vol_name,
                           'new-size'           => $new_size);


    my $vol_resize_output;
    try {
        my $response =
          $api_set_obj->volume_size(%vol_resize_args);
        $vol_resize_output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
    };

    my @err_string = ('Selected volume size is too small',
                      'failed because there is not enough space in the aggregate',
                      'Cannot shrink volume by more than its volume size'
                    );
    foreach my $err_string (@err_string) {
        if ($vol_resize_output =~ /$err_string/) {
            $Log->comment("$pkg_name: Resizing vol-$vol_name failed;Error-$err_string");
            return 0;
        }
    }

    if ($vol_resize_output =~ /size set to (\d+\w)/) {
        $Log->comment("$pkg_name: Resizing of vol-$vol_name completed successfully; new_size-$1");
        sleep(30);
        return 1;
    } else {
        $Log->comment("$pkg_name: Resizing vol-$vol_name failed");
        return 0;
    }
}

#-----------------------------------------------------------------------------------------------

=head2 RaidSqa_get_aggr_show_space

RaidSqa_get_aggr_show_space(api_set_obj=>$api_set_obj,conntype=>$conntype,filer_vol=>$filer_vol,options=>$options);

This subroutine returns HASH ref on success else return 0.

The following arguments are accepted:

=over

=item api_set_obj

(Required): An NACL::APISet object

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item filer_vol

(Required): Name of the aggr for which show_space cmd will be executed.

=item options

(Optional): Default is k for kilobytes.Possible values : 'k', 'm', 'g', and 't' indicates the
            desired units, namely 'kilobytes', 'megabytes', 'gigabytes' and 'terabytes' (respectively).

=back

=cut

#------------------------------------------------------------------------------------------------
sub RaidSqa_get_aggr_show_space {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype  => {type => SCALAR, default => "#ssh", optional => 1},
                        filer_vol         => {type => SCALAR},
                        options  => {type => SCALAR, default => 'k', optional => 1},
                    });

    my $api_set_obj      = $opts{api_set_obj};
    my $filermode        = $opts{filermode};
    my $conntype         = $opts{conntype};
    my $filer_vol        = $opts{filer_vol};
    my $options          = $opts{options};
    my $CMDTIMEOUT       = 300;

    my $cmd        = "priv set -q diag;aggr show_space -"."$options $filer_vol";
    my $raw_output = RaidSqa_filer_cmd(api_set_obj => $api_set_obj, timeout => $CMDTIMEOUT, cmd => $cmd);
    my @output     = split(/\r?\n/, $raw_output);
    my %aggr_show_space;
    my $match = 0;
    foreach my $line (@output) {
        if ($line =~ /Total\s+space\s+(\d+)\w+\s+(\d+)\w+\s+(\d+)\w+/i) {
            $aggr_show_space{'allocated'} = $1;
            $aggr_show_space{'used'}      = $2;
            $aggr_show_space{'available'}     = $3;
            $match                        = 1;
            last;
        } else {
            next;
        }
    } ## end foreach my $line (@output)
    if ($match) {
        return (\%aggr_show_space);
    } else {
        return 0;
    }
}

#------------------------------------------------------------------------------------------------

=head2 RaidSqa_print_status

RaidSqa_print_status(api_set_obj=>$api_set_obj, conntype=>$conntype, type => $type, aggr => $Filer_vol; vol => $Filer_Vol; process => $process , config => $config, version=> $version);

This subroutine prints the aggr status | vol status | process info | config info | version details depending on the data specified.

The following arguments are accepted:

=over

=item api_set_obj

(Required): Hostspec to filer

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item type

(Optional): Type of Status print, Default its aggr, Can be aggr|Vol|Config|Process|Version.

=item aggr

(Optional):  Aggregate Name for which status has to be printed.
          Default prints all aggregate status. Else the status of the named aggregate.

=item vol

(Optional):  Volume  Name for which status has to be printed.
          Default prints all Volumes status. Else the status of the named Volume.

=item Config
al
(Optional):  sysconfig for the system.
          Will be printed only when specified in type or if set to 1.

=item Process

(Optional):  Process information of the system will be printed
          Will be printed only when specified in type or if set to 1.

=item Version

(Optional):  Version information of the system will be printed
          Will be printed only when specified in type or if set to 1.

=back

=cut

#------------------------------------------------------------------------------------------------
sub RaidSqa_print_status {
    my (%opts) =
      RaidSqa_validate(@_,
                        {api_set_obj   => {type => OBJECT, isa     => 'NACL::APISet'},
                         conntype      => {type => SCALAR, default => "#ssh", optional => 1},
                         type          => {type => SCALAR, default => "aggr", optional => 1},
                         aggr          => {type => SCALAR, default => "", optional => 1},
                         vol           => {type => SCALAR, default => "", optional => 1},
                         process       => {type => SCALAR, default => 0, optional => 1},
                         config        => {type => SCALAR, default => 0, optional => 1},
                         version       => {type => SCALAR, default => 0, optional => 1},
                         raid_config   => {type => SCALAR, default => 0, optional => 1},
                         node          => {type => SCALAR, default => "local", optional => 1},
                        }
        );
    my $api_set_obj      = $opts{api_set_obj};
    my $conntype         = $opts{conntype};
    my $type             = $opts{type};
    my $vol              = $opts{vol};
    my $aggr             = $opts{aggr};
    my $process          = $opts{process};
    my $config           = $opts{config};
    my $version          = $opts{version};
    my $node             = $opts{node};
    my $result           = 1;
       if(!defined ($type)) {
        $type = "aggr" ;
    }
    my $output;
    try {
        if ( $type =~ /aggr|all/i) {
            $Log->comment("$pkg_name: print Aggr Information ");
            my $response =
            $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; aggr status $aggr \" ");
            $output = $response->get_raw_output();
            $response =
            $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; aggr status -v $aggr \" ");
            $output = $response->get_raw_output();
            $response =
            $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; aggr status -r $aggr \" ");
            $output = $response->get_raw_output();
            $response =
            $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; aggr show_space -h $aggr \" ");
            $output = $response->get_raw_output();
        }
      if ( $type =~ /vol|all/i) {
            $Log->comment("$pkg_name: print Volume Info ");
            my $response =
            $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; vol status $vol \" ");
            $output = $response->get_raw_output();
            $response =
            $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; vol status -v $vol \" ");
            $output = $response->get_raw_output();
            $response =
            $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; vol status -r $vol \" ");
            $output = $response->get_raw_output();
        }
      if ( ($type =~ /process|all/i ) or $process eq 1 ) {
            $Log->comment("$pkg_name: print Process Info ");
            my $response =
            $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; ps ah \" ");
            $output = $response->get_raw_output();
        }
      if (($type =~ /\bconfig\b|all/i) or $config eq 1 ) {
            $Log->comment("$pkg_name: print config Info ");
            my $response =
            $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; sysconfig -av \" ");
            $output = $response->get_raw_output();
        }
      if (($type =~ /version|all/i) or $version eq 1)  {
            $Log->comment("$pkg_name: print version Info here");
            my $response =
            $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; version -b \" ");
            $output = $response->get_raw_output();
        }
      if (($type =~ /\braid_config\b|all/i))  {
            $Log->comment("$pkg_name: printing raid_config");
            my $response =
            $api_set_obj->system_node_run(node => $node, command => "\"priv set -q diag; raid_config\" ");
            $output = $response->get_raw_output();
        }
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $output = _handle_nacl_exception($exception_object);
        $result = 0;
    };
     return $result;
} ## end sub RaidSqa_print_status

#-----------------------------------------------------------------------------------------------
=head2 RaidSqa_disk_simpull

RaidSqa_disk_simpull(api_set_obj=>$api_set_obj, conntype=>$conntype, disk_list=>\@disk_list);

This subroutine pulls the disk from the filer.

It returns disks pulled and disks not pulled array on success.

The following arguments are accepted:

=over

=item api_set_obj

(Required) : a NACL API object

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item disk_list

(Optional): refrence to the list of disk (but any one of disk_list or disk_name is required)

=back

=cut
#------------------------------------------------------------------------------------------------

sub RaidSqa_disk_simpull {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype  => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_list => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        node    => {type => SCALAR, default => "local", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_list   = $opts{disk_list};
    my $node = $opts{node};

    if (defined($disk_list)) {
        my @disk_list = @$disk_list;
        $api_set_obj->system_node_run(node    => $node,
                                      command => "\"priv set -q diag; disk simpull @disk_list\"");
    } else {
        $Log->comment("$pkg_name: Parameter $disk_list not defined, need to define.");
        return 0;
    }

    my (@disks_pulled,@disks_not_pulled);
    #--------------------------------------------------------------
    #get disks pulled and disks not pulled for passed disk list.
    #--------------------------------------------------------------
    $Log->comment("$pkg_name: get disks pulled and disks not pulled for passed disk list.");
    foreach my $disk_name (@$disk_list) {
        my $flag = 0;
        my ($wait_interval) = 20;
        my $max_wait = 300;
        while($max_wait-=10) {
            my $disk_path_name2 = RaidSqa_get_disk_other_path_name(api_set_obj => $api_set_obj,disk_name => $disk_name);
            $Log->comment("$pkg_name: disk_path_name2 for disk $disk_name : $disk_path_name2");
            if (!$disk_path_name2) {
                $flag = 1;
                last;
            } else {
               $Log->comment("$pkg_name: wait for 20 seconds, to check disk simpull happened.");
               sleep $wait_interval;
            }
        }
        if($flag) {
           push(@disks_pulled,$disk_name);
        } else {
            push(@disks_not_pulled,$disk_name);
        }
    } ## end foreach
    $Log->comment("$pkg_name: disks pulled : @disks_pulled");
    $Log->comment("$pkg_name: disks not pulled : @disks_not_pulled");
    return(\@disks_pulled,\@disks_not_pulled);
} ## end sub RaidSqa_disk_simpull

#-----------------------------------------------------------------------------------------------
=head2 RaidSqa_disk_simpush

RaidSqa_disk_simpush(api_set_obj=>$api_set_obj, filermode=>$filermode, conntype=>$conntype, disk_list=>\@disk_list);

This subroutine pushes the disk to the filer.

It returns disks not pushed in array on success.

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item disk_list

(Optional): refrence to the list of disk (but any one of disk_list or disk_name is required)

=back

=cut
#------------------------------------------------------------------------------------------------

sub RaidSqa_disk_simpush {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype  => {type => SCALAR, default => "#ssh", optional => 1},
                        filermode => {type => SCALAR, default => "normal", optional => 1},
                        disk_list => {type => ARRAYREF | UNDEF, default => undef},
                        node    => {type => SCALAR, default => "local", optional => 1},});

    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $disk_list   = $opts{disk_list};
    my $filermode   = $opts{filermode};
    my $node = $opts{node};

    if ($filermode ne "normal") {
        $conntype = "#console";
    }

    my (@disks_not_pushed,@disks_pushed);
    #my $conn = make_conn($host, $conntype);
    my ($result, $output, @disk_list, $disk_string);

    if (!defined($disk_list)) {
        $Log->comment(
            "parameter $disk_list not defined, need to define");
        return 0;
    }
    my $err_counter = 0;
    #--------------------------------------------------------------
    #Get disks pulled path.
    #--------------------------------------------------------------
    #my ($disks,$disks_pulled_path) = RaidSqa_get_pulled_disks(host => $host);
    #$Log->comment("$pkg_name: disks present in diskpull dir : @$disks");
    #$Log->comment("$pkg_name: disks pulled path in diskpull dir : @$disks_pulled_path");

    my @push_disk_paths = ();
    my @non_pulled_disks = ();
    my $pulled_disk_info = RaidSqa_get_pulled_disks(api_set_obj => $api_set_obj);
    my %adapter_hash = ('v4' => 'v0','v5' => 'v1','v6' => 'v2','v7' => 'v3');
    if(defined($disk_list)) {
       foreach my $disk (@$disk_list) {
           my $adapter = $adapter_hash{$1} if($disk =~ /(v\d)\.(\d+)/);
           my $disk_name2 = "$adapter." . "$2";
           if(exists($pulled_disk_info->{$disk})) {
                push(@push_disk_paths, $pulled_disk_info->{$disk});
           } elsif (exists($pulled_disk_info->{$disk_name2})) {
                push(@push_disk_paths, $pulled_disk_info->{$disk_name2});
           } else {
                $Log->comment("Warn : disk $disk is not been pulled.");
                $err_counter++;
                push @non_pulled_disks, $disk;
           }
       }
    }

    #--------------------------------------------------------------
    #Push back the disks.
    #--------------------------------------------------------------
    my $response =
        $api_set_obj->system_node_run(node    => $node,
                                      command => "\"priv set -q diag; disk simpush @push_disk_paths\"");
        $output = $response->get_raw_output();

    if($output =~ /Failed to open/i) {
       $err_counter++;
       $Log->comment("$pkg_name: Error occured while pushing disks to vsim");
    }

    if($err_counter > 0) {
        return 0;
    }
    #--------------------------------------------------------------
    #Get disk list not pushed to filer of pulled disks.
    #--------------------------------------------------------------
    $Log->comment("Get disk list not pushed to filer of pulled disks.");
    foreach my $disk_name (@$disk_list) {
        my $flag = 0;
        my ($wait_interval) = 20;
        my $max_wait = 300;
        while($max_wait-=10) {
            my $disk_path_name2 = RaidSqa_get_disk_other_path_name(api_set_obj => $api_set_obj,disk_name => $disk_name);
            $Log->comment("$pkg_name: disk_path_name2 for disk $disk_name : $disk_path_name2");
            if ($disk_path_name2) {
                $flag = 1;
                last;
            } else {
               $Log->comment("$pkg_name: wait for 20 seconds, to check disk simpush happened.");
               sleep $wait_interval;
            }
        }
        if($flag) {
           push(@disks_pushed,$disk_name);
        } else {
            push(@disks_not_pushed,$disk_name);
        }
    }
    $Log->comment("$pkg_name: disks pushed : @disks_pushed");
    $Log->comment("$pkg_name: disks not pushed : @disks_not_pushed");
    return (\@disks_not_pushed);
} ## end sub RaidSqa_disk_simpush

#-----------------------------------------------------------------------------------------------
=head2 RaidSqa_get_pulled_disks

RaidSqa_get_pulled_disks(api_set_obj => $api_set_obj,conntype=>$conntype);

This subroutine returns list of pulled disks, pulled disks path present in pulled dir.

It returns an array on success

The following arguments are accepted:

=over

=item api_set_obj

(Required) : a NACL API object

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=back

=cut
#------------------------------------------------------------------------------------------------

sub RaidSqa_get_pulled_disks {
    my (%opts) =
      RaidSqa_validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype  => {type => SCALAR, default => "#ssh", optional => 1},
                        filermode => {type => SCALAR, default => "normal", optional => 1},});
    my $api_set_obj = $opts{api_set_obj};
    my $conntype    = $opts{conntype};
    my $filermode = $opts{filermode};
    if ($filermode ne "normal") {
        $conntype = "#console";
    }

    #my $conn = make_conn($host, "#console");
    my ($result, $output, @disk_list, $disk_string);
    my %hash = ();
    my (@disks_pulled,@disks_pulled_path);
    my $found = 0;
    my $cmd_output = RaidSqa_systemshell_cmd(api_set_obj => $api_set_obj,cmd => "ls /sim/dev/,disks/,pulled");
    $Log->comment("$pkg_name: cmd_output : $cmd_output");
    #my @arr = split(/\r?\n/, $cmd_output);
    my @arr = split(/\s+/,$cmd_output);
    $Log->comment("arr : @arr");
    foreach my $var (@arr) {
        $Log->comment("var : ,$var,");
        if($var =~ /v\d+\.\d+/) {
            my @fields = split(/:/, $var);
            $Log->comment("fields : ,$fields[0],");
            $hash{$fields[0]} = $var;
            #push(@disks_pulled,$fields[0]);
            #push(@disks_pulled_path,$var);
        }
    }
    #$Log->comment("$pkg_name: disks pulled : @disks_pulled");
    #return(\@disks_pulled,\@disks_pulled_path);
    return(\%hash);
} ## end sub RaidSqa_get_pulled_disks

#------------------------------------------------------------------------------------------------

#=================================== INTERNAL SUBROUTINES ======================

sub emslog_parseNR {
    my ($line) = @_;
    my %emslog_nr = ();
    my ($header);
    if ($line =~ /^\s*<NR (.*)\/>\s*$/) {
        $header = "$1";
    } else {
        $Log->comment("$pkg_name: parseNR mismatch: '$line'\n");
        croak $line;
    }

    #
    # Parse the header (NR) section of the event record
    #
    while ($header ne "") {
        if ($header =~ /^\s*([^=]+)=\"([^\"]*)\"\s*(.*)$/) {
            $emslog_nr{$1} = $2;
            $header = $3;
        } else {
            $header = "";
        }
    }
    return (%emslog_nr);
} ## end sub emslog_parseNR

sub emslog_parse {
    my ($line, $emslog_nr, $emslog_entry) = @_;

    if ($line =~ /^<NR/) {
        %{$emslog_nr} = emslog_parseNR($line);
        return 0;
    }

    if ($line =~ /^<LR/) {
        ${$emslog_entry} = $line;
        return 0;
    }

    if ($line =~ /^<\/LR/) {
        ${$emslog_entry} = ${$emslog_entry} . $line;
        return 1;
    }

    ${$emslog_entry} = ${$emslog_entry} . " $line";
    return 0;
} ## end sub emslog_parse

sub emslog_process_line {
    my ($line, $emslog_events, $emslog_event_lines, $emslog_event_by_id, $emslog_event_by_type) =
      @_;

    my ($header, $event, $id, $i);
    my (@headerlist, @eventlist);
    my ($type,       $version);

    if ($line =~ /^\s*<LR (.*)>\s*<(.*)\/>\s*<\/LR>\s*$/) {
        $header = "$1";
        $event  = $2;
    } else {
        $Log->comment("$pkg_name: emslog_process_line: mismatch\n");
        return;
    }

    #
    # Get the id.
    #
    if ($header =~ /\s*id=\"([^\"]+)\"/) {
        $id = $1;
    }

    push @{$emslog_events},      $id;
    push @{$emslog_event_lines}, $line;

    #
    # Parse the header (LR) section of the event record
    #
    while ($header ne "") {
        if ($header =~ /^\s*([^=]+)=\"([^\"]*)\"\s*(.*)$/) {
            $emslog_event_by_id->{$id}{header}{$1} = $2;
            $header = $3;
        } else {
            $header = "";
        }
    }

    if ($event =~ /^\s*(\S+)\s*(.*)/) {
        $type  = $1;
        $event = $2;
    }

    if ($type =~ /^(\S+)_([0-9]+)$/) {
        $type = $1;
        $type =~ s/_/\./g;
    }

    emslog_add_by_type($type, $id, $emslog_event_by_type);
    $emslog_event_by_id->{$id}{type} = $type;

    #
    # Parse the per-event specific part of the event record
    #
    while ($event ne "") {
        if ($event =~ /^\s*([^=]+)=\"([^\"]*)\"\s*(.*)$/) {
            $emslog_event_by_id->{$id}{$1} = $2;
            if (!exists($emslog_event_by_id->{$id}{attr})) {
                $emslog_event_by_id->{$id}{attr} = $1;
            } else {
                $emslog_event_by_id->{$id}{attr} .= ",$1";
            }
            $event = $3;
        } else {
            $event = "";
        }
    } ## end while ($event ne "")
} ## end sub emslog_process_line

sub emslog_add_by_type {
    my ($type, $id, $emslog_event_by_type) = @_;
    push(@{$emslog_event_by_type->{$type}}, $id);
}

#------------------------------------------------------------------------------
=head2 RaidSqa_get_mixed_storage_type_disks

RaidSqa_get_mixed_storage_type_disks(api_set_obj=>$api_set_obj,conntype=>$conntype,storage_type = $storage_type, disk_count  => $disk_count, checksum_type =>$checksum_type, pool => $pool, rpm =>$rpm, exclude_primary_storage_type => $exclude_primary_storage_type);

This subroutine lists the disks based on exclude_primary_storage_type parameter.

The following arguments are accepted:

=over

=item api_set_obj

(Required): Hostspec to filer

=item disk_count

(Required) : Number of disk to be used for volume/aggegate creation. Need to define at least on parameter from disk_count and disk_list for volume creation, if none is defined, it will use disk_count as 6

=item storage_type

(Required)):  Disk storage type. Can be ATA, SAS, FCAL, BSAS, SATA, SSD, FSAS. If nothing is defined  than it tries to find the prefered storage type and use that type of disk.

=item checksum_type

(Optional):  Aggr checksum type. Can be block, zoned, advanced_zoned, default is block

=item pool

(Optional): Name of the spare pool from which to get the list of given checksum_type disks.

= item exclude_primary_storage_type

(Optional): storage_type can be 0 or 1.
            0 - returns mixed disks of compatible type.
            1 - returns disks of only compatible type.
=back

=cut
#------------------------------------------------------------------------------
sub RaidSqa_get_mixed_storage_type_disks {
    my %opts = RaidSqa_validate(@_,
                    {
                        api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        storage_type  => {type=>SCALAR},
                        disk_count    => {type=>SCALAR},
                        checksum_type => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                        pool          => {type=>SCALAR|UNDEF, default=>undef, optional=>1},
                        exclude_primary_storage_type => {type=>SCALAR, default=>1, optional=>1},
                    }
                );
    my $api_set_obj   = $opts{api_set_obj};
    my $storage_type  = $opts{storage_type};
    my $disk_count    = $opts{disk_count};
    my $checksum_type = $opts{checksum_type};
    my $pool          = $opts{pool};
    my $exclude_primary_storage_type = $opts{exclude_primary_storage_type};

    my $storage_type_family = RaidSqa_get_storage_type_family(api_set_obj=>$api_set_obj, storage_type=>$storage_type);
    $Log->comment("$pkg_name: pool : $pool");
    my @disk_list  = ();
    @disk_list = RaidSqa_get_disk_list(api_set_obj   =>$api_set_obj,
                                       pool_name     => $pool,
                                       checksum_type => $checksum_type,
                                       spare_only    => "yes");
    $Log->comment("$pkg_name: disk_list : @disk_list");
    $Log->comment("$pkg_name: storage_type_family : @$storage_type_family");

    ###changed the variable from input_storage_type to storage_type
    my @mixed_storage_types;
    my $disks_by_storage_type = RaidSqa_sort_disks(api_set_obj=>$api_set_obj, disk_list=>[@disk_list], disk_property_name=>'disk_type');
    my @disks_tobe_added = ();
    my $flag = 0;
    if($exclude_primary_storage_type == 1){
        @mixed_storage_types = grep !/^$storage_type/i, @$storage_type_family;
        foreach my $mixed_storage_type (@mixed_storage_types)
        {
            if(defined(@{$disks_by_storage_type->{$mixed_storage_type}})){
                push (@disks_tobe_added, @{$disks_by_storage_type->{$mixed_storage_type}});
            }
            if($disk_count  < scalar(@disks_tobe_added)) {
                @disks_tobe_added = @disks_tobe_added[0..$disk_count-1];
                $flag = 1;
            }
            last if($flag == 1);
        }
     } else {
        @mixed_storage_types = @$storage_type_family;
        @disks_tobe_added = map {shift @{$disks_by_storage_type->{$_}}} @mixed_storage_types;
        @disks_tobe_added = grep /[^()]/, @disks_tobe_added; #This is to avoid having any undef entries
        #Then select remaining disk
        if($disk_count  < scalar(@disks_tobe_added)) {
            @disks_tobe_added = @disks_tobe_added[0..$disk_count-1];
        } else {
            $disk_count -= scalar(@disks_tobe_added);
        }
        my $max_tries = $disk_count*2;
        while($disk_count != 0) {
            foreach my $disk_type (@mixed_storage_types) {
                if(ref($disks_by_storage_type->{$disk_type})) {
                    my $disk = shift @{$disks_by_storage_type->{$disk_type}};
                    $Log->comment("$pkg_name: $disk_count : disk : $disk");
                    if($disk ne ""){
                        push(@disks_tobe_added, $disk);
                        $Log->comment("$pkg_name: disk_count : $disk_count");
                        $disk_count--;
                        last if($disk_count == 0);
                    }
                }
            }
            $max_tries--;
            last if($max_tries == 0);
        }
    }
    return \@disks_tobe_added;
}

#------------------------------------------------------------------------------
=head2 RaidSqa_sort_disks

RaidSqa_sort_disks(api_set_obj=>$api_set_obj,conntype=>$conntype,filermode=> $filermode, disk_list=> \@disk_list,disk_property_name=> $disk_property_name);

This subroutine sorts the disks based on the disk_property_name.

The following arguments are accepted:

=over

=item api_set_obj

(Required): Hostspec to filer

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.
For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#ssh". For filermode "MAINT" it is "#console" by default.

=item disk_list

(Optional): Reference to an array of disks from which specific disks need to be identified.
If not specified, it will consider the list of spares.

=item disk_property_name

(Required): It should be disk_type .

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_sort_disks{
    my (%opts) =
      RaidSqa_validate(@_, {
        api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
        filermode          => {type => SCALAR, default => "normal", optional => 1},
        conntype           => {type => SCALAR, default => "#ssh", optional => 1},
        disk_list          => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
        node               => {type => SCALAR, default => 'local', optional => 1},
        disk_property_name => {type => SCALAR, default => undef, optional => 1}
    });

    my $api_set_obj        = $opts{api_set_obj};
    my $filermode          = $opts{filermode};
    my $conntype           = $opts{conntype};
    my $disk_property_name = $opts{disk_property_name};
    my $disk_list          = $opts{disk_list};
    my $node               = $opts{node};
    my $hash               = {};
    my @disk_list;
    my $disk_property_value;
    # create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }
    my $raid_obj  = new RaidInfo($conn, node => $node, api_set_obj => $api_set_obj);

    if(defined($disk_list)){
        @disk_list = @{$disk_list};
    } else {
        @disk_list = RaidSqa_get_disk_list(api_set_obj=>$api_set_obj);
    }
    foreach my $disk (@disk_list){
        my $disk_obj = $raid_obj->disk("$disk");
        if(!defined($disk_obj)){
            $disk = RaidSqa_find_active_path_disk_name(api_set_obj=>$api_set_obj, disk_name => $disk);
            $disk_obj = $raid_obj->disk("$disk");
            next if(!defined($disk_obj));
        }
        if(defined($disk_obj->parameter($disk_property_name))){
            $disk_property_value = $disk_obj->parameter($disk_property_name);
            push(@{$hash->{$disk_property_value}}, $disk);
        }
    }
    return $hash;
}#end RaidSqa_sort_disks

#------------------------------------------------------------------------------
=head2 RaidSqa_get_storage_type_family

RaidSqa_get_storage_type_family(api_set_obj=>$api_set_obj,conntype=>$conntype,storage_type => $storage_type);

This subroutine returns the storage type.

The following arguments are accepted:

=over

=item api_set_obj

(Required): Hostspec to filer

=item storage_type

(Required)):  Disk storage type. Can be ATA, SAS, FCAL, BSAS, SATA, SSD, FSAS. If nothing is defined  than it tries to find the prefered storage type and use that type of disk.

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_get_storage_type_family {
    my %args = RaidSqa_validate(@_, {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                                     storage_type => {type=>SCALAR}});
    my $api_set_obj         = $args{api_set_obj};
    my $storage_type        = $args{storage_type};
    my @storage_type_groups = (
        ['ATA', 'SATA', 'BSAS','FSAS'],
        ['FCAL', 'SAS'],
        ['SSD', 'BSSD'],
        ['MSATA'],
    );
    my @valid_storage_types = ('ATA', 'SATA', 'BSAS', 'MSATA', 'FCAL', 'SAS','SSD','BSSD','FSAS');

    if(! grep /$args{storage_type}/,  @valid_storage_types) {
        die("  Input storage type is not one of valid storage types");
    }

    foreach my $ref (@storage_type_groups) {
        if(grep(/$args{storage_type}/, @$ref)) {
            return $ref;
        }
    }
    die(" Optional storage type could not be found for $args{storage_type}");

} ## end RaidSqa_get_storage_type_family

#------------------------------------------------------------------------------

=head2 RaidSqa_validate

RaidSqa_validate(@arguments, \%validation_specifications);

This subroutine is overloaded subroutine and drops the parameters from @arguments which are not listed in validation_specifications.
The resultant list of arguments will be then sent to validate subroutine

It returns an array with validated arguments on success

The following arguments are accepted:

=over

=item @argument

(Required): : Orginal list of suvroutines argument which was passed to the parent subroutine

=item \%validation_specification

(Required) : validation specification of arguments which is to be passed to validate subroutine

=back

=cut

#------------------------------------------------------------------------------

sub RaidSqa_validate {
    my @args              = @_;
    my $ref_validate_args = pop @args;
    my %args1             = @args;
    foreach my $arg (keys %args1) {
        if (!defined($ref_validate_args->{$arg})) {
            delete($args1{$arg});
        }
    }

    @args = %args1;
    my @ret_args = validate(@args, $ref_validate_args);
    return @ret_args;
} ## end sub RaidSqa_validate

#------------------------------------------------------------------------------

=head2 _get_raidinfo_object

_get_raidinfo_object(api_set_obj=>$api_set_obj,conntype=>$conntype,storage_type => $storage_type);

This is internal method to get RaidInfo object for disk, group, plex or aggregate.

The following arguments are accepted:

=over

=item api_set_obj

(Required): Hostspec to filer

=item refresh

(Required):  Boolean, If set to 1, RaidInfo object would be refreshed

=item disk_name

(Required):  Name of the disk

=item filer_vol

(Required):  Name of aggregate/volume

=item plex_name

(Required):  Name of the plex

=item group_name

(Required):  Name of the raid group

=back

=cut

#------------------------------------------------------------------------------
sub _get_raidinfo_object {
    my %args = RaidSqa_validate(@_, {
                    api_set_obj => {type => OBJECT, isa => 'NACL::APISet'},
                    raid_obj    => {type => OBJECT, isa => 'RaidInfo'},
                    refresh     => {type => SCALAR, default => 1, optional => 1},
                    disk_name   => {type => SCALAR|UNDEF, default => undef, optional => 1},
                    filer_vol   => {type => SCALAR|UNDEF, default => undef, optional => 1},
                    plex_name   => {type => SCALAR|UNDEF, default => undef, optional => 1},
                    group_name  => {type => SCALAR|UNDEF, default => undef, optional => 1},
                });
    my $raid_object = $args{raid_obj};
    my $api_set_obj = $args{api_set_obj};
    my $refresh     = $args{refresh};

    if($refresh != 0) {
        $raid_object->refresh();
    }

    my $object = undef;
    if(defined($args{disk_name})) {
        $object = $raid_object->disk($args{disk_name});
        if(!defined($object)){
            $Log->warn("$pkg_name: RaidInfo object not found, disk - ".$args{disk_name});
            my $disk_name = RaidSqa_find_active_path_disk_name(api_set_obj => $api_set_obj, disk_name => $args{disk_name});
            $object = $raid_object->disk($disk_name);
        }
        $Log->warn("$pkg_name: RaidInfo object not found, disk - ".$args{disk_name}) if(!defined($object));
    } elsif(defined($args{plex_name})) {
        $object = $raid_object->plex($args{plex_name});
        $Log->warn("$pkg_name: RaidInfo object not found, Plex - ".$args{plex_name}) if(!defined($object));
    } elsif(defined($args{group_name})) {
        $object = $raid_object->raid_group($args{group_name});
        $Log->warn("$pkg_name: RaidInfo object not found, Group - ".$args{group_name}) if(!defined($object));
    } elsif(defined($args{filer_vol})) {
        $object = $raid_object->aggregate($args{filer_vol});
        $object = $raid_object->vol($args{filer_vol}) if(!defined($object));
        $Log->warn("$pkg_name: RaidInfo object not found, Aggregate - ".$args{filer_vol}) if(!defined($object));
    }

    return defined($object) ? $object : 0;
    return 0;
}

#------------------------------------------------------------------------------

=head2 _parse_scramble_dumpblock_file

_parse_scramble_dumpblock_file(filename);

This is internal method to parse a file containing output of scramble dumpblock command.

The following arguments are accepted:

=over

=item filename

(Required): name of the file [should be nfs mounted filename]

=back

=cut

#------------------------------------------------------------------------------

sub _parse_scramble_dumpblock_file {
    my %opts = RaidSqa_validate(@_, {
        file_name    => {type => SCALAR},
    });
    my $file_name = $opts{file_name};

    sudo("chmod 777 $file_name");
    open (FH, "<$file_name") or die "Could not open file_name: $@";
    my $index           = 0;
    my %fields          = ();            #hash with field as key and its index in column as value
    my $stripe_mapping  = {};            #$stripe_mapping->{$stripe_number} = []

    #possible fields => fsid,pub,fileid,level,fbn,vvbn,pvbn_from_parent,pvbn_from_container,wrg,disk,dbn,sno,snapid,seg_total,seg_free,seg_cc,seg_sc
    my @required_fields = ('disk','dbn','fbn','level');

    while(<FH>) {
        my $line = $_;
        next if($line =~ /^\s*#/);
        next if(($line !~ /fsid,pub/) && $index == 0);        #first find header
        chomp($line);
        if($line =~ /fsid,pub/) {
            my @header_fields = split(/,/, $line);
            %fields = map {$_, $index++} @header_fields;
            next;
        }

        my @field_values = split(/,/, $line);
        my %hash = map {$_, $field_values[$fields{$_}]} @required_fields;

        next if(int($hash{level}) != 0);
        $stripe_mapping->{$hash{dbn}} = [] if(!defined($stripe_mapping->{$hash{dbn}}));
        push (@{$stripe_mapping->{$hash{dbn}}}, \%hash);
    }

    return $stripe_mapping;
}

#-------------------------------------------------------------------------------
# This subroutine is to handle exception in common scenario
sub _handle_nacl_exception {
    my ($exception_object) = shift;
    my ($response, $output, $exception_text, $result, $exception_type, $command);
    if (ref($exception_object) !~ /NACL::APISet::Exceptions/) {
        $Log->comment("$pkg_name: _handle_exception:: Invalid exception object is passed to handle exception subroutine");
        return (0, 0);
    } else {
        $exception_type = ref($exception_object);
    }

    $output         = 0;
    $exception_text = $exception_object->text();
    if ($exception_type =~ /ResponseException/) {
        $response = $exception_object->get_response_object();
        $output   = $response->get_raw_output();
        $command  = $response->get_command();
        $Log->comment("$pkg_name: Exception occured of type: $exception_type for command $command");
    } else {
        $Log->comment("$pkg_name: Exception occured of type: $exception_type");
    }

    $Log->comment("$pkg_name: _handle_exception->raw_output :: $output");
    $Log->comment("$pkg_name: _handle_exception->exception_text :: $exception_text");
    return $output;
} ## end sub _handle_nacl_exception

1;
