#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/San/SanFiler7M.pm#3 $

# Copyright (c) 2007-2009 NetApp Inc.
# All rights reserved.
#
## @summary Provide an interface to Filer
## @author knopp@netapp.com, pressley@netapp.com
## @status private
## @pod here

package San::SanFiler7M;
use Tharn;
use San::SanClient::ClientUtil;
use San::SanLog;
use San::SanUtil;
use San::SanCache;
use Time::HiRes qw(gettimeofday);
use Error;
use Filer::Version qw(get_ontap_version);
use List::MoreUtils qw(apply);

#
# Compiler directives.
#
use strict;

#
# Module import.
#
use File::Basename;
use San::SanFiler::Volume;
use San::SanFiler::IGroup;
use San::SanFiler::Target;
use San::SanFiler::ISCSITarget;
use San::SanFiler::Vfiler;
use CmdsDriver;
use XML::Simple;
use Data::Dumper;
use San::SanFiler::Cmds::Portset;
use San::SanFiler::Cmds::Fcp;

use vars qw(
            $_FILER
            @EXPORT
            @ISA
            );

# Init globals that are exported.
# Data modes.
use constant DUMPER_DEBUG_LEVEL => 10;

use vars qw(@EXPORT @ISA);

@ISA = qw(Error SanLog SanUtil Portset Fcp SanCache);
###############################################################################
#                               Public Methods
###############################################################################

###############################################################################
# Method:    DESTROY
# Objective: Clean up and disconnect from telnet session.
# Details:   See POD documentation at the end of this file.
###############################################################################

sub DESTROY
{
    my $self = shift;
    # Get out of all CLI levels if we are still connected.
    # if ($self->{connected}) {
    # Tharn::disconnect($self->{connection});
    # Don't wait, we don't care about the results.
    #}
} # end sub DESTROY


sub destroy
{
    my ($self)=@_;

    $self->sanlog(-msg => "SanFiler::destroy(): disconnecting from filer");
    if ($self->{connected}) {
        Tharn::disconnect($self->{connection});
    }
    $self->{connection} = undef;
}

###############################################################################
# Method:    _new
# Objective: Create new SanFiler7M object
# Details:   See POD documentation at the end of this file.
###############################################################################

sub _new
{
    my($class, %args) = @_;
    my(
       $execute,
       $filer,
       $log,
       $passwd,
       $rc,
       $self,
       $show_cmd,
       $timeout,
       $debug_level,
       $user,
       $connection,
       $conn_type,
       $cmd_obj,
       $filer_version,
       $interface,
       $vfiler,
       $vfiler_ip,
       $partner_mode,
       $iscsi_alias,
       $enable_sancache,
       );

    # Create CLI object.  Create it before processing args so
    # we can perform full error reporting during the arg
    # processing stage.
    $self = {};
    bless $self;

    $debug_level     = 1;
    $filer_version   = "8.0";
    $interface       = "cli";     # cli, zapi, zapi_cli
    $vfiler          = "";
    $vfiler_ip       = undef;
    $partner_mode    = 0;         # 0 = no, 1 = yes
    $connection      = undef;
    $conn_type       = "#rsh";
    $enable_sancache = 0;         # sancache is disabled by default

    # Process arguments.  If an error occurs during the arg processing,
    # The error flag and errmsg values are set and the object is returned.
    # It is up to the caller to check for errors after the new call.
    no strict 'refs';
    foreach (keys %args) {
        if (/^-?filer$/) {
            length($filer = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"filer\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?timeout$/) {
            length($timeout = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"timeout\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?show_cmd$/) {
            length($show_cmd = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"show_cmd\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?debug_level$/) {
            length($debug_level = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"debug_level\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?loghdl$/) {

            length($log = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"loghdl\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?connection$/) {

            length($connection = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"connection\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?conn_type$/) {
            length($conn_type = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"conn_type\": $args{$_}");
                    return $self;
                };
        }
        elsif(/^-?interface$/) {
            length($interface = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"interface\": $args{$_}");
                    return $self;
                };
        }
        elsif(/^-?filer_version$/) {
            length($filer_version = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"filer_version\": $args{$_}");
                    return $self;
                };
        }
        elsif(/^-?vfiler$/) {
            length($vfiler = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"vfiler\": $args{$_}");
                    return $self;
                };
        }
        elsif(/^-?vfiler_ip$/) {
            length($vfiler_ip = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"vfiler_ip\": $args{$_}");
                    return $self;
                };
        }
        elsif(/^-?partner_mode$/) {
            length($partner_mode = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"partner_mode\": $args{$_}");
                    return $self;
                };
        }
        elsif(/^-?enable_sancache$/) {
            length($enable_sancache = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"enable_sancache\": $args{$_}");
                    return $self;
                };
        }
        else {
            $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                            "parameter specified: $_");
            return $self;
        }
    }

    use strict 'refs';
    # Initially unconnected.
    $self->{connected} = 0;

    # Show command mode.  By default when printing, the command
    # issued will be displayed.  This can be turned off by the user.
    $self->{show_cmd} = defined $show_cmd ? $show_cmd : 1;

    # Debug level, higher gives more data, > 100 gives call stack
    $self->set_debug_level($debug_level);

    $self->{show_data}=1;
    $self->{debug}=0;

    # Allow CLI commands to be formed, but not executed.
    $self->{execute} = defined $execute ? $execute : 1;

    # Allow CLI commands to be logged;
    $self->{log}=$log if (defined $log);

    # Determine timeout to use
    $self->{timeout}=(defined $timeout) ? $timeout : 60;

    # Enable or Disable sancache depending on the value of enable_sancache
    # no need to check for error
    if ($enable_sancache == 1)
    {
        $self->enable_sancache();
    }
    else
    {
        $self->disable_sancache();
    }

    # Verify mandatory parameters.
    unless (defined $filer and length $filer > 1) {
        $self->_set_err("Error: " . ref($self) . "::_new() mandatory ",
                        "parameter \"filer\" not specified");
        return $self;
    }

    # If we were given a connection, store it.  Otherwise we should
    # make a connection to the filer.
    unless (defined $connection)
    {
        $self->{$conn_type}= $conn_type;
        $self->{connection}= $self->_connect($filer, $timeout, $conn_type);
        return $self if ($self->error);
        $self->sanlog(-msg => ref($self) . "::_new(): Created a connection");
    }
    else
    {
        $self->{$conn_type}= $conn_type; #$connection;
        $self->{connection}= $connection;
        $self->sanlog(-msg => ref($self) . "::_new(): Using provided connection");
    }

    # Store the rest of the parameters in object.
    $self->{filer}          = $filer;
    $self->{name}           = $filer;
    $self->{filer_version}  = $filer_version;
    $self->{interface}      = $interface;
    $self->{vfiler}         = $vfiler;
    $self->{vfiler_ip}      = $vfiler_ip;
    $self->{partner_mode}   = $partner_mode;

    # Create a CmdsDriver object
    $cmd_obj = new CmdsDriver(-log          => \*ntest::NTEST_STDOUT,
                              -debug_level  => $debug_level);

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: " . ref($self) . "::_new(): Failed to instantiate " .
                        "CmdsDriver object " . $errmsg);
        return $self;
    }

    # Set CmdsDriver connection
    $cmd_obj->connection($self->{connection});

    #
    # Only create a connection for cli
    #
    if($self->{interface} eq "cli")
    {
        #
        # TBD, until we have the ability to rely fully on the zapi interface
        # we'll always make a connection.
        #
        # Set SanFiler connection
        # $self->{connection}=$self->_connect($filer, $timeout);
        # return $self if($self->error);
    }

    #
    # Set all CmdsDriver object parameters
    #
    $cmd_obj->filer_version($filer_version);
    $cmd_obj->interface($interface);
    $cmd_obj->vfiler($vfiler);
    $cmd_obj->vfiler_ip($vfiler_ip);
    $cmd_obj->partner_mode($partner_mode);
    $cmd_obj->filer($filer);

    $self->{cmd_obj} = $cmd_obj;

    # Load filer fcp targets
    $self->{targets} = [];
    if ($self->get_cfmode()=~/partner|cluster|Not Clustered|mixed|dual|standby|single_image/i) {
        $self->_load_targets();
        return $self if ($self->error);
    }

    # Load filer iscsi targets
    $self->{iscsi_targets} = [];
    $self->_load_iscsi_targets();
    return $self if ($self->error);

    # Load iscsi alias
    $self->{iscsi_alias} = undef;
    $self->_load_iscsi_alias();

    # Initialize error structure.
    $self->_clear_err;

    return $self;
} # end sub _new

################################################################################
# Subroutine Name: get_aggrs()
# Objective:       Return aggr list
# Details:         See POD documentation at the end of this file
################################################################################
sub get_aggrs
{
    my ($self)=@_;
    my (
        $cmd,
        @output,
        @return,
        );

    $cmd = "aggr status";

    @output=$self->execute($cmd,-hide=>1);
    if ($self->error()){
        return $ClientUtil::FALSE;
    }

    foreach my $line (@output) {
        # Strip any leading whitespace
        $line =~ s/^\s+//g;

        if ($line =~ /^Aggr\sState/i) {
            next;
        }
        push @return, (split(/\s+/, $line))[0];
    }

    return (wantarray()) ? @return : \@return;
}

###############################################################################
#                   Accessor Methods
###############################################################################
sub filer_version
{
    my ($self, $value) = @_;
    my $cmd_obj = $self->{cmd_obj};

    my $var = $cmd_obj->filer_version($value);
    $self->{filer_version} = $var;

    return $var;
} # end filer_version

sub interface
{
    my ($self, $value) = @_;
    my $cmd_obj = $self->{cmd_obj};

    if(not defined($value))
    {
        return $self->{interface};
    }

    my $var = $cmd_obj->interface($value);
    $self->{interface} = $var;

    #
    # If the interface is being set to cli and there isn't a connection
    # already created, create one.
    #
    if(($self->{interface} eq "cli") && ($cmd_obj->connection() eq ""))
    {
        # Set SanFiler connection
        # $self->{connection}=$self->_connect($self->{filer}, $self->{timeout});
        # return $self if($self->error);

        # Set CmdsDriver connection
        $cmd_obj->connection($self->{connection});
        $self->sanlog(-msg => "SanFiler::interface(): Created a connection");
    }

    return $var;
} # end interface

sub vfiler
{
    my ($self, $value) = @_;
    my $cmd_obj = $self->{cmd_obj};

    my $var = $cmd_obj->vfiler($value);
    $self->{vfiler} = $var;

    return $var;
} # end vfiler

sub vfiler_ip
{
    my ($self, $value) = @_;
    my $cmd_obj = $self->{cmd_obj};

    my $var = $cmd_obj->vfiler_ip($value);
    $self->{vfiler_ip} = $var;

    return $var;
} # end vfiler_ip

sub partner_mode
{
    my ($self, $value) = @_;
    my $cmd_obj = $self->{cmd_obj};

    my $var = $cmd_obj->partner_mode($value);
    $self->{partner_mode} = $var;

    return $var;
} # end partner_mode

sub cmd_obj
{
    my ($self) = @_;
    my $cmd_obj = $self->{cmd_obj};

    return $cmd_obj;
} # end cmd_obj

################################################################################
# Subroutine Name: aggr_exist()
# Objective:       Determines if aggregate exists
# Details:         See POD documentation at the end of this file
################################################################################
sub aggr_exist
{
    my ($self,$name)=@_;
    my (
        $cmd,
        @output,
        );

    unless (length $name){
        $self->_set_err("ERROR: SanFiler::aggr_exist() Required parameter \$name required");
        return $ClientUtil::FALSE;
    }

    $cmd = "aggr status ".$name;

    @output=$self->execute($cmd,-hide=>1);
    if ($self->error()){
        return $ClientUtil::FALSE;
    }

    if ($output[0]=~/No aggregate/i){
        $self->_set_err("Error: SanFiler::aggr_exist() no aggregate found ",@output);
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

################################################################################
# Subroutine Name: aggr_create()
# Objective:       Creates and aggregate
# Details:         See POD documentation at the end of this file
################################################################################
sub aggr_create
{
    my($self, %args) = @_;
    my $name       = "test_aggr";
    my $num_disks  = 3;
    my $blocktype  = undef;    # n/a prior to Boilermaker so no 32 bit default
    my $timeout    = 3600;
    my $raidtype   = "raid_dp";
    my $raidsize   = undef;
    my $force      = 0;
    my $cmd;
    my @output;
    my $cmd_interface = $self->interface();  # default to global value
    my $cmd_obj       = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    foreach (keys %args)
    {
        if (/-?name/)
        {
            length($name = $args{$_}) or
                do
                {
                    $self->_set_err("Invalid value for \"name\":",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?timeout/)
        {
            length($timeout = $args{$_}) or
                do
                {
                    $self->_set_err("Invalid value for \"timeout\":",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?num_disks/)
        {
            length($num_disks = $args{$_}) or
                do
                {
                    $self->_set_err("Invalid value for \"num_disks\"",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?blocktype/)
        {
            length($blocktype = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"blocktype\"",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?raidtype/)
        {
            length($raidtype = $args{$_}) or
                do
                {
                    $self->_set_err("Invalid value for \"raidtype\"",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?raidsize/)
        {
            if (length($raidsize = $args{$_}) and ($raidsize < 2 or $raidsize > 28)) {
                $self->_set_err("Invalid value for \"raidsize\"",
                               "$args{$_}");
                return $self;
            }
        }
        elsif (/-?force/)
        {
            $force = $args{$_};
            if (not length($force) or $force !~ /yes|no|1|0/i)
            {
                $self->_set_err("Invalid value for \"force\"",
                                "$args{$_}");
                return $self;
            };

            #
            # Change any 'yes' or 'no' sent to a 1 or a 0.
            #                        #
            $force = 1 if ($force =~ /yes/i);
            $force = 0 if ($force =~ /no/i);
        }
        elsif (/-?cmd_interface/)
        {
            $cmd_interface = $args{$_};
            if (not length($cmd_interface) or $cmd_interface !~ /cli|zapi|zapi_cli/i)
            {
                $self->_set_err("Invalid value for " .
                                "\"cmd_interface\"","$args{$_}");
                return $self;
            };
        }
        else
        {
            $self->_set_err("Invalid parameter specified: $_");
            return $self;
        }
    }

    if ($cmd_interface =~ /^zapi$/i)
    {
        my %cmd_args = (
                         "-cmd", "aggr-create",
                         "-cmd_filer", $self->get_name(),
                         "-check_result", 1,
                         "-cmd_interface", "zapi",
                         "-cmd_filer_version", "8.0",
                         "-check_output", 0,
                         "-check_result", 1,
                         "-aggregate", $name,
                         "-raid-type", $raidtype,
                         "-disk-count", $num_disks,
                       );
        $cmd_args{"-indirect-type"} = $blocktype if (defined $blocktype);
        $cmd_args{"-raid-size"} = $raidsize if (defined $raidsize);

        #
        # execute the command
        #
        $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

        if ($cmd_obj->error())
        {
            my $errmsg = join "\n", $cmd_obj->errmsg();
            $self->_set_err("Failed calling execute_cmd: " . $errmsg);
            $self->log_cmd_results($resultsHash_ref);
            return $ClientUtil::FALSE;
        }

        $self->log_cmd_results($resultsHash_ref);
    }
    else
    {
        #
        # Put the cmd together for the CLI.  First add the required
        # params then check the optional ones and add them
        #
        my %cmd_args = (
                         "-cmd", "aggr-create",
                         "-cmd_filer", $self->get_name(),
                         "-check_result", 1,
                         "-cmd_interface", "cli",
                         "-cmd_filer_version", "8.0",
                         "-check_output", 0,
                         "-check_result", 1,
                         "-aggregate", $name,
                         "-raid-type", $raidtype,
                         "-disk-count", $num_disks,
                       );
        $cmd_args{"-indirect-type"} = $blocktype if (defined $blocktype);
        $cmd_args{"-raid-size"}     = $raidsize if (defined $raidsize);
        $cmd_args{-force}           = "" if ($force);

        #
        # execute the command
        #
        $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

        if ($cmd_obj->error())
        {
            my $errmsg = join "\n", $cmd_obj->errmsg();
            $self->_set_err("Failed calling execute_cmd: " . $errmsg);
            log_cmd_results($resultsHash_ref);
            return $ClientUtil::FALSE;
        }

        $self->log_cmd_results($resultsHash_ref);

        #
        # Since we checked the result, lets see if our check was OK
        #
        unless ($resultsHash_ref->{result_check})
        {
            $self->_set_err("Unable to create aggr '$name'", $output[0]);
            return $ClientUtil::FALSE;
        }

        #
        # Check to make sure the aggr create started
        #
        my $lines = $resultsHash_ref->{output};
        my @output = split("\n", $lines);
        if ( ("@output"!~/has completed/i) and
             ("@output"!~/has been initiated/i) )
        {
            $self->_set_err("Unable to create aggr '$name'", $output[0]);

            return $ClientUtil::FALSE;
        }

        #
        # If we get here the aggr should either be completed or
        # initiated.  If it's complete, return TRUE.  Otherwise
        # hang around and check the status until it's online.
        #
        if ("@output"=~/has completed/i)
        {
            return $ClientUtil::TRUE;
        }
    } # end of cli case

    #
    # Check the aggr status and look for the aggr to come 'online'
    #
    $self->wait_for_aggr_online(-name       => $name,
                                -timeout    => $timeout);

    return $ClientUtil::TRUE;
}

################################################################################
# Subroutine Name: wait_for_aggr_online()
# Objective:       Waits for an aggr to be in an online state
# Details:         See POD documentation at the end of this file
################################################################################
sub wait_for_aggr_online
{
    my($self, %args) = @_;

    #
    # If we were given a name and/or timeout use them, otherwise use defaults
    #
    my $name     = (defined $args{-name})     ? $args{-name}     : "test_aggr";
    my $timeout  = (defined $args{-timeout})  ? $args{-timeout}  : 3600;
    my $interval = (defined $args{-interval}) ? $args{-interval} : 120;

    #
    # If we were given a cmd interface, use it.  If not use the current
    # value of the SanFiler cmd_interface.
    #
    my $cmd_interface = (defined $args{-cmd_interface}) ?
                                 $args{-cmd_interface}  :
                                 $self->interface();
    my $start_time    = int(gettimeofday);
    my $current_time  = int(gettimeofday);

    while (($current_time - $start_time) < $timeout)
    {

        #
        # Get the status of the aggregate
        #
        my @output = $self->get_aggr_status(-name => $name);

        if ($self->error())
        {
            return $ClientUtil::FALSE;
        }

        $self->sanlog(-msg_level => 3,
                      -msg => "SanFiler::wait_for_aggr_online(): "        .
                              "\nCurrent time is " . $current_time        .
                              "\nStart time is   " . $start_time          .
                              "\nOutput is       " . join ("\n", @output) .
                              "\nTimeout is      " . $timeout);
        #
        # If the aggr is online, we are done, return.
        #
        if($cmd_interface eq "cli")
        {
            if ("@output"=~/online/i)
            {
                $self->sanlog(-msg_level => 3,
                              -msg => "SanFiler::wait_for_aggr_online(): " .
                                      "aggr $name is online");
                return $ClientUtil::TRUE;
            }
        }
        elsif($cmd_interface eq "zapi")
        {
            my $out = join('\n', @output);
            my $hash = $self->xml2hash($out);
            my $state = $hash->{"aggregates"}->{"aggr-info"}->{"state"};

            if ($state =~ /online/i)
            {
                $self->sanlog(-msg_level => 3,
                              -msg => "SanFiler::wait_for_aggr_online(): " .
                                      "aggr $name is online");
                return $ClientUtil::TRUE;
            }
        }
        else # it is zapi-cli
        {
            if ("@output"=~/online/i)
            {
                $self->sanlog(-msg_level => 3,
                              -msg => "SanFiler::wait_for_aggr_online(): " .
                                      "aggr $name is online");
                return $ClientUtil::TRUE;
            }
        }

        &SanUtil::nate_sleep($interval);
        $current_time = int(gettimeofday);
    } # end while

    $self->_set_err("Error: SanFiler::wait_for_aggr_online(): " .
                    "Timeout waiting for aggr '$name' to be online");
    return $ClientUtil::FALSE;
} # end wait_for_aggr_online

################################################################################
# Subroutine Name: get_aggr_status()
# Objective:       Gets the status of an aggr from the filer
# Details:         See POD documentation at the end of this file
################################################################################
sub get_aggr_status
{
    my($self, %args) = @_;
    my $cmd;
    my @output;
    my $resultsHash_ref;
    my $cmd_obj       = $self->cmd_obj();    # default to global value
    my $name          = "test_aggr";

    foreach (keys %args)
    {
        if (/-?name/)
        {
            length($name = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::get_aggr_status invalid value " .
                                    "for \"name\":","$args{$_}");
                    return $self;
                };
        }
        else
        {
            $self->_set_err("SanFiler::get_aggr_status Invalid parameter " .
                            "specified: $_");
            return $self;
        }
    }

    #
    # If we were given a cmd interface, use it.  If not use the current
    # value of the SanFiler cmd_interface.
    #
    my $cmd_interface = (defined $args{-cmd_interface}) ?
                              $args{-cmd_interface}  :
                              $self->interface();

    if ($cmd_interface =~ /^cli$/i)
    {
        my %cmd_args = (
                         "-cmd", "aggr-list-info",
                         "-cmd_filer", $self->get_name(),
                         "-check_result", 1,
                         "-cmd_interface", "cli",
                         "-cmd_filer_version", "8.0",
                         "-check_output", 0,
                         "-check_result", 1,
                         "-aggregate", $name
                       );
        #
        # execute the command
        #
        $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

        if ($cmd_obj->error())
        {
            my $errmsg = join "\n", $cmd_obj->errmsg();
            $self->_set_err("ERROR: SanFiler::get_aggr_status(): " .
                            "Failed calling execute_cmd: " . $errmsg);
            $self->log_cmd_results($resultsHash_ref);
            return $self;
        }

        $self->log_cmd_results($resultsHash_ref);
        my $lines = $resultsHash_ref->{output};
        my @output = split("\n", $lines);
        return @output;
    }
    elsif ($cmd_interface =~ /^zapi$/i)
    {
        my %cmd_args = (
                         "-cmd", "aggr-list-info",
                         "-cmd_filer", $self->get_name(),
                         "-check_result", 1,
                         "-cmd_interface", "zapi",
                         "-cmd_filer_version", "8.0",
                         "-check_output", 0,
                         "-check_result", 1,
                         "-aggregate", $name
                       );
        #
        # execute the command
        #
        $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

        if ($cmd_obj->error())
        {
            my $errmsg = join "\n", $cmd_obj->errmsg();
            $self->_set_err("ERROR: SanFiler::get_aggr_status(): " .
                            "Failed calling execute_cmd: " . $errmsg);
            $self->log_cmd_results($resultsHash_ref);
            return $ClientUtil::FALSE;
        }

        $self->log_cmd_results($resultsHash_ref);


        return $resultsHash_ref->{output};
    }
    else  # it is zapi-cli
    {
        my @cmd_args;

        push(@cmd_args, "aggr");
        push(@cmd_args, "status");
        push(@cmd_args, $name);

        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                            -cmd                      => "system-cli",
                            -cmd_filer                => $self->get_name(),
                            -cmd_interface            => "zapi",
                            -check_result             => 1,
                            -args                     => \@cmd_args,
                            -priv                     => "admin");


        if ($cmd_obj->error())
        {
            my $errmsg = join "\n", $cmd_obj->errmsg();
            $self->_set_err("ERROR: SanFiler::get_aggr_status(): " .
                            "Failed calling execute_cmd: $errmsg");
            $self->log_cmd_results($resultsHash_ref);
            return $self;
        }

        $self->log_cmd_results($resultsHash_ref);

        my $lines = $resultsHash_ref->{output};
        my @output = split("\n", $lines);
        return @output;
    }
}

################################################################################
# Subroutine Name: aggr_destroy()
# Objective:       destroy aggregate
# Details:         See POD documentation at the end of this file
###############################################################################
sub aggr_destroy
{
    my ($self, $name, $force)=@_;
    my (
        $cmd,
        @output,
       );

    $self->_clear_err();

    $cmd = "aggr destroy ".$name;
    $cmd .= " -f" if ($force);

    @output=$self->execute($cmd,-hide=>1);
    if ($self->error()){
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

################################################################################
# Subroutine Name: aggr_offline()
# Objective:       Takes a aggregate offline
# Details:         See POD documentation at the end of this file
################################################################################

sub aggr_offline
{
    my ($self, $name)=@_;
    my (
        $cmd,
        @output,
       );

    $self->_clear_err();

    $cmd = "aggr offline ".$name;

    @output=$self->execute($cmd,-hide=>1);
    if ($self->error()){
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

################################################################################
# Method:     get_aggr_volnames
# Objective:  Return list of volumes in the aggregate name provided
# Details:    See POD documentation at the end of this file
################################################################################
sub get_aggr_volnames
{
    my ($self, $aggr_name) = @_;
    my (
        $cmd,
        @output,
        @return,
        $joined_output,
        @split_output,
        $volume_list,
        @vol_names
        );

    $cmd = "aggr status -v $aggr_name";

    @output = $self->execute($cmd, -hide => 1);
    if ($self->error()) {
        $self->_set_err($self->errmsg());
        return (wantarray()) ? () : [];
    }

    # join the array by newlines, then split by two newlines
    # This will group up sections of the output in the same element of the array.
    $joined_output = join("\n", @output);
    @split_output = split(/\n\n/, $joined_output);

    # only grep for the element that contains "Volumes:"
    @split_output = grep(/Volumes:/, @split_output);

    ($volume_list) = @split_output;

    # get rid of spaces and tabs
    $volume_list =~ s/\s+//g;
    $volume_list =~ s/\t+//g;

    # get rid of "Volumes:"
    $volume_list =~ s/Volumes://;

    # if an aggr is empty it will display "Volumes: <none>"
    $volume_list =~ s/\<none\>//;

    # trad vols will display "Volumes: <N/A>"
    $volume_list =~ s/\<N\/A\>//;

    # split on commas
    @vol_names = split(/,/, $volume_list);

    return (wantarray()) ? @vol_names : \@vol_names;
}

###############################################################################
# Method:    debug  (NOT BEING USED ANY MORE)
# Objective: toggle debug state
# Details:   See POD documentation at the end of this file.
###############################################################################
sub debug()
{
    my ($self,$state) = @_;

    unless (defined $state and length $state){
        $self->_set_err("set_debug() debug state (1/0) is missing");
        return;
    }

    print NTEST_STDOUT "DEBUG ON\n" if ($state==1);

    $self->{debug}=$state;

}

################################################################################
# Subroutine Name: display_targets()
# Objective:       Display Target information
# Details:         See POD documentation at the end of this file
################################################################################
sub display_targets
{
    my ($self,$clientObj)=@_;
    my (
        $cnt,
        $name,
        $header,
        @initObjs,
        @output,
        @buffer,
        );

    $self->_clear_err();

    $name=$self->{name};

    $cnt=0;


    unless (defined ref($clientObj) =~ /SanClient/){
        $self->_set_err("Error: SanFiler::display_targets() SanClient2 argument is required. ");
        return (wantarray()) ? () : [];
    }

    @initObjs=$clientObj->get_initiator_wwpns();

    $header =
        sprintf("Target Detail:\n".
                "Filer:  ".$self->get_name()." (cfmode: ".$self->get_cfmode().")\n".
                "Client: ".$clientObj->hostname()."\n".
                "%-4.4s %-8.8s %-16.16s %-16.16s %-8.8s %-10.10s %-8.8s %-16.16s \n".
                "%4.4s %8.8s %16.16s %16.16s %-8.8s %10.10s %-8.8s %-16.16s ",
                "NAME", "TYPE", "NODENAME","PORTNAME","STATUS","VERSION","INITNAME","PORTNAME",
                "="x4,"="x8,"="x16,"="x16,"="x8,"="x10,"="x8,"="x16);

    push @buffer,$header;

    foreach my $targObj ($self->get_targets()){
        my $portname;
        my $initname;

        foreach my $initObj(@initObjs){
            my $wwpn = $initObj->get_portname();

            if (grep(/$wwpn/i, $targObj->get_initiator_wwpns())){
                $portname=$wwpn;
                $initname=$initObj->get_name();
                last;
            }
        }


        my $summary =
            sprintf("%-4.4s %-8.8s %-16.16s %-16.16s %-8.8s %-10.10s %-8.8s %-16.16s",
                    $targObj->get_name(),
                    $targObj->get_type(),
                    $targObj->get_nodename(),
                    $targObj->get_portname(),
                    $targObj->get_status(),
                    $targObj->get_version(),
                    $initname,
                    $portname
                    );

        push @buffer, $summary;
    }


    push @buffer, sprintf("%93.93s","="x93),"\n\n";


    return (wantarray()) ? @buffer : \@buffer;

}

###############################################################################
# Method:    execute
# Objective: Issue arbitrary command to host and get results.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub execute
{
    my($self, $cmd, %args) = @_;
    my(
       @arr,
       $buffer,
       $hide,
       @rows,
       $cmd_fmt,
       $errmsg,
       $t0,
       $t1,
       );

    $hide=0;


    foreach (keys %args){
        if (/-?hide/) {
            length($hide = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::cmd invalid value for \"hide\":",
                                    "$args{$_}");
                    return $self;
                };
        }
    }

    # Clear errors.
    $self->_clear_err;

    unless (length $cmd) {
        $self->_set_err("cmd: invalid CLI command");
        return [];
    }

    # Check for partner mode
    my $partner_mode =  $self->partner_mode();
    if($partner_mode) {
        $cmd = "partner $cmd";
    }

    # Check the vfiler context and prepend the vfiler command if it's
    # set and not vfiler0
    my $vfiler = $self->get_vfiler_context();
    if (length($vfiler) and ($vfiler ne "vfiler0")) {
        $cmd = "vfiler run -q $vfiler $cmd";
    }
    $self->{last_cmd}=$cmd;

    # Print the command unless the user
    # has specified otherwise.
    $cmd_fmt="FILER(".$self->get_name().")> ".$cmd."\n";
    if ((!$hide or $self->{debug})and $self->{show_cmd}){
        $self->sanlog(-msg => $cmd_fmt, -use_loghdl => 0);
    }

    return [] unless ($self->{execute});

    # Remove the carriage returns from
    # the cli command
    $cmd =~ s/[\n]/ /g;

    # Init execution timer
    $t0 = gettimeofday;

    # Issue command and get results.
    (undef,undef,$buffer,undef)
        = Tharn::say($self->{connection},$cmd,-timeout=>$self->{timeout},
                     -eval=>sub{
                         if (@_) {
                             my $errmsg = join("",@_);
                             $errmsg =~ s/\n//g;
                             print NTEST_STDOUT "$errmsg\n";
                             Tharn::logresult("CONFIG",abort=>0,"\n@_");
                         }
                     });



    $t1=gettimeofday;
    $self->{command_time}=$t1-$t0;

    # Split the buffer into array
    #@arr=split(/\cM\cM\cJ|\cM\cJ/,$buffer);

    @arr=split(/\n/,$buffer);

    # Remove command and trailing blank line results array
    @rows=(@arr[0..scalar(@arr)-1]);

    return (wantarray()) ? @rows : \@rows;


} # end sub cmd

###############################################################################
# Method:    fcp_start
# Objective: Start fcp service
# Details:   See POD documentation at the end of this file.
###############################################################################
sub fcp_start
{
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $interface = $self->interface();
    my $output_hash;

    $self->_clear_err();
    # Build command arguments.
    my %cmd_args = ("-cmd"            => "fcp-service-start",
                    "-cmd_filer"      => $self->get_name(),
                    "-cmd_interface"  => $interface,
                    "-check_result"   => 1);

    # Execute command.
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if($cmd_obj->error()) {
        $self->_set_err("Unable to start fcp service: ", $cmd_obj->errmsg());
        return $SanUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi") {
        $output_hash = $self->xml2hash($output);
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless($resultsHash_ref->{result_check}) {
        if($interface eq "zapi") {
            $self->_set_err("Result check failed:\n",
                            $output_hash->{reason});
            return $SanUtil::FALSE;
        } else {
           $self->_set_err("Result check failed:\n$output");
           return $SanUtil::FALSE;
        }
    }

    return $SanUtil::TRUE;
} # end fcp_start()

###############################################################################
# Method:    fcp_stop
# Objective: Stop fcp service
# Details:   See POD documentation at the end of this file.
###############################################################################
sub fcp_stop
{
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $interface = $self->interface();
    my $output_hash;

    $self->_clear_err();
    # Build command arguments.
    my %cmd_args = ("-cmd"            => "fcp-service-stop",
                    "-cmd_filer"      => $self->get_name(),
                    "-cmd_interface"  => $interface,
                    "-check_result"   => 1);

    # Execute command.
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if($cmd_obj->error()) {
        $self->_set_err("Unable to start fcp service: ", $cmd_obj->errmsg());
        return $SanUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi") {
        $output_hash = $self->xml2hash($output);
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless($resultsHash_ref->{result_check}) {
        if($interface eq "zapi") {
            $self->_set_err("Result check failed:\n",
                            $output_hash->{reason});
            return $SanUtil::FALSE;
        } else {
           $self->_set_err("Result check failed:\n$output");
           return $SanUtil::FALSE;
        }
    }

    return $SanUtil::TRUE;
} # end fcp_stop()

###############################################################################
# Method:    get_exec_time()
# Objective: Get execution time of command execution
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_exec_time
{
    my($self) = @_;

    if (defined $self->{command_time} and length $self->{command_time}){
        return sprintf "%.4f",$self->{command_time}; }

    return "";
}

###############################################################################
# Method:    get_fcp_initiators()
# Objective: get initiators
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_fcp_initiators
{
    my ($self,%args) = @_;
    my (
        @init,
        @output,
        $cmd,
        $errmsg,
        $init_name,
        );

    $self->_clear_err();

    Tharn::logcomment("Get fcp initiators ");

    foreach my $param (keys %args) {
        if ($param =~ /^-?init/) {
            $init_name = $args{$param};
            unless ($init_name) {
                $self->_set_err( "Invalid value passed for the '-init'"
                                . "argument: $args{$param}");
                return $self;
            }
            next;
        }else{
            $self->_set_err( "Invalid argument: $param");
            return $self;
        }
    }

    $cmd="fcp show initiators";
    $cmd = $cmd." ".$init_name if($init_name);

    @output=$self->execute($cmd,-hide=>1);
    if (scalar(@output) <= 1){
        ($errmsg)=$output[0]=~/\:(.*)/;
        $self->_set_err("Error: SanFiler::get_fcp_initiators():",
                        "Unable to get fcp initiator information");
        return (wantarray()) ? () : [];
    }

    # Eg: O/p
    # rtpqa-fas6040-11> fcp show initiators vtic
    # Initiators connected on adapter vtic:
    # Portname                  Group
    # --------                  -----
    # 10:00:00:00:c9:67:29:53   nate_rtpqa-sun280r-73
    # 10:00:00:00:c9:67:29:52   nate_rtpqa-sun280r-73

    if($init_name){
        foreach my $line (@output){
            next if ($line !~ /^(\d+):/);
            my($init) = $line =~/^(\S+)\s+(\S+)/;
            # Get the initiator
            $init =~ s/\://g;
            push(@init, $init);
        }
    }else{
        foreach my $line (@output){
            next if ($line !~ /adapter/i);
            my ($init)=$line=~/adapter\s+(\S+):/;
            push @init,$init;
        }
    }
    return (wantarray()) ? @init : \@init;
}

###############################################################################
# Method:    fcp_adapters_list_info()
# Objective: get "fcp show adapter" output
# Details:   See POD documentation at the end of this file.
###############################################################################
sub fcp_adapters_list_info {
    my ($self, %args) = @_;
    my (@init,        @output,  $cmd,        $errmsg,
        $adapter,     $verbose, %hsh,        $name,
        $description, $version, $adaptLabel, $cfmode);
    $self->_clear_err();
    # Set verbose 'off' as a default
    $verbose = 'off';
    # parse arguments
    foreach (keys %args) {
        if (/-?adapter/) {
            if (not length($adapter = $args{$_})) {
                $self->_set_err("Error: SanFiler::fcp_adapters_list_info()",
                                "Invalid value for argument '-adapter'");
                return $self;
            }
            next;
        } elsif (/-?verbose/) {
            if (not length($verbose = $args{$_})
                and $args{$_} !~ /1|0|t|f|on|off/i) {
                $self->_set_err("Error: SanFiler::fcp_adapters_list_info()",
                                "Invalid value for argument '-verbose'");
                return $self;
            }
            next;
        } else {
            $self->_set_err("Error: SanFiler::fcp_adapters_list_info()",
                            "Invalid parameter specified \'$_\'");
            return $self;
        }
    }
    Tharn::logcomment("Get fcp adapters info");
    my $cmd_obj   = $self->cmd_obj();
    my $interface = $self->interface();
    my $resultsHash_ref;
    my %cmd_args = ("-cmd"           => "fcp-adapter-list-info",
                    "-cmd_interface" => $interface,
                    "-check_result"  => 1,
                    "-check_output"  => 1,);
    # Depending upon the option that we got send verbose option
    if ($verbose =~ /1|t|on/) {
        if ($interface eq 'cli') {
            $cmd_args{"-verbose"} = "";
        } else {
            $cmd_args{"-verbose"} = "true";
        }
    }
    # Depending upon the option that we got send fcp-adapter option
    if ($adapter) {
        $cmd_args{"-fcp-adapter"} = $adapter;
    }
    # Execute command
    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    if ($cmd_obj->error()) {
        $self->_set_err("Error: SanFiler::fcp_adapters_list_info() ",
                        $cmd_obj->errmsg());
        return $SanUtil::FALSE;
    }
    #
    # Since we checked the result, lets see if our check was OK
    #
    unless (   $resultsHash_ref->{result_check}
            && $resultsHash_ref->{output_check}) {
        return $SanUtil::FALSE;
    }
    my $output = $resultsHash_ref->{output};
    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
        # We get different ouput depending upon fcp-adapter option
        # If we does not send fcp-adapter option we get
        # $VAR1 = {
        #  'fcp-config-adapters' => {
        #                           'fcp-config-adapter-info' => [
        #                                                        {
        #                                                          'port-name' => '50:0a:09:81:83:e1:53:26',
        #                                                          'fabric-established' => 'true',
        #                                                          'cache-line-size' => '8',
        #                                                          'external-gbic-enabled' => 'false',
        #                                                          'info-name' => 'Dual-channel, QLogic 2312 (2352)',
        #                                                          'state' => 'online',
        #                                                          ...
        #                                                        },
        #                                                        ...
        #                                                      ]
        #                         },
        #  'status' => 'passed'
        # };
        #
        # If we send fcp-adapter option we get
        # $VAR1 = {
        #  'fcp-config-adapters' => {
        #                           'fcp-config-adapter-info' => {
        #                                                        'port-name' => '50:0a:09:83:83:e1:53:26',
        #                                                        'fabric-established' => 'true',
        #                                                        'cache-line-size' => '8',
        #                                                        'external-gbic-enabled' => 'false',
        #                                                        'info-name' => 'Dual-channel, QLogic 2312 (2352)',
        #                                                        'state' => 'online',
        #                                                        ...
        #                                                      }
        #                         },
        #  'status' => 'passed'
        # };
        # If fcp-adapter option was present, take the fields for that particular adapter
        if ($adapter) {
            my $slot =
              $output_hash->{'fcp-config-adapters'}
              ->{'fcp-config-adapter-info'};
            my @fields = keys %{$slot};
            foreach my $field (@fields) {
                # Create hash, hsh{adapter name}{field name} = field value
                $hsh{$slot->{adapter}}{$field} = $slot->{$field};
            }
        } else {
            # If fcp-adapter option was not present, take all the fields for all the adapters
            my @slots =
              @{$output_hash->{'fcp-config-adapters'}
                  ->{'fcp-config-adapter-info'}};
            my $i = 0;
            foreach (@slots) {
                my @fields = keys %{$slots[$i]};
                foreach my $field (@fields) {
                    # Create hash, hsh{adapter name}{field name} = field value
                    $hsh{$slots[$i]->{adapter}}{$field} = $slots[$i]->{$field};
                }
                $i++;
            }
        }
    } else {
        my @output = split("\n", $output);
        $self->sanlog(-msg       => "Output is:\n@output",
                      -msg_level => 3);
        # Determine fcp cfmode.  The 'fcp adapter' info
        # for adapter label is different.
        $cfmode = $self->get_cfmode();
        if ($cfmode =~ /partner|cluster|Not Clustered|standby|single_image/i) {
            $adaptLabel = "Slot";
        } elsif ($cfmode =~ /mixed|dual/i) {
            $adaptLabel = "Name";
        } else {
            $self->_set_err(
                "Error: SanFiler::fcp_adapters_list_info() The cfmode '$cfmode' in currently",
                "not supported by this library");
            return $ClientUtil::FALSE;
        }
        # Retrieve all fields from output and store in hash
        foreach my $line (@output) {
            # Skip the line if it is not in 'field: value' format like
            # Slot:                    9a
            # Store the field and it's value if found such a line
            next unless (my ($key, $value) = $line =~ /(.*)?:\s+(.*)/i);
            if ($key =~ /^$adaptLabel$/) {
                $name = $value;
                # In mixed mode 'description' and 'firmware rev' only
                # appear under the adapter id.  Use the last values
                # set for these attributes.
                $hsh{$name}{Description} = $description
                  if (defined $description and length $description);
                $hsh{$name}{'Firmware Rev'} = $version
                  if (defined $version and length $version);
                # $self->sanlog(-msg => "REV: ".$hsh{$name}{Rev});
            }
            if ($key =~ /Description/i) {
                $description = $value;
            } elsif ($key =~ /Firmware Rev/) {
                $version = $value;
            }
            # Create hash, hsh{adapter name}{field name} = field value
            if (defined $name and length $name) {
                $hsh{$name}{$key} = $value;
            }
        }
    }
    return %hsh;
}

################################################################################
# Subroutine Name: create_igroup
# Objective:       create and return a igroup object
# Details:         See POD documentation at the end of this file
################################################################################
sub create_igroup
{
    my ($self,%args) = @_;
    my (
        $igrpObj,
        %hArgs,
        );

    $self->_clear_err();

    # Create and return igrp object
    $hArgs{-loghdl}=$self->{log} if (defined $self->{log});
    $igrpObj = IGroup->_new(
                            -parent=>$self,
                            %args,
                            %hArgs,
                           );

    if ($igrpObj->error){
        $self->_set_err( join("\n", $igrpObj->errmsg()) );
        return $self;
    }

    return $igrpObj;
}

################################################################################
# Subroutine Name: destroy_igroup
# Objective:       destroy a named igroup object
# Details:         See POD documentation at the end of this file
################################################################################
sub destroy_igroup
{
    my ($self,$name,%args) = @_;

    $self->_clear_err();

    unless(defined $name and length $name) {
        $self->_set_err("igroup name is a required argument");
        return $ClientUtil::FALSE
    }

    my $igroupObj = $self->get_igroup(-name => $name);

    if ($self->error()) {
        return $ClientUtil::FALSE;
    }

    $igroupObj->destroy(%args);
    if ($igroupObj->error) {
        $self->_set_err($igroupObj->errmsg());
        return $ClientUtil::FALSE
    }


    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    create_vol()
# Objective: Create and return a vol object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub create_vol
{
    my($self,%args) = @_;
    my (
        $aggr,
        $disk_cnt,
        $volobj,
        $filer,
        $name,
        $devices,
        $type,
        $size,
        $space_reserve,
        %hArgs,
        );

    $self->_clear_err();

    # parse arguments
    foreach (keys %args) {
        if (/-?name/) {
            if (not length($name = $args{$_})) {
                $self->_set_err("Error: SanFiler::create_vol()",
                                "Invalid value for argument '-name'");
                return $self;
            }
            next;
        }
        elsif (/-?aggr/) {
            if (not length($aggr = $args{$_})) {
                $self->_set_err("Error: SanFiler::create_vol()",
                                "Invalid value for argument '-aggr'");
                return $self;
            }
            next;
        }
        elsif (/-?disk_cnt/){
            if (not length($disk_cnt = $args{$_}) or $args{$_}<2 )    {
                $self->_set_err("Error: SanFiler::create_vol()",
                                "Invalid value for argument '-disk_cnt'");
                return $self;
            }
            next;
        }
        elsif (/-?size/){
            if (not length($size = $args{$_}))    {
                $self->_set_err("Error: SanFiler::create_vol()",
                                "Invalid value for argument '-size'");
                return $self;
            }
            next;
        }
        elsif (/-?space-reserve/){
            if (not length($space_reserve = $args{$_}))    {
                $self->_set_err("Error: SanFiler::create_vol()",
                                "Invalid value for argument '-space-reserve'");
                return $self;
            }
            next;
        }
        else {
            $self->_set_err("Error: SanFiler::create_vol()",
                            "Invalid parameter specified \'$_\'");
            return $self;
        }
    }

    # check for valid path value
    unless (defined $name) {
        $self->_set_err("Error: SanFiler::create_vol()",
                        "Reguired value missing for parameter '-name'");
        return $self;
    }

    if ((defined $aggr and defined $disk_cnt) or
        (not defined $aggr and not defined $disk_cnt)){
        $self->_set_err("Error: SanFiler::create_vol()",
                        "Invalid option.  Either '-aggr' or '-disk_cnt' must be specified");
        return $self;
    }

    if (defined $aggr) {
        unless (defined $size) {
            $self->_set_err("Error: SanFiler::create_vol()",
                            "Reguired value missing for parameter '-size'");
            return $self;
        }
    }

    if (defined $space_reserve) {
        unless ($space_reserve =~ /(none|volume|file)/i) {
            $self->_set_err("Error: SanFiler::create_vol()",
                            "-space-reserve must be one of volume, file, or none");
            return $self;
        }
    }

    # Create and return vol object
    $hArgs{-loghdl}=$self->{log} if (defined $self->{log});
    $volobj = Volume->_new(%args,-parent=>$self,-create=>1,%hArgs);
    if ($volobj->error){
        $self->_set_err($volobj->errmsg());
        return $self;
    }


    return $volobj;
}


###############################################################################
# Method:    get_version_major()
# Objective: Get filer's major version number (i.e. 7.0, 7.1, 8.0)
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_version_major
{
    my ($self) = @_;
    my (
        $tharnhost,
        $version,
        );

    $self->_clear_err();

    my $cache_item = "get_ontap_version";

    # get host object
    $tharnhost = Tharn::host($self->get_name());
    eval {
        # check if the ontap wafl version is stored in the cache
        if ($self->is_cache_data_valid($cache_item)) {
            ($version) = $self->get_cache_data($cache_item);
        } else {
            $version = (get_ontap_version($tharnhost));
        }
    };
    unless (defined $version) {
        $self->_set_err("Error: SanFiler::get_version_major(): Unable to get filer major version (make sure ZAPIs are enabled - options httpd.admin.enable)\n$@");
        return "";
    }

    Tharn::logcomment("Filer major version is $version");

    # store this in the cache
    $self->store_cache_data($cache_item, $version);

    return $version;

}

###############################################################################
# Method:    get_version()
# Objective: Get filer version
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_version
{
    my ($self) = @_;
    my (
        @output,
        $cmd,
        $version,
        );

    $self->_clear_err();

    $cmd="version";

    @output=$self->execute($cmd,-hide=>1);

    unless (defined $output[0] and length $output[0]){
        $self->_set_err("Error: SanFiler::get_version(): Unable to get filer version");
        return "";
    }


    ($version)=$output[0]=~ /Release\s(.*)\:\s+/i;

    Tharn::logcomment("Filer version string is $version");

    return $version;

}

###############################################################################
# Method:    get_version_string()
# Objective: Get filer version string
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_version_string
{
    my ($self) = @_;
    my $resultsHash_ref;
    my $cmd_obj       = $self->cmd_obj();    # default to global value

    $self->_clear_err();

    if($self->interface() eq "cli") {
        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                        -cmd                    => "system-get-version",
                        -cmd_filer              => $self->{filer},
                        -check_result           => 1);
    } else {
        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                            -cmd                      => "system-get-version",
                            -cmd_filer                => $self->get_name(),
                            -cmd_interface            => "zapi",
                            -check_result             => 1,
                            -priv                     => "admin");
    }

    if($cmd_obj->error()) {
        $self->_set_err("Unable to get filer version: ", $cmd_obj->errmsg());
        return "";
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};
    my @output = join('\n', $output);

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check}) {
        return $output;
    } else {
        $self->_set_err("Result check failed: ", $output);
        return "";
    }
}

###############################################################################
# Method:    is_debug_version()
# Objective: Get filer version
# Details:   See POD documentation at the end of this file.
###############################################################################
sub is_debug_version
{
    my ($self) = @_;
    my $version_string = $self->get_version_string();

    if($self->error()) {
        $self->_set_err("Unable to get filer version string: ",
                        $self->errmsg());
        return "";
    }

    ## We determine whether or not a build is a debug version
    ## based on the presence of "<d>" in the version string"
    ## To support the debug version of gcov build(1dG) pattern matching is added
    if(($version_string =~ m/\<d\>|\<(\d)d\>/i)|| ($version_string =~ m/\<1dG\>|\<(\1dG)1dG\>/i)){
        return $SanUtil::TRUE;
    } else {
        return $SanUtil::FALSE;
    }
}

###############################################################################
# Method:    get_uptime()
# Objective: Get filer uptime
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_uptime
{
    my ($self) = @_;
    my $resultsHash_ref;
    my $cmd_obj       = $self->cmd_obj();    # default to global value

    $self->_clear_err();

    if($self->interface() eq "cli") {
        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                        -cmd                    => "system-get-uptime",
                        -cmd_filer              => $self->{filer},
                        -check_result           => 1);
    } else {
        my @cmd_args;
        push(@cmd_args, "uptime");

        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                            -cmd                      => "system-cli",
                            -cmd_filer                => $self->get_name(),
                            -cmd_interface            => "zapi",
                            -check_result             => 1,
                            -args                     => \@cmd_args,
                            -priv                     => "admin");
    }

    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to get uptime: $errmsg");
        return "";
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};
    my @output = join('\n', $output);

    $self->sanlog(-msg => "output from uptime is $output");

    if($output=~/^uptime:.*/i) {
        $self->_set_err("Unable to get uptime $output");
        return "";
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check}) {
        return (wantarray()) ? @output : \@output;
    } else {
        $self->_set_err("Result check failed");
        return "";
    }

}


################################################################################
# Subroutine Name: get_target
# Objective:       return a target object by name
# Details:         See POD documentation at the end of this file
################################################################################

sub get_target
{
    my ($self,$name) = @_;


    $self->_clear_err();

    unless(defined $name and length $name){
        $self->_set_err("Error: SanFiler::get_target() target name is a required argument");
        return $ClientUtil::FALSE;
    }

    foreach my $obj ( @{$self->{targets}}){
        if ($obj->get_name() =~ /^$name$/){
            return $obj;
        }
    }

    $self->_set_err("Error: SanFiler::get_target() Unable to find target '$name'");

    return $ClientUtil::FALSE;
}



################################################################################
# Subroutine Name: get_targets
# Objective:       return array of target objects
# Details:         See POD documentation at the end of this file
################################################################################
sub get_targets
{
    my ($self,%args) = @_;
    my (
        @targetObjs,
        $dmp,
        $initObjs,
        );

    $self->_clear_err();

    # Default value
    $dmp=1;
    $initObjs=[];

    # parse arguments
    foreach (keys %args) {
        if (/-?dmp/) {
            if (not length($dmp = $args{$_}) and $args{$_} !~ /1|0|t|f|on|off/i) {
                $self->_set_err("Error: SanFiler::get_targets()",
                                "Invalid value for argument '-dmp'");
                return $self;
            }
            next;
        }
        else {
            $self->_set_err("Error: SanFiler::get_targets()",
                            "Invalid parameter specified \'$_\'");
            return $self;
        }

    }

    my $hsh={};

    if (defined $self->{targets}) {
        @targetObjs = @{$self->{targets}};

    }
    else {
        $self->_set_err("Error: SanFiler::get_targets() SanFilers object has no targets");
        return ();
    }

    if (scalar(@$initObjs) gt 0){
        foreach my $targObj (@targetObjs){
            $self->sanlog(-msg => "Target Name: ".$targObj->get_name().
                                  " Initiators=@{$targObj->get_initiator_wwpns()}");
            foreach my $initObj(@$initObjs){
                $self->sanlog(-msg => "Initiator Name: ".$initObj->get_name().
                                      " wwpn=".$initObj->get_portname(),"\n");
            }
        }
    }


    if ($dmp=~/0|f|n|off|f/i){
        my @noPartner;
        foreach my $targObj (@targetObjs){
            next if ($targObj->{type}=~/partner/i);
            push @noPartner, $targObj;
        }

        @targetObjs=@noPartner;
    }


    return (wantarray()) ? @targetObjs : \@targetObjs;

}

################################################################################
# Subroutine Name: get_iscsi_target
# Objective:       return a iscsi target object by name
# Details:         See POD documentation at the end of this file
################################################################################
sub get_iscsi_target {
    my ($self, $name) = @_;

    $self->_clear_err();

    unless(defined $name and length $name) {
        $self->_set_err("Error: SanFiler::get_iscsi_target() target name is a required argument");
        return {};
    }

    if (defined($self->{iscsi_targets})) {
        foreach my $obj (@{$self->{iscsi_targets}}) {
            if ($obj->get_name() =~ /^$name$/){
                return $obj;
            }
        }
    } else {
        return {};
    }

    $self->_set_err("Error: SanFiler::get_iscsi_target() Unable to find target '$name'");
    return {};

}



################################################################################
# Subroutine Name: get_iscsi_targets
# Objective:       return array of iscsi target objects
# Details:         See POD documentation at the end of this file
################################################################################
sub get_iscsi_targets {
    my ($self, %args) = @_;

    my (@targetObjs, @filteredObjs, @added, $state, $valid_ip);
    my ($filer_accesslist_inclusion, $filer_accesslist_exclusion);

    $self->_clear_err();

    foreach (keys %args) {
        if (/-?state/) {
            length($state = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::get_iscsi_targets invalid value for \"-state\":",
                                    "$args{$_}");
                    return (wantarray()) ? () : [];
                }
        } elsif (/-?valid_ip/) {
            length($valid_ip = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::get_iscsi_targets invalid value for \"-valid_ip\":",
                                    "$args{$_}");
                    return (wantarray()) ? () : [];
                }
        } elsif (/-?filer_accesslist_inclusion/) {
            $filer_accesslist_inclusion = $args{$_};
        } elsif (/-?filer_accesslist_exclusion/) {
            $filer_accesslist_exclusion = $args{$_};
        }
    }

    # filer_accesslist_inclusion and filer_accesslist_exclusion are mutually exclusive lists. If both are set
    # then we dont know which list to honour. So we fail if both are set
    if ($filer_accesslist_inclusion ne "" and $filer_accesslist_exclusion ne "") {
        $self->_set_err("Both filer_accesslist_inclusion ($filer_accesslist_inclusion) and " .
                        "filer_accesslist_exclusion ($filer_accesslist_exclusion) are set. Please make sure only one of " .
                        "the arguments to the function are set as both are mutually exclusive");
        return (wantarray()) ? () : [];
    }

    if (defined $self->{iscsi_targets}) {
        if ($filer_accesslist_inclusion ne "") {
            foreach my $iscsi_target (@{$self->{iscsi_targets}}) {
                my $iscsi_target_name = $iscsi_target->get_name();
                if (grep(/$iscsi_target_name/, $filer_accesslist_inclusion)) {
                    if ($iscsi_target->{state} eq "disabled") {
                        $self->_set_err("Target interface ($iscsi_target_name) " .
                                        "is specified in the filer_accesslist_inclusion list but iSCSI is disabled " .
                                        "on the interface (iscsi interface disabled). Please either remove $iscsi_target_name " .
                                        "interface from param FILER_ACCESSLIST_INCLUSION or enable iscsi on it");
                        return (wantarray()) ? () : [];
                    }

                    push (@targetObjs, $iscsi_target);
                }
            }
        } elsif ($filer_accesslist_exclusion ne "") {
            foreach my $iscsi_target (@{$self->{iscsi_targets}}) {
                my $iscsi_target_name = $iscsi_target->get_name();
                next if (grep(/^$iscsi_target_name$/, $filer_accesslist_exclusion));
                push (@targetObjs, $iscsi_target);
            }
        } else {
            @targetObjs = @{$self->{iscsi_targets}};
        }

        # check if both filters are specified
        if (defined($valid_ip) and defined($state)) {
            foreach my $obj (@targetObjs) {
                my $ip = $obj->get_ip();
                if ($ip !~ /^\s*$/i and $obj->get_state() =~ /$state/) {
                    push(@filteredObjs, $obj);
                }
            }
            return (wantarray()) ? @filteredObjs : \@filteredObjs;
        }

        # filter out interfaces w/o IP set
        if (defined($valid_ip)) {
            foreach my $obj (@targetObjs) {
                my $ip = $obj->get_ip();
                if ($ip !~ /^\s*$/i) {
                    push(@filteredObjs, $obj);
                    # generate array to make sure we
                    # do not push a target twice
                    push(@added, $ip);
                }
            }
            # reset return based on filter
            @targetObjs = @filteredObjs;
        }

        # filter by state
        if (defined($state)) {
            foreach my $obj (@targetObjs) {
                my $ip = $obj->get_ip();
                if ($obj->get_state() =~ /$state/) {
                    # avoid pushing target twice into arrary
                    if (grep(/$ip/, @added) == 0) {
                        push(@filteredObjs, $obj);
                    }
                }
            }
            # reset return based on filter
            @targetObjs = @filteredObjs;
        }
    } else {
        $self->_set_err("Error: SanFiler::get_iscsi_targets() SanFilers object has no targets");
        return (wantarray()) ? () : [];
    }

    return (wantarray()) ? @targetObjs : \@targetObjs;

}

###############################################################################
# Method:    get_iscsi_alias()
# Objective: Get filer iscsi alias
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_iscsi_alias
{
    my ($self) = @_;
    return $self->{iscsi_alias};
}

###############################################################################
# Method:    set_iscsi_alias()
# Objective: Set the iscsi alias
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_iscsi_alias
{
    my ($self, %args) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    my ($cmd, $lines, $resultsHash_ref, $new_alias);
    my @output;

    # Default value
    $new_alias=undef;

    # parse arguments
    foreach (keys %args) {
        if (/-?name/) {
            if (not length($new_alias = $args{$_})) {
                $self->_set_err("Error: SanFiler::set_iscsi_alias()",
                                "Invalid value for argument '-name'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Error: SanFiler::set_iscsi_alias()",
                            "Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }

    }

    my %cmd_args = (
                     "-cmd" => "iscsi-target-alias-set-alias",
                     "-cmd_filer" => $self->get_name(),
                     "-check_result" => 1,
                     "-check_output" => 0,
                     "-cmd_interface" => $self->interface(),
                     "-alias-name" => $new_alias,
                   );

    #
    # execute the command
    #
    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    $self->log_cmd_results($resultsHash_ref);
    if ($cmd_obj->error()) {
        $self->_set_err("ERROR: SanFiler::set_iscsi_alias(): " .
                        "Failed calling execute_cmd", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Error: SanFiler::set_iscsi_alias(): " .
                        "Unable to set iscsi alias ", $output[0]);
        return $ClientUtil::FALSE;
    }
    $self->{iscsi_alias} = $new_alias;
    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    clear_iscsi_alias()
# Objective: Clear iscsi alias
# Details:   See POD documentation at the end of this file.
###############################################################################
sub clear_iscsi_alias
{
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    my ($cmd, $lines, $resultsHash_ref);
    my @output;
    my %cmd_args = (
                     "-cmd" => "iscsi-target-alias-clear-alias",
                     "-cmd_filer" => $self->get_name(),
                     "-check_result" => 1,
                     "-check_output" => 0,
                     "-cmd_interface" => $self->interface(),
                   );

    #
    # execute the command
    #
    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    $self->log_cmd_results($resultsHash_ref);
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("ERROR: SanFiler::clear_iscsi_alias(): " .
                        "Failed calling execute_cmd", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Error: SanFiler::clear_iscsi_alias(): " .
                        "Unable to clear iscsi alias ", $output[0]);
        return $ClientUtil::FALSE;
    }
    $self->{iscsi_alias} = undef;
    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    reset_igroup_throttle_all()
# Objective: Will reset igroup throttle value to ZERO for all igroups created
#            by NATE if its value is non-ZERO
# Details:   See POD documentation at the end of this file.
###############################################################################
sub reset_igroup_throttle_all {
    my ($self, %args) = @_;

    my ($match_name);
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();
    foreach (keys %args) {
        if (/-?match_name/) {
            if (not length($match_name = $args{$_})) {
                $self->_set_err("Invalid value for -match_name: $args{$_}");
                return $ClientUtil::FALSE;
            }
            next;
        }
    }

    if (not defined($match_name)) {
        # if no matching pattern is passed then igroup
        # throttle will be set to ZERO on all igroups
        $match_name = "";
    }

    my @igroup_objs = $self->get_igroups();
    if ($self->error) {
        return $ClientUtil::FALSE;
    }

    # Reset igroup throttle to ZERO for each igroup whose name
    # matches with the string $match_name
    foreach my $igroup_obj (@igroup_objs) {
        my $igroup_name = $igroup_obj->get_name();
        if ($igroup_name =~ /$match_name/) {
            $self->set_igroup_throttle(-igroup_name         => $igroup_name,
                                       -disable_borrow      => 0,
                                       -value               => 0);
            if ($self->error) {
                return $ClientUtil::FALSE;
            }
        }
    }

    # Just print igroup throttle info to the logs for debug purposes
    $self->display_igroup_throttles();
    if ($self->error) {
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    display_igroup_throttles()
# Objective: Will display igroup throttle settings
# Details:   See POD documentation at the end of this file.
###############################################################################
sub display_igroup_throttles {
    my ($self, %args) = @_;

    my ($igroup_name, $value, $disable_borrow, $cmd, @output);
    $cmd = "igroup show -t";
    @output = $self->execute($cmd);
    if ($self->error) {
        $self->_set_err("Failed to execute command ($cmd)");
        return (wantarray()) ? () : [];
    }
    return (wantarray()) ? @output : \@output;
}

###############################################################################
# Method:    set_igroup_throttle()
# Objective: Will set igroup throttle value to $args{-value} for the passed
#            igroup. Will also set throttle_borrow to no for the igroup
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_igroup_throttle {
    my ($self, %args) = @_;

    my ($igroup_name, $value, $disable_borrow);

    $self->_clear_err();
    foreach (keys %args) {
        if (/-?igroup_name/) {
            if (not length($igroup_name = $args{$_})) {
                $self->_set_err("Invalid value for -igroup_name: $args{$_}");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?disable_borrow/) {
            if (not length($disable_borrow = $args{$_})) {
                $self->_set_err("Invalid value for -value: $args{$_}");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?value/) {
            if (not length($value = $args{$_})) {
                $self->_set_err("Invalid value for -value: $args{$_}");
                return $ClientUtil::FALSE;
            }
            next;
        }
    }

    # If not passed then make sure that borrow is disabled
    if (not defined($disable_borrow)) {
        $disable_borrow = 1;
    }

    if (not defined($value)) {
        $self->_set_err("-value is a required parameter");
        return $ClientUtil::FALSE;
    }

    my $igrp_obj = $self->get_igroup(-name => $igroup_name);
    if ($self->error) {
        $self->_set_err("Unable to get igroup obj for igroup $igroup_name");
        return $ClientUtil::FALSE;
    }

    $igrp_obj->set(-attr    => "throttle_reserve",
                   -value   => $value);
    if ($igrp_obj->error) {
        $self->_set_err($igrp_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    # No need to set throttle_borrow to no if
    # we set igroup throttle value to ZERO
    if ($value != 0 and $disable_borrow == 1) {
        $igrp_obj->set(-attr    => "throttle_borrow",
                       -value   => "no");
        if ($igrp_obj->error) {
            $self->_set_err($igrp_obj->errmsg());
            return $ClientUtil::FALSE;
        }
    }

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    check_for_misaligned_lun_access()
# Objective: Will check whether the luns are misaligned or not
# Details:   See POD documentation at the end of this file.
###############################################################################
sub check_for_misaligned_lun_access {
    my ($self, %args) = @_;

    my ($data_gather_interval, $lun_misalignment_threshold, @misaligned_accessed_luns);
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();
    foreach (keys %args) {
        if (/-?lun_misalignment_threshold/) {
            if (not length($lun_misalignment_threshold = $args{$_})) {
                $self->_set_err("Invalid value for -lun_misalignment_threshold: $args{$_}");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?data_gather_interval/) {
            if (not length($data_gather_interval = $args{$_})) {
                $self->_set_err("Invalid value for -data_gather_interval: $args{$_}");
                return $ClientUtil::FALSE;
            }
            next;
        }
    }

    my (@counters);
    $counters[0] = "read_align_histo";
    push (@counters, "write_align_histo");
    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "perf-object-get-instances",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -priv                   => "diag",
                        -cmd_interface          => "zapi",
                        -cmd_filer_version      => "8.0",
                        -objectname             => "lun",
                        -counters               => \@counters,
                        -check_output           => 0,
                        -check_result           => 1);

    if ($cmd_obj->error()) {
        my $errmsg = $cmd_obj->errmsg();
        $self->_set_err("Unable to get LUN misaligned histogram: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # Get the output
    my $output = $resultsHash_ref->{output};

    my $output_hash_1 = $self->xml2hash($output);

    # Running the command twice and waiting for $data_gather_interval seconds in between
    # because we want to know how much align/unaligned data is written in 60(default) seconds
    &SanUtil::nate_sleep($data_gather_interval);

    # Documentation link for perf related Zapi commands
    # http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/RironcityN/web/perf/index.html
    $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "perf-object-get-instances",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -priv                   => "diag",
                        -cmd_interface          => "zapi",
                        -cmd_filer_version      => "8.0",
                        -objectname             => "lun",
                        -counters               => \@counters,
                        -check_output           => 0,
                        -check_result           => 1);

    if ($cmd_obj->error()) {
        my $errmsg = $cmd_obj->errmsg();
        $self->_set_err("Unable to get LUN misaligned histogram: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # Get the output
    $output = $resultsHash_ref->{output};

    my $output_hash_2 = $self->xml2hash($output);

    # There are 8 buckets which represent all the IO(reads/writes) on a particular LUN. Zeroth Bucket represents
    # aligned IO and rest of the buckets (from 1st to 7th) represent unaligned IO. In algorithm below we
    # are calculating unaligned IO as a percentage of total IO (aligned + unaligned). Interval is 60 seconds

    my $misaligned_data;
    my $single_lun = 0;

    # The output for the perf-object-get-instances changed for RR.0. Instead
    # of returning a hash keyed by "instance-data", they return a hash
    # keyed by "instance-output".  We will check to see which is defined
    # and use the appropriate key.
    my $data_key = "";
    if (defined($output_hash_1->{instances}->{"instance-data"})) {
        $data_key = "instance-data";
    } elsif (defined($output_hash_1->{instances}->{"instance-output"})) {
        $data_key = "instance-output";
    } else {
        $self->_set_err("Unexpected output retured from "
                      . "the perf-objects-get-instances ZAPI.  We expected "
                      . "that either \$hash->{instances}->{instance-data} or "
                      . "\$hash->{instances}->{instance-data} would be defined.");
        return $ClientUtil::FALSE;
    }

    # Get the amount of misaligned data read in last 1 min
    foreach my $key (keys %{$output_hash_1->{instances}->{"$data_key"}}) {
        my ($before, $after);
        # Hash created by xml2hash from xml file is different if there is only one element in the xml file
        # than hash created from xml file which has more than one element. For example, if there is only
        # one LUN on the filer the hash dump will be created like this:
        # (Note, the example below uses 'instance-data' for RR.0 and newer releases that field
        #  will be 'instance-output', see the value of $data_key above)
        ##--------------------------------------------------------------------------------------------
        ## {
        ##     'instances' => {
        ##                    'instance-data' => {
        ##                                       'name' => '/vol/vol_2tb/2tb-C4hRV4YEnXo9',
        ##                                       'counters' => {
        ##                                                     'counter-data' => {
        ##                                                                       'read_align_histo' => {
        ##                                                                                             'value' => '1136,0,4,0,9,0,15,0'
        ##                                                                                           },
        ##                                                                       'write_align_histo' => {
        ##                                                                                              'value' => '2478526,1504,2970,1551,2898,1541,2807,1547'
        ##                                                                                            }
        ##                                                                     }
        ##                                                   }
        ##                                     }
        ##                  },
        ##     'timestamp' => '1281882602',
        ##     'status' => 'passed'
        ## };
        ##--------------------------------------------------------------------------------------------
        # But if there are more than 1 LUNs(say 2) the hash dump will look like this
        ##--------------------------------------------------------------------------------------------
        ##{
        ##   'instances' => {
        ##                  'instance-data' => {
        ##                                     '/vol/vol_2tb/2tb-C4hRV4YEnXo9' => {
        ##                                                                        'counters' => {
        ##                                                                                      'counter-data' => {
        ##                                                                                                        'read_align_histo' => {
        ##                                                                                                                              'value' => '1136,0,4,0,9,0,15,0'
        ##                                                                                                                            },
        ##                                                                                                        'write_align_histo' => {
        ##                                                                                                                               'value' => '2478526,1504,2970,1551,2898,1541,2807,1547'
        ##                                                                                                                             }
        ##                                                                                                      }
        ##                                                                                    }
        ##                                                                      },
        ##                                     '/vol/tmp_vol/lun1-C4hRV4YSqSrr' => {
        ##                                                                         'counters' => {
        ##                                                                                       'counter-data' => {
        ##                                                                                                         'read_align_histo' => {
        ##                                                                                                                               'value' => '0,0,0,0,0,0,0,0'
        ##                                                                                                                             },
        ##                                                                                                         'write_align_histo' => {
        ##                                                                                                                                'value' => '0,0,0,0,0,0,0,0'
        ##                                                                                                                              }
        ##                                                                                                       }
        ##                                                                                     }
        ##                                                                       }
        ##                                   }
        ##                },
        ##   'timestamp' => '1281882903',
        ##   'status' => 'passed'
        ##}
        ##--------------------------------------------------------------------------------------------
        # Because of this we have to treat the hash differently
        if (defined ($output_hash_1->{instances}->{"$data_key"}->{name})) {
            $key = $output_hash_1->{instances}->{"$data_key"}->{name};
            $before = $output_hash_1->{instances}->{"$data_key"}->{counters}->{"counter-data"}->{read_align_histo}->{value};
            $after  = $output_hash_2->{instances}->{"$data_key"}->{counters}->{"counter-data"}->{read_align_histo}->{value};
            $single_lun = 1;
        } else {
            $before = $output_hash_1->{instances}->{"$data_key"}->{$key}->{counters}->{"counter-data"}->{read_align_histo}->{value};
            $after  = $output_hash_2->{instances}->{"$data_key"}->{$key}->{counters}->{"counter-data"}->{read_align_histo}->{value};
        }
        my @before = split (/\,/, $before);
        my @after = split (/\,/, $after);
        # Get aligned reads
        my $aligned_reads =  shift(@after) - shift(@before);
        my $misaligned_reads = 0;
        # Get all the unaligned reads
        for (my $i = 0; $i < 7; $i++) {
            $misaligned_reads += $after[$i] - $before[$i];
        }
        my $total_reads = $misaligned_reads + $aligned_reads;
        if ($total_reads > 0) {
            my $percent_misaligned_reads = ($misaligned_reads * 100) / ($total_reads);
            if ($percent_misaligned_reads > $lun_misalignment_threshold) {
                push (@misaligned_accessed_luns, $key);
                $misaligned_data->{$key}->{reads} = $percent_misaligned_reads;
            }
        }
        if ($single_lun == 1) {
            # We dont want to iterate through other keys of hash if there is only
            # one LUN on controller head. breaking the for loop
            last;
        }
    }

    # Get the amount of misaligned data written in last 1 min
    $single_lun = 0;
    foreach my $key (keys %{$output_hash_1->{instances}->{"$data_key"}}) {
        my ($before, $after);
        if (defined ($output_hash_1->{instances}->{"$data_key"}->{name})) {
            $key = $output_hash_1->{instances}->{"$data_key"}->{name};
            $before = $output_hash_1->{instances}->{"$data_key"}->{counters}->{"counter-data"}->{write_align_histo}->{value};
            $after  = $output_hash_2->{instances}->{"$data_key"}->{counters}->{"counter-data"}->{write_align_histo}->{value};
            $single_lun = 1;
        } else {
            $before = $output_hash_1->{instances}->{"$data_key"}->{$key}->{counters}->{"counter-data"}->{write_align_histo}->{value};
            $after  = $output_hash_2->{instances}->{"$data_key"}->{$key}->{counters}->{"counter-data"}->{write_align_histo}->{value};
        }
        my @before = split (/\,/, $before);
        my @after = split (/\,/, $after);
        # Get aligned writes
        my $aligned_writes =  shift(@after) - shift(@before);
        my $misaligned_writes = 0;
        # Get all the unaligned writes
        for (my $i = 0; $i < 7; $i++) {
            $misaligned_writes += $after[$i] - $before[$i];
        }

        my $total_writes = $misaligned_writes + $aligned_writes;
        # Dont do anything if there is no IO running on this LUN
        if ($total_writes > 0) {
            my $percent_misaligned_writes = ($misaligned_writes * 100) / ($total_writes);
            # Check for the unalignment
            if ($percent_misaligned_writes > $lun_misalignment_threshold) {
                push (@misaligned_accessed_luns, $key);
                $misaligned_data->{$key}->{writes} = $percent_misaligned_writes;
            }
        }
        if ($single_lun == 1) {
            # We dont want to iterate through other keys of hash if there is only
            # one LUN on the controller head. breaking the for loop
            last;
        }
    }
    if (scalar(@misaligned_accessed_luns) > 0) {
        my $err_string = "Misaligned access (reads/writes) for LUN(s)\n";
        foreach my $lun (@misaligned_accessed_luns) {
            my $reads = 0;
            my $writes = 0;
            $reads = $misaligned_data->{$lun}->{reads};
            $writes = $misaligned_data->{$lun}->{writes};
            $err_string .= "LUN: $lun  misaligned reads = $reads\%, misaligned writes = $writes\%\n";
        }
        $self->_set_err($err_string);
        return $ClientUtil::FALSE;
    }

    # Since we checked the result, lets see if our check was OK
    if (!$resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed for Zapi perf command \'perf-object-get-instances\'");
        return $ClientUtil::FALSE;
    }
    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    get_iscsi_nodename()
# Objective: Get filer iscsi nodename
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_iscsi_nodename
{
    my ($self) = @_;
    my $nodename;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-node-get-name",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1);

    if($cmd_obj->error()) {
        my $errmsg = $cmd_obj->errmsg();
        $self->_set_err("Unable to get iSCSI nodename: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    if ($self->interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        unless (defined $output_hash->{"node-name"}) {
            $self->_set_err("Unable to parse output: {node-name} "
                            ."does not exist");
            return "";
        }
        $nodename = $output_hash->{"node-name"};
    } else {
        $output =~ /iSCSI target nodename:\s+(\S+)/;
        $nodename = $1;
    }

    $self->sanlog(-msg_level => 3,
                  -msg => "output from iscsi nodename is $output");

    #
    # Since we checked the result, lets see if our check was OK
    #
    if(!$resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed");
        return "";
    }
        return $nodename;
}

###############################################################################
# Method:    get_cfmode()
# Objective: Get filer cfmode
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_cfmode
{
    my ($self) = @_;
    my (
        @output,
        $cmd,
        $cfmode,
        $line,
        );

    $self->_clear_err();

    $cmd="fcp show cfmode";

    @output=$self->execute($cmd,-hide=>1);

    unless (defined $output[0] and length $output[0]){
        $self->_set_err("Error: SanFiler::get_cfmode(): Unable to get filer cfmode");
        return "";
    }

    $line=join("\n",@output);

    if ($line=~/not clustered/i) {
        $cfmode="Not Clustered";
    }
    elsif ($line=~/not licensed/i){
        $cfmode="FCP is not licensed";
    }
    else {
        #(output) fcp show cfmode: partner"
        ($cfmode)=$line=~/:\s(\S+)/;
    }

    return $cfmode;

}

###############################################################################
# Method:    set_cfmode
# Objective: Sets the cf mode on the filer.
# Details:   See POD documentation at the end of this file.
# Notes:     The user should have stopped fcp before calling this and should
#            start it afterwards
###############################################################################
sub set_cfmode
{
    my ($self, $cfmode) = @_;
    my (
        $cmd,
        @output,
        $current_cfmode,
        );

    return $ClientUtil::TRUE if ($self->get_cfmode() eq $cfmode);

    $cmd="priv set advanced;fcp set cfmode -f $cfmode";

    # Execute the command
    @output=$self->execute($cmd);
    if(grep /fcp set cfmode: / , @output) {
        my $errmsg = join "\n", @output;
        $self->_set_err("Error: SanFiler::set_cfmode() could not ".
                        "set cfmode to $cfmode: $errmsg");
        return $ClientUtil::FALSE;
    }
    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    get_fcp_nodename()
# Objective: Get filer fcp nodename
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_fcp_nodename
{
    my ($self) = @_;
    my $nodename;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "fcp-node-get-name",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1);

    if($cmd_obj->error()) {
        my $errmsg = $cmd_obj->errmsg();
        $self->_set_err("Unable to get FCP nodename: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    if ($self->interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        unless (defined $output_hash->{"node-name"}) {
            $self->_set_err("Unable to parse output: {node-name} "
                            ."does not exist");
            return "";
        }
        $nodename = $output_hash->{"node-name"};
    } else {
        $output =~ /Fibre Channel nodename:\s+(\S+)\s+/;
        $nodename = $1;
    }

    $self->sanlog(-msg_level => 3,
                  -msg => "output from fcp nodename is $output");

    #
    # Since we checked the result, lets see if our check was OK
    #
    if(!$resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed");
        return "";
    }
        return $nodename;
}

###############################################################################
# Method:    set_fcp_nodename()
# Objective: set filer fcp nodename
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_fcp_nodename
{
    my ($self, $nodename) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "fcp-node-set-name",
                        -cmd_filer              => $self->get_name(),
                        -"node-name"            => $nodename,
                        -check_result           => 1);

    if($cmd_obj->error()) {
        my $errmsg = $cmd_obj->errmsg();
        $self->_set_err("Unable to set FCP nodename: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("Result check failed");
        return $ClientUtil::FALSE;
    }
}

################################################################################
# Subroutine Name: get_igroups
# Objective:       return array of igroup objects
# Details:         See POD documentation at the end of this file
################################################################################
sub get_igroups
{
    my ($self, %args) = @_;
    my %hArgs;
    my @igrpObjs;
    my $name;
    my $initObjs;
    my @nodes;
    my @found_igrpObjs;

    $self->_clear_err();
    my $cmd_obj = $self->cmd_obj();

    foreach (keys %args) {
        if (/-?name/) {
            if (not length($name = $args{$_})) {
                $self->_set_err("Invalid value for argument '-name'");
                return (wantarray()) ? () : [];
            }
            next;
        }
        elsif (/-?init/i) { #initiator groups
            if (length($args{$_}) and ref($args{$_}) eq "ARRAY") {
                foreach my $obj (@{$args{$_}}) {
                    unless (ref($obj) eq "Initiator") {
                        $self->_set_err("Invalid value for argument '-initObjs'");
                        return $self;
                    }
                }

                $initObjs=$args{$_};
            } else {
                $self->_set_err("Invalid value for argument '-initObjs'");
                return $self;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return (wantarray()) ? () : [];
        }
    }

    my %cmd_args = (
                    "-cmd"           => "igroup-list-info",
                    "-cmd_filer"     => $self->get_name(),
                    "-verbose"       => "",
                    "-cmd_interface" => "cli",
                    "-check_result"  => 1,
                    );
    if (defined $name and length $name) {
        $cmd_args{"-initiator-group-name"} = $name
    }

    #
    # execute the command
    #
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if($cmd_obj->error()) {
        my $errmsg = $cmd_obj->errmsg();
        $self->_set_err("Unable to get igroup info: $errmsg");
        return ();
    }

    my $lines = $resultsHash_ref->{output};
    my @output = split("\n", $lines);

    # Go through the output, create and store IGroup objects for all igroups
    foreach my $line (@output) {
        next unless ($line=~/FCP|iSCSI/);

        my ($name, @jnk) = $line =~ /\s+(\S+)/;

        # Create and return igroup object
        $hArgs{-loghdl} = $self->{log} if (defined $self->{log});
        my $grpObj = IGroup->_new(-name   => $name,
                                  -parent => $self,
                                  -_exist => 1,
                                  %hArgs);
        if ($grpObj->error){
            $self->_set_err($grpObj->errmsg());
            return ();
        }
        push @igrpObjs,$grpObj;
    }

   foreach my $obj (@igrpObjs) {
   # Filter igroups if given a name or initiator
        if (defined $name and length $name) {
            if ($obj->get_name() =~ /^$name$/i) {
                return $obj;
            } else {
                next;
            }
        }

        if (defined $initObjs and scalar (@$initObjs) > 1) {
            foreach my $initObj (@$initObjs) {
                push @nodes, $initObj->get_portname();
            }

            $self->sanlog(-msg => "Searching for igroup containing initiator @nodes\n");
            #@nodes=$clientObj->get_nics(-protocol=>$protocol);
            my $exist=0;
            foreach my $node (@nodes){
                if (grep{$_ =~ /^$node$/i} @{$obj->get_nodes()}){
                    $exist=1;
                    last;
                }
            }
            next if (not $exist);
        }

        push @found_igrpObjs,$obj;
    }

    unless (@found_igrpObjs) {
        $self->_set_err("SanFiler object has no igroups");
        return ();
    }

    return (wantarray()) ? @found_igrpObjs : \@found_igrpObjs;
}

################################################################################
# Subroutine Name: get_igroup
# Objective:       return a igroup object by name or initiator
# Details:         See POD documentation at the end of this file
################################################################################
sub get_igroup
{
    my ($self,%args) = @_;
    my $name;
    my @igrpObjs;

    $self->_clear_err();

    my $arg_found = 0;

    # parse arguments
    foreach (keys %args) {

        if (/-?name/) {
            if (not length($name = $args{$_})) {
                $self->_set_err("Invalid value for argument '-name'");
                return (wantarray()) ? () : [];
            }
            $arg_found = 1;
            next;
        }
        elsif (/-?init/i) { #initiator groups
            if (length($args{$_}) and ref($args{$_}) eq "ARRAY") {
                foreach my $obj (@{$args{$_}}) {
                    unless (ref($obj) eq "Initiator") {
                        $self->_set_err("Invalid value for argument '-initObjs'");
                        return $self;
                    }
                }
            } else {
                $self->_set_err("Invalid value for argument '-initObjs'");
                return $self;
            }
            $arg_found = 1;
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return (wantarray()) ? () : [];
        }
    }

    unless($arg_found) {
        $self->_set_err("One or more arguments need to be",
                        "supplied (-name,-initObjs) to locate igroup");
        return (wantarray()) ? () : [];
    }

    if (defined $name and length $name) {
        # Get a current snapshot of the igroups on the system
        # if we are looking for igroup by name - we will get back
        # one object - simply return that object if found
        my $igrpObj = $self->get_igroups(%args);
        if ($igrpObj) {
            return $igrpObj;
        }
    } else {
        @igrpObjs = $self->get_igroups(%args);
    }

    # In this case, either we searched by name and found nothing
    # or we searched by initObjs and found nothing - set error
    if (scalar(@igrpObjs) eq 0){
        $self->_set_err("Unable to find igroup");
        return (wantarray()) ? () : [];
    }

    return (wantarray()) ? @igrpObjs : \@igrpObjs;
}

################################################################################
# Subroutine Name: get_lun_minsize()
# Objective:       Gets the minimum lun size of a type of lun, zapi only.
# Details:         See POD documentation at the end of this file
################################################################################
sub get_lun_minsize
{
    my ($self,%args) = @_;

    my $type;
    my $cmd_obj = $self->{cmd_obj};
    my $filer = $self->get_name();

    $self->_clear_err();

    foreach (keys %args) {
        if (/-?type/) {
            if (not length($type= $args{$_})) {
                $self->_set_err("Invalid value for argument '-type'");
                return $SanUtil::FALSE;
            }
            next;
        }  else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $SanUtil::FALSE;
        }
    }

    unless(defined $type and length $type){
            $self->_set_err("type is a required argument");
            return $SanUtil::FALSE;
    }

    $self->sanlog(-msg => "Get minimum size for lun type $type");

    my %cmd_args = ("-cmd"            => "lun-get-minsize",
                    "-cmd_filer"      => $filer,
                    "-check_result"   => 1,
                    "-cmd_interface"  => "zapi",
                    "-type"           => $type,
                   );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if($cmd_obj->error()) {
        $self->_set_err("CmdsDriver failed: ", $cmd_obj->errmsg());
        $self->sanlog(-msg => join("\n", $cmd_obj->errmsg()) );
        return $SanUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Zapi only so need to convert xml output to hash
    #
    my $output_hash = $self->xml2hash($output);

    #
    # Since we checked the result, see if check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed:\n", $output_hash->{reason});
        return $SanUtil::FALSE;
    }

    return (defined($output_hash->{'min-size'})) ?
           $output_hash->{'min-size'} : undef;
}

################################################################################
# Subroutine Name: lun_get_maxsize()
# Objective:       Gets the maxsize of the lun of a given type that can be
#                  created in a given root volume or qree
# Details:         See POD documentation at the end of this file
################################################################################
sub get_lun_maxsize
{
    my ($self,%args) = @_;

    my $type;
    my $cmd_obj = $self->{cmd_obj};
    my $filer = $self->get_name();
    my $interface = $self->{interface};
    my $this_path;
    my $snapshot_reserve = 1;
    my $size;
    my $unit;
    my $version;

    $self->_clear_err();

    foreach (keys %args) {
        if (/-?type/) {
            if (not length($type= $args{$_})) {
                $self->_set_err("Invalid value for argument '-type'");
                return $SanUtil::FALSE;
            }
            next;
        } elsif (/-?path/) {
            if (not length($this_path= $args{$_})) {
                $self->_set_err("Invalid value for argument '-path'");
                return $SanUtil::FALSE;
            }
            next;
        } elsif (/-?snapshot_reserve/) {
            if ( not length($snapshot_reserve = $args{$_} )
                 or $snapshot_reserve !~ /^(1|0)$/    ) {
                $self->_set_err("Invalid value for argument".
                             " '-snapshot_reserve'");
                 return $SanUtil::FALSE;
            }
            next;
        }
        else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $SanUtil::FALSE;
        }
    }

    unless(defined $type and length $type){
            $self->_set_err("type is a required argument");
            return $SanUtil::FALSE;
    }

    unless(defined $this_path and length $this_path){
            $self->_set_err("path is a required argument");
            return $SanUtil::FALSE;
    }

    $self->sanlog(-msg => "Get maximum size for lun type $type");

    my %cmd_args = ("-cmd"            => "lun-get-maxsize",
                    "-cmd_filer"      => $filer,
                    "-check_result"   => 1,
                    "-cmd_interface"  => $interface,
                    "-path"           => $this_path,
                    "-type"           => $type,
                   );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if($cmd_obj->error()) {
        $self->_set_err("CmdsDriver failed: ", $cmd_obj->errmsg());
        $self->sanlog(-msg => join("\n", $cmd_obj->errmsg()) );
        return $SanUtil::FALSE;
    }

    # Get the output
    my $output = $resultsHash_ref->{output};

    # Since we checked the result, lets see if our check was OK
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $SanUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    if (lc($interface) eq "cli") {
        #
        # Parse lun maxsize cli output
        #
        # Space available for a LUN of type: solaris, aix, hpux, linux, netware, vmware, openvms, xen, solaris_efi or image
        # With fractional reserve = 0: 97725m (102472089600)
        # With currently configured fractional reserve: 48832m (51204063232)
        # With fractional reserve = 100: 48832m (51204063232)
        # Space available for a LUN of type:  windows, windows_gpt, windows_2008 or hyper_v
        # With fractional reserve = 0: 97723m (102470538240)
        # With currently configured fractional reserve: 48830m (51202368000)
        # With fractional reserve = 100:
        my @cli_output = split("\n", $output);
        if ($type =~ /solaris|aix|hpux|linux|netware|vmware|openvms|xen|image/){
            if ($snapshot_reserve) {
                $output = $cli_output[2];
            } else {
                $output = $cli_output[1];
            }
        }
        if ($type =~ /windows|windows_gpt|windows_2008|hyper_v/) {
            if ($snapshot_reserve) {
                $output = $cli_output[6];
            } else {
                $output = $cli_output[5];
            }
        }
        $output =~ /\:\s+(\d+)(\w+)\s+\(/;
        $size=$1;
        $unit=$2;
        return "$size"."$unit";
    } else {
        # The interface is zapi
        my $output_hash = $self->xml2hash($output);
        my $with_snapshot_reserve_output;
        my $without_snapshot_reserve_output;

        if ($output =~ /snapshot\-reserve/) {
            #
            # Parse zapi output for < 8.0
            #
            # <results status="passed">
            #        <with-snapshot-reserve>51301071360</with-snapshot-reserve>
            #        <with-complete-snapshot-reserve>51301071360</with-complete-snapshot-reserve>
            #        <without-snapshot-reserve>102733747200</without-snapshot-reserve>
            # </results>
            if (defined $output_hash->{'with-snapshot-reserve'}) {
                $with_snapshot_reserve_output = $output_hash->{'with-snapshot-reserve'};
            } else {
               $self->_set_err("Value not returned for snapshot reserve: ", $output_hash);
               $with_snapshot_reserve_output = undef;
            }
            if ( defined $output_hash->{'without-snapshot-reserve'}) {
               $without_snapshot_reserve_output = $output_hash->{'without-snapshot-reserve'};
            } else {
              $self->_set_err("Value not returned without snapshot reserve: ", $output_hash);
               $without_snapshot_reserve_output = undef;
            }

            return ($snapshot_reserve) ? $with_snapshot_reserve_output :
                                         $without_snapshot_reserve_output;
        } elsif ($output =~ /fractional\-reserve/) {
            #
            # Parse zapi Output for releases with versions less than 8.0
            #
            # <results status="passed">
            #        <with-currently-configured-fractional-reserve>51284620800</with-currently-configured-fractional-reserve>
            #        <with-complete-fractional-reserve>51284620800</with-complete-fractional-reserve>
            #        <without-fractional-reserve>102717296640</without-fractional-reserve>
            # </results>
            my $output_hash = $self->xml2hash($output);
            return "$output_hash->{'with-currently-configured-fractional-reserve'}";
        }
    }
}

###############################################################################
# Method:    get_ipaddress()
# Objective: Get filer ip address
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_ipaddress
{
    my ($self) = @_;
    my (
        $ip,
        );

    $self->_clear_err();

    if (exists $self->{default_ip} and length $self->{default_ip}){
        $ip = $self->{default_ip};
    }
    else {
        $self->_set_err("Error: SanFiler::get_ipaddress(): Unable to get filer ip address");
        $ip="";
    }


    return $ip;

}
###############################################################################
# Method:    get_partner()
# Objective: Get filer partner
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_partner
{
    my ($self) = @_;
    my (
        @output,
        $cmd,
        $partner,
        $line,
        );

    $self->_clear_err();

    $cmd="cf partner";

    @output=$self->execute($cmd,-hide=>1);

    unless (defined $output[0] and length $output[0]){
        $self->_set_err("Error: SanFiler::get_partner(): Unable to get filer partner");
        return "";
    }

    $line=join("\n",@output);

    if ($line=~/license/i) {
        $partner="None";
    }
    else {
        ($partner)=$line=~/(\S+)/;
    }

    return $partner;

}

###############################################################################
# Method:    get_model()
# Objective: Get model type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_model
{
    my ($self) = @_;
    my (
        @output,
        $cmd,
        $line,
        $model,
        );

    $self->_clear_err();

    $cmd="sysconfig -v";

    @output=$self->execute($cmd,-hide=>1);

    unless (defined $output[0] and length $output[0]){
        $self->_set_err("Error: SanFiler::get_model(): Unable to get filer model");
        return "";
    }

    $line=join("\n",@output);

    ($model)=$line=~/Model Name:\s+([\w\-\d]+)\s+/;

    return $model;
}

###############################################################################
# Method:    get_serial_num()
# Objective: Get system serial_num
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_serial_num
{
    my ($self) = @_;
    my (
        @output,
        $cmd,
        $line,
        $serial_num,
        );

    $self->_clear_err();

    $cmd="sysconfig -v";

    @output=$self->execute($cmd,-hide=>1);

    unless (defined $output[0] and length $output[0]){
        $self->_set_err("Error: SanFiler::get_serial_num(): Unable to get filer serial number");
        return "";
    }

    $line=join("\n",@output);

    ($serial_num)=$line=~/System Serial Number:\s+(\w+)\s+/;

    unless(defined $serial_num){
        $self->_set_err("Error: SanFiler::get_serial_num(): Serial number not found");
        return "";
    }

    return $serial_num;
 }

###############################################################################
# Method:    get_license_info()
# Objective: Return license key for a service
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_license_info
{
    my ($self,$service) = @_;
    my (
        @output,
        $cmd,
        $line,
        $license,
       );

    $self->_clear_err();

    $cmd="license";

    @output=$self->execute($cmd,-hide=>1);

    unless (defined $output[0] and length $output[0]){
        $self->_set_err("Unable to get license information");
        return "";
    }

    $line=join("\n",@output);

    #
    # o/p of license cmd
    # rtpqa-fas3050-1*> license
    #               fcp site ICRNCZG
    #                cf site KDQLGBN

    $line=~/$service\s+(\w+)\s(\w+)\s+/;

    $license = $1." ".$2;

    $self->sanlog(-msg       => ref($self) ."$license \n",
                  -msg_level => 1);

    unless(defined $license){
        $self->_set_err("license info not found");
        return "";
    }

    return $license;
 }

###############################################################################
# Method:    get_name()
# Objective: Get filer name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_name
{
    my ($self) = @_;
    my (
        @output,
        $cmd,
        $name,
        );

    $self->_clear_err();

    return $self->{name} if (defined $self->{name});

    $cmd="hostname";

    @output=$self->execute($cmd,-hide=>1);

    unless (defined $output[0] and length $output[0]){
        $self->_set_err("Error: SanFiler::get_name(): Unable to get filer name");
        return "";
    }

    ($name)=$output[0]=~/(\w+)/;

    $self->{name}=$name;

    return $name;

}

###############################################################################
# Method:    get_vol()
# Objective: get a vol object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_vol
{
    my($self,$name) = @_;
    my (
        $volobj,
        $filer,
        $devices,
    %hArgs,
        );

    $self->_clear_err();

    unless(defined $name and length $name){
        $self->_set_err("Error: SanFiler::get_vol() volume name is a required argument");
        return $self;
    }

    # Create and return lun object
    $hArgs{-loghdl}=$self->{log} if (defined $self->{log});
    $volobj = Volume->_new(-name=>$name,-parent=>$self,%hArgs);
    if ($volobj->error){
        $self->_set_err($volobj->errmsg());
        return $self;
    }


    return $volobj;
}

###############################################################################
# Method:    get_volnames()
# Objective: get volume names
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_volnames
{
    my($self) = @_;
    my (
    $cmd,
    @buffer,
        @output
        );

    $self->_clear_err();

    @buffer=();

    $cmd="vol status";

    @output=$self->execute($cmd,-hide=>1);
    foreach my $line (@output){
    next unless ($line=~/raid/i);
    my($name)=$line=~/\s*(\S+)/;
    push @buffer,$name;
    }

    return (wantarray()) ? @buffer : \@buffer;
}

###############################################################################
# Method:    get_vfiler()
# Objective: Given the name of a vfiler, return the vfiler object.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_vfiler
{
    my($self, %args) = @_;
    my $vfiler_obj;
    my $name;
    my @vfiler_objs;
    my $cmd_obj = $self->{cmd_obj};

    $self->_clear_err();

    foreach (keys %args) {
        if (/^-?name$/) {
            unless(length($name = $args{$_})) {
                $self->_set_err("Invalid value for \"name\"");
            }
        }
    }

    #
    # Get vfiler information from the filer using the vfiler-list-info ZAPI
    #
    my %cmd_args = (-cmd           => "vfiler-list-info",
                    -cmd_filer     => $self->get_name(),
                    -cmd_interface => "zapi",
                    -check_result  => 1);

    #
    # Add in the name if needed.
    #
    $cmd_args{"-vfiler"} = $name if ($name);

    #
    # Execute the command
    #
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to list vfiler info: $errmsg");
        return $ClientUtil::FALSE;
    }

    #
    # Translate the XML returned from the ZAPI into a hash
    #
    my $output = $resultsHash_ref->{output};
    my $output_hash = $self->xml2hash($output);

    if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
        my $dumpOut = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => DUMPER_DEBUG_LEVEL);
    }

    #
    # The command execution may have passed but did the command itself pass?
    # If not, get out.
    #
    my $status = $output_hash->{status};
    if($status eq "failed") {
        my $errmsg = $output_hash->{reason};
        $self->_set_err("Unable to list vfiler info: $errmsg");
        return $ClientUtil::FALSE;
    }

    my $vfilers = $output_hash->{vfilers}->{"vfiler-info"};

    #
    # $vfilers should be a hash but if there is only 1 vfiler (vfiler0),
    # then the hash will have keys that are data values such as
    # name, uuid, dns-info, vfstores, etc.  I'm going to check for some
    # of these keys and if they exist then we we create a vfiler object
    # for the data.  Otherwise, we enter a loop and create an object foreach
    # each vfiler
    #
    unless (ref($vfilers) =~ /HASH/i) {
        $self->_set_err("Unexpected type returned from vfiler-list-info ZAPI");
        return $ClientUtil::FALSE;
    }

    if ((defined($vfilers->{name}) and ref($vfilers->{name}) == undef) and
        (defined($vfilers->{uuid}) and ref($vfilers->{uuid}) == undef)) {
        $self->sanlog(-msg => "SanFiler::get_vfiler: Getting vfiler " .
                              $vfilers->{name},
                      -msg_level => 1);

        #
        # Create the vfiler object and push on the vfilers stack
        #
        $vfiler_obj = San::SanFiler::Vfiler->_new(parent=>$self, %{$vfilers});
        if ($vfiler_obj->error()) {
            my  $errmsg = join "\n", $vfiler_obj->errmsg();
            $self->_set_err("Unable to create vfiler object: $errmsg");
            return $ClientUtil::FALSE;
        }
        push(@vfiler_objs, $vfiler_obj);
    } else {
        #
        # If we are loading a single vfiler, we should never get here
        #
        if ($name) {
            $self->_set_err("Unexpectedly received multiple vfilers " .
                            "from the vfiler-list-info ZAPI");
            return $ClientUtil::FALSE;
        }
        foreach my $vfiler_name (keys(%{$vfilers})) {
            $self->sanlog(-msg => "SanFiler::get_vfiler: Loading vfiler " .
                                  $vfiler_name,
                          -msg_level => 1);
            #
            # Get the hash for an individual vfiler
            #
            my $current_vfiler = $vfilers->{$vfiler_name};

            #
            # Create the vfiler object and push on the vfilers stack
            # Note, adding in name here, when the XML gets translated to
            # a hash, the name becomes the hash key and the {name} field
            # will no longer be in the hash for the vfiler.  This only
            # happens when you have more than one vfiler.
            #
            $current_vfiler->{name} = $vfiler_name;
            $vfiler_obj = San::SanFiler::Vfiler->_new(parent=>$self, %{$current_vfiler});
            if ($vfiler_obj->error()) {
                $self->_set_err("Unable to create vfiler object: ",
                                $vfiler_obj->errmsg());
                return $ClientUtil::FALSE;
            }
            push(@vfiler_objs, $vfiler_obj);

         } # end foreach vfiler
    } # end else for multiple vfilers found

    #
    # Check for name argument - if it was not passed in,
    # return all of the vfiler objects
    #
    unless(defined $name and length $name) {
        return (wantarray()) ? @vfiler_objs : \@vfiler_objs;
    }

    #
    # if the name argument was passed in, return just that
    # vfiler object - if it was not found, this will have been caught above
    #
    return $vfiler_obj;

} # end of get_vfiler()

###############################################################################
# Method:    set_vfiler_context()
# Objective: Set the current vfiler context to the given name.  For this to
#            take effect, we need to set the SanFiler attribute "vfiler" and
#            an optional "vfiler_ip" attribute for CmdsDriver. "vfiler_ip" will
#            only be set if "zapi_use_vfiler_ip" is set to 1.
# Inputs    -name - required - name of vfiler
#           -zapi_use_vfiler_ip - optional -  (0 or 1), use the $vfiler_ip if set
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_vfiler_context
{
    my($self, %args) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my ($name, $ipaddress);
    my $zapi_use_vfiler_ip = 0;

    $self->_clear_err();

    foreach (keys %args) {
        if (/^-?name$/) {
            unless(length($name = $args{$_})) {
                $self->sanlog("Clearing vfiler context");
            }
        } elsif (/^-?zapi_use_vfiler_ip$/) {
            $zapi_use_vfiler_ip = $args{$_};
            if (not length($zapi_use_vfiler_ip)) {
                # Empty String Passed. Using default value
                $self->sanlog("Invalid value for \$zapi_use_vfiler_ip: "
                            . "$zapi_use_vfiler_ip: Using default value of 0");
            } elsif ($zapi_use_vfiler_ip !~ /1|0/) {
                $self->_set_err("Invalid value for '-zapi_use_vfiler_ip': $zapi_use_vfiler_ip");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return (wantarray()) ? () : [];
        }

        #
        # Note, we are not grabbing -interface and -ipaddress where, we just
        # pass them through to Vfiler::get_ipaddress()
        #
    }

    #
    # Check for required arguments
    #
    unless(defined $name){
        $self->_set_err("Vfiler name is a required argument");
        return $self;
    }

    $self->sanlog(-msg => "SanFiler::set_vfiler_context(): Setting vfiler " .
                          "context to $name",
                  -msg_level => 1);

    #
    # If the user just sent in "" to clear the vfiler context,
    # just clear the value of {vfiler},{vfiler_ip} and the CmdsDriver
    # values.
    #
    unless (length($name)) {
        $self->{vfiler}    = $cmd_obj->vfiler("");
        $self->{vfiler_ip} = $cmd_obj->vfiler_ip("");
        return $ClientUtil::TRUE;
    }

    #
    # Based on the name, find the vfiler object
    #
    my $vfiler_obj = $self->get_vfiler(-name => $name);
    if($self->error()) {
        $self->_set_err("Unable to find vfiler $name: " . $self->errmsg());
        return $ClientUtil::FALSE;
    }

    #
    # Get the ipaddress from the vfiler object.  This will error out if
    # you've given and improper 'interface' or 'ipaddress'
    #
    if ($zapi_use_vfiler_ip) {
        $ipaddress = $vfiler_obj->get_ipaddress(%args);
        if($vfiler_obj->error()) {
            $self->_set_err("Unable to find vfiler ip address: " .
                            $vfiler_obj->errmsg());
            return $ClientUtil::FALSE;
        }
    }

    $self->sanlog(-msg => "SanFiler::set_vfiler_context(): Setting " .
                          "cmd_obj vfiler to $name ");
    $self->sanlog(-msg => "SanFiler::set_vfiler_context(): Setting " .
                          "cmd_obj vfiler_ip to $ipaddress") if $zapi_use_vfiler_ip;

    #
    # Save the vfiler name and ip address for SanFiler and CmdsDriver
    #
    $self->{vfiler}    = $cmd_obj->vfiler($name);
    $self->{vfiler_ip} = $cmd_obj->vfiler_ip($ipaddress) if $zapi_use_vfiler_ip;

    return $ClientUtil::TRUE;
} # end of set_vfiler_context()

###############################################################################
# Method:    clear_vfiler_context()
# Objective: Clears the current vfiler context.  For this to
#            take effect, we need to set the SanFiler attribute "vfiler" and
#            we need to set the vfiler_ip attribute and set both of these
#            for CmdsDriver
# Details:   See POD documentation at the end of this file.
###############################################################################
sub clear_vfiler_context
{
    my($self, %args) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $name;

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::clear_vfiler_context(): Clearing vfiler " .
                          "context",
                  -msg_level => 1);

    #
    # Clear the value of {vfiler},{vfiler_ip} and the CmdsDriver
    # values.
    #
    $self->{vfiler}    = $cmd_obj->vfiler("");
    $self->{vfiler_ip} = $cmd_obj->vfiler_ip("");
    return $ClientUtil::TRUE;

} # end of clear_vfiler_context()

###############################################################################
# Method:    get_vfiler_context()
# Objective: Simple wrapper around the vfiler() method
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_vfiler_context
{
    my ($self) = @_;
    return $self->vfiler();
}


###############################################################################
# Method:    giveback()
# Objective: Place filer in giveback mode
# Details:   See POD documentation at the end of this file.
###############################################################################
sub giveback
{
    my ($self, $force) = @_;
    my (
        @output,
        $cmd,
        $cfmode,
        $line,
        );

    $self->_clear_err();

    $force = 0 if !defined($force);

    if($force)
    {
        $cmd="cf giveback -f";
    }
    else
    {
        $cmd="cf giveback";
    }

    @output=$self->execute($cmd,-hide=>1);

    $line=$output[0];

    if ($line =~ /giveback in progress|not in takeover|takeover not completed|not waiting/i) {
        $self->_set_err("Error: SanFiler::giveback(): Unable to perform filer giveback",@output);
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;

}

###############################################################################
# Method:    info
# Objective: Return important information about filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub info
{
    my ($self)=@_;
    my (
        @output,
        );

    push @output,
    "Name:            ".$self->get_name(),
    "Model:           ".$self->get_model(),
    "Version:         ".$self->get_version(),
    "Version (Major): ".$self->get_version_major(),
    "Cfmode:          ".$self->get_cfmode(),
    "Partner:         ".$self->get_partner();

    push @output,"\n";

    return (wantarray()) ? @output: \@output;

}

###############################################################################
# Method:    display_info
# Objective: Return important information about filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub display_info {
    my ($self) = @_;

    my @msg_array = $self->info();
    my $msg = join("\n", @msg_array);

    $self->sanlog(-msg => "Controller Information:\n$msg");

    return $ClientUtil::TRUE;

}

###############################################################################
# Method:    is_giveback_complete()
# Objective: returns status of giveback
# Details:   See POD documentation at the end of this file.
###############################################################################
sub is_giveback_complete
{
    my ($self) = @_;
    my (
    @output,
    );

    $self->_clear_err();

    @output=$self->execute("cf status",-hide=>1);
    if ($self->error()){
    $self->_set_err("ERROR: SanFiler::is_gievaback(): Unable to determine giveback completion",@output);
    return $ClientUtil::FALSE;
    }

    # In case giveback wasn't successfully issued.  Try again.
    if ("@output"=~/Takeover due to negotiated failover/) {
    $self->giveback() if ("@output"=~/Takeover due to negotiated failover/);
    return $ClientUtil::FALSE;
    }

    return ("@output" =~ /Cluster enabled|Controller Failover enabled/i) ? $ClientUtil::TRUE : $ClientUtil::FALSE;
}

###############################################################################
# Method:    is_takeover_complete()
# Objective: returns status of takeover
# Details:   See POD documentation at the end of this file.
###############################################################################
sub is_takeover_complete
{
    my ($self) = @_;
    my (
    @output,
    );

    $self->_clear_err();

    @output=$self->execute("cf status",-hide=>1);
    if ($self->error()){
    $self->_set_err("ERROR: SanFiler::is_takeover_complete(): Unable to determine takeover completion",@output);
    return $ClientUtil::FALSE;
    }

    return ("@output" =~ / has taken over /i) ? $ClientUtil::TRUE : $ClientUtil::FALSE;

}

###############################################################################
# Method:     ip_offline()
# Objective:  Bring filer ip offline
# Details:    See POD documentation at the end of this file.
###############################################################################
sub ip_offline
{
    my ($self, $interface) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $line;
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg => "Bringing filer ip offline");

    unless (defined $interface and length $interface)
    {
        $interface = "e0";
    }

    if($self->interface() eq "cli")
    {
        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                            -cmd                    => "ifconfig-set",
                            -cmd_filer              => $self->get_name(),
                            -check_result           => 1,
                            "-interface-name"       => $interface,
                            "-option-name"          => "down");
    }
    else
    {
        my @cmd_args;
        push(@cmd_args, "ifconfig");
        push(@cmd_args, $interface);
        push(@cmd_args, "down");
        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                            -cmd                      => "system-cli",
                            -cmd_filer                => $self->get_name(),
                            -check_result             => 1,
                            -args                     => \@cmd_args,
                            -priv                     => "admin");
    }

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::ip_offline(): Unable to " .
                        "bring ip offline: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    $line = $resultsHash_ref->{output};
    my @output = split("\n", $line);

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($line);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }
        my $status = $output_hash->{"status"};

        if($status eq "failed")
        {
            $self->_set_err("Error: SanFiler::ip_offline(): Unable to " .
                            "bring ip offline", @output);
            return $ClientUtil::FALSE;
        }
        else
        {
            $line = $output_hash->{"cli-output"};

            if($line =~ /ifconfig:/)
            {
               $self->_set_err("Error: SanFiler::ip_offline(): Unable to " .
                               "bring ip offline", $line);
                return $ClientUtil::FALSE;
            }
        }
    }
    else
    {
        if(@output)
        {
            $self->_set_err("Error: SanFiler::ip_offline(): Unable to " .
                            "bring ip offline", @output);
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::ip_offline(): Failed results " .
                        "check");
        return $ClientUtil::FALSE;
    }
} # end sub ip_offline

###############################################################################
# Method:     ip_online()
# Objective:  Bring filer ip online
# Details:    Currently there is no POD documentation for this method
###############################################################################
sub ip_online
{
    my ($self, $interface) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $line;
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg => "Bringing filer ip online");

    unless (defined $interface and length $interface)
    {
        $interface = "e0";
    }

    if($self->interface() eq "cli")
    {
        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                            -cmd                    => "ifconfig-set",
                            -cmd_filer              => $self->get_name(),
                            -check_result           => 1,
                            "-interface-name"       => $interface,
                            "-option-name"          => "up");
    }
    else
    {
        my @cmd_args;

        push(@cmd_args, "ifconfig");
        push(@cmd_args, $interface);
        push(@cmd_args, "up");
        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                            -cmd                      => "system-cli",
                            -cmd_filer                => $self->get_name(),
                            -check_result             => 1,
                            -args                     => \@cmd_args,
                            -priv                     => "admin");
    }

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::ip_online(): Unable to " .
                        "bring ip online: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    $line = $resultsHash_ref->{output};
    my @output = split("\n", $line);

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($line);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{"status"};

        if($status eq "failed")
        {
            $self->_set_err("Error: SanFiler::ip_online(): Unable to " .
                            "bring ip online", @output);
            return $ClientUtil::FALSE;
        }
        else
        {
            $line = $output_hash->{"cli-output"};

            if($line =~ /ifconfig:/)
            {
                $self->_set_err("Error: SanFiler::ip_online(): Unable to " .
                                "bring ip online", $line);
                return $ClientUtil::FALSE;
            }
        }
    }
    else
    {
        if(@output)
        {
            $self->_set_err("Error: SanFiler::ip_online(): Unable to " .
                            "bring ip online", @output);
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::ip_online(): Failed results " .
                        "check");
        return $ClientUtil::FALSE;
    }
} # end sub ip_online

###############################################################################
# Method:    ip_set
# Objective: Sets an ip on an interface
# Details:   See POD documentation at the end of this file.
###############################################################################
sub ip_set
{
    my (
        $self,
        $interface,
        $address,
        $mask,
    )=@_;
    my ($cmd, @output, @rcfile_from_filer, $notfound, $line);

    $self->_clear_err();

    $self->sanlog(-msg => "ip set");

    if(defined($interface) and length($interface) &&
       defined($address) and length($address))
    {
        $cmd="ifconfig $interface $address";

        if(defined($mask))
        {
            $cmd.=" netmask $mask";
        }

        @output = $self->execute($cmd);

        if(grep /ifconfig: / , @output)
        {
            my $errmsg = join "\n", @output;
            $self->_set_err("Error: SanFiler::ip_set()"                 .
                            " could not properly configure the IP for " .
                            "$interface: $errmsg");
            return $ClientUtil::FALSE;
        }

        $cmd="rdfile /etc/rc";
        @rcfile_from_filer=$self->execute($cmd);

        $cmd="priv set advanced; rm /etc/rc";
        $self->execute($cmd);

        $notfound = 1;

        foreach $line (@rcfile_from_filer)
        {
            if($line =~ /^\s*ifconfig $interface.*/)
            {
                $notfound=0;

                if($line =~ /address/)
                {
                    $line =~ s/address \d+\.\d+\.\d+\.\d+/address $address/g;
                }
                if($line =~ /netmask/)
                {
                    $line =~ s/netmask \d+\.\d+\.\d+\.\d+/netmask $mask/g;
                }
            }

            $cmd='wrfile -a /etc/rc "'.$line.'"';
            $self->execute($cmd);
        }

        if($notfound)
        {
            $line = "ifconfig $interface `hostname`-$interface "     .
                    "mediatype auto flowcontrol full netmask $mask " .
                    "address $address mtusize 1500";

            $cmd='wrfile -a /etc/rc "' . $line . '"';
            $self->execute($cmd);
        }
        return $ClientUtil::TRUE;
    } # end if interface and address are defined
    else
    {
        $self->_set_err("Error: SanFiler::ip_set() required parameters: " .
                        "interface or address have not been supplied");
        return $ClientUtil::FALSE;
    }
} # end ip_set()

###############################################################################
# Method:    iscsi_start
# Objective: Start iscsi service
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_start
{
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $interface = $self->interface();
    my $output_hash;

    $self->_clear_err();
    # Build command arguments.
    my %cmd_args = ("-cmd"            => "iscsi-service-start",
                    "-cmd_filer"      => $self->get_name(),
                    "-cmd_interface"  => $interface,
                    "-check_result"   => 1);

    # Execute command.
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if($cmd_obj->error()) {
        $self->_set_err("Unable to start fcp service: ", $cmd_obj->errmsg());
        return $SanUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi") {
        $output_hash = $self->xml2hash($output);
    }

    # Since we checked the result, lets see if our check was OK
    #
    unless($resultsHash_ref->{result_check}) {
        if($interface eq "zapi") {
            $self->_set_err("Result check failed:\n",
                            $output_hash->{reason});
            return $SanUtil::FALSE;
        } else {
           $self->_set_err("Result check failed:\n", $output);
           return $SanUtil::FALSE;
        }
    }

    return $SanUtil::TRUE;
} # end iscsi_start()

###############################################################################
# Method:    iscsi_stop
# Objective: Stop iscsi service
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_stop
{
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $interface = $self->interface();
    my $output_hash;

    $self->_clear_err();
    # Build command arguments.
    my %cmd_args = ("-cmd"            => "iscsi-service-stop",
                    "-cmd_filer"      => $self->get_name(),
                    "-cmd_interface"  => $interface,
                    "-check_result"   => 1);

    # Execute command.
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if($cmd_obj->error()) {
        $self->_set_err("Unable to start fcp service: ", $cmd_obj->errmsg());
        return $SanUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi") {
        $output_hash = $self->xml2hash($output);
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless($resultsHash_ref->{result_check}) {
        if($interface eq "zapi") {
            $self->_set_err("Result check failed:\n",
                            $output_hash->{reason});
            return $SanUtil::FALSE;
        } else {
           $self->_set_err("Result check failed:\n", $output);
           return $SanUtil::FALSE;
        }
    }

    return $SanUtil::TRUE;
} # end iscsi_stop()

###############################################################################
# Method:    iscsi_interface_enable()
# Objective: Enable iscsi on an ethernet interface
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_interface_enable
{
    my ($self, $interface) = @_;
    my (
        @output,
        $cmd,
        $errmsg,
        $output_hash,
    );

    $self->_clear_err();

    Tharn::logcomment("Enabling iscsi interface $interface");

    my $cmd_interface = $self->interface();
    my $cmd_obj = $self->cmd_obj();

    # check for required arguments
    unless (defined $interface and length $interface){
        $self->_set_err("Error: SanFiler::iscsi_interface_enable() Missing required argument 'interface'");
        return $ClientUtil::FALSE;
    } # end unless (defined...

    if ($cmd_interface eq "cli") {
        $cmd = "iscsi interface enable $interface";

        @output=$self->execute($cmd,-hide=>1);

        if ("@output" =~ /Enabling\s+interface/
                || "@output" =~ /already\s+enabled/){
                return $ClientUtil::TRUE;
        } # end if ("@output"...

        $self->_set_err("Error: SanFiler::iscsi_interface_enable(): Unable to bring interface online",@output);
        return $ClientUtil::FALSE;
    } else {
        # Build command arguments.
        my %cmd_args = ("-cmd"            => "iscsi-interface-enable",
                        "-cmd_filer"      => $self->get_name(),
                        "-cmd_interface"  => $cmd_interface,
                        "-interface-name" => $interface,
                        "-check_result"   => 1);

        # Execute command.
        my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
        if($cmd_obj->error()) {
            $self->_set_err("Unable to enable iscsi interface: ", $cmd_obj->errmsg());
            return $SanUtil::FALSE;
        } # end if ($cmd_obj...

         $self->log_cmd_results($resultsHash_ref);

        #
        # Get the output
        #
        my $output = $resultsHash_ref->{output};

        #
        # since cmd_interface is zapi we have to convert the output from xml
        # to a string.
        #
        $output_hash = $self->xml2hash($output);

        #
        # Since we checked the result, lets see if our check was OK
        #
        unless($resultsHash_ref->{result_check}) {
            $self->_set_err("Result check failed:\n",
                                            $output_hash->{reason});
            return $SanUtil::FALSE;
        } # end unless($resultsHash...

        return $SanUtil::TRUE;
    } # end if ($cmd_interface
} # end sub iscsi_interface_enable()

###############################################################################
# Method:    iscsi_interface_disable()
# Objective: Disable iscsi on an ethernet interface
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_interface_disable
{
    my ($self, $interface) = @_;
    my (
        @output,
        $cmd,
        $errmsg,
        $output_hash,
        );

    $self->_clear_err();

    Tharn::logcomment("Disabling iscsi interface $interface");

    my $cmd_interface = $self->interface();
    my $cmd_obj = $self->cmd_obj();

    # check for required arguments
    unless (defined $interface and length $interface){
        $self->_set_err("Error: SanFiler::iscsi_interface_disable() Missing required argument 'interface'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    if ($cmd_interface eq "cli") {
        $cmd = "iscsi interface disable -f $interface";
        @output=$self->execute($cmd,-hide=>1);

        if ("@output" =~ /Disabling\s+interface/
            || "@output" =~ /already\s+disabled/){
            return $ClientUtil::TRUE;
        } # end if ("@output...

        $self->_set_err("Error: SanFiler::iscsi_interface_disable(): Unable to bring interface offline",@output);
        return $ClientUtil::FALSE;

    } else {
        # Build command arguments.
        my %cmd_args = ("-cmd"            => "iscsi-interface-disable",
                        "-cmd_filer"      => $self->get_name(),
                        "-cmd_interface"  => $cmd_interface,
                        "-interface-name" => $interface,
            "-check_result"   => 1);

        # Execute command.
        my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
        if($cmd_obj->error()) {
            $self->_set_err("Unable to disable iscsi interface: ", $cmd_obj->errmsg());
            return $SanUtil::FALSE;
        } # end if($cmd_obj...

        $self->log_cmd_results($resultsHash_ref);

        #
        # Get the output
        #
        my $output = $resultsHash_ref->{output};

        #
        # since cmd_interface is zapi we have to convert the output from xml
        # to a string.
        #
        $output_hash = $self->xml2hash($output);

        #
        # Since we checked the result, lets see if our check was OK
        #
        unless($resultsHash_ref->{result_check}) {
            $self->_set_err("Result check failed:\n",
                                            $output_hash->{reason});
            return $SanUtil::FALSE;
        } # end unless($resultsHash...

        return $SanUtil::TRUE;

    } # end if ($cmd_interface...
} # end sub iscsi_interface_disable()

###############################################################################
# Method:    iscsi_interface_show
# Objective: Shows iscsi interface information
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_interface_show
{
    my ($self, %args) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $iscsi_interface;
    my $out = "";

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::iscsi_interface_show() iscsi " .
                          "interface show",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    foreach (keys %args)
    {
        if (/-?iscsi_interface/)
        {
            $iscsi_interface = $args{$_};
            if (not length($iscsi_interface))
            {
                $self->_set_err("SanFiler::iscsi_interface_show invalid " .
                                "value for \"iscsi_interface\"","$args{$_}");
                return $self;
            };
        }
        else
        {
            $self->_set_err("SanFiler::iscsi_interface_show Invalid "     .
                            "parameter specified: $_");
            return $self;
        }
    }

    my $resultsHash_ref;
    my @cmd_args;

    push(@cmd_args, "iscsi");
    push(@cmd_args, "interface");
    push(@cmd_args, "show");
    # In this case -a means show all, so if the interface isn't
    # defined we display all
    push(@cmd_args, "-a") if not defined($iscsi_interface);
    push(@cmd_args, $iscsi_interface) if defined($iscsi_interface);

    $resultsHash_ref =
     $cmd_obj->execute_cmd(
                    -cmd                      => "system-cli",
                    -cmd_filer                => $self->get_name(),
                    -cmd_interface            => "zapi",
                    -check_result             => 1,
                    -args                     => \@cmd_args,
                    -priv                     => "admin");

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::iscsi_interface_show(): " .
                        "Unable to get information on interface: $errmsg");
        return "";
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    my $output_hash = $self->xml2hash($output);
    if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
    {
        my $dumpOut = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => DUMPER_DEBUG_LEVEL);
    }

    my $status = $output_hash->{status};

    if($status eq "failed")
    {
        my $errmsg=$output_hash->{reason};
        $self->_set_err("Error: SanFiler::iscsi_interface_show(): ",
                        $errmsg);
        return "";
    }
    else
    {
        if($output =~ /iscsi interface show:/i)
        {
            $self->_set_err("Error: SanFiler::iscsi_interface_show() " .
                            "could not display info on interface: ",
                            $output);
            return "";
        }
    }

    $out = $output_hash->{"cli-output"};

    if(not defined($out))
    {
        $self->_set_err("Error: SanFiler::iscsi_interface_show() " .
                        "could not display info on interface");
        return "";
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $out;
    }
    else
    {
        $self->_set_err("Error: SanFiler::iscsi_interface_show() " .
                        "results check failed");
        return "";
    }
} # end sub iscsi_interface_show

###############################################################################
# Method:    iscsi_connection_list_info()
# Objective: Lists filer iscsi connection parameters
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_connection_list_info
{
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my ($output_hash, $iscsi_connection_list_info);

    # cli part need to be implemented.
    my $interface = "zapi";

    $self->_clear_err();

    # Build command arguments.
    my %cmd_args = (
                    "-cmd"            => "iscsi-connection-list-info",
                    "-cmd_filer"      => $self->get_name(),
                    "-cmd_interface"  => $interface,
                    "-check_result"   => 1);

    # Execute command.
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    if($cmd_obj->error()) {
        $self->_set_err("Unable to execute iscsi connection list info : ", $cmd_obj->errmsg());
        return $SanUtil::FALSE;
    } # end if ($cmd_obj...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    if ($interface eq "zapi") {
        $output_hash = $self->xml2hash($output);
    } # end if ($interface...

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless($resultsHash_ref->{result_check}) {
        if($interface eq "zapi") {
            $self->_set_err("Result check failed:\n",
                            $output_hash->{reason});
            return $SanUtil::FALSE;
        } else {
            $self->_set_err("Result check failed:\n", $output);
            return $SanUtil::FALSE;
        } # end if ($interface...
    } # end unless($resultsHash...

    if ($interface eq "zapi") {
        if($output_hash->{status} eq "failed") {
            $self->_set_err("Error: SanFiler::iscsi_connection_list_info: " .
                            "Unable to get iscsi connection parameter ");
            return $SanUtil::FALSE;
        } else {
            $iscsi_connection_list_info =
                    $output_hash->{'iscsi-connection-list-entries'};
            my $dumpOut = Dumper($iscsi_connection_list_info);
        $self->sanlog(-msg   => "Output Hash is:\n$dumpOut",
                                    msg_level => 7);
        return $iscsi_connection_list_info;
        } # end if ($output_hash...
    } # end if ($interface...

} ## end sub iscsi_connection_list_info()

###############################################################################
# Method:    iscsi_session_list_info()
# Objective: Lists filer iscsi connection parameters
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_session_list_info
{
    my ($self, %args) = @_;
    my ($output_hash, $iscsi_session_list_info, $tsih);

    #
    # Parse arguments
    #
    foreach (keys %args) {
        if (/-?tsih/) {
            if (not length($tsih = $args{$_})) {
                $self->_set_err("Invalid value for argument '-tsih'");
                return $SanUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $SanUtil::FALSE;
        } # end if (/-?tsih/)
    } # end foreach...

    my $cmd_obj = $self->cmd_obj();

    # cli part need to be implemented.
    my $interface = "zapi";

    $self->_clear_err();

    # Build command arguments.
    my %cmd_args = (
                    "-cmd"            => "iscsi-session-list-info",
                    "-cmd_filer"      => $self->get_name(),
                    "-cmd_interface"  => $interface,
                    "-check_result"   => 1);

    if ($tsih) {
        $cmd_args{"-tsih"} = $tsih;
    } # end if ($tsih...

    # Execute command.
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    if($cmd_obj->error()) {
        $self->_set_err("Unable to execute iscsi session list info : ", $cmd_obj->errmsg());
        return $SanUtil::FALSE;
    } # end if($cmd_obj->...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    if ($interface eq "zapi") {
        $output_hash = $self->xml2hash($output);
    } # end if ($interface...

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless($resultsHash_ref->{result_check}) {
        if($interface eq "zapi") {
            $self->_set_err("Result check failed:\n",
                            $output_hash->{reason});
            return $SanUtil::FALSE;
        } else {
           $self->_set_err("Result check failed:\n", $output);
           return $SanUtil::FALSE;
        } # end if($interface...
    } # end unless($resultsHash...

    if ($interface eq "zapi") {
        if($output_hash->{status} eq "failed") {
            $self->_set_err("Error: SanFiler::iscsi_session_list_info: " .
                      "Unable to get iscsi session parameter ");
            return $SanUtil::FALSE;
        } else {
            $iscsi_session_list_info =
                    $output_hash->{'iscsi-session-list-entries'};
        my $dumpOut = Dumper($iscsi_session_list_info);
            $self->sanlog(-msg  => "Output Hash is:\n$dumpOut",
                                msg_level => 7);
            return $iscsi_session_list_info;
        } # end if ($output_hash...
    } # end if ($interface...

} ## end sub iscsi_session_list_info()

###############################################################################
# Method:    iscsi_initiator_list_info()
# Objective: Lists all logged in iscsi initiator
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_initiator_list_info
{
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my ($output_hash, $iscsi_initiator_list_info);

    # cli part need to be implemented.
    my $interface = "zapi";

    $self->_clear_err();

    # Build command arguments.
    my %cmd_args = (
                    "-cmd"            => "iscsi-initiator-list-info",
                    "-cmd_filer"      => $self->get_name(),
                    "-cmd_interface"  => $interface,
                    "-check_result"   => 1);

    # Execute command.
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    if($cmd_obj->error()) {
        $self->_set_err("Unable to execute iscsi initiator list info : ", $cmd_obj->errmsg());
        return $SanUtil::FALSE;
    } # end if ($cmd_obj...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    if ($interface eq "zapi") {
        $output_hash = $self->xml2hash($output);
    } # end if ($interface...

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless($resultsHash_ref->{result_check}) {
        if($interface eq "zapi") {
            $self->_set_err("Result check failed:\n",
                            $output_hash->{reason});
            return $SanUtil::FALSE;
        } else {
            $self->_set_err("Result check failed:\n", $output);
            return $SanUtil::FALSE;
        } # end if ($interface...
    } # end unless($resultsHash...

    if ($interface eq "zapi") {
        if($output_hash->{status} eq "failed") {
            $self->_set_err("Error: SanFiler::iscsi_initiator_list_info: " .
                        "Unable to get iscsi initiator parameter ");
            return $SanUtil::FALSE;
        } else {
            $iscsi_initiator_list_info =
                    $output_hash->{'iscsi-initiator-list-entries'};
            my $dumpOut = Dumper($iscsi_initiator_list_info);
        $self->sanlog(-msg   => "Output Hash is:\n$dumpOut",
                                    msg_level => 7);
        return $iscsi_initiator_list_info;
        } # end if ($output_hash->...
    } # end if ($interface...

} ## end sub iscsi_initiator_list_info()

###############################################################################
# Method:    panic()
# Objective: Panic the filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub panic
{
    my ($self) = @_;
    my (
        $cmd,
        $errmsg,
        );

    $self->_clear_err();

    Tharn::logcomment("Panic the filer");

    $cmd = "priv set advanced;panic";
    &Tharn::say($self->{connection}, $cmd, '-blind');

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    port_online()
# Objective: Bring target port up
# Details:   See POD documentation at the end of this file.
###############################################################################
sub port_online
{
    my ($self,$port) = @_;
    my (
        @output,
        $cmd,
        $errmsg,
        );

    $self->_clear_err();


  Tharn::logcomment("Bringing port up ");

    # check for required arguments
    unless (defined $port and length $port){
        $self->_set_err("Error: SanFiler::port_online() Missing required argument 'port'");
        return $ClientUtil::FALSE;
    }


    $cmd="fcp config $port up";

    @output=$self->execute($cmd,-hide=>1);

    if ("@output" =~ /not found/i){
        $self->_set_err("Error: SanFiler::port_online(): Unable to bring port online",@output);
        return $ClientUtil::FALSE;
    }elsif ("@output"
                =~ /Unable to online adapter while fcp service is stopped/i) {
        $self->_set_err("Error: SanFiler::port_online(): Unable to bring "
                                ."port online",@output);
        return $ClientUtil::FALSE;
    }


    return $ClientUtil::TRUE;


}

###############################################################################
# Method:    port_offline()
# Objective: Bring target port down
# Details:   See POD documentation at the end of this file.
###############################################################################
sub port_offline
{
    my ($self,$port) = @_;
    my (
        @output,
        $cmd,
        $errmsg,
        );

    $self->_clear_err();


  Tharn::logcomment("Bringing port offline ");

    # check for required arguments
    unless (defined $port and length $port){
        $self->_set_err("Error: SanFiler::port_offline() Missing required argument 'port'");
        return $ClientUtil::FALSE;
    }


    $cmd="fcp config $port down";

    @output=$self->execute($cmd,-hide=>1);

    if ("@output" =~ /not found/i){
        $self->_set_err("Error: SanFiler::port_offline(): Unable to bring port offline",@output);
        return $ClientUtil::FALSE;
    }elsif ("@output" =~ /The FCP port is already down/i){
        $self->_set_err("Error: SanFiler::port_offline(): Unable to bring"
                                                ." port offline",@output);
        return $ClientUtil::FALSE;
    }


    return $ClientUtil::TRUE;


}

###############################################################################
# Method:    fcp_adapter_config_media_type
# Objective: Sets the link type on the Fibre Channel target adapter.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub fcp_adapter_config_media_type {
    my ($self, %args) = @_;

    my ($adapter_name, $media_type);
    my $cmd_obj = $self->cmd_obj();
    my $cmd_interface = $self->interface();

    # parse arguments
    foreach (keys %args) {
        if (/-?adapter_name/) {
            if (not defined($adapter_name = $args{$_})) {
                $self->_set_err("Invalid value for argument -adapter_name ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } elsif (/-?media_type/) {
            if (not defined($media_type = $args{$_})) {
                $self->_set_err("Invalid value for argument -media_type ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?adapter_...
    } ## end foreach (keys %args)

    # check for required arguments
    unless (defined $adapter_name and length $adapter_name){
        $self->_set_err("Error: SanFiler::fcp_adapter_config_media_type()"
                            ." Missing required argument 'adapter_name'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    unless (defined $media_type and length $media_type){
        $self->_set_err("Error: SanFiler::fcp_adapter_config_media_type()"
                                ." Missing required argument 'media_type'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fcp_adapter_config_media_type() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fcp-adapter-config-media-type",
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $cmd_interface,
                "-fcp-adapter"   => $adapter_name,
                "-media-type",   => $media_type,
            );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fcp-adapter-config-"
                                        ."media-type: $errmsg");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($cmd_interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fcp-adapter-config-media-type Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...
    } # end if ($cmd_interface...

    return $ClientUtil::TRUE;

} ## end sub fcp_adapter_config_media_type()

###############################################################################
# Method:    sysconfig()
# Objective: Parse the output of sysconfig and returns a hash
# Details:   See POD documentation at the end of this file.
###############################################################################
sub sysconfig {
    my ($self, %args) = @_;
    my ($adapter, $sysconfig_hash, $Cmd_Obj, $resultsHash_ref, $slot);

    # sysconfig has  [ -A | -c | -d | -h | -m | -r | -t | -V ] and
    # sysconfig [ -av ] [ <slot> ]
    # and this API is developed for sysconfig [ -v ] [ <slot> ]
    # it returns a hash which includes cpu and backplane information.

    #
    # Parse arguments
    #
    foreach (keys %args) {
        if (/-?slot/) {
            if (not length($slot = $args{$_})) {
                $self->_set_err("Invalid value for argument '-slot'");
                return $SanUtil::FALSE;
            } # end if (not length...
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $SanUtil::FALSE;
        } # end if (/-?slot...
    } # end foreach...

    $Cmd_Obj = $self->cmd_obj();
    # clear the filer object if it has any previous error flag set.
    $self->_clear_err();

    # Build the command line and execute the command
    my %cmd_args = (
                    -cmd              => "system-get-info",
                    -cmd_filer        => $self->get_name(),
                    -cmd_interface    => "cli",
                    -verbose          => "",
                   );

    if ($slot) {
        $cmd_args{"-slot"} = $slot;
    } # end if ($slot) {

    # execute the command and check for error
    $resultsHash_ref = $Cmd_Obj->execute_cmd(%cmd_args);
    if ($Cmd_Obj->error()) {
        $self->_set_err("Error: Failed to execute sysconfig "
                                        ."command: ", $Cmd_Obj->errmsg());
        return $ClientUtil::FALSE;
    } # end if ($Cmd_Obj->...

    # get the ouput from the output hash
    my $lines = $resultsHash_ref->{output};
    my @output = split("\n", $lines);
    my $j = 0;
    my $count = @output;

    for (my $i = 0 ; $i < $count ; $i++) {
        if ($output[$i] =~ /System ID: (\d+) .*;( partner ID: )?(\d+)?/) {
            $sysconfig_hash->{'System ID'} = $1;
            if ($2) {
                $sysconfig_hash->{'Partner ID'} = $3;
            } # end if ($2...
        } elsif ($output[$i] =~ /System Serial Number: (\d+)/) {
            $sysconfig_hash->{'System Serial Number'} = $1;
        } elsif ($output[$i]
                =~ /slot (\d+): Fibre Channel Target Host Adapter\s(\w+)/) {
            $adapter =  $2;
            $sysconfig_hash->{'adapters'}->{$adapter}->{'slot'} = $1;
            $sysconfig_hash->{'adapters'}->{$adapter}->{'Type'} = "Target";
            $j = ++$i;
            my @slots;
            while ($output[$j] !~ /slot \d+/ && $j < $count ) {
                if ($output[$j] =~ /<(\w+)>/) {
                    $sysconfig_hash->{'adapters'}->{$adapter}->{'State'} = $1;
                } elsif($output[$j] =~ /<(\w+)\s\w+\s\w+>/) {
                    if ($1 !~ /LINK/) {
                        chop($1);
                        $sysconfig_hash->{'adapters'}->{$adapter}->{'State'} = $1;
                    } else {
                        $sysconfig_hash->{'adapters'}->{$adapter}->{'State'}
                                                = "LINK NOT CONNECTED";
                    }# end if ($1...
                } elsif($output[$j] =~ /^\s+\d+/) {
                    $j++;
                    next;
                } elsif($output[$j] =~ /MAC Address/i) {
                    while ($output[$j] =~ /(\w+)\sMAC Address/i) {
                        push @slots, $1;
                        $j++;
                    } # end while ($output[$j]...
                    $sysconfig_hash->{'adapters'}->{$adapter}->{'slots'} = @slots;
                } elsif($output[$j] =~ /ID     Vendor   Model/i) {
                    $j++;
                    next;
                } else {
                    my ($key, $value) = split(/:\s/, $output[$j]);
                    $key =~ s/^\s+//;
                    $value =~ s/^\s+//;
                    if ($key) {
                        $sysconfig_hash->{'adapters'}->{$adapter}->{$key} = $value;
                    } # end if ($key...
                } # end if ($output[$j...
                $j++;
            } # end while ($output[$i]...
        } elsif ($output[$i] =~ /slot (\d+): FC Host Adapter (\w+)/) {
            my $shelf = 0;
            my @slots;
            $adapter = $2;
            $sysconfig_hash->{'adapters'}->{$adapter}->{'Type'} = "Initiator";
            $sysconfig_hash->{'adapters'}->{$adapter}->{'slot'} = $1;
            if ($output[$i] =~ /<(\w+)>/) {
                $sysconfig_hash->{'adapters'}->{$adapter}->{'State'} = $1;
            } elsif($output[$i] =~ /<(\w+)\s\w+\s\w+>/) {
                if ($1 =~ /LINK/) {
                    $sysconfig_hash->{'adapters'}->{$adapter}->{'State'}
                                       = "LINK NOT CONNECTED";
                }# end if ($1...
            } elsif ($output[$i] =~ /<(\w+)\s\(\w+\)>/) {
                $sysconfig_hash->{'adapters'}->{$adapter}->{'State'} = $1;
            } # end if ($output[$i...
            $j = ++$i;

            while ($output[$j] !~ /slot \d+/ && $j < $count ) {
                if($output[$j] =~ /^\s+\d+/) {
                    $j++;
                    next;
                } elsif($output[$j] =~ /MAC Address/i) {
                    while ($output[$j] =~ /(\w+)\sMAC Address/i) {
                        push @slots, $1;
                        $j++;
                    } # end while ($output[$j]...
                    $sysconfig_hash->{'adapters'}->{$adapter}->{'slots'} = \@slots;
                } elsif($output[$j] =~ /ID     Vendor   Model/i) {
                    $j++;
                    next;
                } elsif($output[$j] =~ /Shelf/i) {
                    $shelf++;
                    $j++;
                    next;
                } else {
                    my ($key, $value) = split(/:\s/, $output[$j]);
                    $key =~ s/^\s+//;
                    $value =~ s/^\s+//;
                    if ($key) {
                        $sysconfig_hash->{'adapters'}->{$adapter}->{$key} = $value;
                    } # end if ($key...
                } # end if($output[$j] =~...
                $j++;
            } # end while ($output[$i]...
            if ($shelf) {
                $sysconfig_hash->{'adapters'}->{$adapter}->{'Shelf'} =
                                            "Disk Shelf connected";
            } else {
                $sysconfig_hash->{'adapters'}->{$adapter}->{'Shelf'} =
                                            "Disk Shelf not connected";
            } # end if ($shelf)...
        } else {
            next;
        } # end if ($output[$i...
    } # end for (my $i...

    my $dumpOut = Dumper($sysconfig_hash);
    $self->sanlog(-msg    => " Hash of sysconfig output is:\n$dumpOut",
                  msg_level => 7);

    return $sysconfig_hash;

} ## end sub sysconfig()

###############################################################################
# Method:    takeover()
# Objective: Place filer in takeover mode
# Details:   See POD documentation at the end of this file.
###############################################################################
sub takeover
{
    my ($self, $force) = @_;
    my (
        @output,
        $cmd,
        $cfmode,
        $line,
        );

    $self->_clear_err();

    $force = 0 if !defined($force);

    if($force)
    {
        $cmd="cf takeover -f";
    }
    else
    {
        $cmd="cf takeover";
    }

    @output=$self->execute($cmd,-hide=>1);

    unless (defined $output[0] and length $output[0]){
        $self->_set_err("Error: SanFiler::takeover(): Unable to perform filer takeover",@output);
        return $ClientUtil::FALSE;
    }

    $line=$output[0];

    unless ($line=~/takeover initiated/i) {
        $self->_set_err("Error: SanFiler::takeover(): Unable to perform filer takeover",@output);
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;

}

###############################################################################
# Method:    timeout
# Objective: set or get telnet timeout
# Details:   See POD documentation at the end of this file.
###############################################################################
sub timeout
{
    my ($self,$timeout) = @_;

    if (defined $timeout and length $timeout){
        $self->{timeout}=$timeout;
    }

    return $self->{timeout};
}

################################################################################
# Subroutine Name: vol_exist()
# Objective:       Determines if volume exists
# Details:         See POD documentation at the end of this file
################################################################################
sub vol_exist
{
    my ($self,$name)=@_;
    my (
        $cmd,
        @output,
        );

    unless (length $name){
        $self->_set_err("ERROR: SanFiler::vol_exist() Required parameter \$name required");
        return $ClientUtil::FALSE;
    }

    $cmd = "vol status ".$name;

    @output=$self->execute($cmd,-hide=>1);
    if ($self->error()){
        $self->_set_err($self->errmsg());
        return $ClientUtil::FALSE;
    }

    if ($output[0]=~/No volume/i){
        $self->_set_err("Error: SanFiler::vol_exist() no volume found",@output);
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

################################################################################
# Subroutine Name: lun_exist()
# Objective:       Determines if lun exists
# Details:         See POD documentation at the end of this file
################################################################################
sub lun_exist
{
    my ($self,$name)=@_;
    my (
        $cmd,
        @output,
        );

    unless (length $name){
        $self->_set_err("ERROR: SanFiler::lun_exist() Required parameter \$name required");
        return $ClientUtil::FALSE;
    }

    $cmd = "lun show ".$name;

    @output=$self->execute($cmd,-hide=>1);
    if ($self->error()){
        $self->_set_err($self->errmsg());
        return $ClientUtil::FALSE;
    }

    # we need to search the entire output array for the string
    # in the case of vfilers, this string will be several lines down
    if ("@output" =~ /No such LUN/i){
        $self->_set_err("Error: SanFiler::lun_exist() no LUN ($name) found (".@output.")");
        return $ClientUtil::FALSE;
    }
    elsif ("@output" =~ /lun pathname should start with/i) {
        $self->_set_err("Error: SanFiler::lun_exist() Invalid LUN path/name ($name) (".@output.")");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

################################################################################
# Subroutine Name: lun_config_check()
# Objective:       Perform lun config_check operation
# Details:         See POD documentation at the end of this file
################################################################################
sub lun_config_check
{
    my ($self, %args) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my $output_hash;
    my $ssi = 0;
    my $silent = 0;
    my $verbose = 0;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::lun_config_check()",
                  -msg_level => 1);

    #
    # Parse args hash
    #
    foreach (keys %args) {
        if (/-?ssi/) {
          $ssi = 1;
          next;
      } elsif (/-?verbose/) {
          $verbose = 1;
          next;
      } elsif (/-?silent/) {
          $silent = 1;
          next;
      }
    }

    # Build the command line and execute the command
    my %cmd_args = ("-cmd"              => "lun-config-check-info",
                    "-cmd_filer"        => $self->get_name(),
                    "-check_result"     => 1,
                    "-cmd_interface"    => "cli",
                   );

    if ($silent =~ /1/) {
        $cmd_args{"-silent"} = "";
    }
    if ($verbose =~ /1/) {
        $cmd_args{"-verbose"} = "";
    }
    if ($ssi =~ /1/) {
        $cmd_args{"-ssi"} = "";
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    if ($cmd_obj->error()) {
        $self->_set_err("lun config check failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (!($resultsHash_ref->{result_check}))
    {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    $self->sanlog(-msg       => "lun_config_check returns:\n $output",
                  -msg_level => 7);

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        # TBD - need to fix this method to handle ZAPI and return a HASH
        # instead of an ARRAY - keep the interface hardcoded to CLI for now
        my $lines = $resultsHash_ref->{output};
        my @output = split("\n", $lines);
        return (wantarray()) ? @output : \@output;
    } else {

        my $lines = $resultsHash_ref->{output};
        my @output = split("\n", $lines);
        return (wantarray()) ? @output : \@output;

    }    # end else cli

}

################################################################################
# Subroutine Name: igroup_exist()
# Objective:       Determine if an igroup exists
# Details:         See POD documentation at the end of this file
################################################################################
sub igroup_exist
{
    my ($self, $name) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my $output_hash;

    $self->_clear_err();

    unless (length $name) {
        $self->_set_err("Required parameter \$name required");
        return $ClientUtil::FALSE;
    }

    $self->sanlog(-msg       => "SanFiler::igroup_exist()",
                  -msg_level => 1);

    $resultsHash_ref =
      $cmd_obj->execute_cmd(-cmd                    => "igroup-list-info",
                            -cmd_filer              => $self->get_name(),
                            -check_output           => 1,
                            -check_result           => 1,
                            "-initiator-group-name" => $name,
                            -cmd_interface          => $self->interface());

    if ($cmd_obj->error()) {
        $self->_set_err("igroup list info failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (!($resultsHash_ref->{result_check} && $resultsHash_ref->{output_check}))
    {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    $self->sanlog(-msg       => "igroup_list_info returns:\n $output",
                  -msg_level => 7);

    if ($self->interface() eq "zapi") {
        if ($output =~ /No such initiator/i) {
            return $ClientUtil::FALSE;
        }
    } else {

        my $lines = $resultsHash_ref->{output};
        my @output = split("\n", $lines);
        if ("@output" =~ /No such initiator/i) {
            return $ClientUtil::FALSE;
        }

    }    # end else cli


    return $ClientUtil::TRUE;
} # end igroup_exist()

################################################################################
# Subroutine Name: qtree_exist()
# Objective:       Determines if qtree exists
# Details:         See POD documentation at the end of this file
################################################################################
sub qtree_exist {
    my ($self, %args) = @_;

    my ($cmd, @output, $volume, $qtree);

    # Parse args hash
    foreach (keys %args) {
        if (/-?volume/) {
            length($volume = $args{$_}) or
                do {
                    $self->_set_err("invalid value for volume: ","$args{$_}");
                    return $ClientUtil::FALSE;
                };
        } elsif (/-?qtree/) {
            length($qtree = $args{$_}) or
                do {
                    $self->_set_err("invalid value for qtree: ","$args{$_}");
                    return $ClientUtil::FALSE;
                };
        }
    }

    unless (length $volume){
        $self->_set_err("Required parameter \$volume required");
        return $ClientUtil::FALSE;
    }

    unless (length $qtree){
        $self->_set_err("Required parameter \$qtree required");
        return $ClientUtil::FALSE;
    }

    # Grab the Qtree list for the volume
    $cmd = "qtree status " . $volume;
    @output = $self->execute($cmd, -hide=>1);
    if ($self->error()){
        $self->_set_err($self->errmsg());
        return $ClientUtil::FALSE;
    }

    # Example output for qtrees that do not exist
    # fas2050-rtp03*> qtree status nate_11
    # qtree status: invalid volume name nate_11.

    # Example output for qtrees that do exist
    # fas2050-rtp03*> qtree status nate_1_1_hp2620_rtp11
    # Volume   Tree     Style Oplocks  Status
    # -------- -------- ----- -------- ---------
    # nate_1_1_hp2620_rtp11          unix  enabled  normal
    if ($output[0] =~ /invalid volume/i){
        $self->_set_err("Invalid volume name ($volume) provided");
        return $ClientUtil::FALSE;
    }

    # Search for the qtree on the volume
    if (not grep(/$qtree/, @output)) {
        $self->_set_err("no Qtree ($qtree) found on volume ($volume)");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    vol_status()
# Objective: Get vol status
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vol_status
{
    my ($self, $vol) = @_;
    my (
        @output,
        $cmd,
        $errmsg,
        $version,
        );

    $self->_clear_err();


    unless (defined $vol and length $vol){
        $self->_set_err("Error: SanFiler::vol_status() Missing required argument 'vol'");
        return (wantarray()) ? () : [];
    }

    Tharn::logcomment("Retrieve status for volume '$vol'");


    $cmd="vol status $vol";

    @output=$self->execute($cmd,-hide=>1);

    if ($output[0] =~ /no vol/i){
        ($errmsg)=$output[0]=~/\:(.*)/;
        $self->_set_err("Error: SanFiler::vol_status(): $errmsg");
        return (wantarray()) ? () : [];
    }


    return (wantarray()) ? @output : \@output;
}

################################################################################
# Method:    vol_sis_status
# Objective: Displays SIS status for all volumes
# Details:   See POD documentation at the end of this file
################################################################################
sub vol_sis_status
{
    my ($self) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::vol_sis_status() Displaying SIS status",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    if($self->interface() eq "cli")
    {
        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                        -cmd                    => "sis-status",
                        -cmd_filer              => $self->{filer},
                        -check_result           => 1,
                        -verbose                => "");
    }
    elsif($self->interface() eq "zapi")
    {
        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                        -cmd                    => "sis-status",
                        -cmd_filer              => $self->{filer},
                        -check_result           => 1,
                        -verbose                => "true");
    }
    else
    {
        my @cmd_args;
        push(@cmd_args, "sis");
        push(@cmd_args, "status");
        push(@cmd_args, "-l");

        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                            -cmd                      => "system-cli",
                            -cmd_filer                => $self->get_name(),
                            -cmd_interface            => "zapi",
                            -check_result             => 1,
                            -args                     => \@cmd_args,
                            -priv                     => "admin");
    }

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::vol_sis_status(): Unable to " .
                        "display SIS status: $errmsg");
        return "";
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($output);
        # my $dumpOut = Dumper($output_hash);
        # $self->sanlog(-msg      => "Output Hash is:\n$dumpOut",
        #               msg_level => 7);
        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Error: SanFiler::vol_sis_status() " .
                            "unable to display SIS status $errmsg");
            return "";
        }
    }
    else
    {
        if($output=~/Invalid path/i or $output=~/^status:/ or
           $output=~/^WARNING: SIS is not licensed.*/)
        {
            $self->_set_err("Error: SanFiler::vol_sis_status() unable to " .
                            "display SIS status $output");
            return "";
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $output;
    }
    else
    {
        $self->_set_err("Error: SanFiler::vol_sis_status() " .
                        "results check failed");
        return "";
    }
} # end vol_sis_status

################################################################################
# Method:    filersio_asyncio_active
# Objective: This workload keeps IOs in flight at a constant.
# Details:   See POD documentation at the end of this file
################################################################################
sub filersio_asyncio_active
{
    my ($self, %args) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $resultsHash_ref;
    my $read_percentage;
    my $rand_percentage;
    my $exp_constant;
    my $blk_size;
    my $start_byte;
    my $end_byte;
    my $run_time;
    my $num_active;
    my $filename;
    my $fill_pattern;
    my $verify_pattern;
    my $sync;
    my $create;
    my $interval;

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::filer_asyncio_active() filersio " .
                          "asyncio_active",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    #
    # Parse args hash
    #
    foreach (keys %args)
    {
        if (/-?read_percentage/)
        {
            length($read_percentage = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "read_percentage: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?rand_percentage/)
        {
            length($rand_percentage = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "rand_percentage: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?exp_constant/)
        {
            length($exp_constant = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "exp_constant: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?blk_size/)
        {
            length($blk_size = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "blk_size: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?start_byte/)
        {
            length($start_byte = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "start_byte: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?end_byte/)
        {
            length($end_byte = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "end_byte: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?run_time/)
        {
            length($run_time = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "run_type: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?num_active/)
        {
            length($num_active = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "num_active: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?filename/)
        {
            length($filename = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "filename: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?fill_pattern/)
        {
            length($fill_pattern = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "fill_pattern: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?verify_pattern/)
        {
            length($verify_pattern = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "verify_pattern: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?sync/)
        {
            length($sync = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "sync: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?create/)
        {
            length($create = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "create: ","$args{$_}");
                    return $self;
                };
        }
        elsif (/-?interval/)
        {
            length($interval = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::filersio_asyncio_active(): " .
                                    "invalid value for "                    .
                                    "interval: ","$args{$_}");
                    return $self;
                };
        }
        else
        {
            $self->_set_err("SanFiler::filersio_asyncio_active(): Invalid " .
                            "parameter specified: $_");
            return $self;
        }
    }

    #
    # Check for necessary arguments
    #
    unless(defined $read_percentage and length $read_percentage)
    {
        $self->_set_err("Error: SanFiler::filersio_asyncio_active() "      .
                        "read percentage is a required argument");
        return $ClientUtil::FALSE;
    }

    unless((defined $rand_percentage and length $rand_percentage) or
           (defined $exp_constant and length $exp_constant))
    {
        $self->_set_err("Error: SanFiler::filersio_asyncio_active() "      .
                        "rand percentage or exp constant must be specified");
        return $ClientUtil::FALSE;
    }

    unless(defined $blk_size and length $blk_size)
    {
        $self->_set_err("Error: SanFiler::filersio_asyncio_active() "      .
                        "block size is a required argument");
        return $ClientUtil::FALSE;
    }

    unless(defined $start_byte and length $start_byte)
    {
        $self->_set_err("Error: SanFiler::filersio_asyncio_active() "      .
                        "start byte is a required argument");
        return $ClientUtil::FALSE;
    }

    unless(defined $end_byte and length $end_byte)
    {
        $self->_set_err("Error: SanFiler::filersio_asyncio_active() "      .
                        "end byte is a required argument");
        return $ClientUtil::FALSE;
    }

    unless(defined $run_time and length $run_time)
    {
        $self->_set_err("Error: SanFiler::filersio_asyncio_active() "      .
                        "run time is a required argument");
        return $ClientUtil::FALSE;
    }

    unless(defined $num_active and length $num_active)
    {
        $self->_set_err("Error: SanFiler::filersio_asyncio_active() "      .
                        "num active is a required argument");
        return $ClientUtil::FALSE;
    }

    unless(defined $filename and length $filename)
    {
        $self->_set_err("Error: SanFiler::filersio_asyncio_active() "      .
                        "file name is a required argument");
        return $ClientUtil::FALSE;
    }

    my @cmd_args;
    push(@cmd_args, "filersio");
    push(@cmd_args, "asyncio_active");
    push(@cmd_args, $read_percentage);
    push(@cmd_args, "-r") if defined($rand_percentage);
    push(@cmd_args, $rand_percentage) if defined($rand_percentage);
    push(@cmd_args, "-e") if defined($exp_constant);
    push(@cmd_args, $exp_constant) if defined($exp_constant);
    push(@cmd_args, $blk_size);
    push(@cmd_args, $start_byte);
    push(@cmd_args, $end_byte);
    push(@cmd_args, $run_time);
    push(@cmd_args, $num_active);
    push(@cmd_args, $filename);
    push(@cmd_args, "-fillonce") if defined($fill_pattern);
    push(@cmd_args, $fill_pattern) if defined($fill_pattern);
    push(@cmd_args, "-verifyonce") if defined($verify_pattern);
    push(@cmd_args, $verify_pattern) if defined($verify_pattern);
    push(@cmd_args, "-synchronous") if defined($sync);
    push(@cmd_args, "-create") if defined($create);
    push(@cmd_args, "-print_stats") if defined($interval);
    push(@cmd_args, $interval) if defined($interval);

    $resultsHash_ref =
    $cmd_obj->execute_cmd(
                        -cmd                      => "system-cli",
                        -cmd_filer                => $self->get_name(),
                        -cmd_interface            => "zapi",
                        -cmd_timeout              => 3600,
                        -check_result             => 1,
                        -args                     => \@cmd_args,
                        -priv                     => "diag");

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::filersio_asyncio_active(): Unable to " .
                        "do filersio: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    if($output =~ /^filersio: .*/)
    {
        my $errmsg = $output;
        $self->_set_err("Error: SanFiler::filersio_asyncio_active() " .
                        "ZAPI command failed", $errmsg);
        return $ClientUtil::FALSE;
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::filersio_asyncio_active() " .
                        "result check failed");
        return $ClientUtil::FALSE;
    }
} # end filersio_asyncio_active

################################################################################
# Method:    remove_file
# Objective: Removes a file
# Details:   See POD documentation at the end of this file
################################################################################
sub remove_file
{
    my ($self, %args) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $resultsHash_ref;
    my $file;

    $self->_clear_err();

    #
    # Parse args hash
    #
    foreach (keys %args)
    {
        if (/-?file/)
        {
            length($file = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::remove_file() " .
                                    "invalid value for "                 .
                                    "file: ","$args{$_}");
                    return $self;
                };
        }
        else
        {
            $self->_set_err("SanFiler::remove_file() Invalid "  .
                            "parameter specified: $_");
            return $self;
        }
    }

    unless(defined $file and length $file)
    {
        $self->_set_err("Error: SanFiler::remove_file() "      .
                        "file name is a required argument");
        return $ClientUtil::FALSE;
    }

    $self->sanlog(-msg => "SanFiler::remove_file() Remove file: $file",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    my @cmd_args;
    push(@cmd_args, "rm");
    push(@cmd_args, $file);

    $resultsHash_ref =
    $cmd_obj->execute_cmd(
                        -cmd                      => "system-cli",
                        -cmd_filer                => $self->get_name(),
                        -cmd_interface            => "zapi",
                        -check_result             => 1,
                        -args                     => \@cmd_args,
                        -priv                     => "advanced");

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::remove_file(): Unable to " .
                        "remove file $file: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    if($output =~ /Invalid argument/)
    {
        my $errmsg = $output;
        $self->_set_err("Error: SanFiler::remove_file() " .
                        "ZAPI command failed", $errmsg);
        return $ClientUtil::FALSE;
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::remove_file() " .
                        "result check failed");
        return $ClientUtil::FALSE;
    }
} # end remove_file

################################################################################
# Method:    get_aggr_capacity
# Objective: Get's the current capacity (%) of an aggr
# Details:   See POD documentation at the end of this file
################################################################################
sub get_aggr_capacity
{
    my ($self, %args) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $resultsHash_ref;
    my $aggr;
    my $aggr_capacity;

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::get_aggr_capacity() Get aggr capacity",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    #
    # Parse args hash
    #
    foreach (keys %args)
    {
        if (/-?aggr/)
        {
            length($aggr = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::get_aggr_capacity() " .
                                    "invalid value for "             .
                                    "aggr: ","$args{$_}");
                    return $self;
                };
        }
        else
        {
            $self->_set_err("SanFiler::get_aggr_capacity() Invalid " .
                            "parameter specified: $_");
            return $self;
        }
    }

    unless(defined $aggr and length $aggr)
    {
        $self->_set_err("Error: SanFiler::get_aggr_capacity() "      .
                        "aggr name is a required argument");
        return "";
    }



    my @cmd_args;
    push(@cmd_args, "df");
    push(@cmd_args, "-A");
    push(@cmd_args, $aggr);

    $resultsHash_ref =
    $cmd_obj->execute_cmd(
                        -cmd                      => "system-cli",
                        -cmd_filer                => $self->get_name(),
                        -cmd_interface            => "zapi",
                        -check_result             => 1,
                        -args                     => \@cmd_args,
                        -priv                     => "admin");

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::get_aggr_capacity(): Unable to " .
                        "get aggr capacity: $errmsg");
        return "";
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};
    my @output = split('\n', $output);

    if($output =~ /^df: .*/)
    {
        my $errmsg = $output;
        $self->_set_err("Error: SanFiler::get_aggr_capacity() " .
                        "ZAPI command failed $errmsg");
        return "";
    }

    # pull out the capacity of $aggr
    # Expecting output like this:
    # Aggregate               kbytes       used      avail capacity
    # test_aggr            357104472  107860696  249243776      30%
    foreach my $line (@output)
    {
        next if ($line =~ /\/\.snapshot/);
        if ($line =~ /$aggr.+?\d+.+?\d+.+?\d+.+?(\d+)\%/)
        {
            $aggr_capacity = $1;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $aggr_capacity;
    }
    else
    {
        $self->_set_err("Error: SanFiler::get_aggr_capacity() " .
                        "result check failed");
        return "";
    }
} # end get_aggr_capacity

################################################################################
# Method:    get_aggr_size
# Objective: Get's the size (kb) of an aggr
# Details:   See POD documentation at the end of this file
################################################################################
sub get_aggr_size
{
    my ($self, %args) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $resultsHash_ref;
    my $aggr;
    my $aggr_size;

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::get_aggr_size() Get aggr size",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    #
    # Parse args hash
    #
    foreach (keys %args)
    {
        if (/-?aggr/)
        {
            length($aggr = $args{$_}) or
                do
                {
                    $self->_set_err("SanFiler::get_aggr_size() " .
                                    "invalid value for "         .
                                    "aggr: ","$args{$_}");
                    return $self;
                };
        }
        else
        {
            $self->_set_err("SanFiler::get_aggr_size() Invalid " .
                            "parameter specified: $_");
            return $self;
        }
    }

    unless(defined $aggr and length $aggr)
    {
        $self->_set_err("Error: SanFiler::get_aggr_size() "      .
                        "aggr name is a required argument");
        return "";
    }

    my @cmd_args;
    push(@cmd_args, "df");
    push(@cmd_args, "-A");
    push(@cmd_args, $aggr);

    $resultsHash_ref =
    $cmd_obj->execute_cmd(
                        -cmd                      => "system-cli",
                        -cmd_filer                => $self->get_name(),
                        -cmd_interface            => "zapi",
                        -check_result             => 1,
                        -args                     => \@cmd_args,
                        -priv                     => "admin");

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::get_aggr_size(): Unable to " .
                        "get aggr size: $errmsg");
        return "";
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};
    my @output = split('\n', $output);

    if($output =~ /^df: .*/)
    {
        my $errmsg = $output;
        $self->_set_err("Error: SanFiler::get_aggr_size() " .
                        "ZAPI command failed", $errmsg);
        return "";
    }

    # pull out the total space and capacity of $aggr
    # Expecting output like this:
    # Aggregate               kbytes       used      avail capacity
    # test_aggr            357104472  107860696  249243776      30%
    foreach my $line (@output)
    {
        next if ($line =~ /\/\.snapshot/);
        if ($line =~ /$aggr.+?(\d+).+?\d+.+?\d+.+?\d+\%/)
        {
            $aggr_size = $1;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $aggr_size;
    }
    else
    {
        $self->_set_err("Error: SanFiler::get_aggr_size() " .
                        "result check failed");
        return "";
    }
} # end get_aggr_size

################################################################################
# Method:    get_fcp_status
# Objective: Gets the status of FCP
# Details:   See POD documentation at the end of this file
################################################################################
sub get_fcp_status
{
    my ($self, %args) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $resultsHash_ref;
    my $f_status;

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::get_fcp_status() Get FCP status",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    $resultsHash_ref =
    $cmd_obj->execute_cmd(
                    -cmd                    => "fcp-service-status",
                    -cmd_filer              => $self->{filer},
                    -check_result           => 1);

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::get_fcp_status(): Unable to " .
                        "get fcp status: $errmsg");
        return "";
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    if($self->{interface} eq "zapi")
    {
        my $output_hash = $self->xml2hash($resultsHash_ref->{output});
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            $self->_set_err("Error: SanFiler::get_fcp_status: " .
                            "Unable to get status", $output_hash->{reason});
            return "";
        }
        else
        {
            $f_status = $output_hash->{"is-available"};

            if(not defined($f_status))
            {
                $self->_set_err("Error: SanFiler::get_fcp_status: " .
                                "Unable to get status");
                return "";
            }
        }
    }
    else
    {
        if($output =~ /fcp status:.*/)
        {
            $self->_set_err("Error: SanFiler::get_fcp_status: " .
                            "Unable to get status", $output);
            return "";
        }
        else
        {
            $f_status = $output;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $f_status;
    }
    else
    {
        $self->_set_err("Error: SanFiler::get_fcp_status: " .
                        "results check failed", $output);
        return "";
    }
} # end get_fcp_status

################################################################################
# Method:    get_iscsi_status
# Objective: Gets the status of iSCSI
# Details:   See POD documentation at the end of this file
################################################################################
sub get_iscsi_status
{
    my ($self, %args) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $resultsHash_ref;
    my $i_status;

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::get_iscsi_status() Get iSCSI status",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    $resultsHash_ref =
    $cmd_obj->execute_cmd(
                    -cmd                    => "iscsi-service-status",
                    -cmd_filer              => $self->{filer},
                    -check_result           => 1);

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::get_iscsi_status(): Unable to " .
                        "get iscsi status: $errmsg");
        return "";
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    if($self->{interface} eq "zapi")
    {
        my $output_hash = $self->xml2hash($resultsHash_ref->{output});
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            $self->_set_err("Error: SanFiler::get_iscsi_status: " .
                            "Unable to get status", $output_hash->{reason});
            return "";
        }
        else
        {
            $i_status = $output_hash->{"is-available"};

            if(not defined($i_status))
            {
                $self->_set_err("Error: SanFiler::get_iscsi_status: " .
                                "Unable to get status");
                return "";
            }
        }
    }
    else
    {
        if($output =~ /iscsi status:.*/)
        {
            $self->_set_err("Error: SanFiler::get_iscsi_status: " .
                            "Unable to get status", $output);
            return "";
        }
        else
        {
            $i_status = $output;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $i_status;
    }
    else
    {
        $self->_set_err("Error: SanFiler::get_iscsi_status: " .
                        "results check failed", $output);
        return "";
    }
} # end get_iscsi_status

###############################################################################
# Method:    set_mem_stats()
# Objective: Start mem stats.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_mem_stats()
{
    my ($self) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::set_mem_stats() Setting memory stats",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    if($self->interface() eq "cli")
    {
        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                        -cmd                    => "system-mem-stats",
                        -cmd_filer              => $self->{filer},
                        -check_result           => 1,
                        -cmd_privilege          => "diag",
                        -copy                   => "");
    }
    else
    {
        my @cmd_args;
        push(@cmd_args, "mem_stats");
        push(@cmd_args, "-b");

        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                            -cmd                      => "system-cli",
                            -cmd_filer                => $self->get_name(),
                            -cmd_interface            => "zapi",
                            -check_result             => 1,
                            -args                     => \@cmd_args,
                            -priv                     => "diag");
    }

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::set_mem_stats(): Unable to " .
                        "set mem_stats: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    if($output=~/^mem_stats:.*/i)
    {
        $self->_set_err("Error: SanFiler::set_mem_stats() unable to " .
                        "set mem_stats",$output);
        return $ClientUtil::FALSE;
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::set_mem_stats() " .
                        "result check failed");
        return $ClientUtil::FALSE;
    }
} # end set_mem_stats

###############################################################################
# Method:    get_mem_stats()
# Objective: End mem stats and get the memory utilization values between start
#            and end.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_mem_stats()
{
    my ($self) = @_;
    my $cmd_obj = $self->{cmd_obj};
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::get_mem_stats() Getting memory stats",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    if($self->interface() eq "cli")
    {
        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                        -cmd                    => "system-mem-stats",
                        -cmd_filer              => $self->{filer},
                        -check_result           => 1,
                        -cmd_privilege          => "diag",
                        -compare                => "");
    }
    else
    {
        my @cmd_args;
        push(@cmd_args, "mem_stats");
        push(@cmd_args, "-e");

        $resultsHash_ref =
        $cmd_obj->execute_cmd(
                            -cmd                      => "system-cli",
                            -cmd_filer                => $self->get_name(),
                            -cmd_interface            => "zapi",
                            -check_result             => 1,
                            -args                     => \@cmd_args,
                            -priv                     => "diag");
    }

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::get_mem_stats(): Unable to " .
                        "get mem_stats: $errmsg");
        return "";
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};
    my @output = join('\n', $output);

    if($output=~/^mem_stats:.*/i)
    {
        $self->_set_err("Error: SanFiler::get_mem_stats() unable to " .
                        "get mem_stats $output");
        return "";
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return @output;
    }
    else
    {
        $self->_set_err("Error: SanFiler::get_mem_stats() " .
                        "result check failed");
        return "";
    }
} # end get_mem_stats

###############################################################################
# Method:    console_halt
# Objective: Take a node down from the up state to the loader prompt
# Details:   See POD documentation at the end of this file.
###############################################################################
sub console_halt
{
    my ($self, %args) = @_;

    my (
        $cmd,
        $filer,
        $force,
        $passthru,
        $timeout,
        $expected_prompt,
        $received_prompt,
        $conn,
       );

    $conn = "";
    $timeout = "";
    $cmd = "";
    $force = "";
    $passthru = "";

    $self->_clear_err();

    foreach (keys %args) {
        if (/-?force/) {
            length($force = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"force\"",
                                    "$args{$_}");
                    return $ClientUtil::FALSE;
                };
        } elsif (/-?passthru/) {
            length($passthru = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"passthru\"",
                                    "$args{$_}");
                    return $ClientUtil::FALSE;
                };
        } elsif (/-?connection/) {
            length($conn = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"connection\"",
                                    "$args{$_}");
                    return $ClientUtil::FALSE;
                };
        } elsif (/-?timeout/) {
            length($timeout = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"timeout\"",
                                    "$args{$_}");
                    return $ClientUtil::FALSE;
                };
        }
    }

    if ($force == $ClientUtil::TRUE) {
        $cmd = "halt -f"
    } else {
        $cmd = "halt";
    }

    if ($passthru ne "") {
        $cmd = "$cmd $passthru";
    }

    if ($timeout eq "") {
        $timeout = 60;
    }

    $filer = $self->{name};

    # sort out the console connection to the halting filer
    if ($conn ne "") {
        # Use whatever was given
    } elsif (defined $self->{console_conn}) {
        $conn = $self->{console_conn};
    } else {
        $self->_set_err("Unable to find a console connection for \"$cmd\"");
        return $ClientUtil::FALSE;
    }

    # Bringing the controller to one of the following prompts
    $self->sanlog(-msg => "Issuing $cmd to ($filer)");
    $expected_prompt = "CFE\>|LOADER\>|LOADER\-[AB]\>|^ok|Starting AUTOBOOT|Press Del to abort boot";
    my $minutes_before_disconnect_prompt = "Enter the number of minutes to wait before disconnecting \[[0-9]+\]:";
    (undef, undef, undef, $received_prompt) =
        &Tharn::say($conn, $cmd,
                    '-noprompt', '-prompt', $expected_prompt, $minutes_before_disconnect_prompt,
                    '-timeout', $timeout,
                    '-ignore', "echo ignore",
                    '-comment', "Attempted ($cmd)");

    # if queried to wait for a specific amount of time to disconnect, answer '0' and continue.
    if ($received_prompt =~ /$minutes_before_disconnect_prompt/i) {
        (undef, undef, undef, $received_prompt) =
            &Tharn::say($conn, "0",
                        '-noprompt', '-prompt', $expected_prompt,
                        '-timeout', $timeout,
                        '-ignore', "echo ignore",
                        '-comment', "Attempted ($cmd)");
        if ($received_prompt !~ /$expected_prompt/) {
            $self->_set_err("Command ($cmd) was not executed correctly.");
            return $ClientUtil::FALSE;
        }
    }

    # if the prompt isnt what we expect at this point, then error out
    if ($received_prompt !~ /$expected_prompt/) {
        $self->_set_err("Command ($cmd) was not executed correctly.");
        return $ClientUtil::FALSE;
    }

    # On some controllers, AUTOBOOT may be enabled. In this case, cancel the autoboot
    # by sending a Ctrl-C over the console.
    if ($received_prompt =~ /Starting AUTOBOOT/i) {
        $self->sanlog(-msg => "($filer) has AUTOBOOT turned on, sending a Ctrl-C to cancel it.");
        $expected_prompt = "CFE\>|LOADER\>|LOADER\-[AB]\>|^ok";
        (undef, undef, undef, $received_prompt) =
            &Tharn::say($conn, "\cc",
                        '-noprompt', '-prompt', $expected_prompt,
                        '-timeout', $timeout,
                        '-ignore', "echo ignore",
                        '-comment', "Attempted Ctrl-C");
        if ($received_prompt !~ /$expected_prompt/) {
            $self->_set_err("Command ($cmd) was not executed correctly.");
            return $ClientUtil::FALSE;
        }
    }

    # On 900 series halt_filers with OFP loader (ok boot prompt), need to prevent AUTOBOOT.
    # If we see the string "Press Del to abort boot, Esc to skip POST", we need to send
    # a delete keystroke (Ctrl-? => "\c?") over the console
    if ($received_prompt =~ /Press Del to abort boot/i) {
        $self->sanlog(-msg => "($filer) has AUTOBOOT turned on, sending a Ctrl-? (Delete) to cancel it.");
        $expected_prompt = "CFE\>|LOADER\>|LOADER\-[AB]\>|^ok";
        (undef, undef, undef, $received_prompt) =
            &Tharn::say($conn, "\c?",
                        '-noprompt', '-prompt', $expected_prompt,
                        '-timeout', $timeout,
                        '-ignore', "echo ignore",
                        '-comment', "Attempted Ctrl-? (Delete)");
        if ($received_prompt !~ /$expected_prompt/) {
            $self->_set_err("Could not cancel autoboot.");
            return $ClientUtil::FALSE;
        }
    }

    # it can happen that we matched the boot prompt but it turned out to be a false positive
    # send an empty message over the console to see if we can get the prompt again. Try this
    # 3 times to make sure.
    for (1..3) {
        $expected_prompt = "CFE\>|LOADER\>|LOADER\-[AB]\>|^ok";
        (undef, undef, undef, $received_prompt) =
            &Tharn::say($conn, "\n",
                        '-noprompt', '-prompt', $expected_prompt,
                        '-timeout', $timeout,
                        '-ignore', "echo ignore",
                        '-comment', "Attempted to get loader prompt again.");
    }

    return $ClientUtil::TRUE;
} # end console_halt

###############################################################################
# Method:    boot_ontap
# Objective: Boot a node from the loader prompt to $boot_to_prompt
# Details:   See POD documentation at the end of this file.
###############################################################################
sub boot_ontap
{
    my ($self, %args) = @_;

    my (
        $boot_cmd,
        $filer,
        $timeout,
        $expected_prompt,
        $received_prompt,
        $conn,
        $boot_to_prompt,
       );

    $conn = "";
    $timeout = "";
    $boot_cmd = "";
    $boot_to_prompt = "";

    $self->_clear_err();

    foreach (keys %args) {
        if (/-?boot_to_prompt/) {
            length($boot_to_prompt = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"boot_to_prompt\"",
                                    "$args{$_}");
                    return $ClientUtil::FALSE;
                };
        } elsif (/-?boot_command/) {
            length($boot_cmd = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"boot_command\"",
                                    "$args{$_}");
                    return $ClientUtil::FALSE;
                };
        } elsif (/-?connection/) {
            length($conn = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"connection\"",
                                    "$args{$_}");
                    return $ClientUtil::FALSE;
                };
        }  elsif (/-?timeout/) {
            length($timeout = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"timeout\"",
                                    "$args{$_}");
                    return $ClientUtil::FALSE;
                };
        }
    }

    if ($timeout eq "") {
        $timeout = 60;
    }

    $filer = $self->{name};

    # Check the console connection
    if ($conn ne "") {
        # Use whatever was given
    } elsif (defined $self->{console_conn}) {
        $conn = $self->{console_conn};
    } else {
        $self->_set_err("Unable to find a console connection to \"$boot_cmd\"");
        return $ClientUtil::FALSE;
    }

    # Get the current prompt and make sure it is an accepatble fw prompt
    $expected_prompt = "CFE\>|LOADER\>|LOADER\-[AB]\>|^ok";
    for (1..3) {
        (undef, undef, undef, $received_prompt) =
            &Tharn::say($conn, "\n",
                        '-noprompt', '-prompt', $expected_prompt,
                        '-timeout', $timeout,
                        '-ignore', "echo ignore",
                        '-comment', "Attempted to get loader prompt again.");
        &SanUtil::nate_sleep(1);
    }

    # Boot the halted filer from the CFE/LOADER prompt
    $self->sanlog(-msg => "Booting ONTAP on ($filer)");

    if ($boot_cmd eq "") {
        $boot_cmd = "boot_ontap";
        $boot_cmd .= "; go" if ($received_prompt =~ /CFE\>/);
        # All OFW prompts dont have the boot_ontap command
        $boot_cmd = "boot" if ($received_prompt =~ /^ok/ and $received_prompt !~ /CFE\>|LOADER\>|LOADER\-[AB]\>/);
    }

    $expected_prompt = $boot_to_prompt;
    (undef, undef, undef, $received_prompt) =
        &Tharn::say($conn, $boot_cmd,
                    '-noprompt', '-prompt', $expected_prompt,
                    '-timeout', $timeout,
                    '-ignore', "echo ignore",
                    '-comment', "Attempted ($boot_cmd)");
    if ($received_prompt !~ /$expected_prompt/) {
        $self->_set_err("Command ($boot_cmd) was not executed correctly.");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
} # end boot_ontap

###############################################################################
# Method:    force_reboot()
# Objective: Force reboot (reboot -r) on the filer. This is to simulate bad
#            shutdowns.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub force_reboot
{
    my ($self) = @_;
    my (
        $cmd,
        $errmsg,
       );

    $self->_clear_err();

    my $rebootFiler = $self->get_name();
    $self->sanlog(-msg => "SanFiler::force_reboot Force reboot " .
                          "the filer $rebootFiler",
                  -msg_level => 1);
                  #-msg_level => LOG_VERBOSE_LOW);

    $cmd = "priv set advanced;reboot -r";

    #
    # Send "reboot -r" and change the prompt to be the question returned
    # by the filer.  Once we see this prompt, send "y".  Once the prompt
    # changes, then we're done.
    #
    my $new_connection = &Tharn::connect($rebootFiler . "#telnet");

    my $rebootPrompt = "Are you sure you want to do this\\?";
    my (undef, undef, undef, $prompt) = &Tharn::say($new_connection,
        'reboot -r',
        '-prompt',
        $rebootPrompt);

    #
    # Execute the blind say command 5 times, this takes a total of 15
    # seconds including the sleeps.
    #
    if ($prompt =~ $rebootPrompt)
    {
        for (1..5)
        {
            eval { &Tharn::say($new_connection, 'y', '-blind'); };

            last if $@; # safeguard to save time if the say() fails

            &SanUtil::nate_sleep($_);  # progressively sleep longer
        }

    }
    else
    {
        Tharn::logcomment("Failed to receive prompt: $rebootPrompt");
        &Tharn::disconnect($new_connection);
        return $ClientUtil::FALSE;
    }

    #
    # Reconnect here
    #
    &Tharn::disconnect($new_connection);
    $self->reconnect();
    return $ClientUtil::TRUE;
} # end force_reboot

###############################################################################
# Method:    shutdown
# Objective: Shutdown the controller
# Details:   See POD documentation at the end of this file.
###############################################################################
sub shutdown {
    my ($self) = @_;

    my ($cmd, @output, $status);

    $self->_clear_err();

    $self->cf_disable();
    # Don't care what the status is, just ignore it
    # Clear the error stack
    $self->_clear_err();

    # Grab the cf status and see if the cluster is up and running
    @output = $self->cf_status();
    if (grep(/Cluster enabled|Controller Failover enabled/i, @output) == 0 or $self->error()) {
        # If the cluster is not up, shutdown will fail
        $self->_set_err("Unable to disable cluster service");
        return $ClientUtil::FALSE;
    }

    # Halt the node
    $cmd = "halt -t0";
    @output = &Tharn::say($self->{connection}, $cmd, '-blind');

    # Invalidate the connection on the object
    &Tharn::disconnect($self->{connection}, '-ok');

    # sleep 30 seconds to make sure shutdown occurs
    &SanUtil::nate_sleep(30);

    # Ping the host every 10 seconds until it has completely shutdown
    my $alive = 1;

    for (my $cnt = 0; $cnt < 36; $cnt++) {
        my $status = &SanUtil::ping_host($self->hostname());
        if ($status == $ClientUtil::FALSE) {
            $self->sanlog(-msg => "ONTAP is shutdown");
            $alive = 0;
            last;
        } else {
            $self->sanlog(-msg => "Ping was successful",
                          -msg_level => 7);
            &SanUtil::nate_sleep(10);
        }
    }

    # Machine is still alive, shutdown did not work
    if ($alive) {
        $self->_set_err("ONTAP never shutdown");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    reboot
# Objective: Reboot the controller
# Details:   See POD documentation at the end of this file.
###############################################################################
sub reboot {
    my ($self) = @_;

    my ($cmd, @output, $status);

    $self->_clear_err();

    # Reboot the node
    $cmd = "reboot -t0";
    @output = &Tharn::say($self->{connection}, $cmd, '-blind');

    # Invalidate the connection on the object
    &Tharn::disconnect($self->{connection}, '-ok');

    # Ping the host every 10 seconds until it has completely shutdown
    my $alive = 1;

    for (my $cnt = 0; $cnt < 36; $cnt++) {
        my $status = &SanUtil::ping_host($self->{name});
        if ($status == $ClientUtil::FALSE) {
            $self->sanlog(-msg => "ONTAP is shutdown");
            $alive = 0;
            last;
        } else {
            $self->sanlog(-msg => "Ping was successful",
                          -msg_level => 7);
            &SanUtil::nate_sleep(10);
        }
    }

    # sleep 30 seconds to make sure reboot occurs
    &SanUtil::nate_sleep(30);

    # Ping the host every 10 seconds until it has rebooted
    $alive = 0;

    for (my $cnt = 0; $cnt < 36; $cnt++) {
        my $status = &SanUtil::ping_host($self->{name});
        if ($status == $ClientUtil::FALSE) {
            $self->sanlog(-msg => "ONTAP is shutdown",
                          -msg_level => 7);
            &SanUtil::nate_sleep(10);
        } else {
            $self->sanlog(-msg => "Ping was successful",
                          -msg_level => 7);
            $alive = 1;
            last;
        }
    }

    # Should we do an uptime here just to confirm the reboot?

    # Machine is not alive, reboot did not worked
    if (!$alive) {
        $self->_set_err("ONTAP was not rebooted");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    options_cf_takeover_on_panic
# Objective: turn on and off cf_takeover_on_panic
# Details:   See POD documentation at the end of this file.
###############################################################################
sub options_cf_takeover_on_panic
{
    my ($self, $pass_thru) = @_;
    my (
        $cmd,
        @output,
        );

    $cmd="options cf.takeover.on_panic ".$pass_thru;

    @output=$self->execute($cmd,-hide=>1);

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    options_cf_takeover_on_failure
# Objective: turn on and off cf_takeover_on_failure
# Details:   See POD documentation at the end of this file.
###############################################################################
sub options_cf_takeover_on_failure
{
    my ($self, $pass_thru) = @_;
    my $cmd_obj = $self->{cmd_obj};

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::options_cf_takeover_on_failure() " .
                          "Set option: cf.takeover.on_failure",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    my $resultsHash_ref =
    $cmd_obj->execute_cmd(
                        -cmd                    => "options-set",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -name                   => "cf.takeover.on_failure",
                        -value                  => $pass_thru);

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::options_cf_takeover_on_panic(): " .
                        "Unable to set option: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my @output = split("\n", $resultsHash_ref->{output});

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->{interface} eq "zapi")
    {
        my $output_hash = $self->xml2hash($resultsHash_ref->{output});
        # my $dumpOut = Dumper($output_hash);
        # $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
        #               -msg_level => 7);
        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            $self->_set_err("Error: SanFiler::options_cf_takeover_on_panic: " .
                            "Unable to set option", $output_hash->{reason});
            return $ClientUtil::FALSE;
        }
    }
    else
    {
        if("@output" =~ /options:/)
        {
            $self->_set_err("Error: SanFiler::options_cf_takeover_on_panic: " .
                            "Unable to set option", @output);
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::options_cf_takeover_on_failure: " .
                        "results check failed");
        return $ClientUtil::FALSE;
    }
}

###############################################################################
# Method:    options_cf_giveback_auto_enable
# Objective: turn on and off cf.giveback.auto.enable.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub options_cf_giveback_auto_enable
{
    my ($self, $pass_thru) = @_;
    my (
        $cmd,
        @output,
        );

    $cmd="options cf.giveback.auto.enable ".$pass_thru;

    @output=$self->execute($cmd,-hide=>1);

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    options_lun_enospc_timeout
# Objective: Set lun.enospc.timeout in minute
# Details:   See POD documentation at the end of this file.
###############################################################################
sub options_lun_enospc_timeout
{
    my ($self, $timeout) = @_;

    if ($timeout !~ /\d+/ and ($timeout < 1 or $timeout > 360)) {
        $self->_set_err("Error: SanFiler::options_lun_enospc_timeout(): " .
          "Lun enospc timeout must between 1 and 360 minutes: $timeout");
        return $ClientUtil::FALSE;
    }

    my $cmd = "options lun.enospc.timeout ".$timeout;
    my @output = $self->execute($cmd,-hide=>1);
    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    cf_enable
# Objective: Enable cluster failover.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub cf_enable
{
    my ($self, $pass_thru) = @_;
    my (
        $cmd,
        @output,
        );

    $cmd="cf enable ".$pass_thru;

    @output=$self->execute($cmd,-hide=>1);

    foreach my $line (@output)
    {
        if ($line =~ /clustering already enabled|/)
        {
            last;
        }
        elsif ( $line =~ /cf: /)
        {
            $self->_set_err("Unexpected error attempting to enable cluster failover");
            return $ClientUtil::FALSE;
        }
   }

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    cf_disable
# Objective: Disable cluster fail over.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub cf_disable
{
    my ($self, $pass_thru) = @_;
    my (
        $cmd,
        @output,
        );

    $cmd="cf disable ".$pass_thru;

    @output=$self->execute($cmd,-hide=>1);

    foreach my $line (@output)
    {
        if ($line =~ /clustering already enabled|/)
        {
            last;
        }
        elsif ( $line =~ /cf: /)
        {
            $self->_set_err("Unexpected error attempting to disable cluster failover");
             return $ClientUtil::FALSE;
        }
    }

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    cf_status
# Objective: Check cluster failover status.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub cf_status
{
    my ($self, $pass_thru) = @_;
    my (
        $cmd,
        @output,
        $cfstatus,
        );

    $cmd="cf status ".$pass_thru;

    @output=$self->execute($cmd,-hide=>1);

    unless (defined $output[0] and length $output[0]){
        $self->_set_err("Error: SanFiler::cf_status(): Unable to get cf status");
        return "";
    }

    $cfstatus=join("\n",@output);

    return (wantarray()) ? @output : $cfstatus;
}

###############################################################################
# Method:    cluster_initialize
# Objective: Initialize the system for use in a cluster
# Details:   See POD documentation at the end of this file.
###############################################################################
sub cluster_initialize
{
    my ($self) = @_;
    my $sleeptime = 0;
    my $code;

    $self->_clear_err();

    # Make sure that the system is configured in a cluster
    my $cmd="sysconfig -v";

    my @output=$self->execute($cmd,-hide=>1);

    unless (defined $output[0] and length $output[0]){
        $self->_set_err("Unable to get sysconfig");
        return $ClientUtil::FALSE;
    }

    my $line=join("\n",@output);

    my ($partner)=$line=~/partner ID:\s+(\d+)\s+/;

    unless (defined $partner and length $partner){
        $self->_set_err("System does not have a partner, not in a cluster");
        return $ClientUtil::FALSE;
    }


    ### Adding the below condition since there is no license required for the release greater than 8.1
    ### It is being added as an option, cf.mode
    ### BURT 537249
    my $version = $self->get_version_major();
    if($version gt "8.1") {
        return $ClientUtil::TRUE;
    }


    # License the cluster service
    my $avail_licenses = &SanUtil::get_avail_controller_licenses();
    if (defined $avail_licenses) {
        $code = $avail_licenses->{cluster};
        unless (defined $code) {
            $self->sanlog(-msg => "Unable to obtain cluster service license ".
                                  "from".
                                  "SanUtil::get_avail_controller_licenses");
            if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
                my $dumpOut = Dumper($avail_licenses);
                $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                              -msg_level => DUMPER_DEBUG_LEVEL);
            }
            return $ClientUtil::FALSE;
        }
    } else {
        $self->sanlog(-msg => "Unable to obtain license information from ".
                              "SanUtil::get_avail_controller_licenses");
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($avail_licenses);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }
        return $ClientUtil::FALSE;
    }

    if (! $self->license($code)) {
        return $ClientUtil::FALSE;
    }

    # Enable cluster failover
    if (! $self->cf_enable()) {
        return $ClientUtil::FALSE;
    }

    # Get the cf status, issue several retries if we are in the midst of takeover
    my $count = 0;
    while (1) {
        sleep(15);
        @output = $self->cf_status();
        if (grep(/waiting for partner to recover|unsynchronized log/, @output)) {
            # We cannot attempt giveback yet
            $count++;
            if ($count>=10) {
                $self->_set_err("Giveback failed");
                return $ClientUtil::FALSE;
            }
        } else {
            last;
        }
    }

    # Get the cf status, issue a giveback if needed
    @output = $self->cf_status();
    foreach my $line (@output) {
        if ($line =~ /Cluster enabled.* is up|Controller Failover enabled|takeover disabled because of reason |has disabled takeover by /) {
            # We don't need to do anything, we are not in takeover
        } elsif ($line =~ /attempting takeover| has taken over/) {
            # This system has taken over the partner, issue a giveback
            $self->giveback();
            while ($sleeptime <= 1800) {
                sleep (30);
                if ($self->is_giveback_complete()) {
                     last;
                }
                $sleeptime += 30;
            }
            if ($sleeptime >= 1800) {
                $self->_set_err("Giveback did not complete in the alloted amount of time");
                return $ClientUtil::FALSE;
            }
        } elsif ($line =~ / has been taken over by | is ready for giveback|Waiting for giveback/) {
            # This system has been taken over by the partner, the partner needs to issue a giveback
        }
    }

    return $ClientUtil::TRUE;

}

###############################################################################
# Method:    sysstat
# Objective: Get filer performance statistics
# Details:   See POD documentation at the end of this file.
###############################################################################
sub sysstat
{
    my ($self, $pass_thru) = @_;
    my (
        $cmd,
        $str,
        @output,
        );

    if(grep /interval/, $pass_thru)
    {
        $self->_set_err("Error: SanFiler::sysstat invalid argument (interval) for this method");
        return $ClientUtil::FALSE;
    }

    $cmd="sysstat -c1 ".$pass_thru;

    @output=$self->execute($cmd,-hide=>1);

    if(grep /unrecognized/, @output or grep /usage/, @output)
    {
        $self->_set_err("Error: SanFiler::sys_stat invalid argument",@output);
        return $ClientUtil::FALSE;
    }

    $str = join("\n",@output);

    return (wantarray()) ? @output : $str;
}

###############################################################################
# Method:    lun_stats
# Objective: Gets filer lun statistics
# Details:   See POD documentation at the end of this file.
###############################################################################
sub lun_stats
{
    my ($self, $pass_thru) = @_;
    my (
        $cmd,
        $str,
        @output,
        );

    if(grep /i/, $pass_thru)
    {
        $self->_set_err("Error: SanFiler::lun_stats invalid argument -i for this method");
        return $ClientUtil::FALSE;
    }

    $cmd="lun stats ".$pass_thru;
    @output=$self->execute($cmd,-hide=>1);

    if(grep /unrecognized/, @output or grep /usage/, @output)
    {
        $self->_set_err("Error: SanFiler::lun_stats invalid arguments",@output);
        return $ClientUtil::FALSE;
    }

    $str = join("\n",@output);

    return (wantarray()) ? @output : $str;
}

###############################################################################
# Method:    partner lun stats
# Objective: Gets filer lun statistics
# Details:   See POD documentation at the end of this file.
###############################################################################
sub partner_lun_stats
{
    my ($self, $pass_thru) = @_;
    my (
        $cmd,
        $str,
        @output,
        );

    if(grep /i/, $pass_thru)
    {
        $self->_set_err("Error: SanFiler::lun_stats invalid argument -i for this method");
        return $ClientUtil::FALSE;
    }

    $cmd="partner lun stats ".$pass_thru;

    @output=$self->execute($cmd,-hide=>1);

    if(grep /unrecognized/, @output or grep /usage/, @output)
    {
        $self->_set_err("Error: SanFiler::partner_lun_stats invalid arguments",@output);
        return $ClientUtil::FALSE;
    }

    $str = join("\n",@output);

    return (wantarray()) ? @output : $str;
}

###############################################################################
# Method:    ic_disable_link
# Objective: Execute "ic disable link"
# Details:   See POD documentation at the end of this file.
###############################################################################
sub ic_disable_nic
{
    my ($self, $nic) = @_;
    my (
        $cmd,
        @output,
        );

    unless (defined $nic and length $nic){
        $self->_set_err("Error: SanFiler::ic_disable_nic() Missing required argument 'nic'");
        return (wantarray()) ? () : [];
    }

    $cmd="priv set test;ic disable nic $nic";

    @output=$self->execute($cmd,-hide=>1);

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    ic_enable_link
# Objective: Execute "ic enable link"
# Details:   See POD documentation at the end of this file.
###############################################################################
sub ic_enable_nic
{
    my ($self, $nic) = @_;
    my (
        $cmd,
        @output,
        );

    unless (defined $nic and length $nic){
        $self->_set_err("Error: SanFiler::ic_enable_nic() Missing required argument 'nic'");
        return (wantarray()) ? () : [];
    }

    $cmd="priv set test;ic enable nic $nic";

    @output=$self->execute($cmd,-hide=>1);

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    priv_available
# Objective: to find out if given priv level is available on the filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub priv_available
{
    my ($self, $level) = @_;
    my $text_expected = "Warning: These";
    my $cmd;
    my @output;

    $cmd="priv set $level";

    @output=$self->execute($cmd,-hide=>1);

    #
    # If you are going from a higher priv level to a lower (like from test to admin), the
    # command will succeed and give no output
    #
    unless (length($output[0])) {
        return $ClientUtil::TRUE;
    }

    #
    # If you are goign from a lower priv level to a highter (like from advanced to test),
    # you will be given a warning.
    #
    if($output[0] =~ /$text_expected/i){
        return $ClientUtil::TRUE;
    }else{
        return $ClientUtil::FALSE;
    }
} # end priv_available



###############################################################################
# Method:    ic_reset_link
# Objective: Execute "ic reset link"
# Details:   See POD documentation at the end of this file.
###############################################################################
sub ic_reset_link
{
    my ($self) = @_;
    my (
        $cmd,
        @output,
        );

    $cmd="priv set test;ic reset link";

    @output=$self->execute($cmd,-hide=>1);

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    ic_reset_NIC
# Objective: Execute "ic reset nic"
# Details:   See POD documentation at the end of this file.
###############################################################################
sub ic_reset_NIC
{
    my ($self, $nic) = @_;
    my (
        $cmd,
        @output,
        );

    unless (defined $nic and length $nic){
        $self->_set_err("Error: SanFiler::ic_reset_NIC() Missing required argument 'nic'");
        return (wantarray()) ? () : [];
    }

    $cmd="priv set test;ic reset nic $nic";

    @output=$self->execute($cmd,-hide=>1);

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    ic_stats_error
# Objective: Gets ic statistics/status
# Details:   See POD documentation at the end of this file.
###############################################################################
sub ic_stats_error
{
    my ($self, $pass_thru) = @_;
    my (
        $cmd,
        $str,
        @output,
        );

    $cmd="priv set test;ic stats error ".$pass_thru;
    @output=$self->execute($cmd,-hide=>1);

    if(grep /unrecognized/, @output or grep /usage/, @output)
    {
        $self->_set_err("Error: SanFiler::ic_stats_error invalid arguments",@output);
        return $ClientUtil::FALSE;
    }

    $str = join("\n",@output);

    return (wantarray()) ? @output : $str;
}

###############################################################################
# Method:     ic status
# Objective: Gets interconnect status
# Details:   See POD documentation at the end of this file.
###############################################################################
sub ic_status
{
    my ($self) = @_;
    my (
        $cmd,
        $str,
        @output,
        );

    $cmd="priv set test;ic status";

    @output=$self->execute($cmd,-hide=>1);

    if ($self->error()){
        my $errmsg = join "\n", @output;
        $self->_set_err("SanFiler::ic_status(): $errmsg");
        return $ClientUtil::FALSE;
    }

    $str = join("\n",@output);
    return (wantarray()) ? @output : $str;
}

###############################################################################
# Method:    connection
# Objective: Get/Sets the filer connection. If we are given a value, use it as
#            the connection.  Always return the connection attribute.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub connection
{
    my ($self, $value) = @_;
    if (defined($value))
    {
        $self->{connection} = $value;

        # Set CmdsDriver connection
        my $cmd_obj = $self->cmd_obj();
        $cmd_obj->connection($value);

        $self->sanlog(-msg       => ref($self) . "::" .
                                    "connection (".$self->{conn_type}.") set",
                      -msg_level => 3);
    }
    return $self->{connection};
}

###############################################################################
# Method:    reconnect
# Objective: Creates a new filer connection. This is needed for sitiations
#            like post take/give where a SanFiler object exists but will have
#            a stale connection.  Returns the connection attribute.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub reconnect
{
    my ($self, %args) = @_;

    # dont disconnect the existing connection by default
    my $disconnect_existing_conn = exists($args{-disconnect_existing_conn}) ?
                                   $args{-disconnect_existing_conn} : 0;
    if ($disconnect_existing_conn == 1) {
        Tharn::disconnect($self->{connection}, '-ok');
    }

    my $filer   = $self->get_name();
    my $timeout = $self->{timeout};
    $self->{connection}=$self->_connect($filer, $timeout);
    if ($self->error){
        $self->_set_err("SanFiler::reconnect(): Failed to connect to $filer");
    }
    # Set CmdsDriver connection
    my $cmd_obj = $self->cmd_obj();
    $cmd_obj->connection($self->{connection});
    return $self->{connection};
}

###############################################################################
# Method:    set_vol0_default_exports
# Objective: Sets default /etc/exports vol0 permissions on filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_vol0_default_exports
{
    my ($self) = @_;

    my @lines;

    my $cmd="priv set test; rdfile /etc/exports";
    my @output=$self->execute($cmd);
    if ( $self->error() ){
        $self->_set_err("Failure reading /etc/exports file; ",@output);
        return $SanUtil::FALSE;
    }

    my $found_vol0_entry = 0;
    my $found_vol0_home_entry = 0;
    foreach my $line (@output) {
        if( $line =~ /^\/vol\/vol0\s*$/ ) {
            $found_vol0_entry=1;
        }

        if( $line =~ /^\/vol\/vol0\/home\s*$/ ) {
            $found_vol0_home_entry=1;
        }
    }


    # If we didn't find an entry for /vol/vol0, we will add a default line
    unless ( $found_vol0_entry ){
        push(@lines,"/vol/vol0       -sec=sys,rw,anon=0,nosuid");
    }

    # If we didn't find an entry for /vol/vol0/home, we will add a default line
    unless( $found_vol0_home_entry ){
        push(@lines,"/vol/vol0/home  -sec=sys,rw,nosuid");
    }


    # If found an entry to add, write new entries to the file
    unless ( $found_vol0_entry || $found_vol0_home_entry ) {
        foreach my $line (@lines) {
            $cmd='wrfile -a /etc/exports "' . $line . '"';
            $self->execute($cmd);
            if ( $self->error() ){
                $self->_set_err("Failure running wrfile -a /etc/exports");
                return $SanUtil::FALSE;
            }
        }

        # Running exportfs -a to export newly added entries
        $cmd="priv set test; exportfs -a";
        $self->execute($cmd);
        if ( $self->error() ){
           $self->_set_err("Failure running exportfs -a");
           return $SanUtil::FALSE;
        }
    }

    return $SanUtil::TRUE;
}

################################################################################
# Subroutine Name: ntp_sync()
# Objective:       Sync's the Controller to specified time server
# Details:         See POD documentation at the end of this file
################################################################################
sub ntp_sync
{
    my ($self, %args) = @_;

    my ($cmd, @output, $ntp_server, $ntp_time_zone);
    my ($option, @options);

    $self->_clear_err();

    foreach (keys %args) {
        if (/-?ntp_server/) {
            length($ntp_server = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::ntp_sync invalid value for \"ntp_server\":",
                                    "$args{$_}");
                    return $ClientUtil::FALSE;
                };
        } elsif (/-?ntp_time_zone/) {
            length($ntp_time_zone = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::ntp_sync invalid value for \"ntp_time_zone\":",
                                    "$args{$_}");
                    return $ClientUtil::FALSE;
                };
        }
    }

    # check for India (NB) and proper timezone format
    if ($ntp_time_zone =~ /IST\-5\:30/i) {
        $ntp_time_zone = "Asia\/Calcutta";
    }

    unless(defined $ntp_server and length $ntp_server) {
        $self->_set_err("ERROR: SanFiler::ntp_sync() Required parameter \$ntp_server required");
        return $ClientUtil::FALSE;
    }

    unless(defined $ntp_time_zone and length $ntp_time_zone) {
        $self->_set_err("ERROR: SanFiler::ntp_sync() Required parameter \$ntp_time_zone required");
        return $ClientUtil::FALSE;
    }

    # use rdate to sync to a "close" time before setting ntp
    # need to turn off timed before doing this and re-enable on after running ntpdate
    $cmd = "options timed.enable off ; rdate $ntp_server ; options timed.enable on";
    @output = $self->execute($cmd, -hide => 1);
    if (grep(/Error:/i, @output) > 0) {
        $self->_set_err("ERROR: Error executing command ($cmd)");
        return $ClientUtil::FALSE;
    }

    # all option commands to enable NTP
    @options = ("timed.servers $ntp_server",
                "timed.proto ntp",
                "timed.sched 1h",
                "timed.window 15s",
                "timed.window 15s",
                "timed.enable on");

    # execute options on controller
    foreach $option (@options) {
        $cmd = "options $option";
        @output = $self->execute($cmd, -hide => 1);
        if ($self->error() or (grep(/Error:/i, @output) > 0)) {
            $self->_set_err("ERROR: Error setting option ($option)");
            return $ClientUtil::FALSE;
        }
    }

    # set the timezone
    $cmd = "timezone $ntp_time_zone";
    @output = $self->execute($cmd, -hide => 1);
    if (grep(/Error:/i, @output) > 0) {
        $self->_set_err("ERROR: Error executing command ($cmd)");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;

}

###############################################################################
# Method:    license
# Objective: Licenses the supplied service if not already done.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub license
{
    my ($self, $license) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    unless (defined ($license) and length ($license)) {
        $self->_set_err("Required parameter \$license not supplied");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "license-add",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -code                   => $license);

    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to add license $license: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    my $errmsg;
    if($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }
    } else {
        # Example of new rr_san license output
        #ssan-3140-9b> license
        #                 a_sis ENABLED
        #                    cf site KDQLGBN
        #             cf_remote not licensed
        #                  cifs site GTLYRVH
        #           compression ENABLED
        #     disk_sanitization ENABLED
        #                   fcp site ICRNCZG
        #            flex_clone site QNFYRVH
        #            flex_scale not licensed
        #         flexcache_nfs ENABLED
        #                  http ENABLED
        #     hybrid_aggregates not licensed
        #                 iscsi site IKVAREM
        #            multistore ENABLED
        #      nearstore_option ENABLED
        #                   nfs site UYTCNCG
        #    operations_manager ENABLED
        #                 pamii ENABLED
        #    persistent_archive ENABLED
        #    protection_manager ENABLED
        #  provisioning_manager ENABLED
        #              smdomino not licensed
        #                 smsql not licensed
        #        snapdrive_unix not licensed
        #     snapdrive_windows not licensed
        #              snaplock not licensed
        #   snaplock_enterprise not licensed
        #    snapmanager_hyperv not licensed
        #    snapmanager_oracle not licensed
        #       snapmanager_sap not licensed
        #snapmanager_sharepoint not licensed
        #   snapmanagerexchange not licensed
        #            snapmirror site UUNLDQC
        #       snapmirror_sync ENABLED
        #             snapmover ENABLED
        #           snaprestore site EOYPBIL
        #         snapvalidator site GDFFMLK
        #      storage_services ENABLED
        #          sv_linux_pri ENABLED
        #          sv_ontap_pri not licensed
        #          sv_ontap_sec not licensed
        #           sv_unix_pri ENABLED
        #             sv_vi_pri ENABLED
        #    sv_windows_ofm_pri ENABLED
        #        sv_windows_pri ENABLED
        #      syncmirror_local not licensed
        #              v-series not licensed
        #                   vld ENABLED
        # Example of license add output for pre-enabled servides as of rr_san
        #ssan-3140-9b> license add SPQLGBN
        #license add: "multistore" is included and no license code is necessary.
        if($output !~ /license has been installed/i) {
            if ($output =~
            /is\s+included\s+and\s+no\s+license\s+code\s+is\s+necessary/){
                $self->sanlog(-msg => "License add exception: $output");
                return $ClientUtil::TRUE;
            } else {
                $self->_set_err("could not add license $license on the filer:" .
                            " $output");
                return $ClientUtil::FALSE;
            }
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed");
        return $ClientUtil::FALSE;
    }
} # end license()


###############################################################################
# Method:    add_license
# Objective: Licenses the given service
# Details:   See POD documentation at the end of this file.
###############################################################################
sub add_license
{
    my ($self, $service_string,$type) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $license_code;
    my $major_version = $self->get_version_major();
    my $license_check;
    my $interface = $self->interface();
    ## Converting the given service to lower case, to support Fcp, FCP, fcp etc format.
    my $service = lc($service_string);
    #validate type either site or demo
    #even prefer validating the service

    $self->_clear_err();

    unless (defined ($service) and length ($service)) {
        $self->_set_err("Required parameter \$service not supplied");
        return $ClientUtil::FALSE;
    }

    # v2 license has two different types, one of them is site and the other is demo
    # validating the type and this should be either site or demo
    if (defined ($type) and length ($type)) {

        if (!(($type=~/site/) or ($type=~/demo/))){
            $self->_set_err("The parameter \$type is given an invalid value");
            return $ClientUtil::FALSE;
        }
        }

    #
    # Get all available licenses from standard file defined by SanUtil
    #
        my $avail_licenses = &SanUtil::get_avail_controller_licenses($major_version);

    #
    # Check to make sure that we were given licenses and that
    # required license is found in the standard file defined by SanUtil.
    #
    if (defined $avail_licenses) {
        #
        #If we looked up and found the license code, save it for use.
        #
        if($major_version ge "8.2"){
            if(!defined $type) {
                $type="site";
            }
            $license_code = $avail_licenses->{$service}->{$type};
        }
        else {
            $license_code = $avail_licenses->{$service};
        }
        unless (defined $license_code) {
            $self->sanlog(-msg => "Unable to obtain license information from ".
                              "SanUtil::get_avail_controller_licenses");
            if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
                my $dumpOut = Dumper($avail_licenses);
                $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                              -msg_level => DUMPER_DEBUG_LEVEL);
            }
            return $ClientUtil::FALSE;
        }
    } else {
        $self->sanlog(-msg => "Unable to obtain license information from ".
                              "SanUtil::get_avail_controller_licenses");
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($avail_licenses);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }
        return $ClientUtil::FALSE;
    }

    if($major_version ge "8.2"){
        $license_check=$self->license_exist( -service => $service);
    }
    else {
        $license_check=$self->license_exist( -license => $license_code);
    }

    if($license_check) {
        $self->sanlog(-msg=>"The service $service is present in the filer");
        my $unlicense_check=$self->unlicense($service);
        if($unlicense_check) {
            $self->sanlog(-msg=>"Unlicensing the service $service is successful");
        }
        else {
            $self->sanlog(-msg=>"Unlicensing the service $service failed");
            return $ClientUtil::FALSE;
        }

    } else {
        $self->sanlog(-msg=>"The service $service is not being licensed and adding the same");
    }

    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "license-add",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -interface                 => $interface,
                        -code                   => $license_code);

    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to add license $license_code for $service: $errmsg");
        return $ClientUtil::FALSE;
    }
    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};
    if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($output);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut"
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }
    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    my $errmsg;
    if($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut"
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }
    } else {
        #license add: "multistore" is included and no license code is necessary.

        if(($output !~ /license has been installed/i)|| ($output !~ /successfully added license key/)) {
            if ($output =~
            /is\s+included\s+and\s+no\s+license\s+code\s+is\s+necessary/){
                $self->sanlog(-msg => "License add exception: $output");
                return $ClientUtil::TRUE;
            }
        }
        else {
                $self->_set_err("could not add license $license_code for $service on the filer:" .
                            " $output");
                return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed");
        return $ClientUtil::FALSE;
    }
} # end add_license()

###############################################################################
# Method:    license_exist
# Objective: Returns whether or not a filer service is licensed
# Details:   See POD documentation at the end of this file.
###############################################################################
sub license_exist
{
    my ($self, %args) = @_;
    my $cmd_obj = $self->cmd_obj();
    my ($license,$service);
    my $interface = $self->interface();
    my $output_hash;
    my %cmd_args={};
    my $major_version = $self->get_version_major();

    $self->_clear_err();

    #
    # Parse arguments
    #
    foreach (keys %args) {
        if (/-?license/) {
            if($major_version le "8.1") {
                if (not length($license = $args{$_})) {
                    $self->_set_err("Invalid value for argument '-license'");
                    return $SanUtil::FALSE;
                }
                next;
            }
        }
         if (/-?service/) {
            if($major_version ge "8.2") {
                if (not length($service = $args{$_})) {
                    $self->_set_err("Invalid value for argument '-service'");
                    return $SanUtil::FALSE;
                }
                next;
            }
        }
        else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $SanUtil::FALSE;
        }
    }
    # Build command arguments, depending on the major version
    # cmd_filer_version needs to be updated for the version greater than 8.2
    # since the xml file has been changed from 8.2 for the cli interface alone.
    #
    if(($major_version ge "8.2") && ($interface=~m/cli/i)){
        %cmd_args = ("-cmd"            => "license-list-info",
                        "-cmd_filer"      => $self->get_name(),
                        "-cmd_interface"  => "cli",
                        "-cmd_filer_version" => "8.2",
                        "-service" => $service,
                        "-check_result"   => 1);
    }
    else {

        %cmd_args = ("-cmd"            => "license-list-info",
                        "-cmd_filer"      => $self->get_name(),
                        "-cmd_interface"  => $interface,
                        "-check_result"   => 1);
    }
    # Execute command.
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if($cmd_obj->error()) {
        $self->_set_err("Unable to verify if license $license exists: ",
                        $cmd_obj->errmsg());
        return $SanUtil::FALSE;
    }
    $self->log_cmd_results($resultsHash_ref);
    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($interface eq "zapi") {
        $output_hash = $self->xml2hash($output);
            if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
                my $dumpOut = Dumper($output_hash->{'licenses'});
                $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                              -msg_level => DUMPER_DEBUG_LEVEL);
            }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        if($interface eq "zapi") {
            $self->_set_err("Result check failed:\n",
                            $output_hash->{reason});
            return $SanUtil::FALSE;
        } else {
            $self->sanlog(-msg => "in resulthash_ref check");
            $self->_set_err("Result check failed:\n ", $output);
            return $SanUtil::FALSE;
        }
    }

    if($self->interface() eq "zapi") {
        #
        # Search for license code in zapi output hash
        #
        # The output hash is a hash of an array,
        # where within the array is
        # another hash that has additional
        # detailed information about each license code
        #
        # Below is a sample output of $output_hash->{'licenses'}
        # $VAR1 = {
        #   'license-info' => [
        #                     {
        #                     ...
        #                     },
        #                     {
        #                      'count' => '0',
        #                      'is-licensed' => 'true',
        #                      'is-expired' => 'false',
        #                       'is-site' => 'true',
        #                       'is-demo' => 'false',
        #                      'service' => 'cifs',
        #                       'length' => '0',
        #                       'node-count' => '0',
        #                       'installation-timestamp' => '1265349648',
        #                       'expiration-timestamp' => '0',
        #                       'code' => 'GTLYRVH'
        #                     },
        #                     {
        #                     ...
        #                     }
        #};
        if (defined $output_hash->{'licenses'}->{'license-info'}) {
            my @license_info = @{$output_hash->{'licenses'}->{'license-info'}};
            my $i = 0;
            my $found_license = 0;
            foreach(@license_info){
                if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
                    if (defined $license) {
                        $self->sanlog(-msg => "Code: $license_info[$i]{'code'}",
                                    -msg_level => DUMPER_DEBUG_LEVEL);
                    }
                    elsif (defined $service) {
                        $self->sanlog(-msg => "License: $license_info[$i]{'service'}",
                                    -msg_level => DUMPER_DEBUG_LEVEL);
                    }
                }
                ## Adding the below if condition, it checks whether the license code provided is being licensed or not.
                ## From 8.2, a service name will be provided, since the code has been changed (v2 licensing)
                ## So, check is being made for the service and license code whether it is licensed or not.
                if (defined $license) {
                    if ((defined $license_info[$i]{'code'}) and
                        ($license_info[$i]{'code'} eq $license)) {
                        if (defined $license_info[$i]{'is-licensed'}) {
                            $found_license = 1;
                        }
                    }
                }
                elsif (defined $service) {
                    if ((defined $license_info[$i]{'service'}) and
                        ($license_info[$i]{'service'} =~/$service/i)) {
                            if ((defined $license_info[$i]{'is-licensed'}) and (($license_info[$i]{'is-licensed'})=~/true/i)){
                            $found_license = 1;
                        }
                    }
                }
                $i++;
            }
            #
            # Verify specified license code was found in zapi ouput hash
            #
            if ($found_license) {
                return $SanUtil::TRUE;
            } else {
                $self->sanlog(-msg => "Did not find license $license on filer");
                if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
                    my $dumpOut = Dumper($output_hash);
                    $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                                  -msg_level => DUMPER_DEBUG_LEVEL);
                    }
                return $SanUtil::FALSE;
            }
        } else {
            $self->_set_err("Command did not return license information");
            return $SanUtil::FALSE;
        }
    } else {
        #
        #Search and verify license code was found in cli output
        #
        if (defined $license) {
            if($output !~ /$license/i) {
                $self->sanlog(-msg => "Did not find license $license on filer: ".
                                   "$output");
                return $SanUtil::FALSE;
            } else {
                $self->sanlog(-msg => "Found  license $license on filer: ".
                                   "$output");
                    return $SanUtil::TRUE;
            }
        }
        elsif (defined $service) {
            if($output !~ /$service/i) {
                $self->sanlog(-msg => "Did not find license $service on filer: ".
                                   "$output");
                return $SanUtil::FALSE;
            } else {
                $self->sanlog(-msg => "Found  license $service on filer: ".
                                   "$output");
                    return $SanUtil::TRUE;
            }
        }
    }
} # end license_exist()

###############################################################################
# Method:    unlicense
# Objective: Removes License the supplied service if not already done.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub unlicense
{
    my ($self, $service) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    unless (defined ($service) and length ($service)) {
        $self->_set_err("Required parameter \$service not supplied");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "license-delete",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -service                => $service);

    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to unlicense $service: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed") {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("$errmsg");
            return $ClientUtil::FALSE;
        }
    } else {
        if($output =~ /failed/i) {
            $self->_set_err("could not delete the license for $service on the filer: $output");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed");
        return $ClientUtil::FALSE;
    }
} # end unlicense()

###############################################################################
# Method:    license_fcp
# Objective: Licenses FCP if not already done.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub license_fcp
{
    my ($self,$license) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $code;

    $self->_clear_err();

    #
    # Parse arguments
    #
    # If we were given an optional license code to use,
    # make sure it's not empty.
    #
    if(defined $license and !(length $license)){
            $self->_set_err("license not provided");
            return $ClientUtil::FALSE;
    }

    $self->sanlog(-msg => "SanFiler::license_fcp() - trying to license FCP");

    #
    # If we were not given an optional license code to use,
    # go find one from a standard file defined by SanUtil.
    #
    unless (defined $license) {
        #
        # Get all available licenses from standard file defined by SanUtil
        #
        my $avail_licenses = &SanUtil::get_avail_controller_licenses();
        #
        # Check to make sure that we were given licenses and that
        # an FCP license is found in the standard file defined by SanUtil.
        #
        if (defined $avail_licenses) {
            #
            #If we looked up and found the license code, save it for use.
            #
            $code = $avail_licenses->{fcp};
            unless (defined $code) {
                $self->sanlog(-msg => "Unable to obtain fcp service license ".
                                      "from".
                                      "SanUtil::get_avail_controller_licenses");
                if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
                    my $dumpOut = Dumper($avail_licenses);
                    $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                                  -msg_level => DUMPER_DEBUG_LEVEL);
                }
                return $ClientUtil::FALSE;
            }
        } else {
            $self->sanlog(-msg => "Unable to obtain license information from ".
                                  "SanUtil::get_avail_controller_licenses");
            if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
                my $dumpOut = Dumper($avail_licenses);
                $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                              -msg_level => DUMPER_DEBUG_LEVEL);
            }
            return $ClientUtil::FALSE;
        }
    }

    #
    # Setting FCP license to use
    #
    $license = $code unless (defined ($license));

    #
    # Execute command
    #
    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "license-add",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -code                   => $license);

    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to license FCP: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed") {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("$errmsg");
            return $ClientUtil::FALSE;
        }
    } else {
        if($output !~ /fcp licensed/ and
           $output !~ /license has been installed/i) {
            $self->_set_err("could not license FCP on the filer: $output");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed");
        return $ClientUtil::FALSE;
    }
} # end license_fcp()

###############################################################################
# Method:    unlicense_fcp
# Objective: Removes FCP License
# Details:   See POD documentation at the end of this file.
###############################################################################
sub unlicense_fcp
{
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::unlicense_fcp() - attempting to unlicense FCP");

    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "license-delete",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -service                => "fcp");

    if($cmd_obj->error()) {
        my $errmsg = $cmd_obj->errmsg();
        $self->_set_err("Unable to unlicense FCP: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed") {
            my $errmsg = $output_hash->{reason};
            $self->_set_err("$errmsg");
            return $ClientUtil::FALSE;
        }
    } else {
        if($output =~ /failed/i) {
            $self->_set_err("could not delete the FCP license on the filer: $output");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed");
        return $ClientUtil::FALSE;
    }
} # end unlicense_fcp()

###############################################################################
# Method:    unlicense_iscsi
# Objective: Removes ISCSI License
# Details:   See POD documentation at the end of this file.
###############################################################################
sub unlicense_iscsi
{
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "license-delete",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -service                => "iscsi");

    if($cmd_obj->error()) {
        my $errmsg = $cmd_obj->errmsg();
        $self->_set_err("Unable to unlicense ISCSI: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed") {
            my $errmsg = $output_hash->{reason};
            $self->_set_err("$errmsg");
            return $ClientUtil::FALSE;
        }
    } else {
        if($output =~ /failed/i) {
            $self->_set_err("could not delete the ISCSI license on the filer: $output");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed");
        return $ClientUtil::FALSE;
    }
} # end unlicense_iscsi()

###############################################################################
# Method:    license_iscsi
# Objective: Licenses iSCSI if not already done.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub license_iscsi
{
    my ($self,$license) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $code;

    $self->_clear_err();

    #
    # Parse arguments
    #
    #
    # If we were given an optional license code to use,
    # make sure it's not empty.
    #
    if (defined $license and !(length $license)){
            $self->_set_err("license not provided");
            return $ClientUtil::FALSE;
    }

    $self->sanlog(-msg => "SanFiler::license_iscsi() ".
                          "- trying to license iSCSI");

    #
    # If we were not given an optional license code to use,
    # go find one from a standard file defined by SanUtil.
    #
    unless (defined $license) {
        #
        # Get all available licenses from standard file defined by SanUtil
        #
        my $avail_licenses = &SanUtil::get_avail_controller_licenses();
        #
        # Check to make sure that we were given licenses and that
        # an iSCSI license is found in the standard file defined by SanUtil.
        #
        if (defined $avail_licenses) {
            #
            #If we looked up and found the license code, save it for use.
            #
            $code = $avail_licenses->{iscsi};
            unless (defined $code) {
                $self->sanlog(-msg => "Unable to obtain iscsi service license ".
                                      "from".
                                      "SanUtil::get_avail_controller_licenses");
                if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
                    my $dumpOut = Dumper($avail_licenses);
                    $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                                  -msg_level => DUMPER_DEBUG_LEVEL);
                }
                return $ClientUtil::FALSE;
            }
        } else {
            $self->sanlog(-msg => "Unable to obtain license information from ".
                                  "SanUtil::get_avail_controller_licenses");
            if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
                my $dumpOut = Dumper($avail_licenses);
                $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                              -msg_level => DUMPER_DEBUG_LEVEL);
            }
            return $ClientUtil::FALSE;
        }
    }


    #
    # Setting iSCSI license to use
    #
    $license = $code unless (defined ($license));

    #
    # Execute command
    #
    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "license-add",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -code                   => $license);

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::license_iscsi(): Unable to " .
                        "license iSCSI: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Error: SanFiler::license_iscsi(): $errmsg");
            return $ClientUtil::FALSE;
        }
    }
    else
    {
        if($output !~ /iscsi licensed/i and
       $output !~ /license has been installed/i and
       $output !~ /no license code is necessary/i)
        {
            $self->_set_err("Error: SanFiler::iscsi_license could not ".
                            "license iSCSI on the filer: $output");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::license_iscsi() " .
                        "results check failed");
        return $ClientUtil::FALSE;
    }
} # end sub license iscsi

###############################################################################
# Method:    iscsi_tpgroup_create
# Objective: Creates a tpgroup on a filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_tpgroup_create
{
    my ($self, $name, $interfaces, $tpgtag) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "iscsi tpgroup create");

    unless (defined $name and length $name)
    {
        $self->_set_err("ERROR: SanFiler::iscsi_tpgroup_create() Required " .
                        "parameter \$name not found");
        return $ClientUtil::FALSE;
    }

    my @cmds_args;

    @cmds_args = ("-cmd","iscsi-tpgroup-create");
    push(@cmds_args, "-tpgroup-name", $name);
    push(@cmds_args, "-force", "");
    push(@cmds_args, "-tp-interfaces", $interfaces) if (defined $interfaces);
    push(@cmds_args, "-tpgroup-tag", $tpgtag) if (defined $tpgtag);
    push(@cmds_args, "-cmd_filer", $self->get_name());
    push(@cmds_args, "-check_result", "1");

    my $resultsHash_ref = $cmd_obj->execute_cmd(@cmds_args);


    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::iscsi_tpgroup_create(): Unable to " .
                        "create iscsi tpgroup $name: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Error: SanFiler::iscsi_tpgroup_create(): $errmsg");
            return $ClientUtil::FALSE;
        }
        else
        {
            #
            # Add interfaces manually
            #
            if(defined($interfaces))
            {
                my @interface_list = split(/ /, $interfaces);

                foreach my $interface (@interface_list)
                {
                    $self->iscsi_tpgroup_interface_add($name, $interface);
                }
            }
        }
    }
    else
    {
        if($output =~ /iscsi tpgroup create:/i)
        {
            my $errmsg = $output;
            $self->_set_err("Error: SanFiler::iscsi_tpgroup_create() could" .
                            " not create the tpgroup $name: $errmsg");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::iscsi_tpgroup_create() " .
                        "results check failed");
        return $ClientUtil::FALSE;
    }
} # end sub iscsi_tpgroup_create

###############################################################################
# Method:    iscsi_tpgroup_show
# Objective: Displays the tpgroup information on the filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_tpgroup_show
{
    my($self)   = @_;
    my $cmd_obj = $self->cmd_obj();
    my $errmsg  = "";

    $self->_clear_err();

    $self->sanlog(-msg => "iscsi tpgroup show");

    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-tpgroup-list-info",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -check_output           => 1);

    if($cmd_obj->error()) {
        $self->_set_err("Unable to display tpgroups", $cmd_obj->errmsg());
        return $errmsg;
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (!($resultsHash_ref->{result_check} && $resultsHash_ref->{output_check})) {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

    } else {
        if($output=~/iscsi tpgroup show:/i) {
            $self->_set_err("could not display tpgroups: ", $output);
            $errmsg = $output;
            return $errmsg;
        } else {
            # parse
            my @out = split('\n', $output);
            my @complete = ();

            push(@complete,'<results status="passed"><iscsi-tpgroup-list-entries>');

            #
            # This takes multiple lines of:
            # TPGTag TPGName and TPGInterfaces (separated by commas),
            # all three of the above separated by white space.
            #
            foreach my $line (@out) {
                my ($tag, $name, @interfaces);
                # temp contains the line broken up into tag, name
                # and interfaces
                my @temp = split(' ', $line);
                my $size = @temp;
                my $alua_state = "";
                my $alua_pref  = "";

                $tag = $temp[0];
                next if($tag eq "TPGTag"); # skip header

                $name = $temp[1];
                for (my $j = 2; $j < $size; $j++) {
                    push(@interfaces, $temp[$j]);
                }

                my $aluatot = $self->_get_iscsi_tpgroup_alua_state($name);
                if($self->error()) {
                    $self->_set_err("_get_iscsi_tpgroup_alua_state failed", $self->errmsg());
                    return $errmsg;
                }
                my @alua_states = split(/ /, $aluatot);
                $size = @alua_states;

                $alua_state = $alua_states[0];
                $alua_state = lc($alua_state);

                #
                # It means it has the "state Preferred"
                #
                if($size > 1) {
                    $alua_pref = "true";
                } else {
                    $alua_pref = "false";
                }

                push(@complete,'<iscsi-tpgroup-list-entry-info><tpgroup-tag>'         .
                    $tag.'</tpgroup-tag><tpgroup-name>' . $name . '</tpgroup-name>'   .
                    '<tpgroup-alua-state>'.$alua_state.'</tpgroup-alua-state>'        .
                    '<tpgroup-alua-preferred>'.$alua_pref.'</tpgroup-alua-preferred>' .
                    '<interface-list-entries>');

                foreach my $interface (@interfaces) {
                    if($interface =~ /,/) {
                        $interface =~ s/,\s*//g;
                    }

                    if($interface =~ /none/i) {
                        # DO NOTHING
                    } else {
                        push(@complete, '<interface-list-entry-info>' .
                             '<interface-name>'. $interface           .
                             '</interface-name></interface-list-entry-info>');
                    }
                } # end foreach interface

                push(@complete, '</interface-list-entries>' .
                     '</iscsi-tpgroup-list-entry-info>');
            } # end foreach line

            push(@complete,'</iscsi-tpgroup-list-entries></results>');

            $output = join('', @complete);

        } # end else no error
    } # else interface is cli

    $output = $self->xml2hash($output);
    return $output;

} # end sub iscsi_tpgroup_show

###############################################################################
# Method:    iscsi_tpgroup_exist
# Objective: Checks if a tpgroup exists
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_tpgroup_exist
{
    my ($self, $name) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "Getting existance of tpgroup $name");

    unless (defined $name and length $name)
    {
        $self->_set_err("ERROR: SanFiler::iscsi_tpgroup_exist() Required " .
                        "parameter \$name not found");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-tpgroup-list-info",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1);

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::iscsi_tpgroup_exist(): Unable to " .
                        "determine tpgroup $name existance: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my @output = split("\n", $resultsHash_ref->{output});

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($resultsHash_ref->{output});
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Error: SanFiler::iscsi_tpgroup_exist() " .
                            "no tpgroups found",$errmsg);
            return $ClientUtil::FALSE;
        }
        else
        {
            #
            # Gets an array of hashes
            #
            my $arr = $output_hash->{"iscsi-tpgroup-list-entries"}->
                                    {"iscsi-tpgroup-list-entry-info"};

            #
            # Go through array grabbing hashes
            #
            foreach my $hash1 (@$arr)
            {
                foreach my $key (keys(%{$hash1}))
                {
                    if($key eq "tpgroup-name")
                    {
                        my $entry = $hash1->{$key};

                        if($entry eq $name)
                        {
                            return $ClientUtil::TRUE;
                        }
                    }
                }
            } # end foreach $hash1

            $self->sanlog(-msg       => "TPGroup $name does not exist",
                          -msg_level => 3);

            $self->_set_err("Error: SanFiler::iscsi_tpgroup_exist() " .
                            "tpgroup not found");
            return $ClientUtil::FALSE;
        }
    }
    else
    {
        foreach my $line (@output)
        {
            if($line =~ /$name/)
            {
                return $ClientUtil::TRUE;
            }
        }

        $self->sanlog(-msg       => "TPGroup $name does not exist",
                      -msg_level => 3);

        $self->_set_err("Error: SanFiler::iscsi_tpgroup_exist() " .
                        "tpgroup not found");
        return $ClientUtil::FALSE;
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::iscsi_tpgroup_exist() " .
                        "results check failed");
        return $ClientUtil::FALSE;
    }
} # end sub iscsi_tpgroup_exist

###############################################################################
# Method:    iscsi_tpgroup_destroy
# Objective: Removes a tpgroup
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_tpgroup_destroy
{
    my ($self, $name) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "iscsi tpgroup destroy");

    unless (defined $name and length $name)
    {
        $self->_set_err("ERROR: SanFiler::iscsi_tpgroup_destroy() Required " .
                        "parameter \$name not found");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref;

    if($self->interface() eq "cli")
    {
        $resultsHash_ref =
         $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-tpgroup-destroy",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        "-tpgroup-name"         => $name,
                        -force                  => "");
    }
    else
    {
        my $tpgtag = $self->_get_iscsi_tpgroup_tag($name);

        if($tpgtag eq "")
        {
            $self->_set_err("Error: SanFiler::iscsi_tpgroup_destroy(): Unable to " .
                            "destroy tpgroup $name");
            return $ClientUtil::FALSE;
        }

        $resultsHash_ref =
            $cmd_obj->execute_cmd(
                            -cmd            => "iscsi-tpgroup-destroy",
                            -cmd_filer      => $self->get_name(),
                            -check_result   => 1,
                            "-tpgroup-tag"  => $tpgtag);
    }

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::iscsi_tpgroup_destroy(): Unable to " .
                        "destroy tpgroup $name: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Error: SanFiler::iscsi_tpgroup_destroy(): $errmsg");
            return $ClientUtil::FALSE;
        }
    }
    else
    {
        if($output=~/iscsi tpgroup destroy:/i)
        {
            $self->_set_err("Error: SanFiler::iscsi_tpgroup_destroy() " .
                            "unable to destroy tpgroup $name: ",$output);
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::iscsi_tpgroup_destroy() " .
                        "results check failed");
        return $ClientUtil::FALSE;
    }
} # end sub iscsi_tpgroup_destroy

###############################################################################
# Method:    iscsi_tpgroup_interface_add
# Objective: Creates a tpgroup on a filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_tpgroup_interface_add
{
    my ($self, $name, $interface) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "iscsi tpgroup interface add");

    unless (defined $name and length $name)
    {
        $self->_set_err("ERROR: SanFiler::iscsi_tpgroup_interface_add() Required " .
                        "parameter \$name not found");
        return $ClientUtil::FALSE;
    }

    unless (defined $interface and length $interface)
    {
        $self->_set_err("ERROR: SanFiler::iscsi_tpgroup_interface_add() Required " .
                        "parameter \$interface required");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref;

    if($self->interface() eq "cli")
    {
        $resultsHash_ref =
         $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-tpgroup-interface-add",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        "-interface-name"       => $interface,
                        "-tpgroup-name"         => $name,
                        -force                  => "");
    }
    else
    {
        my $tpgtag = $self->_get_iscsi_tpgroup_tag($name);

        if($tpgtag eq "")
        {
            my $errmsg = join "\n", $cmd_obj->errmsg();
            $self->_set_err("Error: SanFiler::iscsi_tpgroup_interface_add(): Unable to " .
                            "locate tpgroup $name: $errmsg");
            return $ClientUtil::FALSE;
        }

        $resultsHash_ref =
         $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-tpgroup-interface-add",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        "-interface-name"       => $interface,
                        "-tpgroup-tag"          => $tpgtag,
                        -force                  => "");
    }

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::iscsi_tpgroup_interface_add(): Unable to " .
                        "add interface to tpgroup $name: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Error: SanFiler::iscsi_tpgroup_interface_add(): $errmsg");
            return $ClientUtil::FALSE;
        }
    }
    else
    {
        if($output =~ /iscsi tpgroup add:/i)
        {
            my $errmsg = $output;
            $self->_set_err("Error: SanFiler::iscsi_tpgroup_interface_add() could" .
                            " not add interface $interface: $errmsg");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::iscsi_tpgroup_interface_add() " .
                        "results check failed");
        return $ClientUtil::FALSE;
    }
} # end sub iscsi_tpgroup_interface_add

###############################################################################
# Method:    iscsi_tpgroup_interface_remove
# Objective: removes a network interface from existing tpgroup name on a filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_tpgroup_interface_remove
{
    my ($self, $name, $interface) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "iscsi tpgroup interface remove");

    unless (defined $name and length $name) {
        $self->_set_err("Required parameter \$name not found");
        return $ClientUtil::FALSE;
    }

    unless (defined $interface and length $interface) {
        $self->_set_err("Required parameter \$interface required");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref;

    # Execute the command with either cli or ZAPI
    if($self->interface() eq "cli")
    {
        $resultsHash_ref =
         $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-tpgroup-interface-delete",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        "-interface-name"       => $interface,
                        "-tpgroup-name"         => $name,
                        -force                  => "");
    } else {
        my $tpgtag = $self->_get_iscsi_tpgroup_tag($name);

        if($tpgtag eq "") {
            my $errmsg = join "\n", $cmd_obj->errmsg();
            $self->_set_err("Unable to locate tpgroup $name",
                $cmd_obj->errmsg());
            return $ClientUtil::FALSE;
        }

        $resultsHash_ref =
         $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-tpgroup-interface-delete",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        "-interface-name"       => $interface,
                        "-tpgroup-tag"          => $tpgtag,
                        -force                  => "");
    }

    # Check to see if the command execution succeeded
    if($cmd_obj->error()) {
        $self->_set_err("Unable to remove interface from tpgroup $name",
            $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # Get the output
    my $output = $resultsHash_ref->{output};

    # If the interface is zapi we have to convert the output from xml
    # to a string.
    if($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }
    } else {
        if($output =~ /iscsi tpgroup remove:/i) {
            $self->_set_err("Could not remove interface $interface: $output");
            return $ClientUtil::FALSE;
        }
    }

    # Since we checked the result, lets see if our check was OK
    if($resultsHash_ref->{result_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed");
        return $ClientUtil::FALSE;
    }
} # end sub iscsi_tpgroup_interface_remove

###############################################################################
# Method:    iscsi_interface_accesslist_add
# Objective: Creates an accesslist for the initiator on the provided interfaces
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_interface_accesslist_add
{
    my ($self, $iqn, $interface, $force) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "iscsi interface accesslist add");

    if (not defined($force)) {
        $force = "";
    } else {
        $force = 1;
    }

    unless (defined $iqn and length $iqn)
    {
        $self->_set_err("ERROR: SanFiler::iscsi_interface_accesslist_add() Required " .
                        "parameter \$iqn not found");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref;

    if($self->interface() eq "cli")
    {
        if(defined $interface and length $interface)
        {
            $resultsHash_ref =
             $cmd_obj->execute_cmd(
                            -cmd                    => "iscsi-interface-accesslist-add",
                            -cmd_filer              => $self->get_name(),
                            -check_result           => 1,
                            "-initiator-name"       => $iqn,
                            "-interface-name"       => $interface,
                            "-force"                => $force);
        }
        else
        {
            $resultsHash_ref =
             $cmd_obj->execute_cmd(
                            -cmd                    => "iscsi-interface-accesslist-add",
                            -cmd_filer              => $self->get_name(),
                            -check_result           => 1,
                            "-initiator-name"       => $iqn,
                            "-all-interfaces"       => "",
                            "-force"                => $force);
        }
    }
    else
    {
        my @cmd_args;

        push(@cmd_args, "iscsi");
        push(@cmd_args, "interface");
        push(@cmd_args, "accesslist");
        push(@cmd_args, "add");
        if ($force == 1) {
            push(@cmd_args, "-f");
        }
        push(@cmd_args, $iqn);

        if(defined $interface)
        {
            # strip off leading/training whitespace for accurate split
            $interface =~ s/^\s+//;
            $interface =~ s/\s+$//;

            my @i = split(/ /, $interface);
            my $size = @i;

            for(my $j = 0; $j < $size; $j++)
            {
                push(@cmd_args, $i[$j]);
            }
        }
        else
        {
            push(@cmd_args, "-a");
        }

        $resultsHash_ref =
         $cmd_obj->execute_cmd(
                        -cmd                      => "system-cli",
                        -cmd_filer                => $self->get_name(),
                        -check_result             => 1,
                        -args                     => \@cmd_args,
                        -priv                     => "admin");
    }


    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_add(): " .
                        "Unable to add accesslist: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_add(): ",
                            $errmsg);
            return $ClientUtil::FALSE;
        }
        else
        {
            if($output =~ /iscsi interface accesslist add:/i)
            {
                $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_add() " .
                                "could not add accesslist: ",$output);
                return $ClientUtil::FALSE;
            }
        }
    }
    else
    {
        if($output =~ /iscsi interface accesslist add:/i)
        {
            $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_add() " .
                            "could not add accesslist: ",$output);
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::iscsi_interface_access_list_add() " .
                        "results check failed");
        return $ClientUtil::FALSE;
    }
} # end sub iscsi_interface_accesslist_add

###############################################################################
# Method:    iscsi_interface_accesslist_exist
# Objective: Checks to see if an accesslist already exists
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_interface_accesslist_exist
{
    my ($self, $iqn) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "iscsi interface accesslist exist");

    unless (defined $iqn and length $iqn)
    {
        $self->_set_err("ERROR: SanFiler::iscsi_interface_accesslist_exist() Required " .
                        "parameter \$iqn not found");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref;

    if($self->interface() eq "cli")
    {
        $resultsHash_ref =
         $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-interface-accesslist-list-info",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        "-initiator-name"       => $iqn);
    }
    else
    {
        my @cmd_args;

        push(@cmd_args, "iscsi");
        push(@cmd_args, "interface");
        push(@cmd_args, "accesslist");
        push(@cmd_args, "show");
        push(@cmd_args, $iqn);

        $resultsHash_ref =
         $cmd_obj->execute_cmd(
                        -cmd                      => "system-cli",
                        -cmd_filer                => $self->get_name(),
                        -check_result             => 1,
                        -args                     => \@cmd_args,
                        -priv                     => "admin");
    }


    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_exist(): " .
                        "Unable to determine accesslist existance: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_exist(): ",
                            $errmsg);
            return $ClientUtil::FALSE;
        }
        else
        {
            if($output =~ /iscsi interface accesslist show:/i or
               $output =~ /no accesslist/i)
            {
                $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_exist() " .
                                "could not locate accesslist: ",$output);
                return $ClientUtil::FALSE;
            }
        }
    }
    else
    {
        if($output =~ /iscsi interface accesslist show:/i or
           $output =~ /no accesslist/i)
        {
            $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_exist() " .
                            "could not locate accesslist: ",$output);
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::iscsi_interface_access_list_exist() " .
                        "results check failed");
        return $ClientUtil::FALSE;
    }
} # end sub iscsi_interface_accesslist_exist

###############################################################################
# Method:    iscsi_interface_accesslist_remove
# Objective: Removes an accesslist
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_interface_accesslist_remove
{
    my ($self, $iqn, $interface) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "iscsi interface accesslist remove");

    unless (defined $iqn and length $iqn)
    {
        $self->_set_err("ERROR: SanFiler::iscsi_interface_accesslist_remove() Required " .
                        "parameter \$iqn not found");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref;

    if($self->interface() eq "cli")
    {
        if(defined $interface and length $interface)
        {
            $resultsHash_ref =
             $cmd_obj->execute_cmd(
                            -cmd                    => "iscsi-interface-accesslist-remove",
                            -cmd_filer              => $self->get_name(),
                            -check_result           => 1,
                            "-initiator-name"       => $iqn,
                            "-interface-name"       => $interface,
                            -force                  => "");
        }
        else
        {
            $resultsHash_ref =
             $cmd_obj->execute_cmd(
                            -cmd                    => "iscsi-interface-accesslist-remove",
                            -cmd_filer              => $self->get_name(),
                            -check_result           => 1,
                            "-initiator-name"       => $iqn,
                            "-all-interfaces"       => "",
                            -force                  => "");
        }
    }
    else
    {
        my @cmd_args;

        push(@cmd_args, "iscsi");
        push(@cmd_args, "interface");
        push(@cmd_args, "accesslist");
        push(@cmd_args, "remove");
        push(@cmd_args, "-f");
        push(@cmd_args, $iqn);

        if(defined $interface)
        {
            my @i = split(/ /, $interface);
            my $size = @i;

            for(my $j = 0; $j < $size; $j++)
            {
                push(@cmd_args, $i[$j]);
            }
        }
        else
        {
            push(@cmd_args, "-a");
        }

        $resultsHash_ref =
         $cmd_obj->execute_cmd(
                        -cmd                      => "system-cli",
                        -cmd_filer                => $self->get_name(),
                        -check_result             => 1,
                        -args                     => \@cmd_args,
                        -priv                     => "admin");
    }


    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_remove(): " .
                        "Unable to remove accesslist: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_remove(): ",
                            $errmsg);
            return $ClientUtil::FALSE;
        }
        else
        {
            if($output =~ /iscsi interface accesslist remove:/i or
               $output =~ /is not in the accesslist for/i)
            {
                $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_remove() " .
                                "could not remove accesslist: ",$output);
                return $ClientUtil::FALSE;
            }
        }
    }
    else
    {
        if($output =~ /iscsi interface accesslist remove:/i or
           $output =~ /is not in the accesslist for/i)
        {
            $self->_set_err("Error: SanFiler::iscsi_interface_accesslist_remove() " .
                            "could not remove accesslist: ",$output);
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::iscsi_interface_access_list_remove() " .
                        "results check failed");
        return $ClientUtil::FALSE;
    }
} # end sub iscsi_interface_accesslist_remove

###############################################################################
# Method:    iscsi_security_add_chap
# Objective: Sets up CHAP on the filer for an initiator
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_security_add_chap
{
    my (
        $self,
        $initiator,
        $inusername,
        $inpassword,
        $outusername,
        $outpassword,
       ) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg => "iscsi security add chap");

    unless (defined $initiator and length $initiator)
    {
        $self->_set_err("ERROR: SanFiler::iscsi_security_add_chap() Required " .
                        "parameter \$initiator not found");
        return $ClientUtil::FALSE;
    }

    unless (defined $inusername and length $inusername)
    {
        $self->_set_err("ERROR: SanFiler::iscsi_security_add_chap() Required " .
                        "parameter \$inusername not found");
        return $ClientUtil::FALSE;
    }

    unless (defined $inpassword and length $inpassword)
    {
        $self->_set_err("ERROR: SanFiler::iscsi_security_add_chap() Required " .
                        "parameter \$inpassword not found");
        return $ClientUtil::FALSE;
    }

    $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-initiator-delete-auth",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -initiator              => $initiator);

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::iscsi_security_add_chap(): Could " .
                        "not properly configure CHAP on the filer: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    my @cmds_args;

    @cmds_args = ("-cmd","iscsi-initiator-add-auth");
    push(@cmds_args, "-auth-type", "CHAP");
    push(@cmds_args, "-initiator", $initiator);
    push(@cmds_args, "-user-name", $inusername);
    push(@cmds_args, "-password", $inpassword);
    push(@cmds_args, "-cmd_filer", $self->get_name());
    push(@cmds_args, "-check_result", "1");

    #
    # Both outusername and outpassword must be defined
    # to use the out options this checks to make sure
    # of that.
    #
    if(defined($outusername) and length($outusername))
    {
        push(@cmds_args, "-outbound-user-name", $outusername);
        push(@cmds_args, "-outbound-password", $outpassword);
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(@cmds_args);

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::iscsi_security_add_chap(): Could " .
                        "not properly configure CHAP on the filer: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi")
    {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Error: SanFiler::iscsi_security_add_chap(): $errmsg");
            return $ClientUtil::FALSE;
        }
    }
    else
    {
        if(grep /iscsi security add: / , $output)
        {
            my $errmsg = join "\n", $output;
            $self->_set_err("Error: SanFiler::iscsi_config_chap() could not ".
                            "properly configure CHAP on the filer: $errmsg");
            return $ClientUtil::FALSE;
        }
        return $ClientUtil::TRUE;
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::license_iscsi_security_add_chap() " .
                        "results check failed");
        return $ClientUtil::FALSE;
    }
} # end sub iscsi_security_add_chap

###############################################################################
# Method:    options_iscsi_config_mcs
# Objective: Configures Multi-Connections-Sessions
# Details:   See POD documentation at the end of this file.
###############################################################################
sub options_iscsi_config_mcs
{
    my ($self, %args) = @_;
    my (
        $tpname,
        $interfaces,
        $erl,
        $max_conn
       ) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my @output;

    $self->_clear_err();

    $self->sanlog(-msg => "options iscsi configure " .
                          "multi-connections-sessions");

    #
    # $erl = Error Recovery Level
    #
    foreach (keys %args)
    {
        if (/-?tpname/)
        {
            length($tpname = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::options_iscsi_config_mcs " .
                                    "invalid value "                      .
                                    "for \"tpname\": $args{$_}");
                    return $self;
                };
        }
        elsif (/-?interfaces/)
        {
            length($interfaces = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::options_iscsi_config_mcs " .
                                    "invalid value "                      .
                                    "for \"interfaces\": $args{$_}");
                    return $self;
                };
        }
        elsif (/-?erl/)
        {
            length($erl = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::options_iscsi_config_mcs " .
                                    "invalid value "                      .
                                    "for \"erl\": $args{$_}");
                    return $self;
                };
        }
        elsif (/-?max_conn/)
        {
            length($max_conn = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::options_iscsi_config_mcs " .
                                    "invalid value "                      .
                                    "for \"max_conn\": $args{$_}");
                    return $self;
                };
        }
        else
        {
            $self->sanlog(-msg => "SanFiler::options_iscsi_config_mcs " .
                                  "Invalid parameter specified: $_");
        }
    }

    #
    # Check for required parameters
    #
    unless (defined $tpname and length $tpname)
    {
        $self->_set_err("ERROR: SanFiler::options_iscsi_config_mcs() " .
                        "Required parameter \$tpname not found");
        return $ClientUtil::FALSE;
    }

    unless (defined $interfaces and length $interfaces)
    {
        $self->_set_err("ERROR: SanFiler::options_iscsi_config_mcs() " .
                        "Required parameter \$interface not found");
        return $ClientUtil::FALSE;
    }

    #
    # Create valid tpgroup
    #
    if($self->iscsi_tpgroup_exist($tpname))
    {
        # It already exists so we will leave it as it is
        $self->sanlog(-msg  => "Using existing tpgroup");
    }
    else
    {
        # It doesn't exist so let's create it
        $self->_clear_err();
        $self->iscsi_tpgroup_create($tpname, $interfaces);
    }

    #
    # Set max error recovery level
    #
    $erl = "0" unless(defined($erl));
    $resultsHash_ref =
        $cmd_obj->execute_cmd(
                            -cmd            => "options-set",
                            -cmd_filer      => $self->get_name(),
                            -check_result   => 0,
                            -name           => "iscsi.max_error_recovery_level",
                            -value          => $erl);

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::options_iscsi_config_mcs(): " .
                        "Unable to set option: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->_print_resultsHash($resultsHash_ref);

    #
    # Get the output
    #
    @output = split("\n", $resultsHash_ref->{output});

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->{interface} eq "zapi")
    {
        my $output_hash = $self->xml2hash($resultsHash_ref->{output});
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            $self->_set_err("Error: SanFiler::options_iscsi_config_mcs: " .
                            "Unable to set option", $output_hash->{reason});
            return $ClientUtil::FALSE;
        }
    }
    else
    {
        if(grep /option iscsi.max_error_recovery_level: /, @output)
        {
            my $errmsg = join "\n", @output;
            $self->_set_err("Error: SanFiler::options_iscsi_config_mcs() " .
                            "could not properly set the "                  .
                            "max_error_recovery_level: $errmsg");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Set max connections
    #
    $max_conn = "4" unless(defined($max_conn));
    $resultsHash_ref =
    $cmd_obj->execute_cmd(
                        -cmd            => "options-set",
                        -cmd_filer      => $self->get_name(),
                        -check_result   => 1,
                        -name           => "iscsi.max_connections_per_session",
                        -value          => $max_conn);

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::options_iscsi_config_mcs(): " .
                        "Unable to set option: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->_print_resultsHash($resultsHash_ref);

    #
    # Get the output
    #
    @output = split("\n", $resultsHash_ref->{output});

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->{interface} eq "zapi")
    {
        my $output_hash = $self->xml2hash($resultsHash_ref->{output});
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
        {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }        my $status = $output_hash->{status};

        if($status eq "failed")
        {
            $self->_set_err("Error: SanFiler::options_iscsi_config_mcs: " .
                            "Unable to set option", $output_hash->{reason});
            return $ClientUtil::FALSE;
        }
    }
    else
    {
        if(grep /option iscsi.max_connections_per_session: /, @output)
        {
            my $errmsg = join "\n", @output;
            $self->_set_err("Error: SanFiler::options_iscsi_config_mcs() " .
                            "could not properly set the "                  .
                            "max_connections_per_session: $errmsg");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if($resultsHash_ref->{result_check})
    {
        return $ClientUtil::TRUE;
    }
    else
    {
        $self->_set_err("Error: SanFiler::options_iscsi_config_mcs() " .
                        "result check failed");
        return $ClientUtil::FALSE;
    }
}

###############################################################################
# Method:    vfiler_create
# Objective: Creates a virtual filer.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vfiler_create
{
    my ($self, $name, $ip_addresses, $paths) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "Create vfiler $name");


    unless (defined $name and length $name) {
        $self->_set_err("Required parameter \$name not found");
        return $ClientUtil::FALSE;
    }


    unless (defined $ip_addresses and length $ip_addresses) {
        $self->_set_err("Required parameter \$ip_address required");
        return $ClientUtil::FALSE;
    }


    unless (defined $paths and length $paths) {
        $self->_set_err("Required parameter \$path required");
        return $ClientUtil::FALSE;
    }

    #
    # Make sure the list of ips is in the correct format:
    # a space/-i separated list ei.) ip1 -i ip2 -i ip3
    #
    my @ipaddrs = split(/ /, $ip_addresses);
    my $size = @ipaddrs;
    my @CLIipaddrs = ();
    $CLIipaddrs[0] = $ipaddrs[0];

    #
    # Starts at 1 because the first ip address, does not
    # start with a -i
    #
    for(my $i = 1; $i < $size; $i++) {
        $CLIipaddrs[$i] = "-i " . $ipaddrs[$i];
    }
    my $ip_addrs = join(' ', @CLIipaddrs);
    my $resultsHash_ref;

    if($self->interface() eq "cli") {
        $resultsHash_ref =
         $cmd_obj->execute_cmd(
                        -cmd                    => "vfiler-create",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        "-ip-addresses"         => $ip_addrs,
                        "-storage-units"        => $paths,
                        -vfiler                 => $name,
                        "-bypass-setup"         => "");
    } else {
        my @storage_units = split(/ /, $paths);

        $resultsHash_ref =
         $cmd_obj->execute_cmd(
                        -cmd                    => "vfiler-create",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        "-ip-addresses"         => \@ipaddrs,
                        "-storage-units"        => \@storage_units,
                        -vfiler                 => $name);
    }


    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::vfiler_create(): Unable to " .
                        "create vfiler $name: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }

        my $status = $output_hash->{status};

        if($status eq "failed") {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Error: SanFiler::vfiler_create(): $errmsg");
            return $ClientUtil::FALSE;
        }
    } else {
        if($output !~ /The etc configuration directory for vfiler/) {
            my $errmsg = $output;
            $self->_set_err("Error: SanFiler::vfiler_create() could not".
                            " properly create the vfiler $name: $name: $errmsg");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Error: SanFiler::vfiler_create() " .
                        "results check failed");
        return $ClientUtil::FALSE;
    }

    #
    # Create and return a vfiler object
    #
    my $vfiler_obj = $self->get_vfiler(-name => $name);
    return $self if ($self->error);

    return $vfiler_obj;
} # end sub vfiler_create

###############################################################################
# Method:    vfiler_setup
# Objective: Sets up a virtual filer.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vfiler_setup
{
    my ($self, $name, $ip_configs_ref) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "Setup vfiler $name");


    unless (defined $name and length $name) {
        $self->_set_err("Required parameter \$name not found");
        return $ClientUtil::FALSE;
    }

    unless (defined $ip_configs_ref and length $ip_configs_ref) {
        $self->_set_err("Required parameter ip_configs_ref required");
        return $ClientUtil::FALSE;
    }

    my %ip_binding_info;
    #
    # IP configs ref should look like this:
    #    $ip_configs_ref->{interface} = "e0b";
    #    $ip_configs_ref->{ipaddress} = "10.10.10.1";
    #    $ip_configs_ref->{netmask}   = "255.255.255.0";
    #
    $ip_binding_info{"ipbinding-info"} = $ip_configs_ref;

    my $resultsHash_ref;

    $resultsHash_ref =
     $cmd_obj->execute_cmd(
                        -cmd                    => "vfiler-setup",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        "-ipbindings"          => \%ip_binding_info,
                        -vfiler                 => $name,
                         "-cmd_interface"       => "zapi");

    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to setup vfiler $name: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    my $output_hash = $self->xml2hash($output);
    if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
        my $dumpOut = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => DUMPER_DEBUG_LEVEL);
    }

    my $status = $output_hash->{status};

    if($status eq "failed") {
        my $errmsg=$output_hash->{reason};
        $self->_set_err("$errmsg");
        return $ClientUtil::FALSE;
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("results check failed");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
} # end sub vfiler_setup

###############################################################################
# Method:    vfiler_exist
# Objective: Checks if a vfiler exists
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vfiler_exist
{
    my ($self, $name) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "Getting existence of vfiler $name");

    unless (defined $name and length $name) {
        $self->_set_err("Required parameter \$name not found");
        return $ClientUtil::FALSE;
    }

    $self->get_vfiler(-name => $name);
    if ($self->error() and ($self->errmsg() =~ /Vfiler not found/)) {
        $self->_clear_err();
        return $ClientUtil::FALSE;
     }
     return $ClientUtil::TRUE;

} # end sub vfiler_exist

###############################################################################
# Method:    vfiler_destroy
# Objective: Destroys a vfiler
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vfiler_destroy
{
    my ($self, $name, $vfiler_obj) = @_;

    $self->_clear_err();

    $self->sanlog(-msg => "vfiler destroy");

    unless (defined $vfiler_obj and length $vfiler_obj) {
        unless (defined $name and length $name) {
            $self->_set_err("Required parameter \$name not found");
            return $ClientUtil::FALSE;
        }
        $vfiler_obj = $self->get_vfiler(-name => $name);
        if ($self->error()) {
            return $ClientUtil::FALSE;
        }
    }

    $vfiler_obj->stop();
    if ($vfiler_obj->error()) {
        $self->_set_err($vfiler_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $vfiler_obj->destroy();
    if ($vfiler_obj->error()) {
        $self->_set_err($vfiler_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;

} # end sub vfiler_destroy

###############################################################################
# Method:    vif_create
# Objective: Creates a virtual interface
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vif_create
{
    my ($self, %args) = @_;
    my (
        $name,
        $type,
        $interfaces,
        $ip_address,
        $subnet,
       );

    my ($cmd, @output);

    $self->_clear_err();

    $self->sanlog(-msg => "vif create");

    foreach (keys %args)
    {
        if (/-?name/)
        {
            length($name = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::vif_create invalid value for \"name\":",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?type/)
        {
            length($type = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::vif_create invalid value for \"type\":",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?interfaces/)
        {
            length($interfaces = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::vif_create invalid value for \"interfaces\"",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?ip_address/)
        {
            length($ip_address = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::vif_create invalid value for \"ip_address\":",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?subnet/)
        {
            length($subnet = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::vif_create invalid value for \"subnet\"",
                                    "$args{$_}");
                    return $self;
                };
        }
        else
        {
            $self->_set_err("SanFiler::vif_create Invalid parameter specified: $_");
            return $self;
        }
    }

    unless (defined $name and length $name)
    {
        $self->_set_err("ERROR: SanFiler::vif_create() Required " .
                        "parameter \$name not found");
        return $ClientUtil::FALSE;
    }

    unless (defined $type and length $type)
    {
        $self->_set_err("ERROR: SanFiler::vif_create() Required " .
                        "parameter \$type not found");
        return $ClientUtil::FALSE;
    }

    $cmd="vif create $type $name";

    if(defined($interfaces))
    {
        $cmd .= " -b rr $interfaces";
        my @interface_list = split /\s+/, $interfaces;
        foreach my $intf (@interface_list)
        {
            $self->ip_offline($intf);
            if($self->error())
            {
                my $errmsg = join("\n", $self->errmsg());
                $self->_set_err("ERROR: SanFiler::vif_create() . $errmsg");
                return $ClientUtil::FALSE;
            }
        }
    }

    @output=$self->execute($cmd);

    if (grep /vif: / , @output)
    {
        my $errmsg = join "\n", @output;
        $self->_set_err("Error: SanFiler::vif_create() could not "      .
                        "properly create a VIF: name-$name type-$type " .
                        "interfaces-$interfaces: $errmsg");
        return $ClientUtil::FALSE;
    }

    if(defined($ip_address))
    {
        $subnet="255.255.255.0" unless(defined($subnet));

        my $pass = $self->ip_set($name, $ip_address, $subnet);

        if($self->error())
        {
            $self->_set_err("Error: SanFiler::vif_create() could not set " .
                            "ip address");
            return $ClientUtil::FALSE;
        }
    }

    return $ClientUtil::TRUE;
} # end sub vif create

###############################################################################
# Method:    vif_exist
# Objective: Checks to see if a vif exists
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vif_exist
{
    my ($self, $name) = @_;
    my ($cmd, @output);

    $self->_clear_err();

    $self->sanlog(-msg => "vif exist");

    unless (defined $name and length $name)
    {
        $self->_set_err("ERROR: SanFiler::vif_exist() Required " .
                        "parameter \$name not found");
        return $ClientUtil::FALSE;
    }

    $cmd = "ifconfig $name";

    @output = $self->execute($cmd);
    my $outputs = join("\n", @output);

    if($outputs eq "ifconfig: $name: no such interface")
    {
        $self->_set_err("Error: SanFiler::vif_exist() vif not found");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
} # end sub vif_exist

###############################################################################
# Method:    vif_destroy
# Objective: Destroys a virtual interface
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vif_destroy
{
    my ($self, $name) = @_;
    my ($cmd, @output);

    $self->_clear_err();

    $self->sanlog(-msg => "vif destroy");

    unless (defined $name and length $name)
    {
        $self->_set_err("ERROR: SanFiler::vif_destroy() Required " .
                        "parameter \$name not found");
        return $ClientUtil::FALSE;
    }

    $cmd = "vif destroy $name";

    @output = $self->execute($cmd);

    if(grep /$name is not a valid vifname/, @output)
    {
        my $errmsg = join "\n", @output;
        $self->_set_err("Error: SanFiler::vif_destroy() "          .
                        "could not destroy the vif $name because " .
                        "it doesn't exist: $errmsg");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
} # end sub vif_destroy

###############################################################################
# Method:    vlan_create
# Objective: Creates a vlan on a filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vlan_create
{
    my ($self, %args) = @_;
    my (
        $interface,
        $id,
        $ip_address,
        $subnet,
       );
    my ($cmd, @output);

    $self->_clear_err();

    $self->sanlog(-msg => "vlan create");

    foreach (keys %args)
    {
        if (/-?interface/)
        {
            length($interface = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::vlan_create invalid value for \"interface\":",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?id/)
        {
            length($id = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::vlan_create invalid value for \"id\":",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?ip_address/)
        {
            length($ip_address = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::vlan_create invalid value for \"ip_address\":",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?subnet/)
        {
            length($subnet = $args{$_}) or
                do {
                    $self->_set_err("SanFiler::vlan_create invalid value for \"subnet\"",
                                    "$args{$_}");
                    return $self;
                };
        }
        else
        {
            $self->_set_err("SanFiler::vlan_create Invalid parameter specified: $_");
            return $self;
        }
    }

    unless (defined $interface and length $interface)
    {
        $self->_set_err("ERROR: SanFiler::vlan_create() Required " .
                        "parameter \$interface not found");
        return $ClientUtil::FALSE;
    }

    $self->ip_offline($interface);

    unless (defined $id and length $id)
    {
        $self->_set_err("ERROR: SanFiler::vlan_create() Required " .
                        "parameter \$id not found");
        return $ClientUtil::FALSE;
    }

    $cmd="vlan create $interface $id";

    @output=$self->execute($cmd);

    if (grep /vlan: $interface-$id has been created/ , @output)
    {
        if(defined($ip_address))
        {
            $subnet="255.255.255.0" unless(defined($subnet));

            $self->ip_set($interface."-".$id, $ip_address, $subnet);

            if($self->error())
            {
                $self->_set_err("Error: SanFiler::vlan_create() could not " .
                                "set ip address");
                return $ClientUtil::FALSE;
            }
        }

        return $ClientUtil::TRUE;
    }
    else
    {
        my $errmsg = join "\n", @output;
        $self->_set_err("Error: SanFiler::vlan_create() could not " .
                        "properly create the Vlan $interface-$id: $errmsg");
        return $ClientUtil::FALSE;
    }
} # end sub vlan_create

###############################################################################
# Method:    vlan_exist
# Objective: Checks if a vlan exists on a filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vlan_exist
{
    my ($self, $interface, $id) = @_;
    my ($cmd, @output);

    $self->_clear_err();

    $self->sanlog(-msg => "vlan exist");

    unless (length $interface)
    {
        $self->_set_err("ERROR: SanFiler::vlan_exist() Required " .
                        "parameter \$interface not found");
        return $ClientUtil::FALSE;
    }

    unless (length $id)
    {
        $self->_set_err("ERROR: SanFiler::vlan_exist() Required " .
                        "parameter \$id not found");
        return $ClientUtil::FALSE;
    }

    $cmd = 'ifconfig ' . $interface . '-' . $id;

    @output = $self->execute($cmd);
    my $outputs = join("\n", @output);

    if($outputs eq "ifconfig: $interface-$id: no such interface")
    {
        $self->_set_err("Error: SanFiler::vlan_exist() interface " .
                        "$interface not found");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
} # end sub vlan_exist

###############################################################################
# Method:    vlan_destroy
# Objective: Destroys a vlan on a filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vlan_destroy
{
    my ($self, $interface, $id) = @_;
    my ($cmd, @output);

    $self->_clear_err();

    $self->sanlog(-msg => "vlan destroy");

    unless (defined $interface and length $interface)
    {
        $self->_set_err("ERROR: SanFiler::vlan_destroy() Required " .
                        "parameter \$interface not found");
        return $ClientUtil::FALSE;
    }

    unless (defined $id and length $id)
    {
        $self->_set_err("ERROR: SanFiler::vlan_destroy() Required " .
                        "parameter \$id not found");
        return $ClientUtil::FALSE;
    }

    $cmd = "vlan delete $interface $id";

    @output = $self->execute($cmd);

    if(grep /vlan: Delete failed./, @output)
    {
        my $errmsg = join "\n", @output;
        $self->_set_err("Error: SanFiler::vlan_destroy() could not " .
                        "destroy the vlan $interface-$id: $errmsg");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
} # end sub vlan_destroy

################################################################################
# Subroutine Name: filter_inits_by_subnet()
# Objective:       Remove initiators from the $inits array that are inaccessible
#                  from this filers interfaces and are not on the same subnets
################################################################################
sub filter_inits_by_subnet {
    my ($self, %args) = @_;

    my ($inits, @final_inits_list, @matched_subnets_summary, @unmatched_subnets_summary,
        @management_ports_summary, $subnet_matching, $allow_mgmt_port);

    $subnet_matching = 1;
    $allow_mgmt_port = 0;

    # parse arguments
    foreach (keys %args) {
        if (/-?inits/) {
            if (not defined($inits = $args{$_})) {
                $self->_set_err("Invalid value for argument '-inits' => $inits");
                return (wantarray()) ? () : [];
            }
            next;
        } elsif (/-?subnet_matching/) {
            if (not defined($subnet_matching = $args{$_})) {
                $self->_set_err("Invalid value for argument '-subnet_matching' => $subnet_matching");
                return (wantarray()) ? () : [];
            }
        } elsif (/-?allow_mgmt_port/) {
            if (not defined($allow_mgmt_port = $args{$_})) {
                $self->_set_err("Invalid value for argument '-allow_mgmt_port' => $allow_mgmt_port");
                return (wantarray()) ? () : [];
            }
        }
    }

    # check for required params
    if (not scalar($inits)) {
        $self->_set_err("Invalid value for argument '-inits'");
        return (wantarray()) ? () : [];
    }

    # get the target interface information for this filer
    my @targets = $self->get_iscsi_targets();
    if ($self->error()) {
        return (wantarray()) ? () : [];
    }

    # search through the inits and compare the subnets with each interface
    foreach my $initiator (keys(%{$inits})) {
        my $init = $inits->{$initiator};
        my $check_ipv4_subnet = 0;
        my $check_ipv6_subnet = 0;

        # check if this initiator has an ipv4 address
        my $init_ipv4 = $init->{ipv4};
        my $init_ipv4_subnet;
        if (defined($init_ipv4) and $init_ipv4 =~ /\d+\.\d+\.\d+\.\d+/) {
            $check_ipv4_subnet = 1;
            $init_ipv4_subnet = $init->{ipv4_subnet};
        }

        # check if this target has an ipv6 address
        my $init_ipv6 = $init->{ipv6};
        my $init_ipv6_subnet;
        if (defined($init_ipv6)) {
            $check_ipv6_subnet = 1;
            $init_ipv6_subnet = $init->{ipv6_subnet};
        }

        # If both of the IPVx subnets are not defined, notify user and skip
        if (not defined($init_ipv4_subnet) and not defined($init_ipv6_subnet) and
            $check_ipv4_subnet == 0 and $check_ipv6_subnet == 0) {
            $self->sanlog(-msg => "SanFiler::_filter_inits_by_subnet - No valid subnet found for IPv4 or IPv6, " .
                                  "please check your clients configurations.");
            next;
        }

        # skip this target if its a management interface
        if ($init->{is_mgmt_interface} == 1 and $allow_mgmt_port == 0) {
            push(@management_ports_summary, "  $init_ipv4 $init_ipv6\n");
            next;
        }

        my $subnet_match = 0;

        # if requested to do subnet matching, do so now
        if ($subnet_matching == 1) {
            # check if the subnets of any of the interfaces on this controller match the
            # subnets of this initiator for both ipv4 and ipv6
            foreach my $target (@targets) {
                if ($check_ipv4_subnet) {
                    if ($target->get_ipv4_subnet() eq $init_ipv4_subnet) {
                        $subnet_match = 1;
                        last;
                    }
                }

                if ($check_ipv6_subnet) {
                    if ($target->get_ipv6_subnet() eq $init_ipv6_subnet) {
                        $subnet_match = 1;
                        last;
                    }
                }
            }
        } else {
            # otherwise ignore the subnet matching.
            $subnet_match = 1;
        }

        # if a subnet matched this target, push it onto the final array
        if ($subnet_match == 1) {
            push(@final_inits_list, $init);
            push(@matched_subnets_summary,   "  $init_ipv4 $init_ipv6\n");
        } else {
            push(@unmatched_subnets_summary, "  $init_ipv4 $init_ipv6\n");
        }
    }

    my @summary;
    # print out the summary of what was filtered out of the inits list
    if (scalar(@matched_subnets_summary)) {
        push(@summary, "The following initiators are accessible by this controller:\n");
        push(@summary, @matched_subnets_summary);
    }
    if (scalar(@unmatched_subnets_summary)) {
        push(@summary, "\nThe following initiators are NOT accessible by this controller " .
                       "(and have been excluded from the target discovery list):\n");
        push(@summary, @unmatched_subnets_summary);
    }
    if (scalar(@management_ports_summary)) {
        push(@summary, "\nThe following initiators are the management ports of the controllers " .
                       "(and have been excluded from the target discovery list):\n");
        push(@summary, @management_ports_summary);
    }

    $self->sanlog(-msg => "@summary");

    # if no initiators are available, set error.
    if (scalar(@final_inits_list) == 0) {
        $self->_set_err("There are no initiators accessible by any of these controllers target ports.");
        return (wantarray()) ? () : [];
    }

    return (wantarray()) ? @final_inits_list : \@final_inits_list;
}

###############################################################################
# Method:    ifconfig_get_mtu_size
# Objective: Gets the mtusize for the provided interface
# Details:   See POD documentation at the end of this file.
###############################################################################
sub ifconfig_get_mtu_size
{
    my ($self, $interface) = @_;
    my ($cmd, @output);

    $self->_clear_err();

    $self->sanlog(-msg => "ifconfig get mtu size");

    unless (defined $interface and length $interface)
    {
        $self->_set_err("ERROR: SanFiler::ifconfig_get_mtu_size() Required " .
                        "parameter \$interface not found");
        return $ClientUtil::FALSE;
    }

    $cmd = "ifconfig $interface";

    @output=$self->execute($cmd);

    if(grep /ifconfig: / , @output)
    {
        my $errmsg = join "\n", @output;
        $self->_set_err("Error: SanFiler::iscsi_get_mtu_size() could " .
                        "not get mtusize for $interface: $errmsg");
        return $ClientUtil::FALSE;
    }

    foreach my $line (@output)
    {
        if($line =~ /.* mtu (\d+).*/)
        {
            return $1;
        }
    }
} # end sub ifconfig_get_mtu_size

###############################################################################
# Method:    ifconfig_set_mtu_size
# Objective: Sets the MTU size on a given interface.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub ifconfig_set_mtu_size
{
    my ($self, $interface, $mtusize) = @_;
    my (
        $cmd,
        @rcfile_from_filer,
        $line,
        @output,
        );

    $self->_clear_err();

    $self->sanlog(-msg => "ifconfig set mtu size");

    unless (defined $interface and length $interface)
    {
        $self->_set_err("ERROR: SanFiler::ifconfig_set_mtu_size() Required " .
                        "parameter \$interface not found");
        return $ClientUtil::FALSE;
    }

    unless (defined $mtusize and length $mtusize)
    {
        $self->_set_err("ERROR: SanFiler::ifconfig_set_mtu_size() Required " .
                        "parameter \$mtusize not found");
        return $ClientUtil::FALSE;
    }

    $cmd="ifconfig $interface mtusize $mtusize";
    @output=$self->execute($cmd);

    if(grep /ifconfig: $interface/ , @output)
    {
        my $errmsg = join "\n", @output;
        $self->_set_err("Error: SanFiler::iscsi_set_mtu_size() could not " .
                        "set mtu on $interface to $mtusize: $errmsg");
        return $ClientUtil::FALSE;
    }

    $cmd="rdfile /etc/rc";
    @rcfile_from_filer=$self->execute($cmd);

    $cmd="priv set advanced; rm /etc/rc";
    $self->execute($cmd);

    foreach $line (@rcfile_from_filer)
    {
        if($line =~ /^\s*ifconfig $interface.*/)
        {
            if($line =~ /mtusize/)
            {
                $line =~ s/mtusize \d+/mtusize $mtusize/g;
            }
            else
            {
                $line = $line . " mtusize " . $mtusize;
            }
        }

        $cmd='wrfile -a /etc/rc "'.$line.'"';
        @output=$self->execute($cmd);
    }

    return $ClientUtil::TRUE;
} # end sub ifconfig_set_mtu_size

###############################################################################
# Method:    fcp_adapter_online
# Objective: Makes sure an FCP adapter is online.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub fcp_adapter_online
{
    my ($self, %args) = @_;
    my $adapter;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg => "SanFiler::fcp_adapter_online() fcp adapter online",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    if(defined($args{"-adapter"}))
    {
        $adapter = $args{"-adapter"};
    }
    else
    {
        $self->_set_err("ERROR: SanFiler::fcp_adapter_online() Required " .
                        "parameter \$adapter not found");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref =
    $cmd_obj->execute_cmd(
                    -cmd                    => "fcp-adapter-config-up",
                    -cmd_filer              => $self->get_name(),
                    -check_result           => 1,
                    -cmd_interface          => "zapi",
                    "-fcp-adapter"          => $adapter);

    my $output_hash = $self->xml2hash($resultsHash_ref->{output});
    if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
    {
        my $dumpOut = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => DUMPER_DEBUG_LEVEL);
    }

    my $status = $output_hash->{status};

    if($status eq "failed")
    {
        $self->_set_err("Error: SanFiler::fcp_adapter_online: " .
                        "Unable to online adapter",
                        $output_hash->{reason});
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
} # end sub fcp_adapter_online

###############################################################################
# Method:    fcp_setup_lun_trace
# Objective: Sets up lun tracing
# Details:   See POD documentation at the end of this file.
###############################################################################
sub fcp_setup_lun_trace
{
    my ($self, $level) = @_;
    my ($cmd, @output);

    $self->_clear_err();

    $self->sanlog(-msg => "fcp setup lun trace");

    $level = "3" unless(defined($level));
    $cmd = "options lun.trace_level $level";
    @output = $self->execute($cmd);

    if(grep /option lun.trace_level: /, @output)
    {
        my $errmsg = join "\n", @output;
        $self->_set_err("Error: SanFiler::fcp_setup_lun_trace() " .
                        "did not complete properly: $errmsg");
        return $ClientUtil::FALSE;
    }

    $cmd = "priv set advanced; rm /etc/syslog.conf";
    $self->execute($cmd);

    $cmd = "rdfile /etc/syslog.conf.sample";
    @output = $self->execute($cmd);

    foreach my $line (@output)
    {
        if($line =~ /\/dev\/console/)
        {
            $line =~ s/\*.info/\*.debug/g;
        }
        if($line =~ /\/etc\/messages/)
        {
            $line =~ s/\*.info/\*.debug/g;
        }

        $cmd = 'wrfile -a /etc/syslog.conf "'.$line.'"';
        $self->execute($cmd);
    }

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    fcp_nameserver_list_iter_start
# Objective: Get the fabric nameserver objects
# Details:   See POD documentation at the end of this file
###############################################################################
sub fcp_nameserver_list_iter_start {
    my ($self, %args) = @_;
    my ($adapter_name, $interface, $cmd_obj, $nameserver_list);

    # parse arguments
    foreach (keys %args) {
        if (/-?adapter_name/) {
            if (not defined($adapter_name = $args{$_})) {
                $self->_set_err("Invalid value for argument -adapter_name ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?adapter_...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fcp_nameserver_list_iter_start() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fcp-adapter-nameserver-list-iter-start",
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    if ($adapter_name) {
        $cmd_args{"-fcp-adapter"} = $adapter_name;
    } # end if ($adapter_...

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fcp-adapter-nameserver-list"
                                        ."-iter-start: $errmsg");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fcp-adapter-nameserver-list-iter-start Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...

        $nameserver_list->{'records'} = $output_hash->{'records'};
        $nameserver_list->{'tag'}     = $output_hash->{'tag'};
        my $dumpOut     = Dumper($nameserver_list);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 7);
    } # end if ($interface...

    return $nameserver_list;

} ## end sub fcp_nameserver_list_iter_start()

###############################################################################
# Method:    fcp_nameserver_list_iter_next
# Objective: Return items from a fcp-adapter-nameserver-list-iter-start Info
# Details:   See POD documentation at the end of this file
###############################################################################
sub fcp_nameserver_list_iter_next {
    my ($self, %args) = @_;
    my ($maximum, $interface, $cmd_obj, $tag, $nameserver_list_info);

    # parse arguments
    foreach (keys %args) {
        if (/-?maximum/) {
            if (not defined($maximum = $args{$_})) {
                $self->_set_err("Invalid value for argument -maximum ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } elsif (/-?tag/) {
            if (not defined($tag = $args{$_})) {
                $self->_set_err("Invalid value for argument -tag ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?maximum...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    # check fort the required parameters.
    unless (defined $maximum and length $maximum) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-maximum'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    unless (defined $tag and length $tag) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-tag'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fcp_nameserver_list_iter_next() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fcp-adapter-nameserver-list-iter-next",
                "-maximum"       => $maximum,
                "-tag"           => $tag,
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fcp-adapter-nameserver-list"
                                        ."-iter-next: $errmsg");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fcp-adapter-nameserver-list-iter-next Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...

        $nameserver_list_info->{'records'} = $output_hash->{'records'};
        $nameserver_list_info->{'fcp-adapter-nameserver-objects'} =
        $output_hash->{'fcp-adapter-nameserver-objects'};
        my $dumpOut     = Dumper($nameserver_list_info);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 7);
    } # end if ($interface...

    return $nameserver_list_info;

} ## end sub fcp_nameserver_list_iter_next()

###############################################################################
# Method:    fcp_nameserver_list_iter_end
# Objective: Terminate a list iteration and clean up any saved info.
# Details:   See POD documentation at the end of this file
###############################################################################
sub fcp_nameserver_list_iter_end {
    my ($self, %args) = @_;
    my ($interface, $cmd_obj, $tag);

    # parse arguments
    foreach (keys %args) {
        if (/-?tag/) {
            if (not defined($tag = $args{$_})) {
                $self->_set_err("Invalid value for argument -tag ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?tag...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    # check for the required  paramters
    unless (defined $tag and length $tag) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-tag'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fcp_nameserver_list_iter_end() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fcp-adapter-nameserver-list-iter-end",
                "-tag"           => $tag,
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fcp-adapter-nameserver-list"
                                        ."-iter-end: $errmsg");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fcp-adapter-nameserver-list-iter-end Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...

    } # end if ($interface...

    return $ClientUtil::TRUE;

} ## end sub fcp_nameserver_list_iter_end()

###############################################################################
# Method:    fcp_topology_list_iter_start
# Objective: Get the fabric topology for one or more switches
# Details:   See POD documentation at the end of this file
###############################################################################
sub fcp_topology_list_iter_start {
    my ($self, %args) = @_;
    my ($adapter_name, $interface, $cmd_obj, $topology_list);

    # parse arguments
    foreach (keys %args) {
        if (/-?adapter_name/) {
            if (not defined($adapter_name = $args{$_})) {
                $self->_set_err("Invalid value for argument -adapter_name ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?adapter_...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fcp_topology_list_iter_start() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fcp-adapter-topology-list-iter-start",
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    if ($adapter_name) {
        $cmd_args{"-fcp-adapter"} = $adapter_name;
    }

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fcp-adapter-topology-list"
                                        ."-iter-start: $errmsg");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fcp-adapter-topology-list-iter-start Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...

        $topology_list->{'records'} = $output_hash->{'records'};
        $topology_list->{'tag'}     = $output_hash->{'tag'};
        my $dumpOut     = Dumper($topology_list);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 7);
    } # end if ($interface...

    return $topology_list;

} ## end sub fcp_topology_list_iter_start()

###############################################################################
# Method:    fcp_topology_list_iter_next
# Objective: Return items from a fcp-adapter-topology-list-iter-start Info
# Details:   See POD documentation at the end of this file
###############################################################################
sub fcp_topology_list_iter_next {
    my ($self, %args) = @_;
    my ($maximum, $interface, $cmd_obj, $tag, $topology_list_info);

    # parse arguments
    foreach (keys %args) {
        if (/-?maximum/) {
            if (not defined($maximum = $args{$_})) {
                $self->_set_err("Invalid value for argument -maximum ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } elsif (/-?tag/) {
            if (not defined($tag = $args{$_})) {
                $self->_set_err("Invalid value for argument -tag ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?maximum...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    # check fort the required parameters.
    unless (defined $maximum and length $maximum) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-maximum'");
        return $ClientUtil::FALSE;
    }  # end unless(defined...

    unless (defined $tag and length $tag) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-tag'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fcp_topology_list_iter_next() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fcp-adapter-topology-list-iter-next",
                "-maximum"       => $maximum,
                "-tag"           => $tag,
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fcp-adapter-topology-list"
                                        ."-iter-next: $errmsg");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fcp-adapter-topology-list-iter-next Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...
        $topology_list_info->{'records'} = $output_hash->{'records'};
        $topology_list_info->{'fcp-adapter-topology-switches'} =
                            $output_hash->{'fcp-adapter-topology-switches'};

        my $dumpOut     = Dumper($topology_list_info);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 7);
    } # end if ($interface...

    return $topology_list_info;

} ## end sub fcp_topology_list_iter_next()

###############################################################################
# Method:    fcp_topology_list_iter_end
# Objective: Terminate a list iteration and clean up any saved info.
# Details:   See POD documentation at the end of this file
###############################################################################
sub fcp_topology_list_iter_end {
    my ($self, %args) = @_;
    my ($interface, $cmd_obj, $tag);

    # parse arguments
    foreach (keys %args) {
        if (/-?tag/) {
            if (not defined($tag = $args{$_})) {
                $self->_set_err("Invalid value for argument -tag ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?tag...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    # check for the required  paramters
    unless (defined $tag and length $tag) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-tag'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fcp_topology_list_iter_end() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fcp-adapter-topology-list-iter-end",
                "-tag"           => $tag,
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fcp-adapter-topology-list"
                                        ."-iter-end: $errmsg");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fcp-adapter-topology-list-iter-end Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...

        return $ClientUtil::TRUE;
    } # end if ($interface...

} ## end sub fcp_topology_list_iter_end()

###############################################################################
# Method:    fcp_zone_list_iter_start
# Objective: Get the active zone set information from the zone server.
# Details:   See POD documentation at the end of this file
###############################################################################
sub fcp_zone_list_iter_start {
    my ($self, %args) = @_;
    my ($adapter_name, $interface, $cmd_obj, $zone_list);

    # parse arguments
    foreach (keys %args) {
        if (/-?adapter_name/) {
            if (not defined($adapter_name = $args{$_})) {
                $self->_set_err("Invalid value for argument -adapter_name ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?adapter_...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fcp_zone_list_iter_start() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fcp-adapter-zone-list-iter-start",
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    if ($adapter_name) {
        $cmd_args{"-fcp-adapter"} = $adapter_name;
    } # end if ($adapter_...

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fcp-adapter-zone-list"
                                        ."-iter-start: $errmsg");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fcp-adapter-zone-list-iter-start Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...

        $zone_list->{'records'} = $output_hash->{'records'};
        $zone_list->{'tag'}     = $output_hash->{'tag'};
        my $dumpOut     = Dumper($zone_list);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 7);
    } # end if ($interface...

    return $zone_list;

} ## end sub fcp_zone_list_iter_start()

###############################################################################
# Method:    fcp_zone_list_iter_next
# Objective: Returns information returned for each zone entry
# Details:   See POD documentation at the end of this file
###############################################################################
sub fcp_zone_list_iter_next {
    my ($self, %args) = @_;
    my ($maximum, $interface, $cmd_obj, $tag, $zone_list_info);

    # parse arguments
    foreach (keys %args) {
        if (/-?maximum/) {
            if (not defined($maximum = $args{$_})) {
                $self->_set_err("Invalid value for argument -maximum ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } elsif (/-?tag/) {
            if (not defined($tag = $args{$_})) {
                $self->_set_err("Invalid value for argument -tag ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?maximum...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    # check fort the required parameters.
    unless (defined $maximum and length $maximum) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-maximum'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    unless (defined $tag and length $tag) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-tag'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fcp_zone_list_iter_next() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fcp-adapter-zone-list-iter-next",
                "-maximum"       => $maximum,
                "-tag"           => $tag,
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fcp-adapter-zone-list"
                                        ."-iter-next: $errmsg");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fcp-adapter-zone-list-iter-next Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...

        $zone_list_info->{'records'} = $output_hash->{'records'};
        $zone_list_info->{'fcp-adapter-zones'} =
                        $output_hash->{'fcp-adapter-zones'};
        my $dumpOut     = Dumper($zone_list_info);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 7);
    } # end if ($interface...

    return $zone_list_info;

} ## end sub fcp_zone_list_iter_next()

###############################################################################
# Method:    fcp_zone_list_iter_end
# Objective: Terminate a list iteration and clean up any saved info.
# Details:   See POD documentation at the end of this file
###############################################################################
sub fcp_zone_list_iter_end {
    my ($self, %args) = @_;
    my ($interface, $cmd_obj, $tag);

    # parse arguments
    foreach (keys %args) {
        if (/-?tag/) {
            if (not defined($tag = $args{$_})) {
                $self->_set_err("Invalid value for argument -tag ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?tag...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    # check for the required  paramters
    unless (defined $tag and length $tag) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-tag'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fcp_zone_list_iter_end() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fcp-adapter-zone-list-iter-end",
                "-tag"           => $tag,
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fcp-adapter-zone-list"
                                        ."-iter-end: $errmsg");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fcp-adapter-zone-list-iter-end Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...

        return $ClientUtil::TRUE;
    } # end if ($interface...

} ## end sub fcp_zone_list_iter_end()

###############################################################################
# Method:    fc_config_list_iter_start
# Objective: Start iteration through the list of adapter config information.
# Details:   See POD documentation at the end of this file
###############################################################################
sub fc_config_list_iter_start {
    my ($self) = @_;
    my ($interface, $cmd_obj, $config_list);

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fc_config_list_iter_start() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fc-config-list-iter-start",
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fc-config-list-iter-start :");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fc-config-list-iter-start Failed");
            return $ClientUtil::FALSE;
        } # if ($output_...

        $config_list->{'records'} = $output_hash->{'records'};
        $config_list->{'tag'}     = $output_hash->{'tag'};
        my $dumpOut = Dumper($config_list);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 7);
    } # end if ($interface...

    return $config_list;

} ## end sub fc_config_list_iter_start()

###############################################################################
# Method:    fc_config_list_iter_next
# Objective: Obtain a list of FC adapter configuration information for adapters
# Details:   See POD documentation at the end of this file
###############################################################################
sub fc_config_list_iter_next {
    my ($self, %args) = @_;
    my ($maximum, $interface, $cmd_obj, $tag, $config_list_info);

    # parse arguments
    foreach (keys %args) {
        if (/-?maximum/) {
            if (not defined($maximum = $args{$_})) {
                $self->_set_err("Invalid value for argument -maximum ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } elsif (/-?tag/) {
            if (not defined($tag = $args{$_})) {
                $self->_set_err("Invalid value for argument -tag ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?maximum...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    # check fort the required parameters.
    unless (defined $maximum and length $maximum) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-maximum'");
        return $ClientUtil::FALSE;
    }  # end unless(defined...

    unless (defined $tag and length $tag) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-tag'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fc_config_list_iter_next() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fc-config-list-iter-next",
                "-maximum"       => $maximum,
                "-tag"           => $tag,
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fc-config-list-iter-next:");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fc-config-list-iter-next Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...

        $config_list_info->{'records'} = $output_hash->{'records'};
        $config_list_info->{'adapter-info'} = $output_hash->{'adapter-info'};
        my $dumpOut     = Dumper($config_list_info);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 7);
    } # end if ($interface...

    return $config_list_info;

} ## end sub fc_config_list_iter_next()

###############################################################################
# Method:    fc_config_list_iter_end
# Objective: Terminate the fc-config-list iteration.
# Details:   See POD documentation at the end of this file
###############################################################################
sub fc_config_list_iter_end {
    my ($self, %args) = @_;
    my ($interface, $cmd_obj, $tag);

    # parse arguments
    foreach (keys %args) {
        if (/-?tag/) {
            if (not defined($tag = $args{$_})) {
                $self->_set_err("Invalid value for argument -tag ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?tag...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    # check for the required  paramters
    unless (defined $tag and length $tag) {
        $self->_set_err(  "Missing or invalid value for "
                        . "required parameter '-tag'");
        return $ClientUtil::FALSE;
    } # end unless(defined...

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fc_config_list_iter_end() ",
                  -msg_level => 1);

    my %cmd_args = (
                "-cmd"           => "fc-config-list-iter-end",
                "-tag"           => $tag,
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
            );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fc-config-list-iter-end: ");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fc-config-list-iter-end Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...

    } # end if ($interface...

    return $ClientUtil::TRUE;

} ## end sub fc_config_list_iter_end()

###############################################################################
# Method:    fc_config_set_adapter_fc_type
# Objective: Changes the adapter driver and/or configuration state.
# Details:   See POD documentation at the end of this file
###############################################################################
sub fc_config_set_adapter_fc_type {
    my ($self, %args) = @_;
    my ($adapter_name, $fc_type, $interface, $cmd_obj);

    # parse arguments
    foreach (keys %args) {
        if (/-?adapter_name/) {
            if (not defined($adapter_name = $args{$_})) {
                $self->_set_err("Invalid value for argument -adapter_name ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } elsif (/-?fc_type/) {
            if (not defined($fc_type = $args{$_})) {
                $self->_set_err("Invalid value for argument -fc_type ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?adapter_...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fc_config_set_adapter_fc_type() ",
                  -msg_level => 1);

    # check for reqires parameter
    unless (defined $adapter_name) {
        $self->_set_err("Missing required argument '-adapter_name'");
        return $ClientUtil::FALSE;
    } # end unless ($defined...

    unless (defined $fc_type) {
        $self->_set_err("Missing required argument '-fc_type'");
        return $ClientUtil::FALSE;
    } # end unless ($defined...

    my %cmd_args = (
                "-cmd"           => "fc-config-set-adapter-fc-type",
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
                "-adapter-name"  => $adapter_name,
                "-fc-type"       => $fc_type,
            );

    # execute using Cmd_driver
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fc-config-set-adapter-fc-type: ");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fc-config-set-adapter-fc-type Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...
    } # end if ($interface...

    return $ClientUtil::TRUE;

} ## end sub fc_config_set_adapter_fc_type()

###############################################################################
# Method:    fc_config_adapter
# Objective: Brings the adapter online or offline
# Details:   See POD documentation at the end of this file
###############################################################################
sub fc_config_adapter {
    my ($self, %args) = @_;
    my ($adapter_name, $interface, $cmd_obj, $state);
    my %cmd_args;

    # parse arguments
    foreach (keys %args) {
        if (/-?adapter_name/) {
            if (not defined($adapter_name = $args{$_})) {
                $self->_set_err("Invalid value for argument -adapter_name ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } elsif (/-?state/) {
            if (not defined($state = $args{$_})) {
                $self->_set_err("Invalid value for argument -state ");
                return $ClientUtil::FALSE;
            } # end if (not defined...
            next;
        } else {
            $self->_set_err("Invalid value parameters passed ");
            return $ClientUtil::FALSE;
        } # end if (/-?adapter_...
    } ## end foreach (keys %args)

    # Implemented only in zapi
    $interface = "zapi";
    $cmd_obj = $self->cmd_obj();

    $self->_clear_err();
    $self->sanlog(-msg => "SanFiler::fc_config_adapter() ",
                  -msg_level => 1);

    # check for reqires parameter
    unless (defined $adapter_name) {
        $self->_set_err("Missing required argument '-adapter_name'");
        return $ClientUtil::FALSE;
    } # end unless ($defined...

    # check for reqires parameter
    unless (defined $state) {
        $self->_set_err("Missing required argument '-state'");
        return $ClientUtil::FALSE;
    } # end unless ($defined...

    if ($state =~ /enable/i) {
        %cmd_args = (
                "-cmd"           => "fc-config-adapter-enable",
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
                "-adapter-name"  => $adapter_name,
            );
    } elsif ($state =~ /disable/i) {
        %cmd_args = (
                "-cmd"           => "fc-config-adapter-disable",
                "-cmd_filer"     => $self->get_name(),
                "-check_result"  => 1,
                "-check_output"  => 1,
                "-cmd_interface" => $interface,
                "-adapter-name"  => $adapter_name,
            );
    } else {
        $self->_set_err("Invalid value for state");
        return $ClientUtil::FALSE;
    } # end if ($state...

    # execute using Cmd_driver
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fc-config-adapter-enable: ");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    unless ($resultsHash_ref->{output_check}) {
        $self->_set_err("Output check failed: $output");
        return $ClientUtil::FALSE;
    } # end unless($result...

    if ($interface eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if($output_hash->{status} eq "failed") {
            $self->_set_err("fc-config-adapter Failed");
            return $ClientUtil::FALSE;
        } # end if ($output_...
    } # end if ($interface...

    return $ClientUtil::TRUE;

} ## end sub fc_config_adapter()

###############################################################################
# Method:    fcp_stats
# Objective: returns the fcp stats
# Details:   See POD documentation at the end of this file
###############################################################################
sub fcp_stats {
    my ($self, %args) = @_;
    # the subroutine will execute fcp stats and fcp stats -z.
    # for fcp stats it will return a hash which contains stats info.
    # for fcp stats -z it will return TRUE or FALSE based on execution.

    my ($adapter, $fcp_stats, $j, $adapter_name, $cmd, $option);
    foreach (keys %args) {
        if (/-?adapter_name/) {
            if (not length($adapter_name = $args{$_})) {
                $self->_set_err("Invalid value for argument '-adapter_name'");
                return $ClientUtil::FALSE;
            } # end if (not length...
            next;
        } elsif (/-?option/) {
            if ( not length($option = $args{$_})) {
                $self->_set_err("Invalid value for argument".
                             " '-option'");
                 return $ClientUtil::FALSE;
            } # end if (not length...
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        } # end if (/-?slot...
    } # end foreach...

    my $cmd_obj = $self->cmd_obj();

    my %cmd_args = (
                "-cmd"           => "fcp-stats",
                "-cmd_filer"     => $self->get_name(),
                "-cmd_interface" => "cli",
            );

    if ($adapter_name) {
        $cmd_args{"-adapter"} = $adapter_name;
    } # end if ($adapter_...

    if ($option) {
        $cmd_args{"-option"} = " ";
    } # end if ($adapter_...

    # execute using Cmd_driver
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    if ($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed to execute fcp-stats: ");
        $self->sanlog(-msg => $errmsg);
        return $ClientUtil::FALSE;
    } # end if ($cmd_...

    #
    # Get the output
    #
    my @output = split('\n', $resultsHash_ref->{output});

    if ($option) {
        return $ClientUtil::TRUE;
    } else {
        my $count = @output;
        for (my $i = 0 ; $i < $count ; $i++) {
            if ($output[$i] =~ /Statistics for adapter (\w+)/) {
                my $adapter = $1;
                $j = ++$i;
                while($output[$j] !~ /Statistics for adapter/ && $j < $count
                                            && $output[$j] !~ /Message/) {
                    if ($output[$j] =~ /Statistics for SFP on (\w+)/) {
                        $j++;
                        while($output[$j] !~ /Statistics for/ && $j < $count) {
                            my ($key, $value) = split(/:\s/, $output[$j]);
                            $key =~ s/^\s+//;
                            $value =~ s/^\s+//;
                            if ($key) {
                                $fcp_stats->{'adapters'}->{$adapter}->
                                        {'SFP Statistics'}->{$key} = $value;
                            } # end if ($key...
                            $j++;
                        } # end while ($output[$j...
                    } else {
                        my ($key, $value) = split(/:\s/, $output[$j]);
                        $key =~ s/^\s+//;
                        $value =~ s/^\s+//;
                        if ($key) {
                            $fcp_stats->{'adapters'}->{$adapter}->
                                            {$key} = $value;
                        } # end if ($key...
                        $j++;
                    } # end if ($output[$j...
                } # end while ($output[$j...
            } elsif ($output[$i] =~ /Message:\s+sent completed  received/i) {
                $j = ++$i;
                while ($output[$j] =~ /(\w+)\:\s+(\d+)\s+(\d+)\s+(\d+)/) {
                    $fcp_stats->{'adapters'}->{'vtic'}->
                                        {'Message'}->{$1}->{'sent'} = $2;
                    $fcp_stats->{'adapters'}->{'vtic'}->
                                    {'Message'}->{$1}->{'completed'} = $3;
                    $fcp_stats->{'adapters'}->{'vtic'}->
                                    {'Message'}->{$1}->{'received'} = $4;
                    $j++;
                } # end while ($output[$j...
            } # end if ($output[$i]...
        } # end for (my $i...
    } # end if ($option...

    # For user reference
    my $dumpOut = Dumper($fcp_stats);
    $self->sanlog(-msg       => "outout hash is:\n$dumpOut");
                  #-msg_level => 7);

    return $fcp_stats;

} ## end sub fcp_stats()

###############################################################################
# Method:    roll_logs
# Objective: Roll /etc/messages logs to create clean log file
# Details:   See POD documentation at the end of this file
###############################################################################
sub roll_logs {
    my ($self) = @_;

    my ($cmd, $status, $output, @output);
    my $starting_file = -1;

    # Grab a list of files under /etc and filter out the messages files, at the
    # same time sort them in decending order.
    $cmd = "ls /etc/";
    ($output, $status, @output) = &SanUtil::system_zapi($self->get_name(), "$cmd", "diag", 30);
    @output = sort(grep(/messages/, @output));
    if ($status == SanUtil::ZAPI_FAILURE or scalar(@output) == 0) {
        $self->_set_err("Failed to run command to filer (" . $self->get_name() . ": $cmd)");
        return $ClientUtil::FALSE;
    }

    # Grab each item in the output and strip off trailing white spaces
    @output = map {$_ =~ s/\s+$//g;$_} @output;

    # Grab the last file number from output.
    $starting_file = scalar(@output) - 1;

    # Start with the last file and work our way backwards moving [oldest_file - 1] to [oldest_file]
    # Example of what this will perform:
    # @output contains the following
    # messages, messages.0, messages.1, messages.2, messages.3
    # The last file will me scalar(@output) - 1, in this case this will be 3.
    # The code will take messages.2 and move it to messages.3
    # It will continue to move the files down the list until it moves
    # messages to messages.0
    for (my $i = $starting_file; $i > 0; $i--) {
        my ($from_file, $to_file) = ("/etc/" . $output[$i-1], "/etc/" . $output[$i]);
        $self->sanlog(-msg => "Rolling log file ($from_file) to file ($to_file)");
        $cmd = "mv $from_file $to_file";

        ($output, $status) = &SanUtil::system_zapi($self->get_name(), "$cmd", "diag", 30);
        if ($status == SanUtil::ZAPI_FAILURE) {
            $self->_set_err("Failed to run command to filer (" . $self->get_name() . ": $cmd)");
            return $ClientUtil::FALSE;
        }
    }

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    get_ip_addresses
# Objective: Get a list of all IP addresses assigned to a filer
# Details:   See POD documentation at the end of this file
###############################################################################
sub get_ip_addresses
{
    my ($self, %args) = @_;

    my ($cmd, @output, @ips);

    # Execute an ifconfig command on the filer and parse IPs
    $cmd = "ifconfig -a";
    @output = $self->execute($cmd);
    if(grep /ifconfig: / , @output)
    {
        my $errmsg = join "\n", @output;
        $self->_set_err("Could not retrieve IP addresses for this filer" .
                        $errmsg);
        return (wantarray()) ? (): [];
    }

    # Get IPs from the list and push it into an array.
    List::MoreUtils::apply {
        # IPV4 addresses
        push(@ips, $1) if (/inet\s(\d+\.\d+\.\d+\.\d+)/);
        # IPV6 addresses - Compressed format i.e. fe80::2a0:98ff:fe08:422c
        push(@ips, $1) if (/inet6\s(([0-9A-Fa-f]{1,4}:?:[0-9A-Fa-f]{0,4}){4})\s/);
        # IPV6 addresses - Full i.e. FEDC:BA98:7654:3210:FEDC:BA98:7654:3210
        push(@ips, $1) if (/inet6\s(([0-9A-Fa-f]{1,4}:[0-9A-Fa-f]{1,4}){7})/);
    } @output;

    return (wantarray()) ? @ips: \@ips;
}

################################################################################
# Method:    radius_status
# Objective: Check radius service status.
# Details:   See POD documentation at the end of this file
#     Inputs  none
#     Returns $ClientUtil::TRUE - radius is running
#             $ClientUtil::FALSE - radius is stopped
#             undef - failure and set_err
################################################################################
sub radius_status {
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::radius_status()",
                  -msg_level => 1);

    $resultsHash_ref =
      $cmd_obj->execute_cmd(-cmd           => "radius-service-status",
                            -cmd_filer     => $self->get_name(),
                            -check_output  => 1,
                            -check_result  => 1,
                            -cmd_interface => $self->interface());

    if ($cmd_obj->error()) {
        $self->_set_err("radius status failed: ", $cmd_obj->errmsg());
        return undef;
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (!($resultsHash_ref->{result_check} && $resultsHash_ref->{output_check}))
    {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return undef;
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # Parse the output to determine if radius is running.
        if ($output_hash->{"is-available"} eq "true") {
            $output = $ClientUtil::TRUE;
        } elsif ($output_hash->{"is-available"} eq "false") {
            $output = $ClientUtil::FALSE;
        } else {
            $self->_set_err(  "Could not find true/false in is-available.\n"
                            . "Output was: $output\n");
            return undef;
        }

    } else {    # else CLI
        if ($output =~ /^RADIUS client service is running$/) {
            $output = $ClientUtil::TRUE;
        } elsif ($output =~ /^RADIUS client service is not running$/) {
            $output = $ClientUtil::FALSE;
        }

    }    # end else CLI

    # If code gets here, the command was good.
    return $output;

}    # end radius_status

################################################################################
# Method:    radius_start
# Objective: Start radius service.
# Details:   See POD documentation at the end of this file
#     Inputs  none
#     Returns string - start ok
#             $ClientUtil::FALSE - start failed. (and _set_err)
################################################################################
sub radius_start {
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::radius_start()",
                  -msg_level => 1);

    $resultsHash_ref =
      $cmd_obj->execute_cmd(-cmd           => "radius-service-start",
                            -cmd_filer     => $self->get_name(),
                            -check_output  => 1,
                            -check_result  => 1,
                            -cmd_interface => $self->interface());

    if ($cmd_obj->error()) {
        $self->_set_err("radius start failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # If the service was already started, the zapi will return failed.
        # Intercept this case and return success.
        if ($output_hash->{reason} =~
            /^RADIUS client service is already running$/) {
            return $output_hash->{reason};
        }

        # If we get here, the zapi appears to be good.  Set output
        # to the status.
        $output = $output_hash->{status};

    }    # end if zapi

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $output;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

}    # end radius_start

################################################################################
# Method:    radius_stop
# Objective: Stop radius service.
# Details:   See POD documentation at the end of this file
#     Inputs  none
#     Returns string - stop ok
#             $ClientUtil::FALSE - stop failed. (and _set_err)
################################################################################
sub radius_stop {
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::radius_stop()",
                  -msg_level => 1);

    $resultsHash_ref =
      $cmd_obj->execute_cmd(-cmd           => "radius-service-stop",
                            -cmd_filer     => $self->get_name(),
                            -check_output  => 1,
                            -check_result  => 1,
                            -cmd_interface => $self->interface());

    if ($cmd_obj->error()) {
        $self->_set_err("radius stop failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # If the service was already stopped, the zapi will return failed.
        # Intercept this case and return success.
        if ($output_hash->{reason} =~ /^RADIUS client service is not running$/)
        {
            return $output_hash->{reason};
        }

        # If we get here, the zapi appears to be good.  Set output
        # to the status.
        $output = $output_hash->{status};

    }    # end if zapi

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $output;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

}    # end radius_stop

################################################################################
# Method:    radius_stats_zero
# Objective: Zeroes radius stats.
# Details:   See POD documentation at the end of this file
#     Inputs  none
#     Returns $ClientUtil::TRUE - successful
#             $ClientUtil::FALSE - failed (and _set_err)
################################################################################
sub radius_stats_zero {
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::radius_stats_zero() radius stats -z",
                  -msg_level => 1);

    $resultsHash_ref =
      $cmd_obj->execute_cmd(-cmd           => "radius-reset-stats",
                            -cmd_filer     => $self->get_name(),
                            -check_output  => 1,
                            -check_result  => 1,
                            -cmd_interface => $self->interface());

    if ($cmd_obj->error()) {
        $self->_set_err("radius stats failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
    }    # end if zapi

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

}    # end radius_stats_zero

################################################################################
# Method:    radius_stats
# Objective: Get radius stats and return in hash format.
# Details:   See POD documentation at the end of this file
#     Inputs  none
#     Returns $ClientUtil::FALSE - stats failed. (and _set_err)
#             hash of output on success
################################################################################
sub radius_stats {
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my $output_hash;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::radius_stats()",
                  -msg_level => 1);

    $resultsHash_ref =
      $cmd_obj->execute_cmd(-cmd           => "radius-stats-list-info",
                            -cmd_filer     => $self->get_name(),
                            -check_output  => 1,
                            -check_result  => 1,
                            -cmd_interface => $self->interface());

    if ($cmd_obj->error()) {
        $self->_set_err("radius stats failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (!($resultsHash_ref->{result_check} && $resultsHash_ref->{output_check}))
    {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        $output_hash = $self->xml2hash($output);
        my $dumpOut = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # else cli.  Convert the CLI output to hash.
    } else {
        $output_hash = $self->_radius_stats_cli_to_hash(-cli_output => $output);

        # If output hash does not exist, return error.
        if (!$output_hash) {
            $self->_set_err("radius_stats_cli_to_hash failed",
                            $self->errmsg(), $resultsHash_ref->{output});
            return $ClientUtil::FALSE;
        }
    } ## end else [ if ($self->interface()...

    # If code gets here, the command was good.
    # Also send back actual text output.
    $output_hash->{text_output} = $output;

    return $output_hash;

}    # end radius_stats

################################################################################
# Method:    _radius_stats_cli_to_hash
# Objective: Convert radius stats CLI output into hash format.
# Details:
#     Inputs  -cli_output - required - $output from radius_stats cli library
#                 call
#     Returns $stats_hash - hash in the same format as radius stats ZAPI cmd
#             $ClientUtil::FALSE - error condition (and set_err)
################################################################################
sub _radius_stats_cli_to_hash {
    my ($self, %args) = @_;

    my $cli_output;    # arg - required - $output from radius_stats library call
    my $stats_hash;
    my @stats = ("access-accepted-packets", "access-challenged-packets",
                 "access-rejected-packets", "authentication-requests",
                 "denied-packets",          "late-packets",
                 "retransmitted-packets",   "short-packets",
                 "timed-out-packets",       "unknown-packets",
                 "unknown-server-packets");

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::_radius_stats_cli_to_hash()",
                  -msg_level => 3);

    # Parse arguments
    foreach (keys %args) {
        if (/-?cli_output/) {
            $cli_output = $args{$_};
            if (not length($cli_output)) {
                $self->_set_err("Invalid value for argument '-cli_output'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $cli_output) {
        $self->_set_err("Missing required argument '-cli_output'");
        return $ClientUtil::FALSE;
    }

    $self->sanlog(
               -msg => "_radius_stats_cli_to_hash: cli_output=" . "$cli_output",
               -msg_level => 7);

    # Expected CLI output for what to parse:
    # rtpqa-fas6030-13> radius stats
    # RADIUS client statistics
    #   RADIUS access-accepted-packets:    0
    #   RADIUS access-challenged-packets:  0
    #   RADIUS access-rejected-packets:    0
    #   RADIUS authentication-requests:    0
    #   RADIUS denied-packets:             0
    #   RADIUS late-packets:               0
    #   RADIUS retransmitted-packets:      0
    #   RADIUS short-packets:              25
    #   RADIUS timed-out-packets:          0
    #   RADIUS unknown-packets:            0
    #   RADIUS unknown-server-packets:     0

    # Split the CLI output into individual lines
    my @output_array = split(/\n/, $cli_output);

    # Verify 1st line of array has expected results.
    if ($output_array[0] ne "RADIUS client statistics") {
        $self->_set_err(
                 "Radius stats, unexpected output on line 0: $output_array[0]");
        return $ClientUtil::FALSE;
    }

    # Remove the 1st line to continue processing in the loop.
    shift @output_array;

    # If there are no more elements to process, then there is a big problem.
    if (!scalar(@output_array)) {
        $self->_set_err("Radius statistics are missing.");
        return $ClientUtil::FALSE;
    }

    # i will be used as an index into the array.
    my $i = 0;

    # Loop through all statistics.
    foreach my $str (@output_array) {

        # Notice the use of the stats array with index $i.
        # Save the value into the hash.
        if ($str =~ /^  RADIUS $stats[$i]:\s+(\d+)$/) {
            $stats_hash->{"radius-stats"}->{"radius-stats-info"}
              ->{"$stats[$i]"} = $1;
        } else {
            $self->_set_err("Unexpected line in radius stats: $str");
            return $ClientUtil::FALSE;
        }

        $i++;

    }    # end forach $str

    # If $i does not equal the numbers of stats in the stats array,
    # then we didn't process all of the expected stats.
    if ($i != scalar @stats) {
        $self->_set_err("Did not find expected number of radius stats");
        return $ClientUtil::FALSE;
    }

    return $stats_hash;

}    # end _radius_stats_cli_to_hash

################################################################################
# Method:    radius_show
# Objective: Displays radius information and returns a hash.
# Details:   See POD documentation at the end of this file
#     Inputs  none
#     Returns $ClientUtil::FALSE - show failed. (and _set_err)
#             Ptr to radius_hash on success
################################################################################
sub radius_show {
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my $output_hash;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::radius_show()",
                  -msg_level => 1);

    $resultsHash_ref =
      $cmd_obj->execute_cmd(-cmd           => "radius-show-info",
                            -cmd_filer     => $self->get_name(),
                            -check_output  => 1,
                            -check_result  => 1,
                            -cmd_interface => $self->interface());

    if ($cmd_obj->error()) {
        $self->_set_err("radius show failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (!($resultsHash_ref->{result_check} && $resultsHash_ref->{output_check}))
    {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    $self->sanlog(-msg       => "radius_show cmd returns:\n $output",
                  -msg_level => 7);

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        $output_hash = $self->xml2hash($output);
        my $dumpOut = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # If there is only 1 element, the xml2hash returns a HASH instead of
        # ARRAY.
        # In this case, remove the HASH and add the element back in as an ARRAY.
        # This allows the ZAPI hash to match the CLI hash, and easier processing
        # of the output, since everything will always be an ARRAY.
        if (ref($output_hash->{"radius-server-list-entries"}
                  ->{"radius-server-list-entry-info"}) eq "HASH"
          ) {

            # Save the ip and port
            my $info =
              $output_hash->{"radius-server-list-entries"}
              ->{"radius-server-list-entry-info"}->{"radius-server-info"};
            my $ip =
              $output_hash->{"radius-server-list-entries"}
              ->{"radius-server-list-entry-info"}->{"radius-server-ip"};
            my $port =
              $output_hash->{"radius-server-list-entries"}
              ->{"radius-server-list-entry-info"}->{"radius-server-port"};

            # Delete the HASH
            delete $output_hash->{"radius-server-list-entries"}
              ->{"radius-server-list-entry-info"};

            # Add the information back in as an ARRAY.
            $output_hash->{"radius-server-list-entries"}
              ->{"radius-server-list-entry-info"}[0]->{"radius-server-info"} =
              $info;
            $output_hash->{"radius-server-list-entries"}
              ->{"radius-server-list-entry-info"}[0]->{"radius-server-ip"} =
              $ip;
            $output_hash->{"radius-server-list-entries"}
              ->{"radius-server-list-entry-info"}[0]->{"radius-server-port"} =
              $port;

        }    #end if ref eq "HASH"

        # else cli.  Convert the CLI output to hash.
    } else {
        $output_hash = $self->_radius_show_cli_to_hash(-cli_output => $output);

        # Verify we got something back.
        if (!$output_hash) {
            $self->_set_err("radius_show_cli_to_hash failed",
                            $self->errmsg(), $resultsHash_ref->{output});
            return $ClientUtil::FALSE;
        }

    }    # end else cli

    # If code gets here, the command was good.
    return $output_hash;

}    # end radius_show_info

################################################################################
# Method:    _radius_show_cli_to_hash
# Objective: Convert radius show CLI output into hash format.
# Details:
#     Inputs  -cli_output - required - $output from radius_show library call
#     Returns $radius_hash - hash in the same format as radius show ZAPI cmd
#             $ClientUtil::FALSE - no cli_output passed in or error(and set_err)
################################################################################
sub _radius_show_cli_to_hash {
    my ($self, %args) = @_;

    my $cli_output;    # arg - required - $output from radius_show library call
    my $radius_hash;
    my $avail = "is-available";

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::radius_show_cli_to_hash()",
                  -msg_level => 3);

    # Parse arguments
    foreach (keys %args) {
        if (/-?cli_output/) {
            $cli_output = $args{$_};
            if (not length($cli_output)) {
                $self->_set_err("Invalid value for argument '-cli_output'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $cli_output) {
        $self->_set_err("Missing required argument '-cli_output'");
        return $ClientUtil::FALSE;
    }

    $self->sanlog(
                 -msg => "radius_show_cli_to_hash: cli_output=" . "$cli_output",
                 -msg_level => 7);

    # Expected CLI output to parse:
    # rtpqa-fas6030-13*> radius show
    #RADIUS client service is running
    #
    #Default RADIUS server : IP_Addr=10.60.169.227  UDPPort=1812
    #
    #Backup RADIUS server 0: IP_Addr=10.60.132.94  UDPPort=1812
    #
    #Backup RADIUS server 1: IP_Addr=10.60.169.123  UDPPort=1812

    # Split the CLI output into individual lines
    my @output_array = split(/\n/, $cli_output);

    # Read the first line to determine if radius is started/stopped.
    if ($output_array[0] eq "RADIUS client service is running") {
        $radius_hash->{"is-available"} = "true";
    } elsif ($output_array[0] eq "RADIUS client service is not running") {
        $radius_hash->{"is-available"} = "false";
    } else {

        # Can't determine radius status.
        $self->_set_err(
              "RADIUS client service - pattern does not match expected output: "
                . "$output_array[0]");
        return $ClientUtil::FALSE;
    }

    # Next line should be blank.
    if ($output_array[1] ne "") {
        $self->_set_err("Did not find expected blank line: $output_array[1]");
        return $ClientUtil::FALSE;
    }

    # Use a for loop to take a peek at lines 2, 4, and 6 to gather radius info.
    # Notice we step by 2.
    # $info_array_index is the array index for the hash.
    # $output_array_index is the array index into the cli output.
    my $info_array_index = 0;
    for (my $output_array_index = 2 ;
         $output_array_index < 8 ;
         $output_array_index += 2) {
        $self->sanlog(-msg => "radius_show_cli_to_hash: processing line: "
                        . "$output_array[$output_array_index]",
                      -msg_level => 7);

        # If the line does not exist, no more entries, return.
        if (!$output_array[$output_array_index]) {
            return $radius_hash;
        }

        # Get the Info, ip, and port.  Save into hash.
        if ($output_array[$output_array_index] !~
            /^(.*):\s+IP_Addr=(\d+).(\d+).(\d+).(\d+)\s+UDPPort=\s*(\d+)$/) {
            $self->_set_err(
                       "Unexpected output: $output_array[$output_array_index]");
            return $ClientUtil::FALSE;
        }

        $self->sanlog(-msg => "radius_show_cli_to_hash :parsed line: "
                        . "$1 $2 $3 $4 $5 $6",
                      -msg_level => 7);

        # Creating a hash entry (radius-server-list-entries) which is an array
        # of hash where each hash contains these fields:
        #"radius-server-info", "radius-server-ip", "radius-server-port"
        $radius_hash->{"radius-server-list-entries"}
          ->{"radius-server-list-entry-info"}[$info_array_index]
          ->{"radius-server-info"} = $1;
        $radius_hash->{"radius-server-list-entries"}
          ->{"radius-server-list-entry-info"}[$info_array_index]
          ->{"radius-server-ip"} = "$2.$3.$4.$5";
        $radius_hash->{"radius-server-list-entries"}
          ->{"radius-server-list-entry-info"}[$info_array_index]
          ->{"radius-server-port"} = $6;

        # If this is the Default, remove the trailing space.
        if ($radius_hash->{"radius-server-list-entries"}
            ->{"radius-server-list-entry-info"}[$info_array_index]
            ->{"radius-server-info"} eq "Default RADIUS server ") {
            $radius_hash->{"radius-server-list-entries"}
              ->{"radius-server-list-entry-info"}[$info_array_index]
              ->{"radius-server-info"} =~ s/\s+$//;
        }

        # Verify the next line is blank.
        if ($output_array[$output_array_index + 1] ne "") {
            $self->_set_err(  "Did not find expected blank line: "
                            . "$output_array[$output_array_index+1]");
            return $ClientUtil::FALSE;
        }

        #Increment $info_array_index so the hash array index is correct.
        $info_array_index++;

    }    #end for $output_array_index

    my $dumpOut = Dumper($radius_hash);
    $self->sanlog(-msg       => "radius hash from cli_to_hash is:\n$dumpOut",
                  -msg_level => 7);

    return $radius_hash;
}    # end _radius_show_cli_to_hash

################################################################################
# Method:    radius_add
# Objective: Add a radius server.
# Details:   See POD documentation at the end of this file
# Inputs    -host - required - ip or hostname of server to add
#           -port - optional - port number or 'default'
#           -default_option - optional -  (0 or 1), add the -d flag
# Returns   $ClientUtil::TRUE - successful add
#           $ClientUtil::FALSE - add failed. (and _set_err)
################################################################################
sub radius_add {
    my ($self, %args) = @_;

    my $host;    # arg - required - ip or hostname of server to add
    my $port;    # arg - optional - port number or 'default'
    my $default_option = 0;                  # arg - optional - add the -d flag
    my $cmd_obj        = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::radius_add()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?host/) {
            $host = $args{$_};
            if (not length($host)) {
                $self->_set_err("Invalid value for argument '-host'");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?port/) {
            $port = $args{$_};
            if (not length($port)) {
                $self->_set_err("Invalid value for argument '-port'");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?default_option/) {
            $default_option = $args{$_};
            if (not length($default_option) or $default_option !~ /1|0/) {
                $self->_set_err("Invalid value for argument '-default_option'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $host) {
        $self->_set_err("Missing required argument '-host'");
        return $ClientUtil::FALSE;
    }

    # Build the command line and execute the command
    my %cmd_args = (-cmd              => "radius-server-add",
                    -cmd_filer        => $self->get_name(),
                    -check_output     => 1,
                    -check_result     => 1,
                    -cmd_interface    => $self->interface(),
                    "-radius-ip-addr" => $host);

    # Add in port if needed.
    if (defined($port) && $port !~ /^default$/i) {
        $cmd_args{"-port"} = $port;
    }

    # Set -d flag if needed. -default is cli. -is-default-server is for zapi.
    if ($default_option) {
        $cmd_args{"-is-default-server"} = "1";
        $cmd_args{"-default"}           = "";
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #Check for failure
    if ($cmd_obj->error()) {
        $self->_set_err("radius add failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
    }    # end if zapi

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }
}    # end radius_add

################################################################################
# Method:    radius_remove
# Objective: Remove a radius server.
# Details:   See POD documentation at the end of this file
#     Inputs  -host - required - ip or hostname of server to add
#             -port - optional - port number or 'default'
#     Returns $ClientUtil::TRUE - successful remove
#             $ClientUtil::FALSE - remove failed.(and set_err)
################################################################################
sub radius_remove {
    my ($self, %args) = @_;

    my $host;    # arg - required - ip or hostname of server to remove
    my $port;    # arg - optional - port number or 'default'
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::radius_remove()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?host/) {
            $host = $args{$_};
            if (not length($host)) {
                $self->_set_err("Invalid value for argument '-host'");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?port/) {
            $port = $args{$_};
            if (not length($port)) {
                $self->_set_err("Invalid value for argument '-port'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $host) {
        $self->_set_err("Missing required argument '-host'");
        return $ClientUtil::FALSE;
    }

    # Build the command line and execute the command
    my %cmd_args = (-cmd              => "radius-server-remove",
                    -cmd_filer        => $self->get_name(),
                    -check_output     => 1,
                    -check_result     => 1,
                    -cmd_interface    => $self->interface(),
                    "-radius-ip-addr" => $host);

    # Add in port if needed.
    if (defined($port) && $port !~ /^default$/i) {
        $cmd_args{"-port"} = $port;
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if ($cmd_obj->error()) {
        $self->_set_err("radius remove failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
    }    # end if zapi

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }
}    # end radius_remove

################################################################################
# Method:    radius_remove_all_servers
# Objective: Starts radius service and removes all radius servers.
# Details:   See POD documentation at the end of this file
#     Inputs  -radius_hash - optional (the output of the radius_show library
#               call.) See zapidoc for format.
#     Returns $ClientUtil::TRUE - all servers removed
#             $ClientUtil::FALSE - could not remove all servers
################################################################################
sub radius_remove_all_servers {
    my ($self, %args) = @_;

    my $radius_hash;    # arg - optional - ptr to hash output from radius_show
    my $output;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::radius_remove_all_servers()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?radius_hash/) {
            $radius_hash = $args{$_};
            if (!keys %{$radius_hash}) {
                $self->_set_err("Invalid value for argument '-radius_hash'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # If the user did not pass in radius_hash, we need to generate it.
    if (!$radius_hash) {
        $radius_hash = $self->radius_show();

        # Verify radius show was ok.
        if ($self->error) {
            $self->_set_err("radius show failed: ", $self->errmsg());
            return $ClientUtil::FALSE;
        }
    }

    # If radius is not running, we must start it to remove servers.
    if ($radius_hash->{"is-available"} eq "false") {

        # If radius cannot start, abort.
        if (!$self->radius_start()) {
            $self->_set_err("Could not start radius: ", $self->errmsg());
            return $ClientUtil::FALSE;
        }

        # Verify status reports running.
        if (!$self->radius_status()) {
            $self->_set_err("Could not start radius service: ",
                            $self->errmsg());
            return $ClientUtil::FALSE;
        }

    }    # End If radius is stopped, start it.

    # For each array member, remove the radius server.
    foreach my $hash (@{  $radius_hash->{"radius-server-list-entries"}
                            ->{"radius-server-list-entry-info"}}
      ) {

        # Remove the radius server.  If there is a problem, abort.
        $self->radius_remove(-host => $hash->{"radius-server-ip"},
                             -port => $hash->{"radius-server-port"});
        if ($self->error) {
            $self->_set_err("Could not remove radius server: ",
                            $self->errmsg());
            return $ClientUtil::FALSE;
        }

    } ## end foreach my $hash (@{$radius_hash...

    return $ClientUtil::TRUE;

}    # end radius_remove_all_servers

################################################################################
# Method:    iscsi_iptpgroup_show
# Objective: Displays iSCSI IP-based tpgroup information and returns a hash.
# Details:   See POD documentation at the end of this file
################################################################################
sub iscsi_iptpgroup_show
{
    my ($self, %args) = @_;
    my ($vfiler_name, $iptpgroup_tag);
    my ($output_hash, $resultsHash_ref);
    my %cmd_args;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_iptpgroup_show()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?vfiler_name$/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?iptpgroup_tag$/) {
            $iptpgroup_tag = $args{$_};
            if (not length($iptpgroup_tag)) {
                $self->_set_err("Invalid value for argument '-iptpgroup_tag'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    unless (defined $vfiler_name and length $vfiler_name) {
        $self->_set_err("parameter \$vfiler_name not found");
        return $ClientUtil::FALSE;
    }

    %cmd_args = (-cmd           => "iscsi-iptpgroup-list-info",
                 -cmd_filer     => $self->get_name(),
                 -check_output  => 1,
                 -check_result  => 1,
                 -cmd_interface => $self->interface(),
                 -cmd_vfiler    => $vfiler_name);

    #
    # ZAPI only
    #
    $cmd_args{"-iptpgroup-tag"} = $iptpgroup_tag
      if (($self->interface() eq "zapi") and (defined $iptpgroup_tag));

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if ($cmd_obj->error()) {
        $self->_set_err("iscsi_iptpgroup_show failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # If result_check and output_check are not ok, return error.
    #
    if (not(    $resultsHash_ref->{result_check}
            and $resultsHash_ref->{output_check})
      ) {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    $self->sanlog(-msg       => "iscsi_iptpgroup_show cmd returns:\n $output",
                  -msg_level => 7);

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        $output_hash = $self->xml2hash($output);
        my $dumpOut = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # If there is only 1 element, the xml2hash returns a HASH instead of
        # ARRAY.
        # In this case, change the HASH to an ARRAY.
        # This allows the ZAPI hash to match the CLI hash, and easier processing
        # of the output, since everything will always be an ARRAY.
        if (ref($output_hash->{"iscsi-iptpgroup-list-entries"}
                  ->{"iscsi-iptpgroup-list-entry-info"}) eq "HASH"
          ) {
            if (ref($output_hash->{"iscsi-iptpgroup-list-entries"}
                    ->{"iscsi-iptpgroup-list-entry-info"}
                    ->{"ipaddress-list-entries"}) eq "HASH"
            ) {
                $output_hash->{"iscsi-iptpgroup-list-entries"}
                            ->{"iscsi-iptpgroup-list-entry-info"}
                            ->{"ipaddress-list-entries"} =
                            $self->_convert_hash_to_array_of_hash (
                            $output_hash->{"iscsi-iptpgroup-list-entries"}
                                        ->{"iscsi-iptpgroup-list-entry-info"}
                                        ->{"ipaddress-list-entries"});
            }
            $output_hash->{"iscsi-iptpgroup-list-entries"}
                        ->{"iscsi-iptpgroup-list-entry-info"} =
                            $self->_convert_hash_to_array_of_hash (
                            $output_hash->{"iscsi-iptpgroup-list-entries"}
                                        ->{"iscsi-iptpgroup-list-entry-info"});

        } else {   #end if ref eq "HASH"
            foreach my $hash_entry (
                $output_hash->{"iscsi-iptpgroup-list-entries"}
                            ->{"iscsi-iptpgroup-list-entry-info"}) {
                if (ref($hash_entry->{"ipaddress-list-entries"}) eq "HASH") {
                    $hash_entry->{"ipaddress-list-entries"} =
                                $self->_convert_hash_to_array_of_hash (
                                $hash_entry->{"ipaddress-list-entries"});
                }
            }
        }

        # else cli.  Convert the CLI output to hash.
    } else {
        $output_hash = $self->_iscsi_iptpgroup_show_cli_to_hash($output);

        # Verify the hash
        if (!$output_hash) {
            $self->_set_err("_iscsi_iptpgroup_show_cli_to_hash failed",
                            $self->errmsg(), $resultsHash_ref->{output});
            return $ClientUtil::FALSE;
        }
    }    # end else cli

    return $output_hash;
}    # end iscsi_iptpgroup_show

###############################################################################
# Method:    iscsi_iptpgroup_create
# Objective: Creates an iptpgroup on a vfiler
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_iptpgroup_create
{
    my ($self, %args) = @_;
    my ($iptpgroup_name, $vfiler_name, $ip_addresses, $iptpgroup_tag, $force);
    my $cmd_obj = $self->cmd_obj();
    my ($resultsHash_ref, $output);
    my (%cmd_args);

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_iptpgroup_create()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?iptpgroup_name$/) {
            $iptpgroup_name = $args{$_};
            if (not length($iptpgroup_name)) {
                $self->_set_err("Invalid value for argument '-iptpgroup_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?vfiler_name$/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?ip_addresses$/) {
            $ip_addresses = $args{$_};
            if (not length($ip_addresses)) {
                $self->_set_err("Invalid value for argument '-ip_addresses'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?iptpgroup_tag$/) {
            $iptpgroup_tag = $args{$_};
            if (not length($iptpgroup_tag)) {
                $self->_set_err("Invalid value for argument '-iptpgroup_tag'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?force$/) {
            $force = $args{$_};
            if (not length($force) or $force !~ /1|0/) {
                $self->_set_err("Invalid value for argument '-force'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    unless (defined $iptpgroup_name and length $iptpgroup_name) {
        $self->_set_err("Required parameter \$iptpgroup_name not found");
        return $ClientUtil::FALSE;
    }
    unless (defined $vfiler_name and length $vfiler_name) {
        $self->_set_err("Required parameter \$vfiler_name not found");
        return $ClientUtil::FALSE;
    }

    # Build the command line and execute the command
    %cmd_args = (-cmd              => "iscsi-iptpgroup-create",
                 -cmd_filer        => $self->get_name(),
                 -check_output     => 1,
                 -check_result     => 1,
                 -cmd_interface    => $self->interface(),
                 "-iptpgroup-name" => $iptpgroup_name,
                 -cmd_vfiler       => $vfiler_name,);

    # Add optional args if needed.
    $cmd_args{"-iptpgroup-tag"} = $iptpgroup_tag if defined $iptpgroup_tag;

    # CLI Only
    $cmd_args{"-ip-addresses"} = $ip_addresses if defined $ip_addresses;

    # CLI Only
    $cmd_args{"-force"} = "" if $force;

    # Execute
    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    # Check for failure
    if ($cmd_obj->error()) {
        $self->_set_err("iscsi iptpgroup create failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # Get the output
    $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
        my $status = $output_hash->{status};
        if ($status ne "failed" && defined $ip_addresses) {

            # Add ip_addresses if requested for ZAPI
            my @ip_list = split(/\s+/, $ip_addresses);
            foreach my $ip (@ip_list) {
                $self->iscsi_iptpgroup_ipaddr_add(
                                             -iptpgroup_name => $iptpgroup_name,
                                             -ip_address => $ip,
                                             -vfiler_name => $vfiler_name);
                if ($self->error()) {
                    $self->_set_err(  "iscsi_iptpgroup_ipaddr_add failed: "
                                    , $self->errmsg());
                    return $ClientUtil::FALSE;
                }
            }
        } ## end if ($status ne "failed"...
    }    # end if zapi

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }
    return $ClientUtil::TRUE;
}    # end sub iscsi_tpgroup_create

###############################################################################
# Method:    iscsi_iptpgroup_destroy
# Objective: Destroys an iptpgroup on a vfiler
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_iptpgroup_destroy
{
    my ($self, %args) = @_;
    my ($iptpgroup_name, $vfiler_name, $iptpgroup_tag, $force);
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_iptpgroup_destroy()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?iptpgroup_name?/) {
            $iptpgroup_name = $args{$_};
            if (not length($iptpgroup_name)) {
                $self->_set_err("Invalid value for argument '-iptpgroup_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?vfiler_name?/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?iptpgroup_tag?/) {
            $iptpgroup_tag = $args{$_};
            if (not length($iptpgroup_tag)) {
                $self->_set_err("Invalid value for argument '-iptpgroup_tag'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?force?/) {
            $force = $args{$_};
            if (not length($force) or $force !~ /1|0/) {
                $self->_set_err("Invalid value for argument '-force'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    unless (defined $vfiler_name and length $vfiler_name) {
        $self->_set_err("Required parameter \$vfiler_name not found");
        return $ClientUtil::FALSE;
    }

    # Build the command line and execute the command
    my %cmd_args = (-cmd           => "iscsi-iptpgroup-destroy",
                    -cmd_filer     => $self->get_name(),
                    -check_output  => 1,
                    -check_result  => 1,
                    -cmd_interface => $self->interface(),
                    -cmd_vfiler    => $vfiler_name,);

    if ($self->interface() eq "cli") {    # CLI

        unless (defined $iptpgroup_name and length $iptpgroup_name) {
            $self->_set_err("Required parameter \$iptpgroup_name not found");
            return $ClientUtil::FALSE;
        }

        # Add CLI args
        $cmd_args{"-iptpgroup-name"} = $iptpgroup_name;
        $cmd_args{"-force"} = "" if $force;
    } else {                              # ZAPI
        if (defined $iptpgroup_name and length $iptpgroup_name) {
            $iptpgroup_tag =
              $self->_get_iscsi_iptpgroup_tag(
                                             -iptpgroup_name => $iptpgroup_name,
                                             -vfiler_name    => $vfiler_name);
            if (!$iptpgroup_tag) {
                $self->_set_err(
                                "Unable to get tag of iptpgroup $iptpgroup_name"
                              , $self->errmsg());
                return $ClientUtil::FALSE;
            }
        } elsif (defined $iptpgroup_tag and length $iptpgroup_tag) {

            #
            # iptpgroup-tag will only be used if iptpgroup-name is not
            # provided
            #
        } else {
            $self->_set_err(  "Required parameter \$iptpgroup_name or "
                            . "\$iptpgroup_tag not found");
            return $ClientUtil::FALSE;
        }
        $cmd_args{"-iptpgroup-tag"} = $iptpgroup_tag;
    } ## end else [ if ($self->interface()...

    # Execute
    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    # Check for failure
    if ($cmd_obj->error()) {
        $self->_set_err("iscsi iptpgroup destroy failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # Get the output
    my $output = $resultsHash_ref->{output};

    # If the interface is zapi we have to convert the output from xml
    # to a string.
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
    }    # end if zapi

    # Since we checked the result, lets see if our check was OK
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }
    return $ClientUtil::TRUE;
}    # end sub iscsi_iptpgroup_destroy

###############################################################################
# Method:    iscsi_iptpgroup_ipaddr_add
# Objective: Adds an IP address to an iptpgroup in a vfiler
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_iptpgroup_ipaddr_add
{
    my ($self, %args) = @_;
    my ($iptpgroup_name, $vfiler_name, $iptpgroup_tag, $ip_address, $force);
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_iptpgroup_ipaddr_add()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?iptpgroup_name?/) {
            $iptpgroup_name = $args{$_};
            if (not length($iptpgroup_name)) {
                $self->_set_err("Invalid value for argument '-iptpgroup_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?vfiler_name?/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?ip_address?/) {
            $ip_address = $args{$_};
            if (not length($ip_address)) {
                $self->_set_err("Invalid value for argument '-ip_address'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?iptpgroup_tag?/) {
            $iptpgroup_tag = $args{$_};
            if (not length($iptpgroup_tag)) {
                $self->_set_err("Invalid value for argument '-iptpgroup_tag'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?force?/) {
            $force = $args{$_};
            if (not length($force) or $force !~ /1|0/) {
                $self->_set_err("Invalid value for argument '-force'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    unless (defined $vfiler_name and length $vfiler_name) {
        $self->_set_err("Required parameter \$vfiler_name not found");
        return $ClientUtil::FALSE;
    }

    # Build the command line and execute the command
    my %cmd_args = (-cmd           => "iscsi-iptpgroup-ipaddr-add",
                    -cmd_filer     => $self->get_name(),
                    -check_output  => 1,
                    -check_result  => 1,
                    -cmd_interface => $self->interface(),
                    -cmd_vfiler    => $vfiler_name,
                    "-ip-addr"     => $ip_address,);

    if ($self->interface() eq "cli") {    # CLI
        unless (defined $iptpgroup_name and length $iptpgroup_name) {
            $self->_set_err("Required parameter \$iptpgroup_name not found");
            return $ClientUtil::FALSE;
        }

        # Add CLI args
        $cmd_args{"-iptpgroup-name"} = $iptpgroup_name;
        $cmd_args{"-force"} = "" if $force;
    } else {                              # ZAPI
        if (defined $iptpgroup_name and length $iptpgroup_name) {
            $iptpgroup_tag =
              $self->_get_iscsi_iptpgroup_tag(-vfiler_name    => $vfiler_name,
                                              -iptpgroup_name => $iptpgroup_name
                                              );
            if ($iptpgroup_tag eq "") {
                $self->_set_err(
                                "Unable to get tag of iptpgroup $iptpgroup_name"
                              , $self->errmsg());
                return $ClientUtil::FALSE;
            }
        } elsif (defined $iptpgroup_tag and length $iptpgroup_tag) {

            #
            # iptpgroup-tag will only be used if iptpgroup-name is not
            # provided
            #
        } else {
            $self->_set_err(  "Required parameter \$iptpgroup_name or "
                            . "\$iptpgroup_tag not found");
            return $ClientUtil::FALSE;
        }
        $cmd_args{"-iptpgroup-tag"} = $iptpgroup_tag;
    } ## end else [ if ($self->interface()...

    # Execute
    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    # Check for failure
    if ($cmd_obj->error()) {
        $self->_set_err("iscsi iptpgroup ipaddr_add failed: ",
                        $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # Get the output
    my $output = $resultsHash_ref->{output};

    # If the interface is zapi we have to convert the output from xml
    # to a string.
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
    }    # end if zapi

    # Since we checked the result, lets see if our check was OK
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }
    return $ClientUtil::TRUE;
}    # end sub iscsi_iptpgroup_ipaddr_add

###############################################################################
# Method:    iscsi_iptpgroup_ipaddr_delete
# Objective: Removes an IP address from an iptpgroup in a vfiler
# Details:   See POD documentation at the end of this file.
###############################################################################
sub iscsi_iptpgroup_ipaddr_delete
{
    my ($self, %args) = @_;
    my ($iptpgroup_name, $vfiler_name, $iptpgroup_tag, $ip_address, $force);
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_iptpgroup_ipaddr_delete()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?iptpgroup_name?/) {
            $iptpgroup_name = $args{$_};
            if (not length($iptpgroup_name)) {
                $self->_set_err("Invalid value for argument '-iptpgroup_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?vfiler_name?/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?ip_address?/) {
            $ip_address = $args{$_};
            if (not length($ip_address)) {
                $self->_set_err("Invalid value for argument '-ip_address'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?iptpgroup_tag?/) {
            $iptpgroup_tag = $args{$_};
            if (not length($iptpgroup_tag)) {
                $self->_set_err("Invalid value for argument '-iptpgroup_tag'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?force?/) {
            $force = $args{$_};
            if (not length($force) or $force !~ /1|0/) {
                $self->_set_err("Invalid value for argument '-force'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    unless (defined $vfiler_name and length $vfiler_name) {
        $self->_set_err("Required parameter \$vfiler_name not found");
        return $ClientUtil::FALSE;
    }

    # Build the command line and execute the command
    my %cmd_args = (-cmd           => "iscsi-iptpgroup-ipaddr-delete",
                    -cmd_filer     => $self->get_name(),
                    -check_output  => 1,
                    -check_result  => 1,
                    -cmd_interface => $self->interface(),
                    -cmd_vfiler    => $vfiler_name,
                    "-ip-addr"     => $ip_address,);
    if ($self->interface() eq "cli") {    # CLI
        unless (defined $iptpgroup_name and length $iptpgroup_name) {
            $self->_set_err("Required parameter \$iptpgroup_name not found");
            return $ClientUtil::FALSE;
        }

        # Add CLI args
        $cmd_args{"-iptpgroup-name"} = $iptpgroup_name;
        $cmd_args{"-force"} = "" if $force;
    } else {                              # ZAPI
        if (defined $iptpgroup_name and length $iptpgroup_name) {
            $iptpgroup_tag =
              $self->_get_iscsi_iptpgroup_tag(-vfiler_name    => $vfiler_name,
                                              -iptpgroup_name => $iptpgroup_name
                                              );
            if (!$iptpgroup_tag) {
                $self->_set_err(
                         "Unable to find iptpgroup tag for " . $iptpgroup_name);
                return $ClientUtil::FALSE;
            }
        } elsif (defined $iptpgroup_tag and length $iptpgroup_tag) {

            #
            # iptpgroup-tag will only be used if iptpgroup-name is not
            # provided
            #
        } else {
            $self->_set_err(  "Required parameter \$iptpgroup_name or "
                            . "\$iptpgroup_tag not found");
            return $ClientUtil::FALSE;
        }
        $cmd_args{"-iptpgroup-tag"} = $iptpgroup_tag;
    } ## end else [ if ($self->interface()...

    # Execute
    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    # Check for failure
    if ($cmd_obj->error()) {
        $self->_set_err("iscsi iptpgroup ipaddr_delete failed: ",
                        $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # Get the output
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
    }    # end if zapi

    # Since we checked the result, lets see if our check was OK
    if ($resultsHash_ref->{result_check} && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }
    return $ClientUtil::TRUE;
}    # end sub iscsi_iptpgroup_ipaddr_delete


###############################################################################
# Method:    console_connect
# Objective: Create console connection to CLI, log in using specified
#            user, password and wait for initial prompt.
# Inputs:
#            $host        Hostname or IP Address of NZSQL client.
#            $timeout     Connect timeout     (optional)
# Outputs:   Returns cons object (console object) if successful,
#            0 if not (error flag and message will be set).
# Affected Data: If successful, object contains reference to new console
#                object.
###############################################################################
sub console_connect
{
    my ($self, %args) = @_;

    my(
       $connect,
       $errmsg,
       $filer,
       $timeout,
       $filername,
       $conn_str,
       $use_rlm_console,
       $expected_prompt,
       $received_prompt,
      );

    $conn_str = "";
    $timeout = "";
    $use_rlm_console = "";

    foreach (keys %args) {
        if (/-?filername/) {
            length($filername = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"connection\"",
                            "$args{$_}");
                    return {};
                };
        } elsif (/-?timeout/) {
            length($timeout = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"timeout\"",
                            "$args{$_}");
                    return {};
                };
        } elsif (/-?conn_string/) {
            length($conn_str = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"conn_string\"",
                            "$args{$_}");
                    return {};
                };
        } elsif (/-?use_rlm_console/) {
            length($use_rlm_console = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"use_rlm_console\"",
                            "$args{$_}");
                    return {};
                };
        }

    }

    # If we are already connected no pointed in reconnecting
    if (defined $self->{console_conn}) {
        return $self->{console_conn};
    }

    $filer = &Tharn::host($filername);

    $self->{name} = $filer->{hostname};

    # We found the the use_rlm_console was set so use
    # RLM as the console
    if (($use_rlm_console ne "") && ($use_rlm_console == $ClientUtil::TRUE)) {
        $conn_str = "#rlm;conntype=ssh-persist";
    } elsif ($conn_str eq "") {
        # default to regular console
        $conn_str = "#console";
    }

    # No timeout specified defaulting to 60 seconds
    if ($timeout eq "") {
        $timeout = 60;
    }

    $connect = &Tharn::connect($conn_str.";timeout=$timeout;",
                              -eval=>
                                  sub {
                                      if (@_) {
                                          my $errmsg = join("",@_);
                                          $errmsg =~ s/\n//g;
                                          $self->_set_err($errmsg);
                                          $Tharn::InEND=1; #Don't abort
                                          &Tharn::logresult("CONFIG",
                                                            abort=>0,"\n@_");
                                          return {};
                                      }
                                  }
                              );


    # Handle a login prompt if we get one.
    # This will happen if a password is set for root
    # Combine all the known expected prompt in one str
    $expected_prompt = "login:|Password:";
    if (exists ($filer->{prompt_fw})) {
        $expected_prompt .= "|";
        $expected_prompt .= join("|", @{$filer->{prompt_fw}});
    }
    if (exists ($filer->{prompt_virt}) and $filer->{prompt_virt} ne "") {
        $expected_prompt .= "|";
        $expected_prompt .= join("|", @{$filer->{prompt_virt}});
    }
    if (exists ($filer->{prompt_take})) {
        $expected_prompt .= "|";
        $expected_prompt .= join("|", @{$filer->{prompt_take}});
    }
    if (exists ($filer->{prompt_norm})) {
        $expected_prompt .= "|";
        $expected_prompt .= join("|", @{$filer->{prompt_norm}});
    } else {
        $expected_prompt .= "|" . $filername . '(::[^>#]*)?\*?[>#]';
    }

    # if on RLM, we need to pass in system console to enter the console
    if ($conn_str =~ /\#rlm/ or $use_rlm_console == $ClientUtil::TRUE) {
        (undef, undef, undef, $received_prompt) =
            &Tharn::say($connect, "system console",
                        '-swallow',
                        '-noprompt', '-prompt', $expected_prompt,
                        '-timeout', 30,
                        '-ignore', "echo ignore",
                        '-comment', "Attempted system console via RLM");
        if ($received_prompt !~ /$expected_prompt/) {
            # try hitting enter again a few more times until we get the prompt
            for (1..3) {
                (undef, undef, undef, $received_prompt) =
                    &Tharn::say($connect, "\n",
                                '-swallow',
                                '-noprompt', '-prompt', $expected_prompt,
                                '-timeout', $timeout,
                                '-ignore', "echo ignore",
                                '-comment', "Checking active prompt");
                if ($received_prompt =~ /$expected_prompt/) {
                    last;
                }
            }
        }
    } else {
        # otherwise, tickle the prompt
        for (1..2) {
            (undef, undef, undef, $received_prompt) =
                &Tharn::say($connect, "\n",
                            '-swallow',
                            '-noprompt', '-prompt', $expected_prompt,
                            '-timeout', $timeout,
                            '-ignore', "echo ignore",
                            '-comment', "Checking active prompt");
        }
    }

    # Make sure the console is on one of the expected prompts
    if ($received_prompt !~ /$expected_prompt/) {
        $self->_set_err("The received prompt ($received_prompt) wasn't " .
                        "one of the expected prompts ($expected_prompt).");
        return {};
    }

    # Actual handling of the login prompt.
    if ($received_prompt =~ /^login:/){

        my $user = "root";

        if (defined $filer->{username}) {
            $user = $filer->{username};
        }

        # We need to do a blind say on the console to send
        # username and password without entering and waiting
        # for prompts. We don't wait for a password just enter
        # it if we get a login. The sleep in the middle
        # because sometimes it can be slow.
        (undef, undef, undef, $received_prompt) =
            &Tharn::say($connect, "$user",
                        '-swallow',
                        '-noprompt', '-prompt', qr/Password:/,
                        '-timeout', $timeout,
                        '-ignore', "echo ignore",
                        '-comment', "Attempted $user");
        if ($received_prompt =~ /Password:/) {
            &SanUtil::nate_sleep(2);
            (undef, undef, undef, $received_prompt) =
                &Tharn::say($connect, $filer->{default_password} . "\n",
                            '-blind',
                            '-timeout', $timeout,
                            '-ignore', "echo ignore",
                            '-comment', "Attempted " . $filer->{default_password});
            # Sometimes we have to wake up the console after we pass in the password
            for (1..2) {
                &Tharn::say($connect, "\n", '-blind');
            }
        }

    } elsif ($received_prompt =~ /Password:/) {
        # Sometimes (if there is only one account) we might get just the password prompt.
        &SanUtil::nate_sleep(2);
            (undef, undef, undef, $received_prompt) =
                &Tharn::say($connect, $filer->{default_password} . "\n",
                            '-blind',
                            '-timeout', $timeout,
                            '-ignore', "echo ignore",
                            '-comment', "Attempted " . $filer->{default_password});
        for (1..2) {
            &Tharn::say($connect, "\n", '-blind');
        }
    }

    # Just in case the prompt has not been achieved, make sure we get it here
    (undef, undef, undef, $received_prompt) =
        &Tharn::say($connect, "\n",
                    '-swallow',
                    '-timeout', $timeout,
                    '-ignore', "echo ignore",
                    '-comment', "Attempted to get shell prompt");

    # Make sure the console is on one of the expected prompts
    if ($received_prompt !~ /$expected_prompt/) {
        $self->_set_err("The received prompt ($received_prompt) wasn't " .
                        "one of the expected prompts ($expected_prompt).");
        return {};
    }

    # Just save in the consle object by default
    $self->{console_conn} = $connect;

    return $connect;
} # end console_connect()

###############################################################################
# Method:    console_disconnect
# Objective: Disconnect the console connection to CLI.
###############################################################################
sub console_disconnect
{
    my ($self, %args) = @_;
    my $connection = "";

    foreach (keys %args) {
        if (/-?connection/) {
            length($connection = $args{$_}) or
                do {
                    $self->_set_err("Invalid value for \"connection\"",
                            "$args{$_}");
                    return $ClientUtil::FALSE;
                };
        } else {
            $self->_set_err("Invalid parameter specified: $_");
            return $ClientUtil::FALSE;
        }
    }

    # Go through and make sure we have a conn to disconnect
    if ($connection eq "" && !(defined ($self->{console_conn}))) {
        $self->_set_err("Did not find a connection to disconnect");
        return $ClientUtil::FALSE;
    }

    # Use the connection if it is specified and also remove the
    # $self->{console_conn}
    if ($connection ne "") {
        &Tharn::disconnect($connection, '-ok');
        undef $self->{console_conn} if (defined ($self->{console_conn}));
    } elsif (defined($self->{console_conn})) {
        &Tharn::disconnect($self->{console_conn}, '-ok');
        undef $self->{console_conn};
    }

    return $ClientUtil::TRUE;
} # end console_disconnect()

################################################################################
# Method:    options_set
# Objective: Set an option.
# Details:   See POD documentation at the end of this file
#     Inputs  -option - required - string of the option to set
#             -value  - required - value to set
#             -vfiler - optional - vfiler context in which option is set
#     Returns $ClientUtil::TRUE - option set to value
#             $ClientUtil::FALSE - set failed. (and _set_err)
################################################################################
sub options_set {
    my ($self, %args) = @_;
    my $option;    # arg - required - string of the option to set
    my $value;     # arg - required - value of the option to set
    my $vfiler = undef; # arg - optional - vfiler context in which option is set
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my %cmd_args;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::options_set()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?option/) {
            $option = $args{$_};
            if (not length($option)) {
                $self->_set_err("Invalid value for argument '-option'");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?value/) {
            $value = $args{$_};
            if (not length($value)) {
                $self->_set_err("Invalid value for argument '-value'");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?vfiler/) {
            #
            # We won't check length of $vfiler
            # since we also use zero length as default_vfiler
            #
            $vfiler = $args{$_};
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $option) {
        $self->_set_err("Missing required argument '-option'");
        return $ClientUtil::FALSE;
    }
    unless (defined $value) {
        $self->_set_err("Missing required argument '-value'");
        return $ClientUtil::FALSE;
    }

    %cmd_args = (-cmd           => "options-set",
                 -cmd_filer     => $self->{filer},
                 -check_output  => 1,
                 -check_result  => 1,
                 -cmd_interface => $self->interface(),
                 "-name"        => $option,
                 "-value"       => $value);

    $cmd_args{"-cmd_vfiler"} = $vfiler if defined $vfiler;

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if ($cmd_obj->error()) {
        $self->_set_err("execute_cmd for options_set failed: ",
                        $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
    }    # end if zapi

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    }
    ## Adding one elsif part to enable the licensed_feature.fcp.enable
    ## In v2 license, it is mandatory to add the license and enable licensed_feature option.
    unless($resultsHash_ref->{output_check}) {
        # output_check fails for the following as output is more verbose
        if ($option =~ /licensed_feature\.multistore\.enable/) {
            if ($resultsHash_ref->{output} =~ /Enabling vfiler creation/) {
                return $ClientUtil::TRUE;
            }
        } elsif ($option =~ /licensed_feature\.iscsi\.enable/) {
            if ($resultsHash_ref->{output} =~
                /Run \'iscsi start\' to start the iSCSI service/) {
                return $ClientUtil::TRUE;
            }
        } elsif ($option =~ /licensed_feature\.fcp\.enable/) {
            if ($resultsHash_ref->{output} =~
                /Run \'fcp start\' to start the FCP service/) {
                return $ClientUtil::TRUE;
            }
        } else {
        $self->_set_err("output check failed",
                        $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
        }
    }
    unless($resultsHash_ref->{result_check}) {
        $self->_set_err("result check failed",
                        $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }
}    # end options_set

################################################################################
# Method:    options_get
# Objective: Get the value of an option.
# Details:   See POD documentation at the end of this file
#     Inputs  -option - string of the option to get
#             -vfiler - optional - vfiler context of the option to get
#     Returns Success:
#                 $string - current value of the option
#                 undef   - option lookup was ok, but value is not defined
#             Failure:
#                 undef   - option lookup failed or bad args (and _set_err)
################################################################################
sub options_get {
    my ($self, %args) = @_;
    my $option;    # arg - required - string of the option to get
    my $vfiler = undef; # arg - optional - vfiler context of the option to get
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my %cmd_args;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::options_get()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?option/) {
            $option = $args{$_};
            if (not length($option)) {
                $self->_set_err("Invalid value for argument '-option'");
                return undef;
            }
            next;
        } elsif (/-?vfiler/) {
            #
            # We won't check length of $vfiler
            # since we also use zero length as default_vfiler
            #
            $vfiler = $args{$_};
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return undef;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $option) {
        $self->_set_err("Missing required argument '-option'");
        return undef;
    }
    %cmd_args = (-cmd           => "options-get",
                -cmd_filer     => $self->{filer},
                -check_output  => 1,
                -check_result  => 1,
                -cmd_interface => $self->interface(),
                "-name"        => $option);

    $cmd_args{"-cmd_vfiler"} = $vfiler if defined $vfiler;

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if ($cmd_obj->error()) {
        $self->_set_err("execute_cmd for options_get failed: ",
                        $cmd_obj->errmsg());
        return undef;
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (!($resultsHash_ref->{result_check} && $resultsHash_ref->{output_check}))
    {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return undef;
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # If we get here, the zapi appears to be good.  Set output
        # to the value.
        $output = $output_hash->{value};

    } else {    # else CLI.  Grab the option value (\S+). Using \S+ to capture
                # all non-whitespace characters.  Do this because some options
                # print the value, then some whitespace, then more text.
                # Example:
                # autosupport.retry.count      15         (value might be overwritten in takeover)

        if ($output =~ /$option\s+(\S+)/) {
            $output = $1;

            # Handle special case.  Some options have no value, and are followed
            # by a message like  "(value might be overwritten in takeover)".
            # If the 1st character is '(', assume that the option is undefined.
            # Example:
            # autosupport.noteto                      (value might be overwritten in takeover)
            if ($output =~ /^\(/) {
                $output = undef;
            }

        } else {

            # Note that if the (\S+) was not found, then the option was not
            # defined.
            # Example: wafl.default_nt_user
            $output = undef;
        }

    } ## end else [ if ($self->interface()...

    # If code gets here, the command was good.
    return $output;

}    # end options_set

################################################################################
# Method:    priv_set
# Objective: Set the privilege level.
# Details:   See POD documentation at the end of this file
#     Inputs  -level - required - priv level to set (admin, advanced, diag, or
#                                 test)
#             -quiet - optional - adds -q to suppress output.  0 is no -q.
#                                 1 is add -q.  Default is 1.
#     Returns $ClientUtil::TRUE - priv level set ok
#             $ClientUtil::FALSE - priv set failed. (and _set_err)
#     NOTE: Always runs using CLI, not valid for zapi.
################################################################################
sub priv_set {
    my ($self, %args) = @_;
    my $level;    # arg - required - string of the level to set
    my $quiet   = 1;                # arg - optional - add -q to suppress output
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::priv_set()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?level/) {
            $level = $args{$_};
            if (not length($level) or $level !~ /admin|advanced|diag|test/) {
                $self->_set_err("Invalid value for argument '-level'");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?quiet/) {
            $quiet = $args{$_};
            if (not length($quiet) or $quiet !~ /0|1/) {
                $self->_set_err("Invalid value for argument '-quiet'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $level) {
        $self->_set_err("Missing required argument '-level'");
        return $ClientUtil::FALSE;
    }

    # Build the command line and execute the command
    my %cmd_args = (-cmd           => "priv-set",
                    -cmd_filer     => $self->get_name(),
                    -check_output  => 1,
                    -check_result  => 1,
                    -cmd_interface => "cli",
                    "-level"       => $level);

    # Add in quiet if needed.
    if ($quiet) {
        $cmd_args{"-quiet"} = "";
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if ($cmd_obj->error()) {
        $self->_set_err("execute_cmd for priv_set failed: ",
                        $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }
}    # end priv_set

################################################################################
# Method:    set_iscsi_nodename
# Objective: Sets the iscsi nodename.
# Details:   See POD documentation at the end of this file
# Inputs    -nodename - required - nodename to set
# Returns   $ClientUtil::TRUE - successful set
#           $ClientUtil::FALSE - set failed. (and _set_err)
################################################################################
sub set_iscsi_nodename
{
    my ($self, %args) = @_;

    my $nodename;    # arg - required - nodename to set
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::set_iscsi_nodename()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?nodename/) {
            $nodename = $args{$_};
            if (not length($nodename)) {
                $self->_set_err("Invalid value for argument '-nodename'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $nodename) {
        $self->_set_err("Missing required argument '-nodename'");
        return $ClientUtil::FALSE;
    }

    # Build the command line and execute the command
    my %cmd_args = (-cmd           => "iscsi-node-set-name",
                    -cmd_filer     => $self->get_name(),
                    -check_output  => 1,
                    -check_result  => 1,
                    -cmd_interface => $self->interface(),
                    "-node-name"   => $nodename);

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    # Check for failure
    if ($cmd_obj->error()) {
        $self->_set_err("iscsi nodename failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
    }    # end if zapi

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }
}    # end set_iscsi_nodename

################################################################################
# Method:    iscsi_security_set
# Objective: Configure iSCSI security default or specify 1 initiator.
# Details:   See POD documentation at the end of this file
# Inputs    Note: You MUST specify either -default or -initiator.
#           -default - optional - 1 = set default security for all initiators
#                                 0 = do not set default security
#           -initiator   - optional - initiator iqn to set security policy
#           -auth-type   - required - chap,deny,none,radius
#           -user_name   - (required for CHAP only) initiator's user name
#           -password    - (required for CHAP only) initiator's password
#           -out_user_name  - optional - outbound username for 2 way CHAP
#           -out_password   - optional - outbound password for 2 way CHAP
#
#           WARNING!!!!!!!
#           If you wish to set passwords using ZAPIs, PLEASE READ:
#           You CANNOT use plain ASCII text to transmit a password (inbound or
#           outbound) via ZAPI.  You must first translate your ASCII password
#           into an encrypted format.  To do this:
#           Telnet to a linux system. (telnet osiris)
#           cd /u/hual/test
#           ./pwd_gen
#           osiris test > ./pwd_gen -p netapp
#           ad55f657dd12
#           This is the password that you can send via ZAPI.  Your login
#           password from the initiator is 'netapp'.
#
#           Common passwords and translations:
#           netapp        ad55f657dd12
#           netappnetapp  ad55f657dd12684482254d1d
#           netapp1       ad55f657dd1237
#           password      b351f145da0d7445
#
# Returns   $ClientUtil::TRUE - successful
#           $ClientUtil::FALSE - failed. (and _set_err)
################################################################################
sub iscsi_security_set {
    my ($self, %args) = @_;

    # Args.  See header for descriptions.
    my ($default, $initiator, $auth_type, $user_name, $password, $out_user_name,
        $out_password);

    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_security_set()",
                  -msg_level => 3);

    # Parse arguments
    foreach (keys %args) {
        if (/-?initiator/) {
            $initiator = $args{$_};
            if (not length($initiator)) {
                $self->_set_err("Invalid value for argument '-initiator'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?auth_type/) {
            $auth_type = $args{$_};
            if (not length($auth_type)
                or $auth_type !~ /^chap$|^deny$|^none$|^radius$/i) {
                $self->_set_err("Invalid value for argument '-auth_type'");
                return $ClientUtil::FALSE;
            }
        } elsif (/^-?user_name/) {
            $user_name = $args{$_};
            if (not length($user_name)) {
                $self->_set_err("Invalid value for argument '-user_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/^-?password/) {
            $password = $args{$_};
            if (not length($password)) {
                $self->_set_err("Invalid value for argument '-password'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?out_user_name/) {
            $out_user_name = $args{$_};
            if (not length($out_user_name)) {
                $self->_set_err("Invalid value for argument '-out_user_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?out_password/) {
            $out_password = $args{$_};
            if (not length($out_password)) {
                $self->_set_err("Invalid value for argument '-out_password'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?default/) {
            $default = $args{$_};
            if (not length($default) or $default !~ /^1$|^0$/) {
                $self->_set_err("Invalid value for argument '-default'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $auth_type) {
        $self->_set_err("Missing required argument '-auth_type'");
        return $ClientUtil::FALSE;
    }

    # If ZAPI, check to make sure passwords are hex.  If passwords are not hex,
    # return error and warn user.
    if (($self->interface() eq "zapi")
        && (   $password =~ /[^0-9a-fA-F]/
            || $out_password =~ /[^0-9a-fA-F]/)
      ) {
        $self->_set_err( "ERROR: If you wish to set passwords using\n"
                       . "ZAPIs you CANNOT use plain ASCII text to transmit a "
                       . "password (inbound or outbound) via ZAPI. If you are\n"
                       . "just setting the password, and do not intend to "
                       . "login from an initiator, then just send some hex\n"
                       . "characters as a password. Example: ad55f657dd12 "
                       . "If you intend to set a password and login using an\n"
                       . "initiator, please read the POD documentation for "
                       . "iscsi_security_set and use the pwd_gen utility.\n"
                       . "Alternatively, you could just set the password "
                       . "with the CLI command instead of ZAPI.");
        return $ClientUtil::FALSE;
    } ## end if (($self->interface(...

    # If user did not define default=1 or an initiator, error.
    # Also if user defined both default=1 and an initiator, error.
    # We can only take 1 of these args.
    if ((!$default && !$initiator) || ($default && $initiator)) {
        $self->_set_err("Must specify either '-default=>1' OR '-initiator'");
        return $ClientUtil::FALSE;
    }

    # Build the command line and execute the command
    my %cmd_args = (-cmd_filer     => $self->get_name(),
                    -check_output  => 1,
                    -check_result  => 1,
                    -cmd_interface => $self->interface(),
                    "-auth-type"   => $auth_type);

    # Determine which cmd to use, either default or initiator.
    if ($default) {
        $cmd_args{"-cmd"} = "iscsi-initiator-set-default-auth";
    } else {
        $cmd_args{"-cmd"}       = "iscsi-initiator-add-auth";
        $cmd_args{"-initiator"} = $initiator;
    }

    # If the auth_type is radius, change auth-type to CHAP and add the radius
    # flag.  For CLI, pass in "", for ZAPI pass in "true".
    if ($auth_type =~ /^radius$/i) {
        $cmd_args{"-auth-type"} = "chap";
        if ($self->interface() eq "cli") {
            $cmd_args{"-radius"} = "";
        } else {
            $cmd_args{"-radius"} = "true";
        }
    }

    # If the user specified user_name, add it.
    if ($user_name) {
        $cmd_args{"-user-name"} = $user_name;
    }

    # If the user specified password, add it.
    if ($password) {
        $cmd_args{"-password"} = $password;
    }

    # If the user specified out_user_name, add it.
    if ($out_user_name) {
        $cmd_args{"-outbound-user-name"} = $out_user_name;
    }

    # If the user specified out_password, add it.
    if ($out_password) {
        $cmd_args{"-outbound-password"} = $out_password;
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #Check for failure
    if ($cmd_obj->error()) {
        $self->_set_err($cmd_args{"-cmd"} . " failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
    }    # end if zapi

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }
}    # end iscsi_security_set

################################################################################
# Method:    iscsi_security_delete
# Objective: Remove iSCSI security settings for one or more initiators.
# Details:   See POD documentation at the end of this file
# Inputs    -initiator   - required - initiator iqn to delete security settings
#                        - NOTE: For a single initiator to delete you may use:
#                        -initiator => "iqn.1994-05.com.redhat:1e362f8622ba"
#                        - NOTE: To delete security for multiple initiators,
#                          use an array like this:
#                        -initiator => [ "iqn.1994-05.com.redhat:1e362f8622ba",
#                                        "iqn.1994-05.com.redhat:ffffffffffff",
#                                        "iqn.1994-05.com.redhat:eeeeeeeeeeee" ]
# Returns   $ClientUtil::TRUE - successful
#           $ClientUtil::FALSE - failed. (and _set_err)
################################################################################
sub iscsi_security_delete {
    my ($self, %args) = @_;

    # Args.  See header for descriptions.
    my $initiator;
    my @initiator_array;

    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_security_delete()",
                  -msg_level => 3);

    # Parse arguments
    foreach (keys %args) {
        if (/-?initiator/) {
            $initiator = $args{$_};
            if (not length($initiator)) {
                $self->_set_err("Invalid value for argument '-initiator'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $initiator) {
        $self->_set_err("Missing required argument '-initiator'");
        return $ClientUtil::FALSE;
    }

    # If the user passed in a single iqn (not a reference),
    # push this onto the array.
    # If the user passed in an array of iqns, just save the pointer.
    # If the user passed in something else, return error.
    if (ref $initiator eq "") {
        push(@initiator_array, $initiator);
    } elsif (ref $initiator eq "ARRAY") {
        @initiator_array = @{$initiator};
    } else {
        $self->_set_err("'-initiator' must be a string or an array");
        return $ClientUtil::FALSE;
    }

    # Loop on each iqn in the @initiator array
    foreach my $iqn (@initiator_array) {

        # Build the command line and execute the command
        my %cmd_args = (-cmd           => "iscsi-initiator-delete-auth",
                        -cmd_filer     => $self->get_name(),
                        -check_output  => 1,
                        -check_result  => 1,
                        -cmd_interface => $self->interface(),
                        -initiator     => $iqn);

        $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

        #Check for failure
        if ($cmd_obj->error()) {
            $self->_set_err($cmd_args{"-cmd"} . " failed: ",
                            $cmd_obj->errmsg());
            return $ClientUtil::FALSE;
        }

        $self->log_cmd_results($resultsHash_ref);

        #
        # Get the output
        #
        my $output = $resultsHash_ref->{output};

        #
        # If the interface is zapi we have to convert the output from xml
        # to a string.
        #
        if ($self->interface() eq "zapi") {
            my $output_hash = $self->xml2hash($output);
            my $dumpOut     = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => 3);
        }    # end if zapi

        #
        # Since we checked the result, lets see if our check was OK
        #
        if (!(    $resultsHash_ref->{result_check}
               && $resultsHash_ref->{output_check})
          ) {
            $self->_set_err("results check failed", $resultsHash_ref->{output});
            return $ClientUtil::FALSE;
        }

    } ## end foreach my $iqn (@initiator_array)

    # If we get here, all deletes were successful.
    return $ClientUtil::TRUE;
}    # end iscsi_security_delete

################################################################################
# Method:    iscsi_security_generate
# Objective: Generates a random CHAP secret.
# Details:   See POD documentation at the end of this file
#     Inputs  none
#     Returns $output - random password that was generated
#             undef   - failure and set_err
################################################################################
sub iscsi_security_generate {
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_security_generate()",
                  -msg_level => 3);

    $resultsHash_ref =
      $cmd_obj->execute_cmd(-cmd       => "iscsi-auth-generate-chap-password",
                            -cmd_filer => $self->get_name(),
                            -check_output  => 1,
                            -check_result  => 1,
                            -cmd_interface => $self->interface());

    if ($cmd_obj->error()) {
        $self->_set_err("iscsi security generate failed: ", $cmd_obj->errmsg());
        return undef;
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (!($resultsHash_ref->{result_check} && $resultsHash_ref->{output_check}))
    {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return undef;
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # Parse out the generated password
        if ($output_hash->{"secret"} !~ /^(.*)$/) {
            $self->_set_err("Unable to parse the password from ZAPI output");
            return undef;
        }

        # Save the password into output.
        $output = $1;

    } else {    # else CLI
        if ($output !~ /^Generated Random Secret: (.*)$/) {
            $self->_set_err("Unable to parse the password from CLI output");
            return undef;
        } else {

            # Save the password into output.
            $output = $1;
        }

    }    # end else CLI

    # If code gets here, the command was good.
    return $output;
}    # end iscsi_security_generate

################################################################################
# Method: syslog_msg
# Objective: Send the specified message string to the controller's syslogs
################################################################################
sub syslog_msg {
    my ($self, %args) = @_;

    my $msg = exists($args{-msg}) ? $args{-msg} : undef;
    if (not defined($msg)) {
        $self->_set_err("Missing required parameter -msg");
        return $SanUtil::FALSE;
    }

    # syslog this message using logger
    my $cmd = "logger " . $msg;
    $self->execute($cmd, -hide => 1);
    if ($self->error()) {
        return $SanUtil::FALSE;
    }

    return $SanUtil::TRUE;
}

################################################################################
# Method:    iscsi_isns_start
# Objective: Start isns service.
# Details:   See POD documentation at the end of this file
#     Inputs  -vfiler_name - optional - name of vfiler to execute cmd on
#     Returns string - start ok
#             $ClientUtil::FALSE - start failed. (and _set_err)
################################################################################
sub iscsi_isns_start {
    my ($self, %args) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my $vfiler_name;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_isns_start()",
                  -msg_level => 3);

    # Parse arguments
    foreach (keys %args) {
        if (/-?vfiler_name$/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # Build the command line and execute the command
    my %cmd_args = (-cmd            => "iscsi-isns-start",
                    -cmd_filer      => $self->get_name(),
                    -check_output   => 1,
                    -check_result   => 1,
                    -cmd_interface  => $self->interface());

    # Add in vfiler if needed.
    if ($vfiler_name) {
        $cmd_args{"-cmd_vfiler"} = $vfiler_name;
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if ($cmd_obj->error()) {
        $self->_set_err("iscsi isns start failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # If the service was already started, the zapi will return failed.
        # Intercept this case and return success.
        if ($output_hash->{reason} =~ /^iSNS service already started$/) {
            return $output_hash->{reason};
        }

        # If we get here, the zapi appears to be good.  Set output
        # to the status.
        $output = $output_hash->{status};

    } ## end if ($self->interface()...

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $output;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

}    # end iscsi_isns_start

################################################################################
# Method:    iscsi_isns_stop
# Objective: Stop isns service.
# Details:   See POD documentation at the end of this file
#     Inputs  -vfiler_name - optional - name of vfiler to execute cmd on
#     Returns string - stop ok
#             $ClientUtil::FALSE - start failed. (and _set_err)
################################################################################
sub iscsi_isns_stop {
    my ($self, %args) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my $vfiler_name;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_isns_stop()",
                  -msg_level => 3);

    # Parse arguments
    foreach (keys %args) {
        if (/-?vfiler_name$/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # Build the command line and execute the command
    my %cmd_args = (-cmd            => "iscsi-isns-stop",
                    -cmd_filer      => $self->get_name(),
                    -check_output   => 1,
                    -check_result   => 1,
                    -cmd_interface  => $self->interface());

    # Add in vfiler if needed.
    if ($vfiler_name) {
        $cmd_args{"-cmd_vfiler"} = $vfiler_name;
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if ($cmd_obj->error()) {
        $self->_set_err("iscsi isns stop failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # If the service was already stopped, the zapi will return failed.
        # Intercept this case and return success.
        if ($output_hash->{reason} =~ /^iSNS service already stopped$/) {
            return $output_hash->{reason};
        }

        # If we get here, the zapi appears to be good.  Set output
        # to the status.
        $output = $output_hash->{status};

    } ## end if ($self->interface()...

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $output;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

}    # end iscsi_isns_start

################################################################################
# Method:    iscsi_isns_update
# Objective: Update isns registration.
# Details:   See POD documentation at the end of this file
#     Inputs  -vfiler_name - optional - name of vfiler to execute cmd on
#     Returns $ClientUtil::TRUE  - update ok
#             $ClientUtil::FALSE - update failed. (and _set_err)
################################################################################
sub iscsi_isns_update {
    my ($self, %args) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my $vfiler_name;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_isns_update()",
                  -msg_level => 3);

    # Parse arguments
    foreach (keys %args) {
        if (/-?vfiler_name$/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # Build the command line and execute the command
    my %cmd_args = (-cmd            => "iscsi-isns-update",
                    -cmd_filer      => $self->get_name(),
                    -check_output   => 1,
                    -check_result   => 1,
                    -cmd_interface  => $self->interface());

    # Add in vfiler if needed.
    if ($vfiler_name) {
        $cmd_args{"-cmd_vfiler"} = $vfiler_name;
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if ($cmd_obj->error()) {
        $self->_set_err("iscsi isns update failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a hash.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

}    # end iscsi_isns_update

################################################################################
# Method:    iscsi_isns_config
# Objective: Configure isns server.
# Details:   See POD documentation at the end of this file
#     Inputs  -isns_server - required - ip or hostname of isns server
#             -vfiler_name - optional - name of vfiler to execute cmd on
#     Returns $ClientUtil::TRUE  - config successful
#             $ClientUtil::FALSE - config failed.(and set_err)
################################################################################
sub iscsi_isns_config {
    my ($self, %args) = @_;

    my $isns_server;    # arg - required - ip or hostname of isns server
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my $vfiler_name;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_isns_config()",
                  -msg_level => 3);

    # Parse arguments
    foreach (keys %args) {
        if (/-?isns_server$/) {
            $isns_server = $args{$_};
            if (not length($isns_server)) {
                $self->_set_err("Invalid value for argument '-isns_server'");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?vfiler_name$/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $isns_server) {
        $self->_set_err("Missing required argument '-isns_server'");
        return $ClientUtil::FALSE;
    }

    # Build the command line and execute the command
    my %cmd_args = (-cmd            => "iscsi-isns-config",
                    -cmd_filer      => $self->get_name(),
                    -check_output   => 1,
                    -check_result   => 1,
                    -cmd_interface  => $self->interface(),
                    "-isns-ip-addr" => $isns_server);

    # Add in vfiler if needed.
    if ($vfiler_name) {
        $cmd_args{"-cmd_vfiler"} = $vfiler_name;
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if ($cmd_obj->error()) {
        $self->_set_err("iscsi isns config failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a hash.
    #
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        my $dumpOut     = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);
    }    # end if zapi

    #
    # Since we checked the result, lets see if our check was OK
    #
    if (   $resultsHash_ref->{result_check}
        && $resultsHash_ref->{output_check}) {
        return $ClientUtil::TRUE;
    } else {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }
}    # end iscsi_isns_config

################################################################################
# Method:    iscsi_isns_show
# Objective: Displays isns information and returns a hash.
# Details:   See POD documentation at the end of this file
#     Inputs  -vfiler_name - optional - name of vfiler to execute cmd on
#     Returns $ClientUtil::FALSE - show failed. (and _set_err)
#             Ptr to isns_hash on success
################################################################################
sub iscsi_isns_show {
    my ($self, %args) = @_;
    my $cmd_obj = $self->cmd_obj();
    my $resultsHash_ref;
    my $output_hash;
    my $vfiler_name;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_isns_show()",
                  -msg_level => 3);

    # Parse arguments
    foreach (keys %args) {
        if (/-?vfiler_name$/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # Build the command line and execute the command
    my %cmd_args = (-cmd            => "iscsi-isns-get-info",
                    -cmd_filer      => $self->get_name(),
                    -check_output   => 1,
                    -check_result   => 1,
                    -cmd_interface  => $self->interface());

    # Add in vfiler if needed.
    if ($vfiler_name) {
        $cmd_args{"-cmd_vfiler"} = $vfiler_name;
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if ($cmd_obj->error()) {
        $self->_set_err("iscsi isns show failed: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (!($resultsHash_ref->{result_check} && $resultsHash_ref->{output_check})) {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    $self->sanlog(-msg       => "iscsi isns show cmd returns:\n $output",
                  -msg_level => 7);

    #
    # If the interface is zapi we have to convert the output from xml
    # to a hash.
    #
    if ($self->interface() eq "zapi") {
        $output_hash = $self->xml2hash($output);
        my $dumpOut = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # else cli.  Convert the CLI output to hash.
    } else {
        $output_hash = $self->_isns_show_cli_to_hash(-cli_output => $output);

        # Verify we got something back.
        if (!$output_hash) {
            $self->_set_err("isns_show_cli_to_hash failed",
                            $self->errmsg(), $resultsHash_ref->{output});
            return $ClientUtil::FALSE;
        }

    }    # end else cli

    # If code gets here, the command was good.
    return $output_hash;

}    # end iscsi_isns_show

################################################################################
# Subroutine Name: refresh_targets
# Objective:       Refresh fcp target cache
# Details:         See POD documentation at the end of this file
################################################################################
sub refresh_targets {

    my ($self) = @_;

    # Remove existing target cache
    $self->{targets} = undef;
    $self->{targets} = [];

    $self->sanlog(
                -msg => "Refreshing fcp targets on filer " . $self->get_name());

    my $result = $self->_load_targets();
    return ($result == $ClientUtil::TRUE) ? $SanUtil::TRUE : $SanUtil::FALSE;

}

###############################################################################
# Method:    get_root_vol()
# Objective: Get filer root volume
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_root_vol
{
    my ($self) = @_;

    $self->_clear_err();

    my $cmd_obj = $self->cmd_obj();

    # Get the filer version
    my $version = $cmd_obj->filer_version();
    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to get the filer version: $errmsg");
         return "";
    }

    $self->sanlog(-msg => "Volume::get_root_vol(): Getting root volume", -msg_level => 1);

    my $resultsHash_ref =
            $cmd_obj->execute_cmd(
                                    -cmd                  => "volume-get-root-name",
                                    -cmd_filer            => $self->{filer},
                                    -cmd_interface        => "zapi",
                                    -cmd_filer_version    => $version,
                                    -check_result         => 1,
                                    -check_output         => 1);

    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to get root volume: $errmsg");
        return "";
    }

    $self->log_cmd_results($resultsHash_ref);

    unless ($resultsHash_ref->{result_check} && $resultsHash_ref->{output_check}) {
        $self->_set_err("Zapi volume-get-root-name failed to retrieve root volume.");
        return "";
    }

    # Get the output
    my $output = $resultsHash_ref->{output};

    # Convert the output to xml
    my $output_hash = $self->xml2hash($output);

    # Find and return the root volume
    if (defined($output_hash->{volume}) and $output_hash->{volume} ne "") {
        return $output_hash->{volume};
    } else {
        $self->_set_err("Invalid value returned for root volume");
        return "";
    }
}

################################################################################
# Method: get_aggr_bit_depth
# Objective: Returns the bit depth of the aggregate (32 or 64)
################################################################################
sub get_aggr_bit_depth {
    my ($self, %args) = @_;

    my ($aggr);

    $self->_clear_err();

    # Parse the required/optional arguments.
    foreach (keys %args) {
        if (/-?aggr/) {
            unless (length($aggr = $args{$_}) > 0) {
                $self->_set_err("Error: " . ref($self) . "::_get_aggr_bit_depth() " .
                                "Invalid value for argument '-aggr'");
                return undef;
            }
            next;
        } else {
            $self->_set_err("Error: " . ref($self) . "::_get_aggr_bit_depth() " .
                            "Invalid parameter specified: $_");
            return undef;
        }
    }

    # Make sure we received an aggregate
    unless (defined($aggr) and $aggr ne "") {
        $self->_set_err("Error: " . ref($self) . "::_get_aggr_bit_depth() " .
                        "Missing required parameter '-aggr'");
        return undef;
    }

    # Get the aggr status for $aggr
    my @output = $self->get_aggr_status(-name => $aggr);
    if ($self->error) {
        $self->_set_err("Error: " . ref($self) . "::_get_aggr_bit_depth() " .
                        "Unable to determine the aggr status:\n" . $self->errmsg());
        return undef;
    }

    unless (scalar(@output) > 0) {
        $self->_set_err("Error: " . ref($self) . "::_get_aggr_bit_depth() " .
                        "Unexpected output received from aggr status");
        return undef;
    }

    # Look for 32-bit and 64-bit.  If neither are found then we are on a older ontap
    # machine that defaults to 32-bit.  CLI uses a dash as in 32-bit while zapi uses
    # an underscore as in 32_bit.
    if (grep(/32\-bit|32_bit/, @output) > 0) {
        return "32";
    } elsif (grep(/64\-bit|64_bit/, @output) > 0) {
        return "64";
    } else {
        return "32";
    }
}
################################################################################
# Subroutine Name: aggr_64bit_upgrade
# Objective:       Upgrades the specified(32bit-aggr) to 64bit format
# Details:         See POD documentation at the end of this file
################################################################################
sub aggr_64bit_upgrade {
    my ( $self, %args ) = @_;
    my ( $aggr, $wait_for_completion, $mode, $time_out, $sample_interval )
        = ( undef, 0, "normal", 3600, 30 );
    my $cmd_obj = $self->cmd_obj();
    $self->_clear_err();
    foreach ( keys %args ) {
        if (/-?aggr/) {
            if ( not length( $aggr = $args{$_} ) ) {
                $self->_set_err(
                    "Error: SanFiler7M::aggr_64bit_upgrade(): Invalid value for argument '-aggr'"
                );
                return $ClientUtil::FALSE;
            } ## end if ( not length( $aggr...
            next;
        } elsif (/-?wait_for_completion/) {
            if ( not length( $wait_for_completion = $args{$_} ) ) {
                $self->_set_err(
                    "Error: SanFiler7M::aggr_64bit_upgrade(): Invalid value for argument '-wait_for_completion'"
                );
                return $ClientUtil::FALSE;
            } ## end if ( not length( $wait_for_completion...

            # 'wait_for_completion' is not a param for the cli so verify
            # validity of the input here itself.
            unless ( $wait_for_completion == 1 || $wait_for_completion == 0 )
            {
                $self->_set_err(
                    "Error: SanFiler7M::aggr_64bit_upgrade(): Invalid value for argument '-wait_for_completion'"
                );
                return $ClientUtil::FALSE;
            } ## end unless ( $wait_for_completion...
            next;
        } elsif (/-?mode/) {
            if ( not length( $mode = $args{$_} ) ) {
                $self->_set_err(
                    "Error: SanFiler7M::aggr_64bit_upgrade(): Invalid value for argument '-mode'"
                );
                return $ClientUtil::FALSE;
            } ## end if ( not length( $mode...
            next;
        } elsif (/-?time_out/) {
            if ( ( $time_out = $args{$_} ) < 0 ) {
                $self->_set_err(
                    "Error: SanFiler7M::aggr_64bit_upgrade(): Invalid value for argument '-time_out'"
                );
                return $ClientUtil::FALSE;
            } ## end if ( ( $time_out = $args...
            next;
        } elsif (/-?sample_interval/) {
            if ( ( $sample_interval = $args{$_} ) < 0 ) {
                $self->_set_err(
                    "Error: SanFiler7M::aggr_64bit_upgrade(): Invalid value for argument '-sample_interval'"
                );
                return $ClientUtil::FALSE;
            } ## end if ( ( $sample_interval...
            next;
        } else {
            $self->_set_err(
                "Error: SanFiler7M:aggr_64bit_upgrade(): Invalid parameter "
                    . "specified \'$_\'" );
            return $ClientUtil::FALSE;
        } ## end else [ if (/-?aggr/)
    } ## end foreach ( keys %args )

    my $resultsHash_ref = $cmd_obj->execute_cmd(
        -cmd           => "aggr-64bit-upgrade-start",
        -cmd_filer     => $self->get_name(),
        -cmd_interface => "cli",
        -check_result  => 1,
        -aggr          => $aggr,
        -mode          => $mode,
        -cmd_privilege => "diag"
    );
    if ( $cmd_obj->error() ) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err( "Error: SanFiler7M:aggr_64bit_upgrade(): Unable to "
                . "start aggr upgrade: $errmsg" );
        return $ClientUtil::FALSE;
    } ## end if ( $cmd_obj->error())

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};
    my $expected_output = "Started 64-bit upgrade on aggregate $aggr and all contained writable volumes";
    if ( $output !~ /$expected_output/i )
    {
        $self->_set_err(
            "Error: SanFiler7M:aggr_64bit_upgrade(): Failed to start aggr upgrade on '$aggr'"
        );
        return $ClientUtil::FALSE;
    } ## end if ( $output !~ ...

    unless ( $resultsHash_ref->{result_check} ) {
        $self->_set_err( "Error: SanFiler7M:aggr_64bit_upgrade(): "
                . "Result check failed" );
        return $ClientUtil::FALSE;
    }

    # waiting for the scanner to start
    &SanUtil::nate_sleep(30);

    if ($wait_for_completion) {
        $self->wait_for_aggr_upgrade_completion(
            -aggr            => $aggr,
            -time_out        => $time_out,
            -sample_interval => $sample_interval
        );
        if ( $self->error() ) {
            return $ClientUtil::FALSE;
        }
    } ## end if ($wait_for_completion)

    return $ClientUtil::TRUE;
} ## end sub aggr_64bit_upgrade

################################################################################
# Subroutine Name: get_aggr_upgrade_status
# Objective:       Returns aggr upgrade status only
# Details:         See POD documentation at the end of this file
################################################################################
sub get_aggr_upgrade_status {

    my ( $self, %args ) = @_;
    my ( $aggr, $verbose ) = ( undef, 0 );
    my $cmd_obj = $self->cmd_obj();
    my ( @status, %status );
    $self->_clear_err();
    foreach ( keys %args ) {
        if (/-?aggr/) {
            if ( not length( $aggr = $args{$_} ) ) {
                $self->_set_err(
                    "Error: SanFiler7M::get_aggr_upgrade_status(): Invalid value for argument '-aggr'"
                );
                return "";
            } ## end if ( not length( $aggr...
            next;
        } elsif (/-?verbose/) {
            if ( not length( $verbose = $args{$_} ) ) {
                $self->_set_err(
                    "Error: SanFiler7M::get_aggr_upgrade_status: Invalid value for argument '-verbose'"
                );
                return "";
            } ## end if ( not length( $verbose...
            unless ( $verbose == 1 || $verbose == 0 ) {
                $self->_set_err(
                    "Error: SanFiler7M::get_aggr_upgrade_status: Invalid value for argument '-verbose'"
                );
                return "";
            } ## end unless ( $verbose == 1 || ...
            next;
        } else {
            $self->_set_err(
                "Error: SanFiler7M::get_aggr_upgrade_status: Invalid parameter "
                    . "specified \'$_\'" );
            return "";
        } ## end else [ if (/-?verbose/)
    } ## end foreach ( keys %args )

    my $resultsHash_ref = $cmd_obj->execute_cmd(
        -cmd           => "aggr-64bit-upgrade-status",
        -cmd_filer     => $self->get_name(),
        -cmd_interface => "cli",
        -check_result  => 1,
        -aggr          => $aggr,
        -verbose       => "",
        -cmd_privilege => "diag"
    );
    if ( $cmd_obj->error() ) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err(
                  "Error: SanFiler7M::get_aggr_upgrade_status: Unable to "
                . "get aggr upgrade status: $errmsg" );
        return "";
    } ## end if ( $cmd_obj->error())

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};
    unless ( $resultsHash_ref->{result_check} ) {
        $self->_set_err( "Error: SanFiler7M::aggr_64bit_upgrade_status(): "
                . "result check failed" );
        return "";
    }
    # output sample:
    # ==============
    #  Volume  Format    Scanner Status    % Completed  Time to Completion  Progress
    #    temp  upgrading stopped           0            N/A                 fbn 0, inode 0 of 32781, public
    #Storing the output in a hash
    my @op_strings = split( /\n/, $output );
    my ($i,$count,@output);
    foreach my $line(@op_strings){
	$line =~ s/^\s+//;
        $line =~ s/\s+/ /;
	@output = split( / /, $line );
	$count++;
	last if($output[0] =~ /^Volume$/g);
    }
    for ( $i = $count; $i < scalar(@op_strings); $i++ ) {
        $op_strings[$i] =~ s/^\s+//;
        $op_strings[$i] =~ s/\s+/ /;
        @status = split( / /, $op_strings[$i] );
        $status{$status[0]}{$status[1]} = $status[2];
        $status{$status[0]}{$status[1]}= "upgraded" if ( $status[1] =~ /64-bit/i );
        $status{$status[0]}{$status[1]}= "not_started" if ( $status[1] =~ /32-bit/i );
    }
    return \%status;

} ## end sub get_aggr_upgrade_status

################################################################################
# Subroutine Name: wait_for_aggr_upgrade_completion
# Objective:       Waits for aggr upgrade completion
# Details:         See POD documentation at the end of this file
################################################################################
sub wait_for_aggr_upgrade_completion {
    my ( $self, %args ) = @_;
    my ( $aggr, $time_out, $sample_interval ) = ( undef, 3600, 30 );
    $self->_clear_err();
    foreach ( keys %args ) {
        if (/-?aggr/) {
            if ( not length( $aggr = $args{$_} ) ) {
                $self->_set_err(
                    "Error: SanFiler7M::wait_for_aggr_upgrade_completion(): Invalid value for argument '-aggr'"
                );
                return $ClientUtil::FALSE;
            } ## end if ( not length( $aggr...
            next;
        } ## end if (/-?aggr/)
        if (/-?time_out/) {
            if ( ( $time_out = $args{$_} ) < 0 ) {
                $self->_set_err(
                    "Error: SanFiler7M::wait_for_aggr_upgrade_completion(): Invalid value for argument '-time_out'"
                );
                return $ClientUtil::FALSE;
            } ## end if ( ( $time_out = $args...
            next;
        } ## end if (/-?time_out/)
        if (/-?sample_interval/) {
            if ( ( $sample_interval = $args{$_} ) < 0 ) {
                $self->_set_err(
                    "Error: SanFiler7M::wait_for_aggr_upgrade_completion(): Invalid value for argument '-sample_interval'"
                );
                return $ClientUtil::FALSE;
            } ## end if ( ( $sample_interval...
            next;
        } else {
            $self->_set_err(
                "Error: SanFiler7M:wait_for_aggr_upgrade_completion(): Invalid parameter "
                    . "specified \'$_\'" );
            return $ClientUtil::FALSE;
        } ## end else [ if (/-?sample_interval/)
    } ## end foreach ( keys %args )

    while ( $time_out > 0 ) {
    #Getting the status of aggregate 64-bit upgrade
        my $op = $self->get_aggr_upgrade_status( -aggr => $aggr );
        if ( $self->error() ) {
            return $ClientUtil::FALSE;
        }
    my @volumes = keys(%{$op});
        if ( $op->{$volumes[0]} =~ /64-bit/i ) {
        #Aggregate 64-bit upgrade completed
            return $ClientUtil::TRUE;
        } elsif ( $op->{$volumes[0]} =~ /32-bit/i ) {
        #Aggregate 64-bit upgrade not yet started.Hence returning the failure.
            $self->_set_err(
                "Error: SanFiler7M::wait_for_aggr_upgrade_completion(): "
                    . "aggr upgrade is not started" );
            return $ClientUtil::FALSE;
        } elsif ( $op->{$volumes[0]}->{"upgrading"} =~ /stopped/i ) {
        #Aggregate 64-bit upgrade got stopped in between.Hence returning the failure.
            $self->_set_err(
                "Error: SanFiler7M::wait_for_aggr_upgrade_completion(): "
                    . "aggr upgrade is stopped" );
            return $ClientUtil::FALSE;
        }elsif ( $op->{$volumes[0]}->{"upgrading"} =~ /running/i ) {
        #Aggregate 64-bit upgrade is in progress
            &SanUtil::nate_sleep($sample_interval);
        }
        $time_out -= $sample_interval;
    } ## end while ( $time_out > 0 )
    $self->_set_err( "Error: SanFiler7M::wait_for_aggr_upgrade_completion(): "
            . "wait_for_aggr_upgrade_completion() timed out." );
    return $ClientUtil::FALSE;
} ## end sub wait_for_aggr_upgrade_completion

###############################################################################
# Method:    iscsi_portal_show
# Objective: To display the list of target portals over which the filer is
#            currently making available the iSCSI service.
# Details:   See POD documentation at the end of this file
# Inputs     -vfiler_name - optional - name of vfiler to execute cmd on
# Returns    Reference to "HASH", contains information about "IP address",
#            "TCP Port", "TPGroup", "Interface" as hash value - successful
#            execution
#            return {} and set_err - Execution failed
###############################################################################
sub iscsi_portal_show {
    my ($self, %args) = @_;
    my ($output_hash, $resultsHash_ref, $vfiler_name);
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::iscsi_portal_show()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?vfiler_name$/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return {};
            }
        }
    }

    my %cmd_args = (-cmd           => "iscsi-portal-list-info",
                    -cmd_filer     => $self->get_name(),
                    -check_output  => 1,
                    -check_result  => 1,
                    -cmd_interface => $self->interface());

    # Add in vfiler if needed.
    if ($vfiler_name) {
        $cmd_args{"-cmd_vfiler"} = $vfiler_name;
    }

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    if ($cmd_obj->error()) {
        $self->_set_err("iscsi_portal_show failed: ", $cmd_obj->errmsg());
        return {};
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # If result_check and output_check are not ok, return error.
    #
    if (not(    $resultsHash_ref->{result_check}
            and $resultsHash_ref->{output_check})
      ) {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return {};
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    $self->sanlog(-msg       => "iscsi_portal_show cmd returns:\n $output",
                  -msg_level => 7);

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if ($self->interface() eq "zapi") {
        $output_hash = $self->xml2hash($output);
        my $dumpOut = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => 3);

        # If there is only 1 element, the xml2hash returns a HASH instead of
        # ARRAY.
        # In this case, change the HASH to an ARRAY.
        # This allows the ZAPI hash to match the CLI hash, and easier processing
        # of the output, since everything will always be an ARRAY.

        if (ref($output_hash->{"iscsi-portal-list-entries"}
                  ->{"iscsi-portal-list-entry-info"}) eq "HASH"
          ) {
            $output_hash->{"iscsi-portal-list-entries"}
              ->{"iscsi-portal-list-entry-info"} =
              $self->_convert_hash_to_array_of_hash(
                                     $output_hash->{"iscsi-portal-list-entries"}
                                       ->{"iscsi-portal-list-entry-info"});
        }
    } else {
        $output_hash = $self->_iscsi_portal_show_cli_to_hash($output);

        # Verify the hash
        if (!$output_hash) {
            $self->_set_err("_iscsi_portal_show_cli_to_hash failed",
                            $self->errmsg(), $resultsHash_ref->{output});
            return {};
        }
    } ## end else [ if ($self->interface()...

    return $output_hash;
} ## end sub iscsi_portal_show

###############################################################################
# Method:    iscsi_interface_accesslist_show
# Objective: To get the information about interface accesslist.
# Details:   See POD documentation at the end of this file
# Inputs     An array of initiator IQNs (optional)
# Returns    "HASH", contains initiator IQN as key and interfaces(array)
#            as hash value - successful execution
#            return {} and set_err - Execution failed
###############################################################################
sub iscsi_interface_accesslist_show {
    my ($self, @iqn) = @_;
    my ($iqn) = join(' ', @iqn);
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();
    $self->sanlog(-msg => "iscsi interface accesslist show");

    my $resultsHash_ref;

    if ($self->interface() eq "cli") {
        $resultsHash_ref =
          $cmd_obj->execute_cmd(-cmd => "iscsi-interface-accesslist-list-info",
                                -cmd_filer        => $self->get_name(),
                                -check_result     => 1,
                                -check_output     => 1,
                                "-initiator-name" => $iqn);
    } else {
        $resultsHash_ref =
          $cmd_obj->execute_cmd(
                    -cmd          => "system-cli",
                    -cmd_filer    => $self->get_name(),
                    -check_result => 1,
                    -check_output => 1,
                    -args => ["iscsi", "interface", "accesslist", "show", @iqn],
                    -priv => "admin");
    } ## end else [ if ($self->interface()...

    if ($cmd_obj->error()) {
        $self->_set_err("iscsi inereface accesslist show failed: ",
                        $cmd_obj->errmsg());
        return {};
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (not(    $resultsHash_ref->{result_check}
            and $resultsHash_ref->{output_check})
      ) {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return {};
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    # If the interface is zapi we have to convert the output from xml
    # to a string.
    if ($self->interface() eq "zapi") {
        my $output_hash = $self->xml2hash($output);
        if ($self->get_debug_level() >= DUMPER_DEBUG_LEVEL) {
            my $dumpOut = Dumper($output_hash);
            $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                          -msg_level => DUMPER_DEBUG_LEVEL);
        }
    }

    # Sample output of "iscsi interface accesslist show"
    #
    # Initiator Name                      Access List
    #
    # iqn.1991-05.com.microsoft:sanqa-isns-1    e0a, e0b, e0c, e0d, e0e, e0f
    # iqn.1991-05.com.microsoft:sanqa-isns-2    e0a, e0b, e0c, e0d, e0e, e0f

    # Sample output of "iscsi interface accesslist show
    #                                 iqn.1991-05.com.microsoft:sanqa-isns-2"
    # If "iqn.1991-05.com.microsoft:sanqa-isns-2" not have interface
    #
    # Initiator Name                      Access List
    #
    # iqn.1991-05.com.microsoft:sanqa-isns-2    no accesslist

    # Sample output of "iscsi interface accesslist show" if no interface
    # access lists presents
    #
    # No interface access lists

    # Parse the output
    my @out_array = split(/\n/, $output);
    my %out_hash;
    foreach my $line (@out_array) {
        if ($line =~ /^iqn\..*/) {
            my @temp = split(/\s{2,}/, $line);

            # Create array of interfaces
            my @temp1 = split(/, /, $temp[1]);
            if ($temp[1] !~ /no accesslist/i) {

                # Put "Initiator Name" from temp[0] as hash key
                # and "reference to 'array of interfaces'" as value
                $out_hash{$temp[0]} = \@temp1;
            } else {
                $out_hash{$temp[0]} = [];
            }
        } ## end if ($line =~ /^iqn\..*/)
    } ## end foreach my $line (@out_array)
    return %out_hash;
} ## end sub iscsi_interface_accesslist_show

###############################################################################
# Method   : clear_mem_stats
# Objective: Free the pcstats resources
# Details  : See POD documentation at the end of this file
# Inputs   : None
# Returns  : $ClientUtil::TRUE - Successful execution
#            $ClientUtil::FALSE & set_err - Execution failed
###############################################################################
sub clear_mem_stats {
    my ($self) = @_;

    my ($cmd_obj, $resultsHash_ref);
    $cmd_obj = $self->{cmd_obj};

    $self->_clear_err();
    $self->sanlog(-msg => "Start clear_mem_stats");

    if ($self->interface() eq "cli") {
        $resultsHash_ref =
          $cmd_obj->execute_cmd(-cmd           => "system-mem-stats",
                                -cmd_filer     => $self->{filer},
                                -check_result  => 1,
                                -check_output  => 1,
                                -cmd_privilege => "diag",
                                -clear         => "");
    } else {
        $resultsHash_ref =
          $cmd_obj->execute_cmd(-cmd           => "system-cli",
                                -cmd_filer     => $self->get_name(),
                                -cmd_interface => "zapi",
                                -check_result  => 1,
                                -check_output  => 1,
                                -args          => ["mem_stats", "-c"],
                                -priv          => "diag");
    } ## end else [ if ($self->interface()...

    if ($cmd_obj->error()) {
        $self->_set_err("clear_mem_stats failed:", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (not(    $resultsHash_ref->{result_check}
            and $resultsHash_ref->{output_check})
      ) {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    # Get the output
    my $output = $resultsHash_ref->{output};

    if ($output =~ /^mem_stats:.*/i) {
        $self->_set_err("clear_mem_stats failed:", $output);
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;

} ## end sub clear_mem_stats()
###############################################################################
# Method:    snap_create()
# Objective: create snap shot
# Details:   See POD documentation at the end of this file.
###############################################################################
sub snap_create {
    my ($self,%args) = @_;
    my (
        @output,
        $cmd,
        $name,
        $aggr,
        );
    $self->_clear_err();
    # parse arguments
    foreach (keys %args) {
        if (/-?name/) {
            if (not length($name = $args{$_})) {
                $self->_set_err("Error: SanFiler7M::snap_create() Invalid value for argument '-name'");
                return {};
            }
            next;
        }
        if (/-?aggr/) {
            if (not length($aggr = $args{$_})) {
                $self->_set_err("Error: SanFiler7M::snap_create() Invalid value for argument '-aggr'");
                return {};
            }
            next;
        }
        else {
            $self->_set_err("Error: SanFiler7M::snap_create() Invalid parameter specified \'$_\'");
            return {};
        }
    }
    unless (defined $name and length $name){
        $self->_set_err("Error: SanFiler7M::snap_create() Missing required argument '-name'");
        return {};
    }
    unless (defined $aggr and length $aggr){
        $self->_set_err("Error: SanFiler7M::snap_create() Missing required argument '-aggr'");
        return {};
    }
    $self->sanlog(-msg => "Creating snapshot '$name' ");
    # Quote snapname since it might contain spaces
    $cmd="snap create -A $aggr '$name'";
    @output=$self->execute($cmd,-hide=>1);

    if ("@output"!~/creating snapshot\.\.\./i){
        my $errmsg = $output[0];
        $self->_set_err(
            "Error: SanFiler7M::snap_create(): Unable to create " .
            "snapshot '$name: $errmsg");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    snap_delete()
# Objective: delete snap shots
# Details:   See POD documentation at the end of this file.
###############################################################################
sub snap_delete
{
    my ($self,%args) = @_;
    my (
        @buffer,
        @output,
        @tmp,
        $cmd,
        $found,
        $name,
        $aggr,
        );

    $self->_clear_err();
    $found=0;
     # parse arguments
    foreach (keys %args) {
        if (/-?name/) {
            if (not length($name = $args{$_})) {
                $self->_set_err("Error: SanFiler7M::snap_delete() Invalid value for argument '-name'");
                return {};
            }
            next;
        }
        if (/-?aggr/) {
            if (not length($aggr = $args{$_})) {
                $self->_set_err("Error: SanFiler7M::snap_delete() Invalid value for argument '-aggr'");
                return {};
            }
            next;
        }
        else {
            $self->_set_err("Error: SanFiler7M::snap_delete() Invalid parameter specified \'$_\'");
            return {};
        }
    }

    unless (defined $name and length $name){
        $self->_set_err("Error: SanFiler7M::snap_delete() Missing required argument '-name'");
        return {};
    }
    unless (defined $aggr and length $aggr){
        $self->_set_err("Error: SanFiler7M::snap_delete() Missing required argument '-aggr'");
        return {};
    }


    if ($name=~/\*/){
        $self->sanlog(-msg => "deleting all snapshots from aggr '$aggr'");
        $cmd="snap delete -A -a -f $aggr ";
    }
    else {
        $self->sanlog(-msg => "deleting snapshot '$name' from aggr '$aggr");

        # Quote snapname since it might contain spaces
        $cmd="snap delete -A '$aggr' '$name'";
    }


    @output=$self->execute($cmd,-hide=>1);

    unless ($output[0] =~ /deleting snapshot\.\.\./){
        my ($errmsg)=join "\n", @output;
        $self->_set_err("Error: SanFiler7M::snap_delete() $errmsg");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
} # end snap_delete()
###############################################################################
# Method:    snap_restore()
# Objective: restore snap shots
# Details:   See POD documentation at the end of this file.
###############################################################################
sub snap_restore {
    my ($self,%args) = @_;
    my (
        @output,
        $cmd,
        $name,
        $aggr,
        );

    $self->_clear_err();
     # parse arguments
    foreach (keys %args) {
        if (/-?name/) {
            if (not length($name = $args{$_})) {
                $self->_set_err("Error: SanFiler7M::snap_restore() Invalid value for argument '-name'");
                return {};
            }
            next;
        }
        if (/-?aggr/) {
            if (not length($aggr = $args{$_})) {
                $self->_set_err("Error: SanFiler7M::snap_restore() Invalid value for argument '-aggr'");
                return {};
            }
            next;
        }
        else {
            $self->_set_err("Error: SanFiler7M::snap_restore() Invalid parameter specified \'$_\'");
            return {};
        }
    }

    unless (defined $name and length $name){
        $self->_set_err("Error: SanFiler7M::snap_restore() Missing required argument '-name'");
        return {};
    }
    unless (defined $aggr and length $aggr){
        $self->_set_err("Error: SanFiler7M::snap_restore() Missing required argument '-aggr'");
        return {};
    }

    $self->sanlog(-msg => "Restoring snapshot '$name' for aggr '$aggr'");

    # Quote snapname since it might contain spaces
    $cmd="snap restore -A -f -s '$name' $aggr";

    @output=$self->execute($cmd,-hide=>1);

    if ("@output"!~/revert successful/i){
        my $errmsg = $output[0];
        $self->_set_err(
            "Error: Volume::snap_restore(): Unable to restore " .
            "snapshot '$name' on aggr '$aggr': $errmsg");
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}
###############################################################################
# Method:    snap_list()
# Objective: Get list of snapshots
# Details:   See POD documentation at the end of this file.
###############################################################################
sub snap_list
{
    my ($self,%args) = @_;
    my (
        @buffer,
        @output,
        @tmp,
        $cmd,
        $found,
        $filter,
        $aggr,
        );

    $self->_clear_err();
    $found=0;
    # parse arguments
    foreach (keys %args) {
        if (/-?filter/) {
            if (not length($filter = $args{$_})) {
                $self->_set_err("Error: SanFiler7M::snap_list() Invalid value for argument '-filter'");
                return (wantarray()) ? () : [];
            }
            next;
        }
        if (/-?aggr/) {
            if (not length($aggr = $args{$_})) {
                $self->_set_err("Error: SanFiler7M::snap_list() Invalid value for argument '-aggr'");
                return {};
            }
            next;
        }
        else {
            $self->_set_err("Error: SanFiler7M::snap_list() Invalid parameter specified \'$_\'");
            return (wantarray()) ? () : [];
        }
    }
    unless (defined $aggr and length $aggr){
        $self->_set_err("Error: SanFiler7M::snap_list() Missing required argument '-aggr'");
        return {};
    }

    Tharn::logcomment("Listing snapshots for aggregate $aggr");

    $cmd="snap list -A $aggr";

    @output=$self->execute($cmd,-hide=>1);
    if ($output[1] =~ /does not exist/i){
        my ($errmsg)=$output[1]=~/\:(.*)/;
        $self->_set_err("Error: SanFiler7M::snap_list() $errmsg");
        return (wantarray()) ? () : [];
    }

    # Get the field header
    push @buffer,(@output[3..4]);

    # Nate dumps out if it sees '--' in output
    # Lets substitute with '=' before we log.
    $buffer[1]=~s/-/=/g;

    # Store the rest of the data here for filtering
    @tmp=(@output[5..scalar(@output)-1]);

    if (defined $filter and length $filter) {
        $filter=~s/:/\//g;
        $filter=basename($filter);

        foreach my $line (@tmp){
            # take spaces in account for snapname
            my($name)=$line=~/.*:\d+\s+(.*)/;

            if (_trim($name) =~ /$filter/){
                push @buffer,$line;
                $found=1;
            }

        }
        if ($found==0){
            $self->_set_err("Error: SanFiler7m::snap_list() No snapnames with filter '$filter' found");
            return (wantarray()) ? () : [];
        }

    }
    else {
        push @buffer, @tmp;
    }

    push @buffer,"\n\n";
    return (wantarray()) ? @buffer : \@buffer;
} # end snap_list()
###############################################################################
# Method:    snap_exist()
# Objective: Checks if a specific snap exists on the specified aggregate.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub snap_exist
{
    my ($self, %args) = @_;
    my $snapname;
    my $aggr;
    $self->_clear_err();
    # parse arguments
    foreach (keys %args)
    {
        if (/-?snapname/) {
            if (not length($snapname = $args{$_})) {
                $self->_set_err("Error: SanFiler7M:snap_exist() Invalid value " .
                                "for argument '-snapname'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        if (/-?aggr/) {
            if (not length($aggr = $args{$_})) {
                $self->_set_err("Error: SanFiler7M::snap_exist() Invalid value for argument '-aggr'");
                return {};
            }
            next;
        }
        else {
            $self->_set_err("Error: SanFiler7M:snap_exist() Invalid parameter " .
                            "specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }

    if(not defined($snapname) or not length($snapname))
    {
        $self->_set_err("Error: SanFiler7M::snap_exist() " .
                        "Missing required argument '-snapname'");
        return $ClientUtil::FALSE;
    }
    if(not defined($aggr) or not length($aggr))
    {
        $self->_set_err("Error: SanFiler7M::snap_exist() " .
                        "Missing required argument '-aggr'");
        return $ClientUtil::FALSE;
    }

    $self->sanlog(-msg => "SanFiler7M::snap_exist() Checking if $snapname exists",
                  -msg_level => 1);
                  #-msg_level => $LOG_VERBOSE_LOW);

    my @snap_info = $self->snap_list( -aggr => $aggr );

    #
    # The regex is expecting lines in the format of:
    # Jun 6 00:00 snapname
    #
    foreach my $line (@snap_info)
    {
        if($line =~ /\s*\w+\s+\d+\s+\d+:\d+\s+(\S+)/)
        {
            my $sname = $1;

            if($sname eq $snapname)
            {
                return $ClientUtil::TRUE;
            }
        }
    }

    $self->_set_err("SanFiler7M::snap_exist(): Snap $snapname not found");
    return $ClientUtil::FALSE;
} # end snap_exist()

###############################################################################
#                               Private Methods
###############################################################################

###############################################################################
# Method:    _connect
# Objective: Create telnet connection to CLI, log in using specified
#            user, password and wait for initial prompt.
# Inputs:
#            $host        Hostname or IP Address of NZSQL client.
#            $timeout     Telnet timeout     (optional)
# Outputs:   Returns tn object (telnet object) if successful,
#            0 if not (error flag and message will be set).
# Affected Data: If successful, object contains reference to new Telnet
#                object.  Specified IP address, user, password and timeout
#                are also stored in the object.
###############################################################################
sub _connect
{
    my($self,$filername, $timeout, $conn_str) = @_;
    my(
       $connect,
       $errmsg,
       $filer,
       $rsh,
       $match,
       $rc,
       );

    # Variable init.
    $self->{connected} = 0;

    $filer = Tharn::host($filername);

    $self->{name}=$filer->{hostname};

    $self->{default_ip}=$filer->{default_ip};

    if ($conn_str eq "") {
        $conn_str="#rsh";
    }

    $connect = Tharn::connect($conn_str.";timeout=$timeout;inherit=0;",
                              -eval=>sub{
                                  if (@_) {
                                      my $errmsg = join("",@_);
                                      $errmsg =~ s/\n//g;
                                      $self->_set_err($errmsg);
                                      $Tharn::InEND=1; #Don't abort
                                      Tharn::logresult("CONFIG","\n@_");
                                      return $self;
                                  }
                              });

    #unless (defined($timeout) and length($timeout)){
        #$self->{timeout}=$connect->{timeout};
    #}

    $self->{connected} = 1;
    return $connect;

} # end _connect()

###############################################################################
# Method:    _rlm_console_connect
# Objective: Create rlm console connection to CLI, log in using specified
#            user, password and wait for initial prompt.
# Inputs:
#            $host        Hostname or IP Address of NZSQL client.
#            $timeout     Connect timeout     (optional)
# Outputs:   Returns rlm conn object (rlm console  conn object) if successful,
#            0 if not (error flag and message will be set).
# Affected Data: If successful, object contains reference to new console
#                object.
###############################################################################
sub _rlm_console_connect
{
    my($self, $filername, $timeout, $conn_str) = @_;
    my(
       $connect,
       $errmsg,
       $filer,
       $received_prompt,
       $expected_prompt
       );

    $self->sanlog(-msg => "Trying to connect to the rlm console");

    # If we don't have a conn_str defined use rlm
    if ($conn_str eq "") {
        $conn_str="#rlm;conntype=ssh-persist";
    }

    $filer = &Tharn::host($filername);

    $connect = &Tharn::connect($conn_str.";timeout=$timeout;",
                              -eval=>
                                 sub {
                                     if (@_) {
                                         my $errmsg = join("",@_);
                                         $errmsg =~ s/\n//g;
                                         $self->_set_err($errmsg);
                                         $Tharn::InEND=1; #Don't abort
                                         &Tharn::logresult("CONFIG",
                                                           abort=>0,"\n@_");
                                         return {};
                                     }
                                 }
                             );

    # At this point we are connected to the rlm connect to system console now
    $expected_prompt = "Type Ctrl-D to exit.";
    (undef, undef, undef, $received_prompt) =
        &Tharn::say($connect, "system console",
                    '-noprompt', '-prompt', $expected_prompt,
                    '-timeout', $timeout,
                    '-ignore', "echo ignore",
                    '-comment', "Attempted system console");
    if ($received_prompt !~ /$expected_prompt/) {
        $self->_set_err("Did not receive expected prompt ($expected_prompt)");
        return {};
    }

    # Handle a login prompt if we get one.
    # This will happen if a password is set for root
    # Combine all the known expected prompt in one str
    $expected_prompt = "login:|Password:";
    if (exists ($filer->{prompt_fw})) {
        $expected_prompt .= "|";
        $expected_prompt .= join("|", @{$filer->{prompt_fw}});
    }
    if (exists ($filer->{prompt_virt})) {
        $expected_prompt .= "|";
        $expected_prompt .= join("|", @{$filer->{prompt_virt}});
    }
    if (exists ($filer->{prompt_take})) {
        $expected_prompt .= "|";
        $expected_prompt .= join("|", @{$filer->{prompt_take}});
    }
    if (exists ($filer->{prompt_norm})) {
        $expected_prompt .= "|";
        $expected_prompt .= join("|", @{$filer->{prompt_norm}});
    }

    for (1..2) {
        (undef, undef, undef, $received_prompt) =
            &Tharn::say($connect, "\n",
                        '-noprompt', '-prompt', $expected_prompt,
                        '-timeout', $timeout,
                        '-ignore', "echo ignore",
                        '-comment', "Attempted console login");
    }

    # Make sure the console is on one of the expected prompts
    if ($received_prompt !~ /$expected_prompt/) {
        $self->_set_err("The received prompt($received_prompt) wasn't " .
                        "one of the expected prompts($expected_prompt).");
        return {};
    }

    # Actual handling of the login prompt.
    if ($received_prompt =~ /login:/) {
        my $user = "root";

        if (defined $filer->{username}) {
            $user = $filer->{username};
        }

        # We need to do a blind say on the console to send
        # username and password without entering and waiting
        # for prompts. We don't wait for a password just enter
        # it if we get a login. The sleep in the middle is for the
        # rlm because sometimes it can be slow.
        &Tharn::say($connect, "$user",
                    '-blind',
                    '-timeout', $timeout,
                    '-comment', "Attempted system console");
        &SanUtil::nate_sleep(5);
        &Tharn::say($connect, $filer->{default_password},
                    '-blind',
                    '-timeout', $timeout,
                    '-comment', "Attempted system console");
    } elsif ($received_prompt =~ /Password:/) {
        # Sometimes we might get just the password prompt.
        &Tharn::say($connect, $filer->{default_password},
                    '-blind',
                    '-timeout', $timeout,
                    '-comment', "Attempted system console");
    }

    # Sometime when the password is set and after we login we don't
    # see a regular prompt so say something to the console and get a prompt.
    for (1..2) {
        &Tharn::say($connect, "\n",
                    '-blind',
                    '-timeout', $timeout,
                    '-comment', "Attempted console login");
    }

    # Just save in the consle object by default
    $self->{console_conn} = $connect;

    return $connect;
} # end _rlm_console_connect()

###############################################################################
# Method:    _get_iscsi_tpgroup_tag()
# Objective: get a tpgroup's tag by using the name
# Details:   There are none.
###############################################################################
sub _get_iscsi_tpgroup_tag
{
    ##
    ## TBD:
    ## CURRENTLY THIS ONLY SUPPORTS ZAPI... THERE SHOULD BE NO NEED FOR
    ## CLI TO GET THE TPGROUP TAG
    ##

    my ($self, $name)=@_;
    my $cmd_obj = $self->cmd_obj();
    my $tpgtag = "";
    my $flag = 0;

    $self->_clear_err();

    $self->sanlog(-msg => "get tpgroup tag");

    unless (length $name)
    {
        $self->_set_err("ERROR: SanFiler::_get_iscsi_tpgroup_tag() Required " .
                        "parameter \$name required");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref =
        $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-tpgroup-list-info",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 0);

    if($cmd_obj->error())
    {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Error: SanFiler::_get_iscsi_tpgroup_tag(): Unable " .
                        "to determine tpgroup existance: $errmsg");
        return "";
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my @output = split("\n", $resultsHash_ref->{output});
    my $output_hash = $self->xml2hash($resultsHash_ref->{output});
    if($self->get_debug_level() >= DUMPER_DEBUG_LEVEL)
    {
        my $dumpOut = Dumper($output_hash);
        $self->sanlog(-msg       => "Output Hash is:\n$dumpOut",
                      -msg_level => DUMPER_DEBUG_LEVEL);
    }

    my $status = $output_hash->{status};

    if($status eq "failed")
    {
        my $errmsg=$output_hash->{reason};
        $self->_set_err("Error: SanFiler::_get_iscsi_tpgroup_tag() " .
                        "no tpgroups found",$errmsg);
        return "";
    }
    else
    {
        #
        # Gets an array of hashes
        #
        my $arr = $output_hash->{"iscsi-tpgroup-list-entries"}->
                              {"iscsi-tpgroup-list-entry-info"};

        #
        # Go through array grabbing hashes
        #
        foreach my $hash1 (@$arr)
        {
            foreach my $key (keys(%{$hash1}))
            {
                if($key eq "tpgroup-name")
                {
                    my $entry = $hash1->{$key};
                    if($entry eq $name)
                    {
                        $tpgtag = $hash1->{"tpgroup-tag"};
                        $flag = 1;
                    }
                }
            }
        } # end foreach $hash1

        if($flag == 0) # we never found the tpgroup
        {
            $self->_set_err("Error: SanFiler::_get_iscsi_tpgroup_tag(): " .
                            "Unable to locate tpgroup");
            return "";
        }
        else
        {
            return $tpgtag;
        }
    }
} # end sub _get_iscsi_tpgroup_tag

###############################################################################
# Method:    _get_iscsi_tpgroup_alua_state()
# Objective: get a tpgroup's alua state
# Details:   There are none.
###############################################################################
sub _get_iscsi_tpgroup_alua_state
{
    ##
    ## TBD:
    ## CURRENTLY THIS ONLY SUPPORTS CLI... THERE SHOULD BE NO NEED FOR ZAPI
    ## TO NEED TO GET THIS INFO, AS IT IS INCLUDED IN ISCSI_TPGROUP_SHOW
    ##

    my ($self, $name)=@_;
    my $cmd_obj = $self->cmd_obj();
    my $alua = "";

    $self->_clear_err();

    $self->sanlog(-msg => "get tpgroup alua state");

    unless (length $name) {
        $self->_set_err("Missing required argument \$name");
        return "";
    }

    my $resultsHash_ref =
        $cmd_obj->execute_cmd(
                        -cmd                    => "iscsi-tpgroup-alua-show",
                        -cmd_filer              => $self->get_name(),
                        -check_result           => 1,
                        -check_output           => 1);

    if($cmd_obj->error()) {
        $self->_set_err("Unable to get information on tpgroup alua state", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->log_cmd_results($resultsHash_ref);

    # If result_check and output_check are not ok, return error.
    if (!($resultsHash_ref->{result_check} && $resultsHash_ref->{output_check})) {
        $self->_set_err("results check failed", $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    #
    # Get the output
    #
    my @output = split('\n', $resultsHash_ref->{output});

    if($output[0] =~ /iscsi tpgroup alua show:/i) {
        $self->_set_err("no tpgroups found", @output);
        return "";
    } else {
        foreach my $line (@output) {
            my ($tag, $n, $alua, $pref) = split(' ', $line);

            if($n eq $name) {
                if($pref ne "") {
                    return $alua . " " . $pref;
                } else {
                    return $alua;
                }
            }
        }
        return "";
    }
} # end sub _get_iscsi_tpgroup_alua_state

###############################################################################
# Method:    _load_targets()
# Objective: Return important information about filer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _load_targets
{
    my ($self)=@_;
    my (
        $adaptLabel,
        $cfmode,
        $cmd,
        $description,
        $name,
        $nodename,
        $portname,
        $status,
        $version,
        %hsh,
        @output,
        @slots,
        );


    # get all target info and store it in a hash
    $cmd="fcp show adapter -v";

    @output=$self->execute($cmd,-hide=>1);

    # Determine fcp cfmode.  The 'fcp adapter' info
    # for adapter label is different.
    $cfmode=$self->get_cfmode();
    if ($cfmode=~/partner|cluster|Not Clustered|standby|single_image/i) {
        $adaptLabel="Slot";
    }
    elsif ($cfmode=~/mixed|dual/i) {
        $adaptLabel="Name";
    }
    else {
        $self->_set_err("Error: SanFiler::_load_targets() The cfmode '$cfmode' in currently",
                        "not supported by this library");
        return $ClientUtil::FALSE;
    }


    # Retrieve specific fields from output and store in hash
    foreach my $line (@output){

        next unless (
                       $line=~/$adaptLabel\:|^Status\:|^Adapter Type\:|^Firmware Rev\:|^FC Nodename\:/ or
                       $line=~/^FC Portname\:|^Description\:/
                       );

        my ($key,$value)=$line=~/(\S+):\s+(\S+)/i;

        if ($key=~/^$adaptLabel$/){
            $name=$value;

            # In mixed mode 'description' and 'firmware rev' only
            # appear under the adapter id.  Use the last values
            # set for these attributes.
            $hsh{$name}{Description}=$description if (defined $description and length $description);
            $hsh{$name}{Rev}=$version if (defined $version and length $version);

            # $self->sanlog(-msg => "REV: ".$hsh{$name}{Rev});
        }

        if ($key=~/Description/i){
            $description=$value;
        }
        elsif($key=~/Rev/){
            $version=$value;
        }

        if (defined $name and length $name){
            $hsh{$name}{$key}=$value;
        }

    }

    # Create a target object for each target
    @slots=(sort keys %hsh);

    foreach my $slot (@slots){
        $hsh{$slot}{-loghdl}=$self->{log} if (defined $self->{log});
        $hsh{$slot}{-parent}=$self;

        # Create target object
        my $targObj = Target->_new($hsh{$slot});
        if ($targObj->error){
            $self->_set_err($targObj->errmsg());
            return $ClientUtil::FALSE;
        }

        push @{$self->{targets}},$targObj;

    }


    return $ClientUtil::TRUE;
}

###############################################################################
# Method:    refresh_iscsi_targets()
# Objective: Return important information about filer iscsi targets
# Details:   See POD documentation at the end of this file.
###############################################################################
sub refresh_iscsi_targets {

    my ($self) = @_;

    return $self->_load_iscsi_targets();
}

###############################################################################
# Method:    _load_iscsi_targets()
# Objective: Return important information about filer iscsi targets
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _load_iscsi_targets {
    my ($self) = @_;

    my ($cmd, $type, @output, $status);
    my ($nodename, $name, $ipv4, $ipv6, $tpgt, $tpgn, $ipv4_netmask, $ipv6_prefix_len);
    my ($vlan_tag, $vif_cmd, $int, $mac, $version, $state, @iscsi_tpgroup_show);
    my (@vfiler_status, $vfiler, $vfiler_enabled);

    # determine the management ip from the tharnhost file
    my $filer_hostrec = &Tharn::host($self->get_name());
    my $mgmt_ip = $filer_hostrec->{default_ip};

    # get the vfiler information on the system
    # sample output:
    # rtpqa-fas6040-15> vfiler status -a
    # vfiler0                          running
    #   ipspace: default-ipspace
    #   IP address: 10.60.169.124 [e0e]
    #   Path: / [/etc]
    #   UUID: 00000000-0000-0000-0000-000000000000
    #   Protocols allowed: 7
    #   Allowed: proto=rsh
    #   Allowed: proto=ssh
    #   Allowed: proto=nfs
    #   Allowed: proto=cifs
    #   Allowed: proto=iscsi
    #   Allowed: proto=ftp
    #   Allowed: proto=http
    #   Protocols disallowed: 0
    #
    $cmd = "vfiler status -a";
    @vfiler_status = $self->execute($cmd, -hide => 1);
    if ($self->error()) {
        $self->_set_err("command ($cmd) failed");
        return $ClientUtil::FALSE;
    }
    if ("@vfiler_status" =~ /vfilers requires a multistore license|vfilers requires multistore licensed_feature option enabled/) {
        # multistore is not licensed on this system
        $vfiler_enabled = 0;
    } else {
        $vfiler_enabled = 1;
    }

    # get the target portal group tag for that interface
    $cmd = "iscsi tpgroup show";
    @iscsi_tpgroup_show = $self->execute($cmd, -hide => 1);
    if ($self->error()) {
        $self->_set_err("command ($cmd) failed");
        return $ClientUtil::FALSE;
    }

    # For iSCSI we need to goto the filer and locate all enabled interfaces
    # We then collect the ip address to place in the iscsi configuration of the client
    $version = $self->get_version_major();
    if ($version =~ /7\.0/i) {
        $cmd = "iswt interface show";
    } else {
        $cmd = "iscsi interface show";
    }
    @output = $self->execute($cmd, -hide => 1);
    if ($self->error()) {
        $self->_set_err("command ($cmd) failed");
        return $ClientUtil::FALSE;
    }

    # vif = igrp in 8.0+
    $vlan_tag = "";
    if ($version >= 8.0) {
        $vif_cmd = "ifgrp";
    } else {
        $vif_cmd = "vif";
    }

    foreach my $line (@output) {
        # initialize (case where interface is disabled)
        $ipv4 = $ipv6 = $tpgt = $tpgn = "not_set";

        # get state of the interface
        if ($line =~ /\s+(\S+)\s+disabled/) {
            $int = $1;
            $state = "disabled";
        } elsif ($line =~ /\s+(\S+)\s+enabled/) {
            $int = $1;
            $state = "enabled";
        } else {
            $state = "unknown";
            $int = "unknown";
        }
        $name = $int;

        if ($state =~ /enabled|disabled/i) {
            my (@vif_output, $is_vif);
            # first, check to see if the interface is a VIF or VLAN
            my $vif = $int;
            $vif =~ s/\s+$//;
            # VLAN tag is added to send as "-<vlan_tag>"
            $vif =~ s/\-(\d+)$//;
            $vlan_tag = $1;
            if (defined($vif) and $vif ne "") {
                $cmd = "$vif_cmd status $vif";
                @vif_output = $self->execute($cmd, -hide => 1);
                if ($self->error() or grep(/bad\s+(vif|igrp)\s+given/, @vif_output) > 0) {
                    $is_vif = 0;
                } else {
                    $is_vif = 1;
                }
            }

            # get the interface information
            $cmd = "ifconfig $int";
            my @if_output = $self->execute($cmd, -hide => 1);
            if ($self->error()) {
                $self->_set_err("command ($cmd) failed");
                return $ClientUtil::FALSE;
            }

            # Sample output, ip address assigned:
            # e0b: flags=0x2f48867<UP,BROADCAST,RUNNING,MULTICAST,TCPCKSUM> mtu 1500
            #         inet 10.228.128.45 netmask 0xfffff800 broadcast 10.228.135.255
            #         partner inet 10.228.128.47 (not in use)
            #         ether 00:a0:98:06:15:f3 (auto-1000t-fd-up) flowcontrol full
            # Sample output, no ip address:
            # e0c: flags=0x2708866<BROADCAST,RUNNING,MULTICAST,TCPCKSUM> mtu 1500
            #         ether 00:a0:98:06:15:f4 (auto-unknown-down) flowcontrol full

            # Determine if this interface has an ip address assigned.
            # If we find 'inet' on a line, and partner is not on the line,
            # then turn on the flag.
            # Note the check for partner helps to cover this case:
            # e0d: flags=0x2708866<BROADCAST,RUNNING,MULTICAST,TCPCKSUM> mtu 1500
            #         partner inet 1.1.1.1 (not in use)
            #         ether 00:a0:98:08:3b:29 (auto-unknown-down) flowcontrol full
            my $inet_found = 0;
            foreach my $ifconfig_line (@if_output) {
                if ($ifconfig_line =~ /inet/ and $ifconfig_line !~ /partner/) {
                    $inet_found = 1;
                }
            }

            # look through ifconfig output, there may be multiple
            # ip addresses configured on this interface.  each ip address
            # will be a separate iscsi target
            foreach my $ifconfig_line (@if_output) {
                # get IPv4 address and netmask
                if ($ifconfig_line =~ /inet\s+(\S+)\s+netmask.+?(0x\S+)\s+broadcast/) {
                    $ipv4 = $1;
                    $ipv4_netmask = $2;
                } else {
                    $ipv4 = "not_set";
                    $ipv4_netmask = "not_set";
                }

                # get IPv6 address and prefix
                if ($ifconfig_line =~ /inet6\s+(\S+)\s+prefixlen\s+(\S+)/) {
                    $ipv6 = $1;
                    $ipv6_prefix_len = $2;
                } else {
                    $ipv6 = "not_set";
                    $ipv6_prefix_len = "not_set";
                }

                # get MAC address
                if ("@if_output" =~ /ether\s+(\S+)\s+/) {
                    $mac = $1;
                } else {
                    $mac = "not_set";
                }

                # get vfiler for this ip address
                # default it to vfiler0
                $vfiler = "vfiler0";
                my $vfiler_name;
                # loop through each vfiler found in the status output
                foreach my $vfiler_line (@vfiler_status) {
                    # if it is running, set the current name to this vfiler
                    if ($vfiler_line =~ /(\S+)\s+running/) {
                        $vfiler_name = $1;
                        next;
                    }
                    # as we loop through the output, the next line will
                    # contain the IP information.  if this matches the current
                    # $ipv4 variable, we have found the vfiler that is
                    # configured with this IP
                    if ($vfiler_line =~ /$ipv4/) {
                        $vfiler = $vfiler_name;
                        last;
                    }
                }

                if ($vfiler_enabled ) {
                    # get the iscsi nodename of that vfiler
                    $cmd = "vfiler run -q $vfiler iscsi nodename";
                } else {
                    $cmd = "iscsi nodename";
                }
                my @nodename_output = $self->execute($cmd, -hide => 1);
                if ($self->error()) {
                    $self->_set_err("command ($cmd) failed");
                    return $ClientUtil::FALSE;
                }

                if ($nodename_output[0] =~ /iSCSI target nodename:\s+(\S+)/) {
                    $nodename = $1;
                } else {
                    $nodename = "not_set";
                }

                # if this is a VIF, go and get contributors to the VIF
                my (@vif_contrib_interfaces);
                my ($vif_contrib_ints, $vif_type);
                if ($is_vif == 1) {
                    # example output from vif status:
                    # default: transmit 'IP Load balancing', VIF Type 'multi_mode', fail 'log'
                    # vif-iscsi: 2 links, transmit 'IP Load balancing', VIF Type 'multi_mode' fail 'default'
                    # VIF Status     Up      Addr_set
                    # up:
                    # e1b: state up, since 19Aug2009 16:15:38 (10+16:35:19)
                    # mediatype: auto-10g_sr-fd-up
                    $vif_type = grep(/(VIF|IGRP)\s+Type\s+\'(\S+)\'/, @vif_output);
                    $vif_type = $2;
                    my @vif_interfaces = grep(/^\s+\S+\:\s+state\s+\S+/i, @vif_output);
                    if (scalar(@vif_interfaces) > 0) {
                        foreach my $vif_interface (@vif_interfaces) {
                            $vif_interface =~ m/^\s+(\S+)\:\s+state\s+(\S+)/i;
                            $vif_interface = $1;
                            push(@vif_contrib_interfaces, $vif_interface);
                        }
                        # gen scalar to pass to object creation
                        $vif_contrib_ints = join(';', @vif_contrib_interfaces);
                    }
                }

                # look through target port group show output for
                # interface and store the name and tag.
                foreach my $iscsi_tpg_line (@iscsi_tpgroup_show) {
                    # skip header
                    if ($iscsi_tpg_line !~ /Member Interfaces/i) {
                        $iscsi_tpg_line =~ s/^\s+//;
                        my ($tag, $tp_name, $interface) = split(/\s+/, $iscsi_tpg_line);
                        if ($interface eq $int) {
                            $tpgt = $tag;
                            $tpgn = $tp_name;
                            last;
                        }
                    }
                }

                # determine if this port is the management port
                my $is_mgmt_port = 0;
                if ($ipv4 eq $mgmt_ip or $ipv6 eq $mgmt_ip) {
                    $is_mgmt_port = 1;
                }

                # We are now looping through the output of ifconfig
                # each interface will have a MAC address line and possibly
                # a line defining the partner IP config.  If
                # that is the current line we are processing, we don't
                # need to create an iscsi target for that.
                # Also: If we are processing the flags line and an ip address
                # is coming on the next line, then skip creating a target
                # with ip="not_set".
                unless ($ifconfig_line =~ /ether\s+\S+\s+/ or
                        $ifconfig_line =~ /partner/ or
                        (($ifconfig_line =~ /:\s+flags=/) and $inet_found) ) {
                    # Create iscsi target object (if interface is enabled for iSCSI)
                    my $targObj = ISCSITarget->_new(-parent           => $self,
                                                    -name             => $name,
                                                    -state            => $state,
                                                    -ip               => $ipv4,
                                                    -ipv4             => $ipv4,
                                                    -ipv4_netmask     => $ipv4_netmask,
                                                    -ipv6             => $ipv6,
                                                    -ipv6_prefix_len  => $ipv6_prefix_len,
                                                    -tpgt             => $tpgt,
                                                    -tpgn             => $tpgn,
                                                    -mac              => $mac,
                                                    -nodename         => $nodename,
                                                    -is_mgmt_port     => $is_mgmt_port,
                                                    -is_vif           => $is_vif,
                                                    -vlan_tag         => $vlan_tag,
                                                    -vif_ints         => $vif_contrib_ints,
                                                    -vif_type         => $vif_type,
                                                    -vfiler           => $vfiler);
                    if ($targObj->error()) {
                        $self->_set_err($targObj->errmsg());
                        return $ClientUtil::FALSE;
                    }

                    push(@{$self->{iscsi_targets}}, $targObj);

                    $self->sanlog(-msg_level => 7,
                                  -msg => "ISCSI Target created" .
                                          "\nname = $name"       .
                                          "\nipv4 = $ipv4"       .
                                          "\nip netmask = $ipv4_netmask" .
                                          "\nipv6 = $ipv6"       .
                                          "\nipv6_prefix_len = $ipv6_prefix_len" .
                                          "\ntpgt = $tpgt"       .
                                          "\nmac = $mac"         .
                                          "\nnodename = $nodename" .
                                          "\nvfiler = $vfiler"   );
                } # end unless $ifconfig_line
            } # end foreach my $line @if_output
        } # end if state = enable/disabled
    } # end foreach my $line iscsi interface show

    return $ClientUtil::TRUE;

}

###############################################################################
# Method:    _load_iscsi_alias()
# Objective: Return important information about filer iscsi alias
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _load_iscsi_alias {
    my ($self) = @_;
    my $cmd_obj = $self->cmd_obj();

    $self->_clear_err();

    my ($cmd, $lines, $resultsHash_ref, $alias);
    my @output;
    my %cmd_args = (
                     "-cmd" => "iscsi-target-alias-get-alias",
                     "-cmd_filer" => $self->get_name(),
                     "-check_result" => 1,
                     "-check_output" => 0,
                     "-cmd_interface" => $self->interface(),
                   );

    #
    # execute the command
    #
    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    $self->log_cmd_results($resultsHash_ref);
    if ($cmd_obj->error()) {
        $self->_set_err("ERROR: SanFiler::_load_iscsi_alias(): " .
                        "Failed calling execute_cmd", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Error: SanFiler::_load_iscsi_alias(): " .
                        "Unable to get iscsi alias ", $output[0]);
        return $ClientUtil::FALSE;
    }


    #
    # Check to make sure the iscsi alias value was returned
    #
    $lines = $resultsHash_ref->{output};
    @output = split("\n", $lines);
    my $cmd_interface = $self->interface();  # default to global value
    if ($cmd_interface =~ /cli/) {
        foreach my $line (@output) {
            if ($line =~ /iscsi target alias:/i) {
                (undef, undef, undef, $alias) = split(" ", $line);
                $self->{iscsi_alias} = $alias;
                return $ClientUtil::TRUE;
            } elsif ($line =~ /no iscsi target alias assigned/i) {
                $self->{iscsi_alias} = undef;
                return $ClientUtil::TRUE;
            }
        }
        $self->_set_err("Error: SanFiler::_load_iscsi_alias(): " .
                     "Unable to get iscsi alias", $output[0]);
        return $ClientUtil::FALSE;
    } else {
        my $hash = $self->xml2hash($lines);
        my $is_assigned = $hash->{"is-alias-assigned"};

        if ($is_assigned =~ /true/i) {
            $self->{iscsi_alias} = $hash->{"alias-name"};
        } else {
            $self->{iscsi_alias} = undef;
        }
    }
    return $ClientUtil::TRUE;
}


###############################################################################
# Method:    _trim
# Objective: remove leading/triling whitespace
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _trim
{
    my @out = @_;
    for (@out) {
        s/^\s+//;
        s/\s+$//;
    }
    return wantarray ? @out : $out[0];
}

###############################################################################
# Method:    _convert_hash_to_array_of_hash
# Objective: Convert a hash into an array of hashes
###############################################################################
sub _convert_hash_to_array_of_hash
{
    my ($self, $in_hash) = @_;
    my $out_arr_ref;
    my @arr;
    my %new_hash;

    #Copy the hash pointed by the reference to a new hash
    %new_hash = %{$in_hash};
    my $dumpOut = Dumper(%new_hash);
print "hash from iptpgroup show:\n$dumpOut\n";
    $self->sanlog(-msg       => "hash from iptpgroup show:\n$dumpOut",
                  -msg_level => 7);

    #Add the hash to an array
    push (@arr,\%new_hash);

    #Store the array reference to a new variable
    $out_arr_ref = \@arr;
    $dumpOut = Dumper($out_arr_ref);
print "hash from hash_to_array :\n$dumpOut\n";
    $self->sanlog(-msg       => "hash from hash_to_array:\n$dumpOut",
                  -msg_level => 7);

    #Return back the new array reference
    return $out_arr_ref;
}

###############################################################################
# Method:    _iscsi_iptpgroup_show_cli_to_hash
# Objective: Convert the iscsi iptpgroup show CLI output to a ZAPI-compatible
#            hash
###############################################################################
sub _iscsi_iptpgroup_show_cli_to_hash
{
    my ($self, $output) = @_;

    my (@out);
    my ($output_hash);

    # Split the output lines into an array
    @out = split('\n', $output);

    # iscsi-iptpgroup-list-entry-info array index
    my $info_array_index = 0;

    # Walk through each line of output array
    foreach my $line (@out) {

        # This takes multiple lines of:
        # TPGTag Name and Member IP Addresses (separated by commas),
        # all three of the above separated by white space.
        # Sample Output of iscsi ip_tpgroup show:
        # TPGTag  Name                        Member IP Addresses
        # 1002   e0d_default                 10.225.194.236
        # 4000   10.225.194.238_default      10.225.194.238

        # Store the tags, names and IPs in temporary variables
        my ($tag,  $name);
        my (@temp, @ips);

        # break space separated attributes into a temp array
        @temp = split(' ', $line);

        # Store the tag from the temp array since it is the first attribute
        $tag = $temp[0];
        next if ($tag eq "TPGTag");    # skip header

        $name = $temp[1];
        for (my $j = 2 ; $j < scalar(@temp) ; $j++) {

            # Store IPs in the ip array
            push(@ips, $temp[$j]);
        }

        # Build the iptpgroup hash just as the ZAPI hash
        $output_hash->{"iscsi-iptpgroup-list-entries"}
          ->{"iscsi-iptpgroup-list-entry-info"}[$info_array_index]
          ->{"iptpgroup-tag"} = $tag;
        $output_hash->{"iscsi-iptpgroup-list-entries"}
          ->{"iscsi-iptpgroup-list-entry-info"}[$info_array_index]
          ->{"iptpgroup-name"} = $name;

        # ipaddress-list-entries array index
        my $ip_array_index = 0;

        # Add each IP into the proper hash format
        foreach my $ip (@ips) {

            # If IP address field is (none) then no need to add any IP
            # for the ip_tpgroup hash
            if ($ip !~ /none/i) {
                $ip =~ s/,\s*//g if $ip =~ /,/;  # Remove any commas from the IP
                $output_hash->{"iscsi-iptpgroup-list-entries"}
                  ->{"iscsi-iptpgroup-list-entry-info"}[$info_array_index]
                  ->{"ipaddress-list-entries"}[$ip_array_index]
                  ->{"ipaddress-list-entry-info"}->{"ip-address"} = $ip;
                $ip_array_index++;  # Increment only when an IP is added to hash
            }
        }    # end foreach ip

        $info_array_index++;    # Increment after every iptpgroup_name is added
    }    # end foreach line

    my $dumpOut = Dumper($output_hash);
    $self->sanlog(-msg       => "hash from cli_to_hash is:\n$dumpOut",
                  -msg_level => 7);

    return $output_hash;
}    # end _iscsi_iptpgroup_show_cli_to_hash

###############################################################################
# Method:    _get_iscsi_iptpgroup_tag
# Objective: Get the iSCSI IP TPGroup Tag ID given the name of iptpgroup
###############################################################################
sub _get_iscsi_iptpgroup_tag
{
    my ($self, %args) = @_;
    my ($vfiler_name, $iptpgroup_name);
    my ($output_hash);

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::_get_iscsi_iptpgroup_tag()",
                  -msg_level => 1);

    # Parse arguments
    foreach (keys %args) {
        if (/-?vfiler_name?/) {
            $vfiler_name = $args{$_};
            if (not length($vfiler_name)) {
                $self->_set_err("Invalid value for argument '-vfiler_name'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?iptpgroup_name?/) {
            $iptpgroup_name = $args{$_};
            if (not length($iptpgroup_name)) {
                $self->_set_err("Invalid value for argument '-iptpgroup_name'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # Call iscsi iptpgroup show
    $output_hash = $self->iscsi_iptpgroup_show(-vfiler_name => $vfiler_name);
    if ($self->error || not $output_hash) {
        $self->_set_err("iscsi_iptpgroup_show failed", $self->errmsg());
        return $ClientUtil::FALSE;
    }

    # Go through the Hash and find the entry for the given iptpgroup name
    # Return the iptpgroup tag associated with the iptpgroup name
    foreach my $hash (@{  $output_hash->{"iscsi-iptpgroup-list-entries"}
                            ->{"iscsi-iptpgroup-list-entry-info"}}
      ) {
        if ($hash->{"iptpgroup-name"} eq $iptpgroup_name) {
            return $hash->{"iptpgroup-tag"};
        }
    }    # end foreach $hash

    # If not found, return an empty string
    $self->_set_err("IPTPGroup Tag for \"$iptpgroup_name\" not found");
    return "";
} ## end sub _get_iscsi_iptpgroup_tag

################################################################################
# Method:    _isns_show_cli_to_hash
# Objective: Convert iscsi isns show CLI output into hash format.
# Details:
#     Inputs  -cli_output - required - $output from isns_show library call
#     Returns $isns_hash  - hash in the same format as isns show ZAPI cmd
#             $ClientUtil::FALSE - no cli_output passed in or error(and set_err)
################################################################################
sub _isns_show_cli_to_hash {
    my ($self, %args) = @_;

    my $cli_output;    # arg - required - $output from radius_show library call
    my $isns_hash;

    $self->_clear_err();

    $self->sanlog(-msg       => "SanFiler::_isns_show_cli_to_hash()",
                  -msg_level => 3);

    # Parse arguments
    foreach (keys %args) {
        if (/-?cli_output/) {
            $cli_output = $args{$_};
            if (not length($cli_output)) {
                $self->_set_err("Invalid value for argument '-cli_output'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $cli_output) {
        $self->_set_err("Missing required argument '-cli_output'");
        return $ClientUtil::FALSE;
    }

    $self->sanlog(-msg => "_isns_show_cli_to_hash: cli_output=\n$cli_output",
                  -msg_level => 7);

    # Expected CLI output to parse:
    # rtpqa-fas3050-15> iscsi isns show
    # iSNS Entity id:          isns:00000007
    # iSNS Server ip-addr:     10.225.113.127
    # iSNS Status:             Enabled

    # rtpqa-fas3050-16> iscsi isns show
    # iSNS Entity id:          NOT CONFIGURED
    # iSNS Server ip-addr:     NOT CONFIGURED
    # iSNS Status:             Disabled

    # Split the CLI output into individual lines
    my @output_array = split(/\n/, $cli_output);

    # If the output array is not 3 lines, then something is wrong, abort.
    if (scalar(@output_array) != 3) {
        $self->_set_err("iscsi isns show output is not 3 lines!");
        return $ClientUtil::FALSE;
    }

    # Read the first line and save the entity id
    if ($output_array[0] !~ /^iSNS Entity id:\s+(.+)$/) {
        $self->_set_err("Could not parse entity id: $output_array[0]");
        return $ClientUtil::FALSE;
    }
    $isns_hash->{"isns-entity-id"} = $1;

    # Normalize CLI output to match ZAPI.  Change NOT CONFIGURED to blank
    if ($isns_hash->{"isns-entity-id"} eq "NOT CONFIGURED") {
        $isns_hash->{"isns-entity-id"} = {};
    }

    # Read the 2nd line and save the ip address
    if ($output_array[1] !~ /^iSNS Server ip-addr:\s+(.+)$/) {
        $self->_set_err("Could not parse ip address: $output_array[1]");
        return $ClientUtil::FALSE;
    }
    $isns_hash->{"isns-ip-addr"} = $1;

    # Normalize CLI output to match ZAPI.  Change NOT CONFIGURED to blank
    if ($isns_hash->{"isns-ip-addr"} eq "NOT CONFIGURED") {
        $isns_hash->{"isns-ip-addr"} = {};
    }

    # Read the 3rd line and save the status
    if ($output_array[2] !~ /^iSNS Status:\s+(.+)$/) {
        $self->_set_err("Could not parse status: $output_array[2]");
        return $ClientUtil::FALSE;
    }
    $isns_hash->{"is-isns-enabled"} = $1;

    # Normalize CLI output to match ZAPI terms.  Return error if we don't find
    # Enabled or Disabled
    if ($isns_hash->{"is-isns-enabled"} eq "Enabled") {
        $isns_hash->{"is-isns-enabled"} = "true";
    } elsif ($isns_hash->{"is-isns-enabled"} eq "Disabled") {
        $isns_hash->{"is-isns-enabled"} = "false";
    } else {
        $self->_set_err("Status was not Disabled or Enabled: $output_array[2]");
        return $ClientUtil::FALSE;
    }

    my $dumpOut = Dumper($isns_hash);
    $self->sanlog(-msg       => "isns hash from cli_to_hash is:\n$dumpOut",
                  -msg_level => 7);

    return $isns_hash;
}    # end _isns_show_cli_to_hash

###############################################################################
# Method:    _iscsi_portal_show_cli_to_hash
# Objective: Converts the iscsi portal show CLI output to a ZAPI-compatible
#            hash.
# Details:   See POD documentation at the end of this file
# Inputs     CLI output of "iscsi portal show" command - required
# Returns    Reference to "HASH", contains information about
#            "IP address", "TCP Port", "TPGroup", "Interface"
###############################################################################
sub _iscsi_portal_show_cli_to_hash {
    my ($self, $output) = @_;

    my (@out);
    my ($output_hash);

    # If the $output is not defined or equal to "" or equal to 0, set set_err and return {}
    unless ($output) {
        $self->_set_err("CLI output of iscsi_portal_show is not provided",
                        $self->errmsg());
        return {};
    }

    # Split the output lines into an array
    @out = split('\n', $output);

    # iscsi-portal-list-entry-info array index
    my $info_array_index = 0;

    # Walk through each line of output array
    foreach my $line (@out) {

        # This takes multiple lines of:
        # "IP address", "TCP Port", "TPGroup" and "Interface"
        # all four of the above are separated by white space.
        # Sample Output of iscsi portal show:
        #
        # Network portals:
        # IP address        TCP Port  TPGroup  Interface
        # 10.228.128.45        3260    1001    e0b

        # If there are no portal available the output will be as follow
        #
        # No available portals

        # Store the "IP address", "TCP Port", "TPGroup" and "Interface"
        # in temporary variables
        my ($IP_address, $TCP_Port, $TPGroup, $Interface);
        my (@temp);

        if ($line =~ m/No available portals/i) {
            $output_hash->{"iscsi-portal-list-entries"} = {};
            last;
        }

        # break space separated attributes into a temp array
        @temp = split(' ', $line);

        if (($temp[0] =~ m/Network/i) or ($temp[0] =~ m/IP/i)) {
            next;    # skip header lines
        } else {

            # Store the first attribute "IP address" from the temp array.
            $IP_address = $temp[0];

            # Store the second attribute "TCP Port" from the temp array.
            $TCP_Port = $temp[1];

            # Store the third attribute "TPGroup" from the temp array.
            $TPGroup = $temp[2];

            # Store the fourth attribute "Interface" from the temp array.
            $Interface = $temp[3];
        } ## end else [ if (($temp[0] =~ m/Network/i...

        # Build the portal hash just as the ZAPI hash

        $output_hash->{"iscsi-portal-list-entries"}
          ->{"iscsi-portal-list-entry-info"}[$info_array_index]
          ->{"ip-address"} = $IP_address;

        $output_hash->{"iscsi-portal-list-entries"}
          ->{"iscsi-portal-list-entry-info"}[$info_array_index]->{"ip-port"} =
          $TCP_Port;

        $output_hash->{"iscsi-portal-list-entries"}
          ->{"iscsi-portal-list-entry-info"}[$info_array_index]
          ->{"tpgroup-tag"} = $TPGroup;

        $output_hash->{"iscsi-portal-list-entries"}
          ->{"iscsi-portal-list-entry-info"}[$info_array_index]
          ->{"interface-name"} = $Interface;

        $info_array_index++;
    } ## end foreach my $line (@out)

    my $dumpOut = Dumper($output_hash);
    $self->sanlog(-msg       => "hash from cli_to_hash is:\n$dumpOut",
                  -msg_level => 7);

    return $output_hash;
} ## end sub _iscsi_portal_show_cli_to_hash

1;



#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

SanFiler.pm

=head1 Author

Gary Knopp, Tom Masterson, David Pressley

=head1 Description

SanFiler.pm provides an API interface to a filer through the
use of the Telnet library.

=head1 See Also

Package inherits from the following modules. See these modules for further
interface detail.

B<San::Sanlog, San::SanUtil, San::SanFiler::Cmds::Fcp, San::Filer::Cmds::Portset, CmdsDriver, Tharn, Error>

=head1 Attributes

=head2 B<name>

string = Filer Name

=head2 B<filers>

string = Filer Name

=head2 B<igroups>

array = List of Igroup objects

=head2 B<targets>

array = FCP Target objects

=head2 B<iscsi_targets>

array = iSCSI Target objects

=head2 B<iscsi_alias>

string = iSCSI Alias Name

=head2 B<connected>

boolean = <1 | 0>

=head2 B<debug_level>

integer = Current debug level (default: 0)

=head2 B<show_cmd>

boolean = <1 | 0> (default: 1 (true) By default log commands executed)

=head2 B<show_data>

boolean = <1 | 0>

=head2 B<debug>

boolean = <1 | 0>

=head2 B<execute>

boolean = <1 | 0> (default: 1)

=head2 B<log>

boolen = <1 | 0>

=head2 B<timeout>

integer = timeout to use for say calls (default: 60)

=head2 B<connection>

telnet object = <0 | telnet Object>

=head2 B<command_time>

integer = time in seconds of the last command executed

=head2 B<last_cmd>

string = Last command executed

=head2 B<default_ip>

string = IP address of a filer

=head2 B<Errmsgs>

array = error strings (See Error.pm)

=head2 B<interface>

string = <cli | zapi | zapi_cli> (default: cli)

=head2 B<filer_version>

string = version of filer for use with CmdsDriver (cmd_obj attribute) (default: 0)

=head1 Public Methods

=head2 B<new()>

Creates a new SanFiler object.  Uses the specified IP address or machine name
to create a telnet connection. If the connection is successful, the calling
program will be logged into the CLIENT using either the default user/password
specified in the NATE_HOSTS file or a user and password supplied by the caller.

=over 4

=item Inputs

=item * -filer

Name of filer

=item * I<-timeout> (default: Tharnhost telnet timeout entry).

Override the Telnet timeout value.

=item * I<-debug_level> (default: 0).

Verbosity level for Sanlog::sanlog()

=item * I<-show_cmd> <1 | 0> (default: 1 (true) ).

By default commands executed will be logged   This can be
turned off by passing the -show_cmd with a value of 0.

=item * I<-loghdl>

Filehandle to log output to.

=item * I<-connection> (default: "").

Connection to the filer.

=item * I<-conn_type> (default: "#rsh").

Connection type to use when connecting to the filer.
Ignored if -connection given.

=item * I<-cmd_interface> <cli | zapi | zapi_cli> (default: zapi).

Interface type to use when executing commands with CmdsDriver::execute_cmd()

=item * I<-filer_version> (default: "7.3").

Version of ONTap to use with CmdsDriver objects.  Used to reference data
files.  Examples of valid values include 7.2, 7.3, 0.

=item * I<-vfiler> (default: "").

Name of the current vfiler to use when executing commands with
CmdsDriver::execute_cmd().

=item * I<-vfiler_ip> (default: undefined).

IP address of the current vfiler to use when executing commands with
CmdsDriver::execute_cmd().

=item * I<-partner_mode> <yes | 1 | no | 0> (default: 0).

Set to execute commands in partner mode  when executing commands with
CmdsDriver::execute_cmd().

=item Outputs

Returns a SanFiler object.

=item Supported Interfaces

cli and zapi

=item Examples

 # Create a SanFiler object

 $Filer = new SanFiler(
                -filer => 'kipper',
                -timeout=>100);

 if ($Filer->error) {
     print $Filer->errmsg;
     return 0;
 }

=back

=head2 B<create_igroup(%args)>

Creates a new igroup object

=over 4

=item Inputs

=item * -name

Name of new igroup

=item * I<-clientObj>

A reference to a SanClient2 object.  This is only required if an array of
initiator objects are not passed in. By default an initiaor
object already has a reference to a SanClient2 object.  A SanClient2
object is queried for the protocol and system type to use
when creating the igroup.

Name of new igroup

=item * I<-initObjs>

An array reference of Initiator objects or a single initiator object.

=item Outputs

Returns a IGroup object.  If an error occured the error flag
is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

 # Retrieve list of client initiator objects
 @initObjs=$Client->get_initiators();
 if ($Client->error){
        $Test->nlog($Client->errmsg);
        return $TCase::FAIL;
 }

 # Create an IGroup object
 $igrpObj = $Filer->create_igroup(
                -name=>'my_igroup',
                -initObjs=>[@initObjs]);

 if ($Filer->error) {
     $Test->nlog($Filer->errmsg);
     return $TCase::FAIL;
 }

=back

=head2 B<destroy_igroup(%args)>

Destroy a filer igroup

=over 4

=item Inputs

=item * $name

The name of igroup to destroy.

=item * I<-force> (default: 0)

The option will destroy the igroup even if its currently mapped to luns.

=item Outputs

Returns a scalar value of 1 if successful.  If an error occured the error flag is set, errmsg
is set to the failure reason, and 0 is passed back as the return value.

=item Supported Interfaces

cli

=item Examples

    $Test->nlog("-- Destroy igroup");

    $Filer->destroy_igroup("my_igroup",-force=>1);
    if ($Filer->error()){
        $Test->nlog($Filer->errmsg());
        return $TCase::FAIL;
    }

=back

=head2 B<display_targets($clientObj)>

Display detailed target information for a client

=over 4

=item Inputs

=item * $clientObj

Client object for which to display target information

=item Outputs

Returns an array or array reference of target data if successful.  If an error occured
the error flag is set, errmsg is set to the failure reason, and an empty array or array
reference is passed back as the return value.

=item Supported Interfaces

cli

=item Examples

    $Test->log("-- Retrieving target information");

    $Test->log($Filer->display_targets($Client));
    if ($Filer->error){
        $Test->log($Filer->errmsg());
        return $TCase::FAIL;
    }


=back


=head2 B<error()>

This method allows you to determine if any errors occured in the
previous method call.  Note that the error flag is reset at the
beginning of each method, so it should be checked after B<every>
major method call.  If you do not check for errors, your calling
program may fail at some point and you will not know why.

=over 4

=item Inputs

None

=item Outputs

Returns 1 if an error occured in the previous call, 0 otherwise.

=item Supported Interfaces

All

=item Examples

 # Determine if error occurred...
 if ($Filer->error) {
    print "Error encountered in previous method call\n";
 }

=back

=head2 B<errmsg()>

This method allows you to get the text of the error message if an error
occurred on the previous method call.

=over 4

=item Inputs

None

=item Outputs

Returns the text of the last error message set.  If no error message is
set, an empty string will be returned.

=item Supported Interfaces

All

=item Examples

 # Check for errors.
 if ($Filer->error) {
    $errmsg = $Filer->errmsg;
    print "An error occurred: $errmsg\n";
 }

=back

=head2 B<execute($cmd,I<-hide>)>

Executes command on the filer and returns raw output.  Note:  The errorhandler for this
method is currently disabled since I do not know how to interpet the 'result' codes.

=over 4

=item Inputs

=item * $cmd

Command to execute on the host.

=item * I<-hide> default: 0

Hide the command from STDOUT

=item Outputs

Returns an array or an array reference to the data returned by executing the command.
If an error occurred, the error flag is set, errmsg is set to the failure reason, and
an empty array reference is passed back as the return value.

=item Supported Interfaces

cli

=item Examples

 #Display command output from host
 $buf=$Filer->execute("ls -l");
 if ($Filer->error){
    print $Filer->errmsg;
    return 0;
 }

 foreach $line (@$buf){
     print $line."\n";
 }

=back

=head2 B<get_cfmode()>

This method allows you to retrieve the Filer cfmode.

=over 4

=item Outputs

Returns a scalar value containing the cfmode. If the system is not clustered then the value
"Not Clustered" is returned.  If an error occured the error flag is set, errmsg is set to the
failure reason, and "" is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Get ontap version
 $cfmode= $Filer->get_cfmode();

 print "The filer cfmode is $cfmode.\n";

=back

=head2 B<set_cfmode($cfmode)>

Sets the CF mode on a filer.

=over 4

=item Inputs

=item $cfmode

What cfmode to use.

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and 0 is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

    # Set to single-image
    $filer_obj->set_cfmode("single_image");

=back

=head2 B<get_fcp_nodename()>

Gets the fcp nodename on a system

=over 4

=item Inputs

None

=item Outputs

Returns the nodename string if the command is successful. If an error occured
the error flag is set, errmsg is set to the failure reason, and 0 is passed
back as the return value.

=item Supported Interfaces

cli

=item Examples

    my $fcp_nodename = $filer_obj->get_fcp_nodename();

=back

=head2 B<set_fcp_nodename()>

Sets the fcp nodename on a system

=over 4

=item Inputs

nodename

=item Outputs

Returns the nodename string if the command is successful. If an error occured
the error flag is set, errmsg is set to the failure reason, and 0 is passed
back as the return value.

=item Supported Interfaces

cli

=item Examples

    $filer_obj->set_fcp_nodename("50:00:00:0a:11:22:33:44");
    if ($filer_obj->error){
       print $filer_obj->errmsg;
       return 0;
    }

=back

=head2 B<cluster_initialize()>

Initializes the system for use in a cluster.  Validates whether the system
has a partner ID: that is returned via sysconfig.  Licenses the cluster
service.  Performs a cf_enable, and performs a cf giveback if the node is
in takeover.

=over 4

=item Inputs

Nonde

=item Outputs

Returns TRUE if the command is successful. If an error occured
the error flag is set, errmsg is set to the failure reason, and 0 is passed
back as the return value.

=item Supported Interfaces

cli

=item Examples

    $filer_obj->cluster_initialize();
    if ($filer_obj->error){
       print $filer_obj->errmsg;
       return 0;
    }

=back

=head2 B<get_ipaddress()>

This method allows you to retrieve the ip address.

=over 4

=item Outputs

Returns a scalar value containing the ip address. If an error occured the error flag
is set, errmsg is set to the failure reason, and "" is passed back as the return value.

=item Supported Interfaces

All

=item Examples

 # Get ontap version
 $ipaddres= $Filer->get_ipaddress();

 print "The filer ip address is $ipaddress.\n";

=back

=head2 B<get_exec_time()>

This method returns the execution time in seconds of the last snap command statement executed.  Its the
actual time it took to process the statement and does not include any post processing of the data. The
timer used is a high resolution timer with millisecond accuracy.

=over 4

=item Outputs

Returns a scalar value representing the execution time in seconds.

=item Supported Interfaces

All

=item Examples

 #Retrieve execution time

 $time = $Filer->get_exec_time();
 print "Execution Time = $time seconds\n";

=back

=head2 B<get_fcp_initiators(I<%args>)>

Retrieve fcp initiators

=over 4

=item Input

None

=item Outputs

Returns an array or array reference of initiators if successful.  If an error occured the error flag
is set, errmsg is set to the failure reason, and empty array or array ref is passed back as the
return values.

=item Supported Interfaces

All

=item Examples

 # Display fcp initiators
 @inits= $Filer->get_fcp_initiators();
 if ($Filer->error){
     ...
 }

 $Test->log(@inits);

=back

=head2 B<get_igroup(%args)>

Retrieve an igroup object by name

=over 4

=item Inputs

=item * -name | -initObj

Specify either the igroup name or an array reference of initiator objects.

=item Outputs

Returns a igroup object if successful.  If the igroup does not exist or
an error occured the error flag is set, errmsg is set to the failure reason, and an
empty object reference is passed back as the return value.

=item Supported Interfaces

All

=item Examples

    $Test->log("-- Retrieving igroup object 'my_igroup'");
    $igrpObj=$Filer->get_igroup("my_igroup");
    if ($Filer->error){
                $Test->log($Filer->errmsg());
                return $TCase::FAIL;
    }

=back

=head2 B<get_igroups()>

Retrieve a list of all igroup objects on the filer

=over 4

=item Inputs

none

=item Outputs

Returns an array or array reference of igroup objects if successful.  If an error occurs the error flag
is set, errmsg is set to the failure reason, and an empty array is returned.

=item Supported Interfaces

All

=item Examples

    $Test->log("-- Retrieving igroup objects");
    @igrpObjs=$Filer->get_igroups();
    if ($Filer->error){
                $Test->log($Filer->errmsg());
                return $TCase::FAIL;
    }


=back

=head2 B<get_model()>

This method allows you to retrieve the Filer model.

=over 4

=item Outputs

Returns a scalar value containing the filer model.  If an error occured the error flag
is set, errmsg is set to the failure reason, and "" is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Get ontap version
 $model= $Filer->get_model();
 if ($Filer->error){
     ...
 }

 print "The filer model is $model.\n";

=back

=head2 B<get_serial_num()>

This method allows you to retrieve the system serial number.

=over 4

=item Outputs

Returns a scalar value containing the system serial number.  If an error occured the error flag
is set, errmsg is set to the failure reason, and "" is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Get ontap version
 $serial_num = $Filer->get_serial_num();
 if ($Filer->error){
     ...
 }

 print "The system serial number is $serial_num.\n";

=back

=head2 B<get_license_info()>

This method allows you to retrieve the license information for a given service.

=over 4

=item Outputs

Returns a scalar value containing the protocol license information.
If an error occured the error flag is set, errmsg is set to the
failure reason, and "" is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Get ontap version
 $license_info = $Filer->get_license_info("fcp");
 if ($Filer->error){
     ...
 }

 print "The license info for fcp is $license_info.\n";

=back

=head2 B<get_name()>

This method allows retrieve the hostname of ther filer

=over 4

=item Outputs

Returns a scalar value containing the filer name. If an error occured the error flag is set,
errmsg is set to the failure reason, and "" is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Filer name
 print "Filer name is ".$Filer->get_name()."\n";

=back

=head2 B<get_partner()>

This method allows you to retrieve the clustered partner name

=over 4

=item Outputs

Returns a scalar value containing the partner name. If the system is not clustered then the value
"None" is returned.  If an error occured the error flag is set, errmsg is set to the
failure reason, and "" is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Get ontap version
 $partner= $Filer->get_partner();

 print "The filer partner is $partner.\n";

=back

=head2 B<get_target($name)>

Retrieve a volume object by name

=over 4

=item Inputs

=item * $name

Name of filer target

=item Outputs

Returns a target object if successful.  If the target does not exist or
an error occured the error flag is set, errmsg is set to the failure reason, and an
empty object reference is passed back as the return value.

=item Supported Interfaces

All

=item Examples

    $Test->log("-- Retrieving target object '5a'");
    $targObj=$Filer->get_target("5a");
    if ($Filer->error){
                $Test->log($Filer->errmsg());
                return $TCase::FAIL;
    }

=back

=head2 B<get_targets(I<%args>)>

Retrieve a list of all igroup objects on the filer

=over 4

=item Inputs

=item * I<-dmp> (default: 1)

Determines whether all targets are returned if dynamic multipathing
is required

=item Outputs

Returns an array or array reference of target objects if successful.  If an error occurs the error flag
is set, errmsg is set to the failure reason, and an empty array is returned.

=item Supported Interfaces

cli

=item Examples

    $Test->log("-- Retrieving target objects");
    @targObjs=$Filer->get_targets();
    if ($Filer->error){
                $Test->log($Filer->errmsg());
                return $TCase::FAIL;
    }


=back

=head2 B<get_version()>

This method allows you to retrieve the Filer version.

=over 4

=item Outputs

Returns a scalar value containing the filer version.  If an error occured the error flag
is set, errmsg is set to the failure reason, and "" is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Get ontap version
 $version= $Filer->get_version();
 if ($Filer->error){
     ...
 }

 print "The filer version is $version.\n";

=back

=head2 B<get_version_major()>

This method allows you to retrieve the major revision of the Filer (i.e 7.0, 7.1, 8.0)

=over 4

=item Outputs

Returns a scalar value containing the filer major version.  If an error occured the error flag
is set, errmsg is set to the failure reason, and "" is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Get ontap version
 $version= $Filer->get_version_major();
 if ($Filer->error){
     ...
 }

 print "The filer major version is $version.\n";

=back

=head2 B<get_uptime()>

This method allows you to retrieve the Filer uptime information.

=over 4

=item Outputs

Returns an array containing the filer uptime output.  If an error occured the error flag
is set, errmsg is set to the failure reason, and "" is passed back as the return values.

This is a sample uptime output

rtpqa-fas3050-23> uptime
  5:54pm up  4 days,  4:40 0 NFS ops, 0 CIFS ops, 0 HTTP ops, 714829 FCP ops, 120592 iSCSI ops
rtpqa-fas3050-23>


=item Supported Interfaces

cli, zapi

=item Examples

 # Get ontap version
 @output= $Filer->get_uptime();
 if ($Filer->error){
     ...
 }

 foreach my $line (@output) {
     if ($line =~ /(\d+) FCP ops, (\d+) iSCSI ops/) {
         $UptimeHash{'FCP ops'}   = $1;
         $UptimeHash{'iSCSI ops'} = $2;
     }
 } # end foreach my $line (@output)

=back

=head2 B<get_vol($name)>

Retrieve a volume object by name

=over 4

=item Inputs

=item * $name

The name of the volume to retrieve

=item Outputs

Returns a volume object if successful.  If the volume does not exist or
an error occured the error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Supported Interfaces

All

=item Examples

    $Test->log("-- Retrieving vol object 'vol_A'");
    $volObj=$Filer->get_vol("vol_A");
    if ($Filer->error){
                $Test->log($Filer->errmsg());
                return $TCase::FAIL;
    }

=back

=head2 B<get_volnames($name)>

Retrieve a volume object by name

=over 4

=item Outputs

Returns an array or array ref if successful.  If an error occured the error flag
is set, errmsg is set to the failure reason, and an empty array ref i passed back
as the return values.

=item Supported Interfaces

cli

=item Examples

    # Get filer info
    $Test->log("Filer:",$Filer->info());
    if ($Filer->error){
        ...
    }

=back

=head2 B<connection(value)>

Get or Set the connection attribute.  If a value is given,
the value is set.  The value is always returned.
Call the corresponding method in the cmd_obj (CmdsDriver) to
pass the value down to the stored CmdsDriver object.

=over 4

=item Inputs

=item * value

Appropriate value for the connection attribute.  Connectrec returned from connect().

=item Outputs

Returns the attribute value.

=item Supported Interfaces

All

=item Examples

    my $filer_conn = connect ($FILER_CONN);
    $filer_obj->connection($filer_conn);

=back

=head2 B<interface(value)>

Get or Set the interface attribute.  If a value is given,
the value is set.  The value is always returned.
Call the corresponding method in the cmd_obj (CmdsDriver) to
pass the value down to the stored CmdsDriver object.

=over 4

=item Inputs

=item * value (cli | zapi | zapi_cli)

Appropriate value for the interface attribute, cli, zapi or zapi_cli.
The zapi_cli uses the system-cli zapi.

=item Outputs

Returns the attribute value.

=item Supported Interfaces

All

=item Examples

    # use zapi
    $filer_obj->interface("zapi");

=back

=head2 B<vfiler(value)>

Get or Set the vfiler context.  If a value is given,
the value is set.  The value is always returned.
Call the corresponding method in the cmd_obj (CmdsDriver) to
pass the value down to the stored CmdsDriver object.
Note that you can also use set_vfiler_context and
get_vfiler_context

=over 4

=item Inputs

=item * value

Appropriate value for a vfiler name.

=item Outputs

Returns the attribute value.

=item Supported Interfaces

All

=item Examples

    my $vfiler = $cmd_obj->vfiler("vfiler14");

=back

=head2 B<vfiler_ip(value)>

Get or Set the vfiler_ip attribute.  If a value is given,
the value is set.  The value is always returned.
Call the corresponding method in the cmd_obj (CmdsDriver) to
pass the value down to the stored CmdsDriver object.  Note
that you can set this using the set_vfiler_context method as
well

=over 4

=item Inputs

=item * value

Appropriate value for a vfiler IP.

=item Outputs

Returns the attribute value.

=item Supported Interfaces

All

=item Examples

    my $vfiler_ip = $cmd_obj->vfiler_ip("10.60.123.124");

=back

=head2 B<get_vfiler(%args)>

Given a vfiler name, return a Vfiler object for that vfiler.  if
no name is given, return an array of vfiler

=over 4

=item Inputs

=item * -name

The vfiler name

=item Outputs

Returns a Vfiler object.

=item Supported Interfaces

All

=item Examples

    my $vfiler_obj = $filer_obj->get_vfiler("vfiler1");

=back

=head2 B<set_vfiler_context(%args)>

Set the vfiler context for both the filer object and the
underlying cmd_obj (CmdsDriver)

=over 4

=item Inputs

=item * -name

The vfiler name

=item * -zapi_use_vfiler_ip

If set, use vfiler_ip to send ZAPIs. Default is to use vfiler_server_context to
run ZAPIs to vfilers

=item Outputs

Returns TRUE if successful, otherwize FALSE.  If an error occured the error flag
is set and errmsg is set to the failure reason.

=item Supported Interfaces

All

=item Examples

    # Set context to vfiler vfiler1
    $filer_obj->set_vfiler_context(-name => "vfiler1");

    # Set context to vfiler vfiler1 and enable ZAPIs using the Vfiler IP
    $filer_obj->set_vfiler_context(-name   => "vfiler1",
                                   -zapi_use_vfiler_ip => 1);

=back

=head2 B<clear_vfiler_context(%args)>

Clears the vfiler context for both the filer object and the
underlying cmd_obj (CmdsDriver)

=over 4

=item Inputs

=item * None

=item Outputs

Returns TRUE if successful

=item Supported Interfaces

All

=item Examples

    $filer_obj->clear_vfiler_context();

=back

=head2 B<get_vfiler_context(%args)>

Retrieve the current vfiler context for the object

=over 4

=item Inputs

None

=item Outputs

Returns the vfiler name if successful, otherwize FALSE.
If an error occured the error flag
is set and errmsg is set to the failure reason.

=item Supported Interfaces

All

=item Examples

    my $current_context = $filer_obj->get_vfiler_context();

=back

=head2 B<partner_mode(value)>

Get or Set the partner_mode attribute.  If a value is given,
the value is set.  The value is always returned.
Call the corresponding method in the cmd_obj (CmdsDriver) to
pass the value down to the stored CmdsDriver object.

=over 4

=item Inputs

=item * value (0 | 1)

Appropriate value for a partner_mode attribute, 0 or 1.

=item Outputs

Returns the attribute value.

=item Supported Interfaces

All

=item Examples

    # takeover and use partner mode
    $filer_obj->takeover();
    $filer_obj->partner_mode(1);

=back

=head2 B<cmd_obj(value)>

Get the CmdsDriver object.

=over 4

=item Outputs

Returns the cmd_obj attribute, a CmdsDriver object.

=item Supported Interfaces

All

=item Examples

    my $cmdsDriver_Obj = $filer_obj->cmd_obj();

=back

=head2 B<info()>

This method displays important information about the filer

=over 4

=item Outputs

Returns an array or array ref if successful.  If an error occured the error flag
is set, errmsg is set to the failure reason, and an empty array ref i passed back
as the return values.

=item Supported Interfaces

All

=item Examples

 # Get filer info
 $Test->log("Filer:",$Filer->info());
 if ($Filer->error){
     ...
 }

=back

=head2 B<giveback()>

Performs a filer giveback

=over 4

=item Outputs

Returns a scalar value of 1 if successful, otherwize 0.  If an error occured the error flag
is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

 # Giveback
 sleep(180) if $Filer->giveback();
 if ($Filer->error){
     ...
 }

=back

=head2 B<iscsi_start()>

Start up iscsi service

=over 4

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error flag
is set, errmsg is set to the failure reason, and 0 is passed back as the return values.

=item Supported Interfaces

cli and zapi

=item Examples

 # Start iscsi service
 if (not $Filer->iscsi_start()){
        print $Filer->errmsg()."\n";
 }


=back

=head2 B<iscsi_stop()>

Stop iscsi service

=over 4

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error flag
is set, errmsg is set to the failure reason, and 0 is passed back as the return values.

=item Supported Interfaces

cli and zapi

=item Examples

 # Stop iscsi service
 if (not $Filer->iscsi_stop()){
        print $Filer->errmsg()."\n";
 }


=back

=head2 B<iscsi_interface_enable($interface)>

Enable iSCSI service on ethernet interface

=over 4

=item Inputs

=item * $interface

The ethernet interface identifier

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error flag
is set, errmsg is set to the failure reason, and 0 is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Enable iscsi interface
 if (not $Filer->iscsi_interface_enable("e0c")){
        print $Filer->errmsg()."\n";
 }

=back

=head2 B<iscsi_interface_disable($interface)>

Disable iSCSI service on ethernet interface

=over 4

=item Inputs

=item * $interface

The ethernet interface identifier

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error flag
is set, errmsg is set to the failure reason, and 0 is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Disable iscsi interface
 if (not $Filer->iscsi_interface_disable("e2a")){
        print $Filer->errmsg()."\n";
 }

=back

=head2 B<iscsi_interface_show(%args)>

Display information about one or more iscsi interfaces

=over 4

=item Inputs

=item * $iscsi_interface

The ethernet interface identifier

=item Outputs

Returns status of interface if successful. If an error occured the error flag
is set, errmsg is set to the failure reason.

=item Supported Interfaces

zapi_cli

=item Examples

 # Show iscsi interface
 $self->iscsi_interface_show(-iscsi_interface => "e0b");

=back

=head2 B<iscsi_tpgroup_create($self, $name, $interfaces, $tpgtag)>

Creates a tpgroup with the provided information on the filer.

=over 4

=item Inputs

=item $name

Name of the tpgroup

=item $interface

Space seperated listing of interfaces to include in the tpgroup

=item $tpgtag

OPTIONAL: Optionally sets the tpgtag to the provided value, if not
supplied filer will pick.

=item Outputs

The tpgroup tag number.

=item Supported Interfaces

cli

=item Examples

    # Using all parameters
    $filer_obj->iscsi_tpgroup_create("test", "e0a e0b", 1);

=back

=head2 B<iscsi_tpgroup_show($self)>

Displays the tpgroup information on the filer.

=over 4

=item Outputs

The output from the filer regarding iscsi tpgroup show.

=item Supported Interfaces

cli

=item Examples

    # example
    @output = $filer_obj->iscsi_tpgroup_show();

=back

=head2 B<iscsi_tpgroup_exist($self, $name)>

Returns true or false if the provided tpgroup exists or not.

=over 4

=item Inputs

=item $name

The name of the tpgroup to check

=item Outputs

True- Exists, False- Doesn't exist

=item Supported Interfaces

cli

=item Examples

    # example
    $output = $filer_obj->iscsi_tpgroup_exist("testgroup");

=back

=head2 B<iscsi_tpgroup_destroy($self, $name)>

Destroys a tpgroup

=over 4

=item Inputs

=item $name

The name of the tpgroup to destroy

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

    # example
    $filer_obj->iscsi_tpgroup_destroy("testgroup");

=back

=head2 B<iscsi_tpgroup_interface_add($self, $name, $interface)>

Adds a network interface to an existing tpgroup name on a filer

=over 4

=item Inputs

=item $name

Required: Name of the tpgroup

=item $interface

Required: interface to add to the tpgroup

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

zapi_cli

=item Examples

    # Using all parameters
    $filer_obj->iscsi_tpgroup_interface_add("test_tpg", "e0a");
    if ($filer_obj->error()) {
        $Test->nlog ("Cannot add interface e0a to test_tpg",
            $filer_obj->errmsg());
    }

=back

=head2 B<iscsi_tpgroup_interface_remove($self, $name, $interface)>

Removes a network interface from existing tpgroup name on a filer

=over 4

=item Inputs

=item $name

Required: Name of the tpgroup

=item $interface

Required: interface to add to the tpgroup

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

zapi_cli

=item Examples

    # Using all parameters
    $filer_obj->iscsi_tpgroup_interface_remove("test_tpg", "e0a");
    if ($filer_obj->error()) {
        $Test->nlog ("Cannot remove interface e0a from test_tpg",
            $filer_obj->errmsg());
    }

=back

=head2 B<iscsi_interface_accesslist_add($self, $iqn, $interface)>

Adds an accesslist

=over 4

=item Inputs

=item $iqn

The iqn of the initiator.

=item $interface

An interface for the accesslist.

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi_cli

=item Examples

    # example
    $filer_obj->iscsi_interface_accesslist_create("iqn.abcdefg.com:9325", "e0a, e0b");

=back

=head2 B<iscsi_interface_accesslist_exist($self, $iqn)>

Returns true or false if the provided iqn accesslist exists or not.

=over 4

=item Inputs

=item $iqn

The iqn to check for accesslists

=item Outputs

True- Exists, False- Doesn't exist

=item Supported Interfaces

cli and zapi_cli

=item Examples

    # example
    $output = $filer_obj->iscsi_interface_accesslist_exist("iqn.abcdefg");

=back

=head2 B<iscsi_interface_accesslist_remove($self, $iqn, $interface)>

Removes an accesslist

=over 4

=item Inputs

=item $iqn

The iqn of the accesslist to remove

=item $interface

OPTIONAL: The interface to be removed from the accesslist

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi_cli

=item Examples

    # example
    $filer_obj->iscsi_interface_accesslist_destroy("iqn.abcdefg");

    # example with interface to be removed
    $filer_obj->iscsi_interface_accesslist_destroy("iqn.abcdefg", "e0b");

=back

=head2 B<iscsi_security_add_chap($self, $initiator, $inusername, $inpassword, $outusername, $outpassword)>

Configures chap for an initiator.

=over 4

=item Inputs

=item $initiator

The iqn of the initiator.

=item $inusername

The incoming username.

=item $inpassword

The incoming secret/password.

=item $outusername

OPTIONAL: The outgoing username.

=item $outpassword

OPTIONAL: The outgoing secret/password.

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

    # Simple Chap
    $filer_obj->iscsi_security_add_chap("iqn.abcdefg.com:9325", "user", "password");

=back

=head2 B<iscsi_security_set(%args)>

Configure iSCSI security for either 'default' or a specific initiator. You must
specify either default=1 or a specific initiator.  (You cannot specify both.)

  WARNING!!!!!!!
  If you wish to set passwords using ZAPIs, PLEASE READ:
  You CANNOT use plain ASCII text to transmit a password (inbound or
  outbound) via ZAPI.  You must first translate your ASCII password
  into an encrypted format.  To do this:
  Telnet to a linux system. (telnet osiris)
  cd /u/hual/test
  ./pwd_gen
  osiris test > ./pwd_gen -p netapp
  ad55f657dd12
  This is the password that you can send via ZAPI.  Your login
  password from the initiator is 'netapp'.

  Common passwords and translations:
  netapp        ad55f657dd12
  netappnetapp  ad55f657dd12684482254d1d
  netapp1       ad55f657dd1237
  password      b351f145da0d7445

=item Inputs

=item * -default

(Optional) 1 = set security default.  0 = set for single initiator

=item * -initiator

(Optional) iqn of a specific initiator to set security for.

=item * -auth_type

(Required) Must be: chap, deny, none, or radius

=item * -user_name

(Required for chap only) inbound user name.

=item * -password

(Required for chap only) inbound password.

=item * -out_user_name

(Optional) outbound user name.

=item * -out_password

(Optional) outbound password.

=item Outputs

On success, returns: $ClientUtil::TRUE.
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

    # set default security to chap with  user name / password
    $Filer_Obj->iscsi_security_set(-default     => 1,
                                   -auth_type   => "chap",
                                   -user_name   => "mike",
                                   -password    => "netapp");

    # set single initiator to deny
    $Filer_Obj->iscsi_security_set(
                     -initiator => "iqn.1991-05.com.microsoft:w2k8std64-master",
                     -auth_type => "deny");

    # set default to radius
    $Filer_Obj->iscsi_security_set(-default     => 1,
                                   -auth_type   => "radius");

=back

=head2 B<iscsi_security_delete(%args)>

Delete iSCSI security settings for one or more intiators.

=over 4

=item Inputs

=item * -initiator

(Required) iqn of a specific initiator to set security for.  For a single
initiator, you may use -initiator => "iqn.1994-05.com.redhat:1e362f8622ba".
For multiple initiators, you may use
-initiator => [ "iqn.1994-05.com.redhat:1e362f8622ba",
"iqn.1994-05.com.redhat:ffffffffffff", "iqn.1994-05.com.redhat:eeeeeeeeeeee"]

=item Outputs

On success, returns: $ClientUtil::TRUE.
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

    # Delete a single initiator
    $Filer_Obj->iscsi_security_delete(-initiator => "iqn.1991-05.com.microsoft:mike1");

    # Delete multiple initiators
    $Filer_Obj->iscsi_security_delete(-initiator => [
                                            "iqn.1991-05.com.microsoft:mike2",
                                            "iqn.1991-05.com.microsoft:mike3",
                                            "iqn.1991-05.com.microsoft:mike4"]);

=back

=head2 B<iscsi_security_generate()>

Generate a random iSCSI password.

=over 4

=item Inputs

=item * None

=item Outputs

On success, returns a string which contains the random password.
On failure, returns undef.

=item Supported Interfaces

cli, zapi

=item Examples

        my $password = $Filer_Obj->iscsi_security_generate();

=back

=head2 B<iscsi_iptpgroup_show(%args)>

Displays iscsi IP based tpgroup information and returns a hash.

=over 4

=item Inputs

=item * -vfiler_name

(Required) Name of Vfiler whose IP based tpgroups will be displayed

=item Outputs

For successful show, returns: pointer to a iptpgroup_hash.
If the show fails, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   # Display IP based tpgroups
   $iptpgroup_hash = $filer_obj->iscsi_iptpgroup_show(-vfiler_name => "vfi1");

=back

=head2 B<iscsi_iptpgroup_create(%args)>

Creates an iptpgroup with the provided information on the vfiler.

=over 4

=item Inputs

=item * -iptpgroup_name

(Required) Name of IP based tpgroup to create.

=item * -vfiler_name

(Required) Name of Vfiler where the IP based tpgroup will be created

=item * -iptpgroup-tag

(Optional) Provide a tpgroup tag ID between the range of 1 to 256 for the IP
based tpgroup to be created

=item * -ip_addresses

(Optional) Provide a space separated list of IP Addresses to be added to the
IP based tpgroup during creation.
For ZAPI, this task consists of multiple ZAPI calls to add the IPs after the
IP based tpgroup is created.

=item * -force

(Optional) Default value is '0'. Useful for CLI only.
'1' = Use the -f flag to force the creation of IP based tpgroup
'0' = Do not use the -f flag.

=item Outputs

For successful create, returns: $ClientUtil::TRUE.
If the create fails, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   # Create IP based tpgroup with default tag and no IPs
   $filer_obj->iscsi_iptpgroup_create(-iptpgroup_name => "iptpg",
                                      -vfiler_name    => "vfiler1");

   # Create IP based tpgroup with default tag and 2 IPs
   $filer_obj->iscsi_iptpgroup_create(-iptpgroup_name => "iptpg",
                                      -ip_addresses   => "1.2.3.4 5.6.7.8",
                                      -vfiler_name    => "vfiler1");

   # Create IP based tpgroup with default tag, force flag and 2 IPs
   # No force flag for ZAPI and it will behave exactly as above
   $filer_obj->iscsi_iptpgroup_create(-iptpgroup_name => "iptpg",
                                      -ip_addresses   => "1.2.3.4 5.6.7.8",
                                      -force          => 1,
                                      -vfiler_name    => "vfiler1");

   # Create IP based tpgroup with tag 100 and no IPs
   $filer_obj->iscsi_iptpgroup_create(-iptpgroup_name => "iptpg",
                                      -iptpgroup_tag  => "100",
                                      -vfiler_name    => "vfiler1");

   # Create IP based tpgroup with tag 100 and 1 IP
   $filer_obj->iscsi_iptpgroup_create(-iptpgroup_name => "iptpg",
                                      -ip_addresses   => "1.2.3.4",
                                      -iptpgroup_tag  => "100",
                                      -vfiler_name    => "vfiler1");

   # Create IP based tpgroup with tag 100, force flag enabled and 1 IP
   # No force flag for ZAPI and it will behave exactly as above
   $filer_obj->iscsi_iptpgroup_create(-iptpgroup_name => "iptpg",
                                      -ip_addresses   => "1.2.3.4",
                                      -force          => 1,
                                      -iptpgroup_tag  => "100",
                                      -vfiler_name    => "vfiler1");

=back

=head2 B<iscsi_iptpgroup_destroy(%args)>

Destroys an iptpgroup from the vfiler

=over 4

=item Inputs

=item * -iptpgroup_name

(Required for CLI) Name of IP based tpgroup to destroy.
(Optional for ZAPI) Either iptpgroup-tag or iptpgroup-name should be provided.
If both are provided, iptpgroup-name will take precedence over iptpgroup-tag.

=item * -vfiler_name

(Required) Name of Vfiler where the IP based tpgroup will be destroyed

=item * -iptpgroup-tag

(Optional) ZAPI Only. Provide the tpgroup tag ID of the IP based tpgroup to be
destroyed

=item * -force

(Optional) Default value is '0'. Useful for CLI only.
'1' = Use the -f flag to force the destruction of IP based tpgroup
'0' = Do not use the -f flag.

=item Outputs

For successful destroy, returns: $ClientUtil::TRUE.
If the destroy fails, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   # Destroy IP based tpgroup with iptpgroup name
   $filer_obj->iscsi_iptpgroup_destroy(-iptpgroup_name => "iptpg",
                                      -vfiler_name    => "vfiler1");

   # Destroy IP based tpgroup with iptpgroup tag (ZAPI only)
   $filer_obj->iscsi_iptpgroup_destroy(-iptpgroup_tag => "100",
                                      -vfiler_name    => "vfiler1");

   # Destroy IP based tpgroup with iptpgroup name and iptpgroup tag
   # iptpgroup-tag will be ignored, in this case.
   $filer_obj->iscsi_iptpgroup_destroy(-iptpgroup_name => "iptpg",
                                      -iptpgroup_tag  => "100",
                                      -vfiler_name    => "vfiler1");

   # Destroy IP based tpgroup with iptpgroup name and force flag enabled
   $filer_obj->iscsi_iptpgroup_destroy(-iptpgroup_name => "iptpg",
                                      -force           => 1,
                                      -vfiler_name    => "vfiler1");

   # Destroy IP based tpgroup with iptpgroup tag and force flag enabled
   # ZAPI only case - force flag will be ignored
   $filer_obj->iscsi_iptpgroup_destroy(-iptpgroup_tag  => "100",
                                      -force           => 1,
                                      -vfiler_name    => "vfiler1");

=back

=head2 B<iscsi_iptpgroup_ipaddr_add(%args)>

Add an IP address to a IP based tpgroup in a vfiler

=over 4

=item Inputs

=item * -iptpgroup_name

(Required for CLI) Name of IP based tpgroup to ipaddr_add.
(Optional for ZAPI) Either iptpgroup-tag or iptpgroup-name should be provided.
If both are provided, iptpgroup-name will take precedence over iptpgroup-tag.

=item * -vfiler_name

(Required) Name of Vfiler where the IP address with be added to an iptpgroup

=item * -ip_address

(Required) IP address to be added to an iptpgroup. For CLI, a space separated
list of IPs can be added to the iptpgroup. This won't work for ZAPI.

=item * -iptpgroup_tag

(Optional) ZAPI Only. Provide the tpgroup tag ID of the IP based tpgroup to
which the IP address will be added

=item * -force

(Optional) Default value is '0'. Useful for CLI only.
'1' = Use the -f flag to force the addition of IP
'0' = Do not use the -f flag.

=item Outputs

For successful add, returns: $ClientUtil::TRUE.
If the add fails, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   # Add an IP address to IP based tpgroup with iptpgroup name
   $filer_obj->iscsi_iptpgroup_ipaddr_add(-iptpgroup_name => "iptpg",
                                          -ip_address     => "1.2.3.4",
                                          -vfiler_name    => "vfiler1");

   # Add an IP address to IP based tpgroup with iptpgroup tag (ZAPI only)
   $filer_obj->iscsi_iptpgroup_ipaddr_add(-iptpgroup_tag => "100",
                                          -ip_address     => "1.2.3.4",
                                          -vfiler_name    => "vfiler1");

   # Add an IP address to IP based tpgroup with iptpgroup name and iptpgroup tag
   # iptpgroup-tag will be ignored, in this case.
   $filer_obj->iscsi_iptpgroup_ipaddr_add(-iptpgroup_name => "iptpg",
                                          -iptpgroup_tag  => "100",
                                          -ip_address     => "1.2.3.4",
                                          -vfiler_name    => "vfiler1");

   # Add an IP address to IP based tpgroup with iptpgroup name and
   # force flag enabled
   $filer_obj->iscsi_iptpgroup_ipaddr_add(-iptpgroup_name => "iptpg",
                                          -ip_address     => "1.2.3.4",
                                          -force           => 1,
                                          -vfiler_name    => "vfiler1");

   # Add an IP address to IP based tpgroup with iptpgroup tag and
   # force flag enabled
   # ZAPI only case - force flag will be ignored
   $filer_obj->iscsi_iptpgroup_ipaddr_add(-iptpgroup_tag  => "100",
                                          -ip_address     => "1.2.3.4",
                                          -force           => 1,
                                          -vfiler_name    => "vfiler1");

=back

=head2 B<iscsi_iptpgroup_ipaddr_delete(%args)>

Delete an IP address from an IP-based tpgroup in a vfiler

=over 4

=item Inputs

=item * -iptpgroup_name

(Required for CLI) Name of IP based tpgroup to ipaddr_delete.
(Optional for ZAPI) Either iptpgroup-tag or iptpgroup-name should be provided.
If both are provided, iptpgroup-name will take precedence over iptpgroup-tag.

=item * -vfiler_name

(Required) Name of Vfiler where the IP address with be deleted from an iptpgroup

=item * -ip_address

(Required) IP address to be deleted from an iptpgroup. For CLI, a space
separated list of IPs can be deleted from the iptpgroup.
This won't work for ZAPI.

=item * -iptpgroup_tag

(Optional) ZAPI Only. Provide the tpgroup tag ID of the IP based tpgroup from
which the IP address will be deleted

=item * -force

(Optional) Default value is '0'. Useful for CLI only.
'1' = Use the -f flag to force the deletion of IP
'0' = Do not use the -f flag.

=item Outputs

For successful add, returns: $ClientUtil::TRUE.
If the add fails, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   # Delete an IP address from IP based tpgroup with iptpgroup name
   $filer_obj->iscsi_iptpgroup_ipaddr_delete(-iptpgroup_name => "iptpg",
                                             -ip_address     => "1.2.3.4",
                                             -vfiler_name    => "vfiler1");

   # Delete an IP address from IP based tpgroup with iptpgroup tag (ZAPI only)
   $filer_obj->iscsi_iptpgroup_ipaddr_delete(-iptpgroup_tag => "100",
                                          -ip_address     => "1.2.3.4",
                                          -vfiler_name    => "vfiler1");

   # Delete an IP address from IP based tpgroup with iptpgroup name
   # and iptpgroup tag iptpgroup-tag will be ignored, in this case.
   $filer_obj->iscsi_iptpgroup_ipaddr_delete(-iptpgroup_name => "iptpg",
                                          -iptpgroup_tag  => "100",
                                          -ip_address     => "1.2.3.4",
                                          -vfiler_name    => "vfiler1");

   # Delete an IP address from IP based tpgroup with iptpgroup name and
   # force flag enabled
   $filer_obj->iscsi_iptpgroup_ipaddr_delete(-iptpgroup_name => "iptpg",
                                          -ip_address     => "1.2.3.4",
                                          -force           => 1,
                                          -vfiler_name    => "vfiler1");

   # Delete an IP address from IP based tpgroup with iptpgroup tag and
   # force flag enabled
   # ZAPI only case - force flag will be ignored
   $filer_obj->iscsi_iptpgroup_ipaddr_delete(-iptpgroup_tag  => "100",
                                          -ip_address     => "1.2.3.4",
                                          -force           => 1,
                                          -vfiler_name    => "vfiler1");

=back

=head2 B<set_iscsi_nodename(%args)>

Set the iscsi nodename.

=over 4

=item Inputs

=item * -nodename

(Required) iSCSI nodename to set on the filer.

=item Outputs

On success, returns: $ClientUtil::TRUE.
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

       $Filer_Obj->set_iscsi_nodename(-nodename => "iqn.1992-08.com.netapp:chargers");

=back

=head2 B<is_giveback_complete()>

Determine if filer giveback is complete

=over 4

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error flag
is set, errmsg is set to the failure reason, and 0 is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Wait on giveback completion
 sleep(5) until ($Filer->is_giveback_complete());


=back

=head2 B<is_takeover_complete()>

Determine if filer takeover is complete

=over 4

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error flag
is set, errmsg is set to the failure reason, and 0 is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Wait on takeover completion
 sleep(5) until ($Filer->is_takeover_complete());


=back

=head2 B<port_online($port)>

Bring target hba port online

=over 4

=item Inputs

=item * $port

The port identifier

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error flag
is set, errmsg is set to the failure reason, and 0 is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Bring port online
 if (not $Filer->port_online("2a")){
        print $Filer->errmsg()."\n";
 }

=back

=head2 B<port_offline($port)>

Bring target hba port offline

=over 4

=item Inputs

=item * $port

The port identifier

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error flag
is set, errmsg is set to the failure reason, and 0 is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Bring port offline
 if (not $Filer->port_offline("2a")){
        print $Filer->errmsg()."\n";
 }

=back

=head2 B<takeover()>

Performs a filer takeover

=over 4

=item Outputs

Returns a scalar value of 1 if successful, otherwize 0.  If an error occured the error flag
is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

 # Giveback
 sleep(180) if $Filer->takeover();
 if ($Filer->error){
     ...
 }

=back

=head2 B<timeout($sec)>

This method allows you to change the Telnet timeout.  The default value
for timeout is the value specified in the Tharnhost file.

=over 4

=item Inputs

=item * $sec

Specify number of seconds for timeout value.

=item Outputs

If successful, the timeout in the Telnet object is set to the specified
value and the previously set value is returned.  If an error occurred,
the error flag is set, errmsg is set to the failure reason returned
from the Telnet module and 0 is passed back as the return value.

=item Supported Interfaces

All

=item Examples

 # Set Telnet timeout value to 45 seconds.
 $prev = $Filer->timeout(45);
 <check for errors>

=back

=head2 B<vol_status($vol)>

This method allows you to retrieve the volume status

=over 4

=item Inputs

=item * $vol
The volume name to use

=item Outputs

Returns an array or array reference if successful.  If an error occured the error flag
is set, errmsg is set to the failure reason, and empty array ref is passed back as the
return values.

=item Supported Interfaces

cli

=item Examples

 # Get ontap version
 @output= $Filer->vol_status("vol1");
 if ($Filer->error){
     ...
 }

 $Test->log(@output);

=back

=head2 B<options_cf_takeover_on_panic($pass_thru)>

Set cf takeover on panic off or on.

=over 4

=item Inputs

=item $pass_thru

String on or off arguments. Invalid argument -i (interval)

=item Outputs

=item TRUE or FALSE

Set cf takeover on panic off or on.
If an invalid argument is used error flag is set,
errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

    # cf takeover on panic off or on.
    $filer_obj->options_cf_takeover_on_panic("on");

=back

=head2 B<options_cf_takeover_on_failure($pass_thru)>

Set cf takeover on failure off or on.

=over 4

=item Inputs

=item $pass_thru

String on or off arguments. Invalid argument -i (interval)

=item Outputs

=item TRUE or FALSE

Set cf takeover on failure off or on.
If an invalid argument is used error flag is set,
errmsg is set to the failure reason.

=item Supported Interfaces

cli and zapi

=item Examples

    # cf takeover on failure off or on.
    $filer_obj->options_cf_takeover_on_failure("on");

=back

=head2 B<options_cf_giveback_auto_enable($pass_thru)>

Set filer cluster giveback auto enable off or on.

=over 4

=item Inputs

=item $pass_thru

String on or off arguments. Invalid argument -i (interval)

=item Outputs

=item TRUE or FALSE

Set filer cluster giveback auto enable off or on.
If an invalid argument is used error flag is set,
errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

    # cf giveback auto enable off or on.
    $filer_obj->options_cf_giveback_auto_enable("off");

=back

=head2 B<options_lun_enospc_timeout($timeout)>

Set lun enospc timeout to value given

=over 4

=item Inputs

=item $timeout

Timeout value between 1 and 360

=item Outputs

=item TRUE or FALSE

If an invalid timeout is used error flag is set.

=item Supported Interfaces

cli

=item Examples

    $filer_obj->options_lun_enospc_timeout(14);

=back

=head2 B<cf_disable($pass_thru)>

Disable filer cluster failover.

=over 4

=item Inputs

=item $pass_thru

String of cf disable arguments. Invalid argument -i (interval)

=item Outputs

=item TRUE or FALSE

Disable cluster failover. If an invalid argument is used error
flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

    # cf disable
    $filer_obj->cf_disable("");

=back

=head2 B<cf_enable($pass_thru)>

Enable filer cluster failover.

=over 4

=item Inputs

=item $pass_thru

String of cf enable arguments. Invalid argument -i (interval)

=item Outputs

=item TRUE or FALSE

Enable cluster failover. If an invalid argument is used error
flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

    # cf enable
    $filer_obj->cf_enable("");

=back

=head2 B<cf_status($pass_thru)>

Check filer cluster status.

=over 4

=item Inputs

=item $pass_thru

String of cf status arguments. Valid argument -t (Time master status)

=item Outputs

Returns an array or a scalar value containing the filer cluster status.
If an error occured the error flag is set, errmsg is set to the failure reason,
and "" is passed back as the return values.

=item Supported Interfaces

cli

=item Examples

 # Get filer cluster status
 $cfstatus= $Filer->cf_status();
 if ($Filer->error){
     ...
 }

 print "The filer cluster status is $cfstatus.\n";

=back

=head2 B<aggr_exist($name)>

This method allows you to determine if an aggregate exists.

=over 4

=item Inputs

=item * $name
The aggregate name to use

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and 0 is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

 # Check aggregate
 $aggr_exists = $Filer->aggr_exist("test_aggr");
 if ($Filer->error){
     ...
 }

 $Test->log(@output);

=back

=head2 B<aggr_create(%args)>

This method allows you to create an aggregate.  Note that this is a
very simple routine that simply takes and aggr name and an number of
disks.

=over 4

=item Inputs

=item * I<-name> (default: test_aggr)
The aggregate name to use

=item * I<-num_disks> (default: 3)
The number of disks in the aggregate

=item * I<-timeout> (default: 3600)
How long to wait for aggregate to come online

=item * I<-blocktype> (default: undef)
The blocktype of the aggregate, can be 32 or 64

=item * I<-raidtype> (default: raid_dp)
The raidtype of the aggregate, can be raid4 or raid_dp

=item * I<-raidsize> (default: default ONTAP raidsize)
The raidsize of the aggregate, can be between 2 and 28

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and 0 is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

 # Create aggregate
 $rc = $Filer->aggr_create(-name => test_aggr, -blocktype => 64, -num_disks => 5);
 if ($Filer->error){
     ...
 }

=back

=head2 B<get_aggr_volnames($name)>

This method returns an array of volume names that exist in the
specified aggregate "$name"

=over 4

=item Inputs

=item * $aggr_name
The aggregate name to use

=item Outputs

Returns an array of volume names if successful, returns an empty array if not.

=item Examples

 # Check aggregate
 @output = $Filer->get_aggr_volnames("test_aggr");
 if ($Filer->error){
     ...
 }

 $Test->log(@output);

=back

=head2 B<get_aggr_status(%args)>

This method allows you to get the status of an aggregate.

=over 4

=item Inputs

=item * I<-name> (default: test_aggr)
The aggregate name to use

=item Outputs

Returns an array of output gotten from the command: aggr status

=item Supported Interfaces

cli, zapi, and cli_zapi

=item Examples

 # Get aggregate status
 $rc = $Filer->get_aggr_status("test_aggr");
 if ($Filer->error){
     ...
 }

=back

=head2 B<wait_for_aggr_online(%args)>

This method continuously gets the aggregates status until it is
either online, or fails due to a timeout.

=over 4

=item Inputs

=item * I<-name> (default: test_aggr)
The aggregate name to use

=item * I<-timeout> (default: 3600)
How long to wait until the filer gives up on waiting for the aggregate
to come online.

=item * I<-interval> (default: 120)
How frequently the method will check the status of the aggregate.

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and 0 is passed back as
the return value.

=item Supported Interfaces

cli, zapi, and cli_zapi

=item Examples

 # Wait for aggregate online
 $rc = $Filer->wait_for_aggr_online("test_aggr");
 if ($Filer->error){
     ...
 }

=back

=head2 B<get_aggr_volnames($name)>

This method returns an array of volume names that exist in the
specified aggregate "$name"

=over 4

=item Inputs

=item * $aggr_name
The aggregate name to use

=item Outputs

Returns an array of volume names if successful, returns an empty array if not.

=item Supported Interfaces

cli

=item Examples

 # Check aggregate
 @output = $Filer->get_aggr_volnames("test_aggr");
 if ($Filer->error){
     ...
 }

 $Test->log(@output);

=back

=head2 B<aggr_offline($name)>

This method allows you to offline an aggregate.

=over 4

=item Inputs

=item name
The aggregate name to use

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and 0 is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

 # Offline aggregate
 $rc = $Filer->aggr_offline("test_aggr");
 if ($Filer->error){
     ...
 }

=back

=head2 B<aggr_destroy($name, $force)>

This method allows you to destroy an aggregate.

=over 4

=item Inputs

=item name

The aggregate name to use

=item Outputs

=item force <0|1>
Force the issue? (-f switch on aggr destroy)

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and 0 is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

 # Destroy aggregate
 $rc = $Filer->aggr_destroy("test_aggr");
 if ($Filer->error){
     ...
 }

=back

=head2 B<vol_exist($name)>

This method allows you to determine if a volume exists.

=over 4

=item Inputs

=item * $name
The volume name to use

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and 0 is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

 # Check volume
 $vol_exists = $Filer->vol_exist("test_vol");
 if ($Filer->error){
     ...
 }

=back

=head2 B<create_vol(%args)>

Creates a new volume object

=over 4

=item Inputs

=item * -name

Path to new volume.

=item * I<-aggr>

Name of the aggregate to use.  This is only required if you want to
create a flex vol.  Note that either -aggr or -disk_cnt must be
specified but not both.  Also if you use -aggr, you must use -size.

=item * I<-disk_cnt>

Disk Count, the number of disks to use if creating a trad vol.
This is only required if you want to create a trad vol.  Note that
either -disk_cnt or -aggr must be specified but not both.

=item * I<-size>

Size of the volume.  This is only required if you want to create a flex vol.
Note that if -aggr is specified -size must also be specified.

=item * I<-space-reserve>

Set's the space reservation policy for the volume to be created.
Values are 'none', 'file', or 'volume'. If no policy is set, the
controller's default will be used (no policy will be specified).

=item Outputs

Returns a Volume object.  If an error occured the error flag
is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

 # Create an Volume object
 $volObj = $Filer->create_vol(
                -name=>'test_vol',
                -aggr=>'test_aggr',
                -size=>'100g'
                );

 if ($Filer->error) {
     ...
 }

=back

=head2 B<iscsi_connection_list_info()>

lists all iscsi connection information

=over 4

=item Outputs

Returns a hash refernce which includes connection parameters.
If an error occured the error flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

zapi

=item Examples

 # get all iscsi connection information
 $connection_info_hash = $Filer->iscsi_connection_list_info();
 if ($Filer->error) {
     $Test->nlog($Filer->errmsg);
     return $TCase::FAIL;
 }

 =back

=head2 B<iscsi_session_list_info()>

lists all iscsi session information

=over 4

=item Inputs

=item * tsih

target session ID handle for specific session

=item Outputs

Returns a hash refernce which includes session parameters.
If an error occured the error flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

zapi

=item Examples

 # get all iscsi connection information
 $connection_info_hash = $Filer->iscsi_session_list_info(-tsih => $tsih);
 if ($Filer->error) {
     $Test->nlog($Filer->errmsg);
     return $TCase::FAIL;
 }

 =back

=head2 B<iscsi_initiator_list_info()>

lists all iscsi initiators logged in information

=over 4

=item Outputs

Returns a hash refernce which includes initiators logged in.
If an error occured the error flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

zapi

=item Examples

 # get all iscsi connection information
 $connection_info_hash = $Filer->iscsi_initiator_list_info();
 if ($Filer->error) {
     $Test->nlog($Filer->errmsg);
     return $TCase::FAIL;
 }

=back

=head2 B<panic()>

Panic the filer.

=over 4

=item Outputs

Returns a scalar value of 1.

=item Supported Interfaces

cli

=item Examples

    # Panic the filer
    $Filer->panic();

=back

=head2 B<destroy()>

Destroy client object (disconnect from client).

=over 4

=item Supported Interfaces

All

=item Examples

 # Get os version
 $Obj->destroy();

=back

=head2 B<igroup_exist($self, $name)>

Determine whether named igroup exists

=over 4

=item Inputs

=item $name

String of igroup name.

=item Outputs

=item TRUE or FALSE

Returns true if named igroup exists, false if named igroup does not exist. If an
error occurs, error flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

    if($san_filer_obj->igroup_exist($igroup_name)) {
        tstlog("$igroup_name does exist");
    }

=back

=head2 B<lun_exist($self, $name)>

Determine whether named lun exists

=over 4

=item Inputs

=item $name

String of lun name.

=item Outputs

=item TRUE or FALSE

Returns true if named LUN exists, false if named LUN does not exist. If an
error occurs, error flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

    if($san_filer_obj->lun_exist($lun_name)) {
        tstlog("$lun_name does exist");
    }

=back

=head2 B<lun_config_check($self, $option)>

Run the lun config_check utility

=over 4

=item Inputs

=item $option

Option to pass into the command

=item Outputs

=item @output or FALSE

Returns the output from the command if successful, false if not. If an
error occurs, error flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

   my @output = $SanFilerObj->lun_config_check();
   unless ("@output" =~ "No Problems Found") {
        $Test->nlog($SanFilerObj->errmsg());
   }

=back

=head2 B<lun_stats($self, $pass_thru)>

Gets filer lun statistics.

=over 4

=item Inputs

=item $pass_thru

String of lun stats arguments. Invalid argument -i (interval)

=item Outputs

=item @output or $output

Returns output from the lun stats command. If an invalid argument is used error
flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

    # lun stats -o
    if($stat = ($failover_filer_obj->lun_stats("-o")))
    {
        tstlog("-comment","$stat");
    }

=back

=head2 B<partner_lun_stats($self, $pass_thru)>

Gets filer partner lun statistics.

=over 4

=item Inputs

=item $pass_thru

String of partner lun stats arguments. Invalid argument -i (interval)

=item Outputs

=item @output or $str

Returns output from the lun stats command. If an invalid argument is used error
flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

    # partner lun stats
    $takeover_filer_obj->partner_lun_stats("");

=back

=head2 B<sysstat($self, $pass_thru)>

Gets performence statistics from filer.

=over 4

=item Inputs

=item $pass_thru

String of sysstat arguments. interval is an invalid argument.

=item Outputs

=item @output or $output

Returns output from the lun stats command. If an invalid argument is used error
flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

    # sysstat -c 1 -b
    if($stat = ($failover_filer_obj->sysstat("-c 1 -b")))
    {
        tstlog("-comment","$stat");
    }

=back

=head2 B<priv_available($self, $level)>

Execute priv set test to find if its allowed or not.

=over 4

=item Examples

    # is_priv_test_available exaple
    $filer_obj->priv_available("test");

=back

=head2 B<ic_reset_link($self)>

Execute ic link reset.

=over 4

=item Supported Interfaces

cli

=item Examples

    # ic link reset
    $filer_obj->ic_link_reset();

=back

=head2 B<ic_reset_NIC($self, $nic)>

Reset a given ic NIC.

=over 4

=item Inputs

=item $nic

Which nic to reset (0|1).

=item Supported Interfaces

cli

=item Examples

    # reset NIC
    $filer_obj->ic_reset_nic(0);

=back

=head2 B<ic_stats_error($self, $pass_thru)>

Call ic stats error to get ic stats.

=over 4

=item Inputs

=item $pass_thru

String to add to the end of ic stats error command line

=item Outputs

=item @output or $str

Returns output from the ic stats error command. If an invalid argument is used error
flag is set, errmsg is set to the failure reason.

=item Supported Interfaces

cli

=item Examples

    # ic stats
    $filer_obj->ic_stats_error("-v");

=head2 B<ic_disable_nic($self, $nic)>

Disable a given ic NIC.

=over 4

=item Inputs

=item $nic

Which nic to diable (0|1).

=item Supported Interfaces

cli

=item Examples

    # reset NIC
    $filer_obj->ic_disable_nic(0);

=back

=head2 B<ic_enable_nic($self, $nic)>

Enable a given ic NIC.

=over 4

=item Inputs

=item $nic

Which nic to enable (0|1).

=item Supported Interfaces

cli

=item Examples

    # reset NIC
    $filer_obj->ic_enable_nic(0);

=back

=head2 B<ic_status>

Get the status of the interconnect

=over 4

=item Outputs

=item @output or $str

Returns output from the ic status command.

=item Supported Interfaces

cli

=item Examples

    my $ic_stuats = $filer_obj->ic_status();

=back

=head2 B<reconnect>

Connect to the filer and store the connection

=over 4

=item Outputs

=item Connection Object

Returns the object returned from the call to connect.

=item Supported Interfaces

All

=item Examples

    $filer_obj->giveback();
    $filer_obj->reconnect();

=back

=head2 B<vfiler_create($self, $name, $ip_address, $paths)>

Creates a vfiler.

=over 4

=item Inputs

=item $name

Name of the vfiler.

=item $ip_address

The ip address to assign to the vfiler.

=item $paths

The paths to volumes for use by the vfiler.

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

    # Example
    $filer_obj->vfiler_create("vfiler_test", "192.168.1.152", "/vol/vol1");

=back

=head2 B<vfiler_setup($self, $name, $ip_configs_ref)>

Sets up a vfiler.

=over 4

=item Inputs

=item $name

Name of the vfiler.

=item $ip_configs_ref

A reference to the ip configuration hash to pass to CmdsDriver vfiler_setup
For Example:
        $ip_ref->{interface} = "e0d";
        $ip_ref->{ipaddress} = "10.10.10.1";
        $ip_ref->{netmask}   = "255.255.255.0";

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

    # Example

    $filer_obj->vfiler_setup("vfiler_test", $ip_ref);

=back

=head2 B<vfiler_exist($self, $name)>

Returns true or false if the provided vfiler exists or not.

=over 4

=item Inputs

=item $name

The name of the vfiler to check

=item Outputs

True- Exists, False- Doesn't exist

=item Supported Interfaces

cli and zapi

=item Examples

    # example
    $output = $filer_obj->vfiler_exist("vfiler1");

=back

=head2 B<vfiler_destroy($self, $name)>

Destroys a vfiler

=over 4

=item Inputs

=item $name

The name of the vfiler

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

    # example
    $filer_obj->vfiler_destroy("vfiler1");

=back

=head2 B<license_iscsi($self,$license)>

Licenses iSCSI on the filer using the default license or a provided license.

=over 4

=item Inputs

=item I<$license>

OPTIONAL: The license to be used, if not defined then license code will be
looked up in a license file. See SanUtil::get_avail_controller_licenses.

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

    # using default license
    $filer_obj->license_iscsi();

=back

=head2 B<license_fcp($self,$license)>

Licenses FCP on the filer using the default license or a provided license.

=over 4

=item Inputs

=item I<$license>

OPTIONAL: The license to be used, if not defined then license code will be
looked up in a license file. See SanUtil::get_avail_controller_licenses.

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

    # using default license
    $filer_obj->license_fcp();

=back

=head2 B<license_exist(-license => $code)>

Returns whether or not a filer service is licensed based on whether or not the
license code appears in the licensed services list on the filer.

=over 4

=item Inputs

$code

=item Outputs

Returns True if the service is licensed and FALSE the service is not licensed.
If an error occurs, errmsg is set to the failure reason.

=item Supported Interfaces

cli and zapi

=item Examples

    $filer_obj->license_exist(-code => "abcdefg");
    if ($filer_obj->error){
                $Test->log($filer_obj->errmsg());
                return $TCase::FAIL;
    }

=back

=head2 B<unlicense_fcp($self)>

Removes the FCP license on the filer.

=over 4

=item Inputs

None

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

    $filer_obj->unlicense_fcp();
    if ($filer_obj->error){
                $Test->log($filer_obj->errmsg());
                return $TCase::FAIL;
    }

=back

=head2 B<unlicense_iscsi($self)>

Removes the ISCSI license on the filer.

=over 4

=item Inputs

None

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

    $filer_obj->unlicense_iscsi();
    if ($filer_obj->error){
                $Test->log($filer_obj->errmsg());
                return $TCase::FAIL;
    }

=back

=head2 B<unlicense($service)>

Removes the license for the specified service on the filer.

=over 4

=item Inputs

$service

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

    $filer_obj->unlicense("fcp");
    if ($filer_obj->error){
                $Test->log($filer_obj->errmsg());
                return $TCase::FAIL;
    }

=back

=head2 B<license($license)>

Adds the specified license on the filer.

=over 4

=item Inputs

$license

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

    $filer_obj->license("abcdefg");
    if ($filer_obj->error){
                $Test->log($filer_obj->errmsg());
                return $TCase::FAIL;
    }

=back

=head2 B<ip_online($self, $interface)>

Brings an interface up.

=over 4

=item Inputs

=item $interface

OPTIONAL: The interface to bring up. Defaults to e0.

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi_cli

=item Examples

    # example
    $filer_obj->ip_online("e0a");

=back

=head2 B<ip_offline($self, $interface)>

Brings an interface down.

=over 4

=item Inputs

=item $interface

OPTIONAL: The interface to bring down. Defaults to e0.

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi_cli

=item Examples

    # example
    $filer_obj->ip_offline("e0a");

=back

=head2 B<ip_set($self, $interface, $address, $mask)>

Sets the ip address for the interface

=over 4

=item Inputs

=item $interface

The interface to set the ip address on.

=item $address

The ip address to set.

=item $mask

OPTIONAL: The subnet mask to use for the interface

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi_cli

=item Examples

    # Example with mask
    $filer_obj->ip_set("e0a", "192.168.154", "255.255.255.0");

=back

=head2 B<vif_create($self, $name, $type, $interfaces, $ip_address, $subnet)>

Creates a virtual interface using the provided parameters.

=over 4

=item Inputs

=item $name

Name of the VIF.

=item $type

(Single|Multi) VIF

=item $interfaces

OPTIONAL: Space seperated list of interfaces to compose the VIF

=item $ip_address

OPTIONAL: The ip address of the VIF

=item $subnet

OPTIONAL: The subnet mask of the interface (if not supplied when assigning
an IP, /24 is used)

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

    # Basic VIF
    $filer_obj->vif_create("vif0", "single");

=back

=head2 B<vif_exist($self, $name)>

Returns true or false if the provided vif exists or not.

=over 4

=item Inputs

=item $name

The name of the vif to check

=item Outputs

True- Exists, False- Doesn't exist

=item Supported Interfaces

cli

=item Examples

    # example
    $output = $filer_obj->vif_exist("vif0");

=back

=head2 B<vif_destroy($self, $name)>

Destroys a vif

=over 4

=item Inputs

=item $name

The name of the vif to destroy

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

    # example
    $filer_obj->vif_destroy("vif0");

=back

=head2 B<vlan_create($self, $interface, $id, $ip_address, $subnet)>

Creates a vlan for an interface.

=over 4

=item Inputs

=item $interface

The interface to create a vlan for.

=item $id

The id of the new vlan.

=item $ip_address

OPTIONAL: The ip address to assign to the new vlan.

=item $subnet

OPTIONAL: The subnet mask to use for the new vlan, if not defined /24 is used.

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

    # Basic
    $filer_obj->vlan_create("e0a", 10);

=back

=head2 B<vlan_exist($self, $interface, $id)>

Returns true or false if the vlan exists or not.

=over 4

=item Inputs

=item $interface

The interface to check for vlans

=item $id

The id of the vlan for the interface

=item Outputs

True- Exists, False- Doesn't exist

=item Supported Interfaces

cli

=item Examples

    # example
    $output = $filer_obj->vlan_exist("e0a", 10);

=back

=head2 B<vlan_destroy($self, $interface, $id)>

Destroys a vlan

=over 4

=item Inputs

=item $interface

The vlan base interface

=item $id

The vlan id

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

    # example
    $filer_obj->vlan_destroy("e0a", 10);

=back

=head2 B<ifconfig_get_mtu_size($self, $interface)>

Gets the mtu size for an interface

=over 4

=item Inputs

=item $interface

The interface to fetch the mtusize for.

=item Outputs

The mtusize for the interface.

=item Supported Interfaces

cli

=item Examples

    # example
    $mtusize = $filer_obj->ifconfig_get_mtu_size("e0b");

=back

=head2 B<ifconfig_set_mtu_size($self, $interface, $mtusize)>

Sets the mtusize for an interface on a filer.  Uses both ifconfig and
also sets in /etc/rc

=over 4

=item Inputs

=item $interface

The interface to be used.

=item $mtusize

The mtusize to be set on the interface.

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

    # example
    $filer_obj->ifconfig_set_mtu_size("e0b", 9000);

=back

=head2 B<options_iscsi_config_mcs($self, $tpname, $interfaces, $erl, $max_conn)>

Configures MCS on the filer.

=over 4

=item Inputs

=item $tpname

The name for the tpgroup to be created (used for MCS)

=item $interfaces

Space seperated list of interfaces to be used for the tpgroup.

=item $erl

OPTIONAL: The error recovery level, if not supplied set by default to 0.

=item $max_conn

OPTIONAL: The max connections per session, if not supplied set be default to 4.

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli and zapi

=item Examples

    #Basic
    $filer_obj->options_iscsi_config_mcs("tpname", "e0a e0b");

=back

=head2 B<ntp_sync>

Sync a controller to a specified NTP time server

=over 4

=item Inputs

=item $ntp_server

NTP time server that you want to sync to

=item Outputs

=item ClientUtil::TRUE | ClientUtil::FALSE

Returns success or fail

=item Supported Interfaces

cli

=item Examples

    my $status = $filer_obj->ntp_sync(-ntp_server => $NTP_TIME_SERVER,
                                      -ntp_time_zone => $NTP_TIME_ZONE);

=back

=head2 B<fcp_start()>

Start up fcp service

=over 4

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error flag
is set, errmsg is set to the failure reason, and 0 is passed back as the return values.

=item Supported Interfaces

cli and zapi

=item Examples

 # Start fcp service
 if (not $Filer->fcp_start()){
        print $Filer->errmsg()."\n";
 }


=back

=head2 B<fcp_stop()>

Stop fcp service

=over 4

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error flag
is set, errmsg is set to the failure reason, and 0 is passed back as the return values.

=item Supported Interfaces

cli and zapi

=item Examples

 # Stop fcp service
 if (not $Filer->fcp_stop()){
        print $Filer->errmsg()."\n";
 }

=back

=head2 B<fcp_setup_lun_trace($self, $level)>

Sets up lun tracing on the filer.

=over 4

=item Inputs

=item $level

OPTIONAL: The trace level to be used, if not defined, "3" will be used.

=item Outputs

Returns True if successful. If an error occured the error
flag is set, errmsg is set to the failure reason, and False is passed back as
the return value.

=item Supported Interfaces

cli

=item Examples

    # using level 4
    $filer_obj->fcp_setup_lun_trace("4");

=back

=head2 B<fcp_wwpnalias_get(%args)> (from San::SanFiler::Cmds::Fcp)

Get WWPN alias

=over 4

=item Inputs

=item * -alias (optional)

Getting WWPN that mapped to the specified alias

=item * -wwpn (optional)

Get alias(es) that associated with specified wwpn

=item Outputs

Returns a hash. The hash contains {alias_name => wwpn_name}.
Failure or no existing alias will return undef.

  single alias output:
    alias_name => 10:00:00:00:c9:48:ce:9f

  multi aliases output:
    alias_name1 => 10:00:00:00:c9:48:ce:9f
    alias_name2 => 10:00:00:00:c9:48:ce:9f
    alias_name3 => 10:00:00:00:c9:48:ce:9f
    alias_name4 => 10:00:00:00:c9:48:ce:9f
    alias_name5 => 10:00:00:00:c9:48:ce:9f

=item Supported Interfaces

cli and zapi

=item Examples

    #
    # Get alias(es) that mapped to a wwpn
    #
    $hash_ref = $fcp_obj->fcp_wwpnalias_get(-wwpn=>$wwpn_name);
    if ($fcp_obj->error())
    {
       my $errmsg = join("\n", $fcp_obj->errmsg());
       $Test->nlog($errmsg);
       return $TCD::FAIL;
    }

=back

=head2 B<fcp_wwpnalias_remove(%args)> (from San::SanFiler::Cmds::Fcp)

If alias was given, remove that alias which mapped to a WWPN
If wwpn was given, remove alias(es) that mapped to the given WWPN

=over 4

=item Inputs

=item * -alias  or -wwpn (required either)

Name of an alias or wwpn

=item Outputs

 TRUE  = SUCCESS = 1
 FALSE = FAILURE = 0

=item Supported Interfaces

cli and zapi

=item Examples

    #
    # Remove given alias that mapped to a wwpn
    #
    if (not $fcp_obj->fcp_wwpnalias_remove(-alias=>$alias_name))
    {
       my $errmsg = join("\n", $fcp_obj->errmsg());
       $Test->nlog($errmsg);
       return $TCD::FAIL;
    }

=back

=head2 B<fcp_wwpnalias_set(%args)> (from San::SanFiler::Cmds::Fcp)

Create an alias and map to the given wwpn

=over 4

=item Inputs

=item * -alias  (required)

Name of a alias.

=item * -wwpn  (required)

Name of a WWPN.

=item Outputs

 TRUE  = SUCCESS = 1
 FALSE = FAILURE = 0

=item Supported Interfaces

cli and zapi

=item Examples

    #
    # Create an alias and map to a given wwpn
    #
    if (not $fcp_obj->fcp_wwpnalias_set(-alias=>$alias_name,
                                    -wwpn=>$wwpn_name))
    {
       my $errmsg = join("\n", $fcp_obj->errmsg());
       $Test->nlog($errmsg);
       return $TCD::FAIL;
    }

=back

=head2 B<any_portset_exist()> (from San::SanFiler::Cmds::Portset)

Find out if any portsets exist

=over 4

=item Outputs

Returns 1 if a portset exists, 0 if no portsets exist and any output

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output) = $portset_obj->any_portset_exist();

=back

=head2 B<destroy_all_portsets()> (from San::SanFiler::Cmds::Fcp)

Destroys all portsets

=over 4

=item Supported Interfaces

cli and zapi

=item Examples

    $portset_obj->destroy_all_portsets();

=back

=head2 B<get_filer_ontap_version()> (from San::SanFiler::Cmds::Portset)

Gets the filer version

=over 4

=item Outputs

Returns the filer version

=item Supported Interfaces

All

=item Examples

    $version = $portset_obj->get_filer_ontap_version();

=back

=head2 B<get_portset_info(%args)> (from San::SanFiler::Cmds::Portset)

Get information on a specific portset

=over 4

=item Inputs

=item * I<-portset_name>

string = name of the portset

=item Outputs

Returns 1 if command was successful, 0 if command unsuccessful, any output, a hash
of what the command returned

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output, %info) = $portset_obj->get_portset_info(
                                            -portset_name   => $portset_name);

=back

=head2 B<igroup_bound_to_portset(%args)> (from San::SanFiler::Cmds::Portset)

Determine whether or not an igroup is bound to a portset

=over 4

=item Inputs

=item * I<-portset_name>

string = name of the portset

=item * I<-igroup_name>

string = name of the igroup

=item Outputs

Returns 1 if command successful, 0 if command unsuccessful, any output

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output) = $portset_obj->igroup_bound_to_portset(
                                            -portset_name   => $portset_name,
                                            -igroup_name    => $igroup_name);

=back

=head2 B<port_exist_in_portset(%args)> (from San::SanFiler::Cmds::Portset)

Determine whether or not a port exists in a portset

=over 4

=item Inputs

=item * I<-portset_name>

string = name of the portset

=item * I<-port>

string = port value to check

=item * I<-filer>

string = the filer you are running on

=item Outputs

Returns 1 if port exists, 0 if port doesn't exist, any output

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output) = $portset_obj->port_exist_in_portset(
                                            -portset_name   => $portset_name,
                                            -port           => $port,
                                            -filer          => $filer_name);

=back

=head2 B<ports_exist_in_portset(%args)> (from San::SanFiler::Cmds::Portset)

Determine whether or not a set of ports exist in a portset

=over 4

=item Inputs

=item * I<-portset_name>

string = name of the portset

=item * I<-ports_ref>

string = reference to the list of ports to check

=item * I<-filer>

string = the filer you are running on

=item Outputs

Returns 1 if ports exist, 0 if ports don't exist, any output

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output) = $portset_obj->ports_exist_in_portset(
                                            -portset_name   => $portset_name,
                                            -ports_ref      => \@ports,
                                            -filer          => $filer_name);

=back

=head2 B<filersio_asyncio_active(%args)>

Keeps IOs in flight at a constant

=over 4

=item Inputs

=item * -read_percentage (required)

integer = percentage of I/Os that are read, range [0,100]

=item * -rand_percentage (required)

integer = percentage of I/Os with file offsets determined using uniformly
random distribution; rest of them will be sequential
This is actually optional, but either this or exp_constant is required

=item * -exp_constant (required)

integer = exponential constant greater than 0
This is actually optional, but either this or rand_percentage is
required

=item * -blk_size (required)

integer = I/O size eg., 4k , 213

=item * -start_byte (required)

integer = lower bound for I/O access in bytes eg., 0,  213, 4k, 10m

=item * -end_byte (required)

integer = upper bound for I/O access in bytes eg., 10m, 107868, 20g

=item * -run_time (required)

integer = run time in seconds

=item * -num_active (required)

integer = number of IOs that will be in flight at any given point in time

=item * -filename (required)

string = file (path) which will be the target of the workload

=item * I<-fill_pattern>

integer = pattern that the blocks should be filled up with:
pattern is a 4 byte integer between 0 and (2^31 - 1)
-2 and -3 are the only acceptable negative values for the pattern
-2 implies that the block will be filled with the block number
-3 implies that the block will be filled with junk - it provides a faster
way to fill up a file

=item * I<-verify_pattern>

integer = pattern that the blocks being read should be filled up with:
pattern is a 4 byte integer between 0 and (2^31 - 1)
-2 is the only acceptable negative value for the pattern and implies that
the block should be filled up with the block number
NOTE: -fill_pattern and -verify_pattern cannot be specified in the same run

=item * I<-sync>

boolean = run the workload synchronously; default mode is to run the workload
asynchronously and print out the results when it completes
NOTE: the I/Os are always launched asynchronously, this option only affects
the behavious of the user interface; with the -synchronous option, filersio
will run the workload (in the foreground holding the CLI) and print the
results and release the CLI; the default behaviour is to start the workload
in the background and release the CLI immediately; results will be printed
out when the workload completes

=item * I<-create>

boolean = create the file "filename" if it does not exist

=item * I<-interval>

integer = interval between printing statistics in seconds

=item Outputs

Returns 1 if command was successful, 0 if command unsuccessful, any output, a hash
of what the command returned

=item Supported Interfaces

cli, zapi, and zapi_cli

=item Examples

    $filer_obj->filersio_asyncio_active(-read_percentage => "0",
                                        -rand_percentage => "0",
                                        -blk_size        => "8k",
                                        -start_byte      => "0",
                                        -end_byte        => $end_byte,
                                        -run_time        => "0",
                                        -num_active      => "16",
                                        -filename        => $fname,
                                        -fill_pattern    => "0",
                                        -sync            => "1",
                                        -create          => "1");

=back

=head2 B<remove_file(%args)>

Removes a file.

=over 4

=item Inputs

=item * -file

string = name (path) of the file to remove

=item Outputs

Returns 1 if command was successful, 0 if command unsuccessful

=item Supported Interfaces

cli and zapi_cli

=item Examples

    $filer_obj->remove_file(-file => $file_name);

=back

=head2 B<get_fcp_status()>

Gets FCP status.

=over 4

=item Outputs

Returns the status of FCP if successful, otherwise ""

=item Supported Interfaces

cli and zapi

=item Examples

    $status = $filer_obj->get_fcp_status();

=back

=head2 B<get_iscsi_status()>

Gets iSCSI status.

=over 4

=item Outputs

Returns the status of iSCSI if successful, otherwise ""

=item Supported Interfaces

cli and zapi

=item Examples

    $status = $filer_obj->get_iscsi_status();

=back

=head2 B<get_iscsi_target()>

Returns an iSCSI Target object by name.

=over 4

=item Outputs

Returns an iscsi target object by name, otherwise {}

=item Examples

    $iscsi_target_obj = $filer_obj->get_iscsi_target();

=back

=head2 B<get_iscsi_targets()>

Return array of iscsi target object

=over 4

=item Outputs

Returns an array of iscsi target objects, otherwise []

=item Supported Interfaces

cli and zapi

=item Examples

    @iscsi_target_objs = $filer_obj->get_iscsi_targets();

=back

=head2 B<get_iscsi_alias()>

Get the iscsi alias name of the filer

=over 4

=item Outputs

Returns the iscsi alias, otherwise undef

=item Examples

    $iscsi_alias = $filer_obj->get_iscsi_alias();

=back

=head2 B<set_iscsi_alias()>

Set the iscsi alias name for the filer

=over 4

=item Outputs

Returns a scalar value of 1 if successful, otherwise 0

=item Supported Interfaces

cli and zapi

=item Examples

    $filer_obj->set_iscsi_alias(-name => $new_alias);

=back

=head2 B<clear_iscsi_alias()>

Clears the iscsi alias name for the filer

=over 4

=item Outputs

Returns a scalar value of 1 if successful, otherwise 0

=item Supported Interfaces

cli and zapi

=item Examples

    $filer_obj->clear_iscsi_alias();

=back

=head2 B<get_iscsi_nodename()>

Gets the iscsi nodename on a system

=over 4

=item Inputs

None

=item Outputs

Returns the nodename string if the command is successful. If an error occured
the error flag is set, errmsg is set to the failure reason, and 0 is passed
back as the return value.

=item Supported Interfaces

cli

=item Examples

    my $iscsi_nodename = $filer_obj->get_iscsi_nodename();

=back

=head2 B<fcp_adapter_online(%args)>

Onlines an fcp adapter.

=over 4

=item Inputs

=item * -adapter

The fcp adapter to online

=over 4

=item Outputs

Returns a scalar value of 1 if successful, otherwise 0

=item Supported Interfaces

zapi

=item Examples

    $filer_obj->fcp_adapter_online(-adapter = $adapter);

=back

=head2 B<vol_sis_status()>

Gets the SIS status for all volumes.

=over 4

=item Outputs

Returns the SIS status of the volumes if successful, otherwise ""

=item Supported Interfaces

cli, zapi, zapi_cli

=item Examples

    $status = $filer_obj->vol_sis_status();

=back

=head2 B<get_aggr_capacity(%args)>

Gets the current capacity (%) of an aggregate

=over 4

=item Inputs

=item * -aggr

string = name of the aggregate to get the capacity of

=item Outputs

Returns the capacity if it worked, otherwise returns ""

=item Supported Interfaces

cli and zapi_cli

=item Examples

    my $capacity = $filer_obj->get_aggr_capacity(-aggr => "test_aggr");

=back

=head2 B<get_aggr_size(%args)>

Get's the size (kb) of an aggregate

=over 4

=item Inputs

=item * -aggr

string = name of the aggregate to get the size of

=item Outputs

Returns the size of the aggregate in kb if successful, otherwise
returns ""

=item Supported Interfaces

cli and zapi_cli

=item Examples

    my $size = $filer_obj->get_aggr_size(-aggr => "test_aggr");

=back

=head2 B<portset_add(%args)> (from San::SanFiler::Cmds::Portset)

Add ports to a portset

=over 4

=item Inputs

=item * I<-portset_name>

string = name of the portset

=item * I<-ports_ref>

string = a reference to the list of ports

=item Outputs

Returns 1 if command was successful, 0 if command unsuccessful, any output, a hash
of what the command returned

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output, %info) = $portset_obj->portset_add(
                                            -portset_name   => $portset_name,
                                            -ports_ref      => \@ports);

=back

=head2 B<portset_bound_to_igroup(%args)> (from San::SanFiler::Cmds::Portset)

Determine whether or not a portset is bound to an igroup

=over 4

=item Inputs

=item * I<-portset_name>

string = name of the portset

=item * I<-igroup_name>

string = name of the igroup

=item Outputs

Returns 1 if command successful, 0 if command unsuccessful, any output

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output) = $portset_obj->portset_bound_to_igroup(
                                            -portset_name   => $portset_name,
                                            -igroup_name    => $igroup_name);

=back

=head2 B<portset_create(%args)> (from San::SanFiler::Cmds::Portset)

Create a portset

=over 4

=item Inputs

=item * I<-portset_name>

string = name of the portset

=item * I<-portset_type>

string = iscsi or fcp

=item * I<-ports_ref>

string = a reference to the list of ports

=item Outputs

Returns 1 if command was successful, 0 if command unsuccessful, any output, a hash
of what the command returned

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output, %info) = $portset_obj->portset_create(
                                            -portset_name   => $portset_name,
                                            -ports_ref      => \@ports,
                                            -portset_type   => "fcp");

=back

=head2 B<portset_destroy(%args)> (from San::SanFiler::Cmds::Portset)

Destroy a portset

=over 4

=item Inputs

=item * I<-portset_name>

string = name of the portset

=item * I<-force>

string = yes | no, 1 | 0, true | false, etc.

=item Outputs

Returns 1 if command was successful, 0 if command unsuccessful, any output, a hash
of what the command returned

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output, %info) = $portset_obj->portset_create(
                                            -portset_name   => $portset_name,
                                            -force          => "yes");

=back

=head2 B<portset_exist(%args)> (from San::SanFiler::Cmds::Portset)

Determine whether or not a portset exists

=over 4

=item Inputs

=item * I<-portset_name>

string = name of the portset

=item Outputs

Returns 1 if portset exists, 0 if portset doesn't exist, any output

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output) = $portset_obj->portset_bound_to_igroup(
                                            -portset_name   => $portset_name);

=back

=head2 B<portset_remove(%args)> (from San::SanFiler::Cmds::Portset)

Removes ports from portset

=over 4

=item Inputs

=item * I<-portset_name>

string = name of the portset

=item * I<-ports_ref>

string = a reference to the port list

=item Outputs

Returns 1 if command was successful, 0 if command unsuccessful, any output, a hash
of what the command returned

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output, %info) = $portset_obj->portset_create(
                                            -portset_name   => $portset_name,
                                            -ports_ref      => \@ports);

=back

=head2 B<portset_show(%args)> (from San::SanFiler::Cmds::Portset)

List information on a specific portset

=over 4

=item Inputs

=item * I<-portset_name>

string = name of the portset

=item Outputs

Returns 1 if command was successful, 0 if command unsuccessful, any output, a hash
of what the command returned

=item Supported Interfaces

cli and zapi

=item Examples

    my ($result, $output, %info) = $portset_obj->portset_show(
                                            -portset_name   => $portset_name);

=back

=head2 B<force_reboot()>

Force reboot the filer

=over 4

=item Outputs

Returns 1 if command was successful, 0 if command unsuccessful

=item Supported Interfaces

cli

=item Examples

    # force reboot filer
    $filer_obj->force_reboot();

=back

=head2 B<reboot()>

Reboot the filer

=over 4

=item Outputs

Returns 1 if command was successful, 0 if command unsuccessful

=item Supported Interfaces

cli

=item Examples

    # reboot filer
    $filer_obj->reboot();

=back

=head2 B<set_mem_stats()>

Execute mem_stats -b command on filer

=over 4

=item Outputs

Returns 1 if command was successful, 0 if command unsuccessful

=item Supported Interfaces

cli and zapi_cli

=item Examples

    $filer_obj->set_mem_stats();

=back

=head2 B<get_mem_stats()>

Execute mem_stats -e command on filer

=over 4

=item Outputs

Returns mem_stats string

=item Supported Interfaces

cli and zapi_cli

=item Examples

    $mem_stats_string = $filer_obj->get_mem_stats();

=back

=head2 B<console_connect(%args)>

Connect to the console connection. The connection object is saved in
$self->{console_conn} in addition to being returned on success.

=over 4

=item inputs

=item * I<-filername>

REQUIRED: the node to use that needs the console connected.

=item * I<-conn_str>

OPTIONAL: The node connection string to use. Default #console, #rlm if
use_rlm_console is $ClientUtil::TRUE.

=item * I<-use_rlm_console>

OPTIONAL: Use the rlm console "system conosle" instead of the serial
#console connection.

=item * I<-timeout>

OPTIONAL: Time a console command is given to issue the command and reach
an expected prompt.

=item outputs

Returns connection object on success, empty hash otherwise.

=item supported interfaces

cli

=item examples

    $filer_obj = $global_filer_objects->{$halt_filer};
    $console = $filer_obj->console_connect('-filername' => $halt_filer,
                                           '-timeout' => 60,
                                           '-use_rlm_console' => $USE_RLM_CONSOLE);
    if ($filer_obj->error) {
        &SanUtil::nate_log('-fatal', "Unable to open node \"$halt_filer\"'s ".
                           "console.\n" . $filer_obj->errmsg());
    }

=back

=head2 B<console_disconnect(%args)>

Disconnect the already connected console.

=over 4

=item inputs

=item * I<-connection>

OPTIONAL: The connection to disconnect. $self->{console_conn} if not specified.

=item outputs

Return $ClientUtil::TRUE on success, $ClientUtil::FALSE otherwise.

=item supported interfaces

cli

=item examples

    $filer_obj->console_disconnect('-connection' => $console);
    if ($filer_obj->error) {
        &SanUtil::nate_log('-fatal', "inject_fault.thpl: Unable to successfully close node \"$halt_filer\"'s console.\n" . $filer_obj->errmsg());
    }

=back

=head2 B<console_halt(%args)>

Issues a halt command to the console and wait for firmware prompt.

=over 4

=item Inputs

=item * I<-passthru>

OPTIONAL: The additional commands to add to the end of the halt command. For e.g. passthru
string -t 1, would be to execute halt -t 1.

=item * I<-connection>

OPTIONAL: The cli connection to use to execute the commands.
$self->{console_conn} if not specified.

=item * I<-force>

OPTIONAL: Use the halt -f options to bring node down. Default is not to use -f.

=item * I<-timeout>

OPTIONAL: Time to wait for the filer to go to firmware prompt.

=item Outputs

Returns $ClientUtil::TRUE on sucess $ClientUtil::FALSE otherwise.

=item Supported Interfaces

cli

=item Examples

    $filer_obj->console_halt('-timeout' => $FILER_DOWN_TIMEOUT,
                             '-connection' => $console);

    if ($filer_obj->error) {
        &SanUtil::nate_log('-fatal', "Unable to successfully bring " .
                           $filer_obj->name() .
                           "to firmware prompt.\n" . $filer_obj->errmsg());
    }

=back

=head2 B<boot_ontap(%args)>

Issues a boot command on the filer and waits for the boot_to_prompt.

=over 4

=item Inputs

=item * I<-boot_to_prompt>

REQUIRED: The prompt to wait for after issuing the boot cmd.

=item * I<-boot_cmd>

OPTIONAL: Command to issue to boot the node.

=item * I<-connection>

OPTIONAL: The cli console connection to use for booting the controller.
$self->{console_conn} if not specified.

=item * I<-timeout>

OPTIONAL: Time to wait for the filer to go to boot_to_prompt

=item Outputs

Returns $ClientUtil::TRUE on sucess $ClientUtil::FALSE otherwise.

=item Supported Interfaces

cli

=item Examples

    my $expected_prompt = "Waiting for giveback|Waiting for cluster give";
    $filer_obj->boot_ontap('-timeout' => $FILER_DOWN_TIMEOUT,
                           '-boot_to_prompt' => $expected_prompt,
                           '-connection' => $console);

    if ($filer_obj->error) {
        &SanUtil::nate_log('-fatal', "Unable to successfully boot "
                           $filer_obj->name() .
                           "to \"$expected_prompt\" prompt.\n" .
                           $filer_obj->errmsg());
    }

=back

=head2 B<log_cmd_results()> (from San::SanUtil)

Prints out the results hash of a cmdsDriver::execute_cmd() call.

=over 4

=item Inputs

=item * resultsHashRef

    A reference to the results hash of a command. This is what is
    returned by CmdsDriver::execute_cmd().

=item Outputs

None

=item Supported Interfaces

All

=item Examples

    # Print command results hash
    $self->log_cmd_results($resultsHash_ref);

=back

=head2 B<radius_add(%args)>

Adds a radius server to Data ONTAP using the 'radius add' command.

=over 4

=item Inputs

=item * -host

(Required) ip or hostname of radius server to add.

=item * -port

(Optional) Port number of the radius server.  To take the default port, use
'default' or do not provide the -port option.  The default port is 1812.

=item * -default_option

(Optional)
'1' = Use the -d flag to specify this is the default radius server.
'0' = Do not use the -d flag.  Radius server is added to the next available slot.

=item Outputs

For successful add, returns: $ClientUtil::TRUE.
If the add fails, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   # Add radius server 10.10.10.10 with default port to next available slot.
   $filer_obj->radius_add(-host => "10.10.10.10");

   # Add radius server "my_host" using port 10000 and -d flag.
   $filer_obj->radius_add(-host           => "my_host"
                          -port           => 10000,
                          -default_option => 1);


   # Add radius server "my_host" using default port and without -d flag.
   $filer_obj->radius_add(-host           => "my_host",
                          -port           => "default");

=back

=head2 B<radius_remove(%args)>

Removes a radius server from Data ONTAP using the 'radius remove' command.

=over 4

=item Inputs

=item * -host

(Required) ip or hostname of radius server to remove.

=item * -port

(Optional) Port number of the radius server.  To take the default port, use
'default' or do not provide the -port option.  The default port is 1812.

=item Outputs

For successful remove, returns: $ClientUtil::TRUE.
If the remove fails, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   # Remove radius server 10.10.10.10 with default port.
   $filer_obj->radius_remove(-host => "10.10.10.10");

   # Remove radius server "my_host" using port 10000.
   $filer_obj->radius_remove(-host => "my_host"
                             -port => 10000);

=back

=head2 B<radius_remove_all_servers(%args)>

Starts radius service and removes all radius servers from Data ONTAP.

=over 4

=item Inputs

=item * -radius_hash

(Optional) This is the hash output of the radius show command.  If not provided,
the function will call radius_show to obtain it.

=item Outputs

For successful removal of all servers, returns: $ClientUtil::TRUE.
For failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   # Remove all servers, do not supply a hash:
   $Filer_Obj->radius_remove_all_servers();

   # Remove all servers, supply a hash:
   $Filer_Obj->radius_remove_all_servers(-radius_hash => $Radius_Hash));

=back

=head2 B<radius_show()>

Performs the radius show command and returns a hash of radius information.  The
radius hash is in the format of the zapidoc output for radius-show-info.

=over 4

=item Inputs

=item * None

=item Outputs

On success, returns pointer to a radius_hash.
For failure, returns: $ClientUtil::FALSE or error string and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   $radius_hash = $filer_obj->radius_show();

=back

=head2 B<radius_start()>

Start radius service.

=over 4

=item Inputs

=item * None

=item Outputs

On success, returns: $string of output text.
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   $filer_obj->radius_start();

=back

=head2 B<radius_stats()>

Performs the radius stats command and returns a hash of radius statistics.
The hash is in the format of the zapidoc output for radius-stats-list-info.

=over 4

=item Inputs

=item * None

=item Outputs

On success, returns pointer to a hash of radius stats.
For failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   $filer_obj->radius_stats();

=back

=head2 B<radius_stats_zero()>

Zeroes radius statistics.

=over 4

=item Inputs

=item * None

=item Outputs

On success, returns $ClientUtil::TRUE.
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   $filer_obj->radius_stats_zero();

=back

=head2 B<radius_status()>

Check radius service status.

=over 4

=item Inputs

=item * None

=item Outputs

On success, returns: $ClientUtil::TRUE - radius is running or $ClientUtil::FALSE
 - radius is NOT running.
On failure, returns: undef and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   $filer_obj->radius_status();

=back

=head2 B<radius_stop()>

Stop radius service.

=over 4

=item Inputs

=item * None

=item Outputs

On success, returns: $string of output text.
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   $filer_obj->radius_stop();

=back

=head2 B<options_get(%args)>

Returns the value of an option.

=over 4

=item Inputs

=item * -option

(Required) Exact name of the option to lookup.

=item * -vfiler

(Optional) Vfiler context from which option to get

=item Outputs

On success, returns: The value of the option.  Note the value may be undefined.
On failure, returns: undefined and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

       my $val = $Filer_Obj->options_get(-option => "iscsi.auth.radius.enable");

       my $val = $Filer_Obj->options_get(-option => "iscsi.ip_based_tpgroup",
                                         -vfiler => "vfiler1");

=back

=head2 B<options_set(%args)>

Sets the value of an option.

=over 4

=item Inputs

=item * -option

(Required) Exact name of the option to set.

=item * -value

(Required) The value of the option to set.

=item * -vfiler

(Optional) Vfiler context in which option is set

=item Outputs

On success, returns: $ClientUtil::TRUE.
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

       $Filer_Obj->options_set(-option => "iscsi.auth.radius.enable",
                               -value  => "off");

       $Filer_Obj->options_set(-option => "iscsi.ip_based_tpgroup",
                               -value  => "on",
                               -vfiler => "vfiler1");

=back

=head2 B<priv_set(%args)>

Set the cli priv level.

=over 4

=item Inputs

=item * -level

(Required) Priv level to set.  Valid choices: admin, advanced, diag, test

=item * -quiet

(Optional) Suppresses output of the command.  1 = add -q.  0 = do not add -q.
Default: 1

=item Outputs

On success, returns: $ClientUtil::TRUE.
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli

=item Examples

       $Filer_Obj->priv_set(-level => "admin");
       $Filer_Obj->priv_set(-level => "admin",
                            -quiet => 1);

=back

=head2 B<qtree_exist()>

Determine if Qtree exists

=over 4

=item Inputs

=item * Qtree

(Required) Name of qtree to locate.

=item * Volume

(Required) Name of the Volume the qtree is on

=item Outputs

On success, returns $ClientUtil::TRUE;
On failure, returns $ClientUtil::FALSE;

=item Supported Interfaces

cli, zapi

=item Examples

        my $password = $Filer_Obj->iscsi_security_generate();

=back

=head2 B<get_ip_addresses()>

Gets all IP addresses assigned to a filer. It basically executes ifconfig -a
on the controller and puts all IP addresses on a list.

=over 4

=item inputs

None

=item outputs

Returns a list of IP addresses on success or sets the error flag and returns an
empty list if it failed.

=item supported interfaces

cli

=item examples

    my @ips = $filer_obj->get_ip_addresses();
    if ($filer_obj->error) {
        &SanUtil::nate_log("Failed to get ip addresses from the filer");
    }

    foreach my $ip (@ips) {
        # ... do something
    }

=back

=head2 B<refresh_iscsi_targets()>

Refresh the iscsi targets list

=over 4

=item Inputs

=item * None

=item Outputs

On success, returns true
On failure, returns false and errmsg is set.

=item Supported Interfaces

N/A

=item Examples

        $Filer_Obj->refresh_iscsi_targets();

=back

=head2 B<get_lun_maxsize()>

This method returns the maximum possible lun size for given lun type in a volume.

=over 4

=item Inputs

=item * -path

Lun path.

=item * -type

Lun type.

=item * -snapshot_reserve

Indicate whether or not to use snapshot_reserve; valid values are 1 or 0.

Lun type.

=item Outputs

Returns size in bytes for zapi and size with units for cli.

=item Supported Interfaces

cli and zapi

=item Examples

    # Get maxsize possible for a solaris lun in the test volume
    my $maxSize = $Filer->get_lun_maxsize(
                          -type             => "solaris",
                          -path             => "/vol/testVol",
                          -snapshot_reserve => 1);
    if ($Filer->error){
        $Test->log($Filer->errmsg());
        return $TCase::FAIL;
    }


=back

=head2 B<get_lun_minsize()>

This method returns the minimum lun size of a type of lun, zapi only.

=over 4

=item Inputs

=item * -type

Lun type.

=item Outputs

Returns the minimum possible lun size in bytes for a given lun type.


=item Supported Interfaces

zapi

=item Examples

    # Get lun minsize for a solaris lun
    my $minsize = $Filer->get_lun_minsize(-type => "solaris");
    if ($Filer->error){
        $Test->log($Filer->errmsg());
        return $TCase::FAIL;


    }

=back

=head2 B<syslog_msg(%args)>

Syslog a message to the controller syslog.

=over 4

=item inputs

=item * I<-msg>

(Required) The message to syslog.

=item outputs

returns $Sanutil::TRUE if successful, $SanUtil::FALSE and error if not.

=item supported interfaces

cli

=item examples

    $filer_obj->syslog_msg(-msg => "performing takeover");
    if ($filer_obj->error) {
        ....
    }

=back

=head2 B<get_version_string()>

This method gets and returns filer version string.

=over 4

=item Outputs

Returns filer version string.

=item Supported Interfaces

cli and zapi

=item Examples

    # Get filer version string
    my $version = $Filer->get_version_string();
    if ($Filer->error) {
        $Test->log($Filer->errmsg());
        return $TCase::FAIL;
    }

=back

=head2 B<is_debug_version()>

Determine if filer build version is a debug version

=over 4

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the error flag
is set, errmsg is set to the failure reason, and 0 is passed back as
the return values.

=item Supported Interfaces

cli and zapi

=item Examples

    my $ontap_ver_is_debug;
    my $ontap_ver = $Filer->is_debug_version();
    if ($Filer->error){
        $Test->log($Filer->errmsg());
        return $TCase::FAIL;
    }

    # Set a debug version flag
    if ($ontap_ver) {
        $ontap_ver_is_debug = 1;
        $Test->log("ONTAP version is debug");
    } else {
        $ontap_ver_is_debug = 0;
    }

=back

=head2 B<iscsi_isns_config(%args)>

Configure an isns server in Data ONTAP using the 'iscsi isns config' command.

=over 4

=item Inputs

=item * -isns_server

(Required) ip or hostname of isns server to add.

=item * -vfiler_name

(Optional) Name of Vfiler to execute command against

=item Outputs

On success, returns: $ClientUtil::TRUE
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

    $Filer_Obj->iscsi_isns_config(-isns_server => "10.225.113.127");
    if ($Filer_Obj->error()) {
        $Test->nlog($Filer_Obj->errmsg(), $err_label);
        return $TCD::FAIL;
    }

    $Filer_Obj->iscsi_isns_config(-isns_server => "myhostname",
                                  -vfiler_name => "vfiler5");
    if ($Filer_Obj->error()) {
        $Test->nlog($Filer_Obj->errmsg(), $err_label);
        return $TCD::FAIL;
    }

=back

=head2 B<iscsi_isns_start()>

Starts isns service.

=over 4

=item Inputs

=item * -vfiler_name

(Optional) Name of Vfiler to execute command against

=item Outputs

On success, returns: $string of output text. (may be empty on success)
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

    $Filer_Obj->iscsi_isns_start();
    if ($Filer_Obj->error()) {
        $Test->nlog($Filer_Obj->errmsg(), $err_label);
        return $TCD::FAIL;
    }

    $Filer_Obj->iscsi_isns_start(-vfiler_name => "vfiler5");
    if ($Filer_Obj->error()) {
        $Test->nlog($Filer_Obj->errmsg(), $err_label);
        return $TCD::FAIL;
    }

=back

=head2 B<iscsi_isns_stop()>

Stops isns service.

=over 4

=item Inputs

=item * -vfiler_name

(Optional) Name of Vfiler to execute command against

=item Outputs

On success, returns: $string of output text. (may be empty on success)
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

    $Filer_Obj->iscsi_isns_stop();
    if ($Filer_Obj->error()) {
        $Test->nlog($Filer_Obj->errmsg(), $err_label);
        return $TCD::FAIL;
    }

    $Filer_Obj->iscsi_isns_stop(-vfiler_name => "vfiler5");
    if ($Filer_Obj->error()) {
        $Test->nlog($Filer_Obj->errmsg(), $err_label);
        return $TCD::FAIL;
    }

=back

=head2 B<iscsi_isns_update()>

Re-sends Data ONTAP's iscsi info to the isns server.

=over 4

=item Inputs

=item * -vfiler_name

(Optional) Name of Vfiler to execute command against

=item Outputs

On success, returns: $ClientUtil::TRUE
On failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

    $Filer_Obj->iscsi_isns_update();
    if ($Filer_Obj->error()) {
        $Test->nlog($Filer_Obj->errmsg(), $err_label);
        return $TCD::FAIL;
    }

    $Filer_Obj->iscsi_isns_update(-vfiler_name => "vfiler5");
    if ($Filer_Obj->error()) {
        $Test->nlog($Filer_Obj->errmsg(), $err_label);
        return $TCD::FAIL;
    }

=back

=head2 B<iscsi_isns_show()>

Performs the isns show command and returns a hash of isns information.  The
isns hash is in the format of the zapidoc output for iscsi-isns-get-info.
See hash format in example below.  Note that entity id and ip address may be
empty if NOT CONFIGURED.

=over 4

=item Inputs

=item * -vfiler_name

(Optional) Name of Vfiler to execute command against

=item Outputs

On success, returns pointer to an isns_hash.
For failure, returns: $ClientUtil::FALSE and _set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   $isns_hash = $filer_obj->iscsi_isns_show();

   $isns_hash = $filer_obj->iscsi_isns_show(-vfiler_name => "vfiler5");

   # Hash format:
   $VAR1 = {
            'isns-ip-addr' => '10.225.192.36',
            'isns-entity-id' => 'isns:00000044',
            'is-isns-enabled' => 'false'
            };

=back

=head2 B<refresh_targets()>

Refresh fcp target objects.  This method will rebuild the fcp target cache.

=over 4

=item Inputs

none

=item Outputs

Returns a scalar value of 1 if successful.  If an error occured the error flag is set,
errmsg is set to the failure reason, and 0 is passed back as the return value.

=item Supported Interfaces

cli

=item Examples

    $Test->log("Refresh fcp targets");
    unless ($Sanfiler_Obj->refresh_targets()) {
        if ($Sanfiler_Obj->error()){
            $Test->nlog($Sanfiler_Obj->errmsg());
        }
    }

=back

=head2 B<fcp_adapters_list_info()>

Get "fcp show adapter" output.

=over 4

=item Inputs

=item * -adapter

(Optional) Name of adapter to execute command against

=item * -verbose

(Optional) Flag to indicate whether to execute command in verbose or normal mode.
Default is normal mode.

=item Outputs

This method will return hash of all the fields for corresponding adapters if successful.
The fields are different for ZAPI and CLI interfaces.
If an error occured the error flag is set, errmsg is set to the failure reason,
and 0 is passed back as the return value.

=item Supported Interfaces

cli

=item Examples

    $Test->log("Get fcp adapter verbose info for adapter '9a'");
    unless ($Sanfiler_Obj->fcp_adapters_list_info(-verbose => 'on',
                                            -adapter => '9a')) {
        if ($Sanfiler_Obj->error()){
            $Test->nlog($Sanfiler_Obj->errmsg());
        }
    }

    $Test->log("Get fcp adapters verbose info");
    unless ($Sanfiler_Obj->fcp_adapters_list_info(-verbose => 'on') {
        if ($Sanfiler_Obj->error()){
            $Test->nlog($Sanfiler_Obj->errmsg());
        }
    }

    $Test->log("Get fcp adapter info for adapter '9a'");
    unless ($Sanfiler_Obj->fcp_adapters_list_info(-verbose => 'off',
                                            -adapter => '9a')) {
        if ($Sanfiler_Obj->error()){
            $Test->nlog($Sanfiler_Obj->errmsg());
        }
    }

    $Test->log("Get fcp adapters info");
    unless ($Sanfiler_Obj->fcp_adapters_list_info() {
        if ($Sanfiler_Obj->error()){
            $Test->nlog($Sanfiler_Obj->errmsg());
        }
    }

=back

=head2 B<iscsi_portal_show()>

Display the list of target portals ("IP address", "TCP Port", "TPGroup",
"Interface") over which the filer is currently making available the
iSCSI service.

=over 4

=item Inputs

=item * I<-vfiler_name> (Optional)

Name of Vfiler to execute command against

=item Outputs

For successful show, returns: pointer to a portal_hash.
If the show fails, returns: {} and set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   $portal_hash = $filer_obj->iscsi_portal_show();

   $portal_hash = $filer_obj->iscsi_portal_show(-vfiler_name => "vfiler5");

=back

=head2 B<iscsi_interface_accesslist_show(@iqn)>

Fetches interface accesslist information as 'hash' using the 'iscsi interface
accesslist show' command.

=over 4

=item Inputs

=item * I<initiator IQN(s)> (optional)

An array of initiator IQNs for which the interface accesslist information required.

=item Outputs

For successful execution, returns: "HASH", contains initiator IQN as key
and interfaces(array) as hash value.
If there are no accesslists present, then null will be returned, and if the
user ask for the accesslist of a specific initiator which has no accesslist,
then returned hash contain the iqn as key, and the value is null.
If the execution fails, return {} and set_err.

=item Supported Interfaces

cli, zapi

=item Examples

   # Get information about interface accesslist for all node.
   %Out_hash_var1=$SanFilerObj->iscsi_interface_accesslist_show();
   # Get information about interface accesslist for one node.
   %Out_hash_var3=$SanFilerObj->iscsi_interface_accesslist_show("iqn.1986-03.com.sun:01:0003ba29c029.4898d235");

   # Get information about interface accesslist for more than one node.
   %Out_hash_var2=$SanFilerObj->iscsi_interface_accesslist_show("iqn.1991-05.com.microsoft:sanq21312", "iqn.1986-03.com.sun:01:0003ba29c029.4898d235");

=back

=head2 B<clear_mem_stats()>

Execute mem_stats -c command on filer

=over 4

=item Inputs

None

=item Outputs

$ClientUtil::TRUE, Frees the pcstats resources.

$ClientUtil::FALSE & set_err, Did not frees the pcstats resources or there was an execution error.

=item Supported Interfaces

cli and zapi

=item Examples

    $Filer_Obj->clear_mem_stats();

=back

=head2 B<set_vol0_default_exports>

Set the default /etc/exports for vol0 per the following example:

#Auto-generated by setup Fri Aug 10 12:02:21 EDT 2007
/vol/vol0       -sec=sys,rw,anon=0,nosuid
/vol/vol0/home  -sec=sys,rw,nosuid

=over 4

=item Inputs

None

=item Outputs

Returns TRUE if successful, otherwize FALSE.  If an error occured the error flag
is set and errmsg is set to the failure reason.

=item Supported Interfaces

All

=item Examples

    # Set default /etc/exports for vol0
    $filer_obj->set_vol0_default_exports();


=back
=head2 B<snap_create(I<%args>)>

Create a snapshot for a aggregate

=over 4

=item Inputs

=item * -name

The name of the snapshot.

=item * -aggr

The name of the Aggregate

=item Outputs

Returns a scalar value of 1 if successful, otherwise 0.  If an error occured the error flag
is set, errmsg is set to the failure reason.

=item Supported Interfaces:

cli

=item Examples

 # Create a  snapshot named snap3
 $Filer_obj->snap_create(-name => "ndvm_snap",
                            -aggr => $DestAggr);
    if ($Filer_obj->error()) {
        my $errmsg = $Filer_obj->errmsg();
        $Test->nlog("ERROR: Failed to create snapshot " . $errmsg);
        return $TCD::FAIL;
    }    # end if ($Vol_obj->error())

=back
=head2 B<snap_delete(I<%args>)>

delete a snapshot for a aggregate

=over 4

=item Inputs

=item * -name

The name of the snapshot.  Providing the '*' character as the value will delete all snapshots
on the filer aggregate.

=item * -aggr

The name of the Aggregate

=item Outputs

Returns a scalar value of 1 if successful, otherwise 0.  If an error occured the error flag
is set, errmsg is set to the failure reason.

=item Supported Interfaces:

cli

=item Examples

 # Create a  snapshot named snap3
 $Filer_obj->snap_delete(-name => "snap3",
                            -aggr => $src_aggr);
    if ($Filer_obj->error()) {
        my $errmsg = $Filer_obj->errmsg();
        $Test->nlog("ERROR: Failed to delete snapshot " . $errmsg);
        return $TCD::FAIL;
    }    # end if ($Vol_obj->error())

=back
=head2 B<snap_restore(I<%args>)>

restore snapshot for a aggregate

=over 4

=item Inputs

=item * -name

The name of the snapshot.

=item * -aggr

The name of the Aggregate

=item Outputs

Returns a scalar value of 1 if successful, otherwise 0.  If an error occured the error flag
is set, errmsg is set to the failure reason.

=item Supported Interfaces:

cli

=item Examples

 # restore a  snapshot named snap3
 $Filer_obj->snap_restore(-name => "snap3",
                                         -aggr => $src_aggr);
                if ($Filer_obj->error()) {
                    my $error_msg = $Filer_obj->errmsg();
                    $Test->nlog(
                          "ERROR: Failed to get set snap reserve" . $error_msg);
                }    # end if ($Vol_obj->error())
=back
=head2 B<snap_list(I<%args>)>

Retrieve information(raw) on snapshots

=over 4

=item Inputs

=item * I<-filter>

A regular expression to use to filter out which snap records are displayed.  Examples would
be "^snap_1$", "^snap_", "_2$", or "snap" for handling exact matches, starting with, ending
with, and containing, respectivley.

=item * -aggr

The name of the Aggregate

=item Outputs

Returns an array or array reference if successful.  If an error occured the error flag
is set, errmsg is set to the failure reason, and empty array ref is passed back as the
return values.

=item Supported Interfaces:

cli

=item Examples

 # Display snapshot that start with the word 'snap'
 @output= $Filer_obj->snap_list(-filter=>'^mysnap'
                                -aggr => $aggr_name);
 if ($Filer_obj->error){
     $Test->nlog($Filer_obj->errmsg());
 }

 $Test->log(@output);

=back
=head2 B<snap_exist(%args)>

Checks to see if a specific snapshot exists on this aggregate.

=over 4

=item Inputs

=item * -snapname

The name of the snapshot

=item * -aggr

The name of the Aggregate

=item Outputs

Returns a scalar value of 1 if it exists, otherwise 0 and an error is set.

=item Supported Interfaces:

cli

=item Examples

    if($Filer_obj->snap_exist(-snapname => "hourly.0",
                              -aggr    => $aggr_name))
    {
        # do something
    }

=back

=head2 B<fcp_nameserver_list_iter_start(I<%args>)>

Get the fabric nameserver objects the results of which are retreived
by using fcp-adapter-nameserver-list-iter-next.

=over 4

=item Inputs

=item * I<-adapter_name>

Adapter to get nameserver objects for, it is a optional pamameter
if not specified records of all adapter is listed

=item Outputs

Returns a hash refernce which includes number of records and tag value

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fcp_nameserver_list_iter_start(-adapter_name => "e0a");
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())

=back

=head2 B<fcp_nameserver_list_iter_next(I<%args>)>

Information returned for each nameserver entry include the port identifier,
port type, port name, node name, symbolic port name, symbolic node name,
fabric port name, class of service, and registered FC4 types.

=over 4

=item Inputs

=item * I<-maximum>

The maximum number of entries to retrieve.

=item * I<-tag>

Tag from a previous fcp-adapter-nameserver-list-iter-start.

=item Outputs

Returns a hash refernce which includes number of records and nameserver objects

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fcp_nameserver_list_iter_next(
                                                 -maximum => 10,
                                                 -tag     => $tag,
                                    );
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())


=back

=head2 B<fcp_nameserver_list_iter_end(I<%args>)>

Terminate a list iteration and clean up any saved info.

=over 4

=item Inputs

=item * I<-tag>

Tag from a previous fcp-adapter-nameserver-list-iter-start.

=item Outputs

Returns true on success and False on failure

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fcp_nameserver_list_iter_end(
                                                -tag     => $tag,
                                    );
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())


=back

=head2 B<fcp_topology_list_iter_start(I<%args>)>

Get the fabric topology for one or more switches.

=over 4

=item Inputs

=item * I<-adapter_name>

Adapter to get topology information, it is a optional pamameter
if not specified records of all adapter is listed

=item Outputs

Returns a hash refernce which includes number of records and tag value

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fcp_topology_list_iter_start(-adapter_name => "e0a");
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())


=back

=head2 B<fcp_topology_list_iter_next(I<%args>)>

 returned for each switch includes the nodename, logical name,
domain identifier, vendor, release version, and port information.
If a port has devices attached to it, then information about each
attached device is included as well.

=over 4

=item Inputs

=item * I<-maximum>

The maximum number of entries to retrieve.

=item * I<-tag>

Tag from a previous fcp-adapter-topology-list-iter-start.

=item Outputs

Returns a hash refernce which includes number of records and connected FC switch.

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fcp_topology_list_iter_next(
                                                 -maximum => 10,
                                                 -tag     => $tag,
                                    );
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())

=back

=head2 B<fcp_topology_list_iter_end(I<%args>)>

Terminate a list iteration and clean up any saved info.

=over 4

=item Inputs

=item * I<-tag>

Tag from a previous fcp-topology-nameserver-list-iter-start.

=item Outputs

Returns true on success and False on failure

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fcp_topology_list_iter_end(
                                                -tag     => $tag,
                                    );
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())


=back

=head2 B<fcp_zone_list_iter_start(I<%args>)>

Get the active zone set information from the zone server.

=over 4

=item Inputs

=item * I<-adapter_name>

Adapter to get zone information for, it is a optional pamameter
if not specified records of all adapter is listed

=item Outputs

Returns a hash refernce which includes number of records and tag value

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fcp_zone_list_iter_start(-adapter_name => "e0a");
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())

=back

=head2 B<fcp_zone_list_iter_next(I<%args>)>

Returned for each zone entry include the active zone set name,
the zone name, and the zone members.

=over 4

=item Inputs

=item * I<-maximum>

The maximum number of entries to retrieve.

=item * I<-tag>

Tag from a previous fcp-adapter-zone-list-iter-start.

=item Outputs

Returns a hash refernce which includes number of records and List of zone entries

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fcp_zone_list_iter_next(
                                                 -maximum => 10,
                                                 -tag     => $tag,
                                    );
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())

=back

=head2 B<fcp_zone_list_iter_end(I<%args>)>

Terminate a list iteration and clean up any saved info.

=over 4

=item Inputs

=item * I<-tag>

Tag from a previous fcp-adapter-zone-list-iter-start.

=item Outputs

Returns true on success and False on failure

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fcp_zone_list_iter_end(
                                                -tag     => $tag,
                                    );
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())


=back

=head2 B<fc_config_list_iter_start(I<%args>)>

Start iteration through the list of adapter configuration information.

=over 4

=item Outputs

Returns a hash refernce which includes number of records and tag value

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fc_config_list_iter_start();
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())

=back

=head2 B<fc_config_list_iter_next(I<%args>)>

Obtain a list of FC adapter configuration information for adapters
on-board adapters. Does not provide information for add-on adapters.
On-board adapters are generally in slot 0, such as 0a, 0b, etc.

=over 4

=item Inputs

=item * I<-maximum>

The maximum number of entries to retrieve.

=item * I<-tag>

Tag from a previous fc-zone-list-iter-start.

=item Outputs

Returns a hash refernce which includes number of records and list of adapter information

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fc_zone_list_iter_next(
                                                 -maximum => 10,
                                                 -tag     => $tag,
                                    );
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())

=back

=head2 B<fc_config_list_iter_end(I<%args>)>

Terminate a list iteration and clean up any saved info.

=over 4

=item Inputs

=item * I<-tag>

Tag from a previous fc-config-list-iter-start.

=item Outputs

Returns true on success and False on failure

=item Supported Interface:

zapi

=item Examples

    $output_hash = $FilerObj->fc_config_list_iter_end(
                                                -tag     => $tag,
                                    );
    if ($lunObj->error()) {
        $Test->nlog("-- ERROR : " . $lunObj->errmsg());
        return $TCD::FAIL;
    } #end if ($lunObj->error())

=back

=head2 B<fcp_adapter_config_media_type(I<%args>)>

Sets the link type on the Fibre Channel target adapter.

=over 4

=item Inputs

=item * I<-adapter_name>

FC adapter to which media type to be changed.

=item * I<-media_type>

Media type to set. Valid values are point-to-point ("ptp"), loop configuration ("loop"), or automatic ("auto").

=item Outputs

Returns true on success and False on failure

=item Supported Interface:

cli and zapi

=item Examples

    $FilerObj->fcp_adapter_config_media_type(
                                            -adapter_name  => "0b",
                                            -media_type    => "ptp",
                                    );
    if ($FilerObj->error()) {
        $Test->nlog("-- ERROR : " . $FilerObj->errmsg());
        return $TCD::FAIL;
    } #end if ($FilerObj->error())

=back

=head2 B<sysconfig(I<%args>)>

Obtain appliance information which includes cpu and backplane information.

=over 4

=item Inputs

=item * I<-slot>

sysconfig output required to particular slot

=item Outputs

Returns hash on success and False on failure

=item Supported Interface:

cli

=item Examples

    my $hash = $FilerObj->sysconfig();
    if ($FilerObj->error()) {
        $Test->nlog("-- ERROR : " . $FilerObj->errmsg());
        return $TCD::FAIL;
    } #end if ($FilerObj->error())

    or

    my $hash = $FilerObj->sysconfig(-slot => 1);
    if ($FilerObj->error()) {
        $Test->nlog("-- ERROR : " . $FilerObj->errmsg());
        return $TCD::FAIL;
    } #end if ($FilerObj->error())

=back

=head2 B<fc_config_set_adapter_fc_type(I<%args>)>

 bring the specific adapter online

=over 4

=item Inputs

=item * I<-adapter_name>

Specify the adapter which needs to change

=item * I<-fc_type>

Sets the type and state of the adapter

=item Outputs

Returns TRUE or FALSE based on the run.

=item Supported Interface:

ZAPI

=item Examples

    my $result = $FilerObj->fc_config_set_adapter_fc_type(
                                                    -adapter_name => $adapter,
                                                    -fc_type      => "initiator",
                                                    );
    if ($FilerObj->error()) {
        $Test->nlog("-- ERROR : " . $FilerObj->errmsg());
        return $TCD::FAIL;
    } #end if ($FilerObj->error())

=back

=head2 B<fc_config_adapter(I<%args>)>

bring the specific adapter online

=over 4

=item Inputs

=item * I<-adapter_name>

Specify the adapter which needs to enable

=item * I<-state>

Specify the state to which the adapter needs to be changed

=item Outputs

Returns TRUE or FALSE based on the run.

=item Supported Interface:

ZAPI

=item Examples

    my $result = $FilerObj->fc_config_adapter(
                                    -adapter_name => $adapter,
                                    -state        => "enable",
                                );
    if ($FilerObj->error()) {
        $Test->nlog("-- ERROR : " . $FilerObj->errmsg());
        return $TCD::FAIL;
    } #end if ($FilerObj->error())

    or

    my $result = $FilerObj->fc_config_adapter(
                                    -adapter_name => $adapter,
                                    -state        => "disable",
                                );
    if ($FilerObj->error()) {
        $Test->nlog("-- ERROR : " . $FilerObj->errmsg());
        return $TCD::FAIL;
    } #end if ($FilerObj->error())

=back

=head2 B<fcp_stats(I<%args>)>

returns fcp stats

=over 4

=item Inputs

=item * I<-option>

Specify TRUE to zero the fcp stats


=item Outputs

Returns TRUE or FALSE zeroing the stats. and returns hash incase of fcp stats

=item Supported Interface:

CLI

=item Examples

    my $fcp_stats = $FilerObj->fcp_stats();
    if ($FilerObj->error()) {
        $Test->nlog("-- ERROR : " . $FilerObj->errmsg());
        return $TCD::FAIL;
    } #end if ($FilerObj->error())

    or

    # to zero the fcp stats
    my $fcp_stats = $FilerObj->fcp_stats(-option => 1);
    if ($FilerObj->error()) {
        $Test->nlog("-- ERROR : " . $FilerObj->errmsg());
        return $TCD::FAIL;
    } #end if ($FilerObj->error())

=back

=cut
