## @summary SASUtils.pm - library module for various Shasta utilities
##
## @status TESTME
##
## @author anishv@netapp.com
##
## @change 12-01-2006 anishv: Added adapter_force_rediscovery function
## @change 23-07-2009 sreedhar: fixed say() function in set_bootargs method in case promoted build where parameter timeout
## is type error
## @change 19-05-2010 faslur: Added option to pass the path name in make_single_path function
## @change 19-05-2010 faslur: Added get_adpater_name function
## @change 20-10-2010 faslur: Replaced the say() with filer_cmp() to execute the command in Nodescope also
## @change 12-08-2011 gurus: Given C-mode support to filer_cmd() routine.
## @change 27-01-2012 gururajc: Modified the sub device_power_cycle to handle config issue.
## @change 24-02-2012 rajkuma1: Modified the subroutine adapter_reset to handle Hard reset of adapter testcase
## @change 13-02-2012 gururajc: Removed unwanted messages in compare_storage_state sub.
## @change 14-05-2012 gurus@netapp.com: Updated compare_storage_state() with more interactive comments.
## @change 17-05-2012 gurus@netapp.com: Modified device_power_cycle() to support disks parameter.
## @change 08-01-2013 nandeesh: Given C-mode support to abort_filerhammer() and start_filerhammer().
## @change 18-01-2013 nandeesh: Replaced say() with filer_cmd() wherever possible.
## @change 09-07-2013 nandeesh: Modified sub adapter_lip() as per requirement mentioned in burt:716202
## @change 17-07-2013 nandeesh: Modified sub adapter_lip() to support CMode while executing "ucadmin show" command
## @change 23-07-2013 nandeesh: Modified sub get_disks() to collect multiple shelf module types
## @change 03-09-2013 nandeesh: Support for BOAB shelf : burt #548601 in the sub _dev_to_phy_convert()
## @change 26-09-2013 nandeesh: Support for handling the bay level drive notation in Fullsteam in the
##                              sub _dev_to_phy_convert() burt #752713,758584
## @change 27-09-2013 jolie: support in set_filer_options for if someone issues an 'options acp.enabled on' on configs which don't support this BURT 745855
## @change 26-11-2013 nandeesh: Provide support to handle bay level notation of disks in Fullsteam to sub device_power_cycle() see BURT 776131
## @change 28-11-2013 nandeesh: Added support to handle disks in 7Mode setup, see Burt #776131
## @change 09-12-2013 ssrinath: Added support to handle disk names for device_reset subroutine, refer Burt 722057
##

## @pod here

=head1 NAME

SASUtils

=head1 SYNOPSIS

    use SASUtils qw(adapter_reset);

    my $host = host($FILER);

    adapter_reset(host=>$host,...);

=head1 DESCRIPTION
    This is a library module for the Shasta related functions

=head1 ROUTINES

=cut

package Storage::SASUtils;
use strict;
use warnings;

BEGIN {
    use base qw(Exporter);
    our  @EXPORT_OK = qw(
            adapter_reset
            adapter_force_rediscovery
            adapter_zombie
            device_reset
            device_power_cycle
            expander_phy_reset
            expander_phy_mux_reset
            get_shasta_disk_struct
            get_shasta_struct
            get_disks
            ems_error_monitor
            ic_present
            bmc_command
            run_scrub
            start_filerhammer
            abort_filerhammer
            pause_filerhammer
            restart_filerhammer
            set_bootargs
            device_clear_affiliation
            sasutils_scsi_reset
            adapter_lip
            reset_dev
            get_storage_state
            compare_storage_state
            compare_shasta_struct
            compare_storage_and_sasadmin
            compare_sasadmin_dev_stats
            check_shelf_fault
            is_sanown
            is_debug
            disktest
            get_timestamp
            filer_cmd
            panic_filer
            get_coredump_disks
            get_filer_options
            set_filer_options
            get_core_list
            find_core
            savecore
            get_ha_disk_map
            rmfile
            make_single_path
            offline_aggrs
            online_aggrs
            get_aggr_list
            wait_for_sdc_completion
            randomize_array
            unfail_all_disks
            storage_logresult
            log_filer_msg
            get_ra_pair
            disable_dbg_msgs
            phy_disable_enable
            get_adapter_name
        );

    Params::Validate::validation_options(
    on_fail => sub {my @msgs = @_; chomp(@msgs);
        Carp::confess(Tharn::script_result(@msgs))}
   );
};


#use RaidSqa;
use Storage::Ems qw(ems_log_get_mark ems_log_dump_by_type ems_log_dump_by_id ems_event_status ems_log_check);
use Storage::Storage qw( storage_show_disk storage_show_adapter storage_show_expander storage_show_initiators storage_disable_adapter);
use Storage::Fcadmin qw( fcadmin_link_stats fcadmin_reset_adapter fcadmin_reset_dev fcadmin_zombie_adapter);
use Storage::Sasadmin qw( sasadmin_expander sasadmin_expander_phy_state sasadmin_adapter_reset sasadmin_dev_phy_reset sasadmin_expander_phy_mux_reset sasadmin_expander_phy_reset sasadmin_dev_power_cycle sasadmin_expander_phy_dongle_data sasadmin_expander_map sasadmin_adapter_force_rediscovery sasadmin_adapter_zombie sasadmin_dev_phy_clear_affiliation sasadmin_expander_ctrl);
use Storage::Environ qw( environ_status environ_shelf);
use Storage::Scsi qw(scsi_reset);
use Storage::Fcadmin qw(fcadmin_device_map fcadmin_lip fcadmin_bridge_cmd_cemi);
use Storage::Disk qw(disk_show);

use Carp qw(confess croak);
use Params::Validate qw(validate validate_pos ARRAYREF OBJECT SCALAR HASHREF );
use Module::Common qw(validate_mixed);
use Tharn qw(say script_result host logcomment logresult param);

use Utils qw(make_conn);
use Data::Dumper;
use boot;
use RaidInfo;
use NACL::C::Node;
use NACL::CS::SystemNodeHardwareUnifiedConnect;
use NACL::APISet;
use NACL::STask::Node;
use NACL::CS::StorageDisk;
#use raidv2_lib;



our $MAINT_PROMPT = "\\*\>";


=head2 get_disks


    my @ret = get_disks(host => $host, adapter_name=>$adapter,sleep=>$sleep, timeout=>$timeout, filermode=>$filermode,shelf_type=>"DS14-MK2-AT",controller=>"AT-FCX",disk_model=>"SCHT6",raid_type=>"SPARE",aggregate=>"aggr0",adapter_type=>"SAS|FC");

This subrutine returns an array of disks in the system with specific parameters.

The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item adapter_name

(Optional) : List of disks connected to a specific adapter.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item shelf_type

(Optional): List of disks connected to a specific shelf type. For example: DS14-MK2-FC,DS14-MK2-AT, DS14-MK, etc.

=item controller

(Optional): List of disks connected to a specific shelf controller type. For example: LRC, ESH, ESH2,AT-FCX etc.

=item disk_model

(Optional): List of specific model of disks. For example value  X266_MSABR320SSX, will return all 320 Sabre drives in the system.

=item raid_type

(Optional): List of specific raid_type of disks.

Possible values are:

SPARE: List of spare drives in the system.

MY: Disks owned by the filer.

PARTNER: Partner's disks.

FAILED: Failed disks.

AGGREGATE: All filesystem disks.


=item aggregate

(Optional): List of disks in a specific aggregate or traditional volume. For example value of "aggr0" will return all the disks in the aggregate aggr0.

=item adapter_type

(Optional): List of disks on a specific type of adapter. For example: 2322,1068X,FC,"Fibre Channel","SAS", etc. Please check the "storage show adapter" description for a specific adapter.

=back

=cut

sub get_disks
{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            adapter_name => {type=>SCALAR,default=>"all",optional=>1},
            shelf_type => {type=>SCALAR,default=>"all",optional=>1},
            controller => {type=>SCALAR,default=>"all",optional=>1},
            disk_model => {type=>SCALAR,default=>"all",optional=>1},
            raid_type => {type=>SCALAR,default=>"all",optional=>1},
            aggregate => {type=>SCALAR,default=>"all",optional=>1},
            adapter_type => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            }
            );


    if($opts{filermode} ne "normal"){
        $opts{conntype} = "#console";
    }

    my $conn = make_conn($opts{host},$opts{conntype});
    my $filter = 0;#do we want to send back all disks associated with the given host? (then params above should be "all") or do we want something more specific?
    use RaidInfo;
    my $disk;#declare up here so we don't have to keep declaring below
    my @disk_list1;#will use this to filter the option of which disks we want to give back
    my $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});#get all disks and relevant params as a hash
    my @disk_list = keys (%$show_disks);#contains all the disk names, unless we need to filter by params passed in, this is what will ultimately be passed back
    my @disk_ref;#will populate this if we need to filter down certain types of disks
    # filter by disks of a certain raid types or aggregates....
    if(($opts{raid_type} ne "all") || ($opts{aggregate} ne "all")){
    $filter=1;
        my @disk_list2;
    my $aggr;
    my $r = new RaidInfo($conn);
    if ($opts{raid_type} eq "MY"){
        my %disks = $r->my_disks();
        @disk_list2 = _get_disk_array(\%disks);
    }
    elsif($opts{raid_type} eq "PARTNER"){
            my %disks = $r->partner_disks();
        @disk_list2 = _get_disk_array(\%disks);
    }
    elsif($opts{raid_type} eq "FAILED"){
            my %disks = $r->failed_disks();
        @disk_list2 = _get_disk_array(\%disks);
    }
    elsif($opts{raid_type} eq "SPARE"){
        my %disks = $r->spare_disks();
        @disk_list2 = _get_disk_array(\%disks);
    }
    elsif($opts{raid_type} eq "AGGREGATE"){
        my %aggrs = $r->all_aggregates();
            foreach $aggr (values %aggrs){
            my %disks = $aggr->all_disks();
            my @disk_list3 = _get_disk_array(\%disks);
            push @disk_list2, @disk_list3;
        }
    }

    if($opts{aggregate} ne "all"){
        my $agr = $r->aggregate("$opts{aggregate}");
        my %disks = $agr->all_disks();
        @disk_list2 = _get_disk_array(\%disks);
    }
    push @disk_ref, \@disk_list2;
    }


    # Disk on a particular adapter
    if ($opts{adapter_name} ne "all"){
    $filter=1;
        my @disk_list4;

    foreach $disk (@disk_list){
        if(($show_disks->{$disk}->{'SecPath'} =~ /^$opts{adapter}/ ) || ($disk =~ /^$opts{adapter}/)){
            push @disk_list4, $disk;
        }
    }
    push @disk_ref, \@disk_list4;
    }

    # Disk on a certain adapter type
    if ($opts{adapter_type} ne "all"){
        $filter=1;
        my @disk_list8;
        my $adapt;
        my $adapter_info = storage_show_adapter(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype}, timeout=>$opts{timeout});
        my @adapters = keys (%$adapter_info);
        foreach $adapt (@adapters){
            if(($adapter_info->{$adapt}->{'State'} =~ /Enabled/) && ($adapter_info->{$adapt}->{'Description'} =~ /$opts{adapter_type}/)){
                foreach $disk (@disk_list){
                    if(($show_disks->{$disk}->{'SecPath'} =~ /^$adapt/ ) || ($disk =~ /^$adapt/)){
                        push @disk_list8, $disk;
                    }
                }
            }
        }
        push @disk_ref, \@disk_list8;
    }

    # Filter out for disks on a certain shelf type
    if($opts{shelf_type} ne "all"){
    $filter=1;
    my @disk_list5;
    my ($disk, $adapter,@disk_list1);
    #my $environ = environ_status(host=>$opts{host}, filermode=>$opts{filermode});
    #print Dumper($environ);

    foreach $disk (@disk_list){

        my @diskid = split(/\./,$disk);
        $adapter = $diskid[0];
        my $shelfid = $show_disks->{$disk}->{'Shelf'};
        my $environ = environ_shelf(host=>$opts{host}, filermode=>$opts{filermode},adapt=>$adapter);
        my $shelftype = $environ->{$adapter}->{$shelfid}->{'sesproductidentification'};

    if($shelftype =~ /$opts{shelf_type}/){
        push @disk_list5, $disk;
    }
    }
    push @disk_ref, \@disk_list5;
    }

    # Filter out by disks with a certain controller
    if($opts{controller} ne "all"){
        $filter=1;
        my @disk_list6;
        my ($disk, $adapter,@disk_list1);
        #my $environ = environ_status(host=>$opts{host}, filermode=>$opts{filermode});

        foreach $disk (@disk_list){
            my @diskid = split(/\./,$disk);
            $adapter = $diskid[0];
            my $shelfid = $show_disks->{$disk}->{'Shelf'};
            my $environ = environ_shelf(host=>$opts{host}, filermode=>$opts{filermode},adapt=>$adapter);
            my $moduletype = $environ->{$adapter}->{$shelfid}->{'MODULE'};
            my $controller = $opts{controller};
            if($moduletype =~ /$controller/){
                push @disk_list6, $disk;
            }
        }
        push @disk_ref, \@disk_list6;
    }
    # Filter out by disks of a particular model
    # UPDATED: 6/20/13 (simple error: see below
    if($opts{disk_model} ne "all") {
    $filter=1;
    my @disk_list7;
    my $disk;
    my @disk_list1;

    foreach $disk (@disk_list){
        my $modeltype = $show_disks->{$disk}->{'Model'};
            if($opts{disk_model} =~ /$modeltype/){
            push @disk_list7, $disk;
            }
    }
        push @disk_ref, \@disk_list7;
    }

    #if the user did not specify any particular params to filter by, simply return everything
    if (!$filter){
    return @disk_list;
    }
    else{
    my $ref ;
    my @elements = @{$disk_ref[0]};
    foreach $ref (@disk_ref){
        @elements = _get_common_elements($ref,\@elements);
    }
    return @elements;
    }
}#end subroutine



=head2 adapter_reset

    my $ret = sasadmin_adapter_reset(host => $host, adapters=>"$adapter_list",sleep=>$sleep, cycle=>$cycle, timeout=>$timeout, filermode=>$filermode);


This subrutine resets an FC or SAS HBA for n no of cycles.

It returns 1 on success and croaks on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item adapters

(Optional) : A comma or space seperated list of adapters to reset. For example $adapters= "4a 4b 4c" or "4a,4b,4c" is a valid option.

By default it resets all the online adapters.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item cycle

(Optional) : Number of cycles of adapter resets to run. Default is 1.

=item sleep

(Optional) : Sleep time in seconds between each adapter reset. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item fatal_on_failure

(Optional): Stop test on failure. If set to "no" then the test will "fail" instead of fatal. Default is fatal.

=item runlength

(Optional): Length in seconds. Default is "none".

=item emscheck

(Optional): Check ems logs for errors. Default is yes

=item parallel_reset

(Optional): "yes" to reset all adapters in parallel. Default is "no".

=back

=cut

sub adapter_reset
{
my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                        cycle => { type=>SCALAR, default=>1, optional=>1 },
                        interval => { type=>SCALAR, default=>300, optional=>1 },
            adapters => {type=>SCALAR,default=>"all",optional=>1},
            adapter_type => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                    hard_reset => { type=>SCALAR, default=>"no", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            runlength => {type=>SCALAR,default=>"none",optional=>1},
            emscheck => {type=>SCALAR,default=>"yes",optional=>1},
            parallel_reset => {type=>SCALAR,default=>"no",optional=>1},
                                                                                                        }
                                                        );
    my $i;
    my @fc_adapters = ();
    my @sas_adapters = ();
    my $adapt;
    my $fc_adapt;
    my $sas_adapt;
    my @adapters;
    my $ems_check = "no";

    if(($opts{emscheck} eq "yes") && ($opts{filermode} eq "normal")){
        $ems_check = "yes";
    }


    # Get a hash of adapters.
    my $adapter_info = storage_show_adapter(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype}, timeout=>$opts{timeout});
            #print Dumper($adapter_info);
    if($opts{adapters} eq "all"){
         @adapters = keys (%$adapter_info);
    }
    else {
    #Create an array of adapters supplied by the adapters variable.
         @adapters = split(/,|\s+/,$opts{adapters});
    }

    #Now find out online FC and SAS hba and create seperate arrays for FC ans SAS HBA

    foreach $adapt (@adapters){
        if(($adapter_info->{$adapt}->{'State'} =~ /Enabled/) && ($adapter_info->{$adapt}->{'Description'} =~ /^Fibre/)){
            push @fc_adapters,$adapt;
        }
        elsif(($adapter_info->{$adapt}->{'State'} =~ /Enabled/) && ($adapter_info->{$adapt}->{'Description'} =~ /^SAS/)){
            push @sas_adapters,$adapt;
        }
        else{
            if($opts{adapters} ne "all"){
                logresult('fail' ,"Invalid or offline adapter $adapt");
            }
        }
    }
    print "ONLINE FC  adapters:@fc_adapters \n ONLINE SAS adapters:@sas_adapters\n";

    if (($#sas_adapters < 0) && ($#fc_adapters < 0)){
        logresult('script',"No online FC or SAS adapters found");
    }
    my $start_time = time();
    my $elapsed_time;
    my $current_time;
    my $left_time;
    my $mark;

    for($i = 1;$i <= $opts{cycle};$i++)
    {
        logcomment("Starting CYCLE $i of adapter resets.\n");
        if($opts{parallel_reset} eq "yes"){
            if($ems_check eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
            }
        }
        if(($opts{adapter_type} eq "FC") || ($opts{adapter_type} eq "all"))
        {
        foreach $fc_adapt (@fc_adapters){
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }

            logcomment("Resetting FC Adapter $fc_adapt");
            if ($opts{fatal_on_failure} eq "no"){
                eval {
                    fcadmin_reset_adapter(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$fc_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
                };
                if($@){
                    logresult('fail',"Function fcadmin_reset_adapter failed: $@ ");
                }
            }
            else{
            fcadmin_reset_adapter(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$fc_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
            }
            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }


        }
        }
        if(($opts{adapter_type} eq "SAS") || ($opts{adapter_type} eq "all"))
        {
        foreach $sas_adapt (@sas_adapters){
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }

            if ($opts{fatal_on_failure} eq "no"){
                eval {
                    logcomment("Resetting SAS Adapter $sas_adapt");
                    sasadmin_adapter_reset(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$sas_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no",hard_reset=>$opts{hard_reset});
                };
                if($@){
                    logresult('fail',"Function sasadmin_adapter_reset failed: $@ ");
                }
            }
                else{
                    logcomment("Resetting SAS Adapter $sas_adapt");
                    sasadmin_adapter_reset(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$sas_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no",hard_reset=>$opts{hard_reset});
                }
            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }

        }
        }
        if ($opts{runlength} ne "none"){

            $current_time = time();
            $elapsed_time = $current_time - $start_time;
            $left_time = $opts{runlength} - $elapsed_time;
            if ($left_time < 0){
                logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
                #last;
                #exit 0;
                goto AD_RESET_LABEL;
            }
        }
        if($opts{parallel_reset} eq "yes"){
            logcomment("Sleeping for $opts{interval} Secs.");
            sleep $opts{interval};
            if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
            }
        }

    }
    AD_RESET_LABEL: ;

}


=head2 adapter_force_rediscovery

    my $ret = adapter_force_rediscovery(host => $host, adapters=>"$adapter_list",sleep=>$sleep, cycle=>$cycle, timeout=>$timeout, filermode=>$filermode);


This subrutine forces rediscovery on a SAS adapter

It returns 1 on success and croaks on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item adapters

(Optional) : A comma or space seperated list of adapters to rediscover. For example $adapters= "4a 4b 4c" or "4a,4b,4c" is a valid option.

By default it forces rediscovery on all the online adapters.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item cycle

(Optional) : Number of cycles of rediscovery  to run. Default is 1.

=item sleep

(Optional) : Sleep time in seconds between each adapter rediscovery. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item fatal_on_failure

(Optional): Stop test on failure. If set to "no" then the test will "fail" instead of fatal. Default is fatal.

=item runlength

(Optional): Length in seconds. Default is "none".

=item emscheck

(Optional): Check ems logs for errors. Default is yes

=item parallel_reset

(Optional): "yes" to reset all adapters in parallel. Default is "no".

=back

=cut

sub adapter_force_rediscovery
{
my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                        cycle => { type=>SCALAR, default=>1, optional=>1 },
                        interval => { type=>SCALAR, default=>60, optional=>1 },
            adapters => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            runlength => {type=>SCALAR,default=>"none",optional=>1},
            emscheck => {type=>SCALAR,default=>"yes",optional=>1},
            parallel_reset => {type=>SCALAR,default=>"no",optional=>1},
                                                                                                        }
                                                        );
    my $i;
    my @fc_adapters = ();
    my @sas_adapters = ();
    my $adapt;
    my $fc_adapt;
    my $sas_adapt;
    my @adapters;
    my $ems_check = "no";

    if(($opts{emscheck} eq "yes") && ($opts{filermode} eq "normal")){
        $ems_check = "yes";
    }


    # Get a hash of adapters.
    my $adapter_info = storage_show_adapter(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype}, timeout=>$opts{timeout});
            #print Dumper($adapter_info);
    if($opts{adapters} eq "all"){
         @adapters = keys (%$adapter_info);
    }
    else {
    #Create an array of adapters supplied by the adapters variable.
         @adapters = split(/,|\s+/,$opts{adapters});
    }

    #Now find out online FC and SAS hba and create seperate arrays for FC ans SAS HBA

    foreach $adapt (@adapters){
        if(($adapter_info->{$adapt}->{'State'} =~ /Enabled/) && ($adapter_info->{$adapt}->{'Description'} =~ /^SAS/)){
            push @sas_adapters,$adapt;
        }
        else{
            if($opts{adapters} ne "all"){
                logresult('fail' ,"Invalid or offline adapter $adapt");
            }
        }
    }
    print "ONLINE SAS adapters:@sas_adapters\n";

    if ($#sas_adapters < 0){
        logresult('script',"No online SAS adapters found.Adapter rediscovery test is valid only for SAS Adapters");
    }
    my $start_time = time();
    my $elapsed_time;
    my $current_time;
    my $left_time;
    my $mark;

    for($i = 1;$i <= $opts{cycle};$i++)
    {
        logcomment("Starting CYCLE $i of adapter rediscovery.\n");
        if($opts{parallel_reset} eq "yes"){
            if($ems_check eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
            }
        }
        foreach $sas_adapt (@sas_adapters){
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }

            if ($opts{fatal_on_failure} eq "no"){
                eval {
                    logcomment("Forcing Rediscovery on SAS Adapter $sas_adapt");
                    sasadmin_adapter_force_rediscovery(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$sas_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
                };
                if($@){
                    logresult('fail',"Function sasadmin_adapter_force_rediscovery failed: $@ ");
                }
            }
                else{
                    logcomment("Forcing rediscovery on SAS Adapter $sas_adapt");
                    sasadmin_adapter_force_rediscovery(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$sas_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
                }
            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }

        }
        if ($opts{runlength} ne "none"){

            $current_time = time();
            $elapsed_time = $current_time - $start_time;
            $left_time = $opts{runlength} - $elapsed_time;
            if ($left_time < 0){
                logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
                #last;
                #exit 0;
                goto REDISCOVERY_LABEL;
            }
        }
        if($opts{parallel_reset} eq "yes"){
            logcomment("Sleeping for $opts{interval} Secs.");
            sleep $opts{interval};
            if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
            }
        }
    }
    REDISCOVERY_LABEL: ;
}



=head2 adapter_zombie

    my $ret = adapter_zombie(host => $host, adapters=>"$adapter_list",sleep=>$sleep, cycle=>$cycle, timeout=>$timeout, filermode=>$filermode);


This subrutine zombifies an FC or SAS HBA for n no of cycles.

It returns 1 on success and croaks on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item adapters

(Optional) : A comma or space seperated list of adapters to zombifie. For example $adapters= "4a 4b 4c" or "4a,4b,4c" is a valid option.

By default it zombifies all the online adapters.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item cycle

(Optional) : Number of cycles of adapter zombifies to run. Default is 1.

=item sleep

(Optional) : Sleep time in seconds between each adapter zombifie. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item fatal_on_failure

(Optional): Stop test on failure. If set to "no" then the test will "fail" instead of fatal. Default is fatal.

=item runlength

(Optional): Length in seconds. Default is "none".

=item emscheck

(Optional): Check ems logs for errors. Default is yes

=item parallel_reset

(Optional): "yes" to reset all adapters in parallel. Default is "no".

=back

=cut

sub adapter_zombie
{
my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                        cycle => { type=>SCALAR, default=>1, optional=>1 },
                        interval => { type=>SCALAR, default=>300, optional=>1 },
            adapters => {type=>SCALAR,default=>"all",optional=>1},
            adapter_type => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            runlength => {type=>SCALAR,default=>"none",optional=>1},
            emscheck => {type=>SCALAR,default=>"yes",optional=>1},
            parallel_reset => {type=>SCALAR,default=>"no",optional=>1},
                                                                                                        }
                                                        );
    my $i;
    my @fc_adapters = ();
    my @sas_adapters = ();
    my $adapt;
    my $fc_adapt;
    my $sas_adapt;
    my @adapters;
    my $ems_check = "no";

    if(($opts{emscheck} eq "yes") && ($opts{filermode} eq "normal")){
        $ems_check = "yes";
    }


    # Get a hash of adapters.
    my $adapter_info = storage_show_adapter(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype}, timeout=>$opts{timeout});
            #print Dumper($adapter_info);
    if($opts{adapters} eq "all"){
         @adapters = keys (%$adapter_info);
    }
    else {
    #Create an array of adapters supplied by the adapters variable.
         @adapters = split(/,|\s+/,$opts{adapters});
    }

    #Now find out online FC and SAS hba and create seperate arrays for FC ans SAS HBA

    foreach $adapt (@adapters){
        if(($adapter_info->{$adapt}->{'State'} =~ /Enabled/) && ($adapter_info->{$adapt}->{'Description'} =~ /^Fibre/)){
            push @fc_adapters,$adapt;
        }
        elsif(($adapter_info->{$adapt}->{'State'} =~ /Enabled/) && ($adapter_info->{$adapt}->{'Description'} =~ /^SAS/)){
            push @sas_adapters,$adapt;
        }
        else{
            if($opts{adapters} ne "all"){
                logresult('fail' ,"Invalid or offline adapter $adapt");
            }
        }
    }
    print "ONLINE FC  adapters:@fc_adapters \n ONLINE SAS adapters:@sas_adapters\n";

    if (($#sas_adapters < 0) && ($#fc_adapters < 0)){
        logresult('script',"No online FC or SAS adapters found");
    }
    my $start_time = time();
    my $elapsed_time;
    my $current_time;
    my $left_time;
    my $mark;

    for($i = 1;$i <= $opts{cycle};$i++)
    {
        logcomment("Starting CYCLE $i of adapter zombie.\n");
        if($opts{parallel_reset} eq "yes"){
            if($ems_check eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
            }
        }
        if(($opts{adapter_type} eq "FC") || ($opts{adapter_type} eq "all"))
        {
        foreach $fc_adapt (@fc_adapters){
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }

            logcomment("Zombifying FC Adapter $fc_adapt");
            if ($opts{fatal_on_failure} eq "no"){
                eval {
                    fcadmin_zombie_adapter(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$fc_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype});
                };
                if($@){
                    logresult('fail',"Function fcadmin_zombie_adapter failed: $@ ");
                }
            }
            else{
            fcadmin_zombie_adapter(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$fc_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype});
            }
            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }


        }
        }
        if(($opts{adapter_type} eq "SAS") || ($opts{adapter_type} eq "all"))
        {
        foreach $sas_adapt (@sas_adapters){
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }

            if ($opts{fatal_on_failure} eq "no"){
                eval {
                    logcomment("Zombifying SAS Adapter $sas_adapt");
                    sasadmin_adapter_zombie(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$sas_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
                };
                if($@){
                    logresult('fail',"Function sasadmin_adapter_reset failed: $@ ");
                }
            }
                else{
                    logcomment("Zombifying SAS Adapter $sas_adapt");
                    sasadmin_adapter_zombie(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$sas_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
                }
            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }

        }
        }
        if ($opts{runlength} ne "none"){

            $current_time = time();
            $elapsed_time = $current_time - $start_time;
            $left_time = $opts{runlength} - $elapsed_time;
            if ($left_time < 0){
                logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
                #last;
                #exit 0;
                goto ZOM_LABEL;
            }
        }
        if($opts{parallel_reset} eq "yes"){
            logcomment("Sleeping for $opts{interval} Secs.");
            sleep $opts{interval};
            if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
            }
        }

    }
    ZOM_LABEL: ;

}



=head2 device_reset

    device_reset(host=>$host,disks=>$DISKS,cycle=>$CYCLE,interval=>$INTERVAL,filermode=>$FILERMODE,conntype=>$FILER_CONN,fatal_on_failure=>$FATAL_ON_FAILURE,timeout=>$TIMEOUT,runlength=>$runtime);


This subrutine resets  FC or SAS drives for n no of cycles.

It returns 1 on success and croaks on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item disks

(Optional) : A comma or space seperated list of disks to reset. For example $disks= "4a.30  4b.30 4c.30.30  " or "4a.30,4b.30,4c.90.3" is a valid option.

By default it resets all the Drives.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item cycle

(Optional) : Number of cycles of adapter resets to run. Default is 1.

=item interval

(Optional) : Sleep time in seconds between each adapter reset. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "maint"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item fatal_on_failure

(Optional): Stop test on failure. If set to "no" then the test will "fail" instead of fatal. Default is fatal.

=item runlength

(Optional): Length in seconds. Default is "none".

=item emscheck

(Optional): Check ems logs for errors. Default is yes

=item parallel_reset

(Optional): "yes" to reset all devices in parallel. Default is "no".

=back

=cut


sub device_reset {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            cycle => { type=>SCALAR, default=>1, optional=>1 },
            interval => { type=>SCALAR, default=>30, optional=>1 },
            disks => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            runlength => {type=>SCALAR,default=>"none",optional=>1},
            emscheck => {type=>SCALAR,default=>"yes",optional=>1},
            parallel_reset => {type=>SCALAR,default=>"no",optional=>1},
            dualreset => {type=>SCALAR,default=>"0",optional=>1},
            }
            );
    my $i;
    my @fc_disks = ();
    my @sas_disks = ();
    my $disk1;
    my $fc_disk;
    my $sas_disk;
    my @disks;
    my $item;
    my $start_time = time();
    my $elapsed_time;
    my $current_time;
    my $left_time;
    my $ems_check = "no";
    my $mark;
    my $show_disks ;
    my @disk_list ;

    if(($opts{emscheck} eq "yes") && ($opts{filermode} eq "normal")){
        $ems_check = "yes";
    }

    if($opts{disks} eq "all"){
        $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
        @disk_list = keys (%$show_disks);
        @disks = @disk_list;
    }
    else {
        if($opts{dualreset}){
            $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
        }
        @disks = split(/,|\s+/,$opts{disks});
# print "Common Disks : @disks \n";
    }

    foreach $disk1 (@disks){
		my $disk_struct = _dev_to_phy_convert($disk1);
        my $sel_disk = $disk_struct->{$disk1}->{DiskName};
        if($sel_disk =~ /\./){
            if($sel_disk =~ /\.\d+\./){
                push @sas_disks, $sel_disk;
            }
            else{
                push @fc_disks, $sel_disk;
            }
        }
    }
    print "\n SAS DISKS : @sas_disks \n";
    if (($#sas_disks < 0) && ($#fc_disks < 0)){
        logresult('script', "0 matching disks found");
    }


    for($i = 1;$i <= $opts{cycle};$i++)
    {
        logcomment("Starting CYCLE $i of device resets.\n");
        if($opts{parallel_reset} eq "yes"){
            if($ems_check eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
            }
        }
        foreach $fc_disk (@fc_disks){
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }

            logcomment("Resetting FC device $fc_disk");
            if($opts{fatal_on_failure} eq "no"){
                eval {
                    fcadmin_reset_dev(host=>$opts{host},filermode=>$opts{filermode},adp_or_dev_name=>$fc_disk,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
                };
                if($@){
                    logresult('fail',"Function fcadmin_reset_dev  failed: $@ ");
                }
            }
            else{
                fcadmin_reset_dev(host=>$opts{host},filermode=>$opts{filermode},adp_or_dev_name=>$fc_disk,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
            }
            if($opts{dualreset}){
                if(defined($show_disks->{$fc_disk}->{SecPath})){
                    my $sec_disk = $show_disks->{$fc_disk}->{SecPath};
                    logcomment(" dualreset option set . Resetting FC device $sec_disk");
                    if($opts{fatal_on_failure} eq "no"){
                        eval {
                            fcadmin_reset_dev(host=>$opts{host},filermode=>$opts{filermode},adp_or_dev_name=>$sec_disk,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
                        };
                        if($@){
                            logresult('fail',"Function fcadmin_reset_dev  failed: $@ ");
                        }
                    }
                    else{
                            fcadmin_reset_dev(host=>$opts{host},filermode=>$opts{filermode},adp_or_dev_name=>$sec_disk,timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
                    }
                }
                else{
                    logcomment("Disk $fc_disk does not have a secondary path. Cannot do dual reset");
                }
            }

            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                    ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }
            if ($opts{runlength} ne "none"){
                $current_time = time();
                $elapsed_time = $current_time - $start_time;
                $left_time = $opts{runlength} - $elapsed_time;
                if ($left_time < 0){
                    logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
#last;
#exit 0;
                    goto DEV_RESET_LABEL;
                }
            }

        }
        foreach $sas_disk (@sas_disks){
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }
            logcomment("Resetting SAS device $sas_disk");
            if($opts{fatal_on_failure} eq "no"){
                eval {
                    sasadmin_dev_phy_reset(host=>$opts{host},filermode=>$opts{filermode},dev_id=>$sas_disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
                };
                if($@){
                    logresult('fail',"Function sasadmin_dev_phy_reset  failed: $@ ");
                }
            }
            else{
                sasadmin_dev_phy_reset(host=>$opts{host},filermode=>$opts{filermode},dev_id=>$sas_disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
            }
            if($opts{dualreset}){
                if(defined($show_disks->{$sas_disk}->{SecPath})){
                    my $sec_disk = $show_disks->{$sas_disk}->{SecPath};
                    logcomment(" dualreset option set . Resetting SAS device $sec_disk");
                    if($opts{fatal_on_failure} eq "no"){
                        eval {
                            sasadmin_dev_phy_reset(host=>$opts{host},filermode=>$opts{filermode},dev_id=>$sec_disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
                        };
                        if($@){
                            logresult('fail',"Function sasadmin_dev_phy_reset  failed: $@ ");
                        }
                    }
                    else{
                        sasadmin_dev_phy_reset(host=>$opts{host},filermode=>$opts{filermode},dev_id=>$sec_disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
                    }
                }
                else{
                    logcomment("Disk $sas_disk does not have a secondary path. Cannot do dual reset");
                }
            }

            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                    ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }
            if ($opts{runlength} ne "none"){
                $current_time = time();
                $elapsed_time = $current_time - $start_time;
                $left_time = $opts{runlength} - $elapsed_time;
                if ($left_time < 0){
                    logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
#last;
#exit 0;
                    goto DEV_RESET_LABEL;
                }
            }

        }
        if($opts{parallel_reset} eq "yes"){
            logcomment("Sleeping for $opts{interval} Secs.");
            sleep $opts{interval};
            if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
            }
        }
    }
DEV_RESET_LABEL: ;

}


=head2 device_power_cycle

    device_power_cycle(host=>$host,disks=>$DISKS,cycle=>$CYCLE,interval=>$INTERVAL,filermode=>$FILERMODE,conntype=>$FILER_CONN,fatal_on_failure=>$FATAL_ON_FAILURE,timeout=>$TIMEOUT,runlength=>$runtime);


This subrutine power cycles  FC or SAS drives for n no of cycles.

It returns 1 on success and croaks on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item allowDeviceCycleCountOff

(optional) : In this script, we measure the difference of the device power cycle count before and after we issue the power cycle.
Instead of just failing if the count is off, allow of some discrepancy in the count, up to the amount we're specifying with this
variable.  Refer to BURT 717057 for insight in to why this is needed

=item disks

(Optional) : A comma or space seperated list of disks to reset. For example $disks= "4a.30  4b.30 4c.30.30  " or "4a.30,4b.30,4c.90.3" is a valid option.

By default it resets all the Drives.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item cycle

(Optional) : Number of cycles of adapter resets to run. Default is 1.

=item interval

(Optional) : Sleep time in seconds between each adapter reset. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "maint"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item fatal_on_failure

(Optional): Stop test on failure. If set to "no" then the test will "fail" instead of fatal. Default is fatal.

=item runlength

(Optional): Length in seconds. Default is "none".

=item emscheck

(Optional): Check ems logs for errors. Default is yes

=item parallel_reset

(Optional): "yes" to power cycle all devices in parallel. Default is "no".

=item dualreset

(Optional): "1" to reset secondary port at the same time. Default 0


=back

=cut


sub device_power_cycle {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            cycle => { type=>SCALAR, default=>1, optional=>1 },
            interval => { type=>SCALAR, default=>30, optional=>1 },
            disks => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            runlength => {type=>SCALAR,default=>"none",optional=>1},
            emscheck => {type=>SCALAR,default=>"yes",optional=>1},
            parallel_reset => {type=>SCALAR,default=>"no",optional=>1},
            check_count => {type=>SCALAR,default=>"yes",optional=>1},
        allowDeviceCycleCountOff => {type=>SCALAR,default=>1,optional=>1},
            }
            );
    my $i;
    my @fc_disks = ();
    my @sas_disks = ();
    my $disk1;
    my $fc_disk;
    my $sas_disk;
    my @disks;
    my $item;
    my $start_time = time();
    my $elapsed_time;
    my $current_time;
    my $left_time;
    my $ems_check = "no";
    my $mark;
    my $show_disks ;
    my @disk_list ;
    my $allowableOffCount = $opts{allowDeviceCycleCountOff};


    if(($opts{emscheck} eq "yes") && ($opts{filermode} eq "normal")){
        $ems_check = "yes";
    }


    if($opts{disks} eq "all"){
        $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
        @disk_list = keys (%$show_disks);
        @disks = @disk_list;
    } elsif(defined($opts{disks})) {
       @disks = split(/,|\s+/,$opts{disks});
    }
    else {
        $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
        @disk_list = keys (%$show_disks);
        @disks = @disk_list;
    }

    foreach $disk1 (@disks){
        if($disk1 =~ /\.\d+\./){
            push @sas_disks, $disk1;
        }
        else{
            push @fc_disks, $disk1;
        }
    }
    my @esh4_disks ;
    if($#fc_disks >= 0){
        my $moduletype;
        my $shelftype;
        my $environ = environ_status(host=>$opts{host}, filermode=>$opts{filermode});
        foreach my $disk (@fc_disks){
            my @diskid = split(/\./,$disk);
            my $adapter = $diskid[0];
            my $shelfid = $show_disks->{$disk}->{'Shelf'};
            if(defined($environ->{$adapter}->{$shelfid}->{'MODULE'})){
                $moduletype = $environ->{$adapter}->{$shelfid}->{'MODULE'};

                $shelftype = $environ->{$adapter}->{$shelfid}->{'sesproductidentification'};
            }
            else{
                my $disk_sec;
                if(defined($show_disks->{$disk}->{'SecPath'})){
                    $disk_sec = $show_disks->{$disk}->{'SecPath'};
                }
                @diskid = split(/\./,$disk_sec);
                $adapter = $diskid[0];
                $moduletype = $environ->{$adapter}->{$shelfid}->{'MODULE'};
                $shelftype = $environ->{$adapter}->{$shelfid}->{'sesproductidentification'};
            }
            if(($moduletype =~ /ESH4/) && ($shelftype =~ /DS14-Mk4-FC/)){

                push @esh4_disks, $disk;

            }
        }
    }
    my @power_cycle_disks=(@esh4_disks,@sas_disks);
    if(!@power_cycle_disks) {
    logresult('config',"The drives present in the system doesn't support power cycling");
   }

    logcomment("ESH4 + MK4 disks: Count: $#esh4_disks, disk_list: @esh4_disks");

    for($i = 1;$i <= $opts{cycle};$i++)
    {
        logcomment("Starting CYCLE $i of device power cycles.\n");
        if($opts{parallel_reset} eq "yes"){
            if($ems_check eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
            }
        }
# Power cycle of FC drives to be added.
#
        foreach $fc_disk (@esh4_disks){
            logcomment("Power cycling FC device $fc_disk");
            if($opts{fatal_on_failure} eq "no"){
                eval {
                fcadmin_bridge_cmd_cemi(host=>$opts{host},filermode=>$opts{filermode},disk_id=>$fc_disk,timeout=>$opts{timeout},conntype=>$opts{conntype},action=>"pcycle");
                };
                if($@){
                    logresult('fail',"Function fcadmin_bridge_cmd_cemi  failed: $@ ");
                }
            }
            else{
                fcadmin_bridge_cmd_cemi(host=>$opts{host},filermode=>$opts{filermode},disk_id=>$fc_disk,timeout=>$opts{timeout},conntype=>$opts{conntype},action=>"pcycle");
            }

            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                    ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }
            if ($opts{runlength} ne "none"){
                $current_time = time();
                $elapsed_time = $current_time - $start_time;
                $left_time = $opts{runlength} - $elapsed_time;
                if ($left_time < 0){
                    logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
                    goto PCYCLE_LABEL;
                }
            }

        }

        foreach $sas_disk (@sas_disks){
            my $disk_struct = _dev_to_phy_convert($sas_disk);
            my $dongle_data = sasadmin_expander_phy_dongle_data(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$disk_struct->{$sas_disk}->{Adapter},shelf_id=>$disk_struct->{$sas_disk}->{Shelf},phy_num=>$disk_struct->{$sas_disk}->{Phy},timeout=>$opts{timeout},conntype=>$opts{conntype});
            my $powerstatus_before = $dongle_data->{DrivePowerStatus};
            my $cyclecount_before = $dongle_data->{DrivePowerCycleCnt};
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }


            logcomment("Power Cycling SAS device $sas_disk");
            $disk_struct = _dev_to_phy_convert($sas_disk);
            my $sel_disk = $disk_struct->{$sas_disk}->{DiskName};
            logcomment("DEBUG : Power Cycling SAS device $sel_disk");   #retained for debugging purpose only

            if($opts{fatal_on_failure} eq "no"){
                eval {
                    sasadmin_dev_power_cycle(host=>$opts{host},filermode=>$opts{filermode},dev_id=>$sel_disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
                };
                if($@){
                    logresult('fail',"Function sasadmin_dev_power_cycle  failed: $@ ");
                }
            }
            else{
                sasadmin_dev_power_cycle(host=>$opts{host},filermode=>$opts{filermode},dev_id=>$sel_disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
            }


            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                    ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }

                if($opts{check_count}){

                    my $powerstatus_after;
                    my $cyclecount_after;
                    eval {
                        my $dongle_data = sasadmin_expander_phy_dongle_data(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$disk_struct->{$sas_disk}->{Adapter},shelf_id=>$disk_struct->{$sas_disk}->{Shelf},phy_num=>$disk_struct->{$sas_disk}->{Phy},timeout=>$opts{timeout},conntype=>$opts{conntype});
                        $powerstatus_after = $dongle_data->{DrivePowerStatus};
                        $cyclecount_after = $dongle_data->{DrivePowerCycleCnt};
                    };
                    if($@){
                        if($opts{fatal_on_failure} eq "no"){
                            logresult('fail',"Function sasadmin_expander_phy_dongle_data failed: $@");
                        }
                        else{
                            logresult('fatal',"Function sasadmin_expander_phy_dongle_data failed: $@");
                        }
                    }


                    print "CYCLE count after: $cyclecount_after \n";
                    my $cyclecount_diff = $cyclecount_after - $cyclecount_before;
            if($cyclecount_diff > $allowableOffCount) {
#if($powerstatus_before ne $powerstatus_after){
                        if($opts{fatal_on_failure} eq "no"){
                            logresult('fail',"Power cycle count of drive $sel_disk before power cycle was $powerstatus_before and after power cycle is $powerstatus_after \n");
                        }
                        else{
                            logresult('fatal',"Power cycle count of drive $sel_disk before power cycle was $powerstatus_before and after power cycle is $powerstatus_after \n");
                        }

                    }
                }
            }
            if ($opts{runlength} ne "none"){
                $current_time = time();
                $elapsed_time = $current_time - $start_time;
                $left_time = $opts{runlength} - $elapsed_time;
                if ($left_time < 0){
                    logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
#last;
#exit 0;
                    goto PCYCLE_LABEL;
                }
            }
        }
        if($opts{parallel_reset} eq "yes"){
            logcomment("Sleeping for $opts{interval} Secs.");
            sleep $opts{interval};
            if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
            }
        }
    }
PCYCLE_LABEL: ;

}

=head2 expander_phy_reset

    expander_phy_reset(host=>$host,disks=>$DISKS,cycle=>$CYCLE,interval=>$INTERVAL,filermode=>$FILERMODE,conntype=>$FILER_CONN,fatal_on_failure=>$FATAL_ON_FAILURE,timeout=>$TIMEOUT,runlength=>$runtime);

This subroutine resets expander  Phys for n no of cycles.
It returns 1 on success and croaks on failure

The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item disks

(Optional) : A comma or space seperated list of disks to reset. For example $disks= "4a.30  4b.30 4c.30.30  " or "4a.30,4b.30,4c.90.3" is a valid option.
By default it resets all the Drives.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item cycle

(Optional) : Number of cycles of adapter resets to run. Default is 1.

=item interval

(Optional) : Sleep time in seconds between each adapter reset. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "maint"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item fatal_on_failure

(Optional): Stop test on failure. If set to "no" then the test will "fail" instead of fatal. Default is fatal.

=item runlength

(Optional): Run time in seconds. Default is "none".

=item parallel_reset

(Optional): "yes" to  reset all devices in parallel. Default is "no".

=item dualreset

(Optional): "1" to reset secondary port at the same time. Default 0



=back

=cut

sub expander_phy_reset {

    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            cycle => { type=>SCALAR, default=>1, optional=>1 },
            interval => { type=>SCALAR, default=>30, optional=>1 },
            disks => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            runlength => {type=>SCALAR,default=>"none",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            emscheck => {type=>SCALAR,default=>"yes",optional=>1},
            parallel_reset => {type=>SCALAR,default=>"no",optional=>1},
            dualreset => {type=>SCALAR,default=>"0",optional=>1},
            }
            );
    my $i;
    my @fc_disks = ();
    my @sas_disks = ();
    my $disk1;
    my $fc_disk;
    my $sas_disk;
    my @disks;
    my $item;
    my $ems_check = "no";
    my $mark;
    my $start_time = time();
    my $elapsed_time;
    my $current_time;
    my $left_time;
    my $show_disks;

    if(($opts{emscheck} eq "yes") && ($opts{filermode} eq "normal")){
        $ems_check = "yes";
    }

    if($opts{disks} eq "all"){
        $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode});
#my $disk_struct = get_shasta_disk_struct(host=>$opts{host},filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
        @disks = keys (%$show_disks);
    }
    else {
        if($opts{dualreset}){
            $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode});
        }
        @disks = split(/,|\s+/,$opts{disks});
    }

    if($#disks < 0){
        logresult('fatal', "No DS24/ESH4 Drives found. \n");
    }


    for($i = 1;$i <= $opts{cycle};$i++)
    {
        logcomment("Starting CYCLE $i of expander phy resets.\n");
        if($opts{parallel_reset} eq "yes"){
            if($ems_check eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
            }
        }
        foreach $sas_disk (@disks){
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }

            logcomment("Resetting expander phy of  device $sas_disk");
            my $disk_struct = _dev_to_phy_convert($sas_disk);
            if($opts{fatal_on_failure} eq "no"){
                eval {
                    sasadmin_expander_phy_reset(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$disk_struct->{$sas_disk}->{Adapter},shelf_id=>$disk_struct->{$sas_disk}->{Shelf},phy_num=>$disk_struct->{$sas_disk}->{Phy},timeout=>$opts{timeout},conntype=>$opts{conntype});
                };
                if($@){
                    logresult('fail',"Function expander_phy_reset  failed: $@ ");
                }
            }
            else{
                sasadmin_expander_phy_reset(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$disk_struct->{$sas_disk}->{Adapter},shelf_id=>$disk_struct->{$sas_disk}->{Shelf},phy_num=>$disk_struct->{$sas_disk}->{Phy},timeout=>$opts{timeout},conntype=>$opts{conntype});
            }
            if($opts{dualreset}){
                if(defined($show_disks->{$sas_disk}->{SecPath})){
                    my $sec_disk = $show_disks->{$sas_disk}->{SecPath};
                    logcomment("dualreset option set Resetting expander phy of device $sec_disk");
                    my $disk_struct = _dev_to_phy_convert($sec_disk);
                    if($opts{fatal_on_failure} eq "no"){
                        eval {
                            sasadmin_expander_phy_reset(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$disk_struct->{$sec_disk}->{Adapter},shelf_id=>$disk_struct->{$sec_disk}->{Shelf},phy_num=>$disk_struct->{$sec_disk}->{Phy},timeout=>$opts{timeout},conntype=>$opts{conntype});
                        };
                        if($@){
                            logresult('fail',"Function expander_phy_reset  failed: $@ ");
                        }
                    }
                    else{
                        sasadmin_expander_phy_reset(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$disk_struct->{$sec_disk}->{Adapter},shelf_id=>$disk_struct->{$sec_disk}->{Shelf},phy_num=>$disk_struct->{$sec_disk}->{Phy},timeout=>$opts{timeout},conntype=>$opts{conntype});
                    }
                }
                else{
                    logcomment("Secondary path doesnot exist for the drive $sas_disk for dual reset");
                }
            }
            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                    ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }
            if ($opts{runlength} ne "none"){
                $current_time = time();
                $elapsed_time = $current_time - $start_time;
                $left_time = $opts{runlength} - $elapsed_time;
                if ($left_time < 0){
                    logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
#last;
#exit 0;
                    goto PHY_RESET_LABEL;
                }
            }
        }

        if($opts{parallel_reset} eq "yes"){
            logcomment("Sleeping for $opts{interval} Secs.");
            sleep $opts{interval};
            if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
            }
        }
    }
PHY_RESET_LABEL: ;

}

=head2 expander_phy_mux_reset

    expander_phy_mux_reset(host=>$host,disks=>$DISKS,cycle=>$CYCLE,interval=>$INTERVAL,filermode=>$FILERMODE,conntype=>$FILER_CONN,fatal_on_failure=>$FATAL_ON_FAILURE,timeout=>$TIMEOUT,runlength=>$runtime);

This subroutine resets expander muxes for n no of cycles.

It returns 1 on success and croaks on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item disks

(Optional) : A comma or space seperated list of disks to reset. For example $disks= "4a.30  4b.30 4c.30.30  " or "4a.30,4b.30,4c.90.3" is a valid option.

By default it resets all the Drives.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item cycle

(Optional) : Number of cycles of adapter resets to run. Default is 1.

=item interval

(Optional) : Sleep time in seconds between each adapter reset. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "maint"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item fatal_on_failure

(Optional): Stop test on failure. If set to "no" then the test will "fail" instead of fatal. Default is fatal.

=item runlength

(Optional): Run time in seconds. Default is "none".

=item emscheck

(Optional): Check ems logs for errors. Default is yes

=item parallel_reset

(Optional): "yes" to  reset all devices in parallel. Default is "no".


=back

=cut


sub expander_phy_mux_reset {
my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                        cycle => { type=>SCALAR, default=>1, optional=>1 },
                        interval => { type=>SCALAR, default=>300, optional=>1 },
            disks => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            runlength => {type=>SCALAR,default=>"none",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            emscheck => {type=>SCALAR,default=>"yes",optional=>1},
            parallel_reset => {type=>SCALAR,default=>"no",optional=>1},
        }
    );
    my $i;
    my @fc_disks = ();
    my @sas_disks = ();
    my $disk1;
    my $fc_disk;
    my $sas_disk;
    my @disks;
    my $item;
    my $ems_check = "no";
    my $mark;
    my $start_time = time();
    my $elapsed_time;
    my $current_time;
    my $left_time;

    if(($opts{emscheck} eq "yes") && ($opts{filermode} eq "normal")){
        $ems_check = "yes";
    }

    #my $show_disks = storage_show_disk(host=>$host, filermode=>$opts{filermode});
    my $disk_struct = get_shasta_disk_struct(host=>$opts{host},filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
    #print Dumper $disk_struct;
    my @disk_list = grep $disk_struct->{$_}->{DeviceType} eq "SATA", keys (%$disk_struct);
    #print "DISKS : @disk_list \n";
    #my @disk_list = keys (%$disk_struct);
    if($opts{disks} eq "all"){
          @disks = @disk_list;
    }
    else {
            my @disks1 = split(/,|\s+/,$opts{disks});
        my %seen;
        @seen{@disk_list} = ();
        foreach $item (@disks1) {
            if (exists $seen{$item}){
                push(@disks, $item)
            }
            else{
                #logcomment("Disk $item is either not a SATA drive or doesn't exist \n");
        }
    }
    }

    if($#disks < 0){
        logcomment("No SATA Drives found in DS24 enclosure.Skipping the test ");
        goto MUX_RESET_LABEL;
    }


    for($i = 1;$i <= $opts{cycle};$i++)
    {
        logcomment("Starting CYCLE $i of device mux resets.\n");
        if($opts{parallel_reset} eq "yes"){
            if($ems_check eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
            }
        }
        foreach $sas_disk (@disks){
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }

            logcomment("Resetting Mux of  device $sas_disk");
            sasadmin_expander_phy_mux_reset(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$disk_struct->{$sas_disk}->{Adapter},shelf_id=>$disk_struct->{$sas_disk}->{Shelf},phy_num=>$disk_struct->{$sas_disk}->{Phy},timeout=>$opts{timeout},conntype=>$opts{conntype});
            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                    ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }

            if ($opts{runlength} ne "none"){
                $current_time = time();
                $elapsed_time = $current_time - $start_time;
                $left_time = $opts{runlength} - $elapsed_time;
                if ($left_time < 0){
                    logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
                    #last;
                    #exit 0;
                    goto MUX_RESET_LABEL;
                }
            }

        }
        if($opts{parallel_reset} eq "yes"){
            logcomment("Sleeping for $opts{interval} Secs.");
            sleep $opts{interval};
            if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
            }
        }
    }
    MUX_RESET_LABEL: ;

}

=head2 sasutils_scsi_reset

    sasutils_scsi_reset(host=>$host,disks=>$DISKS,cycle=>$CYCLE,interval=>$INTERVAL,filermode=>$FILERMODE,conntype=>$FILER_CONN,fatal_on_failure=>$FATAL_ON_FAILURE,timeout=>$TIMEOUT,runlength=>$runtime);


This subroutine scsi resets  FC or SAS drives for n no of cycles.

It returns 1 on success and croaks on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item disks

(Optional) : A comma or space seperated list of disks to reset. For example $disks= "4a.30  4b.30 4c.30.30  " or "4a.30,4b.30,4c.90.3" is a valid option.

By default it resets all the Drives.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item cycle

(Optional) : Number of cycles of scsi resets to run. Default is 1.

=item interval

(Optional) : Sleep time in seconds between each scsi reset. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "maint"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item fatal_on_failure

(Optional): Stop test on failure. If set to "no" then the test will "fail" instead of fatal. Default is fatal.

=item runlength

(Optional): Length in seconds. Default is "none".

=item emscheck

(Optional): Check ems logs for errors. Default is yes

=item parallel_reset

(Optional): "yes" to reset all devices in parallel. Default is "no".

=item dualreset

(Optional): "1" to reset secondary port at the same time. Default 0

=back

=cut


sub sasutils_scsi_reset {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            cycle => { type=>SCALAR, default=>1, optional=>1 },
            interval => { type=>SCALAR, default=>30, optional=>1 },
            disks => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            runlength => {type=>SCALAR,default=>"none",optional=>1},
            emscheck => {type=>SCALAR,default=>"yes",optional=>1},
            parallel_reset => {type=>SCALAR,default=>"no",optional=>1},
            dualreset => {type=>SCALAR,default=>"0",optional=>1},
            }
            );
    my $i;
    my $disk1;
    my @disks;
    my $item;
    my $start_time = time();
    my $elapsed_time;
    my $current_time;
    my $left_time;
    my $ems_check = "no";
    my $mark;
    my $disk;

    if(($opts{emscheck} eq "yes") && ($opts{filermode} eq "normal")){
        $ems_check = "yes";
    }

    my $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
    my @disk_list = keys (%$show_disks);
    if($opts{disks} eq "all"){
        @disks = @disk_list;
    }
    else {
        my @disks1 = split(/,|\s+/,$opts{disks});
        my %seen;
        @seen{@disk_list} = ();
        foreach $item (@disks1) {
            push(@disks, $item) if exists $seen{$item};
        }
    }

    for($i = 1;$i <= $opts{cycle};$i++)
    {
        logcomment("Starting CYCLE $i of device resets.\n");
        if($opts{parallel_reset} eq "yes"){
            if($ems_check eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
            }
        }
        foreach $disk (@disks){
# Routine is picking up null disks. Add a check to eliminate
# to eliminate null disks.
            next if ($disk =~ /^\s*$/);
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }

            logcomment("scsi reset on $disk");
            if($opts{fatal_on_failure} eq "no"){
                eval {
                    scsi_reset(host=>$opts{host},filermode=>$opts{filermode},device=>$disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
                };
                if($@){
                    logresult('fail',"Function scsi_reset  failed: $@ ");
                }
            }
            else{
                scsi_reset(host=>$opts{host},filermode=>$opts{filermode},device=>$disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
            }
            if($opts{dualreset}){
                if(defined($show_disks->{$disk}->{SecPath})){
                    my $sec_disk = $show_disks->{$disk}->{SecPath};
                    logcomment("dualreset option set  scsi reset on $sec_disk");
                    if($opts{fatal_on_failure} eq "no"){
                        eval {
                            scsi_reset(host=>$opts{host},filermode=>$opts{filermode},device=>$sec_disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
                        };
                        if($@){
                            logresult('fail',"Function scsi_reset  failed: $@ ");
                        }
                    }
                    else{
                        scsi_reset(host=>$opts{host},filermode=>$opts{filermode},device=>$sec_disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
                    }
                }
                else{
                    logcomment("Secondary path not found for dual reset on $disk");
                }
            }
            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                    ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }
            if ($opts{runlength} ne "none"){
                $current_time = time();
                $elapsed_time = $current_time - $start_time;
                $left_time = $opts{runlength} - $elapsed_time;
                if ($left_time < 0){
                    logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
#last;
#exit 0;
                    goto SCSI_RESET_LABEL;
                }
            }

        }
        if($opts{parallel_reset} eq "yes"){
            logcomment("Sleeping for $opts{interval} Secs.");
            sleep $opts{interval};
            if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
            }
        }
    }
SCSI_RESET_LABEL: ;

}

=head2 adapter_lip

    my $ret = adapter_lip(host => $host, adapters=>"$adapter_list",sleep=>$sleep, cycle=>$cycle, timeout=>$timeout, filermode=>$filermode);


This subrutine resets an FC or SAS HBA for n no of cycles.

It returns 1 on success and croaks on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item adapters

(Optional) : A comma or space seperated list of adapters to reset. For example $adapters= "4a 4b 4c" or "4a,4b,4c" is a valid option.

By default it resets all the online adapters.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item cycle

(Optional) : Number of cycles of adapter resets to run. Default is 1.

=item sleep

(Optional) : Sleep time in seconds between each adapter reset. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "MAINT"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item fatal_on_failure

(Optional): Stop test on failure. If set to "no" then the test will "fail" instead of fatal. Default is fatal.

=item runlength

(Optional): Length in seconds. Default is "none".

=item emscheck

(Optional): Check ems logs for errors. Default is yes

=item parallel_reset

(Optional): "yes" to reset all adapters in parallel. Default is "no".

=back

=cut

sub adapter_lip
{
my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                        cycle => { type=>SCALAR, default=>1, optional=>1 },
                        interval => { type=>SCALAR, default=>300, optional=>1 },
            adapters => {type=>SCALAR,default=>"all",optional=>1},
            adapter_type => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            runlength => {type=>SCALAR,default=>"none",optional=>1},
            emscheck => {type=>SCALAR,default=>"yes",optional=>1},
            parallel_reset => {type=>SCALAR,default=>"no",optional=>1},
                                                                                                        }
                                                        );
    my $i;
    my $adapt;
    my $cnt = 0;
    my @adapters = ();
    my @fcadapters;
    my $ems_check = "no";
    my %ht = ();

    if(($opts{emscheck} eq "yes") && ($opts{filermode} eq "normal")){
        $ems_check = "yes";
    }
    my $host = host($opts{host});
    # Get a hash of adapters.
    my $adapter_info = fcadmin_device_map(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype}, timeout=>$opts{timeout});
    if($opts{adapters} eq "all"){
        @fcadapters = keys (%$adapter_info);
        foreach my $adapters (@fcadapters) {
            if ($adapter_info->{$adapters}->{'State'} =~ /UP/) {
                push(@adapters,$adapters);
                $cnt = 1;
            }
        }
        #Burt : 716202 : Need to modify fcsas_errinj.thpl (testcase #10 -> sub adapter_lip) to detect online fc ports. Currently "fcadmin device_map" is being used
        #to detect online FC ports and CNA ports won't show up in that output. By executing the command "ucadmin show" we can list out CNA ports
        if(!$cnt){
            @fcadapters = ();
            if ($host->hosttype =~ /filer-ng/ ) {   #On C-mode, ucadmin should be ran from cluster prompt ::> and NOT from run local which the script is using.
                                                    #Therefore, it can't get active ports to run. Author: locl  Ref to Burt : 716202
                my $filer_ci = NACL::C::Node->find();
                my @arr = NACL::CS::SystemNodeHardwareUnifiedConnect->fetch( command_interface => $filer_ci,
                                                         filter => { 'status' => 'online', 'node' => $host->hostname() },
                                                         requested_fields => [ 'adapter' ],
                                                );
                foreach (@arr) {
                    push(@adapters,$_->{'adapter'});
                }
            } else {
                my $cmd = "ucadmin show";
                my $output = filer_cmd(host=>$opts{host},timeout=>$opts{timeout},cmd=>$cmd);
                if ($output =~ /ucadmin not found/) {
                    logresult('fail',"command ucadmin show failed: $output");
                }
                my @lines = split(/\r*\n/, $output);
                shift(@lines);
                shift(@lines);
                shift(@lines);
                foreach (@lines) {
                    if($_ =~ /Warning|Timed out|entries were displayed/) { next; }
                    if ($_ =~ /\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) {
                        $ht{$1}->{adapter}=$1;
                        $ht{$1}->{current_mode}=$2;
                        $ht{$1}->{current_type}=$3;
                        $ht{$1}->{pending_mode}=$4;
                        $ht{$1}->{pending_type}=$5;
                        $ht{$1}->{status}=$6;
                        push (@fcadapters, $1);
                    }
                }
                $adapter_info = \%ht;
                foreach my $adapters (@fcadapters) {
                    if ($adapter_info->{$adapters}->{'status'} =~ /online/i) {
                        push(@adapters,$adapters);
                    }
                }
            }
        }
    } else {
        #Create an array of adapters supplied by the adapters variable.
        @adapters = split(/,|\s+/,$opts{adapters});
    }
    print "\nADAPTERS: @adapters \n";
    #Now find out online FC and SAS hba and create seperate arrays for FC ans SAS HBA

    if ($#adapters < 0){
        logresult('script',"No online FC adapters found in node $host->hostname() ");
    }
    my $start_time = time();
    my $elapsed_time;
    my $current_time;
    my $left_time;
    my $mark;
    my $fc_adapt;

    for($i = 1;$i <= $opts{cycle};$i++)
    {
        logcomment("Starting CYCLE $i of adapter lips.\n");
        if($opts{parallel_reset} eq "yes"){
            if($ems_check eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
            }
        }
        foreach $fc_adapt (@adapters){
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }

            logcomment("Resetting FC Adapter $fc_adapt");
            if ($opts{fatal_on_failure} eq "no"){
                eval {
                    fcadmin_lip(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$fc_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype});
                };
                if($@){
                    logresult('fail',"Function fcadmin_reset_adapter failed: $@ ");
                }
            }
            else{
            fcadmin_lip(host=>$opts{host},filermode=>$opts{filermode},adapter_name=>$fc_adapt,timeout=>$opts{timeout},conntype=>$opts{conntype});
            }
            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }


        }
        if ($opts{runlength} ne "none"){

            $current_time = time();
            $elapsed_time = $current_time - $start_time;
            $left_time = $opts{runlength} - $elapsed_time;
            if ($left_time < 0){
                logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
                #last;
                #exit 0;
                goto LIP_LABEL;
            }
        }
        if($opts{parallel_reset} eq "yes"){
            logcomment("Sleeping for $opts{interval} Secs.");
            sleep $opts{interval};
            if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
            }
        }

    }
    LIP_LABEL: ;

}

=head2 reset_dev

    sasutils_scsi_reset(host=>$host,disks=>$DISKS,cycle=>$CYCLE,interval=>$INTERVAL,filermode=>$FILERMODE,conntype=>$FILER_CONN,fatal_on_failure=>$FATAL_ON_FAILURE,timeout=>$TIMEOUT,runlength=>$runtime);


This subroutine scsi resets  FC or SAS drives for n no of cycles.

It returns 1 on success and croaks on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item disks

(Optional) : A comma or space seperated list of disks to reset. For example $disks= "4a.30  4b.30 4c.30.30  " or "4a.30,4b.30,4c.90.3" is a valid option.

By default it resets all the Drives.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item cycle

(Optional) : Number of cycles of scsi resets to run. Default is 1.

=item interval

(Optional) : Sleep time in seconds between each scsi reset. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "maint"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item fatal_on_failure

(Optional): Stop test on failure. If set to "no" then the test will "fail" instead of fatal. Default is fatal.

=item runlength

(Optional): Length in seconds. Default is "none".

=item emscheck

(Optional): Check ems logs for errors. Default is yes

=item parallel_reset

(Optional): "yes" to reset all devices in parallel. Default is "no".

=back

=cut


sub reset_dev {
my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                        cycle => { type=>SCALAR, default=>1, optional=>1 },
                        interval => { type=>SCALAR, default=>30, optional=>1 },
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            runlength => {type=>SCALAR,default=>"none",optional=>1},
            emscheck => {type=>SCALAR,default=>"yes",optional=>1},
        }
    );
    my $i;
    my $disk1;
    my @disks;
    my $item;
    my $start_time = time();
    my $elapsed_time;
    my $current_time;
    my $left_time;
    my $ems_check = "no";
    my $mark;
    my $disk;
    my @fc_disks;
    my $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
    my @disk_list = keys (%$show_disks);

    foreach $disk1 (@disk_list){
        if($disk1 =~ /\./){
            unless($disk1 =~ /\.\d+\./){
                push @fc_disks, $disk1;
            }
        }
    }
    if ($#fc_disks < 0){
        logresult('script', "0 FC disks found");
    }


    if(($opts{emscheck} eq "yes") && ($opts{filermode} eq "normal")){
        $ems_check = "yes";
    }

    for($i = 1;$i <= $opts{cycle};$i++)
    {
        logcomment("Starting CYCLE $i of fcadmin reset_dev.\n");
            if($ems_check eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
            }
            if($opts{fatal_on_failure} eq "no"){
                eval {
                    fcadmin_reset_dev(host=>$opts{host},filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
                };
                if($@){
                    logresult('fail',"Function fcadmin_reset_dev  failed: $@ ");
                }
            }
            else{
                    fcadmin_reset_dev(host=>$opts{host},filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype},ems_verify=>"no");
            }
            if ($opts{runlength} ne "none"){
                $current_time = time();
                $elapsed_time = $current_time - $start_time;
                $left_time = $opts{runlength} - $elapsed_time;
                if ($left_time < 0){
                    logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
                    #last;
                    #exit 0;
                    goto RESET_DEV_LABEL;
                }
            }

            logcomment("Sleeping for $opts{interval} Secs.");
            sleep $opts{interval};
            if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
            }
    }
    RESET_DEV_LABEL: ;

}




sub get_shasta_disk_struct {
    my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
            disks => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
    }
    );

    my %bay_phy_map = (
            '0' => '8',
            '1' => '9',
            '2' => '10',
            '3' => '11',
            '4' => '12',
            '5' => '13',
            '6' => '14',
            '7' => '15',
            '8' => '16',
            '9' => '17',
            '10' => '18',
            '11' => '19',
            '12' => '20',
            '13' => '21',
            '14' => '22',
            '15' => '23',
            '16' => '24',
            '17' => '25',
            '18' => '26',
            '19' => '27',
            '20' => '28',
            '21' => '29',
            '22' => '30',
            '23' => '31',
    );
    my @sas_adapters;
    my $adapt;
    my $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
    my @disk_list = keys (%$show_disks);
    my $disk1;
    my @sas_disks;
    my @disk_split;
    my %disk_struct = ();
    my $expander_hash ;
    my $expander_phy_state ;
    #my $expander_hash = sasadmin_expander(host=>$opts{host}, filermode=>$opts{filermode});
    my $adapter_check = " ";
    my $shelf_check = " ";
    my $shelfuid;
    my $shelf1;
    my $channel1;
    foreach $disk1 (@disk_list){
        if($disk1 =~ /\.\d+\./){
            push @sas_disks, $disk1;
            @disk_split = split(/\./,$disk1);
            $disk_struct{$disk1}{Adapter} = $disk_split[0];
            $disk_struct{$disk1}{Shelf} = $disk_split[1];
            $disk_struct{$disk1}{Bay} = $disk_split[2];
            $disk_struct{$disk1}{Phy} = $bay_phy_map{$disk_split[2]};
            $shelf1 = $disk_struct{$disk1}{Shelf};
            $shelf1 =~ s/^0//;
                my $channel = "$disk_split[0]" . "\." . "$disk_split[1]";
                my $channel1 = "$disk_split[0]" . "\." . "$shelf1";
            if (($disk_split[0] ne "$adapter_check") && ($disk_split[1] ne "$shelf_check")){

                $adapter_check = $disk_split[0];
                $shelf_check = $disk_split[1];
                $expander_hash = sasadmin_expander(host=>$opts{host}, filermode=>$opts{filermode},adapter_name=>$disk_split[0],shelf_id=>$disk_split[1],timeout=>$opts{timeout},conntype=>$opts{conntype});
                 $expander_phy_state = sasadmin_expander_phy_state(host=>$opts{host}, filermode=>$opts{filermode},adapter_name=>$disk_split[0],shelf_id=>$disk_split[1],timeout=>$opts{timeout},conntype=>$opts{conntype});

        }
                $disk_struct{$disk1}{WWN} = $expander_hash->{$channel1}->{$disk_struct{$disk1}{Phy}}->{WWN};
                $disk_struct{$disk1}{ShelfWWN} = $expander_hash->{$channel1}->{WWN};

                $disk_struct{$disk1}{Rate} = $expander_hash->{$channel1}->{$disk_struct{$disk1}{Phy}}->{Rate};
                $disk_struct{$disk1}{DeviceType} = $expander_hash->{$channel1}->{$disk_struct{$disk1}{Phy}}->{DeviceType};
                $disk_struct{$disk1}{InvalidDWordCount} = $expander_phy_state->{$channel}->{$disk_struct{$disk1}{Phy}}->{InvalidDWordCount};
                $disk_struct{$disk1}{PhyResetProb} = $expander_phy_state->{$channel}->{$disk_struct{$disk1}{Phy}}->{PhyResetProb};
                $disk_struct{$disk1}{PhyChangeCountS} = $expander_phy_state->{$channel}->{$disk_struct{$disk1}{Phy}}->{PhyChangeCountS};
                $disk_struct{$disk1}{PhyChangeCountH} = $expander_phy_state->{$channel}->{$disk_struct{$disk1}{Phy}}->{PhyChangeCountH};
                $disk_struct{$disk1}{SATAAffiliation} = $expander_phy_state->{$channel}->{$disk_struct{$disk1}{Phy}}->{SATAAffiliation};
                $disk_struct{$disk1}{LossSyncCount} = $expander_phy_state->{$channel}->{$disk_struct{$disk1}{Phy}}->{LossSyncCount};
                $disk_struct{$disk1}{InConCRCError} = $expander_phy_state->{$channel}->{$disk_struct{$disk1}{Phy}}->{InConCRCError};
                $disk_struct{$disk1}{Scrubcount} = $show_disks->{$disk1}->{Scrubcount};
                $disk_struct{$disk1}{PriPort} = $show_disks->{$disk1}->{PriPort};
                $disk_struct{$disk1}{Vendor} = $show_disks->{$disk1}->{Vendor};
                $disk_struct{$disk1}{PowerOnHours} = $show_disks->{$disk1}->{PowerOnHours};
                $disk_struct{$disk1}{Scrublastdone} = $show_disks->{$disk1}->{Scrublastdone};
                $disk_struct{$disk1}{RPM} = $show_disks->{$disk1}->{RPM};
                $disk_struct{$disk1}{SecPort} = $show_disks->{$disk1}->{SecPort};
                $disk_struct{$disk1}{Downrev} = $show_disks->{$disk1}->{Downrev};
                $disk_struct{$disk1}{Model} = $show_disks->{$disk1}->{Model};
                $disk_struct{$disk1}{Serial} = $show_disks->{$disk1}->{Serial};
                $disk_struct{$disk1}{GlistCount} = $show_disks->{$disk1}->{GlistCount};
                $disk_struct{$disk1}{Dynamicallyqualified} = $show_disks->{$disk1}->{Dynamicallyqualified};
                $disk_struct{$disk1}{BlkWritten} = $show_disks->{$disk1}->{BlkWritten};
                $disk_struct{$disk1}{SecPath} = $show_disks->{$disk1}->{SecPath};
                $disk_struct{$disk1}{BlkRead} = $show_disks->{$disk1}->{BlkRead};
                $disk_struct{$disk1}{Rev} = $show_disks->{$disk1}->{Rev};


    }

}
#print Dumper (\%disk_struct);
return (\%disk_struct);
}


=head2 get_shasta_struct

    get_shasta_struct(host=>$host,filermode=>$FILERMODE,conntype=>$FILER_CONN,timeout=>$TIMEOUT);

This function collects data from "sasadmin expander", "sasadmin expander_phy_state", "storage show expander" commands and put into a reference keyed with "Adapter.shelf.phy_number".

It returns a reference on success and croaks on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item timeout

(Optional): timeout value. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "maint"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=back

=cut


sub get_shasta_struct {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            debug => { type=>SCALAR, default=>0, optional=>1 },
            }
            );

    my %serial;
    my $sid = {};
    my $sid1;
    my $storage_expander = storage_show_expander(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype},timeout=>$opts{timeout});
    if($opts{debug}){
        logcomment("DEBUG: Storage show expander hash");
        print Dumper($storage_expander);
    }
    my $shelf;

    my @expans = keys %{$storage_expander};
    foreach my $expans (@expans){
        my @ports = keys %{$storage_expander->{$expans}};
        foreach my $ports (@ports){
            $shelf = $storage_expander->{$expans}->{$ports}->{Shelfid};
            $sid->{$shelf}->{uid} = $expans;
            $serial{$storage_expander->{$expans}->{$ports}->{Serial}} = $expans;
            if($opts{debug}){
                print " EXPANS: $expans PORTS: $ports and Shelfid is $shelf \n";
            }
        }
    }



    my $expander_map = sasadmin_expander_map(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype},timeout=>$opts{timeout});
    if($opts{debug}){
        print "Expander map \n";
        print Dumper($expander_map);
    }
#sleep 10000000;
    my %phy_struct = ();
    my $expander_hash ;
    my $expander_phy_state ;
    my $adapt;
    my $wwn;
    my $id;
    my $id1;
    my $did;
    my $slot;
    my $slot1;
    my $phy_main;

    my @adapters = keys %{$expander_map};
#print "ADAPTERS: @adapters \n";
    foreach $adapt (@adapters){
        my @wwns = keys %{$expander_map->{$adapt}};
        foreach $wwn (@wwns){
            $id = $expander_map->{$adapt}->{$wwn}->{ID};
            $slot = $expander_map->{$adapt}->{$wwn}->{Slot};

            if ($slot eq "A"){
                $slot1 = "B";
            }
            elsif($slot eq "B"){
                $slot1 = "A";
            }
            else{
                logresult('fatal',"Slot is neither A nor B it\'s $slot");
            }

            $expander_hash = sasadmin_expander(host=>$opts{host}, filermode=>$opts{filermode},adapter_name=>$adapt,shelf_id=>$id,timeout=>$opts{timeout},conntype=>$opts{conntype},timeout=>$opts{timeout});
            $expander_phy_state = sasadmin_expander_phy_state(host=>$opts{host}, filermode=>$opts{filermode},adapter_name=>$adapt,shelf_id=>$id,timeout=>$opts{timeout},conntype=>$opts{conntype},timeout=>$opts{timeout});
            if($opts{debug}){
                print"Now it is showing the information of sasadmin expander_phy_state\n";
                print Dumper($expander_phy_state);
            }
#sleep 10000;

            my $channel = "$adapt" . "." . "$id";
#print "CHANNEL $channel \n";
            my @exp_hash = keys %{$expander_hash->{$channel}};
#print Dumper($expander_hash);
#print "PHYS: @exp_hash \n";
            foreach my $phy_id (@exp_hash){
                if($phy_id =~ /^\d+$/){

                    if($id =~ /^\d$/){
                        $id1 = "0" . "$id";
                    }
                    else{
                        $id1 = $id;
                    }
                    $phy_main = "$adapt" . "." . "$id1" . "." . "$phy_id";
                    if (($phy_id >= 8) && ($phy_id <= 31)){
                        $did = $phy_id - 8;
                    }
                    elsif(($phy_id >= 0) && ($phy_id <= 3) && ($expander_map->{$adapt}->{$wwn}->{Product} =~ /-SAS/)){
                        $did = "IN" . "$phy_id";
                    }
                    elsif(($phy_id >= 4) && ($phy_id <= 7) && ($expander_map->{$adapt}->{$wwn}->{Product} =~ /-SAS/)){
                        my $out = $phy_id - 4;
                        $did = "OUT" . "$out";
                    }
                    elsif(($phy_id >= 0) && ($phy_id <= 3) && ($expander_map->{$adapt}->{$wwn}->{Product} =~ /IOM3|IOM6|PCM6/)){
                        $did = "SQR" . "$phy_id";
                    }
                    elsif(($phy_id >= 4) && ($phy_id <= 7) && ($expander_map->{$adapt}->{$wwn}->{Product} =~ /IOM3|IOM6|PCM6/)){
                        $did = "CIR" . "$phy_id";
                    }
                    elsif($phy_id == 36) {
                        $did = "VIR" . "$phy_id";
                    }
                    elsif(($phy_id >= 32) && ($phy_id < 36 )) {
                        $did = "SIL" . "$phy_id";

                    }
                    else{
                        logresult('fatal',"Invalid phy_id $phy_id");
                    }

                    $phy_struct{$phy_main}{ShelfWWN} = $wwn;
#my @sid_array = keys%{$sid};
                    my $shelfuid;
                    my ($k,$v);

                    while(($k,$v) = each(%serial)){
                        if($k eq $expander_map->{$adapt}->{$wwn}->{SerialNumber}){
                            $shelfuid = $v;
                        }
                    }
                    unless(defined($shelfuid)){
                        my @sid_array = keys%{$sid};
                        foreach $sid1 (@sid_array) {
                            if ($sid1 eq $channel) {
                                $shelfuid = $sid->{$sid1}->{uid};
                            }
                        }
                    }

                    $phy_struct{$phy_main}{Module} = $slot;
                    $phy_struct{$phy_main}{DeviceType} = $expander_hash->{$channel}->{$phy_id}->{DeviceType};
                    $phy_struct{$phy_main}{Rate} = $expander_hash->{$channel}->{$phy_id}->{Rate};
                    $phy_struct{$phy_main}{DiskId} = $expander_hash->{$channel}->{$phy_id}->{DiskId};
#Output from "sasadmin expander_phy_state"
                    $phy_struct{$phy_main}{InvalidDWordCount} = $expander_phy_state->{$channel}->{$phy_id}->{InvalidDWordCount};

                    $phy_struct{$phy_main}{DongleFWRev} = $expander_phy_state->{$channel}->{$phy_id}->{DongleFWRev};
                    $phy_struct{$phy_main}{PwrON} = $expander_phy_state->{$channel}->{$phy_id}->{PwrON};
                    $phy_struct{$phy_main}{PwrCclCnt} = $expander_phy_state->{$channel}->{$phy_id}->{PwrCclCnt};
                    $phy_struct{$phy_main}{DongleType} = $expander_phy_state->{$channel}->{$phy_id}->{DongleType};
                    $phy_struct{$phy_main}{PhyResetProb} = $expander_phy_state->{$channel}->{$phy_id}->{PhyResetProb};
                    $phy_struct{$phy_main}{PhyChangeCnt} = $expander_phy_state->{$channel}->{$phy_id}->{PhyChangeCnt};
                    $phy_struct{$phy_main}{LossSyncCount} = $expander_phy_state->{$channel}->{$phy_id}->{LossSyncCount};
                    $phy_struct{$phy_main}{CRCErrorCnt} = $expander_phy_state->{$channel}->{$phy_id}->{CRCErrorCnt};
                    $phy_struct{$phy_main}{RsvCnt} = $expander_phy_state->{$channel}->{$phy_id}->{RsvCnt};
                    $phy_struct{$phy_main}{STATE} = $expander_phy_state->{$channel}->{$phy_id}->{STATE};
                    $phy_struct{$phy_main}{DisparityErrorCount} = $expander_phy_state->{$channel}->{$phy_id}->{DisparityErrorCount};
                    $phy_struct{$phy_main}{RlsCnt} = $expander_phy_state->{$channel}->{$phy_id}->{RlsCnt};
                    $phy_struct{$phy_main}{SATAAffiliation} = $expander_phy_state->{$channel}->{$phy_id}->{SATAAffiliation};

# Output from "storage show expander"
                    if($opts{debug}){
                        print "\n==============\nAdapter: $adapt DID: $did  SLOT $slot SLOT1 $slot1 SUID $shelfuid  WWN $wwn PHYNUM $phy_id\n==================\n";
                        print "Storage show expander \n";
                        print Dumper $storage_expander->{$shelfuid} ;
                        print "Storage show expander \n";
                        print Dumper $storage_expander->{$shelfuid}->{$slot}->{$did};
                        print "\n Expander phy state \n";
                        print Dumper $expander_phy_state->{$channel}->{$phy_id};
                        print "\n Expander hash \n";
                        print Dumper $expander_hash->{$channel}->{$phy_id};
                    }
#       print $storage_expander->{$shelfuid}->{$slot}->{$did}->{LinkRate} ;
#   print "\n";

                    $phy_struct{$phy_main}{PartialPathTimeout} = $storage_expander->{$shelfuid}->{$slot}->{$did}->{PartialPathTimeout};
                    $phy_struct{$phy_main}{PortState} = $storage_expander->{$shelfuid}->{$slot}->{$did}->{PortState};
                    $phy_struct{$phy_main}{InvalidWordCount} = $storage_expander->{$shelfuid}->{$slot}->{$did}->{InvalidWordCount};
                    $phy_struct{$phy_main}{PhyChangeCount} = $storage_expander->{$shelfuid}->{$slot}->{$did}->{PhyChangeCount};
                    $phy_struct{$phy_main}{RunningDisparityCount} = $storage_expander->{$shelfuid}->{$slot}->{$did}->{RunningDisparityCount};
                    $phy_struct{$phy_main}{LinkRate} = $storage_expander->{$shelfuid}->{$slot}->{$did}->{LinkRate};
                    $phy_struct{$phy_main}{LossDwordCount} = $storage_expander->{$shelfuid}->{$slot}->{$did}->{LossDwordCount};
                    $phy_struct{$phy_main}{CRCErrorCount} = $storage_expander->{$shelfuid}->{$slot}->{$did}->{CRCErrorCount};
                    $phy_struct{$phy_main}{PhyResetProblem} = $storage_expander->{$shelfuid}->{$slot}->{$did}->{PhyResetProblem};
                    $phy_struct{$phy_main}{Channel} = $storage_expander->{$shelfuid}->{$slot}->{Channel};

# Output from "storage show expander o/p of other Module in the shelf"
                    $phy_struct{$phy_main}{PartialPathTimeout2} = $storage_expander->{$shelfuid}->{$slot1}->{$did}->{PartialPathTimeout};
                    $phy_struct{$phy_main}{PortState2} = $storage_expander->{$shelfuid}->{$slot1}->{$did}->{PortState};
                    $phy_struct{$phy_main}{InvalidWordCount2} = $storage_expander->{$shelfuid}->{$slot1}->{$did}->{InvalidWordCount};
                    $phy_struct{$phy_main}{PhyChangeCount2} = $storage_expander->{$shelfuid}->{$slot1}->{$did}->{PhyChangeCount};
                    $phy_struct{$phy_main}{RunningDisparityCount2} = $storage_expander->{$shelfuid}->{$slot1}->{$did}->{RunningDisparityCount};
                    $phy_struct{$phy_main}{LinkRate2} = $storage_expander->{$shelfuid}->{$slot1}->{$did}->{LinkRate};
                    $phy_struct{$phy_main}{LossDwordCount2} = $storage_expander->{$shelfuid}->{$slot1}->{$did}->{LossDwordCount};
                    $phy_struct{$phy_main}{CRCErrorCount2} = $storage_expander->{$shelfuid}->{$slot1}->{$did}->{CRCErrorCount};
                    $phy_struct{$phy_main}{PhyResetProblem2} = $storage_expander->{$shelfuid}->{$slot1}->{$did}->{PhyResetProblem};
                    $phy_struct{$phy_main}{Channel2} = $storage_expander->{$shelfuid}->{$slot1}->{Channel};



                }
            }
        }
    }
#print Dumper(\%phy_struct);
#sleep 1000000;
    return (\%phy_struct);
}

=head2 ems_error_monitor

    my @ret = ems_error_monitor(host=>$host, mark=>$mark, filermode=>$filermode,monitor_time=>$monitor_time,sleep_time=>$sleep_time,error_values=>\@errors,fatal_on_failure=>"yes");

This subrutine checks for ems errors

The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item mark

(Required) : Time marker from where to start checking the errors

=item monitor_time

(Optional): Monitor time. Default is 1 sec.

=item sleep_time

(Optional): Sleep time between error checks. Default is 1 sec.

=item error_values

(Optional): Reference to an Array with a list of errors to be checked. By default it will use a list defined in the library

=item fatal_on_failure

(Optional): To stop test on seeing errors? Default is yes.

=back

=cut



sub ems_error_monitor {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                mark => {type=>SCALAR},
                monitor_time => { type=>SCALAR,default=>1,optional=>1 },
                sleep_time => {type=>SCALAR,default=>1,optional=>1},
                error_values => {type=>ARRAYREF, optional =>1 },
                fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
    }
    );
    my @error_values;
    my $mark;

    if(!defined($opts{error_values})){
        my %ems_error_messages=(#"01"=>"monitor.shelf.fault",
                    "02"=>"raid.disk.missing",
                    "03"=>"raid.config.spare.disk.missing",
                    "04"=>"cf.fsm.backupMailboxError",
            "05"=>"fci.adapter.parity",
            "06"=>"fci.adapter.bad",
            "07"=>"sas.adapter.bad",
            "08"=>"fci.adapter.failed",
            "09"=>"sas.adapter.failed",
            "10"=>"fci.adapter.nvram.invalid",
            "11"=>"sas.adapter.nvram.invalid",
            "12"=>"fci.device.invalid.wwn",
            "13"=>"sas.device.invalid.wwn",
            "14"=>"fci.device.invalidate.soft.address",
            "15"=>"sas.device.invalidate.soft.address",
            "16"=>"fci.adapter.firmware.fault",
            "17"=>"sas.adapter.firmware.fault",
            "18"=>"fci.adapter.firmware.pause",
            "19"=>"sas.adapter.firmware.update.failed",
            "20"=>"fci.adapter.not.ready",
            "21"=>"sas.adapter.not.ready",
            "22"=>"sas.adapter.offline",
            "23"=>"fci.device.login.failure",
            "24"=>"fci.adapter.unexpected.status",
            "25"=>"sas.adapter.unexpected.status",
            "26"=>"disk.write.failure",
            "27"=>"diskown.errorReadingOwnership",
            "28"=>"disk.senseError",
            "29"=>"disk.readReservationFailed",
            "30"=>"scsi.cmd.adapterFatal",
            "31"=>"scsi.cmd.adapterHardwareError",
            "32"=>"scsi.cmd.adapterResources",
            "33"=>"scsi.cmd.contingentAllegiance",
            "33"=>"scsi.cmd.noMorePaths",
            "34"=>"raid.disk.unload.done",
            "35"=>"raid.rg.recons.missing",
            "35"=>"raid.disk.maint.start",


                   );
         @error_values=values(%ems_error_messages);
    }
    else{
         @error_values = @{$opts{error_values}};
    }

  my $loop_count = int($opts{monitor_time} / $opts{sleep_time});
   $mark = $opts{mark};

  my $i = 1;
  my $fail = "no";

  while ($i <= $loop_count) {
      my $mark1 = ems_log_get_mark(host=>$opts{host});
      my $event_by_type = ems_log_dump_by_type(host=>$opts{host}, timestamp=>$mark);
      foreach my $msg (@error_values) {
          my $event = $event_by_type->{"$msg"};
      #print "EVENT: $event \n";
          if (defined($event)) {
          if($opts{fatal_on_failure} eq "yes"){
                    logresult('FATAL',"Failed because of: $msg");
          }
          else{
                    logresult('FAIL',"Failed because of: $msg");
            $fail = "yes";
          }

          }
      }
      $mark = $mark1;
      sleep $opts{sleep_time};
      ++$i;
  }

  # Didn't find any error in the given time
  if($fail eq "no"){
    logcomment("No recognizable errors found.");
  }
}


=head2 device_clear_affiliation

    device_clear_affilation(host=>$host,disks=>$DISKS,cycle=>$CYCLE,interval=>$INTERVAL,filermode=>$FILERMODE,conntype=>$FILER_CONN,fatal_on_failure=>$FATAL_ON_FAILURE,timeout=>$TIMEOUT,runlength=>$runtime);


This subrutine clears affiliation of  SATA drives  on Shasta/Shuar system for n no of cycles.

It returns 1 on success and croaks on failure

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item disks

(Optional) : A comma or space seperated list of disks to clear affiliation of. For example $disks= "4a.30  4b.30 4c.30.30  " or "4a.30,4b.30,4c.90.3" is a valid option.

By default it resets all the Drives.

=item timeout

(Optional): timeout value. Default is 300 secs.

=item cycle

(Optional) : Number of cycles of adapter resets to run. Default is 1.

=item interval

(Optional) : Sleep time in seconds between each adapter reset. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "maint"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item fatal_on_failure

(Optional): Stop test on failure. If set to "no" then the test will "fail" instead of fatal. Default is fatal.

=item runlength

(Optional): Length in seconds. Default is "none".

=item emscheck

(Optional): Check ems logs for errors. Default is yes

=item parallel_reset

(Optional): "yes" to clear affiliation of all devices in parallel. Default is "no".

=back

=cut


sub device_clear_affiliation {
my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                        cycle => { type=>SCALAR, default=>1, optional=>1 },
                        interval => { type=>SCALAR, default=>30, optional=>1 },
            disks => {type=>SCALAR,default=>"all",optional=>1},
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            runlength => {type=>SCALAR,default=>"none",optional=>1},
            emscheck => {type=>SCALAR,default=>"yes",optional=>1},
            parallel_reset => {type=>SCALAR,default=>"no",optional=>1},
        }
    );
    my $i;
    my @fc_disks = ();
    my @sas_disks = ();
    my $disk1;
    my $fc_disk;
    my $sas_disk;
    my @disks;
    my $item;
    my $start_time = time();
    my $elapsed_time;
    my $current_time;
    my $left_time;
    my $ems_check = "no";
    my $mark;

    if(($opts{emscheck} eq "yes") && ($opts{filermode} eq "normal")){
        $ems_check = "yes";
    }

    my $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
    my @disk_list = keys (%$show_disks);
    if($opts{disks} eq "all"){
          @disks = @disk_list;
    }
    else {
            my @disks1 = split(/,|\s+/,$opts{disks});
        my %seen;
        @seen{@disk_list} = ();
        foreach $item (@disks1) {
            push(@disks, $item) if exists $seen{$item};
        }
    }

    foreach $disk1 (@disks){
        if($disk1 =~ /\./){
            if($disk1 =~ /\.\d+\./){
                push @sas_disks, $disk1;
            }
            else{
                push @fc_disks, $disk1;
            }
        }
    }
    print "\n SAS DISKS : @sas_disks \n";
    if (($#sas_disks < 0)){
        logresult('fatal', "0 matching disks found");
    }


    for($i = 1;$i <= $opts{cycle};$i++)
    {
        logcomment("Starting CYCLE $i of device resets.\n");
        if($opts{parallel_reset} eq "yes"){
            if($ems_check eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
            }
        }
                foreach $sas_disk (@sas_disks){
            if($opts{parallel_reset} ne "yes"){
                if($ems_check eq "yes"){
                    $mark = ems_log_get_mark(host=>$opts{host});
                }
            }
            logcomment("Clearing affiliation of SAS device $sas_disk");
            if($opts{fatal_on_failure} eq "no"){
                eval {
                    sasadmin_dev_phy_clear_affiliation(host=>$opts{host},filermode=>$opts{filermode},dev_id=>$sas_disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
                };
                if($@){
                    logresult('fail',"Function sasadmin_dev_phy_clear_affiliation  failed: $@ ");
                }
            }
            else{
                    sasadmin_dev_phy_clear_affiliation(host=>$opts{host},filermode=>$opts{filermode},dev_id=>$sas_disk,timeout=>$opts{timeout},conntype=>$opts{conntype});
            }

            if($opts{parallel_reset} ne "yes"){
                logcomment("Sleeping for $opts{interval} Secs.");
                sleep $opts{interval};
                if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
                }
            }
            if ($opts{runlength} ne "none"){
                $current_time = time();
                $elapsed_time = $current_time - $start_time;
                $left_time = $opts{runlength} - $elapsed_time;
                if ($left_time < 0){
                    logcomment(" RUNLENGTH: $opts{runlength} secs complete. Stopping test after $i cycles");
                    #last;
                    #exit 0;
                    goto CLEAR_AFFILIATION_LABEL;
                }
            }

        }
        if($opts{parallel_reset} eq "yes"){
            logcomment("Sleeping for $opts{interval} Secs.");
            sleep $opts{interval};
            if($ems_check eq "yes"){
                ems_error_monitor(host=>$opts{host},mark=>$mark,fatal_on_failure=>$opts{fatal_on_failure});
            }
        }
    }
    CLEAR_AFFILIATION_LABEL: ;

}


=head2 get_storage_state

    get_storage_state(host=>$host,filermode=>$FILERMODE,conntype=>$FILER_CONN,timeout=>$TIMEOUT);

This function returns a reference to a hash with data from "storage show disk", "storage show adapter", disk show -v -z", "raid_config", "vol status -m" and "vol status -f". The function is particulary used by the compare_storage_state function to compare the various storage states.

It returns a reference  to the structure like below on success and croaks on failure

=begin html

    $VAR1 = {

          'RaidFailedDisks' => [], # List of disks failed by raid (aggr status -f) . Excluding "admin failed" disks

          'RaidMaintDisks' => [],  #List of disks in Maint Center (aggr status -m) . Excluding "admin maint" and sanitizing  disks

          'SanOwnFailed' => [
                              '0c.01.19',
                              '0c.01.10'
                            ],     # List of disks failed in disk show -v -z". Excluding "admin failed/maint" disks and sanitizing disks

          'RaidDisks' => [
                           '0c.01.6',
                           '0c.01.0',
               ........
               .....
               ..
               ], # List of disks seen by raid (raid_config)

          'DiskShow' => {
                          '0c.01.6' => {
                                         'HOME' => 'stor-u62',
                                         'SYSID' => '175287864',
                                         'UpCnt' => '4',
                                         'DUID' => '5000C500:01823B67',
                                         'POOL' => 'Pool0',
                                         'PRKEY' => '175287864',
                                         'SERIAL_NUMBER' => '3LN08T7Q00009720EMX6',
                                         'HOME_SYSID' => '175287864',
                                         'OWNER' => 'stor-u62'
                                       },
                       ...........
                       .......
                       ....
                       .
            },  # Hash of "disk show -v -z" output

      'StorageShowAdapter' => {
                                    '0e' => {
                                              'Description' => 'IDE Host Adapter 0e'
                                            },
                                    '0b' => {
                                              'FCNodeName' => '5:00a:098000:01b357',
                                              'State' => 'Disabled',
                                              'LinkDataRate' => '1 Gbit',
                                              'FirmwareRev' => '4.0.24',
                                              'SRAMParity' => 'Yes',
                                              'InUse' => 'No',
                                              'Redundant' => 'Yes',
                                              'FCPacketSize' => '2048',
                                              'Description' => 'Fibre Channel Host Adapter 0b (QLogic 2432 rev. 2)'
                                            },
                        ........
                        ....
                }, # Hash of "Storage show adapter" output

                 'StorageShowDisk' => {
                                 '0c.01.6' => {
                                                'UID' => '5000C500:01823B67:00000000:00000000:00000000:00000000:00000000:00000000:00000000:00000000',
                                                'Dynamicallyqualified' => 'No',
                                                'Vendor' => 'NETAPP',
                                                'Timeinterval' => '29:05:30',
                                                'Model' => 'X286_S15K5146A15',
                                                'Serial' => '3LN08T7Q00009720EMX6',
                                                'Shelf' => '1',
                                                'Scrubcount' => '55',
                                                'Bay' => '6',
                                                'Rev' => 'NA00',
                                                'WWN' => '5:000:c50001:823b64',
                                                'Glistcount' => '0',
                                                'Downrev' => 'no',
                                                'RPM' => '15000',
                                                'Scrublastdone' => '00:08:01',
                                                'PowerOnHours' => '4576',
                                                'BlkWritten' => '678168281',
                                                'BlkRead' => '1433312708',
                                                'PriPort' => 'A',
                                                'LIP count' => '0'
                                              },
                        ..........
                        .....
                 }, # Hash of "storage show disk -a" output
    }

=end html


The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item timeout

(Optional): timeout value. Default is 300 secs.

=item filermode

(Optional) : Specifies to run tests in normal or maintenance mode. Default is normal.

For maintenance mode the variable will be "maint"

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=back

=cut




sub get_storage_state

{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            skip => {type=>SCALAR,default=>"none",optional=>1},
#fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            }
            );


    if($opts{filermode} ne "normal"){
        $opts{conntype} = "#console";
    }

    my $conn = make_conn($opts{host},$opts{conntype});
    my %structure =(); #Hash reference defined
    my $disk;
    my @disk_list1;
    my $sanown_disks;
    my @sanown_disks;
    my @admin_maint_disks;
    my @admin_failed_disks;
    my $cmd;
    my @lines;
    my $is_sanown = is_sanown(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
    my $show_disks = storage_show_disk(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
    # Hash of "storage show disk -a" output
    $structure{StorageShowDisk} = $show_disks;
    my $adapter_info = storage_show_adapter(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype}, timeout=>$opts{timeout});
    # Hash of "storage show adapter" output
    $structure{StorageShowAdapter} = $adapter_info;

    if($is_sanown){
         $sanown_disks = disk_show(host=>$opts{host}, filermode=>$opts{filermode},timeout=>$opts{timeout},conntype=>$opts{conntype});
        # Hash of "disk show -v -z" output
        $structure{DiskShow} = $sanown_disks;
        @sanown_disks = keys (%$sanown_disks); #Total Disks Seen by sanown
    }

    if($opts{skip} ne "raid" ){
        use RaidInfo;
        my $r = new RaidInfo($conn);
        my %disks = $r->all_disks();
        my @raid_disks = _get_disk_array(\%disks);
# All the disks seen by raid (raid_config)
        $structure{RaidDisks} = [@raid_disks];
# The following is a temporary method to get a list of failed and maintenence center disk because raid_config doesn't support failure type of failed disks. We want to ignore the count of "admin failed" disks
        $cmd = "priv set -q diag;aggr status -f";
        my $output;
        my @raid_failed_disks;
        #@admin_failed_disks;
        #(undef, undef, $output) = say($conn, "$cmd","-timeout", $opts{timeout});
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"$cmd");
        @lines = split(/\r*\n/, $output);
        foreach my $line (@lines){
            if (($line =~ /\./) && ($line !~ /admin/)){ # Exclude admin failed disks
                my @line1 = split(/\s+/,$line);
                if($line1[1] =~ /\./){
                    push @raid_failed_disks, $line1[1];
                }
                if($line1[2] =~ /\./){
                    push @raid_failed_disks, $line1[2];
                }
            }
            elsif (($line =~ /\./) && ($line =~ /admin/)){
                my @line1 = split(/\s+/,$line);
                if($line1[2] =~ /\./){
                    push @admin_failed_disks, $line1[2];
                }
            }

        }
# All the disks failed by raid (except admin failed disk)
        $structure{RaidFailedDisks} = [@raid_failed_disks];

        my @maint_disks;
        #@admin_maint_disks;
        $cmd = "priv set -q diag;aggr status -m";
        #(undef, undef, $output) = say($conn, "$cmd","-timeout", $opts{timeout});
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"$cmd");
        @lines = split(/\r*\n/, $output);
        foreach my $line (@lines){
            if (($line =~ /\./) && ($line !~ /admin/) && ($line !~ /sanitiz/)){ # Exclude admin maint or sanitizing disks
                my @line1 = split(/\s+/,$line);
                if($line1[1] =~ /\./){
                    push @maint_disks, $line1[1];
                }
                if($line1[2] =~ /\./){
                    push @maint_disks, $line1[2];
                }
            }
            elsif (($line =~ /\./) && ($line =~ /admin/)){
                my @line1 = split(/\s+/,$line);
                if($line1[2] =~ /\./){
                    push @admin_maint_disks, $line1[2];
                }
            }

        }
# Disks in the maintenence center ( Excluding admin maint and sanitizing disks)
        $structure{RaidMaintDisks} = [@maint_disks];
    }
    # Disks showing up as failed in the "disk show -v -z" output
    if($is_sanown){
        my @sanown_failed = grep $structure{DiskShow}{$_}{POOL} eq "FAILED", keys %{$structure{DiskShow}};
        # Exclude admin failed disks from the disk show above
        @sanown_failed = _get_unique_elements(\@admin_failed_disks,\@sanown_failed);
        # Exclude admin maint or sanitizing disks from the disk show above
        @sanown_failed = _get_unique_elements(\@admin_maint_disks,\@sanown_failed);
        $structure{SanOwnFailed} = [@sanown_failed];
    }
    return (\%structure);
}

=head2 compare_storage_state

    compare_storage_state(initial_state=>$initial_state,final_state=>$final_state,fatal_on_failure=>$fatal_on_failure);

This functions compares structures returned by get_storage_state and reports any unexpected storage and raid failures

Following states are compared:

1. List of disks failed by raid (excluding admin failed disks)
2. List disks in MaintCenter ( Excluding admin maint and sanitizing disks)
3. Total number of disks seen by raid
4. Total number of disks seen by storage
5. Redundant path comparison of disks
6. Total number of disks seen by sanown
7. List of disks failed by sanown ( Excluding admin failed/maint and sanitizing disks)
8. Total number of HBAs
9. Total number of online FC HBAs
10. Total number of online SAS HBAs.

Example to use compare_storage_state and get_storage state function:

Get a snapshot of storage state:
$initial = get_storage_state(host=>$host);

Run your tests.........
......

Take another snapshot after the test is done:
$final = get_storage_state(host=>$host);

Comparing the two snapshots:
compare_storage_state(initial_state=>$initial,final_state=>$final);

The following arguments are accepted:

=over

=item initial_state

(Required): Initial storage data structure returned by get_storage_state()

=item final_state

(Required): Final storage data structure returned by get_storage_state() which is then compared with initial_state above

=item fatal_on_failure

(Optional):  "fatal" and stop on failure . If set to "no" then the test will "fail" instead of fatal. Default is fail.

=back

=cut


sub compare_storage_state
{
    my (%opts) = validate(@_, {
            fatal_on_failure => {type=>SCALAR,default=>"no",optional=>1},
            initial_state => {type=>HASHREF},
            final_state => {type=>HASHREF},
            skip => {type=>SCALAR,default=>"none",optional=>1},
            }
            );

    my $error = 0;
    my $initial_hash;
    my $final_hash;
    my %init_hash = %{$opts{initial_state}};
    my %fin_hash = %{$opts{final_state}};
    my @extra_failed_disks=();
    if($opts{skip} ne "raid" ){

# List of disks showing up as failed in the "aggr status -f" before starting the test


        my @initial_raid_failed_disks =   @{$opts{initial_state}->{RaidFailedDisks}};

# List of disks showing up as failed in the "aggr status -f" at the end of the test

        my @final_raid_failed_disks =   @{$opts{final_state}->{RaidFailedDisks}};

        if($#final_raid_failed_disks > $#initial_raid_failed_disks){
            my $count = $#final_raid_failed_disks - $#initial_raid_failed_disks;
            foreach my $final_disk (@final_raid_failed_disks) {
                my $flag=0;
                foreach my $init_disk(@initial_raid_failed_disks) {
                    if (($final_disk eq $init_disk)||($final_disk eq $opts{initial_state}->{StorageShowDisk}->{$init_disk}->{SecPath})) {
                        $flag=0;
                        last;
                    } else {
                        $flag=1;
                    }
                }
                push(@extra_failed_disks,$final_disk) if($flag);
            }
            logcomment("WARNING: aggr status -f is showing additional $count failed disks.The additional failed disks are @extra_failed_disks. \n At the beginning of the test Failed disks were : @initial_raid_failed_disks \n And at the end Failed disks are:\n @final_raid_failed_disks");
            $error = 1;
        }


# List of disks in the "raid_config" before starting the test
        my @initial_raid_disks = @{$opts{initial_state}->{RaidDisks}};
# List of disks in the "raid_config" at the end of the test
        my @final_raid_disks = @{$opts{final_state}->{RaidDisks}};
        my @missing_raid_disks=();
        if($#final_raid_disks < $#initial_raid_disks){
            my $count = $#initial_raid_disks - $#final_raid_disks;
            foreach my $init_disk(@initial_raid_disks) {
                my $flag=0;
                foreach my $final_disk (@final_raid_disks) {
                    if (($final_disk eq $init_disk)||($final_disk eq $opts{initial_state}->{StorageShowDisk}->{$init_disk}->{SecPath})) {
                        $flag=0;
                        last;
                    } else {
                        $flag=1;
                    }
                }
                push(@missing_raid_disks,$init_disk) if($flag);
            }
            logcomment("WARNING: raid_config is showing $count less raid disks. The missing raid disks are @missing_raid_disks. \n At the beginning of the test raid disks were : @initial_raid_disks \n And at the end raid disks are:\n @final_raid_disks");
            $error = 1;
        }

# List of disks showing up as failed in the "aggr status -m" before starting the test
        my @initial_raid_maint_disks =   @{$opts{initial_state}->{RaidMaintDisks}};
# List of disks showing up as failed in the "aggr status -m" at the end of the test
        my @final_raid_maint_disks =   @{$opts{final_state}->{RaidMaintDisks}};
        my @extra_maint_disks=();
        if($#final_raid_maint_disks > $#initial_raid_maint_disks){
            my $count = $#final_raid_maint_disks - $#initial_raid_maint_disks;
            foreach my $final_disk (@final_raid_maint_disks) {
                my $flag=0;
                foreach my $init_disk(@initial_raid_maint_disks) {
                    if (($final_disk eq $init_disk)||($final_disk eq $opts{initial_state}->{StorageShowDisk}->{$init_disk}->{SecPath})) {
                        $flag=0;
                        last;
                    } else {
                        $flag=1;
                    }
                }
                push(@extra_maint_disks,$final_disk) if($flag);
            }
            logcomment("WARNING: aggr status -m is showing additional $count maint disks. The additional maint disks are @extra_maint_disks. \n At the beginning of the test maint disks were : @initial_raid_maint_disks \n And at the end maint disks are:\n @final_raid_maint_disks");
            $error = 1;
        }
    }

# List of disks in the "storage show disk -a" before starting the test
    my @initial_storage_disks = keys %{$opts{initial_state}->{StorageShowDisk}};
    foreach my $disk (@initial_storage_disks){
        my $serial = $opts{initial_state}->{StorageShowDisk}->{$disk}->{Serial};
    if($serial ne 'undef'){
         #  print "i have found the serial to be <som>$serial<eom>";
            $initial_hash->{$serial}->{PriPath} = $disk;
            $initial_hash->{$serial}->{UID} =
            $opts{initial_state}->{StorageShowDisk}->{$disk}->{UID};
            if(defined($opts{initial_state}->{StorageShowDisk}->{$disk}->{SecPath})){
                $initial_hash->{$serial}->{SecPath} = $opts{initial_state}->{StorageShowDisk}->{$disk}->{SecPath};
        }
        }
    }

# List of disks in the "storage show disk -a" at the end of the test
    my @final_storage_disks = keys %{$opts{final_state}->{StorageShowDisk}};

    foreach my $disk (@final_storage_disks){
        my $serial = $opts{final_state}->{StorageShowDisk}->{$disk}->{Serial};
        if($serial ne 'undef'){
          #  print "i have found the serial to be <som>$serial<eom>";
        $final_hash->{$serial}->{PriPath} = $disk;
            $final_hash->{$serial}->{UID} =
            $opts{final_state}->{StorageShowDisk}->{$disk}->{UID};
        if(defined($opts{final_state}->{StorageShowDisk}->{$disk}->{SecPath})){
                $final_hash->{$serial}->{SecPath} = $opts{final_state}->{StorageShowDisk}->{$disk}->{SecPath};
            }
    }
    }
    #print Dumper $initial_hash;
    #print Dumper $final_hash;

    my @initial_disk_serial = keys %{$initial_hash};
    foreach my $serial (@initial_disk_serial){
        my $disk1 = $initial_hash->{$serial}->{PriPath};
        if(!defined($final_hash->{$serial})){
            logcomment("Warning: Disk id $disk1 . Serial $serial. Not seen in the final state");
            $error = 1;
        }
        else{
            my $uid1 = $initial_hash->{$serial}->{UID};
            my $uid2 = $final_hash->{$serial}->{UID};
            if("$uid1" ne "$uid2"){
                logcomment("Warning: Disk id $disk1 . Serial $serial. UID not matching . Before: $uid1 and after $uid2 ");
                $error = 1;
            }
            if(defined($initial_hash->{$serial}->{SecPath})){
                if(!defined($final_hash->{$serial}->{SecPath})){
                    logcomment("Warning: Disk id $disk1 . Serial $serial. SecPath not defined in the final state");
                    $error = 1;
                }
            }
        }
    }


    if((defined($opts{initial_state}->{DiskShow})) && (defined($opts{final_state}->{DiskShow}))) { # If not defined then we can assume that it's a non-sanown system


# List of disks in the "disk  show -v -z" before starting the test
        my @initial_sanown_disks = keys %{$opts{initial_state}->{DiskShow}};
# List of disks in the "disk show -v -z" at the end of the test
        my @final_sanown_disks = keys %{$opts{final_state}->{DiskShow}};

        my @missing_sanown_disks=();
        if($#final_sanown_disks < $#initial_sanown_disks){
            my $count = $#initial_sanown_disks - $#final_sanown_disks;
            foreach my $init_disk(@initial_sanown_disks) {
                my $flag=0;
                foreach my $final_disk (@final_sanown_disks) {
                    if (($final_disk eq $init_disk)||($final_disk eq $opts{initial_state}->{StorageShowDisk}->{$init_disk}->{SecPath})) {
                        $flag=0;
                        last;
                    } else {
                        $flag=1;
                    }
                }
                push(@missing_sanown_disks,$init_disk) if($flag);
            }
            logcomment("WARNING: disk show -v -z is showing $count less  disks> Missing sanown disks are @missing_sanown_disks. \n At the beginning of the test  disks were : @initial_sanown_disks \n And at the end  disks are:\n @final_sanown_disks");
            $error = 1;
        }


# List of failed disks in the "disk  show -v -z" before starting the test
        my @initial_sanown_failed = @{$opts{initial_state}->{SanOwnFailed}};
# List of failed disks in the "disk show -v -z" at the end of the test
        my @final_sanown_failed = @{$opts{initial_state}->{SanOwnFailed}};
        my @extra_sanown_failed=();
        if($#final_sanown_failed > $#initial_sanown_failed){
            my $count = $#final_sanown_failed - $#initial_sanown_failed;
            foreach my $final_disk (@final_sanown_failed) {
                my $flag=0;
                foreach my $init_disk(@initial_sanown_failed) {
                    if (($final_disk eq $init_disk)||($final_disk eq $opts{initial_state}->{StorageShowDisk}->{$init_disk}->{SecPath})) {
                        $flag=0;
                        last;
                    } else {
                        $flag=1;
                    }
                }
                push(@extra_sanown_failed,$final_disk) if($flag);
            }
            logcomment("WARNING: disk show -v -z is showing additional $count failed disks> Additional disks are @extra_sanown_failed. \n At the beginning of the test failed disks were : @initial_sanown_failed \n And at the end failed disks are:\n @final_sanown_failed");
            $error = 1;
        }
    }

# List of HBAs in the "storage show adapter" before starting the test
    my @initial_storage_adapters = keys %{$opts{initial_state}->{StorageShowAdapter}};
# List of HBAs in the "storage show adapter" at the end
    my @final_storage_adapters = keys %{$opts{final_state}->{StorageShowAdapter}};
    my @initial_fc_adapters;
    my @initial_sas_adapters;
    foreach my $adapt (@initial_storage_adapters){
        if(($opts{initial_state}->{StorageShowAdapter}->{$adapt}->{'State'} =~ /Enabled/) && ($opts{initial_state}->{StorageShowAdapter}->{$adapt}->{'Description'} =~ /^Fibre/)){
            push @initial_fc_adapters,$adapt; #Online FC HBAs
        }
        elsif(($opts{initial_state}->{StorageShowAdapter}->{$adapt}->{'State'} =~ /Enabled/) && ($opts{initial_state}->{StorageShowAdapter}->{$adapt}->{'Description'} =~ /^SAS/)){
            push @initial_sas_adapters,$adapt; #Online SAS HBAs
        }
    }

    my @final_fc_adapters;
    my @final_sas_adapters;
    my @missing_fcadapters=();
    my @missing_sasadapters=();
    foreach my $adapt (@final_storage_adapters){
        if(($opts{final_state}->{StorageShowAdapter}->{$adapt}->{'State'} =~ /Enabled/) && ($opts{final_state}->{StorageShowAdapter}->{$adapt}->{'Description'} =~ /^Fibre/)){
            push @final_fc_adapters,$adapt; #Online FC HBAs
        }
        elsif(($opts{final_state}->{StorageShowAdapter}->{$adapt}->{'State'} =~ /Enabled/) && ($opts{final_state}->{StorageShowAdapter}->{$adapt}->{'Description'} =~ /^SAS/)){
            push @final_sas_adapters,$adapt; #Online SAS HBAs
        }
    }


    if($#final_fc_adapters < $#initial_fc_adapters){
        my $count = $#initial_fc_adapters - $#final_fc_adapters;
        foreach my $init_adap(@initial_fc_adapters) {
                my $flag=0;
                foreach my $final_adap (@final_fc_adapters) {
                    if ($final_adap eq $init_adap) {
                        $flag=0;
                        last;
                    } else {
                        $flag=1;
                    }
                }
                push(@missing_fcadapters,$init_adap) if($flag);
        }
        logcomment("WARNING: storage show adapter is showing $count less  online FC adapters> Missing FC adapters are @missing_fcadapters. \n At the beginning of the test  online FC adapters were : @initial_fc_adapters \n And at the end  online FC adapters are:\n @final_fc_adapters");
        $error = 1;
    }


    if($#final_sas_adapters < $#initial_sas_adapters){
        my $count = $#initial_sas_adapters - $#final_sas_adapters;
        foreach my $init_adap(@initial_sas_adapters) {
                my $flag=0;
                foreach my $final_adap (@final_sas_adapters) {
                    if ($final_adap eq $init_adap) {
                        $flag=0;
                        last;
                    } else {
                        $flag=1;
                    }
                }
                push(@missing_sasadapters,$init_adap) if($flag);
        }
        logcomment("WARNING: storage show adapter is showing $count less  Online SAS HBAs.Missing SAS adapters are @missing_sasadapters \n At the beginning of the test  Online SAS HBAs were : @initial_sas_adapters \n And at the end  Online SAS HBAs are:\n @final_sas_adapters");
        $error = 1;
    }
    if($error == 0){
        logcomment("States match: PASS");
    }
    else{
        print "Initial State: \n";
        print Dumper $opts{initial_state};
        print "Final State: \n";
        print Dumper $opts{final_state};
        if($opts{fatal_on_failure} eq "yes"){
            logresult("fatal", "States do not match. Check the logs above for details");
        }
        else{
            logresult("fail", "States do not match. Check the logs above for details");
        }
    }
}

=head2 check_shelf_fault

    my $hash = check_shelf_fault(host=>$host);

This routine is used to findout if "environ shelf" has any errors

=over

=item host

(Required) : Filer connnection to use.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=item fatal_on_failure

(optional) : To fatal on errors. Default is "yes"

=back

=cut

sub check_shelf_fault {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
            retry => {type=>SCALAR,default=>0,optional=>1},
            }
            );
    my $error = 0;
    my $retry = 0;
    #my $conn = make_conn($opts{host},$opts{conntype});
    SFAULT_LABEL: ;
    my $environ_shelf = environ_status(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},filermode=>$opts{filermode});
    my @channels = keys (%$environ_shelf);
    foreach my $channel (@channels){
        my @shelves_faulted;
        if(defined($environ_shelf->{$channel}->{Shelvesfail})){
#print "\n==============\n";
#print Dumper $environ_shelf;
#print "\n==============\n";
            my @shelves = keys %{$environ_shelf->{$channel}};
#print "SHELVES on channel $channel are @shelves \n";
            foreach my $shelf (@shelves){
                if($shelf =~ /^\d+$/){
                    if($environ_shelf->{$channel}->{$shelf}->{ShelfStatus} !~ /normal|information/){
                        my $fault = $environ_shelf->{$channel}->{$shelf}->{ShelfStatus};
                        logcomment("Status of Channel $channel  Shelf $shelf is faulted and status is : $fault");
                        $error = 1;
                    }
                }

            }
        }
    }
    if(($error == 1) && ($retry < $opts{retry})){
        $retry++;
        logresult("warn", "Shelf fault found. Will check shelf status again after 10 minutes. Retry: $retry");
        sleep 600;
        goto SFAULT_LABEL;
    }


    if($error == 1){
        if($opts{fatal_on_failure} eq "yes"){
            logresult('fatal',"Shelf fault found. Please check the log comment above to find the details");
        }
        else{
            logresult('fail',"Shelf fault found. Please check the log comment above to find the details");
        }
    }
    else{
        logcomment("No Shelf fault found");
    }
}

=head2 is_sanown

    my $hash = is_sanown(host=>$host);

This routine is used to findout whether a system is sanown enabled or not
It return 1 if system is sanown and 0 if the host is not sanown

=over

=item host

(Required) : Filer connnection to use.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut

sub is_sanown{
    my (%opts) = validate(@_, {
                         host => { type=>OBJECT, isa=>'Hostrec' },
                             filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                 conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                             timeout => { type=>SCALAR, default=>300, optional=>1 },
                            }
                     );

    eval { storage_show_initiators(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout}); };
    if($@ =~ /SANOWN not enabled/){
        return 0;
    }
    else{
        return 1;
    }
}

=head2 compare_shasta_struct

    compare_shasta_struct(initial_state=>$initial_state,final_state=>$final_state,fatal_on_failure=>$fatal_on_failure);

This function compares structures returned by get_shasta_struct and reports reports if there is any mismatch. It compares the fields returned by "storage show expander" and "sasadmin expander_phy_state"

Example to use compare_storage_state and get_storage state function:

Get a snapshot of  state:
$initial = get_shasta_struct(host=>$host);

Run your tests.........
......

Take another snapshot after the test is done:
$final = get_shasta_struct(host=>$host);

Comparing the two snapshots:
compare_shasta_struct(initial_state=>$initial,final_state=>$final);

The following arguments are accepted:

=over

=item initial_state

(Required): Initial storage data structure returned by get_shasta_struct()

=item final_state

(Required): Final storage data structure returned by get_shasta_struct() which is then compared with initial_state above

=item fatal_on_failure

(Optional):  "fatal" and stop on failure . If set to 0 then the test will "fail" instead of fatal. Default is 1.

=back

=cut



sub compare_shasta_struct
{
    my (%opts) = validate(@_, {

            fatal_on_failure => {type=>SCALAR,default=>1,optional=>1},
            initial_state => {type=>HASHREF},
            final_state => {type=>HASHREF},
            skip_check => {type=>ARRAYREF, optional =>1 },
            check_only => {type=>ARRAYREF, optional =>1 },
            skip_phys => {type=>ARRAYREF, optional =>1 },
                                                                                                    }
                                                        );

    my $error = 0;
    my $struct = $opts{initial_state};
    my $struct1 = $opts{final_state};
    my @phy_count_before = keys %{$opts{initial_state}};
    my @phy_count_after = keys %{$opts{final_state}};
    if ($#phy_count_before != $#phy_count_after){
        logresult('fail', "Total phy count before reset is $#phy_count_before and after is $#phy_count_after");
        $error = 1;
    }
    my $phy;
    my @fields;
    if(defined($opts{skip_phys})){
        my @check_phys = _get_unique_elements($opts{skip_phys},\@phy_count_before);
        @phy_count_before = @check_phys;
    }
    foreach $phy (@phy_count_before) {
        my @all_fields = keys %{$struct->{$phy}};
        if(defined($opts{check_only})){
            @fields = @{$opts{check_only}};
        }
        elsif(defined($opts{skip_check})){
            @fields = _get_unique_elements($opts{skip_check},\@all_fields);
        }
        else{
            @fields = @all_fields ;
        }


        foreach my $field (@fields){
            if(defined($struct->{$phy}->{$field})){
                my $before_val = $struct->{$phy}->{$field};
                my $after_val = $struct1->{$phy}->{$field};
                if($before_val ne $after_val){
                    logcomment("$field not matching for $phy. Before it was $before_val and after it is $after_val");
                    $error = 1;
                }
            }
        }
    }
    if($error == 1){
        logcomment("Dumper of initial_state ");
        print Dumper $opts{initial_state};
        logcomment("Dumper of final_state ");
        print Dumper $opts{final_state};
        if($opts{fatal_on_failure} == 1 ){
            logresult('fatal', "sasadmin expander_phy_state and/or storage show expander output not matching between before and after test.Please check the logs above for the specific mismatch");
        }
        else{
            logresult('fail', "sasadmin expander_phy_state and/or storage show expander output not matching between before and after test.Please check the logs above for the specific mismatch");
        }
    }
    else {
            logcomment("sasadmin expander_phy_state and storage show expander output matching between before and after test");
    }



}

=head2 compare_storage_and_sasadmin

    compare_storage_and_sasadmin(initial_state=>$initial_state,final_state=>$final_state,fatal_on_failure=>$fatal_on_failure);

This function compares "storage show expander" and "sasadmin expander_phy_state" counters in the structure returned by get_shasta_struct and reports failures.

Usage:
Get a snapshot of  state:
$struct = get_shasta_struct(host=>$host);

Run your tests.........
......

compare_storage_and_sasadmin(host=>$host,struct=>$struct);


The following arguments are accepted:

=over

=item struct

(Required): Storage data structure returned by get_shasta_struct()

=item fatal_on_failure

(Optional):  "fatal" and stop on failure . If set to 0 then the test will "fail" instead of fatal. Default is 1.

=back

=cut


sub compare_storage_and_sasadmin

{
    my (%opts) = validate(@_, {

            fatal_on_failure => {type=>SCALAR,default=>1,optional=>1},
            struct => {type=>HASHREF},
                                                                                                    }
                                                        );

    my $error = 0;
    my $struct = $opts{struct};
    my @phys = keys %{$struct};
    my $sas_phy_reset_prob;
    my $sas_invalid_dword;
    my $sas_phy_change;
    my $sas_loss_sync;
    my $sas_disparity;
    my $stor_phy_reset_prob;
    my $stor_invalid_dword;
    my $stor_phy_change;
    my $stor_loss_sync;
    my $stor_disparity;
    my $sas_rate;
    my $stor_rate;
    my $stor_portstate;

    foreach my $phy (@phys){
        unless($phy =~ /\w+.\w+.28/){
        $sas_rate = $struct->{$phy}->{Rate}; #if defined($struct->{$phy}->{Rate});
        $stor_rate = $struct->{$phy}->{LinkRate}; #if defined($struct->{$phy}->{LinkRate});
        $stor_portstate = $struct->{$phy}->{PortState}; #if defined($struct->{$phy}->{LinkRate});
        if(($sas_rate !~ /Rate unknown|Disabled/) && ($stor_rate !~ /NA/) && ($stor_portstate eq "OK")){
            #print " Phy $phy \n";
        $sas_phy_reset_prob = $struct->{$phy}->{PhyResetProb}; #if defined($struct->{$phy}->{PhyResetProb});
        $sas_invalid_dword = $struct->{$phy}->{InvalidDWordCount}; #if defined($struct->{$phy}->{InvalidDWordCount});
        $sas_phy_change = $struct->{$phy}->{PhyChangeCnt}; #if defined($struct->{$phy}->{PhyChangeCnt});
        $sas_loss_sync = $struct->{$phy}->{LossSyncCount}; #if defined($struct->{$phy}->{LossSyncCount});
        $sas_disparity = $struct->{$phy}->{DisparityErrorCount}; #if defined($struct->{$phy}->{DisparityErrorCount});

        $stor_phy_reset_prob = $struct->{$phy}->{PhyResetProblem}; #if defined($struct->{$phy}->{PhyResetProblem});
        $stor_invalid_dword = $struct->{$phy}->{InvalidWordCount}; #if defined($struct->{$phy}->{InvalidWordCount});
        $stor_phy_change = $struct->{$phy}->{PhyChangeCount}; #if defined($struct->{$phy}->{PhyChangeCount});
        $stor_loss_sync = $struct->{$phy}->{LossDwordCount}; #if defined($struct->{$phy}->{LossDwordCount});
        $stor_disparity = $struct->{$phy}->{RunningDisparityCount}; #if defined($struct->{$phy}->{RunningDisparityCount});

        if($sas_phy_reset_prob != $stor_phy_reset_prob){
            logcomment("PhyResetProblem count for phy $phy not matching between sasadmin expander_phy_state => $sas_phy_reset_prob and storage show expander => $stor_phy_reset_prob");
            $error = 1;
        }
        if($sas_invalid_dword != $stor_invalid_dword){
            logcomment("InvalidDWord count for phy $phy not matching between sasadmin expander_phy_state => $sas_invalid_dword and storage show expander => $stor_invalid_dword");
            $error = 1;
        }
        if($sas_phy_change != $stor_phy_change){
            logcomment("phy_change count for phy $phy not matching between sasadmin expander_phy_state => $sas_phy_change and storage show expander => $stor_phy_change");
            $error = 1;
        }
        if($sas_loss_sync != $stor_loss_sync){
            logcomment("loss_sync count for phy $phy not matching between sasadmin expander_phy_state => $sas_loss_sync and storage show expander => $stor_loss_sync");
            $error = 1;
        }
        if($sas_disparity != $stor_disparity){
            logcomment("disparity count for phy $phy not matching between sasadmin expander_phy_state => $sas_disparity and storage show expander => $stor_disparity");
            $error = 1;
        }

        if($sas_rate ne $stor_rate){
            logcomment("Link Rate for phy $phy not matching between sasadmin expander_phy_state => $sas_rate and storage show expander => $stor_rate");
            $error = 1;
        }


    }

    }
    }

    if($error == 1){
        if($opts{fatal_on_failure} == 1){
            logresult('fatal',"Error Counters not matching between sasadmin expander_phy_state and storage_show_expander. Please check the logs above for details");
        }
        else{
            logresult('fail',"Error Counters not matching between sasadmin expander_phy_state and storage_show_expander. Please check the logs above for details");
        }
    }
    else {
        logcomment ("sasadmin expander_phy_state and storage show expander are matching");
    }
}

=head2 compare_sasadmin_dev_stats

    compare_sasadmin_dev_stats(initial_state=>$initial_state,final_state=>$final_state,fatal_on_failure=>$fatal_on_failure);

This function compares structures returned by sasadmin_dev_stats and reports reports if error counter increases.

Get a snapshot of  state:
$initial = sasadmin_dev_stats(host=>$host);

Run your tests.........
......

Take another snapshot after the test is done:
$final = sasadmin_dev_stats(host=>$host);

Comparing the two snapshots:
compare_sasadmin_dev_stats(initial_state=>$initial,final_state=>$final);

The following arguments are accepted:

=over

=item initial_state

(Required): Initial storage data structure returned by sasadmin_dev_stats()

=item final_state

(Required): Final storage data structure returned by sasadmin_dev_stats() which is then compared with initial_state above

=item fatal_on_failure

(Optional):  "fatal" and stop on failure . If set to 0 then the test will "fail" instead of fatal. Default is 1.

=back

=cut



sub compare_sasadmin_dev_stats
{
    my (%opts) = validate(@_, {

            fatal_on_failure => {type=>SCALAR,default=>1,optional=>1},
            initial_state => {type=>HASHREF},
            final_state => {type=>HASHREF},
            skip_check => {type=>ARRAYREF, optional =>1 },
            check_only => {type=>ARRAYREF, optional =>1 },
            skip_device => {type=>ARRAYREF, optional =>1 },
                                                                                                    }
                                                        );

    my $error = 0;
    my $struct = $opts{initial_state};
    my $struct1 = $opts{final_state};
    my @adap_count_before = keys %{$struct};
    my @adap_count_after = keys %{$struct1};
    if ($#adap_count_before != $#adap_count_after){
        logcomment(" MISMATCH: Total adapter count before test is $#adap_count_before and after is $#adap_count_after");
        $error = 1;
    }
    my $phy;
    my @fields;
    foreach my $adap (@adap_count_before){
        my @dev_before = keys %{$struct->{$adap}};
        my @dev_after = keys %{$struct1->{$adap}};
        if ($#dev_before != $#dev_after){
            logcomment(" MISMATCH: Total dev count on adapter $adap before test is $#dev_before and after is $#dev_after");
            $error = 1;
        }
        if(defined($opts{skip_device})){
            my @check_phys = _get_unique_elements($opts{skip_device},\@dev_before);
            @dev_before = @check_phys;
        }
        foreach $phy (@dev_before) {
            my @all_fields = keys %{$struct->{$adap}->{$phy}};
            if(defined($opts{check_only})){
                @fields = @{$opts{check_only}};
            }
            elsif(defined($opts{skip_check})){
                @fields = _get_unique_elements($opts{skip_check},\@all_fields);
            }
            else{
                @fields = @all_fields ;
            }
            foreach my $field (@fields){
                if(defined($struct->{$adap}->{$phy}->{$field})){
# These are not errors, just regular counters. We need to skip it for comparison.
# UnderRun errors are expected for virtual phy 99. So skip the check
                    unless(($field =~ /CmdsCmpltGood|CmdsCmpltError|FramesIn|FramesOut/) || (($field =~ /UdrnCnt/) && ($phy =~ /\.99/))){
                        my $before_val = $struct->{$adap}->{$phy}->{$field};
                        my $after_val = $struct1->{$adap}->{$phy}->{$field};
                        if($before_val ne $after_val){
                            logcomment(" MISMATCH: $field not matching for $phy. Before it was $before_val and after it is $after_val");
                            $error = 1;
                        }
                    }
                }
            }
        }
    }
    if($error == 1){
        logcomment("Dumper of initial_state ");
        print Dumper $opts{initial_state};
        logcomment("Dumper of final_state ");
        print Dumper $opts{final_state};
        if($opts{fatal_on_failure} == 1 ){
            logresult('fatal', " Errors in  sasadmin dev_stats not matching before and after test. check the logs above for the specific mismatch");
        }
        else{
            logresult('fail', " Errors in sasadmin dev_stats not matching before and after test.Please check the logs above for the specific mismatch");
        }
    }
    else {
            logcomment("sasadmin  dev_stats error counters matching between before and after test");
    }
}






sub testme {

    package main;
    our ($FUNCTION_NAME);
    our $ADAPTERS;
    our $FILER;
    our $FILERMODE;
    package Storage::SASUtils;
    param('FILER', '-mesg', "Filer to test");
    param('FUNCTION_NAME', '-default', "all");
    param('FILERMODE', '-default', "normal");
    my $cmds;
    my $host = host($FILER);
    my $ret;

    if(($FUNCTION_NAME eq "adapter_reset") || ($FUNCTION_NAME eq "all")){
        param('ADAPTERS', '-default', "all");
        my @cmds = (\&adapter_reset);
        foreach $cmds (@cmds){
            if($FILERMODE eq "normal") {
                #Test default connection (rsh) with normal mode
                sleep 20;
                eval {
                    $ret = $cmds->(host=>$host,filermode=>$FILERMODE,sleep=>"30");
                };
                if($@){
                    logresult('fail',"function adapter_reset failed: $@");
                }
                else {
                    logresult('pass'," Got expected results for function adapter_reset ");
                }

                #Test normal mode with telnet connection
                sleep 20;
                eval {
                    $ret = $cmds->(host=>$host,filermode=>$FILERMODE,conntype=>"#telnet",sleep=>"30");
                };
                if($@){
                    logresult('fail',"function adapter_reset failed: $@");
                }
                else {
                    logresult('pass'," Got expected results for function adapter_reset ");
                }
                sleep 20;
                #Test normal mode with console connection
                eval {
                    $ret = $cmds->(host=>$host,filermode=>$FILERMODE,conntype=>"#rsh",sleep=>"30");
                };
                if($@){
                    logresult('fail',"function adapter_reset failed: $@");
                }
                else {
                    logresult('pass'," Got expected results for function adapter_reset ");
                }
                #Test normal mode with supplied list of adapters
                sleep 20;
                eval {
                    $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapters=>$ADAPTERS,sleep=>"30");
                };
                if($@){
                    logresult('fail',"function adapter_reset failed: $@");
                }
                else {
                    logresult('pass'," Got expected results for function adapter_reset ");
                }
            }
            else {
                    # Maintenance Mode test

                eval {
                    $ret = $cmds->(host=>$host,filermode=>"maint",adapters=>$ADAPTERS,sleep=>"30");
                };
                if($@){
                    logresult('fail',"function adapter_reset failed: $@");
                }
                else {
                    logresult('pass'," Got expected results for function adapter_reset ");
                }

            }
        }
    }
}

=head2 ic_present

    my @ret = ic_present(host=>$host);

This subrutine checks if Interconnect is present in the filer.

It returns 1 if IC is present and 0 if not.

The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item timeout

(Optional): timeout value. Default is 300 secs.

=item conntype

(Optional): Connection type for the say command. Default is #rsh

=back

=cut


sub ic_present {
    my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
    }
    );
    my $conn = make_conn($opts{host},$opts{conntype});
    my $output;
#   (undef, undef, $output) = say($conn, "priv set -q diag;ic status",-swallow,"-timeout", $opts{timeout});
        $output = filer_cmd(host => $opts{host}, conntype => $opts{conntype}, timeout => $opts{timeout}, privmode => "diag", cmd => "ic status");
    if($output =~ /Cluster interconnect not found/){
        return 0;
    }
    else{
        return 1;
    }
}

=head2 bmc_command

    my @ret = bmc_command(host=>$host,bmc_command=>"system power cycle");

This subrutine presses control G on a shuar system and issues a particular command and then boots the filer up.


The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item timeout

(Optional): timeout value. Default is 300 secs.

=item bmc_command

(Required): Command to be issued. Valid options are : "system power cycle", "system reset", "system core", "system power off", "system power on".

=item boot_up

(Optional): To boot the filer up after issuing the bmc command. Default is "yes".

=back

=cut


sub bmc_command {

    my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                        bmc_command => {type=>SCALAR},
            # boot_up is useful in cases where we don't want to  check the filer that it has booted up.
                        boot_up => { type=>SCALAR, default=>"yes", optional=>1 },
    }
    );

    my $conn = make_conn($opts{host},"#console");


    my $cmd = $opts{bmc_command};

    my $BMC_PROMPT = "^bmc.*->";
    if($cmd eq "system power cycle"){

        say($conn,"\cG",-noprompt,-prompt,$BMC_PROMPT,-timeout,$opts{timeout});
        say($conn, $cmd, "Continue\\? (?i:\\[y\\/N\\])", -noprompt, -swallow );
        say($conn, "y",-noprompt,-prompt,$BMC_PROMPT ,-timeout,$opts{timeout}, -swallow );
        say($conn, "exit", -blind, -swallow );
        if($opts{boot_up} eq "yes"){
            transit_to($opts{host},"UP");
        }
        #wait_for_rsh($conn,timeout=>$opts{timeout});
    }
    elsif($cmd eq "system power off"){
        say($conn,"\cG",-noprompt,-prompt,$BMC_PROMPT,-timeout,$opts{timeout});
        say($conn, $cmd, "Continue\\? (?i:\\[y\\/N\\])", -noprompt, -swallow );
        say($conn, "y",-noprompt,-prompt,$BMC_PROMPT ,-timeout,$opts{timeout}, -swallow );
        say($conn, "exit", -blind, -swallow );
    }
    elsif($cmd eq "system power on"){
        say($conn,"\cG",-noprompt,-prompt,$BMC_PROMPT,-timeout,$opts{timeout});
        say($conn, "$cmd",-noprompt,-prompt,$BMC_PROMPT ,-timeout,$opts{timeout}, -swallow );
        say($conn, "exit", -blind, -swallow );
        if($opts{boot_up} eq "yes"){
            transit_to($opts{host},"UP");
        }
    }
    elsif($cmd eq "system core"){
        say($conn,"\cG",-noprompt,-prompt,$BMC_PROMPT,-timeout,$opts{timeout});
        say($conn, $cmd, "Continue\\? (?i:\\[y\\/N\\])", -noprompt, -swallow );
        say($conn, "y",-noprompt,-prompt,$BMC_PROMPT ,-timeout,$opts{timeout}, -swallow );
        say($conn, "exit", -blind, -swallow );
        if($opts{boot_up} eq "yes"){
            transit_to($opts{host},"UP");
        }
    }
    elsif($cmd eq "system reset"){
        say($conn,"\cG",-noprompt,-prompt,$BMC_PROMPT,-timeout,$opts{timeout});
        say($conn, $cmd, "Continue\\? (?i:\\[y\\/N\\])", -noprompt, -swallow );
        say($conn, "y",-noprompt,-prompt,$BMC_PROMPT ,-timeout,$opts{timeout}, -swallow );
        say($conn, "exit", -blind, -swallow );
        if($opts{boot_up} eq "yes"){
            transit_to($opts{host},"UP");
        }
    }
    else{
        logresult('config',"Invalid command $cmd");
    }

}

#
=head2 run_scrub

    my @ret = run_scrub(host=>$host);

This subrutine Runs scrub on a system and verifies no corruption is found.

The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item timeout

(Optional): timeout value. Default is 300 secs.

=item volume

(Optional): Volume on which scrub is to be run. Default is all.

=item conntype

(Optional): Connection type to use. Default is #rsh.

=item fatal_on_failure

(Optional): FATAL if scrub finds an error. Default is no, which means it will just "fail" and continue.

=back

=cut


sub run_scrub {

    my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            volume => {type=>SCALAR,default=>undef,optional=>1},
            fatal_on_failure => {type=>SCALAR,default=>"no",optional=>1},
    }
    );


    my $r = make_conn($opts{host},$opts{conntype});
    my $vol;
    if(defined($opts{volume})){
        $vol = $opts{volume};
    }
    else{
        $vol = "";
    }
    #---------------------------------------------------------
    #   Stop scrub so that we can start it from beginning
    #---------------------------------------------------------
    # say($r,"priv set -q diag; aggr scrub stop $vol",-timeout, $opts{timeout});
    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"aggr scrub stop $vol");


    #---------------------------------------------------------
    #   Give scrub 30  secs to stop
    #---------------------------------------------------------

    sleep 30;
    # my ($a,$b,$output) = say($r, "aggr scrub start $vol",-timeout, $opts{timeout});
    my $output = filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"aggr scrub start $vol");
    if (!($output =~ /started scrub/g)) {
        logresult('fatal',"Scrub didnot start");
    }
    sleep 60;
    LABEL1: ;

    #---------------------------------------------------------
    #   Get all raid information from raid_config
    #---------------------------------------------------------

    my $ref = new RaidInfo($r);
    my %groups = $ref->all_raid_groups();
    my $k;
    my $v;
    my $scrub = 1;
    my $fatal = 0;

    #---------------------------------------------------------
    #   Go through each raidgroup and check if the volume is online.
    #---------------------------------------------------------

    while(($k,$v)= each(%groups)){
        my $vol1 = $v->{'tree'};
        my $state = $ref->aggregate("$vol1");

        if((($vol !~ /\w+/) || ($vol1 eq $vol)) && ($state->{'state_name'} eq "online")){
            #print Dumper($v);

    #---------------------------------------------------------
    #   "scrub_progress" -1 means that the scrub is still running
    #---------------------------------------------------------

            if($v->{'scrub_progress'} != -1){
                # More intelligence can be put here to calculate sleep time based on progrss made.
                logcomment ("Scrub in progress sleeping for 300 secs ...");
                sleep 300;
                goto LABEL1;
            }
        }

    }

    #---------------------------------------------------------
    #   Ok the scrub is finished on all raidgroups . now let's check for any errors found by the scrub.
    #---------------------------------------------------------

    $ref = new RaidInfo($r);
    %groups = $ref->all_raid_groups();
    while(($k,$v)= each(%groups)){
        my $vol1 = $v->{'tree'};
        my $state = $ref->aggregate("$vol1");
        if((($vol !~ /\w+/) || ($vol1 eq $vol)) && ($state->{'state_name'}  eq "online")){

            if($v->{'scrub_pi'} != 0){
                logresult('fail',"SCRUB found PI in $k");
                $fatal = 1;
            }
            if($v->{'scrub_cksum_err'} != 0){
                logresult('fail',"SCRUB found checksum error in $k");
                $fatal = 1;
            }
            if($v->{'scrub_media_err'} != 0){
                logresult('fail',"SCRUB found media error in $k");
                $fatal = 1;
            }
        }

    }
    if($fatal == 0){
        logresult('pass',"Scrub ran clean");
    }
    else{
        if($opts{fatal_on_failure} eq "yes"){
            logresult('fatal',"Scrub found errors. Check the failures above");
        }
        else{
            logresult('fail',"Scrub found errors. Check the failures above");
        }

    }


}


=head2 start_filerhammer

    my @ret = start_filerhammer(host=>$host);

This subrutine runs filer hammer on a system

The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item timeout

(Optional): timeout value. Default is 300 secs.

=item volume

(Optional): Volume on which hammer is to be run. Default is all.

=item conntype

(Optional): Connection type to use. Default is #rsh.

=item num

(Optional): Number of hammers to run on each volume. Default is 1.

=item size

(Optional): Size of hammer. Default is 10000


=back

=cut

sub start_filerhammer {

    my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            volume => {type=>SCALAR,default=>undef,optional=>1},
            size => {type=>SCALAR,default=>undef,optional=>1},
            num => {type=>SCALAR,default=>1,optional=>1},
    }
    );


    my $r = make_conn($opts{host},$opts{conntype});
    my $vol;
    my $i;
    my @online_volumes = ();
    my $size;
    if(defined($opts{size})){
        $size = $opts{size};
    }
    else{
        $size = 10000;
    }
    if(defined($opts{volume})){
        @online_volumes = $opts{volume};
    }
    else{
        $vol = "";
        print "Volume $vol \n";
        my $ref = new RaidInfo($r);
        my %vols = $ref->all_volumes();
        my $k;
        my $v;
    #---------------------------------
    #   Get a list of online volumes
    #----------------------------------
        while(($k,$v)= each(%vols)){
            #print Dumper($v);
            if($v->{'wafl_mounted'} == 1){
                push @online_volumes, $k;
            }
        }
    }
    print "Online volumes: @online_volumes \n";
    my $host = host($opts{host});
    for($i=1 ; $i <= $opts{num};$i++){
        foreach $vol (@online_volumes){
           # Checking for the filer mode
            filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"hammer -f -1 /vol/$vol/ham.$i $size");
            # if ($host->hosttype =~ /filer-ng/ ) {
                # say($r,"run local \"priv set -q diag;hammer -f -1 /vol/$vol/ham.$i $size\"",-timeout, $opts{timeout});
            # } else {
                # say($r,"priv set -q diag;hammer -f -1 /vol/$vol/ham.$i $size",-timeout, $opts{timeout});
            # }
        }
    }
}

=head2 abort_filerhammer

    my @ret = abort_filerhammer(host=>$host);

This subrutine aborts filer hammer on a system

The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item timeout

(Optional): timeout value. Default is 300 secs.

=item conntype

(Optional): Connection type to use. Default is #rsh.

=back

=cut


sub abort_filerhammer {

    my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                        filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
    }
    );


    my $r = make_conn($opts{host},$opts{conntype});
    my $wait_interval = 5;
    my $max_wait = 1800;
    my $wait = 0;
    my $hammer_status = "abc";
    my $host = host($opts{host});
    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"hammer abort");
    sleep $wait_interval;
    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"hammer status");
    sleep $wait_interval;
}
=head2 pause_filerhammer

    my @ret = pause_filerhammer(host=>$host);

This subrutine aborts filer hammer on a system

The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item timeout

(Optional): timeout value. Default is 300 secs.

=item conntype

(Optional): Connection type to use. Default is #rsh.

=back

=cut


sub pause_filerhammer {

    my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
    }
    );


    my $r = make_conn($opts{host},$opts{conntype});
    my $wait_interval = 5;
    my $max_wait = 1800;
    my $wait = 0;
    my $hammer_status = "abc";
    # say($r,"priv set -q diag;hammer pause" ,-timeout,  $opts{timeout});
    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"hammer pause");
    sleep $wait_interval;
    while(($hammer_status ne "") && ($wait < $max_wait)){
        # $hammer_status = say($r,"priv set -q diag;hammer status",-timeout, $opts{timeout});
        $hammer_status = filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"hammer status");
        $wait += $wait_interval;
        sleep $wait_interval;
    }
}
=head2 restart_filerhammer

    my @ret = pause_filerhammer(host=>$host);

This subrutine aborts filer hammer on a system

The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item timeout

(Optional): timeout value. Default is 300 secs.

=item conntype

(Optional): Connection type to use. Default is #rsh.

=back

=cut


sub restart_filerhammer {

    my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                        timeout => { type=>SCALAR, default=>300, optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
    }
    );


    filer_cmd(host=>$opts{host},timeout=>$opts{timeout},cmd=>"hammer restart");
}


=head2 set_boot_args

     set_bootargs(host=>$host);

This subrutine sets environmental variables. From the OFW/CFE/LOADER prompt.

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item timeout

(Optional): timeout value. Default is 20 secs.

=item conntype

(Optional): Connection type to use. Default is #rsh.

=item setdefaults

(Optional): "no" to set gdb-on-panic?,gdb-on-nocore?, gdb-on-recursive-panic?  as true, otherwise it will set these as false. Default is "no".

=item option

(Optional): To set a specific parameter. Default is "undef"

=item value

(Optional): To set value of the parameter defined in the option above. Default is "undef"

=back

=cut

sub set_bootargs {

# Later we can put a check if it's a debug build use "bootargs" command.

    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            setdefault => {type=>SCALAR,default=>"no",optional=>1},
            option => {type=>SCALAR,default=>undef,optional=>1},
            value => {type=>SCALAR,default=>undef,optional=>1},
    }
    );
    my ($k,$v);
    if(is_debug(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout})){
        my $r = make_conn($opts{host},$opts{conntype});
        my $host = host($opts{host});
        if(defined($opts{option}) && defined($opts{value})){
            filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"priv set -q test;bootargs set $opts{option} $opts{value}");
        }
         else{

                 if($opts{setdefault} eq "yes"){
                    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"priv set -q test;bootargs set gdb-on-panic? false");
                    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"priv set -q test;bootargs set gdb-on-nocore? true");
                    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"priv set -q test;bootargs set floppy-boot? false");
                }
                else{
                    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"priv set -q test;bootargs set gdb-on-panic? true");
                    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"priv set -q test;bootargs set gdb-on-nocore? true");
                    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"priv set -q test;bootargs set gdb-on-recursive-panic? true");
                    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"priv set -q test;bootargs set floppy-boot? false");
                    filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"priv set -q test;bootargs set AUTOBOOT true");
              }
           }
        } else {
            my $host = $opts{host};
            my $transit_obj = NACL::Transit->new( name => $host );
            $transit_obj->change_state( to => 'FIRMWARE', timeout => "1800" );
            my $r = make_conn($host,"#console");

            if(defined($opts{option}) && defined($opts{value})){
                say($r,"setenv $opts{option} $opts{value}",-prompt,$host->{prompt_fw},-timeout,$opts{timeout});
             }
            else{
                 if($opts{setdefault} eq "yes"){
                     say($r, "setenv gdb-on-panic? false",-prompt,$host->{prompt_fw},-timeout,$opts{timeout});
                    sleep 1;
                    say($r, "setenv gdb-on-nocore? false",-prompt,$host->{prompt_fw},-timeout,$opts{timeout});
                    sleep 1;
                    say ($r, "setenv floppy-boot? false",-prompt,$host->{prompt_fw},-timeout,$opts{timeout});
                    sleep 1;
                }
                else{
                    say($r, "setenv gdb-on-panic? true",-prompt,$host->{prompt_fw},-timeout,$opts{timeout});
                    sleep 1;
                    say($r, "setenv gdb-on-nocore? true",-prompt,$host->{prompt_fw},-timeout,$opts{timeout});
                    sleep 1;
                    say($r, "setenv gdb-on-recursive-panic? true",-prompt,$host->{prompt_fw},-timeout,$opts{timeout});
                    sleep 1;
                    say ($r, "setenv floppy-boot? false",-prompt,$host->{prompt_fw},-timeout,$opts{timeout});
                    sleep 1;
                    say ($r, "setenv AUTOBOOT true",-prompt,$host->{prompt_fw},-timeout,$opts{timeout});
                    sleep 1;
              }
            }
            boot($r);
        }
}

=head2 is_debug

    my $ret = is_debug(host=>$host);

This routine is used to findout whether a system is running debug build or not
It return 1 if build  is debug and 0 if not.

=over

=item host

(Required) : Filer connnection to use.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut


sub is_debug {

    my (%opts) = validate(@_, {
        host      => { type=>OBJECT, isa=>'Hostrec' },
        filermode => { type=>SCALAR, default=>"normal",optional=>1 },
        conntype  => { type=>SCALAR, default=>"#rsh",optional=>1 },
        timeout   => { type=>SCALAR, default=>300 ,optional=>1 },
        });

    use Storage::Misc qw (misc_filer_version);

    my $hash = misc_filer_version(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout});
    #print Dumper($hash);
    if( $hash->{debug} =~ /d/){
        #print "Debug Build \n";
        return 1;
    }
    else{
        return 0;
    }
}

=head2 disktest

    my $ret = disktest(host=>$host);

This routine is to run disktest
It return 1 if build  is debug and 0 if not.

=over

=item host

(Required) : Filer connnection to use.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut


sub disktest($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                adapter_name => {type=>SCALAR,default=>undef,optional=>1},
                disk_list => {type=>SCALAR,default=>undef,optional=>1},
                timeout => {type=>SCALAR,default=>300,optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                option => {type=>SCALAR,default=>"-B -y",optional=>1},
        }
    );
    my $output = "";
    my $adapter_name;
    my $a = "";
    my $b = "";
    my $mark;
    if ($opts{filermode} ne "normal") {
        $opts{conntype} = "#console";
    }

    my $conn = make_conn($opts{host},$opts{conntype});
    my $options = $opts{option};
    if(defined($opts{adapter_name})){
        $options = "$options" . " " . "-a $opts{adapter_name}";
    }
    if(defined($opts{disk_list})){
        $options = "$options" . " " . "-d $opts{disk_list}";
    }
    if(defined($opts{shelf_id})){
        if(!defined($opts{adapter_name})){
            my $err = "Adapter name should be supplied when shelf_id is specified";
            croak $err ;
        }
        my $shelf = "$opts{adapter_name}" . ":" . "$opts{shelf_id}";
        $options = "$options" . " " . "-s $opts{disk_list}";
    }


    if ($opts{filermode} ne "normal") {
        my $cmd = "disktest $options";
            (undef, undef, $output) = say($conn, "$cmd", -prompt, "$MAINT_PROMPT", "-timeout", $opts{timeout});
        if($output !~ /Running disk test/) {
#           print "\n O $output slot is $slot \n";
#           sleep 1000;
            croak $output ;
        }
    }
    else{
        my $cmd = "priv set -q test;disktest $options";
        #print "OPTIONS are $cmd \n";
        # (undef, undef, $output) = say($conn, "$cmd","-timeout", $opts{timeout});
        $output=filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        sleep 5;
        if($output !~ /Running disk test/) {
            croak $output ;
        }
    }
    my @c = split(/\r*\n/,$output);
    my $i = 0;
    my $struct = {};
    my $sas = 0;
    my $fcal = 0;
    my $scsi = 0;
    foreach my $d (@c){
        #print "$i => $d \n";
        $i++;
        my @arr = split(/\s+/,$d);
        if(defined($arr[$#arr])){
        if ($arr[$#arr] =~ /FC-AL/) {
            $fcal = 1;
            $sas = 0;
            $scsi = 0;
        }
        elsif($arr[$#arr] =~ /SAS/){
            $sas = 1;
            $fcal = 0;
            $scsi = 0;
        }
        elsif($arr[$#arr] =~ /SCSI/){
            $sas = 0;
            $fcal = 0;
            $scsi = 1;
        }
        #   my $j = $i + 2 ;
    #
        if($fcal){
            if(($arr[1] =~ /^\d+$/) && ($arr[2] =~ /^\d+$/)){
                $struct->{$arr[0]}->{LinkFailureCount} = $arr[1];
                $struct->{$arr[0]}->{TransportErrorCount} = $arr[2];
                $struct->{$arr[0]}->{LossOfSyncCount} = $arr[3];
                $struct->{$arr[0]}->{InvalidCRCCount} = $arr[4];
                $struct->{$arr[0]}->{FrameInCount} = $arr[5];
                $struct->{$arr[0]}->{FrameOutCount} = $arr[6];
                $struct->{$arr[0]}->{ConfidenceFactor} = $arr[7];
            }
        }
        if($sas){
            if(($arr[2] =~ /^\d+$/) && ($arr[3] =~ /^\d+$/)){
                $struct->{$arr[1]}->{InvDWrdCnt} = $arr[2];
                $struct->{$arr[1]}->{DispErrCnt} = $arr[3];
                $struct->{$arr[1]}->{LossSyncCnt} = $arr[4];
                $struct->{$arr[1]}->{PhyResProb} = $arr[5];
                $struct->{$arr[1]}->{PhyChngCtH} = $arr[6];
                $struct->{$arr[1]}->{PhyChngCtS} = $arr[7];
                $struct->{$arr[1]}->{DevUdrnErrs} = $arr[8];
                $struct->{$arr[1]}->{DevOvrnErrs} = $arr[9];
                $struct->{$arr[1]}->{TrnsErrcnt} = $arr[10];
                $struct->{$arr[1]}->{CRCErrCnt} = $arr[11];
                $struct->{$arr[1]}->{DevQsceCnt} = $arr[12];
                $struct->{$arr[1]}->{FrmsInCnt} = $arr[13];
                $struct->{$arr[1]}->{FrmsOutCnt} = $arr[14];
                if(defined($arr[15])){
                    $struct->{$arr[1]}->{ConfidenceFactor} = $arr[15];
                }
            }
        }
        if($scsi){
            if(($arr[1] =~ /^\d+$/) && ($arr[2] =~ /^\d+$/)){
                $struct->{$arr[0]}->{WriteCount} = $arr[1];
                $struct->{$arr[0]}->{ReadCount} = $arr[2];
                $struct->{$arr[0]}->{IOPS} = $arr[3];
                $struct->{$arr[0]}->{DataRate} = $arr[4];
                $struct->{$arr[0]}->{DataXferSize} = $arr[5];
                my $serror = $arr[6];
                $serror =~ s/\///;
                $struct->{$arr[0]}->{DeviceErrorsS} = $serror;
                $struct->{$arr[0]}->{DeviceErrorsH} = $arr[7];
                $struct->{$arr[0]}->{ConfidenceFactorSCSI} = $arr[8];
            }
        }

    }
    }
    return $struct ;

}
=head2 get_timestamp

    my $ret = get_timestamp();

This routine is to get a timestamp value. Output is something like: 09262007153101.
Which is : month-date-year-hour-minute-second

=over

=back

=cut



sub get_timestamp {

        my %month = (
        "Jan" => "01",
        "Feb" => "02",
        "Mar" => "03",
        "Apr" => "04",
        "May" => "05",
        "Jun" => "06",
        "Jul" => "07",
        "Aug" => "08",
        "Sep" => "09",
        "Oct" => "10",
        "Nov" => "11",
        "Dec" => "12",
    );


    my $time = time();
    my $local = localtime();
    $local =~ s/://g;
    my @local = split(/\s+/,$local);
    my $i= 0;
    my $date = "$month{$local[1]}" . "$local[2]" . "$local[4]" .  "$local[3]";
    return $date;
}



=head2 filer_cmd

    my $ret = filer_cmd();

This routine is to issue command to filer and return the output

=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".

If filer in maintenance mode, the connection type will be "#console".

In case of "normal". If filer is running 10-mode (hostype filer-ng in the host file). It will use "nodescope". Otherwise use regular say via rsh.

filermode "ngsh" is not yet implemented

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=item privmode

(optional) : "priv set" mode for the command. Default is "diag". In case command is vaiable only in test mode then we need to use "test".

=item noprompt

(optional) : In case we just want to issue to command and don't want to wait for the prompt. Default is 0, i.e, it waits for the prompt.


=item cmd


(Required) : command to be issued.

=back

=cut


sub filer_cmd {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            node => {type=>HASHREF, default=>undef, optional=>1},
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => {type=>SCALAR, default=>300,optional=>1},
            ignore_error => {type=>SCALAR, default=>1,optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            privmode => {type=>SCALAR,default=>"diag",optional=>1 },
            noprompt => {type=>SCALAR,default=>0,optional=>1 },
            cmd => {type=>SCALAR},
            }
            );
    my $output;
    my $conn;
    if ($opts{filermode} eq "maint") {
        $opts{conntype} = "#console";
        $conn = make_conn($opts{host},$opts{conntype});
        my $cmd = "$opts{cmd}";
        if($opts{noprompt}){
            (undef, undef, $output) = say($conn, "$cmd", -noprompt , "-timeout", $opts{timeout});
        }
        else{
            (undef, undef, $output) = say($conn, "$cmd", -prompt, "$MAINT_PROMPT", "-timeout", $opts{timeout});
        }

    }
    elsif ($opts{filermode} eq "normal"){
        if($opts{host}->{hosttype} =~ /filer-ng/){
            my $server= new Server;
            if (defined $opts{node}) {
                $server->set_execution_server(-Server => $opts{node});
            } else {
                $server->set_execution_server(-Server => $opts{host}->{hostname});
            }

            my $cmd;
            if(($opts{cmd} eq "Yes")||($opts{cmd} eq "yes")||($opts{cmd} eq "y")||($opts{cmd} eq "No")||($opts{cmd} eq "no")||($opts{cmd} eq "n")) {
                $cmd=$opts{cmd};
            } else {
                $cmd = "run local  \"priv set -q $opts{privmode};$opts{cmd}\"";
            }
            if($opts{noprompt} && ($opts{conntype} eq "#rsh")){
                $output = $server->cli(cmd=>"$cmd",-Timeout=>$opts{timeout},-Ignore_errors=>$opts{ignore_error},noprompt=>1);
            }
            elsif($opts{noprompt} && ($opts{conntype} eq "#console")){
                $output = $server->console(cmd=>"$cmd",-Timeout=>$opts{timeout},-Ignore_errors=>$opts{ignore_error},noprompt=>1);
            }
            elsif($opts{conntype} eq "#console"){
                $output = $server->console(cmd=>"$cmd",-Timeout=>$opts{timeout},-Ignore_errors=>$opts{ignore_error});
            }
            else{
                $output = $server->cli(cmd=>"$cmd",-Timeout=>$opts{timeout},-Ignore_errors=>$opts{ignore_error});
            }
        }
        else{
            $conn = make_conn($opts{host},$opts{conntype});
            my $cmd = "priv set -q $opts{privmode};$opts{cmd}";
            if($opts{noprompt}){
                (undef, undef, $output) = say($conn, "$cmd",-noprompt,"-timeout", $opts{timeout});
            }
            else{
                (undef, undef, $output) = say($conn, "$cmd","-timeout", $opts{timeout});
            }

        }
    }
    elsif($opts{filermode} eq "ngsh"){
        logcomment("NOT YET IMPLEMENTED");
    }
    else{
        die "Unsupported filermode";
    }
    return $output;
}


=head2 panic_filer

    my $ret = panic_filer();

This routine panics the filer and returns a hash of output from panic command

Example of hash returned:



$VAR1 = {

          'DumpcoreFlag' => '0x0',

          'Version' => 'RironcityN_090921_2230',

          'CoredumpDisks' => '4d.22.11 4d.22.12 4d.22.16 4d.22.19 4d.22.18 4d.22.5 4d.22.0 4d.22.14 4d.22.21 4d.22.17 4d.22.9 4d.22.2 4d.22.15 4d.22.13 4d.22.6 4d.22.1 4d.22.7',


          'output' => 'priv set -q diag;panic SCRIPTTEST


PANIC: reboot -d SCRIPTTEST in process xor_other_thread on release NetApp Release RironcityN_090921_2230 on Tue Sep 22 22:02:05 GMT 2009


DBG:scsi.c:5394:Storage adapters reinitializing for core dump
DBG:pmcsas_mpi.c:222:pmcsas[4c]: pmcsas_mpi_soft_reset
DBG:pmcsas_mpi.c:222:pmcsas[4a]: pmcsas_mpi_soft_reset
version: NetApp Release RironcityN_090921_2230: Mon Sep 21 22:43:55 PDT 2009
cc flags: 2d

DUMPCORE: START
DBG:core.c:2109:Dumping core: reboot -d SCRIPTTEST in process xor_other_thread on release NetApp Release RironcityN_090921_2230.
DBG:pmcsas_asyncd.c:8877:pmcsas[4c]: QSFP 0 PRESENCE 0
DBG:pmcsas_asyncd.c:8877:pmcsas[4c]: QSFP 1 PRESENCE 1
DBG:pmcsas_intrd.c:627:pmcsas[4c]: PORT UP -- 4d
DBG:pmcsas_timeoutd.c:410:pmcsas[4c]: PHYs didn\'t come up.
DBG: 4144 msecs elapsed between kicks from location 3 and location 3
DumpcoreFlag = 0x0
Dumping to disks: 4d.22.11 4d.22.12 4d.22.16 4d.22.19 4d.22.18 4d.22.5 4d.22.0 4d.22.14 4d.22.21 4d.22.17 4d.22.9 4d.22.2 4d.22.15 4d.22.13 4d.22.6 4d.22.1 4d.22.7
DBG: 676152 blocks are needed for the dump
DBG: compression deferred until savecore time.
DBG:core.c:6281:There are 4 segments to spray
.....................................
DUMPCORE: END -- '


        };


=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#console".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.

=item nowait

(optional) : In case we just want to issue to command and don't want to wait for any string to check. Default is 0, i.e, it waits for the prompt.


=item  returnbeforedump

(optional) : With ths option set to 1 , function returns as soon as coredump starts. This is useful if we want to do some operations while coredump is in progress. For example: resetting drives while dumping core. Default is 0, i.e, it will wait for the coredump to finish.

=item string

(Optional) : Panic string to use with panic command. Default is "SCRIPT".

=back

=cut


sub panic_filer {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                conntype => {type=>SCALAR,default=>"#console",optional=>1},
                timeout => {type=>SCALAR, default=>800,optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                privmode => {type=>SCALAR,default=>"diag",optional=>1 },
                nowait => {type=>SCALAR,default=>0,optional=>1 },
                returnbeforedump => {type=>SCALAR,default=>0,optional=>1 },
                string => {type=>SCALAR,default=>"SCRIPT",optional=>1 },
        }
    );

    my $output;
    my $a;
    my $b;
    my $conn;
    my $hash;
    my @array;
    my $filer_ci;
     $filer_ci = NACL::C::Node->find();
     my $transit_obj = NACL::Transit->new( name => $opts{host}->{hostname}, 'timeout' => 1800 );
     if ($opts{filermode} eq "normal"){
        $conn = make_conn($opts{host},$opts{conntype});
        my $cmd = "priv set -q diag;panic $opts{string}";
        if($opts{nowait}){
            if ( $filer_ci->is_cmode() ) {
                $output = $transit_obj->{conn}->execute_start(
                    command => 'run local "'.$cmd.'"',
                );
            } else {
                $output = $transit_obj->{conn}->execute_start(
                    command => "$cmd",
                );
            }
            $hash->{output} = $output;
        }
        else{
            if($opts{returnbeforedump}){
                my $match = ".*blocks are needed for the dump\r*\n";
                if ( $filer_ci->is_cmode() ) {
                    $output = $transit_obj->{conn}->execute(
                        command => 'run local "'.$cmd.'"',
                        prompts => [$match],
                        timeout => 3600
                    );
                } else {
                    $output = $transit_obj->{conn}->execute(
                        command => "$cmd",
                        prompts => [$match],
                        timeout => 3600
                    );
                }
            }
            else {
                if ( $filer_ci->is_cmode() ) {
                    $output = $transit_obj->{conn}->execute(
                        command => 'run local "'.$cmd.'"',
                        prompts => ["coredump written"],
                        timeout => 3600
                    );
                } else {
                    $output = $transit_obj->{conn}->execute(
                        command => "$cmd",
                        prompts => ["coredump written"],
                        timeout => 3600
                    );
                }
            }

            @array = split(/\r*\n/,$output);

            foreach my $line (@array){
                if($line =~ /^version:/){
                    my @array1 = split(/\s+/,$line);
                    $array1[3] =~ s/://g;
                    $hash->{Version} = $array1[3];
                }
                elsif($line =~ /^DumpcoreFlag/){
                    my @array1 = split(/\s+/,$line);
                    $hash->{DumpcoreFlag} = $array1[2];
                }
                elsif($line =~ /^Dumping to disks:/){
                    my @array1 = split(/Dumping to disks:\s+/,$line);
                    $hash->{CoredumpDisks} = $array1[1];
                }
            }
            $hash->{output} = $output;
        }
    }
    return $hash ;
}
=head2 coredump_disks

    my $ret = coredump_disks();

This routine returns an array of disks to which core is being dumped.



=over

=item input

(Required) : HASHREF returned by the panic_filer() function above

=back

=over

=cut

sub get_coredump_disks {
    my (%opts) = validate(@_,{
                input => {type=>HASHREF},
        }
    );
    my @disks = split(/\s+/,$opts{input}->{CoredumpDisks});
    return @disks;

}


=head2 get_filer_options

    my $ret = get_filer_options();

This routine gets options from the filer and return a hash reference of options.

Example of hash returned:

$VAR1 = {

          'disk.maint_center.max_disks' => '84',

          'disk.maint_center.rec_allowed_entries' => '5',

          'disk.recovery_needed.count' => '5',

          'disk.maint_center.spares_check' => 'on',

          'disk.auto_assign' => 'on',

          'disk.maint_center.enable' => 'on',

          'disk.maint_center.allowed_entries' => '1'

        };


=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.

=item options

(optional) : To get output of a particular option


=back

=cut

sub get_filer_options{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            options => { type=>SCALAR,default=>"", optional=>1 },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            }
            );

    my $hash;
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"options $opts{options}");
#print "output is : $output \n";
    if ($output !~ /$opts{options}/){
        croak $output;
    }
    if ($output =~ /No such option/){
        croak $output;
    }
    my @optionslist = split(/\r*\n/,$output);
    foreach my $option (@optionslist){
        if($option =~ /\./){
            my @array = split(/\s+/,$option);
            $hash->{$array[0]} = $array[1];
        }
    }
    return $hash;
}

=head2 set_filer_options

    my $ret = set_filer_options();

This routine sets options on the filer


=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.

=item options

(Required) : To get output of a particular option

=item value

(Required) : Value we want to set for above option

=item checkoptions

(Optional) : To verify that the option is actually set. If set to 0 then no check. Default 1


=back

=cut


sub set_filer_options{

    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            options => { type=>SCALAR},
            value => { type=>SCALAR},
            checkoptions => {type=>SCALAR,default=>1,optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            }
            );

    my $options = $opts{options};
    my $value = $opts{value};
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"options $options $value");
    if ($output =~ /(No such option)|(invalid value)/i){
        croak $output;
    }
    if($opts{checkoptions}){
        #my $option = "$options $value";
        sleep 1;
        my $hash = get_filer_options(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},options=>$opts{options},timeout=>$opts{timeout});
        if($hash->{$options} ne "$value"){
            print Dumper $hash;
            croak "Option $options not $value";
        }
    }
    #return 1;
}

=head2 get_core_list()

    my $ret = get_core_list()();

This routine issues savecore -l command and returns a hash of corefiles.

$VAR1 = {

          'core.118060110.2009-09-22.22_02_05.nz' => {

                                                       'OSVersion' => 'Release',

                                                       'PanicString' => 'reboot -d SCRIPTTEST in process xor_other_thread on release NetApp Release RironcityN_090921_2230',

                                                       'PanicTime' => 'Tue Sep 22 22:02:05 GMT 2009'

                                                     },

          'output' => 'core.118060110.2009-09-22.22_02_05.nz

        Panic string: reboot -d SCRIPTTEST in process xor_other_thread on release NetApp Release RironcityN_090921_2230

        Panic time: Tue Sep 22 22:02:05 GMT 2009

        OS version: NetApp Release RironcityN_090921_2230: Mon Sep 21 22:43:55 PDT 2009


'
        };



=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.


=back

=cut


sub get_core_list{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            }
            );

    my $hash;
    my $corefile;
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"savecore -l");
    if($output =~ /core\./){
        $hash->{output} = $output;
        my @op = split(/\r*\n/,$output);
        foreach my $op (@op){
            if($op =~ /core\./){
                $op =~ s/\s+//g;
                $corefile = $op;
                if ($corefile =~ /nzwillbecreatedby/) {
                    logcomment("DEBUG_DT_get_core_list\{\}\:\: Core file generation is still in progress. Retry counter may be reset during this stage.");

                    last;     # just exit get_core_list{} for the core file is still generating.
                }

            }
            elsif($op =~ /Panic string:/){
                $op =~ s/\s+Panic string:\s+//;
                $op =~ s/\s+$//;
                $hash->{$corefile}->{PanicString} = $op;
            }
            elsif($op =~ /Panic time:/){
                $op =~ s/\s+Panic time:\s+//;
                $op =~ s/\s+$//;
                $hash->{$corefile}->{PanicTime} = $op;
            }
            elsif($op =~ /OS version:/){
                my @op1 = split(/\s+/,$op);
                $op1[4] =~ s/://g;
                $hash->{$corefile}->{OSVersion} = $op1[4];
            }
        }
    }
    else{
        $hash->{output} = $output;
    }
    return $hash;

}
=head2 savecore()

    my $ret = savecore();

This routine issues savecore command and returns the output returned


=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.

=item option

(optional) : savecore option. "-l" or "-k" or other supported options. Default is " ".


=back

=cut

sub savecore{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            option => { type=>SCALAR, default=>" ", optional=>1 },
            }
            );

    my $hash;
    my $corefile;
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"savecore $opts{option}");
    if($output =~ /Usage|Invalid option/){
        croak $output;
    }
}

=head2 findcore()

    my $ret = findcore();

This routine looks for a core matching particular panic string in savecore -l output


=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.

=item string

(Required) : Panic string to look for.

=item retry

(Required) : Number of retries to find core. Default is 20. The function will retry 20 times every 30 secs to find a core. So  by default it will give 600 secs to finish savecore.


=back

=cut

sub find_core{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            retry => { type=>SCALAR, default=>30, optional=>1 },
            string => { type=>SCALAR},
            }
            );
    my $retry = 0;
    logcomment("Looking for core file containing string : $opts{string}");
    FIND_CORE_LABEL: ;
    my $hash = get_core_list(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout});
    if($retry >= $opts{retry}){
        goto FIND_CORE_LABEL1;
    }
    if($hash->{output} =~ /savecore operation is in progress/){
        logcomment("Sleep for 60 secs before trying to get a core list. Retry $retry");
        sleep 60;
        $retry++;
        goto FIND_CORE_LABEL;
    }
    elsif($hash->{output} =~ /will be created by the next savecore command/){
        logcomment("Sleep for 60 secs before trying to get a core list. Retry $retry");
        sleep 60;
        $retry++;
        if($retry > 3){ # 3 retries should be enough for this case.
            logresult("warn","Savecore for string $opts{string} didnot succeed and coredump cane be saved in the next savecore command. Probably the root volume is full? Check using df command");
            goto FIND_CORE_LABEL1;
        }
        else{
            goto FIND_CORE_LABEL;
        }

    }
    FIND_CORE_LABEL1: ;
    if($hash->{output} =~ /$opts{string}/){
        logcomment("Panic string $opts{string} found");
        return 1;
    }
    elsif($opts{hash}->{output} =~ /$opts{string}/){
        croak "Panic string $opts{string}  not found in savecore -l output";
    }
}

=head2 get_ha_disk_map()

    my $ret = get_ha_disk_map();

This routine returns a hash /map of drive ids between an ha pair. For example if drive id  is 1a.02.1 on a node and the same drive serial number is 5a.02.1 from the partner then hash key pair will be 1a.02.1 => 5a.02.1.
This is useful when we want to do some operations from both nodes on the same drive.

Output returned is like:

$VAR1 = {

          '1c.22.16' => '4d.22.16',

          '1c.22.20' => '4d.22.20',

          '1c.12.13' => '4d.12.13',

          '1c.12.22' => '4d.12.22',

          '1c.22.14' => '4d.22.14',

          '1c.22.13' => '4d.22.13',

          '1c.12.19' => '4d.12.19',


};


=over

=item host

(Required) : Filer connnection to use. Hostrec

=item hostB

(Required) : Partner filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.


=back

=cut


sub get_ha_disk_map{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            hostB => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            }
            );
    my $hash1;
    my $struct;
    my $disk1 = storage_show_disk(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout});
    my @disks1 = keys %{$disk1};
    #print "Disks @disks1 \n";
    foreach my $d (@disks1){
        my $serial = $disk1->{$d}->{Serial};
        $hash1->{$serial} = $d;
    }
    my @array;
    my $disk2 = storage_show_disk(host=>$opts{hostB},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout});
    #print Dumper $disk2 ;
    my @disks2 = keys %{$disk2};
    #print "Disks @disks2 \n";
    foreach my $d (@disks2){
        my $serial = $disk2->{$d}->{Serial};
        if(defined($hash1->{$serial})){
            $struct->{$hash1->{$serial}} = $d;
        }
        else{
            logcomment("Disk $d on $opts{hostB}->{hostname} doesn't have corresponding disk on the partner with the same serial number");
        }
    }
    return $struct;
}

=head2 rmfile()

    my $ret = rmfile();

This routine removes a file from the filer by using rm command


=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.

=item filename

(Required) : File to be removed


=back

=cut

sub rmfile{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            filename => { type=>SCALAR},
            }
            );

    my $hash;
    my $corefile;
    logcomment("Removing file $opts{filename} from the filer");
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"rm $opts{filename}");
    if($output =~ /No such file|usage|Invalid option/){
        croak $output;
    }
}

=head2 make_single_path()

    my $ret = make_single_path();

This routine removes a file from the filer by using rm command


=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.

=item filename

(Required) : File to be removed


=back

=cut

sub make_single_path{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            path    => {type=>SCALAR,default=>undef,optional=>1},
            }
            );

    my $host=$opts{host};
    my $hash;
    my $corefile;
    my @offlined_adapters;
    my $adapt_to_disable;
    my $filer = $opts{host}->{hostname};
    my $path =$opts{path};

    logcomment("Making $filer single path");
    $hash = storage_show_adapter(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout});
    my @adapters = keys %{$hash};
    print "Adapters: @adapters \n";
    foreach my $adapt (@adapters){
        if(($hash->{$adapt}->{'State'} =~ /Enabled/) && ($hash->{$adapt}->{'Redundant'} =~ /^Yes/)){
           if($path eq "A") {
                    $adapt_to_disable=get_adapter_name(host=>$opts{host},shelf_path=>"B",adapter_name=>$adapt);
           }
           elsif($path eq "B") {
                     $adapt_to_disable=get_adapter_name(host=>$opts{host},shelf_path=>"A",adapter_name=>$adapt);
           }
           else{
                $adapt_to_disable=$adapt;
           }
           logcomment("Disabling adapter $adapt_to_disable");
           storage_disable_adapter(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},adapter_name=>$adapt_to_disable);
           push @offlined_adapters,$adapt_to_disable;
           sleep 30; # Give it some time to settle down
           $hash = storage_show_adapter(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout});
       }
    }
    return @offlined_adapters;
}

=head2 offline_aggrs()

    my $ret = offline_aggrs();

This routine offline all online non root volumes/aggregates.
Returns 1 if succeeds and 0 if fails.

=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.



=back

=cut


sub offline_aggrs {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            }
            );


    use RaidInfo;
    my @online_volumes = ();
    my $r = make_conn($opts{host},$opts{conntype});

# Find all the online volumes on the filer
    my $ref = new RaidInfo($r);
    my %vols = $ref->all_aggregates();
    my $k;
    my $v;
    my $op;
    my $err =0;
    abort_filerhammer(host=>$opts{host},timeout=>$opts{timeout});
    sleep 20;
    while(($k,$v)= each(%vols)){
#   print Dumper($v);
# Skip root aggregate for running traffic.
        if(($v->{'state'} == 1) && ($v->{'root'} == 0)){
            logcomment("Offlining vol/aggr $k");
            my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"aggr offline $k -a -f");
            if($output !~ /now offline/){
                logresult("warn", "Unable to offline volume $v");
                $err = 1;
            }

        }
    }
    if($err){
        return 0;
    }
    else{

        return 1;
    }
}

=head2 online_aggrs()

    my $ret = online_aggrs();

This routine tries to online  offline aggregates
Returns 1 if succeeds and 0 if fails.

=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.

=item aggrs

(Optional) : Comma or space seperated list of aggregates to be onlined. Default undef and tries to online all offlined aggregates


=back

=cut



sub online_aggrs {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            aggrs => { type=>SCALAR, default=>undef, optional=>1 },
            }
            );
    my @aggrs;
    my $err =0;
    if(defined($opts{aggrs})){
        @aggrs = split(/\s+|,/,$opts{aggrs});
    }
    else{
        my @offline_aggrs = ();
        my $r = make_conn($opts{host},$opts{conntype});
# Find all the online volumes on the filer
        my $ref = new RaidInfo($r);
        my %vols = $ref->all_aggregates();
        my $k;
        my $v;
        while(($k,$v)= each(%vols)){
            my $aggr_obj = $ref->aggregate("$k");
            if($aggr_obj->parameter("state_name") eq "offline"){
                push @aggrs,$k;
            }
        }
    }

    foreach my $aggr (@aggrs){
        logcomment("Onlining vol/aggr $aggr");
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"aggr online $aggr -f");
        if($output !~ /now online|already online/){
            logresult("warn", "Unable to online volume $aggr : $output");
            $err = 1;
        }

    }
    if($err){
        return 0;
    }
    else{

        return 1;
    }
}

=head2 wait_for_sdc_completion()

    my $ret = wait_for_sdc_completion();

This routine waits for aggregate to complete Sick disk copy
Returns 1 if succeeds and 0 if fails.

=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.

=item filer_vol

(Optional) : Aggregate/volume to check for SDC.


=back

=cut

sub wait_for_sdc_completion {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            filer_vol => {type=>SCALAR},
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            }
            );


    my $r = make_conn($opts{host},$opts{conntype});
    my $group_obj;
    my $ref = new RaidInfo($r);
    my $aggr_obj = $ref->aggregate($opts{filer_vol});
    my %group_list = $aggr_obj->all_raid_groups();
    my @group_list= keys(%group_list);
    foreach my $group (@group_list){
        $group_obj=$ref->raid_group("$group");
        while($group_obj->parameter("disk_copy_source_id") ne "-1"){
            logcomment("SDC flag set for group $group. Will check again after 120 secs");
            sleep 120;
            $ref->refresh();
            if($group_obj->parameter("disk_copy_target_id") eq "-1"){
                logcomment("disk_copy_target_id set as -1. Which means SDC didn't start which could be because no spare avaiable to start the sick disk copy");
                return 0;
            }
            $group_obj=$ref->raid_group("$group");
        }
    }
    logcomment("No SDC is in progress on $opts{filer_vol}");
    return 1;
}

sub get_aggr_list {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            state_name => {type=>SCALAR, default=>"online", optional=>1 },
            }
            );
    my @aggrs;
    my $r = make_conn($opts{host},$opts{conntype});
# Find all the online volumes on the filer
    my $ref = new RaidInfo($r);
    my %vols = $ref->all_aggregates();
    my $k;
    my $v;
    while(($k,$v)= each(%vols)){
        my $aggr_obj = $ref->aggregate("$k");
        if($aggr_obj->parameter("state_name") eq "$opts{state_name}"){
            push @aggrs,$k;
        }
    }
    return @aggrs;
}

=head2 unfail_all_disks()

    my $ret = unfail_all_disks();

This routine unfail all failed disks using "disk unfail -s"
Returns 1 if succeeds and 0 if fails.

=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.


=back

=cut

sub unfail_all_disks {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            }
            );


    my $r = make_conn($opts{host},$opts{conntype});
    my $result = 1;

    my $ref = new RaidInfo($r);
    my %disks = $ref->failed_disks();
    my @disk_list = _get_disk_array(\%disks);
    foreach my $disk (@disk_list){
        logcomment("Unfailing disk $disk");
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},cmd=>"disk unfail -s $disk");
        if($output !~ /unfailing disk/){
            $result = 0;
            }
    }
    return $result;
}

sub phy_disable_enable{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            disk_id => { type=>SCALAR, default=>undef, optional=>1 },
            shelf_id => { type=>SCALAR, default=>undef, optional=>1 },
            adapter_id => { type=>SCALAR, default=>undef, optional=>1 },
            phy_id => { type=>SCALAR, default=>undef, optional=>1 },
            module_id => { type=>SCALAR, default=>"both", optional=>1,regex=>qr/^(both|A|B)$/},
            ctrl_value => { type=>SCALAR, default=>"0x02", optional=>1 },
            ems_check => { type=>SCALAR, default=>"1", optional=>1 },
            }
            );
    my $phy_struct;
    my $adapter;
    my $shelf;
    my $phy;

    if(defined($opts{disk_id})){
        if($opts{disk_id} !~ /\w+\.\w+.\w+/){
            logresult("warn"," phy_disable_enable function only works on drives in SAS enclosures");
            return 0;
        }
        $phy_struct = _dev_to_phy_convert($opts{disk_id});
        $adapter = $phy_struct->{$opts{disk_id}}->{Adapter};
        $shelf = $phy_struct->{$opts{disk_id}}->{Shelf};
        $phy = $phy_struct->{$opts{disk_id}}->{Phy};
    }
    else{
        if(defined($opts{shelf_id}) && defined($opts{adapter_id}) && defined($opts{phy_id})){
            $adapter = $opts{adapter_id};
            $shelf = $opts{shelf_id};
            $phy = $opts{phy_id};
        }
        else{
            croak "adapter_id , shelf_id and phy_id needs to be passed as a parameter to phy_disable function when disk_id is not defined";
        }
    }
    if($opts{module_id} eq "both"){
        sasadmin_expander_ctrl(host=>$opts{host},adapter_name=>$adapter ,shelf_id=>$shelf,phy_num=>$phy,ctrl_value=>$opts{ctrl_value},module_id=>"A",filermode=>$opts{filermode},timeout=>$opts{timeout});
        sasadmin_expander_ctrl(host=>$opts{host},adapter_name=>$adapter ,shelf_id=>$shelf,phy_num=>$phy,ctrl_value=>$opts{ctrl_value},module_id=>"B",filermode=>$opts{filermode},timeout=>$opts{timeout});
    }
    else{
        sasadmin_expander_ctrl(host=>$opts{host},adapter_name=>$adapter ,shelf_id=>$shelf,phy_num=>$phy,ctrl_value=>$opts{ctrl_value},module_id=>$opts{module_id},filermode=>$opts{filermode},timeout=>$opts{timeout});
    }
    return 1;
}





=head2 storage_logresult()

    my $ret = storage_logresult();

This routine calls logresult function and fails or fatals based on FATAL_ON_FAILURE parameter

=over

=item fatal_on_failure

(Required) : 1 means 'fatal' and 0 means fail

=item string

(required) : string passed to  logresult

=back

=cut

sub storage_logresult {
    my (%opts) = validate(@_, {
            fatal_on_failure => {type=>SCALAR},
            string => {type=>SCALAR},
            }
            );
    if(($opts{fatal_on_failure} eq "0") || ($opts{fatal_on_failure} eq "no")){
        logresult("fail","$opts{string}");
    }
    else{
        logresult('fatal',"$opts{string}");
    }
}

=head2 log_filer_msg()

    my $ret = log_filer_msg();

This routine writes to the filer logs. Useful when we want to corelate script logs with filer logs.

=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".


=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.

=item string

(Required) : String to send to filer logs

=back

=cut

sub log_filer_msg {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            string => {type=>SCALAR},
            }
            );
        filer_cmd(host=>$opts{host},filermode=>$opts{filermode},cmd=>"log $opts{string} 1");
}
=head2 get_ra_pair()

    my $ret = get_ra_pair();

This routine returns a hash of pair of redundant adapters.

=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".

=item timeout

(optional) : timeout value of say command. Default is 300 secs.

=back

=cut


sub get_ra_pair {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            }
            );
    my @disks;
# Get initial state of disks. I.e., list of adapters, disks, failed disks, etc.
    my $initial_state = get_storage_state(host=>$opts{host},filermode=>$opts{filermode},timeout=>$opts{timeout},skip=>"raid");
    my %ra_pair;
    my @adapters;
# List of adapters from "storage show adapter"
    @adapters = keys %{$initial_state->{StorageShowAdapter}};
# Get a list of disks in the system
    @disks = keys %{$initial_state->{StorageShowDisk}};
    my @disk_split1;
    my $secdisk;
    foreach my $adapt (@adapters){

# Below we want to make sure that the adapter is Enabled and redundndant before offlining it.
        if(($initial_state->{StorageShowAdapter}->{$adapt}->{'State'} =~ /Enabled/) && ($initial_state->{StorageShowAdapter}->{$adapt}->{'Redundant'} =~ /^Yes/)){
            foreach my $disk (@disks){
# Information before first dot in the disk id is adapter id.
                my @disk_split = split(/\./,$disk);
                if($disk_split[0] eq "$adapt"){

                    $secdisk = $initial_state->{StorageShowDisk}->{$disk}->{'SecPath'};
# Information before first dot in the $secdisk is adapter id of the secondary path.
                    @disk_split1 = split(/\./,$secdisk);
                    if(!defined($ra_pair{$disk_split1[0]})){
                        $ra_pair{$adapt} = $disk_split1[0];
                    }
# Now we know the secondary path of $adapt move to next adapter.
                    goto DISK_LABEL;
                }
            }
        }
DISK_LABEL:;
    }

# %ra_pair contains the pair of redundant primary=>secondary adapters.
    my @keys = keys %ra_pair;
    if($#keys < 0){
# Config doesn't have redundant adapters.
        logcomment("No Redundant adapters found");
    }
    my ($k,$v);
    while(($k,$v) = each(%ra_pair)){
        $ra_pair{$v} = $k;
    }
        return \%ra_pair;
}

=head2 disable_dbg_msgs()

    my $ret = disable_dbg_msgs();

This routine disable/enable the dbg messages on the console

=over

=item host

(Required) : Filer connnection to use. Hostrec

=item filermode

(Optional) : The mode of filer: "normal" or "maint" or "ngsh". Default is "normal".

=item timeout

(Optional) : timeout value of say command. Default is 300 secs.

=item module

(Optional) : module which needs to be disabled. Default is "all". If "Storage" it will disable a set of modules related to Storage.

=item enable

(Optional) : Enable dbg messages on console. Default is 0 , i.e, it will disable the messages.

=back

=cut

sub disable_dbg_msgs{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            module => { type=>SCALAR, default=>"all", optional=>1 },
            enable => { type=>SCALAR, default=>0, optional=>1 },
            }
            );
    if($opts{enable}){
            filer_cmd(host=>$opts{host},filermode=>$opts{filermode},cmd=>"dbg to console");
            return 1;
    }
    if($opts{module} eq "all"){
            filer_cmd(host=>$opts{host},filermode=>$opts{filermode},cmd=>"dbg to off");
    }
    elsif($opts{module} eq "Storage"){
        my @modules = qw(disk_io sanown esh fcal sas sasmon ses diskfw environ tape ndr sesfw acp nht ra sdm rlm_takeover rlm_cnsl diskmaint qual_file);
        foreach my $module (@modules){
            filer_cmd(host=>$opts{host},filermode=>$opts{filermode},cmd=>"dbg level $module=-1");
        }
    }
    else{
        my @modules = split(/s+|,/,$opts{module});
        foreach my $module (@modules){
            filer_cmd(host=>$opts{host},filermode=>$opts{filermode},cmd=>"dbg level $module=-1");
        }
    }
}





sub _get_disk_array {
    my $disks = shift;
    my $disk;
    my @disk_list;
    foreach $disk (values %{$disks}){
        push @disk_list, $disk->name();
    }
    return @disk_list;
}

sub _get_common_elements {
    my ($array1,$array2) = @_;
    my @disks = ();
    my $item;
    my %seen = ();
    foreach $item (@$array1) {
        $seen{$item} = 1;
    }
    foreach $item (@$array2) {
        if ($seen{$item}){
            push (@disks, $item) ;
        }
    }
    # print Dumper(@disks);
    return @disks;
}

sub _get_unique_elements {
    my ($array1,$array2) = @_;
    my @disks = ();
    my $item;
    my %seen = ();
    foreach $item (@$array1) {
        $seen{$item} = 1;
    }
    foreach $item (@$array2) {
        unless ($seen{$item}){
            push (@disks, $item) ;
        }
    }
    return @disks;
}

sub _dev_to_phy_convert {
    my ($disk) = @_;
    my %disk_struct = ();
    my @disk_split;
    my ($LocShelf, $LocAdapter, $LocBay);
    @disk_split = split(/\./,$disk);
    if($disk =~ /(\d+\.){2}(\d+)|(.*:\w+)/) {   #if the drive names are in the format 1.1.2, 1.2.3, ..  -OR-
                                                #stqa-6280-Brocade1201:10.09, stqa-6280-Brocade1201:10.12
                                                #then if loop will be entered w.r.t burt #758584 and #771089
        my $filer_ci = NACL::C::Node->find();
        my $root = NACL::CS::StorageDisk->fetch(command_interface => $filer_ci,
                                                 'filter'  => { disk=> $disk, },
                                        'requested_fields' => ['host-adapter','shelf','bay'],
                                                           );
        $disk_struct{$disk}{Bay} = $root->{'bay'};
        $disk_struct{$disk}{Phy} = $root->{'bay'} + 8;
        $disk_struct{$disk}{Adapter} = $root->{'hostadapter'};
        $disk_struct{$disk}{Shelf} = $root->{'shelf'};
        $LocShelf = $root->{'shelf'};
        $LocAdapter = $root->{'hostadapter'};
        $LocBay = $root->{'bay'};
    } else {
        if ($disk =~ /(\w+)\.(\w+)\.(\w+)L\d+/) {   #Support for BOAB shelf : burt #548601
            $disk_struct{$disk}{Bay} = $3;
            $disk_struct{$disk}{Phy} = $3 + 8;
            $LocBay = $3;
        } else {
        # print "\nHELLO\n";
            $disk_struct{$disk}{Bay} = $disk_split[2];
            $disk_struct{$disk}{Phy} = $disk_split[2] + 8;
            $LocBay = $disk_split[2];
        }
        $disk_struct{$disk}{Adapter} = $disk_split[0];
        $disk_struct{$disk}{Shelf} = $disk_split[1];
        $LocShelf = $disk_split[1];
        $LocAdapter = $disk_split[0];
    }
    if(($LocShelf !~ /^0\d/ ) && ($LocShelf < 10)) {
        $LocShelf = "0" . "$LocShelf";
    }
	
	my $sel_disk;
	if($LocBay) {
	    $sel_disk = $LocAdapter . "\." . $LocShelf . "\." . $LocBay;
	} else {
		$sel_disk = $LocAdapter . "\." . $LocShelf . "\.";
	}
	
    
    $disk_struct{$disk}{DiskName} = $sel_disk;

    return (\%disk_struct);
}

#takes an array reference and randomizes content in the array using splice
#the subroutine will check to make sure the input does not reference an empty list
#and returns an error if it does
sub randomize_array{
        my $arr=shift;
    if(@$arr==0){
            die("Array passed in to subroutine is empty");
        }
        foreach(@$arr){
                my $i = pop(@$arr);
        #note you need @$arr+1 in rand since you have popped
                my $nextRand=int(rand(@$arr+1));
                splice @$arr,$nextRand,0,$i;
        }
}


#### Function to get the name of the adapter corresponding to the path and the adapter name provided
## @name ::get_adapter_name
## @summary Returns the name of the corresponding adapter corresponding to the path and  the adapter name provided
## @description Returns the name of the adapter on success
##              Retuns "" on failure
## @synopsis
##        get_adapter_name(host => $host,conntype =>"#rsh",adapter_name =>2a,shelf_path =>"A");
## @arg $host     Required: Hostspec to filer.
## @arg $conntype optional: Type of the connection, default is "#rsh"
## @arg $adapter_name Required: Name of the adapter
## @arg $shelf_path Required: path of the shelf/module name


sub get_adapter_name{
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            adapter_name => { type=>SCALAR },
                        shelf_path    => {type=>SCALAR,default=>"A",optional=>1 }
            }
            );
        my $host       = $opts{host};
        my $conntype   = $opts{conntype};
        my $timeout    = $opts{timeout};
        my $shelf_path = $opts{shelf_path};
        my $fc         = make_conn($host,$conntype );
        my $adapter_name=$opts{adapter_name};
        my $adapter_name_corre_path;

            my($output,@lines,$pr, $port1, $sc, $port2, $shelf, $bay,$pr_adapt, $pr_diskid,$sc_adapt, $sc_diskid);
        # (undef, undef, $output) = say($fc, "storage show disk -p");
        $output = filer_cmd(host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"storage show disk -p");
        (@lines) = split(/\n/, $output);
        for(my $i = 2; $i <= $#lines; $i++)
        {
            ($pr, $port1, $sc, $port2, $shelf, $bay) = split(/\s+/, $lines[$i]);
            ($pr_adapt, $pr_diskid) = split(/\./, $pr);
            ($sc_adapt, $sc_diskid) = split(/\./, $sc);
            if($shelf_path eq $port1) {
                if( ($adapter_name eq $pr_adapt) || ($adapter_name eq $sc_adapt) ){
                    $adapter_name_corre_path=$pr_adapt;
                    last;
                }
            }
            elsif($shelf_path eq $port2) {
                if( ($adapter_name eq $pr_adapt) || ($adapter_name eq $sc_adapt) ){
                    $adapter_name_corre_path=$sc_adapt;
                    last;
                }
            }
        }
       logcomment("adapter_name_corre_path=$adapter_name_corre_path\n");
       return $adapter_name_corre_path;
}

#end of get_adapter_name
=head1 SEE ALSO

    Where additional information can be found.


=head1 BUGS

    Description of bugs or what the library does not do and why

=head1 STATUS

    Shared

=head1 ATTRIBUTES

    ONTAP_COMMANDS = sasadmin

    ONTAP_ZAPI_APIS = none

=head1 AUTHOR

    Anish Vashist (anishv@netapp.com)

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

1;
