# $Id: //depot/prod/test/Rsierranevada/lib/Storage/Sasadmin.pm#5 $

## @summary Sasadmin - library module for the sasadmin command
##
## @status TESTME
##
## @author anishv@netapp.com
##
## @change 10-31-2006 anishv: Modified sasadmin_shelf_short, sasadmin_shelf, sasadmin_expander_phy_state to accept no adapter argument by default.
## @change 10-31-2006 anishv: Removed shelf_id argument from sasadmin_shelf_short and sasadmin_shelf functions.
## @change 10-31-2006 anishv: Added adapter error stats to "sasadmin_adapter_state" function.
## @change 10-31-2006 anishv: Added incon CRC error counts to "sasadmin_expander_phy_state" function.
## @change 10-31-2006 anishv: Added hash information to sasadmin_expander_map function
## @change 12-01-2006 anishv: Added sasadmin_adapter_force_rediscovery function
## @change 12-10-2006 anishv: Added sasadmin_dev_stats and sasadmin_adapter_reservation_state function
## @change 12-13-2006 anishv: Added sasadmin_inject function
## @change 01-11-2007 anishv: Added retry in "sasadmin_expander_ctrl" function to get around burt 226220. Also modified "sasadmin_adapter_phy_disable/enable" to take "-f" option.
## @change 05-07-2007 anishv: Changed priv set level to test for functions: sasadmin_adapter_force_rediscovery, sasadmin_adapter_zombie, sasadmin_adapter_clear_affiliation, sasadmin_expander_phy_power_off, sasadmin_expander_phy_power_on, sasadmin_expander_phy_mux_reset, sasadmin_expander_routes, and sasadmin_expander_phy_dongle_data.
## @change 11-11-2009 anishv: Added support for 10-mode nodescope using filer_cmd
## @change 11-12-2009 anishv: Cleanup functions to accomodate recent changes to sasadmin commands.
## @change 19-08-2011 gurus: Removed -f option in sasadmin_adapter_zombie () function.
## @change 01-11-2012 rajkuma1: Added "hard_reset" parameter to the subroutine sasadmin_adapter_reset
## @change 13-07-2012 gurus: Modified sasadmin expander_phy_state () to croak when ran against filer with missing STATE value
## @change 18-03-2013 nandeesh: Added CMode support to sub sasadmin_expander_cli()
## @change 04-09-2013 nandeesh: Modified sub sasadmin_expander_phy_dongle_data() as per burt #718583
##
##
## @pod here

=head1 NAME

Sasadmin

=head1 SYNOPSIS

    use Sasadmin qw(sasadmin_adapter_state);

    my $host = host($FILER);

    my $hash = sasadmin_adapter_state(host=>$host,...);

=head1 DESCRIPTION
    This is a library module for the sasadmin command

=head1 ROUTINES

=cut

package Storage::Sasadmin;

use strict;
use warnings;
BEGIN {
    use base qw(Exporter);
      our  %EXPORT_TAGS = (
      all => [ qw(
            sasadmin_adapter_state
            sasadmin_adapter_reset
            sasadmin_adapter_force_rediscovery
            sasadmin_adapter_offline
            sasadmin_adapter_online
            sasadmin_adapter_phy_disable
            sasadmin_adapter_phy_enable
            sasadmin_adapter_zombie
            sasadmin_adapter_phy_state
            sasadmin_dev_phy_state
            sasadmin_dev_phy_enable
            sasadmin_dev_phy_disable
            sasadmin_dev_phy_reset
            sasadmin_dev_phy_clear_affiliation
            sasadmin_dev_power_cycle
            sasadmin_dev_identify_device
            sasadmin_expander_phy_disable
            sasadmin_expander_phy_enable
            sasadmin_expander_phy_power_off
            sasadmin_expander_phy_power_on
            sasadmin_expander_phy_reset
            sasadmin_expander_phy_mux_reset
            sasadmin_expander_phy_dongle_data
            sasadmin_expander
            sasadmin_expander_phy_state
            sasadmin_expander_routes
            sasadmin_shelf
            sasadmin_channels
            sasadmin_expander_map
            sasadmin_expander_ctrl
            sasadmin_expander_cli
            sasadmin_dev_stats
            sasadmin_adapter_reservation_state
            sasadmin_inject
            sasadmin_expander_set_shelf_id
            )]
            );
    our  @EXPORT_OK = qw(
            sasadmin_adapter_state
            sasadmin_adapter_reset
            sasadmin_adapter_force_rediscovery
            sasadmin_adapter_offline
            sasadmin_adapter_online
            sasadmin_adapter_phy_disable
            sasadmin_adapter_phy_enable
            sasadmin_adapter_zombie
            sasadmin_adapter_phy_state
            sasadmin_dev_phy_state
            sasadmin_dev_phy_enable
            sasadmin_dev_phy_disable
            sasadmin_dev_phy_reset
            sasadmin_dev_phy_clear_affiliation
            sasadmin_dev_power_cycle
            sasadmin_dev_identify_device
            sasadmin_expander_phy_disable
            sasadmin_expander_phy_enable
            sasadmin_expander_phy_power_off
            sasadmin_expander_phy_power_on
            sasadmin_expander_phy_reset
            sasadmin_expander_phy_mux_reset
            sasadmin_expander_phy_dongle_data
            sasadmin_expander
            sasadmin_expander_phy_state
            sasadmin_expander_routes
            sasadmin_shelf
            sasadmin_channels
            sasadmin_expander_map
            sasadmin_expander_ctrl
            sasadmin_expander_cli
            sasadmin_dev_stats
            sasadmin_adapter_reservation_state
            sasadmin_inject
            sasadmin_expander_set_shelf_id
            );
};

use Storage::Ems qw(ems_log_get_mark ems_log_dump_by_type ems_log_dump_by_id ems_event_status ems_log_check);
use NACL::C::Node;
use NACL::APISet;
use Carp qw(confess croak);
use Params::Validate qw(validate validate_pos SCALAR OBJECT HASHREF);
BEGIN {
Params::Validate::validation_options(
    on_fail => sub {my @msgs = @_; chomp(@msgs);
        Carp::confess(Tharn::script_result(@msgs))}
   );
}
use Tharn qw(say script_result host logcomment logresult param);

use Utils qw(make_conn);
use Data::Dumper;
use Storage::Misc qw(filer_cmd);
use Storage::Fiji qw(set_fiji_filter fiji_clear);

our $MAINT_PROMPT = "\\*\>";

=head2 sasadmin_adapter_state

    my $ret = sasadmin_adapter_state(host => $host, adapter_name => $adapter_name,timeout => $timeout);

This routine returns a hash table of information for each channel in the filer.
It returns 1 on success and croaks on failure.

Example of reference returned by the function:

$VAR1 = {

        },
          '1a' => {
                    '3' => {
                             'DisprtErrorCount' => '0',
                             'LossSyncCount' => '0',
                             'PhyChangeCount' => '16',
                             'InvalidDWordCount' => '0',
                             'STATE' => '3.0',
                             'PhyResetProb' => '0'
                           },
                    'CommandCompletedinError' => '3069',
                    '2' => {
                             'DisprtErrorCount' => '0',
                             'LossSyncCount' => '0',
                             'PhyChangeCount' => '6',
                             'InvalidDWordCount' => '0',
                             'STATE' => '3.0',
                             'PhyResetProb' => '0'
                           },
                    '1' => {
                             'DisprtErrorCount' => '0',
                             'LossSyncCount' => '0',
                             'PhyChangeCount' => '4',
                             'InvalidDWordCount' => '0',
                             'STATE' => '3.0',
                             'PhyResetProb' => '0'
                           },
                    'CommandCompletedSuccessfully' => '125706465',
                    'STATE1' => undef,
                    'FramesOut' => '125739239',
                    '0' => {
                             'DisprtErrorCount' => '0',
                             'LossSyncCount' => '0',
                             'PhyChangeCount' => '8',
                             'InvalidDWordCount' => '0',
                             'STATE' => '3.0',
                             'PhyResetProb' => '0'
                           },
                    'STATE' => 'UP',
                    'FramesIn' => '126541032'
                  }
        };



The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Optional) : Specific channel for which the information is
 required. Default is all

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back

=cut

sub sasadmin_adapter_state($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            adapter_name => {type=>SCALAR, default=>undef, optional=>1},
            timeout => {type=>SCALAR, default=>300,optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            }
            );
    my $adapter = "" ;
    if(defined($opts{adapter_name})){
        $adapter = $opts{adapter_name};
    }

    my %structure =();

    my $cmd = "sasadmin adapter_state $adapter";
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

    if($output !~ /State for adapter channel/) {
        croak $output;
    }

    my @lines = split(/\r*\n/, $output);
    my $hba;
#print " \n\n @lines \n";
    foreach (@lines) {
        if($_ =~ /^State/) {
#print " \n\n @lines \n";
            my @fields = split(/\s+/, $_);
            $hba = $fields[4];
            $hba =~ s/://g;
            if(defined($fields[6])){
                $fields[6] =~ s/\(|\)//g;
            }
            $structure{$hba}{'STATE'} = $fields[5];
            $structure{$hba}{'STATE1'} = $fields[6];
#print "STATE: $fields[5] \n";
#print "STATE1: $fields[6] \n";
        }
        elsif($_ =~ /^\s*\d+/){
            my @fields = split(/\s+/, $_);
            #print "$_ :::: $fields[1]\n";
            if($fields[2] =~ /Rate/){
                $structure{$hba}{$fields[1]}->{'STATE'} = "Rate Unknown";
            }
            else{
                $structure{$hba}{$fields[1]}->{'STATE'} = $fields[2];
            }
            $structure{$hba}{$fields[1]}->{'InvalidDWordCount'} = $fields[4];
            $structure{$hba}{$fields[1]}->{'DisprtErrorCount'} = $fields[5];
            $structure{$hba}{$fields[1]}->{'LossSyncCount'} = $fields[6];
            $structure{$hba}{$fields[1]}->{'PhyResetProb'} = $fields[7];
            $structure{$hba}{$fields[1]}->{'PhyChangeCount'} = $fields[8];
        }
        elsif($_ =~ /^\s*Frames\s+In/){
            my @fields = split(/\s+/, $_);
            $structure{$hba}->{'FramesIn'} = $fields[4];
        }
        elsif($_ =~ /^\s*Frames\s+Out/){
            my @fields = split(/\s+/, $_);
            $structure{$hba}->{'FramesOut'} = $fields[4];
        }
        elsif($_ =~ /^\s*Command\s+Completed\s+sucessfully/){
            my @fields = split(/\s+/, $_);
            $structure{$hba}->{'CommandCompletedSuccessfully'} = $fields[5];
        }
        elsif($_ =~ /^\s*Command\s+Completed\s+in\s+error/){
            my @fields = split(/\s+/, $_);
            $structure{$hba}->{'CommandCompletedinError'} = $fields[6];
        }

    }
#        print  Dumper(\%structure);
    return \%structure;
}


=cut



=head2 sasadmin_adapter_reset

    my $ret = sasadmin_adapter_reset(host => $host, adapter_name => $adapter_name,timeout => $timeout);

This routine resets a SAS HBA.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Optional) : Specific channel for which the information is
 required. Default is all.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_adapter_reset($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            adapter_name => { type=>SCALAR },
            timeout => {type=>SCALAR,default=>300,optional=>1},
        hard_reset => {type=>SCALAR,default=>"no",optional=>1 },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            ems_verify => {type=>SCALAR,default=>"no",optional=>1},
            }
            );
    my $adapter_name;

    my $a = "";
    my $b = "";
    my $mark;
    my ($out,$out1);

    if ($opts{hard_reset} eq "yes"){
        my $fiji_point = "(pmcsas.force.hard.reset true (return 1))";
        fiji_clear(host=>$opts{host},fiji_point=>$fiji_point); ##clearing the previous fiji point
            set_fiji_filter(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},fiji_point=>$fiji_point,timeout=>$opts{timeout});
       }

    if($opts{filermode} eq "maint"){
        my $cmd = "sasadmin adapter_reset $opts{adapter_name}";
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        my $slot = $opts{adapter_name};
        chop($slot);
        if($output !~ /Resetting SAS adapter $slot/) {
            croak $output ;
        }
        return 1;
    }
    else{
        if ($opts{ems_verify} eq "yes"){
            $mark = ems_log_get_mark(host=>$opts{host});
        }
        my $slot = $opts{adapter_name};
        chop($slot);
        my $cmd = "sasadmin adapter_reset $opts{adapter_name}";
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if($output =~ /No such adapter/) {
            croak $output ;
        }
        if ($opts{ems_verify} eq "yes"){
            sleep 30;
            my $mesg = ems_log_check(host=>$opts{host},eventtype=>"sas.adapter.reset",key=>"adapterName", value=>"$slot", timestamp=>$mark);

            if ($mesg == 0) {
                logcomment(" Adapter reset event was not found in the ems log"); #To be croak once ems is working
            }
            else {
                logcomment("Found the specified ems event $mesg times in the ems log");
                return 1;
            }
        }


    }
}

=head2 sasadmin_adapter_force_rediscovery

    my $ret = sasadmin_adapter_force_rediscovery(host => $host, adapter_name => $adapter_name,timeout => $timeout);

This routine forces rediscovery of a SAS HBA.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Optional) : Specific channel for which the information is
 required. Default is all.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_adapter_force_rediscovery($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            adapter_name => { type=>SCALAR },
            timeout => {type=>SCALAR,default=>300,optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            ems_verify => {type=>SCALAR,default=>"no",optional=>1},
            }
            );
    my $adapter_name;
    my $a = "";
    my $b = "";
    my $mark;

    if ($opts{filermode} ne "normal") {
        my $cmd = "sasadmin adapter_force_rediscovery $opts{adapter_name}";
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        my $slot = $opts{adapter_name};
        if($output =~ /Not a SAS adapter|failed|No such adapter|denied/){
            croak $output ;
        }
        return 1;
    }
    else{
        if ($opts{ems_verify} eq "yes"){
            $mark = ems_log_get_mark(host=>$opts{host});
        }
        my $slot = $opts{adapter_name};
        chop($slot);
        my $cmd = "sasadmin adapter_force_rediscovery $opts{adapter_name}";
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
        if($output =~ /No such adapter|denied/) {
            croak $output ;
        }
        if ($opts{ems_verify} eq "yes"){
            sleep 30;
            my $mesg = ems_log_check(host=>$opts{host},eventtype=>"sas.adapter.debug",key=>"adapterName", value=>"$slot", timestamp=>$mark);

            if ($mesg == 0) {
                logcomment(" Adapter debug message was not found in the ems log"); #To be croak once ems is working
            }
            else {
                logcomment("Found the specified ems event $mesg times in the ems log");
                return 1;
            }
        }
        else{
            return 1;
        }


    }

}





=head2 sasadmin_adapter_offline

    my $ret = sasadmin_adapter_offline(host => $host, adapter_name => $adapter_name,timeout => $timeout);

This routine offlines a SAS HBA.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required. Default is all.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back


=cut

sub sasadmin_adapter_offline($@){
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            adapter_name => { type=>SCALAR },
            timeout => {type=>SCALAR,default=>300,optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            ems_verify => {type=>SCALAR,default=>"yes",optional=>1},
            }
            );
    my $adapter_name;
    my $a = "";
    my $b = "";
    if ($opts{filermode} ne "normal") {
        my $cmd = "sasadmin adapter_offline $opts{adapter_name}";
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        my $slot = $opts{adapter_name};
        chop($slot);
#            print "\n O $output slot is $slot \n";
        if($output !~ /Taking adapter $opts{adapter_name} offline/) {
#            print "\n O $output slot is $slot \n";
#            sleep 1000;
            croak $output ;
        }
        return 1;
    }
    else{
        my $mark ;
        if($opts{ems_verify} eq "yes"){
            $mark  = ems_log_get_mark(host=>$opts{host});
        }
        my $cmd = "sasadmin adapter_offline $opts{adapter_name}";
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if($output =~ /No such adapter|adapter_offline: failed/) {
            croak $output ;
        }
        if($opts{ems_verify} eq "yes"){
            sleep 30;
            my $mesg = ems_log_check(host=>$opts{host},eventtype=>"sas.adapter.offline",key=>"adapterName", value=>"$opts{adapter_name}", timestamp=>$mark);
            if ($mesg == 0) {
                logcomment(" Adapter reset event was not found in the ems log"); #To be croak once ems is working
            } else {
                logcomment("Found the specified ems event $mesg times in the ems log");
                return 1;
            }
        }
        else{
            return 1;
        }

    }
}





#=========

=head2 sasadmin_adapter_online

    my $ret = sasadmin_adapter_online(host => $host, adapter_name => $adapter_name,timeout => $timeout);

This routine onlines a SAS HBA port.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required. Default is all.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_adapter_online($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            adapter_name => { type=>SCALAR },
            timeout => {type=>SCALAR,default=>300,optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            ems_verify => {type=>SCALAR,default=>"yes",optional=>1},
            }
            );
    my $adapter_name;
    my $a = "";
    my $b = "";

    my $conn = make_conn($opts{host},$opts{conntype});
    if ($opts{filermode} ne "normal") {
        my $cmd = "sasadmin adapter_online $opts{adapter_name}";
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        (undef, undef, $output) = say($conn, "$cmd", -prompt, "$MAINT_PROMPT", "-timeout", $opts{timeout});
#            print "\n O $output slot is $slot \n";
        if($output !~ /Bringing adapter $opts{adapter_name} online/) {
#            print "\n O $output slot is $slot \n";
#            sleep 1000;
            croak $output ;
        }
        return 1;
    }
    else{
        my $mark;
        if($opts{ems_verify} eq "yes"){
            $mark = ems_log_get_mark(host=>$opts{host});
        }
        my $cmd = "sasadmin adapter_online $opts{adapter_name}";
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if($output =~ /No such adapter|adapter_online: failed/) {
            croak $output ;
        }
        if($opts{ems_verify} eq "yes"){
            sleep 5;
            my $mesg = ems_log_check(host=>$opts{host},eventtype=>"sas.adapter.onlining",key=>"adapterName", value=>"$opts{adapter_name}", timestamp=>$mark);
            if ($mesg == 0) {
                logcomment(" Adapter reset event was not found in the ems log"); #To be croak once ems is working
            } else {
                logcomment("Found the specified ems event $mesg times in the ems log");
                return 1;
            }
        }
        else{
            return 1;
        }

    }

}



#======

=head2 sasadmin_adapter_zombie

    my $ret = sasadmin_adapter_zombie(host => $host, adapter_name => $adapter_name,timeout => $timeout);

This routine zombifies a SAS HBA port.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required. Default is all.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back




=cut


sub sasadmin_adapter_zombie($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            adapter_name => { type=>SCALAR },
            timeout => {type=>SCALAR,default=>300,optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            ems_verify => {type=>SCALAR,default=>"yes",optional=>1},
            }
            );
    my $adapter_name;
    my $a = "";
    my $b = "";
    my $mark;

    if ($opts{filermode} ne "normal") {
        my $cmd="sasadmin adapter_zombie $opts{adapter_name} -f";
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        my $slot = $opts{adapter_name};
        chop($slot);
#            print "\n O $output slot is $slot \n";
        if($output =~ /Cannot complete operation on channel $opts{adapter_name}|No such adapter|adapter_offline: failed/) {
#            print "\n O $output slot is $slot \n";
#            sleep 1000;
            croak $output ;
        }
        return 1;
    }
    else{
        if($opts{ems_verify} eq "yes"){
            $mark = ems_log_get_mark(host=>$opts{host});
        }
        my $cmd="priv set test;sasadmin adapter_zombie $opts{adapter_name} -f";
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if($output =~ /Cannot complete operation on channel $opts{adapter_name}|No such adapter|adapter_offline: failed/) {
            croak $output ;
        }
        if($opts{ems_verify} eq "yes"){
            sleep 5;
            my $mesg = ems_log_check(host=>$opts{host},eventtype=>"sas.adapter.offline",key=>"adapterName", value=>"$opts{adapter_name}", timestamp=>$mark);

            if ($mesg == 0) {
                logcomment(" Adapter zombie event was not found in the ems log"); #To be croak once ems is working
            } else {
                logcomment("Found the specified ems event $mesg times in the ems log");
                return 1;
            }
        }
        else{
            return 1;
        }

    }

}




=head2 sasadmin_adapter_phy_disable

    my $ret = sasadmin_adapter_disable(host => $host, adapter_name => $adapter_name,phy_num => $phy_num,timeout => $timeout);

This routine disables a SAS HBA phy.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.

=item phy_num

(Required) : Specific phy no. of HBA to be disabled.

=item flag

(Optional) : Specific flag option to the command. Valid values are "-f" and "". By default it using "-f" option


=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_adapter_phy_disable($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            adapter_name => { type=>SCALAR },
            timeout => {type=>SCALAR,default=>300,optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            flag => {type=>SCALAR,default=>"-f",optional=>1},
            phy_num => {type=>SCALAR},

            }
            );
    my $output;
    my $a;
    my $b;

    if($opts{flag} ne "-f"){
        if ($opts{filermode} ne "normal") {
            $opts{conntype} = "#console";
        }

        my $conn = make_conn($opts{host},$opts{conntype});
        if ($opts{filermode} ne "normal") {
            my $slot = $opts{adapter_name};
            chop($slot);
            my $cmd = "sasadmin adapter_phy_disable $opts{adapter_name} $opts{phy_num}";
            ($a, $b, $output) = say($conn, "$cmd",-swallow,'-noprompt','-wait','Do you wish to continue|No such adapter', "-timeout", $opts{timeout},-ok);
            if($a !~ /Do you wish to continue/) {
                print "A: $a \n B: $b \n O $output \n";
                croak ("$a" . "$output") ;
            }
            ($a, $b, $output) = say($conn, "y",-swallow,'-noprompt','-wait',"Resetting SAS adapter $slot|sasadmin adapter_phy_disable: failed", "-timeout", $opts{timeout},-ok);
            if($a !~ /Resetting SAS adapter $slot/) {
                logresult('fail',"Did not see the \"Resetting SAS adapter\" in the output");  ;
            }
            return 1;
        }
        else{
            $opts{conntype} = "#telnet";
            $conn = make_conn($opts{host},$opts{conntype});
            my $slot = $opts{adapter_name};
            chop($slot);

            my $cmd = "priv set -q diag;sasadmin adapter_phy_disable $opts{adapter_name} $opts{phy_num}";
            ($a, $b, $output) = say($conn, "$cmd",-swallow,'-noprompt','-wait',"Do you wish to continue|No such adapter", "-timeout", $opts{timeout},-ok);
            if($a !~ /Do you wish to continue/) {
                print "A: $a \n B: $b \n O $output \n";
                croak ("$a" . "$output") ;
            }
            ($a, $b, $output) = say($conn, "y",-swallow,'-noprompt','-wait',"Resetting SAS adapter $slot|sasadmin adapter_phy_disable: failed", "-timeout", $opts{timeout},-ok);
            if($a !~ /Resetting SAS adapter $slot/) {
                logresult('fail',"Did not see the \"Resetting SAS adapter\" in the output");  ;
            }
            return 1;

        }
    }
    else{
        if ($opts{filermode} ne "normal") {
            my $slot = $opts{adapter_name};
            chop($slot);
            my $cmd = "sasadmin adapter_phy_disable $opts{adapter_name} $opts{phy_num} -f";
            $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
            if($output =~ /failed|Unrecognized|No such adapter/) {
                croak ($output) ;
            }
            return 1;
        }
        else{
            my $slot = $opts{adapter_name};
            chop($slot);
            my $cmd = "sasadmin adapter_phy_disable $opts{adapter_name} $opts{phy_num} -f";
            $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
            if($output =~ /failed|Unrecognized|No such adapter/) {
                croak ($output) ;
            }
            return 1;

        }
    }
}

=head2 sasadmin_adapter_phy_enable

    my $ret = sasadmin_adapter_enable(host => $host, adapter_name => $adapter_name,phy_num => $phy_num,timeout => $timeout);

This routine enables a SAS HBA phy.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.

=item phy_num

(Required) : Specific phy no. of HBA to be enabled.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item flag

(Optional) : Specific flag option to the command. Valid values are "-f" and "". By default it using "-f" option


=back



=cut


sub sasadmin_adapter_phy_enable($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            adapter_name => { type=>SCALAR },
            timeout => {type=>SCALAR,default=>300,optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            flag => {type=>SCALAR,default=>"-f",optional=>1},
            phy_num => {type=>SCALAR},

            }
            );
    my $output;
    my $a;
    my $b;


    if($opts{flag} ne "-f"){
        if ($opts{filermode} ne "normal") {
            $opts{conntype} = "#console";
        }

        my $conn = make_conn($opts{host},$opts{conntype});
        if ($opts{filermode} ne "normal") {
            my $slot = $opts{adapter_name};
            chop($slot);
            my $cmd = "sasadmin adapter_phy_enable $opts{adapter_name} $opts{phy_num}";
            ($a, $b, $output) = say($conn, "$cmd",-swallow,'-noprompt','-wait','Do you wish to continue|No such adapter', "-timeout", $opts{timeout},-ok);
            if($a !~ /Do you wish to continue/) {
                print "A: $a \n B: $b \n O $output \n";
                croak ("$a" . "$output") ;
            }
#    ($a, $b, $output) = say($conn, "y",-swallow, "-timeout", $opts{timeout});
            ($a, $b, $output) = say($conn, "y",-swallow,'-noprompt','-wait',"Resetting SAS adapter $slot|sasadmin adapter_phy_enable: failed", "-timeout", $opts{timeout},-ok);
            if($a !~ /Resetting SAS adapter $slot/) {
                print "A: $a \n B: $b \n O $output \n";
#print "OUT: $output \n";
#croak  $output;
                logresult('fail',"Did not see the \"Resetting SAS adapter\" in the output");  ;
            }
            return 1;
        }
        else{
            $opts{conntype} = "#telnet";
            $conn = make_conn($opts{host},$opts{conntype});
#        my $mark = ems_log_get_mark(host=>$opts{host});
#        sleep 5;
#        my $mesg = ems_log_check(host=>$opts{host},eventtype=>"sas.adapter.offline",key=>"adapterName", value=>"$opts{adapter_name}", timestamp=>$mark);
            my $slot = $opts{adapter_name};
            chop($slot);

            my $cmd = "priv set -q diag;sasadmin adapter_phy_enable $opts{adapter_name} $opts{phy_num}";
            ($a, $b, $output) = say($conn, "$cmd",-swallow,'-noprompt','-wait','Do you wish to continue|No such adapter', "-timeout", $opts{timeout},-ok);
            if($a !~ /Do you wish to continue/) {
                print "A: $a \n B: $b \n O $output \n";
                croak ("$a" . "$output") ;
            }
#        ($a, $b, $output) = say($conn, "y",-swallow, "-timeout", $opts{timeout},-ok);
            ($a, $b, $output) = say($conn, "y",-swallow,'-noprompt','-wait',"Resetting SAS adapter $slot|sasadmin adapter_phy_enable: failed", "-timeout", $opts{timeout},-ok);
            if($a !~ /Resetting SAS adapter $slot/) {
                print "A: $a \n B: $b \n O $output \n";
#croak $output ;
                logresult('fail',"Did not see the \"Resetting SAS adapter\" in the output");  ;
            }
#        if ($mesg == 0) {
#                logcomment(" Adapter phy disable event was not found in the ems log"); #To be croak once ems is working
#        } else {
#               logcomment("Found the specified ems event $mesg times in the ems log");
    return 1;
#    }

        }
    }
    else{
        if ($opts{filermode} ne "normal") {
            my $slot = $opts{adapter_name};
            chop($slot);
            my $cmd = "sasadmin adapter_phy_enable $opts{adapter_name} $opts{phy_num} -f";
            $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
            if($output =~ /failed|Unrecognized|No such adapter/) {
                croak ($output) ;
            }
            return 1;
        }
        else{
#        my $mark = ems_log_get_mark(host=>$opts{host});
#        sleep 5;
#        my $mesg = ems_log_check(host=>$opts{host},eventtype=>"sas.adapter.offline",key=>"adapterName", value=>"$opts{adapter_name}", timestamp=>$mark);
            my $slot = $opts{adapter_name};
            chop($slot);

            my $cmd = "priv set -q diag;sasadmin adapter_phy_enable $opts{adapter_name} $opts{phy_num} -f";
            $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
            if($output =~ /failed|Unrecognized|No such adapter/) {
                croak ($output) ;
            }
#        if ($mesg == 0) {
#                logcomment(" Adapter phy disable event was not found in the ems log"); #To be croak once ems is working
#        } else {
#               logcomment("Found the specified ems event $mesg times in the ems log");
    return 1;
#    }

        }
    }
}



#===============


=head2 sasadmin_adapter_phy_state

    my $ret = sasadmin_adapter_phy_state(host => $host, adapter_name => $adapter_name,timeout => $timeout);

This routine returns a hash table of information for each channel's phy state in the filer.
It returns 1 on success and croaks on failure.

Example of hash returned by the function:
$VAR1 = {

          '0' => {

                   'InvalidDWordCount' => '0',
                   'PhyResetProblemCount' => '0',
                   'PhysicalPhyNumber' => 4,
                   'DWordSynchLossCount' => '0',
                   'PhyState' => '3.0 Gb/s',
                   'RunningDisparityErrorCount' => '0'
                 },

          1 => {

                 'InvalidDWordCount' => '0',
                 'PhyResetProblemCount' => 22,
                 'PhysicalPhyNumber' => 5,
                 'DWordSynchLossCount' => '0',
                 'PhyState' => 'Rate unknown',
                 'RunningDisparityErrorCount' => '0'

               },

          '6b' => {
                    'PhyCount' => 4
                  },

          2 => {
                 'InvalidDWordCount' => '0',
                 'PhyResetProblemCount' => 12,
                 'PhysicalPhyNumber' => 6,
                 'DWordSynchLossCount' => '0',
                 'PhyState' => 'Rate unknown',
                 'RunningDisparityErrorCount' => '0'
               },

          3 => {

                 'InvalidDWordCount' => '0',
                 'PhyResetProblemCount' => '0',
                 'PhysicalPhyNumber' => 7,
                 'DWordSynchLossCount' => '0',
                 'PhyState' => '3.0 Gb/s',
                 'RunningDisparityErrorCount' => '0'

               }

        };



The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_adapter_phy_state($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            adapter_name => { type=>SCALAR },
            timeout => {type=>SCALAR, default=>"300",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},

            }
            );
    my $output;
    my $adapter_name = $opts{adapter_name};
    my %structure =();
    my $phyno;
    my $rate;

    my $cmd = "sasadmin adapter_phy_state $opts{adapter_name} ";
    $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

    if ($output !~ /Phy Count/){
        croak $output;
    }
    my @phylist = split(/\r*\n/,$output);
    foreach (@phylist){
        my @abc = split(/:\s+|\r*\n|\]\s+/,$_);
#sleep 5;

        if ($_ =~ /Phy Count/){
            $structure{$adapter_name}{PhyCount} = $abc[1];
        }
        elsif ($_ =~ /Physical Phy Number/){

            $abc[0] =~ s/\[//g;
            $phyno = $abc[0];
            $structure{$phyno}{PhysicalPhyNumber} = $abc[2];
        }
        elsif($_ =~ /Invalid DWord/){
            $structure{$phyno}{InvalidDWordCount} = $abc[1];
        }
        elsif($_ =~ /Phy state/){
            $structure{$phyno}{PhyState} = $abc[1];
        }
        elsif($_ =~ /Running Disparity Error/){
            $structure{$phyno}{RunningDisparityErrorCount} = $abc[1];
        }
        elsif($_ =~ /Loss of DWord Synch Count/){
            $structure{$phyno}{DWordSynchLossCount} = $abc[1];
        }
        elsif($_ =~ /Phy Reset Problem Count/){
            $structure{$phyno}{PhyResetProblemCount} = $abc[1];
        }
    }
#print  Dumper(\%structure);
    return \%structure ;
}


=head2 sasadmin_dev_phy_state

    my $ret = sasadmin_dev_phy_state(host => $host, dev_id => $disk_id,timeout => $timeout);

This routine returns a hash table of information for each channel's phy state in the filer.
It returns 1 on success and croaks on failure.

Example of hash returned by the function:

$VAR1 = {

          '6b.129.23' => {

                           'InvalidDWordCount' => '0',
                           'PhyResetProblemCount' => '0',
                           'PhysicalPhyNumber' => 29,
                           'DWordSynchLossCount' => '0',
                           'PhyState' => '3.0 Gb/s',
                           'RunningDisparityErrorCount' => '0'

                         }

        };


The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.


=item dev_id

(Required) : Specific disk for which the information is
 required.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut


sub sasadmin_dev_phy_state($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                dev_id => {type=>SCALAR},

        }
    );
    my $output;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno;
    my $rate;
    my $devid = $opts{dev_id};

        my $cmd = "sasadmin dev_phy_state $devid";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

        if ($output !~ /Physical Phy Number/){
            croak $output;
        }
            my @phylist = split(/\r*\n/,$output);
            foreach (@phylist){
                my @abc = split(/:\s+|\r*\n/,$_);
                $abc[0] =~ s/://g;
                #sleep 5;
                if($abc[0] =~ /Physical Phy Number/){
                    $structure{$devid}{PhysicalPhyNumber} = $abc[1];
                }
                elsif($abc[0] =~ /Phy state/){
                    $structure{$devid}{PhyState} = $abc[1];
                }
                elsif($abc[0] =~ /Invalid DWord Count/){
                    $structure{$devid}{InvalidDWordCount} = $abc[1];
                }
                elsif($abc[0] =~ /Running Disparity Error Count/){
                    $structure{$devid}{RunningDisparityErrorCount} = $abc[1];
                }
                elsif($abc[0] =~ /Loss of DWord/){
                    $structure{$devid}{DWordSynchLossCount} = $abc[1];
                }
                elsif($abc[0] =~ /Phy Reset Problem Count/){
                    $structure{$devid}{PhyResetProblemCount} = $abc[1];
                }
            }
            #print  Dumper(\%structure);
            return \%structure ;
}

=head2 sasadmin_dev_phy_enable

    my $ret = sasadmin_dev_phy_enable(host => $host,dev_id => $disk_id,timeout => $timeout);

This subroutine enables a device phy.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item dev_id

(Required) : Disk Id

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_dev_phy_enable($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                dev_id => {type=>SCALAR},

        }
    );
    my $output;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno;
    my $rate;
    my $devid = $opts{dev_id};

        my $cmd = "sasadmin dev_phy_enable $devid";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if($output =~ /dev_phy_enable: failed|Usage/){
            croak $output ;
        }
        return 1;

}
=head2 sasadmin_dev_phy_disable

    my $ret = sasadmin_dev_phy_disable(host => $host,dev_id => $disk_id,timeout => $timeout);

This subroutine disables a device.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item dev_id

(Required) : Disk Id

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut




sub sasadmin_dev_phy_disable($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                dev_id => {type=>SCALAR},

        }
    );
    my $output;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno;
    my $rate;
    my $devid = $opts{dev_id};
            my $cmd = "sasadmin dev_phy_disable $devid";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        #print "OUTPUT: $output \n\n";
# EMS checks to be added once available
        if($output =~ /dev_phy_disable: failed|Usage/){
            croak $output ;
        }
        return 1;

}

=head2 sasadmin_dev_phy_reset

    my $ret = sasadmin_dev_phy_reset(host => $host,dev_id => $disk_id,timeout => $timeout);

This subroutine resets a device.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item dev_id

(Required) : Disk Id
Optional if adapter_id,shelf_id and bay_id is provided.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_dev_phy_reset($@){
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                dev_id => {type=>SCALAR},

        }
    );
    my $output;
    my $timeout = $opts{timeout};
    my %structure =();
    my $rate;
    my $devid = $opts{dev_id};

        my $cmd = "sasadmin dev_phy_reset $devid";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        #print "OUTPUT: $output \n\n";
# Check for EMS to be added once available
        if($output =~ /dev_phy_reset: failed|Usage/){
            croak $output ;
        }
        return 1;
}

=head2 sasadmin_dev_phy_clear_affiliation

    my $ret = sasadmin_dev_phy_clear_affilation(host => $host,dev_id => $disk_id,timeout => $timeout);

This subroutine clears affiliations of a device.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item dev_id

(Required) : Disk Id

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_dev_phy_clear_affiliation($@){
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            timeout => {type=>SCALAR, default=>"300",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            dev_id => {type=>SCALAR},

            }
            );
    my $output;
    my $timeout = $opts{timeout};
    my %structure =();
    my $rate;
    my $devid = $opts{dev_id};

    my $cmd = "sasadmin dev_phy_clear_affiliation $devid";
    $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
#print "OUTPUT: $output \n\n";
# Check for EMS to be added once available
    if($output =~ /dev_phy_clear_affiliation: failed|Usage/){
        croak $output ;
    }
    return 1;
}


=head2 sasadmin_dev_power_cycle

    my $ret = sasadmin_dev_power_cycle(host => $host, dev_id => $disk_id,timeout => $timeout);

This subroutine power_cycles a device.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item dev_id

(Required) : Specific disk for which the information is
 required.Optional if adapter_id,shelf_id and bay_id is provided.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back


=cut

sub sasadmin_dev_power_cycle($@){
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            timeout => {type=>SCALAR, default=>"300",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            dev_id => {type=>SCALAR},
            ems_verify => {type=>SCALAR,default=>"no",optional=>1},
            }
            );
    my $output;
    my $timeout = $opts{timeout};
    my %structure =();
    my $rate;
    my $devid = $opts{dev_id};
    my $mark;
    my $ems_check = 1;
    if(($opts{ems_verify} ne "yes") || ($opts{filermode} ne "normal")){
        $ems_check = 0;
    }
    my $cmd = "sasadmin dev_power_cycle $devid";
    $mark = ems_log_get_mark(host=>$opts{host}) if($ems_check);
    $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
    if($output =~ /dev_power_cycle: failed|Usage/){
        croak $output ;
    }
    if($ems_check){
        sleep 20;
        my $mesg = 0;
        $mesg = ems_log_check(host=>$opts{host},eventtype=>"sas.adapter.debug",key=>"debug_string", value=>"Starting powercycle",timestamp=>$mark,patternmatch=>1);
        if($mesg == 0){
            logresult("warn","Did not see drive power cycle message in the ems logs after power cycling drive $devid");
        }
        else{
            logcomment("Expected message in the logs");
            return 1;
        }
    }
    else{
        return 1;
    }
}


=head2 sasadmin_dev_identify_device

    my $ret = sasadmin_dev_identify_device(host => $host,dev_id => $disk_id,timeout => $timeout);

This subroutine returns identification information of a device.
It returns 1 on success and croaks on failure.

Function returns the following STRING:

$VAR1 = ' sasadmin dev_identify_device 6b.129.23^M^M
Selected path 6b.129.23 (0x0201811d) is the primary path -- no need to redirect.^M^M
Actual Data Len 512, Status 0x0, Error 0x0, Status Control 0x200^M^M
^M^M
0000: 045a 3fff c837 0010 0000 0000 003f 0000  --- .Z?..7.......?..^M^M
0008: 0000 0000 5a41 4732 3150 4444 2020 2020  --- ....ZAG21PDD    ^M^M
0016: 2020 2020 2020 2020 0003 79ef 0034 4b32  ---         ..y..4K2^M^M
0024: 414f 4132 3241 4849 5441 4348 4920 4844  --- AOA22AHITACHI HD^M^M
0032: 5337 3235 3035 304b 4c41 3336 3020 2020  --- S725050KLA360   ^M^M
0040: 2020 2020 2020 2020 2020 2020 2020 8010  ---               ..^M^M
0048: 0000 2f00 4000 0200 0200 0007 3fff 0010  --- ../.@.......?...^M^M
0056: 003f fc10 00fb 0100 ffff 0fff 0000 0007  --- .?..............^M^M
0064: 0003 0078 0078 00f0 0078 0000 0000 0000  --- ...x.x...x......^M^M
0072: 0000 0000 0000 000f 0306 0000 005e 0040  --- .............^.@^M^M
0080: 00fc 001a 346b 7fe9 4163 3449 3c01 4163  --- ....4k..Ac4I.Ac^M^M
0088: 407f 0078 0000 0000 fffe 0000 80fe 0000  --- @..x............^M^M
0096: 0000 0000 0000 0000 6030 3a38 0000 0000  --- ........`0:8....^M^M
0104: 0000 0000 0000 5a87 5000 cca2 0dc0 ef16  --- ......Z.P.......^M^M
0112: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0300: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0128: 0001 000a 0000 0000 2a82 0cb1 fe20 0001  --- ........*.... ..^M^M
0136: 4000 0000 0000 0000 0000 50b5 45c5 342a  --- @.........P.E.4*^M^M
0144: 0400 0280 3f7f 00c0 0040 4800 8000 0000  --- ....?....@H.....^M^M
0152: 4141 4332 0000 000a 0000 0000 0000 0000  --- AAC2............^M^M
0160: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0168: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0176: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0184: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0192: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0200: 0000 0000 0000 0000 0000 0000 003f 0000  --- .............?..^M^M
0208: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0216: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0224: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0232: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0240: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................^M^M
0248: 0000 0000 0000 0000 0000 0000 0000 f2a5  --- ................^M^M
^M^M

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item dev_id

(Required) : Specific disk for which the information is
 required.Optional if adapter_id,shelf_id and bay_id is provided.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut


sub sasadmin_dev_identify_device($@){
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                dev_id => {type=>SCALAR},

        }
    );
    my $output;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno;
    my $rate;
    my $bayid;
    my $shelfid;
    my $devid = $opts{dev_id};

    if ($opts{filermode} ne "normal") {
        $opts{conntype} = "#console";
    }
    my $conn = make_conn($opts{host},$opts{conntype});
        if ($opts{filermode} ne "normal") {
            my $cmd = "sasadmin dev_identify_device $devid";
            (undef, undef, $output) = say($conn, "$cmd",-swallow,'-prompt',"$MAINT_PROMPT","-timeout", $timeout);
        }
        else{

            my $cmd = "priv set -q diag;sasadmin dev_identify_device $devid";
            (undef, undef, $output) = say($conn, "$cmd",-swallow,"-timeout", $timeout);
        }
        #print "OUTPUT: $output \n\n";
        #EMS check to be added once available
        if($output =~ /dev_identify_device:adt_pathname_to_phys_dev failed|Usage/){
            croak $output ;
        }
        return $output;

}

=head2 sasadmin_expander_phy_power_off

    my $ret = sasadmin_expander_phy_power_off(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,phy_num => $phy_num,timeout => $timeout);

This subroutine power offs a phy on the expander.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.

=item shelf_id

(Required) : Specific shelf for which the information is
 required.

=item phy_num

(Required) : Phy no to be disabled.
 required.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_expander_phy_power_off($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            timeout => {type=>SCALAR, default=>"300",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            phy_num => {type=>SCALAR},
            shelf_id => {type=>SCALAR},
            adapter_name => {type=>SCALAR},

            }
            );
    my $output;
    my $adapter_name = $opts{adapter_name};;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno = $opts{phy_num};;
    my $rate;
    my $shelfid = $opts{shelf_id};
    my $cmd = "sasadmin expander_phy_power_off $adapter_name\.$shelfid $phyno ";
    $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
#EMS check to be added once available
    if($output =~ /expander_phy_power_off: failed|Usage|No such adapter/){
        croak $output ;
    }
    return 1;

}

=head2 sasadmin_expander_phy_power_on

    my $ret = sasadmin_expander_phy_power_on(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,phy_num => $phy_num,timeout => $timeout);

This subroutine power on a phy on the expander.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Adapter name to which shelf is connected

=item shelf_id

(Required) : Shelf id of the phy

=item phy_num

(Required) : Phy no to be enabled.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_expander_phy_power_on($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                phy_num => {type=>SCALAR},
                shelf_id => {type=>SCALAR},
                adapter_name => {type=>SCALAR},

        }
    );
    my $output;
    my $adapter_name = $opts{adapter_name};;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno = $opts{phy_num};;
    my $rate;
    my $shelfid = $opts{shelf_id};
        my $cmd = "sasadmin expander_phy_power_on $adapter_name\.$shelfid $phyno ";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");

        #EMS check to be added once available
        if($output =~ /expander_phy_power_on: failed|Usage|No such adapter/){
            croak $output ;
        }
        return 1;
}



=head2 sasadmin_expander_phy_disable

    my $ret = sasadmin_expander_phy_disable(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,phy_num => $phy_num,timeout => $timeout);

This subroutine disables a phy on the expander.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.

=item shelf_id

(Required) : Specific shelf for which the information is
 required.

=item phy_num

(Required) : Phy no to be disabled.
 required.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_expander_phy_disable($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            timeout => {type=>SCALAR, default=>"300",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            phy_num => {type=>SCALAR},
            shelf_id => {type=>SCALAR},
            adapter_name => {type=>SCALAR},

            }
            );
    my $output;
    my $adapter_name = $opts{adapter_name};;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno = $opts{phy_num};;
    my $rate;
    my $shelfid = $opts{shelf_id};

    my $cmd = "sasadmin expander_phy_disable $adapter_name\.$shelfid $phyno ";
    $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
#print "OUTPUT: $output \n\n";
#EMS check to be added once available
    if($output =~ /expander_phy_disable: failed|Usage|No such adapter/){
        croak $output ;
    }
    return 1;
}


=head2 sasadmin_expander_phy_enable

    my $ret = sasadmin_expander_phy_enable(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,phy_num => $phy_num,timeout => $timeout);

This subroutine enables a phy on the expander.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Adapter name to which shelf is connected

=item shelf_id

(Required) : Shelf id of the phy

=item phy_num

(Required) : Phy no to be enabled.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_expander_phy_enable($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            timeout => {type=>SCALAR, default=>"300",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            phy_num => {type=>SCALAR},
            shelf_id => {type=>SCALAR},
            adapter_name => {type=>SCALAR},

            }
            );
    my $output;
    my $adapter_name = $opts{adapter_name};;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno = $opts{phy_num};;
    my $rate;
    my $shelfid = $opts{shelf_id};
        my $cmd = "sasadmin expander_phy_enable $adapter_name\.$shelfid $phyno ";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
#print "OUTPUT: $output \n\n";
#EMS check to be added once available
        if($output =~ /expander_phy_enable: failed|Usage|No such adapter/){
            croak $output ;
        }
        return 1;

    }



=head2 sasadmin_expander_phy_reset

    my $ret = sasadmin_expander_phy_reset(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,phy_num => $phy_num,timeout => $timeout);

This subroutine resets a phy on the expander.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Adapter name to which expander is attached.

=item shelf_id

(Required) : Shelf id of the phy

=item phy_num

(Required) : Phy no to reset.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_expander_phy_reset($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                options => {type=>SCALAR,default=>" ",optional=>1},
                phy_num => {type=>SCALAR},
                shelf_id => {type=>SCALAR},
                adapter_name => {type=>SCALAR},

        }
    );
    my $output;
    my $adapter_name = $opts{adapter_name};;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno = $opts{phy_num};;
    my $rate;
    my $shelfid = $opts{shelf_id};

        my $cmd = "sasadmin expander_phy_reset $adapter_name\.$shelfid $phyno $opts{options}";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        #print "OUTPUT: $output \n\n";
        #EMS check to be added once available
        if($output =~ /expander_phy_reset: failed|Usage|No such adapter/){
            croak $output ;
        }
        return 1;

}

=head2 sasadmin_expander_phy_mux_reset

    my $ret = sasadmin_expander_phy_mux_reset(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,phy_num => $phy_num,timeout => $timeout);

This subroutine resets a mux on the expander.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Adapter name to which expander is attached.

=item shelf_id

(Required) : Shelf id of the phy

=item phy_num

(Required) : Phy no to reset.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_expander_phy_mux_reset($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            timeout => {type=>SCALAR, default=>"300",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            phy_num => {type=>SCALAR},
            shelf_id => {type=>SCALAR},
            adapter_name => {type=>SCALAR},

            }
            );
    my $output;
    my $adapter_name = $opts{adapter_name};;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno = $opts{phy_num};;
    my $rate;
    my $shelfid = $opts{shelf_id};
    my $cmd = "sasadmin expander_phy_mux_reset $adapter_name\.$shelfid $phyno ";
    $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},privmode=>'test',cmd=>$cmd);
#print "OUTPUT: $output \n\n";
#EMS check to be added once available
    if($output =~ /expander_phy_mux_reset: failed|Usage|No such adapter/){
        croak $output ;
    }
    return 1;

}

=head2 sasadmin_expander_phy_power_off

    my $ret = sasadmin_expander_phy_dongle_data(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,phy_num => $phy_num,timeout => $timeout);

This subroutine returns dongle data of a dongle on a phy
It returns a hash on success and croaks on failure.

$VAR1 = {
          'PartnerPHYstate' => '0x0',
          'DrivePowerOffCnt' => '0',
          'DrivePowerCycleCnt' => '0',
          'ReleaseCount' => '1',
          'DrivePowerOnCnt' => '0',
          'LocalPHYstate' => '0x0',
          'DongleType' => 'SiliconStor 1300B SATA',
          'MuxResetCnt' => '0',
          'LastCommand' => 'unknown command 0x0',
          'InProgressFlags' => '0x0',
          'DongleFWRevision' => 'rev. 1',
          'DrivePowerStatus' => 'ON',
          'ReserveCount' => '1'
        };


The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.

=item shelf_id

(Required) : Specific shelf for which the information is
 required.

=item phy_num

(Required) : Phy for which the dongle data is required
 required.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_expander_phy_dongle_data($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                phy_num => {type=>SCALAR},
                shelf_id => {type=>SCALAR},
                adapter_name => {type=>SCALAR},

        }
    );
    my $output;
    my $adapter_name = $opts{adapter_name};;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno = $opts{phy_num};;
    my $rate;
    my $shelfid = $opts{shelf_id};

        my $cmd = "sasadmin expander_phy_dongle_data $adapter_name\.$shelfid $phyno ";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
        #print "OUTPUT: $output \n\n";
        #EMS check to be added once available
        if($output =~ /expander_phy_dongle_data: failed|Usage|No such adapter|No dongle at PHY $phyno/){
            croak $output ;
        }

    my @lines = split(/\r*\n/, $output);
    my $line;
    my @line1;
    foreach $line (@lines){
        $line =~ s/DONGLE DATA:\s+|^\s+//g;
        @line1 = split (/:\s+/,$line);
        my $line2 = $line1[0];
        $line2 =~ s/:|\s+//g;
        $structure{$line2} = $line1[1];

    }
    return \%structure;
}


=head2 sasadmin_expander

    my $ret = sasadmin_expander(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,timeout => $timeout);

This subroutine returns a hash of information returned by "sasadmin expander" command.
It returns 1 on success and croaks on failure.

Example of structure returned by the function:

$VAR1 = {

          '6b.129' => {

                        'Product' => 'SXP 36x3G',
                        'WWN' => '500e004aaaaaaa3f',

                        20 => {
                                'WWN' => undef,
                                'WidePortPhy' => '0',
                                'Rate' => 'Rate',
                                'DeviceType' => undef,
                                'WidePort' => 2
                              },
                        21 => {
                                'WWN' => undef,
                                'WidePortPhy' => 1,
                                'Rate' => 'Rate',
                                'DeviceType' => undef,
                                'WidePort' => 2
                              },
                        22 => {
                                'WWN' => undef,
                                'WidePortPhy' => 2,
                                'Rate' => 'Rate',
                                'DeviceType' => undef,
                                'WidePort' => 2
                              },
                        23 => {
                                'WWN' => undef,
                                'WidePortPhy' => 3,
                                'Rate' => 'Rate',
                                'DeviceType' => undef,
                                'WidePort' => 2
                              },
                        24 => {
                                'PortPhy' => 19,
                                'Rate' => 'Rate unknown'
                              },
                        25 => {
                                'PortPhy' => 15,
                                'Rate' => 'Rate unknown'
                              },
                        26 => {
                                'PortPhy' => 11,
                                'Rate' => 'Rate unknown'
                              },
                        27 => {
                                'PortPhy' => 7,
                                'Rate' => 'Rate unknown'
                              },
                        28 => {
                                'PortPhy' => 18,
                                'Rate' => 'Rate unknown'
                              },
                        'Rev' => 'REVA',
                        29 => {
                                'WWN' => '500e004aaaaaaa1d',
                                'DiskId' => '6b.aaaaaa3f.23',
                                'PortPhy' => 23,
                                'Rate' => '3.0',
                                'DeviceType' => 'SATA'
                              },
                        'Vendor' => '\'PMCSIERA\',',
                        '0' => {
                                 'PortPhy' => 1,
                  'Rate' => 'Rate unknown'
                               },
                        1 => {
                               'PortPhy' => 5,
                               'Rate' => 'Rate unknown'
                             },
                        2 => {
                               'PortPhy' => 13,
                               'Rate' => 'Rate unknown'
                             },
                        3 => {
                               'PortPhy' => 9,
                               'Rate' => 'Rate unknown'
                             },
                        10 => {
                                'PortPhy' => 8,
                                'Rate' => 'Rate unknown'
                              },
                        4 => {
                               'PortPhy' => '0',
                               'Rate' => 'Rate unknown'
                             },
                        'SerialNumber' => '0xaaaaaa3f',
                        11 => {
                                'WWN' => '500e004aaaaaaa0b',
                                'DiskId' => '6b.aaaaaa3f.20',
                                'PortPhy' => 20,
                                'Rate' => '3.0',
                                'DeviceType' => 'SATA'
                              },
                        5 => {
                               'PortPhy' => 4,
                               'Rate' => 'Rate unknown'
                             },
                        12 => {
                                'WWN' => '500605b00000b9f4',
                                'WidePortPhy' => '0',
                                'Rate' => '3.0',
                                'DeviceType' => 'Initiator',
                                'WidePort' => '0'
                              },
                        6 => {
                               'PortPhy' => 21,
                               'Rate' => 'Rate unknown'
                             },
                        13 => {
                                'WWN' => undef,
                                'WidePortPhy' => 1,
                                'Rate' => 'Rate',
                                'DeviceType' => undef,
                                'WidePort' => '0'
                              },
                        7 => {
                               'PortPhy' => 17,
                               'Rate' => 'Rate unknown'
                             },
                        30 => {
                                'PortPhy' => 10,
                                'Rate' => 'Rate unknown'
                              },
                        14 => {
                                'WWN' => undef,
                                'WidePortPhy' => 2,
                                'Rate' => 'Rate', 'DeviceType' => undef,
                                'WidePort' => '0'
                              },
                        8 => {
                               'PortPhy' => 12,
                               'Rate' => 'Rate unknown'
                             },
                        31 => {
                                'PortPhy' => 6,
                                'Rate' => 'Rate unknown'
                              },
                        15 => {
                                'WWN' => '500605b00000b9f4',
                                'WidePortPhy' => 3,
                                'Rate' => '3.0',
                                'DeviceType' => 'Initiator',
                                'WidePort' => '0'
                              },
                        9 => {
                               'PortPhy' => 16,
                               'Rate' => 'Rate unknown'
                             },
                        'ID' => 129,
                        32 => {
                                'PortPhy' => 2,
                                'Rate' => 'Rate unknown'
                              },
                        16 => {
                                'WWN' => undef,
                                'WidePortPhy' => '0',
                                'Rate' => 'Rate',
                                'DeviceType' => undef,
                                'WidePort' => 1
                              },
                        33 => {
                                'PortPhy' => 3,
                                'Rate' => 'Rate unknown'
                              },
                        17 => {
                                'WWN' => undef,
                                'WidePortPhy' => 1,
                                'Rate' => 'Rate',
                                'DeviceType' => undef,
                                'WidePort' => 1
                              },
                        34 => {
                                'WWN' => '500e004aaaaaaa22',
                                'DiskId' => '6b.aaaaaa3f.22',
                                'PortPhy' => 22,
                                'Rate' => '3.0',
                                'DeviceType' => 'SATA'
                              },
                        18 => {
                                'WWN' => undef,
                                'WidePortPhy' => 2,
                                'Rate' => 'Rate',
                                'DeviceType' => undef,
                                'WidePort' => 1
                              },
                        35 => {
                                'PortPhy' => 14,
                                'Rate' => 'Rate unknown'
                              },
                        19 => {'WWN' => undef,
                                'WidePortPhy' => 3,
                                'Rate' => 'Rate',
                                'DeviceType' => undef,
                                'WidePort' => 1
                              },
                        36 => {
                                'WWN' => '500e004aaaaaaa3e',
                                'DiskId' => '6b.aaaaaa3f.230',
                                'PortPhy' => 'V0',
                                'Rate' => '3.0',
                                'DeviceType' => 'Virtual'
                              }
                      }

        };




The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.

=item shelf_id

(Required) : Specific shelf for which the information is
 required.


=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut


sub sasadmin_expander($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            timeout => {type=>SCALAR, default=>"300",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            shelf_id => {type=>SCALAR,default=>undef,optional=>1},
            adapter_name => {type=>SCALAR},

            }
            );
    my $output;
    my $adapter_name = $opts{adapter_name};
    my $shelfid = $opts{shelf_id};
    my $id;
    my $adapt_shelf;
    my $physical;
    my $portphy;
    my $portphy1;
    my $rate;
    my $devicetype;
    my $wwn;
    my $diskid;
    my $timeout = $opts{timeout};


    my %structure =();

    if (defined($shelfid)){
        $id = "$adapter_name\.$shelfid";
    }
    else{
        $id = $adapter_name;
    }
    my $cmd = "sasadmin expander $id";
    $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
    if($output !~ /WWN/) {
        croak $output;
    }

    my @lines = split(/\r*\n/, $output);
#print " \n\n @lines \n";
    foreach (@lines) {
        if($_ =~ /^WWN/){
            my @abc = split(/\s+|\r*\n/,$_);
            $abc[1] =~ s/,//g;
                $abc[3] =~ s/,//g;
                $abc[6] =~ s/\'//g;
            $adapt_shelf = "$adapter_name\.$abc[3]";
            $structure{$adapt_shelf}{WWN} = $abc[1];
            $structure{$adapt_shelf}{ID} = $abc[3];
            $structure{$adapt_shelf}{SerialNumber} = $abc[6];
        }
        elsif($_ =~ /^Manufacturer/){
            my @abc = split(/\s+|\r*\n/,$_);
            $abc[4] =~ s/\'//g;
            $abc[2] =~ s/\'//g;
            $abc[5] =~ s/\'|\,//g;
                $abc[8] =~ s/\'|\,//g;
            $structure{$adapt_shelf}{Vendor} = $abc[2];
#$structure{$adapt_shelf}{Product} = "$abc[4] $abc[5]";
            $structure{$adapt_shelf}{Product} = "$abc[5]";
            $structure{$adapt_shelf}{Rev} = $abc[8];
            $structure{$adapt_shelf}{Level} = $abc[10];
        }
        elsif($_ =~ /^PHY/){
            my @abc = split(/\s+/,$_);
            $abc[0] =~ s/PHY\[//g;
            if($abc[1] =~ /\/P/){
                my $abc2 = $abc[1];
#$abc[0] =~ s/PHY[|\/|:|]:/ /g;
                $abc[1] =~ s/\D+/ /g;
                my @abc1 = split(/\s+/,$abc[1]);
                $structure{$adapt_shelf}{$abc1[0]}{WidePort} = $abc1[1];
                $structure{$adapt_shelf}{$abc1[0]}{WidePortPhy} = $abc1[2];
                if($abc[2]=~ /Rate/){
                    $structure{$adapt_shelf}{$abc1[0]}{Rate} = "Rate unknown";
                }
                else{
                    $structure{$adapt_shelf}{$abc1[0]}{Rate} = $abc[2];
                    $structure{$adapt_shelf}{$abc1[0]}{DeviceType} = $abc[4];
                    $structure{$adapt_shelf}{$abc1[0]}{WWN} = $abc[5];
                }
                if($abc2 =~ /\/S/){
                    $structure{$adapt_shelf}{$abc1[0]}{RoutingType} = "Subtractive";
                }
                elsif($abc2 =~ /\/T/){
                    $structure{$adapt_shelf}{$abc1[0]}{RoutingType} = "Table";
                }
                else{
                    $structure{$adapt_shelf}{$abc1[0]}{RoutingType} = $abc2 ;
                }


            }
            elsif($abc[0] =~ /\/V255/){
                $abc[0] =~ s/\[|\]//g;
                my @abc3 = split(/\//,$abc[0]);
                if($abc[2]=~ /Rate/){
                    $structure{$adapt_shelf}{$abc3[0]}{Rate} = "Rate unknown";
                }
                else{
                    $structure{$adapt_shelf}{$abc3[0]}{Rate} = $abc[1];
                    $structure{$adapt_shelf}{$abc3[0]}{DeviceType} = $abc[3];
                    $structure{$adapt_shelf}{$abc3[0]}{WWN} = $abc[4];
                    $structure{$adapt_shelf}{$abc3[0]}{BAY} = $abc3[1];
                }
                if($abc3[2] =~ /\/S/){
                    $structure{$adapt_shelf}{$abc3[0]}{RoutingType} = "Subtractive";
                }
                elsif($abc3[2] =~ /\/T/){
                    $structure{$adapt_shelf}{$abc3[0]}{RoutingType} = "Table";
                }
                else{
                    $structure{$adapt_shelf}{$abc3[0]}{RoutingType} = $abc3[2] ;
                }


            }
            else{
                if($abc[0] =~ /\d+/){
                    $physical = $abc[0];
                    $portphy1 = $portphy = $abc[1];
                    $rate = $abc[2];
                    $devicetype = $abc[4];
                    if($_ =~ /Virtual SAS Device/){
                        $wwn = $abc[7];
                        $diskid = $abc[8];
                    }
                    else{
                        $wwn = $abc[6];
                        $diskid = $abc[7];
                    }
                }
                elsif($abc[0] !~ /\d+/){
                    $physical = $abc[1];
                    $portphy1 = $portphy = $abc[2];
                    $rate = $abc[3];
                    $devicetype = $abc[5];
                    if($_ =~ /Virtual SAS Device/){
                        $wwn = $abc[8];
                        $diskid = $abc[9];
                    }
                    else{
                        $wwn = $abc[7];
                        $diskid = $abc[8];
                    }
                }

                $physical =~ s/\///g;
                $portphy =~ s/\/(T|S|D)]://g;
                $structure{$adapt_shelf}{$physical}{BAY} = $portphy;
                if($rate=~ /Rate/){
                    $structure{$adapt_shelf}{$physical}{Rate} = "Rate unknown";
                }
                else{
                    if(defined($diskid)) {$diskid =~ s/\(|\)//g;}
                    $structure{$adapt_shelf}{$physical}{Rate} = $rate;
                    $structure{$adapt_shelf}{$physical}{DeviceType} = $devicetype;
                    $structure{$adapt_shelf}{$physical}{WWN} = $wwn;
                    $structure{$adapt_shelf}{$physical}{DiskId} = $diskid;
                    if($portphy1 =~ /\/S/){
                        $structure{$adapt_shelf}{$physical}{RoutingType} = "Subtractive";
                    }
                    elsif($portphy1 =~ /\/T/){
                        $structure{$adapt_shelf}{$physical}{RoutingType} = "Table";
                    }
                    elsif($portphy1 =~ /\/D/){
                        $structure{$adapt_shelf}{$physical}{RoutingType} = "Direct";
                    }
                    else{
                        $structure{$adapt_shelf}{$physical}{RoutingType} = $portphy1 ;
                    }
                }
            }
        }
    }
#print  Dumper(\%structure);
    return \%structure;

}


=head2 sasadmin_expander_phy_state

    my $ret = sasadmin_expander_phy_state(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,timeout => $timeout);

This subroutine returns a hash of information returned by "sasadmin expander_phy_state" command.
It returns 1 on success and croaks on failure.

Example of structure returned by function:

$VAR1 = {
          '0c.1' => {
                      '11' => {
                                'DongleFWRev' => '3',
                                'PwrON' => 'YES',
                                'BAY' => '3',
                                'DongleType' => 'SPS3G',
                                'PhyChangeCountS' => 'f106715800000000',
                                'PhyChangeCountH' => '6',
                                'RsvCnt' => '0',
                                'DisparityErrorCount' => '39',
                                'InvalidDWordCount' => '41',
                                'PwrCclCnt' => '2',
                                'PhyResetProb' => '0',
                                'LossSyncCount' => '3',
                                'SATAAffiliation' => undef,
                                'STATE' => '3.0',
                                'icCRCErrorCount' => '22',
                                'RlsCnt' => '0'
                              },
                      '21' => {
                                'DongleFWRev' => '3',
                                'PwrON' => 'YES',
                                'BAY' => '13',
                                'DongleType' => 'SPS3G',
                                'PhyChangeCountS' => 'f106715800000000',
                                'PhyChangeCountH' => '6',
                                'RsvCnt' => '0',
                                'DisparityErrorCount' => '38',
                                'InvalidDWordCount' => '39',
                                'PwrCclCnt' => '2',
                                'PhyResetProb' => '0',
                                'LossSyncCount' => '3',
                                'SATAAffiliation' => undef,
                                'STATE' => '3.0',
                                'icCRCErrorCount' => '22',
                                'RlsCnt' => '0'
                              },
                  .............
                  ....
                  ..
}



The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.

=item shelf_id

(Required) : Specific shelf for which the information is
 required.


=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

# Function to be revisited after all the SW and FW changes are done.

sub sasadmin_expander_phy_state($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            timeout => {type=>SCALAR, default=>"300",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            shelf_id => {type=>SCALAR,default=>undef,optional=>1},
            adapter_name => {type=>SCALAR,default=>undef,optional=>1},

            }
            );
    my $output;
    my $adapter_name = $opts{adapter_name};
    my $shelfid = $opts{shelf_id};
    my $id;
    my $adapt_shelf;
    my $physical;
    my $portphy;
    my $rate;
    my $devicetype;
    my $wwn;
    my $diskid;

    my %structure =();
    if(defined($adapter_name) && defined($shelfid)){
        $id = "$adapter_name\.$shelfid";
    }
    elsif(defined($adapter_name)){
        $id = $adapter_name;
    }
    else{
        $id = "";
    }

        my $cmd = "sasadmin expander_phy_state $id";

     $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

    if($output !~ /Affiliation/) {
        croak $output;
    }

    my @lines = split(/\r*\n/, $output);
    foreach my $line (@lines){
        if($line =~ /Expander phy state on/){
            my @line5 = split(/\s+/,$line);
            $id = $line5[4];
            $id =~ s/://;
            my $model = $line5[6];
            $model =~ s/://;
            my $serial = $line5[7];
            $serial =~ s/\)://;
                $structure{$id}{Model} = $model;
                $structure{$id}{Serial} = $serial;
        }

        $line =~ s/\// /g;
        $line =~ s/^ //;
        my @line1 = split(/\s+/,$line);
        my $i = 0;
        my $l;
        if(defined($line1[0])){
            if($line1[0] =~ /\d+/){
                my $phy = $line1[0];
                $structure{$id}{$phy}{BAY} = $line1[1];
                if ($line1[2] =~ /Rate|Disabled/){
                    if($line1[2] =~ /Disabled/){
                        my $state1;
                        if(defined($line1[3])){
                            $state1 = "$line1[2]" . "\/" . "$line1[3]";
                        }
                        else{
                            $state1 = $line1[2];
                        }
                        $structure{$id}{$phy}{STATE} = $state1;
                    }
                    else{
                        $structure{$id}{$phy}{STATE} = "Rate unknown";
                    }
                    if($structure{$id}{Model} =~ /IOM3|IOM6/){
                        $structure{$id}{$phy}{PwrON} = $line1[4];
                        $structure{$id}{$phy}{PwrCclCnt} = $line1[5];
                        $structure{$id}{$phy}{RsvCnt} = $line1[6];
                        $structure{$id}{$phy}{RlsCnt} = $line1[7];
                        $structure{$id}{$phy}{InvalidDWordCount} = $line1[8];
                        $structure{$id}{$phy}{DisparityErrorCount} = $line1[9];
                        $structure{$id}{$phy}{LossSyncCount} = $line1[10];
                        $structure{$id}{$phy}{PhyResetProb} = $line1[11];
                        $structure{$id}{$phy}{CRCErrorCnt} = $line1[12];
                        $structure{$id}{$phy}{PhyChangeCnt} = $line1[13];
                        $structure{$id}{$phy}{SATAAffiliation} = $line1[14];
                    }
                    else{
                        $structure{$id}{$phy}{DongleType} = $line1[4];
                        $structure{$id}{$phy}{DongleFWRev} = $line1[5];
                        $structure{$id}{$phy}{PwrON} = $line1[6];
                        $structure{$id}{$phy}{PwrCclCnt} = $line1[7];
                        $structure{$id}{$phy}{RsvCnt} = $line1[8];
                        $structure{$id}{$phy}{RlsCnt} = $line1[9];
                        $structure{$id}{$phy}{InvalidDWordCount} = $line1[10];
                        $structure{$id}{$phy}{DisparityErrorCount} = $line1[11];
                        $structure{$id}{$phy}{LossSyncCount} = $line1[12];
                        $structure{$id}{$phy}{PhyResetProb} = $line1[13];
                        $structure{$id}{$phy}{CRCErrorCnt} = $line1[14];
                        $structure{$id}{$phy}{PhyChangeCnt} = $line1[15];
                        $structure{$id}{$phy}{SATAAffiliation} = $line1[16];
                    }


                }
                else{
                    if ($line1[2] =~ /---|YES/) {
                       croak "STATE field is missing and hence failing as per the burt 559655";
                    }
                    if($structure{$id}{Model} =~ /IOM3|IOM6/){
                        $structure{$id}{$phy}{STATE} = $line1[2];
                        $structure{$id}{$phy}{PwrON} = $line1[5];
                        $structure{$id}{$phy}{PwrCclCnt} = $line1[6];
                        $structure{$id}{$phy}{RsvCnt} = $line1[7];
                        $structure{$id}{$phy}{RlsCnt} = $line1[8];
                        $structure{$id}{$phy}{InvalidDWordCount} = $line1[9];
                        $structure{$id}{$phy}{DisparityErrorCount} = $line1[10];
                        $structure{$id}{$phy}{LossSyncCount} = $line1[11];
                        $structure{$id}{$phy}{PhyResetProb} = $line1[12];
                        $structure{$id}{$phy}{CRCErrorCnt} = $line1[13];
                        $structure{$id}{$phy}{PhyChangeCnt} = $line1[14];
                        $structure{$id}{$phy}{SATAAffiliation} = $line1[15];
                    }
                    else{
                        $structure{$id}{$phy}{STATE} = $line1[2];
                        $structure{$id}{$phy}{DongleType} = $line1[5];
                        $structure{$id}{$phy}{DongleFWRev} = $line1[6];
                        $structure{$id}{$phy}{PwrON} = $line1[7];
                        $structure{$id}{$phy}{PwrCclCnt} = $line1[8];
                        $structure{$id}{$phy}{RsvCnt} = $line1[9];
                        $structure{$id}{$phy}{RlsCnt} = $line1[10];
                        $structure{$id}{$phy}{InvalidDWordCount} = $line1[11];
                        $structure{$id}{$phy}{DisparityErrorCount} = $line1[12];
                        $structure{$id}{$phy}{LossSyncCount} = $line1[13];
                        $structure{$id}{$phy}{PhyResetProb} = $line1[14];
                        $structure{$id}{$phy}{CRCErrorCnt} = $line1[15];
                        $structure{$id}{$phy}{PhyChangeCnt} = $line1[16];
                        $structure{$id}{$phy}{SATAAffiliation} = $line1[17];
                    }


                }

            }
        }
    }
#print  Dumper(\%structure);
    return \%structure;
}

=head2 sasadmin_expander_routes

    my $ret = sasadmin_expander_routes(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,timeout => $timeout);

This subroutine returns a hash of information returned by "sasadmin expander_routes" command.
It returns 1 on success and croaks on failure.
Example of structure returned by the function:

$VAR1 = {

          '6b.129' => {
                        'RoutesFound' => '0',
                        'RouteIndices' => 1024,
                        'TotalPhys' => 37
                      }

        };

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.

=item shelf_id

(Optional) : Specific shelf for which the information is
 required.


=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut


sub sasadmin_expander_routes($@) {

    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            timeout => {type=>SCALAR, default=>"600",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            shelf_id => {type=>SCALAR,default=>undef,optional=>1},
            adapter_name => {type=>SCALAR},

            }
            );
    my $output;
    my $adapter_name = $opts{adapter_name};
    my $shelfid = $opts{shelf_id};
    my $id;
    my $adapt_shelf;
    my $physical;
    my $portphy;
    my $rate;
    my $devicetype;
    my $wwn;
    my $diskid;
    my $timeout = $opts{timeout};


    my %structure =();
    $id = "$adapter_name\.$shelfid";
    my $cmd = "sasadmin expander_routes $id";
    $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
    if($output !~ /route indices/){
        croak $output;
    }
    my @lines = split(/\r*\n/,$output);
    foreach (@lines){
#sleep 300;
        my @abc = split(/\s+/,$_);
        if($_ =~ /route indices/){
            $abc[4] =~ s/\,//g;
                $structure{$id}{RouteIndices} = $abc[4];
            $structure{$id}{TotalPhys} = $abc[6];
        }
        elsif($_ =~ /routes found/){
            $abc[7] =~ s/\D+//g;
            $structure{$id}{RoutesFound} = $abc[7];
        }
    }
#print  Dumper(\%structure);
    return \%structure;
}

=head2 sasadmin_shelf

    my $ret = sasadmin_shelf(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,timeout => $timeout);

This subroutine returns a hash of information returned by "sasadmin shelf" command.
It returns 1 on success and croaks on failure.

Example of structure returned:

$VAR1 = {

          '6b.129' => {

                        'ShelfSerialNo' => 'aaaaaa3f',
                        '0' => {
                                 'Rate' => 'Rate unknown'
                               },
                        1 => {
                               'Rate' => 'Rate unknown'
                             },
                        2 => {
                               'Rate' => 'Rate unknown'
                             },
                        3 => {
                               'Rate' => 'Rate unknown'
                             },
                        4 => {
                               'Rate' => 'Rate unknown'
                             },
                        10 => {
                                'Rate' => 'Rate unknown'
                              },
                        5 => {
                               'Rate' => 'Rate unknown'
                             },
                        11 => {
                                'Rate' => 'Rate unknown'
                              },
                        20 => {
                                'Rate' => '3.0 Gb/s'
                              },
                        6 => {
                               'Rate' => 'Rate unknown'
                             },
                        12 => {
                                'Rate' => 'Rate unknown'
                              },
                        21 => {
                                'Rate' => 'Rate unknown'
                              },
                        7 => {
                               'Rate' => 'Rate unknown'
                             },
                        13 => {
                                'Rate' => 'Rate unknown'
                              },
                        22 => {
                                'Rate' => '3.0 Gb/s'
                              },
                        'ShelfID' => 129,
                        8 => {
                               'Rate' => 'Rate unknown'
                             },
                        14 => {
                                'Rate' => 'Rate unknown'
                              },
                        23 => {
                                'Rate' => '3.0 Gb/s'
                              },
                        9 => {
                               'Rate' => 'Rate unknown'
                             },
                        15 => {
                                'Rate' => 'Rate unknown'
                },
                        16 => {
                                'Rate' => 'Rate unknown'
                              },
                        17 => {
                                'Rate' => 'Rate unknown'
                              },
                        18 => {
                                'Rate' => 'Rate unknown'
                              },
                        19 => {
                                'Rate' => 'Rate unknown'
                              }
                      }

        };



The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Optional) : Specific channel for which the information is
 required.


=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut



sub sasadmin_shelf_old($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                adapter_name => {type=>SCALAR,default=>undef,optional=>1},

        }
    );
    my $output;
    my $adapter_name = $opts{adapter_name};
    my $id;
    my $adapt_shelf;
    my $physical;
    my $portphy;
    my $rate;
    my $devicetype;
    my $wwn;
    my $diskid;
    my $timeout = $opts{timeout};


    my %structure =();
    if(defined($adapter_name)){
            $id = $adapter_name;
    }
    else{
        $id = "";
    }

        my $cmd = "sasadmin shelf $id";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if($output !~ /Expanders on channel/){
            croak $output;
        }
        $output =~ s/\-|\+|\|//g;
        my $adapt_id;
        my @lines = split(/\r*\n/,$output);
        foreach (@lines){
            if($_ =~ /Expanders on channel/){
                my @line5 = split(/\s+/,$_);
                $adapt_id = $line5[3];
                $adapt_id =~ s/://;
                print "ADAPT : $adapt_id \n";
            }
            my @abc = split(/\s+/,$_);
            if(defined($abc[1])){
            if($abc[1] =~ /ID:/){
                $id = "$adapt_id" . "." . "$abc[2]";
                $structure{$id}{ShelfID} = $abc[2];
                $structure{$id}{SerialNumber} = $abc[5];
                $structure{$id}{SASADDR} = $abc[8];
                $structure{$id}{SLOT} = $abc[10];
            }
            elsif($abc[1] =~ /\d+/){
                $structure{$id}{$abc[1]}{Rate} = "$abc[2] $abc[3]";
                $structure{$id}{$abc[4]}{Rate} = "$abc[5] $abc[6]";
                $structure{$id}{$abc[7]}{Rate} = "$abc[8] $abc[9]";
                $structure{$id}{$abc[10]}{Rate} = "$abc[11] $abc[12]";
            }
            }
        }
        #print  Dumper(\%structure);
        return \%structure;
}

=head2 sasadmin_shelf

    my $ret = sasadmin_shelf(host => $host, adapter_name => $adapter_name,timeout => $timeout);

This subroutine returns a hash of information returned by "sasadmin shelf_short" command.
It returns 1 on success and croaks on failure.

Example of structure returned:
$VAR1 = {

          '6b.129' => {

                        '0' => 'XX',
                        1 => 'XX',
                        2 => 'XX',
                        3 => 'XX',
                        4 => 'XX',
                        10 => 'XX',
                        5 => 'XX',
                        11 => 'XX',
                        20 => 20,
                        6 => 'XX',
                        12 => 'XX',
                        21 => 'XX',
                        7 => 'XX',
                        13 => 'XX',
                        22 => 22,
                        8 => 'XX',
                        14 => 'XX',
                        23 => 23,
                        'ShelfId' => 129,
                        9 => 'XX',
                        15 => 'XX',
                        16 => 'XX',
                        17 => 'XX',
                        18 => 'XX',
                        19 => 'XX'

                      }

        };

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.


=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut



sub sasadmin_shelf($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                adapter_name => {type=>SCALAR,default=>undef,optional=>1},

        }
    );
    my $output;
    my $adapter_name = $opts{adapter_name};
    my $id;
    my $adapt_shelf;
    my $physical;
    my $portphy;
    my $rate;
    my $devicetype;
    my $wwn;
    my $diskid;
    my $timeout = $opts{timeout};
    my $i = 4;
    my $j = 5;
    my $k = 6;
    my $l = 7;


    #$id = "$adapter_name\.$shelfid";

    if ($opts{filermode} ne "normal") {
        $opts{conntype} = "#console";
    }
    my $conn = make_conn($opts{host},$opts{conntype});

    my %structure =();

        my $cmd = "sasadmin shelf $id";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if($output !~ /----/){
            croak $output;
        }
        $output =~ s/\-|\+|\|//g;
        my @lines = split(/\r*\n/,$output);
        my $adapter_shelf;
        foreach (@lines){
            my @abc = split(/\s+/,$_);
            if($#abc == 3){
                if($abc[0] =~ /Expanders/){
                    $id = $abc[3];
                    $id =~ s/://;
                }
            }
            if($#abc == 5){
                $adapter_shelf = "$id" . "." .  "$abc[1]";
                $structure{$adapter_shelf}{ShelfId} = $abc[1];
                $structure{$adapter_shelf}{'0'} = $abc[2];
                $structure{$adapter_shelf}{'1'} = $abc[3];
                $structure{$adapter_shelf}{'2'} = $abc[4];
                $structure{$adapter_shelf}{'3'} = $abc[5];
                $i = 4;
                $j = 5;
                $k = 6;
                $l = 7;
            }
            elsif($#abc == 4){
                $structure{$adapter_shelf}{$i} = $abc[1];
                $structure{$adapter_shelf}{$j} = $abc[2];
                $structure{$adapter_shelf}{$k} = $abc[3];
                $structure{$adapter_shelf}{$l} = $abc[4];
                $i = $i + 4;
                $j = $j + 4;
                $k = $k + 4;
                $l = $l + 4;
                }
            }
        #print  Dumper(\%structure);
        return \%structure;
}


=head2 sasadmin_channels

    my $ret = sasadmin_channels(host => $host, adapter_name => $adapter_name,timeout => $timeout);

This subroutine returns a hash of information returned by "sasadmin channels" command.
It returns 1 on success and croaks on failure.

Example of structure returned:

$VAR1 = {

          '6a' => {
                    'WWN' => '500605b00000b9f0',
                    'PortType' => 'SAS-port',
                    'GIHAID' => 1
                  },
          '6b' => {
                    'WWN' => '500605b00000b9f4',
                    'PortType' => 'SAS-port',
                    'GIHAID' => 2
                  }

        };



The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Optional) : Specific channel for which the information is
 required.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut



sub sasadmin_channels($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
        }
    );
    my $output;

    my $timeout = $opts{timeout};

    my %structure =();

        my $cmd = "sasadmin channels";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if($output !~ /GIHAID/) {
            croak $output;
        }

        my @lines = split(/\r*\n/, $output);
        foreach (@lines){
            if($_ =~ /GIHAID/){
                my @abc = split(/\s+/,$_);
                $structure{$abc[1]}{WWN} = $abc[0];
                $structure{$abc[1]}{PortType} = $abc[2];
                $abc[4] =~ s/\)//g;
                $structure{$abc[1]}{GIHAID} = $abc[4];
            }

        }
        #print  Dumper(\%structure);
        return \%structure;
}

=head2 sasadmin_expander_map

    my $ret = sasadmin_expander_map(host=>$host, adapter_name=>$adapter_name,timeout => $timeout);

This routine returns a hash table of information for each channel in the filer.
It returns 1 on success and croaks on failure.

Example of reference returned by the function:

$VAR1 = {
          '4a' => {
                    '500c0ff10a41d43f' => {
                                            'ID' => '90',
                                            'SerialNumber' => '0x00000000',
                                            'Slot' => 'B',
                                            'Level' => '1',
                                            'Product' => 'NA-2400-SAS',
                                            'Rev' => '1001'
                                          }
                  },
          '1a' => {
                    '00000ff00a420e3f' => {
                                            'ID' => '2',
                                            'SerialNumber' => '0x000a420e',
                                            'Slot' => 'A',
                                            'Level' => '2',
                                            'Product' => 'NA-2400-SAS',
                                            'Rev' => '1001'
                                          },
                    '00000ff00a42633f' => {
                                            'ID' => '3',
                                            'SerialNumber' => '0x000a4263',
                                            'Slot' => 'A',
                                            'Level' => '3',
                                            'Product' => 'NA-2400-SAS',
                                            'Rev' => '1001'
                                          },
                    '10000ff00a42013f' => {
                                            'ID' => '4',
                                            'SerialNumber' => '0x000a4201',
                                            'Slot' => 'A',
                                            'Level' => '4',
                                            'Product' => 'NA-2400-SAS',
                                            'Rev' => '1001'
                                          },
                    '00000ff00a420f3f' => {
                                            'ID' => '1',
                                            'SerialNumber' => '0x000a420f',
                                            'Slot' => 'A',
                                            'Level' => '1',
                                            'Product' => 'NA-2400-SAS',
                                            'Rev' => '1001'
                                          }
                  }
        };


The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Optional) : Specific channel for which the information is
 required. Default is all

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back

=cut

sub sasadmin_expander_map($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                adapter_name => {type=>SCALAR, default=>undef, optional=>1},
                timeout => {type=>SCALAR, default=>300,optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
        }
    );
    my $adapter = undef ;
    my $output = "";
    if(defined($opts{adapter_name})){
        $adapter = $opts{adapter_name};
    }

    my %structure =();

        my $cmd = "sasadmin expander_map $adapter";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
    if($output !~ /Expanders on channel/){
        croak $output;
    }
    my $hba;

    my @lines = split(/\r*\n/, $output);
    foreach (@lines) {
        if($_ =~ /Expanders on channel/){
            my @adapt = split(/\s+/, $_);
            $hba = $adapt[3];
            $hba =~ s/://g;
        }

        if($_ =~ /^Level/) {
            $_ =~ s/'\s+//g;
            my @fields = split(/\s+/, $_);
            $fields[1] =~ s/://g;
            $fields[3] =~ s/,//g;
            $fields[5] =~ s/,//g;
            $fields[8] =~ s/'|,//g;
            $fields[10] =~ s/'|,//g;
            $fields[13] =~ s/'|,//g;
            $fields[15] =~ s/'|,//g;

            $structure{$hba}{"$fields[3]"}{'Level'} = $fields[1];
            $structure{$hba}{"$fields[3]"}{'ID'} = $fields[5];
            $structure{$hba}{"$fields[3]"}{'SerialNumber'} = $fields[8];
            $structure{$hba}{"$fields[3]"}{'Product'} = $fields[10];
            $structure{$hba}{"$fields[3]"}{'Rev'} = $fields[13];
            $structure{$hba}{"$fields[3]"}{'Slot'} = $fields[15];
        }
    }
    return \%structure;
}

=head2 sasadmin_expander_ctrl

    my $ret = sasadmin_expander_ctrl(host=>$host, adapter_name=>$adapter_name,shelf_id=>$shelf_id,shelf_wwn=>$shelf_wwn,phy_num=>$phy_num,ctrl_value=>$ctrl_value,module_id=>$module_id,timeout => $timeout);

This subroutine injects a ctrl error on to a phy.
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Adapter name to which expander is attached.

=item shelf_id

(Required) : Shelf id of the phy. Required only if shelf_wwn is no defined

=item phy_num

(Required) : Phy no to reset.

=item shelf_wwn

(Required) : Shelf wwn. Required only if shelf_id is not defined.

=item ctrl_value

(Required) : Ctrl value of error injection

=item module_id

(Required) : Module id A or B

=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_expander_ctrl($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                phy_num => {type=>SCALAR},
                shelf_id => {type=>SCALAR,optional=>1},
                shelf_wwn => {type=>SCALAR,optional=>1},
                adapter_name => {type=>SCALAR},
                ctrl_value => {type=>SCALAR},
                module_id => {type=>SCALAR},

        }
    );
    my $output;
    my $adapter_name = $opts{adapter_name};;
    my $timeout = $opts{timeout};
    my %structure =();
    my $phyno = $opts{phy_num};;
    my $rate;
    my $shelfid = $opts{shelf_id};
    my $cmd;
    my $retry = 0;
            if(defined($opts{shelf_id})){
                 $cmd = "sasadmin expander_ctrl -h $adapter_name -s $opts{shelf_id} -p $phyno -v $opts{ctrl_value} -l $opts{module_id}";
            }
            elsif(defined($opts{shelf_wwn})){
                 $cmd = "sasadmin expander_ctrl -h $adapter_name -s $opts{shelf_wwn} -p $phyno -v $opts{ctrl_value} -l $opts{module_id}";
            }
            else{
                logresult('fatal',"Neither shelf_id nor shelf_wwn specified");
            }
        CTRL_LABEL: ;
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        #print "OUTPUT: $output \n\n";
        #EMS check to be added once available
        # Retry added to get around to burt: 226220
        if(($output =~ /expander_ctrl:\s*\w*\s*failed/) && ($retry < 1)){
            logcomment("Expander_ctrl failed with : $output. Retrying");
            $retry++;
            sleep 5;
            goto CTRL_LABEL;
        }
        if($output =~ /expander_ctrl:\s*\w*\s*failed|Usage|No such adapter/){
            croak $output ;
        }
        return 1;

}


=head2 sasadmin_expander_cli

    my $ret = sasadmin_expander_cli

This subroutine issues cli command to an IOM
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Adapter name to which expander is attached.

=item shelf_id

(Required) : Shelf id of the phy

=item command

(Required) : Command to be issued to IOM


=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_expander_cli($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            node => {type=>HASHREF, default=>undef, optional=>1},
            timeout => {type=>SCALAR, default=>"300",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            command => {type=>SCALAR},
            shelf_id => {type=>SCALAR},
            adapter_name => {type=>SCALAR},
            return_data => {type=>SCALAR,default=>1,optional=>1 },
            retry => {type=>SCALAR,default=>5,optional=>1 },
            }
            );
    my $output;
    my $adapter_name = $opts{adapter_name};;
    my $timeout = $opts{timeout};
    my $command = $opts{command};
    my %structure =();
    my $rate;
    my $shelfid = $opts{shelf_id};
    my $retry;
    my $filer_ci;

    if (defined $opts{node}) {
        $filer_ci = $opts{node};
    } else {
        $filer_ci = NACL::C::Node->find();
    }
#    $filer_ci = NACL::C::Node->new(hostrec => $opts{host});
    my $api_set_obj;
    $api_set_obj = $filer_ci->get_7m_or_nodescope_apiset();
    $api_set_obj->set_timeout( 'connectrec-timeout' => 1800 );
CLI_LABEL:;
          my $cmd = "sasadmin expander_cli $adapter_name\.$shelfid \'$command\' ";
          eval {
              $output = $api_set_obj->execute_command('command' => $cmd);
          };
          print "response dump is" . Dumper $output;

          if($output =~ /Enclosure Services Unavailable/){
              if($retry <= $opts{retry}){
                  logcomment("Inband CLI command failed because Enclosure services are Unavaiable. Retry: $retry Retry after 20 secs");
                  sleep 20;
                  $retry++;
                  goto CLI_LABEL;
              }
              else{
                  logcomment("Inband cli command failed with enclosure services unavaiable even after $opts{retry} retries with 20 secs interval");
                  croak $output ;
              }
          }
          if($output =~ /DSA DB lock/){
              if($retry <= $opts{retry}){
                  logcomment("Inband CLI command failed because DSA DB lock: UNLOCKED; LOCKs held. Retry: $retry Retry after 20 secs");
                  sleep 20;
                  $retry++;
                  goto CLI_LABEL;
              }
              else{
                  logcomment("Inband cli command failed with DSA DB lock even after $opts{retry} retries with 20 secs interval");
                  croak $output ;
              }
          }
          if(($output =~ /expander_cli: failed/) && ($command !~ /watchdog_tp|reboot/)){
              croak $output ;
          }
          if($output =~ /Usage|not found|Invalid testpoint_code/){
              croak $output ;
          }
		  
		  $retry = 0;
		if($output =~ /resource busy/ && ($retry < 15)) {
			logcomment("Resource busy. Retrying after 60 seconds");
			$retry++;
			sleep 60;
			goto CTRL_LABEL;
		}
		
		if($output =~ /resource busy/ && ($retry == 15)) { 
			 croak $output ;
		}
		  
          if($opts{return_data}){
              return $output;
          } else{
              return 1;
          }

}
=head2 sasadmin_dev_stats

    my $ret = sasadmin_dev_stats(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,timeout => $timeout);

This subroutine returns a hash of information returned by "sasadmin expander_phy_state" command.
It returns 1 on success and croaks on failure.

Example of structure returned by function:
$VAR1 = {
          '0c' => {
                    '0c.20.6' => {
                                   'SATAInitFailCnt' => '0',
                                   'CRCErrCnt' => '0',
                                   'SCSIPtclErrCnt' => '0',
                                   'CmdsCmpltGood' => '115',
                                   'TO4Cnt' => '0',
                                   'TO3Cnt' => '0',
                                   'TrnsErrCnt' => '0',
                                   'InvdCnt' => '4',
                                   'FramesOut' => '331',
                                   'CmdsCmpltError' => '2',
                                   'TO5Cnt' => '0',
                                   'FramesIn' => '124',
                                   'TO6Cnt' => '0',
                                   'UdrnCnt' => '1',
                                   'VctmAbrt' => '0',
                                   'OvrnCnt' => '0',
                                   'DelayCnt' => '0',
                                   'TO2Cnt' => '0',
                                   'QsceCnt' => '0',
                                   'TO1Cnt' => '0'
                                 },
                    '0c.20.18' => {
                                    'SATAInitFailCnt' => '0',
                                    'CRCErrCnt' => '0',
                                    'SCSIPtclErrCnt' => '2',
                                    'CmdsCmpltGood' => '595335',
                                    'TO4Cnt' => '0',
                                    'TO3Cnt' => '0',
                                    'TrnsErrCnt' => '0',
                                    'InvdCnt' => '6',
                                    'FramesOut' => '595572',
                                    'CmdsCmpltError' => '5',
                                    'TO5Cnt' => '0',
                                    'FramesIn' => '595357',
                                    'TO6Cnt' => '0',
                                    'UdrnCnt' => '1',
                                    'VctmAbrt' => '0',
                                    'OvrnCnt' => '0',
                                    'DelayCnt' => '0',
                                    'TO2Cnt' => '0',
                                    'QsceCnt' => '0',
                                    'TO1Cnt' => '0'
                                  },
            ................
            ...
            0c.20.1' => {
                                   'SATAInitFailCnt' => '0',
                                   'CRCErrCnt' => '0',
                                   'SCSIPtclErrCnt' => '0',
                                   'CmdsCmpltGood' => '266854',
                                   'TO4Cnt' => '0',
                                   'TO3Cnt' => '0',
                                   'TrnsErrCnt' => '0',
                                   'InvdCnt' => '4',
                                   'FramesOut' => '533816',
                                   'CmdsCmpltError' => '5',
                                   'TO5Cnt' => '0',
                                   'FramesIn' => '266867',
                                   'TO6Cnt' => '0',
                                   'UdrnCnt' => '1',
                                   'VctmAbrt' => '3',
                                   'OvrnCnt' => '0',
                                   'DelayCnt' => '2',
                                   'TO2Cnt' => '0',
                                   'QsceCnt' => '0',
                                   'TO1Cnt' => '0'
                                 }
                  }
        };




The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.

=item shelf_id

(Required) : Specific shelf for which the information is
 required.


=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

# Function to be revisited after all the SW and FW changes are done.

sub sasadmin_dev_stats($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                adapter_name => {type=>SCALAR,default=>"",optional=>1},

        }
    );
    my $output;
    my $adapter_name = $opts{adapter_name};
    my $id;
    my $adapt_shelf;
    my $physical;
    my $portphy;
    my $rate;
    my $devicetype;
    my $wwn;
    my $diskid;
    my $timeout = $opts{timeout};


    my %structure =();


        my $cmd = "sasadmin dev_stats $adapter_name";
          $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

        if($output !~ /Device stats on channel/) {
            croak $output;
        }

        my @lines = split(/\r*\n/, $output);
        my $channel;
        foreach my $line (@lines){
            my @line1 = split(/\s+/,$line);
            if($line =~ /Device stats on channel/){
                $channel = $line1[4];
                $channel =~ s/://;
            }
            if($line1[0] =~ /\.\d+\./){
                my $i = 0;
                #print "Channel $channel \n";
                #foreach my $l (@line1){
                #    print "$i === $l \n";
                #    $i++;
                #}
                $structure{$channel}{$line1[0]}{CmdsCmpltGood} = $line1[1];
                $structure{$channel}{$line1[0]}{CmdsCmpltError} = $line1[2];
                $structure{$channel}{$line1[0]}{FramesIn} = $line1[3];
                $structure{$channel}{$line1[0]}{FramesOut} = $line1[4];
                $structure{$channel}{$line1[0]}{VctmAbrt} = $line1[5];
                $structure{$channel}{$line1[0]}{QsceCnt} = $line1[6];
                $structure{$channel}{$line1[0]}{UdrnCnt} = $line1[7];
                $structure{$channel}{$line1[0]}{OvrnCnt} = $line1[8];
                $structure{$channel}{$line1[0]}{TrnsErrCnt} = $line1[9];
                $structure{$channel}{$line1[0]}{SCSIPtclErrCnt} = $line1[10];
                $structure{$channel}{$line1[0]}{CRCErrCnt} = $line1[11];
                $structure{$channel}{$line1[0]}{OpenErrCnt} = $line1[12];
                $structure{$channel}{$line1[0]}{InvdCnt} = $line1[13];
                $structure{$channel}{$line1[0]}{DelayCnt} = $line1[14];
                $structure{$channel}{$line1[0]}{SATAInitFailCnt} = $line1[15];
                $structure{$channel}{$line1[0]}{TO1Cnt} = $line1[16];
                $structure{$channel}{$line1[0]}{TO2Cnt} = $line1[17];
                $structure{$channel}{$line1[0]}{TO3Cnt} = $line1[18];
                $structure{$channel}{$line1[0]}{TO4Cnt} = $line1[19];
                #$structure{$channel}{$line1[0]}{TO5Cnt} = $line1[20];
                #$structure{$channel}{$line1[0]}{TO6Cnt} = $line1[21];
            }
        }
    return \%structure;
}

=head2 sasadmin_adapter_reservation_state

    my $ret = sasadmin_adapter_reservation_state(host => $host, adapter_name => $adapter_name,shelf_id => $shelf_id,timeout => $timeout);

This subroutine returns a hash of information returned by "sasadmin expander_phy_state" command.
It returns 1 on success and croaks on failure.

Example of structure returned by function:

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required.

=item shelf_id

(Required) : Specific shelf for which the information is
 required.


=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_adapter_reservation_state($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                timeout => {type=>SCALAR, default=>"300",optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                adapter_name => {type=>SCALAR},

        }
    );
    my $output;
    my $adapter_name = $opts{adapter_name};
    my $id;
    my $adapt_shelf;
    my $physical;
    my $portphy;
    my $rate;
    my $devicetype;
    my $wwn;
    my $diskid;
    my $timeout = $opts{timeout};


    my %structure =();

        my $cmd = "sasadmin adapter_reservation_state $adapter_name";
          $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
        if($output !~ /State valid:/) {
            croak $output;
        }
        my @state = split(/\s+/,$output);
        $structure{$adapter_name}{Unreserved} = $state[5];
        $structure{$adapter_name}{Reserved} = $state[9];
        return \%structure ;
}


=head2 sasadmin_inject

    my $ret = sasadmin_inject(host => $host, adapter_name => $adapter_name,timeout => $timeout,command=>"abort",period=>$period);

This routine injects various "sasadmin inject errors for a specific period of time
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Optional) : Specific channel for which the information is
 required. This is optional if device_name is defined otherwise it's a required option.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item device_name

(Optional) : Device name on which the errors are to be injected. This is optional if adapter_name is defined otherwise it's a required option.

=item command

(Required) :  Command options to inject. Options are : "dev_reset", "adapter_reset", "abort", "timeout", "show", "disable".

=item period

(Optional) : Period how long we want to run the error injections. Default is 300 secs.

=back



=cut

sub sasadmin_inject($@) {
    my (%opts) = validate(@_,{
                host => {type=>OBJECT, isa=>'Hostrec'},
                adapter_name => {type=>SCALAR,default=>undef,optional=>1},
                device_name => {type=>SCALAR,default=>undef,optional=>1},
                period => {type=>SCALAR,default=>300,optional=>1},
                command => {type=>SCALAR},
                timeout => {type=>SCALAR,default=>300,optional=>1},
                filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                ems_verify => {type=>SCALAR,default=>"yes",optional=>1},
        }
    );
    my $output = "";
    my $adapter_name;
    my $a = "";
    my $b = "";
    my $mark;
    my $cmd_option;
    my $cmd1;
    my $cmd;
        $cmd1 = "sasadmin inject";
    if($opts{command} =~ /^dev_reset$/){
        if(defined($opts{adapter_name})){
            $cmd = "$cmd1 dev_reset enable -t $opts{period} -r -c $opts{adapter_name}";
        }
        elsif(defined($opts{device_name})){
            $cmd = "$cmd1 dev_reset enable -t $opts{period} -d $opts{device_name}";
        }
        else{
            logresult('script', "adapter_name or device_name are the required parameters");
        }
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
        if($output =~ /Not a SAS adapter|failed|No such adapter|Usage/){
            croak $output;
        }
        return 1;
    }
    elsif($opts{command} =~ /^abort$/){
        if(defined($opts{adapter_name})){
            $cmd = "$cmd1 abort enable -t $opts{period} -r -c $opts{adapter_name}";
        }
        elsif(defined($opts{device_name})){
            $cmd = "$cmd1 abort enable -t $opts{period} -d $opts{device_name}";
        }
        else{
            logresult('script', "adapter_name or device_name are the required parameters");
        }
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
        if($output =~ /Not a SAS adapter|failed|No such adapter|Usage/){
            croak $output;
        }
        return 1;
    }
    elsif($opts{command} =~ /^timeout$/){
        if(defined($opts{adapter_name})){
            $cmd = "$cmd1 timeout enable -t $opts{period} -r -c $opts{adapter_name}";
        }
        elsif(defined($opts{device_name})){
            $cmd = "$cmd1 timeout enable -t $opts{period} -d $opts{device_name}";
        }
        else{
            logresult('script', "adapter_name or device_name are the required parameters");
        }
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
        if($output =~ /Not a SAS adapter|failed|No such adapter|Usage/){
            croak $output;
        }
        return 1;
    }
    elsif($opts{command} =~ /^adapter_reset$/){
        if(defined($opts{adapter_name})){
            $cmd = "$cmd1 adapter_reset enable -t $opts{period}  -c $opts{adapter_name}";
        }
        else{
            logresult('script', "adapter_name is the required parameter");
        }
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
        if($output =~ /Not a SAS adapter|failed|No such adapter|Usage/){
            croak $output;
        }
        return 1;
    }
    elsif($opts{command} =~ /^disable$/){
            $cmd = "$cmd1 disable";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
            if($output !~ /Disabling all error injection/){
                croak $output;
        }
        return 1;
        }
    elsif($opts{command} =~ /^show$/){
            $cmd = "$cmd1 show";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd,privmode=>"test");
            if($output =~ /Not a SAS adapter|failed|No such adapter|Usage/){
            croak $output;
        }
        return $output;
    }
    else{
        logresult('script', "Invalid command:  %opts{command}");
    }
}



=head2 sasadmin_expander_set_shelf_id

    my $ret = sasadmin_expander_set_shelf_id

This subroutine issues cli command to an IOM
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Adapter name to which expander is attached.

=item shelf_id

(Required) : Shelf id of the phy

=item command

(Required) : Command to be issued to IOM


=item timeout

(Optional) : timeout value. Default is 300 secs.

=back



=cut

sub sasadmin_expander_set_shelf_id($@){
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            timeout => {type=>SCALAR, default=>"300",optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            shelf_id => {type=>SCALAR},
            adapter_name => {type=>SCALAR},
            shelf_id => {type=>SCALAR,default=>undef,optional=>1 },
            adapter_name => {type=>SCALAR,default=>undef,optional=>1 },
            new_shelf_id => {type=>SCALAR,default=>undef,optional=>1 },
            option => {type=>SCALAR,default=>undef,optional=>1 },
            }
            );
    my $output;
    my $cmd;
    if(defined($opts{option})){
        if($opts{option} !~ /^-c$|^-a$/){
            croak "$opts{option} is not a supported subcommand. Only options allowed are -a or -c";
        }
        else{

            $cmd = "sasadmin expander_set_shelf_id $opts{option} ";
        }
    }
    elsif(defined($opts{new_shelf_id})){
        unless(defined($opts{shelf_id}) || defined($opts{adapter_name})){
            croak "shelf_id and adapter_name are required parameters when option new_shelf_id is used";
        }
        $cmd = "sasadmin expander_set_shelf_id $opts{adapter_name}\.$opts{shelf_id} $opts{new_shelf_id} ";
    }
    else{
        croak "Either option or new_shelf_id parameter needs to be set";
    }

    $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

    if($output =~ /failed|Usage/){
        croak $output ;
    }
    else{
        return 1
    }
}



#-----------------
#----------------------------------------------------------------------
# The testme routine to test the exported functions of this library
#
# This routine can be invoked in the following manner
#
# ntest -hw FILER=raid-u24 PKGNAME=Storage::Sasadmin util/test_pkg (test all functions in normal filer mode)
#
#----------------------------------------------------------------------


sub testme {
    package main;
    our ($FILER);
    our ($FUNCTION_NAME);
    our ($DEVICE);
    our ($FILERMODE);
    our ($ADAPTER_NAME);
    our ($BAY_ID);
    our ($DEV_ID);
    our ($SHELF_ID);
    our ($PHY_NUM);
    package Storage::Sasadmin;
    param('FILER',       '-mesg',    "Filer to test");
    param('FUNCTION_NAME',       '-default',    "all");
    param('FILERMODE',       '-default',    "normal");

    if($FUNCTION_NAME =~ /sasadmin_dev/){
        param('DEV_ID',       '-mesg',    "ID of the drive");
    }
    elsif($FUNCTION_NAME =~ /sasadmin_adapter_phy_enable|sasadmin_adapter_phy_disable|sasadmin_expander_phy_enable|sasadmin_expander_phy_disable|sasadmin_expander_phy_reset|sasadmin_expander_phy_mux_reset|sasadmin_expander_phy_power_off|sasadmin_expander_phy_power_on|sasadmin_expander_phy_dongle_data/){

        param('SHELF_ID',       '-mesg',    "SHELF ID of the drive");
        param('ADAPTER_NAME',       '-mesg',    "Adapter id");
        param('PHY_NUM',       '-mesg',    "Phy number");
    }
    elsif($FUNCTION_NAME =~ /sasadmin_adapter_reset|sasadmin_adapter_phy_state|sasadmin_adapter_zombie|sasadmin_adapter_offline|sasadmin_adapter_online/){
        param('ADAPTER_NAME',       '-mesg',    "Adapter id");
    }
    elsif($FUNCTION_NAME =~ /sasadmin_adapter_state/){
        param('ADAPTER_NAME',       '-default',    undef);
    }
    elsif($FUNCTION_NAME =~ /"sasadmin_expander$"|sasadmin_shelf/){
        param('SHELF_ID',       '-default',   undef );
        param('ADAPTER_NAME',       '-mesg',    "Adapter id");
    }
    elsif($FUNCTION_NAME =~ /sasadmin_channels/){
    }
    elsif($FUNCTION_NAME =~ /sasadmin_expander_phy_state|sasadmin_expander/){
        param('SHELF_ID',       '-mesg',    "SHELF ID of the drive");
        param('ADAPTER_NAME',       '-mesg',    "Adapter id");
    }
    elsif($FUNCTION_NAME =~ /all/){
        param('SHELF_ID',       '-mesg',    "SHELF ID of the drive");
        param('ADAPTER_NAME',       '-mesg',    "Adapter id");
        param('PHY_NUM',       '-mesg',    "Phy number");
        param('SHELF_ID',       '-mesg',    "SHELF ID of the drive");
        param('DEV_ID',       '-mesg',    "ID of the drive");
    }
    else{
        tstlog(-fatal,"Cannot recognize $FUNCTION_NAME function \n");
    }
    #logcomment("Testing on FILER $FILER");
    #logcomment("Testing Function $FUNCTION_NAME");
    #logcomment("Testing Device $DEVICE");
    #logcomment("Testing Filermode $FILERMODE");
    #logcomment("Testing Adapter $ADAPTER");

    my $host = host($FILER);
    my $ret;
    my $cmds;

#-------------------------------------------
#Test for sasadmin_adapter_state function
#-------------------------------------------


        if(($FUNCTION_NAME eq "sasadmin_adapter_state") || ($FUNCTION_NAME eq "all")){
         my @cmds =  (\&sasadmin_adapter_state);
    foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_state");
            }
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function $FUNCTION_NAME failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_state");
            }

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_state: $@");
            }

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_state: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_state");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function $FUNCTION_NAME failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_state");
            }

            #test invalid device name in maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_state: $@");
            }

        }

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

            if ($@) {
                    logresult('fail', "Function sasadmin_adapter_state failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_state");
            }
       }
}

#================================
#-------------------------------------------
#Test  sasadmin_adapter_reset function
#-------------------------------------------
        if(($FUNCTION_NAME eq "sasadmin_adapter_reset") || ($FUNCTION_NAME eq "all")){
    my @cmds =  (\&sasadmin_adapter_reset);
    foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_reset");
            }
        sleep 20;
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 20;

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_reset");
            }
        sleep 20;

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_reset: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_reset: $@");
            }
        sleep 20;

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_reset: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_reset: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
        sleep 20;
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_reset");
            }

            #test explicit console connection + maint mode
        sleep 20;
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_reset");
            }

            #test invalid device name in maint mode
        sleep 20;
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_reset: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_reset: $@");
            }

        }

        #test missing mandotary device name
        sleep 20;

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_adapter_reset: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_adapter_reset: $@");
        }
    }
}





#-------------------------------------------
#Test  sasadmin_adapter_offline & sasadmin_adapter_online function
#-------------------------------------------


        if(($FUNCTION_NAME eq "sasadmin_adapter_offline") ||($FUNCTION_NAME eq "sasadmin_adapter_online") || ($FUNCTION_NAME eq "all")){
    my @cmds =  (\&sasadmin_adapter_offline, \&sasadmin_adapter_online);
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            logcomment ( "test rsh connection + normal mode");
            eval {
                $ret = $cmds[0]->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_offline failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_offline");
            }
        sleep 20;
            eval {
                $ret = $cmds[1]->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_online failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_online");
            }

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            logcomment ("Test rsh connection + normal mode  (default connection type and default connection mode)");
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_offline failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function  sasadmin_adapter_offline");
            }
        sleep 20;

            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_online failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function  sasadmin_adapter_online");
            }
        sleep 20;

            #test console connection + normal mode
            logcomment ("Test console connection + normal mode ");
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_offline failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_offline");
            }
        sleep 20;

 eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_online failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_online");
            }
        sleep 20;

            #test invalid adapter name over rsh connection
            logcomment("Test invalid adapter name over rsh connection");
            eval {
                $ret = $cmds[0]->(host=>$host, adapter_name=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_offline: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_offline: $@");
            }
        sleep 20;

eval {
                $ret = $cmds[1]->(host=>$host, adapter_name=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_online: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_online: $@");
            }
        sleep 20;



            #test invalid adapter name over console connection
            logcomment ("Test invalid adapter name over console connection");
            eval {
                $ret = $cmds[0]->(host=>$host, adapter_name=>"abc", conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_offline: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_offline: $@");
            }
        eval {
                    $ret = $cmds[1]->(host=>$host, adapter_name=>"abc", conntype=>"#console");
            };

                if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_online: $@");
                } else {
                    logresult('fail', "Got unexpected result from function sasadmin_adapter_online: $@");
            }


        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            logcomment ("Test implicit console connection + maint mode");
        sleep 20;
            eval {
                $ret = $cmds[0]->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_offline failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_offline");
            }
        eval {
                    $ret = $cmds[1]->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint");
                };

                if ($@) {
                    logresult('fail', "function sasadmin_adapter_online failed: $@");
                } else {
                           print Dumper($ret);
                     logresult('pass', "Got expected result from function sasadmin_adapter_online");
                }



            #test explicit console connection + maint mode
            logcomment("Test explicit console connection + maint mode");
        sleep 20;
            eval {
                $ret = $cmds[0]->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_offline failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_offline");
            }
        sleep 20;
            eval {
                $ret = $cmds[1]->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_online failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_online");
            }


            #test invalid device name in maint mode
            logcomment("Test invalid device name in maint mode");
        sleep 20;
            eval {
                $ret = $cmds[0]->(host=>$host, adapter_name=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_offline: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_offline: $@");
            }
        sleep 20;
            eval {
                $ret = $cmds[1]->(host=>$host, adapter_name=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_online: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_online: $@");
            }

        }

        #test missing mandotary device name
        logcomment("Test missing mandotary device name");
        sleep 20;

        eval {
            $ret = $cmds[0]->(host=>$host,filermode=>"maint");
        };

        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_adapter_offline: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_adapter_offline: $@");
        }
        sleep 20;

        eval {
            $ret = $cmds[1]->(host=>$host,filermode=>"maint");
        };

        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_adapter_online: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_adapter_online: $@");
        }

}

#-------------------------------------------
#Test  sasadmin_adapter_zombie function
#-------------------------------------------


        if(($FUNCTION_NAME eq "sasadmin_adapter_zombie") || ($FUNCTION_NAME eq "all")){
#my @cmds = (\&sasadmin_adapter_reset| \&sasadmin_adapter_phy_state| \&sasadmin_adapter_zombie| \&sasadmin_adapter_offline \&sasadmin_adapter_online);
    my @cmds =  (\&sasadmin_adapter_zombie);
    foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_zombie failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_zombie");
            }
        sleep 5;
        #test a case where adapter is already zombified
        eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME);
            };
        print "adapter $ADAPTER_NAME is  $@ a\n";

            if ($@ =~ /Cannot complete operation on channel $ADAPTER_NAME/) {
                logresult('pass', "function sasadmin_adapter_zombie passed with expected failure: $@");
            } else {
                logresult('fail', "Didnot see expected failure from function sasadmin_adapter_zombie");
            }

        sleep 40;
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_zombie failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 40;

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_zombie failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function $FUNCTION_NAME");
            }
        sleep 40;

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_zombie: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_zombie: $@");
            }
        sleep 40;

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_zombie: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_zombie: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
        sleep 40;
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_zombie failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_zombie");
            }

            #test explicit console connection + maint mode
        sleep 40;
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_zombie failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_zombie");
            }

            #test invalid device name in maint mode
        sleep 40;
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_zombie: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_zombie: $@");
            }

        }

        #test missing mandotary device name
        sleep 40;

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_adapter_zombie: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_adapter_zombie: $@");
        }
    }
      }



#--------------------------------------------------------------------------
#Test  sasadmin_adapter_phy_disable & sasadmin_adapter_phy_enable function
#--------------------------------------------------------------------------


        if(($FUNCTION_NAME eq "sasadmin_adapter_phy_disable") || ($FUNCTION_NAME eq "sasadmin_adapter_phy_enable") || ($FUNCTION_NAME eq "all")){
    my @cmds =  (\&sasadmin_adapter_phy_disable, \&sasadmin_adapter_phy_enable);
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            logcomment ( "TEST 1: test telnet connection + normal mode");
            eval {
                $ret = $cmds[0]->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_disable");
            }
        sleep 20;
            eval {
                $ret = $cmds[1]->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_enable");
            }
sleep 20;
            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            logcomment ("TEST 2: Test rsh connection + normal mode  (default connection type and default connection mode)");
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_disable");
            }
        sleep 20;

            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function  sasadmin_adapter_phy_enable");
            }
        sleep 20;

            #test console connection + normal mode
            logcomment ("TEST 3: Test console connection + normal mode ");
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console",phy_num=>$PHY_NUM);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_disable");
            }
        sleep 20;

 eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console",phy_num=>$PHY_NUM);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_enable");
            }
        sleep 20;

            #test invalid adapter name over rsh connection
            logcomment("TEST 4: Test invalid adapter name over rsh connection");
            eval {
                $ret = $cmds[0]->(host=>$host, adapter_name=>"abc",phy_num=>$PHY_NUM);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_phy_disable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_disable: $@");
            }
        sleep 20;

eval {
                $ret = $cmds[1]->(host=>$host, adapter_name=>"abc",phy_num=>$PHY_NUM);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_phy_enable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_enable: $@");
            }
        sleep 20;



            #test invalid adapter name over console connection
            logcomment ("TEST 5: Test invalid adapter name over console connection");
            eval {
                $ret = $cmds[0]->(host=>$host, adapter_name=>"abc", conntype=>"#console",phy_num=>$PHY_NUM);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_phy_disable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_disable: $@");
            }
        eval {
                    $ret = $cmds[1]->(host=>$host, adapter_name=>"abc", conntype=>"#console",phy_num=>$PHY_NUM);
            };

                if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_phy_enable: $@");
                } else {
                    logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_enable: $@");
            }


        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            logcomment ("TEST 6: Test implicit console connection + maint mode");
        sleep 20;
            eval {
                $ret = $cmds[0]->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint",phy_num=>$PHY_NUM);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_disable");
            }
        eval {
                    $ret = $cmds[1]->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint",phy_num=>$PHY_NUM);
                };

                if ($@) {
                    logresult('fail', "function sasadmin_adapter_phy_enable failed: $@");
                } else {
                           print Dumper($ret);
                     logresult('pass', "Got expected result from function sasadmin_adapter_phy_enable");
                }



            #test explicit console connection + maint mode
            logcomment("TEST 7: Test explicit console connection + maint mode");
        sleep 20;
            eval {
                $ret = $cmds[0]->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint",phy_num=>$PHY_NUM);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_disable");
            }
        sleep 20;
            eval {
                $ret = $cmds[1]->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint",phy_num=>$PHY_NUM);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_enable");
            }


            #test invalid device name in maint mode
            logcomment("TEST 8: Test invalid device name in maint mode");
        sleep 20;
            eval {
                $ret = $cmds[0]->(host=>$host, adapter_name=>"abc", filermode=>"maint",phy_num=>$PHY_NUM);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_phy_disable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_disable: $@");
            }
        sleep 20;
            eval {
                $ret = $cmds[1]->(host=>$host, adapter_name=>"abc", filermode=>"maint",phy_num=>$PHY_NUM);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_phy_enable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_enable: $@");
            }

        }

        #test missing mandotary device name
        logcomment("TEST 9: Test missing mandotary device name");
        sleep 20;

        eval {
            $ret = $cmds[0]->(host=>$host,filermode=>"maint",phy_num=>$PHY_NUM);
        };

        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_adapter_phy_disable: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_disable: $@");
        }
        sleep 20;

        eval {
            $ret = $cmds[1]->(host=>$host,filermode=>"maint",phy_num=>$PHY_NUM);
        };

        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_adapter_phy_enable: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_enable: $@");
        }

}

#--------------------------------------------------------------------------
#Test  sasadmin_adapter_phy_state function
#--------------------------------------------------------------------------


        if(($FUNCTION_NAME eq "sasadmin_adapter_phy_state") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_adapter_phy_state);
        foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_state");
            }
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME);
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_state");
            }

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_phy_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_state: $@");
            }

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_phy_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_state: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_state");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_adapter_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_state");
            }

            #test invalid device name in maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_adapter_phy_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_state: $@");
            }

        }

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_adapter_phy_state failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_adapter_phy_state");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_adapter_phy_state: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_adapter_phy_state: $@");
        }
      }
   }
}



#--------------------------------------------------------------------------
#Test  sasadmin_adapter_dev_state function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_dev_phy_state") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_dev_phy_state);
        foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_state");
            }
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,dev_id=>$DEV_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_state");
            }

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_state: $@");
            }

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc", conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_state: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, dev_id=>$DEV_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_state");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, dev_id=>$DEV_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_state");
            }

            #test invalid device name in maint mode
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_state: $@");
            }

        }

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_dev_phy_state failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_state");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_dev_phy_state: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_dev_phy_state: $@");
        }
      }
   }
}

#----------------------------------

#--------------------------------------------------------------------------
#Test  sasadmin_adapter_dev_phy_disable & sasadmin_dev_phy_enable function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_dev_phy_disable") || ($FUNCTION_NAME eq "sasadmin_dev_phy_enable") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_dev_phy_disable, \&sasadmin_dev_phy_enable);
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds[0]->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_disable");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_enable");
            }
        sleep 25;
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds[0]->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 25;

            #test console connection + normal mode
            eval {
                $ret = $cmds[0]->(host=>$host,dev_id=>$DEV_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_disable");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,dev_id=>$DEV_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_enable");
            }
    sleep 25;
            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds[0]->(host=>$host, dev_id=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_disable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_disable: $@");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host, dev_id=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_enable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_enable: $@");
            }
        sleep 25;

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds[0]->(host=>$host, dev_id=>"abc", conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_disable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_disable: $@");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host, dev_id=>"abc", conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_enable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_enable: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
        sleep 25;
            eval {
                $ret = $cmds[0]->(host=>$host, dev_id=>$DEV_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_disable");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host, dev_id=>$DEV_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_enable");
            }
        sleep 25;

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds[0]->(host=>$host, dev_id=>$DEV_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_disable");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host, dev_id=>$DEV_ID, filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_enable");
            }
        sleep 25;

            #test invalid device name in maint mode
            eval {
                $ret = $cmds[0]->(host=>$host, dev_id=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_disable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_disable: $@");
            }

    sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host, dev_id=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_enable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_enable: $@");
            }
        sleep 25;

        #test missing mandotary device name

        eval {
            $ret = $cmds[0]->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_dev_phy_disable failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_disable");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_dev_phy_disable: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_dev_phy_disable: $@");
        }
      }
    sleep 5;
        eval {
            $ret = $cmds[1]->(host=>$host,filermode=>"maint");
        };

        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_dev_phy_enable: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_dev_phy_enable: $@");
        }
     }
}

#--------------------------------------------------------------------------
#Test  sasadmin_adapter_dev_phy_reset function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME =~ "sasadmin_dev_phy_reset") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_dev_phy_reset);
        foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_reset");
            }
        sleep 20;
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 20;

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,dev_id=>$DEV_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_reset");
            }
        sleep 20;

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_reset: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_reset: $@");
            }
        sleep 20;

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc", conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_reset: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_reset: $@");
            }
        } else {     #maintenance mode testing
        sleep 20;
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, dev_id=>$DEV_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_reset");
            }

            #test explicit console connection + maint mode
        sleep 20;
            eval {
                $ret = $cmds->(host=>$host, dev_id=>$DEV_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_reset");
            }

            #test invalid device name in maint mode
        sleep 20;
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_reset: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_reset: $@");
            }

        }

        #test missing mandotary device name
        sleep 20;

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_dev_phy_reset failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_reset");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_dev_phy_reset: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_dev_phy_reset: $@");
        }
      }
   }
}


#--------------------------------------------------------------------------
#Test  sasadmin_adapter_dev_phy_clear_affiliation function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_dev_phy_clear_affiliation") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_dev_phy_clear_affiliation);
        foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_clear_affiliation failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_clear_affiliation");
            }
        sleep 20;
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_clear_affiliation failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_clear_affiliation");
            }
        sleep 20;

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,dev_id=>$DEV_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function $FUNCTION_NAME failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_clear_affiliation");
            }
        sleep 20;

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_clear_affiliation: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_clear_affiliation: $@");
            }
        sleep 20;

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc", conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_clear_affiliation: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_clear_affiliation: $@");
            }
        } else {     #maintenance mode testing
        sleep 20;
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, dev_id=>$DEV_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_clear_affiliation failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_clear_affiliation");
            }

            #test explicit console connection + maint mode
        sleep 20;
            eval {
                $ret = $cmds->(host=>$host, dev_id=>$DEV_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_phy_clear_affiliation failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_clear_affiliation");
            }

            #test invalid device name in maint mode
        sleep 20;
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_phy_clear_affiliation: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_phy_clear_affiliation: $@");
            }

        }

        #test missing mandotary device name
        sleep 20;

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_dev_phy_clear_affiliation failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_phy_clear_affiliation");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_dev_phy_clear_affiliation: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_dev_phy_clear_affiliation: $@");
        }
      }
   }
}


#--------------------------------------------------------------------------
#Test  sasadmin_adapter_dev_power_cycle function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_dev_power_cycle") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_dev_power_cycle);
        foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_power_cycle failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_power_cycle");
            }
        sleep 20;
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,dev_id=>$DEV_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_power_cycle failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 20;

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,dev_id=>$DEV_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_power_cycle failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_power_cycle");
            }
        sleep 20;

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_power_cycle: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_power_cycle: $@");
            }
        sleep 20;

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc", conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_power_cycle: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_power_cycle: $@");
            }
        } else {     #maintenance mode testing
        sleep 20;
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, dev_id=>$DEV_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_power_cycle failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_power_cycle");
            }

            #test explicit console connection + maint mode
        sleep 20;
            eval {
                $ret = $cmds->(host=>$host, dev_id=>$DEV_ID, filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_dev_power_cycle failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_power_cycle");
            }

            #test invalid device name in maint mode
        sleep 20;
            eval {
                $ret = $cmds->(host=>$host, dev_id=>"abc", filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_dev_power_cycle: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_dev_power_cycle: $@");
            }

        }

        #test missing mandotary device name
        sleep 20;

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_dev_power_cycle failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_dev_power_cycle");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_dev_power_cycle: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_dev_power_cycle: $@");
        }
      }
   }
}

#--------------------------------------------------------------------------
#Test  sasadmin_expander_phy_power_off &  sasadmin_expander_phy_power_on function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_expander_phy_power_off") || ($FUNCTION_NAME eq "sasadmin_expander_phy_power_on") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_expander_phy_power_off, \&sasadmin_expander_phy_power_on);
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds[0]->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_power_off failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_power_off");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_power_on failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_power_on");
            }
        sleep 25;
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_power_off failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_power_on failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 25;

            #test console connection + normal mode
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_power_off failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_power_off");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_power_on failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_power_on");
            }
    sleep 25;
            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_power_off: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_power_off: $@");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);

            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_power_on: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_power_on: $@");
            }
        sleep 25;

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_power_off: $@");
            } else {
                logresult('fail', "Got unexpected result: $@ from function sasadmin_expander_phy_power_off: $@");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_power_on: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_power_on: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
        sleep 25;
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_power_off failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_power_off");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_power_on failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_power_on");
            }
        sleep 25;

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_power_off failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_power_off");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_power_on failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_power_on");
            }
        sleep 25;

            #test invalid device name in maint mode
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_power_off: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_power_off: $@");
            }

    sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_power_on: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_power_on: $@");
            }
        sleep 25;

        #test missing mandotary device name

        eval {
            $ret = $cmds[0]->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_expander_phy_power_off failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_power_off");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_expander_phy_power_off: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_expander_phy_power_off: $@");
        }
      }
    sleep 5;
        eval {
            $ret = $cmds[1]->(host=>$host,filermode=>"maint");
        };

        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_expander_phy_power_on: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_expander_phy_power_on: $@");
        }
     }
}
#--------------------------------------------------------------------------
#Test  sasadmin_expander_phy_disable &  sasadmin_expander_phy_enable function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_expander_phy_disable") || ($FUNCTION_NAME eq "sasadmin_expander_phy_enable") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_expander_phy_disable, \&sasadmin_expander_phy_enable);
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds[0]->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_disable");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_enable");
            }
        sleep 25;
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 25;

            #test console connection + normal mode
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_disable");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_enable");
            }
    sleep 25;
            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_disable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_disable: $@");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);

            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_enable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_enable: $@");
            }
        sleep 25;

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_disable: $@");
            } else {
                logresult('fail', "Got unexpected result: $@ from function sasadmin_expander_phy_disable: $@");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_enable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_enable: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
        sleep 25;
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_disable");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_enable");
            }
        sleep 25;

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_disable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_disable");
            }
        sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_enable failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_enable");
            }
        sleep 25;

            #test invalid device name in maint mode
            eval {
                $ret = $cmds[0]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_disable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_disable: $@");
            }

    sleep 25;
            eval {
                $ret = $cmds[1]->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_enable: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_enable: $@");
            }
        sleep 25;

        #test missing mandotary device name

        eval {
            $ret = $cmds[0]->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_expander_phy_disable failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_disable");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_expander_phy_disable: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_expander_phy_disable: $@");
        }
      }
    sleep 5;
        eval {
            $ret = $cmds[1]->(host=>$host,filermode=>"maint");
        };

        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_expander_phy_enable: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_expander_phy_enable: $@");
        }
     }
}


#--------------------------------------------------------------------------
#Test  sasadmin_expander_phy_reset function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_expander_phy_reset") || ($FUNCTION_NAME eq "all")){
             my $cmds =  \&sasadmin_expander_phy_reset;
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_reset");
            }
        sleep 10;
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 10;

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_reset");
            }
        sleep 10;
            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_reset: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_reset: $@");
            }
        sleep 10;

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_reset: $@");
            } else {
                logresult('fail', "Got unexpected result: $@ from function sasadmin_expander_phy_reset: $@");
            }
        sleep 10;
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_reset");
            }
        sleep 10;

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_reset");
            }
        sleep 10;

            #test invalid device name in maint mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_reset: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_reset: $@");
            }

    sleep 10;

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_expander_phy_reset failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_reset");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_expander_phy_reset: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_expander_phy_reset: $@");
        }
      }
     }
}


#--------------------------------------------------------------------------
#Test  sasadmin_expander_phy_mux_reset function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_expander_phy_mux_reset") || ($FUNCTION_NAME eq "all")){
             my $cmds =  \&sasadmin_expander_phy_mux_reset;
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_mux_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_mux_reset");
            }
        sleep 10;
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_mux_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }
        sleep 10;

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_mux_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_mux_reset");
            }
        sleep 10;
            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_mux_reset: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_mux_reset: $@");
            }
        sleep 10;

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_mux_reset: $@");
            } else {
                logresult('fail', "Got unexpected result: $@ from function sasadmin_expander_phy_mux_reset: $@");
            }
        sleep 10;
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_mux_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_mux_reset");
            }
        sleep 10;

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_mux_reset failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_mux_reset");
            }
        sleep 10;

            #test invalid device name in maint mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID,filermode=>"maint");
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_mux_reset: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_mux_reset: $@");
            }

    sleep 10;

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_expander_phy_mux_reset failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_mux_reset");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_expander_phy_mux_reset: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_expander_phy_mux_reset: $@");
        }
      }
     }
}
#--------------------------------------------------------------------------
#Test  sasadmin_expander_phy_state function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_expander_phy_state") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_expander_phy_state);
    foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_state");
            }
        sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_state");
            }

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_state: $@");
            }

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", conntype=>"#console",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function $FUNCTION_NAME: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_state: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_state");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_state");
            }

            #test invalid device name in maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_state: $@");
            }

        }

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_expander_phy_state failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_state");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_expander_phy_state: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_expander_phy_state: $@");
        }
      }
   }
}
#=================
#--------------------------------------------------------------------------
#Test  sasadmin_expander_phy_dongle_data function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_expander_phy_dongle_data") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_expander_phy_dongle_data);
    foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,shelf_id=>$SHELF_ID,phy_num=>$PHY_NUM,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_state");
            }
        sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_state");
            }

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_state: $@");
            }

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc",phy_num=>$PHY_NUM, conntype=>"#console",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function $FUNCTION_NAME: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_state: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_state");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint",phy_num=>$PHY_NUM,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_phy_state failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_state");
            }

            #test invalid device name in maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", filermode=>"maint",shelf_id=>$SHELF_ID,phy_num=>$PHY_NUM,);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_phy_state: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_phy_state: $@");
            }

        }

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_expander_phy_state failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_phy_state");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_expander_phy_state: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_expander_phy_state: $@");
        }
      }
   }
}
#--------------------------------------------------------------------------
#Test  sasadmin_expander function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_expander") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_expander);
    foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,shelf_id=>$SHELF_ID,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander");
            }
        sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander");
            }

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander: $@");
            }

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", conntype=>"#console",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function $FUNCTION_NAME: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander");
            }

            #test invalid device name in maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander: $@");
            }

        }

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_expander_phy_state failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_expander: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_expander: $@");
        }
      }
   }
}

#--------------------------------------------------------------------------
#Test  sasadmin_expander_routes function
#--------------------------------------------------------------------------
        if(($FUNCTION_NAME eq "sasadmin_expander_routes") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_expander_routes);
    foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_routes failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_routes");
            }
        sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_routes failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_routes failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_routes");
            }

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_routes: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_routes: $@");
            }

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", conntype=>"#console",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_routes: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_routes: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_routes failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_routes");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_expander_routes failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_routes");
            }

            #test invalid device name in maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_expander_routes: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_expander_routes: $@");
            }

        }

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_expander_routes failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_expander_routes");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_expander_routes: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_expander_routes: $@");
        }
      }
   }
}

#--------------------------------------------------------------------------
#Test  sasadmin_shelf function
#--------------------------------------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_shelf") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_shelf);
    foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_shelf failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_shelf");
            }
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_shelf failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_shelf failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_shelf");
            }

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_shelf: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_shelf: $@");
            }

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", conntype=>"#console",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_shelf: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_shelf: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function $FUNCTION_NAME failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_shelf");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_shelf failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_shelf");
            }

            #test invalid device name in maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_shelf: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_shelf: $@");
            }

        }

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_shelf failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_shelf");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_shelf: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_shelf: $@");
        }
      }
   }
}

#----------------------------------------------
#Test for sasadmin_shelf_short
#----------------------------------------------

        if(($FUNCTION_NAME eq "sasadmin_shelf_short") || ($FUNCTION_NAME eq "all")){
             my @cmds =  (\&sasadmin_shelf_short);
    foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE,adapter_name=>$ADAPTER_NAME,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_shelf_short failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_shelf_short");
            }
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_shelf_short failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,adapter_name=>$ADAPTER_NAME,conntype=>"#console",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_shelf_short failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_shelf_short");
            }

            #test invalid adapter name over rsh connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_shelf_short: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_shelf_short: $@");
            }

            #test invalid adapter name over console connection
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", conntype=>"#console",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_shelf_short: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_shelf_short: $@");
            }
        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME,filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_shelf_short failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_shelf_short");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>$ADAPTER_NAME, filermode=>"maint",filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@) {
                logresult('fail', "function sasadmin_shelf_short failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_shelf_short");
            }

            #test invalid device name in maint mode
            eval {
                $ret = $cmds->(host=>$host, adapter_name=>"abc", filermode=>"maint",shelf_id=>$SHELF_ID);
            };

            if ($@ =~ /failed|No such adapter|Usage/ ) {
                logresult('pass', "Got expected failure from function sasadmin_shelf_short: $@");
            } else {
                logresult('fail', "Got unexpected result from function sasadmin_shelf_short: $@");
            }

        }

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

    if($FUNCTION_NAME =~ /sasadmin_channels|sasadmin_adapter_state/){
            if ($@) {
                    logresult('fail', "Function sasadmin_shelf_short failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_shelf_short");
            }
    }
    else{
        if ($@) {
            logresult('pass', "Got expected failure from function sasadmin_shelf_short: $@");
        } else {
            logresult('fail', "Got unexpected result from function sasadmin_shelf_short: $@");
        }
      }
   }
}

#--------------------------------------------------------------------------
#Test  sasadmin_channels function
#--------------------------------------------------------------------------
  if(($FUNCTION_NAME eq "sasadmin_channels") || ($FUNCTION_NAME eq "all")){
     my @cmds =  (\&sasadmin_channels);
    foreach $cmds (@cmds){
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>$FILERMODE);
            };

            if ($@) {
                logresult('fail', "function sasadmin_channels failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_channels");
            }
        #sleep 300;
            #test rsh connection + normal mode

            #test rsh connection + normal mode  (default connection
            #type and default connection mode
            eval {
                $ret = $cmds->(host=>$host);
            };

            if ($@) {
                logresult('fail', "function sasadmin_channels failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function ");
            }

            #test console connection + normal mode
            eval {
                $ret = $cmds->(host=>$host,conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function sasadmin_channels failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_channels");
            }


        } else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host,filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_channels failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_channels");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = $cmds->(host=>$host, filermode=>"maint",filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function sasadmin_channels failed: $@");
            } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_channels");
            }


        }

        #test missing mandotary device name

        eval {
            $ret = $cmds->(host=>$host,filermode=>"maint");
        };

            if ($@) {
                    logresult('fail', "Function sasadmin_channels failed with: $@");
                } else {
                print Dumper($ret);
                logresult('pass', "Got expected result from function sasadmin_channels");
            }
       }
}

#============================================================
}
=head1 SEE ALSO

    Where additional information can be found.

=head1 BUGS

    Description of bugs or what the library does not do and why

=head1 STATUS

    Shared

=head1 ATTRIBUTES

    ONTAP_COMMANDS = sasadmin

    ONTAP_ZAPI_APIS = none

=head1 AUTHOR

    Anish Vashist (anishv@netapp.com)
    $Id: //depot/prod/test/Rsierranevada/lib/Storage/Sasadmin.pm#5 $
    Last revised $Date: 2013/09/10 $

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

1;
